@@ -46,6 +46,7 @@
 #include "nsHTMLFormControlAccessible.h"
 #include "nsHTMLImageAccessible.h"
 #include "nsHTMLLinkAccessible.h"
+#include "nsHTMLPluginAccessible.h"
 #include "nsHTMLSelectAccessible.h"
 #include "nsHTMLTableAccessible.h"
 #include "nsHTMLTextAccessible.h"
@@ -56,16 +57,23 @@
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMHTMLAreaElement.h"
+#include "nsIDOMHTMLObjectElement.h"
 #include "nsIDOMHTMLOptionElement.h"
+#include "nsIDOMHTMLOptGroupElement.h"
+#include "nsIDOMHTMLLegendElement.h"
+#include "nsIDOMHTMLTableElement.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMXULCheckboxElement.h"
 #include "nsIFrame.h"
 #include "nsILink.h"
 #include "nsINameSpaceManager.h"
+#include "nsIObjectFrame.h"
+#include "nsIPluginInstance.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsITextContent.h"
 #include "nsLayoutAtoms.h"
+#include "nsObjectFrame.h"
 #include "nsRootAccessible.h"
 #include "nsString.h"
 #include "nsTextFragment.h"
@@ -75,6 +83,15 @@
 #include "nsXULSelectAccessible.h"
 #include "nsXULTabAccessible.h"
 #include "nsXULTextAccessible.h"
+#include "nsXULTreeAccessible.h"
+#include "nsIAccessible.h"
+#include "nsCaretAccessible.h"
+#include "nsIAccessibleCaret.h"
+
+// For native window support for object/embed/applet tags
+#ifdef XP_WIN
+#include "nsHTMLWin32ObjectAccessible.h"
+#endif
 
 // IFrame
 #include "nsIDocShell.h"
@@ -87,10 +104,12 @@
 nsAccessibilityService::nsAccessibilityService()
 {
   NS_INIT_REFCNT();
+  nsLayoutAtoms::AddRefAtoms();
 }
 
 nsAccessibilityService::~nsAccessibilityService()
 {
+  nsLayoutAtoms::ReleaseAtoms();
 }
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsAccessibilityService, nsIAccessibilityService);
@@ -128,17 +147,28 @@ nsAccessibilityService::GetInfo(nsISupports* aFrame, nsIFrame** aRealFrame, nsIW
   return NS_OK;
 }
 
-void nsAccessibilityService::GetOwnerFor(nsIPresShell *aPresShell, nsIPresShell **aOwnerShell, nsIContent **aOwnerContent)
+void nsAccessibilityService::GetOwnerFor(nsIPresShell *aPresShell,
+                                         nsIPresShell **aOwnerShell,
+                                         nsIContent **aOwnerContent)
 {
+  *aOwnerShell = nsnull;
+  *aOwnerContent = nsnull;
+
   nsCOMPtr<nsIPresContext> presContext;
   aPresShell->GetPresContext(getter_AddRefs(presContext));
   if (!presContext) 
     return;
+
+  nsCOMPtr<nsIDocument> doc;
+  aPresShell->GetDocument(getter_AddRefs(doc));
+
+  if (!doc)
+    return;
+
   nsCOMPtr<nsISupports> pcContainer;
   presContext->GetContainer(getter_AddRefs(pcContainer));
   if (!pcContainer) 
     return;
-  nsCOMPtr<nsISupports> docShellSupports(do_QueryInterface(pcContainer));
 
   nsCOMPtr<nsIDocShellTreeItem> treeItem(do_QueryInterface(pcContainer));
   if (!treeItem) 
@@ -164,17 +194,14 @@ void nsAccessibilityService::GetOwnerFor(nsIPresShell *aPresShell, nsIPresShell
   if (!parentDoc)
     return;
 
-  nsCOMPtr<nsIContent> rootContent;
-  parentDoc->GetRootContent(getter_AddRefs(rootContent));
+  parentDoc->FindContentForSubDocument(doc, aOwnerContent);
 
-  nsCOMPtr<nsIContent> tempContent;
-  parentPresShell->FindContentForShell(docShellSupports, getter_AddRefs(tempContent));
-
-  if (tempContent) {
-    *aOwnerContent = tempContent;
+  if (*aOwnerContent) {
     *aOwnerShell = parentPresShell;
     NS_ADDREF(*aOwnerShell);
-    NS_ADDREF(*aOwnerContent);
+  }
+  else {
+    NS_WARNING("Cannot find content for sub document");
   }
 }
 
@@ -244,58 +271,69 @@ nsAccessibilityService::CreateIFrameAccessible(nsIDOMNode* aDOMNode, nsIAccessib
   GetShellFromNode(aDOMNode, getter_AddRefs(outerWeakShell));
 
   nsCOMPtr<nsIPresShell> outerPresShell(do_QueryReferent(outerWeakShell));
-  if (NS_WARN_IF_FALSE(outerPresShell,
-                       "No outer pres shell in CreateHTMLIFrameAccessible!")) {
+  if (!outerPresShell) {
+    NS_WARNING("No outer pres shell in CreateHTMLIFrameAccessible!");
     return NS_ERROR_FAILURE;
   }
-  
+
   nsCOMPtr<nsIPresContext> outerPresContext;
   outerPresShell->GetPresContext(getter_AddRefs(outerPresContext));
-  if (NS_WARN_IF_FALSE(outerPresContext,
-                       "No outer pres context in CreateHTMLIFrameAccessible!")) {
+  if (!outerPresContext) {
+    NS_WARNING("No outer pres context in CreateHTMLIFrameAccessible!");
     return NS_ERROR_FAILURE;
   }
-  
+
   nsCOMPtr<nsIDocument> doc;
   if (NS_SUCCEEDED(content->GetDocument(*getter_AddRefs(doc))) && doc) {
-    if (outerPresShell) {
-      nsCOMPtr<nsISupports> supps;
-      outerPresShell->GetSubShellFor(content, getter_AddRefs(supps));
-      if (supps) {
-        nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(supps));
-        if (docShell) {
-          nsCOMPtr<nsIPresShell> innerPresShell;
-          docShell->GetPresShell(getter_AddRefs(innerPresShell));
-          if (innerPresShell) {
-            nsCOMPtr<nsIWeakReference> innerWeakShell(do_GetWeakReference(innerPresShell));
-            nsCOMPtr<nsIDocument> innerDoc;
-            innerPresShell->GetDocument(getter_AddRefs(innerDoc)); 
-            if (innerDoc) {
-              nsCOMPtr<nsIAccessible> innerRootAccessible(new nsHTMLIFrameRootAccessible(aDOMNode, innerWeakShell));
-              if (innerRootAccessible) {
-                nsHTMLIFrameAccessible* outerRootAccessible = new nsHTMLIFrameAccessible(aDOMNode, innerRootAccessible, outerWeakShell, innerDoc);
-                if (outerRootAccessible) {
-                  *_retval = NS_STATIC_CAST(nsIAccessible*, outerRootAccessible);
-                  if (*_retval) {
-                    NS_ADDREF(*_retval);
-                    return NS_OK;
-                  }
-                }
-              }
+    nsCOMPtr<nsIDocument> sub_doc;
+    doc->GetSubDocumentFor(content, getter_AddRefs(sub_doc));
+
+    if (sub_doc) {
+      nsCOMPtr<nsIPresShell> innerPresShell;
+      sub_doc->GetShellAt(0, getter_AddRefs(innerPresShell));
+
+      if (innerPresShell) {
+        nsCOMPtr<nsIWeakReference> innerWeakShell =
+          do_GetWeakReference(innerPresShell);
+
+        nsHTMLIFrameRootAccessible *innerRootAccessible =
+          new nsHTMLIFrameRootAccessible(aDOMNode, innerWeakShell);
+
+        if (innerRootAccessible) {
+          nsHTMLIFrameAccessible *outerRootAccessible =
+            new nsHTMLIFrameAccessible(aDOMNode, innerRootAccessible,
+                                       outerWeakShell, sub_doc);
+
+          if (outerRootAccessible) {
+            innerRootAccessible->Init(outerRootAccessible);
+            *_retval = outerRootAccessible;
+            if (*_retval) {
+              NS_ADDREF(*_retval);
+              return NS_OK;
             }
           }
+          else // don't leak the innerRoot
+            delete innerRootAccessible;
         }
       }
     }
   }
+
   return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP 
 nsAccessibilityService::CreateRootAccessible(nsISupports* aPresContext, nsISupports* aFrame, nsIAccessible **_retval)
 {
+  static PRBool alreadyHere = PR_FALSE;
+
   *_retval = nsnull;
 
+  if (alreadyHere) {
+    NS_WARNING("Infinite loop headed off in CreateRootAccessible");
+    return NS_ERROR_FAILURE;
+  }
+
   nsCOMPtr<nsIPresContext> presContext(do_QueryInterface(aPresContext));
   NS_ASSERTION(presContext,"Error non prescontext passed to accessible factory!!!");
 
@@ -318,8 +356,11 @@ nsAccessibilityService::CreateRootAccessible(nsISupports* aPresContext, nsISuppo
     nsCOMPtr<nsIContent> rootContent;
     presShell->GetDocument(getter_AddRefs(document));
     nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(document));
-    if (rootNode)
+    if (rootNode) {
+      alreadyHere = PR_TRUE;
       GetAccessibleFor(rootNode, _retval);
+      alreadyHere = PR_FALSE;
+    }
   }
   else {
     nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(presShell));
@@ -333,6 +374,21 @@ nsAccessibilityService::CreateRootAccessible(nsISupports* aPresContext, nsISuppo
   return NS_OK;
 }
 
+NS_IMETHODIMP 
+nsAccessibilityService::CreateCaretAccessible(nsIDOMNode *aNode, nsIAccessibleEventListener *aListener,
+                                              nsIAccessibleCaret **_retval)
+{
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  *_retval = new nsCaretAccessible(aNode, weakShell, aListener);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+  return NS_OK;
+}
+
  /**
    * HTML widget creation
    */
@@ -409,6 +465,25 @@ nsAccessibilityService::CreateHTMLButtonAccessible(nsISupports *aFrame, nsIAcces
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLButtonAccessibleXBL(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  // reusing the HTML accessible widget and enhancing for XUL
+  *_retval = new nsHTML4ButtonAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsAccessibilityService::CreateHTMLCheckboxAccessible(nsISupports *aFrame, nsIAccessible **_retval)
 {
@@ -427,6 +502,25 @@ nsAccessibilityService::CreateHTMLCheckboxAccessible(nsISupports *aFrame, nsIAcc
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLCheckboxAccessibleXBL(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  // reusing the HTML accessible widget and enhancing for XUL
+  *_retval = new nsHTMLCheckboxAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsAccessibilityService::CreateHTMLComboboxAccessible(nsIDOMNode* aDOMNode, nsISupports* aPresContext, nsIAccessible **_retval)
 {
@@ -456,7 +550,37 @@ nsAccessibilityService::CreateHTMLImageAccessible(nsISupports *aFrame, nsIAccess
   if (NS_FAILED(rv))
     return rv;
 
-  *_retval = new nsHTMLImageAccessible(node, weakShell);
+  *_retval = nsnull;
+  nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(node));
+  if (domElement) {
+    PRBool hasAttribute;
+    rv = domElement->HasAttribute(NS_LITERAL_STRING("usemap"), &hasAttribute);
+    if (NS_SUCCEEDED(rv) && hasAttribute) {
+      //There is a "use map"
+      *_retval = new nsHTMLImageMapAccessible(node, weakShell);
+    }
+    else
+      *_retval = new nsHTMLImageAccessible(node, weakShell);
+  }
+
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLGroupboxAccessible(nsISupports *aFrame, nsIAccessible **_retval)
+{
+  nsIFrame* frame;
+  nsCOMPtr<nsIDOMNode> node;
+  nsCOMPtr<nsIWeakReference> weakShell;
+  nsresult rv = GetInfo(aFrame, &frame, getter_AddRefs(weakShell), getter_AddRefs(node));
+  if (NS_FAILED(rv))
+    return rv;
+
+  *_retval = new nsHTMLGroupboxAccessible(node, weakShell);
   if (! *_retval) 
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -483,6 +607,36 @@ nsAccessibilityService::CreateHTMLListboxAccessible(nsIDOMNode* aDOMNode, nsISup
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLPluginAccessible(nsIDOMNode *aDOMNode, nsIWeakReference *aShell,
+                                                   nsIAccessible **_retval)
+{
+  *_retval = new nsHTMLPluginAccessible(aDOMNode, aShell);
+
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLNativeWindowAccessible(nsIDOMNode *aDOMNode, nsIWeakReference *aShell,
+                                                         PRInt32 aHwnd, nsIAccessible **_retval)
+{
+#ifdef XP_WIN
+  *_retval = new nsHTMLWin32ObjectAccessible(aDOMNode, aShell, aHwnd);
+
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsAccessibilityService::CreateHTMLRadioButtonAccessible(nsISupports *aFrame, nsIAccessible **_retval)
 {
@@ -501,6 +655,25 @@ nsAccessibilityService::CreateHTMLRadioButtonAccessible(nsISupports *aFrame, nsI
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLRadioButtonAccessibleXBL(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  // reusing the HTML accessible widget and enhancing for XUL
+  *_retval = new nsHTMLRadioButtonAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsAccessibilityService::CreateHTMLSelectOptionAccessible(nsIDOMNode* aDOMNode, nsIAccessible *aAccParent, nsISupports* aPresContext, nsIAccessible **_retval)
 {
@@ -538,6 +711,50 @@ nsAccessibilityService::CreateHTMLTableAccessible(nsISupports *aFrame, nsIAccess
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLTableCaptionAccessible(nsIDOMNode *aDOMNode, nsIAccessible **_retval)
+{
+  NS_ENSURE_ARG_POINTER(aDOMNode);
+
+  nsresult rv = NS_OK;
+
+  nsCOMPtr<nsIWeakReference> weakShell;
+  rv = GetShellFromNode(aDOMNode, getter_AddRefs(weakShell));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsHTMLTableCaptionAccessible* accTableCaption =
+    new nsHTMLTableCaptionAccessible(aDOMNode, weakShell);
+
+  NS_ENSURE_TRUE(accTableCaption, NS_ERROR_OUT_OF_MEMORY);
+
+  *_retval = NS_STATIC_CAST(nsIAccessible *, accTableCaption);
+  NS_IF_ADDREF(*_retval);
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsAccessibilityService::CreateHTMLTableHeadAccessible(nsIDOMNode *aDOMNode, nsIAccessible **_retval)
+{
+  NS_ENSURE_ARG_POINTER(aDOMNode);
+
+  nsresult rv = NS_OK;
+
+  nsCOMPtr<nsIWeakReference> weakShell;
+  rv = GetShellFromNode(aDOMNode, getter_AddRefs(weakShell));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsHTMLTableHeadAccessible* accTableHead =
+    new nsHTMLTableHeadAccessible(aDOMNode, weakShell);
+
+  NS_ENSURE_TRUE(accTableHead, NS_ERROR_OUT_OF_MEMORY);
+
+  *_retval = NS_STATIC_CAST(nsIAccessible *, accTableHead);
+  NS_IF_ADDREF(*_retval);
+
+  return rv;
+}
+
 NS_IMETHODIMP
 nsAccessibilityService::CreateHTMLTableCellAccessible(nsISupports *aFrame, nsIAccessible **_retval)
 {
@@ -582,6 +799,11 @@ nsAccessibilityService::CreateHTMLTextAccessible(nsISupports *aFrame, nsIAccessi
       if (theChar == NBSP)
         return NS_ERROR_FAILURE;
     }
+    nsCOMPtr<nsIDOMNode> parentNode;
+    node->GetParentNode(getter_AddRefs(parentNode));
+    nsCOMPtr<nsIDOMHTMLLegendElement> legend(do_QueryInterface(parentNode));
+    if (legend)  // Expose <legend> as the name in a groupbox, not as a ROLE_TEXT accessible
+      return NS_ERROR_FAILURE; 
   }
     
   *_retval = new nsHTMLTextAccessible(node, weakShell);
@@ -794,6 +1016,24 @@ nsAccessibilityService::CreateXULListboxAccessible(nsIDOMNode *aNode, nsIAccessi
   return NS_OK;
 }
 
+NS_IMETHODIMP 
+nsAccessibilityService::CreateXULListitemAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  *_retval = new nsXULListitemAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsAccessibilityService::CreateXULMenubarAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
 {
@@ -1065,6 +1305,104 @@ NS_IMETHODIMP nsAccessibilityService::CreateXULTabsAccessible(nsIDOMNode *aNode,
   return NS_OK;
 }
 
+/**
+  * We can have several cases here. 
+  *  1) a text or html embedded document where the contentDocument
+  *     variable in the object element holds the content
+  *  2) web content that uses a plugin, which means we will
+  *     have to go to the plugin to get the accessible content
+  *  3) An image or imagemap, where the image frame points back to 
+  *     the object element DOMNode
+  */
+nsresult
+nsAccessibilityService::GetHTMLObjectAccessibleFor(nsIDOMNode *aNode, 
+                                                   nsIPresShell *aShell,
+                                                   nsObjectFrame *aFrame,
+                                                   nsIAccessible **_retval)
+{
+  // 1) for object elements containing either HTML or TXT documents
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  nsCOMPtr<nsIDOMHTMLObjectElement> obj(do_QueryInterface(aNode));
+  if (obj)
+    obj->GetContentDocument(getter_AddRefs(domDoc));
+  else
+    domDoc = do_QueryInterface(aNode);
+  if (domDoc)
+    return CreateIFrameAccessible(aNode, _retval);
+
+  // 2) for plugins
+  nsCOMPtr<nsIPluginInstance> pluginInstance ;
+  aFrame->GetPluginInstance(*getter_AddRefs(pluginInstance));
+  if (pluginInstance) {
+    nsCOMPtr<nsIWeakReference> weakShell (do_GetWeakReference(aShell));
+    CreateHTMLPluginAccessible(aNode, weakShell, _retval);
+    return NS_OK;
+  }
+
+  // 3) for images and imagemaps
+  nsCOMPtr<nsIPresContext> context;
+  aShell->GetPresContext(getter_AddRefs(context));
+  if (!context)
+    return NS_ERROR_FAILURE;
+  // we have the object frame, get the image frame
+  nsIFrame *frame;
+  aFrame->FirstChild(context, nsnull, &frame);
+  CreateHTMLImageAccessible(frame, _retval);
+  if (*_retval)
+    return NS_OK;
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsAccessibilityService::CreateXULTreeAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  *_retval = new nsXULTreeAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsAccessibilityService::CreateXULTreeColumnsAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  *_retval = new nsXULTreeColumnsAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsAccessibilityService::CreateXULTreeColumnitemAccessible(nsIDOMNode *aNode, nsIAccessible **_retval)
+{
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIWeakReference> weakShell;
+  GetShellFromNode(aNode, getter_AddRefs(weakShell));
+
+  *_retval = new nsXULTreeColumnitemAccessible(aNode, weakShell);
+  if (! *_retval) 
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*_retval);
+#else
+  *_retval = nsnull;
+#endif // MOZ_XUL
+  return NS_OK;
+}
 
 /**
   * GetAccessibleFor - get an nsIAccessible from a DOM node
@@ -1078,29 +1416,33 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
 
   nsCOMPtr<nsIAccessible> newAcc;
 
-#ifdef MOZ_XUL
-  nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(aNode)); 
-  if (xulElement) {
 #ifdef DEBUG_aaronl
-    // Please leave this in for now, it's a convenient debugging method
-    nsAutoString name;
-    aNode->GetLocalName(name);
-    if (name.Equals(NS_LITERAL_STRING("dropmarker"))) 
-      printf("## aaronl debugging tag name\n");
+  // Please leave this in for now, it's a convenient debugging method
+  nsAutoString name;
+  aNode->GetLocalName(name);
+  if (name.Equals(NS_LITERAL_STRING("INPUT"))) 
+    printf("## aaronl debugging tag name\n");
 
-    nsAutoString className;
-    xulElement->GetAttribute(NS_LITERAL_STRING("class"), className);
-    if (className.Equals(NS_LITERAL_STRING("toolbarbutton-menubutton-dropmarker")))
+  nsAutoString attrib;
+  nsCOMPtr<nsIDOMElement> element(do_QueryInterface(aNode));
+  if (element) {
+    element->GetAttribute(NS_LITERAL_STRING("type"), attrib);
+    if (attrib.Equals(NS_LITERAL_STRING("checkbox")))
       printf("## aaronl debugging attribute\n");
+  }
 #endif
-    nsCOMPtr<nsIAccessibleProvider> accProv(do_QueryInterface(aNode));
-    if (accProv) {
-      accProv->GetAccessible(_retval); 
-      if (*_retval)
-        return NS_OK;
-    }
+  nsCOMPtr<nsIAccessibleProvider> accProv(do_QueryInterface(aNode));
+  if (accProv) {
+    accProv->GetAccessible(_retval); 
+    if (*_retval)
+      return NS_OK;
     return NS_ERROR_FAILURE;
   }
+
+#ifdef MOZ_XUL
+  nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(aNode)); 
+  if (xulElement) 
+    return NS_ERROR_FAILURE;
 #endif // MOZ_XUL
 
   // ---- Get the document for this node  ----
@@ -1157,12 +1499,20 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
 
   NS_ASSERTION(content, "GetAccessibleFor() called with no content.");
 
-  // ---- Try using frame to get IAccessible ----
+  // ---- Try using frame to get nsIAccessible ----
   nsIFrame* frame = nsnull;
   shell->GetPrimaryFrameFor(content, &frame);
   if (!frame)
     return NS_ERROR_FAILURE;
 
+  // ---- object/embed/applet tags all use nsObjectFrames ----
+  nsCOMPtr<nsIAtom> frameType;
+  frame->GetFrameType(getter_AddRefs(frameType));
+  if (frameType.get() == nsLayoutAtoms::objectFrame) {
+    nsObjectFrame* objectFrame = NS_STATIC_CAST(nsObjectFrame*, frame);
+    return GetHTMLObjectAccessibleFor(aNode, shell, objectFrame, _retval);
+  }
+
   frame->GetAccessible(getter_AddRefs(newAcc));
 
   // ---- If link, create link accessible ----
@@ -1175,21 +1525,22 @@ NS_IMETHODIMP nsAccessibilityService::GetAccessibleFor(nsIDOMNode *aNode,
     }
   }
 
-  // ---- If <select> option, create select option accessible
+  // ---- If <select> <option>, create select option accessible
+    
   if (!newAcc) {
     nsCOMPtr<nsIDOMHTMLOptionElement> optionElement(do_QueryInterface(aNode));
     if (optionElement) {
-      // nsHTMLSelectionOptionAccessible's must be created via the parent
-      nsCOMPtr<nsIDOMNode> parentNode;
-      aNode->GetParentNode(getter_AddRefs(parentNode));
-      if (parentNode) {
-        nsCOMPtr<nsIAccessible> parentAccessible;
-        GetAccessibleFor(parentNode, getter_AddRefs(parentAccessible));
-        if (parentAccessible) {
-          nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(shell));
-          newAcc = new nsHTMLSelectOptionAccessible(parentAccessible, aNode, weakShell);
-        }
-      }
+      nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(shell));
+      newAcc = new nsHTMLSelectOptionAccessible(nsnull, aNode, weakShell);
+    }
+  }
+  // See if this is an <optgroup>, 
+  // create the accessible for the optgroup.
+  if (!newAcc) {
+    nsCOMPtr<nsIDOMHTMLOptGroupElement> optGroupElement(do_QueryInterface(aNode));
+    if (optGroupElement) {
+      nsCOMPtr<nsIWeakReference> weakShell(do_GetWeakReference(shell));
+      newAcc = new nsHTMLSelectOptGroupAccessible(nsnull, aNode, weakShell);
     }
   }
 
