@@ -62,7 +62,6 @@
 #include "nsISelection.h"
 #include "nsISelectionController.h"
 #include "nsIServiceManager.h"
-#include "nsIStringBundle.h"
 #include "nsXPIDLString.h"
 
 #include "nsIDOMComment.h"
@@ -88,6 +87,7 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMHTMLInputElement.h"
 #include "nsIDOMHTMLLabelElement.h"
+#include "nsIDOMHTMLObjectElement.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMXULButtonElement.h"
 #include "nsIDOMXULCheckboxElement.h"
@@ -97,6 +97,7 @@
 #include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
 #include "nsString.h"
+#include "nsIPref.h"
 
 // IFrame Helpers
 #include "nsIDocShell.h"
@@ -111,11 +112,13 @@
 #include "nsIDOMCharacterData.h"
 #endif
 
+static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
+
 //#define DEBUG_LEAKS
 
-#ifdef DEBUG_LEAKS
-static gnsAccessibles = 0;
-#endif
+PRUint32 nsAccessible::gInstanceCount = 0;
+nsIStringBundle *nsAccessible::gStringBundle = 0;
+nsIStringBundle *nsAccessible::gKeyStringBundle = 0;
 
 static NS_DEFINE_CID(kStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
 
@@ -423,7 +426,7 @@ PRBool nsAccessibleTreeWalker::GetAccessible()
  * Class nsAccessible
  */
 
-//-------------------------`----------------------------
+//-----------------------------------------------------
 // construction 
 //-----------------------------------------------------
 nsAccessible::nsAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell): mDOMNode(aNode), mPresShell(aShell), mSiblingIndex(eSiblingsUninitialized)
@@ -448,9 +451,21 @@ nsAccessible::nsAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell): mDOMNod
    }
 #endif
 
+   ++gInstanceCount;
 #ifdef DEBUG_LEAKS
-  printf("nsAccessibles=%d\n", ++gnsAccessibles);
+  printf("nsAccessibles=%d\n", gInstanceCount);
 #endif
+
+  if (gInstanceCount == 1) {
+    nsresult rv;
+    nsCOMPtr<nsIStringBundleService> stringBundleService(do_GetService(kStringBundleServiceCID, &rv));
+    if (stringBundleService) {
+      stringBundleService->CreateBundle(ACCESSIBLE_BUNDLE_URL, &gStringBundle);
+      NS_IF_ADDREF(gStringBundle);
+      stringBundleService->CreateBundle(PLATFORM_KEYS_BUNDLE_URL, &gKeyStringBundle);
+      NS_IF_ADDREF(gKeyStringBundle);
+    }
+  }
 }
 
 //-----------------------------------------------------
@@ -458,13 +473,18 @@ nsAccessible::nsAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell): mDOMNod
 //-----------------------------------------------------
 nsAccessible::~nsAccessible()
 {
+  if (--gInstanceCount == 0) {
+    NS_IF_RELEASE(gStringBundle);
+    NS_IF_RELEASE(gKeyStringBundle);
+  }
+
 #ifdef DEBUG_LEAKS
-  printf("nsAccessibles=%d\n", --gnsAccessibles);
+  printf("nsAccessibles=%d\n", gInstanceCount);
 #endif
 }
 
 
-NS_IMETHODIMP nsAccessible::GetAccName(nsAWritableString& _retval)
+NS_IMETHODIMP nsAccessible::GetAccName(nsAString& _retval)
 {
   nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(mDOMNode));
   if (elt) 
@@ -472,6 +492,45 @@ NS_IMETHODIMP nsAccessible::GetAccName(nsAWritableString& _retval)
   return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP nsAccessible::GetAccKeyboardShortcut(nsAString& _retval)
+{
+  static PRInt32 gGeneralAccesskeyModifier = -1;  // magic value of -1 indicates unitialized state
+
+  nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(mDOMNode));
+  if (elt) {
+    nsAutoString accesskey;
+    elt->GetAttribute(NS_LITERAL_STRING("accesskey"), accesskey);
+    if (accesskey.IsEmpty())
+      return NS_OK;
+
+    if (gGeneralAccesskeyModifier == -1) {  // Need to initialize cached global accesskey pref
+      gGeneralAccesskeyModifier = 0;
+      nsresult result;
+      nsCOMPtr<nsIPref> prefService(do_GetService(kPrefCID, &result));
+      if (NS_SUCCEEDED(result) && prefService)
+        prefService->GetIntPref("ui.key.generalAccessKey", &gGeneralAccesskeyModifier);
+    }
+    nsAutoString propertyKey;
+    switch (gGeneralAccesskeyModifier) {
+      case nsIDOMKeyEvent::DOM_VK_CONTROL: propertyKey = NS_LITERAL_STRING("VK_CONTROL"); break;
+      case nsIDOMKeyEvent::DOM_VK_ALT: propertyKey = NS_LITERAL_STRING("VK_ALT"); break;
+      case nsIDOMKeyEvent::DOM_VK_META: propertyKey = NS_LITERAL_STRING("VK_META"); break;
+    }
+    if (!propertyKey.IsEmpty())
+      nsAccessible::GetFullKeyName(propertyKey, accesskey, _retval);
+    else
+      _retval= accesskey;
+    return NS_OK;
+  }
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsAccessible::GetAccId(PRInt32 *aAccId)
+{
+  *aAccId = - NS_REINTERPRET_CAST(PRInt32, (mDOMNode.get()));
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsAccessible::CacheOptimizations(nsIAccessible *aParent, PRInt32 aSiblingIndex, nsIDOMNodeList *aSiblingList)
 {
   if (aParent)
@@ -575,32 +634,34 @@ NS_IMETHODIMP nsAccessible::GetAccChildCount(PRInt32 *aAccChildCount)
   return NS_OK;  
 }
 
-nsresult nsAccessible::GetTranslatedString(const nsAReadableString& aKey, nsAWritableString& aStringOut)
+nsresult nsAccessible::GetTranslatedString(const nsAString& aKey, nsAString& aStringOut)
 {
-  static nsCOMPtr<nsIStringBundle> stringBundle;
-  static PRBool firstTime = PR_TRUE;
-
-  if (firstTime) {
-    firstTime = PR_FALSE;
-    nsresult rv;
-    nsCOMPtr<nsIStringBundleService> stringBundleService = 
-             do_GetService(kStringBundleServiceCID, &rv);
-    if (!stringBundleService) { 
-      NS_WARNING("ERROR: Failed to get StringBundle Service instance.\n");
-      return NS_ERROR_FAILURE;
-    }
-    stringBundleService->CreateBundle(ACCESSIBLE_BUNDLE_URL, getter_AddRefs(stringBundle));
-  }
-
   nsXPIDLString xsValue;
-  if (!stringBundle || 
-    NS_FAILED(stringBundle->GetStringFromName(PromiseFlatString(aKey).get(), getter_Copies(xsValue)))) 
+
+  if (!gStringBundle || 
+    NS_FAILED(gStringBundle->GetStringFromName(PromiseFlatString(aKey).get(), getter_Copies(xsValue)))) 
     return NS_ERROR_FAILURE;
 
   aStringOut.Assign(xsValue);
   return NS_OK;
 }
 
+nsresult nsAccessible::GetFullKeyName(const nsAString& aModifierName, const nsAString& aKeyName, nsAString& aStringOut)
+{
+  nsXPIDLString modifierName, separator;
+
+  if (!gKeyStringBundle ||
+      NS_FAILED(gKeyStringBundle->GetStringFromName(PromiseFlatString(aModifierName).get(), 
+                                                    getter_Copies(modifierName))) ||
+      NS_FAILED(gKeyStringBundle->GetStringFromName(PromiseFlatString(NS_LITERAL_STRING("MODIFIER_SEPARATOR")).get(), 
+                                                    getter_Copies(separator)))) {
+    return NS_ERROR_FAILURE;
+  }
+
+  aStringOut = modifierName + separator + aKeyName; 
+  return NS_OK;
+}
+
 PRBool nsAccessible::IsPartiallyVisible() 
 {
   // We need to know if at least a kMinPixels around the object is visible
@@ -651,7 +712,7 @@ PRBool nsAccessible::IsPartiallyVisible()
   float p2t;
   presContext->GetPixelsToTwips(&p2t);
   PRBool isVisible = PR_FALSE;
-  viewManager->IsRectVisible(containingView, relFrameRect, kMinPixels * p2t, &isVisible);
+  viewManager->IsRectVisible(containingView, relFrameRect, NS_STATIC_CAST(PRUint16, (kMinPixels * p2t)), &isVisible);
 
   return isVisible;
 }
@@ -716,7 +777,7 @@ NS_IMETHODIMP nsAccessible::GetAccState(PRUint32 *aAccState)
 }
 
   /* readonly attribute boolean accFocused; */
-NS_IMETHODIMP nsAccessible::GetAccFocused(nsIAccessible * *aAccFocused) 
+NS_IMETHODIMP nsAccessible::GetAccFocused(nsIAccessible **aAccFocused) 
 { 
   *aAccFocused = nsnull;
 
@@ -995,7 +1056,9 @@ NS_IMETHODIMP nsAccessible::AccGetBounds(PRInt32 *x, PRInt32 *y, PRInt32 *width,
   if (presContext) {
     nsRect orgRectPixels, pageRectPixels;
     GetScreenOrigin(presContext, aBoundingFrame, &orgRectPixels);
-    GetScrollOffset(&pageRectPixels);
+    nsCOMPtr<nsIAccessibleEventReceiver> accessibleEventReceiver(do_QueryInterface(this));
+    if (!accessibleEventReceiver)        // Only the root accessible object supports this interface
+      GetScrollOffset(&pageRectPixels);  // Don't add scroll offsets for the root accessible
     *x += orgRectPixels.x - pageRectPixels.x;
     *y += orgRectPixels.y - pageRectPixels.y;
   }
@@ -1138,7 +1201,7 @@ NS_IMETHODIMP nsAccessible::AccTakeFocus()
   return NS_OK;
 }
 
-NS_IMETHODIMP nsAccessible::AppendStringWithSpaces(nsAWritableString *aFlatString, nsAReadableString& textEquivalent)
+NS_IMETHODIMP nsAccessible::AppendStringWithSpaces(nsAString *aFlatString, const nsAString& textEquivalent)
 {
   // Insert spaces to insure that words from controls aren't jammed together
   if (!textEquivalent.IsEmpty()) {
@@ -1158,7 +1221,7 @@ NS_IMETHODIMP nsAccessible::AppendStringWithSpaces(nsAWritableString *aFlatStrin
  * that node's decendants.
  */
 
-NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent, nsAWritableString *aFlatString)
+NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent, nsAString *aFlatString)
 {
   nsAutoString textEquivalent;
   nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(aContent));
@@ -1218,11 +1281,15 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
     return NS_OK;
   }
 
-  nsCOMPtr<nsIDOMHTMLImageElement> imageContent(do_QueryInterface(aContent));
   nsCOMPtr<nsIDOMHTMLInputElement> inputContent;
-  if (!imageContent)
+  nsCOMPtr<nsIDOMHTMLObjectElement> objectContent;
+  nsCOMPtr<nsIDOMHTMLImageElement> imageContent(do_QueryInterface(aContent));
+  if (!imageContent) {
     inputContent = do_QueryInterface(aContent);
-  if (imageContent || inputContent) {
+    if (!inputContent)
+      objectContent = do_QueryInterface(aContent);
+  }
+  if (imageContent || inputContent || objectContent) {
     nsCOMPtr<nsIDOMElement> elt(do_QueryInterface(aContent));
     NS_ASSERTION(elt, "No DOM element for content node!");
     elt->GetAttribute(NS_LITERAL_STRING("alt"), textEquivalent);
@@ -1232,6 +1299,8 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
       elt->GetAttribute(NS_LITERAL_STRING("name"), textEquivalent);
     if (textEquivalent.IsEmpty())
       elt->GetAttribute(NS_LITERAL_STRING("src"), textEquivalent);
+    if (textEquivalent.IsEmpty())
+      elt->GetAttribute(NS_LITERAL_STRING("data"), textEquivalent); // for <object>s with images
     return AppendStringWithSpaces(aFlatString, textEquivalent);
   }
 
@@ -1239,7 +1308,7 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromContentNode(nsIContent *aContent
 }
 
 
-NS_IMETHODIMP nsAccessible::AppendFlatStringFromSubtree(nsIContent *aContent, nsAWritableString *aFlatString)
+NS_IMETHODIMP nsAccessible::AppendFlatStringFromSubtree(nsIContent *aContent, nsAString *aFlatString)
 {
   // Depth first search for all text nodes that are decendants of content node.
   // Append all the text into one flat string
@@ -1266,7 +1335,7 @@ NS_IMETHODIMP nsAccessible::AppendFlatStringFromSubtree(nsIContent *aContent, ns
   * Checks the label's value first then makes a call to get the 
   *  text from the children if the value is not set.
   */
-NS_IMETHODIMP nsAccessible::AppendLabelText(nsIDOMNode *aLabelNode, nsAWritableString& _retval)
+NS_IMETHODIMP nsAccessible::AppendLabelText(nsIDOMNode *aLabelNode, nsAString& _retval)
 {
   NS_ASSERTION(aLabelNode, "Label Node passed in is null");
   nsCOMPtr<nsIDOMXULLabelElement> labelNode(do_QueryInterface(aLabelNode));
@@ -1286,7 +1355,7 @@ NS_IMETHODIMP nsAccessible::AppendLabelText(nsIDOMNode *aLabelNode, nsAWritableS
 /**
   * Called for HTML work only
   */
-NS_IMETHODIMP nsAccessible::AppendLabelFor(nsIContent *aLookNode, nsAReadableString *aId, nsAWritableString *aLabel)
+NS_IMETHODIMP nsAccessible::AppendLabelFor(nsIContent *aLookNode, const nsAString *aId, nsAString *aLabel)
 {
   PRInt32 numChildren = 0;
 
@@ -1320,7 +1389,7 @@ NS_IMETHODIMP nsAccessible::AppendLabelFor(nsIContent *aLookNode, nsAReadableStr
   *   the DOM tree to the form, concatonating label elements as it goes. Then checks for
   *   labels with the for="controlID" property.
   */
-NS_IMETHODIMP nsAccessible::GetHTMLAccName(nsAWritableString& _retval)
+NS_IMETHODIMP nsAccessible::GetHTMLAccName(nsAString& _retval)
 {
   nsCOMPtr<nsIContent> walkUpContent(do_QueryInterface(mDOMNode));
   nsCOMPtr<nsIDOMHTMLLabelElement> labelElement;
@@ -1375,7 +1444,7 @@ NS_IMETHODIMP nsAccessible::GetHTMLAccName(nsAWritableString& _retval)
   *  the child label for its Name.
   */
 /* wstring getAccName (); */
-NS_IMETHODIMP nsAccessible::GetXULAccName(nsAWritableString& _retval)
+NS_IMETHODIMP nsAccessible::GetXULAccName(nsAString& _retval)
 {
   nsresult rv;
   nsAutoString label;
