@@ -74,8 +74,14 @@
 #include "nsCURILoader.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIScriptGlobalObject.h"
+#include "nsIDOMWindow.h"
+#include "nsIViewManager.h"
+#include "nsIScrollableView.h"
 #include "nsIDOMXULSelectCntrlEl.h"
 #include "nsIDOMXULSelectCntrlItemEl.h"
+#include "nsXULTreeAccessible.h"
+#include "nsITreeSelection.h"
+#include "nsAccessibilityService.h"
 
 NS_INTERFACE_MAP_BEGIN(nsRootAccessible)
   NS_INTERFACE_MAP_ENTRY(nsIAccessibleDocument)
@@ -85,6 +91,7 @@ NS_INTERFACE_MAP_BEGIN(nsRootAccessible)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
   NS_INTERFACE_MAP_ENTRY(nsITimerCallback)
   NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener) 
+  NS_INTERFACE_MAP_ENTRY(nsIScrollPositionListener) 
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIDOMFormListener)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventListener, nsIDOMFormListener)
 NS_INTERFACE_MAP_END_INHERITING(nsAccessible)
@@ -105,9 +112,8 @@ PRUint32 nsRootAccessible::gInstanceCount = 0;
 //-----------------------------------------------------
 nsRootAccessible::nsRootAccessible(nsIWeakReference* aShell):nsAccessible(nsnull,aShell), 
   nsDocAccessibleMixin(aShell), mAccService(do_GetService("@mozilla.org/accessibilityService;1")),
-  mBusy(eBusyStateUninitialized)
+  mBusy(eBusyStateUninitialized), mListener(nsnull), mScrollPositionChangedTicks(0), mScrollablePresShells(nsnull)
 {
-  mListener = nsnull;
   nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
   if (shell) {
     shell->GetDocument(getter_AddRefs(mDocument));
@@ -133,10 +139,12 @@ nsRootAccessible::~nsRootAccessible()
 
   nsLayoutAtoms::ReleaseAtoms();
   RemoveAccessibleEventListener();
+  if (mScrollablePresShells)
+    delete mScrollablePresShells;
 }
 
   /* attribute wstring accName; */
-NS_IMETHODIMP nsRootAccessible::GetAccName(nsAWritableString& aAccName) 
+NS_IMETHODIMP nsRootAccessible::GetAccName(nsAString& aAccName) 
 { 
   return GetTitle(aAccName);
 }
@@ -201,6 +209,20 @@ NS_IMETHODIMP nsRootAccessible::GetAccRole(PRUint32 *aAccRole)
   */
 
   *aAccRole = ROLE_PANE;
+
+  // If it's a <dialog>, use ROLE_DIALOG instead
+  nsCOMPtr<nsIContent> rootContent;
+  mDocument->GetRootContent(getter_AddRefs(rootContent));
+  if (rootContent) {
+    nsCOMPtr<nsIDOMElement> rootElement(do_QueryInterface(rootContent));
+    if (rootElement) {
+      nsAutoString name;
+      rootElement->GetLocalName(name);
+      if (name.Equals(NS_LITERAL_STRING("dialog"))) 
+        *aAccRole = ROLE_DIALOG;
+    }
+  }
+
   return NS_OK;
 }
 
@@ -212,7 +234,7 @@ NS_IMETHODIMP nsRootAccessible::GetAccState(PRUint32 *aAccState)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRootAccessible::GetAccValue(nsAWritableString& aAccValue)
+NS_IMETHODIMP nsRootAccessible::GetAccValue(nsAString& aAccValue)
 {
   return GetURL(aAccValue);
 }
@@ -225,6 +247,144 @@ void nsRootAccessible::Notify(nsITimer *timer)
     if (mListener)
       mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this);
   }
+
+  if (mScrollPositionChangedTicks) {
+    if (++mScrollPositionChangedTicks > 2) {
+      // Whenever scroll position changes, mScrollPositionChangeTicks gets reset to 1
+      // We only want to fire accessibilty scroll event when scrolling stops or pauses
+      // Therefore, we wait for no scroll events to occur between 2 ticks of this timer
+      // That indicates a pause in scrolling, so we fire the accessibilty scroll event
+      nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mLastScrolledPresShell));
+      if (mListener && presShell) {
+        nsCOMPtr<nsIAccessible> docAccessible;
+        if (mPresShell == mLastScrolledPresShell)
+          docAccessible = this;  // Fast way, don't need to create new accessible to fire event on the root
+        else {
+          // Need to create accessible for document that had scrolling occur in it
+          nsCOMPtr<nsIDocument> doc;
+          presShell->GetDocument(getter_AddRefs(doc));
+          nsCOMPtr<nsIDOMNode> docNode(do_QueryInterface(doc));
+          mAccService->GetAccessibleFor(docNode, getter_AddRefs(docAccessible));
+        }
+
+        if (docAccessible)
+          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SCROLLINGEND, docAccessible);
+      }
+      mScrollPositionChangedTicks = 0;
+      mLastScrolledPresShell = nsnull;
+    }
+  }
+}
+  
+void nsRootAccessible::AddScrollListener(nsIPresShell *aPresShell)
+{
+  nsCOMPtr<nsIViewManager> vm;
+
+  if (aPresShell)
+    aPresShell->GetViewManager(getter_AddRefs(vm));
+
+  nsIScrollableView* scrollableView = nsnull;
+  if (vm)
+    vm->GetRootScrollableView(&scrollableView);
+
+  if (!scrollableView)
+    return;
+
+  if (!mScrollablePresShells)
+    mScrollablePresShells = new nsSupportsHashtable(SCROLL_HASH_START_SIZE, PR_TRUE);  // PR_TRUE = thread safe hash table
+
+  if (mScrollablePresShells) {
+    nsISupportsKey key(scrollableView);
+    nsCOMPtr<nsISupports> supports(dont_AddRef(NS_STATIC_CAST(nsISupports*, mScrollablePresShells->Get(&key))));
+    if (supports)  // This scroll view is already being listened to, remove it. We will re-add it w/ current pres shell
+      RemoveScrollListener(aPresShell);
+
+    // Add to hash table, so we can retrieve correct pres shell later
+    nsCOMPtr<nsIWeakReference> weakShell;
+    weakShell = do_GetWeakReference(aPresShell);
+    mScrollablePresShells->Put(&key, weakShell);
+
+    // Add listener
+    scrollableView->AddScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
+  }
+}
+
+void nsRootAccessible::RemoveScrollListener(nsIPresShell *aPresShell)
+{
+  nsCOMPtr<nsIViewManager> vm;
+
+  if (aPresShell)
+    aPresShell->GetViewManager(getter_AddRefs(vm));
+
+  nsIScrollableView* scrollableView = nsnull;
+  if (vm)
+    vm->GetRootScrollableView(&scrollableView);
+
+  if (scrollableView)
+    scrollableView->RemoveScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, this));
+
+  if (mScrollablePresShells) {
+    nsISupportsKey key(scrollableView);
+    nsCOMPtr<nsISupports> supp;
+    mScrollablePresShells->Remove(&key, getter_AddRefs(supp));
+  }
+}
+
+PRBool PR_CALLBACK
+RemoveScrollListenerEnum(nsHashKey *aKey, void *aData, void* aClosure)
+{
+  nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(NS_STATIC_CAST(nsIWeakReference*, aData)));
+
+  // If presShell gone, then our nsIScrollPositionListener will also already be gone
+  if (presShell) {
+    nsCOMPtr<nsIViewManager> vm;
+    if (presShell)
+      presShell->GetViewManager(getter_AddRefs(vm));
+
+    nsIScrollableView* scrollableView = nsnull;
+    if (vm)
+      vm->GetRootScrollableView(&scrollableView);
+
+    if (scrollableView)
+      scrollableView->RemoveScrollPositionListener(NS_STATIC_CAST(nsIScrollPositionListener *, aClosure));
+  }
+
+  return PR_TRUE;
+}
+
+NS_IMETHODIMP nsRootAccessible::ScrollPositionWillChange(nsIScrollableView *aView, nscoord aX, nscoord aY)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsRootAccessible::ScrollPositionDidChange(nsIScrollableView *aScrollableView, nscoord aX, nscoord aY)
+{
+  if (mListener) {
+    if (!mScrollablePresShells)
+      return NS_OK;
+    nsISupportsKey key(aScrollableView);
+    nsCOMPtr<nsIWeakReference> weakShell(dont_AddRef(NS_STATIC_CAST(nsIWeakReference*, mScrollablePresShells->Get(&key))));
+    if (!weakShell)
+      return NS_OK;
+    if (mTimer && (mBusy != eBusyStateDone || (mScrollPositionChangedTicks && mLastScrolledPresShell != weakShell))) {
+      // We need to say finish up our current time work and start a new timer
+      // either because we haven't yet fired our finished loading event, or 
+      // a scroll event from another pres shell is still waiting to be fired
+      Notify(mTimer);
+      mTimer->Cancel();
+    }
+    // Start new timer, if the timer cycles at least 1 full cycle without more scroll position changes, 
+    // then the ::Notify() method will fire the accessibility event for scroll position changes
+    nsresult rv;
+    mTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
+    if (NS_SUCCEEDED(rv)) {
+      const PRUint32 kScrollPosCheckWait = 50;
+      mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kScrollPosCheckWait, NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_SLACK);
+    }
+    mScrollPositionChangedTicks = 1;
+    mLastScrolledPresShell = weakShell;
+  }
+  return NS_OK;
 }
 
 void nsRootAccessible::StartDocReadyTimer()
@@ -258,14 +418,16 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
     // capture Form change events 
     rv = target->AddEventListener(NS_LITERAL_STRING("change"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
+
     // add ourself as a CheckboxStateChange listener (custom event fired in nsHTMLInputElement.cpp)
     rv = target->AddEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
-    // add ourself as a RadiobuttonStateChange listener (custom event fired in nsHTMLInputElement.cpp)
-    rv = target->AddEventListener(NS_LITERAL_STRING("RadiobuttonStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
+
+    // add ourself as a RadioStateChange Listener ( custom event fired in in nsHTMLInputElement.cpp  & radio.xml)
+    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
-    rv = target->AddEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+    rv = target->AddEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to register listener");
 
     rv = target->AddEventListener(NS_LITERAL_STRING("popupshowing"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
@@ -294,12 +456,12 @@ NS_IMETHODIMP nsRootAccessible::AddAccessibleEventListener(nsIAccessibleEventLis
         nsCOMPtr<nsIDocShell> docShell(do_QueryInterface(container));
         if (docShell) {
           mWebProgress = do_GetInterface(docShell);
-          mWebProgress->AddProgressListener(this);
+          mWebProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_LOCATION | 
+                                                  nsIWebProgress::NOTIFY_STATE_DOCUMENT);
         }
       }
     }
     NS_ASSERTION(mWebProgress, "Could not get nsIWebProgress for nsRootAccessible");
-    mWebProgress->AddProgressListener(this);
   }
 
   return NS_OK;
@@ -314,10 +476,11 @@ NS_IMETHODIMP nsRootAccessible::RemoveAccessibleEventListener()
       target->RemoveEventListener(NS_LITERAL_STRING("focus"), NS_STATIC_CAST(nsIDOMFocusListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("change"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("CheckboxStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
-      target->RemoveEventListener(NS_LITERAL_STRING("RadiobuttonStateChange"), NS_STATIC_CAST(nsIDOMFormListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popupshowing"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
       target->RemoveEventListener(NS_LITERAL_STRING("DOMMenuItemActive"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("RadioStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
+      target->RemoveEventListener(NS_LITERAL_STRING("ListitemStateChange"), NS_STATIC_CAST(nsIDOMXULListener*, this), PR_TRUE);
     }
  
     if (mTimer) {
@@ -329,6 +492,10 @@ NS_IMETHODIMP nsRootAccessible::RemoveAccessibleEventListener()
       mWebProgress->RemoveProgressListener(this);
       mWebProgress = nsnull;
     }
+
+    if (mScrollablePresShells)
+      mScrollablePresShells->Enumerate(RemoveScrollListenerEnum, NS_STATIC_CAST(nsIScrollPositionListener*, this));
+
     mListener = nsnull;
   }
 
@@ -359,7 +526,7 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
     // Check to see if it's a select element. If so, need the currently focused option
     nsCOMPtr<nsIDOMHTMLSelectElement> selectElement(do_QueryInterface(targetNode));
     if (selectElement)     // ----- Target Node is an HTML <select> element ------
-      nsHTMLSelectOptionAccessible::GetFocusedOptionNode(mPresShell, targetNode, optionTargetNode);
+      nsHTMLSelectOptionAccessible::GetFocusedOptionNode(targetNode, optionTargetNode);
 
     // for focus events on Radio Groups we give the focus to the selected button
     nsCOMPtr<nsIDOMXULSelectControlElement> selectControl(do_QueryInterface(targetNode));
@@ -369,14 +536,37 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
       optionTargetNode = do_QueryInterface(selectItem);
     }
 
+    // If it's a tree element, need the currently selected item
+    PRInt32 treeIndex = -1;
+    nsCOMPtr<nsITreeBoxObject> treeBox;
+    nsXULTreeAccessible::GetTreeBoxObject(targetNode, getter_AddRefs(treeBox));
+    if (treeBox) {
+      nsCOMPtr<nsITreeSelection> selection;
+      treeBox->GetSelection(getter_AddRefs(selection));
+      if (selection)
+        selection->GetCurrentIndex(&treeIndex);
+    }
+
     nsAutoString eventType;
     aEvent->GetType(eventType);
 
     nsCOMPtr<nsIAccessible> accessible;
 
     if (NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
+      // tree event
+      if (treeBox && treeIndex >= 0 && 
+          (eventType.EqualsIgnoreCase("DOMMenuItemActive") || eventType.EqualsIgnoreCase("select"))) {
+        nsCOMPtr<nsIWeakReference> weakShell;
+        nsAccessibilityService::GetShellFromNode(targetNode, getter_AddRefs(weakShell));
+        accessible = new nsXULTreeitemAccessible(accessible, targetNode, weakShell, treeIndex);
+        if (!accessible)
+          return NS_ERROR_OUT_OF_MEMORY;
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
+        return NS_OK;
+      }
+
       if (eventType.EqualsIgnoreCase("focus") || eventType.EqualsIgnoreCase("DOMMenuItemActive")) { 
-        if (selectControl && optionTargetNode &&
+        if (optionTargetNode &&
             NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) {
           FireAccessibleFocusEvent(accessible, optionTargetNode);
         }
@@ -384,23 +574,26 @@ NS_IMETHODIMP nsRootAccessible::HandleEvent(nsIDOMEvent* aEvent)
           FireAccessibleFocusEvent(accessible, targetNode);
       }
       else if (eventType.EqualsIgnoreCase("change")) {
-        if (optionTargetNode) {   // Set to current option only for HTML selects
-          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_SELECTION, accessible);
-          if (NS_SUCCEEDED(mAccService->GetAccessibleFor(optionTargetNode, getter_AddRefs(accessible)))) 
-            FireAccessibleFocusEvent(accessible, optionTargetNode);
-        }
-        else 
+        if (!selectControl)   // Don't use onchange to fire EVENT_STATE_CHANGE events for selects
           mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
       }
-      else if (eventType.EqualsIgnoreCase("CheckboxStateChange") || 
-               eventType.EqualsIgnoreCase("RadiobuttonStateChange"))
+      else if (eventType.EqualsIgnoreCase("ListitemStateChange")) {
         mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
+      }
+      else if (eventType.EqualsIgnoreCase("CheckboxStateChange")) { 
+        mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
+      }
       else if (eventType.EqualsIgnoreCase("RadioStateChange") ) {
+        // first the XUL radio buttons
         if (targetNode &&
             NS_SUCCEEDED(mAccService->GetAccessibleFor(targetNode, getter_AddRefs(accessible)))) {
           mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
           mListener->HandleEvent(nsIAccessibleEventListener::EVENT_FOCUS, accessible);
         }
+        else { // for the html radio buttons -- apparently the focus code just works. :-)
+          mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, accessible);
+        }
       }
       else if (eventType.EqualsIgnoreCase("popupshowing")) 
         mListener->HandleEvent(nsIAccessibleEventListener::EVENT_MENUPOPUPSTART, accessible);
@@ -452,7 +645,10 @@ NS_IMETHODIMP nsRootAccessible::Change(nsIDOMEvent* aEvent)
 }
 
 // gets Select events when text is selected in a textarea or input
-NS_IMETHODIMP nsRootAccessible::Select(nsIDOMEvent* aEvent) { return NS_OK; }
+NS_IMETHODIMP nsRootAccessible::Select(nsIDOMEvent* aEvent) 
+{
+  return HandleEvent(aEvent);
+}
 
 // gets Input events when text is entered or deleted in a textarea or input
 NS_IMETHODIMP nsRootAccessible::Input(nsIDOMEvent* aEvent) { return NS_OK; }
@@ -477,27 +673,27 @@ NS_IMETHODIMP nsRootAccessible::CommandUpdate(nsIDOMEvent* aEvent) { return NS_O
 
 // ------- nsIAccessibleDocument Methods (5) ---------------
 
-NS_IMETHODIMP nsRootAccessible::GetURL(nsAWritableString& aURL)
+NS_IMETHODIMP nsRootAccessible::GetURL(nsAString& aURL)
 {
   return nsDocAccessibleMixin::GetURL(aURL);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetTitle(nsAWritableString& aTitle)
+NS_IMETHODIMP nsRootAccessible::GetTitle(nsAString& aTitle)
 {
   return nsDocAccessibleMixin::GetTitle(aTitle);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetMimeType(nsAWritableString& aMimeType)
+NS_IMETHODIMP nsRootAccessible::GetMimeType(nsAString& aMimeType)
 {
   return nsDocAccessibleMixin::GetMimeType(aMimeType);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetDocType(nsAWritableString& aDocType)
+NS_IMETHODIMP nsRootAccessible::GetDocType(nsAString& aDocType)
 {
   return nsDocAccessibleMixin::GetDocType(aDocType);
 }
 
-NS_IMETHODIMP nsRootAccessible::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAWritableString& aNameSpaceURI)
+NS_IMETHODIMP nsRootAccessible::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   return nsDocAccessibleMixin::GetNameSpaceURIForID(aNameSpaceID, aNameSpaceURI);
 }
@@ -508,8 +704,24 @@ NS_IMETHODIMP nsRootAccessible::GetDocument(nsIDocument **doc)
 }
 
 NS_IMETHODIMP nsRootAccessible::OnStateChange(nsIWebProgress *aWebProgress,
-  nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+  nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
+  if ((aStateFlags & STATE_IS_DOCUMENT) && (aStateFlags & STATE_STOP)) {
+    // Set up scroll position listener
+    nsCOMPtr<nsIDOMWindow> domWin;
+    aWebProgress->GetDOMWindow(getter_AddRefs(domWin));
+    if (domWin) {
+      nsCOMPtr<nsIDOMDocument> domDoc;
+      domWin->GetDocument(getter_AddRefs(domDoc));
+      nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
+      if (doc) {
+        nsCOMPtr<nsIPresShell> presShell;
+        doc->GetShellAt(0, getter_AddRefs(presShell));
+        AddScrollListener(presShell);
+      }
+    }
+  }
+
   return NS_OK;
 }
 
@@ -518,6 +730,7 @@ NS_IMETHODIMP nsRootAccessible::OnProgressChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress,
   PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -526,10 +739,24 @@ NS_IMETHODIMP nsRootAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, nsIURI *location)
 {
   // Load has been verified, it will occur, about to commence
-  if (mListener && mBusy != eBusyStateLoading) {
+  if (mBusy != eBusyStateLoading) {
     mBusy = eBusyStateLoading; 
-    mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this);
-    StartDocReadyTimer(); 
+    if (mListener)
+      mListener->HandleEvent(nsIAccessibleEventListener::EVENT_STATE_CHANGE, this);
+
+    // Document is going away, remove its scroll position listener
+    nsCOMPtr<nsIDOMWindow> domWin;
+    aWebProgress->GetDOMWindow(getter_AddRefs(domWin));
+    if (domWin) {
+      nsCOMPtr<nsIDOMDocument> domDoc;
+      domWin->GetDocument(getter_AddRefs(domDoc));
+      nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
+      if (doc) {
+        nsCOMPtr<nsIPresShell> presShell;
+        doc->GetShellAt(0, getter_AddRefs(presShell));
+        RemoveScrollListener(presShell);
+      }
+    }
   }
 
   return NS_OK;
@@ -539,13 +766,15 @@ NS_IMETHODIMP nsRootAccessible::OnLocationChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP nsRootAccessible::OnStatusChange(nsIWebProgress *aWebProgress,
   nsIRequest *aRequest, nsresult aStatus, const PRUnichar *aMessage)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
+/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
 NS_IMETHODIMP nsRootAccessible::OnSecurityChange(nsIWebProgress *aWebProgress,
-  nsIRequest *aRequest, PRInt32 state)
+  nsIRequest *aRequest, PRUint32 state)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -564,7 +793,7 @@ nsDocAccessibleMixin::~nsDocAccessibleMixin()
 {
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAWritableString& aURL)
+NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAString& aURL)
 { 
   nsCOMPtr<nsIPresShell> presShell;
   mDocument->GetShellAt(0, getter_AddRefs(presShell));
@@ -575,26 +804,26 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetURL(nsAWritableString& aURL)
   GetDocShellFromPS(presShell, getter_AddRefs(docShell));
 
   nsCOMPtr<nsIWebNavigation> webNav(do_GetInterface(docShell));
-  nsXPIDLCString theURL;
+  nsCAutoString theURL;
   if (webNav) {
     nsCOMPtr<nsIURI> pURI;
     webNav->GetCurrentURI(getter_AddRefs(pURI));
     if (pURI) 
-      pURI->GetSpec(getter_Copies(theURL));
+      pURI->GetSpec(theURL);
   }
   //XXXaaronl Need to use CopyUTF8toUCS2(nsDependentCString(theURL), aURL); when it's written
-  aURL.Assign(NS_ConvertUTF8toUCS2(theURL).get()); 
+  aURL.Assign(NS_ConvertUTF8toUCS2(theURL)); 
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetTitle(nsAWritableString& aTitle)
+NS_IMETHODIMP nsDocAccessibleMixin::GetTitle(nsAString& aTitle)
 {
   // This doesn't leak - we don't own the const pointer that's returned
   aTitle = *(mDocument->GetDocumentTitle());
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAWritableString& aMimeType)
+NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAString& aMimeType)
 {
   nsCOMPtr<nsIDOMNSDocument> domnsDocument(do_QueryInterface(mDocument));
   if (domnsDocument) {
@@ -603,7 +832,7 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetMimeType(nsAWritableString& aMimeType)
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAWritableString& aDocType)
+NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAString& aDocType)
 {
   nsCOMPtr<nsIXULDocument> xulDoc(do_QueryInterface(mDocument));
   nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(mDocument));
@@ -620,7 +849,7 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocType(nsAWritableString& aDocType)
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAWritableString& aNameSpaceURI)
+NS_IMETHODIMP nsDocAccessibleMixin::GetNameSpaceURIForID(PRInt16 aNameSpaceID, nsAString& aNameSpaceURI)
 {
   if (mDocument) {
     nsCOMPtr<nsINameSpaceManager> nameSpaceManager;
@@ -659,3 +888,4 @@ NS_IMETHODIMP nsDocAccessibleMixin::GetDocShellFromPS(nsIPresShell* aPresShell,
   }
   return NS_ERROR_FAILURE;
 }
+
