@@ -22,7 +22,7 @@
  * Contributor(s):
  * Original Author: Eric Vaughan (evaughan@netscape.com)
  * Contributor(s):  John Gaunt (jgaunt@netscape.com)
- *
+ *                  Kyle Yuan (kyle.yuan@sun.com)
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -42,17 +42,21 @@
 #include "nsFormControlAccessible.h"
 #include "nsIAtom.h"
 #include "nsIComboboxControlFrame.h"
-#include "nsIDOMEventReceiver.h"
-#include "nsIDOMHTMLCollection.h"
-#include "nsIDOMHTMLOptionElement.h"
+#include "nsIDOMHTMLOptGroupElement.h"
 #include "nsIDOMHTMLSelectElement.h"
+#include "nsIDOMText.h"
+#include "nsIDOMXULMultSelectCntrlEl.h"
+#include "nsIDOMXULSelectCntrlEl.h"
+#include "nsIDOMXULSelectCntrlItemEl.h"
 #include "nsIFrame.h"
 #include "nsIListControlFrame.h"
 #include "nsISelectControlFrame.h"
+#include "nsIServiceManager.h"
 #include "nsLayoutAtoms.h"
 #include "nsRootAccessible.h"
 #include "nsSelectAccessible.h"
 
+
 /** ------------------------------------------------------ */
 /**  First, the common widgets                             */
 /** ------------------------------------------------------ */
@@ -109,7 +113,32 @@ NS_IMETHODIMP nsSelectListAccessible::GetAccNextSibling(nsIAccessible **_retval)
 nsSelectOptionAccessible::nsSelectOptionAccessible(nsIAccessible* aParent, nsIDOMNode* aDOMNode, nsIWeakReference* aShell):
 nsLeafAccessible(aDOMNode, aShell)
 {
-  mParent = aParent;
+  if (aParent)
+    mParent = aParent;
+  else {
+    nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
+    nsCOMPtr<nsIDOMNode> parentNode, parentNode1;
+    nsCOMPtr<nsIAccessible> parentAccessible, lastChildAcc;
+    aDOMNode->GetParentNode(getter_AddRefs(parentNode));
+    if (parentNode) {
+      // this parent could be a Combobox or a ListBox. Each has a different
+      // was to get to the ListElement. 
+        nsCOMPtr<nsIDOMHTMLOptGroupElement> optGroupElement(do_QueryInterface(parentNode));
+        if (optGroupElement) {
+          parentNode->GetParentNode(getter_AddRefs(parentNode1));
+          parentNode = parentNode1;
+        }
+        accService->GetAccessibleFor(parentNode, getter_AddRefs(parentAccessible));
+        PRUint32 role;
+        do {
+          parentAccessible->GetAccLastChild(getter_AddRefs(lastChildAcc));
+          if (lastChildAcc)
+            lastChildAcc->GetAccRole(&role);
+          parentAccessible = lastChildAcc;
+        } while (role != nsIAccessible::ROLE_LIST && lastChildAcc);
+    }
+    mParent = parentAccessible;
+  }
 }
 
 /** We are a ListItem */
@@ -130,210 +159,42 @@ NS_IMETHODIMP nsSelectOptionAccessible::GetAccParent(nsIAccessible **_retval)
 /**
   * Get our Name from our Content's subtree
   */
-NS_IMETHODIMP nsSelectOptionAccessible::GetAccName(nsAWritableString& _retval)
+NS_IMETHODIMP nsSelectOptionAccessible::GetAccName(nsAString& _retval)
 {
-  nsCOMPtr<nsIContent> content (do_QueryInterface(mDOMNode));
-  if (!content) {
-    return NS_ERROR_FAILURE;
+  // CASE #1 -- great majority of the cases
+  // find the label attribute - this is what the W3C says we should use
+  nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(mDOMNode));
+  NS_ASSERTION(domElement, "No domElement for accessible DOM node!");
+  nsresult rv = domElement->GetAttribute(NS_LITERAL_STRING("label"), _retval) ;
+
+  if (NS_SUCCEEDED(rv) && !_retval.IsEmpty() ) {
+    return NS_OK;
   }
+  
+  // CASE #2 -- no label parameter, get the first child, 
+  // use it if it is a text node
+  nsCOMPtr<nsIDOMNode> child;
+  mDOMNode->GetFirstChild(getter_AddRefs(child));
 
-  nsAutoString option;
-  nsresult rv = AppendFlatStringFromSubtree(content, &option);
-  if (NS_SUCCEEDED(rv)) {
-    // Temp var needed until CompressWhitespace built for nsAWritableString
-    option.CompressWhitespace();
-    _retval.Assign(option);
+  if (child) {
+    nsCOMPtr<nsIDOMText> text (do_QueryInterface(child));
+    if (text) {
+      nsCOMPtr<nsIContent> content (do_QueryInterface(child));
+      if (!content) {
+        return NS_ERROR_FAILURE;
+      }
+      nsAutoString txtValue;
+      rv = AppendFlatStringFromContentNode(content, &txtValue);
+      if (NS_SUCCEEDED(rv)) {
+        // Temp var (txtValue) needed until CompressWhitespace built for nsAString
+        txtValue.CompressWhitespace();
+        _retval.Assign(txtValue);
+        return NS_OK;
+      }
+    }
   }
-  return rv;
-}
-
-/** ------------------------------------------------------ */
-/**  Secondly, the Listbox widget                          */
-/** ------------------------------------------------------ */
-
-/** ----- nsListboxAccessible ----- */
-
-/** Constructor */
-nsListboxAccessible::nsListboxAccessible(nsIDOMNode* aDOMNode, 
-                                                   nsIWeakReference* aShell):
-nsAccessible(aDOMNode, aShell)
-{
-}
-
-/** We are a window, as far as MSAA is concerned */
-NS_IMETHODIMP nsListboxAccessible::GetAccRole(PRUint32 *_retval)
-{
-  *_retval = ROLE_WINDOW;
-  return NS_OK;
-}
-
-/**
-  * We always have 1 child: a subclass of nsSelectListAccessible.
-  */
-NS_IMETHODIMP nsListboxAccessible::GetAccChildCount(PRInt32 *_retval)
-{
-  *_retval = 1;
-  return NS_OK;
-}
-
-/**
-  * As a nsHTMLListboxAccessible we can have the following states:
-  *     STATE_FOCUSED
-  *     STATE_READONLY
-  *     STATE_FOCUSABLE
-  */
-NS_IMETHODIMP nsListboxAccessible::GetAccState(PRUint32 *_retval)
-{
-  // Get focus status from base class
-  nsAccessible::GetAccState(_retval);
-
-  *_retval |= STATE_READONLY | STATE_FOCUSABLE;
-
-  return NS_OK;
-}
-
-/** ------------------------------------------------------ */
-/**  Finally, the Combobox widgets                         */
-/** ------------------------------------------------------ */
-
-/** ----- nsComboboxAccessible ----- */
-
-/**
-  * Constructor -- set initial state - closed, register ourself
-  */
-nsComboboxAccessible::nsComboboxAccessible(nsIDOMNode* aDOMNode, 
-                                                   nsIWeakReference* aShell):
-nsAccessible(aDOMNode, aShell)
-{
-  mRegistered = PR_FALSE;
-  mOpen = PR_FALSE;
-  SetupMenuListener();
-}
-
-/**
-  * Destructor -- If we are registered, remove ourselves as a listener.
-  */
-nsComboboxAccessible::~nsComboboxAccessible()
-{
-  if (mRegistered) {
-    nsCOMPtr<nsIDOMEventReceiver> eventReceiver(do_QueryInterface(mDOMNode));
-    if (eventReceiver) 
-      eventReceiver->RemoveEventListener(NS_LITERAL_STRING("popupshowing"), this, PR_TRUE);   
-  }
-}
-
-/** 
-  * Inherit the ISupports impl from nsAccessible, 
-  *  handle nsIDOMXULListener and nsIAccessibleSelectable ourself 
-  */
-NS_IMPL_ISUPPORTS_INHERITED2(nsComboboxAccessible, nsAccessible, nsIDOMXULListener, nsIAccessibleSelectable)
-
-/**
-  * If we aren't already registered, register ourselves as a
-  *     listener to "popupshowing" events on our DOM node. Set our
-  *     state to registered, but don't notify MSAA as they 
-  *     don't need to know about this state.
-  */
-void
-nsComboboxAccessible::SetupMenuListener()
-{
-  // if not already registered as a popup listener, register ourself
-  if (!mRegistered) {
-    nsCOMPtr<nsIDOMEventReceiver> eventReceiver(do_QueryInterface(mDOMNode));
-    if (eventReceiver && NS_SUCCEEDED(eventReceiver->AddEventListener(NS_LITERAL_STRING("popupshowing"), this, PR_TRUE)))
-      mRegistered = PR_TRUE;
-  }
-}
-
-/** We are a combobox */
-NS_IMETHODIMP nsComboboxAccessible::GetAccRole(PRUint32 *_retval)
-{
-  *_retval = ROLE_COMBOBOX;
-  return NS_OK;
-}
-
-/**
-  * We always have 3 children: TextField, Button, Window. In that order
-  */
-NS_IMETHODIMP nsComboboxAccessible::GetAccChildCount(PRInt32 *_retval)
-{
-  *_retval = 3;
-  return NS_OK;
-}
-
-/**
-  * nsIAccessibleSelectable method. No-op because our selection is returned through
-  *     GetValue(). This _may_ change just to provide additional info for the vendors
-  *     and another option for them to get at stuff. Despite the fact that we are
-  *     single selection only.
-  */
-NS_IMETHODIMP nsComboboxAccessible::GetSelectedChildren(nsISupportsArray **_retval)
-{
-  *_retval = nsnull;
-  return NS_OK;
-}
-
-/**
-  * As a nsComboboxAccessible we can have the following states:
-  *     STATE_FOCUSED
-  *     STATE_READONLY
-  *     STATE_FOCUSABLE
-  *     STATE_HASPOPUP
-  *     STATE_EXPANDED
-  *     STATE_COLLAPSED
-  */
-NS_IMETHODIMP nsComboboxAccessible::GetAccState(PRUint32 *_retval)
-{
-  // Get focus status from base class
-  nsAccessible::GetAccState(_retval);
-
-  if (mOpen)
-    *_retval |= STATE_EXPANDED;
-  else
-    *_retval |= STATE_COLLAPSED;
-
-  *_retval |= STATE_HASPOPUP | STATE_READONLY | STATE_FOCUSABLE;
-
-  return NS_OK;
-}
-
-/**
-  * Set our state to open and (TBD) fire an event to MSAA saying our state
-  *     has changed.
-  */
-NS_IMETHODIMP nsComboboxAccessible::PopupShowing(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_TRUE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
-}
-
-/**
-  * Set our state to not open and (TDB) fire an event to MSAA saying
-  *     our state has changed.
-  */
-NS_IMETHODIMP nsComboboxAccessible::PopupHiding(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_FALSE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
-}
-
-/**
-  * Set our state to not open and (TDB) fire an event to MSAA saying
-  *     our state has changed.
-  */
-NS_IMETHODIMP nsComboboxAccessible::Close(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_FALSE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
+  
+  return NS_ERROR_FAILURE;
 }
 
 /** ----- nsComboboxTextFieldAccessible ----- */
@@ -352,7 +213,7 @@ nsLeafAccessible(aDOMNode, aShell)
   *     and then return that text.
   *     Walks the Frame tree and checks for proper frames.
   */
-NS_IMETHODIMP nsComboboxTextFieldAccessible::GetAccValue(nsAWritableString& _retval)
+NS_IMETHODIMP nsComboboxTextFieldAccessible::GetAccValue(nsAString& _retval)
 {
   nsIFrame* frame = nsAccessible::GetBoundsFrame();
   nsCOMPtr<nsIPresContext> context;
@@ -480,14 +341,14 @@ void nsComboboxButtonAccessible::GetBounds(nsRect& aBounds, nsIFrame** aBounding
   if (!context)
     return;
 
-  frame->FirstChild(context, nsnull, &frame);
+  *aBoundingFrame = frame;  // bounding frame is the ComboboxControlFrame
+  frame->FirstChild(context, nsnull, &frame); // first frame is for the textfield
 #ifdef DEBUG
   if (! nsAccessible::IsCorrectFrameType(frame, nsLayoutAtoms::blockFrame))
     return;
 #endif
 
-  frame->GetNextSibling(aBoundingFrame);
-  frame->GetNextSibling(&frame);
+  frame->GetNextSibling(&frame);  // sibling frame is for the button
 #ifdef DEBUG
   if (! nsAccessible::IsCorrectFrameType(frame, nsLayoutAtoms::gfxButtonControlFrame))
     return;
@@ -514,7 +375,7 @@ NS_IMETHODIMP nsComboboxButtonAccessible::GetAccParent(nsIAccessible **_retval)
 /** 
   * Gets the name from GetAccActionName()
   */
-NS_IMETHODIMP nsComboboxButtonAccessible::GetAccName(nsAWritableString& _retval)
+NS_IMETHODIMP nsComboboxButtonAccessible::GetAccName(nsAString& _retval)
 {
   return GetAccActionName(eAction_Click, _retval);
 }
@@ -525,7 +386,7 @@ NS_IMETHODIMP nsComboboxButtonAccessible::GetAccName(nsAWritableString& _retval)
   *     if we are open -> closed is our name.
   * Uses the frame to get the state, updated on every click
   */
-NS_IMETHODIMP nsComboboxButtonAccessible::GetAccActionName(PRUint8 index, nsAWritableString& _retval)
+NS_IMETHODIMP nsComboboxButtonAccessible::GetAccActionName(PRUint8 index, nsAString& _retval)
 {
   PRBool isOpen = PR_FALSE;
   nsIFrame *boundsFrame = GetBoundsFrame();
