@@ -40,6 +40,7 @@ my %map = (
   'commandhandler',     'embedding/components/commandhandler',
   'composer',           'editor/composer',
   'content',            'content',
+  'content_xsl',        'content/xsl/document',
   'cookie',             'extensions/cookie',
   'ctl',                'extensions/ctl',
   'dbm',                'dbm',
@@ -49,7 +50,7 @@ my %map = (
   'editor',             'editor',
   'embed_base',         'embedding/base',
   'embed_lite',         'embedding/lite',
-  'embedcomponents',    'embedding/components/appstartup embedding/components/windowwatcher embedding/components/printingui embedding/components/build',
+  'embedcomponents',    'embedding/components/appstartup',
   'expat',              'expat',
   'exthandler',         'uriloader/exthandler',
   'filepicker',         'xpfe/components/filepicker',
@@ -106,9 +107,8 @@ my %map = (
   'mozcomps',           'xpfe/components',
   'mozldap',            'directory/xpcom/base',
   'mozMapi32',          'mailnews/mapi',
-  'mozpango',           'intl/ctl/src/pangoLite',
-  'mozpango-thaix',     'intl/ctl/src/thaiShaper',
-  'mozpango-dvngx',     'intl/ctl/src/hindiShaper',
+  'mozpango',           'extensions/ctl/src/pangoLite',
+  'mozpango-thaix',     'extensions/ctl/src/thaiShaper',
   'mozucth',            'xpfe/components/ucth',
   'mozxfer',            'xpfe/components/xfer',
   'mpfilelocprovider',  'modules/mpfilelocprovider',
@@ -123,7 +123,7 @@ my %map = (
   'msgnews',            'mailnews/news',
   'msgsmime',           'mailnews/extensions/smime',
   'necko',              'netwerk/base netwerk/protocol/about netwerk/protocol/data netwerk/protocol/file netwerk/protocol/http netwerk/protocol/jar netwerk/protocol/keyword netwerk/protocol/res netwerk/dns netwerk/socket netwerk/streamconv netwerk/cookie netwerk/build',
-  'necko2',             'netwerk/protocol/ftp netwerk/protocol/gopher netwerk/protocol/viewsource netwerk/build2',
+  'necko2',             'netwerk/protocol/datetime netwerk/protocol/finger netwerk/protocol/ftp netwerk/protocol/gopher netwerk/protocol/viewsource netwerk/build2',
   'nkcache',            'netwerk/cache',
   'npevents',           'modules/plugin/samples/testevents',
   'npsimple',           'modules/plugin/samples/simple',
@@ -175,7 +175,7 @@ my %map = (
   'view',               'view',
   'wallet',             'extensions/wallet',
   'walletviewers',      'extensions/wallet/build extensions/wallet/cookieviewer extensions/wallet/editor extensions/wallet/signonviewer extensions/wallet/walletpreview',
-  'webbrwsr',           'embedding/browser/webBrowser embedding/browser/build',
+  'webbrwsr',           'embedding/browser/webBrowser',
   'webbrowserpersist',  'embedding/components/webbrowserpersist',
   'webshell',           'webshell',
   'webshell_tests',     'webshell/tests',
@@ -187,7 +187,6 @@ my %map = (
   'xmlextras',          'extensions/xmlextras',
   'xmlterm',            'extensions/xmlterm',
   'xpcom',              'xpcom',
-  'xpcomsample',        'xpcom/sample',
   'xpconnect',          'js/src/xpconnect',
   'xpcshell',           'js/src/xpconnect/shell',
   'xpctools',           'js/src/xpconnect/tools',
