@@ -51,7 +51,6 @@
 #include "nsIDOMDocumentTraversal.h"
 #include "nsIDocumentObserver.h"
 #include "nsIDOMEventReceiver.h"
-#include "nsIDiskDocument.h"
 #include "nsIDOMStyleSheetList.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIDOMEventTarget.h"
@@ -69,13 +68,21 @@
 #include "nsIURI.h"
 #include "nsScriptLoader.h"
 #include "nsICSSLoader.h"
+#include "nsIDOMXPathEvaluator.h"
+#include "nsIRadioGroupContainer.h"
+
+#include "pldhash.h"
 
 class nsIEventListenerManager;
 class nsDOMStyleSheetList;
 class nsIOutputStream;
 class nsDocument;
 class nsIDTD;
+class nsXPathDocumentTearoff;
+class nsIRadioVisitor;
+class nsIFormControl;
 
+struct nsRadioGroupStruct;
 
 #if 0
 class nsPostData : public nsIPostData {
@@ -100,7 +107,7 @@ protected:
 class nsDocHeaderData
 {
 public:
-  nsDocHeaderData(nsIAtom* aField, const nsAReadableString& aData)
+  nsDocHeaderData(nsIAtom* aField, const nsAString& aData)
   {
     mField = aField;
     NS_IF_ADDREF(mField);
@@ -163,7 +170,8 @@ public:
                             nsISupports* aSubContent) { return NS_OK; }
   NS_IMETHOD ContentStatesChanged(nsIDocument* aDocument,
                                   nsIContent* aContent1,
-                                  nsIContent* aContent2) { return NS_OK; }
+                                  nsIContent* aContent2,
+                                  PRInt32 aStateMask) { return NS_OK; }
   NS_IMETHOD AttributeChanged(nsIDocument *aDocument,
                               nsIContent*  aContent,
                               PRInt32      aNameSpaceID,
@@ -212,6 +220,29 @@ protected:
   void*         mScriptObject;
 };
 
+
+// Helper structs for the content->subdoc map
+
+class SubDocMapEntry : public PLDHashEntryHdr
+{
+public:
+  // Both of these are strong references
+  nsIContent *mKey; // must be first, to look like PLDHashEntryStub
+  nsIDocument *mSubDocument;
+};
+
+struct FindContentData
+{
+  FindContentData(nsIDocument *aSubDoc)
+    : mSubDocument(aSubDoc), mResult(nsnull)
+  {
+  }
+
+  nsISupports *mSubDocument;
+  nsIContent *mResult;
+};
+
+
 // Base class for our document implementations.
 //
 // Note that this class *implements* nsIDOMXMLDocument, but it's not
@@ -231,10 +262,10 @@ class nsDocument : public nsIDocument,
                    public nsIDOMDocumentTraversal,
                    public nsIDOMDocumentXBL,
                    public nsIDOM3Node,
-                   public nsIDiskDocument,
                    public nsSupportsWeakReference,
                    public nsIDOMEventReceiver,
-                   public nsIScriptObjectPrincipal
+                   public nsIScriptObjectPrincipal,
+                   public nsIRadioGroupContainer
 {
 public:
   NS_DECL_ISUPPORTS
@@ -278,7 +309,7 @@ public:
   /**
    * Return the content language of this document.
    */
-  NS_IMETHOD GetContentLanguage(nsAWritableString& aContentLanguage) const;
+  NS_IMETHOD GetContentLanguage(nsAString& aContentLanguage) const;
 
   /**
    * Return the LoadGroup for the document. May return null.
@@ -286,7 +317,7 @@ public:
   NS_IMETHOD GetDocumentLoadGroup(nsILoadGroup **aGroup) const;
 
   /**
-   * Return the base URL for realtive URLs in the document. May return null (or the document URL).
+   * Return the base URL for relative URLs in the document. May return null (or the document URL).
    */
   NS_IMETHOD GetBaseURL(nsIURI*& aURL) const;
   NS_IMETHOD SetBaseURL(nsIURI* aURL);
@@ -294,15 +325,15 @@ public:
   /**
    * Get/Set the base target of a link in a document.
    */
-  NS_IMETHOD GetBaseTarget(nsAWritableString &aBaseTarget);
-  NS_IMETHOD SetBaseTarget(const nsAReadableString &aBaseTarget);
+  NS_IMETHOD GetBaseTarget(nsAString &aBaseTarget);
+  NS_IMETHOD SetBaseTarget(const nsAString &aBaseTarget);
 
   /**
    * Return a standard name for the document's character set. This will
    * trigger a startDocumentLoad if necessary to answer the question.
    */
-  NS_IMETHOD GetDocumentCharacterSet(nsAWritableString& oCharsetID);
-  NS_IMETHOD SetDocumentCharacterSet(const nsAReadableString& aCharSetID);
+  NS_IMETHOD GetDocumentCharacterSet(nsAString& oCharsetID);
+  NS_IMETHOD SetDocumentCharacterSet(const nsAString& aCharSetID);
 
   NS_IMETHOD GetDocumentCharacterSetSource(PRInt32* aCharsetSource);
   NS_IMETHOD SetDocumentCharacterSetSource(PRInt32 aCharsetSource);
@@ -342,9 +373,9 @@ public:
    * HTML META tags).
    */
   NS_IMETHOD GetHeaderData(nsIAtom* aHeaderField,
-                           nsAWritableString& aData) const;
+                           nsAString& aData) const;
   NS_IMETHOD SetHeaderData(nsIAtom* aheaderField,
-                           const nsAReadableString& aData);
+                           const nsAString& aData);
 
   /**
    * Create a new presentation shell that will use aContext for
@@ -368,9 +399,11 @@ public:
    */
   NS_IMETHOD GetParentDocument(nsIDocument** aParent);
   NS_IMETHOD SetParentDocument(nsIDocument* aParent);
-  NS_IMETHOD AddSubDocument(nsIDocument* aSubDoc);
-  NS_IMETHOD GetNumberOfSubDocuments(PRInt32* aCount);
-  NS_IMETHOD GetSubDocumentAt(PRInt32 aIndex, nsIDocument** aSubDoc);
+
+  NS_IMETHOD SetSubDocumentFor(nsIContent *aContent, nsIDocument* aSubDoc);
+  NS_IMETHOD GetSubDocumentFor(nsIContent *aContent, nsIDocument** aSubDoc);
+  NS_IMETHOD FindContentForSubDocument(nsIDocument *aDocument,
+                                       nsIContent **aContent);
 
   /**
    * Return the root content object for this document.
@@ -393,7 +426,7 @@ public:
   NS_IMETHOD GetNumberOfStyleSheets(PRInt32* aCount);
   NS_IMETHOD GetStyleSheetAt(PRInt32 aIndex, nsIStyleSheet** aSheet);
   NS_IMETHOD GetIndexOfStyleSheet(nsIStyleSheet* aSheet, PRInt32* aIndex);
-  virtual void AddStyleSheet(nsIStyleSheet* aSheet);
+  virtual void AddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags);
   virtual void RemoveStyleSheet(nsIStyleSheet* aSheet);
   
   NS_IMETHOD UpdateStyleSheets(nsISupportsArray* aOldSheets,
@@ -454,7 +487,8 @@ public:
   NS_IMETHOD ContentChanged(nsIContent* aContent,
                             nsISupports* aSubContent);
   NS_IMETHOD ContentStatesChanged(nsIContent* aContent1,
-                                  nsIContent* aContent2);
+                                  nsIContent* aContent2,
+                                  PRInt32 aStateMask);
 
   NS_IMETHOD AttributeWillChange(nsIContent* aChild,
                                  PRInt32 aNameSpaceID,
@@ -492,6 +526,24 @@ public:
   NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager*& aNodeInfoManager);
   NS_IMETHOD AddReference(void *aKey, nsISupports *aReference);
   NS_IMETHOD RemoveReference(void *aKey, nsISupports **aOldReference);
+  NS_IMETHOD SetContainer(nsISupports *aContainer);
+  NS_IMETHOD GetContainer(nsISupports **aContainer);
+
+  // nsIRadioGroupContainer
+  NS_IMETHOD WalkRadioGroup(const nsAString& aName,
+                            nsIRadioVisitor* aVisitor);
+  NS_IMETHOD SetCurrentRadioButton(const nsAString& aName,
+                                   nsIDOMHTMLInputElement* aRadio);
+  NS_IMETHOD GetCurrentRadioButton(const nsAString& aName,
+                                   nsIDOMHTMLInputElement** aRadio);
+  NS_IMETHOD AddToRadioGroup(const nsAString& aName,
+                             nsIFormControl* aRadio);
+  NS_IMETHOD RemoveFromRadioGroup(const nsAString& aName,
+                                  nsIFormControl* aRadio);
+                                        
+  // for radio group
+  nsresult GetRadioGroup(const nsAString& aName,
+                         nsRadioGroupStruct **aRadioGroup);
 
   // nsIDOMNode
   NS_DECL_NSIDOMNODE
@@ -534,14 +586,11 @@ public:
   NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
   NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
 
-  // nsIDiskDocument inteface
-  NS_DECL_NSIDISKDOCUMENT
-
   // nsIDOMEventTarget interface
-  NS_IMETHOD AddEventListener(const nsAReadableString& aType,
+  NS_IMETHOD AddEventListener(const nsAString& aType,
                               nsIDOMEventListener* aListener, 
                               PRBool aUseCapture);
-  NS_IMETHOD RemoveEventListener(const nsAReadableString& aType,
+  NS_IMETHOD RemoveEventListener(const nsAString& aType,
                                  nsIDOMEventListener* aListener, 
                                  PRBool aUseCapture);
   NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool *_retval);
@@ -558,16 +607,17 @@ public:
 
   NS_IMETHOD SetDocumentURL(nsIURI* aURI);
 
-protected:
-  NS_IMETHOD GetDTD(nsIDTD** aDTD) const;
+  virtual nsresult Init();
 
-  virtual void InternalAddStyleSheet(nsIStyleSheet* aSheet);  // subclass hooks for sheet ordering
+protected:
+  // subclass hooks for sheet ordering
+  virtual void InternalAddStyleSheet(nsIStyleSheet* aSheet,
+                                     PRUint32 aFlags);
   virtual void InternalInsertStyleSheetAt(nsIStyleSheet* aSheet,
                                           PRInt32 aIndex);
 
   nsDocument();
   virtual ~nsDocument(); 
-  nsresult Init();
 
   nsIArena* mArena;
   nsString mDocumentTitle;
@@ -575,13 +625,16 @@ protected:
   nsCOMPtr<nsIURI> mDocumentBaseURL;
   nsIPrincipal* mPrincipal;
   nsWeakPtr mDocumentLoadGroup;
-  
+  nsWeakPtr mDocumentContainer;
+
   nsString mCharacterSet;
   PRInt32 mCharacterSetSource;
   
   nsVoidArray mCharSetObservers;
   nsIDocument* mParentDocument;
-  nsVoidArray mSubDocuments;
+
+  PLDHashTable *mSubDocuments;
+
   nsVoidArray mPresShells;
   nsCOMPtr<nsISupportsArray> mChildren; // contains owning references
   nsIContent* mRootContent; // a weak reference to the only element in
@@ -602,6 +655,8 @@ protected:
   // objects in the document's content model
   PRInt32 mNextContentID;
 
+  nsHashtable mRadioGroups;
+
 #ifdef IBMBIDI
   PRBool        mBidiEnabled;
 #endif // IBMBIDI
@@ -632,6 +687,8 @@ private:
   // These are not implemented and not supported.
   nsDocument(const nsDocument& aOther);
   nsDocument& operator=(const nsDocument& aOther);
+
+  nsXPathDocumentTearoff* mXPathDocument;
 };
 
 
