@@ -59,7 +59,6 @@
 #include "nsIDOMDocumentType.h"
 #include "nsIDOMNodeList.h"
 #include "nsIDOMRange.h"
-#include "nsRange.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
 #include "nsHTMLAtoms.h"
@@ -67,20 +66,19 @@
 #include "nsIEnumerator.h"
 #include "nsISelectionPrivate.h"
 #include "nsIFrameSelection.h"
-#include "nsIContentIterator.h"
 #include "nsISupportsArray.h"
 #include "nsIParserService.h"
 #include "nsParserCIID.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptSecurityManager.h"
+#include "nsContentUtils.h"
 #include "nsUnicharUtils.h"
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID,
                      NS_ICHARSETCONVERTERMANAGER_CID);
 static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
 
-nsresult NS_NewContentSubtreeIterator(nsIContentIterator** aInstancePtrResult);
 nsresult NS_NewDomSelection(nsISelection **aDomSelection);
 
 enum nsRangeIterationDirection {
@@ -97,12 +95,12 @@ enum nsRangeIterationDirection {
 class nsDocumentEncoder : public nsIDocumentEncoder
 {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IDOCUMENT_ENCODER_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IDOCUMENT_ENCODER_IID)
 
   nsDocumentEncoder();
   virtual ~nsDocumentEncoder();
 
-  NS_IMETHOD Init(nsIDocument* aDocument, const nsAReadableString& aMimeType, PRUint32 aFlags);
+  NS_IMETHOD Init(nsIDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
 
   /* Interfaces for addref and release and queryinterface */
   NS_DECL_ISUPPORTS
@@ -111,33 +109,33 @@ public:
   NS_IMETHOD SetSelection(nsISelection* aSelection);
   NS_IMETHOD SetRange(nsIDOMRange* aRange);
   NS_IMETHOD SetWrapColumn(PRUint32 aWC);
-  NS_IMETHOD SetCharset(const nsAReadableString& aCharset);
-  NS_IMETHOD GetMimeType(nsAWritableString& aMimeType);
+  NS_IMETHOD SetCharset(const nsAString& aCharset);
+  NS_IMETHOD GetMimeType(nsAString& aMimeType);
   NS_IMETHOD EncodeToStream(nsIOutputStream* aStream);
-  NS_IMETHOD EncodeToString(nsAWritableString& aOutputString);
-  NS_IMETHOD EncodeToStringWithContext(nsAWritableString& aEncodedString, 
-                                       nsAWritableString& aContextString,
-                                       nsAWritableString& aInfoString);
+  NS_IMETHOD EncodeToString(nsAString& aOutputString);
+  NS_IMETHOD EncodeToStringWithContext(nsAString& aEncodedString, 
+                                       nsAString& aContextString,
+                                       nsAString& aInfoString);
   NS_IMETHOD SetNodeFixup(nsIDocumentEncoderNodeFixup *aFixup);
                                        
 protected:
   nsresult SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
-                              PRInt32 aEndOffset, nsAWritableString& aStr);
+                              PRInt32 aEndOffset, nsAString& aStr);
   nsresult SerializeToStringRecursive(nsIDOMNode* aNode,
-                                      nsAWritableString& aStr);
-  nsresult SerializeNodeEnd(nsIDOMNode* aNode, nsAWritableString& aStr);
+                                      nsAString& aStr);
+  nsresult SerializeNodeEnd(nsIDOMNode* aNode, nsAString& aStr);
   nsresult SerializeRangeToString(nsIDOMRange *aRange,
-                                  nsAWritableString& aOutputString);
+                                  nsAString& aOutputString);
   nsresult SerializeRangeNodes(nsIDOMRange* aRange, 
                                nsIDOMNode* aNode, 
-                               nsAWritableString& aString,
+                               nsAString& aString,
                                PRInt32 aDepth);
   nsresult SerializeRangeContextStart(const nsVoidArray& aAncestorArray,
-                                      nsAWritableString& aString);
+                                      nsAString& aString);
   nsresult SerializeRangeContextEnd(const nsVoidArray& aAncestorArray,
-                                    nsAWritableString& aString);
+                                    nsAString& aString);
 
-  nsresult FlushText(nsAWritableString& aString, PRBool aForce);
+  nsresult FlushText(nsAString& aString, PRBool aForce);
 
   static PRBool IsTag(nsIDOMNode* aNode, nsIAtom* aAtom);
   
@@ -167,6 +165,7 @@ protected:
   nsAutoVoidArray   mStartOffsets;
   nsAutoVoidArray   mEndNodes;
   nsAutoVoidArray   mEndOffsets;
+  PRPackedBool      mIsCopying;  // Set to PR_TRUE only while copying
 };
 
 #ifdef XP_MAC
@@ -229,7 +228,7 @@ IsScriptEnabled(nsIDocument *aDoc)
 
 NS_IMETHODIMP
 nsDocumentEncoder::Init(nsIDocument* aDocument,
-                        const nsAReadableString& aMimeType,
+                        const nsAString& aMimeType,
                         PRUint32 aFlags)
 {
   if (!aDocument)
@@ -240,6 +239,7 @@ nsDocumentEncoder::Init(nsIDocument* aDocument,
   mMimeType = aMimeType;
 
   mFlags = aFlags;
+  mIsCopying = PR_FALSE;
 
   return NS_OK;
 }
@@ -266,14 +266,14 @@ nsDocumentEncoder::SetRange(nsIDOMRange* aRange)
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::SetCharset(const nsAReadableString& aCharset)
+nsDocumentEncoder::SetCharset(const nsAString& aCharset)
 {
   mCharset = aCharset;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::GetMimeType(nsAWritableString& aMimeType)
+nsDocumentEncoder::GetMimeType(nsAString& aMimeType)
 {
   aMimeType = mMimeType;
   return NS_OK;
@@ -289,7 +289,7 @@ nsDocumentEncoder::IncludeInContext(nsIDOMNode *aNode)
 nsresult
 nsDocumentEncoder::SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
                                       PRInt32 aEndOffset,
-                                      nsAWritableString& aStr)
+                                      nsAString& aStr)
 {
   PRUint16 type;
 
@@ -308,7 +308,14 @@ nsDocumentEncoder::SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
     case nsIDOMNode::ELEMENT_NODE:
     {
       nsCOMPtr<nsIDOMElement> element = do_QueryInterface(node);
-      mSerializer->AppendElementStart(element, aStr);
+      // Because FixupNode() may have done a shallow copy of aNode
+      // we need to tell the serializer if the original had children.
+      // Some serializers (notably XML) need this information 
+      // in order to handle empty tags properly.
+      PRBool hasChildren;
+      mSerializer->AppendElementStart(element, 
+                                      NS_SUCCEEDED(aNode->HasChildNodes(&hasChildren)) && hasChildren,
+                                      aStr);
       break;
     }
     case nsIDOMNode::TEXT_NODE:
@@ -349,7 +356,7 @@ nsDocumentEncoder::SerializeNodeStart(nsIDOMNode* aNode, PRInt32 aStartOffset,
 
 nsresult
 nsDocumentEncoder::SerializeNodeEnd(nsIDOMNode* aNode,
-                                    nsAWritableString& aStr)
+                                    nsAString& aStr)
 {
   PRUint16 type;
 
@@ -368,7 +375,7 @@ nsDocumentEncoder::SerializeNodeEnd(nsIDOMNode* aNode,
 
 nsresult
 nsDocumentEncoder::SerializeToStringRecursive(nsIDOMNode* aNode,
-                                              nsAWritableString& aStr)
+                                              nsAString& aStr)
 {
   nsresult rv = SerializeNodeStart(aNode, 0, -1, aStr);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -423,7 +430,7 @@ nsDocumentEncoder::IsTag(nsIDOMNode* aNode, nsIAtom* aAtom)
 }
 
 static nsresult
-ConvertAndWrite(nsAReadableString& aString,
+ConvertAndWrite(const nsAString& aString,
                 nsIOutputStream* aStream,
                 nsIUnicodeEncoder* aEncoder)
 {
@@ -484,7 +491,7 @@ ConvertAndWrite(nsAReadableString& aString,
 }
 
 nsresult
-nsDocumentEncoder::FlushText(nsAWritableString& aString, PRBool aForce)
+nsDocumentEncoder::FlushText(nsAString& aString, PRBool aForce)
 {
   if (!mStream)
     return NS_OK;
@@ -650,7 +657,7 @@ static nsresult GetLengthOfDOMNode(nsIDOMNode *aNode, PRUint32 &aCount)
 nsresult
 nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange, 
                                        nsIDOMNode* aNode, 
-                                       nsAWritableString& aString,
+                                       nsAString& aString,
                                        PRInt32 aDepth)
 {
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
@@ -769,7 +776,7 @@ nsDocumentEncoder::SerializeRangeNodes(nsIDOMRange* aRange,
 
 nsresult
 nsDocumentEncoder::SerializeRangeContextStart(const nsVoidArray& aAncestorArray,
-                                              nsAWritableString& aString)
+                                              nsAString& aString)
 {
   PRInt32 i = aAncestorArray.Count();
   nsresult rv = NS_OK;
@@ -793,7 +800,7 @@ nsDocumentEncoder::SerializeRangeContextStart(const nsVoidArray& aAncestorArray,
 
 nsresult
 nsDocumentEncoder::SerializeRangeContextEnd(const nsVoidArray& aAncestorArray,
-                                            nsAWritableString& aString)
+                                            nsAString& aString)
 {
   PRInt32 i = 0;
   PRInt32 count = aAncestorArray.Count();
@@ -818,7 +825,7 @@ nsDocumentEncoder::SerializeRangeContextEnd(const nsVoidArray& aAncestorArray,
 
 nsresult
 nsDocumentEncoder::SerializeRangeToString(nsIDOMRange *aRange,
-                                          nsAWritableString& aOutputString)
+                                          nsAString& aOutputString)
 {
   if (!aRange)
     return NS_OK;
@@ -852,9 +859,12 @@ nsDocumentEncoder::SerializeRangeToString(nsIDOMRange *aRange,
   mEndNodes.Clear();
   mEndOffsets.Clear();
 
-  nsRange::FillArrayWithAncestors(&mCommonAncestors, mCommonParent);
-  nsRange::GetAncestorsAndOffsets(startParent, startOffset, &mStartNodes, &mStartOffsets);
-  nsRange::GetAncestorsAndOffsets(endParent, endOffset, &mEndNodes, &mEndOffsets);
+  nsContentUtils::GetAncestors(mCommonParent, &mCommonAncestors);
+  nsContentUtils::GetAncestorsAndOffsets(startParent, startOffset,
+                                         &mStartNodes, &mStartOffsets);
+  nsContentUtils::GetAncestorsAndOffsets(endParent, endOffset,
+                                         &mEndNodes, &mEndOffsets);
+
   nsCOMPtr<nsIContent> commonContent = do_QueryInterface(mCommonParent);
   mStartRootIndex = mStartNodes.IndexOf(commonContent);
   mEndRootIndex = mEndNodes.IndexOf(commonContent);
@@ -881,15 +891,13 @@ nsDocumentEncoder::SerializeRangeToString(nsIDOMRange *aRange,
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::EncodeToString(nsAWritableString& aOutputString)
+nsDocumentEncoder::EncodeToString(nsAString& aOutputString)
 {
   if (!mDocument)
     return NS_ERROR_NOT_INITIALIZED;
 
   aOutputString.Truncate();
 
-  // xxx Also make sure mString is a mime type "text/html" or "text/plain"
-
   nsCAutoString progId(NS_CONTENTSERIALIZER_CONTRACTID_PREFIX);
   progId.AppendWithConversion(mMimeType);
 
@@ -907,7 +915,7 @@ nsDocumentEncoder::EncodeToString(nsAWritableString& aOutputString)
     rv = mCharsetConverterManager->GetCharsetAtom(mCharset.get(), getter_AddRefs(charsetAtom));
     NS_ENSURE_SUCCESS(rv, rv);
   }
-  mSerializer->Init(mFlags, mWrapColumn, charsetAtom);
+  mSerializer->Init(mFlags, mWrapColumn, charsetAtom, mIsCopying);
 
   if (mSelection) {
     nsCOMPtr<nsIDOMRange> range;
@@ -966,8 +974,6 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  // xxx Also make sure mString is a mime type "text/html" or "text/plain"
-
   mStream = aStream;
 
   nsAutoString buf;
@@ -984,9 +990,9 @@ nsDocumentEncoder::EncodeToStream(nsIOutputStream* aStream)
 }
 
 NS_IMETHODIMP
-nsDocumentEncoder::EncodeToStringWithContext(nsAWritableString& aEncodedString, 
-                                             nsAWritableString& aContextString,
-                                             nsAWritableString& aInfoString)
+nsDocumentEncoder::EncodeToStringWithContext(nsAString& aEncodedString, 
+                                             nsAString& aContextString,
+                                             nsAString& aInfoString)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -1025,13 +1031,13 @@ public:
   nsHTMLCopyEncoder();
   virtual ~nsHTMLCopyEncoder();
 
-  NS_IMETHOD Init(nsIDocument* aDocument, const nsAReadableString& aMimeType, PRUint32 aFlags);
+  NS_IMETHOD Init(nsIDocument* aDocument, const nsAString& aMimeType, PRUint32 aFlags);
 
   // overridden methods from nsDocumentEncoder
   NS_IMETHOD SetSelection(nsISelection* aSelection);
-  NS_IMETHOD EncodeToStringWithContext(nsAWritableString& aEncodedString, 
-                                       nsAWritableString& aContextString,
-                                       nsAWritableString& aInfoString);
+  NS_IMETHOD EncodeToStringWithContext(nsAString& aEncodedString, 
+                                       nsAString& aContextString,
+                                       nsAString& aInfoString);
 
 protected:
 
@@ -1073,12 +1079,13 @@ nsHTMLCopyEncoder::~nsHTMLCopyEncoder()
 
 NS_IMETHODIMP
 nsHTMLCopyEncoder::Init(nsIDocument* aDocument,
-                        const nsAReadableString& aMimetype,
+                        const nsAString& aMimetype,
                         PRUint32 aFlags)
 {
   if (!aDocument)
     return NS_ERROR_INVALID_ARG;
 
+  mIsCopying = PR_TRUE;
   mDocument = aDocument;
 
   mMimeType = NS_LITERAL_STRING("text/html");
@@ -1145,7 +1152,7 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
       nsCOMPtr<nsIDOMElement> bodyElem = do_QueryInterface(selContent);
       nsAutoString wsVal;
       rv = bodyElem->GetAttribute(NS_LITERAL_STRING("style"), wsVal);
-      if (NS_SUCCEEDED(rv) && (kNotFound != wsVal.Find(NS_LITERAL_STRING("-moz-pre-wrap").get())))
+      if (NS_SUCCEEDED(rv) && (kNotFound != wsVal.Find(NS_LITERAL_STRING("-moz-pre-wrap"))))
       {
         mIsTextWidget = PR_TRUE;
         break;
@@ -1210,9 +1217,9 @@ nsHTMLCopyEncoder::SetSelection(nsISelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsHTMLCopyEncoder::EncodeToStringWithContext(nsAWritableString& aEncodedString, 
-                                             nsAWritableString& aContextString,
-                                             nsAWritableString& aInfoString)
+nsHTMLCopyEncoder::EncodeToStringWithContext(nsAString& aEncodedString, 
+                                             nsAString& aContextString,
+                                             nsAString& aInfoString)
 {
   nsresult rv = EncodeToString(aEncodedString);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1258,7 +1265,7 @@ nsHTMLCopyEncoder::EncodeToStringWithContext(nsAWritableString& aEncodedString,
   }
 
   // encode range info : the start and end depth of the selection, where the depth is 
-  // distance down in the parent heirarchy.  Later we will need to add leading/trainlig
+  // distance down in the parent hierarchy.  Later we will need to add leading/trailing
   // whitespace info to this.
   nsAutoString infoString;
   infoString.AppendInt(mStartDepth);
@@ -1287,6 +1294,8 @@ nsHTMLCopyEncoder::IncludeInContext(nsIDOMNode *aNode)
       tag.get() == nsHTMLAtoms::u        ||
       tag.get() == nsHTMLAtoms::tt       ||
       tag.get() == nsHTMLAtoms::s        ||
+      tag.get() == nsHTMLAtoms::big      ||
+      tag.get() == nsHTMLAtoms::small    ||
       tag.get() == nsHTMLAtoms::strike   ||
       tag.get() == nsHTMLAtoms::em       ||
       tag.get() == nsHTMLAtoms::strong   ||
@@ -1304,7 +1313,8 @@ nsHTMLCopyEncoder::IncludeInContext(nsIDOMNode *aNode)
       tag.get() == nsHTMLAtoms::h3       ||
       tag.get() == nsHTMLAtoms::h4       ||
       tag.get() == nsHTMLAtoms::h5       ||
-      tag.get() == nsHTMLAtoms::h6) {
+      tag.get() == nsHTMLAtoms::h6) 
+  {
     return PR_TRUE;
   }
 
@@ -1373,7 +1383,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
         nsAutoString text;
         nodeAsText->SubstringData(0, offset, text);
         text.CompressWhitespace();
-        if (text.Length())
+        if (!text.IsEmpty())
           return NS_OK;
         bResetPromotion = PR_TRUE;
       }
@@ -1458,7 +1468,7 @@ nsHTMLCopyEncoder::GetPromotedPoint(Endpoint aWhere, nsIDOMNode *aNode, PRInt32
         nsAutoString text;
         nodeAsText->SubstringData(offset, len-offset, text);
         text.CompressWhitespace();
-        if (text.Length())
+        if (!text.IsEmpty())
           return NS_OK;
         bResetPromotion = PR_TRUE;
       }
