@@ -68,40 +68,80 @@ class nsDOMAttributeMap;
 class nsIURI;
 class nsINodeInfo;
 
-// Class that holds the child list of a content element and also
-// implements the nsIDOMNodeList interface.
+/**
+ * Class that implements the nsIDOMNodeList interface (a list of children of
+ * the content), by holding a reference to the content and delegating GetLength
+ * and Item to its existing child list.
+ * @see nsIDOMNodeList
+ */
 class nsChildContentList : public nsGenericDOMNodeList 
 {
 public:
+  /**
+   * @param aContent the content whose children will make up the list
+   */
   nsChildContentList(nsIContent *aContent);
   virtual ~nsChildContentList();
 
   // nsIDOMNodeList interface
   NS_DECL_NSIDOMNODELIST
   
+  /** Drop the reference to the content */
   void DropReference();
 
 private:
+  /** The content whose children make up the list (weak reference) */
   nsIContent *mContent;
 };
 
-// There are a set of DOM- and scripting-specific instance variables
-// that may only be instantiated when a content object is accessed
-// through the DOM. Rather than burn actual slots in the content
-// objects for each of these instance variables, we put them off
-// in a side structure that's only allocated when the content is
-// accessed through the DOM.
-typedef struct {
+/**
+ * There are a set of DOM- and scripting-specific instance variables
+ * that may only be instantiated when a content object is accessed
+ * through the DOM. Rather than burn actual slots in the content
+ * objects for each of these instance variables, we put them off
+ * in a side structure that's only allocated when the content is
+ * accessed through the DOM.
+ */
+struct nsDOMSlots
+{
+  /**
+   * An object implementing nsIDOMNodeList for this content (childNodes)
+   * @see nsIDOMNodeList
+   * @see nsGenericHTMLLeafElement::GetChildNodes
+   */
   nsChildContentList *mChildNodes;
+  /**
+   * The .style attribute (an interface that forwards to the actual style rules)
+   * @see nsGenericHTMLElement::GetStyle
+   */
   nsDOMCSSDeclaration *mStyle;
+  /**
+   * An object implementing nsIDOMNamedNodeMap for this content (attributes)
+   * @see nsGenericElement::GetAttributes
+   */
   nsDOMAttributeMap* mAttributeMap;
+  /**
+   * A list of ranges whose endpoints are contained within this nsIContent.
+   * @see nsGenericElement::GetRangeList
+   * @see http://www.w3.org/TR/DOM-Level-2-Traversal-Range/
+   */
   nsVoidArray *mRangeList;
+  /**
+   * The event listener manager for this content (contains a list of listeners)
+   * @see nsGenericElement::GetListenerManager
+   */
   nsIEventListenerManager* mListenerManager;
-  nsIContent* mBindingParent; // The nearest enclosing content node with a binding
-                              // that created us. [Weak]
-} nsDOMSlots;
+  /**
+   * The nearest enclosing content node with a binding that created us. [weak]
+   * @see nsGenericElement::GetBindingParent
+   */
+  nsIContent* mBindingParent;
+};
 
 
+/**
+ * A tearoff class for nsGenericElement to implement the nsIDOM3Node functions
+ */
 class nsNode3Tearoff : public nsIDOM3Node
 {
   NS_DECL_ISUPPORTS
@@ -119,12 +159,17 @@ private:
 };
 
 
-// nsDOMEventRTTearoff is a tearoff class used by nsGenericElement and
-// nsGenericDOMDataNode classes for implemeting the interfaces
-// nsIDOMEventReceiver and nsIDOMEventTarget
-
 #define NS_EVENT_TEAROFF_CACHE_SIZE 4
 
+/**
+ * nsDOMEventRTTearoff is a tearoff class used by nsGenericElement and
+ * nsGenericDOMDataNode classes for implementing the interfaces
+ * nsIDOMEventReceiver and nsIDOMEventTarget
+ *
+ * Use the method nsDOMEventRTTearoff::Create() to create one of these babies.
+ * @see nsDOMEventRTTearoff::Create
+ */
+
 class nsDOMEventRTTearoff : public nsIDOMEventReceiver
 {
 private:
@@ -139,9 +184,11 @@ private:
   static nsDOMEventRTTearoff *mCachedEventTearoff[NS_EVENT_TEAROFF_CACHE_SIZE];
   static PRUint32 mCachedEventTearoffCount;
 
-  // This method gets called by Release() when it's time to delete the
-  // this object, in stead of always deleting the object we'll put the
-  // object in the cache if unless the cache is already full.
+  /**
+   * This method gets called by Release() when it's time to delete the
+   * this object, in stead of always deleting the object we'll put the
+   * object in the cache if unless the cache is already full.
+   */
   void LastRelease();
 
   nsresult GetEventReceiver(nsIDOMEventReceiver **aReceiver);
@@ -149,9 +196,15 @@ private:
 public:
   virtual ~nsDOMEventRTTearoff();
 
-  // Use this static method to create instances of this tearoff class.
+  /**
+   * Use this static method to create instances of nsDOMEventRTTearoff.
+   * @param aContent the content to create a tearoff for
+   */
   static nsDOMEventRTTearoff *Create(nsIContent *aContent);
 
+  /**
+   * Call before shutdown to clear the cache and free memory for this class.
+   */
   static void Shutdown();
 
   // nsISupports
@@ -169,12 +222,18 @@ public:
   NS_IMETHOD HandleEvent(nsIDOMEvent *aEvent);
 
 private:
-  // Strong reference back to the content object from where an
-  // instance of this class was 'torn off'
+  /**
+   * Strong reference back to the content object from where an instance of this
+   * class was 'torn off'
+   */
   nsCOMPtr<nsIContent> mContent;
 };
 
 
+/**
+ * A generic base class for DOM elements, implementing many nsIContent,
+ * nsIDOMNode and nsIDOMElement methods.
+ */
 class nsGenericElement : public nsIHTMLContent
 {
 public:
@@ -183,9 +242,19 @@ public:
 
   NS_DECL_ISUPPORTS
 
+  /**
+   * Initialize this element given a NodeInfo object
+   * @param aNodeInfo information about this type of node
+   */
   nsresult Init(nsINodeInfo *aNodeInfo);
 
-  // Free globals, to be called from module destructor
+  /**
+   * Called during QueryInterface to give the binding manager a chance to
+   * get an interface for this element.
+   */
+  nsresult PostQueryInterface(REFNSIID aIID, void** aInstancePtr);
+
+  /** Free globals, to be called from module destructor */
   static void Shutdown();
 
   // nsIContent interface methods
@@ -194,6 +263,8 @@ public:
                          PRBool aCompileEventHandlers);
   NS_IMETHOD GetParent(nsIContent*& aResult) const;
   NS_IMETHOD SetParent(nsIContent* aParent);
+  NS_IMETHOD_(PRBool) IsAnonymous() const;
+  NS_IMETHOD_(void) SetAnonymous(PRBool aAnonymous);
   NS_IMETHOD GetNameSpaceID(PRInt32& aNameSpaceID) const;
   NS_IMETHOD GetTag(nsIAtom*& aResult) const;
   NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const;
@@ -207,19 +278,19 @@ public:
   //                           PRBool aNotify);
   // NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify);
   // NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
-  // NS_IMETHOD NormalizeAttrString(const nsAReadableString& aStr,
+  // NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
   //                                nsINodeInfo*& aNodeInfo);
   // NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
-  //                    const nsAReadableString& aValue,
+  //                    const nsAString& aValue,
   //                    PRBool aNotify);
   // NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-  //                    const nsAReadableString& aValue,
+  //                    const nsAString& aValue,
   //                    PRBool aNotify);
   // NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
-  //                    nsAWritableString& aResult) const;
+  //                    nsAString& aResult) const;
   // NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, 
   //                    nsIAtom*& aPrefix,
-  //                    nsAWritableString& aResult) const;
+  //                    nsAString& aResult) const;
   // NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute, 
   //                      PRBool aNotify);
   // NS_IMETHOD GetAttrNameAt(PRInt32 aIndex,
@@ -247,6 +318,7 @@ public:
   NS_IMETHOD SetBindingParent(nsIContent* aParent);
   NS_IMETHOD_(PRBool) IsContentOfType(PRUint32 aFlags);
   NS_IMETHOD GetListenerManager(nsIEventListenerManager** aInstancePtrResult);
+  NS_IMETHOD DoneCreatingElement();
 
 
   // nsIStyledContent interface methods
@@ -274,79 +346,108 @@ public:
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD GetBaseURL(nsIURI*& aBaseURL) const;
-  NS_IMETHOD GetBaseTarget(nsAWritableString& aBaseTarget) const;
+  NS_IMETHOD GetBaseTarget(nsAString& aBaseTarget) const;
 
   // nsIDOMNode method implementation
-  NS_IMETHOD GetNodeName(nsAWritableString& aNodeName);
-  NS_IMETHOD GetLocalName(nsAWritableString& aLocalName);
-  NS_IMETHOD GetNodeValue(nsAWritableString& aNodeValue);
-  NS_IMETHOD SetNodeValue(const nsAReadableString& aNodeValue);
+  NS_IMETHOD GetNodeName(nsAString& aNodeName);
+  NS_IMETHOD GetLocalName(nsAString& aLocalName);
+  NS_IMETHOD GetNodeValue(nsAString& aNodeValue);
+  NS_IMETHOD SetNodeValue(const nsAString& aNodeValue);
   NS_IMETHOD GetNodeType(PRUint16* aNodeType);
   NS_IMETHOD GetParentNode(nsIDOMNode** aParentNode);
   NS_IMETHOD GetAttributes(nsIDOMNamedNodeMap** aAttributes);
   NS_IMETHOD GetPreviousSibling(nsIDOMNode** aPreviousSibling);
   NS_IMETHOD GetNextSibling(nsIDOMNode** aNextSibling);
   NS_IMETHOD GetOwnerDocument(nsIDOMDocument** aOwnerDocument);
-  NS_IMETHOD GetNamespaceURI(nsAWritableString& aNamespaceURI);
-  NS_IMETHOD GetPrefix(nsAWritableString& aPrefix);
-  NS_IMETHOD SetPrefix(const nsAReadableString& aPrefix);
+  NS_IMETHOD GetNamespaceURI(nsAString& aNamespaceURI);
+  NS_IMETHOD GetPrefix(nsAString& aPrefix);
+  NS_IMETHOD SetPrefix(const nsAString& aPrefix);
   NS_IMETHOD Normalize();
-  NS_IMETHOD IsSupported(const nsAReadableString& aFeature,
-                         const nsAReadableString& aVersion, PRBool* aReturn);
+  NS_IMETHOD IsSupported(const nsAString& aFeature,
+                         const nsAString& aVersion, PRBool* aReturn);
   NS_IMETHOD HasAttributes(PRBool* aHasAttributes);
 
   // nsIDOMElement method implementation
-  NS_IMETHOD GetTagName(nsAWritableString& aTagName);
-  NS_IMETHOD GetAttribute(const nsAReadableString& aName,
-                          nsAWritableString& aReturn);
-  NS_IMETHOD SetAttribute(const nsAReadableString& aName,
-                          const nsAReadableString& aValue);
-  NS_IMETHOD RemoveAttribute(const nsAReadableString& aName);
-  NS_IMETHOD GetAttributeNode(const nsAReadableString& aName,
+  NS_IMETHOD GetTagName(nsAString& aTagName);
+  NS_IMETHOD GetAttribute(const nsAString& aName,
+                          nsAString& aReturn);
+  NS_IMETHOD SetAttribute(const nsAString& aName,
+                          const nsAString& aValue);
+  NS_IMETHOD RemoveAttribute(const nsAString& aName);
+  NS_IMETHOD GetAttributeNode(const nsAString& aName,
                               nsIDOMAttr** aReturn);
   NS_IMETHOD SetAttributeNode(nsIDOMAttr* aNewAttr, nsIDOMAttr** aReturn);
   NS_IMETHOD RemoveAttributeNode(nsIDOMAttr* aOldAttr, nsIDOMAttr** aReturn);
-  NS_IMETHOD GetElementsByTagName(const nsAReadableString& aTagname,
+  NS_IMETHOD GetElementsByTagName(const nsAString& aTagname,
                                   nsIDOMNodeList** aReturn);
-  NS_IMETHOD GetAttributeNS(const nsAReadableString& aNamespaceURI,
-                            const nsAReadableString& aLocalName,
-                            nsAWritableString& aReturn);
-  NS_IMETHOD SetAttributeNS(const nsAReadableString& aNamespaceURI,
-                            const nsAReadableString& aQualifiedName,
-                            const nsAReadableString& aValue);
-  NS_IMETHOD RemoveAttributeNS(const nsAReadableString& aNamespaceURI,
-                               const nsAReadableString& aLocalName);
-  NS_IMETHOD GetAttributeNodeNS(const nsAReadableString& aNamespaceURI,
-                                const nsAReadableString& aLocalName,
+  NS_IMETHOD GetAttributeNS(const nsAString& aNamespaceURI,
+                            const nsAString& aLocalName,
+                            nsAString& aReturn);
+  NS_IMETHOD SetAttributeNS(const nsAString& aNamespaceURI,
+                            const nsAString& aQualifiedName,
+                            const nsAString& aValue);
+  NS_IMETHOD RemoveAttributeNS(const nsAString& aNamespaceURI,
+                               const nsAString& aLocalName);
+  NS_IMETHOD GetAttributeNodeNS(const nsAString& aNamespaceURI,
+                                const nsAString& aLocalName,
                                 nsIDOMAttr** aReturn);
   NS_IMETHOD SetAttributeNodeNS(nsIDOMAttr* aNewAttr, nsIDOMAttr** aReturn);
-  NS_IMETHOD GetElementsByTagNameNS(const nsAReadableString& aNamespaceURI,
-                                    const nsAReadableString& aLocalName,
+  NS_IMETHOD GetElementsByTagNameNS(const nsAString& aNamespaceURI,
+                                    const nsAString& aLocalName,
                                     nsIDOMNodeList** aReturn);
-  NS_IMETHOD HasAttribute(const nsAReadableString& aName, PRBool* aReturn);
-  NS_IMETHOD HasAttributeNS(const nsAReadableString& aNamespaceURI,
-                            const nsAReadableString& aLocalName,
+  NS_IMETHOD HasAttribute(const nsAString& aName, PRBool* aReturn);
+  NS_IMETHOD HasAttributeNS(const nsAString& aNamespaceURI,
+                            const nsAString& aLocalName,
                             PRBool* aReturn);
 
   // Generic DOMNode implementations
+  /**
+   * Generic implementation of InsertBefore to be called by subclasses
+   * @see nsIDOMNode::InsertBefore
+   */
   nsresult  doInsertBefore(nsIDOMNode* aNewChild, nsIDOMNode* aRefChild,
                            nsIDOMNode** aReturn);
+  /**
+   * Generic implementation of ReplaceChild to be called by subclasses
+   * @see nsIDOMNode::ReplaceChild
+   */
   nsresult  doReplaceChild(nsIDOMNode* aNewChild, nsIDOMNode* aOldChild,
                            nsIDOMNode** aReturn);
+  /**
+   * Generic implementation of RemoveChild to be called by subclasses
+   * @see nsIDOMNode::RemoveChild
+   */
   nsresult  doRemoveChild(nsIDOMNode* aOldChild, nsIDOMNode** aReturn);
 
   //----------------------------------------
 
+  // XXX Unused.  Remove.
   nsresult RenderFrame(nsIPresContext*);
 
+  /**
+   * Add a script event listener with the given attr name (like onclick)
+   * and with the value as JS
+   * @param aAttribute the event listener name
+   * @param aValue the JS to attach
+   */
   nsresult AddScriptEventListener(nsIAtom* aAttribute,
-                                  const nsAReadableString& aValue);
+                                  const nsAString& aValue);
 
+  /**
+   * Load a link, putting together the proper URL from the pieces given.
+   * @param aPresContext the pres context.
+   * @param aVerb how the link will be loaded (replace page, new window, etc.)
+   * @param aBaseURL the base URL to start with (content.baseURL, may be null)
+   * @param aURLSpec the URL of the link (may be relative)
+   * @param aTargetSpec the target (like target=, may be null)
+   * @param aClick whether this was a click or not (if false, it assumes you
+   *        just hovered over the link)
+   */
   nsresult TriggerLink(nsIPresContext* aPresContext,
                        nsLinkVerb aVerb,
                        nsIURI* aBaseURL,
@@ -354,16 +455,45 @@ public:
                        const nsString& aTargetSpec,
                        PRBool aClick);
 
+  /**
+   * Take two text nodes and append the second to the first.
+   * @param aFirst the node which will contain first + second [INOUT]
+   * @param aSecond the node which will be appended
+   */
   nsresult JoinTextNodes(nsIContent* aFirst,
                          nsIContent* aSecond);
 
+  /**
+   * Set .document in the immediate children of said content (but not in
+   * content itself).  SetDocument() in the children will recursively call
+   * this.
+   *
+   * @param aContent the content to get the children of
+   * @param aDocument the document to set
+   * @param aCompileEventHandlers whether to initialize the event handlers in
+   *        the document (used by nsXULElement)
+   */
   static void SetDocumentInChildrenOf(nsIContent* aContent, 
 				      nsIDocument* aDocument, PRBool aCompileEventHandlers);
 
-  static nsresult InternalIsSupported(const nsAReadableString& aFeature,
-                                      const nsAReadableString& aVersion,
+  /**
+   * Check whether a spec feature/version is supported.
+   * @param aFeature the feature ("Views", "Core", "HTML", "Range" ...)
+   * @param aVersion the version ("1.0", "2.0", ...)
+   * @param aReturn whether the feature is supported or not [OUT]
+   */
+  static nsresult InternalIsSupported(const nsAString& aFeature,
+                                      const nsAString& aVersion,
                                       PRBool* aReturn);
 
+  /**
+   * Quick helper to determine whether there are any mutation listeners
+   * of a given type that apply to this content (are at or above it).
+   * @param aContent the content to look for listeners for
+   * @param aType the type of listener (NS_EVENT_BITS_MUTATION_*)
+   * @return whether there are mutation listeners of the specified type for
+   *         this content or not
+   */
   static PRBool HasMutationListeners(nsIContent* aContent,
                                      PRUint32 aType);
 
@@ -372,34 +502,59 @@ protected:
   virtual PRUint32 BaseSizeOf(nsISizeOfHandler *aSizer) const;
 #endif
 
+  /**
+   * Get mDOMSlots, or create it if it doesn't exist
+   * @returen the DOMSlots for this content object
+   */
   nsDOMSlots *GetDOMSlots();
+  /**
+   * Clear mDOMSlots if there is nothing in it.
+   */
   void MaybeClearDOMSlots();
 
+  /** The document for this content */
   nsIDocument* mDocument;                   // WEAK
+  /** The parent content */
   nsIContent* mParent;                      // WEAK
   
+  /** Information about this type of node */
   nsINodeInfo* mNodeInfo;                   // OWNER
+  /** The DOMSlots for this content (see nsDOMSlots for info) */
   nsDOMSlots *mDOMSlots;                    // OWNER
+  /**
+   * A unique content ID (used for layout save/restore)
+   * @see FrameManager::GenerateStateKey
+   */
   PRUint32 mContentID;
 };
 
+/**
+ * A generic element that contains children
+ */
 class nsGenericContainerElement : public nsGenericElement {
 public:
   nsGenericContainerElement();
   virtual ~nsGenericContainerElement();
 
+  /**
+   * Copy attributes and children from another content object
+   * @param aSrcContent the object to copy from
+   * @param aDest the destination object
+   * @param aDeep whether to copy children
+   */
+  // XXX This can probably be static
   NS_IMETHOD CopyInnerTo(nsIContent* aSrcContent,
-                       nsGenericContainerElement* aDest,
-                       PRBool aDeep);
+                         nsGenericContainerElement* aDest,
+                         PRBool aDeep);
 
   // nsIDOMElement methods
-  NS_METHOD GetAttribute(const nsAReadableString& aName,
-                        nsAWritableString& aReturn) 
+  NS_METHOD GetAttribute(const nsAString& aName,
+                         nsAString& aReturn) 
   {
     return nsGenericElement::GetAttribute(aName, aReturn);
   }
-  NS_METHOD SetAttribute(const nsAReadableString& aName,
-                        const nsAReadableString& aValue)
+  NS_METHOD SetAttribute(const nsAString& aName,
+                         const nsAString& aValue)
   {
     return nsGenericElement::SetAttribute(aName, aValue);
   }
@@ -411,12 +566,12 @@ public:
   NS_IMETHOD GetLastChild(nsIDOMNode** aLastChild);
   
   NS_IMETHOD InsertBefore(nsIDOMNode* aNewChild, nsIDOMNode* aRefChild,
-                        nsIDOMNode** aReturn)
+                          nsIDOMNode** aReturn)
   {
     return nsGenericElement::doInsertBefore(aNewChild, aRefChild, aReturn);
   }
   NS_IMETHOD ReplaceChild(nsIDOMNode* aNewChild, nsIDOMNode* aOldChild,
-                        nsIDOMNode** aReturn)
+                          nsIDOMNode** aReturn)
   {
     return nsGenericElement::doReplaceChild(aNewChild, aOldChild, aReturn);
   }
@@ -430,18 +585,18 @@ public:
   }
 
   // Remainder of nsIContent
-  NS_IMETHOD NormalizeAttrString(const nsAReadableString& aStr,
+  NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
                                  nsINodeInfo*& aNodeInfo);
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                          const nsAReadableString& aValue,
-                          PRBool aNotify);
+                     const nsAString& aValue,
+                     PRBool aNotify);
   NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-                     const nsAReadableString& aValue,
+                     const nsAString& aValue,
                      PRBool aNotify);
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     nsAWritableString& aResult) const;
+                     nsAString& aResult) const;
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     nsIAtom*& aPrefix, nsAWritableString& aResult) const;
+                     nsIAtom*& aPrefix, nsAString& aResult) const;
   NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
   NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttribute,
                        PRBool aNotify);
@@ -475,7 +630,12 @@ protected:
   virtual PRUint32 BaseSizeOf(nsISizeOfHandler *aSizer) const;
 #endif
 
+  /**
+   * The attributes (stored as nsGenericAttribute*)
+   * @see nsGenericAttribute
+   * */
   nsVoidArray* mAttributes;
+  /** The list of children (stored as nsIContent*) */
   nsSmallVoidArray mChildren;
 };
 
@@ -488,36 +648,40 @@ protected:
   {0x9c, 0x60, 0x6c, 0xa2, 0x75, 0x35, 0x09, 0xeb} }
 
 // nsIDocumentFragment interface
+/**
+ * These methods are supposed to be used when *all* children of a
+ * document fragment are moved at once into a new parent w/o
+ * changing the relationship between the children. If the moving
+ * operation fails and some children were moved to a new parent and
+ * some weren't, ReconnectChildren() should be called to remove the
+ * children from their possible new parent and re-insert the
+ * children into the document fragment. Once the operation is
+ * complete and all children are successfully moved into their new
+ * parent DropChildReferences() should be called so that the
+ * document fragment will loose its references to the children.
+ */
 class nsIDocumentFragment : public nsIDOMDocumentFragment
 {
 public:
   NS_DEFINE_STATIC_IID_ACCESSOR(NS_IDOCUMENTFRAGMENT_IID)
 
-  // These methods are supposed to be used when *all* children of a
-  // document fragment are moved at once into a new parent w/o
-  // changing the relationship between the children. If the moving
-  // operation fails and some children were moved to a new parent and
-  // some weren't, ReconnectChildren() should be called to remove the
-  // children from their possible new parent and re-insert the
-  // children into the document fragment. Once the operation is
-  // complete and all children are successfully moved into their new
-  // parent DropChildReferences() should be called so that the
-  // document fragment will loose its references to the children.
-
+  /** Tell the children their parent is gone */
   NS_IMETHOD DisconnectChildren() = 0;
+  /** Put all children back in the fragment */
   NS_IMETHOD ReconnectChildren() = 0;
+  /** Drop references to children */
   NS_IMETHOD DropChildReferences() = 0;
 };
 
 
 #define NS_FORWARD_NSIDOMNODE_NO_CLONENODE(_to)                               \
-  NS_IMETHOD GetNodeName(nsAWritableString& aNodeName) {                      \
+  NS_IMETHOD GetNodeName(nsAString& aNodeName) {                              \
     return _to GetNodeName(aNodeName);                                        \
   }                                                                           \
-  NS_IMETHOD GetNodeValue(nsAWritableString& aNodeValue) {                    \
+  NS_IMETHOD GetNodeValue(nsAString& aNodeValue) {                            \
     return _to GetNodeValue(aNodeValue);                                      \
   }                                                                           \
-  NS_IMETHOD SetNodeValue(const nsAReadableString& aNodeValue) {              \
+  NS_IMETHOD SetNodeValue(const nsAString& aNodeValue) {                      \
     return _to SetNodeValue(aNodeValue);                                      \
   }                                                                           \
   NS_IMETHOD GetNodeType(PRUint16* aNodeType) {                               \
@@ -547,16 +711,16 @@ public:
   NS_IMETHOD GetOwnerDocument(nsIDOMDocument** aOwnerDocument) {              \
     return _to GetOwnerDocument(aOwnerDocument);                              \
   }                                                                           \
-  NS_IMETHOD GetNamespaceURI(nsAWritableString& aNamespaceURI) {              \
+  NS_IMETHOD GetNamespaceURI(nsAString& aNamespaceURI) {                      \
     return _to GetNamespaceURI(aNamespaceURI);                                \
   }                                                                           \
-  NS_IMETHOD GetPrefix(nsAWritableString& aPrefix) {                          \
+  NS_IMETHOD GetPrefix(nsAString& aPrefix) {                                  \
     return _to GetPrefix(aPrefix);                                            \
   }                                                                           \
-  NS_IMETHOD SetPrefix(const nsAReadableString& aPrefix) {                    \
+  NS_IMETHOD SetPrefix(const nsAString& aPrefix) {                            \
     return _to SetPrefix(aPrefix);                                            \
   }                                                                           \
-  NS_IMETHOD GetLocalName(nsAWritableString& aLocalName) {                    \
+  NS_IMETHOD GetLocalName(nsAString& aLocalName) {                            \
     return _to GetLocalName(aLocalName);                                      \
   }                                                                           \
   NS_IMETHOD InsertBefore(nsIDOMNode* aNewChild, nsIDOMNode* aRefChild,       \
@@ -579,8 +743,8 @@ public:
   NS_IMETHOD Normalize() {                                                    \
     return _to Normalize();                                                   \
   }                                                                           \
-  NS_IMETHOD IsSupported(const nsAReadableString& aFeature,                   \
-                         const nsAReadableString& aVersion, PRBool* aReturn) {\
+  NS_IMETHOD IsSupported(const nsAString& aFeature,                           \
+                         const nsAString& aVersion, PRBool* aReturn) {        \
     return _to IsSupported(aFeature, aVersion, aReturn);                      \
   }                                                                           \
   NS_IMETHOD HasAttributes(PRBool* aReturn) {                                 \
