@@ -42,6 +42,7 @@
 #include "nsString.h"
 #include "nsIAtom.h"
 #include "nsIDocument.h"
+#include "nsIPrincipal.h"
 
 nsNodeInfoManager* nsNodeInfoManager::gAnonymousNodeInfoManager = nsnull;
 PRUint32 nsNodeInfoManager::gNodeManagerCount = 0;
@@ -154,6 +155,9 @@ nsNodeInfoManager::Init(nsIDocument *aDocument,
 
   mDocument = aDocument;
   mNameSpaceManager = aNameSpaceManager;
+  if (aDocument) {
+    mPrincipal = nsnull;
+  }
 
   return NS_OK;
 }
@@ -162,6 +166,19 @@ nsNodeInfoManager::Init(nsIDocument *aDocument,
 NS_IMETHODIMP
 nsNodeInfoManager::DropDocumentReference()
 {
+  if (mDocument) {
+    // If the document has a uri we'll ask for it's principal. Otherwise we'll
+    // consider this document 'anonymous'. We don't want to call GetPrincipal
+    // on a document that doesn't have a URI since that'll give an assertion
+    // that we're creating a principal without having a uri.
+    // This happens in a few cases where a document is created and then
+    // immediately dropped without ever getting a URI.
+    nsCOMPtr<nsIURI> docUri;
+    mDocument->GetDocumentURL(getter_AddRefs(docUri));
+    if (docUri) {
+      mDocument->GetPrincipal(getter_AddRefs(mPrincipal));
+    }
+  }
   mDocument = nsnull;
 
   return NS_OK;
@@ -208,7 +225,7 @@ NS_IMETHODIMP
 nsNodeInfoManager::GetNodeInfo(const nsAString& aName, nsIAtom *aPrefix,
                                PRInt32 aNamespaceID, nsINodeInfo*& aNodeInfo)
 {
-  NS_ENSURE_ARG(aName.Length());
+  NS_ENSURE_ARG(!aName.IsEmpty());
 
   nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
@@ -222,14 +239,14 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aName,
                                const nsAString& aPrefix, PRInt32 aNamespaceID,
                                nsINodeInfo*& aNodeInfo)
 {
-  NS_ENSURE_ARG(aName.Length());
+  NS_ENSURE_ARG(!aName.IsEmpty());
 
   nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
 
   nsCOMPtr<nsIAtom> prefix;
 
-  if (aPrefix.Length()) {
+  if (!aPrefix.IsEmpty()) {
     prefix = dont_AddRef(NS_NewAtom(aPrefix));
     NS_ENSURE_TRUE(prefix, NS_ERROR_OUT_OF_MEMORY);
   }
@@ -244,14 +261,14 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aName,
                                const nsAString& aNamespaceURI,
                                nsINodeInfo*& aNodeInfo)
 {
-  NS_ENSURE_ARG(aName.Length());
+  NS_ENSURE_ARG(!aName.IsEmpty());
 
   nsCOMPtr<nsIAtom> name(dont_AddRef(NS_NewAtom(aName)));
   NS_ENSURE_TRUE(name, NS_ERROR_OUT_OF_MEMORY);
 
   nsCOMPtr<nsIAtom> prefix;
 
-  if (aPrefix.Length()) {
+  if (!aPrefix.IsEmpty()) {
     prefix = dont_AddRef(NS_NewAtom(aPrefix));
     NS_ENSURE_TRUE(prefix, NS_ERROR_OUT_OF_MEMORY);
   }
@@ -273,7 +290,7 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aQualifiedName,
                                const nsAString& aNamespaceURI,
                                nsINodeInfo*& aNodeInfo)
 {
-  NS_ENSURE_ARG(aQualifiedName.Length());
+  NS_ENSURE_ARG(!aQualifiedName.IsEmpty());
 
   nsAutoString name(aQualifiedName);
   nsAutoString prefix;
@@ -288,14 +305,14 @@ nsNodeInfoManager::GetNodeInfo(const nsAString& aQualifiedName,
 
   nsCOMPtr<nsIAtom> prefixAtom;
 
-  if (prefix.Length()) {
+  if (!prefix.IsEmpty()) {
     prefixAtom = dont_AddRef(NS_NewAtom(prefix));
     NS_ENSURE_TRUE(prefixAtom, NS_ERROR_OUT_OF_MEMORY);
   }
 
   PRInt32 nsid = kNameSpaceID_None;
 
-  if (aNamespaceURI.Length()) {
+  if (!aNamespaceURI.IsEmpty()) {
     NS_ENSURE_TRUE(mNameSpaceManager, NS_ERROR_NOT_INITIALIZED);
 
     nsresult rv = mNameSpaceManager->RegisterNameSpace(aNamespaceURI, nsid);
@@ -328,6 +345,36 @@ nsNodeInfoManager::GetDocument(nsIDocument*& aDocument)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsNodeInfoManager::GetDocumentPrincipal(nsIPrincipal** aPrincipal)
+{
+  NS_ENSURE_ARG_POINTER(aPrincipal);
+  NS_ASSERTION(!mDocument || !mPrincipal,
+               "how'd we end up with both a document and a principal?");
+
+  if (mDocument) {
+    // If the document has a uri we'll ask for it's principal. Otherwise we'll
+    // consider this document 'anonymous'
+    nsCOMPtr<nsIURI> docUri;
+    mDocument->GetDocumentURL(getter_AddRefs(docUri));
+    if (!docUri) {
+      *aPrincipal = nsnull;
+      return NS_OK;
+    }
+    return mDocument->GetPrincipal(aPrincipal);
+  }
+  *aPrincipal = mPrincipal;
+  NS_IF_ADDREF(*aPrincipal);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsNodeInfoManager::SetDocumentPrincipal(nsIPrincipal* aPrincipal)
+{
+  NS_ENSURE_FALSE(mDocument, NS_ERROR_UNEXPECTED);
+  mPrincipal = aPrincipal;
+  return NS_OK;
+}
 
 void
 nsNodeInfoManager::RemoveNodeInfo(nsNodeInfo *aNodeInfo)
