@@ -53,6 +53,8 @@
 #include "nsParserCIID.h"
 #include "nsContentUtils.h"
 #include "nsReadableUtils.h"
+#include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kLWBrkCID, NS_LWBRK_CID);
 static NS_DEFINE_CID(kParserServiceCID, NS_PARSERSERVICE_CID);
@@ -106,6 +108,7 @@ nsPlainTextSerializer::nsPlainTextSerializer()
   mStructs = PR_TRUE;       // will be read from prefs later
   mHeaderStrategy = 1 /*indent increasingly*/;   // ditto
   mQuotesPreformatted = PR_FALSE;                // ditto
+  mDontWrapAnyQuotes = PR_FALSE;                 // ditto
   mSpanLevel = 0;
   for (PRInt32 i = 0; i <= 6; i++) {
     mHeaderCounter[i] = 0;
@@ -148,7 +151,7 @@ NS_IMPL_ISUPPORTS4(nsPlainTextSerializer,
 
 NS_IMETHODIMP 
 nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
-                            nsIAtom* aCharSet)
+                            nsIAtom* aCharSet, PRBool aIsCopying)
 {
 #ifdef DEBUG
   // Check if the major control flags are set correctly.
@@ -202,6 +205,14 @@ nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
       // The quotesPreformatted pref is a temporary measure. See bug 69638.
       prefs->GetBoolPref("editor.quotesPreformatted", &tempBool);
       mQuotesPreformatted = tempBool;
+      // DontWrapAnyQuotes is set according to whether plaintext mail
+      // is wrapping to window width -- see bug 134439.
+      // We'll only want this if we're wrapping and formatted.
+      if (mFlags & nsIDocumentEncoder::OutputWrap || mWrapColumn > 0)
+      {
+        prefs->GetBoolPref("mail.compose.wrap_to_window_width", &tempBool);
+        mDontWrapAnyQuotes = tempBool;
+      }
     }
 
     // XXX We should let the caller pass this in.
@@ -212,15 +223,17 @@ nsPlainTextSerializer::Init(PRUint32 aFlags, PRUint32 aWrapColumn,
     else
       mFlags |= nsIDocumentEncoder::OutputNoFramesContent;
   }
+  mLineBreakDue = PR_FALSE;
+  mFloatingLines = -1;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPlainTextSerializer::Initialize(nsAWritableString* aOutString,
+nsPlainTextSerializer::Initialize(nsAString* aOutString,
                                   PRUint32 aFlags, PRUint32 aWrapCol)
 {
-  nsresult rv = Init(aFlags, aWrapCol, nsnull);
+  nsresult rv = Init(aFlags, aWrapCol, nsnull, PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // XXX This is wrong. It violates XPCOM string ownership rules.
@@ -235,11 +248,15 @@ NS_IMETHODIMP
 nsPlainTextSerializer::AppendText(nsIDOMText* aText, 
                                   PRInt32 aStartOffset,
                                   PRInt32 aEndOffset, 
-                                  nsAWritableString& aStr)
+                                  nsAString& aStr)
 {
   if (mIgnoreAboveIndex != (PRUint32)kNotFound) {
     return NS_OK;
   }
+    
+  NS_ASSERTION(aStartOffset >= 0, "Negative start offset for text fragment!");
+  if ( aStartOffset < 0 )
+    return NS_ERROR_INVALID_ARG;
 
   NS_ENSURE_ARG(aText);
 
@@ -254,7 +271,14 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
   content->GetText(&frag);
 
   if (frag) {
-    length = ((aEndOffset == -1) ? frag->GetLength() : aEndOffset) - aStartOffset;
+    PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
+    NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
+
+    length = endoffset - aStartOffset;
+    if (length <= 0) {
+      return NS_OK;
+    }
+
     if (frag->Is2b()) {
       textstr.Assign(frag->Get2b() + aStartOffset, length);
     }
@@ -265,8 +289,6 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
   mOutputString = &aStr;
 
-  nsAutoString linebuffer;
-
   // We have to split the string across newlines
   // to match parser behavior
   PRInt32 start = 0;
@@ -275,8 +297,7 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
     if(offset>start) {
       // Pass in the line
-      textstr.Mid(linebuffer, start, offset-start);
-      rv = DoAddLeaf(eHTMLTag_text, linebuffer);
+      rv = DoAddLeaf(eHTMLTag_text, Substring(textstr, start, offset-start));
       if (NS_FAILED(rv)) break;
     }
 
@@ -291,8 +312,7 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
   // Consume the last bit of the string if there's any left
   if (NS_SUCCEEDED(rv) & (start < length)) {
     if (start) {
-      textstr.Mid(linebuffer, start, offset-start);
-      rv = DoAddLeaf(eHTMLTag_text, linebuffer);
+      rv = DoAddLeaf(eHTMLTag_text, Substring(textstr, start, length-start));
     }
     else {
       rv = DoAddLeaf(eHTMLTag_text, textstr);
@@ -306,7 +326,8 @@ nsPlainTextSerializer::AppendText(nsIDOMText* aText,
 
 NS_IMETHODIMP 
 nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                          nsAWritableString& aStr)
+                                          PRBool aHasChildren,
+                                          nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -341,7 +362,7 @@ nsPlainTextSerializer::AppendElementStart(nsIDOMElement *aElement,
  
 NS_IMETHODIMP 
 nsPlainTextSerializer::AppendElementEnd(nsIDOMElement *aElement,
-                                        nsAWritableString& aStr)
+                                        nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -372,7 +393,7 @@ nsPlainTextSerializer::AppendElementEnd(nsIDOMElement *aElement,
 }
 
 NS_IMETHODIMP 
-nsPlainTextSerializer::Flush(nsAWritableString& aStr)
+nsPlainTextSerializer::Flush(nsAString& aStr)
 {
   mOutputString = &aStr;
   FlushLine();
@@ -393,7 +414,7 @@ NS_IMETHODIMP
 nsPlainTextSerializer::CloseContainer(const nsIParserNode& aNode)
 {
   PRInt32 type = aNode.GetNodeType();
-  const nsAReadableString&   namestr = aNode.GetText();
+  const nsAString&   namestr = aNode.GetText();
   nsCOMPtr<nsIAtom> name = dont_AddRef(NS_NewAtom(namestr));
   
   mParserNode = NS_CONST_CAST(nsIParserNode *, &aNode);
@@ -408,7 +429,7 @@ nsPlainTextSerializer::AddLeaf(const nsIParserNode& aNode)
   }
 
   eHTMLTags type = (eHTMLTags)aNode.GetNodeType();
-  const nsAReadableString& text = aNode.GetText();
+  const nsAString& text = aNode.GetText();
 
   mParserNode = NS_CONST_CAST(nsIParserNode *, &aNode);
   if ((type == eHTMLTag_text) ||
@@ -539,6 +560,9 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
     return NS_OK;
   }
 
+  if (mLineBreakDue)
+    EnsureVerticalSpace(mFloatingLines);
+
   // Check if this tag's content that should not be output
   if ((type == eHTMLTag_noscript &&
        !(mFlags & nsIDocumentEncoder::OutputNoScriptContent)) ||
@@ -634,15 +658,30 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
   }
   else if (type == eHTMLTag_ol) {
     EnsureVerticalSpace(mULCount + mOLStackIndex == 0 ? 1 : 0);
-         // Must end the current line before we change indention
+    // Must end the current line before we change indention
     if (mOLStackIndex < OLStackSize) {
-      mOLStack[mOLStackIndex++] = 1;  // XXX should get it from the node!
+      nsAutoString startAttr;
+      PRInt32 startVal = 1;
+      if(NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::start, startAttr))){
+        PRInt32 rv = 0;
+        startVal = startAttr.ToInteger(&rv);
+        if (NS_FAILED(rv))
+          startVal = 1;
+      }
+      mOLStack[mOLStackIndex++] = startVal;
     }
     mIndent += kIndentSizeList;  // see ul
   }
   else if (type == eHTMLTag_li) {
     if (mTagStackIndex > 1 && mTagStack[mTagStackIndex-2] == eHTMLTag_ol) {
       if (mOLStackIndex > 0) {
+        nsAutoString valueAttr;
+        if(NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::value, valueAttr))){
+          PRInt32 rv = 0;
+          PRInt32 valueAttrVal = valueAttr.ToInteger(&rv);
+          if (NS_SUCCEEDED(rv))
+            mOLStack[mOLStackIndex-1] = valueAttrVal;
+        }
         // This is what nsBulletFrame does for OLs:
         mInIndentString.AppendInt(mOLStack[mOLStackIndex-1]++, 10);
       }
@@ -654,7 +693,10 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
 
     }
     else {
-      mInIndentString.Append(PRUnichar('*'));
+      static char bulletCharArray[] = "*o+#";
+      NS_ASSERTION(mULCount > 0, "mULCount should be greater than 0 here");
+      char bulletChar = bulletCharArray[(mULCount - 1) % 4];
+      mInIndentString.Append(PRUnichar(bulletChar));
     }
     
     mInIndentString.Append(PRUnichar(' '));
@@ -725,7 +767,8 @@ nsPlainTextSerializer::DoOpenContainer(PRInt32 aTag)
     nsAutoString value;
     nsresult rv = GetAttributeValue(nsHTMLAtoms::type, value);
 
-    if (NS_SUCCEEDED(rv) && value.EqualsIgnoreCase("cite")) {
+    if (NS_SUCCEEDED(rv) &&
+        NS_LossyConvertUCS2toASCII(value).Equals("cite", nsCaseInsensitiveCStringComparator())) {
       mCiteQuoteLevel++;
     }
     else {
@@ -802,30 +845,41 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
            (type == eHTMLTag_li) ||
            (type == eHTMLTag_dt)) {
     // Items that should always end a line, but get no more whitespace
-    EnsureVerticalSpace(0);
+    if (mFloatingLines < 0)
+      mFloatingLines = 0;
+    mLineBreakDue = PR_TRUE;
   } 
   else if (type == eHTMLTag_pre) {
-    EnsureVerticalSpace(1);
+    mFloatingLines = 1;
+    mLineBreakDue = PR_TRUE;
   }
   else if (type == eHTMLTag_ul) {
     mIndent -= kIndentSizeList;
-    if (--mULCount + mOLStackIndex == 0)
-      EnsureVerticalSpace(1);
+    if (--mULCount + mOLStackIndex == 0) {
+      mFloatingLines = 1;
+      mLineBreakDue = PR_TRUE;
+    }
   }
   else if (type == eHTMLTag_ol) {
     FlushLine(); // Doing this after decreasing OLStackIndex would be wrong.
     mIndent -= kIndentSizeList;
     mOLStackIndex--;
-    if (mULCount + mOLStackIndex == 0)
-      EnsureVerticalSpace(1);
-  }
+    if (mULCount + mOLStackIndex == 0) {
+      mFloatingLines = 1;
+      mLineBreakDue = PR_TRUE;
+    }
+  }  
   else if (type == eHTMLTag_dd) {
     mIndent -= kIndentSizeDD;
   }
   else if (type == eHTMLTag_span) {
     --mSpanLevel;
   }
-
+  else if (type == eHTMLTag_div) {
+    if (mFloatingLines < 0)
+      mFloatingLines = 0;
+    mLineBreakDue = PR_TRUE;
+  }
   else if (IsBlockLevel(aTag)
            && type != eHTMLTag_blockquote
            && type != eHTMLTag_script
@@ -835,8 +889,13 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
     // in formatted mode, otherwise 0.
     // This is hard. Sometimes 0 is a better number, but
     // how to know?
-    EnsureVerticalSpace((mFlags & nsIDocumentEncoder::OutputFormatted)
-                        ? 1 : 0);
+    if (mFlags & nsIDocumentEncoder::OutputFormatted)
+      EnsureVerticalSpace(1);
+    else {
+      if (mFloatingLines < 0)
+        mFloatingLines = 0;
+      mLineBreakDue = PR_TRUE;
+    }
   }
 
   //////////////////////////////////////////////////////////////
@@ -869,7 +928,8 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
     nsAutoString value;
     nsresult rv = GetAttributeValue(nsHTMLAtoms::type, value);
 
-    if (NS_SUCCEEDED(rv)  && value.EqualsIgnoreCase("cite")) {
+    if (NS_SUCCEEDED(rv)  &&
+        NS_LossyConvertUCS2toASCII(value).Equals("cite", nsCaseInsensitiveCStringComparator())) {
       mCiteQuoteLevel--;
     }
     else {
@@ -913,13 +973,16 @@ nsPlainTextSerializer::DoCloseContainer(PRInt32 aTag)
 
 nsresult
 nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag, 
-                                 const nsAReadableString& aText)
+                                 const nsAString& aText)
 {
   // If we don't want any output, just return
   if (!DoOutput()) {
     return NS_OK;
   }
   
+  if (mLineBreakDue)
+    EnsureVerticalSpace(mFloatingLines);
+
   eHTMLTags type = (eHTMLTags)aTag;
   
   if ((mTagStackIndex > 1 &&
@@ -951,6 +1014,12 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
       nsAutoString str(aText);
       PRInt32 entity;
       parserService->HTMLConvertEntityToUnicode(str, &entity);
+      if (entity == -1 && 
+          !str.IsEmpty() &&
+          str.First() == (PRUnichar) '#') {
+        PRInt32 err = 0;
+        entity = str.ToInteger(&err, kAutoDetect);  // NCR
+      }
       nsAutoString temp(entity);
       Write(temp);
     }
@@ -975,8 +1044,7 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
     // we want to preserve whitespace.
     if (mFlags & nsIDocumentEncoder::OutputPreformatted ||
         (mPreFormatted && !mWrapColumn) ||
-        ((mFlags & nsIDocumentEncoder::OutputFormatted)
-         && IsInPre())) {
+        IsInPre()) {
       Write(aText);
     }
     else if(!mInWhitespace ||
@@ -1014,15 +1082,13 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
   }
   else if (type == eHTMLTag_img) {
     /* Output (in decreasing order of preference)
-       alt, title or src (URI) attribute */
+       alt, title or nothing */
     // See <http://www.w3.org/TR/REC-html40/struct/objects.html#edef-IMG>
     nsAutoString desc, temp;
     if (NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::alt, desc))) {
       if (!desc.IsEmpty()) {
-        temp.Append(NS_LITERAL_STRING(" [")); // Should we output chars at all here?
         desc.StripChars("\"");
         temp += desc;
-        temp.Append(NS_LITERAL_STRING("] "));
       }
       // If the alt attribute has an empty value (|alt=""|), output nothing
     }
@@ -1033,13 +1099,6 @@ nsPlainTextSerializer::DoAddLeaf(PRInt32 aTag,
       temp += desc;
       temp.Append(NS_LITERAL_STRING("] "));
     }
-    else if (NS_SUCCEEDED(GetAttributeValue(nsHTMLAtoms::src, desc))
-             && !desc.IsEmpty()) {
-      temp.Append(NS_LITERAL_STRING(" <"));
-      desc.StripChars("\"");
-      temp += desc;
-      temp.Append(NS_LITERAL_STRING("> "));
-    }
     if (!temp.IsEmpty()) {
       Write(temp);
     }
@@ -1061,13 +1120,15 @@ nsPlainTextSerializer::EnsureVerticalSpace(PRInt32 noOfRows)
   // If we have something in the indent we probably want to output
   // it and it's not included in the count for empty lines so we don't
   // realize that we should start a new line.
-  if(noOfRows >= 0 && mInIndentString.Length() > 0) {
+  if(noOfRows >= 0 && !mInIndentString.IsEmpty()) {
     EndLine(PR_FALSE);
   }
 
   while(mEmptyLines < noOfRows) {
     EndLine(PR_FALSE);
   }
+  mLineBreakDue = PR_FALSE;
+  mFloatingLines = -1;
 }
 
 /**
@@ -1081,13 +1142,13 @@ nsPlainTextSerializer::EnsureVerticalSpace(PRInt32 noOfRows)
 void
 nsPlainTextSerializer::FlushLine()
 {
-  if(mCurrentLine.Length()>0) {
+  if(!mCurrentLine.IsEmpty()) {
     if(mAtFirstColumn) {
       OutputQuotesAndIndent(); // XXX: Should we always do this? Bug?
     }
 
     Output(mCurrentLine);
-    mAtFirstColumn = mAtFirstColumn && mCurrentLine.Length() == 0;
+    mAtFirstColumn = mAtFirstColumn && mCurrentLine.IsEmpty();
     mCurrentLine.Truncate();
     mCurrentLineWidth = 0;
   }
@@ -1102,7 +1163,7 @@ nsPlainTextSerializer::FlushLine()
 void 
 nsPlainTextSerializer::Output(nsString& aString)
 {
-  if (aString.Length() > 0) {
+  if (!aString.IsEmpty()) {
     mStartedOutput = PR_TRUE;
   }
 
@@ -1127,6 +1188,9 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
 {
   PRUint32 prefixwidth = (mCiteQuoteLevel > 0 ? mCiteQuoteLevel + 1:0)+mIndent;
   
+  if (mLineBreakDue)
+    EnsureVerticalSpace(mFloatingLines);
+
   PRInt32 linelength = mCurrentLine.Length();
   if(0 == linelength) {
     if(0 == aLineFragmentLength) {
@@ -1173,7 +1237,6 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
 #endif
   }
 
-  
   linelength = mCurrentLine.Length();
 
   //  Wrap?
@@ -1266,7 +1329,7 @@ nsPlainTextSerializer::AddToLine(const PRUnichar * aLineFragment,
         // Space stuff new line?
         if(mFlags & nsIDocumentEncoder::OutputFormatFlowed) {
           if(
-              restOfLine.Length() > 0
+              !restOfLine.IsEmpty()
               &&
               (
                 restOfLine[0] == '>' ||
@@ -1345,7 +1408,7 @@ nsPlainTextSerializer::EndLine(PRBool aSoftlinebreak)
   } 
   else {
     // Hard break
-    if(mCurrentLine.Length()>0 || mInIndentString.Length()>0) {
+    if(!mCurrentLine.IsEmpty() || !mInIndentString.IsEmpty()) {
       mEmptyLines=-1;
     }
 
@@ -1356,7 +1419,7 @@ nsPlainTextSerializer::EndLine(PRBool aSoftlinebreak)
     // If we don't have anything "real" to output we have to
     // make sure the indent doesn't end in a space since that
     // would trick a format=flowed-aware receiver.
-    PRBool stripTrailingSpaces = (mCurrentLine.Length() == 0);
+    PRBool stripTrailingSpaces = mCurrentLine.IsEmpty();
     OutputQuotesAndIndent(stripTrailingSpaces);
   }
 
@@ -1366,6 +1429,8 @@ nsPlainTextSerializer::EndLine(PRBool aSoftlinebreak)
   mCurrentLineWidth = 0;
   mAtFirstColumn=PR_TRUE;
   mInWhitespace=PR_TRUE;
+  mLineBreakDue = PR_FALSE;
+  mFloatingLines = -1;
 }
 
 
@@ -1385,7 +1450,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
     for(int i=0; i < mCiteQuoteLevel; i++) {
       quotes.Append(PRUnichar('>'));
     }
-    if (mCurrentLine.Length() > 0) {
+    if (!mCurrentLine.IsEmpty()) {
       /* Better don't output a space here, if the line is empty,
          in case a recieving f=f-aware UA thinks, this were a flowed line,
          which it isn't - it's just empty.
@@ -1400,7 +1465,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
   // Indent if necessary
   PRInt32 indentwidth = mIndent - mInIndentString.Length();
   if (indentwidth > 0
-      && (mCurrentLine.Length() > 0 || mInIndentString.Length() > 0)
+      && (!mCurrentLine.IsEmpty() || !mInIndentString.IsEmpty())
       // Don't make empty lines look flowed
       ) {
     nsAutoString spaces;
@@ -1410,7 +1475,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
     mAtFirstColumn = PR_FALSE;
   }
   
-  if(mInIndentString.Length() > 0) {
+  if(!mInIndentString.IsEmpty()) {
     stringToOutput += mInIndentString;
     mAtFirstColumn = PR_FALSE;
     mInIndentString.Truncate();
@@ -1425,7 +1490,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
     stringToOutput.SetLength(lineLength);
   }
 
-  if(stringToOutput.Length() > 0) {
+  if(!stringToOutput.IsEmpty()) {
     Output(stringToOutput);
   }
     
@@ -1437,7 +1502,7 @@ nsPlainTextSerializer::OutputQuotesAndIndent(PRBool stripTrailingSpaces /* = PR_
  * line wrapping, indentation, whitespace compression and other things.
  */
 void
-nsPlainTextSerializer::Write(const nsAReadableString& aString)
+nsPlainTextSerializer::Write(const nsAString& aString)
 {
 #ifdef DEBUG_wrapping
   printf("Write(%s): wrap col = %d\n",
@@ -1456,16 +1521,15 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
   // that does normal formatted text. The one for preformatted text calls
   // Output directly while the other code path goes through AddToLine.
   if ((mPreFormatted && !mWrapColumn) || IsInPre()
-      || (!mQuotesPreformatted && mSpanLevel > 0
-          //&& Substring(aString, 0, 1) == NS_LITERAL_STRING(">"))) {
-          && aString.First() == PRUnichar('>'))) {
+      || ((((!mQuotesPreformatted && mSpanLevel > 0) || mDontWrapAnyQuotes))
+          && mEmptyLines >= 0 && aString.First() == PRUnichar('>'))) {
     // No intelligent wrapping.
 
     // This mustn't be mixed with intelligent wrapping without clearing
     // the mCurrentLine buffer before!!!
-    NS_WARN_IF_FALSE(mCurrentLine.Length() == 0,
+    NS_WARN_IF_FALSE(mCurrentLine.IsEmpty(),
                  "Mixed wrapping data and nonwrapping data on the same line");
-    if (mCurrentLine.Length() > 0) {
+    if (!mCurrentLine.IsEmpty()) {
       FlushLine();
     }
     
@@ -1476,7 +1540,7 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
         OutputQuotesAndIndent();
       }
 
-      // Find one of '\n' or '\r' using iterators since nsAReadableString
+      // Find one of '\n' or '\r' using iterators since nsAString
       // doesn't have the old FindCharInSet function.
       nsAString::const_iterator iter;           aString.BeginReading(iter);
       nsAString::const_iterator done_searching; aString.EndReading(done_searching);
@@ -1495,9 +1559,8 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
       // Done searching
       if(newline == kNotFound) {
         // No new lines.
-        nsAutoString stringpart;
-        aString.Right(stringpart, totLen-bol);
-        if(stringpart.Length() > 0) {
+        nsAutoString stringpart(Substring(aString, bol, totLen - bol));
+        if(!stringpart.IsEmpty()) {
           PRUnichar lastchar = stringpart[stringpart.Length()-1];
           if((lastchar == '\t') || (lastchar == ' ') ||
              (lastchar == '\r') ||(lastchar == '\n')) {
@@ -1514,8 +1577,7 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
       } 
       else {
         // There is a newline
-        nsAutoString stringpart;
-        aString.Mid(stringpart, bol, newline-bol);
+        nsAutoString stringpart(Substring(aString, bol, newline-bol));
         mInWhitespace = PR_TRUE;
         Output(stringpart);
         // and write the newline
@@ -1571,7 +1633,16 @@ nsPlainTextSerializer::Write(const nsAReadableString& aString)
     } 
     else {
       // There's still whitespace left in the string
-
+      if (nextpos != 0 && (nextpos + 1) < totLen) {
+        offsetIntoBuffer = str.get() + nextpos;
+        // skip '\n' if it is between CJ chars
+        if (offsetIntoBuffer[0] == '\n' && IS_CJ_CHAR(offsetIntoBuffer[-1]) && IS_CJ_CHAR(offsetIntoBuffer[1])) {
+          offsetIntoBuffer = str.get() + bol;
+          AddToLine(offsetIntoBuffer, nextpos-bol);
+          bol = nextpos + 1;
+          continue;
+        }
+      }
       // If we're already in whitespace and not preformatted, just skip it:
       if (mInWhitespace && (nextpos == bol) && !mPreFormatted &&
           !(mFlags & nsIDocumentEncoder::OutputPreformatted)) {
@@ -1629,8 +1700,8 @@ nsPlainTextSerializer::GetAttributeValue(nsIAtom* aName,
 
     PRInt32 count = mParserNode->GetAttributeCount();
     for (PRInt32 i=0;i<count;i++) {
-      const nsAReadableString& key = mParserNode->GetKeyAt(i);
-      if (key.Equals(name)) {
+      const nsAString& key = mParserNode->GetKeyAt(i);
+      if (key.Equals(name, nsCaseInsensitiveStringComparator())) {
         aValueRet = mParserNode->GetValueAt(i);
         aValueRet.StripChars("\"");
         return NS_OK;
@@ -1907,7 +1978,7 @@ PRInt32 GetUnicharStringWidth(const PRUnichar* pwcs, PRInt32 n)
 
   for (;*pwcs && n-- > 0; pwcs++)
     if ((w = GetUnicharWidth(*pwcs)) < 0)
-      return -1;
+      ++width; // Taking 1 as the width of non-printable character, for bug# 94475.
     else
       width += w;
 
