@@ -43,7 +43,6 @@
 #include "nsIScriptElement.h"
 #include "nsIDocShell.h"
 #include "jsapi.h"
-#include "nsIHttpChannel.h"
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
@@ -196,22 +195,6 @@ nsScriptLoader::RemoveObserver(nsIScriptLoaderObserver *aObserver)
   return NS_OK;
 }
 
-static void SplitMimeType(const nsString& aValue, nsString& aType, nsString& aParams)
-{
-  aType.Truncate();
-  aParams.Truncate();
-  PRInt32 semiIndex = aValue.FindChar(PRUnichar(';'));
-  if (-1 != semiIndex) {
-    aValue.Left(aType, semiIndex);
-    aValue.Right(aParams, (aValue.Length() - semiIndex) - 1);
-    aParams.StripWhitespace();
-  }
-  else {
-    aType = aValue;
-  }
-  aType.StripWhitespace();
-}
-
 PRBool
 nsScriptLoader::InNonScriptingContainer(nsIDOMHTMLScriptElement* aScriptElement)
 {
@@ -291,14 +274,20 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
 
   PRBool isJavaScript = PR_TRUE;
   const char* jsVersionString = nsnull;
-  nsAutoString src, type, language;
+  nsAutoString language, type, src;
+
+  // Check the language attribute first, so type can trump language.
+  aElement->GetAttribute(NS_LITERAL_STRING("language"), language);
+  if (!language.IsEmpty()) {
+    isJavaScript = nsParserUtils::IsJavaScriptLanguage(language, &jsVersionString);
+  }
 
   // Check the type attribute to determine language and version.
   aElement->GetType(type);
   if (!type.IsEmpty()) {
     nsAutoString  mimeType;
     nsAutoString  params;
-    SplitMimeType(type, mimeType, params);
+    nsParserUtils::SplitMimeType(type, mimeType, params);
 
     isJavaScript = mimeType.EqualsIgnoreCase("application/x-javascript") || 
                    mimeType.EqualsIgnoreCase("text/javascript");
@@ -321,12 +310,6 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
     }
   }
 
-  // Check the language attribute.
-  aElement->GetAttribute(NS_LITERAL_STRING("language"), language);
-  if (!language.IsEmpty()) {
-    isJavaScript = nsParserUtils::IsJavaScriptLanguage(language, &jsVersionString);
-  }
-
   // If this isn't JavaScript, we don't know how to evaluate.
   // XXX How and where should we deal with other scripting languages??
   if (!isJavaScript) {
@@ -348,7 +331,7 @@ nsScriptLoader::ProcessScriptElement(nsIDOMHTMLScriptElement *aElement,
     
     // Use the SRC attribute value to load the URL
     mDocument->GetBaseURL(*getter_AddRefs(baseURI));
-    rv = NS_NewURI(getter_AddRefs(scriptURI), src, baseURI);
+    rv = NS_NewURI(getter_AddRefs(scriptURI), src, nsnull, baseURI);
     if (NS_FAILED(rv)) {
       return FireErrorNotification(rv, aElement, aObserver);
     }
@@ -568,10 +551,10 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   NS_ASSERTION(principal, "principal required for document");
 
   nsAutoString ret;
-  nsXPIDLCString url;
+  nsCAutoString url;
 
   if (aRequest->mURI) {
-    rv = aRequest->mURI->GetSpec(getter_Copies(url));
+    rv = aRequest->mURI->GetSpec(url);
     if (NS_FAILED(rv)) {
       return rv;
     }
@@ -580,7 +563,7 @@ nsScriptLoader::EvaluateScript(nsScriptLoadRequest* aRequest,
   context->SetProcessingScriptTag(PR_TRUE);
 
   PRBool isUndefined;
-  context->EvaluateString(aScript, nsnull, principal, url,
+  context->EvaluateString(aScript, nsnull, principal, url.get(),
                           aRequest->mLineNo, aRequest->mJSVersion, 
                           ret, &isUndefined);  
 
@@ -637,7 +620,6 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
   if (stringLen) {
     nsAutoString characterSet, preferred;
     nsCOMPtr<nsIUnicodeDecoder> unicodeDecoder;
-    nsCOMPtr<nsIHttpChannel> httpChannel;
 
     nsCOMPtr<nsIChannel> channel;
     nsCOMPtr<nsIRequest> req;
@@ -646,14 +628,12 @@ nsScriptLoader::OnStreamComplete(nsIStreamLoader* aLoader,
     if (NS_FAILED(rv)) return rv;
 
     channel = do_QueryInterface(req);
-
-    httpChannel = do_QueryInterface(channel);
-    if (httpChannel) {
-      nsXPIDLCString charsetheader;
-      rv = httpChannel->GetCharset(getter_Copies(charsetheader));
+    if (channel) {
+      nsCAutoString charsetVal;
+      rv = channel->GetContentCharset(charsetVal);
     
       if (NS_SUCCEEDED(rv)) {
-        characterSet = NS_ConvertASCIItoUCS2(charsetheader);      
+        characterSet = NS_ConvertASCIItoUCS2(charsetVal);      
         nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
 
         if(NS_SUCCEEDED(rv) && calias) {
