@@ -58,6 +58,8 @@
 #include "nsRuleWalker.h"
 #include "nsIHTMLDocument.h"
 #include "nsIDOMHTMLBodyElement.h"
+#include "nsHTMLAtoms.h"
+#include "nsHashtable.h"
 
 #ifdef MOZ_PERF_METRICS
   #include "nsITimeRecorder.h"
@@ -72,6 +74,34 @@
 
 #include "nsISizeOfHandler.h"
 
+// =====================================================
+// nsRuleNodeList
+// A class that represents a chain of rule nodes
+
+struct nsRuleNodeList
+{
+  nsRuleNodeList(nsRuleNode* aRuleNode, nsRuleNodeList* aNext = nsnull)
+    :mRuleNode(aRuleNode), mNext(aNext)
+  {};
+    
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
+    void* result = nsnull;
+    aContext->AllocateFromShell(sz, &result);
+    return result;
+  };
+
+  void Destroy() {
+    if (mNext)
+      mNext->Destroy();
+    mRuleNode->PresContext()->FreeToShell(sizeof(nsRuleNodeList), this);
+  };
+
+  nsRuleNode* mRuleNode;
+  nsRuleNodeList* mNext;
+};
+
+// =====================================================
+
 static NS_DEFINE_IID(kIStyleFrameConstructionIID, NS_ISTYLE_FRAME_CONSTRUCTION_IID);
 
 class StyleSetImpl : public nsIStyleSet 
@@ -124,26 +154,22 @@ public:
 
   virtual nsIStyleContext* ResolveStyleFor(nsIPresContext* aPresContext,
                                            nsIContent* aContent,
-                                           nsIStyleContext* aParentContext,
-                                           PRBool aForceUnique = PR_FALSE);
+                                           nsIStyleContext* aParentContext);
 
   virtual nsIStyleContext* ResolveStyleForNonElement(
                                            nsIPresContext* aPresContext,
-                                           nsIStyleContext* aParentContext,
-                                           PRBool aForceUnique = PR_FALSE);
+                                           nsIStyleContext* aParentContext);
 
   virtual nsIStyleContext* ResolvePseudoStyleFor(nsIPresContext* aPresContext,
                                                  nsIContent* aParentContent,
                                                  nsIAtom* aPseudoTag,
                                                  nsIStyleContext* aParentContext,
-                                                 PRBool aForceUnique = PR_FALSE,
                                                  nsICSSPseudoComparator* aComparator = nsnull);
 
   virtual nsIStyleContext* ProbePseudoStyleFor(nsIPresContext* aPresContext,
                                                nsIContent* aParentContent,
                                                nsIAtom* aPseudoTag,
-                                               nsIStyleContext* aParentContext,
-                                               PRBool aForceUnique = PR_FALSE);
+                                               nsIStyleContext* aParentContext);
 
   NS_IMETHOD Shutdown();
 
@@ -163,6 +189,8 @@ public:
   
   virtual nsresult GetRuleTree(nsRuleNode** aResult);
   virtual nsresult ClearCachedDataInRuleTree(nsIStyleRule* aRule);
+  
+  virtual nsresult AddRuleNodeMapping(nsRuleNode* aRuleNode);
 
   virtual nsresult ClearStyleData(nsIPresContext* aPresContext, nsIStyleRule* aRule, nsIStyleContext* aContext);
 
@@ -177,8 +205,10 @@ public:
                                   nsIStyleContext* aNewParentContext,
                                   nsIStyleContext** aNewStyleContext);
 
-  NS_IMETHOD  HasStateDependentStyle(nsIPresContext* aPresContext,
-                                     nsIContent*     aContent);
+  NS_IMETHOD HasStateDependentStyle(nsIPresContext* aPresContext,
+                                    nsIContent*     aContent,
+                                    PRInt32         aStateMask,
+                                    PRBool*         aResult);
 
   NS_IMETHOD ConstructRootFrame(nsIPresContext* aPresContext,
                                 nsIContent*     aContent,
@@ -206,7 +236,8 @@ public:
                             nsISupports* aSubContent);
   NS_IMETHOD ContentStatesChanged(nsIPresContext* aPresContext, 
                                   nsIContent* aContent1,
-                                  nsIContent* aContent2);
+                                  nsIContent* aContent2,
+                                  PRInt32 aStateMask);
   NS_IMETHOD AttributeChanged(nsIPresContext*  aPresContext,
                               nsIContent* aChild,
                               PRInt32 aNameSpaceID,
@@ -309,8 +340,8 @@ protected:
 
   nsIStyleContext* GetContext(nsIPresContext* aPresContext, 
                               nsIStyleContext* aParentContext,
-                              nsIAtom* aPseudoTag, 
-                              PRBool aForceUnique);
+                              nsIAtom* aPseudoTag);
+
 #ifdef DEBUG
   void  List(FILE* out, PRInt32 aIndent, nsISupportsArray* aSheets);
   void  ListContexts(nsIStyleContext* aRootContext, FILE* out, PRInt32 aIndent);
@@ -338,6 +369,7 @@ protected:
   nsRuleNode* mOldRuleTree; // Used during rule tree reconstruction.
   nsRuleWalker* mRuleWalker;   // This is an instance of a rule walker that can be used
                                // to navigate through our tree.
+  nsHashtable mRuleMappings; // A hashtable from rules to rule node lists.
 
   MOZ_TIMER_DECLARE(mStyleResolutionWatch)
 
@@ -355,7 +387,8 @@ StyleSetImpl::StyleSetImpl()
     mQuirkStyleSheet(nsnull),
     mRuleTree(nsnull),
     mOldRuleTree(nsnull),
-    mRuleWalker(nsnull)
+    mRuleWalker(nsnull),
+    mRuleMappings(32)
 #ifdef MOZ_PERF_METRICS
     ,mTimerEnabled(PR_FALSE)
 #endif
@@ -363,8 +396,8 @@ StyleSetImpl::StyleSetImpl()
   NS_INIT_REFCNT();
   if (gInstances++ == 0)
   {
-    const char kQuirk_href[] = "resource:/res/quirk.css";
-    NS_NewURI (&gQuirkURI, kQuirk_href);
+    static const char kQuirk_href[] = "resource:/res/quirk.css";
+    NS_NewURI (&gQuirkURI, NS_LITERAL_CSTRING(kQuirk_href));
     NS_ASSERTION (gQuirkURI != 0, "Cannot allocate nsStyleSetImpl::gQuirkURI");
   }
 }
@@ -831,9 +864,21 @@ NS_IMETHODIMP StyleSetImpl::EnableQuirkStyleSheet(PRBool aEnable)
       }
     }
   }
+  NS_ASSERTION(mQuirkStyleSheet, "no quirk stylesheet");
   if (mQuirkStyleSheet) {
 #if defined(DEBUG_warren) || defined(DEBUG_attinasi)
     printf( "%s Quirk StyleSheet\n", aEnable ? "Enabling" : "Disabling" );
+#endif
+#ifdef DEBUG_dbaron // XXX Make this |DEBUG| once it stops firing.
+    PRUint32 count = 0;
+    if (mAgentRuleProcessors)
+      mAgentRuleProcessors->Count(&count);
+    PRBool enabledNow;
+    mQuirkStyleSheet->GetEnabled(enabledNow);
+    NS_ASSERTION(count == 0 || aEnable == enabledNow,
+                 "enabling/disabling quirk stylesheet too late");
+    if (count != 0 && aEnable == enabledNow)
+      printf("WARNING: We set the quirks mode too many times.\n"); // we do!
 #endif
     mQuirkStyleSheet->SetEnabled(aEnable);
   }
@@ -887,33 +932,32 @@ EnumRulesMatching(nsISupports* aProcessor, void* aData)
   return PR_TRUE;
 }
 
+/**
+ * |GetContext| implements sharing of style contexts (not just the data
+ * on the rule nodes) between siblings and cousins of the same
+ * generation.  (It works for cousins of the same generation since
+ * |aParentContext| could itself be a shared context.)
+ */
 nsIStyleContext* StyleSetImpl::GetContext(nsIPresContext* aPresContext, 
                                           nsIStyleContext* aParentContext, 
-                                          nsIAtom* aPseudoTag, 
-                                          PRBool aForceUnique)
+                                          nsIAtom* aPseudoTag)
 {
   nsIStyleContext* result = nsnull;
-  
   nsRuleNode* ruleNode = mRuleWalker->GetCurrentNode();
       
-  if ((PR_FALSE == aForceUnique) && (nsnull != aParentContext)) {
+  if (aParentContext)
     aParentContext->FindChildWithRules(aPseudoTag, ruleNode, result);
-  }
 
-  if (nsnull == result) {
-    if (NS_SUCCEEDED(NS_NewStyleContext(&result, aParentContext, aPseudoTag, ruleNode, aPresContext))) {
-      if (PR_TRUE == aForceUnique)
-        result->ForceUnique();
-    }
 #ifdef NOISY_DEBUG
+  if (result)
+    fprintf(stdout, "--- SharedSC %d ---\n", ++gSharedCount);
+  else
     fprintf(stdout, "+++ NewSC %d +++\n", ++gNewCount);
 #endif
-  }
-#ifdef NOISY_DEBUG
-  else {
-    fprintf(stdout, "--- SharedSC %d ---\n", ++gSharedCount);
-  }
-#endif
+
+  if (!result)
+    NS_NewStyleContext(&result, aParentContext, aPseudoTag, ruleNode,
+                       aPresContext);
 
   return result;
 }
@@ -1047,8 +1091,7 @@ void StyleSetImpl::EnsureRuleWalker(nsIPresContext* aPresContext)
 
 nsIStyleContext* StyleSetImpl::ResolveStyleFor(nsIPresContext* aPresContext,
                                                nsIContent* aContent,
-                                               nsIStyleContext* aParentContext,
-                                               PRBool aForceUnique)
+                                               nsIStyleContext* aParentContext)
 {
   MOZ_TIMER_DEBUGLOG(("Start: StyleSetImpl::ResolveStyleFor(), this=%p\n", this));
   STYLESET_START_TIMER(NS_TIMER_STYLE_RESOLUTION);
@@ -1071,7 +1114,7 @@ nsIStyleContext* StyleSetImpl::ResolveStyleFor(nsIPresContext* aPresContext,
       aPresContext->GetMedium(getter_AddRefs(medium));
       RulesMatchingData data(aPresContext, medium, aContent, mRuleWalker);
       FileRules(EnumRulesMatching, &data);
-      result = GetContext(aPresContext, aParentContext, nsnull, aForceUnique);
+      result = GetContext(aPresContext, aParentContext, nsnull);
      
       // Now reset the walker back to the root of the tree.
       mRuleWalker->Reset();
@@ -1085,8 +1128,7 @@ nsIStyleContext* StyleSetImpl::ResolveStyleFor(nsIPresContext* aPresContext,
 
 nsIStyleContext* StyleSetImpl::ResolveStyleForNonElement(
                                                nsIPresContext* aPresContext,
-                                               nsIStyleContext* aParentContext,
-                                               PRBool aForceUnique)
+                                               nsIStyleContext* aParentContext)
 {
   MOZ_TIMER_DEBUGLOG(("Start: StyleSetImpl::ResolveStyleForNonElement(), this=%p\n", this));
   STYLESET_START_TIMER(NS_TIMER_STYLE_RESOLUTION);
@@ -1102,7 +1144,8 @@ nsIStyleContext* StyleSetImpl::ResolveStyleForNonElement(
         mDocRuleProcessors   ||
         mOverrideRuleProcessors) {
       EnsureRuleWalker(aPresContext);
-      result = GetContext(aPresContext, aParentContext, nsnull, aForceUnique);
+      result = GetContext(aPresContext, aParentContext,
+                          nsHTMLAtoms::mozNonElementPseudo);
       NS_ASSERTION(mRuleWalker->AtRoot(), "rule walker must be at root");
     }
   }
@@ -1142,7 +1185,6 @@ nsIStyleContext* StyleSetImpl::ResolvePseudoStyleFor(nsIPresContext* aPresContex
                                                      nsIContent* aParentContent,
                                                      nsIAtom* aPseudoTag,
                                                      nsIStyleContext* aParentContext,
-                                                     PRBool aForceUnique,
                                                      nsICSSPseudoComparator* aComparator)
 {
   MOZ_TIMER_DEBUGLOG(("Start: StyleSetImpl::ResolvePseudoStyleFor(), this=%p\n", this));
@@ -1169,7 +1211,7 @@ nsIStyleContext* StyleSetImpl::ResolvePseudoStyleFor(nsIPresContext* aPresContex
                                    aPseudoTag, aComparator, mRuleWalker);
       FileRules(EnumPseudoRulesMatching, &data);
 
-      result = GetContext(aPresContext, aParentContext, aPseudoTag, aForceUnique);
+      result = GetContext(aPresContext, aParentContext, aPseudoTag);
      
       // Now reset the walker back to the root of the tree.
       mRuleWalker->Reset();
@@ -1184,8 +1226,7 @@ nsIStyleContext* StyleSetImpl::ResolvePseudoStyleFor(nsIPresContext* aPresContex
 nsIStyleContext* StyleSetImpl::ProbePseudoStyleFor(nsIPresContext* aPresContext,
                                                    nsIContent* aParentContent,
                                                    nsIAtom* aPseudoTag,
-                                                   nsIStyleContext* aParentContext,
-                                                   PRBool aForceUnique)
+                                                   nsIStyleContext* aParentContext)
 {
   MOZ_TIMER_DEBUGLOG(("Start: StyleSetImpl::ProbePseudoStyleFor(), this=%p\n", this));
   STYLESET_START_TIMER(NS_TIMER_STYLE_RESOLUTION);
@@ -1212,7 +1253,7 @@ nsIStyleContext* StyleSetImpl::ProbePseudoStyleFor(nsIPresContext* aPresContext,
       FileRules(EnumPseudoRulesMatching, &data);
 
       if (!mRuleWalker->AtRoot())
-        result = GetContext(aPresContext, aParentContext, aPseudoTag, aForceUnique);
+        result = GetContext(aPresContext, aParentContext, aPseudoTag);
  
       // Now reset the walker back to the root of the tree.
       mRuleWalker->Reset();
@@ -1224,9 +1265,19 @@ nsIStyleContext* StyleSetImpl::ProbePseudoStyleFor(nsIPresContext* aPresContext,
   return result;
 }
 
+PRBool PR_CALLBACK DeleteRuleNodeLists(nsHashKey* aKey, void* aData, void* aClosure)
+{
+  nsRuleNodeList* ruleNodeList = (nsRuleNodeList*)aData;
+  ruleNodeList->Destroy();
+  return PR_TRUE;
+}
+
 NS_IMETHODIMP
 StyleSetImpl::Shutdown()
 {
+  mRuleMappings.Enumerate(DeleteRuleNodeLists);
+  mRuleMappings.Reset();
+
   delete mRuleWalker;
   if (mRuleTree)
   {
@@ -1243,6 +1294,17 @@ StyleSetImpl::GetRuleTree(nsRuleNode** aResult)
   return NS_OK;
 }
 
+nsresult 
+StyleSetImpl::AddRuleNodeMapping(nsRuleNode* aRuleNode)
+{
+  nsVoidKey key(aRuleNode->Rule());
+  nsRuleNodeList* ruleList = 
+    new (aRuleNode->PresContext()) nsRuleNodeList(aRuleNode, 
+                                                  NS_STATIC_CAST(nsRuleNodeList*, mRuleMappings.Get(&key)));
+  mRuleMappings.Put(&key, ruleList);
+  return NS_OK;
+}
+
 nsresult
 StyleSetImpl::BeginRuleTreeReconstruct()
 {
@@ -1250,6 +1312,8 @@ StyleSetImpl::BeginRuleTreeReconstruct()
   mRuleWalker = nsnull;
   mOldRuleTree = mRuleTree;
   mRuleTree = nsnull;
+  mRuleMappings.Enumerate(DeleteRuleNodeLists);
+  mRuleMappings.Reset();
   return NS_OK;
 }
 
@@ -1280,6 +1344,20 @@ StyleSetImpl::ClearStyleData(nsIPresContext* aPresContext, nsIStyleRule* aRule,
   // style.left, we really only need to blow away cached
   // data in the position struct.
   if (aContext) {
+    // |aRule| should never be null, but we'll sanity check it just in case.
+    if (aRule) {
+      // Obtain our rule node list and clear out the cached data in all rule nodes
+      // that correspond to this rule.  See bug 99344 for more details as to how
+      // inline style rules can end up with multiple rule nodes in a rule tree.
+      nsVoidKey key(aRule);
+      nsRuleNodeList* ruleList = NS_STATIC_CAST(nsRuleNodeList*, mRuleMappings.Get(&key));
+      for ( ; ruleList; ruleList = ruleList->mNext)
+        ruleList->mRuleNode->ClearCachedData(aRule);
+    }
+    
+    // XXXdwh I'm just being paranoid here.  Also clear out the data starting at the style
+    // context's rule node.  This really should always be done in the for loop above,
+    // but I'm going to leave this here just in case (for now).
     nsRuleNode* ruleNode;
     aContext->GetRuleNode(&ruleNode);
     ruleNode->ClearCachedData(aRule); 
@@ -1368,8 +1446,9 @@ StyleSetImpl::ReParentStyleContext(nsIPresContext* aPresContext,
 }
 
 struct StatefulData : public StateRuleProcessorData {
-  StatefulData(nsIPresContext* aPresContext, nsIAtom* aMedium, nsIContent* aContent)
-    : StateRuleProcessorData(aPresContext, aContent),
+  StatefulData(nsIPresContext* aPresContext, nsIAtom* aMedium,
+               nsIContent* aContent, PRInt32 aStateMask)
+    : StateRuleProcessorData(aPresContext, aContent, aStateMask),
       mMedium(aMedium),
       mStateful(PR_FALSE)
   {}
@@ -1381,9 +1460,16 @@ static PRBool SheetHasStatefulStyle(nsISupports* aProcessor, void *aData)
 {
   nsIStyleRuleProcessor* processor = (nsIStyleRuleProcessor*)aProcessor;
   StatefulData* data = (StatefulData*)aData;
-  if (NS_OK == processor->HasStateDependentStyle(data, data->mMedium)) {
+  PRBool hasStateful;
+  processor->HasStateDependentStyle(data, data->mMedium, &hasStateful);
+  if (hasStateful) {
     data->mStateful = PR_TRUE;
-    return PR_FALSE;  // stop iteration
+    // Stop iteration.  Note that StyleSetImpl::WalkRuleProcessors uses
+    // this to stop its own iteration in some cases, but not all (the
+    // style rule supplier case).  Since this optimization is only for
+    // the case where we have a lot more work to do, it's not worth the
+    // code needed to make the stopping perfect.
+    return PR_FALSE;
   }
   return PR_TRUE; // continue
 }
@@ -1391,7 +1477,9 @@ static PRBool SheetHasStatefulStyle(nsISupports* aProcessor, void *aData)
 // Test if style is dependent on content state
 NS_IMETHODIMP
 StyleSetImpl::HasStateDependentStyle(nsIPresContext* aPresContext,
-                                     nsIContent*     aContent)
+                                     nsIContent*     aContent,
+                                     PRInt32         aStateMask,
+                                     PRBool*         aResult)
 {
   GatherRuleProcessors();
 
@@ -1402,12 +1490,15 @@ StyleSetImpl::HasStateDependentStyle(nsIPresContext* aPresContext,
        mOverrideRuleProcessors)) {  
     nsIAtom* medium = nsnull;
     aPresContext->GetMedium(&medium);
-    StatefulData data(aPresContext, medium, aContent);
+    StatefulData data(aPresContext, medium, aContent, aStateMask);
     WalkRuleProcessors(SheetHasStatefulStyle, &data);
     NS_IF_RELEASE(medium);
-    return ((data.mStateful) ? NS_OK : NS_COMFALSE);
+    *aResult = data.mStateful;
+  } else {
+    *aResult = PR_FALSE;
   }
-  return NS_COMFALSE;
+
+  return NS_OK;
 }
 
 
@@ -1476,9 +1567,11 @@ StyleSetImpl::ContentChanged(nsIPresContext* aPresContext,
 NS_IMETHODIMP
 StyleSetImpl::ContentStatesChanged(nsIPresContext* aPresContext, 
                                    nsIContent* aContent1,
-                                   nsIContent* aContent2)
+                                   nsIContent* aContent2,
+                                   PRInt32 aStateMask)
 {
-  return mFrameConstructor->ContentStatesChanged(aPresContext, aContent1, aContent2);
+  return mFrameConstructor->ContentStatesChanged(aPresContext, aContent1,
+                                                 aContent2, aStateMask);
 }
 
 
