@@ -38,6 +38,7 @@
 
 #include "nsXMLContentSerializer.h"
 
+#include "nsHTMLAtoms.h"
 #include "nsIDOMText.h"
 #include "nsIDOMCDATASection.h"
 #include "nsIDOMProcessingInstruction.h"
@@ -52,6 +53,7 @@
 #include "nsString.h"
 #include "prprf.h"
 #include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 typedef struct {
   nsString mPrefix;
@@ -84,7 +86,7 @@ NS_IMPL_ISUPPORTS1(nsXMLContentSerializer, nsIContentSerializer)
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::Init(PRUint32 flags, PRUint32 aWrapColumn,
-                             nsIAtom* aCharSet)
+                             nsIAtom* aCharSet, PRBool aIsCopying)
 {
   return NS_OK;
 }
@@ -93,7 +95,7 @@ nsresult
 nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode, 
                                        PRInt32 aStartOffset,
                                        PRInt32 aEndOffset,
-                                       nsAWritableString& aStr,
+                                       nsAString& aStr,
                                        PRBool aTranslateEntities,
                                        PRBool aIncrColumn)
 {
@@ -104,17 +106,20 @@ nsXMLContentSerializer::AppendTextData(nsIDOMNode* aNode,
   content->GetText(&frag);
 
   if (frag) {
-    PRInt32 length = ((aEndOffset == -1) ? frag->GetLength() : aEndOffset) - aStartOffset;
+    PRInt32 endoffset = (aEndOffset == -1) ? frag->GetLength() : aEndOffset;
+    PRInt32 length = endoffset - aStartOffset;
     
+    NS_ASSERTION(aStartOffset >= 0, "Negative start offset for text fragment!");
+    NS_ASSERTION(aStartOffset <= endoffset, "A start offset is beyond the end of the text fragment!");
+
     if (length <= 0) {
-      NS_ASSERTION(aStartOffset == 0,
-                   "a start offset is beyond the end of the text fragment!");
       return NS_OK;  // XXX Zero is a legal value, maybe non-zero values should
                      //     be an error.
     }
     
     if (frag->Is2b()) {
-      AppendToString(nsDependentString(frag->Get2b()+aStartOffset, length),
+      const PRUnichar *strStart = frag->Get2b() + aStartOffset;
+      AppendToString(Substring(strStart, strStart + length),
                      aStr,
                      aTranslateEntities,
                      aIncrColumn);
@@ -134,7 +139,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendText(nsIDOMText* aText, 
                                    PRInt32 aStartOffset,
                                    PRInt32 aEndOffset,
-                                   nsAWritableString& aStr)
+                                   nsAString& aStr)
 {
   NS_ENSURE_ARG(aText);
 
@@ -145,7 +150,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendCDATASection(nsIDOMCDATASection* aCDATASection,
                                            PRInt32 aStartOffset,
                                            PRInt32 aEndOffset,
-                                           nsAWritableString& aStr)
+                                           nsAString& aStr)
 {
   NS_ENSURE_ARG(aCDATASection);
   nsresult rv;
@@ -162,7 +167,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendProcessingInstruction(nsIDOMProcessingInstruction* aPI,
                                                     PRInt32 aStartOffset,
                                                     PRInt32 aEndOffset,
-                                                    nsAWritableString& aStr)
+                                                    nsAString& aStr)
 {
   NS_ENSURE_ARG(aPI);
   nsresult rv;
@@ -176,7 +181,7 @@ nsXMLContentSerializer::AppendProcessingInstruction(nsIDOMProcessingInstruction*
 
   AppendToString(NS_LITERAL_STRING("<?"), aStr);
   AppendToString(target, aStr);
-  if (data.Length() > 0) {
+  if (!data.IsEmpty()) {
     AppendToString(NS_LITERAL_STRING(" "), aStr);
     AppendToString(data, aStr);
   }
@@ -189,7 +194,7 @@ NS_IMETHODIMP
 nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
                                       PRInt32 aStartOffset,
                                       PRInt32 aEndOffset,
-                                      nsAWritableString& aStr)
+                                      nsAString& aStr)
 {
   NS_ENSURE_ARG(aComment);
   nsresult rv;
@@ -217,7 +222,7 @@ nsXMLContentSerializer::AppendComment(nsIDOMComment* aComment,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
-                                      nsAWritableString& aStr)
+                                      nsAString& aStr)
 {
   NS_ENSURE_ARG(aDoctype);
   nsresult rv;
@@ -235,7 +240,7 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
   AppendToString(NS_LITERAL_STRING("<!DOCTYPE "), aStr);
   AppendToString(name, aStr);
   PRUnichar quote;
-  if (publicId.Length() > 0) {
+  if (!publicId.IsEmpty()) {
     AppendToString(NS_LITERAL_STRING(" PUBLIC "), aStr);
     if (publicId.FindChar(PRUnichar('"')) == -1) {
       quote = PRUnichar('"');
@@ -247,7 +252,7 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
     AppendToString(publicId, aStr);
     AppendToString(quote, aStr);
 
-    if (systemId.Length()) {
+    if (!systemId.IsEmpty()) {
       AppendToString(PRUnichar(' '), aStr);
       if (systemId.FindChar(PRUnichar('"')) == -1) {
         quote = PRUnichar('"');
@@ -260,7 +265,7 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
       AppendToString(quote, aStr);
     }
   }
-  else if (systemId.Length() > 0) {
+  else if (!systemId.IsEmpty()) {
     if (systemId.FindChar(PRUnichar('"')) == -1) {
       quote = PRUnichar('"');
     }
@@ -273,7 +278,7 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
     AppendToString(quote, aStr);
   }
   
-  if (internalSubset.Length() > 0) {
+  if (!internalSubset.IsEmpty()) {
     AppendToString(PRUnichar(' '), aStr);
     AppendToString(internalSubset, aStr);
   }
@@ -286,8 +291,8 @@ nsXMLContentSerializer::AppendDoctype(nsIDOMDocumentType *aDoctype,
 #define kXMLNS NS_LITERAL_STRING("xmlns")
 
 nsresult
-nsXMLContentSerializer::PushNameSpaceDecl(const nsAReadableString& aPrefix,
-                                          const nsAReadableString& aURI,
+nsXMLContentSerializer::PushNameSpaceDecl(const nsAString& aPrefix,
+                                          const nsAString& aURI,
                                           nsIDOMElement* aOwner)
 {
   NameSpaceDecl* decl = new NameSpaceDecl();
@@ -323,13 +328,13 @@ nsXMLContentSerializer::PopNameSpaceDeclsFor(nsIDOMElement* aOwner)
  * moved/modified elements/attributes 
  */
 PRBool
-nsXMLContentSerializer::ConfirmPrefix(nsAWritableString& aPrefix,
-                                      const nsAReadableString& aURI)
+nsXMLContentSerializer::ConfirmPrefix(nsAString& aPrefix,
+                                      const nsAString& aURI)
 {
   if (aPrefix.Equals(kXMLNS)) {
     return PR_FALSE;
   }
-  if (aURI.Length() == 0) {
+  if (aURI.IsEmpty()) {
     aPrefix.Truncate();
     return PR_FALSE;
   }
@@ -366,7 +371,7 @@ nsXMLContentSerializer::ConfirmPrefix(nsAWritableString& aPrefix,
     return PR_FALSE;
   }
   // If we don't have a prefix, create one
-  else if (aPrefix.Length() == 0) {
+  else if (aPrefix.IsEmpty()) {
     aPrefix.Assign(NS_LITERAL_STRING("a"));
     char buf[128];
     PR_snprintf(buf, sizeof(buf), "%d", mPrefixIndex++);
@@ -379,14 +384,14 @@ nsXMLContentSerializer::ConfirmPrefix(nsAWritableString& aPrefix,
 }
 
 void
-nsXMLContentSerializer::SerializeAttr(const nsAReadableString& aPrefix,
-                                      const nsAReadableString& aName,
-                                      const nsAReadableString& aValue,
-                                      nsAWritableString& aStr,
+nsXMLContentSerializer::SerializeAttr(const nsAString& aPrefix,
+                                      const nsAString& aName,
+                                      const nsAString& aValue,
+                                      nsAString& aStr,
                                       PRBool aDoEscapeEntities)
 {
   AppendToString(PRUnichar(' '), aStr);
-  if (aPrefix.Length() > 0) {
+  if (!aPrefix.IsEmpty()) {
     AppendToString(aPrefix, aStr);
     AppendToString(NS_LITERAL_STRING(":"), aStr);
   }
@@ -403,7 +408,8 @@ nsXMLContentSerializer::SerializeAttr(const nsAReadableString& aPrefix,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
-                                           nsAWritableString& aStr)
+                                           PRBool aHasChildren,
+                                           nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -418,10 +424,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   aElement->GetLocalName(tagLocalName);
   aElement->GetNamespaceURI(tagNamespaceURI);
 
-  PRInt32 namespaceID, elementNamespaceID;
-  content->GetNameSpaceID(elementNamespaceID);
-  if (elementNamespaceID == kNameSpaceID_HTML)
-    ToLowerCase(tagLocalName); // XXX We shouldn't need this hack
+  PRInt32 namespaceID;
     
   PRInt32 index, count;
   nsAutoString nameStr, prefixStr, uriStr, valueStr;
@@ -437,26 +440,15 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
                            *getter_AddRefs(attrName),
                            *getter_AddRefs(attrPrefix));
     
-    if (namespaceID == kNameSpaceID_XMLNS ||
-      elementNamespaceID == kNameSpaceID_HTML /*XXX Hack*/) {
+    if (namespaceID == kNameSpaceID_XMLNS) {
       PRBool hasPrefix = attrPrefix ? PR_TRUE : PR_FALSE;
       content->GetAttr(namespaceID, attrName, uriStr);
 
-      attrName->ToString(nameStr);
-      // XXX We shouldn't need this hack
-      if (elementNamespaceID == kNameSpaceID_HTML) {
-        if (Substring(nameStr, 0, 6).Equals(NS_LITERAL_STRING("xmlns:"))) {
-          nameStr.Cut(0,6);
-          hasPrefix = PR_TRUE;
-        } else if (!nameStr.Equals(kXMLNS)) {
-          continue;
-        }
-      }
-
-      if (!hasPrefix) {
+      if (!attrPrefix) {
         // Default NS attribute does not have prefix (and the name is "xmlns")
         PushNameSpaceDecl(nsString(), uriStr, aElement);
       } else {
+        attrName->ToString(nameStr);
         PushNameSpaceDecl(nameStr, uriStr, aElement);
       }
     }
@@ -467,8 +459,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   addNSAttr = ConfirmPrefix(tagPrefix, tagNamespaceURI);
   // Serialize the qualified name of the element
   AppendToString(NS_LITERAL_STRING("<"), aStr);
-  if (tagPrefix.Length() > 0 &&
-    !(elementNamespaceID == kNameSpaceID_HTML && tagPrefix.Equals(kXMLNS) /*XXX Hack*/)) {
+  if (!tagPrefix.IsEmpty()) {
     AppendToString(tagPrefix, aStr);
     AppendToString(NS_LITERAL_STRING(":"), aStr);
   }
@@ -496,6 +487,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
                            namespaceID,
                            *getter_AddRefs(attrName),
                            *getter_AddRefs(attrPrefix));
+
     if (attrPrefix) {
       attrPrefix->ToString(prefixStr);
     }
@@ -511,9 +503,25 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
     
     content->GetAttr(namespaceID, attrName, valueStr);
     attrName->ToString(nameStr);
-    
-    if (elementNamespaceID == kNameSpaceID_HTML && nameStr.Equals(NS_LITERAL_STRING("xmlns:xmlns")))
-      nameStr.Assign(kXMLNS); // XXX Shouldn't need this hack, breaks case where there really is xmlns:xmlns
+
+    // XXX Hack to get around the fact that MathML can add
+    //     attributes starting with '-', which makes them
+    //     invalid XML.
+    if (!nameStr.IsEmpty() && nameStr.First() == '-')
+      continue;
+
+    if (namespaceID == kNameSpaceID_None) {
+      PRInt32 elementNsID;
+      content->GetNameSpaceID(elementNsID);
+      if (elementNsID == kNameSpaceID_XHTML) {
+        nsCOMPtr<nsIAtom> elementName;
+        content->GetTag(*getter_AddRefs(elementName));
+        if (IsShorthandAttr(attrName, elementName) &&
+            valueStr.IsEmpty()) {
+          valueStr = nameStr;
+        }
+      }
+    }
     SerializeAttr(prefixStr, nameStr, valueStr, aStr, PR_TRUE);
     
     if (addNSAttr) {
@@ -523,9 +531,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
   }
 
   // We don't output a separate end tag for empty element
-  nsCOMPtr<nsIDOMNode> node(do_QueryInterface(aElement));
-  PRBool hasChildren;
-  if (NS_SUCCEEDED(node->HasChildNodes(&hasChildren)) && !hasChildren) {
+  if (!aHasChildren) {
     AppendToString(NS_LITERAL_STRING("/>"), aStr);    
   } else {
     AppendToString(NS_LITERAL_STRING(">"), aStr);    
@@ -536,7 +542,7 @@ nsXMLContentSerializer::AppendElementStart(nsIDOMElement *aElement,
 
 NS_IMETHODIMP 
 nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
-                                         nsAWritableString& aStr)
+                                         nsAString& aStr)
 {
   NS_ENSURE_ARG(aElement);
 
@@ -556,16 +562,10 @@ nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
   aElement->GetLocalName(tagLocalName);
   aElement->GetNamespaceURI(tagNamespaceURI);
 
-  PRInt32 namespaceID;
-  content->GetNameSpaceID(namespaceID);
-  if (namespaceID == kNameSpaceID_HTML)
-    ToLowerCase(tagLocalName); // XXX We shouldn't need this hack
-
   ConfirmPrefix(tagPrefix, tagNamespaceURI);
 
   AppendToString(NS_LITERAL_STRING("</"), aStr);
-  if (tagPrefix.Length() > 0 &&
-    !(namespaceID == kNameSpaceID_HTML && tagPrefix.Equals(kXMLNS) /*XXX Hack*/)) {
+  if (!tagPrefix.IsEmpty()) {
     AppendToString(tagPrefix, aStr);
     AppendToString(NS_LITERAL_STRING(":"), aStr);
   }
@@ -580,7 +580,7 @@ nsXMLContentSerializer::AppendElementEnd(nsIDOMElement *aElement,
 void
 nsXMLContentSerializer::AppendToString(const PRUnichar* aStr,
                                        PRInt32 aLength,
-                                       nsAWritableString& aOutputStr)
+                                       nsAString& aOutputStr)
 {
   PRInt32 length = (aLength == -1) ? nsCRT::strlen(aStr) : aLength;
   
@@ -589,7 +589,7 @@ nsXMLContentSerializer::AppendToString(const PRUnichar* aStr,
 
 void 
 nsXMLContentSerializer::AppendToString(const PRUnichar aChar,
-                                       nsAWritableString& aOutputStr)
+                                       nsAString& aOutputStr)
 {
   aOutputStr.Append(aChar);
 }
@@ -609,15 +609,15 @@ static const char* kAttrEntities[] = {
   "", "", "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "", "", "",
-  "", "", "", "", "&quot;", "", "", "", "&amp;", "&apos;",
+  "", "", "", "", "&quot;", "", "", "", "&amp;", "",
   "", "", "", "", "", "", "", "", "", "",
   "", "", "", "", "", "", "", "", "", "",
   "&lt;", "", "&gt;"
 };
 
 void
-nsXMLContentSerializer::AppendToString(const nsAReadableString& aStr,
-                                       nsAWritableString& aOutputStr,
+nsXMLContentSerializer::AppendToString(const nsAString& aStr,
+                                       nsAString& aOutputStr,
                                        PRBool aTranslateEntities,
                                        PRBool aIncrColumn)
 {
@@ -663,3 +663,94 @@ nsXMLContentSerializer::AppendToString(const nsAReadableString& aStr,
   
   aOutputStr.Append(aStr);
 }
+
+PRBool
+nsXMLContentSerializer::IsShorthandAttr(const nsIAtom* aAttrName,
+                                        const nsIAtom* aElementName)
+{
+  // checked
+  if ((aAttrName == nsHTMLAtoms::checked) &&
+      (aElementName == nsHTMLAtoms::input)) {
+    return PR_TRUE;
+  }
+
+  // compact
+  if ((aAttrName == nsHTMLAtoms::compact) &&
+      (aElementName == nsHTMLAtoms::dir || 
+       aElementName == nsHTMLAtoms::dl ||
+       aElementName == nsHTMLAtoms::menu ||
+       aElementName == nsHTMLAtoms::ol ||
+       aElementName == nsHTMLAtoms::ul)) {
+    return PR_TRUE;
+  }
+
+  // declare
+  if ((aAttrName == nsHTMLAtoms::declare) &&
+      (aElementName == nsHTMLAtoms::object)) {
+    return PR_TRUE;
+  }
+
+  // defer
+  if ((aAttrName == nsHTMLAtoms::defer) &&
+      (aElementName == nsHTMLAtoms::script)) {
+    return PR_TRUE;
+  }
+
+  // disabled
+  if ((aAttrName == nsHTMLAtoms::disabled) &&
+      (aElementName == nsHTMLAtoms::button ||
+       aElementName == nsHTMLAtoms::input ||
+       aElementName == nsHTMLAtoms::optgroup ||
+       aElementName == nsHTMLAtoms::option ||
+       aElementName == nsHTMLAtoms::select ||
+       aElementName == nsHTMLAtoms::textarea)) {
+    return PR_TRUE;
+  }
+
+  // ismap
+  if ((aAttrName == nsHTMLAtoms::ismap) &&
+      (aElementName == nsHTMLAtoms::img ||
+       aElementName == nsHTMLAtoms::input)) {
+    return PR_TRUE;
+  }
+
+  // multiple
+  if ((aAttrName == nsHTMLAtoms::multiple) &&
+      (aElementName == nsHTMLAtoms::select)) {
+    return PR_TRUE;
+  }
+
+  // noresize
+  if ((aAttrName == nsHTMLAtoms::noresize) &&
+      (aElementName == nsHTMLAtoms::frame)) {
+    return PR_TRUE;
+  }
+
+  // noshade
+  if ((aAttrName == nsHTMLAtoms::noshade) &&
+      (aElementName == nsHTMLAtoms::hr)) {
+    return PR_TRUE;
+  }
+
+  // nowrap
+  if ((aAttrName == nsHTMLAtoms::nowrap) &&
+      (aElementName == nsHTMLAtoms::td ||
+       aElementName == nsHTMLAtoms::th)) {
+    return PR_TRUE;
+  }
+
+  // readonly
+  if ((aAttrName == nsHTMLAtoms::readonly) &&
+      (aElementName == nsHTMLAtoms::input ||
+       aElementName == nsHTMLAtoms::textarea)) {
+    return PR_TRUE;
+  }
+
+  // selected
+  if ((aAttrName == nsHTMLAtoms::selected) &&
+      (aElementName == nsHTMLAtoms::option)) {
+    return PR_TRUE;
+  }
+
+  return PR_FALSE;
+}
