@@ -180,6 +180,11 @@ nsresult nsEventListenerManager::RemoveAllListeners(PRBool aScriptOnly)
   return NS_OK;
 }
 
+void nsEventListenerManager::Shutdown()
+{
+    sAddListenerID = JSVAL_VOID;
+}
+
 NS_IMPL_ADDREF(nsEventListenerManager)
 NS_IMPL_RELEASE(nsEventListenerManager)
 
@@ -754,7 +759,7 @@ nsresult nsEventListenerManager::GetIdentifiersForType(nsIAtom* aType, EventArra
 
 NS_IMETHODIMP
 nsEventListenerManager::AddEventListenerByType(nsIDOMEventListener *aListener, 
-                                               const nsAReadableString& aType,
+                                               const nsAString& aType,
                                                PRInt32 aFlags)
 {
   PRInt32 subType;
@@ -766,7 +771,8 @@ nsEventListenerManager::AddEventListenerByType(nsIDOMEventListener *aListener,
     AddEventListener(aListener, arrayType, subType, nsnull, aFlags);
   }
   else {
-    nsStringKey key(aType);
+    const nsPromiseFlatString& flatString = PromiseFlatString(aType); 
+    nsStringKey key(flatString);
     AddEventListener(aListener, eEventArrayType_Hash, NS_EVENT_BITS_NONE, &key, aFlags);
   }
 
@@ -775,7 +781,7 @@ nsEventListenerManager::AddEventListenerByType(nsIDOMEventListener *aListener,
 
 NS_IMETHODIMP
 nsEventListenerManager::RemoveEventListenerByType(nsIDOMEventListener *aListener, 
-                                                  const nsAReadableString& aType,
+                                                  const nsAString& aType,
                                                   PRInt32 aFlags)
 {
   PRInt32 subType;
@@ -787,7 +793,8 @@ nsEventListenerManager::RemoveEventListenerByType(nsIDOMEventListener *aListener
     RemoveEventListener(aListener, arrayType, subType, nsnull, aFlags);
   }
   else {
-    nsStringKey key(aType);
+    const nsPromiseFlatString& flatString = PromiseFlatString(aType); 
+    nsStringKey key(flatString);
     RemoveEventListener(aListener, eEventArrayType_Hash, NS_EVENT_BITS_NONE, &key, aFlags);
   }
 
@@ -866,7 +873,7 @@ NS_IMETHODIMP
 nsEventListenerManager::AddScriptEventListener(nsIScriptContext* aContext,
                                                nsISupports *aObject,
                                                nsIAtom *aName,
-                                               const nsAReadableString& aBody,
+                                               const nsAString& aBody,
                                                PRBool aDeferCompilation)
 {
   nsresult rv;
@@ -951,6 +958,9 @@ nsEventListenerManager::RemoveScriptEventListener(nsIAtom *aName)
   return result;
 }
 
+jsval
+nsEventListenerManager::sAddListenerID = JSVAL_VOID;
+
 NS_IMETHODIMP
 nsEventListenerManager::RegisterScriptEventListener(nsIScriptContext *aContext,
                                                     nsISupports *aObject, 
@@ -992,8 +1002,12 @@ nsEventListenerManager::RegisterScriptEventListener(nsIScriptContext *aContext,
 
   nsCOMPtr<nsIClassInfo> classInfo = do_QueryInterface(aObject);
 
+  if (sAddListenerID == JSVAL_VOID) {
+    sAddListenerID = STRING_TO_JSVAL(::JS_InternString(cx, "addEventListener"));
+  }
+
   if (NS_FAILED(rv = securityManager->CheckPropertyAccess(cx, jsobj,
-                "EventTarget","addEventListener",
+                "EventTarget", sAddListenerID,
                 nsIXPCSecurityManager::ACCESS_SET_PROPERTY))) {
       // XXX set pending exception on the native call context?
     return rv;
@@ -1177,11 +1191,10 @@ nsEventListenerManager::HandleEventSubType(nsListenerStruct* aListenerStruct,
     sgo = do_QueryInterface(aCurrentTarget);
   }
 
+  nsCOMPtr<nsIScriptContext> scx;
   JSContext *cx = nsnull;
 
   if (sgo) {
-    nsCOMPtr<nsIScriptContext> scx;
-
     sgo->GetContext(getter_AddRefs(scx));
 
     if (scx) {
@@ -1190,11 +1203,14 @@ nsEventListenerManager::HandleEventSubType(nsListenerStruct* aListenerStruct,
   }
 
   nsCOMPtr<nsIJSContextStack> stack;
+  JSContext *current_cx = nsnull;
 
   if (cx) {
     stack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
 
     if (stack) {
+      stack->Peek(&current_cx);
+
       stack->Push(cx);
     }
   }
@@ -1208,6 +1224,13 @@ nsEventListenerManager::HandleEventSubType(nsListenerStruct* aListenerStruct,
 
   if (cx && stack) {
     stack->Pop(&cx);
+
+    if (!current_cx) {
+      // No JS is running, but executing the event handler might have
+      // caused some JS to run. Tell the script context that it's done.
+
+      scx->ScriptEvaluated(PR_TRUE);
+    }
   }
 
   return result;
@@ -2326,7 +2349,7 @@ nsresult nsEventListenerManager::HandleEvent(nsIPresContext* aPresContext,
 NS_IMETHODIMP
 nsEventListenerManager::CreateEvent(nsIPresContext* aPresContext,
                                     nsEvent* aEvent,
-                                    const nsAReadableString& aEventType,
+                                    const nsAString& aEventType,
                                     nsIDOMEvent** aDOMEvent)
 {
   *aDOMEvent = nsnull;
@@ -2600,7 +2623,7 @@ nsEventListenerManager::SetListenerTarget(nsISupports* aTarget)
 
 // nsIDOMEventTarget interface
 NS_IMETHODIMP 
-nsEventListenerManager::AddEventListener(const nsAReadableString& aType, 
+nsEventListenerManager::AddEventListener(const nsAString& aType, 
                                          nsIDOMEventListener* aListener, 
                                          PRBool aUseCapture)
 {
@@ -2610,7 +2633,7 @@ nsEventListenerManager::AddEventListener(const nsAReadableString& aType,
 }
 
 NS_IMETHODIMP 
-nsEventListenerManager::RemoveEventListener(const nsAReadableString& aType, 
+nsEventListenerManager::RemoveEventListener(const nsAString& aType, 
                                             nsIDOMEventListener* aListener, 
                                             PRBool aUseCapture)
 {
