@@ -45,7 +45,7 @@
 #include "nsIContent.h"
 #include "nsHTMLValue.h"
 #include "nsVoidArray.h"
-#include "nsINameSpaceManager.h"  // for kNameSpaceID_HTML
+#include "nsINameSpaceManager.h"  // for kNameSpaceID_None
 #include "nsIFormControl.h"
 
 #include "nsIStatefulFrame.h"
@@ -53,9 +53,8 @@
 class nsIDOMAttr;
 class nsIDOMEventListener;
 class nsIDOMNodeList;
-class nsIEventListenerManager;
 class nsIFrame;
-class nsIHTMLAttributes;
+class nsHTMLAttributes;
 class nsIHTMLMappedAttributes;
 class nsIHTMLContent;
 class nsIStyleRule;
@@ -68,6 +67,7 @@ class nsIFormControlFrame;
 class nsIForm;
 class nsIPresState;
 class nsIScrollableView;
+class nsILayoutHistoryState;
 struct nsRect;
 
 
@@ -93,43 +93,43 @@ public:
                         PRBool aDeep);
 
   // Implementation for nsIDOMNode
-  NS_METHOD GetNodeName(nsAWritableString& aNodeName);
-  NS_METHOD GetLocalName(nsAWritableString& aLocalName);
+  NS_METHOD GetNodeName(nsAString& aNodeName);
+  NS_METHOD GetLocalName(nsAString& aLocalName);
 
   // Implementation for nsIDOMElement
-  NS_METHOD GetAttribute(const nsAReadableString& aName,
-                         nsAWritableString& aReturn) 
+  NS_METHOD GetAttribute(const nsAString& aName,
+                         nsAString& aReturn) 
   {
     return nsGenericElement::GetAttribute(aName, aReturn);
   }
-  NS_METHOD SetAttribute(const nsAReadableString& aName,
-                         const nsAReadableString& aValue)
+  NS_METHOD SetAttribute(const nsAString& aName,
+                         const nsAString& aValue)
   {
     return nsGenericElement::SetAttribute(aName, aValue);
   }
-  NS_METHOD GetTagName(nsAWritableString& aTagName);
-  NS_METHOD GetElementsByTagName(const nsAReadableString& aTagname,
+  NS_METHOD GetTagName(nsAString& aTagName);
+  NS_METHOD GetElementsByTagName(const nsAString& aTagname,
                                  nsIDOMNodeList** aReturn);
 
   // Implementation for nsIDOMHTMLElement
-  nsresult GetId(nsAWritableString& aId);
-  nsresult SetId(const nsAReadableString& aId);
-  nsresult GetTitle(nsAWritableString& aTitle);
-  nsresult SetTitle(const nsAReadableString& aTitle);
-  nsresult GetLang(nsAWritableString& aLang);
-  nsresult SetLang(const nsAReadableString& aLang);
-  nsresult GetDir(nsAWritableString& aDir);
-  nsresult SetDir(const nsAReadableString& aDir);
-  nsresult GetClassName(nsAWritableString& aClassName);
-  nsresult SetClassName(const nsAReadableString& aClassName);
+  nsresult GetId(nsAString& aId);
+  nsresult SetId(const nsAString& aId);
+  nsresult GetTitle(nsAString& aTitle);
+  nsresult SetTitle(const nsAString& aTitle);
+  nsresult GetLang(nsAString& aLang);
+  nsresult SetLang(const nsAString& aLang);
+  nsresult GetDir(nsAString& aDir);
+  nsresult SetDir(const nsAString& aDir);
+  nsresult GetClassName(nsAString& aClassName);
+  nsresult SetClassName(const nsAString& aClassName);
   nsresult GetStyle(nsIDOMCSSStyleDeclaration** aStyle);
   nsresult GetOffsetTop(PRInt32* aOffsetTop);
   nsresult GetOffsetLeft(PRInt32* aOffsetLeft);
   nsresult GetOffsetWidth(PRInt32* aOffsetWidth);
   nsresult GetOffsetHeight(PRInt32* aOffsetHeight);
   nsresult GetOffsetParent(nsIDOMElement** aOffsetParent);
-  nsresult GetInnerHTML(nsAWritableString& aInnerHTML);
-  nsresult SetInnerHTML(const nsAReadableString& aInnerHTML);
+  nsresult GetInnerHTML(nsAString& aInnerHTML);
+  nsresult SetInnerHTML(const nsAString& aInnerHTML);
   nsresult GetScrollTop(PRInt32* aScrollTop);
   nsresult SetScrollTop(PRInt32 aScrollTop);
   nsresult GetScrollLeft(PRInt32* aScrollLeft);
@@ -149,18 +149,18 @@ public:
   NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
                          PRBool aCompileEventHandlers);
   NS_IMETHOD GetNameSpaceID(PRInt32& aID) const;
-  NS_IMETHOD NormalizeAttrString(const nsAReadableString& aStr,
+  NS_IMETHOD NormalizeAttrString(const nsAString& aStr,
                                  nsINodeInfo*& aNodeInfo);
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAReadableString& aValue,
+                     const nsAString& aValue,
                           PRBool aNotify);
   NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-                     const nsAReadableString& aValue,
+                     const nsAString& aValue,
                      PRBool aNotify);
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     nsAWritableString& aResult) const;
+                     nsAString& aResult) const;
   NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     nsIAtom*& aPrefix, nsAWritableString& aResult) const;
+                     nsIAtom*& aPrefix, nsAString& aResult) const;
   NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
   NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
                        PRBool aNotify);
@@ -192,12 +192,12 @@ public:
   NS_IMETHOD WalkContentStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD WalkInlineStyleRules(nsRuleWalker* aRuleWalker);
   NS_IMETHOD GetBaseURL(nsIURI*& aBaseURL) const;
-  NS_IMETHOD GetBaseTarget(nsAWritableString& aBaseTarget) const;
+  NS_IMETHOD GetBaseTarget(nsAString& aBaseTarget) const;
 
   //----------------------------------------
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       PRInt32& aHint) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
@@ -208,15 +208,37 @@ public:
   PRUint32 BaseSizeOf(nsISizeOfHandler* aSizer) const;
 #endif
 
-  nsIFormControlFrame *GetFormControlFrame(PRBool aFlushContent)
+  /**
+   * Get the primary form control frame for this content (see
+   * GetFormControlFrameFor)
+   *
+   * @param aFlushContent whether to flush the content sink
+   * @return the primary form control frame
+   */
+  nsIFormControlFrame* GetFormControlFrame(PRBool aFlushContent)
   {
-    if (!mDocument || !mParent) {
+    if (!mDocument) {
       return nsnull;
     }
 
     return GetFormControlFrameFor(this, mDocument, aFlushContent);
   }
 
+  /**
+   * Get the primary frame for this content (see GetPrimaryFrameFor)
+   *
+   * @param aFlushContent whether to flush the content sink
+   * @return the primary frame
+   */
+  nsIFrame* GetPrimaryFrame(PRBool aFlushContent)
+  {
+    if (!mDocument) {
+      return nsnull;
+    }
+
+    return GetPrimaryFrameFor(this, mDocument, aFlushContent);
+  }
+
   //----------------------------------------
 
   // Attribute parsing utilities
@@ -226,100 +248,100 @@ public:
     PRInt32 value;
   };
 
-  static PRBool ParseEnumValue(const nsAReadableString& aValue,
+  static PRBool ParseEnumValue(const nsAString& aValue,
                                EnumTable* aTable,
                                nsHTMLValue& aResult);
 
-  static PRBool ParseCaseSensitiveEnumValue(const nsAReadableString& aValue,
+  static PRBool ParseCaseSensitiveEnumValue(const nsAString& aValue,
                                             EnumTable* aTable,
                                             nsHTMLValue& aResult);
 
   static PRBool EnumValueToString(const nsHTMLValue& aValue,
                                   EnumTable* aTable,
-                                  nsAWritableString& aResult);
+                                  nsAString& aResult);
 
-  static PRBool ParseValueOrPercent(const nsAReadableString& aString,
+  static PRBool ParseValueOrPercent(const nsAString& aString,
                                     nsHTMLValue& aResult,
                                     nsHTMLUnit aValueUnit);
 
-  static PRBool ParseValueOrPercentOrProportional(const nsAReadableString& aString,
+  static PRBool ParseValueOrPercentOrProportional(const nsAString& aString,
                                                   nsHTMLValue& aResult, 
                                                   nsHTMLUnit aValueUnit);
 
   static PRBool ValueOrPercentToString(const nsHTMLValue& aValue,
-                                       nsAWritableString& aResult);
+                                       nsAString& aResult);
 
   static PRBool ValueOrPercentOrProportionalToString(const nsHTMLValue& aValue,
-                                                     nsAWritableString& aResult);
+                                                     nsAString& aResult);
 
-  static PRBool ParseValue(const nsAReadableString& aString, PRInt32 aMin,
+  static PRBool ParseValue(const nsAString& aString, PRInt32 aMin,
                            nsHTMLValue& aResult, nsHTMLUnit aValueUnit);
 
-  static PRBool ParseValue(const nsAReadableString& aString, PRInt32 aMin,
+  static PRBool ParseValue(const nsAString& aString, PRInt32 aMin,
                            PRInt32 aMax, nsHTMLValue& aResult,
                            nsHTMLUnit aValueUnit);
 
-  static PRBool ParseColor(const nsAReadableString& aString,
+  static PRBool ParseColor(const nsAString& aString,
                            nsIDocument* aDocument, nsHTMLValue& aResult);
 
   static PRBool ColorToString(const nsHTMLValue& aValue,
-                              nsAWritableString& aResult);
+                              nsAString& aResult);
 
   static PRBool ParseCommonAttribute(nsIAtom* aAttribute, 
-                                     const nsAReadableString& aValue, 
+                                     const nsAString& aValue, 
                                      nsHTMLValue& aResult);
-  static PRBool ParseAlignValue(const nsAReadableString& aString,
+  static PRBool ParseAlignValue(const nsAString& aString,
                                 nsHTMLValue& aResult);
 
-  PRBool ParseDivAlignValue(const nsAReadableString& aString,
+  PRBool ParseDivAlignValue(const nsAString& aString,
                             nsHTMLValue& aResult) const;
   PRBool DivAlignValueToString(const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
 
-  PRBool ParseTableHAlignValue(const nsAReadableString& aString,
+  PRBool ParseTableHAlignValue(const nsAString& aString,
                                nsHTMLValue& aResult) const;
   PRBool TableHAlignValueToString(const nsHTMLValue& aValue,
-                                  nsAWritableString& aResult) const;
+                                  nsAString& aResult) const;
 
-  PRBool ParseTableCellHAlignValue(const nsAReadableString& aString,
+  PRBool ParseTableCellHAlignValue(const nsAString& aString,
                                    nsHTMLValue& aResult) const;
   PRBool TableCellHAlignValueToString(const nsHTMLValue& aValue,
-                                      nsAWritableString& aResult) const;
+                                      nsAString& aResult) const;
 
-  static PRBool ParseTableVAlignValue(const nsAReadableString& aString,
+  static PRBool ParseTableVAlignValue(const nsAString& aString,
                                       nsHTMLValue& aResult);
 
   static PRBool TableVAlignValueToString(const nsHTMLValue& aValue,
-                                         nsAWritableString& aResult);
+                                         nsAString& aResult);
 
   static PRBool AlignValueToString(const nsHTMLValue& aValue,
-                                   nsAWritableString& aResult);
+                                   nsAString& aResult);
 
   static PRBool VAlignValueToString(const nsHTMLValue& aValue,
-                                    nsAWritableString& aResult);
+                                    nsAString& aResult);
 
   static PRBool ParseImageAttribute(nsIAtom* aAttribute,
-                                    const nsAReadableString& aString,
+                                    const nsAString& aString,
                                     nsHTMLValue& aResult);
 
   static PRBool ImageAttributeToString(nsIAtom* aAttribute,
                                        const nsHTMLValue& aValue,
-                                       nsAWritableString& aResult);
+                                       nsAString& aResult);
 
-  static PRBool ParseFrameborderValue(const nsAReadableString& aString,
+  static PRBool ParseFrameborderValue(const nsAString& aString,
                                       nsHTMLValue& aResult);
 
   static PRBool FrameborderValueToString(const nsHTMLValue& aValue,
-                                         nsAWritableString& aResult);
+                                         nsAString& aResult);
 
-  static PRBool ParseScrollingValue(const nsAReadableString& aString,
+  static PRBool ParseScrollingValue(const nsAString& aString,
                                     nsHTMLValue& aResult);
 
   static PRBool ScrollingValueToString(const nsHTMLValue& aValue,
-                                       nsAWritableString& aResult);
+                                       nsAString& aResult);
 
   nsresult  ReparseStyleAttribute(void);
-  nsresult  ParseStyleAttribute(const nsAReadableString& aValue,
+  nsresult  ParseStyleAttribute(const nsAString& aValue,
                                 nsHTMLValue& aResult);
 
   /** Attribute Mapping Helpers
@@ -354,11 +376,64 @@ public:
   static PRBool GetBackgroundAttributesImpact(const nsIAtom* aAttribute,
                                               PRInt32& aHint);
 
+  /**
+   * Get the primary frame for a piece of content.
+   *
+   * @param aContent the content to get the primary frame for
+   * @param aDocument the document for this content
+   * @param aFlushContent whether to flush the content sink, which creates
+   *        frames for content that do not already have it.  EXPENSIVE.
+   * @return the primary frame
+   */
+  static nsIFrame* GetPrimaryFrameFor(nsIContent* aContent,
+                                      nsIDocument* aDocument,
+                                      PRBool aFlushContent);
+
+  /**
+   * Get the primary form control frame for a piece of content.  Same as
+   * GetPrimaryFrameFor, except it QI's to nsIFormControlFrame.
+   *
+   * @param aContent the content to get the primary frame for
+   * @param aDocument the document for this content
+   * @param aFlushContent whether to flush the content sink, which creates
+   *        frames for content that do not already have it.  EXPENSIVE.
+   * @return the primary frame as nsIFormControlFrame
+   */
   static nsIFormControlFrame* GetFormControlFrameFor(nsIContent* aContent,
                                                      nsIDocument* aDocument,
                                                      PRBool aFlushContent);
+  /**
+   * Get the presentation state for a piece of content, or create it if it does
+   * not exist.  Generally used by SaveState().
+   *
+   * @param aContent the content to get presentation state for.
+   * @param aPresState the presentation state (out param)
+   */
   static nsresult GetPrimaryPresState(nsIHTMLContent* aContent,
                                       nsIPresState** aPresState);
+  /**
+   * Get the layout history object *and* generate the key for a particular
+   * piece of content.
+   *
+   * @param aContent the content to generate the key for
+   * @param aState the history state object (out param)
+   * @param aKey the key (out param)
+   */
+  static nsresult GetLayoutHistoryAndKey(nsIHTMLContent* aContent,
+                                         nsILayoutHistoryState** aState,
+                                         nsACString& aKey);
+  /**
+   * Restore the state for a form control.  Ends up calling
+   * nsIFormControl::RestoreState().
+   *
+   * @param aContent an nsIHTMLContent* pointing to the form control
+   * @param aControl an nsIFormControl* pointing to the form control
+   * @return whether or not the RestoreState() was called and exited
+   *         successfully.
+   */
+  static PRBool RestoreFormControlState(nsIHTMLContent* aContent,
+                                        nsIFormControl* aControl);
+
   static nsresult GetPresContext(nsIHTMLContent* aContent,
                                  nsIPresContext** aPresContext);
 
@@ -377,65 +452,64 @@ public:
 
   nsresult SetFormControlAttribute(nsIForm* aForm, PRInt32 aNameSpaceID,
                                    nsIAtom* aName,
-                                   const nsAReadableString& aValue,
+                                   const nsAString& aValue,
                                    PRBool aNotify);
 
-  nsIHTMLAttributes* mAttributes;
+  nsHTMLAttributes* mAttributes;
 
   // Helper functions for <a> and <area>
-  static nsresult SetProtocolInHrefString(const nsAReadableString &aHref,
-                                          const nsAReadableString &aProtocol,
-                                          nsAWritableString &aResult);
+  static nsresult SetProtocolInHrefString(const nsAString &aHref,
+                                          const nsAString &aProtocol,
+                                          nsAString &aResult);
 
-  static nsresult SetHostInHrefString(const nsAReadableString &aHref,
-                                      const nsAReadableString &aHost,
-                                      nsAWritableString &aResult);
+  static nsresult SetHostInHrefString(const nsAString &aHref,
+                                      const nsAString &aHost,
+                                      nsAString &aResult);
 
-  static nsresult SetHostnameInHrefString(const nsAReadableString &aHref,
-                                          const nsAReadableString &aHostname,
-                                          nsAWritableString &aResult);
+  static nsresult SetHostnameInHrefString(const nsAString &aHref,
+                                          const nsAString &aHostname,
+                                          nsAString &aResult);
 
-  static nsresult SetPathnameInHrefString(const nsAReadableString &aHref,
-                                          const nsAReadableString &aHostname,
-                                          nsAWritableString &aResult);
+  static nsresult SetPathnameInHrefString(const nsAString &aHref,
+                                          const nsAString &aHostname,
+                                          nsAString &aResult);
 
-  static nsresult SetSearchInHrefString(const nsAReadableString &aHref,
-                                        const nsAReadableString &aSearch,
-                                        nsAWritableString &aResult);
+  static nsresult SetSearchInHrefString(const nsAString &aHref,
+                                        const nsAString &aSearch,
+                                        nsAString &aResult);
   
-  static nsresult SetHashInHrefString(const nsAReadableString &aHref,
-                                      const nsAReadableString &aHash,
-                                      nsAWritableString &aResult);
+  static nsresult SetHashInHrefString(const nsAString &aHref,
+                                      const nsAString &aHash,
+                                      nsAString &aResult);
 
-  static nsresult SetPortInHrefString(const nsAReadableString &aHref,
-                                      const nsAReadableString &aPort,
-                                      nsAWritableString &aResult);
+  static nsresult SetPortInHrefString(const nsAString &aHref,
+                                      const nsAString &aPort,
+                                      nsAString &aResult);
 
-  static nsresult GetProtocolFromHrefString(const nsAReadableString &aHref,
-                                            nsAWritableString& aProtocol,
+  static nsresult GetProtocolFromHrefString(const nsAString &aHref,
+                                            nsAString& aProtocol,
                                             nsIDocument *aDocument);
 
-  static nsresult GetHostFromHrefString(const nsAReadableString &aHref,
-                                        nsAWritableString& aHost);
+  static nsresult GetHostFromHrefString(const nsAString &aHref,
+                                        nsAString& aHost);
 
-  static nsresult GetHostnameFromHrefString(const nsAReadableString &aHref,
-                                            nsAWritableString& aHostname);
+  static nsresult GetHostnameFromHrefString(const nsAString &aHref,
+                                            nsAString& aHostname);
 
-  static nsresult GetPathnameFromHrefString(const nsAReadableString &aHref,
-                                            nsAWritableString& aPathname);
+  static nsresult GetPathnameFromHrefString(const nsAString &aHref,
+                                            nsAString& aPathname);
 
-  static nsresult GetSearchFromHrefString(const nsAReadableString &aHref,
-                                          nsAWritableString& aSearch);
+  static nsresult GetSearchFromHrefString(const nsAString &aHref,
+                                          nsAString& aSearch);
 
-  static nsresult GetPortFromHrefString(const nsAReadableString &aHref,
-                                        nsAWritableString& aPort);
+  static nsresult GetPortFromHrefString(const nsAString &aHref,
+                                        nsAString& aPort);
 
-  static nsresult GetHashFromHrefString(const nsAReadableString &aHref,
-                                        nsAWritableString& aHash);
+  static nsresult GetHashFromHrefString(const nsAString &aHref,
+                                        nsAString& aHash);
 protected:
   nsresult SetElementFocus(PRBool aDoFocus);
-
-  nsresult HandleFrameOnloadEvent(nsIDOMEvent* aEvent);
+  nsresult RegUnRegAccessKey(PRBool aDoReg);
 
   PRBool IsEventName(nsIAtom* aName);
 };
@@ -589,15 +663,8 @@ public:
   NS_IMETHOD GetForm(nsIDOMHTMLFormElement** aForm);
   NS_IMETHOD SetForm(nsIDOMHTMLFormElement* aForm,
                      PRBool aRemoveFromForm = PR_TRUE);
-  NS_IMETHOD Init();
-  NS_IMETHOD SaveState(nsIPresContext* aPresContext, nsIPresState** aState)
-  {
-    return NS_OK;
-  }
-  NS_IMETHOD RestoreState(nsIPresContext* aPresContext, nsIPresState* aState)
-  {
-    return NS_OK;
-  }
+  NS_IMETHOD SaveState() { return NS_OK; }
+  NS_IMETHOD RestoreState(nsIPresState* aState) { return NS_OK; }
 
   // nsIContent
   NS_IMETHOD SetParent(nsIContent *aParent);
@@ -605,15 +672,15 @@ public:
                          PRBool aCompileEventHandlers);
 
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAReadableString& aValue, PRBool aNotify);
+                     const nsAString& aValue, PRBool aNotify);
 
   NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-                     const nsAReadableString& aValue,
+                     const nsAString& aValue,
                      PRBool aNotify);
 
 
-  NS_METHOD SetAttribute(const nsAReadableString& aName,
-                         const nsAReadableString& aValue)
+  NS_METHOD SetAttribute(const nsAString& aName,
+                         const nsAString& aValue)
   {
     return nsGenericHTMLElement::SetAttribute(aName, aValue);
   }
@@ -639,15 +706,8 @@ public:
   NS_IMETHOD GetForm(nsIDOMHTMLFormElement** aForm);
   NS_IMETHOD SetForm(nsIDOMHTMLFormElement* aForm,
                      PRBool aRemoveFromForm = PR_TRUE);
-  NS_IMETHOD Init();
-  NS_IMETHOD SaveState(nsIPresContext* aPresContext, nsIPresState** aState)
-  {
-    return NS_OK;
-  }
-  NS_IMETHOD RestoreState(nsIPresContext* aPresContext, nsIPresState* aState)
-  {
-    return NS_OK;
-  }
+  NS_IMETHOD SaveState() { return NS_OK; }
+  NS_IMETHOD RestoreState(nsIPresState* aState) { return NS_OK; }
 
   // nsIContent
   NS_IMETHOD SetParent(nsIContent *aParent);
@@ -655,15 +715,15 @@ public:
                          PRBool aCompileEventHandlers);
 
   NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                     const nsAReadableString& aValue, PRBool aNotify);
+                     const nsAString& aValue, PRBool aNotify);
 
   NS_IMETHOD SetAttr(nsINodeInfo* aNodeInfo,
-                     const nsAReadableString& aValue,
+                     const nsAString& aValue,
                      PRBool aNotify);
+  NS_IMETHOD DoneCreatingElement();
 
-
-  NS_METHOD SetAttribute(const nsAReadableString& aName,
-                         const nsAReadableString& aValue)
+  NS_METHOD SetAttribute(const nsAString& aName,
+                         const nsAString& aValue)
   {
     return nsGenericHTMLElement::SetAttribute(aName, aValue);
   }
@@ -677,35 +737,46 @@ protected:
 
 /**
  * A macro to implement the getter and setter for a given string
- * valued content property. The method uses the generic SetAttr and
- * GetAttribute methods.
+ * valued content property. The method uses the generic GetAttr and
+ * SetAttr methods.
  */
 #define NS_IMPL_STRING_ATTR(_class, _method, _atom)                  \
+  NS_IMPL_STRING_ATTR_DEFAULT_VALUE(_class, _method, _atom, "")
+
+/**
+ * A macro to implement the getter and setter for a given string
+ * valued content property with a default value.
+ * The method uses the generic GetAttr and SetAttr methods.
+ */
+#define NS_IMPL_STRING_ATTR_DEFAULT_VALUE(_class, _method, _atom, _default) \
   NS_IMETHODIMP                                                      \
-  _class::Get##_method(nsAWritableString& aValue)                    \
+  _class::Get##_method(nsAString& aValue)                            \
   {                                                                  \
-    GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::_atom, aValue);          \
+    nsresult rv = GetAttr(kNameSpaceID_None,                         \
+                          nsHTMLAtoms::_atom, aValue);               \
+    if (rv == NS_CONTENT_ATTR_NOT_THERE) {                           \
+      aValue.Assign(NS_LITERAL_STRING(_default));                    \
+    }                                                                \
     return NS_OK;                                                    \
   }                                                                  \
   NS_IMETHODIMP                                                      \
-  _class::Set##_method(const nsAReadableString& aValue)              \
+  _class::Set##_method(const nsAString& aValue)                      \
   {                                                                  \
-    return SetAttr(kNameSpaceID_HTML, nsHTMLAtoms::_atom, aValue,    \
+    return SetAttr(kNameSpaceID_None, nsHTMLAtoms::_atom, aValue,    \
                    PR_TRUE);                                         \
   }
 
 /**
  * A macro to implement the getter and setter for a given boolean
- * valued content property. The method uses the generic SetAttr and
- * GetAttribute methods.
+ * valued content property. The method uses the generic GetAttr and
+ * SetAttr methods.
  */
 #define NS_IMPL_BOOL_ATTR(_class, _method, _atom)                     \
   NS_IMETHODIMP                                                       \
   _class::Get##_method(PRBool* aValue)                                \
   {                                                                   \
     nsHTMLValue val;                                                  \
-    nsresult rv;                                                      \
-    rv = GetHTMLAttribute(nsHTMLAtoms::_atom, val);                   \
+    nsresult rv = GetHTMLAttribute(nsHTMLAtoms::_atom, val);          \
     *aValue = NS_CONTENT_ATTR_NOT_THERE != rv;                        \
     return NS_OK;                                                     \
   }                                                                   \
@@ -717,22 +788,30 @@ protected:
       return SetHTMLAttribute(nsHTMLAtoms::_atom, empty, PR_TRUE);    \
     }                                                                 \
     else {                                                            \
-      UnsetAttr(kNameSpaceID_HTML, nsHTMLAtoms::_atom, PR_TRUE);      \
+      UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::_atom, PR_TRUE);      \
       return NS_OK;                                                   \
     }                                                                 \
   }
 
 /**
  * A macro to implement the getter and setter for a given integer
- * valued content property. The method uses the generic SetAttr and
- * GetAttribute methods.
+ * valued content property. The method uses the generic GetAttr and
+ * SetAttr methods.
  */
 #define NS_IMPL_INT_ATTR(_class, _method, _atom)                    \
+  NS_IMPL_INT_ATTR_DEFAULT_VALUE(_class, _method, _atom, -1)
+
+/**
+ * A macro to implement the getter and setter for a given integer
+ * valued content property with a default value.
+ * The method uses the generic GetAttr and SetAttr methods.
+ */
+#define NS_IMPL_INT_ATTR_DEFAULT_VALUE(_class, _method, _atom, _default) \
   NS_IMETHODIMP                                                     \
   _class::Get##_method(PRInt32* aValue)                             \
   {                                                                 \
     nsHTMLValue value;                                              \
-    *aValue = -1;                                                   \
+    *aValue = _default;                                             \
     if (NS_CONTENT_ATTR_HAS_VALUE ==                                \
         GetHTMLAttribute(nsHTMLAtoms::_atom, value)) {              \
       if (value.GetUnit() == eHTMLUnit_Integer) {                   \
@@ -748,6 +827,39 @@ protected:
     return SetHTMLAttribute(nsHTMLAtoms::_atom, value, PR_TRUE);    \
   }
 
+/**
+ * A macro to implement the getter and the setter for a given pixel
+ * valued content property. The method uses the generic GetAttr and
+ * SetAttr methods.
+ */
+#define NS_IMPL_PIXEL_ATTR(_class, _method, _atom)                  \
+  NS_IMPL_PIXEL_ATTR_DEFAULT_VALUE(_class, _method, _atom, -1)
+
+/**
+ * A macro to implement the getter and the setter for a given pixel
+ * valued content property with a default value.
+ * The method uses the generic GetAttr and SetAttr methods.
+ */
+#define NS_IMPL_PIXEL_ATTR_DEFAULT_VALUE(_class, _method, _atom, _default) \
+  NS_IMETHODIMP                                                     \
+  _class::Get##_method(PRInt32* aValue)                             \
+  {                                                                 \
+    nsHTMLValue value;                                              \
+    *aValue = _default;                                                   \
+    if (NS_CONTENT_ATTR_HAS_VALUE ==                                \
+        GetHTMLAttribute(nsHTMLAtoms::_atom, value)) {              \
+      if (value.GetUnit() == eHTMLUnit_Pixel) {                     \
+        *aValue = value.GetPixelValue();                            \
+      }                                                             \
+    }                                                               \
+    return NS_OK;                                                   \
+  }                                                                 \
+  NS_IMETHODIMP                                                     \
+  _class::Set##_method(PRInt32 aValue)                              \
+  {                                                                 \
+    nsHTMLValue value(aValue, eHTMLUnit_Pixel);                     \
+    return SetHTMLAttribute(nsHTMLAtoms::_atom, value, PR_TRUE);    \
+  }
 
 /**
  * QueryInterface() implementation helper macros
@@ -783,7 +895,7 @@ protected:
 
 #define NS_HTML_CONTENT_INTERFACE_MAP_END                                     \
     {                                                                         \
-      return NS_NOINTERFACE;                                                  \
+      return PostQueryInterface(aIID, aInstancePtr);                          \
     }                                                                         \
                                                                               \
     NS_ADDREF(foundInterface);                                                \
@@ -899,7 +1011,8 @@ nsresult
 NS_NewHTMLImageElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
 nsresult
-NS_NewHTMLInputElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
+NS_NewHTMLInputElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo,
+                       PRBool aFromParser);
 
 nsresult
 NS_NewHTMLInsElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
@@ -965,7 +1078,8 @@ nsresult
 NS_NewHTMLScriptElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
 
 nsresult
-NS_NewHTMLSelectElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo);
+NS_NewHTMLSelectElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo,
+                        PRBool aFromParser);
 
 inline nsresult
 NS_NewHTMLSpacerElement(nsIHTMLContent** aResult, nsINodeInfo *aNodeInfo)
