@@ -42,12 +42,12 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIFormControl.h"
 #include "nsIForm.h"
+#include "nsIFormSubmission.h"
 #include "nsIURL.h"
 
 #include "nsIFrame.h"
@@ -90,26 +90,22 @@ public:
   // overrided nsIFormControl method
   NS_IMETHOD GetType(PRInt32* aType);
   NS_IMETHOD Reset();
-  NS_IMETHOD IsSuccessful(nsIContent* aSubmitElement, PRBool *_retval);
-  NS_IMETHOD GetMaxNumValues(PRInt32 *_retval);
-  NS_IMETHOD GetNamesValues(PRInt32 aMaxNumValues,
-                            PRInt32& aNumValues,
-                            nsString* aValues,
-                            nsString* aNames);
+  NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
+                               nsIContent* aSubmitElement);
 
   // nsIContent overrides...
   NS_IMETHOD GetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,
-                          nsAWritableString& aResult) const;
+                          nsAString& aResult) const;
   NS_IMETHOD SetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,
-                          const nsAReadableString& aValue, PRBool aNotify);
+                          const nsAString& aValue, PRBool aNotify);
   NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
   NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD HandleDOMEvent(nsIPresContext* aPresContext,
                             nsEvent* aEvent,
                             nsIDOMEvent** aDOMEvent,
@@ -121,12 +117,12 @@ public:
 
 protected:
   PRInt8 mType;
+  PRPackedBool mHandlingClick;
 
 private:
   // The analogue of defaultValue in the DOM for input and textarea
-  nsresult SetDefaultValue(const nsAReadableString& aDefaultValue);
-  nsresult GetDefaultValue(nsAWritableString& aDefaultValue);
-
+  nsresult SetDefaultValue(const nsAString& aDefaultValue);
+  nsresult GetDefaultValue(nsAString& aDefaultValue);
 };
 
 
@@ -162,6 +158,7 @@ NS_NewHTMLButtonElement(nsIHTMLContent** aInstancePtrResult,
 nsHTMLButtonElement::nsHTMLButtonElement()
 {
   mType = NS_FORM_BUTTON_SUBMIT; // default
+  mHandlingClick = PR_FALSE;
 }
 
 nsHTMLButtonElement::~nsHTMLButtonElement()
@@ -187,7 +184,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_END
 
 NS_IMETHODIMP
 nsHTMLButtonElement::GetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,
-                                  nsAWritableString& aResult) const
+                                  nsAString& aResult) const
 {
   if (aName == nsHTMLAtoms::disabled) {
     nsresult rv = nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_None, nsHTMLAtoms::disabled, aResult);
@@ -206,13 +203,13 @@ nsHTMLButtonElement::GetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,
 
 NS_IMETHODIMP
 nsHTMLButtonElement::SetAttribute(PRInt32 aNameSpaceID, nsIAtom* aName,
-                                  const nsAReadableString& aValue,
+                                  const nsAString& aValue,
                                   PRBool aNotify)
 {
   nsAutoString value(aValue);
 
   if (aName == nsHTMLAtoms::disabled &&
-      !Compare(value, NS_LITERAL_STRING("false"), nsCaseInsensitiveStringComparator())) {
+      value.Equals(NS_LITERAL_STRING("false"), nsCaseInsensitiveStringComparator())) {
     return UnsetAttr(aNameSpaceID, aName, aNotify);
   }
 
@@ -261,7 +258,7 @@ nsHTMLButtonElement::GetForm(nsIDOMHTMLFormElement** aForm)
 }
 
 NS_IMETHODIMP
-nsHTMLButtonElement::GetType(nsAWritableString& aType)
+nsHTMLButtonElement::GetType(nsAString& aType)
 {
   return AttributeToString(nsHTMLAtoms::type,
                            nsHTMLValue(mType, eHTMLUnit_Enumerated),
@@ -286,13 +283,52 @@ nsHTMLButtonElement::Focus()
   return SetElementFocus(PR_TRUE);
 }
 
+NS_IMETHODIMP
+nsHTMLButtonElement::Click()
+{
+  if (mHandlingClick)
+    return NS_OK;
+
+  mHandlingClick = PR_TRUE;
+  nsCOMPtr<nsIDocument> doc; 
+  GetDocument(*getter_AddRefs(doc));
+
+  if (doc) {
+    PRInt32 numShells = doc->GetNumberOfShells();
+    nsCOMPtr<nsIPresContext> context;
+    for (PRInt32 count=0; count < numShells; count++) {
+      nsCOMPtr<nsIPresShell> shell;
+      doc->GetShellAt(count, getter_AddRefs(shell));
+      if (shell) {
+        shell->GetPresContext(getter_AddRefs(context));
+        if (context) {
+          nsEventStatus status = nsEventStatus_eIgnore;
+          nsMouseEvent event;
+          event.eventStructType = NS_MOUSE_EVENT;
+          event.message = NS_MOUSE_LEFT_CLICK;
+          event.isShift = PR_FALSE;
+          event.isControl = PR_FALSE;
+          event.isAlt = PR_FALSE;
+          event.isMeta = PR_FALSE;
+          event.clickCount = 0;
+          event.widget = nsnull;
+          HandleDOMEvent(context, &event, nsnull,
+                              NS_EVENT_FLAG_INIT, &status);
+        }
+      }
+    }
+  }
+  mHandlingClick = PR_FALSE;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsHTMLButtonElement::SetFocus(nsIPresContext* aPresContext)
 {
   NS_ENSURE_ARG_POINTER(aPresContext);
   // first see if we are disabled or not. If disabled then do nothing.
   nsAutoString disabled;
-  if (NS_CONTENT_ATTR_HAS_VALUE == GetAttribute(kNameSpaceID_HTML,
+  if (NS_CONTENT_ATTR_HAS_VALUE == GetAttribute(kNameSpaceID_None,
                                                 nsHTMLAtoms::disabled,
                                                 disabled)) {
     return NS_OK;
@@ -352,7 +388,7 @@ static nsGenericHTMLElement::EnumTable kButtonTypeTable[] = {
 
 NS_IMETHODIMP
 nsHTMLButtonElement::StringToAttribute(nsIAtom* aAttribute,
-                                       const nsAReadableString& aValue,
+                                       const nsAString& aValue,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::tabindex) {
@@ -383,7 +419,7 @@ nsHTMLButtonElement::StringToAttribute(nsIAtom* aAttribute,
 NS_IMETHODIMP
 nsHTMLButtonElement::AttributeToString(nsIAtom* aAttribute,
                                        const nsHTMLValue& aValue,
-                                       nsAWritableString& aResult) const
+                                       nsAString& aResult) const
 {
   if (aAttribute == nsHTMLAtoms::type) {
     if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
@@ -467,10 +503,25 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
 
     case NS_MOUSE_LEFT_CLICK:
       {
-        // Tell the frame about the click
-        nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
-        if (formControlFrame) {
-          formControlFrame->MouseClicked(aPresContext);
+        if (mType == NS_FORM_BUTTON_SUBMIT || mType == NS_FORM_BUTTON_RESET) {
+          nsFormEvent event;
+          event.eventStructType = NS_FORM_EVENT;
+          event.message         = (mType == NS_FORM_BUTTON_RESET)
+                                   ? NS_FORM_RESET : NS_FORM_SUBMIT;
+          event.originator      = this;
+          nsEventStatus status  = nsEventStatus_eIgnore;
+
+          nsCOMPtr<nsIPresShell> presShell;
+          aPresContext->GetShell(getter_AddRefs(presShell));
+          // If |nsIPresShell::Destroy| has been called due to
+          // handling the event (base class HandleDOMEvent, above),
+          // the pres context will return a null pres shell.  See
+          // bug 125624.
+          if (presShell) {
+            nsCOMPtr<nsIContent> form(do_QueryInterface(mForm));
+            presShell->HandleEventWithTarget(&event, nsnull, form,
+                                             NS_EVENT_FLAG_INIT, &status);
+          }
         }
       }
       break;// NS_MOUSE_LEFT_CLICK
@@ -562,74 +613,67 @@ nsHTMLButtonElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
 #endif
 
 nsresult
-nsHTMLButtonElement::GetDefaultValue(nsAWritableString& aDefaultValue)
+nsHTMLButtonElement::GetDefaultValue(nsAString& aDefaultValue)
 {
-  return GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::value, aDefaultValue);
+  return GetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aDefaultValue);
 }
 
 nsresult
-nsHTMLButtonElement::SetDefaultValue(const nsAReadableString& aDefaultValue)
+nsHTMLButtonElement::SetDefaultValue(const nsAString& aDefaultValue)
 {
-  return SetAttr(kNameSpaceID_HTML, nsHTMLAtoms::value, aDefaultValue, PR_TRUE);
+  return SetAttr(kNameSpaceID_None, nsHTMLAtoms::value, aDefaultValue, PR_TRUE);
 }
 
-nsresult
+NS_IMETHODIMP
 nsHTMLButtonElement::Reset()
 {
   return NS_OK;
 }
 
-nsresult
-nsHTMLButtonElement::IsSuccessful(nsIContent* aSubmitElement,
-                                  PRBool *_retval)
+NS_IMETHODIMP
+nsHTMLButtonElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
+                                       nsIContent* aSubmitElement)
 {
-  *_retval = PR_FALSE;
+  nsresult rv = NS_OK;
+
+  //
+  // We only submit if we were the button pressed
+  //
   if (aSubmitElement != this) {
     return NS_OK;
   }
 
-  // if it's disabled, it won't submit
+  //
+  // Disabled elements don't submit
+  //
   PRBool disabled;
-  nsresult rv = GetDisabled(&disabled);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (disabled) {
-    return NS_OK;
+  rv = GetDisabled(&disabled);
+  if (NS_FAILED(rv) || disabled) {
+    return rv;
   }
 
-  // If there is no name, it won't submit
-  nsAutoString val;
-  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, val);
-  *_retval = rv != NS_CONTENT_ATTR_NOT_THERE;
-  return NS_OK;
-}
-
-nsresult
-nsHTMLButtonElement::GetMaxNumValues(PRInt32 *_retval)
-{
-  *_retval = 1;
-  return NS_OK;
-}
-
-nsresult
-nsHTMLButtonElement::GetNamesValues(PRInt32 aMaxNumValues,
-                                    PRInt32& aNumValues,
-                                    nsString* aValues,
-                                    nsString* aNames)
-{
-  NS_ENSURE_TRUE(aMaxNumValues >= 1, NS_ERROR_UNEXPECTED);
-
-  // We'll of course use the name of the control for the submit
+  //
+  // Get the name (if no name, no submit)
+  //
   nsAutoString name;
-  nsresult rv = GetName(name);
-  NS_ENSURE_SUCCESS(rv, rv);
+  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name);
+  if (NS_FAILED(rv) || rv == NS_CONTENT_ATTR_NOT_THERE) {
+    return rv;
+  }
 
+  //
+  // Get the value
+  //
   nsAutoString value;
   rv = GetValue(value);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
 
-  aNames[0] = name;
-  aValues[0] = value;
-  aNumValues = 1;
-  return NS_OK;
+  //
+  // Submit
+  //
+  rv = aFormSubmission->AddNameValuePair(this, name, value);
+
+  return rv;
 }
