@@ -41,12 +41,11 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
 #include "nsIJSNativeInitializer.h"
 #include "nsSize.h"
 #include "nsIDocument.h"
@@ -68,6 +67,7 @@
 #include "nsGUIEvent.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIDOMWindow.h"
+#include "nsIPref.h"
 
 #include "imgIContainer.h"
 #include "imgILoader.h"
@@ -80,6 +80,8 @@
 
 #include "nsIJSContextStack.h"
 
+static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
+
 // XXX nav attrs: suppress
 
 class nsHTMLImageElement : public nsGenericHTMLLeafElement,
@@ -118,14 +120,12 @@ public:
                         PRUint32 argc, jsval *argv);
 
   // nsIContent
-  NS_IMETHOD SetDocument(nsIDocument* aDocument, PRBool aDeep,
-                         PRBool aCompileEventHandlers);
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute,
                                       PRInt32 aModType,
                                       PRInt32& aHint) const;
@@ -138,16 +138,13 @@ public:
 #endif
 
 protected:
-  nsresult SetSrcInner(nsIURI* aBaseURL, const nsAReadableString& aSrc);
+  nsresult SetSrcInner(nsIURI* aBaseURL, const nsAString& aSrc);
   static nsresult GetCallerSourceURL(nsIURI** sourceURL);
 
   nsresult GetImageFrame(nsIImageFrame** aImageFrame);
   nsresult GetXY(PRInt32* aX, PRInt32* aY);
   nsresult GetWidthHeight(PRInt32* aWidth, PRInt32* aHeight);
 
-  // Only used if this is a script constructed image
-  nsIDocument* mOwnerDocument;
-
   nsCOMPtr<imgIRequest> mRequest;
 };
 
@@ -164,10 +161,34 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
    */
   nsCOMPtr<nsINodeInfo> nodeInfo(aNodeInfo);
   if (!nodeInfo) {
+    nsCOMPtr<nsIJSContextStack> stack =
+      do_GetService("@mozilla.org/js/xpc/ContextStack;1");
+
+    NS_ENSURE_TRUE(stack, NS_ERROR_NOT_AVAILABLE);
+
+    JSContext *cx = nsnull;
+
+    nsresult rv = stack->Peek(&cx);
+
+    if (NS_FAILED(rv) || !cx)
+      return NS_ERROR_UNEXPECTED;
+
+    nsCOMPtr<nsIScriptGlobalObject> globalObject;
+    nsContentUtils::GetStaticScriptGlobal(cx, ::JS_GetGlobalObject(cx),
+                                          getter_AddRefs(globalObject));;
+
+    nsCOMPtr<nsIDOMWindowInternal> win(do_QueryInterface(globalObject));
+    NS_ENSURE_TRUE(win, NS_ERROR_UNEXPECTED);
+
+    nsCOMPtr<nsIDOMDocument> dom_doc;
+    win->GetDocument(getter_AddRefs(dom_doc));
+
+    nsCOMPtr<nsIDocument> doc(do_QueryInterface(dom_doc));
+    NS_ENSURE_TRUE(doc, NS_ERROR_UNEXPECTED);
+
     nsCOMPtr<nsINodeInfoManager> nodeInfoManager;
-    nsresult rv;
-    rv = nsNodeInfoManager::GetAnonymousManager(*getter_AddRefs(nodeInfoManager));
-    NS_ENSURE_SUCCESS(rv, rv);
+    doc->GetNodeInfoManager(*getter_AddRefs(nodeInfoManager));
+    NS_ENSURE_TRUE(nodeInfoManager, NS_ERROR_UNEXPECTED);
 
     rv = nodeInfoManager->GetNodeInfo(nsHTMLAtoms::img, nsnull,
                                       kNameSpaceID_None,
@@ -198,17 +219,10 @@ NS_NewHTMLImageElement(nsIHTMLContent** aInstancePtrResult,
 
 nsHTMLImageElement::nsHTMLImageElement()
 {
-  mOwnerDocument = nsnull;
 }
 
 nsHTMLImageElement::~nsHTMLImageElement()
 {
-  NS_IF_RELEASE(mOwnerDocument);
-
-#ifndef USE_IMG2
-  if (mLoader)
-    mLoader->RemoveFrame(this);
-#endif
 }
 
 
@@ -259,12 +273,12 @@ nsHTMLImageElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Name, name)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Align, align)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, Alt, alt)
-NS_IMPL_INT_ATTR(nsHTMLImageElement, Border, border)
-NS_IMPL_INT_ATTR(nsHTMLImageElement, Hspace, hspace)
+NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Border, border)
+NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Hspace, hspace)
 NS_IMPL_BOOL_ATTR(nsHTMLImageElement, IsMap, ismap)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, LongDesc, longdesc)
 NS_IMPL_STRING_ATTR(nsHTMLImageElement, UseMap, usemap)
-NS_IMPL_INT_ATTR(nsHTMLImageElement, Vspace, vspace)
+NS_IMPL_PIXEL_ATTR(nsHTMLImageElement, Vspace, vspace)
 
 nsresult
 nsHTMLImageElement::GetImageFrame(nsIImageFrame** aImageFrame)
@@ -272,37 +286,12 @@ nsHTMLImageElement::GetImageFrame(nsIImageFrame** aImageFrame)
   NS_ENSURE_ARG_POINTER(aImageFrame);
   *aImageFrame = nsnull;
 
-  if (!mDocument) {
-    return NS_OK;
+  nsIFrame* frame = GetPrimaryFrame(PR_TRUE);
+
+  if (frame) {
+    CallQueryInterface(frame, aImageFrame);
   }
 
-  // Make sure the presentation is up-to-date
-  nsresult rv = mDocument->FlushPendingNotifications();
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
-
-  nsCOMPtr<nsIPresShell> shell;
-  mDocument->GetShellAt(0, getter_AddRefs(shell));
-
-  if (!shell) {
-    return NS_OK;
-  }
-
-  nsCOMPtr<nsIPresContext> context;
-  rv = shell->GetPresContext(getter_AddRefs(context));
-  if (!context) {
-    return NS_OK;
-  }
-
-  nsIFrame* frame = nsnull;
-  rv = shell->GetPrimaryFrameFor(this, &frame);
-  if (!frame || NS_FAILED(rv)) {
-    return rv;
-  }
-
-  CallQueryInterface(frame, aImageFrame);
-
   return NS_OK;
 }
 
@@ -553,7 +542,7 @@ nsHTMLImageElement::SetWidth(PRInt32 aWidth)
 
 NS_IMETHODIMP
 nsHTMLImageElement::StringToAttribute(nsIAtom* aAttribute,
-                                      const nsAReadableString& aValue,
+                                      const nsAString& aValue,
                                       nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::align) {
@@ -579,7 +568,7 @@ nsHTMLImageElement::StringToAttribute(nsIAtom* aAttribute,
 NS_IMETHODIMP
 nsHTMLImageElement::AttributeToString(nsIAtom* aAttribute,
                                       const nsHTMLValue& aValue,
-                                      nsAWritableString& aResult) const
+                                      nsAString& aResult) const
 {
   if (aAttribute == nsHTMLAtoms::align) {
     if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
@@ -721,77 +710,37 @@ NS_IMETHODIMP
 nsHTMLImageElement::Initialize(JSContext* aContext, JSObject *aObj,
                                PRUint32 argc, jsval *argv)
 {
-  nsresult result = NS_OK;
+  if (argc <= 0) {
+    // Nothing to do here if we don't get any arguments.
 
-  // XXX This element is created unattached to any document.  Later
-  // on, it might be used to preload the image cache.  For that, we
-  // need a document (actually a pres context).  The only way to get
-  // one is to associate the image with one at creation time.
-  // This is safer than it used to be since we get the global object
-  // from the static scope chain rather than through the JSCOntext.
-
-  nsCOMPtr<nsIScriptGlobalObject> globalObject;
-  nsContentUtils::GetStaticScriptGlobal(aContext, aObj,
-                                        getter_AddRefs(globalObject));;
-
-  nsCOMPtr<nsIDOMWindowInternal> domWindow(do_QueryInterface(globalObject));
-
-  if (domWindow) {
-    nsCOMPtr<nsIDOMDocument> domDocument;
-    result = domWindow->GetDocument(getter_AddRefs(domDocument));
-    if (NS_SUCCEEDED(result)) {
-      // Maintain the reference
-      result = CallQueryInterface(domDocument, &mOwnerDocument);
-    }
+    return NS_OK;
   }
 
-  if (NS_SUCCEEDED(result) && (argc > 0)) {
-    // The first (optional) argument is the width of the image
-    int32 width;
-    JSBool ret = JS_ValueToInt32(aContext, argv[0], &width);
+  // The first (optional) argument is the width of the image
+  int32 width;
+  JSBool ret = JS_ValueToInt32(aContext, argv[0], &width);
+  NS_ENSURE_TRUE(ret, NS_ERROR_INVALID_ARG);
 
-    if (ret) {
-      nsHTMLValue widthVal((PRInt32)width, eHTMLUnit_Integer);
+  nsHTMLValue widthVal((PRInt32)width, eHTMLUnit_Integer);
 
-      result = SetHTMLAttribute(nsHTMLAtoms::width, widthVal, PR_FALSE);
+  nsresult rv = SetHTMLAttribute(nsHTMLAtoms::width, widthVal, PR_FALSE);
 
-      if (NS_SUCCEEDED(result) && (argc > 1)) {
-        // The second (optional) argument is the height of the image
-        int32 height;
-        ret = JS_ValueToInt32(aContext, argv[1], &height);
+  if (NS_SUCCEEDED(rv) && (argc > 1)) {
+    // The second (optional) argument is the height of the image
+    int32 height;
+    ret = JS_ValueToInt32(aContext, argv[1], &height);
+    NS_ENSURE_TRUE(ret, NS_ERROR_INVALID_ARG);
 
-        if (ret) {
-          nsHTMLValue heightVal((PRInt32)height, eHTMLUnit_Integer);
+    nsHTMLValue heightVal((PRInt32)height, eHTMLUnit_Integer);
 
-          result = SetHTMLAttribute(nsHTMLAtoms::height, heightVal, PR_FALSE);
-        }
-        else {
-          result = NS_ERROR_INVALID_ARG;
-        }
-      }
-    }
-    else {
-      result = NS_ERROR_INVALID_ARG;
-    }
+    rv = SetHTMLAttribute(nsHTMLAtoms::height, heightVal, PR_FALSE);
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
-nsHTMLImageElement::SetDocument(nsIDocument* aDocument,
-                                PRBool aDeep, PRBool aCompileEventHandlers)
-{
-  // If we've been added to the document, we can get rid of
-  // our owner document reference so as to avoid a circular
-  // reference.
-  NS_IF_RELEASE(mOwnerDocument);
-  return nsGenericHTMLLeafElement::SetDocument(aDocument, aDeep,
-                                               aCompileEventHandlers);
-}
-
-NS_IMETHODIMP
-nsHTMLImageElement::GetSrc(nsAWritableString& aSrc)
+nsHTMLImageElement::GetSrc(nsAString& aSrc)
 {
   // Resolve url to an absolute url
   nsresult rv = NS_OK;
@@ -802,11 +751,11 @@ nsHTMLImageElement::GetSrc(nsAWritableString& aSrc)
   GetBaseURL(*getter_AddRefs(baseURL));
 
   // Get href= attribute (relative URL).
-  nsGenericHTMLLeafElement::GetAttr(kNameSpaceID_HTML, nsHTMLAtoms::src,
+  nsGenericHTMLLeafElement::GetAttr(kNameSpaceID_None, nsHTMLAtoms::src,
                                     relURLSpec);
   relURLSpec.Trim(" \t\n\r");
 
-  if (baseURL && relURLSpec.Length() > 0) {
+  if (baseURL && !relURLSpec.IsEmpty()) {
     // Get absolute URL.
     rv = NS_MakeAbsoluteURI(aSrc, relURLSpec, baseURL);
   }
@@ -912,43 +861,23 @@ NS_IMETHODIMP nsHTMLImageElement::FrameChanged(imgIContainer *container, nsISupp
 
 nsresult
 nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
-                                const nsAReadableString& aSrc)
+                                const nsAString& aSrc)
 {
-  nsresult result = SetAttr(kNameSpaceID_HTML, nsHTMLAtoms::src, aSrc,
+  nsresult result = SetAttr(kNameSpaceID_None, nsHTMLAtoms::src, aSrc,
                             PR_TRUE);
 
-  if (NS_SUCCEEDED(result) && mOwnerDocument) {
+  if (NS_SUCCEEDED(result) && !mDocument) {
+    nsCOMPtr<nsIDocument> doc;
+    mNodeInfo->GetDocument(*getter_AddRefs(doc));
+
     nsCOMPtr<nsIPresShell> shell;
 
-    mOwnerDocument->GetShellAt(0, getter_AddRefs(shell));
+    doc->GetShellAt(0, getter_AddRefs(shell));
     if (shell) {
       nsCOMPtr<nsIPresContext> context;
 
       result = shell->GetPresContext(getter_AddRefs(context));
       if (context) {
-        nsSize size;
-        nsHTMLValue val;
-        float p2t;
-
-        context->GetScaledPixelsToTwips(&p2t);
-        result = GetHTMLAttribute(nsHTMLAtoms::width, val);
-
-        if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-          size.width = NSIntPixelsToTwips(val.GetIntValue(), p2t);
-        }
-        else {
-          size.width = 0;
-        }
-
-        result = GetHTMLAttribute(nsHTMLAtoms::height, val);
-
-        if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-          size.height = NSIntPixelsToTwips(val.GetIntValue(), p2t);
-        }
-        else {
-          size.height = 0;
-        }
-
         nsAutoString url;
         if (aBaseURL) {
           result = NS_MakeAbsoluteURI(url, aSrc, aBaseURL);
@@ -960,13 +889,8 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
           url.Assign(aSrc);
         }
 
-        nsSize* specifiedSize = nsnull;
-        if ((size.width > 0) || (size.height > 0)) {
-          specifiedSize = &size;
-        }
-
         nsCOMPtr<nsIURI> uri;
-        result = NS_NewURI(getter_AddRefs(uri), aSrc, aBaseURL);
+        result = NS_NewURI(getter_AddRefs(uri), aSrc, nsnull, aBaseURL);
         if (NS_FAILED(result)) return result;
 
         nsCOMPtr<nsIDocument> document;
@@ -980,11 +904,12 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
         nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalObject));
 
         PRBool shouldLoad = PR_TRUE;
-        result = NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
-                                           uri,
-                                           NS_STATIC_CAST(nsIDOMHTMLImageElement *,
-                                                          this),
-                                           domWin, &shouldLoad);
+        result =
+          NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
+                                    uri,
+                                    NS_STATIC_CAST(nsIDOMHTMLImageElement *,
+                                                   this),
+                                    domWin, &shouldLoad);
         if (NS_SUCCEEDED(result) && !shouldLoad) {
           return NS_OK;
         }
@@ -1001,8 +926,6 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
           return NS_ERROR_FAILURE;
         }
 
-        nsCOMPtr<nsISupports> sup(do_QueryInterface(context));
-
         nsCOMPtr<nsIDocument> doc;
         nsCOMPtr<nsILoadGroup> loadGroup;
         shell->GetDocument(getter_AddRefs(doc));
@@ -1010,40 +933,50 @@ nsHTMLImageElement::SetSrcInner(nsIURI* aBaseURL,
           doc->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
         }
 
-        il->LoadImage(uri, loadGroup, this, sup, nsIRequest::LOAD_NORMAL,
+        il->LoadImage(uri, nsnull, loadGroup, this, context, nsIRequest::LOAD_NORMAL,
                       nsnull, nsnull, getter_AddRefs(mRequest));
       }
     }
-
-    // Only do this the first time since it's only there for
-    // backwards compatability
-    NS_RELEASE(mOwnerDocument);
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-nsHTMLImageElement::SetSrc(const nsAReadableString& aSrc)
+nsHTMLImageElement::SetSrc(const nsAString& aSrc)
 {
+  /*
+   * If caller is not chrome and dom.disable_image_src_set is true,
+   * prevent setting image.src by exiting early
+   */
+
+  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID));
+  if (prefs) {
+    PRBool disableImageSrcSet = PR_FALSE;
+    prefs->GetBoolPref("dom.disable_image_src_set", &disableImageSrcSet);
+
+    if (disableImageSrcSet && !nsContentUtils::IsCallerChrome()) {
+      return NS_OK;
+    }
+  }
+
   nsCOMPtr<nsIURI> baseURL;
-  nsresult result = NS_OK;
+  nsresult rv = NS_OK;
 
   (void) GetCallerSourceURL(getter_AddRefs(baseURL));
 
-  if (mDocument && !baseURL) {
-    result = mDocument->GetBaseURL(*getter_AddRefs(baseURL));
+  nsCOMPtr<nsIDocument> doc;
+  mNodeInfo->GetDocument(*getter_AddRefs(doc));
+
+  if (doc && !baseURL) {
+    rv = doc->GetBaseURL(*getter_AddRefs(baseURL));
   }
 
-  if (mOwnerDocument && !baseURL) {
-    result = mOwnerDocument->GetBaseURL(*getter_AddRefs(baseURL));
+  if (NS_SUCCEEDED(rv)) {
+    rv = SetSrcInner(baseURL, aSrc);
   }
 
-  if (NS_SUCCEEDED(result)) {
-    result = SetSrcInner(baseURL, aSrc);
-  }
-
-  return result;
+  return rv;
 }
 
 #ifdef DEBUG
