@@ -44,7 +44,6 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
@@ -57,7 +56,7 @@
 #include "nsIDOMHTMLCollection.h"
 #include "nsIJSNativeInitializer.h"
 #include "nsISelectElement.h"
-#include "nsIComboboxControlFrame.h"
+#include "nsISelectControlFrame.h"
 
 // Notify/query select frame for selected state
 #include "nsIFormControlFrame.h"
@@ -68,7 +67,8 @@
 #include "nsNodeInfoManager.h"
 #include "nsCOMPtr.h"
 #include "nsLayoutAtoms.h"
-
+#include "nsIEventStateManager.h"
+#include "nsXULAtoms.h"
 
 class nsHTMLOptionElement : public nsGenericHTMLContainerElement,
                             public nsIDOMHTMLOptionElement,
@@ -99,7 +99,7 @@ public:
                         PRUint32 argc, jsval *argv);
 
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       PRInt32& aHint) const;
@@ -108,20 +108,40 @@ public:
 #endif
 
   // nsIOptionElement
-  NS_IMETHOD GetSelectedInternal(PRBool* aValue);
   NS_IMETHOD SetSelectedInternal(PRBool aValue, PRBool aNotify);
   NS_IMETHOD GetValueOrText(nsAString& aValue);
 
+  // nsIContent
+  NS_IMETHOD InsertChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify, 
+                           PRBool aDeepSetDocument);
+  NS_IMETHOD ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex, PRBool aNotify,
+                            PRBool aDeepSetDocument);
+  NS_IMETHOD AppendChildTo(nsIContent* aKid, PRBool aNotify,
+                           PRBool aDeepSetDocument);
+  NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
+
 protected:
-  // Get the primary frame associated with this content
+  /**
+   * Get the primary frame associated with this content
+   * @return the primary frame associated with this content
+   */
   nsIFormControlFrame *GetSelectFrame() const;
 
-  // Get the select content element that contains this option, this
-  // intentionally does not return nsresult, all we care about is if
-  // there's a select associated with this option or not.
+  /**
+   * Get the select content element that contains this option, this
+   * intentionally does not return nsresult, all we care about is if
+   * there's a select associated with this option or not.
+   * @param aSelectElement the select element (out param)
+   */
   void GetSelect(nsIDOMHTMLSelectElement **aSelectElement) const;
 
-  PRBool mIsInitialized;
+  /**
+   * Notify the frame that the option text or value or label has changed.
+   */
+  void NotifyTextChanged();
+
+  PRPackedBool mIsInitialized;
+  PRPackedBool mIsSelected;
 };
 
 nsresult
@@ -170,8 +190,9 @@ NS_NewHTMLOptionElement(nsIHTMLContent** aInstancePtrResult,
 
 
 nsHTMLOptionElement::nsHTMLOptionElement()
+  : mIsInitialized(PR_FALSE),
+    mIsSelected(PR_FALSE)
 {
-  mIsInitialized = PR_FALSE;
 }
 
 nsHTMLOptionElement::~nsHTMLOptionElement()
@@ -241,43 +262,16 @@ nsHTMLOptionElement::GetForm(nsIDOMHTMLFormElement** aForm)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLOptionElement::GetSelectedInternal(PRBool* aValue)
-{
-  // If it's not initialized, initialize it.
-  if (!mIsInitialized) {
-    mIsInitialized = PR_TRUE;
-    PRBool selected;
-    GetDefaultSelected(&selected);
-    // This does not need to be SetSelected (which sets selected in the select)
-    // because we *will* be initialized when we are placed into a select.  Plus
-    // it seems like that's just inviting an infinite loop.
-    SetSelectedInternal(selected, PR_TRUE);
-  }
-  nsAutoString tmpVal;
-  nsresult rv = GetAttr(kNameSpaceID_None,
-                        nsLayoutAtoms::optionSelectedPseudo,
-                        tmpVal);
-  *aValue = !(NS_FAILED(rv) || NS_CONTENT_ATTR_NOT_THERE == rv);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsHTMLOptionElement::SetSelectedInternal(PRBool aValue, PRBool aNotify)
 {
   mIsInitialized = PR_TRUE;
+  mIsSelected = aValue;
 
-  // This affects the display, but what the hey, it's a good place for it
-  if (aValue) {
-    return SetAttr(kNameSpaceID_None,
-                   nsLayoutAtoms::optionSelectedPseudo,
-                   NS_LITERAL_STRING(""),
-                   aNotify);
-  } else {
-    return UnsetAttr(kNameSpaceID_None,
-                     nsLayoutAtoms::optionSelectedPseudo,
-                     aNotify);
-  }
+  if (aNotify && mDocument)
+    mDocument->ContentStatesChanged(this, nsnull, NS_EVENT_STATE_CHECKED);
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -309,8 +303,19 @@ nsHTMLOptionElement::GetSelected(PRBool* aValue)
   NS_ENSURE_ARG_POINTER(aValue);
   *aValue = PR_FALSE;
 
-  // If there is no select element, return the selected
-  return GetSelectedInternal(aValue);
+  // If it's not initialized, initialize it.
+  if (!mIsInitialized) {
+    mIsInitialized = PR_TRUE;
+    PRBool selected;
+    GetDefaultSelected(&selected);
+    // This does not need to be SetSelected (which sets selected in the select)
+    // because we *will* be initialized when we are placed into a select.  Plus
+    // it seems like that's just inviting an infinite loop.
+    SetSelectedInternal(selected, PR_TRUE);
+  }
+
+  *aValue = mIsSelected;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -355,45 +360,32 @@ nsHTMLOptionElement::SetDisabled(PRBool aDisabled)
   if (aDisabled) {
     rv = SetHTMLAttribute(nsHTMLAtoms::disabled, empty, PR_TRUE);
   } else {
-    rv = UnsetAttr(kNameSpaceID_HTML, nsHTMLAtoms::disabled, PR_TRUE);
+    rv = UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::disabled, PR_TRUE);
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP                                                      
-nsHTMLOptionElement::GetLabel(nsAWritableString& aValue)
+nsHTMLOptionElement::GetLabel(nsAString& aValue)
 {                                                                  
-  nsGenericHTMLContainerElement::GetAttr(kNameSpaceID_HTML,
+  nsGenericHTMLContainerElement::GetAttr(kNameSpaceID_None,
                                          nsHTMLAtoms::label, aValue);
   return NS_OK;
 }         
                                                          
 NS_IMETHODIMP                                                      
-nsHTMLOptionElement::SetLabel(const nsAReadableString& aValue)
+nsHTMLOptionElement::SetLabel(const nsAString& aValue)
 {                                                                  
   nsresult result;
 
-  result = nsGenericHTMLContainerElement::SetAttr(kNameSpaceID_HTML,
+  result = nsGenericHTMLContainerElement::SetAttr(kNameSpaceID_None,
                                                   nsHTMLAtoms::label,
                                                   aValue, PR_TRUE);
   // XXX Why does this only happen to the combobox?  and what about
   // when the text gets set and label is blank?
   if (NS_SUCCEEDED(result)) {
-    // No need to flush here, if the select element/frame doesn't
-    // exist yet we don't need to force it to be created only to
-    // notify it about this new label
-    nsIFormControlFrame* fcFrame = GetSelectFrame();
-
-    if (fcFrame) {
-      nsIComboboxControlFrame* selectFrame = nsnull;
-
-      CallQueryInterface(fcFrame, &selectFrame);
-
-      if (selectFrame) {
-        selectFrame->UpdateSelection(PR_FALSE, PR_TRUE, 0);
-      }
-    }
+    NotifyTextChanged();
   }
 
   return NS_OK;
@@ -419,7 +411,7 @@ nsHTMLOptionElement::SetDefaultSelected(PRBool aDefaultSelected)
   if (aDefaultSelected) {
     rv = SetHTMLAttribute(nsHTMLAtoms::selected, empty, PR_TRUE);
   } else {
-    rv = UnsetAttr(kNameSpaceID_HTML, nsHTMLAtoms::selected, PR_TRUE);
+    rv = UnsetAttr(kNameSpaceID_None, nsHTMLAtoms::selected, PR_TRUE);
   }
 
   return rv;
@@ -468,7 +460,7 @@ nsHTMLOptionElement::GetIndex(PRInt32* aIndex)
 
 NS_IMETHODIMP
 nsHTMLOptionElement::StringToAttribute(nsIAtom* aAttribute,
-                                const nsAReadableString& aValue,
+                                const nsAString& aValue,
                                 nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::selected) {
@@ -497,14 +489,19 @@ nsHTMLOptionElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32
     } else if (!GetCommonMappedAttributesImpact(aAttribute, aHint)) {
       aHint = NS_STYLE_HINT_CONTENT;
     }
-  } else  // XXX fix me
-    aHint = NS_STYLE_HINT_NONE;
+  } else {
+    if (aAttribute == nsXULAtoms::menuactive) {
+      aHint = NS_STYLE_HINT_CONTENT;
+    } else {
+      aHint = NS_STYLE_HINT_NONE;
+    }
+  }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHTMLOptionElement::GetText(nsAWritableString& aText)
+nsHTMLOptionElement::GetText(nsAString& aText)
 {
   PRInt32 numNodes, i;
 
@@ -547,7 +544,7 @@ nsHTMLOptionElement::GetText(nsAWritableString& aText)
 }
 
 NS_IMETHODIMP
-nsHTMLOptionElement::SetText(const nsAReadableString& aText)
+nsHTMLOptionElement::SetText(const nsAString& aText)
 {
   PRInt32 numNodes, i;
   PRBool usedExistingTextNode = PR_FALSE;  // Do we need to create a text node?
@@ -570,6 +567,9 @@ nsHTMLOptionElement::SetText(const nsAReadableString& aText)
         result = domText->SetData(aText);
 
         if (NS_SUCCEEDED(result)) {
+          // If we used an existing node, the notification will not happen (the
+          // notification typically happens in AppendChildTo).
+          NotifyTextChanged();
           usedExistingTextNode = PR_TRUE;
         }
 
@@ -604,25 +604,73 @@ nsHTMLOptionElement::SetText(const nsAReadableString& aText)
     }
   }
 
-  if (NS_SUCCEEDED(result)) {
-    // No need to flush here, if there's no frame yet we don't need to
-    // force it to be created just to update the selection in it.
-    nsIFormControlFrame* fcFrame = GetSelectFrame();
-
-    if (fcFrame) {
-      nsIComboboxControlFrame* selectFrame = nsnull;
-
-      CallQueryInterface(fcFrame, &selectFrame);
-
-      if (selectFrame) {
-        selectFrame->UpdateSelection(PR_FALSE, PR_TRUE, 0);
-      }
-    }
-  }
-
   return NS_OK;
 }
 
+void
+nsHTMLOptionElement::NotifyTextChanged()
+{
+  // No need to flush here, if there's no frame yet we don't need to
+  // force it to be created just to update the selection in it.
+  nsIFormControlFrame* fcFrame = GetSelectFrame();
+
+  if (fcFrame) {
+    nsISelectControlFrame* selectFrame = nsnull;
+
+    CallQueryInterface(fcFrame, &selectFrame);
+
+    if (selectFrame) {
+      selectFrame->OnOptionTextChanged(this);
+    }
+  }
+}
+
+//
+// Override nsIContent children changing methods so we can detect when our text
+// is changing
+//
+NS_IMETHODIMP
+nsHTMLOptionElement::InsertChildAt(nsIContent* aKid, PRInt32 aIndex,
+                                   PRBool aNotify, PRBool aDeepSetDocument)
+{
+  nsresult rv = nsGenericHTMLContainerElement::InsertChildAt(aKid, aIndex,
+                                                             aNotify,
+                                                             aDeepSetDocument);
+  NotifyTextChanged();
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLOptionElement::ReplaceChildAt(nsIContent* aKid, PRInt32 aIndex,
+               PRBool aNotify, PRBool aDeepSetDocument)
+{
+  nsresult rv = nsGenericHTMLContainerElement::ReplaceChildAt(aKid, aIndex,
+                                                              aNotify,
+                                                              aDeepSetDocument);
+  NotifyTextChanged();
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLOptionElement::AppendChildTo(nsIContent* aKid, PRBool aNotify, PRBool aDeepSetDocument)
+{
+  nsresult rv = nsGenericHTMLContainerElement::AppendChildTo(aKid,
+                                                             aNotify,
+                                                             aDeepSetDocument);
+  NotifyTextChanged();
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLOptionElement::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
+{
+  nsresult rv = nsGenericHTMLContainerElement::RemoveChildAt(aIndex,
+                                                             aNotify);
+  NotifyTextChanged();
+  return rv;
+}
+
+
 // Options don't have frames - get the select content node
 // then call nsGenericHTMLElement::GetFormControlFrameFor()
 
@@ -715,7 +763,7 @@ nsHTMLOptionElement::Initialize(JSContext* aContext,
         nsAutoString value(NS_REINTERPRET_CAST(const PRUnichar*,
                                                JS_GetStringChars(jsstr)));
 
-        result = nsGenericHTMLContainerElement::SetAttr(kNameSpaceID_HTML,
+        result = nsGenericHTMLContainerElement::SetAttr(kNameSpaceID_None,
                                                         nsHTMLAtoms::value,
                                                         value, PR_FALSE);
         if (NS_FAILED(result)) {
