@@ -42,11 +42,10 @@
 #include "nsIDOMEventReceiver.h"
 #include "nsDOMError.h"
 #include "nsIHTMLContent.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
 #include "nsGenericHTMLElement.h"
 #include "GenericElementCollection.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
@@ -182,11 +181,11 @@ public:
   NS_DECL_NSIDOMHTMLTABLEROWELEMENT
 
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD AttributeToString(nsIAtom* aAttribute,
                                const nsHTMLValue& aValue,
-                               nsAWritableString& aResult) const;
+                               nsAString& aResult) const;
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
                                       PRInt32& aHint) const;
@@ -521,16 +520,16 @@ nsHTMLTableRowElement::DeleteCell(PRInt32 aValue)
   return NS_OK;
 }
 
-NS_IMPL_STRING_ATTR(nsHTMLTableRowElement, Align, align)
+NS_IMPL_STRING_ATTR_DEFAULT_VALUE(nsHTMLTableRowElement, Align, align, "left")
 NS_IMPL_STRING_ATTR(nsHTMLTableRowElement, BgColor, bgcolor)
-NS_IMPL_STRING_ATTR(nsHTMLTableRowElement, Ch, _char)
+NS_IMPL_STRING_ATTR_DEFAULT_VALUE(nsHTMLTableRowElement, Ch, _char, ".")
 NS_IMPL_STRING_ATTR(nsHTMLTableRowElement, ChOff, charoff)
-NS_IMPL_STRING_ATTR(nsHTMLTableRowElement, VAlign, valign)
+NS_IMPL_STRING_ATTR_DEFAULT_VALUE(nsHTMLTableRowElement, VAlign, valign, "middle")
 
 
 NS_IMETHODIMP
 nsHTMLTableRowElement::StringToAttribute(nsIAtom* aAttribute,
-                                  const nsAReadableString& aValue,
+                                  const nsAString& aValue,
                                   nsHTMLValue& aResult)
 {
   /* ignore these attributes, stored simply as strings
@@ -580,7 +579,7 @@ nsHTMLTableRowElement::StringToAttribute(nsIAtom* aAttribute,
 NS_IMETHODIMP
 nsHTMLTableRowElement::AttributeToString(nsIAtom* aAttribute,
                                   const nsHTMLValue& aValue,
-                                  nsAWritableString& aResult) const
+                                  nsAString& aResult) const
 {
   /* ignore these attributes, stored already as strings
      ch
