@@ -47,17 +47,17 @@
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIFormControl.h"
 #include "nsIForm.h"
+#include "nsIFormSubmission.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
-#include "nsIHTMLAttributes.h"
+#include "nsHTMLAttributes.h"
 #include "nsIFormControlFrame.h"
-#include "nsIGfxTextControlFrame.h"
+#include "nsITextControlFrame.h"
 #include "nsIEventStateManager.h"
 #include "nsISizeOfHandler.h"
 #include "nsLinebreakConverter.h"
@@ -72,6 +72,7 @@
 #include "nsIDOMText.h"
 #include "nsReadableUtils.h"
 #include "nsITextContent.h"
+#include "nsITextAreaElement.h"
 
 static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
 
@@ -79,7 +80,8 @@ static NS_DEFINE_CID(kXULControllersCID,  NS_XULCONTROLLERS_CID);
 class nsHTMLTextAreaElement : public nsGenericHTMLContainerFormElement,
                               public nsIDOMHTMLTextAreaElement,
                               public nsIDOMNSHTMLTextAreaElement,
-                              public nsITextControlElement
+                              public nsITextControlElement,
+                              public nsITextAreaElement
 {
 public:
   nsHTMLTextAreaElement();
@@ -103,20 +105,19 @@ public:
   // nsIDOMNSHTMLTextAreaElement
   NS_DECL_NSIDOMNSHTMLTEXTAREAELEMENT
 
+  // nsITextAreaElement
+  NS_DECL_NSITEXTAREAELEMENT
+
   // nsIFormControl
   NS_IMETHOD GetType(PRInt32* aType);
   NS_IMETHOD Reset();
-  NS_IMETHOD IsSuccessful(nsIContent* aSubmitElement, PRBool *_retval);
-  NS_IMETHOD GetMaxNumValues(PRInt32 *_retval);
-  NS_IMETHOD GetNamesValues(PRInt32 aMaxNumValues,
-                            PRInt32& aNumValues,
-                            nsString* aValues,
-                            nsString* aNames);
-  NS_IMETHOD SaveState(nsIPresContext* aPresContext, nsIPresState** aState);
-  NS_IMETHOD RestoreState(nsIPresContext* aPresContext, nsIPresState* aState);
+  NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
+                               nsIContent* aSubmitElement);
+  NS_IMETHOD SaveState();
+  NS_IMETHOD RestoreState(nsIPresState* aState);
 
   // nsITextControlElement
-  NS_IMETHOD SetValueGuaranteed(const nsAString& aValue, nsIGfxTextControlFrame2* aFrame);
+  NS_IMETHOD SetValueGuaranteed(const nsAString& aValue, nsITextControlFrame* aFrame);
   NS_IMETHOD SetValueChanged(PRBool aValueChanged);
 
   // nsIContent
@@ -128,7 +129,7 @@ public:
                            PRBool aDeepSetDocument);
   NS_IMETHOD RemoveChildAt(PRInt32 aIndex, PRBool aNotify);
   NS_IMETHOD StringToAttribute(nsIAtom* aAttribute,
-                               const nsAReadableString& aValue,
+                               const nsAString& aValue,
                                nsHTMLValue& aResult);
   NS_IMETHOD GetAttributeMappingFunction(nsMapRuleToAttributesFunc& aMapRuleFunc) const;
   NS_IMETHOD GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt32 aModType,
@@ -138,16 +139,30 @@ public:
                             nsEventStatus* aEventStatus);
   NS_IMETHOD SetFocus(nsIPresContext* aPresContext);
   NS_IMETHOD RemoveFocus(nsIPresContext* aPresContext);
+
+  nsresult GetInnerHTML(nsAString& aInnerHTML);
+  nsresult SetInnerHTML(const nsAString& aInnerHTML);
+
 #ifdef DEBUG
   NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
 #endif
 
 protected:
   nsCOMPtr<nsIControllers> mControllers;
+  /** The current value.  This is null if the frame owns the value. */
   char*                    mValue;
+  /** Whether or not the value has changed since its default value was given. */
   PRPackedBool             mValueChanged;
 
   NS_IMETHOD SelectAll(nsIPresContext* aPresContext);
+  /**
+   * Get the value, whether it is from the content or the frame.
+   * @param aValue the value [out]
+   * @param aIgnoreWrap whether to ignore the wrap attribute when getting the
+   *        value.  If this is true, linebreaks will not be inserted even if
+   *        wrap=hard.
+   */
+  void GetValueInternal(nsAString& aValue, PRBool aIgnoreWrap);
 };
 
 nsresult
@@ -179,7 +194,7 @@ NS_NewHTMLTextAreaElement(nsIHTMLContent** aInstancePtrResult,
 
 nsHTMLTextAreaElement::nsHTMLTextAreaElement()
 {
-  mValue = 0;
+  mValue = nsnull;
   mValueChanged = PR_FALSE;
 }
 
@@ -200,6 +215,7 @@ NS_HTML_CONTENT_INTERFACE_MAP_BEGIN(nsHTMLTextAreaElement,
   NS_INTERFACE_MAP_ENTRY(nsIDOMHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSHTMLTextAreaElement)
   NS_INTERFACE_MAP_ENTRY(nsITextControlElement)
+  NS_INTERFACE_MAP_ENTRY(nsITextAreaElement)
   NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(HTMLTextAreaElement)
 NS_HTML_CONTENT_INTERFACE_MAP_END
 
@@ -263,7 +279,7 @@ nsHTMLTextAreaElement::SetFocus(nsIPresContext* aPresContext)
   nsAutoString disabled;
 
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_HTML,
+      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_None,
                                                  nsHTMLAtoms::disabled,
                                                  disabled)) {
     return NS_OK;
@@ -328,7 +344,7 @@ nsHTMLTextAreaElement::Select()
   // first see if we are disabled or not. If disabled then do nothing.
   nsAutoString disabled;
   if (NS_CONTENT_ATTR_HAS_VALUE ==
-      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_HTML,
+      nsGenericHTMLContainerFormElement::GetAttr(kNameSpaceID_None,
                                                  nsHTMLAtoms::disabled,
                                                  disabled)) {
     return rv;
@@ -396,7 +412,7 @@ NS_IMPL_INT_ATTR(nsHTMLTextAreaElement, TabIndex, tabindex)
   
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::GetType(nsAWritableString& aType)
+nsHTMLTextAreaElement::GetType(nsAString& aType)
 {
   aType.Assign(NS_LITERAL_STRING("textarea"));
 
@@ -404,25 +420,33 @@ nsHTMLTextAreaElement::GetType(nsAWritableString& aType)
 }
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::GetValue(nsAWritableString& aValue)
+nsHTMLTextAreaElement::GetValue(nsAString& aValue)
 {
+  GetValueInternal(aValue, PR_TRUE);
+  return NS_OK;
+}
+
+void
+nsHTMLTextAreaElement::GetValueInternal(nsAString& aValue, PRBool aIgnoreWrap)
+{
+  // Get the frame.
   // No need to flush here, if there is no frame yet for this textarea
   // there won't be a value in it we don't already have even if we
   // force the frame to be created.
-  nsIFormControlFrame* formControlFrame = GetFormControlFrame(PR_FALSE);
-
-  nsIGfxTextControlFrame2* textControlFrame = nsnull;
-  if (formControlFrame) {
-    CallQueryInterface(formControlFrame, &textControlFrame);
+  nsIFrame* primaryFrame = GetPrimaryFrame(PR_FALSE);
+  nsITextControlFrame* textControlFrame = nsnull;
+  if (primaryFrame) {
+    CallQueryInterface(primaryFrame, &textControlFrame);
   }
 
+  // If the frame exists and owns the value, get it from the frame.  Otherwise
+  // get it from content.
   PRBool frameOwnsValue = PR_FALSE;
   if (textControlFrame) {
     textControlFrame->OwnsValue(&frameOwnsValue);
   }
   if (frameOwnsValue) {
-    formControlFrame->GetProperty(nsHTMLAtoms::value, aValue);
-    return NS_OK;
+    textControlFrame->GetValue(aValue, aIgnoreWrap);
   } else {
     if (!mValueChanged || !mValue) {
       GetDefaultValue(aValue);
@@ -430,15 +454,14 @@ nsHTMLTextAreaElement::GetValue(nsAWritableString& aValue)
       aValue = NS_ConvertUTF8toUCS2(mValue);
     }
   }
-  return NS_OK;
 }
 
 
 NS_IMETHODIMP
 nsHTMLTextAreaElement::SetValueGuaranteed(const nsAString& aValue,
-                                          nsIGfxTextControlFrame2* aFrame)
+                                          nsITextControlFrame* aFrame)
 {
-  nsIGfxTextControlFrame2* textControlFrame = aFrame;
+  nsITextControlFrame* textControlFrame = aFrame;
   nsIFormControlFrame* formControlFrame = textControlFrame;
   if (!textControlFrame) {
     // No need to flush here, if there is no frame for this yet forcing
@@ -474,7 +497,7 @@ nsHTMLTextAreaElement::SetValueGuaranteed(const nsAString& aValue,
 }
 
 NS_IMETHODIMP 
-nsHTMLTextAreaElement::SetValue(const nsAReadableString& aValue)
+nsHTMLTextAreaElement::SetValue(const nsAString& aValue)
 {
   return SetValueGuaranteed(aValue, nsnull);
 }
@@ -492,75 +515,15 @@ nsHTMLTextAreaElement::SetValueChanged(PRBool aValueChanged)
 }
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::GetDefaultValue(nsAWritableString& aDefaultValue)
+nsHTMLTextAreaElement::GetDefaultValue(nsAString& aDefaultValue)
 {
-  nsresult rv;
-  PRInt32 nChildren, i;
-
-  nsAutoString defVal;
-
-  ChildCount(nChildren);
-  for (i = 0; i < nChildren; i++) {
-    nsCOMPtr<nsIContent> child;
-    nsCOMPtr<nsIDOMText> textNode;
-
-    rv = ChildAt(i, *getter_AddRefs(child));
-    NS_ENSURE_SUCCESS(rv, rv);
-    textNode = do_QueryInterface(child);
-    if(textNode) {
-      nsAutoString tmp;
-      textNode->GetData(tmp);
-      defVal.Append(tmp);
-    }
-  }
-
-  aDefaultValue.Assign(defVal);
-  
-  return NS_OK;
+  return GetContentsAsText(aDefaultValue);
 }  
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::SetDefaultValue(const nsAReadableString& aDefaultValue)
+nsHTMLTextAreaElement::SetDefaultValue(const nsAString& aDefaultValue)
 {
-  nsresult rv;
-  PRInt32 nChildren, i;
-  PRBool firstChildUsed = PR_FALSE;
-
-  ChildCount(nChildren);
-  // If a child exist we try to reuse it
-  if (nChildren > 0) {
-    nsCOMPtr<nsIContent> child;
-    nsCOMPtr<nsIDOMText> textNode;
-
-    rv = ChildAt(0, *getter_AddRefs(child));
-    NS_ENSURE_SUCCESS(rv, rv);
-    textNode = do_QueryInterface(child);
-    if(textNode) {
-      rv = textNode->SetData(aDefaultValue);
-      NS_ENSURE_SUCCESS(rv, rv);
-      firstChildUsed = PR_TRUE;
-    }
-  }
-  
-  PRInt32 lastChild = firstChildUsed ? 1 : 0;
-  for (i = nChildren-1; i >= lastChild; i--) {
-    RemoveChildAt(i, PR_TRUE);
-  }
-  
-  if (!firstChildUsed) {
-    nsCOMPtr<nsIContent> textContent;
-    rv = NS_NewTextNode(getter_AddRefs(textContent));
-    NS_ENSURE_SUCCESS(rv, rv);
-      
-    nsCOMPtr<nsIDOMText> textNode;
-    textNode = do_QueryInterface(textContent);
-    rv = textNode->SetData(aDefaultValue);
-    NS_ENSURE_SUCCESS(rv, rv);
-    
-    AppendChildTo(textContent, PR_TRUE, PR_TRUE);
-  }
-  
-  return NS_OK;
+  return ReplaceContentsWithText(aDefaultValue, PR_TRUE);
 }
 
 NS_IMETHODIMP
@@ -615,7 +578,7 @@ nsHTMLTextAreaElement::RemoveChildAt(PRInt32 aIndex, PRBool aNotify)
 
 NS_IMETHODIMP
 nsHTMLTextAreaElement::StringToAttribute(nsIAtom* aAttribute,
-                                         const nsAReadableString& aValue,
+                                         const nsAString& aValue,
                                          nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::disabled) {
@@ -662,7 +625,7 @@ nsHTMLTextAreaElement::GetMappedAttributeImpact(const nsIAtom* aAttribute, PRInt
 {
   // XXX Bug 50280 - It is unclear why we need to do this here for 
   // rows and cols and why the AttributeChanged method in
-  // nsGfxTextControlFrame2 does take care of the entire problem, but
+  // nsTextControlFrame does take care of the entire problem, but
   // it doesn't and this makes things better
   if (aAttribute == nsHTMLAtoms::align ||
       aAttribute == nsHTMLAtoms::rows ||
@@ -792,6 +755,14 @@ nsHTMLTextAreaElement::HandleDOMEvent(nsIPresContext* aPresContext,
   return rv;
 }
 
+// nsITextAreaElement
+NS_IMETHODIMP
+nsHTMLTextAreaElement::DoneAddingChildren()
+{
+  RestoreFormControlState(this, this);
+  return NS_OK;
+}
+
 // nsIFormControl
 
 NS_IMETHODIMP
@@ -805,6 +776,19 @@ nsHTMLTextAreaElement::GetType(PRInt32* aType)
   }
 }
 
+nsresult
+nsHTMLTextAreaElement::GetInnerHTML(nsAString& aInnerHTML)
+{
+  return GetContentsAsText(aInnerHTML);
+}
+
+nsresult
+nsHTMLTextAreaElement::SetInnerHTML(const nsAString& aInnerHTML)
+{
+  return ReplaceContentsWithText(aInnerHTML, PR_TRUE);
+}
+
+
 #ifdef DEBUG
 NS_IMETHODIMP
 nsHTMLTextAreaElement::SizeOf(nsISizeOfHandler* aSizer,
@@ -856,6 +840,78 @@ nsHTMLTextAreaElement::GetControllers(nsIControllers** aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetTextLength(PRInt32 *aTextLength)
+{
+  NS_ENSURE_ARG_POINTER(aTextLength);
+  nsAutoString val;
+  nsresult rv = GetValue(val);
+  *aTextLength = val.Length();
+
+  return rv;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetSelectionStart(PRInt32 *aSelectionStart)
+{
+  NS_ENSURE_ARG_POINTER(aSelectionStart);
+  nsCOMPtr<nsIFormControlFrame> formControlFrame = getter_AddRefs(GetFormControlFrame(PR_TRUE));
+
+  nsCOMPtr<nsITextControlFrame>
+    textControlFrame(do_QueryInterface(formControlFrame));
+    
+  if (textControlFrame) {
+    PRInt32 selectionEnd;
+    return textControlFrame->GetSelectionRange(aSelectionStart, &selectionEnd);
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SetSelectionStart(PRInt32 aSelectionStart)
+{
+  nsCOMPtr<nsIFormControlFrame> formControlFrame = getter_AddRefs(GetFormControlFrame(PR_TRUE));
+
+  nsCOMPtr<nsITextControlFrame>
+    textControlFrame(do_QueryInterface(formControlFrame));
+
+  if (textControlFrame)
+    textControlFrame->SetSelectionStart(aSelectionStart);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::GetSelectionEnd(PRInt32 *aSelectionEnd)
+{
+  NS_ENSURE_ARG_POINTER(aSelectionEnd);
+  nsCOMPtr<nsIFormControlFrame> formControlFrame = getter_AddRefs(GetFormControlFrame(PR_TRUE));
+
+  nsCOMPtr<nsITextControlFrame>
+    textControlFrame(do_QueryInterface(formControlFrame));
+    
+  if (textControlFrame) {
+    PRInt32 selectionStart;
+    return textControlFrame->GetSelectionRange(&selectionStart, aSelectionEnd);
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SetSelectionEnd(PRInt32 aSelectionEnd)
+{
+  nsCOMPtr<nsIFormControlFrame> formControlFrame = getter_AddRefs(GetFormControlFrame(PR_TRUE));
+
+  nsCOMPtr<nsITextControlFrame>
+    textControlFrame(do_QueryInterface(formControlFrame));
+
+  if (textControlFrame)
+    textControlFrame->SetSelectionEnd(aSelectionEnd);
+
+  return NS_OK;
+}
 
 nsresult
 nsHTMLTextAreaElement::Reset()
@@ -874,80 +930,73 @@ nsHTMLTextAreaElement::Reset()
   return NS_OK;
 }
 
-nsresult
-nsHTMLTextAreaElement::IsSuccessful(nsIContent* aSubmitElement,
-                                    PRBool *_retval)
+NS_IMETHODIMP
+nsHTMLTextAreaElement::SubmitNamesValues(nsIFormSubmission* aFormSubmission,
+                                         nsIContent* aSubmitElement)
 {
-  // if it's disabled, it won't submit
+  nsresult rv = NS_OK;
+
+  //
+  // Disabled elements don't submit
+  //
   PRBool disabled;
-  nsresult rv = GetDisabled(&disabled);
-  if (disabled) {
-    *_retval = PR_FALSE;
-    return NS_OK;
+  rv = GetDisabled(&disabled);
+  if (NS_FAILED(rv) || disabled) {
+    return rv;
   }
 
-  // if it dosn't have a name it we don't submit
-  nsAutoString val;
-  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, val);
-  *_retval = rv != NS_CONTENT_ATTR_NOT_THERE;
-  return NS_OK;
-}
-
-nsresult
-nsHTMLTextAreaElement::GetMaxNumValues(PRInt32 *_retval)
-{
-  *_retval = 1;
-  return NS_OK;
-}
-
-nsresult
-nsHTMLTextAreaElement::GetNamesValues(PRInt32 aMaxNumValues,
-                                      PRInt32& aNumValues,
-                                      nsString* aValues,
-                                      nsString* aNames)
-{
-  NS_ENSURE_TRUE(aMaxNumValues >= 1, NS_ERROR_UNEXPECTED);
-
-  // We'll of course use the name of the control for the submit
+  //
+  // Get the name (if no name, no submit)
+  //
   nsAutoString name;
-  nsresult rv = GetName(name);
-  NS_ENSURE_SUCCESS(rv, rv);
+  rv = GetAttr(kNameSpaceID_None, nsHTMLAtoms::name, name);
+  if (NS_FAILED(rv) || rv == NS_CONTENT_ATTR_NOT_THERE) {
+    return rv;
+  }
 
+  //
+  // Get the value
+  //
   nsAutoString value;
-  rv = GetValue(value);
-  NS_ENSURE_SUCCESS(rv, rv);
+  GetValueInternal(value, PR_FALSE);
 
-  aNames[0] = name;
-  aValues[0] = value;
-  aNumValues = 1;
-  return NS_OK;
+  //
+  // Submit
+  //
+  rv = aFormSubmission->AddNameValuePair(this, name, value);
+
+  return rv;
 }
 
+
 NS_IMETHODIMP
-nsHTMLTextAreaElement::SaveState(nsIPresContext* aPresContext,
-                                 nsIPresState** aState)
+nsHTMLTextAreaElement::SaveState()
 {
-  nsresult rv = GetPrimaryPresState(this, aState);
-  if (*aState) {
-    nsString value;
-    GetValue(value);
-    // XXX Should use nsAutoString above but ConvertStringLineBreaks requires
-    // mOwnsBuffer!
-    rv = nsLinebreakConverter::ConvertStringLineBreaks(
-             value,
-             nsLinebreakConverter::eLinebreakPlatform,
-             nsLinebreakConverter::eLinebreakContent);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Converting linebreaks failed!");
-    rv = (*aState)->SetStateProperty(NS_LITERAL_STRING("value"), value);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "value save failed!");
+  nsresult rv = NS_OK;
+
+  // Only save if value != defaultValue (bug 62713)
+  if (mValueChanged) {
+    nsCOMPtr<nsIPresState> state;
+    rv = GetPrimaryPresState(this, getter_AddRefs(state));
+    if (state) {
+      nsAutoString value;
+      GetValueInternal(value, PR_TRUE);
+
+      rv = nsLinebreakConverter::ConvertStringLineBreaks(
+               value,
+               nsLinebreakConverter::eLinebreakPlatform,
+               nsLinebreakConverter::eLinebreakContent);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "Converting linebreaks failed!");
+      rv = state->SetStateProperty(NS_LITERAL_STRING("value"), value);
+      NS_ASSERTION(NS_SUCCEEDED(rv), "value save failed!");
+    }
   }
 
   return rv;
 }
 
 NS_IMETHODIMP
-nsHTMLTextAreaElement::RestoreState(nsIPresContext* aPresContext,
-                                    nsIPresState* aState)
+nsHTMLTextAreaElement::RestoreState(nsIPresState* aState)
 {
   nsresult rv = NS_OK;
 
