@@ -89,7 +89,7 @@ public:
   NS_IMETHOD AddLeaf(const nsIParserNode& aNode);
   NS_IMETHOD AddComment(const nsIParserNode& aNode);
   NS_IMETHOD AddProcessingInstruction(const nsIParserNode& aNode);
-  NS_IMETHOD AddDocTypeDecl(const nsIParserNode& aNode, PRInt32 aMode=0);
+  NS_IMETHOD AddDocTypeDecl(const nsIParserNode& aNode);
 
   // nsIHTMLContentSink
   NS_IMETHOD BeginContext(PRInt32 aID);
@@ -108,10 +108,12 @@ public:
   NS_IMETHOD OpenNoscript(const nsIParserNode& aNode);
   NS_IMETHOD CloseNoscript(const nsIParserNode& aNode);
   NS_IMETHOD GetPref(PRInt32 aTag,PRBool& aPref) { return NS_OK; }
+  NS_IMETHOD_(PRBool) IsFormOnStack() { return PR_FALSE; }
+  
   NS_IMETHOD OpenMap(const nsIParserNode& aNode);
   NS_IMETHOD CloseMap(const nsIParserNode& aNode);
   NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-  NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset) { return NS_OK; }
+  NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
   NS_IMETHOD WillProcessTokens(void) { return NS_OK; }
   NS_IMETHOD DidProcessTokens(void) { return NS_OK; }
   NS_IMETHOD WillProcessAToken(void) { return NS_OK; }
@@ -133,7 +135,7 @@ public:
   nsresult AddAttributes(const nsIParserNode& aNode,
                          nsIContent* aContent);
 
-  nsresult AddText(const nsAReadableString& aString);
+  nsresult AddText(const nsAString& aString);
   nsresult AddTextToContent(nsIHTMLContent* aContent,const nsString& aText);
   nsresult FlushText();
 
@@ -472,10 +474,10 @@ void
 nsHTMLFragmentContentSink::AddBaseTagInfo(nsIHTMLContent* aContent)
 {
   if (aContent) {
-    if (mBaseHREF.Length() > 0) {
+    if (!mBaseHREF.IsEmpty()) {
       aContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::_baseHref, mBaseHREF, PR_FALSE);
     }
-    if (mBaseTarget.Length() > 0) {
+    if (!mBaseTarget.IsEmpty()) {
       aContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::_baseTarget, mBaseTarget, PR_FALSE);
     }
   }
@@ -629,7 +631,15 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
              nodeType == eHTMLTag_xmp) {
             
             // Create a text node holding the content
-            result=AddTextToContent(content,aNode.GetSkippedContent());
+            nsCOMPtr<nsIDTD> dtd;
+            mParser->GetDTD(getter_AddRefs(dtd));
+            NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+            nsAutoString skippedContent;
+            PRInt32 lineNo = 0;
+
+            dtd->CollectSkippedContent(nodeType, skippedContent, lineNo);
+            result=AddTextToContent(content, skippedContent);
           }
           else if (nodeType == eHTMLTag_img || nodeType == eHTMLTag_frame
               || nodeType == eHTMLTag_input)    // elements with 'SRC='
@@ -705,7 +715,7 @@ nsHTMLFragmentContentSink::AddProcessingInstruction(const nsIParserNode& aNode)
  */
 
 NS_IMETHODIMP
-nsHTMLFragmentContentSink::AddDocTypeDecl(const nsIParserNode& aNode, PRInt32 aMode)
+nsHTMLFragmentContentSink::AddDocTypeDecl(const nsIParserNode& aNode)
 {
   return NS_OK;
 }
@@ -767,7 +777,7 @@ nsHTMLFragmentContentSink::PopContent()
 #define NS_ACCUMULATION_BUFFER_SIZE 4096
 
 nsresult
-nsHTMLFragmentContentSink::AddText(const nsAReadableString& aString)
+nsHTMLFragmentContentSink::AddText(const nsAString& aString)
 {
   PRInt32 addLen = aString.Length();
   if (0 == addLen) {
@@ -817,7 +827,7 @@ nsHTMLFragmentContentSink::AddTextToContent(nsIHTMLContent* aContent,const nsStr
   nsresult result=NS_OK;
   
   if(aContent) {
-    if (aText.Length() > 0) {
+    if (!aText.IsEmpty()) {
       nsCOMPtr<nsIContent> text;
       result = NS_NewTextNode(getter_AddRefs(text));
       if (NS_SUCCEEDED(result)) {
@@ -872,12 +882,12 @@ nsHTMLFragmentContentSink::GetAttributeValueAt(const nsIParserNode& aNode,
                                                nsString& aResult)
 {
   // Copy value
-  const nsString& value = aNode.GetValueAt(aIndex);
+  const nsAString& value = aNode.GetValueAt(aIndex);
   aResult.Truncate();
   aResult.Append(value);
 
   // Strip quotes if present
-  if (aResult.Length() > 0) {
+  if (!aResult.IsEmpty()) {
     PRUnichar first = aResult.First();
     if ((first == '\"') || (first == '\'')) {
       if (aResult.Last() == first) {
@@ -903,19 +913,19 @@ nsHTMLFragmentContentSink::AddAttributes(const nsIParserNode& aNode,
   PRInt32 ac = aNode.GetAttributeCount();
   for (PRInt32 i = 0; i < ac; i++) {
     // Get upper-cased key
-    const nsAReadableString& key = aNode.GetKeyAt(i);
+    const nsAString& key = aNode.GetKeyAt(i);
     k.Assign(key);
     ToLowerCase(k);
 
     nsIAtom*  keyAtom = NS_NewAtom(k);
     
     if (NS_CONTENT_ATTR_NOT_THERE == 
-        aContent->GetAttr(kNameSpaceID_HTML, keyAtom, v)) {
+        aContent->GetAttr(kNameSpaceID_None, keyAtom, v)) {
       // Get value and remove mandatory quotes
       GetAttributeValueAt(aNode, i, v);
 
       // Add attribute to content
-      aContent->SetAttr(kNameSpaceID_HTML, keyAtom, v, PR_FALSE);
+      aContent->SetAttr(kNameSpaceID_None, keyAtom, v, PR_FALSE);
     }
     NS_RELEASE(keyAtom);
   }
