@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *    Morten Nilsen <morten@nilsen.com>
+ *    Christian Biesinger <cbiesinger@web.de>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -224,9 +225,20 @@ NS_EXPORT nsresult
 NS_NewImageDocument(nsIDocument** aInstancePtrResult)
 {
   nsImageDocument* doc = new nsImageDocument();
-  if(doc)
-    return doc->QueryInterface(NS_GET_IID(nsIDocument), (void**) aInstancePtrResult);
-  return NS_ERROR_OUT_OF_MEMORY;
+  NS_ENSURE_TRUE(doc, NS_ERROR_OUT_OF_MEMORY);
+
+  nsresult rv = doc->Init();
+
+  if (NS_FAILED(rv)) {
+    delete doc;
+
+    return rv;
+  }
+
+  *aInstancePtrResult = doc;
+  NS_ADDREF(*aInstancePtrResult);
+
+  return NS_OK;
 }
 
 nsImageDocument::nsImageDocument()
@@ -257,16 +269,23 @@ nsImageDocument::StartDocumentLoad(const char* aCommand,
   NS_ENSURE_ARG_POINTER(aContainer);
   mContainer = dont_AddRef(NS_GetWeakReference(aContainer));
 
-  nsresult rv = Init();
-
-  if (NS_FAILED(rv) && rv != NS_ERROR_ALREADY_INITIALIZED) {
+  nsresult rv = nsDocument::StartDocumentLoad(aCommand, aChannel, aLoadGroup,
+                                              aContainer, aDocListener, aReset,
+                                              aSink);
+  if (NS_FAILED(rv)) {
     return rv;
   }
 
-  rv = nsDocument::StartDocumentLoad(aCommand, aChannel, aLoadGroup,
-                                     aContainer, aDocListener, aReset, aSink);
-  if (NS_FAILED(rv)) {
-    return rv;
+  nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+  if (httpChannel) {
+    // The misspelled key 'referer' is as per the HTTP spec
+    nsCAutoString referrerHeader;
+    rv = httpChannel->GetRequestHeader(NS_LITERAL_CSTRING("referer"),
+                                       referrerHeader);
+    
+    if (NS_SUCCEEDED(rv)) {
+      SetReferrer(NS_ConvertASCIItoUCS2(referrerHeader));
+    }
   }
 
   // Create synthetic document
@@ -385,14 +404,28 @@ nsImageDocument::CreateSyntheticDocument()
   }
   image->SetDocument(this, PR_FALSE, PR_TRUE);
 
-  nsXPIDLCString src;
-  mDocumentURL->GetSpec(getter_Copies(src));
+  nsCAutoString src;
+  mDocumentURL->GetSpec(src);
 
-  nsString src_string; src_string.AssignWithConversion(src);
+  NS_ConvertUTF8toUCS2 src_string(src);
   nsHTMLValue val(src_string);
 
   image->SetHTMLAttribute(nsHTMLAtoms::src, val, PR_FALSE);
-  image->SetHTMLAttribute(nsHTMLAtoms::alt, val, PR_FALSE);
+
+  // Create a stringbundle for localized error message
+  nsCOMPtr<nsIStringBundle> bundle;
+  nsCOMPtr<nsIStringBundleService> stringService =
+           do_GetService(kStringBundleServiceCID, &rv);
+  if (NS_SUCCEEDED(rv) && stringService)
+    rv = stringService->CreateBundle(NSIMAGEDOCUMENT_PROPERTIES_URI, getter_AddRefs(bundle));
+  if (NS_SUCCEEDED(rv) && bundle) {
+    const PRUnichar* formatString[1] = { src_string.get() };
+    nsXPIDLString errorMsg;
+    rv = bundle->FormatStringFromName(NS_LITERAL_STRING("InvalidImage").get(), formatString, 1, getter_Copies(errorMsg));
+
+    nsHTMLValue errorText(errorMsg);
+    image->SetHTMLAttribute(nsHTMLAtoms::alt, errorText, PR_FALSE);
+  }
 
   root->AppendChildTo(body, PR_FALSE, PR_FALSE);
   center->AppendChildTo(image, PR_FALSE, PR_FALSE);
@@ -495,15 +528,14 @@ nsresult nsImageDocument::UpdateTitle( void )
     nsAutoString widthStr;
     nsAutoString heightStr;
     nsXPIDLString fileStr;
+    nsAutoString typeStr;
     PRUint32 width = 0, height = 0;
 
     nsCOMPtr<nsIURL> url = do_QueryInterface(mDocumentURL);
     if (url) {
-      nsXPIDLCString pName;
-      url->GetFileName(getter_Copies(pName));
-      if(pName){
-        fileStr.Assign(NS_ConvertUTF8toUCS2(pName));
-      }
+      nsCAutoString pName;
+      url->GetFileName(pName);
+      fileStr.Assign(NS_ConvertUTF8toUCS2(pName));
     }
 
     if (mImageRequest) {
@@ -524,30 +556,57 @@ nsresult nsImageDocument::UpdateTitle( void )
 
       widthStr.AppendInt(width);
       heightStr.AppendInt(height);
+
+      nsXPIDLCString mimeType;
+      mImageRequest->GetMimeType(getter_Copies(mimeType));
+      ToUpperCase(mimeType);
+      nsXPIDLCString::const_iterator start, end;
+      mimeType.BeginReading(start);
+      mimeType.EndReading(end);
+      nsXPIDLCString::const_iterator iter = end;
+      if (FindInReadable(NS_LITERAL_CSTRING("IMAGE/"), start, iter) && iter != end) {
+        // strip out "X-" if any
+        if (*iter == 'X') {
+          ++iter;
+          if (iter != end && *iter == '-') {
+            ++iter;
+            if (iter == end) {
+              // looks like "IMAGE/X-" is the type??  Bail out of here.
+              mimeType.BeginReading(iter);
+            }
+          } else {
+            --iter;
+          }
+        }
+        CopyASCIItoUCS2(Substring(iter, end), typeStr);
+      } else {
+        CopyASCIItoUCS2(mimeType, typeStr);
+      }
     }
     // If we got a filename, display it
-    if(!fileStr.IsEmpty()) {
+    if (!fileStr.IsEmpty()) {
       // if we got a valid size (sometimes we do not) then display it
       if (width != 0 && height != 0){
-        const PRUnichar *formatStrings[3]  = {fileStr.get(), widthStr.get(), heightStr.get()};
-        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensionsAndFile").get(), formatStrings, 3, getter_Copies(valUni));
+        const PRUnichar *formatStrings[4]  = {fileStr.get(), typeStr.get(), widthStr.get(), heightStr.get()};
+        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensionsAndFile").get(), formatStrings, 4, getter_Copies(valUni));
       } else {
-        const PRUnichar *formatStrings[1] = {fileStr.get()};
-        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithoutDimensions").get(), formatStrings, 1, getter_Copies(valUni));
+        const PRUnichar *formatStrings[2] = {fileStr.get(), typeStr.get()};
+        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithoutDimensions").get(), formatStrings, 2, getter_Copies(valUni));
       }
     } else {
       // if we got a valid size (sometimes we do not) then display it
       if (width != 0 && height != 0){
-        const PRUnichar *formatStrings[2]  = {widthStr.get(), heightStr.get()};
-        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensions").get(), formatStrings, 2, getter_Copies(valUni));
+        const PRUnichar *formatStrings[3]  = {typeStr.get(), widthStr.get(), heightStr.get()};
+        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithDimensions").get(), formatStrings, 3, getter_Copies(valUni));
       } else {
-        rv = bundle->GetStringFromName(NS_LITERAL_STRING("ImageTitleWithoutDimensionsAndFile").get(), getter_Copies(valUni));
+        const PRUnichar *formatStrings[1]  = {typeStr.get()};
+        rv = bundle->FormatStringFromName(NS_LITERAL_STRING("ImageTitleWithNeitherDimensionsNorFile").get(), formatStrings, 1, getter_Copies(valUni));
       }
     }
 
     if (NS_SUCCEEDED(rv) && valUni) {
       // set it on the document
-      SetTitle(nsDependentString(valUni));
+      SetTitle(valUni);
     }
   }
   return NS_OK;
