@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Daniel Glazman <glazman@netscape.com>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -125,7 +126,9 @@ struct nsCSSFont : public nsCSSStruct {
   nsCSSFont(const nsCSSFont& aCopy);
   ~nsCSSFont(void);
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mFamily;
   nsCSSValue mStyle;
@@ -150,7 +153,9 @@ struct nsCSSColor : public nsCSSStruct  {
   nsCSSColor(const nsCSSColor& aCopy);
   ~nsCSSColor(void);
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue      mColor;
   nsCSSValue      mBackColor;
@@ -179,7 +184,9 @@ struct nsCSSText : public nsCSSStruct  {
   ~nsCSSText(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mWordSpacing;
   nsCSSValue mLetterSpacing;
@@ -198,8 +205,10 @@ struct nsCSSRect {
   nsCSSRect(void);
   nsCSSRect(const nsCSSRect& aCopy);
   ~nsCSSRect();
+#ifdef DEBUG
   void List(FILE* out = 0, nsCSSProperty aPropID = eCSSProperty_UNKNOWN, PRInt32 aIndent = 0) const;
   void List(FILE* out, PRInt32 aIndent, const nsCSSProperty aTRBL[]) const;
+#endif
 
   nsCSSValue mTop;
   nsCSSValue mRight;
@@ -213,7 +222,9 @@ struct nsCSSDisplay : public nsCSSStruct  {
   ~nsCSSDisplay(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mDirection;
   nsCSSValue mDisplay;
@@ -230,6 +241,11 @@ struct nsCSSDisplay : public nsCSSStruct  {
   // in nsCSSDeclaration objects but because it's needed on the
   // stack when the struct is used in WalkRuleTree.
   nsCSSValue mLang;
+
+  // temp fix for bug 24000 
+  nsCSSValue mBreakBefore;
+  nsCSSValue mBreakAfter;
+  // end temp fix
 };
 
 struct nsCSSMargin : public nsCSSStruct  {
@@ -238,7 +254,9 @@ struct nsCSSMargin : public nsCSSStruct  {
   ~nsCSSMargin(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   void EnsureBorderColors();
 
@@ -262,7 +280,9 @@ struct nsCSSPosition : public nsCSSStruct  {
   ~nsCSSPosition(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue  mWidth;
   nsCSSValue  mMinWidth;
@@ -281,7 +301,9 @@ struct nsCSSList : public nsCSSStruct  {
   ~nsCSSList(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mType;
   nsCSSValue mImage;
@@ -295,7 +317,9 @@ struct nsCSSTable : public nsCSSStruct  { // NEW
   ~nsCSSTable(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mBorderCollapse;
   nsCSSValue mBorderSpacingX;
@@ -316,7 +340,9 @@ struct nsCSSBreaks : public nsCSSStruct  { // NEW
   ~nsCSSBreaks(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mOrphans;
   nsCSSValue mWidows;
@@ -332,7 +358,9 @@ struct nsCSSPage : public nsCSSStruct  { // NEW
   ~nsCSSPage(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mMarks;
   nsCSSValue mSizeWidth;
@@ -365,7 +393,9 @@ struct nsCSSContent : public nsCSSStruct  {
   ~nsCSSContent(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValueList*   mContent;
   nsCSSCounterData* mCounterIncrement;
@@ -380,7 +410,9 @@ struct nsCSSUserInterface : public nsCSSStruct  { // NEW
   ~nsCSSUserInterface(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue      mUserInput;
   nsCSSValue      mUserModify;
@@ -390,6 +422,7 @@ struct nsCSSUserInterface : public nsCSSStruct  { // NEW
   nsCSSValue      mResizer;
   
   nsCSSValueList* mCursor;
+  nsCSSValue      mForceBrokenImageIcon;
 };
 
 struct nsCSSAural : public nsCSSStruct  { // NEW
@@ -398,7 +431,9 @@ struct nsCSSAural : public nsCSSStruct  { // NEW
   ~nsCSSAural(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mAzimuth;
   nsCSSValue mElevation;
@@ -428,7 +463,9 @@ struct nsCSSXUL : public nsCSSStruct  {
   ~nsCSSXUL(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue  mBoxAlign;
   nsCSSValue  mBoxDirection;
@@ -446,7 +483,9 @@ struct nsCSSSVG : public nsCSSStruct {
   ~nsCSSSVG(void);
 
   const nsID& GetID(void);
+#ifdef DEBUG
   void List(FILE* out = stdout, PRInt32 aIndent = 0) const;
+#endif
 
   nsCSSValue mFill;
   nsCSSValue mFillOpacity;
@@ -539,23 +578,23 @@ public:
   nsresult AppendValue(nsCSSProperty aProperty, const nsCSSValue& aValue);
   nsresult AppendStructValue(nsCSSProperty aProperty, void* aStruct);
   nsresult SetValueImportant(nsCSSProperty aProperty);
-  nsresult AppendComment(const nsAReadableString& aComment);
+  nsresult AppendComment(const nsAString& aComment);
   nsresult RemoveProperty(nsCSSProperty aProperty, nsCSSValue& aValue);
 
   nsresult GetValue(nsCSSProperty aProperty, nsCSSValue& aValue);
-  nsresult GetValue(nsCSSProperty aProperty, nsAWritableString& aValue);
-  nsresult GetValue(const nsAReadableString& aProperty, nsAWritableString& aValue);
+  nsresult GetValue(nsCSSProperty aProperty, nsAString& aValue);
+  nsresult GetValue(const nsAString& aProperty, nsAString& aValue);
 
   nsCSSDeclaration* GetImportantValues();
   PRBool GetValueIsImportant(nsCSSProperty aProperty);
-  PRBool GetValueIsImportant(const nsAReadableString& aProperty);
+  PRBool GetValueIsImportant(const nsAString& aProperty);
 
   PRUint32 Count();
-  nsresult GetNthProperty(PRUint32 aIndex, nsAWritableString& aReturn);
+  nsresult GetNthProperty(PRUint32 aIndex, nsAString& aReturn);
 
   PRInt32 GetStyleImpact() const;
 
-  nsresult ToString(nsAWritableString& aString);
+  nsresult ToString(nsAString& aString);
 
   nsCSSDeclaration* Clone() const;
 
@@ -569,46 +608,57 @@ protected:
   nsresult RemoveProperty(nsCSSProperty aProperty);
 
 private:
-  PRBool   AppendValueToString(nsCSSProperty aProperty, nsAWritableString& aResult);
-  PRBool   AppendValueToString(nsCSSProperty aProperty, const nsCSSValue& aValue, nsAWritableString& aResult);
+  nsresult GetValueOrImportantValue(nsCSSProperty aProperty, nsCSSValue& aValue);
+  void     AppendImportanceToString(PRBool aIsImportant, nsAString& aString);
+  PRBool   AppendValueToString(nsCSSProperty aProperty, nsAString& aResult);
+  PRBool   AppendValueOrImportantValueToString(nsCSSProperty aProperty, nsAString& aResult);
+  PRBool   AppendValueToString(nsCSSProperty aProperty, const nsCSSValue& aValue, nsAString& aResult);
   nsCSSDeclaration& operator=(const nsCSSDeclaration& aCopy);
   PRBool operator==(const nsCSSDeclaration& aCopy) const;
 
-  void TryBorderShorthand(nsAWritableString & aString,
-                          PRInt32 & aBorderTopWidth,
-                          PRInt32 & aBorderTopStyle,
-                          PRInt32 & aBorderTopColor,
-                          PRInt32 & aBorderBottomWidth,
-                          PRInt32 & aBorderBottomStyle,
-                          PRInt32 & aBorderBottomColor,
-                          PRInt32 & aBorderLeftWidth,
-                          PRInt32 & aBorderLeftStyle,
-                          PRInt32 & aBorderLeftColor,
-                          PRInt32 & aBorderRightWidth,
-                          PRInt32 & aBorderRightStyle,
-                          PRInt32 & aBorderRightColor);
-  void  TryBorderSideShorthand(nsAWritableString & aString,
-                               nsCSSProperty  aShorthand,
-                               PRInt32 & aBorderWidth,
-                               PRInt32 & aBorderStyle,
-                               PRInt32 & aBorderColor);
-  void  TryMarginOrPaddingShorthand(nsAWritableString & aString,
-                                    nsCSSProperty aShorthand,
-                                    PRInt32 & aTop, PRInt32 & aBottom,
-                                    PRInt32 & aLeft, PRInt32 & aRight);
-  void  TryBackgroundShorthand(nsAWritableString & aString,
+  void   PropertyIsSet(PRInt32 & aPropertyIndex, PRInt32 aIndex, PRUint32 & aSet, PRUint32 aValue);
+  PRBool TryBorderShorthand(nsAString & aString, PRUint32 aPropertiesSet,
+                            PRInt32 aBorderTopWidth,
+                            PRInt32 aBorderTopStyle,
+                            PRInt32 aBorderTopColor,
+                            PRInt32 aBorderBottomWidth,
+                            PRInt32 aBorderBottomStyle,
+                            PRInt32 aBorderBottomColor,
+                            PRInt32 aBorderLeftWidth,
+                            PRInt32 aBorderLeftStyle,
+                            PRInt32 aBorderLeftColor,
+                            PRInt32 aBorderRightWidth,
+                            PRInt32 aBorderRightStyle,
+                            PRInt32 aBorderRightColor);
+  PRBool  TryBorderSideShorthand(nsAString & aString,
+                                 nsCSSProperty  aShorthand,
+                                 PRInt32 aBorderWidth,
+                                 PRInt32 aBorderStyle,
+                                 PRInt32 aBorderColor);
+  PRBool  TryFourSidesShorthand(nsAString & aString,
+                                nsCSSProperty aShorthand,
+                                PRInt32 & aTop,
+                                PRInt32 & aBottom,
+                                PRInt32 & aLeft,
+                                PRInt32 & aRight,
+                                PRBool aClearIndexes);
+  void  TryBackgroundShorthand(nsAString & aString,
                                PRInt32 & aBgColor, PRInt32 & aBgImage,
                                PRInt32 & aBgRepeat, PRInt32 & aBgAttachment,
                                PRInt32 & aBgPositionX,
                                PRInt32 & aBgPositionY);
-  void  TryBackgroundPosition(nsAWritableString & aString,
+  void  UseBackgroundPosition(nsAString & aString,
                               PRInt32 & aBgPositionX,
                               PRInt32 & aBgPositionY);
 
+  PRBool   AllPropertiesSameImportance(PRInt32 aFirst, PRInt32 aSecond,
+                                       PRInt32 aThird, PRInt32 aFourth,
+                                       PRInt32 aFifth, PRInt32 aSixth,
+                                       PRBool & aImportance);
   PRBool   AllPropertiesSameValue(PRInt32 aFirst, PRInt32 aSecond,
                                   PRInt32 aThird, PRInt32 aFourth);
   void     AppendPropertyAndValueToString(nsCSSProperty aProperty,
-                                          nsAWritableString& aResult);
+                                          nsAString& aResult);
 
 protected:
     //
@@ -622,7 +672,6 @@ protected:
     //
     friend class CSSStyleRuleImpl;
     void AddRef(void) {
-      NS_ASSERTION(0 <= mRuleRefs, "bad AddRef");
       mRuleRefs++;
     }
     void Release(void) {
