@@ -42,6 +42,7 @@
 #include "nsICharsetConverterManager.h"
 #include "nsIUnicodeDecoder.h"
 #include "nsICharsetAlias.h"
+#include "nsUnicharUtils.h"
 #include "nsHashtable.h"
 #include "nsIURL.h"
 #include "nsIURL.h"
@@ -55,6 +56,9 @@
 #include "nsITimelineService.h"
 #include "nsIHttpChannel.h"
 #include "nsHTMLAtoms.h"
+#include "nsIConsoleService.h"
+#include "nsIScriptError.h"
+#include "nsIStringBundle.h"
 
 #ifdef INCLUDE_XUL
 #include "nsIXULPrototypeCache.h"
@@ -64,6 +68,7 @@
 #include "nsIDOMStyleSheet.h"
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
+static NS_DEFINE_CID(kCStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
 class CSSLoaderImpl;
 
@@ -72,39 +77,28 @@ MOZ_DECL_CTOR_COUNTER(URLKey)
 class URLKey: public nsHashKey {
 public:
   URLKey(nsIURI* aURL)
-    : nsHashKey(),
-      mURL(aURL),
-      mSpec(nsnull)
+    : mURL(aURL)
   {
     MOZ_COUNT_CTOR(URLKey);
-    NS_ADDREF(mURL);
     mHashValue = 0;
 
-    mURL->GetSpec((char **)&mSpec);
-    if (mSpec) {
-      mHashValue = nsCRT::HashCode(mSpec);
+    mURL->GetSpec(mSpec);
+    if (!mSpec.IsEmpty()) {
+      mHashValue = nsCRT::HashCode(mSpec.get());
     }
   }
 
   URLKey(const URLKey& aKey)
-    : nsHashKey(),
-      mURL(aKey.mURL),
+    : mURL(aKey.mURL),
       mHashValue(aKey.mHashValue),
-      mSpec(nsnull)
+      mSpec(aKey.mSpec)
   {
     MOZ_COUNT_CTOR(URLKey);
-    NS_ADDREF(mURL);
-    if (aKey.mSpec)
-      mSpec = nsCRT::strdup(aKey.mSpec);
   }
 
   virtual ~URLKey(void)
   {
     MOZ_COUNT_DTOR(URLKey);
-    NS_RELEASE(mURL);
-    if (mSpec)
-      nsCRT::free((char *)mSpec);
-    mSpec = nsnull;
   }
 
   virtual PRUint32 HashCode(void) const
@@ -120,7 +114,7 @@ public:
     nsresult result = mURL->Equals(key->mURL, &equals);
     return (NS_SUCCEEDED(result) && equals);
 #else
-    return (nsCRT::strcasecmp(mSpec, key->mSpec) == 0);
+    return (nsCRT::strcasecmp(mSpec.get(), key->mSpec.get()) == 0);
 #endif
   }
 
@@ -129,9 +123,9 @@ public:
     return new URLKey(*this);
   }
 
-  nsIURI*   mURL;
-  PRUint32  mHashValue;
-  const char* mSpec;
+  nsCOMPtr<nsIURI>  mURL;
+  PRUint32          mHashValue;
+  nsSharableCString mSpec;
 };
 
 class SheetLoadData : public nsIStreamLoaderObserver
@@ -226,8 +220,8 @@ public:
   NS_IMETHOD DropDocumentReference(void);
 
   NS_IMETHOD SetCaseSensitive(PRBool aCaseSensitive);
-  NS_IMETHOD SetQuirkMode(PRBool aQuirkMode);
-  NS_IMETHOD SetPreferredSheet(const nsAReadableString& aTitle);
+  NS_IMETHOD SetCompatibilityMode(nsCompatibility aCompatMode);
+  NS_IMETHOD SetPreferredSheet(const nsAString& aTitle);
 
   NS_IMETHOD GetParserFor(nsICSSStyleSheet* aSheet,
                           nsICSSParser** aParser);
@@ -300,7 +294,7 @@ public:
   nsIDocument*  mDocument;  // the document we live for
 
   PRBool        mCaseSensitive; // is document CSS case sensitive
-  PRBool        mNavQuirkMode;  // should CSS be in quirk mode
+  nsCompatibility mCompatMode;
   nsString      mPreferredSheet;    // title of preferred sheet
 
   nsISupportsArray* mParsers;     // array of CSS parsers
@@ -392,12 +386,12 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader, nsIURI* aURL,
     mSheetIndex(aSheetIndex),
     mOwningElement(nsnull),
     mParserToUnblock(nsnull),
-    mDidBlockParser(PR_FALSE),
     mParentSheet(aParentSheet),
     mParentRule(aParentRule),
     mNext(nsnull),
     mParentData(nsnull),
     mPendingChildren(0),
+    mDidBlockParser(PR_FALSE),
     mIsInline(PR_FALSE),
     mIsAgent(PR_FALSE),
     mSyncLoad(PR_FALSE),
@@ -420,12 +414,12 @@ SheetLoadData::SheetLoadData(CSSLoaderImpl* aLoader, nsIURI* aURL,
     mSheetIndex(-1),
     mOwningElement(nsnull),
     mParserToUnblock(nsnull),
-    mDidBlockParser(PR_FALSE),
     mParentSheet(nsnull),
     mParentRule(nsnull),
     mNext(nsnull),
     mParentData(nsnull),
     mPendingChildren(0),
+    mDidBlockParser(PR_FALSE),
     mIsInline(PR_FALSE),
     mIsAgent(PR_TRUE),
     mSyncLoad(PRBool(nsnull == aObserver)),
@@ -455,7 +449,7 @@ CSSLoaderImpl::CSSLoaderImpl(void)
   NS_INIT_REFCNT();
   mDocument = nsnull;
   mCaseSensitive = PR_FALSE;
-  mNavQuirkMode = PR_FALSE;
+  mCompatMode = eCompatibility_FullStandards;
   mParsers = nsnull;
   SetCharset(NS_LITERAL_STRING(""));
 }
@@ -536,14 +530,14 @@ CSSLoaderImpl::SetCaseSensitive(PRBool aCaseSensitive)
 }
 
 NS_IMETHODIMP
-CSSLoaderImpl::SetQuirkMode(PRBool aQuirkMode)
+CSSLoaderImpl::SetCompatibilityMode(nsCompatibility aCompatMode)
 {
-  mNavQuirkMode = aQuirkMode;
+  mCompatMode = aCompatMode;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-CSSLoaderImpl::SetPreferredSheet(const nsAReadableString& aTitle)
+CSSLoaderImpl::SetPreferredSheet(const nsAString& aTitle)
 {
   mPreferredSheet = aTitle;
 
@@ -589,7 +583,7 @@ CSSLoaderImpl::GetParserFor(nsICSSStyleSheet* aSheet,
   }
   if (*aParser) {
     (*aParser)->SetCaseSensitive(mCaseSensitive);
-    (*aParser)->SetQuirkMode(mNavQuirkMode);
+    (*aParser)->SetQuirkMode(mCompatMode == eCompatibility_NavQuirks);
     (*aParser)->SetCharset(mCharset);
     if (aSheet) {
       (*aParser)->SetStyleSheet(aSheet);
@@ -615,6 +609,55 @@ CSSLoaderImpl::RecycleParser(nsICSSParser* aParser)
   return result;
 }
 
+/**
+ * Report an error to the error console.
+ *  @param aErrorName     The name of a string in css.properties.
+ *  @param aParams        The parameters for that string in css.properties.
+ *  @param aParamsLength  The length of aParams.
+ *  @param aErrorFlags    Error/warning flag to pass to nsIScriptError::Init.
+ *
+ * XXX This should be a static method on a class called something like
+ * nsCSSUtils, since it's a general way of accessing css.properties and
+ * will be useful for localizability work on CSS parser error reporting.
+ * However, it would need some way of reporting source file name, text,
+ * line, and column information.
+ */
+static nsresult
+ReportToConsole(const PRUnichar* aMessageName, const PRUnichar **aParams, 
+                PRUint32 aParamsLength, PRUint32 aErrorFlags)
+{
+  nsresult rv;
+  nsCOMPtr<nsIConsoleService> consoleService =
+    do_GetService(NS_CONSOLESERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIScriptError> errorObject =
+    do_CreateInstance(NS_SCRIPTERROR_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIStringBundleService> stringBundleService =
+    do_GetService(kCStringBundleServiceCID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+  nsCOMPtr<nsIStringBundle> bundle;
+  rv = stringBundleService->CreateBundle(
+           "chrome://global/locale/css.properties", getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsXPIDLString errorText;
+  rv = bundle->FormatStringFromName(aMessageName, aParams, aParamsLength,
+                                    getter_Copies(errorText));
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = errorObject->Init(errorText.get(),
+                         NS_LITERAL_STRING("").get(), /* file name */
+                         NS_LITERAL_STRING("").get(), /* source line */
+                         0, /* line number */
+                         0, /* column number */
+                         aErrorFlags,
+                         "CSS Loader");
+  NS_ENSURE_SUCCESS(rv, rv);
+  consoleService->LogMessage(errorObject);
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 SheetLoadData::OnStreamComplete(nsIStreamLoader* aLoader,
                                 nsISupports* aContext,
@@ -627,24 +670,53 @@ SheetLoadData::OnStreamComplete(nsIStreamLoader* aLoader,
   nsresult result = NS_OK;
   nsString *strUnicodeBuffer = nsnull;
 
-  if (aString && aStringLen>0) {
-    nsCOMPtr<nsIRequest> request;
-    aLoader->GetRequest(getter_AddRefs(request));
-    nsXPIDLCString contentType;
-    if (! (mLoader->mNavQuirkMode)) {
-      nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
-      if (channel) {
-        channel->GetContentType(getter_Copies(contentType));
-      }
+  nsCOMPtr<nsIRequest> request;
+  result = aLoader->GetRequest(getter_AddRefs(request));
+  if (NS_FAILED(result))
+    request = nsnull;
+  // If it's an HTTP channel, we want to make sure this is not an
+  // error document we got.
+  PRBool realDocument = PR_TRUE;
+  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(request));
+  if (httpChannel) {
+    PRBool requestSucceeded;
+    result = httpChannel->GetRequestSucceeded(&requestSucceeded);
+    if (NS_SUCCEEDED(result) && !requestSucceeded) {
+      realDocument = PR_FALSE;
     }
-    if (mLoader->mNavQuirkMode ||
-        Compare(contentType,
-                NS_LITERAL_CSTRING("text/css"),
-                nsCaseInsensitiveCStringComparator()) == 0 ||
+  }
+  
+  if (realDocument && aString && aStringLen>0) {
+    nsCAutoString contentType;
+    nsCOMPtr<nsIChannel> channel(do_QueryInterface(request));
+    if (channel) {
+      channel->GetContentType(contentType);
+    }
+    if (mLoader->mCompatMode == eCompatibility_NavQuirks ||
+        contentType.Equals(NS_LITERAL_CSTRING("text/css")) ||
         contentType.IsEmpty()) {
+
+      if (!contentType.IsEmpty() &&
+          contentType != NS_LITERAL_CSTRING("text/css")) {
+        nsCAutoString spec;
+        if (channel) {
+          nsCOMPtr<nsIURI> uri;
+          channel->GetURI(getter_AddRefs(uri));
+          if (uri)
+            uri->GetSpec(spec);
+        }
+
+        const nsAFlatString& specUCS2 = NS_ConvertUTF8toUCS2(spec);
+        const nsAFlatString& ctypeUCS2 = NS_ConvertASCIItoUCS2(contentType);
+        const PRUnichar *strings[] = { specUCS2.get(), ctypeUCS2.get() };
+
+        ReportToConsole(NS_LITERAL_STRING("MimeNotCssWarn").get(), strings, 2,
+                        nsIScriptError::warningFlag);
+      }
+
       /*
        * First determine the charset (if one is indicated)
-       * 1)  Check HTTP charset
+       * 1)  Check nsIChannel::contentCharset
        * 2)  Check @charset rules
        * 3)  Check "charset" attribute of the <LINK> or <?xml-stylesheet?>
        *
@@ -652,16 +724,15 @@ SheetLoadData::OnStreamComplete(nsIStreamLoader* aLoader,
        * default (document charset or ISO-8859-1 if we have no document
        * charset)
        */
-      nsAutoString strHTTPCharset;
-      nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(request);
-      if (httpChannel) {
-        nsXPIDLCString httpCharset;
-        httpChannel->GetCharset(getter_Copies(httpCharset));
-        CopyASCIItoUCS2(httpCharset, strHTTPCharset);
+      nsAutoString strChannelCharset;
+      if (channel) {
+        nsCAutoString charsetVal;
+        channel->GetContentCharset(charsetVal);
+        CopyASCIItoUCS2(charsetVal, strChannelCharset);
       }
       result = NS_ERROR_NOT_AVAILABLE;
-      if (! strHTTPCharset.IsEmpty()) {
-        result = mLoader->SetCharset(strHTTPCharset);
+      if (! strChannelCharset.IsEmpty()) {
+        result = mLoader->SetCharset(strChannelCharset);
       }
       if (NS_FAILED(result)) {
         //  We have no charset or the HTTP charset is not recognized.
@@ -706,38 +777,59 @@ SheetLoadData::OnStreamComplete(nsIStreamLoader* aLoader,
                 // make space for the decoding
                 strUnicodeBuffer->SetCapacity(unicodeLength);
                 unicodeString = (PRUnichar *) strUnicodeBuffer->get();
-                result = decoder->Convert(aString, (PRInt32 *) &aStringLen, unicodeString, &unicodeLength);
-                if (NS_SUCCEEDED(result)) {
-                  strUnicodeBuffer->SetLength(unicodeLength);
-                } else {
-#ifdef DEBUG
-                  /*
-                   * We currently fail often because of XML documents
-                   * which are in UTF-8 importing stylesheets which
-                   * use an ISO-8859-1 char, especially in comments in
-                   * the sheet.  See http://bugzilla.mozilla.org/show_bug.cgi?id=106843
-                   *
-                   * This assertion should at least make it possible
-                   * to catch such errors in chrome, pending this bug
-                   * being fixed
-                   */
-                   
-                  nsXPIDLCString uriStr;
-                  mURL->GetSpec(getter_Copies(uriStr));
-                  nsCAutoString errorMessage;
-                  errorMessage = NS_LITERAL_CSTRING("Decoding sheet from ") +
-                                 uriStr +
-                                 NS_LITERAL_CSTRING(" failed");
-                  NS_ASSERTION(PR_FALSE, errorMessage.get());
-#endif // DEBUG
-                  strUnicodeBuffer->SetLength(0);
-                }
+                PRInt32 totalChars = 0;
+                PRInt32 unicharLength = unicodeLength;
+                do {
+                  PRInt32 srcLength = aStringLen;
+                  result = decoder->Convert(aString, &srcLength, unicodeString, &unicharLength);
+                  
+                  totalChars += unicharLength;
+                  if (NS_FAILED(result)) {
+                    // if we failed, we consume one byte, replace it with U+FFFD
+                    // and try the conversion again.
+                    unicodeString[unicharLength++] = (PRUnichar)0xFFFD;
+                    unicodeString = unicodeString + unicharLength;
+                    unicharLength = unicodeLength - (++totalChars);
+
+                    decoder->Reset();
+
+                    if (((PRUint32) (srcLength + 1)) > aStringLen) {
+                      srcLength = aStringLen;
+                    } else {
+                      srcLength++;
+                    }
+
+                    aString += srcLength;
+                    aStringLen -= srcLength;
+                  }
+                } while (NS_FAILED(result) && (aStringLen > 0));
+                
+                // Don't propagate return code of unicode decoder
+                // since it doesn't reflect on our success or failure
+                // - Ref. bug 87110
+                result = NS_OK;
+                strUnicodeBuffer->SetLength(totalChars);
               }
             }
             NS_RELEASE(decoder);
           }
         }
       }
+    } else {
+      nsCAutoString spec;
+      if (channel) {
+        nsCOMPtr<nsIURI> uri;
+        channel->GetURI(getter_AddRefs(uri));
+        if (uri)
+          uri->GetSpec(spec);
+      }
+
+      const nsAFlatString& specUCS2 = NS_ConvertUTF8toUCS2(spec);
+      const nsAFlatString& ctypeUCS2 = NS_ConvertASCIItoUCS2(contentType);
+      const PRUnichar *strings[] = { specUCS2.get(), ctypeUCS2.get() };
+
+      ReportToConsole(NS_LITERAL_STRING("MimeNotCss").get(), strings, 2,
+                      nsIScriptError::errorFlag);
     }
   }
 
@@ -976,7 +1068,7 @@ CSSLoaderImpl::DidLoadStyle(nsIStreamLoader* aLoader,
   NS_ASSERTION(! mSyncCallback, "getting synchronous callback from netlib");
 #endif
 
-  if (NS_SUCCEEDED(aStatus) && (aStyleData) && (0 < aStyleData->Length()) && (mDocument)) {
+  if (NS_SUCCEEDED(aStatus) && (aStyleData) && (!aStyleData->IsEmpty()) && (mDocument)) {
     nsresult result;
     nsIUnicharInputStream* uin = nsnull;
 
@@ -1000,10 +1092,10 @@ CSSLoaderImpl::DidLoadStyle(nsIStreamLoader* aLoader,
 #ifdef DEBUG
     if (mDocument && NS_FAILED(aStatus)) {  // still have doc, must have failed
       // Dump error message to console.
-      nsXPIDLCString url;
-      aLoadData->mURL->GetSpec(getter_Copies(url));
+      nsCAutoString url;
+      aLoadData->mURL->GetSpec(url);
       nsCAutoString errorMessage(NS_LITERAL_CSTRING("CSSLoaderImpl::DidLoadStyle: Load of URL '") +
-                                 nsDependentCString(url) +
+                                 url +
                                  NS_LITERAL_CSTRING("' failed.  Error code: "));
       errorMessage.AppendInt(NS_ERROR_GET_CODE(aStatus));
       NS_WARNING(errorMessage.get());
@@ -1077,7 +1169,7 @@ static nsresult EnumerateMediaString(const nsString& aStringList, nsStringEnumFu
       subStr.CompressWhitespace(PR_FALSE, PR_TRUE);
     }
 
-    if (0 < subStr.Length()) {
+    if (!subStr.IsEmpty()) {
       status = (*aFunc)(subStr, aData);
     }
 
@@ -1102,7 +1194,7 @@ CSSLoaderImpl::SetMedia(nsICSSStyleSheet* aSheet, const nsString& aMedia)
 {
   nsresult rv = NS_OK;
   aSheet->ClearMedia();
-  if (0 < aMedia.Length()) {
+  if (!aMedia.IsEmpty()) {
     rv = EnumerateMediaString(aMedia, MediumEnumFunc, aSheet);
   }
   return rv;
@@ -1137,8 +1229,8 @@ CSSLoaderImpl::AddPendingSheet(nsICSSStyleSheet* aSheet, PRInt32 aDocIndex,
 PRBool
 CSSLoaderImpl::IsAlternate(const nsString& aTitle)
 {
-  if (0 < aTitle.Length()) {
-    return PRBool(! aTitle.EqualsIgnoreCase(mPreferredSheet));
+  if (!aTitle.IsEmpty()) {
+    return PRBool(! aTitle.Equals(mPreferredSheet, nsCaseInsensitiveStringComparator()));
   }
   return PR_FALSE;
 }
@@ -1212,6 +1304,10 @@ CSSLoaderImpl::InsertChildSheet(nsICSSStyleSheet* aSheet, nsICSSStyleSheet* aPar
     return NS_ERROR_NULL_POINTER;
   }
 
+  // child sheets should always start out enabled, even if they got
+  // cloned off of top-level sheets which were disabled
+  aSheet->SetEnabled(PR_TRUE);
+  
   nsVoidKey key(aParentSheet);
   nsAutoVoidArray*  sheetMap = (nsAutoVoidArray*)mSheetMapTable.Get(&key);
   if (! sheetMap) {
@@ -1282,10 +1378,10 @@ CSSLoaderImpl::LoadSheet(URLKey& aKey, SheetLoadData* aData)
 #ifdef DEBUG
         else {
           // Dump an error message to the console
-          nsXPIDLCString url;
-          aKey.mURL->GetSpec(getter_Copies(url));
+          nsCAutoString url;
+          aKey.mURL->GetSpec(url);
           nsCAutoString errorMessage(NS_LITERAL_CSTRING("CSSLoaderImpl::LoadSheet: Load of URL '") +
-                                     nsDependentCString(url) +
+                                     url +
                                      NS_LITERAL_CSTRING("' failed.  Error code: "));
           errorMessage.AppendInt(NS_ERROR_GET_CODE(result));
           NS_WARNING(errorMessage.get());
@@ -1572,10 +1668,14 @@ CSSLoaderImpl::LoadChildSheet(nsICSSStyleSheet* aParentSheet,
             }
             parentData = parentData->mParentData;
           } while (parentData);
-        
-          (data->mParentData->mPendingChildren)++;
+
+          ++(data->mParentData->mPendingChildren);
         }
         result = LoadSheet(key, data);
+        if (count && NS_FAILED(result)) {
+          // If the load failed, undo the addition of the pending child.
+          --(data->mParentData->mPendingChildren);
+        }
       }
     }
   }
@@ -1631,8 +1731,8 @@ CSSLoaderImpl::LoadAgentSheet(nsIURI* aURL,
       else {
         // Dump an error message to the console
         PRBool ignoreError = PR_FALSE;
-        nsXPIDLCString url;
-        aURL->GetSpec(getter_Copies(url));
+        nsCAutoString url;
+        aURL->GetSpec(url);
         // Ignore userChrome.css and userContent.css failures
 #define USERCHROMECSS "userChrome.css"
 #define USERCONTENTCSS "userContent.css"
@@ -1647,7 +1747,7 @@ CSSLoaderImpl::LoadAgentSheet(nsIURI* aURL,
 
         if (!ignoreError) {
           nsCAutoString errorMessage(NS_LITERAL_CSTRING("CSSLoaderImpl::LoadAgentSheet: Load of URL '") +
-                                     nsDependentCString(url) +
+                                     url +
                                      NS_LITERAL_CSTRING("' failed.  Error code: "));
           errorMessage.AppendInt(NS_ERROR_GET_CODE(result));
           NS_WARNING(errorMessage.get());
@@ -1687,7 +1787,7 @@ nsresult NS_NewCSSLoader(nsICSSLoader** aLoader)
 
 NS_IMETHODIMP CSSLoaderImpl::GetCharset(/*out*/nsAString &aCharsetDest) const
 {
-  NS_ASSERTION(mCharset.Length() > 0, "CSSLoader charset should be set in ctor" );
+  NS_ASSERTION(!mCharset.IsEmpty(), "CSSLoader charset should be set in ctor" );
   nsresult rv = NS_OK;
   aCharsetDest = mCharset;
   return rv;
@@ -1738,7 +1838,7 @@ nsresult CSSLoaderImpl::SetCharset(/*in*/ const char* aStyleSheetData,
   nsString strStyleDataUndecoded;
   strStyleDataUndecoded.AssignWithConversion(aStyleSheetData, aDataLength);
   PRInt32 charsetOffset;
-  if (strStyleDataUndecoded.Length() > 0) {
+  if (!strStyleDataUndecoded.IsEmpty()) {
     nsString str;
     static const char atCharsetStr[] = "@charset";
     if ((charsetOffset = strStyleDataUndecoded.Find(atCharsetStr)) > -1) {
