@@ -47,7 +47,6 @@
 #include "nsIHTMLContent.h"
 #include "nsIStyleRule.h"
 #include "nsIFrame.h"
-#include "nsHTMLIIDs.h"
 #include "nsICSSStyleRule.h"
 #include "nsIStyleRuleProcessor.h"
 #include "nsIStyleContext.h"
@@ -212,8 +211,8 @@ CSSFirstLetterRule::CSSFirstLetterRule(nsIHTMLCSSStyleSheet* aSheet)
 class HTMLCSSStyleSheetImpl : public nsIHTMLCSSStyleSheet,
                               public nsIStyleRuleProcessor {
 public:
-  void* operator new(size_t size);
-  void* operator new(size_t size, nsIArena* aArena);
+  void* operator new(size_t size) CPP_THROW_NEW;
+  void* operator new(size_t size, nsIArena* aArena) CPP_THROW_NEW;
   void operator delete(void* ptr);
 
   HTMLCSSStyleSheetImpl();
@@ -251,7 +250,8 @@ public:
                            nsIAtom* aMedium);
 
   NS_IMETHOD HasStateDependentStyle(StateRuleProcessorData* aData,
-                                    nsIAtom* aMedium);
+                                    nsIAtom* aMedium,
+                                    PRBool* aResult);
 
   // XXX style rule enumerations
 
@@ -286,24 +286,24 @@ protected:
 };
 
 
-void* HTMLCSSStyleSheetImpl::operator new(size_t size)
+void* HTMLCSSStyleSheetImpl::operator new(size_t size) CPP_THROW_NEW
 {
   HTMLCSSStyleSheetImpl* rv = (HTMLCSSStyleSheetImpl*) ::operator new(size);
 #ifdef NS_DEBUG
   if (nsnull != rv) {
-    nsCRT::memset(rv, 0xEE, size);
+    memset(rv, 0xEE, size);
   }
 #endif
   rv->mInHeap = 1;
   return (void*) rv;
 }
 
-void* HTMLCSSStyleSheetImpl::operator new(size_t size, nsIArena* aArena)
+void* HTMLCSSStyleSheetImpl::operator new(size_t size, nsIArena* aArena) CPP_THROW_NEW
 {
   HTMLCSSStyleSheetImpl* rv = (HTMLCSSStyleSheetImpl*) aArena->Alloc(PRInt32(size));
 #ifdef NS_DEBUG
   if (nsnull != rv) {
-    nsCRT::memset(rv, 0xEE, size);
+    memset(rv, 0xEE, size);
   }
 #endif
   rv->mInHeap = 0;
@@ -422,9 +422,11 @@ HTMLCSSStyleSheetImpl::Init(nsIURI* aURL, nsIDocument* aDocument)
 // Test if style is dependent on content state
 NS_IMETHODIMP
 HTMLCSSStyleSheetImpl::HasStateDependentStyle(StateRuleProcessorData* aData,
-                                              nsIAtom* aMedium)
+                                              nsIAtom* aMedium,
+                                              PRBool* aResult)
 {
-  return NS_COMFALSE;
+  *aResult = PR_FALSE;
+  return NS_OK;
 }
 
 
@@ -531,11 +533,10 @@ void HTMLCSSStyleSheetImpl::List(FILE* out, PRInt32 aIndent) const
   for (PRInt32 index = aIndent; --index >= 0; ) fputs("  ", out);
 
   fputs("HTML CSS Style Sheet: ", out);
-  char* urlSpec = nsnull;
-  mURL->GetSpec(&urlSpec);
-  if (urlSpec) {
-    fputs(urlSpec, out);
-    nsCRT::free(urlSpec);
+  nsCAutoString urlSpec;
+  mURL->GetSpec(urlSpec);
+  if (!urlSpec.IsEmpty()) {
+    fputs(urlSpec.get(), out);
   }
   fputs("\n", out);
 }
