@@ -45,7 +45,6 @@
 #include "nsFont.h"
 #include "nsVoidArray.h"
 #include "nsStyleCoord.h"
-#include "nsStyleStruct.h"
 #include "nsStyleConsts.h"
 #include "nsIStyleSet.h"
 #include "nsIPresContext.h"
@@ -56,71 +55,28 @@
 class nsIFrame;
 
 enum nsStyleStructID {
-  eStyleStruct_Font           = 1,
-  eStyleStruct_Color          = 2,
-  eStyleStruct_Background     = 3,
-  eStyleStruct_List           = 4,
-  eStyleStruct_Position       = 5,
-  eStyleStruct_Text           = 6,
-  eStyleStruct_TextReset      = 7,
-  eStyleStruct_Display        = 8,
-  eStyleStruct_Visibility     = 9,
-  eStyleStruct_Content        = 10,
-  eStyleStruct_Quotes         = 11,
-  eStyleStruct_UserInterface  = 12,
-  eStyleStruct_UIReset        = 13,
-  eStyleStruct_Table          = 14,
-  eStyleStruct_TableBorder    = 15,
-  eStyleStruct_Margin         = 16,
-  eStyleStruct_Padding        = 17,
-  eStyleStruct_Border         = 18,
-  eStyleStruct_Outline        = 19,
-  eStyleStruct_XUL            = 20,
-#ifdef MOZ_SVG
-  eStyleStruct_SVG            = 21,
-  eStyleStruct_Max            = eStyleStruct_SVG,
-  eStyleStruct_BorderPaddingShortcut = 22,       // only for use in GetStyle()
-#else
-  eStyleStruct_Max            = eStyleStruct_XUL,
-  eStyleStruct_BorderPaddingShortcut = 21,       // only for use in GetStyle()
-#endif
-  eStyleStruct_Min            = eStyleStruct_Font
+
+/*
+ * Define the constants eStyleStruct_Font, etc.
+ *
+ * The C++ standard, section 7.2, guarantees that enums begin with 0 and
+ * increase by 1.
+ */
+
+#define STYLE_STRUCT(name, checkdata_cb) eStyleStruct_##name,
+#include "nsStyleStructList.h"
+#undef STYLE_STRUCT
+
+nsStyleStructID_Length /* one past the end; length of 0-based list */
+
 };
 
 // Bits for each struct.
-#define NS_STYLE_INHERIT_BIT(sid_)        (1 << (PRInt32(sid_) - 1))
-#define NS_STYLE_INHERIT_FONT             NS_STYLE_INHERIT_BIT(eStyleStruct_Font)
-#define NS_STYLE_INHERIT_COLOR            NS_STYLE_INHERIT_BIT(eStyleStruct_Color)
-#define NS_STYLE_INHERIT_BACKGROUND       NS_STYLE_INHERIT_BIT(eStyleStruct_Background)
-#define NS_STYLE_INHERIT_LIST             NS_STYLE_INHERIT_BIT(eStyleStruct_List)
-#define NS_STYLE_INHERIT_POSITION         NS_STYLE_INHERIT_BIT(eStyleStruct_Position)
-#define NS_STYLE_INHERIT_TEXT             NS_STYLE_INHERIT_BIT(eStyleStruct_Text)
-#define NS_STYLE_INHERIT_TEXT_RESET       NS_STYLE_INHERIT_BIT(eStyleStruct_TextReset)
-#define NS_STYLE_INHERIT_DISPLAY          NS_STYLE_INHERIT_BIT(eStyleStruct_Display)
-#define NS_STYLE_INHERIT_VISIBILITY       NS_STYLE_INHERIT_BIT(eStyleStruct_Visibility)
-#define NS_STYLE_INHERIT_TABLE            NS_STYLE_INHERIT_BIT(eStyleStruct_Table)
-#define NS_STYLE_INHERIT_TABLE_BORDER     NS_STYLE_INHERIT_BIT(eStyleStruct_TableBorder)
-#define NS_STYLE_INHERIT_CONTENT          NS_STYLE_INHERIT_BIT(eStyleStruct_Content)
-#define NS_STYLE_INHERIT_QUOTES           NS_STYLE_INHERIT_BIT(eStyleStruct_Quotes)
-#define NS_STYLE_INHERIT_UI               NS_STYLE_INHERIT_BIT(eStyleStruct_UserInterface)
-#define NS_STYLE_INHERIT_UI_RESET         NS_STYLE_INHERIT_BIT(eStyleStruct_UIReset)
-#define NS_STYLE_INHERIT_MARGIN           NS_STYLE_INHERIT_BIT(eStyleStruct_Margin)
-#define NS_STYLE_INHERIT_PADDING          NS_STYLE_INHERIT_BIT(eStyleStruct_Padding)
-#define NS_STYLE_INHERIT_BORDER           NS_STYLE_INHERIT_BIT(eStyleStruct_Border)
-#define NS_STYLE_INHERIT_OUTLINE          NS_STYLE_INHERIT_BIT(eStyleStruct_Outline)
-#define NS_STYLE_INHERIT_XUL              NS_STYLE_INHERIT_BIT(eStyleStruct_XUL)
-#ifdef MOZ_SVG
-#define NS_STYLE_INHERIT_SVG              NS_STYLE_INHERIT_BIT(eStyleStruct_SVG)
-#endif
-
+#define NS_STYLE_INHERIT_BIT(sid_)        (1 << PRInt32(eStyleStruct_##sid_))
 #define NS_STYLE_INHERIT_MASK             0x00ffffff
 
-// A bit to test whether or not a style context can be shared
-// by siblings.
-#define NS_STYLE_UNIQUE_CONTEXT           0x01000000
-
 // A bit to test whether or not we have any text decorations.
-#define NS_STYLE_HAS_TEXT_DECORATIONS     0x02000000
+#define NS_STYLE_HAS_TEXT_DECORATIONS     0x01000000
 
 #define NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(the_sid) \
   static const nsStyleStructID GetStyleStructID() {return the_sid;}
@@ -142,7 +98,7 @@ struct nsStyleFont : public nsStyleStruct {
   PRInt32 CalcDifference(const nsStyleFont& aOther) const;
   static PRInt32 CalcFontDifference(const nsFont& aFont1, const nsFont& aFont2);
   
-  void* operator new(size_t sz, nsIPresContext* aContext);
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy(nsIPresContext* aContext);
 
   PRUint8 mFlags;       // [inherited] See nsStyleConsts.h
@@ -157,9 +113,6 @@ struct nsStyleFont : public nsStyleStruct {
   nsStyleFont(nsIPresContext* aPresContext);
 };
 
-#define NS_COLORFLAGS_NONE  0x00
-#define NS_COLORFLAGS_THEME 0x01
-
 struct nsStyleColor : public nsStyleStruct {
   nsStyleColor(nsIPresContext* aPresContext);
   nsStyleColor(const nsStyleColor& aOther);
@@ -169,7 +122,7 @@ struct nsStyleColor : public nsStyleStruct {
 
   PRInt32 CalcDifference(const nsStyleColor& aOther) const;
   
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -182,8 +135,6 @@ struct nsStyleColor : public nsStyleStruct {
   // Don't add ANY members to this struct!  We can achieve caching in the rule
   // tree (rather than the style tree) by letting color stay by itself! -dwh
   nscolor mColor;                 // [inherited]
-  PRPackedBool mColorFlags;       // Used to indicate theme colors.  Theme colors
-                                  // must be resolved dynamically.
 };
 
 struct nsStyleBackground : public nsStyleStruct {
@@ -193,7 +144,7 @@ struct nsStyleBackground : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Background)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -237,7 +188,7 @@ struct nsStyleMargin: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Margin)
 
-  void* operator new(size_t sz, nsIPresContext* aContext);
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy(nsIPresContext* aContext);
 
   void RecalcData();
@@ -270,7 +221,7 @@ struct nsStylePadding: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Padding)
 
-  void* operator new(size_t sz, nsIPresContext* aContext);
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy(nsIPresContext* aContext);
 
   void RecalcData();
@@ -347,7 +298,7 @@ struct nsStyleBorder: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Border)
 
-  void* operator new(size_t sz, nsIPresContext* aContext);
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW;
   void Destroy(nsIPresContext* aContext);
 
   PRBool IsBorderSideVisible(PRUint8 aSide) const;
@@ -465,7 +416,8 @@ struct nsStyleBorder: public nsStyleStruct {
 
   // XXX these are deprecated methods
   void CalcBorderFor(const nsIFrame* aFrame, nsMargin& aBorder) const;
- 
+  void CalcBorderFor(const nsIFrame* aFrame, PRUint8 aSide, nscoord& aWidth) const;
+  
 protected:
   PRPackedBool  mHasCachedBorder;
   nsMargin      mCachedBorder;
@@ -512,7 +464,7 @@ struct nsStyleOutline: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Outline)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -596,7 +548,7 @@ struct nsStyleList : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_List)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -621,7 +573,7 @@ struct nsStylePosition : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Position)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -651,7 +603,7 @@ struct nsStyleTextReset : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_TextReset)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -678,7 +630,7 @@ struct nsStyleText : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Text)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -711,7 +663,7 @@ struct nsStyleVisibility : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Visibility)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -745,7 +697,7 @@ struct nsStyleDisplay : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Display)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -799,7 +751,7 @@ struct nsStyleTable: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Table)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -825,7 +777,7 @@ struct nsStyleTableBorder: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_TableBorder)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -876,7 +828,7 @@ struct nsStyleQuotes : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Quotes)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -937,7 +889,7 @@ struct nsStyleContent: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_Content)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -1076,7 +1028,7 @@ struct nsStyleUIReset: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_UIReset)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -1091,6 +1043,7 @@ struct nsStyleUIReset: public nsStyleStruct {
   PRUint8   mUserSelect;      // [reset] (selection-style)
   PRUnichar mKeyEquivalent;   // [reset] XXX what type should this be?
   PRUint8   mResizer;         // [reset]
+  PRUint8   mForceBrokenImageIcon; // [reset]  (0 if not forcing, otherwise forcing)
 };
 
 struct nsStyleUserInterface: public nsStyleStruct {
@@ -1100,7 +1053,7 @@ struct nsStyleUserInterface: public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_UserInterface)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -1128,7 +1081,7 @@ struct nsStyleXUL : public nsStyleStruct {
 
   NS_DEFINE_STATIC_STYLESTRUCTID_ACCESSOR(eStyleStruct_XUL)
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
@@ -1167,7 +1120,7 @@ struct nsStyleSVG : public nsStyleStruct {
   nsStyleSVG(const nsStyleSVG& aSource);
   ~nsStyleSVG();
 
-  void* operator new(size_t sz, nsIPresContext* aContext) {
+  void* operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
     void* result = nsnull;
     aContext->AllocateFromShell(sz, &result);
     return result;
