@@ -37,6 +37,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include "nsStyleStruct.h"
 #include "nsStyleConsts.h"
 #include "nsString.h"
 #include "nsUnitConversion.h"
@@ -53,7 +54,7 @@
 #include "prenv.h"
 
 #ifdef IBMBIDI
-#include "nsIUBidiUtils.h"
+#include "nsBidiUtils.h"
 #endif
 
 inline PRBool IsFixedUnit(nsStyleUnit aUnit, PRBool aEnumOK)
@@ -221,24 +222,24 @@ nsStyleFont::nsStyleFont()
 
 nsStyleFont::nsStyleFont(const nsFont& aFont)
   : mFlags(NS_STYLE_FONT_DEFAULT),
-    mFont(aFont)
+    mFont(aFont),
+    mSize(aFont.size)
 {
-  mSize = aFont.size;
 }
 
 nsStyleFont::nsStyleFont(const nsStyleFont& aSrc)
-:mFont(aSrc.mFont)
+  : mFlags(aSrc.mFlags),
+    mFont(aSrc.mFont),
+    mSize(aSrc.mSize)
 {
-  mSize = aSrc.mSize;
-  mFlags = aSrc.mFlags;
 }
 
 void* 
-nsStyleFont::operator new(size_t sz, nsIPresContext* aContext) {
+nsStyleFont::operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
   void* result = nsnull;
   aContext->AllocateFromShell(sz, &result);
   if (result)
-  nsCRT::zero(result, sz);
+  memset(result, 0, sz);
   return result;
 }
   
@@ -315,11 +316,11 @@ nsStyleMargin::nsStyleMargin(const nsStyleMargin& aSrc) {
 }
 
 void* 
-nsStyleMargin::operator new(size_t sz, nsIPresContext* aContext) {
+nsStyleMargin::operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
   void* result = nsnull;
   aContext->AllocateFromShell(sz, &result);
   if (result)
-  nsCRT::zero(result, sz);
+  memset(result, 0, sz);
   return result;
 }
   
@@ -374,11 +375,11 @@ nsStylePadding::nsStylePadding(const nsStylePadding& aSrc) {
 }
 
 void* 
-nsStylePadding::operator new(size_t sz, nsIPresContext* aContext) {
+nsStylePadding::operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
   void* result = nsnull;
   aContext->AllocateFromShell(sz, &result);
   if (result)
-  nsCRT::zero(result, sz);
+  memset(result, 0, sz);
   return result;
 }
   
@@ -474,11 +475,11 @@ nsStyleBorder::nsStyleBorder(const nsStyleBorder& aSrc)
 }
 
 void* 
-nsStyleBorder::operator new(size_t sz, nsIPresContext* aContext) {
+nsStyleBorder::operator new(size_t sz, nsIPresContext* aContext) CPP_THROW_NEW {
   void* result = nsnull;
   aContext->AllocateFromShell(sz, &result);
   if (result)
-  nsCRT::zero(result, sz);
+  memset(result, 0, sz);
   return result;
 }
   
@@ -605,6 +606,28 @@ nsStyleBorder::CalcBorderFor(const nsIFrame* aFrame, nsMargin& aBorder) const
   }
 }
 
+void 
+nsStyleBorder::CalcBorderFor(const nsIFrame* aFrame, PRUint8 aSide, nscoord& aWidth) const
+{
+  aWidth = 0;
+  // using mCachedBorder as above, doesn't work properly 
+  nsStyleCoord  coord;
+  switch(aSide) {
+  case NS_SIDE_TOP:
+    coord = mBorder.GetTop(coord);
+    break;
+  case NS_SIDE_RIGHT:
+    coord = mBorder.GetRight(coord);
+    break;
+  case NS_SIDE_BOTTOM:
+    coord = mBorder.GetBottom(coord);
+    break;
+  default: // NS_SIDE_LEFT
+    coord = mBorder.GetLeft(coord);
+  }
+  aWidth = CalcSideFor(aFrame, coord, NS_SPACING_BORDER, aSide, mBorderWidths, 3);
+}
+
 nsStyleOutline::nsStyleOutline(nsIPresContext* aPresContext)
 {
   // XXX support mBorderWidths until deprecated methods are removed
@@ -862,7 +885,7 @@ nsStyleTable::nsStyleTable()
   mLayoutStrategy = NS_STYLE_TABLE_LAYOUT_AUTO;
   mCols  = NS_STYLE_TABLE_COLS_NONE;
   mFrame = NS_STYLE_TABLE_FRAME_NONE;
-  mRules = NS_STYLE_TABLE_RULES_ALL;
+  mRules = NS_STYLE_TABLE_RULES_NONE;
   mSpan = 1;
 }
 
@@ -893,12 +916,12 @@ nsStyleTableBorder::nsStyleTableBorder(nsIPresContext* aPresContext)
 { 
   mBorderCollapse = NS_STYLE_BORDER_SEPARATE;
 
-  nsCompatibility compatMode = eCompatibility_Standard;
+  nsCompatibility compatMode = eCompatibility_FullStandards;
   if (aPresContext)
-		aPresContext->GetCompatibilityMode(&compatMode);
-  mEmptyCells = (compatMode == eCompatibility_NavQuirks
+    aPresContext->GetCompatibilityMode(&compatMode);
+  mEmptyCells = (compatMode == eCompatibility_NavQuirks)
                   ? NS_STYLE_TABLE_EMPTY_CELLS_SHOW_BACKGROUND     
-                  : NS_STYLE_TABLE_EMPTY_CELLS_SHOW);
+                  : NS_STYLE_TABLE_EMPTY_CELLS_SHOW;
   mCaptionSide = NS_SIDE_TOP;
   mBorderSpacingX.Reset();
   mBorderSpacingY.Reset();
@@ -932,7 +955,6 @@ PRInt32 nsStyleTableBorder::CalcDifference(const nsStyleTableBorder& aOther) con
 //
 
 nsStyleColor::nsStyleColor(nsIPresContext* aPresContext)
-:mColorFlags(NS_COLORFLAGS_NONE)
 {
   aPresContext->GetDefaultColor(&mColor);
 }
@@ -940,7 +962,6 @@ nsStyleColor::nsStyleColor(nsIPresContext* aPresContext)
 nsStyleColor::nsStyleColor(const nsStyleColor& aSource)
 {
   mColor = aSource.mColor;
-  mColorFlags = aSource.mColorFlags;
 }
 
 PRInt32 nsStyleColor::CalcDifference(const nsStyleColor& aOther) const
@@ -978,6 +999,12 @@ nsStyleBackground::nsStyleBackground(const nsStyleBackground& aSource)
 
 PRInt32 nsStyleBackground::CalcDifference(const nsStyleBackground& aOther) const
 {
+  if (mBackgroundAttachment != aOther.mBackgroundAttachment
+    && (NS_STYLE_BG_ATTACHMENT_FIXED == mBackgroundAttachment) ||
+      (NS_STYLE_BG_ATTACHMENT_FIXED == aOther.mBackgroundAttachment))
+    // this might require creation of a view
+    return NS_STYLE_HINT_FRAMECHANGE;
+
   if ((mBackgroundAttachment == aOther.mBackgroundAttachment) &&
       (mBackgroundFlags == aOther.mBackgroundFlags) &&
       (mBackgroundRepeat == aOther.mBackgroundRepeat) &&
@@ -1075,13 +1102,18 @@ nsStyleVisibility::nsStyleVisibility(const nsStyleVisibility& aSource)
 
 PRInt32 nsStyleVisibility::CalcDifference(const nsStyleVisibility& aOther) const
 {
-  if (mOpacity != aOther.mOpacity)
-    return NS_STYLE_HINT_VISUAL;
+  if (mOpacity != aOther.mOpacity
+      && ((mOpacity < 1.0) != (aOther.mOpacity < 1.0)))
+     // might need to create a view to handle change from 1.0 to partial opacity
+     return NS_STYLE_HINT_FRAMECHANGE;
 
   if ((mDirection == aOther.mDirection) &&
       (mLanguage == aOther.mLanguage)) {
     if ((mVisible == aOther.mVisible)) {
-      return NS_STYLE_HINT_NONE;
+      if (mOpacity == aOther.mOpacity)
+        return NS_STYLE_HINT_NONE;
+      else
+        return NS_STYLE_HINT_VISUAL;
     }
     if ((mVisible != aOther.mVisible) && 
         ((NS_STYLE_VISIBILITY_COLLAPSE == mVisible) || 
@@ -1160,7 +1192,10 @@ nsStyleContent::CalcDifference(const nsStyleContent& aOther) const
       while (0 < ix--) {
         if ((mContents[ix].mType != aOther.mContents[ix].mType) || 
             (mContents[ix].mContent != aOther.mContents[ix].mContent)) {
-          return NS_STYLE_HINT_REFLOW;
+          // Unfortunately we need to reframe here; a simple reflow
+          // will not pick up different text or different image URLs,
+          // since we set all that up in the CSSFrameConstructor
+          return NS_STYLE_HINT_FRAMECHANGE;
         }
       }
       ix = mIncrementCount;
@@ -1356,6 +1391,7 @@ nsStyleUIReset::nsStyleUIReset(void)
   mUserSelect = NS_STYLE_USER_SELECT_AUTO;
   mKeyEquivalent = PRUnichar(0); // XXX what type should this be?
   mResizer = NS_STYLE_RESIZER_AUTO;
+  mForceBrokenImageIcon = 0;
 }
 
 nsStyleUIReset::nsStyleUIReset(const nsStyleUIReset& aSource) 
@@ -1363,6 +1399,7 @@ nsStyleUIReset::nsStyleUIReset(const nsStyleUIReset& aSource)
   mUserSelect = aSource.mUserSelect;
   mKeyEquivalent = aSource.mKeyEquivalent;
   mResizer = aSource.mResizer;
+  mForceBrokenImageIcon = aSource.mForceBrokenImageIcon;
 }
 
 nsStyleUIReset::~nsStyleUIReset(void) 
@@ -1371,8 +1408,9 @@ nsStyleUIReset::~nsStyleUIReset(void)
 
 PRInt32 nsStyleUIReset::CalcDifference(const nsStyleUIReset& aOther) const
 {
-  if (mResizer == aOther.mResizer) {
-    if (mUserSelect == aOther.mUserSelect) {
+  if (mForceBrokenImageIcon == aOther.mForceBrokenImageIcon) {
+    if (mResizer == aOther.mResizer &&
+        mUserSelect == aOther.mUserSelect) {
       if (mKeyEquivalent == aOther.mKeyEquivalent) {
         return NS_STYLE_HINT_NONE;
       }
@@ -1380,6 +1418,5 @@ PRInt32 nsStyleUIReset::CalcDifference(const nsStyleUIReset& aOther) const
     }
     return NS_STYLE_HINT_VISUAL;
   }
-  
-  return NS_STYLE_HINT_VISUAL;
+  return NS_STYLE_HINT_FRAMECHANGE;
 }
