@@ -103,7 +103,7 @@
 class nsIAnonymousContentList : public nsISupports
 {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IANONYMOUSCONTENTLIST_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IANONYMOUSCONTENTLIST_IID)
 
    // nsIAnonymousContentList
   NS_IMETHOD GetInsertionPointCount(PRUint32* aCount)=0;
@@ -254,9 +254,9 @@ public:
   NS_IMETHOD GetSingleInsertionPoint(nsIContent* aParent, nsIContent** aResult, PRUint32* aIndex,  
                                      PRBool* aMultipleInsertionPoints);
 
-  NS_IMETHOD AddLayeredBinding(nsIContent* aContent, const nsAReadableString& aURL);
-  NS_IMETHOD RemoveLayeredBinding(nsIContent* aContent, const nsAReadableString& aURL);
-  NS_IMETHOD LoadBindingDocument(nsIDocument* aBoundDoc, const nsAReadableString& aURL,
+  NS_IMETHOD AddLayeredBinding(nsIContent* aContent, const nsAString& aURL);
+  NS_IMETHOD RemoveLayeredBinding(nsIContent* aContent, const nsAString& aURL);
+  NS_IMETHOD LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& aURL,
                                  nsIDocument** aResult);
 
   NS_IMETHOD AddToAttachedQueue(nsIXBLBinding* aBinding);
@@ -304,7 +304,8 @@ public:
                             nsISupports* aSubContent) { return NS_OK; }
   NS_IMETHOD ContentStatesChanged(nsIDocument* aDocument,
                                   nsIContent* aContent1,
-                                  nsIContent* aContent2) { return NS_OK; }
+                                  nsIContent* aContent2,
+                                  PRInt32 aStateMask) { return NS_OK; }
   NS_IMETHOD AttributeChanged(nsIDocument* aDocument,
                               nsIContent*  aContent,
                               PRInt32      aNameSpaceID,
@@ -769,7 +770,7 @@ nsBindingManager::GetSingleInsertionPoint(nsIContent* aParent, nsIContent** aRes
 }
 
 NS_IMETHODIMP
-nsBindingManager::AddLayeredBinding(nsIContent* aContent, const nsAReadableString& aURL)
+nsBindingManager::AddLayeredBinding(nsIContent* aContent, const nsAString& aURL)
 {
   // First we need to load our binding.
   nsresult rv;
@@ -791,7 +792,7 @@ nsBindingManager::AddLayeredBinding(nsIContent* aContent, const nsAReadableStrin
 }
 
 NS_IMETHODIMP
-nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAReadableString& aURL)
+nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAString& aURL)
 {
   /*
   nsCOMPtr<nsIXBLBinding> binding;
@@ -828,20 +829,20 @@ nsBindingManager::RemoveLayeredBinding(nsIContent* aContent, const nsAReadableSt
 }
 
 NS_IMETHODIMP
-nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAReadableString& aURL,
+nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAString& aURL,
                                       nsIDocument** aResult)
 {
-  nsCAutoString url; url.AssignWithConversion(PromiseFlatString(aURL).get());
+  NS_ConvertUCS2toUTF8 url(aURL);
   
-  nsXPIDLCString otherScheme;
-  nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID);
+  nsCAutoString otherScheme;
+  nsCOMPtr<nsIIOService> ioService = do_GetIOService();
   if (!ioService) return NS_ERROR_FAILURE;
-  ioService->ExtractScheme(url.get(), 0, 0, getter_Copies(otherScheme));
+  ioService->ExtractScheme(url, otherScheme);
 
   nsCOMPtr<nsIURI> docURL;
   aBoundDoc->GetDocumentURL(getter_AddRefs(docURL));
-  nsXPIDLCString scheme;
-  docURL->GetScheme(getter_Copies(scheme));
+  nsCAutoString scheme;
+  docURL->GetScheme(scheme);
 
   // First we need to load our binding.
   *aResult = nsnull;
@@ -857,7 +858,7 @@ nsBindingManager::LoadBindingDocument(nsIDocument* aBoundDoc, const nsAReadableS
   if (!info)
     return NS_ERROR_FAILURE;
 
-  if (!PL_strcmp(scheme, otherScheme))
+  if (!strcmp(scheme.get(), otherScheme.get()))
     info->GetDocument(aResult); // Addref happens here.
     
   return NS_OK;
@@ -948,10 +949,10 @@ nsBindingManager::PutXBLDocumentInfo(nsIXBLDocumentInfo* aDocumentInfo)
 
   nsCOMPtr<nsIURI> uri;
   doc->GetDocumentURL(getter_AddRefs(uri));
-  nsXPIDLCString str;
-  uri->GetSpec(getter_Copies(str));
+  nsCAutoString str;
+  uri->GetSpec(str);
   
-  nsCStringKey key((const char*)str);
+  nsCStringKey key(str.get());
   mDocumentTable->Put(&key, aDocumentInfo);
   return NS_OK;
 }
@@ -967,10 +968,10 @@ nsBindingManager::RemoveXBLDocumentInfo(nsIXBLDocumentInfo* aDocumentInfo)
 
   nsCOMPtr<nsIURI> uri;
   doc->GetDocumentURL(getter_AddRefs(uri));
-  nsXPIDLCString str;
-  uri->GetSpec(getter_Copies(str));
+  nsCAutoString str;
+  uri->GetSpec(str);
   
-  nsCStringKey key((const char*)str);
+  nsCStringKey key(str.get());
   mDocumentTable->Remove(&key);
   return NS_OK;
 }
@@ -1035,11 +1036,11 @@ PRBool PR_CALLBACK MarkForDeath(nsHashKey* aKey, void* aData, void* aClosure)
   nsCAutoString uriStr;
   binding->GetDocURI(uriStr);
   nsCOMPtr<nsIURI> uri;
-  NS_NewURI(getter_AddRefs(uri), uriStr.get());
+  NS_NewURI(getter_AddRefs(uri), uriStr);
   if (uri) {
-    nsXPIDLCString path;
-    uri->GetPath(getter_Copies(path));
-    if (!PL_strncmp(path.get(), "/skin", 5))
+    nsCAutoString path;
+    uri->GetPath(path);
+    if (!strncmp(path.get(), "/skin", 5))
       binding->MarkForDeath();
   }
   return PR_TRUE;
@@ -1048,7 +1049,8 @@ PRBool PR_CALLBACK MarkForDeath(nsHashKey* aKey, void* aData, void* aClosure)
 NS_IMETHODIMP
 nsBindingManager::FlushSkinBindings()
 {
-  mBindingTable->Enumerate(MarkForDeath);
+  if (mBindingTable)
+    mBindingTable->Enumerate(MarkForDeath);
   return NS_OK;
 }
 
