@@ -40,6 +40,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsCOMPtr.h"
+#include "nsIAtom.h"
 #include "nsIXBLBinding.h"
 #include "nsIXBLDocumentInfo.h"
 #include "nsIInputStream.h"
@@ -73,6 +74,7 @@
 #include "nsIXPConnect.h"
 #include "nsIScriptGlobalObjectOwner.h"
 #include "nsIScriptContext.h"
+#include "nsCRT.h"
 
 // Event listeners
 #include "nsIEventListenerManager.h"
@@ -111,9 +113,6 @@
 
 #include "nsXBLBinding.h"
 
-// Static IIDs/CIDs. Try to minimize these.
-static char kNameSpaceSeparator = ':';
-
 // Helper classes
 
 /***********************************************************************/
@@ -127,7 +126,7 @@ XBLFinalize(JSContext *cx, JSObject *obj)
   c->Drop();
 }
 
-nsXBLJSClass::nsXBLJSClass(const nsCString& aClassName)
+nsXBLJSClass::nsXBLJSClass(const nsAFlatCString& aClassName)
 {
   memset(this, 0, sizeof(nsXBLJSClass));
   next = prev = NS_STATIC_CAST(JSCList*, this);
@@ -504,7 +503,7 @@ PRBool PR_CALLBACK BuildContentLists(nsHashKey* aKey, void* aData, void* aClosur
     }
     
     if (!pseudoPoint) {
-      NS_NewXBLInsertionPoint(parent.get(), -1, nsnull, getter_AddRefs(pseudoPoint));
+      NS_NewXBLInsertionPoint(parent.get(), (PRUint32)-1, nsnull, getter_AddRefs(pseudoPoint));
       contentList->AppendElement(pseudoPoint);
     }
 
@@ -842,13 +841,15 @@ nsXBLBinding::InstallEventHandlers()
     nsCOMPtr<nsIXBLPrototypeHandler> handlerChain;
     mPrototypeBinding->GetPrototypeHandlers(getter_AddRefs(handlerChain));
   
-    nsCOMPtr<nsIXBLPrototypeHandler> curr = handlerChain;
     nsXBLEventHandler* currHandler = nsnull;
 
-    while (curr) {
+    for (nsCOMPtr<nsIXBLPrototypeHandler> next, curr = handlerChain; curr; 
+         curr->GetNextHandler(getter_AddRefs(next)), curr = next) {
       // Fetch the event type.
       nsCOMPtr<nsIAtom> eventAtom;
       curr->GetEventName(getter_AddRefs(eventAtom));
+      if (!eventAtom) 
+        continue;
 
       nsCOMPtr<nsIDOMEventReceiver> receiver = do_QueryInterface(mBoundElement);
       // Figure out if we're using capturing or not.
@@ -981,10 +982,6 @@ nsXBLBinding::InstallEventHandlers()
         // Let the listener manager hold on to the handler.
         NS_RELEASE(handler);
       }
-      
-      nsCOMPtr<nsIXBLPrototypeHandler> next;
-      curr->GetNextHandler(getter_AddRefs(next));
-      curr = next;
     }
   }
 
@@ -1233,42 +1230,18 @@ nsXBLBinding::AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc, void* aData,
 
 // Internal helper methods ////////////////////////////////////////////////////////////////
 
-NS_IMETHODIMP
-nsXBLBinding::InitClass(const nsCString& aClassName, nsIScriptContext* aContext, 
-                        nsIDocument* aDocument, void** aScriptObject, void** aClassObject)
+// static
+nsresult
+nsXBLBinding::DoInitJSClass(JSContext *cx, JSObject *global, JSObject *obj,
+                            const nsAFlatCString& aClassName,
+                            void **aClassObject)
 {
-  *aClassObject = nsnull;
-  *aScriptObject = nsnull;
-
-  nsresult rv;
-
-  // Obtain the bound element's current script object.
-  nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSContext* jscontext = (JSContext*)aContext->GetNativeContext();
-
-  nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
-
-  JSObject* global = ::JS_GetGlobalObject(jscontext);
-
-  rv = xpc->WrapNative(jscontext, global, mBoundElement,
-                       NS_GET_IID(nsISupports), getter_AddRefs(wrapper));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  JSObject* object = nsnull;
-
-  rv = wrapper->GetJSObject(&object);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  *aScriptObject = object;
-
   // First ensure our JS class is initialized.
-  jsval vp;
+  jsval val;
   JSObject* proto;
 
-  if ((! ::JS_LookupProperty(jscontext, global, aClassName.get(), &vp)) ||
-      JSVAL_IS_PRIMITIVE(vp)) {
+  if ((!::JS_LookupProperty(cx, global, aClassName.get(), &val)) ||
+      JSVAL_IS_PRIMITIVE(val)) {
     // We need to initialize the class.
 
     nsXBLJSClass* c;
@@ -1289,6 +1262,7 @@ nsXBLBinding::InitClass(const nsCString& aClassName, nsIScriptContext* aContext,
       if (JS_CLIST_IS_EMPTY(&nsXBLService::gClassLRUList)) {
         // We need to create a struct for this class.
         c = new nsXBLJSClass(aClassName);
+
         if (!c)
           return NS_ERROR_OUT_OF_MEMORY;
       } else {
@@ -1310,41 +1284,88 @@ nsXBLBinding::InitClass(const nsCString& aClassName, nsIScriptContext* aContext,
       // Add c to our table.
       (nsXBLService::gClassTable)->Put(&key, (void*)c);
     }
-    
-    // Retrieve the current prototype of the JS object.
-    JSObject* parent_proto = ::JS_GetPrototype(jscontext, object);
+
+    // Retrieve the current prototype of obj.
+    JSObject* parent_proto = ::JS_GetPrototype(cx, obj);
+
+    // The prototype holds a strong reference to its class struct.
+    c->Hold();
 
     // Make a new object prototyped by parent_proto and parented by global.
-    proto = ::JS_InitClass(jscontext,           // context
+    proto = ::JS_InitClass(cx,                  // context
                            global,              // global object
                            parent_proto,        // parent proto 
                            c,                   // JSClass
-                           NULL,                // JSNative ctor
+                           nsnull,              // JSNative ctor
                            0,                   // ctor args
                            nsnull,              // proto props
                            nsnull,              // proto funcs
                            nsnull,              // ctor props (static)
                            nsnull);             // ctor funcs (static)
     if (!proto) {
+      // This will happen if we're OOM or if the security manager
+      // denies defining the new class...
+
       (nsXBLService::gClassTable)->Remove(&key);
-      delete c;
+
+      c->Drop();
+
       return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    // The prototype holds a strong reference to its class struct.
-    c->Hold();
     *aClassObject = (void*)proto;
   }
   else {
-    proto = JSVAL_TO_OBJECT(vp);
+    proto = JSVAL_TO_OBJECT(val);
   }
 
   // Set the prototype of our object to be the new class.
-  ::JS_SetPrototype(jscontext, object, proto);
+  if (!::JS_SetPrototype(cx, obj, proto)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP
+nsXBLBinding::InitClass(const nsCString& aClassName,
+                        nsIScriptContext* aContext, 
+                        nsIDocument* aDocument, void** aScriptObject,
+                        void** aClassObject)
+{
+  *aClassObject = nsnull;
+  *aScriptObject = nsnull;
+
+  nsresult rv;
+
+  // Obtain the bound element's current script object.
+  nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  JSContext* cx = (JSContext*)aContext->GetNativeContext();
+
+  nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
+
+  JSObject* global = ::JS_GetGlobalObject(cx);
+
+  rv = xpc->WrapNative(cx, global, mBoundElement, NS_GET_IID(nsISupports),
+                       getter_AddRefs(wrapper));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  JSObject* object = nsnull;
+  rv = wrapper->GetJSObject(&object);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  *aScriptObject = object;
+
+  // First ensure our JS class is initialized.
+
+  rv = DoInitJSClass(cx, global, object, aClassName, aClassObject);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Root mBoundElement so that it doesn't loose it's binding
   nsCOMPtr<nsIDocument> doc;
-
   mBoundElement->GetDocument(*getter_AddRefs(doc));
 
   if (doc) {
@@ -1392,7 +1413,7 @@ nsXBLBinding::IsInExcludesList(nsIAtom* aTag, const nsString& aList)
   if (aList == NS_LITERAL_STRING("*"))
       return PR_TRUE; // match _everything_!
 
-  PRInt32 indx = aList.Find(element);
+  PRInt32 indx = aList.Find(element, 0);
   if (indx == -1)
     return PR_FALSE; // not in the list at all
 
