@@ -38,6 +38,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsCOMPtr.h"
+#include "nsIAtom.h"
 #include "nsIXBLPrototypeBinding.h"
 #include "nsIXBLDocumentInfo.h"
 #include "nsIInputStream.h"
@@ -77,6 +78,7 @@
 #include "nsXULAtoms.h"
 #include "nsXBLAtoms.h"
 #include "nsXBLProtoImpl.h"
+#include "nsCRT.h"
 
 #include "nsIScriptContext.h"
 
@@ -94,7 +96,7 @@
 
 class nsIXBLAttributeEntry : public nsISupports {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IXBLATTR_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IXBLATTR_IID)
 
   NS_IMETHOD GetSrcAttribute(nsIAtom** aResult) = 0;
   NS_IMETHOD GetDstAttribute(nsIAtom** aResult) = 0;
@@ -143,7 +145,7 @@ protected:
 private:
   // Hide so that only Create() and Destroy() can be used to
   // allocate and deallocate from the heap
-  static void* operator new(size_t) { return 0; }
+  static void* operator new(size_t) CPP_THROW_NEW { return 0; }
   static void operator delete(void*, size_t) {}
 };
 
@@ -160,7 +162,7 @@ NS_IMPL_QUERY_INTERFACE1(nsXBLAttributeEntry, nsIXBLAttributeEntry)
 
 class nsIXBLInsertionPointEntry : public nsISupports {
 public:
-  static const nsIID& GetIID() { static nsIID iid = NS_IXBLINS_IID; return iid; }
+  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IXBLINS_IID)
 
   NS_IMETHOD GetInsertionParent(nsIContent** aResult)=0;
 
@@ -216,7 +218,7 @@ protected:
 private:
   // Hide so that only Create() and Destroy() can be used to
   // allocate and deallocate from the heap
-  static void* operator new(size_t) { return 0; }
+  static void* operator new(size_t) CPP_THROW_NEW { return 0; }
   static void operator delete(void*, size_t) {}
 };
 
@@ -254,12 +256,12 @@ static const PRInt32 kInsInitialSize = (NS_SIZE_IN_HEAP(sizeof(nsXBLInsertionPoi
 NS_IMPL_ISUPPORTS2(nsXBLPrototypeBinding, nsIXBLPrototypeBinding, nsISupportsWeakReference)
 
 // Constructors/Destructors
-nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsAReadableCString& aID, nsIXBLDocumentInfo* aInfo,
+nsXBLPrototypeBinding::nsXBLPrototypeBinding(const nsACString& aID, nsIXBLDocumentInfo* aInfo,
                                              nsIContent* aElement)
-: mInheritStyle(PR_TRUE), 
+: mImplementation(nsnull),
+  mInheritStyle(PR_TRUE), 
   mHasBaseProto(PR_TRUE),
   mResources(nsnull),
-  mImplementation(nsnull),
   mAttributeTable(nsnull), 
   mInsertionPointTable(nsnull),
   mInterfaceTable(nsnull)
@@ -412,7 +414,7 @@ nsXBLPrototypeBinding::LoadResources(PRBool* aResult)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeBinding::AddResource(nsIAtom* aResourceType, const nsAReadableString& aSrc)
+nsXBLPrototypeBinding::AddResource(nsIAtom* aResourceType, const nsAString& aSrc)
 {
   if (!mResources) {
     mResources = new nsXBLPrototypeResources(this);
@@ -585,8 +587,8 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute, PRInt32 aNameSpaceI
         // children of the real element and get the text nodes' values.
         if (aAttribute == nsXBLAtoms::xbltext) {
           nsXBLBinding::GetTextData(aChangedElement, value);
-          value.StripChar('\n');
-          value.StripChar('\r');
+          value.StripChar(PRUnichar('\n'));
+          value.StripChar(PRUnichar('\r'));
           nsAutoString stripVal(value);
           stripVal.StripWhitespace();
           if (stripVal.IsEmpty()) 
@@ -864,93 +866,19 @@ nsXBLPrototypeBinding::GetImmediateChild(nsIAtom* aTag, nsIContent** aResult)
 }
  
 NS_IMETHODIMP
-nsXBLPrototypeBinding::InitClass(const nsCString& aClassName, nsIScriptContext * aContext, void * aScriptObject, void ** aClassObject)
+nsXBLPrototypeBinding::InitClass(const nsCString& aClassName,
+                                 nsIScriptContext * aContext,
+                                 void * aScriptObject, void ** aClassObject)
 {
   NS_ENSURE_ARG_POINTER (aClassObject); 
 
   *aClassObject = nsnull;
 
-  JSContext* jscontext = (JSContext*)aContext->GetNativeContext();
-  JSObject* global = ::JS_GetGlobalObject(jscontext);
+  JSContext* cx = (JSContext*)aContext->GetNativeContext();
   JSObject* scriptObject = (JSObject*) aScriptObject;
 
-  // First ensure our JS class is initialized.
-  jsval vp;
-  JSObject* proto;
-
-  if ((! ::JS_LookupProperty(jscontext, global, aClassName.get(), &vp)) || JSVAL_IS_PRIMITIVE(vp))  {
-    // We need to initialize the class.
-    nsXBLJSClass* c;
-    void* classObject;
-    nsCStringKey key(aClassName);
-    classObject = (nsXBLService::gClassTable)->Get(&key);
-
-    if (classObject) {
-      c = NS_STATIC_CAST(nsXBLJSClass*, classObject);
-      // If c is on the LRU list (i.e., not linked to itself), remove it now!
-      JSCList* link = NS_STATIC_CAST(JSCList*, c);
-      if (c->next != link) {
-        JS_REMOVE_AND_INIT_LINK(link);
-        nsXBLService::gClassLRUListLength--;
-      }
-    } 
-    else {
-      if (JS_CLIST_IS_EMPTY(&nsXBLService::gClassLRUList)) {
-        // We need to create a struct for this class.
-        c = new nsXBLJSClass(aClassName);
-        if (!c)
-          return NS_ERROR_OUT_OF_MEMORY;
-      } 
-      else  {
-        // Pull the least recently used class struct off the list.
-        JSCList* lru = (nsXBLService::gClassLRUList).next;
-        JS_REMOVE_AND_INIT_LINK(lru);
-        nsXBLService::gClassLRUListLength--;
-
-        // Remove any mapping from the old name to the class struct.
-        c = NS_STATIC_CAST(nsXBLJSClass*, lru);
-        nsCStringKey oldKey(c->name);
-        (nsXBLService::gClassTable)->Remove(&oldKey);
-
-        // Change the class name and we're done.
-        nsMemory::Free((void*) c->name);
-        c->name = ToNewCString(aClassName);
-      }
-
-      // Add c to our table.
-      (nsXBLService::gClassTable)->Put(&key, (void*)c);
-    }
-    
-    // Retrieve the current prototype of the JS object.
-    JSObject* parent_proto = ::JS_GetPrototype(jscontext, scriptObject);
-
-    // Make a new object prototyped by parent_proto and parented by global.
-    proto = ::JS_InitClass(jscontext,           // context
-                           global,              // global object
-                           parent_proto,        // parent proto 
-                           c,                   // JSClass
-                           NULL,                // JSNative ctor
-                           0,                   // ctor args
-                           nsnull,              // proto props
-                           nsnull,              // proto funcs
-                           nsnull,              // ctor props (static)
-                           nsnull);             // ctor funcs (static)
-    if (!proto) {
-      (nsXBLService::gClassTable)->Remove(&key);
-      delete c;
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-    // The prototype holds a strong reference to its class struct.
-    c->Hold();
-    *aClassObject = (void *) proto;
-  }
-  else
-    proto = JSVAL_TO_OBJECT(vp);
-
-  ::JS_SetPrototype(jscontext, scriptObject, proto);
-
-  return NS_OK;
+  return nsXBLBinding::DoInitJSClass(cx, ::JS_GetGlobalObject(cx),
+                                     scriptObject, aClassName, aClassObject);
 }
 
 void
@@ -1070,8 +998,8 @@ PRBool PR_CALLBACK SetAttrs(nsHashKey* aKey, void* aData, void* aClosure)
   PRBool attrPresent = PR_TRUE;
   if (src == nsXBLAtoms::xbltext) {
     nsXBLBinding::GetTextData(changeData->mBoundElement, value);
-    value.StripChar('\n');
-    value.StripChar('\r');
+    value.StripChar(PRUnichar('\n'));
+    value.StripChar(PRUnichar('\r'));
     nsAutoString stripVal(value);
     stripVal.StripWhitespace();
 
@@ -1175,7 +1103,8 @@ void
 nsXBLPrototypeBinding::ConstructAttributeTable(nsIContent* aElement)
 {
   nsAutoString inherits;
-  aElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::inherits, inherits);
+  aElement->GetAttr(kNameSpaceID_XBL, nsXBLAtoms::inherits, inherits);
+
   if (!inherits.IsEmpty()) {
     if (!mAttributeTable) {
       mAttributeTable = new nsSupportsHashtable(4);
@@ -1239,7 +1168,7 @@ nsXBLPrototypeBinding::ConstructAttributeTable(nsIContent* aElement)
       // by the template only, and we don't need it anymore.
       // XXXdwh Don't do this for XUL elements, since it faults them into heavyweight
       // elements. Should nuke from the prototype instead.
-      // aElement->UnsetAttr(kNameSpaceID_None, nsXBLAtoms::inherits, PR_FALSE);
+      // aElement->UnsetAttr(kNameSpaceID_XBL, nsXBLAtoms::inherits, PR_FALSE);
 
       token = nsCRT::strtok( newStr, ", ", &newStr );
     }
@@ -1346,7 +1275,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeBinding::ConstructInterfaceTable(const nsAReadableString& aImpls)
+nsXBLPrototypeBinding::ConstructInterfaceTable(const nsAString& aImpls)
 {
   if (!aImpls.IsEmpty()) {
     // Obtain the interface info manager that can tell us the IID
@@ -1420,7 +1349,7 @@ nsXBLPrototypeBinding::AddResourceListener(nsIContent* aBoundElement)
 // Creation Routine ///////////////////////////////////////////////////////////////////////
 
 nsresult
-NS_NewXBLPrototypeBinding(const nsAReadableCString& aRef, nsIContent* aElement, 
+NS_NewXBLPrototypeBinding(const nsACString& aRef, nsIContent* aElement, 
                           nsIXBLDocumentInfo* aInfo, nsIXBLPrototypeBinding** aResult)
 {
   nsXBLPrototypeBinding * binding = new nsXBLPrototypeBinding(aRef, aInfo, aElement);
