@@ -78,6 +78,7 @@
 #include "nsDOMCID.h"
 #include "nsUnicharUtils.h"
 #include "nsReadableUtils.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
                      NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
@@ -96,10 +97,10 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent, const PRUn
                                              const PRUnichar* aAction, const PRUnichar* aCommand,
                                              const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
                                              const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                             const PRUnichar* aClickCount)
+                                             const PRUnichar* aClickCount, const PRUnichar* aPreventDefault)
 {
   NS_INIT_REFCNT();
-  gRefCnt++;
+  ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
     InitAccessKeys();
@@ -107,14 +108,14 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(const PRUnichar* aEvent, const PRUn
   mHandlerText = nsnull;
 
   ConstructPrototype(nsnull, aEvent, aPhase, aAction, aCommand, aKeyCode, aCharCode, aModifiers,
-                     aButton, aClickCount);
+                     aButton, aClickCount, aPreventDefault);
 }
 
 nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
 {
   NS_INIT_REFCNT();
   
-  gRefCnt++;
+  ++gRefCnt;
   if (gRefCnt == 1)
     // Get the primary accelerator key.
     InitAccessKeys();
@@ -125,8 +126,8 @@ nsXBLPrototypeHandler::nsXBLPrototypeHandler(nsIContent* aHandlerElement)
 
 nsXBLPrototypeHandler::~nsXBLPrototypeHandler()
 {
-  gRefCnt--;
-  if (mType != NS_HANDLER_TYPE_XUL && mHandlerText)
+  --gRefCnt;
+  if (!(mType & NS_HANDLER_TYPE_XUL) && mHandlerText)
     nsMemory::Free(mHandlerText);
 }
 
@@ -135,7 +136,7 @@ NS_IMPL_ISUPPORTS1(nsXBLPrototypeHandler, nsIXBLPrototypeHandler)
 NS_IMETHODIMP
 nsXBLPrototypeHandler::GetHandlerElement(nsIContent** aResult)
 {
-  if (mType == NS_HANDLER_TYPE_XUL) {
+  if (mType & NS_HANDLER_TYPE_XUL) {
     *aResult = mHandlerElement;
     NS_IF_ADDREF(*aResult);
   }
@@ -145,7 +146,7 @@ nsXBLPrototypeHandler::GetHandlerElement(nsIContent** aResult)
 }
 
 NS_IMETHODIMP
-nsXBLPrototypeHandler::AppendHandlerText(const nsAReadableString& aText) 
+nsXBLPrototypeHandler::AppendHandlerText(const nsAString& aText) 
 {
   if (mHandlerText) {
     // Append our text to the existing text.
@@ -206,11 +207,21 @@ NS_IMETHODIMP
 nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
                                       nsIDOMEvent* aEvent)
 {
+  nsresult rv = NS_ERROR_FAILURE;
+
+  // Prevent default action?
+  if (mType & NS_HANDLER_TYPE_PREVENTDEFAULT) {
+    aEvent->PreventDefault();
+    // If we prevent default, then it's okay for
+    // mHandlerElement and mHandlerText to be null
+    rv = NS_OK;
+  }
+
   if (!mHandlerElement) // This works for both types of handlers.  In both cases, the union's var should be defined.
-    return NS_ERROR_FAILURE;
+    return rv;
 
   // See if our event receiver is a content node (and not us).
-  PRBool isXULKey = (mType == NS_HANDLER_TYPE_XUL);
+  PRBool isXULKey = (mType & NS_HANDLER_TYPE_XUL);
 
   //XUL handlers shouldn't be triggered by non-trusted events.
   if (isXULKey) {
@@ -230,7 +241,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
 
   // This is a special-case optimization to make command handling fast.
   // It isn't really a part of XBL, but it helps speed things up.
-  if (mType == NS_HANDLER_TYPE_XBL_COMMAND && !isReceiverCommandElement) {
+  if ((mType & NS_HANDLER_TYPE_XBL_COMMAND) && !isReceiverCommandElement) {
     // See if preventDefault has been set.  If so, don't execute.
     PRBool preventDefault;
     nsCOMPtr<nsIDOMNSUIEvent> nsUIEvent(do_QueryInterface(aEvent));
@@ -279,10 +290,10 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
       privateWindow->GetRootFocusController(getter_AddRefs(focusController));
     }
 
-    nsDependentString command(mHandlerText);
-
+    nsCAutoString command;
+    command.AssignWithConversion(mHandlerText);
     if (focusController)
-      focusController->GetControllerForCommand(command, getter_AddRefs(controller));
+      focusController->GetControllerForCommand(command.get(), getter_AddRefs(controller));
     else GetController(aReceiver, getter_AddRefs(controller)); // We're attached to the receiver possibly.
 
     nsAutoString type;
@@ -312,7 +323,7 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     aEvent->PreventDefault();
     
     if (controller)
-      controller->DoCommand(command);
+      controller->DoCommand(command.get());
 
     return NS_OK;
   }
@@ -383,6 +394,11 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     boundDocument->GetScriptGlobalObject(getter_AddRefs(boundGlobal));
   }
 
+  // If we still don't have a 'boundGlobal', we're doomed. bug 95465.
+  NS_ASSERTION(boundGlobal, "failed to get the nsIScriptGlobalObject. bug 95465?");
+  if (!boundGlobal)
+    return NS_OK;
+
   nsCOMPtr<nsIScriptContext> boundContext;
   boundGlobal->GetContext(getter_AddRefs(boundContext));
   if (!boundContext) return NS_OK;
@@ -395,7 +411,6 @@ nsXBLPrototypeHandler::ExecuteHandler(nsIDOMEventReceiver* aReceiver,
     JSObject *global = boundGlobal->GetGlobalJSObject();
     JSContext *cx = (JSContext *)boundContext->GetNativeContext();
 
-    nsresult rv;
     nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
 
     // root
@@ -766,7 +781,7 @@ static const keyCodeData gKeyCodes[] = {
 
 };
 
-PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAReadableString& aKeyName)
+PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAString& aKeyName)
 {
   nsCAutoString keyName;
   keyName.AssignWithConversion(aKeyName);
@@ -775,7 +790,7 @@ PRInt32 nsXBLPrototypeHandler::GetMatchingKeyCode(const nsAReadableString& aKeyN
 
   PRUint32 keyNameLength = keyName.Length();
   const char* keyNameStr = keyName.get();
-  for (int i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
+  for (PRUint16 i = 0; i < (sizeof(gKeyCodes) / sizeof(gKeyCodes[0])); ++i)
     if (keyNameLength == gKeyCodes[i].strlength &&
         !nsCRT::strcmp(gKeyCodes[i].str, keyNameStr))
       return gKeyCodes[i].keycode;
@@ -803,11 +818,11 @@ PRInt32 nsXBLPrototypeHandler::KeyToMask(PRInt32 key)
 }
 
 void
-nsXBLPrototypeHandler::GetEventType(nsAWritableString& aEvent)
+nsXBLPrototypeHandler::GetEventType(nsAString& aEvent)
 {
   mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::event, aEvent);
   
-  if (aEvent.IsEmpty() && mType == NS_HANDLER_TYPE_XUL)
+  if (aEvent.IsEmpty() && (mType & NS_HANDLER_TYPE_XUL))
     // If no type is specified for a XUL <key> element, let's assume that we're "keypress".
     aEvent = NS_LITERAL_STRING("keypress");
 }
@@ -818,14 +833,16 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
                                           const PRUnichar* aAction, const PRUnichar* aCommand,
                                           const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
                                           const PRUnichar* aModifiers, const PRUnichar* aButton,
-                                          const PRUnichar* aClickCount)
+                                          const PRUnichar* aClickCount, const PRUnichar* aPreventDefault)
 {
+  mType = 0;
+
   if (aKeyElement) {
-    mType = NS_HANDLER_TYPE_XUL;
+    mType |= NS_HANDLER_TYPE_XUL;
     mHandlerElement = aKeyElement;
   }
   else {
-    mType = aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
+    mType |= aCommand ? NS_HANDLER_TYPE_XBL_COMMAND : NS_HANDLER_TYPE_XBL_JS;
     mHandlerText = nsnull;
   }
 
@@ -841,35 +858,33 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   nsAutoString event(aEvent);
   if (event.IsEmpty()) {
-    if (mType == NS_HANDLER_TYPE_XUL)
+    if (mType & NS_HANDLER_TYPE_XUL)
       GetEventType(event);
     if (event.IsEmpty())
       return;
   }
 
-  if (event.IsEmpty())
-    return;
   mEventName = getter_AddRefs(NS_NewAtom(event));
 
   if (aPhase) {
-    if (Compare(nsDependentString(aPhase), NS_LITERAL_STRING("capturing")) == 0)
+    const nsDependentString phase(aPhase);
+    if (phase.Equals(NS_LITERAL_STRING("capturing")))
       mPhase = NS_PHASE_CAPTURING;
-    else if (Compare(nsDependentString(aPhase), NS_LITERAL_STRING("target")) == 0)
+    else if (phase.Equals(NS_LITERAL_STRING("target")))
       mPhase = NS_PHASE_TARGET;
   }
 
   // Button and clickcount apply only to XBL handlers and don't apply to XUL key
   // handlers.  
-  nsAutoString button(aButton);
-  nsAutoString clickcount(aClickCount);
-  if (!button.IsEmpty())
-    mDetail = button.First() - '0';
-  if (!clickcount.IsEmpty())
-    mMisc = clickcount.First() - '0';
+  if (aButton && *aButton)
+    mDetail = *aButton - '0';
+
+  if (aClickCount && *aClickCount)
+    mMisc = *aClickCount - '0';
 
   // Modifiers are supported by both types of handlers (XUL and XBL).  
   nsAutoString modifiers(aModifiers);
-  if (mType == NS_HANDLER_TYPE_XUL)
+  if (mType & NS_HANDLER_TYPE_XUL)
     mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::modifiers, modifiers);
   
   if (!modifiers.IsEmpty()) {
@@ -898,7 +913,7 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
 
   nsAutoString key(aCharCode);
   if (key.IsEmpty()) {
-    if (mType == NS_HANDLER_TYPE_XUL) {
+    if (mType & NS_HANDLER_TYPE_XUL) {
       mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::key, key);
       if (key.IsEmpty()) 
         mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::charcode, key);
@@ -917,13 +932,16 @@ nsXBLPrototypeHandler::ConstructPrototype(nsIContent* aKeyElement,
   }
   else {
     key.Assign(aKeyCode);
-    if (mType == NS_HANDLER_TYPE_XUL)
+    if (mType & NS_HANDLER_TYPE_XUL)
       mHandlerElement->GetAttr(kNameSpaceID_None, nsXBLAtoms::keycode, key);
     
     if (!key.IsEmpty())
       mDetail = GetMatchingKeyCode(key);
   }
 
+  nsAutoString preventDefault(aPreventDefault);
+  if (preventDefault.Equals(NS_LITERAL_STRING("true")))
+    mType |= NS_HANDLER_TYPE_PREVENTDEFAULT;
 }
 
 PRBool
@@ -982,11 +1000,12 @@ NS_NewXBLPrototypeHandler(const PRUnichar* aEvent, const PRUnichar* aPhase,
                           const PRUnichar* aAction, const PRUnichar* aCommand,
                           const PRUnichar* aKeyCode, const PRUnichar* aCharCode,
                           const PRUnichar* aModifiers, const PRUnichar* aButton,
-                          const PRUnichar* aClickCount, 
+                          const PRUnichar* aClickCount, const PRUnichar* aPreventDefault,
                           nsIXBLPrototypeHandler** aResult)
 {
   *aResult = new nsXBLPrototypeHandler(aEvent, aPhase, aAction, aCommand, aKeyCode,
-                                       aCharCode, aModifiers, aButton, aClickCount);
+                                       aCharCode, aModifiers, aButton, aClickCount,
+                                       aPreventDefault);
   if (!*aResult)
     return NS_ERROR_OUT_OF_MEMORY;
   NS_ADDREF(*aResult);
