@@ -76,14 +76,16 @@
 #include "nsIXBLDocumentInfo.h"
 #include "nsXBLAtoms.h"
 #include "nsXULAtoms.h"
+#include "nsCRT.h"
 
 #include "nsIXBLPrototypeHandler.h"
 
-#include "nsIChromeRegistry.h"
 #include "nsIPref.h"
 
 #include "nsIPresShell.h"
 #include "nsIDocumentObserver.h"
+#include "nsIFrameManager.h"
+#include "nsIStyleContext.h"
 
 #include "nsIXULPrototypeCache.h"
 #include "nsIDOMLoadListener.h"
@@ -151,14 +153,31 @@ public:
     if (parent)
       parent->IndexOf(mBoundElement, index);
         
+    // If |mBoundElement| is (in addition to having binding |mBinding|)
+    // also a descendant of another element with binding |mBinding|,
+    // then we might have just constructed it due to the
+    // notification of its parent.  (We can know about both if the
+    // binding loads were triggered from the DOM rather than frame
+    // construction.)  So we have to check both whether the element
+    // has a primary frame and whether it's in the undisplayed map
+    // before sending a ContentInserted notification, or bad things
+    // will happen.
     nsCOMPtr<nsIPresShell> shell;
     doc->GetShellAt(0, getter_AddRefs(shell));
     if (shell) {
       nsIFrame* childFrame;
       shell->GetPrimaryFrameFor(mBoundElement, &childFrame);
-      nsCOMPtr<nsIDocumentObserver> obs(do_QueryInterface(shell));
-      if (!childFrame)
-        obs->ContentInserted(doc, parent, mBoundElement, index);
+      if (!childFrame) {
+        // Check to see if it's in the undisplayed content map.
+        nsCOMPtr<nsIFrameManager> frameManager;
+        shell->GetFrameManager(getter_AddRefs(frameManager));
+        nsCOMPtr<nsIStyleContext> sc;
+        frameManager->GetUndisplayedContent(mBoundElement, getter_AddRefs(sc));
+        if (!sc) {
+          nsCOMPtr<nsIDocumentObserver> obs(do_QueryInterface(shell));
+          obs->ContentInserted(doc, parent, mBoundElement, index);
+        }
+      }
     }
   }
 
@@ -191,7 +210,7 @@ protected:
 private:
   // Hide so that only Create() and Destroy() can be used to
   // allocate and deallocate from the heap
-  static void* operator new(size_t) { return 0; }
+  static void* operator new(size_t) CPP_THROW_NEW { return 0; }
   static void operator delete(void*, size_t) {}
 };
 
@@ -312,9 +331,9 @@ nsXBLStreamListener::OnStopRequest(nsIRequest* request, nsISupports* aCtxt, nsre
   	{
       nsCOMPtr<nsIURI> channelURI;
       aChannel->GetURI(getter_AddRefs(channelURI));
-      nsXPIDLCString str;
-      channelURI->GetSpec(getter_Copies(str));
-      printf("Failed to load XBL document %s\n", (const char*)str);
+      nsCAutoString str;
+      channelURI->GetAsciiSpec(str);
+      printf("Failed to load XBL document %s\n", str.get());
   	}
 
     PRUint32 count = mBindingRequests.Count();
@@ -377,9 +396,9 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
     doc->GetBindingManager(getter_AddRefs(bindingManager));
     nsCOMPtr<nsIURI> uri;
     mBindingDocument->GetDocumentURL(getter_AddRefs(uri));
-    nsXPIDLCString str;
-    uri->GetSpec(getter_Copies(str));
-    bindingManager->RemoveLoadingDocListener(nsCAutoString(NS_STATIC_CAST(const char*, str)));
+    nsCAutoString str;
+    uri->GetSpec(str);
+    bindingManager->RemoveLoadingDocListener(str);
 
     nsCOMPtr<nsIContent> root;
     mBindingDocument->GetRootContent(getter_AddRefs(root));
@@ -392,7 +411,7 @@ nsXBLStreamListener::Load(nsIDOMEvent* aEvent)
     nsCOMPtr<nsIXBLDocumentInfo> info;
     nsCOMPtr<nsIBindingManager> xblDocBindingManager;
     mBindingDocument->GetBindingManager(getter_AddRefs(xblDocBindingManager));
-    xblDocBindingManager->GetXBLDocumentInfo(nsCAutoString(NS_STATIC_CAST(const char*, str)), getter_AddRefs(info));
+    xblDocBindingManager->GetXBLDocumentInfo(str, getter_AddRefs(info));
     xblDocBindingManager->RemoveXBLDocumentInfo(info); // Break the self-imposed cycle.
     if (!info) {
       NS_ERROR("An XBL file is malformed.  Did you forget the XBL namespace on the bindings tag?");
@@ -500,16 +519,9 @@ public:
     return rv;
   }
 
-  NS_IMETHOD GetNonBlocking(PRBool *aNonBlocking) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-
-  NS_IMETHOD GetObserver(nsIInputStreamObserver * *aObserver) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-
-  NS_IMETHOD SetObserver(nsIInputStreamObserver * aObserver) {
-    return NS_ERROR_NOT_IMPLEMENTED;
+  NS_IMETHOD IsNonBlocking(PRBool *aNonBlocking) {
+    *aNonBlocking = PR_TRUE;
+    return NS_OK;
   }
 
   // Implementation
@@ -613,7 +625,7 @@ nsXBLService::~nsXBLService(void)
 // This function loads a particular XBL file and installs all of the bindings
 // onto the element.
 NS_IMETHODIMP
-nsXBLService::LoadBindings(nsIContent* aContent, const nsAReadableString& aURL, PRBool aAugmentFlag,
+nsXBLService::LoadBindings(nsIContent* aContent, const nsAString& aURL, PRBool aAugmentFlag,
                            nsIXBLBinding** aBinding, PRBool* aResolveStyle) 
 { 
   *aBinding = nsnull;
@@ -647,7 +659,7 @@ nsXBLService::LoadBindings(nsIContent* aContent, const nsAReadableString& aURL,
         // See if the URIs match.
         nsCAutoString uri;
         styleBinding->GetBindingURI(uri);
-        if (uri.EqualsWithConversion((const PRUnichar *) PromiseFlatString(aURL).get()))
+        if (uri.EqualsWithConversion(NS_ConvertUCS2toUTF8(aURL).get()))
           return NS_OK;
         else {
           FlushStyleBindings(aContent);
@@ -1018,7 +1030,7 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
 
       nsAutoString nameSpace;
 
-      if (prefix.Length() > 0) {
+      if (!prefix.IsEmpty()) {
         nsCOMPtr<nsIAtom> prefixAtom = getter_AddRefs(NS_NewAtom(prefix));
 
         nsCOMPtr<nsIDOM3Node> node(do_QueryInterface(child));
@@ -1047,13 +1059,12 @@ NS_IMETHODIMP nsXBLService::GetBindingInternal(nsIContent* aBoundElement,
       if (hasExtends && (hasDisplay || nameSpace.IsEmpty())) {
         // Look up the prefix.
         // We have a base class binding. Load it right now.
-        nsCAutoString urlCString; urlCString.AssignWithConversion(value);
+        NS_ConvertUCS2toUTF8 urlCString(value);
         nsCOMPtr<nsIURI> docURI;
         doc->GetDocumentURL(getter_AddRefs(docURI));
-        nsXPIDLCString urlStr;
-        docURI->Resolve(urlCString.get(), getter_Copies(urlStr));
-        urlCString = urlStr.get();
-        if (NS_FAILED(GetBindingInternal(aBoundElement, urlCString, aPeekOnly, aIsReady, getter_AddRefs(baseBinding))))
+        nsCAutoString urlStr;
+        docURI->Resolve(urlCString, urlStr);
+        if (NS_FAILED(GetBindingInternal(aBoundElement, urlStr, aPeekOnly, aIsReady, getter_AddRefs(baseBinding))))
           return NS_ERROR_FAILURE; // Binding not yet ready or an error occurred.
         if (!aPeekOnly) {
           // Make sure to set the base prototype.
@@ -1110,9 +1121,11 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
     nsCOMPtr<nsIAtom> tagName;
     if (aBoundElement)
       aBoundElement->GetTag(*getter_AddRefs(tagName));
-    if (!info && bindingManager && (tagName.get() != nsXULAtoms::scrollbar) &&
-        (tagName.get() != nsXULAtoms::thumb) && (tagName.get() != kInputAtom) 
-        && !aForceSyncLoad) {
+    if (!info && bindingManager &&
+        (tagName != nsXULAtoms::scrollbar) &&
+        (tagName != nsXULAtoms::thumb) &&
+        (tagName != kInputAtom) &&
+        (tagName != nsHTMLAtoms::select) && !aForceSyncLoad) {
       // The third line of defense is to investigate whether or not the
       // document is currently being loaded asynchronously.  If so, there's no
       // document yet, but we need to glom on our request so that it will be
@@ -1139,7 +1152,7 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
       // Finally, if all lines of defense fail, we go and fetch the binding
       // document.
       nsCOMPtr<nsIURI> uri;
-      rv = NS_NewURI(getter_AddRefs(uri), aURLStr.get());
+      rv = NS_NewURI(getter_AddRefs(uri), aURLStr);
       NS_ASSERTION(NS_SUCCEEDED(rv), "unable to create a url");
 
       nsCOMPtr<nsIDocument> document;
@@ -1181,6 +1194,106 @@ nsXBLService::LoadBindingDocumentInfo(nsIContent* aBoundElement, nsIDocument* aB
   return NS_OK;
 }
 
+// Helper method for loading an XML doc.
+NS_IMETHODIMP
+nsXBLService::FetchSyncXMLDocument(nsIURI* aURI, nsIDocument** aResult)
+{
+  // Initialize our out pointer to nsnull
+  *aResult = nsnull;
+
+  // Create the XML document
+  nsCOMPtr<nsIDocument> doc;
+  nsresult rv = nsComponentManager::CreateInstance(kXMLDocumentCID, nsnull,
+                                                   NS_GET_IID(nsIDocument),
+                                                   getter_AddRefs(doc));
+
+  if (NS_FAILED(rv)) return rv;
+
+  // XXX This is evil, but we're living in layout, so I'm
+  // just going to do it.
+  nsXMLDocument* xmlDoc = (nsXMLDocument*)(doc.get());
+ 
+  // Now we have to synchronously load the binding file.
+  // Create an XML content sink and a parser. 
+  nsCOMPtr<nsIRequest> request;
+  nsCOMPtr<nsIChannel> channel;
+  rv = NS_NewChannel(getter_AddRefs(channel), aURI, nsnull, nsnull);
+  if (NS_FAILED(rv)) return rv;
+
+  nsCOMPtr<nsIStreamListener> listener;
+    // Now do a blocking synchronous parse of the file.
+  nsCOMPtr<nsIInputStream> in;
+  rv = channel->Open(getter_AddRefs(in));
+
+  // If we couldn't open the channel, then just return.
+  if (NS_FAILED(rv)) return NS_OK;
+
+  request = do_QueryInterface(channel);
+
+  NS_ASSERTION(request != nsnull, "no request info");
+  NS_ASSERTION(in != nsnull, "no input stream");
+  
+  NS_ENSURE_TRUE(in,NS_ERROR_FAILURE);
+
+  nsProxyStream* proxy = new nsProxyStream();
+  NS_ENSURE_TRUE(proxy,NS_ERROR_OUT_OF_MEMORY);
+
+  // Do this after making sure the |channel->Open| succeeded (which it
+  // won't if the file doesn't exist) so that we don't have to go
+  // through the work of breaking the circular references between
+  // content sink, script loader, and document.
+  nsCOMPtr<nsIXMLContentSink> xmlSink;
+  NS_NewXMLContentSink(getter_AddRefs(xmlSink), xmlDoc, aURI, nsnull, nsnull);
+  if (!xmlSink)
+    return NS_ERROR_FAILURE;
+
+  // Call StartDocumentLoad
+  if (NS_FAILED(rv = xmlDoc->StartDocumentLoad("loadAsData", 
+                                               channel, 
+                                               nsnull, 
+                                               nsnull, 
+                                               getter_AddRefs(listener),
+                                               PR_TRUE,
+                                               xmlSink))) {
+    NS_ERROR("Failure to init XML doc prior to load.");
+    return rv;
+  }
+
+  listener->OnStartRequest(request, nsnull);
+  
+  while (PR_TRUE) {
+    char buf[4096];
+    PRUint32 readCount;
+
+    if (NS_FAILED(rv = in->Read(buf, sizeof(buf), &readCount)))
+        break; // error
+  
+    if (readCount == 0)
+        break; // eof
+
+    proxy->SetBuffer(buf, readCount);
+
+    PRUint32 sourceOffset = 0;
+    rv = listener->OnDataAvailable(request, nsnull, proxy, sourceOffset, readCount);
+    sourceOffset += readCount;
+    if (NS_FAILED(rv))
+        break;
+  }
+  listener->OnStopRequest(request, nsnull, NS_OK);
+
+  // don't leak proxy!
+  proxy->Close();
+  delete proxy;
+  
+  // The document is parsed. We now have a prototype document.
+  // Everything worked, so we can just hand this back now.
+  *aResult = doc;
+  NS_IF_ADDREF(*aResult);
+
+  return NS_OK;
+
+}
+  
 NS_IMETHODIMP
 nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoundDocument,
                                    nsIURI* aURI, const nsCString& aRef, 
@@ -1208,15 +1321,16 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
     aBoundDocument->GetDocumentLoadGroup(getter_AddRefs(loadGroup));
   nsCOMPtr<nsIRequest> request;
   nsCOMPtr<nsIChannel> channel;
-  rv = NS_OpenURI(getter_AddRefs(channel), aURI, nsnull, loadGroup);
+  rv = NS_NewChannel(getter_AddRefs(channel), aURI, nsnull, loadGroup);
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsIAtom> tagName;
   if (aBoundElement)
     aBoundElement->GetTag(*getter_AddRefs(tagName)); 
 
-  if (tagName.get() == nsXULAtoms::scrollbar || tagName.get() == nsXULAtoms::thumb || 
-      IsResourceURI(aURI))
+  if (tagName == nsXULAtoms::scrollbar ||
+      tagName == nsXULAtoms::thumb || 
+      tagName == nsHTMLAtoms::select || IsResourceURI(aURI))
     aForceSyncLoad = PR_TRUE;
 
   nsCOMPtr<nsIStreamListener> listener;
@@ -1247,10 +1361,10 @@ nsXBLService::FetchBindingDocument(nsIContent* aBoundElement, nsIDocument* aBoun
     nsCOMPtr<nsIBindingManager> bindingManager;
     if (aBoundDocument)
       aBoundDocument->GetBindingManager(getter_AddRefs(bindingManager));
-    nsXPIDLCString uri;
-    aURI->GetSpec(getter_Copies(uri));
+    nsCAutoString uri;
+    aURI->GetSpec(uri);
     if (bindingManager)
-      bindingManager->PutLoadingDocListener(nsCAutoString(NS_STATIC_CAST(const char*, uri)), xblListener);
+      bindingManager->PutLoadingDocListener(uri, xblListener);
 
     // Add our request.
     nsCAutoString bindingURI(uri);
