@@ -53,41 +53,44 @@
 #include "nsIServiceManager.h"
 #include "nsXPIDLString.h"
 #include "nsIDocShell.h"
+#include "nsIDocShellTreeItem.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIRefreshURI.h"
 #include "nsStyleConsts.h"
 #include "nsIPresShell.h"
 #include "nsGUIEvent.h"
 #include "nsIPresContext.h"
-
+#include "nsIPref.h"
 #include "nsIDOMCSSStyleDeclaration.h"
 #include "nsIDOMViewCSS.h"
 #include "nsIXBLService.h"
 #include "nsIXBLBinding.h"
 #include "nsIBindingManager.h"
 
+static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 
 nsresult
 NS_NewXMLElement(nsIContent** aInstancePtrResult, nsINodeInfo *aNodeInfo)
 {
+  NS_ENSURE_ARG_POINTER(aInstancePtrResult);
+  *aInstancePtrResult = nsnull;
+
   nsXMLElement* it = new nsXMLElement();
 
   if (!it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
+  NS_ADDREF(it);
   nsresult rv = it->Init(aNodeInfo);
 
   if (NS_FAILED(rv)) {
-    delete it;
-
+    NS_RELEASE(it);
     return rv;
   }
 
   *aInstancePtrResult = NS_STATIC_CAST(nsIXMLContent *, it);
 
-  NS_ADDREF(*aInstancePtrResult);
-
   return NS_OK;
 }
 
@@ -154,7 +157,7 @@ nsXMLElement::QueryInterface(REFNSIID aIID, void** aInstancePtr)
     inst = nsContentUtils::GetClassInfoInstance(eDOMClassInfo_Element_id);
     NS_ENSURE_TRUE(inst, NS_ERROR_OUT_OF_MEMORY);
   } else {
-    return NS_NOINTERFACE;
+    return PostQueryInterface(aIID, aInstancePtr);
   }
 
   NS_ADDREF(inst);
@@ -169,7 +172,7 @@ NS_IMPL_ADDREF_INHERITED(nsXMLElement, nsGenericElement)
 NS_IMPL_RELEASE_INHERITED(nsXMLElement, nsGenericElement)
 
 
-static inline nsresult MakeURI(const char *aSpec, nsIURI *aBase, nsIURI **aURI)
+static inline nsresult MakeURI(const nsACString &aSpec, nsIURI *aBase, nsIURI **aURI)
 {
   nsresult rv;
   static NS_DEFINE_CID(ioServCID,NS_IOSERVICE_CID);
@@ -177,7 +180,7 @@ static inline nsresult MakeURI(const char *aSpec, nsIURI *aBase, nsIURI **aURI)
   if (NS_FAILED(rv))
     return rv;
 
-  return service->NewURI(aSpec,aBase,aURI);
+  return service->NewURI(aSpec,nsnull,aBase,aURI);
 }
 
 NS_IMETHODIMP
@@ -195,32 +198,30 @@ nsXMLElement::GetXMLBaseURI(nsIURI **aURI)
     rv = content->GetAttr(kNameSpaceID_XML,kBaseAtom,value);
     PRInt32 value_len;
     if (rv == NS_CONTENT_ATTR_HAS_VALUE) {
-      PRInt32 colon = value.FindChar(':',PR_FALSE);
-      PRInt32 slash = value.FindChar('/',PR_FALSE);
+      PRInt32 colon = value.FindChar(':');
+      PRInt32 slash = value.FindChar('/');
       if (colon > 0 && !( slash >= 0 && slash < colon)) {
         // Yay, we have absolute path!
         // The complex looking if above is to make sure that we do not erroneously
         // think a value of "./this:that" would have a scheme of "./that"
 
-        // XXX URL escape?
-        nsCAutoString str; str.AssignWithConversion(value);
+        NS_ConvertUCS2toUTF8 str(value);
       
-        rv = MakeURI(str.get(),nsnull,aURI);
+        rv = MakeURI(str,nsnull,aURI);
         if (NS_FAILED(rv))
           break;
 
-        if (!base.IsEmpty()) {
-          // XXX URL escape?
-          str.AssignWithConversion(base.get());
-          nsXPIDLCString resolvedStr;
-          rv = (*aURI)->Resolve(str.get(), getter_Copies(resolvedStr));
+        if (!base.IsEmpty()) { // XXXdarin base is always empty
+          str = NS_ConvertUCS2toUTF8(base);
+          nsCAutoString resolvedStr;
+          rv = (*aURI)->Resolve(str, resolvedStr);
           if (NS_FAILED(rv)) break;
           rv = (*aURI)->SetSpec(resolvedStr);
         }
         break;
 
       } else if ((value_len = value.Length()) > 0) {        
-        if (base.Length() > 0) {
+        if (!base.IsEmpty()) {
           if (base[0] == '/') {
             // Do nothing, we are waiting for a scheme starting value
           } else {
@@ -252,12 +253,52 @@ nsXMLElement::GetXMLBaseURI(nsIURI **aURI)
         *aURI = docBase.get();    
         NS_IF_ADDREF(*aURI);  // nsCOMPtr releases this once
       } else {
-        // XXX URL escape?
-        nsCAutoString str; str.AssignWithConversion(base);
-        rv = MakeURI(str.get(),docBase,aURI);
+        NS_ConvertUCS2toUTF8 str(base);
+        rv = MakeURI(str,docBase,aURI);
       }
     }
-  } else {
+
+    // Finally do a security check, almost the same as nsDocument::SetBaseURL()
+    if (*aURI) {
+      nsCOMPtr<nsIScriptSecurityManager> securityManager = 
+        do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
+      if (NS_SUCCEEDED(rv)) {
+        nsCOMPtr<nsIURI> docURI;
+        mDocument->GetDocumentURL(getter_AddRefs(docURI));
+        rv = securityManager->CheckLoadURI(docURI, *aURI, nsIScriptSecurityManager::STANDARD);
+        if (NS_FAILED(rv)) {
+          // Now we need to get the "closest" allowed base URI
+          NS_RELEASE(*aURI);
+
+          if (content) { // content is the last content we tried above
+            nsCOMPtr<nsIContent> parent;
+            content->GetParent(*getter_AddRefs(parent));
+            content = parent;
+            while (content) {
+              nsCOMPtr<nsIXMLContent> xml(do_QueryInterface(content));
+              if (xml) {
+                return xml->GetXMLBaseURI(aURI);
+              }
+              content->GetParent(*getter_AddRefs(parent));
+              content = parent;
+            }
+          }
+          
+          nsCOMPtr<nsIURI> docBase;
+          mDocument->GetBaseURL(*getter_AddRefs(docBase));
+          if (!docBase) {
+            mDocument->GetDocumentURL(getter_AddRefs(docBase));
+          }
+
+          *aURI = docBase.get();
+          NS_IF_ADDREF(*aURI);
+          rv = NS_OK;
+        }
+      }
+    }
+  }
+
+  if (NS_FAILED(rv)) {
     NS_IF_RELEASE(*aURI);
   }
 
@@ -266,7 +307,7 @@ nsXMLElement::GetXMLBaseURI(nsIURI **aURI)
 
 NS_IMETHODIMP
 nsXMLElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
-                      const nsAReadableString& aValue,
+                      const nsAString& aValue,
                       PRBool aNotify)
 {
   return nsGenericContainerElement::SetAttr(aNameSpaceID, aName, aValue,
@@ -275,7 +316,7 @@ nsXMLElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
 
 NS_IMETHODIMP 
 nsXMLElement::SetAttr(nsINodeInfo *aNodeInfo,
-                      const nsAReadableString& aValue,
+                      const nsAString& aValue,
                       PRBool aNotify)
 {
   NS_ENSURE_ARG_POINTER(aNodeInfo);
@@ -288,9 +329,6 @@ nsXMLElement::SetAttr(nsINodeInfo *aNodeInfo,
       //       we do not need to check other attributes. If there
       //       is no href attribute, then this link is simply
       //       untraversible [XLink 3.2].
-      // XXX If a parent of this element is already a simple link, then this
-      //     must not create a link of its own, this is just a normal element
-      //     inside the parent simple XLink element [XLink 3.2].
       mIsLink = PR_TRUE;
     } else {
       mIsLink = PR_FALSE;
@@ -316,16 +354,15 @@ static nsresult WebShellToPresContext(nsIWebShell *aShell,
 }
 
 
-static nsresult CheckLoadURI(nsIURI *aBaseURI, const nsAReadableString& aURI,
+static nsresult CheckLoadURI(nsIURI *aBaseURI, const nsAString& aURI,
                              nsIURI **aAbsURI)
 {
-  // XXX URL escape?
-  nsCAutoString str; str.Assign(NS_ConvertUCS2toUTF8(aURI));
+  NS_ConvertUCS2toUTF8 str(aURI);
 
   *aAbsURI = nsnull;
 
   nsresult rv;
-  rv = MakeURI(str.get(),aBaseURI,aAbsURI);
+  rv = MakeURI(str,aBaseURI,aAbsURI);
   if (NS_SUCCEEDED(rv)) {
     nsCOMPtr<nsIScriptSecurityManager> securityManager = 
              do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
@@ -364,16 +401,33 @@ static inline nsresult SpecialAutoLoadReturn(nsresult aRv, nsLinkVerb aVerb)
   return aRv;
 }
 
+inline PRBool BlockNewWindow(nsIPref *aPref)
+{
+  PRBool blockNewWindow = PR_FALSE;
+  nsCOMPtr<nsIPref> prefs;
+  if (aPref) {
+    prefs = dont_QueryInterface(aPref);
+  } else {
+    prefs = do_GetService(kPrefServiceCID);
+  }
+  if (prefs) {
+    prefs->GetBoolPref("browser.block.target_new_window", &blockNewWindow);
+  }
+  return blockNewWindow;
+}
+
 NS_IMETHODIMP
 nsXMLElement::MaybeTriggerAutoLink(nsIWebShell *aShell)
 {
-  NS_ABORT_IF_FALSE(aShell,"null ptr");
-  if (!aShell)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(aShell);
 
   nsresult rv = NS_OK;
 
   if (mIsLink) {
+    // preload and precalculate length of atom outside of loop
+    const PRUnichar *onloadUnicode;
+    kOnLoadAtom->GetUnicode(&onloadUnicode);
+    nsDependentString onloadString(onloadUnicode);
     do {
       // actuate="onLoad" ?
       nsAutoString value;
@@ -381,10 +435,26 @@ nsXMLElement::MaybeTriggerAutoLink(nsIWebShell *aShell)
                                               kActuateAtom,
                                               value);
       if (rv == NS_CONTENT_ATTR_HAS_VALUE &&
-          value.EqualsAtom(kOnLoadAtom,PR_FALSE)) {
+          value.Equals(onloadString)) {
+
+        // Disable in Mail/News for now. We may want a pref to control
+        // this at some point.
+        nsCOMPtr<nsIDocShellTreeItem> docShellItem(do_QueryInterface(aShell));
+        if (docShellItem) {
+          nsCOMPtr<nsIDocShellTreeItem> rootItem;
+          docShellItem->GetRootTreeItem(getter_AddRefs(rootItem));
+          nsCOMPtr<nsIDocShell> docshell(do_QueryInterface(rootItem));
+          if (docshell) {
+            PRUint32 appType;
+            if (NS_SUCCEEDED(docshell->GetAppType(&appType)) &&
+                appType == nsIDocShell::APP_TYPE_MAIL) {
+              return NS_OK;
+            }
+          }
+        }
 
         // show= ?
-        nsLinkVerb verb = eLinkVerb_Undefined;
+        nsLinkVerb verb = eLinkVerb_Undefined; // basically means same as replace
         rv = nsGenericContainerElement::GetAttr(kNameSpaceID_XLink,
                                                 kShowAtom, value);
         if (NS_FAILED(rv))
@@ -392,7 +462,16 @@ nsXMLElement::MaybeTriggerAutoLink(nsIWebShell *aShell)
 
         // XXX Should probably do this using atoms 
         if (value.Equals(NS_LITERAL_STRING("new"))) {
-          verb = eLinkVerb_New;
+          nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID));
+          PRBool disableNewDuringLoad = PR_FALSE;
+          if (prefs) {
+            prefs->GetBoolPref("dom.disable_open_during_load", &disableNewDuringLoad);
+            if (disableNewDuringLoad)
+              return NS_OK;
+          }
+          if (!BlockNewWindow(prefs)) {
+            verb = eLinkVerb_New;
+          }
         } else if (value.Equals(NS_LITERAL_STRING("replace"))) {
           // We want to actually stop processing the current document now.
           // We do this by returning the correct value so that the one
@@ -472,7 +551,7 @@ nsXMLElement::HandleDOMEvent(nsIPresContext* aPresContext,
           }
           nsAutoString show, href, target;
           nsIURI* baseURL = nsnull;
-          nsLinkVerb verb = eLinkVerb_Undefined;
+          nsLinkVerb verb = eLinkVerb_Undefined; // basically means same as replace
           nsGenericContainerElement::GetAttr(kNameSpaceID_XLink,
                                              kHrefAtom,
                                              href);
@@ -487,7 +566,9 @@ nsXMLElement::HandleDOMEvent(nsIPresContext* aPresContext,
 
           // XXX Should probably do this using atoms 
           if (show.Equals(NS_LITERAL_STRING("new"))) {
-            verb = eLinkVerb_New;
+            if (!BlockNewWindow(nsnull)) {
+              verb = eLinkVerb_New;
+            }
           } else if (show.Equals(NS_LITERAL_STRING("replace"))) {
             verb = eLinkVerb_Replace;
           } else if (show.Equals(NS_LITERAL_STRING("embed"))) {
@@ -581,6 +662,7 @@ NS_IMETHODIMP
 nsXMLElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
+  *aReturn = nsnull;
 
   nsXMLElement* it = new nsXMLElement();
 
@@ -588,17 +670,21 @@ nsXMLElement::CloneNode(PRBool aDeep, nsIDOMNode** aReturn)
     return NS_ERROR_OUT_OF_MEMORY;
   }
 
+  NS_ADDREF(it);
   nsCOMPtr<nsISupports> kungFuDeathGrip(NS_STATIC_CAST(nsIContent *, this));
 
   nsresult rv = it->Init(mNodeInfo);
 
   if (NS_FAILED(rv)) {
+    NS_RELEASE(it);
     return rv;
   }
 
   CopyInnerTo(this, it, aDeep);
 
-  return it->QueryInterface(NS_GET_IID(nsIDOMNode), (void**) aReturn);
+  rv = it->QueryInterface(NS_GET_IID(nsIDOMNode), (void**) aReturn);
+  NS_RELEASE(it);
+  return rv;
 }
 
 #if 0
