@@ -96,9 +96,10 @@
 #include "nsIMIMEService.h"
 #include "nsNetUtil.h"
 #include "nsMimeTypes.h"
-
+#include "nsIEventListenerManager.h"
 #include "nsContentUtils.h"
 #include "nsIElementFactory.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kHTMLStyleSheetCID,NS_HTMLSTYLESHEET_CID);
 
@@ -143,8 +144,8 @@ MyPrefChangedCallback(const char*aPrefName, void* instance_data)
 
 NS_EXPORT nsresult
 NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
-                  const nsAReadableString& aNamespaceURI, 
-                  const nsAReadableString& aQualifiedName, 
+                  const nsAString& aNamespaceURI, 
+                  const nsAString& aQualifiedName, 
                   nsIDOMDocumentType* aDoctype,
                   nsIURI* aBaseURI)
 {
@@ -153,13 +154,18 @@ NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
   *aInstancePtrResult = nsnull;
 
   nsXMLDocument* doc = new nsXMLDocument();
-  if (doc == nsnull)
+  if (!doc)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  nsCOMPtr<nsIDOMDocument> kungFuDeathGrip(doc);
+  rv = doc->Init();
 
-  rv = doc->Reset(nsnull, nsnull);
-  NS_ENSURE_SUCCESS(rv, rv);
+  if (NS_FAILED(rv)) {
+    delete doc;
+
+    return rv;
+  }
+
+  nsCOMPtr<nsIDOMDocument> kungFuDeathGrip(doc);
 
   doc->SetDocumentURL(aBaseURI);
   doc->SetBaseURL(aBaseURI);
@@ -170,7 +176,7 @@ NS_NewDOMDocument(nsIDOMDocument** aInstancePtrResult,
     NS_ENSURE_SUCCESS(rv, rv);
   }
   
-  if (aQualifiedName.Length() > 0) {
+  if (!aQualifiedName.IsEmpty()) {
     nsCOMPtr<nsIDOMElement> root;
     rv = doc->CreateElementNS(aNamespaceURI, aQualifiedName,
                               getter_AddRefs(root));
@@ -193,14 +199,25 @@ NS_EXPORT nsresult
 NS_NewXMLDocument(nsIDocument** aInstancePtrResult)
 {
   nsXMLDocument* doc = new nsXMLDocument();
-  if (doc == nsnull)
-    return NS_ERROR_OUT_OF_MEMORY;
-  return doc->QueryInterface(NS_GET_IID(nsIDocument), (void**) aInstancePtrResult);
+  NS_ENSURE_TRUE(doc, NS_ERROR_OUT_OF_MEMORY);
+
+  nsresult rv = doc->Init();
+
+  if (NS_FAILED(rv)) {
+    delete doc;
+
+    return rv;
+  }
+
+  *aInstancePtrResult = doc;
+  NS_ADDREF(*aInstancePtrResult);
+
+  return NS_OK;
 }
 
 nsXMLDocument::nsXMLDocument() 
   : mAttrStyleSheet(nsnull), mInlineStyleSheet(nsnull), 
-    mParser(nsnull)
+    mCountCatalogSheets(0), mParser(nsnull)
 {
 }
 
@@ -310,20 +327,20 @@ nsXMLDocument::OnRedirect(nsIHttpChannel *aHttpChannel, nsIChannel *aNewChannel)
 }
 
 NS_IMETHODIMP
-nsXMLDocument::EvaluateFIXptr(const nsAReadableString& aExpression, nsIDOMRange **aRange)
+nsXMLDocument::EvaluateFIXptr(const nsAString& aExpression, nsIDOMRange **aRange)
 {
   return nsFIXptr::Evaluate(this, aExpression, aRange);
 }
 
 NS_IMETHODIMP
-nsXMLDocument::Load(const nsAReadableString& aUrl)
+nsXMLDocument::Load(const nsAString& aUrl)
 {
   nsCOMPtr<nsIChannel> channel;
   nsCOMPtr<nsIURI> uri;
   nsresult rv;
   
   // Create a new URI
-  rv = NS_NewURI(getter_AddRefs(uri), aUrl, mDocumentURL);
+  rv = NS_NewURI(getter_AddRefs(uri), aUrl, nsnull, mDocumentURL);
   if (NS_FAILED(rv)) return rv;
 
   // Get security manager, check to see if we're allowed to load this URI
@@ -333,13 +350,23 @@ nsXMLDocument::Load(const nsAReadableString& aUrl)
   if (NS_FAILED(secMan->CheckConnect(nsnull, uri, "XMLDocument", "load")))
     return NS_ERROR_FAILURE;
 
-  // Partial Reset
+  // Partial Reset, need to restore principal for security reasons and
+  // event listener manager so that load listeners etc. will remain.
+  nsCOMPtr<nsIPrincipal> principal(dont_QueryInterface(mPrincipal));
+  nsCOMPtr<nsIEventListenerManager> elm(dont_QueryInterface(mListenerManager));
+
+  Reset(nsnull, nsnull);
+  
+  mPrincipal = principal;
+  mListenerManager = elm;
+  NS_IF_ADDREF(mPrincipal);
+  NS_IF_ADDREF(mListenerManager);
+  
   SetDocumentURL(uri);
   SetBaseURL(uri);
-  mBaseTarget.Truncate();
 
   // Create a channel
-  rv = NS_OpenURI(getter_AddRefs(channel), uri, nsnull, nsnull, this);
+  rv = NS_NewChannel(getter_AddRefs(channel), uri, nsnull, nsnull, this);
   if (NS_FAILED(rv)) return rv;
 
   // Set a principal for this document
@@ -407,23 +434,22 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
     aContentType = nsnull;
   }
 
-  nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
-  if(httpChannel) {
-    nsXPIDLCString charsetheader;
-    rv = httpChannel->GetCharset(getter_Copies(charsetheader));
+  { // check channel's charset...
+    nsCAutoString charsetVal;
+    rv = aChannel->GetContentCharset(charsetVal);
     if (NS_SUCCEEDED(rv)) {
       nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
 
       if(NS_SUCCEEDED(rv) && (nsnull != calias) ) {
         nsAutoString preferred;
-        rv = calias->GetPreferred(NS_ConvertASCIItoUCS2(charsetheader), preferred);
+        rv = calias->GetPreferred(NS_ConvertASCIItoUCS2(charsetVal), preferred);
         if(NS_SUCCEEDED(rv)){            
           charset = preferred;
-          charsetSource = kCharsetFromHTTPHeader;
+          charsetSource = kCharsetFromChannel;
         }
       }
     }
-  } //end of checking http channel
+  } //end of checking channel's charset
 
   static NS_DEFINE_CID(kCParserCID, NS_PARSER_CID);
 
@@ -597,13 +623,13 @@ nsXMLDocument::StartDocumentLoad(const char* aCommand,
     if (aSink)
       sink = do_QueryInterface(aSink);
     else
-      rv = NS_NewXMLContentSink(getter_AddRefs(sink), this, aUrl, webShell);
+      rv = NS_NewXMLContentSink(getter_AddRefs(sink), this, aUrl, webShell, aChannel);
   }
   else {
     if (aSink)
       sink = do_QueryInterface(aSink);
     else
-      rv = NS_NewXMLContentSink(getter_AddRefs(sink), this, aUrl, nsnull);
+      rv = NS_NewXMLContentSink(getter_AddRefs(sink), this, aUrl, nsnull, aChannel);
   }
 
   if (NS_OK == rv) {      
@@ -674,10 +700,16 @@ nsXMLDocument::GetInlineStyleSheet(nsIHTMLCSSStyleSheet** aResult)
   return NS_OK;
 }
 
-void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet)  // subclass hook for sheet ordering
+// subclass hook for sheet ordering
+void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet, PRUint32 aFlags)
 {
-  if (aSheet == mAttrStyleSheet) {  // always first
-    mStyleSheets.InsertElementAt(aSheet, 0);
+  if (aFlags & NS_STYLESHEET_FROM_CATALOG) {
+    // always after other catalog sheets
+    mStyleSheets.InsertElementAt(aSheet, mCountCatalogSheets);
+    ++mCountCatalogSheets;
+  }
+  else if (aSheet == mAttrStyleSheet) {  // always after catalog sheets
+    mStyleSheets.InsertElementAt(aSheet, mCountCatalogSheets);
   }
   else if (aSheet == mInlineStyleSheet) {  // always last
     mStyleSheets.AppendElement(aSheet);
@@ -697,7 +729,8 @@ void nsXMLDocument::InternalAddStyleSheet(nsIStyleSheet* aSheet)  // subclass ho
 void
 nsXMLDocument::InternalInsertStyleSheetAt(nsIStyleSheet* aSheet, PRInt32 aIndex)
 {
-  mStyleSheets.InsertElementAt(aSheet, aIndex + 1); // offset one for the attr style sheet
+  // offset w.r.t. catalog style sheets and the attr style sheet
+  mStyleSheets.InsertElementAt(aSheet, aIndex + mCountCatalogSheets + 1);
 }
 
 // nsIDOMDocument interface
@@ -708,7 +741,7 @@ nsXMLDocument::GetDoctype(nsIDOMDocumentType** aDocumentType)
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateCDATASection(const nsAReadableString& aData, nsIDOMCDATASection** aReturn)
+nsXMLDocument::CreateCDATASection(const nsAString& aData, nsIDOMCDATASection** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
   *aReturn = nsnull;
@@ -733,7 +766,7 @@ nsXMLDocument::CreateCDATASection(const nsAReadableString& aData, nsIDOMCDATASec
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateEntityReference(const nsAReadableString& aName, nsIDOMEntityReference** aReturn)
+nsXMLDocument::CreateEntityReference(const nsAString& aName, nsIDOMEntityReference** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
 
@@ -742,8 +775,8 @@ nsXMLDocument::CreateEntityReference(const nsAReadableString& aName, nsIDOMEntit
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateProcessingInstruction(const nsAReadableString& aTarget, 
-                                           const nsAReadableString& aData, 
+nsXMLDocument::CreateProcessingInstruction(const nsAString& aTarget, 
+                                           const nsAString& aData, 
                                            nsIDOMProcessingInstruction** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -763,12 +796,12 @@ nsXMLDocument::CreateProcessingInstruction(const nsAReadableString& aTarget,
 }
  
 NS_IMETHODIMP    
-nsXMLDocument::CreateElement(const nsAReadableString& aTagName, 
+nsXMLDocument::CreateElement(const nsAString& aTagName, 
                               nsIDOMElement** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
   *aReturn = nsnull;
-  NS_ENSURE_TRUE(aTagName.Length(), NS_ERROR_DOM_INVALID_CHARACTER_ERR);
+  NS_ENSURE_TRUE(!aTagName.IsEmpty(), NS_ERROR_DOM_INVALID_CHARACTER_ERR);
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
   nsresult rv;
@@ -858,8 +891,8 @@ nsXMLDocument::ImportNode(nsIDOMNode* aImportedNode,
 }
 
 NS_IMETHODIMP
-nsXMLDocument::CreateAttributeNS(const nsAReadableString& aNamespaceURI,
-                                 const nsAReadableString& aQualifiedName,
+nsXMLDocument::CreateAttributeNS(const nsAString& aNamespaceURI,
+                                 const nsAString& aQualifiedName,
                                  nsIDOMAttr** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -880,8 +913,8 @@ nsXMLDocument::CreateAttributeNS(const nsAReadableString& aNamespaceURI,
 }
 
 NS_IMETHODIMP
-nsXMLDocument::CreateElementNS(const nsAReadableString& aNamespaceURI,
-                               const nsAReadableString& aQualifiedName,
+nsXMLDocument::CreateElementNS(const nsAString& aNamespaceURI,
+                               const nsAString& aQualifiedName,
                                nsIDOMElement** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -898,14 +931,14 @@ nsXMLDocument::CreateElementNS(const nsAReadableString& aNamespaceURI,
 }
 
 static nsIContent *
-MatchId(nsIContent *aContent, const nsAReadableString& aName)
+MatchId(nsIContent *aContent, const nsAString& aName)
 {
   nsAutoString value;
   nsIContent *result = nsnull;
   PRInt32 ns;
 
   aContent->GetNameSpaceID(ns);
-  if (kNameSpaceID_HTML == ns) {
+  if (kNameSpaceID_XHTML == ns) {
     if ((NS_CONTENT_ATTR_HAS_VALUE == aContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::id, value)) &&
         aName.Equals(value)) {
       return aContent;
@@ -936,7 +969,7 @@ MatchId(nsIContent *aContent, const nsAReadableString& aName)
 }
 
 NS_IMETHODIMP
-nsXMLDocument::GetElementById(const nsAReadableString& aElementId,
+nsXMLDocument::GetElementById(const nsAString& aElementId,
                              nsIDOMElement** aReturn)
 {
   NS_ENSURE_ARG_POINTER(aReturn);
@@ -979,11 +1012,11 @@ nsXMLDocument::SetDefaultStylesheets(nsIURI* aUrl)
       }
     }
     if (NS_OK == result) {
-      AddStyleSheet(mAttrStyleSheet); // tell the world about our new style sheet
+      AddStyleSheet(mAttrStyleSheet, 0); // tell the world about our new style sheet
       
       result = NS_NewHTMLCSSStyleSheet(&mInlineStyleSheet, aUrl, this);
       if (NS_OK == result) {
-        AddStyleSheet(mInlineStyleSheet); // tell the world about our new style sheet
+        AddStyleSheet(mInlineStyleSheet, 0); // tell the world about our new style sheet
       }
     }
   }
@@ -992,14 +1025,14 @@ nsXMLDocument::SetDefaultStylesheets(nsIURI* aUrl)
 }
 
 NS_IMETHODIMP 
-nsXMLDocument::SetBaseTarget(const nsAReadableString &aBaseTarget)
+nsXMLDocument::SetBaseTarget(const nsAString &aBaseTarget)
 {
   mBaseTarget.Assign(aBaseTarget);
   return NS_OK;
 }
 
 NS_IMETHODIMP 
-nsXMLDocument::GetBaseTarget(nsAWritableString &aBaseTarget)
+nsXMLDocument::GetBaseTarget(nsAString &aBaseTarget)
 {
   aBaseTarget.Assign(mBaseTarget);
   return NS_OK;
@@ -1034,7 +1067,6 @@ nsXMLDocument::CreateElement(nsINodeInfo *aNodeInfo, nsIDOMElement** aResult)
   aNodeInfo->GetNamespaceID(namespaceID);
 
   nsCOMPtr<nsIElementFactory> elementFactory;
-
   mNameSpaceManager->GetElementFactory(namespaceID,
                                        getter_AddRefs(elementFactory));
 
