@@ -56,6 +56,7 @@
 
 #include "nsCOMPtr.h"
 #include "nsDOMCID.h"
+#include "nsFixedSizeAllocator.h"
 #include "nsIContent.h"
 #include "nsINodeInfo.h"
 #include "nsICSSParser.h"
@@ -68,6 +69,7 @@
 #include "nsReadableUtils.h"
 #include "nsContentUtils.h"
 #include "nsXULAtoms.h"
+#include "nsCRT.h"
 
 static NS_DEFINE_CID(kCSSParserCID, NS_CSSPARSER_CID);
 
@@ -106,7 +108,7 @@ nsClassList::GetClasses(nsClassList* aList, nsVoidArray& aArray)
 
 
 nsresult
-nsClassList::ParseClasses(nsClassList** aList, const nsAReadableString& aClassString)
+nsClassList::ParseClasses(nsClassList** aList, const nsAString& aClassString)
 {
     static const PRUnichar kNullCh = PRUnichar('\0');
 
@@ -115,7 +117,7 @@ nsClassList::ParseClasses(nsClassList** aList, const nsAReadableString& aClassSt
         *aList = nsnull;
     }
 
-    if (aClassString.Length() > 0) {
+    if (!aClassString.IsEmpty()) {
         nsAutoString classStr(aClassString);  // copy to work buffer
         classStr.Append(kNullCh);  // put an extra null at the end
 
@@ -153,7 +155,7 @@ nsClassList::ParseClasses(nsClassList** aList, const nsAReadableString& aClassSt
 
 nsXULAttribute::nsXULAttribute(nsIContent* aContent,
                                nsINodeInfo* aNodeInfo,
-                               const nsAReadableString& aValue)
+                               const nsAString& aValue)
     : mContent(aContent),
       mNodeInfo(aNodeInfo)
 {
@@ -171,7 +173,7 @@ nsXULAttribute::~nsXULAttribute()
 nsresult
 nsXULAttribute::Create(nsIContent* aContent,
                        nsINodeInfo* aNodeInfo,
-                       const nsAReadableString& aValue,
+                       const nsAString& aValue,
                        nsXULAttribute** aResult)
 {
     NS_ENSURE_ARG_POINTER(aNodeInfo);
@@ -205,20 +207,20 @@ NS_IMPL_RELEASE(nsXULAttribute);
 // nsIDOMNode interface
 
 NS_IMETHODIMP
-nsXULAttribute::GetNodeName(nsAWritableString& aNodeName)
+nsXULAttribute::GetNodeName(nsAString& aNodeName)
 {
     GetQualifiedName(aNodeName);
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsXULAttribute::GetNodeValue(nsAWritableString& aNodeValue)
+nsXULAttribute::GetNodeValue(nsAString& aNodeValue)
 {
     return mValue.GetValue(aNodeValue);
 }
 
 NS_IMETHODIMP
-nsXULAttribute::SetNodeValue(const nsAReadableString& aNodeValue)
+nsXULAttribute::SetNodeValue(const nsAString& aNodeValue)
 {
     return SetValue(aNodeValue);
 }
@@ -287,26 +289,26 @@ nsXULAttribute::GetOwnerDocument(nsIDOMDocument** aOwnerDocument)
 }
 
 NS_IMETHODIMP
-nsXULAttribute::GetNamespaceURI(nsAWritableString& aNamespaceURI)
+nsXULAttribute::GetNamespaceURI(nsAString& aNamespaceURI)
 {
   return mNodeInfo->GetNamespaceURI(aNamespaceURI);
 }
 
 NS_IMETHODIMP
-nsXULAttribute::GetPrefix(nsAWritableString& aPrefix)
+nsXULAttribute::GetPrefix(nsAString& aPrefix)
 {
   return mNodeInfo->GetPrefix(aPrefix);
 }
 
 NS_IMETHODIMP
-nsXULAttribute::SetPrefix(const nsAReadableString& aPrefix)
+nsXULAttribute::SetPrefix(const nsAString& aPrefix)
 {
     // XXX: Validate the prefix string!
 
     nsINodeInfo *newNodeInfo = nsnull;
     nsCOMPtr<nsIAtom> prefix;
 
-    if (aPrefix.Length()) {
+    if (!aPrefix.IsEmpty()) {
         prefix = dont_AddRef(NS_NewAtom(aPrefix));
         NS_ENSURE_TRUE(prefix, NS_ERROR_OUT_OF_MEMORY);
     }
@@ -321,7 +323,7 @@ nsXULAttribute::SetPrefix(const nsAReadableString& aPrefix)
 }
 
 NS_IMETHODIMP
-nsXULAttribute::GetLocalName(nsAWritableString& aLocalName)
+nsXULAttribute::GetLocalName(nsAString& aLocalName)
 {
   return mNodeInfo->GetLocalName(aLocalName);
 }
@@ -380,32 +382,51 @@ nsXULAttribute::Normalize()
 }
 
 NS_IMETHODIMP
-nsXULAttribute::IsSupported(const nsAReadableString& aFeature,
-                            const nsAReadableString& aVersion,
+nsXULAttribute::IsSupported(const nsAString& aFeature,
+                            const nsAString& aVersion,
                             PRBool* aReturn)
 {
   NS_NOTYETIMPLEMENTED("write me");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
+// nsIDOM3Node interface
+
 NS_IMETHODIMP
-nsXULAttribute::GetBaseURI(nsAWritableString &aURI)
+nsXULAttribute::GetBaseURI(nsAString &aURI)
 {
   NS_NOTYETIMPLEMENTED("write me");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsXULAttribute::LookupNamespacePrefix(const nsAReadableString& aNamespaceURI,
-                                      nsAWritableString& aPrefix)
+nsXULAttribute::CompareTreePosition(nsIDOMNode* aOther,
+                                    PRUint16* aReturn)
 {
   NS_NOTYETIMPLEMENTED("write me");
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsXULAttribute::LookupNamespaceURI(const nsAReadableString& aNamespacePrefix,
-                                   nsAWritableString& aNamespaceURI) 
+nsXULAttribute::IsSameNode(nsIDOMNode* aOther,
+                           PRBool* aReturn)
+{
+  NS_NOTYETIMPLEMENTED("write me");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+
+NS_IMETHODIMP
+nsXULAttribute::LookupNamespacePrefix(const nsAString& aNamespaceURI,
+                                      nsAString& aPrefix)
+{
+  NS_NOTYETIMPLEMENTED("write me");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsXULAttribute::LookupNamespaceURI(const nsAString& aNamespacePrefix,
+                                   nsAString& aNamespaceURI) 
 {
   NS_NOTYETIMPLEMENTED("write me");
   return NS_ERROR_NOT_IMPLEMENTED;
@@ -415,7 +436,7 @@ nsXULAttribute::LookupNamespaceURI(const nsAReadableString& aNamespacePrefix,
 // nsIDOMAttr interface
 
 NS_IMETHODIMP
-nsXULAttribute::GetName(nsAWritableString& aName)
+nsXULAttribute::GetName(nsAString& aName)
 {
     GetQualifiedName(aName);
     return NS_OK;
@@ -430,13 +451,13 @@ nsXULAttribute::GetSpecified(PRBool* aSpecified)
 }
 
 NS_IMETHODIMP
-nsXULAttribute::GetValue(nsAWritableString& aValue)
+nsXULAttribute::GetValue(nsAString& aValue)
 {
     return mValue.GetValue(aValue);
 }
 
 NS_IMETHODIMP
-nsXULAttribute::SetValue(const nsAReadableString& aValue)
+nsXULAttribute::SetValue(const nsAString& aValue)
 {
     // We call back to the content node's SetValue() method so we can
     // share all of the work that it does.
@@ -456,7 +477,7 @@ nsXULAttribute::GetOwnerElement(nsIDOMElement** aOwnerElement)
 // Implementation methods
 
 void
-nsXULAttribute::GetQualifiedName(nsAWritableString& aQualifiedName)
+nsXULAttribute::GetQualifiedName(nsAString& aQualifiedName)
 {
     mNodeInfo->GetQualifiedName(aQualifiedName);
 }
@@ -464,7 +485,7 @@ nsXULAttribute::GetQualifiedName(nsAWritableString& aQualifiedName)
 
 
 nsresult
-nsXULAttribute::SetValueInternal(const nsAReadableString& aValue)
+nsXULAttribute::SetValueInternal(const nsAString& aValue)
 {
     return mValue.SetValue( aValue, mNodeInfo->Equals(nsXULAtoms::id) );
 }
@@ -500,6 +521,36 @@ nsXULAttributes::~nsXULAttributes()
     delete mClassList;
 }
 
+static nsFixedSizeAllocator *gAttrAllocator;
+static PRUint32 gRefCnt;
+
+static PRBool
+CreateAttrAllocator()
+{
+    gAttrAllocator = new nsFixedSizeAllocator();
+    if (!gAttrAllocator)
+        return PR_FALSE;
+    
+    const size_t bucketSize = sizeof(nsXULAttributes);
+    const PRInt32 initalElements = 128; // XXX tune me
+    if (NS_FAILED(gAttrAllocator->Init("XUL Attributes", &bucketSize,
+                                       1, bucketSize * initalElements,
+                                       8))) {
+        delete gAttrAllocator;
+        gAttrAllocator = nsnull;
+        return PR_FALSE;
+    }
+
+    return PR_TRUE;
+}
+
+static void
+DestroyAttrAllocator()
+{
+    NS_ASSERTION(!gRefCnt, "Premature call to DestroyAttrAllocator!");
+    delete gAttrAllocator;
+    gAttrAllocator = nsnull;
+}
 
 nsresult
 nsXULAttributes::Create(nsIContent* aContent, nsXULAttributes** aResult)
@@ -508,13 +559,41 @@ nsXULAttributes::Create(nsIContent* aContent, nsXULAttributes** aResult)
     if (! aResult)
         return NS_ERROR_NULL_POINTER;
 
-    if (! (*aResult = new nsXULAttributes(aContent)))
+    if (!gRefCnt && !CreateAttrAllocator())
         return NS_ERROR_OUT_OF_MEMORY;
 
+    // Increment here, because this controls the gAttrAllocator's destruction.
+    // Failure cases below must decrement gRefCnt, and then call
+    // DestroyAttrAllocator if gRefCnt == 0, to avoid leaking a ref that will
+    // not be balanced by a matching Destroy call. (|goto error;| works.)
+    gRefCnt++;
+    void *place = gAttrAllocator->Alloc(sizeof (nsXULAttributes));
+    if (!place)
+        goto error;
+
+    *aResult = ::new (place) nsXULAttributes(aContent);
+    if (!*aResult)
+        goto error;
+
     NS_ADDREF(*aResult);
     return NS_OK;
+
+error:
+    if (!--gRefCnt)
+        DestroyAttrAllocator();
+    return NS_ERROR_OUT_OF_MEMORY;
 }
 
+/* static */ void
+nsXULAttributes::Destroy(nsXULAttributes *aAttrs)
+{
+    aAttrs->~nsXULAttributes();
+    NS_ASSERTION(gAttrAllocator,
+                 "Allocator destroyed with live nsXULAttributes remaining!");
+    gAttrAllocator->Free(aAttrs, sizeof *aAttrs);
+    if (!--gRefCnt)
+        DestroyAttrAllocator();
+}
 
 // nsISupports interface
 
@@ -525,10 +604,21 @@ NS_INTERFACE_MAP_BEGIN(nsXULAttributes)
     NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(XULNamedNodeMap)
 NS_INTERFACE_MAP_END
 
-
 NS_IMPL_ADDREF(nsXULAttributes);
-NS_IMPL_RELEASE(nsXULAttributes);
 
+// Custom release method to go through the fixed-size allocator
+nsrefcnt
+nsXULAttributes::Release()
+{
+    mRefCnt--;
+    NS_LOG_RELEASE(this, mRefCnt, "nsXULAttributes");
+    if (mRefCnt == 0) {
+        mRefCnt = 1;            // stabilize (necessary for this class?)
+        Destroy(this);
+        return 0;
+    }
+    return mRefCnt;
+}
 
 // nsIDOMNamedNodeMap interface
 
@@ -544,7 +634,7 @@ nsXULAttributes::GetLength(PRUint32* aLength)
 }
 
 NS_IMETHODIMP
-nsXULAttributes::GetNamedItem(const nsAReadableString& aName,
+nsXULAttributes::GetNamedItem(const nsAString& aName,
                               nsIDOMNode** aReturn)
 {
     NS_PRECONDITION(aReturn != nsnull, "null ptr");
@@ -591,7 +681,7 @@ nsXULAttributes::SetNamedItem(nsIDOMNode* aArg, nsIDOMNode** aReturn)
 }
 
 NS_IMETHODIMP
-nsXULAttributes::RemoveNamedItem(const nsAReadableString& aName,
+nsXULAttributes::RemoveNamedItem(const nsAString& aName,
                                  nsIDOMNode** aReturn)
 {
     nsCOMPtr<nsIDOMElement> element( do_QueryInterface(mContent) );
@@ -614,8 +704,8 @@ nsXULAttributes::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 }
 
 nsresult
-nsXULAttributes::GetNamedItemNS(const nsAReadableString& aNamespaceURI, 
-                                const nsAReadableString& aLocalName,
+nsXULAttributes::GetNamedItemNS(const nsAString& aNamespaceURI, 
+                                const nsAString& aLocalName,
                                 nsIDOMNode** aReturn)
 {
   NS_NOTYETIMPLEMENTED("write me");
@@ -630,8 +720,8 @@ nsXULAttributes::SetNamedItemNS(nsIDOMNode* aArg, nsIDOMNode** aReturn)
 }
 
 nsresult
-nsXULAttributes::RemoveNamedItemNS(const nsAReadableString& aNamespaceURI, 
-                                   const nsAReadableString& aLocalName,
+nsXULAttributes::RemoveNamedItemNS(const nsAString& aNamespaceURI, 
+                                   const nsAString& aLocalName,
                                    nsIDOMNode** aReturn)
 {
   NS_NOTYETIMPLEMENTED("write me");
@@ -665,12 +755,12 @@ nsresult nsXULAttributes::SetClassList(nsClassList* aClassList)
     return NS_OK;
 }
 
-nsresult nsXULAttributes::UpdateClassList(const nsAReadableString& aValue)
+nsresult nsXULAttributes::UpdateClassList(const nsAString& aValue)
 {
     return nsClassList::ParseClasses(&mClassList, aValue);
 }
 
-nsresult nsXULAttributes::UpdateStyleRule(nsIURI* aDocURL, const nsAReadableString& aValue)
+nsresult nsXULAttributes::UpdateStyleRule(nsIURI* aDocURL, const nsAString& aValue)
 {
     if (aValue.IsEmpty())
     {
