@@ -23,6 +23,7 @@
  *   Chris Waterson <waterson@netscape.com>
  *   Peter Annema <disttsc@bart.nl>
  *   Mike Shaver <shaver@mozilla.org>
+ *   Ben Goodger <ben@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -54,6 +55,7 @@
 #include "nsIAtom.h"
 #include "nsINodeInfo.h"
 #include "nsIControllers.h"
+#include "nsICSSParser.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOMXULElement.h"
@@ -69,12 +71,14 @@
 #include "nsIXBLBinding.h"
 #include "nsIURI.h"
 #include "nsIXULContent.h"
+#include "nsIXULPrototypeCache.h"
 #include "nsIXULTemplateBuilder.h"
 #include "nsIBoxObject.h"
 #include "nsXULAttributes.h"
 #include "nsIChromeEventHandler.h"
 #include "nsXULAttributeValue.h"
 #include "nsIXBLService.h"
+#include "nsLayoutCID.h"
 
 #include "nsGenericElement.h" // for nsCheapVoidArray
 
@@ -94,6 +98,8 @@ class nsIWebShell;
 class nsIObjectInputStream;
 class nsIObjectOutputStream;
 
+static NS_DEFINE_CID(kCSSParserCID, NS_CSSPARSER_CID);
+
 ////////////////////////////////////////////////////////////////////////
 
 #ifdef XUL_PROTOTYPE_ATTRIBUTE_METERING
@@ -195,9 +201,10 @@ public:
           
     virtual ~nsXULPrototypeNode() {}
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
-                               nsIScriptContext* aContext);
+                               nsIScriptContext* aContext) = 0;
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext);
+                                 nsIScriptContext* aContext,
+                                 nsIURI* aDocumentURI) = 0;
 
     void AddRef() { mRefCnt++; };
     void Release() 
@@ -239,8 +246,11 @@ public:
     virtual void ReleaseSubtree() 
     {
       if (mChildren) {
-        for (PRInt32 i = mNumChildren-1; i >= 0; i--)
+        for (PRInt32 i = mNumChildren-1; i >= 0; i--) {
+          if (! mChildren[i])
+            break;
           mChildren[i]->ReleaseSubtree();
+        }
       }
 
       nsXULPrototypeNode::ReleaseSubtree();
@@ -249,7 +259,8 @@ public:
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext);
+                                 nsIScriptContext* aContext,
+                                 nsIURI* aDocumentURI);
 
     PRInt32                  mNumChildren;
     nsXULPrototypeNode**     mChildren;           // [OWNER]
@@ -262,7 +273,37 @@ public:
     nsCOMPtr<nsIStyleRule>   mInlineStyleRule;    // [OWNER]
     nsClassList*             mClassList;
 
-    nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAWritableString& aValue);
+    nsresult GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aValue);
+
+
+    static void ReleaseGlobals() 
+    { 
+        NS_IF_RELEASE(sNodeInfoManager); 
+        NS_IF_RELEASE(sCSSParser);
+    }
+
+protected:
+    static nsINodeInfoManager* GetNodeInfoManager()
+    {
+        if (!sNodeInfoManager) {
+            CallCreateInstance(NS_NODEINFOMANAGER_CONTRACTID, &sNodeInfoManager);
+
+            nsCOMPtr<nsINameSpaceManager> nsmgr;
+            NS_NewNameSpaceManager(getter_AddRefs(nsmgr));
+
+            sNodeInfoManager->Init(nsnull, nsmgr);
+        }
+        return sNodeInfoManager;
+    }
+    static nsINodeInfoManager* sNodeInfoManager;
+
+    static nsICSSParser* GetCSSParser()
+    {
+        if (!sCSSParser)
+            CallCreateInstance(kCSSParserCID, &sCSSParser);
+        return sCSSParser;
+    }
+    static nsICSSParser* sCSSParser;
 };
 
 struct JSRuntime;
@@ -278,7 +319,10 @@ public:
     virtual nsresult Serialize(nsIObjectOutputStream* aStream,
                                nsIScriptContext* aContext);
     virtual nsresult Deserialize(nsIObjectInputStream* aStream,
-                                 nsIScriptContext* aContext);
+                                 nsIScriptContext* aContext,
+                                 nsIURI* aDocumentURI);
+    virtual nsresult DeserializeOutOfLineScript(nsIObjectInputStream* aInput,
+                                                nsIScriptContext* aContext);
 
     nsresult Compile(const PRUnichar* aText, PRInt32 aTextLength,
                      nsIURI* aURI, PRInt32 aLineNo,
@@ -287,9 +331,25 @@ public:
 
     nsCOMPtr<nsIURI>         mSrcURI;
     PRBool                   mSrcLoading;
+    PRBool                   mOutOfLine;
     nsXULDocument*           mSrcLoadWaiters;   // [OWNER] but not COMPtr
     JSObject*                mJSObject;
     const char*              mLangVersion;
+
+    static void ReleaseGlobals() 
+    { 
+        NS_IF_RELEASE(sXULPrototypeCache); 
+    }
+
+protected:
+    static nsIXULPrototypeCache* GetXULCache()
+    {
+        if (!sXULPrototypeCache)
+            CallGetService("@mozilla.org/xul/xul-prototype-cache;1", &sXULPrototypeCache);
+
+        return sXULPrototypeCache;
+    }
+    static nsIXULPrototypeCache* sXULPrototypeCache;
 };
 
 class nsXULPrototypeText : public nsXULPrototypeNode
@@ -306,6 +366,12 @@ public:
         MOZ_COUNT_DTOR(nsXULPrototypeText);
     }
 
+    virtual nsresult Serialize(nsIObjectOutputStream* aStream,
+                               nsIScriptContext* aContext);
+    virtual nsresult Deserialize(nsIObjectInputStream* aStream,
+                                 nsIScriptContext* aContext,
+                                 nsIURI* aDocumentURI);
+
     nsString                 mValue;
 };
 
@@ -369,11 +435,11 @@ public:
     NS_IMETHOD GetNameSpaceID(PRInt32& aNameSpeceID) const;
     NS_IMETHOD GetTag(nsIAtom*& aResult) const;
     NS_IMETHOD GetNodeInfo(nsINodeInfo*& aResult) const;
-    NS_IMETHOD NormalizeAttrString(const nsAReadableString& aStr, nsINodeInfo*& aNodeInfo);
-    NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, const nsAReadableString& aValue, PRBool aNotify);
-    NS_IMETHOD SetAttr(nsINodeInfo *aNodeInfo, const nsAReadableString& aValue, PRBool aNotify);
-    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAWritableString& aResult) const;
-    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom*& aPrefix, nsAWritableString& aResult) const;
+    NS_IMETHOD NormalizeAttrString(const nsAString& aStr, nsINodeInfo*& aNodeInfo);
+    NS_IMETHOD SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, const nsAString& aValue, PRBool aNotify);
+    NS_IMETHOD SetAttr(nsINodeInfo *aNodeInfo, const nsAString& aValue, PRBool aNotify);
+    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsAString& aResult) const;
+    NS_IMETHOD GetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, nsIAtom*& aPrefix, nsAString& aResult) const;
     NS_IMETHOD_(PRBool) HasAttr(PRInt32 aNameSpaceID, nsIAtom* aName) const;
     NS_IMETHOD UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aName, PRBool aNotify);
     NS_IMETHOD GetAttrNameAt(PRInt32 aIndex, PRInt32& aNameSpaceID, 
@@ -389,6 +455,7 @@ public:
                               nsIDOMEvent** aDOMEvent,
                               PRUint32 aFlags,
                               nsEventStatus* aEventStatus);
+    NS_IMETHOD DoneCreatingElement();
 
     NS_IMETHOD GetContentID(PRUint32* aID);
     NS_IMETHOD SetContentID(PRUint32 aID);
@@ -420,10 +487,10 @@ public:
 
     // nsIXULContent
     NS_IMETHOD PeekChildCount(PRInt32& aCount) const;
-    NS_IMETHOD SetLazyState(PRInt32 aFlags);
-    NS_IMETHOD ClearLazyState(PRInt32 aFlags);
-    NS_IMETHOD GetLazyState(PRInt32 aFlag, PRBool& aValue);
-    NS_IMETHOD AddScriptEventListener(nsIAtom* aName, const nsAReadableString& aValue);
+    NS_IMETHOD SetLazyState(LazyState aFlags);
+    NS_IMETHOD ClearLazyState(LazyState aFlags);
+    NS_IMETHOD GetLazyState(LazyState aFlag, PRBool& aValue);
+    NS_IMETHOD AddScriptEventListener(nsIAtom* aName, const nsAString& aValue);
     
     // nsIDOMNode (from nsIDOMElement)
     NS_DECL_NSIDOMNODE
@@ -435,9 +502,9 @@ public:
     NS_DECL_NSIDOMXULELEMENT
 
     // nsIDOMEventTarget interface (from nsIDOMEventReceiver)
-    NS_IMETHOD AddEventListener(const nsAReadableString& aType, nsIDOMEventListener* aListener, 
+    NS_IMETHOD AddEventListener(const nsAString& aType, nsIDOMEventListener* aListener, 
                                 PRBool aUseCapture);
-    NS_IMETHOD RemoveEventListener(const nsAReadableString& aType, nsIDOMEventListener* aListener, 
+    NS_IMETHOD RemoveEventListener(const nsAString& aType, nsIDOMEventListener* aListener, 
                                    PRBool aUseCapture);
     NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool* _retval);
 
@@ -451,7 +518,7 @@ public:
     NS_IMETHOD CompileEventHandler(nsIScriptContext* aContext,
                                    void* aTarget,
                                    nsIAtom *aName,
-                                   const nsAReadableString& aBody,
+                                   const nsAString& aBody,
                                    void** aHandler);
     NS_IMETHOD GetCompiledEventHandler(nsIAtom *aName, void** aHandler);
 
@@ -472,7 +539,7 @@ protected:
     // Implementation methods
     nsresult EnsureContentsGenerated(void) const;
 
-    nsresult ExecuteOnBroadcastHandler(nsIDOMElement* anElement, const nsAReadableString& attrName);
+    nsresult ExecuteOnBroadcastHandler(nsIDOMElement* anElement, const nsAString& attrName);
 
     static nsresult
     ExecuteJSCode(nsIDOMElement* anElement, nsEvent* aEvent);
@@ -480,13 +547,13 @@ protected:
     // Static helpers
     static nsresult
     GetElementsByTagName(nsIDOMNode* aNode,
-                         const nsAReadableString& aTagName,
+                         const nsAString& aTagName,
                          nsRDFDOMNodeList* aElements);
 
     static nsresult
     GetElementsByAttribute(nsIDOMNode* aNode,
-                           const nsAReadableString& aAttributeName,
-                           const nsAReadableString& aAttributeValue,
+                           const nsAString& aAttributeName,
+                           const nsAString& aAttributeValue,
                            nsRDFDOMNodeList* aElements);
 
     static PRBool IsAncestor(nsIDOMNode* aParentNode, nsIDOMNode* aChildNode);
@@ -522,15 +589,41 @@ protected:
         nsCOMPtr<nsIControllers>            mControllers;        // [OWNER]
 
         /**
-         * Attribute values that override the values from the prototype.
+         * Contains the mLazyState in the low two bits, and a pointer
+         * to the nsXULAttributes structure in the high bits.
          */
-        nsXULAttributes*                    mAttributes;
+        PRWord                              mBits;
 
-        /**
-         * The state of our sloth for lazy content model construction via
-         * RDF; see nsIXULContent and nsRDFGenericBuilder.
-         */
-        PRInt32                             mLazyState;
+#define LAZYSTATE_MASK  ((PRWord(1) << LAZYSTATE_BITS) - 1)
+#define ATTRIBUTES_MASK (~LAZYSTATE_MASK)
+
+        nsXULAttributes *
+        GetAttributes() const {
+            return NS_REINTERPRET_CAST(nsXULAttributes *, mBits & ATTRIBUTES_MASK);
+        }
+
+        void
+        SetAttributes(nsXULAttributes *aAttributes) {
+            NS_ASSERTION((NS_REINTERPRET_CAST(PRWord, aAttributes) & ~ATTRIBUTES_MASK) == 0,
+                         "nsXULAttributes pointer is unaligned");
+
+            mBits &= ~ATTRIBUTES_MASK;
+            mBits |= NS_REINTERPRET_CAST(PRWord, aAttributes);
+        }
+
+        LazyState
+        GetLazyState() const {
+            return LazyState(mBits & LAZYSTATE_MASK);
+        }
+
+        void
+        SetLazyState(LazyState aLazyState) {
+            NS_ASSERTION((aLazyState & ~LAZYSTATE_MASK) == 0,
+                         "lazy state includes high bits");
+
+            mBits &= ~LAZYSTATE_MASK;
+            mBits |= PRWord(aLazyState);
+        }
     };
 
     friend struct Slots;
@@ -581,13 +674,18 @@ protected:
     nsresult AddListenerFor(nsINodeInfo *aNodeInfo,
                             PRBool aCompileEventHandlers);
 
+    
+    nsresult HideWindowChrome(PRBool aShouldHide);
+
 protected:
     // Internal accessors. These shadow the 'Slots', and return
     // appropriate default values if there are no slots defined in the
     // delegate.
-    nsINodeInfo*               NodeInfo() const        { return mSlots ? mSlots->mNodeInfo              : mPrototype->mNodeInfo; }
-    nsIControllers*            Controllers() const        { return mSlots ? mSlots->mControllers.get()        : nsnull; }
-    nsXULAttributes*           Attributes() const         { return mSlots ? mSlots->mAttributes               : nsnull; }
+    nsINodeInfo     *NodeInfo() const    { return mSlots ? mSlots->mNodeInfo          : mPrototype->mNodeInfo; }
+    nsIControllers  *Controllers() const { return mSlots ? mSlots->mControllers.get() : nsnull; }
+    nsXULAttributes *Attributes() const  { return mSlots ? mSlots->GetAttributes()    : nsnull; }
+
+    void UnregisterAccessKey(const nsAString& aOldValue);
 
     static nsIXBLService *gXBLService;
 };
