@@ -42,10 +42,7 @@
 /*
 
   An implementation for a Gecko-style content sink that knows how
-  to build an RDF content model from XUL.
-
-  For each container tag, an RDF Sequence object is created that
-  contains the order set of children of that container.
+  to build a content model from XUL.
 
   For more information on XUL, see http://www.mozilla.org/xpfe
 
@@ -74,8 +71,6 @@
 #include "nsINodeInfo.h"
 #include "nsIParser.h"
 #include "nsIPresShell.h"
-#include "nsIRDFCompositeDataSource.h"
-#include "nsIRDFService.h"
 #include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptGlobalObjectOwner.h"
@@ -100,6 +95,7 @@
 #include "prlog.h"
 #include "prmem.h"
 #include "jsapi.h"  // for JSVERSION_*, JS_VersionToString, etc.
+#include "nsCRT.h"
 
 #include "nsIFastLoadService.h"         // XXXbe temporary
 #include "nsIObjectInputStream.h"       // XXXbe temporary
@@ -120,7 +116,6 @@ static PRLogModuleInfo* gLog;
 static const char kXULNameSpaceURI[] = XUL_NAMESPACE_URI;
 
 
-static NS_DEFINE_CID(kCSSParserCID,              NS_CSSPARSER_CID);
 static NS_DEFINE_CID(kNameSpaceManagerCID,       NS_NAMESPACEMANAGER_CID);
 static NS_DEFINE_CID(kXULPrototypeCacheCID,      NS_XULPROTOTYPECACHE_CID);
 
@@ -144,7 +139,7 @@ public:
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
     NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-    NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset);
+    NS_IMETHOD SetDocumentCharset(nsAString& aCharset);
 
     // nsIXULContentSink
     NS_IMETHOD Init(nsIDocument* aDocument, nsIXULPrototypeDocument* aPrototype);
@@ -169,8 +164,8 @@ protected:
     PRBool mConstrainSize;
 
     // namespace management -- XXX combine with ContextStack?
-    void PushNameSpacesFrom(const PRUnichar** aAttributes);
-    
+    nsresult PushNameSpacesFrom(const PRUnichar** aAttributes);
+
     // RDF-specific parsing
     nsresult ParseTag(const PRUnichar* aText, nsINodeInfo*& aNodeInfo);
     nsresult AddAttributes(const PRUnichar** aAttributes, 
@@ -206,7 +201,7 @@ protected:
     nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
     
     
-    nsresult NormalizeAttributeString(const nsAReadableString& aText,
+    nsresult NormalizeAttributeString(const nsAFlatString& aText,
                                       nsINodeInfo*& aNodeInfo);
     nsresult CreateElement(nsINodeInfo *aNodeInfo, nsXULPrototypeElement** aResult);
 
@@ -553,10 +548,6 @@ XULContentSinkImpl::WillInterrupt(void)
 NS_IMETHODIMP 
 XULContentSinkImpl::WillResume(void)
 {
-    nsCOMPtr<nsIXULDocument> xuldoc(do_QueryReferent(mDocument));
-    if (xuldoc)
-        xuldoc->OnResumeContentSink();
-
     // XXX Notify the webshell, if necessary
     return NS_OK;
 }
@@ -570,23 +561,6 @@ XULContentSinkImpl::SetParser(nsIParser* aParser)
     return NS_OK;
 }
 
-static void SplitMimeType(const nsString& aValue, nsString& aType, nsString& aParams)
-{
-  aType.Truncate();
-  aParams.Truncate();
-  PRInt32 semiIndex = aValue.FindChar(PRUnichar(';'));
-  if (-1 != semiIndex) {
-    aValue.Left(aType, semiIndex);
-    aValue.Right(aParams, (aValue.Length() - semiIndex) - 1);
-    aParams.StripWhitespace();
-  }
-  else {
-    aType = aValue;
-  }
-  aType.StripWhitespace();
-}
-
-
 nsresult
 XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
                                      const nsString& aHref,
@@ -600,18 +574,18 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
     nsresult rv = NS_OK;
 
     if (aAlternate) { // if alternate, does it have title?
-        if (0 == aTitle.Length()) { // alternates must have title
+        if (aTitle.IsEmpty()) { // alternates must have title
             return NS_OK; //return without error, for now
         }
     }
 
-    nsAutoString  mimeType;
-    nsAutoString  params;
-    SplitMimeType(aType, mimeType, params);
+    nsAutoString mimeType;
+    nsAutoString params;
+    nsParserUtils::SplitMimeType(aType, mimeType, params);
 
-    if ((0 == mimeType.Length()) || mimeType.EqualsIgnoreCase(kCSSType)) {
+    if ((mimeType.IsEmpty()) || mimeType.EqualsIgnoreCase(kCSSType)) {
         nsCOMPtr<nsIURI> url;
-        rv = NS_NewURI(getter_AddRefs(url), aHref, mDocumentURL);
+        rv = NS_NewURI(getter_AddRefs(url), aHref, nsnull, mDocumentURL);
         if (NS_OK != rv) {
             return NS_OK; // The URL is bad, move along, don't propagate the error (for now)
         }
@@ -622,8 +596,8 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
         // Nope, we need to load it asynchronously
         PRBool blockParser = PR_FALSE;
         if (! aAlternate) {
-            if (0 < aTitle.Length()) {  // possibly preferred sheet
-                if (0 == mPreferredStyle.Length()) {
+            if (!aTitle.IsEmpty()) {  // possibly preferred sheet
+                if (mPreferredStyle.IsEmpty()) {
                     mPreferredStyle = aTitle;
                     mCSSLoader->SetPreferredSheet(aTitle);
                     nsIAtom* defaultStyle = NS_NewAtom("default-style");
@@ -658,7 +632,7 @@ XULContentSinkImpl::ProcessStyleLink(nsIContent* aElement,
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::SetDocumentCharset(nsAWritableString& aCharset)
+XULContentSinkImpl::SetDocumentCharset(nsAString& aCharset)
 {
     nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
     if (doc) {
@@ -707,7 +681,7 @@ XULContentSinkImpl::Init(nsIDocument* aDocument, nsIXULPrototypeDocument* aProto
     rv = htmlContainer->GetCSSLoader(*getter_AddRefs(mCSSLoader));
     if (NS_FAILED(rv)) return rv;
 
-    rv = aDocument->GetNodeInfoManager(*getter_AddRefs(mNodeInfoManager));
+    rv = aPrototype->GetNodeInfoManager(getter_AddRefs(mNodeInfoManager));
     if (NS_FAILED(rv)) return rv;
 
     mState = eInProlog;
@@ -785,39 +759,46 @@ XULContentSinkImpl::FlushText(PRBool aCreateTextNode)
 //----------------------------------------------------------------------
 
 nsresult
-XULContentSinkImpl::NormalizeAttributeString(const nsAReadableString& aText,
+XULContentSinkImpl::NormalizeAttributeString(const nsAFlatString& aText,
                                              nsINodeInfo*& aNodeInfo)
 {
-    nsresult rv;
     PRInt32 nameSpaceID = kNameSpaceID_None;
 
-    // Split the attribute into prefix and tag substrings.
-    nsAutoString prefixStr;
-    nsAutoString attrStr(aText);
-    PRInt32 nsoffset = attrStr.FindChar(kNameSpaceSeparator);
-    if (nsoffset >= 0) {
-        attrStr.Left(prefixStr, nsoffset);
-        attrStr.Cut(0, nsoffset + 1);
-    }
+    nsAFlatString::const_iterator start, end;
+    aText.BeginReading(start);
+    aText.EndReading(end);
+
+    nsAFlatString::const_iterator colon(start);
 
     nsCOMPtr<nsIAtom> prefix;
-    if (prefixStr.Length() > 0) {
-        prefix = dont_AddRef( NS_NewAtom(prefixStr) );
+
+    if (!FindCharInReadable(kNameSpaceSeparator, colon, end)) {
+        colon = start; // No ':' found, reset colon
+    } else if (start != colon) {
+        prefix = dont_AddRef(NS_NewAtom(Substring(start, colon)));
 
         nsCOMPtr<nsINameSpace> ns;
-        rv = GetTopNameSpace(address_of(ns));
-        if (NS_FAILED(rv)) return rv;
+        GetTopNameSpace(address_of(ns));
 
-        rv = ns->FindNameSpaceID(prefix, nameSpaceID);
-        if (NS_FAILED(rv)) return rv;
+        if (ns) {
+            ns->FindNameSpaceID(prefix, nameSpaceID);
+
+            if (nameSpaceID == kNameSpaceID_Unknown) {
+                NS_WARNING("Undeclared prefix used in attribute name.");
+
+                nameSpaceID = kNameSpaceID_None;
+            }
+        } else {
+            NS_WARNING("Undeclared prefix used in attribute name.");
+        }
+
+        ++colon; // Skip over the ':'
     }
 
-    mNodeInfoManager->GetNodeInfo(attrStr, prefixStr, nameSpaceID, aNodeInfo);
-
-    return NS_OK;
+    return mNodeInfoManager->GetNodeInfo(Substring(colon, end), prefix,
+                                         nameSpaceID, aNodeInfo);
 }
 
-
 nsresult
 XULContentSinkImpl::CreateElement(nsINodeInfo *aNodeInfo,
                                   nsXULPrototypeElement** aResult)
@@ -878,9 +859,8 @@ XULContentSinkImpl::HandleStartElement(const PRUnichar *aName,
   // list of an element before creating the element. This is because
   // the namespace prefix for an element might be declared within
   // the attribute list.
-  PushNameSpacesFrom(aAtts);
-
-  nsresult rv;
+  nsresult rv = PushNameSpacesFrom(aAtts);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsINodeInfo> nodeInfo;
   rv = ParseTag(aName, *getter_AddRefs(nodeInfo));
@@ -962,8 +942,10 @@ XULContentSinkImpl::HandleEndElement(const PRUnichar *aName)
         if (! script->mSrcURI && ! script->mJSObject) {
             nsCOMPtr<nsIDocument> doc = do_QueryReferent(mDocument);
 
-            rv = script->Compile(mText, mTextLength, mDocumentURL,
-                                 script->mLineNo, doc, mPrototype);
+            script->mOutOfLine = PR_FALSE;
+            if (doc)
+                script->Compile(mText, mTextLength, mDocumentURL,
+                                script->mLineNo, doc, mPrototype);
         }
 
         FlushText(PR_FALSE);
@@ -1019,7 +1001,11 @@ XULContentSinkImpl::HandleCDataSection(const PRUnichar *aData, PRUint32 aLength)
 }
 
 NS_IMETHODIMP 
-XULContentSinkImpl::HandleDoctypeDecl(const PRUnichar *aDoctype)
+XULContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset, 
+                                      const nsAString & aName, 
+                                      const nsAString & aSystemId, 
+                                      const nsAString & aPublicId,
+                                      nsISupports* aCatalogData)
 {
     return NS_OK;
 }
@@ -1060,13 +1046,13 @@ XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 
       // If there was no href, we can't do
       // anything with this PI
-      if (0 == href.Length()) {
+      if (href.IsEmpty()) {
         return NS_OK;
       }
 
       // Add the overlay to our list of overlays that need to be processed.
       nsCOMPtr<nsIURI> url;
-      nsresult rv = NS_NewURI(getter_AddRefs(url), href, mDocumentURL);
+      nsresult rv = NS_NewURI(getter_AddRefs(url), href, nsnull, mDocumentURL);
       if (NS_FAILED(rv)) {
         // XXX This is wrong, the error message could be out of memory
         //     or something else equally bad, which we should propagate. 
@@ -1086,7 +1072,7 @@ XULContentSinkImpl::HandleProcessingInstruction(const PRUnichar *aTarget,
 
         // If there was no href, we can't do
         // anything with this PI
-        if (0 == href.Length())
+        if (href.IsEmpty())
             return NS_OK;
 
         nsAutoString type;
@@ -1147,7 +1133,7 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
   mState = eInProlog;
 
   NS_NAMED_LITERAL_STRING(name, "xmlns");
-  NS_NAMED_LITERAL_STRING(value, "http://www.w3.org/1999/xhtml");
+  NS_NAMED_LITERAL_STRING(value, "http://www.mozilla.org/newlayout/xml/parsererror.xml");
 
   const PRUnichar* atts[] = {name.get(), value.get(), nsnull};;
     
@@ -1157,7 +1143,8 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
   rv = HandleCharacterData(aErrorText, nsCRT::strlen(aErrorText));
   NS_ENSURE_SUCCESS(rv,rv);  
   
-  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), atts, 1, -1, -1);
+  const PRUnichar* noAtts[] = {0, 0};
+  rv = HandleStartElement(NS_LITERAL_STRING("sourcetext").get(), noAtts, 0, -1, -1);
   NS_ENSURE_SUCCESS(rv,rv);
   
   rv = HandleCharacterData(aSourceText, nsCRT::strlen(aSourceText));
@@ -1177,59 +1164,74 @@ XULContentSinkImpl::ReportError(const PRUnichar* aErrorText,
 // Namespace management
 //
 
-void
+nsresult
 XULContentSinkImpl::PushNameSpacesFrom(const PRUnichar** aAttributes)
 {
-  nsINameSpace* nameSpace = nsnull;
-  if (0 < mNameSpaceStack.Count()) {
-      nameSpace = (nsINameSpace*)mNameSpaceStack[mNameSpaceStack.Count() - 1];
-      NS_ADDREF(nameSpace);
-  }
-  else {
-      gNameSpaceManager->CreateRootNameSpace(nameSpace);
-  }
+    nsCOMPtr<nsINameSpace> nameSpace;
 
-  NS_ASSERTION(nameSpace != nsnull, "no parent namespace");
-  if (! nameSpace)
-      return;
+    if (mNameSpaceStack.Count() > 0) {
+        nameSpace =
+            (nsINameSpace*)mNameSpaceStack.ElementAt(mNameSpaceStack.Count() - 1);
+    } else {
+        gNameSpaceManager->CreateRootNameSpace(*getter_AddRefs(nameSpace));
+        if (! nameSpace)
+            return NS_ERROR_OUT_OF_MEMORY;
+    }
 
-  for (; *aAttributes; aAttributes += 2) {
-      nsAutoString k(aAttributes[0]);
+    static const NS_NAMED_LITERAL_STRING(kNameSpaceDef, "xmlns");
+    static const PRUint32 xmlns_len = 5; // kNameSpaceDef.Length();
+    NS_ASSERTION(kNameSpaceDef.Length() == xmlns_len,
+                 "xmlns_len incorrectly set!");
 
-      // Look for "xmlns" at the start of the attribute name
-      PRInt32 offset = k.Find(kNameSpaceDef);
-      if (0 != offset) {
-        continue;
-      }
+    for (; *aAttributes; aAttributes += 2) {
+        nsDependentString key(aAttributes[0]);
 
-      nsAutoString prefix;
-      if (k.Length() >= PRInt32(sizeof kNameSpaceDef)) {
-        // If the next character is a :, there is a namespace prefix
-        PRUnichar next = k.CharAt(sizeof(kNameSpaceDef)-1);
-        if (':' == next) {
-            k.Right(prefix, k.Length()-sizeof(kNameSpaceDef));
+        // Look for "xmlns" at the start of the attribute name
+
+        PRUint32 key_len = key.Length();
+
+        if (key_len >= xmlns_len &&
+            Substring(key, 0, xmlns_len).Equals(kNameSpaceDef)) {
+            nsCOMPtr<nsIAtom> prefixAtom;
+
+            // If key_len > xmlns_len we have a xmlns:foo type attribute,
+            // extract the prefix. If not, we have a xmlns attribute in
+            // which case there is no prefix.
+
+            if (key_len > xmlns_len) {
+                nsDependentString::const_iterator start, end;
+
+                key.BeginReading(start);
+                key.EndReading(end);
+
+                start.advance(xmlns_len);
+
+                if (*start == ':' && ++start != end) {
+                    prefixAtom =
+                        dont_AddRef(NS_NewAtom(Substring(start, end)));
+                } else {
+                    NS_WARNING("Bad XML namespace declaration 'xmlns:' "
+                               "found!");
+                }
+            }
+
+            nsDependentString value(aAttributes[1]);
+
+            nsCOMPtr<nsINameSpace> child;
+            nsresult rv =
+                nameSpace->CreateChildNameSpace(prefixAtom, value,
+                                                *getter_AddRefs(child));
+            NS_ENSURE_SUCCESS(rv, rv);
+
+            nameSpace = child;
         }
-        else {
-          aAttributes += 2;
-          continue; // it's not "xmlns:"
-        }
-      }
+    }
 
-      // Get the attribute value (the URI for the namespace)
-      // Open a local namespace
-      nsIAtom* prefixAtom = ((0 < prefix.Length()) ? NS_NewAtom(prefix) : nsnull);
-      nsINameSpace* child = nsnull;
-      nameSpace->CreateChildNameSpace(prefixAtom, nsDependentString(aAttributes[1]), child);
-      if (nsnull != child) {
-          NS_RELEASE(nameSpace);
-          nameSpace = child;
-      }
+    nsINameSpace *tmp = nameSpace;
+    mNameSpaceStack.AppendElement(tmp);
+    NS_ADDREF(tmp);
 
-      NS_IF_RELEASE(prefixAtom);
-  }
-
-  // Now push the *last* namespace that we discovered on to the stack.
-  mNameSpaceStack.AppendElement(nameSpace);
+    return NS_OK;
 }
 
 void
@@ -1264,35 +1266,45 @@ nsresult
 XULContentSinkImpl::ParseTag(const PRUnichar* aText, 
                              nsINodeInfo*& aNodeInfo)
 {
-  nsresult rv;
+    // Split the tag into prefix and tag substrings.
 
-  // Split the tag into prefix and tag substrings.
-  nsAutoString prefixStr;
-  nsAutoString tagStr(aText);
-  PRInt32 nsoffset = tagStr.FindChar(kNameSpaceSeparator);
-  if (nsoffset >= 0) {
-      tagStr.Left(prefixStr, nsoffset);
-      tagStr.Cut(0, nsoffset + 1);
-  }
+    nsDependentString text(aText);
 
-  nsCOMPtr<nsIAtom> prefix;
-  if (prefixStr.Length() > 0) {
-      prefix = dont_AddRef( NS_NewAtom(prefixStr) );
-  }
+    nsDependentString::const_iterator start, end;
+    text.BeginReading(start);
+    text.EndReading(end);
 
-  nsCOMPtr<nsINameSpace> ns;
-  rv = GetTopNameSpace(address_of(ns));
-  if (NS_FAILED(rv)) return rv;
+    nsDependentString::const_iterator colon(start);
 
-  PRInt32 namespaceID;
-  rv = ns->FindNameSpaceID(prefix, namespaceID);
-  if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIAtom> prefix;
 
-  nsCOMPtr<nsIAtom> tag(dont_AddRef(NS_NewAtom(tagStr)));
-  if (! tag)
-      return NS_ERROR_OUT_OF_MEMORY;
+    if (!FindCharInReadable(kNameSpaceSeparator, colon, end)) {
+        colon = start; // No ':' found, reset colon
+    } else if (colon != start) {
+        prefix = dont_AddRef(NS_NewAtom(Substring(start, colon)));
 
-  return mNodeInfoManager->GetNodeInfo(tag, prefix, namespaceID, aNodeInfo);
+        ++colon; // Step over ':'
+    }
+
+    nsCOMPtr<nsINameSpace> ns;
+    GetTopNameSpace(address_of(ns));
+
+    PRInt32 namespaceID = kNameSpaceID_None;
+
+    if (ns) {
+        ns->FindNameSpaceID(prefix, namespaceID);
+
+        if (namespaceID == kNameSpaceID_Unknown) {
+            NS_WARNING("Undeclared prefix used in tag name!");
+
+            namespaceID = kNameSpaceID_None;
+        }
+    } else {
+        NS_WARNING("Undeclared prefix used in tag name!");
+    }
+
+    return mNodeInfoManager->GetNodeInfo(Substring(colon, end), prefix,
+                                         namespaceID, aNodeInfo);
 }
 
 nsresult
@@ -1306,7 +1318,7 @@ XULContentSinkImpl::OpenRoot(const PRUnichar** aAttributes,
 
     nsresult rv;
 
-    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_HTML) || 
+    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XHTML) || 
         aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XUL)) {
         PR_LOG(gLog, PR_LOG_ALWAYS,
                ("xul: script tag not allowed as root content element"));
@@ -1355,12 +1367,6 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
 {
     nsresult rv;
 
-    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_HTML) || 
-        aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XUL)) {
-        // Oops, it's a script!
-        return OpenScript(aAttributes, aLineNumber);
-    }
-
     // Create the element
     nsXULPrototypeElement* element;
     rv = CreateElement(aNodeInfo, &element);
@@ -1386,17 +1392,24 @@ XULContentSinkImpl::OpenTag(const PRUnichar** aAttributes,
         return rv;
     }
 
+    // Add the attributes
+    rv = AddAttributes(aAttributes, aAttrLen, element);
+    if (NS_FAILED(rv)) return rv;
+
     children->AppendElement(element);
 
+    if (aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XHTML) || 
+        aNodeInfo->Equals(kScriptAtom, kNameSpaceID_XUL)) {
+        // Do scripty things now.  OpenScript will push the
+        // nsPrototypeScriptElement onto the stack, so we're done after this.
+        return OpenScript(aAttributes, aLineNumber);
+    }
+
     // Push the element onto the context stack, so that child
     // containers will hook up to us as their parent.
     rv = mContextStack.Push(element, mState);
     if (NS_FAILED(rv)) return rv;
 
-    // Add the attributes
-    rv = AddAttributes(aAttributes, aAttrLen, element);
-    if (NS_FAILED(rv)) return rv;
-
     mState = eInDocumentElement;
     return NS_OK;
 }
@@ -1413,14 +1426,16 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
   nsAutoString src;
   while (*aAttributes) {
       const nsDependentString key(aAttributes[0]);
-      if (Compare(key, NS_LITERAL_STRING("src"),nsCaseInsensitiveStringComparator()) == 0) {
+      if (key.Equals(NS_LITERAL_STRING("src"),
+                     nsCaseInsensitiveStringComparator())) {
           src.Assign(aAttributes[1]);
       }
-      else if (Compare(key, NS_LITERAL_STRING("type"),nsCaseInsensitiveStringComparator()) == 0) {
+      else if (key.Equals(NS_LITERAL_STRING("type"),
+                          nsCaseInsensitiveStringComparator())) {
           nsAutoString  type(aAttributes[1]);
           nsAutoString  mimeType;
           nsAutoString  params;
-          SplitMimeType(type, mimeType, params);
+          nsParserUtils::SplitMimeType(type, mimeType, params);
 
           isJavaScript = mimeType.EqualsIgnoreCase("application/x-javascript") ||
                          mimeType.EqualsIgnoreCase("text/javascript");
@@ -1442,7 +1457,8 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
               jsVersionString = JS_VersionToString(jsVersion);
           }
       }
-      else if (Compare(key, NS_LITERAL_STRING("language"),nsCaseInsensitiveStringComparator()) == 0) {
+      else if (key.Equals(NS_LITERAL_STRING("language"),
+                          nsCaseInsensitiveStringComparator())) {
         nsAutoString  lang(aAttributes[1]);
         isJavaScript = nsParserUtils::IsJavaScriptLanguage(lang, &jsVersionString);
       }
@@ -1460,116 +1476,13 @@ XULContentSinkImpl::OpenScript(const PRUnichar** aAttributes,
       // If there is a SRC attribute...
       if (! src.IsEmpty()) {
           // Use the SRC attribute value to load the URL
-          rv = NS_NewURI(getter_AddRefs(script->mSrcURI), src, mDocumentURL);
+          rv = NS_NewURI(getter_AddRefs(script->mSrcURI), src, nsnull, mDocumentURL);
           if (NS_FAILED(rv)) {
               delete script;
               return rv;
           }
       }
 
-      // XXXbe temporary, until we serialize/deserialize everything from
-      //       the nsXULPrototypeDocument on down...
-      nsCOMPtr<nsIFastLoadService> fastLoadService;
-      nsCOMPtr<nsIObjectInputStream> objectInput;
-      nsXULDocument::GetFastLoadService(getter_AddRefs(fastLoadService));
-      if (fastLoadService)
-          fastLoadService->GetInputStream(getter_AddRefs(objectInput));
-
-      if (objectInput) {
-          PRBool useXULCache = PR_TRUE;
-          if (script->mSrcURI) {
-              // NB: we must check the XUL script cache early, to avoid
-              // multiple deserialization attempts for a given script, which
-              // would exhaust the multiplexed stream containing the singly
-              // serialized script.  Note that nsXULDocument::LoadScript
-              // checks the XUL script cache too, in order to handle the
-              // serialization case.
-              //
-              // We need do this only for <script src='strres.js'> and the
-              // like, i.e., out-of-line scripts that are included by several
-              // different XUL documents multiplexed in the FastLoad file.
-              gXULCache->GetEnabled(&useXULCache);
-
-              if (useXULCache) {
-                  gXULCache->GetScript(script->mSrcURI,
-                                       NS_REINTERPRET_CAST(void**, &script->mJSObject));
-              }
-          }
-
-          if (! script->mJSObject) {
-              nsCOMPtr<nsIScriptGlobalObjectOwner> globalOwner(do_QueryInterface(mPrototype));
-              nsCOMPtr<nsIScriptGlobalObject> globalObject;
-              globalOwner->GetScriptGlobalObject(getter_AddRefs(globalObject));
-              NS_ASSERTION(globalObject != nsnull, "no prototype global object!");
-
-              nsCOMPtr<nsIScriptContext> scriptContext;
-              globalObject->GetContext(getter_AddRefs(scriptContext));
-              NS_ASSERTION(scriptContext != nsnull,
-                           "no prototype script context!");
-
-              // Keep track of FastLoad failure via rv2, so we can
-              // AbortFastLoads if things look bad.
-              nsresult rv2 = NS_OK;
-              if (script->mSrcURI) {
-                  nsXPIDLCString spec;
-                  script->mSrcURI->GetSpec(getter_Copies(spec));
-                  rv2 = fastLoadService->StartMuxedDocument(script->mSrcURI,
-                                                            spec,
-                                                            nsIFastLoadService::NS_FASTLOAD_READ);
-                  if (NS_SUCCEEDED(rv2))
-                      rv2 = fastLoadService->SelectMuxedDocument(script->mSrcURI);
-              } else {
-                  // An inline script: check FastLoad multiplexing direction
-                  // and skip script->Deserialize if we're not reading from a
-                  // muxed stream to get inline objects that are contained in
-                  // the current document.
-                  PRInt32 direction;
-                  fastLoadService->GetDirection(&direction);
-                  if (direction != nsIFastLoadService::NS_FASTLOAD_READ)
-                      rv2 = NS_ERROR_NOT_AVAILABLE;
-              }
-
-              // Don't reflect errors into rv: mJSObject will be null
-              // after any error, which suffices to cause the script to
-              // be reloaded (from the src= URI, if any) and recompiled.
-              // We're better off slow-loading than bailing out due to a
-              // FastLoad error.
-              if (NS_SUCCEEDED(rv2))
-                  rv2 = script->Deserialize(objectInput, scriptContext);
-
-              if (NS_SUCCEEDED(rv2) && script->mSrcURI) {
-                  rv2 = fastLoadService->EndMuxedDocument(script->mSrcURI);
-                  if (NS_SUCCEEDED(rv2)) {
-                      // We must reselect the including XUL document now,
-                      // in case the next script in it is an inline one
-                      // whose data comes from its stream, not from the
-                      // now-exhausted document that was multiplexed for
-                      // script->mSrcURI.  Inline scripts are compiled in
-                      // XULContentSinkImpl::CloseContainer.
-                      rv2 = fastLoadService->SelectMuxedDocument(mDocumentURL);
-                  }
-              }
-
-              if (NS_SUCCEEDED(rv2)) {
-                  if (useXULCache && script->mSrcURI) {
-                      PRBool isChrome = PR_FALSE;
-                      script->mSrcURI->SchemeIs("chrome", &isChrome);
-                      if (isChrome) {
-                          gXULCache->PutScript(script->mSrcURI,
-                                               NS_REINTERPRET_CAST(void*, script->mJSObject));
-                      }
-                  }
-              } else {
-                  // If script->mSrcURI is not in the FastLoad multiplex,
-                  // rv2 will be NS_ERROR_NOT_AVAILABLE and we'll try to
-                  // update the FastLoad file to hold a serialization of
-                  // this script, once it has finished loading.
-                  if (rv2 != NS_ERROR_NOT_AVAILABLE)
-                      nsXULDocument::AbortFastLoads();
-              }
-          }
-      }
-
       nsVoidArray* children;
       rv = mContextStack.GetTopChildren(&children);
       if (NS_FAILED(rv)) {
@@ -1618,7 +1531,7 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
           qnameC.Assign(aAttributes[0]);
           PR_LOG(gLog, PR_LOG_ALWAYS,
                  ("xul: unable to parse attribute '%s' at line %d",
-                  NS_ConvertUCS2toUTF8(qnameC), -1)); // XXX pass in line number
+                  NS_ConvertUCS2toUTF8(qnameC).get(), -1)); // XXX pass in line number
 #endif
           // Bring it. We'll just fail to copy an attribute that we
           // can't parse. And that's one less attribute to worry
@@ -1633,16 +1546,16 @@ XULContentSinkImpl::AddAttributes(const PRUnichar** aAttributes,
           nsAutoString extraWhiteSpace;
           PRInt32 cnt = mContextStack.Depth();
           while (--cnt >= 0)
-              extraWhiteSpace.Assign(NS_LITERAL_STRING("  "));
+              extraWhiteSpace.Append(NS_LITERAL_STRING("  "));
           nsAutoString qnameC,valueC;
           qnameC.Assign(aAttributes[0]);
           valueC.Assign(aAttributes[1]);
           PR_LOG(gLog, PR_LOG_DEBUG,
                  ("xul: %.5d. %s    %s=%s",
                   -1, // XXX pass in line number
-                  NS_ConvertUCS2toUTF8(extraWhiteSpace),
-                  NS_ConvertUCS2toUTF8(qnameC),
-                  NS_ConvertUCS2toUTF8(valueC)));
+                  NS_ConvertUCS2toUTF8(extraWhiteSpace).get(),
+                  NS_ConvertUCS2toUTF8(qnameC).get(),
+                  NS_ConvertUCS2toUTF8(valueC).get()));
       }
 #endif
       ++attrs;
