@@ -22,6 +22,7 @@
  * Contributor(s):
  *   Chris Waterson <waterson@netscape.com>
  *   L. David Baron <dbaron@fas.harvard.edu>
+ *   Ben Goodger <ben@netscape.com>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -85,7 +86,8 @@ public:
     NS_IMETHOD SetContext(nsIScriptContext *aContext);
     NS_IMETHOD GetContext(nsIScriptContext **aContext);
     NS_IMETHOD SetNewDocument(nsIDOMDocument *aDocument,
-                              PRBool removeEventListeners);
+                              PRBool aRemoveEventListeners,
+                              PRBool aClearScope);
     NS_IMETHOD SetDocShell(nsIDocShell *aDocShell);
     NS_IMETHOD GetDocShell(nsIDocShell **aDocShell);
     NS_IMETHOD SetOpenerWindow(nsIDOMWindowInternal *aOpener);
@@ -98,7 +100,7 @@ public:
                               nsEventStatus* aEventStatus);
     NS_IMETHOD_(JSObject *) GetGlobalJSObject();
     NS_IMETHOD OnFinalize(JSObject *aObject);
-    NS_IMETHOD SetScriptsEnabled(PRBool aEnabled);
+    NS_IMETHOD SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
 
     // nsIScriptObjectPrincipal methods
     NS_IMETHOD GetPrincipal(nsIPrincipal** aPrincipal);
@@ -140,14 +142,16 @@ public:
     NS_IMETHOD AddOverlayReference(nsIURI* aURI);
     NS_IMETHOD GetOverlayReferences(nsISupportsArray** aResult);
 
-    NS_IMETHOD GetHeaderData(nsIAtom* aField, nsAWritableString& aData) const;
-    NS_IMETHOD SetHeaderData(nsIAtom* aField, const nsAReadableString& aData);
+    NS_IMETHOD GetHeaderData(nsIAtom* aField, nsAString& aData) const;
+    NS_IMETHOD SetHeaderData(nsIAtom* aField, const nsAString& aData);
 
     NS_IMETHOD GetDocumentPrincipal(nsIPrincipal** aResult);
     NS_IMETHOD SetDocumentPrincipal(nsIPrincipal* aPrincipal);
 
     NS_IMETHOD AwaitLoadDone(nsIXULDocument* aDocument, PRBool* aResult);
     NS_IMETHOD NotifyLoadDone();
+    
+    NS_IMETHOD GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager);
 
     // nsIScriptGlobalObjectOwner methods
     NS_DECL_NSISCRIPTGLOBALOBJECTOWNER
@@ -166,6 +170,8 @@ protected:
     PRPackedBool mLoaded;
     nsCOMPtr<nsICollection> mPrototypeWaiters;
 
+    nsCOMPtr<nsINodeInfoManager> mNodeInfoManager;
+
     nsXULPrototypeDocument();
     virtual ~nsXULPrototypeDocument();
     nsresult Init();
@@ -231,10 +237,19 @@ nsXULPrototypeDocument::Init()
     nsresult rv;
 
     rv = NS_NewISupportsArray(getter_AddRefs(mStyleSheetReferences));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
 
     rv = NS_NewISupportsArray(getter_AddRefs(mOverlayReferences));
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    mNodeInfoManager = do_CreateInstance(NS_NODEINFOMANAGER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    nsCOMPtr<nsINameSpaceManager> nsmgr;
+    rv = NS_NewNameSpaceManager(getter_AddRefs(nsmgr));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = mNodeInfoManager->Init(nsnull, nsmgr);
+    NS_ENSURE_SUCCESS(rv, rv);
 
     return NS_OK;
 }
@@ -286,8 +301,6 @@ NS_NewXULPrototypeDocument(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 // nsISerializable methods
 //
 
-#define XUL_FAST_LOAD_VERSION   0
-
 NS_IMETHODIMP
 nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 {
@@ -295,16 +308,75 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 
     PRUint32 version;
     rv = aStream->Read32(&version);
-    if (NS_FAILED(rv)) return rv;
-
-    if (version != XUL_FAST_LOAD_VERSION)
+    if (version != XUL_FASTLOAD_FILE_VERSION)
         return NS_ERROR_FAILURE;
 
-    rv = aStream->ReadObject(PR_TRUE, getter_AddRefs(mURI));
-    if (NS_FAILED(rv)) return rv;
+    rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(mURI));
 
-    // XXXbe more to come
-    return NS_OK;
+    PRUint32 referenceCount;
+    nsCOMPtr<nsIURI> referenceURI;
+
+    PRUint32 i;
+    // nsISupportsArray mStyleSheetReferences
+    rv |= aStream->Read32(&referenceCount);
+    if (NS_FAILED(rv)) return rv;
+    for (i = 0; i < referenceCount; ++i) {
+        rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(referenceURI));
+        
+        mStyleSheetReferences->AppendElement(referenceURI);
+    }
+
+    // nsISupportsArray mOverlayReferences
+    rv |= aStream->Read32(&referenceCount);
+    for (i = 0; i < referenceCount; ++i) {
+        rv |= aStream->ReadObject(PR_TRUE, getter_AddRefs(referenceURI));
+        
+        mOverlayReferences->AppendElement(referenceURI);
+    }
+
+    // nsIPrincipal mDocumentPrincipal
+    nsCOMPtr<nsIScriptSecurityManager> securityManager = 
+             do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID);
+
+    if (! securityManager)
+        return NS_ERROR_FAILURE;
+
+    rv |= NS_ReadOptionalObject(aStream, PR_TRUE, getter_AddRefs(mDocumentPrincipal));
+    if (!mDocumentPrincipal) {
+        // XXX This should be handled by the security manager, see bug 160042
+        PRBool isChrome = PR_FALSE;
+        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
+            rv |= securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
+        else
+            rv |= securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+    }
+    mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
+
+    // nsIScriptGlobalObject mGlobalObject
+    mGlobalObject = new nsXULPDGlobalObject();
+    if (! mGlobalObject)
+        return NS_ERROR_OUT_OF_MEMORY;
+    rv |= mGlobalObject->SetGlobalObjectOwner(this); // does not refcount
+
+    mRoot = new nsXULPrototypeElement(-1);
+    if (! mRoot)
+       return NS_ERROR_OUT_OF_MEMORY;
+
+    nsCOMPtr<nsIScriptContext> scriptContext;
+    rv |= mGlobalObject->GetContext(getter_AddRefs(scriptContext));
+    NS_ASSERTION(scriptContext != nsnull,
+                 "no prototype script context!");
+
+    PRUint32 type;
+    rv |= aStream->Read32(&type);
+
+    if ((nsXULPrototypeNode::Type)type != nsXULPrototypeNode::eType_Element)
+        return NS_ERROR_FAILURE;
+
+    rv |= mRoot->Deserialize(aStream, scriptContext, mURI, mNodeInfoManager);
+    rv |= NotifyLoadDone();
+
+    return rv;
 }
 
 
@@ -313,27 +385,49 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
 {
     nsresult rv;
 
-    rv = aStream->Write32(XUL_FAST_LOAD_VERSION);
-    if (NS_FAILED(rv)) return rv;
+    rv = aStream->Write32(XUL_FASTLOAD_FILE_VERSION);
+    
+    rv |= aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), PR_TRUE);
+    
+    PRUint32 referenceCount;
+    nsCOMPtr<nsIURI> referenceURI;
 
-    rv = aStream->WriteCompoundObject(mURI, NS_GET_IID(nsIURI), PR_TRUE);
-    if (NS_FAILED(rv)) return rv;
+    PRUint32 i;
 
+    // nsISupportsArray mStyleSheetReferences
+    mStyleSheetReferences->Count(&referenceCount);
+    rv |= aStream->Write32(referenceCount);
+    
+    for (i = 0; i < referenceCount; ++i) {
+        mStyleSheetReferences->QueryElementAt(i, NS_GET_IID(nsIURI), getter_AddRefs(referenceURI));
+        
+        rv |= aStream->WriteCompoundObject(referenceURI, NS_GET_IID(nsIURI), PR_TRUE);
+    }
+
+    // nsISupportsArray mOverlayReferences
+    mOverlayReferences->Count(&referenceCount);
+    rv |= aStream->Write32(referenceCount);
+    
+    for (i = 0; i < referenceCount; ++i) {
+        mOverlayReferences->QueryElementAt(i, NS_GET_IID(nsIURI), getter_AddRefs(referenceURI));
+        
+        rv |= aStream->WriteCompoundObject(referenceURI, NS_GET_IID(nsIURI), PR_TRUE);
+    }
+
+    // nsIPrincipal mDocumentPrincipal
+    rv |= NS_WriteOptionalObject(aStream, mDocumentPrincipal, PR_TRUE);
+    
+    // Now serialize the document contents
+    nsCOMPtr<nsIScriptGlobalObject> globalObject;
+    rv |= GetScriptGlobalObject(getter_AddRefs(globalObject));
+    
     nsCOMPtr<nsIScriptContext> scriptContext;
-    rv = mGlobalObject->GetContext(getter_AddRefs(scriptContext));
-    if (NS_FAILED(rv)) return rv;
-
-#if 0
-    rv = mRoot->Serialize(aStream, scriptContext);
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsISupportsArray> mStyleSheetReferences;
-    nsCOMPtr<nsISupportsArray> mOverlayReferences;
-    nsCOMPtr<nsIPrincipal> mDocumentPrincipal;
-
-    nsCOMPtr<nsIScriptGlobalObject> mGlobalObject;
-#endif
-    return NS_OK;
+    rv |= globalObject->GetContext(getter_AddRefs(scriptContext));
+    
+    if (mRoot)
+        rv |= mRoot->Serialize(aStream, scriptContext);
+ 
+    return rv;
 }
 
 
@@ -354,7 +448,17 @@ nsXULPrototypeDocument::GetURI(nsIURI** aResult)
 NS_IMETHODIMP
 nsXULPrototypeDocument::SetURI(nsIURI* aURI)
 {
-    mURI = dont_QueryInterface(aURI);
+    NS_ASSERTION(!mURI, "Can't change the uri of a xul prototype document");
+    if (mURI)
+        return NS_ERROR_ALREADY_INITIALIZED;
+
+    mURI = aURI;
+    if (!mDocumentPrincipal) {
+        // If the document doesn't have a principal yet we'll force the creation of one
+        // so that mNodeInfoManager properly gets one.
+        nsCOMPtr<nsIPrincipal> principal;
+        GetDocumentPrincipal(getter_AddRefs(principal));
+    }
     return NS_OK;
 }
 
@@ -418,9 +522,8 @@ nsXULPrototypeDocument::GetOverlayReferences(nsISupportsArray** aResult)
 }
 
 
-
 NS_IMETHODIMP
-nsXULPrototypeDocument::GetHeaderData(nsIAtom* aField, nsAWritableString& aData) const
+nsXULPrototypeDocument::GetHeaderData(nsIAtom* aField, nsAString& aData) const
 {
     // XXX Not implemented
     aData.Truncate();
@@ -429,7 +532,7 @@ nsXULPrototypeDocument::GetHeaderData(nsIAtom* aField, nsAWritableString& aData)
 
 
 NS_IMETHODIMP
-nsXULPrototypeDocument::SetHeaderData(nsIAtom* aField, const nsAReadableString& aData)
+nsXULPrototypeDocument::SetHeaderData(nsIAtom* aField, const nsAString& aData)
 {
     // XXX Not implemented
     return NS_OK;
@@ -440,6 +543,7 @@ nsXULPrototypeDocument::SetHeaderData(nsIAtom* aField, const nsAReadableString&
 NS_IMETHODIMP
 nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
 {
+    NS_PRECONDITION(mNodeInfoManager, "missing nodeInfoManager");
     if (!mDocumentPrincipal) {
         nsresult rv;
         nsCOMPtr<nsIScriptSecurityManager> securityManager = 
@@ -448,10 +552,17 @@ nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
         if (NS_FAILED(rv))
             return NS_ERROR_FAILURE;
 
-        rv = securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
+        // XXX This should be handled by the security manager, see bug 160042
+        PRBool isChrome = PR_FALSE;
+        if (NS_SUCCEEDED(mURI->SchemeIs("chrome", &isChrome)) && isChrome)
+            rv = securityManager->GetSystemPrincipal(getter_AddRefs(mDocumentPrincipal));
+        else
+            rv = securityManager->GetCodebasePrincipal(mURI, getter_AddRefs(mDocumentPrincipal));
 
         if (NS_FAILED(rv))
             return NS_ERROR_FAILURE;
+
+        mNodeInfoManager->SetDocumentPrincipal(mDocumentPrincipal);
     }
 
     *aResult = mDocumentPrincipal;
@@ -463,10 +574,19 @@ nsXULPrototypeDocument::GetDocumentPrincipal(nsIPrincipal** aResult)
 NS_IMETHODIMP
 nsXULPrototypeDocument::SetDocumentPrincipal(nsIPrincipal* aPrincipal)
 {
+    NS_PRECONDITION(mNodeInfoManager, "missing nodeInfoManager");
     mDocumentPrincipal = aPrincipal;
+    mNodeInfoManager->SetDocumentPrincipal(aPrincipal);
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsXULPrototypeDocument::GetNodeInfoManager(nsINodeInfoManager** aNodeInfoManager)
+{
+    *aNodeInfoManager = mNodeInfoManager;
+    NS_IF_ADDREF(*aNodeInfoManager);
+    return NS_OK;
+}
 
 
 NS_IMETHODIMP
@@ -640,7 +760,8 @@ nsXULPDGlobalObject::GetContext(nsIScriptContext **aContext)
 
 NS_IMETHODIMP
 nsXULPDGlobalObject::SetNewDocument(nsIDOMDocument *aDocument,
-                                    PRBool removeEventListeners)
+                                    PRBool aRemoveEventListeners,
+                                    PRBool aClearScope)
 {
     NS_NOTREACHED("waaah!");
     return NS_ERROR_UNEXPECTED;
@@ -727,7 +848,7 @@ nsXULPDGlobalObject::OnFinalize(JSObject *aObject)
 }
 
 NS_IMETHODIMP
-nsXULPDGlobalObject::SetScriptsEnabled(PRBool aEnabled)
+nsXULPDGlobalObject::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
 {
     // We don't care...
 
