@@ -47,7 +47,6 @@
 #include "nsIElementFactory.h"
 #include "nsINodeInfo.h"
 #include "nsIPrincipal.h"
-#include "nsIRDFContentModelBuilder.h"
 #include "nsIServiceManager.h"
 #include "nsITextContent.h"
 #include "nsIXULContent.h"
@@ -127,17 +126,11 @@ IsElementInBuilder(nsIContent *aContent, nsIXULTemplateBuilder *aBuilder)
 // nsXULContentBuilder
 //
 
-class nsXULContentBuilder : public nsXULTemplateBuilder,
-                            public nsIRDFContentModelBuilder
+class nsXULContentBuilder : public nsXULTemplateBuilder
 {
 public:
-    NS_DECL_ISUPPORTS_INHERITED
-
     // nsIXULTemplateBuilder interface
     NS_IMETHOD Rebuild();
-
-    // nsIRDFContentModelBuilder interface
-    NS_IMETHOD SetRootContent(nsIContent* aElement);
     NS_IMETHOD CreateContents(nsIContent* aElement);
 
     // nsIDocumentObserver interface
@@ -269,7 +262,7 @@ protected:
     virtual PRBool
     CompileSimpleAttributeCondition(PRInt32 aNameSpaceID,
                                     nsIAtom* aAttribute,
-                                    const nsAReadableString& aValue,
+                                    const nsAString& aValue,
                                     InnerNode* aParentNode,
                                     TestNode** aResult);
     /**
@@ -316,9 +309,6 @@ nsIXULSortService*  nsXULContentBuilder::gXULSortService;
 nsIElementFactory*  nsXULContentBuilder::gHTMLElementFactory;
 nsIElementFactory*  nsXULContentBuilder::gXMLElementFactory;
 
-NS_IMPL_ISUPPORTS_INHERITED1(nsXULContentBuilder, nsXULTemplateBuilder,
-                             nsIRDFContentModelBuilder)
-
 NS_IMETHODIMP
 NS_NewXULContentBuilder(nsISupports* aOuter, REFNSIID aIID, void** aResult)
 {
@@ -665,7 +655,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
             // XXX because gcc-2.7.2.3 is too dumb to keep a
             // compiler-generated temporary around.
             NS_ConvertUTF8toUCS2 x(uri);
-            const nsAReadableString& id = x;
+            const nsAString& id = x;
             rv = realKid->SetAttr(kNameSpaceID_None, nsXULAtoms::id, id, PR_FALSE);
             NS_ASSERTION(NS_SUCCEEDED(rv), "unable to set id attribute");
             if (NS_FAILED(rv)) return rv;
@@ -714,7 +704,7 @@ nsXULContentBuilder::BuildContentFromTemplate(nsIContent *aTemplateNode,
             rv = tmplKid->GetAttr(kNameSpaceID_None, nsXULAtoms::value, attrValue);
             if (NS_FAILED(rv)) return rv;
 
-            if ((rv == NS_CONTENT_ATTR_HAS_VALUE) && (attrValue.Length() > 0)) {
+            if ((rv == NS_CONTENT_ATTR_HAS_VALUE) && (!attrValue.IsEmpty())) {
                 nsAutoString value;
                 rv = SubstituteText(*aMatch, attrValue, value);
                 if (NS_FAILED(rv)) return rv;
@@ -890,7 +880,7 @@ nsXULContentBuilder::AddPersistentAttributes(nsIContent* aTemplateNode,
         return NS_OK;
 
     nsAutoString attribute;
-    while (persist.Length() > 0) {
+    while (!persist.IsEmpty()) {
         attribute.Truncate();
 
         PRInt32 offset = persist.FindCharInSet(" ,");
@@ -905,7 +895,7 @@ nsXULContentBuilder::AddPersistentAttributes(nsIContent* aTemplateNode,
 
         attribute.Trim(" ");
 
-        if (attribute.Length() == 0)
+        if (attribute.IsEmpty())
             break;
 
         PRInt32 nameSpaceID;
@@ -989,7 +979,7 @@ nsXULContentBuilder::SynchronizeUsingTemplate(nsIContent* aTemplateNode,
             nsAutoString newvalue;
             SubstituteText(aMatch, attribValue, newvalue);
 
-            if (newvalue.Length() > 0) {
+            if (!newvalue.IsEmpty()) {
                 aRealElement->SetAttr(attribNameSpaceID,
                                       attribName,
                                       newvalue,
@@ -1370,7 +1360,7 @@ nsXULContentBuilder::CreateTemplateContents(nsIContent* aElement,
     nsCOMPtr<nsIRDFResource> resource;
 
     nsCOMPtr<nsIContent> element = aElement;
-    while (element) {
+    while (1) {
         nsXULContentUtils::GetElementRefResource(element, getter_AddRefs(resource));
         if (resource)
             break;
@@ -1379,11 +1369,16 @@ nsXULContentBuilder::CreateTemplateContents(nsIContent* aElement,
         element->GetParent(*getter_AddRefs(parent));
 
         element = parent;
+
+        NS_ASSERTION(element != nsnull,
+                     "walked to the root of the content model without finding template root");
     }
 
-    nsTemplateMatch* match;
+    nsTemplateMatch* match = nsnull;
     mContentSupportMap.Get(element, &match);
 
+    NS_ASSERTION(match != nsnull, "no match in the content support map");
+
     rv = BuildContentFromTemplate(aTemplateElement, aElement, aElement, PR_FALSE, resource, PR_FALSE,
                                   match, aContainer, aNewIndexInContainer);
 
@@ -1543,10 +1538,9 @@ nsXULContentBuilder::IsLazyWidgetItem(nsIContent* aElement)
     if (nameSpaceID != kNameSpaceID_XUL)
         return PR_FALSE;
 
-    if ((tag.get() == nsXULAtoms::tree) || (tag.get() == nsXULAtoms::treeitem) ||
-        (tag.get() == nsXULAtoms::menu) || (tag.get() == nsXULAtoms::menulist) ||
+    if ((tag.get() == nsXULAtoms::menu) || (tag.get() == nsXULAtoms::menulist) ||
         (tag.get() == nsXULAtoms::menubutton) || (tag.get() == nsXULAtoms::toolbarbutton) ||
-        (tag.get() == nsXULAtoms::button) || (tag == nsXULAtoms::outlineritem))
+        (tag.get() == nsXULAtoms::button) || (tag == nsXULAtoms::treeitem))
         return PR_TRUE;
 
     return PR_FALSE;
@@ -1599,7 +1593,7 @@ nsXULContentBuilder::CreateElement(PRInt32 aNameSpaceID,
         rv = nsXULElement::Create(nodeInfo, getter_AddRefs(result));
         if (NS_FAILED(rv)) return rv;
     }
-    else if (aNameSpaceID == kNameSpaceID_HTML) {
+    else if (aNameSpaceID == kNameSpaceID_XHTML) {
         rv = gHTMLElementFactory->CreateInstanceByTag(nodeInfo,
                                                       getter_AddRefs(result));
         if (NS_FAILED(rv)) return rv;
@@ -1646,7 +1640,7 @@ nsXULContentBuilder::SetContainerAttrs(nsIContent *aElement, const nsTemplateMat
     NS_NAMED_LITERAL_STRING(true_, "true");
     NS_NAMED_LITERAL_STRING(false_, "false");
     
-    const nsAReadableString& newcontainer =
+    const nsAString& newcontainer =
         iscontainer ? true_ : false_;
 
     if (oldcontainer != newcontainer) {
@@ -1658,7 +1652,7 @@ nsXULContentBuilder::SetContainerAttrs(nsIContent *aElement, const nsTemplateMat
         nsAutoString oldempty;
         aElement->GetAttr(kNameSpaceID_None, nsXULAtoms::empty, oldempty);
 
-        const nsAReadableString& newempty =
+        const nsAString& newempty =
             (iscontainer && isempty) ? true_ : false_;
 
         if (oldempty != newempty) {
@@ -1758,24 +1752,6 @@ nsXULContentBuilder::Rebuild()
     return Rebuild(mRoot);
 }
 
-//----------------------------------------------------------------------
-//
-// nsIRDFContentModelBuilder methods
-//
-
-NS_IMETHODIMP
-nsXULContentBuilder::SetRootContent(nsIContent* aElement)
-{
-    NS_PRECONDITION(aElement, "null ptr");
-    if (! aElement)
-        return NS_ERROR_NULL_POINTER;
-
-    mRoot = aElement;
-
-    return LoadDataSources();
-}
-
-
 NS_IMETHODIMP
 nsXULContentBuilder::CreateContents(nsIContent* aElement)
 {
@@ -2043,51 +2019,6 @@ nsXULContentBuilder::CloseContainer(nsIContent* aElement)
     nsCOMPtr<nsIAtom> tag;
     aElement->GetTag(*getter_AddRefs(tag));
 
-    if (tag.get() == nsXULAtoms::treeitem) {
-        // Find the tag that contains the children so that we can
-        // remove all of the children. This is a -total- hack, that is
-        // necessary for the tree control because...I'm not sure. But
-        // it's necessary. Maybe we should fix the tree control to
-        // reflow itself when the 'open' attribute changes on a
-        // treeitem.
-        //
-        // OTOH, we could treat this as a (premature?) optimization so
-        // that nodes which are not being displayed don't hang around
-        // taking up space. Unfortunately, the tree widget currently
-        // _relies_ on this behavior and will break if we don't do it
-        // :-(.
-
-        // Find the <treechildren> beneath the <treeitem>...
-        nsCOMPtr<nsIContent> insertionpoint;
-        nsXULContentUtils::FindChildByTag(aElement, kNameSpaceID_XUL,
-                                          nsXULAtoms::treechildren,
-                                          getter_AddRefs(insertionpoint));
-
-        if (insertionpoint) {
-            // ...and blow away all the generated content.
-            RemoveGeneratedContent(insertionpoint);
-        }
-
-        // Force the XUL element to remember that it needs to re-generate
-        // its kids next time around.
-        nsCOMPtr<nsIXULContent> xulcontent = do_QueryInterface(aElement);
-        NS_ASSERTION(xulcontent != nsnull, "not an nsIXULContent");
-        if (! xulcontent)
-            return NS_ERROR_UNEXPECTED;
-
-        xulcontent->SetLazyState(nsIXULContent::eChildrenMustBeRebuilt);
-
-        // Clear the contents-generated attribute so that the next time we
-        // come back, we'll regenerate the kids we just killed.
-        xulcontent->ClearLazyState(nsIXULContent::eContainerContentsBuilt);
-
-        // Remove any instantiations involving this element from the
-        // conflict set.
-        nsTemplateMatchSet firings(mConflictSet.GetPool());
-        nsTemplateMatchSet retractions(mConflictSet.GetPool());
-        mConflictSet.Remove(nsContentTestNode::Element(aElement), firings, retractions);
-    }
-
     return NS_OK;
 }
 
@@ -2204,7 +2135,7 @@ nsXULContentBuilder::CompileContentCondition(nsTemplateRule* aRule,
 
     PRInt32 urivar = mRules.LookupSymbol(uri.get());
     if (! urivar) {
-        if (! mContainerSymbol.Length()) {
+        if (mContainerSymbol.IsEmpty()) {
             // If the container symbol was not explictly declared on
             // the <template> tag, or we haven't seen a previous rule
             // whose <content> condition defined it, then we'll
@@ -2224,7 +2155,7 @@ nsXULContentBuilder::CompileContentCondition(nsTemplateRule* aRule,
     nsAutoString tagstr;
     aCondition->GetAttr(kNameSpaceID_None, nsXULAtoms::tag, tagstr);
 
-    if (tagstr.Length()) {
+    if (!tagstr.IsEmpty()) {
         tag = dont_AddRef(NS_NewAtom(tagstr));
     }
 
@@ -2260,7 +2191,7 @@ nsXULContentBuilder::CompileContentCondition(nsTemplateRule* aRule,
 PRBool
 nsXULContentBuilder::CompileSimpleAttributeCondition(PRInt32 aNameSpaceID,
                                                      nsIAtom* aAttribute,
-                                                     const nsAReadableString& aValue,
+                                                     const nsAString& aValue,
                                                      InnerNode* aParentNode,
                                                      TestNode** aResult)
 {
