@@ -54,9 +54,6 @@
 const char kConsoleServiceContractId[] = "@mozilla.org/consoleservice;1";
 const char kDNSServiceContractId[] = "@mozilla.org/network/dns-service;1";
 
-extern "C" int nsLDAPThreadDataInit(void);
-extern "C" int nsLDAPThreadFuncsInit(LDAP *aLDAP);
-
 // constructor
 //
 nsLDAPConnection::nsLDAPConnection()
@@ -64,6 +61,7 @@ nsLDAPConnection::nsLDAPConnection()
       mBindName(0),
       mPendingOperations(0),
       mRunnable(0),
+      mSSL(PR_FALSE),
       mDNSRequest(0),
       mDNSFinished(PR_FALSE)
 {
@@ -79,23 +77,21 @@ nsLDAPConnection::~nsLDAPConnection()
   PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, ("unbinding\n"));
 
   if (mConnectionHandle) {
-      rc = ldap_unbind_s(mConnectionHandle);
+      // note that the ldap_unbind() call in the 5.0 version of the LDAP C SDK
+      // appears to be exactly identical to ldap_unbind_s(), so it may in fact
+      // still be synchronous
+      //
+      rc = ldap_unbind(mConnectionHandle);
 #ifdef PR_LOGGING
       if (rc != LDAP_SUCCESS) {
-          if (gLDAPLogModule) {
-              PR_LOG(gLDAPLogModule, PR_LOG_WARNING, 
-                     ("nsLDAPConnection::~nsLDAPConnection: %s\n", 
-                      ldap_err2string(rc)));
-          }
+          PR_LOG(gLDAPLogModule, PR_LOG_WARNING, 
+                 ("nsLDAPConnection::~nsLDAPConnection: %s\n", 
+                  ldap_err2string(rc)));
       }
 #endif
   }
 
-#ifdef PR_LOGGING
-  if (gLDAPLogModule) {
-      PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, ("unbound\n"));
-  }
-#endif
+  PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, ("unbound\n"));
 
   if (mBindName) {
       delete mBindName;
@@ -173,10 +169,9 @@ nsLDAPConnection::Release(void)
 
 
 NS_IMETHODIMP
-nsLDAPConnection::Init(const char *aHost, PRInt16 aPort,
-                       const PRUnichar *aBindName,
+nsLDAPConnection::Init(const char *aHost, PRInt16 aPort, PRBool aSSL,
+                       const PRUnichar *aBindName, 
                        nsILDAPMessageListener *aMessageListener)
-
 {
     nsCOMPtr<nsIDNSListener> selfProxy;
     nsresult rv;
@@ -185,17 +180,6 @@ nsLDAPConnection::Init(const char *aHost, PRInt16 aPort,
         return NS_ERROR_ILLEGAL_VALUE;
     }
 
-#ifdef PR_LOGGING
-    // initialize logging, if it hasn't been already
-    //
-    if (!gLDAPLogModule) {
-        gLDAPLogModule = PR_NewLogModule("ldap");
-
-        NS_ABORT_IF_FALSE(gLDAPLogModule, 
-                          "failed to initialize LDAP log module");
-    }
-#endif
-
     // Make sure we haven't called Init earlier, i.e. there's a DNS
     // request pending.
     //
@@ -220,17 +204,14 @@ nsLDAPConnection::Init(const char *aHost, PRInt16 aPort,
     //
     mPort = aPort;
 
+    // Save the SSL flag for later use
+    mSSL = aSSL;
+
     // Save the Init listener reference, we need it when the async
     // DNS resolver has finished.
     //
     mInitListener = aMessageListener;
 
-    // initialize the thread-specific data for the calling thread as necessary
-    //
-    if (!nsLDAPThreadDataInit()) {
-        return NS_ERROR_FAILURE;
-    }
-        
     // allocate a hashtable to keep track of pending operations.
     // 10 buckets seems like a reasonable size, and we do want it to 
     // be threadsafe
@@ -241,7 +222,7 @@ nsLDAPConnection::Init(const char *aHost, PRInt16 aPort,
         return NS_ERROR_FAILURE;
     }
 
-    // Get a proxy object so the callback happens on the main thread.
+    // Get a proxy object so the callback happens on the current thread.
     // This is now a Synchronous proxy, due to the fact that the DNS
     // service hands out data which it later deallocates, and the async
     // proxy makes this unreliable. See bug 102227 for more details.
@@ -307,13 +288,13 @@ nsLDAPConnection::Init(const char *aHost, PRInt16 aPort,
 
 // who we're binding as
 //
-// readonly attribute string bindName
+// readonly attribute wstring bindName
 //
 NS_IMETHODIMP
 nsLDAPConnection::GetBindName(PRUnichar **_retval)
 {
     NS_ENSURE_ARG_POINTER(_retval);
-    
+
     // check for NULL (meaning bind anonymously)
     //
     if (!mBindName) {
@@ -597,80 +578,37 @@ nsLDAPConnectionLoop::Init()
     return NS_OK;
 }
 
-
-// for nsIRunnable.  this thread spins in ldap_result() awaiting the next
-// message.  once one arrives, it dispatches it to the nsILDAPMessageListener 
-// on the main thread.
-//
-// XXX do all returns from this function need to do thread cleanup?
-//
-NS_IMETHODIMP
-nsLDAPConnectionLoop::Run(void)
+// typedef PRBool
+// (*PR_CALLBACK nsHashtableEnumFunc)
+//      (nsHashKey *aKey, void *aData, void* aClosure);
+PRBool PR_CALLBACK
+CheckLDAPOperationResult(nsHashKey *aKey, void *aData, void* aClosure)
 {
     int lderrno;
     nsresult rv;
     PRInt32 returnCode;
     LDAPMessage *msgHandle;
     nsCOMPtr<nsILDAPMessage> msg;
+    PRBool operationFinished = PR_TRUE;
     struct timeval timeout = { 1, 0 }; 
     PRIntervalTime sleepTime = PR_MillisecondsToInterval(40);
 
-    PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, 
-           ("nsLDAPConnection::Run() entered\n"));
+    // we need to access some of the connection loop's objects
+    //
+    nsLDAPConnectionLoop *loop = 
+        NS_STATIC_CAST(nsLDAPConnectionLoop *, aClosure);
 
     // get the console service so we can log messages
     //
     nsCOMPtr<nsIConsoleService> consoleSvc = 
         do_GetService(kConsoleServiceContractId, &rv);
     if (NS_FAILED(rv)) {
-        NS_ERROR("nsLDAPConnection::Run() couldn't get console service");
+        NS_ERROR("CheckLDAPOperationResult() couldn't get console service");
         return NS_ERROR_FAILURE;
     }
 
-    // initialize the thread-specific data for the child thread (as necessary)
-    //
-    if (!nsLDAPThreadDataInit()) {
-        NS_ERROR("nsLDAPConnection::Run() couldn't initialize "
-                   "thread-specific data");
-        return NS_ERROR_FAILURE;
-    }
-
-    // wait for results
-    //
-    while(1) {
-        PRBool operationFinished = PR_TRUE;
-        nsCOMPtr<nsILDAPConnection> conn;
-
-        // Exit this thread if we no longer have an nsLDAPConnection
-        // associcated with it. We also aquire a lock here, to make sure
-        // to avoid a possible race condition when the nsLDAPConnection
-        // is destructed during the call to do_QueryReferent() (since
-        // that function isn't MT safe).
-        //
-        PR_Lock(mLock);
-        conn = do_QueryReferent(mWeakConn);
-        PR_Unlock(mLock);
-
-        if (!conn) {
-            mWeakConn = 0;
-            return NS_OK;
-        }
-
-        nsLDAPConnection *rawConn = NS_STATIC_CAST(nsLDAPConnection *,
-                             NS_STATIC_CAST(nsILDAPConnection *, conn));
-
-        // in case something went wrong on the last iteration, be sure to 
-        // cause nsCOMPtr to release the message before going to sleep in
-        // ldap_result
-        //
-        msg = 0;
-
-        // XXX deal with timeouts better
-        //
-        NS_ASSERTION(rawConn->mConnectionHandle, "nsLDAPConnection::Run(): "
-                     "no connection created.\n");
-        returnCode = ldap_result(rawConn->mConnectionHandle,
-                                 LDAP_RES_ANY, LDAP_MSG_ONE,
+    returnCode = ldap_result(loop->mRawConn->mConnectionHandle,
+                             aKey->HashCode(), LDAP_MSG_ONE,
                                  &timeout, &msgHandle);
 
         // if we didn't error or timeout, create an nsILDAPMessage
@@ -682,20 +620,18 @@ nsLDAPConnectionLoop::Run(void)
             // the connection may not exist yet.  sleep for a while
             // and try again
             //
-            PR_LOG(gLDAPLogModule, PR_LOG_WARNING, 
-                   ("ldap_result() timed out.\n"));
-            conn = 0;
+        PR_LOG(gLDAPLogModule, PR_LOG_WARNING, ("ldap_result() timed out.\n"));
 
             // The sleep here is to avoid a problem where the LDAP
             // Connection/thread isn't ready quite yet, and we want to
             // avoid a very busy loop.
             //
             PR_Sleep(sleepTime);
-            continue;
+        return PR_TRUE;
 
         case -1: // something went wrong 
 
-            lderrno = ldap_get_lderrno(rawConn->mConnectionHandle, 0, 0);
+        lderrno = ldap_get_lderrno(loop->mRawConn->mConnectionHandle, 0, 0);
 
             // Sleep briefly, to avoid a very busy loop again.
             //
@@ -710,31 +646,42 @@ nsLDAPConnectionLoop::Run(void)
                 // the nsLDAPConnection to detect the error, and then
                 // create a new connection.
                 //
+                PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, 
+                       ("CheckLDAPOperationResult(): ldap_result returned" 
+                        " LDAP_SERVER_DOWN"));
                 break;
 
             case LDAP_DECODING_ERROR:
                 consoleSvc->LogStringMessage(
                     NS_LITERAL_STRING("LDAP: WARNING: decoding error; possible corrupt data received").get());
-                NS_WARNING("nsLDAPConnection::Run(): ldaperrno = "
+                NS_WARNING("CheckLDAPOperationResult(): ldaperrno = "
                            "LDAP_DECODING_ERROR after ldap_result()");
                 break;
 
             case LDAP_NO_MEMORY:
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: ERROR: couldn't allocate memory while getting async operation result").get());
+                NS_ERROR("CheckLDAPOperationResult(): Couldn't allocate memory"
+                         " while getting async operation result");
                 // punt and hope things work out better next time around
                 break;
 
-            default:
-                // shouldn't happen; internal error
+            case LDAP_PARAM_ERROR:
+                // I think it's possible to hit a race condition where we're
+                // continuing to poll for a result after the C SDK connection
+                // has removed the operation because the connection has gone
+                // dead.  In theory we should fix this.  Practically, it's
+                // unclear to me whether it matters.
                 //
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: DEBUG: ldaperrno set to unexpected value after ldap_result() call in nsLDAPConnection::Run()").get());
-                NS_WARNING("nsLDAPConnection::Run(): ldaperrno set to "
-                           "unexpected value after ldap_result() "
-                           "call in nsLDAPConnection::Run()");
+                NS_WARNING("CheckLDAPOperationResult(): ldap_result returned"
+                           " LDAP_PARAM_ERROR");
                 break;
 
+            default:
+                NS_ERROR("CheckLDAPOperationResult(): lderrno set to "
+                           "unexpected value after ldap_result() "
+                           "call in nsLDAPConnection::Run()");
+                PR_LOG(gLDAPLogModule, PR_LOG_ERROR, 
+                       ("lderrno = 0x%x", lderrno));
+                break;
             }
             break;
 
@@ -752,10 +699,11 @@ nsLDAPConnectionLoop::Run(void)
             // we want nsLDAPMessage specifically, not a compatible, since
             // we're sharing native objects used by the LDAP C SDK
             //
-            nsLDAPMessage *rawMsg = new nsLDAPMessage();
+            nsLDAPMessage *rawMsg;
+            NS_NEWXPCOM(rawMsg, nsLDAPMessage);
             if (!rawMsg) {
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: ERROR: couldn't allocate memory for new LDAP message; search entry dropped").get());
+            NS_ERROR("CheckLDAPOperationResult(): couldn't allocate memory"
+                     " for new LDAP message; search entry dropped");
                 // punt and hope things work out better next time around
                 break;
             }
@@ -763,7 +711,7 @@ nsLDAPConnectionLoop::Run(void)
             // initialize the message, using a protected method not available
             // through nsILDAPMessage (which is why we need the raw pointer)
             //
-            rv = rawMsg->Init(conn, msgHandle);
+        rv = rawMsg->Init(loop->mRawConn, msgHandle);
 
             switch (rv) {
 
@@ -773,28 +721,24 @@ nsLDAPConnectionLoop::Run(void)
             case NS_ERROR_LDAP_DECODING_ERROR:
                 consoleSvc->LogStringMessage(
                     NS_LITERAL_STRING("LDAP: WARNING: decoding error; possible corrupt data received").get());
-                NS_WARNING("nsLDAPConnection::Run(): ldaperrno = "
+            NS_WARNING("CheckLDAPOperationResult(): ldaperrno = "
                            "LDAP_DECODING_ERROR after ldap_result()");
-                continue;
+            return PR_TRUE;
 
             case NS_ERROR_OUT_OF_MEMORY:
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: ERROR: couldn't allocate memory for new LDAP message; search entry dropped").get());
                 // punt and hope things work out better next time around
-                continue;
+            return PR_TRUE;
 
             case NS_ERROR_ILLEGAL_VALUE:
             case NS_ERROR_UNEXPECTED:
             default:
                 // shouldn't happen; internal error
                 //
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: DEBUG: nsLDAPConnection::Run(): nsLDAPMessage::Init() returned unexpected value").get());
-                NS_WARNING("nsLDAPConnection::Run(): nsLDAPMessage::Init() "
+            NS_ERROR("CheckLDAPOperationResult(): nsLDAPMessage::Init() "
                            "returned unexpected value.");
 
                 // punt and hope things work out better next time around
-                continue;
+            return PR_TRUE;
             }
 
             // now let the scoping mechanisms provided by nsCOMPtr manage
@@ -805,26 +749,87 @@ nsLDAPConnectionLoop::Run(void)
             // invoke the callback on the nsILDAPOperation corresponding to 
             // this message
             //
-            rv = rawConn->InvokeMessageCallback(msgHandle, msg,
+        rv = loop->mRawConn->InvokeMessageCallback(msgHandle, msg, 
                                                     operationFinished);
             if (NS_FAILED(rv)) {
-                consoleSvc->LogStringMessage(
-                    NS_LITERAL_STRING("LDAP: ERROR: problem invoking message callback").get());
-                NS_ERROR("LDAP: ERROR: problem invoking message callback");
+            NS_ERROR("CheckLDAPOperationResult(): error invoking message"
+                     " callback");
                 // punt and hope things work out better next time around
-                continue;
+            return PR_TRUE;
             }
 
-#if 0
-            // sleep for a while to workaround event queue flooding 
-            // (bug 50104) so that it's possible to test cancelling, firing
-            // status, etc.
-            //
-            PR_Sleep(1000);
-#endif
             break;
         }       
 
+    return PR_TRUE;
+}
+
+// for nsIRunnable.  this thread spins in ldap_result() awaiting the next
+// message.  once one arrives, it dispatches it to the nsILDAPMessageListener 
+// on the main thread.
+//
+// XXX do all returns from this function need to do thread cleanup?
+//
+NS_IMETHODIMP
+nsLDAPConnectionLoop::Run(void)
+{
+    PR_LOG(gLDAPLogModule, PR_LOG_DEBUG, 
+           ("nsLDAPConnection::Run() entered\n"));
+
+    // wait for results
+    //
+    while(1) {
+
+        // Exit this thread if we no longer have an nsLDAPConnection
+        // associated with it. We also aquire a lock here, to make sure
+        // to avoid a possible race condition when the nsLDAPConnection
+        // is destructed during the call to do_QueryReferent() (since that
+        // function isn't MT safe).
+        //
+        nsresult rv;
+
+        PR_Lock(mLock);
+        nsCOMPtr<nsILDAPConnection> strongConn = 
+            do_QueryReferent(mWeakConn, &rv);
+        PR_Unlock(mLock);
+
+        if (NS_FAILED(rv)) {
+            mWeakConn = 0;
+            return NS_OK;
+        }
+        // we use a raw connection because we need to call non-interface
+        // methods
+        mRawConn = NS_STATIC_CAST(nsLDAPConnection *, 
+                                  NS_STATIC_CAST(nsILDAPConnection *, 
+                                                 strongConn.get()));
+
+        // XXX deal with timeouts better
+        //
+        NS_ASSERTION(mRawConn->mConnectionHandle, "nsLDAPConnection::Run(): "
+                     "no connection created.\n");
+
+        // We can't enumerate over mPendingOperations itself, because the
+        // callback needs to modify mPendingOperations.  So we clone it first,
+        // and enumerate over the clone.  It kinda sucks that we need to do
+        // this everytime we poll, but the hashtable will pretty much always
+        // be small.
+        //
+        // only clone if the number of pending operations is non-zero
+        // otherwise, put the LDAP connection thread to sleep (briefly)
+        // until there is pending operations..
+        if (mRawConn->mPendingOperations->Count()) {
+          nsHashtable *hashtableCopy = mRawConn->mPendingOperations->Clone();
+          if (hashtableCopy) {
+            hashtableCopy->Enumerate(CheckLDAPOperationResult, this);
+            delete hashtableCopy;
+          } else {
+            // punt and hope it works next time around
+            NS_ERROR("nsLDAPConnectionLoop::Run() error cloning hashtable");
+          }
+        }
+        else {
+          PR_Sleep(PR_MillisecondsToInterval(40));
+        }
     }
 
     // This will never happen, but here just in case.
@@ -894,7 +899,7 @@ nsLDAPConnection::OnFound(nsISupports *aContext,
             // list of IPs.
             //
             PR_NetAddrToString(&netAddress, addrbuf, sizeof(addrbuf));
-            if ((addrbuf[0] == ':') && (nsCRT::strlen(addrbuf) > 7))
+            if ((addrbuf[0] == ':') && (strlen(addrbuf) > 7))
                 mResolvedIP.Append(addrbuf+7);
             else
                 mResolvedIP.Append(addrbuf);
@@ -959,23 +964,40 @@ nsLDAPConnection::OnStopLookup(nsISupports *aContext,
         //
         mConnectionHandle = ldap_init(mResolvedIP.get(),
                                       mPort == -1 ? LDAP_PORT : mPort);
-
         // Check that we got a proper connection, and if so, setup the
         // threading functions for this connection.
         //
         if ( !mConnectionHandle ) {
             rv = NS_ERROR_FAILURE;  // LDAP C SDK API gives no useful error
-        } else if (!nsLDAPThreadFuncsInit(mConnectionHandle)) {
-            rv = NS_ERROR_UNEXPECTED;
         } else {
 #ifdef DEBUG_dmose
             const int lDebug = 0;
             ldap_set_option(mConnectionHandle, LDAP_OPT_DEBUG_LEVEL, &lDebug);
-            ldap_set_option(mConnectionHandle, LDAP_OPT_ASYNC_CONNECT, 
-                            NS_REINTERPRET_CAST(void *, 0));
 #endif
         }
 
+#ifdef MOZ_PSM
+        // This code sets up the current connection to use PSM for SSL
+        // functionality.  Making this use libssldap instead for
+        // non-browser user shouldn't be hard.
+
+        extern nsresult nsLDAPInstallSSL(LDAP *ld, const char *aHostName);
+
+        if (mSSL) {
+            if (ldap_set_option(mConnectionHandle, LDAP_OPT_SSL, LDAP_OPT_ON)
+                != LDAP_SUCCESS ) {
+                NS_ERROR("nsLDAPConnection::OnStopLookup(): Error configuring"
+                         " connection to use SSL");
+                rv = NS_ERROR_UNEXPECTED;
+            }
+
+            rv = nsLDAPInstallSSL(mConnectionHandle, aHostName);
+            if (NS_FAILED(rv)) {
+                NS_ERROR("nsLDAPConnection::OnStopLookup(): Error installing"
+                         " secure LDAP routines for connection");
+            }
+        }
+#endif
         // Create a new runnable object, and increment the refcnt. The
         // thread will also hold a strong ref to the runnable, but we need
         // to make sure it doesn't get destructed until we are done with
