@@ -110,6 +110,7 @@ typedef unsigned long HMTX;
 #include "nsILayoutHistoryState.h"
 #include "nsTextFormatter.h"
 #include "nsPIDOMWindow.h"
+#include "nsPICommandUpdater.h"
 #include "nsIController.h"
 #include "nsIFocusController.h"
 #include "nsGUIEvent.h"
@@ -228,6 +229,27 @@ void nsWebShell::InitFrameData()
   SetMarginHeight(-1);
 }
 
+nsresult
+nsWebShell::EnsureCommandHandler()
+{
+  if (!mCommandManager)
+  {
+    mCommandManager = do_CreateInstance("@mozilla.org/embedcomp/command-manager;1");
+    if (!mCommandManager) return NS_ERROR_OUT_OF_MEMORY;
+    
+    nsCOMPtr<nsPICommandUpdater>       commandUpdater = do_QueryInterface(mCommandManager);
+    if (!commandUpdater) return NS_ERROR_FAILURE;
+    
+    nsCOMPtr<nsIDOMWindow> domWindow = do_GetInterface(NS_STATIC_CAST(nsIInterfaceRequestor *, this));
+    nsresult rv = commandUpdater->Init(domWindow);
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Initting command manager failed");
+  }
+  
+  return mCommandManager ? NS_OK : NS_ERROR_FAILURE;
+}
+
+
+
 NS_IMPL_ADDREF_INHERITED(nsWebShell, nsDocShell)
 NS_IMPL_RELEASE_INHERITED(nsWebShell, nsDocShell)
 
@@ -274,6 +296,13 @@ nsWebShell::GetInterface(const nsIID &aIID, void** aInstancePtr)
                         NS_ERROR_FAILURE);
       return NS_OK;
       }
+   else if(aIID.Equals(NS_GET_IID(nsICommandManager)))
+      {
+      NS_ENSURE_SUCCESS(EnsureCommandHandler(), NS_ERROR_FAILURE);
+      NS_ENSURE_SUCCESS(mCommandManager->QueryInterface(NS_GET_IID(nsICommandManager),
+         aInstancePtr), NS_ERROR_FAILURE);
+      return NS_OK;
+      }
 
    if (!*aInstancePtr || NS_FAILED(rv))
      return nsDocShell::GetInterface(aIID,aInstancePtr);
@@ -426,17 +455,18 @@ struct OnLinkClickEvent : public PLEvent {
   ~OnLinkClickEvent();
 
   void HandleEvent() {
-    mHandler->HandleLinkClickEvent(mContent, mVerb, mURLSpec->get(),
-                                   mTargetSpec->get(), mPostDataStream,
-                                   mHeadersDataStream);
+    mHandler->OnLinkClickSync(mContent, mVerb, mURLSpec.get(),
+                              mTargetSpec.get(), mPostDataStream,
+                              mHeadersDataStream,
+                              nsnull, nsnull);
   }
 
-  nsWebShell*  mHandler;
-  nsString*    mURLSpec;
-  nsString*    mTargetSpec;
-  nsIInputStream* mPostDataStream;
-  nsIInputStream* mHeadersDataStream;
-  nsIContent*     mContent;
+  nsWebShell*     mHandler;
+  nsString       mURLSpec;
+  nsString       mTargetSpec;
+  nsCOMPtr<nsIInputStream> mPostDataStream;
+  nsCOMPtr<nsIInputStream> mHeadersDataStream;
+  nsCOMPtr<nsIContent>     mContent;
   nsLinkVerb      mVerb;
 };
 
@@ -460,14 +490,11 @@ OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
 {
   mHandler = aHandler;
   NS_ADDREF(aHandler);
-  mURLSpec = new nsString(aURLSpec);
-  mTargetSpec = new nsString(aTargetSpec);
+  mURLSpec.Assign(aURLSpec);
+  mTargetSpec.Assign(aTargetSpec);
   mPostDataStream = aPostDataStream;
-  NS_IF_ADDREF(mPostDataStream);
   mHeadersDataStream = aHeadersDataStream;
-  NS_IF_ADDREF(mHeadersDataStream);
   mContent = aContent;
-  NS_IF_ADDREF(mContent);
   mVerb = aVerb;
 
   PL_InitEvent(this, nsnull,
@@ -483,13 +510,7 @@ OnLinkClickEvent::OnLinkClickEvent(nsWebShell* aHandler,
 
 OnLinkClickEvent::~OnLinkClickEvent()
 {
-  NS_IF_RELEASE(mContent);
   NS_IF_RELEASE(mHandler);
-  NS_IF_RELEASE(mPostDataStream);
-  NS_IF_RELEASE(mHeadersDataStream);
-  if (nsnull != mURLSpec) delete mURLSpec;
-  if (nsnull != mTargetSpec) delete mTargetSpec;
-
 }
 
 //----------------------------------------
@@ -525,17 +546,27 @@ nsWebShell::GetEventQueue(nsIEventQueue **aQueue)
   return *aQueue ? NS_OK : NS_ERROR_FAILURE;
 }
 
-void
-nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
-                                 nsLinkVerb aVerb,
-                                 const PRUnichar* aURLSpec,
-                                 const PRUnichar* aTargetSpec,
-                                 nsIInputStream* aPostDataStream,
-                                 nsIInputStream* aHeadersDataStream)
+NS_IMETHODIMP
+nsWebShell::OnLinkClickSync(nsIContent *aContent,
+                            nsLinkVerb aVerb,
+                            const PRUnichar* aURLSpec,
+                            const PRUnichar* aTargetSpec,
+                            nsIInputStream* aPostDataStream,
+                            nsIInputStream* aHeadersDataStream,
+                            nsIDocShell** aDocShell,
+                            nsIRequest** aRequest)
 {
   nsresult rv;
   nsAutoString target(aTargetSpec);
 
+  // Initialize the DocShell / Request
+  if (aDocShell) {
+    *aDocShell = nsnull;
+  }
+  if (aRequest) {
+    *aRequest = nsnull;
+  }
+
   switch(aVerb) {
     case eLinkVerb_New:
       target.Assign(NS_LITERAL_STRING("_blank"));
@@ -556,32 +587,35 @@ nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
         // throwing the click away.
         if (!uri && NS_SUCCEEDED(EnsureContentListener()))
         {
-            nsCOMPtr<nsIURIContentListener> listener = do_QueryInterface(mContentListener);
-            nsCAutoString spec; spec.AssignWithConversion(aURLSpec);
+            NS_ConvertUCS2toUTF8 spec(aURLSpec);
             PRBool abort = PR_FALSE;
             uri = do_CreateInstance(kSimpleURICID, &rv);
             NS_ASSERTION(NS_SUCCEEDED(rv), "can't create simple uri");
             if (NS_SUCCEEDED(rv))
             {
-                rv = uri->SetSpec(spec.get());
+                rv = uri->SetSpec(spec);
                 NS_ASSERTION(NS_SUCCEEDED(rv), "spec is invalid");
                 if (NS_SUCCEEDED(rv))
                 {
-                    listener->OnStartURIOpen(uri, &abort);
+                    mContentListener->OnStartURIOpen(uri, &abort);
                 }
             }
-            return;
+            // We didn't load the URI, so we failed
+            return NS_ERROR_FAILURE;
         }
 
-        rv = InternalLoad(uri,                // New URI
-                          mCurrentURI,        // Referer URI
-                          nsnull,             // No onwer
-                          PR_TRUE,            // Inherit owner from document
-                          target.get(),       // Window target
-                          aPostDataStream,    // Post data stream
-                          aHeadersDataStream, // Headers stream
-                          LOAD_LINK,          // Load type
-                          nsnull);            // No SHEntry
+        return InternalLoad(uri,                // New URI
+                            mCurrentURI,        // Referer URI
+                            nsnull,             // No onwer
+                            PR_TRUE,            // Inherit owner from document
+                            target.get(),       // Window target
+                            aPostDataStream,    // Post data stream
+                            aHeadersDataStream, // Headers stream
+                            LOAD_LINK,          // Load type
+                            nsnull,             // No SHEntry
+                            PR_TRUE,            // first party site
+                            aDocShell,          // DocShell out-param
+                            aRequest);          // Request out-param
       }
       break;
     case eLinkVerb_Embed:
@@ -589,6 +623,7 @@ nsWebShell::HandleLinkClickEvent(nsIContent *aContent,
       //          in NS 4.x
     default:
       NS_ABORT_IF_FALSE(0,"unexpected link verb");
+      return NS_ERROR_UNEXPECTED;
   }
 }
 
@@ -682,6 +717,15 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
   nsCOMPtr<nsIWebShell> kungFuDeathGrip(this);
   nsDocShell::EndPageLoad(aProgress, channel, aStatus);
 
+  // Test if this is the top frame or a subframe
+  PRBool isTopFrame = PR_TRUE;
+  nsCOMPtr<nsIDocShellTreeItem> targetParentTreeItem;
+  rv = GetSameTypeParent(getter_AddRefs(targetParentTreeItem));
+  if (NS_SUCCEEDED(rv) && targetParentTreeItem) 
+  {
+    isTopFrame = PR_FALSE;
+  }
+
   //
   // If the page load failed, then deal with the error condition...
   // Errors are handled as follows:
@@ -694,26 +738,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 
   if(url && NS_FAILED(aStatus)) {
     if (aStatus == NS_ERROR_FILE_NOT_FOUND) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-      GetPromptAndStringBundle(getter_AddRefs(prompter), 
-                                getter_AddRefs(stringBundle));
-       if (stringBundle && prompter) {
-        nsXPIDLString messageStr;
-        nsresult rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("fileNotFound").get(), 
-                                                      getter_Copies(messageStr));
-          
-        if (NS_SUCCEEDED(rv) && messageStr) {
-          nsXPIDLCString spec;
-          url->GetPath(getter_Copies(spec));
-
-          PRUnichar *msg = nsTextFormatter::smprintf(messageStr, (const char*)spec);
-          if (!msg) return NS_ERROR_OUT_OF_MEMORY;
-          
-          prompter->Alert(nsnull, msg);
-          nsTextFormatter::smprintf_free(msg);
-        }
-      }
+      DisplayLoadError(aStatus, url, nsnull);
       return NS_OK;
     }  
 
@@ -730,8 +755,8 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       //
       nsCOMPtr<nsIURI> newURI;
 
-      nsXPIDLCString oldSpec;
-      url->GetSpec(getter_Copies(oldSpec));
+      nsCAutoString oldSpec;
+      url->GetSpec(oldSpec);
       NS_ConvertUTF8toUCS2 oldSpecW(oldSpec);
       
       //
@@ -739,28 +764,75 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
       //
       if (aStatus == NS_ERROR_UNKNOWN_HOST ||
           aStatus == NS_ERROR_CONNECTION_REFUSED ||
-          aStatus == NS_ERROR_NET_TIMEOUT)
+          aStatus == NS_ERROR_NET_TIMEOUT ||
+          aStatus == NS_ERROR_NET_RESET)
       {
-        mURIFixup->CreateFixupURI(oldSpecW.get(),
-            nsIURIFixup::FIXUP_FLAG_ALLOW_KEYWORD_LOOKUP, getter_AddRefs(newURI));
+        PRBool keywordsEnabled = PR_FALSE;
+
+        if(mPrefs) {
+          rv = mPrefs->GetBoolPref("keyword.enabled", &keywordsEnabled);
+          if (NS_FAILED(rv)) return rv;
+        }
+
+        nsCAutoString host;
+        url->GetHost(host);
+
+        nsCAutoString scheme;
+        url->GetScheme(scheme);
+
+        PRInt32 dotLoc = host.FindChar('.');
+
+        // we should only perform a keyword search under the following conditions:
+        // (0) Pref keyword.enabled is true
+        // (1) the url scheme is http (or https)
+        // (2) the url does not have a protocol scheme
+        // If we don't enforce such a policy, then we end up doing keyword searchs on urls
+        // we don't intend like imap, file, mailbox, etc. This could lead to a security
+        // problem where we send data to the keyword server that we shouldn't be. 
+        // Someone needs to clean up keywords in general so we can determine on a per url basis
+        // if we want keywords enabled...this is just a bandaid...
+        if (keywordsEnabled && !scheme.IsEmpty() &&
+           (scheme.Find("http") != 0)) {
+            keywordsEnabled = PR_FALSE;
+        }
+
+        if(keywordsEnabled && (-1 == dotLoc)) {
+          // only send non-qualified hosts to the keyword server
+          nsAutoString keywordSpec; keywordSpec.AssignWithConversion("keyword:");
+          keywordSpec.Append(NS_ConvertUTF8toUCS2(host));
+
+          NS_NewURI(getter_AddRefs(newURI), keywordSpec, nsnull);
+        } // end keywordsEnabled
       }
 
       //
       // Now try change the address, e.g. turn http://foo into http://www.foo.com
       //
-      if (aStatus == NS_ERROR_UNKNOWN_HOST)
+      if (aStatus == NS_ERROR_UNKNOWN_HOST ||
+          aStatus == NS_ERROR_NET_RESET)
       {
-        // Test if keyword lookup produced a new URI or not
         PRBool doCreateAlternate = PR_TRUE;
-        if (newURI)
+        
+        // Skip fixup for anything except a normal document load operation on
+        // the topframe.
+        
+        if (mLoadType != LOAD_NORMAL || !isTopFrame)
         {
-          PRBool sameURI = PR_FALSE;
-          url->Equals(newURI, &sameURI);
-          if (!sameURI)
+          doCreateAlternate = PR_FALSE;
+        }
+        else
+        {
+          // Test if keyword lookup produced a new URI or not
+          if (newURI)
           {
-            // Keyword lookup made a new URI so no need to try an
-            // alternate one.
-            doCreateAlternate = PR_FALSE;
+            PRBool sameURI = PR_FALSE;
+            url->Equals(newURI, &sameURI);
+            if (!sameURI)
+            {
+              // Keyword lookup made a new URI so no need to try an
+              // alternate one.
+              doCreateAlternate = PR_FALSE;
+            }
           }
         }
         if (doCreateAlternate)
@@ -782,12 +854,12 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
         url->Equals(newURI, &sameURI);
         if (!sameURI)
         {
-          nsXPIDLCString newSpec;
-          newURI->GetSpec(getter_Copies(newSpec));
-          nsAutoString newSpecW; newSpecW.AssignWithConversion(newSpec.get());
+          nsCAutoString newSpec;
+          newURI->GetSpec(newSpec);
+          NS_ConvertUTF8toUCS2 newSpecW(newSpec);
 
           // This seems evil, since it is modifying the original URL
-          rv = url->SetSpec(newSpec.get());
+          rv = url->SetSpec(newSpec);
           if (NS_FAILED(rv)) return rv;
 
           return LoadURI(newSpecW.get(),      // URI string
@@ -804,99 +876,19 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
     // It is time to throw an error dialog box, and be done with it...
     //
 
-    // Doc failed to load because the host was not found.
-    if(aStatus == NS_ERROR_UNKNOWN_HOST) {
-      // throw a DNS failure dialog
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-      
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
-      }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("dnsNotFound").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      nsXPIDLCString host;
-      url->GetHost(getter_Copies(host));
-      if (host) {
-        PRUnichar *msg = nsTextFormatter::smprintf(messageStr, host.get());
-        if (!msg) return NS_ERROR_OUT_OF_MEMORY;
-
-        prompter->Alert(nsnull, msg);
-        nsTextFormatter::smprintf_free(msg);
-      }
+    // Errors to be shown only on top-level frames
+    if ((aStatus == NS_ERROR_UNKNOWN_HOST || 
+         aStatus == NS_ERROR_CONNECTION_REFUSED) &&
+            (isTopFrame || mUseErrorPages)) {
+      DisplayLoadError(aStatus, url, nsnull);
     }
-    //
-    // Doc failed to load because we couldn't connect to the server.
-    // throw a connection failure dialog
-    //
-    else if(aStatus == NS_ERROR_CONNECTION_REFUSED) {
-      PRInt32 port = -1;
-
-      rv = url->GetPort(&port);
-      if (NS_FAILED(rv)) return rv;
-
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter), 
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
-      }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("connectionFailure").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      // build up the host:port string.
-      nsXPIDLCString host;
-      url->GetHost(getter_Copies(host));
-      nsCAutoString combo(host);
-      if (port > 0) {
-        combo.Append(':');
-        combo.AppendInt(port);
-      }
-      
-      PRUnichar *msg = nsTextFormatter::smprintf(messageStr, combo.get());
-      if (!msg) return NS_ERROR_OUT_OF_MEMORY;
-
-      prompter->Alert(nsnull, msg);
-      nsTextFormatter::smprintf_free(msg);
+    // Errors to be shown for any frame
+    else if (aStatus == NS_ERROR_NET_TIMEOUT ||
+             aStatus == NS_ERROR_REDIRECT_LOOP ||
+             aStatus == NS_ERROR_UNKNOWN_SOCKET_TYPE ||
+             aStatus == NS_ERROR_NET_RESET) {
+      DisplayLoadError(aStatus, url, nsnull);
     }
-    //
-    // Doc failed to load because the socket function timed out.
-    // throw a timeout dialog
-    //
-    else if(aStatus == NS_ERROR_NET_TIMEOUT) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
-      }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("netTimeout").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      nsXPIDLCString host;
-      url->GetHost(getter_Copies(host));
-      PRUnichar *msg = nsTextFormatter::smprintf(messageStr, host.get());
-      if (!msg) return NS_ERROR_OUT_OF_MEMORY;
-
-      prompter->Alert(nsnull, msg);
-      nsTextFormatter::smprintf_free(msg);
-    } // end NS_ERROR_NET_TIMEOUT
     else if (aStatus == NS_ERROR_DOCUMENT_NOT_CACHED) {
       /* A document that was requested to be fetched *only* from
        * the cache is not in cache. May be this is one of those 
@@ -976,48 +968,13 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
                        inputStream,                       // Post data stream
                        nsnull,                            // No headers stream
                        LOAD_RELOAD_BYPASS_PROXY_AND_CACHE,// Load type
-                       nsnull);                           // No SHEntry
-            
+                       nsnull,                            // No SHEntry
+                       PR_TRUE,                           // first party site
+                       nsnull,                            // No nsIDocShell
+                       nsnull);                           // No nsIRequest
           }
         }
     }
-    //
-    // Doc failed to load because the server generated too many redirects
-    //
-    else if (aStatus == NS_ERROR_REDIRECT_LOOP) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
-      }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("redirectLoop").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      prompter->Alert(nsnull, messageStr);
-    }
-    else if (aStatus == NS_ERROR_UNKNOWN_SOCKET_TYPE) {
-      nsCOMPtr<nsIPrompt> prompter;
-      nsCOMPtr<nsIStringBundle> stringBundle;
-
-      rv = GetPromptAndStringBundle(getter_AddRefs(prompter),
-                                    getter_AddRefs(stringBundle));
-      if (!stringBundle) {
-        return rv;
-      }
-
-      nsXPIDLString messageStr;
-      rv = stringBundle->GetStringFromName(NS_LITERAL_STRING("unknownSocketType").get(),
-                                           getter_Copies(messageStr));
-      if (NS_FAILED(rv)) return rv;
-
-      prompter->Alert(nsnull, messageStr);
-    }
   } // if we have a host
 
   return NS_OK;
@@ -1032,7 +989,7 @@ nsresult nsWebShell::EndPageLoad(nsIWebProgress *aProgress,
 #endif
 
 nsresult
-nsWebShell::GetControllerForCommand ( const nsAReadableString & inCommand, nsIController** outController )
+nsWebShell::GetControllerForCommand ( const char * inCommand, nsIController** outController )
 {
   NS_ENSURE_ARG_POINTER(outController);
   *outController = nsnull;
@@ -1053,7 +1010,7 @@ nsWebShell::GetControllerForCommand ( const nsAReadableString & inCommand, nsICo
 
 
 nsresult
-nsWebShell::IsCommandEnabled ( const nsAReadableString & inCommand, PRBool* outEnabled )
+nsWebShell::IsCommandEnabled ( const char * inCommand, PRBool* outEnabled )
 {
   NS_ENSURE_ARG_POINTER(outEnabled);
   *outEnabled = PR_FALSE;
@@ -1070,7 +1027,7 @@ nsWebShell::IsCommandEnabled ( const nsAReadableString & inCommand, PRBool* outE
 
 
 nsresult
-nsWebShell::DoCommand ( const nsAReadableString & inCommand )
+nsWebShell::DoCommand ( const char * inCommand )
 {
   nsresult rv = NS_ERROR_FAILURE;
   
@@ -1086,81 +1043,81 @@ nsWebShell::DoCommand ( const nsAReadableString & inCommand )
 NS_IMETHODIMP
 nsWebShell::CanCutSelection(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_cut"), aResult );
+  return IsCommandEnabled ( "cmd_cut", aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CanCopySelection(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_copy"), aResult );
+  return IsCommandEnabled ( "cmd_copy", aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CanCopyLinkLocation(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_copyLink"), aResult );
+  return IsCommandEnabled ( "cmd_copyLink", aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CanCopyImageLocation(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_copyImageLocation"),
+  return IsCommandEnabled ( "cmd_copyImageLocation",
                             aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CanCopyImageContents(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_copyImageContents"),
+  return IsCommandEnabled ( "cmd_copyImageContents",
                             aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CanPaste(PRBool* aResult)
 {
-  return IsCommandEnabled ( NS_LITERAL_STRING("cmd_paste"), aResult );
+  return IsCommandEnabled ( "cmd_paste", aResult );
 }
 
 NS_IMETHODIMP
 nsWebShell::CutSelection(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_cut") );
+  return DoCommand ( "cmd_cut" );
 }
 
 NS_IMETHODIMP
 nsWebShell::CopySelection(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_copy") );
+  return DoCommand ( "cmd_copy" );
 }
 
 NS_IMETHODIMP
 nsWebShell::CopyLinkLocation(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_copyLink") );
+  return DoCommand ( "cmd_copyLink" );
 }
 
 NS_IMETHODIMP
 nsWebShell::CopyImageLocation(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_copyImageLocation") );
+  return DoCommand ( "cmd_copyImageLocation" );
 }
 
 NS_IMETHODIMP
 nsWebShell::CopyImageContents(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_copyImageContents") );
+  return DoCommand ( "cmd_copyImageContents" );
 }
 
 NS_IMETHODIMP
 nsWebShell::Paste(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_paste") );
+  return DoCommand ( "cmd_paste" );
 }
 
 NS_IMETHODIMP
 nsWebShell::SelectAll(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_selectAll") );
+  return DoCommand ( "cmd_selectAll" );
 }
 
 
@@ -1173,7 +1130,7 @@ nsWebShell::SelectAll(void)
 NS_IMETHODIMP
 nsWebShell::SelectNone(void)
 {
-  return DoCommand ( NS_LITERAL_STRING("cmd_selectNone") );
+  return DoCommand ( "cmd_selectNone" );
 }
 
 
@@ -1208,7 +1165,7 @@ NS_IMETHODIMP nsWebShell::Create()
   // Set the webshell as the default IContentViewerContainer for the loader...
   mDocLoader->SetContainer(shellAsContainer);
 
-   return nsDocShell::Create();
+  return nsDocShell::Create();
 }
 
 NS_IMETHODIMP nsWebShell::Destroy()
