@@ -45,16 +45,56 @@
 #include "jsapi.h"
 #include "nsIScriptSecurityManager.h"
 
+class nsIDOMWindow;
 class nsIDOMNSHTMLOptionCollection;
 class nsIPluginInstance;
 class nsIForm;
 
+struct nsDOMClassInfoData;
+
+typedef nsIClassInfo* (*nsDOMClassInfoConstructorFnc)
+  (nsDOMClassInfoData* aData);
+
+
+struct nsDOMClassInfoData
+{
+  const char *mName;
+  union {
+    nsDOMClassInfoConstructorFnc mConstructorFptr;
+    nsDOMClassInfoExternalConstructorFnc mExternalConstructorFptr;
+  } u;
+  nsIClassInfo *mCachedClassInfo; // low bit is set to 1 if external,
+                                  // so be sure to mask if necessary!
+  const nsIID *mProtoChainInterface;
+  const nsIID **mInterfaces;
+  PRUint32 mScriptableFlags : 31; // flags must not use more than 31 bits!
+  PRBool mHasClassInterface : 1;
+#ifdef NS_DEBUG
+  PRUint32 mDebugID;
+#endif
+};
+
+struct nsExternalDOMClassInfoData : public nsDOMClassInfoData
+{
+  const nsCID *mConstructorCID;
+};
+
+
+typedef unsigned long PtrBits;
+
+// To be used with the nsDOMClassInfoData::mCachedClassInfo pointer.
+// The low bit is set when we created a generic helper for an external
+// (which holds on to the nsDOMClassInfoData).
+#define GET_CLEAN_CI_PTR(_ptr) (nsIClassInfo*)(PtrBits(_ptr) & ~0x1)
+#define MARK_EXTERNAL(_ptr) (nsIClassInfo*)(PtrBits(_ptr) | 0x1)
+#define IS_EXTERNAL(_ptr) (PtrBits(_ptr) & 0x1)
+
 
 class nsDOMClassInfo : public nsIXPCScriptable,
                        public nsIClassInfo
 {
 public:
-  nsDOMClassInfo(nsDOMClassInfoID aID);
+  nsDOMClassInfo(nsDOMClassInfoData* aData);
   virtual ~nsDOMClassInfo();
 
   NS_DECL_NSIXPCSCRIPTABLE
@@ -73,24 +113,27 @@ public:
   // them the right to do that?
 
   static nsIClassInfo* GetClassInfoInstance(nsDOMClassInfoID aID);
+  static nsIClassInfo* GetClassInfoInstance(nsDOMClassInfoData* aData);
 
   static void ShutDown();
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsDOMClassInfo(aID);
+    return new nsDOMClassInfo(aData);
   }
 
   static nsresult WrapNative(JSContext *cx, JSObject *scope,
                              nsISupports *native, const nsIID& aIID,
                              jsval *vp);
+  static nsresult ThrowJSException(JSContext *cx, nsresult aResult);
 
 protected:
-  const nsDOMClassInfoID mID;
+  const nsDOMClassInfoData* mData;
 
   static nsresult Init();
   static nsresult RegisterClassName(PRInt32 aDOMClassInfoID);
   static nsresult RegisterClassProtos(PRInt32 aDOMClassInfoID);
+  static nsresult RegisterExternalClasses();
 
   // Checks if id is a number and returns the number, if aIsNumber is
   // non-null it's set to true if the id is a number and false if it's
@@ -101,6 +144,7 @@ protected:
   static inline PRBool IsReadonlyReplaceable(JSString *str)
   {
     return (str == sTop_id          ||
+            str == sParent_id       ||
             str == sScrollbars_id   ||
             str == sContent_id      ||
             str == sSidebar_id      ||
@@ -111,6 +155,8 @@ protected:
             str == sStatusbar_id    ||
             str == sDirectories_id  ||
             str == sControllers_id  ||
+            str == sScrollX_id      ||
+            str == sScrollY_id      ||
             str == sLength_id);
   }
 
@@ -126,6 +172,10 @@ protected:
             str == sName_id);
   }
 
+  nsresult doCheckPropertyAccess(JSContext *cx, JSObject *obj, jsval id,
+                                 nsIXPConnectWrappedNative *wrapper,
+                                 PRUint32 accessMode, PRBool isWindow);
+
   static JSClass sDOMConstructorProtoClass;
 
   static nsIXPConnect *sXPConnect;
@@ -137,6 +187,7 @@ protected:
   static PRBool sIsInitialized;
 
   static JSString *sTop_id;
+  static JSString *sParent_id;
   static JSString *sScrollbars_id;
   static JSString *sLocation_id;
   static JSString *sComponents_id;
@@ -183,9 +234,18 @@ protected:
   static JSString *sOnresize_id;
   static JSString *sOnscroll_id;
   static JSString *sScrollIntoView_id;
+  static JSString *sScrollX_id;
+  static JSString *sScrollY_id;
   static JSString *sOpen_id;
+  static JSString *sItem_id;
+  static JSString *sEnumerate_id;
+  static JSString *sNavigator_id;
+  static JSString *sDocument_id;
+  static JSString *sWindow_id;
 
   static const JSClass *sObjectClass;
+
+  static PRBool sDoSecurityCheckInAddProperty;
 };
 
 typedef nsDOMClassInfo nsDOMGenericSH;
@@ -198,7 +258,7 @@ typedef nsDOMClassInfo nsDOMGenericSH;
 class nsEventRecieverSH : public nsDOMGenericSH
 {
 protected:
-  nsEventRecieverSH(nsDOMClassInfoID aID) : nsDOMGenericSH(aID)
+  nsEventRecieverSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
   {
   }
 
@@ -238,7 +298,7 @@ public:
 class nsWindowSH : public nsEventRecieverSH
 {
 protected:
-  nsWindowSH(nsDOMClassInfoID aID) : nsEventRecieverSH(aID)
+  nsWindowSH(nsDOMClassInfoData* aData) : nsEventRecieverSH(aData)
   {
   }
 
@@ -252,10 +312,6 @@ protected:
   static nsresult DefineInterfaceProperty(JSContext *cx, JSObject *obj,
                                           JSString *str);
 
-  nsresult doCheckPropertyAccess(JSContext *cx, JSObject *obj, jsval id,
-                                 nsIXPConnectWrappedNative *wrapper,
-                                 PRUint32 accessMode);
-
 public:
   NS_IMETHOD PreCreate(nsISupports *nativeObj, JSContext *cx,
                        JSObject *globalObj, JSObject **parentObj);
@@ -273,12 +329,38 @@ public:
   NS_IMETHOD Finalize(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                       JSObject *obj);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsresult CacheDocumentProperty(JSContext *cx, JSObject *obj,
+                                        nsIDOMWindow *window);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsWindowSH(aData);
+  }
+};
+
+
+// Location scriptable helper
+
+class nsLocationSH : public nsDOMGenericSH
+{
+protected:
+  nsLocationSH(nsDOMClassInfoData* aData) : nsDOMGenericSH(aData)
   {
-    return new nsWindowSH(aID);
   }
 
-  static PRBool sDoSecurityCheckInAddProperty;
+  virtual ~nsLocationSH()
+  {
+  }
+
+public:
+  NS_IMETHOD CheckAccess(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, PRUint32 mode,
+                         jsval *vp, PRBool *_retval);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsLocationSH(aData);
+  }
 };
 
 
@@ -288,7 +370,7 @@ public:
 class nsNodeSH : public nsEventRecieverSH
 {
 protected:
-  nsNodeSH(nsDOMClassInfoID aID) : nsEventRecieverSH(aID)
+  nsNodeSH(nsDOMClassInfoData* aData) : nsEventRecieverSH(aData)
   {
   }
 
@@ -302,9 +384,9 @@ public:
   NS_IMETHOD AddProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsNodeSH(aID);
+    return new nsNodeSH(aData);
   }
 };
 
@@ -314,7 +396,7 @@ public:
 class nsElementSH : public nsNodeSH
 {
 protected:
-  nsElementSH(nsDOMClassInfoID aID) : nsNodeSH(aID)
+  nsElementSH(nsDOMClassInfoData* aData) : nsNodeSH(aData)
   {
   }
 
@@ -326,9 +408,9 @@ public:
   NS_IMETHOD PostCreate(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                     JSObject *obj);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsElementSH(aID);
+    return new nsElementSH(aData);
   }
 };
 
@@ -338,7 +420,7 @@ public:
 class nsArraySH : public nsDOMClassInfo
 {
 protected:
-  nsArraySH(nsDOMClassInfoID aID) : nsDOMClassInfo(aID)
+  nsArraySH(nsDOMClassInfoData* aData) : nsDOMClassInfo(aData)
   {
   }
 
@@ -353,9 +435,9 @@ public:
   NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsArraySH(aID);
+    return new nsArraySH(aData);
   }
 };
 
@@ -365,7 +447,7 @@ public:
 class nsNamedArraySH : public nsArraySH
 {
 protected:
-  nsNamedArraySH(nsDOMClassInfoID aID) : nsArraySH(aID)
+  nsNamedArraySH(nsDOMClassInfoData* aData) : nsArraySH(aData)
   {
   }
 
@@ -373,7 +455,7 @@ protected:
   {
   }
 
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult) = 0;
 
 public:
@@ -387,7 +469,7 @@ public:
 class nsNamedNodeMapSH : public nsNamedArraySH
 {
 protected:
-  nsNamedNodeMapSH(nsDOMClassInfoID aID) : nsNamedArraySH(aID)
+  nsNamedNodeMapSH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
   {
   }
 
@@ -401,13 +483,13 @@ protected:
                              nsISupports **aResult);
 
   // Override nsNamedArraySH::GetNamedItem()
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsNamedNodeMapSH(aID);
+    return new nsNamedNodeMapSH(aData);
   }
 };
 
@@ -417,7 +499,7 @@ public:
 class nsHTMLCollectionSH : public nsNamedArraySH
 {
 protected:
-  nsHTMLCollectionSH(nsDOMClassInfoID aID) : nsNamedArraySH(aID)
+  nsHTMLCollectionSH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
   {
   }
 
@@ -431,23 +513,48 @@ protected:
                              nsISupports **aResult);
 
   // Override nsNamedArraySH::GetNamedItem()
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLCollectionSH(aID);
+    return new nsHTMLCollectionSH(aData);
   }
 };
 
 
+// ContentList helper
+
+class nsContentListSH : public nsHTMLCollectionSH
+{
+protected:
+  nsContentListSH(nsDOMClassInfoData* aData) : nsHTMLCollectionSH(aData)
+  {
+  }
+
+  virtual ~nsContentListSH()
+  {
+  }
+
+public:
+  NS_IMETHOD PreCreate(nsISupports *nativeObj, JSContext *cx,
+                       JSObject *globalObj, JSObject **parentObj);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsContentListSH(aData);
+  }
+};
+
+
+
 // FomrControlList helper
 
 class nsFormControlListSH : public nsHTMLCollectionSH
 {
 protected:
-  nsFormControlListSH(nsDOMClassInfoID aID) : nsHTMLCollectionSH(aID)
+  nsFormControlListSH(nsDOMClassInfoData* aData) : nsHTMLCollectionSH(aData)
   {
   }
 
@@ -457,13 +564,13 @@ protected:
 
   // Override nsNamedArraySH::GetNamedItem() since our NamedItem() can
   // return either a nsIDOMNode or a nsIHTMLCollection
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsFormControlListSH(aID);
+    return new nsFormControlListSH(aData);
   }
 };
 
@@ -473,7 +580,7 @@ public:
 class nsDocumentSH : public nsNodeSH
 {
 public:
-  nsDocumentSH(nsDOMClassInfoID aID) : nsNodeSH(aID)
+  nsDocumentSH(nsDOMClassInfoData* aData) : nsNodeSH(aData)
   {
   }
 
@@ -482,15 +589,21 @@ public:
   }
 
 public:
+  NS_IMETHOD AddProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
+  NS_IMETHOD DelProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
   NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                         JSObject *obj, jsval id, PRUint32 flags,
                         JSObject **objp, PRBool *_retval);
+  NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
+                         JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
   NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsDocumentSH(aID);
+    return new nsDocumentSH(aData);
   }
 };
 
@@ -500,7 +613,7 @@ public:
 class nsHTMLDocumentSH : public nsDocumentSH
 {
 protected:
-  nsHTMLDocumentSH(nsDOMClassInfoID aID) : nsDocumentSH(aID)
+  nsHTMLDocumentSH(nsDOMClassInfoData* aData) : nsDocumentSH(aData)
   {
   }
 
@@ -508,10 +621,12 @@ protected:
   {
   }
 
-  static nsresult ResolveImpl(nsIXPConnectWrappedNative *wrapper, jsval id,
+  static nsresult ResolveImpl(JSContext *cx,
+                              nsIXPConnectWrappedNative *wrapper, jsval id,
                               nsISupports **result);
-  static JSBool JS_DLL_CALLBACK DocumentOpen(JSContext *cx, JSObject *obj, uintN argc,
-                             jsval *argv, jsval *rval);
+  static JSBool JS_DLL_CALLBACK DocumentOpen(JSContext *cx, JSObject *obj,
+                                             uintN argc, jsval *argv,
+                                             jsval *rval);
 
 public:
   NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
@@ -520,9 +635,9 @@ public:
   NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLDocumentSH(aID);
+    return new nsHTMLDocumentSH(aData);
   }
 };
 
@@ -532,7 +647,7 @@ public:
 class nsHTMLElementSH : public nsElementSH
 {
 protected:
-  nsHTMLElementSH(nsDOMClassInfoID aID) : nsElementSH(aID)
+  nsHTMLElementSH(nsDOMClassInfoData* aData) : nsElementSH(aData)
   {
   }
 
@@ -540,17 +655,18 @@ protected:
   {
   }
 
-  static JSBool JS_DLL_CALLBACK ScrollIntoView(JSContext *cx, JSObject *obj, uintN argc,
-                                               jsval *argv, jsval *rval);
+  static JSBool JS_DLL_CALLBACK ScrollIntoView(JSContext *cx, JSObject *obj,
+                                               uintN argc, jsval *argv,
+                                               jsval *rval);
 
 public:
   NS_IMETHOD NewResolve(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                         JSObject *obj, jsval id, PRUint32 flags,
                         JSObject **objp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLElementSH(aID);
+    return new nsHTMLElementSH(aData);
   }
 };
 
@@ -560,7 +676,7 @@ public:
 class nsHTMLFormElementSH : public nsHTMLElementSH
 {
 protected:
-  nsHTMLFormElementSH(nsDOMClassInfoID aID) : nsHTMLElementSH(aID)
+  nsHTMLFormElementSH(nsDOMClassInfoData* aData) : nsHTMLElementSH(aData)
   {
   }
 
@@ -579,9 +695,14 @@ public:
                          JSObject *obj, jsval id, jsval *vp,
                          PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  NS_IMETHOD NewEnumerate(nsIXPConnectWrappedNative *wrapper,
+                          JSContext *cx, JSObject *obj,
+                          PRUint32 enum_op, jsval *statep,
+                          jsid *idp, PRBool *_retval);
+
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLFormElementSH(aID);
+    return new nsHTMLFormElementSH(aData);
   }
 };
 
@@ -591,7 +712,7 @@ public:
 class nsHTMLSelectElementSH : public nsHTMLElementSH
 {
 protected:
-  nsHTMLSelectElementSH(nsDOMClassInfoID aID) : nsHTMLElementSH(aID)
+  nsHTMLSelectElementSH(nsDOMClassInfoData* aData) : nsHTMLElementSH(aData)
   {
   }
 
@@ -609,9 +730,9 @@ public:
   static nsresult SetOption(JSContext *cx, jsval *vp, PRUint32 aIndex,
                             nsIDOMNSHTMLOptionCollection *aOptCollection);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLSelectElementSH(aID);
+    return new nsHTMLSelectElementSH(aData);
   }
 };
 
@@ -622,7 +743,7 @@ public:
 class nsHTMLExternalObjSH : public nsHTMLElementSH
 {
 protected:
-  nsHTMLExternalObjSH(nsDOMClassInfoID aID) : nsHTMLElementSH(aID)
+  nsHTMLExternalObjSH(nsDOMClassInfoData* aData) : nsHTMLElementSH(aData)
   {
   }
 
@@ -649,7 +770,7 @@ public:
 class nsHTMLAppletElementSH : public nsHTMLExternalObjSH
 {
 protected:
-  nsHTMLAppletElementSH(nsDOMClassInfoID aID) : nsHTMLExternalObjSH(aID)
+  nsHTMLAppletElementSH(nsDOMClassInfoData* aData) : nsHTMLExternalObjSH(aData)
   {
   }
 
@@ -663,9 +784,9 @@ protected:
                                      JSObject **plugin_proto);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLAppletElementSH(aID);
+    return new nsHTMLAppletElementSH(aData);
   }
 };
 
@@ -675,7 +796,7 @@ public:
 class nsHTMLPluginObjElementSH : public nsHTMLAppletElementSH
 {
 protected:
-  nsHTMLPluginObjElementSH(nsDOMClassInfoID aID) : nsHTMLAppletElementSH(aID)
+  nsHTMLPluginObjElementSH(nsDOMClassInfoData* aData) : nsHTMLAppletElementSH(aData)
   {
   }
 
@@ -693,9 +814,9 @@ public:
                         JSObject *obj, jsval id, PRUint32 flags,
                         JSObject **objp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLPluginObjElementSH(aID);
+    return new nsHTMLPluginObjElementSH(aData);
   }
 };
 
@@ -705,7 +826,7 @@ public:
 class nsHTMLOptionCollectionSH : public nsHTMLCollectionSH
 {
 protected:
-  nsHTMLOptionCollectionSH(nsDOMClassInfoID aID) : nsHTMLCollectionSH(aID)
+  nsHTMLOptionCollectionSH(nsDOMClassInfoData* aData) : nsHTMLCollectionSH(aData)
   {
   }
 
@@ -717,9 +838,9 @@ public:
   NS_IMETHOD SetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLOptionCollectionSH(aID);
+    return new nsHTMLOptionCollectionSH(aData);
   }
 };
 
@@ -729,7 +850,7 @@ public:
 class nsPluginSH : public nsNamedArraySH
 {
 protected:
-  nsPluginSH(nsDOMClassInfoID aID) : nsNamedArraySH(aID)
+  nsPluginSH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
   {
   }
 
@@ -743,13 +864,13 @@ protected:
                              nsISupports **aResult);
 
   // Override nsNamedArraySH::GetNamedItem()
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsPluginSH(aID);
+    return new nsPluginSH(aData);
   }
 };
 
@@ -759,7 +880,7 @@ public:
 class nsPluginArraySH : public nsNamedArraySH
 {
 protected:
-  nsPluginArraySH(nsDOMClassInfoID aID) : nsNamedArraySH(aID)
+  nsPluginArraySH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
   {
   }
 
@@ -773,13 +894,13 @@ protected:
                              nsISupports **aResult);
 
   // Override nsNamedArraySH::GetNamedItem()
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsPluginArraySH(aID);
+    return new nsPluginArraySH(aData);
   }
 };
 
@@ -789,7 +910,7 @@ public:
 class nsMimeTypeArraySH : public nsNamedArraySH
 {
 protected:
-  nsMimeTypeArraySH(nsDOMClassInfoID aID) : nsNamedArraySH(aID)
+  nsMimeTypeArraySH(nsDOMClassInfoData* aData) : nsNamedArraySH(aData)
   {
   }
 
@@ -803,13 +924,13 @@ protected:
                              nsISupports **aResult);
 
   // Override nsNamedArraySH::GetNamedItem()
-  virtual nsresult GetNamedItem(nsISupports *aNative, nsAReadableString& aName,
+  virtual nsresult GetNamedItem(nsISupports *aNative, const nsAString& aName,
                                 nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsMimeTypeArraySH(aID);
+    return new nsMimeTypeArraySH(aData);
   }
 };
 
@@ -819,7 +940,7 @@ public:
 class nsStringArraySH : public nsDOMClassInfo
 {
 protected:
-  nsStringArraySH(nsDOMClassInfoID aID) : nsDOMClassInfo(aID)
+  nsStringArraySH(nsDOMClassInfoData* aData) : nsDOMClassInfo(aData)
   {
   }
 
@@ -828,7 +949,7 @@ protected:
   }
 
   virtual nsresult GetStringAt(nsISupports *aNative, PRInt32 aIndex,
-                               nsAWritableString& aResult) = 0;
+                               nsAString& aResult) = 0;
 
 public:
   NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
@@ -841,7 +962,7 @@ public:
 class nsHistorySH : public nsStringArraySH
 {
 protected:
-  nsHistorySH(nsDOMClassInfoID aID) : nsStringArraySH(aID)
+  nsHistorySH(nsDOMClassInfoData* aData) : nsStringArraySH(aData)
   {
   }
 
@@ -850,15 +971,15 @@ protected:
   }
 
   virtual nsresult GetStringAt(nsISupports *aNative, PRInt32 aIndex,
-                               nsAWritableString& aResult);
+                               nsAString& aResult);
 
 public:
   NS_IMETHOD GetProperty(nsIXPConnectWrappedNative *wrapper, JSContext *cx,
                          JSObject *obj, jsval id, jsval *vp, PRBool *_retval);
 
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHistorySH(aID);
+    return new nsHistorySH(aData);
   }
 };
 
@@ -868,7 +989,7 @@ public:
 class nsMediaListSH : public nsStringArraySH
 {
 protected:
-  nsMediaListSH(nsDOMClassInfoID aID) : nsStringArraySH(aID)
+  nsMediaListSH(nsDOMClassInfoData* aData) : nsStringArraySH(aData)
   {
   }
 
@@ -877,12 +998,12 @@ protected:
   }
 
   virtual nsresult GetStringAt(nsISupports *aNative, PRInt32 aIndex,
-                               nsAWritableString& aResult);
+                               nsAString& aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsMediaListSH(aID);
+    return new nsMediaListSH(aData);
   }
 };
 
@@ -892,7 +1013,7 @@ public:
 class nsStyleSheetListSH : public nsArraySH
 {
 protected:
-  nsStyleSheetListSH(nsDOMClassInfoID aID) : nsArraySH(aID)
+  nsStyleSheetListSH(nsDOMClassInfoData* aData) : nsArraySH(aData)
   {
   }
 
@@ -906,9 +1027,35 @@ protected:
                              nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsStyleSheetListSH(aID);
+    return new nsStyleSheetListSH(aData);
+  }
+};
+
+
+// CSSValueList helper
+
+class nsCSSValueListSH : public nsArraySH
+{
+protected:
+  nsCSSValueListSH(nsDOMClassInfoData* aData) : nsArraySH(aData)
+  {
+  }
+
+  virtual ~nsCSSValueListSH()
+  {
+  }
+
+  // Override nsArraySH::GetItemAt() since our list isn't a
+  // nsIDOMNodeList
+  virtual nsresult GetItemAt(nsISupports *aNative, PRUint32 aIndex,
+                             nsISupports **aResult);
+
+public:
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
+  {
+    return new nsCSSValueListSH(aData);
   }
 };
 
@@ -918,7 +1065,7 @@ public:
 class nsCSSStyleDeclSH : public nsStringArraySH
 {
 protected:
-  nsCSSStyleDeclSH(nsDOMClassInfoID aID) : nsStringArraySH(aID)
+  nsCSSStyleDeclSH(nsDOMClassInfoData* aData) : nsStringArraySH(aData)
   {
   }
 
@@ -927,12 +1074,12 @@ protected:
   }
 
   virtual nsresult GetStringAt(nsISupports *aNative, PRInt32 aIndex,
-                               nsAWritableString& aResult);
+                               nsAString& aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsCSSStyleDeclSH(aID);
+    return new nsCSSStyleDeclSH(aData);
   }
 };
 
@@ -942,7 +1089,7 @@ public:
 class nsCSSRuleListSH : public nsArraySH
 {
 protected:
-  nsCSSRuleListSH(nsDOMClassInfoID aID) : nsArraySH(aID)
+  nsCSSRuleListSH(nsDOMClassInfoData* aData) : nsArraySH(aData)
   {
   }
 
@@ -956,9 +1103,9 @@ protected:
                              nsISupports **aResult);
 
 public:
-  static nsIClassInfo *doCreate(nsDOMClassInfoID aID)
+  static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsCSSRuleListSH(aID);
+    return new nsCSSRuleListSH(aData);
   }
 };
 
