@@ -64,6 +64,7 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIDOMJSWindow.h"
+#include "nsIDOMChromeWindow.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsITimer.h"
@@ -78,6 +79,8 @@
 #include "nsPluginArray.h"
 #include "nsMimeTypeArray.h"
 #include "nsIXPCScriptable.h"
+#include "nsPoint.h"
+#include "nsSize.h"
 
 #define DEFAULT_HOME_PAGE "www.mozilla.org"
 #define PREF_BROWSER_STARTUP_HOMEPAGE "browser.startup.homepage"
@@ -126,7 +129,8 @@ public:
   NS_IMETHOD SetContext(nsIScriptContext *aContext);
   NS_IMETHOD GetContext(nsIScriptContext **aContext);
   NS_IMETHOD SetNewDocument(nsIDOMDocument *aDocument,
-                            PRBool removeEventListeners);
+                            PRBool aRemoveEventListeners,
+                            PRBool aClearScopeHint);
   NS_IMETHOD SetDocShell(nsIDocShell* aDocShell);
   NS_IMETHOD GetDocShell(nsIDocShell** aDocShell);
   NS_IMETHOD SetOpenerWindow(nsIDOMWindowInternal *aOpener);
@@ -137,7 +141,7 @@ public:
                             nsEventStatus* aEventStatus);
   NS_IMETHOD_(JSObject *) GetGlobalJSObject();
   NS_IMETHOD OnFinalize(JSObject *aJSObject);
-  NS_IMETHOD SetScriptsEnabled(PRBool aEnabled);
+  NS_IMETHOD SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts);
 
   // nsIScriptObjectPrincipal
   NS_IMETHOD GetPrincipal(nsIPrincipal **prin);
@@ -152,10 +156,10 @@ public:
   NS_DECL_NSIDOMJSWINDOW
 
   // nsIDOMEventTarget
-  NS_IMETHOD AddEventListener(const nsAReadableString& aType,
+  NS_IMETHOD AddEventListener(const nsAString& aType,
                               nsIDOMEventListener* aListener,
                               PRBool aUseCapture);
-  NS_IMETHOD RemoveEventListener(const nsAReadableString& aType,
+  NS_IMETHOD RemoveEventListener(const nsAString& aType,
                                  nsIDOMEventListener* aListener,
                                  PRBool aUseCapture);
   NS_IMETHOD DispatchEvent(nsIDOMEvent* aEvent, PRBool *_retval);
@@ -181,7 +185,13 @@ public:
                                   PRBool* aResult);
   NS_IMETHOD SetMutationListeners(PRUint32 aEventType);
   NS_IMETHOD GetRootFocusController(nsIFocusController** aResult);
+  NS_IMETHOD GetExtantDocument(nsIDOMDocument** aDocument);
+
   NS_IMETHOD ReallyCloseWindow();
+  NS_IMETHOD IsLoadingOrRunningTimeout(PRBool* aResult);
+
+  NS_IMETHOD GetFrameElementInternal(nsIDOMElement** aFrameElement);
+  NS_IMETHOD SetFrameElementInternal(nsIDOMElement* aFrameElement);
 
   // nsIDOMViewCSS
   NS_DECL_NSIDOMVIEWCSS
@@ -196,6 +206,7 @@ public:
   GlobalWindowImpl();
 
   static void ShutDown();
+  static PRBool IsCallerChrome();
 
 protected:
   // Object Management
@@ -206,9 +217,9 @@ protected:
   void GetParentInternal(nsIDOMWindowInternal **parent);
 
   // Window Control Functions
-  NS_IMETHOD OpenInternal(const nsAReadableString& aUrl,
-                          const nsAReadableString& aName,
-                          const nsAReadableString& aOptions,
+  NS_IMETHOD OpenInternal(const nsAString& aUrl,
+                          const nsAString& aName,
+                          const nsAString& aOptions,
                           PRBool aDialog, jsval *argv, PRUint32 argc,
                           nsISupports *aExtraArgument, nsIDOMWindow **aReturn);
   static void CloseWindow(nsISupports* aWindow);
@@ -218,7 +229,7 @@ protected:
   void RunTimeout(nsTimeoutImpl *aTimeout);
   void DropTimeout(nsTimeoutImpl *aTimeout, nsIScriptContext* aContext=nsnull);
   void HoldTimeout(nsTimeoutImpl *aTimeout);
-  nsresult ClearTimeoutOrInterval(PRInt32 aTimerID);
+  nsresult ClearTimeoutOrInterval();
   void ClearAllTimeouts();
   void InsertTimeoutIntoList(nsTimeoutImpl **aInsertionPoint,
                              nsTimeoutImpl *aTimeout);
@@ -233,14 +244,16 @@ protected:
   nsresult SecurityCheckURL(const char *aURL);
   PRBool   CheckForAbusePoint();
 
-  void FlushPendingNotifications();
+  void FlushPendingNotifications(PRBool aFlushReflows);
+  void EnsureReflowFlushAndPaint();
   nsresult CheckSecurityWidthAndHeight(PRInt32* width, PRInt32* height);
   nsresult CheckSecurityLeftAndTop(PRInt32* left, PRInt32* top);
+  static nsresult CheckSecurityIsChromeCaller(PRBool *isChrome);
 
-  nsresult DoPrint(PRBool aDoPreview, nsIPrintSettings* aPS = nsnull);
+  void MakeScriptDialogTitle(const nsAString &aInTitle, nsAString &aOutTitle);
 
   // Helper for window.find()
-  nsresult FindInternal(nsAReadableString& aStr, PRBool caseSensitive,
+  nsresult FindInternal(const nsAString& aStr, PRBool caseSensitive,
                        PRBool backwards, PRBool wrapAround, PRBool wholeWord, 
                        PRBool searchInFrames, PRBool showDialog, 
                        PRBool *aReturn);
@@ -279,9 +292,14 @@ protected:
   PRPackedBool                  mFirstDocumentLoad;
   PRPackedBool                  mIsScopeClear;
   PRPackedBool                  mIsDocumentLoaded; // true between onload and onunload events
+  PRTime                        mLastMouseButtonAction;
+  PRPackedBool                  mFullScreen;
   nsString                      mStatus;
   nsString                      mDefaultStatus;
-  nsString                      mTitle;
+
+  // state preservation for full screen mode
+  nsPoint*                     mOriginalPos;
+  nsSize*                      mOriginalSize;
 
   nsIScriptGlobalObjectOwner*   mGlobalObjectOwner; // Weak Reference
   nsIDocShell*                  mDocShell;  // Weak Reference
@@ -291,8 +309,32 @@ protected:
   nsCOMPtr<nsIDOMPkcs11>        mPkcs11;
   nsCOMPtr<nsIPrincipal>        mDocumentPrincipal;
 
+  nsIDOMElement*                mFrameElement; // WEAK
+
   friend class nsDOMScriptableHelper;
   static nsIXPConnect *sXPConnect;
+  static nsIScriptSecurityManager *sSecMan;
+  static nsIFactory *sComputedDOMStyleFactory;
+};
+
+/*
+ * nsGlobalChromeWindow inherits from GlobalWindowImpl. It is the global
+ * object created for a Chrome Window only.
+ */
+class nsGlobalChromeWindow : public GlobalWindowImpl,
+                             public nsIDOMChromeWindow
+{
+public:
+  // nsISupports
+  NS_DECL_ISUPPORTS_INHERITED
+
+  // nsIDOMChromeWindow interface
+  NS_DECL_NSIDOMCHROMEWINDOW
+
+protected:
+  nsresult GetMainWidget(nsIWidget** aMainWidget);
+
+  nsString mTitle;
 };
 
 /*
@@ -352,6 +394,8 @@ protected:
   MimeTypeArrayImpl* mMimeTypes;
   PluginArrayImpl* mPlugins;
   nsIDocShell* mDocShell; // weak reference
+
+  static jsval       sPrefInternal_id;
 };
 
 class nsIURI;
@@ -378,10 +422,12 @@ public:
   NS_DECL_NSIDOMNSLOCATION
 
 protected:
-  nsresult SetURL(nsIURI* aURL);
-  nsresult SetHrefWithBase(const nsAReadableString& aHref, nsIURI* aBase,
+  nsresult GetURI(nsIURI** aURL);
+  nsresult GetWritableURI(nsIURI** aURL);
+  nsresult SetURI(nsIURI* aURL);
+  nsresult SetHrefWithBase(const nsAString& aHref, nsIURI* aBase,
                            PRBool aReplace);
-  nsresult SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
+  nsresult SetHrefWithContext(JSContext* cx, const nsAString& aHref,
                               PRBool aReplace);
 
   nsresult GetSourceURL(JSContext* cx, nsIURI** sourceURL);
@@ -403,16 +449,19 @@ class nsDOMWindowController : public nsIController
 {
 public:
 	nsDOMWindowController( nsIDOMWindowInternal* aWindow );
+  virtual ~nsDOMWindowController();
   NS_DECL_ISUPPORTS
   NS_DECL_NSICONTROLLER
 
 private:
+  nsresult GetEventStateManager(nsIEventStateManager **esm);
+  static int PR_CALLBACK BrowseWithCaretPrefCallback(const char* aPrefName, void* instance_data);
   nsresult GetPresShell(nsIPresShell **aPresShell);
 	nsresult GetEditInterface( nsIContentViewerEdit** aEditInterface);
   nsresult GetSelectionController(nsISelectionController ** aSelCon);
 
-  nsresult DoCommandWithEditInterface(const nsCString& aCommandName);
-  nsresult DoCommandWithSelectionController(const nsCString& aCommandName);
+  nsresult DoCommandWithEditInterface(const char * aCommandName);
+  nsresult DoCommandWithSelectionController(const char * aCommandName);
 
 	nsIDOMWindowInternal *mWindow;
   PRBool mBrowseWithCaret;
