@@ -103,6 +103,23 @@ static PRBool sReadyForGC = PR_FALSE;
 
 nsScriptNameSpaceManager *gNameSpaceManager;
 
+static nsIJSRuntimeService *sRuntimeService = nsnull;
+JSRuntime *nsJSEnvironment::sRuntime = nsnull;
+
+static const char kJSRuntimeServiceContractID[] =
+  "@mozilla.org/js/xpc/RuntimeService;1";
+static const char kScriptSecurityManagerContractID[] =
+  NS_SCRIPTSECURITYMANAGER_CONTRACTID;
+
+static PRThread *gDOMThread;
+
+static JSGCCallback gOldJSGCCallback;
+
+static PRBool sDidShutdown = PR_FALSE;
+
+static PRInt32 sContextCount = 0;
+
+
 void JS_DLL_CALLBACK
 NS_ScriptErrorReporter(JSContext *cx,
                        const char *message,
@@ -159,7 +176,7 @@ NS_ScriptErrorReporter(JSContext *cx,
 
           errorevent.fileName = fileName.get();
           errorevent.errorMsg = msg.get();
-          errorevent.lineNr = report ? report->lineno : -1;
+          errorevent.lineNr = report ? report->lineno : 0;
 
           // HandleDOMEvent() must be synchronous for the recursion block
           // (errorDepth) to work.
@@ -372,6 +389,8 @@ nsJSContext::nsJSContext(JSRuntime *aRuntime) : mGCOnDestruction(PR_TRUE)
 {
   NS_INIT_REFCNT();
 
+  ++sContextCount;
+
   mDefaultJSOptions = JSOPTION_PRIVATE_IS_NSISUPPORTS
 #ifdef DEBUG
     | JSOPTION_STRICT   // lint catching for development
@@ -415,8 +434,6 @@ nsJSContext::nsJSContext(JSRuntime *aRuntime) : mGCOnDestruction(PR_TRUE)
   InvalidateContextAndWrapperCache();
 }
 
-const char kScriptSecurityManagerContractID[] = NS_SCRIPTSECURITYMANAGER_CONTRACTID;
-
 nsJSContext::~nsJSContext()
 {
   mSecurityManager = nsnull; // Force release
@@ -441,22 +458,21 @@ nsJSContext::~nsJSContext()
   // Let xpconnect destroy the JSContext when it thinks the time is right.
   nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID()));
   if (xpc) {
-#if 0
-    // This code is temporarily taken out until we find the real cause
-    // of bug 112195. Something in the profile migration code relies
-    // on GC being run at context destruction time when the profile
-    // migration dialog is closed down. If GC isn't run at that point,
-    // something stays around in memory and keeps the browser from
-    // starting up after the migration of a profile.
     PRBool do_gc = mGCOnDestruction && !sGCTimer && sReadyForGC;
-#else
-    PRBool do_gc = mGCOnDestruction && !sGCTimer;
-#endif
 
     xpc->ReleaseJSContext(mContext, !do_gc);
   } else {
     ::JS_DestroyContext(mContext);
   }
+
+  --sContextCount;
+
+  if (!sContextCount && sDidShutdown) {
+    // The last context is being deleted, and we're already in the
+    // process of shutting down, release the JS runtime service.
+
+    NS_IF_RELEASE(sRuntimeService);
+  }
 }
 
 // QueryInterface implementation for nsJSContext
@@ -473,7 +489,7 @@ NS_IMPL_RELEASE(nsJSContext)
 
 
 NS_IMETHODIMP
-nsJSContext::EvaluateStringWithValue(const nsAReadableString& aScript,
+nsJSContext::EvaluateStringWithValue(const nsAString& aScript,
                                      void *aScopeObject,
                                      nsIPrincipal *aPrincipal,
                                      const char *aURL,
@@ -544,7 +560,7 @@ nsJSContext::EvaluateStringWithValue(const nsAReadableString& aScript,
   jsval val;
 
   if (ok) {
-    JSVersion newVersion;
+    JSVersion newVersion = JSVERSION_UNKNOWN;
 
     // SecurityManager said "ok", but don't execute if aVersion is specified
     // and unknown.  Do execute with the default version (and avoid thrashing
@@ -552,7 +568,7 @@ nsJSContext::EvaluateStringWithValue(const nsAReadableString& aScript,
     ok = (!aVersion ||
           (newVersion = ::JS_StringToVersion(aVersion)) != JSVERSION_UNKNOWN);
     if (ok) {
-      JSVersion oldVersion;
+      JSVersion oldVersion = JSVERSION_UNKNOWN;
 
       if (aVersion)
         oldVersion = ::JS_SetVersion(mContext, newVersion);
@@ -592,13 +608,13 @@ nsJSContext::EvaluateStringWithValue(const nsAReadableString& aScript,
 }
 
 NS_IMETHODIMP
-nsJSContext::EvaluateString(const nsAReadableString& aScript,
+nsJSContext::EvaluateString(const nsAString& aScript,
                             void *aScopeObject,
                             nsIPrincipal *aPrincipal,
                             const char *aURL,
                             PRUint32 aLineNo,
                             const char* aVersion,
-                            nsAWritableString& aRetValue,
+                            nsAString& aRetValue,
                             PRBool* aIsUndefined)
 {
   if (!mScriptsEnabled) {
@@ -662,7 +678,7 @@ nsJSContext::EvaluateString(const nsAReadableString& aScript,
   jsval val;
 
   if (ok) {
-    JSVersion newVersion;
+    JSVersion newVersion = JSVERSION_UNKNOWN;
 
     // SecurityManager said "ok", but don't execute if aVersion is specified
     // and unknown.  Do execute with the default version (and avoid thrashing
@@ -670,7 +686,7 @@ nsJSContext::EvaluateString(const nsAReadableString& aScript,
     ok = (!aVersion ||
           (newVersion = ::JS_StringToVersion(aVersion)) != JSVERSION_UNKNOWN);
     if (ok) {
-      JSVersion oldVersion;
+      JSVersion oldVersion = JSVERSION_UNKNOWN;
 
       if (aVersion)
         oldVersion = ::JS_SetVersion(mContext, newVersion);
@@ -750,14 +766,14 @@ nsJSContext::CompileScript(const PRUnichar* aText,
 
   *aScriptObject = nsnull;
   if (ok) {
-    JSVersion newVersion;
+    JSVersion newVersion = JSVERSION_UNKNOWN;
 
     // SecurityManager said "ok", but don't compile if aVersion is specified
     // and unknown.  Do compile with the default version (and avoid thrashing
     // the context's version) if aVersion is not specified.
     if (!aVersion ||
         (newVersion = ::JS_StringToVersion(aVersion)) != JSVERSION_UNKNOWN) {
-      JSVersion oldVersion;
+      JSVersion oldVersion = JSVERSION_UNKNOWN;
       if (aVersion)
         oldVersion = ::JS_SetVersion(mContext, newVersion);
 
@@ -792,7 +808,7 @@ nsJSContext::CompileScript(const PRUnichar* aText,
 NS_IMETHODIMP
 nsJSContext::ExecuteScript(void* aScriptObject,
                            void *aScopeObject,
-                           nsAWritableString* aRetValue,
+                           nsAString* aRetValue,
                            PRBool* aIsUndefined)
 {
   if (!mScriptsEnabled) {
@@ -888,7 +904,7 @@ AtomToEventHandlerName(nsIAtom *aName, char *charName, PRUint32 charNameSize)
 
 NS_IMETHODIMP
 nsJSContext::CompileEventHandler(void *aTarget, nsIAtom *aName,
-                                 const nsAReadableString& aBody,
+                                 const nsAString& aBody,
                                  PRBool aShared, void** aHandler)
 {
   JSPrincipals *jsprin = nsnull;
@@ -939,7 +955,7 @@ nsJSContext::CompileFunction(void* aTarget,
                              const nsCString& aName,
                              PRUint32 aArgCount,
                              const char** aArgArray,
-                             const nsAReadableString& aBody,
+                             const nsAString& aBody,
                              const char* aURL,
                              PRUint32 aLineNo,
                              PRBool aShared,
@@ -989,6 +1005,11 @@ NS_IMETHODIMP
 nsJSContext::CallEventHandler(void *aTarget, void *aHandler, PRUint32 argc,
                               void *argv, PRBool *aBoolResult, PRBool aReverseReturnResult)
 {
+  if (!mScriptsEnabled) {
+    *aBoolResult = !aReverseReturnResult;
+    return NS_OK;
+  }
+
   // This one's a lot easier than EvaluateString because we don't have to
   // hassle with principals: they're already compiled into the JS function.
   nsresult rv;
@@ -1523,7 +1544,7 @@ nsJSContext::GetScriptsEnabled(PRBool *aEnabled)
 }
 
 NS_IMETHODIMP
-nsJSContext::SetScriptsEnabled(PRBool aEnabled)
+nsJSContext::SetScriptsEnabled(PRBool aEnabled, PRBool aFireTimeouts)
 {
   mScriptsEnabled = aEnabled;
 
@@ -1531,7 +1552,7 @@ nsJSContext::SetScriptsEnabled(PRBool aEnabled)
   GetGlobalObject(getter_AddRefs(global));
 
   if (global) {
-    global->SetScriptsEnabled(aEnabled);
+    global->SetScriptsEnabled(aEnabled, aFireTimeouts);
   }
 
   return NS_OK;
@@ -1611,14 +1632,6 @@ nsJSContext::FireGCTimer()
   first = PR_FALSE;
 }
 
-nsIJSRuntimeService *nsJSEnvironment::sRuntimeService = nsnull;
-JSRuntime *nsJSEnvironment::sRuntime = nsnull;
-
-const char kJSRuntimeServiceContractID[] = "@mozilla.org/js/xpc/RuntimeService;1";
-static PRThread *gDOMThread;
-
-static JSGCCallback gOldJSGCCallback;
-
 static JSBool JS_DLL_CALLBACK
 DOMGCCallback(JSContext *cx, JSGCStatus status)
 {
@@ -1707,10 +1720,14 @@ void nsJSEnvironment::ShutDown()
   delete gNameSpaceManager;
   gNameSpaceManager = nsnull;
 
-  if (sRuntimeService) {
-    nsServiceManager::ReleaseService(kJSRuntimeServiceContractID,
-                                     sRuntimeService);
+  if (!sContextCount) {
+    // We're being shutdown, and there are no more contexts
+    // alive, release the JS runtime service.
+
+    NS_IF_RELEASE(sRuntimeService);
   }
+
+  sDidShutdown = PR_TRUE;
 }
 
 // static
