@@ -51,6 +51,7 @@
 #include "prprf.h"
 #include "prmem.h"
 #include "nsCOMPtr.h"
+#include "nsEscape.h"
 #include "nsJSUtils.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsICodebasePrincipal.h"
@@ -63,9 +64,10 @@
 #include "nsDOMClassInfo.h"
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetConverterManager2.h"
+#include "nsCRT.h"
 
 
-static nsresult EscapeNonAsciiInURI(nsAReadableString& aHref, nsAWritableCString& aEscapedHref)
+static nsresult EscapeNonAsciiInURI(const nsAString& aHref, nsACString& aEscapedHref)
 {
   aEscapedHref.Truncate(0);
 
@@ -113,12 +115,12 @@ static nsresult EscapeNonAsciiInURI(nsAReadableString& aHref, nsAWritableCString
   NS_ENSURE_SUCCESS(rv, rv);
 
   const nsAFlatString &tmp = PromiseFlatString(aHref);
-  PRInt32 len;
+  PRInt32 len = tmp.Length();
+  PRInt32 bufferLen;
 
-  rv = encoder->GetMaxLength(tmp.get(), tmp.Length(), &len);
+  rv = encoder->GetMaxLength(tmp.get(), len, &bufferLen);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 bufferLen = len;
   char* buffer = (char *) nsMemory::Alloc(bufferLen);
   NS_ENSURE_TRUE(buffer, NS_ERROR_OUT_OF_MEMORY);
 
@@ -178,6 +180,8 @@ NS_IMETHODIMP_(void) LocationImpl::SetDocShell(nsIDocShell *aDocShell)
 nsresult
 LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
 {
+  *aLoadInfo = nsnull;
+
   nsresult result;
   // Get JSContext from stack.
   nsCOMPtr<nsIJSContextStack>
@@ -191,6 +195,16 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
   if (NS_FAILED(stack->Peek(&cx)))
     return NS_ERROR_FAILURE;
 
+  if (!cx) {
+    // No cx means that there's no JS running, or at least no JS that
+    // was run through code that properly pushed a context onto the
+    // context stack (as all code that runs JS off of web pages
+    // does). Going further from here will crash, so lets not do
+    // that...
+
+    return NS_OK;
+  }
+
   // Get security manager.
   nsCOMPtr<nsIScriptSecurityManager>
     secMan(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &result));
@@ -230,9 +244,38 @@ LocationImpl::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
   return NS_OK;
 }
 
+nsresult
+LocationImpl::GetURI(nsIURI** aURI)
+{
+  *aURI = nsnull;
+
+  nsresult rv;
+
+  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell, &rv));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  return webNav->GetCurrentURI(aURI);
+}
 
 nsresult
-LocationImpl::SetURL(nsIURI* aURI)
+LocationImpl::GetWritableURI(nsIURI** aURI)
+{
+  *aURI = nsnull;
+
+  nsCOMPtr<nsIURI> uri;
+
+  nsresult rv = GetURI(getter_AddRefs(uri));
+  if (NS_FAILED(rv) || !uri) {
+    return rv;
+  }
+
+  return uri->Clone(aURI);
+}
+
+nsresult
+LocationImpl::SetURI(nsIURI* aURI)
 {
   if (mDocShell) {
     nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
@@ -243,40 +286,33 @@ LocationImpl::SetURL(nsIURI* aURI)
 
     webNav->Stop(nsIWebNavigation::STOP_CONTENT);
     return mDocShell->LoadURI(aURI, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE);
+                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHash(nsAWritableString& aHash)
+LocationImpl::GetHash(nsAString& aHash)
 {
-  nsAutoString href;
+  aHash.SetLength(0);
+
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetHref(href);
+  result = GetURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  if (url) {
+    nsCAutoString ref;
 
-    if (uri) {
-      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-      nsXPIDLCString ref;
+    result = url->GetRef(ref);
+    // XXX danger... this may result in non-ASCII octets!
+    NS_UnescapeURL(ref);
 
-      if (url) {
-        result = url->GetRef(getter_Copies(ref));
-      }
-
-      if (NS_SUCCEEDED(result) && ref && *ref) {
-        aHash.Assign(NS_LITERAL_STRING("#"));
-        aHash.Append(NS_ConvertASCIItoUCS2(ref));
-      }
-      else {
-        aHash.SetLength(0);
-      }
+    if (NS_SUCCEEDED(result) && !ref.IsEmpty()) {
+      aHash.Assign(NS_LITERAL_STRING("#") + NS_ConvertASCIItoUCS2(ref));
     }
   }
 
@@ -284,36 +320,27 @@ LocationImpl::GetHash(nsAWritableString& aHash)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHash(const nsAReadableString& aHash)
+LocationImpl::SetHash(const nsAString& aHash)
 {
-  nsAutoString href;
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  if (url) {
+    url->SetRef(NS_ConvertUCS2toUTF8(aHash));
 
-    if (NS_FAILED(result))
-      return result;
+    if (mDocShell) {
+      nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
 
-    nsCOMPtr<nsIURL> url(do_QueryInterface(uri, &result));
-
-    if (url) {
-      url->SetRef(NS_ConvertUCS2toUTF8(aHash).get());
-
-      if (mDocShell) {
-        nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
-
-        if (NS_SUCCEEDED(CheckURL(url, getter_AddRefs(loadInfo))))
-          // We're not calling nsIWebNavigation->Stop, we don't
-          // want to stop the load when we're just scrolling to a
-          // named anchor in the document. See bug 114975.
-          mDocShell->LoadURI(url, loadInfo,
-                             nsIWebNavigation::LOAD_FLAGS_NONE);
-      }
+      if (NS_SUCCEEDED(CheckURL(url, getter_AddRefs(loadInfo))))
+        // We're not calling nsIWebNavigation->Stop, we don't want to
+        // stop the load when we're just scrolling to a named anchor
+        // in the document. See bug 114975.
+        mDocShell->LoadURI(url, loadInfo,
+                           nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
     }
   }
 
@@ -321,39 +348,22 @@ LocationImpl::SetHash(const nsAReadableString& aHash)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHost(nsAWritableString& aHost)
+LocationImpl::GetHost(nsAString& aHost)
 {
-  nsAutoString href;
-  nsresult result = NS_OK;
+  aHost.SetLength(0);
 
-  result = GetHref(href);
+  nsCOMPtr<nsIURI> uri;
+  nsresult result;
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  result = GetURI(getter_AddRefs(uri));
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  if (uri) {
+    nsCAutoString hostport;
 
-    if (uri) {
-      nsXPIDLCString host;
+    result = uri->GetHostPort(hostport);
 
-      result = uri->GetHost(getter_Copies(host));
-
-      if (NS_SUCCEEDED(result)) {
-        PRInt32 port;
-
-        CopyASCIItoUCS2(nsDependentCString(host), aHost);
-
-        uri->GetPort(&port);
-
-        if (port != -1) {
-          aHost.Append(PRUnichar(':'));
-
-          nsAutoString tmpHost;
-          tmpHost.AppendInt(port);
-
-          aHost.Append(tmpHost);
-        }
-      }
+    if (NS_SUCCEEDED(result)) {
+      aHost = NS_ConvertUTF8toUCS2(hostport);
     }
   }
 
@@ -361,22 +371,38 @@ LocationImpl::GetHost(nsAWritableString& aHost)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHost(const nsAReadableString& aHost)
+LocationImpl::SetHost(const nsAString& aHost)
 {
-  nsAutoString href;
-  nsresult result = NS_OK;
+  nsCOMPtr<nsIURI> uri;
+  nsresult result;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  if (uri) {
+    uri->SetHostPort(NS_ConvertUCS2toUTF8(aHost));
+    SetURI(uri);
+  }
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  return result;
+}
 
-    if (uri) {
-      uri->SetHost(NS_ConvertUCS2toUTF8(aHost).get());
+NS_IMETHODIMP
+LocationImpl::GetHostname(nsAString& aHostname)
+{
+  aHostname.SetLength(0);
 
-      SetURL(uri);
+  nsCOMPtr<nsIURI> uri;
+  nsresult result;
+
+  result = GetURI(getter_AddRefs(uri));
+
+  if (uri) {
+    nsCAutoString host;
+
+    result = uri->GetHost(host);
+
+    if (NS_SUCCEEDED(result)) {
+      aHostname = NS_ConvertUTF8toUCS2(host);
     }
   }
 
@@ -384,26 +410,38 @@ LocationImpl::SetHost(const nsAReadableString& aHost)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetHostname(nsAWritableString& aHostname)
+LocationImpl::SetHostname(const nsAString& aHostname)
 {
-  nsAutoString href;
-  nsresult result = NS_OK;
+  nsCOMPtr<nsIURI> uri;
+  nsresult result;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  if (uri) {
+    uri->SetHost(NS_ConvertUCS2toUTF8(aHostname));
+    SetURI(uri);
+  }
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  return result;
+}
 
-    if (uri) {
-      nsXPIDLCString host;
+NS_IMETHODIMP
+LocationImpl::GetHref(nsAString& aHref)
+{
+  aHref.SetLength(0);
 
-      result = uri->GetHost(getter_Copies(host));
+  nsCOMPtr<nsIURI> uri;
+  nsresult result;
 
-      if (NS_SUCCEEDED(result)) {
-        CopyASCIItoUCS2(nsDependentCString(host), aHostname);
-      }
+  result = GetURI(getter_AddRefs(uri));
+
+  if (uri) {
+    nsCAutoString uriString;
+
+    result = uri->GetSpec(uriString);
+
+    if (NS_SUCCEEDED(result)) {
+      aHref = NS_ConvertUTF8toUCS2(uriString);
     }
   }
 
@@ -411,54 +449,7 @@ LocationImpl::GetHostname(nsAWritableString& aHostname)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetHostname(const nsAReadableString& aHostname)
-{
-  nsAutoString href;
-  nsresult result = NS_OK;
-
-  result = GetHref(href);
-
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      uri->SetHost(NS_ConvertUCS2toUTF8(aHostname).get());
-      SetURL(uri);
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::GetHref(nsAWritableString& aHref)
-{
-  nsresult result = NS_OK;
-
-  nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell));
-
-  if (webNav) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = webNav->GetCurrentURI(getter_AddRefs(uri));
-
-    if (NS_SUCCEEDED(result) && uri) {
-      nsXPIDLCString uriString;
-
-      result = uri->GetSpec(getter_Copies(uriString));
-
-      if (NS_SUCCEEDED(result))
-        CopyASCIItoUCS2(nsDependentCString(uriString), aHref);
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::SetHref(const nsAReadableString& aHref)
+LocationImpl::SetHref(const nsAString& aHref)
 {
   nsAutoString oldHref;
   nsresult rv = NS_OK;
@@ -495,7 +486,7 @@ LocationImpl::SetHref(const nsAReadableString& aHref)
 }
 
 nsresult
-LocationImpl::SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
+LocationImpl::SetHrefWithContext(JSContext* cx, const nsAString& aHref,
                                  PRBool aReplace)
 {
   nsCOMPtr<nsIURI> base;
@@ -511,7 +502,7 @@ LocationImpl::SetHrefWithContext(JSContext* cx, const nsAReadableString& aHref,
 }
 
 nsresult
-LocationImpl::SetHrefWithBase(const nsAReadableString& aHref,
+LocationImpl::SetHrefWithBase(const nsAString& aHref,
                               nsIURI* aBase, PRBool aReplace)
 {
   nsresult result;
@@ -526,9 +517,9 @@ LocationImpl::SetHrefWithBase(const nsAReadableString& aHref,
     (void) EscapeNonAsciiInURI(aHref, escapedHref);
 
   if (escapedHref.IsEmpty())
-    result = NS_NewURI(getter_AddRefs(newUri), aHref, aBase);
+    result = NS_NewURI(getter_AddRefs(newUri), aHref, nsnull, aBase);
   else
-    result = NS_NewURI(getter_AddRefs(newUri), escapedHref.get(), aBase);
+    result = NS_NewURI(getter_AddRefs(newUri), escapedHref, nsnull, aBase);
 
   if (newUri && mDocShell) {
     nsCOMPtr<nsIDocShellLoadInfo> loadInfo;
@@ -572,38 +563,30 @@ LocationImpl::SetHrefWithBase(const nsAReadableString& aHref,
 
     webNav->Stop(nsIWebNavigation::STOP_CONTENT);
     return mDocShell->LoadURI(newUri, loadInfo,
-                              nsIWebNavigation::LOAD_FLAGS_NONE);
+                              nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   return result;
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPathname(nsAWritableString& aPathname)
+LocationImpl::GetPathname(nsAString& aPathname)
 {
-  nsAutoString href;
+  aPathname.SetLength(0);
+
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  aPathname.Truncate();
+  result = GetURI(getter_AddRefs(uri));
 
-  result = GetHref(href);
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  if (url) {
+    nsCAutoString file;
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+    result = url->GetFilePath(file);
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-
-      if (url) {
-        nsXPIDLCString file;
-        result = url->GetFilePath(getter_Copies(file));
-
-        if (NS_SUCCEEDED(result)) {
-          CopyASCIItoUCS2(nsDependentCString(file), aPathname);
-        }
-      }
+    if (NS_SUCCEEDED(result)) {
+      aPathname = NS_ConvertUTF8toUCS2(file);
     }
   }
 
@@ -611,21 +594,39 @@ LocationImpl::GetPathname(nsAWritableString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::SetPathname(const nsAReadableString& aPathname)
+LocationImpl::SetPathname(const nsAString& aPathname)
 {
-  nsAutoString href;
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  if (uri) {
+    uri->SetPath(NS_ConvertUCS2toUTF8(aPathname));
+    SetURI(uri);
+  }
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
+  return result;
+}
 
-    if (uri) {
-      uri->SetPath(NS_ConvertUCS2toUTF8(aPathname).get());
-      SetURL(uri);
+NS_IMETHODIMP
+LocationImpl::GetPort(nsAString& aPort)
+{
+  aPort.SetLength(0);
+
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = NS_OK;
+
+  result = GetURI(getter_AddRefs(uri));
+
+  if (uri) {
+    PRInt32 port;
+    uri->GetPort(&port);
+
+    if (-1 != port) {
+      nsAutoString portStr;
+      portStr.AppendInt(port);
+      aPort.Append(portStr);
     }
   }
 
@@ -633,176 +634,112 @@ LocationImpl::SetPathname(const nsAReadableString& aPathname)
 }
 
 NS_IMETHODIMP
-LocationImpl::GetPort(nsAWritableString& aPort)
+LocationImpl::SetPort(const nsAString& aPort)
 {
-  nsAutoString href;
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
+  if (uri) {
+    // perhaps use nsReadingIterators at some point?
+    NS_ConvertUCS2toUTF8 portStr(aPort);
+    const char *buf = portStr.get();
+    PRInt32 port = -1;
 
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      aPort.SetLength(0);
-
-      PRInt32 port;
-      uri->GetPort(&port);
-
-      if (-1 != port) {
-        nsAutoString portStr;
-        portStr.AppendInt(port);
-        aPort.Append(portStr);
-      }
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::SetPort(const nsAReadableString& aPort)
-{
-  nsAutoString href;
-  nsresult result = NS_OK;
-
-  result = GetHref(href);
-
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      // perhaps use nsReadingIterators at some point?
-      NS_ConvertUCS2toUTF8 portStr(aPort);
-      const char *buf = portStr.get();
-      PRInt32 port = -1;
-
-      if (buf) {
-        if (*buf == ':') {
-          port = atol(buf+1);
-        }
-        else {
-          port = atol(buf);
-        }
-      }
-      uri->SetPort(port);
-      SetURL(uri);
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::GetProtocol(nsAWritableString& aProtocol)
-{
-  nsAutoString href;
-  nsresult result = NS_OK;
-
-  result = GetHref(href);
-
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      nsXPIDLCString protocol;
-
-      result = uri->GetScheme(getter_Copies(protocol));
-
-      if (NS_SUCCEEDED(result)) {
-        aProtocol.Assign(NS_ConvertASCIItoUCS2(protocol));
-        aProtocol.Append(PRUnichar(':'));
-      }
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::SetProtocol(const nsAReadableString& aProtocol)
-{
-  nsAutoString href;
-  nsresult result = NS_OK;
-
-  result = GetHref(href);
-
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      uri->SetScheme(NS_ConvertUCS2toUTF8(aProtocol).get());
-      SetURL(uri);
-    }
-  }
-
-  return result;
-}
-
-NS_IMETHODIMP
-LocationImpl::GetSearch(nsAWritableString& aSearch)
-{
-  nsAutoString href;
-  nsresult result = NS_OK;
-
-  result = GetHref(href);
-
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      nsXPIDLCString search;
-      nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-
-      if (url) {
-        result = url->GetEscapedQuery(getter_Copies(search));
-      }
-
-      if (NS_SUCCEEDED(result) && search && *search) {
-        aSearch.Assign(NS_LITERAL_STRING("?"));
-        aSearch.Append(NS_ConvertASCIItoUCS2(search));
+    if (buf) {
+      if (*buf == ':') {
+        port = atol(buf+1);
       }
       else {
-        aSearch.SetLength(0);
+        port = atol(buf);
       }
     }
+
+    uri->SetPort(port);
+    SetURI(uri);
+  }
+
+  return result;
+}
+
+NS_IMETHODIMP
+LocationImpl::GetProtocol(nsAString& aProtocol)
+{
+  aProtocol.SetLength(0);
+
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = NS_OK;
+
+  result = GetURI(getter_AddRefs(uri));
+
+  if (uri) {
+    nsCAutoString protocol;
+
+    result = uri->GetScheme(protocol);
+
+    if (NS_SUCCEEDED(result)) {
+      aProtocol.Assign(NS_ConvertASCIItoUCS2(protocol));
+      aProtocol.Append(PRUnichar(':'));
+    }
+  }
+
+  return result;
+}
+
+NS_IMETHODIMP
+LocationImpl::SetProtocol(const nsAString& aProtocol)
+{
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = NS_OK;
+
+  result = GetWritableURI(getter_AddRefs(uri));
+
+  if (uri) {
+    uri->SetScheme(NS_ConvertUCS2toUTF8(aProtocol));
+    SetURI(uri);
+  }
+
+  return result;
+}
+
+NS_IMETHODIMP
+LocationImpl::GetSearch(nsAString& aSearch)
+{
+  aSearch.SetLength(0);
+
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = NS_OK;
+
+  result = GetURI(getter_AddRefs(uri));
+
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+
+  if (url) {
+    nsCAutoString search;
+
+    result = url->GetQuery(search);
+
+    if (NS_SUCCEEDED(result) && !search.IsEmpty()) {
+      aSearch.Assign(NS_LITERAL_STRING("?") + NS_ConvertUTF8toUCS2(search));
+    }
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-LocationImpl::SetSearch(const nsAReadableString& aSearch)
+LocationImpl::SetSearch(const nsAString& aSearch)
 {
-  nsAutoString href;
+  nsCOMPtr<nsIURI> uri;
   nsresult result = NS_OK;
 
-  result = GetHref(href);
+  result = GetWritableURI(getter_AddRefs(uri));
 
-  if (NS_SUCCEEDED(result)) {
-    nsCOMPtr<nsIURI> uri;
-
-    result = NS_NewURI(getter_AddRefs(uri), href);
-
-    if (uri) {
-      nsCOMPtr<nsIURL> url(do_QueryInterface(uri, &result));
-
-      if (url) {
-        result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch).get());
-
-        SetURL(uri);
-      }
-    }
+  nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
+  if (url) {
+    result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch));
+    SetURI(uri);
   }
 
   return result;
@@ -869,7 +806,7 @@ LocationImpl::Reload()
 }
 
 NS_IMETHODIMP
-LocationImpl::Replace(const nsAReadableString& aUrl)
+LocationImpl::Replace(const nsAString& aUrl)
 {
   nsresult rv = NS_OK;
 
@@ -901,7 +838,7 @@ LocationImpl::Replace(const nsAReadableString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::Assign(const nsAReadableString& aUrl)
+LocationImpl::Assign(const nsAString& aUrl)
 {
   nsAutoString oldHref;
   nsresult result = NS_OK;
@@ -922,7 +859,7 @@ LocationImpl::Assign(const nsAReadableString& aUrl)
 }
 
 NS_IMETHODIMP
-LocationImpl::ToString(nsAWritableString& aReturn)
+LocationImpl::ToString(nsAString& aReturn)
 {
   return GetHref(aReturn);
 }
