@@ -60,9 +60,12 @@
 #include "nsIByteArrayInputStream.h"
 #include "nsIWindowMediator.h"
 #include "nsIDOMWindowInternal.h"
+#include "nsIDOMDocument.h"
 #include "nsIJSConsoleService.h"
 #include "nsIConsoleService.h"
 #include "nsXPIDLString.h"
+#include "prprf.h"
+#include "nsEscape.h"
 
 static NS_DEFINE_CID(kSimpleURICID, NS_SIMPLEURI_CID);
 static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
@@ -124,18 +127,26 @@ nsresult nsJSThunk::EvaluateScript()
     NS_ENSURE_ARG_POINTER(mChannel);
 
     // Get the script string to evaluate...
-    nsXPIDLCString script;
-    rv = mURI->GetPath(getter_Copies(script));
+    nsCAutoString script;
+    rv = mURI->GetPath(script);
     if (NS_FAILED(rv)) return rv;
 
     // If mURI is just "javascript:", we bring up the JavaScript console
     // and return NS_ERROR_DOM_RETVAL_UNDEFINED.
-    if (*((const char*)script) == '\0') {
+    if (script.IsEmpty()) {
         rv = BringUpConsole();
         if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
         return NS_ERROR_DOM_RETVAL_UNDEFINED;
     }
 
+    // Unescape the script
+    NS_UnescapeURL(script);
+
+    // Get the url.
+    nsCAutoString url;
+    rv = mURI->GetSpec(url);
+    if (NS_FAILED(rv)) return rv;
+
     // Get an interface requestor from the channel callbacks.
     nsCOMPtr<nsIInterfaceRequestor> callbacks;
     rv = mChannel->GetNotificationCallbacks(getter_AddRefs(callbacks));
@@ -169,6 +180,19 @@ nsresult nsJSThunk::EvaluateScript()
         return NS_ERROR_FAILURE;
     }
 
+    // Now get the DOM Document.  Accessing the document will create one
+    // if necessary.  So, basically, this call ensures that a document gets
+    // created -- if necessary.
+    nsCOMPtr<nsIDOMWindow> domWindow(do_QueryInterface(global, &rv));
+    if (domWindow) {
+        nsCOMPtr<nsIDOMDocument> doc;
+
+        rv = domWindow->GetDocument(getter_AddRefs(doc));
+        NS_ASSERTION(doc, "No DOMDocument!");
+    }
+    if (NS_FAILED(rv)) {
+        return NS_ERROR_FAILURE;
+    }
 
     nsCOMPtr<nsIScriptContext> scriptContext;
     rv = global->GetContext(getter_AddRefs(scriptContext));
@@ -253,8 +277,8 @@ nsresult nsJSThunk::EvaluateScript()
         rv = scriptContext->EvaluateString(scriptString,
                                            nsnull,      // obj
                                            principal,
-                                           nsnull,      // url
-                                           0,           // line no
+                                           url.get(),   // url
+                                           1,           // line no
                                            nsnull,
                                            result,
                                            &bIsUndefined);
@@ -329,7 +353,13 @@ nsresult nsJSThunk::BringUpConsole()
 // nsIStreamIO implementation...
 //
 NS_IMETHODIMP
-nsJSThunk::Open(char* *contentType, PRInt32 *contentLength)
+nsJSThunk::Open()
+{
+    return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsJSThunk::GetContentType(nsACString &aContentType)
 {
     //
     // At this point the script has already been evaluated...
@@ -337,9 +367,22 @@ nsJSThunk::Open(char* *contentType, PRInt32 *contentLength)
     //
     // If the resultant script evaluation actually does return a value, we
     // treat it as html.
-    *contentType = nsCRT::strdup("text/html");
-    *contentLength = mLength;
+    //
+    aContentType = NS_LITERAL_CSTRING("text/html");
+    return NS_OK;
+}
 
+NS_IMETHODIMP
+nsJSThunk::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset.Truncate();
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsJSThunk::GetContentLength(PRInt32 *result)
+{
+    *result = mLength;
     return NS_OK;
 }
 
@@ -381,7 +424,7 @@ nsJSThunk::GetOutputStream(nsIOutputStream* *aOutputStream)
 }
 
 NS_IMETHODIMP
-nsJSThunk::GetName(char* *aName)
+nsJSThunk::GetName(nsACString &aName)
 {
     return mURI->GetSpec(aName);
 }
@@ -471,7 +514,7 @@ NS_INTERFACE_MAP_END
 //
 
 NS_IMETHODIMP
-nsJSChannel::GetName(PRUnichar * *aResult)
+nsJSChannel::GetName(nsACString &aResult)
 {
     return mStreamChannel->GetName(aResult);
 }
@@ -652,17 +695,29 @@ nsJSChannel::GetSecurityInfo(nsISupports * *aSecurityInfo)
 }
 
 NS_IMETHODIMP
-nsJSChannel::GetContentType(char * *aContentType)
+nsJSChannel::GetContentType(nsACString &aContentType)
 {
     return mStreamChannel->GetContentType(aContentType);
 }
 
 NS_IMETHODIMP
-nsJSChannel::SetContentType(const char *aContentType)
+nsJSChannel::SetContentType(const nsACString &aContentType)
 {
     return mStreamChannel->SetContentType(aContentType);
 }
 
+NS_IMETHODIMP
+nsJSChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    return mStreamChannel->GetContentCharset(aContentCharset);
+}
+
+NS_IMETHODIMP
+nsJSChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    return mStreamChannel->SetContentCharset(aContentCharset);
+}
+
 NS_IMETHODIMP
 nsJSChannel::GetContentLength(PRInt32 *aContentLength)
 {
@@ -713,15 +768,64 @@ nsJSProtocolHandler::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
     return rv;
 }
 
+nsresult 
+nsJSProtocolHandler::EnsureUTF8Spec(const nsAFlatCString &aSpec, const char *aCharset, 
+                                    nsACString &aUTF8Spec)
+{
+  aUTF8Spec.Truncate();
+
+  // assume UTF-8 if the spec contains unescaped non ASCII
+  if (!nsCRT::IsAscii(aSpec.get())) 
+    return NS_OK;
+  
+  nsCAutoString unescapedSpec; 
+  NS_UnescapeURL(aSpec.get(), aSpec.Length(), 
+                 esc_OnlyNonASCII, unescapedSpec);
+
+  if (IsASCII(unescapedSpec))
+    return NS_OK;
+  
+  nsresult rv;
+  if (!mCharsetConverterManager) {
+    mCharsetConverterManager = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  nsCOMPtr<nsIAtom> charsetAtom;
+  rv = mCharsetConverterManager->GetCharsetAtom2(aCharset, getter_AddRefs(charsetAtom));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  if (mCharsetAtom != charsetAtom) {
+    rv = mCharsetConverterManager->GetUnicodeDecoder(charsetAtom, 
+                                                     getter_AddRefs(mUnicodeDecoder));
+    NS_ENSURE_SUCCESS(rv, rv);
+    mCharsetAtom = charsetAtom;
+  }
+
+  PRInt32 srcLen = unescapedSpec.Length();
+  PRInt32 dstLen;
+  rv = mUnicodeDecoder->GetMaxLength(unescapedSpec.get(), srcLen, &dstLen);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRUnichar *ustr = (PRUnichar *) nsMemory::Alloc(dstLen * sizeof(PRUnichar));
+  NS_ENSURE_TRUE(ustr, NS_ERROR_OUT_OF_MEMORY);
+
+  rv = mUnicodeDecoder->Convert(unescapedSpec.get(), &srcLen, ustr, &dstLen);
+  if (NS_SUCCEEDED(rv)) {
+    NS_ConvertUCS2toUTF8 rawUTF8Spec(ustr, dstLen);
+    NS_EscapeURL(rawUTF8Spec, esc_AlwaysCopy | esc_OnlyNonASCII, aUTF8Spec);
+  }
+  nsMemory::Free(ustr);
+
+  return rv;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // nsIProtocolHandler methods:
 
 NS_IMETHODIMP
-nsJSProtocolHandler::GetScheme(char* *result)
+nsJSProtocolHandler::GetScheme(nsACString &result)
 {
-    *result = nsCRT::strdup("javascript");
-    if (!*result)
-        return NS_ERROR_OUT_OF_MEMORY;
+    result = "javascript";
     return NS_OK;
 }
 
@@ -740,7 +844,9 @@ nsJSProtocolHandler::GetProtocolFlags(PRUint32 *result)
 }
 
 NS_IMETHODIMP
-nsJSProtocolHandler::NewURI(const char *aSpec, nsIURI *aBaseURI,
+nsJSProtocolHandler::NewURI(const nsACString &aSpec,
+                            const char *aCharset, // ignore charset info
+                            nsIURI *aBaseURI,
                             nsIURI **result)
 {
     nsresult rv;
@@ -760,7 +866,19 @@ nsJSProtocolHandler::NewURI(const char *aSpec, nsIURI *aBaseURI,
     if (NS_FAILED(rv))
         return rv;
 
-    rv = url->SetSpec((char*)aSpec);
+    if (!aCharset || !nsCRT::strcasecmp("UTF-8", aCharset))
+      rv = url->SetSpec(aSpec);
+    else {
+      nsCAutoString utf8Spec;
+      rv = EnsureUTF8Spec(PromiseFlatCString(aSpec), aCharset, utf8Spec);
+      if (NS_SUCCEEDED(rv)) {
+        if (utf8Spec.IsEmpty())
+          rv = url->SetSpec(aSpec);
+        else
+          rv = url->SetSpec(utf8Spec);
+      }
+    }
+
     if (NS_FAILED(rv)) {
         NS_RELEASE(url);
         return rv;
@@ -804,7 +922,7 @@ nsJSProtocolHandler::AllowPort(PRInt32 port, const char *scheme, PRBool *_retval
 
 ////////////////////////////////////////////////////////////////////////////////
 
-static nsModuleComponentInfo gJSModuleInfo[] = {
+static const nsModuleComponentInfo gJSModuleInfo[] = {
     { "JavaScript Protocol Handler",
       NS_JSPROTOCOLHANDLER_CID,
       NS_NETWORK_PROTOCOL_CONTRACTID_PREFIX "javascript",
