@@ -42,6 +42,7 @@
 #include "nsHTMLEditRules.h"
 #include "nsTextEditUtils.h"
 #include "nsHTMLEditUtils.h"
+#include "nsWSRunObject.h"
 
 #include "nsEditorEventListeners.h"
 
@@ -118,11 +119,6 @@
 #include "nsIPref.h"
 const PRUnichar nbsp = 160;
 
-// HACK - CID for NS_CTRANSITIONAL_DTD_CID so that we can get at transitional dtd
-#define NS_CTRANSITIONAL_DTD_CID \
-{ 0x4611d482, 0x960a, 0x11d4, { 0x8e, 0xb0, 0xb6, 0x17, 0x66, 0x1b, 0x6f, 0x7c } }
-
-
 static NS_DEFINE_CID(kCRangeCID,      NS_RANGE_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 
@@ -169,13 +165,13 @@ static nsCOMPtr<nsIDOMNode> GetTableParent(nsIDOMNode* aNode)
 }
 
 
-NS_IMETHODIMP nsHTMLEditor::LoadHTML(const nsAReadableString & aInString)
+NS_IMETHODIMP nsHTMLEditor::LoadHTML(const nsAString & aInString)
 {
   nsAutoString charset;
   return LoadHTMLWithCharset(aInString, charset);
 }
 
-NS_IMETHODIMP nsHTMLEditor::LoadHTMLWithCharset(const nsAReadableString & aInputString, const nsAReadableString & aCharset)
+NS_IMETHODIMP nsHTMLEditor::LoadHTMLWithCharset(const nsAString & aInputString, const nsAString & aCharset)
 {
   nsresult res = NS_OK;
   if (!mRules) return NS_ERROR_NOT_INITIALIZED;
@@ -248,48 +244,40 @@ NS_IMETHODIMP nsHTMLEditor::LoadHTMLWithCharset(const nsAReadableString & aInput
 }
 
 
-NS_IMETHODIMP nsHTMLEditor::InsertHTML(const nsAReadableString & aInString)
+NS_IMETHODIMP nsHTMLEditor::InsertHTML(const nsAString & aInString)
 {
   nsAutoString charset;
   return InsertHTMLWithCharset(aInString, charset);
 }
 
-nsresult nsHTMLEditor::InsertHTMLWithContext(const nsAReadableString & aInputString, const nsAReadableString & aContextStr, const nsAReadableString & aInfoStr)
+nsresult
+nsHTMLEditor::InsertHTMLWithContext(const nsAString & aInputString,
+                                    const nsAString & aContextStr,
+                                    const nsAString & aInfoStr)
 {
   nsAutoString charset;
-  return InsertHTMLWithCharsetAndContext(aInputString, charset, aContextStr, aInfoStr);
+  return InsertHTMLWithCharsetAndContext(aInputString, charset,
+                                         aContextStr, aInfoStr);
 }
 
 
-NS_IMETHODIMP nsHTMLEditor::InsertHTMLWithCharset(const nsAReadableString & aInputString, const nsAReadableString & aCharset)
+NS_IMETHODIMP
+nsHTMLEditor::InsertHTMLWithCharset(const nsAString & aInputString,
+                                    const nsAString & aCharset)
 {
-  return InsertHTMLWithCharsetAndContext(aInputString, aCharset, nsAutoString(), nsAutoString());
+  return InsertHTMLWithCharsetAndContext(aInputString, aCharset,
+                                         nsAutoString(), nsAutoString());
 }
 
 
-nsresult nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAReadableString & aInputString,
-                                                       const nsAReadableString & aCharset,
-                                                       const nsAReadableString & aContextStr,
-                                                       const nsAReadableString & aInfoStr)
+nsresult
+nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAString & aInputString,
+                                              const nsAString & aCharset,
+                                              const nsAString & aContextStr,
+                                              const nsAString & aInfoStr)
 {
   if (!mRules) return NS_ERROR_NOT_INITIALIZED;
 
-/* all this is unneeded: parser handles this for us
-
-  // First, make sure there are no return chars in the document.
-  // Bad things happen if you insert returns (instead of dom newlines, \n)
-  // into an editor document.
-  nsAutoString inputString (aInputString);  // hope this does copy-on-write
-
-  // Windows linebreaks: Map CRLF to LF:
-  inputString.ReplaceSubstring(NS_LITERAL_STRING("\r\n").get(),
-                               NS_LITERAL_STRING("\n").get());
- 
-  // Mac linebreaks: Map any remaining CR to LF:
-  inputString.ReplaceSubstring(NS_LITERAL_STRING("\r").get(),
-                               NS_LITERAL_STRING("\n").get());
-*/
-
   // force IME commit; set up rules sniffing and batching
   ForceCompositionEnd();
   nsAutoEditBatch beginBatching(this);
@@ -319,7 +307,6 @@ nsresult nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAReadableString &
                                             &rangeStartHint, &rangeEndHint);
   NS_ENSURE_SUCCESS(res, res);
 
-
   // make a list of what nodes in docFrag we need to move
   nsCOMPtr<nsISupportsArray> nodeList;
   res = CreateListOfNodesToPaste(fragmentAsNode, address_of(nodeList), rangeStartHint, rangeEndHint);
@@ -393,6 +380,20 @@ nsresult nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAReadableString &
     if (!parentNode) res = NS_ERROR_FAILURE;
     if (NS_FAILED(res)) return res;
     
+    // if there are any invisible br's after our insertion point, remove them.
+    // this is because if there is a br at end of what we paste, it will make
+    // the invisible br visible.
+    nsWSRunObject wsObj(this, parentNode, offsetOfNewNode);
+    if ( nsTextEditUtils::IsBreak(wsObj.mEndReasonNode) && 
+        !IsVisBreak(wsObj.mEndReasonNode) )
+    {
+      res = DeleteNode(wsObj.mEndReasonNode);
+      if (NS_FAILED(res)) return res;
+    }
+    
+    // remeber if we are in a link.  
+    PRBool bStartedInLink = IsInLink(parentNode);
+  
     // are we in a text node?  If so, split it.
     if (IsTextNode(parentNode))
     {
@@ -603,9 +604,96 @@ nsresult nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAReadableString &
     // Now collapse the selection to the end of what we just inserted:
     if (lastInsertNode) 
     {
-      res = GetNodeLocation(lastInsertNode, address_of(parentNode), &offsetOfNewNode);
-      NS_ENSURE_SUCCESS(res, res);
-      selection->Collapse(parentNode, offsetOfNewNode+1);
+      // set selection to the end of what we just pasted.
+      nsCOMPtr<nsIDOMNode> selNode, tmp, visNode, highTable;
+      PRInt32 selOffset;
+      
+      // but dont cross tables
+      if (!nsHTMLEditUtils::IsTable(lastInsertNode))
+      {
+        res = GetLastEditableLeaf(lastInsertNode, address_of(selNode));
+        if (NS_FAILED(res)) return res;
+        tmp = selNode;
+        while (tmp && (tmp != lastInsertNode))
+        {
+          if (nsHTMLEditUtils::IsTable(tmp))
+            highTable = tmp;
+          nsCOMPtr<nsIDOMNode> parent = tmp;
+          tmp->GetParentNode(getter_AddRefs(parent));
+          tmp = parent;
+        }
+        if (highTable)
+          selNode = highTable;
+      }
+      if (!selNode) 
+        selNode = lastInsertNode;
+      if (IsTextNode(selNode) || (IsContainer(selNode) && !nsHTMLEditUtils::IsTable(selNode)))  
+      {
+        res = GetLengthOfDOMNode(selNode, (PRUint32&)selOffset);
+        if (NS_FAILED(res)) return res;
+      }
+      else // we need to find a container for selection.  Look up.
+      {
+        tmp = selNode;
+        res = GetNodeLocation(tmp, address_of(selNode), &selOffset);
+        ++selOffset;  // want to be *after* last leaf node in paste
+        if (NS_FAILED(res)) return res;
+      }
+      
+      // make sure we dont end up with selection collapsed after an invisible break node
+      nsWSRunObject wsRunObj(this, selNode, selOffset);
+      PRInt32 outVisOffset=0;
+      PRInt16 visType=0;
+      res = wsRunObj.PriorVisibleNode(selNode, selOffset, address_of(visNode), &outVisOffset, &visType);
+      if (NS_FAILED(res)) return res;
+      if (visType == nsWSRunObject::eBreak)
+      {
+        // we are after a break.  Is it visible?  Despite the name, 
+        // PriorVisibleNode does not make that determination for breaks.
+        // It also may not return the break in visNode.  We have to pull it
+        // out of the nsWSRunObject's state.
+        if (!IsVisBreak(wsRunObj.mStartReasonNode))
+        {
+          // dont leave selection past an invisible break;
+          // reset {selNode,selOffset} to point before break
+          res = GetNodeLocation(wsRunObj.mStartReasonNode, address_of(selNode), &selOffset);
+          // we want to be inside any inline style prior to break
+          nsWSRunObject wsRunObj(this, selNode, selOffset);
+          res = wsRunObj.PriorVisibleNode(selNode, selOffset, address_of(visNode), &outVisOffset, &visType);
+          if (NS_FAILED(res)) return res;
+          if (visType == nsWSRunObject::eText ||
+              visType == nsWSRunObject::eNormalWS)
+          {
+            selNode = visNode;
+            selOffset = outVisOffset;  // PriorVisibleNode already set offset to _after_ the text or ws
+          }
+          else if (visType == nsWSRunObject::eSpecial)
+          {
+            // prior visible thing is an image or some other non-text thingy.  
+            // We want to be right after it.
+            res = GetNodeLocation(wsRunObj.mStartReasonNode, address_of(selNode), &selOffset);
+            ++selOffset;
+          }
+        }
+      }
+      selection->Collapse(selNode, selOffset);
+      
+      // if we just pasted a link, discontinue link style
+      nsCOMPtr<nsIDOMNode> link;
+      if (!bStartedInLink && IsInLink(selNode, address_of(link)))
+      {
+        // so, if we just pasted a link, I split it.  Why do that instead of just
+        // nudging selection point beyond it?  Because it might have ended in a BR
+        // that is not visible.  If so, the code above just placed selection
+        // inside that.  So I split it instead.
+        nsCOMPtr<nsIDOMNode> leftLink;
+        PRInt32 linkOffset;
+        res = SplitNodeDeep(link, selNode, selOffset, &linkOffset, PR_TRUE, address_of(leftLink));
+        if (NS_FAILED(res)) return res;
+        res = GetNodeLocation(leftLink, address_of(selNode), &selOffset);
+        if (NS_FAILED(res)) return res;
+        selection->Collapse(selNode, selOffset+1);
+      }
     }
   }
   
@@ -613,8 +701,32 @@ nsresult nsHTMLEditor::InsertHTMLWithCharsetAndContext(const nsAReadableString &
   return res;
 }
 
+
+PRBool
+nsHTMLEditor::IsInLink(nsIDOMNode *aNode, nsCOMPtr<nsIDOMNode> *outLink)
+{
+  if (!aNode) 
+    return PR_FALSE;
+  if (outLink)
+    *outLink = nsnull;
+  nsCOMPtr<nsIDOMNode> tmp, node = aNode;
+  while (node)
+  {
+    if (nsHTMLEditUtils::IsLink(node)) 
+    {
+      if (outLink)
+        *outLink = node;
+      return PR_TRUE;
+    }
+    tmp = node;
+    tmp->GetParentNode(getter_AddRefs(node));
+  }
+  return PR_FALSE;
+}
+
+
 nsresult
-nsHTMLEditor::StripFormattingNodes(nsIDOMNode *aNode)
+nsHTMLEditor::StripFormattingNodes(nsIDOMNode *aNode, PRBool aListOnly)
 {
   NS_ENSURE_TRUE(aNode, NS_ERROR_NULL_POINTER);
 
@@ -626,7 +738,8 @@ nsHTMLEditor::StripFormattingNodes(nsIDOMNode *aNode)
     aNode->GetParentNode(getter_AddRefs(parent));
     if (parent)
     {
-      res = parent->RemoveChild(aNode, getter_AddRefs(ignored));
+      if (!aListOnly || nsHTMLEditUtils::IsList(parent))
+        res = parent->RemoveChild(aNode, getter_AddRefs(ignored));
       return res;
     }
   }
@@ -640,7 +753,7 @@ nsHTMLEditor::StripFormattingNodes(nsIDOMNode *aNode)
     {
       nsCOMPtr<nsIDOMNode> tmp;
       child->GetPreviousSibling(getter_AddRefs(tmp));
-      res = StripFormattingNodes(child);
+      res = StripFormattingNodes(child, aListOnly);
       NS_ENSURE_SUCCESS(res, res);
       child = tmp;
     }
@@ -664,9 +777,9 @@ NS_IMETHODIMP nsHTMLEditor::PrepareTransferable(nsITransferable **transferable)
     // we want to get out of the transferable
     if ((mFlags & eEditorPlaintextMask) == 0)  // This should only happen in html editors, not plaintext
     {
-      (*transferable)->AddDataFlavor(kJPEGImageMime);
       (*transferable)->AddDataFlavor(kHTMLMime);
       (*transferable)->AddDataFlavor(kFileMime);
+      //(*transferable)->AddDataFlavor(kJPEGImageMime);
     }
     (*transferable)->AddDataFlavor(kUnicodeMime);
   }
@@ -675,8 +788,8 @@ NS_IMETHODIMP nsHTMLEditor::PrepareTransferable(nsITransferable **transferable)
 }
 
 NS_IMETHODIMP nsHTMLEditor::InsertFromTransferable(nsITransferable *transferable, 
-                                                   const nsAReadableString & aContextStr,
-                                                   const nsAReadableString & aInfoStr)
+                                                   const nsAString & aContextStr,
+                                                   const nsAString & aInfoStr)
 {
   nsresult rv = NS_OK;
   char* bestFlavor = nsnull;
@@ -737,43 +850,40 @@ NS_IMETHODIMP nsHTMLEditor::InsertFromTransferable(nsITransferable *transferable
         if ( fileURL )
         {
           PRBool insertAsImage = PR_FALSE;
-          char *fileextension = nsnull;
-          rv = fileURL->GetFileExtension( &fileextension );
-          if ( NS_SUCCEEDED(rv) && fileextension )
+          nsCAutoString fileextension;
+          rv = fileURL->GetFileExtension( fileextension );
+          if ( NS_SUCCEEDED(rv) && !fileextension.IsEmpty() )
           {
-            if ( (nsCRT::strcasecmp( fileextension, "jpg" ) == 0 )
-              || (nsCRT::strcasecmp( fileextension, "jpeg" ) == 0 )
-              || (nsCRT::strcasecmp( fileextension, "gif" ) == 0 )
-              || (nsCRT::strcasecmp( fileextension, "png" ) == 0 ) )
+            if ( (nsCRT::strcasecmp( fileextension.get(), "jpg" ) == 0 )
+              || (nsCRT::strcasecmp( fileextension.get(), "jpeg" ) == 0 )
+              || (nsCRT::strcasecmp( fileextension.get(), "gif" ) == 0 )
+              || (nsCRT::strcasecmp( fileextension.get(), "png" ) == 0 ) )
             {
               insertAsImage = PR_TRUE;
             }
           }
-          if (fileextension) nsCRT::free(fileextension);
           
-          char *urltext = nsnull;
-          rv = fileURL->GetSpec( &urltext );
-          if ( NS_SUCCEEDED(rv) && urltext && urltext[0] != 0)
+          nsCAutoString urltext;
+          rv = fileURL->GetSpec( urltext );
+          if ( NS_SUCCEEDED(rv) && !urltext.IsEmpty() )
           {
-            len = strlen(urltext);
             if ( insertAsImage )
             {
               stuffToPaste.Assign(NS_LITERAL_STRING("<IMG src=\""));
-              stuffToPaste.AppendWithConversion ( urltext, len );
+              stuffToPaste.Append(NS_ConvertUTF8toUCS2(urltext));
               stuffToPaste.Append(NS_LITERAL_STRING("\" alt=\"\" >"));
             }
             else /* insert as link */
             {
               stuffToPaste.Assign(NS_LITERAL_STRING("<A href=\""));
-              stuffToPaste.AppendWithConversion ( urltext, len );
+              stuffToPaste.Append(NS_ConvertUTF8toUCS2(urltext));
               stuffToPaste.Append(NS_LITERAL_STRING("\">"));
-              stuffToPaste.AppendWithConversion ( urltext, len );
+              stuffToPaste.Append(NS_ConvertUTF8toUCS2(urltext));
               stuffToPaste.Append(NS_LITERAL_STRING("</A>"));
             }
             nsAutoEditBatch beginBatching(this);
             rv = InsertHTML(stuffToPaste);
           }
-          if (urltext) nsCRT::free(urltext);
         }
       }
     }
@@ -790,11 +900,7 @@ NS_IMETHODIMP nsHTMLEditor::InsertFromTransferable(nsITransferable *transferable
       
   // Try to scroll the selection into view if the paste/drop succeeded
   if (NS_SUCCEEDED(rv))
-  {
-    nsCOMPtr<nsISelectionController> selCon;
-    if (NS_SUCCEEDED(GetSelectionController(getter_AddRefs(selCon))) && selCon)
-      selCon->ScrollSelectionIntoView(nsISelectionController::SELECTION_NORMAL, nsISelectionController::SELECTION_FOCUS_REGION);
-  }
+    ScrollSelectionIntoView(PR_FALSE);
 
   return rv;
 }
@@ -1107,7 +1213,7 @@ NS_IMETHODIMP nsHTMLEditor::DoDrag(nsIDOMEvent *aDragEvent)
   nsCOMPtr<nsIDOMDocument> domdoc;
   rv = GetDocument(getter_AddRefs(domdoc));
   if (NS_FAILED(rv)) return rv;
-	
+  
   nsCOMPtr<nsIDocument> doc = do_QueryInterface(domdoc);
   if (doc)
   {
@@ -1318,7 +1424,7 @@ NS_IMETHODIMP nsHTMLEditor::CanPaste(PRInt32 aSelectionType, PRBool *aCanPaste)
   
   // the flavors that we can deal with
   const char* const textEditorFlavors[] = { kUnicodeMime, nsnull };
-  const char* const htmlEditorFlavors[] = { kJPEGImageMime, kHTMLMime, nsnull };
+  const char* const htmlEditorFlavors[] = { kHTMLMime, kJPEGImageMime, nsnull };
 
   nsCOMPtr<nsISupportsArray> flavorsList;
   rv = nsComponentManager::CreateInstance(NS_SUPPORTSARRAY_CONTRACTID, nsnull, 
@@ -1380,7 +1486,7 @@ NS_IMETHODIMP nsHTMLEditor::PasteAsQuotation(PRInt32 aSelectionType)
   return PasteAsCitedQuotation(citation, aSelectionType);
 }
 
-NS_IMETHODIMP nsHTMLEditor::PasteAsCitedQuotation(const nsAReadableString & aCitation,
+NS_IMETHODIMP nsHTMLEditor::PasteAsCitedQuotation(const nsAString & aCitation,
                                                   PRInt32 aSelectionType)
 {
   nsAutoEditBatch beginBatching(this);
@@ -1490,7 +1596,7 @@ NS_IMETHODIMP nsHTMLEditor::PasteAsPlaintextQuotation(PRInt32 aSelectionType)
   return rv;
 }
 
-NS_IMETHODIMP nsHTMLEditor::InsertAsQuotation(const nsAReadableString & aQuotedText,
+NS_IMETHODIMP nsHTMLEditor::InsertAsQuotation(const nsAString & aQuotedText,
                                               nsIDOMNode **aNodeInserted)
 {
   if (mFlags & eEditorPlaintextMask)
@@ -1507,9 +1613,12 @@ NS_IMETHODIMP nsHTMLEditor::InsertAsQuotation(const nsAReadableString & aQuotedT
 // in that here, quoted material is enclosed in a <pre> tag
 // in order to preserve the original line wrapping.
 NS_IMETHODIMP
-nsHTMLEditor::InsertAsPlaintextQuotation(const nsAReadableString & aQuotedText,
+nsHTMLEditor::InsertAsPlaintextQuotation(const nsAString & aQuotedText,
                                          nsIDOMNode **aNodeInserted)
 {
+  if (mWrapToWindow)
+    return nsPlaintextEditor::InsertAsQuotation(aQuotedText, aNodeInserted);
+
   nsresult rv;
 
   // The quotesPreformatted pref is a temporary measure. See bug 69638.
@@ -1517,7 +1626,7 @@ nsHTMLEditor::InsertAsPlaintextQuotation(const nsAReadableString & aQuotedText,
   PRBool quotesInPre;
   nsCOMPtr<nsIPref> prefs = do_GetService(kPrefServiceCID, &rv);
   if (NS_SUCCEEDED(rv) && prefs)
-    rv = prefs->GetBoolPref("editor.quotesPreformatted", &quotesInPre);
+    prefs->GetBoolPref("editor.quotesPreformatted", &quotesInPre);
 
   nsCOMPtr<nsIDOMNode> preNode;
   // get selection
@@ -1601,10 +1710,10 @@ nsHTMLEditor::InsertAsPlaintextQuotation(const nsAReadableString & aQuotedText,
 }
 
 NS_IMETHODIMP
-nsHTMLEditor::InsertAsCitedQuotation(const nsAReadableString & aQuotedText,
-                                     const nsAReadableString & aCitation,
+nsHTMLEditor::InsertAsCitedQuotation(const nsAString & aQuotedText,
+                                     const nsAString & aCitation,
                                      PRBool aInsertHTML,
-                                     const nsAReadableString & aCharset,
+                                     const nsAString & aCharset,
                                      nsIDOMNode **aNodeInserted)
 {
   nsCOMPtr<nsIDOMNode> newNode;
@@ -1675,9 +1784,9 @@ nsHTMLEditor::InsertAsCitedQuotation(const nsAReadableString & aQuotedText,
 }
 
 nsresult nsHTMLEditor::CreateDOMFragmentFromPaste(nsIDOMNSRange *aNSRange,
-                                                  const nsAReadableString & aInputString,
-                                                  const nsAReadableString & aContextStr,
-                                                  const nsAReadableString & aInfoStr,
+                                                  const nsAString & aInputString,
+                                                  const nsAString & aContextStr,
+                                                  const nsAString & aInfoStr,
                                                   nsCOMPtr<nsIDOMNode> *outFragNode,
                                                   PRInt32 *outRangeStartHint,
                                                   PRInt32 *outRangeEndHint)
@@ -1721,8 +1830,6 @@ nsresult nsHTMLEditor::CreateDOMFragmentFromPaste(nsIDOMNSRange *aNSRange,
   res = aNSRange->CreateContextualFragment(aInputString, getter_AddRefs(docfrag));
   NS_ENSURE_SUCCESS(res, res);
   *outFragNode = do_QueryInterface(docfrag);
-  res = StripFormattingNodes(*outFragNode);
-  NS_ENSURE_SUCCESS(res, res);
   
   if (contextfrag)
   {
@@ -1733,15 +1840,18 @@ nsresult nsHTMLEditor::CreateDOMFragmentFromPaste(nsIDOMNSRange *aNSRange,
     // no longer have fragmentAsNode in tree
     contextDepth--;
   }
-  
+ 
+  res = StripFormattingNodes(*outFragNode, PR_TRUE);
+  NS_ENSURE_SUCCESS(res, res);
+ 
   // get the infoString contents
   nsAutoString numstr1, numstr2;
   if (aInfoStr.Length())
   {
     PRInt32 err, sep;
     sep = aInfoStr.FindChar((PRUnichar)',');
-    aInfoStr.Left(numstr1, sep);
-    aInfoStr.Right(numstr2, aInfoStr.Length() - (sep+1));
+    numstr1 = Substring(aInfoStr, 0, sep);
+    numstr2 = Substring(aInfoStr, sep+1, aInfoStr.Length() - (sep+1));
     *outRangeStartHint = numstr1.ToInteger(&err) + contextDepth;
     *outRangeEndHint   = numstr2.ToInteger(&err) + contextDepth;
   }
@@ -1934,9 +2044,10 @@ nsHTMLEditor::ScanForListAndTableStructure( PRBool aEnd,
       else structureNode = GetTableParent(pNode);
       if (structureNode == aListOrTable)
       {
-        if (pNode == originalNode)
-          break;  // we are starting right off with a list item of the list
-        *outReplaceNode = pNode;
+        if (bList)
+          *outReplaceNode = structureNode;
+        else
+          *outReplaceNode = pNode;
         break;
       }
     }
