@@ -119,13 +119,13 @@ static const PRBool gNoisy = PR_FALSE;
 // Add the CSS style corresponding to the HTML inline style defined
 // by aProperty aAttribute and aValue to the selection
 NS_IMETHODIMP nsHTMLEditor::SetCSSInlineProperty(nsIAtom *aProperty, 
-                            const nsAReadableString & aAttribute, 
-                            const nsAReadableString & aValue)
+                            const nsAString & aAttribute, 
+                            const nsAString & aValue)
 {
   nsresult res = NS_OK;
   PRBool useCSS;
 
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
   if (useCSS) {
     res = SetInlineProperty(aProperty, aAttribute, aValue);
   }
@@ -133,8 +133,8 @@ NS_IMETHODIMP nsHTMLEditor::SetCSSInlineProperty(nsIAtom *aProperty,
 }
 
 NS_IMETHODIMP nsHTMLEditor::SetInlineProperty(nsIAtom *aProperty, 
-                            const nsAReadableString & aAttribute, 
-                            const nsAReadableString & aValue)
+                            const nsAString & aAttribute, 
+                            const nsAString & aValue)
 {
   if (!aProperty) { return NS_ERROR_NULL_POINTER; }
   if (!mRules) { return NS_ERROR_NOT_INITIALIZED; }
@@ -313,8 +313,8 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
                                             PRInt32 aStartOffset,
                                             PRInt32 aEndOffset,
                                             nsIAtom *aProperty, 
-                                            const nsAReadableString *aAttribute,
-                                            const nsAReadableString *aValue)
+                                            const nsAString *aAttribute,
+                                            const nsAString *aValue)
 {
   if (!aTextNode) return NS_ERROR_NULL_POINTER;
   nsresult res = NS_OK;
@@ -385,8 +385,8 @@ nsHTMLEditor::SetInlinePropertyOnTextNode( nsIDOMCharacterData *aTextNode,
 nsresult
 nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
                                        nsIAtom *aProperty, 
-                                       const nsAReadableString *aAttribute,
-                                       const nsAReadableString *aValue)
+                                       const nsAString *aAttribute,
+                                       const nsAString *aValue)
 {
   if (!aNode || !aProperty) return NS_ERROR_NULL_POINTER;
 
@@ -397,7 +397,7 @@ nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
   ToLowerCase(tag);
   
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   if (useCSS) {
     // we are in CSS mode
@@ -524,7 +524,7 @@ nsHTMLEditor::SetInlinePropertyOnNode( nsIDOMNode *aNode,
 
 nsresult nsHTMLEditor::SplitStyleAboveRange(nsIDOMRange *inRange, 
                                             nsIAtom *aProperty, 
-                                            const nsAReadableString *aAttribute)
+                                            const nsAString *aAttribute)
 {
   if (!inRange) return NS_ERROR_NULL_POINTER;
   nsresult res;
@@ -545,15 +545,12 @@ nsresult nsHTMLEditor::SplitStyleAboveRange(nsIDOMRange *inRange,
   PRBool sameNode = (startNode==endNode);
   
   // split any matching style nodes above the start of range
-  res = SplitStyleAbovePoint(address_of(startNode), &startOffset, aProperty, aAttribute);
-  if (NS_FAILED(res)) return res;
-
-  if (sameNode && (startNode != origStartNode))
   {
-    // our startNode got split.  This changes the offset of the end of our range.
-    endOffset -= origStartOffset;
+    nsAutoTrackDOMPoint tracker(mRangeUpdater, address_of(endNode), &endOffset);
+    res = SplitStyleAbovePoint(address_of(startNode), &startOffset, aProperty, aAttribute);
+    if (NS_FAILED(res)) return res;
   }
-  
+
   // second verse, same as the first...
   res = SplitStyleAbovePoint(address_of(endNode), &endOffset, aProperty, aAttribute);
   if (NS_FAILED(res)) return res;
@@ -568,7 +565,7 @@ nsresult nsHTMLEditor::SplitStyleAboveRange(nsIDOMRange *inRange,
 nsresult nsHTMLEditor::SplitStyleAbovePoint(nsCOMPtr<nsIDOMNode> *aNode,
                                            PRInt32 *aOffset,
                                            nsIAtom *aProperty,          // null here means we split all properties
-                                           const nsAReadableString *aAttribute,
+                                           const nsAString *aAttribute,
                                            nsCOMPtr<nsIDOMNode> *outLeftNode,
                                            nsCOMPtr<nsIDOMNode> *outRightNode)
 {
@@ -580,7 +577,7 @@ nsresult nsHTMLEditor::SplitStyleAbovePoint(nsCOMPtr<nsIDOMNode> *aNode,
   PRInt32 offset;
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   PRBool isSet;
   while (tmp && !IsBlockNode(tmp))
@@ -624,7 +621,7 @@ PRBool nsHTMLEditor::NodeIsProperty(nsIDOMNode *aNode)
 
 nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode, 
                                    nsIAtom *aProperty,   // null here means remove all properties
-                                   const nsAReadableString *aAttribute, 
+                                   const nsAString *aAttribute, 
                                    PRBool aChildrenOnly)
 {
   if (!aNode) return NS_ERROR_NULL_POINTER;
@@ -646,7 +643,8 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
   // then process the node itself
   if ( !aChildrenOnly && 
         (aProperty && NodeIsType(aNode, aProperty) || // node is prop we asked for
-        (aProperty == nsIEditProperty::href && nsHTMLEditUtils::IsLink(aNode))) || // but check for link (<a href=...)
+        (aProperty == nsIEditProperty::href && nsHTMLEditUtils::IsLink(aNode)) || // but check for link (<a href=...)
+        (aProperty == nsIEditProperty::name && nsHTMLEditUtils::IsNamedAnchor(aNode))) || // and for named anchors
         (!aProperty && NodeIsProperty(aNode)))  // or node is any prop and we asked for that
   {
     // if we weren't passed an attribute, then we want to 
@@ -677,7 +675,7 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
   }
   else {
     PRBool useCSS;
-    IsCSSEnabled(&useCSS);
+    GetIsCSSEnabled(&useCSS);
 
     if (!aChildrenOnly
         && useCSS && mHTMLCSSUtils->IsCSSEditableProperty(aNode, aProperty, aAttribute)) {
@@ -697,25 +695,14 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
                                                       &propertyValue);
         // remove the node if it is a span, if its style attribute is empty or absent,
         // and if it does not have a class nor an id
-        nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(aNode);
-        nsAutoString styleVal;
-        PRBool isStyleSet;
-        res = GetAttributeValue(elem,  NS_LITERAL_STRING("style"), styleVal, &isStyleSet);
-        if (NS_FAILED(res)) return res;
-        if (NodeIsType(aNode, nsIEditProperty::span) && (!isStyleSet || (0 == styleVal.Length()))) {
-          PRBool hasClassOrId ;
-          res = mHTMLCSSUtils->HasClassOrID(elem, hasClassOrId);
-          if (!hasClassOrId) {
-            res = RemoveContainer(aNode);
-            if (NS_FAILED(res)) return res;
-          }
-        }
+        nsCOMPtr<nsIDOMElement> element = do_QueryInterface(aNode);
+        res = RemoveElementIfNoStyleOrIdOrClass(element, nsIEditProperty::span);
       }
     }
   }  
   if ( aProperty == nsIEditProperty::font &&    // or node is big or small and we are setting font size
        (NodeIsType(aNode, nsIEditProperty::big) || NodeIsType(aNode, nsIEditProperty::small)) &&
-       !Compare(*aAttribute,NS_LITERAL_STRING("size"),nsCaseInsensitiveStringComparator()))       
+       aAttribute->Equals(NS_LITERAL_STRING("size"),nsCaseInsensitiveStringComparator()))       
   {
     res = RemoveContainer(aNode);  // if we are setting font size, remove any nested bigs and smalls
   }
@@ -723,7 +710,7 @@ nsresult nsHTMLEditor::RemoveStyleInside(nsIDOMNode *aNode,
 }
 
 PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode, 
-                                     const nsAReadableString *aAttribute)
+                                     const nsAString *aAttribute)
 {
   if (!aNode || !aAttribute) return PR_FALSE;  // ooops
   nsCOMPtr<nsIContent> content = do_QueryInterface(aNode);
@@ -741,10 +728,10 @@ PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode,
     if (!attrName) continue;  // ooops
     attrName->ToString(attrString);
     // if it's the attribute we know about, keep looking
-    if (!Compare(attrString,*aAttribute,nsCaseInsensitiveStringComparator())) continue;
+    if (attrString.Equals(*aAttribute,nsCaseInsensitiveStringComparator())) continue;
     // if it's a special _moz... attribute, keep looking
     attrString.Left(tmp,4);
-    if (!Compare(tmp,NS_LITERAL_STRING("_moz"),nsCaseInsensitiveStringComparator())) continue;
+    if (tmp.Equals(NS_LITERAL_STRING("_moz"),nsCaseInsensitiveStringComparator())) continue;
     // otherwise, it's another attribute, so return false
     return PR_FALSE;
   }
@@ -754,7 +741,7 @@ PRBool nsHTMLEditor::IsOnlyAttribute(nsIDOMNode *aNode,
 }
 
 PRBool nsHTMLEditor::HasAttr(nsIDOMNode *aNode, 
-                             const nsAReadableString *aAttribute)
+                             const nsAString *aAttribute)
 {
   if (!aNode) return PR_FALSE;
   if (!aAttribute || aAttribute->IsEmpty()) return PR_TRUE;  // everybody has the 'null' attribute
@@ -772,8 +759,8 @@ PRBool nsHTMLEditor::HasAttr(nsIDOMNode *aNode,
 
 
 PRBool nsHTMLEditor::HasAttrVal(nsIDOMNode *aNode, 
-                                const nsAReadableString *aAttribute, 
-                                const nsAReadableString *aValue)
+                                const nsAString *aAttribute, 
+                                const nsAString *aValue)
 {
   if (!aNode) return PR_FALSE;
   if (!aAttribute || aAttribute->IsEmpty()) return PR_TRUE;  // everybody has the 'null' attribute
@@ -798,10 +785,67 @@ PRBool nsHTMLEditor::HasAttrVal(nsIDOMNode *aNode,
   attNode->GetValue(attrVal);
   
   // do values match?
-  if (!Compare(attrVal,*aValue,nsCaseInsensitiveStringComparator())) return PR_TRUE;
+  if (attrVal.Equals(*aValue,nsCaseInsensitiveStringComparator())) return PR_TRUE;
   return PR_FALSE;
 }
 
+nsresult nsHTMLEditor::PromoteRangeIfStartsOrEndsInNamedAnchor(nsIDOMRange *inRange)
+{
+  if (!inRange) return NS_ERROR_NULL_POINTER;
+  nsresult res;
+  nsCOMPtr<nsIDOMNode> startNode, endNode, parent, tmp;
+  PRInt32 startOffset, endOffset, tmpOffset;
+  
+  res = inRange->GetStartContainer(getter_AddRefs(startNode));
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetStartOffset(&startOffset);
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetEndContainer(getter_AddRefs(endNode));
+  if (NS_FAILED(res)) return res;
+  res = inRange->GetEndOffset(&endOffset);
+  if (NS_FAILED(res)) return res;
+
+  tmp = startNode;
+  while ( tmp && 
+          !nsTextEditUtils::IsBody(tmp) &&
+          !nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    tmp = parent;
+  }
+  if (!tmp) return NS_ERROR_NULL_POINTER;
+  if (nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    startNode = parent;
+    startOffset = tmpOffset;
+  }
+
+  tmp = endNode;
+  while ( tmp && 
+          !nsTextEditUtils::IsBody(tmp) &&
+          !nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    tmp = parent;
+  }
+  if (!tmp) return NS_ERROR_NULL_POINTER;
+  if (nsHTMLEditUtils::IsNamedAnchor(tmp))
+  {
+    res = GetNodeLocation(tmp, address_of(parent), &tmpOffset);
+    if (NS_FAILED(res)) return res;
+    endNode = parent;
+    endOffset = tmpOffset + 1;
+  }
+
+  res = inRange->SetStart(startNode, startOffset);
+  if (NS_FAILED(res)) return res;
+  res = inRange->SetEnd(endNode, endOffset);
+  return res;
+}
 
 nsresult nsHTMLEditor::PromoteInlineRange(nsIDOMRange *inRange)
 {
@@ -893,12 +937,12 @@ PRBool nsHTMLEditor::IsAtEndOfNode(nsIDOMNode *aNode, PRInt32 aOffset)
 
 nsresult
 nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty, 
-                             const nsAReadableString *aAttribute,
-                             const nsAReadableString *aValue,
+                             const nsAString *aAttribute,
+                             const nsAString *aValue,
                              PRBool *aFirst, 
                              PRBool *aAny, 
                              PRBool *aAll,
-                             nsAWritableString *outValue)
+                             nsAString *outValue)
 {
   if (!aProperty)
     return NS_ERROR_NULL_POINTER;
@@ -919,7 +963,7 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
   PRBool first=PR_TRUE;
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   nsCOMPtr<nsISelection>selection;
   result = GetSelection(getter_AddRefs(selection));
@@ -970,8 +1014,8 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
       }
       if (!useCSS) {
         nsCOMPtr<nsIDOMNode> resultNode;
-        IsTextPropertySetByContent(collapsedNode, aProperty, 0, 0,
-                                   isSet, getter_AddRefs(resultNode));
+        IsTextPropertySetByContent(collapsedNode, aProperty, aAttribute, aValue,
+                                   isSet, getter_AddRefs(resultNode), outValue);
         *aFirst = *aAny = *aAll = isSet;
         return NS_OK;
       }
@@ -986,6 +1030,12 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
     nsCOMPtr<nsIContent> content;
     nsAutoString firstValue, theValue;
     iter->CurrentNode(getter_AddRefs(content));
+    nsCOMPtr<nsIDOMNode> endNode;
+    PRInt32 endOffset;
+    result = range->GetEndContainer(getter_AddRefs(endNode));
+    if (NS_FAILED(result)) return result;
+    result = range->GetEndOffset(&endOffset);
+    if (NS_FAILED(result)) return result;
     while (NS_ENUMERATOR_FALSE == iter->IsDone())
     {
       nsCOMPtr<nsIDOMNode> node = do_QueryInterface(content);
@@ -1019,6 +1069,10 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
             skipNode = PR_TRUE;
           }
         }
+        else if (node == endNode && !endOffset)
+        {
+          skipNode = PR_TRUE;
+        }
       }
       else
       { // handle non-text leaf nodes here
@@ -1101,18 +1155,18 @@ nsHTMLEditor::GetInlinePropertyBase(nsIAtom *aProperty,
 
 
 NS_IMETHODIMP nsHTMLEditor::GetInlineProperty(nsIAtom *aProperty, 
-                                              const nsAReadableString &aAttribute, 
-                                              const nsAReadableString &aValue,
+                                              const nsAString &aAttribute, 
+                                              const nsAString &aValue,
                                               PRBool *aFirst, 
                                               PRBool *aAny, 
                                               PRBool *aAll)
 {
   if (!aProperty || !aFirst || !aAny || !aAll)
     return NS_ERROR_NULL_POINTER;
-  const nsAReadableString *att = nsnull;
+  const nsAString *att = nsnull;
   if (aAttribute.Length())
     att = &aAttribute;
-  const nsAReadableString *val = nsnull;
+  const nsAString *val = nsnull;
   if (aValue.Length())
     val = &aValue;
   return GetInlinePropertyBase( aProperty, att, val, aFirst, aAny, aAll, nsnull);
@@ -1120,19 +1174,19 @@ NS_IMETHODIMP nsHTMLEditor::GetInlineProperty(nsIAtom *aProperty,
 
 
 NS_IMETHODIMP nsHTMLEditor::GetInlinePropertyWithAttrValue(nsIAtom *aProperty, 
-                                              const nsAReadableString &aAttribute, 
-                                              const nsAReadableString &aValue,
+                                              const nsAString &aAttribute, 
+                                              const nsAString &aValue,
                                               PRBool *aFirst, 
                                               PRBool *aAny, 
                                               PRBool *aAll,
-                                              nsAWritableString &outValue)
+                                              nsAString &outValue)
 {
   if (!aProperty || !aFirst || !aAny || !aAll)
     return NS_ERROR_NULL_POINTER;
-  const nsAReadableString *att = nsnull;
+  const nsAString *att = nsnull;
   if (aAttribute.Length())
     att = &aAttribute;
-  const nsAReadableString *val = nsnull;
+  const nsAString *val = nsnull;
   if (aValue.Length())
     val = &aValue;
   return GetInlinePropertyBase( aProperty, att, val, aFirst, aAny, aAll, &outValue);
@@ -1144,12 +1198,12 @@ NS_IMETHODIMP nsHTMLEditor::RemoveAllInlineProperties()
   return RemoveInlinePropertyImpl(nsnull, nsnull);
 }
 
-NS_IMETHODIMP nsHTMLEditor::RemoveInlineProperty(nsIAtom *aProperty, const nsAReadableString &aAttribute)
+NS_IMETHODIMP nsHTMLEditor::RemoveInlineProperty(nsIAtom *aProperty, const nsAString &aAttribute)
 {
   return RemoveInlinePropertyImpl(aProperty, &aAttribute);
 }
 
-nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAReadableString *aAttribute)
+nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsAString *aAttribute)
 {
   if (!mRules)    return NS_ERROR_NOT_INITIALIZED;
   ForceCompositionEnd();
@@ -1165,14 +1219,15 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsARea
   selection->GetIsCollapsed(&isCollapsed);
 
   PRBool useCSS;
-  IsCSSEnabled(&useCSS);
+  GetIsCSSEnabled(&useCSS);
 
   if (isCollapsed)
   {
     // manipulating text attributes on a collapsed selection only sets state for the next text insertion
 
     // For links, aProperty uses "href", use "a" instead
-    if (aProperty == nsIEditProperty::href)
+    if (aProperty == nsIEditProperty::href ||
+        aProperty == nsIEditProperty::name)
       aProperty = nsIEditProperty::a;
 
     if (aProperty) return mTypeInState->ClearProp(aProperty, nsAutoString(*aAttribute));
@@ -1206,15 +1261,23 @@ nsresult nsHTMLEditor::RemoveInlinePropertyImpl(nsIAtom *aProperty, const nsARea
       
       nsCOMPtr<nsIDOMRange> range( do_QueryInterface(currentItem) );
 
-      // adjust range to include any ancestors who's children are entirely selected
-      res = PromoteInlineRange(range);
+      if (aProperty == nsIEditProperty::name)
+      {
+        // promote range if it starts or end in a named anchor and we
+        // want to remove named anchors
+        res = PromoteRangeIfStartsOrEndsInNamedAnchor(range);
+      }
+      else {
+        // adjust range to include any ancestors who's children are entirely selected
+        res = PromoteInlineRange(range);
+      }
       if (NS_FAILED(res)) return res;
-      
+
       // remove this style from ancestors of our range endpoints, 
       // splitting them as appropriate
       res = SplitStyleAboveRange(range, aProperty, aAttribute);
       if (NS_FAILED(res)) return res;
-      
+
       // check for easy case: both range endpoints in same text node
       nsCOMPtr<nsIDOMNode> startNode, endNode;
       res = range->GetStartContainer(getter_AddRefs(startNode));
@@ -1738,7 +1801,7 @@ nsHTMLEditor::RelativeFontChangeOnNode( PRInt32 aSizeChange,
 }
 
 NS_IMETHODIMP 
-nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAWritableString &outFace)
+nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAString &outFace)
 {
   if (!aMixed)
       return NS_ERROR_FAILURE;
@@ -1777,7 +1840,7 @@ nsHTMLEditor::GetFontFaceState(PRBool *aMixed, nsAWritableString &outFace)
 }
 
 NS_IMETHODIMP 
-nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAWritableString &aOutColor)
+nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAString &aOutColor)
 {
   if (!aMixed)
       return NS_ERROR_NULL_POINTER;
@@ -1810,15 +1873,53 @@ nsHTMLEditor::GetFontColorState(PRBool *aMixed, nsAWritableString &aOutColor)
 // can handle CSS styles (for instance, Composer can, Messenger can't) and if
 // the CSS preference is checked
 nsresult
-nsHTMLEditor::IsCSSEnabled(PRBool *aIsSet)
+nsHTMLEditor::GetIsCSSEnabled(PRBool *aIsCSSEnabled)
 {
-  *aIsSet = PR_FALSE;
+  *aIsCSSEnabled = PR_FALSE;
   if (mCSSAware) {
     // TBD later : removal of mCSSAware and use only the presence of mHTMLCSSUtils
     if (mHTMLCSSUtils) {
-      *aIsSet = mHTMLCSSUtils->IsCSSPrefChecked();
+      *aIsCSSEnabled = mHTMLCSSUtils->IsCSSPrefChecked();
     }
   }
   return NS_OK;
 }
 
+nsresult
+nsHTMLEditor::HasStyleOrIdOrClass(nsIDOMElement * aElement, PRBool *aHasStyleOrIdOrClass)
+{
+  NS_ENSURE_TRUE(aElement, NS_ERROR_NULL_POINTER);
+  nsCOMPtr<nsIDOMNode> node  = do_QueryInterface(aElement);
+
+
+  // remove the node if its style attribute is empty or absent,
+  // and if it does not have a class nor an id
+  nsAutoString styleVal;
+  PRBool isStyleSet;
+  *aHasStyleOrIdOrClass = PR_TRUE;
+  nsresult res = GetAttributeValue(aElement,  NS_LITERAL_STRING("style"), styleVal, &isStyleSet);
+  if (NS_FAILED(res)) return res;
+  if (!isStyleSet || (0 == styleVal.Length())) {
+    res = mHTMLCSSUtils->HasClassOrID(aElement, *aHasStyleOrIdOrClass);
+    if (NS_FAILED(res)) return res;
+  }
+  return res;
+}
+
+nsresult
+nsHTMLEditor::RemoveElementIfNoStyleOrIdOrClass(nsIDOMElement * aElement, nsIAtom * aTag)
+{
+  NS_ENSURE_TRUE(aElement, NS_ERROR_NULL_POINTER);
+  nsCOMPtr<nsIDOMNode> node  = do_QueryInterface(aElement);
+
+  // early way out if node is not the right kind of element
+  if (!NodeIsType(node, aTag)) {
+    return NS_OK;
+  }
+  PRBool hasStyleOrIdOrClass;
+  nsresult res = HasStyleOrIdOrClass(aElement, &hasStyleOrIdOrClass);
+  if (!hasStyleOrIdOrClass) {
+    res = RemoveContainer(node);
+  }
+  return res;
+}
