@@ -55,6 +55,7 @@
 #include "nsLayoutCID.h"
 #include "nsIDOMCharacterData.h"
 #include "nsEditorUtils.h"
+#include "nsCRT.h"
 
 const PRUnichar nbsp = 160;
 
@@ -76,12 +77,15 @@ static PRBool IsInlineNode(nsIDOMNode* node)
 nsWSRunObject::nsWSRunObject(nsHTMLEditor *aEd, nsIDOMNode *aNode, PRInt32 aOffset) :
 mNode(aNode)
 ,mOffset(aOffset)
+,mPRE(PR_FALSE)
 ,mStartNode()
 ,mStartOffset(0)
 ,mStartReason(0)
+,mStartReasonNode()
 ,mEndNode()
 ,mEndOffset(0)
 ,mEndReason(0)
+,mEndReasonNode()
 ,mFirstNBSPNode()
 ,mFirstNBSPOffset(0)
 ,mLastNBSPNode()
@@ -218,63 +222,69 @@ nsWSRunObject::InsertBreak(nsCOMPtr<nsIDOMNode> *aInOutParent,
   res = FindRun(*aInOutParent, *aInOutOffset, &beforeRun, PR_FALSE);
   res = FindRun(*aInOutParent, *aInOutOffset, &afterRun, PR_TRUE);
   
-  // handle any changes needed to ws run after inserted br
-  if (!afterRun)
   {
-    // dont need to do anything.  just insert break.  ws wont change.
-  }
-  else if (afterRun->mType & eTrailingWS)
-  {
-    // dont need to do anything.  just insert break.  ws wont change.
-  }
-  else if (afterRun->mType == eLeadingWS)
-  {
-    // delete the leading ws that is after insertion point.  We don't
-    // have to (it would still not be significant after br), but it's 
-    // just more aesthetically pleasing to.
-    res = DeleteChars(*aInOutParent, *aInOutOffset, afterRun->mEndNode, afterRun->mEndOffset);
-    NS_ENSURE_SUCCESS(res, res);
-  }
-  else if (afterRun->mType == eNormalWS)
-  {
-    // need to determine if break at front of non-nbsp run.  if so
-    // convert run to nbsp.
-    WSPoint thePoint;
-    res = GetCharAfter(*aInOutParent, *aInOutOffset, &thePoint);
-    if ( (NS_SUCCEEDED(res)) && thePoint.mTextNode && (nsCRT::IsAsciiSpace(thePoint.mChar)) )
+    // some scoping for nsAutoTrackDOMPoint.  This will track our insertion point
+    // while we tweak any surrounding whitespace
+    nsAutoTrackDOMPoint tracker(mHTMLEditor->mRangeUpdater, aInOutParent, aInOutOffset);
+
+    // handle any changes needed to ws run after inserted br
+    if (!afterRun)
     {
-      WSPoint prevPoint;
-      res = GetCharBefore(thePoint, &prevPoint);
-      if ( (NS_FAILED(res)) || (prevPoint.mTextNode && !nsCRT::IsAsciiSpace(prevPoint.mChar)) )
+      // dont need to do anything.  just insert break.  ws wont change.
+    }
+    else if (afterRun->mType & eTrailingWS)
+    {
+      // dont need to do anything.  just insert break.  ws wont change.
+    }
+    else if (afterRun->mType == eLeadingWS)
+    {
+      // delete the leading ws that is after insertion point.  We don't
+      // have to (it would still not be significant after br), but it's 
+      // just more aesthetically pleasing to.
+      res = DeleteChars(*aInOutParent, *aInOutOffset, afterRun->mEndNode, afterRun->mEndOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
+    else if (afterRun->mType == eNormalWS)
+    {
+      // need to determine if break at front of non-nbsp run.  if so
+      // convert run to nbsp.
+      WSPoint thePoint;
+      res = GetCharAfter(*aInOutParent, *aInOutOffset, &thePoint);
+      if ( (NS_SUCCEEDED(res)) && thePoint.mTextNode && (nsCRT::IsAsciiSpace(thePoint.mChar)) )
       {
-        // we are at start of non-nbsps.  convert to a single nbsp.
-        res = ConvertToNBSP(thePoint);
-        NS_ENSURE_SUCCESS(res, res);
+        WSPoint prevPoint;
+        res = GetCharBefore(thePoint, &prevPoint);
+        if ( (NS_FAILED(res)) || (prevPoint.mTextNode && !nsCRT::IsAsciiSpace(prevPoint.mChar)) )
+        {
+          // we are at start of non-nbsps.  convert to a single nbsp.
+          res = ConvertToNBSP(thePoint);
+          NS_ENSURE_SUCCESS(res, res);
+        }
       }
     }
-  }
-  
-  // handle any changes needed to ws run before inserted br
-  if (!beforeRun)
-  {
-    // dont need to do anything.  just insert break.  ws wont change.
-  }
-  else if (beforeRun->mType & eLeadingWS)
-  {
-    // dont need to do anything.  just insert break.  ws wont change.
-  }
-  else if (beforeRun->mType == eTrailingWS)
-  {
-    // need to delete the trailing ws that is before insertion point, because it 
-    // would become significant after break inserted.
-    res = DeleteChars(beforeRun->mStartNode, beforeRun->mStartOffset, *aInOutParent, *aInOutOffset);
-    NS_ENSURE_SUCCESS(res, res);
-  }
-  else if (beforeRun->mType == eNormalWS)
-  {
-    // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
-    res = CheckTrailingNBSP(beforeRun, *aInOutParent, *aInOutOffset);
-    NS_ENSURE_SUCCESS(res, res);
+    
+    // handle any changes needed to ws run before inserted br
+    if (!beforeRun)
+    {
+      // dont need to do anything.  just insert break.  ws wont change.
+    }
+    else if (beforeRun->mType & eLeadingWS)
+    {
+      // dont need to do anything.  just insert break.  ws wont change.
+    }
+    else if (beforeRun->mType == eTrailingWS)
+    {
+      // need to delete the trailing ws that is before insertion point, because it 
+      // would become significant after break inserted.
+      res = DeleteChars(beforeRun->mStartNode, beforeRun->mStartOffset, *aInOutParent, *aInOutOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
+    else if (beforeRun->mType == eNormalWS)
+    {
+      // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
+      res = CheckTrailingNBSP(beforeRun, *aInOutParent, *aInOutOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
   }
   
   // ready, aim, fire!
@@ -282,7 +292,7 @@ nsWSRunObject::InsertBreak(nsCOMPtr<nsIDOMNode> *aInOutParent,
 }
 
 nsresult 
-nsWSRunObject::InsertText(const nsAReadableString& aStringToInsert, 
+nsWSRunObject::InsertText(const nsAString& aStringToInsert, 
                           nsCOMPtr<nsIDOMNode> *aInOutParent, 
                           PRInt32 *aInOutOffset,
                           nsIDOMDocument *aDoc)
@@ -307,50 +317,56 @@ nsWSRunObject::InsertText(const nsAReadableString& aStringToInsert,
   res = FindRun(*aInOutParent, *aInOutOffset, &beforeRun, PR_FALSE);
   res = FindRun(*aInOutParent, *aInOutOffset, &afterRun, PR_TRUE);
   
-  // handle any changes needed to ws run after inserted text
-  if (!afterRun)
   {
-    // dont need to do anything.  just insert text.  ws wont change.
-  }
-  else if (afterRun->mType & eTrailingWS)
-  {
-    // dont need to do anything.  just insert text.  ws wont change.
-  }
-  else if (afterRun->mType == eLeadingWS)
-  {
-    // delete the leading ws that is after insertion point, because it 
-    // would become significant after text inserted.
-    res = DeleteChars(*aInOutParent, *aInOutOffset, afterRun->mEndNode, afterRun->mEndOffset);
-    NS_ENSURE_SUCCESS(res, res);
-  }
-  else if (afterRun->mType == eNormalWS)
-  {
-    // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
-    res = CheckLeadingNBSP(afterRun, *aInOutParent, *aInOutOffset);
-    NS_ENSURE_SUCCESS(res, res);
-  }
-  
-  // handle any changes needed to ws run before inserted text
-  if (!beforeRun)
-  {
-    // dont need to do anything.  just insert text.  ws wont change.
-  }
-  else if (beforeRun->mType & eLeadingWS)
-  {
-    // dont need to do anything.  just insert text.  ws wont change.
-  }
-  else if (beforeRun->mType == eTrailingWS)
-  {
-    // need to delete the trailing ws that is before insertion point, because it 
-    // would become significant after text inserted.
-    res = DeleteChars(beforeRun->mStartNode, beforeRun->mStartOffset, *aInOutParent, *aInOutOffset);
-    NS_ENSURE_SUCCESS(res, res);
-  }
-  else if (beforeRun->mType == eNormalWS)
-  {
-    // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
-    res = CheckTrailingNBSP(beforeRun, *aInOutParent, *aInOutOffset);
-    NS_ENSURE_SUCCESS(res, res);
+    // some scoping for nsAutoTrackDOMPoint.  This will track our insertion point
+    // while we tweak any surrounding whitespace
+    nsAutoTrackDOMPoint tracker(mHTMLEditor->mRangeUpdater, aInOutParent, aInOutOffset);
+
+    // handle any changes needed to ws run after inserted text
+    if (!afterRun)
+    {
+      // dont need to do anything.  just insert text.  ws wont change.
+    }
+    else if (afterRun->mType & eTrailingWS)
+    {
+      // dont need to do anything.  just insert text.  ws wont change.
+    }
+    else if (afterRun->mType == eLeadingWS)
+    {
+      // delete the leading ws that is after insertion point, because it 
+      // would become significant after text inserted.
+      res = DeleteChars(*aInOutParent, *aInOutOffset, afterRun->mEndNode, afterRun->mEndOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
+    else if (afterRun->mType == eNormalWS)
+    {
+      // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
+      res = CheckLeadingNBSP(afterRun, *aInOutParent, *aInOutOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
+    
+    // handle any changes needed to ws run before inserted text
+    if (!beforeRun)
+    {
+      // dont need to do anything.  just insert text.  ws wont change.
+    }
+    else if (beforeRun->mType & eLeadingWS)
+    {
+      // dont need to do anything.  just insert text.  ws wont change.
+    }
+    else if (beforeRun->mType == eTrailingWS)
+    {
+      // need to delete the trailing ws that is before insertion point, because it 
+      // would become significant after text inserted.
+      res = DeleteChars(beforeRun->mStartNode, beforeRun->mStartOffset, *aInOutParent, *aInOutOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
+    else if (beforeRun->mType == eNormalWS)
+    {
+      // try to change an nbsp to a space, if possible, just to prevent nbsp proliferation
+      res = CheckTrailingNBSP(beforeRun, *aInOutParent, *aInOutOffset);
+      NS_ENSURE_SUCCESS(res, res);
+    }
   }
   
   // next up, tweak head and tail of string as needed.
@@ -456,6 +472,17 @@ nsWSRunObject::DeleteWSBackward()
   NS_ENSURE_SUCCESS(res, res);
   if (!point.mTextNode) return NS_OK;  // nothing to delete
   
+  if (mPRE)  // easy case, preformatted ws
+  {
+    if (nsCRT::IsAsciiSpace(point.mChar) || (point.mChar == nbsp))
+    {
+      nsCOMPtr<nsIDOMNode> node(do_QueryInterface(point.mTextNode));
+      PRInt32 startOffset = point.mOffset;
+      PRInt32 endOffset = point.mOffset+1;
+      return DeleteChars(node, startOffset, node, endOffset);
+    }
+  }
+  
   // callers job to insure that previous char is really ws.
   // If it is normal ws, we need to delete the whole run
   if (nsCRT::IsAsciiSpace(point.mChar))
@@ -500,6 +527,17 @@ nsWSRunObject::DeleteWSForward()
   NS_ENSURE_SUCCESS(res, res);
   if (!point.mTextNode) return NS_OK;  // nothing to delete
   
+  if (mPRE)  // easy case, preformatted ws
+  {
+    if (nsCRT::IsAsciiSpace(point.mChar) || (point.mChar == nbsp))
+    {
+      nsCOMPtr<nsIDOMNode> node(do_QueryInterface(point.mTextNode));
+      PRInt32 startOffset = point.mOffset;
+      PRInt32 endOffset = point.mOffset+1;
+      return DeleteChars(node, startOffset, node, endOffset);
+    }
+  }
+  
   // callers job to insure that next char is really ws.
   // If it is normal ws, we need to delete the whole run
   if (nsCRT::IsAsciiSpace(point.mChar))
@@ -547,6 +585,7 @@ nsWSRunObject::PriorVisibleNode(nsIDOMNode *aNode,
   if (!aNode || !outVisNode || !outVisOffset || !outType)
     return NS_ERROR_NULL_POINTER;
     
+  *outType = eNone;
   WSFragment *run;
   nsresult res = FindRun(aNode, aOffset, &run, PR_FALSE);
   
@@ -583,8 +622,8 @@ nsWSRunObject::PriorVisibleNode(nsIDOMNode *aNode,
   }
   
   // if we get here then nothing in ws data to find.  return start reason
-  *outVisNode = mStartNode;
-  *outVisOffset = mStartOffset;
+  *outVisNode = mStartReasonNode;
+  *outVisOffset = mStartOffset;  // this really isn't meaningful if mStartReasonNode!=mStartNode
   *outType = mStartReason;
   return NS_OK;
 }
@@ -638,8 +677,8 @@ nsWSRunObject::NextVisibleNode (nsIDOMNode *aNode,
   }
   
   // if we get here then nothing in ws data to find.  return end reason
-  *outVisNode = mEndNode;
-  *outVisOffset = mEndOffset;
+  *outVisNode = mEndReasonNode;
+  *outVisOffset = mEndOffset; // this really isn't meaningful if mEndReasonNode!=mEndNode
   *outType = mEndReason;
   return NS_OK;
 }
@@ -679,8 +718,8 @@ nsWSRunObject::GetWSNodes()
   // block boundary.
   nsresult res = NS_OK;
   
-  nsCOMPtr<nsIDOMNode> blockParent, curStartNode, curEndNode;
-  PRInt32 curStartOffset, curEndOffset;
+  nsCOMPtr<nsIDOMNode> blockParent;
+  DOMPoint start(mNode, mOffset), end(mNode, mOffset);
   if (IsBlockNode(mNode)) blockParent = mNode;
   else blockParent = mHTMLEditor->GetBlockNodeParent(mNode);
 
@@ -699,6 +738,12 @@ nsWSRunObject::GetWSNodes()
       PRInt32 pos;
       for (pos=mOffset-1; pos>=0; pos--)
       {
+        // sanity bounds check the char position.  bug 136165
+        if (pos >= textFrag->GetLength())
+        {
+          NS_NOTREACHED("looking beyond end of text fragment");
+          continue;
+        }
         PRUnichar theChar = textFrag->CharAt(pos);
         if (!nsCRT::IsAsciiSpace(theChar))
         {
@@ -707,6 +752,7 @@ nsWSRunObject::GetWSNodes()
             mStartNode = mNode;
             mStartOffset = pos+1;
             mStartReason = eText;
+            mStartReasonNode = mNode;
             break;
           }
           // as we look backwards update our earliest found nbsp
@@ -719,50 +765,24 @@ nsWSRunObject::GetWSNodes()
             mLastNBSPOffset = pos;
           }
         }
+        start.SetPoint(mNode,pos);
       }
     }
-    if (!mStartNode)
-    {
-      // we didnt find beginning of whitespace.  remember this text node
-      // (and offset 0) as the extent to which we have looked back.
-      curStartNode = mNode;
-      curStartOffset = 0;
-    }
   }
 
   nsCOMPtr<nsIDOMNode> priorNode;
   while (!mStartNode)
   {
     // we haven't found the start of ws yet.  Keep looking
-    
-    // do we have a curStartNode? If not, get one.
-    if (!curStartNode)
-    {
-      res = GetPreviousWSNode(mNode, mOffset, blockParent, address_of(curStartNode));
-      NS_ENSURE_SUCCESS(res, res);
-      priorNode = curStartNode;
-    }
-    else
-    {
-      res = GetPreviousWSNode(curStartNode, blockParent, address_of(priorNode));
-      NS_ENSURE_SUCCESS(res, res);
-    }
+    res = GetPreviousWSNode(start, blockParent, address_of(priorNode));
+    NS_ENSURE_SUCCESS(res, res);
     if (priorNode)
     {
       if (IsBlockNode(priorNode))
       {
-        // we encountered a new block.  therefore no more ws.
-        if (mHTMLEditor->IsTextNode(curStartNode))
-        {
-          mStartNode = curStartNode;
-          mStartOffset = curStartOffset;
-        }
-        else
-        {
-          mHTMLEditor->GetNodeLocation(curStartNode, address_of(mStartNode), &mStartOffset);
-          mStartOffset++;
-        }
+        start.GetPoint(mStartNode, mStartOffset);
         mStartReason = eOtherBlock;
+        mStartReasonNode = priorNode;
       }
       else if (mHTMLEditor->IsTextNode(priorNode))
       {
@@ -775,91 +795,67 @@ nsWSRunObject::GetWSNodes()
         PRInt32 len;
         res = textNode->GetTextLength(&len);
         NS_ENSURE_SUCCESS(res, res);
-        PRInt32 pos;
-        for (pos=len-1; pos>=0; pos--)
+
+        if (len < 1)
         {
-          PRUnichar theChar = textFrag->CharAt(pos);
-          if (!nsCRT::IsAsciiSpace(theChar))
-          {
-            if (theChar != nbsp)
-            {
-              mStartNode = priorNode;
-              mStartOffset = pos+1;
-              mStartReason = eText;
-              break;
-            }
-            // as we look backwards update our earliest found nbsp
-            mFirstNBSPNode = priorNode;
-            mFirstNBSPOffset = pos;
-            // also keep track of latest nbsp so far
-            if (!mLastNBSPNode)
-            {
-              mLastNBSPNode = priorNode;
-              mLastNBSPOffset = pos;
-            }
-          }
-        }
-        if (!mStartNode)
-        {
-          // we didnt find beginning of whitespace.  remember this text node
-          // (and offset 0) as the extent to which we have looked back.
-          curStartNode = priorNode;
-          curStartOffset = 0;
-        }
-      }
-      else if (nsTextEditUtils::IsBreak(priorNode))
-      {
-        // we encountered a break.  therefore no more ws.
-        if (mHTMLEditor->IsTextNode(curStartNode))
-        {
-          mStartNode = curStartNode;
-          mStartOffset = curStartOffset;
+          // Zero length text node. Set start point to it
+          // so we can get past it!
+          start.SetPoint(priorNode,0);
         }
         else
         {
-          mHTMLEditor->GetNodeLocation(curStartNode, address_of(mStartNode), &mStartOffset);
-          mStartOffset++;
+          PRInt32 pos;
+          for (pos=len-1; pos>=0; pos--)
+          {
+            // sanity bounds check the char position.  bug 136165
+            if (pos >= textFrag->GetLength())
+            {
+              NS_NOTREACHED("looking beyond end of text fragment");
+              continue;
+            }
+            PRUnichar theChar = textFrag->CharAt(pos);
+            if (!nsCRT::IsAsciiSpace(theChar))
+            {
+              if (theChar != nbsp)
+              {
+                mStartNode = priorNode;
+                mStartOffset = pos+1;
+                mStartReason = eText;
+                mStartReasonNode = priorNode;
+                break;
+              }
+              // as we look backwards update our earliest found nbsp
+              mFirstNBSPNode = priorNode;
+              mFirstNBSPOffset = pos;
+              // also keep track of latest nbsp so far
+              if (!mLastNBSPNode)
+              {
+                mLastNBSPNode = priorNode;
+                mLastNBSPOffset = pos;
+              }
+            }
+            start.SetPoint(priorNode,pos);
+          }
         }
-        mStartReason = eBreak;
       }
       else
       {
-        // it's a special node, like <img>, that is not a block and not
+        // it's a break or a special node, like <img>, that is not a block and not
         // a break but still serves as a terminator to ws runs.
-        if (mHTMLEditor->IsTextNode(curStartNode))
-        {
-          mStartNode = curStartNode;
-          mStartOffset = curStartOffset;
-        }
+        start.GetPoint(mStartNode, mStartOffset);
+        if (nsTextEditUtils::IsBreak(priorNode))
+          mStartReason = eBreak;
         else
-        {
-          mHTMLEditor->GetNodeLocation(curStartNode, address_of(mStartNode), &mStartOffset);
-          mStartOffset++;
-        }
-        mStartReason = eSpecial;
+          mStartReason = eSpecial;
+        mStartReasonNode = priorNode;
       }
     }
     else
     {
       // no prior node means we exhausted blockParent
-      if (!curStartNode)
-      {
-        // we never found anything to work with, so start 
-        // is at beginning of block parent
-        mStartNode = blockParent;
-        mStartOffset = 0;
-      }
-      else if (mHTMLEditor->IsTextNode(curStartNode))
-      {
-        mStartNode = curStartNode;
-        mStartOffset = curStartOffset;
-      }
-      else
-      {
-        mHTMLEditor->GetNodeLocation(curStartNode, address_of(mStartNode), &mStartOffset);
-        mStartOffset++;
-      }
+      start.GetPoint(mStartNode, mStartOffset);
       mStartReason = eThisBlock;
+      mStartReasonNode = blockParent;
     } 
   }
   
@@ -878,6 +874,12 @@ nsWSRunObject::GetWSNodes()
       PRInt32 pos;
       for (pos=mOffset; pos<len; pos++)
       {
+        // sanity bounds check the char position.  bug 136165
+        if ((pos<0) || (pos>=textFrag->GetLength()))
+        {
+          NS_NOTREACHED("looking beyond end of text fragment");
+          continue;
+        }
         PRUnichar theChar = textFrag->CharAt(pos);
         if (!nsCRT::IsAsciiSpace(theChar))
         {
@@ -886,6 +888,7 @@ nsWSRunObject::GetWSNodes()
             mEndNode = mNode;
             mEndOffset = pos;
             mEndReason = eText;
+            mEndReasonNode = mNode;
             break;
           }
           // as we look forwards update our latest found nbsp
@@ -898,49 +901,25 @@ nsWSRunObject::GetWSNodes()
             mFirstNBSPOffset = pos;
           }
         }
+        end.SetPoint(mNode,pos);
       }
     }
-    if (!mEndNode)
-    {
-      // we didnt find end of whitespace.  remember this text node
-      // (and offset len) as the extent to which we have looked ahead.
-      curEndNode = mNode;
-      curEndOffset = len;
-    }
   }
 
   nsCOMPtr<nsIDOMNode> nextNode;
   while (!mEndNode)
   {
     // we haven't found the end of ws yet.  Keep looking
-
-    // do we have a curEndNode? If not, get one.
-    if (!curEndNode)
-    {
-      res = GetNextWSNode(mNode, mOffset, blockParent, address_of(curEndNode));
-      NS_ENSURE_SUCCESS(res, res);
-      nextNode = curEndNode;
-    }
-    else
-    {
-      res = GetNextWSNode(curEndNode, blockParent, address_of(nextNode));
-      NS_ENSURE_SUCCESS(res, res);
-    }
+    res = GetNextWSNode(end, blockParent, address_of(nextNode));
+    NS_ENSURE_SUCCESS(res, res);
     if (nextNode)
     {
       if (IsBlockNode(nextNode))
       {
         // we encountered a new block.  therefore no more ws.
-        if (mHTMLEditor->IsTextNode(curEndNode))
-        {
-          mEndNode = curEndNode;
-          mEndOffset = curEndOffset;
-        }
-        else
-        {
-          mHTMLEditor->GetNodeLocation(curEndNode, address_of(mEndNode), &mEndOffset);
-        }
+        end.GetPoint(mEndNode, mEndOffset);
         mEndReason = eOtherBlock;
+        mEndReasonNode = nextNode;
       }
       else if (mHTMLEditor->IsTextNode(nextNode))
       {
@@ -954,90 +933,68 @@ nsWSRunObject::GetWSNodes()
         PRInt32 len;
         res = textNode->GetTextLength(&len);
         NS_ENSURE_SUCCESS(res, res);
-        PRInt32 pos;
-        for (pos=0; pos<len; pos++)
+
+        if (len < 1)
         {
-          PRUnichar theChar = textFrag->CharAt(pos);
-          if (!nsCRT::IsAsciiSpace(theChar))
-          {
-            if (theChar != nbsp)
-            {
-              mEndNode = nextNode;
-              mEndOffset = pos;
-              mEndReason = eText;
-              break;
-            }
-            // as we look forwards update our latest found nbsp
-            mLastNBSPNode = nextNode;
-            mLastNBSPOffset = pos;
-            // also keep track of earliest nbsp so far
-            if (!mFirstNBSPNode)
-            {
-              mFirstNBSPNode = nextNode;
-              mFirstNBSPOffset = pos;
-            }
-          }
-        }
-        if (!mEndNode)
-        {
-          // we didnt find end of whitespace.  remember this text node
-          // (and offset len) as the extent to which we have looked ahead.
-          curEndNode = nextNode;
-          curEndOffset = len;
-        }
-      }
-      else if (nsTextEditUtils::IsBreak(nextNode))
-      {
-        // we encountered a break.  therefore no more ws.
-        if (mHTMLEditor->IsTextNode(curEndNode))
-        {
-          mEndNode = curEndNode;
-          mEndOffset = curEndOffset;
+          // Zero length text node. Set end point to it
+          // so we can get past it!
+          end.SetPoint(nextNode,0);
         }
         else
         {
-          mHTMLEditor->GetNodeLocation(curEndNode, address_of(mEndNode), &mEndOffset);
+          PRInt32 pos;
+          for (pos=0; pos<len; pos++)
+          {
+            // sanity bounds check the char position.  bug 136165
+            if (pos >= textFrag->GetLength())
+            {
+              NS_NOTREACHED("looking beyond end of text fragment");
+              continue;
+            }
+            PRUnichar theChar = textFrag->CharAt(pos);
+            if (!nsCRT::IsAsciiSpace(theChar))
+            {
+              if (theChar != nbsp)
+              {
+                mEndNode = nextNode;
+                mEndOffset = pos;
+                mEndReason = eText;
+                mEndReasonNode = nextNode;
+                break;
+              }
+              // as we look forwards update our latest found nbsp
+              mLastNBSPNode = nextNode;
+              mLastNBSPOffset = pos;
+              // also keep track of earliest nbsp so far
+              if (!mFirstNBSPNode)
+              {
+                mFirstNBSPNode = nextNode;
+                mFirstNBSPOffset = pos;
+              }
+            }
+            end.SetPoint(nextNode,pos+1);
+          }
         }
-        mEndReason = eBreak;
       }
       else
       {
-        // it's a special node, like <img>, that is not a block and not
-        // a break but still serves as a terminator to ws runs.
-        if (mHTMLEditor->IsTextNode(curEndNode))
-        {
-          mEndNode = curEndNode;
-          mEndOffset = curEndOffset;
-        }
+        // we encountered a break or a special node, like <img>, 
+        // that is not a block and not a break but still 
+        // serves as a terminator to ws runs.
+        end.GetPoint(mEndNode, mEndOffset);
+        if (nsTextEditUtils::IsBreak(nextNode))
+          mEndReason = eBreak;
         else
-        {
-          mHTMLEditor->GetNodeLocation(curEndNode, address_of(mEndNode), &mEndOffset);
-        }
-        mEndReason = eSpecial;
+          mEndReason = eSpecial;
+        mEndReasonNode = nextNode;
       }
     }
     else
     {
       // no next node means we exhausted blockParent
-      if (!curEndNode)
-      {
-        // we never found anything to work with, so end 
-        // is at end of block parent
-        mEndNode = blockParent;
-        PRUint32 count;
-        mHTMLEditor->GetLengthOfDOMNode(blockParent, count);
-        mEndOffset = count;
-      }
-      else if (mHTMLEditor->IsTextNode(curEndNode))
-      {
-        mEndNode = curEndNode;
-        mEndOffset = curEndOffset;
-      }
-      else
-      {
-        mHTMLEditor->GetNodeLocation(curEndNode, address_of(mEndNode), &mEndOffset);
-      }
+      end.GetPoint(mEndNode, mEndOffset);
       mEndReason = eThisBlock;
+      mEndReasonNode = blockParent;
     } 
   }
 
@@ -1050,11 +1007,11 @@ nsWSRunObject::GetRuns()
   ClearRuns();
   
   // handle some easy cases first
-  
-  // if we are surrounded by text or special, it's all one
+  mHTMLEditor->IsPreformatted(mNode, &mPRE);
+  // if it's preformatedd, or if we are surrounded by text or special, it's all one
   // big normal ws run
-  if ( ((mStartReason == eText) || (mStartReason == eSpecial)) &&
-       ((mEndReason == eText) || (mEndReason == eSpecial) || (mEndReason == eBreak)) )
+  if ( mPRE || (((mStartReason == eText) || (mStartReason == eSpecial)) &&
+       ((mEndReason == eText) || (mEndReason == eSpecial) || (mEndReason == eBreak))) )
   {
     return MakeSingleWSRun(eNormalWS);
   }
@@ -1274,51 +1231,16 @@ nsWSRunObject::GetPreviousWSNode(nsIDOMNode *aStartNode,
 }
 
 nsresult 
-nsWSRunObject::GetNextWSNode(nsIDOMNode *aStartNode, 
-                             nsIDOMNode *aBlockParent, 
-                             nsCOMPtr<nsIDOMNode> *aNextNode)
+nsWSRunObject::GetPreviousWSNode(DOMPoint aPoint,
+                                nsIDOMNode  *aBlockParent, 
+                                nsCOMPtr<nsIDOMNode> *aPriorNode)
 {
-  // can't really recycle various getnext/prior routines because we have special needs
-  // here.  Need to step into inline containers but not block containers.
-  if (!aStartNode || !aBlockParent || !aNextNode) return NS_ERROR_NULL_POINTER;
-  
-  *aNextNode = 0;
-  nsresult res = aStartNode->GetNextSibling(getter_AddRefs(*aNextNode));
-  NS_ENSURE_SUCCESS(res, res);
-  nsCOMPtr<nsIDOMNode> temp, curNode = aStartNode;
-  while (!*aNextNode)
-  {
-    // we have exhausted nodes in parent of aStartNode.
-    res = curNode->GetParentNode(getter_AddRefs(temp));
-    NS_ENSURE_SUCCESS(res, res);
-    if (!temp) return NS_ERROR_NULL_POINTER;
-    if (temp == aBlockParent)
-    {
-      // we have exhausted nodes in the block parent.  The convention here is to return null.
-      *aNextNode = nsnull;
-      return NS_OK;
-    }
-    // we have a parent: look for next sibling
-    res = temp->GetNextSibling(getter_AddRefs(*aNextNode));
-    NS_ENSURE_SUCCESS(res, res);
-    curNode = temp;
-  }
-  // we have a next node.  If it's a block, return it.
-  if (IsBlockNode(*aNextNode))
-    return NS_OK;
-  // else if it's a container, get deep leftmost child
-  else if (mHTMLEditor->IsContainer(*aNextNode))
-  {
-    temp = mHTMLEditor->GetLeftmostChild(*aNextNode);
-    if (temp)
-      *aNextNode = temp;
-    return NS_OK;
-  }
-  // else return the node itself
-  return NS_OK;
+  nsCOMPtr<nsIDOMNode> node;
+  PRInt32 offset;
+  aPoint.GetPoint(node, offset);
+  return GetPreviousWSNode(node,offset,aBlockParent,aPriorNode);
 }
 
-
 nsresult 
 nsWSRunObject::GetPreviousWSNode(nsIDOMNode *aStartNode,
                                 PRInt16      aOffset, 
@@ -1371,6 +1293,62 @@ nsWSRunObject::GetPreviousWSNode(nsIDOMNode *aStartNode,
   return NS_OK;
 }
 
+nsresult 
+nsWSRunObject::GetNextWSNode(nsIDOMNode *aStartNode, 
+                             nsIDOMNode *aBlockParent, 
+                             nsCOMPtr<nsIDOMNode> *aNextNode)
+{
+  // can't really recycle various getnext/prior routines because we have special needs
+  // here.  Need to step into inline containers but not block containers.
+  if (!aStartNode || !aBlockParent || !aNextNode) return NS_ERROR_NULL_POINTER;
+  
+  *aNextNode = 0;
+  nsresult res = aStartNode->GetNextSibling(getter_AddRefs(*aNextNode));
+  NS_ENSURE_SUCCESS(res, res);
+  nsCOMPtr<nsIDOMNode> temp, curNode = aStartNode;
+  while (!*aNextNode)
+  {
+    // we have exhausted nodes in parent of aStartNode.
+    res = curNode->GetParentNode(getter_AddRefs(temp));
+    NS_ENSURE_SUCCESS(res, res);
+    if (!temp) return NS_ERROR_NULL_POINTER;
+    if (temp == aBlockParent)
+    {
+      // we have exhausted nodes in the block parent.  The convention here is to return null.
+      *aNextNode = nsnull;
+      return NS_OK;
+    }
+    // we have a parent: look for next sibling
+    res = temp->GetNextSibling(getter_AddRefs(*aNextNode));
+    NS_ENSURE_SUCCESS(res, res);
+    curNode = temp;
+  }
+  // we have a next node.  If it's a block, return it.
+  if (IsBlockNode(*aNextNode))
+    return NS_OK;
+  // else if it's a container, get deep leftmost child
+  else if (mHTMLEditor->IsContainer(*aNextNode))
+  {
+    temp = mHTMLEditor->GetLeftmostChild(*aNextNode);
+    if (temp)
+      *aNextNode = temp;
+    return NS_OK;
+  }
+  // else return the node itself
+  return NS_OK;
+}
+
+nsresult 
+nsWSRunObject::GetNextWSNode(DOMPoint aPoint,
+                             nsIDOMNode  *aBlockParent, 
+                             nsCOMPtr<nsIDOMNode> *aNextNode)
+{
+  nsCOMPtr<nsIDOMNode> node;
+  PRInt32 offset;
+  aPoint.GetPoint(node, offset);
+  return GetNextWSNode(node,offset,aBlockParent,aNextNode);
+}
+
 nsresult 
 nsWSRunObject::GetNextWSNode(nsIDOMNode *aStartNode,
                              PRInt16     aOffset, 
@@ -1449,7 +1427,7 @@ nsWSRunObject::PrepareToDeleteRangePriv(nsWSRunObject* aEndObject)
     NS_ENSURE_SUCCESS(res, res);
   }
   // adjust normal ws in afterRun if needed
-  if (afterRun && (afterRun->mType == eNormalWS))
+  if (afterRun && (afterRun->mType == eNormalWS) && !aEndObject->mPRE)
   {
     if ( (beforeRun && (beforeRun->mType == eLeadingWS)) ||
          (!beforeRun && ((mStartReason & eBlock) || (mStartReason == eBreak))) )
@@ -1470,7 +1448,7 @@ nsWSRunObject::PrepareToDeleteRangePriv(nsWSRunObject* aEndObject)
     res = DeleteChars(beforeRun->mStartNode, beforeRun->mStartOffset, mNode, mOffset);
     NS_ENSURE_SUCCESS(res, res);
   }
-  else if (beforeRun && (beforeRun->mType == eNormalWS))
+  else if (beforeRun && (beforeRun->mType == eNormalWS) && !mPRE)
   {
     if ( (afterRun && (afterRun->mType == eTrailingWS)) ||
          (afterRun && (afterRun->mType == eNormalWS))   ||
@@ -1706,7 +1684,6 @@ nsWSRunObject::GetCharAfter(WSPoint &aPoint, WSPoint *outPoint)
   if (aPoint.mOffset < len)
   {
     *outPoint = aPoint;
-    outPoint->mOffset;
     outPoint->mChar = GetCharAt(aPoint.mTextNode, aPoint.mOffset);
   }
   else if (idx < (PRInt32)(numNodes-1))
@@ -1774,12 +1751,12 @@ nsWSRunObject::ConvertToNBSP(WSPoint aPoint)
   // first, insert an nbsp
   nsAutoTxnsConserveSelection dontSpazMySelection(mHTMLEditor);
   nsAutoString nbspStr(nbsp);
-  nsresult res = mHTMLEditor->InsertTextIntoTextNodeImpl(nbspStr, textNode, aPoint.mOffset);
+  nsresult res = mHTMLEditor->InsertTextIntoTextNodeImpl(nbspStr, textNode, aPoint.mOffset, PR_TRUE);
   NS_ENSURE_SUCCESS(res, res);
   
   // next, find range of ws it will replace
   nsCOMPtr<nsIDOMNode> startNode, endNode;
-  PRInt32 startOffset, endOffset;
+  PRInt32 startOffset=0, endOffset=0;
   
   res = GetAsciiWSBounds(eAfter, node, aPoint.mOffset+1, address_of(startNode), 
                          &startOffset, address_of(endNode), &endOffset);
@@ -1803,7 +1780,7 @@ nsWSRunObject::GetAsciiWSBounds(PRInt16 aDir, nsIDOMNode *aNode, PRInt32 aOffset
     return NS_ERROR_NULL_POINTER;
 
   nsCOMPtr<nsIDOMNode> startNode, endNode;
-  PRInt32 startOffset, endOffset;
+  PRInt32 startOffset=0, endOffset=0;
   
   nsresult res = NS_OK;
   
@@ -1938,9 +1915,7 @@ nsWSRunObject::GetCharAt(nsITextContent *aTextNode, PRInt32 aOffset)
   nsresult res = aTextNode->GetText(&textFrag);
   NS_ENSURE_SUCCESS(res, 0);
   
-  PRInt32 len;
-  res = aTextNode->GetTextLength(&len);
-  NS_ENSURE_SUCCESS(res, 0);
+  PRInt32 len = textFrag->GetLength();
   if (!len) 
     return 0;
   if (aOffset>=len) 
@@ -2115,7 +2090,7 @@ nsWSRunObject::CheckTrailingNBSPOfRun(WSFragment *aRun)
         return NS_ERROR_NULL_POINTER;
       nsAutoTxnsConserveSelection dontSpazMySelection(mHTMLEditor);
       nsAutoString spaceStr(PRUnichar(32));
-      res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset);
+      res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset, PR_TRUE);
       NS_ENSURE_SUCCESS(res, res);
   
       // finally, delete that nbsp
@@ -2157,7 +2132,7 @@ nsWSRunObject::CheckTrailingNBSP(WSFragment *aRun, nsIDOMNode *aNode, PRInt32 aO
       return NS_ERROR_NULL_POINTER;
     nsAutoTxnsConserveSelection dontSpazMySelection(mHTMLEditor);
     nsAutoString spaceStr(PRUnichar(32));
-    res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset);
+    res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset, PR_TRUE);
     NS_ENSURE_SUCCESS(res, res);
   
     // finally, delete that nbsp
@@ -2199,7 +2174,7 @@ nsWSRunObject::CheckLeadingNBSP(WSFragment *aRun, nsIDOMNode *aNode, PRInt32 aOf
       return NS_ERROR_NULL_POINTER;
     nsAutoTxnsConserveSelection dontSpazMySelection(mHTMLEditor);
     nsAutoString spaceStr(PRUnichar(32));
-    res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset);
+    res = mHTMLEditor->InsertTextIntoTextNodeImpl(spaceStr, textNode, thePoint.mOffset, PR_TRUE);
     NS_ENSURE_SUCCESS(res, res);
   
     // finally, delete that nbsp
