@@ -99,6 +99,7 @@
 #include "nsIPref.h"
 #include "nsStyleConsts.h"
 #include "nsIStyleContext.h"
+#include "nsUnicharUtils.h"
 
 #include "nsAOLCiter.h"
 #include "nsInternetCiter.h"
@@ -112,11 +113,6 @@
 
 const PRUnichar nbsp = 160;
 
-// HACK - CID for NS_CTRANSITIONAL_DTD_CID so that we can get at transitional dtd
-#define NS_CTRANSITIONAL_DTD_CID \
-{ 0x4611d482, 0x960a, 0x11d4, { 0x8e, 0xb0, 0xb6, 0x17, 0x66, 0x1b, 0x6f, 0x7c } }
-
-
 static NS_DEFINE_CID(kCContentIteratorCID, NS_CONTENTITERATOR_CID);
 static NS_DEFINE_CID(kCRangeCID,      NS_RANGE_CID);
 static NS_DEFINE_CID(kCDOMSelectionCID,      NS_DOMSELECTION_CID);
@@ -139,6 +135,8 @@ nsPlaintextEditor::nsPlaintextEditor()
 , mIgnoreSpuriousDragEvent(PR_FALSE)
 , mRules(nsnull)
 , mIsComposing(PR_FALSE)
+, mWrapToWindow(PR_FALSE)
+, mWrapColumn(0)
 , mMaxTextLength(-1)
 , mInitTriggerCounter(0)
 {
@@ -255,17 +253,8 @@ nsPlaintextEditor::EndEditorInit()
   return res;
 }
 
-nsresult
-nsPlaintextEditor::IsCSSEnabled(PRBool *aIsSet)
-{
-  // STUB
-  *aIsSet = PR_FALSE;
-  return NS_OK;
-}
-
-
 NS_IMETHODIMP 
-nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSet) 
+nsPlaintextEditor::SetDocumentCharacterSet(const nsAString & characterSet) 
 { 
   nsresult result; 
 
@@ -276,7 +265,6 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
     nsCOMPtr<nsIDOMDocument>domdoc; 
     result = GetDocument(getter_AddRefs(domdoc)); 
     if (NS_SUCCEEDED(result) && domdoc) { 
-      nsAutoString newMetaString; 
       nsCOMPtr<nsIDOMNodeList>metaList; 
       nsCOMPtr<nsIDOMNode>metaNode; 
       nsCOMPtr<nsIDOMElement>metaElement; 
@@ -294,20 +282,26 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
           metaElement = do_QueryInterface(metaNode); 
           if (!metaElement) continue; 
 
-          nsString currentValue; 
+          nsAutoString currentValue; 
           if (NS_FAILED(metaElement->GetAttribute(NS_LITERAL_STRING("http-equiv"), currentValue))) continue; 
 
-          if (kNotFound != currentValue.Find("content-type", PR_TRUE)) { 
+          if (FindInReadable(NS_LITERAL_STRING("content-type"),
+                             currentValue,
+                             nsCaseInsensitiveStringComparator())) { 
             NS_NAMED_LITERAL_STRING(content, "content");
             if (NS_FAILED(metaElement->GetAttribute(content, currentValue))) continue; 
 
-            NS_NAMED_LITERAL_STRING(charset, "charset=");
-            PRInt32 offset = currentValue.Find(charset.get(), PR_TRUE); 
-            if (kNotFound != offset) {
-              currentValue.Left(newMetaString, offset); // copy current value before "charset=" (e.g. text/html) 
-              newMetaString.Append(charset); 
-              newMetaString.Append(characterSet); 
-              result = nsEditor::SetAttribute(metaElement, content, newMetaString); 
+            NS_NAMED_LITERAL_STRING(charsetEquals, "charset=");
+            nsAString::const_iterator originalStart, start, end;
+            originalStart = currentValue.BeginReading(start);
+            currentValue.EndReading(end);
+            if (FindInReadable(charsetEquals, start, end,
+                               nsCaseInsensitiveStringComparator())) {
+
+              // set attribute to <original prefix> charset=text/html
+              result = nsEditor::SetAttribute(metaElement, content,
+                                              Substring(originalStart, start) +
+                                              charsetEquals + characterSet); 
               if (NS_SUCCEEDED(result)) 
                 newMetaCharset = PR_FALSE; 
               break; 
@@ -336,10 +330,9 @@ nsPlaintextEditor::SetDocumentCharacterSet(const nsAReadableString & characterSe
                 // not undoable, undo should undo CreateNode 
                 result = metaElement->SetAttribute(NS_LITERAL_STRING("http-equiv"), NS_LITERAL_STRING("Content-Type")); 
                 if (NS_SUCCEEDED(result)) { 
-                  newMetaString.Assign(NS_LITERAL_STRING("text/html;charset=")); 
-                  newMetaString.Append(characterSet); 
                   // not undoable, undo should undo CreateNode 
-                  result = metaElement->SetAttribute(NS_LITERAL_STRING("content"), newMetaString); 
+                  result = metaElement->SetAttribute(NS_LITERAL_STRING("content"),
+                                                     NS_LITERAL_STRING("text/html;charset=") + characterSet); 
                 } 
               } 
             } 
@@ -547,7 +540,7 @@ NS_IMETHODIMP nsPlaintextEditor::HandleKeyPress(nsIDOMKeyEvent* aKeyEvent)
    to TypedText() to determine what action to take, but without passing
    an event.
    */
-NS_IMETHODIMP nsPlaintextEditor::TypedText(const nsAReadableString& aString,
+NS_IMETHODIMP nsPlaintextEditor::TypedText(const nsAString& aString,
                                       PRInt32 aAction)
 {
   nsAutoPlaceHolderBatch batch(this, gTypingTxnName);
@@ -958,7 +951,7 @@ NS_IMETHODIMP nsPlaintextEditor::DeleteSelection(nsIEditor::EDirection aAction)
   return result;
 }
 
-NS_IMETHODIMP nsPlaintextEditor::InsertText(const nsAReadableString &aStringToInsert)
+NS_IMETHODIMP nsPlaintextEditor::InsertText(const nsAString &aStringToInsert)
 {
   if (!mRules) { return NS_ERROR_NOT_INITIALIZED; }
 
@@ -1201,41 +1194,15 @@ nsPlaintextEditor::GetBodyStyleContext(nsIStyleContext** aStyleContext)
 }
 
 //
-// Get the wrap width for the root of the document.
+// Get the wrap width
 //
 NS_IMETHODIMP 
 nsPlaintextEditor::GetWrapWidth(PRInt32 *aWrapColumn)
 {
-  nsresult res;
-
   if (! aWrapColumn)
     return NS_ERROR_NULL_POINTER;
 
-  *aWrapColumn = -1;        // default: no wrap
-
-  nsCOMPtr<nsIStyleContext> styleContext;
-  res = GetBodyStyleContext(getter_AddRefs(styleContext));
-  if (NS_FAILED(res)) return res;
-  NS_ASSERTION(styleContext, "styleContext is null!");
-  if (!styleContext) return NS_ERROR_FAILURE;
-
-  const nsStyleText* styleText =
-    (const nsStyleText*)styleContext->GetStyleData(eStyleStruct_Text);
-
-  if (NS_STYLE_WHITESPACE_PRE == styleText->mWhiteSpace)
-  {
-    *aWrapColumn = 0;   // wrap to window width
-  }
-  else if (NS_STYLE_WHITESPACE_MOZ_PRE_WRAP == styleText->mWhiteSpace)
-  {
-    const nsStylePosition* stylePosition =
-      (const nsStylePosition*)styleContext->GetStyleData(eStyleStruct_Position); 
-    if (stylePosition->mWidth.GetUnit() == eStyleUnit_Chars)
-      *aWrapColumn = stylePosition->mWidth.GetIntValue(); 
-    else 
-      return NS_ERROR_UNEXPECTED;
-  }
-
+  *aWrapColumn = mWrapColumn;
   return NS_OK;
 }
 
@@ -1265,6 +1232,15 @@ nsPlaintextEditor::SetWrapWidth(PRInt32 aWrapColumn)
 {
   nsresult res;
 
+  mWrapColumn = aWrapColumn;
+
+  // Make sure we're a plaintext editor, otherwise we shouldn't
+  // do the rest of this.
+  PRUint32 flags = 0;
+  GetFlags(&flags);
+  if (!(flags & eEditorPlaintextMask))
+    return NS_OK;
+
   // Ought to set a style sheet here ...
   // Probably should keep around an mPlaintextStyleSheet for this purpose.
   nsCOMPtr<nsIDOMElement> bodyElement;
@@ -1294,19 +1270,29 @@ nsPlaintextEditor::SetWrapWidth(PRInt32 aWrapColumn)
   // Make sure we have fixed-width font.  This should be done for us,
   // but it isn't, see bug 22502, so we have to add "font: -moz-fixed;".
   // Only do this if we're wrapping.
-  PRUint32 flags = 0;
-  GetFlags(&flags);
   if ((flags & eEditorEnableWrapHackMask) && aWrapColumn >= 0)
     styleValue.Append(NS_LITERAL_STRING("font-family: -moz-fixed; "));
 
+  // If "mail.compose.wrap_to_window_width" is set, and we're a mail editor,
+  // then remember our wrap width (for output purposes) but set the visual
+  // wrapping to window width.
+  // We may reset mWrapToWindow here, based on the pref's current value.
+  if (flags & eEditorMailMask)
+  {
+    nsresult rv;
+    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
+    if (NS_SUCCEEDED(rv))
+      prefs->GetBoolPref("mail.compose.wrap_to_window_width", &mWrapToWindow);
+  }
+
   // and now we're ready to set the new whitespace/wrapping style.
-  if (aWrapColumn > 0)        // Wrap to a fixed column
+  if (aWrapColumn > 0 && !mWrapToWindow)        // Wrap to a fixed column
   {
     styleValue.Append(NS_LITERAL_STRING("white-space: -moz-pre-wrap; width: "));
     styleValue.AppendInt(aWrapColumn);
     styleValue.Append(NS_LITERAL_STRING("ch;"));
   }
-  else if (aWrapColumn == 0)
+  else if (mWrapToWindow || aWrapColumn == 0)
     styleValue.Append(NS_LITERAL_STRING("white-space: -moz-pre-wrap;"));
   else
     styleValue.Append(NS_LITERAL_STRING("white-space: pre;"));
@@ -1436,9 +1422,9 @@ NS_IMETHODIMP nsPlaintextEditor::CanCopy(PRBool *aCanCopy)
 
 // Shared between OutputToString and OutputToStream
 NS_IMETHODIMP
-nsPlaintextEditor::GetAndInitDocEncoder(const nsAReadableString& aFormatType,
+nsPlaintextEditor::GetAndInitDocEncoder(const nsAString& aFormatType,
                                         PRUint32 aFlags,
-                                        const nsAReadableString& aCharset,
+                                        const nsAString& aCharset,
                                         nsIDocumentEncoder** encoder)
 {
   nsCOMPtr<nsIPresShell> presShell;
@@ -1524,9 +1510,9 @@ nsPlaintextEditor::GetAndInitDocEncoder(const nsAReadableString& aFormatType,
 
 
 NS_IMETHODIMP 
-nsPlaintextEditor::OutputToString(nsAWritableString& aOutputString,
-                                  const nsAReadableString& aFormatType,
-                                  PRUint32 aFlags)
+nsPlaintextEditor::OutputToString(const nsAString& aFormatType,
+                                  PRUint32 aFlags,
+                                  nsAString& aOutputString)
 {
   PRBool cancel, handled;
   nsString resultString;
@@ -1559,8 +1545,8 @@ nsPlaintextEditor::OutputToString(nsAWritableString& aOutputString,
 
 NS_IMETHODIMP
 nsPlaintextEditor::OutputToStream(nsIOutputStream* aOutputStream,
-                             const nsAReadableString& aFormatType,
-                             const nsAReadableString& aCharset,
+                             const nsAString& aFormatType,
+                             const nsAString& aCharset,
                              PRUint32 aFlags)
 {
   nsresult rv;
@@ -1685,7 +1671,7 @@ static nsICiter* MakeACiter()
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::InsertAsQuotation(const nsAReadableString& aQuotedText,
+nsPlaintextEditor::InsertAsQuotation(const nsAString& aQuotedText,
                                      nsIDOMNode **aNodeInserted)
 {
   // We have the text.  Cite it appropriately:
@@ -1732,17 +1718,17 @@ nsPlaintextEditor::InsertAsQuotation(const nsAReadableString& aQuotedText,
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::PasteAsCitedQuotation(const nsAReadableString& aCitation,
+nsPlaintextEditor::PasteAsCitedQuotation(const nsAString& aCitation,
                                          PRInt32 aSelectionType)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsPlaintextEditor::InsertAsCitedQuotation(const nsAReadableString& aQuotedText,
-                                          const nsAReadableString& aCitation,
+nsPlaintextEditor::InsertAsCitedQuotation(const nsAString& aQuotedText,
+                                          const nsAString& aCitation,
                                           PRBool aInsertHTML,
-                                          const nsAReadableString& aCharset,
+                                          const nsAString& aCharset,
                                           nsIDOMNode **aNodeInserted)
 {
   return InsertAsQuotation(aQuotedText, aNodeInserted);
@@ -1781,9 +1767,10 @@ nsPlaintextEditor::Rewrap(PRBool aRespectNewlines)
 
   if (isCollapsed)    // rewrap the whole document
   {
-    rv = OutputToString(current, format,
+    rv = OutputToString(format,
                         nsIDocumentEncoder::OutputFormatted
-                        | nsIDocumentEncoder::OutputLFLineBreak);
+                        | nsIDocumentEncoder::OutputLFLineBreak,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1800,10 +1787,11 @@ nsPlaintextEditor::Rewrap(PRBool aRespectNewlines)
   }
   else                // rewrap only the selection
   {
-    rv = OutputToString(current, format,
+    rv = OutputToString(format,
                         nsIDocumentEncoder::OutputFormatted
                         | nsIDocumentEncoder::OutputLFLineBreak
-                        | nsIDocumentEncoder::OutputSelectionOnly);
+                        | nsIDocumentEncoder::OutputSelectionOnly,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1844,8 +1832,7 @@ nsPlaintextEditor::StripCites()
 
   if (isCollapsed)    // rewrap the whole document
   {
-    rv = OutputToString(current, format,
-                               nsIDocumentEncoder::OutputFormatted);
+    rv = OutputToString(format, nsIDocumentEncoder::OutputFormatted, current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1862,9 +1849,10 @@ nsPlaintextEditor::StripCites()
   }
   else                // rewrap only the selection
   {
-    rv = OutputToString(current, format,
-                               nsIDocumentEncoder::OutputFormatted
-                                | nsIDocumentEncoder::OutputSelectionOnly);
+    rv = OutputToString(format,
+                        nsIDocumentEncoder::OutputFormatted
+                        | nsIDocumentEncoder::OutputSelectionOnly,
+                        current);
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsICiter> citer = dont_AddRef(MakeACiter());
@@ -1894,7 +1882,7 @@ nsPlaintextEditor::GetEmbeddedObjects(nsISupportsArray** aNodeList)
 #endif
 
 NS_IMETHODIMP
-nsPlaintextEditor::SetCompositionString(const nsAReadableString& aCompositionString, nsIPrivateTextRangeList* aTextRangeList,nsTextEventReply* aReply)
+nsPlaintextEditor::SetCompositionString(const nsAString& aCompositionString, nsIPrivateTextRangeList* aTextRangeList,nsTextEventReply* aReply)
 {
   NS_ASSERTION(aTextRangeList, "null ptr");
   if(nsnull == aTextRangeList)
@@ -1915,26 +1903,35 @@ nsPlaintextEditor::SetCompositionString(const nsAReadableString& aCompositionStr
   if (NS_FAILED(result)) return result;
 
   mIMETextRangeList = aTextRangeList;
-  nsAutoPlaceHolderBatch batch(this, gIMETxnName);
 
-  result = InsertText(aCompositionString);
-
-  mIMEBufferLength = aCompositionString.Length();
-
-  if (!mPresShellWeak) return NS_ERROR_NOT_INITIALIZED;
-  nsCOMPtr<nsIPresShell> ps = do_QueryReferent(mPresShellWeak);
-  if (!ps) return NS_ERROR_NOT_INITIALIZED;
-  ps->GetCaret(getter_AddRefs(caretP));
-  caretP->SetCaretDOMSelection(selection);
-
-  caretP->GetCaretCoordinates(nsICaret::eIMECoordinates, selection,
-            &(aReply->mCursorPosition), &(aReply->mCursorIsCollapsed));
-
-  // second part of 23558 fix:
-  if (aCompositionString.IsEmpty()) 
+  // we need the nsAutoPlaceHolderBatch destructor called before hitting
+  // GetCaretCoordinates so the states in Frame system sync with content
+  // therefore, we put the nsAutoPlaceHolderBatch into a inner block
   {
-    mIMETextNode = nsnull;
+    nsAutoPlaceHolderBatch batch(this, gIMETxnName);
+
+    result = InsertText(aCompositionString);
+
+    mIMEBufferLength = aCompositionString.Length();
+
+    if (!mPresShellWeak) 
+      return NS_ERROR_NOT_INITIALIZED;
+    nsCOMPtr<nsIPresShell> ps = do_QueryReferent(mPresShellWeak);
+    if (!ps) 
+      return NS_ERROR_NOT_INITIALIZED;
+    ps->GetCaret(getter_AddRefs(caretP));
+    caretP->SetCaretDOMSelection(selection);
+
+    // second part of 23558 fix:
+    if (aCompositionString.IsEmpty()) 
+    {
+      mIMETextNode = nsnull;
+    }
   }
+
+  result = caretP->GetCaretCoordinates(nsICaret::eIMECoordinates, selection,
+            &(aReply->mCursorPosition), &(aReply->mCursorIsCollapsed));
+  NS_ASSERTION(NS_SUCCEEDED(result), "cannot get caret position");
   
   return result;
 }
@@ -2115,10 +2112,16 @@ void nsPlaintextEditor::HandleEventListenerError()
 #endif
 
 nsresult
-nsPlaintextEditor::SetCSSEquivalentToHTMLStyle(nsIDOMElement * aElement,
-                                               const nsAReadableString & aAttribute,
-                                               const nsAReadableString & aValue)
+nsPlaintextEditor::SetAttributeOrEquivalent(nsIDOMElement * aElement,
+                                            const nsAString & aAttribute,
+                                            const nsAString & aValue)
 {
-  // STUB
-  return NS_OK;
+  return nsEditor::SetAttribute(aElement, aAttribute, aValue);
+}
+
+nsresult
+nsPlaintextEditor::RemoveAttributeOrEquivalent(nsIDOMElement * aElement,
+                                               const nsAString & aAttribute)
+{
+  return nsEditor::RemoveAttribute(aElement, aAttribute);
 }
