@@ -40,6 +40,7 @@
 
 #include "nsEditor.h"
 #include "nsTextEditUtils.h"
+#include "nsCRT.h"
 
 #include "nsCOMPtr.h"
 #include "nsIServiceManager.h"
@@ -147,6 +148,10 @@ nsTextEditRules::Init(nsPlaintextEditor *aEditor, PRUint32 aFlags)
   res = CreateBogusNodeIfNeeded(selection);   // this method handles null selection, which should never happen anyway
   if (NS_FAILED(res)) return res;
 
+  // insure trailing br node
+  res = CreateTrailingBRIfNeeded();
+  if (NS_FAILED(res)) return res;
+  
   // create a range that is the entire body contents
   nsCOMPtr<nsIDOMRange> wholeDoc = do_CreateInstance(kRangeCID);
   if (!wholeDoc) return NS_ERROR_NULL_POINTER;
@@ -224,6 +229,10 @@ nsTextEditRules::AfterEdit(PRInt32 action, nsIEditor::EDirection aDirection)
     res = CreateBogusNodeIfNeeded(selection);
     if (NS_FAILED(res)) return res;
     
+    // insure trailing br node
+    res = CreateTrailingBRIfNeeded();
+    if (NS_FAILED(res)) return res;
+    
 #ifdef IBMBIDI
     /* After inserting text the cursor Bidi level must be set to the level of the inserted text.
      * This is difficult, because we cannot know what the level is until after the Bidi algorithm
@@ -464,8 +473,8 @@ nsTextEditRules::WillInsertText(PRInt32          aAction,
                                 nsISelection *aSelection, 
                                 PRBool          *aCancel,
                                 PRBool          *aHandled,
-                                const nsAReadableString *inString,
-                                nsAWritableString *outString,
+                                const nsAString *inString,
+                                nsAString *outString,
                                 PRInt32          aMaxLength)
 {  
   if (!aSelection || !aCancel || !aHandled) { return NS_ERROR_NULL_POINTER; }
@@ -555,7 +564,7 @@ nsTextEditRules::WillInsertText(PRInt32          aAction,
 
     if (singleLineNewlineBehavior == eReplaceWithSpaces)
     {
-      //nsAWritableString destString;
+      //nsAString destString;
       //NormalizeCRLF(outString,destString);
 
       tString.ReplaceChar(CRLF, ' ');
@@ -624,13 +633,12 @@ nsTextEditRules::WillInsertText(PRInt32          aAction,
     // it is to search for both tabs and newlines.
     if (isPRE)
     {
-      NS_NAMED_LITERAL_STRING(newlineStr, "\n");
-      char newlineChar = '\n';
+      NS_NAMED_LITERAL_STRING(newlineStr, LFSTR);
       while (unicodeBuf && (pos != -1) && ((PRUint32)pos < tString.Length()))
       {
         PRInt32 oldPos = pos;
         PRInt32 subStrLen;
-        pos = tString.FindChar(newlineChar, PR_FALSE, oldPos);
+        pos = tString.FindChar(nsCRT::LF, oldPos);
         
         if (pos != -1) 
         {
@@ -694,8 +702,8 @@ nsTextEditRules::WillInsertText(PRInt32          aAction,
     else
     {
       NS_NAMED_LITERAL_STRING(tabStr, "\t");
-      NS_NAMED_LITERAL_STRING(newlineStr, "\n");
-      char specialChars[] = {'\t','\n',0};
+      NS_NAMED_LITERAL_STRING(newlineStr, LFSTR);
+      char specialChars[] = {TAB, nsCRT::LF, 0};
       nsAutoString tabString(NS_LITERAL_STRING("    "));
       while (unicodeBuf && (pos != -1) && ((PRUint32)pos < tString.Length()))
       {
@@ -953,8 +961,14 @@ nsTextEditRules::DidDeleteSelection(nsISelection *aSelection,
     if (!strLength)
     {
       res = mEditor->DeleteNode(startNode);
+      if (NS_FAILED(res)) return res;
     }
   }
+  // We prevent the caret from sticking on the left of prior BR
+  // (i.e. the end of previous line) after this deletion.  Bug 92124
+  nsCOMPtr<nsISelectionPrivate> selPriv = do_QueryInterface(aSelection);
+  if (selPriv) res = selPriv->SetInterlinePosition(PR_TRUE);
+
   return res;
 }
 
@@ -1050,8 +1064,8 @@ nsTextEditRules::DidRedo(nsISelection *aSelection, nsresult aResult)
 
 nsresult
 nsTextEditRules::WillOutputText(nsISelection *aSelection, 
-                                const nsAReadableString  *aOutputFormat,
-                                nsAWritableString *aOutString,                                
+                                const nsAString  *aOutputFormat,
+                                nsAString *aOutString,                                
                                 PRBool   *aCancel,
                                 PRBool   *aHandled)
 {
@@ -1143,7 +1157,6 @@ nsTextEditRules::ReplaceNewlines(nsIDOMRange *aRange)
   // replace newlines with breaks.  have to do this left to right,
   // since inserting the break can split the text node, and the
   // original node becomes the righthand node.
-  char newlineChar[] = {'\n',0};
   res = arrayOfNodes->Count(&nodeCount);
   if (NS_FAILED(res)) return res;
   for (j = 0; j < nodeCount; j++)
@@ -1158,7 +1171,7 @@ nsTextEditRules::ReplaceNewlines(nsIDOMRange *aRange)
     do 
     {
       textNode->GetData(tempString);
-      offset = tempString.FindCharInSet(newlineChar);
+      offset = tempString.FindChar(nsCRT::LF);
       if (offset == -1) break; // done with this node
       
       // delete the newline
@@ -1182,6 +1195,29 @@ nsTextEditRules::ReplaceNewlines(nsIDOMRange *aRange)
   return res;
 }
 
+nsresult
+nsTextEditRules::CreateTrailingBRIfNeeded()
+{
+  // but only if we aren't a single line edit field
+  if (mFlags & nsIPlaintextEditor::eEditorSingleLineMask)
+    return NS_OK;
+  nsresult res = NS_OK;
+  nsCOMPtr<nsIDOMNode> lastChild, unused;
+  if (!mBody) return NS_ERROR_NULL_POINTER;
+  res = mBody->GetLastChild(getter_AddRefs(lastChild));
+  // assuming CreateBogusNodeIfNeeded() has been called first
+  if (NS_FAILED(res)) return res;  
+  if (!lastChild) return NS_ERROR_NULL_POINTER;
+  if (!nsTextEditUtils::IsBreak(lastChild))
+  {
+    nsAutoTxnsConserveSelection dontSpazMySelection(mEditor);
+    PRUint32 rootLen;
+    res = mEditor->GetLengthOfDOMNode(mBody, rootLen);
+    if (NS_FAILED(res)) return res; 
+    res = CreateMozBR(mBody, rootLen, address_of(unused));
+  }
+  return res;
+}
 
 nsresult
 nsTextEditRules::CreateBogusNodeIfNeeded(nsISelection *aSelection)
@@ -1247,8 +1283,8 @@ nsTextEditRules::CreateBogusNodeIfNeeded(nsISelection *aSelection)
 
 nsresult
 nsTextEditRules::TruncateInsertionIfNeeded(nsISelection *aSelection, 
-                                           const nsAReadableString  *aInString,
-                                           nsAWritableString  *aOutString,
+                                           const nsAString  *aInString,
+                                           nsAString  *aOutString,
                                            PRInt32          aMaxLength)
 {
   if (!aSelection || !aInString || !aOutString) {return NS_ERROR_NULL_POINTER;}
@@ -1302,7 +1338,7 @@ nsTextEditRules::ResetIMETextPWBuf()
 }
 
 nsresult
-nsTextEditRules::RemoveIMETextFromPWBuf(PRInt32 &aStart, nsAWritableString *aIMEString)
+nsTextEditRules::RemoveIMETextFromPWBuf(PRInt32 &aStart, nsAString *aIMEString)
 {
   if (!aIMEString) {
     return NS_ERROR_NULL_POINTER;
@@ -1324,7 +1360,7 @@ nsTextEditRules::RemoveIMETextFromPWBuf(PRInt32 &aStart, nsAWritableString *aIME
 }
 
 nsresult
-nsTextEditRules::EchoInsertionToPWBuff(PRInt32 aStart, PRInt32 aEnd, nsAWritableString *aOutString)
+nsTextEditRules::EchoInsertionToPWBuff(PRInt32 aStart, PRInt32 aEnd, nsAString *aOutString)
 {
   if (!aOutString) {return NS_ERROR_NULL_POINTER;}
 
