@@ -48,6 +48,7 @@
 #include "IEHtmlDocument.h"
 #include "PropertyDlg.h"
 #include "PromptService.h"
+#include "HelperAppDlg.h"
 #include "WindowCreator.h"
 
 #include "nsCWebBrowser.h"
@@ -75,8 +76,11 @@ static HANDLE s_hHackedNonReentrancy = NULL;
 #define NS_PROMPTSERVICE_CID \
   {0xa2112d6a, 0x0e28, 0x421f, {0xb4, 0x6a, 0x25, 0xc0, 0xb3, 0x8, 0xcb, 0xd0}}
 
+#define NS_HELPERAPPLAUNCHERDIALOG_CID \
+    {0xf68578eb, 0x6ec2, 0x4169, {0xae, 0x19, 0x8c, 0x62, 0x43, 0xf0, 0xab, 0xe1}}
+
 static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
+static NS_DEFINE_CID(kHelperAppLauncherDialogCID, NS_HELPERAPPLAUNCHERDIALOG_CID);
 
 // Macros to return errors from bad calls to the automation
 // interfaces and sets a descriptive string on IErrorInfo so VB programmers
@@ -112,7 +116,7 @@ public:
 // Prefs
 
 static const char *c_szPrefsHomePage = "browser.startup.homepage";
-static const char *c_szDefaultPage   = "resource:/res/MozillaControl.html";
+static const char *c_szDefaultPage   = "data:text/html,<html><body bgcolor=\"#00FF00\"><p>Mozilla Control</p></body></html>";
 
 // Registry keys and values
 
@@ -186,6 +190,19 @@ CMozillaBrowser::~CMozillaBrowser()
     Terminate();
 }
 
+// See bug 127982:
+//
+// Microsoft's InlineIsEqualGUID global function is multiply defined
+// in ATL and/or SDKs with varying namespace requirements. To save the control
+// from future grief, this method is used instead. 
+static inline BOOL _IsEqualGUID(REFGUID rguid1, REFGUID rguid2)
+{
+   return (
+	  ((PLONG) &rguid1)[0] == ((PLONG) &rguid2)[0] &&
+	  ((PLONG) &rguid1)[1] == ((PLONG) &rguid2)[1] &&
+	  ((PLONG) &rguid1)[2] == ((PLONG) &rguid2)[2] &&
+	  ((PLONG) &rguid1)[3] == ((PLONG) &rguid2)[3]);
+}
 
 STDMETHODIMP CMozillaBrowser::InterfaceSupportsErrorInfo(REFIID riid)
 {
@@ -195,9 +212,9 @@ STDMETHODIMP CMozillaBrowser::InterfaceSupportsErrorInfo(REFIID riid)
         &IID_IWebBrowser2,
         &IID_IWebBrowserApp
     };
-    for (int i=0;i<(sizeof(arr)/sizeof(arr[0]));i++)
+    for (int i = 0; i < (sizeof(arr) / sizeof(arr[0])); i++)
     {
-        if (::ATL::InlineIsEqualGUID(*arr[i],riid))
+        if (_IsEqualGUID(*arr[i], riid))
             return S_OK;
     }
     return S_FALSE;
@@ -655,10 +672,10 @@ LRESULT CMozillaBrowser::OnSaveAs(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL
         _makepath(szDataPath, drive, dir, szDataFile, "");
 
         nsCOMPtr<nsILocalFile> file;
-        NS_NewLocalFile(T2A(szFile), TRUE, getter_AddRefs(file));
+        NS_NewNativeLocalFile(nsDependentCString(T2A(szFile)), TRUE, getter_AddRefs(file));
 
         nsCOMPtr<nsILocalFile> dataPath;
-        NS_NewLocalFile(szDataPath, TRUE, getter_AddRefs(dataPath));
+        NS_NewNativeLocalFile(nsDependentCString(szDataPath), TRUE, getter_AddRefs(dataPath));
 
         persist->SaveDocument(nsnull, file, dataPath, nsnull, 0, 0);
     }
@@ -723,7 +740,7 @@ LRESULT CMozillaBrowser::OnViewSource(WORD wNotifyCode, WORD wID, HWND hWndCtl,
 {
     NG_TRACE_METHOD(CMozillaBrowser::OnViewSource);
 
-    if (mWebBrowserContainer->m_pCurrentURI)
+    if (mWebBrowserContainer->mCurrentURI)
     {
         // No URI to view!
         NG_ASSERT(0);
@@ -731,12 +748,12 @@ LRESULT CMozillaBrowser::OnViewSource(WORD wNotifyCode, WORD wID, HWND hWndCtl,
     }
 
     // Get the current URI
-    nsXPIDLCString aURI;
-    mWebBrowserContainer->m_pCurrentURI->GetSpec(getter_Copies(aURI));
+    nsCAutoString aURI;
+    mWebBrowserContainer->mCurrentURI->GetSpec(aURI);
 
     nsAutoString strURI;
     strURI.Assign(NS_LITERAL_STRING("view-source:"));
-    strURI.AppendWithConversion(aURI);
+    strURI.Append(NS_ConvertUTF8toUCS2(aURI));
 
     CIPtr(IDispatch) spDispNew;
     VARIANT_BOOL bCancel = VARIANT_FALSE;
@@ -975,7 +992,7 @@ HRESULT CMozillaBrowser::Initialize()
     // Create an object to represent the path
     nsCOMPtr<nsILocalFile> binDir;
     USES_CONVERSION;
-    NS_NewLocalFile(T2A(szBinDirPath), TRUE, getter_AddRefs(binDir));
+    NS_NewNativeLocalFile(nsDependentCString(T2A(szBinDirPath)), TRUE, getter_AddRefs(binDir));
     rv = NS_InitEmbedding(binDir, provider);
 
     // Load preferences service
@@ -1033,6 +1050,15 @@ HRESULT CMozillaBrowser::Initialize()
             promptFactory,
             PR_TRUE); // replace existing
 
+        // Helper app launcher dialog
+        nsCOMPtr<nsIFactory> helperAppDlgFactory;
+        rv = NS_NewHelperAppLauncherDlgFactory(getter_AddRefs(helperAppDlgFactory));
+        rv = nsComponentManager::RegisterFactory(kHelperAppLauncherDialogCID,
+            "Helper App Launcher Dialog",
+            "@mozilla.org/helperapplauncherdialog;1",
+            helperAppDlgFactory,
+            PR_TRUE); // replace existing
+
         // create our local object
         CWindowCreator *creator = new CWindowCreator();
         nsCOMPtr<nsIWindowCreator> windowCreator;
@@ -1092,8 +1118,6 @@ HRESULT CMozillaBrowser::CreateBrowser()
 
     nsresult rv;
 
-    PRBool aAllowPlugins = PR_TRUE;
-
     // Create the web browser
     mWebBrowser = do_CreateInstance(NS_WEBBROWSER_CONTRACTID, &rv);
     if (NS_FAILED(rv))
@@ -1106,8 +1130,14 @@ HRESULT CMozillaBrowser::CreateBrowser()
 
     // Configure what the web browser can and cannot do
     nsCOMPtr<nsIWebBrowserSetup> webBrowserAsSetup(do_QueryInterface(mWebBrowser));
-    // webBrowserAsSetup->SetProperty(nsIWebBrowserSetup::SETUP_ALLOW_PLUGINS, aAllowPlugins);
-    // webBrowserAsSetup->SetProperty(nsIWebBrowserSetup::SETUP_CONTAINS_CHROME, PR_TRUE);
+
+    // Allow plugins?
+    const PRBool kAllowPlugins = PR_TRUE;
+    webBrowserAsSetup->SetProperty(nsIWebBrowserSetup::SETUP_ALLOW_PLUGINS, kAllowPlugins);
+
+    // Host chrome or content?
+    const PRBool kHostChrome = PR_FALSE; // Hardcoded for now
+    webBrowserAsSetup->SetProperty(nsIWebBrowserSetup::SETUP_IS_CHROME_WRAPPER, PR_FALSE);
 
     // Create the webbrowser window
     mWebBrowserAsWin = do_QueryInterface(mWebBrowser);
@@ -1434,32 +1464,27 @@ HRESULT CMozillaBrowser::UnloadBrowserHelpers()
 // Print document
 HRESULT CMozillaBrowser::PrintDocument(BOOL promptUser)
 {
-    nsresult rv;
+    // Print the contents
+    nsCOMPtr<nsIWebBrowserPrint> browserAsPrint = do_GetInterface(mWebBrowser);
+    NS_ASSERTION(browserAsPrint, "No nsIWebBrowserPrint!");
 
     PRBool oldPrintSilent = PR_FALSE;
-    nsCOMPtr<nsIPrintOptions> printService = 
-             do_GetService(kPrintOptionsCID, &rv);
-    if (printService)
+    nsCOMPtr<nsIPrintSettings> printSettings;
+    browserAsPrint->GetGlobalPrintSettings(getter_AddRefs(printSettings));
+    if (printSettings) 
     {
-        printService->GetPrintSilent(&oldPrintSilent);
-        printService->SetPrintSilent(promptUser ? PR_FALSE : PR_TRUE);
+        printSettings->GetPrintSilent(&oldPrintSilent);
+        printSettings->SetPrintSilent(promptUser ? PR_FALSE : PR_TRUE);
     }
 
-    // Print the contents
-    nsCOMPtr<nsIWebBrowserPrint> browserAsPrint = do_QueryInterface(mWebBrowser);
-    nsCOMPtr<nsIDOMWindow> window;
-    mWebBrowser->GetContentDOMWindow(getter_AddRefs(window));
-    if (window)
-    {
-        PrintListener *listener = new PrintListener;
-        nsCOMPtr<nsIWebProgressListener> printListener = do_QueryInterface(listener);
-        browserAsPrint->Print(window, nsnull, nsnull);
-        listener->WaitForComplete();
-    }
+    PrintListener *listener = new PrintListener;
+    nsCOMPtr<nsIWebProgressListener> printListener = do_QueryInterface(listener);
+    browserAsPrint->Print(printSettings, nsnull);
+    listener->WaitForComplete();
 
-    if (printService)
+    if (printSettings)
     {
-        printService->SetPrintSilent(oldPrintSilent);
+        printSettings->SetPrintSilent(oldPrintSilent);
     }
     
     return S_OK;
@@ -1777,7 +1802,7 @@ HRESULT STDMETHODCALLTYPE CMozillaBrowser::Navigate(BSTR URL, VARIANT __RPC_FAR
 
     // Check for a view-source op - this is a bit kludgy
     // TODO
-    if (sUrl.CompareWithConversion(L"view-source:", PR_TRUE, 12) == 0)
+    if (NS_LossyConvertUCS2toASCII(sUrl).Compare("view-source:", PR_TRUE, 12) == 0)
      {
         // Broken code - appears to want to replace view-source: with view: to 
         // get Mozilla to respond to the IE view-source: protocol.
@@ -2360,9 +2385,9 @@ HRESULT STDMETHODCALLTYPE CMozillaBrowser::get_LocationURL(BSTR __RPC_FAR *Locat
     if (uri)
     {
         USES_CONVERSION;
-        nsXPIDLCString aURI;
-        uri->GetSpec(getter_Copies(aURI));
-        *LocationURL = SysAllocString(A2OLE((const char *) aURI));
+        nsCAutoString aURI;
+        uri->GetAsciiSpec(aURI);
+        *LocationURL = SysAllocString(A2OLE(aURI.get()));
     }
     else
     {
@@ -2598,14 +2623,14 @@ HRESULT STDMETHODCALLTYPE CMozillaBrowser::get_Visible(VARIANT_BOOL __RPC_FAR *p
         RETURN_E_UNEXPECTED();
     }
 
-    NG_ASSERT_POINTER(pBool, int);
+    NG_ASSERT_POINTER(pBool, VARIANT_BOOL);
     if (pBool == NULL)
     {
         RETURN_E_INVALIDARG();
     }
 
-    //TODO: Implement get_Visible
-    *pBool = VARIANT_FALSE;
+    //TODO: Implement get_Visible?
+    *pBool = VARIANT_TRUE;
     return S_OK;
 }
 
@@ -2620,7 +2645,7 @@ HRESULT STDMETHODCALLTYPE CMozillaBrowser::put_Visible(VARIANT_BOOL Value)
         RETURN_E_UNEXPECTED();
     }
 
-    //TODO: Implement put_Visible
+    //TODO: Implement put_Visible?
     return S_OK;
 }
 
@@ -3301,8 +3326,8 @@ void PrintListener::WaitForComplete()
     ::CloseHandle(hFakeEvent);
 }
 
-/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aStateFlags, in unsigned long aStatus); */
-NS_IMETHODIMP PrintListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long aStateFlags, in nsresult aStatus); */
+NS_IMETHODIMP PrintListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
   if (aStatus == nsIWebProgressListener::STATE_STOP) {
     mComplete = PR_TRUE;
@@ -3313,23 +3338,23 @@ NS_IMETHODIMP PrintListener::OnStateChange(nsIWebProgress *aWebProgress, nsIRequ
 /* void onProgressChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aCurSelfProgress, in long aMaxSelfProgress, in long aCurTotalProgress, in long aMaxTotalProgress); */
 NS_IMETHODIMP PrintListener::OnProgressChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aCurSelfProgress, PRInt32 aMaxSelfProgress, PRInt32 aCurTotalProgress, PRInt32 aMaxTotalProgress)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 /* void onLocationChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsIURI location); */
 NS_IMETHODIMP PrintListener::OnLocationChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, nsIURI *location)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 /* void onStatusChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in nsresult aStatus, in wstring aMessage); */
 NS_IMETHODIMP PrintListener::OnStatusChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, nsresult aStatus, const PRUnichar *aMessage)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
-NS_IMETHODIMP PrintListener::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 state)
+/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
+NS_IMETHODIMP PrintListener::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
