@@ -134,8 +134,8 @@ NS_IMETHODIMP CWebBrowserContainer::OnProgressChange(nsIWebProgress *aProgress,
 
 
 
-/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest request, in long progressStateFlags, in unsinged long aStatus); */
-NS_IMETHODIMP CWebBrowserContainer::OnStateChange(nsIWebProgress* aWebProgress, nsIRequest *aRequest, PRInt32 progressStateFlags, nsresult aStatus)
+/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest request, in unsigned long progressStateFlags, in unsinged long aStatus); */
+NS_IMETHODIMP CWebBrowserContainer::OnStateChange(nsIWebProgress* aWebProgress, nsIRequest *aRequest, PRUint32 progressStateFlags, nsresult aStatus)
 {
     nsresult rv = NS_OK;
 
@@ -155,9 +155,9 @@ NS_IMETHODIMP CWebBrowserContainer::OnStateChange(nsIWebProgress* aWebProgress,
                 rv = channel->GetURI(getter_AddRefs(uri));
                 if (NS_SUCCEEDED(rv))
                 {
-                    nsXPIDLCString aURI;
-                    uri->GetSpec(getter_Copies(aURI));
-                    NG_TRACE(_T("CWebBrowserContainer::OnStateChange->Doc Start(..., %s, \"\")\n"), A2CT(aURI));
+                    nsCAutoString aURI;
+                    uri->GetAsciiSpec(aURI);
+                    NG_TRACE(_T("CWebBrowserContainer::OnStateChange->Doc Start(..., %s, \"\")\n"), A2CT(aURI.get()));
                 }
             }
 
@@ -186,14 +186,14 @@ NS_IMETHODIMP CWebBrowserContainer::OnStateChange(nsIWebProgress* aWebProgress,
             if (!aChannel) return NS_ERROR_NULL_POINTER;
 
             rv = aChannel->GetURI(getter_AddRefs(pURI));
-            if (NS_FAILED(rv)) return rv;
+            if (NS_FAILED(rv)) return NS_OK;
 
-            nsXPIDLCString aURI;
-            rv = pURI->GetSpec(getter_Copies(aURI));
-            if (NS_FAILED(rv)) return rv;
+            nsCAutoString aURI;
+            rv = pURI->GetAsciiSpec(aURI);
+            if (NS_FAILED(rv)) return NS_OK;
 
             USES_CONVERSION;
-            BSTR bstrURI = SysAllocString(A2OLE((const char *) aURI)); 
+            BSTR bstrURI = SysAllocString(A2OLE(aURI.get())); 
                 
             // Fire a DocumentComplete event
             CComVariant vURI(bstrURI);
@@ -219,15 +219,15 @@ NS_IMETHODIMP CWebBrowserContainer::OnStateChange(nsIWebProgress* aWebProgress,
 
         if (progressStateFlags & STATE_STOP)
         {
-            nsXPIDLCString aURI;
+            nsCAutoString aURI;
             if (m_pCurrentURI)
             {
-                m_pCurrentURI->GetSpec(getter_Copies(aURI));
+                m_pCurrentURI->GetAsciiSpec(aURI);
             }
 
             // Fire a NavigateComplete event
             USES_CONVERSION;
-            BSTR bstrURI = SysAllocString(A2OLE((const char *) aURI));
+            BSTR bstrURI = SysAllocString(A2OLE(aURI.get()));
             m_pEvents1->Fire_NavigateComplete(bstrURI);
 
             // Fire a NavigateComplete2 event
@@ -279,7 +279,7 @@ NS_IMETHODIMP CWebBrowserContainer::OnLocationChange(nsIWebProgress* aWebProgres
 {
 //    nsXPIDLCString aPath;
 //    location->GetPath(getter_Copies(aPath));
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -301,9 +301,9 @@ CWebBrowserContainer::OnStatusChange(nsIWebProgress* aWebProgress,
 NS_IMETHODIMP 
 CWebBrowserContainer::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                        nsIRequest *aRequest, 
-                                       PRInt32 state)
+                                       PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 
@@ -324,8 +324,8 @@ NS_IMETHODIMP CWebBrowserContainer::OnStartURIOpen(nsIURI *pURI, PRBool *aAbortO
     NG_ASSERT(m_pCurrentURI);
     m_pCurrentURI->AddRef();
 
-    nsXPIDLCString aURI;
-    m_pCurrentURI->GetSpec(getter_Copies(aURI));
+    nsCAutoString aURI;
+    m_pCurrentURI->GetSpec(aURI);
 
     // Setup the post data
     CComVariant vPostDataRef;
@@ -336,7 +336,7 @@ NS_IMETHODIMP CWebBrowserContainer::OnStartURIOpen(nsIURI *pURI, PRBool *aAbortO
 
 
     // Fire a BeforeNavigate event
-    BSTR bstrURI = SysAllocString(A2OLE((const char *)aURI));
+    BSTR bstrURI = SysAllocString(A2OLE(aURI.get()));
     BSTR bstrTargetFrameName = NULL;
     BSTR bstrHeaders = NULL;
     VARIANT_BOOL bCancel = VARIANT_FALSE;
@@ -357,7 +357,7 @@ NS_IMETHODIMP CWebBrowserContainer::OnStartURIOpen(nsIURI *pURI, PRBool *aAbortO
     SysFreeString(bstrTargetFrameName);
     SysFreeString(bstrHeaders);
 
-    if (bCancel == VARIANT_TRUE)
+    if (bCancel != VARIANT_FALSE)
     {
         *aAbortOpen = PR_TRUE;
         return NS_ERROR_ABORT;
