@@ -54,6 +54,7 @@
 #include <nsIChromeEventHandler.h>
 #include <nsIContentViewer.h>
 #include <nsIContentViewerEdit.h>
+#include <nsIWebBrowserSetup.h>
 #include "nsIWebBrowserPrint.h"
 #include "nsIClipboardCommands.h"
 #include "docshell/nsCDefaultURIFixup.h"
@@ -65,7 +66,6 @@
 // for profiles
 #include <nsMPFileLocProvider.h>
 #include "nsIWebBrowserPrint.h"
-#include "nsIPrintOptions.h"
 
 // all of our local includes
 #include "EmbedPrivate.h"
@@ -80,7 +80,6 @@
 #include "PtMozilla.h"
 
 static NS_DEFINE_CID(kAppShellCID, NS_APPSHELL_CID);
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 
 static const char sWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
 
@@ -209,6 +208,11 @@ EmbedPrivate::Setup()
 	nsCOMPtr<nsIWebBrowser> webBrowser;
 	mWindow->GetWebBrowser(getter_AddRefs(webBrowser));
 
+	// Configure what the web browser can and cannot do
+	PRBool aAllowPlugins = PR_TRUE;
+	nsCOMPtr<nsIWebBrowserSetup> webBrowserAsSetup(do_QueryInterface(webBrowser));
+	webBrowserAsSetup->SetProperty(nsIWebBrowserSetup::SETUP_ALLOW_PLUGINS, aAllowPlugins);
+
 	// get a handle on the navigation object
 	mNavigation = do_QueryInterface(webBrowser);
 
@@ -237,7 +241,7 @@ EmbedPrivate::Setup()
 
 	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(webBrowser));
 	if (print)
-		print->GetPrintSettings(getter_AddRefs(m_PrintSettings));
+		print->GetGlobalPrintSettings(getter_AddRefs(m_PrintSettings));
 
 	return NS_OK;
 }
@@ -527,7 +531,7 @@ EmbedPrivate::Print(PpPrintContext_t *pc)
   nsCOMPtr<nsIWebBrowserPrint> print( do_GetInterface( mWindow->mWebBrowser ) );
 
   m_PrintSettings->SetEndPageRange((PRInt32) pc);
-  print->Print(window, m_PrintSettings, mPrint);
+  print->Print(m_PrintSettings, mPrint);
 }
 
 nsresult
