@@ -26,7 +26,6 @@
 // Local Includes
 #include "nsDocShellTreeOwner.h"
 #include "nsWebBrowser.h"
-#include "nsNonPersistAuthPrompt.h"
 
 // Helper Classes
 #include "nsIGenericFactory.h"
@@ -34,9 +33,12 @@
 #include "nsSize.h"
 #include "nsHTMLReflowState.h"
 #include "nsIServiceManager.h"
+#include "nsComponentManagerUtils.h"
 #include "nsXPIDLString.h"
+#include "nsIAtom.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
+#include "nsISimpleEnumerator.h"
 
 // Interfaces needed to be included
 #include "nsIContextMenuListener.h"
@@ -51,6 +53,7 @@
 #include "nsIDOMMouseEvent.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIDOMNamedNodeMap.h"
+#include "nsIWebNavigation.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIPresShell.h"
 #include "nsPIDOMWindow.h"
@@ -61,13 +64,45 @@
 #include "nsIWindowWatcher.h"
 #include "nsPIWindowWatcher.h"
 #include "nsIPrompt.h"
-#include "nsIWalletService.h"
 #include "nsRect.h"
 #include "nsIWebBrowserChromeFocus.h"
+#include "nsIDragDropOverride.h"
+#include "nsIContent.h"
 
 static const char sWindowWatcherContractID[] = "@mozilla.org/embedcomp/window-watcher;1";
 
 
+//
+// GetEventReceiver
+//
+// A helper routine that navigates the tricky path from a |nsWebBrowser| to
+// a |nsIDOMEventReceiver| via the window root and chrome event handler.
+//
+static nsresult
+GetEventReceiver ( nsWebBrowser* inBrowser, nsIDOMEventReceiver** outEventRcvr )
+{
+  nsCOMPtr<nsIDOMWindow> domWindow;
+  inBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
+  NS_ENSURE_TRUE(domWindow, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
+  NS_ENSURE_TRUE(domWindowPrivate, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIDOMWindowInternal> rootWindow;
+  domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
+  NS_ENSURE_TRUE(rootWindow, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIChromeEventHandler> chromeHandler;
+  nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
+  NS_ENSURE_TRUE(piWin, NS_ERROR_FAILURE);
+  piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
+  NS_ENSURE_TRUE(chromeHandler, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMEventReceiver> rcvr = do_QueryInterface(chromeHandler);
+  *outEventRcvr = rcvr.get();
+  NS_IF_ADDREF(*outEventRcvr);
+  
+  return NS_OK;
+}
+
 
 //*****************************************************************************
 //***    nsDocShellTreeOwner: Object Management
@@ -187,7 +222,7 @@ NS_IMETHODIMP nsDocShellTreeOwner::FindItemWithName(const PRUnichar* aName,
     if (domWindow) {
       nsAutoString ourName;
       domWindow->GetName(ourName);
-      if (name.EqualsIgnoreCase(ourName)) {
+      if (name.Equals(ourName, nsCaseInsensitiveStringComparator())) {
         *aFoundItem = mWebBrowser->mDocShellAsItem;
         NS_IF_ADDREF(*aFoundItem);
         return NS_OK;
@@ -585,6 +620,18 @@ NS_IMETHODIMP nsDocShellTreeOwner::SetVisibility(PRBool aVisibility)
     return NS_ERROR_NULL_POINTER;
 }
 
+NS_IMETHODIMP nsDocShellTreeOwner::GetEnabled(PRBool *aEnabled)
+{
+  NS_ENSURE_ARG_POINTER(aEnabled);
+  *aEnabled = PR_TRUE;
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsDocShellTreeOwner::SetEnabled(PRBool aEnabled)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 NS_IMETHODIMP nsDocShellTreeOwner::GetMainWidget(nsIWidget** aMainWidget)
 {
     return NS_ERROR_NULL_POINTER;
@@ -640,17 +687,17 @@ nsDocShellTreeOwner::OnProgressChange(nsIWebProgress* aProgress,
 NS_IMETHODIMP
 nsDocShellTreeOwner::OnStateChange(nsIWebProgress* aProgress,
                                    nsIRequest* aRequest,
-                                   PRInt32 aProgressStateFlags,
+                                   PRUint32 aProgressStateFlags,
                                    nsresult aStatus)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP nsDocShellTreeOwner::OnLocationChange(nsIWebProgress* aWebProgress,
                                                     nsIRequest* aRequest,
                                                     nsIURI* aURI)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -659,15 +706,15 @@ nsDocShellTreeOwner::OnStatusChange(nsIWebProgress* aWebProgress,
                                     nsresult aStatus,
                                     const PRUnichar* aMessage)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP 
 nsDocShellTreeOwner::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                       nsIRequest *aRequest, 
-                                      PRInt32 state)
+                                      PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 
@@ -773,7 +820,22 @@ nsDocShellTreeOwner :: AddChromeListeners ( )
         rv = NS_ERROR_OUT_OF_MEMORY;
     }
   }
-   
+  
+  // install the external dragDrop handler
+  if ( !mChromeDragHandler ) {
+    mChromeDragHandler = do_CreateInstance("@mozilla.org:/content/content-area-dragdrop;1", &rv);
+    NS_ASSERTION(mChromeDragHandler, "Couldn't create the chrome drag handler");
+    if ( mChromeDragHandler ) {
+      nsCOMPtr<nsIDOMEventReceiver> rcvr;
+      GetEventReceiver(mWebBrowser, getter_AddRefs(rcvr));
+      nsCOMPtr<nsIOverrideDragSource> srcOverride ( do_QueryInterface(mWebBrowserChrome) );
+      nsCOMPtr<nsIOverrideDropSite> siteOverride ( do_QueryInterface(mWebBrowserChrome) );
+      nsCOMPtr<nsIDOMEventTarget> rcvrTarget(do_QueryInterface(rcvr));
+      mChromeDragHandler->HookupTo(rcvrTarget, NS_STATIC_CAST(nsIWebNavigation*, mWebBrowser),
+                                    srcOverride, siteOverride);
+    }
+  }
+
   return rv;
   
 } // AddChromeListeners
@@ -790,7 +852,9 @@ nsDocShellTreeOwner :: RemoveChromeListeners ( )
     mChromeContextMenuListener->RemoveChromeListeners();
     NS_RELEASE(mChromeContextMenuListener);
   }
-  
+  if ( mChromeDragHandler )
+    mChromeDragHandler->Detach();
+
   return NS_OK;
 }
 
@@ -809,6 +873,11 @@ public:
 
     NS_DECL_ISUPPORTS
     NS_DECL_NSITOOLTIPTEXTPROVIDER
+    
+protected:
+    nsCOMPtr<nsIAtom>   mTag_dialog;
+    nsCOMPtr<nsIAtom>   mTag_dialogheader;
+    nsCOMPtr<nsIAtom>   mTag_window;
 };
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(DefaultTooltipTextProvider, nsITooltipTextProvider)
@@ -816,6 +885,12 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(DefaultTooltipTextProvider, nsITooltipTextProvider
 DefaultTooltipTextProvider::DefaultTooltipTextProvider()
 {
     NS_INIT_REFCNT();
+    
+    // There are certain element types which we don't want to use
+    // as tool tip text. 
+    mTag_dialog       = getter_AddRefs(NS_NewAtom("dialog"));
+    mTag_dialogheader = getter_AddRefs(NS_NewAtom("dialogheader"));
+    mTag_window       = getter_AddRefs(NS_NewAtom("window"));   
 }
 
 /* void getNodeText (in nsIDOMNode aNode, out wstring aText); */
@@ -823,7 +898,7 @@ NS_IMETHODIMP DefaultTooltipTextProvider::GetNodeText(nsIDOMNode *aNode, PRUnich
 {
   NS_ENSURE_ARG_POINTER(aNode);
   NS_ENSURE_ARG_POINTER(aText);
-
+    
   nsString outText;
 
   PRBool found = PR_FALSE;
@@ -831,15 +906,24 @@ NS_IMETHODIMP DefaultTooltipTextProvider::GetNodeText(nsIDOMNode *aNode, PRUnich
   while ( !found && current ) {
     nsCOMPtr<nsIDOMElement> currElement ( do_QueryInterface(current) );
     if ( currElement ) {
-      // first try the normal title attribute...
-      currElement->GetAttribute(NS_LITERAL_STRING("title"), outText);
-      if ( outText.Length() )
-        found = PR_TRUE;
-      else {
-        // ...ok, that didn't work, try it in the XLink namespace
-        currElement->GetAttributeNS(NS_LITERAL_STRING("http://www.w3.org/1999/xlink"), NS_LITERAL_STRING("title"), outText);
-        if ( outText.Length() )
-          found = PR_TRUE;
+      nsCOMPtr<nsIContent> content(do_QueryInterface(currElement));
+      if (content) {
+        nsCOMPtr<nsIAtom> tagAtom;
+        content->GetTag(*getter_AddRefs(tagAtom));
+        if (tagAtom != mTag_dialog &&
+            tagAtom != mTag_dialogheader &&
+            tagAtom != mTag_window) {
+          // first try the normal title attribute...
+          currElement->GetAttribute(NS_LITERAL_STRING("title"), outText);
+          if ( outText.Length() )
+            found = PR_TRUE;
+          else {
+            // ...ok, that didn't work, try it in the XLink namespace
+            currElement->GetAttributeNS(NS_LITERAL_STRING("http://www.w3.org/1999/xlink"), NS_LITERAL_STRING("title"), outText);
+            if ( outText.Length() )
+              found = PR_TRUE;
+          }
+        }
       }
     }
     
@@ -908,23 +992,8 @@ ChromeTooltipListener :: ~ChromeTooltipListener ( )
 NS_IMETHODIMP
 ChromeTooltipListener :: AddChromeListeners ( )
 {  
-  if ( !mEventReceiver ) {
-    nsCOMPtr<nsIDOMWindow> domWindow;
-    mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-    NS_ENSURE_TRUE(domWindow, NS_ERROR_FAILURE);
-
-    nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
-    NS_ENSURE_TRUE(domWindowPrivate, NS_ERROR_FAILURE);
-    nsCOMPtr<nsIDOMWindowInternal> rootWindow;
-    domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
-    NS_ENSURE_TRUE(rootWindow, NS_ERROR_FAILURE);
-    nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-    nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
-    piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-    NS_ENSURE_TRUE(chromeHandler, NS_ERROR_FAILURE);
-
-    mEventReceiver = do_QueryInterface(chromeHandler);
-  }
+  if ( !mEventReceiver )
+    GetEventReceiver(mWebBrowser, getter_AddRefs(mEventReceiver));
   
   // Register the appropriate events for tooltips, but only if
   // the embedding chrome cares.
@@ -1134,7 +1203,7 @@ ChromeTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
     if ( eventTarget )
       mPossibleTooltipNode = do_QueryInterface(eventTarget);
     if ( mPossibleTooltipNode ) {
-      nsresult rv = mTooltipTimer->Init(sTooltipCallback, this, kTooltipShowTime, NS_PRIORITY_HIGH);
+      nsresult rv = mTooltipTimer->Init(sTooltipCallback, this, kTooltipShowTime, PR_TRUE);
       if (NS_FAILED(rv))
         mPossibleTooltipNode = nsnull;
     }
@@ -1153,7 +1222,7 @@ ChromeTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
 // Tell the registered chrome that they should show the tooltip
 //
 NS_IMETHODIMP
-ChromeTooltipListener :: ShowTooltip ( PRInt32 inXCoords, PRInt32 inYCoords, const nsAReadableString & inTipText )
+ChromeTooltipListener :: ShowTooltip ( PRInt32 inXCoords, PRInt32 inYCoords, const nsAString & inTipText )
 {
   nsresult rv = NS_OK;
   
@@ -1266,7 +1335,7 @@ ChromeTooltipListener :: CreateAutoHideTimer ( )
   
   mAutoHideTimer = do_CreateInstance("@mozilla.org/timer;1");
   if ( mAutoHideTimer )
-    mAutoHideTimer->Init(sAutoHideCallback, this, kTooltipAutoHideTime, NS_PRIORITY_HIGH);
+    mAutoHideTimer->Init(sAutoHideCallback, this, kTooltipAutoHideTime, PR_TRUE);
 
 } // CreateAutoHideTimer
 
@@ -1375,23 +1444,8 @@ ChromeContextMenuListener :: RemoveContextMenuListener()
 NS_IMETHODIMP
 ChromeContextMenuListener :: AddChromeListeners ( )
 {  
-  if ( !mEventReceiver ) {
-    nsCOMPtr<nsIDOMWindow> domWindow;
-    mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-    NS_ENSURE_TRUE(domWindow, NS_ERROR_FAILURE);
-
-    nsCOMPtr<nsPIDOMWindow> domWindowPrivate = do_QueryInterface(domWindow);
-    NS_ENSURE_TRUE(domWindowPrivate, NS_ERROR_FAILURE);
-    nsCOMPtr<nsIDOMWindowInternal> rootWindow;
-    domWindowPrivate->GetPrivateRoot(getter_AddRefs(rootWindow));
-    NS_ENSURE_TRUE(rootWindow, NS_ERROR_FAILURE);
-    nsCOMPtr<nsIChromeEventHandler> chromeHandler;
-    nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(rootWindow));
-    piWin->GetChromeEventHandler(getter_AddRefs(chromeHandler));
-    NS_ENSURE_TRUE(chromeHandler, NS_ERROR_FAILURE);
-
-    mEventReceiver = do_QueryInterface(chromeHandler);
-  }
+  if ( !mEventReceiver )
+    GetEventReceiver(mWebBrowser, getter_AddRefs(mEventReceiver));
   
   // Register the appropriate events for context menus, but only if
   // the embedding chrome cares.
@@ -1467,27 +1521,27 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
       element->GetTagName(tag);
 
       // Test what kind of element we're dealing with here
-      if (!Compare(tag, NS_LITERAL_STRING("img"), nsCaseInsensitiveStringComparator()))
+      if (tag.Equals(NS_LITERAL_STRING("img"), nsCaseInsensitiveStringComparator()))
       {
         flags |= nsIContextMenuListener::CONTEXT_IMAGE;
         targetDOMnode = node;
         // if we see an image, keep searching for a possible anchor
       }
-      else if (!Compare(tag, NS_LITERAL_STRING("input"), nsCaseInsensitiveStringComparator()))
+      else if (tag.Equals(NS_LITERAL_STRING("input"), nsCaseInsensitiveStringComparator()))
       {
         // INPUT element - button, combo, checkbox, text etc.
         flags |= nsIContextMenuListener::CONTEXT_INPUT;
         targetDOMnode = node;
         break; // exit do-while
       }
-      else if (!Compare(tag, NS_LITERAL_STRING("textarea"), nsCaseInsensitiveStringComparator()))
+      else if (tag.Equals(NS_LITERAL_STRING("textarea"), nsCaseInsensitiveStringComparator()))
       {
         // text area
         flags |= nsIContextMenuListener::CONTEXT_TEXT;
         targetDOMnode = node;
         break; // exit do-while
       }
-      else if (!Compare(tag, NS_LITERAL_STRING("html"), nsCaseInsensitiveStringComparator()))
+      else if (tag.Equals(NS_LITERAL_STRING("html"), nsCaseInsensitiveStringComparator()))
       {
         // only care about this if no other context was found.
         if (!flags) {
@@ -1496,6 +1550,12 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
         }
         break; // exit do-while
       }
+      else if ( tag.EqualsWithConversion("object", PR_TRUE) ||  /* XXX what about images and documents? */
+                tag.EqualsWithConversion("embed",  PR_TRUE) ||
+                tag.EqualsWithConversion("applet", PR_TRUE) )
+      {                // always consume events for plugins and Java 
+        return NS_OK;  // who may throw their own context menus
+      }
 
       // Test if the element has an associated link
       nsCOMPtr<nsIDOMNamedNodeMap> attributes;
@@ -1503,8 +1563,7 @@ ChromeContextMenuListener :: ContextMenu ( nsIDOMEvent* aMouseEvent )
       if (attributes)
       {
         nsCOMPtr<nsIDOMNode> hrefNode;
-        nsAutoString href(NS_LITERAL_STRING("href"));
-        attributes->GetNamedItem(href, getter_AddRefs(hrefNode));
+        attributes->GetNamedItem(NS_LITERAL_STRING("href"), getter_AddRefs(hrefNode));
         if (hrefNode)
         {
           flags |= nsIContextMenuListener::CONTEXT_LINK;
