@@ -53,7 +53,8 @@
 #include "nsIURI.h"
 #include "nsIWebBrowserPersist.h"
 #include "nsCWebBrowserPersist.h"
-#include "nsIPrintSettings.h"
+#include "nsIWebBrowserPrint.h"
+#include "nsIServiceManager.h"
 
 // for painting the background window
 #include "nsIRenderingContext.h"
@@ -61,17 +62,10 @@
 
 // Printing Includes
 #include "nsIContentViewer.h"
-#include "nsIContentViewerFile.h"
-
-// Print Options
-#include "nsIPrintOptions.h"
-#include "nsGfxCIID.h"
-#include "nsIServiceManager.h"
 
 // PSM2 includes
 #include "nsISecureBrowserUI.h"
 
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 static NS_DEFINE_CID(kWebShellCID, NS_WEB_SHELL_CID);
 static NS_DEFINE_CID(kChildCID, NS_CHILD_CID);
 static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
@@ -84,6 +78,7 @@ static NS_DEFINE_CID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
 nsWebBrowser::nsWebBrowser() : mDocShellTreeOwner(nsnull), 
    mInitInfo(nsnull),
    mContentType(typeContentWrapper),
+   mActivating(PR_FALSE),
    mParentNativeWindow(nsnull),
    mProgressListener(nsnull),
    mBackgroundColor(0),
@@ -159,7 +154,6 @@ NS_INTERFACE_MAP_BEGIN(nsWebBrowser)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserSetup)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserPersist)
     NS_INTERFACE_MAP_ENTRY(nsIWebBrowserFocus)
-    NS_INTERFACE_MAP_ENTRY(nsIWebBrowserPrint)
     NS_INTERFACE_MAP_ENTRY(nsIWebProgressListener)
     NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
 NS_INTERFACE_MAP_END
@@ -175,8 +169,22 @@ NS_IMETHODIMP nsWebBrowser::GetInterface(const nsIID& aIID, void** aSink)
    if(NS_SUCCEEDED(QueryInterface(aIID, aSink)))
       return NS_OK;
 
-   if(mDocShell)
-      return mDocShellAsReq->GetInterface(aIID, aSink);
+   if (mDocShell) {
+       if (aIID.Equals(NS_GET_IID(nsIWebBrowserPrint))) {
+           nsCOMPtr<nsIContentViewer> viewer;
+           mDocShell->GetContentViewer(getter_AddRefs(viewer));
+           if (viewer) {
+               nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint(do_QueryInterface(viewer));
+               nsIWebBrowserPrint* print = (nsIWebBrowserPrint*)webBrowserPrint.get();
+               NS_ASSERTION(print, "This MUST support this interface!");
+               NS_ADDREF(print);
+               *aSink = print;
+               return NS_OK;
+           }
+       } else {
+           return mDocShellAsReq->GetInterface(aIID, aSink);
+       }
+   }
 
    return NS_NOINTERFACE;
 }
@@ -226,7 +234,7 @@ NS_IMETHODIMP nsWebBrowser::BindListener(nsISupports *aListener, const nsIID& aI
     if (aIID.Equals(NS_GET_IID(nsIWebProgressListener))) {
         nsCOMPtr<nsIWebProgressListener> listener = do_QueryInterface(aListener, &rv);
         if (NS_FAILED(rv)) return rv;
-        rv = mWebProgress->AddProgressListener(listener);
+        rv = mWebProgress->AddProgressListener(listener, nsIWebProgress::NOTIFY_ALL);
     }
     else if (aIID.Equals(NS_GET_IID(nsISHistoryListener))) {      
       nsCOMPtr<nsISHistory> shistory(do_GetInterface(mDocShell, &rv));
@@ -434,6 +442,8 @@ NS_IMETHODIMP nsWebBrowser::SetItemType(PRInt32 aItemType)
 {
     NS_ENSURE_TRUE((aItemType == typeContentWrapper || aItemType == typeChromeWrapper), NS_ERROR_FAILURE);
     mContentType = aItemType;
+    if (mDocShellAsItem)
+        mDocShellAsItem->SetItemType(mContentType == typeChromeWrapper ? typeChrome : typeContent);
     return NS_OK;
 }
 
@@ -729,6 +739,12 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
             // obsolete
         }
         break;
+    case nsIWebBrowserSetup::SETUP_IS_CHROME_WRAPPER:
+        {
+           NS_ENSURE_TRUE((aValue == PR_TRUE || aValue == PR_FALSE), NS_ERROR_INVALID_ARG);
+           SetItemType(aValue ? typeChromeWrapper : typeContentWrapper);
+        }
+        break;
     default:
         rv = NS_ERROR_INVALID_ARG;
   
@@ -741,8 +757,8 @@ NS_IMETHODIMP nsWebBrowser::SetProperty(PRUint32 aId, PRUint32 aValue)
 // nsWebBrowser::nsIWebProgressListener
 //*****************************************************************************
 
-/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long aStateFlags, in unsigned long aStatus); */
-NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 aStateFlags, PRUint32 aStatus)
+/* void onStateChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long aStateFlags, in nsresult aStatus); */
+NS_IMETHODIMP nsWebBrowser::OnStateChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 aStateFlags, nsresult aStatus)
 {
     if (mPersist)
     {
@@ -793,8 +809,8 @@ NS_IMETHODIMP nsWebBrowser::OnStatusChange(nsIWebProgress *aWebProgress, nsIRequ
     return NS_OK;
 }
 
-/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in long state); */
-NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRInt32 state)
+/* void onSecurityChange (in nsIWebProgress aWebProgress, in nsIRequest aRequest, in unsigned long state); */
+NS_IMETHODIMP nsWebBrowser::OnSecurityChange(nsIWebProgress *aWebProgress, nsIRequest *aRequest, PRUint32 state)
 {
     if (mProgressListener)
     {
@@ -1027,15 +1043,15 @@ NS_IMETHODIMP nsWebBrowser::Create()
                               nsnull, nsnull, nsnull, &widgetInit);  
       }
 
+   nsCOMPtr<nsIDocShell> docShell(do_CreateInstance(kWebShellCID));
+   NS_ENSURE_SUCCESS(SetDocShell(docShell), NS_ERROR_FAILURE);
+
    // get the system default window background colour
    {
       nsCOMPtr<nsILookAndFeel> laf = do_GetService(kLookAndFeelCID);
       laf->GetColor(nsILookAndFeel::eColor_WindowBackground, mBackgroundColor);
    }
 
-   nsCOMPtr<nsIDocShell> docShell(do_CreateInstance(kWebShellCID));
-   NS_ENSURE_SUCCESS(SetDocShell(docShell), NS_ERROR_FAILURE);
-
    // the docshell has been set so we now have our listener registrars.
    if (mListenerArray) {
       // we had queued up some listeners, let's register them now.
@@ -1102,7 +1118,7 @@ NS_IMETHODIMP nsWebBrowser::Create()
    if (NS_SUCCEEDED(rv))
    {
        mSecurityUI = do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
-       if (NS_SUCCEEDED(rv))mSecurityUI->Init(domWindow, nsnull);
+       if (NS_SUCCEEDED(rv))mSecurityUI->Init(domWindow);
    }
 
    mDocShellTreeOwner->AddToWatcher(); // evil twin of Remove in SetDocShell(0)
@@ -1297,6 +1313,20 @@ NS_IMETHODIMP nsWebBrowser::SetVisibility(PRBool aVisibility)
    return NS_OK;
 }
 
+NS_IMETHODIMP nsWebBrowser::GetEnabled(PRBool *aEnabled)
+{
+  if (mInternalWidget)
+    return mInternalWidget->IsEnabled(aEnabled);
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsWebBrowser::SetEnabled(PRBool aEnabled)
+{
+  if (mInternalWidget)
+    return mInternalWidget->Enable(aEnabled);
+  return NS_ERROR_FAILURE;
+}
+
 NS_IMETHODIMP nsWebBrowser::GetMainWidget(nsIWidget** mainWidget)
 {
    NS_ENSURE_ARG_POINTER(mainWidget);
@@ -1574,6 +1604,7 @@ NS_IMETHODIMP nsWebBrowser::GetPrimaryContentWindow(nsIDOMWindowInternal **aDOMW
   *aDOMWindow = 0;
 
   nsCOMPtr<nsIDocShellTreeItem> item;
+  NS_ENSURE_TRUE(mDocShellTreeOwner, NS_ERROR_FAILURE);
   mDocShellTreeOwner->GetPrimaryContentShell(getter_AddRefs(item));
   NS_ENSURE_TRUE(item, NS_ERROR_FAILURE);
 
@@ -1597,6 +1628,13 @@ NS_IMETHODIMP nsWebBrowser::GetPrimaryContentWindow(nsIDOMWindowInternal **aDOMW
 /* void activate (); */
 NS_IMETHODIMP nsWebBrowser::Activate(void)
 {
+  // stop infinite recursion from windows with onfocus handlers that
+  // reactivate the window
+  if (mActivating)
+    return NS_OK;
+
+  mActivating = PR_TRUE;
+
   // try to set focus on the last focused window as stored in the
   // focus controller object.
   nsCOMPtr<nsIDOMWindow> domWindowExternal;
@@ -1604,21 +1642,23 @@ NS_IMETHODIMP nsWebBrowser::Activate(void)
   nsCOMPtr<nsIDOMWindowInternal> domWindow;
   domWindow = do_QueryInterface(domWindowExternal);
   nsCOMPtr<nsPIDOMWindow> piWin(do_QueryInterface(domWindow));
-  nsCOMPtr<nsIFocusController> focusController;
-  piWin->GetRootFocusController(getter_AddRefs(focusController));
   PRBool needToFocus = PR_TRUE;
-  if (focusController) {
-    // Go ahead and mark the focus controller as being active.  We have
-    // to do this even before the activate message comes in.
-    focusController->SetActive(PR_TRUE);
+  if (piWin) {
+    nsCOMPtr<nsIFocusController> focusController;
+    piWin->GetRootFocusController(getter_AddRefs(focusController));
+    if (focusController) {
+      // Go ahead and mark the focus controller as being active.  We have
+      // to do this even before the activate message comes in.
+      focusController->SetActive(PR_TRUE);
 
-    nsCOMPtr<nsIDOMWindowInternal> focusedWindow;
-    focusController->GetFocusedWindow(getter_AddRefs(focusedWindow));
-    if (focusedWindow) {
-      needToFocus = PR_FALSE;
-      focusController->SetSuppressFocus(PR_TRUE, "Activation Suppression");
-      domWindow->Focus(); // This sets focus, but we'll ignore it.  
-                         // A subsequent activate will cause us to stop suppressing.
+      nsCOMPtr<nsIDOMWindowInternal> focusedWindow;
+      focusController->GetFocusedWindow(getter_AddRefs(focusedWindow));
+      if (focusedWindow) {
+        needToFocus = PR_FALSE;
+        focusController->SetSuppressFocus(PR_TRUE, "Activation Suppression");
+        domWindow->Focus(); // This sets focus, but we'll ignore it.  
+                           // A subsequent activate will cause us to stop suppressing.
+      }
     }
   }
 
@@ -1656,32 +1696,38 @@ NS_IMETHODIMP nsWebBrowser::Activate(void)
     }
   }
 
+  mActivating = PR_FALSE;
   return NS_OK;
 }
 
 /* void deactivate (); */
 NS_IMETHODIMP nsWebBrowser::Deactivate(void)
 {
-    /* At this time we don't clear mWWatch's ActiveWindow; we just allow
-       the presumed other newly active window to set it when it comes in.
-       This seems harmless and maybe safer, but we have no real evidence
-       either way just yet. */
-
-    NS_ENSURE_STATE(mDocShell);
-    nsCOMPtr<nsIPresShell> presShell;
-    mDocShell->GetPresShell(getter_AddRefs(presShell));
-    if(!presShell)
-        return NS_OK;
-
-    nsCOMPtr<nsIDOMWindow> domWindow;
-    GetContentDOMWindow(getter_AddRefs(domWindow));
-    if (domWindow) {
-        nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(domWindow);
-        if(privateDOMWindow)
-          privateDOMWindow->Deactivate();
-    }
+  /* At this time we don't clear mWWatch's ActiveWindow; we just allow
+     the presumed other newly active window to set it when it comes in.
+     This seems harmless and maybe safer, but we have no real evidence
+     either way just yet. */
 
+  NS_ENSURE_STATE(mDocShell);
+  nsCOMPtr<nsIPresShell> presShell;
+  mDocShell->GetPresShell(getter_AddRefs(presShell));
+  if(!presShell)
     return NS_OK;
+
+  nsCOMPtr<nsIDOMWindow> domWindow;
+  GetContentDOMWindow(getter_AddRefs(domWindow));
+  if (domWindow) {
+    nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(domWindow);
+    if(privateDOMWindow) {
+      nsCOMPtr<nsIFocusController> focusController;
+      privateDOMWindow->GetRootFocusController(getter_AddRefs(focusController));
+      if (focusController)
+        focusController->SetActive(PR_FALSE);
+      privateDOMWindow->Deactivate();
+    }
+  }
+
+  return NS_OK;
 }
 
 /* void setFocusAtFirstElement (); */
@@ -1756,149 +1802,3 @@ NS_IMETHODIMP nsWebBrowser::SetFocusedElement(nsIDOMElement * aFocusedElement)
   return NS_OK;
 }
 
-
-/* helper function */
-nsresult nsWebBrowser::DoPrintOrPrintPreview(nsIDOMWindow *aDOMWindow, 
-                                             nsIPrintSettings *aThePrintSettings,
-                                             nsIWebProgressListener *aProgressListener,
-                                             PRBool            aDoPrinting)
-{
-  // this is if we want to have the ability to switch between having 
-  // each window have their own PrintSettings, or having them all share it.
-#ifdef ENABLE_GLOBAL_PRINTSETTINGS
-  PRBool useGlobalPrintSettings = PR_FALSE;
-  nsresult rv = NS_ERROR_FAILURE;
-  nsCOMPtr<nsIPrintOptions> printService = do_GetService(kPrintOptionsCID, &rv);
-  if (NS_SUCCEEDED(rv) && printService) {
-    nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID);
-    if (pref) {
-      pref->GetBoolPref(&useGlobalPrintSettings);
-      if (useGlobalPrintSettings && aThePrintSettings != nsnull) {
-        printService->GetPrintSettingsValues("print.use_global_printsettings", aThePrintSettings);
-      }
-    }
-  }
-#endif
-
-  nsresult rv = NS_ERROR_FAILURE;
-  // XXX this next line may need to be changed
-  // it is unclear what the correct way is to get the document.
-  nsCOMPtr<nsIDOMWindow> thisDOMWin;
-  GetContentDOMWindow(getter_AddRefs(thisDOMWin));
-  if (aDOMWindow == thisDOMWin.get()) {
-     nsCOMPtr<nsIContentViewer> contentViewer;
-     mDocShell->GetContentViewer(getter_AddRefs(contentViewer));
-     if (contentViewer) {
-       nsCOMPtr<nsIContentViewerFile> contentViewerFile(do_QueryInterface(contentViewer));
-       if (contentViewerFile) {
-         if (aDoPrinting) {
-           // should we print silent?
-           PRBool printSilent = PR_FALSE;
-           if (aThePrintSettings) {
-             aThePrintSettings->GetPrintSilent(&printSilent);
-           }
-           rv = contentViewerFile->Print(printSilent, aThePrintSettings, aProgressListener);
-         } else {
-           rv = contentViewerFile->PrintPreview(aThePrintSettings);
-         }
-       }
-     }
-  }
-
-#ifdef ENABLE_GLOBAL_PRINTSETTINGS
-  if (useGlobalPrintSettings && aThePrintSettings != nsnull && printService) {
-    printService->SetPrintSettingsValues(aThePrintSettings);
-  }
-#endif
-
-  return rv;
-}
-
-//*****************************************************************************
-// nsWebBrowser::nsIWebBrowserPrint
-//*****************************************************************************   
-
-/* void Print (in nsIDOMWindow aDOMWindow, in nsIPrintSettings aThePrintSettings); */
-NS_IMETHODIMP nsWebBrowser::Print(nsIDOMWindow *aDOMWindow, 
-                                  nsIPrintSettings *aThePrintSettings,
-                                  nsIWebProgressListener * aProgressListener)
-{
-  return DoPrintOrPrintPreview(aDOMWindow, aThePrintSettings, aProgressListener, PR_TRUE);
-}
-
-/* void PrintPreview (in nsIPrintSettings aThePrintSettings); */
-NS_IMETHODIMP nsWebBrowser::PrintPreview(nsIDOMWindow *aDOMWindow, 
-                                         nsIPrintSettings *aThePrintSettings)
-{
-  return DoPrintOrPrintPreview(aDOMWindow, aThePrintSettings, nsnull, PR_FALSE);
-}
-
-  /* void Cancel (); */
-NS_IMETHODIMP nsWebBrowser::Cancel(void)
-{
-  nsresult rv;
-  nsCOMPtr<nsIPrintOptions> printService = 
-           do_GetService(kPrintOptionsCID, &rv);
-  if (NS_SUCCEEDED(rv) && printService) {
-    return printService->SetIsCancelled(PR_TRUE);
-  }
-  return NS_OK;
-}
-
-/* attribute nsIPrintSettings printSettings; */
-/**
- * Creates a PrintSettings Object to be used for Printing 
- */
-NS_IMETHODIMP nsWebBrowser::GetPrintSettings(nsIPrintSettings * *aPrintSettings)
-{
-  nsresult rv = NS_OK;
-  if (!mPrintSettings) {
-    nsCOMPtr<nsIPrintOptions> printService = do_GetService(kPrintOptionsCID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      printService->CreatePrintSettings(getter_AddRefs(mPrintSettings));
-    }
-  }
-  *aPrintSettings = mPrintSettings;
-  NS_IF_ADDREF(*aPrintSettings);
-  return rv;
-}
-
-NS_IMETHODIMP nsWebBrowser::SetPrintSettings(nsIPrintSettings * aPrintSettings)
-{
-  mPrintSettings = aPrintSettings;
-  return NS_OK;
-}
-
-
-/* attribute nsIPrintSettings globalPrintSettings; */
-/**
- * gets/set the PS into the the PrintOptions
- */
-NS_IMETHODIMP nsWebBrowser::GetGlobalPrintSettings(nsIPrintSettings * *aPrintSettings)
-{
-  NS_ENSURE_ARG_POINTER(aPrintSettings);
-  NS_ENSURE_ARG_POINTER(*aPrintSettings);
-
-  nsresult rv = NS_OK;
-  if (aPrintSettings != nsnull && *aPrintSettings != nsnull) {
-    nsCOMPtr<nsIPrintOptions> printService = do_GetService(kPrintOptionsCID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      printService->GetPrintSettingsValues(aPrintSettings);
-    }
-  }
-  *aPrintSettings = mPrintSettings;
-  NS_IF_ADDREF(*aPrintSettings);
-  return rv;
-}
-
-NS_IMETHODIMP nsWebBrowser::SetGlobalPrintSettings(nsIPrintSettings * aPrintSettings)
-{
-  NS_ENSURE_ARG_POINTER(aPrintSettings);
-  nsresult rv = NS_OK;
-  nsCOMPtr<nsIPrintOptions> printService = do_GetService(kPrintOptionsCID, &rv);
-  if (NS_SUCCEEDED(rv)) {
-    printService->SetPrintSettingsValues(aPrintSettings);
-  }
-  return NS_OK;
-}
-
