@@ -19,19 +19,16 @@
  * Contributors:
  *   Conrad Carlen <ccarlen@netscape.com>
  *   Simon Fraser  <sfraser@netscape.com>
+ *   Akkana Peck  <akkana@netscape.com>
  */
 
 #include "nsWebBrowserFind.h"
 
+// Only need this for NS_FIND_CONTRACTID,
+// else we could use nsIDOMRange.h and nsIFind.h.
 #include "nsFind.h"
 
 #include "nsIComponentManager.h"
-#ifdef TEXT_SVCS_TEST
-#include "nsITextServicesDocument.h"
-#include "nsTextServicesCID.h"
-#include "nsIServiceManager.h"
-#include "nsIPref.h"
-#endif /* TEXT_SVCS_TEST */
 #include "nsIScriptGlobalObject.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
@@ -42,27 +39,25 @@
 #include "nsIEnumerator.h"
 #include "nsIDocShellTreeItem.h"
 #include "nsIPresShell.h"
+#include "nsIPresContext.h"
+#include "nsIEventStateManager.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
 #include "nsIFocusController.h"
+#include "nsISelectionController.h"
 #include "nsISelection.h"
 #include "nsReadableUtils.h"
+#include "nsIDOMHTMLElement.h"
+#include "nsIDOMHTMLDocument.h"
+#include "nsIContent.h"
+#include "nsContentCID.h"
 
 #if DEBUG
 #include "nsIWebNavigation.h"
 #include "nsXPIDLString.h"
 #endif
 
-#if defined(TEXT_SVCS_TEST) && defined(XP_UNIX) && defined(DEBUG)
-#include <sys/time.h>
-#endif
-
-#ifdef TEXT_SVCS_TEST
-PRBool gUseTextServices = PR_FALSE;
-
-static NS_DEFINE_CID(kCTextServicesDocumentCID, NS_TEXTSERVICESDOCUMENT_CID);
-static NS_DEFINE_CID(kPrefServiceCID,       NS_PREF_CID);
-#endif /* TEXT_SVCS_TEST */
+static NS_DEFINE_IID(kRangeCID, NS_RANGE_CID);
 
 //*****************************************************************************
 // nsWebBrowserFind
@@ -91,17 +86,6 @@ NS_IMETHODIMP nsWebBrowserFind::FindNext(PRBool *outDidFind)
 {
     nsresult rv;
 
-#ifdef TEXT_SVCS_TEST
-    // Figure out whether to use the old or new version:
-    gUseTextServices = PR_TRUE;
-    nsCOMPtr<nsIPref> prefService = do_GetService(kPrefServiceCID, &rv);
-    if (NS_SUCCEEDED(rv) && prefService) {
-        PRBool newFind = PR_FALSE;
-        prefService->GetBoolPref("browser.new_find", &newFind);
-        gUseTextServices = !newFind;
-    }
-#endif /* TEXT_SVCS_TEST */
-
     NS_ENSURE_ARG_POINTER(outDidFind);
     *outDidFind = PR_FALSE;
 
@@ -109,9 +93,9 @@ NS_IMETHODIMP nsWebBrowserFind::FindNext(PRBool *outDidFind)
 
     nsCOMPtr<nsIDOMWindow> searchFrame = do_QueryReferent(mCurrentSearchFrame);
     NS_ENSURE_TRUE(searchFrame, NS_ERROR_NOT_INITIALIZED);
-    
+
     // first, look in the current frame. If found, return.
-    rv = SearchInFrame(searchFrame, outDidFind);
+    rv = SearchInFrame(searchFrame, PR_FALSE, outDidFind);
     if (NS_FAILED(rv)) return rv;
     if (*outDidFind)
         return OnFind(searchFrame);     // we are done
@@ -162,21 +146,31 @@ NS_IMETHODIMP nsWebBrowserFind::FindNext(PRBool *outDidFind)
         {
             searchFrame = do_GetInterface(curItem, &rv);
             if (NS_FAILED(rv)) break;
-            
+
             OnStartSearchFrame(searchFrame);
 
-            rv = SearchInFrame(searchFrame, outDidFind);
+            rv = SearchInFrame(searchFrame, PR_FALSE, outDidFind);
             if (NS_FAILED(rv)) return rv;
             if (*outDidFind)
                 return OnFind(searchFrame);     // we are done
-                
+
             OnEndSearchFrame(searchFrame);
         }
-        
+
         if (curItem.get() == startingItem.get())
             doFind = PR_TRUE;       // start looking in frames after this one
     };
 
+    if (!mWrapFind)
+    {
+        // remember where we left off
+        SetCurrentSearchFrame(searchFrame);
+        return NS_OK;
+    }
+
+    // From here on, we're wrapping, first through the other frames,
+    // then finally from the beginning of the starting frame back to
+    // the starting point.
 
     // because nsISimpleEnumerator is totally lame and isn't resettable, I
     // have to make a new one
@@ -195,18 +189,19 @@ NS_IMETHODIMP nsWebBrowserFind::FindNext(PRBool *outDidFind)
 
         if (curItem.get() == startingItem.get())
         {
-            // ideally, we should search the part of the starting frame up
-            // to the point where we left off. We could do this by keeping
-            // its nsIFindAndReplace around.        
+            rv = SearchInFrame(searchFrame, PR_TRUE, outDidFind);
+            if (NS_FAILED(rv)) return rv;
+            if (*outDidFind)
+                return OnFind(searchFrame);        // we are done
             break;
         }
 
         searchFrame = do_GetInterface(curItem, &rv);
         if (NS_FAILED(rv)) break;
-        
+
         OnStartSearchFrame(searchFrame);
 
-        rv = SearchInFrame(searchFrame, outDidFind);
+        rv = SearchInFrame(searchFrame, PR_FALSE, outDidFind);
         if (NS_FAILED(rv)) return rv;
         if (*outDidFind)
             return OnFind(searchFrame);        // we are done
@@ -289,6 +284,198 @@ NS_IMETHODIMP nsWebBrowserFind::SetMatchCase(PRBool aMatchCase)
     return NS_OK;
 }
 
+void nsWebBrowserFind::SetSelectionAndScroll(nsIDOMRange* aRange,
+                                             nsISelectionController *aSelCon)
+{
+  nsCOMPtr<nsISelection> selection;
+  if (!aSelCon) return;
+
+  aSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL,
+                        getter_AddRefs(selection));
+  if (!selection) return;
+  selection->RemoveAllRanges();
+  selection->AddRange(aRange);
+
+  // Scroll if necessary to make the selection visible:
+  aSelCon->ScrollSelectionIntoView
+    (nsISelectionController::SELECTION_NORMAL,
+     nsISelectionController::SELECTION_FOCUS_REGION, PR_TRUE);
+}
+
+// Adapted from nsTextServicesDocument::GetDocumentContentRootNode
+nsresult nsWebBrowserFind::GetRootNode(nsIDOMDocument* aDomDoc,
+                                       nsIDOMNode **aNode)
+{
+  nsresult rv;
+
+  NS_ENSURE_ARG_POINTER(aNode);
+  *aNode = 0;
+
+  nsCOMPtr<nsIDOMHTMLDocument> htmlDoc = do_QueryInterface(aDomDoc);
+  if (htmlDoc)
+  {
+    // For HTML documents, the content root node is the body.
+    nsCOMPtr<nsIDOMHTMLElement> bodyElement;
+    rv = htmlDoc->GetBody(getter_AddRefs(bodyElement));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_ARG_POINTER(bodyElement);
+    return bodyElement->QueryInterface(NS_GET_IID(nsIDOMNode),
+                                       (void **)aNode);
+  }
+
+  // For non-HTML documents, the content root node will be the doc element.
+  nsCOMPtr<nsIDOMElement> docElement;
+  rv = aDomDoc->GetDocumentElement(getter_AddRefs(docElement));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_ARG_POINTER(docElement);
+  return docElement->QueryInterface(NS_GET_IID(nsIDOMNode), (void **)aNode);
+}
+
+nsresult nsWebBrowserFind::SetRangeAroundDocument(nsIDOMRange* aSearchRange,
+                                                  nsIDOMRange* aStartPt,
+                                                  nsIDOMRange* aEndPt,
+                                                  nsIDOMDocument* aDoc)
+{
+    nsCOMPtr<nsIDOMNode> bodyNode;
+    nsresult rv = GetRootNode(aDoc, getter_AddRefs(bodyNode));
+    nsCOMPtr<nsIContent> bodyContent (do_QueryInterface(bodyNode));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_ARG_POINTER(bodyContent);
+
+    PRInt32 childCount;
+    rv = bodyContent->ChildCount(childCount);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    aSearchRange->SetStart(bodyNode, 0);
+    aSearchRange->SetEnd(bodyNode, childCount);
+
+    if (mFindBackwards)
+    {
+        aStartPt->SetStart(bodyNode, childCount);
+        aStartPt->SetEnd(bodyNode, childCount);
+        aEndPt->SetStart(bodyNode, 0);
+        aEndPt->SetEnd(bodyNode, 0);
+    }
+    else
+    {
+        aStartPt->SetStart(bodyNode, 0);
+        aStartPt->SetEnd(bodyNode, 0);
+        aEndPt->SetStart(bodyNode, childCount);
+        aEndPt->SetEnd(bodyNode, childCount);
+    }
+
+    return NS_OK;
+}
+
+// Set the range to go from the end of the current selection to the
+// end of the document (forward), or beginning to beginning (reverse).
+// or around the whole document if there's no selection.
+nsresult
+nsWebBrowserFind::GetSearchLimits(nsIDOMRange* aSearchRange,
+                                  nsIDOMRange* aStartPt,
+                                  nsIDOMRange* aEndPt,
+                                  nsIDOMDocument* aDoc,
+                                  nsISelectionController* aSelCon,
+                                  PRBool aWrap)
+{
+    NS_ENSURE_ARG_POINTER(aSelCon);
+    nsCOMPtr<nsISelection> selection;
+    aSelCon->GetSelection(nsISelectionController::SELECTION_NORMAL,
+                          getter_AddRefs(selection));
+    if (!selection) return NS_ERROR_BASE;
+
+    // There is a selection.
+    PRInt32 count = -1;
+    nsresult rv = selection->GetRangeCount(&count);
+    if (count < 1)
+        return SetRangeAroundDocument(aSearchRange, aStartPt, aEndPt, aDoc);
+
+    // Need bodyNode, for the start/end of the document
+    nsCOMPtr<nsIDOMNode> bodyNode;
+    rv = GetRootNode(aDoc, getter_AddRefs(bodyNode));
+    nsCOMPtr<nsIContent> bodyContent (do_QueryInterface(bodyNode));
+    NS_ENSURE_ARG_POINTER(bodyContent);
+
+    PRInt32 childCount;
+    rv = bodyContent->ChildCount(childCount);
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    // There are four possible range endpoints we might use:
+    // DocumentStart, SelectionStart, SelectionEnd, DocumentEnd.
+
+    nsCOMPtr<nsIDOMRange> range;
+    nsCOMPtr<nsIDOMNode> node;
+    PRInt32 offset;
+
+    // Forward, not wrapping: SelEnd to DocEnd
+    if (!mFindBackwards && !aWrap)
+    {
+        // This isn't quite right, since the selection's ranges aren't
+        // necessarily in order; but they usually will be.
+        selection->GetRangeAt(count-1, getter_AddRefs(range));
+        if (!range) return NS_ERROR_UNEXPECTED;
+        range->GetEndContainer(getter_AddRefs(node));
+        if (!node) return NS_ERROR_UNEXPECTED;
+        range->GetEndOffset(&offset);
+
+        aSearchRange->SetStart(node, offset);
+        aSearchRange->SetEnd(bodyNode, childCount);
+        aStartPt->SetStart(node, offset);
+        aStartPt->SetEnd(node, offset);
+        aEndPt->SetStart(bodyNode, childCount);
+        aEndPt->SetEnd(bodyNode, childCount);
+    }
+    // Backward, not wrapping: DocStart to SelStart
+    else if (mFindBackwards && !aWrap)
+    {
+        selection->GetRangeAt(0, getter_AddRefs(range));
+        if (!range) return NS_ERROR_UNEXPECTED;
+        range->GetStartContainer(getter_AddRefs(node));
+        if (!node) return NS_ERROR_UNEXPECTED;
+        range->GetStartOffset(&offset);
+
+        aSearchRange->SetStart(bodyNode, 0);
+        aSearchRange->SetEnd(bodyNode, childCount);
+        aStartPt->SetStart(node, offset);
+        aStartPt->SetEnd(node, offset);
+        aEndPt->SetStart(bodyNode, 0);
+        aEndPt->SetEnd(bodyNode, 0);
+    }
+    // Forward, wrapping: DocStart to SelEnd
+    else if (!mFindBackwards && aWrap)
+    {
+        selection->GetRangeAt(count-1, getter_AddRefs(range));
+        if (!range) return NS_ERROR_UNEXPECTED;
+        range->GetEndContainer(getter_AddRefs(node));
+        if (!node) return NS_ERROR_UNEXPECTED;
+        range->GetEndOffset(&offset);
+
+        aSearchRange->SetStart(bodyNode, 0);
+        aSearchRange->SetEnd(bodyNode, childCount);
+        aStartPt->SetStart(bodyNode, 0);
+        aStartPt->SetEnd(bodyNode, 0);
+        aEndPt->SetStart(node, offset);
+        aEndPt->SetEnd(node, offset);
+    }
+    // Backward, wrapping: SelStart to DocEnd
+    else if (mFindBackwards && aWrap)
+    {
+        selection->GetRangeAt(0, getter_AddRefs(range));
+        if (!range) return NS_ERROR_UNEXPECTED;
+        range->GetStartContainer(getter_AddRefs(node));
+        if (!node) return NS_ERROR_UNEXPECTED;
+        range->GetStartOffset(&offset);
+
+        aSearchRange->SetStart(bodyNode, 0);
+        aSearchRange->SetEnd(bodyNode, childCount);
+        aStartPt->SetStart(bodyNode, childCount);
+        aStartPt->SetEnd(bodyNode, childCount);
+        aEndPt->SetStart(node, offset);
+        aEndPt->SetEnd(node, offset);
+    }
+    return NS_OK;
+}
+
 /* attribute boolean searchFrames; */
 NS_IMETHODIMP nsWebBrowserFind::GetSearchFrames(PRBool *aSearchFrames)
 {
@@ -369,67 +556,67 @@ NS_IMETHODIMP nsWebBrowserFind::SetSearchParentFrames(PRBool aSearchParentFrames
     return NS_OK;
 }
 
+void nsWebBrowserFind::MoveFocusToCaret(nsIDOMWindow *aWindow)
+{
+  nsCOMPtr<nsIDocShell> docShell;
+  GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
+  if (!docShell)
+    return;
+  nsCOMPtr<nsIPresShell> presShell;
+  docShell->GetPresShell(getter_AddRefs(presShell));
+  if (!presShell) 
+    return;
+
+  nsCOMPtr<nsIPresContext> presContext;
+  presShell->GetPresContext(getter_AddRefs(presContext));
+  if (!presContext) 
+    return;
+
+  nsCOMPtr<nsIEventStateManager> esm;
+  presContext->GetEventStateManager(getter_AddRefs(esm));
+
+  if (esm) {
+    PRBool isSelectionWithFocus;
+    esm->MoveFocusToCaret(PR_TRUE, &isSelectionWithFocus);
+  }
+}
+
 /*
     This method handles finding in a single window (aka frame).
 
 */
-nsresult nsWebBrowserFind::SearchInFrame(nsIDOMWindow* aWindow, PRBool* aDidFind)
+nsresult nsWebBrowserFind::SearchInFrame(nsIDOMWindow* aWindow,
+                                         PRBool aWrapping,
+                                         PRBool* aDidFind)
 {
-#if defined(TEXT_SVCS_TEST) && defined(XP_UNIX) && defined(DEBUG)
-    struct timeval timeAtStart;
-    gettimeofday(&timeAtStart, 0);
-#endif /* TEXT_SVCS_TEST */
-
     NS_ENSURE_ARG(aWindow);
-    
+    NS_ENSURE_ARG_POINTER(aDidFind);
+
+    *aDidFind = PR_FALSE;
+
     nsresult rv = NS_OK;
+    nsCOMPtr<nsIPresShell> presShell;
+
+    nsCOMPtr<nsIDOMDocument> domDoc;    
+    rv = aWindow->GetDocument(getter_AddRefs(domDoc));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_ARG_POINTER(domDoc);
 
     // if this is a different frame to last time, throw away the mTSFind
     // and make a new one. The nsIFindAndReplace is *not* stateless;
     // it remembers the last search offset etc.
     nsCOMPtr<nsIDOMWindow> searchFrame = do_QueryReferent(mCurrentSearchFrame);    
-#ifdef TEXT_SVCS_TEST
-    if (mTSFind && (searchFrame.get() != aWindow))
-        mTSFind = nsnull;       // throw away old one, if any
+    // Get the selection controller -- we'd better do this every time,
+    // since the doc might have changed since the last time.
+    nsCOMPtr<nsIDocShell> docShell;
+    rv = GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_ARG_POINTER(docShell);
+    docShell->GetPresShell(getter_AddRefs(presShell));
+    NS_ENSURE_ARG_POINTER(presShell);
 
-    if (gUseTextServices)
-    {
-        if (!mTSFind)
-        {
-            mTSFind = do_CreateInstance(NS_FINDANDREPLACE_CONTRACTID, &rv);
-            if (NS_FAILED(rv))
-                return rv;
-        }
-    }
-    else {
-
-#endif /* TEXT_SVCS_TEST */
-    if (mFind && (searchFrame.get() != aWindow))
-        mFind = nsnull;       // throw away old one, if any
     if (!mFind)
-    {
-#ifdef DEBUG
-        printf("Making a new nsFind object\n");
-#endif
-        nsCOMPtr<nsIDOMDocument> domDoc;    
-        rv = aWindow->GetDocument(getter_AddRefs(domDoc));
-        NS_ENSURE_SUCCESS(rv, rv);
-        NS_ENSURE_ARG_POINTER(domDoc);
-        nsCOMPtr<nsIDocShell> docShell;
-        rv = GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
-        NS_ENSURE_SUCCESS(rv, rv);
-        NS_ENSURE_ARG_POINTER(docShell);
-        nsCOMPtr<nsIPresShell> presShell;
-        docShell->GetPresShell(getter_AddRefs(presShell));
-        NS_ENSURE_ARG_POINTER(presShell);
-
-        mFind = new nsFind();   // XXX make this a service
-        rv = mFind->SetDomDoc(domDoc, presShell);
-    }
-
-#ifdef TEXT_SVCS_TEST
-    }
-#endif /* TEXT_SVCS_TEST */
+        mFind = do_CreateInstance(NS_FIND_CONTRACTID, &rv);
 
 #if DEBUG_smfr
     {
@@ -449,50 +636,49 @@ nsresult nsWebBrowserFind::SearchInFrame(nsIDOMWindow* aWindow, PRBool* aDidFind
     }
 #endif
 
-#ifdef TEXT_SVCS_TEST
-    if (gUseTextServices)
-    {
-        nsCOMPtr<nsITextServicesDocument> txtDoc;
-        rv = MakeTSDocument(aWindow, getter_AddRefs(txtDoc));
-        if (NS_FAILED(rv) || !txtDoc)
-            return rv;
-    
-        (void) mTSFind->SetCaseSensitive(mMatchCase);
-        (void) mTSFind->SetFindBackwards(mFindBackwards);
-        (void) mTSFind->SetWrapFind(mWrapFind);
-        (void) mTSFind->SetEntireWord(mEntireWord);
-
-        rv = mTSFind->SetTsDoc(txtDoc);
-        if (NS_FAILED(rv))
-            return rv;
-
-        rv =  mTSFind->Find(mSearchString.get(), aDidFind);
-    }
-    else
-    {
-#endif /* TEXT_SVCS_TEST */
-
     (void) mFind->SetCaseSensitive(mMatchCase);
     (void) mFind->SetFindBackwards(mFindBackwards);
-    (void) mFind->SetWrapFind(mWrapFind);
-    (void) mFind->SetEntireWord(mEntireWord);
 
-    rv =  mFind->Find(mSearchString.get(), aDidFind);
+    // XXX Make and set a line breaker here, once that's implemented.
+    (void) mFind->SetWordBreaker(0);
 
-#ifdef TEXT_SVCS_TEST
+    nsCOMPtr<nsISelectionController> selCon (do_QueryInterface(presShell));
+    NS_ENSURE_ARG_POINTER(selCon);
+
+    nsCOMPtr<nsIDOMRange> searchRange (do_CreateInstance(kRangeCID));
+    NS_ENSURE_ARG_POINTER(searchRange);
+    nsCOMPtr<nsIDOMRange> startPt (do_CreateInstance(kRangeCID));
+    NS_ENSURE_ARG_POINTER(startPt);
+    nsCOMPtr<nsIDOMRange> endPt (do_CreateInstance(kRangeCID));
+    NS_ENSURE_ARG_POINTER(endPt);
+
+    nsCOMPtr<nsIDOMRange> foundRange;
+
+    // If !aWrapping, search from selection to end
+    if (!aWrapping)
+        rv = GetSearchLimits(searchRange, startPt, endPt, domDoc, selCon,
+                             PR_FALSE);
+
+    // If aWrapping, search the part of the starting frame
+    // up to the point where we left off.
+    else
+        rv = GetSearchLimits(searchRange, startPt, endPt, domDoc, selCon,
+                             PR_TRUE);
+
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv =  mFind->Find(mSearchString.get(), searchRange, startPt, endPt,
+                      getter_AddRefs(foundRange));
+
+    if (NS_SUCCEEDED(rv) && foundRange)
+    {
+        *aDidFind = PR_TRUE;
+        SetSelectionAndScroll(foundRange, selCon);
     }
-    if (gUseTextServices)
-        mTSFind->SetTsDoc(nsnull);
 
-#if defined(XP_UNIX) && defined(DEBUG)
-    struct timeval timeAtEnd;
-    gettimeofday(&timeAtEnd, 0);
-    double sec = (timeAtEnd.tv_sec - timeAtStart.tv_sec) +
-        (((double)timeAtEnd.tv_usec - (double)timeAtStart.tv_usec) / 1000000.);
-    printf("nsWebBrowserFind::SearchInFrame (%s) took %f sec\n",
-           (gUseTextServices ? "txtsvc" : "new find"), sec);
-#endif /* XP_UNIX && DEBUG */
-#endif /* TEXT_SVCS_TEST */
+    if (*aDidFind) 
+      MoveFocusToCaret(aWindow);
+
     return rv;
 }
 
@@ -548,46 +734,6 @@ nsresult nsWebBrowserFind::OnFind(nsIDOMWindow *aFoundWindow)
     return NS_OK;
 }
 
-#ifdef TEXT_SVCS_TEST
-nsresult nsWebBrowserFind::MakeTSDocument(nsIDOMWindow* aWindow, nsITextServicesDocument** aDoc)
-{
-    NS_ENSURE_ARG(aWindow);
-    NS_ENSURE_ARG_POINTER(aDoc);
-
-    nsresult rv;
-    *aDoc = NULL;
-
-    nsCOMPtr<nsITextServicesDocument>  tempDoc(do_CreateInstance(kCTextServicesDocumentCID, &rv));
-    if (NS_FAILED(rv) || !tempDoc)
-        return rv;
-
-    nsCOMPtr<nsIDocShell> docShell;
-    rv = GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
-    if (NS_FAILED(rv) || !docShell)
-        return NS_ERROR_FAILURE;
-	
-    nsCOMPtr<nsIPresShell> presShell;
-    docShell->GetPresShell(getter_AddRefs(presShell));
-    if (!presShell)
-        return NS_ERROR_FAILURE;
-
-    nsCOMPtr<nsIDOMDocument> domDoc;    
-    rv = aWindow->GetDocument(getter_AddRefs(domDoc));
-    if (!domDoc)
-        return NS_ERROR_FAILURE;
-
-    rv = tempDoc->InitWithDocument(domDoc, presShell);
-    if (NS_FAILED(rv))
-        return rv;
-
-    // Return the resulting text services document.
-    *aDoc = tempDoc;
-    NS_IF_ADDREF(*aDoc);
-  
-    return rv;
-}
-#endif /* TEXT_SVCS_TEST */
-
 /*---------------------------------------------------------------------------
 
   GetDocShellFromWindow
