@@ -74,6 +74,10 @@
 #include "nsIWebNavigation.h"
 #include "nsIWindowCreator.h"
 #include "nsIXPConnect.h"
+#include "nsPIDOMWindow.h"
+
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 
 #ifdef XP_UNIX
 // please see bug 78421 for the eventual "right" fix for this
@@ -542,11 +546,11 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     nsCOMPtr<nsIWebBrowserChrome> parentChrome(do_GetInterface(parentTreeOwner));
 
     // is the parent (if any) modal? if so, we must be, too.
-    PRBool weAreModal = PR_FALSE;
-    if (parentChrome)
+    PRBool weAreModal = (chromeFlags & nsIWebBrowserChrome::CHROME_MODAL) != 0;
+    if (!weAreModal && parentChrome)
       parentChrome->IsWindowModal(&weAreModal);
 
-    if (weAreModal || (chromeFlags & nsIWebBrowserChrome::CHROME_MODAL)) {
+    if (weAreModal) {
       rv = queueGuard.Push();
       if (NS_SUCCEEDED(rv)) {
         windowIsModal = PR_TRUE;
@@ -652,27 +656,32 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
       }
     }
 
-    // Get the calling context off the JS context stack
+    // Set the new window's referrer from the calling context's document:
+
+    // get the calling context off the JS context stack
     nsCOMPtr<nsIJSContextStack> stack = do_GetService(sJSStackContractID);
 
     JSContext* ccx = nsnull;
 
+    // get its document, if any
     if (stack && NS_SUCCEEDED(stack->Peek(&ccx)) && ccx) {
-      nsCOMPtr<nsIScriptGlobalObject> sgo;
 
+      nsCOMPtr<nsIScriptGlobalObject> sgo;
       nsWWJSUtils::nsGetStaticScriptGlobal(ccx, ::JS_GetGlobalObject(ccx),
                                            getter_AddRefs(sgo));
 
-      nsCOMPtr<nsIDOMWindow> w(do_QueryInterface(sgo));
-
+      nsCOMPtr<nsPIDOMWindow> w(do_QueryInterface(sgo));
       if (w) {
+        /* use the URL from the *extant* document, if any. The usual accessor
+           GetDocument will synchronously create an about:blank document if
+           it has no better answer, and we only care about a real document.
+           Also using GetDocument to force document creation seems to
+           screw up focus in the hidden window; see bug 36016.
+        */
         nsCOMPtr<nsIDOMDocument> document;
-
-        // Get the document from the window.
-        w->GetDocument(getter_AddRefs(document));
+        w->GetExtantDocument(getter_AddRefs(document));
 
         nsCOMPtr<nsIDocument> doc(do_QueryInterface(document));
-
         if (doc) { 
           nsCOMPtr<nsIURI> uri;
           doc->GetDocumentURL(getter_AddRefs(uri));
@@ -684,7 +693,7 @@ nsWindowWatcher::OpenWindowJS(nsIDOMWindow *aParent,
     }
 
     newDocShell->LoadURI(uriToLoad, loadInfo,
-                         nsIWebNavigation::LOAD_FLAGS_NONE);
+                         nsIWebNavigation::LOAD_FLAGS_NONE, PR_TRUE);
   }
 
   if (windowIsNew)
@@ -1076,6 +1085,16 @@ void nsWindowWatcher::CheckWindowName(nsString& aName)
     }
 }
 
+#define NS_CALCULATE_CHROME_FLAG_FOR(feature, flag)               \
+    prefBranch->GetBoolPref(feature, &forceEnable);               \
+    if (forceEnable && !isChrome) {                               \
+      chromeFlags |= flag;                                        \
+    } else {                                                      \
+      chromeFlags |= WinHasOption(aFeatures, feature,             \
+                                  0, &presenceFlag)               \
+                     ? flag : 0;                                  \
+    }
+
 /**
  * Calculate the chrome bitmask from a string list of features.
  * @param aFeatures a string containing a list of named chrome features
@@ -1114,27 +1133,48 @@ PRUint32 nsWindowWatcher::CalculateChromeFlags(const char *aFeatures,
 
   /* Next, allow explicitly named options to override the initial settings */
 
-  chromeFlags |= WinHasOption(aFeatures, "titlebar", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_TITLEBAR : 0;
-  chromeFlags |= WinHasOption(aFeatures, "close", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_WINDOW_CLOSE : 0;
-  chromeFlags |= WinHasOption(aFeatures, "toolbar", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_TOOLBAR : 0;
-  chromeFlags |= WinHasOption(aFeatures, "location", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_LOCATIONBAR : 0;
-  chromeFlags |= (WinHasOption(aFeatures, "directories", 0, &presenceFlag) ||
-                  WinHasOption(aFeatures, "personalbar", 0, &presenceFlag))
-                 ? nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR : 0;
-  chromeFlags |= WinHasOption(aFeatures, "status", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_STATUSBAR : 0;
-  chromeFlags |= WinHasOption(aFeatures, "menubar", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_MENUBAR : 0;
-  chromeFlags |= WinHasOption(aFeatures, "scrollbars", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_SCROLLBARS : 0;
-  chromeFlags |= WinHasOption(aFeatures, "resizable", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_WINDOW_RESIZE : 0;
-  chromeFlags |= WinHasOption(aFeatures, "minimizable", 0, &presenceFlag)
-                 ? nsIWebBrowserChrome::CHROME_WINDOW_MIN : 0;
+  nsCOMPtr<nsIScriptSecurityManager>
+    securityManager(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
+  NS_ENSURE_TRUE(securityManager, NS_ERROR_FAILURE);
+
+  PRBool isChrome = PR_FALSE;
+  securityManager->SubjectPrincipalIsSystem(&isChrome);
+
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  nsresult rv;
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, PR_TRUE);
+
+  rv = prefs->GetBranch("dom.disable_window_open_feature.", getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv, PR_TRUE);
+
+  PRBool forceEnable = PR_FALSE;
+
+  NS_CALCULATE_CHROME_FLAG_FOR("titlebar",
+                               nsIWebBrowserChrome::CHROME_TITLEBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("close",
+                               nsIWebBrowserChrome::CHROME_WINDOW_CLOSE);
+  NS_CALCULATE_CHROME_FLAG_FOR("toolbar",
+                               nsIWebBrowserChrome::CHROME_TOOLBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("location",
+                               nsIWebBrowserChrome::CHROME_LOCATIONBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("directories",
+                               nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("personalbar",
+                               nsIWebBrowserChrome::CHROME_PERSONAL_TOOLBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("status",
+                               nsIWebBrowserChrome::CHROME_STATUSBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("menubar",
+                               nsIWebBrowserChrome::CHROME_MENUBAR);
+  NS_CALCULATE_CHROME_FLAG_FOR("scrollbars",
+                               nsIWebBrowserChrome::CHROME_SCROLLBARS);
+  NS_CALCULATE_CHROME_FLAG_FOR("resizable",
+                               nsIWebBrowserChrome::CHROME_WINDOW_RESIZE);
+  NS_CALCULATE_CHROME_FLAG_FOR("minimizable",
+                               nsIWebBrowserChrome::CHROME_WINDOW_MIN);
+
+  chromeFlags |= WinHasOption(aFeatures, "popup", 0, &presenceFlag)
+                 ? nsIWebBrowserChrome::CHROME_WINDOW_POPUP : 0; 
 
   /* OK.
      Normal browser windows, in spite of a stated pattern of turning off
@@ -1188,16 +1228,10 @@ PRUint32 nsWindowWatcher::CalculateChromeFlags(const char *aFeatures,
      chromeFlags->copy_history
    */
 
-  //Check security state for use in determing window dimensions
-  nsCOMPtr<nsIScriptSecurityManager>
-    securityManager(do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID));
-  NS_ENSURE_TRUE(securityManager, NS_ERROR_FAILURE);
-
+  // Check security state for use in determing window dimensions
   PRBool enabled;
   nsresult res =
     securityManager->IsCapabilityEnabled("UniversalBrowserWrite", &enabled);
-
-   res = securityManager->IsCapabilityEnabled("UniversalBrowserWrite", &enabled);
  
   if (NS_FAILED(res) || !enabled) {
     //If priv check fails, set all elements to minimum reqs., else leave them alone.
@@ -1205,6 +1239,7 @@ PRUint32 nsWindowWatcher::CalculateChromeFlags(const char *aFeatures,
     chromeFlags |= nsIWebBrowserChrome::CHROME_WINDOW_CLOSE;
     chromeFlags &= ~nsIWebBrowserChrome::CHROME_WINDOW_LOWERED;
     chromeFlags &= ~nsIWebBrowserChrome::CHROME_WINDOW_RAISED;
+    chromeFlags &= ~nsIWebBrowserChrome::CHROME_WINDOW_POPUP;
     //XXX Temporarily removing this check to allow modal dialogs to be
     //raised from script.  A more complete security based fix is needed.
     //chromeFlags &= ~nsIWebBrowserChrome::CHROME_MODAL;
@@ -1225,7 +1260,7 @@ nsWindowWatcher::WinHasOption(const char *aOptions, const char *aName,
 
   while (PR_TRUE) {
     while (nsCRT::IsAsciiSpace(*aOptions))
-      aOptions++;
+      ++aOptions;
 
     comma = PL_strchr(aOptions, ',');
     if (comma)
@@ -1308,43 +1343,6 @@ nsWindowWatcher::FindItemWithName(
   return rv;
 }
 
-/*   Force docshell initialization. This will allow, for instance, script
-   executed immediately following window.open to access the docshell
-   (which can be something as straightforward as newWindow.document.write,
-   or something more indirect, such as window.resizeTo.)
-     This step is not necessary in Mozilla, since its content docshells
-   are loaded during XUL chrome docshell initialization (though it may
-   be desirable; to be determined). But this step is important for probably
-   all embedding apps. */
-/* Note this is not in use at time of writing (7 Nov 01), its functionality
-   replaced by lazy instantiation of the content viewer in nsDocShell
-   and by the DOM document in nsGlobalWindow. This method to be removed
-   when we're quite sure the lazy way will work. */
-nsresult
-nsWindowWatcher::InitializeDocshell(nsIWebBrowserChrome *aBrowserChrome)
-{
-return NS_OK;
-  nsCOMPtr<nsIDocShell> docshell;
-
-  // embedded window path: docshell corresponding to the primary content
-  nsCOMPtr<nsIDOMWindow> browserWindow(do_GetInterface(aBrowserChrome));
-  if (browserWindow) {
-    nsCOMPtr<nsIScriptGlobalObject> sgo(do_QueryInterface(browserWindow));
-    if (sgo)
-      sgo->GetDocShell(getter_AddRefs(docshell));
-  }
-  // XUL Window path (should we decide this to be useful)
-  // XXX: teach nsContentTreeOwner to give up its nsIDocShell
-  if (!docshell) {
-  }
-
-  // asking mozilla's docshell for its document will force it to generate one
-  if (docshell)
-    nsCOMPtr<nsIDOMDocument> domdoc(do_GetInterface(docshell));
-
-  return NS_OK;
-}
-
 /* Fetch the nsIDOMWindow corresponding to the given nsIDocShellTreeItem.
    This forces the creation of a script context, if one has not already
    been created. Note it also sets the window's opener to the parent,
@@ -1692,7 +1690,7 @@ nsWindowWatcher::AddSupportsTojsvals(nsISupports *aArg,
 
       p->GetData(&data);
 
-      JSString *str = ::JS_NewString(cx, data, nsCRT::strlen(data));
+      JSString *str = ::JS_NewString(cx, data, strlen(data));
       NS_ENSURE_TRUE(str, NS_ERROR_OUT_OF_MEMORY);
 
       *aArgv = STRING_TO_JSVAL(str);
