@@ -14,13 +14,13 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   David Epstein <depstein@netscape.com> 
+ *   David Epstein <depstein@netscape.com>
  *   Dharma Sirnapalli <dsirnapalli@netscape.com>
  *	 Ashish Bhatt <ashishbhatt@netscape.com>
  *
@@ -59,6 +59,12 @@
 #include "domwindow.h"
 #include "selection.h"
 #include "nsProfile.h"
+#include "nsIClipboardCmd.h"
+#include "nsIObserServ.h"
+#include "nsIFile.h"
+#include "nsIWebBrow.h"
+#include "nsIWebProg.h"
+#include "nsIWebBrowFind.h"
 #include "QaUtils.h"
 #include <stdio.h>
 
@@ -81,33 +87,22 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_TESTS_CREATEPROFILE, OnTestsCreateprofile)
 	ON_COMMAND(ID_TESTS_ADDWEBPROGLISTENER, OnTestsAddWebProgListener)
 	ON_COMMAND(ID_TESTS_ADDHISTORYLISTENER, OnTestsAddHistoryListener)
-	ON_COMMAND(ID_INTERFACES_NSIFILE, OnInterfacesNsifile)
+	ON_COMMAND(ID_TESTS_REMOVEHISTORYLISTENER, OnTestsRemovehistorylistener)
+
 	ON_COMMAND(ID_TOOLS_REMOVEGHPAGE, OnToolsRemoveGHPage)
 	ON_COMMAND(ID_TOOLS_REMOVEALLGH, OnToolsRemoveAllGH)
 	ON_COMMAND(ID_TOOLS_TESTYOURMETHOD, OnToolsTestYourMethod)
 	ON_COMMAND(ID_TOOLS_TESTYOURMETHOD2, OnToolsTestYourMethod2)
+
 	ON_COMMAND(ID_VERIFYBUGS_70228, OnVerifybugs70228)
-    ON_COMMAND(ID_CLIPBOARDCMD_PASTE, OnPasteTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_COPYSELECTION, OnCopyTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_SELECTALL, OnSelectAllTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_SELECTNONE, OnSelectNoneTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_CUTSELECTION, OnCutSelectionTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_COPYLINKLOCATION, copyLinkLocationTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_CANCOPYSELECTION, canCopySelectionTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_CANCUTSELECTION, canCutSelectionTest)
-    ON_COMMAND(ID_CLIPBOARDCMD_CANPASTE, canPasteTest)
-	ON_COMMAND(ID_INTERFACES_NSIREQUEST_CANCEL, OnInterfacesNsirequest)
-	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_RUNALLTESTS, OnInterfacesNsidomwindow)
-	ON_COMMAND(ID_INTERFACES_NSIDIRECTORYSERVICE_INIT, OnInterfacesNsidirectoryservice)
-	ON_COMMAND(ID_INTERFACES_NSISELECTION_RUNALLTESTS, OnInterfacesNsiselection)
 	ON_COMMAND(ID_VERIFYBUGS_90195, OnVerifybugs90195)
-	ON_COMMAND(ID_INTERFACES_NSIPROFILE_RUNALLTESTS, OnInterfacesNsiprofile)
-	ON_COMMAND(ID_INTERFACES_NSISHISTORY_GETCOUNT, OnInterfacesNsishistory)
-	ON_COMMAND(ID_TESTS_REMOVEHISTORYLISTENER, OnTestsRemovehistorylistener)
-	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_GETCANGOBACK, OnInterfacesNsiwebnav)
+
+	ON_COMMAND(ID_INTERFACES_NSIDIRECTORYSERVICE_INIT, OnInterfacesNsidirectoryservice)
 	ON_COMMAND(ID_INTERFACES_NSIDIRECTORYSERVICE_REGISTERPROVIDER, OnInterfacesNsidirectoryservice)
 	ON_COMMAND(ID_INTERFACES_NSIDIRECTORYSERVICE_RUNALLTESTS, OnInterfacesNsidirectoryservice)
 	ON_COMMAND(ID_INTERFACES_NSIDIRECTORYSERVICE_UNREGISTERPROVIDER, OnInterfacesNsidirectoryservice)
+
+	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_RUNALLTESTS, OnInterfacesNsidomwindow)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_GETDOMDOCUMENT, OnInterfacesNsidomwindow)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_GETFRAMES, OnInterfacesNsidomwindow)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_GETNAME, OnInterfacesNsidomwindow)
@@ -121,6 +116,10 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_SCROLLBYPAGES, OnInterfacesNsidomwindow)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_SCROLLTO, OnInterfacesNsidomwindow)
 	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_SIZETOCONTENT, OnInterfacesNsidomwindow)
+	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_GETTEXTZOOM, OnInterfacesNsidomwindow)
+	ON_COMMAND(ID_INTERFACES_NSIDOMWINDOW_SETTEXTZOOM, OnInterfacesNsidomwindow)
+
+	ON_COMMAND(ID_INTERFACES_NSISELECTION_RUNALLTESTS, OnInterfacesNsiselection)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_GETANCHORNODE, OnInterfacesNsiselection)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_ADDRANGE, OnInterfacesNsiselection)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_COLLAPSE, OnInterfacesNsiselection)
@@ -140,6 +139,8 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_SELECTALLCHILDREN, OnInterfacesNsiselection)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_SELECTIONLANGUAGECHANGE, OnInterfacesNsiselection)
 	ON_COMMAND(ID_INTERFACES_NSISELECTION_TOSTRING, OnInterfacesNsiselection)
+
+	ON_COMMAND(ID_INTERFACES_NSIPROFILE_RUNALLTESTS, OnInterfacesNsiprofile)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_CLONEPROFILE, OnInterfacesNsiprofile)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_CREATENEWPROFILE, OnInterfacesNsiprofile)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_DELETEPROFILE, OnInterfacesNsiprofile)
@@ -150,6 +151,8 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_RENAMEPROFILE, OnInterfacesNsiprofile)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_SETCURRENTPROFILE, OnInterfacesNsiprofile)
 	ON_COMMAND(ID_INTERFACES_NSIPROFILE_SHUTDOWNCURRENTPROFILE, OnInterfacesNsiprofile)
+
+	ON_COMMAND(ID_INTERFACES_NSISHISTORY_GETCOUNT, OnInterfacesNsishistory)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_GETENTRYATINDEX, OnInterfacesNsishistory)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_GETINDEX, OnInterfacesNsishistory)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_GETMAXLENGTH, OnInterfacesNsishistory)
@@ -161,6 +164,8 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_NSIHISTORYENTRY_GETTITLE, OnInterfacesNsishistory)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_NSIHISTORYENTRY_GETURI, OnInterfacesNsishistory)
 	ON_COMMAND(ID_INTERFACES_NSISHISTORY_NSIHISTORYENTRY_RUNALLTESTS, OnInterfacesNsishistory)
+
+	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_GETCANGOBACK, OnInterfacesNsiwebnav)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_GETCANGOFORWARD, OnInterfacesNsiwebnav)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_GETCURRENTURI, OnInterfacesNsiwebnav)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_GETDOCUMENT, OnInterfacesNsiwebnav)
@@ -173,17 +178,75 @@ BEGIN_MESSAGE_MAP(CTests, CWnd)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_RUNALLTESTS, OnInterfacesNsiwebnav)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_SETSESSIONHISTORY, OnInterfacesNsiwebnav)
 	ON_COMMAND(ID_INTERFACES_NSIWEBNAV_STOP, OnInterfacesNsiwebnav)
+
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_GETLOADFLAGS, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_GETLOADGROUP, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_GETNAME, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_GETSTATUS, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_ISPENDING, OnInterfacesNsirequest)
+	ON_COMMAND(ID_INTERFACES_NSIREQUEST_CANCEL, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_RESUME, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_SETLOADFLAGS, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_SETLOADGROUP, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_SUSPEND, OnInterfacesNsirequest)
 	ON_COMMAND(ID_INTERFACES_NSIREQUEST_RUNALLTESTS, OnInterfacesNsirequest)
+
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_CANCUTSELECTION, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_CANCOPYSELECTION, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_CANPASTE, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_COPYLINKLOCATION, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_COPYSELECTION, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_CUTSELECTION, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_PASTE, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_SELECTALL, OnInterfacesNsiclipboardcommands)
+	ON_COMMAND(ID_INTERFACES_NSICLIPBOARDCOMMANDS_SELECTNONE, OnInterfacesNsiclipboardcommands)
+
+	ON_COMMAND(ID_INTERFACES_NSIOBSERVERSERVICE_ADDOBSERVERS, OnInterfacesNsiobserverservice)
+	ON_COMMAND(ID_INTERFACES_NSIOBSERVERSERVICE_ENUMERATEOBSERVERS, OnInterfacesNsiobserverservice)
+	ON_COMMAND(ID_INTERFACES_NSIOBSERVERSERVICE_NOTIFYOBSERVERS, OnInterfacesNsiobserverservice)
+	ON_COMMAND(ID_INTERFACES_NSIOBSERVERSERVICE_REMOVEOBSERVERS, OnInterfacesNsiobserverservice)
+	ON_COMMAND(ID_INTERFACES_NSIOBSERVERSERVICE_RUNALLTESTS, OnInterfacesNsiobserverservice)
+
+	ON_COMMAND(ID_INTERFACES_NSIFILE_APPENDRELATICEPATH, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_COPYTO, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_CREATE, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_EXISTS, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_INITWITHPATH, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_MOVETO, OnInterfacesNsifile)
+	ON_COMMAND(ID_INTERFACES_NSIFILE_RUNALLTESTS, OnInterfacesNsifile)
+
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_RUNALLTESTS, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_ADDWEBBROWSERLISTENER, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_REMOVEWEBBROWSERLISTENER, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_GETCONTAINERWINDOW, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_SETCONTAINERWINDOW, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_GETPARENTURICONTENTLISTENER, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_SETPARENTURICONTENTLISTENER, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_GETCONTENTDOMWINDOW, OnInterfacesNsiwebbrowser)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSER_NSIWBSETUPSETPROPERTY, OnInterfacesNsiwebbrowser)
+
+	ON_COMMAND(ID_INTERFACES_NSIWEBPROGRESS_RUNALLTESTS, OnInterfacesNsiwebprogress)
+	ON_COMMAND(ID_INTERFACES_NSIWEBPROGRESS_ADDPROGRESSLISTENER, OnInterfacesNsiwebprogress)
+	ON_COMMAND(ID_INTERFACES_NSIWEBPROGRESS_REMOVEPROGRESSLISTENER, OnInterfacesNsiwebprogress)
+	ON_COMMAND(ID_INTERFACES_NSIWEBPROGRESS_GETDOMWINDOW, OnInterfacesNsiwebprogress)
+
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_RUNALLTESTS, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETSEARCHSTRINGTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETSEARCHSTRINGTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_FINDNEXTTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETFINDBACKWARDSTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETFINDBACKWARDSTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETWRAPFINDTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETWRAPFINDTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETENTIREWORDTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETENTIREWORDTEST, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETMATCHCASE, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETMATCHCASE, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_SETSEARCHFRAMES, OnInterfacesNsiwebbrowfind)
+	ON_COMMAND(ID_INTERFACES_NSIWEBBROWSERFIND_GETSEARCHFRAMES, OnInterfacesNsiwebbrowfind)
+
 	//}}AFX_MSG_MAP
+
 END_MESSAGE_MAP()
 
 
@@ -197,7 +260,7 @@ CTests::CTests(nsIWebBrowser* mWebBrowser,
 	qaWebNav = mWebNav;
 
 	qaBrowserImpl = mpBrowserImpl;
-}	
+}
 
 CTests::~CTests()
 {
@@ -208,12 +271,11 @@ CTests::~CTests()
 // *********************************************************
 // *********************************************************
 
-void CTests::OnTestsChangeUrl() 
+void CTests::OnTestsChangeUrl()
 {
-	char *theUrl = "http://www.aol.com/";
 	CUrlDialog myDialog;
-	//nsresult rv;  
 
+	//nsresult rv;
 
 	if (!qaWebNav)
 	{
@@ -224,39 +286,11 @@ void CTests::OnTestsChangeUrl()
 	if (myDialog.DoModal() == IDOK)
 	{
 		QAOutput("Begin Change URL test.", 1);
-		strcpy(theUrl, myDialog.m_urlfield);
-		rv = qaWebNav->LoadURI(NS_ConvertASCIItoUCS2(theUrl).get(), 
-						nsIWebNavigation::LOAD_FLAGS_BYPASS_HISTORY,
-            nsnull,
-            nsnull,
-            nsnull);
+		rv = qaWebNav->LoadURI(NS_ConvertASCIItoUCS2(myDialog.m_urlfield).get(),
+								myDialog.m_flagvalue, nsnull,nsnull, nsnull);
 	    RvTestResult(rv, "rv LoadURI() test", 1);
-		FormatAndPrintOutput("The url = ", theUrl, 2); 
-
-/*
-		char *uriSpec;
-		nsCOMPtr<nsIURI> pURI;
-		// GetcurrentURI() declared in nsIP3PUI.idl
-		// used with webNav obj in nsP3PObserverHTML.cpp, line 239
-		// this will be used as an indep routine to verify the URI load
-		rv = qaWebNav->GetCurrentURI( getter_AddRefs( pURI ) );
-		if(NS_FAILED(rv) || !pURI)
-			AfxMessageBox("Bad result for GetCurrentURI().");
-
-		rv = pURI->GetSpec(&uriSpec);
-		if (NS_FAILED(rv))
-			AfxMessageBox("Bad result for GetSpec().");
-
-		AfxMessageBox("Start URL validation test().");
-		if (strcmp(uriSpec, theUrl) == 0)
-		{
-			QAOutput("Url loaded successfully. Test Passed.", 2);	
-		}
-		else
-		{
-			QAOutput("Url didn't load successfully. Test Failed.", 2);
-		}
-*/
+		FormatAndPrintOutput("The url = ", myDialog.m_urlfield, 2);
+		FormatAndPrintOutput("The flag = ", myDialog.m_flagvalue, 1);
 		QAOutput("End Change URL test.", 1);
 	}
 	else
@@ -266,7 +300,7 @@ void CTests::OnTestsChangeUrl()
 
 // *********************************************************
 
-void CTests::OnTestsGlobalHistory() 
+void CTests::OnTestsGlobalHistory()
 {
 	// create instance of myHistory object. Call's XPCOM
 	// service manager to pass the contract ID.
@@ -275,9 +309,6 @@ void CTests::OnTestsGlobalHistory()
 	CUrlDialog myDialog;
 
 	PRBool theRetVal = PR_FALSE;
-    
-	//nsresult rv; 
-
 
 	nsCOMPtr<nsIGlobalHistory> myHistory(do_GetService(NS_GLOBALHISTORY_CONTRACTID));
 
@@ -298,7 +329,7 @@ void CTests::OnTestsGlobalHistory()
 		// see if url is already in the GH file (pre-AddPage() test)
 		rv = myHistory->IsVisited(theUrl, &theRetVal);
 	    RvTestResult(rv, "rv IsVisited() test", 1);
-		FormatAndPrintOutput("The IsVisited() boolean return value = ", theRetVal, 1); 
+		FormatAndPrintOutput("The IsVisited() boolean return value = ", theRetVal, 1);
 
 		if (theRetVal)
 			QAOutput("URL has been visited. Won't execute AddPage().", 2);
@@ -335,9 +366,9 @@ void CTests::OnTestsGlobalHistory()
 
 // *********************************************************
 
-void CTests::OnTestsCreateFile() 
+void CTests::OnTestsCreateFile()
 {
-   	//nsresult rv;  
+   	//nsresult rv;
 	PRBool exists;
     nsCOMPtr<nsILocalFile> theTestFile(do_GetService(NS_LOCAL_FILE_CONTRACTID));
 
@@ -350,7 +381,7 @@ void CTests::OnTestsCreateFile()
 
 	QAOutput("Start Create File test.", 2);
 
-	rv = theTestFile->InitWithPath("c:\\temp\\theFile.txt");
+	rv = theTestFile->InitWithNativePath(NS_LITERAL_CSTRING("c:\\temp\\theFile.txt"));
 	rv = theTestFile->Exists(&exists);
 
 	QAOutput("File (theFile.txt) doesn't exist. We'll create it.\r\n", 1);
@@ -360,13 +391,13 @@ void CTests::OnTestsCreateFile()
 
 // *********************************************************
 
-void CTests::OnTestsCreateprofile() 
+void CTests::OnTestsCreateprofile()
 {
     CProfilesDlg    myDialog;
     nsresult        rv;
 
 	if (myDialog.DoModal() == IDOK)
-    {       
+    {
 //      NS_WITH_SERVICE(nsIProfile, profileService, NS_PROFILE_CONTRACTID, &rv);
 		nsCOMPtr<nsIProfile> theProfServ(do_GetService(NS_PROFILE_CONTRACTID,&rv));
 		if (NS_FAILED(rv))
@@ -385,7 +416,7 @@ void CTests::OnTestsCreateprofile()
     }
 	else
 	   QAOutput("Profile switch test not executed.", 2);
-	
+
 }
 
 // *********************************************************
@@ -395,7 +426,7 @@ void CTests::OnTestsAddWebProgListener()
     nsWeakPtr weakling(
         dont_AddRef(NS_GetWeakReference(NS_STATIC_CAST(nsIWebProgressListener*, qaBrowserImpl))));
     rv = qaWebBrowser->AddWebBrowserListener(weakling, NS_GET_IID(nsIWebProgressListener));
-	
+
 	RvTestResult(rv, "AddWebBrowserListener(). Add Web Prog Lstnr test", 2);
 }
 
@@ -410,9 +441,9 @@ void CTests::OnTestsAddHistoryListener()
 	RvTestResult(rv, "AddWebBrowserListener(). Add History Lstnr test", 2);
 }
 
-void CTests::OnTestsRemovehistorylistener() 
+void CTests::OnTestsRemovehistorylistener()
 {
-  // addSHistoryListener test
+  // RemoveSHistoryListener test
 	nsWeakPtr weakling(
         dont_AddRef(NS_GetWeakReference(NS_STATIC_CAST(nsISHistoryListener*, qaBrowserImpl))));
 	rv = qaWebBrowser->RemoveWebBrowserListener(weakling, NS_GET_IID(nsISHistoryListener));
@@ -424,7 +455,7 @@ void CTests::OnTestsRemovehistorylistener()
 //					TOOLS to help us
 
 
-void CTests::OnToolsRemoveGHPage() 
+void CTests::OnToolsRemoveGHPage()
 {
 	char *theUrl = "http://www.bogussite.com/";
 	CUrlDialog myDialog;
@@ -441,8 +472,6 @@ void CTests::OnToolsRemoveGHPage()
 		QAOutput("Could not get the history object.", 2);
 		return;
 	}
-//	nsCOMPtr<nsIBrowserHistory> myHistory(do_GetInterface(myGHistory));
-
 
 	if (myDialog.DoModal() == IDOK)
 	{
@@ -468,8 +497,6 @@ void CTests::OnToolsRemoveGHPage()
 void CTests::OnToolsRemoveAllGH()
 {
 
-	//nsresult rv; 
-
 	nsCOMPtr<nsIGlobalHistory> myGHistory(do_GetService(NS_GLOBALHISTORY_CONTRACTID));
 	if (!myGHistory)
 	{
@@ -486,10 +513,8 @@ void CTests::OnToolsRemoveAllGH()
 
 	rv = myHistory->RemoveAllPages();
 	RvTestResult(rv, "removeAllPages(). Test .", 2);
-	
-	QAOutput("End removal of all pages from the GH file.", 2);
 
-	// removeAllPages()
+	QAOutput("End removal of all pages from the GH file.", 2);
 
 }
 
@@ -497,349 +522,249 @@ void CTests::OnToolsRemoveAllGH()
 void CTests::OnToolsTestYourMethod()
 {
 	// place your test code here
+
+	// nsIWebBrowserFind methods
+	nsCOMPtr<nsIWebBrowserFind> qaWBFind(do_GetInterface(qaWebBrowser, &rv));
+	if (!qaWBFind)
+		QAOutput("Didn't get WebBrowserFind object.", 2);
+	else
+		RvTestResult(rv, "nsIWebBrowserFind object test", 2);
+
+	CUrlDialog myDialog;
+
+	nsString searchString;
+	PRBool didFind = PR_FALSE;
+	PRBool didFindBackwards = PR_FALSE;
+	PRBool didWrapFind = PR_FALSE;
+	PRBool didEntireWord = PR_FALSE;
+	PRBool didMatchCase = PR_FALSE;
+	PRBool didSearchFrames = PR_FALSE;
+	if (myDialog.DoModal() == IDOK) {
+
+		// SetSearchString()
+		searchString.AssignWithConversion(myDialog.m_urlfield);
+		rv = qaWBFind->SetSearchString(searchString.get());
+		RvTestResult(rv, "nsIWebBrowserFind::SetSearchString() test", 2);
+	}
+		// GetSearchString()
+		nsXPIDLString stringBuf;
+		CString csSearchStr;
+		rv = qaWBFind->GetSearchString(getter_Copies(stringBuf));
+		RvTestResult(rv, "nsIWebBrowserFind::GetSearchString() test", 2);
+		csSearchStr = stringBuf.get();
+		FormatAndPrintOutput("The searched string value = ", csSearchStr, 2);
+
+	// FindNext()
+	didFind = PR_TRUE;
+	rv = qaWBFind->FindNext(&didFind);
+	RvTestResult(rv, "nsIWebBrowserFind::FindNext(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didFind = ", didFind, 2);
+
+	didFind = PR_FALSE;
+	rv = qaWBFind->FindNext(&didFind);
+	RvTestResult(rv, "nsIWebBrowserFind::FindNext(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didFind = ", didFind, 2);
+
+	// SetFindBackwards()
+	didFindBackwards = PR_TRUE;
+	rv = qaWBFind->SetFindBackwards(didFindBackwards);
+	RvTestResult(rv, "nsIWebBrowserFind::SetFindBackwards(PR_TRUE) object test", 2);
+
+	didFindBackwards = PR_FALSE;
+	rv = qaWBFind->SetFindBackwards(didFindBackwards);
+	RvTestResult(rv, "nsIWebBrowserFind::SetFindBackwards(PR_FALSE) object test", 2);
+
+	// GetFindBackwards()
+	didFindBackwards = PR_TRUE;
+	rv = qaWBFind->GetFindBackwards(&didFindBackwards);
+	RvTestResult(rv, "nsIWebBrowserFind::GetFindBackwards(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didFindBackwards = ", didFindBackwards, 2);
+
+	didFindBackwards = PR_FALSE;
+	rv = qaWBFind->GetFindBackwards(&didFindBackwards);
+	RvTestResult(rv, "nsIWebBrowserFind::GetFindBackwards(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didFindBackwards = ", didFindBackwards, 2);
+
+	// SetWrapFind()
+	didWrapFind = PR_TRUE;
+	rv = qaWBFind->SetWrapFind(didWrapFind);
+	RvTestResult(rv, "nsIWebBrowserFind::SetWrapFind(PR_TRUE) object test", 2);
+
+	didWrapFind = PR_FALSE;
+	rv = qaWBFind->SetWrapFind(didWrapFind);
+	RvTestResult(rv, "nsIWebBrowserFind::SetWrapFind(PR_FALSE) object test", 2);
+
+	// GetWrapFind()
+	didWrapFind = PR_TRUE;
+	rv = qaWBFind->GetWrapFind(&didWrapFind);
+	RvTestResult(rv, "nsIWebBrowserFind::GetWrapFind(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didWrapFind = ", didWrapFind, 2);
+
+	didWrapFind = PR_FALSE;
+	rv = qaWBFind->GetWrapFind(&didWrapFind);
+	RvTestResult(rv, "nsIWebBrowserFind::GetWrapFind(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didWrapFind = ", didWrapFind, 2);
+
+	// SetEntireWord()
+	didEntireWord = PR_TRUE;
+	rv = qaWBFind->SetEntireWord(didEntireWord);
+	RvTestResult(rv, "nsIWebBrowserFind::SetEntireWord(PR_TRUE) object test", 2);
+
+	didEntireWord = PR_FALSE;
+	rv = qaWBFind->SetEntireWord(didEntireWord);
+	RvTestResult(rv, "nsIWebBrowserFind::SetEntireWord(PR_FALSE) object test", 2);
+
+	// GetEntireWord()
+	didEntireWord = PR_TRUE;
+	rv = qaWBFind->GetEntireWord(&didEntireWord);
+	RvTestResult(rv, "nsIWebBrowserFind::GetEntireWord(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didEntireWord = ", didEntireWord, 2);
+
+	didEntireWord = PR_FALSE;
+	rv = qaWBFind->GetEntireWord(&didEntireWord);
+	RvTestResult(rv, "nsIWebBrowserFind::GetEntireWord(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didEntireWord = ", didEntireWord, 2);
+
+	// SetMatchCase()
+	didMatchCase = PR_TRUE;
+	rv = qaWBFind->SetMatchCase(didMatchCase);
+	RvTestResult(rv, "nsIWebBrowserFind::SetMatchCase(PR_TRUE) object test", 2);
+
+	didMatchCase = PR_FALSE;
+	rv = qaWBFind->SetMatchCase(didMatchCase);
+	RvTestResult(rv, "nsIWebBrowserFind::SetMatchCase(PR_FALSE) object test", 2);
+
+	// GetMatchCase()
+	didMatchCase = PR_TRUE;
+	rv = qaWBFind->GetMatchCase(&didMatchCase);
+	RvTestResult(rv, "nsIWebBrowserFind::GetMatchCase(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didMatchCase = ", didMatchCase, 2);
+
+	didMatchCase = PR_FALSE;
+	rv = qaWBFind->GetMatchCase(&didMatchCase);
+	RvTestResult(rv, "nsIWebBrowserFind::GetMatchCase(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didMatchCase = ", didMatchCase, 2);
+
+	// SetSearchFrames()
+	didSearchFrames = PR_TRUE;
+	rv = qaWBFind->SetSearchFrames(didSearchFrames);
+	RvTestResult(rv, "nsIWebBrowserFind::SetSearchFrames(PR_TRUE) object test", 2);
+
+	didSearchFrames = PR_FALSE;
+	rv = qaWBFind->SetSearchFrames(didSearchFrames);
+	RvTestResult(rv, "nsIWebBrowserFind::SetSearchFrames(PR_FALSE) object test", 2);
+
+	// GetSearchFrames()
+	didSearchFrames = PR_TRUE;
+	rv = qaWBFind->GetSearchFrames(&didSearchFrames);
+	RvTestResult(rv, "nsIWebBrowserFind::GetSearchFrames(PR_TRUE) object test", 2);
+	FormatAndPrintOutput("returned didSearchFrames = ", didSearchFrames, 2);
+
+	didSearchFrames = PR_FALSE;
+	rv = qaWBFind->GetSearchFrames(&didSearchFrames);
+	RvTestResult(rv, "nsIWebBrowserFind::GetSearchFrames(PR_FALSE) object test", 2);
+	FormatAndPrintOutput("returned didSearchFrames = ", didSearchFrames, 2);
 }
 
 // ***********************************************************************
 void CTests::OnToolsTestYourMethod2()
 {
 	// place your test code here
-}
 
-// ***********************************************************************
-// ************************** Interface Tests ****************************
-// ***********************************************************************
+	nsCAutoString theSpec;
+	nsCOMPtr<nsIURI> theURI;
+	nsCOMPtr<nsIChannel> theChannel;
+	nsCOMPtr<nsILoadGroup> theLoadGroup(do_CreateInstance(NS_LOADGROUP_CONTRACTID));
 
-// nsIFile:
+	theSpec = "javascript:document.write('TEST')";
+	FormatAndPrintOutput("the uri spec = ", theSpec, 2);
 
-void CTests::OnInterfacesNsifile() 
-{
-   nsCOMPtr<nsILocalFile> theTestFile(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-   nsCOMPtr<nsILocalFile> theFileOpDir(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
+	rv = NS_NewURI(getter_AddRefs(theURI), theSpec);
+	RvTestResult(rv, "NS_NewURI", 2);
 
-    if (!theTestFile)
- 	{
-		QAOutput("File object doesn't exist. No File tests performed.", 2);
-		return;
-	}
-	if (!theFileOpDir)
- 	{
-		QAOutput("File object doesn't exist. No File tests performed.", 2);
-		return;
-	}
+	rv = NS_NewChannel(getter_AddRefs(theChannel), theURI, nsnull, theLoadGroup);
+	RvTestResult(rv, "NS_OpenURI", 2);
 
-	QAOutput("Begin nsIFile tests.", 2);
+	nsCOMPtr<nsIStreamListener> listener(NS_STATIC_CAST(nsIStreamListener*, qaBrowserImpl));
+	nsCOMPtr<nsIWeakReference> thisListener(dont_AddRef(NS_GetWeakReference(listener)));
+	qaWebBrowser->AddWebBrowserListener(thisListener, NS_GET_IID(nsIStreamListener));
 
-	InitWithPathTest(theTestFile);
-	AppendRelativePathTest(theTestFile);
-	FileCreateTest(theTestFile);
-	FileExistsTest(theTestFile);
+	// this calls nsIStreamListener::OnDataAvailable()
+	rv = theChannel->AsyncOpen(listener, nsnull);
+	RvTestResult(rv, "AsyncOpen()", 2);
 
-	// FILE COPY test
+	nsCOMPtr<nsIRequest> theRequest = do_QueryInterface(theChannel);
 
-	FileCopyTest(theTestFile, theFileOpDir);	
-
-	// FILE MOVE test
-
-	FileMoveTest(theTestFile, theFileOpDir);	
-
-	QAOutput("End nsIFile tests.", 2);	
-}
-
-// ***********************************************************************
-// Individual nsIFile tests
-
-void CTests::InitWithPathTest(nsILocalFile *theTestFile)
-{
-	rv = theTestFile->InitWithPath("c:\\temp\\");
-	RvTestResult(rv, "InitWithPath() test (initializing file path)", 2);
-}
-
-void CTests::AppendRelativePathTest(nsILocalFile *theTestFile)
-{
-	rv = theTestFile->AppendRelativePath("myFile.txt");
-	RvTestResult(rv, "AppendRelativePath() test (append file to the path)", 2);
-}
-
-void CTests::FileCreateTest(nsILocalFile *theTestFile)
-{
-	rv = theTestFile->Exists(&exists);
-	if (!exists)
-	{
-		QAOutput("File doesn't exist. We'll try creating it.", 2);
-		rv = theTestFile->Create(nsIFile::NORMAL_FILE_TYPE, 0777);
-		RvTestResult(rv, " File Create() test ('myFile.txt')", 2);
-	}
-	else
-		QAOutput("File already exists (myFile.txt). We won't create it.", 2);
-}
-
-void CTests::FileExistsTest(nsILocalFile *theTestFile)
-{
-	rv = theTestFile->Exists(&exists);
-	if (!exists)
-		QAOutput("Exists() test Failed. File (myFile.txt) doesn't exist.", 2);
-	else
-		QAOutput("Exists() test Passed. File (myFile.txt) exists.", 2);
+	CNsIRequest::IsPendingReqTest(theRequest);
+	CNsIRequest::GetStatusReqTest(theRequest);
+	CNsIRequest::SuspendReqTest(theRequest);	
+	CNsIRequest::ResumeReqTest(theRequest);	
+	CNsIRequest::CancelReqTest(theRequest);	
+	CNsIRequest::SetLoadGroupTest(theRequest, theLoadGroup);	
+	CNsIRequest::GetLoadGroupTest(theRequest);
 
 }
 
-void CTests::FileCopyTest(nsILocalFile *theTestFile, nsILocalFile *theFileOpDir)
-{
-	QAOutput("Start File Copy test.", 2);
-
-	rv = theFileOpDir->InitWithPath("c:\\temp\\");
-	if (NS_FAILED(rv))
-		QAOutput("The target dir wasn't found.", 2);
-	else
-		QAOutput("The target dir was found.", 2);
-
-	rv = theTestFile->InitWithPath("c:\\temp\\myFile.txt");
-	if (NS_FAILED(rv))
-		QAOutput("The path wasn't found.", 2);
-	else
-		QAOutput("The path was found.", 2);
-
-	rv = theTestFile->CopyTo(theFileOpDir, "myFile2.txt");
-	RvTestResult(rv, "rv CopyTo() test", 2);
-
-	rv = theTestFile->InitWithPath("c:\\temp\\myFile2.txt");
-	rv = theTestFile->Exists(&exists);
-	if (!exists)
-		QAOutput("File didn't copy. CopyTo() test Failed.", 2);
-	else
-		QAOutput("File copied. CopyTo() test Passed.", 2);
-}
-
-void CTests::FileMoveTest(nsILocalFile *theTestFile, nsILocalFile *theFileOpDir)
-{
-	QAOutput("Start File Move test.", 2);
-
-	rv = theFileOpDir->InitWithPath("c:\\Program Files\\");
-	if (NS_FAILED(rv))
-		QAOutput("The target dir wasn't found.", 2);
-
-	rv = theTestFile->InitWithPath("c:\\temp\\myFile2.txt");
-	if (NS_FAILED(rv))
-		QAOutput("The path wasn't found.", 2);
-
-	rv = theTestFile->MoveTo(theFileOpDir, "myFile2.txt");
-	RvTestResult(rv, "MoveTo() test", 2);
-
-	rv = theTestFile->InitWithPath("c:\\Program Files\\myFile2.txt");
-	rv = theTestFile->Exists(&exists);
-	if (!exists)
-		QAOutput("File wasn't moved. MoveTo() test Failed.", 2);
-	else
-		QAOutput("File was moved. MoveTo() test Passed.", 2);
-}
-
-// ***********************************************************************
-//DHARMA	- nsIClipboardCommands
-// Checking the paste() method.
-void CTests::OnPasteTest()
-{
-    QAOutput("testing paste command", 1);
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->Paste();
-		RvTestResult(rv, "nsIClipboardCommands::Paste()' rv test", 1);
-
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the copySelection() method.
-void CTests::OnCopyTest()
-{
-    QAOutput("testing copyselection command");
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->CopySelection();
-		RvTestResult(rv, "nsIClipboardCommands::CopySelection()' rv test", 1);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the selectAll() method.
-void CTests::OnSelectAllTest()
-{
-    QAOutput("testing selectall method");
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->SelectAll();
-		RvTestResult(rv, "nsIClipboardCommands::SelectAll()' rv test", 1);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the selectNone() method.
-void CTests::OnSelectNoneTest()
-{
-    QAOutput("testing selectnone method");
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->SelectNone();
-		RvTestResult(rv, "nsIClipboardCommands::SelectNone()' rv test", 1);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the cutSelection() method.
-void CTests::OnCutSelectionTest()
-{
-    QAOutput("testing cutselection method");
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->CutSelection();
-		RvTestResult(rv, "nsIClipboardCommands::CutSelection()' rv test", 1);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the copyLinkLocation() method.
-void CTests::copyLinkLocationTest()
-{
-    QAOutput("testing CopyLinkLocation method", 2);
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->CopyLinkLocation();
-		RvTestResult(rv, "nsIClipboardCommands::CopyLinkLocation()' rv test", 1);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the canCopySelection() method.
-void CTests::canCopySelectionTest()
-{
-    PRBool canCopySelection = PR_FALSE;
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-       rv = clipCmds->CanCopySelection(&canCopySelection);
-	   RvTestResult(rv, "nsIClipboardCommands::CanCopySelection()' rv test", 1);
-
-       if(canCopySelection)
-          QAOutput("The selection you made Can be copied", 2);
-       else
-          QAOutput("Either you did not make a selection or The selection you made Cannot be copied", 2);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the canCutSelection() method.
-void CTests::canCutSelectionTest()
-{
-    PRBool canCutSelection = PR_FALSE;
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-       rv = clipCmds->CanCutSelection(&canCutSelection);
-	   RvTestResult(rv, "nsIClipboardCommands::CanCutSelection()' rv test", 1);
-
-	   if(canCutSelection)
-          QAOutput("The selection you made Can be cut", 2);
-       else
-          QAOutput("Either you did not make a selection or The selection you made Cannot be cut", 2);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-// Checking the canPaste() method.
-void CTests::canPasteTest()
-{
-    PRBool canPaste = PR_FALSE;
-    nsCOMPtr<nsIClipboardCommands> clipCmds = do_GetInterface(qaWebBrowser);
-    if (clipCmds)
-	{
-        rv = clipCmds->CanPaste(&canPaste);
-	    RvTestResult(rv, "nsIClipboardCommands::CanPaste()' rv test", 1);
-
-		if(canPaste)
-			QAOutput("The clipboard contents can be pasted here", 2);
-		else
-			QAOutput("The clipboard contents cannot be pasted here", 2);
-	}
-	else
-		QAOutput("We didn't get the clipboard object.", 1);
-}
-
-//DHARMA
-
-
 // ***********************************************************************
 // ***************** Bug Verifications ******************
 // ***********************************************************************
 
-void CTests::OnVerifybugs70228() 
+void CTests::OnVerifybugs70228()
 {
-	nsCOMPtr<nsIHelperAppLauncherDialog> 
+	nsCOMPtr<nsIHelperAppLauncherDialog>
 			myHALD(do_CreateInstance(NS_IHELPERAPPLAUNCHERDLG_CONTRACTID));
 	if (!myHALD)
 		QAOutput("Object not created. It should be. It's a component!", 2);
 	else
-		QAOutput("Object is created. It's a component!", 2);	
+		QAOutput("Object is created. It's a component!", 2);
 
 /*
-nsCOMPtr<nsIHelperAppLauncher> 
+nsCOMPtr<nsIHelperAppLauncher>
 			myHAL(do_CreateInstance(NS_IHELPERAPPLAUNCHERDLG_CONTRACTID));
 
 	rv = myHALD->show(myHal, nsnull);
-*/	
+*/
 }
 
-BOOL CTests::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
+BOOL CTests::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
 {
-   // To handle Menu handlers add here. Don't have to do if not handling 
+   // To handle Menu handlers add here. Don't have to do if not handling
    // menu handlers
 	nCommandID = nID ;
 
 	return CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
 }
 
-void CTests::OnInterfacesNsirequest() 
+void CTests::OnInterfacesNsirequest()
 {
 	CNsIRequest oNsIRequest(qaWebBrowser,/*qaBaseWindow,qaWebNav,*/ qaBrowserImpl);
 	oNsIRequest.OnStartTests(nCommandID);
 }
 
-void CTests::OnInterfacesNsidirectoryservice() 
+void CTests::OnInterfacesNsidirectoryservice()
 {
 	CNsIDirectoryService oNsIDirectoryService;
 	oNsIDirectoryService.StartTests(nCommandID);
 }
 
-void CTests::OnInterfacesNsidomwindow() 
+void CTests::OnInterfacesNsidomwindow()
 {
 	CDomWindow oDomWindow(qaWebBrowser) ;
-	oDomWindow.OnStartTests(nCommandID);		
+	oDomWindow.OnStartTests(nCommandID);
 }
 
-void CTests::OnInterfacesNsiselection() 
+void CTests::OnInterfacesNsiselection()
 {
 	CSelection oSelection(qaWebBrowser);
 	oSelection.OnStartTests(nCommandID);
 }
 
-void CTests::OnVerifybugs90195() 
+void CTests::OnVerifybugs90195()
 {
     nsWeakPtr weakling(
         dont_AddRef(NS_GetWeakReference(NS_STATIC_CAST(nsITooltipListener*, qaBrowserImpl))));
     rv = qaWebBrowser->AddWebBrowserListener(weakling, NS_GET_IID(nsITooltipListener));
-	
+
 	RvTestResult(rv, "AddWebBrowserListener(). Add Tool Tip Lstnr test", 2);
 
 /*	nsCOMPtr<nsITooltipTextProvider> oTooltipTextProvider = do_GetService(NS_TOOLTIPTEXTPROVIDER_CONTRACTID) ;
@@ -848,22 +773,60 @@ void CTests::OnVerifybugs90195()
 */
 }
 
-void CTests::OnInterfacesNsiprofile() 
+void CTests::OnInterfacesNsiprofile()
 {
 	CProfile oProfile(qaWebBrowser);
-	oProfile.OnStartTests(nCommandID);	
+	oProfile.OnStartTests(nCommandID);
 }
 
-void CTests::OnInterfacesNsishistory() 
+void CTests::OnInterfacesNsishistory()
 {
 	CNsIHistory oHistory(qaWebNav);
-	oHistory.OnStartTests(nCommandID);	
+	oHistory.OnStartTests(nCommandID);
 }
 
 
-void CTests::OnInterfacesNsiwebnav() 
+void CTests::OnInterfacesNsiwebnav()
 {
 	CNsIWebNav oWebNav(qaWebNav);
-	oWebNav.OnStartTests(nCommandID);	
-}		
+	oWebNav.OnStartTests(nCommandID);
+}
 
+
+void CTests::OnInterfacesNsiclipboardcommands()
+{
+	CNsIClipBoardCmd  oClipCmd(qaWebBrowser) ;
+	oClipCmd.OnStartTests(nCommandID);
+}
+
+
+void CTests::OnInterfacesNsiobserverservice()
+{
+	CnsIObserServ oObserv  ;
+	oObserv.OnStartTests(nCommandID);
+}
+
+
+void CTests::OnInterfacesNsifile()
+{
+	CNsIFile oFile ;
+	oFile.OnStartTests(nCommandID);
+}
+
+void CTests::OnInterfacesNsiwebbrowser()
+{
+	CNsIWebBrowser oWebBrowser(qaWebBrowser, qaBrowserImpl);
+	oWebBrowser.OnStartTests(nCommandID);
+}
+
+void CTests::OnInterfacesNsiwebprogress()
+{
+	CnsiWebProg oWebProgress(qaWebBrowser, qaBrowserImpl);
+	oWebProgress.OnStartTests(nCommandID);
+}
+
+void CTests::OnInterfacesNsiwebbrowfind()
+{
+	CNsIWebBrowFind oWebBrowFind(qaWebBrowser, qaBrowserImpl);
+	oWebBrowFind.OnStartTests(nCommandID);
+}
\ No newline at end of file
