@@ -1,39 +1,32 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: Mozilla-sample-code 1.0
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  *
- * The Original Code is mozilla.org code.
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
  *
- * The Initial Developer of the Original Code is 
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * Contributor(s):
  *   Chak Nanga <chak@netscape.com> 
- *
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ *   Rod Spears <rods@netscape.com>
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -67,10 +60,7 @@
 #include "BrowserImpl.h"
 #include "BrowserFrm.h"
 #include "Dialogs.h"
-
-// Print Includes
-#include "PrintProgressDialog.h"
-#include "PrintSetupDialog.h"
+#include "CPageSetupPropSheet.h"
 
 // Mozilla Includes
 #include "nsIWidget.h"
@@ -131,6 +121,7 @@ BEGIN_MESSAGE_MAP(CBrowserView, CWnd)
 	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateCopy)
 	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdatePaste)
 	ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, OnUpdateFilePrint)
+	ON_UPDATE_COMMAND_UI(ID_FILE_PRINTSETUP, OnUpdatePrintSetup)
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -148,13 +139,12 @@ CBrowserView::CBrowserView()
 	mbDocumentLoading = PR_FALSE;
 
     m_pFindDlg = NULL;
-    m_pPrintProgressDlg = NULL;
-
     m_bUrlBarClipOp = FALSE;
     m_bCurrentlyPrinting = FALSE;
 
     m_SecurityState = SECURITY_STATE_INSECURE;
 
+  m_InPrintPreview = FALSE;
 }
 
 CBrowserView::~CBrowserView()
@@ -268,11 +258,6 @@ HRESULT CBrowserView::CreateBrowser()
 	// Finally, show the web browser window
 	mBaseWindow->SetVisibility(PR_TRUE);
 
-	nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
-  if (print) {
-    print->GetPrintSettings(getter_AddRefs(m_PrintSettings));
-  }
-
 	return S_OK;
 }
 
@@ -399,8 +384,8 @@ void CBrowserView::OnViewSource()
 		return;
 
 	// Get the uri string associated with the nsIURI object
-	nsXPIDLCString uriString;
-	rv = currentURI->GetSpec(getter_Copies(uriString));
+	nsCAutoString uriString;
+	rv = currentURI->GetSpec(uriString);
 	if(NS_FAILED(rv))
 		return;
 
@@ -693,12 +678,12 @@ void CBrowserView::OnFileSaveAs()
         if(persist)
         {
             nsCOMPtr<nsILocalFile> file;
-            NS_NewLocalFile(T2A(pStrFullPath), TRUE, getter_AddRefs(file));
+            NS_NewNativeLocalFile(nsDependentCString(T2A(pStrFullPath)), TRUE, getter_AddRefs(file));
 
             nsCOMPtr<nsILocalFile> dataPath;
             if (pStrDataPath)
             {
-                NS_NewLocalFile(pStrDataPath, TRUE, getter_AddRefs(dataPath));
+                NS_NewNativeLocalFile(nsDependentCString(pStrDataPath), TRUE, getter_AddRefs(dataPath));
             }
 
             if(bSaveAll)
@@ -814,11 +799,10 @@ void CBrowserView::OnSaveLinkAs()
 
 	// Get the "path" portion (see nsIURI.h for more info
 	// on various parts of a URI)
-	nsXPIDLCString path;
-	linkURI->GetPath(getter_Copies(path));
+	nsCAutoString fileName;
+	linkURI->GetPath(fileName);
 
 	// The path may have the "/" char in it - strip those
-	nsCAutoString fileName(path);
 	fileName.StripChars("\\/");
 
 	// Now, use this file name in a File Save As dlg...
@@ -840,7 +824,7 @@ void CBrowserView::OnSaveLinkAs()
 		if(persist)
         {
             nsCOMPtr<nsILocalFile> file;
-            NS_NewLocalFile(strFullPath.GetBuffer(0), TRUE, getter_AddRefs(file));
+            NS_NewNativeLocalFile(nsDependentCString(strFullPath.GetBuffer(0)), TRUE, getter_AddRefs(file));
             persist->SaveURI(linkURI, nsnull, file);
         }
     }
@@ -864,8 +848,8 @@ void CBrowserView::OnSaveImageAs()
 
 	// Get the "path" portion (see nsIURI.h for more info
 	// on various parts of a URI)
-	nsXPIDLCString path;
-	linkURI->GetPath(getter_Copies(path));
+	nsCAutoString path;
+	linkURI->GetPath(path);
 
 	// The path may have the "/" char in it - strip those
 	nsCAutoString fileName(path);
@@ -886,7 +870,7 @@ void CBrowserView::OnSaveImageAs()
         if(persist)
         {
             nsCOMPtr<nsILocalFile> file;
-            NS_NewLocalFile(strFullPath.GetBuffer(0), TRUE, getter_AddRefs(file));
+            NS_NewNativeLocalFile(nsDependentCString(strFullPath.GetBuffer(0)), TRUE, getter_AddRefs(file));
             persist->SaveURI(linkURI, nsnull, file);
         }
     }
@@ -984,116 +968,83 @@ LRESULT CBrowserView::OnFindMsg(WPARAM wParam, LPARAM lParam)
 
 void CBrowserView::OnFilePrint()
 {
-  nsresult rv;
-  nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
-  if (NS_SUCCEEDED(rv)) 
-  {
-    prefs->SetBoolPref("print.use_native_print_dialog", PR_TRUE);
-    prefs->SetBoolPref("print.show_print_progress", PR_FALSE);
-  }
-  else
-	NS_ASSERTION(PR_FALSE, "Could not get preferences service");
-
-  nsCOMPtr<nsIDOMWindow> domWindow;
-	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-  if(domWindow) {
-	  nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
-	  if(print)
-	  {
-      CPrintProgressDialog  dlg(mWebBrowser, domWindow, m_PrintSettings);
-
-	    nsCOMPtr<nsIURI> currentURI;
-	    nsresult rv = mWebNav->GetCurrentURI(getter_AddRefs(currentURI));
-      if(NS_SUCCEEDED(rv) || currentURI) 
-      {
-	      nsXPIDLCString path;
-	      currentURI->GetPath(getter_Copies(path));
-        dlg.SetURI(path.get());
+  nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	if(print)
+	{
+    if (!m_PrintSettings) 
+    {
+      if (NS_FAILED(print->GetGlobalPrintSettings(getter_AddRefs(m_PrintSettings)))) {
+        return;
       }
-      m_bCurrentlyPrinting = TRUE;
-      dlg.DoModal();
-      m_bCurrentlyPrinting = FALSE;
     }
+    nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	  if(print)
+    {
+      print->Print(m_PrintSettings, nsnull);
+    }
+
   }
 }
 
 void CBrowserView::OnFilePrintPreview()
 {
-  nsCOMPtr<nsIDOMWindow> domWindow;
-	mWebBrowser->GetContentDOMWindow(getter_AddRefs(domWindow));
-  if(domWindow) {
-	  nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
-	  if(print)
-	  {
-      print->PrintPreview(domWindow, m_PrintSettings);
+  nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	if(print)
+	{
+    if (!m_PrintSettings) 
+    {
+      if (NS_FAILED(print->GetGlobalPrintSettings(getter_AddRefs(m_PrintSettings)))) {
+        return;
+      }
+    }
+    if (!m_InPrintPreview) 
+    {
+      if (NS_SUCCEEDED(print->PrintPreview(m_PrintSettings, nsnull, nsnull))) 
+      {
+        m_InPrintPreview = TRUE;
+
+        CMenu* menu = mpBrowserFrame->GetMenu();
+        if (menu) 
+        {
+          menu->CheckMenuItem( ID_FILE_PRINTPREVIEW, MF_CHECKED );
+        }
+      }
+    } 
+    else 
+    {
+      if (NS_SUCCEEDED(print->ExitPrintPreview())) 
+      {
+        m_InPrintPreview = FALSE;
+        CMenu* menu = mpBrowserFrame->GetMenu();
+        if (menu) 
+        {
+          menu->CheckMenuItem( ID_FILE_PRINTPREVIEW, MF_UNCHECKED );
+        }
+      }
     }
   }
 }
 
-static float GetFloatFromStr(const char* aStr, float aMaxVal = 1.0)
-{
-  float val;
-  sscanf(aStr, "%f", &val);
-  if (val <= aMaxVal) {
-    return val;
-  } else {
-    return 0.5;
-  }
-}
-
-static PRUnichar* GetUnicodeFromCString(const CString& aStr)
-{
-  nsString str;
-  str.AssignWithConversion(LPCSTR(aStr));
-  return ToNewUnicode(str);
-}
-
 void CBrowserView::OnFilePrintSetup()
 {
-  CPrintSetupDialog  dlg(m_PrintSettings);
-  if (dlg.DoModal() == IDOK && m_PrintSettings != NULL) {
-    m_PrintSettings->SetMarginTop(GetFloatFromStr(dlg.m_TopMargin));
-    m_PrintSettings->SetMarginLeft(GetFloatFromStr(dlg.m_LeftMargin));
-    m_PrintSettings->SetMarginRight(GetFloatFromStr(dlg.m_RightMargin));
-    m_PrintSettings->SetMarginBottom(GetFloatFromStr(dlg.m_BottomMargin));
+  nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	if(print)
+  {
+    if (!m_PrintSettings) 
+    {
+      if (NS_FAILED(print->GetGlobalPrintSettings(getter_AddRefs(m_PrintSettings)))) {
+        return;
+      }
+    }
+    CPageSetupPropSheet propSheet(_T("Page Setup"), this);
 
-    m_PrintSettings->SetScaling(double(dlg.m_Scaling) / 100.0);
-    m_PrintSettings->SetPrintBGColors(dlg.m_PrintBGColors);
-    m_PrintSettings->SetPrintBGColors(dlg.m_PrintBGImages);
-
-    short  type;
-    double width;
-    double height;
-    dlg.GetPaperSizeInfo(type, width, height);
-    m_PrintSettings->SetPaperSizeType(type);
-    m_PrintSettings->SetPaperWidth(width);
-    m_PrintSettings->SetPaperHeight(height);
-
-    PRUnichar* uStr;
-    uStr = GetUnicodeFromCString(dlg.m_HeaderLeft);
-    m_PrintSettings->SetHeaderStrLeft(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
-
-    uStr = GetUnicodeFromCString(dlg.m_HeaderMiddle);
-    m_PrintSettings->SetHeaderStrCenter(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
-
-    uStr = GetUnicodeFromCString(dlg.m_HeaderRight);
-    m_PrintSettings->SetHeaderStrRight(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
-
-    uStr = GetUnicodeFromCString(dlg.m_FooterLeft);
-    m_PrintSettings->SetFooterStrLeft(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
-
-    uStr = GetUnicodeFromCString(dlg.m_FooterMiddle);
-    m_PrintSettings->SetFooterStrCenter(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
-
-    uStr = GetUnicodeFromCString(dlg.m_FooterRight);
-    m_PrintSettings->SetFooterStrRight(uStr);
-    if (uStr != nsnull) nsMemory::Free(uStr);
+    propSheet.SetPrintSettingsValues(m_PrintSettings);
 
+    int status = propSheet.DoModal();
+    if (status == IDOK) 
+    {
+      propSheet.GetPrintSettingsValues(m_PrintSettings);
+    }
   }
 }
 
@@ -1103,6 +1054,12 @@ void CBrowserView::OnUpdateFilePrint(CCmdUI* pCmdUI)
 	pCmdUI->Enable(!m_bCurrentlyPrinting);
 }
 
+/////////////////////////////////////////////////////////////////////////////
+void CBrowserView::OnUpdatePrintSetup(CCmdUI* pCmdUI)
+{
+	pCmdUI->Enable(!m_bCurrentlyPrinting && !m_InPrintPreview);
+}
+
 
 
 // Called from the busy state related methods in the 
@@ -1118,6 +1075,24 @@ void CBrowserView::OnUpdateFilePrint(CCmdUI* pCmdUI)
 //
 void CBrowserView::UpdateBusyState(PRBool aBusy)
 {
+  if (mbDocumentLoading && !aBusy && m_InPrintPreview)
+  {
+      nsCOMPtr<nsIWebBrowserPrint> print(do_GetInterface(mWebBrowser));
+	    if(print)
+	    {
+          PRBool isDoingPP;
+          print->GetDoingPrintPreview(&isDoingPP);
+          if (!isDoingPP) 
+          {
+              m_InPrintPreview = FALSE;
+              CMenu* menu = mpBrowserFrame->GetMenu();
+              if (menu) 
+              {
+                  menu->CheckMenuItem( ID_FILE_PRINTPREVIEW, MF_UNCHECKED );
+              }
+          }
+      }
+  }
 	mbDocumentLoading = aBusy;
 }
 
