@@ -1,41 +1,33 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: Mozilla-sample-code 1.0
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  *
- * The Original Code is mozilla.org code.
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
  *
- * The Initial Developer of the Original Code is 
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * Contributor(s):
  *   Chak Nanga <chak@netscape.com> 
  *   Conrad Carlen <ccarlen@netscape.com> 
  *
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
- *
  * ***** END LICENSE BLOCK ***** */
 
 // File Overview....
@@ -56,12 +48,14 @@
 #include "stdafx.h"
 #include "MfcEmbed.h"
 #include "BrowserFrm.h"
+#include "EditorFrm.h"
 #include "winEmbedFileLocProvider.h"
 #include "ProfileMgr.h"
 #include "BrowserImpl.h"
 #include "nsIWindowWatcher.h"
 #include "plstr.h"
 #include "Preferences.h"
+#include "nsCRT.h"
 #include <io.h>
 #include <fcntl.h>
 
@@ -78,6 +72,12 @@ static char THIS_FILE[] = __FILE__;
  {0xa2112d6a, 0x0e28, 0x421f, {0xb4, 0x6a, 0x25, 0xc0, 0xb3, 0x8, 0xcb, 0xd0}}
 static NS_DEFINE_CID(kPromptServiceCID, NS_PROMPTSERVICE_CID);
 
+// this is for overriding the Mozilla default PrintingPromptService component
+#include "PrintingPromptService.h"
+#define NS_PRINTINGPROMPTSERVICE_CID \
+ {0xe042570c, 0x62de, 0x4bb6, { 0xa6, 0xe0, 0x79, 0x8e, 0x3c, 0x7, 0xb4, 0xdf}}
+static NS_DEFINE_CID(kPrintingPromptServiceCID, NS_PRINTINGPROMPTSERVICE_CID);
+
 // this is for overriding the Mozilla default HelperAppLauncherDialog
 #include "HelperAppService.h"
 #define NS_HELPERAPPLAUNCHERDIALOG_CID \
@@ -87,6 +87,7 @@ static NS_DEFINE_CID(kHelperAppLauncherDialogCID, NS_HELPERAPPLAUNCHERDIALOG_CID
 BEGIN_MESSAGE_MAP(CMfcEmbedApp, CWinApp)
 	//{{AFX_MSG_MAP(CMfcEmbedApp)
 	ON_COMMAND(ID_NEW_BROWSER, OnNewBrowser)
+	ON_COMMAND(ID_NEW_EDITORWINDOW, OnNewEditor)
 	ON_COMMAND(ID_MANAGE_PROFILES, OnManageProfiles)
 	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
     ON_COMMAND(ID_EDIT_PREFERENCES, OnEditPreferences)
@@ -214,6 +215,29 @@ nsresult CMfcEmbedApp::OverrideComponents()
           ::FreeLibrary(overlib);
     }
 
+    // replace Mozilla's default PrintingPromptService with our own, if the
+    // expected override DLL is present
+    overlib = ::LoadLibrary(kComponentsLibname);
+    if (overlib) {
+        InitPrintingPromptServiceType InitLib;
+        MakeFactoryType MakeFactory;
+        InitLib = reinterpret_cast<InitPrintingPromptServiceType>(::GetProcAddress(overlib, kPrintingPromptServiceInitFuncName));
+        MakeFactory = reinterpret_cast<MakeFactoryType>(::GetProcAddress(overlib, kPrintingPromptServiceFactoryFuncName));
+
+        if (InitLib && MakeFactory) {
+            InitLib(overlib);
+
+            nsCOMPtr<nsIFactory> printingPromptFactory;
+            rv = MakeFactory(getter_AddRefs(printingPromptFactory));
+            if (NS_SUCCEEDED(rv))
+                nsComponentManager::RegisterFactory(kPrintingPromptServiceCID,
+                                                    "Printing Prompt Service",
+                                                    "@mozilla.org/embedcomp/printingprompt-service;1",
+                                                    printingPromptFactory,
+                                                    PR_TRUE); // replace existing
+        } else
+          ::FreeLibrary(overlib);
+    }
     return rv;
 }
 
@@ -272,6 +296,22 @@ BOOL CMfcEmbedApp::InitInstance()
 
 	Enable3dControls();
 
+	//
+	// 1. Determine the name of the dir from which the MRE based app is being run
+	// from [It's OK to do this even if you're not running in an MRE env]
+	//
+	// 2. Create an nsILocalFile out of it which will passed in to NS_InitEmbedding()
+	//
+	// Please see http://www.mozilla.org/projects/embedding/MRE.html
+	// for more info. on MRE
+
+	char curDir[_MAX_PATH+1];
+	::GetCurrentDirectory(_MAX_PATH, curDir);
+	nsresult rv;
+	nsCOMPtr<nsILocalFile> mreAppDir;
+	rv = NS_NewNativeLocalFile(nsDependentCString(curDir), TRUE, getter_AddRefs(mreAppDir));
+	NS_ASSERTION(NS_SUCCEEDED(rv), "failed to create mreAppDir localfile");
+
 	// Take a look at 
 	// http://www.mozilla.org/projects/xpcom/file_locations.html
 	// for more info on File Locations
@@ -283,8 +323,7 @@ BOOL CMfcEmbedApp::InitInstance()
         return FALSE;
     }
 
-    nsresult rv;
-    rv = NS_InitEmbedding(nsnull, provider);
+    rv = NS_InitEmbedding(mreAppDir, provider);
     if(NS_FAILED(rv))
     {
         ASSERT(FALSE);
@@ -330,8 +369,12 @@ BOOL CMfcEmbedApp::InitInstance()
 CBrowserFrame* CMfcEmbedApp::CreateNewBrowserFrame(PRUint32 chromeMask,
 												   PRInt32 x, PRInt32 y,
 												   PRInt32 cx, PRInt32 cy,
-												   PRBool bShowWindow)
+												   PRBool bShowWindow,
+												   PRBool bIsEditor
+												   )
 {
+	UINT resId = bIsEditor ? IDR_EDITOR : IDR_MAINFRAME;
+
 	// Setup a CRect with the requested window dimensions
 	CRect winSize(x, y, cx, cy);
 
@@ -344,9 +387,12 @@ CBrowserFrame* CMfcEmbedApp::CreateNewBrowserFrame(PRUint32 chromeMask,
 	strTitle.LoadString(IDR_MAINFRAME);
 
 	// Now, create the browser frame
-	CBrowserFrame* pFrame = new CBrowserFrame(chromeMask);
+	CBrowserFrame* pFrame = bIsEditor ? ( new  CEditorFrame(chromeMask) ) :
+					    ( new  CBrowserFrame(chromeMask) );
+	pFrame->SetEditable(bIsEditor);
+
 	if (!pFrame->Create(NULL, strTitle, WS_OVERLAPPEDWINDOW, 
-					winSize, NULL, MAKEINTRESOURCE(IDR_MAINFRAME), 0L, NULL))
+					winSize, NULL, MAKEINTRESOURCE(resId), 0L, NULL))
 	{
 		return NULL;
 	}
@@ -376,6 +422,18 @@ void CMfcEmbedApp::OnNewBrowser()
 		pBrowserFrame->m_wndBrowserView.LoadHomePage();
 }
 
+void CMfcEmbedApp::OnNewEditor() 
+{
+    CEditorFrame *pEditorFrame = (CEditorFrame *)CreateNewBrowserFrame(nsIWebBrowserChrome::CHROME_ALL, 
+                                    -1, -1, -1, -1,
+                                    PR_TRUE,PR_TRUE);
+    if (pEditorFrame)
+    {
+        pEditorFrame->m_wndBrowserView.OpenURL("about:blank");
+        pEditorFrame->InitEditor();
+    }
+}
+
 // This gets called anytime a BrowserFrameWindow is
 // closed i.e. by choosing the "close" menu item from
 // a window's system menu or by dbl clicking on the
