@@ -1,40 +1,32 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: Mozilla-sample-code 1.0
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  *
- * The Original Code is mozilla.org code.
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
  *
- * The Initial Developer of the Original Code is 
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * Contributor(s):
  *   Chak Nanga <chak@netscape.com> 
  *
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
- *
  * ***** END LICENSE BLOCK ***** */
 
 #include "stdafx.h"
@@ -250,7 +242,7 @@ NS_IMETHODIMP CHelperAppLauncherDialog::Show(nsIHelperAppLauncher *aLauncher,
         m_FileName = dlg.m_OpenWithAppName;
 
         nsCOMPtr<nsILocalFile> openWith;
-        nsresult rv = NS_NewLocalFile(m_FileName, PR_FALSE, getter_AddRefs(openWith));
+        nsresult rv = NS_NewNativeLocalFile(nsDependentCString(m_FileName), PR_FALSE, getter_AddRefs(openWith));
         if (NS_FAILED(rv))
             return aLauncher->LaunchWithApplication(nsnull, PR_FALSE);
         else
@@ -279,7 +271,7 @@ NS_IMETHODIMP CHelperAppLauncherDialog::PromptForSaveToFile(nsISupports *aWindow
     {
         m_FileName = cf.GetPathName(); // Will be like: c:\tmp\junk.exe
 
-        return NS_NewLocalFile(m_FileName, PR_FALSE, _retval);
+        return NS_NewNativeLocalFile(nsDependentCString(m_FileName), PR_FALSE, _retval);
     }
     else
         return NS_ERROR_FAILURE;
@@ -530,8 +522,8 @@ CProgressDlg::~CProgressDlg()
 }
 
 NS_IMETHODIMP CProgressDlg::OnStateChange(nsIWebProgress *aWebProgress, 
-                                          nsIRequest *aRequest, PRInt32 aStateFlags, 
-                                          PRUint32 aStatus)
+                                          nsIRequest *aRequest, PRUint32 aStateFlags, 
+                                          nsresult aStatus)
 {
     if((aStateFlags & STATE_STOP) && (aStateFlags & STATE_IS_DOCUMENT))
     {
@@ -565,20 +557,20 @@ NS_IMETHODIMP CProgressDlg::OnProgressChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP CProgressDlg::OnLocationChange(nsIWebProgress *aWebProgress, 
                                              nsIRequest *aRequest, nsIURI *location)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CProgressDlg::OnStatusChange(nsIWebProgress *aWebProgress, 
                                            nsIRequest *aRequest, nsresult aStatus, 
                                            const PRUnichar *aMessage)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 NS_IMETHODIMP CProgressDlg::OnSecurityChange(nsIWebProgress *aWebProgress, 
-                                             nsIRequest *aRequest, PRInt32 state)
+                                             nsIRequest *aRequest, PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 BOOL CProgressDlg::OnInitDialog() 
@@ -592,8 +584,8 @@ BOOL CProgressDlg::OnInitDialog()
         nsresult rv = m_HelperAppLauncher->GetSource(getter_AddRefs(srcUri));
         if(NS_SUCCEEDED(rv))
         {
-            nsXPIDLCString uriString;
-            srcUri->GetSpec(getter_Copies(uriString));
+            nsCAutoString uriString;
+            srcUri->GetSpec(uriString);
 
             m_SavingFrom.SetWindowText(uriString.get());
         }
