@@ -1,44 +1,35 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: Mozilla-sample-code 1.0
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * Copyright (c) 2002 Netscape Communications Corporation and
+ * other contributors
  *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this Mozilla sample software and associated documentation files
+ * (the "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to permit
+ * persons to whom the Software is furnished to do so, subject to the
+ * following conditions:
  *
- * The Original Code is mozilla.org code.
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
  *
- * The Initial Developer of the Original Code is 
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998
- * the Initial Developer. All Rights Reserved.
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  *
  * Contributor(s):
  *  Doug Turner <dougt@netscape.com> 
  *  Adam Lock <adamlock@netscape.com>
  *
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
- *
  * ***** END LICENSE BLOCK ***** */
 
-
 #include <stdio.h>
 #include "stdafx.h"
 
@@ -61,6 +52,8 @@
 #include "nsIProfileChangeStatus.h"
 #include "nsIURI.h"
 #include "plstr.h"
+#include "nsIInterfaceRequestor.h"
+#include "nsCRT.h"
 
 // Local header files
 #include "winEmbed.h"
@@ -68,6 +61,11 @@
 #include "WindowCreator.h"
 #include "resource.h"
 
+// Printing header files
+#include "nsIPrintSettings.h"
+#include "nsIWebBrowserPrint.h"
+
+
 #define MAX_LOADSTRING 100
 
 const TCHAR *szWindowClass = _T("WINEMBED");
@@ -127,7 +125,19 @@ public:
 
 int main(int argc, char *argv[])
 {
-    printf("\nYou are embedded, man!\n\n");
+    printf("You are embedded, man!\n\n");
+    printf("******************************************************************\n");
+    printf("*                                                                *\n");
+    printf("*  IMPORTANT NOTE:                                               *\n");
+    printf("*                                                                *\n");
+    printf("*  WinEmbed is not supported!!! Do not raise bugs on it unless   *\n");
+    printf("*  it is badly broken (e.g. crash on start/exit, build errors)   *\n");
+    printf("*  or you have the patch to make it better! MFCEmbed is now our  *\n");
+    printf("*  embedding test application on Win32 and all testing should    *\n");
+    printf("*  be done on that.                                              *\n");
+    printf("*                                                                *\n");
+    printf("******************************************************************\n");
+    printf("\n\n");
     
     // Sophisticated command-line parsing in action
     char *szFirstURL = "http://www.mozilla.org/projects/embedding";
@@ -166,30 +176,31 @@ int main(int argc, char *argv[])
         NS_TermEmbedding();
         return 1;
     }
-
-	// Now register an observer to watch for profile changes
-    nsCOMPtr<nsIObserverService> observerService(do_GetService("@mozilla.org/observer-service;1"));
-
-    ProfileChangeObserver *observer = new ProfileChangeObserver;
-    observer->AddRef();
-    observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-approve-change", PR_FALSE);
-    observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-change-teardown", PR_FALSE);
-    observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-after-change", PR_FALSE);
-
-    InitializeWindowCreator();
-
-    // Open the initial browser window
-    OpenWebPage(gFirstURL);
-
-	// Main message loop.
-    // NOTE: We use a fake event and a timeout in order to process idle stuff for
-    //       Mozilla every 1/10th of a second.
-    PRBool runCondition = PR_TRUE;
     WPARAM rv;
-    rv = AppCallbacks::RunEventLoop(runCondition);
+    {    
+    	// Now register an observer to watch for profile changes
+        nsCOMPtr<nsIObserverService> observerService(do_GetService("@mozilla.org/observer-service;1"));
 
-    observer->Release();
+        ProfileChangeObserver *observer = new ProfileChangeObserver;
+        observer->AddRef();
+        observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-approve-change", PR_TRUE);
+        observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-change-teardown", PR_TRUE);
+        observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, observer), "profile-after-change", PR_TRUE);
 
+        InitializeWindowCreator();
+
+        // Open the initial browser window
+        OpenWebPage(gFirstURL);
+
+	    // Main message loop.
+        // NOTE: We use a fake event and a timeout in order to process idle stuff for
+        //       Mozilla every 1/10th of a second.
+        PRBool runCondition = PR_TRUE;
+
+        rv = AppCallbacks::RunEventLoop(runCondition);
+
+        observer->Release();
+    }
     // Close down Embedding APIs
     NS_TermEmbedding();
 
@@ -421,12 +432,12 @@ void SaveWebPage(nsIWebBrowser *aWebBrowser)
         nsCOMPtr<nsIWebBrowserPersist> persist(do_QueryInterface(aWebBrowser));
 
         nsCOMPtr<nsILocalFile> file;
-        NS_NewLocalFile(szFile, TRUE, getter_AddRefs(file));
+        NS_NewNativeLocalFile(nsDependentCString(szFile), TRUE, getter_AddRefs(file));
 
         nsCOMPtr<nsILocalFile> dataPath;
         if (pszDataPath)
         {
-            NS_NewLocalFile(pszDataPath, TRUE, getter_AddRefs(dataPath));
+            NS_NewNativeLocalFile(nsDependentCString(pszDataPath), TRUE, getter_AddRefs(dataPath));
         }
 
         persist->SaveDocument(nsnull, file, dataPath, nsnull, 0, 0);
@@ -592,10 +603,12 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     }
     nsCOMPtr<nsIWebBrowser> webBrowser;
     nsCOMPtr<nsIWebNavigation> webNavigation;
+    nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint;
     if (chrome)
     {
         chrome->GetWebBrowser(getter_AddRefs(webBrowser));
         webNavigation = do_QueryInterface(webBrowser);
+        webBrowserPrint = do_GetInterface(webBrowser);
     }
 
     // Test the message
@@ -680,16 +693,17 @@ BOOL CALLBACK BrowserDlgProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
                 //       contentviewer AT ALL. This code below will break one
                 //       day but will have to do until the embedding API has
                 //       a cleaner way to do the same thing.
-
-                nsCOMPtr <nsIDocShell> rootDocShell = do_GetInterface(webBrowser);
-                nsCOMPtr<nsIContentViewer> pContentViewer;
-                nsresult res = rootDocShell->GetContentViewer(getter_AddRefs(pContentViewer));
-
-                if (NS_SUCCEEDED(res))
-                {
-                    nsCOMPtr<nsIContentViewerFile> spContentViewerFile = do_QueryInterface(pContentViewer); 
-                    spContentViewerFile->Print(PR_TRUE, nsnull, (nsIWebProgressListener*)nsnull);
-                }
+              if (webBrowserPrint)
+              {
+                  nsCOMPtr<nsIPrintSettings> printSettings;
+                  webBrowserPrint->GetGlobalPrintSettings(getter_AddRefs(printSettings));
+                  NS_ASSERTION(printSettings, "You can't PrintPreview without a PrintSettings!");
+                  if (printSettings) 
+                  {
+                      printSettings->SetPrintSilent(PR_TRUE);
+                      webBrowserPrint->Print(printSettings, (nsIWebProgressListener*)nsnull);
+                  }
+              }
             }
             break;
 
@@ -1074,6 +1088,9 @@ nativeWindow WebBrowserChromeUI::CreateNativeWindow(nsIWebBrowserChrome* chrome)
   SetWindowLong(hwndBrowser, GWL_USERDATA, (LONG)chrome);  // save the browser LONG_PTR.
   SetWindowLong(hwndBrowser, GWL_STYLE, GetWindowLong(hwndBrowser, GWL_STYLE) | WS_CLIPCHILDREN);
 
+  // Activate the window
+  PostMessage(hwndDialog, WM_ACTIVATE, WA_ACTIVE, 0);
+
   gDialogCount++;
 
   return hwndBrowser;
@@ -1196,8 +1213,8 @@ void WebBrowserChromeUI::UpdateCurrentURI(nsIWebBrowserChrome *aChrome)
     webNavigation->GetCurrentURI(getter_AddRefs(currentURI));
     if (currentURI)
     {
-        nsXPIDLCString uriString;
-        currentURI->GetSpec(getter_Copies(uriString));
+        nsCAutoString uriString;
+        currentURI->GetAsciiSpec(uriString);
         HWND hwndDlg = GetBrowserDlgFromChrome(aChrome);
         SetDlgItemText(hwndDlg, IDC_ADDRESS, uriString.get());
     }
@@ -1285,11 +1302,21 @@ void WebBrowserChromeUI::ShowWindow(nsIWebBrowserChrome *aChrome, PRBool aShow)
 
 void WebBrowserChromeUI::SizeTo(nsIWebBrowserChrome *aChrome, PRInt32 aWidth, PRInt32 aHeight)
 {
-  HWND win = GetBrowserDlgFromChrome(aChrome);
-  RECT winRect;
+  HWND hchrome = GetBrowserDlgFromChrome(aChrome);
+  HWND hbrowser = GetBrowserFromChrome(aChrome);
+  RECT chromeRect, browserRect;
 
-  ::GetWindowRect(win, &winRect);
-  ::MoveWindow(win, winRect.left, winRect.top, aWidth, aHeight, TRUE);
+  ::GetWindowRect(hchrome,  &chromeRect);
+  ::GetWindowRect(hbrowser, &browserRect);
+
+  PRInt32 decoration_x = (browserRect.left - chromeRect.left) + 
+                         (chromeRect.right - browserRect.right);
+  PRInt32 decoration_y = (browserRect.top - chromeRect.top) + 
+                         (chromeRect.bottom - browserRect.bottom);
+
+  ::MoveWindow(hchrome, chromeRect.left, chromeRect.top,
+      aWidth+decoration_x,
+      aHeight+decoration_y, TRUE);
 }
 
 //
@@ -1346,7 +1373,7 @@ nsresult AppCallbacks::CreateBrowserWindow(PRUint32 aChromeFlags,
   nsCOMPtr<nsIObserverService> observerService(do_GetService("@mozilla.org/observer-service;1"));
   if (observerService)
     observerService->AddObserver(NS_STATIC_CAST(nsIObserver *, chrome),
-                                 "profile-change-teardown", PR_FALSE);
+                                 "profile-change-teardown", PR_TRUE);
 
   // if opened as chrome, it'll be made visible after the chrome has loaded.
   // otherwise, go ahead and show it now.
