@@ -48,11 +48,11 @@
 #include "nsIPrompt.h"
 #include "nsIWindowWatcher.h"
 #include "nsVoidArray.h"
-#include "xp_core.h"
 #include "prmem.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsIIOService.h"
 #include "nsNetCID.h"
+#include "nsTextFormatter.h"
 
 static const char *kCookiesPermFileName = "cookperm.txt";
 
@@ -184,7 +184,8 @@ Permission_Check(
      const char * hostname,
      PRInt32 type,
      PRBool warningPref,
-     PRUnichar * message)
+     const char * message_string,
+     int count_for_message)
 {
   PRBool permission;
 
@@ -199,9 +200,15 @@ Permission_Check(
   }
 
   /* we need to prompt */
+  PRUnichar * message_fmt =
+      CKutil_Localize(NS_ConvertASCIItoUCS2(message_string).get());
+  PRUnichar * message = nsTextFormatter::smprintf(message_fmt,
+                            hostname ? hostname : "", count_for_message);
   PRBool rememberChecked = permission_GetRememberChecked(type);
   PRUnichar * remember_string = CKutil_Localize(NS_LITERAL_STRING("RememberThisDecision").get());
   permission = permission_CheckConfirmYN(aPrompter, message, remember_string, &rememberChecked);
+  nsTextFormatter::smprintf_free(message);
+  nsMemory::Free(message_fmt);
 
   /* see if we need to remember this decision */
   if (rememberChecked) {
@@ -219,6 +226,7 @@ Permission_Check(
     permission_changed = PR_TRUE;
     Permission_Save();
   }
+  Recycle(remember_string);
   return permission;
 }
 
@@ -397,7 +405,7 @@ Permission_Save() {
   for (PRInt32 i = 0; i < hostCount; ++i) {
     hostStruct = NS_STATIC_CAST(permission_HostStruct*, permission_list->ElementAt(i));
     if (hostStruct) {
-      strm.write(hostStruct->host, nsCRT::strlen(hostStruct->host));
+      strm.write(hostStruct->host, strlen(hostStruct->host));
 
       PRInt32 typeCount = hostStruct->permissionList->Count();
       for (PRInt32 typeIndex=0; typeIndex<typeCount; typeIndex++) {
@@ -480,7 +488,7 @@ PERMISSION_Read() {
     PRUint32 nextPermissionIndex = 0;
     hostIndex = 0;
 
-    if ((permissionIndex=buffer.FindChar('\t', PR_FALSE, hostIndex)+1) == 0) {
+    if ((permissionIndex=buffer.FindChar('\t', hostIndex)+1) == 0) {
       continue;      
     }
 
@@ -497,7 +505,7 @@ PERMISSION_Read() {
       if (nextPermissionIndex == buffer.Length()+1) {
         break;
       }
-      if ((nextPermissionIndex=buffer.FindChar('\t', PR_FALSE, permissionIndex)+1) == 0) {
+      if ((nextPermissionIndex=buffer.FindChar('\t', permissionIndex)+1) == 0) {
         nextPermissionIndex = buffer.Length()+1;
       }
       buffer.Mid(permissionString, permissionIndex, nextPermissionIndex-permissionIndex-1);
@@ -685,7 +693,7 @@ PERMISSION_DeletePersistentUserData(void)
   nsCOMPtr<nsIFile> cookiesPermFile;
   res = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR, getter_AddRefs(cookiesPermFile));
   if (NS_SUCCEEDED(res)) {
-    res = cookiesPermFile->Append(kCookiesPermFileName);
+    res = cookiesPermFile->AppendNative(nsDependentCString(kCookiesPermFileName));
     if (NS_SUCCEEDED(res))
         (void) cookiesPermFile->Remove(PR_FALSE);
   }
@@ -698,11 +706,11 @@ PERMISSION_Add(const char * objectURL, PRBool permission, PRInt32 type,
     return;
   }
   nsresult rv = NS_OK;
-  char *host = nsnull;
-  PRUint32 start,end;
+  nsCAutoString host;
   NS_ASSERTION(ioService, "IOService not available");
-  rv = ioService->ExtractUrlPart(objectURL, nsIIOService::url_Host |
-                                 nsIIOService::url_Port, &start, &end, &host);
+  rv = ioService->ExtractUrlPart(nsDependentCString(objectURL),
+                                 nsIIOService::url_Host |
+                                 nsIIOService::url_Port, host);
 
   /*
    * if permission is false, it will be added to the permission list
@@ -710,7 +718,7 @@ PERMISSION_Add(const char * objectURL, PRBool permission, PRInt32 type,
    *    true permission being added
    */
   if (permission) {
-    char * hostPtr = host;
+    char * hostPtr = (char *)host.get();
     while (PR_TRUE) {
       permission_Unblock(hostPtr, type);
       hostPtr = PL_strchr(hostPtr, '.');
@@ -719,10 +727,9 @@ PERMISSION_Add(const char * objectURL, PRBool permission, PRInt32 type,
       }
       hostPtr++; /* get passed the period */
     }
-    Recycle(host);
     return;
   }
-  Permission_AddHost(host, permission, type, PR_TRUE);
+  Permission_AddHost(ToNewCString(host), permission, type, PR_TRUE);
 }
 
 PUBLIC void
@@ -732,11 +739,11 @@ PERMISSION_TestForBlocking(const char * objectURL, PRBool* blocked, PRInt32 type
     return;
   }
   nsresult rv = NS_OK;
-  nsXPIDLCString host;
-  PRUint32 start,end;
+  nsCAutoString host;
   NS_ASSERTION(ioService, "IOService not available");
-  rv = ioService->ExtractUrlPart(objectURL, nsIIOService::url_Host |
-                                 nsIIOService::url_Port, &start, &end, getter_Copies(host));
+  rv = ioService->ExtractUrlPart(nsDependentCString(objectURL),
+                                 nsIIOService::url_Host |
+                                 nsIIOService::url_Port, host);
 
   const char * hostPtr = host.get();
   while (PR_TRUE) {
