@@ -36,7 +36,7 @@ const MSG_UNKNOWN   = getMsg ("unknown");
 
 client.defaultNick = getMsg( "defaultNick" );
 
-client.version = "0.8.5";
+client.version = "0.8.8";
 
 client.TYPE = "IRCClient";
 client.COMMAND_CHAR = "/";
@@ -406,18 +406,18 @@ function initHost(obj)
                                "event-tracer", true /* negate */,
                                false /* disable */);
 
-    obj.linkRE = /((\w+):\/\/[^<>\[\]()\'\"\s]+|www(\.[^.<>\[\]()\'\"\s]+){2,})/;
+    obj.linkRE = /((\w+):[^<>\[\]()\'\"\s]+|www(\.[^.<>\[\]()\'\"\s]+){2,})/;
 
     obj.munger = new CMunger();
     obj.munger.enabled = true;
+    obj.munger.addRule ("link", obj.linkRE, insertLink);
     obj.munger.addRule ("mailto",
        /(?:\s|\W|^)((mailto:)?[^<>\[\]()\'\"\s]+@[^.<>\[\]()\'\"\s]+\.[^<>\[\]()\'\"\s]+)/i,
                         insertMailToLink);
-    obj.munger.addRule ("link", obj.linkRE, insertLink);
     obj.munger.addRule ("bugzilla-link", /(?:\s|\W|^)(bug\s+#?\d{3,6})/i,
                         insertBugzillaLink);
     obj.munger.addRule ("channel-link",
-                    /(?:\s|\W|^)[@+]?(#[^<>\[\]()\'\"\s]+[^:,.<>\[\]()\'\"\s])/i,
+                /(?:\s|\W|^)[@+]?(#[^<>\[\](){}\"\s]*[^:,.<>\[\](){}\'\"\s])/i,
                         insertChannelLink);
     
     obj.munger.addRule ("face",
@@ -458,10 +458,10 @@ function insertLink (matchText, containerTag)
 
     var href;
     
-    if (matchText.indexOf ("://") == -1)
-        href = "http://" + matchText;
-    else
+    if (matchText.match (/^[a-zA-Z-]+:/))
         href = matchText;
+    else
+        href = "http://" + matchText;
     
     var anchor = document.createElementNS ("http://www.w3.org/1999/xhtml",
                                            "html:a");
@@ -630,7 +630,7 @@ function mircChangeColor (colorInfo, containerTag, data)
         delete data.currFgColor;
     if (ary.length >= 4)
     {
-        bgColor = ary[3];
+        var bgColor = ary[3];
         if (bgColor > 16)
             bgColor &= 16;
         if (bgColor.length == 1)
@@ -1449,6 +1449,17 @@ function updateChannel (obj)
     
     client.statusBar["channel-mode"].setAttribute("value", mode);
     client.statusBar["channel-users"].setAttribute("value", users);
+    var regex = new RegExp ("(\\S{" + client.MAX_WORD_DISPLAY + ",})", "g");
+    var ary = topic.match(regex);
+    if (ary && ary.length)
+    {
+        for (var i = 0; i < ary.length; ++i)
+        {
+            var hyphenated = hyphenateWord(ary[i], client.MAX_WORD_DISPLAY);
+            topic = topic.replace(ary[i], hyphenated);
+        }
+    }        
+
     client.statusBar["channel-topic"].firstChild.data = topic;
 
 }
@@ -1653,7 +1664,7 @@ function setCurrentObject (obj)
     }
     if (tb)
     {
-        tb.setAttribute ("selected", "false");
+        tb.selected = false;
         tb.setAttribute ("state", "normal");
     }
     
@@ -1663,20 +1674,24 @@ function setCurrentObject (obj)
 
     /* Unselect currently selected users. */
     userList = document.getElementById("user-list");
-    /* Remove curently selection items before this tree gets rerooted,
-     * because it seems to remember the selections for eternity if not. */
-    userList.clearSelection ();
-
-    if (obj.TYPE == "IRCChannel")
-        client.rdf.setTreeRoot ("user-list", obj.getGraphResource());
-    else
-        client.rdf.setTreeRoot ("user-list", client.rdf.resNullChan);
+    if (isVisible("user-list-box"))
+    {
+        /* Remove currently selected items before this tree gets rerooted,
+         * because it seems to remember the selections for eternity if not. */
+        if (userList.treeBoxObject.selection)
+            userList.treeBoxObject.selection.clearSelection ();
 
+        if (obj.TYPE == "IRCChannel")
+            client.rdf.setTreeRoot ("user-list", obj.getGraphResource());
+        else
+            client.rdf.setTreeRoot ("user-list", client.rdf.resNullChan);
+    }
+    
     client.currentObject = obj;
     tb = getTabForObject(obj);
     if (tb)
     {
-        tb.setAttribute ("selected", "true");
+        tb.selected = true;
         tb.setAttribute ("state", "current");
     }
     
@@ -1723,7 +1738,11 @@ function scrollDown ()
     window.frames[0].scrollTo(0, window.frames[0].document.height);
 }
 
-function notifyActivity (source)
+/* valid values for |what| are "superfluous", "activity", and "attention".
+ * final value for state is dependant on priority of the current state, and the
+ * new state. the priority is: normal < superfluous < activity < attention.
+ */
+function setTabState (source, what)
 {
     if (typeof source != "object")
         source = client.viewsArray[source].source;
@@ -1733,24 +1752,44 @@ function notifyActivity (source)
     
     if ("currentObject" in client && client.currentObject != source)
     {
-        if (tb.getAttribute ("state") == "normal")
-        {       
-            tb.setAttribute ("state", "activity");
-            if (!(vk in client.activityList))
+        var state = tb.getAttribute ("state");
+        if (state == what)
+        {
+            /* if the tab state has an equal priority to what we are setting
+             * then blink it */
+            tb.setAttribute ("state", "normal");
+            setTimeout (setTabState, 200, vk, what);
+        }
+        else
+        {
+            if (state == "normal" || state == "superfluous" ||
+               (state == "activity" && what == "attention"))
             {
-                client.activityList[vk] = "+";
+                /* if the tab state has a lower priority than what we are
+                 * setting, change it to the new state */
+                tb.setAttribute ("state", what);
+                /* we only change the activity list if priority has increased */
+                if (what == "attention")
+                   client.activityList[vk] = "!";
+                else if (what == "activity")
+                    client.activityList[vk] = "+";
+                else
+                {
+                   /* this is functionally equivalent to "+" for now */
+                   client.activityList[vk] = "-";
+                }
                 updateTitle();
             }
+            else
+            {
+                /* the current state of the tab has a higher priority than the
+                 * new state.
+                 * blink the new lower state quickly, then back to the old */
+                tb.setAttribute ("state", what);
+                setTimeout (setTabState, 200, vk, state);
+            }
         }
-        else if (tb.getAttribute("state") == "activity")
-            /* if act light is already lit, blink it real quick */
-        {
-            tb.setAttribute ("state", "normal");
-            setTimeout ("notifyActivity(" + vk + ");", 200);
-        }
-        
     }
-    
 }
 
 function notifyAttention (source)
@@ -1769,7 +1808,7 @@ function notifyAttention (source)
     }
 
     if (client.FLASH_WINDOW)
-        window.GetAttention();
+        window.getAttention();
     
 }
 
@@ -1829,7 +1868,7 @@ function getTabForObject (source, create)
                          "nsDragAndDrop.startDrag(event, tabDNDObserver);");
         tb.setAttribute ("href", source.getURL());
         tb.setAttribute ("name", source.name);
-        tb.setAttribute ("onclick", "onTabClick('" + id + "');");
+        tb.setAttribute ("onclick", "onTabClick(" + id.quote() + ");");
         tb.setAttribute ("crop", "right");
         
         tb.setAttribute ("class", "tab-bottom view-button");
@@ -1850,31 +1889,6 @@ function getTabForObject (source, create)
     
 }
 
-function retrieveURLFromData (aData, flavour)
-{
-    switch (flavour) 
-    {
-        case "text/unicode":
-            if (aData.search(client.linkRE) != -1)
-                return aData;
-            else
-                return null;
-
-        case "text/x-moz-url":
-            var data = aData.toString();
-            var separator = data.indexOf("\n");
-            if (separator != -1)
-                data = data.substr(0, separator);
-            return data;
-
-        case "application/x-moz-file":
-            return aData.URL;
-    }
-
-    return null;                                                   
-}
-
-
 var contentDropObserver = new Object();
 
 contentDropObserver.onDragOver =
@@ -1893,8 +1907,8 @@ function tabdnd_dover (aEvent, aFlavour, aDragSession)
 contentDropObserver.onDrop =
 function tabdnd_drop (aEvent, aXferData, aDragSession)
 {
-    var url = retrieveURLFromData(aXferData.data, aXferData.flavour.contentType);
-    if (!url)
+    var url = transferUtils.retrieveURLFromData(aXferData.data, aXferData.flavour.contentType);
+    if (!url || url.search(client.linkRE) == -1)
         return;
     
     if (url.search(/\.css$/i) != -1  && confirm (getMsg("tabdnd_drop", url)))
@@ -2120,8 +2134,11 @@ function __display(message, msgtype, sourceObj, destObj)
 
     function setAttribs (obj, c, attrs)
     {
-        for (var a in attrs)
-            obj.setAttribute (a, attrs[a]);
+        if (attrs)
+        {
+            for (var a in attrs)
+                obj.setAttribute (a, attrs[a]);
+        }
         obj.setAttribute ("class", c);
         obj.setAttribute ("msg-type", msgtype);
         obj.setAttribute ("msg-dest", toAttr);
@@ -2152,7 +2169,7 @@ function __display(message, msgtype, sourceObj, destObj)
     if      (fromUser && sourceObj == me)  fromAttr = me.nick + " ME!";
     else if (fromUser)                     fromAttr = sourceObj.nick;
     else if (typeof sourceObj == "object") fromAttr = sourceObj.name;
-        
+
     var toType = (destObj) ? destObj.TYPE : "unk";
     var toAttr;
     
@@ -2165,7 +2182,7 @@ function __display(message, msgtype, sourceObj, destObj)
 
     /* isImportant means to style the messages as important, and flash the
      * window, getAttention means just flash the window. */
-    var isImportant = false, getAttention = false;
+    var isImportant = false, getAttention = false, isSuperfluous = false;
     var viewType = this.TYPE;
     var code;
     var msgRow = document.createElementNS("http://www.w3.org/1999/xhtml",
@@ -2205,8 +2222,11 @@ function __display(message, msgtype, sourceObj, destObj)
         if (sourceObj != me)
         {
             nick = sourceObj.properNick;
-            if ("getURL" in sourceObj)
+            if (!nick)
+                nick = sourceObj.name + "@" + sourceObj.host;
+            else if ("getURL" in sourceObj)
                 nickURL = sourceObj.getURL();
+            
             if (toType == "IRCUser") /* msg from user to me */
             {
                 getAttention = true;
@@ -2273,6 +2293,7 @@ function __display(message, msgtype, sourceObj, destObj)
     }
     else
     {
+        isSuperfluous = true;
         if (!client.debugMode && msgtype in client.responseCodeMap)
         {
             code = client.responseCodeMap[msgtype];
@@ -2350,9 +2371,25 @@ function __display(message, msgtype, sourceObj, destObj)
 
     addHistory (this, msgRow, canMergeData, canCollapseRow);
     if (isImportant || getAttention)
-        notifyAttention(this);
+    {
+        setTabState(this, "attention");
+        if (client.FLASH_WINDOW)
+            window.getAttention();
+    }
     else
-        notifyActivity (this);
+    {
+        if (isSuperfluous)
+            setTabState(this, "superfluous");
+        else
+            setTabState(this, "activity");
+    }
+
+    if (isImportant && client.COPY_MESSAGES)
+    {
+        if ("network" in o && o.network != this)
+            o.network.displayHere("{" + this.name + "} " + message, msgtype,
+                                  sourceObj, destObj);
+    }
 }
 
 function addHistory (source, obj, mergeData, collapseRow)
@@ -2621,15 +2658,23 @@ function my_getselectedusers ()
     var cell; /* reference to each selected cell of the tree object */
     var rv_ary = new Array; /* return value arrray for CIRCChanUser objects */
 
-    for (var i = 0; i < tree.selectedItems.length; i++)
+    var rangeCount = tree.view.selection.getRangeCount();
+    for (var i = 0; i < rangeCount; ++i)
     {
-        /* First, set the reference to the XUL element. */
-        cell = tree.selectedItems[i].firstChild.childNodes[2].firstChild;
+        var start = {}, end = {};
+        tree.view.selection.getRangeAt(i, start, end);
+        for (var k = start.value; k <= end.value; ++k)
+        {
+            var item = tree.contentView.getItemAtIndex(k);
 
-        /* Now, create an instance of CIRCChaneUser by passing the text
-       *  of the cell to the getUser function of this CIRCChannel instance.
-       */
-        rv_ary[i] = this.getUser( cell.getAttribute("value") );
+            /* First, set the reference to the XUL element. */
+            cell = item.firstChild.childNodes[2];
+            
+            /* Now, create an instance of CIRCChaneUser by passing the text
+          *  of the cell to the getUser function of this CIRCChannel instance.
+          */
+            rv_ary[i] = this.getUser( cell.getAttribute("label") );
+        }
     }
 
     /* 
