@@ -46,6 +46,7 @@
 #include "nsIObserverService.h"
 #include "nsIEventQueueService.h"
 #include "nsLiteralString.h"
+#include "nsCRT.h"
 
 extern nsresult EvaluateAdminConfigScript(const char *js_buffer, size_t length,
                                           const char *filename, 
@@ -314,7 +315,7 @@ nsresult nsAutoConfig::downloadAutoConfig()
         return rv;
 
     // open a channel for the url
-    rv = NS_OpenURI(getter_AddRefs(channel),url, nsnull, nsnull, nsnull, nsIRequest::INHIBIT_PERSISTENT_CACHING | nsIRequest::LOAD_BYPASS_CACHE);
+    rv = NS_NewChannel(getter_AddRefs(channel),url, nsnull, nsnull, nsnull, nsIRequest::INHIBIT_PERSISTENT_CACHING | nsIRequest::LOAD_BYPASS_CACHE);
     if (NS_FAILED(rv)) 
         return rv;
 
@@ -378,7 +379,7 @@ nsresult nsAutoConfig::downloadAutoConfig()
             mTimer = do_CreateInstance("@mozilla.org/timer;1",&rv);
             if (NS_FAILED(rv)) 
                 return rv;
-            rv = mTimer->Init(this, minutes * 60 * 1000, NS_PRIORITY_NORMAL, 
+            rv = mTimer->Init(this, minutes * 60 * 1000, PR_TRUE, 
                              NS_TYPE_REPEATING_SLACK);
             if (NS_FAILED(rv)) 
                 return rv;
@@ -445,7 +446,7 @@ nsresult nsAutoConfig::readOfflineFile()
     if (NS_FAILED(rv)) 
         return rv;
     
-    failoverFile->Append("failover.jsc");
+    failoverFile->AppendNative(NS_LITERAL_CSTRING("failover.jsc"));
     rv = evaluateLocalFile(failoverFile);
     if (NS_FAILED(rv)) 
         NS_WARNING("Couldn't open failover.jsc, going back to default prefs");
@@ -491,7 +492,7 @@ nsresult nsAutoConfig::writeFailoverFile()
     if (NS_FAILED(rv)) 
         return rv;
     
-    failoverFile->Append("failover.jsc");
+    failoverFile->AppendNative(NS_LITERAL_CSTRING("failover.jsc"));
     
     rv = NS_NewLocalFileOutputStream(getter_AddRefs(outStr), failoverFile);
     if (NS_FAILED(rv)) 
@@ -501,7 +502,7 @@ nsresult nsAutoConfig::writeFailoverFile()
     return rv;
 }
 
-nsresult nsAutoConfig::getEmailAddr(nsAWritableCString & emailAddr)
+nsresult nsAutoConfig::getEmailAddr(nsACString & emailAddr)
 {
     
     nsresult rv;
@@ -519,18 +520,18 @@ nsresult nsAutoConfig::getEmailAddr(nsAWritableCString & emailAddr)
     // Checking prefValue and its length.  Since by default the preference 
     // is set to nothing
     PRUint32 len;
-    if (NS_SUCCEEDED(rv) && (len = nsCRT::strlen(prefValue)) > 0) {
+    if (NS_SUCCEEDED(rv) && (len = strlen(prefValue)) > 0) {
         emailAddr = NS_LITERAL_CSTRING("mail.account.") + 
             nsDependentCString(prefValue, len) + NS_LITERAL_CSTRING(".identities");
         rv = mPrefBranch->GetCharPref(PromiseFlatCString(emailAddr).get(),
                                       getter_Copies(prefValue));
-        if (NS_FAILED(rv) || (len = nsCRT::strlen(prefValue)) == 0) 
+        if (NS_FAILED(rv) || (len = strlen(prefValue)) == 0) 
             return rv;
         emailAddr = NS_LITERAL_CSTRING("mail.identity.") + 
             nsDependentCString(prefValue, len) + NS_LITERAL_CSTRING(".useremail");
         rv = mPrefBranch->GetCharPref(PromiseFlatCString(emailAddr).get(),
                                       getter_Copies(prefValue));
-        if (NS_FAILED(rv)  || (len = nsCRT::strlen(prefValue)) == 0) 
+        if (NS_FAILED(rv)  || (len = strlen(prefValue)) == 0) 
             return rv;
         emailAddr = nsDependentCString(prefValue, len);
     }
