@@ -72,7 +72,6 @@
 #include "nsIDOMProcessingInstruction.h"
 #include "nsIDOMText.h"
 #include "nsINameSpaceManager.h"
-
 #include "txAtom.h"
 
 #ifndef NULL
@@ -331,8 +330,6 @@ class Document : public Node
         Document(nsIDOMDocument* aDocument);
         ~Document();
 
-        PRBool inHashTableDeletion();
-
         Element* getDocumentElement();
         DocumentType* getDoctype();
         DOMImplementation* getImplementation();
@@ -401,9 +398,16 @@ class Document : public Node
 
         Element* getElementById(const String aID);
 
-    private:
-        PRBool bInHashTableDeletion;
+        PRInt32 namespaceURIToID(const String& aNamespaceURI);
+        void namespaceIDToURI(PRInt32 aNamespaceID, String& aNamespaceURI);
 
+#ifdef DEBUG
+protected:
+        PRBool bInHashTableDeletion;
+        friend class MozillaObjectWrapper;
+#endif
+
+    private:
         nsObjectHashtable *wrapperHashTable;
 
         nsCOMPtr<nsINameSpaceManager> nsNSManager;
@@ -421,8 +425,9 @@ class Element : public Node
         const String& getTagName();
         const String& getAttribute(const String& aName);
         void setAttribute(const String& aName, const String& aValue);
-        void setAttributeNS(const String& aNamespaceURI, const String& aName,
-                    const String& aValue);
+        void setAttributeNS(const String& aNamespaceURI,
+                            const String& aName,
+                            const String& aValue);
         void removeAttribute(const String& aName);
         Attr* getAttributeNode(const String& aName);
         Attr* setAttributeNode(Attr* aNewAttr);
@@ -466,13 +471,13 @@ class CharacterData : public Node
 
         const String& getData();
         void setData(const String& aSource);
-        PRInt32 getLength() const;
+        PRUint32 getLength() const;
 
-        String& substringData(PRInt32 aOffset, PRInt32 aCount, String& aDest);
+        String& substringData(PRUint32 aOffset, PRUint32 aCount, String& aDest);
         void appendData(const String& aSource);
-        void insertData(PRInt32 aOffset, const String& aSource);
-        void deleteData(PRInt32 aOffset, PRInt32 aCount);
-        void replaceData(PRInt32 aOffset, PRInt32 aCount, const String& aSource);
+        void insertData(PRUint32 aOffset, const String& aSource);
+        void deleteData(PRUint32 aOffset, PRUint32 aCount);
+        void replaceData(PRUint32 aOffset, PRUint32 aCount, const String& aSource);
 
     private:
         String nodeValue;
@@ -487,7 +492,7 @@ class Text : public CharacterData
         Text(nsIDOMText* aText, Document* aOwner);
         ~Text();
 
-        Text* splitText(PRInt32 aOffset);
+        Text* splitText(PRUint32 aOffset);
 };
 
 /*
