@@ -73,6 +73,57 @@
 
 static NS_DEFINE_IID(kDocLoaderServiceCID, NS_DOCUMENTLOADER_SERVICE_CID);
 
+////////////////////////////////////////////////////////////////////////////////
+// nsSingleSignOnProfileObserver
+// Signon data is used by both nsSingleSignOnPrompt and the wallet service.
+// This observer is global and initialized by the first consumer of signon data.
+
+class nsSingleSignOnProfileObserver : public nsIObserver
+{
+public:
+    nsSingleSignOnProfileObserver() { NS_INIT_REFCNT(); }
+    virtual ~nsSingleSignOnProfileObserver() {}
+    
+    NS_DECL_ISUPPORTS
+    
+    NS_IMETHODIMP Observe(nsISupports*, const char *aTopic, const PRUnichar *someData) 
+    {
+        if (!nsCRT::strcmp(aTopic, "profile-before-change")) {
+            SI_ClearUserData();
+        if (!nsCRT::strcmp(someData, NS_LITERAL_STRING("shutdown-cleanse").get()))
+            SI_DeletePersistentUserData();
+        }
+        return NS_OK;
+    }
+};
+NS_IMPL_THREADSAFE_ISUPPORTS1(nsSingleSignOnProfileObserver, nsIObserver)
+
+static nsresult EnsureSingleSignOnProfileObserver()
+{
+  static nsSingleSignOnProfileObserver *gSignOnProfileObserver;
+  
+  if (!gSignOnProfileObserver) {      
+    nsCOMPtr<nsIObserverService> observerService(do_GetService("@mozilla.org/observer-service;1"));
+    if (!observerService)
+      return NS_ERROR_FAILURE;
+      
+    gSignOnProfileObserver = new nsSingleSignOnProfileObserver;
+    if (!gSignOnProfileObserver)
+      return NS_ERROR_OUT_OF_MEMORY;
+
+    // The observer service holds the only ref to the observer
+    // It thus has the lifespan of the observer service
+    nsresult rv = observerService->AddObserver(gSignOnProfileObserver, "profile-before-change", PR_FALSE);
+    if (NS_FAILED(rv)) {
+      delete gSignOnProfileObserver;
+      gSignOnProfileObserver = nsnull; 
+      return rv;
+    }
+  }
+  return NS_OK;
+}
+
+////////////////////////////////////////////////////////////////////////////////
 
 nsWalletlibService::nsWalletlibService()
 {
@@ -224,10 +275,9 @@ NS_IMETHODIMP nsWalletlibService::Observe(nsISupports *aSubject, const char *aTo
     // A login succeeded; store the password.
     nsCOMPtr<nsIURI> uri = do_QueryInterface(aSubject);
     if (uri) {
-      nsXPIDLCString spec;
-      uri->GetSpec(getter_Copies(spec));
-      if (spec)
-        SI_StorePassword(spec, nsnull, someData);
+      nsCAutoString spec;
+      if (NS_SUCCEEDED(uri->GetSpec(spec)))
+        SI_StorePassword(spec.get(), nsnull, someData);
     }
   }
   else if (!nsCRT::strcmp(aTopic, "login-failed")) {
@@ -235,10 +285,10 @@ NS_IMETHODIMP nsWalletlibService::Observe(nsISupports *aSubject, const char *aTo
     // the URL where the failure occurred.
     nsCOMPtr<nsIURI> uri = do_QueryInterface(aSubject);
     if (uri) {
-      nsXPIDLCString spec;
-      uri->GetSpec(getter_Copies(spec));
-      if (spec)
-        SI_RemoveUser(spec, nsnull);
+      nsCAutoString spec;
+      uri->GetSpec(spec);
+      if (NS_SUCCEEDED(uri->GetSpec(spec)))
+        SI_RemoveUser(spec.get(), nsnull);
     }
   }
   return NS_OK;
@@ -329,6 +379,9 @@ nsresult nsWalletlibService::Init()
   }
   else
     NS_ASSERTION(PR_FALSE, "Could not get nsIObserverService");
+    
+  rv = EnsureSingleSignOnProfileObserver();
+  NS_ASSERTION(NS_SUCCEEDED(rv), "Failed to register profile change observer");
 
   // Get the global document loader service...  
   nsCOMPtr<nsIDocumentLoader> docLoaderService = 
@@ -336,7 +389,8 @@ nsresult nsWalletlibService::Init()
   if (NS_SUCCEEDED(rv) && docLoaderService) {
     nsCOMPtr<nsIWebProgress> progress(do_QueryInterface(docLoaderService, &rv));
     if (NS_SUCCEEDED(rv))
-        (void) progress->AddProgressListener((nsIWebProgressListener*)this);
+        (void) progress->AddProgressListener((nsIWebProgressListener*)this,
+                                        nsIWebProgress::NOTIFY_STATE_DOCUMENT);
   }
   else
     NS_ASSERTION(PR_FALSE, "Could not get nsIDocumentLoader");
@@ -355,7 +409,7 @@ nsresult nsWalletlibService::Init()
 NS_IMETHODIMP
 nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress, 
                                   nsIRequest *aRequest, 
-                                  PRInt32 progressStateFlags, 
+                                  PRUint32 progressStateFlags,
                                   nsresult aStatus)
 {
     nsresult rv = NS_OK;
@@ -395,8 +449,8 @@ nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress,
             return NS_OK;
           }
 
-          nsXPIDLCString spec;
-          rv = uri->GetSpec(getter_Copies(spec));
+          nsCAutoString spec;
+          rv = uri->GetSpec(spec);
           if (NS_FAILED(rv)) return rv;
 
           nsCOMPtr<nsIDOMHTMLCollection> forms;
@@ -429,7 +483,7 @@ nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress,
                         nsAutoString type;
                         rv = inputElement->GetType(type);
                         if (NS_SUCCEEDED(rv)) {
-                          if (Compare(type, NS_LITERAL_STRING("password"), nsCaseInsensitiveStringComparator()) == 0) {
+                          if (type.Equals(NS_LITERAL_STRING("password"), nsCaseInsensitiveStringComparator())) {
                             passwordCount++;
                           }
                         }
@@ -449,8 +503,11 @@ nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress,
                         nsAutoString type;
                         rv = inputElement->GetType(type);
                         if (NS_SUCCEEDED(rv)) {
-                          if ((type.IsEmpty()) || (Compare(type, NS_LITERAL_STRING("text"), nsCaseInsensitiveStringComparator()) == 0) ||
-                            (Compare(type, NS_LITERAL_STRING("password"), nsCaseInsensitiveStringComparator()) == 0)) {
+                          if (type.IsEmpty() ||
+                              type.Equals(NS_LITERAL_STRING("text"),
+                                          nsCaseInsensitiveStringComparator()) ||
+                              type.Equals(NS_LITERAL_STRING("password"),
+                                          nsCaseInsensitiveStringComparator())) {
                             nsAutoString field;
                             rv = inputElement->GetName(field);
                             if (NS_SUCCEEDED(rv)) {
@@ -472,7 +529,7 @@ nsWalletlibService::OnStateChange(nsIWebProgress* aWebProgress,
                                     wwatch->GetNewPrompter(0, getter_AddRefs(prompter));
                                 }
                                 if (prompter) {
-                                  SINGSIGN_RestoreSignonData(prompter, spec, nameString, &valueString, elementNumber++);
+                                  SINGSIGN_RestoreSignonData(prompter, spec.get(), nameString, &valueString, elementNumber++);
                                 }
                                 if (valueString) {
                                   value = valueString;
@@ -508,7 +565,8 @@ nsWalletlibService::OnProgressChange(nsIWebProgress *aWebProgress,
                                      PRInt32 aCurTotalProgress,
                                      PRInt32 aMaxTotalProgress)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -516,7 +574,8 @@ nsWalletlibService::OnLocationChange(nsIWebProgress* aWebProgress,
                                      nsIRequest* aRequest,
                                      nsIURI *location)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
@@ -526,16 +585,18 @@ nsWalletlibService::OnStatusChange(nsIWebProgress* aWebProgress,
                                    nsresult aStatus,
                                    const PRUnichar* aMessage)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;    
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
 NS_IMETHODIMP
 nsWalletlibService::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                      nsIRequest *aRequest, 
-                                     PRInt32 state)
+                                     PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    NS_NOTREACHED("notification excluded in AddProgressListener(...)");
+    return NS_OK;
 }
 
 
@@ -569,55 +630,19 @@ nsWalletlibService::WALLET_Decrypt (const char *crypt, PRUnichar **text) {
   return rv;
 }
 
-////////////////////////////////////////////////////////////////////////////////
-// nsSingleSignOnProfileObserver
-// nsSingleSignOnPrompt itself can't be a profile change observer because, if the
-// last instance has been destroyed at the time of a profile change, there's
-// nothing to get the notification.
-
-class nsSingleSignOnProfileObserver : public nsIObserver
-{
-public:
-    nsSingleSignOnProfileObserver() { NS_INIT_REFCNT(); }
-    virtual ~nsSingleSignOnProfileObserver() {}
-    
-    NS_DECL_ISUPPORTS
-    
-    NS_IMETHODIMP Observe(nsISupports*, const char *aTopic, const PRUnichar *someData) 
-    {
-        if (!nsCRT::strcmp(aTopic, "profile-before-change")) {
-            SI_ClearUserData();
-        if (!nsCRT::strcmp(someData, NS_LITERAL_STRING("shutdown-cleanse").get()))
-            SI_DeletePersistentUserData();
-        }
-        return NS_OK;
-    }
-};
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsSingleSignOnProfileObserver, nsIObserver)
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsSingleSignOnPrompt
 
-PRBool nsSingleSignOnPrompt::mgRegisteredObserver = PR_FALSE;
-
 NS_IMPL_THREADSAFE_ISUPPORTS2(nsSingleSignOnPrompt,
-                              nsISingleSignOnPrompt,
+                              nsIAuthPromptWrapper,
                               nsIAuthPrompt)
 
 nsresult
 nsSingleSignOnPrompt::Init()
 {
-  if (!mgRegisteredObserver) {
-    nsSingleSignOnProfileObserver *observer = new nsSingleSignOnProfileObserver;
-    if (!observer)
-        return NS_ERROR_OUT_OF_MEMORY;
-    nsCOMPtr<nsIObserverService> svc(do_GetService("@mozilla.org/observer-service;1"));
-    if (!svc) return NS_ERROR_FAILURE;
-    // The observer service holds the only ref to the observer
-    // It thus has the lifespan of the observer service
-    svc->AddObserver(observer, "profile-before-change", PR_FALSE);
-    mgRegisteredObserver = PR_TRUE;
-  }
+  nsresult rv = EnsureSingleSignOnProfileObserver();
+  NS_ASSERTION(NS_SUCCEEDED(rv), "Failed to register profile change observer");
   return NS_OK;
 }
 
