@@ -378,9 +378,6 @@ si_3ButtonConfirm(PRUnichar * szMessage, nsIDOMWindowInternal* window) {
   return Wallet_3ButtonConfirm(szMessage, window);
 }
 
-// This will go away once select is passed a prompter interface
-#include "nsAppShellCIDs.h" // TODO remove later
-
 PRIVATE PRBool
 si_SelectDialog(const PRUnichar* szMessage, nsIPrompt* dialog, PRUnichar** pList, PRInt32* pCount) {
   if (si_UserHasBeenSelected) {
@@ -394,6 +391,9 @@ si_SelectDialog(const PRUnichar* szMessage, nsIPrompt* dialog, PRUnichar** pList
   PRUnichar * title_string = Wallet_Localize("SelectUserTitleLine");
   rv = dialog->Select( title_string, szMessage, *pCount, NS_CONST_CAST(const PRUnichar**, pList), &selectedIndex, &rtnValue );
   Recycle(title_string);
+  if (selectedIndex >= *pCount) {
+    return PR_FALSE; // out-of-range selection
+  }
   *pCount = selectedIndex;
   si_UserHasBeenSelected = PR_TRUE;
   return rtnValue;  
@@ -750,11 +750,19 @@ si_GetURL(const char * passwordRealm) {
     }
     return (si_SignonURLStruct *) (si_signon_list->ElementAt(0));
   }
+
   PRInt32 urlCount = LIST_COUNT(si_signon_list);
-  for (PRInt32 i=0; i<urlCount; i++) {
-    url = NS_STATIC_CAST(si_SignonURLStruct*, si_signon_list->ElementAt(i));
-    if(url->passwordRealm && !PL_strcmp(passwordRealm, url->passwordRealm)) {
-      return url;
+  if (urlCount) {
+    // If the last char of passwordRealm is '/' then strip it before making comparison.
+    nsCAutoString realmWithoutTrailingSlash(passwordRealm);
+    if (!realmWithoutTrailingSlash.IsEmpty() && realmWithoutTrailingSlash.Last() == '/')
+      realmWithoutTrailingSlash.Truncate(realmWithoutTrailingSlash.Length()-1);
+
+    for (PRInt32 i=0; i<urlCount; i++) {
+      url = NS_STATIC_CAST(si_SignonURLStruct*, si_signon_list->ElementAt(i));
+      if(url->passwordRealm && !PL_strcmp(realmWithoutTrailingSlash.get(), url->passwordRealm)) {
+        return url;
+      }
     }
   }
   return (NULL);
@@ -767,11 +775,6 @@ si_RemoveUser(const char *passwordRealm, const nsString& userName, PRBool save,
   si_SignonUserStruct * user;
   si_SignonDataStruct * data;
 
-  /* do nothing if signon preference is not enabled */
-  if (!si_GetSignonRememberingPref()) {
-    return PR_FALSE;
-  }
-
   si_lock_signon_list();
 
   /* get URL corresponding to host */
@@ -860,6 +863,16 @@ SINGSIGN_RemoveReject(const char *host) {
   return rv;
 }
 
+PRIVATE void
+si_PutReject(const char * passwordRealm, const nsString& userName, PRBool save);
+
+PUBLIC nsresult
+SINGSIGN_AddReject(const char *host /*, const char *userName*/) {
+  si_PutReject(host, nsString(/*thisParameter_isObsolete*/), PR_TRUE);
+// @see http://bonsai.mozilla.org/cvsblame.cgi?file=mozilla/extensions/wallet/src/singsign.cpp&rev=1.212&mark=1693#1650
+  return NS_OK;
+}
+
 /* Determine if a specified url/user exists */
 PRIVATE PRBool
 si_CheckForUser(const char *passwordRealm, const nsString& userName) {
@@ -2058,12 +2071,29 @@ SINGSIGN_RememberSignonData
      nsIDOMWindowInternal* window)
 {
     
-  nsXPIDLCString strippedRealm;
+  nsCAutoString strippedRealm;
   nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID);
   if (!ioService) return;
-  ioService->ExtractUrlPart(passwordRealm, nsIIOService::url_Host, 0, 0, getter_Copies(strippedRealm));
-  if (strippedRealm) {
-    si_RememberSignonData(dialog, strippedRealm, signonData, window);
+
+  /* Hacky security check: If address is of a scheme that
+      doesn't support hostnames, we have no host to get the signon data from,
+      so we must not attempt to restore the signon data (bug 159484)
+  */
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = ioService->NewURI(nsDependentCString(passwordRealm),
+                                      nsnull, nsnull, getter_AddRefs(uri));
+  if (NS_FAILED(result)) {
+      return;
+  }
+  nsCAutoString tempHost;
+  result = uri->GetHost(tempHost);
+  if (NS_FAILED(result)) {
+      return;
+  }
+
+  ioService->ExtractUrlPart(nsDependentCString(passwordRealm), nsIIOService::url_Host, strippedRealm);
+  if (!strippedRealm.IsEmpty()) {
+    si_RememberSignonData(dialog, strippedRealm.get(), signonData, window);
   }
 }
 
@@ -2173,11 +2203,28 @@ si_RestoreSignonData(nsIPrompt* dialog, const char* passwordRealm, const PRUnich
 
 PUBLIC void
 SINGSIGN_RestoreSignonData(nsIPrompt* dialog, const char* passwordRealm, const PRUnichar* name, PRUnichar** value, PRUint32 elementNumber) {
-  nsXPIDLCString strippedRealm;
+  nsCAutoString strippedRealm;
   nsCOMPtr<nsIIOService> ioService = do_GetService(NS_IOSERVICE_CONTRACTID);
   if (!ioService) return;
-  ioService->ExtractUrlPart(passwordRealm, nsIIOService::url_Host, 0, 0, getter_Copies(strippedRealm));
-  si_RestoreSignonData(dialog, strippedRealm, name, value, elementNumber);
+
+  /* Hacky security check: If address is of a scheme that
+      doesn't support hostnames, we have no host to get the signon data from,
+      so we must not attempt to restore the signon data (bug 159484)
+  */
+  nsCOMPtr<nsIURI> uri;
+  nsresult result = ioService->NewURI(nsDependentCString(passwordRealm),
+                                      nsnull, nsnull, getter_AddRefs(uri));
+  if (NS_FAILED(result)) {
+      return;
+  }
+  nsCAutoString tempHost;
+  result = uri->GetHost(tempHost);
+  if (NS_FAILED(result)) {
+      return;
+  }
+
+  ioService->ExtractUrlPart(nsDependentCString(passwordRealm), nsIIOService::url_Host, strippedRealm);
+  si_RestoreSignonData(dialog, (char*)strippedRealm.get(), name, value, elementNumber);
 }
 
 /*
@@ -2282,6 +2329,7 @@ si_DoDialogIfPrefIsOff(
     prompt_string = Wallet_Localize("PromptForData");
   }
 
+  nsAutoString data(defaultText);
   switch (dlg) {
     case promptUsernameAndPassword:
       res = dialog->PromptUsernameAndPassword(prompt_string,
@@ -2299,6 +2347,7 @@ si_DoDialogIfPrefIsOff(
                                    pressedOK);
       break;
     case prompt:
+      *resultText = ToNewUnicode(data);
       res = dialog->Prompt(prompt_string,
                            text,
                            resultText,
@@ -2532,7 +2581,7 @@ SI_InSequence(const nsString& sequence, PRInt32 number)
 }
 
 PUBLIC void
-SI_FindValueInArgs(const nsAReadableString& results, const nsAReadableString& name, nsAWritableString& value)
+SI_FindValueInArgs(const nsAString& results, const nsAString& name, nsAString& value)
 {
   /* note: name must start and end with a vertical bar */
   nsReadingIterator<PRUnichar> start, end, barPos;
@@ -2654,7 +2703,7 @@ SINGSIGN_Enumerate
   hostStruct = NS_STATIC_CAST(si_SignonURLStruct*, si_signon_list->ElementAt(hostNumber));
   NS_ASSERTION(hostStruct, "corrupt singlesignon list");
   *host = (char *) nsMemory::Clone
-    (hostStruct->passwordRealm, nsCRT::strlen(hostStruct->passwordRealm) + 1);
+    (hostStruct->passwordRealm, strlen(hostStruct->passwordRealm) + 1);
   NS_ENSURE_ARG_POINTER(host);
   userStruct = NS_STATIC_CAST
     (si_SignonUserStruct*, hostStruct->signonUser_list.ElementAt(userNumber));
@@ -2716,7 +2765,7 @@ SINGSIGN_RejectEnumerate
   NS_ASSERTION(reject, "corrupt reject list");
 
   *host = (char *) nsMemory::Clone
-    (reject->passwordRealm, nsCRT::strlen(reject->passwordRealm) + 1);
+    (reject->passwordRealm, strlen(reject->passwordRealm) + 1);
   NS_ENSURE_ARG_POINTER(host);
   return NS_OK;
 }
