@@ -48,6 +48,8 @@
 #include "nsIPrivateDOMImplementation.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
+#include "nsPrintfCString.h"
+#include "nsCRT.h"
 #include "nsIURI.h"
 #include "nsILoadGroup.h"
 #include "nsNetUtil.h"
@@ -162,7 +164,7 @@ NS_INTERFACE_MAP_BEGIN(nsXMLHttpRequest)
   NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
   NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
-  NS_INTERFACE_MAP_ENTRY_DOM_CLASSINFO(XMLHttpRequest)
+  NS_INTERFACE_MAP_ENTRY_EXTERNAL_DOM_CLASSINFO(XMLHttpRequest)
 NS_INTERFACE_MAP_END
 
 
@@ -173,15 +175,13 @@ NS_IMPL_RELEASE(nsXMLHttpRequest)
 /* void addEventListener (in string type, in nsIDOMEventListener
    listener); */
 NS_IMETHODIMP
-nsXMLHttpRequest::AddEventListener(const nsAReadableString& type,
+nsXMLHttpRequest::AddEventListener(const nsAString& type,
                                    nsIDOMEventListener *listener,
                                    PRBool useCapture)
 {
   NS_ENSURE_ARG(listener);
   nsresult rv;
 
-  // I know, I know - strcmp's. But it's only for a couple of 
-  // cases...and they are short strings. :-)
   if (type.Equals(LOADSTR)) {
     if (!mLoadEventListeners) {
       rv = NS_NewISupportsArray(getter_AddRefs(mLoadEventListeners));
@@ -207,7 +207,7 @@ nsXMLHttpRequest::AddEventListener(const nsAReadableString& type,
 /* void removeEventListener (in string type, in nsIDOMEventListener
    listener); */
 NS_IMETHODIMP 
-nsXMLHttpRequest::RemoveEventListener(const nsAReadableString & type,
+nsXMLHttpRequest::RemoveEventListener(const nsAString & type,
                                       nsIDOMEventListener *listener,
                                       PRBool useCapture)
 {
@@ -331,22 +331,19 @@ NS_IMETHODIMP nsXMLHttpRequest::GetResponseXML(nsIDOMDocument **aResponseXML)
  * from HTTP headers.
  */
 nsresult
-nsXMLHttpRequest::DetectCharset(nsAWritableString& aCharset)
+nsXMLHttpRequest::DetectCharset(nsAString& aCharset)
 {
   aCharset.Truncate();
   nsresult rv;
-  nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel,&rv));
-  if(httpChannel) {
-    nsXPIDLCString charsetheader;
-    rv = httpChannel->GetCharset(getter_Copies(charsetheader));
-    if (NS_SUCCEEDED(rv)) {
-      nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
-      if(NS_SUCCEEDED(rv) && calias) {
-        nsAutoString preferred;
-        rv = calias->GetPreferred(NS_ConvertASCIItoUCS2(charsetheader), preferred);
-        if(NS_SUCCEEDED(rv)) {
-          aCharset.Assign(preferred);        
-        }
+  nsCAutoString charsetVal;
+  rv = mChannel->GetContentCharset(charsetVal);
+  if (NS_SUCCEEDED(rv)) {
+    nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID,&rv));
+    if(NS_SUCCEEDED(rv) && calias) {
+      nsAutoString preferred;
+      rv = calias->GetPreferred(NS_ConvertASCIItoUCS2(charsetVal), preferred);
+      if(NS_SUCCEEDED(rv)) {
+        aCharset.Assign(preferred);        
       }
     }
   }
@@ -356,6 +353,11 @@ nsXMLHttpRequest::DetectCharset(nsAWritableString& aCharset)
 nsresult
 nsXMLHttpRequest::ConvertBodyToText(PRUnichar **aOutBuffer)
 {
+  // This code here is basically a copy of a similar thing in
+  // nsScanner::Append(const char* aBuffer, PRUint32 aLen).
+  // If we get illegal characters in the input we replace 
+  // them and don't just fail.
+
   *aOutBuffer = nsnull;
 
   PRInt32 dataLen = mResponseBody.Length();
@@ -392,46 +394,51 @@ nsXMLHttpRequest::ConvertBodyToText(PRUnichar **aOutBuffer)
   if (NS_FAILED(rv))
     return rv;
 
-  // This loop here is basically a copy of a similar thing in nsScanner.
-  // It will exit on first iteration in normal cases, but if we get illegal
-  // characters in the input we replace them and then continue.
   const char * inBuffer = mResponseBody.get();
-  PRUnichar * outBuffer = nsnull;
-  PRInt32 outBufferIndex = 0, inBufferLength = dataLen;
-  PRInt32 outLen;
-  for(;;) {
-    rv = decoder->GetMaxLength(inBuffer,inBufferLength,&outLen);
-    if (NS_FAILED(rv))
-      return rv;
+  PRInt32 outBufferLength;
+  rv = decoder->GetMaxLength(inBuffer, dataLen, &outBufferLength);
+  if (NS_FAILED(rv))
+    return rv;
 
-    PRUnichar * newOutBuffer = NS_STATIC_CAST(PRUnichar*,nsMemory::Realloc(outBuffer,(outLen+outBufferIndex+1) * sizeof(PRUnichar)));
-    if (!newOutBuffer) {
-      nsMemory::Free(outBuffer);
-      return NS_ERROR_OUT_OF_MEMORY;
-    }
-    outBuffer = newOutBuffer;
+  PRUnichar * outBuffer = 
+    NS_STATIC_CAST(PRUnichar*, nsMemory::Alloc((outBufferLength + 1) *
+                                               sizeof(PRUnichar)));
+  if (!outBuffer) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
 
-    rv = decoder->Convert(inBuffer,&inBufferLength,&outBuffer[outBufferIndex],&outLen);
-    if (rv == NS_OK) {
-      break;
-    } else if (rv == NS_ERROR_ILLEGAL_INPUT) {
+  PRInt32 totalChars = 0,
+          outBufferIndex = 0,
+          outLen = outBufferLength;
+
+  do {
+    PRInt32 inBufferLength = dataLen;
+    rv = decoder->Convert(inBuffer, 
+                          &inBufferLength, 
+                          &outBuffer[outBufferIndex], 
+                          &outLen);
+    totalChars += outLen;
+    if (NS_FAILED(rv)) {
       // We consume one byte, replace it with U+FFFD
       // and try the conversion again.
-      outBuffer[outBufferIndex + outLen] = (PRUnichar)0xFFFD;
-      outLen++;
+      outBuffer[outBufferIndex + outLen++] = (PRUnichar)0xFFFD;
       outBufferIndex += outLen;
-      inBuffer = &inBuffer[outLen];
-      inBufferLength -= outLen;
-    } else if (NS_FAILED(rv)) {
-      nsMemory::Free(outBuffer);
-      return rv; // We bail out on other critical errors
-    } else /*if (rv != NS_OK)*/ {
-      NS_ABORT_IF_FALSE(0,"This should not happen, nsIUnicodeDecoder::Convert() succeeded partially");
-      nsMemory::Free(outBuffer);
-      return NS_ERROR_FAILURE;
+      outLen = outBufferLength - (++totalChars);
+
+      decoder->Reset();
+
+      if((inBufferLength + 1) > dataLen) {
+        inBufferLength = dataLen;
+      } else {
+        inBufferLength++;
+      }
+
+      inBuffer = &inBuffer[inBufferLength];
+      dataLen -= inBufferLength;
     }
-  }
-  outBuffer[outBufferIndex + outLen] = '\0';
+  } while ( NS_FAILED(rv) && (dataLen > 0) );
+
+  outBuffer[totalChars] = '\0';
   *aOutBuffer = outBuffer;
 
   return NS_OK;
@@ -479,10 +486,15 @@ nsXMLHttpRequest::GetStatusText(char * *aStatusText)
   NS_ENSURE_ARG_POINTER(aStatusText);
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
 
-  if (httpChannel) {
-    return httpChannel->GetResponseStatusText(aStatusText);
-  }
   *aStatusText = nsnull;
+
+  if (httpChannel) {
+    nsCAutoString text;
+    nsresult rv = httpChannel->GetResponseStatusText(text);
+    if (NS_FAILED(rv)) return rv;
+    *aStatusText = ToNewCString(text);
+    return *aStatusText ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  }
   
   return NS_OK;
 }
@@ -492,7 +504,10 @@ NS_IMETHODIMP
 nsXMLHttpRequest::Abort()
 {
   if (mReadRequest) {
-    return mReadRequest->Cancel(NS_BINDING_ABORTED);
+    mReadRequest->Cancel(NS_BINDING_ABORTED);
+  }
+  if (mChannel) {
+    mChannel->Cancel(NS_BINDING_ABORTED);
   }
   
   return NS_OK;
@@ -532,8 +547,13 @@ nsXMLHttpRequest::GetResponseHeader(const char *header, char **_retval)
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
 
   *_retval = nsnull;
-  if (httpChannel)
-    return httpChannel->GetResponseHeader(header, _retval);
+  if (httpChannel) {
+    nsCAutoString buf;
+    nsresult rv = httpChannel->GetResponseHeader(nsDependentCString(header), buf);
+    if (NS_FAILED(rv)) return rv;
+    *_retval = ToNewCString(buf);
+    return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+  }
   
   return NS_OK;
 }
@@ -564,24 +584,24 @@ nsXMLHttpRequest::OpenRequest(const char *method,
   if (NS_FAILED(rv)) return rv;
 
   if (user) {
-    nsCAutoString prehost;
-    prehost.Assign(user);
+    nsCAutoString userpass;
+    userpass.Assign(user);
     if (password) {
-      prehost.Append(":");
-      prehost.Append(password);
+      userpass.Append(":");
+      userpass.Append(password);
     }
-    uri->SetPreHost(prehost.get());
+    uri->SetUserPass(userpass);
     authp = PR_TRUE;
   }
 
-  rv = NS_OpenURI(getter_AddRefs(mChannel), uri, nsnull, nsnull);
+  rv = NS_NewChannel(getter_AddRefs(mChannel), uri, nsnull, nsnull);
   if (NS_FAILED(rv)) return rv;
   
   //mChannel->SetAuthTriedWithPrehost(authp);
 
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
   if (httpChannel) {
-    rv = httpChannel->SetRequestMethod(method);
+    rv = httpChannel->SetRequestMethod(nsDependentCString(method));
   }
 
   ChangeState(XML_HTTP_REQUEST_OPENED);
@@ -716,14 +736,13 @@ nsXMLHttpRequest::GetStreamForWString(const PRUnichar* aStr,
   }
   
   // If no content type header was set by the client, we set it to text/xml.
-  nsXPIDLCString header;
-  if( NS_OK != httpChannel->GetRequestHeader("Content-Type", getter_Copies(header)) )  
-    httpChannel->SetRequestHeader("Content-Type", "text/xml" );
+  nsCAutoString header;
+  if( NS_FAILED(httpChannel->GetRequestHeader(NS_LITERAL_CSTRING("Content-Type"), header)) )  
+    httpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Content-Type"),
+                                  NS_LITERAL_CSTRING("text/xml") );
 
   // set the content length header
-  char charLengthBuf [32];
-  PR_snprintf(charLengthBuf, sizeof(charLengthBuf), "%d", charLength);
-  httpChannel->SetRequestHeader("Content-Length", charLengthBuf );
+  httpChannel->SetRequestHeader(NS_LITERAL_CSTRING("Content-Length"), nsPrintfCString("%d", charLength) );
 
   // Shove in the trailing and leading CRLF
   postData[0] = nsCRT::CR;
@@ -764,23 +783,25 @@ nsXMLHttpRequest::StreamReaderFunc(nsIInputStream* in,
   // Copy for our own use
   xmlHttpRequest->mResponseBody.Append(fromRawSegment,count);
 
-  nsresult rv;
+  nsresult rv = NS_OK;
 
-  // Give the same data to the parser.
+  if (xmlHttpRequest->mParseResponseBody) {
+    // Give the same data to the parser.
 
-  // We need to wrap the data in a new lightweight stream and pass that
-  // to the parser, because calling ReadSegments() recursively on the same
-  // stream is not supported.
-  nsCOMPtr<nsISupports> supportsStream;
-  rv = NS_NewByteInputStream(getter_AddRefs(supportsStream),fromRawSegment,count);
+    // We need to wrap the data in a new lightweight stream and pass that
+    // to the parser, because calling ReadSegments() recursively on the same
+    // stream is not supported.
+    nsCOMPtr<nsISupports> supportsStream;
+    rv = NS_NewByteInputStream(getter_AddRefs(supportsStream),fromRawSegment,count);
 
-  if (NS_SUCCEEDED(rv)) {
-    nsCOMPtr<nsIInputStream> copyStream(do_QueryInterface(supportsStream));
-    if (copyStream) {
-      rv = xmlHttpRequest->mXMLParserStreamListener->OnDataAvailable(xmlHttpRequest->mReadRequest,xmlHttpRequest->mContext,copyStream,toOffset,count);
-    } else {
-      NS_ERROR("NS_NewByteInputStream did not give out nsIInputStream!");
-      rv = NS_ERROR_UNEXPECTED;
+    if (NS_SUCCEEDED(rv)) {
+      nsCOMPtr<nsIInputStream> copyStream(do_QueryInterface(supportsStream));
+      if (copyStream) {
+        rv = xmlHttpRequest->mXMLParserStreamListener->OnDataAvailable(xmlHttpRequest->mReadRequest,xmlHttpRequest->mContext,copyStream,toOffset,count);
+      } else {
+        NS_ERROR("NS_NewByteInputStream did not give out nsIInputStream!");
+        rv = NS_ERROR_UNEXPECTED;
+      }
     }
   }
 
@@ -813,8 +834,31 @@ nsXMLHttpRequest::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
   mReadRequest = request;
   mContext = ctxt;
+  mParseResponseBody = PR_TRUE;
   ChangeState(XML_HTTP_REQUEST_LOADED);
-  return mXMLParserStreamListener->OnStartRequest(request,ctxt);
+  if (mOverrideMimeType.IsEmpty()) {
+    // If we are not overriding the mime type, we can gain a huge performance
+    // win by not even trying to parse non-XML data. This also protects us
+    // from the situation where we have an XML document and sink, but HTML (or other)
+    // parser, which can produce unreliable results.
+    nsCAutoString type;
+    mChannel->GetContentType(type);
+    nsACString::const_iterator start, end;
+    type.BeginReading(start);
+    type.EndReading(end);
+    if (!FindInReadable(NS_LITERAL_CSTRING("xml"), start, end)) {
+      mParseResponseBody = PR_FALSE;
+    }
+  } else {
+    nsresult status;
+    request->GetStatus(&status);
+    nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
+    if (channel && NS_SUCCEEDED(status)) {
+      channel->SetContentType(mOverrideMimeType);
+    }
+  }
+
+  return mParseResponseBody ? mXMLParserStreamListener->OnStartRequest(request,ctxt) : NS_OK;
 }
 
 /* void onStopRequest (in nsIRequest request, in nsISupports ctxt, in nsresult status, in wstring statusArg); */
@@ -824,7 +868,7 @@ nsXMLHttpRequest::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult
   nsCOMPtr<nsIParser> parser(do_QueryInterface(mXMLParserStreamListener));
   NS_ABORT_IF_FALSE(parser, "stream listener was expected to be a parser");
 
-  nsresult rv = mXMLParserStreamListener->OnStopRequest(request,ctxt,status);
+  nsresult rv = mParseResponseBody ? mXMLParserStreamListener->OnStopRequest(request,ctxt,status) : NS_OK;
   mXMLParserStreamListener = nsnull;
   mReadRequest = nsnull;
   mContext = nsnull;
@@ -964,14 +1008,14 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
   }
 
   // Ignore argument if method is GET, there is no point in trying to upload anything
-  nsXPIDLCString method;
+  nsCAutoString method;
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
 
   if (httpChannel) {
-    httpChannel->GetRequestMethod(getter_Copies(method)); // If GET, method name will be uppercase
+    httpChannel->GetRequestMethod(method); // If GET, method name will be uppercase
   }
 
-  if (aBody && httpChannel && nsCRT::strcmp("GET", method.get()) != 0) {
+  if (aBody && httpChannel && !method.Equals(NS_LITERAL_CSTRING("GET"))) {
     nsXPIDLString serial;
     nsCOMPtr<nsIInputStream> postDataStream;
 
@@ -980,43 +1024,56 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
     if (NS_FAILED(rv)) 
       return NS_ERROR_FAILURE;
 
-    if (dataType == nsIDataType::VTYPE_INTERFACE || 
-      dataType == nsIDataType::VTYPE_INTERFACE_IS) {
-      nsCOMPtr<nsISupports> supports;
-      nsID *iid;
-      rv = aBody->GetAsInterface(&iid, getter_AddRefs(supports));
-      if (NS_FAILED(rv)) 
-        return NS_ERROR_FAILURE;
-      if (iid) 
-        nsMemory::Free(iid);
-
-      // document?
-      nsCOMPtr<nsIDOMDocument> doc(do_QueryInterface(supports));
-      if (doc) {
-        nsCOMPtr<nsIDOMSerializer> serializer(do_CreateInstance(NS_XMLSERIALIZER_CONTRACTID, &rv));
-        if (NS_FAILED(rv)) return NS_ERROR_FAILURE;  
-      
-        rv = serializer->SerializeToString(doc, getter_Copies(serial));
+    switch (dataType) {
+    case nsIDataType::VTYPE_INTERFACE:
+    case nsIDataType::VTYPE_INTERFACE_IS:
+      {
+        nsCOMPtr<nsISupports> supports;
+        nsID *iid;
+        rv = aBody->GetAsInterface(&iid, getter_AddRefs(supports));
         if (NS_FAILED(rv)) 
           return NS_ERROR_FAILURE;
-      } else {
-        // nsISupportsWString?
-        nsCOMPtr<nsISupportsWString> wstr(do_QueryInterface(supports));
-        if (wstr) {
-          wstr->GetData(getter_Copies(serial));
+        if (iid) 
+          nsMemory::Free(iid);
+
+        // document?
+        nsCOMPtr<nsIDOMDocument> doc(do_QueryInterface(supports));
+        if (doc) {
+          nsCOMPtr<nsIDOMSerializer> serializer(do_CreateInstance(NS_XMLSERIALIZER_CONTRACTID, &rv));
+          if (NS_FAILED(rv)) return NS_ERROR_FAILURE;  
+      
+          rv = serializer->SerializeToString(doc, getter_Copies(serial));
+          if (NS_FAILED(rv)) 
+            return NS_ERROR_FAILURE;
         } else {
-          // stream?
-          nsCOMPtr<nsIInputStream> stream(do_QueryInterface(supports));
-          if (stream) {
-            postDataStream = stream;
+          // nsISupportsWString?
+          nsCOMPtr<nsISupportsWString> wstr(do_QueryInterface(supports));
+          if (wstr) {
+            wstr->GetData(getter_Copies(serial));
+          } else {
+            // stream?
+            nsCOMPtr<nsIInputStream> stream(do_QueryInterface(supports));
+            if (stream) {
+              postDataStream = stream;
+            }
           }
         }
       }
-    } else {
+      break;
+    case nsIDataType::VTYPE_VOID:
+    case nsIDataType::VTYPE_EMPTY:
+      // Makes us act as if !aBody, don't upload anything
+      break;
+    case nsIDataType::VTYPE_EMPTY_ARRAY:
+    case nsIDataType::VTYPE_ARRAY:
+      // IE6 throws error here, so we do that as well
+      return NS_ERROR_INVALID_ARG;
+    default:
       // try variant string
       rv = aBody->GetAsWString(getter_Copies(serial));    
       if (NS_FAILED(rv)) 
         return rv;
+      break;
     }
 
     if (serial) {
@@ -1033,6 +1090,10 @@ nsXMLHttpRequest::Send(nsIVariant *aBody)
       NS_ASSERTION(uploadChannel, "http must support nsIUploadChannel");
 
       rv = uploadChannel->SetUploadStream(postDataStream, nsnull, -1);
+      // Reset the method to its original value
+      if (httpChannel) {
+          httpChannel->SetRequestMethod(method);
+      }
     }
   }
 
@@ -1189,7 +1250,8 @@ nsXMLHttpRequest::SetRequestHeader(const char *header, const char *value)
   nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(mChannel));
 
   if (httpChannel)
-    return httpChannel->SetRequestHeader(header, value);
+    return httpChannel->SetRequestHeader(nsDependentCString(header),
+                                         nsDependentCString(value));
   
   return NS_OK;
 }
@@ -1214,6 +1276,15 @@ nsXMLHttpRequest::GetReadyState(PRInt32 *aState)
   return NS_OK;
 }
 
+/* void   overrideMimeType(in string mimetype); */
+NS_IMETHODIMP
+nsXMLHttpRequest::OverrideMimeType(const char* aMimeType)
+{
+  // XXX Should we do some validation here?
+  mOverrideMimeType.Assign(aMimeType);
+  return NS_OK;
+}
+
 // nsIDOMEventListener
 nsresult
 nsXMLHttpRequest::HandleEvent(nsIDOMEvent* aEvent)
@@ -1254,6 +1325,12 @@ nsXMLHttpRequest::Unload(nsIDOMEvent* aEvent)
 nsresult
 nsXMLHttpRequest::Abort(nsIDOMEvent* aEvent)
 {
+  if (mReadRequest) {
+    mReadRequest->Cancel(NS_BINDING_ABORTED);
+  }
+  if (mChannel) {
+    mChannel->Cancel(NS_BINDING_ABORTED);
+  }
   mDocument = nsnull;
   ChangeState(XML_HTTP_REQUEST_UNINITIALIZED);
 #ifdef IMPLEMENT_SYNC_LOAD
@@ -1355,7 +1432,7 @@ nsXMLHttpRequest::ChangeState(nsXMLHttpRequestState aState, PRBool aBroadcast)
 NS_IMPL_ISUPPORTS1(nsXMLHttpRequest::nsHeaderVisitor, nsIHttpHeaderVisitor)
 
 NS_IMETHODIMP nsXMLHttpRequest::
-nsHeaderVisitor::VisitHeader(const char *header, const char *value)
+nsHeaderVisitor::VisitHeader(const nsACString &header, const nsACString &value)
 {
     mHeaders.Append(header);
     mHeaders.Append(": ");
