@@ -111,6 +111,40 @@ typedef void * nsNativeDeviceContext;
 /* print: print while in print preview */
 #define NS_ERROR_GFX_PRINTER_PRINT_WHILE_PREVIEW          \
   NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+18)
+/* requested page size not supported by printer */
+#define NS_ERROR_GFX_PRINTER_PAPER_SIZE_NOT_SUPPORTED \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+19)
+/* requested page orientation not supported */
+#define NS_ERROR_GFX_PRINTER_ORIENTATION_NOT_SUPPORTED \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+20)
+/* requested colorspace not supported (like printing "color" on a "grayscale"-only printer) */
+#define NS_ERROR_GFX_PRINTER_COLORSPACE_NOT_SUPPORTED \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+21)
+/* too many copies requested */
+#define NS_ERROR_GFX_PRINTER_TOO_MANY_COPIES \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+22)
+/* driver configuration error */
+#define NS_ERROR_GFX_PRINTER_DRIVER_CONFIGURATION_ERROR \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+23)
+/* Xprint module specific: Xprt server broken */
+#define NS_ERROR_GFX_PRINTER_XPRINT_BROKEN_XPRT \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+24)
+/* The document is still being loaded, can't Print Preview */
+#define NS_ERROR_GFX_PRINTER_DOC_IS_BUSY_PP \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+25)
+/* The document was asked to be destroyed while we were preparing printing */
+#define NS_ERROR_GFX_PRINTER_DOC_WAS_DESTORYED \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+26)
+/* Cannot Print or Print Preview XUL Documents */
+#define NS_ERROR_GFX_PRINTER_NO_XUL \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+27)
+/* The toolkit no longer supports the Print Dialog (for embedders) */
+#define NS_ERROR_GFX_NO_PRINTDIALOG_IN_TOOLKIT \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+28)
+/* The was wasn't any Print Prompt service registered (this shouldn't happen) */
+#define NS_ERROR_GFX_NO_PRINTROMPTSERVICE \
+  NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_GFX,NS_ERROR_GFX_PRINTER_BASE+29)
+
 
 /**
  * Conts need for Print Preview
@@ -158,8 +192,7 @@ typedef void * nsPalette;
     eSystemFont_Field,
 
     eSystemFont_Tooltips,        // moz
-    eSystemFont_Widget,
-    eSystemFont_Theme
+    eSystemFont_Widget
   } nsSystemFontID;
 
 class nsIDeviceContext : public nsISupports
@@ -341,8 +374,9 @@ public:
   NS_IMETHOD  SetZoom(float aZoom) = 0;
   NS_IMETHOD  GetZoom(float &aZoom) const = 0;
 
-  //get and set the text zoom value used for display-time
-  //scaling. default is 1.0 (no zoom)
+  //get and set the text zoom value used for display-time scaling.
+  //default is 1.0 (no zoom).  The device context doesn't do anything
+  //with this value.  It's merely a convenient place to store it.
   NS_IMETHOD  SetTextZoom(float aTextZoom) = 0;
   NS_IMETHOD  GetTextZoom(float &aTextZoom) const = 0;
 
@@ -352,13 +386,6 @@ public:
   //in the device context for re-use.
   NS_IMETHOD  GetDrawingSurface(nsIRenderingContext &aContext, nsDrawingSurface &aSurface) = 0;
 
-  //functions for handling gamma correction of output device
-  NS_IMETHOD  GetGamma(float &aGamms) = 0;
-  NS_IMETHOD  SetGamma(float aGamma) = 0;
-
-  //XXX the return from this really needs to be ref counted somehow. MMP
-  NS_IMETHOD  GetGammaTable(PRUint8 *&aGammaTable) = 0;
-
   /**
    * Check to see if a particular named font exists.
    * @param aFontName character string of font face name
@@ -441,6 +468,17 @@ public:
   NS_IMETHOD GetDeviceContextFor(nsIDeviceContextSpec *aDevice,
                                  nsIDeviceContext *&aContext) = 0;
 
+  /**
+   * This is enables the DeviceContext to anything it needs to do for Printing
+   * before Reflow and BeginDocument is where work can be done after reflow.
+   * @param aTitle - itle of Document
+   * @param aPrintToFileName - name of file to print to, if NULL then don't print to file
+   *
+   * @return error status
+   */
+  NS_IMETHOD PrepareDocument(PRUnichar * aTitle, 
+                             PRUnichar*  aPrintToFileName) = 0;
+
   //XXX need to work out re-entrancy issues for these APIs... MMP
   /**
    * Inform the output device that output of a document is beginning
@@ -448,9 +486,18 @@ public:
    * EndDocument().
    * XXX needs to take parameters so that feedback can be given to the
    * app regarding pagination progress and aborting print operations?
+   *
+   * @param aTitle - itle of Document
+   * @param aPrintToFileName - name of file to print to, if NULL then don't print to file
+   * @param aStartPage - starting page number (must be greater than zero)
+   * @param aEndPage - ending page number     (must be less than or equal to number of pages)
+   *
    * @return error status
    */
-  NS_IMETHOD BeginDocument(PRUnichar * aTitle) = 0;
+  NS_IMETHOD BeginDocument(PRUnichar*  aTitle, 
+                           PRUnichar*  aPrintToFileName,
+                           PRInt32     aStartPage, 
+                           PRInt32     aEndPage) = 0;
 
   /**
    * Inform the output device that output of a document is ending.
@@ -460,6 +507,13 @@ public:
    */
   NS_IMETHOD EndDocument(void) = 0;
 
+  /**
+   * Inform the output device that output of a document is being aborted.
+   * Must be matched 1:1 with BeginDocument()
+   * @return error status
+   */
+  NS_IMETHOD AbortDocument(void) = 0;
+
   /**
    * Inform the output device that output of a page is beginning
    * Used for print related device contexts. Must be matched 1:1 with
