@@ -52,6 +52,7 @@ class nsTransform2D;
 class nsString;
 class nsIDeviceContext;
 class nsIRegion;
+class nsIAtom;
 
 struct nsFont;
 struct nsPoint;
@@ -287,7 +288,7 @@ public:
    * Sets the font for the RenderingContext
    * @param aFont The font to use in the RenderingContext
    */
-  NS_IMETHOD SetFont(const nsFont& aFont) = 0;
+  NS_IMETHOD SetFont(const nsFont& aFont, nsIAtom* aLangGroup) = 0;
 
   /**
    * Sets the font for the RenderingContext
@@ -411,6 +412,17 @@ public:
    */
   NS_IMETHOD InvertRect(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight) = 0;
 
+  /**
+   * For platforms (e.g., Cocoa) that implicitly double buffer, this call can be used
+   * to force a buffer flush following the painting of a rectangle.  This
+   * call needs to be used any time drawing of rects is being done "on the fly",
+   * outside of the normal painting process.
+   * Examples include the blinking caret and tabbing through subimages in an
+   * image map.
+   */
+  NS_IMETHOD FlushRect(const nsRect& aRect) = 0;
+  NS_IMETHOD FlushRect(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight) = 0;
+  
   /**
    * Draw a poly in the current foreground color
    * @param aPoints points to use for the drawing, last must equal first
@@ -778,6 +790,13 @@ public:
    */
   NS_IMETHOD DestroyCachedBackbuffer(void) = 0;
 
+  /**
+   * Some platforms may not want a backbuffer at all. Returning false
+   * here allows them to achieve that
+   *
+   * @param aUseBackbuffer PR_TRUE if we should use a backbuffer, PR_FALSE if not
+   */
+  NS_IMETHOD UseBackbuffer(PRBool* aUseBackbuffer) = 0;
 
 
 #ifdef MOZ_MATHML
@@ -981,6 +1000,10 @@ struct nsBoundingMetrics {
           to follow the current one. Depending on the font, this
           could be greater than or less than the right bearing. */
 
+  nsBoundingMetrics() {
+    Clear();
+  }
+
   //////////
   // Utility methods and operators:
 
