@@ -41,19 +41,9 @@
  
 #include "nsIPref.h" 
 #include "prenv.h" /* for PR_GetEnv */ 
-
-#include "nsIDOMWindow.h" 
-#include "nsIServiceManager.h" 
-#include "nsIDialogParamBlock.h" 
-#include "nsISupportsPrimitives.h"
-#include "nsIWindowWatcher.h"
-#include "nsIDOMWindowInternal.h"
-
+#include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
-#include "nsISupportsArray.h"
-
-//#include "prmem.h"
-//#include "plstr.h"
+#include "nsCRT.h"
 
 //----------------------------------------------------------------------------------
 // The printer data is shared between the PrinterEnumerator and the nsDeviceContextSpecG
@@ -105,7 +95,9 @@ nsDeviceContextSpecBeOS :: ~nsDeviceContextSpecBeOS()
 } 
  
 static NS_DEFINE_IID(kIDeviceContextSpecIID, NS_IDEVICE_CONTEXT_SPEC_IID); 
+#ifdef USE_POSTSCRIPT
 static NS_DEFINE_IID(kIDeviceContextSpecPSIID, NS_IDEVICE_CONTEXT_SPEC_PS_IID); 
+#endif /* USE_POSTSCRIPT */
  
 #if 0 
 NS_IMPL_ISUPPORTS1(nsDeviceContextSpecBeOS, nsIDeviceContextSpec)
@@ -124,6 +116,7 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: QueryInterface(REFNSIID aIID, void** aI
     return NS_OK; 
   } 
  
+#ifdef USE_POSTSCRIPT
   if (aIID.Equals(kIDeviceContextSpecPSIID)) 
   { 
     nsIDeviceContextSpecPS* tmp = this; 
@@ -131,6 +124,7 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: QueryInterface(REFNSIID aIID, void** aI
     NS_ADDREF_THIS(); 
     return NS_OK; 
   }
+#endif /* USE_POSTSCRIPT */
 
   static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 
@@ -149,76 +143,17 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: QueryInterface(REFNSIID aIID, void** aI
 NS_IMPL_ADDREF(nsDeviceContextSpecBeOS)
 NS_IMPL_RELEASE(nsDeviceContextSpecBeOS)
 
-
-/** -------------------------------------------------------
- */
-static nsresult DisplayXPDialog(nsIPrintSettings* aPS,
-                                const char* aChromeURL, 
-                                PRBool& aClickedOK)
-{
-  NS_ASSERTION(aPS, "Must have a print settings!");
-
-  aClickedOK = PR_FALSE;
-  nsresult rv = NS_ERROR_FAILURE;
-
-  // create a nsISupportsArray of the parameters 
-  // being passed to the window
-  nsCOMPtr<nsISupportsArray> array;
-  NS_NewISupportsArray(getter_AddRefs(array));
-  if (!array) return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIPrintSettings> ps = aPS;
-  nsCOMPtr<nsISupports> psSupports(do_QueryInterface(ps));
-  NS_ASSERTION(psSupports, "PrintSettings must be a supports");
-  array->AppendElement(psSupports);
-
-  nsCOMPtr<nsIDialogParamBlock> ioParamBlock(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
-  if (ioParamBlock) {
-    ioParamBlock->SetInt(0, 0);
-    nsCOMPtr<nsISupports> blkSupps(do_QueryInterface(ioParamBlock));
-    NS_ASSERTION(blkSupps, "IOBlk must be a supports");
-
-    array->AppendElement(blkSupps);
-    nsCOMPtr<nsISupports> arguments(do_QueryInterface(array));
-    NS_ASSERTION(array, "array must be a supports");
-
-    nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
-    if (wwatch) {
-      nsCOMPtr<nsIDOMWindow> active;
-      wwatch->GetActiveWindow(getter_AddRefs(active));    
-      nsCOMPtr<nsIDOMWindowInternal> parent = do_QueryInterface(active);
-
-      nsCOMPtr<nsIDOMWindow> newWindow;
-      rv = wwatch->OpenWindow(parent, aChromeURL,
-            "_blank", "chrome,modal,centerscreen", array,
-            getter_AddRefs(newWindow));
-    }
-  }
-
-  if (NS_SUCCEEDED(rv)) {
-    PRInt32 buttonPressed = 0;
-    ioParamBlock->GetInt(0, &buttonPressed);
-    if (buttonPressed == 1) {
-      aClickedOK = PR_TRUE;
-    } else {
-      rv = NS_ERROR_ABORT;
-    }
-  } else {
-    rv = NS_ERROR_ABORT;
-  }
-  return rv;
-}
-
-
 /** -------------------------------------------------------
  *  Initialize the nsDeviceContextSpecBeOS
  *  @update   dc 2/15/98
  *  @update   syd 3/2/99
  */
-NS_IMETHODIMP nsDeviceContextSpecBeOS::Init(nsIPrintSettings* aPS, PRBool aQuiet)
+NS_IMETHODIMP nsDeviceContextSpecBeOS::Init(nsIPrintSettings* aPS)
 {
   nsresult rv = NS_ERROR_FAILURE;
   NS_ASSERTION(nsnull != aPS, "No print settings.");
+
+  mPrintSettings = aPS;
   
   // if there is a current selection then enable the "Selection" radio button
   if (aPS != nsnull) {
@@ -254,99 +189,88 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS::Init(nsIPrintSettings* aPS, PRBool aQuiet
     return rv;
   }
 
-  if (!aQuiet ) {
-    rv = DisplayXPDialog(aPS, 
-                         "chrome://global/content/printdialog.xul", canPrint);    
-  }
-  else {
-    canPrint = PR_TRUE;
-  }
   
   GlobalPrinters::GetInstance()->FreeGlobalPrinters();
 
-  if (canPrint) {
-    if (aPS != nsnull) {
-      aPS->GetPrinterName(&printer);
-      aPS->GetPrintReversed(&reversed);
-      aPS->GetPrintInColor(&color);
-      aPS->GetPaperSize(&paper_size);
-      aPS->GetOrientation(&orientation);
-      aPS->GetPrintCommand(&command);
-      aPS->GetPrintRange(&printRange);
-      aPS->GetToFileName(&printfile);
-      aPS->GetPrintToFile(&tofile);
-      aPS->GetStartPageRange(&fromPage);
-      aPS->GetEndPageRange(&toPage);
-      aPS->GetNumCopies(&copies);
-      aPS->GetMarginTop(&dtop);
-      aPS->GetMarginLeft(&dleft);
-      aPS->GetMarginBottom(&dbottom);
-      aPS->GetMarginRight(&dright);
+  if (aPS != nsnull) {
+    aPS->GetPrinterName(&printer);
+    aPS->GetPrintReversed(&reversed);
+    aPS->GetPrintInColor(&color);
+    aPS->GetPaperSize(&paper_size);
+    aPS->GetOrientation(&orientation);
+    aPS->GetPrintCommand(&command);
+    aPS->GetPrintRange(&printRange);
+    aPS->GetToFileName(&printfile);
+    aPS->GetPrintToFile(&tofile);
+    aPS->GetStartPageRange(&fromPage);
+    aPS->GetEndPageRange(&toPage);
+    aPS->GetNumCopies(&copies);
+    aPS->GetMarginTop(&dtop);
+    aPS->GetMarginLeft(&dleft);
+    aPS->GetMarginBottom(&dbottom);
+    aPS->GetMarginRight(&dright);
 
-      if (command != nsnull && printfile != nsnull) {
-        // ToDo: Use LocalEncoding instead of UTF-8 (see bug 73446)
-        strcpy(mPrData.command, NS_ConvertUCS2toUTF8(command).get());  
-        strcpy(mPrData.path,    NS_ConvertUCS2toUTF8(printfile).get());
-      }
-      if (printer != nsnull) 
-        strcpy(mPrData.printer, NS_ConvertUCS2toUTF8(printer).get());        
+    if (command != nsnull && printfile != nsnull) {
+      // ToDo: Use LocalEncoding instead of UTF-8 (see bug 73446)
+      strcpy(mPrData.command, NS_ConvertUCS2toUTF8(command).get());  
+      strcpy(mPrData.path,    NS_ConvertUCS2toUTF8(printfile).get());
+    }
+    if (printer != nsnull) 
+      strcpy(mPrData.printer, NS_ConvertUCS2toUTF8(printer).get());        
 #ifdef DEBUG_rods
-      printf("margins:       %5.2f,%5.2f,%5.2f,%5.2f\n", dtop, dleft, dbottom, dright);
-      printf("printRange     %d\n", printRange);
-      printf("fromPage       %d\n", fromPage);
-      printf("toPage         %d\n", toPage);
+    printf("margins:       %5.2f,%5.2f,%5.2f,%5.2f\n", dtop, dleft, dbottom, dright);
+    printf("printRange     %d\n", printRange);
+    printf("fromPage       %d\n", fromPage);
+    printf("toPage         %d\n", toPage);
 #endif /* DEBUG_rods */
-    } else {
+  } else {
 #ifdef VMS
-      // Note to whoever puts the "lpr" into the prefs file. Please contact me
-      // as I need to make the default be "print" instead of "lpr" for OpenVMS.
-      strcpy(mPrData.command, "print");
+    // Note to whoever puts the "lpr" into the prefs file. Please contact me
+    // as I need to make the default be "print" instead of "lpr" for OpenVMS.
+    strcpy(mPrData.command, "print");
 #else
-      strcpy(mPrData.command, "lpr ${MOZ_PRINTER_NAME:+'-P'}${MOZ_PRINTER_NAME}");
+    strcpy(mPrData.command, "lpr ${MOZ_PRINTER_NAME:+'-P'}${MOZ_PRINTER_NAME}");
 #endif /* VMS */
-    }
+  }
 
-    mPrData.top       = dtop;
-    mPrData.bottom    = dbottom;
-    mPrData.left      = dleft;
-    mPrData.right     = dright;
-    mPrData.fpf       = !reversed;
-    mPrData.grayscale = !color;
-    mPrData.size      = paper_size;
-    mPrData.orientation = orientation;
-    mPrData.toPrinter = !tofile;
-    mPrData.copies = copies;
+  mPrData.top       = dtop;
+  mPrData.bottom    = dbottom;
+  mPrData.left      = dleft;
+  mPrData.right     = dright;
+  mPrData.fpf       = !reversed;
+  mPrData.grayscale = !color;
+  mPrData.size      = paper_size;
+  mPrData.orientation = orientation;
+  mPrData.toPrinter = !tofile;
+  mPrData.copies = copies;
 
-    // PWD, HOME, or fail 
-    
-    if (!printfile) {
-      if ( ( path = PR_GetEnv( "PWD" ) ) == (char *) nsnull ) 
-        if ( ( path = PR_GetEnv( "HOME" ) ) == (char *) nsnull )
-          strcpy(mPrData.path, "mozilla.ps");
-          
-      if ( path != (char *) nsnull )
-        sprintf(mPrData.path, "%s/mozilla.ps", path);
-      else
-        return NS_ERROR_FAILURE;
-    }
-    
+  // PWD, HOME, or fail 
+  
+  if (!printfile) {
+    if ( ( path = PR_GetEnv( "PWD" ) ) == (char *) nsnull ) 
+      if ( ( path = PR_GetEnv( "HOME" ) ) == (char *) nsnull )
+        strcpy(mPrData.path, "mozilla.ps");
+        
+    if ( path != (char *) nsnull )
+      sprintf(mPrData.path, "%s/mozilla.ps", path);
+    else
+      return NS_ERROR_FAILURE;
+  }
+  
 #ifdef NOT_IMPLEMENTED_YET
-    if (mGlobalNumPrinters) {
-       for(int i = 0; (i < mGlobalNumPrinters) && !mQueue; i++) {
-          if (!(mGlobalPrinterList->StringAt(i)->CompareWithConversion(mPrData.printer, TRUE, -1)))
-             mQueue = PrnDlg.SetPrinterQueue(i);
-       }
-    }
+  if (mGlobalNumPrinters) {
+     for(int i = 0; (i < mGlobalNumPrinters) && !mQueue; i++) {
+        if (!(mGlobalPrinterList->StringAt(i)->CompareWithConversion(mPrData.printer, TRUE, -1)))
+           mQueue = PrnDlg.SetPrinterQueue(i);
+     }
+  }
 #endif /* NOT_IMPLEMENTED_YET */
-    
-    if (command != nsnull) {
-      nsMemory::Free(command);
-    }
-    if (printfile != nsnull) {
-      nsMemory::Free(printfile);
-    }
-
-    return NS_OK;
+  
+  if (command != nsnull) {
+    nsMemory::Free(command);
+  }
+  if (printfile != nsnull) {
+    nsMemory::Free(printfile);
   }
 
   return rv;
@@ -358,7 +282,7 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: GetToPrinter( PRBool &aToPrinter )
         return NS_OK;
 } 
 
-NS_IMETHODIMP nsDeviceContextSpecBeOS::GetPrinterName ( char **aPrinter )
+NS_IMETHODIMP nsDeviceContextSpecBeOS::GetPrinterName ( const char **aPrinter )
 {
    *aPrinter = &mPrData.printer[0];
    return NS_OK;
@@ -449,13 +373,13 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: GetLeftMargin ( float &value )
   return NS_OK; 
 } 
  
-NS_IMETHODIMP nsDeviceContextSpecBeOS :: GetCommand ( char **aCommand )      
+NS_IMETHODIMP nsDeviceContextSpecBeOS::GetCommand ( const char **aCommand )      
 { 
   *aCommand = &mPrData.command[0]; 
   return NS_OK; 
 } 
  
-NS_IMETHODIMP nsDeviceContextSpecBeOS :: GetPath ( char **aPath )      
+NS_IMETHODIMP nsDeviceContextSpecBeOS::GetPath ( const char **aPath )      
 { 
   *aPath = &mPrData.path[0]; 
   return NS_OK; 
@@ -476,8 +400,10 @@ NS_IMETHODIMP nsDeviceContextSpecBeOS :: ClosePrintManager()
   return NS_OK;
 }  
 
-
-
+NS_IMETHODIMP nsDeviceContextSpecBeOS::GetPageSizeInTwips(PRInt32 *aWidth, PRInt32 *aHeight)
+{
+  return mPrintSettings->GetPageSizeInTwips(aWidth, aHeight);
+}
 
 //  Printer Enumerator
 nsPrinterEnumeratorBeOS::nsPrinterEnumeratorBeOS()
@@ -487,15 +413,6 @@ nsPrinterEnumeratorBeOS::nsPrinterEnumeratorBeOS()
 
 NS_IMPL_ISUPPORTS1(nsPrinterEnumeratorBeOS, nsIPrinterEnumerator)
 
-NS_IMETHODIMP nsPrinterEnumeratorBeOS::EnumeratePrintersExtended(PRUint32* aCount, PRUnichar*** aResult)
-{
-  NS_ENSURE_ARG(aCount);
-  NS_ENSURE_ARG_POINTER(aResult);
-  *aCount = 0;
-  *aResult = nsnull;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsPrinterEnumeratorBeOS::EnumeratePrinters(PRUint32* aCount, PRUnichar*** aResult)
 {
   NS_ENSURE_ARG(aCount);
@@ -549,21 +466,23 @@ NS_IMETHODIMP nsPrinterEnumeratorBeOS::EnumeratePrinters(PRUint32* aCount, PRUni
   return NS_OK;
 }
 
+/* readonly attribute wstring defaultPrinterName; */
+NS_IMETHODIMP nsPrinterEnumeratorBeOS::GetDefaultPrinterName(PRUnichar * *aDefaultPrinterName)
+{
+  NS_ENSURE_ARG_POINTER(aDefaultPrinterName);
+  *aDefaultPrinterName = nsnull;
+  return NS_OK;
+}
+
+/* void initPrintSettingsFromPrinter (in wstring aPrinterName, in nsIPrintSettings aPrintSettings); */
+NS_IMETHODIMP nsPrinterEnumeratorBeOS::InitPrintSettingsFromPrinter(const PRUnichar *aPrinterName, nsIPrintSettings *aPrintSettings)
+{
+    return NS_OK;
+}
+
 NS_IMETHODIMP nsPrinterEnumeratorBeOS::DisplayPropertiesDlg(const PRUnichar *aPrinter, nsIPrintSettings *aPrintSettings)
 {
-  /* fixme: We simply ignore the |aPrinter| argument here
-   * We should get the supported printer attributes from the printer and 
-   * populate the print job options dialog with these data instead of using 
-   * the "default set" here.
-   * However, this requires changes on all platforms and is another big chunk
-   * of patches ... ;-(
-   */
-
-  PRBool pressedOK;
-  return DisplayXPDialog(aPrintSettings,
-                         "chrome://global/content/printjoboptions.xul", 
-                         pressedOK);
-
+  return NS_OK;
 }
 
 //----------------------------------------------------------------------
@@ -577,11 +496,13 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
   mGlobalPrinterList = new nsStringArray();
   if (!mGlobalPrinterList) 
     return NS_ERROR_OUT_OF_MEMORY;
-      
+
+#ifdef USE_POSTSCRIPT      
   /* add an entry for the default printer (see nsPostScriptObj.cpp) */
   mGlobalPrinterList->AppendString(
     nsString(NS_ConvertASCIItoUCS2(NS_POSTSCRIPT_DRIVER_NAME "default")));
   mGlobalNumPrinters++;
+#endif /* USE_POSTSCRIPT */
 
   /* get the list of printers */
   char *printerList = nsnull;
@@ -605,7 +526,8 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
     printerList = strdup(printerList);
     if (!printerList)
       return NS_ERROR_OUT_OF_MEMORY;    
-    
+
+#ifdef USE_POSTSCRIPT    
     for( name = PL_strtok_r(printerList, " ", &tok_lasts) ; 
          name != nsnull ; 
          name = PL_strtok_r(nsnull, " ", &tok_lasts) )
@@ -615,7 +537,8 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
         nsString(NS_ConvertASCIItoUCS2(name)));
       mGlobalNumPrinters++;      
     }
-    
+#endif /* USE_POSTSCRIPT */
+
     free(printerList);
   }
       
