@@ -49,8 +49,10 @@
 #include "nsFontMetricsGTK.h"
 #include "nsGfxCIID.h"
 
+#ifdef USE_POSTSCRIPT
 #include "nsGfxPSCID.h"
 #include "nsIDeviceContextPS.h"
+#endif /* USE_POSTSCRIPT */
 #ifdef USE_XPRINT
 #include "nsGfxXPrintCID.h"
 #include "nsIDeviceContextXPrint.h"
@@ -59,6 +61,10 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 
+#ifdef MOZ_WIDGET_GTK
+#include "gdksuperwin.h"
+#endif /* MOZ_WIDGET_GTK */
+
 #include <X11/Xatom.h>
 
 #include "nsDeviceContextSpecG.h"
@@ -72,7 +78,11 @@ static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
 #define GDK_DEFAULT_FONT1 "-*-helvetica-medium-r-*--*-120-*-*-*-*-iso8859-1"
 #define GDK_DEFAULT_FONT2 "-*-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"
+
+#ifdef MOZ_WIDGET_GTK
+// this is specific to gtk 1.2
 extern GdkFont *default_font;
+#endif /* MOZ_WIDGET_GTK */
 
 /**
  * A singleton instance of nsSystemFontsGTK is created by the first
@@ -89,7 +99,7 @@ class nsSystemFontsGTK {
     const nsFont& GetButtonFont() { return mButtonFont; }
 
   private:
-    nsresult GetSystemFontInfo(GdkFont* iFont, nsFont* aFont,
+    nsresult GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont,
                                float aPixelsToTwips) const;
 
     /*
@@ -165,6 +175,8 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
     return NS_ERROR_FAILURE;
   }
 
+#ifdef MOZ_WIDGET_GTK
+
   if (aNativeWidget) {
     // superwin?
     if (GDK_IS_SUPERWIN(aNativeWidget)) {
@@ -180,6 +192,20 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
     }
   }
 
+#endif /* MOZ_WIDGET_GTK */
+
+#ifdef MOZ_WIDGET_GTK2
+
+  if (aNativeWidget) {
+    // can only be a gdk window
+    if (GDK_IS_WINDOW(aNativeWidget))
+      mDeviceWindow = GDK_WINDOW(aNativeWidget);
+    else 
+      NS_WARNING("unsupported native widget type!");
+  }
+
+#endif
+
   nsCOMPtr<nsIScreen> screen;
   mScreenManager->GetPrimaryScreen ( getter_AddRefs(screen) );
   if ( screen ) {
@@ -223,6 +249,10 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
   sb = gtk_vscrollbar_new(NULL);
   gtk_widget_ref(sb);
   gtk_object_sink(GTK_OBJECT(sb));
+#ifdef MOZ_WIDGET_GTK2
+  gtk_widget_ensure_style(sb);
+  gtk_widget_queue_resize(sb);
+#endif /* MOZ_WIDGET_GTK2 */
   gtk_widget_size_request(sb,&req);
   mScrollbarWidth = req.width;
   gtk_widget_destroy(sb);
@@ -231,6 +261,10 @@ NS_IMETHODIMP nsDeviceContextGTK::Init(nsNativeWidget aNativeWidget)
   sb = gtk_hscrollbar_new(NULL);
   gtk_widget_ref(sb);
   gtk_object_sink(GTK_OBJECT(sb));
+#ifdef MOZ_WIDGET_GTK2
+  gtk_widget_ensure_style(sb);
+  gtk_widget_queue_resize(sb);
+#endif /* MOZ_WIDGET_GTK2 */
   gtk_widget_size_request(sb,&req);
   mScrollbarHeight = req.height;
   gtk_widget_destroy(sb);
@@ -330,9 +364,11 @@ NS_IMETHODIMP nsDeviceContextGTK::SupportsNativeWidgets(PRBool &aSupportsWidgets
 
 NS_IMETHODIMP nsDeviceContextGTK::GetScrollBarDimensions(float &aWidth, float &aHeight) const
 {
-  aWidth = mScrollbarWidth * mPixelsToTwips;
-  aHeight = mScrollbarHeight * mPixelsToTwips;
- 
+  float scale;
+  GetCanonicalPixelScale(scale);
+  aWidth = mScrollbarWidth * mPixelsToTwips * scale;
+  aHeight = mScrollbarHeight * mPixelsToTwips * scale;
+
   return NS_OK;
 }
 
@@ -397,7 +433,7 @@ NS_IMETHODIMP nsDeviceContextGTK::ConvertPixel(nscolor aColor,
 
 NS_IMETHODIMP nsDeviceContextGTK::CheckFontExistence(const nsString& aFontName)
 {
-  return nsFontMetricsGTK::FamilyExists(aFontName);
+  return nsFontMetricsGTK::FamilyExists(this, aFontName);
 }
 
 NS_IMETHODIMP nsDeviceContextGTK::GetDeviceSurfaceDimensions(PRInt32 &aWidth, PRInt32 &aHeight)
@@ -496,6 +532,7 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceContextFor(nsIDeviceContextSpec *aDev
   }
   else
 #endif /* USE_XPRINT */
+#ifdef USE_POSTSCRIPT
   if (method == pmPostScript) { // PostScript
     // default/PS
     static NS_DEFINE_CID(kCDeviceContextPS, NS_DEVICECONTEXTPS_CID);
@@ -519,12 +556,13 @@ NS_IMETHODIMP nsDeviceContextGTK::GetDeviceContextFor(nsIDeviceContextSpec *aDev
                               (void **)&aContext);
     return rv;
   }
+#endif /* USE_POSTSCRIPT */
   
   NS_WARNING("no print module created.");
   return NS_ERROR_UNEXPECTED;
 }
 
-NS_IMETHODIMP nsDeviceContextGTK::BeginDocument(PRUnichar * aTitle)
+NS_IMETHODIMP nsDeviceContextGTK::BeginDocument(PRUnichar * aTitle, PRUnichar* aPrintToFileName, PRInt32 aStartPage, PRInt32 aEndPage)
 {
   return NS_OK;
 }
@@ -534,6 +572,11 @@ NS_IMETHODIMP nsDeviceContextGTK::EndDocument(void)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsDeviceContextGTK::AbortDocument(void)
+{
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDeviceContextGTK::BeginPage(void)
 {
   return NS_OK;
@@ -639,7 +682,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(label);
 
   GtkStyle *style = gtk_widget_get_style(label);
-  GetSystemFontInfo(style->font, &mDefaultFont, aPixelsToTwips);
+  GetSystemFontInfo(style, &mDefaultFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -655,7 +698,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(entry);
 
   style = gtk_widget_get_style(entry);
-  GetSystemFontInfo(style->font, &mFieldFont, aPixelsToTwips);
+  GetSystemFontInfo(style, &mFieldFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -663,6 +706,8 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   GtkWidget *accel_label = gtk_accel_label_new("M");
   GtkWidget *menuitem = gtk_menu_item_new();
   GtkWidget *menu = gtk_menu_new();
+  gtk_object_ref(GTK_OBJECT(menu));
+  gtk_object_sink(GTK_OBJECT(menu));
 
   gtk_container_add(GTK_CONTAINER(menuitem), accel_label);
   gtk_menu_append(GTK_MENU(menu), menuitem);
@@ -673,7 +718,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(accel_label);
 
   style = gtk_widget_get_style(accel_label);
-  GetSystemFontInfo(style->font, &mMenuFont, aPixelsToTwips);
+  GetSystemFontInfo(style, &mMenuFont, aPixelsToTwips);
 
   gtk_widget_unref(menu);
 
@@ -694,7 +739,7 @@ nsSystemFontsGTK::nsSystemFontsGTK(float aPixelsToTwips)
   gtk_widget_realize(label);
 
   style = gtk_widget_get_style(label);
-  GetSystemFontInfo(style->font, &mButtonFont, aPixelsToTwips);
+  GetSystemFontInfo(style, &mButtonFont, aPixelsToTwips);
 
   gtk_widget_destroy(window);  // no unref, windows are different
 
@@ -721,10 +766,67 @@ ListFontProps(XFontStruct *aFont, Display *aDisplay)
 }
 #endif
 
+#define LOCATE_MINUS(pos, str)  { \
+   pos = str.FindChar('-'); \
+   if (pos < 0) \
+     return ; \
+  }
+#define NEXT_MINUS(pos, str) { \
+   pos = str.FindChar('-', pos+1); \
+   if (pos < 0) \
+     return ; \
+  }  
+ 
+static void
+AppendFontFFREName(nsString& aString, const char* aXLFDName)
+{
+  // convert fontname from XFLD to FFRE and append, ie. from
+  // -adobe-courier-medium-o-normal--14-140-75-75-m-90-iso8859-15
+  // to
+  // adobe-courier-iso8859-15
+  nsCAutoString nameStr(aXLFDName);
+  PRInt32 pos1, pos2;
+  // remove first '-' and everything before it. 
+  LOCATE_MINUS(pos1, nameStr);
+  nameStr.Cut(0, pos1+1);
+
+  // skip foundry and family name
+  LOCATE_MINUS(pos1, nameStr);
+  NEXT_MINUS(pos1, nameStr);
+  pos2 = pos1;
+
+  // find '-' just before charset registry
+  for (PRInt32 i=0; i < 10; i++) {
+    NEXT_MINUS(pos2, nameStr);
+  }
+
+  // remove everything in between
+  nameStr.Cut(pos1, pos2-pos1);
+
+  aString.AppendWithConversion(nameStr.get());
+}
+
 static void
 AppendFontName(XFontStruct* aFontStruct, nsString& aString, Display *aDisplay)
 {
   unsigned long pr = 0;
+  // we first append the FFRE name to reconstruct font more faithfully
+#ifdef MOZ_WIDGET_GTK
+  unsigned long font_atom = gdk_atom_intern("FONT", FALSE);
+#endif
+#ifdef MOZ_WIDGET_GTK2
+  unsigned long font_atom = gdk_x11_get_xatom_by_name("FONT");
+#endif
+  ::XGetFontProperty(aFontStruct, font_atom, &pr);
+  if(pr) {
+    char* xlfdName = ::XGetAtomName(aDisplay, pr);
+    AppendFontFFREName(aString, xlfdName);
+    ::XFree(xlfdName);
+  }
+ 
+  aString.Append(PRUnichar(','));
+
+  // next, we need to append family name to cover more encodings.
   ::XGetFontProperty(aFontStruct, XA_FAMILY_NAME, &pr);
   if (!pr)
     ::XGetFontProperty(aFontStruct, XA_FULL_NAME, &pr);
@@ -774,9 +876,15 @@ GetFontSize(XFontStruct *aFontStruct, float aPixelsToTwips)
 }
 
 nsresult
-nsSystemFontsGTK::GetSystemFontInfo(GdkFont* iFont, nsFont* aFont, float aPixelsToTwips) const
+nsSystemFontsGTK::GetSystemFontInfo(GtkStyle *aStyle, nsFont* aFont, float aPixelsToTwips) const
 {
-  GdkFont *theFont = iFont;
+#ifdef MOZ_WIDGET_GTK
+  GdkFont *theFont = aStyle->font;
+#endif /* MOZ_WIDGET_GTK */
+
+#ifdef MOZ_WIDGET_GTK2
+  GdkFont *theFont = gtk_style_get_font(aStyle);
+#endif /* MOZ_WIDGET_GTK2 */
 
   aFont->style       = NS_FONT_STYLE_NORMAL;
   aFont->weight      = NS_FONT_WEIGHT_NORMAL;
@@ -785,9 +893,11 @@ nsSystemFontsGTK::GetSystemFontInfo(GdkFont* iFont, nsFont* aFont, float aPixels
   // do we have the default_font defined by GTK/GDK then
   // we use it, if not then we load helvetica, if not then
   // we load fixed font else we error out.
+#ifdef MOZ_WIDGET_GTK
   if (!theFont)
     theFont = default_font; // GTK default font
-  
+#endif
+
   if (!theFont)
     theFont = ::gdk_font_load( GDK_DEFAULT_FONT1 );
   
@@ -824,12 +934,13 @@ nsSystemFontsGTK::GetSystemFontInfo(GdkFont* iFont, nsFont* aFont, float aPixels
     nsString& fontName = aFont->name;
     fontName.Truncate();
     for (;;) {
-      AppendFontName(*fontStructs, fontName, fontDisplay);
-      ++fontStructs;
+      // we need to append FFRE name instead of family name in this case
+      AppendFontFFREName(fontName, *fontNames);
+      ++fontNames;
       --numFonts;
       if (numFonts == 0)
         break;
-      fontName.Append(NS_LITERAL_STRING(","));
+      fontName.Append(PRUnichar(','));
     }
   }
   return NS_OK;
