@@ -53,29 +53,42 @@
 
 #include <gdk/gdkprivate.h>
 
-#undef DEBUG_NATIVE_THEME
-
 NS_IMPL_ISUPPORTS1(nsNativeThemeGTK, nsITheme)
 
-GtkWidget* gButtonWidget = nsnull;
-GtkWidget* gCheckboxWidget = nsnull;
-GtkWidget* gScrollbarWidget = nsnull;
-GtkWidget* gGripperWidget = nsnull;
+GtkWidget* gButtonWidget;
+GtkWidget* gCheckboxWidget;
+GtkWidget* gScrollbarWidget;
+GtkWidget* gGripperWidget;
+GtkWidget* gEntryWidget;
+GtkWidget* gDropdownButonWidget;
+GtkWidget* gArrowWidget;
+GtkWidget* gDropdownButtonWidget;
+GtkWidget* gHandleBoxWidget;
+GtkWidget* gFrameWidget;
+GtkWidget* gProtoWindow;
+GtkWidget* gProgressWidget;
+GtkWidget* gTabWidget;
+GtkTooltips* gTooltipWidget;
 
 nsNativeThemeGTK::nsNativeThemeGTK()
-  : mProtoWindow(nsnull),
-    mProtoLayout(nsnull)
+  : mProtoLayout(nsnull)
 {
   NS_INIT_ISUPPORTS();
-  mDisabledAtom = getter_AddRefs(NS_NewAtom("disabled"));
-  mCheckedAtom = getter_AddRefs(NS_NewAtom("checked"));
-  mSelectedAtom = getter_AddRefs(NS_NewAtom("selected"));
+  mDisabledAtom = do_GetAtom("disabled");
+  mCheckedAtom = do_GetAtom("checked");
+  mSelectedAtom = do_GetAtom("selected");
+  mInputCheckedAtom = do_GetAtom("_moz-input-checked");
+  mInputAtom = do_GetAtom("input");
+  mFocusedAtom = do_GetAtom("focused");
+  mFirstTabAtom = do_GetAtom("first-tab");
 }
 
 nsNativeThemeGTK::~nsNativeThemeGTK() {
   // This will destroy all of our widgets
-  if (mProtoWindow)
-    gtk_widget_destroy(mProtoWindow);
+  if (gProtoWindow)
+    gtk_widget_destroy(gProtoWindow);
+  if (gTooltipWidget)
+    gtk_object_unref(GTK_OBJECT(gTooltipWidget));
 }
 
 static void GetPrimaryPresShell(nsIFrame* aFrame, nsIPresShell** aResult)
@@ -114,13 +127,6 @@ static PRInt32 GetContentState(nsIFrame* aFrame)
   return flags;
 }
 
-static PRBool HasAttrValue(nsIContent* aContent, nsIAtom* aAtom, const char* aStr)
-{
-  nsAutoString attr;
-  aContent->GetAttr(kNameSpaceID_None, aAtom, attr);
-  return attr.EqualsIgnoreCase(aStr);
-}
-
 static PRBool CheckBooleanAttr(nsIFrame* aFrame, nsIAtom* aAtom)
 {
   if (!aFrame)
@@ -129,7 +135,8 @@ static PRBool CheckBooleanAttr(nsIFrame* aFrame, nsIAtom* aAtom)
   aFrame->GetContent(getter_AddRefs(content));
   nsAutoString attr;
   nsresult res = content->GetAttr(kNameSpaceID_None, aAtom, attr);
-  if (res == NS_CONTENT_ATTR_NO_VALUE)
+  if (res == NS_CONTENT_ATTR_NO_VALUE ||
+      (res != NS_CONTENT_ATTR_NOT_THERE && attr.IsEmpty()))
     return PR_TRUE; // This handles the HTML case (an attr with no value is like a true val)
   return attr.EqualsIgnoreCase("true"); // This handles the XUL case.
 }
@@ -139,27 +146,6 @@ PRBool nsNativeThemeGTK::IsDisabled(nsIFrame* aFrame)
   return CheckBooleanAttr(aFrame, mDisabledAtom);
 }
   
-PRBool nsNativeThemeGTK::IsChecked(nsIFrame* aFrame)
-{
-  if (!aFrame)
-    return NS_OK;
-  nsCOMPtr<nsIContent> content;
-  aFrame->GetContent(getter_AddRefs(content));
-  nsAutoString checked;
-  nsresult res = content->GetAttr(kNameSpaceID_None, mCheckedAtom, checked);
-  if (res == NS_CONTENT_ATTR_NO_VALUE)
-    return PR_TRUE; // XXXdwh Can the HTML form control's checked property differ
-                    // from the checked attribute?  If so, will need an IsContentofType
-                    // HTML followed by a QI to nsIDOMHTMLInputElement to grab the prop.
-  return checked.EqualsIgnoreCase("true"); // This handles the XUL case.
-}
-
-PRBool nsNativeThemeGTK::IsSelected(nsIFrame* aFrame)
-{
-  return CheckBooleanAttr(aFrame, mSelectedAtom);
-}
-
-
 nsresult
 GetSystemColor(PRUint8 aWidgetType, nsILookAndFeel::nsColorID& aColorID)
 {
@@ -167,10 +153,8 @@ GetSystemColor(PRUint8 aWidgetType, nsILookAndFeel::nsColorID& aColorID)
   case NS_THEME_BUTTON:
   case NS_THEME_TOOLBAR_BUTTON:
   case NS_THEME_TAB:
-    {
-      aColorID = nsILookAndFeel::eColor_buttontext;
-      return NS_OK;
-    }
+    aColorID = nsILookAndFeel::eColor_buttontext;
+    return NS_OK;
   }
   return NS_ERROR_FAILURE;
 }
@@ -182,16 +166,15 @@ GetSystemFont(PRUint8 aWidgetType, nsSystemFontID& aFont)
   case NS_THEME_BUTTON:
   case NS_THEME_TOOLBAR_BUTTON:
   case NS_THEME_TAB:
-    {
       aFont = eSystemFont_Button;
       return NS_OK;
-    }
   }
   return NS_ERROR_FAILURE;
 }
 
 void
-nsNativeThemeGTK::GetGtkWidgetState(nsIFrame* aFrame, GtkWidgetState* aState)
+nsNativeThemeGTK::GetGtkWidgetState(PRUint8 aWidgetType,
+                                    nsIFrame* aFrame, GtkWidgetState* aState)
 {
   if (!aFrame) {
     aState->active = PR_FALSE;
@@ -203,7 +186,11 @@ nsNativeThemeGTK::GetGtkWidgetState(nsIFrame* aFrame, GtkWidgetState* aState)
   } else {
     PRInt32 eventState = GetContentState(aFrame);
     aState->active = (eventState & NS_EVENT_STATE_ACTIVE);
-    aState->focused = (eventState & NS_EVENT_STATE_FOCUS);
+    if (aWidgetType == NS_THEME_TEXTFIELD ||
+        aWidgetType == NS_THEME_RADIO_CONTAINER)
+      aState->focused = CheckBooleanAttr(aFrame, mFocusedAtom);
+    else
+      aState->focused = (eventState & NS_EVENT_STATE_FOCUS);
     aState->inHover = (eventState & NS_EVENT_STATE_HOVER);
     aState->disabled = IsDisabled(aFrame);
     aState->isDefault = PR_FALSE; // XXX fix me
@@ -231,18 +218,18 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
   transformMatrix->TransformCoord(&cr.x, &cr.y, &cr.width, &cr.height);
   GdkRectangle gdk_clip = {cr.x, cr.y, cr.width, cr.height};
 
+  GtkWidgetState state;
+  GetGtkWidgetState(aWidgetType, aFrame, &state);
+
   switch (aWidgetType) {
     
   case NS_THEME_BUTTON:
   case NS_THEME_TOOLBAR_BUTTON:
     {
       EnsureButtonWidget();
-
-      GtkWidgetState buttonState;
-      GetGtkWidgetState(aFrame, &buttonState);
       GtkReliefStyle relief = (aWidgetType == NS_THEME_BUTTON) ? GTK_RELIEF_NORMAL : GTK_RELIEF_NONE;
-
-      moz_gtk_button_paint(window, gButtonWidget->style, &gdk_rect, &gdk_clip, &buttonState, relief );
+      moz_gtk_button_paint(window, gButtonWidget->style, &gdk_rect, &gdk_clip,
+                           &state, relief );
     }
     break;
 
@@ -251,6 +238,8 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
     {
       EnsureCheckBoxWidget();
       
+      nsIAtom* atom = nsnull;
+
       if (aFrame) {
         // For XUL checkboxes and radio buttons, the state of the parent
         // determines our state.
@@ -258,26 +247,20 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
         aFrame->GetContent(getter_AddRefs(content));
         if (content->IsContentOfType(nsIContent::eXUL))
           aFrame->GetParent(&aFrame);
+        else {
+          nsCOMPtr<nsIAtom> tag;
+          content->GetTag(*getter_AddRefs(tag));
+          if (tag == mInputAtom)
+            atom = mInputCheckedAtom;
+        }
       }
       
-      GtkToggleButtonState checkBoxState;
-      GetGtkWidgetState(aFrame, (GtkWidgetState*)&checkBoxState);
-      nsIAtom* atom = (aWidgetType == NS_THEME_CHECKBOX) ? mCheckedAtom : mSelectedAtom;
-      checkBoxState.selected = CheckBooleanAttr(aFrame, atom);
+      if (!atom)
+        atom = (aWidgetType == NS_THEME_CHECKBOX) ? mCheckedAtom : mSelectedAtom;
       
-#ifdef DEBUG_NATIVE_THEME
-      printf("paint checkbox: aRect=(%d,%d,%d,%d), aClipRect=(%d,%d,%d,%d)\n",
-             aRect.x, aRect.y, aRect.width, aRect.height, aClipRect.x,
-             aClipRect.y, aClipRect.width, aClipRect.height);
-
-      printf("          gdk_rect=(%d,%d,%d,%d), gdk_clip=(%d,%d,%d,%d)\n",
-             gdk_rect.x, gdk_rect.y, gdk_rect.width, gdk_rect.height,
-             gdk_clip.x, gdk_clip.y, gdk_clip.width, gdk_clip.height);
-#endif
-
       moz_gtk_checkbox_paint(window, gCheckboxWidget->style, &gdk_rect,
-                             &gdk_clip, &checkBoxState,
-                             (aWidgetType==NS_THEME_RADIO) ? "radiobutton" : "checkbutton");
+                             &gdk_clip, &state, CheckBooleanAttr(aFrame, atom),
+                             (aWidgetType == NS_THEME_RADIO));
     }
     break;
 
@@ -287,107 +270,132 @@ nsNativeThemeGTK::DrawWidgetBackground(nsIRenderingContext* aContext,
   case NS_THEME_SCROLLBAR_BUTTON_RIGHT:
     {
       EnsureScrollbarWidget();
-
-      GtkWidgetState buttonState;
-      GetGtkWidgetState(aFrame, &buttonState);
-      
+      EnsureArrowWidget();
       GtkArrowType arrowType = GtkArrowType(aWidgetType - NS_THEME_SCROLLBAR_BUTTON_UP);
-      
-#ifdef DEBUG_NATIVE_THEME
-      printf("paint scrollbar button, rect=(%d,%d,%d,%d), clip=(%d,%d,%d,%d)\n",
-             gdk_rect.x, gdk_rect.y, gdk_rect.width, gdk_rect.height,
-             gdk_clip.x, gdk_clip.y, gdk_clip.width, gdk_clip.height);
-#endif
-      moz_gtk_scrollbar_button_paint(window, gScrollbarWidget->style, &gdk_rect, &gdk_clip,
-                                     &buttonState, arrowType);
+      moz_gtk_scrollbar_button_paint(window, gScrollbarWidget->style,
+                                     &gdk_rect, &gdk_clip, &state, arrowType);
     }
     break;
 
   case NS_THEME_SCROLLBAR_TRACK_VERTICAL:
   case NS_THEME_SCROLLBAR_TRACK_HORIZONTAL:
-    {
-      EnsureScrollbarWidget();
-
-      GtkWidgetState troughState;
-      GetGtkWidgetState(aFrame, &troughState);
-
-      moz_gtk_scrollbar_trough_paint(window, gScrollbarWidget->style,
-                                     &gdk_rect, &gdk_clip, &troughState);
-    }
+    EnsureScrollbarWidget();
+    moz_gtk_scrollbar_trough_paint(window, gScrollbarWidget->style, &gdk_rect,
+                                   &gdk_clip, &state);
     break;
 
   case NS_THEME_SCROLLBAR_THUMB_VERTICAL:
   case NS_THEME_SCROLLBAR_THUMB_HORIZONTAL:
-    {
-      EnsureScrollbarWidget();
-
-      GtkWidgetState thumbState;
-      GetGtkWidgetState(aFrame, &thumbState);
-
-#ifdef DEBUG_NATIVE_THEME
-      printf("paint thumb, rect=(%d,%d,%d,%d), clip=(%d,%d,%d,%d)\n",
-             gdk_rect.x, gdk_rect.y, gdk_rect.width, gdk_rect.height,
-             gdk_clip.x, gdk_clip.y, gdk_clip.width, gdk_clip.height);
-#endif
-      moz_gtk_scrollbar_thumb_paint(window, gScrollbarWidget->style,
-                                    &gdk_rect, &gdk_clip, &thumbState);
-    }
+    EnsureScrollbarWidget();
+    moz_gtk_scrollbar_thumb_paint(window, gScrollbarWidget->style, &gdk_rect,
+                                  &gdk_clip, &state);
     break;
 
   case NS_THEME_TOOLBAR_GRIPPER:
-    {
-      EnsureGripperWidget();
-
-      GtkWidgetState state;
-      GetGtkWidgetState(aFrame, &state);
-
-      moz_gtk_gripper_paint(window, gGripperWidget->style, &gdk_rect,
-                            &gdk_clip, &state);
-    }
+    EnsureGripperWidget();
+    moz_gtk_gripper_paint(window, gGripperWidget->style, &gdk_rect, &gdk_clip,
+                          &state);
     break;
 
+  case NS_THEME_DROPDOWN_TEXTFIELD:
+    if (aFrame)
+      // look at the parent frame (the textbox)
+      aFrame->GetParent(&aFrame);
+
+    // fall through
+
+  case NS_THEME_TEXTFIELD:
+    EnsureEntryWidget();
+    moz_gtk_entry_paint(window, gEntryWidget->style, &gdk_rect, &gdk_clip,
+                        &state);
+    break;
+
+  case NS_THEME_DROPDOWN_BUTTON:
+    EnsureArrowWidget();
+    moz_gtk_dropdown_arrow_paint(window, gArrowWidget->style, &gdk_rect,
+                                 &gdk_clip, &state);
+    break;
+
+  case NS_THEME_CHECKBOX_CONTAINER:
+  case NS_THEME_RADIO_CONTAINER:
+    EnsureCheckBoxWidget();
+    moz_gtk_container_paint(window, gCheckboxWidget->style, &gdk_rect,
+                            &gdk_clip, &state,
+                            (aWidgetType == NS_THEME_RADIO_CONTAINER));
+    break;
+
+  case NS_THEME_TOOLBOX:
+    EnsureHandleBoxWidget();
+    moz_gtk_toolbar_paint(window, gHandleBoxWidget->style, &gdk_rect,
+                          &gdk_clip);
+    break;
+
+  case NS_THEME_TOOLTIP:
+    EnsureTooltipWidget();
+    moz_gtk_tooltip_paint(window, gTooltipWidget->tip_window->style, &gdk_rect,
+                          &gdk_clip);
+    break;
+
+  case NS_THEME_STATUSBAR_PANEL:
+    EnsureFrameWidget();
+    moz_gtk_frame_paint(window, gFrameWidget->style, &gdk_rect, &gdk_clip);
+    break;
+
+  case NS_THEME_PROGRESSBAR:
+  case NS_THEME_PROGRESSBAR_VERTICAL:
+    EnsureProgressBarWidget();
+    moz_gtk_progressbar_paint(window, gProgressWidget->style, &gdk_rect,
+                              &gdk_clip);
+    break;
+
+  case NS_THEME_PROGRESSBAR_CHUNK:
+  case NS_THEME_PROGRESSBAR_CHUNK_VERTICAL:
+    EnsureProgressBarWidget();
+    moz_gtk_progress_chunk_paint(window, gProgressWidget->style, &gdk_rect,
+                                 &gdk_clip);
+    break;
+
+  case NS_THEME_TAB_PANELS:
+    EnsureTabWidget();
+    moz_gtk_tabpanels_paint(window, gTabWidget->style, &gdk_rect, &gdk_clip);
+    break;
+
+  case NS_THEME_TAB:
+  case NS_THEME_TAB_LEFT_EDGE:
+  case NS_THEME_TAB_RIGHT_EDGE:
+    {
+      EnsureTabWidget();
+      gint tab_flags = 0;
+
+      if (aWidgetType == NS_THEME_TAB && CheckBooleanAttr(aFrame, mSelectedAtom))
+        tab_flags |= MOZ_GTK_TAB_SELECTED;
+      else if (aWidgetType == NS_THEME_TAB_LEFT_EDGE)
+        tab_flags |= MOZ_GTK_TAB_BEFORE_SELECTED;
+
+      nsCOMPtr<nsIContent> content;
+      aFrame->GetContent(getter_AddRefs(content));
+      if (content->HasAttr(kNameSpaceID_None, mFirstTabAtom))
+        tab_flags |= MOZ_GTK_TAB_FIRST;
+
+      moz_gtk_tab_paint(window, gTabWidget->style, &gdk_rect, &gdk_clip,
+                        tab_flags);
+      break;
+    }
   }
 
   return NS_OK;
 }
 
-#define RANGE_CLASS(w) GTK_RANGE_CLASS(GTK_OBJECT(w)->klass)
-
-void
-nsNativeThemeGTK::GetScrollbarMetrics(gint* slider_width,
-                                      gint* trough_border,
-                                      gint* stepper_size,
-                                      gint* stepper_spacing)
+inline void
+WidgetBorderToMargin(GtkWidget* aWidget, nsMargin* aResult)
 {
-  EnsureScrollbarWidget();
-
-  if (slider_width)
-    *slider_width = gtk_style_get_prop_experimental(gScrollbarWidget->style,
-                                                    "GtkRange::slider_width",
-                                                    RANGE_CLASS(gScrollbarWidget)->slider_width);
-
-  if (trough_border)
-    *trough_border = gtk_style_get_prop_experimental(gScrollbarWidget->style,
-                                                     "GtkRange::trough_border",
-                                                     gScrollbarWidget->style->klass->xthickness);
-
-  if (stepper_size)
-    *stepper_size = gtk_style_get_prop_experimental(gScrollbarWidget->style,
-                                                    "GtkRange::stepper_size",
-                                                    RANGE_CLASS(gScrollbarWidget)->stepper_size);
-
-  if (stepper_spacing)
-    *stepper_spacing = gtk_style_get_prop_experimental(gScrollbarWidget->style,
-                                                       "GtkRange::stepper_spacing",
-                                                       RANGE_CLASS(gScrollbarWidget)->stepper_slider_spacing);
-
+  aResult->left = aResult->right = aWidget->style->klass->xthickness;
+  aResult->top = aResult->bottom = aWidget->style->klass->ythickness;
 }
 
 NS_IMETHODIMP
-nsNativeThemeGTK::GetWidgetBorder(nsIDeviceContext* aContext, 
-                                  nsIFrame* aFrame,
-                                  PRUint8 aWidgetType,
-                                  nsMargin* aResult)
+nsNativeThemeGTK::GetWidgetBorder(nsIDeviceContext* aContext, nsIFrame* aFrame,
+                                  PRUint8 aWidgetType, nsMargin* aResult)
 {
   aResult->top = aResult->bottom = aResult->left = aResult->right = 0;
 
@@ -396,18 +404,54 @@ nsNativeThemeGTK::GetWidgetBorder(nsIDeviceContext* aContext,
   case NS_THEME_SCROLLBAR_TRACK_HORIZONTAL:
     {
       gint trough_border;
-      GetScrollbarMetrics(nsnull, &trough_border, nsnull, nsnull);
+      EnsureScrollbarWidget();
+      moz_gtk_get_scrollbar_metrics(nsnull, &trough_border, nsnull, nsnull, nsnull);
       aResult->top = aResult->bottom = aResult->left = aResult->right = trough_border;
     }
     break;
+  case NS_THEME_TEXTFIELD:
+  case NS_THEME_DROPDOWN_TEXTFIELD:
+    EnsureEntryWidget();
+    WidgetBorderToMargin(gEntryWidget, aResult);
+    break;
+  case NS_THEME_BUTTON:
+  case NS_THEME_TOOLBAR_BUTTON:
+    EnsureButtonWidget();
+    WidgetBorderToMargin(gButtonWidget, aResult);
+    break;
+  case NS_THEME_TOOLBAR_GRIPPER:
+    EnsureGripperWidget();
+    WidgetBorderToMargin(gGripperWidget, aResult);
+    break;
+  case NS_THEME_DROPDOWN_BUTTON:
+    EnsureArrowWidget();
+    WidgetBorderToMargin(gDropdownButtonWidget, aResult);
+    break;
+  case NS_THEME_CHECKBOX_CONTAINER:
+  case NS_THEME_RADIO_CONTAINER:
+    aResult->top = aResult->bottom = aResult->left = aResult->right = 1;
+    break;
+  case NS_THEME_STATUSBAR_PANEL:
+    EnsureFrameWidget();
+    WidgetBorderToMargin(gFrameWidget, aResult);
+    break;
+  case NS_THEME_PROGRESSBAR:
+  case NS_THEME_PROGRESSBAR_VERTICAL:
+    EnsureProgressBarWidget();
+    WidgetBorderToMargin(gProgressWidget, aResult);
+    break;
+  case NS_THEME_TAB_PANELS:
+    EnsureTabWidget();
+    WidgetBorderToMargin(gTabWidget, aResult);
+    break;
   }
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNativeThemeGTK::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame* aFrame,
-                                       PRUint8 aWidgetType,
+nsNativeThemeGTK::GetMinimumWidgetSize(nsIRenderingContext* aContext,
+                                       nsIFrame* aFrame, PRUint8 aWidgetType,
                                        nsSize* aResult, PRBool* aIsOverridable)
 {
   aResult->width = aResult->height = 0;
@@ -422,7 +466,8 @@ nsNativeThemeGTK::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
         EnsureScrollbarWidget();
 
         gint slider_width, stepper_size;
-        GetScrollbarMetrics(&slider_width, nsnull, &stepper_size, nsnull);
+        moz_gtk_get_scrollbar_metrics(&slider_width, nsnull, &stepper_size,
+                                      nsnull, nsnull);
 
         aResult->width = slider_width;
         aResult->height = stepper_size;
@@ -434,24 +479,52 @@ nsNativeThemeGTK::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
       {
         EnsureScrollbarWidget();
 
-        gint slider_width;
-        GetScrollbarMetrics(&slider_width, nsnull, nsnull, nsnull);
+        gint slider_width, min_slider_size;
+        moz_gtk_get_scrollbar_metrics(&slider_width, nsnull, nsnull, nsnull,
+                                      &min_slider_size);
 
         if (aWidgetType == NS_THEME_SCROLLBAR_THUMB_VERTICAL) {
           aResult->width = slider_width;
-          aResult->height = RANGE_CLASS(gScrollbarWidget)->min_slider_size;
+          aResult->height = min_slider_size;
         } else {
-          aResult->width = RANGE_CLASS(gScrollbarWidget)->min_slider_size;
+          aResult->width = min_slider_size;
           aResult->height = slider_width;
         }
 
-#ifdef DEBUG_NATIVE_THEME
-        printf("scrollbar thumb min size: (%d,%d)\n", aResult->width,
-               aResult->height);
-#endif
         *aIsOverridable = PR_FALSE;
       }
       break;
+  case NS_THEME_DROPDOWN_BUTTON:
+    {
+      EnsureArrowWidget();
+
+      // First, get the minimum size for the button itself.
+      aResult->width = 2 * (1 + gDropdownButtonWidget->style->klass->xthickness);
+      aResult->height = 2 * (1 + gDropdownButtonWidget->style->klass->ythickness);
+
+      // Now, add in the requested size of the arrow.
+      // Note: the minimum arrow size is fixed at 11 pixels.
+
+      aResult->width += 11 + GTK_MISC(gArrowWidget)->xpad * 2;
+      aResult->height += 11 + GTK_MISC(gArrowWidget)->ypad * 2;      
+      *aIsOverridable = PR_FALSE;
+    }
+    break;
+  case NS_THEME_CHECKBOX:
+  case NS_THEME_RADIO:
+  case NS_THEME_CHECKBOX_CONTAINER:
+  case NS_THEME_RADIO_CONTAINER:
+    {
+      EnsureCheckBoxWidget();
+
+      gint indicator_size, indicator_spacing;
+      _gtk_check_button_get_props (GTK_CHECK_BUTTON(gCheckboxWidget), &indicator_size,
+                                   &indicator_spacing);
+
+      aResult->width = aResult->height = indicator_size;
+      *aIsOverridable = PR_FALSE;
+    }
+    break;
   }
 
   return NS_OK;
@@ -463,8 +536,6 @@ nsNativeThemeGTK::WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType,
 {
   // Some widget types just never change state.
   if (aWidgetType == NS_THEME_TOOLBOX || aWidgetType == NS_THEME_TOOLBAR ||
-      aWidgetType == NS_THEME_SCROLLBAR_TRACK_VERTICAL || 
-      aWidgetType == NS_THEME_SCROLLBAR_TRACK_HORIZONTAL || 
       aWidgetType == NS_THEME_STATUSBAR || aWidgetType == NS_THEME_STATUSBAR_PANEL ||
       aWidgetType == NS_THEME_STATUSBAR_RESIZER_PANEL ||
       aWidgetType == NS_THEME_PROGRESSBAR_CHUNK ||
@@ -507,18 +578,69 @@ nsNativeThemeGTK::ThemeSupportsWidget(nsIPresContext* aPresContext,
 {
   switch (aWidgetType) {
   case NS_THEME_BUTTON:
+  case NS_THEME_RADIO:
   case NS_THEME_CHECKBOX:
+  case NS_THEME_TOOLBOX:
+    // case NS_THEME_TOOLBAR:  (not in skin)
+  case NS_THEME_TOOLBAR_BUTTON:
+  case NS_THEME_TOOLBAR_DUAL_BUTTON: // so we can override the border with 0
+    // case NS_THEME_TOOLBAR_DUAL_BUTTON_DROPDOWN:
+    // case NS_THEME_TOOLBAR_SEPARATOR:
+  case NS_THEME_TOOLBAR_GRIPPER:
+  case NS_THEME_STATUSBAR:
+  case NS_THEME_STATUSBAR_PANEL:
+    // case NS_THEME_RESIZER:  (n/a for gtk)
+    // case NS_THEME_LISTBOX:
+    // case NS_THEME_LISTBOX_LISTITEM:
+    // case NS_THEME_TREEVIEW:
+    // case NS_THEME_TREEVIEW_TREEITEM:
+    // case NS_THEME_TREEVIEW_TWISTY:
+    // case NS_THEME_TREEVIEW_LINE:
+    // case NS_THEME_TREEVIEW_HEADER:
+    // case NS_THEME_TREEVIEW_HEADER_CELL:
+    // case NS_THEME_TREEVIEW_HEADER_SORTARROW:
+    // case NS_THEME_TREEVIEW_TWISTY_OPEN:
+    case NS_THEME_PROGRESSBAR:
+    case NS_THEME_PROGRESSBAR_CHUNK:
+    case NS_THEME_PROGRESSBAR_VERTICAL:
+    case NS_THEME_PROGRESSBAR_CHUNK_VERTICAL:
+    case NS_THEME_TAB:
+    // case NS_THEME_TAB_PANEL:
+    case NS_THEME_TAB_LEFT_EDGE:
+    case NS_THEME_TAB_RIGHT_EDGE:
+    case NS_THEME_TAB_PANELS:
+  case NS_THEME_TOOLTIP:
+    // case NS_THEME_SPINNER:
+    // case NS_THEME_SPINNER_UP_BUTTON:
+    // case NS_THEME_SPINNER_DOWN_BUTTON:
+    // case NS_THEME_SCROLLBAR:  (n/a for gtk)
   case NS_THEME_SCROLLBAR_BUTTON_UP:
   case NS_THEME_SCROLLBAR_BUTTON_DOWN:
   case NS_THEME_SCROLLBAR_BUTTON_LEFT:
   case NS_THEME_SCROLLBAR_BUTTON_RIGHT:
-  case NS_THEME_SCROLLBAR_TRACK_VERTICAL:
   case NS_THEME_SCROLLBAR_TRACK_HORIZONTAL:
-  case NS_THEME_SCROLLBAR_THUMB_VERTICAL:
+  case NS_THEME_SCROLLBAR_TRACK_VERTICAL:
   case NS_THEME_SCROLLBAR_THUMB_HORIZONTAL:
-  case NS_THEME_TOOLBAR_BUTTON:
-  case NS_THEME_TOOLBAR_GRIPPER:
-  case NS_THEME_RADIO:
+  case NS_THEME_SCROLLBAR_THUMB_VERTICAL:
+    // case NS_THEME_SCROLLBAR_GRIPPER_HORIZONTAL:  (n/a for gtk)
+    // case NS_THEME_SCROLLBAR_GRIPPER_VERTICAL:  (n/a for gtk)
+  case NS_THEME_TEXTFIELD:
+    // case NS_THEME_TEXTFIELD_CARET:
+    // case NS_THEME_DROPDOWN:
+  case NS_THEME_DROPDOWN_BUTTON:
+    // case NS_THEME_DROPDOWN_TEXT:
+  case NS_THEME_DROPDOWN_TEXTFIELD:
+    // case NS_THEME_SLIDER:
+    // case NS_THEME_SLIDER_THUMB:
+    // case NS_THEME_SLIDER_THUMB_START:
+    // case NS_THEME_SLIDER_THUMB_END:
+    // case NS_THEME_SLIDER_TICK:
+  case NS_THEME_CHECKBOX_CONTAINER:
+  case NS_THEME_RADIO_CONTAINER:
+    // case NS_THEME_WINDOW:
+    // case NS_THEME_DIALOG:
+    // case NS_THEME_MENU:
+    // case NS_THEME_MENUBAR:
     return PR_TRUE;
   }
 
@@ -539,10 +661,10 @@ nsNativeThemeGTK::WidgetIsContainer(PRUint8 aWidgetType)
 void
 nsNativeThemeGTK::SetupWidgetPrototype(GtkWidget* widget)
 {
-  if (!mProtoWindow) {
-    mProtoWindow = gtk_window_new(GTK_WINDOW_POPUP);
+  if (!gProtoWindow) {
+    gProtoWindow = gtk_window_new(GTK_WINDOW_POPUP);
     mProtoLayout = gtk_fixed_new();
-    gtk_container_add(GTK_CONTAINER(mProtoWindow), mProtoLayout);
+    gtk_container_add(GTK_CONTAINER(gProtoWindow), mProtoLayout);
   }
 
   gtk_container_add(GTK_CONTAINER(mProtoLayout), widget);
@@ -586,3 +708,72 @@ nsNativeThemeGTK::EnsureGripperWidget()
   }
 }
 
+void
+nsNativeThemeGTK::EnsureEntryWidget()
+{
+  if (!gEntryWidget) {
+    gEntryWidget = gtk_entry_new();
+    SetupWidgetPrototype(gEntryWidget);
+  }
+}
+
+void
+nsNativeThemeGTK::EnsureArrowWidget()
+{
+  if (!gArrowWidget) {
+    gDropdownButtonWidget = gtk_button_new();
+    SetupWidgetPrototype(gDropdownButtonWidget);
+    gArrowWidget = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
+    gtk_container_add(GTK_CONTAINER(gDropdownButtonWidget), gArrowWidget);
+    gtk_widget_set_rc_style(gArrowWidget);
+    gtk_widget_realize(gArrowWidget);
+  }
+}   
+
+void
+nsNativeThemeGTK::EnsureHandleBoxWidget()
+{
+  if (!gHandleBoxWidget) {
+    gHandleBoxWidget = gtk_handle_box_new();
+    SetupWidgetPrototype(gHandleBoxWidget);
+  }
+}
+
+void
+nsNativeThemeGTK::EnsureTooltipWidget()
+{
+  if (!gTooltipWidget) {
+    gTooltipWidget = gtk_tooltips_new();
+    gtk_tooltips_force_window(gTooltipWidget);
+    gtk_widget_set_rc_style(gTooltipWidget->tip_window);
+    gtk_widget_realize(gTooltipWidget->tip_window);
+  }
+}
+
+void
+nsNativeThemeGTK::EnsureFrameWidget()
+{
+  if (!gFrameWidget) {
+    gFrameWidget = gtk_frame_new(NULL);
+    SetupWidgetPrototype(gFrameWidget);
+  }
+}
+
+void
+nsNativeThemeGTK::EnsureProgressBarWidget()
+{
+  if (!gProgressWidget) {
+    gProgressWidget = gtk_progress_bar_new();
+    SetupWidgetPrototype(gProgressWidget);
+  }
+}
+
+void
+nsNativeThemeGTK::EnsureTabWidget()
+{
+  if (!gTabWidget) {
+    gTabWidget = gtk_notebook_new();
+    SetupWidgetPrototype(gTabWidget);
+  }
+}
+
