@@ -201,8 +201,10 @@ NS_IMETHODIMP nsDeviceContextMac :: SupportsNativeWidgets(PRBool &aSupportsWidge
 NS_IMETHODIMP nsDeviceContextMac :: GetScrollBarDimensions(float &aWidth, float &aHeight) const
 {
   // XXX Should we push this to widget library
-  aWidth = 16 * mDevUnitsToAppUnits;
-  aHeight = 16 * mDevUnitsToAppUnits;
+  float scale;
+  GetCanonicalPixelScale(scale);
+  aWidth = 16 * mDevUnitsToAppUnits * scale;
+  aHeight = 16 * mDevUnitsToAppUnits * scale;
   return NS_OK;
 }
 
@@ -319,9 +321,66 @@ NS_IMETHODIMP nsDeviceContextMac :: GetSystemFont(nsSystemFontID aID, nsFont *aF
         Str255 fontName;
         SInt16 fontSize;
         Style fontStyle;
-        ::GetThemeFont(fontID, smSystemScript, fontName, &fontSize, &fontStyle);
+
+        ScriptCode sysScript = ::GetScriptManagerVariable (smSysScript);
+        
+        // the Korean , TradChinese and SimpChinese localization return very ugly
+        // font face for theme font, the size of ASCII part in those font very small and
+        // they look very very poor on QuickDraw without anti-alias. We need to use the roman
+        // theme font instead so the user can read those UI
+        
+        if ((smKorean == sysScript) ||
+            (smTradChinese == sysScript) ||
+            (smSimpChinese == sysScript))
+          sysScript = smRoman;
+          
+        ::GetThemeFont(fontID, sysScript, fontName, &fontSize, &fontStyle);
         fontName[fontName[0]+1] = 0;
-        aFont->name.AssignWithConversion( (char*)&fontName[1] );
+        
+        OSStatus err;
+        // the theme font could contains font name in different encoding. 
+        // we need ot covert them to unicode according to the font's text encoding.
+        aFont->name.Truncate(0);
+        TECObjectRef converter = 0;
+        TextEncoding fontEncoding = 0;
+        TextEncoding unicodeEncoding = ::CreateTextEncoding(kTextEncodingUnicodeDefault, 
+                                                              kTextEncodingDefaultVariant,
+                                                              kTextEncodingDefaultFormat);
+                                                              
+        // MacOS 8.6 do not have FMxxx etc so we have to check
+        if (HaveFontManager90()) {       
+          FMFontFamily fontFamily;
+          fontFamily = ::FMGetFontFamilyFromName(fontName);
+          err = ::FMGetFontFamilyTextEncoding(fontFamily, &fontEncoding);
+        } else {
+          short	fondID;
+          ::GetFNum(fontName, &fondID);
+          ScriptCode script = ::FontToScript(fondID);
+          err = ::UpgradeScriptInfoToTextEncoding(script, kTextLanguageDontCare, 
+                                                  kTextRegionDontCare, NULL, &fontEncoding);
+
+        }
+        if (err == noErr)
+        {
+           err = ::TECCreateConverter(&converter, fontEncoding, unicodeEncoding);
+           if (err == noErr)
+           {
+               PRUnichar unicodeFontName[sizeof(fontName)];
+               ByteCount actualInputLength, actualOutputLength;
+               err = ::TECConvertText(converter, &fontName[1], fontName[0], 
+                                      &actualInputLength, 
+                                      (TextPtr)unicodeFontName, sizeof(unicodeFontName),
+                                      &actualOutputLength);  
+               unicodeFontName[actualOutputLength / sizeof(PRUnichar)] = PRUnichar('\0');
+               aFont->name.Assign(unicodeFontName);
+               ::TECDisposeConverter(converter);
+           }             
+        }
+        NS_ASSERTION(aFont->name.Length() > 0, "empty font name");
+        if (aFont->name.Length()  == 0) 
+        {
+           aFont->name.AssignWithConversion( (char*)&fontName[1], fontName[0] );
+        }
         aFont->size = NSToCoordRound(float(fontSize) * dev2app);
 
         if (fontStyle & bold)
@@ -432,13 +491,15 @@ nsDeviceContextMac :: FindScreenForSurface ( nsIScreen** outScreen )
     return;
   }
   
-  // we have a widget stashed inside, get a native WindowRef out of it
   nsIWidget* widget = reinterpret_cast<nsIWidget*>(mWidget);      // PRAY!
   NS_ASSERTION ( widget, "No Widget --> No Window" );
   if ( !widget ) {
     NS_IF_ADDREF(*outScreen = mPrimaryScreen.get());              // bail out with the main screen just to be safe.
     return;
-  }  
+  }
+
+#if !MOZ_WIDGET_COCOA
+  // we have a widget stashed inside, get a native WindowRef out of it
  	WindowRef window = reinterpret_cast<WindowRef>(widget->GetNativeData(NS_NATIVE_DISPLAY));
 
   StPortSetter  setter(window);
@@ -467,6 +528,10 @@ nsDeviceContextMac :: FindScreenForSurface ( nsIScreen** outScreen )
   }
   else
     *outScreen = nsnull;
+#else
+  // in cocoa, we don't have a windowPtr! bail out with the main screen
+  NS_IF_ADDREF(*outScreen = mPrimaryScreen.get());
+#endif
   
 } // FindScreenForSurface
 
@@ -642,7 +707,10 @@ NS_IMETHODIMP nsDeviceContextMac::GetDeviceContextFor(nsIDeviceContextSpec *aDev
  *  See documentation in nsIDeviceContext.h
  *	@update 12/9/98 dwc
  */
-NS_IMETHODIMP nsDeviceContextMac::BeginDocument(PRUnichar * aTitle)
+NS_IMETHODIMP nsDeviceContextMac::BeginDocument(PRUnichar * aTitle, 
+                                                PRUnichar*  aPrintToFileName,
+                                                PRInt32     aStartPage, 
+                                                PRInt32     aEndPage)
 {
 #if !TARGET_CARBON
 GrafPtr	thePort;
@@ -659,7 +727,7 @@ GrafPtr	thePort;
     nsresult rv = NS_ERROR_FAILURE;
     nsCOMPtr<nsIPrintingContext> printingContext = do_QueryInterface(mSpec);
     if (printingContext)
-        rv = printingContext->BeginDocument();
+        rv = printingContext->BeginDocument(aStartPage, aEndPage);
     return rv;
 #endif
 }
@@ -686,6 +754,15 @@ NS_IMETHODIMP nsDeviceContextMac::EndDocument(void)
 #endif
 }
 
+/** ---------------------------------------------------
+ *  See documentation in nsIDeviceContext.h
+ *	@update 12/9/98 dwc
+ */
+NS_IMETHODIMP nsDeviceContextMac::AbortDocument(void)
+{
+    return EndDocument();
+}
+
 
 /** ---------------------------------------------------
  *  See documentation in nsIDeviceContext.h
@@ -761,7 +838,8 @@ PRUint32 FontNameKey::HashCode(void) const
 
 PRBool FontNameKey::Equals(const nsHashKey *aKey) const
 {
-  return mString.EqualsIgnoreCase(((FontNameKey*)aKey)->mString);
+  return mString.Equals(((FontNameKey*)aKey)->mString,
+                        nsCaseInsensitiveStringComparator());
 }
 
 nsHashKey* FontNameKey::Clone(void) const
@@ -898,7 +976,7 @@ bool nsDeviceContextMac :: GetMacFontNumber(const nsString& aFontName, short &aF
 	InitFontInfoList();
     FontNameKey key(aFontName);
 	aFontNum = (short)gFontInfoList->Get(&key);
-	return (aFontNum != 0) && (kFontIDSymbol != aFontNum);
+	return (aFontNum != 0);
 }
 
 
@@ -976,6 +1054,11 @@ PRUint32 nsDeviceContextMac::GetScreenResolution()
 	return mPixelsPerInch;
 }
 
+PRBool nsDeviceContextMac::HaveFontManager90()
+{
+  return (kUnresolvedCFragSymbolAddress != (UInt32) FMGetFontFamilyFromName);
+}
+
 
 #pragma mark -
 //------------------------------------------------------------------------
@@ -1087,10 +1170,23 @@ EnumerateFont(nsHashKey *aKey, void *aData, void* closure)
   EnumerateFontInfo* info = (EnumerateFontInfo*) closure;
   PRUnichar** array = info->mArray;
   int j = info->mCount;
-  
+  PRBool match = PR_FALSE;
+#if TARGET_CARBON
+  // we need to match the cast of FMFontFamily in nsDeviceContextMac :: InitFontInfoList()
+  FMFontFamily fontFamily = (FMFontFamily) aData;
+  TextEncoding fontEncoding;
+  OSStatus status = ::FMGetFontFamilyTextEncoding(fontFamily, &fontEncoding);
+  if (noErr == status) {
+    ScriptCode script;
+    status = ::RevertTextEncodingToScriptInfo(fontEncoding, &script, nsnull, nsnull);
+    match = ((noErr == status) && (script == info->mScript));
+  }
+#else
   short	fondID = (short) aData;
   ScriptCode script = ::FontToScript(fondID);
-	if(script == info->mScript) {
+	match = (script == info->mScript) ;
+#endif
+  if (match) {
 	  PRUnichar* str = ToNewUnicode(((FontNameKey*)aKey)->mString);
 	  if (!str) {
 	    for (j = j - 1; j >= 0; j--) {
