@@ -39,12 +39,53 @@
 #include "nsRenderingContextMac.h"
 #include "nsDeviceContextMac.h"
 #include "nsCarbonHelpers.h"
+#include "nsRegionPool.h"
 
 #include <MacTypes.h>
-#include <QuickDraw.h>
+#include <Quickdraw.h>
 
 #include "nsGfxUtils.h"
 
+#if 0
+#if TARGET_CARBON
+// useful region debugging code.
+static OSStatus PrintRgnRectProc(UInt16 message, RgnHandle rgn, const Rect *inRect, void *refCon)
+{
+  UInt32*   rectCount = (UInt32*)refCon;
+  
+  switch (message)
+  {
+    case kQDRegionToRectsMsgInit:
+      printf("Dumping region 0x%X\n", rgn);
+      break;
+      
+    case kQDRegionToRectsMsgParse:
+      printf("Rect %d t,l,r,b: %ld, %ld, %ld, %ld\n", *rectCount, inRect->top, inRect->left, inRect->right, inRect->bottom);
+      (*rectCount)++;
+      break;
+      
+    case kQDRegionToRectsMsgTerminate:
+      printf("\n");
+      break;
+  }
+  
+  return noErr;
+}
+
+static void PrintRegionOutline(RgnHandle inRgn)
+{
+  static RegionToRectsUPP sCountRectProc = nsnull;
+  if (!sCountRectProc)
+    sCountRectProc = NewRegionToRectsUPP(PrintRgnRectProc);
+  
+  UInt32    rectCount = 0;  
+  ::QDRegionToRects(inRgn, kQDParseRegionFromTopLeft, sCountRectProc, &rectCount);
+}
+#endif // TARGET_CARBON
+#endif
+
+#pragma mark -
+
 /** ---------------------------------------------------
  *	See documentation in nsImageMac.h
  *	@update 
@@ -329,7 +370,7 @@ NS_IMETHODIMP nsImageMac::Draw(nsIRenderingContext &aContext, nsDrawingSurface a
     CopyBitsWithMask((BitMap*)(&mImagePixmap),
         mMaskBitsHandle ? (BitMap*)(&mMaskPixmap) : nsnull, mAlphaDepth,
         (BitMap*)(*destPixels),
-        srcRect, maskRect, dstRect);
+        srcRect, maskRect, dstRect, PR_TRUE);
 	}
 	
 	return NS_OK;
@@ -360,6 +401,10 @@ NS_IMETHODIMP nsImageMac :: DrawToImage(nsIImage* aDstImage, PRInt32 aDX, PRInt3
   if (!mImageBitsHandle)
     return NS_ERROR_FAILURE;
 
+#ifdef MOZ_WIDGET_COCOA
+  nsGraphicsUtils::SetPortToKnownGoodPort();
+#endif
+
   // lock and set up bits handles
   LockImagePixels(PR_FALSE);
   LockImagePixels(PR_TRUE);
@@ -380,10 +425,10 @@ NS_IMETHODIMP nsImageMac :: DrawToImage(nsIImage* aDstImage, PRInt32 aDX, PRInt3
   PixMap* destPixels;
   dstMacImage->GetPixMap(&destPixels);
   NS_ASSERTION(destPixels, "No dest pixels!");
-          
+
   CopyBitsWithMask((BitMap*)(&mImagePixmap),
       mMaskBitsHandle ? (BitMap*)(&mMaskPixmap) : nsnull, mAlphaDepth,
-      (BitMap*)(destPixels), srcRect, maskRect, dstRect);
+      (BitMap*)(destPixels), srcRect, maskRect, dstRect, PR_FALSE);
   
   aDstImage->UnlockImagePixels(PR_FALSE);
   aDstImage->UnlockImagePixels(PR_TRUE);
@@ -478,14 +523,14 @@ nsImageMac::UnlockImagePixels(PRBool aMaskPixels)
 #pragma mark -
 
 /** -----------------------------------------------------------------
- *	Create a PixMap, filling in ioPixMap
+ *  Create a PixMap, filling in ioPixMap
  */
-OSErr nsImageMac::CreatePixMap(	PRInt32 aWidth, PRInt32 aHeight, 
-								PRInt32 aDepth, CTabHandle aColorTable, 
-								PixMap& ioPixMap, Handle& ioBitsHandle)
-{								
+OSErr nsImageMac::CreatePixMap( PRInt32 aWidth, PRInt32 aHeight, 
+                PRInt32 aDepth, CTabHandle aColorTable, 
+                PixMap& ioPixMap, Handle& ioBitsHandle)
+{
     return CreatePixMapInternal(aWidth, aHeight, aDepth, aColorTable, 
-						        		        ioPixMap, ioBitsHandle, PR_FALSE);
+                                ioPixMap, ioBitsHandle, PR_FALSE);
 }
 
 
@@ -776,15 +821,21 @@ OSErr nsImageMac::AllocateGWorld(PRInt16 depth, CTabHandle colorTable, const Rec
 	return memFullErr;
 }
 
-void nsImageMac::CopyBitsWithMask(BitMap* srcBits, BitMap* maskBits, PRInt16 maskDepth, BitMap* destBits,
-        const Rect& srcRect, const Rect& maskRect, const Rect& destRect)
+void nsImageMac::CopyBitsWithMask(const BitMap* srcBits, const BitMap* maskBits, PRInt16 maskDepth, const BitMap* destBits,
+        const Rect& srcRect, const Rect& maskRect, const Rect& destRect, PRBool inDrawingToPort)
 {
   if (maskBits)
   {
-    if (maskDepth > 1)
-      ::CopyDeepMask(srcBits, maskBits, destBits, &srcRect, &maskRect, &destRect, srcCopy, nsnull);
-    else
-      ::CopyMask(srcBits, maskBits, destBits, &srcRect, &maskRect, &destRect);
+    StRegionFromPool    clipRegion;
+    
+    if (inDrawingToPort)
+    {
+      // we need to pass in the clip region, even if it doesn't intersect the image, to avoid a bug
+      // on Mac OS X that causes bad image drawing (see bug 137295).
+      ::GetClip(clipRegion);
+    }
+    
+    ::CopyDeepMask(srcBits, maskBits, destBits, &srcRect, &maskRect, &destRect, srcCopy, inDrawingToPort ? clipRegion : (RgnHandle)nsnull);
   }
   else
   {
@@ -843,7 +894,7 @@ nsImageMac :: ConvertToPICT ( PicHandle* outPicture )
   		// copy from the destination into our temp GWorld, to get the background
       CopyBitsWithMask((BitMap*)(&mImagePixmap),
           mMaskBitsHandle ? (BitMap*)(&mMaskPixmap) : nsnull, mAlphaDepth,
-          (BitMap*)(*tempPixMap), picFrame, maskFrame, picFrame);
+          (BitMap*)(*tempPixMap), picFrame, maskFrame, picFrame, PR_FALSE);
       
   		// now copy into the picture
   		GWorldPtr currPort;
@@ -947,7 +998,7 @@ nsImageMac::ConvertToIcon(  const nsRect& aSrcRegion,
     }
     
     //returns null if there size specified isn't a valid size for an icon
-    OSType iconType = MakeIconType(aIconSize, aIconDepth, false);
+    OSType iconType = MakeIconType(aIconSize, aIconDepth, PR_FALSE);
     if (iconType == nil) {
         return NS_ERROR_INVALID_ARG;
     } 
@@ -1034,7 +1085,7 @@ nsImageMac::ConvertAlphaToIconMask(  const nsRect& aSrcRegion,
     *aOutIconType = nsnull;
     
     //returns null if there size specified isn't a valid size for an icon
-    OSType iconType = MakeIconType(aMaskSize, aMaskDepth, true);
+    OSType iconType = MakeIconType(aMaskSize, aMaskDepth, PR_TRUE);
     if (iconType == nil) {
         return NS_ERROR_INVALID_ARG;
     } 
@@ -1562,7 +1613,7 @@ nsresult nsImageMac::DrawTileQuickly(nsIRenderingContext &aContext,
 	  		// CopyBits will do the truncation for us at the edges
         CopyBitsWithMask((BitMap*)(&mImagePixmap),
             mMaskBitsHandle ? (BitMap*)(&mMaskPixmap) : nsnull, mAlphaDepth,
-            (BitMap*)(*destPixels), imageRect, imageRect, imageDestRect);
+            (BitMap*)(*destPixels), imageRect, imageRect, imageDestRect, PR_TRUE);
 	  	}
   	}
   
@@ -1745,7 +1796,7 @@ nsresult nsImageMac::DrawTileQuickly(nsIRenderingContext &aContext,
     // finally, copy to the destination
     CopyBitsWithMask((BitMap*)(*tilingPixels),
         maskingPixels ? (BitMap*)(*maskingPixels) : nsnull, mAlphaDepth,
-        (BitMap*)(*destPixels), tileRect, tileRect, tileDestRect);
+        (BitMap*)(*destPixels), tileRect, tileRect, tileDestRect, PR_TRUE);
     
   } // scope for locks
 
