@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: NPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -57,23 +57,6 @@
 #include "nsRegionPool.h"
 #include "nsGfxUtils.h"
 
-//
-// Return true if we are on Mac OS X, caching the result after the first call
-// Yes, this needs to go somehwere better.
-//
-static PRBool OnMacOSX()
-{
-  static PRBool gInitVer = PR_FALSE;
-  static PRBool gOnMacOSX = PR_FALSE;
-  if(! gInitVer) {
-    long version;
-    OSErr err = ::Gestalt(gestaltSystemVersion, &version);
-    gOnMacOSX = (err == noErr && version >= 0x00001000);
-    gInitVer = PR_TRUE;
-  }
-  return gOnMacOSX;
-}
-
 
 static void 
 ConvertGeckoToNativeRect(const nsRect& aSrc, Rect& aDst) 
@@ -143,7 +126,8 @@ GetAttribute(nsIFrame* aFrame, nsIAtom* inAttribute, nsCString& outValue)
   nsresult res = content->GetAttr(kNameSpaceID_None, inAttribute, attr);
   outValue = NS_LossyConvertUCS2toASCII(attr).get();
 
-  return ( res != NS_CONTENT_ATTR_NO_VALUE );
+  return ( res != NS_CONTENT_ATTR_NO_VALUE &&
+           !(res != NS_CONTENT_ATTR_NOT_THERE && attr.IsEmpty()));
 }
 
 
@@ -179,6 +163,20 @@ CheckBooleanAttr(nsIFrame* aFrame, nsIAtom* aAtom)
 }
 
 
+//
+// DoNothing
+//
+// An eraseProc for drawing theme buttons so that we don't erase over anything
+// that might be drawn before us in the background layer. Does absolutely
+// nothing.
+//
+static pascal void
+DoNothing(const Rect *bounds, UInt32 eraseData, SInt16 depth, Boolean isColorDev)
+{
+  // be gentle, erase nothing.
+}
+
+
 #ifdef XP_MAC
 #pragma mark -
 #endif
@@ -187,9 +185,12 @@ CheckBooleanAttr(nsIFrame* aFrame, nsIAtom* aAtom)
 NS_IMPL_ISUPPORTS1(nsNativeThemeMac, nsITheme)
 
 nsNativeThemeMac::nsNativeThemeMac()
+  : mEraseProc(nsnull)
 {
   NS_INIT_ISUPPORTS();
 
+  mEraseProc = NewThemeEraseUPP(DoNothing);
+
   mCheckedAtom = do_GetAtom("checked");
   mDisabledAtom = do_GetAtom("disabled");
   mSelectedAtom = do_GetAtom("selected");
@@ -200,11 +201,16 @@ nsNativeThemeMac::nsNativeThemeMac()
   mCurPosAtom = do_GetAtom("curpos");
   mMaxPosAtom = do_GetAtom("maxpos");
   mScrollbarAtom = do_GetAtom("scrollbar");
+  mClassAtom = do_GetAtom("class");
+  mSortDirectionAtom = do_GetAtom("sortDirection");
+  mInputAtom = do_GetAtom("input");
+  mInputCheckedAtom = do_GetAtom("_moz-input-checked");
 }
 
 nsNativeThemeMac::~nsNativeThemeMac()
 {
-
+  if ( mEraseProc )
+    ::DisposeThemeEraseUPP(mEraseProc);
 }
 
 
@@ -224,9 +230,9 @@ nsNativeThemeMac::IsDefaultButton(nsIFrame* aFrame)
 
   
 PRBool
-nsNativeThemeMac::IsChecked(nsIFrame* aFrame)
+nsNativeThemeMac::IsChecked(nsIFrame* aFrame, PRBool aIsHTML)
 {
-  return CheckBooleanAttr(aFrame, mCheckedAtom);
+  return CheckBooleanAttr(aFrame, aIsHTML ? mInputCheckedAtom : mCheckedAtom);
 }
 
 
@@ -237,6 +243,40 @@ nsNativeThemeMac::IsSelected(nsIFrame* aFrame)
 }
 
 
+PRBool
+nsNativeThemeMac::IsSortedColumn(nsIFrame* aFrame)
+{
+  // if the "sortDirection" attribute is set, we're the sorted column
+  nsCAutoString mode;
+  if ( GetAttribute(aFrame, mSortDirectionAtom, mode) )
+    return !mode.IsEmpty();
+  
+  return PR_FALSE;
+}
+
+
+PRBool
+nsNativeThemeMac::IsSortReversed(nsIFrame* aFrame)
+{
+  nsCAutoString mode;
+  if ( GetAttribute(aFrame, mSortDirectionAtom, mode) )
+    return mode.Equals("descending");
+  
+  return PR_FALSE;
+}
+
+
+PRBool
+nsNativeThemeMac::DoTabsPointUp(nsIFrame* aFrame)
+{
+  nsCAutoString mode;
+  if ( GetAttribute(aFrame, mClassAtom, mode) )
+    return mode.Find("tab-bottom") != kNotFound;
+  
+  return PR_FALSE;
+}
+
+
 PRBool
 nsNativeThemeMac::IsIndeterminate(nsIFrame* aFrame)
 {
@@ -270,17 +310,25 @@ nsNativeThemeMac::GetScrollbarParent(nsIFrame* inButton, nsPoint* outOffset)
     // |scrollbar|. If not, keep going up the chain.
     nsCOMPtr<nsIContent> content;
     currFrame->GetContent(getter_AddRefs(content));
-    nsCOMPtr<nsIAtom> tag;
-    content->GetTag(*getter_AddRefs(tag));
-    if ( tag == mScrollbarAtom )
-      found = PR_TRUE;
+    NS_ASSERTION(content, "Couldn't get content from frame, are we in a scrollbar?");
+    if ( content ) {
+      nsCOMPtr<nsIAtom> tag;
+      content->GetTag(*getter_AddRefs(tag));
+      if ( tag == mScrollbarAtom )
+        found = PR_TRUE;
+      else {
+        // drat, add to our offset and check the parent
+        nsPoint offsetFromParent;
+        currFrame->GetOrigin(offsetFromParent);
+        *outOffset += offsetFromParent;
+        
+        currFrame->GetParent(&currFrame);
+      }
+    }
     else {
-      // drat, add to our offset and check the parent
-      nsPoint offsetFromParent;
-      currFrame->GetOrigin(offsetFromParent);
-      *outOffset += offsetFromParent;
-      
-      currFrame->GetParent(&currFrame);
+      // hrm, no content, we're probably not in a scrollbar. just bail
+      currFrame = nsnull;
+      found = PR_TRUE;
     }
   } while ( !found && currFrame );
     
@@ -359,7 +407,8 @@ nsNativeThemeMac::DrawRadio ( const Rect& inBoxRect, PRBool inChecked, PRBool in
 
 void
 nsNativeThemeMac::DrawButton ( ThemeButtonKind inKind, const Rect& inBoxRect, PRBool inIsDefault, 
-                                  PRBool inDisabled, PRInt32 inState )
+                                  PRBool inDisabled, ThemeButtonValue inValue, ThemeButtonAdornment inAdornment,
+                                  PRInt32 inState )
 {
   ThemeButtonDrawInfo info;
   if ( inDisabled )
@@ -367,20 +416,28 @@ nsNativeThemeMac::DrawButton ( ThemeButtonKind inKind, const Rect& inBoxRect, PR
   else
     info.state = ((inState & NS_EVENT_STATE_ACTIVE) && (inState & NS_EVENT_STATE_HOVER)) ? 
                     kThemeStatePressed : kThemeStateActive;
-  info.value = kThemeButtonOn;
-  info.adornment = kThemeAdornmentNone;
+  info.value = inValue;
+  info.adornment = inAdornment;
   if ( inState & NS_EVENT_STATE_FOCUS )    
     info.adornment = kThemeAdornmentFocus;
   if ( inIsDefault )
     info.adornment |= kThemeAdornmentDefault;
 
-  ::DrawThemeButton ( &inBoxRect, inKind, &info, nsnull, nsnull, nsnull, 0L );
+  ::DrawThemeButton ( &inBoxRect, inKind, &info, nsnull, mEraseProc, nsnull, 0L );
 }
 
 
 void
 nsNativeThemeMac::DrawToolbar ( const Rect& inBoxRect )
 {
+#if 0
+  const PRInt32 kThemeBrushToolbarBackground = 52;    // from 3.4.1 headers
+  ::SetThemeBackground(kThemeBrushToolbarBackground, 24, true);
+  ::EraseRect(&inBoxRect);
+  ::SetThemeBackground(kThemeBrushWhite, 24, true);
+printf("told to draw at %ld %ld w %ld h %ld\n", inBoxRect.left, inBoxRect.top, inBoxRect.right-inBoxRect.left,
+        inBoxRect.bottom - inBoxRect.top);
+#endif
   ThemeDrawState drawState = kThemeStateActive;
   ::DrawThemeWindowHeader(&inBoxRect, drawState);
 }
@@ -395,7 +452,7 @@ nsNativeThemeMac::DrawEditText ( const Rect& inBoxRect, PRBool inIsDisabled )
   ::BackPat(GetQDGlobalsWhite(&whitePat));
   ::EraseRect(&inBoxRect);
   
-  ThemeDrawState drawState = inIsDisabled ? kThemeStateActive : kThemeStateDisabled;
+  ThemeDrawState drawState = inIsDisabled ? kThemeStateDisabled : kThemeStateActive;
   ::DrawThemeEditTextFrame(&inBoxRect, drawState);
 #endif
 }
@@ -410,7 +467,7 @@ nsNativeThemeMac::DrawListBox ( const Rect& inBoxRect, PRBool inIsDisabled )
   ::BackPat(GetQDGlobalsWhite(&whitePat));
   ::EraseRect(&inBoxRect);
   
-  ThemeDrawState drawState = inIsDisabled ? kThemeStateActive : kThemeStateDisabled;
+  ThemeDrawState drawState = inIsDisabled ? kThemeStateDisabled : kThemeStateActive;
   ::DrawThemeListBoxFrame(&inBoxRect, drawState);
 #endif
 }
@@ -439,14 +496,22 @@ nsNativeThemeMac::DrawProgress ( const Rect& inBoxRect, PRBool inIsDisabled, PRB
 void
 nsNativeThemeMac::DrawTabPanel ( const Rect& inBoxRect, PRBool inIsDisabled )
 {
-  ThemeDrawState drawState = inIsDisabled ? kThemeStateActive : kThemeStateDisabled;
-  ::DrawThemeTabPane(&inBoxRect, kThemeStateActive);
+  ThemeDrawState drawState = inIsDisabled ? kThemeStateDisabled : kThemeStateActive;
+  ::DrawThemeTabPane(&inBoxRect, drawState);
+}
+
+
+void
+nsNativeThemeMac::DrawSeparator ( const Rect& inBoxRect, PRBool inIsDisabled )
+{
+  ThemeDrawState drawState = inIsDisabled ? kThemeStateDisabled : kThemeStateActive;
+  ::DrawThemeSeparator(&inBoxRect, drawState);
 }
 
 
 void
 nsNativeThemeMac::DrawTab ( const Rect& inBoxRect, PRBool inIsDisabled, PRBool inIsFrontmost, 
-                              PRBool inIsHorizontal, PRInt32 inState )
+                              PRBool inIsHorizontal, PRBool inTabBottom, PRInt32 inState )
 {
   ThemeTabStyle style = 0L;
   if ( inIsFrontmost ) {
@@ -464,7 +529,8 @@ nsNativeThemeMac::DrawTab ( const Rect& inBoxRect, PRBool inIsDisabled, PRBool i
       style = kThemeTabNonFront;  
   }
 
-  ::DrawThemeTab(&inBoxRect, style, kThemeTabNorth, nsnull, 0L);
+  ThemeTabDirection direction = inTabBottom ? kThemeTabSouth : kThemeTabNorth; // don't yet support vertical tabs
+  ::DrawThemeTab(&inBoxRect, style, direction, nsnull, 0L);
 }
 
 
@@ -480,15 +546,33 @@ nsNativeThemeMac::DrawFullScrollbar ( const Rect& inSbarRect, PRInt32 inWidgetHi
                                        PRBool inIsDisabled, PRInt32 inMax, PRInt32 inValue, PRInt32 inState )
 {
   ThemeTrackDrawInfo info;
+  static Rect lastScrollbarDrawn = {0, 0, 0, 0};
+  static WindowRef lastScrollbarWindow = nsnull;
 
+  // this is a serious, serious hack, but we seem to have some coordinate rounding problems
+  // that I really can't figure out. I'll take 40 whacks for this, and even buy liquor to
+  // the person that can fix it. The problem comes from the fact that we're determining the
+  // location of the scrollbar's frame by walking up the frame parent chain from a child
+  // of the scrollbar. Sometimes (not always) a rounding error in gfx occurs and we end up with the
+  // bottom of the scrollbar being off by a pixel or two from the last time we drew it. 
+  // Rather nasty jiggling ensues as we draw different pieces of the scrollbar with and 
+  // without the rounding error. The only solution i could come up with was to remember where
+  // we last drew, and if it's a close enough match, revert back to the last scrollbar rect. Seems
+  // to work pretty well.
+  Rect scrollbarRect = inSbarRect;
+  if ( ::FrontWindow() == lastScrollbarWindow && 
+        abs(scrollbarRect.bottom - lastScrollbarDrawn.bottom) <= 2 &&
+              scrollbarRect.left == lastScrollbarDrawn.left )
+    scrollbarRect = lastScrollbarDrawn;
+  
   // the scrollbar is horizontal if the width is greater than the height. Too bad the API
   // doesn't tell us which is which.
   PRBool isHorizontal = 
-          (inSbarRect.right - inSbarRect.left) > (inSbarRect.bottom - inSbarRect.top);
+          (scrollbarRect.right - scrollbarRect.left) > (scrollbarRect.bottom - scrollbarRect.top);
    
   // compute the number of lines in our view. It's probably safe to assume that
   // the height of the scrollbar is the height of the scrollable view
-  PRInt32 viewSize = isHorizontal ? (inSbarRect.right - inSbarRect.left) : (inSbarRect.bottom - inSbarRect.top);
+  PRInt32 viewSize = isHorizontal ? (scrollbarRect.right - scrollbarRect.left) : (scrollbarRect.bottom - scrollbarRect.top);
   viewSize /= inLineHeight;
 
   // Figure out if something should be drawn depressed
@@ -510,7 +594,7 @@ nsNativeThemeMac::DrawFullScrollbar ( const Rect& inSbarRect, PRInt32 inWidgetHi
 //XXX this is true for all controls, but scrollbars are the ones you notice the most
 
   info.kind = kThemeMediumScrollBar;
-  info.bounds = inSbarRect;
+  info.bounds = scrollbarRect;
   info.min = 0;
   info.max = inMax;
   info.value = inValue;
@@ -522,11 +606,17 @@ nsNativeThemeMac::DrawFullScrollbar ( const Rect& inSbarRect, PRInt32 inWidgetHi
   
   ::DrawThemeTrack(&info, nsnull, nsnull, 0L);
 
-#ifdef DEBUG_PINK
+  // update our statics for hack detection
+  lastScrollbarDrawn = scrollbarRect;
+  lastScrollbarWindow = ::FrontWindow();
+  
+//#ifdef DEBUG_PINK
+#if 1
   // some debug info for helping diagnose problems
   printf("--- BEGIN scrollbar debug info\n");
   printf("-- widget drawn is %ld\n", inWidgetHit);
   printf("bounds (%ld, %ld), (%ld, %ld)\n",inSbarRect.left, inSbarRect.top, inSbarRect.right, inSbarRect.bottom );
+  printf("bounds (%ld, %ld), (%ld, %ld) adjusted\n",scrollbarRect.left, scrollbarRect.top, scrollbarRect.right, scrollbarRect.bottom );
   if ( isHorizontal )
     printf("horizontal\n");
   else
@@ -575,29 +665,63 @@ nsNativeThemeMac::DrawWidgetBackground(nsIRenderingContext* aContext, nsIFrame*
   ::ClipRect(&clipRect);
 #endif
   
+  PRBool isHTML = PR_FALSE;
   // for some widgets, the parent determines the appropriate state. grab the parent instead.
   if ( aWidgetType == NS_THEME_CHECKBOX || aWidgetType == NS_THEME_RADIO ) {
     nsCOMPtr<nsIContent> content;
     aFrame->GetContent(getter_AddRefs(content));
     if (content->IsContentOfType(nsIContent::eXUL))
       aFrame->GetParent(&aFrame);
+    else {
+      nsCOMPtr<nsIAtom> tag;
+      content->GetTag(*getter_AddRefs(tag));
+      if (tag == mInputAtom)
+        isHTML = PR_TRUE;
+    }
   }
   
   PRInt32 eventState = GetContentState(aFrame);
 
   switch ( aWidgetType ) {
   
+    case NS_THEME_DIALOG:
+      ::SetThemeBackground(kThemeBrushDialogBackgroundActive, 24, true);
+      ::EraseRect(&macRect);
+      ::SetThemeBackground(kThemeBrushWhite, 24, true);
+      break;
+      
+    case NS_THEME_MENU:
+    printf("!!! draw menu bg\n");
+      ::SetThemeBackground(kThemeBrushDialogBackgroundActive, 24, true);
+      ::EraseRect(&macRect);
+      ::SetThemeBackground(kThemeBrushWhite, 24, true);
+      break;
+
+    case NS_THEME_TOOLTIP:
+    {
+      RGBColor yellow = {65535,65535,45000};
+      ::RGBBackColor(&yellow);
+      ::EraseRect(&macRect);
+      ::SetThemeBackground(kThemeBrushWhite, 24, true);
+      break;
+    }
+
     case NS_THEME_CHECKBOX:
-      DrawCheckbox ( macRect, IsChecked(aFrame), IsDisabled(aFrame), eventState );
+      DrawCheckbox ( macRect, IsChecked(aFrame, isHTML), IsDisabled(aFrame), eventState );
       break;    
     case NS_THEME_RADIO:
       DrawRadio ( macRect, IsSelected(aFrame), IsDisabled(aFrame), eventState );
       break;
     case NS_THEME_BUTTON:
-      DrawButton ( kThemePushButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame), eventState );
+      DrawButton ( kThemePushButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame), 
+                    kThemeButtonOn, kThemeAdornmentNone, eventState );
       break;      
     case NS_THEME_TOOLBAR_BUTTON:
-      DrawButton ( kThemePushButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame), eventState );
+      DrawButton ( kThemePushButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame),
+                    kThemeButtonOn, kThemeAdornmentNone, eventState );
+      break;
+    case NS_THEME_TOOLBAR_SEPARATOR:
+      DrawSeparator ( macRect, IsDisabled(aFrame) );
       break;
       
     case NS_THEME_TOOLBAR:
@@ -607,7 +731,8 @@ nsNativeThemeMac::DrawWidgetBackground(nsIRenderingContext* aContext, nsIFrame*
       break;
       
     case NS_THEME_DROPDOWN:
-      DrawButton ( kThemePopupButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame), eventState );
+      DrawButton ( kThemePopupButton, macRect, IsDefaultButton(aFrame), IsDisabled(aFrame), 
+                    kThemeButtonOn, kThemeAdornmentNone, eventState );
       break;
     case NS_THEME_DROPDOWN_BUTTON:
       // do nothing, this is covered by the DROPDOWN case
@@ -629,9 +754,32 @@ nsNativeThemeMac::DrawWidgetBackground(nsIRenderingContext* aContext, nsIFrame*
       break;
 
     case NS_THEME_TREEVIEW_TWISTY:
-      DrawButton ( kThemeDisclosureButton, macRect, PR_FALSE, IsDisabled(aFrame), eventState );      
+      DrawButton ( kThemeDisclosureButton, macRect, PR_FALSE, IsDisabled(aFrame), 
+                    kThemeDisclosureRight, kThemeAdornmentNone, eventState );
       break;
-   
+    case NS_THEME_TREEVIEW_TWISTY_OPEN:
+      DrawButton ( kThemeDisclosureButton, macRect, PR_FALSE, IsDisabled(aFrame), 
+                    kThemeDisclosureDown, kThemeAdornmentNone, eventState );
+      break;
+    case NS_THEME_TREEVIEW_HEADER_CELL:
+      DrawButton ( kThemeListHeaderButton, macRect, PR_FALSE, IsDisabled(aFrame), 
+                    IsSortedColumn(aFrame) ? kThemeButtonOn : kThemeButtonOff,
+                    IsSortReversed(aFrame) ? kThemeAdornmentHeaderButtonSortUp : kThemeAdornmentNone,
+                    eventState );      
+      break;
+    case NS_THEME_TREEVIEW_TREEITEM:
+    case NS_THEME_TREEVIEW:
+      ::SetThemeBackground(kThemeBrushWhite, 24, true);
+      ::EraseRect ( &macRect );
+      break;
+    case NS_THEME_TREEVIEW_HEADER:
+      // do nothing, taken care of by individual header cells
+    case NS_THEME_TREEVIEW_HEADER_SORTARROW:
+      // do nothing, taken care of by treeview header
+    case NS_THEME_TREEVIEW_LINE:
+      // do nothing, these lines don't exist on macos
+      break;
+       
 #if 0
     case NS_THEME_SCROLLBAR:
       break;
@@ -670,6 +818,10 @@ nsNativeThemeMac::DrawWidgetBackground(nsIRenderingContext* aContext, nsIFrame*
           maxPos = 1;
         PRInt32 curPos = CheckIntAttr(scrollbar, mCurPosAtom) / kLineHeight;
         
+#if 0
+        ++macScrollbarRect.left;
+        ++macScrollbarRect.bottom;
+#endif
         DrawFullScrollbar ( macScrollbarRect, aWidgetType, kLineHeight, IsDisabled(aFrame),
                               maxPos, curPos, eventState);
       }
@@ -681,9 +833,8 @@ nsNativeThemeMac::DrawWidgetBackground(nsIRenderingContext* aContext, nsIFrame*
       break;
     
     case NS_THEME_TAB:
-      DrawTab(macRect, IsDisabled(aFrame), IsSelected(aFrame), PR_TRUE, eventState);
-      break;
-      
+      DrawTab(macRect, IsDisabled(aFrame), IsSelected(aFrame), PR_TRUE, DoTabsPointUp(aFrame), eventState);
+      break;      
     case NS_THEME_TAB_PANELS:
       DrawTabPanel(macRect, IsDisabled(aFrame));
       break;
@@ -714,8 +865,9 @@ nsNativeThemeMac::GetWidgetBorder(nsIDeviceContext* aContext,
   switch ( aWidgetType ) {
   
     case NS_THEME_BUTTON:
-      if ( OnMacOSX() )
-        aResult->SizeTo(14,2,14,2);   // 14px left/right endcaps in aqua.
+      if ( nsRenderingContextMac::OnMacOSX() )
+        aResult->SizeTo(kAquaPushButtonEndcaps, kAquaPushButtonTopBottom, 
+                            kAquaPushButtonEndcaps, kAquaPushButtonTopBottom);
       else
         aResult->SizeTo(5,2,5,2);     // 5px for AGA
       break;
@@ -725,8 +877,9 @@ nsNativeThemeMac::GetWidgetBorder(nsIDeviceContext* aContext,
       break;
 
     case NS_THEME_DROPDOWN:
-      if ( OnMacOSX() )
-        aResult->SizeTo(9,2,20,2);     // 9px left endcaps in aqua, bigger right border for button
+      if ( nsRenderingContextMac::OnMacOSX() )
+        aResult->SizeTo(kAquaDropdownLeftEndcap, kAquaPushButtonTopBottom, 
+                          kAquaDropwdonRightEndcap, kAquaPushButtonTopBottom);
       else
         aResult->SizeTo(3,0,3,0);     // 3px for AGA
       break;
@@ -767,6 +920,14 @@ nsNativeThemeMac::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
 #if TARGET_CARBON
   switch ( aWidgetType ) {
   
+    case NS_THEME_BUTTON:
+    {
+      SInt32 buttonHeight = 0;
+      ::GetThemeMetric(kThemeMetricPushButtonHeight, &buttonHeight);
+      aResult->SizeTo(kAquaPushButtonEndcaps*2, buttonHeight);
+      break;
+    }
+      
     case NS_THEME_CHECKBOX:
     {
       SInt32 boxHeight = 0, boxWidth = 0;
@@ -802,8 +963,14 @@ nsNativeThemeMac::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
       break;
       
     case NS_THEME_TEXTFIELD:
-      //XXX ?
+    {
+      // at minimum, we should be tall enough for 9pt text.
+      SInt32 shadow = 0, frameOutset = 0;
+      ::GetThemeMetric(kThemeMetricEditTextWhitespace, &shadow);
+      ::GetThemeMetric(kThemeMetricEditTextFrameOutset, &frameOutset);
+      aResult->SizeTo(0, (shadow + frameOutset) * 2 + 9);      
       break;
+    }
       
     case NS_THEME_PROGRESSBAR:
     {
@@ -814,6 +981,7 @@ nsNativeThemeMac::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
     }
 
     case NS_THEME_TREEVIEW_TWISTY:
+    case NS_THEME_TREEVIEW_TWISTY_OPEN:   
     {
       SInt32 twistyHeight = 0, twistyWidth = 0;
       ::GetThemeMetric(kThemeMetricDisclosureButtonWidth, &twistyWidth);
@@ -823,6 +991,15 @@ nsNativeThemeMac::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
       break;
     }
     
+    case NS_THEME_TREEVIEW_HEADER:
+    case NS_THEME_TREEVIEW_HEADER_CELL:
+    {
+      SInt32 headerHeight = 0;
+      ::GetThemeMetric(kThemeMetricListHeaderHeight, &headerHeight);
+      aResult->SizeTo(0, headerHeight);
+      break;
+    }
+      
     case NS_THEME_SCROLLBAR:
     case NS_THEME_SCROLLBAR_BUTTON_UP:
     case NS_THEME_SCROLLBAR_BUTTON_DOWN:
@@ -842,6 +1019,7 @@ nsNativeThemeMac::GetMinimumWidgetSize(nsIRenderingContext* aContext, nsIFrame*
       SInt32 scrollbarWidth = 0;
       ::GetThemeMetric(kThemeMetricScrollBarWidth, &scrollbarWidth);
       aResult->SizeTo(scrollbarWidth, scrollbarWidth);
+      *aIsOverridable = PR_FALSE;
       break;
     }
 
@@ -860,6 +1038,7 @@ nsNativeThemeMac::WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType,
   switch ( aWidgetType ) {
     case NS_THEME_TOOLBOX:
     case NS_THEME_TOOLBAR:
+    case NS_THEME_TOOLBAR_BUTTON:
     case NS_THEME_SCROLLBAR_TRACK_VERTICAL: 
     case NS_THEME_SCROLLBAR_TRACK_HORIZONTAL:
     case NS_THEME_STATUSBAR:
@@ -873,6 +1052,8 @@ nsNativeThemeMac::WidgetStateChanged(nsIFrame* aFrame, PRUint8 aWidgetType,
     case NS_THEME_TAB_PANELS:
     case NS_THEME_TAB_PANEL:
     case NS_THEME_TEXTFIELD:
+    case NS_THEME_DIALOG:
+    case NS_THEME_MENU:
       *aShouldRepaint = PR_FALSE;
       return NS_OK;
   }
@@ -913,6 +1094,11 @@ nsNativeThemeMac::ThemeSupportsWidget(nsIPresContext* aPresContext,
   PRBool retVal = PR_FALSE;
   
   switch ( aWidgetType ) {
+    case NS_THEME_DIALOG:
+    case NS_THEME_WINDOW:
+    case NS_THEME_MENU:
+    case NS_THEME_TOOLTIP:
+    
     case NS_THEME_CHECKBOX:
     case NS_THEME_CHECKBOX_CONTAINER:
     case NS_THEME_RADIO:
@@ -930,7 +1116,7 @@ nsNativeThemeMac::ThemeSupportsWidget(nsIPresContext* aPresContext,
     case NS_THEME_PROGRESSBAR_VERTICAL:
     case NS_THEME_PROGRESSBAR_CHUNK:
     case NS_THEME_PROGRESSBAR_CHUNK_VERTICAL:
-    case NS_THEME_TREEVIEW_TWISTY:
+    case NS_THEME_TOOLBAR_SEPARATOR:
     
     case NS_THEME_LISTBOX:
     
@@ -939,6 +1125,15 @@ nsNativeThemeMac::ThemeSupportsWidget(nsIPresContext* aPresContext,
     case NS_THEME_TAB_LEFT_EDGE:
     case NS_THEME_TAB_RIGHT_EDGE:
     
+    case NS_THEME_TREEVIEW_TWISTY:
+    case NS_THEME_TREEVIEW_TWISTY_OPEN:
+    case NS_THEME_TREEVIEW:
+    case NS_THEME_TREEVIEW_HEADER:
+    case NS_THEME_TREEVIEW_HEADER_CELL:
+    case NS_THEME_TREEVIEW_HEADER_SORTARROW:
+    case NS_THEME_TREEVIEW_TREEITEM:
+    case NS_THEME_TREEVIEW_LINE:
+    
     case NS_THEME_SCROLLBAR:
     case NS_THEME_SCROLLBAR_BUTTON_UP:
     case NS_THEME_SCROLLBAR_BUTTON_DOWN:
@@ -951,7 +1146,7 @@ nsNativeThemeMac::ThemeSupportsWidget(nsIPresContext* aPresContext,
     case NS_THEME_SCROLLBAR_TRACK_VERTICAL:
     case NS_THEME_SCROLLBAR_TRACK_HORIZONTAL:
       // for now, only use on osx since i haven't yet verified on os9
-      if ( OnMacOSX() )
+      if ( nsRenderingContextMac::OnMacOSX() )
         retVal = PR_TRUE;
       break;
   
