@@ -58,7 +58,7 @@
 
 #include <FixMath.h>
 #include <Gestalt.h>
-#include <QuickDraw.h>
+#include <Quickdraw.h>
 
 #include "nsRegionPool.h"
 #include "nsFontUtils.h"
@@ -216,7 +216,12 @@ void nsRenderingContextMac::SelectDrawingSurface(nsDrawingSurfaceMac* aSurface,
 
   nsGraphicsUtils::SafeSetPort(mPort);
 
+#ifndef MOZ_WIDGET_COCOA
+  // Cocoa widgets automatically set the correct origin, and 
+  // it can differ from this value (because of an oddity where
+  // the widget shrinks when its partially offscreen).  
 	::SetOrigin(-mGS->mOffx, -mGS->mOffy);		// line order...
+#endif
 
 	if (aChanges & kClippingChanged)
 		::SetClip(mGS->mClipRegion);			// ...does matter
@@ -282,7 +287,9 @@ nsresult nsRenderingContextMac::SetPortTextState()
 void nsRenderingContextMac::SetupPortState()
 {
   nsGraphicsUtils::SafeSetPort(mPort);
+#ifndef MOZ_WIDGET_COCOA
 	::SetOrigin(-mGS->mOffx, -mGS->mOffy);
+#endif
 	::SetClip(mGS->mClipRegion);
 }
 
@@ -556,7 +563,7 @@ NS_IMETHODIMP nsRenderingContextMac::GetHints(PRUint32& aResult)
 {
 	PRUint32 result = 0;
 
-	// QuickDraw is prefered over to ATSUI for drawing 7-bit text
+	// QuickDraw is preferred over to ATSUI for drawing 7-bit text
 	// (it's not 8-bit: the name of the constant is misleading)
 	result |= NS_RENDERING_HINT_FAST_8BIT_TEXT;
 	aResult = result;
@@ -755,9 +762,9 @@ NS_IMETHODIMP nsRenderingContextMac::SetColor(nscolor aColor)
 	#define COLOR8TOCOLOR16(color8)	 ((color8 << 8) | color8)
 
 	RGBColor color;
-	color.red = COLOR8TOCOLOR16(NS_GET_R(aColor));
+	color.red   = COLOR8TOCOLOR16(NS_GET_R(aColor));
 	color.green = COLOR8TOCOLOR16(NS_GET_G(aColor));
-	color.blue = COLOR8TOCOLOR16(NS_GET_B(aColor));
+	color.blue  = COLOR8TOCOLOR16(NS_GET_B(aColor));
 	::RGBForeColor(&color);
 	mGS->mColor = aColor ;
 
@@ -794,13 +801,13 @@ NS_IMETHODIMP nsRenderingContextMac::GetLineStyle(nsLineStyle &aLineStyle)
 
 //------------------------------------------------------------------------
 
-NS_IMETHODIMP nsRenderingContextMac::SetFont(const nsFont& aFont)
+NS_IMETHODIMP nsRenderingContextMac::SetFont(const nsFont& aFont, nsIAtom* aLangGroup)
 {
 	NS_IF_RELEASE(mGS->mFontMetrics);
 
 	if (mContext)
-		mContext->GetMetricsFor(aFont, mGS->mFontMetrics);
-		
+		mContext->GetMetricsFor(aFont, aLangGroup, mGS->mFontMetrics);
+
 	mChanges |= kFontChanged;
 		
 	return NS_OK;
@@ -1482,6 +1489,37 @@ NS_IMETHODIMP nsRenderingContextMac::InvertRect(nscoord aX, nscoord aY, nscoord
 	return NS_OK;
 }
 
+NS_IMETHODIMP
+nsRenderingContextMac::FlushRect(const nsRect& aRect)
+{
+  return FlushRect(aRect.x, aRect.y, aRect.width, aRect.height);
+}
+
+NS_IMETHODIMP
+nsRenderingContextMac::FlushRect(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight)
+{
+#ifdef MOZ_WIDGET_COCOA
+    if (mPort) {
+        SetupPortState();
+
+        nscoord x,y,w,h;
+        Rect	therect;
+    
+        x = aX;
+        y = aY;
+        w = aWidth;
+        h = aHeight;
+    
+        mGS->mTMatrix.TransformCoord(&x, &y, &w, &h);
+        RgnHandle rgn = ::NewRgn();
+        ::SetRectRgn(rgn, pinToShort(x), pinToShort(y), pinToShort(x + w), pinToShort(y + h));
+        ::QDFlushPortBuffer(mPort, rgn);
+        ::DisposeRgn(rgn);
+    }
+#endif
+    return NS_OK;
+}
+
 NS_IMETHODIMP nsRenderingContextMac::GetGraphics(nsIGraphics* *aGraphics)
 {
 	NS_ASSERTION((aGraphics != nsnull), "aGraphics is NULL");
@@ -1533,3 +1571,30 @@ nsRenderingContextMac::ReleaseBackbuffer(void)
   SelectOffScreenDrawingSurface(nsnull);
   return NS_OK;
 }
+
+
+// override to not use a back buffer on MacOS X
+NS_IMETHODIMP 
+nsRenderingContextMac::UseBackbuffer(PRBool* aUseBackbuffer)
+{
+  *aUseBackbuffer = !OnMacOSX();
+  return NS_OK;
+}
+
+
+//
+// Return true if we are on Mac OS X, caching the result after the first call
+//
+PRBool
+nsRenderingContextMac::OnMacOSX()
+{
+  static PRBool gInitVer = PR_FALSE;
+  static PRBool gOnMacOSX = PR_FALSE;
+  if(! gInitVer) {
+    long version;
+    OSErr err = ::Gestalt(gestaltSystemVersion, &version);
+    gOnMacOSX = (err == noErr && version >= 0x00001000);
+    gInitVer = PR_TRUE;
+  }
+  return gOnMacOSX;
+}
