@@ -50,10 +50,12 @@
 #include "nsMacUnicodeFontInfo.h"
 #include "nsICharRepresentable.h"
 
+#include <FixMath.h>
+
 
 #define BAD_FONT_NUM -1
 #define BAD_SCRIPT 0x7F
-#define STACK_TREASHOLD 1000
+#define STACK_TRESHOLD 1000
 static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
 
 //#define DISABLE_TEC_FALLBACK
@@ -127,43 +129,17 @@ static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
   )
 #define BAD_TEXT_ENCODING 0xFFFFFFFF
 
+// all the chracter should not be drawn if there are no glyph
+#define IS_ZERO_WIDTH_CHAR(c) ( \
+  IN_RANGE(c, 0x200b, 0x200f) || \
+  IN_RANGE(c, 0x202a, 0x202e) \
+  )
+  
 
-
+#define IN_ARABIC_PRESENTATION_A(a) ((0xfb50 <= (a)) && ((a) <= 0xfdff))
 #define IN_ARABIC_PRESENTATION_B(a) ((0xfe70 <= (a)) && ((a) <= 0xfeff))
+#define IN_ARABIC_PRESENTATION_A_OR_B(a) (IN_ARABIC_PRESENTATION_A(a) || IN_ARABIC_PRESENTATION_B(a))
 
-//------------------------------------------------------------------------
-static UnicodeToTextInfo gConverters[32] = { 
-	nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull,
-	nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull,
-	nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull,
-	nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull, nsnull
-};
-//------------------------------------------------------------------------
-UnicodeToTextInfo nsUnicodeRenderingToolkit :: GetConverterByScript(ScriptCode sc)
-{
-  // because the Mac QuickDraw BIDI support are quite different from other platform
-  // we try not to use them and use the XP BIDI feature
-  // those text will be drawn by ATSUI intead one character at a time
-  if ((sc == smArabic) || (sc == smHebrew))
-     return nsnull;
-	NS_PRECONDITION(sc < 32, "illegal script id");
-  if(sc >= 32)
-    return nsnull;
-  if (gConverters[sc] != nsnull) {
-    return gConverters[sc];
-  }
-  OSStatus err = noErr;
-    
-  //
-  TextEncoding scriptEncoding;
-  err = ::UpgradeScriptInfoToTextEncoding(sc, kTextLanguageDontCare, kTextRegionDontCare, nsnull, &scriptEncoding);
-  if ( noErr == err ) 
- 	  err = ::CreateUnicodeToTextInfoByEncoding(scriptEncoding, &gConverters[sc] );
-
-  if (noErr != err) 
-    gConverters[sc] = nsnull;
-  return gConverters[sc];
-}
 //------------------------------------------------------------------------
 #pragma mark -
 //------------------------------------------------------------------------
@@ -183,9 +159,8 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
   const PRUnichar *aCharPt, 
   nsTextDimensions& oDim, 
   short origFontNum, 
-  const short* scriptFallbackFonts)
+  nsUnicodeFontMappingMac& fontMapping)
 {
-  OSStatus err = noErr;
   char buf[20];
   ByteCount processBytes = 0;
   ByteCount outLen = 0;
@@ -193,6 +168,7 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
   nsUnicodeFallbackCache* cache = GetTECFallbackCache();
   short aWidth;
   FontInfo finfo;
+  const short *scriptFallbackFonts = fontMapping.GetScriptFallbackFonts();
   
   if ((0xf780 <= *aCharPt) && (*aCharPt <= 0xf7ff))
   {
@@ -212,17 +188,10 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
   {
     if (BAD_SCRIPT == fallbackScript)
       return PR_FALSE;
-      
-    UnicodeToTextInfo fallbackConverter = GetConverterByScript(fallbackScript);
-    if (fallbackConverter) 
-    {
-      err = ::ConvertFromUnicodeToText(fallbackConverter, (ByteCount)(2), 
-                                       (ConstUniCharArrayPtr) aCharPt, 
-                                        kUnicodeLooseMappingsMask , 0, NULL, 0, NULL,
-                                        STACK_TREASHOLD, &processBytes, &outLen, 
-                                       (LogicalAddress)buf);
-      if (outLen > 0) 
-      {
+    
+    if(fontMapping.ConvertUnicodeToGlyphs(scriptFallbackFonts[fallbackScript], aCharPt, 1,
+        buf, STACK_TRESHOLD, outLen, processBytes, kUnicodeLooseMappingsMask))
+    {  
         ::TextFont(scriptFallbackFonts[fallbackScript]);
         GetScriptTextWidth(buf, outLen, aWidth);
         ::GetFontInfo(&finfo);
@@ -230,7 +199,6 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
         oDim.descent = finfo.descent;
         oDim.width = aWidth;
         ::TextFont(origFontNum);
-      }
     }
     return PR_TRUE;
   }
@@ -239,27 +207,19 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
   {
     if (BAD_FONT_NUM != scriptFallbackFonts[fallbackScript])
     {
-      UnicodeToTextInfo fallbackConverter = GetConverterByScript(fallbackScript);
-      if (fallbackConverter) 
-      {
-        err = ::ConvertFromUnicodeToText(fallbackConverter, (ByteCount)(2), 
-                                         (ConstUniCharArrayPtr) aCharPt, 
-                                         kUnicodeLooseMappingsMask , 0, NULL, 0, NULL,
-                                         STACK_TREASHOLD, &processBytes, &outLen, 
-                                         (LogicalAddress)buf);
-        if (outLen > 0)
+        if(fontMapping.ConvertUnicodeToGlyphs(scriptFallbackFonts[fallbackScript], aCharPt, 1,
+            buf, STACK_TRESHOLD, outLen, processBytes, kUnicodeLooseMappingsMask))
         {
-          NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
-          ::TextFont(scriptFallbackFonts[fallbackScript]);
-          GetScriptTextWidth(buf, outLen, aWidth);
-          ::GetFontInfo(&finfo);
-          oDim.ascent = finfo.ascent;
-          oDim.descent = finfo.descent;
-          oDim.width = aWidth;
-          ::TextFont(origFontNum);        
-          break;
-        }                                      
-      }          
+            NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
+            ::TextFont(scriptFallbackFonts[fallbackScript]);
+            GetScriptTextWidth(buf, outLen, aWidth);
+            ::GetFontInfo(&finfo);
+            oDim.ascent = finfo.ascent;
+            oDim.descent = finfo.descent;
+            oDim.width = aWidth;
+            ::TextFont(origFontNum);        
+            break;
+        }          
     }
   }
   
@@ -273,20 +233,84 @@ nsUnicodeRenderingToolkit::TECFallbackGetDimensions(
 }
 //------------------------------------------------------------------------
 
+#if MOZ_MATHML
+PRBool nsUnicodeRenderingToolkit::TECFallbackGetBoundingMetrics(
+    const PRUnichar *aCharPt,
+    nsBoundingMetrics& oBoundingMetrics,
+    short fontNum,
+    nsUnicodeFontMappingMac& fontMapping)
+{
+    char buf[STACK_TRESHOLD];
+    ByteCount processBytes = 0;
+    ByteCount outLen = 0;
+    ScriptCode fallbackScript;
+    nsUnicodeFallbackCache* cache = GetTECFallbackCache();
+    const short *scriptFallbackFonts = fontMapping.GetScriptFallbackFonts();
+    
+    if((0xf780 <= *aCharPt) && (*aCharPt <= 0xf7ff))
+    {
+        // If we are encountering our PUA characters for User-Defined characters, we better
+        // just drop the high-byte and return the width for the low-byte.
+        *buf = (*aCharPt & 0x00FF);
+        GetScriptTextBoundingMetrics(buf, 1, ::FontToScript(fontNum), oBoundingMetrics);
+        return PR_TRUE;
+    }
+    else if(cache->Get(*aCharPt, fallbackScript))
+    {
+        if(BAD_SCRIPT == fallbackScript)
+            return PR_FALSE;
+        
+        if(fontMapping.ConvertUnicodeToGlyphs(scriptFallbackFonts[fallbackScript], aCharPt, 1,
+            buf, STACK_TRESHOLD, outLen, processBytes, kUnicodeLooseMappingsMask))
+        {  
+            ::TextFont(scriptFallbackFonts[fallbackScript]);
+            GetScriptTextBoundingMetrics(buf, outLen, fallbackScript, oBoundingMetrics);
+            ::TextFont(fontNum);
+        }
+        return PR_TRUE;
+    }
+    
+    for(fallbackScript = 0; fallbackScript < 32; fallbackScript++)
+    {
+        if(BAD_FONT_NUM != scriptFallbackFonts[fallbackScript])
+        {
+            if(fontMapping.ConvertUnicodeToGlyphs(scriptFallbackFonts[fallbackScript], aCharPt, 1,
+                buf, STACK_TRESHOLD, outLen, processBytes, kUnicodeLooseMappingsMask))
+            {
+                NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
+                ::TextFont(scriptFallbackFonts[fallbackScript]);
+                GetScriptTextBoundingMetrics(buf, outLen, fallbackScript, oBoundingMetrics);           
+                ::TextFont(fontNum);
+                break;
+            }   
+        }
+    }
+    
+    if(0 == outLen)
+        fallbackScript = BAD_SCRIPT;
+        
+    // put into cache
+    cache->Set(*aCharPt, fallbackScript);
+    
+    return (BAD_SCRIPT != fallbackScript);
+}
+#endif // MOZ_MATHML
+//------------------------------------------------------------------------
+
 PRBool nsUnicodeRenderingToolkit :: TECFallbackDrawChar(
   const PRUnichar *aCharPt, 
   PRInt32 x, 
   PRInt32 y, 
   short& oWidth, 
   short origFontNum, 
-  const short* scriptFallbackFonts)
+  nsUnicodeFontMappingMac& fontMapping)
 {
-  OSStatus err = noErr;
   char buf[20];
   ByteCount processBytes = 0;
   ByteCount outLen = 0;
   ScriptCode fallbackScript;
   nsUnicodeFallbackCache* cache = GetTECFallbackCache();
+  const short *scriptFallbackFonts = fontMapping.GetScriptFallbackFonts();
   
   // since we always call TECFallbackGetWidth before TECFallbackDrawChar
   // we could assume that we can always get the script code from cache.
@@ -303,21 +327,13 @@ PRBool nsUnicodeRenderingToolkit :: TECFallbackDrawChar(
   {
     if (BAD_SCRIPT == fallbackScript)
       return PR_FALSE;
-      
-    UnicodeToTextInfo fallbackConverter = GetConverterByScript(fallbackScript);
-    if (fallbackConverter) 
+    
+    if(fontMapping.ConvertUnicodeToGlyphs(scriptFallbackFonts[fallbackScript], aCharPt, 1,
+        buf, STACK_TRESHOLD, outLen, processBytes, kUnicodeLooseMappingsMask))
     {
-      err = ::ConvertFromUnicodeToText(fallbackConverter, (ByteCount)(2), 
-                                       (ConstUniCharArrayPtr) aCharPt, 
-                                       kUnicodeLooseMappingsMask , 0, NULL, 0, NULL,
-                                       STACK_TREASHOLD, &processBytes, &outLen, 
-                                       (LogicalAddress)buf);
-      if (outLen > 0) 
-      {
         ::TextFont(scriptFallbackFonts[fallbackScript]);
         DrawScriptText(buf, outLen, x, y, oWidth);
         ::TextFont(origFontNum);
-      }
     }
     return PR_TRUE;
   }
@@ -328,10 +344,21 @@ static PRUnichar gSymbolReplacement[]={0xf8ee,0xf8f9,0xf8f0,0xf8fb,0x3008,0x3009
 
 //------------------------------------------------------------------------
 
-static nsresult FormBIsolated(PRUnichar aChar, nsMacUnicodeFontInfo& aInfo, PRUnichar* aOutChar)
+static nsresult FormAorBIsolated(PRUnichar aChar, nsMacUnicodeFontInfo& aInfo, PRUnichar* aOutChar)
 {
   static const PRUnichar arabicisolated[]=
   {
+    // start of form a
+    0xFB50, 0x0671,  0xFB52, 0x067B,  0xFB56, 0x067E,  0xFB5A, 0x0680,  0xFB5E, 0x067A,
+    0xFB62, 0x067F,  0xFB66, 0x0679,  0xFB6A, 0x06A4,  0xFB6E, 0x06A6,  0xFB72, 0x0684,
+    0xFB76, 0x0683,  0xFB7A, 0x0686,  0xFB7E, 0x0687,  0xFB82, 0x068D,  0xFB84, 0x068C,
+    0xFB86, 0x068E,  0xFB88, 0x0688,  0xFB8A, 0x0698,  0xFB8C, 0x0691,  0xFB8E, 0x06A9,
+    0xFB92, 0x06AF,  0xFB96, 0x06B3,  0xFB9A, 0x06B1,  0xFB9E, 0x06BA,  0xFBA0, 0x06BB,
+    0xFBA4, 0x06C0,  0xFBA6, 0x06C1,  0xFBAA, 0x06BE,  0xFBAE, 0x06D2,  0xFBB0, 0x06D3,
+    0xFBD3, 0x06AD,  0xFBD7, 0x06C7,  0xFBD9, 0x06C6,  0xFBDB, 0x06C8,  0xFBDD, 0x0677,
+    0xFBDE, 0x06CB,  0xFBE0, 0x06C5,  0xFBE2, 0x06C9,  0xFBE4, 0x06D0,  0xFBFC, 0x06CC,
+
+    // start of form b
     0xFE70, 0x064B,  0xFE72, 0x064C,  0xFE74, 0x064D,  0xFE76, 0x064E,  0xFE78, 0x064F,
     0xFE7A, 0x0650,  0xFE7C, 0x0651,  0xFE7E, 0x0652,  0xFE80, 0x0621,  0xFE81, 0x0622,
     0xFE83, 0x0623,  0xFE85, 0x0624,  0xFE87, 0x0625,  0xFE89, 0x0626,  0xFE8D, 0x0627,
@@ -395,10 +422,10 @@ nsUnicodeRenderingToolkit::ATSUIFallbackGetDimensions(
       return PR_TRUE;
     }
   }
-  if (IN_ARABIC_PRESENTATION_B(*aCharPt))
+  if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
   {      
     PRUnichar isolated;
-    if (NS_SUCCEEDED( FormBIsolated(*aCharPt, info, &isolated))) 
+    if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) 
     {
       if(NS_SUCCEEDED(ATSUIFallbackGetDimensions(&isolated, oDim, origFontNum, 
                                                  aSize, aBold, aItalic, aColor))) 
@@ -430,6 +457,83 @@ nsUnicodeRenderingToolkit::ATSUIFallbackGetDimensions(
 }
 //------------------------------------------------------------------------
 
+#ifdef MOZ_MATHML
+PRBool
+nsUnicodeRenderingToolkit::ATSUIFallbackGetBoundingMetrics(
+  const PRUnichar *aCharPt,
+  nsBoundingMetrics& oBoundingMetrics,
+  short origFontNum,
+  short aSize, PRBool aBold, PRBool aItalic, nscolor aColor)
+{
+
+  nsMacUnicodeFontInfo info;
+  if (nsATSUIUtils::IsAvailable()  
+      && (IN_STANDARD_MAC_ROMAN_FONT(*aCharPt) 
+         ||IN_SYMBOL_FONT(*aCharPt)
+         ||SPECIAL_IN_SYMBOL_FONT(*aCharPt)
+         ||info.HasGlyphFor(*aCharPt)))
+  {
+    mATSUIToolkit.PrepareToDraw(mPort, mContext );
+    nsresult res;
+    if (SPECIAL_IN_SYMBOL_FONT(*aCharPt))
+    {
+      short rep = 0;
+      if ((*aCharPt) > 0x230b)
+         rep = (*aCharPt) - 0x2325;
+      else
+         rep = (*aCharPt) - 0x2308;
+      res = mATSUIToolkit.GetBoundingMetrics(gSymbolReplacement+rep, oBoundingMetrics, aSize, 
+                                             origFontNum, 
+                                             aBold, aItalic, aColor);
+    }
+    else
+    {
+      res = mATSUIToolkit.GetBoundingMetrics(aCharPt, oBoundingMetrics, aSize, 
+                                             origFontNum, 
+                                             aBold, aItalic, aColor);
+    }
+    if (NS_SUCCEEDED(res))
+    {
+      return PR_TRUE;
+    }
+  }
+  if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
+  {  
+    PRUnichar isolated;
+    if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) 
+    {
+      if(NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(&isolated, oBoundingMetrics, origFontNum, 
+                                                      aSize, aBold, aItalic, aColor))) 
+      {
+        return PR_TRUE;
+      }
+    }                                                 
+  }
+
+  // we know some ATSUI font do not have bold, turn it off and try again
+  if (aBold)
+  {
+    if (NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
+                                                     aSize, PR_FALSE, aItalic, aColor))) 
+    {
+      return PR_TRUE;
+    }
+  }
+  // we know some ATSUI font do not have italic, turn it off and try again
+  if (aItalic) 
+  {
+    if (NS_SUCCEEDED(ATSUIFallbackGetBoundingMetrics(aCharPt, oBoundingMetrics, origFontNum, 
+                                                     aSize, PR_FALSE, PR_FALSE, aColor))) 
+    {
+      return PR_TRUE;
+    }
+  }
+  return PR_FALSE;
+}
+#endif // MOZ_MATHML
+
+//------------------------------------------------------------------------
+
 PRBool nsUnicodeRenderingToolkit :: ATSUIFallbackDrawChar(
   const PRUnichar *aCharPt, 
   PRInt32 x,   PRInt32 y, 
@@ -464,10 +568,10 @@ PRBool nsUnicodeRenderingToolkit :: ATSUIFallbackDrawChar(
     if (NS_SUCCEEDED(res))
       return PR_TRUE;
   }
-  if (IN_ARABIC_PRESENTATION_B(*aCharPt))
+  if (IN_ARABIC_PRESENTATION_A_OR_B(*aCharPt))
   {      
     PRUnichar isolated;
-    if (NS_SUCCEEDED( FormBIsolated(*aCharPt, info, &isolated))) {
+    if (NS_SUCCEEDED( FormAorBIsolated(*aCharPt, info, &isolated))) {
       if (NS_SUCCEEDED(ATSUIFallbackDrawChar(&isolated, x, y, oWidth, origFontNum, 
                                              aSize, aBold, aItalic, aColor))) 
       {
@@ -564,7 +668,7 @@ PRBool nsUnicodeRenderingToolkit :: TransliterateFallbackGetWidth(
       CGrafPtr thePort;
       ::GetPort((GrafPtr*)&thePort);
 	    short aSize = ::GetPortTextSize(thePort);		 		
-  		PRInt32 l=nsCRT::strlen(conv);
+  		PRInt32 l=strlen(conv);
     	if((l>3) && ('^' == conv[0]) && ('(' == conv[1]) && (')' == conv[l-1])) // sup
     	{
     		short small = aSize * 2 / 3;
@@ -624,7 +728,7 @@ PRBool nsUnicodeRenderingToolkit :: TransliterateFallbackDrawChar(
 	    CGrafPtr thePort;
 	    ::GetPort((GrafPtr*)&thePort);
 	    short aSize = ::GetPortTextSize(thePort);		
-    	PRInt32 l=nsCRT::strlen(conv);
+    	PRInt32 l=strlen(conv);
     	if((l>3) && ('^' == conv[0]) && ('(' == conv[1]) && (')' == conv[l-1])) // sup
     	{
     		short small = aSize * 2 / 3;
@@ -961,6 +1065,95 @@ void nsUnicodeRenderingToolkit :: GetScriptTextWidth(
 {
  	oWidth = ::TextWidth(buf, 0, aLen);
 }
+
+#if MOZ_MATHML
+//------------------------------------------------------------------------
+void nsUnicodeRenderingToolkit::GetScriptTextBoundingMetrics(
+    const char* buf,
+    ByteCount aLen,
+    ScriptCode aScript,
+    nsBoundingMetrics& oBoundingMetrics)
+{
+    Point scale = { 1, 1 };
+    Fixed stackWidths[STACK_TRESHOLD], *widths;
+    Fixed stackLefts[STACK_TRESHOLD], *lefts;
+    Rect stackRects[STACK_TRESHOLD], *rects;
+    OSStatus err;
+
+    NS_PRECONDITION(aLen > 0, "length must be greater than 0");
+
+    if(aLen > STACK_TRESHOLD)
+    {
+        widths = (Fixed*) nsMemory::Alloc(aLen * sizeof(Fixed));
+        lefts = (Fixed*) nsMemory::Alloc(aLen * sizeof(Fixed));
+        rects = (Rect*) nsMemory::Alloc(aLen * sizeof(Fixed));
+        
+        // if any of the allocations failed the 'else' case below will be executed
+    }
+    else
+    {
+        widths = stackWidths;
+        lefts = stackLefts;
+        rects = stackRects;
+    }
+
+    if(!GetOutlinePreferred())
+        SetOutlinePreferred(PR_TRUE);
+
+    if(widths && lefts && rects &&
+        (err = ::OutlineMetrics(aLen, buf, scale, scale, NULL, NULL, widths, lefts, rects)) == noErr)
+    {
+        ByteCount byteIndex = 0, glyphIndex = 0;
+
+        while(byteIndex < aLen)
+        {
+            nsBoundingMetrics bounds;
+            bounds.leftBearing = rects[glyphIndex].left + FixRound(lefts[glyphIndex]);
+            bounds.rightBearing = rects[glyphIndex].right + FixRound(lefts[glyphIndex]);
+            bounds.ascent = rects[glyphIndex].bottom;
+            bounds.descent = -rects[glyphIndex].top;
+            bounds.width = FixRound(widths[glyphIndex]);
+
+            if(glyphIndex == 0)
+                oBoundingMetrics = bounds;
+            else
+                oBoundingMetrics += bounds;
+
+            // for two byte characters byteIndex will increase by 2
+            //   while glyph index will only increase by 1
+            if(CharacterByteType((Ptr) buf, byteIndex, aScript) == smFirstByte)
+                byteIndex += 2;
+            else
+                byteIndex++;
+            glyphIndex++;
+        }
+    }
+    else
+    {
+        NS_WARNING("OulineMetrics failed");
+
+        FontInfo fInfo;
+        ::GetFontInfo(&fInfo);
+
+        oBoundingMetrics.leftBearing = 0;
+        oBoundingMetrics.rightBearing = ::TextWidth(buf, 0, aLen);
+        oBoundingMetrics.ascent = fInfo.ascent;
+        oBoundingMetrics.descent = fInfo.descent;
+        oBoundingMetrics.width = oBoundingMetrics.rightBearing;
+    }
+
+    if(aLen > STACK_TRESHOLD)
+    {
+        if(widths)
+            nsMemory::Free(widths);
+        if(lefts)
+            nsMemory::Free(lefts);
+        if(rects)
+            nsMemory::Free(rects);
+    }
+}
+#endif // MOZ_MATHML
+
 //------------------------------------------------------------------------
 void nsUnicodeRenderingToolkit :: DrawScriptText(
 	const char* buf, 
@@ -972,7 +1165,7 @@ void nsUnicodeRenderingToolkit :: DrawScriptText(
 	::MoveTo(x, y);
 	::DrawText(buf,0,aLen);
 	
-	Point		penLoc;
+	Point   penLoc;
 	::GetPen(&penLoc);
 	oWidth = penLoc.h - x;
 }
@@ -981,11 +1174,11 @@ void nsUnicodeRenderingToolkit :: DrawScriptText(
 nsresult 
 nsUnicodeRenderingToolkit::GetTextSegmentWidth(
 			const PRUnichar *aString, PRUint32 aLength, 
-			short fontNum, const short *scriptFallbackFonts, 
+			short fontNum, nsUnicodeFontMappingMac& fontMapping, 
 			PRUint32& oWidth)
 {
   nsTextDimensions dim;
-  nsresult res = GetTextSegmentDimensions(aString, aLength, fontNum, scriptFallbackFonts, dim);
+  nsresult res = GetTextSegmentDimensions(aString, aLength, fontNum, fontMapping, dim);
   oWidth = dim.width;
   return res;
 }
@@ -995,7 +1188,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentWidth(
 nsresult 
 nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       const PRUnichar *aString, PRUint32 aLength, 
-      short fontNum, const short *scriptFallbackFonts, 
+      short fontNum, nsUnicodeFontMappingMac& fontMapping, 
       nsTextDimensions& oDim)
 {
   oDim.Clear();
@@ -1007,12 +1200,12 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
   char *heapBuf = nsnull;
   PRUint32 heapBufSize = 0;
   short thisWidth = 0;
-  char stackBuf[STACK_TREASHOLD];
+  char stackBuf[STACK_TRESHOLD];
   char *buf ;
   ByteCount processBytes;
   ByteCount outLen;
-  OSStatus err = noErr;
-
+  const short *scriptFallbackFonts = fontMapping.GetScriptFallbackFonts();
+  
   ::TextFont(fontNum);
   
   FontInfo fInfo;
@@ -1022,13 +1215,10 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
   segDim.descent = fInfo.descent;
   oDim.Combine(segDim);
   
-  ScriptCode script = ::FontToScript(fontNum);
-  UnicodeToTextInfo converter = GetConverterByScript(script);
-    
   // find buf from stack or heap. We only need to do this once in this function.
   // put this out of the loop for performance...
   ByteCount bufLen = aLength * 2 + 10;
-  if (bufLen > STACK_TREASHOLD)
+  if (bufLen > STACK_TRESHOLD)
   {
     if (bufLen > heapBufSize)
     {
@@ -1043,38 +1233,29 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
   } 
   else 
   {
-    bufLen = STACK_TREASHOLD;
+    bufLen = STACK_TRESHOLD;
     buf = stackBuf;
   }
   do {
-    if (converter)
+    outLen = 0;
+    processBytes = 0;
+
+    if(fontMapping.ConvertUnicodeToGlyphs(fontNum, aString, aLength - processLen,
+            buf, bufLen, outLen, processBytes, 0))
     {
-      outLen = 0;
-      err = noErr;
-      processBytes = 0;
-      err = ::ConvertFromUnicodeToText(converter, (ByteCount)(2*(aLength - processLen)), 
-                                       (ConstUniCharArrayPtr) aString, 
-                                       0 , 0, NULL, 0, NULL,
-                                       bufLen, &processBytes, &outLen, 
-                                       (LogicalAddress)buf);
-                  
-      // no mater if failed or not, as long as it convert some text, we process it.
-      if (outLen > 0)
-      {
         GetScriptTextWidth(buf, outLen, thisWidth);
       
         segDim.Clear();
         segDim.width = thisWidth;
         oDim.Combine(segDim);
-
+    
         NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
-
+    
         PRInt32 processUnicode = processBytes / 2;
         processLen += processUnicode;
         aString += processUnicode;
-      }
-         
     }
+         
     // Cannot precess by TEC, process one char a time by fallback mechanism
     if (processLen < aLength)
     {
@@ -1083,7 +1264,7 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       
 #ifndef DISABLE_TEC_FALLBACK
       // Fallback by try different Script code
-     fallbackDone = TECFallbackGetDimensions(aString, segDim, fontNum, scriptFallbackFonts);
+     fallbackDone = TECFallbackGetDimensions(aString, segDim, fontNum, fontMapping);
 #endif
 
       //
@@ -1096,10 +1277,8 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       {
         const nsFont *font;
         mGS->mFontMetrics->GetFont(font);
-        float textZoom;
-        mContext->GetTextZoom(textZoom);
         fallbackDone = ATSUIFallbackGetDimensions(aString, segDim, fontNum, 
-                                                  font->size * textZoom, 
+                                                  font->size, 
                                                   (font->weight > NS_FONT_WEIGHT_NORMAL), 
                                                   ((NS_FONT_STYLE_ITALIC ==  font->style) || 
                                                    (NS_FONT_STYLE_OBLIQUE ==  font->style)),
@@ -1107,6 +1286,12 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
       }
 
 #endif
+		  if(! fallbackDone) {
+		     if(IS_ZERO_WIDTH_CHAR(*aString))
+		     {
+		        fallbackDone = PR_TRUE;
+		     }
+		  }
 #ifndef DISABLE_LATIN_FALLBACK
       if (!fallbackDone) 
       {
@@ -1166,9 +1351,133 @@ nsUnicodeRenderingToolkit::GetTextSegmentDimensions(
 }
 //------------------------------------------------------------------------
 
+#ifdef MOZ_MATHML
+nsresult
+nsUnicodeRenderingToolkit::GetTextSegmentBoundingMetrics(
+      const PRUnichar *aString, PRUint32 aLength,
+      short fontNum, nsUnicodeFontMappingMac& fontMapping,
+      nsBoundingMetrics& oBoundingMetrics)
+{
+  oBoundingMetrics.Clear();
+  if(aLength == 0) 
+    return NS_OK;
+  NS_PRECONDITION(BAD_FONT_NUM != fontNum, "illegal font num");
+  PRBool firstTime = PR_TRUE;
+  PRUint32 processLen = 0;
+  nsBoundingMetrics segBoundingMetrics;
+  char *heapBuf = nsnull;
+  PRUint32 heapBufSize = 0;
+  char stackBuf[STACK_TRESHOLD];
+  char *buf;
+  ByteCount processBytes;
+  ByteCount outLen;
+  
+  ::TextFont(fontNum);
+  ScriptCode script = ::FontToScript(fontNum);
+  
+  // find buf from stack or heap. We only need to do this once in this function.
+  // put this out of the loop for performance...
+  ByteCount bufLen = aLength * 2 + 10;
+  if (bufLen > STACK_TRESHOLD)
+  {
+    if (bufLen > heapBufSize)
+    {
+      if (heapBuf)
+        nsMemory::Free(heapBuf);
+      heapBuf = (char*) nsMemory::Alloc(bufLen);
+      heapBufSize = bufLen;
+      if (nsnull == heapBuf) 
+        return NS_ERROR_OUT_OF_MEMORY;
+    } 
+    buf = heapBuf;
+  } 
+  else 
+  {
+    bufLen = STACK_TRESHOLD;
+    buf = stackBuf;
+  }
+
+  do {
+    outLen = 0;
+    processBytes = 0;
+        
+    if(fontMapping.ConvertUnicodeToGlyphs(fontNum, aString, aLength - processLen,
+        buf, bufLen, outLen, processBytes, 0))
+    {
+        segBoundingMetrics.Clear();
+        GetScriptTextBoundingMetrics(buf, outLen, script, segBoundingMetrics);
+        
+        if(firstTime) {
+            firstTime = PR_FALSE;
+            oBoundingMetrics = segBoundingMetrics;
+        }
+        else
+            oBoundingMetrics += segBoundingMetrics;
+        
+        NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
+        
+        PRInt32 processUnicode = processBytes / 2;
+        processLen += processUnicode;
+        aString += processUnicode;
+    }
+    
+    // Cannot process by TEC, process one char a time by fallback mechanism
+    if (processLen < aLength)
+    {
+      PRBool fallbackDone = PR_FALSE;
+      segBoundingMetrics.Clear();
+
+#ifndef DISABLE_TEC_FALLBACK
+      fallbackDone = TECFallbackGetBoundingMetrics(aString, segBoundingMetrics, fontNum, fontMapping);
+#endif
+
+#ifndef DISABLE_ATSUI_FALLBACK  
+      // Fallback by using ATSUI
+      if (!fallbackDone)  
+      {
+        const nsFont *font;
+        mGS->mFontMetrics->GetFont(font);
+        fallbackDone = ATSUIFallbackGetBoundingMetrics(aString, segBoundingMetrics, fontNum, 
+                                                  font->size, 
+                                                  (font->weight > NS_FONT_WEIGHT_NORMAL), 
+                                                  ((NS_FONT_STYLE_ITALIC ==  font->style) || 
+                                                   (NS_FONT_STYLE_OBLIQUE ==  font->style)),
+                                                  mGS->mColor );
+      }
+
+#endif
+      if(! fallbackDone) {
+         if(IS_ZERO_WIDTH_CHAR(*aString))
+         {
+           fallbackDone = PR_TRUE;
+         }
+      }
+
+      if (firstTime) {
+        firstTime = PR_FALSE;
+        oBoundingMetrics = segBoundingMetrics;
+      }
+      else
+        oBoundingMetrics += segBoundingMetrics;
+      // for fallback measure/drawing, we always do one char a time.
+      aString++;
+      processLen++;
+    }
+  } while (processLen < aLength);
+  
+  // release buffer if it is from heap
+  if (heapBuf)
+    nsMemory::Free(heapBuf);
+  
+  return NS_OK;
+}
+#endif // MOZ_MATHML
+//------------------------------------------------------------------------
+
+
 nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
 			const PRUnichar *aString, PRUint32 aLength, 
-			short fontNum, const short *scriptFallbackFonts, 
+			short fontNum, nsUnicodeFontMappingMac& fontMapping, 
 			PRInt32 x, PRInt32 y, PRUint32& oWidth)
 {
 	if(aLength == 0) {
@@ -1181,20 +1490,18 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
     char *heapBuf = nsnull;
     PRUint32 heapBufSize = 0;
     short thisWidth = 0;
-    char stackBuf[STACK_TREASHOLD];
+    char stackBuf[STACK_TRESHOLD];
     char *buf ;
     ByteCount processBytes;
     ByteCount outLen;
-  	OSStatus err = noErr;
+  	const short *scriptFallbackFonts = fontMapping.GetScriptFallbackFonts();
 
     ::TextFont(fontNum);
-    ScriptCode script = ::FontToScript(fontNum);
-    UnicodeToTextInfo converter = GetConverterByScript(script);
   	
   	// find buf from stack or heap. We only need to do this once in this function.
   	// put this out of the loop for performance...
   	ByteCount bufLen = aLength * 2 + 10;
-  	if( bufLen > STACK_TREASHOLD)
+  	if( bufLen > STACK_TRESHOLD)
   	{
   	 	if(bufLen > heapBufSize )
   	 	{
@@ -1208,37 +1515,29 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
   	 	} 
   	 	buf = heapBuf;
   	 } else {
-  	 	bufLen = STACK_TREASHOLD;
+  	 	bufLen = STACK_TRESHOLD;
   	 	buf = stackBuf;
   	}
 
     do {
-  	  if(converter)
-  	  {
-	  	outLen = 0;
- 	  	err = noErr;
-  	  	processBytes = 0;
-  	  	err = ::ConvertFromUnicodeToText(converter, (ByteCount)(2*(aLength - processLen)), (ConstUniCharArrayPtr) aString, 
-  	  	 						0 , 0, NULL, 0, NULL,
-  	  	 						bufLen, &processBytes, &outLen, 
-  	  	                        (LogicalAddress)buf);
-  	  	   	  	 
-  	  	 // no matter if failed or not, as long as it convert some text, we process it.
-  	  	 if(outLen > 0)
-  	  	 {
-			DrawScriptText(buf, outLen, x, y, thisWidth);
-			
-			textWidth += thisWidth;
-  		    x += thisWidth;			
-
- 	  	 	NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
-
-  	  	 	PRInt32 processUnicode = processBytes / 2;
-  	  	 	processLen += processUnicode;
-  	  	 	aString += processUnicode;
-  	  	 }
+      outLen = 0;
+      processBytes = 0;
+        
+      if(fontMapping.ConvertUnicodeToGlyphs(fontNum, aString, aLength - processLen,
+                buf, bufLen, outLen, processBytes, 0))
+      {
+        DrawScriptText(buf, outLen, x, y, thisWidth);
+        
+        textWidth += thisWidth;
+        x += thisWidth;			
+        
+        NS_PRECONDITION(0 == (processBytes % 2), "strange conversion result");
+        
+        PRInt32 processUnicode = processBytes / 2;
+        processLen += processUnicode;
+        aString += processUnicode;
+      }
   	  	 
-  	  }
   	  // Cannot precess by TEC, process one char a time by fallback mechanism
   	  if( processLen < aLength)
   	  {
@@ -1246,7 +1545,7 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
 
 #ifndef DISABLE_TEC_FALLBACK
 		  // Fallback by try different Script code
-		  fallbackDone = TECFallbackDrawChar(aString, x, y, thisWidth, fontNum, scriptFallbackFonts);
+		  fallbackDone = TECFallbackDrawChar(aString, x, y, thisWidth, fontNum, fontMapping);
 #endif
 		  //
 		  // We really don't care too much of performance after this
@@ -1258,15 +1557,21 @@ nsresult nsUnicodeRenderingToolkit :: DrawTextSegment(
 		  if(! fallbackDone)  {
 		  	const nsFont *font;
 			  mGS->mFontMetrics->GetFont(font);
-			  float textZoom;
-			  mContext->GetTextZoom(textZoom);
 		  	fallbackDone = ATSUIFallbackDrawChar(aString, x, y, thisWidth, fontNum, 
-									  		font->size * textZoom, 
+									  		font->size, 
 									  		(font->weight > NS_FONT_WEIGHT_NORMAL), 
 									  		((NS_FONT_STYLE_ITALIC ==  font->style) || (NS_FONT_STYLE_OBLIQUE ==  font->style)),
 									  		mGS->mColor );
 		  }
 #endif
+		  if(! fallbackDone) {
+		     if(IS_ZERO_WIDTH_CHAR(*aString))
+		     {
+		        thisWidth = 0;
+		        fallbackDone = PR_TRUE;
+		     }
+		  }
+      
 #ifndef DISABLE_LATIN_FALLBACK
 		  if(! fallbackDone)
 		  	fallbackDone = LatinFallbackDrawChar(aString, x, y, thisWidth);
@@ -1336,21 +1641,20 @@ nsUnicodeRenderingToolkit::GetTextDimensions(const PRUnichar *aString, PRUint32
   nsTextDimensions textDim;
   nsTextDimensions thisDim;
   
-  const short *scriptFallbackFonts = fontmap->GetScriptFallbackFonts();
   for(i =0, start = 0; i < aLength; i++)
   {
     fontNum[ i % 2] = fontmap->GetFontID(aString[i]);
     if ((fontNum[0] != fontNum[1]) && (0 != i))
     {  // start new font run...
       thisDim.Clear();
-      res =  GetTextSegmentDimensions(aString+start, i - start, fontNum[(i + 1) % 2], scriptFallbackFonts, thisDim);
+      res =  GetTextSegmentDimensions(aString+start, i - start, fontNum[(i + 1) % 2], *fontmap, thisDim);
       if (NS_FAILED (res)) 
         return res;    
       textDim.Combine(thisDim);
       start = i;
     }
   }
-  res = GetTextSegmentDimensions(aString+start, aLength - start, fontNum[(i + 1) % 2], scriptFallbackFonts, thisDim);
+  res = GetTextSegmentDimensions(aString+start, aLength - start, fontNum[(i + 1) % 2], *fontmap, thisDim);
   if (NS_FAILED (res)) 
     return res;    
   textDim.Combine(thisDim);
@@ -1360,6 +1664,58 @@ nsUnicodeRenderingToolkit::GetTextDimensions(const PRUnichar *aString, PRUint32
   aDim.descent = NSToCoordRound(float(textDim.descent) * mP2T);
   return res;  
 }
+//------------------------------------------------------------------------
+
+#ifdef MOZ_MATHML
+nsresult
+nsUnicodeRenderingToolkit::GetTextBoundingMetrics(const PRUnichar *aString, PRUint32 aLength,
+                                                  nsBoundingMetrics &aBoundingMetrics, PRInt32 *aFontID)
+{
+  nsresult res = NS_OK;
+  nsFontMetricsMac *metrics = (nsFontMetricsMac*) mGS->mFontMetrics;
+  nsUnicodeFontMappingMac* fontmap = metrics->GetUnicodeFontMapping();
+
+  PRUint32 i;
+  short fontNum[2];
+  fontNum[0] = fontNum[1] = BAD_FONT_NUM;
+  PRUint32 start;
+  PRBool firstTime = PR_TRUE;
+  nsBoundingMetrics thisBoundingMetrics;
+
+  for(i =0, start = 0; i < aLength; i++)
+  {
+    fontNum[ i % 2] = fontmap->GetFontID(aString[i]);
+    if ((fontNum[0] != fontNum[1]) && (0 != i))
+    {  // start new font run...
+      res = GetTextSegmentBoundingMetrics(aString+start, i - start, fontNum[(i + 1) % 2], *fontmap, thisBoundingMetrics);
+      if (NS_FAILED (res))
+        return res;
+      if (firstTime) {
+        firstTime = PR_FALSE;
+        aBoundingMetrics = thisBoundingMetrics;
+      }
+      else
+        aBoundingMetrics += thisBoundingMetrics;
+      start = i;
+    }
+  }
+  res = GetTextSegmentBoundingMetrics(aString+start, aLength - start, fontNum[(i + 1) % 2], *fontmap, thisBoundingMetrics);
+  if (NS_FAILED (res))
+    return res;
+  if (firstTime)
+    aBoundingMetrics = thisBoundingMetrics;
+  else
+    aBoundingMetrics += thisBoundingMetrics;
+
+  aBoundingMetrics.leftBearing = NSToCoordRound(float(aBoundingMetrics.leftBearing) * mP2T);
+  aBoundingMetrics.rightBearing = NSToCoordRound(float(aBoundingMetrics.rightBearing) * mP2T);
+  aBoundingMetrics.ascent = NSToCoordRound(float(aBoundingMetrics.ascent) * mP2T);
+  aBoundingMetrics.descent = NSToCoordRound(float(aBoundingMetrics.descent) * mP2T);
+  aBoundingMetrics.width = NSToCoordRound(float(aBoundingMetrics.width) * mP2T);
+
+  return res;
+}
+#endif // MOZ_MATHML
 
 //------------------------------------------------------------------------
 
@@ -1379,7 +1735,6 @@ nsUnicodeRenderingToolkit::DrawString(const PRUnichar *aString, PRUint32 aLength
   PRUint32 i;
   PRInt32 currentX = aX;
   PRUint32 thisWidth = 0;
-  const short *scriptFallbackFonts = fontmap->GetScriptFallbackFonts();
 	
   if (aSpacing)
   {
@@ -1401,7 +1756,7 @@ nsUnicodeRenderingToolkit::DrawString(const PRUnichar *aString, PRUint32 aLength
 
       PRInt32 transformedX = currentX, ignoreY = 0;
       mGS->mTMatrix.TransformCoord(&transformedX, &ignoreY);
-      res = DrawTextSegment(aString+i, drawLen, curFontNum, scriptFallbackFonts, transformedX, transformedY, thisWidth);
+      res = DrawTextSegment(aString+i, drawLen, curFontNum, *fontmap, transformedX, transformedY, thisWidth);
 	    if (NS_FAILED(res))
 	 		  return res;
       
@@ -1431,7 +1786,7 @@ nsUnicodeRenderingToolkit::DrawString(const PRUnichar *aString, PRUint32 aLength
 	          PRInt32 transformedX = currentX, ignoreY = 0;
 	          mGS->mTMatrix.TransformCoord(&transformedX, &ignoreY);
 	          
-            res = DrawTextSegment(aString + start, i - start, thisFont, scriptFallbackFonts, transformedX, transformedY, thisWidth);
+            res = DrawTextSegment(aString + start, i - start, thisFont, *fontmap, transformedX, transformedY, thisWidth);
 	    	  	if (NS_FAILED(res))
 	    	 		  return res;
 	    	 		
@@ -1444,7 +1799,7 @@ nsUnicodeRenderingToolkit::DrawString(const PRUnichar *aString, PRUint32 aLength
 
 	    PRInt32 transformedX = currentX, ignoreY = 0;
 	    mGS->mTMatrix.TransformCoord(&transformedX, &ignoreY);
-      res = DrawTextSegment(aString+start, aLength-start, thisFont, scriptFallbackFonts, transformedX, transformedY, thisWidth);
+      res = DrawTextSegment(aString+start, aLength-start, thisFont, *fontmap, transformedX, transformedY, thisWidth);
     if (NS_FAILED(res))
       return res;
   }
