@@ -32,6 +32,7 @@
 #include "nsILanguageAtomService.h"
 #include "nsIServiceManager.h"
 #include "nsUnicharUtils.h"
+#include "nsCRT.h"
 
 
 NS_IMPL_ISUPPORTS3(DeviceContextImpl, nsIDeviceContext, nsIObserver, nsISupportsWeakReference)
@@ -42,9 +43,7 @@ DeviceContextImpl::DeviceContextImpl()
   mFontCache = nsnull;
   mDevUnitsToAppUnits = 1.0f;
   mAppUnitsToDevUnits = 1.0f;
-  mGammaValue = 1.0f;
   mCPixelScale = 1.0f;
-  mGammaTable = new PRUint8[256];
   mZoom = 1.0f;
   mTextZoom = 1.0f;
   mWidget = nsnull;
@@ -76,12 +75,6 @@ DeviceContextImpl::~DeviceContextImpl()
     mFontCache = nsnull;
   }
 
-  if (nsnull != mGammaTable)
-  {
-    delete[] mGammaTable;
-    mGammaTable = nsnull;
-  }
-
   if (nsnull != mFontAliasTable) {
     mFontAliasTable->Enumerate(DeleteValue);
     delete mFontAliasTable;
@@ -114,9 +107,6 @@ void DeviceContextImpl::CommonInit(void)
   mInitialized = PR_TRUE;
 #endif
 
-  for (PRInt32 cnt = 0; cnt < 256; cnt++)
-    mGammaTable[cnt] = cnt;
-
   // register as a memory-pressure observer to free font resources
   // in low-memory situations.
   nsCOMPtr<nsIObserverService> obs(do_GetService("@mozilla.org/observer-service;1"));
@@ -325,10 +315,7 @@ NS_IMETHODIMP DeviceContextImpl::GetMetricsFor(const nsFont& aFont, nsIFontMetri
 
 NS_IMETHODIMP DeviceContextImpl::SetZoom(float aZoom)
 {
-  if (mZoom != aZoom) {
-    mZoom = aZoom;
-    FlushFontCache();
-  }
+  mZoom = aZoom;
   return NS_OK;
 }
 
@@ -340,10 +327,7 @@ NS_IMETHODIMP DeviceContextImpl::GetZoom(float &aZoom) const
 
 NS_IMETHODIMP DeviceContextImpl::SetTextZoom(float aTextZoom)
 {
-  if (mTextZoom != aTextZoom) {
-    mTextZoom = aTextZoom;
-    FlushFontCache();
-  }
+  mTextZoom = aTextZoom;
   return NS_OK;
 }
 
@@ -353,43 +337,6 @@ NS_IMETHODIMP DeviceContextImpl::GetTextZoom(float &aTextZoom) const
   return NS_OK;
 }
 
-NS_IMETHODIMP DeviceContextImpl::GetGamma(float &aGamma)
-{
-  aGamma = mGammaValue;
-  return NS_OK;
-}
-
-NS_IMETHODIMP DeviceContextImpl::SetGamma(float aGamma)
-{
-  if (aGamma != mGammaValue)
-  {
-    //we don't need to-recorrect existing images for this case
-    //so pass in 1.0 for the current gamma regardless of what it
-    //really happens to be. existing images will get a one time
-    //re-correction when they're rendered the next time. MMP
-
-    SetGammaTable(mGammaTable, 1.0f, aGamma);
-
-    mGammaValue = aGamma;
-  }
-  return NS_OK;
-}
-
-NS_IMETHODIMP DeviceContextImpl::GetGammaTable(PRUint8 *&aGammaTable)
-{
-  //XXX we really need to ref count this somehow. MMP
-  aGammaTable = mGammaTable;
-  return NS_OK;
-}
-
-void DeviceContextImpl::SetGammaTable(PRUint8 * aTable, float aCurrentGamma, float aNewGamma)
-{
-  double fgval = (1.0f / aCurrentGamma) * (1.0f / aNewGamma);
-
-  for (PRInt32 cnt = 0; cnt < 256; cnt++)
-    aTable[cnt] = (PRUint8)(pow((double)cnt * (1. / 256.), fgval) * 255.99999999);
-}
-
 NS_IMETHODIMP DeviceContextImpl::GetDepth(PRUint32& aDepth)
 {
   aDepth = 24;
@@ -471,7 +418,7 @@ PRUint32 FontAliasKey::HashCode(void) const
 
 PRBool FontAliasKey::Equals(const nsHashKey *aKey) const
 {
-  return mString.EqualsIgnoreCase(((FontAliasKey*)aKey)->mString);
+  return mString.Equals(((FontAliasKey*)aKey)->mString, nsCaseInsensitiveStringComparator());
 }
 
 nsHashKey* FontAliasKey::Clone(void) const
@@ -705,6 +652,7 @@ nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
     return NS_OK;
   }
 
+  NS_POSTCONDITION(NS_SUCCEEDED(rv), "font metrics should not be null - bug 136248");
   return rv;
 }
 
