@@ -40,6 +40,7 @@
 #include "nsCoord.h"
 #include "nsUnitConversion.h"
 #include "nsReadableUtils.h"
+#include "nsIPrintSession.h"
 
 NS_IMPL_ISUPPORTS1(nsPrintSettings, nsIPrintSettings)
 
@@ -48,29 +49,32 @@ NS_IMPL_ISUPPORTS1(nsPrintSettings, nsIPrintSettings)
  *	@update 6/21/00 dwc
  */
 nsPrintSettings::nsPrintSettings() :
+  mPrintOptions(0L),
   mPrintRange(kRangeAllPages),
   mStartPageNum(1),
   mEndPageNum(1),
   mScaling(1.0),
-  mNumCopies(1),
   mPrintBGColors(PR_FALSE),
   mPrintBGImages(PR_FALSE),
-  mPrintOptions(0L),
-  mPrintReversed(PR_FALSE),
-  mPrintInColor(PR_TRUE),
-  mOrientation(kPortraitOrientation),
-  mPrintToFile(PR_FALSE),
   mPrintFrameTypeUsage(kUseInternalDefault),
   mPrintFrameType(kFramesAsIs),
+  mHowToEnableFrameUI(kFrameEnableNone),
+  mIsCancelled(PR_FALSE),
+  mPrintSilent(PR_FALSE),
+	mPrintPreview(PR_FALSE),
+  mShrinkToFit(PR_TRUE),
+  mShowPrintProgress(PR_TRUE),
   mPrintPageDelay(500),
-  mPaperSizeType(kPaperSizeDefined),
   mPaperData(0),
+  mPaperSizeType(kPaperSizeDefined),
   mPaperWidth(8.5),
   mPaperHeight(11.0),
   mPaperSizeUnit(kPaperSizeInches),
-  mPrintSilent(PR_FALSE),
-  mHowToEnableFrameUI(kFrameEnableNone),
-  mIsCancelled(PR_FALSE)
+  mPrintReversed(PR_FALSE),
+  mPrintInColor(PR_TRUE),
+  mOrientation(kPortraitOrientation),
+  mNumCopies(1),
+  mPrintToFile(PR_FALSE)
 {
   NS_INIT_ISUPPORTS();
 
@@ -88,6 +92,17 @@ nsPrintSettings::nsPrintSettings() :
 
 }
 
+/** ---------------------------------------------------
+ *  See documentation in nsPrintSettingsImpl.h
+ *	@update 6/21/00 dwc
+ */
+nsPrintSettings::nsPrintSettings(const nsPrintSettings& aPS)
+{
+  NS_INIT_ISUPPORTS();
+
+  *this = aPS;
+}
+
 /** ---------------------------------------------------
  *  See documentation in nsPrintSettingsImpl.h
  *	@update 6/21/00 dwc
@@ -96,6 +111,34 @@ nsPrintSettings::~nsPrintSettings()
 {
 }
 
+/* [noscript] attribute nsIPrintSession printSession; */
+NS_IMETHODIMP nsPrintSettings::GetPrintSession(nsIPrintSession **aPrintSession)
+{
+  NS_ENSURE_ARG_POINTER(aPrintSession);
+  *aPrintSession = nsnull;
+  
+  nsCOMPtr<nsIPrintSession> session = do_QueryReferent(mSession);
+  if (!session)
+    return NS_ERROR_NOT_INITIALIZED;
+  *aPrintSession = session;
+  NS_ADDREF(*aPrintSession);
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetPrintSession(nsIPrintSession *aPrintSession)
+{
+  // Clearing it by passing NULL is not allowed. That's why we
+  // use a weak ref so that it doesn't have to be cleared.
+  NS_ENSURE_ARG(aPrintSession);
+  
+  mSession = getter_AddRefs(NS_GetWeakReference(aPrintSession));
+  if (!mSession) {
+    // This may happen if the implementation of this object does
+    // not support weak references - programmer error.
+    NS_ERROR("Could not get a weak reference from aPrintSession");
+    return NS_ERROR_FAILURE;
+  }
+  return NS_OK;
+}
 
 /* attribute long startPageRange; */
 NS_IMETHODIMP nsPrintSettings::GetStartPageRange(PRInt32 *aStartPageRange)
@@ -184,8 +227,12 @@ NS_IMETHODIMP nsPrintSettings::GetPrinterName(PRUnichar * *aPrinter)
 }
 NS_IMETHODIMP nsPrintSettings::SetPrinterName(const PRUnichar * aPrinter)
 {
-   mPrinter = aPrinter;
-   return NS_OK;
+  if (aPrinter) {
+    mPrinter = aPrinter;
+  } else {
+    mPrinter.SetLength(0);
+  }
+  return NS_OK;
 }
 
 /* attribute long numCopies; */
@@ -210,7 +257,11 @@ NS_IMETHODIMP nsPrintSettings::GetPrintCommand(PRUnichar * *aPrintCommand)
 }
 NS_IMETHODIMP nsPrintSettings::SetPrintCommand(const PRUnichar * aPrintCommand)
 {
-  mPrintCommand = aPrintCommand;
+  if (aPrintCommand) {
+    mPrintCommand = aPrintCommand;
+  } else {
+    mPrintCommand.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -236,7 +287,11 @@ NS_IMETHODIMP nsPrintSettings::GetToFileName(PRUnichar * *aToFileName)
 }
 NS_IMETHODIMP nsPrintSettings::SetToFileName(const PRUnichar * aToFileName)
 {
-  mToFileName = aToFileName;
+  if (aToFileName) {
+    mToFileName = aToFileName;
+  } else {
+    mToFileName.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -370,8 +425,11 @@ NS_IMETHODIMP nsPrintSettings::GetTitle(PRUnichar * *aTitle)
 }
 NS_IMETHODIMP nsPrintSettings::SetTitle(const PRUnichar * aTitle)
 {
-  NS_ENSURE_ARG_POINTER(aTitle);
-  mTitle = aTitle;
+  if (aTitle) {
+    mTitle = aTitle;
+  } else {
+    mTitle.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -388,8 +446,11 @@ NS_IMETHODIMP nsPrintSettings::GetDocURL(PRUnichar * *aDocURL)
 }
 NS_IMETHODIMP nsPrintSettings::SetDocURL(const PRUnichar * aDocURL)
 {
-  NS_ENSURE_ARG_POINTER(aDocURL);
-  mURL = aDocURL;
+  if (aDocURL) {
+    mURL = aDocURL;
+  } else {
+    mURL.SetLength(0);
+  }
   return NS_OK;
 }
 
@@ -401,7 +462,7 @@ NS_IMETHODIMP
 nsPrintSettings::GetPrintOptions(PRInt32 aType, PRBool *aTurnOnOff)
 {
   NS_ENSURE_ARG_POINTER(aTurnOnOff);
-  *aTurnOnOff = mPrintOptions & aType;
+  *aTurnOnOff = mPrintOptions & aType ? PR_TRUE : PR_FALSE;
   return NS_OK;
 }
 /** ---------------------------------------------------
@@ -577,6 +638,53 @@ NS_IMETHODIMP nsPrintSettings::SetPrintSilent(PRBool aPrintSilent)
   return NS_OK;
 }
 
+/* attribute boolean shrinkToFit; */
+NS_IMETHODIMP nsPrintSettings::GetShrinkToFit(PRBool *aShrinkToFit)
+{
+  NS_ENSURE_ARG_POINTER(aShrinkToFit);
+  *aShrinkToFit = mShrinkToFit;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetShrinkToFit(PRBool aShrinkToFit)
+{
+  mShrinkToFit = aShrinkToFit;
+  return NS_OK;
+}
+
+/* attribute boolean showPrintProgress; */
+NS_IMETHODIMP nsPrintSettings::GetShowPrintProgress(PRBool *aShowPrintProgress)
+{
+  NS_ENSURE_ARG_POINTER(aShowPrintProgress);
+  *aShowPrintProgress = mShowPrintProgress;
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetShowPrintProgress(PRBool aShowPrintProgress)
+{
+  mShowPrintProgress = aShowPrintProgress;
+  return NS_OK;
+}
+
+/* attribute wstring paperName; */
+NS_IMETHODIMP nsPrintSettings::GetPaperName(PRUnichar * *aPaperName)
+{
+  NS_ENSURE_ARG_POINTER(aPaperName);
+  if (mPaperName.Length()) {
+    *aPaperName = ToNewUnicode(mPaperName);
+  } else {
+    *aPaperName = nsnull;
+  }
+  return NS_OK;
+}
+NS_IMETHODIMP nsPrintSettings::SetPaperName(const PRUnichar * aPaperName)
+{
+  if (aPaperName) {
+    mPaperName = aPaperName;
+  } else {
+    mPaperName.SetLength(0);
+  }
+  return NS_OK;
+}
+
 /* attribute boolean howToEnableFrameUI; */
 NS_IMETHODIMP nsPrintSettings::GetHowToEnableFrameUI(PRInt16 *aHowToEnableFrameUI)
 {
@@ -708,3 +816,83 @@ nsPrintSettings::GetPageSizeInTwips(PRInt32 *aWidth, PRInt32 *aHeight)
   return NS_OK;
 }
 
+nsresult 
+nsPrintSettings::_Clone(nsIPrintSettings **_retval)
+{
+  nsPrintSettings* printSettings = new nsPrintSettings(*this);
+  return printSettings->QueryInterface(NS_GET_IID(nsIPrintSettings), (void**)_retval); // ref counts
+}
+
+/* nsIPrintSettings clone (); */
+NS_IMETHODIMP 
+nsPrintSettings::Clone(nsIPrintSettings **_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+  return _Clone(_retval);
+}
+
+/* void assign (in nsIPrintSettings aPS); */
+nsresult 
+nsPrintSettings::_Assign(nsIPrintSettings *aPS)
+{
+  nsPrintSettings *ps = NS_STATIC_CAST(nsPrintSettings*, aPS);
+  *this = *ps;
+  return NS_OK;
+}
+
+/* void assign (in nsIPrintSettings aPS); */
+NS_IMETHODIMP 
+nsPrintSettings::Assign(nsIPrintSettings *aPS)
+{
+  NS_ENSURE_ARG(aPS);
+  return _Assign(aPS);
+}
+
+//-------------------------------------------
+nsPrintSettings& nsPrintSettings::operator=(const nsPrintSettings& rhs)
+{
+  if (this == &rhs) {
+    return *this;
+  }
+
+  mStartPageNum        = rhs.mStartPageNum;
+  mEndPageNum          = rhs.mEndPageNum;
+  mMargin              = rhs.mMargin;
+  mScaling             = rhs.mScaling;
+  mPrintBGColors       = rhs.mPrintBGColors;
+  mPrintBGImages       = rhs.mPrintBGImages;
+  mPrintRange          = rhs.mPrintRange;
+  mTitle               = rhs.mTitle;
+  mURL                 = rhs.mURL;
+  mHowToEnableFrameUI  = rhs.mHowToEnableFrameUI;
+  mIsCancelled         = rhs.mIsCancelled;
+  mPrintFrameTypeUsage = rhs.mPrintFrameTypeUsage;
+  mPrintFrameType      = rhs.mPrintFrameType;
+  mPrintSilent         = rhs.mPrintSilent;
+  mShrinkToFit         = rhs.mShrinkToFit;
+  mShowPrintProgress   = rhs.mShowPrintProgress;
+  mPaperName           = rhs.mPaperName;
+  mPaperSizeType       = rhs.mPaperSizeType;
+  mPaperData           = rhs.mPaperData;
+  mPaperWidth          = rhs.mPaperWidth;
+  mPaperHeight         = rhs.mPaperHeight;
+  mPaperSizeUnit       = rhs.mPaperSizeUnit;
+  mPrintReversed       = rhs.mPrintReversed;
+  mPrintInColor        = rhs.mPrintInColor;
+  mPaperSize           = rhs.mPaperSize;
+  mOrientation         = rhs.mOrientation;
+  mPrintCommand        = rhs.mPrintCommand;
+  mNumCopies           = rhs.mNumCopies;
+  mPrinter             = rhs.mPrinter;
+  mPrintToFile         = rhs.mPrintToFile;
+  mToFileName          = rhs.mToFileName;
+  mPrintPageDelay      = rhs.mPrintPageDelay;
+
+  for (PRInt32 i=0;i<NUM_HEAD_FOOT;i++) {
+    mHeaderStrs[i] = rhs.mHeaderStrs[i];
+    mFooterStrs[i] = rhs.mFooterStrs[i];
+  }
+
+  return *this;
+}
+
