@@ -132,7 +132,7 @@ nsTransform2D       *theTransform;
       theTransform->SetToIdentity();  
 	    theTransform->AddScale(app2dev, app2dev);
 
-#ifdef XP_UNIX
+#if defined(XP_UNIX) && !defined(XP_MACOSX)
       srcRect.SetRect(0,0,tvrect.width,tvrect.height);
       SetClipRect(srcRect, nsClipCombine_kReplace, clip);
 #endif
@@ -530,6 +530,13 @@ NS_IMETHODIMP nsRenderingContextImpl::DestroyCachedBackbuffer(void)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsRenderingContextImpl::UseBackbuffer(PRBool* aUseBackbuffer)
+{
+  *aUseBackbuffer = PR_TRUE;
+  return NS_OK;
+}
+
+
 PRBool nsRenderingContextImpl::RectFitsInside(const nsRect& aRect, PRInt32 aWidth, PRInt32 aHeight) const
 {
   if (aRect.width > aWidth)
@@ -887,8 +894,11 @@ NS_IMETHODIMP nsRenderingContextImpl::DrawImage(imgIContainer *aImage, const nsR
   nsPoint pt;
   nsRect sr;
 
-  pt = *aDestPoint;
-  mTranMatrix->TransformCoord(&pt.x, &pt.y);
+  float fX = float(aDestPoint->x), fY = float(aDestPoint->y);
+  mTranMatrix->Transform(&fX, &fY);
+  
+  pt.x = NSToCoordRound(fX);
+  pt.y = NSToCoordRound(fY);
 
   sr = *aSrcRect;
   mTranMatrix->TransformCoord(&sr.x, &sr.y, &sr.width, &sr.height);
@@ -974,3 +984,15 @@ NS_IMETHODIMP nsRenderingContextImpl::DrawScaledTile(imgIContainer *aImage, nsco
 }
 
 #endif
+
+NS_IMETHODIMP
+nsRenderingContextImpl::FlushRect(const nsRect& aRect)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsRenderingContextImpl::FlushRect(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight)
+{
+    return NS_OK;
+}
