@@ -27,7 +27,6 @@
 #ifdef COLOR_256
 #include "il_util.h"
 #endif
-#include "nsIPref.h"
 #include "nsIServiceManager.h"
 #include "nsCOMPtr.h"
 #include "nsIScreenManager.h"
@@ -43,13 +42,8 @@
 #define NOT_SETUP 0x33
 static PRBool gIsWarp4 = NOT_SETUP;
 
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
-
-PRBool nsDeviceContextOS2::gRound = PR_FALSE;
 PRUint32 nsDeviceContextOS2::sNumberOfScreens = 0;
 
-static char* nav4rounding = "font.size.nav4rounding";
-
 nsDeviceContextOS2 :: nsDeviceContextOS2()
   : DeviceContextImpl()
 {
@@ -61,14 +55,13 @@ nsDeviceContextOS2 :: nsDeviceContextOS2()
   mPaletteInfo.palette = NULL;
 #endif
   mPrintDC = NULL;
-  mPixelScale = 1.0f;
   mWidth = -1;
   mHeight = -1;
   mSpec = nsnull;
   mCachedClientRect = PR_FALSE;
   mCachedFullRect = PR_FALSE;
   mSupportsRasterFonts = PR_FALSE;
-  mPelsPerMeter = 0;
+  mPrintingStarted = PR_FALSE;
 #ifdef XP_OS2
   mPrintState = nsPrintState_ePreBeginDoc;
 #endif
@@ -80,19 +73,6 @@ nsDeviceContextOS2 :: nsDeviceContextOS2()
                      ulValues, sizeof(ulValues));
     gIsWarp4 = (ulValues[0] >= 20) && (ulValues[1] >= 40);
   }
-
-#ifndef XP_OS2
-  nsresult res = NS_ERROR_FAILURE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res));
-  if (NS_SUCCEEDED(res)) {
-    static PRBool roundingInitialized = PR_FALSE;
-    if (!roundingInitialized) {
-      roundingInitialized = PR_TRUE;
-      PrefChanged(nav4rounding, this);
-    }
-    prefs->RegisterCallback(nav4rounding, PrefChanged, this);
-  }
-#endif
 }
 
 nsDeviceContextOS2::~nsDeviceContextOS2()
@@ -119,7 +99,7 @@ nsresult nsDeviceContextOS2::Init( nsNativeWidget aWidget)
 nsresult nsDeviceContextOS2::Init( nsNativeDeviceContext aContext,
                                    nsIDeviceContext *aOrigContext)
 {
-//  float origscale, newscale;
+  float origscale, newscale;
   float t2d, a2d;
 
   mPrintDC = (HDC)aContext;
@@ -139,13 +119,16 @@ nsresult nsDeviceContextOS2::Init( nsNativeDeviceContext aContext,
 
   CommonInit( mPrintDC);
 
-//  GetTwipsToDevUnits( newscale);
+  GetTwipsToDevUnits( newscale);
+
+// On OS/2, origscale can be different based on the video resolution.
+// On 640x480, it's 1/15, on everything else it is 1/12.
+// For consistent printing, 1/15 is the correct value to use.
+// It is the closest to 4.x and to Windows.
 //  aOrigContext->GetTwipsToDevUnits( origscale);
+  origscale = 1.0/15.0;
 
-//  mPixelScale = newscale / origscale;
-
-  mPixelScale = (float)mPelsPerMeter / 3622.0f; /* This is ugly - it prevents different size printing on different */
-                                                /* Resolutions */
+  mCPixelScale = newscale / origscale;
 
   aOrigContext->GetTwipsToDevUnits( t2d);
   aOrigContext->GetAppUnitsToDevUnits( a2d);
@@ -161,13 +144,9 @@ nsresult nsDeviceContextOS2::Init( nsNativeDeviceContext aContext,
   // XXX hsb says there are margin problems, must be from here...
 #ifdef DEBUG
   printf( "Got surface of size %d x %d pixels (%d Kb)\n", mWidth, mHeight, mWidth * mHeight * mDepth / 8 / 1024);
-  printf( "mPixelScale = %f\n", mPixelScale);
+  printf( "mCPixelScale = %f\n", mCPixelScale);
 #endif
 
-  // We need to begin a document now, because the client is entitled at
-  // this point to do stuff like create fonts, which required the PS to
-  // be associated with a DC which has been DEVESC_STARTDOC'd.
-  BeginDocument(nsnull);
 #endif
 
   return NS_OK;
@@ -183,8 +162,6 @@ void nsDeviceContextOS2 :: CommonInit(HDC aDC)
 
   mPixelsToTwips = 1.0f / mTwipsToPixels;
 
-  mPelsPerMeter = alArray[CAPS_VERTICAL_RESOLUTION];
-
   mDepth = alArray[CAPS_COLOR_BITCOUNT];
 #ifdef COLOR_256
   mPaletteInfo.isPaletteDevice = !!(alArray[CAPS_ADDITIONAL_GRAPHICS] & CAPS_PALETTE_MANAGER);
@@ -338,16 +315,11 @@ NS_IMETHODIMP nsDeviceContextOS2 :: SupportsNativeWidgets(PRBool &aSupportsWidge
   return NS_OK;
 }
 
-NS_IMETHODIMP nsDeviceContextOS2 :: GetCanonicalPixelScale(float &aScale) const
-{
-  aScale = mPixelScale;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsDeviceContextOS2 :: GetScrollBarDimensions(float &aWidth, float &aHeight) const
 {
-  aWidth = ::WinQuerySysValue( HWND_DESKTOP, SV_CXVSCROLL) * mDevUnitsToAppUnits;
-  aHeight = ::WinQuerySysValue( HWND_DESKTOP, SV_CYHSCROLL) * mDevUnitsToAppUnits;
+  float scale = mCPixelScale;
+  aWidth = ::WinQuerySysValue( HWND_DESKTOP, SV_CXVSCROLL) * mDevUnitsToAppUnits * scale;
+  aHeight = ::WinQuerySysValue( HWND_DESKTOP, SV_CYHSCROLL) * mDevUnitsToAppUnits * scale;
   return NS_OK;
 }
 
@@ -835,16 +807,16 @@ NS_IMETHODIMP nsDeviceContextOS2 :: GetDeviceContextFor(nsIDeviceContextSpec *aD
   NS_ADDREF(aDevice);
   
   int numCopies = 0;
-  int toPrinter = 0;
+  int printerDest = 0;
   char *file = nsnull;
 
   ((nsDeviceContextSpecOS2 *)aDevice)->GetPRTQUEUE(pq);
   ((nsDeviceContextSpecOS2 *)aDevice)->GetCopies(numCopies);
-  ((nsDeviceContextSpecOS2 *)aDevice)->GetToPrinter(toPrinter);
-  if (!toPrinter) 
+  ((nsDeviceContextSpecOS2 *)aDevice)->GetDestination(printerDest);
+  if (!printerDest) 
      ((nsDeviceContextSpecOS2 *)aDevice)->GetPath(&file);
 
-  HDC dc = PrnOpenDC(pq, "Mozilla", numCopies, toPrinter, file);
+  HDC dc = PrnOpenDC(pq, "Mozilla", numCopies, printerDest, file);
 
   if (!dc) {
      return NS_ERROR_FAILURE; //PMERROR("DevOpenDC");
@@ -890,7 +862,7 @@ nsresult nsDeviceContextOS2::CreateFontAliasTable()
 }
 
 // Printing ------------------------------------------------------------------
-nsresult nsDeviceContextOS2::BeginDocument(PRUnichar * aTitle)
+nsresult nsDeviceContextOS2::PrepareDocument(PRUnichar * aTitle, PRUnichar* aPrintToFileName)
 {
   nsresult rv = NS_OK;
 
@@ -901,19 +873,12 @@ nsresult nsDeviceContextOS2::BeginDocument(PRUnichar * aTitle)
 
     PSZ pszDocName = title != nsnull?title:"Mozilla Document";
 
-    long lResult;
-
-    // ENDDOC first so we work on Lexmark printer drivers
-    long   lOutCount = 2;
-    USHORT usJobID = 0;
-    lResult = ::DevEscape(mPrintDC, DEVESC_ENDDOC,
-                          0, NULL,
-                          &lOutCount, (PBYTE)&usJobID);
-
     long lDummy = 0;
-    lResult = ::DevEscape(mPrintDC, DEVESC_STARTDOC,
-                          strlen(pszDocName) + 1, pszDocName,
-                          &lDummy, NULL);
+    long lResult = ::DevEscape(mPrintDC, DEVESC_STARTDOC,
+                               strlen(pszDocName) + 1, pszDocName,
+                               &lDummy, NULL);
+
+    mPrintingStarted = PR_TRUE;
 
     if (lResult == DEV_OK)
       rv = NS_OK;
@@ -928,6 +893,12 @@ nsresult nsDeviceContextOS2::BeginDocument(PRUnichar * aTitle)
   return rv;
 }
 
+nsresult nsDeviceContextOS2::BeginDocument(PRUnichar * aTitle, PRUnichar* aPrintToFileName, PRInt32 aStartPage, PRInt32 aEndPage)
+{
+  // Everything is done in PrepareDocument
+  return NS_OK;
+}
+
 nsresult nsDeviceContextOS2::EndDocument()
 {
   if (NULL != mPrintDC)
@@ -946,16 +917,32 @@ nsresult nsDeviceContextOS2::EndDocument()
   return NS_OK;
 }
 
-nsresult nsDeviceContextOS2::BeginPage()
-{
-  return NS_OK;
-}
-
-nsresult nsDeviceContextOS2::EndPage()
+nsresult nsDeviceContextOS2::AbortDocument()
 {
   if (NULL != mPrintDC)
   {
+    long ldummy = 0;
+    long lResult = ::DevEscape(mPrintDC, DEVESC_ABORTDOC, 0, NULL,
+                               &ldummy, NULL);
+    if (lResult == DEV_OK)
+      return NS_OK;
+    else
+      return NS_ERROR_ABORT;
+  }
 
+  return NS_OK;
+}
+
+
+nsresult nsDeviceContextOS2::BeginPage()
+{
+  if (mPrintingStarted) {
+    mPrintingStarted = PR_FALSE;
+    return NS_OK;
+  }
+
+  if (NULL != mPrintDC)
+  {
     long lDummy = 0;
     long lResult = ::DevEscape(mPrintDC, DEVESC_NEWFRAME, 0, NULL,
                                &lDummy, NULL);
@@ -963,10 +950,16 @@ nsresult nsDeviceContextOS2::EndPage()
     if (lResult == DEV_OK)
       return NS_OK;
     else
-      return NS_ERROR_GFX_PRINTER_ENDPAGE;
+      return NS_ERROR_GFX_PRINTER_STARTPAGE;
   }
 
   return NS_OK;
+
+}
+
+nsresult nsDeviceContextOS2::EndPage()
+{
+   return NS_OK;
 }
 
 char* 
