@@ -20,6 +20,7 @@
  *
  */
 
+#include <stdlib.h>
 #include "nsDeviceContextSpecOS2.h"
 
 #include "nsReadableUtils.h"
@@ -28,12 +29,8 @@
 #include "nsIPref.h"
 #include "prenv.h" /* for PR_GetEnv */
 
-#include "nsIDOMWindow.h"
 #include "nsIServiceManager.h"
-#include "nsIDialogParamBlock.h"
-#include "nsISupportsPrimitives.h"
-#include "nsIWindowWatcher.h"
-#include "nsIDOMWindowInternal.h"
+#include "nsUnicharUtils.h"
 
 PRINTDLG nsDeviceContextSpecOS2::PrnDlg;
 
@@ -54,6 +51,7 @@ public:
   PRBool    PrintersAreAllocated()       { return mGlobalPrinterList != nsnull; }
   PRInt32   GetNumPrinters()             { return mGlobalNumPrinters; }
   nsString* GetStringAt(PRInt32 aInx)    { return mGlobalPrinterList->StringAt(aInx); }
+  void      GetDefaultPrinterName(PRUnichar*& aDefaultPrinterName);
 
 protected:
   GlobalPrinters() {}
@@ -95,6 +93,114 @@ static NS_DEFINE_IID(kIDeviceContextSpecIID, NS_IDEVICE_CONTEXT_SPEC_IID);
 static NS_DEFINE_IID(kIDeviceContextSpecXPIID, NS_IDEVICE_CONTEXT_SPEC_XP_IID);
 #endif
 
+void SetupDevModeFromSettings(int printer, nsIPrintSettings* aPrintSettings)
+{
+  if (aPrintSettings) {
+    int bufferSize = 3 * sizeof(DJP_ITEM);
+    PBYTE pDJP_Buffer = new BYTE[bufferSize];
+    memset(pDJP_Buffer, 0, bufferSize);
+    PDJP_ITEM pDJP = (PDJP_ITEM) pDJP_Buffer;
+
+    HDC hdc = nsDeviceContextSpecOS2::PrnDlg.GetDCHandle(printer);
+    char* driver = nsDeviceContextSpecOS2::PrnDlg.GetDriverType(printer);
+
+    // Setup Orientation
+    PRInt32 orientation;
+    aPrintSettings->GetOrientation(&orientation);
+    if (!strcmp(driver, "LASERJET"))
+      pDJP->lType = DJP_ALL;
+    else
+      pDJP->lType = DJP_CURRENT;
+    pDJP->cb = sizeof(DJP_ITEM);
+    pDJP->ulNumReturned = 1;
+    pDJP->ulProperty = DJP_SJ_ORIENTATION;
+    pDJP->ulValue = orientation == nsIPrintSettings::kPortraitOrientation?DJP_ORI_PORTRAIT:DJP_ORI_LANDSCAPE;
+    pDJP++;
+
+    // Setup Number of Copies
+    PRInt32 copies;
+    aPrintSettings->GetNumCopies(&copies);
+    pDJP->cb = sizeof(DJP_ITEM);
+    pDJP->lType = DJP_CURRENT;
+    pDJP->ulNumReturned = 1;
+    pDJP->ulProperty = DJP_SJ_COPIES;
+    pDJP->ulValue = copies;
+    pDJP++;
+
+    pDJP->cb = sizeof(DJP_ITEM);
+    pDJP->lType = DJP_NONE;
+    pDJP->ulNumReturned = 1;
+    pDJP->ulProperty = 0;
+    pDJP->ulValue = 0;
+
+    long rc = GreEscape (hdc, DEVESC_SETJOBPROPERTIES, bufferSize, pDJP_Buffer, 
+                         nsDeviceContextSpecOS2::PrnDlg.GetPrintDriverSize(printer), 
+                         PBYTE(nsDeviceContextSpecOS2::PrnDlg.GetPrintDriver(printer)));
+
+    delete [] pDJP_Buffer;
+    DevCloseDC(hdc);
+  }
+}
+
+nsresult nsDeviceContextSpecOS2::SetPrintSettingsFromDevMode(nsIPrintSettings* aPrintSettings, int printer)
+{
+  if (aPrintSettings == nsnull)
+    return NS_ERROR_FAILURE;
+
+  int bufferSize = 3 * sizeof(DJP_ITEM);
+  PBYTE pDJP_Buffer = new BYTE[bufferSize];
+  memset(pDJP_Buffer, 0, bufferSize);
+  PDJP_ITEM pDJP = (PDJP_ITEM) pDJP_Buffer;
+
+  HDC hdc = nsDeviceContextSpecOS2::PrnDlg.GetDCHandle(printer);
+
+  //Get Number of Copies from Job Properties
+  pDJP->lType = DJP_CURRENT;
+  pDJP->cb = sizeof(DJP_ITEM);
+  pDJP->ulNumReturned = 1;
+  pDJP->ulProperty = DJP_SJ_COPIES;
+  pDJP->ulValue = 1;
+  pDJP++;
+
+  //Get Orientation from Job Properties
+  pDJP->lType = DJP_CURRENT;
+  pDJP->cb = sizeof(DJP_ITEM);
+  pDJP->ulNumReturned = 1;
+  pDJP->ulProperty = DJP_SJ_ORIENTATION;
+  pDJP->ulValue = 1;
+  pDJP++;
+
+  pDJP->lType = DJP_NONE;
+  pDJP->cb = sizeof(DJP_ITEM);
+  pDJP->ulNumReturned = 1;
+  pDJP->ulProperty = 0;
+  pDJP->ulValue = 0;
+
+  long rc = GreEscape(hdc, DEVESC_QUERYJOBPROPERTIES, bufferSize, pDJP_Buffer, 
+                      nsDeviceContextSpecOS2::PrnDlg.GetPrintDriverSize(printer),
+                      PBYTE(nsDeviceContextSpecOS2::PrnDlg.GetPrintDriver(printer)));
+
+  pDJP = (PDJP_ITEM) pDJP_Buffer;
+  if ((rc == DEV_OK) || (rc == DEV_WARNING)) { 
+    while (pDJP->lType != DJP_NONE) {
+      if ((pDJP->ulProperty == DJP_SJ_ORIENTATION) && (pDJP->lType > 0)){
+        if ((pDJP->ulValue == DJP_ORI_PORTRAIT) || (pDJP->ulValue == DJP_ORI_REV_PORTRAIT))
+          aPrintSettings->SetOrientation(nsIPrintSettings::kPortraitOrientation);
+        else
+         aPrintSettings->SetOrientation(nsIPrintSettings::kLandscapeOrientation);
+      }
+      if ((pDJP->ulProperty == DJP_SJ_COPIES) && (pDJP->lType > 0)){
+        aPrintSettings->SetNumCopies(PRInt32(pDJP->ulValue));
+      }
+      pDJP = DJP_NEXT_STRUCTP(pDJP);
+    }
+  }
+  
+  delete [] pDJP_Buffer;
+  DevCloseDC(hdc);  
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDeviceContextSpecOS2 :: QueryInterface(REFNSIID aIID, void** aInstancePtr)
 {
   if (nsnull == aInstancePtr)
@@ -145,146 +251,79 @@ NS_IMPL_RELEASE(nsDeviceContextSpecOS2)
  * toolkits including:
  * - GTK+-toolkit:
  *   file:     mozilla/gfx/src/gtk/nsDeviceContextSpecG.cpp
- *   function: NS_IMETHODIMP nsDeviceContextSpecGTK::Init(PRBool aQuiet)
+ *   function: NS_IMETHODIMP nsDeviceContextSpecGTK::Init(PRBool aPrintPreview )
  * - Xlib-toolkit: 
  *   file:     mozilla/gfx/src/xlib/nsDeviceContextSpecXlib.cpp 
- *   function: NS_IMETHODIMP nsDeviceContextSpecXlib::Init(PRBool aQuiet)
+ *   function: NS_IMETHODIMP nsDeviceContextSpecXlib::Init(PRBool aPrintPreview )
  * - Qt-toolkit:
  *   file:     mozilla/gfx/src/qt/nsDeviceContextSpecQT.cpp
- *   function: NS_IMETHODIMP nsDeviceContextSpecQT::Init(PRBool aQuiet)
+ *   function: NS_IMETHODIMP nsDeviceContextSpecQT::Init(PRBool aPrintPreview )
  * 
  * ** Please update the other toolkits when changing this function.
  */
-NS_IMETHODIMP nsDeviceContextSpecOS2::Init(nsIPrintSettings* aPS, PRBool aQuiet)
+NS_IMETHODIMP nsDeviceContextSpecOS2::Init(nsIPrintSettings* aPS, PRBool aIsPrintPreview)
 {
   nsresult rv = NS_ERROR_FAILURE;
 
   mPrintSettings = aPS;
   NS_ASSERTION(aPS, "Must have a PrintSettings!");
 
-  // if there is a current selection then enable the "Selection" radio button
-  if (aPS != nsnull) {
-    PRBool isOn;
-    aPS->GetPrintOptions(nsIPrintSettings::kEnableSelectionRB, &isOn);
-    nsCOMPtr<nsIPref> pPrefs = do_GetService(NS_PREF_CONTRACTID, &rv);
-    if (NS_SUCCEEDED(rv)) {
-      (void) pPrefs->SetBoolPref("print.selection_radio_enabled", isOn);
-    }
-  }
-
-  char      *path;
-  PRBool     canPrint       = PR_FALSE;
-  PRBool     tofile         = PR_FALSE;
-  PRInt16    printRange     = nsIPrintSettings::kRangeAllPages;
-  PRInt32    fromPage       = 1;
-  PRInt32    toPage         = 1;
-  PRInt32    copies         = 1;
-  PRUnichar *printer        = nsnull;
-  PRUnichar *printfile      = nsnull;
-
   rv = GlobalPrinters::GetInstance()->InitializeGlobalPrinters();
   if (NS_FAILED(rv)) {
     return rv;
   }
  
-  if ( !aQuiet ) {
-    rv = NS_ERROR_FAILURE;
-    // create a nsISupportsArray of the parameters 
-    // being passed to the window
-    nsCOMPtr<nsISupportsArray> array;
-    NS_NewISupportsArray(getter_AddRefs(array));
-    if (!array) return NS_ERROR_FAILURE;
+  if (aPS) {
+    PRBool     tofile         = PR_FALSE;
+    PRInt32    copies         = 1;
+    PRUnichar *printer        = nsnull;
+    PRUnichar *printfile      = nsnull;
 
-    nsCOMPtr<nsIPrintSettings> ps = aPS;
-    nsCOMPtr<nsISupports> psSupports(do_QueryInterface(ps));
-    NS_ASSERTION(psSupports, "PrintSettings must be a supports");
-    array->AppendElement(psSupports);
+    mPrintSettings->GetPrinterName(&printer);
+    mPrintSettings->GetToFileName(&printfile);
+    mPrintSettings->GetPrintToFile(&tofile);
+    mPrintSettings->GetNumCopies(&copies);
 
-    nsCOMPtr<nsIDialogParamBlock> ioParamBlock(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
-    if (ioParamBlock) {
-      ioParamBlock->SetInt(0, 0);
-      nsCOMPtr<nsISupports> blkSupps(do_QueryInterface(ioParamBlock));
-      NS_ASSERTION(blkSupps, "IOBlk must be a supports");
-
-      array->AppendElement(blkSupps);
-      nsCOMPtr<nsISupports> arguments(do_QueryInterface(array));
-      NS_ASSERTION(array, "array must be a supports");
-
-      nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
-      if (wwatch) {
-        nsCOMPtr<nsIDOMWindow> active;
-        wwatch->GetActiveWindow(getter_AddRefs(active));    
-        nsCOMPtr<nsIDOMWindowInternal> parent = do_QueryInterface(active);
-
-        nsCOMPtr<nsIDOMWindow> newWindow;
-        rv = wwatch->OpenWindow(parent, "chrome://global/content/printdialog.xul",
-              "_blank", "chrome,modal,centerscreen", array,
-              getter_AddRefs(newWindow));
-      }
+    if ((copies == 0)  ||  (copies > 999)) {
+       GlobalPrinters::GetInstance()->FreeGlobalPrinters();
+       return NS_ERROR_FAILURE;
     }
 
-    if (NS_SUCCEEDED(rv)) {
-      PRInt32 buttonPressed = 0;
-      ioParamBlock->GetInt(0, &buttonPressed);
-      if (buttonPressed == 1) 
-        canPrint = PR_TRUE;
-      else 
-      {
-         GlobalPrinters::GetInstance()->FreeGlobalPrinters();
-         return NS_ERROR_ABORT;
-      }
-    } else 
-       return NS_ERROR_ABORT;
-  } else {
-    canPrint = PR_TRUE;
-  }
-
-  if (canPrint) {
-    if (mPrintSettings) {
-      mPrintSettings->GetPrinterName(&printer);
-      mPrintSettings->GetPrintRange(&printRange);
-      mPrintSettings->GetToFileName(&printfile);
-      mPrintSettings->GetPrintToFile(&tofile);
-      mPrintSettings->GetStartPageRange(&fromPage);
-      mPrintSettings->GetEndPageRange(&toPage);
-      mPrintSettings->GetNumCopies(&copies);
-
-      if ((copies == 0)  ||  (copies > 999)) {
-         GlobalPrinters::GetInstance()->FreeGlobalPrinters();
-         return NS_ERROR_FAILURE;
-      }
-
-      if (printfile != nsnull) {
-        // ToDo: Use LocalEncoding instead of UTF-8 (see bug 73446)
-        strcpy(mPrData.path,    NS_ConvertUCS2toUTF8(printfile).get());
-      }
-      if (printer != nsnull) 
-        strcpy(mPrData.printer, NS_ConvertUCS2toUTF8(printer).get());  
+    if (printfile != nsnull) {
+      // ToDo: Use LocalEncoding instead of UTF-8 (see bug 73446)
+      strcpy(mPrData.path,    NS_ConvertUCS2toUTF8(printfile).get());
     }
+    if (printer != nsnull) 
+      strcpy(mPrData.printer, NS_ConvertUCS2toUTF8(printer).get());  
 
-    mPrData.toPrinter = !tofile;
+    if (aIsPrintPreview) 
+      mPrData.destination = printPreview; 
+    else if (tofile)  
+      mPrData.destination = printToFile;
+    else  
+      mPrData.destination = printToPrinter;
     mPrData.copies = copies;
 
     rv = GlobalPrinters::GetInstance()->InitializeGlobalPrinters();
     if (NS_FAILED(rv))
       return rv;
 
+    const nsAFlatString& printerUCS2 = NS_ConvertUTF8toUCS2(mPrData.printer);
     int numPrinters = GlobalPrinters::GetInstance()->GetNumPrinters();
     if (numPrinters) {
        for(int i = 0; (i < numPrinters) && !mQueue; i++) {
-          if (!(GlobalPrinters::GetInstance()->GetStringAt(i)->CompareWithConversion(mPrData.printer, TRUE, -1)))
+          if ((GlobalPrinters::GetInstance()->GetStringAt(i)->Equals(printerUCS2, nsCaseInsensitiveStringComparator()))) {
+             SetupDevModeFromSettings(i, aPS);
              mQueue = PrnDlg.SetPrinterQueue(i);
+          }
        }
     }
 
     if (printfile != nsnull) 
       nsMemory::Free(printfile);
-    
+  
     if (printer != nsnull) 
       nsMemory::Free(printer);
-    
-    GlobalPrinters::GetInstance()->FreeGlobalPrinters();
-    return NS_OK;
   }
 
   GlobalPrinters::GetInstance()->FreeGlobalPrinters();
@@ -292,9 +331,9 @@ NS_IMETHODIMP nsDeviceContextSpecOS2::Init(nsIPrintSettings* aPS, PRBool aQuiet)
 }
 
 
-NS_IMETHODIMP nsDeviceContextSpecOS2 :: GetToPrinter( PRBool &aToPrinter )     
+NS_IMETHODIMP nsDeviceContextSpecOS2 :: GetDestination( int &aDestination )     
 {
-  aToPrinter = mPrData.toPrinter;
+  aDestination = mPrData.destination;
   return NS_OK;
 }
 
@@ -345,15 +384,6 @@ nsPrinterEnumeratorOS2::nsPrinterEnumeratorOS2()
 
 NS_IMPL_ISUPPORTS1(nsPrinterEnumeratorOS2, nsIPrinterEnumerator)
 
-NS_IMETHODIMP nsPrinterEnumeratorOS2::EnumeratePrintersExtended(PRUint32* aCount, PRUnichar*** aResult)
-{
-  NS_ENSURE_ARG(aCount);
-  NS_ENSURE_ARG_POINTER(aResult);
-  *aCount = 0;
-  *aResult = nsnull;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsPrinterEnumeratorOS2::EnumeratePrinters(PRUint32* aCount, PRUnichar*** aResult)
 {
   NS_ENSURE_ARG(aCount);
@@ -406,6 +436,36 @@ NS_IMETHODIMP nsPrinterEnumeratorOS2::EnumeratePrinters(PRUint32* aCount, PRUnic
   return NS_OK;
 }
 
+NS_IMETHODIMP nsPrinterEnumeratorOS2::GetDefaultPrinterName(PRUnichar * *aDefaultPrinterName)
+{
+  NS_ENSURE_ARG_POINTER(aDefaultPrinterName);
+  GlobalPrinters::GetInstance()->GetDefaultPrinterName(*aDefaultPrinterName);
+  return NS_OK;
+}
+
+/* void initPrintSettingsFromPrinter (in wstring aPrinterName, in nsIPrintSettings aPrintSettings); */
+NS_IMETHODIMP nsPrinterEnumeratorOS2::InitPrintSettingsFromPrinter(const PRUnichar *aPrinterName, nsIPrintSettings *aPrintSettings)
+{
+   NS_ENSURE_ARG_POINTER(aPrinterName);
+   NS_ENSURE_ARG_POINTER(aPrintSettings);
+
+   if (!*aPrinterName) 
+     return NS_OK;
+
+  if (NS_FAILED(GlobalPrinters::GetInstance()->InitializeGlobalPrinters())) 
+    return NS_ERROR_FAILURE;
+
+  PRInt32 numPrinters = GlobalPrinters::GetInstance()->GetNumPrinters();
+  for(int i = 0; i < numPrinters; i++) {
+    if ((GlobalPrinters::GetInstance()->GetStringAt(i)->Equals(aPrinterName, nsCaseInsensitiveStringComparator()))) 
+      nsDeviceContextSpecOS2::SetPrintSettingsFromDevMode(aPrintSettings, i);
+  }
+
+  // Free them, we won't need them for a while
+  GlobalPrinters::GetInstance()->FreeGlobalPrinters();
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsPrinterEnumeratorOS2::DisplayPropertiesDlg(const PRUnichar *aPrinter, nsIPrintSettings *aPrintSettings)
 {
   nsresult rv = GlobalPrinters::GetInstance()->InitializeGlobalPrinters();
@@ -416,11 +476,14 @@ NS_IMETHODIMP nsPrinterEnumeratorOS2::DisplayPropertiesDlg(const PRUnichar *aPri
   PRInt32 numPrinters = GlobalPrinters::GetInstance()->GetNumPrinters();
 
   for(int i = 0; i < numPrinters; i++) {
-    if (!(GlobalPrinters::GetInstance()->GetStringAt(i)->CompareWithConversion(aPrinter, TRUE, -1))) {
-       if ( nsDeviceContextSpecOS2::PrnDlg.ShowProperties(i) ) 
+    if ((GlobalPrinters::GetInstance()->GetStringAt(i)->Equals(aPrinter, nsCaseInsensitiveStringComparator()))) {
+       SetupDevModeFromSettings(i, aPrintSettings);
+       if ( nsDeviceContextSpecOS2::PrnDlg.ShowProperties(i) ) {
+          nsDeviceContextSpecOS2::SetPrintSettingsFromDevMode(aPrintSettings, i);
           return NS_OK;
-       else
+       } else {
           return NS_ERROR_FAILURE;
+       }
     }
   }
   return NS_ERROR_FAILURE;
@@ -442,15 +505,30 @@ nsresult GlobalPrinters::InitializeGlobalPrinters ()
 
   int defaultPrinter = nsDeviceContextSpecOS2::PrnDlg.GetDefaultPrinter();
   for (int i = 0; i < mGlobalNumPrinters; i++) {
-    char *printer = nsDeviceContextSpecOS2::PrnDlg.GetPrinter(i);
+    nsXPIDLCString printer;
+    nsDeviceContextSpecOS2::PrnDlg.GetPrinter(i, getter_Copies(printer));
     if ( defaultPrinter == i ) 
-       mGlobalPrinterList->InsertStringAt(nsString(NS_ConvertASCIItoUCS2(printer)), 0);
+       mGlobalPrinterList->InsertStringAt(NS_ConvertASCIItoUCS2(printer), 0);
     else 
-       mGlobalPrinterList->AppendString(nsString(NS_ConvertASCIItoUCS2(printer)));
+       mGlobalPrinterList->AppendString(NS_ConvertASCIItoUCS2(printer));
   } 
   return NS_OK;
 }
 
+void GlobalPrinters::GetDefaultPrinterName(PRUnichar*& aDefaultPrinterName)
+{
+  aDefaultPrinterName = nsnull;
+
+  if (GetNumPrinters() == 0)
+     return;
+
+  int defaultPrinter = nsDeviceContextSpecOS2::PrnDlg.GetDefaultPrinter();
+  nsXPIDLCString printer;
+  nsDeviceContextSpecOS2::PrnDlg.GetPrinter(defaultPrinter, getter_Copies(printer));
+
+  aDefaultPrinterName = ToNewUnicode(NS_ConvertASCIItoUCS2(printer));
+}
+
 void GlobalPrinters::FreeGlobalPrinters()
 {
   delete mGlobalPrinterList;
@@ -463,7 +541,6 @@ void GlobalPrinters::FreeGlobalPrinters()
 // OS/2 Printing   - was in libprint.cpp
 //---------------------------------------------------------------------------
 static HMODULE hmodRes;
-static BOOL prnEscape (HDC hdc, long lEscape);
 
 #define SHIFT_PTR(ptr,offset) ( *((LONG*)&ptr) += offset )
 
@@ -612,16 +689,16 @@ int PRINTDLG::GetDefaultPrinter ()
    return mDefaultQueue;
 }
 
-char* PRINTDLG::GetPrinter (int numPrinter)
+PRINTDLG::GetPrinter (int numPrinter, char** printerName)
 {
-   const char* pq = NULL;
-  
    if (numPrinter > mQueueCount)
       return NULL;
-
-   pq = mPQBuf [numPrinter]->QueueName();
-
-   return (char *)pq;
+ 
+   nsCAutoString pName(mPQBuf [numPrinter]->QueueName());
+ 
+   pName.ReplaceChar('\r', ' ');
+   pName.StripChars("\n");
+   *printerName = ToNewCString(pName);
 }
 
 PRTQUEUE* PRINTDLG::SetPrinterQueue (int numPrinter)
@@ -636,6 +713,47 @@ PRTQUEUE* PRINTDLG::SetPrinterQueue (int numPrinter)
    return new PRTQUEUE (*pPQ);
 }
 
+PLONG PRINTDLG::GetPrintDriverSize (int printer)
+{
+   LONG size = mPQBuf[GetIndex(printer)]->PQI3().pDriverData->cb;
+
+   PLONG driverSize = (PLONG)&size;
+   return driverSize;
+}
+
+PDRIVDATA PRINTDLG::GetPrintDriver (int printer)
+{
+   if (printer > mQueueCount)
+      return NULL;
+
+   return mPQBuf[GetIndex(printer)]->PQI3().pDriverData;
+}
+
+HDC PRINTDLG::GetDCHandle (int numPrinter)
+{
+    HDC hdc = 0;
+    int index = GetIndex(numPrinter);
+    DEVOPENSTRUC dop;
+
+    dop.pszLogAddress      = 0; 
+    dop.pszDriverName      = (char *) mPQBuf[index]->DriverName();
+    dop.pdriv              = mPQBuf[index]->PQI3 ().pDriverData;
+    dop.pszDataType        = 0; 
+    dop.pszComment         = 0;
+    dop.pszQueueProcName   = 0;     
+    dop.pszQueueProcParams = 0;   
+    dop.pszSpoolerParams   = 0;     
+    dop.pszNetworkParams   = 0;     
+
+    hdc = ::DevOpenDC( 0, OD_INFO, "*", 9, (PDEVOPENDATA) &dop, NULLHANDLE);
+    return hdc;
+}
+
+char* PRINTDLG::GetDriverType (int printer)
+{
+  return (char *)mPQBuf[GetIndex(printer)]->DriverName ();
+}
+
 BOOL PRINTDLG::ShowProperties (int index)
 {
     BOOL          rc = FALSE;
@@ -689,7 +807,7 @@ BOOL PRINTDLG::ShowProperties (int index)
 /*  Job management                                                          */
 /****************************************************************************/
 
-HDC PrnOpenDC( PRTQUEUE *pInfo, PSZ pszApplicationName, int copies, int toPrinter, char *file )
+HDC PrnOpenDC( PRTQUEUE *pInfo, PSZ pszApplicationName, int copies, int destination, char *file )
 {
    HDC hdc = 0;
    PSZ pszLogAddress;
@@ -700,15 +818,13 @@ HDC PrnOpenDC( PRTQUEUE *pInfo, PSZ pszApplicationName, int copies, int toPrinte
    if (!pInfo || !pszApplicationName)
       return hdc;
 
-   char pszQueueProcParams[CCHMAXPATH] = "COP=";
-   char numCopies[12];
-   itoa (copies, numCopies, 10);
-   strcat (pszQueueProcParams, numCopies);
-
-   if ( toPrinter ) {
+   if ( destination ) {
       pszLogAddress = pInfo->PQI3 ().pszName;
       pszDataType = "PM_Q_STD";
-      dcType = OD_QUEUED;
+      if ( destination == 2 )
+         dcType = OD_METAFILE;
+      else
+         dcType = OD_QUEUED;
    } else {
       if (file && strlen(file) != 0) 
          pszLogAddress = (PSZ) file;
@@ -724,7 +840,7 @@ HDC PrnOpenDC( PRTQUEUE *pInfo, PSZ pszApplicationName, int copies, int toPrinte
     dop.pszDataType        = pszDataType; 
     dop.pszComment         = pszApplicationName;
     dop.pszQueueProcName   = pInfo->PQI3 ().pszPrProc;     
-    dop.pszQueueProcParams = pszQueueProcParams;   
+    dop.pszQueueProcParams = 0;
     dop.pszSpoolerParams   = 0;     
     dop.pszNetworkParams   = 0;     
 
@@ -741,21 +857,6 @@ if (hdc == 0)
    return hdc;
 }
 
-BOOL prnEscape( HDC hdc, long lEscape)
-{
-   BOOL rc = FALSE;
-
-   if( hdc)
-   {
-      long lDummy = 0;
-      long lResult = ::DevEscape( hdc, lEscape, 0, NULL, &lDummy, NULL);
-      rc = (lResult == DEV_OK);
-   }
-
-   return rc;
-}
-
-
 /* find the selected form */
 BOOL PrnQueryHardcopyCaps( HDC hdc, PHCINFO pHCInfo)
 {
