@@ -52,23 +52,48 @@
 #include "Symbol.h"
 #include "nsReadableUtils.h"
 
+struct SubstituteMap {
+  const char *name;
+  PRUint8    italic;
+  PRBool     bold;
+  PRInt16    index;
+};
 
+static SubstituteMap gSubstituteMap[] = {
+  { "serif", 0,0,0},
+  { "serif", 0,1,1},
+  { "serif", 1,1,2},
+  { "serif", 1,0,3},
+  { "sans-serif", 0,0,4},
+  { "sans-serif", 0,1,5},
+  { "sans-serif", 1,1,6},
+  { "sans-serif", 1,0,7},
+  { "monospace", 0,0,8},
+  { "monospace", 0,1,9},
+  { "monospace", 1,1,10},
+  { "monospace", 1,0,11},
+};
+
+
+static  const PRUint32 gNumSubstituteMap = sizeof(gSubstituteMap)/sizeof(SubstituteMap);
+
+#define NS_IS_BOLD(weight)  ((weight) >= 401 ? 1 : 0)
 
 // this is the basic font set supported currently
 DefFonts gSubstituteFonts[] = 
 {
   {"Times-Roman","Times",400,0,&Times_RomanAFM,AFMTimes_RomanChars,-1},
-  {"Times-Bold","Times",700,0,&Times_BoldAFM,AFMTimes_RomanChars,-1},
-  {"Times-BoldItalic","Times",700,1,&Times_BoldItalicAFM,AFMTimes_RomanChars,-1},
-  {"Times-Italic","Times",400,1,&Times_ItalicAFM,AFMTimes_RomanChars,-1},
+  {"Times-Bold","Times",700,0,&Times_BoldAFM,AFMTimes_BoldChars,-1},
+  {"Times-BoldItalic","Times",700,1,&Times_BoldItalicAFM,AFMTimes_BoldItalicChars,-1},
+  {"Times-Italic","Times",400,1,&Times_ItalicAFM,AFMTimes_ItalicChars,-1},
   {"Helvetica","Helvetica",400,0,&HelveticaAFM,AFMHelveticaChars,-1},
-  {"Helvetica-Bold","Helvetica",700,0,&Helvetica_BoldAFM,AFMHelveticaChars,-1},
-  {"Helvetica-BoldOblique","Helvetica",700,2,&Helvetica_BoldObliqueAFM,AFMHelveticaChars,-1},
-  {"Helvetica_Oblique","Helvetica",400,2,&Helvetica_ObliqueAFM,AFMHelveticaChars,-1},
+  {"Helvetica-Bold","Helvetica",700,0,&Helvetica_BoldAFM,AFMHelvetica_BoldChars,-1},
+  {"Helvetica-BoldOblique","Helvetica",700,2,&Helvetica_BoldObliqueAFM,AFMHelvetica_BoldObliqueChars,-1},
+  {"Helvetica_Oblique","Helvetica",400,2,&Helvetica_ObliqueAFM,AFMHelvetica_ObliqueChars,-1},
   {"Courier","Courier",400,0,&CourierAFM,AFMCourierChars,-1},
-  {"Courier-Bold","Courier",700,0,&Courier_BoldAFM,AFMCourierChars,-1},
-  {"Courier-BoldOblique","Courier",700,2,&Courier_BoldObliqueAFM,AFMCourierChars,-1},
-  {"Courier-Oblique","Courier",400,2,&Courier_ObliqueAFM,AFMCourierChars,-1},
+  {"Courier-Bold","Courier",700,0,&Courier_BoldAFM,AFMCourier_BoldChars,-1},
+  {"Courier-BoldOblique","Courier",700,2,&Courier_BoldObliqueAFM,AFMCourier_BoldObliqueChars,-1},
+  {"Courier-Oblique","Courier",400,2,&Courier_ObliqueAFM,AFMCourier_ObliqueChars,-1},
   {"Symbol","Symbol",400,0,&SymbolAFM,AFMSymbolChars,-1}
 };
 
@@ -79,7 +104,7 @@ DefFonts gSubstituteFonts[] =
  */
 static struct keyname_st
 {
-  char *name;
+  const char *name;
   AFMKey key;
 } keynames[] =
 {
@@ -277,15 +302,36 @@ PRInt16
 nsAFMObject::CreateSubstituteFont(const nsFont &aFontName)
 {
 PRInt16     ourfont = 0;
+PRUint32    i;
+
+ 
+ for(i=0;i<gNumSubstituteMap;i++){
+    if(aFontName.name.EqualsWithConversion(gSubstituteMap[i].name, PR_TRUE) && 
+       (aFontName.style != NS_FONT_STYLE_NORMAL)== gSubstituteMap[i].italic &&
+       NS_IS_BOLD(aFontName.weight) == gSubstituteMap[i].bold) {
+      ourfont = gSubstituteMap[i].index;
+      break;
+   }
+  }
+
+  // no matches in the substitution table -- default to times
+  if(i == gNumSubstituteMap){
+    printf(" NO FONT WAS FOUND \n");
+    if(aFontName.style == NS_FONT_STYLE_NORMAL){
+      ourfont = NS_IS_BOLD(aFontName.weight) ? 1 : 0;
+    } else {
+      ourfont = NS_IS_BOLD(aFontName.weight) ? 3 : 4;
+    }
+  }
 
   mPSFontInfo = new AFMFontInformation;
   memset(mPSFontInfo,0,sizeof(AFMFontInformation));
 
   // put in default AFM data, can't find the correct AFM file
-  memcpy(mPSFontInfo,&Times_RomanAFM,sizeof(AFMFontInformation));
+  memcpy(mPSFontInfo,gSubstituteFonts[ourfont].mFontInfo,sizeof(AFMFontInformation));
   mPSFontInfo->mAFMCharMetrics = new AFMscm[mPSFontInfo->mNumCharacters];
   memset(mPSFontInfo->mAFMCharMetrics,0,sizeof(AFMscm)*mPSFontInfo->mNumCharacters);
-  memcpy(mPSFontInfo->mAFMCharMetrics,AFMTimes_RomanChars,Times_RomanAFM.mNumCharacters*sizeof(AFMscm));
+  memcpy(mPSFontInfo->mAFMCharMetrics,gSubstituteFonts[ourfont].mCharInfo,Times_RomanAFM.mNumCharacters*sizeof(AFMscm));
   return ourfont;
 }
 
@@ -494,7 +540,7 @@ PRBool  found = PR_FALSE;
 
   while((upper >=lower) && !found) {
     midpoint = (lower+upper)/2;
-    if(keynames[midpoint].name == NULL){
+    if(keynames[midpoint].name == nsnull) {
       break;
     }
     cmpvalue = strcmp(aKey,keynames[midpoint].name);
@@ -617,7 +663,7 @@ void
 nsAFMObject::ReadCharMetrics (AFMFontInformation *aFontInfo,PRInt32 aNumCharacters)
 {
 PRInt32 i = 0,ivalue,first=1;
-AFMscm  *cm = NULL;
+AFMscm  *cm = nsnull;
 AFMKey  key;
 PRBool  done = PR_FALSE;
 double  notyet;
@@ -787,15 +833,24 @@ float     totallen=0.0f;
  cptr = (PRUnichar*)aString;
 
   for(i=0;i<aLength;i++,cptr++){
-    asciichar = (*cptr)&0x00ff;
-    idx = asciichar-32;
-    fwidth = (PRInt32)(mPSFontInfo->mAFMCharMetrics[idx].mW0x);
-    //    if ( (*cptr == 0x0020) || (*cptr == 0x002c) )
-    //   printf("fwidth = %d\n", fwidth);
+    fwidth = 0;
     if (*cptr & 0xff00)
-       fwidth = 1056;
-    if ( (*cptr  == 0x0020) || (*cptr == 0x002c) )
-      fwidth = 1056;  // space and comma are half size of a CJK width
+    {
+      if (0x0400 == (*cptr & 0xff00)) { // Cyrillic
+        fwidth = 600;
+      } else {
+        fwidth = 1056;
+      }
+    } else {
+      // < 0x0100
+      asciichar = (*cptr)&0x00ff;
+      idx = asciichar-32;
+      if(idx >= 0 )
+        fwidth = (PRInt32)(mPSFontInfo->mAFMCharMetrics[idx].mW0x);
+      else if (*cptr  == 0x0020) 
+        fwidth = 1056;
+    }
+
     totallen += fwidth;
   }
 
