@@ -20,6 +20,7 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -38,16 +39,19 @@
 
 #include "nsRenderingContextPS.h"
 #include "nsFontMetricsPS.h"
-#include <math.h>
 #include "nsDeviceContextPS.h"
 #include "nsPostScriptObj.h"  
 #include "nsIRegion.h"      
-#include "nsIImage.h"      
+#include "nsIImage.h"
+#include "imgIContainer.h"
+#include "gfxIImageFrame.h"
+#include "nsIInterfaceRequestor.h"
+#include "nsIInterfaceRequestorUtils.h"
 
-static NS_DEFINE_IID(kIRenderingContextIID, NS_IRENDERING_CONTEXT_IID);
+#include <math.h>
 
 // Macro to convert from TWIPS (1440 per inch) to POINTS (72 per inch)
-#define NS_PIXELS_TO_POINTS(x) (x * 10)
+#define NS_PIXELS_TO_POINTS(x) ((x) * 10)
 
 #define FLAG_CLIP_VALID       0x0001
 #define FLAG_CLIP_CHANGED     0x0002
@@ -70,7 +74,7 @@ public:
   PS_State        *mNext;
   nsTransform2D   mMatrix;
   nsRect          mLocalClip;
-  nsIFontMetrics  *mFontMetrics;
+  nsCOMPtr<nsIFontMetrics>  mFontMetrics;
   nscolor         mCurrentColor;
   nscolor         mTextColor;
   nsLineStyle     mLineStyle;
@@ -83,13 +87,13 @@ public:
  */
 PS_State :: PS_State()
 {
-  mNext = nsnull;
+  mNext         = nsnull;
   mMatrix.SetToIdentity();  
   mLocalClip.x = mLocalClip.y = mLocalClip.width = mLocalClip.height = 0;
-  mFontMetrics = nsnull;
+  mFontMetrics  = nsnull;
   mCurrentColor = NS_RGB(0, 0, 0);
-  mTextColor = NS_RGB(0, 0, 0);
-  mLineStyle = nsLineStyle_kSolid;
+  mTextColor    = NS_RGB(0, 0, 0);
+  mLineStyle    = nsLineStyle_kSolid;
 }
 
 /** ---------------------------------------------------
@@ -97,13 +101,15 @@ PS_State :: PS_State()
  *  Default Constructor for the state
  *	@update 12/21/98 dwc
  */
-PS_State :: PS_State(PS_State &aState):mMatrix(&aState.mMatrix),mLocalClip(aState.mLocalClip)
+PS_State :: PS_State(PS_State &aState) : 
+  mMatrix(&aState.mMatrix),
+  mLocalClip(aState.mLocalClip)
 {
   mNext = &aState;
-  //mClipRegion = NULL;
+  //mClipRegion = nsnull;
   mCurrentColor = aState.mCurrentColor;
   mFontMetrics = nsnull;
-  //mFont = NULL;
+  //mFont = nsnull;
   mFlags = ~FLAGS_ALL;
   mTextColor = aState.mTextColor;
   mLineStyle = aState.mLineStyle;
@@ -115,18 +121,17 @@ PS_State :: PS_State(PS_State &aState):mMatrix(&aState.mMatrix),mLocalClip(aStat
  */
 PS_State :: ~PS_State()
 {
-  //if (NULL != mClipRegion){
+  //if (nsnull != mClipRegion){
     //VERIFY(::DeleteObject(mClipRegion));
-    //mClipRegion = NULL;
+    //mClipRegion = nsnull;
   //}
 
   //don't delete this because it lives in the font metrics
-  //mFont = NULL;
+  //mFont = nsnull;
 }
 
 
-static NS_DEFINE_IID(kRenderingContextIID, NS_IRENDERING_CONTEXT_IID);
-
+NS_IMPL_THREADSAFE_ISUPPORTS1(nsRenderingContextPS, nsIRenderingContext)
 
 /** ---------------------------------------------------
  *  See documentation in nsIRenderingContext.h
@@ -142,25 +147,25 @@ nsRenderingContextPS :: nsRenderingContextPS()
 
   mStateCache = new nsVoidArray();
 
-  PushState();
-
   mP2T = 1.0f;
+
+  PushState();
 }
 
 /** ---------------------------------------------------
  *  See documentation in nsIRenderingContext.h
  *	@update 12/21/98 dwc
  */
-nsRenderingContextPS :: ~nsRenderingContextPS()
+nsRenderingContextPS::~nsRenderingContextPS()
 {
-  if (nsnull != mStateCache){
+  if (mStateCache){
     PRInt32 cnt = mStateCache->Count();
 
     while (--cnt >= 0){
       PS_State *state = (PS_State *)mStateCache->ElementAt(cnt);
       mStateCache->RemoveElementAt(cnt);
 
-      if (nsnull != state)
+      if (state)
         delete state;
     }
 
@@ -171,58 +176,26 @@ nsRenderingContextPS :: ~nsRenderingContextPS()
   mTranMatrix = nsnull;
 }
 
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-nsresult
-nsRenderingContextPS :: QueryInterface(REFNSIID aIID, void** aInstancePtr)
-{
-
-  if (nsnull == aInstancePtr)
-    return NS_ERROR_NULL_POINTER;
-
-  if (aIID.Equals(kIRenderingContextIID)){
-    nsIRenderingContext* tmp = this;
-    *aInstancePtr = (void*) tmp;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-
-  static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
-
-  if (aIID.Equals(kISupportsIID)){
-    nsIRenderingContext* tmp = this;
-    nsISupports* tmp2 = tmp;
-    *aInstancePtr = (void*) tmp2;
-    NS_ADDREF_THIS();
-    return NS_OK;
-  }
-
-  return NS_NOINTERFACE;
-}
-
-NS_IMPL_ADDREF(nsRenderingContextPS)
-NS_IMPL_RELEASE(nsRenderingContextPS)
-
 /** ---------------------------------------------------
  *  See documentation in nsIRenderingContext.h
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP
-nsRenderingContextPS :: Init(nsIDeviceContext* aContext)
+nsRenderingContextPS::Init(nsIDeviceContext* aContext)
 {
-float app2dev;
+  float app2dev;
+
+  NS_ENSURE_TRUE(nsnull != aContext, NS_ERROR_NULL_POINTER);
 
   mContext = aContext;
-  if (mContext) {
-  	mPSObj = ((nsDeviceContextPS*)mContext)->GetPrintContext();
-  }
-  NS_IF_ADDREF(mContext);
+
+  mPSObj = NS_REINTERPRET_CAST(nsDeviceContextPS *, mContext.get())->GetPrintContext();
+
+  NS_ENSURE_TRUE(nsnull != mPSObj, NS_ERROR_NULL_POINTER);
 
   // initialize the matrix
   mContext->GetAppUnitsToDevUnits(app2dev);
-	mTranMatrix->AddScale(app2dev, app2dev);
+  mTranMatrix->AddScale(app2dev, app2dev);
   mContext->GetDevUnitsToAppUnits(mP2T);
   return NS_OK;
 }
@@ -296,8 +269,8 @@ nsRenderingContextPS :: Reset()
 NS_IMETHODIMP 
 nsRenderingContextPS :: GetDeviceContext(nsIDeviceContext *&aContext)
 {
-  NS_IF_ADDREF(mContext);
   aContext = mContext;
+  NS_IF_ADDREF(aContext);
   return NS_OK;
 }
 
@@ -543,12 +516,25 @@ nsRenderingContextPS :: GetLineStyle(nsLineStyle &aLineStyle)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: SetFont(const nsFont& aFont)
+nsRenderingContextPS::SetFont(const nsFont& aFont, nsIAtom* aLangGroup)
 {
-  NS_IF_RELEASE(mFontMetrics);
-  if (mContext) {
-  	mContext->GetMetricsFor(aFont, mFontMetrics);
+  nsCOMPtr<nsIFontMetrics> newMetrics;
+  nsresult rv = mContext->GetMetricsFor( aFont, aLangGroup, *getter_AddRefs(newMetrics) );
+
+  if (NS_SUCCEEDED(rv)) {
+    rv = SetFont(newMetrics);
   }
+  return rv;
+}
+
+/** ---------------------------------------------------
+ *  See documentation in nsIRenderingContext.h
+ *	@update 12/21/98 dwc
+ */
+NS_IMETHODIMP 
+nsRenderingContextPS::SetFont(nsIFontMetrics *aFontMetrics)
+{
+  mFontMetrics = (nsFontMetricsPS *)aFontMetrics;
   return NS_OK;
 }
 
@@ -557,12 +543,10 @@ nsRenderingContextPS :: SetFont(const nsFont& aFont)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: SetFont(nsIFontMetrics *aFontMetrics)
+nsRenderingContextPS::GetFontMetrics(nsIFontMetrics *&aFontMetrics)
 {
-
-  NS_IF_RELEASE(mFontMetrics);
-  mFontMetrics = aFontMetrics;
-  NS_IF_ADDREF(mFontMetrics);
+  aFontMetrics = (nsIFontMetrics *)mFontMetrics;
+  NS_IF_ADDREF(aFontMetrics);
   return NS_OK;
 }
 
@@ -571,22 +555,9 @@ nsRenderingContextPS :: SetFont(nsIFontMetrics *aFontMetrics)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: GetFontMetrics(nsIFontMetrics *&aFontMetrics)
+nsRenderingContextPS::Translate(nscoord aX, nscoord aY)
 {
-
-  NS_IF_ADDREF(mFontMetrics);
-  aFontMetrics = mFontMetrics;
-  return NS_OK;
-}
-
-/** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *	@update 12/21/98 dwc
- */
-NS_IMETHODIMP 
-nsRenderingContextPS :: Translate(nscoord aX, nscoord aY)
-{
-	mTranMatrix->AddTranslation((float)aX,(float)aY);
+  mTranMatrix->AddTranslation((float)aX,(float)aY);
   return NS_OK;
 }
 
@@ -994,7 +965,7 @@ nsRenderingContextPS :: GetWidth(char ch, nscoord& aWidth)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: GetWidth(PRUnichar ch, nscoord &aWidth, PRInt32 *aFontID)
+nsRenderingContextPS::GetWidth(PRUnichar ch, nscoord &aWidth, PRInt32 *aFontID)
 {
   PRUnichar buf[1];
   buf[0] = ch;
@@ -1006,7 +977,7 @@ nsRenderingContextPS :: GetWidth(PRUnichar ch, nscoord &aWidth, PRInt32 *aFontID
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: GetWidth(const char* aString, nscoord& aWidth)
+nsRenderingContextPS::GetWidth(const char* aString, nscoord& aWidth)
 {
   return GetWidth(aString, strlen(aString),aWidth);
 }
@@ -1016,16 +987,14 @@ nsRenderingContextPS :: GetWidth(const char* aString, nscoord& aWidth)
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: GetWidth(const char* aString,PRUint32 aLength,nscoord& aWidth)
+nsRenderingContextPS::GetWidth(const char* aString,PRUint32 aLength,nscoord& aWidth)
 {
-
-  if (nsnull != mFontMetrics){
-    ((nsFontMetricsPS*)mFontMetrics)->GetStringWidth(aString,aWidth,aLength);
-    return NS_OK;
-  } else {
-    return NS_ERROR_FAILURE;
+  nsresult rv = NS_ERROR_FAILURE;
+  if (mFontMetrics) {
+    rv = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get())->GetStringWidth(aString,aWidth,aLength);
   }
-
+  
+  return rv;
 }
 
 /** ---------------------------------------------------
@@ -1033,7 +1002,7 @@ nsRenderingContextPS :: GetWidth(const char* aString,PRUint32 aLength,nscoord& a
  *	@update 12/21/98 dwc
  */
 NS_IMETHODIMP 
-nsRenderingContextPS :: GetWidth(const nsString& aString, nscoord& aWidth, PRInt32 *aFontID)
+nsRenderingContextPS::GetWidth(const nsString& aString, nscoord& aWidth, PRInt32 *aFontID)
 {
   return GetWidth(aString.get(), aString.Length(), aWidth, aFontID);
 }
@@ -1045,14 +1014,13 @@ nsRenderingContextPS :: GetWidth(const nsString& aString, nscoord& aWidth, PRInt
 NS_IMETHODIMP 
 nsRenderingContextPS :: GetWidth(const PRUnichar *aString,PRUint32 aLength,nscoord &aWidth, PRInt32 *aFontID)
 {
+  nsresult rv = NS_ERROR_FAILURE;
 
-  if (nsnull != mFontMetrics){
-    ((nsFontMetricsPS*)mFontMetrics)->GetStringWidth(aString,aWidth,aLength);
-    return NS_OK;
-  } else {
-    return NS_ERROR_FAILURE;
+  if (mFontMetrics) {
+    rv = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get())->GetStringWidth(aString, aWidth, aLength);
   }
 
+  return rv;
 }
 
 /** --------------------------------------------------- */
@@ -1061,29 +1029,35 @@ NS_IMETHODIMP
 nsRenderingContextPS :: GetTextDimensions(const char* aString, PRUint32 aLength,
                                           nsTextDimensions& aDimensions)
 {
-  if (nsnull != mFontMetrics){
-    ((nsFontMetricsPS*)mFontMetrics)->GetStringWidth(aString,aDimensions.width,aLength);
-     mFontMetrics->GetMaxAscent(aDimensions.ascent);
-     mFontMetrics->GetMaxDescent(aDimensions.descent);
-    return NS_OK;
-  } else {
-    return NS_ERROR_FAILURE;
+  nsresult rv = NS_ERROR_FAILURE;
+
+  if (mFontMetrics) {
+    nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
+    metrics->GetStringWidth(aString, aDimensions.width, aLength);
+    metrics->GetMaxAscent(aDimensions.ascent);
+    metrics->GetMaxDescent(aDimensions.descent);
+    rv = NS_OK;
   }
+  
+  return rv;
 }
 
 NS_IMETHODIMP
 nsRenderingContextPS :: GetTextDimensions(const PRUnichar* aString, PRUint32 aLength,
                                           nsTextDimensions& aDimensions, PRInt32* aFontID)
 {
-  if (nsnull != mFontMetrics){
-    ((nsFontMetricsPS*)mFontMetrics)->GetStringWidth(aString,aDimensions.width,aLength);
+  nsresult rv = NS_ERROR_FAILURE;
+
+  if (mFontMetrics) {
+    nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
+    metrics->GetStringWidth(aString, aDimensions.width, aLength);
      //XXX temporary - bug 96609
-     mFontMetrics->GetMaxAscent(aDimensions.ascent);
-     mFontMetrics->GetMaxDescent(aDimensions.descent);
-    return NS_OK;
-  } else {
-    return NS_ERROR_FAILURE;
+    metrics->GetMaxAscent(aDimensions.ascent);
+    metrics->GetMaxDescent(aDimensions.descent);
+    rv = NS_OK;
   }
+
+  return rv;
 }
 
 /** ---------------------------------------------------
@@ -1112,7 +1086,7 @@ PRInt32       y = aY;
   }
 
 	mTranMatrix->TransformCoord(&x, &y);
-  PostscriptTextOut(aString, aLength, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aLength, (const nscoord*) (aSpacing ? dx0 : NULL), PR_FALSE);
+  PostscriptTextOut(aString, aLength, NS_PIXELS_TO_POINTS(x), NS_PIXELS_TO_POINTS(y), aLength, (const nscoord*) (aSpacing ? dx0 : nsnull), PR_FALSE);
 
   if ((nsnull != aSpacing) && (dx0 != dxMem)) {
     delete [] dx0;
@@ -1121,7 +1095,7 @@ PRInt32       y = aY;
 #if 0
   //this doesn't need to happen here anymore, but a
   //new api will come along that will need this stuff. MMP
-  if (nsnull != mFontMetrics){
+  if (mFontMetrics) {
     nsFont *font;
     mFontMetrics->GetFont(font);
     PRUint8 decorations = font->decorations;
@@ -1153,7 +1127,7 @@ PRInt32         y = aY;
 nsIFontMetrics  *fMetrics;
 
   nsCOMPtr<nsIAtom> langGroup = nsnull;
-  ((nsFontMetricsPS*)mFontMetrics)->GetLangGroup(getter_AddRefs(langGroup));
+  mFontMetrics->GetLangGroup(getter_AddRefs(langGroup));
   mPSObj->setlanggroup(langGroup.get());
 
   /* build up conversion table */
@@ -1179,7 +1153,7 @@ nsIFontMetrics  *fMetrics;
 
   fMetrics = mFontMetrics;
 
-  if (nsnull != fMetrics){
+  if (fMetrics) {
     const nsFont *font;
     fMetrics->GetFont(font);
     PRUint8 decorations = font->decorations;
@@ -1295,18 +1269,10 @@ NS_IMETHODIMP
 nsRenderingContextPS::DrawTile(nsIImage *aImage,nscoord aX0,nscoord aY0,nscoord aX1,nscoord aY1,
                                                     nscoord aWidth,nscoord aHeight)
 {
-
   return NS_OK;
 }
 
 
-#ifdef USE_IMG2
-
-#include "imgIContainer.h"
-#include "gfxIImageFrame.h"
-#include "nsIInterfaceRequestor.h"
-#include "nsIInterfaceRequestorUtils.h"
-
 /* [noscript] void drawImage (in imgIContainer aImage, [const] in nsRect aSrcRect, [const] in nsPoint aDestPoint); */
 NS_IMETHODIMP nsRenderingContextPS::DrawImage(imgIContainer *aImage, const nsRect * aSrcRect, const nsPoint * aDestPoint)
 {
@@ -1317,13 +1283,12 @@ NS_IMETHODIMP nsRenderingContextPS::DrawImage(imgIContainer *aImage, const nsRec
   mTranMatrix->TransformCoord(&pt.x, &pt.y);
 
   sr = *aSrcRect;
-#if 0
-  // need to do this if we fix the comments below
+  // We need to transform the whole source rect, since it has
+  // dimensions in twips and we want pixels
   mTranMatrix->TransformCoord(&sr.x, &sr.y, &sr.width, &sr.height);
 
   sr.x = aSrcRect->x;
   sr.y = aSrcRect->y;
-#endif
   mTranMatrix->TransformNoXLateCoord(&sr.x, &sr.y);
 
   nsCOMPtr<gfxIImageFrame> iframe;
@@ -1349,12 +1314,12 @@ NS_IMETHODIMP nsRenderingContextPS::DrawImage(imgIContainer *aImage, const nsRec
 NS_IMETHODIMP nsRenderingContextPS::DrawScaledImage(imgIContainer *aImage, const nsRect * aSrcRect, const nsRect * aDestRect)
 {
   nsRect dr;
-  nsRect sr;
 
   dr = *aDestRect;
   mTranMatrix->TransformCoord(&dr.x, &dr.y, &dr.width, &dr.height);
 
 #if 0
+  nsRect sr;
   // need to do this if we fix the comments below
   sr = *aSrcRect;
   mTranMatrix->TransformCoord(&sr.x, &sr.y, &sr.width, &sr.height);
@@ -1382,8 +1347,6 @@ NS_IMETHODIMP nsRenderingContextPS::DrawScaledImage(imgIContainer *aImage, const
   return NS_OK;
 }
 
-#endif
-
 #ifdef MOZ_MATHML
   /**
    * Returns metrics (in app units) of an 8-bit character string
@@ -1448,9 +1411,11 @@ nsAutoString    fontFamily;
   //mStates->mFont = mCurrFont = tfont;
   mStates->mFontMetrics = mFontMetrics;
 
+  nsFontMetricsPS *metrics = NS_REINTERPRET_CAST(nsFontMetricsPS *, mFontMetrics.get());
+
   // get the fontfamily we are using, not what we want, but what we are using
-  fontFamily.AssignWithConversion(((nsFontMetricsPS*)mFontMetrics)->mAFMInfo->mPSFontInfo->mFamilyName);
-  fontIndex = ((nsFontMetricsPS*)mFontMetrics)->GetFontIndex();
+  fontFamily.AssignWithConversion(metrics->mAFMInfo->mPSFontInfo->mFamilyName);
+  fontIndex = metrics->GetFontIndex();
 
   mPSObj->setscriptfont(fontIndex,fontFamily,fontHeight,font->style,font->variant,font->weight,font->decorations);
 }
@@ -1516,32 +1481,5 @@ HPEN nsRenderingContextPS :: SetupDottedPen(void)
   return mCurrPen;
 }
 
-#endif
+#endif /* NOTNOW */
 
-
-#ifdef DC
-NS_IMETHODIMP
-nsRenderingContextPS::GetColor(nsString& aColor)
-{
-  char cbuf[40];
-  PR_snprintf(cbuf, sizeof(cbuf), "#%02x%02x%02x",
-              NS_GET_R(mCurrentColor),
-              NS_GET_G(mCurrentColor),
-              NS_GET_B(mCurrentColor));
-  aColor = cbuf;
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsRenderingContextPS::SetColor(const nsString& aColor)
-{
-  nscolor rgb;
-  if (NS_ColorNameToRGB(aColor, &rgb)) {
-    SetColor(rgb);
-  }
-  else if (NS_HexToRGB(aColor, &rgb)) {
-    SetColor(rgb);
-  }
-  return NS_OK;
-}
-#endif
