@@ -47,13 +47,8 @@
 #include "nsGfxCIID.h"
 #include "nsReadableutils.h"
 
-// Print Options
-#include "nsIPrintOptions.h"
 #include "nsString.h"
-static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 
-// Size of the color cube
-#define COLOR_CUBE_SIZE       216
 #define DOC_TITLE_LENGTH      64
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
@@ -351,8 +346,12 @@ NS_IMETHODIMP nsDeviceContextWin :: SetCanonicalPixelScale(float aScale)
 
 NS_IMETHODIMP nsDeviceContextWin :: GetScrollBarDimensions(float &aWidth, float &aHeight) const
 {
-  aWidth = ::GetSystemMetrics(SM_CXVSCROLL) * mDevUnitsToAppUnits;
-  aHeight = ::GetSystemMetrics(SM_CXHSCROLL) * mDevUnitsToAppUnits;
+  float scale;
+  GetCanonicalPixelScale(scale);
+
+  aWidth  = ::GetSystemMetrics(SM_CXVSCROLL) * mDevUnitsToAppUnits * scale;
+  aHeight = ::GetSystemMetrics(SM_CXHSCROLL) * mDevUnitsToAppUnits * scale;
+
   return NS_OK;
 }
 
@@ -499,6 +498,15 @@ nsresult nsDeviceContextWin :: GetSysFontInfo(HDC aHDC, nsSystemFontID anID, nsF
     return NS_ERROR_FAILURE;
   }
 
+  // we have problem on Simplified Chinese system because the system report
+  // the default font size is 8. but if we use 8, the text display very
+  // Ugly. force it to be at least 9 on that system (cp936)
+
+  if ((-9 < ptrLogFont->lfHeight) && 
+      (ptrLogFont->lfHeight < 9) && 
+      (936 == ::GetACP())) 
+    ptrLogFont->lfHeight = (ptrLogFont->lfHeight > 0) ? 9 : -9;
+
   return CopyLogFontToNSFont(&aHDC, ptrLogFont, aFont);
 }
 
@@ -604,225 +612,6 @@ NS_IMETHODIMP nsDeviceContextWin::GetDepth(PRUint32& aDepth)
   return NS_OK;
 }
 
-#define COLOR_CUBE_SIZE 216
-
-nscolor map[COLOR_CUBE_SIZE] = {
-NS_RGB(0x0,  0x0,  0x0 ),
-NS_RGB(0x0,  0x0,  0x33),
-NS_RGB(0x0,  0x0,  0x66),
-NS_RGB(0x0,  0x0,  0x99),
-NS_RGB(0x0,  0x0,  0xcc),
-NS_RGB(0x0,  0x0,  0xff),
-NS_RGB(0x0,  0x33, 0x0 ),
-NS_RGB(0x0,  0x33, 0x33),
-NS_RGB(0x0,  0x33, 0x66),
-NS_RGB(0x0,  0x33, 0x99),
-NS_RGB(0x0,  0x33, 0xcc),
-NS_RGB(0x0,  0x33, 0xff),
-NS_RGB(0x0,  0x66, 0x0 ),
-NS_RGB(0x0,  0x66, 0x33),
-NS_RGB(0x0,  0x66, 0x66),
-NS_RGB(0x0,  0x66, 0x99),
-NS_RGB(0x0,  0x66, 0xcc),
-NS_RGB(0x0,  0x66, 0xff),
-NS_RGB(0x0,  0x99, 0x0 ),
-NS_RGB(0x0,  0x99, 0x33),
-NS_RGB(0x0,  0x99, 0x66),
-NS_RGB(0x0,  0x99, 0x99),
-NS_RGB(0x0,  0x99, 0xcc),
-NS_RGB(0x0,  0x99, 0xff),
-NS_RGB(0x0,  0xcc, 0x0 ),
-NS_RGB(0x0,  0xcc, 0x33),
-NS_RGB(0x0,  0xcc, 0x66),
-NS_RGB(0x0,  0xcc, 0x99),
-NS_RGB(0x0,  0xcc, 0xcc),
-NS_RGB(0x0,  0xcc, 0xff),
-NS_RGB(0x0,  0xff, 0x0 ),
-NS_RGB(0x0,  0xff, 0x33),
-NS_RGB(0x0,  0xff, 0x66),
-NS_RGB(0x0,  0xff, 0x99),
-NS_RGB(0x0,  0xff, 0xcc),
-NS_RGB(0x0,  0xff, 0xff),
-NS_RGB(0x33, 0x0,  0x0 ),
-NS_RGB(0x33, 0x0,  0x33),
-NS_RGB(0x33, 0x0,  0x66),
-NS_RGB(0x33, 0x0,  0x99),
-NS_RGB(0x33, 0x0,  0xcc),
-NS_RGB(0x33, 0x0,  0xff),
-NS_RGB(0x33, 0x33, 0x0 ),
-NS_RGB(0x33, 0x33, 0x33),
-NS_RGB(0x33, 0x33, 0x66),
-NS_RGB(0x33, 0x33, 0x99),
-NS_RGB(0x33, 0x33, 0xcc),
-NS_RGB(0x33, 0x33, 0xff),
-NS_RGB(0x33, 0x66, 0x0 ),
-NS_RGB(0x33, 0x66, 0x33),
-NS_RGB(0x33, 0x66, 0x66),
-NS_RGB(0x33, 0x66, 0x99),
-NS_RGB(0x33, 0x66, 0xcc),
-NS_RGB(0x33, 0x66, 0xff),
-NS_RGB(0x33, 0x99, 0x0 ),
-NS_RGB(0x33, 0x99, 0x33),
-NS_RGB(0x33, 0x99, 0x66),
-NS_RGB(0x33, 0x99, 0x99),
-NS_RGB(0x33, 0x99, 0xcc),
-NS_RGB(0x33, 0x99, 0xff),
-NS_RGB(0x33, 0xcc, 0x0 ),
-NS_RGB(0x33, 0xcc, 0x33),
-NS_RGB(0x33, 0xcc, 0x66),
-NS_RGB(0x33, 0xcc, 0x99),
-NS_RGB(0x33, 0xcc, 0xcc),
-NS_RGB(0x33, 0xcc, 0xff),
-NS_RGB(0x33, 0xff, 0x0 ),
-NS_RGB(0x33, 0xff, 0x33),
-NS_RGB(0x33, 0xff, 0x66),
-NS_RGB(0x33, 0xff, 0x99),
-NS_RGB(0x33, 0xff, 0xcc),
-NS_RGB(0x33, 0xff, 0xff),
-NS_RGB(0x66, 0x0,  0x0 ),
-NS_RGB(0x66, 0x0,  0x33),
-NS_RGB(0x66, 0x0,  0x66),
-NS_RGB(0x66, 0x0,  0x99),
-NS_RGB(0x66, 0x0,  0xcc),
-NS_RGB(0x66, 0x0,  0xff),
-NS_RGB(0x66, 0x33, 0x0 ),
-NS_RGB(0x66, 0x33, 0x33),
-NS_RGB(0x66, 0x33, 0x66),
-NS_RGB(0x66, 0x33, 0x99),
-NS_RGB(0x66, 0x33, 0xcc),
-NS_RGB(0x66, 0x33, 0xff),
-NS_RGB(0x66, 0x66, 0x0 ),
-NS_RGB(0x66, 0x66, 0x33),
-NS_RGB(0x66, 0x66, 0x66),
-NS_RGB(0x66, 0x66, 0x99),
-NS_RGB(0x66, 0x66, 0xcc),
-NS_RGB(0x66, 0x66, 0xff),
-NS_RGB(0x66, 0x99, 0x0 ),
-NS_RGB(0x66, 0x99, 0x33),
-NS_RGB(0x66, 0x99, 0x66),
-NS_RGB(0x66, 0x99, 0x99),
-NS_RGB(0x66, 0x99, 0xcc),
-NS_RGB(0x66, 0x99, 0xff),
-NS_RGB(0x66, 0xcc, 0x0 ),
-NS_RGB(0x66, 0xcc, 0x33),
-NS_RGB(0x66, 0xcc, 0x66),
-NS_RGB(0x66, 0xcc, 0x99),
-NS_RGB(0x66, 0xcc, 0xcc),
-NS_RGB(0x66, 0xcc, 0xff),
-NS_RGB(0x66, 0xff, 0x0 ),
-NS_RGB(0x66, 0xff, 0x33),
-NS_RGB(0x66, 0xff, 0x66),
-NS_RGB(0x66, 0xff, 0x99),
-NS_RGB(0x66, 0xff, 0xcc),
-NS_RGB(0x66, 0xff, 0xff),
-NS_RGB(0x99, 0x0,  0x0 ),
-NS_RGB(0x99, 0x0,  0x33),
-NS_RGB(0x99, 0x0,  0x66),
-NS_RGB(0x99, 0x0,  0x99),
-NS_RGB(0x99, 0x0,  0xcc),
-NS_RGB(0x99, 0x0,  0xff),
-NS_RGB(0x99, 0x33, 0x0 ),
-NS_RGB(0x99, 0x33, 0x33),
-NS_RGB(0x99, 0x33, 0x66),
-NS_RGB(0x99, 0x33, 0x99),
-NS_RGB(0x99, 0x33, 0xcc),
-NS_RGB(0x99, 0x33, 0xff),
-NS_RGB(0x99, 0x66, 0x0 ),
-NS_RGB(0x99, 0x66, 0x33),
-NS_RGB(0x99, 0x66, 0x66),
-NS_RGB(0x99, 0x66, 0x99),
-NS_RGB(0x99, 0x66, 0xcc),
-NS_RGB(0x99, 0x66, 0xff),
-NS_RGB(0x99, 0x99, 0x0 ),
-NS_RGB(0x99, 0x99, 0x33),
-NS_RGB(0x99, 0x99, 0x66),
-NS_RGB(0x99, 0x99, 0x99),
-NS_RGB(0x99, 0x99, 0xcc),
-NS_RGB(0x99, 0x99, 0xff),
-NS_RGB(0x99, 0xcc, 0x0 ),
-NS_RGB(0x99, 0xcc, 0x33),
-NS_RGB(0x99, 0xcc, 0x66),
-NS_RGB(0x99, 0xcc, 0x99),
-NS_RGB(0x99, 0xcc, 0xcc),
-NS_RGB(0x99, 0xcc, 0xff),
-NS_RGB(0x99, 0xff, 0x0 ),
-NS_RGB(0x99, 0xff, 0x33),
-NS_RGB(0x99, 0xff, 0x66),
-NS_RGB(0x99, 0xff, 0x99),
-NS_RGB(0x99, 0xff, 0xcc),
-NS_RGB(0x99, 0xff, 0xff),
-NS_RGB(0xcc, 0x0,  0x0 ),
-NS_RGB(0xcc, 0x0,  0x33),
-NS_RGB(0xcc, 0x0,  0x66),
-NS_RGB(0xcc, 0x0,  0x99),
-NS_RGB(0xcc, 0x0,  0xcc),
-NS_RGB(0xcc, 0x0,  0xff),
-NS_RGB(0xcc, 0x33, 0x0 ),
-NS_RGB(0xcc, 0x33, 0x33),
-NS_RGB(0xcc, 0x33, 0x66),
-NS_RGB(0xcc, 0x33, 0x99),
-NS_RGB(0xcc, 0x33, 0xcc),
-NS_RGB(0xcc, 0x33, 0xff),
-NS_RGB(0xcc, 0x66, 0x0 ),
-NS_RGB(0xcc, 0x66, 0x33),
-NS_RGB(0xcc, 0x66, 0x66),
-NS_RGB(0xcc, 0x66, 0x99),
-NS_RGB(0xcc, 0x66, 0xcc),
-NS_RGB(0xcc, 0x66, 0xff),
-NS_RGB(0xcc, 0x99, 0x0 ),
-NS_RGB(0xcc, 0x99, 0x33),
-NS_RGB(0xcc, 0x99, 0x66),
-NS_RGB(0xcc, 0x99, 0x99),
-NS_RGB(0xcc, 0x99, 0xcc),
-NS_RGB(0xcc, 0x99, 0xff),
-NS_RGB(0xcc, 0xcc, 0x0 ),
-NS_RGB(0xcc, 0xcc, 0x33),
-NS_RGB(0xcc, 0xcc, 0x66),
-NS_RGB(0xcc, 0xcc, 0x99),
-NS_RGB(0xcc, 0xcc, 0xcc),
-NS_RGB(0xcc, 0xcc, 0xff),
-NS_RGB(0xcc, 0xff, 0x0 ),
-NS_RGB(0xcc, 0xff, 0x33),
-NS_RGB(0xcc, 0xff, 0x66),
-NS_RGB(0xcc, 0xff, 0x99),
-NS_RGB(0xcc, 0xff, 0xcc),
-NS_RGB(0xcc, 0xff, 0xff),
-NS_RGB(0xff, 0x0,  0x0 ),
-NS_RGB(0xff, 0x0,  0x33),
-NS_RGB(0xff, 0x0,  0x66),
-NS_RGB(0xff, 0x0,  0x99),
-NS_RGB(0xff, 0x0,  0xcc),
-NS_RGB(0xff, 0x0,  0xff),
-NS_RGB(0xff, 0x33, 0x0 ),
-NS_RGB(0xff, 0x33, 0x33),
-NS_RGB(0xff, 0x33, 0x66),
-NS_RGB(0xff, 0x33, 0x99),
-NS_RGB(0xff, 0x33, 0xcc),
-NS_RGB(0xff, 0x33, 0xff),
-NS_RGB(0xff, 0x66, 0x0 ),
-NS_RGB(0xff, 0x66, 0x33),
-NS_RGB(0xff, 0x66, 0x66),
-NS_RGB(0xff, 0x66, 0x99),
-NS_RGB(0xff, 0x66, 0xcc),
-NS_RGB(0xff, 0x66, 0xff),
-NS_RGB(0xff, 0x99, 0x0 ),
-NS_RGB(0xff, 0x99, 0x33),
-NS_RGB(0xff, 0x99, 0x66),
-NS_RGB(0xff, 0x99, 0x99),
-NS_RGB(0xff, 0x99, 0xcc),
-NS_RGB(0xff, 0x99, 0xff),
-NS_RGB(0xff, 0xcc, 0x0 ),
-NS_RGB(0xff, 0xcc, 0x33),
-NS_RGB(0xff, 0xcc, 0x66),
-NS_RGB(0xff, 0xcc, 0x99),
-NS_RGB(0xff, 0xcc, 0xcc),
-NS_RGB(0xff, 0xcc, 0xff),
-NS_RGB(0xff, 0xff, 0x0 ),
-NS_RGB(0xff, 0xff, 0x33),
-NS_RGB(0xff, 0xff, 0x66),
-NS_RGB(0xff, 0xff, 0x99),
-NS_RGB(0xff, 0xff, 0xcc),
-NS_RGB(0xff, 0xff, 0xff)};
 
 NS_IMETHODIMP nsDeviceContextWin::GetPaletteInfo(nsPaletteInfo& aPaletteInfo)
 {
@@ -831,38 +620,14 @@ NS_IMETHODIMP nsDeviceContextWin::GetPaletteInfo(nsPaletteInfo& aPaletteInfo)
   aPaletteInfo.numReserved = mPaletteInfo.numReserved;
 
   if (NULL == mPaletteInfo.palette) {
-    // Create a logical palette
-    BYTE         tmp[sizeof(LOGPALETTE) + ((COLOR_CUBE_SIZE + 20) * sizeof(PALETTEENTRY))];
-    LPLOGPALETTE logPal = (LPLOGPALETTE)tmp;
-
-    logPal->palVersion = 0x300;
-    logPal->palNumEntries = COLOR_CUBE_SIZE + 20;
-  
-    // Initialize it from the default Windows palette
-    HPALETTE  hDefaultPalette = (HPALETTE)::GetStockObject(DEFAULT_PALETTE);
-  
-    // First ten system colors
-    ::GetPaletteEntries(hDefaultPalette, 0, 10, logPal->palPalEntry);
-
-    // Last ten system colors
-    ::GetPaletteEntries(hDefaultPalette, 10, 10, &logPal->palPalEntry[COLOR_CUBE_SIZE + 10]);
-  
-    // Now set the color cube entries.
-    PALETTEENTRY* entry = &logPal->palPalEntry[10];
-    for (PRInt32 i = 0; i < COLOR_CUBE_SIZE; i++) {
-      entry->peRed = NS_GET_R(map[i]);
-      entry->peGreen = NS_GET_G(map[i]);
-      entry->peBlue = NS_GET_B(map[i]);
-      entry->peFlags = 0;
-  
-      entry++;
-    }
-  
-    // Create a GDI palette
-    mPaletteInfo.palette = ::CreatePalette(logPal);
+    HWND    hwnd = (HWND)mWidget;
+    HDC     hdc = ::GetDC(hwnd);
+    mPaletteInfo.palette = ::CreateHalftonePalette(hdc);  
+    ::ReleaseDC(hwnd, hdc);                                                     
   }
 
   aPaletteInfo.palette = mPaletteInfo.palette;
+                                         
   return NS_OK;
 }
 
@@ -960,17 +725,13 @@ NS_IMETHODIMP nsDeviceContextWin :: GetDeviceContextFor(nsIDeviceContextSpec *aD
   devSpecWin->GetDriverName(drivername); // sets pointer do not free
 
   HDC dc = NULL;
-  if (devSpecWin->IsDEVMODEGlobalHandle()) {
-    HGLOBAL hGlobal;
-    devSpecWin->GetGlobalDevMode(hGlobal);
-    LPDEVMODE devmode = (DEVMODE *)::GlobalLock(hGlobal);
-    dc = ::CreateDC(drivername, devicename, NULL, devmode);
-    ::GlobalUnlock(hGlobal);
-  } else {
-    LPDEVMODE devmode;
-    devSpecWin->GetDevMode(devmode);
+  LPDEVMODE devmode;
+  devSpecWin->GetDevMode(devmode);
+  NS_ASSERTION(devmode, "DevMode can't be NULL here");
+  if (devmode) {
     dc = ::CreateDC(drivername, devicename, NULL, devmode);
   }
+  devSpecWin->UnlockDevMode();
 
   return devConWin->Init(dc, this); // take ownership of the DC
 }
@@ -1000,7 +761,7 @@ static void DisplayLastError()
 #endif
 
 
-NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle)
+NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle, PRUnichar* aPrintToFileName, PRInt32 aStartPage, PRInt32 aEndPage)
 {
   nsresult rv = NS_ERROR_GFX_PRINTER_STARTDOC;
 
@@ -1016,26 +777,14 @@ NS_IMETHODIMP nsDeviceContextWin :: BeginDocument(PRUnichar * aTitle)
     char *title = GetACPString(titleStr);
 
     char* docName = nsnull;
-    nsCOMPtr<nsIPrintOptions> printService = do_GetService(kPrintOptionsCID, &rv);
-    if (printService) {
-      PRBool printToFile = PR_FALSE;
-      printService->GetPrintToFile(&printToFile);
-      if (printToFile) {
-        PRUnichar* uStr;
-        printService->GetToFileName(&uStr);
-        if (uStr != nsnull) {
-          nsAutoString str(uStr);
-          if (str.Length() > 0) {
-            docName = ToNewCString(str);
-          }
-          nsMemory::Free(uStr);
-        }
-      }
+    nsAutoString str(aPrintToFileName);
+    if (str.Length() > 0) {
+      docName = ToNewCString(str);
     }
     docinfo.cbSize = sizeof(docinfo);
     docinfo.lpszDocName = title != nsnull?title:"Mozilla Document";
 
-#ifdef DEBUG_rodsX
+#ifdef DEBUG_rods
     docinfo.lpszOutput = "p.ps";
 #else
     docinfo.lpszOutput = docName;
@@ -1073,6 +822,21 @@ NS_IMETHODIMP nsDeviceContextWin :: EndDocument(void)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsDeviceContextWin :: AbortDocument(void)
+{
+  if (NULL != mDC)
+  {
+    if (::AbortDoc(mDC) > 0) {
+      return NS_OK;
+    } else {
+      DISPLAY_LAST_ERROR
+      return NS_ERROR_ABORT;
+    }
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsDeviceContextWin :: BeginPage(void)
 {
   if (NULL != mDC)
