@@ -40,11 +40,10 @@
 #include "nsImageWin.h"
 #include "nsRenderingContextWin.h"
 
-#define MAX_BUFFER_WIDTH        128
-#define MAX_BUFFER_HEIGHT       128
-
 static nsresult BuildDIB(LPBITMAPINFOHEADER  *aBHead,PRInt32 aWidth,PRInt32 aHeight,PRInt32 aDepth,PRInt8  *aNumBitPix);
 
+
+
 static PRBool
 IsWindowsNT()
 {
@@ -230,50 +229,6 @@ nsresult nsImageWin :: Init(PRInt32 aWidth, PRInt32 aHeight, PRInt32 aDepth,nsMa
 void 
 nsImageWin :: ImageUpdated(nsIDeviceContext *aContext, PRUint8 aFlags, nsRect *aUpdateRect)
 {
-  // XXX Any gamma correction should be done in the image library, and not
-  // here...
-#if 0
-  if (aFlags & nsImageUpdateFlags_kColorMapChanged){
-    PRUint8 *gamma = aContext->GetGammaTable();
-
-    if (mColorMap->NumColors > 0){
-      PRUint8* cpointer = mColorTable;
-
-      for(PRInt32 i = 0; i < mColorMap->NumColors; i++){
-        *cpointer++ = gamma[mColorMap->Index[(3 * i) + 2]];
-        *cpointer++ = gamma[mColorMap->Index[(3 * i) + 1]];
-        *cpointer++ = gamma[mColorMap->Index[(3 * i)]];
-        *cpointer++ = 0;
-      }
-    }
-  }else if ((aFlags & nsImageUpdateFlags_kBitsChanged) &&(nsnull != aUpdateRect)){
-    if (0 == mNumPaletteColors){
-      PRInt32 x, y, span = CalcBytesSpan(mBHead->biWidth), idx;
-      PRUint8 *pixels = mImageBits + 
-        (mBHead->biHeight - aUpdateRect->y - aUpdateRect->height) * span + 
-        aUpdateRect->x * 3;
-      PRUint8 *gamma;
-      float    gammaValue;
-      aContext->GetGammaTable(gamma);
-      aContext->GetGamma(gammaValue);
-
-      // Gamma correct the image
-      if (1.0 != gammaValue){
-        for (y = 0; y < aUpdateRect->height; y++){
-          for (x = 0, idx = 0; x < aUpdateRect->width; x++){
-            pixels[idx] = gamma[pixels[idx]];
-            idx++;
-            pixels[idx] = gamma[pixels[idx]];
-            idx++;
-            pixels[idx] = gamma[pixels[idx]];
-            idx++;
-          }
-          pixels += span;
-        }
-      }
-    }
-  }
-#endif
 }
 
 //------------------------------------------------------------
@@ -424,6 +379,12 @@ nsImageWin :: CreateDDB(nsDrawingSurface aSurface)
   ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
 
   if (TheHDC != NULL){
+    // Temporary fix for bug 135226 until we do better decode-time
+    // dithering and paletized storage of images. Bail on the 
+    // optimization to DDB if we're on a paletted device.
+    int rasterCaps = ::GetDeviceCaps(TheHDC, RASTERCAPS);
+    if (RC_PALETTE == (rasterCaps & RC_PALETTE)) return;
+    
     if (mSizeImage > 0){
        if (mAlphaDepth == 8) {
          CreateImageWithAlphaBits(TheHDC);
@@ -497,9 +458,8 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
   }
 
   if (nsnull != TheHDC){
+    PRBool  didComposite = PR_FALSE;
     if (!IsOptimized() || nsnull==mHBitmap){
-      PRBool  didComposite = PR_FALSE;
-
       rop = SRCCOPY;
 
       if (nsnull != mAlphaBits){
@@ -548,11 +508,30 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
         if (nsnull != mAlphaBits){
           if (1==mAlphaDepth){
             MONOBITMAPINFO  bmi(mAlphaWidth, mAlphaHeight);
-
-            ::StretchDIBits(TheHDC, aDX, aDY, aDWidth, aDHeight,aSX, srcy, aSWidth, aSHeight, mAlphaBits,
-                                          (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCAND);
-             rop = SRCPAINT;
-             //rop = SRCCOPY;
+            if (canRaster == DT_RASPRINTER) {
+              MONOBITMAPINFO  bmi(mAlphaWidth, mAlphaHeight);
+              if (mImageBits != nsnull) {
+                CompositeBitsInMemory(TheHDC,aDX,aDY,aDWidth,aDHeight,aSX,aSY,aSWidth,aSHeight,srcy,
+                                      mAlphaBits,&bmi,mImageBits,mBHead,mNumPaletteColors);
+                didComposite = PR_TRUE;  
+              } else {
+                ConvertDDBtoDIB(); // Create mImageBits
+                if (mImageBits != nsnull) {  
+                  CompositeBitsInMemory(TheHDC,aDX,aDY,aDWidth,aDHeight,aSX,aSY,aSWidth,aSHeight,srcy,
+                                        mAlphaBits,&bmi,mImageBits,mBHead,mNumPaletteColors);
+                  // Clean up the image bits    
+                  delete [] mImageBits;
+                  mImageBits = nsnull;
+                  didComposite = PR_TRUE;         
+                } else {
+                  NS_WARNING("Could not composite bits in memory because conversion to DIB failed\n");
+                }
+              }
+            } else {
+              ::StretchDIBits(TheHDC, aDX, aDY, aDWidth, aDHeight,aSX, srcy, aSWidth, aSHeight, mAlphaBits,
+                             (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCAND);
+              rop = SRCPAINT;
+            }
           }
         }
         // if this is for a printer.. we have to convert it back to a DIB
@@ -569,12 +548,13 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
                 blendFunction.BlendFlags = 0;
                 blendFunction.SourceConstantAlpha = 255;
                 blendFunction.AlphaFormat = 1 /*AC_SRC_ALPHA*/;
-                gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, srcy, aSWidth, aSHeight, blendFunction);
-              } else {
+                gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY, aSWidth, aSHeight, blendFunction);
+              } else { 
                 ::StretchBlt(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY,aSWidth, aSHeight, rop);
               }
             }else{
-              PrintDDB(aSurface,aDX,aDY,aDWidth,aDHeight,rop);
+              if (! didComposite) 
+                PrintDDB(aSurface,aDX,aDY,aDWidth,aDHeight,rop);
             }
           }
         } else {
@@ -586,8 +566,8 @@ nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
             blendFunction.BlendFlags = 0;
             blendFunction.SourceConstantAlpha = 255;
             blendFunction.AlphaFormat = 1 /*AC_SRC_ALPHA*/;
-            gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, srcy, aSWidth, aSHeight, blendFunction);
-          } else {
+            gAlphaBlend(TheHDC, aDX, aDY, aDWidth, aDHeight, srcDC, aSX, aSY, aSWidth, aSHeight, blendFunction);
+          } else { 
             ::StretchBlt(TheHDC,aDX,aDY,aDWidth,aDHeight,srcDC,aSX,aSY,aSWidth,aSHeight,rop);
           }
         }
@@ -632,7 +612,7 @@ void nsImageWin::DrawComposited24(unsigned char *aBits, int aX, int aY, int aWid
 void nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY, int aDWidth, int aDHeight,
                     int aSX, int aSY, int aSWidth, int aSHeight)
 {
-  HDC memDC = CreateCompatibleDC(TheHDC);
+  HDC memDC = ::CreateCompatibleDC(TheHDC);
   unsigned char *screenBits;
   ALPHA24BITMAPINFO bmi(aSWidth, aSHeight);
   HBITMAP tmpBitmap = ::CreateDIBSection(memDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS,
@@ -652,7 +632,7 @@ void nsImageWin::DrawComposited(HDC TheHDC, int aDX, int aDY, int aDWidth, int a
 
   ::SelectObject(memDC, oldBitmap);
   ::DeleteObject(tmpBitmap);
-  ::DeleteObject(memDC);
+  ::DeleteDC(memDC);
 }
 
 /** ---------------------------------------------------
@@ -672,343 +652,300 @@ NS_IMETHODIMP nsImageWin :: Draw(nsIRenderingContext &aContext, nsDrawingSurface
 NS_IMETHODIMP nsImageWin::DrawTile(nsIRenderingContext &aContext,
                                    nsDrawingSurface aSurface,
                                    PRInt32 aSXOffset, PRInt32 aSYOffset,
-                                   const nsRect &aTileRect)
+                                   const nsRect &aDestRect)
 {
-  float            scale;
+  float           scale;
+  unsigned char   *targetRow,*imageRow,*alphaRow;
+  PRBool          result;
+  PRInt32         numTiles,x0,y0,x1,y1,destScaledWidth,destScaledHeight;
+  PRInt32         validWidth,validHeight,validX,validY,targetRowBytes;
+  PRInt32         temp1,temp2,ScaledTileHeight_Offset,ScaledTileWidth_Offset;
+  PRInt32         x,y,width,height,canRaster;
+  nsCOMPtr<nsIDeviceContext> theDeviceContext;
+  HDC             theHDC;
+  nsRect          destRect,srcRect;
+  nscoord         ScaledTileWidth,ScaledTileHeight;
 
-  {
-    nsCOMPtr<nsIDeviceContext> theDeviceContext;
-    aContext.GetDeviceContext(*getter_AddRefs(theDeviceContext));
-    theDeviceContext->GetCanonicalPixelScale(scale);
-  }
+  aContext.GetDeviceContext(*getter_AddRefs(theDeviceContext));
+  theDeviceContext->GetCanonicalPixelScale(scale);
+
+  destScaledWidth  = PR_MAX(PRInt32(mBHead->biWidth*scale), 1);
+  destScaledHeight = PR_MAX(PRInt32(mBHead->biHeight*scale), 1);
   
-
-  PRInt32
-    destScaledWidth  = PR_MAX(int(mBHead->biWidth*scale), 1),
-    destScaledHeight = PR_MAX(int(mBHead->biHeight*scale), 1);
-
-  PRInt32
-    validX = 0,
-    validY = 0,
-    validWidth  = mBHead->biWidth,
-    validHeight = mBHead->biHeight;
+  validX = 0;
+  validY = 0;
+  validWidth  = mBHead->biWidth;
+  validHeight = mBHead->biHeight;
 
   
   // limit the image rectangle to the size of the image data which
   // has been validated.
   if (mDecodedY2 < mBHead->biHeight) {
     validHeight = mDecodedY2 - mDecodedY1;
-    destScaledHeight = PR_MAX(int(validHeight*scale), 1);
+    destScaledHeight = PR_MAX(PRInt32(validHeight*scale), 1);
   }
   if (mDecodedX2 < mBHead->biWidth) {
     validWidth = mDecodedX2 - mDecodedX1;
-    destScaledWidth = PR_MAX(int(validWidth*scale), 1);
+    destScaledWidth = PR_MAX(PRInt32(validWidth*scale), 1);
   }
   if (mDecodedY1 > 0) {   
     validHeight -= mDecodedY1;
-    destScaledHeight = PR_MAX(int(validHeight*scale), 1);
+    destScaledHeight = PR_MAX(PRInt32(validHeight*scale), 1);
     validY = mDecodedY1;
   }
   if (mDecodedX1 > 0) {
     validWidth -= mDecodedX1;
-    destScaledWidth = PR_MAX(int(validWidth*scale), 1);
+    destScaledWidth = PR_MAX(PRInt32(validWidth*scale), 1);
     validX = mDecodedX1; 
   }
 
-  // put the width and hieght into the devices coordinates
 
-  PRInt32 aY0 = aTileRect.y - aSYOffset,
-          aX0 = aTileRect.x - aSXOffset,
-          aY1 = aTileRect.y + aTileRect.height,
-          aX1 = aTileRect.x + aTileRect.width;
+  // put the DestRect into absolute coordintes of the device
+  y0 = aDestRect.y - aSYOffset;
+  x0 = aDestRect.x - aSXOffset;
+  y1 = aDestRect.y + aDestRect.height;
+  x1 = aDestRect.x + aDestRect.width;
 
   // this is the width and height of the image in pixels
   // we need to map this to the pixel height of the device
-  nscoord imageScaledWidth = PR_MAX(int(mBHead->biWidth*scale), 1);
-  nscoord imageScaledHeight = PR_MAX(int(mBHead->biHeight*scale), 1);
+  ScaledTileWidth = PR_MAX(PRInt32(mBHead->biWidth*scale), 1);
+  ScaledTileHeight = PR_MAX(PRInt32(mBHead->biHeight*scale), 1);
 
-  nscoord tileWidth = aTileRect.width;
-  nscoord tileHeight = aTileRect.height;
-
-  PRBool              tryAgain = PR_FALSE;
-  nsRect              destRect,srcRect,tvrect;
-  HDC                 TheHDC,offDC,maskDC;
-  PRInt32             x,y,width,height,canRaster,TileBufferWidth,TileBufferHeight;
-  HBITMAP             maskBits,tileBits,oldBits,oldMaskBits; 
-
-
-  // The slower tiling will need to be used for the following cases:
-  // 1.) Printers 2.) When in 256 color mode 3.) when the tile is larger than the buffer
-  tvrect.SetRect(0,0,aX1-aX0,aY1-aY0);
   ((nsDrawingSurfaceWin *)aSurface)->GetTECHNOLOGY(&canRaster);
 
+  // do alpha depth equal to 8 here.. this needs some special attention
 
-  if( mAlphaDepth == 8){
-    /** 
-     *  do alpha depth equal to 8 here.. this needs some special attention
-     *  draw the alpha and the bitmap to an offscreen buffer.. for the blend.. first 
-     */
-    // get the current HDC to draw to..
-    ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
-    if (NULL == TheHDC){
-      return (PR_FALSE);
-    } 
+  if ( mAlphaDepth == 8) {
+    unsigned char *screenBits=nsnull;
+    HDC           memDC=nsnull;
+    HBITMAP       tmpBitmap=nsnull,oldBitmap;
+    unsigned char alpha;
+    PRInt32       targetBytesPerPixel,imageBytesPerPixel;
+
+    if (!mImageBits) {
+      ConvertDDBtoDIB();
+    }
+
+    // draw the alpha and the bitmap to an offscreen buffer.. for the blend.. first 
+    ((nsDrawingSurfaceWin *)aSurface)->GetDC(&theHDC);
+    if (theHDC) {
     // create a buffer for the blend            
-    HDC memDC = CreateCompatibleDC(TheHDC);
-    unsigned char *screenBits;
-    width = aX1-aX0;
-    height = aY1-aY0;
-    ALPHA24BITMAPINFO bmi(width, height);
-    HBITMAP tmpBitmap = ::CreateDIBSection(memDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, (LPVOID *)&screenBits, NULL, 0);
-    HBITMAP oldBitmap = (HBITMAP)::SelectObject(memDC, tmpBitmap);
-    if(0 == tmpBitmap){
-      ::DeleteObject(memDC);
-      tryAgain = PR_TRUE;
+      memDC = CreateCompatibleDC(theHDC);
+      width = x1-x0;
+      height = y1-y0;
+
+      ALPHA24BITMAPINFO bmi(width, height);
+      tmpBitmap = ::CreateDIBSection(memDC, (LPBITMAPINFO)&bmi, DIB_RGB_COLORS, (LPVOID *)&screenBits, NULL, 0);
+      oldBitmap = (HBITMAP)::SelectObject(memDC, tmpBitmap);
+
+      // number of bytes in a row on a 32 bit boundary
+      targetRowBytes = (bmi.bmiHeader.biWidth * bmi.bmiHeader.biBitCount) >> 5;  // number of 32 bit longs
+      if (((PRUint32)bmi.bmiHeader.biWidth * bmi.bmiHeader.biBitCount) & 0x1F) { // make sure its a multiple of 32
+        targetRowBytes++;     // or else there will not be enough bytes per line
+      }
+      targetRowBytes <<= 2;   // divide by 4 to get the number of bytes from the number of 32 bit longs
+
+      targetBytesPerPixel = bmi.bmiHeader.biBitCount/8;
+    }
+
+    if (!tmpBitmap) {
+      if (memDC) {
+        ::DeleteDC(memDC);
+      }
+      // this failed..and will fall into the slow blitting code
+      NS_WARNING("The Creation of the tmpBitmap failed \n");
     } else {
       // Copy from the HDC to the memory DC
-      ::StretchBlt(memDC, 0, 0, width, height,TheHDC, 0, 0, width, height, SRCCOPY);
+      // this will be the image on the screen into a buffer for the blend.
+      ::StretchBlt(memDC, 0, 0, width, height,theHDC, aDestRect.x, aDestRect.y, width, height, SRCCOPY);
   
-      PRInt32 targetRowBytes = ((width * 3) + 3) & ~3;
-      unsigned char *targetRow,*imageRow,*alphaRow;
-      PRInt32 temp1,temp2,imageScaledHeightPlus,imageScaledWidthPlus;
+      imageBytesPerPixel = mBHead->biBitCount/8;
+      ScaledTileHeight_Offset = ScaledTileHeight + aSYOffset;
+      ScaledTileWidth_Offset = ScaledTileWidth + aSXOffset;
 
-
-      imageScaledHeightPlus = imageScaledHeight + aSYOffset;
-      imageScaledWidthPlus = imageScaledWidth + aSXOffset;
-      temp1 = mRowBytes + 3 * aSXOffset;
+      // this is the adjusted width of these imagebuffers
+      temp1 = mRowBytes + imageBytesPerPixel * aSXOffset;
       temp2 = mARowBytes + aSXOffset;
 
       for (int y = 0,byw=aSYOffset; y < height; y++,byw++) {
-        if(byw >= imageScaledHeightPlus){
+        if (byw >= ScaledTileHeight_Offset) {
           byw = aSYOffset;
         }
         targetRow = screenBits + y * targetRowBytes;
-        imageRow = mImageBits + byw * temp1;
-        alphaRow = mAlphaBits + byw * temp2;
+        imageRow = mImageBits + (byw * temp1);
+        alphaRow = mAlphaBits + (byw * temp2);
 
-        for (int x=0,bxw=aSXOffset;x<width;x++,targetRow+=3,imageRow+=3,bxw++, alphaRow++) {
-          if(bxw>=imageScaledWidthPlus){
+        for (int x=0,bxw=aSXOffset;x<width;x++,targetRow+=targetBytesPerPixel,imageRow+=imageBytesPerPixel,bxw++, alphaRow++) {
+          // if we went past the row width of our buffer.. go back and start again
+          if (bxw>=ScaledTileWidth_Offset) {
             bxw = aSXOffset;
-            imageRow = mImageBits + byw * mRowBytes + ((3*bxw)%mRowBytes);
-            alphaRow = mAlphaBits + byw * mARowBytes + (bxw%mRowBytes);
+            imageRow = mImageBits + (byw * temp1);
+            alphaRow = mAlphaBits + (byw * temp2);
           }
-          unsigned alpha = *alphaRow;
 
+          alpha = *alphaRow;
           MOZ_BLEND(targetRow[0], targetRow[0], imageRow[0], alpha);
           MOZ_BLEND(targetRow[1], targetRow[1], imageRow[1], alpha);
           MOZ_BLEND(targetRow[2], targetRow[2], imageRow[2], alpha);
         }
       }
 
-      // Copy back to the HDC 
-      ::StretchBlt(TheHDC, 0, 0, width, height,memDC, 0, 0, width, height, SRCCOPY);
+      // copy the blended image back to the screen
+      ::StretchBlt(theHDC, aDestRect.x, aDestRect.y, width, height,memDC, 0, 0, width, height, SRCCOPY);
 
       ::SelectObject(memDC, oldBitmap);
       ::DeleteObject(tmpBitmap);
-      ::DeleteObject(memDC);
+      ::DeleteDC(memDC);
   
-    return(PR_TRUE);
+    return(NS_OK);
+    } 
+  }
+
+  numTiles = (aDestRect.width*aDestRect.height)/(ScaledTileWidth*ScaledTileHeight);
+
+  // if alpha is less than 8,not printing, and not 8 bit palette image then we can do
+  // a progressive tile and the tile is at least 8 times smaller than the area to update
+  // if the number of tiles are less than 32 the progressive doubling can slow down
+  // because of the creation of the offscreens, DC's etc.
+  if ( (mAlphaDepth < 8) && (canRaster!=DT_RASPRINTER) && (256!=mNumPaletteColors) && 
+          (numTiles > 32) ) {
+    result = ProgressiveDoubleBlit(aSurface,aDestRect.width, aDestRect.height,
+                             ScaledTileWidth,ScaledTileHeight, x0,y0,x1,y1);  
+    if (result ) {
+      return(NS_OK);
     }
   }
 
-
-  // we have to use the old way.. for 256 color mode and printing.. slow, but will always work.
-  if ((mAlphaDepth>8) || ((mAlphaDepth==8)&&tryAgain) || (canRaster==DT_RASPRINTER) || (256==mNumPaletteColors)
-      || (imageScaledWidth>MAX_BUFFER_WIDTH) || (imageScaledHeight>MAX_BUFFER_HEIGHT)){
-    for(y=aY0;y<aY1;y+=imageScaledHeight){
-      for(x=aX0;x<aX1;x+=imageScaledWidth){
-      Draw(aContext, aSurface,
-           0, 0, PR_MIN(validWidth, aX1-x), PR_MIN(validHeight, aY1-y),
-           x, y, PR_MIN(destScaledWidth, aX1-x), PR_MIN(destScaledHeight, aY1-y));
-      }
-    } 
-    return(PR_TRUE);
-  } else if(mAlphaDepth == 8 ){
-  }
-  
-  // create a larger tile from the smaller one
-  ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
-  if (NULL == TheHDC){
-    return (PR_FALSE);
-  }
-
-  // can not create a compatible bitmap with an offscreen DC (it will be black and white)
-  // so we will create a screen compatible bitmap and then install this into the offscreen DC
-  tvrect.SetRect(0,0,aX1-aX0,aY1-aY0);
-  offDC = ::CreateCompatibleDC(TheHDC);
-  if (NULL ==offDC){
-    return (PR_FALSE);
-  }
-
-  if (tileWidth < tvrect.width){
-    TileBufferWidth = MAX_BUFFER_WIDTH;
-  } else {
-    TileBufferWidth = tileWidth;
-  }
-
-  if (tileHeight < tvrect.height){
-    TileBufferHeight = MAX_BUFFER_HEIGHT;
-  } else {
-    TileBufferHeight = tileHeight;
-  }
-
-  tileBits = ::CreateCompatibleBitmap(TheHDC, TileBufferWidth,TileBufferHeight);
-
-  if (NULL == tileBits){
-    ::DeleteObject(offDC);
-    return (PR_FALSE);
-  }
-  oldBits =(HBITMAP) ::SelectObject(offDC,tileBits);
-
-
-  if (1==mAlphaDepth) {
-    // larger tile for mask
-    maskDC = ::CreateCompatibleDC(TheHDC);
-    if (NULL ==maskDC){
-      ::SelectObject(offDC,oldBits);
-      ::DeleteObject(tileBits);
-      ::DeleteObject(offDC);
-      return (PR_FALSE);
+  // if we got to this point.. everything else failed.. and the slow blit backstop
+  // will finish this tiling
+  for (y=y0;y<y1;y+=ScaledTileHeight) {
+    for (x=x0;x<x1;x+=ScaledTileWidth) {
+    Draw(aContext, aSurface,
+         0, 0, PR_MIN(validWidth, x1-x), PR_MIN(validHeight, y1-y),
+         x, y, PR_MIN(destScaledWidth, x1-x), PR_MIN(destScaledHeight, y1-y));
     }
-    maskBits = ::CreateCompatibleBitmap(TheHDC, TileBufferWidth, TileBufferHeight);
-    if (NULL ==maskBits){
-      ::SelectObject(offDC,oldBits);
-      ::DeleteObject(tileBits);
-      ::DeleteObject(offDC);
-      ::DeleteObject(maskDC);
-      return (PR_FALSE);
-    }
-
-    oldMaskBits = (HBITMAP)::SelectObject(maskDC,maskBits);
-
-    // get the mask into our new tiled mask
-    MONOBITMAPINFO  bmi(mAlphaWidth,mAlphaHeight);
-    ::StretchDIBits(maskDC, 0, 0, imageScaledWidth, imageScaledHeight,0, 0, 
-                    mAlphaWidth, mAlphaHeight, mAlphaBits,(LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCCOPY);
-
-    ::BitBlt(maskDC,0,0,imageScaledWidth,imageScaledHeight,maskDC,0,0,SRCCOPY);
-    srcRect.SetRect(0,0,imageScaledWidth,imageScaledHeight);
-    BuildTile(maskDC,srcRect,TileBufferWidth/2,TileBufferHeight/2,SRCCOPY);
-  }
-
-  // put the initial tile of background image into the offscreen
-  if (!IsOptimized() || nsnull==mHBitmap){
-    ::StretchDIBits(offDC, 0, 0, imageScaledWidth, imageScaledHeight,0, 0, imageScaledWidth, imageScaledHeight, mImageBits,
-                    (LPBITMAPINFO)mBHead, 256 == mNumPaletteColors ? DIB_PAL_COLORS:DIB_RGB_COLORS,
-                    SRCCOPY);
-  }else{
-    // need to select install this bitmap into this DC first
-    HBITMAP oldBits;
-
-    oldBits = (HBITMAP)::SelectObject(TheHDC, mHBitmap);
-    ::BitBlt(offDC,0,0,imageScaledWidth,imageScaledHeight,TheHDC,0,0,SRCCOPY);
-    ::SelectObject(TheHDC, oldBits);
-  }
-
-  srcRect.SetRect(0,0,imageScaledWidth,imageScaledHeight);
-  BuildTile(offDC,srcRect,TileBufferWidth/2,TileBufferHeight/2,SRCCOPY);
-
-  // now duplicate our tile into the background
-  destRect = srcRect;
-  width = destRect.width;
-  height = destRect.height;
-
-  if (1!=mAlphaDepth) {
-    for(y=aY0;y<aY1;y+=srcRect.height){
-      for(x=aX0;x<aX1;x+=srcRect.width){
-        destRect.x = x;
-        destRect.y = y;
-        ::BitBlt(TheHDC,x,y,width,height,offDC,0,0,SRCCOPY);
-      }
-    } 
-  } else {
-    for(y=aY0;y<aY1;y+=srcRect.height){
-      for(x=aX0;x<aX1;x+=srcRect.width){
-        destRect.x = x;
-        destRect.y = y;
-        ::BitBlt(TheHDC,x,y,width,height,maskDC,0,0,SRCAND);
-        ::BitBlt(TheHDC,x,y,width,height,offDC,0,0,SRCPAINT);
-      }
-    } 
-    ::SelectObject(maskDC,oldMaskBits);
-    ::DeleteObject(maskBits);
-    ::DeleteObject(maskDC);
-  }
-
-  ::SelectObject(offDC,oldBits);
-  ::DeleteObject(tileBits);
-  ::DeleteObject(offDC);
-
-  return NS_OK;
+  } 
+  return(NS_OK);
 }
 
 /** ---------------------------------------------------
- *  See documentation in nsIRenderingContext.h
- *  @update 3/16/00 dwc
+ *  See documentation in nsImageWin.h
+ *  @update 4/16/02 dwc
  */
-PRBool 
-nsImageWin :: PatBltTile(nsIRenderingContext &aContext, nsDrawingSurface aSurface,
-                                nscoord aX0,nscoord aY0,nscoord aX1,nscoord aY1,
-                                nscoord aWidth, nscoord aHeight)
+PRBool
+nsImageWin::ProgressiveDoubleBlit(nsDrawingSurface aSurface,
+                              PRInt32 aDestBufferWidth, PRInt32 aDestBufferHeight,
+                              PRInt32 aScaledTileWidth,PRInt32 aScaledTileHeight,
+                              PRInt32 aX0,PRInt32 aY0,
+                              PRInt32 aX1,PRInt32 aY1)
 {
-  HDC     TheHDC;
-  HBRUSH  hBrush,oldBrush;
-  BOOL    success,problem=FALSE;
-  DWORD   rop;
-  HBITMAP theBits;
-  POINT   originalPoint;
+  PRInt32 x,y,width,height;
+  nsRect  destRect,srcRect;
+  HDC     theHDC,offDC,maskDC;
+  HBITMAP maskBits,tileBits,oldBits,oldMaskBits; 
 
-  if (PR_FALSE==mCanOptimize) {
-    return (PR_FALSE);
-  }
-  
-  if (nsnull==mHBitmap) {
-    CreateDDB(aSurface);
-  }
+    // create a larger tile from the smaller one
+    ((nsDrawingSurfaceWin *)aSurface)->GetDC(&theHDC);
+    if (NULL == theHDC) {
+      return (PR_FALSE);
+    }
 
-  ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
-  if (NULL == TheHDC){
-    return (PR_FALSE);
-  }
+    // so we will create a screen compatible bitmap and then install this into the offscreen DC
+    offDC = ::CreateCompatibleDC(theHDC);
 
-  // default copy mode
-  rop = PATCOPY;
+    if (NULL ==offDC) {
+        return (PR_FALSE);
+    }
 
+    tileBits = ::CreateCompatibleBitmap(theHDC, aDestBufferWidth,aDestBufferHeight);
 
-  // we have to reset the origin here..
-  ::SetBrushOrgEx(TheHDC,aX0,aY0,&originalPoint);
+    if (NULL == tileBits) {
+      ::DeleteDC(offDC);
+      return (PR_FALSE);
+    }
+    oldBits =(HBITMAP) ::SelectObject(offDC,tileBits);
 
-  // if there is an alpha layer, lay down the mask first
-  if (1==mAlphaDepth){
-    ((nsDrawingSurfaceWin *)aSurface)->GetDC(&TheHDC);
-    theBits = ::CreateBitmap(mAlphaWidth,mAlphaHeight,1,1,NULL);
-    MONOBITMAPINFO  bmi(mAlphaWidth, mAlphaHeight);
-    SetDIBits(TheHDC,theBits,0,mAlphaHeight,mAlphaBits,(LPBITMAPINFO)&bmi,DIB_RGB_COLORS);
-    hBrush = CreatePatternBrush(theBits);
-    oldBrush = (HBRUSH)SelectObject(TheHDC,hBrush);
-    success = PatBlt( TheHDC, aX0,aY0,aX1-aX0,aY1-aY0,0xA000C9);
-    SelectObject(TheHDC,oldBrush);
-    DeleteObject(hBrush);
-    DeleteObject(theBits);
-    rop = 0xFA0089;
-  }
+    if (1==mAlphaDepth) {
+      // larger tile for mask
+      maskDC = ::CreateCompatibleDC(theHDC);
+      if (NULL ==maskDC){
+        ::SelectObject(offDC,oldBits);
+        ::DeleteObject(tileBits);
+        ::DeleteDC(offDC);
+        return (PR_FALSE);
+      }
+      maskBits = ::CreateCompatibleBitmap(theHDC, aDestBufferWidth, aDestBufferHeight);
+      if (NULL ==maskBits) {
+        ::SelectObject(offDC,oldBits);
+        ::DeleteObject(tileBits);
+        ::DeleteDC(offDC);
+        ::DeleteDC(maskDC);
+        return (PR_FALSE);
+      }
 
-  // do a pattern blit
-  if (mHBitmap == NULL) {
-    problem = TRUE;
-  }
+      oldMaskBits = (HBITMAP)::SelectObject(maskDC,maskBits);
 
-  hBrush = CreatePatternBrush(mHBitmap);
-  oldBrush = (HBRUSH)SelectObject(TheHDC,hBrush);
-  success = PatBlt( TheHDC, aX0,aY0,aX1-aX0,aY1-aY0,rop);
-  SelectObject(TheHDC,oldBrush);
-  DeleteObject(hBrush);
+      // get the mask into our new tiled mask
+      MONOBITMAPINFO  bmi(mAlphaWidth,mAlphaHeight);
+      ::StretchDIBits(maskDC, 0, 0, aScaledTileWidth, aScaledTileHeight,0, 0, 
+                      mAlphaWidth, mAlphaHeight, mAlphaBits,(LPBITMAPINFO)&bmi, DIB_RGB_COLORS, SRCCOPY);
 
-  ::SetBrushOrgEx(TheHDC,originalPoint.x,originalPoint.y,NULL);
+      ::BitBlt(maskDC,0,0,aScaledTileWidth,aScaledTileHeight,maskDC,0,0,SRCCOPY);
+      srcRect.SetRect(0,0,aScaledTileWidth,aScaledTileHeight);
+      BuildTile(maskDC,srcRect,aDestBufferWidth/2,aDestBufferHeight/2,SRCCOPY);
+    }
 
-  return (PR_TRUE);
+    // put the initial tile of background image into the offscreen
+    if (!IsOptimized() || nsnull==mHBitmap) {
+      ::StretchDIBits(offDC, 0, 0, aScaledTileWidth, aScaledTileHeight,0, 0, aScaledTileWidth, aScaledTileHeight, mImageBits,
+                      (LPBITMAPINFO)mBHead, 256 == mNumPaletteColors ? DIB_PAL_COLORS:DIB_RGB_COLORS,
+                      SRCCOPY);
+    } else {
+      // need to select install this bitmap into this DC first
+      HBITMAP oldBits;
+      oldBits = (HBITMAP)::SelectObject(theHDC, mHBitmap);
+      ::BitBlt(offDC,0,0,aScaledTileWidth,aScaledTileHeight,theHDC,0,0,SRCCOPY);
+      ::SelectObject(theHDC, oldBits);
+    }
+
+    srcRect.SetRect(0,0,aScaledTileWidth,aScaledTileHeight);
+    BuildTile(offDC,srcRect,aDestBufferWidth/2,aDestBufferHeight/2,SRCCOPY);
+
+    // now duplicate our tile into the background
+    destRect = srcRect;
+    width = destRect.width;
+    height = destRect.height;
+
+    if (1!=mAlphaDepth) {
+      for (y=aY0;y<aY1;y+=srcRect.height) {
+        for (x=aX0;x<aX1;x+=srcRect.width) {
+          destRect.x = x;
+          destRect.y = y;
+          ::BitBlt(theHDC,x,y,width,height,offDC,0,0,SRCCOPY);
+        }
+      } 
+    } else {
+      for (y=aY0;y<aY1;y+=srcRect.height) {
+        for (x=aX0;x<aX1;x+=srcRect.width) {
+          destRect.x = x;
+         destRect.y = y;
+          ::BitBlt(theHDC,x,y,width,height,maskDC,0,0,SRCAND);
+          ::BitBlt(theHDC,x,y,width,height,offDC,0,0,SRCPAINT);
+        }
+      } 
+      ::SelectObject(maskDC,oldMaskBits);
+      ::DeleteObject(maskBits);
+      ::DeleteDC(maskDC);
+    }
+
+  ::SelectObject(offDC,oldBits);
+  ::DeleteObject(tileBits);
+  ::DeleteDC(offDC);
+
+  return(PR_TRUE);
 }
 
+
 ALPHABLENDPROC nsImageWin::gAlphaBlend = NULL;
 
 PRBool nsImageWin::CanAlphaBlend(void)
@@ -1142,7 +1079,7 @@ nsImageWin::PrintDDB(nsDrawingSurface aSurface,PRInt32 aX, PRInt32 aY, PRInt32 a
 
   if (mIsOptimized == PR_TRUE){
     if (mHBitmap != nsnull){
-      ConvertDDBtoDIB(aWidth,aHeight);
+      ConvertDDBtoDIB();
       ((nsDrawingSurfaceWin *)aSurface)->GetDC(&theHDC);
 
       if (mBHead->biBitCount == 8) {
@@ -1171,7 +1108,7 @@ nsImageWin::PrintDDB(nsDrawingSurface aSurface,PRInt32 aX, PRInt32 aY, PRInt32 a
  * @return the result of the operation, if NS_OK, then the pixelmap is unoptimized
  */
 nsresult 
-nsImageWin::ConvertDDBtoDIB(PRInt32 aWidth, PRInt32 aHeight)
+nsImageWin::ConvertDDBtoDIB()
 {
   PRInt32             numbytes,tWidth,tHeight;
   BITMAP              srcinfo;
@@ -1366,7 +1303,7 @@ nsImageWin::GetBits()
 {
   // if mImageBits did not exist.. then
   if (!mImageBits) {
-    ConvertDDBtoDIB(GetWidth(), GetHeight());
+    ConvertDDBtoDIB();
     mDIBTemp = PR_TRUE;   // only set to true if the DIB is being created here as temporary
   }
 
@@ -1526,6 +1463,6 @@ CompositeBitsInMemory(HDC aTheHDC, int aDX, int aDY, int aDWidth, int aDHeight,
       }
       ::DeleteObject(tmpBitmap);
     }
-    ::DeleteObject(memDC);
+    ::DeleteDC(memDC);
   }
 }
