@@ -46,6 +46,7 @@
 #include "prprf.h"
 #include "nsDrawingSurfaceWin.h"
 #include "nsGfxCIID.h"
+#include "nsUnicharUtils.h"
 
 //#define GFX_DEBUG
 
@@ -444,7 +445,15 @@ nsresult nsRenderingContextWin :: SetupDC(HDC aOldDC, HDC aNewDC)
   mCurrTextColor = mCurrentColor;
   ::SetBkMode(aNewDC, TRANSPARENT);
   ::SetPolyFillMode(aNewDC, WINDING);
-  ::SetStretchBltMode(aNewDC, COLORONCOLOR);
+  // Temporary fix for bug 135226 until we do better decode-time
+  // dithering and paletized storage of images.
+  nsPaletteInfo palInfo;
+  mContext->GetPaletteInfo(palInfo);
+  if (palInfo.isPaletteDevice)
+    ::SetStretchBltMode(aNewDC, HALFTONE);
+  else
+    ::SetStretchBltMode(aNewDC, COLORONCOLOR);                                  
+
   ::SetTextAlign(aNewDC, TA_BASELINE);
 
   if (nsnull != aOldDC)
@@ -481,13 +490,10 @@ nsresult nsRenderingContextWin :: SetupDC(HDC aOldDC, HDC aNewDC)
 #endif
   
   // If this is a palette device, then select and realize the palette
-  nsPaletteInfo palInfo;
-  mContext->GetPaletteInfo(palInfo);
-
   if (palInfo.isPaletteDevice && palInfo.palette)
   {
     // Select the palette in the background
-    ::SelectPalette(aNewDC, (HPALETTE)palInfo.palette, PR_FALSE);
+    ::SelectPalette(aNewDC, (HPALETTE)palInfo.palette, PR_TRUE);
     ::RealizePalette(aNewDC);
   }
 
@@ -506,8 +512,6 @@ nsresult nsRenderingContextWin :: CommonInit(void)
   mInitialized = PR_TRUE;
 #endif
 
-  mContext->GetGammaTable(mGammaTable);
-
   return SetupDC(nsnull, mDC);
 }
 
@@ -538,9 +542,9 @@ NS_IMETHODIMP nsRenderingContextWin :: LockDrawingSurface(PRInt32 aX, PRInt32 aY
 
     mContext->GetPaletteInfo(palInfo);
     if(palInfo.isPaletteDevice && palInfo.palette){
-      ::SelectPalette(mDC,(HPALETTE)palInfo.palette,PR_FALSE);
+      ::SelectPalette(mDC,(HPALETTE)palInfo.palette,PR_TRUE);
       ::RealizePalette(mDC);
-      ::UpdateColors(mDC);
+      ::UpdateColors(mDC);                                                      
     }
   }
 
@@ -963,9 +967,9 @@ NS_IMETHODIMP nsRenderingContextWin :: GetClipRegion(nsIRegion **aRegion)
 NS_IMETHODIMP nsRenderingContextWin :: SetColor(nscolor aColor)
 {
   mCurrentColor = aColor;
-  mColor = RGB(mGammaTable[NS_GET_R(aColor)],
-               mGammaTable[NS_GET_G(aColor)],
-               mGammaTable[NS_GET_B(aColor)]);
+  mColor = RGB(NS_GET_R(aColor),
+               NS_GET_G(aColor),
+               NS_GET_B(aColor));
   return NS_OK;
 }
 
@@ -987,10 +991,10 @@ NS_IMETHODIMP nsRenderingContextWin :: GetLineStyle(nsLineStyle &aLineStyle)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextWin :: SetFont(const nsFont& aFont)
+NS_IMETHODIMP nsRenderingContextWin :: SetFont(const nsFont& aFont, nsIAtom* aLangGroup)
 {
   NS_IF_RELEASE(mFontMetrics);
-  mContext->GetMetricsFor(aFont, mFontMetrics);
+  mContext->GetMetricsFor(aFont, aLangGroup, mFontMetrics);
 
   return NS_OK;
 }
@@ -1520,8 +1524,7 @@ NS_IMETHODIMP nsRenderingContextWin :: GetWidth(const char* aString,
     // space.
     if ((1 == aLength) && (aString[0] == ' '))
     {
-      nsFontMetricsWin* fontMetricsWin = (nsFontMetricsWin*)mFontMetrics;
-      return fontMetricsWin->GetSpaceWidth(aWidth);
+      return mFontMetrics->GetSpaceWidth(aWidth);
     }
 
     SIZE  size;
@@ -1613,17 +1616,16 @@ nsRenderingContextWin::GetTextDimensions(const char*       aString,
     nscoord prevBreakState_Width = 0; // accumulated width to this point
 
     // Initialize OUT parameters
-    aLastWordDimensions.ascent = mCurrFontWin->mMaxAscent;
-    aLastWordDimensions.descent = mCurrFontWin->mMaxDescent;
+    mFontMetrics->GetMaxAscent(aLastWordDimensions.ascent);
+    mFontMetrics->GetMaxDescent(aLastWordDimensions.descent);
     aLastWordDimensions.width = -1;
     aNumCharsFit = 0;
 
     // Iterate each character in the string and determine which font to use
-    nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
     nscoord width = 0;
     PRInt32 start = 0;
     nscoord aveCharWidth;
-    metrics->GetAveCharWidth(aveCharWidth);
+    mFontMetrics->GetAveCharWidth(aveCharWidth);
 
     while (start < aLength) {
       // Estimate how many characters will fit. Do that by diving the available
@@ -1664,7 +1666,7 @@ nsRenderingContextWin::GetTextDimensions(const char*       aString,
       // Measure the text
       nscoord twWidth = 0;
       if ((1 == numChars) && (aString[start] == ' ')) {
-        metrics->GetSpaceWidth(twWidth);
+        mFontMetrics->GetSpaceWidth(twWidth);
       } 
       else if (numChars > 0) {
         SIZE  size;
@@ -1718,7 +1720,7 @@ nsRenderingContextWin::GetTextDimensions(const char*       aString,
           numChars = aBreaks[breakIndex] - start;
           
           if ((1 == numChars) && (aString[start] == ' ')) {
-            metrics->GetSpaceWidth(twWidth);
+            mFontMetrics->GetSpaceWidth(twWidth);
           } 
           else if (numChars > 0) {
             SIZE  size;
@@ -1735,8 +1737,8 @@ nsRenderingContextWin::GetTextDimensions(const char*       aString,
     }
 
     aDimensions.width = width;
-    aDimensions.ascent = mCurrFontWin->mMaxAscent;
-    aDimensions.descent = mCurrFontWin->mMaxDescent;
+    mFontMetrics->GetMaxAscent(aDimensions.ascent);
+    mFontMetrics->GetMaxDescent(aDimensions.descent);
 
     return NS_OK;
   }
@@ -2061,12 +2063,16 @@ nsRenderingContextWin::GetTextDimensions(const PRUnichar*  aString,
   aLastWordDimensions.Clear();
   aLastWordDimensions.width = -1;
 
+  PRInt32 count = fonts.Count();
+  if (!count)
+    return NS_OK;
   nsFontWin* fontWin = (nsFontWin*)fonts[0];
+  NS_ASSERTION(fontWin, "internal error in do_BreakGetTextDimensions");
   aDimensions.ascent = fontWin->mMaxAscent;
   aDimensions.descent = fontWin->mMaxDescent;
 
   // fast path - normal case, quick return if there is only one font
-  if (fonts.Count() == 1)
+  if (count == 1)
     return NS_OK;
 
   // get the last break index.
@@ -2151,10 +2157,13 @@ nsRenderingContextWin::GetTextDimensions(const char*       aString,
                                          PRUint32          aLength,
                                          nsTextDimensions& aDimensions)
 {
-  SetupFontAndColor(); // need this for the case it is space ' '
+  if (!mFontMetrics) {
+    aDimensions.Clear();
+    return NS_ERROR_FAILURE;
+  }
   GetWidth(aString, aLength, aDimensions.width);
-  aDimensions.ascent = mCurrFontWin->mMaxAscent;
-  aDimensions.descent = mCurrFontWin->mMaxDescent;
+  mFontMetrics->GetMaxAscent(aDimensions.ascent);
+  mFontMetrics->GetMaxDescent(aDimensions.descent);
   return NS_OK;
 }
 
@@ -2289,7 +2298,18 @@ do_DrawString(const nsFontSwitch* aFontSwitch,
       x = data->mX;
       y = data->mY;
       data->mTranMatrix->TransformCoord(&x, &y);
-      fontWin->DrawString(data->mDC, x, y, str, 1);
+      if (IS_HIGH_SURROGATE(*str) && 
+          ((str+1)<end) && 
+          IS_LOW_SURROGATE(*(str+1))) 
+      {
+        // special case for surrogate pair
+        fontWin->DrawString(data->mDC, x, y, str, 2);
+        // we need to advance data->mX and str twice
+        data->mX += *data->mSpacing++;
+        ++str;
+      } else {
+        fontWin->DrawString(data->mDC, x, y, str, 1);
+      }
       data->mX += *data->mSpacing++;
       ++str;
     }
@@ -2620,9 +2640,14 @@ NS_IMETHODIMP nsRenderingContextWin :: CopyOffScreenBits(nsDrawingSurface aSrcSu
       mContext->GetPaletteInfo(palInfo);
 
       if (palInfo.isPaletteDevice && palInfo.palette){
-        ::SelectPalette(destdc, (HPALETTE)palInfo.palette, PR_FALSE);
-        ::RealizePalette(destdc);
-        ::UpdateColors(destdc);
+        ::SelectPalette(destdc, (HPALETTE)palInfo.palette, PR_TRUE);
+        // this is called to many times here.. taking this out because it breaks
+        // embedding.. its like fighting palettes.  All the palette stuff should
+        // be taken out.. but its so late in the beta cycle... I am taking the safe
+        // road until I can get all this figured out.. and completed correctly.
+        // Opened bug #153367 to take care of this palette issue.
+        //::RealizePalette(destdc);
+        ::UpdateColors(mDC);                                                      
       }
 
       if (aCopyFlags & NS_COPYBITS_XFORM_SOURCE_VALUES)
@@ -2766,8 +2791,10 @@ void nsRenderingContextWin :: SetupFontAndColor(void)
 
     mCurrFont = tfont;
 
-    nsFontMetricsWin* metrics = (nsFontMetricsWin*)mFontMetrics;
-    mCurrFontWin = metrics->GetFontFor(mCurrFont);
+    // nsFontMetricsWin vs. nsFontMetricsWinA
+    // When making changes in the font code, set |useAFunctions = 1| in nsGfxFactoryWin
+    // to verify that the changes didn't let the 'A' versions out of sync. 
+    NS_ASSERTION(((nsFontMetricsWin*)mFontMetrics)->GetFontFor(mCurrFont), "internal error");
 
     mCurrFontMetrics = mFontMetrics;
   }
