@@ -57,14 +57,19 @@
 #include "xprintutil.h"
 #include "prenv.h" /* for PR_GetEnv */
 
-/* misc defines */
-#define XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING 1
-
-#ifdef XPRINT_NOT_YET /* ToDo: make this dynamically */
-#define NS_XPRINT_RGB_DITHER XLIB_RGB_DITHER_NONE
-#else
-#define NS_XPRINT_RGB_DITHER ((mDepth>12)?(XLIB_RGB_DITHER_NONE):(XLIB_RGB_DITHER_MAX))
-#endif
+/* NS_XPRINT_RGB_DITHER: Macro to check whether we should dither or not.
+ * In theory we only have to look at the visual and depth ("TrueColor" with 
+ * enougth bits for the colors or GrayScale/StaticGray with enougth bits for
+ * the grayscale shades).
+ * In real life some Xprt DDX do not have the GrayScale/StaticGray visuals and
+ * we emulate grayscale support with a PseudoColor visual+grayscale palette
+ * (that's why we test for |mIsGrayscale| explicitly)...
+ */
+#define NS_XPRINT_RGB_DITHER \
+    (((mDepth >  12 && mVisual->c_class==TrueColor)  || \
+      (mDepth >=  7 && mVisual->c_class==GrayScale)  || \
+      (mDepth >=  7 && mVisual->c_class==StaticGray) || \
+      (mIsGrayscale == PR_TRUE)) ?(XLIB_RGB_DITHER_NONE):(XLIB_RGB_DITHER_MAX))
 
 #ifdef PR_LOGGING 
 /* DEBUG: use 
@@ -89,25 +94,25 @@ PR_END_EXTERN_C
 /** ---------------------------------------------------
  *  Default Constructor
  */
-nsXPrintContext::nsXPrintContext()
+nsXPrintContext::nsXPrintContext() :
+  mXlibRgbHandle(nsnull),
+  mPDisplay(nsnull),
+  mScreen(nsnull),
+  mVisual(nsnull),
+  mDrawable(None),
+  mGC(nsnull),
+  mDepth(0),
+  mPContext(None),
+  mJobStarted(PR_FALSE),
+  mIsGrayscale(PR_FALSE), /* default is color output */
+  mIsAPrinter(PR_TRUE),   /* default destination is printer */
+  mPrintFile(nsnull),
+  mXpuPrintToFileHandle(nsnull),
+  mPrintResolution(0L),
+  mContext(nsnull)
 {
   NS_INIT_REFCNT();
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::nsXPrintContext()\n"));
-  
-  mXlibRgbHandle = (XlibRgbHandle *)nsnull;
-  mPDisplay    = (Display *)nsnull;
-  mPContext    = (XPContext)None;
-  mScreen      = (Screen *)nsnull;
-  mVisual      = (Visual *)nsnull;
-  mDrawable    = (Drawable)None;
-  mGC          = nsnull;
-  mDepth       = 0;
-  mIsGrayscale = PR_FALSE; /* default is color output */
-  mIsAPrinter  = PR_TRUE;  /* default destination is printer */
-  mPrintFile   = nsnull;
-  mXpuPrintToFileHandle = nsnull;
-  mPrintResolution = 0L;
-  mContext     = nsnull;
 }
 
 /** ---------------------------------------------------
@@ -119,27 +124,27 @@ nsXPrintContext::~nsXPrintContext()
  
   if (mPDisplay)
   {
+    if (mJobStarted)
+    {
+      /* Clean-up if noone else did it yet... */
+      AbortDocument();
+    }
+
     if (mGC)
     {
       mGC->Release();
       mGC = nsnull;
     }
-    
-    if (mPContext != None)
-    {
-      XPU_TRACE(XpDestroyContext(mPDisplay, mPContext));
-      mPContext = None;
-    }
-    
+       
     if (mXlibRgbHandle)
     {
       xxlib_rgb_destroy_handle(mXlibRgbHandle);
       mXlibRgbHandle = nsnull;
     }
-        
-    XPU_TRACE(XCloseDisplay(mPDisplay));
     
+    XPU_TRACE(XpuClosePrinterDisplay(mPDisplay, mPContext));           
     mPDisplay = nsnull;
+    mPContext = None;
   }
   
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::~nsXPrintContext() done.\n"));
@@ -147,32 +152,55 @@ nsXPrintContext::~nsXPrintContext()
 
 NS_IMPL_ISUPPORTS1(nsXPrintContext, nsIDrawingSurfaceXlib)
 
+
+static nsresult
+AlertBrokenXprt(Display *pdpy)
+{
+  /* Check for broken Xprt
+   * Xfree86 Xprt is the only known broken version of Xprt (see bug 120211
+   * for a list of tested&working Xprt servers) ...
+   * XXX: We should look at XVendorRelease(), too - but there is no feedback
+   * from XFree86.org when this issue will be fixed... ;-(
+   */
+  if (!(strstr(XServerVendor(pdpy), "XFree86") /*&& XVendorRelease(pdpy) < 45000000L*/))
+    return NS_OK;
+
+  /* Bad version found... log the issue and show the dialog and warn the user... */
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG,
+         ("nsXPrintContext::AlertBrokenXprt: vendor: '%s', release=%ld\n",
+          XServerVendor(pdpy), (long)XVendorRelease(pdpy)));
+
+  /* Dialog disabled ? */
+  if (PR_GetEnv("MOZILLA_XPRINT_DISABLE_BROKEN_XFREE86_WARNING") != nsnull)
+    return NS_OK;
+   
+  return NS_ERROR_GFX_PRINTER_XPRINT_BROKEN_XPRT;
+}
+
 NS_IMETHODIMP 
 nsXPrintContext::Init(nsDeviceContextXp *dc, nsIDeviceContextSpecXp *aSpec)
 {
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::Init()\n"));
   nsresult rv = NS_ERROR_FAILURE;
 
-  int   prefDepth = 8;  /* 24 or 8 for PS DDX, 24, 8, 1 for PCL DDX... 
-                         * I wish current Xprt would have a 1bit/8bit StaticGray 
-                         * visual for the PS DDX... ;-( 
-                         */
-/* I can't get any other visual than the 8bit peudocolor one working... BAD...
- * This env var allows others to test this without hacking their own binaries...
- */
-#ifdef XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING  
-  if( PR_GetEnv("MOZILLA_XPRINT_EXPERIMENTAL_USE_24BIT_VISUAL") != nsnull )
+  int   prefDepth = 24;  /* 24 or 8 for PS DDX, 24, 8, 1 for PCL DDX... 
+                          * I wish current Xprt would have a 1bit/8bit StaticGray 
+                          * visual for the PS DDX... ;-( 
+                          */
+  /* Safeguard for production use (see bug 80562),
+   * Make sure we can switch back to the
+   * extensively tested 8bit Pseudocolor visual on demand... */
+  if( PR_GetEnv("MOZILLA_XPRINT_EXPERIMENTAL_DISABLE_24BIT_VISUAL") != nsnull )
   {
-    prefDepth = 24;
+    prefDepth = 8;
   }
-#endif /* XPRINT_MAKE_24BIT_VISUAL_AVAILABLE_FOR_TESTING */  
   
   unsigned short width, height;
   XRectangle rect;
 
   if (NS_FAILED(XPU_TRACE(rv = SetupPrintContext(aSpec))))
     return rv;
-  
+
   mScreen = XpGetScreenOfContext(mPDisplay, mPContext);
   mScreenNumber = XScreenNumberOfScreen(mScreen);
 
@@ -206,21 +234,48 @@ nsXPrintContext::Init(nsDeviceContextXp *dc, nsIDeviceContextSpecXp *aSpec)
         xargs.pseudogray      = True;
         mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
       }
+
+      if (!mXlibRgbHandle)
+      {
+        PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("trying black/white\n"));
+
+        /* Last attempt using StaticGray 1bit (b/w printer) */
+        xargs.xtemplate.c_class = StaticGray;
+        xargs.xtemplate.depth   = 1;
+        xargs.xtemplate_mask    = VisualClassMask|VisualDepthMask;
+        xargs.pseudogray        = False;
+        mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+      }
     }
   }
   else
   {
     PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("printing color\n"));
-    xargs.xtemplate.depth   = prefDepth;
-    xargs.xtemplate.c_class = (prefDepth>12)?(TrueColor):(PseudoColor);
-    xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
-    mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    if (prefDepth > 12)
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("trying TrueColor %d bit\n", prefDepth));
+      xargs.xtemplate.depth   = prefDepth;
+      xargs.xtemplate.c_class = TrueColor;
+      xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
+      mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    }  
+
+    if (!mXlibRgbHandle)
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("trying PseudoColor 8 bit\n"));
+      xargs.xtemplate.depth   = 8;
+      xargs.xtemplate.c_class = PseudoColor;
+      xargs.xtemplate_mask    = VisualDepthMask|VisualClassMask;
+      mXlibRgbHandle = xxlib_rgb_create_handle(mPDisplay, mScreen, &xargs);
+    }  
   }
   
+  /* No XlibRgb handle ? Either we do not have a matching visual or no memory... */
   if (!mXlibRgbHandle)
-    return NS_ERROR_GFX_PRINTER_INVALID_ATTRIBUTE;
+    return NS_ERROR_GFX_PRINTER_COLORSPACE_NOT_SUPPORTED;
 
   XpGetPageDimensions(mPDisplay, mPContext, &width, &height, &rect);
+
   rv = SetupWindow(rect.x, rect.y, rect.width, rect.height);
   if (NS_FAILED(rv))
     return rv;
@@ -264,17 +319,26 @@ nsXPrintContext::SetupWindow(int x, int y, int width, int height)
   mVisual     = xxlib_rgb_get_visual(mXlibRgbHandle);
   mDepth      = xxlib_rgb_get_depth(mXlibRgbHandle);
 
-  background = XWhitePixel(mPDisplay, mScreenNumber);
-  foreground = XBlackPixel(mPDisplay, mScreenNumber);  
+  background = xxlib_rgb_xpixel_from_rgb(mXlibRgbHandle, NS_TO_XXLIB_RGB(NS_RGB(0xFF, 0xFF, 0xFF))); /* white */
+  foreground = xxlib_rgb_xpixel_from_rgb(mXlibRgbHandle, NS_TO_XXLIB_RGB(NS_RGB(0x00, 0x00, 0x00))); /* black */
   parent_win = XRootWindow(mPDisplay, mScreenNumber);                                         
                                              
   xattributes.background_pixel = background;
   xattributes.border_pixel     = foreground;
   xattributes.colormap         = xxlib_rgb_get_cmap(mXlibRgbHandle);
   xattributes_mask             = CWBorderPixel | CWBackPixel;
-  if( xattributes.colormap )
+  if( xattributes.colormap != None )
+  {
     xattributes_mask |= CWColormap;
 
+    /* XXX: Why do we need this ? See bug 80562 ("Xprint does not support any
+     * other visuals than Xprt's default one..").*/
+    if (mDepth > 12)
+    {
+      XInstallColormap(mPDisplay, xattributes.colormap);
+    }  
+  }
+
   mDrawable = (Drawable)XCreateWindow(mPDisplay, parent_win, x, y,
                                       width, height, 0,
                                       mDepth, InputOutput, mVisual, xattributes_mask,
@@ -289,25 +353,167 @@ nsXPrintContext::SetupWindow(int x, int y, int width, int height)
 }
 
 
+nsresult nsXPrintContext::SetMediumSize(const char *aPaperName)
+{
+  nsresult                rv = NS_ERROR_GFX_PRINTER_PAPER_SIZE_NOT_SUPPORTED;
+  XpuMediumSourceSizeList mlist;
+  int                     mlist_count;
+  int                     i;
+  char                   *paper_name,
+                         *alloc_paper_name; /* for free() ... */
+  paper_name = alloc_paper_name = strdup(aPaperName);
+  if (!paper_name)
+    return NS_ERROR_OUT_OF_MEMORY;
+  
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: Requested page '%s'\n", paper_name));
+
+  mlist = XpuGetMediumSourceSizeList(mPDisplay, mPContext, &mlist_count);
+  if( !mlist )
+  {
+    return NS_ERROR_GFX_PRINTER_PAPER_SIZE_NOT_SUPPORTED;
+  }
+
+  XpuMediumSourceSizeRec *match = nsnull;
+
+#ifdef PR_LOGGING 
+  /* Print page sizes for the log... */
+  for( i = 0 ; i < mlist_count ; i++ )
+  {
+    XpuMediumSourceSizeRec *curr = &mlist[i];
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("got '%s'/'%s'\t%d %f %f %f %f\n", 
+           XPU_NULLXSTR(curr->tray_name), curr->medium_name, curr->mbool, 
+           curr->ma1, curr->ma2, curr->ma3, curr->ma4));
+  }
+#endif /* PR_LOGGING */
+
+  char *s;
+  
+  /* Did we get a tray name and paper name (e.g. "manual/din-a4") ? */
+  if ((s = strchr(paper_name, '/')) != nsnull)
+  {
+    const char *tray_name;
+    *s = '\0';
+    tray_name  = paper_name;
+    paper_name = s+1;
+    
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: searching for '%s'/'%s'\n", tray_name, paper_name));
+    match = XpuFindMediumSourceSizeByName(mlist, mlist_count, tray_name, paper_name);
+  }
+  else
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("SetMediumSize: searching for '%s'\n", paper_name));
+    match = XpuFindMediumSourceSizeByName(mlist, mlist_count, nsnull, paper_name);
+  }
+  
+  /* Found a match ? */
+  if (match)
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG,
+           ("match '%s'/'%s' !\n", XPU_NULLXSTR(match->tray_name), match->medium_name));
+           
+    /* Set document's paper size */
+    if( XpuSetDocMediumSourceSize(mPDisplay, mPContext, match) == 1 )
+      rv = NS_OK;  
+  }
+  
+  XpuFreeMediumSourceSizeList(mlist);
+  free(alloc_paper_name);
+  
+  return rv;
+}
+
+nsresult nsXPrintContext::SetOrientation(int landscape)
+{
+  const char         *orientation;
+  XpuOrientationList  list;
+  int                 list_count;
+  XpuOrientationRec  *match;
+
+  /* which orientation ? */
+  switch( landscape )
+  {
+    case 1 /* NS_LANDSCAPE */: orientation = "landscape"; break;
+    case 0 /* NS_PORTRAIT */:  orientation = "portrait";  break;
+    default:  
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, 
+             ("Unsupported orientation %d.\n", landscape));  
+      return NS_ERROR_GFX_PRINTER_ORIENTATION_NOT_SUPPORTED;
+  }
+  
+  /* Print the used orientation to the log... */
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("orientation=%s\n", orientation));    
+
+  /* Get list of supported orientations */
+  list = XpuGetOrientationList(mPDisplay, mPContext, &list_count);
+  if( !list )
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuGetOrientationList() failure.\n"));  
+    return NS_ERROR_GFX_PRINTER_ORIENTATION_NOT_SUPPORTED;
+  }
+
+#ifdef PR_LOGGING 
+  int i;
+  /* Print orientations for the log... */
+  for( i = 0 ; i < list_count ; i++ )
+  {
+    XpuOrientationRec *curr = &list[i];
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("got orientation='%s'\n", curr->orientation));
+  }
+#endif /* PR_LOGGING */
+
+  /* Find requested orientation */
+  match = XpuFindOrientationByName(list, list_count, orientation);
+  if (!match)
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuFindOrientationByName() failure.\n"));  
+    XpuFreeOrientationList(list);
+    return NS_ERROR_GFX_PRINTER_ORIENTATION_NOT_SUPPORTED;
+  }
+
+  /* Set orientation */
+  if (XpuSetDocOrientation(mPDisplay, mPContext, match) != 1)
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuSetDocOrientation() failure.\n"));  
+    
+    /* We have a "match" in the list of supported orientations but we are not
+     * allowed to set it... Well - if this happens and we only have ONE entry
+     * in this list then we can safely assume that this is the only choice...
+     * (please correct me if I am wrong) 
+     */
+    if (list_count != 1)
+    {
+      /* ... otherwise we have a problem... */
+      XpuFreeOrientationList(list);
+      return NS_ERROR_GFX_PRINTER_ORIENTATION_NOT_SUPPORTED;
+    }
+  }
+  
+  XpuFreeOrientationList(list);
+
+  return NS_OK;
+}
+
+
 nsresult
 nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
 {
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::SetupPrintContext()\n"));
   
-  int    printSize;
-  float  top, bottom, left, right;
-  int    landscape;
-  char  *buf;
+  float        top, bottom, left, right;
+  int          landscape;
+  int          num_copies;
+  const char  *printername;
+  nsresult     rv;
 
   // Get the Attributes
   aSpec->GetToPrinter(mIsAPrinter);
   aSpec->GetGrayscale(mIsGrayscale);
-  aSpec->GetSize(printSize);
   aSpec->GetTopMargin(top);
   aSpec->GetBottomMargin(bottom);
   aSpec->GetLeftMargin(left);
   aSpec->GetRightMargin(right);
   aSpec->GetLandscape(landscape);
+  aSpec->GetCopies(num_copies);
   
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, 
          ("nsXPrintContext::SetupPrintContext: borders top=%f, bottom=%f, left=%f, right=%f\n", 
@@ -316,7 +522,7 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
   /* get destination printer (we need this when printing to file as
    * the printer DDX in Xprt generates the data...) 
    */
-  aSpec->GetPrinterName(&buf);
+  aSpec->GetPrinterName(&printername);
   
   /* Are we "printing" to a file instead to the print queue ? */
   if (!mIsAPrinter) 
@@ -340,38 +546,42 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
    */
   PR_SetEnv("XSUNTRANSPORT=xxx");
      
-  /* get printer, either by "name" (foobar) or "name@display" (foobar@gaja:5)
-   * ToDo: report error to user (dialog)
-   */
-  if( XpuGetPrinter(buf, &mPDisplay, &mPContext) != 1 )
+  /* Get printer, either by "name" (foobar) or "name@display" (foobar@gaja:5) */
+  if( XpuGetPrinter(printername, &mPDisplay, &mPContext) != 1 )
     return NS_ERROR_GFX_PRINTER_NAME_NOT_FOUND;
+
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, 
+         ("nsXPrintContext::SetupPrintContext: name='%s', display='%s', vendor='%s', release=%ld\n",
+          printername,
+          XDisplayString(mPDisplay), 
+          XServerVendor(mPDisplay),
+          (long)XVendorRelease(mPDisplay)));
+          
+  if (NS_FAILED(rv = AlertBrokenXprt(mPDisplay)))
+    return rv;
+    
+  if( XpQueryExtension(mPDisplay, &mXpEventBase, &mXpErrorBase) == False )
+    return NS_ERROR_UNEXPECTED;
     
 #ifdef XPRINT_DEBUG_SOMETIMES_USEFULL
   dumpXpAttributes(mPDisplay, mPContext);
-#endif /* DEBUG */
+#endif /* XPRINT_DEBUG_SOMETIMES_USEFULL */
+  
+  const char *paper_name = nsnull;
+  aSpec->GetPaperName(&paper_name);
+  
+  if (NS_FAILED(XPU_TRACE(rv = SetMediumSize(paper_name))))
+    return rv;
+  
+  if (NS_FAILED(XPU_TRACE(rv = SetOrientation(landscape))))
+    return rv;
 
-  /* which orientation ? */
-  switch (landscape)
-  {
-    case 1 /* NS_LANDSCAPE */:
-      if (XpuSetContentOrientation(mPDisplay,mPContext, XPDocAttr, "landscape") != 1)
-      {
-        NS_WARNING("orientation 'landscape' not supported on this printer");
-        return NS_ERROR_FAILURE;
-      }
-      break;
-    case 0 /* NS_PORTRAIT */:
-      if (XpuSetContentOrientation(mPDisplay,mPContext, XPDocAttr, "portrait") != 1)
-      {
-        NS_WARNING("orientation 'portrait' not supported on this printer");
-        return NS_ERROR_FAILURE;
-      }
-      break;
-    default:  
-      NS_WARNING("unsupported orientation");
-      return NS_ERROR_FAILURE;
-  }    
-    
+  if (NS_FAILED(XPU_TRACE(rv = SetResolution())))
+    return rv;
+   
+  if (XPU_TRACE(XpuSetDocumentCopies(mPDisplay, mPContext, num_copies)) != 1)
+    return NS_ERROR_GFX_PRINTER_TOO_MANY_COPIES;
+        
   /* set printer context
    * WARNING: after this point it is no longer allows to change job attributes
    * only after the XpSetContext() call the alllication is allowed to make 
@@ -380,11 +590,13 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
    */
   XPU_TRACE(XpSetContext(mPDisplay, mPContext));
 
-  /* get default printer resolution. May fail if Xprt is misconfigured.
-   * ToDo: Report error to user (dialog)
-   */
-  if( XpuGetResolution(mPDisplay, mPContext, &mPrintResolution) == False )
-    return NS_ERROR_GFX_PRINTER_INVALID_ATTRIBUTE;
+#ifdef XPRINT_DEBUG_SOMETIMES_USEFULL
+  dumpXpAttributes(mPDisplay, mPContext);
+#endif /* XPRINT_DEBUG_SOMETIMES_USEFULL */
+
+  /* Get default printer resolution. May fail if Xprt is misconfigured.*/
+  if( XpuGetResolution(mPDisplay, mPContext, &mPrintResolution) != 1 )
+    return NS_ERROR_GFX_PRINTER_DRIVER_CONFIGURATION_ERROR;
 
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("print resolution %ld\n", (long)mPrintResolution));
   
@@ -392,68 +604,132 @@ nsXPrintContext::SetupPrintContext(nsIDeviceContextSpecXp *aSpec)
   XpSelectInput(mPDisplay, mPContext, XPPrintMask);  
 
   return NS_OK;
-}  
-  
-/* MyConvertUCS2ToLocalEncoding:
- * Note that this function is _only_ a _hack_ until RFE 73446 
- * (http://bugzilla.mozilla.org/show_bug.cgi?id=73446 - "RFE: 
- * Need NS_ConvertUCS2ToLocalEncoding() and 
- * NS_ConvertLocalEncodingToUCS2()") gets implemented...
- * Below we need COMPOUNT_TEXT which usually is the same as the Xserver's 
- * local encoding - this hack should at least work for C/POSIX 
- * and *.UTF-8 locales...
- */
-static 
-char *MyConvertUCS2ToLocalEncoding( PRUnichar *str )
-{
-  /* Use strdup() to avoid any silly effects... 
-   */
-  return PL_strdup(NS_ConvertUCS2toUTF8(str).get());
-}  
-
+}
 
 NS_IMETHODIMP
-nsXPrintContext::BeginDocument( PRUnichar *aTitle )
+nsXPrintContext::SetResolution( void )
 {
-  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::BeginDocument()\n"));
-  
-  char *s = nsnull, 
-       *job_title;
-       
-  if( aTitle != nsnull )
-    job_title = s = MyConvertUCS2ToLocalEncoding(aTitle); 
-  else
-    job_title = (char *)"Mozilla document without title";  
+  XpuResolutionList list;
+  int               list_count;
+  XpuResolutionRec *match;
+  int               i;
+  long              default_resolution;
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::SetResolution().\n"));  
 
-  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::BeginDocument: document title: '%s'\n", XPU_NULLXSTR(job_title)));
+  list = XpuGetResolutionList(mPDisplay, mPContext, &list_count);
+  if( !list )
+    return NS_ERROR_GFX_PRINTER_DRIVER_CONFIGURATION_ERROR;
+
+#ifdef PR_LOGGING 
+  /* Print resolutions for the log... */
+  for( i = 0 ; i < list_count ; i++ )
+  {
+    XpuResolutionRec *curr = &list[i];
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("got resolution=%ld\n", (long)curr->dpi));
+  }
+#endif /* PR_LOGGING */
+
+  /* We rely on printer default resolution if we have one... */
+  if( XpuGetResolution(mPDisplay, mPContext, &default_resolution) == 1 )
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("using printers default resolution=%ld.\n", default_resolution));  
+    XpuFreeResolutionList(list);
+    return NS_OK;
+  }
+
+  /* XXX: Hardcoded resolution values... */
+  match = XpuFindResolution(list, list_count, 300, 300); 
+  if (!match)
+  {
+    /* Find a match between 300-600, lower resolution is better */
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("searching between 300-600, lower resolution is better...\n"));
+    match = XpuFindResolution(list, list_count, 300, 600); 
+
+    if (!match)
+    {
+      /* Find a match between 150-300, higher resolution is better */
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("searching between 150-300, higher resolution is better...\n"));
+      match = XpuFindResolution(list, list_count, 300, 150); 
+
+      if (!match)
+      {
+        /* If there is still no match then use the first one from the matches we
+         * obtained...*/
+         match = &list[0];
+      }
+    }  
+  }
+
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("setting resolution to %ld DPI.\n", match->dpi));  
+    
+  if (XpuSetDocResolution(mPDisplay, mPContext, match) != 1)
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuSetDocResolution() failure.\n"));
+ 
+    /* We have a "match" in the list of supported resolutions but we are not
+     * allowed to set it... Well - if this happens and we only have ONE entry
+     * in this list then we can safely assume that this is the only choice...
+     * (please correct me if I am wrong) 
+     */
+    if (list_count != 1)
+    {
+      /* ... otherwise we have a problem... */
+      XpuFreeResolutionList(list);
+      return NS_ERROR_GFX_PRINTER_DRIVER_CONFIGURATION_ERROR;
+    }
+  }
   
+  XpuFreeResolutionList(list);
+  
+  return NS_OK;
+}
+  
+NS_IMETHODIMP
+nsXPrintContext::BeginDocument( PRUnichar * aTitle, PRUnichar* aPrintToFileName, PRInt32 aStartPage, PRInt32 aEndPage )
+{
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::BeginDocument(aTitle='%s')\n", ((aTitle)?(NS_ConvertUCS2toUTF8(aTitle).get()):("<NULL>"))));
+  
+  nsXPIDLCString job_title;
+       
+  if (aTitle)
+  {
+    /* Note that this is _only_ a _hack_ until bug 73446 
+     * ("RFE: Need NS_ConvertUCS2ToLocalEncoding() and 
+     * NS_ConvertLocalEncodingToUCS2()") is implemented...
+     * Below we need COMPOUNT_TEXT which usually is the same as the Xserver's 
+     * local encoding - this hack should at least work for C/POSIX 
+     * and *.UTF-8 locales...
+     */
+    job_title.Assign(NS_ConvertUCS2toUTF8(aTitle));
+  }
+  else
+  {
+    job_title.Assign(NS_LITERAL_CSTRING("Mozilla document without title"));
+  }
+ 
   /* Set the Job Attributes */
   XpuSetJobTitle(mPDisplay, mPContext, job_title);
   
-  if( s != nsnull ) 
-    PL_strfree(s);
-
   // Check the output type
   if(mIsAPrinter) 
   {
-    XPU_TRACE(XpStartJob(mPDisplay, XPSpool));
-    XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, XPStartJobNotify));
+    XPU_TRACE(XpuStartJobToSpooler(mPDisplay));
   } 
   else 
   {   
-    XPU_TRACE(XpStartJob(mPDisplay, XPGetData));
-    
-    if( XPU_TRACE(mXpuPrintToFileHandle = XpuPrintToFile(mPDisplay, mPContext, mPrintFile)) == nsnull )
+    if( XPU_TRACE(mXpuPrintToFileHandle = XpuStartJobToFile(mPDisplay, mPContext, mPrintFile)) == nsnull )
     {
       PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, 
              ("nsXPrintContext::BeginDocument(): XpuPrintToFile failure %s/(%d)\n", 
              strerror(errno), errno));
 
       return NS_ERROR_GFX_PRINTER_COULD_NOT_OPEN_FILE;
-    }
-    
-    XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, XPStartJobNotify));
+    }    
   } 
+
+  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, mXpEventBase, XPStartJobNotify));
+
+  mJobStarted = PR_TRUE;
   
   return NS_OK;
 }  
@@ -461,11 +737,10 @@ nsXPrintContext::BeginDocument( PRUnichar *aTitle )
 NS_IMETHODIMP
 nsXPrintContext::BeginPage()
 {
-  // Move the print window according to the given margin
-  // XMoveWindow(mPDisplay, mDrawable, 100, 100);
-  
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::BeginPage()\n"));
+   
   XPU_TRACE(XpStartPage(mPDisplay, mDrawable));
-  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, XPStartPageNotify));
+  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, mXpEventBase, XPStartPageNotify));
 
   return NS_OK;
 }
@@ -473,8 +748,10 @@ nsXPrintContext::BeginPage()
 NS_IMETHODIMP 
 nsXPrintContext::EndPage()
 {
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::EndPage()\n"));
+
   XPU_TRACE(XpEndPage(mPDisplay));
-  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, XPEndPageNotify));
+  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, mXpEventBase, XPEndPageNotify));
   
   return NS_OK;
 }
@@ -482,8 +759,10 @@ nsXPrintContext::EndPage()
 NS_IMETHODIMP 
 nsXPrintContext::EndDocument()
 {
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::EndDocument()\n"));
+
   XPU_TRACE(XpEndJob(mPDisplay));
-  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, XPEndJobNotify));
+  XPU_TRACE(XpuWaitForPrintNotify(mPDisplay, mXpEventBase, XPEndJobNotify));
 
   /* Are we printing to a file ? */
   if( !mIsAPrinter )
@@ -494,8 +773,49 @@ nsXPrintContext::EndDocument()
     {
       PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuWaitForPrintFileChild returned success.\n"));
     }
-  }
+    else
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuWaitForPrintFileChild returned failure.\n"));
+    }
     
+    mXpuPrintToFileHandle = nsnull;
+  }
+
+  mJobStarted = PR_FALSE;
+    
+  return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsXPrintContext::AbortDocument()
+{
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::AbortDocument()\n"));
+
+  if( mJobStarted )
+  {
+    PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("canceling...\n"));
+    XPU_TRACE(XpCancelJob(mPDisplay, True));
+  }  
+
+  /* Are we printing to a file ? */
+  if( !mIsAPrinter && mXpuPrintToFileHandle )
+  {   
+    if( XPU_TRACE(XpuWaitForPrintFileChild(mXpuPrintToFileHandle)) == XPGetDocFinished )
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuWaitForPrintFileChild returned success.\n"));
+    }
+    else
+    {
+      PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("XpuWaitForPrintFileChild returned failure.\n"));
+    }
+
+    mXpuPrintToFileHandle = nsnull;
+  }
+
+  mJobStarted = PR_FALSE;
+
+  PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("print job aborted.\n"));
+
   return NS_OK;
 }
 
@@ -604,12 +924,12 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
           (int)aSX, (int)aSY, (int)aSWidth, (int)aSHeight,
           (int)aDX, (int)aDY, (int)aDWidth, (int)aDHeight));
   
-  double   scaler;
+  double   scalingFactor;
   int      prev_res = 0,
            dummy;
   long     imageResolution;
-  PRInt32  aDWidth_scaled;
-  PRInt32  aDHeight_scaled;
+  PRInt32  aDWidth_scaled,
+           aDHeight_scaled;
   nsresult rv = NS_OK;
   
   PRInt32 aSrcWidth  = aImage->GetWidth();
@@ -626,15 +946,40 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
     return NS_OK;
   }
 
-  float pixelscaler = 1.0;
-  mContext->GetCanonicalPixelScale(pixelscaler);
-  scaler = 1.0 / pixelscaler;
+  /* Get the general scaling factor for this device */
+  float pixelscale = 1.0;
+  mContext->GetCanonicalPixelScale(pixelscale);
+  scalingFactor = 1.0 / pixelscale;
 
-  imageResolution = (double)mPrintResolution * scaler;
-  aDWidth_scaled  = (double)aDWidth  * scaler;
-  aDHeight_scaled = (double)aDHeight * scaler;
+  /* Calculate image scaling factors */
+  double scale_x = double(aSWidth)  / (double(aDWidth)  * scalingFactor);
+  double scale_y = double(aSHeight) / (double(aDHeight) * scalingFactor);
   
-  /* set image resolution */
+  /* Which scaling factor is the "bigger" one ? 
+   * (XpSetImageResolution() sets one image resolution for both X- and
+   * Y-axis - therefore we want to get the "best"(=smallest job size) out
+   * of the scaling factor...).
+   */
+  scalingFactor *= (scale_x < scale_y)?(scale_x):(scale_y);
+  
+  /* Adjust destination size to the match the scaling factor */
+  imageResolution = long(   double(mPrintResolution) * scalingFactor);
+  aDWidth_scaled  = PRInt32(double(aDWidth)          * scalingFactor);
+  aDHeight_scaled = PRInt32(double(aDHeight)         * scalingFactor);
+
+  /* Image scaled to 0 width/height ? */
+  NS_ASSERTION(!((aDWidth_scaled <= 0) || (aDHeight_scaled <= 0)),
+               "Image scaled to zero width/height");
+  if( (aDWidth_scaled <= 0) || (aDHeight_scaled <= 0) )
+    return NS_OK;
+
+  /* Image scaled to zero-width/height ? */
+  NS_ASSERTION(imageResolution != 0, "Image resolution must not be 0");
+  NS_ASSERTION(imageResolution >= 0, "Image resolution must not be negative");
+  if( imageResolution <= 0 )
+    return NS_OK;
+
+  /* Set image resolution */
   if( XpSetImageResolution(mPDisplay, mPContext, imageResolution, &prev_res) )
   {
     /* draw image, Xprt will do the main scaling part... */
@@ -690,6 +1035,8 @@ nsXPrintContext::DrawImageBitsScaled(xGC *xgc, nsIImage *aImage,
     return NS_OK;
   }
   
+  aImage->LockImagePixels(PR_FALSE);
+  
   PRUint8 *image_bits    = aImage->GetBits();
   PRInt32  row_bytes     = aImage->GetLineStride();
   PRUint8  imageDepth    = 24; /* R8G8B8 packed. Thanks to "tor" for that hint... */
@@ -708,7 +1055,11 @@ nsXPrintContext::DrawImageBitsScaled(xGC *xgc, nsIImage *aImage,
                                       image_bits, row_bytes,
                                       aSrcWidth, aSrcHeight);
     if (!composed_bits)
+    {
+      aImage->UnlockImagePixels(PR_FALSE);
       return NS_ERROR_FAILURE;
+    }
+
     image_bits = composed_bits;
     alphaBits = nsnull; /* ComposeAlphaImage() handled the alpha channel. 
                          * Once we enable XPRINT_SERVER_SIDE_ALPHA_COMPOSING 
@@ -725,7 +1076,10 @@ nsXPrintContext::DrawImageBitsScaled(xGC *xgc, nsIImage *aImage,
   PRUint8 *srcimg_data           = image_bits;
   PRUint8 *dstimg_data           = (PRUint8 *)PR_Malloc((aDHeight+1) * dstimg_bytes_per_line); 
   if (!dstimg_data)
+  {
+    aImage->UnlockImagePixels(PR_FALSE);
     return NS_ERROR_FAILURE;
+  }
 
   RectStretch(aSX, aSY, aSX+aSWidth-1, aSY+aSHeight-1,
               0, 0, (aDWidth-1), (aDHeight-1),
@@ -746,6 +1100,8 @@ nsXPrintContext::DrawImageBitsScaled(xGC *xgc, nsIImage *aImage,
   if (composed_bits) 
     PR_Free(composed_bits);
   
+  aImage->UnlockImagePixels(PR_FALSE);
+
   return rv;
 }
 
@@ -758,6 +1114,8 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
   PR_LOG(nsXPrintContextLM, PR_LOG_DEBUG, ("nsXPrintContext::DrawImage(%d/%d/%d(=dummy)/%d(=dummy))\n",
          (int)aX, (int)aY, (int)dummy1, (int)dummy2));
 
+  aImage->LockImagePixels(PR_FALSE);
+
   nsresult rv;
   PRInt32  width         = aImage->GetWidth();
   PRInt32  height        = aImage->GetHeight();
@@ -776,7 +1134,11 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
                                       image_bits, row_bytes,
                                       width, height);
     if (!composed_bits)
+    {
+      aImage->UnlockImagePixels(PR_FALSE);
       return NS_ERROR_FAILURE;
+    }
+
     image_bits = composed_bits;
     alphaBits = nsnull;
   }
@@ -787,10 +1149,13 @@ nsXPrintContext::DrawImage(xGC *xgc, nsIImage *aImage,
 
   if (composed_bits)
     PR_Free(composed_bits);
+
+  aImage->UnlockImagePixels(PR_FALSE);
+
   return rv;                     
 }
 
-                          
+                         
 // Draw the bitmap, this draw just has destination coordinates
 nsresult
 nsXPrintContext::DrawImageBits(xGC *xgc,
@@ -887,12 +1252,13 @@ nsXPrintContext::DrawImageBits(xGC *xgc,
   }
 
 
-  xxlib_draw_rgb_image(mXlibRgbHandle,
-                       mDrawable,
-                       image_gc,
-                       aX, aY, aWidth, aHeight,
-                       NS_XPRINT_RGB_DITHER,
-                       image_bits, row_bytes);
+  xxlib_draw_xprint_scaled_rgb_image(mXlibRgbHandle,
+                                     mDrawable,
+                                     mPrintResolution, XpGetImageResolution(mPDisplay, mPContext),
+                                     image_gc,
+                                     aX, aY, aWidth, aHeight,
+                                     NS_XPRINT_RGB_DITHER,
+                                     image_bits, row_bytes);
   
   if( alpha_pixmap != None ) 
   {   
