@@ -210,8 +210,7 @@ COtherDTD::~COtherDTD(){
     delete mNodeAllocator;
     mNodeAllocator=nsnull;
   }
-  
-  NS_IF_RELEASE(mTokenizer);
+
   NS_IF_RELEASE(mSink);
 }
  
@@ -274,18 +273,14 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
   eAutoDetectResult result=eUnknownDetect;
 
   if(mEnableStrict) {
-    if(eViewSource==aParserContext.mParserCommand) {
-      if(PR_TRUE==aParserContext.mMimeType.EqualsWithConversion(kPlainTextContentType)) {
-        result=eValidDetect;
-      }
-    }
-    else {
+    if(aParserContext.mParserCommand != eViewSource) {
       if(PR_TRUE==aParserContext.mMimeType.EqualsWithConversion(kPlainTextContentType)) {
         result=eValidDetect;
       }
       else if(PR_TRUE==aParserContext.mMimeType.EqualsWithConversion(kHTMLTextContentType)) {
         switch(aParserContext.mDTDMode) {
-          case eDTDMode_strict:
+          case eDTDMode_full_standards:
+          case eDTDMode_almost_standards:
             result=ePrimaryDetect;
             break;
           default:
@@ -299,10 +294,11 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
         if(BufferContainsHTML(aBuffer,theBufHasXML)){
           result = eValidDetect ;
           if(0==aParserContext.mMimeType.Length()) {
-            aParserContext.SetMimeType(NS_ConvertASCIItoUCS2(kHTMLTextContentType));
+            aParserContext.SetMimeType(NS_LITERAL_CSTRING(kHTMLTextContentType));
             if(!theBufHasXML) {
               switch(aParserContext.mDTDMode) {
-                case eDTDMode_strict:
+                case eDTDMode_full_standards:
+                case eDTDMode_almost_standards:
                   result=ePrimaryDetect;
                   break;
                 default:
@@ -329,7 +325,9 @@ COtherDTD::CanParse(CParserContext& aParserContext, const nsString& aBuffer,
   * @param	aSink
   * @return	error code (almost always 0)
   */
-nsresult COtherDTD::WillBuildModel(  const CParserContext& aParserContext,nsIContentSink* aSink){
+nsresult COtherDTD::WillBuildModel(const CParserContext& aParserContext,
+                                   nsITokenizer* aTokenizer,
+                                   nsIContentSink* aSink){
   nsresult result=NS_OK;
 
   mFilename=aParserContext.mScanner->GetFilename();
@@ -339,6 +337,7 @@ nsresult COtherDTD::WillBuildModel(  const CParserContext& aParserContext,nsICon
   mHasOpenScript=PR_FALSE;
   mDTDMode=aParserContext.mDTDMode;
   mParserCommand=aParserContext.mParserCommand;
+  mTokenizer = aTokenizer;
 
   if((!aParserContext.mPrevContext) && (aSink)) {
 
@@ -487,10 +486,22 @@ nsresult COtherDTD::DidBuildModel(nsresult anErrorCode,PRBool aNotifySink,nsIPar
   return result; 
 }  
 
-nsresult  
-COtherDTD::Terminate(nsIParser* aParser) 
+NS_IMETHODIMP_(void)  
+COtherDTD::Terminate() 
 { 
-  return mDTDState=NS_ERROR_HTMLPARSER_STOPPARSING; 
+  mDTDState = NS_ERROR_HTMLPARSER_STOPPARSING; 
+}
+
+NS_IMETHODIMP_(PRInt32)  
+COtherDTD::GetType() 
+{ 
+  return NS_IPARSER_FLAG_HTML; 
+}
+
+NS_IMETHODIMP 
+COtherDTD::CollectSkippedContent(PRInt32 aTag, nsAString& aContent, PRInt32 &aLineNo)
+{
+  return NS_OK;
 }
 
 /** 
@@ -582,20 +593,20 @@ nsresult COtherDTD::DidHandleStartTag(nsIParserNode& aNode,eHTMLTags aChildTag){
           //we should only enable user-defined entities in debug builds...
 
         PRInt32 theCount=aNode.GetAttributeCount();
-        const nsString* theNamePtr=0;
-        const nsString* theValuePtr=0;
+        const nsAString* theNamePtr=0;
+        const nsAString* theValuePtr=0;
 
         if(theCount) {
           PRInt32 theIndex=0;
           for(theIndex=0;theIndex<theCount;theIndex++){
-            nsAutoString theKey(aNode.GetKeyAt(theIndex));
-            if(!Compare(theKey, NS_LITERAL_STRING("ENTITY"), nsCaseInsensitiveStringComparator())) {
-              const nsString& theName=aNode.GetValueAt(theIndex);
+            const nsAString& theKey = aNode.GetKeyAt(theIndex);
+            if(theKey.Equals(NS_LITERAL_STRING("ENTITY"), nsCaseInsensitiveStringComparator())) {
+              const nsAString& theName=aNode.GetValueAt(theIndex);
               theNamePtr=&theName;
             }
-            else if(!Compare(theKey, NS_LITERAL_STRING("VALUE"), nsCaseInsensitiveStringComparator())) {
+            else if(theKey.Equals(NS_LITERAL_STRING("VALUE"), nsCaseInsensitiveStringComparator())) {
               //store the named enity with the context...
-              const nsString& theValue=aNode.GetValueAt(theIndex);
+              const nsAString& theValue=aNode.GetValueAt(theIndex);
               theValuePtr=&theValue;
             }
           }
@@ -687,7 +698,7 @@ nsresult COtherDTD::HandleStartToken(CToken* aToken) {
   //Begin by gathering up attributes...  
  
   nsresult  result=NS_OK;
-  nsCParserNode* theNode=mNodeAllocator->CreateNode(aToken,mLineNumber,mTokenAllocator);
+  nsCParserNode* theNode=mNodeAllocator->CreateNode(aToken, mTokenAllocator);
   if(theNode) {
    
     eHTMLTags     theChildTag=(eHTMLTags)aToken->GetTypeID();
@@ -700,7 +711,7 @@ nsresult COtherDTD::HandleStartToken(CToken* aToken) {
       result=WillHandleStartTag(aToken,theChildTag,*theNode);
       if(NS_OK==result) {
  
-        mLineNumber += aToken->mNewlineCount;
+        mLineNumber += aToken->GetNewlineCount();
  
         PRBool theTagWasHandled=PR_FALSE; 
  
@@ -774,7 +785,7 @@ nsresult COtherDTD::HandleEndToken(CToken* aToken) {
       }
       CElement* theElement=gElementTable->mElements[theParent];
       if(theElement) { 
-        nsCParserNode* theNode=mNodeAllocator->CreateNode(aToken,mLineNumber,mTokenAllocator);
+        nsCParserNode* theNode=mNodeAllocator->CreateNode(aToken, mTokenAllocator);
         if(theNode) {
           result=theElement->HandleEndToken(theNode,theChildTag,mBodyContext,mSink);
           IF_FREE(theNode, mNodeAllocator);
@@ -869,7 +880,7 @@ nsresult COtherDTD::HandleEntityToken(CToken* aToken) {
     eHTMLTags theParent=mBodyContext->Last();
     CElement* theElement=gElementTable->mElements[theParent];
     if(theElement) {
-      nsCParserNode theNode(aToken,mLineNumber,0);
+      nsCParserNode theNode(aToken, 0);
       result=theElement->HandleStartToken(&theNode,eHTMLTag_text,mBodyContext,mSink);  
     }
   }
@@ -908,7 +919,7 @@ PRBool COtherDTD::CanContain(PRInt32 aParent,PRInt32 aChild) const {
  * become useful.
  */ 
 NS_IMETHODIMP
-COtherDTD::StringTagToIntTag(const nsAReadableString &aTag,
+COtherDTD::StringTagToIntTag(const nsAString &aTag,
                              PRInt32* aIntTag) const
 {
   *aIntTag = nsHTMLTags::LookupTag(aTag);
@@ -928,7 +939,7 @@ COtherDTD::IntTagToStringTag(PRInt32 aIntTag) const
 }  
 
 NS_IMETHODIMP
-COtherDTD::ConvertEntityToUnicode(const nsAReadableString& aEntity,
+COtherDTD::ConvertEntityToUnicode(const nsAString& aEntity,
                                   PRInt32* aUnicode) const
 {
   *aUnicode = nsHTMLEntities::EntityToUnicode(aEntity);
@@ -985,23 +996,6 @@ PRBool COtherDTD::IsInlineElement(PRInt32 aChildID,PRInt32 aParentID) const {
 PRBool COtherDTD::IsContainer(PRInt32 aTag) const {
   return gElementTable->mElements[eHTMLTags(aTag)]->IsContainer();
 }
-
- 
-/**
- * Retrieve the preferred tokenizer for use by this DTD.
- * @update  gess12/28/98
- * @param   none
- * @return  ptr to tokenizer
- */
-nsresult COtherDTD::GetTokenizer(nsITokenizer*& aTokenizer) {
-  nsresult result=NS_OK;
-  if(!mTokenizer) {
-    result=NS_NewHTMLTokenizer(&mTokenizer,mDTDMode,mDocType,mParserCommand);
-  }
-  aTokenizer=mTokenizer;
-  return result;
-}
- 
  
 /**
  * 
