@@ -38,9 +38,10 @@
  * ***** END LICENSE BLOCK ***** */
        
   
+#include "nsIAtom.h"
 #include "nsDTDUtils.h" 
 #include "CNavDTD.h" 
-#include "nsIParserNode.h" 
+#include "nsIParserNode.h"
 #include "nsParserNode.h" 
 #include "nsIChannel.h"
 #include "nsIServiceManager.h"
@@ -452,30 +453,36 @@ nsDTDContext::~nsDTDContext() {
 
 #ifdef DEBUG
 
-CNamedEntity* nsDTDContext::GetEntity(const nsString& aName)const {
-  PRInt32 theCount=mEntities.GetSize();
-  PRInt32 theIndex=0;
+CNamedEntity* nsDTDContext::GetEntity(const nsAString& aName)const {
+  if (aName.Length() > 2) {
+    nsAString::const_iterator start, end;
+    aName.BeginReading(start);
+    aName.EndReading(end);
+    
 
-  PRInt32 theLen=aName.Length();
-  PRUnichar theChar=aName.Last();
+    PRInt32 theLen=aName.Length();
+    PRUnichar theChar=aName.Last();
 
-  if(theLen>2) {
-    if(kSemicolon==theChar) {
-      theLen--;
+    // skip past leading and trailing quotes/etc
+    if(kQuote==*start) {
+      start++;
     }
-
-    const PRUnichar *theBuf=aName.get();
-    if(kQuote==theBuf[0]) {
-      theBuf++;
-      theLen--;
-    }
-    if(kQuote==theChar) {
-      theLen--;
+    
+    if(kSemicolon==theChar ||
+       kQuote == theChar) {
+      end--;
     }
+    
   
+    const nsAString& entityName = Substring(start, end);
+    
+    PRInt32 theCount=mEntities.GetSize();
+    PRInt32 theIndex=0;
     for(theIndex=0;theIndex<theCount;theIndex++) {
       CNamedEntity *theResult=(CNamedEntity*)mEntities.ObjectAt(theIndex);
-      if(theResult && theResult->mName.EqualsWithConversion(theBuf,PR_TRUE,theLen)){
+      if(theResult &&
+         theResult->mName.Equals(entityName,
+                                 nsCaseInsensitiveStringComparator())) {
         return theResult;
       }
     }
@@ -483,7 +490,7 @@ CNamedEntity* nsDTDContext::GetEntity(const nsString& aName)const {
   return 0;
 }
 
-CNamedEntity*  nsDTDContext::RegisterEntity(const nsString& aName,const nsString& aValue) {
+CNamedEntity*  nsDTDContext::RegisterEntity(const nsAString& aName,const nsAString& aValue) {
   CNamedEntity *theEntity=GetEntity(aName);
   if(!GetEntity(aName)){
     theEntity=new CNamedEntity(aName,aValue);
@@ -591,7 +598,7 @@ public:
 	  while(expn--) {
 		  ndex = ((root<=aValue) && (root)) ? (aValue/root): 0;
 		  aValue%=root;
-      aString.AppendWithConversion(aCharSet[ndex+((root>1)*offset)]);
+      aString.Append(PRUnichar(aCharSet[ndex+((root>1)*offset)]));
 		  root/=base;
     }
   }
@@ -706,8 +713,14 @@ public:
    * @param   aString -- will hold result
    */
   static void RomanString(PRInt32 aValue,nsString& aString) {
-	  static char digitsA[] = "ixcm";
-	  static char digitsB[] = "vld?";
+	  static PRUnichar digitsA[] = { PRUnichar('i'),
+                                   PRUnichar('x'),
+                                   PRUnichar('c'),
+                                   PRUnichar('m') };
+	  static PRUnichar digitsB[] = { PRUnichar('v'),
+                                   PRUnichar('l'),
+                                   PRUnichar('d'),
+                                   PRUnichar('?') };
 
     aString.Truncate();
     if(aValue<0)
@@ -726,20 +739,20 @@ public:
 		  romanPos--;
 		  switch(decStr[digitPos]) {
 			  case	'0':	break;
-			  case	'3':	aString.AppendWithConversion(digitsA[romanPos]);
-			  case	'2':	aString.AppendWithConversion(digitsA[romanPos]);
-			  case	'1':	aString.AppendWithConversion(digitsA[romanPos]);
+			  case	'3':	aString.Append(digitsA[romanPos]);
+			  case	'2':	aString.Append(digitsA[romanPos]);
+			  case	'1':	aString.Append(digitsA[romanPos]);
 				  break;
-			  case	'4':	aString.AppendWithConversion(digitsA[romanPos]);
+			  case	'4':	aString.Append(digitsA[romanPos]);
 			  case 	'5':	case	'6':
 			  case	'7':	case 	'8':	
-          aString.AppendWithConversion(digitsB[romanPos]);
+          aString.Append(digitsB[romanPos]);
 					for(n=0;n<(decStr[digitPos]-'5');n++)
-            aString.AppendWithConversion(digitsA[romanPos]);
+            aString.Append(digitsA[romanPos]);
 				  break;
 			  case	'9':	          
-          aString.AppendWithConversion(digitsA[romanPos]);
-          aString.AppendWithConversion(digitsA[romanPos]);
+          aString.Append(digitsA[romanPos]);
+          aString.Append(digitsA[romanPos]);
 					break;
 		  }
 	  }
@@ -783,7 +796,7 @@ public:
 	  modChar=aValue-(repCount*seriesLen);
 	  
     for(count=0;count<=repCount;count++) {
-      aString.AppendWithConversion(kFootnoteSet[modChar]);
+      aString.Append(PRUnichar(kFootnoteSet[modChar]));
     }
   }
 
@@ -842,10 +855,10 @@ PRInt32 nsDTDContext::IncrementCounter(eHTMLTags aTag,nsIParserNode& aNode,nsStr
   CAbacus::eNumFormat  theNumFormat=CAbacus::eDecimal;
 
   for(theIndex=0;theIndex<theCount;theIndex++){
-    nsAutoString theKey(aNode.GetKeyAt(theIndex));
-    const nsString& theValue=aNode.GetValueAt(theIndex);
+    const nsAString& theKey=aNode.GetKeyAt(theIndex);
+    const nsAString& theValue=aNode.GetValueAt(theIndex);
 
-    if(!Compare(theKey, NS_LITERAL_STRING("name"), nsCaseInsensitiveStringComparator())){
+    if(theKey.Equals(NS_LITERAL_STRING("name"), nsCaseInsensitiveStringComparator())){
       theEntity=GetEntity(theValue);
       if(!theEntity) {
         theEntity=RegisterEntity(theValue,theValue);
@@ -853,13 +866,15 @@ PRInt32 nsDTDContext::IncrementCounter(eHTMLTags aTag,nsIParserNode& aNode,nsStr
       }
       aTag=eHTMLTag_userdefined;
     }
-    else if(!Compare(theKey, NS_LITERAL_STRING("noincr"), nsCaseInsensitiveStringComparator())){
+    else if(theKey.Equals(NS_LITERAL_STRING("noincr"), nsCaseInsensitiveStringComparator())){
       theIncrValue=0;
     }
-    else if(!Compare(theKey, NS_LITERAL_STRING("format"), nsCaseInsensitiveStringComparator())){
-      PRUnichar theChar=theValue.CharAt(0);
+    else if(theKey.Equals(NS_LITERAL_STRING("format"), nsCaseInsensitiveStringComparator())){
+      nsAString::const_iterator start;
+      
+      PRUnichar theChar=*theValue.BeginReading(start);
       if('"'==theChar)
-        theChar=theValue.CharAt(1);
+        theChar=*(++start);
       switch(theChar){
         case 'A': case 'a': theNumFormat=CAbacus::eAlpha;   break;
         case 'B': case 'b': theNumFormat=CAbacus::eBinary;  break;
@@ -873,9 +888,9 @@ PRInt32 nsDTDContext::IncrementCounter(eHTMLTags aTag,nsIParserNode& aNode,nsStr
       }
       //determine numbering style
     }
-    else if(!Compare(theKey, NS_LITERAL_STRING("value"), nsCaseInsensitiveStringComparator())){
+    else if(theKey.Equals(NS_LITERAL_STRING("value"), nsCaseInsensitiveStringComparator())){
       PRInt32 err=0;
-      theNewValue=theValue.ToInteger(&err);
+      theNewValue=atoi(NS_LossyConvertUCS2toASCII(theValue).get());
       if(!err) {
 
         theIncrValue=0;
@@ -1226,7 +1241,7 @@ public:
  *
  * @return  ptr to new token (or 0).
  */
-CToken* nsTokenAllocator::CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAReadableString& aString) {
+CToken* nsTokenAllocator::CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAString& aString) {
 
   CToken* result=0;
 
@@ -1369,7 +1384,8 @@ nsNodeAllocator::~nsNodeAllocator() {
 #endif
 }
   
-nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,nsTokenAllocator* aTokenAllocator) {
+nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,  
+                                           nsTokenAllocator* aTokenAllocator) {
   nsCParserNode* result=0;
 
 #ifdef HEAP_ALLOCATED_NODES
@@ -1381,10 +1397,10 @@ nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,ns
 
   result=NS_STATIC_CAST(nsCParserNode*,mSharedNodes.Pop());
   if(result) {
-    result->Init(aToken,aLineNumber,aTokenAllocator,this);
+    result->Init(aToken, aTokenAllocator,this);
   }
   else{
-    result=nsCParserNode::Create(aToken,aLineNumber,aTokenAllocator,this);
+    result=nsCParserNode::Create(aToken, aTokenAllocator,this);
 #ifdef DEBUG_TRACK_NODES
     mCount++;
     AddNode(NS_STATIC_CAST(nsCParserNode*,result));
@@ -1392,7 +1408,7 @@ nsCParserNode* nsNodeAllocator::CreateNode(CToken* aToken,PRInt32 aLineNumber,ns
     IF_HOLD(result);
   }
 #else
-  result=nsCParserNode::Create(aToken,aLineNumber,aTokenAllocator,this);
+  result=nsCParserNode::Create(aToken, aTokenAllocator,this);
   IF_HOLD(result);
 #endif
   return result;
@@ -1489,7 +1505,7 @@ NS_IMPL_ISUPPORTS1(nsObserverEntry, nsIObserverEntry)
 nsObserverEntry::nsObserverEntry(const nsAString& aTopic) : mTopic(aTopic) 
 {
   NS_INIT_ISUPPORTS();
-  nsCRT::zero(mObservers,sizeof(mObservers));
+  memset(mObservers, 0, sizeof(mObservers));
 }
 
 nsObserverEntry::~nsObserverEntry() {
@@ -1508,7 +1524,8 @@ nsObserverEntry::~nsObserverEntry() {
 NS_IMETHODIMP
 nsObserverEntry::Notify(nsIParserNode* aNode,
                         nsIParser* aParser,
-                        nsISupports* aWebShell) 
+                        nsISupports* aWebShell,
+                        const PRUint32 aFlags) 
 {
   NS_ENSURE_ARG_POINTER(aNode);
   NS_ENSURE_ARG_POINTER(aParser);
@@ -1558,7 +1575,7 @@ nsObserverEntry::Notify(nsIParserNode* aNode,
           if (observer) {
             result = observer->Notify(aWebShell, channel,
                                       nsHTMLTags::GetStringValue(theTag),
-                                      &keys, &values);
+                                      &keys, &values, aFlags);
             if (NS_FAILED(result)) {
               break;
             }
