@@ -234,12 +234,12 @@ public:
 //used for named entities and counters (XXX debug only)
 class CNamedEntity {
 public:
-  CNamedEntity(const nsString& aName,const nsString& aValue) : mName(), mValue() {
+  CNamedEntity(const nsAString& aName,const nsAString& aValue) : mName(), mValue() {
     PRUnichar theFirst=aName.First();
     PRUnichar theLast=aName.Last();
     PRInt32   theLen=aName.Length();
     if((2<theLen) && (theFirst==theLast) && (kQuote==theFirst)) {
-      aName.Mid(mName,1,theLen-2);
+      mName = Substring(aName, 1, theLen - 2);
     } 
     else mName=aName;
 
@@ -247,7 +247,7 @@ public:
     theLast=aValue.Last();
     theLen=aValue.Length();
     if((2<theLen) && (theFirst==theLast) && (kQuote==theFirst)) {
-      aValue.Mid(mValue,1,theLen-2);
+      mValue = Substring(aValue, 1, theLen - 2);
     }
     else mValue=aValue;
 
@@ -273,7 +273,7 @@ public:
 
                   nsTokenAllocator();
   virtual         ~nsTokenAllocator();
-  virtual CToken* CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAReadableString& aString);
+  virtual CToken* CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag, const nsAString& aString);
   virtual CToken* CreateTokenOfType(eHTMLTokenTypes aType,eHTMLTags aTag);
 
   nsFixedSizeAllocator& GetArenaPool() { return mArenaPool; }
@@ -303,7 +303,7 @@ public:
   
                          nsNodeAllocator();
   virtual                ~nsNodeAllocator();
-  virtual nsCParserNode* CreateNode(CToken* aToken=nsnull,PRInt32 aLineNumber=1,nsTokenAllocator* aTokenAllocator=0);
+  virtual nsCParserNode* CreateNode(CToken* aToken=nsnull, nsTokenAllocator* aTokenAllocator=0);
 
   nsFixedSizeAllocator&  GetArenaPool() { return mNodePool; }
 
@@ -389,8 +389,8 @@ public:
   void            AllocateCounters(void);
   PRInt32         IncrementCounter(eHTMLTags aTag,nsIParserNode& aNode,nsString& aResult);
     
-  CNamedEntity*   RegisterEntity(const nsString& aName,const nsString& aValue);
-  CNamedEntity*   GetEntity(const nsString& aName)const;
+  CNamedEntity*   RegisterEntity(const nsAString& aName,const nsAString& aValue);
+  CNamedEntity*   GetEntity(const nsAString& aName) const;
 #endif
 };
 
@@ -498,7 +498,7 @@ inline PRBool BufferContainsHTML(const nsString& aBuffer,
     PRInt32 theCount = 0;
     PRInt32 theTagCount = 0;
 
-    nsAReadableString::const_iterator iter, end;
+    nsAString::const_iterator iter, end;
     aBuffer.BeginReading(iter);
     aBuffer.EndReading(end);
 
@@ -516,7 +516,7 @@ inline PRBool BufferContainsHTML(const nsString& aBuffer,
 
       ++iter; // step over the '<' character
 
-      nsAReadableString::const_iterator tag_end(iter);
+      nsAString::const_iterator tag_end(iter);
 
       aBuffer.EndReading(end);
 
@@ -571,7 +571,8 @@ public:
 
   NS_IMETHOD Notify(nsIParserNode* aNode,
                     nsIParser* aParser,
-                    nsISupports* aWebShell);
+                    nsISupports* aWebShell,
+                    const PRUint32 aFlags);
 
   nsresult   AddObserver(nsIElementObserver* aObserver,eHTMLTags aTag);
   void       RemoveObserver(nsIElementObserver* aObserver);
@@ -580,6 +581,7 @@ public:
 protected:
   nsAutoString mTopic; // This will rarely be empty, so make it an auto string
   nsVoidArray* mObservers[NS_HTML_TAG_MAX + 1];
+  friend class nsMatchesTopic;
 };
 
 /*********************************************************************************************/
@@ -645,6 +647,80 @@ inline PRBool HasOptionalEndTag(eHTMLTags aTag) {
   return FindTagInSet(aTag,gHasOptionalEndTags,sizeof(gHasOptionalEndTags)/sizeof(eHTMLTag_body));
 }
 
+static void
+InPlaceConvertLineEndings( nsAString& aString )
+{
+    // go from '\r\n' or '\r' to '\n'
+  nsAString::iterator iter;
+  aString.BeginWriting(iter);
+
+  PRUnichar* S = iter.get();
+  size_t N = iter.size_forward();
+
+    // this fragment must be the entire string because
+    //  (a) no multi-fragment string is writable, so only an illegal cast could give us one, and
+    //  (b) else we would have to do more work (watching for |to| to fall off the end)
+  NS_ASSERTION(aString.Length() == N, "You cheated... multi-fragment strings are never writable!");
+
+    // we scan/convert in two phases (but only one pass over the string)
+    // until we have to skip a character, we only need to touch end-of-line chars
+    // after that, we'll have to start moving every character we want to keep
+
+    // use array indexing instead of pointers, because compilers optimize that better
+
+
+    // this first loop just converts line endings... no characters get moved
+  size_t i = 0;
+  PRBool just_saw_cr = PR_FALSE;
+  for ( ; i < N; ++i )
+    {
+        // if it's something we need to convert...
+      if ( S[i] == '\r' )
+        {
+          S[i] = '\n';
+          just_saw_cr = PR_TRUE;
+        }
+      else
+        {
+            // else, if it's something we need to skip...
+            //   i.e., a '\n' immediately following a '\r',
+            //   then we need to start moving any character we want to keep
+            //   and we have a second loop for that, so get out of this one
+          if ( S[i] == '\n' && just_saw_cr )
+            break;
+
+          just_saw_cr = PR_FALSE;
+        }
+    }
+
+
+    // this second loop handles the rest of the buffer, moving characters down
+    //  _and_ converting line-endings as it goes
+    //  start the loop at |from = i| so that that |just_saw_cr| gets cleared automatically
+  size_t to = i;
+  for ( size_t from = i; from < N; ++from )
+    {
+        // if it's something we need to convert...
+      if ( S[from] == '\r' )
+        {
+          S[to++] = '\n';
+          just_saw_cr = PR_TRUE;
+        }
+      else
+        {
+            // else, if it's something we need to copy...
+            //  i.e., NOT a '\n' immediately following a '\r'
+          if ( S[from] != '\n' || !just_saw_cr )
+            S[to++] = S[from];
+
+          just_saw_cr = PR_FALSE;
+        }
+    }
+
+    // if we chopped characters out of the string, we need to shorten it logically
+  if ( to < N )
+    aString.SetLength(to);
+}
 #endif
 
 
