@@ -51,7 +51,8 @@
 #include "prmem.h"
 #include "nsTextFormatter.h"
 
-static const char* kDTDDirectory = "dtd/";
+static const char* kWhitespace = " \r\n\t"; // Optimized for typical cases
+static const char* kDTDDirectory = "res/dtd/";
 
 /***************************** EXPAT CALL BACKS *******************************/
 
@@ -164,61 +165,126 @@ Driver_HandleEndDoctypeDecl(void *aUserData)
   }
 }
 
-
 PR_STATIC_CALLBACK(int)
-Driver_HandleExternalEntityRef(XML_Parser parser,
+Driver_HandleExternalEntityRef(void* aExternalEntityRefHandler,
                                const XML_Char *openEntityNames,
                                const XML_Char *base,
                                const XML_Char *systemId,
                                const XML_Char *publicId)
 {
-  int result = PR_TRUE;
- 
-  // Load the external entity into a buffer
-  nsCOMPtr<nsIInputStream> in;
-  nsAutoString absURL;
-
-  nsresult rv = nsExpatDriver::OpenInputStream(publicId, systemId, base, getter_AddRefs(in), absURL);
-
-  if (NS_FAILED(rv)) {
-    return result;
+  NS_ASSERTION(aExternalEntityRefHandler, "expat driver should exist");
+  if (aExternalEntityRefHandler) {
+    return NS_STATIC_CAST(nsExpatDriver*,
+      aExternalEntityRefHandler)->HandleExternalEntityRef(
+        (const PRUnichar*)openEntityNames, (const PRUnichar*)base,
+        (const PRUnichar*)systemId, (const PRUnichar*)publicId);
   }
-
-  nsCOMPtr<nsIUnicharInputStream> uniIn;
-
-  NS_NewUTF8ConverterStream(getter_AddRefs(uniIn), in, 1024);
-  
-  if (uniIn) {
-    XML_Parser entParser = 
-      XML_ExternalEntityParserCreate(parser, 0, (const XML_Char*) NS_LITERAL_STRING("UTF-16").get());
-
-    if (entParser) {
-      PRUint32 readCount = 0;
-      PRUnichar tmpBuff[1024] = {0};
-      PRUnichar *uniBuf = tmpBuff;
-      
-      XML_SetBase(entParser, (const XML_Char*) absURL.get());
-
-      while (NS_SUCCEEDED(uniIn->Read(uniBuf, 0, 1024, &readCount))) {
-        if (readCount) {
-          // Pass the buffer to expat for parsing
-          result = XML_Parse(entParser, (char *)uniBuf,  readCount * sizeof(PRUnichar), 0);
-        }
-        else {
-          // done reading
-          result = XML_Parse(entParser, nsnull, 0, 1);
-          break;
-        }
-      }
-      XML_ParserFree(entParser);
-    }
-  }
-  
-  return result;
+  return 1;
 }
 
 /***************************** END CALL BACKS *********************************/
 
+/***************************** CATALOG UTILS **********************************/
+
+// Initially added for bug 113400 to switch from the remote "XHTML 1.0 plus
+// MathML 2.0" DTD to the the lightweight customized version that Mozilla uses.
+// Since Mozilla is not validating, no need to fetch a *huge* file at each click.
+// XXX The cleanest solution here would be to fix Bug 98413: Implement XML Catalogs 
+struct nsCatalogData {
+  const char* mPublicID;
+  const char* mLocalDTD;
+  const char* mAgentSheet;
+};
+
+// The order of this table is guestimated to be in the optimum order
+static const nsCatalogData kCatalogTable[] = {
+ {"-//W3C//DTD XHTML 1.0 Transitional//EN",    "xhtml11.dtd", nsnull },
+ {"-//W3C//DTD XHTML 1.1//EN",                 "xhtml11.dtd", nsnull },
+ {"-//W3C//DTD XHTML 1.0 Strict//EN",          "xhtml11.dtd", nsnull },
+ {"-//W3C//DTD XHTML 1.0 Frameset//EN",        "xhtml11.dtd", nsnull },
+ {"-//W3C//DTD XHTML Basic 1.0//EN",           "xhtml11.dtd", nsnull },
+ {"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN", "mathml.dtd",  "resource:/res/mathml.css" },
+ {"-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN", "mathml.dtd", "resource:/res/mathml.css" },
+ {"-//W3C//DTD SVG 20001102//EN",              "svg.dtd",     nsnull },
+ {nsnull, nsnull, nsnull}
+};
+
+static const nsCatalogData*
+LookupCatalogData(const PRUnichar* aPublicID)
+{
+  nsCAutoString publicID;
+  publicID.AssignWithConversion(aPublicID);
+
+  // linear search for now since the number of entries is going to
+  // be negligible, and the fix for bug 98413 would get rid of this
+  // code anyway
+  const nsCatalogData* data = kCatalogTable;
+  while (data->mPublicID) {
+    if (publicID.Equals(data->mPublicID)) {
+      return data;
+    }
+    ++data;
+  }
+  return nsnull;
+}
+
+// aCatalogData can be null. If not null, it provides a hook to additional
+// built-in knowledge on the resource that we are trying to load.
+// aDTD is an in/out parameter.  Returns true if the local DTD specified in the
+// catalog data exists or if the filename contained within the url exists in
+// the special DTD directory. If either of this exists, aDTD is set to the
+// file: url that points to the DTD file found in the local DTD directory AND
+// the old URI is relased.
+static PRBool
+IsLoadableDTD(const nsCatalogData* aCatalogData, nsCOMPtr<nsIURI>* aDTD)
+{
+  PRBool isLoadable = PR_FALSE;
+  nsresult res = NS_OK;
+
+  if (!aDTD || !*aDTD) {
+    NS_ASSERTION(0, "Null parameter.");
+    return PR_FALSE;
+  }
+
+  nsCAutoString fileName;
+  if (aCatalogData) {
+    // remap the DTD to a known local DTD
+    fileName.Assign(aCatalogData->mLocalDTD);
+  }
+  if (fileName.IsEmpty()) {
+    // try to see if the user has installed the DTD file -- we extract the
+    // filename.ext of the DTD here. Hence, for any DTD for which we have
+    // no predefined mapping, users just have to copy the DTD file to our
+    // special DTD directory and it will be picked
+    nsCOMPtr<nsIURL> dtdURL;
+    dtdURL = do_QueryInterface(*aDTD, &res);
+    if (NS_FAILED(res)) {
+      return PR_FALSE;
+    }
+    res = dtdURL->GetFileName(fileName);
+    if (NS_FAILED(res) || fileName.IsEmpty()) {
+      return PR_FALSE;
+    }
+  }
+  nsSpecialSystemDirectory dtdPath(nsSpecialSystemDirectory::OS_CurrentProcessDirectory);
+  dtdPath += PromiseFlatCString(nsDependentCString(kDTDDirectory) + fileName).get();
+  if (dtdPath.Exists()) {
+    // The DTD was found in the local DTD directory.
+    // Set aDTD to a file: url pointing to the local DTD
+    nsFileURL dtdFile(dtdPath);
+    nsCOMPtr<nsIURI> dtdURI;
+    NS_NewURI(getter_AddRefs(dtdURI), dtdFile.GetURLString());
+    if (dtdURI) {
+      *aDTD = dtdURI;
+      isLoadable = PR_TRUE;
+    }
+  }
+
+  return isLoadable;
+}
+
+/***************************** END CATALOG UTILS ******************************/
+
 NS_IMPL_ISUPPORTS2(nsExpatDriver,
                    nsITokenizer,
                    nsIDTD);
@@ -234,12 +300,13 @@ NS_NewExpatDriver(nsIDTD** aResult) {
 
 nsExpatDriver::nsExpatDriver()
   :mExpatParser(0), 
-   mSink(0), 
-   mInDoctype(0), 
-   mInCData(0), 
-   mBytesParsed(0),
+   mInCData(0),
+   mInDoctype(0),
    mBytePosition(0),
-   mInternalState(NS_OK)
+   mInternalState(NS_OK),
+   mBytesParsed(0),
+   mSink(0), 
+   mCatalogData(nsnull)
 {
   NS_INIT_REFCNT();
 }
@@ -306,11 +373,8 @@ nsExpatDriver::HandleComment(const PRUnichar *aValue)
   NS_ASSERTION(mSink, "content sink not found!");
 
   if (mInDoctype) {
-    // We do not want comments popping out of the doctype...
-    mDoctypeText.Append(NS_LITERAL_STRING("<!--"));
     mDoctypeText.Append(aValue);
-    mDoctypeText.Append(NS_LITERAL_STRING("-->"));
-  } 
+  }
   else if (mSink){
     mInternalState = mSink->HandleComment(aValue);
   }
@@ -377,11 +441,42 @@ nsExpatDriver::HandleEndCdataSection()
   return NS_OK;
 }
 
+/**
+ * DOCTYPE declaration is covered with very strict rules, which
+ * makes our life here simpler because the XML parser has already
+ * detected errors. The only slightly problematic case is whitespace
+ * between the tokens. There MUST be whitespace between the tokens
+ * EXCEPT right before > and [.
+ */
+static void
+GetDocTypeToken(nsString& aStr,
+                nsString& aToken,
+                PRBool aQuotedString)
+{
+  aStr.Trim(kWhitespace,PR_TRUE,PR_FALSE); // If we don't do this we must look ahead
+                                           // before Cut() and adjust the cut amount.
+  if (aQuotedString) {    
+    PRInt32 endQuote = aStr.FindChar(aStr[0],1);
+    aStr.Mid(aToken,1,endQuote-1);
+    aStr.Cut(0,endQuote+1);
+  } else {    
+    static const char* kDelimiter = " >[\r\n\t"; // Optimized for typical cases
+    PRInt32 tokenEnd = aStr.FindCharInSet(kDelimiter);
+    if (tokenEnd > 0) {
+      aStr.Left(aToken, tokenEnd);
+      aStr.Cut(0, tokenEnd);
+    }
+  }
+}
+
 nsresult 
 nsExpatDriver::HandleStartDoctypeDecl()
 {
   mInDoctype = PR_TRUE;
-  mDoctypeText.Assign(NS_LITERAL_STRING("<!DOCTYPE "));
+  // Consuming a huge DOCTYPE translates to numerous
+  // allocations. In an effort to avoid too many allocations
+  // setting mDoctypeText's capacity to be 20K ( just a guesstimate! ).
+  mDoctypeText.SetCapacity(20480);
   return NS_OK;
 }
 
@@ -389,135 +484,131 @@ nsresult
 nsExpatDriver::HandleEndDoctypeDecl() 
 {
   NS_ASSERTION(mSink, "content sink not found!");
-
+ 
   mInDoctype = PR_FALSE;
-  mDoctypeText.Append(PRUnichar('>'));
-  
+
   if(mSink) {
-    mInternalState = mSink->HandleDoctypeDecl(mDoctypeText.get());
-  }
+    // let the sink know any additional knowledge that we have about the document
+    // (currently, from bug 124570, we only expect to pass additional agent sheets
+    // needed to layout the XML vocabulary of the document)
+    nsIURI* data = nsnull;
+    if (mCatalogData && mCatalogData->mAgentSheet) {
+      NS_NewURI(&data, mCatalogData->mAgentSheet);
+    }
   
-  mDoctypeText.Truncate();
+    nsAutoString name;
+    GetDocTypeToken(mDoctypeText, name, PR_FALSE);
+
+    nsAutoString token, publicId, systemId;
+    GetDocTypeToken(mDoctypeText, token, PR_FALSE);
+    if (token.Equals(NS_LITERAL_STRING("PUBLIC"))) {
+      GetDocTypeToken(mDoctypeText, publicId, PR_TRUE);
+      GetDocTypeToken(mDoctypeText, systemId, PR_TRUE);
+    }
+    else if (token.Equals(NS_LITERAL_STRING("SYSTEM"))) {
+      GetDocTypeToken(mDoctypeText, systemId, PR_TRUE);
+    }
+
+    // The rest is the internal subset (minus whitespace)
+    mDoctypeText.Trim(kWhitespace);
+
+    mInternalState = mSink->HandleDoctypeDecl(mDoctypeText, 
+                                              name, 
+                                              systemId, 
+                                              publicId, 
+                                              data);
+    NS_IF_RELEASE(data);
+  }
+
+  mDoctypeText.SetCapacity(0);
+
   return NS_OK;
 }
 
-// Initially added for bug 113400 to switch from the remote "XHTML 1.0 plus
-// MathML 2.0" DTD to the the lightweight customized version that Mozilla uses.
-// Since Mozilla is not validating, no need to fetch a *huge* file at each click.
-// XXX The cleanest solution here would be to fix Bug 98413: Implement XML Catalogs 
-struct nsCatalogEntry {
-  const char* mPublicID;
-  const char* mLocalDTD;
-};
-
-static const nsCatalogEntry kCatalogTable[] = {
- {"-//W3C//DTD XHTML 1.1 plus MathML 2.0//EN", "mathml.dtd"},
- {"-//W3C//DTD SVG 20001102//EN", "svg.dtd"},
- {nsnull, nsnull}
-};
-
-static void
-RemapDTD(const XML_Char* aPublicID, nsAWritableCString& aLocalDTD)
+int 
+nsExpatDriver::HandleExternalEntityRef(const PRUnichar *openEntityNames,
+                                       const PRUnichar *base,
+                                       const PRUnichar *systemId,
+                                       const PRUnichar *publicId)
 {
-  nsCAutoString publicID;
-  publicID.AssignWithConversion((const PRUnichar*)aPublicID);
+  int result = 1;
 
-  // linear search for now since the number of entries is going to
-  // be negligible, and the fix for bug 98413 would get rid of this
-  // code anyway
-  aLocalDTD.Truncate();
-  const nsCatalogEntry* data = kCatalogTable;
-  while (data->mPublicID) {
-    if (publicID.Equals(data->mPublicID)) {
-      aLocalDTD = data->mLocalDTD;
-      return;
-    }
-    ++data;
-  }
-}
+  // Load the external entity into a buffer
+  nsCOMPtr<nsIInputStream> in;
+  nsAutoString absURL;
 
-// aDTD is an in/out parameter.  Returns true if the aDTD is a chrome url or if the
-// filename contained within the url exists in the special DTD directory ("dtd"
-// relative to the current process directory).  For the latter case, aDTD is set
-// to the file: url that points to the DTD file found in the local DTD directory
-// AND the old URI is relased.
-static PRBool
-IsLoadableDTD(const XML_Char* aFPIStr, nsCOMPtr<nsIURI>* aDTD)
-{
-  PRBool isLoadable = PR_FALSE;
-  nsresult res = NS_OK;
+  nsresult rv = OpenInputStream(publicId, systemId, base, getter_AddRefs(in), absURL);
 
-  if (!aDTD || !*aDTD) {
-    NS_ASSERTION(0, "Null parameter.");
-    return PR_FALSE;
+  if (NS_FAILED(rv)) {
+    return result;
   }
 
-  // Return true if the url is a chrome url
-  res = (*aDTD)->SchemeIs("chrome", &isLoadable);
+  nsCOMPtr<nsIUnicharInputStream> uniIn;
 
-  // If the url is not a chrome url, check to see if a DTD file of the same name
-  // exists in the special DTD directory
-  if (!isLoadable) {
-    // try to see if we can map the public ID to a known local DTD
-    nsXPIDLCString fileName;
-    RemapDTD(aFPIStr, fileName);
-    if (fileName.IsEmpty()) {
-      // try to see if the user has installed the DTD file -- we extract the
-      // filename.ext of the DTD here. Hence, for any DTD for which we have
-      // no predefined mapping, users just have to copy the DTD file to our
-      // special DTD directory and it will be picked
-      nsCOMPtr<nsIURL> dtdURL;
-      dtdURL = do_QueryInterface(*aDTD, &res);
-      if (NS_FAILED(res)) {
-        return PR_FALSE;
-      }
-      res = dtdURL->GetFileName(getter_Copies(fileName));
-      if (NS_FAILED(res) || fileName.IsEmpty()) {
-        return PR_FALSE;
-      }
-    }
-    nsSpecialSystemDirectory dtdPath(nsSpecialSystemDirectory::OS_CurrentProcessDirectory);
-    dtdPath += PromiseFlatCString(nsDependentCString(kDTDDirectory) + fileName).get();
-    if (dtdPath.Exists()) {
-      // The DTD was found in the local DTD directory.
-      // Set aDTD to a file: url pointing to the local DTD
-      nsFileURL dtdFile(dtdPath);
-      nsCOMPtr<nsIURI> dtdURI;
-      NS_NewURI(getter_AddRefs(dtdURI), dtdFile.GetURLString());
-      if (dtdURI) {
-        *aDTD = dtdURI;
-        isLoadable = PR_TRUE;
+  NS_NewUTF8ConverterStream(getter_AddRefs(uniIn), in, 1024);
+  
+  if (uniIn) {
+    XML_Parser entParser = 
+      XML_ExternalEntityParserCreate(mExpatParser, 0, (const XML_Char*) NS_LITERAL_STRING("UTF-16").get());
+
+    if (entParser) {
+      PRUint32 readCount = 0;
+      PRUnichar tmpBuff[1024] = {0};
+      PRUnichar *uniBuf = tmpBuff;
+
+      XML_SetBase(entParser, (const XML_Char*) absURL.get());
+
+      while (NS_SUCCEEDED(uniIn->Read(uniBuf, 0, 1024, &readCount)) && result) {
+        if (readCount) {
+          // Pass the buffer to expat for parsing
+          result = XML_Parse(entParser, (char *)uniBuf,  readCount * sizeof(PRUnichar), 0);
+        }
+        else {
+          // done reading
+          result = XML_Parse(entParser, nsnull, 0, 1);
+          break;
+        }
       }
+      XML_ParserFree(entParser);
     }
   }
-
-  return isLoadable;
+  
+  return result;
 }
 
 nsresult
-nsExpatDriver::OpenInputStream(const XML_Char* aFPIStr,
-                               const XML_Char* aURLStr, 
-                               const XML_Char* aBaseURL, 
+nsExpatDriver::OpenInputStream(const PRUnichar* aFPIStr,
+                               const PRUnichar* aURLStr, 
+                               const PRUnichar* aBaseURL, 
                                nsIInputStream** in, 
                                nsAString& aAbsURL) 
 {
   nsresult rv;
   nsCOMPtr<nsIURI> baseURI;  
-  rv = NS_NewURI(getter_AddRefs(baseURI), NS_ConvertUCS2toUTF8((const PRUnichar*)aBaseURL).get());
+  rv = NS_NewURI(getter_AddRefs(baseURI), NS_ConvertUCS2toUTF8(aBaseURL).get());
   if (NS_SUCCEEDED(rv) && baseURI) {
     nsCOMPtr<nsIURI> uri;
-    rv = NS_NewURI(getter_AddRefs(uri), NS_ConvertUCS2toUTF8((const PRUnichar*)aURLStr).get(), baseURI);
+    rv = NS_NewURI(getter_AddRefs(uri), NS_ConvertUCS2toUTF8(aURLStr).get(), baseURI);
     if (NS_SUCCEEDED(rv) && uri) {
-      if (IsLoadableDTD(aFPIStr, address_of(uri))) {
-        rv = NS_OpenURI(in, uri);
-        nsXPIDLCString absURL;
-        uri->GetSpec(getter_Copies(absURL));
-        CopyASCIItoUCS2(absURL, aAbsURL);
-      } 
-      else {
-        rv = NS_ERROR_NOT_IMPLEMENTED;
+      // check if it is alright to load this uri
+      PRBool isChrome = PR_FALSE;
+      uri->SchemeIs("chrome", &isChrome);
+      if (!isChrome) {
+        // since the url is not a chrome url, check to see if we can map the DTD
+        // to a known local DTD, or if a DTD file of the same name exists in the
+        // special DTD directory
+        if (aFPIStr) {
+          // see if the Formal Public Identifier (FPI) maps to a catalog entry
+          mCatalogData = LookupCatalogData(aFPIStr);
+        }
+        if (!IsLoadableDTD(mCatalogData, address_of(uri)))
+          return NS_ERROR_NOT_IMPLEMENTED;
       }
-    }    
+      rv = NS_OpenURI(in, uri);
+      nsCAutoString absURL;
+      uri->GetSpec(absURL);
+      aAbsURL = NS_ConvertUTF8toUCS2(absURL);
+    }
   }
   return rv;
 }
@@ -580,10 +671,15 @@ nsExpatDriver::HandleError(const char *aBuffer,
   nsParserMsgUtils::GetLocalizedStringByID(XMLPARSER_PROPERTIES, code, description);
 
   if (code == XML_ERROR_TAG_MISMATCH) {
-    description.Append(NS_LITERAL_STRING(". Expected: "));
-    description.Append(NS_LITERAL_STRING("</"));
-    description.Append((const PRUnichar*)XML_GetMismatchedTag(mExpatParser));
-    description.Append(NS_LITERAL_STRING(">."));
+    nsAutoString msg;
+    nsParserMsgUtils::GetLocalizedStringByName(XMLPARSER_PROPERTIES, "Expected", msg);
+    // . Expected: </%S>.
+    PRUnichar *message = nsTextFormatter::smprintf(msg.get(), (const PRUnichar*)XML_GetMismatchedTag(mExpatParser));
+    if (!message) {
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+    description.Append(message);
+    nsTextFormatter::smprintf_free(message);
   }
   
   nsAutoString sourceLine;
@@ -717,7 +813,6 @@ NS_IMETHODIMP
 nsExpatDriver::ConsumeToken(nsScanner& aScanner,
                             PRBool& aFlushTokens)
 {
-
   // Ask the scanner to send us all the data it has
   // scanned and pass that data to expat.
   
@@ -730,9 +825,8 @@ nsExpatDriver::ConsumeToken(nsScanner& aScanner,
   
   while (start != end) {
     PRUint32 fragLength = PRUint32(start.size_forward());
-    const PRUnichar* expatBuffer = start.get();
     
-    mInternalState = ParseBuffer((const char *)expatBuffer, 
+    mInternalState = ParseBuffer((const char *)start.get(), 
                                  fragLength * sizeof(PRUnichar), 
                                  aFlushTokens);
     
@@ -775,7 +869,7 @@ nsExpatDriver::CanParse(CParserContext& aParserContext,
     else {
       if (0 == aParserContext.mMimeType.Length() &&
           kNotFound != aBuffer.Find("<?xml ")) {
-        aParserContext.SetMimeType(NS_LITERAL_STRING(kXMLTextContentType));
+        aParserContext.SetMimeType(NS_LITERAL_CSTRING(kXMLTextContentType));
         result=eValidDetect;
       }
     }
@@ -785,7 +879,8 @@ nsExpatDriver::CanParse(CParserContext& aParserContext,
 }
 
 NS_IMETHODIMP 
-nsExpatDriver::WillBuildModel(const CParserContext& aParserContext, 
+nsExpatDriver::WillBuildModel(const CParserContext& aParserContext,
+                              nsITokenizer* aTokenizer,
                               nsIContentSink* aSink)
 {
 
@@ -809,10 +904,12 @@ nsExpatDriver::WillBuildModel(const CParserContext& aParserContext,
   XML_SetProcessingInstructionHandler(mExpatParser, Driver_HandleProcessingInstruction);
   XML_SetDefaultHandlerExpand(mExpatParser, Driver_HandleDefault);
   XML_SetExternalEntityRefHandler(mExpatParser, Driver_HandleExternalEntityRef);
+  XML_SetExternalEntityRefHandlerArg(mExpatParser, this);
   XML_SetCommentHandler(mExpatParser, Driver_HandleComment);
   XML_SetCdataSectionHandler(mExpatParser, Driver_HandleStartCdataSection,
                              Driver_HandleEndCdataSection);
 
+  XML_SetParamEntityParsing(mExpatParser, XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE);
   XML_SetDoctypeDeclHandler(mExpatParser, Driver_HandleStartDoctypeDecl, Driver_HandleEndDoctypeDecl);
 
   // Set up the user data.
@@ -843,12 +940,6 @@ nsExpatDriver::DidBuildModel(nsresult anErrorCode,
   return result;
 }
 
-NS_IMETHODIMP nsExpatDriver::GetTokenizer(nsITokenizer*& aTokenizer)
-{
-  aTokenizer = this;
-  return NS_OK;
-}
-
 NS_IMETHODIMP                     
 nsExpatDriver::WillTokenize(PRBool aIsFinalChunk,
                             nsTokenAllocator* aTokenAllocator)
@@ -880,14 +971,27 @@ nsExpatDriver::GetMostDerivedIID(void) const
   return NS_GET_IID(nsIDTD);
 }
 
-NS_IMETHODIMP  nsExpatDriver::Terminate(nsIParser* aParser)
+NS_IMETHODIMP_(void)
+nsExpatDriver::Terminate()
 {
   XML_BlockParser(mExpatParser); // XXX - not sure what happens to the unparsed data.
-  return mInternalState = NS_ERROR_HTMLPARSER_STOPPARSING;
+  mInternalState = NS_ERROR_HTMLPARSER_STOPPARSING;
+}
+
+NS_IMETHODIMP_(PRInt32)
+nsExpatDriver::GetType()
+{
+  return NS_IPARSER_FLAG_XML;
 }
 
 /*************************** Unused methods ***************************************/ 
 
+NS_IMETHODIMP 
+nsExpatDriver::CollectSkippedContent(PRInt32 aTag, nsAString& aContent, PRInt32 &aLineNo)
+{
+  return NS_OK;
+}
+
 NS_IMETHODIMP_(CToken*)
 nsExpatDriver::PushTokenFront(CToken* aToken)
 {
@@ -967,7 +1071,7 @@ nsExpatDriver::CanContain(PRInt32 aParent,PRInt32 aChild) const
 }
 
 NS_IMETHODIMP 
-nsExpatDriver::StringTagToIntTag(const nsAReadableString &aTag, PRInt32* aIntTag) const
+nsExpatDriver::StringTagToIntTag(const nsAString &aTag, PRInt32* aIntTag) const
 {
   return NS_OK;
 }
@@ -979,7 +1083,7 @@ nsExpatDriver::IntTagToStringTag(PRInt32 aIntTag) const
 }
 
 NS_IMETHODIMP 
-nsExpatDriver::ConvertEntityToUnicode(const nsAReadableString& aEntity, PRInt32* aUnicode) const
+nsExpatDriver::ConvertEntityToUnicode(const nsAString& aEntity, PRInt32* aUnicode) const
 {
   return NS_OK;
 }
