@@ -35,12 +35,14 @@
  * the terms of any one of the NPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+#include "nsIAtom.h"
 #include "nsLoggingSink.h"
 #include "nsHTMLTags.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "prprf.h"
 
+
 static NS_DEFINE_IID(kIContentSinkIID, NS_ICONTENT_SINK_IID);
 static NS_DEFINE_IID(kIHTMLContentSinkIID, NS_IHTML_CONTENT_SINK_IID);
 static NS_DEFINE_IID(kILoggingSinkIID, NS_ILOGGING_SINK_IID);
@@ -76,6 +78,7 @@ nsLoggingSink::nsLoggingSink() {
   mOutput = 0;
 	mLevel=-1;
   mSink=0;
+  mParser=0;
 }
 
 nsLoggingSink::~nsLoggingSink() { 
@@ -197,6 +200,12 @@ nsLoggingSink::SetParser(nsIParser* aParser)  {
     theResult=mSink->SetParser(aParser);
   }
   
+  NS_IF_RELEASE(mParser);
+  
+  mParser = aParser;
+  
+  NS_IF_ADDREF(mParser);
+
   return theResult;
 }
 
@@ -285,7 +294,7 @@ nsLoggingSink::AddProcessingInstruction(const nsIParserNode& aNode){
  */
 
 NS_IMETHODIMP
-nsLoggingSink::AddDocTypeDecl(const nsIParserNode& aNode, PRInt32 aMode) {
+nsLoggingSink::AddDocTypeDecl(const nsIParserNode& aNode) {
 
 #ifdef VERBOSE_DEBUG
   DebugDump("<",aNode.GetText(),(mNodeStackPos)*2);
@@ -295,7 +304,7 @@ nsLoggingSink::AddDocTypeDecl(const nsIParserNode& aNode, PRInt32 aMode) {
 
   //then proxy the call to the real sink if you have one.
   if(mSink) {
-    theResult=mSink->AddDocTypeDecl(aNode,aMode);
+    theResult=mSink->AddDocTypeDecl(aNode);
   }
   
   return theResult;
@@ -333,7 +342,7 @@ nsLoggingSink::AddComment(const nsIParserNode& aNode){
 NS_IMETHODIMP
 nsLoggingSink::SetTitle(const nsString& aValue) {
    
-  char* tmp;
+  char* tmp = nsnull;
   GetNewCString(aValue, &tmp);
 	WriteTabs(mOutput,++mLevel);
   if(tmp) {
@@ -575,8 +584,8 @@ nsLoggingSink::WriteAttributes(const nsIParserNode& aNode) {
   for (PRInt32 i = 0; i < ac; i++) {
     char* key=nsnull;
     char* value=nsnull;
-    const nsAReadableString& k = aNode.GetKeyAt(i);
-    const nsString& v = aNode.GetValueAt(i);
+    const nsAString& k = aNode.GetKeyAt(i);
+    const nsAString& v = aNode.GetValueAt(i);
 
     GetNewCString(k, &key);
     if(key) {
@@ -610,8 +619,16 @@ nsLoggingSink::WriteAttributes(const nsIParserNode& aNode) {
   }
 
   if (0 != strchr(gSkippedContentTags, aNode.GetNodeType())) {
-    char* content;
-    GetNewCString(aNode.GetSkippedContent(), &content);
+    nsCOMPtr<nsIDTD> dtd;
+    mParser->GetDTD(getter_AddRefs(dtd));
+    NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+    nsString theString;
+    PRInt32 lineNo = 0;
+
+    dtd->CollectSkippedContent(aNode.GetNodeType(), theString, lineNo);
+    char* content = nsnull;
+    GetNewCString(theString, &content);
     if(content) {
       PR_fprintf(mOutput, " <content value=\"");
       PR_fprintf(mOutput, "%s\"/>\n", content) ;
@@ -630,7 +647,14 @@ nsLoggingSink::WillWriteAttributes(const nsIParserNode& aNode)
     return PR_TRUE;
   }
   if (0 != strchr(gSkippedContentTags, aNode.GetNodeType())) {
-    const nsString& content = aNode.GetSkippedContent();
+    nsCOMPtr<nsIDTD> dtd;
+    mParser->GetDTD(getter_AddRefs(dtd));
+    NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+    
+    nsString content;
+    PRInt32 lineNo = 0;
+
+    dtd->CollectSkippedContent(aNode.GetNodeType(), content, lineNo);
     if (content.Length() > 0) {
       return PR_TRUE;
     }
@@ -697,7 +721,7 @@ nsLoggingSink::LeafNode(const nsIParserNode& aNode)
 }
 
 nsresult 
-nsLoggingSink::QuoteText(const nsAReadableString& aValue, nsString& aResult) {
+nsLoggingSink::QuoteText(const nsAString& aValue, nsString& aResult) {
   aResult.Truncate();
     /*
       if you're stepping through the string anyway, why not use iterators instead of forcing the string to copy?
@@ -734,7 +758,7 @@ nsLoggingSink::QuoteText(const nsAReadableString& aValue, nsString& aResult) {
  * @param aResult - String coverted to char*.
  */
 nsresult
-nsLoggingSink::GetNewCString(const nsAReadableString& aValue, char** aResult)
+nsLoggingSink::GetNewCString(const nsAString& aValue, char** aResult)
 {
   nsresult result=NS_OK;
   nsAutoString temp;
