@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -390,7 +390,7 @@ array_join_sub(JSContext *cx, JSObject *obj, JSString *sep, JSBool literalize,
         MAKE_BUSY(he);
     }
     sepstr = NULL;
-    seplen = sep->length;
+    seplen = JSSTRING_LENGTH(sep);
 
     v = JSVAL_NULL;
     for (index = 0; index < length; index++) {
@@ -525,7 +525,7 @@ array_join(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
     JSString *str;
 
-    if (argc == 0)
+    if (JSVAL_IS_VOID(argv[0]))
 	return array_join_sub(cx, obj, &comma, JS_FALSE, rval, JS_FALSE);
     str = js_ValueToString(cx, argv[0]);
     if (!str)
@@ -543,12 +543,26 @@ array_nyi(JSContext *cx, const char *what)
 }
 #endif
 
+static JSBool
+InitArrayElements(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
+{
+    jsuint index;
+    jsid id;
+
+    for (index = 0; index < length; index++) {
+	if (!IndexToId(cx, index, &id))
+	    return JS_FALSE;
+	if (!OBJ_SET_PROPERTY(cx, obj, id, &vector[index]))
+	    return JS_FALSE;
+    }
+    return JS_TRUE;
+}
+
 static JSBool
 InitArrayObject(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
 {
     jsval v;
     jsid id;
-    jsuint index;
 
     if (!IndexToValue(cx, length, &v))
 	return JS_FALSE;
@@ -561,17 +575,7 @@ InitArrayObject(JSContext *cx, JSObject *obj, jsuint length, jsval *vector)
     }
     if (!vector)
 	return JS_TRUE;
-    for (index = 0; index < length; index++) {
-	if (!IndexToId(cx, index, &id))
-	    return JS_FALSE;
-	if (!OBJ_DEFINE_PROPERTY(cx, obj, id, vector[index],
-                                 JS_PropertyStub, JS_PropertyStub,
-                                 JSPROP_ENUMERATE,
-                                 NULL)) {
-	    return JS_FALSE;
-	}
-    }
-    return JS_TRUE;
+    return InitArrayElements(cx, obj, length, vector);
 }
 
 static JSBool
@@ -600,12 +604,12 @@ array_reverse(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
         /* This part isn't done yet. */
 
         if (!OBJ_LOOKUP_PROPERTY(cx, obj, id, &obj2, &prop))
-                    return JS_FALSE;
-                if (!prop) {
-                    OBJ_DELETE_PROPERTY(cx, obj, id2, &v); /* v is junk. */
-                    continue;
-                }
-                OBJ_DROP_PROPERTY(cx, obj2, prop);
+            return JS_FALSE;
+        if (!prop) {
+            OBJ_DELETE_PROPERTY(cx, obj, id2, &v); /* v is junk. */
+            continue;
+        }
+        OBJ_DROP_PROPERTY(cx, obj2, prop);
 #endif
 
 	if (!OBJ_SET_PROPERTY(cx, obj, id, &v2))
@@ -618,81 +622,85 @@ array_reverse(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
     return JS_TRUE;
 }
 
-typedef struct QSortArgs {
+typedef struct HSortArgs {
     void         *vec;
     size_t       elsize;
     void         *pivot;
     JSComparator cmp;
     void         *arg;
-} QSortArgs;
+} HSortArgs;
 
 static int
 sort_compare(const void *a, const void *b, void *arg);
 
+static int
+sort_compare_strings(const void *a, const void *b, void *arg);
+
 static void
-js_qsort_r(QSortArgs *qa, int lo, int hi)
+HeapSortHelper(HSortArgs *qa, int lo, int hi)
 {
-    void *pivot, *vec, *arg, *a, *b;
+    void *pivot, *vec, *vec2, *arg, *a, *b;
     size_t elsize;
     JSComparator cmp;
     JSBool fastmove;
-    int i, j, lohi, hilo;
+    int j, hiDiv2;
 
     pivot = qa->pivot;
     vec = qa->vec;
     elsize = qa->elsize;
+    vec2 =  (char *)vec - 2 * elsize;
     cmp = qa->cmp;
     arg = qa->arg;
 
-    fastmove = (cmp == sort_compare);
+    fastmove = (cmp == sort_compare || cmp == sort_compare_strings);
 #define MEMMOVE(p,q,n) \
     (fastmove ? (void)(*(jsval*)(p) = *(jsval*)(q)) : (void)memmove(p, q, n))
 
-    while (lo < hi) {
-        i = lo;
-        j = hi;
-        a = (char *)vec + i * elsize;
+    if (lo == 1) {
+        j = 2;
+        b = (char *)vec + elsize;
+        if (j < hi && cmp((char *)vec, b, arg) < 0)
+            j++;
+        a = (char *)vec + (hi - 1) * elsize;
+        b = (char *)vec2 + j * elsize;
+        if (cmp(a, b, arg) >= 0)
+            return;
+
+        MEMMOVE(pivot, a, elsize);
+        MEMMOVE(a, b, elsize);
+        lo = j;
+    } else {
+        a = (char *)vec2 + lo * elsize;
         MEMMOVE(pivot, a, elsize);
-        while (i < j) {
-            b = (char *)vec + j * elsize;
-            if (cmp(b, pivot, arg) >= 0) {
-                j--;
-                continue;
-            }
-            MEMMOVE(a, b, elsize);
-            while (cmp(a, pivot, arg) <= 0) {
-                i++;
-                a = (char *)vec + i * elsize;
-                if (i == j)
-                    goto store_pivot;
-            }
-            MEMMOVE(b, a, elsize);
-        }
-        if (i > lo) {
-      store_pivot:
-            MEMMOVE(a, pivot, elsize);
-        }
-        if (i - lo < hi - i) {
-            lohi = i - 1;
-            if (lo < lohi)
-                js_qsort_r(qa, lo, lohi);
-            lo = i + 1;
-        } else {
-            hilo = i + 1;
-            if (hilo < hi)
-                js_qsort_r(qa, hilo, hi);
-            hi = i - 1;
-        }
     }
 
+    hiDiv2 = hi/2;
+    while (lo <= hiDiv2) {
+        j = lo + lo;
+        a = (char *)vec2 + j * elsize;
+        b = (char *)vec + (j - 1) * elsize;
+        if (j < hi && cmp(a, b, arg) < 0)
+            j++;
+        b = (char *)vec2 + j * elsize;
+        if (cmp(pivot, b, arg) >= 0)
+            break;
+
+        a = (char *)vec2 + lo * elsize;
+        MEMMOVE(a, b, elsize);
+        lo = j;
+    }
+
+    a = (char *)vec2 + lo * elsize;
+    MEMMOVE(a, pivot, elsize);
 #undef MEMMOVE
 }
 
 JSBool
-js_qsort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg)
+js_HeapSort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg) 
 {
     void *pivot;
-    QSortArgs qa;
+    HSortArgs qa;
+    int i;
 
     pivot = malloc(elsize);
     if (!pivot)
@@ -702,7 +710,12 @@ js_qsort(void *vec, size_t nel, size_t elsize, JSComparator cmp, void *arg)
     qa.pivot = pivot;
     qa.cmp = cmp;
     qa.arg = arg;
-    js_qsort_r(&qa, 0, (int)(nel - 1));
+
+    for (i = nel/2; i > 0; i--)
+        HeapSortHelper(&qa, i, (int) nel);
+    while (nel > 2)
+        HeapSortHelper(&qa, 1, (int) --nel);
+
     free(pivot);
     return JS_TRUE;
 }
@@ -764,6 +777,14 @@ sort_compare(const void *a, const void *b, void *arg)
     return (int)cmp;
 }
 
+static int
+sort_compare_strings(const void *a, const void *b, void *arg)
+{
+    jsval av = *(const jsval *)a, bv = *(const jsval *)b;
+
+    return (int) js_CompareStrings(JSVAL_TO_STRING(av), JSVAL_TO_STRING(bv));
+}
+
 /* XXXmccabe do the sort helper functions need to take int?  (Or can we claim
  * that 2^32 * 32 is too large to worry about?)  Something dumps when I change
  * to unsigned int; is qsort using -1 as a fencepost?
@@ -776,6 +797,13 @@ array_sort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     jsuint len, newlen, i;
     jsval *vec;
     jsid id;
+    size_t nbytes;
+
+    /*
+     * Optimize the default compare function case if all of obj's elements
+     * have values of type string.
+     */
+    JSBool all_strings;
 
     if (argc > 0) {
 	if (JSVAL_IS_PRIMITIVE(argv[0])) {
@@ -784,15 +812,27 @@ array_sort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	    return JS_FALSE;
 	}
 	fval = argv[0];
+        all_strings = JS_FALSE; /* non-default compare function */
     } else {
 	fval = JSVAL_NULL;
+        all_strings = JS_TRUE;  /* check for all string values */
     }
 
     if (!js_GetLengthProperty(cx, obj, &len))
 	return JS_FALSE;
     if (len == 0)
         return JS_TRUE;
-    vec = (jsval *) JS_malloc(cx, (size_t) len * sizeof(jsval));
+
+    /*
+     * Test for size_t overflow, which could lead to indexing beyond the end
+     * of the malloc'd vector.
+     */
+    nbytes = len * sizeof(jsval);
+    if (nbytes != (double) len * sizeof(jsval)) {
+        JS_ReportOutOfMemory(cx);
+        return JS_FALSE;
+    }
+    vec = (jsval *) JS_malloc(cx, nbytes);
     if (!vec)
 	return JS_FALSE;
 
@@ -816,27 +856,31 @@ array_sort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
             if (!prop) {
                 vec[i] = JSVAL_VOID;
                 continue;
-            } else {
-                newlen++;
-                OBJ_DROP_PROPERTY(cx, obj2, prop);
             }
+            OBJ_DROP_PROPERTY(cx, obj2, prop);
+            newlen++;
         }
 #endif
 	ca.status = OBJ_GET_PROPERTY(cx, obj, id, &vec[i]);
 	if (!ca.status)
 	    goto out;
+
+        /* We know JSVAL_IS_STRING yields 0 or 1, so avoid a branch via &=. */
+        all_strings &= JSVAL_IS_STRING(vec[i]); 
     }
 
     ca.context = cx;
     ca.fval = fval;
     ca.status = JS_TRUE;
-    if (!js_qsort(vec, (size_t) len, sizeof(jsval), sort_compare, &ca)) {
+    if (!js_HeapSort(vec, (size_t) len, sizeof(jsval),
+                     all_strings ? sort_compare_strings : sort_compare,
+                     &ca)) {
 	JS_ReportOutOfMemory(cx);
 	ca.status = JS_FALSE;
     }
 
     if (ca.status) {
-	ca.status = InitArrayObject(cx, obj, newlen, vec);
+	ca.status = InitArrayElements(cx, obj, newlen, vec);
 	if (ca.status)
 	    *rval = OBJECT_TO_JSVAL(obj);
 #if JS_HAS_SPARSE_ARRAYS
