@@ -99,7 +99,7 @@ JS_DHashMatchEntryStub(JSDHashTable *table,
                        const JSDHashEntryHdr *entry,
                        const void *key)
 {
-    JSDHashEntryStub *stub = (JSDHashEntryStub *)entry;
+    const JSDHashEntryStub *stub = (const JSDHashEntryStub *)entry;
 
     return stub->key == key;
 }
@@ -172,9 +172,9 @@ JS_DHashTableInit(JSDHashTable *table, JSDHashTableOps *ops, void *data,
     uint32 nbytes;
 
 #ifdef DEBUG
-    if (entrySize > 6 * sizeof(void *)) {
+    if (entrySize > 10 * sizeof(void *)) {
         fprintf(stderr,
-                "jsdhash: for the table at address 0x%p, the given entrySize"
+                "jsdhash: for the table at address %p, the given entrySize"
                 " of %lu %s favors chaining over double hashing.\n",
                 (void *)table,
                 (unsigned long) entrySize,
@@ -458,8 +458,7 @@ ChangeTable(JSDHashTable *table, int deltaLog2)
                                    oldEntry->keyHash, JS_DHASH_ADD);
             JS_ASSERT(JS_DHASH_ENTRY_IS_FREE(newEntry));
             moveEntry(table, oldEntry, newEntry);
-            newEntry->keyHash =
-                oldEntry->keyHash | (newEntry->keyHash & COLLISION_FLAG);
+            newEntry->keyHash = oldEntry->keyHash;
         }
         oldEntryAddr += entrySize;
     }
@@ -476,10 +475,11 @@ JS_DHashTableOperate(JSDHashTable *table, const void *key, JSDHashOperator op)
     uint32 size;
     int deltaLog2;
 
-    /* Avoid 0 and 1 hash codes, they indicate free and removed entries. */
     keyHash = table->ops->hashKey(table, key);
-    ENSURE_LIVE_KEYHASH(keyHash);
     keyHash *= JS_DHASH_GOLDEN_RATIO;
+
+    /* Avoid 0 and 1 hash codes, they indicate free and removed entries. */
+    ENSURE_LIVE_KEYHASH(keyHash);
     keyHash &= ~COLLISION_FLAG;
 
     switch (op) {
@@ -608,8 +608,14 @@ JS_DHashTableEnumerate(JSDHashTable *table, JSDHashEnumerator etor, void *arg)
         entryAddr += entrySize;
     }
 
-    /* Shrink or compress if a quarter or more of all entries are removed. */
-    if (table->removedCount >= capacity >> 2) {
+    /*
+     * Shrink or compress if a quarter or more of all entries are removed, or
+     * if the table is underloaded according to the configured minimum alpha,
+     * and is not minimal-size already.
+     */
+    if (table->removedCount >= capacity >> 2 ||
+        (capacity > JS_DHASH_MIN_SIZE &&
+         table->entryCount <= MIN_LOAD(table, capacity))) {
         METER(table->stats.enumShrinks++);
         capacity = table->entryCount;
         capacity += capacity >> 1;
