@@ -50,12 +50,13 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIURL.h"
+#include "nsIStandardURL.h"
 #include "nsNetUtil.h"
 #endif
 #ifndef NO_SUBSCRIPT_LOADER
 #include "mozJSSubScriptLoader.h"
 #endif
-
+#include "nsIComponentLoaderManager.h"
 // For reporting errors with the console service
 #include "nsIScriptError.h"
 #include "nsIConsoleService.h"
@@ -266,7 +267,7 @@ EvalInSandbox(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
         do_CreateInstance(kStandardURLContractID);
     if (!stdUrl ||
         NS_FAILED(stdUrl->Init(nsIStandardURL::URLTYPE_STANDARD, 80,
-                               URL8.get(), nsnull)) ||
+                               URL8, nsnull, nsnull)) ||
         !(iURL = do_QueryInterface(stdUrl))) {
         JS_ReportError(cx, "Can't create URL for evalInSandbox");
         return JS_FALSE;
@@ -390,8 +391,7 @@ BackstagePass::NewResolve(nsIXPConnectWrappedNative *wrapper,
 }
 
 mozJSComponentLoader::mozJSComponentLoader()
-    : mCompMgr(nsnull),
-      mRuntime(nsnull),
+    : mRuntime(nsnull),
       mModules(nsnull),
       mGlobals(nsnull),
       mXPCOMKey(0),
@@ -428,7 +428,8 @@ mozJSComponentLoader::~mozJSComponentLoader()
 {
     if (mInitialized) {
         mInitialized = PR_FALSE;
-        PL_HashTableEnumerateEntries(mModules, UnloadAndReleaseModules,
+        PL_HashTableEnumerateEntries(mModules, 
+                                     UnloadAndReleaseModules,
                                      mCompMgr);
         PL_HashTableDestroy(mModules);
         mModules = nsnull;
@@ -479,7 +480,11 @@ NS_IMETHODIMP
 mozJSComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
 {
     mCompMgr = aCompMgr;
+         
     nsresult rv;
+    mLoaderManager = do_QueryInterface(mCompMgr, &rv);
+    if (NS_FAILED(rv))
+        return rv;
 
     /* initialize registry handles */
     mRegistry = do_QueryInterface(aReg, &rv);
@@ -604,75 +609,29 @@ mozJSComponentLoader::SetRegistryInfo(const char *registryLocation,
                                       nsIFile *component)
 {
     nsresult rv;
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-    eRegistryLocation = (char*)registryLocation;
-
-
-    nsRegistryKey key;
-
-    rv = mRegistry->AddSubtreeRaw(mXPCOMKey, eRegistryLocation, &key);
-    if (registryLocation != eRegistryLocation)
-        nsMemory::Free(eRegistryLocation);
-
-    if (NS_FAILED(rv))
-        return rv;
-
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
+    
     PRInt64 modDate;
-
-    if (NS_FAILED(rv = component->GetLastModifiedTime(&modDate)) ||
-        NS_FAILED(rv = mRegistry->SetLongLong(key, JSlastModValueName, &modDate)))
-        return rv;
-
-    PRInt64 fileSize;
-    if (NS_FAILED(rv = component->GetFileSize(&fileSize)) ||
-        NS_FAILED(rv = mRegistry->SetLongLong(key, JSfileSizeValueName, &fileSize)))
+    rv = component->GetLastModifiedTime(&modDate);
+    if (NS_FAILED(rv))
         return rv;
 
 #ifdef DEBUG_shaver_off
     fprintf(stderr, "SetRegistryInfo(%s) => (%d,%d)\n", registryLocation,
             modDate, fileSize);
 #endif
-
-    return NS_OK;
+    return mLoaderManager->SaveFileInfo(component, registryLocation, modDate);                                                          
 }
 
 nsresult
-mozJSComponentLoader::RemoveRegistryInfo(const char *registryLocation)
+mozJSComponentLoader::RemoveRegistryInfo(nsIFile *component, const char *registryLocation)
 {
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
     nsresult rv;
-    if (!mRegistry.get())
-        return NS_OK;           // silent failure
-
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-    eRegistryLocation = (char*)registryLocation;
-
-
-    rv = mRegistry->RemoveSubtree(mXPCOMKey, eRegistryLocation);
-        
-    if (registryLocation != eRegistryLocation)
-            nsMemory::Free(eRegistryLocation);
-
-    return rv;
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
+    
+    return mLoaderManager->RemoveFileInfo(component, registryLocation);                                                          
 }
 
 
@@ -680,46 +639,16 @@ PRBool
 mozJSComponentLoader::HasChanged(const char *registryLocation,
                                  nsIFile *component)
 {
-
-    /* if we don't have a registry handle, force registration of component */
-    if (!mRegistry)
-        return PR_TRUE;
-
     nsresult rv;
-    PRUint32 length = strlen(registryLocation);
-    char* eRegistryLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)registryLocation, 1, &length, (PRUint8**)&eRegistryLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eRegistryLocation == nsnull)    //  No escaping required
-        eRegistryLocation = (char*)registryLocation;
-
-    nsRegistryKey key;
-    int r = NS_FAILED(mRegistry->GetSubtreeRaw(mXPCOMKey, eRegistryLocation, &key));
-    if (registryLocation != eRegistryLocation)
-        nsMemory::Free(eRegistryLocation);
-    if (r)
-        return PR_TRUE;
-
-    /* check modification date */
-    PRInt64 regTime, lastTime;
-    if (NS_FAILED(mRegistry->GetLongLong(key, JSlastModValueName, &regTime)))
-        return PR_TRUE;
+    if (!mLoaderManager)
+        return NS_ERROR_FAILURE;
     
-    if (NS_FAILED(component->GetLastModifiedTime(&lastTime)) || LL_NE(lastTime, regTime))
-        return PR_TRUE;
+    PRInt64 lastTime;
+    component->GetLastModifiedTime(&lastTime);
 
-    /* check file size */
-    PRInt64 regSize;
-    if (NS_FAILED(mRegistry->GetLongLong(key, JSfileSizeValueName, &regSize)))
-        return PR_TRUE;
-    PRInt64 size;
-    if (NS_FAILED(component->GetFileSize(&size)) || LL_NE(size,regSize) )
-        return PR_TRUE;
-
-    return PR_FALSE;
+    PRBool hasChanged = PR_TRUE;
+    mLoaderManager->HasFileChanged(component, registryLocation, lastTime, &hasChanged);                                                          
+    return hasChanged;
 }
 
 NS_IMETHODIMP
@@ -733,7 +662,7 @@ mozJSComponentLoader::AutoRegisterComponent(PRInt32 when,
 
     const char jsExtension[] = ".js";
     int jsExtensionLen = 3;
-    nsXPIDLCString leafName;
+    nsCAutoString leafName;
 
     *registered = PR_FALSE;
 
@@ -742,28 +671,28 @@ mozJSComponentLoader::AutoRegisterComponent(PRInt32 when,
     if (NS_FAILED(rv = component->IsFile(&isFile)) || !isFile)
         return rv;
 
-    if (NS_FAILED(rv = component->GetLeafName(getter_Copies(leafName))))
+    if (NS_FAILED(rv = component->GetNativeLeafName(leafName)))
         return rv;
-    int len = PL_strlen(leafName);
+    int len = leafName.Length();
     
     /* if it's not *.js, return now */
     if (len < jsExtensionLen || // too short
-        PL_strcasecmp(leafName + len - jsExtensionLen, jsExtension))
+        PL_strcasecmp(leafName.get() + len - jsExtensionLen, jsExtension))
         return NS_OK;
 
 #ifdef DEBUG_shaver_off
     fprintf(stderr, "mJCL: registering JS component %s\n",
-            (const char *)leafName);
+            leafName.get());
 #endif
       
     rv = AttemptRegistration(component, PR_FALSE);
 #ifdef DEBUG_shaver_off
     if (NS_SUCCEEDED(rv))
-        fprintf(stderr, "registered module %s\n", (const char *)leafName);
+        fprintf(stderr, "registered module %s\n", leafName.get());
     else if (rv == NS_ERROR_FACTORY_REGISTER_AGAIN) 
-        fprintf(stderr, "deferred module %s\n", (const char *)leafName);
+        fprintf(stderr, "deferred module %s\n", leafName.get());
     else
-        fprintf(stderr, "failed to register %s\n", (const char *)leafName);
+        fprintf(stderr, "failed to register %s\n", leafName.get());
 #endif    
     *registered = (PRBool) NS_SUCCEEDED(rv);
     return NS_OK;
@@ -780,7 +709,7 @@ mozJSComponentLoader::AutoUnregisterComponent(PRInt32 when,
 
     const char jsExtension[] = ".js";
     int jsExtensionLen = 3;
-    nsXPIDLCString leafName;
+    nsCAutoString leafName;
 
     *unregistered = PR_FALSE;
 
@@ -789,21 +718,21 @@ mozJSComponentLoader::AutoUnregisterComponent(PRInt32 when,
     if (NS_FAILED(rv = component->IsFile(&isFile)) || !isFile)
         return rv;
 
-    if (NS_FAILED(rv = component->GetLeafName(getter_Copies(leafName))))
+    if (NS_FAILED(rv = component->GetNativeLeafName(leafName)))
         return rv;
-    int len = PL_strlen(leafName);
+    int len = leafName.Length();
     
     /* if it's not *.js, return now */
     if (len < jsExtensionLen || // too short
-        PL_strcasecmp(leafName + len - jsExtensionLen, jsExtension))
+        PL_strcasecmp(leafName.get() + len - jsExtensionLen, jsExtension))
         return NS_OK;
 
     rv = UnregisterComponent(component);
 #ifdef DEBUG_dp
     if (NS_SUCCEEDED(rv))
-        fprintf(stderr, "unregistered module %s\n", (const char *)leafName);
+        fprintf(stderr, "unregistered module %s\n", leafName.get());
     else
-        fprintf(stderr, "failed to unregister %s\n", (const char *)leafName);
+        fprintf(stderr, "failed to unregister %s\n", leafName.get());
 #endif    
     *unregistered = (PRBool) NS_SUCCEEDED(rv);
     return NS_OK;
@@ -828,7 +757,7 @@ mozJSComponentLoader::AttemptRegistration(nsIFile *component,
     
     /* no need to check registry data on deferred reg */
     if (!deferred && !HasChanged(registryLocation, component))
-        goto out;
+        return NS_OK;
     
     module = ModuleForLocation(registryLocation, component);
     if (!module)
@@ -851,9 +780,7 @@ mozJSComponentLoader::AttemptRegistration(nsIFile *component,
           // get the file name
           if (component)
           {
-            nsXPIDLCString leafName;
-            component->GetLeafName(getter_Copies(leafName));
-            fileName.AssignWithConversion(leafName);
+            component->GetLeafName(fileName);
           }
           
           // this string can't come from a string bundle, because we
@@ -927,7 +854,7 @@ mozJSComponentLoader::UnregisterComponent(nsIFile *component)
     if (NS_SUCCEEDED(rv))
     {
         // Remove any autoreg specific info. Ignore error.
-        RemoveRegistryInfo(registryLocation);
+        RemoveRegistryInfo(component, registryLocation);
     }
         
     return rv;
@@ -1152,6 +1079,9 @@ mozJSComponentLoader::GlobalForLocation(const char *aLocation,
 
     nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(component);
 
+#ifdef XPCONNECT_STANDALONE
+    nsString temp;
+#endif
     char *location;             // declare before first jump to out:
     jsval retval;
     nsXPIDLCString displayPath;
@@ -1163,7 +1093,16 @@ mozJSComponentLoader::GlobalForLocation(const char *aLocation,
         goto out;
     }
 
-    NS_GetURLSpecFromFile(localFile, getter_Copies(displayPath));
+    // Quick hack to unbust XPCONNECT_STANDALONE.
+    // This leaves the jsdebugger with a non-URL pathname in the 
+    // XPCONNECT_STANDALONE case - but at least it builds and runs otherwise.
+    // See: http://bugzilla.mozilla.org/show_bug.cgi?id=121438 
+#ifdef XPCONNECT_STANDALONE
+    localFile->GetPath(temp);
+    displayPath = NS_LossyConvertUCS2toASCII(temp);
+#else
+    NS_GetURLSpecFromFile(localFile, displayPath);
+#endif
     rv = localFile->OpenANSIFileDesc("r", &fileHandle);
     if (NS_FAILED(rv)) {
         global = nsnull;
@@ -1347,7 +1286,7 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSComponentLoader);
 NS_GENERIC_FACTORY_CONSTRUCTOR(mozJSSubScriptLoader);
 #endif
 
-static nsModuleComponentInfo components[] = {
+static const nsModuleComponentInfo components[] = {
     { "JS component loader", MOZJSCOMPONENTLOADER_CID,
       mozJSComponentLoaderContractID, mozJSComponentLoaderConstructor,
       RegisterJSLoader, UnregisterJSLoader },
