@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -45,6 +45,7 @@
 #include "nsIXPCScriptable.h"
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentRegistrar.h"
 #include "jsapi.h"
 #include "jsprf.h"
 #include "nscore.h"
@@ -78,13 +79,6 @@
 
 /***************************************************************************/
 
-static void SetupRegistry()
-{
-    nsComponentManager::AutoRegister(nsIComponentManagerObsolete::NS_Startup, nsnull);
-}
-
-/***************************************************************************/
-
 #ifdef JS_THREADSAFE
 #define DoBeginRequest(cx) JS_BeginRequest((cx))
 #define DoEndRequest(cx)   JS_EndRequest((cx))
@@ -316,7 +310,7 @@ GC(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
         js_DumpGCHeap = stdout;
     }
 #endif
-    js_ForceGC(cx);
+    JS_GC(cx);
 #ifdef GC_MARK_DEBUG
     if (js_DumpGCHeap != stdout)
         fclose(js_DumpGCHeap);
@@ -833,122 +827,125 @@ main(int argc, char **argv)
 
     gErrFile = stderr;
     gOutFile = stdout;
+    {
+        nsCOMPtr<nsIServiceManager> servMan;
+        rv = NS_InitXPCOM2(getter_AddRefs(servMan), nsnull, nsnull);
+        if (NS_FAILED(rv)) {
+            printf("NS_InitXPCOM failed!\n");
+            return 1;
+        }
+        {
+            nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(servMan);
+            NS_ASSERTION(registrar, "Null nsIComponentRegistrar");
+            if (registrar)
+                registrar->AutoRegister(nsnull);
+        }
 
-    rv = NS_InitXPCOM2(NULL, NULL, NULL);
-    if (NS_FAILED(rv)) {
-        printf("NS_InitXPCOM failed!\n");
-        return 1;
-    }
-
-    SetupRegistry();
-
-    nsCOMPtr<nsIJSRuntimeService> rtsvc = do_GetService("@mozilla.org/js/xpc/RuntimeService;1");
-    // get the JSRuntime from the runtime svc
-    if (!rtsvc) {
-        printf("failed to get nsJSRuntimeService!\n");
-        return 1;
-    } 
+        nsCOMPtr<nsIJSRuntimeService> rtsvc = do_GetService("@mozilla.org/js/xpc/RuntimeService;1");
+        // get the JSRuntime from the runtime svc
+        if (!rtsvc) {
+            printf("failed to get nsJSRuntimeService!\n");
+            return 1;
+        }
     
-    if (NS_FAILED(rtsvc->GetRuntime(&rt)) || !rt) {
-        printf("failed to get JSRuntime from nsJSRuntimeService!\n");
-        return 1;
-    }
+        if (NS_FAILED(rtsvc->GetRuntime(&rt)) || !rt) {
+            printf("failed to get JSRuntime from nsJSRuntimeService!\n");
+            return 1;
+        }
 
-    jscontext = JS_NewContext(rt, 8192);
-    if (!jscontext) {
-        printf("JS_NewContext failed!\n");
-        return 1;
-    }
+        jscontext = JS_NewContext(rt, 8192);
+        if (!jscontext) {
+            printf("JS_NewContext failed!\n");
+            return 1;
+        }
 
-    JS_SetErrorReporter(jscontext, my_ErrorReporter);
+        JS_SetErrorReporter(jscontext, my_ErrorReporter);
 
-    nsCOMPtr<nsIXPConnect> xpc = do_GetService(nsIXPConnect::GetCID());
-    if (!xpc) {
-        printf("failed to get nsXPConnect service!\n");
-        return 1;
-    }
+        nsCOMPtr<nsIXPConnect> xpc = do_GetService(nsIXPConnect::GetCID());
+        if (!xpc) {
+            printf("failed to get nsXPConnect service!\n");
+            return 1;
+        }
 
-    // Since the caps security system might set a default security manager
-    // we will be sure that the secman on this context gives full trust.
-    // That way we can avoid getting principals from the caps security manager
-    // just to shut it up. Also, note that even though our secman will allow
-    // anything, we set the flags to '0' so it ought never get called anyway.
-    nsCOMPtr<nsIXPCSecurityManager> secman = 
-        NS_STATIC_CAST(nsIXPCSecurityManager*, new FullTrustSecMan());
-    xpc->SetSecurityManagerForJSContext(jscontext, secman, 0);
+        // Since the caps security system might set a default security manager
+        // we will be sure that the secman on this context gives full trust.
+        // That way we can avoid getting principals from the caps security manager
+        // just to shut it up. Also, note that even though our secman will allow
+        // anything, we set the flags to '0' so it ought never get called anyway.
+        nsCOMPtr<nsIXPCSecurityManager> secman =
+            NS_STATIC_CAST(nsIXPCSecurityManager*, new FullTrustSecMan());
+        xpc->SetSecurityManagerForJSContext(jscontext, secman, 0);
 
-//    xpc->SetCollectGarbageOnMainThreadOnly(PR_TRUE); 
-//    xpc->SetDeferReleasesUntilAfterGarbageCollection(PR_TRUE); 
+        //    xpc->SetCollectGarbageOnMainThreadOnly(PR_TRUE);
+        //    xpc->SetDeferReleasesUntilAfterGarbageCollection(PR_TRUE);
 
 #ifdef TEST_TranslateThis
-    nsCOMPtr<nsIXPCFunctionThisTranslator> 
-        translator(new nsXPCFunctionThisTranslator);
-    xpc->SetFunctionThisTranslator(NS_GET_IID(nsITestXPCFunctionCallback), translator, nsnull);
+        nsCOMPtr<nsIXPCFunctionThisTranslator>
+            translator(new nsXPCFunctionThisTranslator);
+        xpc->SetFunctionThisTranslator(NS_GET_IID(nsITestXPCFunctionCallback), translator, nsnull);
 #endif
     
-    nsCOMPtr<nsIJSContextStack> cxstack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
-    if (!cxstack) {
-        printf("failed to get the nsThreadJSContextStack service!\n");
-        return 1;
-    }
+        nsCOMPtr<nsIJSContextStack> cxstack = do_GetService("@mozilla.org/js/xpc/ContextStack;1");
+        if (!cxstack) {
+            printf("failed to get the nsThreadJSContextStack service!\n");
+            return 1;
+        }
 
-    if(NS_FAILED(cxstack->Push(jscontext))) {
-        printf("failed to push the current JSContext on the nsThreadJSContextStack!\n");
-        return 1;
-    }
+        if(NS_FAILED(cxstack->Push(jscontext))) {
+            printf("failed to push the current JSContext on the nsThreadJSContextStack!\n");
+            return 1;
+        }
 
-    glob = JS_NewObject(jscontext, &global_class, NULL, NULL);
-    if (!glob)
-        return 1;
-    if (!JS_InitStandardClasses(jscontext, glob))
-        return 1;
-    if (!JS_DefineFunctions(jscontext, glob, glob_functions))
-        return 1;
-    if (NS_FAILED(xpc->InitClasses(jscontext, glob)))
-        return 1;
+        glob = JS_NewObject(jscontext, &global_class, NULL, NULL);
+        if (!glob)
+            return 1;
+        if (!JS_InitStandardClasses(jscontext, glob))
+            return 1;
+        if (!JS_DefineFunctions(jscontext, glob, glob_functions))
+            return 1;
+        if (NS_FAILED(xpc->InitClasses(jscontext, glob)))
+            return 1;
 
-    argc--;
-    argv++;
+        argc--;
+        argv++;
 
-    result = ProcessArgs(jscontext, glob, argv, argc);
+        result = ProcessArgs(jscontext, glob, argv, argc);
 
 
 #ifdef TEST_InitClassesWithNewWrappedGlobal
-    // quick hacky test...
+        // quick hacky test...
 
-    JSContext* foo = JS_NewContext(rt, 8192);
-    nsCOMPtr<nsIXPCTestNoisy> bar(new TestGlobal());
-    nsCOMPtr<nsIXPConnectJSObjectHolder> baz;
-    xpc->InitClassesWithNewWrappedGlobal(foo, bar, NS_GET_IID(nsIXPCTestNoisy),
-                                         PR_TRUE, getter_AddRefs(baz));
-    bar = nsnull;
-    baz = nsnull;
-    js_ForceGC(foo);
-    JS_DestroyContext(foo);
+        JSContext* foo = JS_NewContext(rt, 8192);
+        nsCOMPtr<nsIXPCTestNoisy> bar(new TestGlobal());
+        nsCOMPtr<nsIXPConnectJSObjectHolder> baz;
+        xpc->InitClassesWithNewWrappedGlobal(foo, bar, NS_GET_IID(nsIXPCTestNoisy),
+                                             PR_TRUE, getter_AddRefs(baz));
+        bar = nsnull;
+        baz = nsnull;
+        JS_GC(foo);
+        JS_DestroyContext(foo);
 #endif
 
 //#define TEST_CALL_ON_WRAPPED_JS_AFTER_SHUTDOWN 1
 
 #ifdef TEST_CALL_ON_WRAPPED_JS_AFTER_SHUTDOWN
-    // test of late call and release (see below)
-    nsCOMPtr<nsIJSContextStack> bogus;
-    xpc->WrapJS(jscontext, glob, NS_GET_IID(nsIJSContextStack), 
-                (void**) getter_AddRefs(bogus));
+        // test of late call and release (see below)
+        nsCOMPtr<nsIJSContextStack> bogus;
+        xpc->WrapJS(jscontext, glob, NS_GET_IID(nsIJSContextStack),
+                    (void**) getter_AddRefs(bogus));
 #endif
 
-    JS_ClearScope(jscontext, glob);
-    js_ForceGC(jscontext);
-    JSContext *oldcx;
-    cxstack->Pop(&oldcx);
-    NS_ASSERTION(oldcx == jscontext, "JS thread context push/pop mismatch");
-    cxstack = nsnull;
-    js_ForceGC(jscontext);
-    JS_DestroyContext(jscontext);
-    xpc->SyncJSContexts();
-    xpc = nsnull;   // force nsCOMPtr to Release the service
-    secman = nsnull;
-    rtsvc = nsnull;
-
+        JS_ClearScope(jscontext, glob);
+        JS_GC(jscontext);
+        JSContext *oldcx;
+        cxstack->Pop(&oldcx);
+        NS_ASSERTION(oldcx == jscontext, "JS thread context push/pop mismatch");
+        cxstack = nsnull;
+        JS_GC(jscontext);
+        JS_DestroyContext(jscontext);
+        xpc->SyncJSContexts();
+    } // this scopes the nsCOMPtrs
+    // no nsCOMPtrs are allowed to be alive when you call NS_ShutdownXPCOM
     rv = NS_ShutdownXPCOM( NULL );
     NS_ASSERTION(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM failed");
 
@@ -964,6 +961,9 @@ main(int argc, char **argv)
 
 /***************************************************************************/
 
+/* XXXbe writeme */
+#if 0
+
 #include "jsatom.h"
 #ifdef DEBUG
 int
@@ -1005,3 +1005,5 @@ void Datom(JSAtom *atom) { if (atom) DumpAtom(&atom->entry, 0, gErrFile); }
 //void Dxpc(int depth) {Dobj(GetXPConnect(), depth);}
 JS_END_EXTERN_C
 #endif
+
+#endif
