@@ -6,7 +6,7 @@
  * the License at http://www.mozilla.org/NPL/
  *
  * Software distributed under the License is distributed on an "AS
- * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express oqr
+ * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  *
@@ -41,27 +41,19 @@
 #include "nsIScriptError.h"
 #include "nsIServiceManager.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentRegistrar.h"
 #include "nsIJSContextStack.h"
 #include "nsIJSRuntimeService.h"
 #include "nsMemory.h"
 #include "nsIXPCSecurityManager.h"
 #include "nsICategoryManager.h"
-#include "nsAWritableString.h"
+#include "nsAString.h"
 #include "nsIVariant.h"
 
 #include "jsapi.h"
-#include "jsgc.h"   // for js_ForceGC
 
 #include "xpctest.h"
 
-/***************************************************************************/
-// initialization stuff for the xpcom runtime
-
-static void SetupRegistry()
-{
-    nsComponentManager::AutoRegister(nsIComponentManagerObsolete::NS_Startup, nsnull);
-}
-
 /***************************************************************************/
 // host support for jsengine
 
@@ -712,71 +704,76 @@ int main()
 
     gErrFile = stderr;
     gOutFile = stdout;
+    {
+        nsCOMPtr<nsIServiceManager> servMan;
+        NS_InitXPCOM2(getter_AddRefs(servMan), nsnull, nsnull);
+        nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(servMan);
+        NS_ASSERTION(registrar, "Null nsIComponentRegistrar");
+        if(registrar)
+            registrar->AutoRegister(nsnull);
+    
+        // get the JSRuntime from the runtime svc, if possible
+        nsCOMPtr<nsIJSRuntimeService> rtsvc =
+                 do_GetService("@mozilla.org/js/xpc/RuntimeService;1", &rv);
+        if(NS_FAILED(rv) || NS_FAILED(rtsvc->GetRuntime(&rt)) || !rt)
+            DIE("FAILED to get a JSRuntime");
 
-    SetupRegistry();
+        jscontext = JS_NewContext(rt, 8192);
+        if(!jscontext)
+            DIE("FAILED to create a JSContext");
 
-    // get the JSRuntime from the runtime svc, if possible
-    nsCOMPtr<nsIJSRuntimeService> rtsvc = 
-             do_GetService("@mozilla.org/js/xpc/RuntimeService;1", &rv);
-    if(NS_FAILED(rv) || NS_FAILED(rtsvc->GetRuntime(&rt)) || !rt)
-        DIE("FAILED to get a JSRuntime");
+        JS_SetErrorReporter(jscontext, my_ErrorReporter);
 
-    jscontext = JS_NewContext(rt, 8192);
-    if(!jscontext) 
-        DIE("FAILED to create a JSContext");
+        nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
+        if(!xpc)
+            DIE("FAILED to get xpconnect service\n");
 
-    JS_SetErrorReporter(jscontext, my_ErrorReporter);
+        nsCOMPtr<nsIJSContextStack> cxstack =
+                 do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
+        if(NS_FAILED(rv))
+            DIE("FAILED to get the nsThreadJSContextStack service!\n");
 
-    nsCOMPtr<nsIXPConnect> xpc(do_GetService(nsIXPConnect::GetCID(), &rv));
-    if(!xpc) 
-        DIE("FAILED to get xpconnect service\n");
+        if(NS_FAILED(cxstack->Push(jscontext)))
+            DIE("FAILED to push the current jscontext on the nsThreadJSContextStack service!\n");
 
-    nsCOMPtr<nsIJSContextStack> cxstack = 
-             do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
-    if(NS_FAILED(rv)) 
-        DIE("FAILED to get the nsThreadJSContextStack service!\n");
+        // XXX I'd like to replace this with code that uses a wrapped xpcom object
+        // as the global object. The old TextXPC did this. The support for this
+        // is not working now in the new xpconnect code.
 
-    if(NS_FAILED(cxstack->Push(jscontext)))
-        DIE("FAILED to push the current jscontext on the nsThreadJSContextStack service!\n");
+        glob = JS_NewObject(jscontext, &global_class, NULL, NULL);
+        if (!glob)
+            DIE("FAILED to create global object");
+        if (!JS_InitStandardClasses(jscontext, glob))
+            DIE("FAILED to init standard classes");
+        if (!JS_DefineFunctions(jscontext, glob, glob_functions))
+            DIE("FAILED to define global functions");
+        if (NS_FAILED(xpc->InitClasses(jscontext, glob)))
+            DIE("FAILED to init xpconnect classes");
 
-    // XXX I'd like to replace this with code that uses a wrapped xpcom object
-    // as the global object. The old TextXPC did this. The support for this 
-    // is not working now in the new xpconnect code.
+        /**********************************************/
+        // run the tests...
 
-    glob = JS_NewObject(jscontext, &global_class, NULL, NULL);
-    if (!glob)
-        DIE("FAILED to create global object");
-    if (!JS_InitStandardClasses(jscontext, glob))
-        DIE("FAILED to init standard classes");
-    if (!JS_DefineFunctions(jscontext, glob, glob_functions))
-        DIE("FAILED to define global functions");
-    if (NS_FAILED(xpc->InitClasses(jscontext, glob)))
-        DIE("FAILED to init xpconnect classes");
+        TestCategoryManmager();
+        TestSecurityManager(jscontext, glob, xpc);
+        TestArgFormatter(jscontext, glob, xpc);
+        TestThreadJSContextStack(jscontext);
 
-    /**********************************************/
-    // run the tests...
+        /**********************************************/
 
-    TestCategoryManmager();
-    TestSecurityManager(jscontext, glob, xpc);
-    TestArgFormatter(jscontext, glob, xpc);
-    TestThreadJSContextStack(jscontext);
+        if(NS_FAILED(cxstack->Pop(nsnull)))
+            DIE("FAILED to pop the current jscontext from the nsThreadJSContextStack service!\n");
 
-    /**********************************************/
-
-    if(NS_FAILED(cxstack->Pop(nsnull)))
-        DIE("FAILED to pop the current jscontext from the nsThreadJSContextStack service!\n");
-
-    JS_ClearScope(jscontext, glob);
-    js_ForceGC(jscontext);
-    js_ForceGC(jscontext);
-    JS_DestroyContext(jscontext);
-    xpc->SyncJSContexts();
-    xpc->DebugDump(4);
-
-    cxstack = nsnull;   // release service held by nsCOMPtr
-    xpc     = nsnull;   // release service held by nsCOMPtr
-    rtsvc   = nsnull;   // release service held by nsCOMPtr
+        JS_ClearScope(jscontext, glob);
+        JS_GC(jscontext);
+        JS_GC(jscontext);
+        JS_DestroyContext(jscontext);
+        xpc->SyncJSContexts();
+        xpc->DebugDump(4);
 
+        cxstack = nsnull;   // release service held by nsCOMPtr
+        xpc     = nsnull;   // release service held by nsCOMPtr
+        rtsvc   = nsnull;   // release service held by nsCOMPtr
+    }
     rv = NS_ShutdownXPCOM( NULL );
     NS_ASSERTION(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM FAILED");
 
