@@ -80,7 +80,6 @@ class nsIView;
 class nsIWidget;
 class nsIDOMRange;
 class nsISelectionController;
-class nsIPercentHeightObserver;
 #ifdef ACCESSIBILITY
 class nsIAccessible;
 #endif
@@ -144,7 +143,7 @@ typedef PRUint32 nsFrameState;
 
 // If this bit is set, then there is a child frame in the frame that
 // extends outside this frame's bounding box. The implication is that
-// the frames rect does not completely cover its children and
+// the frame's rect does not completely cover its children and
 // therefore operations like rendering and hit testing (for example)
 // must operate differently.
 #define NS_FRAME_OUTSIDE_CHILDREN                     0x00000008
@@ -166,7 +165,7 @@ typedef PRUint32 nsFrameState;
 // its lifetime.
 #define NS_FRAME_HAS_LOADED_IMAGES                    0x00000080
 
-// If this bit is set, then the frame is has been moved out of the flow,
+// If this bit is set, then the frame has been moved out of the flow,
 // e.g., it is absolutely positioned or floated
 #define NS_FRAME_OUT_OF_FLOW                          0x00000100
 
@@ -210,6 +209,10 @@ typedef PRUint32 nsFrameState;
 // If this bit is set the frame has descendant with a view
 #define NS_FRAME_HAS_CHILD_WITH_VIEW                  0x00040000
 
+// If this bit is set, then reflow may be dispatched from the current
+// frame instead of the root frame.
+#define NS_FRAME_REFLOW_ROOT                          0x00080000
+
 // The lower 20 bits of the frame state word are reserved by this API.
 #define NS_FRAME_RESERVED                             0x000FFFFF
 
@@ -247,11 +250,6 @@ enum nsSpread {
   eSpreadDown   = 2
 };
 
-enum nsContextProviderRelationship {
-  eContextProvider_Ancestor = 0,
-  eContextProvider_Descendant = 1
-};
-
 // Carried out margin flags
 #define NS_CARRIED_TOP_MARGIN_IS_AUTO    0x1
 #define NS_CARRIED_BOTTOM_MARGIN_IS_AUTO 0x2
@@ -349,6 +347,19 @@ typedef PRUint32 nsReflowStatus;
   ((_completionStatus) | NS_INLINE_BREAK | NS_INLINE_BREAK_AFTER |      \
    NS_INLINE_MAKE_BREAK_TYPE(NS_STYLE_CLEAR_LINE))
 
+// The frame (not counting a continuation) did not fit in the available height and 
+// wasn't at the top of a page. If it was at the top of a page, then it is not 
+// possible to reflow it again with more height, so we don't set it in that case.
+#define NS_FRAME_TRUNCATED  0x0010
+#define NS_FRAME_IS_TRUNCATED(status) \
+  (0 != ((status) & NS_FRAME_TRUNCATED))
+#define NS_FRAME_SET_TRUNCATION(status, aReflowState, aMetrics) \
+  if (!aReflowState.mFlags.mIsTopOfPage &&                      \
+      aReflowState.availableHeight < aMetrics.height)           \
+    status |= NS_FRAME_TRUNCATED;                               \
+  else                                                          \
+    status &= ~NS_FRAME_TRUNCATED;
+
 //----------------------------------------------------------------------
 
 /**
@@ -409,6 +420,11 @@ public:
    */
   NS_IMETHOD  Destroy(nsIPresContext* aPresContext) = 0;
 
+  /*
+   * Notify the frame that it has been removed as the primary frame for its content
+   */
+  virtual void RemovedAsPrimaryFrame(nsIPresContext* aPresContext) {}
+
   /**
    * Called to set the initial list of frames. This happens after the frame
    * has been initialized.
@@ -668,7 +684,7 @@ public:
                          nsIFrame**      aFirstChild) const = 0;
 
   /**
-   * Child frames are linked together in a singly-linked
+   * Child frames are linked together in a singly-linked list
    */
   nsresult GetNextSibling(nsIFrame** aNextSibling) const {
     *aNextSibling = mNextSibling;
@@ -681,7 +697,7 @@ public:
   }
 
   /**
-   * Paint is responsible for painting the a frame. The aWhichLayer
+   * Paint is responsible for painting the frame. The aWhichLayer
    * argument indicates which layer of painting should be done during
    * the call.
    */
@@ -691,6 +707,12 @@ public:
                     nsFramePaintLayer    aWhichLayer,
                     PRUint32             aFlags = 0) = 0;
 
+  /* 
+   * Does the frame paint its background? If not, then all or part of it will be
+   * painted by ancestors.
+   */
+  virtual PRBool CanPaintBackground() { return PR_TRUE; }
+
   /**
    * Event handling of GUI events.
    *
@@ -961,6 +983,18 @@ public:
                                 nsPoint&        aOffset,
                                 nsIView**       aView) const = 0;
 
+  /**
+   * Returns the offset from this frame's upper left corner to the upper
+   * left corner of the view returned by a call to GetView(). aOffset
+   * will contain the offset to the view or (0,0) if the frame has no
+   * view. aView will contain a pointer to the view returned by GetView().
+   * aView is optional, that is, you may pass null if you are not interested
+   * in getting a pointer to the view.
+   */
+  NS_IMETHOD  GetOriginToViewOffset(nsIPresContext* aPresContext,
+                                    nsPoint&        aOffset,
+                                    nsIView**       aView) const = 0;
+
   /**
    * Returns the window that contains this frame. If this frame has a
    * view and the view has a window, then this frames window is
@@ -995,8 +1029,8 @@ public:
    *  to redraw event if aSelected == the frame selection with the last parameter.
    *  data in struct may be changed when passed in.
    *  @param aRange is the range that will dictate if the frames need to be redrawn null means the whole content needs to be redrawn
-   *  @param aSelected is it selected
-   *  @param aSpread should is spread selection to flow elements around it? or go down to its children?
+   *  @param aSelected is it selected?
+   *  @param aSpread should it spread the selection to flow elements around it? or go down to its children?
    */
   NS_IMETHOD  SetSelected(nsIPresContext* aPresContext,
                           nsIDOMRange*    aRange,
@@ -1076,38 +1110,29 @@ public:
 #endif
 
   /**
-   *  Called during appending or cancelling a reflow command to give frames notice
-   *  of reflow commands that will be targeted below them. 
-   */
-  NS_IMETHOD ReflowCommandNotify(nsIPresShell*        aShell,
-                                 nsHTMLReflowCommand* aRC,
-                                 PRBool               aCommandAdded) = 0;
-
-  /**
-   * Called in style ReResolution to get the frame that contains the style context that is the
-   * parent style context for this frame. 
-   * @param aProviderFrame: set to the frame who's style context is to be used as the parent context
-   *                        in reresolving style for this frame. Note that this may or may not be the
-   *                        parent frame of this frame.
-   * @param aRelatioship : indicates how the provider frame relates to this frame: 
-   *                       i.e. if the parent context provider frame is a child of this frame, 
-   *                       then aRelationship must be set to eContextProvider_Descendant, 
-   *                       if a parent, grandparent etc. set to eContextProvider_Ancestor 
-   *                       (this is used to control recursion in style reresolution)
+   * Get the frame whose style context should be the parent of this
+   * frame's style context (i.e., provide the parent style context).
+   * This frame must either be an ancestor of this frame or a child.  If
+   * this frame returns a child frame, then the child frame must be sure
+   * to return a grandparent or higher!
    *
-   * if return value is NS_OK then aProvdierFrame must be set, otherwise it is ignored.
-   *
-   * PRECONDITIONS: aProviderFrame cannot be null
-   * POSTCONDITIONS: *aProfiderFrame will be a valid frame if this routine succeeds
-   *                 aRelationship will be set to a legal enum value indicating the correct relationship
+   * @param aPresContext:   PresContext
+   * @param aProviderFrame: The frame whose style context should be the
+   *                        parent of this frame's style context.  Null
+   *                        is permitted, and means that this frame's
+   *                        style context should be the root of the
+   *                        style context tree.
+   * @param aIsChild:       True if |aProviderFrame| is set to a child
+   *                        of this frame; false if it is an ancestor or
+   *                        null.
    */
-  NS_IMETHOD GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                           nsIFrame** aProviderFrame, 
-                                           nsContextProviderRelationship& aRelationship) = 0;
+  NS_IMETHOD GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                        nsIFrame**      aProviderFrame,
+                                        PRBool*         aIsChild) = 0;
 
   /**
    * Determines whether a frame is visible for painting
-   * this takes into acount whether it is painting selection for printin
+   * this takes into account whether it is painting a selection or printing.
    * @param aPresContext PresContext
    * @param aRenderingContext PresContext
    * @param aCheckVis indicates whether it should check for CSS visibility, 
@@ -1153,7 +1178,7 @@ protected:
   nsIContent*      mContent;
   nsIStyleContext* mStyleContext;
   nsIFrame*        mParent;
-  nsIFrame*        mNextSibling;  // singly linked list of frames
+  nsIFrame*        mNextSibling;  // singly-linked list of frames
   nsFrameState     mState;
 
 private:
