@@ -55,6 +55,7 @@ class nsIDeviceContext;
 class nsIRenderingContext;
 class nsIPageSequenceFrame;
 class nsString;
+class nsAString;
 class nsStringArray;
 class nsICaret;
 class nsIStyleContext;
@@ -124,7 +125,8 @@ public:
   NS_IMETHOD Init(nsIDocument* aDocument,
                   nsIPresContext* aPresContext,
                   nsIViewManager* aViewManager,
-                  nsIStyleSet* aStyleSet) = 0;
+                  nsIStyleSet* aStyleSet,
+                  PRBool aIsQuirksMode) = 0;
 
   /**
    * All callers are responsible for calling |Destroy| after calling
@@ -264,28 +266,6 @@ public:
   NS_IMETHOD GetLayoutObjectFor(nsIContent*   aContent,
                                 nsISupports** aResult) const = 0;
 
-  /** 
-   * Returns the object that acts as a subshell for aContent.
-   * For example, for an html frame aResult will be an nsIDocShell.
-   */
-  NS_IMETHOD GetSubShellFor(nsIContent*   aContent,
-                            nsISupports** aResult) const = 0;
-
-  /**
-   * Establish a relationship between aContent and aSubShell.
-   * aSubShell will be returned from GetSubShellFor(aContent, ...);
-   */
-  NS_IMETHOD SetSubShellFor(nsIContent*  aContent,
-                            nsISupports* aSubShell) = 0;
-
-  /**
-   * Find the content in the map that has aSubShell set
-   * as its subshell.  If there is more than one mapping for
-   * aSubShell, the result is undefined.
-   */
-  NS_IMETHOD FindContentForShell(nsISupports* aSubShell,
-                                 nsIContent** aContent) const = 0;
-
   /**
    * Gets the placeholder frame associated with the specified frame. This is
    * a helper frame that forwards the request to the frame manager.
@@ -363,7 +343,7 @@ public:
    * Scrolls the view of the document so that the anchor with the specified
    * name is displayed at the top of the window
    */
-  NS_IMETHOD GoToAnchor(const nsString& aAnchorName) = 0;
+  NS_IMETHOD GoToAnchor(const nsAString& aAnchorName) = 0;
 
   /**
    * Scrolls the view of the document so that the frame is displayed at the 
@@ -544,19 +524,6 @@ public:
    */
   NS_IMETHOD UnsuppressPainting() = 0;
 
-  enum InterruptType {Timeout};
-  /**
-   * Notify aFrame via a reflow command when an aInterruptType event occurs 
-   */
-  NS_IMETHOD SendInterruptNotificationTo(nsIFrame*     aFrame,
-                                         InterruptType aInterruptType) = 0;
-  /**
-   * Cancel Notifications to aFrame when an aInterruptType event occurs 
-   */
-  NS_IMETHOD CancelInterruptNotificationTo(nsIFrame*     aFrame,
-                                           InterruptType aInterruptType) = 0;
-
-
   /**
    * See if reflow verification is enabled. To enable reflow verification add
    * "verifyreflow:1" to your NSPR_LOG_MODULES environment variable
