@@ -49,7 +49,7 @@
 #include "nsIFrameManager.h"
 #include "nsBidiFrames.h"
 #include "nsITextFrame.h"
-#include "nsIUBidiUtils.h"
+#include "nsBidiUtils.h"
 
 static const PRUnichar kSpace            = 0x0020;
 static const PRUnichar kLineSeparator    = 0x2028;
@@ -64,6 +64,9 @@ static const PRUnichar kPDF              = 0x202C;
 static const PRUnichar ALEF              = 0x05D0;
 #endif
 
+#define CHAR_IS_HEBREW(c) ((0x0590 <= (c)) && ((c)<= 0x05FF))
+// Note: The above code are moved from gfx/src/windows/nsRenderingContextWin.cpp
+
 extern nsresult
 NS_NewContinuingTextFrame(nsIPresShell* aPresShell, nsIFrame** aResult);
 extern nsresult
@@ -73,10 +76,9 @@ nsBidiPresUtils::nsBidiPresUtils() : mArraySize(8),
                                      mIndexMap(nsnull),
                                      mLevels(nsnull),
                                      mSuccess(NS_ERROR_FAILURE),
-                                     mBidiEngine(nsnull),
-                                     mUnicodeUtils(nsnull)
+                                     mBidiEngine(nsnull)
 {
-  mBidiEngine = do_GetService("@mozilla.org/intl/bidi;1");
+  mBidiEngine = new nsBidi();
   if (mBidiEngine) {
     mSuccess = NS_OK;
   }
@@ -90,6 +92,7 @@ nsBidiPresUtils::~nsBidiPresUtils()
   if (mIndexMap) {
     delete[] mIndexMap;
   }
+  delete mBidiEngine;
 }
 
 PRBool
@@ -148,7 +151,8 @@ nsresult
 nsBidiPresUtils::Resolve(nsIPresContext* aPresContext,
                          nsIFrame*       aBlockFrame,
                          nsIFrame*       aFirstChild,
-                         PRBool&         aForceReflow)
+                         PRBool&         aForceReflow,
+                         PRBool          aIsVisualFormControl)
 {
   aForceReflow = PR_FALSE;
   mLogicalFrames.Clear();
@@ -208,8 +212,11 @@ nsBidiPresUtils::Resolve(nsIPresContext* aPresContext,
   }
 
   PRBool isVisual;
-  aPresContext->IsVisualMode(isVisual);
-
+  if (aIsVisualFormControl) {
+    isVisual = PR_FALSE;
+  } else {
+    aPresContext->IsVisualMode(isVisual);
+  }
   mSuccess = mBidiEngine->CountRuns(&runCount);
   if (NS_FAILED(mSuccess) ) {
     return mSuccess;
@@ -525,7 +532,7 @@ nsBidiPresUtils::Reorder(nsIPresContext* aPresContext,
       return NS_ERROR_OUT_OF_MEMORY;
     }
   }
-  nsCRT::zero(mLevels, sizeof(PRUint8) * mArraySize);
+  memset(mLevels, 0, sizeof(PRUint8) * mArraySize);
 
   nsIFrame* frame;
   PRInt32   i;
@@ -542,7 +549,7 @@ nsBidiPresUtils::Reorder(nsIPresContext* aPresContext,
     mSuccess = NS_ERROR_OUT_OF_MEMORY;
   }
   else {
-    nsCRT::zero(mIndexMap, sizeof(PRUint32) * mArraySize);
+    memset(mIndexMap, 0, sizeof(PRUint32) * mArraySize);
 
     mSuccess = mBidiEngine->ReorderVisual(mLevels, count, mIndexMap);
 
@@ -820,12 +827,19 @@ nsBidiPresUtils::RemoveBidiContinuation(nsIPresContext* aPresContext,
       delete frame;
       ++aOffset;
     }
-    else if (parent != nsnull) {
-      parent->RemoveFrame(aPresContext, *presShell, nsLayoutAtoms::nextBidi,
-                          frame);
-    }
     else {
-      frame->Destroy(aPresContext);
+      nsFrameState frameState;
+      frame->GetFrameState(&frameState);
+      if (frameState & NS_FRAME_IS_BIDI) {
+        // only delete Bidi frames
+        if (parent != nsnull) {
+          parent->RemoveFrame(aPresContext, *presShell,
+                              nsLayoutAtoms::nextBidi, frame);
+        }
+        else {
+          frame->Destroy(aPresContext);
+        }
+      }
     }
   }
   if (aNextFrame) {
@@ -860,13 +874,8 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
                                    PRBool           aIsOddLevel,
                                    PRBool           aIsBidiSystem)
 {
+  NS_ASSERTION(aIsOddLevel == 0 || aIsOddLevel == 1, "aIsOddLevel should be 0 or 1");
   nsresult rv = NS_OK;
-  if (!mUnicodeUtils) {
-    mUnicodeUtils = do_GetService("@mozilla.org/intl/unicharbidiutil;1");
-    if (!mUnicodeUtils) {
-      return NS_ERROR_FAILURE;
-    }
-  }
   // ahmed 
   //adjusted for correct numeral shaping  
   PRUint32 bidiOptions;
@@ -874,11 +883,11 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
   switch (GET_BIDI_OPTION_NUMERAL(bidiOptions)) {
 
     case IBMBIDI_NUMERAL_HINDI:
-      mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
+      HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
       break;
 
     case IBMBIDI_NUMERAL_ARABIC:
-      mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
+      HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
       break;
 
     case IBMBIDI_NUMERAL_REGULAR:
@@ -886,11 +895,11 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
       switch (aCharType) {
 
         case eCharType_EuropeanNumber:
-          mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
+          HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
           break;
 
         case eCharType_ArabicNumber:
-          mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
+          HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
           break;
 
         default:
@@ -900,9 +909,9 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
       
     case IBMBIDI_NUMERAL_HINDICONTEXT:
       if ( ( (GET_BIDI_OPTION_DIRECTION(bidiOptions)==IBMBIDI_TEXTDIRECTION_RTL) && (IS_ARABIC_DIGIT (aText[0])) ) || (eCharType_ArabicNumber == aCharType) )
-        mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
+        HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_HINDI);
       else if (eCharType_EuropeanNumber == aCharType)
-        mUnicodeUtils->HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
+        HandleNumbers(aText,aTextLength,IBMBIDI_NUMERAL_ARABIC);
       break;
 
     default:
@@ -929,7 +938,7 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
     if (mBuffer.Length() < aTextLength) {
       mBuffer.SetLength(aTextLength);
     }
-    PRUnichar* buffer = (PRUnichar*)mBuffer.get();
+    PRUnichar* buffer = NS_CONST_CAST(PRUnichar*, mBuffer.get());
 
     if (doReverse) {
       rv = mBidiEngine->WriteReverse(aText, aTextLength, buffer,
@@ -940,16 +949,39 @@ nsBidiPresUtils::FormatUnicodeText(nsIPresContext*  aPresContext,
       }
     }
     if (doShape) {
-      rv = mUnicodeUtils->ArabicShaping(aText, aTextLength, buffer, (PRUint32 *)&newLen);
+      rv = ArabicShaping(aText, aTextLength, buffer, (PRUint32 *)&newLen,
+                         PR_FALSE, PR_FALSE);
       if (NS_SUCCEEDED(rv) ) {
         aTextLength = newLen;
         memcpy(aText, buffer, aTextLength * sizeof(PRUnichar) );
       }
     }
   }
+  StripBidiControlCharacters(aText, aTextLength);
   return rv;
 }
 
+void
+nsBidiPresUtils::StripBidiControlCharacters(PRUnichar* aText,
+                                            PRInt32&   aTextLength) const
+{
+  if ( (nsnull == aText) || (aTextLength < 1) ) {
+    return;
+  }
+
+  PRInt32 stripLen = 0;
+
+  for (PRInt32 i = 0; i < aTextLength; i++) {
+    if (mBidiEngine->IsBidiControl(aText[i])) {
+      ++stripLen;
+    }
+    else {
+      aText[i - stripLen] = aText[i];
+    }
+  }
+  aTextLength -= stripLen;
+}
+ 
 #if 0 // XXX: for the future use ???
 void
 RemoveDiacritics(PRUnichar* aText,
@@ -1044,12 +1076,11 @@ nsBidiPresUtils::CalculateCharType(PRInt32& aOffset,
   aOffset = offset;
 }
 
-nsresult nsBidiPresUtils::GetBidiEngine(nsIBidi** aBidiEngine)
+nsresult nsBidiPresUtils::GetBidiEngine(nsBidi** aBidiEngine)
 {
   nsresult rv = NS_ERROR_FAILURE;
   if (mBidiEngine) {
     *aBidiEngine = mBidiEngine;
-    NS_ADDREF(*aBidiEngine);
     rv = NS_OK;
   }
   return rv; 
@@ -1081,8 +1112,6 @@ nsresult nsBidiPresUtils::RenderText(PRUnichar*           aText,
   PRUint8 charType;
   PRUint8 prevType = eCharType_LeftToRight;
   nsBidiLevel level;
-  PRInt32 lineOffset     = 0;
-  PRInt32 runLength      = 0;
 
   PRUint32 hints = 0;
   aRenderingContext.GetHints(hints);
@@ -1097,26 +1126,38 @@ nsresult nsBidiPresUtils::RenderText(PRUnichar*           aText,
     if (NS_FAILED(rv))
       return rv;
 
-    runLength = limit - start;
-    lineOffset = start;
+    PRInt32 subRunLength = limit - start;
+    PRInt32 lineOffset = start;
     PRInt32 typeLimit = PR_MIN(limit, aLength);
-    CalculateCharType(lineOffset, typeLimit, limit, runLength, runCount, charType, prevType);
+    PRInt32 subRunCount = 1;
+    PRInt32 subRunLimit = typeLimit;
 
-    if (eCharType_RightToLeftArabic == charType) {
-      isBidiSystem = (hints & NS_RENDERING_HINT_ARABIC_SHAPING);
-    }
-    if (isBidiSystem && (CHARTYPE_IS_RTL(charType) ^ isRTL) ) {
-      // set reading order into DC
-      isRTL = !isRTL;
-      aRenderingContext.SetRightToLeftText(isRTL);
-    }
-    FormatUnicodeText(aPresContext, aText + start, length,
-                      (nsCharType)charType, level & 1,
-                      isBidiSystem);
+    while (subRunCount > 0) {
+      // CalculateCharType can increment subRunCount if the run
+      // contains mixed character types
+      CalculateCharType(lineOffset, typeLimit, subRunLimit, subRunLength, subRunCount, charType, prevType);
 
-    aRenderingContext.GetWidth(aText + start, length, width, nsnull);
-    aRenderingContext.DrawString(aText + start, length, aX, aY, width);
-    aX += width;
+      if (eCharType_RightToLeftArabic == charType) {
+        isBidiSystem = (hints & NS_RENDERING_HINT_ARABIC_SHAPING);
+      }
+      if (isBidiSystem && (CHARTYPE_IS_RTL(charType) ^ isRTL) ) {
+        // set reading order into DC
+        isRTL = !isRTL;
+        aRenderingContext.SetRightToLeftText(isRTL);
+      }
+      FormatUnicodeText(aPresContext, aText + start, subRunLength,
+                        (nsCharType)charType, level & 1,
+                        isBidiSystem);
+
+      aRenderingContext.GetWidth(aText + start, subRunLength, width, nsnull);
+      aRenderingContext.DrawString(aText + start, subRunLength, aX, aY, width);
+      aX += width;
+
+      --subRunCount;
+      start = lineOffset;
+      subRunLimit = typeLimit;
+      subRunLength = typeLimit - lineOffset;
+    } // while
   } // for
 
   // Restore original reading order
@@ -1126,4 +1167,44 @@ nsresult nsBidiPresUtils::RenderText(PRUnichar*           aText,
   return NS_OK;
 }
   
+nsresult
+nsBidiPresUtils::ReorderUnicodeText(PRUnichar*       aText,
+                                    PRInt32&         aTextLength,
+                                    nsCharType       aCharType,
+                                    PRBool           aIsOddLevel,
+                                    PRBool           aIsBidiSystem)
+{
+  NS_ASSERTION(aIsOddLevel == 0 || aIsOddLevel == 1, "aIsOddLevel should be 0 or 1");
+  nsresult rv = NS_OK;
+  PRBool doReverse = PR_FALSE;
+
+  if (aIsBidiSystem) {
+    if ( (CHARTYPE_IS_RTL(aCharType)) ^ (aIsOddLevel) )
+      doReverse = PR_TRUE;
+  }
+  else {
+    if (aIsOddLevel)
+      doReverse = PR_TRUE;
+  }
+
+  if (doReverse) {
+    PRInt32    newLen;
+
+    if (mBuffer.Length() < aTextLength) {
+      mBuffer.SetLength(aTextLength);
+    }
+    PRUnichar* buffer = NS_CONST_CAST(PRUnichar*, mBuffer.get());
+
+    if (doReverse) {
+      rv = mBidiEngine->WriteReverse(aText, aTextLength, buffer,
+                                     NSBIDI_DO_MIRRORING, &newLen);
+      if (NS_SUCCEEDED(rv) ) {
+        aTextLength = newLen;
+        memcpy(aText, buffer, aTextLength * sizeof(PRUnichar) );
+      }
+    }
+  }
+  return rv;
+}
+
 #endif // IBMBIDI
