@@ -57,8 +57,6 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIServiceManager.h"
-#include "nsBoxLayoutState.h"
-#include "nsIBox.h"
 #include "nsIDOMElement.h"
 #include "nsContentPolicyUtils.h"
 #include "nsIScriptGlobalObject.h"
@@ -67,6 +65,9 @@
 #include "nsXPIDLString.h"
 
 #include "prprf.h"
+#include "nsContentPolicyUtils.h"
+#include "nsIScriptGlobalObject.h"
+#include "nsIDOMDocument.h"
 #ifdef IBMBIDI
 #include "nsBidiPresUtils.h"
 #endif // IBMBIDI
@@ -149,12 +150,12 @@ nsPresContext::nsPresContext()
 {
   NS_INIT_REFCNT();
   mCompatibilityMode = eCompatibility_Standard;
-  mCompatibilityLocked = PR_FALSE;
   mWidgetRenderingMode = eWidgetRendering_Gfx; 
   mImageAnimationMode = imgIContainer::kNormalAnimMode;
   mImageAnimationModePref = imgIContainer::kNormalAnimMode;
 
-  SetBackgroundDraw(PR_TRUE);		// always draw the background
+  SetBackgroundImageDraw(PR_TRUE);		// always draw the background
+  SetBackgroundColorDraw(PR_TRUE);
 
   mStopped = PR_FALSE;
   mStopChrome = PR_TRUE;
@@ -452,27 +453,6 @@ nsPresContext::GetUserPreferences()
     mFontScaler = prefInt;
   }
 
-  if (NS_SUCCEEDED(mPrefs->GetIntPref("nglayout.compatibility.mode", &prefInt))) {
-    // XXX this should really be a state on the webshell instead of using prefs
-    switch (prefInt) {
-      case 1: 
-        mCompatibilityLocked = PR_TRUE;  
-        mCompatibilityMode = eCompatibility_Standard;   
-        break;
-      case 2: 
-        mCompatibilityLocked = PR_TRUE;  
-        mCompatibilityMode = eCompatibility_NavQuirks;  
-        break;
-      case 0:   // auto
-      default:
-        mCompatibilityLocked = PR_FALSE;  
-        break;
-    }
-  }
-  else {
-    mCompatibilityLocked = PR_FALSE;  // auto
-  }
-
   if (NS_SUCCEEDED(mPrefs->GetIntPref("nglayout.widget.mode", &prefInt))) {
     mWidgetRenderingMode = (enum nsWidgetRendering)prefInt;  // bad cast
   }
@@ -605,30 +585,11 @@ nsPresContext::ClearStyleDataAndReflow()
     mShell->GetStyleSet(getter_AddRefs(set));
     set->ClearStyleData(this, nsnull, nsnull);
 
-    // Have the root frame's style context remap its style based on the
-    // user preferences
-    nsIFrame* rootFrame;
-
-    mShell->GetRootFrame(&rootFrame);
-    if (rootFrame) {
-      // boxes know how to coelesce style changes. So if our root frame is a box
-      // then tell it to handle it. If its a block we are stuck with a full top to bottom
-      // reflow. -EDV
-      nsIFrame* child = nsnull;
-      rootFrame->FirstChild(this, nsnull, &child);
-      nsresult rv;
-      nsCOMPtr<nsIBox> box = do_QueryInterface(child, &rv);
-      if (NS_SUCCEEDED(rv) && box) {
-        nsBoxLayoutState state(this);
-        box->MarkStyleChange(state);
-      } else {
-        // Force a reflow of the root frame
-        // XXX We really should only do a reflow if a preference that affects
-        // formatting changed, e.g., a font change. If it's just a color change
-        // then we only need to repaint...
-        mShell->StyleChangeReflow();
-      }
-    }
+    // Force a reflow of the root frame
+    // XXX We really should only do a reflow if a preference that affects
+    // formatting changed, e.g., a font change. If it's just a color change
+    // then we only need to repaint...
+    mShell->StyleChangeReflow();
   }
 
   return NS_OK;
@@ -800,18 +761,15 @@ nsPresContext::GetCompatibilityMode(nsCompatibility* aResult)
 NS_IMETHODIMP
 nsPresContext::SetCompatibilityMode(nsCompatibility aMode)
 {
-  if (! mCompatibilityLocked) {
-    mCompatibilityMode = aMode;
-  }
+  mCompatibilityMode = aMode;
+
+  NS_ENSURE_TRUE(mShell, NS_OK);
 
   // enable/disable the QuirkSheet
-  NS_ASSERTION(mShell, "PresShell must be set on PresContext before calling nsPresContext::SetCompatibilityMode");
-  if (mShell) {
-    nsCOMPtr<nsIStyleSet> set;
-    nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
-    if (NS_SUCCEEDED(rv) && set) {
-      set->EnableQuirkStyleSheet((mCompatibilityMode != eCompatibility_Standard) ? PR_TRUE : PR_FALSE);
-    }
+  nsCOMPtr<nsIStyleSet> set;
+  mShell->GetStyleSet(getter_AddRefs(set));
+  if (set) {
+    set->EnableQuirkStyleSheet(mCompatibilityMode != eCompatibility_Standard);
   }
   return NS_OK;
 }
@@ -956,7 +914,6 @@ nsPresContext::GetBaseURL(nsIURI** aResult)
 NS_IMETHODIMP
 nsPresContext::ResolveStyleContextFor(nsIContent* aContent,
                                       nsIStyleContext* aParentContext,
-                                      PRBool aForceUnique,
                                       nsIStyleContext** aResult)
 {
   NS_PRECONDITION(aResult, "null out param");
@@ -966,8 +923,7 @@ nsPresContext::ResolveStyleContextFor(nsIContent* aContent,
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
   if (NS_SUCCEEDED(rv)) {
     if (set) {
-      result = set->ResolveStyleFor(this, aContent, aParentContext,
-                                    aForceUnique);
+      result = set->ResolveStyleFor(this, aContent, aParentContext);
       if (nsnull == result) {
         rv = NS_ERROR_OUT_OF_MEMORY;
       }
@@ -980,7 +936,6 @@ nsPresContext::ResolveStyleContextFor(nsIContent* aContent,
 NS_IMETHODIMP
 nsPresContext::ResolveStyleContextForNonElement(
                                       nsIStyleContext* aParentContext,
-                                      PRBool aForceUnique,
                                       nsIStyleContext** aResult)
 {
   NS_PRECONDITION(aResult, "null out param");
@@ -989,8 +944,7 @@ nsPresContext::ResolveStyleContextForNonElement(
   nsCOMPtr<nsIStyleSet> set;
   nsresult rv = mShell->GetStyleSet(getter_AddRefs(set));
   if (NS_SUCCEEDED(rv) && set) {
-    result = set->ResolveStyleForNonElement(this, aParentContext,
-                                            aForceUnique);
+    result = set->ResolveStyleForNonElement(this, aParentContext);
     if (!result)
       rv = NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1002,18 +956,16 @@ NS_IMETHODIMP
 nsPresContext::ResolvePseudoStyleContextFor(nsIContent* aParentContent,
                                             nsIAtom* aPseudoTag,
                                             nsIStyleContext* aParentContext,
-                                            PRBool aForceUnique,
                                             nsIStyleContext** aResult)
 {
   return ResolvePseudoStyleWithComparator(aParentContent, aPseudoTag, aParentContext,
-                                          aForceUnique, nsnull, aResult);
+                                          nsnull, aResult);
 }
 
 NS_IMETHODIMP
 nsPresContext::ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
                                                 nsIAtom* aPseudoTag,
                                                 nsIStyleContext* aParentContext,
-                                                PRBool aForceUnique,
                                                 nsICSSPseudoComparator* aComparator,
                                                 nsIStyleContext** aResult)
 {
@@ -1025,7 +977,7 @@ nsPresContext::ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
   if (NS_SUCCEEDED(rv)) {
     if (set) {
       result = set->ResolvePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                          aParentContext, aForceUnique, aComparator);
+                                          aParentContext, aComparator);
       if (nsnull == result) {
         rv = NS_ERROR_OUT_OF_MEMORY;
       }
@@ -1039,7 +991,6 @@ NS_IMETHODIMP
 nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
                                           nsIAtom* aPseudoTag,
                                           nsIStyleContext* aParentContext,
-                                          PRBool aForceUnique,
                                           nsIStyleContext** aResult)
 {
   NS_PRECONDITION(aResult, "null out param");
@@ -1050,7 +1001,7 @@ nsPresContext::ProbePseudoStyleContextFor(nsIContent* aParentContent,
   if (NS_SUCCEEDED(rv)) {
     if (set) {
       result = set->ProbePseudoStyleFor(this, aParentContent, aPseudoTag,
-                                        aParentContext, aForceUnique);
+                                        aParentContext);
     }
   }
   *aResult = result;
@@ -1473,10 +1424,52 @@ nsPresContext::LoadImage(const nsString& aURL,
 {
   // look and see if we have a loader for the target frame.
 
+  nsresult rv;
+
   nsVoidKey key(aTargetFrame);
   nsImageLoader *loader = NS_REINTERPRET_CAST(nsImageLoader*, mImageLoaders.Get(&key)); // addrefs
 
+  nsCOMPtr<nsIDocument> doc;
+  rv = mShell->GetDocument(getter_AddRefs(doc));
+  if (NS_FAILED(rv)) return rv;
+
+  nsCOMPtr<nsIURI> baseURI;
+  doc->GetBaseURL(*getter_AddRefs(baseURI));
+
+  nsCOMPtr<nsIURI> uri;
+  NS_NewURI(getter_AddRefs(uri), aURL, nsnull, baseURI);
+
   if (!loader) {
+    nsCOMPtr<nsIContent> content;
+    aTargetFrame->GetContent(getter_AddRefs(content));
+
+    // Check with the content-policy things to make sure this load is permitted.
+    nsresult rv;
+    nsCOMPtr<nsIDOMElement> element(do_QueryInterface(content));
+
+    if (content && element) {
+      nsCOMPtr<nsIDocument> document;
+      rv = content->GetDocument(*getter_AddRefs(document));
+
+      // If there is no document, skip the policy check
+      // XXXldb This really means the document is being destroyed, so
+      // perhaps we're better off skipping the load entirely.
+      if (document) {
+        nsCOMPtr<nsIScriptGlobalObject> globalScript;
+        rv = document->GetScriptGlobalObject(getter_AddRefs(globalScript));
+
+        if (globalScript) {
+          nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(globalScript));
+
+          PRBool shouldLoad = PR_TRUE;
+          rv = NS_CheckContentLoadPolicy(nsIContentPolicy::IMAGE,
+                                         uri, element, domWin, &shouldLoad);
+          if (NS_SUCCEEDED(rv) && !shouldLoad)
+            return NS_ERROR_FAILURE;
+        }
+      }
+    }
+
     nsImageLoader *newLoader = new nsImageLoader();
     if (!newLoader)
       return NS_ERROR_OUT_OF_MEMORY;
@@ -1502,7 +1495,7 @@ nsPresContext::LoadImage(const nsString& aURL,
     aTargetFrame->SetFrameState(state);
   }
 
-  loader->Load(aURL);
+  loader->Load(uri);
 
   loader->GetRequest(aRequest);
 
@@ -1711,7 +1704,7 @@ nsPresContext::GetIsBidiSystem(PRBool& aResult) const
 }
 
 NS_IMETHODIMP
-nsPresContext::GetBidiCharset(nsAWritableString &aCharSet)
+nsPresContext::GetBidiCharset(nsAString &aCharSet)
 {
   aCharSet = mCharset;
   return NS_OK;
@@ -1779,9 +1772,31 @@ nsPresContext::ThemeChanged()
   if (!mShell)
     return NS_OK;
 
-  return mShell->ReconstructStyleData(PR_TRUE);
+  return mShell->ReconstructStyleData(PR_FALSE);
 }
 
+NS_IMETHODIMP
+nsPresContext::SysColorChanged()
+{
+  if (mLookAndFeel) {
+     // Don't use the cached values for the system colors
+    mLookAndFeel->LookAndFeelChanged();
+  }
+   
+  // Reset default background and foreground colors for the document since
+  // they may be using system colors
+  GetDocumentColorPreferences();
+
+  // Clear out all of the style data since it may contain RGB values
+  // which originated from system colors.
+  if (mShell) {
+    // Clear out all our style data.
+    nsCOMPtr<nsIStyleSet> set;
+    mShell->GetStyleSet(getter_AddRefs(set));
+    set->ClearStyleData(this, nsnull, nsnull);
+  }
+  return NS_OK;
+}
 
 #ifdef MOZ_REFLOW_PERF
 NS_IMETHODIMP
