@@ -51,7 +51,7 @@
 #include "nsIObserver.h"
 #include "nsILookAndFeel.h"
 #ifdef IBMBIDI
-#include "nsIUBidiUtils.h"
+#include "nsBidiUtils.h"
 #endif
 
 #include "nsHashtable.h"
@@ -83,26 +83,21 @@ public:
   NS_IMETHOD ClearStyleDataAndReflow(void);
   NS_IMETHOD ResolveStyleContextFor(nsIContent* aContent,
                                     nsIStyleContext* aParentContext,
-                                    PRBool aForceUnique,
                                     nsIStyleContext** aResult);
   NS_IMETHOD ResolveStyleContextForNonElement(nsIStyleContext* aParentContext,
-                                              PRBool aForceUnique,
                                               nsIStyleContext** aResult);
   NS_IMETHOD ResolvePseudoStyleContextFor(nsIContent* aParentContent,
                                           nsIAtom* aPseudoTag,
                                           nsIStyleContext* aParentContext,
-                                          PRBool aForceUnique,
                                           nsIStyleContext** aResult);
   NS_IMETHOD ResolvePseudoStyleWithComparator(nsIContent* aParentContent,
                                           nsIAtom* aPseudoTag,
                                           nsIStyleContext* aParentContext,
-                                          PRBool aForceUnique,
                                           nsICSSPseudoComparator* aComparator,
                                           nsIStyleContext** aResult);
   NS_IMETHOD ProbePseudoStyleContextFor(nsIContent* aParentContent,
                                         nsIAtom* aPseudoTag,
                                         nsIStyleContext* aParentContext,
-                                        PRBool aForceUnique,
                                         nsIStyleContext** aResult);
   NS_IMETHOD ReParentStyleContext(nsIFrame* aFrame, 
                                   nsIStyleContext* aNewParentContext);
@@ -168,8 +163,10 @@ public:
   NS_IMETHOD SetIsRenderingOnlySelection(PRBool aVal) { mIsRenderingOnlySelection = aVal; return NS_OK; }
   NS_IMETHOD IsRenderingOnlySelection(PRBool* aResult);
 
-  NS_IMETHOD GetBackgroundDraw(PRBool &aCanDraw) { aCanDraw = mDrawBackground; return NS_OK; }
-  NS_IMETHOD SetBackgroundDraw(PRBool aCanDraw) { mDrawBackground = aCanDraw; return NS_OK; }
+  NS_IMETHOD GetBackgroundImageDraw(PRBool &aCanDraw) { aCanDraw = mDrawImageBackground; return NS_OK; }
+  NS_IMETHOD SetBackgroundImageDraw(PRBool aCanDraw) { mDrawImageBackground = aCanDraw; return NS_OK; }
+  NS_IMETHOD GetBackgroundColorDraw(PRBool &aCanDraw) { aCanDraw = mDrawColorBackground; return NS_OK; }
+  NS_IMETHOD SetBackgroundColorDraw(PRBool aCanDraw) { mDrawColorBackground = aCanDraw; return NS_OK; }
 
 #ifdef MOZ_REFLOW_PERF
   NS_IMETHOD CountReflows(const char * aName, PRUint32 aType, nsIFrame * aFrame);
@@ -196,12 +193,13 @@ public:
 //Mohamed  17-1-01
   NS_IMETHOD SetIsBidiSystem(PRBool aIsBidi);
   NS_IMETHOD GetIsBidiSystem(PRBool &aResult) const;
-  NS_IMETHOD GetBidiCharset(nsAWritableString &aCharSet);
+  NS_IMETHOD GetBidiCharset(nsAString &aCharSet);
 //Mohamed End
 #endif // IBMBIDI
 
   NS_IMETHOD GetTheme(nsITheme** aResult);
   NS_IMETHOD ThemeChanged();
+  NS_IMETHOD SysColorChanged();
 
 protected:
   nsPresContext();
@@ -250,7 +248,8 @@ protected:
   PRUint8               mFocusRingWidth;          // set in GetUserPrefs
   PRPackedBool          mFocusRingOnAnything;     // set in GetUserPrefs
 
-  PRPackedBool  				mDrawBackground;
+  PRPackedBool  				mDrawImageBackground;
+  PRPackedBool  				mDrawColorBackground;
   PRUint8               mDefaultBackgroundImageAttachment;
   PRUint8               mDefaultBackgroundImageRepeat;
 
@@ -264,7 +263,6 @@ protected:
   nsCOMPtr<nsIURI>      mBaseURL;
 
   nsCompatibility       mCompatibilityMode;
-  PRPackedBool          mCompatibilityLocked;
   nsWidgetRendering     mWidgetRenderingMode;
   PRPackedBool          mImageAnimationStopped;   // image animation stopped
 
