@@ -37,7 +37,6 @@
 #include "nsCOMPtr.h"
 #include "nsBulletFrame.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLValue.h"
 #include "nsHTMLContainerFrame.h"
@@ -46,7 +45,7 @@
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsIDocument.h"
-#include "nsHTMLReflowCommand.h"
+#include "nsReflowPath.h"
 #include "nsIRenderingContext.h"
 #include "nsILoadGroup.h"
 #include "nsIURL.h"
@@ -133,7 +132,7 @@ nsBulletFrame::Init(nsIPresContext*  aPresContext,
     GetBaseURI(getter_AddRefs(baseURI));
 
     nsCOMPtr<nsIURI> imgURI;
-    NS_NewURI(getter_AddRefs(imgURI), myList->mListStyleImage, baseURI);
+    NS_NewURI(getter_AddRefs(imgURI), myList->mListStyleImage, nsnull, baseURI);
 
     if (!mListener) {
       nsBulletListener *listener;
@@ -145,7 +144,7 @@ nsBulletFrame::Init(nsIPresContext*  aPresContext,
       NS_RELEASE(listener);
     }
 
-    il->LoadImage(imgURI, loadGroup, mListener, aPresContext, nsIRequest::LOAD_NORMAL, nsnull, nsnull, getter_AddRefs(mImageRequest));
+    il->LoadImage(imgURI, nsnull, loadGroup, mListener, aPresContext, nsIRequest::LOAD_NORMAL, nsnull, nsnull, getter_AddRefs(mImageRequest));
   }
 
   return NS_OK;
@@ -264,7 +263,7 @@ nsBulletFrame::Paint(nsIPresContext*      aPresContext,
         GetListItemText(aPresContext, *myList, text);
 
         if (NS_STYLE_DIRECTION_RTL == vis->mDirection) {
-          nsStr::Delete2(text, 0, 1);
+          text.Cut(0, 1);
           text.Append(NS_LITERAL_STRING("."));
         }
         break;
@@ -319,6 +318,11 @@ nsBulletFrame::Paint(nsIPresContext*      aPresContext,
     case NS_STYLE_LIST_STYLE_MOZ_KHMER:
     case NS_STYLE_LIST_STYLE_MOZ_HANGUL:
     case NS_STYLE_LIST_STYLE_MOZ_HANGUL_CONSONANT:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_NUMERIC:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_AM:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ER:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
       aPresContext->GetMetricsFor(myFont->mFont, getter_AddRefs(fm));
       GetListItemText(aPresContext, *myList, text);
       aRenderingContext.SetFont(fm);
@@ -452,23 +456,23 @@ static void RomanToText(PRInt32 ordinal, nsString& result, const char* achars, c
     romanPos--;
     addOn.SetLength(0);
     switch(*dp) {
-      case '3':  addOn.AppendWithConversion(achars[romanPos]);
-      case '2':  addOn.AppendWithConversion(achars[romanPos]);
-      case '1':  addOn.AppendWithConversion(achars[romanPos]);
+      case '3':  addOn.Append(PRUnichar(achars[romanPos]));
+      case '2':  addOn.Append(PRUnichar(achars[romanPos]));
+      case '1':  addOn.Append(PRUnichar(achars[romanPos]));
         break;
       case '4':
-        addOn.AppendWithConversion(achars[romanPos]);
+        addOn.Append(PRUnichar(achars[romanPos]));
         // FALLTHROUGH
       case '5': case '6':
       case '7': case  '8':
-        addOn.AppendWithConversion(bchars[romanPos]);
+        addOn.Append(PRUnichar(bchars[romanPos]));
         for(n=0;n<(*dp-'5');n++) {
-          addOn.AppendWithConversion(achars[romanPos]);
+          addOn.Append(PRUnichar(achars[romanPos]));
         }
         break;
       case '9':
-        addOn.AppendWithConversion(achars[romanPos]);
-        addOn.AppendWithConversion(achars[romanPos+1]);
+        addOn.Append(PRUnichar(achars[romanPos]));
+        addOn.Append(PRUnichar(achars[romanPos+1]));
         break;
       default:
         break;
@@ -603,6 +607,51 @@ static PRUnichar gHangulConsonantChars[HANGUL_CONSONANT_CHARS_SIZE] =
 0x314d, 0x314e
 };
 
+// Ge'ez set of Ethiopic ordered list. There are other locale-dependent sets.
+// For the time being, let's implement two Ge'ez sets only
+// per Momoi san's suggestion in bug 102252. 
+// For details, refer to http://www.ethiopic.org/Collation/OrderedLists.html.
+#define ETHIOPIC_HALEHAME_CHARS_SIZE 26
+static PRUnichar gEthiopicHalehameChars[ETHIOPIC_HALEHAME_CHARS_SIZE] =
+{                                      
+0x1200, 0x1208, 0x1210, 0x1218, 0x1220, 0x1228,
+0x1230, 0x1240, 0x1260, 0x1270, 0x1280, 0x1290,
+0x12a0, 0x12a8, 0x12c8, 0x12d0, 0x12d8, 0x12e8,
+0x12f0, 0x1308, 0x1320, 0x1330, 0x1338, 0x1340,
+0x1348, 0x1350
+};
+#define ETHIOPIC_HALEHAME_AM_CHARS_SIZE 33
+static PRUnichar gEthiopicHalehameAmChars[ETHIOPIC_HALEHAME_AM_CHARS_SIZE] =
+{                                      
+0x1200, 0x1208, 0x1210, 0x1218, 0x1220, 0x1228,
+0x1230, 0x1238, 0x1240, 0x1260, 0x1270, 0x1278,
+0x1280, 0x1290, 0x1298, 0x12a0, 0x12a8, 0x12b8,
+0x12c8, 0x12d0, 0x12d8, 0x12e0, 0x12e8, 0x12f0,
+0x1300, 0x1308, 0x1320, 0x1328, 0x1330, 0x1338,
+0x1340, 0x1348, 0x1350
+};
+#define ETHIOPIC_HALEHAME_TI_ER_CHARS_SIZE 31
+static PRUnichar gEthiopicHalehameTiErChars[ETHIOPIC_HALEHAME_TI_ER_CHARS_SIZE] =
+{                                      
+0x1200, 0x1208, 0x1210, 0x1218, 0x1228, 0x1230,
+0x1238, 0x1240, 0x1250, 0x1260, 0x1270, 0x1278,
+0x1290, 0x1298, 0x12a0, 0x12a8, 0x12b8, 0x12c8,
+0x12d0, 0x12d8, 0x12e0, 0x12e8, 0x12f0, 0x1300,
+0x1308, 0x1320, 0x1328, 0x1330, 0x1338, 0x1348,
+0x1350
+};
+#define ETHIOPIC_HALEHAME_TI_ET_CHARS_SIZE 34
+static PRUnichar gEthiopicHalehameTiEtChars[ETHIOPIC_HALEHAME_TI_ET_CHARS_SIZE] =
+{                                      
+0x1200, 0x1208, 0x1210, 0x1218, 0x1220, 0x1228,
+0x1230, 0x1238, 0x1240, 0x1250, 0x1260, 0x1270,
+0x1278, 0x1280, 0x1290, 0x1298, 0x12a0, 0x12a8,
+0x12b8, 0x12c8, 0x12d0, 0x12d8, 0x12e0, 0x12e8,
+0x12f0, 0x1300, 0x1308, 0x1320, 0x1328, 0x1330,
+0x1338, 0x1340, 0x1348, 0x1350
+};
+
+
 // We know cjk-ideographic need 31 characters to display 99,999,999,999,999,999
 // georgian and armenian need 6 at most
 // hebrew may need more...
@@ -925,6 +974,64 @@ static void GeorgianToText(PRInt32 ordinal, nsString& result)
   }
 }
 
+// Convert ordinal to Ethiopic numeric representation.
+// The detail is available at http://www.ethiopic.org/Numerals/
+// The algorithm used here was almost a verbatim copy of 
+// the pseudo-code put up there by Daniel Yacob <yacob@geez.org>.
+// Another reference is Unicode 3.0 standard section 11.1. 
+
+static void EthiopicToText(PRInt32 ordinal, nsString& result)
+{  
+  nsAutoString asciiNumberString;      // decimal string representation of ordinal
+  DecimalToText(ordinal, asciiNumberString);
+  PRInt32 n = asciiNumberString.Length() - 1;
+
+  // Iterate from the lowest digit to higher digits
+  for (PRInt32 place = 0; place <= n; place++) {
+    PRUnichar asciiTen = '0'; 
+    PRUnichar asciiOne = asciiNumberString.CharAt(n - place);
+
+    place++;
+
+    if (place <= n) 
+      asciiTen = asciiNumberString.CharAt(n - place);
+
+    // '00' is not represented and has to be skipped.
+    if (asciiOne == '0' && asciiTen == '0' && place < n) 
+      continue;
+
+    nsAutoString ethioNumber;
+
+    // calculate digits at  10^(2*place) and 10^(2*place+1) 
+    if (asciiTen > '0' || asciiOne > '1' || place == 1) 
+    {
+      if (asciiTen > '0') 
+      {
+        // map onto Ethiopic "tens": U+1372=Ethiopic number ten
+        ethioNumber += (PRUnichar) ((PRInt32) asciiTen +  0x1372 - 0x31); 
+      }
+      if (asciiOne > '0') 
+      {
+        //map onto Ethiopic "ones": 0x1369=Ethiopic digit one
+        ethioNumber += (PRUnichar) ((PRInt32) asciiOne + 0x1369 - 0x31); 
+      }
+    }
+
+   // Now add 'cental-place' specifiers in terms of power of hundred
+
+   // if (place > 1) : The lowest two digits don't need 'cental-place' specifier
+     if (place & 2)   // if odd power of hundred 
+       ethioNumber += (PRUnichar) 0x137B;   // append Ethiopic number hundred 
+
+     // append Ethiopic number ten thousand every four decimal digits
+     for (PRInt32 j = 0; j < place / 4; j++) 
+       ethioNumber += (PRUnichar) 0x137C;   // 0x137C = Ethiopic number ten thousand
+
+     result.Insert(ethioNumber, 0);
+  }  
+}
+
+
 void
 nsBulletFrame::GetListItemText(nsIPresContext* aCX,
                                const nsStyleList& aListStyle,
@@ -1103,6 +1210,26 @@ nsBulletFrame::GetListItemText(nsIPresContext* aCX,
     case NS_STYLE_LIST_STYLE_MOZ_HANGUL_CONSONANT:
       CharListToText(mOrdinal, result, gHangulConsonantChars, HANGUL_CONSONANT_CHARS_SIZE);
       break;
+
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME:
+      CharListToText(mOrdinal, result, gEthiopicHalehameChars, ETHIOPIC_HALEHAME_CHARS_SIZE);
+      break;
+
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_NUMERIC:
+      EthiopicToText(mOrdinal, result);
+      break;
+
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_AM:
+      CharListToText(mOrdinal, result, gEthiopicHalehameAmChars, ETHIOPIC_HALEHAME_AM_CHARS_SIZE);
+      break;
+
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ER:
+      CharListToText(mOrdinal, result, gEthiopicHalehameTiErChars, ETHIOPIC_HALEHAME_TI_ER_CHARS_SIZE);
+      break;
+
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
+      CharListToText(mOrdinal, result, gEthiopicHalehameTiEtChars, ETHIOPIC_HALEHAME_TI_ET_CHARS_SIZE);
+      break;
   }
 #ifdef IBMBIDI
   if (NS_STYLE_DIRECTION_RTL != vis->mDirection)
@@ -1315,6 +1442,11 @@ nsBulletFrame::GetDesiredSize(nsIPresContext*  aCX,
     case NS_STYLE_LIST_STYLE_MOZ_KHMER:
     case NS_STYLE_LIST_STYLE_MOZ_HANGUL:
     case NS_STYLE_LIST_STYLE_MOZ_HANGUL_CONSONANT:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_NUMERIC:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_AM:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ER:
+    case NS_STYLE_LIST_STYLE_MOZ_ETHIOPIC_HALEHAME_TI_ET:
       GetListItemText(aCX, *myList, text);
       fm->GetHeight(aMetrics.height);
       aReflowState.rendContext->SetFont(fm);
@@ -1334,62 +1466,70 @@ nsBulletFrame::Reflow(nsIPresContext* aPresContext,
 {
   DO_GLOBAL_REFLOW_COUNT("nsBulletFrame", aReflowState.reason);
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aMetrics, aStatus);
-  if (eReflowReason_Incremental == aReflowState.reason) {
+
+  PRBool isStyleChange = PR_FALSE;
+  if (eReflowReason_StyleChange == aReflowState.reason)
+    isStyleChange = PR_TRUE;
+  else if (eReflowReason_Incremental == aReflowState.reason) {
+    // XXXwaterson The reflow had better be targeted at this frame.
     nsReflowType type;
-    aReflowState.reflowCommand->GetType(type);
+    aReflowState.path->mReflowCommand->GetType(type);
 
     /* if the style changed, see if we need to load a new url */
-    if (eReflowType_StyleChanged == type) {
-      nsCOMPtr<nsIURI> baseURI;
-      GetBaseURI(getter_AddRefs(baseURI));
+    if (eReflowType_StyleChanged == type)
+      isStyleChange = PR_TRUE;
+  }
 
-      const nsStyleList* myList = (const nsStyleList*)mStyleContext->GetStyleData(eStyleStruct_List);
+  if (isStyleChange) {
+    nsCOMPtr<nsIURI> baseURI;
+    GetBaseURI(getter_AddRefs(baseURI));
 
-      if (!myList->mListStyleImage.IsEmpty()) {
+    const nsStyleList* myList = (const nsStyleList*)mStyleContext->GetStyleData(eStyleStruct_List);
 
-        if (!mListener) {
-          nsBulletListener *listener;
-          NS_NEWXPCOM(listener, nsBulletListener);
-          NS_ADDREF(listener);
-          listener->SetFrame(this);
-          listener->QueryInterface(NS_GET_IID(imgIDecoderObserver), getter_AddRefs(mListener));
-          NS_ASSERTION(mListener, "queryinterface for the listener failed");
-          NS_RELEASE(listener);
-        }
+    if (!myList->mListStyleImage.IsEmpty()) {
+
+      if (!mListener) {
+        nsBulletListener *listener;
+        NS_NEWXPCOM(listener, nsBulletListener);
+        NS_ADDREF(listener);
+        listener->SetFrame(this);
+        listener->QueryInterface(NS_GET_IID(imgIDecoderObserver), getter_AddRefs(mListener));
+        NS_ASSERTION(mListener, "queryinterface for the listener failed");
+        NS_RELEASE(listener);
+      }
 
 
-        nsCOMPtr<nsIURI> newURI;
-        NS_NewURI(getter_AddRefs(newURI), myList->mListStyleImage, baseURI);
+      nsCOMPtr<nsIURI> newURI;
+      NS_NewURI(getter_AddRefs(newURI), myList->mListStyleImage, nsnull, baseURI);
 
-        PRBool needNewRequest = PR_TRUE;
+      PRBool needNewRequest = PR_TRUE;
 
-        if (mImageRequest) {
-          // Reload the image, maybe...
-          nsCOMPtr<nsIURI> oldURI;
-          mImageRequest->GetURI(getter_AddRefs(oldURI));
-          if (oldURI) {
-            PRBool same;
-            newURI->Equals(oldURI, &same);
-            if (same) {
-              needNewRequest = PR_FALSE;
-            } else {
-              mImageRequest->Cancel(NS_ERROR_FAILURE);
-              mImageRequest = nsnull;
-            }
+      if (mImageRequest) {
+        // Reload the image, maybe...
+        nsCOMPtr<nsIURI> oldURI;
+        mImageRequest->GetURI(getter_AddRefs(oldURI));
+        if (oldURI) {
+          PRBool same;
+          newURI->Equals(oldURI, &same);
+          if (same) {
+            needNewRequest = PR_FALSE;
+          } else {
+            mImageRequest->Cancel(NS_ERROR_FAILURE);
+            mImageRequest = nsnull;
           }
         }
+      }
 
-        if (needNewRequest) {
-          nsresult rv;
-          nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1", &rv));
-          if (NS_FAILED(rv))
-            return rv;
+      if (needNewRequest) {
+        nsresult rv;
+        nsCOMPtr<imgILoader> il(do_GetService("@mozilla.org/image/loader;1", &rv));
+        if (NS_FAILED(rv))
+          return rv;
 
-          nsCOMPtr<nsILoadGroup> loadGroup;
-          GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
+        nsCOMPtr<nsILoadGroup> loadGroup;
+        GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
 
-          il->LoadImage(newURI, loadGroup, mListener, aPresContext, nsIRequest::LOAD_NORMAL, nsnull, nsnull, getter_AddRefs(mImageRequest));
-        }
+        il->LoadImage(newURI, nsnull, loadGroup, mListener, aPresContext, nsIRequest::LOAD_NORMAL, nsnull, nsnull, getter_AddRefs(mImageRequest));
       }
     }
   }
@@ -1410,6 +1550,7 @@ nsBulletFrame::Reflow(nsIPresContext* aPresContext,
     aMetrics.maxElementSize->height = aMetrics.height;
   }
   aStatus = NS_FRAME_COMPLETE;
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aMetrics);
   return NS_OK;
 }
 
