@@ -49,7 +49,6 @@
 #include "nsVoidArray.h"
 #include "nsISizeOfHandler.h"
 #include "nsHTMLReflowCommand.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsIFrameManager.h"
 #include "nsIPresShell.h"
@@ -61,6 +60,7 @@
 #include "nsGfxCIID.h"
 #include "nsIServiceManager.h"
 #include "nsCSSRendering.h"
+#include "nsTransform2D.h"
 
 static NS_DEFINE_CID(kRegionCID, NS_REGION_CID);
 
@@ -104,15 +104,17 @@ nsContainerFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                       nsIAtom*        aListName,
                                       nsIFrame*       aChildList)
 {
-//  NS_PRECONDITION(mFrames.IsEmpty(), "already initialized");
-
   nsresult  result;
   if (!mFrames.IsEmpty()) {
     // We already have child frames which means we've already been
     // initialized
+#ifdef DEBUG_dbaron // XXX Fix asserts and remove this ifdef.
+    NS_NOTREACHED("unexpected second call to SetInitialChildList");
+#endif
     result = NS_ERROR_UNEXPECTED;
   } else if (aListName) {
     // All we know about is the unnamed principal child list
+    NS_NOTREACHED("unknown frame list");
     result = NS_ERROR_INVALID_ARG;
   } else {
 #ifdef NS_DEBUG
@@ -207,9 +209,11 @@ nsContainerFrame::PaintChild(nsIPresContext*      aPresContext,
                              nsFramePaintLayer    aWhichLayer,
                              PRUint32             aFlags)
 {
+  NS_ASSERTION(aFrame, "no frame to paint!");
+  if (!aFrame) return;
   nsIView *pView;
   aFrame->GetView(aPresContext, &pView);
-  if (nsnull == pView) {
+  if (!pView) {
     nsRect kidRect;
     aFrame->GetRect(kidRect);
     nsFrameState state;
@@ -246,12 +250,35 @@ nsContainerFrame::PaintChild(nsIPresContext*      aPresContext,
       // coordinate system.
       damageArea.x -= kidRect.x;
       damageArea.y -= kidRect.y;
+
+
+      // The transform components are saved and restored instead 
+      // of using PushState and PopState because they are too slow
+      // because they also save and restore the clip state.
+      // Note: Setting a negative translation to restore the 
+      // state does not work because the floating point errors can accumulate
+      // causing the display of some frames to be off by one pixel. 
+      // This happens frequently when running in 120DPI mode where frames are
+      // often positioned at 1/2 pixel locations and small floating point errors
+      // will cause the frames to vary their pixel x location during scrolling
+      // operations causes a single scan line of pixels to be shifted left relative
+      // to the other scan lines for the same text. 
+
+      // Save the transformation matrix's translation components.
+      float xMatrix; 
+      float yMatrix;
+      nsTransform2D *theTransform; 
+      aRenderingContext.GetCurrentTransform(theTransform);
+      NS_ASSERTION(theTransform != nsnull, "The rendering context transform is null");
+      theTransform->GetTranslation(&xMatrix, &yMatrix);
+
       aRenderingContext.Translate(kidRect.x, kidRect.y);
 
       // Paint the kid
       aFrame->Paint(aPresContext, aRenderingContext, damageArea, aWhichLayer, aFlags);
-      // don't use PushState and PopState, because they're slow
-      aRenderingContext.Translate(-kidRect.x, -kidRect.y);
+
+      // Restore the transformation matrix's translation components.
+      theTransform->SetTranslation(xMatrix, yMatrix);
 
 #ifdef NS_DEBUG
       // Draw a border around the child
@@ -293,8 +320,16 @@ nsContainerFrame::GetFrameForPointUsing(nsIPresContext* aPresContext,
   FirstChild(aPresContext, aList, &kid);
   *aFrame = nsnull;
   tmp.MoveTo(aPoint.x - mRect.x, aPoint.y - mRect.y);
+
+  nsPoint originOffset;
+  nsIView *view = nsnull;
+  nsresult rv = GetOriginToViewOffset(aPresContext, originOffset, &view);
+
+  if (NS_SUCCEEDED(rv) && view)
+    tmp += originOffset;
+
   while (nsnull != kid) {
-    nsresult rv = kid->GetFrameForPoint(aPresContext, tmp, aWhichLayer, &hit);
+    rv = kid->GetFrameForPoint(aPresContext, tmp, aWhichLayer, &hit);
 
     if (NS_SUCCEEDED(rv) && hit) {
       *aFrame = hit;
@@ -367,11 +402,11 @@ static PRBool
 TranslatePointTo(nsPoint& aPoint, nsIView* aChildView, nsIView* aParentView)
 {
   do {
-    nsRect  bounds;
+    nscoord x, y;
 
-    aChildView->GetBounds(bounds);
-    aPoint.x += bounds.x;
-    aPoint.y += bounds.y;
+    aChildView->GetPosition(&x, &y);
+    aPoint.x += x;
+    aPoint.y += y;
     aChildView->GetParent(aChildView);
   } while (aChildView && (aChildView != aParentView));
 
@@ -402,7 +437,11 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
     nsCOMPtr<nsIViewManager> vm;
     view->GetViewManager(*getter_AddRefs(vm));
 
-    if (containingView != parentView) {
+    // it's possible for the parentView to be nonnull but containingView to be
+    // null, when the parent view doesn't belong to this frame tree but to
+    // the frame tree of some enclosing document. We do nothing in that case,
+    // but we have to check that containingView is nonnull or we will crash.
+    if (nsnull != containingView && containingView != parentView) {
       // it is possible for parent view not to have a frame attached to it
       // kind of an anonymous view. This happens with native scrollbars and
       // the clip view. To fix this we need to go up and parentView chain
@@ -416,10 +455,10 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
         if (data)
           break;
       
-        nsRect bounds;
-        parentView->GetBounds(bounds);
-        origin.x -= bounds.x;
-        origin.y -= bounds.y;
+        nscoord x, y;
+        parentView->GetPosition(&x, &y);
+        origin.x -= x;
+        origin.y -= y;
         parentView->GetParent(parentView);
       }
      
@@ -431,7 +470,12 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
         // We have the origin in the coordinate space of the containing view,
         // but we need it in the coordinate space of the parent view so do a
         // view translation
-        NS_VERIFY(TranslatePointTo(origin, containingView, parentView), "translation failed");
+#ifdef DEBUG
+        PRBool ok = 
+#endif
+        TranslatePointTo(origin, containingView, parentView);
+
+        NS_ASSERTION(ok, "translation failed");
       }
     }
 
@@ -488,12 +532,14 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
       PositionFrameView(aPresContext, aFrame);
     }
 
+    nsRect newSize(0, 0, frameSize.width, frameSize.height);
     if (0 == (aFlags & NS_FRAME_NO_SIZE_VIEW)) {
       // If the frame has child frames that stick outside the content
       // area, then size the view large enough to include those child
       // frames
       if ((kidState & NS_FRAME_OUTSIDE_CHILDREN) && aCombinedArea) {
         vm->ResizeView(aView, *aCombinedArea);
+        newSize = *aCombinedArea;
       } else {
         nsRect bounds;
         aView->GetBounds(bounds);
@@ -505,10 +551,15 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
         // This is because some frames do not invalidate themselves properly. see bug 73825.
         // Once bug 73825 is fixed, we should always pass PR_TRUE instead of 
         // frameSize.width == width && frameSize.height >= height.
-        nsRect newSize(0, 0, frameSize.width, frameSize.height);
         vm->ResizeView(aView, newSize, 
                        (frameSize.width == bounds.width && frameSize.height >= bounds.height));
       }
+    } else {
+      aView->GetBounds(newSize);
+      nscoord x, y;
+      aView->GetPosition(&x, &y);
+      newSize.x -= x;
+      newSize.y -= y;
     }
   
     const nsStyleBackground* bg;
@@ -526,9 +577,18 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     // See if the view should be hidden or visible
     PRBool  viewIsVisible = PR_TRUE;
     PRBool  viewHasTransparentContent =
-        !isCanvas &&
-        (!hasBG ||
-         (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT));
+         (!hasBG ||
+         (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT) ||
+         !aFrame->CanPaintBackground());
+    if (isCanvas && viewHasTransparentContent) {
+      nsIView* rootView;
+      vm->GetRootView(rootView);
+      nsIView* rootParent;
+      rootView->GetParent(rootParent);
+      if (nsnull == rootParent) {
+        viewHasTransparentContent = PR_FALSE;
+      }
+    }
 
     if (NS_STYLE_VISIBILITY_COLLAPSE == vis->mVisible) {
       viewIsVisible = PR_FALSE;
@@ -542,14 +602,15 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
         NS_RELEASE(widget);
       }
       else {
-        // If it's a scroll frame, then hide the view. This means that
+        // If it's a scroll frame or a list control frame which is derived from the scrollframe, 
+        // then hide the view. This means that
         // child elements can't override their parent's visibility, but
         // it's not practical to leave it visible in all cases because
         // the scrollbars will be showing
         nsIAtom*  frameType;
         aFrame->GetFrameType(&frameType);
 
-        if (frameType == nsLayoutAtoms::scrollFrame) {
+        if (frameType == nsLayoutAtoms::scrollFrame || frameType == nsLayoutAtoms::listControlFrame) {
           viewIsVisible = PR_FALSE;
 
         } else {
@@ -589,18 +650,23 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
 
     // Make sure z-index is correct
     PRInt32                zIndex = 0;
+    PRInt32                oldZIndex;
+    PRBool                 oldAutoZIndex;
+    PRBool                 oldTopMost;
     PRBool                 autoZIndex = PR_FALSE;
     const nsStylePosition* position;
 
+    aView->GetZIndex(oldZIndex, oldAutoZIndex, oldTopMost);
+
     aFrame->GetStyleData(eStyleStruct_Position, (const nsStyleStruct*&)position);
     if (position->mZIndex.GetUnit() == eStyleUnit_Integer) {
       zIndex = position->mZIndex.GetIntValue();
 
     } else if (position->mZIndex.GetUnit() == eStyleUnit_Auto) {
       autoZIndex = PR_TRUE;
-    }
-    
-    vm->SetViewZIndex(aView, autoZIndex, zIndex);
+    } 
+  
+    vm->SetViewZIndex(aView, autoZIndex, zIndex, oldTopMost);
 
     // There are two types of clipping:
     // - 'clip' which only applies to absolutely positioned elements, and is
@@ -617,7 +683,7 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     hasClip = display->IsAbsolutelyPositioned() && (display->mClipFlags & NS_STYLE_CLIP_RECT);
     hasOverflowClip = isBlockLevel && (display->mOverflow == NS_STYLE_OVERFLOW_HIDDEN);
     if (hasClip || hasOverflowClip) {
-      nsRect  clipRect, overflowClipRect;
+      nsRect  clipRect;
 
       if (hasClip) {
         // Start with the 'auto' values and then factor in user specified values
@@ -626,27 +692,15 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
         if (display->mClipFlags & NS_STYLE_CLIP_RECT) {
           if (0 == (NS_STYLE_CLIP_TOP_AUTO & display->mClipFlags)) {
             clipRect.y = display->mClip.y;
-            if (clipRect.y > 0) {
-              viewHasTransparentContent = PR_TRUE;
-            }
           }
           if (0 == (NS_STYLE_CLIP_LEFT_AUTO & display->mClipFlags)) {
             clipRect.x = display->mClip.x;
-            if (clipRect.x > 0) {
-              viewHasTransparentContent = PR_TRUE;
-            }
           }
           if (0 == (NS_STYLE_CLIP_RIGHT_AUTO & display->mClipFlags)) {
             clipRect.width = display->mClip.width;
-            if (clipRect.width < frameSize.width) {
-              viewHasTransparentContent = PR_TRUE;
-            }
           }
           if (0 == (NS_STYLE_CLIP_BOTTOM_AUTO & display->mClipFlags)) {
             clipRect.height = display->mClip.height;
-            if (clipRect.height < frameSize.height) {
-              viewHasTransparentContent = PR_TRUE;
-            }
           }
         }
       }
@@ -654,37 +708,42 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
       if (hasOverflowClip) {
         const nsStyleBorder* borderStyle;
         const nsStylePadding* paddingStyle;
-        nsMargin              border, padding;
+        nsMargin border, padding;
 
         aFrame->GetStyleData(eStyleStruct_Border, (const nsStyleStruct*&)borderStyle);
         aFrame->GetStyleData(eStyleStruct_Padding, (const nsStyleStruct*&)paddingStyle);
         
         // XXX We don't support the 'overflow-clip' property yet so just use the
         // content area (which is the default value) as the clip shape
-        overflowClipRect.SetRect(0, 0, frameSize.width, frameSize.height);
+        nsRect overflowClipRect(0, 0, frameSize.width, frameSize.height);
         borderStyle->GetBorder(border);
         overflowClipRect.Deflate(border);
         // XXX We need to handle percentage padding
         if (paddingStyle->GetPadding(padding)) {
           overflowClipRect.Deflate(padding);
         }
+
+        if (hasClip) {
+          // If both 'clip' and 'overflow-clip' apply then use the intersection
+          // of the two
+          clipRect.IntersectRect(clipRect, overflowClipRect);
+        } else {
+          clipRect = overflowClipRect;
+        }
       }
 
-      // If both 'clip' and 'overflow-clip' apply then use the intersection
-      // of the two
-      if (hasClip && hasOverflowClip) {
-        clipRect.IntersectRect(clipRect, overflowClipRect);
+      // If part of the view is being clipped out, then mark it transparent
+      if (clipRect.y > newSize.y
+          || clipRect.x > newSize.x
+          || clipRect.XMost() < newSize.XMost()
+          || clipRect.YMost() < newSize.YMost()) {
+        viewHasTransparentContent = PR_TRUE;
       }
 
       // Set clipping of child views.
       nsIRegion *region = CreateRegion();
       if (region != nsnull) {
-        if (hasClip) {
-          region->SetTo(clipRect.x, clipRect.y, clipRect.width, clipRect.height);
-        } else {
-          region->SetTo(overflowClipRect.x, overflowClipRect.y,
-                        overflowClipRect.width, overflowClipRect.height);
-        }
+        region->SetTo(clipRect.x, clipRect.y, clipRect.width, clipRect.height);
         vm->SetViewChildClipRegion(aView, region);
         NS_RELEASE(region);
       }
