@@ -45,7 +45,6 @@
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsHTMLContainerFrame.h"
-#include "nsHTMLIIDs.h"
 #include "nsCSSRendering.h"
 #include "nsIScrollableView.h"
 #include "nsWidgetsCID.h"
@@ -67,12 +66,12 @@
 #include "nsINodeInfo.h"
 #include "nsIScrollbarFrame.h"
 #include "nsIScrollbarMediator.h"
-#include "nsIGfxTextControlFrame.h"
+#include "nsITextControlFrame.h"
 #include "nsIDOMHTMLTextAreaElement.h"
 
 #include "nsIPrintPreviewContext.h"
 #include "nsIURI.h"
-
+#include "nsGUIEvent.h"
 //----------------------------------------------------------------------
 
 class nsGfxScrollFrameInner : public nsIDocumentObserver, 
@@ -104,7 +103,8 @@ public:
                             nsISupports* aSubContent) { return NS_OK; }
   NS_IMETHOD ContentStatesChanged(nsIDocument* aDocument,
                                   nsIContent* aContent1,
-                                  nsIContent* aContent2) { return NS_OK; }
+                                  nsIContent* aContent2,
+                                  PRInt32 aStateMask) { return NS_OK; }
   NS_IMETHOD AttributeChanged(nsIDocument *aDocument,
                               nsIContent*  aContent,
                               PRInt32      aNameSpaceID,
@@ -400,7 +400,7 @@ nsGfxScrollFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
                                          nsISupportsArray& aAnonymousChildren)
 {
   // The anonymous <div> used by <inputs> never gets scrollbars.
-  nsCOMPtr<nsIGfxTextControlFrame2> textFrame(do_QueryInterface(mParent));
+  nsCOMPtr<nsITextControlFrame> textFrame(do_QueryInterface(mParent));
   if (textFrame) {
     // Make sure we are not a text area.
     nsCOMPtr<nsIContent> content;
@@ -460,6 +460,10 @@ NS_IMETHODIMP
 nsGfxScrollFrame::Destroy(nsIPresContext* aPresContext)
 
 {
+  nsIScrollableView *view = mInner->GetScrollableView(aPresContext);
+  NS_ASSERTION(view, "unexpected null pointer");
+  if (view)
+    view->RemoveScrollPositionListener(mInner);
   return nsBoxFrame::Destroy(aPresContext);
 }
 
@@ -602,19 +606,9 @@ nsGfxScrollFrame::Paint(nsIPresContext*   aPresContext,
 {
 nsresult result;
 
-  // there are some mechanisms to turn off background painting for print and print preview, these 
-  // need to be turned on for the time being.. just for this paint.. then reset to the
-  // original values before this routine returns.
-  PRBool canDraw;
-  aPresContext->GetBackgroundDraw(canDraw);
-  aPresContext->SetBackgroundDraw(PR_TRUE);
 
   // Paint our children
-  result = nsBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
-                                 aWhichLayer);
-
-  aPresContext->SetBackgroundDraw(canDraw);
-
+  result = nsBoxFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,aWhichLayer);
   return result;
 
 }
@@ -802,6 +796,7 @@ nsGfxScrollFrame::Reflow(nsIPresContext*      aPresContext,
       mInner->mMaxElementSize.height = size->height;
   }
   
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -899,6 +894,8 @@ nsGfxScrollFrameInner::AttributeChanged(nsIDocument *aDocument,
      hframe->GetContent(getter_AddRefs(hcontent));
      vframe->GetContent(getter_AddRefs(vcontent));
 
+     nsIFrame  * targetFrame = nsnull;
+     nsCOMPtr<nsIContent> targetContent;
      if (hcontent.get() == aContent  || vcontent.get() == aContent)
      {
         nscoord x = 0;
@@ -911,6 +908,8 @@ nsGfxScrollFrameInner::AttributeChanged(nsIDocument *aDocument,
 
            // convert it to an integer
            x = value.ToInteger(&error);
+           targetFrame = hframe;
+           targetContent = hcontent;
         }
 
         if (NS_CONTENT_ATTR_HAS_VALUE == vcontent->GetAttr(kNameSpaceID_None, nsXULAtoms::curpos, value))
@@ -919,8 +918,23 @@ nsGfxScrollFrameInner::AttributeChanged(nsIDocument *aDocument,
 
            // convert it to an integer
            y = value.ToInteger(&error);
+           targetFrame = vframe;
+           targetContent = vcontent;
         }
 
+        nsScrollbarEvent  event;
+        event.eventStructType = NS_SCROLLBAR_EVENT;
+        event.message = NS_SCROLL_EVENT;
+        event.flags = 0;        
+        nsEventStatus status = nsEventStatus_eIgnore;
+        nsCOMPtr<nsIPresShell> presShell;
+        mOuter->mPresContext->GetShell(getter_AddRefs(presShell));
+        
+        // Fire the onScroll event.
+        if (presShell && targetFrame && targetContent)
+          presShell->HandleEventWithTarget(&event, targetFrame, targetContent,
+                                            NS_EVENT_FLAG_INIT, &status);
+
         nsIScrollableView* s = GetScrollableView(mOuter->mPresContext);
         s->RemoveScrollPositionListener(this);
         ScrollbarChanged(mOuter->mPresContext, x*mOnePixel, y*mOnePixel);
@@ -1298,7 +1312,7 @@ nsGfxScrollFrameInner::Layout(nsBoxLayoutState& aState)
       frame->GetStyleData(eStyleStruct_Visibility, (const nsStyleStruct*&)ourVis);
 
       if (NS_STYLE_DIRECTION_RTL == ourVis->mDirection) {
-        nsCOMPtr<nsIGfxTextControlFrame2> textControl(
+        nsCOMPtr<nsITextControlFrame> textControl(
           do_QueryInterface(mOuter->mParent) );
         if (textControl) {
           needsLayout = PR_TRUE;
