@@ -49,19 +49,18 @@
 #include "nsGUIEvent.h"
 #include "nsIDocument.h"
 #include "nsIURL.h"
-#include "nsIPtr.h"
 #include "nsPlaceholderFrame.h"
 #include "nsIHTMLContent.h"
 #include "nsHTMLParts.h"
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsViewsCID.h"
-#include "nsHTMLIIDs.h"
 #include "nsIDOMEvent.h"
 #include "nsIScrollableView.h"
 #include "nsWidgetsCID.h"
 #include "nsIStyleSet.h"
 #include "nsCOMPtr.h"
+#include "nsReflowPath.h"
 
 static NS_DEFINE_CID(kCChildCID, NS_CHILD_CID);
 
@@ -143,9 +142,9 @@ nsHTMLContainerFrame::Paint(nsIPresContext*      aPresContext,
  */
 nsresult
 nsHTMLContainerFrame::CreateNextInFlow(nsIPresContext* aPresContext,
-                                       nsIFrame* aOuterFrame,
-                                       nsIFrame* aFrame,
-                                       nsIFrame*& aNextInFlowResult)
+                                       nsIFrame*       aOuterFrame,
+                                       nsIFrame*       aFrame,
+                                       nsIFrame*&      aNextInFlowResult)
 {
   aNextInFlowResult = nsnull;
 
@@ -450,6 +449,7 @@ nsHTMLContainerFrame::CreateViewForFrame(nsIPresContext* aPresContext,
                                          nsIFrame* aContentParentFrame,
                                          PRBool aForce)
 {
+  PRBool isTopMostView = PR_FALSE;
   nsIView* view;
   aFrame->GetView(aPresContext, &view);
   // If we don't yet have a view, see if we need a view
@@ -490,12 +490,18 @@ nsHTMLContainerFrame::CreateViewForFrame(nsIPresContext* aPresContext,
           ("nsHTMLContainerFrame::CreateViewForFrame: frame=%p relatively positioned",
           aFrame));
         aForce = PR_TRUE;
+        isTopMostView = PR_TRUE;
       } else if (display->IsAbsolutelyPositioned()) {
         NS_FRAME_LOG(NS_FRAME_TRACE_CALLS,
           ("nsHTMLContainerFrame::CreateViewForFrame: frame=%p absolutely positioned",
           aFrame));
         aForce = PR_TRUE;
-      }
+        isTopMostView = PR_TRUE;
+      } 
+    }
+
+    if (NS_STYLE_POSITION_FIXED == display->mPosition) {
+      isTopMostView = PR_TRUE;
     }
 
     // See if the frame is a scrolled frame
@@ -586,7 +592,7 @@ nsHTMLContainerFrame::CreateViewForFrame(nsIPresContext* aPresContext,
             autoZIndex = PR_TRUE;
           }
           
-          viewManager->SetViewZIndex(view, autoZIndex, zIndex);
+          viewManager->SetViewZIndex(view, autoZIndex, zIndex, isTopMostView);
           // XXX Drop it at the end of the document order until we can do better
           viewManager->InsertChild(parentView, view, nsnull, PR_TRUE);
 
@@ -601,7 +607,7 @@ nsHTMLContainerFrame::CreateViewForFrame(nsIPresContext* aPresContext,
             
             if (nsnull == zParentView) {
               nsIFrame* zParentFrame = nsnull;
-              
+
               aContentParentFrame->GetParentWithView(aPresContext, &zParentFrame);
               NS_ASSERTION(zParentFrame, "GetParentWithView failed");
               zParentFrame->GetView(aPresContext, &zParentView);
@@ -683,3 +689,102 @@ nsHTMLContainerFrame::CreateViewForFrame(nsIPresContext* aPresContext,
   }
   return NS_OK;
 }
+
+void
+nsHTMLContainerFrame::CheckInvalidateBorder(nsIPresContext* aPresContext,
+                                            nsHTMLReflowMetrics& aDesiredSize,
+                                            const nsHTMLReflowState& aReflowState)
+{
+  // XXX This method ought to deal with padding as well
+  // If this is a style change reflow targeted at this frame, we must repaint
+  // everything (well, we really just have to repaint the borders but we're
+  // a bunch of lazybones).
+  if (aReflowState.reason == eReflowReason_Incremental) {
+    nsHTMLReflowCommand *command = aReflowState.path->mReflowCommand;
+    if (command) {
+      nsReflowType type;
+      command->GetType(type);
+      if (type == eReflowType_StyleChanged) {
+
+#ifdef NOISY_BLOCK_INVALIDATE
+        printf("%p invalidate 1 (%d, %d, %d, %d)\n",
+               this, 0, 0, mRect.width, mRect.height);
+#endif
+
+        // Lots of things could have changed so damage our entire bounds
+        nsRect damageRect(0, 0, mRect.width, mRect.height);
+        if (!damageRect.IsEmpty()) {
+          Invalidate(aPresContext,damageRect);
+        }
+
+        return;
+      }
+    }
+  }
+
+  // If we changed size, we must invalidate the parts of us that have changed
+  // to make the border show up.
+  if ((aReflowState.reason == eReflowReason_Incremental ||
+       aReflowState.reason == eReflowReason_Dirty)) {
+    nsMargin border = aReflowState.mComputedBorderPadding -
+                      aReflowState.mComputedPadding;
+
+    // See if our width changed
+    if ((aDesiredSize.width != mRect.width) && (border.right > 0)) {
+      nsRect damageRect;
+
+      if (aDesiredSize.width < mRect.width) {
+        // Our new width is smaller, so we need to make sure that
+        // we paint our border in its new position
+        damageRect.x = aDesiredSize.width - border.right;
+        damageRect.width = border.right;
+        damageRect.y = 0;
+        damageRect.height = aDesiredSize.height;
+      } else {
+        // Our new width is larger, so we need to erase our border in its
+        // old position
+        damageRect.x = mRect.width - border.right;
+        damageRect.width = border.right;
+        damageRect.y = 0;
+        damageRect.height = mRect.height;
+      }
+
+#ifdef NOISY_BLOCK_INVALIDATE
+      printf("%p invalidate 2 (%d, %d, %d, %d)\n",
+             this, damageRect.x, damageRect.y, damageRect.width, damageRect.height);
+#endif
+      if (!damageRect.IsEmpty()) {
+        Invalidate(aPresContext, damageRect);
+      }
+    }
+
+    // See if our height changed
+    if ((aDesiredSize.height != mRect.height) && (border.bottom > 0)) {
+      nsRect  damageRect;
+
+      if (aDesiredSize.height < mRect.height) {
+        // Our new height is smaller, so we need to make sure that
+        // we paint our border in its new position
+        damageRect.x = 0;
+        damageRect.width = aDesiredSize.width;
+        damageRect.y = aDesiredSize.height - border.bottom;
+        damageRect.height = border.bottom;
+
+      } else {
+        // Our new height is larger, so we need to erase our border in its
+        // old position
+        damageRect.x = 0;
+        damageRect.width = mRect.width;
+        damageRect.y = mRect.height - border.bottom;
+        damageRect.height = border.bottom;
+      }
+#ifdef NOISY_BLOCK_INVALIDATE
+      printf("%p invalidate 3 (%d, %d, %d, %d)\n",
+             this, damageRect.x, damageRect.y, damageRect.width, damageRect.height);
+#endif
+      if (!damageRect.IsEmpty()) {
+        Invalidate(aPresContext, damageRect);
+      }
+    }
+  }
+}
