@@ -44,12 +44,15 @@
 #include "nsStyleConsts.h"
 #include "nsILineBreaker.h"
 #include "nsIWordBreaker.h"
-#include "nsHTMLIIDs.h"
 #include "nsIServiceManager.h"
 #include "nsUnicharUtilCIID.h"
+#include "nsUnicharUtils.h"
 #include "nsICaseConversion.h"
 #include "prenv.h"
 #include "nsIPref.h"
+#ifdef IBMBIDI
+#include "nsLayoutAtoms.h"
+#endif
 
 
 PRPackedBool nsTextTransformer::sWordSelectPrefInited = PR_FALSE;
@@ -88,7 +91,7 @@ nsAutoTextBuffer::GrowTo(PRInt32 aNewSize, PRBool aCopyToHead)
       return NS_ERROR_OUT_OF_MEMORY;
     }
     memcpy(&newBuffer[aCopyToHead ? 0 : mBufferLen],
-                  mBuffer, sizeof(PRUnichar) * mBufferLen);
+           mBuffer, sizeof(PRUnichar) * mBufferLen);
     if (mBuffer != mAutoBuffer) {
       delete [] mBuffer;
     }
@@ -177,6 +180,9 @@ nsTextTransformer::nsTextTransformer(nsILineBreaker* aLineBreaker,
   aPresContext->
     GetLanguageSpecificTransformType(&mLanguageSpecificTransformType);
 
+#ifdef IBMBIDI
+  mPresContext = aPresContext;
+#endif
   if (aLineBreaker == nsnull && aWordBreaker == nsnull )
     NS_ASSERTION(0, "invalid creation of nsTextTransformer");
   
@@ -200,6 +206,22 @@ nsTextTransformer::Init(nsIFrame* aFrame,
                         PRInt32 aStartingOffset,
                         PRBool aLeaveAsAscii)
 {
+#ifdef IBMBIDI
+  PRBool bidiEnabled;
+
+  mPresContext->GetBidiEnabled(&bidiEnabled);
+  if (bidiEnabled) {
+    PRBool isBidiSystem;
+    aFrame->GetBidiProperty(mPresContext, nsLayoutAtoms::charType,
+                            (void**)&mCharType, sizeof(mCharType));
+    mPresContext->GetIsBidiSystem(isBidiSystem);
+    if (mCharType == eCharType_RightToLeftArabic && !isBidiSystem) {
+      SetNeedsArabicShaping(PR_TRUE);
+    }
+    SetNeedsNumericShaping(PR_TRUE);
+  }
+#endif
+
   // Get the contents text content
   nsresult rv;
   nsCOMPtr<nsITextContent> tc = do_QueryInterface(aContent, &rv);
@@ -207,11 +229,12 @@ nsTextTransformer::Init(nsIFrame* aFrame,
     tc->GetText(&mFrag);
 
     // Sanitize aStartingOffset
-    if (NS_WARN_IF_FALSE(aStartingOffset >= 0, "bad starting offset")) {
+    if (aStartingOffset < 0) {
+      NS_WARNING("bad starting offset");
       aStartingOffset = 0;
     }
-    else if (NS_WARN_IF_FALSE(aStartingOffset <= mFrag->GetLength(),
-                              "bad starting offset")) {
+    else if (aStartingOffset > mFrag->GetLength()) {
+      NS_WARNING("bad starting offset");
       aStartingOffset = mFrag->GetLength();
     }
     mOffset = aStartingOffset;
@@ -802,6 +825,7 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
   PRBool isWhitespace = PR_FALSE;
   PRUnichar* result = nsnull;
   PRBool prevBufferPos;
+  PRBool skippedWhitespace = PR_FALSE;
 
   // Initialize OUT parameter
   *aWasTransformed = PR_FALSE;
@@ -833,6 +857,22 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
       case eNormal:
         if (XP_IS_SPACE(firstChar)) {
           offset = ScanNormalWhiteSpace_F();
+
+          // if this is just a '\n', and characters before and after it are CJK chars, 
+          // we will skip this one.
+          if (firstChar == '\n' && 
+              offset - mOffset == 1 && 
+              mOffset > 0 &&
+              offset < fragLen) 
+          {
+            PRUnichar lastChar = frag->CharAt(mOffset - 1);
+            PRUnichar nextChar = frag->CharAt(offset);
+            if (IS_CJ_CHAR(lastChar) && IS_CJ_CHAR(nextChar)) {
+              skippedWhitespace = PR_TRUE;
+              --mBufferPos;
+              mOffset = offset;
+              continue;            }
+          }
           if (firstChar != ' ') {
             *aWasTransformed = PR_TRUE;
           }
@@ -987,15 +1027,26 @@ nsTextTransformer::GetNextWord(PRBool aInWord,
             eLanguageSpecificTransformType_None) {
           LanguageSpecificTransform(result, wordLen, aWasTransformed);
         }
+#ifdef IBMBIDI
+        if (NeedsArabicShaping()) {
+          DoArabicShaping(result, wordLen, aWasTransformed);
+        }
+        if (NeedsNumericShaping()) {
+          DoNumericShaping(result, wordLen, aWasTransformed);
+        }
+#endif
       }
     }
 
     break;
   }
 
+  *aIsWhiteSpaceResult = isWhitespace;
   *aWordLenResult = wordLen;
   *aContentLenResult = offset - mOffset;
-  *aIsWhiteSpaceResult = isWhitespace;
+
+  // we need to adjust the length if a '\n' has been skip between CJK chars
+  *aContentLenResult += (skippedWhitespace ? 1 : 0);
 
   // If the word length doesn't match the content length then we transformed
   // the text
@@ -1359,8 +1410,94 @@ nsTextTransformer::GetPrevWord(PRBool aInWord,
   return result;
 }
 
-//----------------------------------------------------------------------
+#ifdef IBMBIDI
+void
+nsTextTransformer::DoArabicShaping(PRUnichar* aText, 
+                                   PRInt32& aTextLength, 
+                                   PRBool* aWasTransformed)
+{
+  if (aTextLength <= 0)
+    return;
+  
+  PRInt32 newLen;
+  PRBool isVisual;
+  mPresContext->IsVisualMode(isVisual);
 
+  nsAutoString buf;
+  buf.SetLength(aTextLength);
+  PRUnichar* buffer = (PRUnichar*)buf.get();
+  
+  ArabicShaping(aText, buf.Length(), buffer, (PRUint32 *)&newLen, !isVisual, !isVisual);
+
+  PRUnichar *source = buffer;
+  PRUnichar *target = aText;
+  for (PRInt32 i = 0; i < newLen; i++) {
+    if (*source == CH_ZWNJ || *source == CH_ZWJ) {
+      source++;
+    }
+    else {
+      *target++ = *source++;
+    }
+  }
+
+  aTextLength = target - aText;
+  *aWasTransformed = PR_TRUE;
+}
+
+void
+nsTextTransformer::DoNumericShaping(PRUnichar* aText, 
+                                    PRInt32& aTextLength, 
+                                    PRBool* aWasTransformed)
+{
+  if (aTextLength <= 0)
+    return;
+
+  PRUint32 bidiOptions;
+  mPresContext->GetBidi(&bidiOptions);
+
+  switch (GET_BIDI_OPTION_NUMERAL(bidiOptions)) {
+
+    case IBMBIDI_NUMERAL_HINDI:
+      HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_HINDI);
+      break;
+
+    case IBMBIDI_NUMERAL_ARABIC:
+      HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_ARABIC);
+      break;
+
+    case IBMBIDI_NUMERAL_REGULAR:
+
+      switch (mCharType) {
+
+        case eCharType_EuropeanNumber:
+          HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_ARABIC);
+          break;
+
+        case eCharType_ArabicNumber:
+          HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_HINDI);
+          break;
+
+        default:
+          break;
+      }
+      break;
+
+    case IBMBIDI_NUMERAL_HINDICONTEXT:
+      if (((GET_BIDI_OPTION_DIRECTION(bidiOptions)==IBMBIDI_TEXTDIRECTION_RTL) &&
+           (IS_ARABIC_DIGIT (aText[0]))) ||
+          (eCharType_ArabicNumber == mCharType))
+        HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_HINDI);
+      else if (eCharType_EuropeanNumber == mCharType)
+        HandleNumbers(aText, aTextLength, IBMBIDI_NUMERAL_ARABIC);
+      break;
+
+    default:
+      break;
+  }
+}
+#endif
+
+//----------------------------------------------------------------------
 // Self test logic for this class. This will (hopefully) make sure
 // that the forward and backward word iterator methods continue to
 // function as people change things...
@@ -1606,11 +1743,12 @@ nsTextTransformer::Init2(const nsTextFragment* aFrag,
   mFrag = aFrag;
 
   // Sanitize aStartingOffset
-  if (NS_WARN_IF_FALSE(aStartingOffset >= 0, "bad starting offset")) {
+  if (aStartingOffset < 0) {
+    NS_WARNING("bad starting offset");
     aStartingOffset = 0;
   }
-  else if (NS_WARN_IF_FALSE(aStartingOffset <= mFrag->GetLength(),
-                            "bad starting offset")) {
+  else if (aStartingOffset > mFrag->GetLength()) {
+    NS_WARNING("bad starting offset");
     aStartingOffset = mFrag->GetLength();
   }
   mOffset = aStartingOffset;
