@@ -1,4 +1,3 @@
-
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
  * The contents of this file are subject to the Netscape Public
@@ -27,7 +26,6 @@
 #include "nsHTMLContainerFrame.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsHTMLIIDs.h"
 #include "nsIComponentManager.h"
 #include "nsIStreamListener.h"
 #include "nsIURL.h"
@@ -63,7 +61,6 @@
 #define ALL_VIS    0x000F
 #define NONE_VIS   0x0000
 
-static NS_DEFINE_IID(kIFramesetFrameIID, NS_IFRAMESETFRAME_IID);
 static NS_DEFINE_IID(kLookAndFeelCID, NS_LOOKANDFEEL_CID);
 
 /*******************************************************************************
@@ -231,19 +228,29 @@ nsHTMLFramesetFrame::nsHTMLFramesetFrame()
 
 nsHTMLFramesetFrame::~nsHTMLFramesetFrame()
 {
-  if (mRowSizes) delete [] mRowSizes;
-  if (mRowSpecs) delete [] mRowSpecs;
-  if (mColSizes) delete [] mColSizes;
-  if (mColSpecs) delete [] mColSpecs;
-  if (mVerBorders) delete[] mVerBorders;
-  if (mHorBorders) delete[] mHorBorders;
+  delete [] mRowSizes;
+  delete [] mRowSpecs;
+  delete [] mColSizes;
+  delete [] mColSpecs;
+  delete[] mVerBorders;
+  delete[] mHorBorders;
+
   mRowSizes = mColSizes = nsnull;
   mRowSpecs = mColSpecs = nsnull;
-  nsCOMPtr<nsIPrefBranchInternal> prefBranch(do_QueryReferent(mPrefBranchWeakRef));
+
+  nsCOMPtr<nsIPrefBranchInternal> prefBranch =
+    do_QueryReferent(mPrefBranchWeakRef);
+
   if (prefBranch) {
-    nsresult rv = prefBranch->RemoveObserver(kFrameResizePref, this);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Can't remove frameset as pref branch observer");
+#ifdef DEBUG
+    nsresult rv =
+#endif
+    prefBranch->RemoveObserver(kFrameResizePref, this);
+
+    NS_ASSERTION(NS_SUCCEEDED(rv),
+		 "Can't remove frameset as pref branch observer");
   }
+
   mPrefBranchWeakRef = nsnull;
 }
 
@@ -252,7 +259,7 @@ nsresult nsHTMLFramesetFrame::QueryInterface(const nsIID& aIID,
 {
   if (NULL == aInstancePtr) {
     return NS_ERROR_NULL_POINTER;
-  } else if (aIID.Equals(kIFramesetFrameIID)) {
+  } else if (aIID.Equals(NS_GET_IID(nsHTMLFramesetFrame))) {
     *aInstancePtr = (void*)this;
     return NS_OK;
   } else if (aIID.Equals(NS_GET_IID(nsIObserver))) {
@@ -327,7 +334,8 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
   mTopLevelFrameset = (nsHTMLFramesetFrame*)this;
   while (parentFrame) {
     nsHTMLFramesetFrame* frameset;
-    rv = parentFrame->QueryInterface(kIFramesetFrameIID, (void**)&frameset);
+    rv = parentFrame->QueryInterface(NS_GET_IID(nsHTMLFramesetFrame),
+				     (void**)&frameset);
     if (NS_SUCCEEDED(rv)) {
       mTopLevelFrameset = frameset;
       parentFrame->GetParent((nsIFrame**)&parentFrame);
@@ -398,13 +406,13 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
       continue;
     nsCOMPtr<nsIAtom> tag;
     child->GetTag(*getter_AddRefs(tag));
-    if ((nsHTMLAtoms::frameset == tag) || (nsHTMLAtoms::frame == tag)) {
+    if (tag == nsHTMLAtoms::frameset || tag == nsHTMLAtoms::frame) {
       nsCOMPtr<nsIStyleContext> kidSC;
-      nsresult         result;
+      nsresult result;
 
       aPresContext->ResolveStyleContextFor(child, mStyleContext,
-                                           PR_FALSE, getter_AddRefs(kidSC));
-      if (nsHTMLAtoms::frameset == tag) {
+                                           getter_AddRefs(kidSC));
+      if (tag == nsHTMLAtoms::frameset) {
         result = NS_NewHTMLFramesetFrame(shell, &frame);
 
         mChildTypes[mChildCount] = FRAMESET;
@@ -446,7 +454,7 @@ nsHTMLFramesetFrame::Init(nsIPresContext*  aPresContext,
     nsHTMLFramesetBlankFrame* blankFrame = new (shell.get()) nsHTMLFramesetBlankFrame;
     nsCOMPtr<nsIStyleContext> pseudoStyleContext;
     aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::framesetBlankPseudo,
-                                               mStyleContext, PR_FALSE,
+                                               mStyleContext,
                                                getter_AddRefs(pseudoStyleContext));
     if(blankFrame)
       blankFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
@@ -635,9 +643,9 @@ PRInt32 nsHTMLFramesetFrame::GetBorderWidth(nsIPresContext* aPresContext,
   float p2t;
   aPresContext->GetScaledPixelsToTwips(&p2t);
   nsHTMLValue htmlVal;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**)&content);
-  if (nsnull != content) {
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     if (NS_CONTENT_ATTR_HAS_VALUE == (content->GetHTMLAttribute(nsHTMLAtoms::border, htmlVal))) {
       nsHTMLUnit unit = htmlVal.GetUnit();
       PRInt32 intVal = 0;
@@ -649,13 +657,12 @@ PRInt32 nsHTMLFramesetFrame::GetBorderWidth(nsIPresContext* aPresContext,
       if (intVal < 0) {
         intVal = 0;
       }
-      NS_RELEASE(content);
+
       if (forcing && intVal == 0) {
         intVal = DEFAULT_BORDER_WIDTH_PX;
       }
       return NSIntPixelsToTwips(intVal, p2t);
     }
-    NS_RELEASE(content);
   }
 
   if (mParentBorderWidth > 0 ||
@@ -699,26 +706,24 @@ nsHTMLFramesetFrame::GetDesiredSize(nsIPresContext*          aPresContext,
 nsHTMLFramesetFrame* nsHTMLFramesetFrame::GetFramesetParent(nsIFrame* aChild)
 {
   nsHTMLFramesetFrame* parent = nsnull;
-  nsIContent* content = nsnull;
-  aChild->GetContent(&content);
-  if (nsnull != content) { 
-    nsIContent* contentParent = nsnull;
-    content->GetParent(contentParent);
-    if (nsnull != contentParent) {
-      nsIHTMLContent* contentParent2 = nsnull;
-      contentParent->QueryInterface(kIHTMLContentIID, (void**)&contentParent2);
-      if (nsnull != contentParent2) {
-        nsIAtom* tag;
-        contentParent2->GetTag(tag);
-        if (nsHTMLAtoms::frameset == tag) {
-          aChild->GetParent((nsIFrame**)&parent);
-        }
-        NS_IF_RELEASE(tag);
-        NS_RELEASE(contentParent2);
+  nsCOMPtr<nsIContent> content;
+  aChild->GetContent(getter_AddRefs(content));
+
+  if (content) { 
+    nsCOMPtr<nsIContent> contentParent;
+    content->GetParent(*getter_AddRefs(contentParent));
+
+    nsCOMPtr<nsIHTMLContent> contentParent2 =
+      do_QueryInterface(contentParent);
+
+    if (contentParent2) {
+      nsCOMPtr<nsIAtom> tag;
+      contentParent2->GetTag(*getter_AddRefs(tag));
+
+      if (tag == nsHTMLAtoms::frameset) {
+        aChild->GetParent((nsIFrame**)&parent);
       }
-      NS_RELEASE(contentParent);
     }
-    NS_RELEASE(content);
   }
 
   return parent;
@@ -852,9 +857,10 @@ void nsHTMLFramesetFrame::ParseRowCol(nsIPresContext* aPresContext, nsIAtom* aAt
 {
   nsHTMLValue value;
   nsAutoString rowsCols;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**)&content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     if (NS_CONTENT_ATTR_HAS_VALUE == content->GetHTMLAttribute(aAttrType, value)) {
       if (eHTMLUnit_String == value.GetUnit()) {
         value.GetStringValue(rowsCols);
@@ -865,11 +871,9 @@ void nsHTMLFramesetFrame::ParseRowCol(nsIPresContext* aPresContext, nsIAtom* aAt
           (*aSpecs)[i] = specs[i];
         }
         delete [] specs;
-        NS_RELEASE(content);
         return;
       }
     }
-    NS_RELEASE(content);
   }
   aNumSpecs = 1; 
   *aSpecs = new nsFramesetSpec[1];
@@ -900,7 +904,7 @@ nsHTMLFramesetFrame::ParseRowColSpec(nsIPresContext* aPresContext,
   PRInt32 count = 1;
   while (commaX >= 0) {
     count++;
-    commaX = aSpec.FindChar(COMMA, PR_FALSE,commaX + 1);
+    commaX = aSpec.FindChar(COMMA, commaX + 1);
   }
 
   if (count > aMaxNumValues) {
@@ -915,7 +919,7 @@ nsHTMLFramesetFrame::ParseRowColSpec(nsIPresContext* aPresContext,
 
   for (PRInt32 i = 0; i < count; i++) {
     // Find our comma
-    commaX = aSpec.FindChar(COMMA, PR_FALSE,start);
+    commaX = aSpec.FindChar(COMMA, start);
     PRInt32 end = (commaX < 0) ? specLen : commaX;
 
     // Note: If end == start then it means that the token has no
@@ -1044,11 +1048,10 @@ nsFrameborder GetFrameBorderHelper(nsIHTMLContent* aContent)
 nsFrameborder nsHTMLFramesetFrame::GetFrameBorder() 
 {
   nsFrameborder result = eFrameborder_Notset;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**) &content);
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
   if (content) {
     result = GetFrameBorderHelper(content);
-    NS_RELEASE(content);
   }
   if (eFrameborder_Notset == result) {
     return mParentFrameborder;
@@ -1059,11 +1062,11 @@ nsFrameborder nsHTMLFramesetFrame::GetFrameBorder()
 nsFrameborder nsHTMLFramesetFrame::GetFrameBorder(nsIContent* aContent)
 {
   nsFrameborder result = eFrameborder_Notset;
-  nsIHTMLContent* content = nsnull;
-  aContent->QueryInterface(kIHTMLContentIID, (void**) &content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     result = GetFrameBorderHelper(content);
-    NS_RELEASE(content);
   }
   if (eFrameborder_Notset == result) {
     return GetFrameBorder();
@@ -1074,9 +1077,10 @@ nsFrameborder nsHTMLFramesetFrame::GetFrameBorder(nsIContent* aContent)
 nscolor nsHTMLFramesetFrame::GetBorderColor() 
 {
   nscolor result = NO_COLOR;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**)&content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     nsHTMLValue value;
     if (NS_CONTENT_ATTR_HAS_VALUE == (content->GetHTMLAttribute(nsHTMLAtoms::bordercolor, value))) {
       if ((eHTMLUnit_Color == value.GetUnit()) ||
@@ -1084,7 +1088,6 @@ nscolor nsHTMLFramesetFrame::GetBorderColor()
         result = value.GetColorValue();
       }
     }
-    NS_RELEASE(content);
   }
   if (NO_COLOR == result) {
     return mParentBorderColor;
@@ -1095,9 +1098,10 @@ nscolor nsHTMLFramesetFrame::GetBorderColor()
 nscolor nsHTMLFramesetFrame::GetBorderColor(nsIContent* aContent) 
 {
   nscolor result = NO_COLOR;
-  nsIHTMLContent* content = nsnull;
-  aContent->QueryInterface(kIHTMLContentIID, (void**)&content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(aContent));
+
+  if (content) {
     nsHTMLValue value;
     if (NS_CONTENT_ATTR_HAS_VALUE == (content->GetHTMLAttribute(nsHTMLAtoms::bordercolor, value))) {
       if ((eHTMLUnit_Color == value.GetUnit()) ||
@@ -1105,7 +1109,6 @@ nscolor nsHTMLFramesetFrame::GetBorderColor(nsIContent* aContent)
         result = value.GetColorValue();
       }
     }
-    NS_RELEASE(content);
   }
   if (NO_COLOR == result) {
     return GetBorderColor();
@@ -1213,7 +1216,7 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
         borderFrame = new (shell.get()) nsHTMLFramesetBorderFrame(borderWidth, PR_FALSE, PR_FALSE);
         nsIStyleContext* pseudoStyleContext;
         aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::horizontalFramesetBorderPseudo,
-                                                   mStyleContext, PR_FALSE,
+                                                   mStyleContext,
                                                    &pseudoStyleContext);
         borderFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
         NS_RELEASE(pseudoStyleContext);
@@ -1242,7 +1245,6 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
             nsIStyleContext* pseudoStyleContext;
             aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::verticalFramesetBorderPseudo,
                                                        mStyleContext,
-                                                       PR_FALSE,
                                                        &pseudoStyleContext);
             borderFrame->Init(aPresContext, mContent, this, pseudoStyleContext, nsnull);
             NS_RELEASE(pseudoStyleContext);
@@ -1398,6 +1400,8 @@ nsHTMLFramesetFrame::Reflow(nsIPresContext*          aPresContext,
 
   aStatus = NS_FRAME_COMPLETE;
   mDrag.UnSet();
+
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -1405,7 +1409,7 @@ PRBool
 nsHTMLFramesetFrame::ChildIsFrameset(nsIFrame* aChild) 
 {
   nsIFrame* childFrame = nsnull;
-  aChild->QueryInterface(kIFramesetFrameIID, (void**)&childFrame);
+  aChild->QueryInterface(NS_GET_IID(nsHTMLFramesetFrame), (void**)&childFrame);
   if (childFrame) {
     return PR_TRUE;
   }
@@ -1445,19 +1449,16 @@ PRBool
 nsHTMLFramesetFrame::GetNoResize(nsIFrame* aChildFrame) 
 {
   PRBool result = PR_FALSE;
-  nsIContent* content = nsnull;
-  aChildFrame->GetContent(&content);
-  if (nsnull != content) {
-    nsIHTMLContent* htmlContent = nsnull;
-    content->QueryInterface(kIHTMLContentIID, (void**)&htmlContent);
-    if (nsnull != htmlContent) {
-      nsHTMLValue value;
-      if (NS_CONTENT_ATTR_HAS_VALUE == htmlContent->GetHTMLAttribute(nsHTMLAtoms::noresize, value)) {
-        result = PR_TRUE;
-      }
-      NS_RELEASE(htmlContent);
+  nsCOMPtr<nsIContent> content;
+  aChildFrame->GetContent(getter_AddRefs(content));
+
+  nsCOMPtr<nsIHTMLContent> htmlContent(do_QueryInterface(content));
+
+  if (htmlContent) {
+    nsHTMLValue value;
+    if (NS_CONTENT_ATTR_HAS_VALUE == htmlContent->GetHTMLAttribute(nsHTMLAtoms::noresize, value)) {
+      result = PR_TRUE;
     }
-    NS_RELEASE(content);
   }
   return result;
 }
@@ -1498,9 +1499,9 @@ nsHTMLFramesetFrame::RecalculateBorderResize()
     if (child) {
       nsCOMPtr<nsIAtom> tag;
       child->GetTag(*getter_AddRefs(tag));
-      if (nsHTMLAtoms::frameset == tag) {
+      if (tag == nsHTMLAtoms::frameset) {
         childTypes[frameOrFramesetChildIndex++] = FRAMESET;
-      } else if (nsHTMLAtoms::frame == tag) {
+      } else if (tag == nsHTMLAtoms::frame) {
         childTypes[frameOrFramesetChildIndex++] = FRAME;
       }
       // Don't overflow childTypes array
