@@ -51,7 +51,6 @@
 #include "nsIAtom.h"
 #include "nsIPresContext.h"
 #include "nsIHTMLContent.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLAtoms.h"
 #include "nsIStyleContext.h"
@@ -59,6 +58,10 @@
 #include "nsStyleUtil.h"
 #include "nsFont.h"
 #include "nsCOMPtr.h"
+#ifdef ACCESSIBILITY
+#include "nsIAccessibilityService.h"
+#endif
+#include "nsIServiceManager.h"
 
 class nsLegendFrame;
 
@@ -100,6 +103,14 @@ public:
                            nsIAtom*        aListName,
                            nsIFrame*       aOldFrame,
                            nsIFrame*       aNewFrame);
+  NS_IMETHOD  GetFrameForPoint(nsIPresContext*   aPresContext,
+                               const nsPoint&    aPoint, 
+                               nsFramePaintLayer aWhichLayer,
+                               nsIFrame**        aFrame);
+#ifdef ACCESSIBILITY  
+  NS_IMETHOD  GetAccessible(nsIAccessible** aAccessible);
+#endif
+
 #ifdef DEBUG
   NS_IMETHOD GetFrameName(nsAString& aResult) const {
     return MakeFrameName(NS_LITERAL_STRING("FieldSet"), aResult);
@@ -278,25 +289,28 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
 {
   DO_GLOBAL_REFLOW_COUNT("nsFieldSetFrame", aReflowState.reason);
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
- 
+
   // Initialize OUT parameter
    aStatus = NS_FRAME_COMPLETE;
 
+  if (nsnull != aDesiredSize.maxElementSize) {
+    aDesiredSize.maxElementSize->width = 0;
+    aDesiredSize.maxElementSize->height = 0;
+  }
+  
    //------------ Handle Incremental Reflow -----------------
-   nsIFrame* incrementalChild = nsnull;
    PRBool reflowContent = PR_TRUE;
    PRBool reflowLegend = PR_TRUE;
    nsReflowReason reason = aReflowState.reason;
 
     if ( aReflowState.reason == eReflowReason_Incremental ) {
-        nsReflowType  reflowType;
-        aReflowState.reflowCommand->GetType(reflowType);
+        nsHTMLReflowCommand *command = aReflowState.path->mReflowCommand;
 
         // See if it's targeted at us
-        nsIFrame* targetFrame;    
-        aReflowState.reflowCommand->GetTarget(targetFrame);
+        if (command) {
+            nsReflowType  reflowType;
+            command->GetType(reflowType);
 
-        if (this == targetFrame) {
             switch (reflowType) {
 
               case eReflowType_StyleChanged: 
@@ -320,15 +334,18 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
                     NS_ERROR("Unexpected Reflow Type");
             }
         } else {
-             aReflowState.reflowCommand->GetNext(incrementalChild);
-
              reflowContent = PR_FALSE;
              reflowLegend = PR_FALSE;
 
-             if (incrementalChild == mLegendFrame)
-                 reflowLegend = PR_TRUE;
-             else if (incrementalChild == mContentFrame)
-                 reflowContent = PR_TRUE;       
+             nsReflowPath::iterator iter = aReflowState.path->FirstChild();
+             nsReflowPath::iterator end = aReflowState.path->EndChildren();
+
+             for ( ; iter != end; ++iter) {
+                 if (*iter == mLegendFrame)
+                     reflowLegend = PR_TRUE;
+                 else if (*iter == mContentFrame)
+                     reflowContent = PR_TRUE;
+             }
         }
     }
 
@@ -350,7 +367,7 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
 
     // availSize could have unconstrained values, don't perform any addition on them
     nsSize availSize(aReflowState.mComputedWidth, aReflowState.availableHeight);
-  
+
     // get our border and padding
     const nsMargin &borderPadding = aReflowState.mComputedBorderPadding;
     const nsMargin &padding       = aReflowState.mComputedPadding;
@@ -368,12 +385,12 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
 
         if (reflowLegend) {
           nsHTMLReflowState legendReflowState(aPresContext, aReflowState,
-                                              mLegendFrame, nsSize(NS_INTRINSICSIZE,NS_INTRINSICSIZE));
+                                              mLegendFrame, nsSize(NS_INTRINSICSIZE,NS_INTRINSICSIZE),
+                                              reason);
 
           // always give the legend as much size as it needs
           legendReflowState.mComputedWidth = NS_INTRINSICSIZE;
           legendReflowState.mComputedHeight = NS_INTRINSICSIZE;
-          legendReflowState.reason = reason;
 
           nsHTMLReflowMetrics legendDesiredSize(0,0);
 
@@ -395,7 +412,7 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
           mLegendSpace = 0;
           if (mLegendRect.height > border.top) {
               // center the border on the legend
-              mLegendSpace = mLegendRect.height - borderPadding.top;
+              mLegendSpace = mLegendRect.height - border.top;
           } else {
               mLegendRect.y = (border.top - mLegendRect.height)/2;
           }
@@ -442,12 +459,9 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
             }
 #endif
             nsHTMLReflowState kidReflowState(aPresContext, aReflowState, mContentFrame,
-                                             availSize);
-
-            kidReflowState.reason = reason;
-
-            nsHTMLReflowMetrics kidDesiredSize(0,0);
+                                             availSize, reason);
 
+            nsHTMLReflowMetrics kidDesiredSize(aDesiredSize.maxElementSize, aDesiredSize.mFlags);
             // Reflow the frame
             ReflowChild(mContentFrame, aPresContext, kidDesiredSize, kidReflowState,
                         kidReflowState.mComputedMargin.left, kidReflowState.mComputedMargin.top,
@@ -509,20 +523,26 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
     }
 
     if (mLegendFrame) {
+      // use the computed width if the inner content does not fill it
+      if (aReflowState.mComputedWidth != NS_INTRINSICSIZE && aReflowState.mComputedWidth > contentRect.width ) 
+        contentRect.width = aReflowState.mComputedWidth;
+      // if the content rect is larger then the  legend we can align the legend
       if (contentRect.width > mLegendRect.width) {
         PRInt32 align = ((nsLegendFrame*)mLegendFrame)->GetAlign();
-  
+
         switch(align) {
           case NS_STYLE_TEXT_ALIGN_RIGHT:
             mLegendRect.x = contentRect.width - mLegendRect.width + borderPadding.left;
             break;
           case NS_STYLE_TEXT_ALIGN_CENTER:
-            mLegendRect.x = contentRect.width/2 - mLegendRect.width/2 + borderPadding.left;
+            float p2t;
+            aPresContext->GetPixelsToTwips(&p2t);
+            mLegendRect.x = NSIntPixelsToTwips((nscoord) NSToIntRound((float)(contentRect.width/2 - mLegendRect.width/2 + borderPadding.left) / p2t),p2t);
         }
   
       } else {
-        contentRect.width = mLegendRect.width + borderPadding.left + borderPadding.right;
-        aDesiredSize.width = contentRect.width;
+        //otherwise make place for the legend
+        contentRect.width = mLegendRect.width;
       }
       // place the legend
       nsRect actualLegendRect(mLegendRect);
@@ -554,18 +574,7 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
        if (aDesiredSize.height < min)
            aDesiredSize.height = min;
     }
-
-    if (aReflowState.mComputedWidth == NS_INTRINSICSIZE) {
-        aDesiredSize.width = borderPadding.left + 
-                             contentRect.width +
-                             borderPadding.right;
-    } else {
-       nscoord min = borderPadding.left + borderPadding.right + mLegendRect.width;
-       aDesiredSize.width = aReflowState.mComputedWidth + borderPadding.left + borderPadding.right;
-       if (aDesiredSize.width < min)
-           aDesiredSize.width = min;
-    }
-
+    aDesiredSize.width = contentRect.width + borderPadding.left + borderPadding.right;
     aDesiredSize.ascent  = aDesiredSize.height;
     aDesiredSize.descent = 0;
     aDesiredSize.mMaximumWidth = aDesiredSize.width;
@@ -588,6 +597,7 @@ nsFieldSetFrame::Reflow(nsIPresContext*          aPresContext,
       printf("  and preferred size = %d\n", aDesiredSize.mMaximumWidth);
 
 #endif
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -671,4 +681,25 @@ nsFieldSetFrame::ReplaceFrame(nsIPresContext* aPresContext,
                                      aNewFrame);
 }
 
+NS_IMETHODIMP
+nsFieldSetFrame::GetFrameForPoint(nsIPresContext*   aPresContext,
+                     const nsPoint&    aPoint, 
+                     nsFramePaintLayer aWhichLayer,
+                     nsIFrame**        aFrame)
+{
+  // this should act like a block, so we need to override
+  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND), aFrame);
+}
 
+#ifdef ACCESSIBILITY
+NS_IMETHODIMP nsFieldSetFrame::GetAccessible(nsIAccessible** aAccessible)
+{
+  nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
+
+  if (accService) {
+    return accService->CreateHTMLGroupboxAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+  }
+
+  return NS_ERROR_FAILURE;
+}
+#endif
