@@ -36,7 +36,6 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsFileControlFrame.h"
-#include "nsFormFrame.h"
 
 
 #include "nsIElementFactory.h"
@@ -45,7 +44,6 @@
 #include "nsIAtom.h"
 #include "nsIPresContext.h"
 #include "nsIHTMLContent.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLAtoms.h"
 #include "nsIPresState.h"
 #include "nsWidgetsCID.h"
@@ -63,7 +61,6 @@
 #include "nsIDOMMouseListener.h"
 #include "nsIPresShell.h"
 #include "nsIDOMHTMLInputElement.h"
-#include "nsIStatefulFrame.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIComponentManager.h"
 #include "nsIDOMWindowInternal.h"
@@ -72,6 +69,7 @@
 #include "nsINodeInfo.h"
 #include "nsIDOMEventReceiver.h"
 #include "nsIScriptGlobalObject.h"
+#include "nsILocalFile.h"
 
 #include "nsContentCID.h"
 static NS_DEFINE_CID(kHTMLElementFactoryCID,   NS_HTML_ELEMENT_FACTORY_CID);
@@ -93,7 +91,6 @@ NS_NewFileControlFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
 
 nsFileControlFrame::nsFileControlFrame():
   mTextFrame(nsnull), 
-  mFormFrame(nsnull),
   mTextContent(nsnull),
   mCachedState(nsnull)
 {
@@ -115,10 +112,6 @@ nsFileControlFrame::~nsFileControlFrame()
     delete mCachedState;
     mCachedState = nsnull;
   }
-  if (mFormFrame) {
-    mFormFrame->RemoveFormControlFrame(*this);
-    mFormFrame = nsnull;
-  }
 }
 
 NS_IMETHODIMP
@@ -149,10 +142,16 @@ nsFileControlFrame::CreateAnonymousContent(nsIPresContext* aPresContext,
 
   if (NS_SUCCEEDED(rv)) {
     mTextContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::type, NS_LITERAL_STRING("text"), PR_FALSE);
-    if (nsFormFrame::GetDisabled(this)) {
-      nsCOMPtr<nsIDOMHTMLInputElement> textControl = do_QueryInterface(mTextContent);
-      if (textControl) {
-        textControl->SetDisabled(nsFormFrame::GetDisabled(this));
+    nsCOMPtr<nsIDOMHTMLInputElement> textControl = do_QueryInterface(mTextContent);
+    if (textControl) {
+      textControl->SetDisabled(nsFormControlHelper::GetDisabled(mContent));
+      // Initialize value when we create the content in case the value was set
+      // before we got here
+      nsCOMPtr<nsIDOMHTMLInputElement> fileContent = do_QueryInterface(mContent);
+      if (fileContent) {
+        nsAutoString value;
+        fileContent->GetValue(value);
+        textControl->SetValue(value);
       }
     }
     aChildList.AppendElement(mTextContent);
@@ -199,9 +198,6 @@ nsFileControlFrame::QueryInterface(const nsIID& aIID, void** aInstancePtr)
   } else  if (aIID.Equals(NS_GET_IID(nsIDOMMouseListener))) {
     *aInstancePtr = (void*)(nsIDOMMouseListener*) this;
     return NS_OK;
-  } else  if (aIID.Equals(NS_GET_IID(nsIStatefulFrame))) {
-    *aInstancePtr = (void*)(nsIStatefulFrame*) this;
-    return NS_OK;
   }
   return nsHTMLContainerFrame::QueryInterface(aIID, aInstancePtr);
 }
@@ -302,13 +298,12 @@ nsFileControlFrame::MouseClick(nsIDOMEvent* aMouseEvent)
 
   nsCOMPtr<nsILocalFile> currentFile = do_CreateInstance("@mozilla.org/file/local;1");
   if (currentFile && !defaultName.IsEmpty()) {
-    result = currentFile->InitWithUnicodePath(defaultName.get());
+    result = currentFile->InitWithPath(defaultName);
     if (NS_SUCCEEDED(result)) {
-      PRUnichar *leafName = nsnull;
-      currentFile->GetUnicodeLeafName(&leafName);
-      if (leafName) {
-        filePicker->SetDefaultString(leafName);
-        nsMemory::Free(leafName);
+      nsAutoString leafName;
+      currentFile->GetLeafName(leafName);
+      if (!leafName.IsEmpty()) {
+        filePicker->SetDefaultString(leafName.get());
       }
 
       // set directory
@@ -334,12 +329,10 @@ nsFileControlFrame::MouseClick(nsIDOMEvent* aMouseEvent)
   nsCOMPtr<nsILocalFile> localFile;
   result = filePicker->GetFile(getter_AddRefs(localFile));
   if (localFile) {
-    PRUnichar *nativePath = nsnull;
-    result = localFile->GetUnicodePath(&nativePath);
-    if (nativePath) {
-      nsAutoString pathName(nativePath);
-      mTextFrame->SetProperty(mPresContext, nsHTMLAtoms::value, pathName);
-      nsMemory::Free(nativePath);
+    nsAutoString unicodePath;
+    result = localFile->GetPath(unicodePath);
+    if (!unicodePath.IsEmpty()) {
+      mTextFrame->SetProperty(mPresContext, nsHTMLAtoms::value, unicodePath);
       return NS_OK;
     }
   }
@@ -358,9 +351,7 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
 
   aStatus = NS_FRAME_COMPLETE;
 
-  if (mFormFrame == nsnull && eReflowReason_Initial == aReflowState.reason) {
-    // add ourself as an nsIFormControlFrame
-    nsFormFrame::AddFormControlFrame(aPresContext, *NS_STATIC_CAST(nsIFrame*, this));
+  if (eReflowReason_Initial == aReflowState.reason) {
     mTextFrame = GetTextControlFrame(aPresContext, this);
     if (!mTextFrame) return NS_ERROR_UNEXPECTED;
     if (mCachedState) {
@@ -374,6 +365,9 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
   // except for when style is used to change its size.
   nsresult rv = nsAreaFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
   if (NS_SUCCEEDED(rv) && mTextFrame != nsnull) {
+    const nsStyleVisibility* vis;
+    GetStyleData(eStyleStruct_Visibility, (const nsStyleStruct*&)vis);
+
     nsIFrame * child;
     FirstChild(aPresContext, nsnull, &child);
     while (child == mTextFrame) {
@@ -385,34 +379,49 @@ NS_IMETHODIMP nsFileControlFrame::Reflow(nsIPresContext*          aPresContext,
       mTextFrame->GetRect(txtRect);
       child->GetRect(buttonRect);
 
-      // check to see if we must reflow just the texField
-      // because style width or height was set.
+      // check to see if we must reflow just the area frame again 
+      // in order for the text field to be the correct height
+      // reflowing just the textfield (for some reason) 
+      // messes up the button's rect
       if (txtRect.width + buttonRect.width != aDesiredSize.width ||
           txtRect.height != aDesiredSize.height) {
-
-        nsSize txtAvailSize(aDesiredSize.width - buttonRect.width, aDesiredSize.height);
+        nsSize txtAvailSize(aReflowState.availableWidth, aDesiredSize.height);
         nsHTMLReflowMetrics txtKidSize(&txtAvailSize);
-        nsHTMLReflowState   txtKidReflowState(aPresContext, aReflowState, mTextFrame, txtAvailSize);
-        txtKidReflowState.reason = eReflowReason_Resize;
-        txtKidReflowState.mComputedWidth  = txtAvailSize.width;
-        txtKidReflowState.mComputedHeight = txtAvailSize.height;
-        mTextFrame->WillReflow(aPresContext);
-        nsReflowStatus status;
-        rv = mTextFrame->Reflow(aPresContext, txtKidSize, txtKidReflowState, status);
-        if (NS_FAILED(rv)) return rv;
-        rv = mTextFrame->DidReflow(aPresContext, &txtKidReflowState, aStatus);
-        if (NS_FAILED(rv)) return rv;
+        nsHTMLReflowState   txtKidReflowState(aPresContext, aReflowState, this, txtAvailSize,
+                                              eReflowReason_Resize);
+        txtKidReflowState.mComputedHeight = aDesiredSize.height;
+        rv = nsAreaFrame::WillReflow(aPresContext);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Should have succeeded");
+        rv = nsAreaFrame::Reflow(aPresContext, txtKidSize, txtKidReflowState, aStatus);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Should have succeeded");
+        rv = nsAreaFrame::DidReflow(aPresContext, &txtKidReflowState, aStatus);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "Should have succeeded");
 
-        // now adjust the frame positions
-        buttonRect.x = aDesiredSize.width - buttonRect.width + aReflowState.mComputedBorderPadding.left;
+        // If LTR then re-calc and set the correct rect
+        if (NS_STYLE_DIRECTION_RTL != vis->mDirection) {
+          // now adjust the frame positions
+          txtRect.y      = aReflowState.mComputedBorderPadding.top;
+          txtRect.height = aDesiredSize.height;
+          mTextFrame->SetRect(aPresContext, txtRect);
+        }
+      }
+
+      // Do RTL positioning
+      // for some reason the areaframe does set the X coord of the rects correctly
+      // so we must redo them here
+      // and we must make sure the text field is the correct height
+      if (NS_STYLE_DIRECTION_RTL == vis->mDirection) {
+        buttonRect.x      = aReflowState.mComputedBorderPadding.left;
         child->SetRect(aPresContext, buttonRect);
-        txtRect.y      = aReflowState.mComputedBorderPadding.top;
-        txtRect.height = aDesiredSize.height;
-        txtRect.width  = aDesiredSize.width - buttonRect.width;
+        txtRect.x         = aDesiredSize.width - txtRect.width + aReflowState.mComputedBorderPadding.left;
+        txtRect.y         = aReflowState.mComputedBorderPadding.top;
+        txtRect.height    = aDesiredSize.height;
         mTextFrame->SetRect(aPresContext, txtRect);
       }
+
     }
   }
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -492,18 +501,16 @@ NS_IMETHODIMP
 nsFileControlFrame::GetName(nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (mContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = mContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
+
+  nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
+
+  if (formControl) {
+    nsHTMLValue value;
+    result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
+    if (NS_CONTENT_ATTR_HAS_VALUE == result) {
+      if (eHTMLUnit_String == value.GetUnit()) {
+        value.GetStringValue(*aResult);
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
@@ -523,14 +530,14 @@ nsFileControlFrame::AttributeChanged(nsIPresContext* aPresContext,
     nsCOMPtr<nsIDOMHTMLInputElement> textControl = do_QueryInterface(mTextContent);
     if (textControl)
     {
-      textControl->SetDisabled(nsFormFrame::GetDisabled(this));
+      textControl->SetDisabled(nsFormControlHelper::GetDisabled(mContent));
     }
   } else if (nsHTMLAtoms::size == aAttribute) {
     nsString value;
     if (nsnull != mTextContent && NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::size, value)) {
       mTextContent->SetAttr(kNameSpaceID_None, nsHTMLAtoms::size, value, PR_TRUE);
       if (aHint != NS_STYLE_HINT_REFLOW) {
-        nsFormFrame::StyleChangeReflow(aPresContext, this);
+        nsFormControlHelper::StyleChangeReflow(aPresContext, this);
       }
     }
   }
@@ -545,7 +552,7 @@ nsFileControlFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                                      nsIFrame** aFrame)
 {
 #ifndef DEBUG_NEWFRAME
-  if ( nsFormFrame::GetDisabled(this) && mRect.Contains(aPoint) ) {
+  if ( nsFormControlHelper::GetDisabled(mContent) && mRect.Contains(aPoint) ) {
     const nsStyleVisibility* vis = 
       (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
       
@@ -611,12 +618,12 @@ nsresult nsFileControlFrame::RequiresWidget(PRBool& aRequiresWidget)
 
 NS_IMETHODIMP nsFileControlFrame::SetProperty(nsIPresContext* aPresContext,
                                               nsIAtom* aName,
-                                              const nsAReadableString& aValue)
+                                              const nsAString& aValue)
 {
   nsresult rv = NS_OK;
   if (nsHTMLAtoms::value == aName) {
     if (mTextFrame) {
-      mTextFrame->SetTextControlFrameState(aValue);                                         
+      mTextFrame->SetValue(aValue);
     } else {
       if (mCachedState) delete mCachedState;
       mCachedState = new nsString(aValue);
@@ -626,13 +633,13 @@ NS_IMETHODIMP nsFileControlFrame::SetProperty(nsIPresContext* aPresContext,
   return rv;
 }      
 
-NS_IMETHODIMP nsFileControlFrame::GetProperty(nsIAtom* aName, nsAWritableString& aValue)
+NS_IMETHODIMP nsFileControlFrame::GetProperty(nsIAtom* aName, nsAString& aValue)
 {
   aValue.Truncate();  // initialize out param
 
   if (nsHTMLAtoms::value == aName) {
     if (mTextFrame) {
-      mTextFrame->GetTextControlFrameState(aValue);
+      mTextFrame->GetValue(aValue, PR_FALSE);
     }
     else if (mCachedState) {
       aValue.Assign(*mCachedState);
@@ -661,54 +668,6 @@ nsFileControlFrame::Paint(nsIPresContext*      aPresContext,
   return nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
 }
 
-//----------------------------------------------------------------------
-// nsIStatefulFrame
-//----------------------------------------------------------------------
-NS_IMETHODIMP
-nsFileControlFrame::SaveState(nsIPresContext* aPresContext, nsIPresState** aState)
-{
-  NS_ENSURE_ARG_POINTER(aState);
-
-  // Don't save state before we are initialized
-  if (!mTextFrame && !mCachedState) {
-    return NS_OK;
-  }
-
-  // Get the value string
-  nsAutoString stateString;
-  nsresult res = GetProperty(nsHTMLAtoms::value, stateString);
-  NS_ENSURE_SUCCESS(res, res);
-
-  // Compare to default value, and only save if needed (Bug 62713)
-  nsAutoString defaultStateString;
-  nsCOMPtr<nsIDOMHTMLInputElement> formControl(do_QueryInterface(mContent));
-  if (formControl) {
-    formControl->GetDefaultValue(defaultStateString);
-  }
-
-  if (! stateString.Equals(defaultStateString)) {
-
-    // Construct a pres state and store value in it.
-    res = NS_NewPresState(aState);
-    NS_ENSURE_SUCCESS(res, res);
-    res = (*aState)->SetStateProperty(NS_LITERAL_STRING("value"), stateString);
-  }
-
-  return res;
-}
-
-NS_IMETHODIMP
-nsFileControlFrame::RestoreState(nsIPresContext* aPresContext, nsIPresState* aState)
-{
-  NS_ENSURE_ARG_POINTER(aState);
-
-  nsAutoString string;
-  aState->GetStateProperty(NS_LITERAL_STRING("value"), string);
-  SetProperty(aPresContext, nsHTMLAtoms::value, string);
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsFileControlFrame::OnContentReset()
 {
