@@ -44,7 +44,6 @@
 #include "nsIStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
-#include "nsHTMLIIDs.h"
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsCoord.h"
@@ -63,7 +62,6 @@
 #include "nsStyleConsts.h"
 #include "nsUnitConversion.h"
 #include "nsStyleUtil.h"
-#include "nsFormFrame.h"
 #include "nsIContent.h"
 #include "nsStyleUtil.h"
 #include "nsINameSpaceManager.h"
@@ -113,7 +111,6 @@ nsFormControlFrame::nsFormControlFrame()
   : nsLeafFrame()
 {
   mDidInit        = PR_FALSE;
-  mFormFrame      = nsnull;
   mSuggestedWidth = NS_FORMSIZE_NOTSET;
   mSuggestedHeight = NS_FORMSIZE_NOTSET;
   mPresContext    = nsnull;
@@ -127,17 +124,13 @@ nsFormControlFrame::nsFormControlFrame()
 
 nsFormControlFrame::~nsFormControlFrame()
 {
-  if (mFormFrame) {
-    mFormFrame->RemoveFormControlFrame(*this);
-    // This method only removes from radio lists if we are a radio input
-    mFormFrame->RemoveRadioControlFrame(this);
-    mFormFrame = nsnull;
-  }
 }
 
 NS_IMETHODIMP
 nsFormControlFrame::Destroy(nsIPresContext *aPresContext)
 {
+  // XXXldb Do we really need to do this?  Shouldn't only those frames
+  // that use it do it?
   nsFormControlFrame::RegUnRegAccessKey(aPresContext, NS_STATIC_CAST(nsIFrame*, this), PR_FALSE);
   return nsLeafFrame::Destroy(aPresContext);
 }
@@ -553,11 +546,6 @@ nsFormControlFrame::Reflow(nsIPresContext*          aPresContext,
     mDidInit = PR_TRUE;
   }
 
-  // add ourself as an nsIFormControlFrame
-  if (!mFormFrame && (eReflowReason_Initial == aReflowState.reason)) {
-    nsFormFrame::AddFormControlFrame(aPresContext, *NS_STATIC_CAST(nsIFrame*, this));
-  }
-
 #if 0
   nsresult skiprv = SkipResizeReflow(mCacheSize, mCachedMaxElementSize, aPresContext, 
                                      aDesiredSize, aReflowState, aStatus);
@@ -570,6 +558,7 @@ nsFormControlFrame::Reflow(nsIPresContext*          aPresContext,
 
   aStatus = NS_FRAME_COMPLETE;
   SetupCachedSizes(mCacheSize, mCachedMaxElementSize, aDesiredSize);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -594,10 +583,9 @@ nsFormControlFrame::RegUnRegAccessKey(nsIPresContext* aPresContext, nsIFrame * a
     nsCOMPtr<nsIContent> content;
     if (NS_SUCCEEDED(aFrame->GetContent(getter_AddRefs(content)))) {
 #if 1
-      PRInt32 nameSpaceID;
-      content->GetNameSpaceID(nameSpaceID);
       nsAutoString resultValue;
-      rv = content->GetAttr(nameSpaceID, nsHTMLAtoms::accesskey, accessKey);
+      rv = content->GetAttr(kNameSpaceID_None,
+                            nsHTMLAtoms::accesskey, accessKey);
 #else
       nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(content));
       if (inputElement) {
@@ -630,10 +618,12 @@ nsFormControlFrame::RegUnRegAccessKey(nsIPresContext* aPresContext, nsIFrame * a
   if (NS_CONTENT_ATTR_NOT_THERE != rv) {
     nsCOMPtr<nsIEventStateManager> stateManager;
     if (NS_SUCCEEDED(aPresContext->GetEventStateManager(getter_AddRefs(stateManager)))) {
+      nsCOMPtr<nsIContent> content;
+      aFrame->GetContent(getter_AddRefs(content));
       if (aDoReg) {
-        return stateManager->RegisterAccessKey(aFrame, nsnull, (PRUint32)accessKey.First());
+        return stateManager->RegisterAccessKey(content, (PRUint32)accessKey.First());
       } else {
-        return stateManager->UnregisterAccessKey(aFrame, nsnull, (PRUint32)accessKey.First());
+        return stateManager->UnregisterAccessKey(content, (PRUint32)accessKey.First());
       }
     }
   }
@@ -687,15 +677,15 @@ nsFormControlFrame::GetMaxLength(PRInt32* aSize)
 {
   *aSize = -1;
   nsresult result = NS_CONTENT_ATTR_NOT_THERE;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**) &content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     nsHTMLValue value;
     result = content->GetHTMLAttribute(nsHTMLAtoms::maxlength, value);
     if (eHTMLUnit_Integer == value.GetUnit()) { 
       *aSize = value.GetIntValue();
     }
-    NS_RELEASE(content);
   }
   return result;
 }
@@ -705,15 +695,15 @@ nsFormControlFrame::GetSizeFromContent(PRInt32* aSize) const
 {
   *aSize = -1;
   nsresult result = NS_CONTENT_ATTR_NOT_THERE;
-  nsIHTMLContent* content = nsnull;
-  mContent->QueryInterface(kIHTMLContentIID, (void**) &content);
-  if (nsnull != content) {
+
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(mContent));
+
+  if (content) {
     nsHTMLValue value;
     result = content->GetHTMLAttribute(nsHTMLAtoms::size, value);
     if (eHTMLUnit_Integer == value.GetUnit()) { 
       *aSize = value.GetIntValue();
     }
-    NS_RELEASE(content);
   }
   return result;
 }
@@ -737,18 +727,16 @@ NS_IMETHODIMP
 nsFormControlFrame::GetName(nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (mContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = mContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
+
+  nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
+
+  if (formControl) {
+    nsHTMLValue value;
+    result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
+    if (NS_CONTENT_ATTR_HAS_VALUE == result) {
+      if (eHTMLUnit_String == value.GetUnit()) {
+        value.GetStringValue(*aResult);
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
@@ -759,18 +747,16 @@ NS_IMETHODIMP
 nsFormControlFrame::GetValue(nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (mContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = mContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::value, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
+
+  nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
+
+  if (formControl) {
+    nsHTMLValue value;
+    result = formControl->GetHTMLAttribute(nsHTMLAtoms::value, value);
+    if (NS_CONTENT_ATTR_HAS_VALUE == result) {
+      if (eHTMLUnit_String == value.GetUnit()) {
+        value.GetStringValue(*aResult);
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
@@ -857,11 +843,11 @@ nsFormControlFrame::GetFont(nsIPresContext* aPresContext,
 
 nsresult
 nsFormControlFrame::GetDefaultCheckState(PRBool *aState)
-{	nsresult res = NS_OK;
-  nsIDOMHTMLInputElement* inputElement;
-  if (NS_OK == mContent->QueryInterface(NS_GET_IID(nsIDOMHTMLInputElement), (void**)&inputElement)) {
+{
+  nsresult res = NS_OK;
+  nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(mContent);
+  if (inputElement) {
     res = inputElement->GetDefaultChecked(aState);
-    NS_RELEASE(inputElement);
   }
 	return res;
 }
@@ -870,10 +856,9 @@ nsresult
 nsFormControlFrame::SetDefaultCheckState(PRBool aState)
 {
 	nsresult res = NS_OK;
-  nsIDOMHTMLInputElement* inputElement;
-  if (NS_OK == mContent->QueryInterface(NS_GET_IID(nsIDOMHTMLInputElement), (void**)&inputElement)) {
+  nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(mContent);
+  if (inputElement) {
     res = inputElement->SetDefaultChecked(aState);
-    NS_RELEASE(inputElement);
   }
 	return res;
 }
@@ -882,10 +867,9 @@ nsresult
 nsFormControlFrame::GetCurrentCheckState(PRBool *aState)
 {
 	nsresult res = NS_OK;
-  nsIDOMHTMLInputElement* inputElement;
-  if (NS_OK == mContent->QueryInterface(NS_GET_IID(nsIDOMHTMLInputElement), (void**)&inputElement)) {
+  nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(mContent);
+  if (inputElement) {
     res = inputElement->GetChecked(aState);
-    NS_RELEASE(inputElement);
   }
 	return res;
 }
@@ -894,22 +878,21 @@ nsresult
 nsFormControlFrame::SetCurrentCheckState(PRBool aState)
 {
 	nsresult res = NS_OK;
-  nsIDOMHTMLInputElement* inputElement;
-  if (NS_OK == mContent->QueryInterface(NS_GET_IID(nsIDOMHTMLInputElement), (void**)&inputElement)) {
+  nsCOMPtr<nsIDOMHTMLInputElement> inputElement = do_QueryInterface(mContent);
+  if (inputElement) {
     inputElement->SetChecked(aState); 
-   NS_RELEASE(inputElement);
   }
 	return res;
 }
 
 NS_IMETHODIMP
-nsFormControlFrame::SetProperty(nsIPresContext* aPresContext, nsIAtom* aName, const nsAReadableString& aValue)
+nsFormControlFrame::SetProperty(nsIPresContext* aPresContext, nsIAtom* aName, const nsAString& aValue)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFormControlFrame::GetProperty(nsIAtom* aName, nsAWritableString& aValue)
+nsFormControlFrame::GetProperty(nsIAtom* aName, nsAString& aValue)
 {
   aValue.Truncate();
   return NS_OK;
@@ -981,23 +964,32 @@ nsFormControlFrame::GetAbsoluteFramePosition(nsIPresContext* aPresContext,
   float p2t;
   aPresContext->GetTwipsToPixels(&t2p);
   aPresContext->GetPixelsToTwips(&p2t);
-  
+
    // Add in frame's offset from it it's containing view
   nsIView *containingView = nsnull;
   nsPoint offset;
   rv = aFrame->GetOffsetFromView(aPresContext, offset, &containingView);
+
   if (NS_SUCCEEDED(rv) && (nsnull != containingView)) {
     aAbsoluteTwipsRect.x += offset.x;
     aAbsoluteTwipsRect.y += offset.y;
 
     nsPoint viewOffset;
     containingView->GetPosition(&viewOffset.x, &viewOffset.y);
+
     nsIView * parent;
     containingView->GetParent(parent);
 
     // if we don't have a parent view then 
     // check to see if we have a widget and adjust our offset for the widget
     if (parent == nsnull) {
+      // account for space above and to the left of the containingView origin.
+      // the widget is aligned with containingView's bounds, not its origin
+      nsRect bounds;
+      containingView->GetBounds(bounds);
+      aAbsoluteTwipsRect.x += viewOffset.x - bounds.x;
+      aAbsoluteTwipsRect.y += viewOffset.y - bounds.y;
+
       nsIWidget * widget;
       containingView->GetWidget(widget);
       if (nsnull != widget) {
@@ -1012,7 +1004,6 @@ nsFormControlFrame::GetAbsoluteFramePosition(nsIPresContext* aPresContext,
       }
       rv = NS_OK;
     } else {
-
       while (nsnull != parent) {
         nsPoint po;
         parent->GetPosition(&po.x, &po.y);
@@ -1029,6 +1020,13 @@ nsFormControlFrame::GetAbsoluteFramePosition(nsIPresContext* aPresContext,
         nsIWidget * widget;
         parent->GetWidget(widget);
         if (nsnull != widget) {
+          // account for space above and to the left of the containingView origin.
+          // the widget is aligned with containingView's bounds, not its origin
+          nsRect bounds;
+          parent->GetBounds(bounds);
+          aAbsoluteTwipsRect.x += po.x - bounds.x;
+          aAbsoluteTwipsRect.y += po.y - bounds.y;
+
           // Add in the absolute offset of the widget.
           nsRect absBounds;
           nsRect lc;
