@@ -44,7 +44,6 @@
 #include "nsIStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
-#include "nsHTMLIIDs.h"
 #include "nsIView.h"
 #include "nsIViewManager.h"
 #include "nsCoord.h"
@@ -63,7 +62,6 @@
 #include "nsStyleConsts.h"
 #include "nsUnitConversion.h"
 #include "nsStyleUtil.h"
-#include "nsFormFrame.h"
 #include "nsIContent.h"
 #include "nsStyleUtil.h"
 #include "nsINameSpaceManager.h"
@@ -90,11 +88,6 @@ static NS_DEFINE_CID(kStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
                                               // it a synonym for "soft" 
 #define kTextControl_Wrap_Off  "OFF"
 
-// Temporary - Test of full time Standard mode for forms (Bug 91602)
-#include "nsIPref.h"
-PRPackedBool nsFormControlHelper::mCompatFirstTime = PR_TRUE;
-PRPackedBool nsFormControlHelper::mUseEitherMode  = PR_FALSE;
-
 
 MOZ_DECL_CTOR_COUNTER(nsFormControlHelper)
 
@@ -141,13 +134,13 @@ void nsFormControlHelper::PlatformToDOMLineBreaks(nsString &aString)
   aString.ReplaceSubstring(NS_LITERAL_STRING("\r").get(), NS_LITERAL_STRING("\n").get());
 }
 
-PRBool nsFormControlHelper::GetBool(const nsAReadableString& aValue)
+PRBool nsFormControlHelper::GetBool(const nsAString& aValue)
 {
   return aValue.Equals(NS_STRING_TRUE);
 }
 
 void nsFormControlHelper::GetBoolString(const PRBool aValue,
-                                        nsAWritableString& aResult)
+                                        nsAString& aResult)
 {
   if (aValue)
     aResult.Assign(NS_STRING_TRUE); 
@@ -156,22 +149,6 @@ void nsFormControlHelper::GetBoolString(const PRBool aValue,
 }
 
 
-nsCompatibility
-nsFormControlHelper::GetRepChars(nsIPresContext* aPresContext, char& char1, char& char2) 
-{
-  nsCompatibility mode;
-  nsFormControlHelper::GetFormCompatibilityMode(aPresContext, mode);
-  if (eCompatibility_Standard == mode) {
-    char1 = 'W';
-    char2 = 'w';
-    return eCompatibility_Standard;
-  } else {
-    char1 = 'W';
-    char2 = 'w';
-    return eCompatibility_NavQuirks;
-  }
-}
-
 nsresult nsFormControlHelper::GetFrameFontFM(nsIPresContext* aPresContext, 
                                          nsIFormControlFrame * aFrame,
                                          nsIFontMetrics** aFontMet)
@@ -195,15 +172,14 @@ nsFormControlHelper::GetWrapProperty(nsIContent * aContent, nsString &aOutValue)
 {
   aOutValue.SetLength(0);
   nsresult result = NS_CONTENT_ATTR_NOT_THERE;
-  nsIHTMLContent* content = nsnull;
-  aContent->QueryInterface(kIHTMLContentIID, (void**) &content);
-  if (nsnull != content) {
+  nsCOMPtr<nsIHTMLContent> content(do_QueryInterface(aContent));
+
+  if (content) {
     nsHTMLValue value;
     result = content->GetHTMLAttribute(nsHTMLAtoms::wrap, value);
     if (eHTMLUnit_String == value.GetUnit()) { 
       value.GetStringValue(aOutValue);
     }
-    NS_RELEASE(content);
   }
   return result;
 }
@@ -219,18 +195,14 @@ nsFormControlHelper::GetWrapPropertyEnum(nsIContent * aContent, nsHTMLTextWrap&
 
   if (NS_CONTENT_ATTR_NOT_THERE != result) {
 
-    nsAutoString wrapHard; wrapHard.AssignWithConversion(kTextControl_Wrap_Hard);
-    nsAutoString wrapPhysical; wrapPhysical.AssignWithConversion(kTextControl_Wrap_Physical);
-    if (wrap.EqualsIgnoreCase(wrapHard)) {
+    if (wrap.EqualsIgnoreCase(kTextControl_Wrap_Hard)) {
       aWrapProp = eHTMLTextWrap_Hard;
       return result;
     }
 
-    nsAutoString wrapSoft; wrapSoft.AssignWithConversion(kTextControl_Wrap_Soft);
-    nsAutoString wrapVirtual; wrapVirtual.AssignWithConversion(kTextControl_Wrap_Virtual);
-    if (wrap.EqualsIgnoreCase(wrapSoft) ||
-        wrap.EqualsIgnoreCase(wrapVirtual) ||
-        wrap.EqualsIgnoreCase(wrapPhysical)) {
+    if (wrap.EqualsIgnoreCase(kTextControl_Wrap_Soft) ||
+        wrap.EqualsIgnoreCase(kTextControl_Wrap_Virtual) ||
+        wrap.EqualsIgnoreCase(kTextControl_Wrap_Physical)) {
       aWrapProp = eHTMLTextWrap_Soft;
       return result;
     }
@@ -263,7 +235,7 @@ nsFormControlHelper::CalcNavQuirkSizing(nsIPresContext*      aPresContext,
   maxCharWidth = NSToCoordRound(maxCharWidth * t2p);
 
   char char1, char2;
-  GetRepChars(aPresContext, char1, char2);
+  GetRepChars(char1, char2);
 
   nscoord char1Width, char2Width;
   aRendContext->GetWidth(char1, char1Width);
@@ -383,16 +355,12 @@ nsFormControlHelper::GetTextSize(nsIPresContext* aPresContext, nsIFormControlFra
   }
 
   char char1, char2;
-  nsCompatibility mode = GetRepChars(aPresContext, char1, char2);
+  GetRepChars(char1, char2);
   nscoord char1Width, char2Width;
   aRendContext->GetWidth(char1, char1Width);
   aRendContext->GetWidth(char2, char2Width);
 
-  if (eCompatibility_Standard == mode) {
-    return ((char1Width + char2Width) / 2) + 1;
-  } else {
-    return (char1Width + char2Width) / 2;
-  }
+  return ((char1Width + char2Width) / 2) + 1;
 }  
   
 nscoord
@@ -402,13 +370,13 @@ nsFormControlHelper::GetTextSize(nsIPresContext* aPresContext, nsIFormControlFra
 {
   nsAutoString val;
   char char1, char2;
-  GetRepChars(aPresContext, char1, char2);
+  GetRepChars(char1, char2);
   int i;
   for (i = 0; i < aNumChars; i+=2) {
-    val.AppendWithConversion(char1);  
+    val.Append(PRUnichar(char1));
   }
   for (i = 1; i < aNumChars; i+=2) {
-    val.AppendWithConversion(char2);  
+    val.Append(PRUnichar(char2));
   }
   return GetTextSize(aPresContext, aFrame, val, aSize, aRendContext);
 }
@@ -433,21 +401,20 @@ nsFormControlHelper::CalculateSize (nsIPresContext*       aPresContext,
   aDesiredSize.width  = CSS_NOTSET;
   aDesiredSize.height = CSS_NOTSET;
 
-  nsIContent* iContent = nsnull;
-  aFrame->GetFormContent((nsIContent*&) iContent);
-  if (!iContent) {
-    return 0;
-  }
-  nsIHTMLContent* hContent = nsnull;
-  nsresult result = iContent->QueryInterface(kIHTMLContentIID, (void**)&hContent);
-  if ((NS_OK != result) || !hContent) {
-    NS_RELEASE(iContent);
+  nsCOMPtr<nsIContent> iContent;
+  aFrame->GetFormContent(*getter_AddRefs(iContent));
+
+  nsCOMPtr<nsIHTMLContent> hContent(do_QueryInterface(iContent));
+
+  if (!hContent) {
     return 0;
   }
+
   nsAutoString valAttr;
   nsresult valStatus = NS_CONTENT_ATTR_NOT_THERE;
   if (nsnull != aSpec.mColValueAttr) {
-    valStatus = hContent->GetAttr(kNameSpaceID_HTML, aSpec.mColValueAttr, valAttr);
+    valStatus = hContent->GetAttr(kNameSpaceID_None, aSpec.mColValueAttr,
+                                  valAttr);
   }
   nsHTMLValue colAttr;
   nsresult colStatus = NS_CONTENT_ATTR_NOT_THERE;
@@ -476,9 +443,6 @@ nsFormControlHelper::CalculateSize (nsIPresContext*       aPresContext,
   }
 #endif
 
-  nsCompatibility qMode;
-  nsFormControlHelper::GetFormCompatibilityMode(aPresContext, qMode);
-
   // determine the width, char height, row height
   if (NS_CONTENT_ATTR_HAS_VALUE == colStatus) {  // col attr will provide width
     PRInt32 col = ((colAttr.GetUnit() == eHTMLUnit_Pixel) ? colAttr.GetPixelValue() : colAttr.GetIntValue());
@@ -490,23 +454,9 @@ nsFormControlHelper::CalculateSize (nsIPresContext*       aPresContext,
       aDesiredSize.width = NSIntPixelsToTwips(col, p2t);
     } else {
       col = (col <= 0) ? 1 : col; // XXX why a default of 1 char, why hide it
-      if (eCompatibility_NavQuirks == qMode) {
-        nsCOMPtr<nsIFontMetrics> fontMet;
-        nsresult res = GetFrameFontFM(aPresContext, aFrame, getter_AddRefs(fontMet));
-        if (NS_SUCCEEDED(res) && fontMet) {
-          aRendContext->SetFont(fontMet);
-          aSpec.mColDefaultSize = col;
-          charWidth = CalcNavQuirkSizing(aPresContext, aRendContext, fontMet, 
-                                         aFrame, aSpec, aDesiredSize);
-        } else {
-          charWidth = GetTextSize(aPresContext, aFrame, col, aDesiredSize, aRendContext);
-        }
-      }
     }
-    if (eCompatibility_Standard == qMode) {
-      if (aSpec.mColSizeAttrInPixels) {
-        aWidthExplicit = PR_TRUE;
-      }
+    if (aSpec.mColSizeAttrInPixels) {
+      aWidthExplicit = PR_TRUE;
     }
     aMinSize.width = aDesiredSize.width;
   } else {
@@ -519,22 +469,7 @@ nsFormControlHelper::CalculateSize (nsIPresContext*       aPresContext,
       charWidth = GetTextSize(aPresContext, aFrame, 1, aDesiredSize, aRendContext);
       aDesiredSize.width = aSpec.mColDefaultSize;
     } else  {                                     // use default width in num characters
-      if (eCompatibility_NavQuirks == qMode) {
-        nsCOMPtr<nsIFontMetrics> fontMet;
-        nsresult res = GetFrameFontFM(aPresContext, aFrame, getter_AddRefs(fontMet));
-        if (NS_SUCCEEDED(res) && fontMet) {
-          aRendContext->SetFont(fontMet);
-          // this passes in a 
-          charWidth = CalcNavQuirkSizing(aPresContext, aRendContext, fontMet, 
-                                         aFrame, aSpec, aDesiredSize);
-        } else {
-          NS_ASSERTION(fontMet, "Couldn't get Font Metrics"); 
-          aDesiredSize.width = 300;  // arbitrary values
-          aDesiredSize.width = 1500;
-        }
-      } else {
-        charWidth = GetTextSize(aPresContext, aFrame, aSpec.mColDefaultSize, aDesiredSize, aRendContext); 
-      }
+      charWidth = GetTextSize(aPresContext, aFrame, aSpec.mColDefaultSize, aDesiredSize, aRendContext); 
     }
     aMinSize.width = aDesiredSize.width;
     if (CSS_NOTSET != aCSSSize.width) {  // css provides width
@@ -549,41 +484,35 @@ nsFormControlHelper::CalculateSize (nsIPresContext*       aPresContext,
   aMinSize.height = aDesiredSize.height;
 
   // determine the height
-  if (eCompatibility_Standard == qMode) {
-    nsHTMLValue rowAttr;
-    nsresult rowStatus = NS_CONTENT_ATTR_NOT_THERE;
-    if (nsnull != aSpec.mRowSizeAttr) {
-      rowStatus = hContent->GetHTMLAttribute(aSpec.mRowSizeAttr, rowAttr);
-    }
-    if (NS_CONTENT_ATTR_HAS_VALUE == rowStatus) { // row attr will provide height
-      PRInt32 rowAttrInt = ((rowAttr.GetUnit() == eHTMLUnit_Pixel) 
-                              ? rowAttr.GetPixelValue() : rowAttr.GetIntValue());
-      numRows = (rowAttrInt > 0) ? rowAttrInt : 1;
-      aDesiredSize.height = aDesiredSize.height * numRows;
-    } else {
-      aDesiredSize.height = aDesiredSize.height * aSpec.mRowDefaultSize;
-      if (CSS_NOTSET != aCSSSize.height) {  // css provides height
-        NS_ASSERTION(aCSSSize.height > 0, "form control's computed height is <= 0"); 
-        if (NS_INTRINSICSIZE != aCSSSize.height) {
-          aDesiredSize.height = PR_MAX(aDesiredSize.height,aCSSSize.height);
-          aHeightExplicit = PR_TRUE;
-        }
+  nsHTMLValue rowAttr;
+  nsresult rowStatus = NS_CONTENT_ATTR_NOT_THERE;
+  if (nsnull != aSpec.mRowSizeAttr) {
+    rowStatus = hContent->GetHTMLAttribute(aSpec.mRowSizeAttr, rowAttr);
+  }
+  if (NS_CONTENT_ATTR_HAS_VALUE == rowStatus) { // row attr will provide height
+    PRInt32 rowAttrInt = ((rowAttr.GetUnit() == eHTMLUnit_Pixel) 
+                            ? rowAttr.GetPixelValue() : rowAttr.GetIntValue());
+    numRows = (rowAttrInt > 0) ? rowAttrInt : 1;
+    aDesiredSize.height = aDesiredSize.height * numRows;
+  } else {
+    aDesiredSize.height = aDesiredSize.height * aSpec.mRowDefaultSize;
+    if (CSS_NOTSET != aCSSSize.height) {  // css provides height
+      NS_ASSERTION(aCSSSize.height > 0, "form control's computed height is <= 0"); 
+      if (NS_INTRINSICSIZE != aCSSSize.height) {
+        aDesiredSize.height = PR_MAX(aDesiredSize.height,aCSSSize.height);
+        aHeightExplicit = PR_TRUE;
       }
     }
   }
 
-  NS_RELEASE(hContent);
   if (ATTR_NOTSET == numRows) {
     numRows = (aRowHeight > 0) ? (aDesiredSize.height / aRowHeight) : 0;
   }
 
-  NS_RELEASE(iContent);
-
   return numRows;
 }
 
 
-// this handles all of the input types rather than having them do it.
 nsresult  
 nsFormControlHelper::GetFont(nsIFormControlFrame * aFormFrame,
                              nsIPresContext*       aPresContext, 
@@ -592,69 +521,7 @@ nsFormControlHelper::GetFont(nsIFormControlFrame * aFormFrame,
 {
   const nsStyleFont* styleFont = (const nsStyleFont*)aStyleContext->GetStyleData(eStyleStruct_Font);
 
-  nsWidgetRendering m;
-  aPresContext->GetWidgetRenderingMode(&m);
-
-  // only add in padding if we are not Gfx, excluding the text widgets
-  PRBool requiresWidget = PR_FALSE;
-  aFormFrame->RequiresWidget(requiresWidget);
-
-  PRInt32 type;
-  aFormFrame->GetType(&type);
-
-  if (type!=NS_FORM_INPUT_TEXT &&
-      type!=NS_FORM_TEXTAREA   &&
-      type!=NS_FORM_INPUT_PASSWORD) {
-    if (PR_TRUE != requiresWidget && eWidgetRendering_Gfx == m) {
-      aFont = &styleFont->mFont;
-      return NS_OK;
-    }
-  }
-
-  nsCompatibility mode;
-  nsFormControlHelper::GetFormCompatibilityMode(aPresContext, mode);
-
-  if (eCompatibility_Standard == mode) {
-    aFont = &styleFont->mFont;
-    return NS_OK;
-  }
-
-  switch (type) {
-    case NS_FORM_INPUT_TEXT:
-    case NS_FORM_TEXTAREA:
-    case NS_FORM_INPUT_PASSWORD:
-      aFont = &styleFont->mFont;
-      break;
-    case NS_FORM_INPUT_BUTTON:
-    case NS_FORM_INPUT_SUBMIT:
-    case NS_FORM_INPUT_RESET:
-    case NS_FORM_SELECT:
-      NS_ASSERTION(0, "getting the font here has been predicated");
-#if 0
-      if ((styleFont->mFlags & NS_STYLE_FONT_FACE_EXPLICIT) || 
-          (styleFont->mFlags & NS_STYLE_FONT_SIZE_EXPLICIT)) {
-        aFont = styleFont->mFixedFont;
-        aFont.weight = NS_FONT_WEIGHT_NORMAL;  // always normal weight
-        aFont.size = styleFont->mFont.size;    // normal font size
-        if (0 == (styleFont->mFlags & NS_STYLE_FONT_FACE_EXPLICIT)) {
-          aFont.name = "Arial";  // XXX windows specific font
-        }
-      } else {
-        // use arial, scaled down one HTML size
-        // italics, decoration & variant(?) get used
-        aFont = styleFont->mFont;
-        aFont.name = "Arial";  // XXX windows specific font
-        aFont.weight = NS_FONT_WEIGHT_NORMAL; 
-        const nsFont& normal = aPresContext->GetDefaultFontDeprecated();
-        PRInt32 scaler;
-        aPresContext->GetFontScaler(&scaler);
-        float scaleFactor = nsStyleUtil::GetScalingFactor(scaler);
-        PRInt32 fontIndex = nsStyleUtil::FindNextSmallerFontSize(aFont.size, (PRInt32)normal.size, scaleFactor, aPresContext);
-        aFont.size = nsStyleUtil::CalcFontPointSize(fontIndex, (PRInt32)normal.size, scaleFactor, aPresContext);
-      }
-#endif
-      break;
-  }
+  aFont = &styleFont->mFont;
   return NS_OK;
 }
 
@@ -829,10 +696,10 @@ nsresult
 nsFormControlHelper::GetName(nsIContent* aContent, nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (nsnull != aContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = aContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
+  if (aContent) {
+    nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(aContent));
+
+    if (formControl) {
       nsHTMLValue value;
       result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
       if (NS_CONTENT_ATTR_HAS_VALUE == result) {
@@ -840,53 +707,12 @@ nsFormControlHelper::GetName(nsIContent* aContent, nsAString* aResult)
           value.GetStringValue(*aResult);
         }
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
 }
 
 
-nsresult
-nsFormControlHelper::GetValue(nsIContent* aContent, nsAString* aResult)
-{
-  nsresult result = NS_FORM_NOTOK;
-  if (nsnull != aContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = aContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::value, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
-      }
-      NS_RELEASE(formControl);
-    }
-  }
-  return result;
-}
-
-nsresult
-nsFormControlHelper::GetInputElementValue(nsIContent* aContent, nsString* aText, PRBool aInitialValue)
-{
-  nsresult result = NS_OK;
-  nsIDOMHTMLInputElement* inputElem = nsnull;
-  result = aContent->QueryInterface(NS_GET_IID(nsIDOMHTMLInputElement), (void**)&inputElem);
-  if ((NS_OK == result) && inputElem) {
-    if (PR_TRUE == aInitialValue) {
-      result = inputElem->GetDefaultValue(*aText);
-    }
-    else {
-      result = inputElem->GetValue(*aText);
-    }
-
-    NS_RELEASE(inputElem);
-  }
-  return result;
-}
-
 //----------------------------------------------------------------------------------
 // Return localised string for resource string (e.g. "Submit" -> "Submit Query")
 // This code is derived from nsBookmarksService::Init() and cookie_Localize()
@@ -918,116 +744,6 @@ nsFormControlHelper::GetLocalizedString(const char * aPropFileName, const PRUnic
   return rv;
 }
 
-// Return value of disabled attribute or PR_FALSE if none set
-nsresult
-nsFormControlHelper::GetDisabled(nsIContent* aContent, PRBool* oIsDisabled)
-{
-  nsCOMPtr<nsIHTMLContent> formControl = do_QueryInterface(aContent);
-  nsHTMLValue value;
-  nsresult result = formControl->GetHTMLAttribute(nsHTMLAtoms::disabled, value);
-  if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-    if (eHTMLUnit_Empty == value.GetUnit()) {
-      *oIsDisabled = PR_TRUE;
-    } else {
-      *oIsDisabled = PR_FALSE;
-    }
-  } else {
-    *oIsDisabled = PR_FALSE;
-  }
-  return NS_OK;
-}
-
-// manual submission helper method
-// aPresShell        - If the PresShell is null then the PresContext will 
-//                     get its own and use itstatic nsresult
-// aFormFrame        - The HTML Form's frame
-// aFormControlFrame - The form controls frame that is calling this
-//                     it can be null
-// aDoSubmit         - Submit = TRUE, Reset = FALSE
-// Indicates whether to do DOM Processing of the event or to do regular frame processing
-nsresult
-nsFormControlHelper::DoManualSubmitOrReset(nsIPresContext* aPresContext,
-                                           nsIPresShell*   aPresShell,
-                                           nsIFrame*       aFormFrame, 
-                                           nsIFrame*       aFormControlFrame,
-                                           PRBool          aDoSubmit,
-                                           PRBool          aDoDOMEvent) 
-{
-  NS_ENSURE_ARG_POINTER(aPresContext);
-  NS_ENSURE_ARG_POINTER(aFormFrame);
-
-  nsresult result = NS_OK;
-
-  nsCOMPtr<nsIContent> formContent;
-  aFormFrame->GetContent(getter_AddRefs(formContent));
-
-  // Here we save a pointer to the form control content
-  // so we can get its frame again after the Shell has processed the event.
-  //
-  // Control's Frame may get destroyed during the processing of the event (by the Shell)
-  // meaning: aFormControlFrame becomes invalid, so instead of using aFormControlFrame
-  // we use the saved content and the shell to go back and get the frame and 
-  // use it only if it isn't null
-  nsCOMPtr<nsIContent> controlContent;
-  if (aDoSubmit && aFormControlFrame != nsnull) {
-    aFormControlFrame->GetContent(getter_AddRefs(controlContent));
-  }
-
-  if (formContent) {
-    //Either use the PresShell passed in or go get it from the PresContext
-    nsCOMPtr<nsIPresShell> shell; // this will do our clean up
-    if (aPresShell == nsnull) {
-      result = aPresContext->GetShell(getter_AddRefs(shell));
-      aPresShell = shell.get(); // not AddRefing because shell will clean up
-    }
-
-    // With a valid PreShell handle the event
-    if (NS_SUCCEEDED(result) && nsnull != aPresShell) {
-
-      // Get originator for event (failure is non-fatal)
-      nsCOMPtr<nsIContent> formControl;
-      aFormControlFrame->GetContent(getter_AddRefs(formControl));
-
-      nsFormEvent event;
-      event.eventStructType = NS_FORM_EVENT;
-      event.message         = aDoSubmit?NS_FORM_SUBMIT:NS_FORM_RESET;
-      event.originator      = formControl;
-      nsEventStatus status  = nsEventStatus_eIgnore;
-      if (aDoDOMEvent) {
-        aPresShell->HandleDOMEventWithTarget(formContent, &event, &status);
-      } else {
-        aPresShell->HandleEventWithTarget(&event, nsnull, formContent, NS_EVENT_FLAG_INIT, &status);
-      }
-    }
-  }
-  return result;
-}
-
-// Temporary - Test of full time Standard mode for forms (Bug 91602)
-//
-void nsFormControlHelper::GetFormCompatibilityMode(nsIPresContext* aPresContext, 
-                                                   nsCompatibility& aCompatMode)
-{
-  if (mCompatFirstTime) {
-    nsCOMPtr<nsIPref> prefService(do_GetService(NS_PREF_CONTRACTID));
-    if (prefService) {
-      PRBool useEitherMode;
-      if (NS_SUCCEEDED(prefService->GetBoolPref("layout.forms.use_standard_or_quirks", &useEitherMode))) {
-        if (useEitherMode) {
-          mUseEitherMode = PR_TRUE;
-        }
-      }
-    }
-    mCompatFirstTime = PR_FALSE;
-  }
-
-  if (mUseEitherMode) {
-    aPresContext->GetCompatibilityMode(&aCompatMode);
-  } else {
-    aCompatMode = eCompatibility_Standard;
-  }
-}
-
 nsresult
 nsFormControlHelper::Reset(nsIFrame* aFrame, nsIPresContext* aPresContext)
 {
@@ -1043,37 +759,19 @@ nsFormControlHelper::Reset(nsIFrame* aFrame, nsIPresContext* aPresContext)
   return NS_ERROR_FAILURE;
 }
 
-nsresult
-nsFormControlHelper::SaveContentState(nsIFrame* aFrame,
-                                      nsIPresContext* aPresContext,
-                                      nsIPresState** aState)
+void
+nsFormControlHelper::StyleChangeReflow(nsIPresContext* aPresContext,
+                                       nsIFrame* aFrame)
 {
-  nsCOMPtr<nsIContent> controlContent;
-  aFrame->GetContent(getter_AddRefs(controlContent));
+  nsCOMPtr<nsIPresShell> shell;
+  aPresContext->GetShell(getter_AddRefs(shell));
 
-  nsCOMPtr<nsIFormControl> control = do_QueryInterface(controlContent);
-  if (control) {
-    control->SaveState(aPresContext, aState);
-    return NS_OK;
+  nsHTMLReflowCommand* reflowCmd;
+  nsresult rv = NS_NewHTMLReflowCommand(&reflowCmd, aFrame,
+                                        eReflowType_StyleChanged);
+  if (NS_SUCCEEDED(rv)) {
+    shell->AppendReflowCommand(reflowCmd);
   }
-
-  return NS_ERROR_FAILURE;
 }
 
-nsresult
-nsFormControlHelper::RestoreContentState(nsIFrame* aFrame,
-                                         nsIPresContext* aPresContext,
-                                         nsIPresState* aState)
-{
-  nsCOMPtr<nsIContent> controlContent;
-  aFrame->GetContent(getter_AddRefs(controlContent));
 
-  nsCOMPtr<nsIFormControl> control = do_QueryInterface(controlContent);
-  if (control) {
-    control->RestoreState(aPresContext, aState);
-    return NS_OK;
-  }
-
-  NS_NOTREACHED("no content");
-  return NS_ERROR_FAILURE;
-}
