@@ -43,7 +43,6 @@
 #include "nsIFormControlFrame.h"
 #include "nsHTMLParts.h"
 #include "nsIFormControl.h"
-#include "nsFormFrame.h"
 
 #include "nsIRenderingContext.h"
 #include "nsIPresContext.h"
@@ -51,13 +50,12 @@
 #include "nsIStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
-#include "nsHTMLIIDs.h"
 #include "nsISupports.h"
 #include "nsHTMLAtoms.h"
 #include "nsIImage.h"
 #include "nsStyleUtil.h"
 #include "nsStyleConsts.h"
-#include "nsIHTMLAttributes.h"
+#include "nsIHTMLContent.h"
 #include "nsIWidget.h"
 #include "nsIComponentManager.h"
 #include "nsIView.h"
@@ -69,6 +67,10 @@
 #include "nsFormControlFrame.h"
 #include "nsIFrameManager.h"
 #include "nsINameSpaceManager.h"
+#include "nsReflowPath.h"
+#include "nsIServiceManager.h"
+#include "nsIDOMHTMLButtonElement.h"
+#include "nsIDOMHTMLInputElement.h"
 #ifdef ACCESSIBILITY
 #include "nsIAccessibilityService.h"
 #endif
@@ -100,7 +102,6 @@ nsHTMLButtonControlFrame::nsHTMLButtonControlFrame()
   mPreviousCursor = eCursor_standard;
   mTranslatedRect = nsRect(0,0,0,0);
   mDidInit = PR_FALSE;
-  mRenderer.SetNameSpace(kNameSpaceID_None);
 
   mCacheSize.width             = -1;
   mCacheSize.height            = -1;
@@ -116,10 +117,6 @@ NS_IMETHODIMP
 nsHTMLButtonControlFrame::Destroy(nsIPresContext *aPresContext)
 {
   nsFormControlFrame::RegUnRegAccessKey(aPresContext, NS_STATIC_CAST(nsIFrame*, this), PR_FALSE);
-  if (mFormFrame) {
-    mFormFrame->RemoveFormControlFrame(*this);
-    mFormFrame = nsnull;
-  }
   return nsHTMLContainerFrame::Destroy(aPresContext);
 }
 
@@ -151,7 +148,7 @@ nsHTMLButtonControlFrame::Init(nsIPresContext*  aPresContext,
   // Resolve style and initialize the frame
   nsIStyleContext* styleContext;
   aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::buttonContentPseudo,
-                                            mStyleContext, PR_FALSE,
+                                            mStyleContext,
                                             &styleContext);
   mFrames.FirstChild()->Init(aPresContext, mContent, this, styleContext, nsnull);
   NS_RELEASE(styleContext);                                           
@@ -193,7 +190,14 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetAccessible(nsIAccessible** aAccessibl
   nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
 
   if (accService) {
-    return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    nsCOMPtr<nsIContent> content;
+    GetContent(getter_AddRefs(content));
+    nsCOMPtr<nsIDOMHTMLButtonElement> buttonElement(do_QueryInterface(content));
+    if (buttonElement) //If turned XBL-base form control off, the frame contains HTML 4 button
+      return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    nsCOMPtr<nsIDOMHTMLInputElement> inputElement(do_QueryInterface(content));
+    if (inputElement) //If turned XBL-base form control on, the frame contains normal HTML button
+      return accService->CreateHTMLButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
   }
 
   return NS_ERROR_FAILURE;
@@ -237,18 +241,16 @@ NS_IMETHODIMP
 nsHTMLButtonControlFrame::GetName(nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (mContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = mContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
+
+  nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
+
+  if (formControl) {
+    nsHTMLValue value;
+    result = formControl->GetHTMLAttribute(nsHTMLAtoms::name, value);
+    if (NS_CONTENT_ATTR_HAS_VALUE == result) {
+      if (eHTMLUnit_String == value.GetUnit()) {
+        value.GetStringValue(*aResult);
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
@@ -258,18 +260,16 @@ NS_IMETHODIMP
 nsHTMLButtonControlFrame::GetValue(nsAString* aResult)
 {
   nsresult result = NS_FORM_NOTOK;
-  if (mContent) {
-    nsIHTMLContent* formControl = nsnull;
-    result = mContent->QueryInterface(kIHTMLContentIID, (void**)&formControl);
-    if ((NS_OK == result) && formControl) {
-      nsHTMLValue value;
-      result = formControl->GetHTMLAttribute(nsHTMLAtoms::value, value);
-      if (NS_CONTENT_ATTR_HAS_VALUE == result) {
-        if (eHTMLUnit_String == value.GetUnit()) {
-          value.GetStringValue(*aResult);
-        }
+
+  nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
+
+  if (formControl) {
+    nsHTMLValue value;
+    result = formControl->GetHTMLAttribute(nsHTMLAtoms::value, value);
+    if (NS_CONTENT_ATTR_HAS_VALUE == result) {
+      if (eHTMLUnit_String == value.GetUnit()) {
+        value.GetStringValue(*aResult);
       }
-      NS_RELEASE(formControl);
     }
   }
   return result;
@@ -299,21 +299,8 @@ nsHTMLButtonControlFrame::IsSubmit(PRInt32 type)
 void
 nsHTMLButtonControlFrame::MouseClicked(nsIPresContext* aPresContext) 
 {
-  if ((nsnull != mFormFrame) && !nsFormFrame::GetDisabled(this)) {
-    PRInt32 type;
-    GetType(&type);
-
-    if (IsReset(type) == PR_TRUE) {
-      // do Reset & Frame processing of event
-      nsFormControlHelper::DoManualSubmitOrReset(aPresContext, nsnull, mFormFrame, 
-                                                 this, PR_FALSE, PR_FALSE); 
-    }
-    else if (IsSubmit(type) == PR_TRUE) {
-      // do Submit & Frame processing of event
-      nsFormControlHelper::DoManualSubmitOrReset(aPresContext, nsnull, mFormFrame, 
-                                                 this, PR_TRUE, PR_FALSE); 
-    }
-  }
+// This is no longer called; click events are handled in
+// nsHTMLInputElement::HandleDOMEvent().
 }
 
 void 
@@ -374,7 +361,8 @@ nsHTMLButtonControlFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                                            nsFramePaintLayer aWhichLayer,
                                            nsIFrame** aFrame)
 {
-  if (mRect.Contains(aPoint)) {
+  if (aWhichLayer == NS_FRAME_PAINT_LAYER_FOREGROUND &&
+      mRect.Contains(aPoint)) {
     const nsStyleVisibility* vis = 
       (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
       
@@ -391,9 +379,6 @@ nsHTMLButtonControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                               nsIAtom*        aListName,
                                               nsIFrame*       aChildList)
 {
-  // add ourself as an nsIFormControlFrame
-  nsFormFrame::AddFormControlFrame(aPresContext, *NS_STATIC_CAST(nsIFrame*, this));
-
   // get the frame manager and the style context of the new parent frame
   // this is used whent he children are reparented below
   // NOTE: the whole reparenting should not need to happen: see bugzilla bug 51767
@@ -430,15 +415,18 @@ nsHTMLButtonControlFrame::Paint(nsIPresContext*      aPresContext,
                                 PRUint32             aFlags)
 {
   PRBool isVisible;
-  if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_TRUE, &isVisible)) && !isVisible) {
+  if (aWhichLayer != NS_FRAME_PAINT_LAYER_FOREGROUND ||
+      (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_TRUE, &isVisible)) && !isVisible)) {
     return NS_OK;
   }
 
   nsRect rect(0, 0, mRect.width, mRect.height);
-  mRenderer.PaintButton(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, rect);
+  mRenderer.PaintButton(aPresContext, aRenderingContext, aDirtyRect, rect);
 
 #if 0 // old way
-  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_BACKGROUND);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATERS);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FOREGROUND);
 
 #else // temporary
     // XXX This is temporary
@@ -458,7 +446,9 @@ nsHTMLButtonControlFrame::Paint(nsIPresContext*      aPresContext,
 
   aRenderingContext.SetClipRect(rect, nsClipCombine_kIntersect, clipEmpty);
 
-  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_BACKGROUND);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FLOATERS);
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, NS_FRAME_PAINT_LAYER_FOREGROUND);
 
   aRenderingContext.PopState(clipEmpty);
 
@@ -513,9 +503,8 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   DO_GLOBAL_REFLOW_COUNT("nsHTMLButtonControlFrame", aReflowState.reason);
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
 
-  if (!mFormFrame && (eReflowReason_Initial == aReflowState.reason)) {
+  if (eReflowReason_Initial == aReflowState.reason) {
     nsFormControlFrame::RegUnRegAccessKey(aPresContext, NS_STATIC_CAST(nsIFrame*, this), PR_TRUE);
-    nsFormFrame::AddFormControlFrame(aPresContext, *NS_STATIC_CAST(nsIFrame*, this));
   }
 
 #if 0
@@ -586,45 +575,41 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
     availSize.height = PR_MAX(availSize.height,0);
   }
   
-  nsHTMLReflowState reflowState(aPresContext, aReflowState, firstKid, availSize);
+  // XXX Proper handling of incremental reflow...
+  nsReflowReason reason = aReflowState.reason;
+  if (eReflowReason_Incremental == reason) {
+    // See if it's targeted at us
+    nsHTMLReflowCommand *command = aReflowState.path->mReflowCommand;
+    if (command) {
+      Invalidate(aPresContext, nsRect(0,0,mRect.width,mRect.height), PR_FALSE);
+
+      nsReflowType  reflowType;
+      command->GetType(reflowType);
+      if (eReflowType_StyleChanged == reflowType) {
+        reason = eReflowReason_StyleChange;
+      }
+      else {
+        reason = eReflowReason_Resize;
+      }
+    }
+  }
+
+  nsHTMLReflowState reflowState(aPresContext, aReflowState, firstKid, availSize, reason);
   //reflowState.computedWidth = availSize;
 
   // XXX remove the following when the reflow state is fixed
   //ButtonHack(reflowState, "html4 button's area");
 
-  // XXX Proper handling of incremental reflow...
-  if (eReflowReason_Incremental == aReflowState.reason) {
-    nsIFrame* targetFrame;
-    
-    // See if it's targeted at us
-    aReflowState.reflowCommand->GetTarget(targetFrame);
-    if (this == targetFrame) {
-      Invalidate(aPresContext, nsRect(0,0,mRect.width,mRect.height), PR_FALSE);
-
-      nsReflowType  reflowType;
-      aReflowState.reflowCommand->GetType(reflowType);
-      if (eReflowType_StyleChanged == reflowType) {
-        reflowState.reason = eReflowReason_StyleChange;
-      }
-      else {
-        reflowState.reason = eReflowReason_Resize;
-      }
-    } else {
-      nsIFrame* nextFrame;
-
-      // Remove the next frame from the reflow path
-      aReflowState.reflowCommand->GetNext(nextFrame);  
-    }
-  }
-
   ReflowChild(firstKid, aPresContext, aDesiredSize, reflowState,
               focusPadding.left + aReflowState.mComputedBorderPadding.left,
               focusPadding.top + aReflowState.mComputedBorderPadding.top,
               0, aStatus);
 
   // calculate the min internal size so the contents gets centered correctly
-  nscoord minInternalWidth  = aReflowState.mComputedMinWidth  == 0?0:aReflowState.mComputedMinWidth - 
-    (aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right);
+  // minInternalWidth is not being used at all and causes a warning--commenting
+  // out until someone wants it.
+  //  nscoord minInternalWidth  = aReflowState.mComputedMinWidth  == 0?0:aReflowState.mComputedMinWidth - 
+  //    (aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right);
   nscoord minInternalHeight = aReflowState.mComputedMinHeight == 0?0:aReflowState.mComputedMinHeight - 
     (aReflowState.mComputedBorderPadding.top + aReflowState.mComputedBorderPadding.bottom);
 
@@ -705,6 +690,7 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   aStatus = NS_FRAME_COMPLETE;
 
   nsFormControlFrame::SetupCachedSizes(mCacheSize, mCachedMaxElementSize, aDesiredSize);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -757,7 +743,7 @@ nsresult nsHTMLButtonControlFrame::RequiresWidget(PRBool& aRequiresWidget)
 
 
 NS_IMETHODIMP nsHTMLButtonControlFrame::SetProperty(nsIPresContext* aPresContext,
-                                                    nsIAtom* aName, const nsAReadableString& aValue)
+                                                    nsIAtom* aName, const nsAString& aValue)
 {
   if (nsHTMLAtoms::value == aName) {
     nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
@@ -768,7 +754,7 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::SetProperty(nsIPresContext* aPresContext
   return NS_OK;
 }
 
-NS_IMETHODIMP nsHTMLButtonControlFrame::GetProperty(nsIAtom* aName, nsAWritableString& aValue)
+NS_IMETHODIMP nsHTMLButtonControlFrame::GetProperty(nsIAtom* aName, nsAString& aValue)
 {
   if (nsHTMLAtoms::value == aName) {
     nsCOMPtr<nsIHTMLContent> formControl(do_QueryInterface(mContent));
