@@ -42,7 +42,6 @@
 #include "nsIAtom.h"
 #include "nsIPresContext.h"
 #include "nsIHTMLContent.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLAtoms.h"
 #include "nsIPresState.h"
 #include "nsWidgetsCID.h"
@@ -175,7 +174,8 @@ nsIsIndexFrame::GetInputFrame(nsIPresContext* aPresContext,
 {
   nsCOMPtr<nsIPresShell> presShell;
   aPresContext->GetShell(getter_AddRefs(presShell));
-  if (presShell) {
+  if (!mInputContent) NS_WARNING("null content - cannot restore state");
+  if (presShell && mInputContent) {
     nsIFrame *frame;
     presShell->GetPrimaryFrameFor(mInputContent, &frame);
     if (frame) {
@@ -192,7 +192,7 @@ nsIsIndexFrame::GetInputValue(nsIPresContext* aPresContext,
   nsIFormControlFrame* frame = nsnull;
   GetInputFrame(aPresContext, &frame);
   if (frame) {
-    ((nsNewFrame*)frame)->GetTextControlFrameState(oString);
+    ((nsNewFrame*)frame)->GetValue(oString, PR_FALSE);
   }
   return NS_OK;
 }
@@ -204,7 +204,7 @@ nsIsIndexFrame::SetInputValue(nsIPresContext* aPresContext,
   nsIFormControlFrame* frame = nsnull;
   GetInputFrame(aPresContext, &frame);
   if (frame) {
-    ((nsNewFrame*)frame)->SetTextControlFrameState(aString);
+    ((nsNewFrame*)frame)->SetValue(aString);
   }
   return NS_OK;
 }
@@ -327,9 +327,9 @@ nsIsIndexFrame::ScrollIntoView(nsIPresContext* aPresContext)
 
 
 NS_IMETHODIMP nsIsIndexFrame::Reflow(nsIPresContext*          aPresContext, 
-                                         nsHTMLReflowMetrics&     aDesiredSize,
-                                         const nsHTMLReflowState& aReflowState, 
-                                         nsReflowStatus&          aStatus)
+                                     nsHTMLReflowMetrics&     aDesiredSize,
+                                     const nsHTMLReflowState& aReflowState, 
+                                     nsReflowStatus&          aStatus)
 {
   DO_GLOBAL_REFLOW_COUNT("nsIsIndexFrame", aReflowState.reason);
   DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
@@ -337,6 +337,7 @@ NS_IMETHODIMP nsIsIndexFrame::Reflow(nsIPresContext*          aPresContext,
   // The Areaframe takes care of all our reflow 
   // (except for when style is used to change its size?)
   nsresult rv = nsAreaFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -444,11 +445,10 @@ nsIsIndexFrame::OnSubmit(nsIPresContext* aPresContext)
 
       // Necko's MakeAbsoluteURI doesn't reuse the baseURL's rel path if it is
       // passed a zero length rel path.
-      nsXPIDLCString relPath;
-      docURL->GetSpec(getter_Copies(relPath));
-      NS_ASSERTION(relPath, "Rel path couldn't be formed in form submit!\n");
-      if (relPath) {
-        href.AppendWithConversion(relPath);
+      nsCAutoString relPath;
+      docURL->GetSpec(relPath);
+      if (!relPath.IsEmpty()) {
+        href = NS_ConvertUTF8toUCS2(relPath);
 
         // If re-using the same URL, chop off old query string (bug 25330)
         PRInt32 queryStart = href.FindChar('?');
@@ -456,6 +456,7 @@ nsIsIndexFrame::OnSubmit(nsIPresContext* aPresContext)
           href.Truncate(queryStart);
         }
       } else {
+        NS_ERROR("Rel path couldn't be formed in form submit!\n");
         return NS_ERROR_OUT_OF_MEMORY;
       }
 
@@ -464,12 +465,12 @@ nsIsIndexFrame::OnSubmit(nsIPresContext* aPresContext)
     nsCOMPtr<nsIURI> actionURL;
     nsXPIDLCString scheme;
     PRBool isJSURL = PR_FALSE;
-    if (NS_SUCCEEDED(result = NS_NewURI(getter_AddRefs(actionURL), href, docURL))) {
+    if (NS_SUCCEEDED(result = NS_NewURI(getter_AddRefs(actionURL), href, nsnull, docURL))) {
       result = actionURL->SchemeIs("javascript", &isJSURL);
     }
     // Append the URI encoded variable/value pairs for GET's
     if (!isJSURL) { // Not for JS URIs, see bug 26917
-        if (href.FindChar('?', PR_FALSE, 0) == kNotFound) { // Add a ? if needed
+        if (href.FindChar('?') == kNotFound) { // Add a ? if needed
           href.Append(PRUnichar('?'));
         } else {                              // Adding to existing query string
           if (href.Last() != '&' && href.Last() != '?') {   // Add a & if needed
