@@ -43,7 +43,6 @@
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsVoidArray.h"
-#include "nsHTMLIIDs.h"
 
 #ifdef DEBUG_TABLE_STRATEGY
 static PRInt32 gsDebugCount = 0;
@@ -139,7 +138,7 @@ PRBool BasicTableLayoutStrategy::Initialize(nsIPresContext*          aPresContex
   nscoord minWidth, prefWidth;
   mTableFrame->CalcMinAndPreferredWidths(aPresContext, aReflowState, PR_FALSE, minWidth, prefWidth);
   if (hasPctCol && mTableFrame->IsAutoWidth()) {
-    prefWidth = CalcPctAdjTableWidth(aReflowState, boxWidth, p2t);
+    prefWidth = CalcPctAdjTableWidth(*aPresContext, aReflowState, boxWidth, p2t);
   }
   // calc the desired width, considering if there is a specified width. 
   // don't use nsTableFrame::CalcDesiredWidth because it is based on table column widths.
@@ -155,9 +154,6 @@ PRBool BasicTableLayoutStrategy::Initialize(nsIPresContext*          aPresContex
 
 #ifdef DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugTimeMethod(nsTableFrame::eInit, *mTableFrame, (nsHTMLReflowState&)aReflowState, PR_FALSE);
-#endif
-#ifdef DEBUG_TABLE_REFLOW
-  printf("Initialized min=%d des=%d pref=%d\n", minWidth, desWidth, prefWidth);
 #endif
   return result;
 }
@@ -195,7 +191,7 @@ ResetPctValues(nsTableFrame* aTableFrame,
   // initialize the col percent and cell percent values to 0.
   PRInt32 colX;
   for (colX = 0; colX < aNumCols; colX++) { 
-    nsTableColFrame* colFrame = aTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = aTableFrame->GetColFrame(colX); 
     if (colFrame) {
       colFrame->SetWidth(PCT, WIDTH_NOT_SET);
       colFrame->SetWidth(PCT_ADJ, WIDTH_NOT_SET);
@@ -207,6 +203,7 @@ PRBool
 BasicTableLayoutStrategy::BalanceColumnWidths(nsIPresContext*          aPresContext,
                                               const nsHTMLReflowState& aReflowState)
 {
+  if (!aPresContext) ABORT1(PR_FALSE);
 #ifdef DEBUG_TABLE_STRATEGY
   printf("BalanceColumnWidths en count=%d \n", gsDebugCount++); mTableFrame->Dump(aPresContext, PR_FALSE, PR_TRUE, PR_FALSE);
 #endif
@@ -220,8 +217,16 @@ BasicTableLayoutStrategy::BalanceColumnWidths(nsIPresContext*          aPresCont
 
   PRInt32 numCols = mTableFrame->GetColCount();
   PRBool tableIsAutoWidth = mTableFrame->IsAutoWidth();
-  nscoord horBorderPadding = aReflowState.mComputedBorderPadding.left + 
-                             aReflowState.mComputedBorderPadding.right;
+
+  nscoord horOffset; 
+  // get the reduction in available horizontal space due to borders and padding
+  if (mTableFrame->IsBorderCollapse()) {
+    nsMargin offset = mTableFrame->GetChildAreaOffset(*aPresContext, &aReflowState);
+    horOffset = offset.left + offset.right;
+  }
+  else {
+    horOffset = aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right;
+  }
 
   // determine if the table is auto/fixed and get the fixed width if available
   nscoord maxWidth = mTableFrame->CalcBorderBoxWidth(aPresContext, aReflowState);
@@ -238,19 +243,19 @@ BasicTableLayoutStrategy::BalanceColumnWidths(nsIPresContext*          aPresCont
   // An auto table returns a new table width based on percent cells/cols if they exist
   nscoord perAdjTableWidth = 0;
   if (mTableFrame->HasPctCol()) {
-    perAdjTableWidth = AssignPctColumnWidths(aReflowState, maxWidth, tableIsAutoWidth, p2t);
+    perAdjTableWidth = AssignPctColumnWidths(*aPresContext, aReflowState, maxWidth, tableIsAutoWidth, p2t);
     if (perAdjTableWidth > 0) {
       // if an auto table has a pct col or cell, set the preferred table width 
       // here so that CalcPctAdjTableWidth wont't need to be called by the table
       mTableFrame->SetPreferredWidth(perAdjTableWidth);
     }
     perAdjTableWidth = PR_MIN(perAdjTableWidth, maxWidth);
-    perAdjTableWidth -= horBorderPadding;
+    perAdjTableWidth -= horOffset;
     perAdjTableWidth = PR_MAX(perAdjTableWidth, 0);
   }
 
   // reduce the maxWidth by border and padding, since we will be dealing with content width
-  maxWidth -= horBorderPadding;
+  maxWidth -= horOffset;
   maxWidth = PR_MAX(0, maxWidth);
 
   PRInt32 numNonZeroWidthCols = 0;
@@ -258,7 +263,7 @@ BasicTableLayoutStrategy::BalanceColumnWidths(nsIPresContext*          aPresCont
   nscoord minTableWidth = 0;
   PRInt32 colX;
   for (colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord colMinWidth = colFrame->GetMinWidth();
     mTableFrame->SetColumnWidth(colX, colMinWidth);
@@ -400,7 +405,7 @@ void BasicTableLayoutStrategy::AllocateFully(nscoord&        aTotalAllocated,
 {
   PRInt32 numCols = mTableFrame->GetColCount(); 
   for (PRInt32 colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     if (!CanAllocate(aWidthType, aAllocTypes[colX], colFrame)) {
       continue;
@@ -454,7 +459,8 @@ void BasicTableLayoutStrategy::AllocateUnconstrained(PRInt32  aAllocAmount,
       }
       else {
         if (aExclude0Pro) {
-          nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+          nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
+          if (!colFrame) continue;
           if (colFrame->GetConstraint() == e0ProportionConstraint) {
             aAllocTypes[colX] = FINISHED;
           }
@@ -467,7 +473,8 @@ void BasicTableLayoutStrategy::AllocateUnconstrained(PRInt32  aAllocAmount,
   PRInt32 numColsAllocated = 0; 
   PRInt32 totalAllocated   = 0;
   for (colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);  
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
+    if (!colFrame) continue; 
     PRBool skipColumn = aExclude0Pro && (e0ProportionConstraint == colFrame->GetConstraint());
     if (FINISHED != aAllocTypes[colX] && !skipColumn ) {
       divisor += mTableFrame->GetColumnWidth(colX);
@@ -477,8 +484,9 @@ void BasicTableLayoutStrategy::AllocateUnconstrained(PRInt32  aAllocAmount,
   for (colX = 0; colX < numCols; colX++) { 
     if (FINISHED != aAllocTypes[colX]) {
       if (aExclude0Pro) {
-        nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
-        if (colFrame && (e0ProportionConstraint == colFrame->GetConstraint())) {
+        nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
+        if (!colFrame) continue;
+        if (e0ProportionConstraint == colFrame->GetConstraint()) {
           continue;
         }
       }
@@ -533,7 +541,7 @@ BasicTableLayoutStrategy::ComputeNonPctColspanWidths(const nsHTMLReflowState& aR
 
   PRInt32 colX;
   for (colX = numCols - 1; colX >= 0; colX--) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     colFrame->SetWidth(MIN_ADJ, WIDTH_NOT_SET);
     colFrame->SetWidth(FIX_ADJ, WIDTH_NOT_SET);
@@ -698,7 +706,7 @@ BasicTableLayoutStrategy::ComputeNonPctColspanWidths(PRInt32           aWidthInd
   PRInt32 spanX;
   // accumulate the various divisors to be used later
   for (spanX = 0; spanX < aColSpan; spanX++) {
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(aColIndex + spanX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(aColIndex + spanX); 
     if (!colFrame) continue;
     nscoord minWidth = PR_MAX(colFrame->GetMinWidth(), 0);
     nscoord pctWidth = PR_MAX(colFrame->GetPctWidth(), 0);
@@ -802,7 +810,7 @@ BasicTableLayoutStrategy::ComputeNonPctColspanWidths(PRInt32           aWidthInd
     // get the correct numerator in a similar fashion to getting the divisor
     for (spanX = 0; spanX < aColSpan; spanX++) {
       if (usedWidth >= availWidth) break;
-      nsTableColFrame* colFrame = mTableFrame->GetColFrame(aColIndex + spanX);
+      nsTableColFrame* colFrame = mTableFrame->GetColFrame(aColIndex + spanX); 
       if (!colFrame) continue;
       nscoord minWidth = colFrame->GetMinWidth();
       nscoord pctWidth = PR_MAX(colFrame->GetPctWidth(), 0);
@@ -974,7 +982,7 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
     nscoord fixWidth = WIDTH_NOT_SET;
     
     // Get column frame and reset it
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     NS_ASSERTION(nsnull != colFrame, "bad col frame");
     colFrame->ResetSizingInfo();
@@ -1079,7 +1087,7 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
   }
 //set the table col fixed width if present
   for (colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord fixColWidth = colFrame->GetWidth(FIX);
     // use the style width of a col only if the col hasn't gotten a fixed width from any cell
@@ -1095,12 +1103,12 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
   }
   PRBool* pctRequest = (hasPctCol) ? nsnull : &hasPctCol;
   ComputeNonPctColspanWidths(aReflowState, PR_FALSE, aPixelToTwips, pctRequest);
-
+  PRInt32 numEffCols = mTableFrame->GetEffectiveColCount();
   // figure the proportional widths for porportional cols
   if (rawPropTotal > 0)  {
     // get the total desired widths
     nscoord desTotal = 0;
-    for (colX = 0; colX < numCols; colX++) { 
+    for (colX = 0; colX < numEffCols; colX++) { 
       nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
       if (!colFrame) continue;
       // consider 1*, 2*, etc cols, but ignore 0* cols
@@ -1111,7 +1119,7 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
     // find the largest combined prop size considering each prop col and
     // its desired size
     nscoord maxPropTotal = 0;
-    for (colX = 0; colX < numCols; colX++) { 
+    for (colX = 0; colX < numEffCols; colX++) { 
       nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
       nscoord rawProp = colFrame->GetWidth(MIN_PRO);
       if (rawProp > 0) {
@@ -1122,7 +1130,7 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
       }
     }
     // set MIN_PRO widths based on the maxPropTotal
-    for (colX = 0; colX < numCols; colX++) { 
+    for (colX = 0; colX < numEffCols; colX++) { 
       nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
       if (!colFrame) continue;
       nscoord rawProp = colFrame->GetWidth(MIN_PRO);
@@ -1140,7 +1148,8 @@ BasicTableLayoutStrategy::AssignNonPctColumnWidths(nsIPresContext*          aPre
 
   // Set the table col width for each col to the content min. 
   for (colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
+    if (!colFrame) continue;
     nscoord minWidth = colFrame->GetMinWidth();
     mTableFrame->SetColumnWidth(colX, minWidth);
   }
@@ -1159,7 +1168,7 @@ BasicTableLayoutStrategy::ReduceOverSpecifiedPctCols(nscoord aExcess)
 {
   nscoord numCols = mTableFrame->GetColCount();
   for (PRInt32 colX = numCols - 1; (colX >= 0) && (aExcess > 0); colX--) {
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord pctWidth = colFrame->GetWidth(PCT);
     nscoord reduction = 0;
@@ -1198,7 +1207,8 @@ inline nscoord WrapupAssignPctColumnWidths(nsTableFrame*            aTableFrame,
 #endif
 
 nscoord 
-BasicTableLayoutStrategy::CalcPctAdjTableWidth(const nsHTMLReflowState& aReflowState,
+BasicTableLayoutStrategy::CalcPctAdjTableWidth(nsIPresContext&          aPresContext,
+                                               const nsHTMLReflowState& aReflowState,
                                                nscoord                  aAvailWidthIn,
                                                float                    aPixelToTwips)
 {
@@ -1219,7 +1229,7 @@ BasicTableLayoutStrategy::CalcPctAdjTableWidth(const nsHTMLReflowState& aReflowS
     rawPctValues[colX] = 0.0f;
   }
 
-  nsMargin borderPadding = mTableFrame->GetBorderPadding(aReflowState);
+  nsMargin borderPadding = mTableFrame->GetContentAreaOffset(aPresContext, &aReflowState);
   nscoord availWidth = aAvailWidthIn;
   if (NS_UNCONSTRAINEDSIZE != availWidth) {
     // adjust the avail width to exclude table border, padding and cell spacing
@@ -1227,7 +1237,7 @@ BasicTableLayoutStrategy::CalcPctAdjTableWidth(const nsHTMLReflowState& aReflowS
   }
 
   for (colX = 0; colX < numCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord maxColBasis = -1;
     // Scan the cells in the col 
@@ -1282,7 +1292,8 @@ BasicTableLayoutStrategy::CalcPctAdjTableWidth(const nsHTMLReflowState& aReflowS
   nscoord fixDesTotalNoPct = 0;    // total of fix or des widths of cols without pct
 
   for (colX = 0; colX < numCols; colX++) {
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
+    if (!colFrame) continue;
     nscoord fixWidth = colFrame->GetFixWidth();
     nscoord fixDesWidth = (fixWidth > 0) ? fixWidth : colFrame->GetDesWidth();
     fixDesTotal += fixDesWidth;
@@ -1329,7 +1340,8 @@ BasicTableLayoutStrategy::CalcPctAdjTableWidth(const nsHTMLReflowState& aReflowS
 
 // Determine percentage col widths for each col frame
 nscoord 
-BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflowState,
+BasicTableLayoutStrategy::AssignPctColumnWidths(nsIPresContext&          aPresContext,
+                                                const nsHTMLReflowState& aReflowState,
                                                 nscoord                  aAvailWidth,
                                                 PRBool                   aTableIsAutoWidth,
                                                 float                    aPixelToTwips)
@@ -1349,11 +1361,11 @@ BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflow
   // on percent cells/cols. This probably should only be a NavQuirks thing, since
   // a percentage based cell or column on an auto table should force the column to auto
   nscoord basis = (aTableIsAutoWidth) 
-                  ? CalcPctAdjTableWidth(aReflowState, aAvailWidth, aPixelToTwips)
+                  ? CalcPctAdjTableWidth(aPresContext, aReflowState, aAvailWidth, aPixelToTwips)
                   : aAvailWidth;
 
   // adjust the basis to exclude table border, padding and cell spacing
-  nsMargin borderPadding = mTableFrame->GetBorderPadding(aReflowState);
+  nsMargin borderPadding = mTableFrame->GetContentAreaOffset(aPresContext, &aReflowState);
   basis -= borderPadding.left + borderPadding.right + mCellSpacingTotal;
 
   nscoord colPctTotal = 0;
@@ -1361,7 +1373,7 @@ BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflow
   // Determine the percentage contribution for cols and for cells with colspan = 1
   // Iterate backwards, similarly to the reasoning in AssignNonPctColumnWidths
   for (colX = numCols - 1; colX >= 0; colX--) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord maxColPctWidth = WIDTH_NOT_SET;
     float maxColPct = 0.0f;
@@ -1431,7 +1443,7 @@ BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflow
       }
       // determine if the cell spans cols which have a pct value
       for (PRInt32 spanX = 0; (spanX < colSpan) && !done; spanX++) {
-        nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX);
+        nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX); 
         if (!colFrame) continue;
         if (colFrame->GetWidth(PCT) > 0) {
           mTableFrame->SetHasCellSpanningPctCol(PR_TRUE);
@@ -1511,7 +1523,7 @@ BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflow
         // accumulate the spanTotal as the max of MIN, DES, FIX, PCT
         PRInt32 spanX;
         for (spanX = 0; spanX < colSpan; spanX++) {
-          nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX);
+          nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX); 
           if (!colFrame) continue;
           nscoord colPctWidth = colFrame->GetWidth(PCT);
           if (colPctWidth > 0) { // skip pct cols
@@ -1546,7 +1558,7 @@ BasicTableLayoutStrategy::AssignPctColumnWidths(const nsHTMLReflowState& aReflow
           // record the percent contributions for the spanned cols
           PRInt32 usedColumns = colSpan;
           for (spanX = colSpan-1; spanX >= 0; spanX--) {
-            nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX);
+            nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX + spanX); 
             if (!colFrame) continue;
             if ((colFrame->GetWidth(PCT) > 0) || (canSkipPctAdj && (colFrame->GetWidth(PCT_ADJ) > 0))) {
               // dont use pct cols or if we can skip the pct adj event do not take the PCT_ADJ cols
@@ -1621,7 +1633,7 @@ void BasicTableLayoutStrategy::CalculateTotals(PRInt32* aTotalCounts,
   PRInt32 colX;
 
   for (colX = 0; colX < numEffCols; colX++) { 
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     nscoord minCol = colFrame->GetMinWidth();
     aTotalCounts[MIN_CON]++;
@@ -1675,12 +1687,12 @@ void BasicTableLayoutStrategy::CalculateTotals(PRInt32* aTotalCounts,
 }
 
 
-struct nsColInfo {
-  nsColInfo(nsTableColFrame* aFrame,
-            PRInt32          aIndex,
-            PRInt32          aMinWidth,
-            PRInt32          aWidth,
-            PRInt32          aMaxWidth)
+struct ColInfo {
+  ColInfo(nsTableColFrame* aFrame,
+          PRInt32          aIndex,
+          PRInt32          aMinWidth,
+          PRInt32          aWidth,
+          PRInt32          aMaxWidth)
     : mFrame(aFrame), mIndex(aIndex), mMinWidth(aMinWidth), 
       mWidth(aWidth), mMaxWidth(aMaxWidth), mWeight(0)
   {}
@@ -1695,7 +1707,7 @@ struct nsColInfo {
 void
 AC_Wrapup(nsTableFrame* aTableFrame,
           PRInt32       aNumItems, 
-          nsColInfo**   aColInfo,
+          ColInfo**     aColInfo,
           PRBool        aAbort = PR_FALSE)
 {
   if (aColInfo) {
@@ -1713,7 +1725,7 @@ AC_Wrapup(nsTableFrame* aTableFrame,
 
 void
 AC_Increase(PRInt32     aNumAutoCols,
-            nsColInfo** aColInfo,
+            ColInfo**   aColInfo,
             PRInt32     aDivisor,
             PRInt32&    aAvailWidth,
             float       aPixelToTwips)
@@ -1742,7 +1754,7 @@ AC_Increase(PRInt32     aNumAutoCols,
 
 void
 AC_Decrease(PRInt32     aNumAutoCols,
-            nsColInfo** aColInfo,
+            ColInfo**   aColInfo,
             PRInt32     aDivisor,
             PRInt32&    aExcess,
             float       aPixelToTwips)
@@ -1768,15 +1780,15 @@ AC_Decrease(PRInt32     aNumAutoCols,
 }
 
 void 
-AC_Sort(nsColInfo** aColInfo, PRInt32 aNumCols)
+AC_Sort(ColInfo** aColInfo, PRInt32 aNumCols)
 {
   // sort the cols based on the Weight 
   for (PRInt32 j = aNumCols - 1; j > 0; j--) {
     for (PRInt32 i = 0; i < j; i++) { 
       if (aColInfo[i]->mWeight < aColInfo[i+1]->mWeight) { // swap them
-        nsColInfo* save = aColInfo[i];
-        aColInfo[i]     = aColInfo[i+1];
-        aColInfo[i+1]   = save;
+        ColInfo* save = aColInfo[i];
+        aColInfo[i]   = aColInfo[i+1];
+        aColInfo[i+1] = save;
       }
     }
   }
@@ -1820,7 +1832,7 @@ void BasicTableLayoutStrategy::AllocateConstrained(PRInt32  aAvailWidth,
   PRInt32 colX;
   // find out how many constrained cols there are
   for (colX = 0; colX < numCols; colX++) {
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     if (!CanAllocate(aWidthType, aAllocTypes[colX], colFrame)) {
       continue;
@@ -1829,15 +1841,15 @@ void BasicTableLayoutStrategy::AllocateConstrained(PRInt32  aAvailWidth,
   }
 
   // allocate storage for the constrained cols. Only they get adjusted.
-  nsColInfo** colInfo = new nsColInfo*[numConstrainedCols];
+  ColInfo** colInfo = new ColInfo*[numConstrainedCols];
   if (!colInfo) return;
-  memset(colInfo, 0, numConstrainedCols * sizeof(nsColInfo *));
+  memset(colInfo, 0, numConstrainedCols * sizeof(ColInfo *));
 
   PRInt32 maxMinDiff = 0;
   PRInt32 constrColX = 0;
   // set the col info entries for each constrained col
   for (colX = 0; colX < numCols; colX++) {
-    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX);
+    nsTableColFrame* colFrame = mTableFrame->GetColFrame(colX); 
     if (!colFrame) continue;
     if (!CanAllocate(aWidthType, aAllocTypes[colX], colFrame)) {
       continue;
@@ -1847,6 +1859,8 @@ void BasicTableLayoutStrategy::AllocateConstrained(PRInt32  aAvailWidth,
     // proportional and desired widths are handled together
     PRBool haveProWidth = PR_FALSE;
     if (DES_CON == aWidthType) {
+      // Take into acount MIN_adj width as it has been included in totals before
+      maxWidth = PR_MAX(maxWidth, colFrame->GetWidth(MIN_ADJ));
       nscoord proWidth = colFrame->GetWidth(MIN_PRO);
       if (proWidth >= 0) {
         haveProWidth = PR_TRUE;
@@ -1862,7 +1876,7 @@ void BasicTableLayoutStrategy::AllocateConstrained(PRInt32  aAvailWidth,
     maxWidth = PR_MAX(maxWidth, minWidth);
     maxMinDiff += maxWidth - minWidth;
     nscoord startWidth = (aStartAtMin) ? minWidth : maxWidth;
-    colInfo[constrColX] = new nsColInfo(colFrame, colX, minWidth, startWidth, maxWidth);
+    colInfo[constrColX] = new ColInfo(colFrame, colX, minWidth, startWidth, maxWidth);
     if (!colInfo[constrColX]) {
       AC_Wrapup(mTableFrame, numConstrainedCols, colInfo, PR_TRUE);
       return;
