@@ -40,7 +40,7 @@
 #include "nsTableCellFrame.h"
 #include "nsTableFrame.h"
 #include "nsTableRowGroupFrame.h"
-#include "nsHTMLReflowCommand.h"
+#include "nsReflowPath.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
@@ -48,11 +48,9 @@
 #include "nsCSSRendering.h"
 #include "nsIContent.h"
 #include "nsIHTMLContent.h"
-#include "nsHTMLIIDs.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLValue.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsVoidArray.h"
 #include "nsIView.h"
 #include "nsStyleUtil.h"
@@ -77,8 +75,15 @@
 
 nsTableCellFrame::nsTableCellFrame()
 {
-  mBits.mColIndex        = 0;
+  mBits.mColIndex  = 0;
   mPriorAvailWidth = 0;
+
+  SetContentEmpty(PR_FALSE);
+  SetNeedSpecialReflow(PR_FALSE);
+  SetHadSpecialReflow(PR_FALSE);
+  SetHasPctOverHeight(PR_FALSE);
+  SetNeedPass2Reflow(PR_TRUE);
+
 #ifdef DEBUG_TABLE_REFLOW_TIMING
   mTimer = new nsReflowTimer(this);
   mBlockTimer = new nsReflowTimer(this);
@@ -100,7 +105,7 @@ nsTableCellFrame::GetNextCell() const
   while (childFrame) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       return (nsTableCellFrame*)childFrame;
     }
     childFrame->GetNextSibling(&childFrame);
@@ -150,28 +155,46 @@ NoComputedHeightBetween(const nsHTMLReflowState&  aReflowState,
 
 // nsIPercentHeightObserver methods
 
-nsresult
+void
 nsTableCellFrame::NotifyPercentHeight(const nsHTMLReflowState& aReflowState)
 {
-  if (!HadSpecialReflow()) {
+  if (!NeedSpecialReflow()) {
     // Only initiate a special reflow if we will be able to construct a computed height 
     // on the cell that will result in the frame getting a computed height. This can only 
-    // happen (but not sufficient) if there no computed height already set between the 
+    // happen (but not sufficient) if there is no computed height already set between the 
     // initiating frame and the cell.
     for (const nsHTMLReflowState* rs = aReflowState.parentReflowState; rs; rs = rs->parentReflowState) {
       if ((NS_UNCONSTRAINEDSIZE != rs->mComputedHeight) && (0 != rs->mComputedHeight)) {
-        return NS_OK;
+        return;
       }
       // stop when we reach the cell frame
       if (rs->frame == this) {
-        nsTableFrame::NotifyAncestorsOfSpecialReflow((nsIFrame&)*this);
-        SetNeedSpecialReflow(PR_TRUE);
-        return NS_OK;
+        nsTableFrame::RequestSpecialHeightReflow(*rs);
+        return;
       }
     }
     NS_ASSERTION(PR_FALSE, "program error in NotifyPercentHeight");
   }
-  return NS_OK;
+}
+
+// The cell needs to observe its block and things inside its block but nothing below that
+PRBool 
+nsTableCellFrame::NeedsToObserve(const nsHTMLReflowState& aReflowState)
+{
+  PRBool result = PR_FALSE;
+  const nsHTMLReflowState* parentRS = aReflowState.parentReflowState;
+  if (parentRS && (parentRS->mPercentHeightObserver == this)) { // cell observes the parent
+    result = PR_TRUE;
+    parentRS = parentRS->parentReflowState;
+    if (parentRS && (parentRS->mPercentHeightObserver == this)) { // cell observers the grand parent
+      parentRS = parentRS->parentReflowState;
+      if (parentRS && (parentRS->mPercentHeightObserver == this)) { 
+        // cell observes the great grand parent, so we have gone too deep
+        result = PR_FALSE;
+      }
+    }
+  }
+  return result;
 }
 
 nsresult 
@@ -389,6 +412,32 @@ nsTableCellFrame::DecorateForSelection(nsIPresContext* aPresContext,
   return NS_OK;
 }
 
+void
+nsTableCellFrame::PaintUnderlay(nsIPresContext&           aPresContext,
+                                nsIRenderingContext&      aRenderingContext,
+                                const nsRect&             aDirtyRect,
+                                PRUint32&                 aFlags,
+                                const nsStyleTableBorder& aCellTableStyle,
+                                const nsStyleBorder&      aStyleBorder,
+                                PRBool                    aVisibleBackground,
+                                PRBool&                   aPaintChildren)
+{
+  if (aVisibleBackground) {
+    nsRect rect(0, 0, mRect.width, mRect.height);
+    nsCSSRendering::PaintBackground(&aPresContext, aRenderingContext, this,
+                                    aDirtyRect, rect, aStyleBorder, 0, 0, PR_TRUE);
+    // draw the border only when there is content or showing empty cells
+    if (!GetContentEmpty() || NS_STYLE_TABLE_EMPTY_CELLS_SHOW == aCellTableStyle.mEmptyCells) {
+      PRIntn skipSides = GetSkipSides();
+      nsCSSRendering::PaintBorder(&aPresContext, aRenderingContext, this,
+                                  aDirtyRect, rect, aStyleBorder, mStyleContext, skipSides);
+    }
+  }
+
+  // tell Paint to paint the children
+  aPaintChildren = PR_TRUE;
+}
+
 NS_METHOD 
 nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
                         nsIRenderingContext& aRenderingContext,
@@ -396,58 +445,38 @@ nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
                         nsFramePaintLayer    aWhichLayer,
                         PRUint32             aFlags)
 {
+  NS_ENSURE_TRUE(aPresContext, NS_ERROR_NULL_POINTER);
   PRBool isVisible;
   if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_FALSE, &isVisible)) && !isVisible) {
     return NS_OK;
   }
-  const nsStyleDisplay* disp =
-    (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
-  const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
- 
+
+  PRBool paintChildren   = PR_TRUE;
+
   if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+    PRBool paintBackground = PR_FALSE;
+    const nsStyleBorder*      myBorder       = nsnull;
+    const nsStyleTableBorder* cellTableStyle = nsnull;
+    const nsStyleVisibility* vis = 
+       (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
     if (vis->IsVisibleOrCollapsed()) {
+      myBorder = (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border); NS_ENSURE_TRUE(myBorder, NS_ERROR_NULL_POINTER);
 
-      const nsStyleBackground* myColor = (const nsStyleBackground*)mStyleContext->GetStyleData(eStyleStruct_Background);
-#ifdef OLD_TABLE_SELECTION
-      myColor = GetColorStyleFromSelection(myColor);
-#endif
+      GetStyleData(eStyleStruct_TableBorder, ((const nsStyleStruct *&)cellTableStyle));
 
-      const nsStyleBorder* myBorder =
-        (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
-      NS_ASSERTION(nsnull!=myColor, "bad style color");
-      NS_ASSERTION(nsnull!=myBorder, "bad style spacing");
+      // paint the background when the cell is not empty or when showing empty cells or background
+      paintBackground = (!GetContentEmpty()                                             ||
+                         NS_STYLE_TABLE_EMPTY_CELLS_SHOW == cellTableStyle->mEmptyCells || 
+                         NS_STYLE_TABLE_EMPTY_CELLS_SHOW_BACKGROUND == cellTableStyle->mEmptyCells);
+    }
+  
+    PaintUnderlay(*aPresContext, aRenderingContext, aDirtyRect, aFlags, *cellTableStyle,
+                  *myBorder, paintBackground, paintChildren);
 
-
-      const nsStyleTableBorder* cellTableStyle;
-      GetStyleData(eStyleStruct_TableBorder, ((const nsStyleStruct *&)cellTableStyle)); 
-      nsRect  rect(0, 0, mRect.width, mRect.height);
-
-      // draw the background except when the cell is empty and 'empty-cells: hide' is set
-      if (!GetContentEmpty() || 
-          NS_STYLE_TABLE_EMPTY_CELLS_SHOW            == cellTableStyle->mEmptyCells || 
-          NS_STYLE_TABLE_EMPTY_CELLS_SHOW_BACKGROUND == cellTableStyle->mEmptyCells) {
-        nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *myBorder, 0, 0);
-      }
-      // draw the border except when the cell is empty and 'empty-cells: hide || -moz-show-background' is set
-      if (!GetContentEmpty() || 
-          NS_STYLE_TABLE_EMPTY_CELLS_SHOW == cellTableStyle->mEmptyCells) {
-        PRIntn skipSides = GetSkipSides();
-        nsTableFrame* tableFrame = nsnull;  // I should be checking my own style context, but border-collapse isn't inheriting correctly
-        nsresult rv = nsTableFrame::GetTableFrame(this, tableFrame);
-        if ((NS_SUCCEEDED(rv)) && tableFrame) {
-          const nsStyleTableBorder* tableStyle;
-          tableFrame->GetStyleData(eStyleStruct_TableBorder, ((const nsStyleStruct *&)tableStyle)); 
-          if (NS_STYLE_BORDER_SEPARATE == tableFrame->GetBorderCollapseStyle()) {
-            nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *myBorder, mStyleContext, skipSides);
-          }
-        }
-      }
-#ifndef OLD_TABLE_SELECTION
+    if (vis->IsVisibleOrCollapsed()) {
+      const nsStyleBackground* myColor = 
+        (const nsStyleBackground*)mStyleContext->GetStyleData(eStyleStruct_Background); NS_ENSURE_TRUE(myColor, NS_ERROR_NULL_POINTER);
       DecorateForSelection(aPresContext, aRenderingContext,myColor); //ignore return value
-#endif //OLD_TABLE_SELECTION
     }
   }
 
@@ -459,31 +488,38 @@ nsTableCellFrame::Paint(nsIPresContext*      aPresContext,
   }
 #endif
 
-  // if the cell originates in a row and/or col that is collapsed, the
-  // bottom and/or right portion of the cell is painted by translating
-  // the rendering context.
-  PRBool clipState;
-  nsPoint offset;
-  GetCollapseOffset(aPresContext, offset);
-  if ((0 != offset.x) || (0 != offset.y)) {
-    aRenderingContext.PushState();
-    aRenderingContext.Translate(offset.x, offset.y);
-    aRenderingContext.SetClipRect(nsRect(-offset.x, -offset.y, mRect.width, mRect.height),
-                                nsClipCombine_kIntersect, clipState);
-  }
-  else{
-    if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
+  // paint the children unless we've been told not to
+  if (paintChildren) {
+    const nsStyleDisplay* disp =
+      (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
+    // if the cell originates in a row and/or col that is collapsed, the
+    // bottom and/or right portion of the cell is painted by translating
+    // the rendering context.
+    PRBool clipState;
+    nsPoint offset;
+    GetCollapseOffset(aPresContext, offset);
+    if ((0 != offset.x) || (0 != offset.y)) {
       aRenderingContext.PushState();
-      SetOverflowClipRect(aRenderingContext);
-    }    
-  }
-  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
-  if ((0 != offset.x) || (0 != offset.y)) {
-    aRenderingContext.PopState(clipState);
-  }
-  else { 
-    if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
-      aRenderingContext.PopState(clipState);         
+      aRenderingContext.Translate(offset.x, offset.y);
+      aRenderingContext.SetClipRect(nsRect(-offset.x, -offset.y, mRect.width, mRect.height),
+                                    nsClipCombine_kIntersect, clipState);
+    }
+    else {
+      if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
+        aRenderingContext.PushState();
+        SetOverflowClipRect(aRenderingContext);
+      }    
+    }
+
+    PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, aFlags);
+
+    if ((0 != offset.x) || (0 != offset.y)) {
+      aRenderingContext.PopState(clipState);
+    }
+    else { 
+      if ((NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow) || HasPctOverHeight()) {
+        aRenderingContext.PopState(clipState);         
+      }
     }
   } 
   
@@ -582,10 +618,9 @@ void nsTableCellFrame::VerticallyAlignChild(nsIPresContext*          aPresContex
   const nsStyleTextReset* textStyle =
       (const nsStyleTextReset*)mStyleContext->GetStyleData(eStyleStruct_TextReset);
   /* XXX: remove tableFrame when border-collapse inherits */
-  nsTableFrame* tableFrame = nsnull;
-  (void) nsTableFrame::GetTableFrame(this, tableFrame);
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
   nsMargin borderPadding;
-  GetCellBorder (borderPadding, tableFrame);
+  GetBorderWidth (p2t, borderPadding);
   nsMargin padding = nsTableFrame::GetPadding(aReflowState, this);
   borderPadding += padding;
   
@@ -676,15 +711,14 @@ nsTableCellFrame::HasVerticalAlignBaseline()
 PRInt32 nsTableCellFrame::GetRowSpan()
 {  
   PRInt32 rowSpan=1;
-  nsIHTMLContent *hc=nsnull;
-  nsresult rv = mContent->QueryInterface(kIHTMLContentIID, (void**) &hc);
-  if (NS_OK==rv) {
+  nsCOMPtr<nsIHTMLContent> hc(do_QueryInterface(mContent));
+
+  if (hc) {
     nsHTMLValue val;
     hc->GetHTMLAttribute(nsHTMLAtoms::rowspan, val); 
     if (eHTMLUnit_Integer == val.GetUnit()) { 
        rowSpan=val.GetIntValue(); 
     }
-    NS_RELEASE(hc);
   }
   return rowSpan;
 }
@@ -692,15 +726,14 @@ PRInt32 nsTableCellFrame::GetRowSpan()
 PRInt32 nsTableCellFrame::GetColSpan()
 {  
   PRInt32 colSpan=1;
-  nsIHTMLContent *hc=nsnull;
-  nsresult rv = mContent->QueryInterface(kIHTMLContentIID, (void**) &hc);
-  if (NS_OK==rv) {
+  nsCOMPtr<nsIHTMLContent> hc(do_QueryInterface(mContent));
+
+  if (hc) {
     nsHTMLValue val;
     hc->GetHTMLAttribute(nsHTMLAtoms::colspan, val); 
     if (eHTMLUnit_Integer == val.GetUnit()) { 
        colSpan=val.GetIntValue(); 
     }
-    NS_RELEASE(hc);
   }
   return colSpan;
 }
@@ -759,20 +792,6 @@ CalcUnpaginagedHeight(nsIPresContext*       aPresContext,
   return computedHeight;
 }
 
-static nscoord
-CalcHeightOfPrevInFlows(nsTableCellFrame& aCell)
-{
-  nscoord height = 0;
-  nsIFrame* prevInFlow;
-  for (aCell.GetPrevInFlow(&prevInFlow); prevInFlow; prevInFlow->GetPrevInFlow(&prevInFlow)) {
-    nsRect rect;
-    prevInFlow->GetRect(rect);
-    height += rect.height;
-  }
-  return height;
-}
-
-
 NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
                                    nsHTMLReflowMetrics&     aDesiredSize,
                                    const nsHTMLReflowState& aReflowState,
@@ -783,7 +802,6 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
 #if defined DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState);
 #endif
-
   float p2t;
   aPresContext->GetScaledPixelsToTwips(&p2t);
 
@@ -795,14 +813,10 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
 
   nsresult rv = NS_OK;
 
-  if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableWidth)     &&
-      ((NS_UNCONSTRAINEDSIZE == aReflowState.mComputedHeight) || 
-       (0                    == aReflowState.mComputedHeight))  && 
-      !mPrevInFlow                                              && 
-      nsTableFrame::IsPctHeight(mStyleContext)) {
-    nsTableFrame::NotifyAncestorsOfSpecialReflow(*this);
-    SetNeedSpecialReflow(PR_TRUE);
-  }
+  // see if a special height reflow needs to occur due to having a pct height
+  if (!NeedSpecialReflow()) 
+    nsTableFrame::CheckRequestSpecialHeightReflow(aReflowState);
+
   // this should probably be cached somewhere
   nsCompatibility compatMode;
   aPresContext->GetCompatibilityMode(&compatMode);
@@ -822,13 +836,13 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
 
   PRBool contentEmptyBeforeReflow = GetContentEmpty();
   /* XXX: remove tableFrame when border-collapse inherits */
-  nsTableFrame* tableFrame=nsnull;
-  rv = nsTableFrame::GetTableFrame(this, tableFrame);
+  nsTableFrame* tableFrame = nsnull;
+  rv = nsTableFrame::GetTableFrame(this, tableFrame); if (!tableFrame) ABORT1(NS_ERROR_NULL_POINTER);
   nsTableFrame* tableFrameFirstInFlow = (nsTableFrame*)tableFrame->GetFirstInFlow();
 
   nsMargin borderPadding = aReflowState.mComputedPadding;
   nsMargin border;
-  GetCellBorder(border, tableFrame);
+  GetBorderWidth(p2t, border);
   if ((NS_UNCONSTRAINEDSIZE == availSize.width) || !contentEmptyBeforeReflow) {
     borderPadding += border;
   }
@@ -846,29 +860,19 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
 
   PRBool  isStyleChanged = PR_FALSE;
   if (eReflowReason_Incremental == aReflowState.reason) {
-    // We *must* do this otherwise incremental reflow that's
-    // passing through will not work right.
-    nsIFrame* next;
-    aReflowState.reflowCommand->GetNext(next);
-
-    // if it is a StyleChanged reflow targeted at this cell frame,
-    // handle that here
-    // first determine if this frame is the target or not
-    nsIFrame *target=nsnull;
-    rv = aReflowState.reflowCommand->GetTarget(target);
-    if ((PR_TRUE==NS_SUCCEEDED(rv)) && target) {
-      if (this == target) {
-        nsReflowType type;
-        aReflowState.reflowCommand->GetType(type);
-        if (eReflowType_StyleChanged == type) {
-          isStyleChanged = PR_TRUE;
-        }
-        else {
-          NS_ASSERTION(PR_FALSE, "table cell target of illegal incremental reflow type");
-        }
+    // if the path has a reflow command then the cell must be the target of a style change 
+    nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
+    if (command) {
+      // if there are other reflow commands targeted at the cell's block, these will
+      // be subsumed by the style change reflow
+      nsReflowType type;
+      command->GetType(type);
+      if (eReflowType_StyleChanged == type) {
+        isStyleChanged = PR_TRUE;
       }
+      else NS_ASSERTION(PR_FALSE, "table cell target of illegal incremental reflow type");
     }
-    // if any of these conditions are not true, we just pass the reflow command down
+    // else the reflow command will be passed down to the child
   }
 
   // Try to reflow the child into the available space. It might not
@@ -886,8 +890,7 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
   nscoord computedPaginatedHeight = 0;
 
   if (aReflowState.mFlags.mSpecialHeightReflow || 
-      (HadSpecialReflow() && ((eReflowReason_Incremental == aReflowState.reason) || 
-                              (eReflowReason_Resize == aReflowState.reason)))) {
+      (HadSpecialReflow() && (eReflowReason_Incremental == aReflowState.reason))) {
     ((nsHTMLReflowState&)aReflowState).mComputedHeight = mRect.height - topInset - bottomInset;
     DISPLAY_REFLOW_CHANGE();
   }
@@ -901,19 +904,19 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
   else {
     SetHasPctOverHeight(PR_FALSE);
   }
-  nsHTMLReflowState kidReflowState(aPresContext, aReflowState, firstKid, availSize);
-  // mIPercentHeightObserver is for non table related frames inside cells
-  kidReflowState.mPercentHeightObserver = (nsIPercentHeightObserver *)this;
 
-  // If it was a style change targeted at us, then reflow the child using
-  // the special reflow reason
+  // If it was a style change targeted at us, then reflow the child with a style change reason
+  nsReflowReason reason = aReflowState.reason;
   if (isStyleChanged) {
-    kidReflowState.reason = eReflowReason_StyleChange;
-    kidReflowState.reflowCommand = nsnull;
+    reason = eReflowReason_StyleChange;
     // the following could be optimized with a fair amount of effort
     tableFrame->SetNeedStrategyInit(PR_TRUE);
   }
 
+  nsHTMLReflowState kidReflowState(aPresContext, aReflowState, firstKid, availSize, reason);
+  // mIPercentHeightObserver is for non table related frames inside cells
+  kidReflowState.mPercentHeightObserver = (nsIPercentHeightObserver *)this;
+
   // Assume the inner child will stay positioned exactly where it is. Later in
   // VerticallyAlignChild() we'll move it if it turns out to be wrong. This
   // avoids excessive movement and is more stable
@@ -1013,7 +1016,7 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
     if ((pos->mHeight.GetUnit() != eStyleUnit_Coord) &&
         (pos->mHeight.GetUnit() != eStyleUnit_Percent)) {
       // Standard mode should probably be 0 pixels high instead of 1
-      PRInt32 pixHeight = (eCompatibility_Standard == compatMode) ? 1 : 2;
+      PRInt32 pixHeight = (eCompatibility_NavQuirks == compatMode) ? 2 : 1;
       kidSize.height = NSIntPixelsToTwips(pixHeight, p2t);
       if ((nsnull != aDesiredSize.maxElementSize) && (0 == pMaxElementSize->height))
         pMaxElementSize->height = kidSize.height;
@@ -1061,13 +1064,6 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
   aDesiredSize.ascent  += kidSize.ascent;
   aDesiredSize.descent += kidSize.descent;
 
-  if (aDesiredSize.mFlags & NS_REFLOW_CALC_MAX_WIDTH) {
-    aDesiredSize.mMaximumWidth = kidSize.mMaximumWidth;
-    if (NS_UNCONSTRAINEDSIZE != aDesiredSize.mMaximumWidth) {
-      aDesiredSize.mMaximumWidth += leftInset + rightInset;
-      aDesiredSize.mMaximumWidth = nsTableFrame::RoundToPixel(aDesiredSize.mMaximumWidth, p2t);
-    }
-  }
   if (aDesiredSize.maxElementSize) {
     *aDesiredSize.maxElementSize = *pMaxElementSize;
     if ((0 != pMaxElementSize->height) && (NS_UNCONSTRAINEDSIZE != pMaxElementSize->height)) {
@@ -1080,6 +1076,17 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
       aDesiredSize.maxElementSize->width = nsTableFrame::RoundToPixel(aDesiredSize.maxElementSize->width, p2t);
     }
   }
+  if (aDesiredSize.mFlags & NS_REFLOW_CALC_MAX_WIDTH) {
+    aDesiredSize.mMaximumWidth = kidSize.mMaximumWidth;
+    if (NS_UNCONSTRAINEDSIZE != aDesiredSize.mMaximumWidth) {
+      aDesiredSize.mMaximumWidth += leftInset + rightInset;
+      aDesiredSize.mMaximumWidth = nsTableFrame::RoundToPixel(aDesiredSize.mMaximumWidth, p2t);
+    }
+    // make sure the preferred width is at least as big as the max element width
+    if (aDesiredSize.maxElementSize) {
+      aDesiredSize.mMaximumWidth = PR_MAX(aDesiredSize.mMaximumWidth, aDesiredSize.maxElementSize->width);
+    }
+  }
 
   if (aReflowState.mFlags.mSpecialHeightReflow) {
     if (aDesiredSize.height > mRect.height) {
@@ -1087,37 +1094,23 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
       // the height that they could honor in the pass 2 reflow
       SetHasPctOverHeight(PR_TRUE);
     }
-    aDesiredSize.height = mRect.height;
+    if (NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) {
+      aDesiredSize.height = mRect.height;
+    }
     SetNeedSpecialReflow(PR_FALSE);
     SetHadSpecialReflow(PR_TRUE);
   }
-  else if (HadSpecialReflow() && ((eReflowReason_Incremental == aReflowState.reason) || 
-                                  (eReflowReason_Resize == aReflowState.reason))) {
-    // if the block height value hasn't changed, use the last height of the cell, otherwise ignore it
-    if (GetLastBlockHeight() == priorBlockHeight) {
-      aDesiredSize.height = mRect.height;
-    }
-    else {
-      SetHadSpecialReflow(PR_FALSE);
-    }
-  }
-  else if (computedPaginatedHeight > 0) {
-    nscoord height = computedPaginatedHeight + topInset + bottomInset - CalcHeightOfPrevInFlows(*this);
-    if (NS_FRAME_COMPLETE == aStatus) {
-      if (mPrevInFlow) height -= topInset;
-      height = PR_MAX(aDesiredSize.height, height);
-      if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableHeight) && (height > aReflowState.availableHeight)) {
-        height = aReflowState.availableHeight;
-        aStatus = NS_FRAME_NOT_COMPLETE;
+  else if (HadSpecialReflow()) {
+    if (eReflowReason_Incremental == aReflowState.reason) {
+      // with an unconstrained height, if the block height value hasn't changed, 
+      // use the last height of the cell.
+      if ((NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) && 
+          (GetLastBlockHeight() == priorBlockHeight)) {
+        aDesiredSize.height = mRect.height;
       }
     }
-    else {
-      height -= bottomInset;
-      if (aDesiredSize.height < height) {
-        height = aDesiredSize.height;
-      }
-    }
-    aDesiredSize.height = height;
+    // XXX should probably call SetHadSpecialReflow(PR_FALSE) when things change so that
+    // nothing inside the cell has a percent height, but it is not easy determining this 
   }
 
   // remember the desired size for this reflow
@@ -1127,54 +1120,18 @@ NS_METHOD nsTableCellFrame::Reflow(nsIPresContext*          aPresContext,
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState, &aDesiredSize, aStatus);
 #endif
 
+  if (NS_UNCONSTRAINEDSIZE == aReflowState.availableWidth) {
+    SetNeedPass2Reflow(PR_TRUE);
+  }
+  else if ((eReflowReason_Initial == aReflowState.reason) || 
+           (eReflowReason_Resize  == aReflowState.reason)) { 
+    SetNeedPass2Reflow(PR_FALSE);
+  }
+
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
-/**
-  *
-  * Update the border style to map to the HTML border style
-  *
-  */
-void nsTableCellFrame::MapHTMLBorderStyle(nsIPresContext* aPresContext, 
-                                          nsStyleBorder&  aBorderStyle,
-                                          nsTableFrame*   aTableFrame)
-{
-  //adjust the border style based on the table rules attribute
-
-  /* The RULES code below has been disabled because collapsing borders have been disabled 
-     and RULES depend on collapsing borders
-
-  const nsStyleTable* tableStyle;
-  aTableFrame->GetStyleData(eStyleStruct_Table, (const nsStyleStruct *&)tableStyle);
-
-  switch (tableStyle->mRules)
-  {
-  case NS_STYLE_TABLE_RULES_NONE:
-    aBorderStyle.SetBorderStyle(NS_SIDE_TOP, NS_STYLE_BORDER_STYLE_NONE);
-    aBorderStyle.SetBorderStyle(NS_SIDE_LEFT, NS_STYLE_BORDER_STYLE_NONE);
-    aBorderStyle.SetBorderStyle(NS_SIDE_BOTTOM, NS_STYLE_BORDER_STYLE_NONE);
-    aBorderStyle.SetBorderStyle(NS_SIDE_RIGHT, NS_STYLE_BORDER_STYLE_NONE);
-    break;
-
-  case NS_STYLE_TABLE_RULES_COLS:
-	  aBorderStyle.SetBorderStyle(NS_SIDE_TOP, NS_STYLE_BORDER_STYLE_NONE);
-    aBorderStyle.SetBorderStyle(NS_SIDE_BOTTOM, NS_STYLE_BORDER_STYLE_NONE);
-    break;
-
-  case NS_STYLE_TABLE_RULES_ROWS:
-    aBorderStyle.SetBorderStyle(NS_SIDE_LEFT, NS_STYLE_BORDER_STYLE_NONE);
-    aBorderStyle.SetBorderStyle(NS_SIDE_RIGHT, NS_STYLE_BORDER_STYLE_NONE);
-    break;
-
-  default:
-    // do nothing for "GROUPS" or "ALL" or for any illegal value
-    // "GROUPS" will be handled in nsTableFrame::ProcessGroupRules
-    break;
-  }
-  */
-}
-
-
 PRBool nsTableCellFrame::ConvertToPixelValue(nsHTMLValue& aValue, PRInt32 aDefault, PRInt32& aResult)
 {
   if (aValue.GetUnit() == eHTMLUnit_Pixel)
@@ -1203,10 +1160,6 @@ void nsTableCellFrame::MapBorderPadding(nsIPresContext* aPresContext)
   // get the table frame style context, and from it get cellpadding, cellspacing, and border info
   const nsStyleTable* tableStyle;
   tableFrame->GetStyleData(eStyleStruct_Table, (const nsStyleStruct *&)tableStyle);
- 
-  float p2t;
-  aPresContext->GetPixelsToTwips(&p2t);
-
 
   MapVAlignAttribute(aPresContext, tableFrame);
   MapHAlignAttribute(aPresContext, tableFrame);
@@ -1401,13 +1354,16 @@ nsTableCellFrame::GetNextCellInColumn(nsITableCellLayout **aCellLayout)
 }
 
 nsresult 
-NS_NewTableCellFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
+NS_NewTableCellFrame(nsIPresShell* aPresShell, 
+                     PRBool        aIsBorderCollapse,
+                     nsIFrame**    aNewFrame)
 {
   NS_PRECONDITION(aNewFrame, "null OUT ptr");
   if (nsnull == aNewFrame) {
     return NS_ERROR_NULL_POINTER;
   }
-  nsTableCellFrame* it = new (aPresShell) nsTableCellFrame;
+  nsTableCellFrame* it = (aIsBorderCollapse) ? new (aPresShell) nsBCTableCellFrame 
+                                             : new (aPresShell) nsTableCellFrame;
   if (nsnull == it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1415,27 +1371,16 @@ NS_NewTableCellFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
   return NS_OK;
 }
 
-
-
-/* ----- methods from CellLayoutData ----- */
-
-void 
-nsTableCellFrame::GetCellBorder(nsMargin&     aBorder, 
-                                nsTableFrame* aTableFrame)
+nsMargin* 
+nsTableCellFrame::GetBorderWidth(float      aPixelsToTwips,
+                                 nsMargin&  aBorder) const
 {
   aBorder.left = aBorder.right = aBorder.top = aBorder.bottom = 0;
-  if (nsnull==aTableFrame) {
-    return;
-  }
 
-  if (NS_STYLE_BORDER_SEPARATE == aTableFrame->GetBorderCollapseStyle()) {
-    const nsStyleBorder* borderData;
-    GetStyleData(eStyleStruct_Border, (const nsStyleStruct*&)borderData);
-    borderData->GetBorder(aBorder);
-  } 
-  else {
-    NS_ASSERTION(PR_FALSE, "not implemented");
-  }
+  const nsStyleBorder* borderData;
+  GetStyleData(eStyleStruct_Border, (const nsStyleStruct*&)borderData);
+  borderData->GetBorder(aBorder);
+  return &aBorder;
 }
 
 NS_IMETHODIMP
@@ -1502,3 +1447,139 @@ nsTableCellFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
   return NS_OK;
 }
 #endif
+
+// nsBCTableCellFrame
+
+nsBCTableCellFrame::nsBCTableCellFrame()
+:nsTableCellFrame()
+{
+  mTopBorder = mRightBorder = mBottomBorder = mLeftBorder = 0;
+}
+
+nsBCTableCellFrame::~nsBCTableCellFrame()
+{
+}
+
+NS_IMETHODIMP
+nsBCTableCellFrame::GetFrameType(nsIAtom** aType) const
+{
+  NS_PRECONDITION(nsnull != aType, "null OUT parameter pointer");
+  *aType = nsLayoutAtoms::bcTableCellFrame; 
+  NS_ADDREF(*aType);
+  return NS_OK;
+}
+
+#ifdef DEBUG
+NS_IMETHODIMP
+nsBCTableCellFrame::GetFrameName(nsAString& aResult) const
+{
+  return MakeFrameName(NS_LITERAL_STRING("BCTableCell"), aResult);
+}
+#endif
+
+void 
+nsBCTableCellFrame::SetBorderWidth(const nsMargin& aBorder)
+{
+  mTopBorder    = aBorder.top;
+  mRightBorder  = aBorder.right;
+  mBottomBorder = aBorder.bottom;
+  mLeftBorder   = aBorder.left;
+}
+
+nsMargin* 
+nsBCTableCellFrame::GetBorderWidth(float      aPixelsToTwips,
+                                   nsMargin&  aBorder) const
+{
+  aBorder.top    = (aPixelsToTwips) ? NSToCoordRound(aPixelsToTwips * mTopBorder) :   mTopBorder;
+  aBorder.right  = (aPixelsToTwips) ? NSToCoordRound(aPixelsToTwips * mRightBorder) : mRightBorder;
+  aBorder.bottom = (aPixelsToTwips) ? NSToCoordRound(aPixelsToTwips * mBottomBorder): mBottomBorder;
+  aBorder.left   = (aPixelsToTwips) ? NSToCoordRound(aPixelsToTwips * mLeftBorder):   mLeftBorder;
+  return &aBorder;
+}
+
+nscoord 
+nsBCTableCellFrame::GetBorderWidth(PRUint8 aSide) const
+{
+  switch(aSide) {
+  case NS_SIDE_TOP:
+    return (PRUint8)mTopBorder;
+  case NS_SIDE_RIGHT:
+    return (PRUint8)mRightBorder;
+  case NS_SIDE_BOTTOM:
+    return (PRUint8)mBottomBorder;
+  default:
+    return (PRUint8)mLeftBorder;
+  }
+}
+
+void 
+nsBCTableCellFrame::SetBorderWidth(PRUint8 aSide,
+                                   nscoord aValue)
+{
+  switch(aSide) {
+  case NS_SIDE_TOP:
+    mTopBorder = aValue;
+    break;
+  case NS_SIDE_RIGHT:
+    mRightBorder = aValue;
+    break;
+  case NS_SIDE_BOTTOM:
+    mBottomBorder = aValue;
+    break;
+  default:
+    mLeftBorder = aValue;
+  }
+}
+
+void
+nsBCTableCellFrame::PaintUnderlay(nsIPresContext&           aPresContext,
+                                  nsIRenderingContext&      aRenderingContext,
+                                  const nsRect&             aDirtyRect,
+                                  PRUint32&                 aFlags,
+                                  const nsStyleTableBorder& aCellTableStyle,
+                                  const nsStyleBorder&      aStyleBorder,
+                                  PRBool                    aVisibleBackground,
+                                  PRBool&                   aPaintChildren)
+{
+  // Draw the background only during pass1.
+  if (aVisibleBackground && !(aFlags & BORDER_COLLAPSE_BACKGROUNDS)) { 
+    // make border-width reflect border-collapse assigned border
+    GET_PIXELS_TO_TWIPS(&aPresContext, p2t);
+    nsMargin borderWidth;
+    GetBorderWidth(p2t, borderWidth);
+
+    nsStyleBorder myBorder = aStyleBorder;
+
+    nsStyleCoord coord(borderWidth.top);
+    myBorder.mBorder.SetTop(coord);
+    coord.SetCoordValue(borderWidth.right);
+    myBorder.mBorder.SetRight(coord);
+    coord.SetCoordValue(borderWidth.bottom);
+    myBorder.mBorder.SetBottom(coord);
+    coord.SetCoordValue(borderWidth.left);
+    myBorder.mBorder.SetLeft(coord);
+    myBorder.RecalcData();
+
+    nsRect rect(0, 0, mRect.width, mRect.height);
+    nsCSSRendering::PaintBackground(&aPresContext, aRenderingContext, this,
+                                    aDirtyRect, rect, myBorder, 0, 0, PR_TRUE);
+    // borders are painted by nsTableFrame
+  }
+
+  // don't paint the children if it's pass1
+  aPaintChildren = (aFlags & BORDER_COLLAPSE_BACKGROUNDS);  
+}
+
+#ifdef DEBUG
+NS_IMETHODIMP
+nsBCTableCellFrame::SizeOf(nsISizeOfHandler* aHandler, 
+                           PRUint32*         aResult) const
+{
+  if (!aResult) {
+    return NS_ERROR_NULL_POINTER;
+  }
+  PRUint32 sum = sizeof(*this);
+  *aResult = sum;
+  return NS_OK;
+}
+#endif
