@@ -39,14 +39,13 @@
 #include "nsTableFrame.h"
 #include "nsIHTMLTableColElement.h"
 #include "nsIDOMHTMLTableColElement.h"
-#include "nsHTMLReflowCommand.h"
+#include "nsReflowPath.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIHTMLContent.h"
 #include "nsHTMLParts.h"
 #include "nsHTMLAtoms.h"
-#include "nsHTMLIIDs.h"
 #include "nsCOMPtr.h"
 #include "nsCSSRendering.h"
 #include "nsIPresShell.h"
@@ -250,29 +249,6 @@ nsTableColGroupFrame::SetInitialChildList(nsIPresContext* aPresContext,
     return NS_OK; 
   }
 
-  nsIFrame* kidFrame = aChildList;
-  while (kidFrame) {
-    nsIAtom* kidType;
-    kidFrame->GetFrameType(&kidType);
-    if (nsLayoutAtoms::tableColFrame == kidType) {
-      // Set the preliminary values for the column frame
-      PRInt32 span = ((nsTableColFrame*)kidFrame)->GetSpan();
-      if (span > 1) {
-        nsTableColFrame* firstSpannedCol;
-        tableFrame->CreateAnonymousColFrames(*aPresContext, *this, span - 1, eColAnonymousCol,
-                                             PR_FALSE, (nsTableColFrame*)kidFrame, (nsIFrame **)&firstSpannedCol);
-        nsIFrame* spanner = kidFrame;
-        kidFrame->GetNextSibling(&kidFrame); // need to do this before we insert the new frames
-        nsFrameList newChildren(aChildList); // used as a convience to hook up siblings
-        newChildren.InsertFrames(this, (nsTableColFrame*)spanner, (nsIFrame *)firstSpannedCol);
-        NS_RELEASE(kidType);
-        continue;
-      }
-    }
-    NS_IF_RELEASE(kidType);
-    kidFrame->GetNextSibling(&kidFrame); 
-  }
-
   mFrames.AppendFrames(this, aChildList);
   return NS_OK;
 }
@@ -433,21 +409,9 @@ nsTableColGroupFrame::Paint(nsIPresContext*      aPresContext,
   if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_FALSE, &isVisible)) && !isVisible) {
     return NS_OK;
   }
-  if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
-    nsCompatibility mode;
-    aPresContext->GetCompatibilityMode(&mode);
-    if (eCompatibility_Standard == mode) {
-      const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-      if (vis->IsVisibleOrCollapsed()) {
-        const nsStyleBorder* border =
-          (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
-        nsRect rect(0, 0, mRect.width, mRect.height);
-        nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *border, 0, 0);
-      }
-    }
-  }
+
+  // Standards mode background painting removed.  See bug 4510
+
   PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
   return NS_OK;
 }
@@ -472,7 +436,7 @@ nsTableColGroupFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                                    nsIFrame**     aFrame)
 {
   // this should act like a block, so we need to override
-  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND), aFrame);
+  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, PR_FALSE, aFrame);
 }
 
 NS_METHOD nsTableColGroupFrame::Reflow(nsIPresContext*          aPresContext,
@@ -515,6 +479,7 @@ NS_METHOD nsTableColGroupFrame::Reflow(nsIPresContext*          aPresContext,
     aDesiredSize.maxElementSize->height=0;
   }
   aStatus = NS_FRAME_COMPLETE;
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -525,22 +490,18 @@ NS_METHOD nsTableColGroupFrame::IncrementalReflow(nsIPresContext*          aPres
 {
   nsresult  rv = NS_OK;
 
-  // determine if this frame is the target or not
-  nsIFrame *target=nsnull;
-  rv = aReflowState.reflowCommand->GetTarget(target);
-  if ((PR_TRUE==NS_SUCCEEDED(rv)) && (nsnull!=target))
-  {
-    if (this==target)
-      rv = IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
-    else
-    {
-      // Get the next frame in the reflow chain
-      nsIFrame* nextFrame;
-      aReflowState.reflowCommand->GetNext(nextFrame);
-      rv = IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, nextFrame);
-    }
-  }
-  return rv;
+  // the col group is a target if its path has a reflow command
+  nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
+  if (command)
+    IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
+
+  // see if the chidren are targets as well
+  nsReflowPath::iterator iter = aReflowState.path->FirstChild();
+  nsReflowPath::iterator end  = aReflowState.path->EndChildren();
+  for (; iter != end; ++iter)
+    IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, *iter);
+
+  return NS_OK;
 }
 
 NS_METHOD nsTableColGroupFrame::IR_TargetIsMe(nsIPresContext*          aPresContext,
@@ -551,9 +512,9 @@ NS_METHOD nsTableColGroupFrame::IR_TargetIsMe(nsIPresContext*          aPresCont
   nsresult rv = NS_OK;
   aStatus = NS_FRAME_COMPLETE;
   nsReflowType type;
-  aReflowState.reflowCommand->GetType(type);
+  aReflowState.path->mReflowCommand->GetType(type);
   nsIFrame *objectFrame;
-  aReflowState.reflowCommand->GetChildFrame(objectFrame); 
+  aReflowState.path->mReflowCommand->GetChildFrame(objectFrame); 
   const nsStyleDisplay *childDisplay=nsnull;
   if (nsnull!=objectFrame)
     objectFrame->GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)childDisplay));
