@@ -45,9 +45,7 @@
 #include "nsCSSRendering.h"
 #include "nsIContent.h"
 #include "nsVoidArray.h"
-#include "nsIPtr.h"
 #include "prinrval.h"
-#include "nsHTMLIIDs.h"
 #include "nsLayoutAtoms.h"
 #include "nsHTMLParts.h"
 #include "nsIPresShell.h"
@@ -233,7 +231,14 @@ nsTableOuterFrame::SetInitialChildList(nsIPresContext* aPresContext,
   }
   else {
     mFrames.SetFrames(aChildList);
-    mInnerTableFrame = aChildList;
+    mInnerTableFrame = nsnull;
+    if (aChildList) {
+      nsCOMPtr<nsIAtom> fType;
+      aChildList->GetFrameType(getter_AddRefs(fType));
+      if (nsLayoutAtoms::tableFrame == fType.get()) {
+        mInnerTableFrame = (nsTableFrame*)aChildList;
+      }
+    }
   }
 
   return NS_OK;
@@ -408,23 +413,30 @@ NS_IMETHODIMP nsTableOuterFrame::SetSelected(nsIPresContext* aPresContext,
   return result;
 }
 
-// GetParentStyleContextProvider:
-//  The innerTableFame is he parent style context provider
-//  Fortunately, we cache that as a data member, so just return the cached pointer value
-//
 NS_IMETHODIMP 
-nsTableOuterFrame::GetParentStyleContextProvider(nsIPresContext* aPresContext,
-                                                 nsIFrame** aProviderFrame, 
-                                                 nsContextProviderRelationship& aRelationship)
+nsTableOuterFrame::GetParentStyleContextFrame(nsIPresContext* aPresContext,
+                                              nsIFrame**      aProviderFrame,
+                                              PRBool*         aIsChild)
 {
-  NS_ASSERTION(aProviderFrame && aPresContext, "null argument: aPresContext and-or aProviderFrame");
-  if (aProviderFrame) {
-    // parent context provider is the innerTableFrame
-    NS_ASSERTION(mInnerTableFrame, "innerTableFrame should not be null");
-    *aProviderFrame = mInnerTableFrame;
-    aRelationship = eContextProvider_Descendant;
+  // The table outer frame and the (inner) table frame split the style
+  // data by giving the table frame the style context associated with
+  // the table content node and creating a style context for the outer
+  // frame that is a *child* of the table frame's style context,
+  // matching the :table-outer (should be :-moz-table-outer!)
+  // pseudo-element.  html.css has a rule that causes that
+  // pseudo-element (and thus the outer table) to inherit *some* style
+  // properties from the table frame.  The children of the table inherit
+  // directly from the inner table, and the outer table's style context
+  // is a leaf.
+
+  if (!mInnerTableFrame) {
+    *aProviderFrame = this;
+    *aIsChild = PR_FALSE;
+    return NS_ERROR_FAILURE;
   }
-  return ((aProviderFrame != nsnull) && (*aProviderFrame != nsnull)) ? NS_OK : NS_ERROR_FAILURE;
+  *aProviderFrame = mInnerTableFrame;
+  *aIsChild = PR_TRUE;
+  return NS_OK;
 }
 
 // INCREMENTAL REFLOW HELPER FUNCTIONS 
@@ -467,21 +479,46 @@ FixAutoMargins(nscoord            aAvailWidth,
   }
 }
 
+void
+nsTableOuterFrame::InitChildReflowState(nsIPresContext&    aPresContext,                     
+                                        nsHTMLReflowState& aReflowState)
+                                    
+{
+  nsMargin collapseBorder;
+  nsMargin collapsePadding(0,0,0,0);
+  nsMargin* pCollapseBorder  = nsnull;
+  nsMargin* pCollapsePadding = nsnull;
+  if ((aReflowState.frame == mInnerTableFrame) && (mInnerTableFrame->IsBorderCollapse())) {
+    if (mInnerTableFrame->NeedToCalcBCBorders()) {
+      mInnerTableFrame->CalcBCBorders(aPresContext);
+    }
+    pCollapseBorder  = mInnerTableFrame->GetBCBorder(aPresContext, PR_FALSE, collapseBorder);
+    pCollapsePadding = &collapsePadding;
+  }
+  aReflowState.Init(&aPresContext, -1, -1, pCollapseBorder, pCollapsePadding);
+}
+
 // get the margin and padding data. nsHTMLReflowState doesn't handle the
 // case of auto margins
 void
-GetMarginPadding(nsIPresContext*          aPresContext,                     
-                 const nsHTMLReflowState& aOuterRS,
-                 nsIFrame*                aChildFrame,
-                 nsMargin&                aMargin,
-                 nsMargin&                aMarginNoAuto,
-                 nsMargin&                aPadding)
+nsTableOuterFrame::GetMarginPadding(nsIPresContext*          aPresContext,                     
+                                    const nsHTMLReflowState& aOuterRS,
+                                    nsIFrame*                aChildFrame,
+                                    nsMargin&                aMargin,
+                                    nsMargin&                aMarginNoAuto,
+                                    nsMargin&                aPadding)
 {
+  if (!aPresContext) ABORT0();
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
   // construct a reflow state to compute margin and padding. Auto margins
   // will not be computed at this time.
+
+  // create an init the child reflow state
   nsHTMLReflowState childRS(aPresContext, aOuterRS, aChildFrame, 
                             nsSize(aOuterRS.availableWidth, aOuterRS.availableHeight), 
-                            eReflowReason_Resize);
+                            eReflowReason_Resize, PR_FALSE);
+  InitChildReflowState(*aPresContext, childRS);
+
   nsRect childRect;
   aChildFrame->GetRect(childRect);
   FixAutoMargins(aOuterRS.availableWidth, childRect.width, childRS); 
@@ -545,8 +582,8 @@ nsSize
 GetContainingBlockSize(const nsHTMLReflowState& aOuterRS)
 {
   nsSize size(0,0);
-  const nsHTMLReflowState* containRS = 
-    nsHTMLReflowState::GetContainingBlockReflowState(aOuterRS.parentReflowState);
+  const nsHTMLReflowState* containRS =
+    aOuterRS.parentReflowState->mCBReflowState;
 
   if (containRS) {
     size.width = containRS->mComputedWidth;
@@ -932,6 +969,7 @@ nsTableOuterFrame::OuterReflowChild(nsIPresContext*            aPresContext,
                                     nsReflowReason             aReflowReason,
                                     nsReflowStatus&            aStatus)
 { 
+  if (!aPresContext) ABORT1(NS_ERROR_NULL_POINTER);
   aMargin = aPadding = nsMargin(0,0,0,0);
 
   nscoord availWidth = GetChildAvailWidth(aPresContext, aChildFrame, aOuterRS,
@@ -947,9 +985,10 @@ nsTableOuterFrame::OuterReflowChild(nsIPresContext*            aPresContext,
     availHeight = nsTableFrame::RoundToPixel(availHeight, p2t, eAlwaysRoundDown);
   }
 
-  nsHTMLReflowState childRS(aPresContext, aOuterRS, aChildFrame,
-                            nsSize(availWidth, availHeight));
-  childRS.reason = aReflowReason;
+  // create and init the child reflow state
+  nsHTMLReflowState childRS(aPresContext, aOuterRS, aChildFrame, nsSize(availWidth, availHeight), 
+                            aReflowReason);
+  InitChildReflowState(*aPresContext, childRS);
   childRS.mPercentHeightObserver = nsnull; // the observer is for non table related frames inside cells
 
 
@@ -971,6 +1010,15 @@ nsTableOuterFrame::OuterReflowChild(nsIPresContext*            aPresContext,
     }
   }
 
+  // see if we need to reset top of page due to a caption
+  if (mCaptionFrame) {
+    PRUint8 captionSide = GetCaptionSide();
+    if (((NS_SIDE_BOTTOM == captionSide) && (mCaptionFrame == aChildFrame)) || 
+        ((NS_SIDE_TOP == captionSide) && (mInnerTableFrame == aChildFrame))) {
+      childRS.mFlags.mIsTopOfPage = PR_FALSE;
+    }
+  }
+
   // use the current position as a best guess for placement
   nsRect childRect;
   aChildFrame->GetRect(childRect);
@@ -1017,24 +1065,18 @@ nsTableOuterFrame::IncrementalReflow(nsIPresContext*          aPresContext,
                                      const nsHTMLReflowState& aReflowState,
                                      nsReflowStatus&          aStatus)
 {
-  nsresult  rv = NS_OK;
+  // the outer table is a target if its path has a reflow command
+  nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
+  if (command)
+    IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
 
-  // determine if this frame is the target or not
-  nsIFrame* target=nsnull;
-  rv = aReflowState.reflowCommand->GetTarget(target);
-  if (NS_SUCCEEDED(rv) && target) {
-    if (this == target) {
-      rv = IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
-    }
-    else {
-      // Get the next frame in the reflow chain
-      nsIFrame* nextFrame;
-      aReflowState.reflowCommand->GetNext(nextFrame);
-      NS_ASSERTION(nextFrame, "next frame in reflow command is null"); 
-      rv = IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, nextFrame);
-    }
-  }
-  return rv;
+  // see if the chidren are targets as well
+  nsReflowPath::iterator iter = aReflowState.path->FirstChild();
+  nsReflowPath::iterator end  = aReflowState.path->EndChildren();
+  for (; iter != end; ++iter)
+    IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, *iter);
+
+  return NS_OK;
 }
 
 nsresult 
@@ -1108,10 +1150,14 @@ nsTableOuterFrame::IR_TargetIsCaptionFrame(nsIPresContext*           aPresContex
   nsSize   containSize = GetContainingBlockSize(aOuterRS);
 
   // for now just reflow the table if a style changed. This should be improved
-  nsReflowType reflowCommandType;
-  aOuterRS.reflowCommand->GetType(reflowCommandType);
-  PRBool needInnerReflow = (eReflowType_StyleChanged == reflowCommandType)
-                            ? PR_TRUE : PR_FALSE;
+  PRBool needInnerReflow = PR_FALSE;
+  nsHTMLReflowCommand* command = aOuterRS.path->mReflowCommand;
+  if (command) {
+    nsReflowType reflowCommandType;
+    command->GetType(reflowCommandType);
+    if (eReflowType_StyleChanged == reflowCommandType)
+      needInnerReflow = PR_TRUE;
+  }
 
   if (mMinCaptionWidth != captionMES.width) {  
     // set the new caption min width, and set state to reflow the inner table if necessary
@@ -1230,17 +1276,17 @@ nsresult nsTableOuterFrame::IR_TargetIsMe(nsIPresContext*           aPresContext
                                           nsReflowStatus&           aStatus)
 {
   nsresult rv = NS_OK;
+  nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
   nsReflowType type;
-  aReflowState.reflowCommand->GetType(type);
+  command->GetType(type);
   nsIFrame* objectFrame;
-  aReflowState.reflowCommand->GetChildFrame(objectFrame); 
+  command->GetChildFrame(objectFrame); 
   switch (type) {
   case eReflowType_ReflowDirty:
      rv = IR_ReflowDirty(aPresContext, aDesiredSize, aReflowState, aStatus);
     break;
 
   case eReflowType_StyleChanged :    
-  case eReflowType_Timeout :    
     rv = IR_InnerTableReflow(aPresContext, aDesiredSize, aReflowState, aStatus);
     break;
 
@@ -1281,11 +1327,10 @@ nsTableOuterFrame::IR_InnerTableReflow(nsIPresContext*           aPresContext,
   // StyleChange reflow reasons down to the children so that they
   // don't over-optimize their reflow.
   nsReflowReason ReflowReason = eReflowReason_Incremental;
-  nsIFrame* target = nsnull;
-  aOuterRS.reflowCommand->GetTarget(target);
-  if (this == target) {
+  nsHTMLReflowCommand* command = aOuterRS.path->mReflowCommand;
+  if (command) {
     nsReflowType type;
-    aOuterRS.reflowCommand->GetType(type);
+    command->GetType(type);
     if (eReflowType_StyleChanged == type) {
       ReflowReason = eReflowReason_StyleChange;
     }
@@ -1520,10 +1565,15 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
     aDesiredSize.width  = mRect.width;
     aDesiredSize.height = mRect.height;
 
-    // We know our view (if we have one) has been positioned
-    // correctly, but it's up to us to make sure that our child views
-    // are correctly positioned, too.
-    nsContainerFrame::PositionChildViews(aPresContext, this);
+    // for floaters, our view has not been positioned yet as we have not been placed
+    //  - the block code will position our views after the floater is placed
+    if (aOuterRS.mStyleDisplay &&
+        !aOuterRS.mStyleDisplay->IsFloating()) {
+      // We know our view (if we have one) has been positioned
+      // correctly, but it's up to us to make sure that our child views
+      // are correctly positioned, too.
+      nsContainerFrame::PositionChildViews(aPresContext, this);
+    }
   }
   else if (eReflowReason_Incremental == aOuterRS.reason) {
     rv = IncrementalReflow(aPresContext, aDesiredSize, aOuterRS, aStatus);
@@ -1637,6 +1687,7 @@ NS_METHOD nsTableOuterFrame::Reflow(nsIPresContext*          aPresContext,
 #if defined DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aOuterRS, &aDesiredSize, aStatus);
 #endif
+  NS_FRAME_SET_TRUNCATION(aStatus, aOuterRS, aDesiredSize);
   return rv;
 }
 
