@@ -41,15 +41,14 @@
 #include "nsIPresContext.h"
 #include "nsIStyleContext.h"
 #include "nsStyleConsts.h"
-#include "nsIHTMLAttributes.h"
+#include "nsIHTMLContent.h"
 #include "nsHTMLAtoms.h"
 #include "nsIContent.h"
 #include "nsTableFrame.h"
 #include "nsTableCellFrame.h"
 #include "nsIView.h"
-#include "nsHTMLReflowCommand.h"
+#include "nsReflowPath.h"
 #include "nsCSSRendering.h"
-#include "nsHTMLIIDs.h"
 #include "nsLayoutAtoms.h"
 #include "nsHTMLParts.h"
 #include "nsTableColGroupFrame.h"
@@ -68,11 +67,6 @@ struct nsTableCellReflowState : public nsHTMLReflowState
                          const nsSize&            aAvailableSpace,
                          nsReflowReason           aReason);
 
-  nsTableCellReflowState(nsIPresContext*          aPresContext,
-                         const nsHTMLReflowState& aParentReflowState,
-                         nsIFrame*                aFrame,
-                         const nsSize&            aAvailableSpace);
-
   void FixUp(const nsSize& aAvailSpace);
 };
 
@@ -83,16 +77,43 @@ nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresCon
                                                nsReflowReason           aReason)
   :nsHTMLReflowState(aPresContext, aParentRS, aFrame, aAvailSpace, aReason)
 {
-  FixUp(aAvailSpace);
 }
 
-nsTableCellReflowState::nsTableCellReflowState(nsIPresContext*          aPresContext,
-                                               const nsHTMLReflowState& aParentRS,
-                                               nsIFrame*                aFrame,
-                                               const nsSize&            aAvailSpace)
-  :nsHTMLReflowState(aPresContext, aParentRS, aFrame, aAvailSpace)
+void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace)
 {
-  FixUp(aAvailSpace);
+  // fix the mComputed values during a pass 2 reflow since the cell can be a percentage base
+  if (NS_UNCONSTRAINEDSIZE != aAvailSpace.width) {
+    if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
+      mComputedWidth = aAvailSpace.width - mComputedBorderPadding.left - mComputedBorderPadding.right;
+      mComputedWidth = PR_MAX(0, mComputedWidth);
+    }
+    if (NS_UNCONSTRAINEDSIZE != mComputedHeight) {
+      if (NS_UNCONSTRAINEDSIZE != aAvailSpace.height) {
+        mComputedHeight = aAvailSpace.height - mComputedBorderPadding.top - mComputedBorderPadding.bottom;
+        mComputedHeight = PR_MAX(0, mComputedHeight);
+      }
+    }
+  }
+}
+
+void
+nsTableRowFrame::InitChildReflowState(nsIPresContext&         aPresContext,
+                                      const nsSize&           aAvailSize,
+                                      PRBool                  aBorderCollapse,
+                                      float                   aPixelsToTwips,
+                                      nsTableCellReflowState& aReflowState)                                    
+{
+  nsMargin collapseBorder;
+  nsMargin* pCollapseBorder = nsnull;
+  if (aBorderCollapse) {
+    // we only reflow cells, so don't need to check frame type
+    nsBCTableCellFrame* bcCellFrame = (nsBCTableCellFrame*)aReflowState.frame;
+    if (bcCellFrame) {
+      pCollapseBorder = bcCellFrame->GetBorderWidth(aPixelsToTwips, collapseBorder);
+    }
+  }
+  aReflowState.Init(&aPresContext, -1, -1, pCollapseBorder);
+  aReflowState.FixUp(aAvailSize);
 }
 
 void 
@@ -132,23 +153,6 @@ nsTableRowFrame::SetPctHeight(float  aPctValue,
   }
 }
 
-void nsTableCellReflowState::FixUp(const nsSize& aAvailSpace)
-{
-  // fix the mComputed values during a pass 2 reflow since the cell can be a percentage base
-  if (NS_UNCONSTRAINEDSIZE != aAvailSpace.width) {
-    if (NS_UNCONSTRAINEDSIZE != mComputedWidth) {
-      mComputedWidth = aAvailSpace.width - mComputedBorderPadding.left - mComputedBorderPadding.right;
-      mComputedWidth = PR_MAX(0, mComputedWidth);
-    }
-    if (NS_UNCONSTRAINEDSIZE != mComputedHeight) {
-      if (NS_UNCONSTRAINEDSIZE != aAvailSpace.height) {
-        mComputedHeight = aAvailSpace.height - mComputedBorderPadding.top - mComputedBorderPadding.bottom;
-        mComputedHeight = PR_MAX(0, mComputedHeight);
-      }
-    }
-  }
-}
-
 // 'old' is old cached cell's desired size
 // 'new' is new cell's size including style constraints
 static PRBool
@@ -217,7 +221,7 @@ nsTableRowFrame::SetInitialChildList(nsIPresContext* aPresContext,
        kidFrame->GetNextSibling(&kidFrame)) {
     nsCOMPtr<nsIAtom> frameType;
     kidFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       if (((nsTableCellFrame*)kidFrame)->GetRowSpan() > 1) {
         mState |= NS_FRAME_OUTSIDE_CHILDREN;
       }
@@ -241,7 +245,7 @@ nsTableRowFrame::AppendFrames(nsIPresContext* aPresContext,
   for (nsIFrame* childFrame = aFrameList; childFrame; childFrame->GetNextSibling(&childFrame)) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       // Add the cell to the cell map
       tableFrame->AppendCell(*aPresContext, (nsTableCellFrame&)*childFrame, GetRowIndex());
       // XXX this could be optimized with some effort
@@ -272,12 +276,13 @@ nsTableRowFrame::InsertFrames(nsIPresContext* aPresContext,
   nsTableFrame::GetTableFrame(this, tableFrame);
   
   // gather the new frames (only those which are cells) into an array
-  nsTableCellFrame* prevCellFrame = (nsTableCellFrame *)nsTableFrame::GetFrameAtOrBefore(aPresContext, this, aPrevFrame, nsLayoutAtoms::tableCellFrame);
+  nsIAtom* cellFrameType = (tableFrame->IsBorderCollapse()) ? nsLayoutAtoms::bcTableCellFrame : nsLayoutAtoms::tableCellFrame;
+  nsTableCellFrame* prevCellFrame = (nsTableCellFrame *)nsTableFrame::GetFrameAtOrBefore(aPresContext, this, aPrevFrame, cellFrameType);
   nsVoidArray cellChildren;
   for (nsIFrame* childFrame = aFrameList; childFrame; childFrame->GetNextSibling(&childFrame)) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       cellChildren.AppendElement(childFrame);
       // XXX this could be optimized with some effort
       tableFrame->SetNeedStrategyInit(PR_TRUE);
@@ -315,7 +320,7 @@ nsTableRowFrame::RemoveFrame(nsIPresContext* aPresContext,
   if (tableFrame) {
     nsCOMPtr<nsIAtom> frameType;
     aOldFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       nsTableCellFrame* cellFrame = (nsTableCellFrame*)aOldFrame;
       PRInt32 colIndex;
       cellFrame->GetColIndex(colIndex);
@@ -372,7 +377,7 @@ nsTableRowFrame::GetFirstCell()
   while (childFrame) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       return (nsTableCellFrame*)childFrame;
     }
     childFrame->GetNextSibling(&childFrame);
@@ -398,7 +403,7 @@ nsTableRowFrame::DidResize(nsIPresContext*          aPresContext,
   while (childFrame) {
     nsCOMPtr<nsIAtom> frameType;
     childFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       nsTableCellFrame* cellFrame = (nsTableCellFrame*)childFrame;
       nscoord cellHeight = mRect.height + GetHeightOfRowsSpannedBelowFirst(*cellFrame, *tableFrame);
 
@@ -534,7 +539,7 @@ nsTableRowFrame::CalcHeight(const nsHTMLReflowState& aReflowState)
   for (nsIFrame* kidFrame = mFrames.FirstChild(); kidFrame; kidFrame->GetNextSibling(&kidFrame)) {
     nsCOMPtr<nsIAtom> frameType;
     kidFrame->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    if (IS_TABLE_CELL(frameType.get())) {
       nscoord availWidth = ((nsTableCellFrame *)kidFrame)->GetPriorAvailWidth();
       nsSize desSize = ((nsTableCellFrame *)kidFrame)->GetDesiredSize();
       if ((NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) && !mPrevInFlow) {
@@ -578,22 +583,6 @@ NS_METHOD nsTableRowFrame::Paint(nsIPresContext*      aPresContext,
   if (NS_SUCCEEDED(IsVisibleForPainting(aPresContext, aRenderingContext, PR_FALSE, &isVisible)) && !isVisible) {
     return NS_OK;
   }
-  if ((NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) &&
-      !(aFlags && (NS_ROW_FRAME_PAINT_SKIP_ROW == aFlags))) {
-    nsCompatibility mode;
-    aPresContext->GetCompatibilityMode(&mode);
-    if (eCompatibility_Standard == mode) {
-      const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-      if (vis->IsVisibleOrCollapsed()) {
-        const nsStyleBorder* border =
-          (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
-        nsRect rect(0, 0, mRect.width, mRect.height);
-        nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                        aDirtyRect, rect, *border, 0, 0);
-      }
-    }
-  }
 
 #ifdef DEBUG
   // for debug...
@@ -602,19 +591,18 @@ NS_METHOD nsTableRowFrame::Paint(nsIPresContext*      aPresContext,
     aRenderingContext.DrawRect(0, 0, mRect.width, mRect.height);
   }
 #endif
+  // Standards mode background painting removed.  See bug 4510
 
-  if (!(aFlags && (NS_ROW_FRAME_PAINT_SKIP_CELLS == aFlags))) {
-    const nsStyleDisplay* disp = (const nsStyleDisplay*)
-    mStyleContext->GetStyleData(eStyleStruct_Display);
-    if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
-      aRenderingContext.PushState();
-      SetOverflowClipRect(aRenderingContext);
-    }
-    PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
-    if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
-      PRBool clipState;
-      aRenderingContext.PopState(clipState);
-    }
+  const nsStyleDisplay* disp = (const nsStyleDisplay*)
+  mStyleContext->GetStyleData(eStyleStruct_Display);
+  if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
+    aRenderingContext.PushState();
+    SetOverflowClipRect(aRenderingContext);
+  }
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, aFlags);
+  if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
+    PRBool clipState;
+    aRenderingContext.PopState(clipState);
   }
   return NS_OK;
 
@@ -661,7 +649,7 @@ void nsTableRowFrame::PaintChildren(nsIPresContext*      aPresContext,
         aRenderingContext.PushState();
         aRenderingContext.Translate(kidRect.x, kidRect.y);
         kid->Paint(aPresContext, aRenderingContext, kidDamageArea,
-                   aWhichLayer);
+                   aWhichLayer, aFlags);
 #ifdef DEBUG
         if ((NS_FRAME_PAINT_LAYER_DEBUG == aWhichLayer) &&
             GetShowFrameBorders()) {
@@ -718,15 +706,6 @@ nsTableRowFrame::GetFrameForPoint(nsIPresContext*        aPresContext,
     return NS_OK;
   }
 
-  if ( inThisFrame && (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND)) {
-    const nsStyleVisibility* vis = 
-      (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-    if (vis->IsVisible()) {
-      *aFrame = this;
-      return NS_OK;
-    }
-  }
-
   return NS_ERROR_FAILURE;
 }
 
@@ -874,6 +853,24 @@ GetComputedWidth(const nsHTMLReflowState& aReflowState,
   return computedWidth;
 }
 
+// subtract the heights of aRow's prev in flows from the unpaginated height
+static
+nscoord CalcHeightFromUnpaginatedHeight(nsIPresContext*  aPresContext,
+                                        nsTableRowFrame& aRow)
+{
+  nscoord height = 0;
+  nsTableRowFrame* firstInFlow = (nsTableRowFrame*)aRow.GetFirstInFlow(); if (!firstInFlow) ABORT1(0);
+  if (firstInFlow->HasUnpaginatedHeight()) {
+    height = firstInFlow->GetUnpaginatedHeight(aPresContext);
+    for (nsIFrame* prevInFlow = aRow.GetPrevInFlow(); prevInFlow; prevInFlow->GetPrevInFlow(&prevInFlow)) {
+      nsRect rect;
+      prevInFlow->GetRect(rect);
+      height -= rect.height;
+    }
+  }
+  return PR_MAX(height, 0);
+}
+
 // Called for a dirty or resize reflow. Reflows all the existing table cell 
 // frames unless aDirtyOnly is PR_TRUE in which case only reflow the dirty frames
 
@@ -886,44 +883,43 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
                                 PRBool                   aDirtyOnly)
 {
   aStatus = NS_FRAME_COMPLETE;
-  if (!mFrames.FirstChild()) return NS_OK;
 
-  nsTableFrame* tableFrame = &aTableFrame;
-  if (!tableFrame) return NS_ERROR_NULL_POINTER;
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
+  PRBool borderCollapse = (((nsTableFrame*)aTableFrame.GetFirstInFlow())->IsBorderCollapse());
+
   nsIFrame* tablePrevInFlow;
-  tableFrame->GetPrevInFlow(&tablePrevInFlow);
+  aTableFrame.GetPrevInFlow(&tablePrevInFlow);
   PRBool isPaginated;
   aPresContext->IsPaginated(&isPaginated);
 
   nsresult rv = NS_OK;
 
-  nscoord cellSpacingX = tableFrame->GetCellSpacingX();
+  nscoord cellSpacingX = aTableFrame.GetCellSpacingX();
   PRInt32 cellColSpan = 1;  // must be defined here so it's set properly for non-cell kids
   
   nsTableIteration dir = (aReflowState.availableWidth == NS_UNCONSTRAINEDSIZE)
                          ? eTableLTR : eTableDIR;
   nsTableIterator iter(aPresContext, *this, dir);
   // remember the col index of the previous cell to handle rowspans into this row
-  PRInt32 firstPrevColIndex = (iter.IsLeftToRight()) ? -1 : tableFrame->GetColCount();
+  PRInt32 firstPrevColIndex = (iter.IsLeftToRight()) ? -1 : aTableFrame.GetColCount();
   PRInt32 prevColIndex  = firstPrevColIndex;
   nscoord x = 0; // running total of children x offset
 
-  nsTableFrame* tableFirstInFlow = (nsTableFrame*)tableFrame->GetFirstInFlow();
-  PRBool isAutoLayout = tableFrame->IsAutoLayout();
+  nsTableFrame* tableFirstInFlow = (nsTableFrame*)aTableFrame.GetFirstInFlow();
+  PRBool isAutoLayout = aTableFrame.IsAutoLayout();
   PRBool needToNotifyTable = PR_TRUE;
   nscoord paginatedHeight = 0;
   // If the incremental reflow command is a StyleChanged reflow and
   // it's target is the current frame, then make sure we send
   // StyleChange reflow reasons down to the children so that they
   // don't over-optimize their reflow.
-  
-  nsIFrame* target = nsnull;
+
   PRBool notifyStyleChange = PR_FALSE;
   if (eReflowReason_Incremental == aReflowState.reason) {
-    aReflowState.reflowCommand->GetTarget(target);
-    if (this == target) {
+    nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
+    if (command) {
       nsReflowType type;
-      aReflowState.reflowCommand->GetType(type);
+      command->GetType(type);
       if (eReflowType_StyleChanged == type) {
         notifyStyleChange = PR_TRUE;
       }
@@ -945,7 +941,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
       doReflowChild = PR_FALSE;
     }
     if (aReflowState.mFlags.mSpecialHeightReflow) {
-      if (!isPaginated && (nsLayoutAtoms::tableCellFrame == frameType.get() &&
+      if (!isPaginated && (IS_TABLE_CELL(frameType.get()) &&
                            !((nsTableCellFrame*)kidFrame)->NeedSpecialReflow())) {
         kidFrame = iter.Next(); 
         continue;
@@ -954,22 +950,22 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
 
     // Reflow the child frame
     if (doReflowChild) {
-      if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+      if (IS_TABLE_CELL(frameType.get())) {
         nsTableCellFrame* cellFrame = (nsTableCellFrame*)kidFrame;
         PRInt32 cellColIndex;
         cellFrame->GetColIndex(cellColIndex);
-        cellColSpan = tableFrame->GetEffectiveColSpan(*cellFrame);
+        cellColSpan = aTableFrame.GetEffectiveColSpan(*cellFrame);
    
         x += cellSpacingX;
         // If the adjacent cell is in a prior row (because of a rowspan) add in the space
         if ((iter.IsLeftToRight() && (prevColIndex != (cellColIndex - 1))) ||
             (!iter.IsLeftToRight() && (prevColIndex != cellColIndex + cellColSpan))) {
-          x += GetSpaceBetween(prevColIndex, cellColIndex, cellColSpan, *tableFrame, 
+          x += GetSpaceBetween(prevColIndex, cellColIndex, cellColSpan, aTableFrame, 
                                cellSpacingX, iter.IsLeftToRight());
         }
         // Calculate the available width for the table cell using the known column widths
         nscoord availColWidth, availCellWidth;
-        CalcAvailWidth(*tableFrame, GetComputedWidth(aReflowState, *tableFrame),
+        CalcAvailWidth(aTableFrame, GetComputedWidth(aReflowState, aTableFrame),
                        *cellFrame, cellSpacingX, availColWidth, availCellWidth);
         if (0 == availColWidth)  availColWidth  = NS_UNCONSTRAINEDSIZE;
         if (0 == availCellWidth) availCellWidth = NS_UNCONSTRAINEDSIZE;
@@ -985,13 +981,16 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
         nsIFrame* kidNextInFlow;
         kidFrame->GetNextInFlow(&kidNextInFlow);
         nsSize cellDesiredSize = cellFrame->GetDesiredSize();
-        if ((availCellWidth != cellFrame->GetPriorAvailWidth())           ||
+        if ((availCellWidth != cellFrame->GetPriorAvailWidth())       ||
             (cellDesiredSize.width > cellFrame->GetPriorAvailWidth()) ||
             (eReflowReason_StyleChange == aReflowState.reason)        ||
             isPaginated                                               ||
+            (cellFrame->NeedPass2Reflow() &&
+             NS_UNCONSTRAINEDSIZE != aReflowState.availableWidth)     ||
             (aReflowState.mFlags.mSpecialHeightReflow && cellFrame->NeedSpecialReflow()) ||
-            HasPctHeight() ||
-            notifyStyleChange ){
+            (!aReflowState.mFlags.mSpecialHeightReflow && cellFrame->HadSpecialReflow()) ||
+            HasPctHeight()                                            ||
+            notifyStyleChange) {
           // Reflow the cell to fit the available width, height
           nsSize  kidAvailSize(availColWidth, aReflowState.availableHeight);
           nsReflowReason reason = eReflowReason_Resize;
@@ -1032,6 +1031,8 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
           // Reflow the child
           nsTableCellReflowState kidReflowState(aPresContext, aReflowState, 
                                                 kidFrame, kidAvailSize, reason);
+          InitChildReflowState(*aPresContext, kidAvailSize, borderCollapse, p2t, kidReflowState);
+
           nsReflowStatus status;
           rv = ReflowChild(kidFrame, aPresContext, desiredSize, kidReflowState,
                            x, 0, 0, status);
@@ -1052,7 +1053,7 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
 
           // allow the table to determine if/how the table needs to be rebalanced
           if (cellToWatch && needToNotifyTable) {
-            needToNotifyTable = !tableFrame->CellChangedWidth(*cellFrame, oldMaxWidth, oldMaxElemWidth);
+            needToNotifyTable = !aTableFrame.CellChangedWidth(*cellFrame, oldMaxWidth, oldMaxElemWidth);
           }
 
           // If any of the cells are not complete, then we're not complete
@@ -1065,28 +1066,35 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
           desiredSize.width = priorSize.width;
           desiredSize.height = priorSize.height;
 
-          // Because we may have moved the frame we need to make sure any views are
-          // positioned properly. We have to do this, because any one of our parent
-          // frames could have moved and we have no way of knowing...
-          nsTableFrame::RePositionViews(aPresContext, kidFrame);
+          // if we are in a floated table, our position is not yet established, so we cannot reposition our views
+          // the containing glock will do this for us after positioning the table
+          const nsStyleDisplay *display;
+          aTableFrame.GetStyleData(eStyleStruct_Display, ((const nsStyleStruct *&)display));
+          if (display && !display->IsFloating()) {
+            // Because we may have moved the frame we need to make sure any views are
+            // positioned properly. We have to do this, because any one of our parent
+            // frames could have moved and we have no way of knowing...
+            nsTableFrame::RePositionViews(aPresContext, kidFrame);
+          }
         }
         
-        if ((NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) && !mPrevInFlow) {
-          // Calculate the cell's actual size given its pass2 size. This function
-          // takes into account the specified height (in the style), and any special
-          // logic needed for backwards compatibility
-          CalculateCellActualSize(kidFrame, desiredSize.width, 
-                                  desiredSize.height, availCellWidth);
-
+        if (NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) {
+          if (!mPrevInFlow) {
+            // Calculate the cell's actual size given its pass2 size. This function
+            // takes into account the specified height (in the style), and any special
+            // logic needed for backwards compatibility
+            CalculateCellActualSize(kidFrame, desiredSize.width, 
+                                    desiredSize.height, availCellWidth);
+          }
           // height may have changed, adjust descent to absorb any excess difference
           nscoord ascent = cellFrame->GetDesiredAscent();
           nscoord descent = desiredSize.height - ascent;
-          UpdateHeight(desiredSize.height, ascent, descent, tableFrame, cellFrame);
+          UpdateHeight(desiredSize.height, ascent, descent, &aTableFrame, cellFrame);
         }
         else {
-          PRInt32 rowSpan = tableFrame->GetEffectiveRowSpan((nsTableCellFrame&)*kidFrame);
-          if ((1 == rowSpan) && (desiredSize.height > paginatedHeight)) {
-            paginatedHeight = desiredSize.height;
+          paginatedHeight = PR_MAX(paginatedHeight, desiredSize.height);
+          PRInt32 rowSpan = aTableFrame.GetEffectiveRowSpan((nsTableCellFrame&)*kidFrame);
+          if (1 == rowSpan) {
             SetContentHeight(paginatedHeight);
           }
         }
@@ -1102,13 +1110,14 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
       else {// it's an unknown frame type, give it a generic reflow and ignore the results
         nsTableCellReflowState kidReflowState(aPresContext, aReflowState,
                                               kidFrame, nsSize(0,0), eReflowReason_Resize);
+        InitChildReflowState(*aPresContext, nsSize(0,0), PR_FALSE, p2t, kidReflowState);
         nsHTMLReflowMetrics desiredSize(nsnull);
         nsReflowStatus  status;
         ReflowChild(kidFrame, aPresContext, desiredSize, kidReflowState, 0, 0, 0, status);
         kidFrame->DidReflow(aPresContext, nsnull, NS_FRAME_REFLOW_FINISHED);
       }
     }
-    else if (nsLayoutAtoms::tableCellFrame == frameType.get()) {
+    else if (IS_TABLE_CELL(frameType.get())) {
       // we need to account for the cell's width even if it isn't reflowed
       nsRect rect;
       kidFrame->GetRect(rect);
@@ -1124,8 +1133,35 @@ nsTableRowFrame::ReflowChildren(nsIPresContext*          aPresContext,
 
   // just set our width to what was available. The table will calculate the width and not use our value.
   aDesiredSize.width = aReflowState.availableWidth;
-  aDesiredSize.height = (NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) 
-                        ? CalcHeight(aReflowState) : paginatedHeight; 
+
+  if (aReflowState.mFlags.mSpecialHeightReflow) {
+    aDesiredSize.height = mRect.height;
+  }
+  else if (NS_UNCONSTRAINEDSIZE == aReflowState.availableHeight) {
+    aDesiredSize.height = CalcHeight(aReflowState);
+    if (mPrevInFlow) {
+      nscoord height = CalcHeightFromUnpaginatedHeight(aPresContext, *this);
+      aDesiredSize.height = PR_MAX(aDesiredSize.height, height);
+    }
+    else {
+      if (isPaginated && HasStyleHeight()) {
+        // set the unpaginated height so next in flows can try to honor it
+        SetHasUnpaginatedHeight(PR_TRUE);
+        SetUnpaginatedHeight(aPresContext, aDesiredSize.height);
+      }
+      if (isPaginated && HasUnpaginatedHeight()) {
+        aDesiredSize.height = PR_MAX(aDesiredSize.height, GetUnpaginatedHeight(aPresContext));
+      }
+    }
+  }
+  else { // constrained height, paginated
+    aDesiredSize.height = paginatedHeight;
+    if (aDesiredSize.height <= aReflowState.availableHeight) {
+      nscoord height = CalcHeightFromUnpaginatedHeight(aPresContext, *this);
+      aDesiredSize.height = PR_MAX(aDesiredSize.height, height);
+      aDesiredSize.height = PR_MIN(aDesiredSize.height, aReflowState.availableHeight);
+    }
+  }
 
   return rv;
 }
@@ -1136,23 +1172,20 @@ NS_METHOD nsTableRowFrame::IncrementalReflow(nsIPresContext*          aPresConte
                                              nsTableFrame&            aTableFrame,
                                              nsReflowStatus&          aStatus)
 {
-  nsresult rv = NS_OK;
   CalcHeight(aReflowState); // need to recalculate it based on last reflow sizes
  
-  // determine if this frame is the target or not
-  nsIFrame* target = nsnull;
-  rv = aReflowState.reflowCommand->GetTarget(target);
-  if (target) {
-    if (this == target)
-      rv = IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aTableFrame, aStatus);
-    else {
-      // Get the next frame in the reflow chain
-      nsIFrame* nextFrame;
-      aReflowState.reflowCommand->GetNext(nextFrame);
-      rv = IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aTableFrame, aStatus, nextFrame);
-    }
-  }
-  return rv;
+  // the row is a target if its path has a reflow command
+  nsHTMLReflowCommand* command = aReflowState.path->mReflowCommand;
+  if (command)
+    IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aTableFrame, aStatus);
+
+  // see if the chidren are targets as well
+  nsReflowPath::iterator iter = aReflowState.path->FirstChild();
+  nsReflowPath::iterator end  = aReflowState.path->EndChildren();
+  for (; iter != end; ++iter)
+    IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aTableFrame, aStatus, *iter);
+
+  return NS_OK;
 }
 
 NS_METHOD 
@@ -1165,7 +1198,7 @@ nsTableRowFrame::IR_TargetIsMe(nsIPresContext*          aPresContext,
   nsresult rv = NS_FRAME_COMPLETE;
 
   nsReflowType type;
-  aReflowState.reflowCommand->GetType(type);
+  aReflowState.path->mReflowCommand->GetType(type);
   switch (type) {
     case eReflowType_ReflowDirty: 
       // Reflow the dirty child frames. Typically this is newly added frames.
@@ -1244,7 +1277,10 @@ nsTableRowFrame::IR_TargetIsChild(nsIPresContext*          aPresContext,
     // update its maximum width. 
     nsHTMLReflowMetrics cellMet(&kidMaxElementSize, isAutoLayout ? 
                                                     NS_REFLOW_CALC_MAX_WIDTH : 0);
-    nsTableCellReflowState kidRS(aPresContext, aReflowState, aNextFrame, cellAvailSize);
+    GET_PIXELS_TO_TWIPS(aPresContext, p2t);
+    nsTableCellReflowState kidRS(aPresContext, aReflowState, aNextFrame, cellAvailSize, 
+                                 aReflowState.reason);
+    InitChildReflowState(*aPresContext, cellAvailSize, aTableFrame.IsBorderCollapse(), p2t, kidRS); 
 
     // Remember the current desired size, we'll need it later
     nscoord oldCellMinWidth     = cellFrame->GetPass1MaxElementWidth();
@@ -1395,30 +1431,13 @@ nsTableRowFrame::Reflow(nsIPresContext*          aPresContext,
   rv = nsTableFrame::GetTableFrame(this, tableFrame);
   if (!tableFrame) return NS_ERROR_NULL_POINTER;
 
-  if ((NS_UNCONSTRAINEDSIZE != aReflowState.availableWidth) &&  !mPrevInFlow) {
-      // see if an extra reflow will be necessary when there is a pct height but no height basis 
-    if ( ((NS_UNCONSTRAINEDSIZE == aReflowState.mComputedHeight)  || 
-          (0                    == aReflowState.mComputedHeight)) && 
-         nsTableFrame::IsPctHeight(mStyleContext)) {
-      const nsHTMLReflowState* parentRS = aReflowState.parentReflowState;
-      if (parentRS && parentRS->frame) {
-        nsCOMPtr<nsIAtom> fType;
-        parentRS->frame->GetFrameType(getter_AddRefs(fType));
-        if (nsLayoutAtoms::tableRowGroupFrame == fType.get()) {
-          nscoord pctBasis = ((nsTableRowGroupFrame*)parentRS->frame)->GetHeightBasis(*parentRS);
-          if (0 == pctBasis) {
-            nsTableFrame::NotifyAncestorsOfSpecialReflow(*this);
-            SetNeedSpecialReflow(PR_TRUE);
-          }
-        }
-      }
-    }
-  }
+  // see if a special height reflow needs to occur due to having a pct height
+  if (!NeedSpecialReflow()) 
+    nsTableFrame::CheckRequestSpecialHeightReflow(aReflowState);
 
   switch (aReflowState.reason) {
   case eReflowReason_Initial:
     rv = ReflowChildren(aPresContext, aDesiredSize, aReflowState, *tableFrame, aStatus, PR_FALSE);
-    aStatus = NS_FRAME_COMPLETE;
     break;
 
   case eReflowReason_Resize:
@@ -1445,6 +1464,7 @@ nsTableRowFrame::Reflow(nsIPresContext*          aPresContext,
 #if defined DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState, &aDesiredSize, aStatus);
 #endif
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -1492,13 +1512,20 @@ nsTableRowFrame::ReflowCellFrame(nsIPresContext*          aPresContext,
                                  nscoord                  aAvailableHeight,
                                  nsReflowStatus&          aStatus)
 {
+  nsTableFrame* tableFrame = nsnull;
+  nsTableFrame::GetTableFrame(this, tableFrame); if (!tableFrame) ABORT1(0);
+
   // Reflow the cell frame with the specified height. Use the existing width
   nsSize  cellSize;
   aCellFrame->GetSize(cellSize);
   
   nsSize  availSize(cellSize.width, aAvailableHeight);
+  PRBool borderCollapse = ((nsTableFrame*)tableFrame->GetFirstInFlow())->IsBorderCollapse();
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
   nsTableCellReflowState cellReflowState(aPresContext, aReflowState, aCellFrame, availSize,
                                          eReflowReason_Resize);
+  InitChildReflowState(*aPresContext, availSize, borderCollapse, p2t, cellReflowState);
+
   nsHTMLReflowMetrics desiredSize(nsnull);
 
   ReflowChild(aCellFrame, aPresContext, desiredSize, cellReflowState,
@@ -1534,7 +1561,7 @@ nsTableRowFrame::InsertCellFrame(nsTableCellFrame* aFrame,
   for (nsIFrame* child = mFrames.FirstChild(); child; child->GetNextSibling(&child)) {
     nsCOMPtr<nsIAtom> frameType;
     child->GetFrameType(getter_AddRefs(frameType));
-    if (nsLayoutAtoms::tableCellFrame != frameType.get()) {
+    if (!IS_TABLE_CELL(frameType.get())) {
       nsTableCellFrame* cellFrame = (nsTableCellFrame*)child;
       PRInt32 colIndex;
       cellFrame->GetColIndex(colIndex);
@@ -1550,7 +1577,8 @@ nsTableRowFrame::InsertCellFrame(nsTableCellFrame* aFrame,
 void 
 nsTableRowFrame::RemoveCellFrame(nsTableCellFrame* aFrame)
 {
-  mFrames.RemoveFrame(aFrame);
+  if (!mFrames.RemoveFrame(aFrame))
+    NS_ASSERTION(PR_FALSE, "frame not in list");
 }
 
 NS_IMETHODIMP
@@ -1582,6 +1610,7 @@ void
 nsTableRowFrame::SetUnpaginatedHeight(nsIPresContext* aPresContext,
                                       nscoord         aValue)
 {
+  NS_ASSERTION(!mPrevInFlow, "program error");
   // Get the property 
   nscoord* value = (nscoord*)nsTableFrame::GetProperty(aPresContext, this, nsLayoutAtoms::rowUnpaginatedHeightProperty, PR_TRUE);
   if (value) {
@@ -1593,7 +1622,7 @@ nscoord
 nsTableRowFrame::GetUnpaginatedHeight(nsIPresContext* aPresContext)
 {
   // See if the property is set
-  nscoord* value = (nscoord*)nsTableFrame::GetProperty(aPresContext, this, nsLayoutAtoms::rowUnpaginatedHeightProperty);
+  nscoord* value = (nscoord*)nsTableFrame::GetProperty(aPresContext, GetFirstInFlow(), nsLayoutAtoms::rowUnpaginatedHeightProperty);
   if (value) 
     return *value;
   else 
