@@ -45,8 +45,7 @@
 #include "nsStyleConsts.h"
 #include "nsIContent.h"
 #include "nsIView.h"
-#include "nsHTMLReflowCommand.h"
-#include "nsHTMLIIDs.h"
+#include "nsReflowPath.h"
 #include "nsIDeviceContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsIStyleSet.h"
@@ -180,7 +179,7 @@ nsTableRowGroupFrame::InitRepeatedFrame(nsIPresContext*       aPresContext,
       nsIAtom*  frameType;
       copyCellFrame->GetFrameType(&frameType);
 
-      if (nsLayoutAtoms::tableCellFrame == frameType) {
+      if (IS_TABLE_CELL(frameType)) {
   #ifdef NS_DEBUG
         nsIContent* content1;
         nsIContent* content2;
@@ -220,22 +219,6 @@ NS_METHOD nsTableRowGroupFrame::Paint(nsIPresContext*      aPresContext,
     return NS_OK;
   }
 
-  nsCompatibility mode;
-  aPresContext->GetCompatibilityMode(&mode);
-
-  if ((NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) &&  
-      (eCompatibility_Standard == mode)) {
-    const nsStyleVisibility* vis = 
-    (const nsStyleVisibility*)mStyleContext->GetStyleData(eStyleStruct_Visibility);
-    if (vis->IsVisibleOrCollapsed()) {
-      const nsStyleBorder* border =
-        (const nsStyleBorder*)mStyleContext->GetStyleData(eStyleStruct_Border);
-      nsRect rect(0,0,mRect.width, mRect.height);
-      nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, this,
-                                      aDirtyRect, rect, *border, 0, 0);
-    }
-  }
-
 #ifdef DEBUG
   // for debug...
   if ((NS_FRAME_PAINT_LAYER_DEBUG == aWhichLayer) && GetShowFrameBorders()) {
@@ -243,34 +226,18 @@ NS_METHOD nsTableRowGroupFrame::Paint(nsIPresContext*      aPresContext,
     aRenderingContext.DrawRect(0, 0, mRect.width, mRect.height);
   }
 #endif
+  // Standards mode background painting removed.  See bug 4510
 
-  if ((NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) &&  
-      (eCompatibility_Standard == mode)) {
-    // paint the backgrouds of the rows, skipping the cells
-    nsIFrame* kid = mFrames.FirstChild();
-    while (kid) {
-      PaintChild(aPresContext, aRenderingContext, aDirtyRect, kid, aWhichLayer, NS_ROW_FRAME_PAINT_SKIP_CELLS);
-      kid->GetNextSibling(&kid);
-    }
-    // paint the backgrounds of the cells and their descendants, skipping the rows
-    kid = mFrames.FirstChild();
-    while (kid) {
-      PaintChild(aPresContext, aRenderingContext, aDirtyRect, kid, aWhichLayer, NS_ROW_FRAME_PAINT_SKIP_ROW);
-      kid->GetNextSibling(&kid);
-    }
+  const nsStyleDisplay* disp = (const nsStyleDisplay*)
+    mStyleContext->GetStyleData(eStyleStruct_Display);
+  if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
+    aRenderingContext.PushState();
+    SetOverflowClipRect(aRenderingContext);
   }
-  else {
-    const nsStyleDisplay* disp = (const nsStyleDisplay*)
-      mStyleContext->GetStyleData(eStyleStruct_Display);
-    if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
-      aRenderingContext.PushState();
-      SetOverflowClipRect(aRenderingContext);
-    }
-    PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);
-    if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
-      PRBool clipState;
-      aRenderingContext.PopState(clipState);
-    }
+  PaintChildren(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer, aFlags);
+  if (disp && (NS_STYLE_OVERFLOW_HIDDEN == disp->mOverflow)) {
+    PRBool clipState;
+    aRenderingContext.PopState(clipState);
   }
   return NS_OK;
   /*nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect, aWhichLayer);*/
@@ -316,7 +283,7 @@ void nsTableRowGroupFrame::PaintChildren(nsIPresContext*      aPresContext,
                            damageArea.width, damageArea.height);
       aRenderingContext.PushState();
       aRenderingContext.Translate(kidRect.x, kidRect.y);
-      kid->Paint(aPresContext, aRenderingContext, kidDamageArea, aWhichLayer);
+      kid->Paint(aPresContext, aRenderingContext, kidDamageArea, aWhichLayer, aFlags);
 #ifdef DEBUG
       if ((NS_FRAME_PAINT_LAYER_DEBUG == aWhichLayer) &&
           GetShowFrameBorders()) {
@@ -337,7 +304,7 @@ nsTableRowGroupFrame::GetFrameForPoint(nsIPresContext* aPresContext,
                                    nsIFrame**     aFrame)
 {
   // this should act like a block, so we need to override
-  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, (aWhichLayer == NS_FRAME_PAINT_LAYER_BACKGROUND), aFrame);
+  return GetFrameForPointUsing(aPresContext, aPoint, nsnull, aWhichLayer, PR_FALSE, aFrame);
 }
 
 // Position and size aKidFrame and update our reflow state. The origin of
@@ -360,6 +327,28 @@ nsTableRowGroupFrame::PlaceChild(nsIPresContext*        aPresContext,
   }
 }
 
+void
+nsTableRowGroupFrame::InitChildReflowState(nsIPresContext&    aPresContext, 
+                                           PRBool             aBorderCollapse,
+                                           float              aPixelsToTwips,
+                                           nsHTMLReflowState& aReflowState)                                    
+{
+  nsMargin collapseBorder;
+  nsMargin padding(0,0,0,0);
+  nsMargin* pCollapseBorder = nsnull;
+  if (aBorderCollapse) {
+    if (aReflowState.frame) {
+      nsCOMPtr<nsIAtom> fType;
+      aReflowState.frame->GetFrameType(getter_AddRefs(fType));
+      if (nsLayoutAtoms::tableRowFrame == fType.get()) {
+        nsTableRowFrame* rowFrame = (nsTableRowFrame*)aReflowState.frame;
+        pCollapseBorder = rowFrame->GetBCBorderWidth(aPixelsToTwips, collapseBorder);
+      }
+    }
+  }
+  aReflowState.Init(&aPresContext, -1, -1, pCollapseBorder, &padding);
+}
+
 // Reflow the frames we've already created. If aDirtyOnly is set then only
 // reflow dirty frames. This assumes that all of the dirty frames are contiguous.
 NS_METHOD 
@@ -369,11 +358,17 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
                                      nsReflowStatus&        aStatus,
                                      nsTableRowFrame*       aStartFrame,
                                      PRBool                 aDirtyOnly,
-                                     nsTableRowFrame**      aFirstRowReflowed)
+                                     nsTableRowFrame**      aFirstRowReflowed,
+                                     PRBool*                aPageBreakBeforeEnd)
 {
+  if (aPageBreakBeforeEnd) 
+    *aPageBreakBeforeEnd = PR_FALSE;
+
   nsTableFrame* tableFrame = nsnull;
-  nsresult rv = nsTableFrame::GetTableFrame(this, tableFrame);
-  if (NS_FAILED(rv) || !tableFrame) return rv;
+  nsresult rv = nsTableFrame::GetTableFrame(this, tableFrame); if (!tableFrame) ABORT1(rv);
+
+  PRBool borderCollapse = tableFrame->IsBorderCollapse();
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
 
   nscoord cellSpacingY = tableFrame->GetCellSpacingY();
 
@@ -422,13 +417,12 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
       // it's target is the current frame, then make sure we send
       // StyleChange reflow reasons down to the children so that they
       // don't over-optimize their reflow.
-      nsIFrame* target = nsnull;
       nsReflowReason reason = aReflowState.reason;
       if (eReflowReason_Incremental == aReflowState.reason) {
-        aReflowState.reflowState.reflowCommand->GetTarget(target);
-        if (this == target) {
+        nsHTMLReflowCommand* command = aReflowState.reflowState.path->mReflowCommand;
+        if (command) {
           nsReflowType type;
-          aReflowState.reflowState.reflowCommand->GetType(type);
+          command->GetType(type);
           if (eReflowType_StyleChanged == type) {
             reason = eReflowReason_StyleChange;
           }
@@ -439,9 +433,9 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
       }
       nsHTMLReflowState kidReflowState(aPresContext, aReflowState.reflowState, kidFrame,
                                        kidAvailSize, reason);
+      InitChildReflowState(*aPresContext, borderCollapse, p2t, kidReflowState);
      
-      // If this isn't the first row frame, then we can't be at the top of
-      // the page anymore...
+      // If this isn't the first row, then we can't be at the top of the page
       if (kidFrame != GetFirstFrame()) {
         kidReflowState.mFlags.mIsTopOfPage = PR_FALSE;
       }
@@ -459,6 +453,14 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
           *aFirstRowReflowed = (nsTableRowFrame*)kidFrame;
         }
       }
+      if (isPaginated && aPageBreakBeforeEnd && !*aPageBreakBeforeEnd && 
+          (nsLayoutAtoms::tableRowFrame == kidType.get())) {
+        nsTableRowFrame* nextRow = ((nsTableRowFrame*)kidFrame)->GetNextRow();
+        if (nextRow) {
+          *aPageBreakBeforeEnd = nsTableFrame::PageBreakAfter(*kidFrame, nextRow);
+        }
+      }
+
     } else {
       // were done reflowing, so see if we need to reposition the rows that follow
       if (lastReflowedRow) { 
@@ -489,6 +491,9 @@ nsTableRowGroupFrame::ReflowChildren(nsIPresContext*        aPresContext,
     }
   }
 
+  if (aReflowState.reflowState.mFlags.mSpecialHeightReflow) {
+    aDesiredSize.height = mRect.height;
+  }
   return rv;
 }
 
@@ -555,7 +560,7 @@ HasMoreThanOneCell(nsTableCellMap* aCellMap,
     PRInt32 colIndex = 0;
     PRInt32 count = 0;
     do {
-	    cellData = aCellMap->GetCellAt(aRowIndex, colIndex);
+	    cellData = aCellMap->GetDataAt(aRowIndex, colIndex);
 	    if (cellData && (cellData->GetCellFrame() || cellData->IsRowSpan()))
 		    count++;
       if (count > 1) 
@@ -566,6 +571,20 @@ HasMoreThanOneCell(nsTableCellMap* aCellMap,
   return PR_FALSE;
 }   
 
+static void
+CacheRowHeightsForPrinting(nsIPresContext*  aPresContext,
+                           nsTableRowFrame* aFirstRow)
+{
+  for (nsTableRowFrame* row = aFirstRow; row; row = row->GetNextRow()) {
+    if (!row->GetPrevInFlow()) {
+      nsRect rect;
+      row->GetRect(rect);
+      row->SetHasUnpaginatedHeight(PR_TRUE);
+      row->SetUnpaginatedHeight(aPresContext, rect.height);
+    }
+  }
+}
+
 // This calculates the height of rows starting at aStartRowFrameIn and takes into account 
 // style height on the row group, style heights on rows and cells, style heights on rowspans. 
 // Actual row heights will be adjusted later if the table has a style height.
@@ -581,6 +600,9 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
   nsTableFrame::GetTableFrame(this, tableFrame);
   if (!aPresContext || !tableFrame) return;
 
+  PRBool isPaginated;
+  aPresContext->IsPaginated(&isPaginated);
+
   // all table cells have the same top and bottom margins, namely cellSpacingY
   nscoord cellSpacingY = tableFrame->GetCellSpacingY();
   float p2t;
@@ -620,7 +642,7 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
   if (numRows > 0) {
     rowInfo = new RowInfo[numRows];
     if (!rowInfo) return;
-    nsCRT::memset (rowInfo, 0, numRows*sizeof(RowInfo));
+    memset (rowInfo, 0, numRows*sizeof(RowInfo));
   } 
   else return;
 
@@ -634,16 +656,24 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
   PRInt32 rowIndex; // the index in rowInfo, not among the rows in the row group
   nsTableRowFrame* rowFrame;
   for (rowFrame = startRowFrame, rowIndex = 0; rowFrame; rowFrame = rowFrame->GetNextRow(), rowIndex++) {
-    if (rowFrame->HasPctHeight()) {
-      rowInfo[rowIndex].hasPctHeight = PR_TRUE;
-      rowInfo[rowIndex].pctHeight = nsTableFrame::RoundToPixel(rowFrame->GetHeight(pctHeightBasis), p2t);
+    nscoord nonPctHeight = rowFrame->GetContentHeight();
+    if (isPaginated) {
+      nsRect rowRect;
+      rowFrame->GetRect(rowRect);
+      nonPctHeight = PR_MAX(nonPctHeight, rowRect.height);
+    }
+    if (!rowFrame->GetPrevInFlow()) {
+      if (rowFrame->HasPctHeight()) {
+        rowInfo[rowIndex].hasPctHeight = PR_TRUE;
+        rowInfo[rowIndex].pctHeight = nsTableFrame::RoundToPixel(rowFrame->GetHeight(pctHeightBasis), p2t);
+      }
+      rowInfo[rowIndex].hasStyleHeight = rowFrame->HasStyleHeight();
+      nonPctHeight = PR_MAX(nonPctHeight, rowFrame->GetFixedHeight());
     }
-    nscoord nonPctHeight = PR_MAX(rowFrame->GetContentHeight(), rowFrame->GetFixedHeight());
     UpdateHeights(rowInfo[rowIndex], nonPctHeight, heightOfRows, heightOfUnStyledRows);
-    rowInfo[rowIndex].hasStyleHeight = rowFrame->HasStyleHeight();
 
     if (!rowInfo[rowIndex].hasStyleHeight) {
-      if (HasMoreThanOneCell(tableFrame->GetCellMap(), rowIndex)) {
+      if (isPaginated || HasMoreThanOneCell(tableFrame->GetCellMap(), rowIndex + startRowIndex)) {
         rowInfo[rowIndex].isSpecial = PR_TRUE;
         // iteratate the row's cell frames to see if any do not have rowspan > 1
         nsTableCellFrame* cellFrame = rowFrame->GetFirstCell();
@@ -669,7 +699,10 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
     // Get the height of cells with rowspans and allocate any extra space to the rows they span 
     // iteratate the child frames and process the row frames among them
     for (rowFrame = startRowFrame, rowIndex = 0; rowFrame; rowFrame = rowFrame->GetNextRow(), rowIndex++) {
-      if (tableFrame->RowHasSpanningCells(startRowIndex + rowIndex)) {
+      // See if the row has an originating cell with rowspan > 1. We cannot determine this for a row in a 
+      // continued row group by calling RowHasSpanningCells, because the row's fif may not have any originating
+      // cells yet the row may have a continued cell which originates in it.
+      if (mPrevInFlow || tableFrame->RowHasSpanningCells(startRowIndex + rowIndex)) {
         nsTableCellFrame* cellFrame = rowFrame->GetFirstCell();
         // iteratate the row's cell frames 
         while (cellFrame) {
@@ -785,6 +818,7 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
     }
   }
 
+  PRBool styleHeightAllocation = PR_FALSE;
   nscoord rowGroupHeight = startRowGroupHeight + heightOfRows + ((numRows - 1) * cellSpacingY);
   // if we have a style height, allocate the extra height to unconstrained rows
   if ((aReflowState.mComputedHeight > rowGroupHeight) && 
@@ -795,6 +829,7 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
     nscoord divisor = (haveUnStyledRows) 
                       ? heightOfUnStyledRows : heightOfRows;
     if (divisor > 0) {
+      styleHeightAllocation = PR_TRUE;
       for (rowIndex = 0; rowIndex < numRows; rowIndex++) {
         if (!haveUnStyledRows || !rowInfo[rowIndex].hasStyleHeight) {
           // The amount of additional space each row gets is based on the
@@ -839,6 +874,11 @@ nsTableRowGroupFrame::CalculateRowHeights(nsIPresContext*          aPresContext,
     yOrigin += rowHeight + cellSpacingY;
   }
 
+  if (isPaginated && styleHeightAllocation) {
+    // since the row group has a style height, cache the row heights, so next in flows can honor them 
+    CacheRowHeightsForPrinting(aPresContext, GetFirstRow());
+  }
+
   DidResizeRows(*aPresContext, aReflowState, startRowFrame);
 
   aDesiredSize.height = rowGroupHeight; // Adjust our desired size
@@ -897,6 +937,7 @@ nsTableRowGroupFrame::CreateContinuingRowFrame(nsIPresContext& aPresContext,
                                                nsIFrame&       aRowFrame,
                                                nsIFrame**      aContRowFrame)
 {
+  // XXX what is the row index?
   if (!aContRowFrame) {NS_ASSERTION(PR_FALSE, "bad call"); return;}
   // create the continuing frame which will create continuing cell frames
   aStyleSet.CreateContinuingFrame(&aPresContext, &aRowFrame, this, aContRowFrame);
@@ -912,7 +953,7 @@ nsTableRowGroupFrame::CreateContinuingRowFrame(nsIPresContext& aPresContext,
   PushChildren(&aPresContext, *aContRowFrame, &aRowFrame);
 }
 
-nscoord
+void
 nsTableRowGroupFrame::SplitSpanningCells(nsIPresContext&          aPresContext,
                                          const nsHTMLReflowState& aReflowState,
                                          nsIStyleSet&             aStyleSet,                                         
@@ -924,9 +965,8 @@ nsTableRowGroupFrame::SplitSpanningCells(nsIPresContext&          aPresContext,
   PRInt32 rowIndex = aRowFrame.GetRowIndex();
   PRInt32 colCount = aTableFrame.GetColCount();
   nsTableCellFrame* prevCellFrame = nsnull;
+  nsTableRowFrame* rowToMoveToNextPage = nsnull;
  
-  nscoord tallestCell = 0;
-
   for (PRInt32 colX = 0; colX < colCount; colX++) {
     nsTableCellFrame* cellFrame = aTableFrame.GetCellInfoAt(rowIndex, colX);
     if (!cellFrame) continue;
@@ -934,20 +974,19 @@ nsTableRowGroupFrame::SplitSpanningCells(nsIPresContext&          aPresContext,
     cellFrame = (nsTableCellFrame*)cellFrame->GetLastInFlow();
 
     // See if the cell frame is really in this row, or whether it's a cell spanning from a previous row
-    PRInt32 realRowIndex;
-    cellFrame->GetRowIndex(realRowIndex);
-    if (realRowIndex == rowIndex) {
+    PRInt32 cellRowIndex;
+    cellFrame->GetRowIndex(cellRowIndex);
+    if (cellRowIndex == rowIndex) { // cell originates in this row
       prevCellFrame = cellFrame;
     } 
-    else {
+    else { // cell originates in a prior row
       nsTableRowFrame* parentFrame;
       nsPoint          cellOrigin;
       nsReflowStatus   status;
 
       // Ask the cell frame's parent to reflow it to the height of all the
       // rows it spans between its origin and aRowEndY
-      cellFrame->GetParent((nsIFrame**)&parentFrame);
-      if (!parentFrame) {NS_ASSERTION(PR_FALSE, "bad parent"); return 0;}
+      cellFrame->GetParent((nsIFrame**)&parentFrame); if (!parentFrame) ABORT0();
       parentFrame->GetOrigin(cellOrigin);
       nscoord cellAvailHeight = aRowEndY - cellOrigin.y;
       nscoord cellHeight = parentFrame->ReflowCellFrame(&aPresContext, aReflowState, cellFrame, 
@@ -960,7 +999,7 @@ nsTableRowGroupFrame::SplitSpanningCells(nsIPresContext&          aPresContext,
         if (contCellFrame) {
           if (aContRowFrame) {
             aContRowFrame->InsertCellFrame(contCellFrame, colX);
-          }
+          } 
           else {
             aRowFrame.InsertCellFrame(contCellFrame, prevCellFrame);
           }
@@ -973,10 +1012,8 @@ nsTableRowGroupFrame::SplitSpanningCells(nsIPresContext&          aPresContext,
         aContRowFrame->InsertCellFrame(cellFrame, colX);
         prevCellFrame = cellFrame;
       }
-      tallestCell = PR_MAX(tallestCell, cellHeight);
     }
   }
-  return tallestCell;
 }
 
 nsresult
@@ -998,35 +1035,45 @@ nsTableRowGroupFrame::SplitRowGroup(nsIPresContext*          aPresContext,
   presShell->GetStyleSet(getter_AddRefs(styleSet));
   if (!styleSet) {NS_ASSERTION(PR_FALSE, "no style set"); return NS_ERROR_NULL_POINTER;}
 
-  float p2t;
-  aPresContext->GetPixelsToTwips(&p2t);
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
+  PRBool borderCollapse = ((nsTableFrame*)aTableFrame->GetFirstInFlow())->IsBorderCollapse();
 
   nscoord availWidth  = nsTableFrame::RoundToPixel(aReflowState.availableWidth, p2t);
   nscoord availHeight = nsTableFrame::RoundToPixel(aReflowState.availableHeight, p2t);
-  nscoord lastDesiredHeight = 0;
+
+  // get the page height
+  nsRect actualRect;
+  nsRect adjRect;
+  aPresContext->GetPageDim(&actualRect, &adjRect);
+  nscoord pageHeight = actualRect.height;
+
+  PRBool isTopOfPage = aReflowState.mFlags.mIsTopOfPage;
+  nscoord cellSpacingY = aTableFrame->GetCellSpacingY();
 
   // Walk each of the row frames looking for the first row frame that
   // doesn't fit in the available space
   for (nsTableRowFrame* rowFrame = GetFirstRow(); rowFrame; rowFrame = rowFrame->GetNextRow()) {
     PRBool rowIsOnCurrentPage = PR_TRUE;
-    PRBool degenerateRow = PR_FALSE;
     nsRect bounds;
     rowFrame->GetRect(bounds);
     if (bounds.YMost() > availHeight) {
-      nsRect actualRect;
-      nsRect adjRect;
-      aPresContext->GetPageDim(&actualRect, &adjRect);
       nsIFrame* contRowFrame = nsnull;
-      nscoord pageHeight = actualRect.height;
       // reflow the row in the availabe space and have it split if it is the 1st
       // row or there is at least 5% of the current page available 
       if (!prevRowFrame || (availHeight - aDesiredSize.height > pageHeight / 20)) { 
-        // Reflow the row in the available space and have it split
-        nsSize              availSize(availWidth, availHeight - bounds.y);
-        nsHTMLReflowState   rowReflowState(aPresContext, aReflowState, rowFrame,
-                                          availSize, eReflowReason_Resize);
+        nsSize availSize(availWidth, PR_MAX(availHeight - bounds.y, 0));
+        // don't let the available height exceed what CalculateRowHeights set for it
+        availSize.height = PR_MIN(availSize.height, bounds.height);
+
+        nsHTMLReflowState rowReflowState(aPresContext, aReflowState, rowFrame, availSize, 
+                                         eReflowReason_Resize);
+        InitChildReflowState(*aPresContext, borderCollapse, p2t, rowReflowState);
+        rowReflowState.mFlags.mIsTopOfPage = isTopOfPage; // set top of page
+
         nsHTMLReflowMetrics desiredSize(nsnull);
 
+        // A row that has all originating cells with rowspan>1 will return a desired height 
+        // of the largest cell (in contrast to an unconstrained height reflow which returns 0).
         rv = ReflowChild(rowFrame, aPresContext, desiredSize, rowReflowState,
                          0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
         rowFrame->SizeTo(aPresContext, desiredSize.width, desiredSize.height);
@@ -1035,27 +1082,26 @@ nsTableRowGroupFrame::SplitRowGroup(nsIPresContext*          aPresContext,
 
         if (NS_FRAME_IS_NOT_COMPLETE(aStatus)) {
           // the row frame is incomplete and all of the cells' block frames have split
-          CreateContinuingRowFrame(*aPresContext, *styleSet.get(), *rowFrame, &contRowFrame);
-          aDesiredSize.height += desiredSize.height;
-          if (prevRowFrame) {
-            aDesiredSize.height += aTableFrame->GetCellSpacingY();
-          }
-        } 
-        else if (0 == desiredSize.height) {
-          // the row frame is complete because it had no cells originating in it.
-          CreateContinuingRowFrame(*aPresContext, *styleSet.get(), *rowFrame, &contRowFrame);
-          aStatus = NS_FRAME_NOT_COMPLETE;
-          aDesiredSize.height = availHeight;
-          degenerateRow = PR_TRUE;
-        }
-        else {
-          // the row frame is complete because it's minimum height was greater than the 
-          // the available height we gave it 
-          if (aDesiredSize.height > 0) {
-            // put the row on the next page since it needs more height than it was given
-            rowIsOnCurrentPage = PR_FALSE;
+          if ((desiredSize.height <= rowReflowState.availableHeight) || isTopOfPage) {
+            // the row stays on this page because either it split ok or we're on the top of page
+            // if top of page and the height exceeded the avail height,then there will be data loss
+            NS_WARN_IF_FALSE(desiredSize.height <= rowReflowState.availableHeight, 
+                            "data loss - incomplete row needed more height than available, on top of page");
+            CreateContinuingRowFrame(*aPresContext, *styleSet.get(), *rowFrame, &contRowFrame);
+            aDesiredSize.height += desiredSize.height;
+            if (prevRowFrame) 
+              aDesiredSize.height += cellSpacingY;
           }
           else {
+            // put the row on the next page to give it more height 
+            rowIsOnCurrentPage = PR_FALSE;
+          }
+        } 
+        else {
+          // The row frame is complete because it's minimum height was greater than the 
+          // the available height we gave it.
+          if (isTopOfPage) { 
+            // We're on top of the page, so keep the row on this page. There will be data loss.
             nsIFrame* nextRowFrame;
 
             // Push the row frame that follows
@@ -1063,50 +1109,59 @@ nsTableRowGroupFrame::SplitRowGroup(nsIPresContext*          aPresContext,
           
             if (nextRowFrame) {
               PushChildren(aPresContext, nextRowFrame, rowFrame);
+              aStatus = NS_FRAME_NOT_COMPLETE;
             }
-            aStatus = nextRowFrame ? NS_FRAME_NOT_COMPLETE : NS_FRAME_COMPLETE;
-            aDesiredSize.height = bounds.YMost();
+            aDesiredSize.height += desiredSize.height;
+            if (prevRowFrame) 
+              aDesiredSize.height += cellSpacingY;
+            NS_WARN_IF_FALSE(PR_FALSE, "data loss - complete row needed more height than available, on top of page");
+          }
+          else {  
+            // We're not on top of the page, so put the row on the next page to give it more height 
+            rowIsOnCurrentPage = PR_FALSE;
           }
         }
       }
-      else rowIsOnCurrentPage = PR_FALSE;
-
-      if (!rowIsOnCurrentPage) {
-        // Push this row frame and those that follow to the next-in-flow
-        PushChildren(aPresContext, rowFrame, prevRowFrame);
-        aStatus = NS_FRAME_NOT_COMPLETE;
+      else {
+        // put the row on the next page to give it more height
+        rowIsOnCurrentPage = PR_FALSE;
       }
-      if (prevRowFrame) {
-        nscoord tallestCell = 
-          SplitSpanningCells(*aPresContext, aReflowState, *styleSet, *aTableFrame, 
-                             *rowFrame, aDesiredSize.height, (nsTableRowFrame*)contRowFrame);
-        if (degenerateRow) {
-          aDesiredSize.height = lastDesiredHeight + aTableFrame->GetCellSpacingY() + tallestCell;
+
+      PRBool willGetAnotherReflow = PR_FALSE;
+      if (!rowIsOnCurrentPage) {
+        if (prevRowFrame) {
+          PushChildren(aPresContext, rowFrame, prevRowFrame);
+          aStatus = NS_FRAME_NOT_COMPLETE;
         }
+        else {
+          // We can't push children, so let our parent reflow us again with more space
+          aDesiredSize.height = bounds.YMost();
+          willGetAnotherReflow = PR_TRUE;
+        }
+      }
+
+      if (prevRowFrame && !willGetAnotherReflow) {
+        SplitSpanningCells(*aPresContext, aReflowState, *styleSet, *aTableFrame, 
+                           *rowFrame, aDesiredSize.height, (nsTableRowFrame*)contRowFrame);
       }
       break;
     }
     else {
       aDesiredSize.height = bounds.YMost();
-      lastDesiredHeight   = aDesiredSize.height;
       prevRowFrame = rowFrame;
+      // see if there is a page break after the row
+      nsTableRowFrame* nextRow = rowFrame->GetNextRow();
+      if (nextRow && nsTableFrame::PageBreakAfter(*rowFrame, nextRow)) {
+        PushChildren(aPresContext, nextRow, rowFrame);
+        aStatus = NS_FRAME_NOT_COMPLETE;
+        break;
+      }
     }
+    isTopOfPage = PR_FALSE; // after the 1st row, we can't be on top of the page any more.
   }
   return NS_OK;
 }
 
-static void
-CacheRowHeightsForPrinting(nsIPresContext*  aPresContext,
-                           nsTableRowFrame* aFirstRow)
-{
-  for (nsTableRowFrame* row = aFirstRow; row; row = row->GetNextRow()) {
-    nsRect rect;
-    row->GetRect(rect);
-    row->SetHasUnpaginatedHeight(PR_TRUE);
-    row->SetUnpaginatedHeight(aPresContext, rect.height);
-  }
-}
-
 /** Layout the entire row group.
   * This method stacks rows vertically according to HTML 4.0 rules.
   * Rows are responsible for layout of their children.
@@ -1123,19 +1178,23 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState);
 #endif
 
-  nsresult rv=NS_OK;
-  aStatus = NS_FRAME_COMPLETE;
+  nsresult rv = NS_OK;
+  aStatus     = NS_FRAME_COMPLETE;
         
+  PRBool isPaginated;
+  aPresContext->IsPaginated(&isPaginated);
+
   nsTableFrame* tableFrame = nsnull;
   rv = nsTableFrame::GetTableFrame(this, tableFrame);
   if (!aPresContext || !tableFrame) return NS_ERROR_NULL_POINTER;
 
+  // see if a special height reflow needs to occur due to having a pct height
+  if (!NeedSpecialReflow()) 
+    nsTableFrame::CheckRequestSpecialHeightReflow(aReflowState);
+
   nsRowGroupReflowState state(aReflowState, tableFrame);
   PRBool haveDesiredHeight = PR_FALSE;
 
-  PRBool isPaginated;
-  aPresContext->IsPaginated(&isPaginated);
-
   if (eReflowReason_Incremental == aReflowState.reason) {
     rv = IncrementalReflow(aPresContext, aDesiredSize, state, aStatus);
   } 
@@ -1143,13 +1202,10 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
     // Check for an overflow list
     MoveOverflowToChildList(aPresContext);
   
-    if (isPaginated && aReflowState.mFlags.mSpecialHeightReflow) {
-      // cache row height info for printing, now that row heights are known. 
-      CacheRowHeightsForPrinting(aPresContext, GetFirstRow());
-    }
     // Reflow the existing frames. 
+    PRBool splitDueToPageBreak = PR_FALSE;
     rv = ReflowChildren(aPresContext, aDesiredSize, state, aStatus,
-                        nsnull, PR_FALSE);
+                        nsnull, PR_FALSE, nsnull, &splitDueToPageBreak);
   
     // Return our desired rect
     aDesiredSize.width = aReflowState.availableWidth;
@@ -1166,6 +1222,9 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
     // are row spans unless we do this step
     if (aReflowState.mFlags.mSpecialHeightReflow) {
       DidResizeRows(*aPresContext, aReflowState);
+      if (isPaginated) {
+        CacheRowHeightsForPrinting(aPresContext, GetFirstRow());
+      }
     }
     else if ((eReflowReason_Initial != aReflowState.reason) || 
              isTableUnconstrainedReflow                     ||
@@ -1175,7 +1234,8 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
     }
 
     // See if all the frames fit
-    if (aDesiredSize.height > aReflowState.availableHeight) {
+    if ((NS_FRAME_NOT_COMPLETE == aStatus) || splitDueToPageBreak || 
+        (aDesiredSize.height > aReflowState.availableHeight)) {
       // Nope, find a place to split the row group 
       PRBool specialReflow = (PRBool)aReflowState.mFlags.mSpecialHeightReflow;
       ((nsHTMLReflowState::ReflowStateFlags&)aReflowState.mFlags).mSpecialHeightReflow = PR_FALSE;
@@ -1183,7 +1243,7 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
       SplitRowGroup(aPresContext, aDesiredSize, aReflowState, tableFrame, aStatus);
 
       ((nsHTMLReflowState::ReflowStateFlags&)aReflowState.mFlags).mSpecialHeightReflow = specialReflow;
-   }
+    }
   }
   SetHasStyleHeight((NS_UNCONSTRAINEDSIZE != aReflowState.mComputedHeight) &&
                     (aReflowState.mComputedHeight > 0)); 
@@ -1202,6 +1262,7 @@ nsTableRowGroupFrame::Reflow(nsIPresContext*          aPresContext,
 #if defined DEBUG_TABLE_REFLOW_TIMING
   nsTableFrame::DebugReflow(this, (nsHTMLReflowState&)aReflowState, &aDesiredSize, aStatus);
 #endif
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
@@ -1212,23 +1273,23 @@ nsTableRowGroupFrame::IncrementalReflow(nsIPresContext*        aPresContext,
                                         nsRowGroupReflowState& aReflowState,
                                         nsReflowStatus&        aStatus)
 {
-  nsresult  rv = NS_OK;
+  // the row group is a target if its path has a reflow command
+  nsHTMLReflowCommand* command = aReflowState.reflowState.path->mReflowCommand;
+  if (command)
+    IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
 
-  // determine if this frame is the target or not
-  nsIFrame* target = nsnull;
-  rv = aReflowState.reflowState.reflowCommand->GetTarget(target);
-  if (NS_SUCCEEDED(rv) && target) {
-    if (this == target)
-      rv = IR_TargetIsMe(aPresContext, aDesiredSize, aReflowState, aStatus);
-    else {
-      // Get the next frame in the reflow chain
-      nsIFrame* nextFrame;
-      aReflowState.reflowState.reflowCommand->GetNext(nextFrame);
+  // see if the chidren are targets as well
+  // XXXwaterson Note that this will cause us to RecoverState (which
+  // is O(n) in the number of child rows) once for each reflow
+  // target. It'd probably be better to invert the loops; i.e., walk
+  // the rows, checking each to see if it's an IR target (which could
+  // be done in O(1) if we do hashing in the reflow path).
+  nsReflowPath::iterator iter = aReflowState.reflowState.path->FirstChild();
+  nsReflowPath::iterator end = aReflowState.reflowState.path->EndChildren();
+  for (; iter != end; ++iter)
+    IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, *iter);
 
-      rv = IR_TargetIsChild(aPresContext, aDesiredSize, aReflowState, aStatus, nextFrame);
-    }
-  }
-  return rv;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -1353,7 +1414,7 @@ nsTableRowGroupFrame::IR_TargetIsMe(nsIPresContext*        aPresContext,
 {
   nsresult rv = NS_FRAME_COMPLETE;
   nsReflowType type;
-  aReflowState.reflowState.reflowCommand->GetType(type);
+  aReflowState.reflowState.path->mReflowCommand->GetType(type);
 
   switch (type) {
     case eReflowType_ReflowDirty: {
@@ -1454,6 +1515,8 @@ nsTableRowGroupFrame::RecoverState(nsRowGroupReflowState& aReflowState,
   nsTableFrame::GetTableFrame(this, tableFrame);
   nscoord cellSpacingY = tableFrame->GetCellSpacingY();
 
+  aReflowState.y = 0;
+
   // Walk the list of children up to aKidFrame
   for (nsIFrame* frame = mFrames.FirstChild(); frame && (frame != aKidFrame); frame->GetNextSibling(&frame)) {
        nsCOMPtr<nsIAtom> fType;
@@ -1530,6 +1593,10 @@ nsTableRowGroupFrame::IR_TargetIsChild(nsIPresContext*        aPresContext,
 {
   nsresult rv;
   
+  nsTableFrame* tableFrame = nsnull; 
+  nsTableFrame::GetTableFrame(this, tableFrame); if (!tableFrame) ABORT1(NS_ERROR_NULL_POINTER);
+  GET_PIXELS_TO_TWIPS(aPresContext, p2t);
+
   // Recover the state as if aNextFrame is about to be reflowed
   RecoverState(aReflowState, aNextFrame);
 
@@ -1540,9 +1607,11 @@ nsTableRowGroupFrame::IR_TargetIsChild(nsIPresContext*        aPresContext,
   // Reflow the child giving it as much room as it wants. We'll deal with
   // splitting later after final determination of rows heights taking into
   // account cells with row spans...
-  nsSize              kidAvailSize(aReflowState.availSize.width, NS_UNCONSTRAINEDSIZE);
-  nsHTMLReflowState   kidReflowState(aPresContext, aReflowState.reflowState,
-                                     aNextFrame, kidAvailSize);
+  nsSize            kidAvailSize(aReflowState.availSize.width, NS_UNCONSTRAINEDSIZE);
+  nsHTMLReflowState kidReflowState(aPresContext, aReflowState.reflowState, aNextFrame, 
+                                   kidAvailSize, aReflowState.reason);
+  InitChildReflowState(*aPresContext, tableFrame->IsBorderCollapse(), p2t, kidReflowState);
+
   nsSize              kidMaxElementSize;
   nsHTMLReflowMetrics desiredSize(aDesiredSize.maxElementSize ? &kidMaxElementSize : nsnull,
                                   aDesiredSize.mFlags);
@@ -1715,6 +1784,28 @@ nsTableRowGroupFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) cons
 }
 #endif
 
+nsMargin* 
+nsTableRowGroupFrame::GetBCBorderWidth(float     aPixelsToTwips,
+                                       nsMargin& aBorder)
+{
+  aBorder.left = aBorder.right = 0;
+
+  nsTableRowFrame* firstRowFrame = nsnull;
+  nsTableRowFrame* lastRowFrame = nsnull;
+  for (nsTableRowFrame* rowFrame = GetFirstRow(); rowFrame; rowFrame = rowFrame->GetNextRow()) {
+    if(!firstRowFrame) {
+      firstRowFrame = rowFrame;
+    }
+    lastRowFrame = rowFrame;
+  }
+  if (firstRowFrame) {
+    aBorder.top    = firstRowFrame->GetTopBCBorderWidth(&aPixelsToTwips);
+    aBorder.bottom = lastRowFrame->GetBottomBCBorderWidth(&aPixelsToTwips);
+  }
+
+  return &aBorder;
+}
+
 //nsILineIterator methods for nsTableFrame
 NS_IMETHODIMP
 nsTableRowGroupFrame::GetNumLines(PRInt32* aResult)
@@ -1757,7 +1848,7 @@ nsTableRowGroupFrame::GetLine(PRInt32    aLineNumber,
   *aLineFlags = 0;/// should we fill these in later?
   // not gonna touch aLineBounds right now
 
-  CellData* firstCellData = cellMap->GetCellAt(aLineNumber, 0);
+  CellData* firstCellData = cellMap->GetDataAt(aLineNumber, 0);
   if(!firstCellData) 
     return NS_ERROR_FAILURE;
 
@@ -1767,7 +1858,7 @@ nsTableRowGroupFrame::GetLine(PRInt32    aLineNumber,
     while((aLineNumber > 0)&&(!(*aFirstFrameOnLine)))
     {
       aLineNumber--;
-      firstCellData = cellMap->GetCellAt(aLineNumber, 0);
+      firstCellData = cellMap->GetDataAt(aLineNumber, 0);
       *aFirstFrameOnLine = (nsIFrame*)firstCellData->GetCellFrame();
     }
   }
@@ -1847,7 +1938,7 @@ nsTableRowGroupFrame::FindFrameAt(PRInt32    aLineNumber,
   PRBool gotParentRect = PR_FALSE;
   for(int i =0;i < cellCount; i++)
   {
-    cellData = cellMap->GetCellAt(aLineNumber, i);
+    cellData = cellMap->GetDataAt(aLineNumber, i);
     tempFrame = (nsIFrame*)cellData->GetCellFrame();
 
     if(!tempFrame)
@@ -1919,11 +2010,11 @@ nsTableRowGroupFrame::GetNextSiblingOnLine(nsIFrame*& aFrame,
   PRInt32& colIndexRef = colIndex;
   cellFrame->GetColIndex(colIndexRef);
 
-  CellData* cellData = cellMap->GetCellAt(aLineNumber, colIndex + 1);
+  CellData* cellData = cellMap->GetDataAt(aLineNumber, colIndex + 1);
   
   if(!cellData)// if this isnt a valid cell, drop down and check the next line
   {
-    cellData = cellMap->GetCellAt(aLineNumber + 1, 0);
+    cellData = cellMap->GetDataAt(aLineNumber + 1, 0);
     if(!cellData)
     {
       //*aFrame = nsnull;
@@ -1940,14 +2031,14 @@ nsTableRowGroupFrame::GetNextSiblingOnLine(nsIFrame*& aFrame,
     while((tempCol > 0) && (!aFrame))
     {
       tempCol--;
-      cellData = cellMap->GetCellAt(aLineNumber, tempCol);
+      cellData = cellMap->GetDataAt(aLineNumber, tempCol);
       aFrame = (nsIFrame*)cellData->GetCellFrame();
       if(!aFrame && (tempCol==0))
       {
         while((tempRow > 0) && (!aFrame))
         {
           tempRow--;
-          cellData = cellMap->GetCellAt(tempRow, 0);
+          cellData = cellMap->GetDataAt(tempRow, 0);
           aFrame = (nsIFrame*)cellData->GetCellFrame();
         }
       }
