@@ -25,7 +25,6 @@
 #include "nsIHTMLContent.h"
 #include "nsFrame.h"
 #include "nsLineLayout.h"
-#include "nsHTMLIIDs.h"
 #include "nsIPresContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
@@ -33,6 +32,7 @@
 #include "nsStyleConsts.h"
 #include "nsINameSpaceManager.h"
 #include "nsString.h"
+#include "nsUnicharUtils.h"
 #include "nsIRenderingContext.h"
 #include "nsIFontMetrics.h"
 #include "nsStyleUtil.h"
@@ -46,12 +46,15 @@
 #include "nsNetUtil.h"
 #include "nsIURI.h"
 
+#include "nsILookAndFeel.h"
 #include "nsIDeviceContext.h"
 #include "nsCSSRendering.h"
 #include "prprf.h"         // For PR_snprintf()
 
+#include "nsIDOMWindow.h"
+#include "nsIDialogParamBlock.h"
 #include "nsIWindowWatcher.h"
-#include "nsIPrompt.h"
+#include "nsIStringBundle.h"
 
 #include "nsMathMLOperators.h"
 #include "nsMathMLChar.h"
@@ -132,25 +135,53 @@ CheckFontExistence(nsIPresContext* aPresContext, const nsString& aFontName)
 }
 
 // alert the user if some of the needed MathML fonts are not installed.
+// it is non-modal (i.e., it doesn't wait for input from the user)
 static void
 AlertMissingFonts(nsString& aMissingFonts)
 {
+  nsCOMPtr<nsIStringBundleService> sbs(do_GetService(NS_STRINGBUNDLE_CONTRACTID));
+  if (!sbs)
+    return;
+
+  nsCOMPtr<nsIStringBundle> sb;
+  sbs->CreateBundle("resource:/res/fonts/mathfont.properties", getter_AddRefs(sb));
+  if (!sb)
+    return;
+
+  nsXPIDLString title, message;
+  const PRUnichar* strings[] = { aMissingFonts.get() };
+  sb->GetStringFromName(NS_LITERAL_STRING("mathfont_missing_dialog_title").get(), getter_Copies(title));
+  sb->FormatStringFromName(NS_LITERAL_STRING("mathfont_missing_dialog_message").get(),
+                           strings, 1, getter_Copies(message));
+
   nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
-  if (wwatch) {
-    nsCOMPtr<nsIPrompt> prompter;
-    wwatch->GetNewPrompter(0, getter_AddRefs(prompter));
-    if (prompter) {
-      nsAutoString message;
-      message.AssignWithConversion(
-"This page contains MathML. Mozilla has detected that some "
-"of the fonts needed for quality rendering of MathML are missing "
-"from your system. Without these fonts, Mozilla cannot render "
-"the page properly. To get a rendering of professional quality "
-"you should install the following fonts: ");
-      message.Append(aMissingFonts);
-      prompter->Alert(0, message.get());
-    }
-  }
+  if (!wwatch)
+    return;
+
+  nsCOMPtr<nsIDialogParamBlock> paramBlock(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  if (!paramBlock)
+    return;
+
+  // copied from nsICommonDialogs.idl which curiously isn't part of the build
+  // (mozilla/xpfe/appshell/public/nsICommonDialogs.idl)
+  enum {eMsg=0, eCheckboxMsg=1, eIconClass=2, eTitleMessage=3, eEditfield1Msg=4,
+        eEditfield2Msg=5, eEditfield1Value=6, eEditfield2Value=7, eButton0Text=8,
+        eButton1Text=9, eButton2Text=10, eButton3Text=11,eDialogTitle=12};
+  enum {eButtonPressed=0, eCheckboxState=1, eNumberButtons=2, eNumberEditfields=3,
+        eEditField1Password=4};
+
+  paramBlock->SetInt(eNumberButtons, 1);
+  paramBlock->SetString(eIconClass, NS_LITERAL_STRING("alert-icon").get());
+  paramBlock->SetString(eDialogTitle, title.get());
+  paramBlock->SetString(eMsg, message.get());
+
+  nsCOMPtr<nsIDOMWindow> parent;
+  wwatch->GetActiveWindow(getter_AddRefs(parent));
+
+  nsCOMPtr<nsIDOMWindow> dialog;
+  wwatch->OpenWindow(parent, "chrome://global/content/commonDialog.xul", "_blank",
+                     "dependent,centerscreen,chrome,titlebar", paramBlock,
+                     getter_AddRefs(dialog));
 }
 
 // helper to trim off comments from data in a MathFont Property File
@@ -558,7 +589,7 @@ nsGlyphTable::GetBoundingMetrics(nsIRenderingContext& aRenderingContext,
   if (aGlyphCode.font) {
     // glyph not associated to our primary font, it comes from an external font
     mFontName.StringAt(aGlyphCode.font, aFont.name);
-    aRenderingContext.SetFont(aFont);
+    aRenderingContext.SetFont(aFont, nsnull);
   }
 
   //if (mType == NS_TABLE_TYPE_UNICODE)
@@ -570,7 +601,7 @@ nsGlyphTable::GetBoundingMetrics(nsIRenderingContext& aRenderingContext,
   if (aGlyphCode.font) {
     // restore our primary font in the rendering context
     mFontName.StringAt(0, aFont.name);
-    aRenderingContext.SetFont(aFont);
+    aRenderingContext.SetFont(aFont, nsnull);
   }
   return rv;
 }
@@ -593,7 +624,7 @@ nsGlyphTable::DrawGlyph(nsIRenderingContext& aRenderingContext,
   if (aGlyphCode.font) {
     // glyph not associated to our primary font, it comes from an external font
     mFontName.StringAt(aGlyphCode.font, aFont.name);
-    aRenderingContext.SetFont(aFont);
+    aRenderingContext.SetFont(aFont, nsnull);
   }
 
   //if (mType == NS_TABLE_TYPE_UNICODE)
@@ -605,7 +636,7 @@ nsGlyphTable::DrawGlyph(nsIRenderingContext& aRenderingContext,
   if (aGlyphCode.font) {
     // restore our primary font in the rendering context
     mFontName.StringAt(0, aFont.name);
-    aRenderingContext.SetFont(aFont);
+    aRenderingContext.SetFont(aFont, nsnull);
   }
   if (aClipRect) {
     aRenderingContext.PopState(clipState);
@@ -687,7 +718,6 @@ public:
 
   // Find the subset of glyph tables that are applicable to the given char,
   // knowing that the stretchy style context of the char has the given font.
-  // aGlyphTableList is not cleared here, we just append to what may exist there.
   nsresult
   GetListFor(nsIPresContext* aPresContext,
              nsMathMLChar*   aChar,
@@ -837,7 +867,7 @@ struct StretchyFontEnumContext {
 
 // check if the current font is associated to a known glyph table, if so the
 // glyph table is added to the list of tables that can be used for the char
-static PRBool PR_CALLBACK
+static PRBool
 StretchyFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
 {
   if (aGeneric) return PR_FALSE; // stop now
@@ -850,7 +880,7 @@ StretchyFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
     nsGlyphTable* glyphTable = gGlyphTableList->TableAt(i);
     nsAutoString fontName;
     glyphTable->GetPrimaryFontName(fontName);
-    if (fontName.EqualsIgnoreCase(aFamily) &&
+    if (fontName.Equals(aFamily, nsCaseInsensitiveStringComparator()) &&
         glyphTable->Has(currPresContext, currChar)) {
       currList->AppendElement(glyphTable); // the table is retained
       return PR_TRUE; // don't stop
@@ -918,7 +948,7 @@ nsGlyphTableList::GetPreferredListAt(nsIPresContext* aPresContext,
         aGlyphTableList->Clear();
       }
       aGlyphTableList->AppendElement(glyphTable);
-      *aCount++;
+      ++*aCount;
     }
     glyphTable = TableAt(++index);
   } 
@@ -935,12 +965,12 @@ struct PreferredFontEnumContext {
 };
 
 // mark a glyph table as a preferred table that can be used for a char
-static PRBool PR_CALLBACK
+static PRBool
 PreferredFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
 {
   PRInt32 i;
   nsAutoString fontName;
-  nsGlyphTable* glyphTable;
+  nsGlyphTable* glyphTable = nsnull;
   PreferredFontEnumContext* context = (PreferredFontEnumContext*)aData;
   // see if the table already exists in mTableList[0..mDefaultCount-1]
   PRBool found = PR_FALSE;
@@ -948,7 +978,7 @@ PreferredFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
   for (i = 0; i < count; i++) {
     glyphTable = gGlyphTableList->TableAt(i);
     glyphTable->GetPrimaryFontName(fontName);
-    if (fontName.EqualsIgnoreCase(aFamily)) {
+    if (fontName.Equals(aFamily, nsCaseInsensitiveStringComparator())) {
       found = PR_TRUE;
       break;
     }
@@ -960,7 +990,7 @@ PreferredFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
     for (i = 0; i < count; i++) {
       glyphTable = gGlyphTableList->AdditionalTableAt(i);
       glyphTable->GetPrimaryFontName(fontName);
-      if (fontName.EqualsIgnoreCase(aFamily)) {
+      if (fontName.Equals(aFamily, nsCaseInsensitiveStringComparator())) {
         found = PR_TRUE;
         break;
       }
@@ -1027,7 +1057,7 @@ struct MathFontEnumContext {
   nsString*       mMissingFamilyList;
 };
 
-static PRBool PR_CALLBACK
+static PRBool
 MathFontEnumCallback(const nsString& aFamily, PRBool aGeneric, void *aData)
 {
   // check if the font is missing
@@ -1214,22 +1244,24 @@ nsMathMLChar::SetData(nsIPresContext* aPresContext,
   mData = aData;
   // some assumptions until proven otherwise
   // note that mGlyph is not initialized
+  mOperator = -1;
   mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
   mBoundingMetrics.Clear();
   mGlyphTable = nsnull;
   // check if stretching is applicable ...
   if (gGlyphTableList && (1 == mData.Length())) {
-    PRInt32 k = nsMathMLOperators::FindStretchyOperator(mData[0]);
-    if (k != kNotFound) {
-      mDirection = nsMathMLOperators::GetStretchyDirectionAt(k);
+    mOperator = nsMathMLOperators::FindStretchyOperator(mData[0]);
+    if (mOperator >= 0) {
+      mDirection = nsMathMLOperators::GetStretchyDirectionAt(mOperator);
       // default tentative table (not the one that is necessarily going to be used)
       mGlyphTable = gGlyphTableList->GetGlyphTableFor(aPresContext, this);
       // commom case: we won't bother with the stretching if there is
       // no glyph table for us...
       if (!mGlyphTable) {
-        mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
         // never try to stretch this operator again
-        nsMathMLOperators::DisableStretchyOperatorAt(k);
+        nsMathMLOperators::DisableStretchyOperatorAt(mOperator);
+        mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
+        mOperator = -1;
       }
     }
   }
@@ -1314,8 +1346,6 @@ nsMathMLChar::SetData(nsIPresContext* aPresContext,
 */
 // -----------------------------------------------------------------------------------
 
-#undef PR_ABS
-#define PR_ABS(x) ((x) < 0 ? -(x) : (x))
 
 // plain TeX settings (TeXbook p.152)
 #define NS_MATHML_DELIMITER_FACTOR      0.901f
@@ -1423,6 +1453,14 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   nsresult rv = NS_OK;
   nsStretchDirection direction = aStretchDirection;
 
+  // if we have been called before, and we didn't actually stretch, our
+  // direction may have been set to NS_STRETCH_DIRECTION_UNSUPPORTED.
+  // So first set our direction back to its instrinsic value
+  if (mOperator >= 0) {
+    // mOperator is initialized in SetData() and remains unchanged
+    mDirection = nsMathMLOperators::GetStretchyDirectionAt(mOperator);
+  }
+
   // if no specified direction, attempt to stretch in our preferred direction
   if (direction == NS_STRETCH_DIRECTION_DEFAULT) {
     direction = mDirection;
@@ -1439,17 +1477,17 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   // XXXrbs get rid of this hardcoding - bug 118600
   PRUnichar uchar = mData[0];
   if (kSqrChar == uchar) {                        // Special to the sqrt char. Due to
-    fontName.Assign(NS_LITERAL_STRING("CMSY10")); // assumptions in the sqrt code, we need
+    fontName.Assign(NS_LITERAL_STRING("CMSY10,Math2")); // assumptions in the sqrt code, we need
     SetFirstFamily(theFont, fontName);         // to force precedence on this TeX font
   }
-  aRenderingContext.SetFont(theFont);
+  aRenderingContext.SetFont(theFont, nsnull);
   rv = aRenderingContext.GetBoundingMetrics(mData.get(),
                                             PRUint32(mData.Length()),
                                             mBoundingMetrics);
   if (NS_FAILED(rv)) {
     NS_WARNING("GetBoundingMetrics failed");
     // ensure that the char later behaves like a normal char
-    // XXX to reset in dynamic updates @ ContentChanged()
+    // (will be reset back to its intrinsic value in case of dynamic updates)
     mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
     return rv;
   }
@@ -1460,7 +1498,8 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   // quick return if there is nothing special about this char
   if (!mGlyphTable || (mDirection != direction)) {
     // ensure that the char later behaves like a normal char
-    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED; // XXX to reset in dynamic updates
+    // (will be reset back to its intrinsic value in case of dynamic updates)
+    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
     return NS_OK;
   }
 
@@ -1483,9 +1522,10 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
     targetSize = aContainerSize.width;
   }
   // if we are not a largeop in display mode, return if size fits
-  if (!largeop && IsSizeOK(charSize, targetSize, aStretchHint)) {
+  if (targetSize <= 0 || (!largeop && IsSizeOK(charSize, targetSize, aStretchHint))) {
     // ensure that the char later behaves like a normal char
-    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED; // XXX to reset in dynamic updates
+    // (will be reset back to its intrinsic value in case of dynamic updates)
+    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
     return NS_OK;
   }
 
@@ -1512,9 +1552,8 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   // initialize the search list for this char
   PRBool alreadyCSS = PR_FALSE;
   nsAutoVoidArray tableList;
-  PRInt32 index = nsMathMLOperators::FindStretchyOperator(uchar);
   // see if there are user-specified preferred tables for the variants of this char
-  PRInt32 count, t = nsGlyphTableList::gVariants[index];
+  PRInt32 count, t = nsGlyphTableList::gVariants[mOperator];
   gGlyphTableList->GetPreferredListAt(aPresContext, t, &tableList, &count);
   if (!count) {
     // get a list that attempts to honor the css font-family
@@ -1538,7 +1577,7 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
     }
     glyphTable->GetPrimaryFontName(fontName);
     SetFirstFamily(theFont, fontName);
-    aRenderingContext.SetFont(theFont);
+    aRenderingContext.SetFont(theFont, nsnull);
 #ifdef NOISY_SEARCH
     char str[50];
     fontName.ToCString(str, sizeof(str));
@@ -1591,7 +1630,7 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
   ////////////////////////////////////////////////////////////////////////////////////
 
   // see if there are user-specified preferred tables for the parts of this char
-  t = nsGlyphTableList::gParts[index];
+  t = nsGlyphTableList::gParts[mOperator];
   gGlyphTableList->GetPreferredListAt(aPresContext, t, &tableList, &count);
   if (!count && !alreadyCSS) {
     // we didn't do this earlier... so we need to do it here:
@@ -1635,7 +1674,7 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
     // See if the parts of this table fit in the desired space ///////////////////////
     glyphTable->GetPrimaryFontName(fontName);
     SetFirstFamily(theFont, fontName);
-    aRenderingContext.SetFont(theFont);
+    aRenderingContext.SetFont(theFont, nsnull);
     // Compute the bounding metrics of all partial glyphs
     PRInt32 i;
     nsGlyphCode chdata[4];
@@ -1726,7 +1765,8 @@ nsMathMLChar::Stretch(nsIPresContext*      aPresContext,
 done:
   if (bestGlyph == startingGlyph) { // nothing happened
     // ensure that the char behaves like a normal char
-    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED; // XXX to reset in dynamic updates
+    // (will be reset back to its intrinsic value in case of dynamic updates)
+    mDirection = NS_STRETCH_DIRECTION_UNSUPPORTED;
   }
   else {
     // will stretch
@@ -1785,6 +1825,7 @@ nsMathMLChar::ComposeChildren(nsIPresContext*      aPresContext,
   for (i = 0, child = mSibling; child; child = child->mSibling, i++) {
     // child chars should just inherit our values - which may change between calls...
     child->mData = mData;
+    child->mOperator = mOperator;
     child->mDirection = mDirection;
     child->mStyleContext = mStyleContext;
     child->mGlyphTable = aGlyphTable; // the child is associated to this table
@@ -1825,7 +1866,8 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
                     nsIRenderingContext& aRenderingContext,
                     const nsRect&        aDirtyRect,
                     nsFramePaintLayer    aWhichLayer,
-                    nsIFrame*            aForFrame)
+                    nsIFrame*            aForFrame,
+                    const nsRect*        aSelectedRect)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIStyleContext> parentContext(dont_AddRef(mStyleContext->GetParent()));
@@ -1839,35 +1881,66 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
 
   const nsStyleVisibility *visib = NS_STATIC_CAST(const nsStyleVisibility*,
     styleContext->GetStyleData(eStyleStruct_Visibility));
+  if (!visib->IsVisible())
+    return NS_OK;
 
-  if (visib->IsVisible() && NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer)
-  {
-    if (mRect.width && mRect.height) {
-      // Paint our background and border
-      PRIntn skipSides = 0; //aForFrame->GetSkipSides();
+  // if the leaf style context that we use for stretchy chars has a background
+  // color we use it -- this feature is mostly used for testing and debugging
+  // purposes. Normally, users will set the background on the container frame.
+  if (NS_FRAME_PAINT_LAYER_BACKGROUND == aWhichLayer) {
+    // paint the selection background -- beware MathML frames overlap a lot
+    if (aSelectedRect && !aSelectedRect->IsEmpty()) {
+      // get color to use for selection from the look&feel object
+      nsCOMPtr<nsILookAndFeel> lf;
+      aPresContext->GetLookAndFeel(getter_AddRefs(lf));
+      if (lf) {
+        nscolor bgColor = NS_RGB(0, 0, 0);
+        lf->GetColor(nsILookAndFeel::eColor_TextSelectBackground, bgColor);
+        aRenderingContext.SetColor(bgColor);
+        aRenderingContext.FillRect(*aSelectedRect);
+      }
+    }
+    else if (mRect.width && mRect.height) {
       const nsStyleBorder *border = NS_STATIC_CAST(const nsStyleBorder*,
         styleContext->GetStyleData(eStyleStruct_Border));
-      const nsStyleOutline *outline = NS_STATIC_CAST(const nsStyleOutline*,
-        styleContext->GetStyleData(eStyleStruct_Outline));
       const nsStyleBackground *backg = NS_STATIC_CAST(const nsStyleBackground*,
         styleContext->GetStyleData(eStyleStruct_Background));
-
       nsRect rect(mRect); //0, 0, mRect.width, mRect.height);
-      nsCSSRendering::PaintBackgroundWithSC(aPresContext, aRenderingContext, aForFrame,
-                                      aDirtyRect, rect, *backg, *border, 0, 0);
+      if (styleContext != parentContext.get() &&
+          0 == (backg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT))
+        nsCSSRendering::PaintBackgroundWithSC(aPresContext, aRenderingContext, aForFrame,
+                                              aDirtyRect, rect, *backg, *border, 0, 0);
+      //else
+      //  our container frame will take care of painting its background
+      //  nsCSSRendering::PaintBackground(aPresContext, aRenderingContext, aForFrame,
+      //                                  aDirtyRect, rect, *border, 0, 0);
+#if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
+      // for visual debug
+      PRIntn skipSides = 0; //aForFrame->GetSkipSides();
+      const nsStyleOutline *outline = NS_STATIC_CAST(const nsStyleOutline*,
+        styleContext->GetStyleData(eStyleStruct_Outline));
       nsCSSRendering::PaintBorder(aPresContext, aRenderingContext, aForFrame,
                                   aDirtyRect, rect, *border, styleContext, skipSides);
       nsCSSRendering::PaintOutline(aPresContext, aRenderingContext, aForFrame,
                                    aDirtyRect, rect, *border, *outline, styleContext, 0);
+#endif
     }
   }
 
-  if (visib->IsVisible() && NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer)
-  {
+  if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
     // Set color ...
     const nsStyleColor *color = NS_STATIC_CAST(const nsStyleColor*,
       styleContext->GetStyleData(eStyleStruct_Color));
-    aRenderingContext.SetColor(color->mColor);
+    nscolor fgColor = color->mColor;
+    if (aSelectedRect && !aSelectedRect->IsEmpty()) {
+      // get color to use for selection from the look&feel object
+      nsCOMPtr<nsILookAndFeel> lf;
+      aPresContext->GetLookAndFeel(getter_AddRefs(lf));
+      if (lf) {
+        lf->GetColor(nsILookAndFeel::eColor_TextSelectForeground, fgColor);
+      }
+    }
+    aRenderingContext.SetColor(fgColor);
 
     nsAutoString fontName;
     const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
@@ -1880,10 +1953,10 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
       PRUint32 len = PRUint32(mData.Length());
       PRUnichar uchar = mData[0];
       if ((1 == len) && (kSqrChar == uchar)) {        // Special to the sqrt char. Due to
-        fontName.Assign(NS_LITERAL_STRING("CMSY10")); // assumptions in the sqrt code, we need
+        fontName.Assign(NS_LITERAL_STRING("CMSY10,Math2")); // assumptions in the sqrt code, we need
         SetFirstFamily(theFont, fontName);        // to force precedence on this TeX font
       }
-      aRenderingContext.SetFont(theFont);
+      aRenderingContext.SetFont(theFont, nsnull);
 //printf("Painting %04X like a normal char\n", mData[0]);
 //aRenderingContext.SetColor(NS_RGB(255,0,0));
       aRenderingContext.DrawString(mData.get(), len, mRect.x, mRect.y + mBoundingMetrics.ascent);
@@ -1892,7 +1965,7 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
       // Set the stretchy font and grab some metrics to adjust the placements ...
       mGlyphTable->GetPrimaryFontName(fontName);
       SetFirstFamily(theFont, fontName);
-      aRenderingContext.SetFont(theFont);
+      aRenderingContext.SetFont(theFont, nsnull);
       // if there is a glyph of appropriate size, paint that glyph
       if (mGlyph) {
 //printf("Painting %04X with a glyph of appropriate size\n", mData[0]);
@@ -1907,7 +1980,7 @@ nsMathMLChar::Paint(nsIPresContext*      aPresContext,
 //if (!mStyleContext->Equals(child->mStyleContext))
 //  printf("char contexts are out of sync\n");
             child->Paint(aPresContext, aRenderingContext,
-                         aDirtyRect, aWhichLayer, aForFrame);
+                         aDirtyRect, aWhichLayer, aForFrame, aSelectedRect);
           }
           return NS_OK; // that's all folks
         }
@@ -1950,7 +2023,7 @@ nsMathMLChar::PaintVertically(nsIPresContext*      aPresContext,
   PRInt32 i;
   nsGlyphCode ch, chdata[4];
   nsBoundingMetrics bm, bmdata[4];
-  nscoord stride, offset[3], start[3], end[3];
+  nscoord stride = 0, offset[3], start[3], end[3];
   nscoord width = aRect.width;
   nsGlyphCode glue = aGlyphTable->GlueOf(aPresContext, aChar);
   for (i = 0; i < 4; i++) {
@@ -2001,21 +2074,21 @@ nsMathMLChar::PaintVertically(nsIPresContext*      aPresContext,
   // draw top, middle, bottom
   for (i = 0; i < 3; i++) {
     ch = chdata[i];
-#ifdef SHOW_BORDERS
-    // bounding box of the part
-    aRenderingContext.SetColor(NS_RGB(0,0,0));
-    aRenderingContext.DrawRect(nsRect(dx,start[i],width+30*(i+1),end[i]-start[i]));
-#endif
     // glue can be null, and other parts could have been set to glue
     if (ch) {
+#ifdef SHOW_BORDERS
+      // bounding box of the part
+      aRenderingContext.SetColor(NS_RGB(0,0,0));
+      aRenderingContext.DrawRect(nsRect(dx,start[i],width+30*(i+1),end[i]-start[i]));
+#endif
       dy = offset[i];
-      if (0 == i) {
+      if (0 == i) { // top
         clipRect.SetRect(dx, aRect.y, width, aRect.height);
       }
-      else if (1 == i) {
+      else if (1 == i) { // middle
         clipRect.SetRect(dx, end[0], width, start[2]-end[0]);
       }
-      else {
+      else { // bottom
         clipRect.SetRect(dx, start[2], width, end[2]-start[2]);
       }
       if (!clipRect.IsEmpty()) {
@@ -2055,14 +2128,14 @@ nsMathMLChar::PaintVertically(nsIPresContext*      aPresContext,
         rbearing = bmdata[first].rightBearing;
       }
       else {
-        NS_ASSERTION(PR_FALSE, "Cannot stretch - All parts missing");
+        NS_ERROR("Cannot stretch - All parts missing");
         return NS_ERROR_UNEXPECTED;
       }
       // paint the rule between the parts
-      aRenderingContext.FillRect(aRect.x + lbearing,
-                                 end[first] - onePixel,
-                                 rbearing - lbearing,
-                                 start[last] - end[first] + 2*onePixel);
+      nsRect rule(aRect.x + lbearing, end[first] - onePixel,
+                  rbearing - lbearing, start[last] - end[first] + 2*onePixel);
+      if (!rule.IsEmpty())
+        aRenderingContext.FillRect(rule);
       first = last;
       last++;
     }
@@ -2091,7 +2164,7 @@ nsMathMLChar::PaintVertically(nsIPresContext*      aPresContext,
         count++;
         dy += stride;
         aGlyphTable->DrawGlyph(aRenderingContext, aFont, glue, dx, dy);
-        // NS_ASSERTION(5000 == count, "Error - glyph table is incorrectly set");
+        NS_ASSERTION(1000 != count, "something is probably wrong somewhere");
         if (1000 == count) return NS_ERROR_UNEXPECTED;
       }
       aRenderingContext.PopState(clipState);
@@ -2128,7 +2201,7 @@ nsMathMLChar::PaintHorizontally(nsIPresContext*      aPresContext,
   PRInt32 i;
   nsGlyphCode ch, chdata[4];
   nsBoundingMetrics bm, bmdata[4];
-  nscoord stride, offset[3], start[3], end[3];
+  nscoord stride = 0, offset[3], start[3], end[3];
   dy = aRect.y;
   nsGlyphCode glue = aGlyphTable->GlueOf(aPresContext, aChar);
   for (i = 0; i < 4; i++) {
@@ -2188,13 +2261,13 @@ nsMathMLChar::PaintHorizontally(nsIPresContext*      aPresContext,
                                  end[i] - start[i], bmdata[i].ascent + bmdata[i].descent));
 #endif
       dx = offset[i];
-      if (0 == i) {
+      if (0 == i) { // left
         clipRect.SetRect(dx, aRect.y, aRect.width, aRect.height);
       }
-      else if (1 == i) {
+      else if (1 == i) { // middle
         clipRect.SetRect(end[0], aRect.y, start[2]-end[0], aRect.height);
       }
-      else {
+      else { // right
         clipRect.SetRect(start[2], aRect.y, end[2]-start[2], aRect.height);
       }
       if (!clipRect.IsEmpty()) {
@@ -2233,14 +2306,14 @@ nsMathMLChar::PaintHorizontally(nsIPresContext*      aPresContext,
         descent = bmdata[first].descent;
       }
       else {
-        NS_ASSERTION(PR_FALSE, "Cannot stretch - All parts missing");
+        NS_ERROR("Cannot stretch - All parts missing");
         return NS_ERROR_UNEXPECTED;
       }
       // paint the rule between the parts
-      aRenderingContext.FillRect(end[first] - onePixel,
-                                 dy - ascent,
-                                 start[last] - end[first] + 2*onePixel,
-                                 ascent + descent);
+      nsRect rule(end[first] - onePixel, dy - ascent,
+                  start[last] - end[first] + 2*onePixel, ascent + descent);
+      if (!rule.IsEmpty())
+        aRenderingContext.FillRect(rule);
       first = last;
       last++;
     }
@@ -2269,7 +2342,7 @@ nsMathMLChar::PaintHorizontally(nsIPresContext*      aPresContext,
         count++;
         dx += stride;
         aGlyphTable->DrawGlyph(aRenderingContext, aFont, glue, dx, dy);
-        // NS_ASSERTION(5000 == count, "Error - glyph table is incorrectly set");
+        NS_ASSERTION(1000 != count, "something is probably wrong somewhere");
         if (1000 == count) return NS_ERROR_UNEXPECTED;
       }
       aRenderingContext.PopState(clipState);
