@@ -22,12 +22,10 @@
  */
 
 #include "nsCOMPtr.h"
-#include "nsHTMLReflowCommand.h"
 #include "nsHTMLParts.h"
 #include "nsIHTMLContent.h"
 #include "nsFrame.h"
 #include "nsLineLayout.h"
-#include "nsHTMLIIDs.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsHTMLAtoms.h"
@@ -41,7 +39,7 @@
 
 #include "nsIDOMText.h"
 #include "nsITextContent.h"
-
+#include "nsIDOMMutationEvent.h"
 #include "nsIFrameManager.h"
 #include "nsStyleChangeList.h"
 
@@ -64,8 +62,7 @@ NS_IMPL_QUERY_INTERFACE_INHERITED1(nsMathMLContainerFrame, nsHTMLContainerFrame,
 // =============================================================================
 
 // error handlers
-// by default show the Unicode REPLACEMENT CHARACTER U+FFFD
-// when a frame with bad markup can not be rendered
+// provide a feedback to the user when a frame with bad markup can not be rendered
 nsresult
 nsMathMLContainerFrame::ReflowError(nsIPresContext*      aPresContext,
                                     nsIRenderingContext& aRenderingContext,
@@ -81,10 +78,10 @@ nsMathMLContainerFrame::ReflowError(nsIPresContext*      aPresContext,
   // Set font
   const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
     mStyleContext->GetStyleData(eStyleStruct_Font));
-  aRenderingContext.SetFont(font->mFont);
+  aRenderingContext.SetFont(font->mFont, nsnull);
 
   // bounding metrics
-  nsAutoString errorMsg(PRUnichar(0xFFFD));
+  nsAutoString errorMsg; errorMsg.AssignWithConversion("invalid-markup");
   rv = aRenderingContext.GetBoundingMetrics(errorMsg.get(),
                                             PRUint32(errorMsg.Length()),
                                             mBoundingMetrics);
@@ -119,27 +116,27 @@ nsMathMLContainerFrame::PaintError(nsIPresContext*      aPresContext,
                                    const nsRect&        aDirtyRect,
                                    nsFramePaintLayer    aWhichLayer)
 {
-  if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer)
-  {
+  if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer) {
     NS_ASSERTION(NS_MATHML_HAS_ERROR(mPresentationData.flags),
                  "There is nothing wrong with this frame!");
     // Set color and font ...
     const nsStyleFont *font = NS_STATIC_CAST(const nsStyleFont*,
       mStyleContext->GetStyleData(eStyleStruct_Font));
-    const nsStyleColor *color = NS_STATIC_CAST(const nsStyleColor*,
-      mStyleContext->GetStyleData(eStyleStruct_Color));
-    aRenderingContext.SetColor(color->mColor);
-    aRenderingContext.SetFont(font->mFont);
+    aRenderingContext.SetFont(font->mFont, nsnull);
+
+    aRenderingContext.SetColor(NS_RGB(255,0,0));
+    aRenderingContext.FillRect(0, 0, mRect.width, mRect.height);
+    aRenderingContext.SetColor(NS_RGB(255,255,255));
 
     nscoord ascent;
     nsCOMPtr<nsIFontMetrics> fm;
     aRenderingContext.GetFontMetrics(*getter_AddRefs(fm));
     fm->GetMaxAscent(ascent);
 
-    nsAutoString errorMsg(PRUnichar(0xFFFD));
+    nsAutoString errorMsg; errorMsg.AssignWithConversion("invalid-markup");
     aRenderingContext.DrawString(errorMsg.get(),
                                  PRUint32(errorMsg.Length()),
-                                 mRect.x, mRect.y + ascent);
+                                 0, ascent);
   }
   return NS_OK;
 }
@@ -149,35 +146,6 @@ nsMathMLContainerFrame::PaintError(nsIPresContext*      aPresContext,
  * =============================================================================
  */
 
-// This is the method used to set the frame as an embellished container.
-// It checks if the first (non-empty) child is embellished. Hence, calls
-// must be bottom-up. The method must only be called from within frames who are
-// entitled to be potential embellished operators as per the MathML REC.
-NS_IMETHODIMP
-nsMathMLContainerFrame::EmbellishOperator()
-{
-  nsIFrame* firstChild = mFrames.FirstChild();
-  if (firstChild && IsEmbellishOperator(firstChild)) {
-    // Cache the first child
-    mEmbellishData.flags |= NS_MATHML_EMBELLISH_OPERATOR;
-    mEmbellishData.next = firstChild;
-    // Cache also the inner-most embellished frame at the core of the hierarchy
-    nsIMathMLFrame* mathMLFrame;
-    firstChild->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-    nsEmbellishData embellishData;
-    mathMLFrame->GetEmbellishData(embellishData);
-    mEmbellishData.core = embellishData.core;
-    mEmbellishData.direction = embellishData.direction;
-  }
-  else {
-    mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_OPERATOR;
-    mEmbellishData.next = nsnull;
-    mEmbellishData.core = nsnull;
-    mEmbellishData.direction = NS_STRETCH_DIRECTION_UNSUPPORTED;
-  }
-  return NS_OK;
-}
-
 // helper method to facilitate getting the reflow and bounding metrics
 void
 nsMathMLContainerFrame::GetReflowAndBoundingMetricsFor(nsIFrame*            aFrame,
@@ -233,8 +201,8 @@ nsMathMLContainerFrame::GetPreferredStretchSize(nsIPresContext*      aPresContex
   else {
     // compute a size that doesn't include embellishements
     NS_ASSERTION(NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags) ||
-                 NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags) ||
-                 NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags),
+                 NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags) ||
+                 NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags),
                  "invalid call to GetPreferredStretchSize");
     nsRect rect;
     PRBool firstTime = PR_TRUE;
@@ -258,10 +226,10 @@ nsMathMLContainerFrame::GetPreferredStretchSize(nsIPresContext*      aPresContex
         mathMLFrame->GetEmbellishData(childData);
         if (NS_MATHML_IS_EMBELLISH_OPERATOR(childData.flags) &&
             childData.direction == aStretchDirection &&
-            childData.next) {
+            childData.nextFrame) {
           // embellishements are not included, only consider the inner first child itself
           nsIMathMLFrame* mathMLchildFrame;
-          childData.next->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLchildFrame);
+          childData.nextFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLchildFrame);
           if (mathMLchildFrame) {
             mathMLFrame = mathMLchildFrame;
           }
@@ -272,14 +240,14 @@ nsMathMLContainerFrame::GetPreferredStretchSize(nsIPresContext*      aPresContex
       if (firstTime) {
         firstTime = PR_FALSE;
         bm = bmChild;
-        if (!NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags) &&
-            !NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags)) {
+        if (!NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags) &&
+            !NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags)) {
           // we may get here for cases such as <msup><mo>...</mo> ... </msup>
           break;
         }
       }
       else {
-        if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags)) {
+        if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags)) {
           // if we get here, it means this is container that will stack its children
           // vertically and fire an horizontal stretch on each them. This is the case
           // for \munder, \mover, \munderover. We just sum-up the size vertically.
@@ -289,7 +257,7 @@ nsMathMLContainerFrame::GetPreferredStretchSize(nsIPresContext*      aPresContex
           if (bm.rightBearing < bmChild.rightBearing)
             bm.rightBearing = bmChild.rightBearing;
         }
-        else if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags)) {
+        else if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags)) {
           // just sum-up the sizes horizontally.
           bm += bmChild;
         }
@@ -313,11 +281,11 @@ nsMathMLContainerFrame::Stretch(nsIPresContext*      aPresContext,
 {
   if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags)) {
 
-    if (NS_MATHML_STRETCH_WAS_DONE(mEmbellishData.flags)) {
+    if (NS_MATHML_STRETCH_WAS_DONE(mPresentationData.flags)) {
       NS_WARNING("it is wrong to fire stretch more than once on a frame");
       return NS_OK;
     }
-    mEmbellishData.flags |= NS_MATHML_STRETCH_DONE;
+    mPresentationData.flags |= NS_MATHML_STRETCH_DONE;
 
     if (NS_MATHML_HAS_ERROR(mPresentationData.flags)) {
       NS_WARNING("it is wrong to fire stretch on a erroneous frame");
@@ -326,15 +294,15 @@ nsMathMLContainerFrame::Stretch(nsIPresContext*      aPresContext,
 
     // Pass the stretch to the first non-empty child ...
 
-    nsIFrame* childFrame = mEmbellishData.next;
+    nsIFrame* childFrame = mEmbellishData.nextFrame;
     if (childFrame) {
       nsIMathMLFrame* mathMLFrame;
       childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
       NS_ASSERTION(mathMLFrame, "Something is wrong somewhere");
       if (mathMLFrame) {
         PRBool stretchAll =
-          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags) ||
-          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags);
+          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags) ||
+          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags);
 
         // And the trick is that the child's rect.x is still holding the descent,
         // and rect.y is still holding the ascent ...
@@ -381,7 +349,7 @@ nsMathMLContainerFrame::Stretch(nsIPresContext*      aPresContext,
         if (stretchAll) {
 
           nsStretchDirection stretchDir =
-            NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags) ?
+            NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags) ?
               NS_STRETCH_DIRECTION_VERTICAL : NS_STRETCH_DIRECTION_HORIZONTAL;
 
           GetPreferredStretchSize(aPresContext, aRenderingContext, STRETCH_CONSIDER_EMBELLISHMENTS,
@@ -389,7 +357,7 @@ nsMathMLContainerFrame::Stretch(nsIPresContext*      aPresContext,
 
           childFrame = mFrames.FirstChild();
           while (childFrame) {
-            if (childFrame != mEmbellishData.next) {
+            if (childFrame != mEmbellishData.nextFrame) {
               childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
               if (mathMLFrame) {
                 // retrieve the metrics that was stored at the previous pass
@@ -415,11 +383,16 @@ nsMathMLContainerFrame::Stretch(nsIPresContext*      aPresContext,
         // container and so we put the spacing, otherwise we don't include the spacing,
         // the outermost embellished container will take care of it.
 
-        if (!IsEmbellishOperator(mParent)) {
-
+        nsEmbellishData parentData;
+        GetEmbellishDataFrom(mParent, parentData);
+        // ensure that we are the embellished child, not just a sibling
+        // (need to test coreFrame since <mfrac> resets other things)
+        if (parentData.coreFrame != mEmbellishData.coreFrame) {
+          // (we fetch values from the core since they may use units that depend
+          // on style data, and style changes could have occured in the core since
+          // our last visit there)
           nsEmbellishData coreData;
-          mEmbellishData.core->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-          mathMLFrame->GetEmbellishData(coreData);
+          GetEmbellishDataFrom(mEmbellishData.coreFrame, coreData);
 
           mBoundingMetrics.width += coreData.leftSpace + coreData.rightSpace;
           aDesiredStretchSize.width = mBoundingMetrics.width;
@@ -474,7 +447,7 @@ nsMathMLContainerFrame::FinalizeReflow(nsIPresContext*      aPresContext,
   // because it excludes the particular case of the core <mo>...</mo> itself.
   // (<mo> needs to fire stretch on its MathMLChar in any case to initialize it)
   PRBool placeOrigin = !NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags) ||
-                       (mEmbellishData.core != this && !mEmbellishData.next &&
+                       (mEmbellishData.coreFrame != this && !mEmbellishData.nextFrame &&
                         mEmbellishData.direction == NS_STRETCH_DIRECTION_UNSUPPORTED);
   Place(aPresContext, aRenderingContext, placeOrigin, aDesiredSize);
 
@@ -483,15 +456,17 @@ nsMathMLContainerFrame::FinalizeReflow(nsIPresContext*      aPresContext,
     // Don't go without checking to see if our parent will later fire a Stretch() command
     // targeted at us. The Stretch() will cause the rect.x and rect.y to clear...
     PRBool parentWillFireStretch = PR_FALSE;
-    nsEmbellishData parentData;
     nsIMathMLFrame* mathMLFrame;
     mParent->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
     if (mathMLFrame) {
-      mathMLFrame->GetEmbellishData(parentData);
-      if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(parentData.flags) ||
-          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(parentData.flags) ||
-          (NS_MATHML_IS_EMBELLISH_OPERATOR(parentData.flags)
-            && parentData.next == this))
+      nsEmbellishData embellishData;
+      nsPresentationData presentationData;
+      mathMLFrame->GetEmbellishData(embellishData);
+      mathMLFrame->GetPresentationData(presentationData);
+      if (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(presentationData.flags) ||
+          NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(presentationData.flags) ||
+          (NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags)
+            && embellishData.nextFrame == this))
       {
         parentWillFireStretch = PR_TRUE;
       }
@@ -500,11 +475,11 @@ nsMathMLContainerFrame::FinalizeReflow(nsIPresContext*      aPresContext,
       // There is nobody who will fire the stretch for us, we do it ourselves!
 
       PRBool stretchAll =
-        /* NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags) || */
-        NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags);
+        /* NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags) || */
+        NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags);
 
       nsBoundingMetrics defaultSize;
-      if (mEmbellishData.core == this /* case of a bare <mo>...</mo> itself */
+      if (mEmbellishData.coreFrame == this /* case of a bare <mo>...</mo> itself */
           || stretchAll) { /* or <mover><mo>...</mo>...</mover>, or friends */
         // use our current size as computed earlier by Place()
         defaultSize = aDesiredSize.mBoundingMetrics;
@@ -728,6 +703,49 @@ nsMathMLContainerFrame::PropagateScriptStyleFor(nsIPresContext* aPresContext,
  * =============================================================================
  */
 
+// We use this to wrap non-MathML frames so that foreign elements (e.g.,
+// html:img) can mix better with other surrounding MathML markups.
+// Currently we only wrap nsInlineFrames because problems were observed only
+// in the presence of such frames. By construction, a foreign frame wrapper
+// has one and only one child, and the life of the wrapper is bound to the
+// life of that unique child. Not all child list operations are applicable
+// with a wrapper. One must either use the parent (or the unique child)
+// for such operations (@see nsMathMLForeignFrameWrapper).
+nsresult
+nsMathMLContainerFrame::WrapForeignFrames(nsIPresContext* aPresContext)
+{
+  nsIFrame* next = mFrames.FirstChild();
+  while (next) {
+    nsIFrame* child = next;
+    next->GetNextSibling(&next);
+    nsInlineFrame* inlineFrame;
+    child->QueryInterface(nsInlineFrame::kInlineFrameCID, (void**)&inlineFrame);
+    if (inlineFrame) {
+      // create a new wrapper frame to wrap this child
+      nsCOMPtr<nsIPresShell> shell;
+      aPresContext->GetShell(getter_AddRefs(shell));
+      nsIFrame* wrapper;
+      nsresult rv = NS_NewMathMLForeignFrameWrapper(shell, &wrapper);
+      if (NS_FAILED(rv)) return rv;
+      nsCOMPtr<nsIStyleContext> newStyleContext;
+      aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::mozAnonymousBlock,
+                                                 mStyleContext,
+                                                 getter_AddRefs(newStyleContext));
+      rv = wrapper->Init(aPresContext, mContent, this, newStyleContext, nsnull);
+      if (NS_FAILED(rv)) {
+        wrapper->Destroy(aPresContext);
+        return rv;
+      }
+      mFrames.ReplaceFrame(this, child, wrapper);
+      child->SetParent(wrapper);
+      child->SetNextSibling(nsnull);
+      aPresContext->ReParentStyleContext(child, newStyleContext);
+      wrapper->SetInitialChildList(aPresContext, nsnull, child);
+    }
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMathMLContainerFrame::Paint(nsIPresContext*      aPresContext,
                               nsIRenderingContext& aRenderingContext,
@@ -815,14 +833,13 @@ nsMathMLContainerFrame::Init(nsIPresContext*  aPresContext,
   // XXX the best fix is to skip these in nsTextFrame
   CompressWhitespace(aContent);
 
+  MapAttributesIntoCSS(aPresContext, aContent);
+
   // let the base class do its Init()
-  nsresult rv;
-  rv = nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+  return nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 
-  // now, inherit the scriptlevel and displaystyle from our parent
-  GetPresentationDataFrom(aParent, mPresentationData);
-
-  return rv;
+  // ...We will build our automatic MathML data once the entire <math>...</math>
+  // tree is constructed.
 }
 
 // This method is called in a bottom-up manner, as we ascend the frame tree
@@ -833,8 +850,7 @@ nsMathMLContainerFrame::SetInitialChildList(nsIPresContext* aPresContext,
                                             nsIFrame*       aChildList)
 {
   // First, let the base class do its job
-  nsresult rv;
-  rv = nsHTMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  nsresult rv = nsHTMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
 
   // Next, since we are an inline frame, and since we are a container, we have to
   // be very careful with the way we treat our children. Things look okay when
@@ -846,44 +862,204 @@ nsMathMLContainerFrame::SetInitialChildList(nsIPresContext* aPresContext,
   // In short, the nsInlineFrame class expects a number of *invariants* that are not
   // met when we mix things.
 
-  // So what we do here is to wrap children that happen to be nsInlineFrames in
-  // anonymous block frames.
-  // XXX Question: Do we have to handle Insert/Remove/Append on behalf of
-  //     these anonymous blocks?
-  //     Note: By construction, our anonymous blocks have only one child.
+  // So wrap foreign children in nsMathMLForeignFrameWrapper frames
+  WrapForeignFrames(aPresContext);
+  return rv;
 
-  nsIFrame* next = mFrames.FirstChild();
-  while (next) {
-    nsIFrame* child = next;
-    next->GetNextSibling(&next);
-    nsInlineFrame* inlineFrame = nsnull;
-    child->QueryInterface(nsInlineFrame::kInlineFrameCID, (void**)&inlineFrame);
-    if (inlineFrame) {
-      // create a new anonymous block frame to wrap this child...
-      nsCOMPtr<nsIPresShell> shell;
-      aPresContext->GetShell(getter_AddRefs(shell));
-      nsIFrame* anonymous;
-      rv = NS_NewBlockFrame(shell, &anonymous);
-      if (NS_FAILED(rv))
-        return rv;
-      nsCOMPtr<nsIStyleContext> newStyleContext;
-      aPresContext->ResolvePseudoStyleContextFor(mContent, nsHTMLAtoms::mozAnonymousBlock,
-                                                 mStyleContext, PR_FALSE,
-                                                 getter_AddRefs(newStyleContext));
-      rv = anonymous->Init(aPresContext, mContent, this, newStyleContext, nsnull);
-      if (NS_FAILED(rv)) {
-        anonymous->Destroy(aPresContext);
-        return rv;
-      }
-      mFrames.ReplaceFrame(this, child, anonymous);
-      child->SetParent(anonymous);
-      child->SetNextSibling(nsnull);
-      aPresContext->ReParentStyleContext(child, newStyleContext);
-      anonymous->SetInitialChildList(aPresContext, nsnull, child);
+  // ...We will build our automatic MathML data once the entire <math>...</math>
+  // tree is constructed.
+}
+
+// Note that this method re-builds the automatic data in the children -- not
+// in aParentFrame itself (except for those particular operations that the
+// parent frame may do in its TransmitAutomaticData()).
+/* static */ void
+nsMathMLContainerFrame::RebuildAutomaticDataForChildren(nsIPresContext* aPresContext,
+                                                        nsIFrame*       aParentFrame)
+{
+  // 1. As we descend the tree, make each child frame inherit data from
+  // the parent
+  // 2. As we ascend the tree, transmit any specific change that we want
+  // down the subtrees
+  nsIFrame* childFrame;
+  aParentFrame->FirstChild(aPresContext, nsnull, &childFrame);
+  while (childFrame) {
+    nsIMathMLFrame* childMathMLFrame;
+    childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&childMathMLFrame);
+    if (childMathMLFrame) {
+      childMathMLFrame->InheritAutomaticData(aPresContext, aParentFrame);
     }
+    RebuildAutomaticDataForChildren(aPresContext, childFrame);
+    childFrame->GetNextSibling(&childFrame);
+  }
+  nsIMathMLFrame* mathMLFrame;
+  aParentFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
+  if (mathMLFrame) {
+    mathMLFrame->TransmitAutomaticData(aPresContext);
+  }
+}
+
+/* static */ nsresult
+nsMathMLContainerFrame::ReLayoutChildren(nsIPresContext* aPresContext,
+                                         nsIFrame*       aParentFrame)
+{
+  // walk-up to the first frame that is a MathML frame, stop if we reach <math>
+  PRInt32 parentScriptLevel = 0;
+  nsIFrame* frame = aParentFrame;
+  while (frame) {
+    // stop if it is a MathML frame
+    nsIMathMLFrame* mathMLFrame;
+    frame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
+    if (mathMLFrame) {
+      nsPresentationData parentData;
+      mathMLFrame->GetPresentationData(parentData);
+      parentScriptLevel = parentData.scriptLevel;
+      break;
+    }
+    // stop if we reach the root <math> tag
+    nsCOMPtr<nsIAtom> tag;
+    nsCOMPtr<nsIContent> content;
+    frame->GetContent(getter_AddRefs(content));
+    NS_ASSERTION(content, "dangling frame without a content node");
+    if (!content)
+      return NS_ERROR_FAILURE;
+    content->GetTag(*getter_AddRefs(tag));
+    if (tag.get() == nsMathMLAtoms::math) {
+      break;
+    }
+    // mark the frame dirty, and continue to climb up
+    nsFrameState state;
+    frame->GetFrameState(&state);
+    frame->SetFrameState(state | NS_FRAME_IS_DIRTY | NS_FRAME_HAS_DIRTY_CHILDREN);
+    frame->GetParent(&frame);
+  }
+  NS_ASSERTION(frame, "bad MathML markup - could not find the top <math> element");
+  if (!frame)
+    return NS_OK;
+
+  // re-sync the presentation data and embellishment data of our children
+#ifdef DEBUG_rbs
+  nsEmbellishData oldData;
+  GetEmbellishDataFrom(frame, oldData);
+#endif
+
+  RebuildAutomaticDataForChildren(aPresContext, frame);
+
+#ifdef DEBUG_rbs
+  nsEmbellishData newData;
+  GetEmbellishDataFrom(frame, newData);
+  NS_ASSERTION((NS_MATHML_IS_EMBELLISH_OPERATOR(oldData.flags) == 
+                NS_MATHML_IS_EMBELLISH_OPERATOR(newData.flags)) && 
+               (oldData.coreFrame == newData.coreFrame),
+               "embellished hierarchy has to be rebuilt too");
+#endif
+
+  // re-resolve the style data to sync any change of script sizes
+  nsIFrame* childFrame;
+  aParentFrame->FirstChild(aPresContext, nsnull, &childFrame);
+  while (childFrame) {
+    nsIMathMLFrame* mathMLFrame;
+    childFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
+    if (mathMLFrame) {
+      // propagate using the base method to make sure that the control
+      // is passed on to MathML frames that may be overloading the method
+      mathMLFrame->ReResolveScriptStyle(aPresContext, parentScriptLevel);
+    }
+    else {
+      PropagateScriptStyleFor(aPresContext, childFrame, parentScriptLevel);
+    }
+    childFrame->GetNextSibling(&childFrame);
   }
 
-  return rv;
+  // Ask our parent frame to reflow us
+  nsCOMPtr<nsIPresShell> presShell;
+  aPresContext->GetShell(getter_AddRefs(presShell));
+  return frame->ReflowDirtyChild(presShell, nsnull);
+}
+
+// There are precise rules governing children of a MathML frame,
+// and properties such as the scriptlevel depends on those rules.
+// Hence for things to work, callers must use Append/Insert/etc wisely.
+
+nsresult
+nsMathMLContainerFrame::ChildListChanged(nsIPresContext* aPresContext,
+                                         PRInt32         aModType)
+{
+  if (aModType != nsIDOMMutationEvent::REMOVAL) {
+    // wrap any new foreign child that may have crept in
+    WrapForeignFrames(aPresContext);
+  }
+  return ReLayoutChildren(aPresContext, this);
+}
+
+NS_IMETHODIMP
+nsMathMLContainerFrame::AppendFrames(nsIPresContext* aPresContext,
+                                     nsIPresShell&   aPresShell,
+                                     nsIAtom*        aListName,
+                                     nsIFrame*       aFrameList)
+{
+  if (aListName) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  if (aFrameList) {
+    mFrames.AppendFrames(this, aFrameList);
+    return ChildListChanged(aPresContext, nsIDOMMutationEvent::ADDITION);
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMathMLContainerFrame::InsertFrames(nsIPresContext* aPresContext,
+                                     nsIPresShell&   aPresShell,
+                                     nsIAtom*        aListName,
+                                     nsIFrame*       aPrevFrame,
+                                     nsIFrame*       aFrameList)
+{
+  if (aListName) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  if (aFrameList) {
+    // Insert frames after aPrevFrame
+    mFrames.InsertFrames(this, aPrevFrame, aFrameList);
+    return ChildListChanged(aPresContext, nsIDOMMutationEvent::ADDITION);
+  }
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMathMLContainerFrame::RemoveFrame(nsIPresContext* aPresContext,
+                                    nsIPresShell&   aPresShell,
+                                    nsIAtom*        aListName,
+                                    nsIFrame*       aOldFrame)
+{
+  if (aListName) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  // remove the child frame
+  mFrames.DestroyFrame(aPresContext, aOldFrame);
+  return ChildListChanged(aPresContext, nsIDOMMutationEvent::REMOVAL);
+}
+
+NS_IMETHODIMP
+nsMathMLContainerFrame::ReplaceFrame(nsIPresContext* aPresContext,
+                                     nsIPresShell&   aPresShell,
+                                     nsIAtom*        aListName,
+                                     nsIFrame*       aOldFrame,
+                                     nsIFrame*       aNewFrame)
+{
+  if (aListName || !aOldFrame || !aNewFrame) {
+    return NS_ERROR_INVALID_ARG;
+  }
+  // Replace the old frame with the new frame in the list
+  mFrames.ReplaceFrame(this, aOldFrame, aNewFrame);
+
+  // XXX now destroy the old frame, really? the usage of ReplaceFrame() vs.
+  // XXX ReplaceFrameAndDestroy() is ambiguous - see bug 122748
+  // XXX The style system doesn't call ReplaceFrame() and that's why
+  // XXX nobody seems to have been biten by the ambiguity yet
+  aOldFrame->Destroy(aPresContext);
+
+  return ChildListChanged(aPresContext, nsIDOMMutationEvent::MODIFICATION);
 }
 
 NS_IMETHODIMP
@@ -894,17 +1070,19 @@ nsMathMLContainerFrame::AttributeChanged(nsIPresContext* aPresContext,
                                          PRInt32         aModType, 
                                          PRInt32         aHint)
 {
-  nsresult rv = nsHTMLContainerFrame::AttributeChanged(aPresContext, aChild,
-                                                       aNameSpaceID, aAttribute, aModType, aHint);
-  if (NS_FAILED(rv)) return rv;
-  nsCOMPtr<nsIPresShell> shell;
-  nsHTMLReflowCommand *reflowCmd;
-  aPresContext->GetShell(getter_AddRefs(shell));
-  rv = NS_NewHTMLReflowCommand(&reflowCmd, this,
-                               eReflowType_ContentChanged,
-                               nsnull, aAttribute);
-  if (NS_SUCCEEDED(rv) && shell) shell->AppendReflowCommand(reflowCmd);
-  return rv;
+  if (aAttribute == nsMathMLAtoms::mathcolor_      ||
+      aAttribute == nsMathMLAtoms::color_          ||
+      aAttribute == nsMathMLAtoms::mathsize_       ||
+      aAttribute == nsMathMLAtoms::fontsize_       ||
+      aAttribute == nsMathMLAtoms::fontfamily_     ||
+      aAttribute == nsMathMLAtoms::mathbackground_ ||
+      aAttribute == nsMathMLAtoms::background_) {
+    MapAttributesIntoCSS(aPresContext, this);
+  }
+
+  nsCOMPtr<nsIPresShell> presShell;
+  aPresContext->GetShell(getter_AddRefs(presShell));
+  return ReflowDirtyChild(presShell, nsnull);
 }
 
 // helper function to reflow token elements
@@ -921,22 +1099,11 @@ nsMathMLContainerFrame::ReflowTokenFor(nsIFrame*                aFrame,
 
   // See if this is an incremental reflow
   if (aReflowState.reason == eReflowReason_Incremental) {
-    nsIFrame* targetFrame;
-    aReflowState.reflowCommand->GetTarget(targetFrame);
 #ifdef MATHML_NOISY_INCREMENTAL_REFLOW
 printf("nsMathMLContainerFrame::ReflowTokenFor:IncrementalReflow received by: ");
 nsFrame::ListTag(stdout, aFrame);
-printf("for target: ");
-nsFrame::ListTag(stdout, targetFrame);
 printf("\n");
 #endif
-    if (aFrame == targetFrame) {
-    }
-    else {
-      // Remove the next frame from the reflow path
-      nsIFrame* nextFrame;
-      aReflowState.reflowCommand->GetNext(nextFrame);
-    }
   }
 
   // initializations needed for empty markup like <mtag></mtag>
@@ -983,6 +1150,7 @@ printf("\n");
   NS_STATIC_CAST(nsMathMLContainerFrame*,
                  aFrame)->FinalizeReflow(aPresContext, *aReflowState.rendContext,
                                          aDesiredSize);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -1000,28 +1168,32 @@ nsMathMLContainerFrame::PlaceTokenFor(nsIFrame*            aFrame,
   aDesiredSize.width = aDesiredSize.height = 0;
   aDesiredSize.ascent = aDesiredSize.descent = 0;
 
-  nsRect rect;
-  nsIFrame* childFrame;
-  aFrame->FirstChild(aPresContext, nsnull, &childFrame);
-  while (childFrame) {
-    childFrame->GetRect(rect);
-    aDesiredSize.width += rect.width;
-    if (aDesiredSize.descent < rect.x) aDesiredSize.descent = rect.x;
-    if (aDesiredSize.ascent < rect.y) aDesiredSize.ascent = rect.y;
-    childFrame->GetNextSibling(&childFrame);
-  }
+  const nsStyleFont* font;
+  aFrame->GetStyleData(eStyleStruct_Font, (const nsStyleStruct*&)font);
+  nsCOMPtr<nsIFontMetrics> fm;
+  aPresContext->GetMetricsFor(font->mFont, getter_AddRefs(fm));
+  nscoord ascent, descent;
+  fm->GetMaxAscent(ascent);
+  fm->GetMaxDescent(descent);
+
+  nsBoundingMetrics bm;
+  NS_STATIC_CAST(nsMathMLContainerFrame*, aFrame)->GetBoundingMetrics(bm);
+  aDesiredSize.mBoundingMetrics = bm;
+  aDesiredSize.width = bm.width;
+  aDesiredSize.ascent = PR_MAX(bm.ascent, ascent);
+  aDesiredSize.descent = PR_MAX(bm.descent, descent);
   aDesiredSize.height = aDesiredSize.ascent + aDesiredSize.descent;
-  NS_STATIC_CAST(nsMathMLContainerFrame*,
-                 aFrame)->GetBoundingMetrics(aDesiredSize.mBoundingMetrics);
 
   if (aPlaceOrigin) {
     nscoord dy, dx = 0;
+    nsRect rect;
+    nsIFrame* childFrame;
     aFrame->FirstChild(aPresContext, nsnull, &childFrame);
     while (childFrame) {
       childFrame->GetRect(rect);
       nsHTMLReflowMetrics childSize(nsnull);
       childSize.width = rect.width;
-      childSize.height = rect.height;
+      childSize.height = aDesiredSize.height; //rect.height;
 
       // place and size the child
       dy = aDesiredSize.ascent - rect.y;
@@ -1077,25 +1249,11 @@ nsMathMLContainerFrame::Reflow(nsIPresContext*          aPresContext,
 
   // See if this is an incremental reflow
   if (aReflowState.reason == eReflowReason_Incremental) {
-    nsIFrame* targetFrame;
-    aReflowState.reflowCommand->GetTarget(targetFrame);
 #ifdef MATHML_NOISY_INCREMENTAL_REFLOW
 printf("nsMathMLContainerFrame::Reflow:IncrementalReflow received by: ");
 nsFrame::ListTag(stdout, this);
-printf("for target: ");
-nsFrame::ListTag(stdout, targetFrame);
 printf("\n");
 #endif
-    if (this == targetFrame) {
-      // XXX We are the target of the incremental reflow.
-      // Rather than reflowing everything, see if we can speedup things
-      // by just doing the minimal work needed to update ourselves
-    }
-    else {
-      // Remove the next frame from the reflow path
-      nsIFrame* nextFrame;
-      aReflowState.reflowCommand->GetNext(nextFrame);
-    }
   }
 
   /////////////
@@ -1132,12 +1290,12 @@ printf("\n");
   // after finishing the stretching of the embellished child - bug 117652
 
   if (!NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags) &&
-      (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags) ||
-       NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mEmbellishData.flags))) {
+      (NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags) ||
+       NS_MATHML_WILL_STRETCH_ALL_CHILDREN_HORIZONTALLY(mPresentationData.flags))) {
 
     // get the stretchy direction
     nsStretchDirection stretchDir =
-      NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mEmbellishData.flags) 
+      NS_MATHML_WILL_STRETCH_ALL_CHILDREN_VERTICALLY(mPresentationData.flags) 
       ? NS_STRETCH_DIRECTION_VERTICAL 
       : NS_STRETCH_DIRECTION_HORIZONTAL;
 
@@ -1175,6 +1333,7 @@ printf("\n");
   FinalizeReflow(aPresContext, *aReflowState.rendContext, aDesiredSize);
 
   aStatus = NS_FRAME_COMPLETE;
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -1303,7 +1462,13 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
       aDesiredSize.descent = childSize.descent;
       mBoundingMetrics = bmChild;
       // update to include the left correction
-      mBoundingMetrics.leftBearing += leftCorrection;
+      // but leave <msqrt> alone because the sqrt glyph itself is there first
+      nsCOMPtr<nsIAtom> tag;
+      mContent->GetTag(*getter_AddRefs(tag));
+      if (tag.get() == nsMathMLAtoms::msqrt_)
+        leftCorrection = 0;
+      else
+        mBoundingMetrics.leftBearing += leftCorrection;
     }
     else {
       if (aDesiredSize.descent < childSize.descent)
@@ -1350,8 +1515,16 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
       nsCOMPtr<nsIAtom> childFrameType;
       childFrame->GetFrameType(getter_AddRefs(childFrameType));
       GetReflowAndBoundingMetricsFor(childFrame, childSize, bmChild);
+      GetItalicCorrection(bmChild, leftCorrection, italicCorrection);
       dy = aDesiredSize.ascent - childSize.ascent;
-      if (0 < count) {
+      if (0 == count) {
+        // for <msqrt>, the sqrt glyph itself is there first
+        nsCOMPtr<nsIAtom> tag;
+        mContent->GetTag(*getter_AddRefs(tag));
+        if (tag.get() == nsMathMLAtoms::msqrt_)
+          leftCorrection = 0;
+      }
+      else {
         // add inter frame spacing
         nscoord space = GetInterFrameSpacing(mPresentationData.scriptLevel,
           prevFrameType, childFrameType);
@@ -1359,7 +1532,6 @@ nsMathMLContainerFrame::Place(nsIPresContext*      aPresContext,
       }
       count++;
       prevFrameType = childFrameType;
-      GetItalicCorrection(bmChild, leftCorrection, italicCorrection);
       // add left correction
       dx += leftCorrection;
       FinishReflowChild(childFrame, aPresContext, nsnull, childSize, dx, dy, 0);
@@ -1434,6 +1606,7 @@ nsMathMLContainerFrame::FixInterFrameSpacing(nsIPresContext*      aPresContext,
 
 
 //==========================
+
 nsresult
 NS_NewMathMLmathBlockFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
 {
@@ -1449,6 +1622,8 @@ NS_NewMathMLmathBlockFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
   return NS_OK;
 }
 
+//==========================
+
 nsresult
 NS_NewMathMLmathInlineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
 {
