@@ -79,9 +79,6 @@ public:
         PRBool               aPlaceOrigin,
         nsHTMLReflowMetrics& aDesiredSize);
 
-  NS_IMETHOD
-  EmbellishOperator();
-
   NS_IMETHOD
   UpdatePresentationDataFromChildAt(nsIPresContext* aPresContext,
                                     PRInt32         aFirstIndex,
@@ -105,7 +102,7 @@ public:
 
   // --------------------------------------------------------------------------
   // Overloaded nsHTMLContainerFrame methods -- see documentation in nsIFrame.h
- 
+
   NS_IMETHOD
   GetFrameType(nsIAtom** aType) const;
 
@@ -121,6 +118,32 @@ public:
                       nsIAtom*        aListName,
                       nsIFrame*       aChildList);
 
+  NS_IMETHOD
+  AppendFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aFrameList);
+
+  NS_IMETHOD
+  InsertFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aPrevFrame,
+               nsIFrame*       aFrameList);
+
+  NS_IMETHOD
+  RemoveFrame(nsIPresContext* aPresContext,
+              nsIPresShell&   aPresShell,
+              nsIAtom*        aListName,
+              nsIFrame*       aOldFrame);
+
+  NS_IMETHOD
+  ReplaceFrame(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aOldFrame,
+               nsIFrame*       aNewFrame);
+
   NS_IMETHODIMP
   ReflowDirtyChild(nsIPresShell* aPresShell, 
                    nsIFrame*     aChild);
@@ -137,18 +160,10 @@ public:
             nsDidReflowStatus         aStatus)
 
   {
-    mEmbellishData.flags &= ~NS_MATHML_STRETCH_DONE;
+    mPresentationData.flags &= ~NS_MATHML_STRETCH_DONE;
     return nsHTMLContainerFrame::DidReflow(aPresContext, aReflowState, aStatus);
   }
 
-  NS_IMETHOD
-  AttributeChanged(nsIPresContext* aPresContext,
-                   nsIContent*     aChild,
-                   PRInt32         aNameSpaceID,
-                   nsIAtom*        aAttribute,
-                   PRInt32         aModType, 
-                   PRInt32         aHint);
-
   NS_IMETHOD 
   Paint(nsIPresContext*      aPresContext,
         nsIRenderingContext& aRenderingContext,
@@ -156,9 +171,45 @@ public:
         nsFramePaintLayer    aWhichLayer,
         PRUint32             aFlags = 0);
 
+  // Notification when an attribute is changed. The MathML module uses the
+  // following paradigm:
+  //
+  // 1. If the MathML frame class doesn't have any cached automatic data that
+  //    depends on the attribute: we just reflow (e.g., this happens with <msub>,
+  //    <msup>, <mmultiscripts>, etc). This is the default behavior implemented
+  //    by this base class.
+  //
+  // 2. If the MathML frame class has cached automatic data that depends on
+  //    the attribute:
+  //    2a. If the automatic data to update resides only within the descendants,
+  //        we just re-layout them using ReLayoutChildren(aPresContext, this);
+  //        (e.g., this happens with <ms>).
+  //    2b. If the automatic data to update affects us in some way, we ask our parent
+  //        to re-layout its children using ReLayoutChildren(aPresContext, mParent);
+  //        Therefore, there is an overhead here in that our siblings are re-laid
+  //        too (e.g., this happens with <mstyle>, <munder>, <mover>, <munderover>). 
+  NS_IMETHOD
+  AttributeChanged(nsIPresContext* aPresContext,
+                   nsIContent*     aChild,
+                   PRInt32         aNameSpaceID,
+                   nsIAtom*        aAttribute,
+                   PRInt32         aModType, 
+                   PRInt32         aHint);
+
   // --------------------------------------------------------------------------
   // Additional methods 
 
+  // helper to re-sync the automatic data in our children and notify our parent to
+  // reflow us when changes (e.g., append/insert/remove) happen in our child list
+  virtual nsresult
+  ChildListChanged(nsIPresContext* aPresContext,
+                   PRInt32         aModType);
+
+  // helper to wrap non-MathML frames so that foreign elements (e.g., html:img)
+  // can mix better with other surrounding MathML markups
+  virtual nsresult
+  WrapForeignFrames(nsIPresContext* aPresContext);
+
   // helper to get the preferred size that a container frame should use to fire
   // the stretch on its stretchy child frames.
   virtual void
@@ -168,10 +219,8 @@ public:
                           nsStretchDirection   aStretchDirection,
                           nsBoundingMetrics&   aPreferredStretchSize);
 
-  // error handlers to report than an error (typically invalid markup)
-  // was encountered during reflow. By default the user will see the
-  // Unicode REPLACEMENT CHAR U+FFFD at the spot where the error was
-  // encountered.
+  // error handlers to provide a visual feedback to the user when an error
+  // (typically invalid markup) was encountered during reflow.
   virtual nsresult
   ReflowError(nsIPresContext*      aPresContext,
               nsIRenderingContext& aRenderingContext,
@@ -244,7 +293,7 @@ public:
   static void
   PropagateScriptStyleFor(nsIPresContext* aPresContext,
                           nsIFrame*       aFrame,
-                          PRInt32         aFrameScriptLevel);
+                          PRInt32         aParentScriptLevel);
 
   // helper to let the update of presentation data pass through
   // a subtree that may contain non-MathML container frames
@@ -264,6 +313,34 @@ public:
                                        PRUint32        aFlagsValues,
                                        PRUint32        aFlagsToUpdate);
 
+  // helper to let the rebuild of automatic data (presentation data
+  // and embellishement data) walk through a subtree that may contain
+  // non-MathML container frames. Note that this method re-builds the
+  // automatic data in the children -- not in aParentFrame itself (except
+  // for those particular operations that the parent frame may do in its
+  // TransmitAutomaticData()). The reason it works this way is because
+  // a container frame knows what it wants for its children, whereas children
+  // have no clue who their parent is. For example, it is <mfrac> who knows
+  // that its children have to be in scriptsizes, and has to transmit this
+  // information to them. Hence, when changes occur in a child frame, the child
+  // has to request the re-build from its parent. Unfortunately, the extra cost
+  // for this is that it will re-sync in the siblings of the child as well.
+  static void
+  RebuildAutomaticDataForChildren(nsIPresContext* aPresContext,
+                                  nsIFrame*       aParentFrame);
+
+  // helper to blow away the automatic data cached in a frame's subtree and
+  // re-layout its subtree to reflect changes that may have happen. In the
+  // event where aParentFrame isn't a MathML frame, it will first walk up to
+  // the ancestor that is a MathML frame, and re-layout from there -- this is
+  // to guarantee that automatic data will be rebuilt properly. Note that this
+  // method re-builds the automatic data in the children -- not in the parent
+  // frame itself (except for those particular operations that the parent frame
+  // may do do its TransmitAutomaticData()). @see RebuildAutomaticDataForChildren
+  static nsresult
+  ReLayoutChildren(nsIPresContext* aPresContext,
+                   nsIFrame*       aParentFrame);
+
 protected:
   virtual PRIntn GetSkipSides() const { return 0; }
 };
@@ -281,28 +358,95 @@ class nsMathMLmathBlockFrame : public nsBlockFrame {
 public:
   friend nsresult NS_NewMathMLmathBlockFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame);
 
-  // beware, mFrames is not set by nsBlockFrame, FirstChild() is your friend
-  // when you need to access the child list of the block
+  // beware, mFrames is not set by nsBlockFrame
+  // cannot use mFrames{.FirstChild()|.etc} since the block code doesn't set mFrames
   NS_IMETHOD
   SetInitialChildList(nsIPresContext* aPresContext,
                       nsIAtom*        aListName,
                       nsIFrame*       aChildList)
   {
-    nsresult rv;
+    nsresult rv = nsBlockFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+    // re-resolve our subtree to set any mathml-expected data
+    nsMathMLContainerFrame::MapAttributesIntoCSS(aPresContext, this);
+    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(aPresContext, this);
+    return rv;
+  }
 
-    rv = nsBlockFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  NS_IMETHOD
+  Reflow(nsIPresContext*          aPresContext,
+         nsHTMLReflowMetrics&     aDesiredSize,
+         const nsHTMLReflowState& aReflowState,
+         nsReflowStatus&          aStatus)
+  {
+    if (mScriptStyleChanged) {
+      mScriptStyleChanged = PR_FALSE;
+      nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+    }
+    return nsBlockFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
+  }
 
-    // re-resolve our subtree to set any mathml-expected scriptsizes
-    nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+  NS_IMETHOD
+  AppendFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aFrameList)
+  {
+    nsresult rv = nsBlockFrame::AppendFrames(aPresContext, aPresShell, aListName, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
 
+  NS_IMETHOD
+  InsertFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aPrevFrame,
+               nsIFrame*       aFrameList)
+  {
+    nsresult rv = nsBlockFrame::InsertFrames(aPresContext, aPresShell, aListName, aPrevFrame, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
+
+  NS_IMETHOD
+  ReplaceFrame(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aOldFrame,
+               nsIFrame*       aNewFrame)
+  {
+    nsresult rv = nsBlockFrame::ReplaceFrame(aPresContext, aPresShell, aListName, aOldFrame, aNewFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
+
+  NS_IMETHOD
+  RemoveFrame(nsIPresContext* aPresContext,
+              nsIPresShell&   aPresShell,
+              nsIAtom*        aListName,
+              nsIFrame*       aOldFrame)
+  {
+    nsresult rv = nsBlockFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
     return rv;
   }
 
 protected:
   nsMathMLmathBlockFrame() {}
   virtual ~nsMathMLmathBlockFrame() {}
+
+  NS_IMETHOD
+  DidSetStyleContext(nsIPresContext* aPresContext)
+  {
+    mScriptStyleChanged = PR_TRUE;
+    return nsBlockFrame::DidSetStyleContext(aPresContext);
+  }
+
+  PRBool mScriptStyleChanged;
 };
 
+// --------------
+
 class nsMathMLmathInlineFrame : public nsInlineFrame {
 public:
   friend nsresult NS_NewMathMLmathInlineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame);
@@ -312,17 +456,84 @@ public:
                       nsIAtom*        aListName,
                       nsIFrame*       aChildList)
   {
-    nsresult rv;
-    rv = nsInlineFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+    nsresult rv = nsInlineFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+    // re-resolve our subtree to set any mathml-expected data
+    nsMathMLContainerFrame::MapAttributesIntoCSS(aPresContext, this);
+    nsMathMLContainerFrame::RebuildAutomaticDataForChildren(aPresContext, this);
+    return rv;
+  }
 
-    // re-resolve our subtree to set any mathml-expected scriptsizes
-    nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+  NS_IMETHOD
+  Reflow(nsIPresContext*          aPresContext,
+         nsHTMLReflowMetrics&     aDesiredSize,
+         const nsHTMLReflowState& aReflowState,
+         nsReflowStatus&          aStatus)
+  {
+    if (mScriptStyleChanged) {
+      mScriptStyleChanged = PR_FALSE;
+      nsMathMLContainerFrame::PropagateScriptStyleFor(aPresContext, this, 0);
+    }
+    return nsInlineFrame::Reflow(aPresContext, aDesiredSize, aReflowState, aStatus);
+  }
 
+  NS_IMETHOD
+  AppendFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aFrameList)
+  {
+    nsresult rv = nsInlineFrame::AppendFrames(aPresContext, aPresShell, aListName, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
+
+  NS_IMETHOD
+  InsertFrames(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aPrevFrame,
+               nsIFrame*       aFrameList)
+  {
+    nsresult rv = nsInlineFrame::InsertFrames(aPresContext, aPresShell, aListName, aPrevFrame, aFrameList);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
+
+  NS_IMETHOD
+  ReplaceFrame(nsIPresContext* aPresContext,
+               nsIPresShell&   aPresShell,
+               nsIAtom*        aListName,
+               nsIFrame*       aOldFrame,
+               nsIFrame*       aNewFrame)
+  {
+    nsresult rv = nsInlineFrame::ReplaceFrame(aPresContext, aPresShell, aListName, aOldFrame, aNewFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
+    return rv;
+  }
+
+  NS_IMETHOD
+  RemoveFrame(nsIPresContext* aPresContext,
+              nsIPresShell&   aPresShell,
+              nsIAtom*        aListName,
+              nsIFrame*       aOldFrame)
+  {
+    nsresult rv = nsInlineFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
+    nsMathMLContainerFrame::ReLayoutChildren(aPresContext, this);
     return rv;
   }
 
 protected:
   nsMathMLmathInlineFrame() {}
   virtual ~nsMathMLmathInlineFrame() {}
+
+  NS_IMETHOD
+  DidSetStyleContext(nsIPresContext* aPresContext)
+  {
+    mScriptStyleChanged = PR_TRUE;
+    return nsInlineFrame::DidSetStyleContext(aPresContext);
+  }
+
+  PRBool mScriptStyleChanged;
 };
+
 #endif /* nsMathMLContainerFrame_h___ */
