@@ -23,8 +23,55 @@
 #include "nsMathMLFrame.h"
 #include "nsMathMLChar.h"
 
+// used to map attributes into CSS rules
+#include "nsIDocument.h"
+#include "nsIStyleSet.h"
+#include "nsIStyleSheet.h"
+#include "nsICSSStyleSheet.h"
+#include "nsIDOMCSSStyleSheet.h"
+#include "nsICSSRule.h"
+#include "nsICSSStyleRule.h"
+#include "nsStyleChangeList.h"
+#include "nsIFrameManager.h"
+#include "nsNetUtil.h"
+#include "nsIURI.h"
+#include "nsContentCID.h"
+static NS_DEFINE_CID(kCSSStyleSheetCID, NS_CSS_STYLESHEET_CID);
+
+
 NS_IMPL_QUERY_INTERFACE1(nsMathMLFrame, nsIMathMLFrame)
 
+NS_IMETHODIMP
+nsMathMLFrame::InheritAutomaticData(nsIPresContext* aPresContext,
+                                    nsIFrame*       aParent) 
+{
+  mEmbellishData.flags = 0;
+  mEmbellishData.nextFrame = nsnull;
+  mEmbellishData.coreFrame = nsnull;
+  mEmbellishData.direction = NS_STRETCH_DIRECTION_UNSUPPORTED;
+  mEmbellishData.leftSpace = 0;
+  mEmbellishData.rightSpace = 0;
+
+  mPresentationData.flags = 0;
+  mPresentationData.mstyle = nsnull;
+  mPresentationData.scriptLevel = 0;
+
+  // by default, just inherit the display & scriptlevel of our parent
+  nsPresentationData parentData;
+  GetPresentationDataFrom(aParent, parentData);
+  mPresentationData.mstyle = parentData.mstyle;
+  mPresentationData.scriptLevel = parentData.scriptLevel;
+  if (NS_MATHML_IS_DISPLAYSTYLE(parentData.flags)) {
+    mPresentationData.flags |= NS_MATHML_DISPLAYSTYLE;
+  }
+
+#if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
+  mPresentationData.flags |= NS_MATHML_SHOW_BOUNDING_METRICS;
+#endif
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMathMLFrame::UpdatePresentationData(nsIPresContext* aPresContext,
                                       PRInt32         aScriptLevelIncrement,
@@ -53,69 +100,80 @@ nsMathMLFrame::UpdatePresentationData(nsIPresContext* aPresContext,
   return NS_OK;
 }
 
-/* static */ PRBool
-nsMathMLFrame::IsEmbellishOperator(nsIFrame* aFrame)
-{
-  NS_PRECONDITION(aFrame, "null arg");
-  if (!aFrame) return PR_FALSE;
-  nsIMathMLFrame* mathMLFrame;
-  aFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-  if (!mathMLFrame) return PR_FALSE;
-  nsEmbellishData embellishData;
-  mathMLFrame->GetEmbellishData(embellishData);
-  return NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags);
-}
-
 // Helper to give a style context suitable for doing the stretching of
 // a MathMLChar. Frame classes that use this should ensure that the 
 // extra leaf style contexts given to the MathMLChars are acessible to
 // the Style System via the Get/Set AdditionalStyleContext() APIs.
-/* static */ PRBool
+/* static */ void
 nsMathMLFrame::ResolveMathMLCharStyle(nsIPresContext*  aPresContext,
                                       nsIContent*      aContent,
                                       nsIStyleContext* aParentStyleContext,
-                                      nsMathMLChar*    aMathMLChar)
+                                      nsMathMLChar*    aMathMLChar,
+                                      PRBool           aIsMutableChar)
 {
-  nsAutoString data;
-  aMathMLChar->GetData(data);
-  PRBool isStretchy = nsMathMLOperators::IsMutableOperator(data);
-  nsIAtom* fontAtom = (isStretchy) ?
+  nsIAtom* fontAtom = (aIsMutableChar) ?
     nsMathMLAtoms::fontstyle_stretchy :
-    nsMathMLAtoms::fontstyle_anonymous;
+    nsMathMLAtoms::fontstyle_anonymous; // savings
   nsCOMPtr<nsIStyleContext> newStyleContext;
   nsresult rv = aPresContext->ResolvePseudoStyleContextFor(aContent, fontAtom, 
-                                             aParentStyleContext, PR_FALSE,
+                                             aParentStyleContext,
                                              getter_AddRefs(newStyleContext));
   if (NS_SUCCEEDED(rv) && newStyleContext)
     aMathMLChar->SetStyleContext(newStyleContext);
+}
 
-  return isStretchy;
+/* static */ void
+nsMathMLFrame::GetEmbellishDataFrom(nsIFrame*        aFrame,
+                                    nsEmbellishData& aEmbellishData)
+{
+  // initialize OUT params
+  aEmbellishData.flags = 0;
+  aEmbellishData.nextFrame = nsnull;
+  aEmbellishData.coreFrame = nsnull;
+  aEmbellishData.direction = NS_STRETCH_DIRECTION_UNSUPPORTED;
+  aEmbellishData.leftSpace = 0;
+  aEmbellishData.rightSpace = 0;
+
+  if (aFrame) {
+    nsIMathMLFrame* mathMLFrame;
+    aFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
+    if (mathMLFrame) {
+      mathMLFrame->GetEmbellishData(aEmbellishData);
+    }
+  }
 }
 
 // helper to get the presentation data of a frame, by possibly walking up
 // the frame hierarchy if we happen to be surrounded by non-MathML frames.
 /* static */ void
 nsMathMLFrame::GetPresentationDataFrom(nsIFrame*           aFrame,
-                                       nsPresentationData& aPresentationData)
+                                       nsPresentationData& aPresentationData,
+                                       PRBool              aClimbTree)
 {
+  // initialize OUT params
+  aPresentationData.flags = 0;
+  aPresentationData.mstyle = nsnull;
+  aPresentationData.scriptLevel = 0;
+
   nsIFrame* frame = aFrame;
   while (frame) {
     nsIMathMLFrame* mathMLFrame;
     frame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
     if (mathMLFrame) {
-      nsPresentationData presentationData;
-      mathMLFrame->GetPresentationData(presentationData);
-      aPresentationData.mstyle = presentationData.mstyle;
-      aPresentationData.scriptLevel = presentationData.scriptLevel;
-      if (NS_MATHML_IS_DISPLAYSTYLE(presentationData.flags)) {
-        aPresentationData.flags |= NS_MATHML_DISPLAYSTYLE;
-      }
+      mathMLFrame->GetPresentationData(aPresentationData);
+      break;
+    }
+    // stop if the caller doesn't want to lookup beyond the frame
+    if (!aClimbTree) {
       break;
     }
     // stop if we reach the root <math> tag
     nsCOMPtr<nsIAtom> tag;
     nsCOMPtr<nsIContent> content;
     frame->GetContent(getter_AddRefs(content));
+    NS_ASSERTION(content, "dangling frame without a content node");
+    if (!content)
+      break;
     content->GetTag(*getter_AddRefs(tag));
     if (tag.get() == nsMathMLAtoms::math) {
       const nsStyleDisplay* display;
@@ -127,6 +185,18 @@ nsMathMLFrame::GetPresentationDataFrom(nsIFrame*           aFrame,
     }
     frame->GetParent(&frame);
   }
+  NS_ASSERTION(frame, "bad MathML markup - could not find the top <math> element");
+}
+
+/* static */ PRBool
+nsMathMLFrame::HasNextSibling(nsIFrame* aFrame)
+{
+  if (aFrame) {
+    nsIFrame* sibling;
+    aFrame->GetNextSibling(&sibling);
+    return sibling != nsnull;
+  }
+  return PR_FALSE;
 }
 
 // helper to get an attribute from the content or the surrounding <mstyle> hierarchy
@@ -230,7 +300,7 @@ nsMathMLFrame::GetAxisHeight(nsIRenderingContext& aRenderingContext,
 #endif
   nscoord xHeight;
   aFontMetrics->GetXHeight(xHeight);
-  PRUnichar minus = '-';
+  PRUnichar minus = 0x2212; // not '-', but official Unicode minus sign
   nsBoundingMetrics bm;
   nsresult rv = aRenderingContext.GetBoundingMetrics(&minus, PRUint32(1), bm);
   if (NS_SUCCEEDED(rv)) {
@@ -240,18 +310,6 @@ nsMathMLFrame::GetAxisHeight(nsIRenderingContext& aRenderingContext,
     // fall-back to the other version
     GetAxisHeight(aFontMetrics, aAxisHeight);
   }
-
-#if 0
-  nscoord oldAxis;
-  GetAxisHeight(aFontMetrics, oldAxis);
-
-  PRUnichar plus = '+';
-  rv = aRenderingContext.GetBoundingMetrics(&plus, PRUint32(1), bm);
-  nscoord plusAxis = bm.ascent - (bm.ascent + bm.descent)/2;;
-
-  printf("xheight:%4d Axis:%4d oldAxis:%4d  plusAxis:%4d\n",
-          xHeight, aAxisHeight, oldAxis, plusAxis);
-#endif
 }
 
 // ================
@@ -287,14 +345,10 @@ nsMathMLFrame::ParseNumericValue(nsString&   aString,
   aString.CompressWhitespace(); //  aString is not a const in this code...
 
   PRInt32 stringLength = aString.Length();
+  if (!stringLength)
+    return PR_FALSE;
 
-  if (!stringLength) return PR_FALSE;
-
-  nsAutoString number(aString);
-  number.SetLength(0);
-
-  nsAutoString unit(aString);
-  unit.SetLength(0);
+  nsAutoString number, unit;
 
   // Gather up characters that make up the number
   PRBool gotDot = PR_FALSE;
@@ -313,18 +367,23 @@ nsMathMLFrame::ParseNumericValue(nsString&   aString,
     number.Append(c);
   }
 
+  // on exit, also return a nicer string version of the value in case
+  // the caller wants it (e.g., this removes whitespace before units)
+  aString.Assign(number);
+  aString.Append(unit);
+
   // Convert number to floating point
   PRInt32 errorCode;
   float floatValue = number.ToFloat(&errorCode);
-  if (NS_FAILED(errorCode)) return PR_FALSE;
+  if (errorCode)
+    return PR_FALSE;
 
   nsCSSUnit cssUnit;
-  if (0 == unit.Length()) {
+  if (unit.IsEmpty()) {
     cssUnit = eCSSUnit_Number; // no explicit unit, this is a number that will act as a multiplier
   }
   else if (unit.Equals(NS_LITERAL_STRING("%"))) {
-    floatValue = floatValue / 100.0f;
-    aCSSValue.SetPercentValue(floatValue);
+    aCSSValue.SetPercentValue(floatValue / 100.0f);
     return PR_TRUE;
   }
   else if (unit.Equals(NS_LITERAL_STRING("em"))) cssUnit = eCSSUnit_EM;
@@ -389,7 +448,7 @@ nsMathMLFrame::ParseNamedSpaceValue(nsIFrame*   aMathMLmstyleFrame,
 
   // See if it is one of the 'namedspace' (ranging 1/18em...7/18em)
   PRInt32 i = 0;
-  nsIAtom* namedspaceAtom;
+  nsIAtom* namedspaceAtom = nsnull;
   if (aString.Equals(NS_LITERAL_STRING("veryverythinmathspace"))) {
     i = 1;
     namedspaceAtom = nsMathMLAtoms::veryverythinmathspace_;
@@ -440,3 +499,224 @@ nsMathMLFrame::ParseNamedSpaceValue(nsIFrame*   aMathMLmstyleFrame,
 
   return PR_FALSE;
 }
+
+// ================
+// Utils to map attributes into CSS rules (work-around to bug 69409 which
+// is not scheduled to be fixed anytime soon)
+//
+
+static const PRInt32 kMathMLversion1 = 1;
+static const PRInt32 kMathMLversion2 = 2;
+
+struct
+nsCSSMapping {
+  PRInt32        compatibility;
+  const nsIAtom* attrAtom;
+  const char*    cssProperty;
+};
+
+static void
+GetMathMLAttributeStyleSheet(nsIPresContext* aPresContext,
+                             nsIStyleSheet** aSheet)
+{
+  static const char kTitle[] = "Internal MathML/CSS Attribute Style Sheet";
+  *aSheet = nsnull;
+
+  // first, look if the attribute stylesheet is already there
+  nsCOMPtr<nsIPresShell> presShell;
+  aPresContext->GetShell(getter_AddRefs(presShell));
+  nsCOMPtr<nsIStyleSet> styleSet;
+  presShell->GetStyleSet(getter_AddRefs(styleSet));
+  if (!styleSet)
+    return;
+  nsAutoString title;
+  for (PRInt32 i = styleSet->GetNumberOfAgentStyleSheets() - 1; i >= 0; --i) {
+    nsCOMPtr<nsIStyleSheet> sheet = getter_AddRefs(styleSet->GetAgentStyleSheetAt(i));
+    nsCOMPtr<nsICSSStyleSheet> cssSheet(do_QueryInterface(sheet));
+    if (cssSheet) {
+      cssSheet->GetTitle(title);
+      if (title.Equals(NS_ConvertASCIItoUCS2(kTitle))) {
+        *aSheet = sheet;
+        NS_IF_ADDREF(*aSheet);
+        return;
+      }
+    }
+  }
+
+  // then, create a new one if it isn't yet there
+  nsCOMPtr<nsIURI> uri;
+  NS_NewURI(getter_AddRefs(uri), "about:internal-mathml-attribute-stylesheet");
+  if (!uri)
+    return;
+  nsCOMPtr<nsICSSStyleSheet> cssSheet(do_CreateInstance(kCSSStyleSheetCID));
+  if (!cssSheet)
+    return;
+  cssSheet->Init(uri);
+  cssSheet->SetTitle(NS_ConvertASCIItoUCS2(kTitle));
+  cssSheet->SetDefaultNameSpaceID(kNameSpaceID_MathML);
+  nsCOMPtr<nsIStyleSheet> sheet(do_QueryInterface(cssSheet));
+
+  // insert the stylesheet into the styleset without notifying observers
+  styleSet->AppendAgentStyleSheet(sheet);
+  *aSheet = sheet;
+  NS_ADDREF(*aSheet);
+}
+
+/* static */ PRInt32
+nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
+                                    nsIContent*     aContent)
+{
+  // normal case, quick return if there are no attributes
+  NS_ASSERTION(aContent, "null arg");
+  PRInt32 attrCount = 0;
+  if (aContent)
+    aContent->GetAttrCount(attrCount);
+  if (!attrCount)
+    return 0;
+
+  // need to initialize here -- i.e., after registering nsMathMLAtoms
+  static const nsCSSMapping
+  kCSSMappingTable[] = {
+    {kMathMLversion2, nsMathMLAtoms::mathcolor_,      "color:"},
+    {kMathMLversion1, nsMathMLAtoms::color_,          "color:"},
+    {kMathMLversion2, nsMathMLAtoms::mathsize_,       "font-size:"},
+    {kMathMLversion1, nsMathMLAtoms::fontsize_,       "font-size:"},
+    {kMathMLversion1, nsMathMLAtoms::fontfamily_,     "font-family:"},
+    {kMathMLversion2, nsMathMLAtoms::mathbackground_, "background-color:"},
+    {kMathMLversion1, nsMathMLAtoms::background_,     "background-color:"},
+    {0, nsnull, nsnull}
+  };
+
+  nsCOMPtr<nsIDocument> doc;
+  nsCOMPtr<nsIStyleSheet> sheet;
+  nsCOMPtr<nsICSSStyleSheet> cssSheet;
+  nsCOMPtr<nsIDOMCSSStyleSheet> domSheet;
+
+  PRInt32 nameSpaceID;
+  nsCOMPtr<nsIAtom> prefix;
+  nsCOMPtr<nsIAtom> attrAtom;
+  PRInt32 ruleCount = 0;
+  for (PRInt32 i = 0; i < attrCount; ++i) {
+    aContent->GetAttrNameAt(i, nameSpaceID, *getter_AddRefs(attrAtom), *getter_AddRefs(prefix));
+
+    // lookup the equivalent CSS property
+    const nsCSSMapping* map = kCSSMappingTable;
+    while (map->attrAtom && map->attrAtom != attrAtom)
+      ++map;
+    if (!map->attrAtom)
+      continue;
+    nsAutoString cssProperty(NS_ConvertASCIItoUCS2(map->cssProperty));
+
+    nsAutoString attrValue;
+    aContent->GetAttr(nameSpaceID, attrAtom, attrValue);
+    if (attrValue.IsEmpty())
+      continue;
+
+    const PRUnichar* attrName;
+    attrAtom->GetUnicode(&attrName);
+
+    // don't add rules that are already in mathml.css
+    // (this will also clean up whitespace before units - see bug 125303)
+    if (attrAtom == nsMathMLAtoms::fontsize_ || attrAtom == nsMathMLAtoms::mathsize_) {
+      nsCSSValue cssValue;
+      nsAutoString numericValue(attrValue);
+      if (!ParseNumericValue(numericValue, cssValue))
+        continue;
+      // on exit, ParseNumericValue also returns a nicer string
+      // in which the whitespace before the unit is cleaned up 
+      cssProperty.Append(numericValue);
+    }
+    else
+      cssProperty.Append(attrValue);
+
+    // make a style rule that maps to the equivalent CSS property
+    nsAutoString cssRule;
+    cssRule.Assign(NS_LITERAL_STRING("[")  + nsDependentString(attrName) +
+                   NS_LITERAL_STRING("='") + attrValue +
+                   NS_LITERAL_STRING("']{") + cssProperty + NS_LITERAL_STRING("}"));
+
+    if (!sheet) {
+      // first time... we do this to defer the lookup up to the
+      // point where we encounter attributes that actually matter
+      aContent->GetDocument(*getter_AddRefs(doc));
+      if (!doc) 
+        return 0;
+      GetMathMLAttributeStyleSheet(aPresContext, getter_AddRefs(sheet));
+      if (!sheet)
+        return 0;
+      // by construction, these cannot be null at this point
+      cssSheet = do_QueryInterface(sheet);
+      domSheet = do_QueryInterface(sheet);
+      NS_ASSERTION(cssSheet && domSheet, "unexpected null pointers");
+      // we will keep the sheet orphan as we populate it. This way,
+      // observers of the document won't be notified and we avoid any troubles
+      // that may come from reconstructing the frame tree. Our rules only need
+      // a re-resolve of style data and a reflow, not a reconstruct-all...
+      sheet->SetOwningDocument(nsnull);
+    }
+
+    // check for duplicate, if a similar rule is already there, don't bother to add another one
+    // XXX bug 142648 - GetSourceSelectorText is in the format *[color=blue] (i.e., no quotes...) 
+    // XXXrbs need to keep this in sync with the fix for bug 142648
+    nsAutoString selector;
+    selector.Assign(NS_LITERAL_STRING("*[") + nsDependentString(attrName) +
+                    NS_LITERAL_STRING("=") + attrValue +
+                    NS_LITERAL_STRING("]"));
+    PRInt32 k, count;
+    cssSheet->StyleRuleCount(count);
+    for (k = 0; k < count; ++k) {
+      nsAutoString tmpSelector;
+      nsCOMPtr<nsICSSRule> tmpRule;
+      cssSheet->GetStyleRuleAt(k, *getter_AddRefs(tmpRule));
+      nsCOMPtr<nsICSSStyleRule> tmpStyleRule(do_QueryInterface(tmpRule));
+      tmpStyleRule->GetSourceSelectorText(tmpSelector);
+      if (tmpSelector.Equals(selector)) {
+        k = -1;
+        break;
+      }
+    }
+    if (k >= 0) {
+      // insert the rule (note: when the sheet already has @namespace and
+      // friends, insert after them, e.g., at the end, otherwise it won't work)
+      // For MathML 2, insert at the end to give it precedence
+      PRInt32 pos = (map->compatibility == kMathMLversion2) ? count : 0;
+      PRUint32 index;
+      domSheet->InsertRule(cssRule, pos, &index);
+      ++ruleCount;
+    }
+  }
+  // restore the sheet to its owner
+  if (sheet) {
+    sheet->SetOwningDocument(doc);
+  }
+
+  return ruleCount;
+}
+
+/* static */ PRInt32
+nsMathMLFrame::MapAttributesIntoCSS(nsIPresContext* aPresContext,
+                                    nsIFrame*       aFrame)
+{
+  nsCOMPtr<nsIContent> content;
+  aFrame->GetContent(getter_AddRefs(content));
+  PRInt32 ruleCount = MapAttributesIntoCSS(aPresContext, content);
+  if (!ruleCount)
+    return 0;
+
+  // now, re-resolve the style contexts in our subtree
+  nsCOMPtr<nsIPresShell> presShell;
+  aPresContext->GetShell(getter_AddRefs(presShell));
+  if (presShell) {
+    nsCOMPtr<nsIFrameManager> fm;
+    presShell->GetFrameManager(getter_AddRefs(fm));
+    if (fm) {
+      PRInt32 maxChange, minChange = NS_STYLE_HINT_NONE;
+      nsStyleChangeList changeList;
+      fm->ComputeStyleChangeFor(aPresContext, aFrame,
+                                kNameSpaceID_None, nsnull,
+                                changeList, minChange, maxChange);
+    }
+  }
+
+  return ruleCount;
+}
