@@ -24,7 +24,6 @@
 #include "nsIHTMLContent.h"
 #include "nsFrame.h"
 #include "nsLineLayout.h"
-#include "nsHTMLIIDs.h"
 #include "nsIPresContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
@@ -99,7 +98,6 @@ nsMathMLmactionFrame::Init(nsIPresContext*  aPresContext,
                            nsIStyleContext* aContext,
                            nsIFrame*        aPrevInFlow)
 {
-  nsresult rv;
   nsAutoString value, prefix;
 
   // Init our local attributes
@@ -150,7 +148,7 @@ nsMathMLmactionFrame::Init(nsIPresContext*  aPresContext,
         aParent->GetStyleContext(getter_AddRefs(parentStyleContext));
         nsIStyleContext* newStyleContext;
         aPresContext->ResolveStyleContextFor(aContent, parentStyleContext,
-                                             PR_FALSE, &newStyleContext);
+                                             &newStyleContext);
         if (!newStyleContext) 
           mRestyle.Truncate();
         else {
@@ -166,13 +164,7 @@ nsMathMLmactionFrame::Init(nsIPresContext*  aPresContext,
   }
 
   // Let the base class do the rest
-  rv = nsMathMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
-
-#if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
-  mPresentationData.flags |= NS_MATHML_SHOW_BOUNDING_METRICS;
-#endif
-
-  return rv;
+  return nsMathMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
 }
 
 // return the frame whose number is given by the attribute selection="number"
@@ -221,23 +213,9 @@ nsMathMLmactionFrame::GetSelectedFrame()
 
   // if the selected child is an embellished operator,
   // we become embellished as well
-  mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_OPERATOR;
-  mEmbellishData.next = nsnull;
-  mEmbellishData.core = nsnull;
-  mEmbellishData.direction = NS_STRETCH_DIRECTION_UNSUPPORTED;
-  if (mSelectedFrame) {
-    nsIMathMLFrame* mathMLFrame;
-    mSelectedFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-    if (mathMLFrame) {
-      nsEmbellishData embellishData;
-      mathMLFrame->GetEmbellishData(embellishData);
-      if (NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags)) {
-        mEmbellishData.flags |= NS_MATHML_EMBELLISH_OPERATOR;
-        mEmbellishData.next = mSelectedFrame; // yes!
-        mEmbellishData.core = embellishData.core;
-        mEmbellishData.direction = embellishData.direction;
-      }
-    }
+  GetEmbellishDataFrom(mSelectedFrame, mEmbellishData);
+  if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags)) {
+    mEmbellishData.nextFrame = mSelectedFrame; // yes!
   }
 
   return mSelectedFrame;
@@ -316,8 +294,7 @@ nsMathMLmactionFrame::Paint(nsIPresContext*      aPresContext,
 #if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
   // visual debug
   if (NS_FRAME_PAINT_LAYER_FOREGROUND == aWhichLayer &&
-      NS_MATHML_PAINT_BOUNDING_METRICS(mPresentationData.flags))
-  {
+      NS_MATHML_PAINT_BOUNDING_METRICS(mPresentationData.flags)) {
     aRenderingContext.SetColor(NS_RGB(0,0,255));
 
     nscoord x = mReference.x + mBoundingMetrics.leftBearing;
@@ -344,9 +321,7 @@ nsMathMLmactionFrame::Reflow(nsIPresContext*          aPresContext,
   mBoundingMetrics.Clear();
   nsIFrame* childFrame = GetSelectedFrame();
   if (childFrame) {
-    nsSize availSize(aReflowState.mComputedWidth, aReflowState.mComputedHeight);
-    nsHTMLReflowState childReflowState(aPresContext, aReflowState,
-                                       childFrame, availSize);
+    nsReflowReason reason = aReflowState.reason;
     if (mWasRestyled) {
       mWasRestyled = PR_FALSE;
       // If we have just been restyled, make sure to reflow our
@@ -356,8 +331,12 @@ nsMathMLmactionFrame::Reflow(nsIPresContext*          aPresContext,
       // MouseClick() below). But that reason can be (and usually, it is)
       // changed in the reflow chain and we don't have much control other
       // than making sure that the right value is reset here.
-      childReflowState.reason = eReflowReason_StyleChange;
+      reason = eReflowReason_StyleChange;
     }
+
+    nsSize availSize(aReflowState.mComputedWidth, aReflowState.mComputedHeight);
+    nsHTMLReflowState childReflowState(aPresContext, aReflowState,
+                                       childFrame, availSize, reason);
     rv = ReflowChild(childFrame, aPresContext, aDesiredSize,
                      childReflowState, aStatus);
     childFrame->SetRect(aPresContext,
@@ -366,6 +345,7 @@ nsMathMLmactionFrame::Reflow(nsIPresContext*          aPresContext,
     mBoundingMetrics = aDesiredSize.mBoundingMetrics;
     FinalizeReflow(aPresContext, *aReflowState.rendContext, aDesiredSize);
   }
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return rv;
 }
 
