@@ -28,7 +28,6 @@
 #include "nsIHTMLContent.h"
 #include "nsFrame.h"
 #include "nsLineLayout.h"
-#include "nsHTMLIIDs.h"
 #include "nsIPresContext.h"
 #include "nsHTMLAtoms.h"
 #include "nsUnitConversion.h"
@@ -70,20 +69,23 @@ nsMathMLmunderoverFrame::~nsMathMLmunderoverFrame()
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::Init(nsIPresContext*  aPresContext,
-                              nsIContent*      aContent,
-                              nsIFrame*        aParent,
-                              nsIStyleContext* aContext,
-                              nsIFrame*        aPrevInFlow)
+nsMathMLmunderoverFrame::AttributeChanged(nsIPresContext* aPresContext,
+                                          nsIContent*     aContent,
+                                          PRInt32         aNameSpaceID,
+                                          nsIAtom*        aAttribute,
+                                          PRInt32         aModType, 
+                                          PRInt32         aHint)
 {
-  nsresult rv = nsMathMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+  if (nsMathMLAtoms::accent_ == aAttribute ||
+      nsMathMLAtoms::accentunder_ == aAttribute) {
+    // When we have automatic data to update within ourselves, we ask our
+    // parent to re-layout its children
+    return ReLayoutChildren(aPresContext, mParent);
+  }
 
-  mEmbellishData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
-
-#if defined(NS_DEBUG) && defined(SHOW_BOUNDING_BOX)
-  mPresentationData.flags |= NS_MATHML_SHOW_BOUNDING_METRICS;
-#endif
-  return rv;
+  return nsMathMLContainerFrame::
+         AttributeChanged(aPresContext, aContent, aNameSpaceID,
+                          aAttribute, aModType, aHint);
 }
 
 NS_IMETHODIMP
@@ -95,12 +97,12 @@ nsMathMLmunderoverFrame::UpdatePresentationData(nsIPresContext* aPresContext,
   nsMathMLContainerFrame::UpdatePresentationData(aPresContext,
     aScriptLevelIncrement, aFlagsValues, aFlagsToUpdate);
   // disable the stretch-all flag if we are going to act like a subscript-superscript pair
-  if ( NS_MATHML_IS_MOVABLELIMITS(mPresentationData.flags) &&
+  if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishData.flags) &&
       !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
-    mEmbellishData.flags &= ~NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
+    mPresentationData.flags &= ~NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
   }
   else {
-    mEmbellishData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
+    mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
   }
   return NS_OK;
 }
@@ -146,23 +148,29 @@ nsMathMLmunderoverFrame::UpdatePresentationDataFromChildAt(nsIPresContext* aPres
   }
   return NS_OK;
 
-  // XXX For #2, if the inner <mo> changes, is has to trigger 
-  // XXX a re-computation of all flags that depend on its state
-  // XXX in the entire embellished hierarchy
+  // For #2, changing the accent attribute will trigger a re-build of
+  // all automatic data in the embellished hierarchy
 }
 
 NS_IMETHODIMP
-nsMathMLmunderoverFrame::SetInitialChildList(nsIPresContext* aPresContext,
-                                             nsIAtom*        aListName,
-                                             nsIFrame*       aChildList)
+nsMathMLmunderoverFrame::InheritAutomaticData(nsIPresContext* aPresContext,
+                                              nsIFrame*       aParent)
 {
-  nsresult rv;
-  rv = nsMathMLContainerFrame::SetInitialChildList(aPresContext, aListName, aChildList);
+  // let the base class get the default from our parent
+  nsMathMLContainerFrame::InheritAutomaticData(aPresContext, aParent);
 
-  // check whether or not this is an embellished operator
-  EmbellishOperator();
+  mPresentationData.flags |= NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMathMLmunderoverFrame::TransmitAutomaticData(nsIPresContext* aPresContext)
+{
+  // At this stage, all our children are in sync and we can fully
+  // resolve our own mEmbellishData struct
+  //---------------------------------------------------------------------
 
-  // set our accent and accentunder flags
   /* 
   The REC says:
 
@@ -174,158 +182,100 @@ nsMathMLmunderoverFrame::SetInitialChildList(nsIPresContext* aPresContext,
   of accentunder depending on underscript.
   */
 
-  // get our overscript and underscript frames
-  PRInt32 count = 0;
-  nsIFrame* baseFrame = nsnull;
-  nsIFrame* underscriptFrame = nsnull;
   nsIFrame* overscriptFrame = nsnull;
-  nsIFrame* childFrame = mFrames.FirstChild();
-  while (childFrame) {
-    if (0 == count) baseFrame = childFrame;
-    if (1 == count) underscriptFrame = childFrame;
-    if (2 == count) { overscriptFrame = childFrame; break; }
-    count++;
-    childFrame->GetNextSibling(&childFrame);
-  }
+  nsIFrame* underscriptFrame = nsnull;
+  nsIFrame* baseFrame = mFrames.FirstChild();
+  if (baseFrame)
+    baseFrame->GetNextSibling(&underscriptFrame);
+  if (underscriptFrame)
+    underscriptFrame->GetNextSibling(&overscriptFrame);
+  if (!baseFrame || !underscriptFrame || !overscriptFrame)
+    return NS_OK; // a visual error indicator will be reported later during layout
+
+  // if our base is an embellished operator, let its state bubble to us (in particular,
+  // this is where we get the flag for NS_MATHML_EMBELLISH_MOVABLELIMITS). Our flags
+  // are reset to the default values of false if the base frame isn't embellished.
+  GetEmbellishDataFrom(baseFrame, mEmbellishData);
+  if (NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags))
+    mEmbellishData.nextFrame = baseFrame;
 
-  nsIMathMLFrame* underscriptMathMLFrame = nsnull;
-  nsIMathMLFrame* overscriptMathMLFrame = nsnull;
-  nsIMathMLFrame* mathMLFrame;
-  nsEmbellishData embellishData;
   nsAutoString value;
 
-  mPresentationData.flags &= ~NS_MATHML_MOVABLELIMITS; // default is false
-  mPresentationData.flags &= ~NS_MATHML_ACCENTUNDER; // default of accentunder is false
-  mPresentationData.flags &= ~NS_MATHML_ACCENTOVER; // default of accent is false
+  // The default value of accentunder is false, unless the underscript is embellished
+  // and its core <mo> is an accent
+  nsEmbellishData embellishData;
+  GetEmbellishDataFrom(underscriptFrame, embellishData);
+  if (NS_MATHML_EMBELLISH_IS_ACCENT(embellishData.flags))
+    mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTUNDER;
+  else
+    mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTUNDER;
 
-  // see if the baseFrame has movablelimits="true" or if it is an
-  // embellished operator whose movablelimits attribute is set to true
-  if (baseFrame && NS_MATHML_IS_EMBELLISH_OPERATOR(mEmbellishData.flags)) {
-    nsCOMPtr<nsIContent> baseContent;
-    baseFrame->GetContent(getter_AddRefs(baseContent));
-    if (NS_CONTENT_ATTR_HAS_VALUE == baseContent->GetAttr(kNameSpaceID_None, 
-                     nsMathMLAtoms::movablelimits_, value)) {
-      if (value.Equals(NS_LITERAL_STRING("true"))) {
-        mPresentationData.flags |= NS_MATHML_MOVABLELIMITS;
-      }
-    }
-    else { // no attribute, get the value from the core
-      rv = mEmbellishData.core->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-      if (NS_SUCCEEDED(rv) && mathMLFrame) {
-        mathMLFrame->GetEmbellishData(embellishData);
-        if (NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(embellishData.flags)) {
-          mPresentationData.flags |= NS_MATHML_MOVABLELIMITS;
-        }
-      }
-    }
+  // if we have an accentunder attribute, it overrides what the underscript said
+  if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
+                   nsMathMLAtoms::accentunder_, value)) {
+    if (value.Equals(NS_LITERAL_STRING("true")))
+      mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTUNDER;
+    else if (value.Equals(NS_LITERAL_STRING("false"))) 
+      mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTUNDER;
   }
 
-  // see if the underscriptFrame is <mo> or an embellished operator
-  if (underscriptFrame) {
-    rv = underscriptFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&underscriptMathMLFrame);
-    if (NS_SUCCEEDED(rv) && underscriptMathMLFrame) {
-      underscriptMathMLFrame->GetEmbellishData(embellishData);
-      // core of the underscriptFrame
-      if (NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags) && embellishData.core) {
-        rv = embellishData.core->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-        if (NS_SUCCEEDED(rv) && mathMLFrame) {
-          mathMLFrame->GetEmbellishData(embellishData);
-          // if we have the accentunder attribute, tell the core to behave as 
-          // requested (otherwise leave the core with its default behavior)
-          if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
-                          nsMathMLAtoms::accentunder_, value))
-          {
-            if (value.Equals(NS_LITERAL_STRING("true"))) embellishData.flags |= NS_MATHML_EMBELLISH_ACCENT;
-            else if (value.Equals(NS_LITERAL_STRING("false"))) embellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENT;
-            mathMLFrame->SetEmbellishData(embellishData);
-          }
+  // The default value of accent is false, unless the overscript is embellished
+  // and its core <mo> is an accent
+  GetEmbellishDataFrom(overscriptFrame, embellishData);
+  if (NS_MATHML_EMBELLISH_IS_ACCENT(embellishData.flags))
+    mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTOVER;
+  else
+    mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTOVER;
 
-          // sync the presentation data: record whether we have an accentunder
-          if (NS_MATHML_EMBELLISH_IS_ACCENT(embellishData.flags))
-            mPresentationData.flags |= NS_MATHML_ACCENTUNDER;
-        }
-      }
-    }
-  }
-  
-  // see if the overscriptFrame is <mo> or an embellished operator
-  if (overscriptFrame) {
-    rv = overscriptFrame->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&overscriptMathMLFrame);
-    if (NS_SUCCEEDED(rv) && overscriptMathMLFrame) {
-      overscriptMathMLFrame->GetEmbellishData(embellishData);
-      // core of the overscriptFrame
-      if (NS_MATHML_IS_EMBELLISH_OPERATOR(embellishData.flags) && embellishData.core) {
-        rv = embellishData.core->QueryInterface(NS_GET_IID(nsIMathMLFrame), (void**)&mathMLFrame);
-        if (NS_SUCCEEDED(rv) && mathMLFrame) {
-          mathMLFrame->GetEmbellishData(embellishData);
-          // if we have the accent attribute, tell the core to behave as 
-          // requested (otherwise leave the core with its default behavior)
-          if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
-                          nsMathMLAtoms::accent_, value))
-          {
-            if (value.Equals(NS_LITERAL_STRING("true"))) embellishData.flags |= NS_MATHML_EMBELLISH_ACCENT;
-            else if (value.Equals(NS_LITERAL_STRING("false"))) embellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENT;
-            mathMLFrame->SetEmbellishData(embellishData);
-          }
-
-          // sync the presentation data: record whether we have an accent
-          if (NS_MATHML_EMBELLISH_IS_ACCENT(embellishData.flags))
-            mPresentationData.flags |= NS_MATHML_ACCENTOVER;
-        }
-      }
-    }
+  // if we have an accent attribute, it overrides what the overscript said
+  if (NS_CONTENT_ATTR_HAS_VALUE == mContent->GetAttr(kNameSpaceID_None, 
+                   nsMathMLAtoms::accent_, value)) {
+    if (value.Equals(NS_LITERAL_STRING("true")))
+      mEmbellishData.flags |= NS_MATHML_EMBELLISH_ACCENTOVER;
+    else if (value.Equals(NS_LITERAL_STRING("false"))) 
+      mEmbellishData.flags &= ~NS_MATHML_EMBELLISH_ACCENTOVER;
   }
 
-  //The REC says:
-  /*
-  Within underscript, <munderover> always sets displaystyle to "false",
-  but increments scriptlevel by 1 only when accentunder is "false". 
+  // disable the stretch-all flag if we are going to act like a superscript
+  if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishData.flags) &&
+      !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags))
+    mPresentationData.flags &= ~NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
 
-  Within overscript, <munderover> always sets displaystyle to "false", 
-  but increments scriptlevel by 1 only when accent is "false".
-  */
+  // Now transmit any change that we want to our children so that they
+  // can update their mPresentationData structs
+  //---------------------------------------------------------------------
 
-  /*
+  /* The REC says:
+     Within underscript, <munderover> always sets displaystyle to "false",
+     but increments scriptlevel by 1 only when accentunder is "false". 
+
+     Within overscript, <munderover> always sets displaystyle to "false", 
+     but increments scriptlevel by 1 only when accent is "false".
+ 
      The TeXBook treats 'over' like a superscript, so p.141 or Rule 13a
      say it shouldn't be compressed. However, The TeXBook says
      that math accents and \overline change uncramped styles to their
      cramped counterparts.
   */
-  if (overscriptMathMLFrame) 
-  {
-    PRInt32 increment = NS_MATHML_IS_ACCENTOVER(mPresentationData.flags)
-                      ? 0 : 1;
-    PRUint32 compress = NS_MATHML_IS_ACCENTOVER(mPresentationData.flags)
-                      ? NS_MATHML_COMPRESSED : 0;
-    overscriptMathMLFrame->UpdatePresentationData(aPresContext, increment,
-      ~NS_MATHML_DISPLAYSTYLE | compress,
-       NS_MATHML_DISPLAYSTYLE | compress);
-    overscriptMathMLFrame->UpdatePresentationDataFromChildAt(aPresContext, 0, -1, increment,
-      ~NS_MATHML_DISPLAYSTYLE | compress,
-       NS_MATHML_DISPLAYSTYLE | compress);
-  }
+  PRInt32 increment = NS_MATHML_EMBELLISH_IS_ACCENTOVER(mEmbellishData.flags)
+    ? 0 : 1;
+  PRUint32 compress = NS_MATHML_EMBELLISH_IS_ACCENTOVER(mEmbellishData.flags)
+    ? NS_MATHML_COMPRESSED : 0;
+  PropagatePresentationDataFor(aPresContext, overscriptFrame, increment,
+    ~NS_MATHML_DISPLAYSTYLE | compress,
+     NS_MATHML_DISPLAYSTYLE | compress);
 
   /*
      The TeXBook treats 'under' like a subscript, so p.141 or Rule 13a 
      say it should be compressed
   */
-  if (underscriptMathMLFrame) {
-    PRInt32 increment = NS_MATHML_IS_ACCENTUNDER(mPresentationData.flags)? 0 : 1;
-    underscriptMathMLFrame->UpdatePresentationData(aPresContext, increment,
-      ~NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED,
-       NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED);
-    underscriptMathMLFrame->UpdatePresentationDataFromChildAt(aPresContext, 0, -1, increment,
-      ~NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED,
-       NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED);
-  }
+  increment = NS_MATHML_EMBELLISH_IS_ACCENTUNDER(mEmbellishData.flags)
+    ? 0 : 1;
+  PropagatePresentationDataFor(aPresContext, underscriptFrame, increment,
+    ~NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED,
+     NS_MATHML_DISPLAYSTYLE | NS_MATHML_COMPRESSED);
 
-  // disable the stretch-all flag if we are going to act like a subscript-superscript pair
-  if ( NS_MATHML_IS_MOVABLELIMITS(mPresentationData.flags) &&
-      !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
-    mEmbellishData.flags &= ~NS_MATHML_STRETCH_ALL_CHILDREN_HORIZONTALLY;
-  }
-
-  return rv;
+  return NS_OK;
 }
 
 /*
@@ -338,7 +288,7 @@ The REC says:
    used for limits on symbols such as &sum;.
 
 i.e.,:
- if ( NS_MATHML_IS_MOVABLELIMITS(mPresentationData.flags) &&
+ if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishDataflags) &&
      !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
   // place like subscript-superscript pair
  }
@@ -347,16 +297,13 @@ i.e.,:
  }
 */
 
-
 NS_IMETHODIMP
 nsMathMLmunderoverFrame::Place(nsIPresContext*      aPresContext,
                                nsIRenderingContext& aRenderingContext,
                                PRBool               aPlaceOrigin,
                                nsHTMLReflowMetrics& aDesiredSize)
 {
-  nsresult rv = NS_OK;
-
-  if ( NS_MATHML_IS_MOVABLELIMITS(mPresentationData.flags) &&
+  if ( NS_MATHML_EMBELLISH_IS_MOVABLELIMITS(mEmbellishData.flags) &&
       !NS_MATHML_IS_DISPLAYSTYLE(mPresentationData.flags)) {
     // place like sub-superscript pair
     return nsMathMLmsubsupFrame::PlaceSubSupScript(aPresContext,
@@ -369,47 +316,32 @@ nsMathMLmunderoverFrame::Place(nsIPresContext*      aPresContext,
   ////////////////////////////////////
   // Get the children's desired sizes
 
-  PRInt32 count = 0;
   nsBoundingMetrics bmBase, bmUnder, bmOver;
   nsHTMLReflowMetrics baseSize (nsnull);
   nsHTMLReflowMetrics underSize (nsnull);
   nsHTMLReflowMetrics overSize (nsnull);
-  nsIFrame* baseFrame = nsnull;
   nsIFrame* overFrame = nsnull;
   nsIFrame* underFrame = nsnull;
-
-  nsIFrame* childFrame = mFrames.FirstChild();
-  while (childFrame) {
-    if (0 == count) {
-      // base 
-      baseFrame = childFrame;
-      GetReflowAndBoundingMetricsFor(baseFrame, baseSize, bmBase);
-    }
-    else if (1 == count) {
-      // under
-      underFrame = childFrame;
-      GetReflowAndBoundingMetricsFor(underFrame, underSize, bmUnder);
-    }
-    else if (2 == count) {
-      // over
-      overFrame = childFrame;
-      GetReflowAndBoundingMetricsFor(overFrame, overSize, bmOver);
-    }
-    count++;
-    childFrame->GetNextSibling(&childFrame);
-  }
-  if (3 != count) {
+  nsIFrame* baseFrame = mFrames.FirstChild();
+  if (baseFrame)
+    baseFrame->GetNextSibling(&underFrame);
+  if (underFrame)
+    underFrame->GetNextSibling(&overFrame);
+  if (!baseFrame || !underFrame || !overFrame || HasNextSibling(overFrame)) {
     // report an error, encourage people to get their markups in order
     NS_WARNING("invalid markup");
     return ReflowError(aPresContext, aRenderingContext, aDesiredSize);
   }
+  GetReflowAndBoundingMetricsFor(baseFrame, baseSize, bmBase);
+  GetReflowAndBoundingMetricsFor(underFrame, underSize, bmUnder);
+  GetReflowAndBoundingMetricsFor(overFrame, overSize, bmOver);
 
   ////////////////////
   // Place Children
 
   const nsStyleFont* font =
     (const nsStyleFont*) mStyleContext->GetStyleData (eStyleStruct_Font);
-  aRenderingContext.SetFont(font->mFont);
+  aRenderingContext.SetFont(font->mFont, nsnull);
   nsCOMPtr<nsIFontMetrics> fm;
   aRenderingContext.GetFontMetrics(*getter_AddRefs(fm));
 
@@ -426,7 +358,7 @@ nsMathMLmunderoverFrame::Place(nsIPresContext*      aPresContext,
   nscoord underDelta1 = 0; // gap between base and underscript
   nscoord underDelta2 = 0; // extra space beneath underscript
 
-  if (!NS_MATHML_IS_ACCENTUNDER(mPresentationData.flags)) {
+  if (!NS_MATHML_EMBELLISH_IS_ACCENTUNDER(mEmbellishData.flags)) {
     // Rule 13a, App. G, TeXbook
     GetItalicCorrection (bmBase, italicCorrection);
     nscoord bigOpSpacing2, bigOpSpacing4, bigOpSpacing5, dummy; 
@@ -452,7 +384,7 @@ nsMathMLmunderoverFrame::Place(nsIPresContext*      aPresContext,
   nscoord overDelta1 = 0; // gap between base and overscript
   nscoord overDelta2 = 0; // extra space above overscript
 
-  if (!NS_MATHML_IS_ACCENTOVER(mPresentationData.flags)) {    
+  if (!NS_MATHML_EMBELLISH_IS_ACCENTOVER(mEmbellishData.flags)) {    
     // Rule 13a, App. G, TeXbook
     GetItalicCorrection (bmBase, italicCorrection);
     nscoord bigOpSpacing1, bigOpSpacing3, bigOpSpacing5, dummy; 
