@@ -123,17 +123,17 @@ These are called extraColumns/Rows.
 */
 
 nsGrid::nsGrid():mBox(nsnull),
-                 mColumns(nsnull), 
                  mRows(nsnull),
+                 mColumns(nsnull), 
                  mRowBox(nsnull),
                  mColumnBox(nsnull),
                  mNeedsRebuild(PR_TRUE),
-                 mCellMap(nsnull),
-                 mMarkingDirty(PR_FALSE),
-                 mColumnCount(0),
                  mRowCount(0),
+                 mColumnCount(0),
                  mExtraRowCount(0),
-                 mExtraColumnCount(0)
+                 mExtraColumnCount(0),
+                 mCellMap(nsnull),
+                 mMarkingDirty(PR_FALSE)
 {
     MOZ_COUNT_CTOR(nsGrid);
 }
@@ -279,7 +279,9 @@ nsGrid::FindRowsAndColumns(nsIBox** aRows, nsIBox** aColumns)
 
   // find the boxes that contain our rows and columns
   nsIBox* child = nsnull;
-  mBox->GetChildBox(&child);
+  // if we have <grid></grid> then mBox will be null (bug 125689)
+  if (mBox)
+    mBox->GetChildBox(&child);
 
   while(child)
   {
@@ -414,8 +416,10 @@ nsGrid::BuildCellMap(PRInt32 aRows, PRInt32 aColumns, nsGridCell** aCells)
 {
   PRInt32 size = aRows*aColumns;
   PRInt32 oldsize = mRowCount*mColumnCount;
-  if (size == 0)
+  if (size == 0) {
+    delete[] mCellMap;
     (*aCells) = nsnull;
+  }
   else {
     if (size > oldsize) {
       delete[] mCellMap;
