@@ -68,7 +68,6 @@
 #include "nsIStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsCOMPtr.h"
-#include "nsHTMLIIDs.h"
 #include "nsUnitConversion.h"
 #include "nsINameSpaceManager.h"
 #include "nsHTMLAtoms.h"
@@ -95,6 +94,9 @@
 #include "nsIScrollableView.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsIWidget.h"
+#include "nsIEventStateManager.h"
+#include "nsIDOMDocument.h"
+#include "nsIDOMElement.h"
 #include "nsITheme.h"
 
 // Needed for Print Preview
@@ -372,8 +374,8 @@ nsBoxFrame::Init(nsIPresContext*  aPresContext,
            GetView(aPresContext, &view);
         }
 
-        nsIWidget* widget;
-        view->GetWidget(widget);
+        nsCOMPtr<nsIWidget> widget;
+        view->GetWidget(*getter_AddRefs(widget));
 
         if (!widget)
            view->CreateWidget(kWidgetCID);   
@@ -391,6 +393,9 @@ nsBoxFrame::Init(nsIPresContext*  aPresContext,
 
   mInner->UpdateMouseThrough();
 
+  // register access key
+  rv = RegUnregAccessKey(aPresContext, PR_TRUE);
+
   return rv;
 }
 
@@ -883,12 +888,16 @@ nsBoxFrame::IsInitialReflowForPrintPreview(nsBoxLayoutState& aState,
 }
 
 NS_IMETHODIMP
-nsBoxFrame::Reflow(nsIPresContext*   aPresContext,
-                     nsHTMLReflowMetrics&     aDesiredSize,
-                     const nsHTMLReflowState& aReflowState,
-                     nsReflowStatus&          aStatus)
+nsBoxFrame::Reflow(nsIPresContext*          aPresContext,
+                   nsHTMLReflowMetrics&     aDesiredSize,
+                   const nsHTMLReflowState& aReflowState,
+                   nsReflowStatus&          aStatus)
 {
+  // If you make changes to this method, please keep nsLeafBoxFrame::Reflow
+  // in sync, if the changes are applicable there.
+
   DO_GLOBAL_REFLOW_COUNT("nsBoxFrame", aReflowState.reason);
+  DISPLAY_REFLOW(aPresContext, this, aReflowState, aDesiredSize, aStatus);
 
   NS_ASSERTION(aReflowState.mComputedWidth >=0 && aReflowState.mComputedHeight >= 0, "Computed Size < 0");
 
@@ -1010,7 +1019,7 @@ nsBoxFrame::Reflow(nsIPresContext*   aPresContext,
   aDesiredSize.width  = r.width;
   aDesiredSize.height = r.height;
   aDesiredSize.ascent = ascent;
-  aDesiredSize.descent = 0;
+  aDesiredSize.descent = r.height - ascent;
 
   // max sure the max element size reflects
   // our min width
@@ -1054,6 +1063,7 @@ nsBoxFrame::Reflow(nsIPresContext*   aPresContext,
   }
 #endif
 
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
   return NS_OK;
 }
 
@@ -1216,6 +1226,9 @@ nsBoxFrame::Destroy(nsIPresContext* aPresContext)
   if (mState & NS_STATE_IS_ROOT)
       mInner->GetDebugPref(aPresContext);
 
+  // unregister access key
+  RegUnregAccessKey(aPresContext, PR_FALSE);
+
   SetLayoutManager(nsnull);
 
   // recycle the Inner via the shell's arena.
@@ -1356,109 +1369,117 @@ nsBoxFrame::AppendFrames(nsIPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
-                               nsIContent* aChild,
-                               PRInt32 aNameSpaceID,
-                               nsIAtom* aAttribute,
-                               PRInt32 aModType, 
-                               PRInt32 aHint)
+                             nsIContent* aChild,
+                             PRInt32 aNameSpaceID,
+                             nsIAtom* aAttribute,
+                             PRInt32 aModType,
+                             PRInt32 aHint)
 {
-    nsresult rv = nsContainerFrame::AttributeChanged(aPresContext, aChild,
-                                              aNameSpaceID, aAttribute, aModType, aHint);
+  nsresult rv = nsContainerFrame::AttributeChanged(aPresContext, aChild,
+                                                   aNameSpaceID, aAttribute,
+                                                   aModType, aHint);
 
-    if (aAttribute == nsXULAtoms::mousethrough) {
-       mInner->UpdateMouseThrough();
-    }
+  if (aAttribute == nsHTMLAtoms::width       ||
+      aAttribute == nsHTMLAtoms::height      ||
+      aAttribute == nsHTMLAtoms::align       ||
+      aAttribute == nsHTMLAtoms::valign      ||
+      aAttribute == nsHTMLAtoms::left        ||
+      aAttribute == nsHTMLAtoms::top         ||
+      aAttribute == nsXULAtoms::flex         ||
+      aAttribute == nsXULAtoms::orient       ||
+      aAttribute == nsXULAtoms::pack         ||
+      aAttribute == nsXULAtoms::dir          ||
+      aAttribute == nsXULAtoms::mousethrough ||
+      aAttribute == nsXULAtoms::equalsize) {
 
-    if (aAttribute == nsXULAtoms::ordinal) {
-      nsCOMPtr<nsIPresShell> shell;
-      aPresContext->GetShell(getter_AddRefs(shell));
-      nsBoxLayoutState state(shell);
-      
-      nsIBox* parent;
-      GetParentBox(&parent);
-      parent->RelayoutChildAtOrdinal(state, this);
-      nsIFrame* parentFrame;
-      parent->GetFrame(&parentFrame);
-      nsBoxFrame* parentBoxFrame = (nsBoxFrame*) parentFrame;
-      if (parentBoxFrame)
-        parentBoxFrame->CheckFrameOrder();
-      parent->MarkDirty(state);
-    } else if (aAttribute == nsHTMLAtoms::width ||
-        aAttribute == nsHTMLAtoms::height ||
-        aAttribute == nsHTMLAtoms::align  ||
+    if (aAttribute == nsHTMLAtoms::align  ||
         aAttribute == nsHTMLAtoms::valign ||
-        aAttribute == nsHTMLAtoms::left ||
-        aAttribute == nsHTMLAtoms::top ||
-        aAttribute == nsXULAtoms::flex ||
-        aAttribute == nsXULAtoms::orient ||
-        aAttribute == nsXULAtoms::pack ||
-        aAttribute == nsXULAtoms::dir ||
-        aAttribute == nsXULAtoms::equalsize) {
+        aAttribute == nsXULAtoms::orient  ||
+        aAttribute == nsXULAtoms::pack    ||
+        aAttribute == nsXULAtoms::dir     ||
+        aAttribute == nsXULAtoms::debug) {
 
-       if (aAttribute == nsXULAtoms::orient || aAttribute == nsXULAtoms::dir || 
-           aAttribute == nsXULAtoms::debug || aAttribute == nsHTMLAtoms::align || 
-           aAttribute == nsHTMLAtoms::valign || aAttribute == nsXULAtoms::pack) {
-          mInner->mValign = nsBoxFrame::vAlign_Top;
-          mInner->mHalign = nsBoxFrame::hAlign_Left;
+      mInner->mValign = nsBoxFrame::vAlign_Top;
+      mInner->mHalign = nsBoxFrame::hAlign_Left;
 
-          PRBool orient = PR_TRUE;
-          GetInitialOrientation(orient); 
-          if (orient)
-            mState |= NS_STATE_IS_HORIZONTAL;
-          else
-            mState &= ~NS_STATE_IS_HORIZONTAL;
+      PRBool orient = PR_TRUE;
+      GetInitialOrientation(orient); 
+      if (orient)
+        mState |= NS_STATE_IS_HORIZONTAL;
+      else
+        mState &= ~NS_STATE_IS_HORIZONTAL;
 
-          PRBool normal = PR_TRUE;
-          GetInitialDirection(normal);
-          if (normal)
-            mState |= NS_STATE_IS_DIRECTION_NORMAL;
-          else
-            mState &= ~NS_STATE_IS_DIRECTION_NORMAL;
+      PRBool normal = PR_TRUE;
+      GetInitialDirection(normal);
+      if (normal)
+        mState |= NS_STATE_IS_DIRECTION_NORMAL;
+      else
+        mState &= ~NS_STATE_IS_DIRECTION_NORMAL;
 
-          GetInitialVAlignment(mInner->mValign);
-          GetInitialHAlignment(mInner->mHalign);
-  
-          PRBool equalSize = PR_FALSE;
-          GetInitialEqualSize(equalSize); 
-          if (equalSize)
-                mState |= NS_STATE_EQUAL_SIZE;
-            else
-                mState &= ~NS_STATE_EQUAL_SIZE;
- 
-          PRBool debug = mState & NS_STATE_SET_TO_DEBUG;
-          PRBool debugSet = mInner->GetInitialDebug(debug); 
-          if (debugSet) {
-                mState |= NS_STATE_DEBUG_WAS_SET;
-                if (debug)
-                    mState |= NS_STATE_SET_TO_DEBUG;
-                else
-                    mState &= ~NS_STATE_SET_TO_DEBUG;
-          } else {
-                mState &= ~NS_STATE_DEBUG_WAS_SET;
-          }
+      GetInitialVAlignment(mInner->mValign);
+      GetInitialHAlignment(mInner->mHalign);
 
+      PRBool equalSize = PR_FALSE;
+      GetInitialEqualSize(equalSize); 
+      if (equalSize)
+        mState |= NS_STATE_EQUAL_SIZE;
+      else
+        mState &= ~NS_STATE_EQUAL_SIZE;
 
-          PRBool autostretch = mState & NS_STATE_AUTO_STRETCH;
-          GetInitialAutoStretch(autostretch);
-          if (autostretch)
-                mState |= NS_STATE_AUTO_STRETCH;
-             else
-                mState &= ~NS_STATE_AUTO_STRETCH;
-       } 
+      PRBool debug = mState & NS_STATE_SET_TO_DEBUG;
+      PRBool debugSet = mInner->GetInitialDebug(debug);
+      if (debugSet) {
+        mState |= NS_STATE_DEBUG_WAS_SET;
 
-       if (aAttribute == nsHTMLAtoms::left || aAttribute == nsHTMLAtoms::top)
-         mState &= ~NS_STATE_STACK_NOT_POSITIONED;
+        if (debug)
+          mState |= NS_STATE_SET_TO_DEBUG;
+        else
+          mState &= ~NS_STATE_SET_TO_DEBUG;
+      } else {
+        mState &= ~NS_STATE_DEBUG_WAS_SET;
+      }
 
+      PRBool autostretch = mState & NS_STATE_AUTO_STRETCH;
+      GetInitialAutoStretch(autostretch);
+      if (autostretch)
+        mState |= NS_STATE_AUTO_STRETCH;
+      else
+        mState &= ~NS_STATE_AUTO_STRETCH;
     }
-          
-          
-  
+    else if (aAttribute == nsHTMLAtoms::left ||
+             aAttribute == nsHTMLAtoms::top) {
+      mState &= ~NS_STATE_STACK_NOT_POSITIONED;
+    }
+    else if (aAttribute == nsXULAtoms::mousethrough) {
+      mInner->UpdateMouseThrough();
+    }
+  }
+  else if (aAttribute == nsXULAtoms::ordinal) {
     nsCOMPtr<nsIPresShell> shell;
     aPresContext->GetShell(getter_AddRefs(shell));
-    nsBoxLayoutState state(aPresContext);
-    MarkDirty(state);
+    nsBoxLayoutState state(shell);
+    
+    nsIBox* parent;
+    GetParentBox(&parent);
+    parent->RelayoutChildAtOrdinal(state, this);
+    nsIFrame* parentFrame;
+    parent->GetFrame(&parentFrame);
+    nsBoxFrame* parentBoxFrame = (nsBoxFrame*) parentFrame;
+    if (parentBoxFrame)
+      parentBoxFrame->CheckFrameOrder();
+    parent->MarkDirty(state);
+  }
+  // If the accesskey changed, register for the new value
+  // The old value has been unregistered in nsXULElement::SetAttr
+  else if (aAttribute == nsXULAtoms::accesskey) {
+    RegUnregAccessKey(aPresContext, PR_TRUE);
+  }
+
+  nsCOMPtr<nsIPresShell> shell;
+  aPresContext->GetShell(getter_AddRefs(shell));
+  nsBoxLayoutState state(aPresContext);
+  MarkDirty(state);
 
-  
   return rv;
 }
 
@@ -1977,7 +1998,8 @@ nsBoxFrame::GetFrameForPoint(nsIPresContext* aPresContext,
     return NS_ERROR_FAILURE;
 
   nsIView* view = nsnull;
-  GetView(aPresContext, &view);
+  nsPoint originOffset;
+  GetOriginToViewOffset(aPresContext, originOffset, &view);
 
   // get the debug frame.
   if (view || (mState & NS_STATE_IS_ROOT))
@@ -2002,6 +2024,10 @@ nsBoxFrame::GetFrameForPoint(nsIPresContext* aPresContext,
   FirstChild(aPresContext, nsnull, &kid);
   *aFrame = nsnull;
   tmp.MoveTo(aPoint.x - mRect.x, aPoint.y - mRect.y);
+
+  if (view)
+    tmp += originOffset;
+
   while (nsnull != kid) {
     // have we hit a child before
     PRBool haveKid = (hit != nsnull);
@@ -2793,4 +2819,52 @@ nsBoxFrame::CreateViewForFrame(nsIPresContext* aPresContext,
   return NS_OK;
 }
 
+// If you make changes to this function, check its counterparts
+// in nsTextBoxFrame and nsAreaFrame
+nsresult
+nsBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext,
+                              PRBool aDoReg)
+{
+  // if we have no content, we can't do anything
+  if (!mContent)
+    return NS_ERROR_FAILURE;
+
+  // find out what type of element this is
+  nsCOMPtr<nsIAtom> atom;
+  nsresult rv = mContent->GetTag(*getter_AddRefs(atom));
+  if (NS_FAILED(rv))
+    return rv;
+
+  // only support accesskeys for the following elements
+  if (atom != nsXULAtoms::button &&
+      atom != nsXULAtoms::checkbox &&
+      atom != nsXULAtoms::radio) {
+
+    return NS_OK;
+  }
+
+  nsAutoString accessKey;
+  mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, accessKey);
+
+  if (accessKey.IsEmpty())
+    return NS_OK;
+
+  // With a valid PresContext we can get the ESM 
+  // and register the access key
+  nsCOMPtr<nsIEventStateManager> esm;
+  aPresContext->GetEventStateManager(getter_AddRefs(esm));
+
+  rv = NS_OK;
+
+  if (esm) {
+    PRUint32 key = accessKey.First();
+    if (aDoReg)
+      rv = esm->RegisterAccessKey(mContent, key);
+    else
+      rv = esm->UnregisterAccessKey(mContent, key);
+  }
+
+  return rv;
+}
+
 
