@@ -60,11 +60,10 @@
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
 #include "nsIPresShell.h"
-#include "nsHTMLIIDs.h"
 #include "nsIImage.h"
 #include "nsIWidget.h"
 #include "nsHTMLAtoms.h"
-#include "nsIHTMLAttributes.h"
+#include "nsIHTMLContent.h"
 #include "nsIDocument.h"
 #include "nsIHTMLDocument.h"
 #include "nsIStyleContext.h"
@@ -427,7 +426,7 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
   nsCOMPtr<nsIURI> baseURI;
   GetBaseURI(getter_AddRefs(baseURI));
   nsCOMPtr<nsIURI> srcURI;
-  NS_NewURI(getter_AddRefs(srcURI), mSrc, baseURI);
+  NS_NewURI(getter_AddRefs(srcURI), mSrc, nsnull, baseURI);
 
   if (mImageRequest) {
     nsCOMPtr<nsIURI> requestURI;
@@ -457,7 +456,7 @@ nsImageBoxFrame::UpdateImage(nsIPresContext*  aPresContext, PRBool& aResize)
   nsCOMPtr<nsILoadGroup> loadGroup;
   GetLoadGroup(aPresContext, getter_AddRefs(loadGroup));
 
-  il->LoadImage(srcURI, loadGroup, mListener, aPresContext, mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
+  il->LoadImage(srcURI, nsnull, loadGroup, mListener, aPresContext, mLoadFlags, nsnull, nsnull, getter_AddRefs(mImageRequest));
 
   aResize = PR_TRUE;
 }
@@ -558,6 +557,13 @@ nsImageBoxFrame::DidSetStyleContext( nsIPresContext* aPresContext )
   if (mUseSrcAttr || mSuppressStyleCheck)
     return NS_OK; // No more work required, since the image isn't specified by style.
 
+  // If we're using a native theme implementation, we shouldn't draw anything.
+  const nsStyleDisplay* disp =
+    (const nsStyleDisplay*)mStyleContext->GetStyleData(eStyleStruct_Display);
+  if (disp->mAppearance && nsBox::gTheme && 
+      nsBox::gTheme->ThemeSupportsWidget(nsnull, disp->mAppearance))
+    return NS_OK;
+
   // If list-style-image changes, we have a new image.
   nsAutoString newSrc;
   if (myList->mListStyleImage.Equals(mSrc))
@@ -633,6 +639,13 @@ nsImageBoxFrame::GetPrefSize(nsBoxLayoutState& aState, nsSize& aSize)
   AddInset(aSize);
   nsIBox::AddCSSPrefSize(aState, this, aSize);
 
+  nsSize minSize(0,0);
+  nsSize maxSize(0,0);
+  GetMinSize(aState, minSize);
+  GetMaxSize(aState, maxSize);
+
+  BoundsCheck(minSize, aSize, maxSize);
+
   return NS_OK;
 }
 
