@@ -399,10 +399,16 @@ nsMenuFrame::HandleEvent(nsIPresContext* aPresContext,
   if (aEvent->message == NS_KEY_PRESS && !IsDisabled()) {
     nsKeyEvent* keyEvent = (nsKeyEvent*)aEvent;
     PRUint32 keyCode = keyEvent->keyCode;
-    if (keyCode == NS_VK_UP || keyCode == NS_VK_DOWN) {
-      if (!IsOpen()) 
+    if ((keyCode == NS_VK_F4 && !mMenuParent) && IsOpen() &&
+      !keyEvent->isAlt && !keyEvent->isShift && !keyEvent->isControl) 
+      OpenMenu(PR_FALSE); // Close menu on unmodified F4
+    else if (keyCode == NS_VK_UP || keyCode == NS_VK_DOWN || 
+       (keyCode == NS_VK_F4 && !keyEvent->isAlt && !keyEvent->isShift && 
+       !keyEvent->isControl && !mMenuParent)) 
+      // Plain or modified down or up arrow will open any menu
+      // Unmodified F4 will open <menulist> as well
+      if (!IsOpen())
         OpenMenu(PR_TRUE);
-    }
   }
   else if (aEvent->message == NS_MOUSE_LEFT_BUTTON_DOWN && !IsDisabled() && IsMenu() ) {
     PRBool isMenuBar = PR_FALSE;
@@ -550,7 +556,7 @@ nsMenuFrame::ToggleMenuState()
 }
 
 void
-nsMenuFrame::FireMenuDOMEvent(nsAReadableString& aDOMEventName)
+nsMenuFrame::FireMenuDOMEvent(const nsAString& aDOMEventName)
 {
   // Fire a DOM event for the title change.
   nsCOMPtr<nsIDOMEvent> event;
@@ -685,9 +691,11 @@ nsMenuFrame::ActivateMenu(PRBool aActivateFlag)
     if (view) {
       nsCOMPtr<nsIViewManager> viewManager;
       view->GetViewManager(*getter_AddRefs(viewManager));
-      viewManager->SetViewVisibility(view, nsViewVisibility_kHide);
-      nsRect r(0, 0, 0, 0);
-      viewManager->ResizeView(view, r);
+      if (viewManager) { // the view manager can be null during widget teardown
+        viewManager->SetViewVisibility(view, nsViewVisibility_kHide);
+        nsRect r(0, 0, 0, 0);
+        viewManager->ResizeView(view, r);
+      }
     }
     // set here so hide chain can close the menu as well.
     mMenuOpen = PR_FALSE;
@@ -915,6 +923,12 @@ nsMenuFrame::OpenMenuInternal(PRBool aActivateFlag)
 void
 nsMenuFrame::GetMenuChildrenElement(nsIContent** aResult)
 {
+  if (!mContent)
+  {
+    *aResult = nsnull;
+    return;
+  }
+  
   nsresult rv;
   nsCOMPtr<nsIXBLService> xblService = 
            do_GetService("@mozilla.org/xbl;1", &rv);
@@ -935,6 +949,45 @@ nsMenuFrame::GetMenuChildrenElement(nsIContent** aResult)
   }
 }
 
+PRBool
+nsMenuFrame::IsSizedToPopup(nsIContent* aContent, PRBool aRequireAlways)
+{
+  nsCOMPtr<nsIAtom> tag;
+  aContent->GetTag(*getter_AddRefs(tag));
+  PRBool sizeToPopup;
+  if (tag == nsHTMLAtoms::select)
+    sizeToPopup = PR_TRUE;
+  else {
+    nsAutoString sizedToPopup;
+    aContent->GetAttr(kNameSpaceID_None, nsXULAtoms::sizetopopup, sizedToPopup);
+    sizeToPopup = (sizedToPopup.EqualsIgnoreCase("always") ||
+                   (!aRequireAlways && sizedToPopup.EqualsIgnoreCase("pref")));
+  }
+  
+  return sizeToPopup;
+}
+
+NS_IMETHODIMP
+nsMenuFrame::GetMinSize(nsBoxLayoutState& aBoxLayoutState, nsSize& aSize)
+{
+  // Our min size is the popup size (same as the pref size) if
+  // sizetopopup="always" is set.  However, we first need to check
+  // to see if a min size was set in CSS.
+  PRBool collapsed = PR_FALSE;
+  IsCollapsed(aBoxLayoutState, collapsed);
+  if (collapsed) {
+    aSize.width = aSize.height = 0;
+    return NS_OK;
+  }
+
+  nsIFrame* popupChild = mPopupFrames.FirstChild();
+
+  if (popupChild && IsSizedToPopup(mContent, PR_TRUE))
+    return GetPrefSize(aBoxLayoutState, aSize);
+
+  return nsBoxFrame::GetMinSize(aBoxLayoutState, aSize);
+}
+
 NS_IMETHODIMP
 nsMenuFrame::DoLayout(nsBoxLayoutState& aState)
 {
@@ -948,10 +1001,7 @@ nsMenuFrame::DoLayout(nsBoxLayoutState& aState)
   nsIFrame* popupChild = mPopupFrames.FirstChild();
 
   if (popupChild) {
-    nsAutoString sizedToPopup;
-    mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::sizetopopup, sizedToPopup);
-    PRBool sizeToPopup = (sizedToPopup.EqualsIgnoreCase("true"));
-    
+    PRBool sizeToPopup = IsSizedToPopup(mContent, PR_FALSE);
     nsIBox* ibox = nsnull;
     nsresult rv2 = popupChild->QueryInterface(NS_GET_IID(nsIBox), (void**)&ibox);
     NS_ASSERTION(NS_SUCCEEDED(rv2) && ibox,"popupChild is not box!!");
@@ -1170,12 +1220,12 @@ nsMenuFrame::RePositionPopup(nsBoxLayoutState& aState)
 }
 
 NS_IMETHODIMP
-nsMenuFrame::ShortcutNavigation(PRUint32 aLetter, PRBool& aHandledFlag)
+nsMenuFrame::ShortcutNavigation(nsIDOMKeyEvent* aKeyEvent, PRBool& aHandledFlag)
 {
   nsIFrame* frame = mPopupFrames.FirstChild();
   if (frame) {
     nsMenuPopupFrame* popup = (nsMenuPopupFrame*)frame;
-    popup->ShortcutNavigation(aLetter, aHandledFlag);
+    popup->ShortcutNavigation(aKeyEvent, aHandledFlag);
   } 
 
   return NS_OK;
@@ -1217,8 +1267,15 @@ nsMenuFrame::Escape(PRBool& aHandledFlag)
 NS_IMETHODIMP
 nsMenuFrame::Enter()
 {
-  if ( IsDisabled() )
+  if (IsDisabled()) {
+#ifdef XP_WIN
+    // behavior on Windows - close the popup chain
+    if (mMenuParent)
+      mMenuParent->DismissChain();
+#endif   // #ifdef XP_WIN
+    // this menu item was disabled - exit
     return NS_OK;
+  }
     
   if (!mMenuOpen) {
     // The enter key press applies to us.
@@ -1714,10 +1771,7 @@ nsMenuFrame::OnCreate()
               else grandChild->SetAttr(kNameSpaceID_None, nsHTMLAtoms::disabled, commandAttr, PR_TRUE);
             }
 
-            commandAttr.Assign(NS_LITERAL_STRING(""));
-            menuAttr.Assign(NS_LITERAL_STRING(""));
-
-            // The menu's label and checked states need to be updated to match the command.
+            // The menu's label, accesskey, and checked states need to be updated to match the command.
             // Note that (unlike the disabled state) if the command has *no* label for either, we
             // assume the menu is supplying its own.
             commandContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::checked, commandAttr);
@@ -1726,9 +1780,13 @@ nsMenuFrame::OnCreate()
               if (!commandAttr.IsEmpty()) 
                 grandChild->SetAttr(kNameSpaceID_None, nsHTMLAtoms::checked, commandAttr, PR_TRUE);
             }
-            
-            commandAttr.Assign(NS_LITERAL_STRING(""));
-            menuAttr.Assign(NS_LITERAL_STRING(""));
+
+            commandContent->GetAttr(kNameSpaceID_None, nsHTMLAtoms::accesskey, commandAttr);
+            grandChild->GetAttr(kNameSpaceID_None, nsHTMLAtoms::accesskey, menuAttr);
+            if (!commandAttr.Equals(menuAttr)) {
+              if (!commandAttr.IsEmpty()) 
+                grandChild->SetAttr(kNameSpaceID_None, nsHTMLAtoms::accesskey, commandAttr, PR_TRUE);
+            }
 
             commandContent->GetAttr(kNameSpaceID_None, nsXULAtoms::label, commandAttr);
             grandChild->GetAttr(kNameSpaceID_None, nsXULAtoms::label, menuAttr);
@@ -1945,11 +2003,7 @@ nsMenuFrame::GetPrefSize(nsBoxLayoutState& aState, nsSize& aSize)
   aSize.height = 0;
   nsresult rv = nsBoxFrame::GetPrefSize(aState, aSize);
 
-  nsAutoString sizedToPopup;
-  mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::sizetopopup, sizedToPopup);
-  PRBool sizeToPopup = (sizedToPopup.EqualsIgnoreCase("true"));
-
-  if (sizeToPopup) {
+  if (IsSizedToPopup(mContent, PR_FALSE)) {
     nsSize tmpSize(-1,0);
     nsIBox::AddCSSPrefSize(aState, this, tmpSize);
     nscoord flex;
@@ -1970,6 +2024,20 @@ nsMenuFrame::GetPrefSize(nsBoxLayoutState& aState, nsSize& aSize)
 
       ibox->GetPrefSize(aState, tmpSize);
       aSize.width = tmpSize.width;
+
+      // We now need to ensure that aSize is within the min size - max size range.
+      // If we are using sizetopopup="always", we know the min size will be the same
+      // as the pref size, and do not need to call GetMinSize (in fact, doing so will
+      // result in infinite recursion).
+
+      nsSize minSize, maxSize;
+      if (IsSizedToPopup(mContent, PR_TRUE))
+        minSize = aSize;
+      else
+        GetMinSize(aState, minSize);
+
+      GetMaxSize(aState, maxSize);
+      BoundsCheck(minSize, aSize, maxSize);
     }
   }
 
