@@ -357,7 +357,7 @@ nsSplitterFrame::Init(nsIPresContext*  aPresContext,
         aContent->SetAttr(kNameSpaceID_None, nsXULAtoms::orient,
                           NS_LITERAL_STRING("vertical"), PR_FALSE);
         nsCOMPtr<nsIStyleContext> parent = dont_AddRef(aContext->GetParent());
-        aPresContext->ResolveStyleContextFor(aContent, parent, PR_FALSE,
+        aPresContext->ResolveStyleContextFor(aContent, parent,
                                              getter_AddRefs(newContext));
         aContext = newContext;
       }
@@ -753,6 +753,9 @@ nsresult
 nsSplitterFrameInner::MouseUp(nsIDOMEvent* aMouseEvent)
 {  
   mPressed = PR_FALSE;
+
+  mOuter->CaptureMouse(mOuter->mPresContext, PR_FALSE);
+
   return NS_OK;
 }
 
@@ -856,11 +859,15 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
         childBox->GetBounds(r);
         r.Inflate(margin);
 
-        nsAutoString fixed;
+        nsAutoString fixed, hidden;
         content->GetAttr(kNameSpaceID_None, nsXULAtoms::fixed, fixed);
+        // We need to check for hidden attribute too, since treecols with
+        // the hidden="true" attribute are not really hidden, just collapsed
+        content->GetAttr(kNameSpaceID_None, nsHTMLAtoms::hidden, hidden);
 
-        if (count < childIndex) {
-            if (!fixed.Equals(NS_LITERAL_STRING("true"))) {              
+        NS_NAMED_LITERAL_STRING(attrTrue, "true");
+        if (!attrTrue.Equals(fixed) && !attrTrue.Equals(hidden)) {
+            if (count < childIndex) {
                 mChildInfosBefore[mChildInfosBeforeCount].child   = childBox;
                 mChildInfosBefore[mChildInfosBeforeCount].min     = isHorizontal ? minSize.width : minSize.height;
                 mChildInfosBefore[mChildInfosBeforeCount].max     = isHorizontal ? maxSize.width : maxSize.height;
@@ -869,16 +876,14 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
                 mChildInfosBefore[mChildInfosBeforeCount].index   = count;
                 mChildInfosBefore[mChildInfosBeforeCount].changed = mChildInfosBefore[mChildInfosBeforeCount].current;
                 mChildInfosBeforeCount++;
-            }
-        } else if (count > childIndex) {
-            if (!fixed.Equals(NS_LITERAL_STRING("true"))) {   
+            } else if (count > childIndex) {
                 mChildInfosAfter[mChildInfosAfterCount].child   = childBox;
                 mChildInfosAfter[mChildInfosAfterCount].min     = isHorizontal ? minSize.width : minSize.height;
                 mChildInfosAfter[mChildInfosAfterCount].max     = isHorizontal ? maxSize.width : maxSize.height;
                 mChildInfosAfter[mChildInfosAfterCount].current = isHorizontal ? r.width : r.height;
                 mChildInfosAfter[mChildInfosAfterCount].flex    = flex;
                 mChildInfosAfter[mChildInfosAfterCount].index   = count;
-                mChildInfosAfter[mChildInfosAfterCount].changed   = mChildInfosAfter[mChildInfosAfterCount].current;
+                mChildInfosAfter[mChildInfosAfterCount].changed = mChildInfosAfter[mChildInfosAfterCount].current;
                 mChildInfosAfterCount++;
             }
         } 
@@ -941,6 +946,8 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
     
   //printf("Pressed mDragStartPx=%d\n",mDragStartPx);
 
+  mOuter->CaptureMouse(mOuter->mPresContext, PR_TRUE);
+
   return NS_OK;
 }
 
@@ -958,7 +965,6 @@ nsSplitterFrameInner::MouseMove(nsIDOMEvent* aMouseEvent)
   mOuter->mContent->SetAttr(kNameSpaceID_None, nsXULAtoms::state, NS_LITERAL_STRING("dragging"), PR_TRUE);
 
   RemoveListener();
-  mOuter->CaptureMouse(mOuter->mPresContext, PR_TRUE);
   mDragging = PR_TRUE;
 
   return NS_OK;
@@ -1262,8 +1268,6 @@ nsSplitterFrameInner::AddRemoveSpace(nscoord aDiff,
   aSpaceLeft = aDiff;
 }
 
-#define PR_ABS(x) (x < 0 ? -x : x)
-
 /**
  * Ok if we want to resize a child we will know the actual size in pixels we want it to be.
  * This is not the preferred size. But they only way we can change a child is my manipulating its
