@@ -44,6 +44,7 @@
 // See documentation in associated header file
 //
 
+#include "nsReadableUtils.h"
 #include "nsTextBoxFrame.h"
 #include "nsCOMPtr.h"
 #include "nsIDeviceContext.h"
@@ -60,14 +61,18 @@
 #include "nsIPref.h"
 #include "nsXPIDLString.h"
 #include "nsIServiceManager.h"
+#include "nsIDocument.h"
+#include "nsIDOMDocument.h"
+#include "nsIDOMElement.h"
+#include "nsIEventStateManager.h"
 #include "nsITheme.h"
 #include "nsUnicharUtils.h"
 
 #ifdef IBMBIDI
-#include "nsIUBidiUtils.h"
+#include "nsBidiUtils.h"
 #include "nsBidiPresUtils.h"
-#include "nsReadableUtils.h"
 #endif // IBMBIDI
+#include "nsReadableUtils.h"
 
 #define ELLIPSIS "..."
 
@@ -135,6 +140,11 @@ nsTextBoxFrame::AttributeChanged(nsIPresContext* aPresContext,
         Redraw(state);
     }
 
+    // If the accesskey changed, register for the new value
+    // The old value has been unregistered in nsXULElement::SetAttr
+    if (aAttribute == nsXULAtoms::accesskey || aAttribute == nsXULAtoms::control)
+        RegUnregAccessKey(aPresContext, PR_TRUE);
+
     return NS_OK;
 }
 
@@ -158,14 +168,28 @@ nsTextBoxFrame::Init(nsIPresContext*  aPresContext,
                      nsIStyleContext* aContext,
                      nsIFrame*        aPrevInFlow)
 {
-    nsresult  rv = nsLeafBoxFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+    nsresult rv = nsTextBoxFrameSuper::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
+    if (NS_FAILED(rv))
+        return rv;
 
     mState |= NS_STATE_NEED_LAYOUT;
     PRBool aResize;
     PRBool aRedraw;
     UpdateAttributes(aPresContext, nsnull, aResize, aRedraw); /* update all */
 
-    return rv;
+    // register access key
+    RegUnregAccessKey(aPresContext, PR_TRUE);
+
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsTextBoxFrame::Destroy(nsIPresContext* aPresContext)
+{
+    // unregister access key
+    RegUnregAccessKey(aPresContext, PR_FALSE);
+
+    return nsTextBoxFrameSuper::Destroy(aPresContext);
 }
 
 PRBool
@@ -261,6 +285,7 @@ nsTextBoxFrame::UpdateAttributes(nsIPresContext*  aPresContext,
         UpdateAccessTitle();
         aResize = PR_TRUE;
     }
+
 }
 
 NS_IMETHODIMP
@@ -400,7 +425,7 @@ nsTextBoxFrame::PaintTitle(nsIPresContext*      aPresContext,
       aRenderingContext.FillRect(textRect.x, textRect.y + baseline - offset, mRect.width, size);
     }
  
-    aRenderingContext.SetFont(fontStyle->mFont);
+    aRenderingContext.SetFont(fontStyle->mFont, nsnull);
 
     CalculateUnderline(aRenderingContext);
 
@@ -685,7 +710,7 @@ nsTextBoxFrame::UpdateAccessTitle()
     nsMenuBarListener::GetMenuAccessKey(&menuAccessKey);
     if (menuAccessKey) {
         if (!mAccessKey.IsEmpty()) {
-            if ((mTitle.Find(mAccessKey, PR_TRUE) == kNotFound) 
+            if (( !FindInReadable(mAccessKey, mTitle, nsCaseInsensitiveStringComparator()))
                 || AlwaysAppendAccessKey()) 
             {
                 nsAutoString tmpstring(NS_LITERAL_STRING("("));
@@ -693,10 +718,16 @@ nsTextBoxFrame::UpdateAccessTitle()
                 ToUpperCase(tmpstring);
                 tmpstring.Append(NS_LITERAL_STRING(")"));
                 PRInt32 offset = mTitle.RFind("...");
-                if (offset != kNotFound)
+                if (offset != kNotFound) {
                     mTitle.Insert(tmpstring,NS_STATIC_CAST(PRUint32, offset));
-                else
-                    mTitle += tmpstring;
+                } else {
+                    PRUint32 l = mTitle.Length();
+                    if((l > 0) && (PRUnichar(':')==mTitle[l-1])) {
+                      mTitle.Insert(tmpstring,l-1);
+                    } else {
+                      mTitle += tmpstring;
+                    }
+                }
             }
         }
     }
@@ -717,17 +748,34 @@ nsTextBoxFrame::UpdateAccessIndex()
             if (!mAccessKeyInfo)
                 mAccessKeyInfo = new nsAccessKeyInfo();
 
+            nsAString::const_iterator start, end;
+                
+            mCroppedTitle.BeginReading(start);
+            mCroppedTitle.EndReading(end);
+            
+            // remember the beginning of the string
+            nsAString::const_iterator originalStart = start;
+
+            PRBool found;
             if (!AlwaysAppendAccessKey()) {
-                // not appending access key - do case-sensitive search first
-                mAccessKeyInfo->mAccesskeyIndex = mCroppedTitle.Find(mAccessKey, PR_FALSE);
-                if (mAccessKeyInfo->mAccesskeyIndex == kNotFound) {
+                // not appending access key - do case-sensitive search
+                // first
+                found = FindInReadable(mAccessKey, start, end);
+                if (!found) {
                     // didn't find it - perform a case-insensitive search
-                    mAccessKeyInfo->mAccesskeyIndex = mCroppedTitle.Find(mAccessKey, PR_TRUE);
+                    start = originalStart;
+                    found = FindInReadable(mAccessKey, start, end,
+                                           nsCaseInsensitiveStringComparator());
                 }
             } else {
-                // use case-insensitive, reverse find for appended access keys
-                mAccessKeyInfo->mAccesskeyIndex = mCroppedTitle.RFind(mAccessKey, PR_TRUE);
+                found = RFindInReadable(mAccessKey, start, end,
+                                        nsCaseInsensitiveStringComparator());
             }
+            
+            if (found)
+                mAccessKeyInfo->mAccesskeyIndex = Distance(originalStart, start);
+            else
+                mAccessKeyInfo->mAccesskeyIndex = kNotFound;
         }
     }
 }
@@ -850,3 +898,49 @@ nsTextBoxFrame::GetFrameName(nsAString& aResult) const
     return NS_OK;
 }
 #endif
+
+// If you make changes to this function, check its counterparts 
+// in nsBoxFrame and nsAreaFrame
+nsresult
+nsTextBoxFrame::RegUnregAccessKey(nsIPresContext* aPresContext,
+                                  PRBool          aDoReg)
+{
+    // if we have no content, we can't do anything
+    if (!mContent)
+        return NS_ERROR_FAILURE;
+
+    // check if we have a |control| attribute
+    // do this check first because few elements have control attributes, and we
+    // can weed out most of the elements quickly.
+
+    // XXXjag a side-effect is that we filter out anonymous <label>s
+    // in e.g. <menu>, <menuitem>, <button>. These <label>s inherit
+    // |accesskey| and would otherwise register themselves, overwriting
+    // the content we really meant to be registered.
+    if (!mContent->HasAttr(kNameSpaceID_None, nsXULAtoms::control))
+        return NS_OK;
+
+    // see if we even have an access key
+    nsAutoString accessKey;
+    mContent->GetAttr(kNameSpaceID_None, nsXULAtoms::accesskey, accessKey);
+
+    if (accessKey.IsEmpty())
+        return NS_OK;
+
+    nsresult rv = NS_OK;
+
+    // With a valid PresContext we can get the ESM 
+    // and (un)register the access key
+    nsCOMPtr<nsIEventStateManager> esm;
+    aPresContext->GetEventStateManager(getter_AddRefs(esm));
+
+    if (esm) {
+        PRUint32 key = accessKey.First();
+        if (aDoReg)
+            rv = esm->RegisterAccessKey(mContent, key);
+        else
+            rv = esm->UnregisterAccessKey(mContent, key);
+    }
+
+    return rv;
+}
