@@ -35,7 +35,6 @@
 #include "nsIMsgHeaderParser.h"
 #include "nsILDAPMessage.h"
 #include "nsLDAP.h"
-#include "prlog.h"
 #include "nsReadableUtils.h"
 #include "nsIStringBundle.h"
 #include "nsISupportsPrimitives.h"
@@ -186,12 +185,12 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
     nsCOMPtr<nsIStringBundle> ldapBundle, ldapACBundle;
 
     rv = stringBundleSvc->CreateBundle(
-        "chrome://mozldap/locale/ldapErrors.properties",
+        "chrome://mozldap/locale/ldap.properties",
         getter_AddRefs(ldapBundle));
     if (NS_FAILED(rv)) {
         NS_ERROR("nsAbLDAPAutoCompleteFormatter::FormatException():"
                  " error creating string bundle"
-                 " chrome://mozldap/locale/ldapErrors.properties");
+                 " chrome://mozldap/locale/ldap.properties");
         return rv;
     } 
 
@@ -223,7 +222,7 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
     if (NS_FAILED(rv)) {
         NS_ERROR("nsAbLDAPAutoCompleteFormatter::FormatException"
                    "(): error getting 'errCode' string from bundle "
-                   "chrome://mozldap/locale/ldapErrors.properties");
+                   "chrome://mozldap/locale/ldap.properties");
         return rv;
     }
 
@@ -244,7 +243,7 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
         if (NS_FAILED(rv)) {
             NS_ERROR("nsAbLDAPAutoCompleteFormatter::FormatException"
                      "(): error getting string 2 from bundle "
-                     "chrome://mozldap/locale/ldapErrors.properties");
+                     "chrome://mozldap/locale/ldap.properties");
             return rv;
         }
   
@@ -269,7 +268,7 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
             NS_ERROR("nsAbLDAPAutoCompleteFormatter::FormatException"
                      "(): error getting specific non LDAP error-string "
                      "from bundle "
-                     "chrome://mozldap/locale/ldapErrors.properties");
+                     "chrome://mozldap/locale/ldap.properties");
             return rv;
         }
     }
@@ -286,7 +285,7 @@ nsAbLDAPAutoCompFormatter::FormatException(PRInt32 aState,
         if (NS_FAILED(rv)) {
             NS_ERROR("nsAbLDAPAutoCompleteFormatter::FormatException()"
                      "(): error getting hint string from bundle "
-                     "chrome://mozldap/locale/ldapErrors.properties");
+                     "chrome://mozldap/locale/ldap.properties");
             return rv;
         }
     }
@@ -433,9 +432,9 @@ nsAbLDAPAutoCompFormatter::GetAttributes(PRUint32 *aCount, char ** *aAttrs)
 // generate an nsIAutoCompleteItem from this.
 //
 nsresult
-nsAbLDAPAutoCompFormatter::ProcessFormat(const nsAReadableString & aFormat,
+nsAbLDAPAutoCompFormatter::ProcessFormat(const nsAString & aFormat,
                                          nsILDAPMessage *aMessage, 
-                                         nsAWritableCString *aValue,
+                                         nsACString *aValue,
                                          nsCStringArray *aAttrs)
 {
     nsresult rv;    // temp for return values
@@ -571,7 +570,7 @@ nsAbLDAPAutoCompFormatter::ParseAttrName(
     nsReadingIterator<PRUnichar> &aIterEnd, 
     PRBool aAttrRequired,                       // required?  or just optional?
     nsCOMPtr<nsIConsoleService> &aConsoleSvc,   // no need to reacquire this
-    nsAWritableCString &aAttrName)              // attribute token
+    nsACString &aAttrName)                      // attribute token
 {
     // reset attrname, and move past the opening brace
     //
@@ -621,10 +620,10 @@ nsAbLDAPAutoCompFormatter::ParseAttrName(
 
 nsresult
 nsAbLDAPAutoCompFormatter::AppendFirstAttrValue(
-    nsAReadableCString &aAttrName, // attr to get
+    const nsACString &aAttrName, // attr to get
     nsILDAPMessage *aMessage, // msg to get values from
     PRBool aAttrRequired, // is this a required value?
-    nsAWritableCString &aValue)
+    nsACString &aValue)
 {
     // get the attribute values for the field which will be used 
     // to fill in nsIAutoCompleteItem::value
@@ -684,13 +683,13 @@ nsAbLDAPAutoCompFormatter::AppendFirstAttrValue(
 
 // attribute AString nameFormat;
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::GetNameFormat(nsAWritableString & aNameFormat)
+nsAbLDAPAutoCompFormatter::GetNameFormat(nsAString & aNameFormat)
 {
     aNameFormat  = mNameFormat;
     return NS_OK;
 }
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::SetNameFormat(const nsAReadableString & aNameFormat)
+nsAbLDAPAutoCompFormatter::SetNameFormat(const nsAString & aNameFormat)
 {
     mNameFormat = aNameFormat; 
     return NS_OK;
@@ -698,13 +697,13 @@ nsAbLDAPAutoCompFormatter::SetNameFormat(const nsAReadableString & aNameFormat)
 
 // attribute AString addressFormat;
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::GetAddressFormat(nsAWritableString & aAddressFormat)
+nsAbLDAPAutoCompFormatter::GetAddressFormat(nsAString & aAddressFormat)
 {
     aAddressFormat  = mAddressFormat;
     return NS_OK;
 }
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::SetAddressFormat(const nsAReadableString & 
+nsAbLDAPAutoCompFormatter::SetAddressFormat(const nsAString & 
                                             aAddressFormat)
 {
     mAddressFormat = aAddressFormat; 
@@ -713,13 +712,13 @@ nsAbLDAPAutoCompFormatter::SetAddressFormat(const nsAReadableString &
 
 // attribute AString commentFormat;
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::GetCommentFormat(nsAWritableString & aCommentFormat)
+nsAbLDAPAutoCompFormatter::GetCommentFormat(nsAString & aCommentFormat)
 {
     aCommentFormat  = mCommentFormat;
     return NS_OK;
 }
 NS_IMETHODIMP 
-nsAbLDAPAutoCompFormatter::SetCommentFormat(const nsAReadableString & 
+nsAbLDAPAutoCompFormatter::SetCommentFormat(const nsAString & 
                                             aCommentFormat)
 {
     mCommentFormat = aCommentFormat; 
