@@ -59,9 +59,7 @@
 #include "nsAbMDBCardProperty.h"
 
 #include "mdb.h"
-#include "prlog.h"
 #include "prprf.h"
-#include "prmem.h"
 
 // XXX todo
 // fix this -1,0,1 crap, use an enum or #define
@@ -102,30 +100,6 @@ NS_IMPL_ISUPPORTS_INHERITED4(nsAbMDBDirectory, nsRDFResource,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-
-
-// Not called
-nsresult nsAbMDBDirectory::AddMailList(const char *uriName)
-{
-  if (!uriName)
-    return NS_ERROR_NULL_POINTER;
-
-  nsresult rv = NS_OK;
-  nsCOMPtr<nsIRDFService> rdf(do_GetService("@mozilla.org/rdf/rdf-service;1", &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-  
-  nsCOMPtr<nsIRDFResource> res;
-  rv = rdf->GetResource(uriName, getter_AddRefs(res));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  nsCOMPtr<nsIAbDirectory> directory(do_QueryInterface(res, &rv));
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  mSubDirectories->AppendElement(directory);
-   
-  return rv;
-}
-
 nsresult nsAbMDBDirectory::RemoveCardFromAddressList(nsIAbCard* card)
 {
   nsresult rv = NS_OK;
@@ -261,7 +235,7 @@ NS_IMETHODIMP nsAbMDBDirectory::Init(const char* aURI)
   nsCOMPtr<nsIURI> uri = do_CreateInstance (NS_STANDARDURL_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = uri->SetSpec(aURI);
+  rv = uri->SetSpec(nsDependentCString(aURI));
   NS_ENSURE_SUCCESS(rv, rv);
 
   mIsValidURI = PR_TRUE;
@@ -269,20 +243,18 @@ NS_IMETHODIMP nsAbMDBDirectory::Init(const char* aURI)
     nsCOMPtr<nsIURL> url = do_QueryInterface(uri);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsXPIDLCString queryString;
-  rv = url->GetEscapedQuery (getter_Copies(queryString));
+  nsCAutoString queryString;
+  rv = url->GetQuery (queryString);
 
-  nsXPIDLCString path;
-  rv = url->GetPath (getter_Copies(path));
+  nsCAutoString path;
+  rv = url->GetPath (path);
   mPath = path;
 
-  PRUint32 queryStringLength;
-  if (queryString.get () && (queryStringLength = nsCRT::strlen (queryString)))
+  if (!queryString.IsEmpty())
   {
-    int pathLength = nsCRT::strlen (path) - queryStringLength - 1;
-    mPath.Truncate (pathLength);
+    mPath.Truncate(path.Length() - queryString.Length() - 1);
 
-    mURINoQuery.Truncate (mURINoQuery.Length () - queryStringLength - 1);
+    mURINoQuery.Truncate(mURINoQuery.Length() - queryString.Length() - 1);
 
     mQueryString = queryString;
 
@@ -412,7 +384,7 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsIEnumerator* *result)
   if (mIsQueryURI)
   {
     nsresult rv;
-    rv =  StartSearch ();
+    rv = StartSearch();
     NS_ENSURE_SUCCESS(rv, rv);
 
     // TODO
@@ -420,15 +392,19 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsIEnumerator* *result)
     // results after search is complete
     nsCOMPtr<nsISupportsArray> array;
     NS_NewISupportsArray(getter_AddRefs(array));
-    mSearchCache.Enumerate (enumerateSearchCache, (void* )array);
+    mSearchCache.Enumerate(enumerateSearchCache, (void*)array);
     return array->Enumerate(result);
   }
 
   if (mURI && mIsMailingList == -1)
   {
-    NS_ConvertUTF8toUCS2 file(&(mURI[strlen(kMDBDirectoryRoot)]));
-    PRInt32 pos = file.Find("/");
-    if (pos != kNotFound)
+    /* directory URIs are of the form
+     * moz-abmdbdirectory://foo
+     * mailing list URIs are of the form
+     * moz-abmdbdirectory://foo/bar
+     */
+    NS_ENSURE_TRUE(strlen(mURI) > kMDBDirectoryRootLen, NS_ERROR_UNEXPECTED);
+    if (strchr(mURI + kMDBDirectoryRootLen, '/'))
       mIsMailingList = 1;
     else
       mIsMailingList = 0;
@@ -449,7 +425,7 @@ NS_IMETHODIMP nsAbMDBDirectory::GetChildCards(nsIEnumerator* *result)
 
 NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
 {
-  nsresult rv;
+  nsresult rv = NS_OK;
 
   if (mIsQueryURI) {
     // if this is a query, delete the cards from the directory (without the query)
@@ -459,7 +435,7 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
     nsCOMPtr<nsIAddressBook> addressBook = do_GetService(NS_ADDRESSBOOK_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
-  nsCOMPtr<nsIAddrDatabase> database;
+    nsCOMPtr<nsIAddrDatabase> database;
     rv = addressBook->GetAbDatabaseFromURI(mURINoQuery.get(), getter_AddRefs(database));
     NS_ENSURE_SUCCESS(rv,rv);
 
@@ -469,7 +445,7 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
     nsCOMPtr<nsIRDFResource> resource;
     rv = gRDFService->GetResource(mURINoQuery.get(), getter_AddRefs(resource));
     NS_ENSURE_SUCCESS(rv, rv);
-
+    
     nsCOMPtr<nsIAbDirectory> directory = do_QueryInterface(resource, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -477,9 +453,9 @@ NS_IMETHODIMP nsAbMDBDirectory::DeleteCards(nsISupportsArray *cards)
     NS_ENSURE_SUCCESS(rv, rv);
 
     rv = database->RemoveListener(this);
-                NS_ENSURE_SUCCESS(rv, rv);
-  return rv;
-}
+    NS_ENSURE_SUCCESS(rv, rv);
+    return rv;
+  }
 
   if (!mDatabase)
     rv = GetAbDatabase();
@@ -644,11 +620,8 @@ NS_IMETHODIMP nsAbMDBDirectory::HasDirectory(nsIAbDirectory *dir, PRBool *hasDir
   return rv;
 }
 
-NS_IMETHODIMP nsAbMDBDirectory::CreateNewDirectory(PRUint32 prefCount, const char **prefName, const PRUnichar **prefValue)
+NS_IMETHODIMP nsAbMDBDirectory::CreateNewDirectory(nsIAbDirectoryProperties *aProperties)
 {
-  if (!*prefName || !*prefValue)
-    return NS_ERROR_NULL_POINTER;
-
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -678,8 +651,7 @@ NS_IMETHODIMP nsAbMDBDirectory::AddMailList(nsIAbDirectory *list)
     nsCOMPtr<nsIAbDirectory> newlist = getter_AddRefs(NS_STATIC_CAST(nsIAbDirectory*, dblistproperty));
     newlist->CopyMailList(list);
     list = newlist;
-    dblist = do_QueryInterface(list);
-    rv = NS_OK;
+    dblist = do_QueryInterface(list, &rv);
   }
 
   mDatabase->CreateMailListAndAddToDB(list, PR_TRUE);
@@ -687,10 +659,13 @@ NS_IMETHODIMP nsAbMDBDirectory::AddMailList(nsIAbDirectory *list)
 
   PRUint32 dbRowID;
   dblist->GetDbRowID(&dbRowID);
-  char *listUri = PR_smprintf("%s/MailList%ld", mURI, dbRowID);
+
+  nsCAutoString listUri;
+  listUri = nsDependentCString(mURI) + NS_LITERAL_CSTRING("/MailList");
+  listUri.AppendInt(dbRowID);
 
   nsCOMPtr<nsIAbDirectory> newList;
-  rv = AddDirectory(listUri, getter_AddRefs(newList));
+  rv = AddDirectory(listUri.get(), getter_AddRefs(newList));
   nsCOMPtr<nsIAbMDBDirectory> dbnewList(do_QueryInterface(newList));
   if (NS_SUCCEEDED(rv) && newList)
   {
@@ -760,9 +735,13 @@ NS_IMETHODIMP nsAbMDBDirectory::DropCard(nsIAbCard* aCard, PRBool needToCopyCard
 
   if (mURI && mIsMailingList == -1)
   {
-    NS_ConvertUTF8toUCS2 file(&(mURI[strlen(kMDBDirectoryRoot)]));
-    PRInt32 pos = file.Find("/");
-    if (pos != kNotFound)
+    /* directory URIs are of the form
+     * moz-abmdbdirectory://foo
+     * mailing list URIs are of the form
+     * moz-abmdbdirectory://foo/bar
+     */
+    NS_ENSURE_TRUE(strlen(mURI) > kMDBDirectoryRootLen, NS_ERROR_UNEXPECTED);
+    if (strchr(mURI + kMDBDirectoryRootLen, '/'))
       mIsMailingList = 1;
     else
       mIsMailingList = 0;
@@ -905,37 +884,42 @@ NS_IMETHODIMP nsAbMDBDirectory::OnAnnouncerGoingAway(nsIAddrDBAnnouncer *instiga
 
 NS_IMETHODIMP nsAbMDBDirectory::StartSearch()
 {
-  if (mIsQueryURI == PR_FALSE)
+  if (!mIsQueryURI)
     return NS_ERROR_FAILURE;
 
   nsresult rv;
 
   mPerformingQuery = PR_TRUE;
-  mSearchCache.Reset ();
+  mSearchCache.Reset();
 
   nsCOMPtr<nsIAbDirectoryQueryArguments> arguments = do_CreateInstance(NS_ABDIRECTORYQUERYARGUMENTS_CONTRACTID,&rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<nsIAbBooleanExpression> expression;
-  rv = nsAbQueryStringToExpression::Convert (mQueryString.get (),
+  rv = nsAbQueryStringToExpression::Convert(mQueryString.get(),
     getter_AddRefs(expression));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = arguments->SetExpression (expression);
+  rv = arguments->SetExpression(expression);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Set the return properties to
   // return nsIAbCard interfaces
   nsCStringArray properties;
-  properties.AppendCString (nsCAutoString ("card:nsIAbCard"));
-  CharPtrArrayGuard returnProperties (PR_FALSE);
-  rv = CStringArrayToCharPtrArray::Convert (properties,returnProperties.GetSizeAddr(),
+  properties.AppendCString(nsCAutoString("card:nsIAbCard"));
+  CharPtrArrayGuard returnProperties(PR_FALSE);
+  rv = CStringArrayToCharPtrArray::Convert(properties,returnProperties.GetSizeAddr(),
           returnProperties.GetArrayAddr(), PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
-  rv = arguments->SetReturnProperties (returnProperties.GetSize(), returnProperties.GetArray());
+  rv = arguments->SetReturnProperties(returnProperties.GetSize(), returnProperties.GetArray());
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = arguments->SetQuerySubDirectories (PR_TRUE);
+  // don't search the subdirectories 
+  // if the current directory is a mailing list, it won't have any subdirectories
+  // if the current directory is a addressbook, searching both it
+  // and the subdirectories (the mailing lists), will yield duplicate results
+  // because every entry in a mailing list will be an entry in the parent addressbook
+  rv = arguments->SetQuerySubDirectories(PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Set the the query listener
@@ -944,30 +928,29 @@ NS_IMETHODIMP nsAbMDBDirectory::StartSearch()
     new nsAbDirSearchListener (this);
   queryListener = _queryListener;
 
-
   // Get the directory without the query
   nsCOMPtr<nsIRDFResource> resource;
-  rv = gRDFService->GetResource (mURINoQuery.get (), getter_AddRefs (resource));
+  rv = gRDFService->GetResource (mURINoQuery.get(), getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIAbDirectory> directory (do_QueryInterface(resource, &rv));
+  nsCOMPtr<nsIAbDirectory> directory(do_QueryInterface(resource, &rv));
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Initiate the proxy query with the no query directory
   nsCOMPtr<nsIAbDirectoryQueryProxy> queryProxy = 
-      do_CreateInstance (NS_ABDIRECTORYQUERYPROXY_CONTRACTID, &rv);
+      do_CreateInstance(NS_ABDIRECTORYQUERYPROXY_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = queryProxy->Initiate (directory);
+  rv = queryProxy->Initiate(directory);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = queryProxy->DoQuery (arguments, queryListener, -1, 0, &mContext);
+  rv = queryProxy->DoQuery(arguments, queryListener, -1, 0, &mContext);
   return NS_OK;
 }
 
 NS_IMETHODIMP nsAbMDBDirectory::StopSearch()
 {
-  if (mIsQueryURI == PR_FALSE)
+  if (!mIsQueryURI)
     return NS_ERROR_FAILURE;
 
   return NS_OK;
@@ -999,13 +982,13 @@ nsresult nsAbMDBDirectory::GetAbDatabase()
 {
   if (!mDatabase && mURI) {
     nsresult rv;
-    
+
     nsCOMPtr<nsIAddressBook> addressBook = do_GetService(NS_ADDRESSBOOK_CONTRACTID, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
-    
+
     rv = addressBook->GetAbDatabaseFromURI(mURI, getter_AddRefs(mDatabase));
     NS_ENSURE_SUCCESS(rv,rv);
-    
+
     rv = mDatabase->AddListener(this);
     NS_ENSURE_SUCCESS(rv, rv);
   }
