@@ -64,7 +64,6 @@
 #include "nsIAppShellService.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIContentViewer.h"
-#include "nsIContentViewerFile.h"
 #include "nsIDocShell.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
@@ -73,6 +72,8 @@
 #include "nsSpecialSystemDirectory.h"
 #include "nsIFilePicker.h"
 #include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "nsVoidArray.h"
 #include "nsIAbCard.h"
 #include "nsIAbMDBCard.h"
@@ -92,6 +93,11 @@
 #define LDIF_LINEBREAK_LEN     2
 #endif
 
+// our schema is not fixed yet, but we still want some sort of objectclass
+// for now, use obsolete in the class name, hinting that this will change
+// see bugs bug #116692 and #118454
+#define MOZ_AB_OBJECTCLASS "mozillaAbPersonObsolete"
+
 // for now, these should be in the same order as they are in the import code
 // see importMsgProperties and nsImportStringBundle.h
 
@@ -106,40 +112,35 @@ struct ExportAttributesTableStruct
 
 // XXX todo, merge with what's in nsAbLDAPProperties.cpp, so we can
 // use this for LDAP and LDIF export
-// XXX todo what about _AimScreenName, or other generic columns?
 //
 // here's how we're coming up with the ldapPropertyName values
 // if they are specified in RFC 2798, use them
-// else use the 4.x LDIF attribute names
-//
-// the import code has to support 4.x LDIF, and the more attributes we support
-// the less lossy we are when going from AB -> LDIF -> AB
-//
-// one day, the nsnull values will be replaced by attribute names
-// and we'll fix the import code to handle them.
-// until then, AB -> LDIF -> AB will be lossy.  see bug #119360
+// else use the 4.x LDIF attribute names (for example, "xmozillanickname"
+// as we want to allow export from mozilla back to 4.x, and other apps
+// are probably out there that can handle 4.x LDIF)
+// else use the MOZ_AB_LDIF_PREFIX prefix, see nsIAddrDatabase.idl
 static ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = { 
   {kFirstNameColumn, "givenName", PR_TRUE},
   {kLastNameColumn, "sn", PR_TRUE},
   {kDisplayNameColumn, "cn", PR_TRUE},
   {kNicknameColumn, "xmozillanickname", PR_TRUE},
   {kPriEmailColumn, "mail", PR_TRUE},
-  {k2ndEmailColumn, nsnull, PR_TRUE},
+  {k2ndEmailColumn, MOZ_AB_LDIF_PREFIX "SecondEmail", PR_TRUE},
   {kPreferMailFormatColumn, "xmozillausehtmlmail", PR_FALSE},
   {kLastModifiedDateColumn, "modifytimestamp", PR_FALSE},
-  {kWorkPhoneColumn, "telehoneNumber", PR_TRUE}, 
+  {kWorkPhoneColumn, "telephoneNumber", PR_TRUE}, 
   {kHomePhoneColumn, "homePhone", PR_TRUE},
   {kFaxColumn, "facsimileTelephoneNumber", PR_TRUE},
   {kPagerColumn, "pager", PR_TRUE},
   {kCellularColumn, "mobile", PR_TRUE},
   {kHomeAddressColumn, "homePostalAddress", PR_TRUE},
-  {kHomeAddress2Column, nsnull, PR_TRUE},
-  {kHomeCityColumn, nsnull, PR_TRUE},
-  {kHomeStateColumn, nsnull, PR_TRUE},
-  {kHomeZipCodeColumn, nsnull, PR_TRUE},
-  {kHomeCountryColumn, nsnull, PR_TRUE},
+  {kHomeAddress2Column, MOZ_AB_LDIF_PREFIX "HomePostalAddress2", PR_TRUE},
+  {kHomeCityColumn, MOZ_AB_LDIF_PREFIX "HomeLocalityName", PR_TRUE},
+  {kHomeStateColumn, MOZ_AB_LDIF_PREFIX "HomeState", PR_TRUE},
+  {kHomeZipCodeColumn, MOZ_AB_LDIF_PREFIX "HomePostalCode", PR_TRUE},
+  {kHomeCountryColumn, MOZ_AB_LDIF_PREFIX "HomeCountryName", PR_TRUE},
   {kWorkAddressColumn, "postalAddress", PR_TRUE},
-  {kWorkAddress2Column, nsnull, PR_TRUE}, 
+  {kWorkAddress2Column, MOZ_AB_LDIF_PREFIX "PostalAddress2", PR_TRUE}, 
   {kWorkCityColumn, "l", PR_TRUE}, 
   {kWorkStateColumn, "st", PR_TRUE}, 
   {kWorkZipCodeColumn, "postalCode", PR_TRUE}, 
@@ -147,11 +148,11 @@ static ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = {
   {kJobTitleColumn, "title", PR_TRUE},
   {kDepartmentColumn, "ou", PR_TRUE},
   {kCompanyColumn, "o", PR_TRUE},
-  {kWebPage1Column, "homeurl", PR_TRUE},
-  {kWebPage2Column, "workurl", PR_TRUE},
-  {kBirthYearColumn, nsnull, PR_TRUE},
-  {kBirthMonthColumn, nsnull, PR_TRUE}, 
-  {kBirthDayColumn, nsnull, PR_TRUE}, 
+  {kWebPage1Column, "workurl", PR_TRUE},
+  {kWebPage2Column, "homeurl", PR_TRUE},
+  {kBirthYearColumn, nsnull, PR_TRUE}, // unused for now
+  {kBirthMonthColumn, nsnull, PR_TRUE}, // unused for now
+  {kBirthDayColumn, nsnull, PR_TRUE}, // unused for now
   {kCustom1Column, "custom1", PR_TRUE},
   {kCustom2Column, "custom2", PR_TRUE},
   {kCustom3Column, "custom3", PR_TRUE},
@@ -164,7 +165,7 @@ static ExportAttributesTableStruct EXPORT_ATTRIBUTES_TABLE[] = {
 //
 nsAddressBook::nsAddressBook()
 {
-  NS_INIT_REFCNT();
+  NS_INIT_ISUPPORTS();
 }
 
 nsAddressBook::~nsAddressBook()
@@ -180,13 +181,12 @@ NS_IMPL_QUERY_INTERFACE2(nsAddressBook, nsIAddressBook, nsICmdLineHandler);
 // nsIAddressBook
 //
 
-NS_IMETHODIMP nsAddressBook::NewAddressBook
-(nsIRDFCompositeDataSource* db, PRUint32 prefCount, const char **prefName, const PRUnichar **prefValue)
+NS_IMETHODIMP nsAddressBook::NewAddressBook(nsIAbDirectoryProperties *aProperties)
 {
-    if(!db || !*prefName || !*prefValue)
-        return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(aProperties);
+
+  nsresult rv;
 
-    nsresult rv = NS_OK;
     nsCOMPtr<nsIRDFService> rdfService = do_GetService (NS_RDF_CONTRACTID "/rdf-service;1", &rv);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -197,21 +197,21 @@ NS_IMETHODIMP nsAddressBook::NewAddressBook
     nsCOMPtr<nsIAbDirectory> parentDir = do_QueryInterface(parentResource, &rv);
     NS_ENSURE_SUCCESS(rv, rv);
         
-    rv = parentDir->CreateNewDirectory (prefCount, prefName, prefValue);
+  rv = parentDir->CreateNewDirectory(aProperties);
     return rv;
 }
 
 NS_IMETHODIMP nsAddressBook::DeleteAddressBooks
-(nsIRDFCompositeDataSource* db, nsISupportsArray *parentDir, nsISupportsArray *aResourceArray)
+(nsIRDFDataSource* aDS, nsISupportsArray *aParentDir, nsISupportsArray *aResourceArray)
 {
-  NS_ENSURE_ARG_POINTER(db);
-  NS_ENSURE_ARG_POINTER(parentDir);
+  NS_ENSURE_ARG_POINTER(aDS);
+  NS_ENSURE_ARG_POINTER(aParentDir);
   NS_ENSURE_ARG_POINTER(aResourceArray);
-
-    return DoCommand(db, NC_RDF_DELETE, parentDir, aResourceArray);
+  
+  return DoCommand(aDS, NC_RDF_DELETE, aParentDir, aResourceArray);
 }
 
-nsresult nsAddressBook::DoCommand(nsIRDFCompositeDataSource* db,
+nsresult nsAddressBook::DoCommand(nsIRDFDataSource* db,
                                   const char *command,
                                   nsISupportsArray *srcArray,
                                   nsISupportsArray *argumentArray)
@@ -260,11 +260,11 @@ NS_IMETHODIMP nsAddressBook::GetAbDatabaseFromURI(const char *aURI, nsIAddrDatab
   NS_ENSURE_ARG_POINTER(aDB);
   
   nsresult rv;
-    
-    nsCOMPtr<nsIAddrBookSession> abSession = 
-      do_GetService(NS_ADDRBOOKSESSION_CONTRACTID, &rv); 
+  
+  nsCOMPtr<nsIAddrBookSession> abSession = 
+    do_GetService(NS_ADDRBOOKSESSION_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
-    
+
   nsFileSpec* dbPath;
   rv = abSession->GetUserProfileDirectory(&dbPath);
   NS_ENSURE_SUCCESS(rv,rv);
@@ -280,20 +280,20 @@ NS_IMETHODIMP nsAddressBook::GetAbDatabaseFromURI(const char *aURI, nsIAddrDatab
     return NS_ERROR_UNEXPECTED;
 
   nsCAutoString file(aURI + kMDBDirectoryRootLen);
-      PRInt32 pos = file.Find("/");
-      if (pos != kNotFound)
-        file.Truncate(pos);
-      (*dbPath) += file.get();
-      
-      nsCOMPtr<nsIAddrDatabase> addrDBFactory = 
-        do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
+  PRInt32 pos = file.Find("/");
+  if (pos != kNotFound)
+    file.Truncate(pos);
+  (*dbPath) += file.get();
+  
+  nsCOMPtr<nsIAddrDatabase> addrDBFactory = 
+    do_GetService(NS_ADDRDATABASE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
-      
+
   rv = addrDBFactory->Open(dbPath, PR_TRUE, aDB, PR_TRUE);
   NS_ENSURE_SUCCESS(rv,rv);
-      
-      delete dbPath;
-      
+
+  delete dbPath;
+    
   return NS_OK;
 }
 
@@ -358,9 +358,10 @@ NS_IMETHODIMP nsAddressBook::MailListNameExists(const PRUnichar *name, PRBool *e
             DIR_Server *server = (DIR_Server *)pDirectories->ElementAt(i);
             if (server->dirType == PABDirectory)
             {
-                nsAutoString dbfile; dbfile.AssignWithConversion(server->fileName);
-                PRInt32 pos = dbfile.Find("na2");
-                if (pos >= 0) /* check: this is a 4.x file, remove when conversion is done */
+                /* check: this is a 4.x file, remove when conversion is done */
+                PRUint32 fileNameLen = strlen(server->fileName);
+                if ((fileNameLen > kABFileName_PreviousSuffixLen) && 
+                     strcmp(server->fileName + fileNameLen - kABFileName_PreviousSuffixLen, kABFileName_PreviousSuffix) == 0)
                     continue;
 
                 nsCOMPtr<nsIAddrDatabase> database;
@@ -389,12 +390,15 @@ protected:
     PRBool mStoreLocAsHome;
     PRBool mDeleteDB;
     PRBool mImportingComm4x;
+    PRInt32 mLFCount;
+    PRInt32 mCRCount;
 
     nsresult ParseLDIFFile();
     void AddLdifRowToDatabase(PRBool bIsList);
     void AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char* valueSlot, PRBool bIsList);
+    void ClearLdifRecordBuffer();
 
-    nsresult GetLdifStringRecord(char* buf, PRInt32 len, PRInt32* stopPos);
+    nsresult GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos);
     nsresult str_parse_line(char *line, char    **type, char **value, int *vlen);
     char * str_getline( char **next );
 
@@ -417,6 +421,8 @@ AddressBookParser::AddressBookParser(nsIFileSpec * fileSpec, PRBool migrating, n
     mDeleteDB = PR_TRUE;
   mStoreLocAsHome = bStoreLocAsHome;
   mImportingComm4x = bImportingComm4x;
+  mLFCount = 0;                                                              
+  mCRCount = 0;
 }
 
 AddressBookParser::~AddressBookParser(void)
@@ -695,43 +701,41 @@ char * AddressBookParser::str_getline( char **next )
  * get one ldif record
  * 
  */
-nsresult AddressBookParser::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32* stopPos)
+nsresult AddressBookParser::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos)
 {
-    PRInt32 LFCount = 0;
-    PRInt32 CRCount = 0;
-
-    for (; *stopPos < len; (*stopPos)++) 
+    for (; stopPos < len; stopPos++) 
     {
-        char c = buf[*stopPos];
+        char c = buf[stopPos];
 
         if (c == 0xA)
         {
-            LFCount++;
+            mLFCount++;
         }
         else if (c == 0xD)
         {
-            CRCount++;
+            mCRCount++;
         }
         else if ( c != 0xA && c != 0xD)
         {
-            if (LFCount == 0 && CRCount == 0)
+            if (mLFCount == 0 && mCRCount == 0)
                  mLine.Append(c);
-            else if (( LFCount > 1) || ( CRCount > 2 && LFCount ) ||
-                ( !LFCount && CRCount > 1 ))
+            else if (( mLFCount > 1) || ( mCRCount > 2 && mLFCount ) ||
+                ( !mLFCount && mCRCount > 1 ))
             {
                 return NS_OK;
             }
-            else if ((LFCount == 1 || CRCount == 1))
+            else if ((mLFCount == 1 || mCRCount == 1))
             {
                  mLine.Append('\n');
                  mLine.Append(c);
-                LFCount = 0;
-                CRCount = 0;
+                mLFCount = 0;
+                mCRCount = 0;
             }
         }
     }
-    if (*stopPos ==len && (LFCount > 1) || (CRCount > 2 && LFCount) ||
-        (!LFCount && CRCount > 1))
+
+    if ((stopPos == len) && (mLFCount > 1) || (mCRCount > 2 && mLFCount) ||
+        (!mLFCount && mCRCount > 1))
         return NS_OK;
     else
         return NS_ERROR_FAILURE;
@@ -744,7 +748,8 @@ nsresult AddressBookParser::ParseLDIFFile()
     PRInt32 startPos = 0;
     PRInt32 len = 0;
     PRBool bEof = PR_FALSE;
-    nsVoidArray listPosArray;
+    nsVoidArray listPosArray;   // where each list/group starts in ldif file
+    nsVoidArray listSizeArray;  // size of the list/group info
     PRInt32 savedStartPos = 0;
     PRInt32 filePos = 0;
 
@@ -754,7 +759,7 @@ nsresult AddressBookParser::ParseLDIFFile()
         {
             startPos = 0;
 
-            while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, &startPos)))
+            while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
             {
                 if (mLine.Find("groupOfNames") == kNotFound)
                     AddLdifRowToDatabase(PR_FALSE);
@@ -762,8 +767,8 @@ nsresult AddressBookParser::ParseLDIFFile()
                 {
                     //keep file position for mailing list
                     listPosArray.AppendElement((void*)savedStartPos);
-                    if (mLine.Length() > 0)
-                        mLine.Truncate();
+                    listSizeArray.AppendElement((void*)(filePos + startPos-savedStartPos));
+                    ClearLdifRecordBuffer();
                 }
                 savedStartPos = filePos + startPos;
             }
@@ -775,18 +780,26 @@ nsresult AddressBookParser::ParseLDIFFile()
         AddLdifRowToDatabase(PR_FALSE); 
 
     // mail Lists
-    PRInt32 i;
+    PRInt32 i, pos, size;
     PRInt32 listTotal = listPosArray.Count();
+    char *listBuf;
+    ClearLdifRecordBuffer();  // make sure the buffer is clean
     for (i = 0; i < listTotal; i++)
     {
-        PRInt32 pos = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        pos  = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        size = NS_PTR_TO_INT32(listSizeArray.ElementAt(i));
         if (NS_SUCCEEDED(mFileSpec->Seek(pos)))
         {
-            if (NS_SUCCEEDED(mFileSpec->Read(&pBuf, (PRInt32)sizeof(buf), &len)) && len > 0)
+
+            // Allocate enough space for the lists/groups as the size varies.
+            listBuf = (char *) PR_Malloc(size);
+            if (!listBuf)
+              continue;
+            if (NS_SUCCEEDED(mFileSpec->Read(&listBuf, size, &len)) && len > 0)
             {
                 startPos = 0;
 
-                while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, &startPos)))
+                while (NS_SUCCEEDED(GetLdifStringRecord(listBuf, len, startPos)))
                 {
                     if (mLine.Find("groupOfNames") != kNotFound)
                     {
@@ -796,6 +809,7 @@ nsresult AddressBookParser::ParseLDIFFile()
                     }
                 }
             }
+            PR_FREEIF(listBuf);
         }
     }
     return NS_OK;
@@ -803,6 +817,14 @@ nsresult AddressBookParser::ParseLDIFFile()
 
 void AddressBookParser::AddLdifRowToDatabase(PRBool bIsList)
 {
+    // If no data to process then reset CR/LF counters and return.
+    if (mLine.IsEmpty())
+    {
+      mLFCount = 0;
+      mCRCount = 0;
+      return;
+    }
+
     nsCOMPtr <nsIMdbRow> newRow;
     if (mDatabase)
     {
@@ -832,14 +854,24 @@ void AddressBookParser::AddLdifRowToDatabase(PRBool bIsList)
         else
             continue; // parse error: continue with next loop iteration
     }
-    delete [] saveCursor;
+    nsMemory::Free(saveCursor);
     mDatabase->AddCardRowToDB(newRow);    
 
     if (bIsList)
         mDatabase->AddListDirNode(newRow);
 
-    if (mLine.Length() > 0)
-        mLine.Truncate();
+    // Clear buffer for next record
+    ClearLdifRecordBuffer();
+}
+
+void AddressBookParser::ClearLdifRecordBuffer()
+{
+  if (mLine.Length() > 0)
+  {
+      mLine.Truncate();
+      mLFCount = 0;
+      mCRCount = 0;
+  }
 }
 
 // We have two copies of this function in the code, one here for import and 
@@ -964,7 +996,7 @@ void AddressBookParser::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot,
         mDatabase->AddHomePhone(newRow, column.get());
 
       else if ( kNotFound != colType.Find("homeurl") )
-        mDatabase->AddWebPage1(newRow, column.get());
+        mDatabase->AddWebPage2(newRow, column.get());
       break; // 'h'
 
     case 'i':
@@ -1172,7 +1204,7 @@ void AddressBookParser::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot,
 
     case 'w':
       if ( kNotFound != colType.Find("workurl") )
-        mDatabase->AddWebPage2(newRow, column.get());
+        mDatabase->AddWebPage1(newRow, column.get());
 
       break; // 'w'
 
@@ -1248,7 +1280,11 @@ NS_IMETHODIMP nsAddressBook::ConvertLDIFtoMAB(nsIFileSpec *fileSpec, PRBool migr
     rv = abParser.ParseFile();
     NS_ENSURE_SUCCESS(rv, rv);
 
-    rv = fileSpec->CloseStream();
+    fileSpec->CloseStream();
+
+    // Commit the changes if 'db' is set.
+    if (db)
+      rv = db->Commit(nsAddrDBCommitType::kLargeCommit);
     return rv;
 }
 
@@ -1324,8 +1360,8 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
     }
   }
 
-  nsXPIDLCString leafName;
-  rv = localFile->GetLeafName(getter_Copies(leafName));
+  nsCAutoString leafName;
+  rv = localFile->GetNativeLeafName(leafName);
   NS_ENSURE_SUCCESS(rv,rv);
 
   // we're looking for .tab, .txt, .csv, .ldi or .ldif
@@ -1334,8 +1370,7 @@ NS_IMETHODIMP nsAddressBook::ExportAddressBook(nsIAbDirectory *aDirectory)
   // check if they equal .tab,.txt, or .csv, if so, use those
   // otherwise, treat as LDIF
   if (leafName.Length() > EXTENSION_LENGTH) {
-    nsCAutoString extension;
-    leafName.Right(extension, EXTENSION_LENGTH); 
+    nsCAutoString extension(Substring(leafName, leafName.Length() - EXTENSION_LENGTH, EXTENSION_LENGTH));
 
     // treat .TXT, .Tab, .csV like .txt, .tab, and .csv
     ToLowerCase(extension);
@@ -1395,7 +1430,7 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
   for (i = 0; i < EXPORT_ATTRIBUTES_TABLE_COUNT; i++) {
     if (EXPORT_ATTRIBUTES_TABLE[i].includeForPlainText) {
       // XXX localize this?
-      length = PL_strlen(EXPORT_ATTRIBUTES_TABLE[i].abColName);
+      length = strlen(EXPORT_ATTRIBUTES_TABLE[i].abColName);
       rv = outputStream->Write(EXPORT_ATTRIBUTES_TABLE_COUNT[i].abColName, length, &writeCount);
       NS_ENSURE_SUCCESS(rv,rv);
       if (length != writeCount)
@@ -1441,7 +1476,10 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
             if (EXPORT_ATTRIBUTES_TABLE[i].includeForPlainText) {
               rv = card->GetCardValue(EXPORT_ATTRIBUTES_TABLE[i].abColName, getter_Copies(value));
               NS_ENSURE_SUCCESS(rv,rv);
-             
+
+              // For notes, make sure CR/LF is converted to spaces 
+              // to avoid creating multiple lines for a single card.
+              //
               // the import code expects .txt, .tab, .csv files to
               // have non-ASCII data in the system charset
               //
@@ -1451,7 +1489,19 @@ nsAddressBook::ExportDirectoryToDelimitedText(nsIAbDirectory *aDirectory, const
               //
               // the solution is to export / import as LDIF.
               // non-ASCII data is treated as base64 encoded UTF-8 in LDIF
-              rv = importService->SystemStringFromUnicode(value.get(), valueCStr);
+              if (!strcmp(EXPORT_ATTRIBUTES_TABLE[i].abColName, kNotesColumn))
+              {
+                nsAutoString tempStr(value.get());
+                if (!tempStr.IsEmpty())
+                {
+                  tempStr.ReplaceChar(nsCRT::CR, ' ');
+                  tempStr.ReplaceChar(nsCRT::LF, ' ');
+                }
+                rv = importService->SystemStringFromUnicode(tempStr.get(), valueCStr);
+              }
+              else
+                rv = importService->SystemStringFromUnicode(value.get(), valueCStr);
+
               if (NS_FAILED(rv)) {
                 NS_ASSERTION(0, "failed to convert string to system charset.  use LDIF");
                 valueCStr = "?";
@@ -1596,6 +1646,18 @@ nsAddressBook::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *a
               // ldif doesn't export mutliple addresses
             }
           }
+        
+          nsCString optionalLDIF;
+          rv = GetOptionalLDIFForCard(card, optionalLDIF);
+          NS_ENSURE_SUCCESS(rv,rv);
+
+          length = optionalLDIF.Length();
+          if (length) {
+            rv = outputStream->Write(optionalLDIF.get(), length, &writeCount);
+            NS_ENSURE_SUCCESS(rv,rv);
+            if (length != writeCount)
+              return NS_ERROR_FAILURE;
+          }
 
           // write out the linebreak that seperates the cards
           rv = outputStream->Write(LDIF_LINEBREAK, LDIF_LINEBREAK_LEN, &writeCount);
@@ -1615,7 +1677,7 @@ nsAddressBook::ExportDirectoryToLDIF(nsIAbDirectory *aDirectory, nsILocalFile *a
   return NS_OK;
 }
 
-nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsACString &aResult)
 {
   nsresult rv;
   nsXPIDLString attrValue;
@@ -1691,7 +1753,7 @@ nsresult nsAddressBook::AppendLDIFForMailList(nsIAbCard *aCard, nsAFlatCString &
   return NS_OK;
 }
 
-nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard, nsACString &aResult)
 {
   nsXPIDLString email;
   nsXPIDLString displayName;
@@ -1720,32 +1782,50 @@ nsresult nsAddressBook::AppendDNForCard(const char *aProperty, nsIAbCard *aCard,
   return rv;
 }
 
-nsresult nsAddressBook::AppendBasicLDIFForCard(nsIAbCard *aCard, nsAFlatCString &aResult)
+nsresult nsAddressBook::AppendBasicLDIFForCard(nsIAbCard *aCard, nsACString &aResult)
 {
   nsresult rv = AppendDNForCard("dn", aCard, aResult);
   NS_ENSURE_SUCCESS(rv,rv);
   aResult += LDIF_LINEBREAK \
     "objectclass: top" LDIF_LINEBREAK \
     "objectclass: person" LDIF_LINEBREAK \
-    "objectclass: organizationalPerson" LDIF_LINEBREAK 
-    "objectclass: inetOrgPerson" LDIF_LINEBREAK;
+    "objectclass: organizationalPerson" LDIF_LINEBREAK \
+    "objectclass: inetOrgPerson" LDIF_LINEBREAK \
+    "objectclass: " MOZ_AB_OBJECTCLASS LDIF_LINEBREAK;  
+
 
   return rv;
 }
 
-nsresult nsAddressBook::AppendProperty(const char *aProperty, const PRUnichar *aValue, nsAFlatCString &aResult)
+PRBool nsAddressBook::IsSafeLDIFString(const PRUnichar *aStr)
+{
+  // follow RFC 2849 to determine if something is safe "as is" for LDIF
+  if (aStr[0] == PRUnichar(' ') ||
+      aStr[0] == PRUnichar(':') ||
+      aStr[0] == PRUnichar('<')) 
+    return PR_FALSE;
+
+  PRUint32 i;
+  PRUint32 len = nsCRT::strlen(aStr);
+  for (i=0; i<len; i++) {
+    // If string contains CR or LF, it is not safe for LDIF
+    // and MUST be base64 encoded
+    if ((aStr[i] == PRUnichar('\n')) ||
+        (aStr[i] == PRUnichar('\r')) ||
+        (!nsCRT::IsAscii(aStr[i])))
+      return PR_FALSE;
+  }
+  return PR_TRUE;
+}
+
+nsresult nsAddressBook::AppendProperty(const char *aProperty, const PRUnichar *aValue, nsACString &aResult)
 {
   NS_ENSURE_ARG_POINTER(aValue);
 
   aResult += aProperty;
  
-  // follow RFC 2849 for when we MUST base64 encode
-  if (nsCRT::IsAscii(aValue) && 
-    aValue[0] != PRUnichar(' ') &&
-    aValue[0] != PRUnichar('\n') &&
-    aValue[0] != PRUnichar('\r') &&
-    aValue[0] != PRUnichar(':') &&
-    aValue[0] != PRUnichar('<')) {
+  // if the string is not safe "as is", base64 encode it
+  if (IsSafeLDIFString(aValue)) {
     aResult += NS_LITERAL_CSTRING(": ") + NS_LossyConvertUCS2toASCII(aValue);
   }
   else {
@@ -1760,5 +1840,43 @@ nsresult nsAddressBook::AppendProperty(const char *aProperty, const PRUnichar *a
   return NS_OK;
 }
 
+
+nsresult nsAddressBook::GetOptionalLDIFForCard(nsIAbCard *aCard, nsACString &aResult)
+{
+  nsresult rv;
+  nsCOMPtr<nsIAbMDBCard> dbcard = do_QueryInterface(aCard, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  NS_NAMED_LITERAL_CSTRING(kOptionalLDIFExportMapPrefix, "mail.addr_book.export_ldif_map.");
+  PRUint32 i, prefCount, prefixLen=kOptionalLDIFExportMapPrefix.Length();
+  char **prefNames;
+
+  rv = prefBranch->GetChildList(kOptionalLDIFExportMapPrefix.get(), &prefCount, &prefNames);
+  if (NS_SUCCEEDED(rv) && prefCount > 0) {
+    for (i = 0; i < prefCount; i++) {
+      nsXPIDLString genericValue;
+      rv = dbcard->GetStringAttribute(prefNames[i]+prefixLen, getter_Copies(genericValue));
+      if (NS_SUCCEEDED(rv) && !genericValue.IsEmpty()) {
+        nsXPIDLCString mapValue;
+        rv = prefBranch->GetCharPref(prefNames[i], getter_Copies(mapValue));
+        if (NS_SUCCEEDED(rv) && mapValue.Length()) {
+          rv = AppendProperty(mapValue.get(), genericValue.get(), aResult);
+          aResult += LDIF_LINEBREAK;
+        }
+      }
+    }
+    NS_FREE_XPCOM_ALLOCATED_POINTER_ARRAY(prefCount, prefNames);
+  }
+  return NS_OK;
+}
+
+
 CMDLINEHANDLER_IMPL(nsAddressBook,"-addressbook","general.startup.addressbook","chrome://messenger/content/addressbook/addressbook.xul","Start with the addressbook.",NS_ADDRESSBOOKSTARTUPHANDLER_CONTRACTID,"Addressbook Startup Handler",PR_FALSE,"", PR_TRUE)
 
