@@ -289,7 +289,7 @@ nsMsgSearchAdapter::EscapeQuoteImapSearchProtocol(const PRUnichar *imapCommand)
 
 char *nsMsgSearchAdapter::UnEscapeSearchUrl (const char *commandSpecificData)
 {
-  char *result = (char*) PR_Malloc (nsCRT::strlen(commandSpecificData) + 1);
+  char *result = (char*) PR_Malloc (strlen(commandSpecificData) + 1);
 	if (result)
 	{
 		char *resultPtr = result;
@@ -317,10 +317,13 @@ char *nsMsgSearchAdapter::UnEscapeSearchUrl (const char *commandSpecificData)
 }
 
 
-void
-nsMsgSearchAdapter::GetSearchCharsets(nsString &srcCharset, nsString& dstCharset)
+nsresult 
+nsMsgSearchAdapter::GetSearchCharsets(PRUnichar **srcCharset, PRUnichar **dstCharset)
 {
   nsresult rv;
+  NS_ENSURE_ARG(srcCharset);
+  NS_ENSURE_ARG(dstCharset);
+
   if (m_defaultCharset.IsEmpty())
   {
     m_forceAsciiSearch = PR_FALSE;  // set the default value in case of error
@@ -331,9 +334,9 @@ nsMsgSearchAdapter::GetSearchCharsets(nsString &srcCharset, nsString& dstCharset
       rv = prefs->GetBoolPref("mailnews.force_ascii_search", &m_forceAsciiSearch);
     }
   }
-  srcCharset.Assign(m_defaultCharset.IsEmpty() ?
+  *srcCharset = nsCRT::strdup(m_defaultCharset.IsEmpty() ?
                     NS_LITERAL_STRING("ISO-8859-1").get() : m_defaultCharset.get());
-  dstCharset = srcCharset;
+  *dstCharset = nsCRT::strdup(*srcCharset);
 
 	if (m_scope)
 	{
@@ -347,7 +350,7 @@ nsMsgSearchAdapter::GetSearchCharsets(nsString &srcCharset, nsString& dstCharset
         {
             nsXPIDLString folderCharset;
             folder->GetCharset(getter_Copies(folderCharset));
-            dstCharset.Assign(folderCharset);
+            *dstCharset = nsCRT::strdup(folderCharset);
         }
 	}
 
@@ -357,8 +360,8 @@ nsMsgSearchAdapter::GetSearchCharsets(nsString &srcCharset, nsString& dstCharset
 	// the source. (CS_DEFAULT is an indication that the charset
 	// was undefined or unavailable.)
   // ### well, it's not really anymore. Is there an equivalent?
-  if (!nsCRT::strcmp(dstCharset.get(), m_defaultCharset.get()))
-		dstCharset = srcCharset;
+  if (!nsCRT::strcmp(*dstCharset, m_defaultCharset.get()))
+    *dstCharset = nsCRT::strdup(*srcCharset);
 
 	if (m_forceAsciiSearch)
 	{
@@ -369,8 +372,9 @@ nsMsgSearchAdapter::GetSearchCharsets(nsString &srcCharset, nsString& dstCharset
 		// If the dest csid is ISO Latin 1 or MacRoman, attempt to convert the 
 		// source text to US-ASCII. (Not for now.)
 		// if ((dst_csid == CS_LATIN1) || (dst_csid == CS_MAC_ROMAN))
-			dstCharset.Assign(NS_LITERAL_STRING("us-ascii"));
+    *dstCharset = nsCRT::strdup(NS_LITERAL_STRING("us-ascii").get());
 	}
+  return NS_OK;
 }
 
 nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool reallyDredd, const PRUnichar *srcCharset, const PRUnichar *destCharset, char **ppOutTerm)
@@ -493,9 +497,9 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
     {
       nsXPIDLCString arbitraryHeaderTerm;
       term->GetArbitraryHeader(getter_Copies(arbitraryHeaderTerm));
-      if (nsCRT::strlen((const char *) arbitraryHeaderTerm) > 0)
+      if (strlen((const char *) arbitraryHeaderTerm) > 0)
 		  {
-			  arbitraryHeader = new char [nsCRT::strlen((const char *)arbitraryHeaderTerm) + 6];  // 6 bytes for SPACE \" .... \" SPACE
+			  arbitraryHeader = new char [strlen((const char *)arbitraryHeaderTerm) + 6];  // 6 bytes for SPACE \" .... \" SPACE
 			  if (!arbitraryHeader)
 				  return NS_ERROR_OUT_OF_MEMORY;
 			  arbitraryHeader[0] = '\0';
@@ -604,7 +608,7 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
 			{
 				char *oldValue = value;
 				// max escaped length is one extra character for every character in the cmd.
-				char *newValue = (char*)PR_Malloc(sizeof(char) * (2*nsCRT::strlen(value) + 1));
+				char *newValue = (char*)PR_Malloc(sizeof(char) * (2*strlen(value) + 1));
 				if (newValue)
 				{
 					char *p = newValue;
@@ -631,10 +635,10 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
 		}
 	}
 
-	int len = nsCRT::strlen(whichMnemonic) + nsCRT::strlen(value) + (useNot ? nsCRT::strlen(m_kImapNot) : 0) + 
-		(useQuotes ? 2 : 0) + nsCRT::strlen(m_kImapHeader) + 
-		(orHeaderMnemonic ? (nsCRT::strlen(m_kImapHeader) + nsCRT::strlen(m_kImapOr) + (useNot ? nsCRT::strlen(m_kImapNot) : 0) + 
-		nsCRT::strlen(orHeaderMnemonic) + nsCRT::strlen(value) + 2 /*""*/) : 0) + 1;
+	int len = strlen(whichMnemonic) + strlen(value) + (useNot ? strlen(m_kImapNot) : 0) + 
+		(useQuotes ? 2 : 0) + strlen(m_kImapHeader) + 
+		(orHeaderMnemonic ? (strlen(m_kImapHeader) + strlen(m_kImapOr) + (useNot ? strlen(m_kImapNot) : 0) + 
+		strlen(orHeaderMnemonic) + strlen(value) + 2 /*""*/) : 0) + 1;
 	char *encoding = new char[len];
 	if (encoding)
 	{
@@ -664,8 +668,8 @@ nsresult nsMsgSearchAdapter::EncodeImapTerm (nsIMsgSearchTerm *term, PRBool real
 		
 		// kmcentee, don't let the encoding end with whitespace, 
 		// this throws off later url STRCMP
-		if (*encoding && *(encoding + nsCRT::strlen(encoding) - 1) == ' ')
-			*(encoding + nsCRT::strlen(encoding) - 1) = '\0';
+		if (*encoding && *(encoding + strlen(encoding) - 1) == ' ')
+			*(encoding + strlen(encoding) - 1) = '\0';
 	}
 
 	if (value && valueWasAllocated)
@@ -728,7 +732,7 @@ nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray
 		err = EncodeImapTerm (pTerm, reallyDredd, srcCharset, destCharset, &termEncodings[i]);
 		if (NS_SUCCEEDED(err) && nsnull != termEncodings[i])
 		{
-			encodingLength += nsCRT::strlen(termEncodings[i]) + 1;
+			encodingLength += strlen(termEncodings[i]) + 1;
 			expression = expression->AddSearchTerm(pTerm,termEncodings[i]);
 		}
 	}
@@ -736,7 +740,7 @@ nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray
 	if (NS_SUCCEEDED(err)) 
 	{
 		// Catenate the intermediate encodings together into a big string
-		char *totalEncoding = new char [encodingLength + (!reallyDredd ? nsCRT::strlen(m_kImapUnDeleted) : 0) + 1];
+		char *totalEncoding = new char [encodingLength + (!reallyDredd ? strlen(m_kImapUnDeleted) : 0) + 1];
 		nsCString encodingBuff;
 
 		if (totalEncoding)
@@ -750,7 +754,7 @@ nsresult nsMsgSearchAdapter::EncodeImap (char **ppOutEncoding, nsISupportsArray
 			if (!reallyDredd)
 			{
 				encodingBuff.Append(m_kImapUnDeleted);
-				offset = nsCRT::strlen(m_kImapUnDeleted);
+				offset = strlen(m_kImapUnDeleted);
 			}
 
 			expression->GenerateEncodeStr(&encodingBuff);
@@ -858,6 +862,9 @@ nsMsgSearchValidityTable::nsMsgSearchValidityTable ()
 			SetValidButNotShown (i,j, PR_FALSE);
 		}
 	m_numAvailAttribs = 0;   // # of attributes marked with at least one available operator
+  // assume default is Subject, which it is for mail and news search
+  // it's not for LDAP, so we'll call SetDefaultAttrib()
+  m_defaultAttrib = nsMsgSearchAttrib::Subject;
 }
 
 NS_IMPL_ISUPPORTS1(nsMsgSearchValidityTable, nsIMsgSearchValidityTable)
@@ -958,10 +965,16 @@ nsMsgSearchValidityTable::GetAvailableOperators(nsMsgSearchAttribValue aAttribut
                                                 PRUint32 *aLength,
                                                 nsMsgSearchOpValue **aResult)
 {
+    nsMsgSearchAttribValue attr;
+    if (aAttribute == nsMsgSearchAttrib::Default)
+      attr = m_defaultAttrib;
+    else
+      attr = aAttribute;
+
     PRUint32 totalOperators=0;
     PRInt32 i;
     for (i=0; i<nsMsgSearchOp::kNumMsgSearchOperators; i++) {
-        if (m_table[aAttribute][i].bitAvailable)
+        if (m_table[attr][i].bitAvailable)
             totalOperators++;
     }
 
@@ -971,7 +984,7 @@ nsMsgSearchValidityTable::GetAvailableOperators(nsMsgSearchAttribValue aAttribut
     
     PRUint32 numStored = 0;
     for (i=0; i<nsMsgSearchOp::kNumMsgSearchOperators;i++) {
-        if (m_table[aAttribute][i].bitAvailable)
+        if (m_table[attr][i].bitAvailable)
             array[numStored++] = i;
     }
 
@@ -981,6 +994,13 @@ nsMsgSearchValidityTable::GetAvailableOperators(nsMsgSearchAttribValue aAttribut
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgSearchValidityTable::SetDefaultAttrib(nsMsgSearchAttribValue aAttribute)
+{
+  m_defaultAttrib = aAttribute;
+  return NS_OK;
+}
+
 
 nsMsgSearchValidityManager::nsMsgSearchValidityManager ()
 {
@@ -1018,22 +1038,22 @@ nsresult nsMsgSearchValidityManager::GetTable (int whichTable, nsIMsgSearchValid
 	case nsMsgSearchScope::offlineMail:
     if (!m_offlineMailTable)
       err = InitOfflineMailTable ();
-    if (customHeaders && m_offlineMailTable)
-      err = InitOtherHeadersInTable(m_offlineMailTable, customHeaders.get());
+    if (m_offlineMailTable)
+      err = SetOtherHeadersInTable(m_offlineMailTable, customHeaders.get());
 		*ppOutTable = m_offlineMailTable;
 		break;
 	case nsMsgSearchScope::onlineMail:
 		if (!m_onlineMailTable)
       err = InitOnlineMailTable ();
-    if (customHeaders && m_onlineMailTable)
-      err = InitOtherHeadersInTable(m_onlineMailTable, customHeaders.get());
+    if (m_onlineMailTable)
+      err = SetOtherHeadersInTable(m_onlineMailTable, customHeaders.get());
 		*ppOutTable = m_onlineMailTable;
 		break;
 	case nsMsgSearchScope::onlineMailFilter:
 		if (!m_onlineMailFilterTable)
 			err = InitOnlineMailFilterTable ();
-    if (customHeaders && m_onlineMailFilterTable)
-      err = InitOtherHeadersInTable(m_onlineMailFilterTable, customHeaders.get());
+    if (m_onlineMailFilterTable)
+      err = SetOtherHeadersInTable(m_onlineMailFilterTable, customHeaders.get());
 		*ppOutTable = m_onlineMailFilterTable;
 		break;
 	case nsMsgSearchScope::news:
@@ -1044,8 +1064,8 @@ nsresult nsMsgSearchValidityManager::GetTable (int whichTable, nsIMsgSearchValid
 	case nsMsgSearchScope::localNews:
 		if (!m_localNewsTable)
 			err = InitLocalNewsTable();
-    if (customHeaders && m_localNewsTable)
-      err = InitOtherHeadersInTable(m_localNewsTable, customHeaders.get());
+    if (m_localNewsTable)
+      err = SetOtherHeadersInTable(m_localNewsTable, customHeaders.get());
 		*ppOutTable = m_localNewsTable;
 		break;
 #ifdef DOING_EXNEWSSEARCH
@@ -1055,13 +1075,16 @@ nsresult nsMsgSearchValidityManager::GetTable (int whichTable, nsIMsgSearchValid
 		*ppOutTable = m_newsExTable;
 		break;
 #endif
-#ifdef DOING_LDAP
 	case nsMsgSearchScope::LDAP:
 		if (!m_ldapTable)
 			err = InitLdapTable ();
 		*ppOutTable = m_ldapTable;
 		break;
-#endif
+  case nsMsgSearchScope::LocalAB:
+		if (!m_localABTable)
+			err = InitLocalABTable ();
+		*ppOutTable = m_localABTable;
+		break;
 	default:                 
 		NS_ASSERTION(PR_FALSE, "invalid table type");
 		err = NS_MSG_ERROR_INVALID_SEARCH_TERM;
@@ -1085,17 +1108,23 @@ nsMsgSearchValidityManager::NewTable(nsIMsgSearchValidityTable **aTable)
 }
 
 nsresult 
-nsMsgSearchValidityManager::InitOtherHeadersInTable (nsIMsgSearchValidityTable *aTable, const char *customHeaders)
+nsMsgSearchValidityManager::SetOtherHeadersInTable (nsIMsgSearchValidityTable *aTable, const char *customHeaders)
 {
-  PRUint32 strlen = PL_strlen(customHeaders);
+  PRUint32 customHeadersLength = strlen(customHeaders);
   PRUint32 numHeaders=0;
   PRUint32 i=0;
-  for (i=0;i <strlen; i++)
-    if (customHeaders[i] == ':')
-      numHeaders++;
-  numHeaders++;
+  if (customHeadersLength)
+  {
+    for (i=0;i <customHeadersLength; i++)
+      if (customHeaders[i] == ':')
+        numHeaders++;
+    numHeaders++;
+  }
+
   NS_ASSERTION(nsMsgSearchAttrib::OtherHeader + numHeaders < nsMsgSearchAttrib::kNumMsgSearchAttributes, "more headers than the table can hold");
-  for (i=nsMsgSearchAttrib::OtherHeader+1;i< (nsMsgSearchAttrib::OtherHeader + numHeaders+1) && nsMsgSearchAttrib::kNumMsgSearchAttributes;i++)
+
+  PRUint32 maxHdrs= PR_MIN(nsMsgSearchAttrib::OtherHeader + numHeaders+1, nsMsgSearchAttrib::kNumMsgSearchAttributes);
+  for (i=nsMsgSearchAttrib::OtherHeader+1;i< maxHdrs;i++)
   {
     aTable->SetAvailable (i, nsMsgSearchOp::Contains, 1);   // added for arbitrary headers
     aTable->SetEnabled   (i, nsMsgSearchOp::Contains, 1); 
@@ -1107,7 +1136,7 @@ nsMsgSearchValidityManager::InitOtherHeadersInTable (nsIMsgSearchValidityTable *
     aTable->SetEnabled   (i, nsMsgSearchOp::Isnt, 1);
   }
    //because custom headers can change; so reset the table for those which are no longer used. 
-  for (PRUint32 j=nsMsgSearchAttrib::OtherHeader+numHeaders+1; j < nsMsgSearchAttrib::kNumMsgSearchAttributes; j++) 
+  for (PRUint32 j=maxHdrs; j < nsMsgSearchAttrib::kNumMsgSearchAttributes; j++) 
   {
     for (PRUint32 k=0; k < nsMsgSearchOp::kNumMsgSearchOperators; k++) 
     {
@@ -1117,3 +1146,114 @@ nsMsgSearchValidityManager::InitOtherHeadersInTable (nsIMsgSearchValidityTable *
   }
   return NS_OK;
 }
+
+
+nsresult nsMsgSearchValidityManager::EnableDirectoryAttribute(nsIMsgSearchValidityTable *table, nsMsgSearchAttribValue aSearchAttrib)
+{
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::Contains, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::Contains, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::DoesntContain, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::DoesntContain, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::Is, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::Is, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::Isnt, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::Isnt, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::BeginsWith, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::BeginsWith, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::EndsWith, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::EndsWith, 1);
+        table->SetAvailable (aSearchAttrib, nsMsgSearchOp::SoundsLike, 1);
+        table->SetEnabled   (aSearchAttrib, nsMsgSearchOp::SoundsLike, 1);
+        return NS_OK;
+}
+
+nsresult nsMsgSearchValidityManager::InitLdapTable()
+{
+  NS_ASSERTION(!m_ldapTable,"don't call this twice!");
+
+	nsresult rv = NewTable (getter_AddRefs(m_ldapTable));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = m_ldapTable->SetDefaultAttrib(nsMsgSearchAttrib::Name);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+	rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::Name);
+	NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::Email);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::PhoneNumber);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::Organization);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::Department);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::City);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_ldapTable, nsMsgSearchAttrib::Street);
+  NS_ENSURE_SUCCESS(rv,rv);
+  return rv;
+}
+
+nsresult nsMsgSearchValidityManager::InitLocalABTable()
+{
+  NS_ASSERTION(!m_localABTable,"don't call this twice!");
+
+	nsresult rv = NewTable (getter_AddRefs(m_localABTable));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = m_localABTable->SetDefaultAttrib(nsMsgSearchAttrib::Name);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+	rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Name);
+	NS_ENSURE_SUCCESS(rv,rv);
+ 
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Email);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Nickname);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::WorkPhone);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::HomePhone);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Fax);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Pager);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Mobile);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  //this needs to be handled in a generic way.
+  //we need to get the string generically, and then map it to _AimScreenName
+  //I think we're going to have to add custom headers to AB search as a pref
+  //and let the ns tree define one for screen name.
+  //rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::ScreenName);
+  //NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Title);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Organization);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::Department);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = EnableDirectoryAttribute(m_localABTable, nsMsgSearchAttrib::AdditionalEmail);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  return rv;
+}
+
+
