@@ -48,6 +48,7 @@
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "nsIFileSpec.h"
+#include "nsILocalFile.h"
 #ifdef XP_MAC
 #include "nsIAppleFileDecoder.h"
 #endif
@@ -69,8 +70,11 @@
 // gecko
 #include "nsLayoutCID.h"
 #include "nsIMarkupDocumentViewer.h"
-#include "nsIContentViewerFile.h"
 #include "nsIContentViewer.h" 
+#include "nsIWebShell.h" 
+
+// embedding
+#include "nsIWebBrowserPrint.h"
 
 /* for access to docshell */
 #include "nsIDOMWindowInternal.h"
@@ -123,9 +127,9 @@
 #include "nsIStringBundle.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
-
-// Find / Find Again 
-#include "nsIFindComponent.h"
+#include "nsIPrefBranchInternal.h"
+#include "nsCExternalHandlerService.h"
+#include "nsIMIMEService.h"
 
 static NS_DEFINE_CID(kIStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
 static NS_DEFINE_CID(kCMsgMailSessionCID, NS_MSGMAILSESSION_CID); 
@@ -144,6 +148,8 @@ static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
 
 #define FOUR_K 4096
 #define MESSENGER_SAVE_DIR_PREF_NAME "messenger.save.dir"
+#define MAILNEWS_ALLOW_PLUGINS_PREF_NAME "mailnews.message_display.allow.plugins"
+
 //
 // Convert an nsString buffer to plain text...
 //
@@ -181,8 +187,7 @@ ConvertBufToPlainText(nsString &aConBuf)
 
     parser->SetContentSink(sink);
 
-    nsAutoString mimeStr(NS_LITERAL_STRING("text/html").get());
-    parser->Parse(aConBuf, 0, mimeStr, PR_FALSE, PR_TRUE);
+    parser->Parse(aConBuf, 0, NS_LITERAL_CSTRING("text/html"), PR_FALSE, PR_TRUE);
 
     //
     // Now if we get here, we need to get from ASCII text to 
@@ -197,6 +202,40 @@ ConvertBufToPlainText(nsString &aConBuf)
   return rv;
 }
 
+nsresult ConvertAndSanitizeFileName(const char * displayName, PRUnichar ** unicodeResult, char ** result)
+{
+  nsCAutoString unescapedName(displayName);
+
+  /* we need to convert the UTF-8 fileName to platform specific character set.
+     The display name is in UTF-8 because it has been escaped from JS
+  */ 
+  NS_UnescapeURL(unescapedName);
+  NS_ConvertUTF8toUCS2 ucs2Str(unescapedName);
+
+  nsresult rv = NS_OK;
+#if defined(XP_MAC)
+  /* We need to truncate the name to 31 characters, this even on MacOS X until the file API
+     correctly support long file name. Using a nsILocalFile will do the trick...
+  */
+  nsCOMPtr<nsILocalFile> aLocalFile(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID, &rv));
+  if (NS_SUCCEEDED(aLocalFile->SetLeafName(ucs2Str)))
+  {
+    aLocalFile->GetLeafName(ucs2Str);
+  }
+#endif
+
+  // replace platform specific path separator and illegale characters to avoid any confusion
+  ucs2Str.ReplaceChar(FILE_PATH_SEPARATOR FILE_ILLEGAL_CHARACTERS, '-');
+
+  if (result)
+    rv = ConvertFromUnicode(nsMsgI18NFileSystemCharset(), ucs2Str, result);
+
+  if (unicodeResult)
+    *unicodeResult = ToNewUnicode(ucs2Str);
+
+ return rv;
+}
+
 // ***************************************************
 // jefft - this is a rather obscured class serves for Save Message As File,
 // Save Message As Template, and Save Attachment to a file
@@ -260,12 +299,12 @@ public:
 //
 nsMessenger::nsMessenger() 
 {
-	NS_INIT_REFCNT();
-	mScriptObject = nsnull;
-	mWindow = nsnull;
+  NS_INIT_REFCNT();
+  mScriptObject = nsnull;
+  mWindow = nsnull;
   mMsgWindow = nsnull;
   mStringBundle = nsnull;
-
+  mSendingUnsentMsgs = PR_FALSE;
   //	InitializeFolderRoot();
 }
 
@@ -278,13 +317,26 @@ nsMessenger::~nsMessenger()
 }
 
 
-NS_IMPL_ISUPPORTS1(nsMessenger, nsIMessenger)
+NS_IMPL_ISUPPORTS3(nsMessenger, nsIMessenger, nsIObserver, nsISupportsWeakReference)
+NS_IMPL_GETSET(nsMessenger, SendingUnsentMsgs, PRBool, mSendingUnsentMsgs);
 
 NS_IMETHODIMP    
 nsMessenger::SetWindow(nsIDOMWindowInternal *aWin, nsIMsgWindow *aMsgWindow)
 {
+  nsCOMPtr<nsIPrefBranchInternal> pbi;
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  if (prefs)
+  {
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    if (prefBranch)
+      pbi = do_QueryInterface(prefBranch);
+  }
 	if(!aWin)
 	{
+    // it isn't an error to pass in null for aWin, in fact it means we are shutting
+    // down and we should start cleaning things up...
+
 		if (mMsgWindow)
 		{
 			nsCOMPtr<nsIMsgStatusFeedback> aStatusFeedback;
@@ -292,9 +344,12 @@ nsMessenger::SetWindow(nsIDOMWindowInternal *aWin, nsIMsgWindow *aMsgWindow)
 			mMsgWindow->GetStatusFeedback(getter_AddRefs(aStatusFeedback));
 			if (aStatusFeedback)
 				aStatusFeedback->SetDocShell(nsnull, nsnull);
+
+      // Remove pref observer
+      if (pbi)
+        pbi->RemoveObserver(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, this);
 		}
-    // it isn't an error to pass in null for aWin, in fact it means we are shutting
-    // down and we should start cleaning things up...
+
 		return NS_OK;
 	}
 
@@ -336,6 +391,11 @@ nsMessenger::SetWindow(nsIDOMWindowInternal *aWin, nsIMsgWindow *aMsgWindow)
                 aStatusFeedback->SetDocShell(mDocShell, mWindow);
             }
             aMsgWindow->GetTransactionManager(getter_AddRefs(mTxnMgr));
+
+            // Add pref observer
+            if (pbi)
+              pbi->AddObserver(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, this, PR_TRUE);
+            SetDisplayProperties();
         }
     }
   }
@@ -369,6 +429,42 @@ NS_IMETHODIMP nsMessenger::SetDisplayCharset(const PRUnichar * aCharset)
   return NS_OK;
 }
 
+nsresult
+nsMessenger::SetDisplayProperties()
+{
+  // For now, the only property we will set is allowPlugins but we might do more in the future...
+
+  nsresult rv;
+
+  if (!mDocShell)
+    return NS_ERROR_FAILURE;
+ 
+  PRBool allowPlugins = PR_FALSE;
+
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  if (NS_SUCCEEDED(rv))
+  {
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    if (NS_SUCCEEDED(rv))
+      (void)prefBranch->GetBoolPref(MAILNEWS_ALLOW_PLUGINS_PREF_NAME, &allowPlugins);
+  }
+  
+  return mDocShell->SetAllowPlugins(allowPlugins);
+}
+
+NS_IMETHODIMP
+nsMessenger::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData)
+{
+  if (!nsCRT::strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID))
+  {
+    nsDependentString prefName(aData);
+    if (prefName.Equals(NS_LITERAL_STRING(MAILNEWS_ALLOW_PLUGINS_PREF_NAME)))
+      SetDisplayProperties();
+  }
+
+  return NS_OK;
+}
 
 nsresult
 nsMessenger::PromptIfFileExists(nsFileSpec &fileSpec)
@@ -407,8 +503,7 @@ nsMessenger::PromptIfFileExists(nsFileSpec &fileSpec)
             nsCOMPtr<nsIFilePicker> filePicker =
                 do_CreateInstance("@mozilla.org/filepicker;1", &rv);
             if (NS_FAILED(rv)) return rv;
-            // XXX i18n fix me
-            filePicker->Init(nsnull, NS_LITERAL_STRING("Save Attachment").get(), nsIFilePicker::modeSave);
+            filePicker->Init(nsnull, GetString(NS_LITERAL_STRING("SaveAttachment").get()), nsIFilePicker::modeSave);
             filePicker->SetDefaultString(path.get());
             filePicker->AppendFilters(nsIFilePicker::filterAll);
             
@@ -423,7 +518,7 @@ nsMessenger::PromptIfFileExists(nsFileSpec &fileSpec)
                 return NS_ERROR_FAILURE;
 
             nsCOMPtr<nsILocalFile> localFile;
-            nsXPIDLCString filePath;
+            nsCAutoString filePath;
             
             rv = filePicker->GetFile(getter_AddRefs(localFile));
             if (NS_FAILED(rv)) return rv;
@@ -431,9 +526,10 @@ nsMessenger::PromptIfFileExists(nsFileSpec &fileSpec)
             rv = SetLastSaveDirectory(localFile);
             NS_ENSURE_SUCCESS(rv,rv);
 
-            rv = localFile->GetPath(getter_Copies(filePath));
+            rv = localFile->GetNativePath(filePath);
             if (NS_FAILED(rv)) return rv;
-            fileSpec = (const char*) filePath;
+
+            fileSpec = filePath.get();
             return NS_OK;
         }
     }
@@ -444,76 +540,6 @@ nsMessenger::PromptIfFileExists(nsFileSpec &fileSpec)
     return rv;
 }
 
-nsresult
-nsMessenger::InitializeSearch( nsIFindComponent *finder )
-{
-    nsresult rv = NS_OK;
-    if (!finder) return NS_ERROR_NULL_POINTER;
-
-    if (!mSearchContext )
-    {
-        nsCOMPtr<nsIInterfaceRequestor> docShellIR = do_QueryInterface(mDocShell);
-        if (!docShellIR) return NS_ERROR_FAILURE;
-        
-        nsCOMPtr<nsIDOMWindowInternal> domWindow;
-        docShellIR->GetInterface(NS_GET_IID(nsIDOMWindowInternal), getter_AddRefs(domWindow));
-        if (!domWindow) return NS_ERROR_FAILURE;
-        
-        // we need to get the nsIDOMWindowInternal for mWebShell
-        // Create the search context for this browser window.
-        rv = finder->CreateContext(domWindow, nsnull, getter_AddRefs(mSearchContext));
-    }
-
-    return rv;
-}
-
-
-NS_IMETHODIMP
-nsMessenger::Find()
-{
-    nsresult rv = NS_OK;
-    PRBool   found = PR_FALSE;
-
-    // Get find component.
-    nsCOMPtr <nsIFindComponent> finder = do_GetService(NS_IFINDCOMPONENT_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    if (!finder) return NS_ERROR_FAILURE;
-
-    // Make sure we've initialized searching for this document.
-    rv = InitializeSearch( finder );
-    if (NS_FAILED(rv)) return rv;
-
-    // Perform find via find component.
-    if (mSearchContext) {
-            rv = finder->Find( mSearchContext, &found );
-    }
-
-    return rv;
-}
-
-NS_IMETHODIMP
-nsMessenger::FindAgain()
-{
-    nsresult rv = NS_OK;
-    PRBool   found = PR_FALSE;
-
-    // Get find component.
-    nsCOMPtr <nsIFindComponent> finder = do_GetService(NS_IFINDCOMPONENT_CONTRACTID, &rv);
-    if (NS_FAILED(rv)) return rv;
-    if (!finder) return NS_ERROR_FAILURE;
-
-    // Make sure we've initialized searching for this document.
-    rv = InitializeSearch( finder );
-    if (NS_FAILED(rv)) return rv;
-
-    // Perform find via find component.
-    if (mSearchContext) {
-            rv = finder->FindNext( mSearchContext, &found );
-    }
-
-    return rv;
-}
-
 NS_IMETHODIMP
 nsMessenger::OpenURL(const char * url)
 {
@@ -672,44 +698,41 @@ NS_IMETHODIMP
 nsMessenger::SaveAttachment(const char * contentType, const char * url,
                             const char * displayName, const char * messageUri)
 {
-  nsresult rv = NS_ERROR_OUT_OF_MEMORY;
-  char *unescapedUrl = nsnull;
-  nsCOMPtr<nsIFilePicker> filePicker =
-      do_CreateInstance("@mozilla.org/filepicker;1", &rv);
-  char * unescapedDisplayName = nsnull;
-  PRInt16 dialogResult;
-  nsCOMPtr<nsILocalFile> localFile;
-  nsCOMPtr <nsILocalFile> lastSaveDir;
-  nsCOMPtr<nsIFileSpec> fileSpec;
-  nsXPIDLCString filePath;
-
-  if (NS_FAILED(rv)) goto done;
-  if (!url) goto done;
+  NS_ENSURE_ARG_POINTER(url);
 
 #ifdef DEBUG_MESSENGER
   printf("nsMessenger::SaveAttachment(%s)\n",url);
 #endif    
+
+  nsresult rv = NS_ERROR_OUT_OF_MEMORY;
+  char *unescapedUrl = nsnull;
+  nsCOMPtr<nsIFilePicker> filePicker =
+      do_CreateInstance("@mozilla.org/filepicker;1", &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRInt16 dialogResult;
+  nsCOMPtr<nsILocalFile> localFile;
+  nsCOMPtr<nsILocalFile> lastSaveDir;
+  nsCOMPtr<nsIFileSpec> fileSpec;
+  nsXPIDLCString filePath;
+  nsXPIDLString defaultDisplayString;
+
   unescapedUrl = PL_strdup(url);
-  if (!unescapedUrl) goto done;
+  if (!unescapedUrl)
+    return NS_ERROR_OUT_OF_MEMORY;
 
   nsUnescape(unescapedUrl);
   
-  unescapedDisplayName = nsCRT::strdup(displayName);
-  if (!unescapedDisplayName) goto done;
-  nsUnescape(unescapedDisplayName);
-    
-  /* we need to convert the UTF-8 fileName to platform specific character set.
-     The display name is in UTF-8 because it has been escaped from JS
-  */ 
-  
+  rv = ConvertAndSanitizeFileName(displayName, getter_Copies(defaultDisplayString), nsnull);
+  if (NS_FAILED(rv)) goto done;
+
   filePicker->Init(
       nsnull, 
-      GetString(NS_LITERAL_STRING("Save Attachment").get()),
+      GetString(NS_LITERAL_STRING("SaveAttachment").get()),
       nsIFilePicker::modeSave
       );
-  filePicker->SetDefaultString(NS_ConvertUTF8toUCS2(unescapedDisplayName).get());
+  filePicker->SetDefaultString(defaultDisplayString.get());
   filePicker->AppendFilters(nsIFilePicker::filterAll);
-  nsCRT::free(unescapedDisplayName);
   
   rv = GetLastSaveDirectory(getter_AddRefs(lastSaveDir));
   if (NS_SUCCEEDED(rv) && lastSaveDir) {
@@ -718,24 +741,24 @@ nsMessenger::SaveAttachment(const char * contentType, const char * url,
 
   filePicker->Show(&dialogResult);
   if (dialogResult == nsIFilePicker::returnCancel)
-      goto done;
+  {
+    rv = NS_OK;
+    goto done;
+  }
 
   rv = filePicker->GetFile(getter_AddRefs(localFile));
   if (NS_FAILED(rv)) goto done;
   
-  rv = SetLastSaveDirectory(localFile);
-  if (NS_FAILED(rv)) 
-    goto done;
-  
-  rv = localFile->GetPath(getter_Copies(filePath));
-  fileSpec = do_CreateInstance("@mozilla.org/filespec;1", &rv);
+  (void)SetLastSaveDirectory(localFile);
+
+  rv = NS_NewFileSpecFromIFile(localFile, getter_AddRefs(fileSpec));
   if (NS_FAILED(rv)) goto done;
-  fileSpec->SetNativePath(filePath);
+
   rv = SaveAttachment(fileSpec, unescapedUrl, messageUri, contentType, nsnull);
 
 done:
-    PR_FREEIF(unescapedUrl);
-    return rv;
+  PR_FREEIF(unescapedUrl);
+  return rv;
 }
 
 
@@ -753,14 +776,14 @@ nsMessenger::SaveAllAttachments(PRUint32 count,
     nsCOMPtr<nsILocalFile> lastSaveDir;
     nsCOMPtr<nsIFileSpec> fileSpec;
     nsXPIDLCString dirName;
-    char *unescapedUrl = nsnull, *unescapedName = nsnull, *tempCStr = nsnull;
+    char *unescapedUrl = nsnull, *unescapedName = nsnull;
     nsSaveAllAttachmentsState *saveState = nsnull;
     PRInt16 dialogResult;
 
     if (NS_FAILED(rv)) goto done;
     filePicker->Init(
         nsnull, 
-        GetString(NS_LITERAL_STRING("Save All Attachments").get()),
+        GetString(NS_LITERAL_STRING("SaveAllAttachments").get()),
         nsIFilePicker::modeGetFolder
         );
 
@@ -779,8 +802,8 @@ nsMessenger::SaveAllAttachments(PRUint32 count,
     rv = SetLastSaveDirectory(localFile);
     if (NS_FAILED(rv)) 
       goto done;
-    
-    rv = localFile->GetPath(getter_Copies(dirName));
+
+    rv = localFile->GetNativePath(dirName);
     if (NS_FAILED(rv)) goto done;
     rv = NS_NewFileSpec(getter_AddRefs(fileSpec));
     if (NS_FAILED(rv)) goto done;
@@ -795,13 +818,11 @@ nsMessenger::SaveAllAttachments(PRUint32 count,
         nsFileSpec aFileSpec((const char *) dirName);
         unescapedUrl = PL_strdup(urlArray[0]);
         nsUnescape(unescapedUrl);
-        unescapedName = PL_strdup(displayNameArray[0]);
-        nsUnescape(unescapedName);
-        rv = ConvertFromUnicode(nsMsgI18NFileSystemCharset(), NS_ConvertUTF8toUCS2(unescapedName),
-                                &tempCStr);
-        if (NS_FAILED(rv)) goto done;
-        PR_FREEIF(unescapedName);
-        unescapedName = tempCStr;
+
+        rv = ConvertAndSanitizeFileName(displayNameArray[0], nsnull, &unescapedName);
+        if (NS_FAILED(rv))
+          goto done;
+
         aFileSpec += unescapedName;
         rv = PromptIfFileExists(aFileSpec);
         if (NS_FAILED(rv)) return rv;
@@ -879,10 +900,9 @@ nsMessenger::SaveAs(const char* url, PRBool asFile, nsIMsgIdentity* identity, ns
         if (NS_FAILED(rv)) 
           goto done;
         
-        nsXPIDLCString tmpFileName;
-        rv = localFile->GetLeafName(getter_Copies(tmpFileName));
+        nsAutoString fileName;
+        rv = localFile->GetLeafName(fileName);
         if (NS_FAILED(rv)) goto done;
-        nsCAutoString fileName(tmpFileName);
             
         // First, check if they put ANY extension on the file, if not,
         // then we should look at the type of file they have chosen and
@@ -904,11 +924,9 @@ nsMessenger::SaveAs(const char* url, PRBool asFile, nsIMsgIdentity* identity, ns
         // XXX argh!  converting from nsILocalFile to nsFileSpec ... oh baby, lets drop from unicode to ascii too
         //        nsXPIDLString path;
         //        localFile->GetUnicodePath(getter_Copies(path));
-        nsXPIDLCString path;
-        localFile->GetPath(getter_Copies(path));
-        nsCOMPtr<nsIFileSpec> fileSpec = do_CreateInstance("@mozilla.org/filespec;1", &rv);
+        nsCOMPtr<nsIFileSpec> fileSpec;
+        rv = NS_NewFileSpecFromIFile(localFile, getter_AddRefs(fileSpec));
         if (NS_FAILED(rv)) goto done;
-        fileSpec->SetNativePath(path);
 
         aListener = new nsSaveMsgListener(fileSpec, this);
         if (!aListener) {
@@ -925,7 +943,7 @@ nsMessenger::SaveAs(const char* url, PRBool asFile, nsIMsgIdentity* identity, ns
         switch (saveAsFileType) {
         case 0:
         default:
-            rv = messageService->SaveMessageToDisk(url, fileSpec, PR_TRUE,
+            rv = messageService->SaveMessageToDisk(url, fileSpec, PR_FALSE,
                                                    urlListener, nsnull,
                                                    PR_FALSE, mMsgWindow);
             break;
@@ -950,9 +968,10 @@ nsMessenger::SaveAs(const char* url, PRBool asFile, nsIMsgIdentity* identity, ns
             aListener->m_channel = nsnull;
             rv = NS_NewInputStreamChannel(getter_AddRefs(aListener->m_channel),
                                           aURL, 
-                                          nsnull,      // inputStream
-                                          nsnull,      // contentType
-                                          -1);         // contentLength
+                                          nsnull,                 // inputStream
+                                          NS_LITERAL_CSTRING(""), // contentType
+                                          NS_LITERAL_CSTRING(""), // contentCharset
+                                          -1);                    // contentLength
             if (NS_FAILED(rv)) goto done;
 
             aListener->m_outputFormat.AssignWithConversion(saveAsFileType == 1 ? TEXT_HTML : TEXT_PLAIN);
@@ -1383,7 +1402,7 @@ NS_IMETHODIMP nsMessenger::SetDocumentCharset(const PRUnichar *characterSet)
 class SendLaterListener: public nsIMsgSendLaterListener
 {
 public:
-  SendLaterListener(void);
+  SendLaterListener(nsIMessenger *);
   virtual ~SendLaterListener(void);
 
   // nsISupports interface
@@ -1400,22 +1419,34 @@ public:
 
   /* void OnStopSending (in nsresult aStatus, in wstring aMsg, in PRUint32 aTotalTried, in PRUint32 aSuccessful); */
   NS_IMETHOD OnStopSending(nsresult aStatus, const PRUnichar *aMsg, PRUint32 aTotalTried, PRUint32 aSuccessful);
+protected:
+  nsWeakPtr m_messenger; 
 };
 
 NS_IMPL_ISUPPORTS1(SendLaterListener, nsIMsgSendLaterListener)
 
-SendLaterListener::SendLaterListener()
+SendLaterListener::SendLaterListener(nsIMessenger *aMessenger)
 {
+  m_messenger = getter_AddRefs(NS_GetWeakReference(aMessenger));
   NS_INIT_REFCNT();
 }
 
 SendLaterListener::~SendLaterListener()
 {
+  nsCOMPtr <nsIMessenger> messenger = do_QueryReferent(m_messenger);
+  // best to be defensive about this, in case OnStopSending doesn't get called.
+  if (messenger)
+    messenger->SetSendingUnsentMsgs(PR_FALSE);
+  m_messenger = nsnull;
 }
 
 nsresult
 SendLaterListener::OnStartSending(PRUint32 aTotalMessageCount)
 {
+  // this never gets called :-(
+  nsCOMPtr <nsIMessenger> messenger = do_QueryReferent(m_messenger);
+  if (messenger)
+    messenger->SetSendingUnsentMsgs(PR_TRUE);
   return NS_OK;
 }
 
@@ -1441,33 +1472,38 @@ SendLaterListener::OnStopSending(nsresult aStatus, const PRUnichar *aMsg, PRUint
             aTotalTried, aSuccessful);
 #endif
 
+  nsCOMPtr <nsIMessenger> messenger = do_QueryReferent(m_messenger);
+  if (messenger)
+    messenger->SetSendingUnsentMsgs(PR_FALSE);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsMessenger::SendUnsentMessages(nsIMsgIdentity *aIdentity)
+nsMessenger::SendUnsentMessages(nsIMsgIdentity *aIdentity, nsIMsgWindow *aMsgWindow)
 {
-	nsresult rv;
-	nsCOMPtr<nsIMsgSendLater> pMsgSendLater; 
-	rv = nsComponentManager::CreateInstance(kMsgSendLaterCID, NULL,NS_GET_IID(nsIMsgSendLater),
-																					(void **)getter_AddRefs(pMsgSendLater)); 
-	if (NS_SUCCEEDED(rv) && pMsgSendLater) 
-	{ 
+    nsresult rv;
+    nsCOMPtr<nsIMsgSendLater> pMsgSendLater; 
+    rv = nsComponentManager::CreateInstance(kMsgSendLaterCID, NULL,NS_GET_IID(nsIMsgSendLater),
+																				    (void **)getter_AddRefs(pMsgSendLater)); 
+    if (NS_SUCCEEDED(rv) && pMsgSendLater) 
+    { 
 #ifdef DEBUG
-		printf("We succesfully obtained a nsIMsgSendLater interface....\n"); 
+        printf("We succesfully obtained a nsIMsgSendLater interface....\n"); 
 #endif
 
-    SendLaterListener *sendLaterListener = new SendLaterListener();
-    if (!sendLaterListener)
-        return NS_ERROR_FAILURE;
+      SendLaterListener *sendLaterListener = new SendLaterListener(this);
+      if (!sendLaterListener)
+          return NS_ERROR_OUT_OF_MEMORY;
 
-    NS_ADDREF(sendLaterListener);
-    pMsgSendLater->AddListener(sendLaterListener);
+      NS_ADDREF(sendLaterListener);
+      pMsgSendLater->AddListener(sendLaterListener);
+      pMsgSendLater->SetMsgWindow(aMsgWindow);
+      mSendingUnsentMsgs = PR_TRUE;
 
-    pMsgSendLater->SendUnsentMessages(aIdentity); 
-    NS_RELEASE(sendLaterListener);
-	} 
-	return NS_OK;
+      pMsgSendLater->SendUnsentMessages(aIdentity); 
+      NS_RELEASE(sendLaterListener);
+    } 
+    return NS_OK;
 }
 
 NS_IMETHODIMP nsMessenger::DoPrint()
@@ -1485,9 +1521,11 @@ NS_IMETHODIMP nsMessenger::DoPrint()
 
   if (viewer) 
   {
-    nsCOMPtr<nsIContentViewerFile> viewerFile = do_QueryInterface(viewer);
-    if (viewerFile) {
-      rv = viewerFile->Print(PR_FALSE, nsnull, (nsIWebProgressListener*)nsnull);
+    nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint = do_QueryInterface(viewer);
+    if (webBrowserPrint) {
+      nsCOMPtr<nsIPrintSettings> printSettings;
+      webBrowserPrint->GetGlobalPrintSettings(getter_AddRefs(printSettings));
+      rv = webBrowserPrint->Print(printSettings, (nsIWebProgressListener*)nsnull);
     }
 #ifdef DEBUG_MESSENGER
     else {
@@ -1643,15 +1681,17 @@ nsSaveMsgListener::OnStartRequest(nsIRequest* request, nsISupports* aSupport)
     }
     
 #ifdef XP_MAC
-  /* On Mac, if we are saving an appledouble or applesingle attachment, we need to use an Apple File Decoder */
   if (!m_contentType.IsEmpty())
+  {
+    nsFileSpec realSpec;
+    m_fileSpec->GetFileSpec(&realSpec);
+
+  	/* if we are saving an appledouble or applesingle attachment, we need to use an Apple File Decoder */
     if ((nsCRT::strcasecmp(m_contentType.get(), APPLICATION_APPLEFILE) == 0) ||
         (nsCRT::strcasecmp(m_contentType.get(), MULTIPART_APPLEDOUBLE) == 0))
     {        
       /* ggrrrrr, I have a nsFileSpec but I need a nsILocalFile... */
       nsCOMPtr<nsILocalFile> outputFile;
-      nsFileSpec realSpec;
-      m_fileSpec->GetFileSpec(&realSpec);
       NS_FileSpecToIFile(&realSpec, getter_AddRefs(outputFile));
       
       nsCOMPtr<nsIAppleFileDecoder> appleFileDecoder = do_CreateInstance(NS_IAPPLEFILEDECODER_CONTRACTID, &rv);
@@ -1662,6 +1702,24 @@ nsSaveMsgListener::OnStartRequest(nsIRequest* request, nsISupports* aSupport)
           m_outputStream = do_QueryInterface(appleFileDecoder, &rv);
       }
     }
+    else
+    {
+      /* we need to set the correct application type and creator base on the content-type */
+      nsCOMPtr<nsIMIMEService> mimeService (do_GetService(NS_MIMESERVICE_CONTRACTID));
+      if (mimeService)
+      {
+        nsCOMPtr<nsIMIMEInfo> mimeinfo;
+        if (NS_SUCCEEDED(mimeService->GetFromMIMEType(m_contentType.get(), getter_AddRefs(mimeinfo))))
+        {
+          PRUint32 aMacType;
+          PRUint32 aMacCreator;
+          if (NS_SUCCEEDED(mimeinfo->GetMacType(&aMacType)) && NS_SUCCEEDED(mimeinfo->GetMacCreator(&aMacCreator)))
+            realSpec.SetFileTypeAndCreator((OSType)aMacType, (OSType)aMacCreator);
+        }
+      }
+      
+    }
+  }
 #endif
 
     return rv;
@@ -1690,7 +1748,7 @@ nsSaveMsgListener::OnStopRequest(nsIRequest* request, nsISupports* aSupport,
       rv = nsMsgI18NSaveAsCharset(TEXT_PLAIN, nsMsgI18NFileSystemCharset(), 
                                   m_msgBuffer.get(), &conBuf); 
       if ( NS_SUCCEEDED(rv) && (conBuf) )
-        conLength = nsCRT::strlen(conBuf);
+        conLength = strlen(conBuf);
     }
 
     if ( (NS_SUCCEEDED(rv)) && (conBuf) )
@@ -1718,8 +1776,7 @@ nsSaveMsgListener::OnStopRequest(nsIRequest* request, nsISupports* aSupport,
       if (m_saveAllAttachmentsState->m_curIndex <
           m_saveAllAttachmentsState->m_count)
       {
-          char * unescapedUrl = nsnull, * unescapedName = nsnull, 
-               * tempCStr = nsnull;
+          char * unescapedUrl = nsnull, * unescapedName = nsnull;
           nsSaveAllAttachmentsState *state = m_saveAllAttachmentsState;
           PRUint32 i = state->m_curIndex;
           nsCOMPtr<nsIFileSpec> fileSpec;
@@ -1729,13 +1786,11 @@ nsSaveMsgListener::OnStopRequest(nsIRequest* request, nsISupports* aSupport,
           if (NS_FAILED(rv)) goto done;
           unescapedUrl = PL_strdup(state->m_urlArray[i]);
           nsUnescape(unescapedUrl);
-          unescapedName = PL_strdup(state->m_displayNameArray[i]);
-          nsUnescape(unescapedName);
-          rv = ConvertFromUnicode(nsMsgI18NFileSystemCharset(), NS_ConvertUTF8toUCS2(unescapedName),
-                                  &tempCStr);
-          if (NS_FAILED(rv)) goto done;
-          PR_FREEIF(unescapedName);
-          unescapedName = tempCStr;
+ 
+          rv = ConvertAndSanitizeFileName(state->m_displayNameArray[i], nsnull, &unescapedName);
+          if (NS_FAILED(rv))
+            goto done;
+
           aFileSpec += unescapedName;
           rv = m_messenger->PromptIfFileExists(aFileSpec);
           if (NS_FAILED(rv)) goto done;
@@ -1781,7 +1836,7 @@ nsSaveMsgListener::OnDataAvailable(nsIRequest* request,
     {
       if (maxReadCount > available)
         maxReadCount = available;
-      nsCRT::memset(m_dataBuffer, 0, FOUR_K+1);
+      memset(m_dataBuffer, 0, FOUR_K+1);
       rv = inStream->Read(m_dataBuffer, maxReadCount, &readCount);
 
       // rhp:
