@@ -62,6 +62,7 @@
 #include "nsFileStream.h"
 #include "nsMsgUtils.h"
 #include "nsIFileSpec.h" 
+#include "nsILocalFile.h"
 #include "nsIURL.h"
 #include "nsNetCID.h"
 #include "nsISmtpService.h"
@@ -89,7 +90,11 @@
 #define ACCOUNT_PREFIX "account"
 #define SERVER_PREFIX "server"
 #define ID_PREFIX "id"
-#define OFFLINE_STATUS_CHANGED_TOPIC "network:offline-status-changed"
+#define ABOUT_TO_GO_OFFLINE_TOPIC "network:offline-about-to-go-offline"
+#define ACCOUNT_DELIMITER ","
+#define APPEND_ACCOUNTS_VERSION_PREF_NAME "append_preconfig_accounts.version"
+#define MAILNEWS_ROOT_PREF "mailnews."
+#define PREF_MAIL_ACCOUNTMANAGER_APPEND_ACCOUNTS "mail.accountmanager.appendaccounts"
 
 static NS_DEFINE_CID(kMsgAccountCID, NS_MSGACCOUNT_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
@@ -164,7 +169,7 @@ nsMsgAccountManager::~nsMsgAccountManager()
       if (NS_SUCCEEDED(rv))
 	  {    
       observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
-      observerService->RemoveObserver(this, "network:offline-status-changed");
+      observerService->RemoveObserver(this, ABOUT_TO_GO_OFFLINE_TOPIC);
     }
   }
 }
@@ -187,7 +192,7 @@ nsresult nsMsgAccountManager::Init()
   {    
     observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
     observerService->AddObserver(this, "quit-application" , PR_TRUE);
-    observerService->AddObserver(this, "network:offline-status-changed", PR_TRUE);
+    observerService->AddObserver(this, ABOUT_TO_GO_OFFLINE_TOPIC, PR_TRUE);
     observerService->AddObserver(this, "session-logout", PR_TRUE);
     observerService->AddObserver(this, "profile-before-change", PR_TRUE);
   }
@@ -203,13 +208,15 @@ nsresult nsMsgAccountManager::Shutdown()
   {
     WriteToFolderCache(m_msgFolderCache);
   }
-  nsCOMPtr<nsIMsgBiffManager> biffService = do_GetService(NS_MSGBIFFMANAGER_CONTRACTID, &rv);
-  if (NS_SUCCEEDED(rv) && biffService)
-    biffService->Shutdown();
 
   CloseCachedConnections();
   UnloadAccounts();
-
+  
+  //shutdown removes nsIIncomingServer listener from biff manager, so do it after accounts have been unloaded
+  nsCOMPtr<nsIMsgBiffManager> biffService = do_GetService(NS_MSGBIFFMANAGER_CONTRACTID, &rv);
+  if (NS_SUCCEEDED(rv) && biffService)
+    biffService->Shutdown();
+  
   if (m_prefs) {
     nsServiceManager::ReleaseService(kPrefServiceCID, m_prefs);
     m_prefs = 0;
@@ -243,7 +250,7 @@ NS_IMETHODIMP nsMsgAccountManager::Observe(nsISupports *aSubject, const char *aT
     return NS_OK;
   }
   
-  if (!nsCRT::strcmp(aTopic,"network:offline-status-changed"))
+  if (!nsCRT::strcmp(aTopic, ABOUT_TO_GO_OFFLINE_TOPIC))
   {
     nsAutoString dataString(NS_LITERAL_STRING("offline"));
     if (someData)
@@ -610,6 +617,8 @@ nsMsgAccountManager::RemoveAccount(nsIMsgAccount *aAccount)
     rv = server->GetKey(getter_Copies(serverKey));
     NS_ENSURE_SUCCESS(rv,rv);
 
+    LogoutOfServer(server); // close cached connections and forget session password
+
     // invalidate the FindServer() cache if we are removing the cached server
     if (m_lastFindServerResult) {
         nsXPIDLCString cachedServerKey;
@@ -656,12 +665,12 @@ nsMsgAccountManager::RemoveAccount(nsIMsgAccount *aAccount)
     rv = server->RemoveFiles();
     NS_ASSERTION(NS_SUCCEEDED(rv), "failed to remove the files associated with server");
 
-    rv = server->ForgetPassword();
-    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to remove the password associated with server");
     
     // now clear out the server once and for all.
     // watch out! could be scary
     server->ClearAllValues();
+
+    rootFolder->Shutdown(PR_TRUE);
   }
   nsCOMPtr<nsISupportsArray> identityArray;
   
@@ -892,7 +901,6 @@ nsMsgAccountManager::hashUnloadServer(nsHashKey *aKey, void *aData,
     if (NS_FAILED(rv)) return PR_TRUE;
     
 	nsMsgAccountManager *accountManager = (nsMsgAccountManager*)closure;
-
 	accountManager->NotifyServerUnloaded(server);
 
 	nsCOMPtr<nsIFolder> rootFolder;
@@ -908,18 +916,23 @@ nsMsgAccountManager::hashUnloadServer(nsHashKey *aKey, void *aData,
 
 }
 
-PRBool
+/* static */ void nsMsgAccountManager::LogoutOfServer(nsIMsgIncomingServer *aServer)
+{
+    nsresult rv = aServer->CloseCachedConnections();
+    NS_ASSERTION(NS_SUCCEEDED(rv), "CloseCachedConnections failed");
+    rv = aServer->ForgetSessionPassword();
+    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to remove the password associated with server");
+}
+
+/* static */ PRBool
 nsMsgAccountManager::hashLogoutOfServer(nsHashKey *aKey, void *aData,
                                                void *closure)
 {
     nsresult rv;
     nsCOMPtr<nsIMsgIncomingServer> server =
       do_QueryInterface((nsISupports*)aData, &rv);
-    if (NS_FAILED(rv)) return PR_TRUE;
-    
-    rv = server->CloseCachedConnections();
-    NS_ASSERTION(NS_SUCCEEDED(rv), "CloseCachedConnections failed");
-    server->ForgetSessionPassword();
+    if (NS_SUCCEEDED(rv))
+      LogoutOfServer(server);
 
     return PR_TRUE;
 }
@@ -947,12 +960,7 @@ nsresult nsMsgAccountManager::GetFolderCache(nsIMsgFolderCache* *aFolderCache)
     
     // TODO: Make nsIMsgFolderCache::Init take an nsIFile and
     // avoid this conversion.
-    nsXPIDLCString pathBuf;   
-    rv = cacheFile->GetPath(getter_Copies(pathBuf));
-    if (NS_FAILED(rv)) return rv;
-    rv = NS_NewFileSpec(getter_AddRefs(cacheFileSpec));
-    if (NS_FAILED(rv)) return rv;
-    rv = cacheFileSpec->SetNativePath(pathBuf);
+    rv = NS_NewFileSpecFromIFile(cacheFile, getter_AddRefs(cacheFileSpec));
     if (NS_FAILED(rv)) return rv;
                
     m_msgFolderCache->Init(cacheFileSpec);
@@ -982,7 +990,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
   nsIMsgIncomingServer *server = (nsIMsgIncomingServer*)aData;
   PRBool emptyTrashOnExit = PR_FALSE;
   PRBool cleanupInboxOnExit = PR_FALSE;
-  nsresult rv, rv1, rv2;
+  nsresult rv;
     
   server->GetEmptyTrashOnExit(&emptyTrashOnExit);
   nsCOMPtr <nsIImapIncomingServer> imapserver = do_QueryInterface(server);
@@ -1010,7 +1018,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
            server->GetPassword(getter_Copies(passwd));
          }
          if (!isImap || (isImap && (!serverRequiresPasswordForAuthentication || (passwd && 
-                        nsCRT::strlen((const char*) passwd)))))
+                        strlen((const char*) passwd)))))
          {
            nsCOMPtr<nsIUrlListener> urlListener;
            nsCOMPtr<nsIMsgAccountManager> accountManager = 
@@ -1035,12 +1043,12 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
              {  
                rv = aEnumerator->CurrentItem(getter_AddRefs(aSupport));
                nsCOMPtr<nsIMsgFolder>inboxFolder = do_QueryInterface(aSupport);
-               nsXPIDLString folderName;
-               inboxFolder->GetName(getter_Copies(folderName));
-               if (folderName && Compare(folderName, NS_LITERAL_STRING("INBOX"), nsCaseInsensitiveStringComparator()) ==0)
+               PRUint32 flags;
+               inboxFolder->GetFlags(&flags);
+               if (flags & MSG_FOLDER_FLAG_INBOX)
                {
-                 rv1 = inboxFolder->Compact(urlListener, nsnull /* msgwindow */);
-                 if (NS_SUCCEEDED(rv1))
+                 rv = inboxFolder->Compact(urlListener, nsnull /* msgwindow */);
+                 if (NS_SUCCEEDED(rv))
                    accountManager->SetFolderDoingCleanupInbox(inboxFolder);
                  break;
                 }
@@ -1051,15 +1059,15 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
                           
            if (emptyTrashOnExit)
            {
-             rv2 = folder->EmptyTrash(nsnull, urlListener);
-             if (isImap && NS_SUCCEEDED(rv2))
+             rv = folder->EmptyTrash(nsnull, urlListener);
+             if (isImap && NS_SUCCEEDED(rv))
                accountManager->SetFolderDoingEmptyTrash(folder);
            }
                     
            if (isImap && urlListener)
            {
              PRBool inProgress = PR_FALSE;
-             if (cleanupInboxOnExit && NS_SUCCEEDED(rv1))
+             if (cleanupInboxOnExit)
              {
                accountManager->GetCleanupInboxInProgress(&inProgress);
                while (inProgress)
@@ -1072,7 +1080,7 @@ PRBool PR_CALLBACK nsMsgAccountManager::cleanupOnExit(nsHashKey *aKey, void *aDa
                    eventQueue->ProcessPendingEvents();
                }
              }
-             if (emptyTrashOnExit && NS_SUCCEEDED(rv2))
+             if (emptyTrashOnExit)
              {
                accountManager->GetEmptyTrashInProgress(&inProgress);
                while (inProgress)
@@ -1109,6 +1117,7 @@ NS_IMETHODIMP
 nsMsgAccountManager::GetAccounts(nsISupportsArray **_retval)
 {
   nsresult rv;
+  
   rv = LoadAccounts();
   if (NS_FAILED(rv)) return rv;
   
@@ -1283,6 +1292,9 @@ nsMsgAccountManager::LoadAccounts()
   //Ensure biff service has started
   nsCOMPtr<nsIMsgBiffManager> biffService = 
            do_GetService(NS_MSGBIFFMANAGER_CONTRACTID, &rv);
+
+  if (NS_SUCCEEDED(rv))
+    biffService->Init();
   
   // Ensure messenger OS integration service has started
   // note, you can't expect the integrationService to be there
@@ -1296,9 +1308,84 @@ nsMsgAccountManager::LoadAccounts()
   if (NS_SUCCEEDED(rv)) {
     rv = m_prefs->CopyCharPref(PREF_MAIL_ACCOUNTMANAGER_ACCOUNTS,
                                         getter_Copies(accountList));
+    
+    /** 
+     * Check to see if we need to add pre-configured accounts.
+     * Following prefs are important to note in understanding the procedure here.
+     *
+     * 1. pref("mailnews.append_preconfig_accounts.version", version number);
+     * This pref registers the current version in the user prefs file. A default value 
+     * is stored in mailnews.js file. If a given vendor needs to add more preconfigured 
+     * accounts, the default version number can be increased. Comparing version 
+     * number from user's prefs file and the default one from mailnews.js, we
+     * can add new accounts and any other version level changes that need to be done.
+     *
+     * 2. pref("mail.accountmanager.appendaccounts", <comma separated account list>);
+     * This pref contains the list of pre-configured accounts that ISP/Vendor wants to
+     * to add to the existing accounts list. 
+     */
+    nsCOMPtr<nsIPrefBranch> defaultsPrefBranch;
+    rv = m_prefs->GetDefaultBranch(MAILNEWS_ROOT_PREF, getter_AddRefs(defaultsPrefBranch));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    rv = m_prefs->GetBranch(MAILNEWS_ROOT_PREF, getter_AddRefs(prefBranch));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    PRInt32 appendAccountsCurrentVersion=0;
+    PRInt32 appendAccountsDefaultVersion=0;
+    rv = prefBranch->GetIntPref(APPEND_ACCOUNTS_VERSION_PREF_NAME, &appendAccountsCurrentVersion);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    rv = defaultsPrefBranch->GetIntPref(APPEND_ACCOUNTS_VERSION_PREF_NAME, &appendAccountsDefaultVersion);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    // Update the account list if needed
+    if ((appendAccountsCurrentVersion <= appendAccountsDefaultVersion)) {
+
+      // Get a list of pre-configured accounts
+      nsXPIDLCString appendAccountList;
+      rv = m_prefs->CopyCharPref(PREF_MAIL_ACCOUNTMANAGER_APPEND_ACCOUNTS,
+                                          getter_Copies(appendAccountList));
+
+      // If there are pre-configured accounts, we need to add them to the existing list.
+      if (appendAccountList.Length() > 0) {
+        if (accountList.Length() > 0) {
+          nsCStringArray existingAccountsArray;
+          existingAccountsArray.ParseString(accountList.get(), ACCOUNT_DELIMITER);
+
+          // Tokenize the data and add each account if it is not already there 
+          // in the user's current mailnews account list
+          char *newAccountStr;
+          char *preConfigAccountsStr = ToNewCString(appendAccountList);
+  
+          char *token = nsCRT::strtok(preConfigAccountsStr, ACCOUNT_DELIMITER, &newAccountStr);
+
+          nsCAutoString newAccount;
+          while (token) {
+            if (token && *token) {
+              newAccount.Assign(token);
+              newAccount.StripWhitespace();
+
+              if (existingAccountsArray.IndexOf(newAccount) == -1) {
+                accountList += ",";
+                accountList += newAccount;
+              }
+            }
+            token = nsCRT::strtok(newAccountStr, ACCOUNT_DELIMITER, &newAccountStr);
+          }
+          PR_Free(preConfigAccountsStr);
+        }
+        else {
+          accountList = appendAccountList;
+        }
+        // Increase the version number so that updates will happen as and when needed
+        rv = prefBranch->SetIntPref(APPEND_ACCOUNTS_VERSION_PREF_NAME, appendAccountsCurrentVersion + 1);
+      }
+    }
   }
   
-  if (NS_FAILED(rv) || !accountList || !accountList[0]) {
+  if (!accountList || !accountList[0]) {
 #ifdef DEBUG_ACCOUNTMANAGER
     printf("No accounts.\n");
 #endif
@@ -1622,8 +1709,8 @@ nsMsgAccountManager::findAccountByKey(nsISupports* element, void *aData)
 
 NS_IMETHODIMP nsMsgAccountManager::AddIncomingServerListener(nsIIncomingServerListener *serverListener)
 {
-    m_incomingServerListeners->AppendElement(serverListener);
-    return NS_OK;
+   m_incomingServerListeners->AppendElement(serverListener);
+   return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgAccountManager::RemoveIncomingServerListener(nsIIncomingServerListener *serverListener)
@@ -1655,6 +1742,7 @@ NS_IMETHODIMP nsMsgAccountManager::NotifyServerUnloaded(nsIMsgIncomingServer *se
 {
 	nsresult rv;
 	PRUint32 count;
+  server->SetFilterList(nsnull); // clear this to cut shutdown leaks. we are always passing valid non-null server here. 
 	rv = m_incomingServerListeners->Count(&count);
 	if (NS_FAILED(rv)) return rv;
 
@@ -1876,6 +1964,24 @@ nsMsgAccountManager::findServer(nsISupports *aElement, void *data)
   return PR_TRUE;
 }
 
+NS_IMETHODIMP
+nsMsgAccountManager::GetFirstIdentityForServer(nsIMsgIncomingServer *aServer, nsIMsgIdentity **aIdentity)
+{
+  NS_ENSURE_ARG_POINTER(aServer);
+  NS_ENSURE_ARG_POINTER(aIdentity);
+
+  nsCOMPtr<nsISupportsArray> identities;
+  nsresult rv = GetIdentitiesForServer(aServer, getter_AddRefs(identities));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMsgIdentity> identity;
+  rv = identities->QueryElementAt(0, NS_GET_IID(nsIMsgIdentity),
+                                  (void **)getter_AddRefs(identity));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_IF_ADDREF(*aIdentity = identity);
+  return rv;
+}
+
 NS_IMETHODIMP
 nsMsgAccountManager::GetIdentitiesForServer(nsIMsgIncomingServer *server,
                                             nsISupportsArray **_retval)
