@@ -40,6 +40,7 @@
 #include "nsIMsgHdr.h"
 #include "nsIMsgThread.h"
 #include "nsIDBFolderInfo.h"
+#include "nsIMsgSearchSession.h"
 
 #define MSGHDR_CACHE_LOOK_AHEAD_SIZE  25    // Allocate this more to avoid reallocation on new mail.
 #define MSGHDR_CACHE_MAX_SIZE         8192  // Max msghdr cache entries.
@@ -55,11 +56,10 @@ nsMsgThreadedDBView::~nsMsgThreadedDBView()
   /* destructor code */
 }
 
-NS_IMETHODIMP nsMsgThreadedDBView::Open(nsIMsgFolder *folder, nsMsgViewSortTypeValue sortType, nsMsgViewSortOrderValue sortOrder, nsMsgViewFlagsTypeValue viewFlags, PRInt32 *pCount)
+NS_IMETHODIMP nsMsgThreadedDBView::Open(nsIMsgFolder *folder, nsMsgViewSortTypeValue sortType, nsMsgViewSortOrderValue sortOrder, nsMsgViewFlagsTypeValue viewFlags, PRBool aTreatRecipientAsAuthor, PRInt32 *pCount)
 {
-	nsresult rv;
-	rv = nsMsgDBView::Open(folder, sortType, sortOrder, viewFlags, pCount);
-    NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv = nsMsgDBView::Open(folder, sortType, sortOrder, viewFlags, aTreatRecipientAsAuthor, pCount);
+  NS_ENSURE_SUCCESS(rv, rv);
 
   // Preset msg hdr cache size for performance reason.
   if (m_db)
@@ -101,37 +101,44 @@ NS_IMETHODIMP nsMsgThreadedDBView::Close()
 NS_IMETHODIMP nsMsgThreadedDBView::ReloadFolderAfterQuickSearch()
 {
   mIsSearchView = PR_FALSE;
+  m_searchSession = nsnull;
   m_sortValid = PR_FALSE;  //force a sort
   nsresult rv = NS_OK;
+  nsMsgKeyArray preservedSelection;
+  SaveAndClearSelection(&preservedSelection);  
+
+  // restore saved id array and flags array
+  // first, remove all the search hits
+  PRInt32 oldSize = GetSize();
+  m_keys.RemoveAll();
+  m_flags.RemoveAll();
+  m_levels.RemoveAll();
+
+  // this needs to happen after we remove all the keys, since RowCountChanged() will call our GetRowCount()
+  if (mTree)
+    mTree->RowCountChanged(0, -oldSize);
+
   if (m_preSearchKeys.GetSize())
   {
-    // restore saved id array and flags array
-    // first, remove all the search hits
-    PRInt32 oldSize = GetSize();
-    m_keys.RemoveAll();
-    m_flags.RemoveAll();
-    m_levels.RemoveAll();
-
-    // this needs to happen after we remove all the keys, since RowCountChanged() will call our GetRowCount()
-    if (mOutliner)
-      mOutliner->RowCountChanged(0, -oldSize);
-
     m_keys.InsertAt(0, &m_preSearchKeys);
     m_flags.InsertAt(0, &m_preSearchFlags);
     m_levels.InsertAt(0, &m_preSearchLevels);
     ClearPreSearchInfo();
     ClearPrevIdArray();  // previous cached info about non threaded display is not useful
-    InitSort(m_sortType, m_sortOrder);
-
-    // now, account for adding all the pre search items back.
-    // this needs to happen after we add back the keys, as RowCountChanged() will call our GetRowCount()
-    if (mOutliner)
-      mOutliner->RowCountChanged(0, GetSize());
+    Sort(m_sortType, m_sortOrder);
   }
   else
   {
     rv = InitThreadedView(nsnull);
   }
+
+  // now, account for adding all the pre search items back.
+  // this needs to happen after we add back the keys, as RowCountChanged() will call our GetRowCount()
+  if (mTree)
+    mTree->RowCountChanged(0, GetSize());
+
+  RestoreSelection(&preservedSelection);
+
   return rv;
 }
 
@@ -213,14 +220,23 @@ nsresult nsMsgThreadedDBView::AddKeys(nsMsgKey *pKeys, PRInt32 *pFlags, const ch
 NS_IMETHODIMP nsMsgThreadedDBView::Sort(nsMsgViewSortTypeValue sortType, nsMsgViewSortOrderValue sortOrder)
 {
   nsresult rv;
-  nsMsgKeyArray preservedSelection;
-  SaveAndClearSelection(&preservedSelection);
-  
+
   PRInt32 rowCountBeforeSort = GetSize();
 
-  if (!rowCountBeforeSort)
+  if (!rowCountBeforeSort) {
+    // still need to setup our flags even when no articles - bug 98183.
+    m_sortType = sortType;
+    if (sortType == nsMsgViewSortType::byThread)
+      m_viewFlags |= nsMsgViewFlagsType::kThreadedDisplay;
+    else
+      m_viewFlags &= ~nsMsgViewFlagsType::kThreadedDisplay;
+    SaveSortInfo(sortType, sortOrder);
     return NS_OK;
+  }
+
   
+  nsMsgKeyArray preservedSelection;
+  SaveAndClearSelection(&preservedSelection);
   // if the client wants us to forget our cached id arrays, they
   // should build a new view. If this isn't good enough, we
   // need a method to do that.
@@ -243,13 +259,13 @@ NS_IMETHODIMP nsMsgThreadedDBView::Sort(nsMsgViewSortTypeValue sortType, nsMsgVi
         m_sortValid = PR_TRUE;
         
         // the sort may have changed the number of rows
-        // before we restore the selection, tell the outliner
+        // before we restore the selection, tell the tree
         // do this before we call restore selection
         // this is safe when there is no selection.
         rv = AdjustRowCount(rowCountBeforeSort, GetSize());
         
         RestoreSelection(&preservedSelection);
-        if (mOutliner) mOutliner->Invalidate();
+        if (mTree) mTree->Invalidate();
         return NS_OK;
       }
       else
@@ -260,13 +276,13 @@ NS_IMETHODIMP nsMsgThreadedDBView::Sort(nsMsgViewSortTypeValue sortType, nsMsgVi
           Sort(sortType, sortOrder);
         
         // the sort may have changed the number of rows
-        // before we update the selection, tell the outliner
+        // before we update the selection, tell the tree
         // do this before we call restore selection
         // this is safe when there is no selection.
         rv = AdjustRowCount(rowCountBeforeSort, GetSize());
         
         RestoreSelection(&preservedSelection);
-        if (mOutliner) mOutliner->Invalidate();
+        if (mTree) mTree->Invalidate();
         return NS_OK;
       }
     }
@@ -292,13 +308,13 @@ NS_IMETHODIMP nsMsgThreadedDBView::Sort(nsMsgViewSortTypeValue sortType, nsMsgVi
   rv = nsMsgDBView::Sort(sortType, sortOrder);
   SaveSortInfo(sortType, sortOrder);
   // the sort may have changed the number of rows
-  // before we restore the selection, tell the outliner
+  // before we restore the selection, tell the tree
   // do this before we call restore selection
   // this is safe when there is no selection.
   rv = AdjustRowCount(rowCountBeforeSort, GetSize());
 
   RestoreSelection(&preservedSelection);
-  if (mOutliner) mOutliner->Invalidate();
+  if (mTree) mTree->Invalidate();
   NS_ENSURE_SUCCESS(rv,rv);
   return NS_OK;
 }
@@ -527,12 +543,7 @@ nsresult nsMsgThreadedDBView::InitSort(nsMsgViewSortTypeValue sortType, nsMsgVie
 
 nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey, PRBool ensureListed)
 {
-  nsresult	rv = NS_OK;
-  if (mIsSearchView)
-  {
-    OnHeaderAddedOrDeleted();  //db has changed and we are not adding hdr to view so clear the cached info..
-    return NS_OK; // do not add a new message to search view.
-  }
+  nsresult	rv;
   // views can override this behaviour, which is to append to view.
   // This is the mail behaviour, but threaded views want
   // to insert in order...
@@ -540,6 +551,16 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
   rv = m_db->GetMsgHdrForKey(newKey, getter_AddRefs(msgHdr));
   if (NS_SUCCEEDED(rv) && msgHdr != nsnull)
   {
+    if (mIsSearchView)
+    {
+      PRBool match=PR_FALSE;
+      OnHeaderAddedOrDeleted();  //db has changed, so clear the cached info..
+      nsCOMPtr <nsIMsgSearchSession> searchSession = do_QueryReferent(m_searchSession);
+      if (searchSession)
+        searchSession->MatchHdr(msgHdr, m_db, &match);
+      if (!match)
+        return NS_OK; // do not add a new message if there isn't a match.
+    }
     PRUint32 msgFlags;
     msgHdr->GetFlags(&msgFlags);
     if ((m_viewFlags & nsMsgViewFlagsType::kUnreadOnly) && !ensureListed && (msgFlags & MSG_FLAG_READ))
@@ -547,7 +568,9 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
     // Currently, we only add the header in a threaded view if it's a thread.
     // We used to check if this was the first header in the thread, but that's
     // a bit harder in the unreadOnly view. But we'll catch it below.
-    if (! (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay))// || msgHdr->GetMessageKey() == m_messageDB->GetKeyOfFirstMsgInThread(msgHdr->GetMessageKey()))
+
+    // for search view we don't support threaded display so just add it to the view.   
+    if (! (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay) || mIsSearchView)// || msgHdr->GetMessageKey() == m_messageDB->GetKeyOfFirstMsgInThread(msgHdr->GetMessageKey()))
       rv = AddHdr(msgHdr);
     else	// need to find the thread we added this to so we can change the hasnew flag
       // added message to existing thread, but not to view
@@ -562,7 +585,7 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
         // if we have a collapsed thread which just got a new
         // top of thread, change the keys array.
         PRInt32 level = FindLevelInThread(msgHdr, threadIndex);
-        if ((flags & MSG_FLAG_ELIDED)
+        if (((flags & MSG_FLAG_ELIDED) || threadCount == 1)
           && (!(m_viewFlags & nsMsgViewFlagsType::kUnreadOnly) || !(msgFlags & MSG_FLAG_READ)))
         {
           if (level == 0) {
@@ -590,10 +613,8 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
           // removing it, installing this header as king, and expanding it.
           if (level == 0)	
           {
-            CollapseByIndex(threadIndex, nsnull);
-            // call base class, so child won't get promoted.
-            nsMsgDBView::RemoveByIndex(threadIndex);	
-            newFlags |= MSG_VIEW_FLAG_ISTHREAD | MSG_VIEW_FLAG_HASCHILDREN | MSG_FLAG_ELIDED;
+            newFlags |= MSG_VIEW_FLAG_ISTHREAD | MSG_VIEW_FLAG_HASCHILDREN;
+            insertIndex = threadIndex;
           }
           m_keys.InsertAt(insertIndex, newKey);
           m_flags.InsertAt(insertIndex, newFlags, 1);
@@ -602,10 +623,16 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
 
           // the call to NoteChange() has to happen after we add the key
           // as NoteChange() will call RowCountChanged() which will call our GetRowCount()
-          NoteChange(insertIndex, 1, nsMsgViewNotificationCode::insertOrDelete);
+          if (level != 0)
+            NoteChange(insertIndex, 1, nsMsgViewNotificationCode::insertOrDelete);
 
           if (level == 0)
+          {
+            CollapseByIndex(threadIndex, nsnull);
+            // call base class, so child won't get promoted.
+            // nsMsgDBView::RemoveByIndex(threadIndex);	
             ExpandByIndex(threadIndex, nsnull);
+          }
         }
       }
       else // adding msg to thread that's not in view.
@@ -624,11 +651,8 @@ nsresult nsMsgThreadedDBView::OnNewHeader(nsMsgKey newKey, nsMsgKey aParentKey,
   return rv;
 }
 
-nsMsgViewIndex nsMsgThreadedDBView::GetInsertInfoForNewHdr(nsIMsgDBHdr *newHdr, nsMsgViewIndex threadIndex, PRInt32 targetLevel)
+nsMsgViewIndex nsMsgThreadedDBView::GetInsertInfoForNewHdr(nsIMsgDBHdr *newHdr, nsMsgViewIndex parentIndex, PRInt32 targetLevel)
 {
-  nsMsgKey threadParent;
-  newHdr->GetThreadParent(&threadParent);
-  nsMsgViewIndex parentIndex = m_keys.FindIndex(threadParent, threadIndex);
   PRInt32 viewSize = GetSize();
   nsMsgViewIndex insertIndex = parentIndex + 1;
   if (parentIndex != nsMsgViewIndex_None)
@@ -670,14 +694,16 @@ nsresult nsMsgThreadedDBView::RemoveByIndex(nsMsgViewIndex index)
   
   flags = m_flags[index];
   
-  if (! (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay))
+  //we don't support threaded view in quick search
+  if (! (m_viewFlags & nsMsgViewFlagsType::kThreadedDisplay) || mIsSearchView) 
     return nsMsgDBView::RemoveByIndex(index);
   
   nsCOMPtr <nsIMsgThread> threadHdr; 
   GetThreadContainingIndex(index, getter_AddRefs(threadHdr));
   NS_ENSURE_SUCCESS(rv, rv);
-  PRUint32 numThreadChildren;
-  threadHdr->GetNumChildren(&numThreadChildren);
+  PRUint32 numThreadChildren = 0; // if we can't get thread, it's already deleted and thus has 0 children
+  if (threadHdr)
+    threadHdr->GetNumChildren(&numThreadChildren);
   // check if we're the top level msg in the thread, and we're not collapsed.
   if ((flags & MSG_VIEW_FLAG_ISTHREAD) && !(flags & MSG_FLAG_ELIDED) && (flags & MSG_VIEW_FLAG_HASCHILDREN))
   {
@@ -794,15 +820,20 @@ nsMsgThreadedDBView::OnSearchHit(nsIMsgDBHdr* aMsgHdr, nsIMsgFolder *folder)
   m_flags.Add(msgFlags);
 
   // this needs to happen after we add the key, as RowCountChanged() will call our GetRowCount()
-  if (mOutliner)
-    mOutliner->RowCountChanged(GetSize() - 1, 1);
+  if (mTree)
+    mTree->RowCountChanged(GetSize() - 1, 1);
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgThreadedDBView::OnSearchDone(nsresult status)
-{  
+{
+  if (m_sortType != nsMsgViewSortType::byThread)//we do not find levels for the results.
+  {
+    m_sortValid = PR_FALSE;       //sort the results 
+    Sort(m_sortType, m_sortOrder);
+  }
   return NS_OK;
 }
 
@@ -818,10 +849,11 @@ nsMsgThreadedDBView::OnNewSearch()
   m_keys.RemoveAll();
   m_levels.RemoveAll();
   m_flags.RemoveAll();
-  
+  m_currentlyDisplayedMsgKey = nsMsgKey_None;   // need to clear this, so we won't restore it.
+
   // this needs to happen after we remove all the keys, since RowCountChanged() will call our GetRowCount()
-  if (mOutliner)
-    mOutliner->RowCountChanged(0, -oldSize);
+  if (mTree)
+    mTree->RowCountChanged(0, -oldSize);
 
   ClearPrevIdArray(); // previous cached info about non threaded display is not useful
   mIsSearchView = PR_TRUE;  
@@ -843,3 +875,19 @@ void nsMsgThreadedDBView::ClearPreSearchInfo()
   m_preSearchLevels.RemoveAll();
   m_preSearchFlags.RemoveAll();
 }
+
+NS_IMETHODIMP
+nsMsgThreadedDBView::CloneDBView(nsIMessenger *aMessengerInstance, nsIMsgWindow *aMsgWindow, nsIMsgDBViewCommandUpdater *aCmdUpdater, nsIMsgDBView **_retval)
+{
+  nsMsgThreadedDBView* newMsgDBView;
+  NS_NEWXPCOM(newMsgDBView, nsMsgThreadedDBView);
+
+  if (!newMsgDBView)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  nsresult rv = CopyDBView(newMsgDBView, aMessengerInstance, aMsgWindow, aCmdUpdater);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  NS_IF_ADDREF(*_retval = newMsgDBView);
+  return NS_OK;
+}
