@@ -37,7 +37,6 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "msgCore.h"
-#include "xp_core.h"
 #include "nsReadableUtils.h"
 #include "nsMsgDBFolder.h"
 #include "nsMsgFolderFlags.h"
@@ -61,6 +60,7 @@
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 
+#include <time.h>
 
 #define oneHour 3600000000
 #include "nsMsgUtils.h"
@@ -131,6 +131,8 @@ NS_IMETHODIMP nsMsgDBFolder::Shutdown(PRBool shutdownChildren)
 				}
 			}
 		}
+    // Ask base class shutdown itself.
+    nsMsgFolder::Shutdown(shutdownChildren);
 	}
 	return NS_OK;
 }
@@ -426,10 +428,10 @@ nsresult nsMsgDBFolder::ReadDBFolderInfo(PRBool force)
               if (!mInitializedFromCache)
               {
                 folderInfo->GetFlags((PRInt32 *)&mFlags);
-#ifdef DEBUG_bienvenu
+#ifdef DEBUG_bienvenu1
                 nsXPIDLString name;
                 GetName(getter_Copies(name));
-                NS_ASSERTION(Compare(name, NS_LITERAL_STRING("Trash")) || (mFlags & MSG_FOLDER_FLAG_TRASH), "lost trash flag");
+                NS_ASSERTION(Compare(name, kLocalizedTrashName) || (mFlags & MSG_FOLDER_FLAG_TRASH), "lost trash flag");
 #endif
                 mInitializedFromCache = PR_TRUE;
               }
@@ -529,7 +531,7 @@ NS_IMETHODIMP nsMsgDBFolder::GetOfflineFileTransport(nsMsgKey msgKey, PRUint32 *
     mPath->GetNativePath(getter_Copies(nativePath));
 
     nsCOMPtr <nsILocalFile> localStore;
-    rv = NS_NewLocalFile(nativePath, PR_TRUE, getter_AddRefs(localStore));
+    rv = NS_NewNativeLocalFile(nativePath, PR_TRUE, getter_AddRefs(localStore));
     if (NS_SUCCEEDED(rv) && localStore)
     {
       NS_DEFINE_CID(kFileTransportServiceCID, NS_FILETRANSPORTSERVICE_CID);
@@ -539,7 +541,11 @@ NS_IMETHODIMP nsMsgDBFolder::GetOfflineFileTransport(nsMsgKey msgKey, PRUint32 *
       if (NS_FAILED(rv))
         return rv;
       
-      rv = fts->CreateTransport(localStore, PR_RDWR | PR_CREATE_FILE, 0664, aFileChannel);
+      rv = fts->CreateTransport(localStore,
+                                PR_RDWR | PR_CREATE_FILE,
+                                0664,
+                                PR_TRUE,
+                                aFileChannel);
 
       if (NS_SUCCEEDED(rv))
       {
@@ -586,6 +592,7 @@ NS_IMETHODIMP nsMsgDBFolder::GetOfflineStoreOutputStream(nsIOutputStream **outpu
     nsFileSpec fileSpec;
     mPath->GetFileSpec(&fileSpec);
     rv = NS_NewIOFileStream(getter_AddRefs(supports), fileSpec, PR_WRONLY | PR_CREATE_FILE, 00700);
+    NS_ENSURE_SUCCESS(rv, rv);
     supports->QueryInterface(NS_GET_IID(nsIOutputStream), (void **) outputStream);
 
     nsCOMPtr <nsIRandomAccessStore> seekable = do_QueryInterface(supports);
@@ -749,39 +756,31 @@ NS_IMETHODIMP nsMsgDBFolder::OnKeyAdded(nsMsgKey aKeyChanged, nsMsgKey  aParentK
 nsresult nsMsgDBFolder::OnKeyAddedOrDeleted(nsMsgKey aKeyChanged, nsMsgKey  aParentKey , PRInt32 aFlags, 
                         nsIDBChangeListener * aInstigator, PRBool added, PRBool doFlat, PRBool doThread)
 {
-	nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
-	nsCOMPtr<nsIMsgDBHdr> parentDBHdr;
-	nsresult rv = mDatabase->GetMsgHdrForKey(aKeyChanged, getter_AddRefs(msgDBHdr));
-	if(NS_FAILED(rv))
-		return rv;
-
-	rv = mDatabase->GetMsgHdrForKey(aParentKey, getter_AddRefs(parentDBHdr));
-	if(NS_FAILED(rv))
-		return rv;
-
-	if(msgDBHdr)
-	{
-		nsCOMPtr<nsISupports> msgSupports(do_QueryInterface(msgDBHdr));
-		nsCOMPtr<nsISupports> folderSupports;
-		rv = QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(folderSupports));
-		if(msgSupports && NS_SUCCEEDED(rv) && doFlat)
-		{
-			if(added)
-				NotifyItemAdded(folderSupports, msgSupports, "flatMessageView");
-			else
-				NotifyItemDeleted(folderSupports, msgSupports, "flatMessageView");
-		}
-		if(msgSupports && folderSupports)
-		{
-			if(added)
-				NotifyItemAdded(folderSupports, msgSupports, "threadMessageView");
-			else
-				NotifyItemDeleted(folderSupports, msgSupports, "threadMessageView");
-		}
-		UpdateSummaryTotals(PR_TRUE);
-	}
-	return NS_OK;
-
+  nsCOMPtr<nsIMsgDBHdr> msgDBHdr;
+  nsresult rv = mDatabase->GetMsgHdrForKey(aKeyChanged, getter_AddRefs(msgDBHdr));
+  if(NS_SUCCEEDED(rv) && msgDBHdr)
+  {
+    nsCOMPtr<nsISupports> msgSupports(do_QueryInterface(msgDBHdr));
+    nsCOMPtr<nsISupports> folderSupports;
+    rv = QueryInterface(NS_GET_IID(nsISupports), getter_AddRefs(folderSupports));
+    if(msgSupports && NS_SUCCEEDED(rv) && doFlat)
+    {
+      if(added)
+        NotifyItemAdded(folderSupports, msgSupports, "flatMessageView");
+      else
+        NotifyItemDeleted(folderSupports, msgSupports, "flatMessageView");
+    }
+    if(msgSupports && folderSupports)
+    {
+      if(added)
+        NotifyItemAdded(folderSupports, msgSupports, "threadMessageView");
+      else
+        NotifyItemDeleted(folderSupports, msgSupports, "threadMessageView");
+    }
+    UpdateSummaryTotals(PR_TRUE);
+  }
+  return NS_OK;
+  
 }
 
 
@@ -1138,9 +1137,9 @@ nsMsgDBFolder::MarkAllMessagesRead(void)
   
   if(NS_SUCCEEDED(rv))
   {
-    EnableNotifications(allMessageCountNotifications, PR_FALSE);
+    EnableNotifications(allMessageCountNotifications, PR_FALSE, PR_TRUE /*dbBatching*/);
     rv = mDatabase->MarkAllRead(nsnull);
-    EnableNotifications(allMessageCountNotifications, PR_TRUE);
+    EnableNotifications(allMessageCountNotifications, PR_TRUE, PR_TRUE /*dbBatching*/);
     mDatabase->SetSummaryValid(PR_TRUE);
     mDatabase->Commit(nsMsgDBCommitType::kLargeCommit);
   }
@@ -1160,37 +1159,37 @@ NS_IMETHODIMP nsMsgDBFolder::MarkThreadRead(nsIMsgThread *thread)
 NS_IMETHODIMP
 nsMsgDBFolder::OnStartRunningUrl(nsIURI *aUrl)
 {
-	NS_PRECONDITION(aUrl, "just a sanity check");
-    return NS_OK;
+  NS_PRECONDITION(aUrl, "just a sanity check");
+  return NS_OK;
 }
 
 NS_IMETHODIMP
 nsMsgDBFolder::OnStopRunningUrl(nsIURI *aUrl, nsresult aExitCode)
 {
-	NS_PRECONDITION(aUrl, "just a sanity check");
-	nsCOMPtr<nsIMsgMailNewsUrl> mailUrl = do_QueryInterface(aUrl);
-	if (mailUrl)
-	{
-		PRBool updatingFolder = PR_FALSE;
-		if (NS_SUCCEEDED(mailUrl->GetUpdatingFolder(&updatingFolder)) && updatingFolder)
-		{
+  NS_PRECONDITION(aUrl, "just a sanity check");
+  nsCOMPtr<nsIMsgMailNewsUrl> mailUrl = do_QueryInterface(aUrl);
+  if (mailUrl)
+  {
+    PRBool updatingFolder = PR_FALSE;
+    if (NS_SUCCEEDED(mailUrl->GetUpdatingFolder(&updatingFolder)) && updatingFolder)
+    {
       NotifyFolderEvent(mFolderLoadedAtom);
-
-//GGGG			 check for new mail here and call SetNewMessages...?? -- ONE OF THE 2 PLACES
-			if(mDatabase)
-			{
-				nsresult rv;
-				PRBool hasNewMessages;
-
-				rv = mDatabase->HasNew(&hasNewMessages);
-				SetHasNewMessages(hasNewMessages);
-			}
-		}
-
+      
+      //GGGG			 check for new mail here and call SetNewMessages...?? -- ONE OF THE 2 PLACES
+      if(mDatabase)
+      {
+        nsresult rv;
+        PRBool hasNewMessages;
+        
+        rv = mDatabase->HasNew(&hasNewMessages);
+        SetHasNewMessages(hasNewMessages);
+      }
+    }
+    
     // be sure to remove ourselves as a url listener
     mailUrl->UnRegisterListener(this);
-	}
-    return NS_OK;
+  }
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMsgDBFolder::GetRetentionSettings(nsIMsgRetentionSettings **settings)
@@ -1333,7 +1332,8 @@ nsresult nsMsgDBFolder::WriteStartOfNewLocalMessage()
 nsresult nsMsgDBFolder::StartNewOfflineMessage()
 {
   nsresult rv = GetOfflineStoreOutputStream(getter_AddRefs(m_tempMessageStream));
-  WriteStartOfNewLocalMessage();
+  if (NS_SUCCEEDED(rv))
+    WriteStartOfNewLocalMessage();
   m_numOfflineMsgLines = 0;
   return rv;
 }
@@ -1371,17 +1371,7 @@ nsresult nsMsgDBFolder::CompactOfflineStore(nsIMsgWindow *inWindow)
   nsresult rv;
   nsCOMPtr <nsIMsgFolderCompactor> folderCompactor =  do_CreateInstance(NS_MSGOFFLINESTORECOMPACTOR_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv) && folderCompactor)
-  {
-    GetDatabase(nsnull);
-    if (NS_SUCCEEDED(rv)) 
-    {
-      nsCOMPtr<nsIFileSpec> pathSpec;
-      rv = GetPath(getter_AddRefs(pathSpec));
-      rv = folderCompactor->Init(this, mBaseMessageURI, mDatabase, pathSpec, inWindow);
-      if (NS_SUCCEEDED(rv))
-        rv = folderCompactor->StartCompacting();
-    }
-  }
+      rv = folderCompactor->Compact(this, inWindow);
   return rv;
 }
 
@@ -1514,7 +1504,7 @@ nsMsgDBFolder::CompactAllOfflineStores(nsIMsgWindow *aWindow, nsISupportsArray *
   folderCompactor = do_CreateInstance(NS_MSGOFFLINESTORECOMPACTOR_CONTRACTID, &rv);
 
   if (NS_SUCCEEDED(rv) && folderCompactor)
-    rv = folderCompactor->StartCompactingAll(aOfflineFolderArray, aWindow, PR_FALSE, nsnull);
+    rv = folderCompactor->CompactAll(aOfflineFolderArray, aWindow, PR_FALSE, nsnull);
   return rv;
 }
 
@@ -1589,7 +1579,7 @@ nsMsgDBFolder::MatchOrChangeFilterDestination(nsIMsgFolder *newFolder, PRBool ca
           rv = server->GetCanHaveFilters(&canHaveFilters);
           if (NS_SUCCEEDED(rv) && canHaveFilters) 
           {
-            rv = server->GetFilterList(getter_AddRefs(filterList));
+            rv = server->GetFilterList(nsnull, getter_AddRefs(filterList));
             if (filterList && NS_SUCCEEDED(rv))
             {
               rv = filterList->MatchOrChangeFilterTarget(oldUri, newUri, caseInsensitive, found);
@@ -1604,3 +1594,30 @@ nsMsgDBFolder::MatchOrChangeFilterDestination(nsIMsgFolder *newFolder, PRBool ca
 
   return rv;
 }
+
+NS_IMETHODIMP
+nsMsgDBFolder::GetDBTransferInfo(nsIDBFolderInfo **aTransferInfo)
+{
+  nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
+  nsCOMPtr <nsIMsgDatabase> db;
+  GetDBFolderInfoAndDB(getter_AddRefs(dbFolderInfo), getter_AddRefs(db));
+  if (dbFolderInfo)
+    dbFolderInfo->GetTransferInfo(aTransferInfo);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgDBFolder::SetDBTransferInfo(nsIDBFolderInfo *aTransferInfo)
+{
+  NS_ENSURE_ARG(aTransferInfo);
+  nsCOMPtr <nsIDBFolderInfo> dbFolderInfo;
+  nsCOMPtr <nsIMsgDatabase> db;
+  GetMsgDatabase(nsnull, getter_AddRefs(db));
+  if (db)
+  {
+    db->GetDBFolderInfo(getter_AddRefs(dbFolderInfo));
+    if(dbFolderInfo)
+      dbFolderInfo->InitFromTransferInfo(aTransferInfo);
+  }
+  return NS_OK;
+}
