@@ -37,11 +37,9 @@
  * ***** END LICENSE BLOCK ***** */
 
 // as does this
-#define NS_IMPL_IDS
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetAlias.h"
 #include "nsIPlatformCharset.h"
-#undef NS_IMPL_IDS
 #include "nsIServiceManager.h"
 #include "nsICharsetConverterManager2.h"
 
@@ -68,7 +66,6 @@ static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kCMimeConverterCID, NS_MIME_CONVERTER_CID);
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 static NS_DEFINE_CID(kEntityConverterCID, NS_ENTITYCONVERTER_CID);
-static NS_DEFINE_CID(kSaveAsCharsetCID, NS_SAVEASCHARSET_CID);
 
 //
 // International functions necessary for composition
@@ -396,10 +393,10 @@ const char * nsMsgI18NFileSystemCharset()
 }
 
 // MIME encoder, output string should be freed by PR_FREE
-char * nsMsgI18NEncodeMimePartIIStr(const char *header, const char *charset, PRBool bUseMime) 
+char * nsMsgI18NEncodeMimePartIIStr(const char *header, PRBool structured, const char *charset, PRInt32 fieldnamelen, PRBool usemime) 
 {
   // No MIME, convert to the outgoing mail charset.
-  if (PR_FALSE == bUseMime) {
+  if (PR_FALSE == usemime) {
     char *convertedStr;
     if (NS_SUCCEEDED(ConvertFromUnicode(charset, NS_ConvertUTF8toUCS2(header), &convertedStr)))
       return (convertedStr);
@@ -408,13 +405,11 @@ char * nsMsgI18NEncodeMimePartIIStr(const char *header, const char *charset, PRB
   }
 
   char *encodedString = nsnull;
-  nsIMimeConverter *converter;
-  nsresult res = nsComponentManager::CreateInstance(kCMimeConverterCID, nsnull, 
-                                           NS_GET_IID(nsIMimeConverter), (void **)&converter);
-  if (NS_SUCCEEDED(res) && nsnull != converter) {
-    res = converter->EncodeMimePartIIStr_UTF8(header, charset, kMIME_ENCODED_WORD_SIZE, &encodedString);
-    NS_RELEASE(converter);
-  }
+  nsresult res;
+  nsCOMPtr<nsIMimeConverter> converter = do_GetService(kCMimeConverterCID, &res);
+  if (NS_SUCCEEDED(res) && nsnull != converter)
+    res = converter->EncodeMimePartIIStr_UTF8(header, structured, charset, fieldnamelen, kMIME_ENCODED_WORD_SIZE, &encodedString);
+
   return NS_SUCCEEDED(res) ? encodedString : nsnull;
 }
 
@@ -438,7 +433,8 @@ PRBool nsMsgI18Nmultibyte_charset(const char *charset)
     if (NS_SUCCEEDED(res)) {
       res = ccm2->GetCharsetData2(charsetAtom, NS_LITERAL_STRING(".isMultibyte").get(), &charsetData);
       if (NS_SUCCEEDED(res)) {
-        result = !Compare(charsetData, NS_LITERAL_STRING("true"), nsCaseInsensitiveStringComparator());
+        result = charsetData.Equals(NS_LITERAL_STRING("true"),
+                                    nsCaseInsensitiveStringComparator());
       }
     }
   }
@@ -476,7 +472,8 @@ PRBool nsMsgI18N7bit_data_part(const char *charset, const char *inString, const
         srcLen = ((size - consumedLen) >= 512) ? 512 : (size - consumedLen);  // buffer len or remaining src len
         unicharLength = 512;
         res = decoder->Convert(currentSrcPtr, &srcLen, unicharBuff, &unicharLength);
-        if (NS_FAILED(res))
+        // break, if error or the input string not consumed
+        if (NS_FAILED(res) || !srcLen)
           break;
         for (PRInt32 i = 0; i < unicharLength; i++) {
           if (unicharBuff[i] > 127) {
@@ -494,7 +491,7 @@ PRBool nsMsgI18N7bit_data_part(const char *charset, const char *inString, const
   return result;
 }
 
-PRBool nsMsgI18Ncheck_data_in_charset_range(const char *charset, const PRUnichar* inString)
+PRBool nsMsgI18Ncheck_data_in_charset_range(const char *charset, const PRUnichar* inString, char **fallbackCharset)
 {
   if (!charset || !*charset || !inString || !*inString)
     return PR_TRUE;
@@ -537,6 +534,14 @@ PRBool nsMsgI18Ncheck_data_in_charset_range(const char *charset, const PRUnichar
     }    
   }
 
+  // if the conversion was not successful then try fallback to other charsets
+  if (!result && fallbackCharset) {
+    nsXPIDLCString convertedString;
+    res = nsMsgI18NSaveAsCharset("text/plain", charset, inString, 
+                                 getter_Copies(convertedString), fallbackCharset);
+    result = (NS_SUCCEEDED(res) && NS_ERROR_UENC_NOMAPPING != res);
+  }
+
   return result;
 }
 
@@ -563,7 +568,8 @@ nsMsgI18NParseMetaCharset(nsFileSpec* fileSpec)
     if (*buffer == nsCRT::CR || *buffer == nsCRT::LF || *buffer == 0) 
       continue; 
 
-    for (int i = 0; i < (int)PL_strlen(buffer); i++) { 
+    PRUint32 len = PL_strlen(buffer);
+    for (PRUint32 i = 0; i < len; i++) { 
       buffer[i] = toupper(buffer[i]); 
     } 
 
@@ -583,7 +589,9 @@ nsMsgI18NParseMetaCharset(nsFileSpec* fileSpec)
       token = nsCRT::strtok(cp, seps, &newStr); 
       if (token != NULL) 
       { 
-        PL_strcpy(charset, token); 
+        PL_strncpy(charset, token, sizeof(charset));
+        charset[sizeof(charset)-1] = '\0';
+        break;
       } 
     } 
   } 
@@ -611,7 +619,8 @@ nsresult nsMsgI18NConvertToEntity(const nsString& inString, nsString* outString)
   return res;
 }
 
-nsresult nsMsgI18NSaveAsCharset(const char* contentType, const char *charset, const PRUnichar* inString, char** outString)
+nsresult nsMsgI18NSaveAsCharset(const char* contentType, const char *charset, 
+                                const PRUnichar* inString, char** outString, char **fallbackCharset)
 {
   NS_ASSERTION(contentType, "null ptr- contentType");
   NS_ASSERTION(charset, "null ptr- charset");
@@ -634,72 +643,101 @@ nsresult nsMsgI18NSaveAsCharset(const char* contentType, const char *charset, co
     return NS_ERROR_ILLEGAL_VALUE;  // not supported type
   }
 
-  // Resolve charset alias
-  nsAutoString aCharset; aCharset.AssignWithConversion(charset);
-  nsCOMPtr<nsICharsetAlias> calias(do_GetService(kCharsetAliasCID, &res)); 
-  if (NS_SUCCEEDED(res)) {
-    nsAutoString aAlias(aCharset);
-    if (aAlias.Length()) {
-      res = calias->GetPreferred(aAlias, aCharset);
-    }
-  }
-  if (NS_FAILED(res)) {
-    return res;
-  }
+  nsCOMPtr <nsICharsetConverterManager2> ccm2 = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &res);
+  NS_ENSURE_SUCCESS(res, res);
 
-  nsCOMPtr <nsISaveAsCharset> aConv;  // charset converter plus entity, NCR generation
-  res = nsComponentManager::CreateInstance(kSaveAsCharsetCID, NULL, 
-                                           NS_GET_IID(nsISaveAsCharset), getter_AddRefs(aConv));
-  if(NS_SUCCEEDED(res)) {
-    // attribute: 
-    // html text - charset conv then fallback to entity or NCR
-    // plain text - charset conv then fallback to '?'
-    char charset_buf[kMAX_CSNAME+1];
-    if (bTEXT_HTML) {
-      // For ISO-8859-1 only, convert to entity first (always generate entites like &nbsp;).
-      res = aConv->Init(aCharset.ToCString(charset_buf, kMAX_CSNAME+1), 
-                        aCharset.EqualsIgnoreCase("ISO-8859-1") ?
-                          nsISaveAsCharset::attr_htmlTextDefault :
-                          nsISaveAsCharset::attr_EntityAfterCharsetConv + nsISaveAsCharset::attr_FallbackDecimalNCR, 
-                        nsIEntityConverter::html32);
-    }
-    else {
-      // fallback for text/plain: first try transliterate then '?'
-      res = aConv->Init(aCharset.ToCString(charset_buf, kMAX_CSNAME+1), 
-                        nsISaveAsCharset::attr_FallbackQuestionMark + nsISaveAsCharset::attr_EntityAfterCharsetConv, 
-                        nsIEntityConverter::transliterate);
-    }
-    if (NS_SUCCEEDED(res)) {
-      // Mapping characters in a certain range (required for Japanese only)
-      if (!nsCRT::strcasecmp("ISO-2022-JP", charset)) {
-        nsCOMPtr<nsIPref> prefs(do_GetService(kPrefCID, &res)); 
-        if (nsnull != prefs && NS_SUCCEEDED(res)) {
-          PRBool return_val;
+  nsCOMPtr <nsIAtom> charsetAtom;
+  res = ccm2->GetCharsetAtom(NS_ConvertASCIItoUCS2(charset).get(), getter_AddRefs(charsetAtom));
+  NS_ENSURE_SUCCESS(res, res);
 
-          if (NS_FAILED(prefs->GetBoolPref("mailnews.send_hankaku_kana", &return_val)))
-            return_val = PR_FALSE;  // no pref means need the mapping
+  const PRUnichar *charsetName;
+  res = charsetAtom->GetUnicode(&charsetName);
+  NS_ENSURE_SUCCESS(res, res);
 
-          if (!return_val) {
-            nsCOMPtr <nsITextTransform> textTransform;
-            res = nsComponentManager::CreateInstance(NS_HANKAKUTOZENKAKU_CONTRACTID, nsnull, 
-                                                     NS_GET_IID(nsITextTransform), getter_AddRefs(textTransform));
-            if (NS_SUCCEEDED(res)) {
-              nsAutoString aText(inString);
-              nsAutoString aResult;
-              res = textTransform->Change(aText, aResult);
-              if (NS_SUCCEEDED(res)) {
-                return aConv->Convert(aResult.get(), outString);
-              }
-            }
-          }
-        }
+  // charset converter plus entity, NCR generation
+  nsCOMPtr <nsISaveAsCharset> conv = do_CreateInstance(NS_SAVEASCHARSET_CONTRACTID, &res);
+  NS_ENSURE_SUCCESS(res, res);
+
+  // attribute: 
+  // html text - charset conv then fallback to entity or NCR
+  // plain text - charset conv then fallback to '?'
+  if (bTEXT_HTML)
+    // For ISO-8859-1 only, convert to entity first (always generate entites like &nbsp;).
+    res = conv->Init(NS_ConvertUCS2toUTF8(charsetName).get(), 
+                     !nsCRT::strcmp(charsetName, NS_LITERAL_STRING("ISO-8859-1").get()) ?
+                     nsISaveAsCharset::attr_htmlTextDefault :
+                     nsISaveAsCharset::attr_EntityAfterCharsetConv + nsISaveAsCharset::attr_FallbackDecimalNCR, 
+                     nsIEntityConverter::html32);
+  else
+    // fallback for text/plain: first try transliterate then '?'
+    res = conv->Init(NS_ConvertUCS2toUTF8(charsetName).get(), 
+                     nsISaveAsCharset::attr_FallbackQuestionMark + nsISaveAsCharset::attr_EntityAfterCharsetConv, 
+                     nsIEntityConverter::transliterate);
+  NS_ENSURE_SUCCESS(res, res);
+
+  const PRUnichar *input = inString;
+
+  // Mapping characters in a certain range (required for Japanese only)
+  nsAutoString mapped;
+  if (!nsCRT::strcmp(charsetName, NS_LITERAL_STRING("ISO-2022-JP").get())) {
+    static PRInt32 sSendHankakuKana = -1;
+    if (sSendHankakuKana < 0) {
+      nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &res));
+      NS_ENSURE_SUCCESS(res, res);
+      PRBool sendHankaku;
+      // Get a hidden 4.x pref with no UI, get it only once.
+      if (NS_FAILED(prefs->GetBoolPref("mailnews.send_hankaku_kana", &sendHankaku)))
+        sSendHankakuKana = 0;  // no pref means need the mapping
+      else
+        sSendHankakuKana = sendHankaku ? 1 : 0;
+    }
+
+    if (!sSendHankakuKana) {
+      nsCOMPtr <nsITextTransform> textTransform = do_CreateInstance(NS_HANKAKUTOZENKAKU_CONTRACTID, &res);
+        
+      if (NS_SUCCEEDED(res)) {
+        res = textTransform->Change(inString, nsCRT::strlen(inString), mapped);
+        if (NS_SUCCEEDED(res))
+          input = mapped.get();
       }
-
-      // Convert to charset
-      res = aConv->Convert(inString, outString);
     }
   }
- 
+
+  // Convert to charset
+  res = conv->Convert(input, outString);
+
+  // If the converer cannot encode to the charset,
+  // then fallback to pref sepcified charsets.
+  if (NS_ERROR_UENC_NOMAPPING == res && !bTEXT_HTML && fallbackCharset) {
+    nsCOMPtr<nsIPref> pref(do_GetService(NS_PREF_CONTRACTID, &res));
+    NS_ENSURE_SUCCESS(res, res);
+
+    nsCAutoString prefString("intl.fallbackCharsetList.");
+    prefString.Append(charset);
+    nsXPIDLCString fallbackList;
+    res = pref->GetCharPref(prefString.get(), getter_Copies(fallbackList));
+    // do the fallback only if there is a pref for the charset
+    if (NS_FAILED(res) || fallbackList.IsEmpty())
+      return NS_ERROR_UENC_NOMAPPING;
+
+    res = conv->Init(fallbackList.get(), 
+                     nsISaveAsCharset::attr_FallbackQuestionMark + 
+                     nsISaveAsCharset::attr_EntityAfterCharsetConv +
+                     nsISaveAsCharset::attr_CharsetFallback, 
+                     nsIEntityConverter::transliterate);
+    NS_ENSURE_SUCCESS(res, res);
+
+    // free whatever we have now
+    PR_FREEIF(*outString);  
+
+    res = conv->Convert(input, outString);
+    NS_ENSURE_SUCCESS(res, res);
+
+    // get the actual charset used for the conversion
+    if (NS_FAILED(conv->GetCharset(fallbackCharset)))
+      *fallbackCharset = nsnull;
+  }
+
   return res;
 }
 
@@ -722,11 +760,12 @@ nsMsgI18NGetAcceptLanguage(void)
   {
     nsXPIDLString prefValue;
 
-    nsCRT::memset(lang, 0, sizeof(lang));
+    memset(lang, 0, sizeof(lang));
     res = prefs->GetLocalizedUnicharPref("intl.accept_languages", getter_Copies(prefValue));
 	  if (NS_SUCCEEDED(res) && prefValue) 
     {
       PL_strncpy(lang, NS_ConvertUCS2toUTF8(prefValue).get(), sizeof(lang));
+      lang[sizeof(lang)-1] = '\0';
     }
 	  else 
 		  PL_strcpy(lang, "en");
@@ -738,7 +777,7 @@ nsMsgI18NGetAcceptLanguage(void)
 }
 
 
-// taken from nsFileSpec::GetNativePAthString()
+// taken from nsFileSpec::GetNativePathString()
 void
 nsMsgGetNativePathString(const char *aPath, nsString& aResult)
 {
