@@ -63,7 +63,8 @@
 #include "nsIWebShell.h"
 #include "nsIWebShellWindow.h"
 #include "nsIAuthPrompt.h"
-#include "nsIWalletService.h"
+#include "nsIObserverService.h"
+#include "nsNetUtil.h"
 #include "nsIWindowWatcher.h"
 #include "nsIStringBundle.h"
 
@@ -73,18 +74,15 @@
 #include "nsIInterfaceRequestorUtils.h"
 
 #include "nsIMsgAccountManager.h"
-
-#ifdef DEBUG_sspitzer
-#define DEBUG_MSGINCOMING_SERVER
-#endif /* DEBUG_sspitzer */
+#include "nsCPasswordManager.h"
+#include "nsIMsgMdnGenerator.h"
+#include "nsMsgFolderFlags.h"
+#include "nsMsgUtils.h"
 
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kWalletServiceCID, NS_WALLETSERVICE_CID);
 static NS_DEFINE_CID(kMsgFilterServiceCID, NS_MSGFILTERSERVICE_CID);
 
-#define OFFLINE_STATUS_CHANGED_TOPIC "network:offline-status-changed"
-
 #define PORT_NOT_SET -1
 
 MOZ_DECL_CTOR_COUNTER(nsMsgIncomingServer)
@@ -95,7 +93,8 @@ nsMsgIncomingServer::nsMsgIncomingServer():
     m_biffState(nsIMsgFolder::nsMsgBiffState_NoMail),
     m_serverBusy(PR_FALSE),
     m_canHaveFilters(PR_FALSE),
-    m_displayStartupPage(PR_TRUE)
+    m_displayStartupPage(PR_TRUE),
+    mPerformingBiff(PR_FALSE)
 {
   NS_INIT_REFCNT();
 }
@@ -188,6 +187,20 @@ nsMsgIncomingServer::PerformBiff()
 	return NS_ERROR_NOT_IMPLEMENTED;	
 }
 
+
+NS_IMETHODIMP nsMsgIncomingServer::GetPerformingBiff(PRBool *aPerformingBiff)
+{
+  NS_ENSURE_ARG_POINTER(aPerformingBiff);
+	*aPerformingBiff = mPerformingBiff;
+	return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgIncomingServer::SetPerformingBiff(PRBool aPerformingBiff)
+{
+	mPerformingBiff = aPerformingBiff;
+  return NS_OK;
+}
+
 NS_IMPL_GETSET(nsMsgIncomingServer, BiffState, PRUint32, m_biffState);
 
 NS_IMETHODIMP nsMsgIncomingServer::WriteToFolderCache(nsIMsgFolderCache *folderCache)
@@ -225,14 +238,6 @@ nsMsgIncomingServer::GetCanHaveFilters(PRBool *canHaveFilters)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsMsgIncomingServer::SetCanHaveFilters(PRBool canHaveFilters)
-{
-    // derived class should override if they need to do this.
-    m_canHaveFilters = canHaveFilters;
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsMsgIncomingServer::GetCanBeDefaultServer(PRBool *canBeDefaultServer)
 {
@@ -250,6 +255,42 @@ nsMsgIncomingServer::GetCanSearchMessages(PRBool *canSearchMessages)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgIncomingServer::GetCanCompactFoldersOnServer(PRBool *canCompactFoldersOnServer)
+{
+    // derived class should override if they need to do this.
+    NS_ENSURE_ARG_POINTER(canCompactFoldersOnServer);
+    *canCompactFoldersOnServer = PR_TRUE;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgIncomingServer::GetCanUndoDeleteOnServer(PRBool *canUndoDeleteOnServer)
+{
+    // derived class should override if they need to do this.
+    NS_ENSURE_ARG_POINTER(canUndoDeleteOnServer);
+    *canUndoDeleteOnServer = PR_TRUE;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgIncomingServer::GetCanEmptyTrashOnExit(PRBool *canEmptyTrashOnExit)
+{
+    // derived class should override if they need to do this.
+    NS_ENSURE_ARG_POINTER(canEmptyTrashOnExit);
+    *canEmptyTrashOnExit = PR_TRUE;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsMsgIncomingServer::GetIsSecureServer(PRBool *isSecureServer)
+{
+    // derived class should override if they need to do this.
+    NS_ENSURE_ARG_POINTER(isSecureServer);
+    *isSecureServer = PR_TRUE;
+    return NS_OK;
+}
+
 // construct <localStoreType>://[<username>@]<hostname
 NS_IMETHODIMP
 nsMsgIncomingServer::GetServerURI(char* *aResult)
@@ -412,15 +453,11 @@ nsMsgIncomingServer::GetFileValue(const char* prefname,
   
   nsCOMPtr<nsILocalFile> prefLocal;
   nsCOMPtr<nsIFileSpec> outSpec;
-  nsXPIDLCString pathBuf;
   
   nsresult rv = m_prefs->GetFileXPref(fullPrefName.get(), getter_AddRefs(prefLocal));
   if (NS_FAILED(rv)) return rv;
-  rv = NS_NewFileSpec(getter_AddRefs(outSpec));
-  if (NS_FAILED(rv)) return rv;
-  rv = prefLocal->GetPath(getter_Copies(pathBuf));
-  if (NS_FAILED(rv)) return rv;
-  rv = outSpec->SetNativePath((const char *)pathBuf);
+
+  rv = NS_NewFileSpecFromIFile(prefLocal, getter_AddRefs(outSpec));
   if (NS_FAILED(rv)) return rv;
   
   *spec = outSpec;
@@ -771,16 +808,22 @@ nsMsgIncomingServer::StorePassword()
     rv = GetPassword(getter_Copies(pwd));
     if (NS_FAILED(rv)) return rv;
 
-    nsCOMPtr<nsIWalletService> walletservice = 
-             do_GetService(kWalletServiceCID, &rv);
+    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsXPIDLCString serverSpec;
+    rv = GetServerURI(getter_Copies(serverSpec));
     if (NS_FAILED(rv)) return rv;
 
-    nsXPIDLCString serverUri;
-    rv = GetServerURI(getter_Copies(serverUri));
-    if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIURI> uri;
+    NS_NewURI(getter_AddRefs(uri), serverSpec);
 
-    nsAutoString password; password.AssignWithConversion((const char *)pwd);
-    rv = walletservice->SI_StorePassword((const char *)serverUri, nsnull, password.get());
+    //this is need to make sure wallet service has been created
+    rv = CreateServicesForPasswordManager();
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = observerService->NotifyObservers(uri, "login-succeeded", NS_ConvertUTF8toUCS2(pwd).get());
+    NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
 
@@ -788,20 +831,25 @@ NS_IMETHODIMP
 nsMsgIncomingServer::ForgetPassword()
 {
     nsresult rv;
-    nsCOMPtr<nsIWalletService> walletservice = 
-             do_GetService(kWalletServiceCID, &rv);
+
+    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsXPIDLCString serverSpec;
+    rv = GetServerURI(getter_Copies(serverSpec));
     if (NS_FAILED(rv)) return rv;
 
-    
-    nsXPIDLCString serverUri;
-    rv = GetServerURI(getter_Copies(serverUri));
-    if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIURI> uri;
+    NS_NewURI(getter_AddRefs(uri), serverSpec);
+
+    //this is need to make sure wallet service has been created
+    rv = CreateServicesForPasswordManager();
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = observerService->NotifyObservers(uri, "login-failed", nsnull);
+    NS_ENSURE_SUCCESS(rv,rv);
 
     rv = SetPassword("");
-    if (NS_FAILED(rv)) return rv;
-    
-    
-    rv = walletservice->SI_RemoveUser((const char *)serverUri, nsnull);
     return rv;
 }
 
@@ -994,10 +1042,16 @@ nsMsgIncomingServer::clearPrefEnum(const char *aPref, void *aClosure)
     prefs->ClearUserPref(aPref);
 }
 
-nsresult
-nsMsgIncomingServer::GetFilterList(nsIMsgFilterList **aResult)
+NS_IMETHODIMP
+nsMsgIncomingServer::SetFilterList(nsIMsgFilterList *aFilterList)
 {
+  mFilterList = aFilterList;
+  return NS_OK;
+}
 
+nsresult
+nsMsgIncomingServer::GetFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilterList **aResult)
+{
   nsresult rv;
   
   if (!mFilterList) {
@@ -1021,12 +1075,11 @@ nsMsgIncomingServer::GetFilterList(nsIMsgFilterList **aResult)
           do_GetService(kMsgFilterServiceCID, &rv);
       NS_ENSURE_SUCCESS(rv, rv);
       
-      rv = filterService->OpenFilterList(mFilterFile, msgFolder, getter_AddRefs(mFilterList));
+      rv = filterService->OpenFilterList(mFilterFile, msgFolder, aMsgWindow, getter_AddRefs(mFilterList));
       NS_ENSURE_SUCCESS(rv, rv);
   }
   
-  *aResult = mFilterList;
-  NS_IF_ADDREF(*aResult);
+  NS_IF_ADDREF(*aResult = mFilterList);
   return NS_OK;
     
 }
@@ -1135,7 +1188,7 @@ nsMsgIncomingServer::GetRealHostName(char **aResult)
   nsresult rv;
   rv = GetCharValue("realhostname", aResult);
   NS_ENSURE_SUCCESS(rv, rv);
-  if (!*aResult || (nsCRT::strlen(*aResult) == 0))
+  if (!*aResult || (strlen(*aResult) == 0))
     return(GetHostName(aResult));
 
   if (PL_strchr(*aResult, ':'))
@@ -1153,7 +1206,7 @@ nsMsgIncomingServer::GetRealUsername(char **aResult)
   nsresult rv;
   rv = GetCharValue("realuserName", aResult);
   NS_ENSURE_SUCCESS(rv, rv);
-  if (!*aResult || (nsCRT::strlen(*aResult) == 0))
+  if (!*aResult || (strlen(*aResult) == 0))
     return(GetUsername(aResult));
 
   return rv;
@@ -1563,3 +1616,166 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIntAttribute(const char *aName, PRInt32 *v
 {
   return GetIntValue(aName, val);
 }
+
+// Check if the password is available and return a boolean indicating whether 
+// it is being authenticated or not.
+NS_IMETHODIMP 
+nsMsgIncomingServer::GetIsAuthenticated(PRBool *isAuthenticated)
+{
+  nsresult rv = NS_OK;
+  NS_ENSURE_ARG_POINTER(isAuthenticated);
+
+  *isAuthenticated = PR_FALSE;
+  // If the password is empty, check to see if it is stored and to be retrieved
+  if (m_password.IsEmpty()) {
+    nsCOMPtr <nsIPasswordManagerInternal> passwordMgrInt = do_GetService(NS_PASSWORDMANAGER_CONTRACTID, &rv);
+    if(NS_SUCCEEDED(rv) && passwordMgrInt) {
+
+      // Get the current server URI
+      nsXPIDLCString currServerUri;
+      rv = GetServerURI(getter_Copies(currServerUri));
+      NS_ENSURE_SUCCESS(rv, rv);
+
+      // Obtain the server URI which is in the format <protocol>://<userid>@<hostname>.
+      // Password manager uses the same format when it stores the password on user's request.
+
+      nsCAutoString hostFound;
+      nsAutoString userNameFound;
+      nsAutoString passwordFound;
+
+      // Get password entry corresponding to the host URI we are passing in.
+      rv = passwordMgrInt->FindPasswordEntry(currServerUri, nsString(), nsString(),
+                                             hostFound, userNameFound, passwordFound);
+      if (NS_FAILED(rv)) {
+        return rv;
+      }
+
+      // If a match is found, password element is filled in. Convert the
+      // obtained password and store it for the session.
+      if (!passwordFound.IsEmpty()) {
+        rv = SetPassword(NS_ConvertUCS2toUTF8(passwordFound).get());
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+    }
+  }
+  *isAuthenticated = !m_password.IsEmpty();
+  return rv;
+}
+
+NS_IMETHODIMP
+nsMsgIncomingServer::ConfigureTemporaryReturnReceiptsFilter(nsIMsgFilterList *filterList)
+{
+  NS_ENSURE_ARG_POINTER(filterList);
+  nsresult rv;
+
+  nsCOMPtr<nsIMsgAccountManager> accountMgr = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIMsgIdentity> identity;
+  rv = accountMgr->GetFirstIdentityForServer(this, getter_AddRefs(identity));
+  NS_ENSURE_SUCCESS(rv, rv);
+  
+  PRBool useCustomPrefs = PR_FALSE;
+  PRInt32 incorp = nsIMsgMdnGenerator::eIncorporateInbox;
+
+  identity->GetBoolAttribute("use_custom_prefs", &useCustomPrefs);
+  if (useCustomPrefs)
+    rv = GetIntValue("incorporate_return_receipt", &incorp);
+  else
+  {
+    nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv, rv);
+    nsCOMPtr<nsIPrefBranch> prefBranch;
+    rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+    NS_ENSURE_SUCCESS(rv, rv);
+    rv = prefBranch->GetIntPref("mail.incorporate.return_receipt", &incorp);
+  }
+
+  PRBool enable = (incorp == nsIMsgMdnGenerator::eIncorporateSent);
+
+  // this is a temporary, internal mozilla filter
+  // it will not show up in the UI, it will not be written to disk
+  NS_NAMED_LITERAL_STRING(internalReturnReceiptFilterName, "mozilla-temporary-internal-MDN-receipt-filter");
+
+  nsCOMPtr<nsIMsgFilter> newFilter;
+  rv = filterList->GetFilterNamed(internalReturnReceiptFilterName.get(),
+                                  getter_AddRefs(newFilter));
+  if (newFilter)
+      newFilter->SetEnabled(enable);
+  else if (enable)
+  {
+    nsCOMPtr<nsIMsgFolder> rootFolder;
+    rv = GetRootMsgFolder(getter_AddRefs(rootFolder));
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    PRUint32 numFolders;
+    nsCOMPtr<nsIMsgFolder> sentFolder;
+
+    rootFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_SENTMAIL, 1,
+                                   &numFolders,
+                                   getter_AddRefs(sentFolder));
+    if (sentFolder)
+    {
+      filterList->CreateFilter(internalReturnReceiptFilterName.get(),
+                               getter_AddRefs(newFilter));
+      if (newFilter)
+      {
+        newFilter->SetEnabled(PR_TRUE);
+        // this internal filter is temporary
+        // and should not show up in the UI or be written to disk
+        newFilter->SetTemporary(PR_TRUE);  
+       
+        nsCOMPtr<nsIMsgSearchTerm> term;
+        nsCOMPtr<nsIMsgSearchValue> value;
+        
+        rv = newFilter->CreateTerm(getter_AddRefs(term));
+        if (NS_SUCCEEDED(rv))
+        {
+          rv = term->GetValue(getter_AddRefs(value));
+          if (NS_SUCCEEDED(rv))
+          {
+            // XXX todo
+            // determine if ::OtherHeader is the best way to do this.
+            // see nsMsgSearchOfflineMail::MatchTerms()
+            value->SetAttrib(nsMsgSearchAttrib::OtherHeader);
+            value->SetStr(NS_LITERAL_STRING("multipart/report").get());
+            term->SetAttrib(nsMsgSearchAttrib::OtherHeader);  
+            term->SetOp(nsMsgSearchOp::Contains);
+            term->SetBooleanAnd(PR_TRUE);
+            term->SetArbitraryHeader("Content-Type");
+            term->SetValue(value);
+            newFilter->AppendTerm(term);
+          }
+        }
+        rv = newFilter->CreateTerm(getter_AddRefs(term));
+        if (NS_SUCCEEDED(rv))
+        {
+          rv = term->GetValue(getter_AddRefs(value));
+          if (NS_SUCCEEDED(rv))
+          {
+            // XXX todo
+            // determine if ::OtherHeader is the best way to do this.
+            // see nsMsgSearchOfflineMail::MatchTerms()
+            value->SetAttrib(nsMsgSearchAttrib::OtherHeader);
+            value->SetStr(NS_LITERAL_STRING("disposition-notification").get());
+            term->SetAttrib(nsMsgSearchAttrib::OtherHeader);
+            term->SetOp(nsMsgSearchOp::Contains);
+            term->SetBooleanAnd(PR_TRUE);
+            term->SetArbitraryHeader("Content-Type");
+            term->SetValue(value);
+            newFilter->AppendTerm(term);
+          }
+        }
+        newFilter->SetAction(nsMsgFilterAction::MoveToFolder);
+        nsXPIDLCString actionTargetFolderUri;
+        rv = sentFolder->GetURI(getter_Copies(actionTargetFolderUri));
+        if (NS_SUCCEEDED(rv))
+        {
+          newFilter->SetActionTargetFolderUri(actionTargetFolderUri);
+          filterList->InsertFilterAt(0, newFilter);
+        }
+      }
+    }
+  }
+  return rv;
+}
