@@ -59,6 +59,7 @@
 #include "nsMsgSimulateError.h"
 #include "nsITextToSubURI.h"
 #include "nsEscape.h"
+#include "nsIFileURL.h"
 #include "nsNetCID.h"
 
 
@@ -149,6 +150,9 @@ nsMsgAttachmentHandler::~nsMsgAttachmentHandler()
   if (mFileSpec && mDeleteFile)
     mFileSpec->Delete(PR_FALSE);
 
+  delete mFileSpec;
+  mFileSpec=nsnull;
+
   PR_FREEIF(m_charset);
   PR_FREEIF(m_type);
   PR_FREEIF(m_content_id);
@@ -360,7 +364,7 @@ nsMsgAttachmentHandler::PickEncoding(const char *charset, nsIMsgSend *mime_deliv
     
     if (mURL)
     {
-      mURL->GetSpec(getter_Copies(turl));
+      mURL->GetSpec(turl);
       
       tailName = PL_strrchr(turl, '/');
       if (tailName) 
@@ -602,7 +606,7 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
   }
   mOutFile = do_QueryInterface(outputStream);
 
-  mURL->GetSpec(getter_Copies(url_string));
+  mURL->GetSpec(url_string);
 
 #ifdef XP_MAC
   if ( !m_bogus_attachment && nsMsgIsLocalFile(url_string))
@@ -629,8 +633,10 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
       m_x_mac_creator = PL_strdup(filetype);
     }
 
+    long dataSize = 0;
+    long resSize = 0;
+  
     PRBool sendResourceFork = PR_TRUE;
-    PRBool sendDataFork = PR_TRUE;
     PRBool icGaveNeededInfo = PR_FALSE;
  
     nsCOMPtr<nsIInternetConfigService> icService (do_GetService(NS_INTERNETCONFIGSERVICE_CONTRACTID));
@@ -641,7 +647,14 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
       if (NS_SUCCEEDED(rv) && icFlags != -1 && !(icFlags & nsIInternetConfigService::eIICMapFlag_NotOutgoingMask))
       {
         sendResourceFork = (icFlags & nsIInternetConfigService::eIICMapFlag_ResourceForkMask);
-        sendDataFork = (icFlags & nsIInternetConfigService::eIICMapFlag_DataForkMask);
+
+        if (sendResourceFork)
+        {
+          // before deciding to send the resource fork along the data fork, check if we have one,
+          // else don't need to use apple double.
+          if (::FSpGetFileSize(&fsSpec, &dataSize, &resSize) == noErr && resSize == 0)
+            sendResourceFork = PR_FALSE;
+        }
         
         icGaveNeededInfo = PR_TRUE;
       }
@@ -650,9 +663,6 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
     if (! icGaveNeededInfo)
     {
       // If InternetConfig cannot help us, then just try our best...
-      long dataSize = 0;
-      long resSize = 0;
-  
       // first check if we have a resource fork
       if (::FSpGetFileSize(&fsSpec, &dataSize, &resSize) == noErr)
         sendResourceFork = (resSize > 0);
@@ -666,24 +676,23 @@ nsMsgAttachmentHandler::SnarfAttachment(nsMsgCompFields *compFields)
           nsresult rv = fileUrl->SetSpec(url_string);
           if (NS_SUCCEEDED(rv))
           {
-            char  *ext;
-            rv = fileUrl->GetFileExtension(&ext);
-            if (NS_SUCCEEDED(rv) && ext && *ext)
+            nsCAutoString ext;
+            rv = fileUrl->GetFileExtension(ext);
+            if (NS_SUCCEEDED(rv) && !ext.IsEmpty())
             {
               sendResourceFork =
-                 PL_strcasecmp(ext, "TXT") &&
-                 PL_strcasecmp(ext, "JPG") &&
-                 PL_strcasecmp(ext, "GIF") &&
-                 PL_strcasecmp(ext, "TIF") &&
-                 PL_strcasecmp(ext, "HTM") &&
-                 PL_strcasecmp(ext, "HTML") &&
-                 PL_strcasecmp(ext, "ART") &&
-                 PL_strcasecmp(ext, "XUL") &&
-                 PL_strcasecmp(ext, "XML") &&
-                 PL_strcasecmp(ext, "CSS") &&
-                 PL_strcasecmp(ext, "JS");
+                 PL_strcasecmp(ext.get(), "TXT") &&
+                 PL_strcasecmp(ext.get(), "JPG") &&
+                 PL_strcasecmp(ext.get(), "GIF") &&
+                 PL_strcasecmp(ext.get(), "TIF") &&
+                 PL_strcasecmp(ext.get(), "HTM") &&
+                 PL_strcasecmp(ext.get(), "HTML") &&
+                 PL_strcasecmp(ext.get(), "ART") &&
+                 PL_strcasecmp(ext.get(), "XUL") &&
+                 PL_strcasecmp(ext.get(), "XML") &&
+                 PL_strcasecmp(ext.get(), "CSS") &&
+                 PL_strcasecmp(ext.get(), "JS");
             }
-            PR_FREEIF(ext);
           }
         }
       }
@@ -889,7 +898,7 @@ nsMsgAttachmentHandler::LoadDataFromFile(nsFileSpec& fSpec, nsString &sigData, P
   readBuf = (char *)PR_Malloc(readSize + 1);
   if (!readBuf)
     return NS_ERROR_OUT_OF_MEMORY;
-  nsCRT::memset(readBuf, 0, readSize + 1);
+  memset(readBuf, 0, readSize + 1);
 
   readSize = tempFile.read(readBuf, readSize);
   tempFile.close();
@@ -985,7 +994,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
     if (m_real_name && *m_real_name)
       printfString = nsTextFormatter::smprintf(msg, m_real_name);
     else
-    if (NS_SUCCEEDED(mURL->GetSpec(getter_Copies(turl))) && (turl))
+    if (NS_SUCCEEDED(mURL->GetSpec(turl)) && (turl))
       {
         nsCAutoString unescapeUrl(turl);
         nsUnescape (NS_CONST_CAST(char*, unescapeUrl.get()));
@@ -1068,7 +1077,7 @@ nsMsgAttachmentHandler::UrlExit(nsresult status, const PRUnichar* aMsg)
               tData = ToNewCString(conData);
             if (tData)
             {
-              (void) tempfile.write(tData, nsCRT::strlen(tData));
+              (void) tempfile.write(tData, strlen(tData));
               PR_FREEIF(tData);
             }
             tempfile.close();
