@@ -66,13 +66,11 @@ nsMsgCompFields::nsMsgCompFields()
   m_useMultipartAlternative = PR_FALSE;
   m_uuEncodeAttachments = PR_FALSE;
   m_returnReceipt = PR_FALSE;
-  m_receiptType = 0;
+  m_bodyIsAsciiOnly = PR_FALSE;
 
   nsCOMPtr<nsIPref> prefs (do_GetService(NS_PREF_CONTRACTID));
   if (prefs) 
   {
-    prefs->GetBoolPref("mail.request.return_receipt_on", &m_returnReceipt);
-    prefs->GetIntPref("mail.request.return_receipt", &m_receiptType);
     // Get the default charset from pref, use this as a mail charset.
     nsXPIDLString charset;
     prefs->GetLocalizedUnicharPref("mailnews.send_default_charset", getter_Copies(charset));
@@ -99,7 +97,8 @@ nsMsgCompFields::~nsMsgCompFields()
 
 nsresult nsMsgCompFields::SetAsciiHeader(MsgHeaderID header, const char *value)
 {
-  NS_VERIFY (header >= 0 && header < MSG_MAX_HEADERS, "Invalid message header index!");
+  NS_ASSERTION(header >= 0 && header < MSG_MAX_HEADERS,
+               "Invalid message header index!");
 
   int rv = NS_OK;
   char* old = m_headers[header]; /* Done with careful paranoia, in case the
@@ -126,7 +125,9 @@ nsresult nsMsgCompFields::SetAsciiHeader(MsgHeaderID header, const char *value)
 
 const char* nsMsgCompFields::GetAsciiHeader(MsgHeaderID header)
 {
-  NS_VERIFY (header >= 0 && header < MSG_MAX_HEADERS, "Invalid message header index!");
+  NS_ASSERTION(header >= 0 && header < MSG_MAX_HEADERS,
+               "Invalid message header index!");
+
   return m_headers[header] ? m_headers[header] : "";
 }
 
@@ -395,6 +396,18 @@ NS_IMETHODIMP nsMsgCompFields::GetReturnReceipt(PRBool *_retval)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMsgCompFields::SetReceiptHeaderType(PRInt32 value)
+{
+    m_receiptHeaderType = value;
+    return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgCompFields::GetReceiptHeaderType(PRInt32 *_retval)
+{
+    *_retval = m_receiptHeaderType;
+    return NS_OK;
+}
+
 NS_IMETHODIMP nsMsgCompFields::SetAttachVCard(PRBool value)
 {
   m_attachVCard = value;
@@ -443,6 +456,20 @@ NS_IMETHODIMP nsMsgCompFields::GetUuEncodeAttachments(PRBool *_retval)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsMsgCompFields::SetBodyIsAsciiOnly(PRBool value)
+{
+  m_bodyIsAsciiOnly = value;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgCompFields::GetBodyIsAsciiOnly(PRBool *_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  *_retval = m_bodyIsAsciiOnly;
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsMsgCompFields::SetBody(const PRUnichar *value)
 {
   PR_FREEIF(m_body);
@@ -783,3 +810,18 @@ NS_IMETHODIMP nsMsgCompFields::GetDefaultCharacterSet(char * *aDefaultCharacterS
   *aDefaultCharacterSet = nsCRT::strdup(m_DefaultCharacterSet.get());
   return *aDefaultCharacterSet ? NS_OK : NS_ERROR_OUT_OF_MEMORY; 
 }
+
+NS_IMETHODIMP nsMsgCompFields::CheckCharsetConversion(char **fallbackCharset, PRBool *_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+
+  nsCAutoString headers;
+  for (PRInt16 i = 0; i < MSG_MAX_HEADERS; i++)
+    headers.Append(m_headers[i]);
+
+  // charset conversion check
+  *_retval = nsMsgI18Ncheck_data_in_charset_range(GetCharacterSet(), NS_ConvertUTF8toUCS2(headers.get()).get(),
+                                                  fallbackCharset);
+
+  return NS_OK;
+}
