@@ -60,10 +60,49 @@
 #include "nsMsgUtils.h"
 #include "nsMsgSimulateError.h"
 
+#include "nsIMsgCompUtils.h"
+#include "nsIMsgMdnGenerator.h"
+
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID); 
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kHTTPHandlerCID, NS_HTTPPROTOCOLHANDLER_CID);
 
+NS_IMPL_ISUPPORTS1(nsMsgCompUtils, nsIMsgCompUtils)
+
+nsMsgCompUtils::nsMsgCompUtils()
+{
+  NS_INIT_ISUPPORTS();
+}
+
+nsMsgCompUtils::~nsMsgCompUtils()
+{
+}
+
+NS_IMETHODIMP nsMsgCompUtils::MimeMakeSeparator(const char *prefix,
+                                                char **_retval)
+{
+  NS_ENSURE_ARG_POINTER(prefix);
+  NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = mime_make_separator(prefix);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgCompUtils::MsgGenerateMessageId(nsIMsgIdentity *identity,
+                                                    char **_retval)
+{
+  NS_ENSURE_ARG_POINTER(identity);
+  NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = msg_generate_message_id(identity);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsMsgCompUtils::GetMsgMimeConformToStandard(PRBool *_retval)
+{
+  NS_ENSURE_ARG_POINTER(_retval);
+  *_retval = nsMsgMIMEGetConformToStandard();
+  return NS_OK;
+}
+
 //
 // Hopefully, someone will write and XP call like this eventually!
 //
@@ -220,6 +259,19 @@ nsMsgStripLine (char * string)
 //
 #define UA_PREF_PREFIX "general.useragent."
 
+#define ENCODE_AND_PUSH(name, structured, body, charset, usemime) \
+  { \
+    PUSH_STRING((name)); \
+    convbuf = nsMsgI18NEncodeMimePartIIStr((body), (structured), (charset), strlen(name), (usemime)); \
+    if (convbuf) { \
+      PUSH_STRING (convbuf); \
+      PR_FREEIF(convbuf); \
+    } \
+    else \
+      PUSH_STRING((body)); \
+    PUSH_NEWLINE (); \
+  }
+
 char * 
 mime_generate_headers (nsMsgCompFields *fields,
                        const char *charset,
@@ -234,7 +286,8 @@ mime_generate_headers (nsMsgCompFields *fields,
     return nsnull;
   }
 
-  int size = 0;
+  PRBool usemime = nsMsgMIMEGetConformToStandard();
+  PRInt32 size = 0;
   char *buffer = 0, *buffer_tail = 0;
   PRBool isDraft =
   deliver_mode == nsIMsgSend::nsMsgSaveAsDraft ||
@@ -253,8 +306,9 @@ mime_generate_headers (nsMsgCompFields *fields,
   const char* pPriority;
   const char* pReference;
   const char* pOtherHdr;
+  char *convbuf;
 
-  nsCAutoString headerBuf;    // accumulate header strings for charset conversion check
+  nsCAutoString headerBuf;    // accumulate header strings to get length
   headerBuf.Truncate(0);
 
   NS_ASSERTION (fields, "null fields");
@@ -289,19 +343,6 @@ mime_generate_headers (nsMsgCompFields *fields,
   /* Multiply by 3 here to make enough room for MimePartII conversion */
   size += 3 * headerBuf.Length();
 
-  const char *pBcc = fields->GetBcc();
-  if (pBcc)
-    headerBuf.Append(pBcc);
-  // alert the user if headers contain characters out of charset range (e.g. multilingual data)
-  if (!nsMsgI18Ncheck_data_in_charset_range(charset, NS_ConvertUTF8toUCS2(headerBuf.get()).get())) {
-    PRBool proceedTheSend;
-    rv = nsMsgAskBooleanQuestionByID(aPrompt, NS_MSG_MULTILINGUAL_SEND, &proceedTheSend);
-    if (!proceedTheSend) {
-      *status = NS_ERROR_ABORT;
-      return nsnull;
-    }
-  }
-
   /* Add a bunch of slop for the static parts of the headers. */
   /* size += 2048; */
   size += 2560;
@@ -313,7 +354,6 @@ mime_generate_headers (nsMsgCompFields *fields,
   buffer_tail = buffer;
 
   if (pMessageID && *pMessageID) {
-    char *convbuf = NULL;
     PUSH_STRING ("Message-ID: ");
     PUSH_STRING (pMessageID);
     PUSH_NEWLINE ();
@@ -322,63 +362,33 @@ mime_generate_headers (nsMsgCompFields *fields,
     * coorelate the MDN reports to the original message. Here will be
     * the right place
     */
+
     if (fields->GetReturnReceipt() && 
-      (fields->GetReturnReceiptType() == 2 ||
-      fields->GetReturnReceiptType() == 3) && 
       (deliver_mode != nsIMsgSend::nsMsgSaveAsDraft &&
       deliver_mode != nsIMsgSend::nsMsgSaveAsTemplate))
     {
-      PRInt32 receipt_header_type = 0;
+        PRInt32 receipt_header_type = nsIMsgMdnGenerator::eDntType;
+        fields->GetReceiptHeaderType(&receipt_header_type);
 
-      if (prefs) 
-        prefs->GetIntPref("mail.receipt.request_header_type", &receipt_header_type);
-
-      // 0 = MDN Disposition-Notification-To: ; 1 = Return-Receipt-To: ; 2 =
-      // both MDN DNT & RRT headers
-      if (receipt_header_type == 1) {
-        RRT_HEADER:
-        PUSH_STRING ("Return-Receipt-To: ");
-        convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pFrom, charset, nsMsgMIMEGetConformToStandard());
-        if (convbuf) {    /* MIME-PartII conversion */
-          PUSH_STRING (convbuf);
-          PR_FREEIF(convbuf);
-        }
-        else
-          PUSH_STRING (pFrom);
-        PUSH_NEWLINE ();
+        // nsIMsgMdnGenerator::eDntType = MDN Disposition-Notification-To: ;
+        // nsIMsgMdnGenerator::eRrtType  = Return-Receipt-To: ; 
+        // nsIMsgMdnGenerator::eDntRrtType = both MDN DNT & RRT headers
+      if (receipt_header_type == nsIMsgMdnGenerator::eRrtType) {
+RRT_HEADER:
+        ENCODE_AND_PUSH("Return-Receipt-To: ", PR_TRUE, pFrom, charset, usemime);
       }
       else  {
-        PUSH_STRING ("Disposition-Notification-To: ");
-        convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pFrom, charset,
-                        nsMsgMIMEGetConformToStandard());
-        if (convbuf) {     /* MIME-PartII conversion */
-          PUSH_STRING (convbuf);
-          PR_FREEIF(convbuf);
-        }
-        else
-          PUSH_STRING (pFrom);
-        PUSH_NEWLINE ();
-        if (receipt_header_type == 2)
+        ENCODE_AND_PUSH("Disposition-Notification-To: ", PR_TRUE, pFrom, charset, usemime);
+        if (receipt_header_type == nsIMsgMdnGenerator::eDntRrtType)
           goto RRT_HEADER;
       }
     }
+
 #ifdef SUPPORT_X_TEMPLATE_NAME
     if (deliver_mode == MSG_SaveAsTemplate) {
-      PUSH_STRING ("X-Template: ");
-      char * pStr = fields->GetTemplateName();
-      if (pStr) {
-        convbuf = nsMsgI18NEncodeMimePartIIStr((char *)
-                   pStr,
-                   charset,
-                   nsMsgMIMEGetConformToStandard());
-        if (convbuf) {
-          PUSH_STRING (convbuf);
-          PR_FREEIF(convbuf);
-        }
-        else
-          PUSH_STRING(pStr);
-      }
-      PUSH_NEWLINE ();
+      const char *pStr = fields->GetTemplateName();
+      pStr = pStr ? pStr : "";
+      ENCODE_AND_PUSH("X-Template: ", PR_FALSE, pStr, charset, usemime);
     }
 #endif /* SUPPORT_X_TEMPLATE_NAME */
   }
@@ -404,49 +414,19 @@ mime_generate_headers (nsMsgCompFields *fields,
         ((gmtoffset >= 0 ? gmtoffset : -gmtoffset) % 60));
   buffer_tail += PL_strlen (buffer_tail);
 
-  if (pFrom && *pFrom) {
-    char *convbuf;
-    PUSH_STRING ("From: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pFrom, charset,
-                    nsMsgMIMEGetConformToStandard());
-
-    if (convbuf) {    /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pFrom);
-    PUSH_NEWLINE ();
+  if (pFrom && *pFrom) 
+  {
+    ENCODE_AND_PUSH("From: ", PR_TRUE, pFrom, charset, usemime);
   }
 
   if (pReplyTo && *pReplyTo)
   {
-    char *convbuf;
-    PUSH_STRING ("Reply-To: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pReplyTo, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf) {     /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pReplyTo);
-    PUSH_NEWLINE ();
+    ENCODE_AND_PUSH("Reply-To: ", PR_TRUE, pReplyTo, charset, usemime);
   }
 
   if (pOrg && *pOrg)
   {
-    char *convbuf;
-    PUSH_STRING ("Organization: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pOrg, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf) {     /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pOrg);
-    PUSH_NEWLINE ();
+    ENCODE_AND_PUSH("Organization: ", PR_FALSE, pOrg, charset, usemime);
   }
 
   // X-Mozilla-Draft-Info
@@ -460,7 +440,13 @@ mime_generate_headers (nsMsgCompFields *fields,
       PUSH_STRING("vcard=0");
     PUSH_STRING("; ");
     if (fields->GetReturnReceipt()) {
-      char *type = PR_smprintf("%d", (int) fields->GetReturnReceiptType());
+      // slight change compared to 4.x; we used to use receipt= to tell
+      // whether the draft/template has request for either MDN or DNS or both
+      // return receipt; since the DNS is out of the picture we now use the
+      // header type + 1 to tell whether user has requested the return receipt
+      PRInt32 headerType = 0;
+      fields->GetReceiptHeaderType(&headerType);
+      char *type = PR_smprintf("%d", (int)headerType + 1);
       if (type)
       {
         PUSH_STRING("receipt=");
@@ -484,10 +470,10 @@ mime_generate_headers (nsMsgCompFields *fields,
            do_GetService(kHTTPHandlerCID, &rv); 
   if (NS_SUCCEEDED(rv) && pHTTPHandler)
   {
-        nsXPIDLCString userAgentString;
-        pHTTPHandler->GetUserAgent(getter_Copies(userAgentString));
+        nsCAutoString userAgentString;
+        pHTTPHandler->GetUserAgent(userAgentString);
 
-    if (userAgentString)
+    if (!userAgentString.IsEmpty())
     {
       // PUSH_STRING ("X-Mailer: ");  // To be more standards compliant
       PUSH_STRING ("User-Agent: ");  
@@ -509,26 +495,14 @@ mime_generate_headers (nsMsgCompFields *fields,
   if (pNewsGrp && *pNewsGrp) {
     /* turn whitespace into a comma list
     */
-    char *ptr, *ptr2;
-    char *n2;
-    char *convbuf;
-
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pNewsGrp, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf)
-        n2 = nsMsgStripLine (convbuf);
-    else {
-      ptr = PL_strdup(pNewsGrp);
-      if (!ptr) {
-        PR_FREEIF(buffer);
-        return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
-      }
-          n2 = nsMsgStripLine(ptr);
-      NS_ASSERTION(n2 == ptr, "n2 != ptr"); /* Otherwise, the PR_Free below is
-                             gonna choke badly. */
+    char *duppedNewsGrp = PL_strdup(pNewsGrp);
+    if (!duppedNewsGrp) {
+      PR_FREEIF(buffer);
+      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
     }
+    char *n2 = nsMsgStripLine(duppedNewsGrp);
 
-    for(ptr=n2; *ptr != '\0'; ptr++) {
+    for(char *ptr = n2; *ptr != '\0'; ptr++) {
       /* find first non white space */
       while(!IS_SPACE(*ptr) && *ptr != ',' && *ptr != '\0')
         ptr++;
@@ -540,7 +514,7 @@ mime_generate_headers (nsMsgCompFields *fields,
         *ptr = ',';
 
       /* find next non white space */
-      ptr2 = ptr+1;
+      char *ptr2 = ptr+1;
       while(IS_SPACE(*ptr2))
         ptr2++;
 
@@ -590,33 +564,21 @@ mime_generate_headers (nsMsgCompFields *fields,
       PUSH_NEWLINE ();
     }
 
-    PR_FREEIF(n2);
+    PR_FREEIF(duppedNewsGrp);
   }
 
   /* #### shamelessly duplicated from above */
   if (pFollow && *pFollow) {
     /* turn whitespace into a comma list
     */
-    char *ptr, *ptr2;
-    char *n2;
-    char *convbuf;
-
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pFollow, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf)
-      n2 = nsMsgStripLine (convbuf);
-    else {
-      ptr = PL_strdup(pFollow);
-      if (!ptr) {
-        PR_FREEIF(buffer);
-        return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
-      }
-      n2 = nsMsgStripLine (ptr);
-      NS_ASSERTION(n2 == ptr, "n2 != ptr"); /* Otherwise, the PR_Free below is
-                             gonna choke badly. */
+    char *duppedFollowup = PL_strdup(pFollow);
+    if (!duppedFollowup) {
+      PR_FREEIF(buffer);
+      return nsnull; /* NS_ERROR_OUT_OF_MEMORY */
     }
+    char *n2 = nsMsgStripLine (duppedFollowup);
 
-    for (ptr=n2; *ptr != '\0'; ptr++) {
+    for (char *ptr = n2; *ptr != '\0'; ptr++) {
       /* find first non white space */
       while(!IS_SPACE(*ptr) && *ptr != ',' && *ptr != '\0')
         ptr++;
@@ -628,7 +590,7 @@ mime_generate_headers (nsMsgCompFields *fields,
         *ptr = ',';
 
       /* find next non white space */
-      ptr2 = ptr+1;
+      char *ptr2 = ptr+1;
       while(IS_SPACE(*ptr2))
         ptr2++;
 
@@ -638,52 +600,20 @@ mime_generate_headers (nsMsgCompFields *fields,
 
     PUSH_STRING ("Followup-To: ");
     PUSH_STRING (n2);
-    PR_Free (n2);
+    PR_Free (duppedFollowup);
     PUSH_NEWLINE ();
   }
 
   if (pTo && *pTo) {
-    char *convbuf;
-    PUSH_STRING ("To: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pTo, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf) {     /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pTo);
-
-    PUSH_NEWLINE ();
+    ENCODE_AND_PUSH("To: ", PR_TRUE, pTo, charset, usemime);
   }
  
   if (pCc && *pCc) {
-    char *convbuf;
-    PUSH_STRING ("CC: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pCc, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf) {   /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pCc);
-    PUSH_NEWLINE ();
+    ENCODE_AND_PUSH("CC: ", PR_TRUE, pCc, charset, usemime);
   }
 
   if (pSubject && *pSubject) {
-
-    char *convbuf;
-    PUSH_STRING ("Subject: ");
-    convbuf = nsMsgI18NEncodeMimePartIIStr((char *)pSubject, charset,
-                    nsMsgMIMEGetConformToStandard());
-    if (convbuf) {  /* MIME-PartII conversion */
-      PUSH_STRING (convbuf);
-      PR_Free(convbuf);
-    }
-    else
-      PUSH_STRING (pSubject);
-    PUSH_NEWLINE ();
+    ENCODE_AND_PUSH("Subject: ", PR_FALSE, pSubject, charset, usemime);
   }
   
   if (pPriority && *pPriority)
@@ -719,7 +649,7 @@ mime_generate_headers (nsMsgCompFields *fields,
       while (references && PL_strlen(references) >= 986 && trimAt) {
         ptr = PL_strchr(trimAt+1, '<');
         if (ptr)
-          nsCRT::memmove(trimAt, ptr, PL_strlen(ptr)+1); // including the
+          memmove(trimAt, ptr, PL_strlen(ptr)+1); // including the
         else
           break;
       }
@@ -809,6 +739,7 @@ mime_generate_attachment_headers (const char *type, const char *encoding,
 
   NS_ASSERTION (encoding, "null encoding");
 
+  PRBool usemime = nsMsgMIMEGetConformToStandard();
   PRInt32 parmFolding = 0;
   if (NS_SUCCEEDED(rv) && prefs) 
     prefs->GetIntPref("mail.strictly_mime.parm_folding", &parmFolding);
@@ -817,12 +748,11 @@ mime_generate_attachment_headers (const char *type, const char *encoding,
   char *encodedRealName = nsnull;
   if (real_name)
   {
-    encodedRealName = nsMsgI18NEncodeMimePartIIStr(real_name,
-      nsMsgI18NFileSystemCharset(), nsMsgMIMEGetConformToStandard());
+    encodedRealName = nsMsgI18NEncodeMimePartIIStr(real_name, PR_FALSE, nsMsgI18NFileSystemCharset(), 0, usemime);
     if (!encodedRealName || !*encodedRealName)
     {
       /* Let's try one more time using UTF8 */
-        encodedRealName = nsMsgI18NEncodeMimePartIIStr(real_name, "UTF-8", nsMsgMIMEGetConformToStandard());
+        encodedRealName = nsMsgI18NEncodeMimePartIIStr(real_name, PR_FALSE, "UTF-8", 0, usemime);
       if (!encodedRealName || !*encodedRealName)
       {
         PR_FREEIF(encodedRealName);
@@ -846,7 +776,8 @@ mime_generate_attachment_headers (const char *type, const char *encoding,
   {
     
     char charset_label[65] = "";   // Content-Type: charset
-    PL_strcpy(charset_label, charset);
+    PL_strncpy(charset_label, charset, sizeof(charset_label)-1);
+    charset_label[sizeof(charset_label)-1] = '\0';
     
     /* If the characters are all 7bit, then it's better (and true) to
     claim the charset to be US-  rather than Latin1.  Should we
@@ -1564,7 +1495,7 @@ msg_pick_real_name (nsMsgAttachmentHandler *attachment, const PRUnichar *propose
   else //Let's extract the name from the URL
   {
     nsXPIDLCString url;
-  attachment->mURL->GetSpec(getter_Copies(url));
+  attachment->mURL->GetSpec(url);
 
   s = url;
   s2 = PL_strchr (s, ':');
@@ -1680,14 +1611,12 @@ nsMsgNewURL(nsIURI** aInstancePtrResult, const char * aSpec)
   {
     if (PL_strstr(aSpec, "://") == nsnull)
     {
-      nsAutoString newSpec(NS_LITERAL_STRING("http://"));
-      newSpec.AppendWithConversion(aSpec);
-      nsCAutoString newspecC;
-      newspecC.AssignWithConversion(newSpec);
-		  rv = pNetService->NewURI(newspecC.get(), nsnull, aInstancePtrResult);
-  	}
-  	else
-		  rv = pNetService->NewURI(aSpec, nsnull, aInstancePtrResult);
+      //XXXjag Temporary fix for bug 139362 until the real problem(bug 70083) get fixed
+      nsCAutoString uri(NS_LITERAL_CSTRING("http://") + nsDependentCString(aSpec));
+      rv = pNetService->NewURI(uri, nsnull, nsnull, aInstancePtrResult);
+    }
+    else
+      rv = pNetService->NewURI(nsDependentCString(aSpec), nsnull, nsnull, aInstancePtrResult);
   }
   return rv;
 }
@@ -1747,19 +1676,19 @@ char *
 nsMsgParseURLHost(const char *url)
 {
   nsIURI        *workURI = nsnull;
-  char          *retVal = nsnull;
   nsresult      rv;
 
   rv = nsMsgNewURL(&workURI, url);
   if (NS_FAILED(rv) || !workURI)
     return nsnull;
   
-  rv = workURI->GetHost(&retVal);
+  nsCAutoString host;
+  rv = workURI->GetHost(host);
   NS_IF_RELEASE(workURI);
   if (NS_FAILED(rv))
     return nsnull;
-  else
-    return retVal;
+
+  return ToNewCString(host);
 }
 
 char *
@@ -1793,7 +1722,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
   char        *cp = nsnull;
   char        *cp1 = nsnull;
 
-  rv = aURL->GetPath(getter_Copies(file));
+  rv = aURL->GetPath(file);
   if ( NS_SUCCEEDED(rv) && file)
   {
     char *newFile = PL_strdup(file);
@@ -1827,7 +1756,7 @@ GenerateFileNameFromURI(nsIURI *aURL)
   cp1 = nsnull;
 
 
-  rv = aURL->GetSpec(getter_Copies(spec));
+  rv = aURL->GetSpec(spec);
   if ( NS_SUCCEEDED(rv) && spec)
   {
     char *newSpec = PL_strdup(spec);
@@ -1881,8 +1810,8 @@ mime_gen_content_id(PRUint32 aPartNum, const char *aEmailAddress)
   const char        *domain = nsnull;
   const char        *defaultDomain = "@netscape.com";
 
-  nsCRT::memset(rand_buf1, 0, randLen-1);
-  nsCRT::memset(rand_buf2, 0, randLen-1);
+  memset(rand_buf1, 0, randLen-1);
+  memset(rand_buf2, 0, randLen-1);
 
   GenerateGlobalRandomBytes(rand_buf1, randLen);
   GenerateGlobalRandomBytes(rand_buf2, randLen);
@@ -2014,8 +1943,7 @@ ConvertBufToPlainText(nsString &aConBuf, PRBool formatflowed /* = PR_FALSE */)
 
     parser->SetContentSink(sink);
 
-    nsAutoString contentType; contentType = NS_LITERAL_STRING("text/html");
-    parser->Parse(aConBuf, 0, contentType, PR_FALSE, PR_TRUE);
+    parser->Parse(aConBuf, 0, NS_LITERAL_CSTRING("text/html"), PR_FALSE, PR_TRUE);
     //
     // Now if we get here, we need to get from ASCII text to 
     // UTF-8 format or there is a problem downstream...
