@@ -58,6 +58,7 @@
 #include "nsIWindowWatcher.h"
 #include "nsIDOMWindow.h"
 #include "nsEscape.h"
+#include "nsIContentViewer.h"
 
 #include "nsIDocShell.h"
 #include "nsIScriptGlobalObject.h"
@@ -75,6 +76,7 @@
 
 #include "nsMsgBaseCID.h"
 #include "nsIMsgAccountManager.h"
+#include "nsIMsgMdnGenerator.h"
 
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 #include "prlog.h"
@@ -91,6 +93,13 @@ static PRBool _just_to_be_sure_we_create_only_on_compose_service_ = PR_FALSE;
 
 #define PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS  "mail.compose.max_recycled_windows"
 
+#define MAILNEWS_ROOT_PREF                         "mailnews."
+#define HTMLDOMAINUPDATE_VERSION_PREF_NAME         "global_html_domains.version"
+#define HTMLDOMAINUPDATE_DOMAINLIST_PREF_NAME      "global_html_domains"
+#define USER_CURRENT_HTMLDOMAINLIST_PREF_NAME      "html_domains"
+#define USER_CURRENT_PLAINTEXTDOMAINLIST_PREF_NAME "plaintext_domains"
+#define DOMAIN_DELIMITER                           ","
+
 #ifdef MSGCOMP_TRACE_PERFORMANCE
 static PRLogModuleInfo *MsgComposeLogModule = nsnull;
 
@@ -133,8 +142,7 @@ nsMsgComposeService::nsMsgComposeService()
   mCachedWindows = nsnull;
 }
 
-/* the following macro actually implement addref, release and query interface for our component. */
-NS_IMPL_ISUPPORTS3(nsMsgComposeService, nsIMsgComposeService, nsIObserver, nsICmdLineHandler);
+NS_IMPL_ISUPPORTS4(nsMsgComposeService, nsIMsgComposeService, nsIObserver, nsICmdLineHandler, nsISupportsWeakReference);
 
 nsMsgComposeService::~nsMsgComposeService()
 {
@@ -154,21 +162,27 @@ nsresult nsMsgComposeService::Init()
 
   // Register observers
 
-  // register for profile change, we will need to clear the cache.
+  // Register for quit application and profile change, we will need to clear the cache.
   nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
   if (NS_SUCCEEDED(rv))
-    rv = observerService->AddObserver(this, "profile-do-change", PR_FALSE);
+  {
+    rv = observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_TRUE);
+    rv = observerService->AddObserver(this, "profile-do-change", PR_TRUE);
+  }
 
+  // Register some pref observer
   nsCOMPtr<nsIPrefBranch> prefBranch;
   rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
   if (NS_SUCCEEDED(rv))
   {
     nsCOMPtr<nsIPrefBranchInternal> pbi = do_QueryInterface(prefBranch, &rv);
     if (NS_SUCCEEDED(rv))
-      rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, PR_FALSE);
+      rv = pbi->AddObserver(PREF_MAIL_COMPOSE_MAXRECYCLEDWINDOWS, this, PR_TRUE);
   }
 
 	Reset();
+
+  AddGlobalHtmlDomains();
 	
   return rv;
 }
@@ -243,11 +257,14 @@ nsresult nsMsgComposeService::OpenWindow(const char *chrome, nsIMsgComposeParams
       {
         if (mCachedWindows[i].window && (mCachedWindows[i].htmlCompose == composeHTML) && mCachedWindows[i].listener)
         {
-          mCachedWindows[i].listener->OnReopen(params);
-          rv = ShowCachedComposeWindow(mCachedWindows[i].window, PR_TRUE);
+          /* We need to save the window pointer as OnReopen will call nsMsgComposeService::InitCompose which will
+             clear the cache entry if everything goes well
+          */
+          nsCOMPtr<nsIDOMWindowInternal> domWindow(mCachedWindows[i].window);
+          rv = ShowCachedComposeWindow(domWindow, PR_TRUE);
           if (NS_SUCCEEDED(rv))
           {
-            mCachedWindows[i].Clear();
+            mCachedWindows[i].listener->OnReopen(params);
             return NS_OK;
           }
         }
@@ -284,15 +301,21 @@ void nsMsgComposeService::CloseWindow(nsIDOMWindowInternal *domWindow)
     if (globalObj)
     {
       globalObj->GetDocShell(getter_AddRefs(docshell));
-      nsCOMPtr<nsIDocShellTreeItem>  treeItem(do_QueryInterface(docshell));
-      nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
-      treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
-      if (treeItem)
+      if (docshell)
       {
-        nsCOMPtr<nsIBaseWindow> baseWindow;
-        baseWindow = do_QueryInterface(treeOwner);
-        if (baseWindow)
-          baseWindow->Destroy();
+        nsCOMPtr<nsIDocShellTreeItem>  treeItem(do_QueryInterface(docshell));
+        if (treeItem)
+        {
+          nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
+          treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
+          if (treeOwner)
+          {
+            nsCOMPtr<nsIBaseWindow> baseWindow;
+            baseWindow = do_QueryInterface(treeOwner);
+            if (baseWindow)
+              baseWindow->Destroy();
+          }
+        }
       }
     }
   }
@@ -301,7 +324,7 @@ void nsMsgComposeService::CloseWindow(nsIDOMWindowInternal *domWindow)
 NS_IMETHODIMP
 nsMsgComposeService::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *someData)
 {
-  if (!nsCRT::strcmp(aTopic,"profile-do-change"))
+  if (!strcmp(aTopic,"profile-do-change") || !strcmp(aTopic,NS_XPCOM_SHUTDOWN_OBSERVER_ID))
   {
     DeleteCachedWindows();
     return NS_OK;
@@ -352,9 +375,13 @@ nsMsgComposeService::DetermineComposeHTML(nsIMsgIdentity *aIdentity, MSG_Compose
 
 NS_IMETHODIMP
 nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const char *originalMsgURI,
-  MSG_ComposeType type, MSG_ComposeFormat format, nsIMsgIdentity * identity, nsIMsgWindow *aMsgWindow)
+  MSG_ComposeType type, MSG_ComposeFormat format, nsIMsgIdentity * aIdentity, nsIMsgWindow *aMsgWindow)
 {
   nsresult rv;
+
+  nsCOMPtr<nsIMsgIdentity> identity = aIdentity;
+  if (!identity)
+    GetDefaultIdentity(getter_AddRefs(identity));
   
   /* Actually, the only way to implement forward inline is to simulate a template message. 
      Maybe one day when we will have more time we can change that
@@ -370,13 +397,13 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const ch
       switch(type)
       {
         case nsIMsgCompType::ForwardInline:
-            rv = pMsgDraft->OpenDraftMsg(uriToOpen.get(), nsnull, identity, PR_TRUE, aMsgWindow);
+            rv = pMsgDraft->OpenDraftMsg(uriToOpen.get(), originalMsgURI, identity, PR_TRUE, aMsgWindow);
           break;
         case nsIMsgCompType::Draft:
             rv = pMsgDraft->OpenDraftMsg(uriToOpen.get(), nsnull, identity, PR_FALSE, aMsgWindow);
           break;
         case nsIMsgCompType::Template:
-            rv = pMsgDraft->OpenEditorTemplate(uriToOpen.get(), nsnull, identity, aMsgWindow);
+            rv = pMsgDraft->OpenEditorTemplate(uriToOpen.get(), identity, aMsgWindow);
           break;
       }
     }
@@ -400,10 +427,10 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const ch
           nsCAutoString group;
           nsCAutoString host;
         
-          PRInt32 slashpos = newsURI.FindChar('/');
+          PRInt32 slashpos = newsURI.RFindChar('/');
           if (slashpos > 0 )
           {
-            // uri is "host/group"
+            // uri is "[s]news://host[:port]/group"
             newsURI.Left(host, slashpos);
             newsURI.Right(group, newsURI.Length() - slashpos - 1);
           }
@@ -416,6 +443,16 @@ nsMsgComposeService::OpenComposeWindow(const char *msgComposeWindowURL, const ch
       else
         pMsgComposeParams->SetOriginalMsgURI(originalMsgURI);
         
+      PRBool requestReturnReceipt = PR_FALSE;
+      PRInt32 receiptType = nsIMsgMdnGenerator::eDntType;
+      if (identity)
+      {
+        identity->GetRequestReturnReceipt(&requestReturnReceipt);
+        identity->GetReceiptHeaderType(&type);
+      }
+      pMsgCompFields->SetReturnReceipt(requestReturnReceipt);
+      pMsgCompFields->SetReceiptHeaderType(receiptType);
+
       pMsgComposeParams->SetComposeFields(pMsgCompFields);
 
       if(mLogComposePerformance)
@@ -577,6 +614,16 @@ NS_IMETHODIMP nsMsgComposeService::InitCompose(nsIDOMWindowInternal *aWindow,
 {
   nsresult rv;
   
+  /* we need to remove the window from the cache
+  */
+  PRInt32 i;
+  for (i = 0; i < mMaxRecycledWindows; i ++)
+    if (mCachedWindows[i].window == aWindow)
+    {
+      mCachedWindows[i].Clear();
+      break;
+    }
+
   nsCOMPtr <nsIMsgCompose> msgCompose = do_CreateInstance(NS_MSGCOMPOSE_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv,rv);
 	
@@ -675,7 +722,11 @@ nsMsgComposeService::CacheWindow(nsIDOMWindowInternal *aWindow, PRBool aComposeH
   nsresult rv;
 
   PRInt32 i;
+  PRInt32 sameTypeId = -1;
+  PRInt32 oppositeTypeId = -1;
+  
   for (i = 0; i < mMaxRecycledWindows; i ++)
+  {
     if (!mCachedWindows[i].window)
     {
       rv = ShowCachedComposeWindow(aWindow, PR_FALSE);
@@ -684,6 +735,34 @@ nsMsgComposeService::CacheWindow(nsIDOMWindowInternal *aWindow, PRBool aComposeH
       
       return rv;
     }
+    else
+      if (mCachedWindows[i].htmlCompose == aComposeHTML)
+      {
+        if (sameTypeId == -1)
+          sameTypeId = i;
+      }
+      else
+      {
+        if (oppositeTypeId == -1)
+          oppositeTypeId = i;
+      }
+  }
+  
+  /* Looks like the cache is full. In the case we try to cache a type (html or plain text) of compose window which is not
+     already cached, we should replace an opposite one with this new one. That would allow users to be able to take advantage
+     of the cached compose window when they switch from one type to another one
+  */
+  if (sameTypeId == -1 && oppositeTypeId != -1)
+  {
+    CloseWindow(mCachedWindows[oppositeTypeId].window);
+    mCachedWindows[oppositeTypeId].Clear();
+    
+    rv = ShowCachedComposeWindow(aWindow, PR_FALSE);
+    if (NS_SUCCEEDED(rv))
+      mCachedWindows[oppositeTypeId].Initialize(aWindow, aListener, aComposeHTML);
+    
+    return rv;
+  }
   
   return NS_ERROR_NOT_AVAILABLE;
 }
@@ -708,9 +787,33 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
   NS_ENSURE_SUCCESS(rv,rv);
   
   if (webShellContainer) {
+
+    // the window need to be sticky before we hide it.
+    nsCOMPtr<nsIContentViewer> contentViewer;
+    rv = docShell->GetContentViewer(getter_AddRefs(contentViewer));
+    NS_ENSURE_SUCCESS(rv,rv);
+    
+    rv = contentViewer->SetSticky(!aShow);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    // disable (enable) the cached window
     nsCOMPtr <nsIWebShellWindow> webShellWindow = do_QueryInterface(webShellContainer, &rv);
     NS_ENSURE_SUCCESS(rv,rv);
 
+    nsCOMPtr<nsIDocShellTreeItem>  treeItem(do_QueryInterface(docShell, &rv));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
+    rv = treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsCOMPtr<nsIBaseWindow> baseWindow;
+    baseWindow = do_QueryInterface(treeOwner, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    baseWindow->SetEnabled(aShow);
+    NS_ENSURE_SUCCESS(rv,rv);
+
     // hide (show) the cached window
     rv = webShellWindow->Show(aShow);
     NS_ENSURE_SUCCESS(rv,rv);
@@ -735,6 +838,114 @@ nsresult nsMsgComposeService::ShowCachedComposeWindow(nsIDOMWindowInternal *aCom
   return rv;
 }
 
+nsresult nsMsgComposeService::AddGlobalHtmlDomains()
+{
+
+  nsresult rv;
+  nsCOMPtr<nsIPref> prefs = do_GetService(NS_PREF_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  rv = prefs->GetBranch(MAILNEWS_ROOT_PREF, getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIPrefBranch> defaultsPrefBranch;
+  rv = prefs->GetDefaultBranch(MAILNEWS_ROOT_PREF, getter_AddRefs(defaultsPrefBranch));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  /** 
+   * Check to see if we need to add any global domains.
+   * If so, make sure the following prefs are added to mailnews.js
+   *
+   * 1. pref("mailnews.global_html_domains.version", version number);
+   * This pref registers the current version in the user prefs file. A default value is stored
+   * in mailnews file. Depending the changes we plan to make we can move the default version number.
+   * Comparing version number from user's prefs file and the default one from mailnews.js, we
+   * can effect ppropriate changes.
+   *
+   * 2. pref("mailnews.global_html_domains", <comma separated domain list>);
+   * This pref contains the list of html domains that ISP can add to make that user's contain all
+   * of these under the HTML domains in the Mail&NewsGrpus|Send Format under global preferences.
+   */
+  PRInt32 htmlDomainListCurrentVersion, htmlDomainListDefaultVersion;
+  rv = prefBranch->GetIntPref(HTMLDOMAINUPDATE_VERSION_PREF_NAME, &htmlDomainListCurrentVersion);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = defaultsPrefBranch->GetIntPref(HTMLDOMAINUPDATE_VERSION_PREF_NAME, &htmlDomainListDefaultVersion);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  // Update the list as needed
+  if (htmlDomainListCurrentVersion <= htmlDomainListDefaultVersion) {
+    // Get list of global domains need to be added
+    nsXPIDLCString globalHtmlDomainList;
+    rv = prefBranch->GetCharPref(HTMLDOMAINUPDATE_DOMAINLIST_PREF_NAME, getter_Copies(globalHtmlDomainList));
+
+    if (NS_SUCCEEDED(rv) && !globalHtmlDomainList.IsEmpty()) {
+
+      // Get user's current HTML domain set for send format
+      nsXPIDLCString currentHtmlDomainList;
+      rv = prefBranch->GetCharPref(USER_CURRENT_HTMLDOMAINLIST_PREF_NAME, getter_Copies(currentHtmlDomainList));
+      NS_ENSURE_SUCCESS(rv,rv);
+
+      nsCAutoString newHtmlDomainList(currentHtmlDomainList);
+      // Get the current html domain list into new list var
+      nsCStringArray htmlDomainArray;
+      if (!currentHtmlDomainList.IsEmpty()) 
+        htmlDomainArray.ParseString(currentHtmlDomainList.get(), DOMAIN_DELIMITER);
+
+      // Get user's current Plaintext domain set for send format
+      nsXPIDLCString currentPlaintextDomainList;
+      rv = prefBranch->GetCharPref(USER_CURRENT_PLAINTEXTDOMAINLIST_PREF_NAME, getter_Copies(currentPlaintextDomainList));
+      NS_ENSURE_SUCCESS(rv,rv);
+
+      // Get the current plaintext domain list into new list var
+      nsCStringArray plaintextDomainArray;
+      if (!currentPlaintextDomainList.IsEmpty()) 
+        plaintextDomainArray.ParseString(currentPlaintextDomainList.get(), DOMAIN_DELIMITER);
+    
+      if (htmlDomainArray.Count() || plaintextDomainArray.Count()) {
+        // Tokenize the data and add each html domain if it is not alredy there in 
+        // the user's current html or plaintext domain lists
+        char *newData;
+        char *globalData = ToNewCString(globalHtmlDomainList);
+  
+        char *token = nsCRT::strtok(globalData, DOMAIN_DELIMITER, &newData);
+
+        nsCAutoString htmlDomain;
+        while (token) {
+          if (token && *token) {
+            htmlDomain.Assign(token);
+            htmlDomain.StripWhitespace();
+
+            if (htmlDomainArray.IndexOf(htmlDomain) == -1  && 
+                plaintextDomainArray.IndexOf(htmlDomain) == -1) {
+              if (!newHtmlDomainList.IsEmpty())
+                newHtmlDomainList += DOMAIN_DELIMITER;
+              newHtmlDomainList += htmlDomain;
+            }
+          }
+          token = nsCRT::strtok(newData, DOMAIN_DELIMITER, &newData);
+        }
+        PR_FREEIF(globalData);
+      }
+      else
+      {
+        // User has no domains listed either in html or plain text category.
+        // Assign the global list to be the user's current html domain list
+        newHtmlDomainList = globalHtmlDomainList;
+      }
+
+      // Set user's html domain pref with the updated list
+      rv = prefBranch->SetCharPref(USER_CURRENT_HTMLDOMAINLIST_PREF_NAME, newHtmlDomainList.get());
+      NS_ENSURE_SUCCESS(rv,rv);
+
+      // Increase the version to avoid running the update code unless needed (based on default version)
+      rv = prefBranch->SetIntPref(HTMLDOMAINUPDATE_VERSION_PREF_NAME, htmlDomainListCurrentVersion + 1);
+      NS_ENSURE_SUCCESS(rv,rv);
+    }
+  }
+  return NS_OK;
+}
 
 CMDLINEHANDLER_IMPL(nsMsgComposeService, "-compose", "general.startup.messengercompose", DEFAULT_CHROME,
                     "Start with messenger compose.", NS_MSGCOMPOSESTARTUPHANDLER_CONTRACTID, "Messenger Compose Startup Handler",
