@@ -56,7 +56,9 @@
 #include "nsMsgComposeStringBundle.h"
 #include "nsMsgCompUtils.h"
 #include "prcmon.h"
+#include "nsImapCore.h"
 #include "nsIMsgImapMailFolder.h"
+#include "nsIImapIncomingServer.h"
 #include "nsIEventQueueService.h"
 #include "nsMsgSimulateError.h"
 
@@ -423,13 +425,14 @@ nsresult
 LocateMessageFolder(nsIMsgIdentity   *userIdentity, 
                     nsMsgDeliverMode aFolderType,
                     const char       *aFolderURI,
-		    nsIMsgFolder     **msgFolder)
+                    nsIMsgFolder     **msgFolder)
 {
   nsresult                  rv = NS_OK;
 
   RETURN_SIMULATED_ERROR(SIMULATED_SEND_ERROR_5, NS_ERROR_FAILURE)
   
   if (!msgFolder) return NS_ERROR_NULL_POINTER;
+  *msgFolder = nsnull;
 
   if (!aFolderURI || (PL_strlen(aFolderURI) == 0)) {
     return NS_ERROR_INVALID_ARG;
@@ -448,17 +451,53 @@ LocateMessageFolder(nsIMsgIdentity   *userIdentity,
 
     nsCOMPtr <nsIMsgFolder> folderResource;
     folderResource = do_QueryInterface(resource, &rv);
-    if (NS_SUCCEEDED(rv) && folderResource) {
+    if (NS_SUCCEEDED(rv) && folderResource) 
+    {
         // don't check validity of folder - caller will handle creating it
-	*msgFolder = folderResource;
-	NS_ADDREF(*msgFolder);
-	return NS_OK;
+      nsCOMPtr<nsIMsgIncomingServer> server; //make sure that folder hierarchy is built so that legitimate parent-child relationship is established
+      folderResource->GetServer(getter_AddRefs(server));
+      if (server)
+      {
+        nsCOMPtr<nsIMsgFolder> rootMsgFolder;
+        server->GetRootMsgFolder(getter_AddRefs(rootMsgFolder));
+        if (rootMsgFolder)
+        {
+          nsCOMPtr<nsIImapIncomingServer> imapServer = do_QueryInterface(server);
+          // Make sure an specific IMAP folder has correct personal namespace
+          // See bugzilla bug 90494 (http://bugzilla.mozilla.org/show_bug.cgi?id=90494)
+          if (imapServer)
+          {
+            nsXPIDLCString folderUriWithNamespace;
+            imapServer->GetUriWithNamespacePrefixIfNecessary(kPersonalNamespace, aFolderURI, getter_Copies(folderUriWithNamespace));
+            if (!folderUriWithNamespace.IsEmpty())
+              rv = rootMsgFolder->GetChildWithURI(folderUriWithNamespace, PR_TRUE, PR_FALSE, msgFolder);
+            else
+              rv = rootMsgFolder->GetChildWithURI(aFolderURI, PR_TRUE, PR_FALSE, msgFolder);
+          }
+          else
+            rv = rootMsgFolder->GetChildWithURI(aFolderURI, PR_TRUE, imapServer == nsnull /*caseInsensitive*/, msgFolder);
+          /* we didn't find the folder so we will have to create new one.
+             CreateIfMissing does that provided we pass in a dummy folder */
+          if (!*msgFolder)
+          {
+            *msgFolder = folderResource;  
+            NS_ADDREF(*msgFolder);
+          }
+          return rv;
+        }
+        else
+          return NS_MSG_ERROR_FOLDER_MISSING;
+      }
+      else
+	      return NS_MSG_ERROR_FOLDER_MISSING;
     }
-    else {
-	return NS_ERROR_FAILURE;
+    else 
+    {
+	    return NS_ERROR_FAILURE;
     }
   }
-  else {
+  else 
+  {
     PRUint32                  cnt = 0;
     PRUint32                  i;
 
@@ -563,7 +602,7 @@ MessageFolderIsLocal(nsIMsgIdentity   *userIdentity,
   rv = nsComponentManager::CreateInstance(kStandardUrlCID, nsnull, NS_GET_IID(nsIURL), getter_AddRefs(url));
   if (NS_FAILED(rv)) return rv;
 
-  rv = url->SetSpec(aFolderURI);
+  rv = url->SetSpec(nsDependentCString(aFolderURI));
   if (NS_FAILED(rv)) return rv;
  
   /* mailbox:/ means its local (on disk) */
