@@ -211,6 +211,7 @@ public:
   NS_IMETHOD  DeliverFileAsMail();
   NS_IMETHOD  DeliverFileAsNews();
   void        DoDeliveryExitProcessing(nsIURI * aUrl, nsresult aExitCode, PRBool aCheckForMail);
+  nsresult    FormatStringWithSMTPHostNameByID(PRInt32 aMsgId, PRUnichar **aString);
 
   nsresult    DoFcc();
   nsresult    StartMessageCopyOperation(nsIFileSpec        *aFileSpec, 
@@ -225,6 +226,9 @@ public:
   nsresult    SaveInSentFolder();
   nsresult    SaveAsTemplate();
 
+  // Check to see if it's ok to save msgs to the configured folder.
+  PRBool CanSaveMessagesToFolder(const char *folderURL);
+
   //
   // FCC operations...
   //
@@ -307,7 +311,7 @@ public:
   nsCOMPtr<nsIDOMWindowInternal>  mParentWindow;
   nsCOMPtr<nsIMsgProgress>        mSendProgress;
   nsCOMPtr<nsIMsgSendListener>    mListener;
-
+  nsCOMPtr<nsIMsgStatusFeedback>  mStatusFeedback;
   nsCOMPtr<nsIRequest>      mRunningRequest;
   PRBool                    mSendMailAlso;
   nsIFileSpec               *mReturnFileSpec;     // a holder for file spec's to be returned to caller
