@@ -55,7 +55,7 @@ struct mdbOid gAllOfflineOpsTableOID;
 
 
 nsMailDatabase::nsMailDatabase()
-    : m_reparse(PR_FALSE), m_folderSpec(nsnull), m_folderStream(nsnull)
+    : m_reparse(PR_FALSE), m_folderSpec(nsnull), m_folderStream(nsnull), m_ownFolderStream(PR_FALSE)
 {
   m_mdbAllOfflineOpsTable = nsnull;
 }
@@ -68,6 +68,13 @@ nsMailDatabase::~nsMailDatabase()
     m_mdbAllOfflineOpsTable->Release();
 }
 
+NS_IMETHODIMP nsMailDatabase::SetFolderStream(nsIOFileStream *aFileStream)
+{
+  NS_ASSERTION(!m_folderStream || !aFileStream, "m_folderStream is not null and we are assigning a non null stream to it");
+  m_folderStream = aFileStream; //m_folderStream is set externally, so m_ownFolderStream is false
+  m_ownFolderStream = PR_FALSE;
+  return NS_OK;
+}
 
 
 NS_IMETHODIMP nsMailDatabase::Open(nsIFileSpec *aFolderName, PRBool create, PRBool upgrading, nsIMsgDatabase** pMessageDB)
@@ -233,23 +240,27 @@ nsresult nsMailDatabase::OnNewPath (nsFileSpec &newPath)
 // cache m_folderStream to make updating mozilla status flags fast
 NS_IMETHODIMP nsMailDatabase::StartBatch()
 {
-#ifndef XP_MAC
-  if (!m_folderStream)
-	  m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
-#endif  
-return NS_OK;
+  if (!m_folderStream)  //only if we create a stream, set m_ownFolderStream to true.
+  {
+    m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+    m_ownFolderStream = PR_TRUE;
+  }
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsMailDatabase::EndBatch()
 {
-#ifndef XP_MAC
-  if (m_folderStream)
+  if (m_ownFolderStream)   //only if we own the stream, then we should close it
   {
-    m_folderStream->close();  
-    delete m_folderStream;
+    if (m_folderStream)
+    {
+      m_folderStream->close();  
+      delete m_folderStream;
+    }
+    m_folderStream = nsnull;
+    m_ownFolderStream = PR_FALSE;
   }
-  m_folderStream = nsnull;
-#endif
   return NS_OK;
 }
 
@@ -258,14 +269,21 @@ NS_IMETHODIMP nsMailDatabase::DeleteMessages(nsMsgKeyArray* nsMsgKeys, nsIDBChan
 {
 	nsresult ret = NS_OK;
   if (!m_folderStream)
+  {
 	  m_folderStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
+    m_ownFolderStream = PR_TRUE;
+  }
 	ret = nsMsgDatabase::DeleteMessages(nsMsgKeys, instigator);
-	if (m_folderStream)
+  if (m_ownFolderStream)//only if we own the stream, then we should close it
+  {
+	  if (m_folderStream)
     {
       m_folderStream->close();
-	  delete m_folderStream;
+	    delete m_folderStream;
     }
-	m_folderStream = NULL;
+	  m_folderStream = nsnull;
+    m_ownFolderStream = PR_FALSE;
+  }
 	SetFolderInfoValid(m_folderSpec, 0, 0);
 	return ret;
 }
@@ -276,7 +294,7 @@ PRBool nsMailDatabase::SetHdrFlag(nsIMsgDBHdr *msgHdr, PRBool bSet, MsgFlags fla
 {
   nsIOFileStream *fileStream = NULL;
   PRBool		ret = PR_FALSE;
-  
+
   if (nsMsgDatabase::SetHdrFlag(msgHdr, bSet, flag))
   {
     UpdateFolderFlag(msgHdr, bSet, flag, &fileStream);
@@ -288,6 +306,7 @@ PRBool nsMailDatabase::SetHdrFlag(nsIMsgDBHdr *msgHdr, PRBool bSet, MsgFlags fla
     }
     ret = PR_TRUE;
   }
+
   return ret;
 }
 
@@ -313,6 +332,7 @@ void nsMailDatabase::UpdateFolderFlag(nsIMsgDBHdr *mailHdr, PRBool bSet,
 							  MsgFlags flag, nsIOFileStream **ppFileStream)
 {
   static char buf[50];
+  PRInt32 folderStreamPos = 0; //saves the folderStream pos in case we are sharing the stream with other code
   nsIOFileStream *fileStream = (m_folderStream) ? m_folderStream : *ppFileStream;
   //#ifdef GET_FILE_STUFF_TOGETHER
 #ifdef XP_MAC
@@ -338,6 +358,11 @@ void nsMailDatabase::UpdateFolderFlag(nsIMsgDBHdr *mailHdr, PRBool bSet,
     {
       fileStream = new nsIOFileStream(nsFileSpec(*m_folderSpec));
     }
+    else if (!m_ownFolderStream)
+    {
+      m_folderStream->flush();
+      folderStreamPos = m_folderStream->tell();
+    }
     if (fileStream) 
     {
       PRUint32 msgOffset;
@@ -434,12 +459,10 @@ void nsMailDatabase::UpdateFolderFlag(nsIMsgDBHdr *mailHdr, PRBool bSet,
     }
   }
   //#endif // GET_FILE_STUFF_TOGETHER
-#ifdef XP_MAC
-  if (!m_folderStream /*&& fid != gIncorporateFID*/)	/* ducarroz: Do we still need this ?? */
-#else
-    if (!m_folderStream)
-#endif
-      *ppFileStream = fileStream; // This tells the caller that we opened the file, and please to close it.
+  if (!m_folderStream)
+    *ppFileStream = fileStream; // This tells the caller that we opened the file, and please to close it.
+  else if (!m_ownFolderStream)
+    m_folderStream->seek(PR_SEEK_SET, folderStreamPos);
 }
 
 NS_IMETHODIMP nsMailDatabase::GetSummaryValid(PRBool *aResult)
@@ -577,8 +600,15 @@ NS_IMETHODIMP nsMailDatabase::GetOfflineOpForKey(nsMsgKey msgKey, PRBool create,
       if (*offlineOp)
         (*offlineOp)->SetMessageKey(msgKey);
       nsCOMPtr <nsIMsgDBHdr> msgHdr;
-      
-      GetMsgHdrForKey(msgKey, getter_AddRefs(msgHdr));
+
+      // GetMsgHdrForKey will return an empty header if no header exists,
+      // so we only want to get the hdr if the db contains a hdr for
+      // that key - otherwise, we'll crunch the new flags.
+      PRBool containsKey;
+      rv = ContainsKey(msgKey , &containsKey);
+
+      if (NS_SUCCEEDED(rv) && containsKey)
+        GetMsgHdrForKey(msgKey, getter_AddRefs(msgHdr));
       if (msgHdr)
       {
         imapMessageFlagsType imapFlags = kNoImapMsgFlag;
@@ -696,73 +726,70 @@ NS_IMETHODIMP nsMailDatabase::ListAllOfflineDeletes(nsMsgKeyArray *offlineDelete
 /* static */
 nsresult nsMailDatabase::SetFolderInfoValid(nsFileSpec *folderName, int num, int numunread)
 {
-	nsLocalFolderSummarySpec	summarySpec(*folderName);
-	nsFileSpec					summaryPath(summarySpec);
-	nsresult		err = NS_OK;
-	PRBool			bOpenedDB = PR_FALSE;
-
-	if (!folderName->Exists())
-		return NS_MSG_ERROR_FOLDER_SUMMARY_MISSING;
-
-	// should we have type safe downcast methods again?
-	nsMailDatabase *pMessageDB = (nsMailDatabase *) nsMailDatabase::FindInCache(summaryPath);
-	if (pMessageDB == NULL)
-	{
-		pMessageDB = new nsMailDatabase();
-		if(!pMessageDB)
-			return NS_ERROR_OUT_OF_MEMORY;
-
-		pMessageDB->m_folderSpec = new nsLocalFolderSummarySpec();
-		if(!pMessageDB->m_folderSpec)
-		{
-			delete pMessageDB;
-			return NS_ERROR_OUT_OF_MEMORY;
-		}
-
-		*(pMessageDB->m_folderSpec) = summarySpec;
-		// ### this does later stuff (marks latered messages unread), which may be a problem
-		err = pMessageDB->OpenMDB(summaryPath, PR_FALSE);
-		if (err != NS_OK)
-		{
-			delete pMessageDB;
-			pMessageDB = NULL;
-		}
-		bOpenedDB = PR_TRUE;
-	}
-	else
-		pMessageDB->AddRef();
-
-
-	if (pMessageDB == NULL)
-	{
+  nsLocalFolderSummarySpec	summarySpec(*folderName);
+  nsFileSpec					summaryPath(summarySpec);
+  nsresult		err = NS_OK;
+  PRBool			bOpenedDB = PR_FALSE;
+  
+  if (!folderName->Exists())
+    return NS_MSG_ERROR_FOLDER_SUMMARY_MISSING;
+  
+  // should we have type safe downcast methods again?
+  nsMailDatabase *pMessageDB = (nsMailDatabase *) nsMailDatabase::FindInCache(summaryPath);
+  if (pMessageDB == nsnull)
+  {
+    pMessageDB = new nsMailDatabase();
+    if(!pMessageDB)
+      return NS_ERROR_OUT_OF_MEMORY;
+    
+    pMessageDB->m_folderSpec = new nsLocalFolderSummarySpec();
+    if(!pMessageDB->m_folderSpec)
+    {
+      delete pMessageDB;
+      return NS_ERROR_OUT_OF_MEMORY;
+    }
+    
+    *(pMessageDB->m_folderSpec) = summarySpec;
+    // ### this does later stuff (marks latered messages unread), which may be a problem
+    err = pMessageDB->OpenMDB(summaryPath, PR_FALSE);
+    if (err != NS_OK)
+    {
+      delete pMessageDB;
+      pMessageDB = nsnull;
+    }
+    bOpenedDB = PR_TRUE;
+  }
+  
+  if (pMessageDB == nsnull)
+  {
 #ifdef DEBUG
-		printf("Exception opening summary file\n");
+    printf("Exception opening summary file\n");
 #endif
-		return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
-	}
-	
-	{
-		nsFileSpec::TimeStamp actualFolderTimeStamp;
-		folderName->GetModDate(actualFolderTimeStamp) ;
-
-		pMessageDB->m_dbFolderInfo->SetFolderSize(folderName->GetFileSize());
-		pMessageDB->m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
-		pMessageDB->m_dbFolderInfo->ChangeNumVisibleMessages(num);
-		pMessageDB->m_dbFolderInfo->ChangeNumNewMessages(numunread);
-		pMessageDB->m_dbFolderInfo->ChangeNumMessages(num);
-	}
-	// if we opened the db, then we'd better close it. Otherwise, we found it in the cache,
-	// so just commit and release.
-	if (bOpenedDB)
-	{
-		pMessageDB->Close(PR_TRUE);
-	}
-	else if (pMessageDB)
-	{ 
-		err = pMessageDB->Commit(nsMsgDBCommitType::kLargeCommit);
-		pMessageDB->Release();
-	}
-	return err;
+    return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
+  }
+  
+  {
+    nsFileSpec::TimeStamp actualFolderTimeStamp;
+    folderName->GetModDate(actualFolderTimeStamp) ;
+    
+    pMessageDB->m_dbFolderInfo->SetFolderSize(folderName->GetFileSize());
+    pMessageDB->m_dbFolderInfo->SetFolderDate(actualFolderTimeStamp);
+    pMessageDB->m_dbFolderInfo->ChangeNumVisibleMessages(num);
+    pMessageDB->m_dbFolderInfo->ChangeNumNewMessages(numunread);
+    pMessageDB->m_dbFolderInfo->ChangeNumMessages(num);
+  }
+  // if we opened the db, then we'd better close it. Otherwise, we found it in the cache,
+  // so just commit and release.
+  if (bOpenedDB)
+  {
+    pMessageDB->Close(PR_TRUE);
+  }
+  else if (pMessageDB)
+  { 
+    err = pMessageDB->Commit(nsMsgDBCommitType::kLargeCommit);
+    pMessageDB->Release();
+  }
+  return err;
 }
 
 
@@ -912,4 +939,3 @@ NS_IMETHODIMP nsMsgOfflineOpEnumerator::HasMoreElements(PRBool *aResult)
   *aResult = !mDone;
   return NS_OK;
 }
-
