@@ -38,6 +38,8 @@
 #include "nsIServiceManager.h"
 #include "nsIMsgIdentity.h"
 #include "nsIMsgCompFields.h"
+#include "nsCRT.h"
+#include "nsReadableUtils.h"
 
 // String bundle for smime. Class static.
 nsCOMPtr<nsIStringBundle> nsMsgComposeSecure::mSMIMEBundle = nsnull;
@@ -191,6 +193,7 @@ char
 NS_IMPL_ISUPPORTS1(nsMsgSMIMEComposeFields, nsIMsgSMIMECompFields)
 
 nsMsgSMIMEComposeFields::nsMsgSMIMEComposeFields()
+:mSignMessage(PR_FALSE), mAlwaysEncryptMessage(PR_FALSE)
 {
   NS_INIT_ISUPPORTS();
 }
@@ -211,13 +214,13 @@ NS_IMETHODIMP nsMsgSMIMEComposeFields::GetSignMessage(PRBool *_retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgSMIMEComposeFields::SetAlwaysEncryptMessage(PRBool value)
+NS_IMETHODIMP nsMsgSMIMEComposeFields::SetRequireEncryptMessage(PRBool value)
 {
   mAlwaysEncryptMessage = value;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgSMIMEComposeFields::GetAlwaysEncryptMessage(PRBool *_retval)
+NS_IMETHODIMP nsMsgSMIMEComposeFields::GetRequireEncryptMessage(PRBool *_retval)
 {
   *_retval = mAlwaysEncryptMessage;
   return NS_OK;
@@ -343,10 +346,67 @@ void nsMsgComposeSecure::InitializeSMIMEBundle()
                               getter_AddRefs(mSMIMEBundle));
 }
 
+void nsMsgComposeSecure::SetError(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string)
+{
+  if (!sendReport || !bundle_string)
+    return;
+
+  if (mErrorAlreadyReported)
+    return;
+
+  mErrorAlreadyReported = PR_TRUE;
+  
+  nsXPIDLString errorString;
+  nsresult res;
+
+  res = GetSMIMEBundleString(bundle_string,
+                             getter_Copies(errorString));
+
+  if (NS_SUCCEEDED(res) && !errorString.IsEmpty())
+  {
+    sendReport->SetMessage(nsIMsgSendReport::process_Current,
+                           errorString.get(),
+                           PR_TRUE);
+  }
+}
+
+void nsMsgComposeSecure::SetErrorWithParam(nsIMsgSendReport *sendReport, const PRUnichar *bundle_string, const char *param)
+{
+  if (!sendReport || !bundle_string || !param)
+    return;
+
+  if (mErrorAlreadyReported)
+    return;
+
+  mErrorAlreadyReported = PR_TRUE;
+  
+  nsXPIDLString errorString;
+  nsresult res;
+  const PRUnichar *params[1];
+
+  NS_ConvertASCIItoUCS2 ucs2(param);
+  params[0]= ucs2.get();
+
+  res = SMIMEBundleFormatStringFromName(bundle_string,
+                                        params,
+                                        1,
+                                        getter_Copies(errorString));
+
+  if (NS_SUCCEEDED(res) && !errorString.IsEmpty())
+  {
+    sendReport->SetMessage(nsIMsgSendReport::process_Current,
+                           errorString.get(),
+                           PR_TRUE);
+  }
+}
+
 nsresult nsMsgComposeSecure::ExtractEncryptionState(nsIMsgIdentity * aIdentity, nsIMsgCompFields * aComposeFields, PRBool * aSignMessage, PRBool * aEncrypt)
 {
   if (!aComposeFields && !aIdentity)
-    return NS_OK; // kick out...invalid args....
+    return NS_ERROR_FAILURE; // kick out...invalid args....
+
+  NS_ENSURE_ARG(aSignMessage);
+  NS_ENSURE_ARG(aEncrypt);
 
   nsCOMPtr<nsISupports> securityInfo;
   if (aComposeFields)
@@ -358,15 +418,26 @@ nsresult nsMsgComposeSecure::ExtractEncryptionState(nsIMsgIdentity * aIdentity,
     if (smimeCompFields)
     {
       smimeCompFields->GetSignMessage(aSignMessage);
-      smimeCompFields->GetAlwaysEncryptMessage(aEncrypt);
+      smimeCompFields->GetRequireEncryptMessage(aEncrypt);
+      return NS_OK;
     }
   }
-  else if (aIdentity)  // get the default info from the identity....
-  {
-    aIdentity->GetBoolAttribute("encrypt_mail_always", aEncrypt);
-    aIdentity->GetBoolAttribute("sign_mail", aSignMessage);
+
+  // get the default info from the identity....
+  PRInt32 ep = 0;
+  nsresult testrv = aIdentity->GetIntAttribute("encryptionpolicy", &ep);
+  if (NS_FAILED(testrv)) {
+    *aEncrypt = PR_FALSE;
+  }
+  else {
+    *aEncrypt = (ep > 0);
   }
 
+  testrv = aIdentity->GetBoolAttribute("sign_mail", aSignMessage);
+  if (NS_FAILED(testrv))
+  {
+    *aSignMessage = PR_FALSE;
+  }
   return NS_OK;
 }
 
@@ -378,6 +449,7 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsOutputFileStream *
                                                            nsIMsgSendReport *sendReport,
                                                            PRBool aIsDraft)
 {
+  mErrorAlreadyReported = PR_FALSE;
   nsresult rv = NS_OK;
 
   PRBool encryptMessages = PR_FALSE;
@@ -409,17 +481,17 @@ NS_IMETHODIMP nsMsgComposeSecure::BeginCryptoEncapsulation(nsOutputFileStream *
   switch (mCryptoState)
 	{
 	case mime_crypto_clear_signed:
-    rv = MimeInitMultipartSigned(PR_TRUE);
+    rv = MimeInitMultipartSigned(PR_TRUE, sendReport);
     break;
 	case mime_crypto_opaque_signed:
     PR_ASSERT(0);    /* #### no api for this yet */
     rv = -1;
 	  break;
 	case mime_crypto_signed_encrypted:
-    rv = MimeInitEncryption(PR_TRUE);
+    rv = MimeInitEncryption(PR_TRUE, sendReport);
 	  break;
 	case mime_crypto_encrypted:
-    rv = MimeInitEncryption(PR_FALSE);
+    rv = MimeInitEncryption(PR_FALSE, sendReport);
 	  break;
 	case mime_crypto_none:
 	  /* This can happen if mime_crypto_hack_certs() decided to turn off
@@ -436,24 +508,24 @@ FAIL:
 }
 
 /* void finishCryptoEncapsulation (in boolean aAbort); */
-NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort)
+NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort, nsIMsgSendReport *sendReport)
 {
   nsresult rv;
 
   if (!aAbort) {
 	  switch (mCryptoState) {
 		case mime_crypto_clear_signed:
-      rv = MimeFinishMultipartSigned (PR_TRUE);
+      rv = MimeFinishMultipartSigned (PR_TRUE, sendReport);
       break;
     case mime_crypto_opaque_signed:
       PR_ASSERT(0);    /* #### no api for this yet */
       rv = NS_ERROR_FAILURE;
       break;
     case mime_crypto_signed_encrypted:
-      rv = MimeFinishEncryption (PR_TRUE);
+      rv = MimeFinishEncryption (PR_TRUE, sendReport);
 		  break;
 		case mime_crypto_encrypted:
-		  rv = MimeFinishEncryption (PR_FALSE);
+		  rv = MimeFinishEncryption (PR_FALSE, sendReport);
 		  break;
 		default:
 		  PR_ASSERT(0);
@@ -464,7 +536,7 @@ NS_IMETHODIMP nsMsgComposeSecure::FinishCryptoEncapsulation(PRBool aAbort)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
+nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter, nsIMsgSendReport *sendReport)
 {
   /* First, construct and write out the multipart/signed MIME header data.
    */
@@ -476,7 +548,7 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
 										&mMultipartSignedBoundary);
   if (NS_FAILED(rv)) goto FAIL;
 
-  L = nsCRT::strlen(header);
+  L = strlen(header);
 
   if (aOuter){
 	  /* If this is the outer block, write it to the file. */
@@ -512,7 +584,7 @@ nsresult nsMsgComposeSecure::MimeInitMultipartSigned(PRBool aOuter)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
+nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign, nsIMsgSendReport *sendReport)
 {
   nsresult rv;
 
@@ -530,7 +602,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
 				MIME_SMIME_ENCRYPTED_CONTENT_DESCRIPTION);
   PRInt32 L;
   if (!s) return NS_ERROR_OUT_OF_MEMORY;
-  L = nsCRT::strlen(s);
+  L = strlen(s);
   if (PRInt32(mStream->write(s, L)) < L) {
 	  return NS_ERROR_FAILURE;
   }
@@ -560,17 +632,19 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
   PR_ASSERT(mSelfEncryptionCert);
   PR_SetError(0,0);
   mEncryptionCinfo = do_CreateInstance(NS_CMSMESSAGE_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return 0;
-  rv = mEncryptionCinfo->CreateEncrypted(mCerts); // XXX Fix this later XXX //
+  if (NS_FAILED(rv)) return rv;
+  rv = mEncryptionCinfo->CreateEncrypted(mCerts);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
 	  goto FAIL;
 	}
 
   mEncryptionContext = do_CreateInstance(NS_CMSENCODER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return 0;
+  if (NS_FAILED(rv)) return rv;
 
-  rv = mEncryptionContext->Start(mEncryptionCinfo, mime_crypto_write_base64, mCryptoEncoderData); // XXX Fix this later XXX //
+  rv = mEncryptionContext->Start(mEncryptionCinfo, mime_crypto_write_base64, mCryptoEncoderData);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
 	  goto FAIL;
 	}
 
@@ -578,7 +652,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
 	 the data to be encrypted, and initialize the sign-hashing code too.
    */
   if (aSign) {
-	  rv = MimeInitMultipartSigned(PR_FALSE);
+	  rv = MimeInitMultipartSigned(PR_FALSE, sendReport);
 	  if (NS_FAILED(rv)) goto FAIL;
 	}
 
@@ -586,7 +660,7 @@ nsresult nsMsgComposeSecure::MimeInitEncryption(PRBool aSign)
   return rv;
 }
 
-nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
+nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter, nsIMsgSendReport *sendReport)
 {
   int status;
   nsresult rv;
@@ -640,7 +714,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
 		goto FAIL;
 	}
 
-	L = nsCRT::strlen(header);
+	L = strlen(header);
 	if (aOuter) {
 		/* If this is the outer block, write it to the file. */
     if (PRInt32(mStream->write(header, L)) < L) {
@@ -652,7 +726,6 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   }
 
 	PR_Free(header);
-	if (status < 0) goto FAIL;
 
   /* Create the signature...
    */
@@ -663,6 +736,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   PR_SetError(0,0);
   rv = cinfo->CreateSigned(mSelfSigningCert, mSelfEncryptionCert, sec_item_data, sec_item_len);
   if (NS_FAILED(rv))	{
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
@@ -684,12 +758,14 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
   PR_SetError(0,0);
   rv = encoder->Start(cinfo, mime_crypto_write_base64, mSigEncoderData);
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
   // We're not passing in any data, so no update needed.
   rv = encoder->Finish();
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotSign").get());
 	  goto FAIL;
 	}
 
@@ -714,7 +790,7 @@ nsresult nsMsgComposeSecure::MimeFinishMultipartSigned (PRBool aOuter)
 		rv = NS_ERROR_OUT_OF_MEMORY;
 		goto FAIL;
 	}
-	L = nsCRT::strlen(header);
+	L = strlen(header);
 	if (aOuter) {
 		/* If this is the outer block, write it to the file. */
 		if (PRInt32(mStream->write(header, L)) < L)
@@ -734,14 +810,14 @@ FAIL:
 /* Helper function for mime_finish_crypto_encapsulation() to close off
    an opaque crypto object (for encrypted or signed-and-encrypted messages.)
  */
-nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign)
+nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign, nsIMsgSendReport *sendReport)
 {
   nsresult rv;
 
   /* If this object is both encrypted and signed, close off the
 	 signature first (since it's inside.) */
   if (aSign) {
-	  rv = MimeFinishMultipartSigned (PR_FALSE);
+	  rv = MimeFinishMultipartSigned (PR_FALSE, sendReport);
     if (NS_FAILED(rv)) {
       goto FAIL;
     }
@@ -753,6 +829,7 @@ nsresult nsMsgComposeSecure::MimeFinishEncryption (PRBool aSign)
 
   rv = mEncryptionContext->Finish();
   if (NS_FAILED(rv)) {
+    SetError(sendReport, NS_LITERAL_STRING("ErrorCanNotEncrypt").get());
     goto FAIL;
 	}
 
@@ -795,7 +872,6 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
     return res;
   }
 
-  nsXPIDLString errorString;
   PRBool no_clearsigning_p = PR_FALSE;
 
   PR_ASSERT(aEncrypt || aSign);
@@ -804,65 +880,74 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 
   // must have both the signing and encryption certs to sign
 	if ((mSelfSigningCert == nsnull) && aSign) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("NoSenderSigningCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
+    SetError(sendReport, NS_LITERAL_STRING("NoSenderSigningCert").get());
+    res = NS_ERROR_FAILURE;
 		goto FAIL;
 	}
 
 	if ((mSelfEncryptionCert == nsnull) && aSign) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("SignNoSenderEncryptionCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
+    SetError(sendReport, NS_LITERAL_STRING("SignNoSenderEncryptionCert").get());
+    res = NS_ERROR_FAILURE;
     goto FAIL;
   }
 
 	if ((mSelfEncryptionCert == nsnull) && aEncrypt) {
-    res = GetSMIMEBundleString(NS_LITERAL_STRING("NoSenderEncriptionCert").get(),
-                               getter_Copies(errorString));
-    if ( NS_SUCCEEDED(res) ) {
-      res = NS_ERROR_FAILURE;
-    }
+    SetError(sendReport, NS_LITERAL_STRING("NoSenderEncryptionCert").get());
+    res = NS_ERROR_FAILURE;
     goto FAIL;
   }
 
   pHeader->ExtractHeaderAddressMailboxes(nsnull,aRecipients, &all_mailboxes);
   pHeader->RemoveDuplicateAddresses(nsnull, all_mailboxes, 0, PR_FALSE /*removeAliasesToMe*/, &mailboxes);
-  PR_FREEIF(all_mailboxes);
+  if (all_mailboxes) {
+    nsMemory::Free(all_mailboxes);
+    all_mailboxes = nsnull;
+  }
 
   if (mailboxes) {
 	  pHeader->ParseHeaderAddresses (nsnull, mailboxes, 0, &mailbox_list, &count);
+    nsMemory::Free(mailboxes);
+    mailboxes = nsnull;
   }
-  PR_FREEIF(mailboxes);
   if (count < 0) return count;
 
+  if (aEncrypt && mSelfEncryptionCert) {
+    // Make sure self's configured cert is prepared for being used
+    // as an email recipient cert.
+    
+    mSelfEncryptionCert->SaveSMimeProfile();
+  }
+
   /* If the message is to be encrypted, then get the recipient certs */
   if (aEncrypt) {
 	  mailbox = mailbox_list;
-	  for (; count > 0; count--) {
-      nsCOMPtr<nsIX509Cert> cert;
-		  certdb->GetCertByEmailAddress(nsnull, mailbox, getter_AddRefs(cert));
-		  if (!cert) {
-        // failure to find an encryption cert is
-        // fatal for now. We won't be able to encrypt anyway
-        // ssaux 12/03/2001.
-        const PRUnichar *params[1];
-        // here I assume that mailbox contains ascii rather than utf8.
-        NS_ConvertASCIItoUCS2 wstr_mailbox(mailbox);
-        params[0]= wstr_mailbox.get();
-        res = 
-          SMIMEBundleFormatStringFromName(NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(),
-                                          params,
-                                          1,
-                                          getter_Copies(errorString));
 
-        if ( NS_SUCCEEDED(res) ) {
-          res = NS_ERROR_FAILURE;
+    PRBool already_added_self_cert = PR_FALSE;
+
+	  for (; count > 0; count--) {
+	    nsCString mailbox_lowercase;
+	    ToLowerCase(nsDependentCString(mailbox), mailbox_lowercase);
+      nsCOMPtr<nsIX509Cert> cert;
+      certdb->GetCertByEmailAddress(nsnull, mailbox_lowercase.get(), getter_AddRefs(cert));
+      PRBool foundValidCert = PR_FALSE;
+
+		  if (cert) {
+        PRUint32 verification_result;
+
+        if (NS_SUCCEEDED(
+            cert->VerifyForUsage(nsIX509Cert::CERT_USAGE_EmailRecipient, &verification_result))
+            &&
+            nsIX509Cert::VERIFIED_OK == verification_result)
+        {
+          foundValidCert = PR_TRUE;
         }
+      }
+      
+      if (!foundValidCert) {
+        // Failure to find a valid encryption cert is fatal.
+        // here I assume that mailbox contains ascii rather than utf8.
+        SetErrorWithParam(sendReport, NS_LITERAL_STRING("MissingRecipientEncryptionCert").get(), mailbox);
+        res = NS_ERROR_FAILURE;
         goto FAIL;
 		  }
 
@@ -872,19 +957,27 @@ nsresult nsMsgComposeSecure::MimeCryptoHackCerts(const char *aRecipients,
 		 of the recipients if we're sending a signed-but-not-encrypted
 		 message.)
 	   */
+
+      PRBool isSame;
+      if (NS_SUCCEEDED(cert->IsSameCert(mSelfEncryptionCert, &isSame))
+          && isSame) {
+        already_added_self_cert = PR_TRUE;
+      }
+
       mCerts->AppendElement(cert);
-		  mailbox += nsCRT::strlen(mailbox) + 1;
+      // To understand this loop, especially the "+= strlen +1", look at the documentation
+      // of ParseHeaderAddresses. Basically, it returns a list of zero terminated strings.
+		  mailbox += strlen(mailbox) + 1;
 	  }
+    
+    if (!already_added_self_cert) {
+      mCerts->AppendElement(mSelfEncryptionCert);
+    }
 	}
 FAIL:
-  PR_FREEIF(mailbox_list);
-
-  if (NS_FAILED(res) && errorString.get() && sendReport) {
-    sendReport->SetMessage(nsIMsgSendReport::process_Current,
-                           errorString.get(),
-                           PR_TRUE);
+  if (mailbox_list) {
+    nsMemory::Free(mailbox_list);
   }
-
   return res;
 }
 
