@@ -36,6 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "msgCore.h"
+#include "netCore.h"
 #include "nsImapOfflineSync.h"
 #include "nsImapMailFolder.h"
 #include "nsMsgFolderFlags.h"
@@ -103,7 +104,10 @@ nsImapOfflineSync::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
     m_curTempFile->Delete(PR_FALSE);
     m_curTempFile = nsnull;
   }
-  if (NS_SUCCEEDED(exitCode))
+  // NS_BINDING_ABORTED is used for the user pressing stop, which
+  // should cause us to abort the offline process. Other errors
+  // should allow us to continue.
+  if (exitCode != NS_BINDING_ABORTED)
     rv = ProcessNextOperation();
   else if (m_listener)  // notify main observer.
     m_listener->OnStopRunningUrl(url, exitCode);
@@ -207,18 +211,18 @@ void nsImapOfflineSync::AdvanceToFirstIMAPFolder()
 
 void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *currentOp)
 {
-	nsMsgKeyArray matchingFlagKeys;
-	PRUint32 currentKeyIndex = m_KeyIndex;
-	imapMessageFlagsType matchingFlags;
+  nsMsgKeyArray matchingFlagKeys;
+  PRUint32 currentKeyIndex = m_KeyIndex;
+  imapMessageFlagsType matchingFlags;
   currentOp->GetNewFlags(&matchingFlags);
   imapMessageFlagsType flagOperation;
   imapMessageFlagsType newFlags;
 	
-	do
+  do
   {	// loop for all messsages with the same flags
     nsMsgKey curKey;
     currentOp->GetMessageKey(&curKey);
-		matchingFlagKeys.Add(curKey);
+    matchingFlagKeys.Add(curKey);
     currentOp->ClearOperation(nsIMsgOfflineImapOperation::kFlagsChanged);
     currentOp = nsnull;
     if (++currentKeyIndex < m_CurrentKeys.GetSize())
@@ -229,20 +233,20 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *current
       currentOp->GetFlagOperation(&flagOperation);
       currentOp->GetNewFlags(&newFlags);
     }
-	} while (currentOp && (flagOperation & nsIMsgOfflineImapOperation::kFlagsChanged) && (newFlags == matchingFlags) );
+  } while (currentOp && (flagOperation & nsIMsgOfflineImapOperation::kFlagsChanged) && (newFlags == matchingFlags) );
 	
   currentOp = nsnull;
 	
   if (matchingFlagKeys.GetSize() > 0)
   {
     nsCAutoString uids;
-	  nsImapMailFolder::AllocateUidStringFromKeys(matchingFlagKeys.GetArray(), matchingFlagKeys.GetSize(), uids);
+    nsImapMailFolder::AllocateUidStringFromKeys(matchingFlagKeys.GetArray(), matchingFlagKeys.GetSize(), uids);
     PRUint32 curFolderFlags;
     m_currentFolder->GetFlags(&curFolderFlags);
 
-	  if (uids.get() && (curFolderFlags & MSG_FOLDER_FLAG_IMAPBOX)) 
-	  {
-	    nsresult rv = NS_OK;
+    if (uids.get() && (curFolderFlags & MSG_FOLDER_FLAG_IMAPBOX)) 
+    {
+      nsresult rv = NS_OK;
       nsCOMPtr <nsIMsgImapMailFolder> imapFolder = do_QueryInterface(m_currentFolder);
       nsCOMPtr <nsIURI> uriToSetFlags;
       if (imapFolder)
@@ -255,8 +259,7 @@ void nsImapOfflineSync::ProcessFlagOperation(nsIMsgOfflineImapOperation *current
             mailnewsUrl->RegisterListener(this);
         }
       }
-
-	  }
+    }
   }
   else
     ProcessNextOperation();
@@ -415,7 +418,7 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *current
     if (NS_SUCCEEDED(rv) && destFolder)
     {
       nsCOMPtr <nsIMsgImapMailFolder> imapFolder = do_QueryInterface(m_currentFolder);
-      if (imapFolder)
+      if (imapFolder && DestFolderOnSameServer(destFolder))
       {
         rv = imapFolder->ReplayOfflineMoveCopy(matchingFlagKeys.GetArray(), matchingFlagKeys.GetSize(), PR_TRUE, destFolder,
           this, m_window);
@@ -444,6 +447,20 @@ void nsImapOfflineSync::ProcessMoveOperation(nsIMsgOfflineImapOperation *current
   }
 }
 
+// I'm tempted to make this a method on nsIMsgFolder, but that interface
+// is already so huge, and there are only a few places in the code that do this.
+// If there end up to be more places that need this, then we can reconsider.
+PRBool nsImapOfflineSync::DestFolderOnSameServer(nsIMsgFolder *destFolder)
+{
+  nsCOMPtr <nsIMsgIncomingServer> srcServer;
+  nsCOMPtr <nsIMsgIncomingServer> dstServer;
+
+  PRBool sameServer = PR_FALSE;
+  if (NS_SUCCEEDED(m_currentFolder->GetServer(getter_AddRefs(srcServer))) 
+    && NS_SUCCEEDED(destFolder->GetServer(getter_AddRefs(dstServer))))
+    dstServer->Equals(srcServer, &sameServer);
+  return sameServer;
+}
 
 void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *currentOp)
 {
@@ -496,7 +513,7 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *current
     if (NS_SUCCEEDED(rv) && destFolder)
     {
       nsCOMPtr <nsIMsgImapMailFolder> imapFolder = do_QueryInterface(m_currentFolder);
-      if (imapFolder)
+      if (imapFolder && DestFolderOnSameServer(destFolder))
       {
         rv = imapFolder->ReplayOfflineMoveCopy(matchingFlagKeys.GetArray(), matchingFlagKeys.GetSize(), PR_FALSE, destFolder,
                        this, m_window);
@@ -527,27 +544,28 @@ void nsImapOfflineSync::ProcessCopyOperation(nsIMsgOfflineImapOperation *current
 
 void nsImapOfflineSync::ProcessEmptyTrash(nsIMsgOfflineImapOperation *currentOp)
 {
-	m_currentFolder->EmptyTrash(m_window, this);
-	m_currentDB->RemoveOfflineOp(currentOp);
-	m_currentDB = nsnull;	// empty trash deletes the database?
+  m_currentFolder->EmptyTrash(m_window, this);
+  // don't need to remove the current op because emptying trash will
+  // delete the database.
+  m_currentDB = nsnull;	// empty trash deletes the database?
 }
 
 // returns PR_TRUE if we found a folder to create, PR_FALSE if we're done creating folders.
 PRBool nsImapOfflineSync::CreateOfflineFolders()
 {
-	while (m_currentFolder)
-	{
-		PRUint32 flags;
+  while (m_currentFolder)
+  {
+    PRUint32 flags;
     m_currentFolder->GetFlags(&flags);
-		PRBool offlineCreate = (flags & MSG_FOLDER_FLAG_CREATED_OFFLINE) != 0;
-		if (offlineCreate)
-		{
-			if (CreateOfflineFolder(m_currentFolder))
-				return PR_TRUE;
-		}
-		AdvanceToNextFolder();
-	}
-	return PR_FALSE;
+    PRBool offlineCreate = (flags & MSG_FOLDER_FLAG_CREATED_OFFLINE) != 0;
+    if (offlineCreate)
+    {
+      if (CreateOfflineFolder(m_currentFolder))
+        return PR_TRUE;
+    }
+    AdvanceToNextFolder();
+  }
+  return PR_FALSE;
 }
 
 PRBool nsImapOfflineSync::CreateOfflineFolder(nsIMsgFolder *folder)
@@ -682,7 +700,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
         else if (folderFlags & MSG_FOLDER_FLAG_IMAPBOX)
         {
           //					if (imapFolder->GetHasOfflineEvents())
-          //						XP_ASSERT(PR_FALSE);
+          //						NS_ASSERTION(PR_FALSE, "Hardcoded assertion");
           
           if (!m_pseudoOffline)	// if pseudo offline, falls through to playing ops back.
           {
@@ -718,7 +736,7 @@ nsresult nsImapOfflineSync::ProcessNextOperation()
     if (!folderInfo)
       m_currentDB->GetDBFolderInfo(getter_AddRefs(folderInfo));
     // user canceled the lite select! if GetCurrentUIDValidity() == 0
-    if ((m_KeyIndex < m_CurrentKeys.GetSize()) && (m_pseudoOffline || (GetCurrentUIDValidity() != 0) || !(folderFlags & MSG_FOLDER_FLAG_IMAPBOX)) )
+    if (folderInfo && (m_KeyIndex < m_CurrentKeys.GetSize()) && (m_pseudoOffline || (GetCurrentUIDValidity() != 0) || !(folderFlags & MSG_FOLDER_FLAG_IMAPBOX)) )
     {
       PRInt32 curFolderUidValidity;
       folderInfo->GetImapUidValidity(&curFolderUidValidity);
@@ -947,9 +965,34 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
           rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_INBOX, 1, &numFolders, getter_AddRefs(inbox));
           if (inbox)
           {
-            rv = inbox->GetNewMessages(m_window, this);
-            if (NS_SUCCEEDED(rv))
-              return rv; // otherwise, fall through.
+            nsCOMPtr <nsIMsgFolder> offlineImapFolder;
+            nsCOMPtr <nsIMsgImapMailFolder> imapInbox = do_QueryInterface(inbox);
+            if (imapInbox)
+            {
+              rootMsgFolder->GetFoldersWithFlag(MSG_FOLDER_FLAG_OFFLINE, 1, &numFolders, getter_AddRefs(offlineImapFolder));
+              if (!offlineImapFolder)
+              {
+                // no imap folders configured for offline use - check if the account is set up
+                // so that we always download inbox msg bodies for offline use
+                nsCOMPtr <nsIImapIncomingServer> imapServer = do_QueryInterface(m_currentServer);
+                if (imapServer)
+                {
+                  PRBool downloadBodiesOnGetNewMail = PR_FALSE;
+                  imapServer->GetDownloadBodiesOnGetNewMail(&downloadBodiesOnGetNewMail);
+                  if (downloadBodiesOnGetNewMail)
+                    offlineImapFolder = inbox;
+                }
+              }
+            }
+            // if this isn't an imap inbox, or we have an offline imap sub-folder, then update the inbox.
+            // otherwise, it's an imap inbox for an account with no folders configured for offline use,
+            // so just advance to the next server.
+            if (!imapInbox || offlineImapFolder)
+            {
+              rv = inbox->GetNewMessages(m_window, this);
+              if (NS_SUCCEEDED(rv))
+                return rv; // otherwise, fall through.
+            }
           }
         }
       }
@@ -963,8 +1006,8 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
   }
   AdvanceToNextFolder();
 
-	while (m_currentFolder)
-	{
+  while (m_currentFolder)
+  {
     PRUint32 folderFlags;
 
     m_currentDB = nsnull;
@@ -973,10 +1016,14 @@ nsresult nsImapOfflineDownloader::ProcessNextOperation()
       imapFolder = do_QueryInterface(m_currentFolder);
     m_currentFolder->GetFlags(&folderFlags);
 		// need to check if folder has offline events, or is configured for offline
-		if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
-      return m_currentFolder->DownloadAllForOffline(this, m_window);
-    else
-      AdvanceToNextFolder();
+    if (imapFolder && folderFlags & MSG_FOLDER_FLAG_OFFLINE)
+    {
+      rv = m_currentFolder->DownloadAllForOffline(this, m_window);
+      if (NS_SUCCEEDED(rv) || rv == NS_BINDING_ABORTED)
+        return rv;
+      // if this fails and the user didn't cancel/stop, fall through to code that advances to next folder
+    }
+    AdvanceToNextFolder();
   }
   if (m_listener)
     m_listener->OnStopRunningUrl(nsnull, NS_OK);
