@@ -142,7 +142,6 @@ public:
 	NS_IMETHOD GetFlagsForUID(PRUint32 uid, PRBool *foundIt, imapMessageFlagsType *flags);
 	NS_IMETHOD GetSupportedUserFlags(PRUint16 *flags);
 
-	NS_IMETHOD GetStreamConsumer (nsISupports **aSupport);
   NS_IMETHOD GetRunningUrl(nsIURI **aUrl);
   
   // Tell thread to die. This can only be called by imap service
@@ -152,7 +151,7 @@ public:
   // Get last active time stamp
   NS_IMETHOD GetLastActiveTimeStamp(PRTime *aTimeStamp);
 
-	NS_IMETHOD PseudoInterruptMsgLoad(nsIImapUrl *aImapUrl, PRBool
+	NS_IMETHOD PseudoInterruptMsgLoad(nsIMsgFolder *aImapFolder, PRBool
                                       *interrupted);
   NS_IMETHOD GetSelectedMailboxName(char ** folderName);
   NS_IMETHOD ResetToAuthenticatedState();
@@ -174,7 +173,7 @@ public:
   static void EscapeUserNamePasswordString(const char *strToEscape, nsCString *resultStr);
 
 	// used to start fetching a message.
-  void GetShouldDownloadArbitraryHeaders(PRBool *aResult);
+  void GetShouldDownloadAllHeaders(PRBool *aResult);
   void GetArbitraryHeadersToDownload(char **aResult);
   virtual void AdjustChunkSize();
   virtual void FetchMessage(const char * messageIds, 
@@ -223,7 +222,7 @@ public:
 	// Used for MIME parts on demand.
 	void	SetContentModified(IMAP_ContentModifiedType modified);
 	PRBool	GetShouldFetchAllParts();
-
+        PRBool  GetIgnoreExpunges() {return m_ignoreExpunges;}
 	// Generic accessors required by the imap parser
 	char * CreateNewLineFromSocket();
 	PRInt32 GetConnectionStatus();
@@ -252,7 +251,7 @@ public:
 
 	// utility function calls made by the server
 
-	PRUnichar * CreatePRUnicharStringFromUTF7(const char * aSourceString);
+	static PRUnichar * CreatePRUnicharStringFromUTF7(const char * aSourceString);
 
 	void Copy(const char * messageList, const char *destinationMailbox, 
                                     PRBool idsAreUid);
@@ -304,7 +303,9 @@ public:
 	void AddFolderRightsForUser(const char *mailboxName, const char *userName, const char *rights);
 	// Clears all rights for a given folder, for all users.
 	void ClearAllFolderRights(const char *mailboxName, nsIMAPNamespace *nsForMailbox);
+        void RefreshFolderACLView(const char *mailboxName, nsIMAPNamespace *nsForMailbox);
 
+        nsresult SetFolderAdminUrl(const char *mailboxName);
   void WaitForFEEventCompletion();
   void HandleMemoryFailure();
 	void HandleCurrentUrlError();
@@ -323,8 +324,8 @@ private:
 	nsCOMPtr<nsIImapUrl>		m_runningUrl; // the nsIImapURL that is currently running
 	nsImapAction	m_imapAction;  // current imap action associated with this connnection...
 
+	nsCString		m_hostName;
 	char			*m_userName;
-	char			*m_hostName;
 	char			*m_serverKey;
 	char			*m_dataOutputBuf;
 	nsMsgLineStreamBuffer * m_inputStreamBuffer;
@@ -483,7 +484,8 @@ private:
 	void OnSubscribe(const char * aSourceMailbox);
 	void OnUnsubscribe(const char * aSourceMailbox);
   void RefreshACLForFolderIfNecessary(const char * mailboxName);
-	void OnRefreshACLForFolder(const char * aSourceMailbox);
+	void RefreshACLForFolder(const char * aSourceMailbox);
+        void GetACLForFolder(const char *aMailboxName);
 	void OnRefreshAllACLs();
 	void OnListFolder(const char * aSourceMailbox, PRBool aBool);
 	void OnStatusForFolder(const char * sourceMailbox);
@@ -518,7 +520,10 @@ private:
 	void List(const char *mailboxPattern, PRBool addDirectoryIfNecessary);
 	void Subscribe(const char *mailboxName);
 	void Unsubscribe(const char *mailboxName);
-  PRBool DeleteSubFolders(const char* mailboxName);
+    // Some imap servers include the mailboxName following the dir-separator in the list of 
+    // subfolders of the mailboxName. In fact, they are the same. So we should decide if
+    // we should delete such subfolder and provide feedback if the delete operation succeed.
+    PRBool DeleteSubFolders(const char* aMailboxName, PRBool & aDeleteSelf);
 	PRBool  RenameHierarchyByHand(const char *oldParentMailboxName, 
                                   const char *newParentMailboxName);
 
@@ -539,6 +544,7 @@ private:
   PRInt32		m_chunkStartSize;
   PRInt32		m_maxChunkSize;
   PRBool		m_fetchByChunks;
+  PRBool                m_ignoreExpunges;
   PRInt32		m_chunkSize;
   PRInt32		m_chunkThreshold;
   TLineDownloadCache m_downloadLineCache;
