@@ -232,7 +232,7 @@ void nsImapServerResponseParser::ParseIMAPServerResponse(const char *currentComm
 						" didn't get the number of tagged responses we expected");
 					numberOfTaggedResponsesReceived = fNumberOfTaggedResponsesExpected;
           if (commandToken && !nsCRT::strcasecmp(commandToken, "authenticate") && placeInTokenString && 
-            !nsCRT::strncasecmp(placeInTokenString, "CRAM-MD5", nsCRT::strlen("CRAM-MD5")))
+            !nsCRT::strncasecmp(placeInTokenString, "CRAM-MD5", strlen("CRAM-MD5")))
           {
             // we need to store the digest from the server if we are using CRAM-MD5. 
             cramResponse_data();
@@ -605,8 +605,8 @@ void nsImapServerResponseParser::response_data()
 							fCurrentResponseUID = atoi(fNextToken);
 							// if this token ends in ')', then it is the last token
 							// else we advance
-							if ( *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')')
-								fNextToken += nsCRT::strlen(fNextToken) - 1;
+							if ( *(fNextToken + strlen(fNextToken) - 1) == ')')
+								fNextToken += strlen(fNextToken) - 1;
 						}
 					}
 					else if (!PL_strcasecmp(fNextToken, "MESSAGES"))
@@ -617,8 +617,8 @@ void nsImapServerResponseParser::response_data()
 							fNumberOfExistingMessages = atoi(fNextToken);
 							// if this token ends in ')', then it is the last token
 							// else we advance
-							if ( *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')')
-								fNextToken += nsCRT::strlen(fNextToken) - 1;
+							if ( *(fNextToken + strlen(fNextToken) - 1) == ')')
+								fNextToken += strlen(fNextToken) - 1;
 						}
 					}
 					else if (!PL_strcasecmp(fNextToken, "UNSEEN"))
@@ -629,8 +629,8 @@ void nsImapServerResponseParser::response_data()
 							fNumberOfUnseenMessages = atoi(fNextToken);
 							// if this token ends in ')', then it is the last token
 							// else we advance
-							if ( *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')')
-								fNextToken += nsCRT::strlen(fNextToken) - 1;
+							if ( *(fNextToken + strlen(fNextToken) - 1) == ')')
+								fNextToken += strlen(fNextToken) - 1;
 						}
 					}
 					else if (*fNextToken == ')')
@@ -809,7 +809,7 @@ void nsImapServerResponseParser::mailbox_list(PRBool discoveredFromLsub)
 				boxSpec->box_flags |= kNoselect;	
 			// we ignore flag extensions
 			
-			endOfFlags = *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')';
+			endOfFlags = *(fNextToken + strlen(fNextToken) - 1) == ')';
 			fNextToken = GetNextToken();
 		} while (!endOfFlags && ContinueParse());
 		
@@ -915,8 +915,11 @@ void nsImapServerResponseParser::mailbox(nsImapMailboxSpec *boxSpec)
 		boxSpec->connection->GetCurrentUrl()->AllocateCanonicalPath(boxname, boxSpec->hierarchySeparator, &boxSpec->allocatedPathName);
 		nsIURI * aURL = nsnull;
 		boxSpec->connection->GetCurrentUrl()->QueryInterface(NS_GET_IID(nsIURI), (void **) &aURL);
-		if (aURL)
-			aURL->GetHost(&boxSpec->hostName);
+		if (aURL) {
+            nsCAutoString host;
+			aURL->GetHost(host);
+            boxSpec->hostName = ToNewCString(host);
+        }
 		NS_IF_RELEASE(aURL);
         if (boxname)
             PL_strfree( boxname);
@@ -968,7 +971,8 @@ void nsImapServerResponseParser::numeric_mailbox_data()
 		}
 		else if (!PL_strcasecmp(fNextToken, "EXPUNGE"))
 		{
-			fFlagState->ExpungeByIndex((PRUint32) tokenNumber);
+                        if (!fServerConnection.GetIgnoreExpunges())
+			  fFlagState->ExpungeByIndex((PRUint32) tokenNumber);
 			skip_to_CRLF();
 		}
 		else
@@ -1035,8 +1039,8 @@ void nsImapServerResponseParser::msg_fetch()
 				
 				// if this token ends in ')', then it is the last token
 				// else we advance
-				if ( *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')')
-					fNextToken += nsCRT::strlen(fNextToken) - 1;
+				if ( *(fNextToken + strlen(fNextToken) - 1) == ')')
+					fNextToken += strlen(fNextToken) - 1;
 				else
 					fNextToken = GetNextToken();
 			}
@@ -1069,7 +1073,7 @@ void nsImapServerResponseParser::msg_fetch()
 				fDownloadingHeaders = PR_TRUE;
         BeginMessageDownload(MESSAGE_RFC822); // initialize header parser
 				// specific message header fields
-				while (ContinueParse() && fNextToken[nsCRT::strlen(fNextToken)-1] != ']')
+				while (ContinueParse() && fNextToken[strlen(fNextToken)-1] != ']')
 					fNextToken = GetNextToken();
 				if (ContinueParse())
 				{
@@ -1087,15 +1091,16 @@ void nsImapServerResponseParser::msg_fetch()
 					char *startPartNum = fNextToken + 5;
 					if (whereHeader > startPartNum)
 					{
-						char *partNum = (char *)PR_CALLOC((whereHeader - startPartNum) * sizeof (char));
+            PRInt32 partLength = whereHeader - startPartNum - 1; //-1 for the dot!
+            char *partNum = (char *)PR_CALLOC((partLength + 1) * sizeof (char));
 						if (partNum)
 						{
-							PL_strncpy(partNum, startPartNum, (whereHeader - startPartNum));
+              PL_strncpy(partNum, startPartNum, partLength);
 							if (ContinueParse())
 							{
 								if (PL_strstr(fNextToken, "FIELDS"))
 								{
-									while (ContinueParse() && fNextToken[nsCRT::strlen(fNextToken)-1] != ']')
+									while (ContinueParse() && fNextToken[strlen(fNextToken)-1] != ']')
 										fNextToken = GetNextToken();
 								}
 								if (ContinueParse())
@@ -1174,8 +1179,8 @@ void nsImapServerResponseParser::msg_fetch()
 				
 				// if this token ends in ')', then it is the last token
 				// else we advance
-				if ( *(fNextToken + nsCRT::strlen(fNextToken) - 1) == ')')
-					fNextToken += nsCRT::strlen(fNextToken) - 1;
+				if ( *(fNextToken + strlen(fNextToken) - 1) == ')')
+					fNextToken += strlen(fNextToken) - 1;
 				else
 					fNextToken = GetNextToken();
 			}
@@ -1498,6 +1503,20 @@ void nsImapServerResponseParser::flags()
 					 && !PL_strncasecmp(fNextToken, "$Forwarded",10))
 				messageFlags |= kImapMsgForwardedFlag;
 				break;
+                        case 'L':
+                                if (fSupportsUserDefinedFlags & kImapMsgSupportUserFlag
+					 && !PL_strncasecmp(fNextToken, "$Label", 6))
+                                {
+                                  PRInt32 labelValue = fNextToken[6];
+                                  if (labelValue > '0')
+                                  {
+                                    // turn off any previous label flags
+                                    messageFlags &= ~kImapMsgLabelFlags;
+                                    // turn on this label flag
+                                    messageFlags |= (labelValue - '0') << 9;
+                                  }
+                                }
+				break;
 			default:
 				break;
 			}
@@ -1603,7 +1622,7 @@ void nsImapServerResponseParser::resp_text_code()
 {
 	// this is a special case way of advancing the token
 	// strtok won't break up "[ALERT]" into separate tokens
-	if (nsCRT::strlen(fNextToken) > 1)
+	if (strlen(fNextToken) > 1)
 		fNextToken++;
 	else 
 		fNextToken = GetNextToken();
@@ -1668,11 +1687,12 @@ void nsImapServerResponseParser::resp_text_code()
 				{
 					fSettablePermanentFlags |= kImapMsgDraftFlag;
 				}
-				else if (!PL_strncasecmp(fNextToken, "\\*)]", 4))
+				else if (!PL_strncasecmp(fNextToken, "\\*", 2))
 				{
 					 fSupportsUserDefinedFlags |= kImapMsgSupportUserFlag;
 					 fSupportsUserDefinedFlags |= kImapMsgSupportForwardedFlag;
 					 fSupportsUserDefinedFlags |= kImapMsgSupportMDNSentFlag;
+                                         fSupportsUserDefinedFlags |= kImapMsgLabelFlags;
 				}
 			} while (!at_end_of_line() && ContinueParse());
 			if (fFlagState)
@@ -2223,43 +2243,43 @@ void nsImapServerResponseParser::myrights_data()
 
 void nsImapServerResponseParser::acl_data()
 {
-	fNextToken = GetNextToken();
-	if (ContinueParse() && !at_end_of_line())
-	{
-		char *mailboxName = CreateAstring();	// PL_strdup(fNextToken);
-		if (mailboxName && ContinueParse())
-		{
-			fNextToken = GetNextToken();
-			while (ContinueParse() && !at_end_of_line())
-			{
-				char *userName = CreateAstring(); // PL_strdup(fNextToken);
-				if (userName && ContinueParse())
-				{
-					fNextToken = GetNextToken();
-					if (ContinueParse())
-					{
-						char *rights = CreateAstring(); // PL_strdup(fNextToken);
-						if (rights)
-						{
-							fServerConnection.AddFolderRightsForUser(mailboxName, userName, rights);
-							PR_Free(rights);
-						}
-						else
-							HandleMemoryFailure();
-
-						if (ContinueParse())
-							fNextToken = GetNextToken();
-					}
-					PR_Free(userName);
-				}
-				else
-					HandleMemoryFailure();
-			}
-			PR_Free(mailboxName);
-		}
-		else
-			HandleMemoryFailure();
-	}
+  fNextToken = GetNextToken();
+  if (ContinueParse() && !at_end_of_line())
+  {
+    char *mailboxName = CreateAstring();	// PL_strdup(fNextToken);
+    if (mailboxName && ContinueParse())
+    {
+      fNextToken = GetNextToken();
+      while (ContinueParse() && !at_end_of_line())
+      {
+        char *userName = CreateAstring(); // PL_strdup(fNextToken);
+        if (userName && ContinueParse())
+        {
+          fNextToken = GetNextToken();
+          if (ContinueParse())
+          {
+            char *rights = CreateAstring(); // PL_strdup(fNextToken);
+            if (rights)
+            {
+              fServerConnection.AddFolderRightsForUser(mailboxName, userName, rights);
+              PR_Free(rights);
+            }
+            else
+              HandleMemoryFailure();
+            
+            if (ContinueParse())
+              fNextToken = GetNextToken();
+          }
+          PR_Free(userName);
+        }
+        else
+          HandleMemoryFailure();
+      }
+      PR_Free(mailboxName);
+    }
+    else
+      HandleMemoryFailure();
+  }
 }
 
 
@@ -2456,7 +2476,7 @@ PRBool nsImapServerResponseParser::msg_fetch_literal(PRBool chunk, PRInt32 origi
 
 			if (ContinueParse())
 			{
-				charsReadSoFar += nsCRT::strlen(fCurrentLine);
+				charsReadSoFar += strlen(fCurrentLine);
 				if (!fDownloadingHeaders && fCurrentCommandIsSingleMessageFetch)
 				{
 					fServerConnection.ProgressEventFunctionUsingId(IMAP_DOWNLOADING_MESSAGE);
@@ -2465,7 +2485,7 @@ PRBool nsImapServerResponseParser::msg_fetch_literal(PRBool chunk, PRInt32 origi
 				}
 				if (charsReadSoFar > numberOfCharsInThisChunk)
 				{	// this is rare.  If this msg ends in the middle of a line then only display the actual message.
-					char *displayEndOfLine = (fCurrentLine + nsCRT::strlen(fCurrentLine) - (charsReadSoFar - numberOfCharsInThisChunk));
+					char *displayEndOfLine = (fCurrentLine + strlen(fCurrentLine) - (charsReadSoFar - numberOfCharsInThisChunk));
 					char saveit = *displayEndOfLine;
 					*displayEndOfLine = 0;
 					fServerConnection.HandleMessageDownLoadLine(fCurrentLine, !lastChunk);
@@ -2474,7 +2494,7 @@ PRBool nsImapServerResponseParser::msg_fetch_literal(PRBool chunk, PRInt32 origi
 				}
 				else
 				{
-					lastCRLFwasCRCRLF = (*(fCurrentLine + nsCRT::strlen(fCurrentLine) - 1) == nsCRT::CR);
+					lastCRLFwasCRCRLF = (*(fCurrentLine + strlen(fCurrentLine) - 1) == nsCRT::CR);
 					fServerConnection.HandleMessageDownLoadLine(fCurrentLine, !lastChunk && (charsReadSoFar == numberOfCharsInThisChunk));
 				}
 			}
@@ -2492,7 +2512,7 @@ PRBool nsImapServerResponseParser::msg_fetch_literal(PRBool chunk, PRInt32 origi
 			// move the lexical analyzer state to the end of this message because this message
 			// fetch ends in the middle of this line.
 			//fCurrentTokenPlaceHolder = fLineOfTokens + nsCRT::strlen(fCurrentLine) - (charsReadSoFar - numberOfCharsInThisChunk);
-			AdvanceTokenizerStartingPoint(nsCRT::strlen(fCurrentLine) - (charsReadSoFar - numberOfCharsInThisChunk));
+			AdvanceTokenizerStartingPoint(strlen(fCurrentLine) - (charsReadSoFar - numberOfCharsInThisChunk));
 			fNextToken = GetNextToken();
 		}
 		else
@@ -2533,6 +2553,12 @@ PRInt32 nsImapServerResponseParser::FolderUID()
 	return fFolderUIDValidity;
 }
 
+void nsImapServerResponseParser::SetCurrentResponseUID(PRUint32 uid)
+{
+  if (uid > 0)
+    fCurrentResponseUID = uid;
+}
+
 PRUint32 nsImapServerResponseParser::CurrentResponseUID()
 {
 	return fCurrentResponseUID;
@@ -2598,8 +2624,11 @@ nsImapMailboxSpec *nsImapServerResponseParser::CreateCurrentMailboxSpec(const ch
 			nsIURI * aUrl = nsnull;
 			nsresult rv = NS_OK;
 			returnSpec->connection->GetCurrentUrl()->QueryInterface(NS_GET_IID(nsIURI), (void **) &aUrl);
-			if (NS_SUCCEEDED(rv) && aUrl)
-				aUrl->GetHost(&returnSpec->hostName);
+			if (NS_SUCCEEDED(rv) && aUrl) {
+                nsCAutoString host;
+				aUrl->GetHost(host);
+                returnSpec->hostName = ToNewCString(host);
+            }
 			NS_IF_RELEASE(aUrl);
 			
 		}
