@@ -77,22 +77,23 @@ static NS_DEFINE_CID(kCImapHostSessionListCID, NS_IIMAPHOSTSESSIONLIST_CID);
 
 nsImapUrl::nsImapUrl()
 {
-	m_listOfMessageIds = nsnull;
-	m_sourceCanonicalFolderPathSubString = nsnull;
-	m_destinationCanonicalFolderPathSubString = nsnull;
-	m_listOfMessageIds = nsnull;
+  m_listOfMessageIds = nsnull;
+  m_sourceCanonicalFolderPathSubString = nsnull;
+  m_destinationCanonicalFolderPathSubString = nsnull;
+  m_listOfMessageIds = nsnull;
   m_tokenPlaceHolder = nsnull;
   m_searchCriteriaString = nsnull;
-	m_idsAreUids = PR_FALSE;
-	m_mimePartSelectorDetected = PR_FALSE;
-	m_allowContentChange = PR_TRUE;	// assume we can do MPOD.
+  m_idsAreUids = PR_FALSE;
+  m_mimePartSelectorDetected = PR_FALSE;
+  m_allowContentChange = PR_TRUE;	// assume we can do MPOD.
   m_fetchPartsOnDemand = PR_FALSE; // but assume we're not doing it :-)
   m_msgLoadingFromCache = PR_FALSE;
-	m_contentModified = IMAP_CONTENT_NOT_MODIFIED;
-	m_validUrl = PR_TRUE;	// assume the best.
-	m_flags = 0;
-	m_onlineSubDirSeparator = '/'; 
-
+  m_externalLinkUrl = PR_TRUE; // we'll start this at true, and set it false in nsImapService::CreateStartOfImapUrl
+  m_contentModified = IMAP_CONTENT_NOT_MODIFIED;
+  m_validUrl = PR_TRUE;	// assume the best.
+  m_flags = 0;
+  m_onlineSubDirSeparator = '/'; 
+  
   // ** jt - the following are not ref counted
   m_copyState = nsnull;
   m_fileSpec = nsnull;
@@ -106,8 +107,8 @@ nsImapUrl::nsImapUrl()
 
 nsresult nsImapUrl::Initialize()
 {
-	nsresult rv = nsComponentManager::CreateInstance(kCImapMockChannel, nsnull, NS_GET_IID(nsIImapMockChannel), getter_AddRefs(m_mockChannel));      
-	return rv;
+  nsresult rv = nsComponentManager::CreateInstance(kCImapMockChannel, nsnull, NS_GET_IID(nsIImapMockChannel), getter_AddRefs(m_mockChannel));      
+  return rv;
 }
 
 
@@ -152,7 +153,6 @@ nsImapUrl::~nsImapUrl()
   
 NS_IMPL_ADDREF_INHERITED(nsImapUrl, nsMsgMailNewsUrl)
 
-
 NS_IMPL_RELEASE_INHERITED(nsImapUrl, nsMsgMailNewsUrl)
 
 NS_INTERFACE_MAP_BEGIN(nsImapUrl)
@@ -167,24 +167,24 @@ NS_INTERFACE_MAP_END_INHERITING(nsMsgMailNewsUrl)
 
 NS_IMETHODIMP nsImapUrl::GetRequiredImapState(nsImapState * aImapUrlState)
 {
-	if (aImapUrlState)
-	{
-		// the imap action determines the state we must be in...check the 
-		// the imap action.
-
-		if (m_imapAction & 0x10000000)
-			*aImapUrlState = nsImapSelectedState;
-		else
-			*aImapUrlState = nsImapAuthenticatedState;
-	}
-
-	return NS_OK;
+  if (aImapUrlState)
+  {
+    // the imap action determines the state we must be in...check the 
+    // the imap action.
+    
+    if (m_imapAction & 0x10000000)
+      *aImapUrlState = nsImapSelectedState;
+    else
+      *aImapUrlState = nsImapAuthenticatedState;
+  }
+  
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsImapUrl::GetImapAction(nsImapAction * aImapAction)
 {
   *aImapAction = m_imapAction;
-	return NS_OK;
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsImapUrl::SetImapAction(nsImapAction aImapAction)
@@ -207,9 +207,8 @@ NS_IMETHODIMP nsImapUrl::GetImapFolder(nsIMsgFolder **aMsgFolder)
 NS_IMETHODIMP nsImapUrl::SetImapFolder(nsIMsgFolder  * aMsgFolder)
 {
   nsresult rv;
-  m_imapFolder =
-      getter_AddRefs(NS_GetWeakReference(aMsgFolder, &rv));
-	return rv;
+  m_imapFolder = getter_AddRefs(NS_GetWeakReference(aMsgFolder, &rv));
+  return rv;
 }
 
 NS_IMETHODIMP nsImapUrl::GetImapMailFolderSink(nsIImapMailFolderSink **
@@ -228,9 +227,8 @@ NS_IMETHODIMP nsImapUrl::GetImapMailFolderSink(nsIImapMailFolderSink **
 NS_IMETHODIMP nsImapUrl::SetImapMailFolderSink(nsIImapMailFolderSink  * aImapMailFolderSink)
 {
     nsresult rv;
-    m_imapMailFolderSink =
-        getter_AddRefs(NS_GetWeakReference(aImapMailFolderSink, &rv));
-	return rv;
+    m_imapMailFolderSink = getter_AddRefs(NS_GetWeakReference(aImapMailFolderSink, &rv));
+    return rv;
 }
  
 NS_IMETHODIMP nsImapUrl::GetImapMessageSink(nsIImapMessageSink ** aImapMessageSink)
@@ -246,10 +244,9 @@ NS_IMETHODIMP nsImapUrl::GetImapMessageSink(nsIImapMessageSink ** aImapMessageSi
 
 NS_IMETHODIMP nsImapUrl::SetImapMessageSink(nsIImapMessageSink  * aImapMessageSink)
 {
-    nsresult rv;
-    m_imapMessageSink = 
-        getter_AddRefs(NS_GetWeakReference(aImapMessageSink, &rv));
-	return rv;
+  nsresult rv;
+  m_imapMessageSink = getter_AddRefs(NS_GetWeakReference(aImapMessageSink, &rv));
+  return rv;
 }
 
 NS_IMETHODIMP nsImapUrl::GetImapServerSink(nsIImapServerSink ** aImapServerSink)
@@ -265,10 +262,9 @@ NS_IMETHODIMP nsImapUrl::GetImapServerSink(nsIImapServerSink ** aImapServerSink)
 
 NS_IMETHODIMP nsImapUrl::SetImapServerSink(nsIImapServerSink  * aImapServerSink)
 {
-    nsresult rv;
-    m_imapServerSink = 
-        getter_AddRefs(NS_GetWeakReference(aImapServerSink, &rv));
-	return rv;
+  nsresult rv;
+  m_imapServerSink = getter_AddRefs(NS_GetWeakReference(aImapServerSink, &rv));
+  return rv;
 }
 
 NS_IMETHODIMP nsImapUrl::GetImapExtensionSink(nsIImapExtensionSink ** aImapExtensionSink)
@@ -284,10 +280,9 @@ NS_IMETHODIMP nsImapUrl::GetImapExtensionSink(nsIImapExtensionSink ** aImapExten
 
 NS_IMETHODIMP nsImapUrl::SetImapExtensionSink(nsIImapExtensionSink  * aImapExtensionSink)
 {
-    nsresult rv;
-    m_imapExtensionSink =
-        getter_AddRefs(NS_GetWeakReference(aImapExtensionSink, &rv));
-	return rv;
+  nsresult rv;
+  m_imapExtensionSink = getter_AddRefs(NS_GetWeakReference(aImapExtensionSink, &rv));
+  return rv;
 }
 
 NS_IMETHODIMP nsImapUrl::GetImapMiscellaneousSink(nsIImapMiscellaneousSink **
@@ -305,10 +300,9 @@ NS_IMETHODIMP nsImapUrl::GetImapMiscellaneousSink(nsIImapMiscellaneousSink **
 NS_IMETHODIMP nsImapUrl::SetImapMiscellaneousSink(nsIImapMiscellaneousSink  *
                                               aImapMiscellaneousSink)
 {
-    nsresult rv;
-    m_imapMiscellaneousSink =
-        getter_AddRefs(NS_GetWeakReference(aImapMiscellaneousSink, &rv));
-	return rv;
+  nsresult rv;
+  m_imapMiscellaneousSink = getter_AddRefs(NS_GetWeakReference(aImapMiscellaneousSink, &rv));
+  return rv;
 }
 
         
@@ -316,7 +310,7 @@ NS_IMETHODIMP nsImapUrl::SetImapMiscellaneousSink(nsIImapMiscellaneousSink  *
 // End nsIImapUrl specific support
 ////////////////////////////////////////////////////////////////////////////////////
 
-NS_IMETHODIMP nsImapUrl::SetSpec(const char * aSpec)
+NS_IMETHODIMP nsImapUrl::SetSpec(const nsACString &aSpec)
 {
   nsresult rv = nsMsgMailNewsUrl::SetSpec(aSpec);
   if (NS_SUCCEEDED(rv))
@@ -324,7 +318,7 @@ NS_IMETHODIMP nsImapUrl::SetSpec(const char * aSpec)
   return rv;
 }
 
-NS_IMETHODIMP nsImapUrl::SetQuery(const char *aQuery)
+NS_IMETHODIMP nsImapUrl::SetQuery(const nsACString &aQuery)
 {
   nsresult rv = nsMsgMailNewsUrl::SetQuery(aQuery);
   if (NS_SUCCEEDED(rv))
@@ -334,19 +328,18 @@ NS_IMETHODIMP nsImapUrl::SetQuery(const char *aQuery)
 
 nsresult nsImapUrl::ParseUrl()
 {
-	nsresult rv = NS_OK;
+  nsresult rv = NS_OK;
   // extract the user name
-  GetPreHost(getter_Copies(m_userName));
-
-	char * imapPartOfUrl = nsnull;
-	rv = GetPath(&imapPartOfUrl);
-    imapPartOfUrl = nsUnescape(imapPartOfUrl);
-	if (NS_SUCCEEDED(rv) && imapPartOfUrl && imapPartOfUrl+1)
-	{
-		ParseImapPart(imapPartOfUrl+1);  // GetPath leaves leading '/' in the path!!!
-		nsCRT::free(imapPartOfUrl);
-	}
-
+  GetUserPass(m_userName);
+  
+  nsCAutoString imapPartOfUrl;
+  rv = GetPath(imapPartOfUrl);
+  NS_UnescapeURL(imapPartOfUrl);
+  if (NS_SUCCEEDED(rv) && !imapPartOfUrl.IsEmpty())
+  {
+    ParseImapPart((char*)imapPartOfUrl.get()+1);  // GetPath leaves leading '/' in the path!!!
+  }
+  
   return NS_OK;
 }
 
@@ -368,7 +361,7 @@ NS_IMETHODIMP nsImapUrl::CreateListOfMessageIdsString(char ** aResult)
 	if (nsnull == aResult || !m_listOfMessageIds) 
 		return  NS_ERROR_NULL_POINTER;
 
-  PRInt32 bytesToCopy = nsCRT::strlen(m_listOfMessageIds);
+  PRInt32 bytesToCopy = strlen(m_listOfMessageIds);
 
 	// mime may have glommed a "&part=" for a part download
 	// we return the entire message and let mime extract
@@ -460,275 +453,275 @@ nsImapUrl::GetChildDiscoveryDepth(PRInt32* result)
 // this method is only called from the imap thread
 NS_IMETHODIMP nsImapUrl::GetMsgFlags(imapMessageFlagsType *result)	// kAddMsgFlags or kSubtractMsgFlags only
 {
-	*result = m_flags;
+  *result = m_flags;
   return NS_OK;
 }
 
 void nsImapUrl::ParseImapPart(char *imapPartOfUrl)
 {
-	m_tokenPlaceHolder = imapPartOfUrl;
-	m_urlidSubString = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
-
-	if (!m_urlidSubString)
-	{
-		m_validUrl = PR_FALSE;
-		return;
-	}
-	
+  m_tokenPlaceHolder = imapPartOfUrl;
+  m_urlidSubString = m_tokenPlaceHolder ? nsIMAPGenericParser::Imapstrtok_r(nsnull, IMAP_URL_TOKEN_SEPARATOR, &m_tokenPlaceHolder) : (char *)NULL;
+  
+  if (!m_urlidSubString)
+  {
+    m_validUrl = PR_FALSE;
+    return;
+  }
+  
   if (!nsCRT::strcasecmp(m_urlidSubString, "fetch"))
-	{
-		m_imapAction   					 = nsImapMsgFetch;
-		ParseUidChoice();
-		PR_FREEIF(m_sourceCanonicalFolderPathSubString);
-		ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		ParseListOfMessageIds();
-	}
-	else /* if (fInternal) no concept of internal - not sure there will be one */
-	{
-		if (!nsCRT::strcasecmp(m_urlidSubString, "header"))
-		{
-			m_imapAction   					 = nsImapMsgHeader;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "deletemsg"))
-		{
-			m_imapAction   					 = nsImapDeleteMsg;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "uidexpunge"))
-		{
-			m_imapAction   					 = nsImapUidExpunge;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "deleteallmsgs"))
-		{
-			m_imapAction   					 = nsImapDeleteAllMsgs;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "addmsgflags"))
-		{
-			m_imapAction   					 = nsImapAddMsgFlags;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseMsgFlags();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "subtractmsgflags"))
-		{
-			m_imapAction   					 = nsImapSubtractMsgFlags;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseMsgFlags();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "setmsgflags"))
-		{
-			m_imapAction   					 = nsImapSetMsgFlags;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseMsgFlags();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinecopy"))
-		{
-			m_imapAction   					 = nsImapOnlineCopy;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinemove"))
-		{
-			m_imapAction   					 = nsImapOnlineMove;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinetoofflinecopy"))
-		{
-			m_imapAction   					 = nsImapOnlineToOfflineCopy;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinetoofflinemove"))
-		{
-			m_imapAction   					 = nsImapOnlineToOfflineMove;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "offlinetoonlinecopy"))
-		{
-			m_imapAction   					 = nsImapOfflineToOnlineMove;
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "search"))
-		{
-			m_imapAction   					 = nsImapSearch;
-			ParseUidChoice();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseSearchCriteriaString();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "test"))
-		{
-			m_imapAction   					 = nsImapTest;
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "select"))
-		{
-			m_imapAction   					 = nsImapSelectFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			if (m_tokenPlaceHolder && *m_tokenPlaceHolder)
-				ParseListOfMessageIds();
-			else
-				m_listOfMessageIds = PL_strdup("");
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "liteselect"))
-		{
-			m_imapAction   					 = nsImapLiteSelectFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "selectnoop"))
-		{
-			m_imapAction   					 = nsImapSelectNoopFolder;
-			m_listOfMessageIds = PL_strdup("");
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "expunge"))
-		{
-			m_imapAction   					 = nsImapExpungeFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			m_listOfMessageIds = PL_strdup("");		// no ids to UNDO
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "create"))
-		{
-			m_imapAction   					 = nsImapCreateFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "ensureExists"))
-		{
-			m_imapAction   					 = nsImapEnsureExistsFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverchildren"))
-		{
-			m_imapAction   					 = nsImapDiscoverChildrenUrl;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverlevelchildren"))
-		{
-			m_imapAction   					 = nsImapDiscoverLevelChildrenUrl;
-			ParseChildDiscoveryDepth();
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverallboxes"))
-		{
-			m_imapAction   					 = nsImapDiscoverAllBoxesUrl;
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverallandsubscribedboxes"))
-		{
-			m_imapAction   					 = nsImapDiscoverAllAndSubscribedBoxesUrl;
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "delete"))
-		{
-			m_imapAction   					 = nsImapDeleteFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "rename"))
-		{
-			m_imapAction   					 = nsImapRenameFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "movefolderhierarchy"))
-		{
-			m_imapAction   					 = nsImapMoveFolderHierarchy;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			if (m_tokenPlaceHolder && *m_tokenPlaceHolder)	// handle promote to root
-				ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "list"))
-		{
-			m_imapAction   					 = nsImapLsubFolders;
-			ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "biff"))
-		{
-			m_imapAction   					 = nsImapBiff;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-			ParseListOfMessageIds();
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "netscape"))
-		{
-			m_imapAction   					 = nsImapGetMailAccountUrl;
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "appendmsgfromfile"))
-		{
-			m_imapAction					 = nsImapAppendMsgFromFile;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "appenddraftfromfile"))
-		{
-			m_imapAction					 = nsImapAppendDraftFromFile;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+  {
+    m_imapAction  = nsImapMsgFetch;
+    ParseUidChoice();
+    PR_FREEIF(m_sourceCanonicalFolderPathSubString);
+    ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    ParseListOfMessageIds();
+  }
+  else /* if (fInternal) no concept of internal - not sure there will be one */
+  {
+    if (!nsCRT::strcasecmp(m_urlidSubString, "header"))
+    {
+      m_imapAction = nsImapMsgHeader;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "deletemsg"))
+    {
+      m_imapAction = nsImapDeleteMsg;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "uidexpunge"))
+    {
+      m_imapAction = nsImapUidExpunge;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "deleteallmsgs"))
+    {
+      m_imapAction = nsImapDeleteAllMsgs;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "addmsgflags"))
+    {
+      m_imapAction = nsImapAddMsgFlags;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseMsgFlags();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "subtractmsgflags"))
+    {
+      m_imapAction = nsImapSubtractMsgFlags;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseMsgFlags();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "setmsgflags"))
+    {
+      m_imapAction = nsImapSetMsgFlags;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseMsgFlags();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinecopy"))
+    {
+      m_imapAction = nsImapOnlineCopy;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinemove"))
+    {
+      m_imapAction = nsImapOnlineMove;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinetoofflinecopy"))
+    {
+      m_imapAction = nsImapOnlineToOfflineCopy;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "onlinetoofflinemove"))
+    {
+      m_imapAction = nsImapOnlineToOfflineMove;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "offlinetoonlinecopy"))
+    {
+      m_imapAction = nsImapOfflineToOnlineMove;
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "search"))
+    {
+      m_imapAction = nsImapSearch;
+      ParseUidChoice();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseSearchCriteriaString();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "test"))
+    {
+      m_imapAction = nsImapTest;
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "select"))
+    {
+      m_imapAction = nsImapSelectFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      if (m_tokenPlaceHolder && *m_tokenPlaceHolder)
+        ParseListOfMessageIds();
+      else
+        m_listOfMessageIds = PL_strdup("");
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "liteselect"))
+    {
+      m_imapAction = nsImapLiteSelectFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "selectnoop"))
+    {
+      m_imapAction = nsImapSelectNoopFolder;
+      m_listOfMessageIds = PL_strdup("");
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "expunge"))
+    {
+      m_imapAction = nsImapExpungeFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      m_listOfMessageIds = PL_strdup("");		// no ids to UNDO
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "create"))
+    {
+      m_imapAction = nsImapCreateFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "ensureExists"))
+    {
+      m_imapAction = nsImapEnsureExistsFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverchildren"))
+    {
+      m_imapAction = nsImapDiscoverChildrenUrl;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverlevelchildren"))
+    {
+      m_imapAction = nsImapDiscoverLevelChildrenUrl;
+      ParseChildDiscoveryDepth();
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverallboxes"))
+    {
+      m_imapAction = nsImapDiscoverAllBoxesUrl;
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "discoverallandsubscribedboxes"))
+    {
+      m_imapAction = nsImapDiscoverAllAndSubscribedBoxesUrl;
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "delete"))
+    {
+      m_imapAction = nsImapDeleteFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "rename"))
+    {
+      m_imapAction = nsImapRenameFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "movefolderhierarchy"))
+    {
+      m_imapAction = nsImapMoveFolderHierarchy;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      if (m_tokenPlaceHolder && *m_tokenPlaceHolder)	// handle promote to root
+        ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "list"))
+    {
+      m_imapAction = nsImapLsubFolders;
+      ParseFolderPath(&m_destinationCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "biff"))
+    {
+      m_imapAction = nsImapBiff;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+      ParseListOfMessageIds();
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "netscape"))
+    {
+      m_imapAction = nsImapGetMailAccountUrl;
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "appendmsgfromfile"))
+    {
+      m_imapAction = nsImapAppendMsgFromFile;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "appenddraftfromfile"))
+    {
+      m_imapAction = nsImapAppendDraftFromFile;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
       ParseUidChoice();
       if (m_tokenPlaceHolder && *m_tokenPlaceHolder)
-          ParseListOfMessageIds();
+        ParseListOfMessageIds();
       else
         m_listOfMessageIds = nsCRT::strdup("");
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "subscribe"))
-		{
-			m_imapAction					 = nsImapSubscribe;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "unsubscribe"))
-		{
-			m_imapAction					 = nsImapUnsubscribe;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshacl"))
-		{
-			m_imapAction					= nsImapRefreshACL;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshfolderurls"))
-		{
-			m_imapAction					= nsImapRefreshFolderUrls;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshallacls"))
-		{
-			m_imapAction					= nsImapRefreshAllACLs;
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "listfolder"))
-		{
-			m_imapAction					= nsImapListFolder;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "upgradetosubscription"))
-		{
-			m_imapAction					= nsImapUpgradeToSubscription;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else if (!nsCRT::strcasecmp(m_urlidSubString, "folderstatus"))
-		{
-			m_imapAction					= nsImapFolderStatus;
-			ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
-		}
-		else
-		{
-			m_validUrl = PR_FALSE;	
-		}
-	}
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "subscribe"))
+    {
+      m_imapAction = nsImapSubscribe;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "unsubscribe"))
+    {
+      m_imapAction = nsImapUnsubscribe;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshacl"))
+    {
+      m_imapAction = nsImapRefreshACL;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshfolderurls"))
+    {
+      m_imapAction = nsImapRefreshFolderUrls;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "refreshallacls"))
+    {
+      m_imapAction = nsImapRefreshAllACLs;
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "listfolder"))
+    {
+      m_imapAction = nsImapListFolder;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "upgradetosubscription"))
+    {
+      m_imapAction = nsImapUpgradeToSubscription;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else if (!nsCRT::strcasecmp(m_urlidSubString, "folderstatus"))
+    {
+      m_imapAction = nsImapFolderStatus;
+      ParseFolderPath(&m_sourceCanonicalFolderPathSubString);
+    }
+    else
+    {
+      m_validUrl = PR_FALSE;	
+    }
+        }
 }
 
 
@@ -759,7 +752,7 @@ NS_IMETHODIMP nsImapUrl::AddOnlineDirectoryIfNecessary(const char *onlineMailbox
 		// This invariant should be maintained by libmsg when reading/writing the prefs.
 		// We are only supporting online directories whose online delimiter is /
 		// Therefore, the online directory must end in a slash.
-		NS_ASSERTION (onlineDir[nsCRT::strlen(onlineDir) - 1] == '/', 
+		NS_ASSERTION (onlineDir[strlen(onlineDir) - 1] == '/', 
                       "online directory not ended with a slash\n");
 #endif
         nsIMAPNamespace *ns = nsnull;
@@ -778,8 +771,8 @@ NS_IMETHODIMP nsImapUrl::AddOnlineDirectoryIfNecessary(const char *onlineMailbox
 
 			// The namespace for this mailbox is the root ("").
 			// Prepend the online server directory
-			int finalLen = nsCRT::strlen(onlineDir) +
-                nsCRT::strlen(onlineMailboxName) + 1;
+			int finalLen = strlen(onlineDir) +
+			               strlen(onlineMailboxName) + 1;
 			newOnlineName = (char *)PR_Malloc(finalLen);
 			if (newOnlineName)
 			{
@@ -834,7 +827,7 @@ NS_IMETHODIMP nsImapUrl::AllocateServerPath(const char * canonicalPath, char onl
   NS_ENSURE_ARG(sourcePath);
   NS_ENSURE_ARG(resultPath);
   PRInt32 extra = 0;
-  PRInt32 len = nsCRT::strlen(sourcePath);
+  PRInt32 len = strlen(sourcePath);
   const char *src = sourcePath;
   PRInt32 i;
   for ( i = 0; i < len; i++)
@@ -1224,14 +1217,13 @@ NS_IMETHODIMP nsImapUrl::GetUri(char** aURI)
     CreateCanonicalSourceFolderPathString(getter_Copies(theFile));
     nsCString fullFolderPath("/");
     fullFolderPath += (const char *) m_userName;
-    char *hostName = nsnull;
-    rv = GetHost(&hostName);
+    nsCAutoString hostName;
+    rv = GetHost(hostName);
     fullFolderPath += '@';
     fullFolderPath += hostName;
     fullFolderPath += '/';
     fullFolderPath.Append(theFile);
 
-    PR_FREEIF(hostName);
     char * baseMessageURI;
     nsCreateImapBaseMessageURI(fullFolderPath.get(), &baseMessageURI);
     nsCAutoString uriStr;
@@ -1246,6 +1238,7 @@ NS_IMETHODIMP nsImapUrl::GetUri(char** aURI)
 NS_IMPL_GETSET(nsImapUrl, AddDummyEnvelope, PRBool, m_addDummyEnvelope);
 NS_IMPL_GETSET(nsImapUrl, CanonicalLineEnding, PRBool, m_canonicalLineEnding);
 NS_IMPL_GETTER(nsImapUrl::GetMsgLoadingFromCache, PRBool, m_msgLoadingFromCache);
+NS_IMPL_GETSET(nsImapUrl, ExternalLinkUrl, PRBool, m_externalLinkUrl);
 
 NS_IMETHODIMP nsImapUrl::SetMsgLoadingFromCache(PRBool loadingFromCache)
 {
@@ -1254,13 +1247,14 @@ NS_IMETHODIMP nsImapUrl::SetMsgLoadingFromCache(PRBool loadingFromCache)
   if (loadingFromCache)
   {
     nsCOMPtr<nsIMsgFolder> folder;
-	  nsMsgKey key;
+    nsMsgKey key;
 
-     nsresult rv = NS_OK;
     nsCAutoString folderURI;
     rv = nsParseImapMessageURI(mURI.get(), folderURI, &key, nsnull);
     NS_ENSURE_SUCCESS(rv, rv);
 
+    if (m_imapAction != nsImapMsgFetch) // only do this on msg fetch, i.e., if user is reading msg.
+      return rv;
     rv = GetMsgFolder(getter_AddRefs(folder));
 
     nsCOMPtr <nsIMsgDatabase> database;
