@@ -32,6 +32,14 @@ static NS_DEFINE_CID(kAbCardPropertyCID,	NS_ABCARDPROPERTY_CID);
 
 #define	kWhitespace	" \t\b\r\n"
 
+#define ADD_FIELD_TO_DB_ROW(pdb, func, dbRow, val, uniStr)    \
+  if (!val.IsEmpty())                                         \
+  {                                                           \
+        ConvertToUnicode(val.get(), uniStr);                  \
+        pdb->func(dbRow, NS_ConvertUCS2toUTF8(uniStr).get()); \
+  }
+
+
 // If we get a line longer than 16K it's just toooooo bad!
 #define kEudoraAddressBufferSz	(16 * 1024)
 
@@ -111,7 +119,7 @@ nsresult nsEudoraAddress::ImportAddresses( PRUint32 *pBytes, PRBool *pAbort, con
 		if (wasTruncated)
 			pLine[kEudoraAddressBufferSz - 1] = 0;
 		if (NS_SUCCEEDED( rv)) {
-			PRInt32	len = nsCRT::strlen( pLine);
+			PRInt32	len = strlen( pLine);
 			ProcessLine( pLine, len, errors);
 			rv = pSrc->Eof( &eof);
 			if (pBytes)
@@ -245,8 +253,6 @@ CAliasEntry *nsEudoraAddress::ProcessAlias( const char *pLine, PRInt32 len, nsSt
 	PRInt32		cnt = GetAliasName( pLine, len, name);
 	pLine += cnt;
 	len -= cnt;
-	if (!cnt || !len)
-		return( nsnull);
 
 	// we have 3 known forms of addresses in Eudora
 	// 1) real name <email@address>
@@ -256,6 +262,8 @@ CAliasEntry *nsEudoraAddress::ProcessAlias( const char *pLine, PRInt32 len, nsSt
 	// 5) <email@address> (Real name)
 	
 	CAliasEntry *pEntry = new CAliasEntry( name);
+	if (!cnt || !len)
+		return(pEntry);
 	
 	// Theoretically, an alias is just an RFC822 email adress, but it may contain
 	// an alias to another alias as the email!  I general, it appears close
@@ -311,13 +319,8 @@ CAliasEntry *nsEudoraAddress::ProcessAlias( const char *pLine, PRInt32 len, nsSt
 		}
 	}
 	
-	if (pEntry->m_list.Count())
-		return( pEntry);
-	
-	// TLR: FIXME: Put an error string in the log indicating this alias could not be parsed.
-	
-	delete pEntry;
-	return( nsnull);	
+  // Always return the entry even if there's no other attribute associated with the contact.
+  return( pEntry);	
 }
 
 
@@ -538,7 +541,7 @@ void DumpAliasArray( nsVoidArray& a)
 	for (PRInt32 i = 0; i < cnt; i++) {
 		pEntry = (CAliasEntry *)a.ElementAt( i);
 		IMPORT_LOG1( "\tAlias: %s\n", pEntry->m_name.get());
-		if (pEntry->m_list.Count() > 1) {
+    if (pEntry->m_list.Count() > 1) {
 			IMPORT_LOG1( "\tList count #%ld\n", pEntry->m_list.Count());
 			for (PRInt32 j = 0; j < pEntry->m_list.Count(); j++) {
 				pData = (CAliasData *) pEntry->m_list.ElementAt( j);
@@ -548,7 +551,7 @@ void DumpAliasArray( nsVoidArray& a)
 				IMPORT_LOG1( "\t\tnickName: %s\n", pData->m_nickName.get());
 			}
 		}
-		else {
+		else if (pEntry->m_list.Count()) {
 			pData = (CAliasData *) pEntry->m_list.ElementAt( 0);
 			IMPORT_LOG1( "\t\temail: %s\n", pData->m_email.get());
 			IMPORT_LOG1( "\t\trealName: %s\n", pData->m_realName.get());
@@ -564,7 +567,7 @@ CAliasEntry *nsEudoraAddress::ResolveAlias( nsCString& name)
 	CAliasEntry *pEntry;
 	for (PRInt32 i = 0; i < max; i++) {
 		pEntry = (CAliasEntry *) m_alias.ElementAt( i);
-		if (!name.CompareWithConversion( pEntry->m_name.get(), PR_TRUE))
+		if (name.Equals( pEntry->m_name, nsCaseInsensitiveCStringComparator()))
 			return( pEntry);
 	}
 	
@@ -584,7 +587,9 @@ void nsEudoraAddress::ResolveEntries( nsCString& name, nsVoidArray& list, nsVoid
 	for (i = 0; i < max; i++) {
 		pData = (CAliasData *)list.ElementAt( i);
 		// resolve the email to an existing alias!
-		if (name.CompareWithConversion( pData->m_email.get(), PR_TRUE) && ((pEntry = ResolveAlias( pData->m_email)) != nsnull)) {
+		if (!name.Equals( pData->m_email,
+                          nsCaseInsensitiveCStringComparator()) &&
+            ((pEntry = ResolveAlias( pData->m_email)) != nsnull)) {
 			// This new entry has all of the entries for this puppie.
 			// Resolve all of it's entries!
 			ResolveEntries( pEntry->m_name, pEntry->m_list, result);
@@ -614,29 +619,48 @@ PRInt32 nsEudoraAddress::FindAlias( nsCString& name)
 void nsEudoraAddress::BuildABCards( PRUint32 *pBytes, nsIAddrDatabase *pDb)
 {
 	CAliasEntry *	pEntry;
-	CAliasData *	pData;
 	PRInt32			max = m_alias.Count();
 	PRInt32			i;
-	PRInt32			numAddrs;
-	nsCString		email;
-	nsVoidArray		entries;
+	nsVoidArray		emailList;
+  nsVoidArray membersArray;// Remember group members.
+  nsVoidArray groupsArray; // Remember groups.
 	
 	// First off, run through the list and build person cards - groups/lists have to be done later
 	for (i = 0; i < max; i++) {
 		pEntry = (CAliasEntry *) m_alias.ElementAt( i);
-		ResolveEntries( pEntry->m_name, pEntry->m_list, entries);		
-		numAddrs = entries.Count();
-		if (numAddrs == 1) {
-			pData = (CAliasData *)entries.ElementAt( 0);
-			BuildSingleCard( pEntry, pData, pDb);
-		}
-		entries.Clear();
+		ResolveEntries( pEntry->m_name, pEntry->m_list, emailList);		
+    if (emailList.Count() > 1)
+    {
+      // Remember group members uniquely and add them to db later.
+      RememberGroupMembers(membersArray, emailList);
+      // Remember groups and add them to db later.
+      groupsArray.AppendElement(pEntry);
+    }
+    else
+      AddSingleCard( pEntry, emailList, pDb);
+
+		emailList.Clear();
 
 		if (pBytes) {
 			// This isn't exact but it will get us close enough
 			*pBytes += (pEntry->m_name.Length() + pEntry->m_notes.Length() + 10);
 		}
 	}
+
+  // Make sure group members exists before adding groups.
+  nsresult rv = AddGroupMembersAsCards(membersArray, pDb);
+  if (NS_FAILED(rv))
+    return;
+
+  // Now add the lists/groups (now that all cards have been added).
+  max = groupsArray.Count();
+  for (i = 0; i < max; i++)
+  {
+    pEntry = (CAliasEntry *) groupsArray.ElementAt(i);
+    ResolveEntries( pEntry->m_name, pEntry->m_list, emailList);
+    AddSingleList(pEntry, emailList, pDb);
+    emailList.Clear();
+  }
 }
 
 void nsEudoraAddress::ExtractNoteField( nsCString& note, nsCString& value, const char *pFieldName)
@@ -654,7 +678,7 @@ void nsEudoraAddress::ExtractNoteField( nsCString& note, nsCString& value, const
 	PRInt32 idx = note.Find( field);
 	if (idx != -1) {
 		idx += field.Length();
-		PRInt32 endIdx = note.FindChar( '>', PR_FALSE, idx);
+		PRInt32 endIdx = note.FindChar( '>', idx);
 		if (endIdx == -1)
 			endIdx = note.Length() - 1;
 		note.Mid( value, idx, endIdx - idx);
@@ -697,37 +721,56 @@ void nsEudoraAddress::SplitString( nsCString& val1, nsCString& val2)
 	}
 }
 
-void nsEudoraAddress::BuildSingleCard( CAliasEntry *pEntry, CAliasData *pData, nsIAddrDatabase *pDb)
+void nsEudoraAddress::AddSingleCard( CAliasEntry *pEntry, nsVoidArray &emailList, nsIAddrDatabase *pDb)
 {
-	// we have a nickname always
-	// a real name sometimes
-	// an email address always
-	
-	// anything else found in notes!
-	
+  // We always have a nickname and everything else is optional.
+  // Map both home and work related fiedls to our address card. Eudora
+  // fields that can't be mapped will be left in the 'note' field!
 	nsIMdbRow* newRow = nsnull;
 	pDb->GetNewRow( &newRow); 
-	
-	/*
-		
-	*/
+  if (!newRow)
+    return;
 
-	nsCString	displayName(pData->m_realName);
-	nsCString	name;
-	nsCString	fax;
-	nsCString	phone;
-	nsCString	address;
-	nsCString	address2;
+	nsCString	displayName, name, firstName, lastName;
+	nsCString	fax, phone, mobile, webLink;
+	nsCString	address, address2, city, state, zip, country;
+	nsCString	phoneWK, webLinkWK, title, company;
+  nsCString	addressWK, address2WK, cityWK, stateWK, zipWK, countryWK;
 	nsCString	note(pEntry->m_notes);
-	if (note.Length() > 0) {
+
+	if (note.Length() > 0)
+  {
 		ExtractNoteField( note, fax, "fax");
 		ExtractNoteField( note, phone, "phone");
+    ExtractNoteField( note, mobile, "mobile");
 		ExtractNoteField( note, address, "address");
+    ExtractNoteField( note, city, "city");
+    ExtractNoteField( note, state, "state");
+    ExtractNoteField( note, zip, "zip");
+    ExtractNoteField( note, country, "country");
 		ExtractNoteField( note, name, "name");
+    ExtractNoteField( note, firstName, "first");
+    ExtractNoteField( note, lastName, "last");
+    ExtractNoteField( note, webLink, "web");
+
+    ExtractNoteField( note, addressWK, "address2");
+    ExtractNoteField( note, cityWK, "city2");
+    ExtractNoteField( note, stateWK, "state2");
+    ExtractNoteField( note, zipWK, "zip2");
+    ExtractNoteField( note, countryWK, "country2");
+		ExtractNoteField( note, phoneWK, "phone2");
+    ExtractNoteField( note, title, "title");
+    ExtractNoteField( note, company, "company");
+    ExtractNoteField( note, webLinkWK, "web2");
 	}
-	if (displayName.IsEmpty())
+
+  CAliasData *pData = emailList.Count() ? (CAliasData *)emailList.ElementAt(0) : nsnull;
+
+  if (pData && !pData->m_realName.IsEmpty())
+    displayName = pData->m_realName;
+	else if (!name.IsEmpty())
 		displayName = name;
-	if (displayName.IsEmpty())
+	else
 		displayName = pEntry->m_name;
 	
 	address.ReplaceSubstring( "\x03", "\x0D\x0A");
@@ -736,43 +779,57 @@ void nsEudoraAddress::BuildSingleCard( CAliasEntry *pEntry, CAliasData *pData, n
 	fax.ReplaceSubstring( "\x03", " ");
 	phone.ReplaceSubstring( "\x03", " ");
 	name.ReplaceSubstring( "\x03", " ");
+  city.ReplaceSubstring( "\x03", " ");
+	state.ReplaceSubstring( "\x03", " ");
+  zip.ReplaceSubstring( "\x03", " ");
+  country.ReplaceSubstring( "\x03", " ");
+
+  addressWK.ReplaceSubstring( "\x03", "\x0D\x0A");
+  SplitString( addressWK, address2WK);
+	phoneWK.ReplaceSubstring( "\x03", " ");
+  cityWK.ReplaceSubstring( "\x03", " ");
+	stateWK.ReplaceSubstring( "\x03", " ");
+  zipWK.ReplaceSubstring( "\x03", " ");
+  countryWK.ReplaceSubstring( "\x03", " ");
+  title.ReplaceSubstring( "\x03", " ");
+  company.ReplaceSubstring( "\x03", " ");
 	
-	if (newRow) {
+	if (newRow)
+  {
 		nsAutoString uniStr;
 
-		ConvertToUnicode( displayName.get(), uniStr);
-		pDb->AddDisplayName( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
+    // Home related fields.
+    ADD_FIELD_TO_DB_ROW(pDb, AddDisplayName, newRow, displayName, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddNickName, newRow, pEntry->m_name, uniStr);
+    if (pData)
+      ADD_FIELD_TO_DB_ROW(pDb, AddPrimaryEmail, newRow, pData->m_email, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddFirstName, newRow, firstName, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddLastName, newRow, lastName, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWebPage2, newRow, webLink, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddFaxNumber, newRow, fax, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomePhone, newRow, phone, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeAddress, newRow, address, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeAddress2, newRow, address2, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeCity, newRow, city, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeZipCode, newRow, zip, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeState, newRow, state, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddHomeCountry, newRow, country, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddCellularNumber, newRow, mobile, uniStr);
 
-		ConvertToUnicode( pEntry->m_name.get(), uniStr);
-		pDb->AddNickName( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
+    // Work related fields.
+    ADD_FIELD_TO_DB_ROW(pDb, AddJobTitle, newRow, title, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddCompany, newRow, company, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWebPage1, newRow, webLinkWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkPhone, newRow, phoneWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkAddress, newRow, addressWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkAddress2, newRow, address2WK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkCity, newRow, cityWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkZipCode, newRow, zipWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkState, newRow, stateWK, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddWorkCountry, newRow, countryWK, uniStr);
 
-		ConvertToUnicode( pData->m_email.get(), uniStr);
-		pDb->AddPrimaryEmail( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-
-		if (!fax.IsEmpty()) {
-			ConvertToUnicode( fax.get(), uniStr);
-			pDb->AddFaxNumber( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-		}
-
-		if (!phone.IsEmpty()) {
-			ConvertToUnicode( phone.get(), uniStr);
-			pDb->AddHomePhone( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-		}
-
-		if (!address.IsEmpty()) {
-			ConvertToUnicode( address.get(), uniStr);
-			pDb->AddHomeAddress( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-		}
-
-		if (!address2.IsEmpty()) {
-			ConvertToUnicode( address2.get(), uniStr);
-			pDb->AddHomeAddress2( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-		}
-
-		if (!note.IsEmpty()) {
-			ConvertToUnicode( note.get(), uniStr);
-			pDb->AddNotes( newRow, NS_ConvertUCS2toUTF8(uniStr).get());
-		}
+    // Lastly, note field.
+    ADD_FIELD_TO_DB_ROW(pDb, AddNotes, newRow, note, uniStr);
 
 		pDb->AddCardRowToDB( newRow);
 
@@ -780,4 +837,93 @@ void nsEudoraAddress::BuildSingleCard( CAliasEntry *pEntry, CAliasData *pData, n
 	}		
 }
 
+//
+// Since there is no way to check if a card for a given email address already exists,
+// elements in 'membersArray' are make unique. So for each email address in 'emailList'
+// we check it in 'membersArray' and if it's not there then we add it to 'membersArray'.
+//
+void nsEudoraAddress::RememberGroupMembers(nsVoidArray &membersArray, nsVoidArray &emailList)
+{
+  PRInt32 cnt = emailList.Count();
+  CAliasData *pData;
+
+  for (PRInt32 i = 0; i < cnt; i++)
+  {
+    pData = (CAliasData *)emailList.ElementAt(i);
+    if (!pData)
+      continue;
+
+    PRInt32 memberCnt = membersArray.Count();
+    PRInt32 j = 0;
+    for (j = 0; j < memberCnt; j++)
+    {
+      if (pData == membersArray.ElementAt(j))
+        break;
+    }
+    if (j >= memberCnt)
+      membersArray.AppendElement(pData); // add to member list
+  }
+}
+
+nsresult nsEudoraAddress::AddGroupMembersAsCards(nsVoidArray &membersArray, nsIAddrDatabase *pDb)
+{
+  PRInt32 max = membersArray.Count();
+  CAliasData *pData;
+  nsresult rv = NS_OK;
+  nsCOMPtr <nsIMdbRow> newRow;
+  nsAutoString uniStr;
+  nsCAutoString	displayName;
+
+  for (PRInt32 i = 0; i < max; i++)
+  {
+    pData = (CAliasData *)membersArray.ElementAt(i);
+
+    if (!pData || (pData->m_email.IsEmpty()))
+      continue;
+
+    rv = pDb->GetNewRow(getter_AddRefs(newRow)); 
+    if (NS_FAILED(rv) || !newRow)
+      return rv;
+
+    if (!pData->m_realName.IsEmpty())
+      displayName = pData->m_realName;
+    else if (!pData->m_nickName.IsEmpty())
+      displayName = pData->m_nickName;
+    else
+      displayName.Truncate();
+
+    ADD_FIELD_TO_DB_ROW(pDb, AddDisplayName, newRow, displayName, uniStr);
+    ADD_FIELD_TO_DB_ROW(pDb, AddPrimaryEmail, newRow, pData->m_email, uniStr);
+    rv = pDb->AddCardRowToDB( newRow);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  return rv;
+}
+
+nsresult nsEudoraAddress::AddSingleList(CAliasEntry *pEntry, nsVoidArray &emailList, nsIAddrDatabase *pDb)
+{
+  // Create a list.
+  nsCOMPtr <nsIMdbRow> newRow;
+  nsresult rv = pDb->GetNewListRow(getter_AddRefs(newRow)); 
+  if (NS_FAILED(rv) || !newRow)
+      return rv;
+
+  rv = pDb->AddListName(newRow, pEntry->m_name.get());
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // Now add the members.
+  PRInt32 max = emailList.Count();
+  for (PRInt32 i = 0; i < max; i++)
+  {
+    CAliasData *pData = (CAliasData *)emailList.ElementAt(i);
+    nsCAutoString ldifValue(NS_LITERAL_CSTRING("mail=") + nsDependentCString(pData->m_email.get()));
+    rv = pDb->AddLdifListMember(newRow, ldifValue.get());
+  }
+
+  rv = pDb->AddCardRowToDB(newRow);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  rv = pDb->AddListDirNode(newRow);
+  return rv;
+}
 
