@@ -49,6 +49,7 @@
 
 #include "nsMimeTypes.h"
 #include "nsIPref.h"
+#include "nsMsgUtils.h"
 
 static NS_DEFINE_CID( kMsgSendCID, NS_MSGSEND_CID);
 static NS_DEFINE_CID( kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID); 
@@ -175,12 +176,12 @@ nsEudoraCompose::nsEudoraCompose()
 	m_pIdentity = nsnull;
 	m_pHeaders = p_test_headers;
 	if (m_pHeaders)
-		m_headerLen = nsCRT::strlen( m_pHeaders);
+		m_headerLen = strlen( m_pHeaders);
 	else
 		m_headerLen = 0;
 	m_pBody = p_test_body;
 	if (m_pBody)
-		m_bodyLen = nsCRT::strlen( m_pBody);
+		m_bodyLen = strlen( m_pBody);
 	else
 		m_bodyLen = 0;
 
@@ -390,7 +391,7 @@ void nsEudoraCompose::GetHeaderValue( const char *pData, PRInt32 dataLen, const
 		return;
 
 	PRInt32	start = 0;
-	PRInt32 len = nsCRT::strlen( pHeader);
+	PRInt32 len = strlen( pHeader);
 	const char *pChar = pData;
 	if (!nsCRT::strncasecmp( pHeader, pData, len)) {
 		start = len;
@@ -503,7 +504,7 @@ void nsEudoraCompose::ExtractType( nsString& str)
 	// valid multipart!
 	if (str.Length() > 10) {
 		str.Left( tStr, 10);
-		if (!Compare(tStr, NS_LITERAL_STRING("multipart/"), nsCaseInsensitiveStringComparator()))
+		if (tStr.Equals(NS_LITERAL_STRING("multipart/"), nsCaseInsensitiveStringComparator()))
 			str.Truncate();
 	}
 }
@@ -537,10 +538,10 @@ nsMsgAttachedFile * nsEudoraCompose::GetLocalAttachments( void)
 	nsMsgAttachedFile *a = (nsMsgAttachedFile *) new nsMsgAttachedFile[count + 1];
 	if (!a)
 		return( nsnull);
-	nsCRT::memset(a, 0, sizeof(nsMsgAttachedFile) * (count + 1));
+	memset(a, 0, sizeof(nsMsgAttachedFile) * (count + 1));
 	
 	nsresult			rv;
-	char *				urlStr;
+	nsXPIDLCString   	urlStr;
 	ImportAttachment *	pAttach;
 
 	for (PRInt32 i = 0; i < count; i++) {
@@ -552,21 +553,20 @@ nsMsgAttachedFile * nsEudoraCompose::GetLocalAttachments( void)
 		pAttach = (ImportAttachment *) m_pAttachments->ElementAt( i);
 		a[i].file_spec = new nsFileSpec;
 		pAttach->pAttachment->GetFileSpec( a[i].file_spec);
-		urlStr = nsnull;
-		pAttach->pAttachment->GetURLString( &urlStr);
+		urlStr.Adopt(0);
+		pAttach->pAttachment->GetURLString(getter_Copies(urlStr));
 		if (!urlStr) {
 			CleanUpAttach( a, count);
 			return( nsnull);
 		}
-		rv = m_pIOService->NewURI( urlStr, nsnull, getter_AddRefs(a[i].orig_url));
-		nsCRT::free( urlStr);
+		rv = m_pIOService->NewURI( urlStr, nsnull, nsnull, getter_AddRefs(a[i].orig_url));
 		if (NS_FAILED( rv)) {
 			CleanUpAttach( a, count);
 			return( nsnull);
 		}
 
 		a[i].type = nsCRT::strdup( pAttach->mimeType);
-		a[i].description = nsCRT::strdup( pAttach->description);
+		a[i].real_name = nsCRT::strdup( pAttach->description);
 		a[i].encoding = nsCRT::strdup( ENCODING_BINARY);
 	}
 
@@ -681,6 +681,19 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 	}
 	uniBody.Truncate();
 
+
+  // See if it's a draft msg (ie, no From: or no To: AND no Cc: AND no Bcc:).
+  // Eudora saves sent and draft msgs in Out folder (ie, mixed) and it does
+  // store Bcc: header in the msg itself.
+  nsMsgDeliverMode mode = nsIMsgSend::nsMsgDeliverNow;
+  PRUnichar *from, *to, *cc, *bcc;
+  rv = m_pMsgFields->GetFrom(&from);
+  rv = m_pMsgFields->GetTo(&to);
+  rv = m_pMsgFields->GetCc(&cc);
+  rv = m_pMsgFields->GetBcc(&bcc);
+  if ( (!from || !*from) || ((!to || !*to) && (!cc || !*cc) && (!bcc || !*bcc)) )
+    mode = nsIMsgSend::nsMsgSaveAsDraft;
+
 	if (NS_FAILED( rv)) {
 
 		rv = m_pSendProxy->CreateAndSendMessage(
@@ -689,7 +702,7 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 										m_pMsgFields,	                // message fields
 										PR_FALSE,		                  // digest = NO
 										PR_TRUE,		                  // dont_deliver = YES, make a file
-										nsIMsgSend::nsMsgDeliverNow,  // mode
+										mode,                         // mode
 										nsnull,			                  // no message to replace
 										pMimeType,		                // body type
 										m_pBody,		                  // body pointer
@@ -710,7 +723,7 @@ nsresult nsEudoraCompose::SendTheMessage( nsIFileSpec *pMsg)
 										m_pMsgFields,	                // message fields
 										PR_FALSE,		                  // digest = NO
 										PR_TRUE,		                  // dont_deliver = YES, make a file
-										nsIMsgSend::nsMsgDeliverNow,  // mode
+										mode,                         // mode
 										nsnull,			                  // no message to replace
 										pMimeType,		                // body type
 										body.get(),		                      // body pointer
@@ -947,27 +960,23 @@ nsresult nsEudoraCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	char	lastChar = 0;
 
 	if (NS_SUCCEEDED( rv)) {
-		rv = pDst->Write( copy.m_pBuffer + copy.m_writeOffset, copy.m_bytesInBuf - copy.m_writeOffset, &written);
+    rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 		if (copy.m_bytesInBuf)
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
-		copy.m_writeOffset = copy.m_bytesInBuf;
+    if (NS_SUCCEEDED(rv))
+		  copy.m_writeOffset = copy.m_bytesInBuf;
 	}
 
 	while ((state.offset < state.size) && NS_SUCCEEDED( rv)) {
 		rv = FillMailBuffer( &state, copy);
 		if (NS_SUCCEEDED( rv)) {
-			rv = pDst->Write( copy.m_pBuffer, copy.m_bytesInBuf, &written);
+      rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
-			if (NS_SUCCEEDED( rv)) {
-				if (written != copy.m_bytesInBuf) {
-					rv = NS_ERROR_FAILURE;
-					IMPORT_LOG0( "*** Error writing to destination mailbox\n");
-				}
-				else
-					copy.m_writeOffset = copy.m_bytesInBuf;
-			}
+			if (NS_SUCCEEDED( rv))
+		    copy.m_writeOffset = copy.m_bytesInBuf;
+	    else
+        IMPORT_LOG0( "*** Error writing to destination mailbox\n");
 		}
-
 	}
 
 	pSrc->CloseStream();
@@ -1063,14 +1072,17 @@ nsresult nsEudoraCompose::WriteHeaders( nsIFileSpec *pDst, SimpleBufferTonyRCopi
 	nsCString	val;
 	nsCString	replaceVal;
 	PRInt32		written;
-	nsresult	rv = NS_ERROR_FAILURE;
+	nsresult	rv = NS_OK; // it's ok if we don't have the first header on the predefined lists.
 	PRInt32		specialHeader;
 	PRBool		specials[kMaxSpecialHeaders];
+  PRBool    hasDateHeader = PR_FALSE;
 	int			i;
 
 	for (i = 0; i < kMaxSpecialHeaders; i++)
 		specials[i] = PR_FALSE;
 
+  // m_pHeaders - contains headers from a Eudora msg.
+  // newHeaders - contains headers from a mozilla msg (more headers here).
 	do {
 		GetNthHeader( m_pHeaders, m_headerLen, n, header, val, PR_FALSE);
 		// GetNthHeader( newHeaders.m_pBuffer, newHeaders.m_writeOffset, n, header, val, PR_FALSE);
@@ -1090,6 +1102,9 @@ nsresult nsEudoraCompose::WriteHeaders( nsIFileSpec *pDst, SimpleBufferTonyRCopi
 					val = replaceVal;
 			}
 			if (val.Length()) {
+        // See if we're writing out a Date: header.
+        if (!nsCRT::strcasecmp(header.get(), "Date"))
+          hasDateHeader = PR_TRUE;
 				rv = pDst->Write( header.get(), header.Length(), &written);
 				if (NS_SUCCEEDED( rv))
 					rv = pDst->Write( ": ", 2, &written);
@@ -1103,6 +1118,14 @@ nsresult nsEudoraCompose::WriteHeaders( nsIFileSpec *pDst, SimpleBufferTonyRCopi
 		n++;
 	} while (NS_SUCCEEDED( rv) && (header.Length() != 0));
 
+  // If we don't have Date: header so far then use the default one (taken from Eudora "From " line).
+  if (!hasDateHeader)
+  {
+    rv = pDst->Write(m_defaultDate.get(), m_defaultDate.Length(), &written);
+    if (NS_SUCCEEDED( rv))
+      rv = pDst->Write( "\x0D\x0A", 2, &written);
+  }
+
 	for (i = 0; (i < kMaxSpecialHeaders) && NS_SUCCEEDED( rv); i++) {
 		if (!specials[i]) {
 			header = gSpecialHeaders[i];
