@@ -104,7 +104,7 @@ PRBool nsEudoraWin32::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni)
 			// Command line is Eudora mailfolder eudora.ini
 			int	idx = -1;
 			if (str.CharAt( 0) == '"') {
-				idx = str.FindChar( '"', PR_FALSE, 1);
+				idx = str.FindChar( '"', 1);
 				if (idx != -1)
 					idx++;
 			}
@@ -117,10 +117,10 @@ PRBool nsEudoraWin32::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni)
 				while (str.CharAt( idx) == ' ') idx++;
 				int eIdx = -1;
 				if (str.CharAt( idx) == '"') {
-					eIdx = str.FindChar( '"', PR_FALSE, idx);
+					eIdx = str.FindChar( '"', idx);
 				}
 				else {
-					eIdx = str.FindChar( ' ', PR_FALSE, idx);
+					eIdx = str.FindChar( ' ', idx);
 				}
 				idx = eIdx;
 			}
@@ -130,10 +130,10 @@ PRBool nsEudoraWin32::FindEudoraLocation( nsIFileSpec *pFolder, PRBool findIni)
 				while (str.CharAt( idx) == ' ') idx++;
 				int endIdx = -1;
 				if (str.CharAt( idx) == '"') {
-					endIdx = str.FindChar( '"', PR_FALSE, idx);
+					endIdx = str.FindChar( '"', idx);
 				}
 				else {
-					endIdx = str.FindChar( ' ', PR_FALSE, idx);
+					endIdx = str.FindChar( ' ', idx);
 				}
 				if (endIdx != -1) {
 					nsCString	path;
@@ -215,7 +215,7 @@ nsresult nsEudoraWin32::ScanMailDir( nsIFileSpec *pFolder, nsISupportsArray *pAr
 	if (NS_SUCCEEDED( rv) && exists && isFile) {
 		rv = descMap->GetFileContents( &pContents);
 		if (NS_SUCCEEDED( rv) && pContents) {
-			len = nsCRT::strlen( pContents);	
+			len = strlen( pContents);	
 			if (NS_SUCCEEDED( rv))
 				rv = ScanDescmap( pFolder, pArray, pImport, pContents, len);
 			nsCRT::free( pContents);
@@ -694,7 +694,7 @@ void nsEudoraWin32::GetAccountName( const char *pSection, nsString& str)
 
 	nsCString	s(pSection);
 	
-	if (!Compare(s, NS_LITERAL_CSTRING("Settings"), nsCaseInsensitiveCStringComparator())) {
+	if (s.Equals(NS_LITERAL_CSTRING("Settings"), nsCaseInsensitiveCStringComparator())) {
 		str.Assign(NS_LITERAL_STRING("Eudora "));
 		str.AppendWithConversion( pSection);
 	}
@@ -703,7 +703,7 @@ void nsEudoraWin32::GetAccountName( const char *pSection, nsString& str)
 		str.AssignWithConversion(pSection);
 		if (s.Length() > 8) {
 			s.Left( tStr, 8); 
-			if (!Compare(tStr, NS_LITERAL_CSTRING("Persona-"), nsCaseInsensitiveCStringComparator())) {
+			if (tStr.Equals(NS_LITERAL_CSTRING("Persona-"), nsCaseInsensitiveCStringComparator())) {
 				s.Right( tStr, s.Length() - 8);
 				str.AssignWithConversion(tStr.get());
 			}
@@ -872,7 +872,7 @@ void nsEudoraWin32::SetIdentities(nsIMsgAccountManager *accMgr, nsIMsgAccount *a
 		IMPORT_LOG1( "\temail: %s\n", email.get());
 	}
 
-	SetSmtpServer( accMgr, acc, server.get(), nsnull);
+	SetSmtpServer( accMgr, acc, server.get(), userName);
 		
 }
 
@@ -1378,9 +1378,8 @@ nsresult nsEudoraWin32::FoundAddressBook( nsIFileSpec *spec, const PRUnichar *pN
 		nsCRT::free( pLeaf);
 		nsString	tStr;
 		name.Right( tStr, 4);
-		if (! Compare(tStr,
-                      NS_LITERAL_STRING(".txt"),
-                      nsCaseInsensitiveStringComparator())) {
+		if (tStr.Equals(NS_LITERAL_STRING(".txt"),
+                        nsCaseInsensitiveStringComparator())) {
 			name.Left( tStr, name.Length() - 4);
 			name = tStr;
 		}
@@ -1419,11 +1418,11 @@ void nsEudoraWin32::ConvertPath( nsCString& str)
 	PRInt32		start = 0;
 	nsCString	search;
 
-	idx = str.FindChar( '\\', PR_FALSE, idx);
+	idx = str.FindChar( '\\', idx);
 	if ((idx == 2) && (str.CharAt( 1) == ':')) {
 		str.Left( path, 3);
 		idx++;
-		idx = str.FindChar( '\\', PR_FALSE, idx);
+		idx = str.FindChar( '\\', idx);
 		start = 3;
 		if ((idx == -1) && (str.Length() > 3)) {
 			str.Right( temp, str.Length() - start);
@@ -1442,7 +1441,7 @@ void nsEudoraWin32::ConvertPath( nsCString& str)
 		path.Append( findFileData.cFileName);
 		idx++;
 		start = idx;
-		idx = str.FindChar( '\\', PR_FALSE, idx);
+		idx = str.FindChar( '\\', idx);
 		FindClose( h);
 		if (idx != -1)
 			path.Append( '\\');
