@@ -30,6 +30,7 @@
 #include "nsIURI.h"
 #include "nsIProxyObjectManager.h"
 #include "nsProxiedService.h"
+#include "nsMsgI18N.h"
 
 #include "nsMsgBaseCID.h"
 #include "nsMsgCompCID.h"
@@ -47,6 +48,8 @@
 #include "OutlookDebugLog.h"
 
 #include "nsMimeTypes.h"
+#include "nsIPref.h"
+#include "nsMsgUtils.h"
 
 static NS_DEFINE_CID( kMsgSendCID, NS_MSGSEND_CID);
 static NS_DEFINE_CID( kMsgCompFieldsCID, NS_MSGCOMPFIELDS_CID); 
@@ -173,12 +176,12 @@ nsOutlookCompose::nsOutlookCompose()
 	m_pIdentity = nsnull;
 	m_pHeaders = p_test_headers;
 	if (m_pHeaders)
-		m_headerLen = nsCRT::strlen( m_pHeaders);
+		m_headerLen = strlen( m_pHeaders);
 	else
 		m_headerLen = 0;
 	m_pBody = p_test_body;
 	if (m_pBody)
-		m_bodyLen = nsCRT::strlen( m_pBody);
+		m_bodyLen = strlen( m_pBody);
 	else
 		m_bodyLen = 0;
 
@@ -387,7 +390,7 @@ void nsOutlookCompose::GetHeaderValue( const char *pData, PRInt32 dataLen, const
 		return;
 
 	PRInt32	start = 0;
-	PRInt32 len = nsCRT::strlen( pHeader);
+	PRInt32 len = strlen( pHeader);
 	const char *pChar = pData;
 	if (!nsCRT::strncasecmp( pHeader, pData, len)) {
 		start = len;
@@ -535,10 +538,10 @@ nsMsgAttachedFile * nsOutlookCompose::GetLocalAttachments( void)
 	nsMsgAttachedFile *a = (nsMsgAttachedFile *) new nsMsgAttachedFile[count + 1];
 	if (!a)
 		return( nsnull);
-	nsCRT::memset(a, 0, sizeof(nsMsgAttachedFile) * (count + 1));
+	memset(a, 0, sizeof(nsMsgAttachedFile) * (count + 1));
 	
 	nsresult			rv;
-	char *				urlStr;
+	nsXPIDLCString		urlStr;
 	OutlookAttachment *	pAttach;
 
 	for (PRInt32 i = 0; i < count; i++) {
@@ -550,30 +553,40 @@ nsMsgAttachedFile * nsOutlookCompose::GetLocalAttachments( void)
 		pAttach = (OutlookAttachment *) m_pAttachments->ElementAt( i);
 		a[i].file_spec = new nsFileSpec;
 		pAttach->pAttachment->GetFileSpec( a[i].file_spec);
-		urlStr = nsnull;
-		pAttach->pAttachment->GetURLString( &urlStr);
+		urlStr.Adopt(0);
+		pAttach->pAttachment->GetURLString(getter_Copies(urlStr));
 		if (!urlStr) {
 			CleanUpAttach( a, count);
 			return( nsnull);
 		}
-		rv = m_pIOService->NewURI( urlStr, nsnull, getter_AddRefs(a[i].orig_url));
-		nsCRT::free( urlStr);
+		rv = m_pIOService->NewURI( urlStr, nsnull, nsnull, getter_AddRefs(a[i].orig_url));
 		if (NS_FAILED( rv)) {
 			CleanUpAttach( a, count);
 			return( nsnull);
 		}
 
 		a[i].type = nsCRT::strdup( pAttach->mimeType);
-		a[i].description = nsCRT::strdup( pAttach->description);
+		a[i].real_name = nsCRT::strdup( pAttach->description);
 		a[i].encoding = nsCRT::strdup( ENCODING_BINARY);
 	}
 
 	return( a);
 }
 
+void nsOutlookCompose::ConvertSystemStringToUnicode( const char *pStr, nsString& uniStr)
+{
+  if (!m_pImportService)
+    m_pImportService = do_GetService(NS_IMPORTSERVICE_CONTRACTID);
+  NS_ASSERTION(m_pImportService, "no import service, can't convert");
+
+  if (m_pImportService)
+    m_pImportService->SystemStringToUnicode( pStr, uniStr);
+  else
+    uniStr.AssignWithConversion( pStr);
+}
 
 // Test a message send????
-nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg)
+nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg, nsMsgDeliverMode mode)
 {
 	nsresult	rv = CreateComponents();
 	if (NS_SUCCEEDED( rv))
@@ -601,11 +614,23 @@ nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg)
 	bodyType = headerVal;
 	ExtractType( bodyType);
 	ExtractCharset( headerVal);
-	charSet = headerVal;
-	if (headerVal.Length()) {
-        asciiHeaderVal.AssignWithConversion(headerVal);
-		m_pMsgFields->SetCharacterSet(asciiHeaderVal.get());
+  // Use platform charset as default if the msg doesn't specify one
+  // (ie, no 'charset' param in the Content-Type: header). As the last
+  // resort we'll use the mail defaul charset.
+  if (headerVal.IsEmpty())
+  {
+    headerVal.AssignWithConversion(nsMsgI18NFileSystemCharset());
+    if (headerVal.IsEmpty())
+    { // last resort
+      nsXPIDLString defaultCharset;
+      nsCOMPtr<nsIPref> prefs(do_GetService(NS_PREF_CONTRACTID, &rv));
+      if (NS_SUCCEEDED(rv))
+        rv = prefs->GetLocalizedUnicharPref("mailnews.view_default_charset", getter_Copies(defaultCharset));
+      headerVal.Assign(defaultCharset.IsEmpty() ? NS_LITERAL_STRING("ISO-8859-1").get() : defaultCharset.get());
     }
+  }
+  m_pMsgFields->SetCharacterSet( NS_LossyConvertUCS2toASCII(headerVal).get() );
+  charSet = headerVal;
 	GetHeaderValue( m_pHeaders, m_headerLen, "CC:", headerVal);
 	if (headerVal.Length())
 		m_pMsgFields->SetCc( headerVal.get());
@@ -626,17 +651,27 @@ nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg)
 	// IMPORT_LOG0( "Outlook compose calling CreateAndSendMessage\n");
 	nsMsgAttachedFile *pAttach = GetLocalAttachments();
 
+  // Do body conversion here
+  nsString	uniBody;
+  ConvertSystemStringToUnicode( m_pBody, uniBody);
+
+  nsCString	body, theCharset;
+  theCharset.AssignWithConversion(charSet);
+
+  rv = nsMsgI18NConvertFromUnicode( theCharset, uniBody, body);
+  uniBody.Truncate();
+
 	rv = m_pSendProxy->CreateAndSendMessage(
                     nsnull,			                  // no editor shell
 										m_pIdentity,	                // dummy identity
 										m_pMsgFields,	                // message fields
 										PR_FALSE,		                  // digest = NO
 										PR_TRUE,		                  // dont_deliver = YES, make a file
-										nsIMsgSend::nsMsgDeliverNow,	// mode
+										mode,	                        // mode
 										nsnull,			                  // no message to replace
 										pMimeType,		                // body type
-										m_pBody,		                  // body pointer
-										m_bodyLen,		                // body length
+                    NS_FAILED(rv) ? m_pBody : body.get(),	// body pointer
+                    NS_FAILED(rv) ? m_bodyLen : body.Length(),	// body length
 										nsnull,			                  // remote attachment data
 										pAttach,		                  // local attachments
 										nsnull,			                  // related part
@@ -645,7 +680,7 @@ nsresult nsOutlookCompose::SendTheMessage( nsIFileSpec *pMsg)
 										m_pListener,		              // listener
                     nsnull);                      // password
 
-	
+
 	// IMPORT_LOG0( "Returned from CreateAndSendMessage\n");
 
 	if (pAttach)
@@ -868,27 +903,23 @@ nsresult nsOutlookCompose::CopyComposedMessage( nsCString& fromLine, nsIFileSpec
 	char	lastChar = 0;
 
 	if (NS_SUCCEEDED( rv)) {
-		rv = pDst->Write( copy.m_pBuffer + copy.m_writeOffset, copy.m_bytesInBuf - copy.m_writeOffset, &written);
+    rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 		if (copy.m_bytesInBuf)
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
-		copy.m_writeOffset = copy.m_bytesInBuf;
+    if (NS_SUCCEEDED(rv))
+      copy.m_writeOffset = copy.m_bytesInBuf;
 	}
 
 	while ((state.offset < state.size) && NS_SUCCEEDED( rv)) {
 		rv = FillMailBuffer( &state, copy);
 		if (NS_SUCCEEDED( rv)) {
-			rv = pDst->Write( copy.m_pBuffer, copy.m_bytesInBuf, &written);
+      rv = EscapeFromSpaceLine(pDst, copy.m_pBuffer + copy.m_writeOffset, copy.m_pBuffer+copy.m_bytesInBuf);
 			lastChar = copy.m_pBuffer[copy.m_bytesInBuf - 1];
-			if (NS_SUCCEEDED( rv)) {
-				if (written != copy.m_bytesInBuf) {
-					rv = NS_ERROR_FAILURE;
-					IMPORT_LOG0( "*** Error writing to destination mailbox\n");
-				}
-				else
-					copy.m_writeOffset = copy.m_bytesInBuf;
-			}
+			if (NS_SUCCEEDED( rv))
+		    copy.m_writeOffset = copy.m_bytesInBuf;
+	    else
+        IMPORT_LOG0( "*** Error writing to destination mailbox\n");
 		}
-
 	}
 
 	pSrc->CloseStream();
@@ -984,7 +1015,7 @@ nsresult nsOutlookCompose::WriteHeaders( nsIFileSpec *pDst, SimpleBufferTonyRCop
 	nsCString	val;
 	nsCString	replaceVal;
 	PRInt32		written;
-	nsresult	rv = NS_ERROR_FAILURE;
+	nsresult	rv = NS_OK; // it's ok if we don't have the first header on the predefined lists.
 	PRInt32		specialHeader;
 	PRBool		specials[kMaxSpecialHeaders];
 	int			i;
@@ -999,6 +1030,10 @@ nsresult nsOutlookCompose::WriteHeaders( nsIFileSpec *pDst, SimpleBufferTonyRCop
 			if ((specialHeader = IsSpecialHeader( header.get())) != -1) {
 				header.Append( ':');
 				GetHeaderValue( newHeaders.m_pBuffer, newHeaders.m_writeOffset - 1, header.get(), val, PR_FALSE);
+        // Bug 145150 - Turn "Content-Type: application/ms-tnef" into "Content-Type: text/plain"
+        //              so the body text can be displayed normally (instead of in an attachment).
+        if (!PL_strcasecmp(header.get(), "Content-Type:") && !PL_strcasecmp(val.get(), "application/ms-tnef"))
+          val.Assign("text/plain");
 				header.Truncate( header.Length() - 1);
 				specials[specialHeader] = PR_TRUE;
 			}
