@@ -37,11 +37,9 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-#define NS_IMPL_IDS
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetAlias.h"
 #include "nsIPlatformCharset.h"
-#undef NS_IMPL_IDS
 #include "nsICharsetConverterManager2.h"
 
 #include "nsCRT.h"
@@ -662,7 +660,7 @@ nsresult nsImportService::LoadModuleInfo( const char *pClsId, const char *pSuppo
 	rv = module->GetName( &pName);
 	if (NS_SUCCEEDED( rv)) {
 		theTitle = pName;
-		delete [] pName;
+                nsMemory::Free(pName);
 	}
 	else
 		theTitle.Assign(NS_LITERAL_STRING("Unknown"));
@@ -670,7 +668,7 @@ nsresult nsImportService::LoadModuleInfo( const char *pClsId, const char *pSuppo
 	rv = module->GetDescription( &pName);
 	if (NS_SUCCEEDED( rv)) {
 		theDescription = pName;
-		delete [] pName;
+                nsMemory::Free(pName);
 	}
 	else
 		theDescription.Assign(NS_LITERAL_STRING("Unknown description"));
@@ -758,7 +756,7 @@ void nsImportModuleList::ClearList( void)
 			m_pList[i] = nsnull;
 		}
 		m_count = 0;
-		nsMemory::Free( m_pList);
+		delete [] m_pList;
 		m_pList = nsnull;
 		m_alloc = 0;
 	}
@@ -771,14 +769,16 @@ void nsImportModuleList::AddModule( const nsCID& cid, const char *pSupports, con
 		m_alloc = 10;
 		m_pList = new ImportModuleDesc *[m_alloc];
 		m_count = 0;
-		nsCRT::memset( m_pList, 0, sizeof( ImportModuleDesc *) * m_alloc);
+		memset( m_pList, 0, sizeof( ImportModuleDesc *) * m_alloc);
 	}
 	
 	if (m_count == m_alloc) {
 		ImportModuleDesc **pList = new ImportModuleDesc *[m_alloc + 10];
-		nsCRT::memset( &(pList[m_alloc]), 0, sizeof( ImportModuleDesc *) * 10);
+		memset( &(pList[m_alloc]), 0, sizeof( ImportModuleDesc *) * 10);
 		memcpy( pList, m_pList, sizeof( ImportModuleDesc *) * m_alloc);
-		delete [] m_pList;
+    for(int i = 0; i < m_count; i++)
+      delete m_pList[i];
+    delete [] m_pList;
 		m_pList = pList;
 		m_alloc += 10;
 	}
