@@ -25,6 +25,8 @@
 #include "nsAbBaseCID.h"
 #include "nsIAbCard.h"
 #include "nsReadableUtils.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 
 static NS_DEFINE_CID(kAbCardPropertyCID, NS_ABCARDPROPERTY_CID);
 
@@ -37,7 +39,6 @@ static NS_DEFINE_CID(kAbCardPropertyCID, NS_ABCARDPROPERTY_CID);
 
 // ONLY included for the ldif code that was copied over from nsAddressBook.cpp
 // Maybe re-write that code and get rid of this? (isspace is what is needed)
-#include "xp_core.h"
 #include "plstr.h"
 #include "prmem.h"
 #include "prprf.h"
@@ -47,6 +48,8 @@ nsTextAddress::nsTextAddress()
 {
     m_database = nsnull;
     m_fieldMap = nsnull;
+    m_LFCount = 0;
+    m_CRCount = 0;
 }
 
 nsTextAddress::~nsTextAddress()
@@ -125,7 +128,7 @@ nsresult nsTextAddress::ImportAddresses( PRBool *pAbort, const PRUnichar *pName,
             *pProgress = (PRUint32)loc;
         rv = ReadRecord( pSrc, pLine, kTextAddressBufferSz, m_delim, &lineLen);
         if (NS_SUCCEEDED( rv)) {
-            rv = ProcessLine( pLine, nsCRT::strlen( pLine), errors);
+            rv = ProcessLine( pLine, strlen( pLine), errors);
             if (NS_FAILED( rv)) {
                 IMPORT_LOG0( "*** Error processing text record.\n");
             }
@@ -173,7 +176,7 @@ nsresult nsTextAddress::ReadRecord( nsIFileSpec *pSrc, char *pLine, PRInt32 buff
             rv = NS_ERROR_FAILURE;
         }
         else if (NS_SUCCEEDED( rv)) {
-            lineLen = nsCRT::strlen( pLine);
+            lineLen = strlen( pLine);
         }
     } while (NS_SUCCEEDED( rv) && !IsLineComplete( pLine, lineLen, delim));
 
@@ -439,7 +442,7 @@ nsresult nsTextAddress::DetermineDelim( nsIFileSpec *pSrc)
         if (wasTruncated)
             pLine[kTextAddressBufferSz - 1] = 0;
         if (NS_SUCCEEDED( rv)) {
-            lineLen = nsCRT::strlen( pLine);
+            lineLen = strlen( pLine);
             tabCount = CountFields( pLine, lineLen, 9);
             commaCount = CountFields( pLine, lineLen, ',');
             if (tabCount > commaCount)
@@ -584,7 +587,7 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
         if (wasTruncated)
             pLine[kTextAddressBufferSz - 1] = 0;
         if (NS_SUCCEEDED( rv)) {
-            lineLen = nsCRT::strlen( pLine);            
+            lineLen = strlen( pLine);            
             tabCount += CountFields( pLine, lineLen, 9);
             commaCount += CountFields( pLine, lineLen, ',');
             pChar = pLine;
@@ -674,6 +677,9 @@ nsresult nsTextAddress::IsLDIFFile( nsIFileSpec *pSrc, PRBool *pIsLDIF)
 #define IS_SPACE(VAL)                \
     (((((intn)(VAL)) & 0x7f) == ((intn)(VAL))) && isspace((intn)(VAL)) )
 
+// XXX TODO fix me
+// use the NSPR base64 library.  see plbase64.h
+// see bug #145367
 static unsigned char b642nib[0x80] = {
     0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
     0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
@@ -698,6 +704,7 @@ static unsigned char b642nib[0x80] = {
  * into components "type" and "value".  if a double colon separates type from
  * value, then value is encoded in base 64, and parse_line un-decodes it
  * (in place) before returning.
+ * in LDIF, non-ASCII data is treated as base64 encoded UTF-8 
  */
 
 nsresult nsTextAddress::str_parse_line(
@@ -845,43 +852,39 @@ char * nsTextAddress::str_getline( char **next )
  */
 nsresult nsTextAddress::GetLdifStringRecord(char* buf, PRInt32 len, PRInt32& stopPos)
 {
-    PRInt32 LFCount = 0;
-    PRInt32 CRCount = 0;
-
     for (; stopPos < len; stopPos++) 
     {
         char c = buf[stopPos];
 
         if (c == 0xA)
         {
-            LFCount++;
+            m_LFCount++;
         }
         else if (c == 0xD)
         {
-            CRCount++;
+            m_CRCount++;
         }
         else if ( c != 0xA && c != 0xD)
         {
-            if (LFCount == 0 && CRCount == 0)
+            if (m_LFCount == 0 && m_CRCount == 0)
                  m_ldifLine.Append(c);
-            else if (( LFCount > 1) || ( CRCount > 2 && LFCount ) ||
-                ( !LFCount && CRCount > 1 ))
+            else if (( m_LFCount > 1) || ( m_CRCount > 2 && m_LFCount ) ||
+                ( !m_LFCount && m_CRCount > 1 ))
             {
                 return NS_OK;
             }
-            else if ((LFCount == 1 || CRCount == 1))
+            else if ((m_LFCount == 1 || m_CRCount == 1))
             {
-                 m_ldifLine.Append('\n');
-                 m_ldifLine.Append(c);
-                LFCount = 0;
-                CRCount = 0;
+                m_ldifLine.Append('\n');
+                m_ldifLine.Append(c);
+                m_LFCount = 0;
+                m_CRCount = 0;
             }
         }
     }
-    if ((LFCount == 1 || CRCount == 1) && stopPos == len)
-        m_ldifLine.Append('\n');
-    if ((stopPos == len) && (LFCount > 1) || (CRCount > 2 && LFCount) ||
-        (!LFCount && CRCount > 1))
+
+    if ((stopPos == len) && (m_LFCount > 1) || (m_CRCount > 2 && m_LFCount) ||
+        (!m_LFCount && m_CRCount > 1))
         return NS_OK;
     else
         return NS_ERROR_FAILURE;
@@ -894,7 +897,8 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
     PRInt32 startPos = 0;
     PRInt32 len = 0;
     PRBool bEof = PR_FALSE;
-    nsVoidArray listPosArray;
+    nsVoidArray listPosArray;   // where each list/group starts in ldif file
+    nsVoidArray listSizeArray;  // size of the list/group info
     PRInt32 savedStartPos = 0;
     PRInt32 filePos = 0;
 
@@ -906,14 +910,14 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
 
             while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
             {
-                if (m_ldifLine.Find("groupOfNames") == -1)
+                 if (m_ldifLine.Find("groupOfNames") == -1)
                     AddLdifRowToDatabase(PR_FALSE);
                 else
                 {
                     //keep file position for mailing list
                     listPosArray.AppendElement((void*)savedStartPos);
-                    if (m_ldifLine.Length() > 0)
-                        m_ldifLine.Truncate();
+                    listSizeArray.AppendElement((void*)(filePos + startPos-savedStartPos));
+                    ClearLdifRecordBuffer();
                 }
                 savedStartPos = filePos + startPos;
             }
@@ -924,19 +928,27 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
     //last row
     if (m_ldifLine.Length() > 0 && m_ldifLine.Find("groupOfNames") == -1)
         AddLdifRowToDatabase(PR_FALSE); 
+
     // mail Lists
-    PRInt32 i;
+    PRInt32 i, pos, size;
     PRInt32 listTotal = listPosArray.Count();
+    char *listBuf;
+    ClearLdifRecordBuffer();  // make sure the buffer is clean
     for (i = 0; i < listTotal; i++)
     {
-        PRInt32 pos = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        pos  = NS_PTR_TO_INT32(listPosArray.ElementAt(i));
+        size = NS_PTR_TO_INT32(listSizeArray.ElementAt(i));
         if (NS_SUCCEEDED(pSrc->Seek(pos)))
         {
-            if (NS_SUCCEEDED(pSrc->Read(&pBuf, (PRInt32)sizeof(buf), &len)) && len > 0)
+            // Allocate enough space for the lists/groups as the size varies.
+            listBuf = (char *) PR_Malloc(size);
+            if (!listBuf)
+              continue;
+            if (NS_SUCCEEDED(pSrc->Read(&listBuf, size, &len)) && len > 0)
             {
                 startPos = 0;
 
-                while (NS_SUCCEEDED(GetLdifStringRecord(buf, len, startPos)))
+                while (NS_SUCCEEDED(GetLdifStringRecord(listBuf, len, startPos)))
                 {
                     if (m_ldifLine.Find("groupOfNames") != -1)
                     {
@@ -946,6 +958,7 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
                     }
                 }
             }
+            PR_FREEIF(listBuf);
         }
     }
     return NS_OK;
@@ -953,13 +966,21 @@ nsresult nsTextAddress::ParseLdifFile( nsIFileSpec *pSrc, PRUint32 *pProgress)
 
 void nsTextAddress::AddLdifRowToDatabase(PRBool bIsList)
 {
-    nsIMdbRow* newRow = nsnull;
+    // If no data to process then reset CR/LF counters and return.
+    if (m_ldifLine.IsEmpty())
+    {
+      m_LFCount = 0;
+      m_CRCount = 0;
+      return;
+    }
+
+    nsCOMPtr <nsIMdbRow> newRow;
     if (m_database)
     {
         if (bIsList)
-            m_database->GetNewListRow(&newRow); 
+            m_database->GetNewListRow(getter_AddRefs(newRow)); 
         else
-            m_database->GetNewRow(&newRow); 
+            m_database->GetNewRow(getter_AddRefs(newRow)); 
 
         if (!newRow)
             return;
@@ -981,14 +1002,24 @@ void nsTextAddress::AddLdifRowToDatabase(PRBool bIsList)
         else
             continue; // parse error: continue with next loop iteration
     }
-    delete [] saveCursor;
+    nsMemory::Free(saveCursor);
     m_database->AddCardRowToDB(newRow);    
 
     if (bIsList)
         m_database->AddListDirNode(newRow);
         
-    if (m_ldifLine.Length() > 0)
-        m_ldifLine.Truncate();
+    // Clear buffer for next record
+    ClearLdifRecordBuffer();
+}
+
+void nsTextAddress::ClearLdifRecordBuffer()
+{
+  if (m_ldifLine.Length() > 0)
+  {
+      m_ldifLine.Truncate();
+      m_LFCount = 0;
+      m_CRCount = 0;
+  }
 }
 
 // We have two copies of this function in the code, one here for import and 
@@ -1000,7 +1031,8 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
   nsCAutoString colType(typeSlot);
   nsCAutoString column(valueSlot);
 
-  // 4.x exports attributes like "givenname", mozilla does "givenName" to be compliant with RFC 2798
+  // 4.x exports attributes like "givenname", 
+  // mozilla does "givenName" to be compliant with RFC 2798
   ToLowerCase(colType);
 
     mdb_u1 firstByte = (mdb_u1)(colType.get())[0];
@@ -1084,7 +1116,7 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
         else if (colType.Equals("homepostaladdress") )
     m_database->AddHomeAddress(newRow, column.get());
       else if (colType.Equals("homeurl") )
-        m_database->AddWebPage1(newRow, column.get());
+        m_database->AddWebPage2(newRow, column.get());
       break; // 'h'
 
     case 'i':
@@ -1106,10 +1138,51 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
       if (colType.Equals("mail") )
         m_database->AddPrimaryEmail(newRow, column.get());
       else if ( colType.Equals("mobile") )
-    m_database->AddCellularNumber(newRow, column.get());
+        m_database->AddCellularNumber(newRow, column.get());
       else if (colType.Equals("member") && bIsList )
         m_database->AddLdifListMember(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "secondemail"))
+        m_database->Add2ndEmail(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "postaladdress2"))
+        m_database->AddWorkAddress2(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "homepostaladdress2"))
+        m_database->AddHomeAddress2(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "homelocalityname"))
+        m_database->AddHomeCity(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "homestate"))
+        m_database->AddHomeState(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "homepostalcode"))
+        m_database->AddHomeZipCode(newRow, column.get());
+      else if (colType.Equals(MOZ_AB_LDIF_PREFIX "homecountryname"))
+        m_database->AddHomeCountry(newRow, column.get());
+      else {
+        // handle all the optional LDIF attributes
+        // like mozillaaimscreenname (mozillaAimScreenName)
+        //
+        // note, all optional LDIF attributes begin with the "mozilla" prefix
+        // see bug #119360.  that's why the import code to handle them
+        // is under the "m" clause.
+        nsresult rv;
+        nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get pref service");
+        if (NS_FAILED(rv))
+          break;
 
+        nsCOMPtr<nsIPrefBranch> prefBranch;
+        rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get pref branch");
+        if (NS_FAILED(rv))
+          break;
+
+        nsCAutoString prefName;
+        prefName = NS_LITERAL_CSTRING("mail.addr_book.import_ldif_map.") + colType;
+
+        nsXPIDLCString columnName;
+        rv = prefBranch->GetCharPref(prefName.get(), getter_Copies(columnName));
+        if (NS_SUCCEEDED(rv) && columnName.Length()) {
+          m_database->AddRowValue(newRow, columnName, NS_ConvertUTF8toUCS2(column));
+        }
+      }
       break; // 'm'
 
     case 'n':
@@ -1186,7 +1259,7 @@ void nsTextAddress::AddLdifColToDatabase(nsIMdbRow* newRow, char* typeSlot, char
 
     case 'w':
       if ( colType.Equals("workurl") )
-        m_database->AddWebPage2(newRow, column.get());
+        m_database->AddWebPage1(newRow, column.get());
 
       break; // 'w'
 
