@@ -44,6 +44,7 @@
 #include "nsMsgMessageFlags.h"
 #include "nsIDBFolderInfo.h"
 #include "nsIInputStream.h"
+#include "nsILocalFile.h"
 #include "nsMsgLocalFolderHdrs.h"
 #include "nsMsgBaseCID.h"
 #include "nsMsgDBCID.h"
@@ -69,6 +70,10 @@
 #include "nsMsgUtils.h"
 #include "prprf.h"
 #include "nsEscape.h"
+#include "nsIMimeHeaders.h"
+
+#include "nsIMsgMdnGenerator.h"
+#include "nsMsgSearchCore.h"
 
 static NS_DEFINE_CID(kCMailDB, NS_MAILDB_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
@@ -80,141 +85,124 @@ NS_IMPL_ISUPPORTS_INHERITED2(nsMsgMailboxParser, nsParseMailMessageState, nsIStr
 
 // Whenever data arrives from the connection, core netlib notifices the protocol by calling
 // OnDataAvailable. We then read and process the incoming data from the input stream. 
-NS_IMETHODIMP nsMsgMailboxParser::OnDataAvailable(nsIRequest *request, nsISupports *ctxt, nsIInputStream *aIStream, PRUint32 sourceOffset, 
-												  PRUint32 aLength)
+NS_IMETHODIMP nsMsgMailboxParser::OnDataAvailable(nsIRequest *request, nsISupports *ctxt, nsIInputStream *aIStream, PRUint32 sourceOffset, PRUint32 aLength)
 {
-	// right now, this really just means turn around and process the url
-	nsresult rv = NS_OK;
-	nsCOMPtr<nsIURI> url = do_QueryInterface(ctxt, &rv);
-	if (NS_SUCCEEDED(rv))
-		rv = ProcessMailboxInputStream(url, aIStream, aLength);
-	return rv;
+    // right now, this really just means turn around and process the url
+    nsresult rv = NS_OK;
+    nsCOMPtr<nsIURI> url = do_QueryInterface(ctxt, &rv);
+    if (NS_SUCCEEDED(rv))
+        rv = ProcessMailboxInputStream(url, aIStream, aLength);
+    return rv;
 }
 
 NS_IMETHODIMP nsMsgMailboxParser::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
-	nsTime currentTime;
-	m_startTime = currentTime;
+    nsTime currentTime;
+    m_startTime = currentTime;
 
 
-	// extract the appropriate event sinks from the url and initialize them in our protocol data
-	// the URL should be queried for a nsIMailboxURL. If it doesn't support a mailbox URL interface then
-	// we have an error.
-	nsresult rv = NS_OK;
+    // extract the appropriate event sinks from the url and initialize them in our protocol data
+    // the URL should be queried for a nsIMailboxURL. If it doesn't support a mailbox URL interface then
+    // we have an error.
+    nsresult rv = NS_OK;
 
     nsCOMPtr<nsIIOService> ioServ(do_GetService(kIOServiceCID, &rv));
 
-	nsCOMPtr<nsIMailboxUrl> runningUrl = do_QueryInterface(ctxt, &rv);
+    nsCOMPtr<nsIMailboxUrl> runningUrl = do_QueryInterface(ctxt, &rv);
 
-	nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(ctxt);
+    nsCOMPtr<nsIMsgMailNewsUrl> url = do_QueryInterface(ctxt);
+    nsCOMPtr<nsIMsgFolder> folder = do_QueryReferent(m_folder);
 
-	if (NS_SUCCEEDED(rv) && runningUrl)
-	{
-		url->GetStatusFeedback(getter_AddRefs(m_statusFeedback));
+    if (NS_SUCCEEDED(rv) && runningUrl && folder)
+    {
+        url->GetStatusFeedback(getter_AddRefs(m_statusFeedback));
 
-		// okay, now fill in our event sinks...Note that each getter ref counts before
-		// it returns the interface to us...we'll release when we are done
-        nsXPIDLCString fileName;
-        nsXPIDLCString folderName;
-        rv = url->GetFilePath(getter_Copies(fileName));
-		url->GetFileName(getter_Copies(folderName));
-        
-        nsXPIDLCString tempfolder;
-        rv = nsStdUnescape((char*)folderName.get(), getter_Copies(tempfolder));
+        // okay, now fill in our event sinks...Note that each getter ref counts before
+        // it returns the interface to us...we'll release when we are done
 
-        // convert from OS native charset to unicode
-        rv = ConvertToUnicode(nsMsgI18NFileSystemCharset(), tempfolder, m_folderName);
-        if (NS_FAILED(rv))
-          m_folderName.AssignWithConversion(tempfolder);
+        folder->GetName(getter_Copies(m_folderName));
 
-		if (fileName)
-		{
-            char* result = nsnull;
-            rv = nsStdUnescape((char*)fileName.get(), &result);
-			nsFilePath dbPath(result);
-            CRTFREEIF(result);
-			nsFileSpec dbName(dbPath);
+        nsCOMPtr<nsIFileSpec> path;
+        folder->GetPath(getter_AddRefs(path));
+       
+        if (path)
+        {
+            nsFileSpec dbName;
+            path->GetFileSpec(&dbName);
+            // the size of the mailbox file is our total base line for measuring progress
+            m_graph_progress_total = dbName.GetFileSize();
+            UpdateStatusText(LOCAL_STATUS_SELECTING_MAILBOX);
 
-			// the size of the mailbox file is our total base line for measuring progress
-			m_graph_progress_total = dbName.GetFileSize();
-			UpdateStatusText(LOCAL_STATUS_SELECTING_MAILBOX);
-			
-			nsCOMPtr<nsIMsgDatabase> mailDB;
-			rv = nsComponentManager::CreateInstance(kCMailDB, nsnull, NS_GET_IID(nsIMsgDatabase), (void **) getter_AddRefs(mailDB));
-			if (NS_SUCCEEDED(rv) && mailDB)
-			{
-				nsCOMPtr <nsIFileSpec> dbFileSpec;
-				NS_NewFileSpecWithSpec(dbName, getter_AddRefs(dbFileSpec));
-				rv = mailDB->Open(dbFileSpec, PR_TRUE, PR_TRUE, (nsIMsgDatabase **) getter_AddRefs(m_mailDB));
-                                if (m_mailDB)
-                                  m_mailDB->AddListener(this);
-			}
-			NS_ASSERTION(m_mailDB, "failed to open mail db parsing folder");
-#ifdef DEBUG_mscott
-			printf("url file = %s\n", (const char *)fileName);
-#endif
-		}
- 	}
-
-	// need to get the mailbox name out of the url and call SetMailboxName with it.
-	// then, we need to open the mail db for this parser.
-	return rv;
+            nsCOMPtr<nsIMsgDatabase> mailDB;
+            rv = nsComponentManager::CreateInstance(kCMailDB, nsnull, NS_GET_IID(nsIMsgDatabase), (void **) getter_AddRefs(mailDB));
+            if (NS_SUCCEEDED(rv) && mailDB)
+            {
+                //Use OpenFolderDB to always open the db so that db's m_folder is set correctly.
+                rv = mailDB->OpenFolderDB(folder, PR_TRUE, PR_TRUE, (nsIMsgDatabase **) getter_AddRefs(m_mailDB));
+                if (m_mailDB)
+                    m_mailDB->AddListener(this);
+            }
+            NS_ASSERTION(m_mailDB, "failed to open mail db parsing folder");
+        }
+    }
 
+    // need to get the mailbox name out of the url and call SetMailboxName with it.
+    // then, we need to open the mail db for this parser.
+    return rv;
 }
 
 // stop binding is a "notification" informing us that the stream associated with aURL is going away. 
 NS_IMETHODIMP nsMsgMailboxParser::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult aStatus)
 {
-	DoneParsingFolder(aStatus);
-	// what can we do? we can close the stream?
-	m_urlInProgress = PR_FALSE;  // don't close the connection...we may be re-using it.
+    DoneParsingFolder(aStatus);
+    // what can we do? we can close the stream?
+    m_urlInProgress = PR_FALSE;  // don't close the connection...we may be re-using it.
 
-        if (m_mailDB)
-          m_mailDB->RemoveListener(this);
-	// and we want to mark ourselves for deletion or some how inform our protocol manager that we are 
-	// available for another url if there is one....
+    if (m_mailDB)
+        m_mailDB->RemoveListener(this);
+    // and we want to mark ourselves for deletion or some how inform our protocol manager that we are 
+    // available for another url if there is one....
 #ifdef DEBUG1
-	// let's dump out the contents of our db, if possible.
-	if (m_mailDB)
-	{
-		nsMsgKeyArray	keys;
-		nsCAutoString	author;
-		nsCAutoString	subject;
+    // let's dump out the contents of our db, if possible.
+    if (m_mailDB)
+    {
+        nsMsgKeyArray	keys;
+        nsCAutoString	author;
+        nsCAutoString	subject;
 
-//		m_mailDB->PrePopulate();
-		m_mailDB->ListAllKeys(keys);
+    //	m_mailDB->PrePopulate();
+        m_mailDB->ListAllKeys(keys);
         PRUint32 size = keys.GetSize();
-		for (PRUint32 keyindex = 0; keyindex < size; keyindex++)
-		{
-			nsCOMPtr<nsIMsgDBHdr> msgHdr;
-			nsresult ret =
+        for (PRUint32 keyindex = 0; keyindex < size; keyindex++)
+        {
+            nsCOMPtr<nsIMsgDBHdr> msgHdr;
+            nsresult ret =
                 m_mailDB->GetMsgHdrForKey(keys[keyindex],
                                           getter_AddRefs(msgHdr));
-			if (NS_SUCCEEDED(ret) && msgHdr)
-			{
-				nsMsgKey key;
+            if (NS_SUCCEEDED(ret) && msgHdr)
+            {
+                nsMsgKey key;
 
-				msgHdr->GetMessageKey(&key);
-				msgHdr->GetAuthor(&author);
-				msgHdr->GetSubject(&subject);
+                msgHdr->GetMessageKey(&key);
+                msgHdr->GetAuthor(&author);
+                msgHdr->GetSubject(&subject);
 #ifdef DEBUG_bienvenu
-				// leak nsString return values...
-				printf("hdr key = %d, author = %s subject = %s\n", key, author.get(), subject.get());
+                // leak nsString return values...
+                printf("hdr key = %d, author = %s subject = %s\n", key, author.get(), subject.get());
 #endif
-			}
-		}
-		m_mailDB->Close(PR_TRUE);
-	}
+            }
+        }
+        m_mailDB->Close(PR_TRUE);
+    }
 #endif
 
 
-	// be sure to clear any status text and progress info..
-	m_graph_progress_received = 0;
-	UpdateProgressPercent();
-	UpdateStatusText(LOCAL_STATUS_DOCUMENT_DONE);
-
-	return NS_OK;
+    // be sure to clear any status text and progress info..
+    m_graph_progress_received = 0;
+    UpdateProgressPercent();
+    UpdateStatusText(LOCAL_STATUS_DOCUMENT_DONE);
 
+    return NS_OK;
 }
 
 
@@ -273,12 +261,20 @@ nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 	m_updateAsWeGo = PR_TRUE;
 	m_ignoreNonMailFolder = PR_FALSE;
 	m_isRealMailFolder = PR_TRUE;
+  m_folder=nsnull;
+
 }
 
 nsMsgMailboxParser::~nsMsgMailboxParser()
 {
 }
 
+void nsMsgMailboxParser::SetFolder(nsIMsgFolder *aFolder)
+{
+  if (aFolder)
+    m_folder = getter_AddRefs(NS_GetWeakReference(aFolder));
+}
+
 void nsMsgMailboxParser::UpdateStatusText (PRUint32 stringID)
 {
 	if (m_statusFeedback)
@@ -638,6 +634,14 @@ NS_IMETHODIMP nsParseMailMessageState::SetMailDB(nsIMsgDatabase *mailDB)
 	return NS_OK;
 }
 
+NS_IMETHODIMP nsParseMailMessageState::SetDBFolderStream(nsIOFileStream *fileStream)
+{
+  NS_ASSERTION(m_mailDB, "m_mailDB is not set");
+  if (m_mailDB)
+    m_mailDB->SetFolderStream(fileStream);
+  return NS_OK;
+}
+
 /* #define STRICT_ENVELOPE */
 
 PRBool
@@ -663,7 +667,7 @@ nsParseMailMessageState::IsEnvelopeLine(const char *buf, PRInt32 buf_size)
 
   if (buf_size < 29) return PR_FALSE;
   if (*buf != 'F') return PR_FALSE;
-  if (nsCRT::strncmp(buf, "From ", 5)) return PR_FALSE;
+  if (strncmp(buf, "From ", 5)) return PR_FALSE;
 
   end = buf + buf_size;
   date = buf + 5;
@@ -728,7 +732,7 @@ nsParseMailMessageState::IsEnvelopeLine(const char *buf, PRInt32 buf_size)
 
   if (buf_size < 5) return PR_FALSE;
   if (*buf != 'F') return PR_FALSE;
-  if (nsCRT::strncmp(buf, "From ", 5)) return PR_FALSE;
+  if (strncmp(buf, "From ", 5)) return PR_FALSE;
 
 #endif /* !STRICT_ENVELOPE */
 
@@ -784,7 +788,7 @@ void nsParseMailMessageState::GetAggregateHeader (nsVoidArray &list, struct mess
 	{
 		header = (struct message_header*) list.ElementAt(i);
 		length += (header->length + 1); //+ for ","
-		NS_ASSERTION(header->length == (PRInt32)nsCRT::strlen(header->value), "header corrupted");
+		NS_ASSERTION(header->length == (PRInt32)strlen(header->value), "header corrupted");
 	}
 
 	if (length > 0)
@@ -1040,7 +1044,7 @@ int nsParseMailMessageState::InternSubject (struct message_header *header)
 		return 0;
 	}
 
-	NS_ASSERTION (header->length == (short) nsCRT::strlen(header->value), "subject corrupt while parsing message");
+	NS_ASSERTION (header->length == (short) strlen(header->value), "subject corrupt while parsing message");
 
 	key = (char *) header->value;  /* #### const evilness */
 
@@ -1055,7 +1059,8 @@ int nsParseMailMessageState::InternSubject (struct message_header *header)
 	 we just parsed the subject header anyway, we expect that parsing
 	 to be smartest.  (After all, what if someone just went in and
 	 edited the subject line by hand?) */
-	if (NS_MsgStripRE((const char **) &key, &L))
+  nsXPIDLCString modifiedSubject;
+	if (NS_MsgStripRE((const char **) &key, &L, getter_Copies(modifiedSubject)))
     flags |= MSG_FLAG_HAS_RE;
 	else
     flags &= ~MSG_FLAG_HAS_RE;
@@ -1065,11 +1070,12 @@ int nsParseMailMessageState::InternSubject (struct message_header *header)
 
 	// Condense the subject text into as few MIME-2 encoded words as possible.
 #ifdef WE_CONDENSE_MIME_STRINGS
-	char *condensedKey = msg_condense_mime2_string(key);
+  char *condensedKey = msg_condense_mime2_string(modifiedSubject.IsEmpty() ? key : modifiedSubject.get());
 #else
 	char *condensedKey = nsnull;
 #endif
-	m_newMsgHdr->SetSubject(condensedKey ? condensedKey : key);
+  m_newMsgHdr->SetSubject(condensedKey ? condensedKey : 
+                          (modifiedSubject.IsEmpty() ? key : modifiedSubject.get()));
 	PR_FREEIF(condensedKey);
 
 	return 0;
@@ -1087,7 +1093,7 @@ nsresult nsParseMailMessageState::InternRfc822 (struct message_header *header,
 	if (!header || header->length == 0)
 		return NS_OK;
 
-	NS_ASSERTION (header->length == (short) nsCRT::strlen (header->value), "invalid message_header");
+	NS_ASSERTION (header->length == (short) strlen (header->value), "invalid message_header");
 	NS_ASSERTION (ret_name != nsnull, "null ret_name");
 
 	if (m_HeaderAddressParser)
@@ -1202,7 +1208,8 @@ int nsParseMailMessageState::FinalizeHeaders()
       
       flags &= ~MSG_FLAG_OFFLINE; // don't keep MSG_FLAG_OFFLINE for local msgs
       if (mdn_dnt && !(origFlags & MSG_FLAG_READ) &&
-        !(origFlags & MSG_FLAG_MDN_REPORT_SENT))
+          !(origFlags & MSG_FLAG_MDN_REPORT_SENT) &&
+          !(flags & MSG_FLAG_MDN_REPORT_SENT))
         flags |= MSG_FLAG_MDN_REPORT_NEEDED;
       
       m_newMsgHdr->SetFlags(flags);
@@ -1210,8 +1217,11 @@ int nsParseMailMessageState::FinalizeHeaders()
         m_newMsgHdr->SetPriority(priorityFlags);
   
       // convert the flag values (0xE000000) to label values (0-5)
-      labelFlags = ((flags & MSG_FLAG_LABELS) >> 25);
-      m_newMsgHdr->SetLabel(labelFlags);
+      if (mozstatus2) // only do this if we have a mozstatus2 header
+      {
+        labelFlags = ((flags & MSG_FLAG_LABELS) >> 25);
+        m_newMsgHdr->SetLabel(labelFlags);
+      }
       if (delta < 0xffff) 
       {		/* Only use if fits in 16 bits. */
         m_newMsgHdr->SetStatusOffset((PRUint16) delta);
@@ -1233,12 +1243,14 @@ int nsParseMailMessageState::FinalizeHeaders()
       group in the summary list, and only being able to sort on the first
         group rather than the whole list.  It's worth it. */
         char * ch;
-        NS_ASSERTION (recipient->length == (PRUint16) nsCRT::strlen(recipient->value), "invalid recipient");
+        NS_ASSERTION (recipient->length == (PRUint16) strlen(recipient->value), "invalid recipient");
         ch = PL_strchr(recipient->value, ',');
         if (ch)
         {
-          *ch = 0;
-          recipient->length = nsCRT::strlen(recipient->value);
+          /* generate a new string that terminates before the , */
+          nsCAutoString firstGroup;
+          firstGroup.Assign(recipient->value, ch - recipient->value);
+          m_newMsgHdr->SetRecipients(firstGroup.get());
         }
         m_newMsgHdr->SetRecipients(recipient->value);
       }
@@ -1300,18 +1312,23 @@ int nsParseMailMessageState::FinalizeHeaders()
             md5_bin[8], md5_bin[9], md5_bin[10],md5_bin[11],
             md5_bin[12],md5_bin[13],md5_bin[14],md5_bin[15]);
           md5_header.value = md5_data;
-          md5_header.length = nsCRT::strlen(md5_data);
+          md5_header.length = strlen(md5_data);
           id = &md5_header;
         }
         
         /* Take off <> around message ID. */
         if (id->value[0] == '<')
           id->value++, id->length--;
-        if (id->value[id->length-1] == '>')
-          ((char *) id->value) [id->length-1] = 0, id->length--; /* #### const */
-        
-        m_newMsgHdr->SetMessageId(id->value);
-        
+
+        if (id->value[id->length-1] == '>') {
+          /* generate a new null-terminated string without the final > */
+          nsCAutoString rawMsgId;
+          rawMsgId.Assign(id->value, id->length - 1);
+          m_newMsgHdr->SetMessageId(rawMsgId.get());
+        } else {
+          m_newMsgHdr->SetMessageId(id->value);
+        }
+
         if (!mozstatus && statush)
         {
           /* Parse a little bit of the Berkeley Mail status header. */
@@ -1368,8 +1385,15 @@ int nsParseMailMessageState::FinalizeHeaders()
                 end++;
               if (*charset)
               {
-                *end = '\0';
-                m_newMsgHdr->SetCharset(charset);
+                if (*end != '\0') {
+                  // if we're not at the very end of the line, we need
+                  // to generate a new string without the trailing crud
+                  nsCAutoString rawCharSet;
+                  rawCharSet.Assign(charset, end - charset);
+                  m_newMsgHdr->SetCharset(rawCharSet.get());
+                } else {
+                  m_newMsgHdr->SetCharset(charset);
+                }
               }
             }
           }
@@ -1407,7 +1431,7 @@ nsParseNewMailState::nsParseNewMailState()
 NS_IMPL_ISUPPORTS_INHERITED1(nsParseNewMailState, nsMsgMailboxParser, nsIMsgFilterHitNotify)
 
 nsresult
-nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, nsFileSpec &folder, nsIOFileStream *inboxFileStream)
+nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, nsFileSpec &folder, nsIOFileStream *inboxFileStream, nsIMsgWindow *aMsgWindow)
 {
     nsresult rv;
 	m_position = folder.GetFileSize();
@@ -1434,59 +1458,12 @@ nsParseNewMailState::Init(nsIFolder *rootFolder, nsIMsgFolder *downloadFolder, n
     nsCOMPtr<nsIMsgIncomingServer> server;
     rv = rootMsgFolder->GetServer(getter_AddRefs(server));
     if (NS_SUCCEEDED(rv))
-        rv = server->GetFilterList(getter_AddRefs(m_filterList));
-
+      rv = server->GetFilterList(aMsgWindow, getter_AddRefs(m_filterList));
     m_logFile = nsnull;
-#ifdef DOING_MDN
+   
 	if (m_filterList)
-	{
-		const char *folderName = nsnull;
-		PRInt32 int_pref = 0;
-		PREF_GetIntPref("mail.incorporate.return_receipt", &int_pref);
-		if (int_pref == 1)
-		{
-			nsIMsgFolder *folderInfo = nsnull;
-			int status = 0;
-			char *defaultFolderName =
-				msg_MagicFolderName(master->GetPrefs(),
-									MSG_FOLDER_FLAG_SENTMAIL, &status); 
-			if (defaultFolderName)
-			{
-				folderInfo = master->FindMailFolder(defaultFolderName, PR_FALSE);
-				if (folderInfo && folderInfo->GetMailFolderInfo())
-					folderName = folderInfo->GetMailFolderInfo()->GetPathname();
-				XP_FREE(defaultFolderName);
-			}
-		}
-		if (folderName)
-		{
-			MSG_Filter *newFilter = new MSG_Filter(filterInboxRule, "receipt");
-			if (newFilter)
-			{
-				MSG_Rule *rule = nsnull;
-				MSG_SearchValue value;
-				newFilter->SetDescription("incorporate mdn report");
-				newFilter->SetEnabled(PR_TRUE);
-				newFilter->GetRule(&rule);
-				newFilter->SetFilterList(m_filterList);
-				value.attribute = attribOtherHeader;
-				value.u.string = "multipart/report";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_TRUE, "Content-Type");
-				value.u.string = "disposition-notification";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_TRUE, "Content-Type");
-#if 0
-				value.u.string = "delivery-status";
-				rule->AddTerm(attribOtherHeader, opContains,
-							  &value, PR_FALSE, "Content-Type");
-#endif
-				rule->SetAction(nsMsgFilterActionMoveToFolder, (void*)folderName);
-				m_filterList->InsertFilterAt(0, newFilter);
-			}
-		}
-	}
-#endif // DOING_MDN
+      rv = server->ConfigureTemporaryReturnReceiptsFilter(m_filterList);
+
 	m_usingTempDB = PR_FALSE;
 	m_tmpdbName = nsnull;
 	m_disableFilters = PR_FALSE;
@@ -1592,13 +1569,14 @@ nsOutputFileStream * nsParseNewMailState::GetLogFile ()
     nsCOMPtr<nsIFile> logDir;
     NS_GetSpecialDirectory(NS_APP_MAIL_50_DIR, getter_AddRefs(logDir));
 
-    nsXPIDLCString pathBuf;
-    logDir->Append("filter.log");
-    nsresult rv = logDir->GetPath(getter_Copies(pathBuf));
+    logDir->AppendNative(NS_LITERAL_CSTRING("filter.log"));
+
+    nsCAutoString pathBuf;
+    nsresult rv = logDir->GetNativePath(pathBuf);
     if (NS_FAILED(rv)) return nsnull;
-    nsCOMPtr<nsIFileSpec> outSpec;
+
+		nsFileSpec logFile(pathBuf.get());
 
-		nsFileSpec logFile(pathBuf);
 		m_logFile = new nsOutputFileStream(logFile, PR_WRONLY | PR_CREATE_FILE, 00600);
 	}
 	return m_logFile;
@@ -1705,52 +1683,12 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 			{
 				msgHdr->GetFlags(&msgFlags);
 
-				if (msgFlags & MSG_FLAG_MDN_REPORT_NEEDED &&
-					!isRead)
-				{
-					struct message_header to;
-					struct message_header cc;
-					GetAggregateHeader (m_toList, &to);
-					GetAggregateHeader (m_ccList, &cc);
-					msgHdr->SetFlags(msgFlags & ~MSG_FLAG_MDN_REPORT_NEEDED);
-					msgHdr->OrFlags(MSG_FLAG_MDN_REPORT_SENT, &newFlags);
-
-#if DOING_MDN	// leave it to the user action
-					if (actionType == nsMsgFilterActionDelete)
-					{
-						MSG_ProcessMdnNeededState processMdnNeeded
-							(MSG_ProcessMdnNeededState::eDeleted,
-							 m_pane, m_folder, msgHdr->GetMessageKey(),
-							 &state->m_return_path, &state->m_mdn_dnt, 
-							 &to, &cc, &state->m_subject, 
-							 &state->m_date, &state->m_mdn_original_recipient,
-							 &state->m_message_id, state->m_headers, 
-							 (PRInt32) state->m_headers_fp, PR_TRUE);
-					}
-					else
-					{
-						MSG_ProcessMdnNeededState processMdnNeeded
-							(MSG_ProcessMdnNeededState::eProcessed,
-							 m_pane, m_folder, msgHdr->GetMessageKey(),
-							 &state->m_return_path, &state->m_mdn_dnt, 
-							 &to, &cc, &state->m_subject, 
-							 &state->m_date, &state->m_mdn_original_recipient,
-							 &state->m_message_id, state->m_headers, 
-							 (PRInt32) state->m_headers_fp, PR_TRUE);
-					}
-#endif
-					char *tmp = (char*) to.value;
-					PR_FREEIF(tmp);
-					tmp = (char*) cc.value;
-					PR_FREEIF(tmp);
-				}
-				nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
-				if (NS_SUCCEEDED(err))
-					m_msgMovedByFilter = PR_TRUE;
-
-			}
+        nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
+        if (NS_SUCCEEDED(err))
+          m_msgMovedByFilter = PR_TRUE;
+      }
       *applyMore = PR_FALSE; 
-			break;
+      break;
 		case nsMsgFilterAction::MarkRead:
 			MarkFilteredMessageRead(msgHdr);
 			break;
@@ -1778,7 +1716,9 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
             {
                 nsMsgLabelValue filterLabel;
                 filter->GetActionLabel(&filterLabel);
-                msgHdr->SetLabel(filterLabel);
+                nsMsgKey msgKey;
+                msgHdr->GetMessageKey(&msgKey);
+                m_mailDB->SetLabel(msgKey, filterLabel);
             }
 			break;
 		default:
@@ -2284,7 +2224,7 @@ PRInt32	ParseOutgoingMessage::ParseFolderLine(const char *line, PRUint32 lineLen
 		m_lastBodyLineEmpty = (m_state == nsIMsgParseMailMsgState::ParseBodyState && (EMPTY_MESSAGE_LINE(line)));
 
 		// make sure we mangle naked From lines
-		if (line[0] == 'F' && !nsCRT::strncmp(line, "From ", 5))
+		if (line[0] == 'F' && !strncmp(line, "From ", 5))
 		{
 			res = XP_FileWrite (">", 1, m_out_file);
 			if (res < 1)
