@@ -62,6 +62,9 @@
 #include "nsMsgMimeCID.h"
 #include "prlog.h"
 #include "prprf.h"
+#include "nsIMimeHeaders.h"
+#include "nsIMsgWindow.h"
+#include "nsIMsgMailNewsUrl.h"
 
 static PRLogModuleInfo * gMimeEmitterLogModule = nsnull;
 
@@ -259,7 +262,7 @@ nsMimeBaseEmitter::LocalizeHeaderName(const char *aHeaderName, const char *aDefa
 {
   char *retVal = MimeGetStringByName(aHeaderName);
 
-  if (!retVal)
+  if (retVal)
     return retVal;
   else
     return nsCRT::strdup(aDefaultName);
@@ -413,7 +416,7 @@ nsMimeBaseEmitter::AddAttachmentField(const char *field, const char *value)
 NS_IMETHODIMP
 nsMimeBaseEmitter::UtilityWrite(const char *buf)
 {
-  PRInt32     tmpLen = nsCRT::strlen(buf);
+  PRInt32     tmpLen = strlen(buf);
   PRUint32    written;
 
   Write(buf, tmpLen, &written);
@@ -424,7 +427,7 @@ nsMimeBaseEmitter::UtilityWrite(const char *buf)
 NS_IMETHODIMP
 nsMimeBaseEmitter::UtilityWriteCRLF(const char *buf)
 {
-  PRInt32     tmpLen = nsCRT::strlen(buf);
+  PRInt32     tmpLen = strlen(buf);
   PRUint32    written;
 
   Write(buf, tmpLen, &written);
@@ -440,6 +443,10 @@ nsMimeBaseEmitter::Write(const char *buf, PRUint32 size, PRUint32 *amountWritten
   nsresult rv = NS_OK;
   PRUint32            needToWrite;
 
+#ifdef DEBUG_BenB
+  // If you want to see libmime output...
+  printf("%s", buf);
+#endif
 
   PR_LOG(gMimeEmitterLogModule, PR_LOG_ALWAYS, (buf));
   //
@@ -558,15 +565,15 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
         (PL_strcasecmp(aCharset, "ISO-8859-1")) &&
         (PL_strcasecmp(aCharset, "UTF-8")) )
   {
-    char    *contentType = nsnull;
+    nsCAutoString contentType;
     
-    if (NS_SUCCEEDED(mChannel->GetContentType(&contentType)) && contentType)
+    if (NS_SUCCEEDED(mChannel->GetContentType(contentType)) && !contentType.IsEmpty())
     {
-      char *cPtr = (char *) PL_strcasestr(contentType, "charset=");
+      char *cPtr = (char *) PL_strcasestr(contentType.get(), "charset=");
 
       if (cPtr)
       {
-        char  *ptr = contentType;
+        char  *ptr = (char *) contentType.get();
         while (*ptr)
         {
           if ( (*ptr == ' ') || (*ptr == ';') ) 
@@ -582,14 +589,9 @@ nsMimeBaseEmitter::UpdateCharacterSet(const char *aCharset)
         }
       }
 
-      char *newContentType = (char *) PR_smprintf("%s; charset=%s", contentType, aCharset);
-      if (newContentType)
-      {
-        mChannel->SetContentType(newContentType); 
-        PR_FREEIF(newContentType);
-      }
-      
-      PR_FREEIF(contentType);
+      // have to recompute strlen since contentType could have an embedded null byte
+      mChannel->SetContentType(nsDependentCString(contentType.get()));
+      mChannel->SetContentCharset(nsDependentCString(aCharset));
     }
   }
 
@@ -625,6 +627,22 @@ nsMimeBaseEmitter::AddHeaderField(const char *field, const char *value)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMimeBaseEmitter::AddAllHeaders(const char *allheaders, 
+                                 const PRInt32 allheadersize)
+{
+    nsresult rv;
+    nsCOMPtr<nsIMsgMailNewsUrl> msgurl (do_QueryInterface(mURL));
+    if (msgurl)
+    {
+        nsCOMPtr<nsIMimeHeaders> mimeHeaders = do_CreateInstance(NS_IMIMEHEADERS_CONTRACTID, &rv);
+        NS_ENSURE_SUCCESS(rv,rv);
+        mimeHeaders->Initialize(allheaders, allheadersize);
+        msgurl->SetMimeHeaders(mimeHeaders);
+    }
+    return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // The following code is responsible for formatting headers in a manner that is
 // identical to the normal XUL output.
@@ -752,10 +770,7 @@ nsMimeBaseEmitter::WriteHTMLHeaders()
 
   // Now, we need to either append the headers we built up to the 
   // overall body or output to the stream.
-  if ( (!mDocHeader) && (mFormat == nsMimeOutput::nsMimeMessageXULDisplay) )
-    mBody.Append(mHTMLHeaders);
-  else
-    UtilityWriteCRLF(mHTMLHeaders.get());
+  UtilityWriteCRLF(mHTMLHeaders.get());
 
   mHTMLHeaders = "";
 
@@ -765,7 +780,7 @@ nsMimeBaseEmitter::WriteHTMLHeaders()
 nsresult
 nsMimeBaseEmitter::DumpSubjectFromDate()
 {
-  mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" name=\"header-part1\">");
+  mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" class=\"header-part1\">");
 
     // This is the envelope information
     OutputGenericHeader(HEADER_SUBJECT);
@@ -795,7 +810,7 @@ nsMimeBaseEmitter::DumpToCC()
   // which looked weird.
   if (toField || ccField || bccField || newsgroupField)
   {
-    mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" name=\"header-part2\">");
+    mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" class=\"header-part2\">");
 
     OutputGenericHeader(HEADER_TO);
     OutputGenericHeader(HEADER_CC);
@@ -816,7 +831,7 @@ nsMimeBaseEmitter::DumpRestOfHeaders()
   if (mHeaderDisplayType != nsMimeHeaderDisplayTypes::AllHeaders)
     return NS_OK;
 
-  mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" name=\"header-part3\">");
+  mHTMLHeaders.Append("<table border=0 cellspacing=0 cellpadding=0 width=\"100%\" class=\"header-part3\">");
   
   for (i=0; i<mHeaderArray->Count(); i++)
   {
