@@ -37,6 +37,7 @@
 
 #include "nsICMS.h"
 #include "mimecms.h"
+#include "mimemsig.h"
 #include "nsCRT.h"
 #include "nspr.h"
 #include "nsEscape.h"
@@ -48,6 +49,7 @@
 #include "nsIMimeMiscStatus.h"
 #include "nsIMsgSMIMEHeaderSink.h"
 #include "nsCOMPtr.h"
+#include "nsIX509Cert.h"
 
 
 #define MIME_SUPERCLASS mimeEncryptedClass
@@ -73,7 +75,7 @@ MimeEncryptedCMSClassInitialize(MimeEncryptedCMSClass *clazz)
   MimeObjectClass    *oclass = (MimeObjectClass *)    clazz;
   MimeEncryptedClass *eclass = (MimeEncryptedClass *) clazz;
 
-  PR_ASSERT(!oclass->class_initialized);
+  NS_ASSERTION(!oclass->class_initialized, "1.2 <mscott@netscape.com> 01 Nov 2001 17:59");
 
   eclass->crypto_init          = MimeCMS_init;
   eclass->crypto_write         = MimeCMS_write;
@@ -100,6 +102,7 @@ typedef struct MimeCMSdata
   MimeObject *self;
   PRBool parent_is_encrypted_p;
   PRBool parent_holds_stamp_p;
+  nsCOMPtr<nsIMsgSMIMEHeaderSink> smimeHeaderSink;
   
   MimeCMSdata()
   :output_fn(nsnull),
@@ -193,7 +196,6 @@ MimeEncryptedCMS_encrypted_p (MimeObject *obj)
 }
 
 // extern MimeMessageClass mimeMessageClass;			/* gag */
-extern MimeObjectClass mimeMultipartSignedClass;	/* double gag */
 
 extern int ParseRFC822Addresses (const char *line,
 									 char **names, char **addresses);
@@ -211,7 +213,6 @@ MimeCMSHeadersAndCertsMatch(MimeObject *obj,
   nsXPIDLCString from_name;
   nsXPIDLCString sender_addr;
   nsXPIDLCString sender_name;
-  nsXPIDLCString cert_name;
   nsXPIDLCString cert_addr;
   PRBool match = PR_TRUE;
 
@@ -219,11 +220,11 @@ MimeCMSHeadersAndCertsMatch(MimeObject *obj,
    */
   if (content_info)
 	{
-	  content_info->GetSignerCommonName (getter_Copies(cert_name));
 	  content_info->GetSignerEmailAddress (getter_Copies(cert_addr));
 	}
-  if (!cert_name && !cert_addr) goto DONE;
 
+  if (!cert_addr)
+    goto DONE;
 
   /* Find the headers of the MimeMessage which is the parent (or grandparent)
 	 of this object (remember, crypto objects nest.) */
@@ -369,6 +370,31 @@ MimeCMS_init(MimeObject *obj,
 	data->parent_holds_stamp_p =
 	  mime_crypto_stamped_p (obj->parent->parent);
 
+  mime_stream_data *msd = (mime_stream_data *) (data->self->options->stream_closure);
+  if (msd)
+  {
+    nsIChannel *channel = msd->channel;  // note the lack of ref counting...
+    if (channel)
+    {
+      nsCOMPtr<nsIURI> uri;
+      nsCOMPtr<nsIMsgWindow> msgWindow;
+      nsCOMPtr<nsIMsgHeaderSink> headerSink;
+      nsCOMPtr<nsIMsgMailNewsUrl> msgurl;
+      nsCOMPtr<nsISupports> securityInfo;
+      channel->GetURI(getter_AddRefs(uri));
+      if (uri)
+        msgurl = do_QueryInterface(uri);
+      if (msgurl)
+        msgurl->GetMsgWindow(getter_AddRefs(msgWindow));
+      if (msgWindow)
+        msgWindow->GetMsgHeaderSink(getter_AddRefs(headerSink));
+      if (headerSink)
+        headerSink->GetSecurityInfo(getter_AddRefs(securityInfo));
+      if (securityInfo)
+        data->smimeHeaderSink = do_QueryInterface(securityInfo);
+    } // if channel
+  } // if msd
+
   return data;
 }
 
@@ -399,6 +425,21 @@ MimeCMS_eof (void *crypto_closure, PRBool abort_p)
   	return -1;
   }
 
+  // if we are the child of the topmost message, aNestLevel == 1
+  int aNestLevel = 0;
+
+  if (data->self) {
+    MimeObject *walker = data->self;
+    while (walker) {
+      // Crypto mime objects are transparent wrt nesting.
+      if (!mime_typep(walker, (MimeObjectClass *) &mimeEncryptedClass)
+          && !mime_typep(walker, (MimeObjectClass *) &mimeMultipartSignedClass)) {
+        ++aNestLevel;
+      }
+      walker = walker->parent;
+    }
+  }
+
   /* Hand an EOF to the crypto library.  It may call data->output_fn.
 	 (Today, the crypto library has no flushing to do, but maybe there
 	 will be someday.)
@@ -410,51 +451,105 @@ MimeCMS_eof (void *crypto_closure, PRBool abort_p)
   PR_SetError(0, 0);
   rv = data->decoder_context->Finish(getter_AddRefs(data->content_info));
 
-  /* Is the content info encrypted? */
-  if (data->content_info) {
-    data->ci_is_encrypted = PR_TRUE;
-  }
-
   if (NS_FAILED(rv))
 	  data->verify_error = PR_GetError();
 
-  mime_stream_data *msd = (mime_stream_data *) (data->self->options->stream_closure);
-  if (msd)
-  {
-    nsIChannel *channel = msd->channel;  // note the lack of ref counting...
-    if (channel)
-    {
-      nsCOMPtr<nsIURI> uri;
-      nsCOMPtr<nsIMsgWindow> msgWindow;
-      nsCOMPtr<nsIMsgHeaderSink> headerSink;
-      nsCOMPtr<nsIMsgMailNewsUrl> msgurl;
-      nsCOMPtr<nsISupports> securityInfo;
-      nsCOMPtr<nsIMsgSMIMEHeaderSink> smimeHeaderSink;
-      channel->GetURI(getter_AddRefs(uri));
-      if (uri)
-        msgurl = do_QueryInterface(uri);
-      if (msgurl)
-        msgurl->GetMsgWindow(getter_AddRefs(msgWindow));
-      if (msgWindow)
-        msgWindow->GetMsgHeaderSink(getter_AddRefs(headerSink));
-      if (headerSink)
-        headerSink->GetSecurityInfo(getter_AddRefs(securityInfo));
-      if (securityInfo)
-        smimeHeaderSink = do_QueryInterface(securityInfo);
-      if (smimeHeaderSink)
-      {
-          smimeHeaderSink->EncryptionStatus(
-            data->ci_is_encrypted 
-            && !data->verify_error
-            && !data->decode_error
-            && NS_SUCCEEDED(rv)
-          );
-      }
-    } // if channel
-  } // if msd
-
   data->decoder_context = 0;
 
+  nsCOMPtr<nsIX509Cert> certOfInterest;
+
+  if (!data->smimeHeaderSink)
+    return 0;
+
+  PRInt32 maxNestLevel = 0;
+  data->smimeHeaderSink->MaxWantedNesting(&maxNestLevel);
+
+  if (aNestLevel > maxNestLevel)
+    return 0;
+
+  PRInt32 status = nsICMSMessageErrors::SUCCESS;
+
+  if (data->verify_error
+      || data->decode_error
+      || NS_FAILED(rv))
+  {
+    status = nsICMSMessageErrors::GENERAL_ERROR;
+  }
+
+  if (!data->content_info)
+  {
+    status = nsICMSMessageErrors::GENERAL_ERROR;
+
+    // Although a CMS message could be either encrypted or opaquely signed,
+    // what we see is most likely encrypted, because if it were
+    // signed only, we probably would have been able to decode it.
+
+    data->ci_is_encrypted = PR_TRUE;
+  }
+  else
+  {
+    rv = data->content_info->ContentIsEncrypted(&data->ci_is_encrypted);
+
+    if (NS_SUCCEEDED(rv) && data->ci_is_encrypted) {
+      data->content_info->GetEncryptionCert(getter_AddRefs(certOfInterest));
+    }
+    else {
+      // Existing logic in mimei assumes, if !ci_is_encrypted, then it is signed.
+      // Make sure it indeed is signed.
+
+      PRBool testIsSigned;
+      rv = data->content_info->ContentIsSigned(&testIsSigned);
+
+      if (NS_FAILED(rv) || !testIsSigned) {
+        // Neither signed nor encrypted?
+        // We are unable to understand what we got, do not try to indicate S/Mime status.
+        return 0;
+      }
+
+      rv = data->content_info->VerifySignature();
+
+	    if (NS_FAILED(rv)) {
+        if (NS_ERROR_MODULE_SECURITY == NS_ERROR_GET_MODULE(rv)) {
+          status = NS_ERROR_GET_CODE(rv);
+        }
+        else if (NS_ERROR_NOT_IMPLEMENTED == rv) {
+          status = nsICMSMessageErrors::VERIFY_ERROR_PROCESSING;
+        }
+      }
+      else {
+        if (MimeCMSHeadersAndCertsMatch(data->self,
+											                  data->content_info,
+											                  &data->sender_addr))
+        {
+          status = nsICMSMessageErrors::SUCCESS;
+        }
+        else
+        {
+          status = nsICMSMessageErrors::VERIFY_HEADER_MISMATCH;
+        }
+      }
+
+      data->content_info->GetSignerCert(getter_AddRefs(certOfInterest));
+    }
+  }
+
+  if (data->ci_is_encrypted)
+  {
+    data->smimeHeaderSink->EncryptionStatus(
+      aNestLevel,
+      status,
+      certOfInterest
+    );
+  }
+  else
+  {
+    data->smimeHeaderSink->SignedStatus(
+      aNestLevel,
+      status,
+      certOfInterest
+    );
+  }
+
   return 0;
 }
 
@@ -499,7 +594,7 @@ MimeCMS_MakeSAURL(MimeObject *obj)
 		  break;
 		}
 	  obj = obj->parent;
-	  PR_ASSERT(obj);
+	  NS_ASSERTION(obj, "1.2 <mscott@netscape.com> 01 Nov 2001 17:59");
 	}
 
 
@@ -514,7 +609,7 @@ MimeCMS_MakeSAURL(MimeObject *obj)
 	  PR_FREEIF(id);
 	  PR_FREEIF(url);
 
-    stamp_url = (char *) PR_MALLOC(nsCRT::strlen(url2) + 50);
+	  stamp_url = (char *) PR_MALLOC(strlen(url2) + 50);
 	  if (stamp_url)
 		{
       PL_strcpy(stamp_url, "about:security?advisor=");
