@@ -62,6 +62,7 @@
 #include "nsCExternalHandlerService.h"
 #include "nsIMIMEService.h"
 #include "nsIMIMEInfo.h"
+#include "nsIMsgHeaderParser.h"
 
 //
 // Header strings...
@@ -168,6 +169,7 @@ typedef enum {
                                   // DON'T remove.
 } nsMsgBoolHeaderSet;
 
+#ifdef NS_DEBUG
 extern "C" void
 mime_dump_attachments ( nsMsgAttachmentData *attachData )
 {
@@ -180,23 +182,22 @@ mime_dump_attachments ( nsMsgAttachmentData *attachData )
 
     if ( tmp->url ) 
     {
-      char *spec = nsnull;
-
-      tmp->url->GetSpec(&spec);
-      printf("URL               : %s\n", spec);
-      PR_FREEIF(spec);
+      nsCAutoString spec;
+      tmp->url->GetSpec(spec);
+      printf("URL               : %s\n", spec.get());
     }
 
-    printf("Desired Type      : %s\n", tmp->desired_type);
-    printf("Real Type         : %s\n", tmp->real_type);
-    printf("Real Encoding     : %s\n", tmp->real_encoding); 
-    printf("Description       : %s\n", tmp->description);
-    printf("Mac Type          : %s\n", tmp->x_mac_type);
-    printf("Mac Creator       : %s\n", tmp->x_mac_creator);
+    printf("Desired Type      : %s\n", tmp->desired_type ? tmp->desired_type : "nsnull");
+    printf("Real Type         : %s\n", tmp->real_type ? tmp->real_type : "nsnull");
+    printf("Real Encoding     : %s\n", tmp->real_encoding ? tmp->real_encoding : "nsnull"); 
+    printf("Description       : %s\n", tmp->description) ? tmp->description : "nsnull";
+    printf("Mac Type          : %s\n", tmp->x_mac_type ? tmp->x_mac_type : "nsnull");
+    printf("Mac Creator       : %s\n", tmp->x_mac_creator ? tmp->x_mac_creator : "nsnull");
     i++;
     tmp++;
   }
 }
+#endif
 
 nsresult
 CreateTheComposeWindow(nsIMsgCompFields     *compFields,
@@ -216,11 +217,11 @@ mime_dump_attachments ( attachmentList );
   nsMsgAttachmentData *curAttachment = attachmentList;
   if (curAttachment)
   {
-    nsXPIDLCString spec;
+    nsCAutoString spec;
 
     while (curAttachment && curAttachment->real_name) //JFD: Why do we check for real_name?
     {
-      rv = curAttachment->url->GetSpec(getter_Copies(spec));
+      rv = curAttachment->url->GetSpec(spec);
       if (NS_SUCCEEDED(rv))
       {
         nsCOMPtr<nsIMsgAttachment> attachment = do_CreateInstance(NS_MSGATTACHMENT_CONTRACTID, &rv);
@@ -308,7 +309,7 @@ CreateCompositionFields(const char        *from,
   NS_ENSURE_TRUE(cFields, NS_ERROR_OUT_OF_MEMORY);
   
   // Now set all of the passed in stuff...
-  cFields->SetCharacterSet(charset);
+  cFields->SetCharacterSet(!nsCRT::strcasecmp("us-ascii", charset) ? "ISO-8859-1" : charset);
   
   char *val;
   
@@ -457,8 +458,7 @@ mime_parse_stream_write ( nsMIMESession *stream, const char *buf, PRInt32 size )
 static void
 mime_free_attach_data ( nsMsgAttachmentData *attachData)
 {
-  PRInt32     i;
-    struct nsMsgAttachmentData  *tmp = attachData;
+  struct nsMsgAttachmentData  *tmp = attachData;
 
   while (tmp && tmp->real_name) 
   {
@@ -521,7 +521,8 @@ mime_draft_process_attachments(mime_draft_data *mdd)
 
   //It's possible we must treat the message body as attachment!
   PRBool bodyAsAttachment = PR_FALSE;
-  if (  mdd->messageBody->type && *mdd->messageBody->type &&
+  if (  mdd->messageBody &&
+        mdd->messageBody->type && *mdd->messageBody->type &&
         ( PL_strcasestr(mdd->messageBody->type, "text/html") == nsnull ) &&
         ( PL_strcasestr(mdd->messageBody->type, "text/plain") == nsnull ) &&
         ( PL_strcasecmp(mdd->messageBody->type, "text") != 0 )
@@ -555,13 +556,12 @@ mime_draft_process_attachments(mime_draft_data *mdd)
 
     if ( tmpFile->orig_url ) 
     {
-      char  *tmpSpec = nsnull;
-      if (NS_FAILED(tmpFile->orig_url->GetSpec(&tmpSpec)))
+      nsCAutoString tmpSpec;
+      if (NS_FAILED(tmpFile->orig_url->GetSpec(tmpSpec)))
         goto FAIL;
 
-      if (NS_FAILED(nsMimeNewURI(&(tmp->url), tmpSpec, nsnull)))
+      if (NS_FAILED(nsMimeNewURI(&(tmp->url), tmpSpec.get(), nsnull)))
       {
-        PR_FREEIF(tmpSpec);
         goto FAIL; 
       }
 
@@ -571,7 +571,7 @@ mime_draft_process_attachments(mime_draft_data *mdd)
         if (tmpFile->real_name)
           NS_MsgSACopy ( &(tmp->real_name), tmpFile->real_name );
         else
-          NS_MsgSACopy ( &(tmp->real_name), tmpSpec );
+          NS_MsgSACopy ( &(tmp->real_name), tmpSpec.get() );
       }
     }
 
@@ -627,12 +627,12 @@ mime_fix_up_html_address( char **addr)
     PRInt32 newLen = 0;
     do 
     {
-      newLen = nsCRT::strlen(*addr) + 3 + 1;
+      newLen = strlen(*addr) + 3 + 1;
       *addr = (char *) PR_REALLOC(*addr, newLen);
       NS_ASSERTION (*addr, "out of memory fixing up html address");
       lt = PL_strchr(*addr, '<');
       NS_ASSERTION(lt, "couldn't find < char in address");
-      nsCRT::memmove(lt+4, lt+1, newLen - 4 - (lt - *addr));
+      memmove(lt+4, lt+1, newLen - 4 - (lt - *addr));
       *lt++ = '&';
       *lt++ = 'l';
       *lt++ = 't';
@@ -699,6 +699,28 @@ MimeGetNamedString(PRInt32 id)
   return retString;
 }
 
+/* given an address string passed though parameter "address", this one will be converted
+   and returned through the same parameter. The original string will be destroyed
+*/
+static void UnquoteMimeAddress(nsIMsgHeaderParser* parser, char** address)
+{
+  if (parser && address && *address && **address)
+  {
+    char *result;
+    if (NS_SUCCEEDED(parser->UnquotePhraseOrAddr(*address, PR_FALSE, &result)))
+    {
+      if (result && *result)
+      {
+        PR_Free(*address);
+        *address = result;
+        return;
+      }
+      PR_FREEIF(result);
+    }
+  }
+
+  return;
+}
 
 static void 
 mime_insert_all_headers(char            **body,
@@ -706,6 +728,8 @@ mime_insert_all_headers(char            **body,
                         MSG_ComposeFormat composeFormat,
                         char            *mailcharset)
 {
+  nsCOMPtr<nsIMsgHeaderParser> parser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
+
   PRBool htmlEdit = (composeFormat == nsIMsgCompFormat::HTML);
   char *newBody = NULL;
   char *html_tag = nsnull;
@@ -801,10 +825,16 @@ mime_insert_all_headers(char            **body,
     */
     if (nsCRT::strcasecmp(name, "bcc") != 0)
     {
-      if (htmlEdit) mime_fix_up_html_address(&c2);
+      if (htmlEdit)
+        mime_fix_up_html_address(&c2);
         
-      mime_intl_insert_message_header_1(&newBody, &c2, name, name, mailcharset,
-                      htmlEdit);
+      if (!nsCRT::strcasecmp(name, "resent-from") || !nsCRT::strcasecmp(name, "from") ||
+          !nsCRT::strcasecmp(name, "resent-to") || !nsCRT::strcasecmp(name, "to") ||
+          !nsCRT::strcasecmp(name, "resent-cc") || !nsCRT::strcasecmp(name, "cc") ||
+          !nsCRT::strcasecmp(name, "reply-to"))
+        UnquoteMimeAddress(parser, &c2);
+
+      mime_intl_insert_message_header_1(&newBody, &c2, name, name, mailcharset, htmlEdit);
     }
     PR_Free(name);
     PR_Free(c2);
@@ -839,27 +869,21 @@ mime_insert_normal_headers(char             **body,
                            MSG_ComposeFormat  composeFormat,
                            char             *mailcharset)
 {
-  char *newBody = NULL;
+  char *newBody = nsnull;
   char *subject = MimeHeaders_get(headers, HEADER_SUBJECT, PR_FALSE, PR_FALSE);
-  char *resent_comments = MimeHeaders_get(headers, HEADER_RESENT_COMMENTS,
-                      PR_FALSE, PR_FALSE);
-  char *resent_date = MimeHeaders_get(headers, HEADER_RESENT_DATE, PR_FALSE,
-                    PR_TRUE); 
-  char *resent_from = MimeHeaders_get(headers, HEADER_RESENT_FROM, PR_FALSE,
-                    PR_TRUE);
+  char *resent_comments = MimeHeaders_get(headers, HEADER_RESENT_COMMENTS, PR_FALSE, PR_FALSE);
+  char *resent_date = MimeHeaders_get(headers, HEADER_RESENT_DATE, PR_FALSE, PR_TRUE); 
+  char *resent_from = MimeHeaders_get(headers, HEADER_RESENT_FROM, PR_FALSE, PR_TRUE);
   char *resent_to = MimeHeaders_get(headers, HEADER_RESENT_TO, PR_FALSE, PR_TRUE);
   char *resent_cc = MimeHeaders_get(headers, HEADER_RESENT_CC, PR_FALSE, PR_TRUE);
   char *date = MimeHeaders_get(headers, HEADER_DATE, PR_FALSE, PR_TRUE);
   char *from = MimeHeaders_get(headers, HEADER_FROM, PR_FALSE, PR_TRUE);
   char *reply_to = MimeHeaders_get(headers, HEADER_REPLY_TO, PR_FALSE, PR_TRUE);
-  char *organization = MimeHeaders_get(headers, HEADER_ORGANIZATION,
-                     PR_FALSE, PR_FALSE);
+  char *organization = MimeHeaders_get(headers, HEADER_ORGANIZATION, PR_FALSE, PR_FALSE);
   char *to = MimeHeaders_get(headers, HEADER_TO, PR_FALSE, PR_TRUE);
   char *cc = MimeHeaders_get(headers, HEADER_CC, PR_FALSE, PR_TRUE);
-  char *newsgroups = MimeHeaders_get(headers, HEADER_NEWSGROUPS, PR_FALSE,
-                     PR_TRUE);
-  char *followup_to = MimeHeaders_get(headers, HEADER_FOLLOWUP_TO, PR_FALSE,
-                    PR_TRUE);
+  char *newsgroups = MimeHeaders_get(headers, HEADER_NEWSGROUPS, PR_FALSE, PR_TRUE);
+  char *followup_to = MimeHeaders_get(headers, HEADER_FOLLOWUP_TO, PR_FALSE, PR_TRUE);
   char *references = MimeHeaders_get(headers, HEADER_REFERENCES, PR_FALSE, PR_TRUE);
   const char *html_tag = nsnull;
   if (*body)
@@ -872,6 +896,15 @@ mime_insert_normal_headers(char             **body,
     resent_from = MimeHeaders_get(headers, HEADER_RESENT_SENDER, PR_FALSE,
                     PR_TRUE); 
   
+  nsCOMPtr<nsIMsgHeaderParser> parser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
+  UnquoteMimeAddress(parser, &resent_from);
+  UnquoteMimeAddress(parser, &resent_to);
+  UnquoteMimeAddress(parser, &resent_cc);
+  UnquoteMimeAddress(parser, &reply_to);
+  UnquoteMimeAddress(parser, &from);
+  UnquoteMimeAddress(parser, &to);
+  UnquoteMimeAddress(parser, &cc);
+
   if (htmlEdit)
   {
     NS_MsgSACopy(&(newBody), "<HTML> <BR><BR>");
@@ -1048,6 +1081,12 @@ mime_insert_micro_headers(char            **body,
   if (!date)
     date = MimeHeaders_get(headers, HEADER_RESENT_DATE, PR_FALSE, PR_TRUE);
   
+  nsCOMPtr<nsIMsgHeaderParser> parser = do_GetService(NS_MAILNEWS_MIME_HEADER_PARSER_CONTRACTID);
+  UnquoteMimeAddress(parser, &resent_from);
+  UnquoteMimeAddress(parser, &from);
+  UnquoteMimeAddress(parser, &to);
+  UnquoteMimeAddress(parser, &cc);
+
   if (htmlEdit)
   {
     NS_MsgSACopy(&(newBody), "<HTML> <BR><BR>");
@@ -1155,7 +1194,7 @@ mime_insert_forwarded_message_headers(char            **body,
     // convert body from mail charset to UTF-8
     char *utf8 = NULL;
     nsAutoString ucs2;
-    if (NS_SUCCEEDED(nsMsgI18NConvertToUnicode(nsCAutoString(mailcharset), nsCAutoString(*body), ucs2))) {
+    if (NS_SUCCEEDED(ConvertToUnicode(mailcharset, *body, ucs2))) {
       utf8 = ToNewUTF8String(ucs2);
       if (NULL != utf8) {
         PR_Free(*body);
@@ -1215,6 +1254,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
   PRBool sign_p = PR_FALSE;   /* #### how do we determine this? */
   PRBool forward_inline = PR_FALSE;
   PRBool bodyAsAttachment = PR_FALSE;
+  PRBool charsetOverride = PR_FALSE;
   
   NS_ASSERTION (mdd, "null mime draft data");
   
@@ -1248,6 +1288,14 @@ mime_parse_stream_complete (nsMIMESession *stream)
     mdd->obj = 0;
     if (mdd->options) 
     {
+      // save the override flag before it's unavailable
+      charsetOverride = mdd->options->override_charset;
+      if ((!mdd->mailcharset || charsetOverride) && mdd->options->default_charset)
+      {
+        PR_Free(mdd->mailcharset);
+        mdd->mailcharset = nsCRT::strdup(mdd->options->default_charset);
+      }
+
       // mscott: aren't we leaking a bunch of trings here like the charset strings and such?
       delete mdd->options;
       mdd->options = 0;
@@ -1343,7 +1391,11 @@ mime_parse_stream_complete (nsMIMESession *stream)
         int receiptType = 0;
         fields->SetReturnReceipt(PR_TRUE); 
         sscanf(parm, "%d", &receiptType);
-        fields->SetReturnReceipt((PRInt32) receiptType);
+        // slight change compared to 4.x; we used to use receipt= to tell
+        // whether the draft/template has request for either MDN or DNS or both
+        // return receipt; since the DNS is out of the picture we now use the
+        // header type - 1 to tell whether user has requested the return receipt
+        fields->SetReceiptHeaderType(((PRInt32)receiptType) - 1);
       }
       PR_FREEIF(parm);
       parm = MimeHeaders_get_parameter(draftInfo, "uuencode", NULL, NULL);
@@ -1368,7 +1420,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
         if( PL_strcasestr(mdd->messageBody->type, "text/html"))
           composeFormat = nsIMsgCompFormat::HTML;
         else if ( PL_strcasestr(mdd->messageBody->type, "text/plain") ||
-                  PL_strcasecmp(mdd->messageBody->type, "text") )
+                  !PL_strcasecmp(mdd->messageBody->type, "text") )
           composeFormat = nsIMsgCompFormat::PlainText;
         else
         {
@@ -1388,7 +1440,7 @@ mime_parse_stream_complete (nsMIMESession *stream)
         body = (char *)PR_MALLOC (bodyLen + 1);
         if (body)
         {
-          nsCRT::memset (body, 0, bodyLen+1);
+          memset (body, 0, bodyLen+1);
         
           nsInputFileStream inputFile(*(mdd->messageBody->file_spec));
           if (inputFile.is_open())
@@ -1437,36 +1489,32 @@ mime_parse_stream_complete (nsMIMESession *stream)
       // Ok, if we are here, then we should look at the charset and convert
       // to UTF-8...
       //
-      if (!forward_inline)
+      if (!forward_inline && body)
       {
-        char *bodyCharset = MimeHeaders_get_parameter (mdd->messageBody->type, "charset", NULL, NULL);
-        if (bodyCharset && body)
+        char *mimeCharset = nsnull;
+        // Get a charset from the header if no override is set.
+        if (!charsetOverride)
+          mimeCharset = MimeHeaders_get_parameter (mdd->messageBody->type, "charset", NULL, NULL);
+        // If no charset is specified in the header then use the default.
+        char *bodyCharset = mimeCharset ? mimeCharset : mdd->mailcharset;
+        if (bodyCharset)
         {
-          // Now do conversion to UTF-8 for output
+          // Now do conversion to Unicode for output
           nsAutoString tempUnicodeString;
           if (NS_SUCCEEDED(ConvertToUnicode(bodyCharset, body, tempUnicodeString)))
-          {
-            char *utf8Str = nsCRT::strdup(NS_ConvertUCS2toUTF8(tempUnicodeString.get()).get());
-            if (utf8Str)
-            {
-              PR_FREEIF(body);
-              body = utf8Str;
-            }
-          }
-
-          PR_FREEIF(bodyCharset);
+            fields->SetBody(tempUnicodeString.get());
+          else
+            fields->SetBody(NS_ConvertASCIItoUCS2(body).get());
+          PR_Free(body);
+          body = nsnull;
+          PR_FREEIF(mimeCharset);
         }
       }
 
       // convert from UTF-8 to UCS2
       if (body)
       {
-        nsString ucs2;
-        if (NS_SUCCEEDED(nsMsgI18NConvertToUnicode(nsCAutoString("UTF-8"), nsCAutoString(body), ucs2)))
-          fields->SetBody(ucs2.get());
-        else
-          fields->SetBody(NS_ConvertASCIItoUCS2(body).get());
-      
+        fields->SetBody(NS_ConvertUTF8toUCS2(body).get());
         PR_Free(body);
       }
 
@@ -1680,24 +1728,20 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
   if (!mdd || !headers) 
     return -1;
 
-  if ( !mdd->options->is_multipart_msg ) 
+  if (mdd->options->decompose_init_count) 
   {
-    if (mdd->options->decompose_init_count) 
-    {
-      mdd->options->decompose_init_count++;
-      NS_ASSERTION(mdd->curAttachment, "missing attachment in mime_decompose_file_init_fn");
-      if (mdd->curAttachment) {
-        char *ct = MimeHeaders_get(headers, HEADER_CONTENT_TYPE, PR_TRUE, PR_FALSE);
-        if (ct)
-          NS_MsgSACopy(&(mdd->curAttachment->type), ct);
-        PR_FREEIF(ct);
-      }
-      return 0;
-    }
-    else {
-      mdd->options->decompose_init_count++;
+    mdd->options->decompose_init_count++;
+    NS_ASSERTION(mdd->curAttachment, "missing attachment in mime_decompose_file_init_fn");
+    if (mdd->curAttachment) {
+      char *ct = MimeHeaders_get(headers, HEADER_CONTENT_TYPE, PR_TRUE, PR_FALSE);
+      if (ct)
+        NS_MsgSACopy(&(mdd->curAttachment->type), ct);
+      PR_FREEIF(ct);
     }
+    return 0;
   }
+  else
+    mdd->options->decompose_init_count++;
 
   nAttachments = mdd->attachments_count;
 
@@ -1753,7 +1797,7 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
     }
     
     newAttachment = attachments + nAttachments;
-    nsCRT::memset ( newAttachment, 0, sizeof (nsMsgAttachedFile) * 2 );
+    memset ( newAttachment, 0, sizeof (nsMsgAttachedFile) * 2 );
   }
 
   char *workURLSpec = nsnull;
@@ -1807,29 +1851,8 @@ mime_decompose_file_init_fn ( void *stream_closure, MimeHeaders *headers )
     newAttachment->description = nsCRT::strdup(workURLSpec);
 
   nsFileSpec *tmpSpec = nsnull;
-  if (newAttachment->real_name && *newAttachment->real_name)
   {
-    //Need some convertion to native file system character set
-    nsAutoString outStr;
-    char * fileName = nsnull;
-    nsresult rv = ConvertToUnicode(msgCompHeaderInternalCharset(), newAttachment->real_name, outStr);
-    if (NS_SUCCEEDED(rv))
-    {
-      rv = ConvertFromUnicode(nsMsgI18NFileSystemCharset(), outStr, &fileName);
-      if (NS_FAILED(rv))
-        fileName = PL_strdup(newAttachment->real_name);
-    }
-    else
-      fileName = PL_strdup(newAttachment->real_name);
-
-    tmpSpec = nsMsgCreateTempFileSpec(fileName);
-    PR_FREEIF(fileName);
-  }
-  else
-  {
-    // if we didn't get a real name for the attachment, then we should
-    // ask the mime service what extension this content type should use and use nsmail.that file extension
-    // if that doesn't work then just use "nsmail.tmp"
+    // Let's build a temp file with an extension based on the content-type: nsmail.<extension>
 
     nsCAutoString  newAttachName ("nsmail");
     PRBool extensionAdded = PR_FALSE;
@@ -1962,11 +1985,8 @@ mime_decompose_file_close_fn ( void *stream_closure )
   if ( !mdd || !mdd->tmpFileStream )
     return -1;
   
-  if ( !mdd->options->is_multipart_msg ) 
-  {
-    if ( --mdd->options->decompose_init_count > 0 )
+  if ( --mdd->options->decompose_init_count > 0 )
       return 0;
-  }
   
   if (mdd->decoder_data) {
     MimeDecoderDestroy(mdd->decoder_data, PR_FALSE);
@@ -1975,7 +1995,10 @@ mime_decompose_file_close_fn ( void *stream_closure )
 
   if (mdd->tmpFileStream->GetIStream())
     mdd->tmpFileStream->close();
+
   delete mdd->tmpFileStream;
+  mdd->tmpFileStream = nsnull;
+
   delete mdd->tmpFileSpec;
   mdd->tmpFileSpec = nsnull;
   
@@ -1990,8 +2013,8 @@ mime_bridge_create_draft_stream(
                           nsMimeOutputType    format_out)
 {
   int                     status = 0;
-  nsMIMESession           *stream = NULL;
-  struct mime_draft_data  *mdd = NULL;
+  nsMIMESession           *stream = nsnull;
+  struct mime_draft_data  *mdd = nsnull;
   MimeObject              *obj;
 
   if ( !uri ) 
@@ -2002,24 +2025,24 @@ mime_bridge_create_draft_stream(
     return nsnull;
 
   // first, convert the rdf msg uri into a url that represents the message...
-  nsXPIDLCString turl;
-  if (NS_FAILED(uri->GetSpec(getter_Copies(turl))))
+  nsCAutoString turl;
+  if (NS_FAILED(uri->GetSpec(turl)))
     return nsnull;
 
   nsCOMPtr <nsIMsgMessageService> msgService;
-  nsresult rv = GetMessageServiceFromURI(turl, getter_AddRefs(msgService));
+  nsresult rv = GetMessageServiceFromURI(turl.get(), getter_AddRefs(msgService));
   if (NS_FAILED(rv)) 
     return nsnull;
 
   nsCOMPtr<nsIURI> aURL;
-  rv = msgService->GetUrlForUri(turl, getter_AddRefs(aURL), nsnull);
+  rv = msgService->GetUrlForUri(turl.get(), getter_AddRefs(aURL), nsnull);
   if (NS_FAILED(rv)) 
     return nsnull;
 
-  nsXPIDLCString urlString;
-  if (NS_SUCCEEDED(aURL->GetSpec(getter_Copies(urlString))))
+  nsCAutoString urlString;
+  if (NS_SUCCEEDED(aURL->GetSpec(urlString)))
   {
-    mdd->url_name = nsCRT::strdup(urlString);
+    mdd->url_name = ToNewCString(urlString);
     if (!(mdd->url_name))
     {
       PR_FREEIF(mdd);
