@@ -39,28 +39,29 @@
 #include "mimesun.h"	/*   |     |     |--- MimeSunAttachment				*/
 #include "mimemsig.h"	/*   |     |     |--- MimeMultipartSigned (abstract)*/
 #ifdef ENABLE_SMIME
-#include "mimemcms.h" /*   |     |           |---MimeMultipartSignedCMS */
+#include "mimemcms.h"   /*   |     |           |---MimeMultipartSignedCMS   */
 #endif
 #include "mimecryp.h"	/*   |     |--- MimeEncrypted (abstract)			*/
 #ifdef ENABLE_SMIME
 #include "mimecms.h"	/*   |     |     |--- MimeEncryptedPKCS7			*/
 #endif
-
 #include "mimemsg.h"	/*   |     |--- MimeMessage							*/
 #include "mimeunty.h"	/*   |     |--- MimeUntypedText						*/
 #include "mimeleaf.h"	/*   |--- MimeLeaf (abstract)						*/
 #include "mimetext.h"	/*   |     |--- MimeInlineText (abstract)			*/
 #include "mimetpla.h"	/*   |     |     |--- MimeInlineTextPlain			*/
-#include "mimetpfl.h" /*   |     |     |--- MimeInlineTextPlainFlowed         */
+#include "mimethpl.h"	/*   |     |     |     |--- M.I.TextHTMLAsPlaintext */
+#include "mimetpfl.h"   /*   |     |     |--- MimeInlineTextPlainFlowed     */
 #include "mimethtm.h"	/*   |     |     |--- MimeInlineTextHTML			*/
+#include "mimethsa.h"	/*   |     |     |     |--- M.I.TextHTMLSanitized   */
 #include "mimetric.h"	/*   |     |     |--- MimeInlineTextRichtext		*/
 #include "mimetenr.h"	/*   |     |     |     |--- MimeInlineTextEnriched	*/
-/* SUPPORTED VIA PLUGIN    |     |     |--------- MimeInlineTextCalendar  */
-
-#include "nsIPref.h"
+/* SUPPORTED VIA PLUGIN      |     |     |--- MimeInlineTextVCard           */
+/* SUPPORTED VIA PLUGIN      |     |     |--- MimeInlineTextCalendar        */
 #include "mimeiimg.h"	/*   |     |--- MimeInlineImage						*/
 #include "mimeeobj.h"	/*   |     |--- MimeExternalObject					*/
 #include "mimeebod.h"	/*   |--- MimeExternalBody							*/
+                        /* If you add classes here,also add them to mimei.h */
 #include "prlog.h"
 #include "prmem.h"
 #include "prenv.h"
@@ -78,6 +79,8 @@
 #include "nsMimeStringResources.h"
 #include "nsMimeTypes.h"
 #include "nsMsgUtils.h"
+#include "nsIPref.h"
+#include "imgILoader.h"
 
 #define	IMAP_EXTERNAL_CONTENT_HEADER "X-Mozilla-IMAP-Part"
 
@@ -230,8 +233,8 @@ mime_new (MimeObjectClass *clazz, MimeHeaders *hdrs,
   int status;
 
   /* Some assertions to verify that this isn't random junk memory... */
-  PR_ASSERT(clazz->class_name && nsCRT::strlen(clazz->class_name) > 0);
-  PR_ASSERT(size > 0 && size < 1000);
+  NS_ASSERTION(clazz->class_name && strlen(clazz->class_name) > 0, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
+  NS_ASSERTION(size > 0 && size < 1000, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 
   if (!clazz->class_initialized)
 	{
@@ -239,7 +242,7 @@ mime_new (MimeObjectClass *clazz, MimeHeaders *hdrs,
 	  if (status < 0) return 0;
 	}
 
-  PR_ASSERT(clazz->initialize && clazz->finalize);
+  NS_ASSERTION(clazz->initialize && clazz->finalize, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 
   if (hdrs)
 	{
@@ -253,7 +256,7 @@ mime_new (MimeObjectClass *clazz, MimeHeaders *hdrs,
   memset(object, 0, size);
   object->clazz = clazz;
   object->headers = hdrs;
-  object->showAttachmentIcon = PR_FALSE; /* initialize ricardob's new member. */
+  object->dontShowAsAttachment = PR_FALSE;
 
   if (override_content_type && *override_content_type)
 	object->content_type = nsCRT::strdup(override_content_type);
@@ -287,6 +290,75 @@ mime_free (MimeObject *object)
   PR_Free(object);
 }
 
+
+PRBool mime_is_allowed_class(const MimeObjectClass *clazz,
+                             PRInt32 types_of_classes_to_disallow)
+{
+  if (types_of_classes_to_disallow == 0)
+    return PR_TRUE;
+  PRBool avoid_html = (types_of_classes_to_disallow >= 1);
+  PRBool avoid_images = (types_of_classes_to_disallow >= 2);
+  PRBool avoid_strange_content = (types_of_classes_to_disallow >= 3);
+  PRBool allow_only_vanilla_classes = (types_of_classes_to_disallow == 100);
+
+  if (allow_only_vanilla_classes)
+    /* A "safe" class is one that is unlikely to have security bugs or to
+       allow security exploits or one that is essential for the usefulness
+       of the application, even for paranoid users.
+       What's included here is more personal judgement than following
+       strict rules, though, unfortunately.
+       The function returns true only for known good classes, i.e. is a
+       "whitelist" in this case.
+       This idea comes from Georgi Guninski.
+    */
+    return
+      (
+        clazz == (MimeObjectClass *)&mimeInlineTextPlainClass ||
+        clazz == (MimeObjectClass *)&mimeInlineTextPlainFlowedClass ||
+        clazz == (MimeObjectClass *)&mimeInlineTextHTMLSanitizedClass ||
+        clazz == (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass ||
+           /* The latter 2 classes bear some risk, because they use the Gecko
+              HTML parser, but the user has the option to make an explicit
+              choice in this case, via html_as. */
+        clazz == (MimeObjectClass *)&mimeMultipartMixedClass ||
+        clazz == (MimeObjectClass *)&mimeMultipartAlternativeClass ||
+        clazz == (MimeObjectClass *)&mimeMultipartDigestClass ||
+        clazz == (MimeObjectClass *)&mimeMultipartAppleDoubleClass ||
+        clazz == (MimeObjectClass *)&mimeMessageClass ||
+        clazz == (MimeObjectClass *)&mimeExternalObjectClass ||
+                               /*    mimeUntypedTextClass? -- does uuencode */
+#ifdef ENABLE_SMIME
+        clazz == (MimeObjectClass *)&mimeMultipartSignedCMSClass ||
+        clazz == (MimeObjectClass *)&mimeEncryptedCMSClass ||
+#endif
+        clazz == 0
+      );
+
+  /* Contrairy to above, the below code is a "blacklist", i.e. it
+     *excludes* some "bad" classes. */
+  return
+     !(
+        (avoid_html
+         && (
+              clazz == (MimeObjectClass *)&mimeInlineTextHTMLClass
+                         /* Should not happen - we protect against that in
+                            mime_find_class(). Still for safety... */
+            )) ||
+        (avoid_images
+         && (
+              clazz == (MimeObjectClass *)&mimeInlineImageClass
+            )) ||
+        (avoid_strange_content
+         && (
+              clazz == (MimeObjectClass *)&mimeInlineTextEnrichedClass ||
+              clazz == (MimeObjectClass *)&mimeInlineTextRichtextClass ||
+              clazz == (MimeObjectClass *)&mimeSunAttachmentClass ||
+              clazz == (MimeObjectClass *)&mimeExternalBodyClass
+            ))
+      );
+}
+
+
 MimeObjectClass *
 mime_find_class (const char *content_type, MimeHeaders *hdrs,
 				 MimeDisplayOptions *opts, PRBool exact_match_p)
@@ -295,7 +367,33 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
   MimeObjectClass *tempClass = 0;
   contentTypeHandlerInitStruct  ctHandlerInfo;
 
-  /* 
+  // Read some prefs
+  nsIPref *pref = GetPrefServiceManager(opts); 
+  PRInt32 html_as = 0;  // def. see below
+  PRInt32 types_of_classes_to_disallow = 0;  /* Let only a few libmime classes
+       process incoming data. This protects from bugs (e.g. buffer overflows)
+       and from security loopholes (e.g. allowing unchecked HTML in some
+       obscure classes, although the user has html_as > 0).
+       This option is mainly for the UI of html_as.
+       0 = allow all available classes
+       1 = Use hardcoded blacklist to avoid rendering (incoming) HTML
+       2 = ... and images
+       3 = ... and some other uncommon content types
+       100 = Use hardcoded whitelist to avoid even more bugs(buffer overflows).
+           This mode will limit the features available (e.g. uncommon
+           attachment types and inline images) and is for paranoid users.
+       */
+  if (pref)
+  {
+    pref->GetIntPref("mailnews.display.html_as", &html_as);
+    pref->GetIntPref("mailnews.display.disallow_mime_handlers",
+                     &types_of_classes_to_disallow);
+    if (types_of_classes_to_disallow > 0 && html_as == 0)
+         // We have non-sensical prefs. Do some fixup.
+      html_as = 1;
+  }
+
+  /*
   * What we do first is check for an external content handler plugin. 
   * This will actually extend the mime handling by calling a routine 
   * which will allow us to load an external content type handler
@@ -304,25 +402,64 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
   */
   if ((tempClass = mime_locate_external_content_handler(content_type, &ctHandlerInfo)) != NULL)
   {
-    clazz = (MimeObjectClass *)tempClass;
+    if (types_of_classes_to_disallow > 0
+        && (!nsCRT::strncasecmp(content_type, "text/x-vcard", 12) ||
+            !nsCRT::strncasecmp(content_type, "text/calendar", 13))
+       )
+      /* Use a little hack to prevent some dangerous plugins, which ship
+         with Mozilla, to run.
+         For the truely user-installed plugins, we rely on the judgement
+         of the user. */
+    {
+      if (!exact_match_p)
+        clazz = (MimeObjectClass *)&mimeExternalObjectClass; // As attachment
+    }
+    else
+      clazz = (MimeObjectClass *)tempClass;
   }
   else
   {
     if (!content_type || !*content_type ||
-      !nsCRT::strcasecmp(content_type, "text"))  /* with no / in the type */
+        !nsCRT::strcasecmp(content_type, "text"))  /* with no / in the type */
       clazz = (MimeObjectClass *)&mimeUntypedTextClass;
     
-      /* Subtypes of text...
+    /* Subtypes of text...
     */
     else if (!nsCRT::strncasecmp(content_type,			"text/", 5))
     {
       if      (!nsCRT::strcasecmp(content_type+5,		"html"))
-        clazz = (MimeObjectClass *)&mimeInlineTextHTMLClass;
+      {
+        if (opts
+            && opts->format_out == nsMimeOutput::nsMimeMessageSaveAs)
+          // SaveAs in new modes doesn't work yet.
+        {
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLClass;
+          types_of_classes_to_disallow = 0;
+        }
+        else if (html_as == 0) // Render sender's HTML
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLClass;
+        else if (html_as == 1) // convert HTML to plaintext
+          // Do a HTML->TXT->HTML conversion, see mimethpl.h.
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass;
+        else if (html_as == 2) // display HTML source
+          /* This is for the freaks. Treat HTML as plaintext,
+             which will cause the HTML source to be displayed.
+             Not very user-friendly, but some seem to want this. */
+          clazz = (MimeObjectClass *)&mimeInlineTextPlainClass;
+        else if (html_as == 3) // Sanitize
+          // Strip all but allowed HTML
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLSanitizedClass;
+        else // Goofy pref
+          /* User has an unknown pref value. Maybe he used a newer Mozilla
+             with a new alternative to avoid HTML. Defaulting to option 1,
+             which is less dangerous than defaulting to the raw HTML. */
+          clazz = (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass;
+      }
       else if (!nsCRT::strcasecmp(content_type+5,		"enriched"))
         clazz = (MimeObjectClass *)&mimeInlineTextEnrichedClass;
       else if (!nsCRT::strcasecmp(content_type+5,		"richtext"))
         clazz = (MimeObjectClass *)&mimeInlineTextRichtextClass;
-      else if      (!nsCRT::strcasecmp(content_type+5,		"rtf"))
+      else if (!nsCRT::strcasecmp(content_type+5,		"rtf"))
         clazz = (MimeObjectClass *)&mimeExternalObjectClass;
       else if (!nsCRT::strcasecmp(content_type+5,		"plain"))
       {
@@ -330,12 +467,10 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
         clazz = (MimeObjectClass *)&mimeInlineTextPlainClass;
 
         PRBool disable_format_flowed = PR_FALSE;
-        nsIPref *pref = GetPrefServiceManager(opts); 
         if (pref)
-          (void)pref->GetBoolPref(
-                           "mailnews.display.disable_format_flowed_support",
-                           &disable_format_flowed);
-        
+          pref->GetBoolPref("mailnews.display.disable_format_flowed_support",
+                            &disable_format_flowed);
+
         if(!disable_format_flowed)
         {
           // Check for format=flowed, damn, it is already stripped away from
@@ -354,11 +489,11 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
           // a nsMimeTypes.h but that one isn't included. Bug?
           char *content_type_format =
             (content_type_row
-             ? MimeHeaders_get_parameter(content_type_row, "format", NULL, NULL)
+             ? MimeHeaders_get_parameter(content_type_row, "format", NULL,NULL)
              : 0);
 
           if (content_type_format && !nsCRT::strcasecmp(content_type_format,
-                                                           "flowed"))
+                                                        "flowed"))
             clazz = (MimeObjectClass *)&mimeInlineTextPlainFlowedClass;
           PR_FREEIF(content_type_format);
           PR_FREEIF(content_type_row);
@@ -379,7 +514,7 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
       else if (!nsCRT::strcasecmp(content_type+10,	"digest"))
         clazz = (MimeObjectClass *)&mimeMultipartDigestClass;
       else if (!nsCRT::strcasecmp(content_type+10,	"appledouble") ||
-        !nsCRT::strcasecmp(content_type+10,	"header-set"))
+               !nsCRT::strcasecmp(content_type+10,	"header-set"))
         clazz = (MimeObjectClass *)&mimeMultipartAppleDoubleClass;
       else if (!nsCRT::strcasecmp(content_type+10,	"parallel"))
         clazz = (MimeObjectClass *)&mimeMultipartParallelClass;
@@ -392,7 +527,7 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
         know about. */
         char *ct = (hdrs
           ? MimeHeaders_get(hdrs, HEADER_CONTENT_TYPE,
-										PR_FALSE, PR_FALSE)
+                            PR_FALSE, PR_FALSE)
                     : 0);
         char *proto = (ct
           ? MimeHeaders_get_parameter(ct, PARAM_PROTOCOL, NULL, NULL)
@@ -400,18 +535,20 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
         char *micalg = (ct
           ? MimeHeaders_get_parameter(ct, PARAM_MICALG, NULL, NULL)
           : 0);
-        
-          if (proto && (!nsCRT::strcasecmp(proto, APPLICATION_XPKCS7_SIGNATURE) &&
-                          micalg && (!nsCRT::strcasecmp(micalg, PARAM_MICALG_MD5) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_2) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_3) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_4) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_5) ||
-                                                 !nsCRT::strcasecmp(micalg, PARAM_MICALG_MD2))))
-                        clazz = (MimeObjectClass *)&mimeMultipartSignedCMSClass;
-                  else
-                        clazz = 0;          
+
+          if (proto
+              && (!nsCRT::strcasecmp(proto, APPLICATION_XPKCS7_SIGNATURE)
+                  && micalg
+                  && (!nsCRT::strcasecmp(micalg, PARAM_MICALG_MD5) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_2) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_3) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_4) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_SHA1_5) ||
+                      !nsCRT::strcasecmp(micalg, PARAM_MICALG_MD2))))
+            clazz = (MimeObjectClass *)&mimeMultipartSignedCMSClass;
+          else
+            clazz = 0;          
         PR_FREEIF(proto);
         PR_FREEIF(micalg);
         PR_FREEIF(ct);
@@ -442,44 +579,52 @@ mime_find_class (const char *content_type, MimeHeaders *hdrs,
     
     /* The magic image types which we are able to display internally...
     */
-    else if (!nsCRT::strcasecmp(content_type,			IMAGE_GIF)  ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_JPG) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_PJPG) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_PNG) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_XBM)  ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_XBM2) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_XBM3) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_BMP) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_ICO) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_MNG) ||
-      !nsCRT::strcasecmp(content_type,			IMAGE_JNG))
-      clazz = (MimeObjectClass *)&mimeInlineImageClass;
+    else if (!nsCRT::strncasecmp(content_type,		"image/", 6)) {
+        nsCOMPtr<imgILoader> loader(do_GetService("@mozilla.org/image/loader;1"));
+        PRBool isReg = PR_FALSE;
+        loader->SupportImageWithMimeType(content_type, &isReg);
+        if (isReg)
+          clazz = (MimeObjectClass *)&mimeInlineImageClass;
+        else
+          clazz = (MimeObjectClass *)&mimeExternalObjectClass;
+    }
     
 #ifdef ENABLE_SMIME
-    else if (!nsCRT::strcasecmp(content_type, APPLICATION_XPKCS7_MIME))
+    else if (!nsCRT::strcasecmp(content_type, APPLICATION_XPKCS7_MIME)
+             || !nsCRT::strcasecmp(content_type, APPLICATION_PKCS7_MIME))
 	        clazz = (MimeObjectClass *)&mimeEncryptedCMSClass;
 #endif
     /* A few types which occur in the real world and which we would otherwise
     treat as non-text types (which would be bad) without this special-case...
     */
     else if (!nsCRT::strcasecmp(content_type,			APPLICATION_PGP) ||
-    !nsCRT::strcasecmp(content_type,			APPLICATION_PGP2))
-    clazz = (MimeObjectClass *)&mimeInlineTextPlainClass;
-    
+             !nsCRT::strcasecmp(content_type,			APPLICATION_PGP2))
+      clazz = (MimeObjectClass *)&mimeInlineTextPlainClass;
+
     else if (!nsCRT::strcasecmp(content_type,			SUN_ATTACHMENT))
       clazz = (MimeObjectClass *)&mimeSunAttachmentClass;
     
-      /* Everything else gets represented as a clickable link.
+    /* Everything else gets represented as a clickable link.
     */
     else if (!exact_match_p)
-      clazz = (MimeObjectClass *)&mimeExternalObjectClass;  
+      clazz = (MimeObjectClass *)&mimeExternalObjectClass;
+
+    if (!mime_is_allowed_class(clazz, types_of_classes_to_disallow))
+    {
+      /* Do that check here (not after the if block), because we want to allow
+         user-installed plugins. */
+      if(!exact_match_p)
+        clazz = (MimeObjectClass *)&mimeExternalObjectClass;
+      else
+        clazz = 0;
+    }
   }
 
- if (!exact_match_p)
-   PR_ASSERT(clazz);
+  if (!exact_match_p)
+    NS_ASSERTION(clazz, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
   if (!clazz) return 0;
 
-  PR_ASSERT(clazz);
+  NS_ASSERTION(clazz, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 
   if (clazz && !clazz->class_initialized)
 	{
@@ -574,7 +719,7 @@ mime_create (const char *content_type, MimeHeaders *hdrs,
 
   clazz = mime_find_class(content_type, hdrs, opts, PR_FALSE);
 
-  PR_ASSERT(clazz);
+  NS_ASSERTION(clazz, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
   if (!clazz) goto FAIL;
 
   if (opts && opts->part_to_load)
@@ -618,6 +763,8 @@ mime_create (const char *content_type, MimeHeaders *hdrs,
           (clazz != (MimeObjectClass *)&mimeInlineTextPlainClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextPlainFlowedClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextHTMLClass) &&
+          (clazz != (MimeObjectClass *)&mimeInlineTextHTMLSanitizedClass) &&
+          (clazz != (MimeObjectClass *)&mimeInlineTextHTMLAsPlaintextClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextRichtextClass) &&
           (clazz != (MimeObjectClass *)&mimeInlineTextEnrichedClass) &&
           (clazz != (MimeObjectClass *)&mimeMessageClass) &&
@@ -661,7 +808,7 @@ mime_classinit(MimeObjectClass *clazz)
   if (clazz->class_initialized)
 	return 0;
 
-  PR_ASSERT(clazz->class_initialize);
+  NS_ASSERTION(clazz->class_initialize, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
   if (!clazz->class_initialize)
 	return -1;
 
@@ -734,8 +881,8 @@ mime_part_address(MimeObject *obj)
 	  char buf [20];
 	  char *higher = 0;
 	  MimeContainer *cont = (MimeContainer *) obj->parent;
-	  PR_ASSERT(mime_typep(obj->parent,
-						   (MimeObjectClass *)&mimeContainerClass));
+	  NS_ASSERTION(mime_typep(obj->parent,
+                    (MimeObjectClass *)&mimeContainerClass), "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 	  for (i = 0; i < cont->nchildren; i++)
 		if (cont->children[i] == obj)
 		  {
@@ -744,7 +891,7 @@ mime_part_address(MimeObject *obj)
 		  }
 	  if (j == -1)
 		{
-		  PR_ASSERT(0);
+		  NS_ASSERTION(0, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 		  return 0;
 		}
 
@@ -759,7 +906,7 @@ mime_part_address(MimeObject *obj)
 		return nsCRT::strdup(buf);
 	  else
 		{
-		  char *s = (char *)PR_MALLOC(nsCRT::strlen(higher) + nsCRT::strlen(buf) + 3);
+		  char *s = (char *)PR_MALLOC(strlen(higher) + strlen(buf) + 3);
 		  if (!s)
 			{
 			  PR_Free(higher);
@@ -863,7 +1010,7 @@ mime_get_crypto_state (MimeObject *obj,
   if (signed_ok_ret) *signed_ok_ret = PR_FALSE;
   if (encrypted_ok_ret) *encrypted_ok_ret = PR_FALSE;
 
-  PR_ASSERT(obj);
+  NS_ASSERTION(obj, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
   if (!obj) return;
 
   if (!mime_typep (obj, (MimeObjectClass *) &mimeMessageClass))
@@ -924,7 +1071,7 @@ mime_set_crypto_stamp(MimeObject *obj, PRBool signed_p, PRBool encrypted_p)
 		  obj->options->state)
 		{
 		  /* decrypt_p and write_html_p are incompatible. */
-		  PR_ASSERT(!obj->options->write_html_p);
+		  NS_ASSERTION(!obj->options->write_html_p, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 		  obj->options->state->decrypted_p = PR_TRUE;
     }
 
@@ -987,7 +1134,7 @@ mime_find_security_info_of_part(const char *part, MimeObject *obj,
 	  MimeContainer *cont = (MimeContainer *) obj;
 	  if (cont->nchildren >= 1)
 		{
-		  PR_ASSERT(cont->nchildren == 1);
+		  NS_ASSERTION(cont->nchildren == 1, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 		  obj = cont->children[0];
 		}
 	}
@@ -1031,8 +1178,8 @@ mime_find_security_info_of_part(const char *part, MimeObject *obj,
 		*verify_error_return = verify_error;
 
 
-	  PR_ASSERT(mime_typep(obj, (MimeObjectClass *) &mimeContainerClass) &&
-				((MimeContainer *) obj)->nchildren <= 1);
+	  NS_ASSERTION(mime_typep(obj, (MimeObjectClass *) &mimeContainerClass) &&
+				((MimeContainer *) obj)->nchildren <= 1, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 
 	  obj = ((((MimeContainer *) obj)->nchildren > 0)
 			 ? ((MimeContainer *) obj)->children[0]
@@ -1077,7 +1224,7 @@ mime_set_url_part(const char *url, const char *part, PRBool append_p)
 		}
 	}
 
-  result = (char *) PR_MALLOC(nsCRT::strlen(url) + nsCRT::strlen(part) + 10);
+  result = (char *) PR_MALLOC(strlen(url) + strlen(part) + 10);
   if (!result) return 0;
 
   if (part_begin)
@@ -1110,7 +1257,7 @@ mime_set_url_part(const char *url, const char *part, PRBool append_p)
 
   /* Semi-broken kludge to omit a trailing "?part=0". */
   {
-	int L = nsCRT::strlen(result);
+	int L = strlen(result);
 	if (L > 6 &&
 		(result[L-7] == '?' || result[L-7] == '&') &&
 		!nsCRT::strcmp("part=0", result + L - 6))
@@ -1135,7 +1282,7 @@ mime_set_url_imap_part(const char *url, const char *imappart, const char *libmim
 	  *whereCurrent = 0;
   }
 	
-  result = (char *) PR_MALLOC(nsCRT::strlen(url) + nsCRT::strlen(imappart) + nsCRT::strlen(libmimepart) + 17);
+  result = (char *) PR_MALLOC(strlen(url) + strlen(imappart) + strlen(libmimepart) + 17);
   if (!result) return 0;
 
   PL_strcpy(result, url);
@@ -1143,7 +1290,7 @@ mime_set_url_imap_part(const char *url, const char *imappart, const char *libmim
   PL_strcat(result, imappart);
   PL_strcat(result, "&part=");
   PL_strcat(result, libmimepart);
-  result[nsCRT::strlen(result)] = 0;
+  result[strlen(result)] = 0;
 
   if (whereCurrent)
 	  *whereCurrent = '/';
@@ -1274,7 +1421,7 @@ mime_find_suggested_name_of_part(const char *part, MimeObject *obj)
    */
   if (result && obj->encoding && *obj->encoding)
 	{
-	  PRInt32 L = nsCRT::strlen(result);
+	  PRInt32 L = strlen(result);
 	  const char **exts = 0;
 
 	  /*
@@ -1296,7 +1443,7 @@ mime_find_suggested_name_of_part(const char *part, MimeObject *obj)
 	  while (exts && *exts)
 		{
 		  const char *ext = *exts;
-		  PRInt32 L2 = nsCRT::strlen(ext);
+		  PRInt32 L2 = strlen(ext);
 		  if (L > L2 + 1 &&							/* long enough */
 			  result[L - L2 - 1] == '.' &&			/* '.' in right place*/
 			  !nsCRT::strcasecmp(ext, result + (L - L2)))	/* ext matches */
@@ -1454,8 +1601,8 @@ mime_parse_url_options(const char *url, MimeDisplayOptions *options)
 	  else if (nsCRT::strcmp(options->part_to_load, "1"))	/* not 1 */
 		{
 		  const char *prefix = "1.";
-		  char *s = (char *) PR_MALLOC(nsCRT::strlen(options->part_to_load) +
-									  nsCRT::strlen(prefix) + 1);
+		  char *s = (char *) PR_MALLOC(strlen(options->part_to_load) +
+		                               strlen(prefix) + 1);
 		  if (!s) return MIME_OUT_OF_MEMORY;
 		  PL_strcpy(s, prefix);
 		  PL_strcat(s, options->part_to_load);
@@ -1494,7 +1641,7 @@ MimeOptions_write(MimeDisplayOptions *opt, char *data, PRInt32 length,
 	  else
 		{
 		  char sep[] = "<BR><HR WIDTH=\"90%\" SIZE=4><BR>";
-		  int lstatus = opt->output_fn(sep, nsCRT::strlen(sep), closure);
+		  int lstatus = opt->output_fn(sep, strlen(sep), closure);
 		  opt->state->separator_suppressed_p = PR_FALSE;
 		  if (lstatus < 0) return lstatus;
 		}
@@ -1521,7 +1668,7 @@ MimeObject_write(MimeObject *obj, char *output, PRInt32 length,
 	{
 	  int status = MimeObject_output_init(obj, 0);
 	  if (status < 0) return status;
-	  PR_ASSERT(obj->options->state->first_data_written_p);
+	  NS_ASSERTION(obj->options->state->first_data_written_p, "1.1 <rhp@netscape.com> 19 Mar 1999 12:00");
 	}
 
   return MimeOptions_write(obj->options, output, length, user_visible_p);
@@ -1577,8 +1724,12 @@ MimeObject_output_init(MimeObject *obj, const char *content_type)
         }
 			  
         if (!(obj->options->override_charset)) {
-          PR_FREEIF(obj->options->default_charset);
-          obj->options->default_charset = MimeHeaders_get_parameter(ct, "charset", nsnull, nsnull);
+          char *charset = MimeHeaders_get_parameter(ct, "charset", nsnull, nsnull);
+          if (charset)
+          {
+            PR_FREEIF(obj->options->default_charset);
+            obj->options->default_charset = charset;
+          }
         }
 			  PR_Free(ct);
 			}
