@@ -35,9 +35,9 @@
  * the terms of any one of the NPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
+#include "prlog.h"
 #include "nsCOMPtr.h"
 #include "modlmime.h"
-#include "xp_core.h"
 #include "nsCRT.h"
 #include "mimeobj.h"
 #include "mimemsg.h"
@@ -86,10 +86,26 @@
 #include "nsITransport.h"
 #include "mimeebod.h"
 #include "mimeeobj.h"
+// <for functions="HTML2Plaintext,HTMLSantinize">
+#include "nsXPCOM.h"
+#include "nsParserCIID.h"
+#include "nsIParser.h"
+#include "nsIHTMLContentSink.h"
+#include "nsIContentSerializer.h"
+#include "nsLayoutCID.h"
+#include "nsIComponentManager.h"
+#include "nsReadableUtils.h"
+#include "nsIHTMLToTextSink.h"
+#include "mozISanitizingSerializer.h"
+// </for>
 
 
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
+// <for functions="HTML2Plaintext,HTMLSantinize">
+static NS_DEFINE_CID(kParserCID, NS_PARSER_CID);
+static NS_DEFINE_CID(kNavDTDCID, NS_CNAVDTD_CID);
+// </for>
 
 #ifdef HAVE_MIME_DATA_SLOT
 #define LOCK_LAST_CACHED_MESSAGE
@@ -131,7 +147,7 @@ ProcessBodyAsAttachment(MimeObject *obj, nsMsgAttachmentData **data)
     return NS_ERROR_OUT_OF_MEMORY;
 
   tmp = *data;
-  nsCRT::memset(*data, 0, (n + 1) * sizeof(nsMsgAttachmentData));
+  memset(*data, 0, (n + 1) * sizeof(nsMsgAttachmentData));
   tmp->real_type = child->content_type ? nsCRT::strdup(child->content_type) : NULL;
   tmp->real_encoding = child->encoding ? nsCRT::strdup(child->encoding) : NULL;
   disp = MimeHeaders_get(child->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
@@ -240,13 +256,11 @@ ValidateRealName(nsMsgAttachmentData *aAttach, MimeHeaders *aHdrs)
 
   // Special case...if this is a enclosed RFC822 message, give it a nice
   // name.
-  if (aAttach->real_type && !nsCRT::strcasecmp(aAttach->real_type, MESSAGE_RFC822) && 
-     (!aAttach->real_name || *aAttach->real_name == 0))
+  if (aAttach->real_type && !nsCRT::strcasecmp(aAttach->real_type, MESSAGE_RFC822))
   {
-    if (aHdrs->munged_subject)
-    {
+    NS_ASSERTION(aHdrs, "How comes the object's headers is null!");
+    if (aHdrs && aHdrs->munged_subject)
       aAttach->real_name = PR_smprintf("%s.eml", aHdrs->munged_subject);
-    }
     else
       NS_MsgSACopy(&(aAttach->real_name), "ForwardedMessage.eml");
     return;
@@ -289,106 +303,106 @@ ValidateRealName(nsMsgAttachmentData *aAttach, MimeHeaders *aHdrs)
 static  PRInt32     attIndex = 0;
 
 nsresult
-BuildAttachmentList(MimeObject *aChild, nsMsgAttachmentData *aAttachData,
-                    const char *aMessageURL, PRBool addExternalBodiesOnly)
+GenerateAttachmentData(MimeObject *object, const char *aMessageURL, MimeDisplayOptions *options,
+                       PRBool isAnAppleDoublePart, nsMsgAttachmentData *aAttachData)
 {
-  char                  *disp;
-  PRInt32               i;
-  PRInt32               j;
-  MimeContainer         *cobj = (MimeContainer *) aChild;
-  nsMsgAttachmentData   *tmp = nsnull;
-  PRBool                isAlternativeOrRelated;
-  PRBool                isIMAPPart = PR_FALSE;
-  char                  *charset = nsnull;
+  nsXPIDLCString imappart;
+  nsXPIDLCString part;
+  PRBool isIMAPPart;
 
-  if ( (!aChild) || (!cobj->children) || 
-       (mime_typep(aChild, (MimeObjectClass *)&mimeExternalBodyClass)))
+  /* be sure the object has not be marked as Not to be an attachment */
+  if (object->dontShowAsAttachment)
     return NS_OK;
 
-  if (MimeObjectChildIsMessageBody(aChild, &isAlternativeOrRelated))
-    i = 1;
-  else
-    i = 0;
+  part.Adopt(mime_part_address(object));
+  if (part.IsEmpty()) 
+    return NS_ERROR_OUT_OF_MEMORY;
 
-  for (; i<cobj->nchildren ; i++) 
+  if (options->missing_parts)
+    imappart.Adopt(mime_imap_part_address(object));
+
+  char *urlSpec = nsnull;
+  if (!imappart.IsEmpty())
   {
-    MimeObject    *child = cobj->children[i];
-    nsXPIDLCString imappart;
-    nsXPIDLCString part;
+    isIMAPPart = PR_TRUE;
+    urlSpec = mime_set_url_imap_part(aMessageURL, imappart.get(), part.get());
+  }
+  else
+  {
+    isIMAPPart = PR_FALSE;
+    urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
+  }
 
-    /*
-      if we are processing an inline RFC822 message, we should skip its childern
-    */
-    PRBool isAnInlineMessage = mime_typep(child, (MimeObjectClass *) &mimeMessageClass);
-    
-    /*
-      AppleDouble part need special care: we need to fetch the part as well it's two
-      children for the needed info as they could be anywhere, eventually, they won't contain
-      a name or file name. In any case we need to build only one attachment data
-    */
-    PRBool isAnAppleDoublePart = mime_typep(child, (MimeObjectClass *) &mimeMultipartAppleDoubleClass) &&
-                                 ((MimeContainer *)child)->nchildren == 2;
-    
-    PRBool fetchExternalBodiesFromChildren = addExternalBodiesOnly; // inherit from our parent.
-    // whenever we encounter an inlined message, we want to only fetch external bodies from that point
-    // forward
-    if (!addExternalBodiesOnly && isAnInlineMessage)
-      fetchExternalBodiesFromChildren = PR_TRUE;  
+  if (!urlSpec)
+    return NS_ERROR_OUT_OF_MEMORY;
 
-    if (!isAnAppleDoublePart /* && !isAnInlineMessage */)
-      if ( NS_FAILED(BuildAttachmentList((MimeObject *)child, aAttachData, aMessageURL, fetchExternalBodiesFromChildren)) )
-        return NS_OK;
+  nsMsgAttachmentData *tmp = &(aAttachData[attIndex++]);
+  nsresult rv = nsMimeNewURI(&(tmp->url), urlSpec, nsnull);
 
-    // okay, if the current recursive call only wants us to list external body attachments
-    // then kick out of this method if we are an inlined part and our child class isn't an 
-    // external body or an apple double attachment!
-    
-    if (addExternalBodiesOnly && !mime_typep((MimeObject *)child, (MimeObjectClass *)&mimeExternalObjectClass) && !isAnAppleDoublePart )
-        return NS_OK;
+	PR_FREEIF(urlSpec);
 
-    part.Adopt(mime_part_address(child));
-    if (part.IsEmpty()) 
-      return NS_ERROR_OUT_OF_MEMORY;
+  if ( (NS_FAILED(rv)) || (!tmp->url) )
+    return NS_ERROR_OUT_OF_MEMORY;
 
-    if (aChild->options->missing_parts)
-      imappart.Adopt(mime_imap_part_address(child));
-
-    char *urlSpec = nsnull;
-    if (!imappart.IsEmpty())
-    {
-      isIMAPPart = PR_TRUE;
-      urlSpec = mime_set_url_imap_part(aMessageURL, imappart.get(), part.get());
-    }
-    else
-      urlSpec = mime_set_url_part(aMessageURL, part.get(), PR_TRUE);
+  tmp->real_type = object->content_type ? nsCRT::strdup(object->content_type) : nsnull;
+  tmp->real_encoding = object->encoding ? nsCRT::strdup(object->encoding) : nsnull;
   
-    if (!urlSpec)
-      return NS_ERROR_OUT_OF_MEMORY;
+  PRInt32 i;
+  char *charset = nsnull;
+  char *disp = MimeHeaders_get(object->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
+  if (disp) 
+  {
+    tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, nsnull);
+    if (isAnAppleDoublePart)
+      for (i = 0; i < 2 && !tmp->real_name; i ++)
+      {
+        PR_FREEIF(disp);
+        PR_FREEIF(charset);
+        disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
+        tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, nsnull);
+      }
 
-    tmp = &(aAttachData[attIndex++]);
-    nsresult rv = nsMimeNewURI(&(tmp->url), urlSpec, nsnull);
-
-	  PR_FREEIF(urlSpec);
-
-    if ( (NS_FAILED(rv)) || (!tmp->url) )
-      return NS_ERROR_OUT_OF_MEMORY;
-
-    tmp->real_type = child->content_type ? nsCRT::strdup(child->content_type) : NULL;
-    tmp->real_encoding = child->encoding ? nsCRT::strdup(child->encoding) : NULL;
-    disp = MimeHeaders_get(child->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
-
-    if (disp) 
+    if (tmp->real_name)
     {
-      tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, NULL);
+      // check encoded type
+      //
+      // The parameter of Content-Disposition must use RFC 2231.
+      // But old Netscape 4.x and Outlook Express etc. use RFC2047.
+      // So we should parse both types.
+
+      char *fname = nsnull;
+      fname = mime_decode_filename(tmp->real_name, charset, options);
+      PR_FREEIF(charset);
+
+      if (fname && fname != tmp->real_name)
+      {
+        PR_FREEIF(tmp->real_name);
+        tmp->real_name = fname;
+      }
+    }
+
+    PR_FREEIF(disp);
+  }
+
+  disp = MimeHeaders_get(object->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
+  if (disp)
+  {
+    tmp->x_mac_type   = MimeHeaders_get_parameter(disp, PARAM_X_MAC_TYPE, nsnull, nsnull);
+    tmp->x_mac_creator= MimeHeaders_get_parameter(disp, PARAM_X_MAC_CREATOR, nsnull, nsnull);
+    
+    if (!tmp->real_name || *tmp->real_name == 0)
+    {
+      PR_FREEIF(tmp->real_name);
+      tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, nsnull);
       if (isAnAppleDoublePart)
-        for (j = 0; j < 2 && !tmp->real_name; j ++)
+        for (i = 0; i < 2 && !tmp->real_name; i ++)
         {
           PR_FREEIF(disp);
           PR_FREEIF(charset);
-          disp = MimeHeaders_get(((MimeContainer *)child)->children[j]->headers, HEADER_CONTENT_DISPOSITION, PR_FALSE, PR_FALSE);
-          tmp->real_name = MimeHeaders_get_parameter(disp, "filename", &charset, NULL);
+          disp = MimeHeaders_get(((MimeContainer *)object)->children[i]->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
+          tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, nsnull);
         }
-
+      
       if (tmp->real_name)
       {
         // check encoded type
@@ -397,93 +411,115 @@ BuildAttachmentList(MimeObject *aChild, nsMsgAttachmentData *aAttachData,
         // But old Netscape 4.x and Outlook Express etc. use RFC2047.
         // So we should parse both types.
 
-        char *fname = NULL;
-        fname = mime_decode_filename(tmp->real_name, charset, aChild->options);
+        char *fname = nsnull;
+        fname = mime_decode_filename(tmp->real_name, charset, options);
         PR_FREEIF(charset);
 
         if (fname && fname != tmp->real_name)
         {
-          PR_FREEIF(tmp->real_name);
+          PR_Free(tmp->real_name);
           tmp->real_name = fname;
         }
       }
-
-      PR_FREEIF(disp);
     }
+    PR_FREEIF(disp);
+  }
 
-    disp = MimeHeaders_get(child->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
-    if (disp)
+  tmp->description = MimeHeaders_get(object->headers, HEADER_CONTENT_DESCRIPTION, 
+                                     PR_FALSE, PR_FALSE);
+
+  // Now, do the right thing with the name!
+  if (!tmp->real_name && nsCRT::strcasecmp(tmp->real_type, MESSAGE_RFC822))
+  {
+    /* If this attachment doesn't have a name, just give it one... */
+    tmp->real_name = MimeGetStringByID(MIME_MSG_DEFAULT_ATTACHMENT_NAME);
+    if (tmp->real_name)
     {
-      tmp->x_mac_type   = MimeHeaders_get_parameter(disp, PARAM_X_MAC_TYPE, NULL, NULL);
-      tmp->x_mac_creator= MimeHeaders_get_parameter(disp, PARAM_X_MAC_CREATOR, NULL, NULL);
-      
-      if (!tmp->real_name || *tmp->real_name == 0)
+      char *newName = PR_smprintf(tmp->real_name, part.get());
+      if (newName)
       {
-        PR_FREEIF(tmp->real_name);
-        tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, NULL);
-        if (isAnAppleDoublePart)
-          for (j = 0; j < 2 && !tmp->real_name; j ++)
-          {
-            PR_FREEIF(disp);
-            PR_FREEIF(charset);
-            disp = MimeHeaders_get(((MimeContainer *)child)->children[j]->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE);
-            tmp->real_name = MimeHeaders_get_parameter(disp, "name", &charset, NULL);
-          }
-        
-        if (tmp->real_name)
-        {
-          // check encoded type
-          //
-          // The parameter of Content-Disposition must use RFC 2231.
-          // But old Netscape 4.x and Outlook Express etc. use RFC2047.
-          // So we should parse both types.
-
-          char *fname = NULL;
-          fname = mime_decode_filename(tmp->real_name, charset, 
-                                       aChild->options);
-          PR_FREEIF(charset);
-
-          if (fname && fname != tmp->real_name)
-          {
-            PR_Free(tmp->real_name);
-            tmp->real_name = fname;
-          }
-        }
+        PR_Free(tmp->real_name);
+        tmp->real_name = newName;
       }
-      PR_FREEIF(disp);
     }
+    else
+      tmp->real_name = mime_part_address(object);
+  }
+  ValidateRealName(tmp, object->headers);
 
-    tmp->description = MimeHeaders_get(child->headers, HEADER_CONTENT_DESCRIPTION, 
-                                       PR_FALSE, PR_FALSE);
+  if (isIMAPPart)
+  {
+    // If we get here, we should mark this attachment as not being
+    // downloaded. 
+    tmp->notDownloaded = PR_TRUE;
+  }
 
-    // Now, do the right thing with the name!
-    if (!tmp->real_name)
-    {
-      /* If this attachment doesn't have a name, just give it one... */
-      tmp->real_name = MimeGetStringByID(MIME_MSG_DEFAULT_ATTACHMENT_NAME);
-      if (tmp->real_name)
-      {
-        char *newName = PR_smprintf(tmp->real_name, part.get());
-        if (newName)
+  return NS_OK;
+}
+
+nsresult
+BuildAttachmentList(MimeObject *anObject, nsMsgAttachmentData *aAttachData, const char *aMessageURL)
+{
+  nsresult              rv;
+  PRInt32               i;
+  MimeContainer         *cobj = (MimeContainer *) anObject;
+
+  if ( (!anObject) || (!cobj->children) || (!cobj->nchildren) ||
+       (mime_typep(anObject, (MimeObjectClass *)&mimeExternalBodyClass)))
+    return NS_OK;
+
+  for (i = 0; i < cobj->nchildren ; i++) 
+  {
+    MimeObject    *child = cobj->children[i];
+
+    // Skip the first child if it's in fact a message body
+    if (i == 0)                                         // it's the first child
+      if (child->content_type)                          // and it's content-type is one of folowing...
+        if (!nsCRT::strcasecmp (child->content_type, TEXT_PLAIN) ||
+            !nsCRT::strcasecmp (child->content_type, TEXT_HTML) ||
+            !nsCRT::strcasecmp (child->content_type, TEXT_MDL) ||
+            !nsCRT::strcasecmp (child->content_type, MULTIPART_ALTERNATIVE) ||
+            !nsCRT::strcasecmp (child->content_type, MULTIPART_RELATED))
         {
-          PR_Free(tmp->real_name);
-          tmp->real_name = newName;
+          if (child->headers) // and finally, be sure it doesn't have a content-disposition: attachment 
+          {
+            char * disp = MimeHeaders_get (child->headers, HEADER_CONTENT_DISPOSITION, PR_TRUE, PR_FALSE);
+            if (!disp || nsCRT::strcasecmp (disp, "attachment"))
+              continue;
+          }
+          else
+            continue;
         }
-      }
-      else
-        tmp->real_name = mime_part_address(child);
-    }
-    ValidateRealName(tmp, child->headers);
 
-    if (isIMAPPart)
+    
+    // We should generate an attachment for leaf object only but...
+    PRBool isALeafObject = mime_subclass_p(child->clazz, (MimeObjectClass *) &mimeLeafClass);
+
+    // ...we will generate an attachment for inline message too.
+    PRBool isAnInlineMessage = mime_typep(child, (MimeObjectClass *) &mimeMessageClass);
+    
+    // AppleDouble part need special care: we need to fetch the part as well its two
+    // children for the needed info as they could be anywhere, eventually, they won't contain
+    // a name or file name. In any case we need to build only one attachment data
+    PRBool isAnAppleDoublePart = mime_typep(child, (MimeObjectClass *) &mimeMultipartAppleDoubleClass) &&
+                                 ((MimeContainer *)child)->nchildren == 2;
+
+    if (isALeafObject || isAnInlineMessage || isAnAppleDoublePart)
     {
-      // If we get here, we should mark this attachment as not being
-      // downloaded. 
-      tmp->notDownloaded = PR_TRUE;
+      rv = GenerateAttachmentData(child, aMessageURL, anObject->options, isAnAppleDoublePart, aAttachData);
+      NS_ENSURE_SUCCESS(rv, rv);
+    }
+    
+    // Now build the attachment list for the children of our object...
+    if (!isALeafObject && !isAnAppleDoublePart)
+    {
+      rv = BuildAttachmentList((MimeObject *)child, aAttachData, aMessageURL);
+      NS_ENSURE_SUCCESS(rv, rv);
     }
   }
 
   return NS_OK;
+
 }
 
 extern "C" nsresult
@@ -493,13 +529,17 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
   MimeContainer         *cobj;
   PRInt32               n;
   PRBool                isAlternativeOrRelated = PR_FALSE;
+  PRBool                isAnInlineMessage;
 
   if (!data) 
     return 0;
-  *data = NULL;
+  *data = nsnull;
 
   obj = mime_get_main_object(tobj);
-  if ( (obj) && (!mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass)) )
+  if (!obj)
+    return 0;
+
+  if (!mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass))
   {
     if (!PL_strcasecmp(obj->content_type, MESSAGE_RFC822))
       return 0;
@@ -507,24 +547,38 @@ MimeGetAttachmentList(MimeObject *tobj, const char *aMessageURL, nsMsgAttachment
       return ProcessBodyAsAttachment(obj, data);
   }
 
-  (void) MimeObjectChildIsMessageBody(obj, &isAlternativeOrRelated);
-  if (isAlternativeOrRelated)
+  if (mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeMultipartAlternativeClass))
     return 0;
   
+  isAnInlineMessage = mime_typep(obj, (MimeObjectClass *) &mimeMessageClass);
+
   cobj = (MimeContainer*) obj;
   n = CountTotalMimeAttachments(cobj);
   if (n <= 0) 
     return n;
 
+  //in case of an inline message (as body), we need an extra slot for the message itself
+  //that we will fill later...
+  if (isAnInlineMessage)
+    n ++;
+
   *data = (nsMsgAttachmentData *)PR_Malloc( (n + 1) * sizeof(nsMsgAttachmentData));
   if (!*data) 
     return NS_ERROR_OUT_OF_MEMORY;
 
   attIndex = 0;
-  nsCRT::memset(*data, 0, (n + 1) * sizeof(nsMsgAttachmentData));
+  memset(*data, 0, (n + 1) * sizeof(nsMsgAttachmentData));
   
   // Now, build the list!
-  return BuildAttachmentList((MimeObject *) cobj, *data, aMessageURL, PR_FALSE);
+
+  nsresult rv;
+
+  if (isAnInlineMessage)
+  {
+    rv = GenerateAttachmentData(obj, aMessageURL, obj->options, PR_FALSE, *data);
+    NS_ENSURE_SUCCESS(rv, rv);
+  }
+  return BuildAttachmentList((MimeObject *) cobj, *data, aMessageURL);
 }
 
 extern "C" void
@@ -564,8 +618,6 @@ NotifyEmittersOfAttachmentList(MimeDisplayOptions     *opt,
 
   while (tmp->url)
   {
-    char  *spec;
-
     if (!tmp->real_name)
     {
       ++i;
@@ -573,15 +625,12 @@ NotifyEmittersOfAttachmentList(MimeDisplayOptions     *opt,
       continue;
     }
 
-    spec = nsnull;
+    nsCAutoString spec;
     if ( tmp->url ) 
-      tmp->url->GetSpec(&spec);
+      tmp->url->GetSpec(spec);
 
-    mimeEmitterStartAttachment(opt, tmp->real_name, tmp->real_type, spec, tmp->notDownloaded);
-    mimeEmitterAddAttachmentField(opt, HEADER_X_MOZILLA_PART_URL, spec);
-
-	  if (spec)
-		  nsMemory::Free(spec);
+    mimeEmitterStartAttachment(opt, tmp->real_name, tmp->real_type, spec.get(), tmp->notDownloaded);
+    mimeEmitterAddAttachmentField(opt, HEADER_X_MOZILLA_PART_URL, spec.get());
 
     if ( (opt->format_out == nsMimeOutput::nsMimeMessageQuoting) || 
          (opt->format_out == nsMimeOutput::nsMimeMessageBodyQuoting) || 
@@ -625,7 +674,7 @@ nsMimeNewURI(nsIURI** aInstancePtrResult, const char *aSpec, nsIURI *aBase)
   if (NS_FAILED(res)) 
     return NS_ERROR_FACTORY_NOT_REGISTERED;
 
-  return pService->NewURI(aSpec, aBase, aInstancePtrResult);
+  return pService->NewURI(nsDependentCString(aSpec), nsnull, aBase, aInstancePtrResult);
 }
 
 extern "C" nsresult 
@@ -886,8 +935,6 @@ mime_display_stream_complete (nsMIMESession *stream)
     //
     if (!msd->options->part_to_load)
     {
-      extern void  mime_dump_attachments ( nsMsgAttachmentData *attachData );
-
       nsMsgAttachmentData *attachments;
       nsresult rv = MimeGetAttachmentList(obj, msd->url_name, &attachments);
       if (NS_SUCCEEDED(rv))
@@ -1067,9 +1114,6 @@ mime_image_begin(const char *image_url, const char *content_type,
         mailUrl->GetImageCacheSession(getter_AddRefs(memCacheSession));
         if (memCacheSession)
         {
-          nsXPIDLCString spec;
-
-          uri->GetSpec(getter_Copies(spec));
           nsCOMPtr<nsICacheEntryDescriptor> entry;
 
           // we may need to convert the image_url into just a part url - in any case,
@@ -1148,8 +1192,8 @@ mime_image_make_image_html(void *image_closure)
   else
     url = mid->url;
 
-  buf = (char *) PR_MALLOC (nsCRT::strlen(prefix) + nsCRT::strlen(suffix) +
-                           nsCRT::strlen(url) + 20) ;
+  buf = (char *) PR_MALLOC (strlen(prefix) + strlen(suffix) +
+                            strlen(url) + 20) ;
   if (!buf) 
     return 0;
   *buf = 0;
@@ -1184,27 +1228,6 @@ mime_image_write_buffer(char *buf, PRInt32 size, void *image_closure)
   return size;
 }
 
-// 
-// Utility for finding HTML part.
-//
-static MimeObject*
-mime_find_text_html_part_1(MimeObject* obj)
-{
-  if (mime_subclass_p(obj->clazz,
-                      (MimeObjectClass*) &mimeInlineTextHTMLClass)) {
-    return obj;
-  }
-  if (mime_subclass_p(obj->clazz, (MimeObjectClass*) &mimeContainerClass)) {
-    MimeContainer* cobj = (MimeContainer*) obj;
-    PRInt32 i;
-    for (i=0 ; i<cobj->nchildren ; i++) {
-      MimeObject* result = mime_find_text_html_part_1(cobj->children[i]);
-      if (result) return result;
-    }
-  }
-  return NULL;
-}
-
 MimeObject*
 mime_get_main_object(MimeObject* obj)
 {
@@ -1237,7 +1260,7 @@ mime_get_main_object(MimeObject* obj)
         obj = nsnull;
     }
   }
-  return NULL;
+  return nsnull;
 }
 
 PRBool MimeObjectChildIsMessageBody(MimeObject *obj, 
@@ -1291,8 +1314,8 @@ PRBool MimeObjectChildIsMessageBody(MimeObject *obj,
 			 !nsCRT::strcasecmp (firstChild->content_type, TEXT_MDL) ||
 			 !nsCRT::strcasecmp (firstChild->content_type, MULTIPART_ALTERNATIVE) ||
 			 !nsCRT::strcasecmp (firstChild->content_type, MULTIPART_RELATED) ||
-			 !nsCRT::strcasecmp (firstChild->content_type, MESSAGE_NEWS) ||
-			 !nsCRT::strcasecmp (firstChild->content_type, MESSAGE_RFC822))
+       !nsCRT::strcasecmp (firstChild->content_type, MESSAGE_NEWS) ||
+       !nsCRT::strcasecmp (firstChild->content_type, MESSAGE_RFC822))
 		bRet = PR_TRUE;
 	else
 		bRet = PR_FALSE;
@@ -1438,17 +1461,17 @@ mime_bridge_create_display_stream(
   msd->firstCheck = PR_TRUE;
 
   // Store the URL string for this decode operation
-  char *urlString;
+  nsCAutoString urlString;
   nsresult rv;
 
   // Keep a hold of the channel...
   msd->channel = aChannel;
-  rv = uri->GetSpec(&urlString);
+  rv = uri->GetSpec(urlString);
   if (NS_SUCCEEDED(rv))
   {
-    if ((urlString) && (*urlString))
+    if (!urlString.IsEmpty())
     {
-      msd->url_name = nsCRT::strdup(urlString);
+      msd->url_name = ToNewCString(urlString);
       if (!(msd->url_name))
       {
         PR_FREEIF(msd);
@@ -1457,7 +1480,6 @@ mime_bridge_create_display_stream(
       nsCOMPtr<nsIMsgMessageUrl> msgUrl = do_QueryInterface(uri);
       if (msgUrl)
           msgUrl->GetOriginalSpec(&msd->orig_url_name);
-      PR_FREEIF(urlString);
     }
   }
   
@@ -1500,7 +1522,6 @@ mime_bridge_create_display_stream(
 		case nsMimeOutput::nsMimeMessageSplitDisplay:   // the wrapper HTML output to produce the split header/body display
 		case nsMimeOutput::nsMimeMessageHeaderDisplay:  // the split header/body display
 		case nsMimeOutput::nsMimeMessageBodyDisplay:    // the split header/body display
-    case nsMimeOutput::nsMimeMessageXULDisplay:
       msd->options->fancy_headers_p = PR_TRUE;
       msd->options->output_vcard_buttons_p = PR_TRUE;
       msd->options->fancy_links_p = PR_TRUE;
@@ -1618,7 +1639,7 @@ mime_bridge_create_display_stream(
     return 0;
   }
   
-  nsCRT::memset (stream, 0, sizeof (*stream));  
+  memset (stream, 0, sizeof (*stream));  
   stream->name           = "MIME Conversion Stream";
   stream->complete       = mime_display_stream_complete;
   stream->abort          = mime_display_stream_abort;
@@ -1713,6 +1734,26 @@ mimeEmitterAddHeaderField(MimeDisplayOptions *opt, const char *field, const char
   return NS_ERROR_FAILURE;
 }
 
+extern "C" nsresult     
+mimeEmitterAddAllHeaders(MimeDisplayOptions *opt, const char *allheaders, const PRInt32 allheadersize)
+{
+  // Check for draft processing...
+  if (NoEmitterProcessing(opt->format_out))
+    return NS_OK;
+
+  mime_stream_data  *msd = GetMSD(opt);
+  if (!msd) 
+    return NS_ERROR_FAILURE;
+
+  if (msd->output_emitter)
+  {
+    nsIMimeEmitter *emitter = (nsIMimeEmitter *)msd->output_emitter;
+    return emitter->AddAllHeaders(allheaders, allheadersize);
+  }
+
+  return NS_ERROR_FAILURE;
+}
+
 extern "C" nsresult     
 mimeEmitterStartAttachment(MimeDisplayOptions *opt, const char *name, const char *contentType, const char *url,
                            PRBool aNotDownloaded)
@@ -1953,11 +1994,11 @@ ResetChannelCharset(MimeObject *obj)
     char *ct = MimeHeaders_get (obj->headers, HEADER_CONTENT_TYPE, PR_FALSE, PR_FALSE); 
     if ( (ct) && (msd) && (msd->channel) ) 
     { 
-      char *ptr = PL_strstr(ct, "charset="); 
+      char *ptr = strstr(ct, "charset="); 
       if (ptr)
       {
         // First, setup the channel!
-        msd->channel->SetContentType(ct); 
+        msd->channel->SetContentType(nsDependentCString(ct));
 
         // Second, if this is a Save As operation, then we need to convert
         // to override the output charset!
@@ -1993,145 +2034,200 @@ ResetChannelCharset(MimeObject *obj)
 }
 
   ////////////////////////////////////////////////////////////
-  // Function to get up mail/news font
+  // Function to get up mail/news fontlang 
   ////////////////////////////////////////////////////////////
 
 
-nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed, char *fontName, PRUint32 nameBuffSize, 
-                         PRInt32 *fontPixelSize, PRInt32 *fontSizePercentage)
+nsresult GetMailNewsFont(MimeObject *obj, PRBool styleFixed,  PRInt32 *fontPixelSize, 
+		                     PRInt32 *fontSizePercentage, nsCString& fontLang)
 {
   nsresult rv = NS_OK;
 
-  nsIPref *aPrefs = GetPrefServiceManager(obj->options);
-  if (aPrefs) {
+  nsIPref *prefs = GetPrefServiceManager(obj->options);
+  if (prefs) {
     MimeInlineText  *text = (MimeInlineText *) obj;
-    nsCAutoString aCharset;
-    PRUnichar *unicode = nsnull;
-    nsCAutoString convertedStr;
-    nsCAutoString variableFontType;   // serif, sans-serif
+    nsCAutoString charset;
 
     // get a charset
     if (!text->initializeCharset)
       ((MimeInlineTextClass*)&mimeInlineTextClass)->initialize_charset(obj);
 
     if (!text->charset || !(*text->charset))
-      aCharset.Assign("us-ascii");
+      charset.Assign("us-ascii");
     else
-      aCharset.Assign(text->charset);
+      charset.Assign(text->charset);
 
-    // get variable font type
-    char* variable_font_type = nsnull;
-    rv = aPrefs->CopyCharPref("font.default", &variable_font_type);
+    nsCOMPtr<nsICharsetConverterManager2> charSetConverterManager2;
+    nsCOMPtr <nsIAtom> charsetAtom;
+    nsCOMPtr<nsIAtom> langGroupAtom;
+    const PRUnichar* langGroup = nsnull;
+    nsCAutoString prefStr;
+
+    ToLowerCase(charset);
+
+    charSetConverterManager2 = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+    if ( NS_FAILED(rv))
+      return rv;
+
+    // get a language, e.g. x-western, ja
+    rv = charSetConverterManager2->GetCharsetAtom2(charset.get(),getter_AddRefs(charsetAtom));
+    if (NS_FAILED(rv))
+      return rv;
+    rv = charSetConverterManager2->GetCharsetLangGroup(charsetAtom,getter_AddRefs(langGroupAtom));
+    if (NS_FAILED(rv))
+      return rv;
+    rv = langGroupAtom->GetUnicode(&langGroup);
     if (NS_FAILED(rv))
       return rv;
-    if (!variable_font_type)
-      return NS_ERROR_FAILURE;
-    variableFontType.Assign(variable_font_type);
-    PR_FREEIF(variable_font_type);
 
-    // if indicated by the pref, do language sensitive font selection
-    PRBool  languageSensitiveFont = PR_FALSE;
-    rv = aPrefs->GetBoolPref("mailnews.language_sensitive_font", &languageSensitiveFont);
-    if (NS_SUCCEEDED(rv) && languageSensitiveFont) {
-      nsCOMPtr<nsICharsetConverterManager> aCharSets;
-      nsCOMPtr<nsIAtom> aLangGroup;
-      const PRUnichar* langGroup = nsnull;
-      nsCAutoString aPrefStr;
-      
-      if (styleFixed) {
-        aPrefStr.Assign("font.name.monospace.");
-      }
-      else {
-        aPrefStr.Assign("font.name.");
-        aPrefStr.Append(variableFontType);
-        aPrefStr.Append(".");
-      }
+    fontLang.AssignWithConversion(langGroup);
 
-      ToLowerCase(aCharset);
+    // get a font size from pref
+    prefStr.Assign(!styleFixed ? "font.size.variable." : "font.size.fixed.");
+    prefStr.AppendWithConversion(langGroup);
+    rv = prefs->GetIntPref(prefStr.get(), fontPixelSize);
+    if (NS_FAILED(rv))
+      return rv;
 
-      aCharSets = do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID);
-      if (!aCharSets)
-        return NS_ERROR_FAILURE;
-        
-      // get a language, e.g. x-western, ja
-      nsAutoString u;
-      u.AssignWithConversion(aCharset.get());
-      rv = aCharSets->GetCharsetLangGroup(&u, getter_AddRefs(aLangGroup));
-      if (NS_FAILED(rv))
-        return rv;
-      rv = aLangGroup->GetUnicode(&langGroup);
-      if (NS_FAILED(rv))
-        return rv;
-  
-      // append the language to the pref string
-      aPrefStr.AppendWithConversion(langGroup);
+    // get original font size
+    PRInt32 originalSize;
+    rv = prefs->GetDefaultIntPref(prefStr.get(), &originalSize);
+    if (NS_FAILED(rv))
+      return rv;
 
-      // get a font name from pref, could be non ascii (need charset conversion)
-      // this is not necessary if we insert this tag after the message is converted to UTF-8
-      rv = aPrefs->CopyUnicharPref(aPrefStr.get(), &unicode);
-      if (NS_FAILED(rv))
-        return rv;
+    // calculate percentage
+    *fontSizePercentage = originalSize ? 
+                          (PRInt32)((float)*fontPixelSize / (float)originalSize * 100) : 0;
 
-     nsAutoString familyname(unicode);
-#if defined(XP_UNIX)
-     // extract "courier" from "adobe-courier-iso8859-1"
-     nsAutoString tmp;
-     familyname.Right(tmp, familyname.Length()-familyname.FindChar(PRUnichar('-'))-1);
-     tmp.Left(familyname, tmp.FindChar(PRUnichar('-')));
-#endif
-      rv = nsMsgI18NConvertFromUnicode(nsCAutoString("UTF-8"), familyname, convertedStr);
-      PR_FREEIF(unicode);
-      if (NS_FAILED(rv))
-        return rv;
-
-      if (convertedStr.Length() >= nameBuffSize)
-        return NS_ERROR_FAILURE;
-
-      PL_strcpy(fontName, convertedStr.get());
-
-      // get a font size from pref
-      aPrefStr.Assign(!styleFixed ? "font.size.variable." : "font.size.fixed.");
-      aPrefStr.AppendWithConversion(langGroup);
-      rv = aPrefs->GetIntPref(aPrefStr.get(), fontPixelSize);
-      if (NS_FAILED(rv))
-        return rv;
-
-      // get original font size
-      PRInt32 originalSize;
-      rv = aPrefs->GetDefaultIntPref(aPrefStr.get(), &originalSize);
-      if (NS_FAILED(rv))
-        return rv;
-
-      // calculate percentage
-      *fontSizePercentage = (PRInt32)((float)*fontPixelSize / (float)originalSize * 100);
-
-    }
-    // otherwise, use the mailnews font setting from pref
-    else {
-
-      // get a font name from pref, could be non ascii (need charset conversion)
-      // this is not necessary if we insert this tag after the message is converted to UTF-8
-      rv = aPrefs->CopyUnicharPref(!styleFixed ? "mailnews.font.name.html" : "mailnews.font.name.plain", &unicode);
-      if (NS_FAILED(rv))
-        return rv;
-
-      rv = nsMsgI18NConvertFromUnicode(nsCAutoString("UTF-8"), nsAutoString(unicode), convertedStr);
-      PR_FREEIF(unicode);
-      if (NS_FAILED(rv))
-        return rv;
-
-      if (convertedStr.Length() >= nameBuffSize)
-        return NS_ERROR_FAILURE;
-
-      PL_strcpy(fontName, convertedStr.get());
-
-      // get a font size from pref
-      rv = aPrefs->GetIntPref(!styleFixed ? "mailnews.font.size.html" : "mailnews.font.size.plain", fontPixelSize);
-      if (NS_FAILED(rv))
-        return rv;
-
-    }
   }
 
   return NS_OK;
 }
+
+/* This function syncronously converts an HTML document (as string)
+   to plaintext (as string) using the Gecko converter.
+
+   flags: see nsIDocumentEncoder.h
+*/
+// TODO: |printf|s?
+/* <copy from="mozilla/htmlparser/test/outsinks/Convert.cpp"
+         author="akk"
+         adapted-by="Ben Bucksch"
+         comment=" 'This code would not have been possible without akk.' ;-P.
+                   No, really. "
+   > */
+nsresult
+HTML2Plaintext(const nsString& inString, nsString& outString,
+               PRUint32 flags, PRUint32 wrapCol)
+{
+  nsresult rv = NS_OK;
+
+#if DEBUG_BenB
+  printf("Converting HTML to plaintext\n");
+  char* charstar = ToNewUTF8String(inString);
+  printf("HTML source is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  // Create a parser
+  nsCOMPtr<nsIParser> parser = do_CreateInstance(kParserCID);
+  NS_ENSURE_TRUE(parser, NS_ERROR_FAILURE);
+
+  // Create the appropriate output sink
+  nsCOMPtr<nsIContentSink> sink =
+                               do_CreateInstance(NS_PLAINTEXTSINK_CONTRACTID);
+  NS_ENSURE_TRUE(sink, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIHTMLToTextSink> textSink(do_QueryInterface(sink));
+  NS_ENSURE_TRUE(textSink, NS_ERROR_FAILURE);
+
+  textSink->Initialize(&outString, flags, wrapCol);
+
+  parser->SetContentSink(sink);
+  nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID);
+  NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+
+  parser->RegisterDTD(dtd);
+
+  rv = parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"),
+                     PR_FALSE, PR_TRUE);
+
+  // Aah! How can NS_ERROR and NS_ABORT_IF_FALSE be no-ops in release builds???
+  if (NS_FAILED(rv))
+  {
+    NS_ERROR("Parse() failed!");
+    return rv;
+  }
+
+#if DEBUG_BenB
+  charstar = ToNewUTF8String(outString);
+  printf("Plaintext is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  return rv;
+}
+// </copy>
+
+
+
+/* This function syncronously sanitizes an HTML document (string->string)
+   using the Gecko ContentSink mozISanitizingHTMLSerializer.
+
+   flags: currently unused
+   allowedTags: see mozSanitizingHTMLSerializer::ParsePrefs()
+*/
+// copied from HTML2Plaintext above
+nsresult
+HTMLSanitize(const nsString& inString, nsString& outString,
+             PRUint32 flags, const nsAString& allowedTags)
+{
+  nsresult rv = NS_OK;
+
+#if DEBUG_BenB
+  printf("Sanitizing HTML\n");
+  char* charstar = ToNewUTF8String(inString);
+  printf("Original HTML is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  // Create a parser
+  nsCOMPtr<nsIParser> parser = do_CreateInstance(kParserCID);
+  NS_ENSURE_TRUE(parser, NS_ERROR_FAILURE);
+
+  // Create the appropriate output sink
+  nsCOMPtr<nsIContentSink> sink =
+                    do_CreateInstance(MOZ_SANITIZINGHTMLSERIALIZER_CONTRACTID);
+  NS_ENSURE_TRUE(sink, NS_ERROR_FAILURE);
+
+  nsCOMPtr<mozISanitizingHTMLSerializer> sanSink(do_QueryInterface(sink));
+  NS_ENSURE_TRUE(sanSink, NS_ERROR_FAILURE);
+
+  sanSink->Initialize(&outString, flags, allowedTags);
+
+  parser->SetContentSink(sink);
+  nsCOMPtr<nsIDTD> dtd = do_CreateInstance(kNavDTDCID);
+  NS_ENSURE_TRUE(dtd, NS_ERROR_FAILURE);
+
+  parser->RegisterDTD(dtd);
+
+  rv = parser->Parse(inString, 0, NS_LITERAL_CSTRING("text/html"),
+                     PR_FALSE, PR_TRUE);
+  if (NS_FAILED(rv))
+  {
+    NS_ERROR("Parse() failed!");
+    return rv;
+  }
+
+#if DEBUG_BenB
+  charstar = ToNewUTF8String(outString);
+  printf("Sanitized HTML is:\n--------------------\n%s--------------------\n",
+         charstar);
+  delete[] charstar;
+#endif
+
+  return rv;
+}
