@@ -70,12 +70,12 @@
 #include "nsICategoryManager.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
+#include "nsIObservableInputStream.h"
+#include "nsIObservableOutputStream.h"
 
 #define PREF_MAIL_DISPLAY_GLYPH "mail.display_glyph"
 #define PREF_MAIL_DISPLAY_STRUCT "mail.display_struct"
 
-#define PREF_MAIL_MIME_XUL_OUTPUT "mail.mime_xul_output"
-
 static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 
 ////////////////////////////////////////////////////////////////
@@ -202,6 +202,13 @@ bridge_new_new_uri(void *bridgeStream, nsIURI *aURI, PRInt32 aOutputType)
               *override_charset = PR_TRUE;
 
             // notify the default to msgWindow (for the menu check mark)
+            // do not set the default in case of nsMimeMessageDraftOrTemplate
+            // or nsMimeMessageEditorTemplate because it is already set 
+            // when the message is displayed and doing it again may overwrite 
+            // the correct MIME charset parsed from the message header
+            if (aOutputType != nsMimeOutput::nsMimeMessageDraftOrTemplate &&
+                aOutputType != nsMimeOutput::nsMimeMessageEditorTemplate)
+            {
             nsCOMPtr<nsIMsgMailNewsUrl> msgurl (do_QueryInterface(aURI));
             if (msgurl)
             {
@@ -211,6 +218,7 @@ bridge_new_new_uri(void *bridgeStream, nsIURI *aURI, PRInt32 aOutputType)
               {
                 msgWindow->SetMailCharacterSet(NS_ConvertASCIItoUCS2(*default_charset).get());
                 msgWindow->SetCharsetOverride(*override_charset);
+                }
               }
             }
 
@@ -229,20 +237,19 @@ bridge_new_new_uri(void *bridgeStream, nsIURI *aURI, PRInt32 aOutputType)
             }
           }
         }
-        char *urlString;
-        if (NS_SUCCEEDED(aURI->GetSpec(&urlString)))
+        nsCAutoString urlString;
+        if (NS_SUCCEEDED(aURI->GetSpec(urlString)))
         {
-          if ((urlString) && (*urlString))
+          if (!urlString.IsEmpty())
           {
             CRTFREEIF(*url_name);
-            *url_name = nsCRT::strdup(urlString);
+            *url_name = ToNewCString(urlString);
             if (!(*url_name))
               return NS_ERROR_OUT_OF_MEMORY;
 
             // rhp: Ugh, this is ugly...but it works.
             if (fixup_pointer)
               *fixup_pointer = (const char *)*url_name;
-            CRTFREEIF(urlString);
           }
         }
       }
@@ -348,7 +355,7 @@ nsStreamConverter::DetermineOutputFormat(const char *url,  nsMimeOutputType *aNe
   // have to be represented via the "%2F" value
   if (format)
   {
-    format += nsCRT::strlen("?outformat=");
+    format += strlen("?outformat=");
     while (*format == ' ')
       ++format;
 
@@ -375,8 +382,8 @@ nsStreamConverter::DetermineOutputFormat(const char *url,  nsMimeOutputType *aNe
               )
           {
             *ptr = '/';
-            memmove(ptr+1, ptr+3, nsCRT::strlen(ptr+3));
-            *(ptr + nsCRT::strlen(ptr+3) + 1) = '\0';
+            memmove(ptr+1, ptr+3, strlen(ptr+3));
+            *(ptr + strlen(ptr+3) + 1) = '\0';
             ptr += 3;
           }
         }
@@ -392,7 +399,7 @@ nsStreamConverter::DetermineOutputFormat(const char *url,  nsMimeOutputType *aNe
   {
     if (header)
     {
-      PRInt32 lenOfHeader = nsCRT::strlen("?header=");
+      PRInt32 lenOfHeader = strlen("?header=");
 
       char *ptr2 = PL_strcasestr ("only", (header+lenOfHeader));
       char *ptr3 = PL_strcasestr ("quote", (header+lenOfHeader));
@@ -446,24 +453,9 @@ nsStreamConverter::DetermineOutputFormat(const char *url,  nsMimeOutputType *aNe
     }
     else
     {
-      // okay, we are just doing a regular message display url
-      // since we didn't have any special extensions after the url...
-      // we need to know if we need to use the application/vnd.mozilla.xul+xml
-      // or text/html emitter.
-      // check the desired output content type that was passed into AsyncConvertData.
-
-      if (mDesiredOutputType && Compare(nsDependentString(mDesiredOutputType), NS_LITERAL_STRING("application/vnd.mozilla.xul+xml"),nsCaseInsensitiveStringComparator()) == 0)
-      {
-        CRTFREEIF(mOutputFormat);
-        mOutputFormat = nsCRT::strdup("application/vnd.mozilla.xul+xml");
-        *aNewType = nsMimeOutput::nsMimeMessageXULDisplay;
-      }
-      else
-      {
-        CRTFREEIF(mOutputFormat);
-        mOutputFormat = nsCRT::strdup("text/html");
-        *aNewType = nsMimeOutput::nsMimeMessageBodyDisplay;
-      }
+      CRTFREEIF(mOutputFormat);
+      mOutputFormat = nsCRT::strdup("text/html");
+      *aNewType = nsMimeOutput::nsMimeMessageBodyDisplay;
     }
   }
   else // this is a part that should just come out raw!
@@ -479,15 +471,15 @@ nsStreamConverter::DetermineOutputFormat(const char *url,  nsMimeOutputType *aNe
       {
         // store the real content type...mOutputFormat gets deleted later on...
 		// and make sure we only get our own value.
-		char *nextField = PL_strcasestr(typeField + nsCRT::strlen("&type="),"&");
+		char *nextField = PL_strcasestr(typeField + strlen("&type="),"&");
 		if (nextField)
 		{
 			*nextField = 0;
-        mRealContentType = typeField + nsCRT::strlen("&type=");
+        mRealContentType = typeField + strlen("&type=");
 			*nextField = '&';
 		}
 		else
-          mRealContentType = typeField + nsCRT::strlen("&type=");
+          mRealContentType = typeField + strlen("&type=");
         if (mRealContentType.Equals("message/rfc822"))
         {
           mRealContentType = "text/plain";
@@ -574,8 +566,9 @@ NS_INTERFACE_MAP_END
 
 NS_IMETHODIMP nsStreamConverter::Init(nsIURI *aURI, nsIStreamListener * aOutListener, nsIChannel *aChannel)
 {
-  nsresult rv = NS_OK;
-  
+  NS_ENSURE_ARG_POINTER(aURI);
+
+  nsresult rv = NS_OK; 
   mOutListener = aOutListener;
   CRTFREEIF(mDesiredOutputType);
   
@@ -584,9 +577,9 @@ NS_IMETHODIMP nsStreamConverter::Init(nsIURI *aURI, nsIStreamListener * aOutList
   
   if (!mAlreadyKnowOutputType)
   {
-    nsXPIDLCString urlSpec;
-    rv = aURI->GetSpec(getter_Copies(urlSpec));
-    DetermineOutputFormat(urlSpec, &newType);
+    nsCAutoString urlSpec;
+    rv = aURI->GetSpec(urlSpec);
+    DetermineOutputFormat(urlSpec.get(), &newType);
     mAlreadyKnowOutputType = PR_TRUE;
   }
   else
@@ -595,10 +588,6 @@ NS_IMETHODIMP nsStreamConverter::Init(nsIURI *aURI, nsIStreamListener * aOutList
   mOutputType = newType;  
   switch (newType)
   {
-    case nsMimeOutput::nsMimeMessageXULDisplay:
-      CRTFREEIF(mOutputFormat);
-      mOutputFormat = nsCRT::strdup("application/vnd.mozilla.xul+xml");
-    break;
 		case nsMimeOutput::nsMimeMessageSplitDisplay:    // the wrapper HTML output to produce the split header/body display
       mWrapperOutput = PR_TRUE;
       CRTFREEIF(mOutputFormat);
@@ -702,11 +691,17 @@ NS_IMETHODIMP nsStreamConverter::Init(nsIURI *aURI, nsIStreamListener * aOutList
   if (NS_SUCCEEDED(rv))
   {
     nsCOMPtr<nsIInputStreamObserver> inObs = do_GetInterface(mEmitter, &rv);
-    if (NS_SUCCEEDED(rv))
-      mInputStream->SetObserver(inObs);
+    if (NS_SUCCEEDED(rv)) {
+      nsCOMPtr<nsIObservableInputStream> observableIn(do_QueryInterface(mInputStream, &rv));
+      if (NS_SUCCEEDED(rv))
+        observableIn->SetObserver(inObs);
+    }
     nsCOMPtr<nsIOutputStreamObserver> outObs = do_GetInterface(mEmitter, &rv);
-    if (NS_SUCCEEDED(rv))
-      mOutputStream->SetObserver(outObs);
+    if (NS_SUCCEEDED(rv)) {
+      nsCOMPtr<nsIObservableOutputStream> observableOut(do_QueryInterface(mOutputStream, &rv));
+      if (NS_SUCCEEDED(rv))
+        observableOut->SetObserver(outObs);
+    }
   }
   
   // initialize our emitter
@@ -767,9 +762,9 @@ NS_IMETHODIMP nsStreamConverter::GetContentType(char **aOutputContentType)
   if (!mRealContentType.IsEmpty())
     *aOutputContentType = ToNewCString(mRealContentType);
   else if (nsCRT::strcasecmp(mOutputFormat, "raw") == 0)
-		*aOutputContentType = (char *) nsMemory::Clone(UNKNOWN_CONTENT_TYPE, nsCRT::strlen(UNKNOWN_CONTENT_TYPE) + 1);
+		*aOutputContentType = (char *) nsMemory::Clone(UNKNOWN_CONTENT_TYPE, strlen(UNKNOWN_CONTENT_TYPE) + 1);
 	else
-		*aOutputContentType = (char *) nsMemory::Clone(mOutputFormat, nsCRT::strlen(mOutputFormat) + 1);
+		*aOutputContentType = (char *) nsMemory::Clone(mOutputFormat, strlen(mOutputFormat) + 1);
 	return NS_OK;
 }
 
@@ -855,6 +850,20 @@ nsStreamConverter::SetIdentity(nsIMsgIdentity * aIdentity)
 	return NS_OK;
 }
 
+NS_IMETHODIMP
+nsStreamConverter::SetOriginalMsgURI(const char * originalMsgURI)
+{
+  mOriginalMsgURI = originalMsgURI;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsStreamConverter::GetOriginalMsgURI(char ** result)
+{
+  if (!result) return NS_ERROR_NULL_POINTER;
+  *result = ToNewCString(mOriginalMsgURI);
+  return NS_OK;
+}
 
 /////////////////////////////////////////////////////////////////////////////
 // Methods for nsIStreamListener...
@@ -887,15 +896,14 @@ const char output[] = "\
 </FRAMESET>\
 </HTML>";
 
-    char *url = nsnull;
-    if (NS_FAILED(mURI->GetSpec(&url)))
+    nsCAutoString url;
+    if (NS_FAILED(mURI->GetSpec(url)))
       return NS_ERROR_FAILURE;
   
-    PR_snprintf(outBuf, sizeof(outBuf), output, url, url);
-    PR_FREEIF(url);
+    PR_snprintf(outBuf, sizeof(outBuf), output, url.get(), url.get());
     
     if (mEmitter)
-      mEmitter->Write(outBuf, nsCRT::strlen(outBuf), &written);
+      mEmitter->Write(outBuf, strlen(outBuf), &written);
     mTotalRead += written;
 
     // rhp: will this stop the stream???? Not sure.    
@@ -1006,13 +1014,8 @@ nsStreamConverter::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresul
 
       if (workHeaders)
       {
-        static NS_DEFINE_CID(kCIMimeHeadersCID, NS_IMIMEHEADERS_CID);
         nsresult rv;
-        nsCOMPtr<nsIMimeHeaders> mimeHeaders;
-        
-        rv = nsComponentManager::CreateInstance(kCIMimeHeadersCID, 
-          nsnull, NS_GET_IID(nsIMimeHeaders), 
-          (void **) getter_AddRefs(mimeHeaders)); 
+        nsCOMPtr<nsIMimeHeaders> mimeHeaders = do_CreateInstance(NS_IMIMEHEADERS_CONTRACTID, &rv);
         
         if (NS_SUCCEEDED(rv))
         {
