@@ -37,6 +37,7 @@
 
 #include "msgCore.h"
 #include "nntpCore.h"
+#include "netCore.h"
 #include "nsXPIDLString.h"
 #include "nsIMsgNewsFolder.h"
 #include "nsIStringBundle.h"
@@ -68,6 +69,7 @@ nsresult nsNewsDownloader::DownloadArticles(nsIMsgWindow *window, nsIMsgFolder *
   
   m_folder = folder;
   m_window = window;
+  m_numwrote = 0;
   
   PRBool headersToDownload = GetNextHdrToRetrieve();
   // should we have a special error code for failure here?
@@ -95,13 +97,13 @@ nsNewsDownloader::nsNewsDownloader(nsIMsgWindow *window, nsIMsgDatabase *msgDB,
 
 nsNewsDownloader::~nsNewsDownloader()
 {
-	if (m_listener)
-	  m_listener->OnStopRunningUrl(/* don't have a url */nsnull, m_status);
-	if (m_newsDB)
-	{
-		m_newsDB->Commit(nsMsgDBCommitType::kLargeCommit);
-		m_newsDB = nsnull;
-	}
+  if (m_listener)
+    m_listener->OnStopRunningUrl(/* don't have a url */nsnull, m_status);
+  if (m_newsDB)
+  {
+    m_newsDB->Commit(nsMsgDBCommitType::kLargeCommit);
+    m_newsDB = nsnull;
+  }
 }
 
 NS_IMETHODIMP nsNewsDownloader::OnStartRunningUrl(nsIURI* url)
@@ -246,52 +248,52 @@ NS_IMETHODIMP DownloadNewsArticlesToOfflineStore::OnStartRunningUrl(nsIURI* url)
 
 NS_IMETHODIMP DownloadNewsArticlesToOfflineStore::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
 {
-	m_status = exitCode;
-	if (m_newsHeader != nsnull)
-	{
+  m_status = exitCode;
+  if (m_newsHeader != nsnull)
+  {
 #ifdef DEBUG_bienvenu
-//		XP_Trace("finished retrieving %ld\n", m_newsHeader->GetMessageKey());
+    //		XP_Trace("finished retrieving %ld\n", m_newsHeader->GetMessageKey());
 #endif
-		if (m_newsDB)
-		{
+    if (m_newsDB)
+    {
       nsMsgKey msgKey;
       m_newsHeader->GetMessageKey(&msgKey);
-			m_newsDB->MarkMarked(msgKey, PR_FALSE, nsnull);
-		}
-	}	
-	m_newsHeader = nsnull;
-	return nsNewsDownloader::OnStopRunningUrl(url, exitCode);
+      m_newsDB->MarkMarked(msgKey, PR_FALSE, nsnull);
+    }
+  }	
+  m_newsHeader = nsnull;
+  return nsNewsDownloader::OnStopRunningUrl(url, exitCode);
 }
 
 int DownloadNewsArticlesToOfflineStore::FinishDownload()
 {
-	return 0;
+  return 0;
 }
 
 
 NS_IMETHODIMP nsNewsDownloader::OnSearchHit(nsIMsgDBHdr *header, nsIMsgFolder *folder)
 {
   NS_ENSURE_ARG(header);
-
+  
   
   PRUint32 msgFlags;
   header->GetFlags(&msgFlags);
-	// only need to download articles we don't already have...
-	if (! (msgFlags & MSG_FLAG_OFFLINE))
-	{
+  // only need to download articles we don't already have...
+  if (! (msgFlags & MSG_FLAG_OFFLINE))
+  {
     nsMsgKey key;
     header->GetMessageKey(&key);
-		m_keysToDownload.Add(key);
+    m_keysToDownload.Add(key);
 #ifdef HAVE_PORT
-		char *statusTemplate = XP_GetString (MK_MSG_ARTICLES_TO_RETRIEVE);
-		char *statusString = PR_smprintf (statusTemplate,  (long) newsArticleState->m_keysToDownload.GetSize());
-		if (statusString)
-		{
-			FE_Progress (newsArticleState->m_context, statusString);
-			XP_FREE(statusString);
-		}
+    char *statusTemplate = XP_GetString (MK_MSG_ARTICLES_TO_RETRIEVE);
+    char *statusString = PR_smprintf (statusTemplate,  (long) newsArticleState->m_keysToDownload.GetSize());
+    if (statusString)
+    {
+      FE_Progress (newsArticleState->m_context, statusString);
+      XP_FREE(statusString);
+    }
 #endif
-	}
+  }
   return NS_OK;
 }
 
@@ -302,7 +304,8 @@ NS_IMETHODIMP nsNewsDownloader::OnSearchDone(nsresult status)
     if (m_listener)
       return m_listener->OnStopRunningUrl(nsnull, NS_OK);
   }
-  nsresult rv = DownloadArticles(m_window, m_folder, &m_keysToDownload);
+  nsresult rv = DownloadArticles(m_window, m_folder, 
+                  /* we've already set m_keysToDownload, so don't pass it in */ nsnull);
   if (NS_FAILED(rv))
     if (m_listener)
       m_listener->OnStopRunningUrl(nsnull, rv);
@@ -316,14 +319,14 @@ NS_IMETHODIMP nsNewsDownloader::OnNewSearch()
 
 int DownloadNewsArticlesToOfflineStore::StartDownload()
 {
-	m_newsDB->GetMsgHdrForKey(m_keyToDownload, getter_AddRefs(m_newsHeader));
-	return 0;
+  m_newsDB->GetMsgHdrForKey(m_keyToDownload, getter_AddRefs(m_newsHeader));
+  return 0;
 }
 
 DownloadNewsArticlesToOfflineStore::DownloadNewsArticlesToOfflineStore(nsIMsgWindow *window, nsIMsgDatabase *db, nsIUrlListener *listener)
 	: nsNewsDownloader(window, db, listener)
 {
-	m_newsDB = db;
+  m_newsDB = db;
 }
 
 DownloadNewsArticlesToOfflineStore::~DownloadNewsArticlesToOfflineStore()
@@ -335,10 +338,10 @@ DownloadMatchingNewsArticlesToNewsDB::DownloadMatchingNewsArticlesToNewsDB
 	 nsIUrlListener *listener) :
 	 DownloadNewsArticlesToOfflineStore(window, newsDB, listener)
 {
-	m_window = window;
-	m_folder = folder;
-	m_newsDB = newsDB;
-	m_downloadFromKeys = PR_TRUE;	// search term matching means downloadFromKeys.
+  m_window = window;
+  m_folder = folder;
+  m_newsDB = newsDB;
+  m_downloadFromKeys = PR_TRUE;	// search term matching means downloadFromKeys.
 }
 
 DownloadMatchingNewsArticlesToNewsDB::~DownloadMatchingNewsArticlesToNewsDB()
@@ -357,10 +360,13 @@ nsMsgDownloadAllNewsgroups::nsMsgDownloadAllNewsgroups(nsIMsgWindow *window, nsI
   m_window = window;
   m_listener = listener;
   m_downloaderForGroup = new DownloadMatchingNewsArticlesToNewsDB(window, nsnull, nsnull, this);
+  NS_IF_ADDREF(m_downloaderForGroup);
+  m_downloadedHdrsForCurGroup = PR_FALSE;
 }
 
 nsMsgDownloadAllNewsgroups::~nsMsgDownloadAllNewsgroups()
 {
+  NS_IF_RELEASE(m_downloaderForGroup);
 }
 
 NS_IMETHODIMP nsMsgDownloadAllNewsgroups::OnStartRunningUrl(nsIURI* url)
@@ -373,7 +379,25 @@ nsMsgDownloadAllNewsgroups::OnStopRunningUrl(nsIURI* url, nsresult exitCode)
 {
   nsresult rv = exitCode;
   if (NS_SUCCEEDED(exitCode) || exitCode == NS_MSG_NEWS_ARTICLE_NOT_FOUND)
-    rv = ProcessNextGroup();
+  {
+    if (m_downloadedHdrsForCurGroup)
+    {
+      PRBool savingArticlesOffline = PR_FALSE;
+      nsCOMPtr <nsIMsgNewsFolder> newsFolder = do_QueryInterface(m_currentFolder);
+      if (newsFolder)
+        newsFolder->GetSaveArticleOffline(&savingArticlesOffline);
+
+      m_downloadedHdrsForCurGroup = PR_FALSE;
+      if (savingArticlesOffline) // skip this group - we're saving to it already
+        rv = ProcessNextGroup();
+      else
+        rv = DownloadMsgsForCurrentGroup();
+    }
+    else
+    {
+      rv = ProcessNextGroup();
+    }
+  }
   else if (m_listener)  // notify main observer.
     m_listener->OnStopRunningUrl(url, exitCode);
    
@@ -489,7 +513,7 @@ nsresult DownloadMatchingNewsArticlesToNewsDB::RunSearch(nsIMsgFolder *folder, n
 
   searchSession->RegisterListener(this);
   rv = searchSession->AddScopeTerm(nsMsgSearchScope::localNews, folder);
-	return searchSession->Search(m_window);
+  return searchSession->Search(m_window);
 }
 
 nsresult nsMsgDownloadAllNewsgroups::ProcessNextGroup()
@@ -513,12 +537,17 @@ nsresult nsMsgDownloadAllNewsgroups::ProcessNextGroup()
     if (m_listener)
       return m_listener->OnStopRunningUrl(nsnull, NS_OK);
   }
+  m_downloadedHdrsForCurGroup = PR_TRUE;
+  return m_currentFolder->GetNewMessages(m_window, this);
+}
 
+nsresult nsMsgDownloadAllNewsgroups::DownloadMsgsForCurrentGroup()
+{
   nsCOMPtr <nsIMsgDatabase> db;
   nsCOMPtr <nsISupportsArray> termList;
   nsCOMPtr <nsIMsgDownloadSettings> downloadSettings;
   m_currentFolder->GetMsgDatabase(m_window, getter_AddRefs(db));
-  rv = m_currentFolder->GetDownloadSettings(getter_AddRefs(downloadSettings));
+  nsresult rv = m_currentFolder->GetDownloadSettings(getter_AddRefs(downloadSettings));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr <nsIMsgNewsFolder> newsFolder = do_QueryInterface(m_currentFolder);
