@@ -38,7 +38,6 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#define NS_IMPL_IDS
 #include "nsIPref.h"
 #include "prlog.h"
 
@@ -79,7 +78,9 @@
 
 #include "nsXPIDLString.h"
 
-#include "nsIWalletService.h"
+#include "nsIObserverService.h"
+#include "nsNetUtil.h"
+#include "nsIAuthPrompt.h"
 #include "nsIURL.h"
 #include "nsNetCID.h"
 #include "nsINntpUrl.h"
@@ -91,6 +92,7 @@
 #include "nsReadableUtils.h"
 #include "nsNewsDownloader.h"
 #include "nsIStringBundle.h"
+#include "nsEscape.h"
 
 // we need this because of an egcs 1.0 (and possibly gcc) compiler bug
 // that doesn't allow you to call ::nsISupports::GetIID() inside of a class
@@ -100,8 +102,6 @@ static NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
 static NS_DEFINE_CID(kCNewsDB, NS_NEWSDB_CID);
 static NS_DEFINE_CID(kMsgMailSessionCID, NS_MSGMAILSESSION_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-static NS_DEFINE_CID(kWalletServiceCID, NS_WALLETSERVICE_CID);
-static NS_DEFINE_CID(kStandardUrlCID, NS_STANDARDURL_CID);
 
 // ###tw  This really ought to be the most
 // efficient file reading size for the current
@@ -129,7 +129,6 @@ nsMsgNewsFolder::nsMsgNewsFolder(void) : nsMsgLineBuffer(nsnull, PR_FALSE),
     SetLookingForCRLF(PR_FALSE);
   }
 
-  mMessages = nsnull;
 //  NS_INIT_REFCNT(); done by superclass
 }
 
@@ -385,30 +384,31 @@ nsresult nsMsgNewsFolder::GetDatabase(nsIMsgWindow *aMsgWindow)
     rv = GetPath(getter_AddRefs(pathSpec));
     if (NS_FAILED(rv)) return rv;
     
-    nsresult folderOpen = NS_OK;
-    nsCOMPtr <nsIMsgDatabase> newsDBFactory;
-    
-    rv = nsComponentManager::CreateInstance(kCNewsDB, nsnull, NS_GET_IID(nsIMsgDatabase), getter_AddRefs(newsDBFactory));
-    if (NS_SUCCEEDED(rv) && newsDBFactory) {
-      folderOpen = newsDBFactory->OpenFolderDB(this, PR_TRUE, PR_FALSE, getter_AddRefs(mDatabase));
-    }
+    nsCOMPtr <nsIMsgDatabase> newsDBFactory = do_CreateInstance(kCNewsDB, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
+
+    nsresult folderOpen = newsDBFactory->OpenFolderDB(this, PR_TRUE, PR_FALSE, getter_AddRefs(mDatabase));
     
     if(folderOpen == NS_MSG_ERROR_FOLDER_SUMMARY_MISSING || folderOpen == NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE)
       folderOpen = newsDBFactory->OpenFolderDB(this, PR_TRUE, PR_TRUE, getter_AddRefs(mDatabase));
 
-    if (mDatabase) {
-      if(mAddListener)
-        rv = mDatabase->AddListener(this);
+    if (NS_FAILED(folderOpen))
+      return folderOpen;
+
+    if(mAddListener)
+      rv = mDatabase->AddListener(this);
       
-      nsCOMPtr<nsINewsDatabase> db(do_QueryInterface(mDatabase, &rv));
-      if (NS_FAILED(rv)) return rv;        
+    nsCOMPtr<nsINewsDatabase> db = do_QueryInterface(mDatabase, &rv);
+    if (NS_FAILED(rv)) 
+      return rv;        
       
-      rv = db->SetReadSet(mReadSet);
-      if (NS_FAILED(rv)) return rv;        
-      rv = UpdateSummaryTotals(PR_TRUE);
-    }
-    if (NS_FAILED(rv)) return rv;
-    
+    rv = db->SetReadSet(mReadSet);
+    if (NS_FAILED(rv)) 
+      return rv;        
+
+    rv = UpdateSummaryTotals(PR_TRUE);
+    if (NS_FAILED(rv)) 
+      return rv;
   }
   return NS_OK;
 }
@@ -467,6 +467,14 @@ nsMsgNewsFolder::GetCanFileMessages(PRBool *aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsMsgNewsFolder::GetCanDeleteMessages(PRBool *aCanDeleteMessages)
+{
+  NS_ENSURE_ARG_POINTER(aCanDeleteMessages);
+  *aCanDeleteMessages = PR_FALSE;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsMsgNewsFolder::GetCanCreateSubfolders(PRBool *aResult)
 {
@@ -510,23 +518,26 @@ nsMsgNewsFolder::GetMessages(nsIMsgWindow *aMsgWindow, nsISimpleEnumerator* *res
 
 NS_IMETHODIMP nsMsgNewsFolder::GetFolderURL(char **url)
 {
-  if(!url)
-    return NS_ERROR_NULL_POINTER;
+  NS_ENSURE_ARG_POINTER(url);
 
-  nsCOMPtr<nsIFileSpec> pathSpec;
-  nsresult rv = GetPath(getter_AddRefs(pathSpec));
-  if (NS_FAILED(rv)) return rv;
+  nsXPIDLCString hostName;
+  nsresult rv = GetHostname(getter_Copies(hostName));
+  nsXPIDLCString groupName;
+  rv = GetAsciiName(getter_Copies(groupName));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsFileSpec path;
-  rv = pathSpec->GetFileSpec(&path);
-  if (NS_FAILED(rv)) return rv;
-#if defined(XP_MAC)
-  nsCAutoString tmpPath((nsFilePath)path); //ducarroz: please don't cast a nsFilePath to char* on Mac
-  *url = PR_smprintf("%s%s", NEWS_SCHEME, tmpPath.get());
-#else
-  const char *pathName = path;
-  *url = PR_smprintf("%s%s", NEWS_SCHEME, pathName);
-#endif
+  nsCOMPtr<nsIMsgIncomingServer> server;
+  rv = GetServer(getter_AddRefs(server));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  PRInt32 port;
+  PRBool isSecure = PR_FALSE;
+  rv = server->GetIsSecure(&isSecure);
+  NS_ENSURE_SUCCESS(rv, rv);
+  rv = server->GetPort(&port);
+  NS_ENSURE_SUCCESS(rv, rv);
+  const char *newsScheme = (isSecure) ? SNEWS_SCHEME : NEWS_SCHEME;
+  *url = PR_smprintf("%s//%s:%ld/%s", newsScheme, hostName.get(), port, groupName.get());
   return NS_OK;
 
 }
@@ -610,6 +621,9 @@ NS_IMETHODIMP nsMsgNewsFolder::Delete()
   rv = pathSpec->GetFileSpec(&path);
   if (NS_FAILED(rv)) return rv;
   
+  // delete local store, if it exists
+  if (path.Exists())
+    path.Delete(PR_FALSE);
   // Remove summary file.	
   nsNewsSummarySpec summarySpec(path);
   summarySpec.Delete(PR_FALSE);
@@ -697,7 +711,7 @@ nsresult nsMsgNewsFolder::AbbreviatePrettyName(PRUnichar ** prettyName, PRInt32
   PRInt32 pos = 0;
   while(1)
   {
-    pos = name.FindChar('.', PR_FALSE, pos);
+    pos = name.FindChar('.', pos);
     if(pos == -1)
     {
       totalwords++;
@@ -768,9 +782,7 @@ nsMsgNewsFolder::GetDBFolderInfoAndDB(nsIDBFolderInfo **folderInfo, nsIMsgDataba
     return NS_ERROR_NULL_POINTER; 
 
   openErr = GetDatabase(nsnull);
-
-  *db = mDatabase;
-  NS_IF_ADDREF(*db);
+  NS_IF_ADDREF(*db = mDatabase);
   if (NS_SUCCEEDED(openErr)&& *db)
     openErr = (*db)->GetDBFolderInfo(folderInfo);
   return openErr;
@@ -952,12 +964,21 @@ nsMsgNewsFolder::DeleteMessages(nsISupportsArray *messages, nsIMsgWindow *aMsgWi
   nsXPIDLCString messageID;
   rv = msgHdr->GetMessageId(getter_Copies(messageID));
   NS_ENSURE_SUCCESS(rv,rv);
+  
+  // we need to escape the message ID, 
+  // it might contain characters which will mess us up later, like #
+  // see bug #120502
+  char *escapedMessageID = nsEscape(messageID.get(), url_Path);
+  if (!escapedMessageID)
+    return NS_ERROR_OUT_OF_MEMORY;
 
   nsCAutoString cancelURL((const char *)serverURI);
   cancelURL += '/';
-  cancelURL += (const char *)messageID;
+  cancelURL += escapedMessageID;
   cancelURL += "?cancel";
 
+  PR_FREEIF(escapedMessageID);
+
   nsXPIDLCString messageURI;
   rv = GetUriForMsg(msgHdr, getter_Copies(messageURI));
   NS_ENSURE_SUCCESS(rv,rv);
@@ -970,16 +991,15 @@ nsMsgNewsFolder::DeleteMessages(nsISupportsArray *messages, nsIMsgWindow *aMsgWi
 
 NS_IMETHODIMP nsMsgNewsFolder::GetNewMessages(nsIMsgWindow *aMsgWindow, nsIUrlListener *aListener)
 {
-  NS_ASSERTION(aListener == nsnull, "news can't currently listen for this finishing");
-  return GetNewsMessages(aMsgWindow, PR_FALSE);
+  return GetNewsMessages(aMsgWindow, PR_FALSE, aListener);
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::GetNextNMessages(nsIMsgWindow *aMsgWindow)
 {
-  return GetNewsMessages(aMsgWindow, PR_TRUE);
+  return GetNewsMessages(aMsgWindow, PR_TRUE, nsnull);
 }
 
-nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow *aMsgWindow, PRBool aGetOld)
+nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow *aMsgWindow, PRBool aGetOld, nsIUrlListener *aUrlListener)
 {
   nsresult rv = NS_OK;
 
@@ -989,7 +1009,7 @@ nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow *aMsgWindow, PRBool aGetO
   
   if (isNewsServer) {
     // get new messages only works on a newsgroup, not a news server
-		return NS_OK;
+    return NS_OK;
   }
 
   nsCOMPtr <nsINntpService> nntpService = do_GetService(NS_NNTPSERVICE_CONTRACTID, &rv);
@@ -999,7 +1019,14 @@ nsresult nsMsgNewsFolder::GetNewsMessages(nsIMsgWindow *aMsgWindow, PRBool aGetO
   rv = GetNntpServer(getter_AddRefs(nntpServer));
   if (NS_FAILED(rv)) return rv;
   
-  rv = nntpService->GetNewNews(nntpServer, mURI, aGetOld, this, aMsgWindow, nsnull);
+  nsCOMPtr <nsIURI> resultUri;
+  rv = nntpService->GetNewNews(nntpServer, mURI, aGetOld, this, aMsgWindow, getter_AddRefs(resultUri));
+  if (aUrlListener && NS_SUCCEEDED(rv) && resultUri)
+  {
+    nsCOMPtr<nsIMsgMailNewsUrl> msgUrl (do_QueryInterface(resultUri));
+    if (msgUrl)
+      msgUrl->RegisterListener(aUrlListener);
+  }
   return rv;
 }
 
@@ -1232,12 +1259,11 @@ nsresult nsMsgNewsFolder::CreateNewsgroupUrlForSignon(const char *inUriStr, cons
 {
     nsresult rv;
     PRInt32 port = 0;
-    nsXPIDLCString spec;
 
-    nsCOMPtr<nsIURL> url;
-    nsComponentManager::CreateInstance(kStandardUrlCID, nsnull, NS_GET_IID(nsIURL), (void **) getter_AddRefs(url));
+    nsCOMPtr<nsIURL> url = do_CreateInstance(NS_STANDARDURL_CONTRACTID, &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
 
-    rv = url->SetSpec(inUriStr);
+    rv = url->SetSpec(nsDependentCString(inUriStr));
     if (NS_FAILED(rv)) return rv;
 
     rv = url->GetPort(&port);
@@ -1261,19 +1287,22 @@ nsresult nsMsgNewsFolder::CreateNewsgroupUrlForSignon(const char *inUriStr, cons
         if (NS_FAILED(rv)) return rv;
     }
 
-    rv = url->SetRef(ref);
+    rv = url->SetRef(nsDependentCString(ref));
     if (NS_FAILED(rv)) return rv;
 
-    rv = url->GetSpec(result);
-    return rv;
+    nsCAutoString spec;
+    rv = url->GetSpec(spec);
+    if (NS_FAILED(rv)) return rv;
+
+    *result = ToNewCString(spec);
+    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::ForgetGroupUsername()
 {
     nsresult rv;
-    nsCOMPtr<nsIWalletService> walletservice = 
-             do_GetService(kWalletServiceCID, &rv);
-    if (NS_FAILED(rv)) return rv;
+    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
 
     rv = SetGroupUsername(nsnull);
     if (NS_FAILED(rv)) return rv;
@@ -1282,17 +1311,23 @@ NS_IMETHODIMP nsMsgNewsFolder::ForgetGroupUsername()
     rv = CreateNewsgroupUsernameUrlForSignon(mURI, getter_Copies(signonURL));
     if (NS_FAILED(rv)) return rv;
 
-    rv = walletservice->SI_RemoveUser((const char *)signonURL, nsnull);
+    nsCOMPtr<nsIURI> uri;
+    NS_NewURI(getter_AddRefs(uri), signonURL);
+
+    //this is need to make sure wallet service has been created
+    rv = CreateServicesForPasswordManager();
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = observerService->NotifyObservers(uri, "login-failed", nsnull);
+    NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::ForgetGroupPassword()
 {
-    nsresult rv;
-    nsCOMPtr<nsIWalletService> walletservice = 
-             do_GetService(kWalletServiceCID, &rv);
-    if (NS_FAILED(rv)) return rv;
-
+    nsresult rv = NS_OK;
+    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
+    NS_ENSURE_SUCCESS(rv,rv);
     rv = SetGroupPassword(nsnull);
     if (NS_FAILED(rv)) return rv;
 
@@ -1300,7 +1335,15 @@ NS_IMETHODIMP nsMsgNewsFolder::ForgetGroupPassword()
     rv = CreateNewsgroupPasswordUrlForSignon(mURI, getter_Copies(signonURL));
     if (NS_FAILED(rv)) return rv;
 
-    rv = walletservice->SI_RemoveUser((const char *)signonURL, nsnull);
+    nsCOMPtr<nsIURI> uri;
+    NS_NewURI(getter_AddRefs(uri), signonURL);
+
+    //this is need to make sure wallet service has been created
+    rv = CreateServicesForPasswordManager();
+    NS_ENSURE_SUCCESS(rv, rv);
+
+    rv = observerService->NotifyObservers(uri, "login-failed", nsnull);
+    NS_ENSURE_SUCCESS(rv,rv);
     return rv;
 }
 
@@ -1490,6 +1533,12 @@ NS_IMETHODIMP nsMsgNewsFolder::SetReadSetFromStr(const char *newsrcLine)
 
     if (!mReadSet) return NS_ERROR_OUT_OF_MEMORY;
 
+    // Now that mReadSet is recreated, make sure it's stored in the db as well.
+    nsresult rv;
+    nsCOMPtr<nsINewsDatabase> db = do_QueryInterface(mDatabase, &rv);
+    if (NS_SUCCEEDED(rv) && db) // it's ok not to have a db here.
+      db->SetReadSet(mReadSet);
+
     return NS_OK;
 }
 
@@ -1574,7 +1623,9 @@ nsMsgNewsFolder::GetNntpServer(nsINntpIncomingServer **result)
 // it removes the cancelled message from the db
 NS_IMETHODIMP nsMsgNewsFolder::RemoveMessage(nsMsgKey key)
 {
-  return mDatabase->DeleteMessage(key, nsnull, PR_TRUE);
+  nsresult rv = GetDatabase(nsnull);
+  NS_ENSURE_SUCCESS(rv, rv); // if GetDatabase succeeds, mDatabase will be non-null
+  return mDatabase->DeleteMessage(key, nsnull, PR_FALSE);
 }
 
 NS_IMETHODIMP nsMsgNewsFolder::CancelComplete()
@@ -1705,7 +1756,7 @@ NS_IMETHODIMP nsMsgNewsFolder::NotifyDownloadedLine(const char *line, nsMsgKey k
     {
       PRUint32 count = 0;
       rv = m_tempMessageStream->Write(line, 
-         nsCRT::strlen(line), &count);
+           strlen(line), &count);
       if (NS_SUCCEEDED(rv))
         rv = m_tempMessageStream->Write(MSG_LINEBREAK, MSG_LINEBREAK_LEN, &count);
       NS_ASSERTION(NS_SUCCEEDED(rv), "failed to write to stream");
@@ -1781,3 +1832,12 @@ NS_IMETHODIMP nsMsgNewsFolder::GetPersistElided(PRBool *aPersistElided)
   NS_ENSURE_SUCCESS(rv,rv);
   return rv;
 }
+
+
+NS_IMETHODIMP nsMsgNewsFolder::Shutdown(PRBool shutdownChildren)
+{
+  mInitialized = PR_FALSE;
+  mReadSet = nsnull;
+  return nsMsgDBFolder::Shutdown(shutdownChildren);
+}
+
