@@ -92,7 +92,7 @@ NS_INTERFACE_MAP_BEGIN(nsNntpIncomingServer)
     NS_INTERFACE_MAP_ENTRY(nsINntpIncomingServer)
     NS_INTERFACE_MAP_ENTRY(nsIUrlListener)
     NS_INTERFACE_MAP_ENTRY(nsISubscribableServer)
-    NS_INTERFACE_MAP_ENTRY(nsIOutlinerView)
+    NS_INTERFACE_MAP_ENTRY(nsITreeView)
 NS_INTERFACE_MAP_END_INHERITING(nsMsgIncomingServer)
 
 nsNntpIncomingServer::nsNntpIncomingServer() : nsMsgLineBuffer(nsnull, PR_FALSE)
@@ -271,13 +271,9 @@ nsNntpIncomingServer::GetNewsrcRootPath(nsIFileSpec **aNewsrcRootPath)
     
     // Make the resulting nsIFileSpec
     // TODO: Convert arg to nsILocalFile and avoid this
-    nsXPIDLCString pathBuf;
-    rv = localFile->GetPath(getter_Copies(pathBuf));
-    if (NS_FAILED(rv)) return rv;
     nsCOMPtr<nsIFileSpec> outSpec;
-    rv = NS_NewFileSpec(getter_AddRefs(outSpec));
+    rv = NS_NewFileSpecFromIFile(localFile, getter_AddRefs(outSpec));
     if (NS_FAILED(rv)) return rv;
-    outSpec->SetNativePath(pathBuf);
     
     if (!havePref || !exists)
         rv = SetNewsrcRootPath(outSpec);
@@ -306,7 +302,7 @@ nsresult nsNntpIncomingServer::SetupNewsrcSaveTimer()
 		mNewsrcSaveTimer->Cancel();
 	}
     mNewsrcSaveTimer = do_CreateInstance("@mozilla.org/timer;1");
-	mNewsrcSaveTimer->Init(OnNewsrcSaveTimer, (void*)this, timeInMSUint32, NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_SLACK);
+	mNewsrcSaveTimer->Init(OnNewsrcSaveTimer, (void*)this, timeInMSUint32, PR_TRUE, NS_TYPE_REPEATING_SLACK);
 
     return NS_OK;
 }
@@ -621,7 +617,7 @@ nsNntpIncomingServer::PerformExpand(nsIMsgWindow *aMsgWindow)
 
 // remove this when news supports filters
 NS_IMETHODIMP
-nsNntpIncomingServer::GetFilterList(nsIMsgFilterList **aResult)
+nsNntpIncomingServer::GetFilterList(nsIMsgWindow *aMsgWindow, nsIMsgFilterList **aResult)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -758,9 +754,9 @@ checkIfSubscribedFunction(nsCString &aElement, void *aData)
 NS_IMETHODIMP
 nsNntpIncomingServer::ContainsNewsgroup(const char *name, PRBool *containsGroup)
 {
-	NS_ASSERTION(name && nsCRT::strlen(name),"no name");
+	NS_ASSERTION(name && strlen(name),"no name");
 	if (!name || !containsGroup) return NS_ERROR_NULL_POINTER;
-	if (!nsCRT::strlen(name)) return NS_ERROR_FAILURE;
+	if (!strlen(name)) return NS_ERROR_FAILURE;
 
 	*containsGroup = !(mSubscribedNewsgroups.EnumerateForwards((nsCStringArrayEnumFunc)checkIfSubscribedFunction, (void *)name));
 	return NS_OK;
@@ -771,9 +767,9 @@ nsNntpIncomingServer::SubscribeToNewsgroup(const char *name)
 {
 	nsresult rv;
 
-	NS_ASSERTION(name && nsCRT::strlen(name),"no name");
+	NS_ASSERTION(name && strlen(name),"no name");
 	if (!name) return NS_ERROR_NULL_POINTER;
-	if (!nsCRT::strlen(name)) return NS_ERROR_FAILURE;
+	if (!strlen(name)) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIMsgFolder> msgfolder;
 	rv = GetRootMsgFolder(getter_AddRefs(msgfolder));
@@ -1716,9 +1712,9 @@ nsNntpIncomingServer::SetSearchValue(const char *searchValue)
     mSubscribeSearchResult.Clear();
     mGroupsOnServer.EnumerateForwards((nsCStringArrayEnumFunc)buildSubscribeSearchResult, (void *)this);
 
-    if (mOutliner) {
-     mOutliner->Invalidate();
-     mOutliner->InvalidateScrollbar();
+    if (mTree) {
+     mTree->Invalidate();
+     mTree->InvalidateScrollbar();
     }
     return NS_OK;
 }
@@ -1738,17 +1734,17 @@ nsNntpIncomingServer::GetRowCount(PRInt32 *aRowCount)
 }
 
 NS_IMETHODIMP 
-nsNntpIncomingServer::GetSelection(nsIOutlinerSelection * *aSelection)
+nsNntpIncomingServer::GetSelection(nsITreeSelection * *aSelection)
 {
-  *aSelection = mOutlinerSelection;
+  *aSelection = mTreeSelection;
   NS_IF_ADDREF(*aSelection);
   return NS_OK;
 }
 
 NS_IMETHODIMP 
-nsNntpIncomingServer::SetSelection(nsIOutlinerSelection * aSelection)
+nsNntpIncomingServer::SetSelection(nsITreeSelection * aSelection)
 {
-  mOutlinerSelection = aSelection;
+  mTreeSelection = aSelection;
   return NS_OK;
 }
 
@@ -1863,6 +1859,24 @@ nsNntpIncomingServer::IsValidRow(PRInt32 row)
     return ((row >= 0) && (row < mSubscribeSearchResult.Count()));
 }
 
+NS_IMETHODIMP 
+nsNntpIncomingServer::GetImageSrc(PRInt32 row, const PRUnichar *colID, nsAString& _retval)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsNntpIncomingServer::GetProgressMode(PRInt32 row, const PRUnichar *colID, PRInt32* _retval)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP 
+nsNntpIncomingServer::GetCellValue(PRInt32 row, const PRUnichar *colID, nsAString& _retval)
+{
+  return NS_OK;
+}
+
 NS_IMETHODIMP 
 nsNntpIncomingServer::GetCellText(PRInt32 row, const PRUnichar *colID, nsAString& _retval)
 {
@@ -1884,9 +1898,9 @@ nsNntpIncomingServer::GetCellText(PRInt32 row, const PRUnichar *colID, nsAString
 }
 
 NS_IMETHODIMP 
-nsNntpIncomingServer::SetOutliner(nsIOutlinerBoxObject *outliner)
+nsNntpIncomingServer::SetTree(nsITreeBoxObject *tree)
 {
-  mOutliner = outliner;
+  mTree = tree;
   return NS_OK;
 }
 
