@@ -22,6 +22,7 @@
  *     Daniel Veditz <dveditz@netscape.com>
  *     Samir Gehani <sgehani@netscape.com>
  *     Mitch Stoltz <mstoltz@netscape.com>
+ *     Jeroen Dobbelaere <jeroen.dobbelaere@acunia.com>
  */
 
 /* 
@@ -44,14 +45,19 @@
 #define READTYPE  PRInt32
 #include "zlib.h"
 #include "nsISupportsUtils.h"
-#include "nsZlibAllocator.h"
+#include "nsRecyclingAllocator.h"
 
 /**
  * Globals
  *
  * Global allocator used with zlib. Destroyed in module shutdown.
  */
-nsZlibAllocator *gZlibAllocator = NULL;
+#define NBUCKETS 6
+#define BY4ALLOC_ITEMS 320
+nsRecyclingAllocator *gZlibAllocator = NULL;
+
+// For placement new used for arena allocations of zip file list
+#include <new.h>
 
 #else /* STANDALONE */
 
@@ -382,148 +388,17 @@ void ProcessWindowsMessages()
 // we startup and disabled after we startup if memory is a concern.
 //***********************************************************
 
-void * nsZlibAllocator::zAlloc(PRUint32 items, PRUint32 bytes)
-{
-  PRUint32 totalsize = items * bytes;
-  PRInt32 freeAllocatedBucketIndex = -1;
-  PRUint32 i;
-  PRUint32 size;
-  void *ptr;
-
-  for (i = 0; i < NBUCKETS; i++)
-  {
-    size = mMemBucket[i].size;
-    ptr = mMemBucket[i].ptr;
-
-    // Dont look at buckets with no memory allocated or less memory than
-    // what we need.
-    // Can we do this check without claiming the bucket? I think we can
-    // because the next thing we do is try claim this bucket.
-    if (!ptr || size < totalsize)
-      continue;
-
-    // Try Claim a bucket. If we cant, skip it.
-    if (!Claim(i))
-      continue;
-
-    if (size == totalsize)
-    {
-      // Let go of any freeAllocatedBucket that we claimed
-      if (freeAllocatedBucketIndex >= 0)
-        Unclaim(freeAllocatedBucketIndex);
-
-      // zero out memory and return it
-      memset(ptr, 0, totalsize);
-      return ptr;
-    }
-    // Meanwhile, remember a free allocated bucket.
-    // At this point we know the bucket is not in use and it has more
-    // than what we need
-    if (freeAllocatedBucketIndex < 0)
-      freeAllocatedBucketIndex = i;
-    else
-    {
-      // See if this bucket is closer to what we need
-      if (size < mMemBucket[freeAllocatedBucketIndex].size)
-      {
-        Unclaim(freeAllocatedBucketIndex);
-        freeAllocatedBucketIndex = i;
-      }
-      else
-        // Undo our claim as freeAllocatedBucketIndex does better than this one
-        Unclaim(i);
-    }
-  }
-
-  // See if we have an allocated bucket
-  if (freeAllocatedBucketIndex >= 0)
-  {
-    ptr = mMemBucket[freeAllocatedBucketIndex].ptr;
-    // Clear it, Mark it used and return ptr
-    // We need to clear only the size that was requested although
-    // the bucket may be larger
-    memset(ptr, 0, totalsize);
-    return ptr;
-  }
-
-  // We dont have that memory already
-  // Allocate. Make sure we bump up our allocations of x4 allocations
-  int realitems = items;
-  if (bytes == 4 && items < BY4ALLOC_ITEMS)
-    realitems = BY4ALLOC_ITEMS;
-  ptr = calloc(realitems, bytes);
-
-  // Take care of no memory situation
-  if (!ptr)
-    return ptr;
-  
-  // Find a free unallocated bucket and store allocation
-  for (i = 0; i < NBUCKETS; i++)
-  {
-    // If bucket cannot be calimed, continue search.
-    if (!Claim(i))
-      continue;
-
-    if (!mMemBucket[i].ptr)
-    {
-      // Found free slot. Store it
-      mMemBucket[i].ptr = ptr;
-      mMemBucket[i].size = realitems * bytes;
-      return ptr;
-    }
-
-    // This is an already allocated bucket. Cant use this one.
-    Unclaim(i);
-  }
-#ifdef DEBUG_dp
-  // Warn if we are failing over to calloc and not storing it
-  // This says we have a misdesigned memory pool. The intent was
-  // once the pool was full, we would never fail over to calloc.
-  printf("zalloc %d [%dx%d] - FAILOVER 0x%p Memory pool has sizes: ",
-         items*bytes, items, bytes, ptr);
-  for (i = 0; i < NBUCKETS; i++)
-  {
-    printf("%d ", mMemBucket[i].size);
-  }
-  printf("\n");
-#endif
-
-  return ptr;
-}
-
-void nsZlibAllocator::zFree(void *ptr)
-{
-  PRUint32 i;
-
-  for (i = 0; i < NBUCKETS; i++)
-  {
-    if (mMemBucket[i].ptr == ptr)
-    {
-      // Ah ha. One of the slots we allocated.
-      // Nothing to do. Mark it unused.
-      Unclaim(i);
-      return;
-    }
-  }
-
-#ifdef DEBUG_dp
-  // Warn if we are failing over to free.
-  // This says we have a misdesigned memory pool. The intent was
-  // once the pool was full, we would never fail over to free.
-  printf("DEBUG: zlib memory pool freeing 0x%p\n", ptr);
-#endif
-
-  // Failover to free
-  free(ptr);
-  return;
-}
-
 PR_STATIC_CALLBACK(void *)
 zlibAlloc(void *opaque, uInt items, uInt size)
 {
-  nsZlibAllocator *zallocator = (nsZlibAllocator *)opaque;
-  if (zallocator)
-    return zallocator->zAlloc(items, size);
+  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
+  if (zallocator) {
+    // Bump up x4 allocations
+    PRUint32 realitems = items;
+    if (size == 4 && items < BY4ALLOC_ITEMS)
+      realitems = BY4ALLOC_ITEMS;
+     return zallocator->Calloc(realitems, size);
+  }
   else
     return calloc(items, size);
 }
@@ -531,9 +406,9 @@ zlibAlloc(void *opaque, uInt items, uInt size)
 PR_STATIC_CALLBACK(void)
 zlibFree(void *opaque, void *ptr)
 {
-  nsZlibAllocator *zallocator = (nsZlibAllocator *)opaque;
+  nsRecyclingAllocator *zallocator = (nsRecyclingAllocator *)opaque;
   if (zallocator)
-    zallocator->zFree(ptr);
+    zallocator->Free(ptr);
   else
     free(ptr);
   return;
@@ -636,6 +511,17 @@ PRInt32 nsZipArchive::CloseArchive()
 	mFd = 0;
   }
 
+#ifndef STANDALONE
+  // delete arena for filelist
+  PL_FinishArenaPool(&mArena);
+
+  // CAUTION:
+  // We dont need to delete each of the nsZipItem as the memory for
+  // the zip item and the filename it holds are both allocated from the Arena.
+  // Hence, destroying the Arena is like destroying all the memory
+  // for all the nsZipItem in one shot. But if the ~nsZipItem is doing
+  // anything more than cleaning up memory, we should start calling it.
+#else
   // delete nsZipItems in table
   nsZipItem* pItem;
   for ( int i = 0; i < ZIP_TABSIZE; ++i)
@@ -648,6 +534,8 @@ PRInt32 nsZipArchive::CloseArchive()
       pItem = mFiles[i];
     }
   }
+#endif
+
   return ZIP_OK;
 }
 
@@ -1022,7 +910,7 @@ PRInt32 nsZipArchive::BuildFileList()
     PRUint32 endsig;
     PRBool bEndsigFound = PR_FALSE;
 
-    for (endp -= sizeof(ZipEnd); endp >= buf; endp--)
+    for (endp -= ZIPEND_SIZE; endp >= buf; endp--)
     {
 		endsig = xtolong(endp);
 		if (endsig == ENDSIG)
@@ -1050,7 +938,7 @@ PRInt32 nsZipArchive::BuildFileList()
       status = ZIP_ERR_CORRUPT;
 
     //-- backward read must overlap ZipEnd length
-    pos += sizeof(ZipEnd);
+    pos += ZIPEND_SIZE;
 
   } /* while looking for end signature */
 
@@ -1063,7 +951,7 @@ PRInt32 nsZipArchive::BuildFileList()
     //-- we think we found the central directory, read in the first chunk
     pos = 0;
     bufsize = PR_Read( mFd, &buf, sizeof(buf) );
-    if (bufsize < sizeof(ZipCentral) + sizeof(ZipEnd))
+    if (bufsize < (PRInt32)(ZIPCENTRAL_SIZE + ZIPEND_SIZE))
     {
       // We know we read the end sig and got pointed at the central
       // directory--there should be at least this much
@@ -1092,8 +980,15 @@ PRInt32 nsZipArchive::BuildFileList()
     PRUint32 namelen = xtoint( central->filename_len );
     PRUint32 extralen = xtoint( central->extrafield_len );
     PRUint32 commentlen = xtoint( central->commentfield_len );
-    
+#ifndef STANDALONE
+    // Arena allocate the nsZipItem
+    void *mem;
+    PL_ARENA_ALLOCATE(mem, &mArena, sizeof(nsZipItem));
+    // Use placement new to arena allcoate the nsZipItem
+    nsZipItem* item = mem ? new (mem) nsZipItem() : nsnull;
+#else
     nsZipItem* item = new nsZipItem();
+#endif
     if (!item)
     {
       status = ZIP_ERR_MEMORY;
@@ -1120,12 +1015,24 @@ PRInt32 nsZipArchive::BuildFileList()
     item->time = xtoint( central->time );
     item->date = xtoint( central->date );
 
-    pos += sizeof(ZipCentral);
+    pos += ZIPCENTRAL_SIZE;
 
     //-------------------------------------------------------
     // get the item name
     //-------------------------------------------------------
+#ifndef STANDALONE
+    PL_ARENA_ALLOCATE(mem, &mArena, (namelen + 1));
+    item->name = (char *) mem;
+    if ( !item->name )
+    {
+      status = ZIP_ERR_MEMORY;
+      // No need to delete name. It gets deleted only when the entire arena
+      // goes away.
+      break;
+    }
+#else
     item->name = new char[namelen + 1];
+#endif
     if ( !item->name )
     {
       status = ZIP_ERR_MEMORY;
@@ -1163,7 +1070,7 @@ PRInt32 nsZipArchive::BuildFileList()
     // set up to process the next item at the top of loop
     //-------------------------------------------------------
     leftover = (PRUint32)(bufsize - pos);
-    if ( leftover < (extralen + commentlen + sizeof(ZipCentral)) )
+    if ( leftover < (extralen + commentlen + ZIPCENTRAL_SIZE) )
     {
       //-- not enough data left to process at top of loop.
       //-- move leftover and read more
@@ -1184,13 +1091,20 @@ PRInt32 nsZipArchive::BuildFileList()
     }
 
     //-- make sure we've read enough
-    if ( (PRUint32)bufsize < pos + sizeof(ZipCentral) )
+    if ( bufsize < pos + ZIPCENTRAL_SIZE )
     {
       status = ZIP_ERR_CORRUPT;
       break;
     }
   } /* while reading central directory records */
 
+#if defined(DEBUG)
+  if (status != ZIP_OK) {
+    const char* msgs[] = { "ZIP_OK", "ZIP_ERR_GENERAL", "ZIP_ERR_MEMORY", "ZIP_ERR_DISK", "ZIP_ERR_CORRUPT", "ZIP_ERR_PARAM", "ZIP_ERR_FNF", "ZIP_ERR_UNSUPPORTED", "ZIP_ERR_SMALLBUF", "UNKNOWN" };
+    printf("nsZipArchive::BuildFileList  status = %d '%s'\n", (int)status, msgs[(status <= 0 && status >= -8) ? -status : 9]);
+  }
+#endif
+
   return status;
 }
   
@@ -1251,14 +1165,14 @@ PRInt32  nsZipArchive::SeekToItem(const nsZipItem* aItem)
       return ZIP_ERR_CORRUPT;
 
     ZipLocal   Local;
-    if ( PR_Read(mFd, (char*)&Local, sizeof(ZipLocal)) != (READTYPE)sizeof(ZipLocal)
+    if ( PR_Read(mFd, (char*)&Local, ZIPLOCAL_SIZE) != (READTYPE) ZIPLOCAL_SIZE
          || xtolong( Local.signature ) != LOCALSIG )
     {
       //-- read error or local header not found
       return ZIP_ERR_CORRUPT;
     }
 
-    ((nsZipItem*)aItem)->offset += sizeof(Local) +
+    ((nsZipItem*)aItem)->offset += ZIPLOCAL_SIZE +
                                    xtoint( Local.filename_len ) +
                                    xtoint( Local.extrafield_len );
     ((nsZipItem*)aItem)->flags |= ZIFLAG_DATAOFFSET;
@@ -1404,7 +1318,7 @@ PRInt32 nsZipArchive::InflateItem( const nsZipItem* aItem, PRFileDesc* fOut,
 #ifndef STANDALONE
   //-- ensure we have our zlib allocator for better performance
   if (!gZlibAllocator) {
-    gZlibAllocator = new nsZlibAllocator();
+    gZlibAllocator = new nsRecyclingAllocator(NBUCKETS, NS_DEFAULT_RECYCLE_TIMEOUT, "libjar");
   }
 
   zs.zalloc = zlibAlloc;
@@ -1703,15 +1617,17 @@ cleanup:
 MOZ_DECL_CTOR_COUNTER(nsZipArchive)
 
 nsZipArchive::nsZipArchive()
-  : kMagic(ZIP_MAGIC), mFd(0)
+  : kMagic(ZIP_MAGIC), kArenaBlockSize(1*1024), mFd(0)
 {
   MOZ_COUNT_CTOR(nsZipArchive);
 
   // initialize the table to NULL
-  for ( int i = 0; i < ZIP_TABSIZE; ++i) {
-    mFiles[i] = 0;
-  }
+  memset(mFiles, 0, sizeof mFiles);
 
+#ifndef STANDALONE
+  // Initialize our arena
+  PL_INIT_ARENA_POOL(&mArena, "ZipArena", kArenaBlockSize);
+#endif
 }
 
 nsZipArchive::~nsZipArchive()
@@ -1728,22 +1644,19 @@ nsZipArchive::~nsZipArchive()
 // nsZipItem constructor and destructor
 //------------------------------------------
 
-MOZ_DECL_CTOR_COUNTER(nsZipItem)
-
 nsZipItem::nsZipItem() : name(0), offset(0), next(0), flags(0)
 {
-  MOZ_COUNT_CTOR(nsZipItem);
 }
 
 nsZipItem::~nsZipItem()
 {
+#ifdef STANDALONE
   if (name != 0 )
   {
     delete [] name;
     name = 0;
   }
-
-  MOZ_COUNT_DTOR(nsZipItem);
+#endif
 }
 
 //------------------------------------------
