@@ -33,6 +33,52 @@
 
 #include "nsRect.h"
 
+//////////////////////////////////////////////////////////////////////
+// Declaration of the static callbacks
+
+static int PR_CALLBACK BeginGIF(
+  void*    aClientData,
+  PRUint32 aLogicalScreenWidth, 
+  PRUint32 aLogicalScreenHeight,
+  PRUint8  aBackgroundRGBIndex);
+  
+static int PR_CALLBACK HaveDecodedRow(
+  void* aClientData,
+  PRUint8* aRowBufPtr,   // Pointer to single scanline temporary buffer
+  int aXOffset,          // With respect to GIF logical screen origin
+  int aLength,           // Length of the row?
+  int aRow,              // Row number?
+  int aDuplicateCount,   // Number of times to duplicate the row?
+  PRUint8 aDrawMode,     // il_draw_mode
+  int aInterlacePass);
+    
+static int PR_CALLBACK NewPixmap();
+
+static int PR_CALLBACK EndGIF(
+  void*    aClientData,
+  int      aAnimationLoopCount);
+  
+static int PR_CALLBACK BeginImageFrame(
+  void*    aClientData,
+  PRUint32 aFrameNumber,   /* Frame number, 1-n */
+  PRUint32 aFrameXOffset,  /* X offset in logical screen */
+  PRUint32 aFrameYOffset,  /* Y offset in logical screen */
+  PRUint32 aFrameWidth,    
+  PRUint32 aFrameHeight,   
+  GIF_RGB* aTransparencyChromaKey);
+static int PR_CALLBACK EndImageFrame(
+  void*    aClientData, 
+  PRUint32 aFrameNumber,
+  PRUint32 aDelayTimeout,
+  PRUint32 aDisposal);
+static int PR_CALLBACK SetupColorspaceConverter();
+static int PR_CALLBACK ResetPalette();
+static int PR_CALLBACK InitTransparentPixel();
+static int PR_CALLBACK DestroyTransparentPixel();
+
+static int PR_CALLBACK HaveImageAll(
+  void* aClientData);
+
 //////////////////////////////////////////////////////////////////////
 // GIF Decoder Implementation
 // This is an adaptor between GIF2 and imgIDecoder
@@ -146,7 +192,12 @@ static NS_METHOD ReadDataOut(nsIInputStream* in,
                              PRUint32 *writeCount)
 {
   nsGIFDecoder2 *decoder = NS_STATIC_CAST(nsGIFDecoder2*, closure);
-  *writeCount = decoder->ProcessData((unsigned char*)fromRawSegment, count);
+  nsresult rv = decoder->ProcessData((unsigned char*)fromRawSegment, count, writeCount);
+  if (NS_FAILED(rv)) {
+    *writeCount = 0;
+    return rv;
+  }
+
   return NS_OK;
 }
 
@@ -188,15 +239,17 @@ nsGIFDecoder2::FlushImageData()
 }
 
 //******************************************************************************
-PRUint32 nsGIFDecoder2::ProcessData(unsigned char *data, PRUint32 count)
+nsresult nsGIFDecoder2::ProcessData(unsigned char *data, PRUint32 count, PRUint32 *_retval)
 {
   // Push the data to the GIF decoder
   
   // First we ask if the gif decoder is ready for more data, and if so, push it.
   // In the new decoder, we should always be able to process more data since
   // we don't wait to decode each frame in an animation now.
-  if(gif_write_ready(mGIFStruct)) {
-    gif_write(mGIFStruct, data, count);
+  if (gif_write_ready(mGIFStruct)) {
+    PRStatus result = gif_write(mGIFStruct, data, count);
+    if (result != PR_SUCCESS)
+      return NS_ERROR_FAILURE;
   }
 
   if (mImageFrame && mObserver) {
@@ -205,22 +258,16 @@ PRUint32 nsGIFDecoder2::ProcessData(unsigned char *data, PRUint32 count)
     mLastFlushedPass = mCurrentPass;
   }
 
-  return count; // we always consume all the data
+  *_retval = count;
+
+  return NS_OK;
 }
 
 //******************************************************************************
 /* unsigned long writeFrom (in nsIInputStream inStr, in unsigned long count); */
 NS_IMETHODIMP nsGIFDecoder2::WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_retval)
 {
-  inStr->ReadSegments(
-    ReadDataOut, // Callback
-    this,     
-    count, 
-    _retval);
-
-    // if error
-    //mRequest->Cancel(NS_BINDING_ABORTED); // XXX is this the correct error ?
-  return NS_OK;
+  return inStr->ReadSegments(ReadDataOut, this,  count, _retval);
 }
 
 
@@ -370,9 +417,12 @@ int HaveDecodedRow(
 
     // initalize the frame and append it to the container
     decoder->mImageFrame = do_CreateInstance("@mozilla.org/gfx/image/frame;2");
-    decoder->mImageFrame->Init(
-      decoder->mGIFStruct->x_offset, decoder->mGIFStruct->y_offset, 
-      decoder->mGIFStruct->width, decoder->mGIFStruct->height, format);
+    if (NS_FAILED(decoder->mImageFrame->Init(
+          decoder->mGIFStruct->x_offset, decoder->mGIFStruct->y_offset, 
+          decoder->mGIFStruct->width, decoder->mGIFStruct->height, format))) {
+      decoder->mImageFrame = 0;
+      return 0;
+    }
 
     decoder->mImageContainer->AppendFrame(decoder->mImageFrame);
 
@@ -396,7 +446,6 @@ int HaveDecodedRow(
     nscoord width;
 
     decoder->mImageFrame->GetWidth(&width);
-    PRUint32 iwidth = width;
 
     gfx_format format;
     decoder->mImageFrame->GetFormat(&format);
@@ -426,16 +475,20 @@ int HaveDecodedRow(
     switch (format) {
     case gfxIFormats::RGB:
       {
-        while(rowBufIndex != decoder->mGIFStruct->rowend) {
-#if defined(XP_MAC) || defined(XP_MACOSX)
-          *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
-#endif
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-          ++rowBufIndex;
-        }
-
+          if (cmap) {// cmap could have null value if the global color table flag is 0
+              while (rowBufIndex != decoder->mGIFStruct->rowend) {
+              #if defined(XP_MAC) || defined(XP_MACOSX)
+                  *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
+              #endif
+                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
+                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
+                  *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
+                  ++rowBufIndex;
+            }
+          }
+          else 
+              memset(decoder->mRGBLine, 0, bpr);   
+          
         for (int i=0; i<aDuplicateCount; i++)
           decoder->mImageFrame->SetImageData(decoder->mRGBLine,
                                              bpr, (aRowNumber+i)*bpr);
@@ -443,12 +496,16 @@ int HaveDecodedRow(
       break;
     case gfxIFormats::BGR:
       {
-        while(rowBufIndex != decoder->mGIFStruct->rowend) {
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-          *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-          ++rowBufIndex;
+        if (cmap) {// cmap could have null value if the global color table flag is 0
+            while (rowBufIndex != decoder->mGIFStruct->rowend) {
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
+                ++rowBufIndex;
+            }
         }
+        else 
+            memset(decoder->mRGBLine, 0, bpr);   
 
         for (int i=0; i<aDuplicateCount; i++)
           decoder->mImageFrame->SetImageData(decoder->mRGBLine,
@@ -458,42 +515,44 @@ int HaveDecodedRow(
     case gfxIFormats::RGB_A1:
     case gfxIFormats::BGR_A1:
       {
-        if (decoder->mGIFStruct->is_transparent &&
+        gfx_color transColor = 0;
+        if (cmap && // cmap could have null value if the global color table flag is 0
+            decoder->mGIFStruct->is_transparent &&
             (decoder->mGIFStruct->tpixel < cmapsize)) {
-          gfx_color transColor = 0;
           transColor |= cmap[decoder->mGIFStruct->tpixel].red;
           transColor |= cmap[decoder->mGIFStruct->tpixel].green << 8;
           transColor |= cmap[decoder->mGIFStruct->tpixel].blue << 16;
-          decoder->mImageFrame->SetTransparentColor(transColor);
         }
-
+        decoder->mImageFrame->SetTransparentColor(transColor);
         memset(decoder->mRGBLine, 0, bpr);
         memset(decoder->mAlphaLine, 0, abpr);
         PRUint32 iwidth = (PRUint32)width;
-        for (PRUint32 x=0; x<iwidth; x++) {
-          if (*rowBufIndex != decoder->mGIFStruct->tpixel) {
-#if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-#else
-#if defined(XP_MAC) || defined(XP_MACOSX)
-          *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
-#endif
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
-            *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
-#endif
-            decoder->mAlphaLine[x>>3] |= 1<<(7-x&0x7);
-          } else {
-#if defined(XP_MAC) || defined(XP_MACOSX)
-            rgbRowIndex+=4;
-#else
-            rgbRowIndex+=3;
-#endif
-          }
+        if (cmap) {// cmap could have null value if the global color table flag is 0
+            for (PRUint32 x=0; x<iwidth; x++) {
+                if (*rowBufIndex != decoder->mGIFStruct->tpixel) {
+        #if defined(XP_PC) || defined(XP_BEOS) || defined(MOZ_WIDGET_PHOTON)
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
+        #else
+        #if defined(XP_MAC) || defined(XP_MACOSX)
+              *rgbRowIndex++ = 0; // Mac is always 32bits per pixel, this is pad
+        #endif
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].red;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].green;
+                *rgbRowIndex++ = cmap[PRUint8(*rowBufIndex)].blue;
+        #endif
+                decoder->mAlphaLine[x>>3] |= 1<<(7-x&0x7);
+              } else {
+        #if defined(XP_MAC) || defined(XP_MACOSX)
+                rgbRowIndex+=4;
+        #else
+                rgbRowIndex+=3;
+        #endif
+              }
 
-          ++rowBufIndex;
+              ++rowBufIndex;
+            }
         }
         for (int i=0; i<aDuplicateCount; i++) {
           decoder->mImageFrame->SetAlphaData(decoder->mAlphaLine,
