@@ -34,7 +34,7 @@
 #include "nsIURL.h"
 #include "nsNetUtil.h"
 #include "nsIFile.h"
-#include "nsIFileChannel.h"
+#include "nsIFileURL.h"
 #include "nsIMIMEService.h"
 #include "nsCExternalHandlerService.h"
 
@@ -72,7 +72,7 @@ nsresult nsIconChannel::Init(nsIURI* uri)
 ////////////////////////////////////////////////////////////////////////////////
 // nsIRequest methods:
 
-NS_IMETHODIMP nsIconChannel::GetName(PRUnichar* *result)
+NS_IMETHODIMP nsIconChannel::GetName(nsACString &result)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -161,7 +161,7 @@ void InvertRows(unsigned char * aInitialBuffer, PRUint32 sizeOfBuffer, PRUint32
   nsMemory::Free(temporaryRowHolder);
 }
 
-nsresult nsIconChannel::ExtractIconInfoFromUrl(nsIFile ** aLocalFile, PRUint32 * aDesiredImageSize, char ** aContentType, char ** aFileExtension)
+nsresult nsIconChannel::ExtractIconInfoFromUrl(nsIFile ** aLocalFile, PRUint32 * aDesiredImageSize, nsACString &aContentType, nsACString &aFileExtension)
 {
   nsresult rv = NS_OK;
   nsCOMPtr<nsIMozIconURI> iconURI (do_QueryInterface(mUrl, &rv));
@@ -298,10 +298,10 @@ void ConvertMaskBitMap(unsigned char * aBitMaskBuffer, PBITMAPINFOHEADER pBitMap
 NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports *ctxt)
 {
   nsXPIDLCString contentType;
-  nsXPIDLCString filePath;
+  nsCAutoString filePath;
   nsCOMPtr<nsIFile> localFile; // file we want an icon for
   PRUint32 desiredImageSize;
-  nsresult rv = ExtractIconInfoFromUrl(getter_AddRefs(localFile), &desiredImageSize, getter_Copies(contentType), getter_Copies(filePath));
+  nsresult rv = ExtractIconInfoFromUrl(getter_AddRefs(localFile), &desiredImageSize, contentType, filePath);
   NS_ENSURE_SUCCESS(rv, rv);
 
   // if the file exists, we are going to use it's real attributes...otherwise we only want to use it for it's extension...
@@ -312,7 +312,7 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
  
   if (localFile)
   {
-    localFile->GetPath(getter_Copies(filePath));
+    localFile->GetNativePath(filePath);
     localFile->Exists(&fileExists);
   }
 
@@ -324,7 +324,7 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
   else
     infoFlags |= SHGFI_SMALLICON;
 
-  if ( (!filePath.get()) && (contentType.get() && *contentType.get()) ) // if we have a content type without a file extension...then use it!
+  if ( (filePath.IsEmpty()) && (contentType.get() && *contentType.get()) ) // if we have a content type without a file extension...then use it!
   {
     nsCOMPtr<nsIMIMEService> mimeService (do_GetService(NS_MIMESERVICE_CONTRACTID, &rv));
     nsCOMPtr<nsIMIMEInfo> mimeObject;
@@ -336,10 +336,7 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
       nsXPIDLCString fileExt;
       mimeObject->GetPrimaryExtension(getter_Copies(fileExt));
       // we need to insert a '.' b4 the extension...
-      nsCAutoString formattedFileExt;
-      formattedFileExt = ".";
-      formattedFileExt.Append(fileExt.get());
-      filePath.Assign(formattedFileExt);
+      filePath = NS_LITERAL_CSTRING(".") + fileExt;
     }
   }
 
@@ -354,9 +351,9 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
     {
       nsCString iconBuffer;
 
+     // Alloc and zero init
      HANDLE h_bmp_info = GlobalAlloc (GHND, sizeof (BITMAPINFO) + (sizeof (RGBQUAD) * 256));
      BITMAPINFO* pBitMapInfo = (BITMAPINFO *) GlobalLock (h_bmp_info);
-     memset (pBitMapInfo, NULL, sizeof (BITMAPINFO) + (sizeof (RGBQUAD) * 255));
      pBitMapInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
      pBitMapInfo->bmiHeader.biPlanes = 1;
      pBitMapInfo->bmiHeader.biBitCount = 0;
@@ -374,8 +371,16 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
           // temporary hack alert...currently moz-icon urls only support 16, 24 and 32 bit color. we don't support
           // 8, 4 or 1 bit color yet. return an error in those cases
           if (pBitMapInfo->bmiHeader.biBitCount <= 8)
-            return NS_ERROR_FAILURE; 
-
+          {
+            nsMemory::Free(buffer);
+            GlobalUnlock(h_bmp_info);
+            GlobalFree(h_bmp_info);
+            DeleteDC(pDC);
+            DeleteObject(pIconInfo.hbmColor);
+            DeleteObject(pIconInfo.hbmMask);
+            DestroyIcon(sfi.hIcon);
+            return NS_ERROR_FAILURE;
+	  }
           // The first 2 bytes into our output buffer needs to be the width and the height (in pixels) of the icon
           // as specified by our data format.
           iconBuffer.Assign((char) pBitMapInfo->bmiHeader.biWidth);
@@ -416,7 +421,10 @@ NS_IMETHODIMP nsIconChannel::AsyncOpen(nsIStreamListener *aListener, nsISupports
       GlobalUnlock(h_bmp_info);
       GlobalFree(h_bmp_info);
       DeleteDC(pDC);
+      DeleteObject(pIconInfo.hbmColor);
+      DeleteObject(pIconInfo.hbmMask);
     } // if we got icon info
+    DestroyIcon(sfi.hIcon);
   } // if we got sfi
 
   return rv;
@@ -434,19 +442,30 @@ NS_IMETHODIMP nsIconChannel::SetLoadFlags(PRUint32 aLoadAttributes)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsIconChannel::GetContentType(char* *aContentType) 
+NS_IMETHODIMP nsIconChannel::GetContentType(nsACString &aContentType) 
 {
-  if (!aContentType) return NS_ERROR_NULL_POINTER;
-
-  *aContentType = nsCRT::strdup("image/icon");
-  if (!*aContentType) return NS_ERROR_OUT_OF_MEMORY;
+  aContentType = NS_LITERAL_CSTRING("image/icon");
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsIconChannel::SetContentType(const char *aContentType)
+nsIconChannel::SetContentType(const nsACString &aContentType)
 {
-  //It doesn't make sense to set the content-type on this type
+  // It doesn't make sense to set the content-type on this type
+  // of channel...
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsIconChannel::GetContentCharset(nsACString &aContentCharset) 
+{
+  aContentCharset.Truncate();
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsIconChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+  // It doesn't make sense to set the content-charset on this type
   // of channel...
   return NS_ERROR_FAILURE;
 }
