@@ -28,6 +28,7 @@
 #include "nsIChannel.h"
 #include "nsIHttpChannel.h"
 #include "nsICachingChannel.h"
+#include "nsIInterfaceRequestor.h"
 #include "nsIIOService.h"
 #include "nsILoadGroup.h"
 #include "nsIProxyObjectManager.h"
@@ -35,6 +36,9 @@
 #include "nsIStreamListener.h"
 #include "nsIURI.h"
 #include "nsXPIDLString.h"
+#include "nsCRT.h"
+
+#include "netCore.h"
 
 #include "imgCache.h"
 #include "imgRequest.h"
@@ -42,7 +46,7 @@
 
 #include "ImageLogging.h"
 
-#include "nsMimeTypes.h"
+#include "nsIComponentRegistrar.h"
 
 #ifdef DEBUG_pavlov
 #include "nsIEnumerator.h"
@@ -107,8 +111,11 @@ inline int merge_flags(const nsLoadFlags& inFlags, nsLoadFlags& outFlags)
   return 1;
 }
 
-/* imgIRequest loadImage (in nsIURI aURI, in nsILoadGroup aLoadGroup, in imgIDecoderObserver aObserver, in nsISupports aCX, in nsLoadFlags aLoadFlags, in nsISupports cacheKey, in imgIRequest aRequest); */
-NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsILoadGroup *aLoadGroup, imgIDecoderObserver *aObserver, nsISupports *aCX, nsLoadFlags aLoadFlags, nsISupports *cacheKey, imgIRequest *aRequest, imgIRequest **_retval)
+/* imgIRequest loadImage (in nsIURI aURI, in nsIURI parentURI, in nsILoadGroup aLoadGroup, in imgIDecoderObserver aObserver, in nsISupports aCX, in nsLoadFlags aLoadFlags, in nsISupports cacheKey, in imgIRequest aRequest); */
+
+NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsIURI *parentURI, nsILoadGroup *aLoadGroup,
+                                   imgIDecoderObserver *aObserver, nsISupports *aCX, nsLoadFlags aLoadFlags,
+                                   nsISupports *cacheKey, imgIRequest *aRequest, imgIRequest **_retval)
 {
   NS_ASSERTION(aURI, "imgLoader::LoadImage -- NULL URI pointer");
 
@@ -116,8 +123,8 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsILoadGroup *aLoadGroup, imgID
     return NS_ERROR_NULL_POINTER;
 
 #if defined(PR_LOGGING)
-  nsXPIDLCString spec;
-  aURI->GetSpec(getter_Copies(spec));
+  nsCAutoString spec;
+  aURI->GetAsciiSpec(spec);
   LOG_SCOPE_WITH_PARAM(gImgLog, "imgLoader::LoadImage", "aURI", spec.get());
 #endif
 
@@ -289,6 +296,11 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsILoadGroup *aLoadGroup, imgID
     ioserv->NewChannelFromURI(aURI, getter_AddRefs(newChannel));
     if (!newChannel) return NS_ERROR_FAILURE;
 
+    nsCOMPtr<nsIHttpChannel> newHttpChannel = do_QueryInterface(newChannel);
+    if (newHttpChannel) {
+      newHttpChannel->SetDocumentURI(parentURI);
+    }
+
     if (aLoadGroup) {
       PRUint32 flags;
       aLoadGroup->GetLoadFlags(&flags);
@@ -310,8 +322,6 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsILoadGroup *aLoadGroup, imgID
 
     request->Init(newChannel, entry, cacheId, aCX);
 
-    PR_LOG(gImgLog, PR_LOG_DEBUG,
-           ("[this=%p] imgLoader::LoadImage -- Calling channel->AsyncOpen()\n", this));
 
     // create the proxy listener
     ProxyListener *pl = new ProxyListener(NS_STATIC_CAST(nsIStreamListener *, request));
@@ -322,33 +332,41 @@ NS_IMETHODIMP imgLoader::LoadImage(nsIURI *aURI, nsILoadGroup *aLoadGroup, imgID
 
     NS_ADDREF(pl);
 
-    // set the referrer if this is an HTTP request
-    nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(newChannel));
-    if (aLoadGroup && httpChannel) {
-      nsresult rv;
-      // Get the defloadRequest from the loadgroup
-      nsCOMPtr<nsIRequest> defLoadRequest;
-      rv = aLoadGroup->GetDefaultLoadRequest(getter_AddRefs(defLoadRequest));
+    if (aLoadGroup) {
+      // Get the notification callbacks from the load group and set them on the channel
+      nsCOMPtr<nsIInterfaceRequestor> interfaceRequestor;
+      aLoadGroup->GetNotificationCallbacks(getter_AddRefs(interfaceRequestor));
+      if (interfaceRequestor)
+        newChannel->SetNotificationCallbacks(interfaceRequestor);
 
-      if (NS_SUCCEEDED(rv) && defLoadRequest) {
-        nsCOMPtr<nsIChannel> reqChannel(do_QueryInterface(defLoadRequest));
+      // set the referrer if this is an HTTP request
+      nsCOMPtr<nsIHttpChannel> httpChannel(do_QueryInterface(newChannel));
 
-        if (reqChannel) {
-          // Get the referrer from the loadchannel
-          nsCOMPtr<nsIURI> referrer;
-          rv = reqChannel->GetURI(getter_AddRefs(referrer));
-          if (NS_SUCCEEDED(rv)) {
-            // Set the referrer
-            httpChannel->SetReferrer(referrer, nsIHttpChannel::REFERRER_INLINES);
+      if (httpChannel) {
+        nsresult rv;
+        // Get the defloadRequest from the loadgroup
+        nsCOMPtr<nsIRequest> defLoadRequest;
+        rv = aLoadGroup->GetDefaultLoadRequest(getter_AddRefs(defLoadRequest));
+
+        if (NS_SUCCEEDED(rv) && defLoadRequest) {
+          nsCOMPtr<nsIChannel> reqChannel(do_QueryInterface(defLoadRequest));
+
+          if (reqChannel) {
+            // Get the referrer from the loadchannel
+            nsCOMPtr<nsIURI> referrer;
+            rv = reqChannel->GetURI(getter_AddRefs(referrer));
+            if (NS_SUCCEEDED(rv)) {
+              // Set the referrer
+              httpChannel->SetReferrer(referrer, nsIHttpChannel::REFERRER_INLINES);
+            }
           }   
         }
       }
     }
 
-    /* XXX Are we calling AsyncOpen() too early?  Is it possible for
-       AsyncOpen to result in an OnStartRequest to the channel before
-       we call CreateNewProxyForRequest() ?
-     */
+    PR_LOG(gImgLog, PR_LOG_DEBUG,
+           ("[this=%p] imgLoader::LoadImage -- Calling channel->AsyncOpen()\n", this));
+
     nsresult asyncOpenResult = newChannel->AsyncOpen(NS_STATIC_CAST(nsIStreamListener *, pl), nsnull);
 
     NS_RELEASE(pl);
@@ -509,6 +527,112 @@ imgLoader::CreateNewProxyForRequest(imgRequest *aRequest, nsILoadGroup *aLoadGro
   return NS_OK;
 }
 
+NS_IMETHODIMP imgLoader::SupportImageWithMimeType(const char* aMimeType, PRBool *_retval)
+{
+  *_retval = PR_FALSE;
+  nsCOMPtr<nsIComponentRegistrar> reg;
+  nsresult rv = NS_GetComponentRegistrar(getter_AddRefs(reg));
+  if (NS_FAILED(rv))
+    return rv;
+  nsCAutoString mimeType(aMimeType);
+  ToLowerCase(mimeType);
+  nsCAutoString decoderId(NS_LITERAL_CSTRING("@mozilla.org/image/decoder;2?type=") + mimeType);
+  return reg->IsContractIDRegistered(decoderId.get(),  _retval);
+}
+
+NS_IMETHODIMP imgLoader::SupportImageWithContents(const char* aContents, PRUint32 aLength, char** aContentType)
+{
+  return GetMimeTypeFromContent(aContents, aLength, aContentType);
+}
+
+/* static */
+nsresult imgLoader::GetMimeTypeFromContent(const char* aContents, PRUint32 aLength, char** aContentType)
+{
+  *aContentType = nsnull;
+  /* Is it a GIF? */
+  if (aLength >= 4 && !nsCRT::strncmp(aContents, "GIF8", 4))  {
+    *aContentType = nsCRT::strndup("image/gif", 9);
+    return NS_OK;
+  }
+
+  /* or a PNG? */
+  if (aLength >= 4 && ((unsigned char)aContents[0]==0x89 &&
+                   (unsigned char)aContents[1]==0x50 &&
+                   (unsigned char)aContents[2]==0x4E &&
+                   (unsigned char)aContents[3]==0x47))
+  { 
+    *aContentType = nsCRT::strndup("image/png", 9);
+    return NS_OK;
+  }
+
+  /* maybe a JPEG (JFIF)? */
+  /* JFIF files start with SOI APP0 but older files can start with SOI DQT
+   * so we test for SOI followed by any marker, i.e. FF D8 FF
+   * this will also work for SPIFF JPEG files if they appear in the future.
+   *
+   * (JFIF is 0XFF 0XD8 0XFF 0XE0 <skip 2> 0X4A 0X46 0X49 0X46 0X00)
+   */
+  if (aLength >= 3 &&
+     ((unsigned char)aContents[0])==0xFF &&
+     ((unsigned char)aContents[1])==0xD8 &&
+     ((unsigned char)aContents[2])==0xFF)
+  {
+    *aContentType = nsCRT::strndup("image/jpeg", 10);
+    return NS_OK;
+  }
+
+  /* or how about ART? */
+  /* ART begins with JG (4A 47). Major version offset 2.
+   * Minor version offset 3. Offset 4 must be NULL.
+   */
+  if (aLength >= 5 &&
+   ((unsigned char) aContents[0])==0x4a &&
+   ((unsigned char) aContents[1])==0x47 &&
+   ((unsigned char) aContents[4])==0x00 )
+  {
+    *aContentType = nsCRT::strndup("image/x-jg", 10);
+    return NS_OK;
+  }
+
+  if (aLength >= 2 && !nsCRT::strncmp(aContents, "BM", 2)) {
+    *aContentType = nsCRT::strndup("image/bmp", 9);
+    return NS_OK;
+  }
+
+  // ICOs always begin with a 2-byte 0 followed by a 2-byte 1.
+  if (aLength >= 4 && !memcmp(aContents, "\000\000\001\000", 4)) {
+    *aContentType = nsCRT::strndup("image/x-icon", 12);
+    return NS_OK;
+  }
+
+  if (aLength >= 4 && ((unsigned char)aContents[0]==0x8A &&
+                   (unsigned char)aContents[1]==0x4D &&
+                   (unsigned char)aContents[2]==0x4E &&
+                   (unsigned char)aContents[3]==0x47))
+  { 
+    *aContentType = nsCRT::strndup("video/x-mng", 11);
+    return NS_OK;
+  }
+
+  if (aLength >= 4 && ((unsigned char)aContents[0]==0x8B &&
+                   (unsigned char)aContents[1]==0x4A &&
+                   (unsigned char)aContents[2]==0x4E &&
+                   (unsigned char)aContents[3]==0x47))
+  { 
+    *aContentType = nsCRT::strndup("image/x-jng", 11);
+    return NS_OK;
+  }
+
+  if (aLength >= 8 && !nsCRT::strncmp(aContents, "#define ", 8)) {
+    *aContentType = nsCRT::strndup("image/x-xbitmap", 15);
+    return NS_OK;
+  }
+
+  /* none of the above?  I give up */
+  /* don't raise an exception, simply return null */
+  return NS_OK;
+}
+
 /**
  * proxy stream listener class used to handle multipart/x-mixed-replace
  */
@@ -542,10 +666,10 @@ NS_IMETHODIMP ProxyListener::OnStartRequest(nsIRequest *aRequest, nsISupports *c
 
   nsCOMPtr<nsIChannel> channel(do_QueryInterface(aRequest));
   if (channel) {
-    nsXPIDLCString contentType;
-    nsresult rv = channel->GetContentType(getter_Copies(contentType));
+    nsCAutoString contentType;
+    nsresult rv = channel->GetContentType(contentType);
 
-    if (contentType.get()) {
+    if (!contentType.IsEmpty()) {
      /* If multipart/x-mixed-replace content, we'll insert a MIME decoder
         in the pipeline to handle the content and pass it along to our
         original listener.
