@@ -101,16 +101,7 @@ pref("mail.strictly_mime_headers",          true);
 pref("mail.file_attach_binary",             false);
 pref("mail.show_headers",                   1); // some
 pref("mail.pane_config",                    0);
-// the format for "mail.addr_book.mapit_url.format" is:
-// @A1 == address, part 1
-// @A2 == address, part 2
-// @CI == city
-// @ST == state
-// @ZI == zip code
-// @CO == country
-//
-// if the format is "", no mapit buttons will appear in the addressbook
-pref("mail.addr_book.mapit_url.format", "http://www.mapquest.com/maps/map.adp?country=@CO&address=@A1%20@A2&city=@CI&state=@ST&zipcode=@ZI");
+pref("mail.addr_book.mapit_url.format", "chrome://messenger-region/locale/region.properties");
 
 // the format for "mail.addr_book.quicksearchquery.format" is:
 // @V == the escaped value typed in the quick search bar in the addressbook
@@ -127,17 +118,21 @@ pref("mail.addr_book.displayName.lastnamefirst", false); // generate display nam
 pref("mail.attach_vcard",                   false);
 pref("mail.html_compose",                   true);
 pref("mail.compose.other.header",	    "");
-localDefPref("mail.fcc_folder",                     "");
+pref("mail.fcc_folder",                     "");
 pref("mail.encrypt_outgoing_mail",          false);
 pref("mail.crypto_sign_outgoing_mail",      false);
-pref("mail.default_html_action", 0);    // 0=ask, 1=plain, 2=html, 3=both
-pref("mail.smtp.ssl",                       0); // 0 = no, 1 = try, 2 = must use SSL
-pref("mail.mdn.report.not_in_to_cc", 0);      // 0: Never 1: Always 2: Ask me
-pref("mail.mdn.report.outside_domain", 2);    // 0: Never 1: Always 2: Ask me
-pref("mail.mdn.report.other", 2);   // 0: Never 1: Always 2: Ask me 3: Denial
-pref("mail.incorporate.return_receipt", 1); // 0: inbox/filter 1: Sent folder
-pref("mail.request.return_receipt", 2);     // 1: DSN 2: MDN 3: Both
-pref("mail.receipt.request_header_type", 0); // 0: MDN-DNT header  1: RRT header 2: Both (MC)
+pref("mail.default_html_action", 0);          // 0=ask, 1=plain, 2=html, 3=both
+pref("mail.smtp.ssl",0);                      // 0 = no, 1 = try, 2 = must use SSL
+
+pref("mail.mdn.report.not_in_to_cc", 2);               // 0: Never 1: Always 2: Ask me
+pref("mail.mdn.report.outside_domain", 2);             // 0: Never 1: Always 2: Ask me
+pref("mail.mdn.report.other", 2);                      // 0: Never 1: Always 2: Ask me 3: Denial
+
+pref("mail.incorporate.return_receipt", 0);            // 0: Inbox/filter 1: Sent folder
+pref("mail.request.return_receipt", 2);                // 1: DSN 2: MDN 3: Both
+pref("mail.receipt.request_header_type", 0);           // 0: MDN-DNT header  1: RRT header 2: Both (MC)
+pref("mail.receipt.request_return_receipt_on", false);
+pref("mail.mdn.report.enabled", true);                 // false: Never send true: Send sometimes
 
 pref("news.default_cc",                     "");
 pref("news.default_fcc",                    ""); // mailbox:URL or Imap://Host/OnlineFolderName
@@ -149,6 +144,7 @@ pref("news.max_articles",                   500);
 pref("news.mark_old_read",                  false);
 
 pref("mailnews.wraplength",                 72);
+pref("mail.compose.wrap_to_window_width",   false);
 
 pref("mailnews.reply_on_top",               0); // 0=bottom 1=top 2=select+bottom 3=select+top
 
@@ -156,12 +152,12 @@ pref("mailnews.reply_on_top",               0); // 0=bottom 1=top 2=select+botto
 pref("mailnews.reply_header_type",          1);
 // locale which affects date format, set empty string to use application default locale
 pref("mailnews.reply_header_locale",        "en-US");
-pref("mailnews.reply_header_authorwrote",   "%s wrote");
+pref("mailnews.reply_header_authorwrote",   "chrome://messenger/locale/messengercompose/composeMsgs.properties");
 pref("mailnews.reply_header_ondate",        "On %s");
 // separator to separate between date and author
 pref("mailnews.reply_header_separator",     ", ");
 pref("mailnews.reply_header_colon",         ":");
-pref("mailnews.reply_header_originalmessage",   "--- Original Message ---");
+pref("mailnews.reply_header_originalmessage",   "chrome://messenger/locale/messengercompose/composeMsgs.properties");
 
 pref("mail.purge_threshhold",                100);
 pref("mail.prompt_purge_threshhold",             false);   
@@ -211,9 +207,10 @@ pref("ldap_1.directory6.attributes.telephoneNumber", "Phone Number:homephone");
 pref("ldap_1.directory6.attributes.street", "State:st");
 pref("ldap_1.directory6.filter1.repeatFilterForWords", false);
 
-pref("ldap_2.autoComplete.interval",							650);
-pref("ldap_2.autoComplete.enabled",								true);
+pref("ldap_2.autoComplete.interval", 650);
+pref("ldap_2.autoComplete.enabled", true);
 pref("ldap_2.autoComplete.useDirectory", false);
+pref("ldap_2.autoComplete.useAddressBooks", true);
 pref("ldap_2.autoComplete.skipDirectoryIfLocalMatchFound", true);
 pref("ldap_2.autoComplete.directoryServer", "");
 
@@ -316,9 +313,24 @@ pref("mail.server.default.canFileMessages", true);
 pref("mail.smtpserver.default.auth_method", 1); // auth any
 pref("mail.smtpserver.default.try_ssl", 0);
 
-pref("mail.display_glyph", true);   // see <http://www.bucksch.org/1/projects/mozilla/16507>
-pref("mail.display_struct", true);  // ditto
-pref("mail.send_struct", false);   // ditto
+// For the next 3 prefs, see <http://www.bucksch.org/1/projects/mozilla/16507>
+pref("mail.display_glyph", true);   // TXT->HTML :-) etc. in viewer
+pref("mail.display_struct", true);  // TXT->HTML *bold* etc. in viewer; ditto
+pref("mail.send_struct", false);   // HTML->HTML *bold* etc. during Send; ditto
+// For the next 4 prefs, see <http://www.bucksch.org/1/projects/mozilla/108153>
+pref("mailnews.display.prefer_plaintext", false);  // Ignore HTML parts in multipart/alternative
+pref("mailnews.display.html_as", 0);  // How to display HTML parts. 0 = Render the sender's HTML; 1 = HTML->TXT->HTML; 2 = Show HTML source; 3 = Sanitize HTML
+pref("mailnews.display.html_sanitizer.allowed_tags", "html head title body p br div(lang,title) h1 h2 h3 h4 h5 h6 ul ol li(value,start,compact) dl dt dd blockquote(type,cite) pre noscript noframes strong em sub sup span(lang,title) acronym(title) abbr(title) del(title,cite,datetime) ins(title,cite,datetime) q(cite) a(href,name,title) img(alt,title,longdesc) base(href) area(alt) applet(alt) object(alt) var samp dfn address kbd code cite s strike tt b i table(align) caption tr(align,valign) td(rowspan,colspan,align,valign) th(rowspan,colspan,align,valign)");
+pref("mailnews.display.disallow_mime_handlers", 0);  /* Let only a few classes process incoming data. This protects from bugs (e.g. buffer overflows) and from security loopholes (e.g. allowing unchecked HTML in some obscure classes, although the user has html_as > 0).
+This option is mainly for the UI of html_as.
+0 = allow all available classes
+1 = Use hardcoded blacklist to avoid rendering (incoming) HTML
+2 = ... and inline images
+3 = ... and some other uncommon content types
+100 = Use hardcoded whitelist to avoid even more bugs(buffer overflows).
+      This mode will limit the features available (e.g. uncommon
+      attachment types and inline images) and is for paranoid users.
+*/
 
 pref("mail.forward_message_mode", 0); // 0=default as attachment 2=forward as inline with attachments, (obsolete 4.x value)1=forward as quoted (mapped to 2 in mozilla)
 
@@ -342,8 +354,10 @@ pref("mail.warn_on_send_accel_key", true);
 pref("mail.enable_autocomplete",true);
 pref("mailnews.html_domains","");
 pref("mailnews.plaintext_domains","");
+pref("mailnews.global_html_domains.version",1);
 
 pref("mail.biff.play_sound",true);
+pref("mail.biff.show_alert", true);
 
 pref("mail.content_disposition_type", 0);
 
@@ -351,31 +365,36 @@ pref("mailnews.show_send_progress", true); //Will show a progress dialog when sa
 pref("mail.server.default.retainBy", 1);
 
 // for manual upgrades of certain UI features.
-// 1 -> 2 is for the folder pane outliner landing, to hide the
+// 1 -> 2 is for the folder pane tree landing, to hide the
 // unread and total columns, see msgMail3PaneWindow.js
 pref("mail.ui.folderpane.version", 1);                                          
 
 // for manual upgrades of certain UI features.
-// 1 -> 2 is for the thread pane outliner landing, to hide the
+// 1 -> 2 is for the thread pane tree landing, to hide the
 // labels column, see msgMail3PaneWindow.js
 pref("mailnews.ui.threadpane.version", 1);                                          
 // for manual upgrades of certain UI features.
-// 1 -> 2 is for the ab results pane outliner landing
+// 1 -> 2 is for the ab results pane tree landing
 // to hide the non default columns in the addressbook dialog
 // see abCommon.js and addressbook.js
 pref("mailnews.ui.addressbook_results.version", 1);                                          
 // for manual upgrades of certain UI features.
-// 1 -> 2 is for the ab results pane outliner landing
+// 1 -> 2 is for the ab results pane tree landing
 // to hide the non default columns in the addressbook sidebar panel
 // see abCommon.js and addressbook-panel.js
 pref("mailnews.ui.addressbook_panel_results.version", 1);                                          
 // for manual upgrades of certain UI features.
-// 1 -> 2 is for the ab results pane outliner landing
+// 1 -> 2 is for the ab results pane tree landing
 // to hide the non default columns in the select addresses dialog
 // see abCommon.js and abSelectAddressesDialog.js
-pref("mailnews.ui.select_addresses_results.version", 1);                                          
+pref("mailnews.ui.select_addresses_results.version", 1); 
+// for manual upgrades of certain UI features.
+// 1 -> 2 is for the ab results pane
+// to hide the non default columns in the advanced directory search dialog
+// see abCommon.js and ABSearchDialog.js
+pref("mailnews.ui.advanced_directory_search_results.version", 1);                                         
 //If set to a number greater than 0, msg compose windows will be recycled in order to open them quickly
-pref("mail.compose.max_recycled_windows", 0); 
+pref("mail.compose.max_recycled_windows", 1); 
 
 // true makes it so we persist the open state of news server when starting up the 3 pane
 // this is costly, as it might result in network activity.
@@ -400,4 +419,14 @@ pref("mailnews.labels.color.2", "#FF9900"); // default: orange
 pref("mailnews.labels.color.3", "#009900"); // default: green
 pref("mailnews.labels.color.4", "#3333FF"); // default: blue
 pref("mailnews.labels.color.5", "#993399"); // default: purple
- 
+
+//default null headers
+//example "X-Warn: XReply", list of hdrs separated by ": "
+pref("mailnews.customHeaders", ""); 
+
+pref("mailnews.fakeaccount.show", false);
+pref("mailnews.fakeaccount.server", "");
+
+// message display properties
+pref("mailnews.message_display.disable_remote_image", false);
+pref("mailnews.message_display.allow.plugins", true);
