@@ -37,7 +37,7 @@
 
 // TODO: Implement Java callbacks
 
-#include "xp_core.h"
+#include "prtypes.h"
 #include "nsplugin.h"
 #include "ns4xPlugin.h"
 #include "ns4xPluginInstance.h"
@@ -51,6 +51,9 @@
 #include "nsIPref.h"
 #include "nsPluginLogging.h"
 
+#include "nsIPluginInstancePeer2.h"
+#include "nsIJSContextStack.h"
+
 #ifdef XP_MAC
 #include <Resources.h>
 #endif
@@ -62,6 +65,12 @@
 #include "gtkxtbin.h"
 #endif
 
+// POST/GET stream type
+enum eNPPStreamTypeInternal {
+  eNPPStreamTypeInternal_Get,
+  eNPPStreamTypeInternal_Post
+};
+
 ////////////////////////////////////////////////////////////////////////
 // CID's && IID's
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
@@ -76,7 +85,7 @@ static NS_DEFINE_IID(kMemoryCID, NS_MEMORY_CID);
 static NS_DEFINE_IID(kIMemoryIID, NS_IMEMORY_IID);
 static NS_DEFINE_IID(kIPluginStreamListenerIID, NS_IPLUGINSTREAMLISTENER_IID);
 
-XP_BEGIN_PROTOS
+PR_BEGIN_EXTERN_C
 
   ////////////////////////////////////////////////////////////////////////
   // Static stub functions that are exported to the 4.x plugin as entry
@@ -180,7 +189,7 @@ XP_BEGIN_PROTOS
 #pragma pointers_in_A0
 #endif
 
-XP_END_PROTOS
+PR_END_EXTERN_C
 
 
 ////////////////////////////////////////////////////////////////////////
@@ -271,7 +280,7 @@ ns4xPlugin::ns4xPlugin(NPPluginFuncs* callbacks, PRLibrary* aLibrary, NP_PLUGINS
   NS_TRY_SAFE_CALL_RETURN(error, CallNPP_MainEntryProc(pfnMain, 
                                                        &(ns4xPlugin::CALLBACKS), 
                                                        &fCallbacks, 
-                                                       &fShutdownEntry), aLibrary);
+                                                       &fShutdownEntry), aLibrary, nsnull);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL, ("NPP MainEntryProc called, return=%d\n",error));
 
@@ -384,35 +393,11 @@ ns4xPlugin::CreatePlugin(nsIServiceManagerObsolete* aServiceMgr,
   memcpy((void*) &(plptr->fCallbacks), (void*)&callbacks, sizeof(callbacks));
 #endif
 
-#if defined(XP_PC) && !defined(XP_WIN)
-  // XXX this probably should be factored out and
-  //      just use trailing XP_WIN.
-  NP_GETENTRYPOINTS pfnGetEntryPoints = (NP_GETENTRYPOINTS)PR_FindSymbol(aLibrary, "NP_GetEntryPoints");
-
-  if (pfnGetEntryPoints == NULL)
-    return NS_ERROR_FAILURE;
-
-  NPPluginFuncs callbacks;
-  memset((void*) &callbacks, 0, sizeof(callbacks));
-
-  callbacks.size = sizeof(callbacks);
-
-  if (pfnGetEntryPoints(&callbacks) != NS_OK)
-    return NS_ERROR_FAILURE; // XXX
-
-  if (HIBYTE(callbacks.version) < NP_VERSION_MAJOR)
-    return NS_ERROR_FAILURE;
-
-  NP_PLUGINSHUTDOWN pfnShutdown = (NP_PLUGINSHUTDOWN)PR_FindSymbol(aLibrary, "NP_Shutdown");
-
-  // create the new plugin handler
-  *aResult = new ns4xPlugin(&callbacks, aLibrary, pfnShutdown, aServiceMgr);
-#elif defined(XP_WIN)
+#ifdef XP_WIN
   // Note: on Windows, we must use the fCallback because plugins may change
   // the function table. The Shockwave installer makes changes in the table while running
   *aResult = new ns4xPlugin(nsnull, aLibrary, nsnull, aServiceMgr);
-#endif
-#ifdef XP_PC
+
   if (*aResult == NULL)
     return NS_ERROR_OUT_OF_MEMORY;
 
@@ -444,6 +429,113 @@ ns4xPlugin::CreatePlugin(nsIServiceManagerObsolete* aServiceMgr,
     return NS_ERROR_UNEXPECTED;
 #endif
 
+#ifdef XP_OS2
+  // XXX Do we need to do this on OS/2 or can we look more like Windows?
+  NP_GETENTRYPOINTS pfnGetEntryPoints = (NP_GETENTRYPOINTS)PR_FindSymbol(aLibrary, "NP_GetEntryPoints");
+
+  if (pfnGetEntryPoints == NULL)
+    return NS_ERROR_FAILURE;
+
+  NPPluginFuncs callbacks;
+  memset((void*) &callbacks, 0, sizeof(callbacks));
+
+  callbacks.size = sizeof(callbacks);
+
+  if (pfnGetEntryPoints(&callbacks) != NS_OK)
+    return NS_ERROR_FAILURE; // XXX
+
+  if (HIBYTE(callbacks.version) < NP_VERSION_MAJOR)
+    return NS_ERROR_FAILURE;
+
+  NP_PLUGINSHUTDOWN pfnShutdown = (NP_PLUGINSHUTDOWN)PR_FindSymbol(aLibrary, "NP_Shutdown");
+
+  // create the new plugin handler
+  *aResult = new ns4xPlugin(&callbacks, aLibrary, pfnShutdown, aServiceMgr);
+
+  if (*aResult == NULL)
+    return NS_ERROR_OUT_OF_MEMORY;
+
+  NS_ADDREF(*aResult);
+
+  // we must init here because the plugin may call NPN functions 
+  // when we call into the NP_Initialize entry point - NPN functions
+  // require that mBrowserManager be set up
+  if (NS_FAILED((*aResult)->Initialize())) {
+    NS_RELEASE(*aResult);
+    return NS_ERROR_FAILURE;
+  }
+
+  // the NP_Initialize entry point was misnamed as NP_PluginInit,
+  // early in plugin project development.  Its correct name is
+  // documented now, and new developers expect it to work.  However,
+  // I don't want to break the plugins already in the field, so
+  // we'll accept either name
+
+  NP_PLUGININIT pfnInitialize = (NP_PLUGININIT)PR_FindSymbol(aLibrary, "NP_Initialize");
+
+  if (!pfnInitialize)
+    pfnInitialize = (NP_PLUGININIT)PR_FindSymbol(aLibrary, "NP_PluginInit");
+
+  if (pfnInitialize == NULL)
+    return NS_ERROR_UNEXPECTED; // XXX Right error?
+
+  //Fixes problem where the OS/2 native multimedia plugins weren't working
+  //   on mozilla though did work on 4.x.  Problem is that they expect the
+  //   current working directory to be the plugins dir.  Since these plugins
+  //   are no longer maintained and they represent the majority of the OS/2
+  //   plugin contingency, we'll have to make them work here.
+
+#define MAP_DISKNUM_TO_LETTER(n) ('A' + (n - 1))
+#define MAP_LETTER_TO_DISKNUM(c) (toupper(c)-'A'+1)
+
+  unsigned long origDiskNum, pluginDiskNum, logicalDisk;
+
+  char pluginPath[CCHMAXPATH], origPath[CCHMAXPATH];
+  strcpy(pluginPath, aFileName);
+  char* slash = strrchr(pluginPath, '\\');
+  *slash = '\0';
+
+  DosQueryCurrentDisk( &origDiskNum, &logicalDisk );
+  pluginDiskNum = MAP_LETTER_TO_DISKNUM(pluginPath[0]);
+
+  origPath[0] = MAP_DISKNUM_TO_LETTER(origDiskNum);
+  origPath[1] = ':';
+  origPath[2] = '\\';
+
+  ULONG len = CCHMAXPATH-3;
+  APIRET rc = DosQueryCurrentDir(0, &origPath[3], &len);
+  NS_ASSERTION(NO_ERROR == rc,"DosQueryCurrentDir failed");
+
+  BOOL bChangedDir = FALSE;
+  BOOL bChangedDisk = FALSE;
+  if (pluginDiskNum != origDiskNum) {
+    rc = DosSetDefaultDisk(pluginDiskNum);
+    NS_ASSERTION(NO_ERROR == rc,"DosSetDefaultDisk failed");
+    bChangedDisk = TRUE;
+  }
+
+  if (stricmp(origPath, pluginPath) != 0) {
+    rc = DosSetCurrentDir(pluginPath);
+    NS_ASSERTION(NO_ERROR == rc,"DosSetCurrentDir failed");
+    bChangedDir = TRUE;
+  }
+
+  nsresult rv = pfnInitialize(&(ns4xPlugin::CALLBACKS));
+
+  if (bChangedDisk) {
+    rc= DosSetDefaultDisk(origDiskNum);
+    NS_ASSERTION(NO_ERROR == rc,"DosSetDefaultDisk failed");
+  }
+  if (bChangedDir) {
+    rc = DosSetCurrentDir(origPath);
+    NS_ASSERTION(NO_ERROR == rc,"DosSetCurrentDir failed");
+  }
+  
+  if (!NS_SUCCEEDED(rv)) {
+    return NS_ERROR_UNEXPECTED;
+  }
+#endif
+
 #if defined(XP_MAC)
   short appRefNum = ::CurResFile();
   short pluginRefNum;
@@ -470,7 +562,7 @@ ns4xPlugin::CreatePlugin(nsIServiceManagerObsolete* aServiceMgr,
   NS_TRY_SAFE_CALL_RETURN(error, CallNPP_MainEntryProc(pfnMain, 
                                                        &(ns4xPlugin::CALLBACKS), 
                                                        &callbacks, 
-                                                       &pfnShutdown), fLibrary);
+                                                       &pfnShutdown), fLibrary, nsnull);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_BASIC, ("NPP MainEntryProc called: return=%d\n",error));
 
@@ -610,7 +702,7 @@ ns4xPlugin::Shutdown(void)
     CallNPP_ShutdownProc(fShutdownEntry);
     ::CloseResFile(fPluginRefNum);
 #else
-    NS_TRY_SAFE_CALL_VOID(fShutdownEntry(), fLibrary);
+    NS_TRY_SAFE_CALL_VOID(fShutdownEntry(), fLibrary, nsnull);
 #endif
 
     fShutdownEntry = nsnull;
@@ -653,6 +745,52 @@ ns4xPlugin::GetValue(nsPluginVariable variable, void *value)
   }
 }
 
+// Create a new NPP GET or POST url stream that may have a notify callback
+NPError MakeNew4xStreamInternal (NPP npp, 
+                                 const char *relativeURL, 
+                                 const char *target, 
+                                 eNPPStreamTypeInternal type,  /* GET or POST */
+                                 PRBool bDoNotify = PR_FALSE,
+                                 void *notifyData = nsnull,
+                                 uint32 len = 0, 
+                                 const char *buf = nsnull, 
+                                 NPBool file = PR_FALSE)
+{
+  if(!npp) return NPERR_INVALID_INSTANCE_ERROR;
+
+  nsIPluginInstance *inst = (nsIPluginInstance *) npp->ndata;
+
+  NS_ASSERTION(inst != NULL, "null instance");
+  if (inst == NULL) 
+    return NPERR_INVALID_INSTANCE_ERROR;
+
+  nsCOMPtr<nsIPluginManager> pm = do_GetService(kPluginManagerCID);
+  NS_ASSERTION(pm, "failed to get plugin manager");
+  if (!pm) return NPERR_GENERIC_ERROR;
+
+  nsIPluginStreamListener* listener = nsnull;
+  if(target == nsnull)
+    ((ns4xPluginInstance*)inst)->NewNotifyStream(&listener, notifyData, bDoNotify, relativeURL);
+
+  switch (type) {
+  case eNPPStreamTypeInternal_Get:
+    {
+      if(NS_FAILED(pm->GetURL(inst, relativeURL, target, listener)))
+        return NPERR_GENERIC_ERROR;
+      break;
+    }
+  case eNPPStreamTypeInternal_Post:
+    {
+      if(NS_FAILED(pm->PostURL(inst, relativeURL, len, buf, file, target, listener)))
+        return NPERR_GENERIC_ERROR;
+      break;
+    }
+  default:
+    NS_ASSERTION(0, "how'd I get here");
+  }
+
+  return NPERR_NO_ERROR;
+}
 
 ////////////////////////////////////////////////////////////////////////
 //
@@ -665,33 +803,7 @@ _geturl(NPP npp, const char* relativeURL, const char* target)
   NPN_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPN_GetURL: npp=%p, target=%s, url=%s\n", (void *)npp, target, relativeURL));
 
-  if(!npp)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  nsIPluginInstance *inst = (nsIPluginInstance *) npp->ndata;
-
-  NS_ASSERTION(inst != NULL, "null instance");
-  NS_ASSERTION(gServiceMgr != NULL, "null service manager");
-
-  if (inst == NULL)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  if(gServiceMgr == nsnull)
-    return NPERR_GENERIC_ERROR;
-
-  nsIPluginStreamListener* listener = nsnull;
-  if(target == nsnull)
-    inst->NewStream(&listener);
-
-  nsIPluginManager * pm;
-  gServiceMgr->GetService(kPluginManagerCID, kIPluginManagerIID, (nsISupports**)&pm);
-
-  if(pm->GetURL(inst, relativeURL, target, listener) != NS_OK)
-    return NPERR_GENERIC_ERROR;
-
-  NS_RELEASE(pm);
-
-  return NPERR_NO_ERROR;
+  return MakeNew4xStreamInternal (npp, relativeURL, target, eNPPStreamTypeInternal_Get);
 }
 
 
@@ -702,33 +814,7 @@ _geturlnotify(NPP npp, const char* relativeURL, const char* target, void* notify
   NPN_PLUGIN_LOG(PLUGIN_LOG_NORMAL, 
     ("NPN_GetURLNotify: npp=%p, target=%s, notify=%p, url=%s\n", (void*)npp, target, notifyData, relativeURL));
 
-  if(!npp)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  nsIPluginInstance *inst = (nsIPluginInstance *) npp->ndata;
-
-  NS_ASSERTION(inst != NULL, "null instance");
-  NS_ASSERTION(gServiceMgr != NULL, "null service manager");
-
-  if (inst == NULL)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  if(gServiceMgr == nsnull)
-    return NPERR_GENERIC_ERROR;
-
-  nsIPluginStreamListener* listener = nsnull;
-  if(target == nsnull)
-    ((ns4xPluginInstance*)inst)->NewNotifyStream(&listener, notifyData);
-
-  nsIPluginManager * pm;
-  gServiceMgr->GetService(kPluginManagerCID, kIPluginManagerIID, (nsISupports**)&pm);
-
-  if(pm->GetURL(inst, relativeURL, target, listener) != NS_OK)
-    return NPERR_GENERIC_ERROR;
-
-  NS_RELEASE(pm);
-
-  return NPERR_NO_ERROR;
+  return MakeNew4xStreamInternal (npp, relativeURL, target, eNPPStreamTypeInternal_Get, PR_TRUE, notifyData);
 }
 
 
@@ -741,33 +827,8 @@ _posturlnotify(NPP npp, const char *relativeURL, const char *target,
   ("NPN_PostURLNotify: npp=%p, target=%s, len=%d, file=%d, notify=%p, url=%s, buf=%s\n",
   (void*)npp, target, len, file, notifyData, relativeURL, buf));
 
-  if(!npp)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  nsIPluginInstance *inst = (nsIPluginInstance *) npp->ndata;
-
-  NS_ASSERTION(inst != NULL, "null instance");
-  NS_ASSERTION(gServiceMgr != NULL, "null service manager");
-
-  if (inst == NULL)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  if(gServiceMgr == nsnull)
-    return NPERR_GENERIC_ERROR;
-
-  nsIPluginStreamListener* listener = nsnull;
-  if(target == nsnull)
-    ((ns4xPluginInstance*)inst)->NewNotifyStream(&listener, notifyData);
-
-  nsIPluginManager * pm;
-  gServiceMgr->GetService(kPluginManagerCID, kIPluginManagerIID, (nsISupports**)&pm);
-
-  if(pm->PostURL(inst, relativeURL, len, buf, file, target, listener) != NS_OK)
-    return NPERR_GENERIC_ERROR;
-
-  NS_RELEASE(pm);
-
-  return NPERR_NO_ERROR;
+  return MakeNew4xStreamInternal (npp, relativeURL, target, eNPPStreamTypeInternal_Post, 
+                                  PR_TRUE, notifyData, len, buf, file);
 }
 
 
@@ -780,33 +841,8 @@ _posturl(NPP npp, const char *relativeURL, const char *target,
   ("NPN_PostURL: npp=%p, target=%s, file=%d, len=%d, url=%s, buf=%s\n",
   (void*)npp, target, file, len, relativeURL, buf));
 
-  if(!npp)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  nsIPluginInstance *inst = (nsIPluginInstance *) npp->ndata;
-
-  NS_ASSERTION(inst != NULL, "null instance");
-  NS_ASSERTION(gServiceMgr != NULL, "null service manager");
-
-  if (inst == NULL)
-    return NPERR_INVALID_INSTANCE_ERROR;
-
-  if(gServiceMgr == nsnull)
-    return NPERR_GENERIC_ERROR;
-
-  nsIPluginStreamListener* listener = nsnull;
-  if(target == nsnull)
-    inst->NewStream(&listener);
-
-  nsIPluginManager * pm;
-  gServiceMgr->GetService(kPluginManagerCID, kIPluginManagerIID, (nsISupports**)&pm);
-
-  if(pm->PostURL(inst, relativeURL, len, buf, file, target, listener) != NS_OK)
-    return NPERR_GENERIC_ERROR;
-
-  NS_RELEASE(pm);
-
-  return NPERR_NO_ERROR;
+ return MakeNew4xStreamInternal (npp, relativeURL, target, eNPPStreamTypeInternal_Post, 
+                                 PR_FALSE, nsnull, len, buf, file);
 }
 
 
@@ -1262,6 +1298,37 @@ _setvalue(NPP npp, NPPVariable variable, void *result)
       NPBool bTransparent = (result != nsnull);
       return inst->SetTransparent(bTransparent);
     }
+        
+    case NPPVjavascriptPushCallerBool:
+      {
+        nsresult rv;
+        nsCOMPtr<nsIJSContextStack> contextStack = do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
+        if (NS_SUCCEEDED(rv)) {
+          NPBool bPushCaller = (result != nsnull);
+          if (bPushCaller) {
+            nsCOMPtr<nsIPluginInstancePeer> peer;
+            rv = inst->GetPeer(getter_AddRefs(peer));
+            if (NS_SUCCEEDED(rv)) {
+              nsCOMPtr<nsIPluginInstancePeer2> peer2 = do_QueryInterface(peer, &rv);
+              if (NS_SUCCEEDED(rv) && peer2) {
+                JSContext *cx;
+                rv = peer2->GetJSContext(&cx);
+                if (NS_SUCCEEDED(rv))
+                  rv = contextStack->Push(cx);
+              }
+            }
+          } else {
+            rv = contextStack->Pop(nsnull);
+          }
+        }
+        return NS_SUCCEEDED(rv) ? NPERR_NO_ERROR : NPERR_GENERIC_ERROR;
+      }
+      break;
+    
+    case NPPVpluginKeepLibraryInMemory: {
+      NPBool bCached = (result != nsnull);
+      return inst->SetCached(bCached);
+    }
 
     default:
       return NPERR_NO_ERROR;
