@@ -40,7 +40,7 @@
 #include "prlog.h"
 #include "prmem.h"
 #include "nscore.h"
-
+#include "prenv.h"
 
 #include "ns4xPluginInstance.h"
 #include "ns4xPluginStreamListener.h"
@@ -64,7 +64,6 @@
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID); // needed for NS_TRY_SAFE_CALL_*
 static NS_DEFINE_IID(kCPluginManagerCID, NS_PLUGINMANAGER_CID);
 static NS_DEFINE_IID(kIPluginStreamListenerIID, NS_IPLUGINSTREAMLISTENER_IID);
-static NS_DEFINE_IID(kIPluginStreamListener2IID, NS_IPLUGINSTREAMLISTENER2_IID);
 static NS_DEFINE_IID(kIPluginInstanceIID, NS_IPLUGININSTANCE_IID); 
 static NS_DEFINE_IID(kIPluginTagInfoIID, NS_IPLUGINTAGINFO_IID); 
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
@@ -72,27 +71,32 @@ static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 ///////////////////////////////////////////////////////////////////////////////
 // ns4xPluginStreamListener Methods
 
-NS_IMPL_ISUPPORTS2(ns4xPluginStreamListener, nsIPluginStreamListener, 
-                                             nsIPluginStreamListener2);
+NS_IMPL_ISUPPORTS1(ns4xPluginStreamListener, nsIPluginStreamListener);
 
 ///////////////////////////////////////////////////////////////////////////////
 
 ns4xPluginStreamListener::ns4xPluginStreamListener(nsIPluginInstance* inst, 
-                                                   void* notifyData)
+                                                   void* notifyData,
+                                                   const char* aURL)
     : mNotifyData(notifyData),
+      mStreamBuffer(nsnull),
+      mNotifyURL(nsnull),  
       mStreamStarted(PR_FALSE),
       mStreamCleanedUp(PR_FALSE),
+      mCallNotify(PR_FALSE),      
       mStreamInfo(nsnull)
 {
   NS_INIT_REFCNT();
   mInst = (ns4xPluginInstance*) inst;
-  mStreamBuffer=nsnull;
   mPosition = 0;
-  mCurrentStreamOffset = -1;
+  mStreamBufferSize = 0;
   // Initialize the 4.x interface structure
   memset(&mNPStream, 0, sizeof(mNPStream));
 
   NS_IF_ADDREF(mInst);
+
+  if (aURL)
+    mNotifyURL = PL_strdup(aURL);
 }
 
 
@@ -100,12 +104,13 @@ ns4xPluginStreamListener::ns4xPluginStreamListener(nsIPluginInstance* inst,
 ns4xPluginStreamListener::~ns4xPluginStreamListener(void)
 {
   // remove itself from the instance stream list
-  if(mInst) {
+  ns4xPluginInstance *inst = mInst;
+  if(inst) {
     nsInstanceStream * prev = nsnull;
-    for(nsInstanceStream *is = mInst->mStreams; is != nsnull; is = is->mNext) {
+    for(nsInstanceStream *is = inst->mStreams; is != nsnull; is = is->mNext) {
       if(is->mPluginStreamListener == this) {
         if(prev == nsnull)
-          mInst->mStreams = is->mNext;
+          inst->mStreams = is->mNext;
         else
           prev->mNext = is->mNext;
 
@@ -115,79 +120,106 @@ ns4xPluginStreamListener::~ns4xPluginStreamListener(void)
       prev = is;
     }
   }
-  NS_IF_RELEASE(mInst);
-}
 
+  // For those cases when NewStream is never called, we still may need to fire a
+  // notification callback. Return network error as fallback reason because for other
+  // cases, notify should have already been called for other reasons elsewhere.
+  CallURLNotify(NPRES_NETWORK_ERR);
 
-///////////////////////////////////////////////////////////////////////////////
-nsresult ns4xPluginStreamListener::CleanUpStream(NPReason reason)
-{
-  if(!mStreamStarted || mStreamCleanedUp)
-    return NS_OK;
-
-  if(!mInst || !mInst->IsStarted())
-    return NS_ERROR_FAILURE;
-
-  NPP npp;
-  const NPPluginFuncs *callbacks = nsnull;
-
-  mInst->GetCallbacks(&callbacks);
-  mInst->GetNPP(&npp);
-
-  if(!callbacks)
-    return NS_ERROR_FAILURE;
-
-  NPError error;
-
-  if (callbacks->destroystream != NULL)
-  {
-    PRLibrary* lib = nsnull;
-    lib = mInst->fLibrary;
-
-    NS_TRY_SAFE_CALL_RETURN(error, CallNPP_DestroyStreamProc(callbacks->destroystream,
-                                                               npp,
-                                                               &mNPStream,
-                                                               reason), lib);
-
-    NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
-    ("NPP DestroyStream called: this=%p, npp=%p, reason=%d, return=%d, url=%s\n",
-    this, npp, reason, error, mNPStream.url));
-
-    if(error != NPERR_NO_ERROR)
-      return NS_ERROR_FAILURE;
-  }
-
-  // check to see if we have a call back and a
-  // XXX nasty hack for Shockwave Registration. 
-  //     we seem to crash doing URLNotify so just always exclude it.
-  //     See bug 85334.
-  const char macromediaurl[] = "http://pinger.macromedia.com";
-  if (callbacks->urlnotify != NULL && mNotifyData != nsnull &&
-      PL_strncasecmp(mNPStream.url, macromediaurl, sizeof(macromediaurl) - 1) != 0 )  {
-    PRLibrary* lib = nsnull;
-    lib = mInst->fLibrary;
-
-    NS_TRY_SAFE_CALL_VOID(CallNPP_URLNotifyProc(callbacks->urlnotify,
-                                                npp,
-                                                mNPStream.url,
-                                                nsPluginReason_Done,
-                                                mNotifyData), lib);
-
-    NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
-    ("NPP URLNotify called: this=%p, npp=%p, notify=%p, url=%s\n",
-    this, npp, mNotifyData, mNPStream.url));
-  }
-
-  // lets get rid of the buffer if we made one globally
+  // lets get rid of the buffer
   if (mStreamBuffer)
   {
     PR_Free(mStreamBuffer);
     mStreamBuffer=nsnull;
   }
 
+
+  NS_IF_RELEASE(inst);
+
+  if (mNotifyURL)
+    PL_strfree(mNotifyURL);
+}
+
+///////////////////////////////////////////////////////////////////////////////
+nsresult ns4xPluginStreamListener::CleanUpStream(NPReason reason)
+{
+  nsresult rv = NS_ERROR_FAILURE;
+
+  if(mStreamCleanedUp)
+    return NS_OK;
+
+  if(!mInst || !mInst->IsStarted())
+    return rv;
+
+  const NPPluginFuncs *callbacks = nsnull;
+  mInst->GetCallbacks(&callbacks);
+  if(!callbacks)
+    return rv;
+
+  NPP npp;
+  mInst->GetNPP(&npp);
+
+  if (mStreamStarted && callbacks->destroystream != NULL)
+  {
+    PRLibrary* lib = nsnull;
+    lib = mInst->fLibrary;
+    NPError error;
+    NS_TRY_SAFE_CALL_RETURN(error, CallNPP_DestroyStreamProc(callbacks->destroystream,
+                                                               npp,
+                                                               &mNPStream,
+                                                               reason), lib, mInst);
+
+    NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
+    ("NPP DestroyStream called: this=%p, npp=%p, reason=%d, return=%d, url=%s\n",
+    this, npp, reason, error, mNPStream.url));
+
+    if(error == NPERR_NO_ERROR)
+      rv = NS_OK;
+  }
+
   mStreamCleanedUp = PR_TRUE;
   mStreamStarted   = PR_FALSE;
-  return NS_OK;
+
+  // fire notification back to plugin, just like before
+  CallURLNotify(reason);
+
+  return rv;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+void ns4xPluginStreamListener::CallURLNotify(NPReason reason)
+{
+  if(!mCallNotify || !mInst || !mInst->IsStarted())
+    return;
+
+  mCallNotify = PR_FALSE; // only do this ONCE and prevent recursion
+
+  const NPPluginFuncs *callbacks = nsnull;
+  mInst->GetCallbacks(&callbacks);
+  if(!callbacks)
+    return;
+  
+  if (callbacks->urlnotify) {
+
+    NPP npp;
+    mInst->GetNPP(&npp);
+
+    NS_TRY_SAFE_CALL_VOID(CallNPP_URLNotifyProc(callbacks->urlnotify,
+                                                npp,
+                                                mNotifyURL,
+                                                reason,
+                                                mNotifyData), mInst->fLibrary, mInst);
+
+    NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
+    ("NPP URLNotify called: this=%p, npp=%p, notify=%p, reason=%d, url=%s\n",
+    this, npp, mNotifyData, reason, mNotifyURL));
+  }
+
+  // Let's not leak this stream listener. Release the reference to the stream listener 
+  // added for the notify callback in NewNotifyStream. 
+  // Note: This may destroy us if we are not being destroyed already.
+  NS_RELEASE_THIS();
 }
 
 
@@ -232,7 +264,7 @@ ns4xPluginStreamListener::OnStartBinding(nsIPluginStreamInfo* pluginInfo)
                                                        (char *)contentType,
                                                        &mNPStream,
                                                        seekable,
-                                                       &streamType), mInst->fLibrary);
+                                                       &streamType), mInst->fLibrary, mInst);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPP NewStream called: this=%p, npp=%p, mime=%s, seek=%d, type=%d, return=%d, url=%s\n",
@@ -260,297 +292,179 @@ ns4xPluginStreamListener::OnStartBinding(nsIPluginStreamInfo* pluginInfo)
       return NS_ERROR_FAILURE;
   }
 
-  // create buffer for stream here because we know the size
-  mStreamBuffer = (char*) PR_Malloc((PRUint32)MAX_PLUGIN_NECKO_BUFFER);
-  if (!mStreamBuffer)
-  {
-    NS_ASSERTION(PR_FALSE,"failed to create 4.x plugin stream buffer or size MAX_PLUGIN_NECKO_BUFFER");
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   mStreamStarted = PR_TRUE;
   return NS_OK;
 }
 
-
 ///////////////////////////////////////////////////////////////////////////////
 NS_IMETHODIMP
 ns4xPluginStreamListener::OnDataAvailable(nsIPluginStreamInfo* pluginInfo,
                                           nsIInputStream* input,
                                           PRUint32 length)
 {
+  nsresult rv = NS_ERROR_FAILURE;
   if (!mInst || !mInst->IsStarted())
-    return NS_ERROR_FAILURE;
+    return rv;
 
   const NPPluginFuncs *callbacks = nsnull;
-  NPP npp;
-
   mInst->GetCallbacks(&callbacks);
+  // check out if plugin implements NPP_Write call
+  if(!callbacks || !callbacks->write || !length)
+    return rv; // it'll cancel necko transaction 
+  
+  if (!mStreamBuffer)
+  {
+    // to optimize the mem usage & performance we have to allocate mStreamBuffer here
+    // in first ODA when length of data available in input stream is known.
+    // mStreamBuffer will be freed in DTOR.
+    // we also have to remember the size of that buff
+    // to make safe consecutive Read() calls form input stream into our buff.
+    if (length >= MAX_PLUGIN_NECKO_BUFFER) {
+        // ">" is rare case for decoded stream, but lets eat it all
+        mStreamBufferSize = length;    
+    } else {
+      PRUint32 contentLength;
+      pluginInfo->GetLength(&contentLength);
+      if (contentLength < MAX_PLUGIN_NECKO_BUFFER) {
+        // this is most common case for contentLength < 16k
+        mStreamBufferSize = length < contentLength ? contentLength:length; 
+      } else {
+        mStreamBufferSize = MAX_PLUGIN_NECKO_BUFFER;
+      }
+    }
+    mStreamBuffer = (char*) PR_Malloc(mStreamBufferSize);
+    if (!mStreamBuffer)
+      return NS_ERROR_OUT_OF_MEMORY;
+  }
+  
+  // prepare NPP_ calls params
+  NPP npp;    
   mInst->GetNPP(&npp);
-
-  if(!callbacks)
-    return NS_ERROR_FAILURE;
-
-  PRInt32   numtowrite = 0;
-  PRInt32   amountRead = 0;
-  PRInt32   writeCount = 0;
-  PRUint32  leftToRead = 0;         // just in case OnDataaAvail tries to overflow our buffer
-  PRBool   createdHere = PR_FALSE;  // we did malloc in locally, so we must free locally
-
   pluginInfo->GetURL(&mNPStream.url);
   pluginInfo->GetLastModified((PRUint32*)&(mNPStream.lastmodified));
 
-  if (callbacks->write == nsnull || length == 0)
-    return NS_OK; // XXX ?
+  PRInt32 streamOffset;
+  pluginInfo->GetStreamOffset(&streamOffset);
+  mPosition = streamOffset;
+  streamOffset += length; 
+  // Set new stream offset for the next ODA call
+  // regardless of how following NPP_Write call will behave
+  // we pretend to consume all data from the input stream.
+  // It's possible that current steam position will be overwritten
+  // from NPP_RangeRequest call made from NPP_Write, so
+  // we cannot call SetStreamOffset after NPP_Write.
+  // Note: there is a special case when data flow
+  // should be temporarily stopped if NPP_WriteReady returns 0 (bug #89270)
+  pluginInfo->SetStreamOffset(streamOffset);
 
-  if (nsnull == mStreamBuffer)
-  {
-    // create the buffer here because we failed in OnStartBinding
-    // XXX why don't we always get an OnStartBinding? This will protect us.
-    mStreamBuffer = (char*) PR_Malloc(length);
-    if (!mStreamBuffer)
-    return NS_ERROR_OUT_OF_MEMORY;
-    createdHere = PR_TRUE;
+  PRUint32 bytesToRead = mStreamBufferSize;
+  if (length < mStreamBufferSize) {
+    // do not read more that supplier wants us to read
+    bytesToRead = length;
   }
 
-  if (length > MAX_PLUGIN_NECKO_BUFFER)  // what if Necko gives us a lot of data?
+  do 
   {
-    leftToRead = length - MAX_PLUGIN_NECKO_BUFFER; // break it up
-    length     = MAX_PLUGIN_NECKO_BUFFER;
-  }
-  nsresult rv = input->Read(mStreamBuffer, length, (PRUint32*)&amountRead);
-  if (NS_FAILED(rv))
-    goto error;
+    PRInt32 amountRead = 0;
+    rv = input->Read(mStreamBuffer, bytesToRead, (PRUint32*)&amountRead);
+    if (amountRead == 0 || NS_FAILED(rv)) {
+      NS_WARNING("input->Read() returns no data, it's almost impossible to get here");
+      break;
+    }
 
-  // amountRead tells us how many bytes were put in the buffer
-  // WriteReady returns to us how many bytes the plugin is 
-  // ready to handle  - we have to keep calling WriteReady and
-  // Write until the buffer is empty
-  while (amountRead > 0)
-  {
-    if (callbacks->writeready != NULL)
-    {
-      PRLibrary* lib = nsnull;
-      lib = mInst->fLibrary;
+    // this loop in general case will end on length <= 0, without extra input->Read() call
+    length -= amountRead;
+    
+    char *ptrStreamBuffer = mStreamBuffer; // tmp ptr
 
-      NS_TRY_SAFE_CALL_RETURN(numtowrite, CallNPP_WriteReadyProc(callbacks->writeready,
-                                                                   npp,
-                                                                   &mNPStream), lib);
-
-      NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
-      ("NPP WriteReady called: this=%p, npp=%p, return(towrite)=%d, url=%s\n",
-      this, npp, numtowrite, mNPStream.url));
-
-      // if WriteReady returned 0, the plugin is not ready to handle 
-      // the data, return FAILURE for now
-      if (numtowrite <= 0) {
-        NS_ASSERTION(numtowrite,"WriteReady returned Zero");
-        rv = NS_ERROR_FAILURE;
-        goto error;
+    // it is possible plugin's NPP_Write() returns 0 byte consumed
+    // we use zeroBytesWriteCount to count situation like this
+    // and break the loop
+    PRInt32 zeroBytesWriteCount = 0;
+    
+    // amountRead tells us how many bytes were put in the buffer
+    // WriteReady returns to us how many bytes the plugin is 
+    // ready to handle  - we have to keep calling WriteReady and
+    // Write until amountRead > 0
+    for(;;) 
+    { // it breaks on (amountRead <= 0) or on error
+      PRInt32   numtowrite;
+      if (callbacks->writeready)
+      {
+        NS_TRY_SAFE_CALL_RETURN(numtowrite, 
+          CallNPP_WriteReadyProc(callbacks->writeready, npp, &mNPStream),
+          mInst->fLibrary, mInst);
+       
+        NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
+          ("NPP WriteReady called: this=%p, npp=%p, return(towrite)=%d, url=%s\n",
+          this, npp, numtowrite, mNPStream.url));
+        
+        // if WriteReady returned 0, the plugin is not ready to handle 
+        // the data, return FAILURE for now
+        if (numtowrite <= 0) {
+          NS_ASSERTION(numtowrite,"WriteReady returned Zero");
+          rv = NS_ERROR_FAILURE;
+          break;
+        }
+        if (numtowrite > amountRead)
+          numtowrite = amountRead;
       }
-
-      if (numtowrite > amountRead)
+      else
+      {
+        // if WriteReady is not supported by the plugin, 
+        // just write the whole buffer
         numtowrite = amountRead;
-    }
-    else 
-    {
-      // if WriteReady is not supported by the plugin, 
-      // just write the whole buffer
-      numtowrite = length;
-    }
-
-    if (numtowrite > 0)
-    {
-      NS_TRY_SAFE_CALL_RETURN(writeCount, CallNPP_WriteProc(callbacks->write,
-                                                            npp,
-                                                            &mNPStream, 
-                                                            mPosition,
-                                                            numtowrite,
-                                                            (void *)mStreamBuffer), mInst->fLibrary);
-
-      NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
-      ("NPP Write called: this=%p, npp=%p, pos=%d, len=%d, buf=%s, return(written)=%d,  url=%s\n",
-      this, npp, mPosition, numtowrite, (char *)mStreamBuffer, writeCount, mNPStream.url));
-
-      if (writeCount < 0) {
-        rv = NS_ERROR_FAILURE;
-        goto error;
       }
-
-      amountRead -= writeCount;
-      mPosition += writeCount;
-      if (amountRead > 0)
-        memmove(mStreamBuffer,mStreamBuffer+writeCount,amountRead); 
-    }
-  }
-
-  rv = NS_OK;
-
-error:
-  if (PR_TRUE == createdHere) // cleanup buffer if we made it locally
-  {
-    PR_Free(mStreamBuffer);
-    mStreamBuffer=nsnull;
-  }
-
-  if (leftToRead > 0)  // if we have more to read in this pass, do it recursivly
-  {
-    OnDataAvailable(pluginInfo, input, leftToRead);
-  }
+      
+      PRInt32   writeCount = 0; // bytes consumed by plugin instance
+      
+      NS_TRY_SAFE_CALL_RETURN(writeCount, 
+        CallNPP_WriteProc(callbacks->write, npp, &mNPStream, mPosition, numtowrite, (void *)ptrStreamBuffer),
+        mInst->fLibrary, mInst);
+      
+      NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
+        ("NPP Write called: this=%p, npp=%p, pos=%d, len=%d, buf=%s, return(written)=%d,  url=%s\n",
+        this, npp, mPosition, numtowrite, (char *)ptrStreamBuffer, writeCount, mNPStream.url));
+      
+      if (writeCount > 0)
+      {
+        mPosition += writeCount;
+        amountRead -= writeCount;
+        if (amountRead <= 0)
+          break; // in common case we'll break for(;;) loop here 
+        
+        zeroBytesWriteCount = 0;
+        if (writeCount % sizeof(long)) {
+          // memmove will take care  about alignment 
+          memmove(ptrStreamBuffer,ptrStreamBuffer+writeCount,amountRead);
+        } else {
+          // if aligned we can use ptrStreamBuffer += to eliminate memmove()
+          ptrStreamBuffer += writeCount;
+        }
+      } 
+      else if (writeCount == 0)
+      {
+        // if NPP_Write() returns writeCount == 0 lets say 3 times in a raw
+        // lets consider this as end of ODA call (plugin isn't hungry, or broken) without an error.
+        if (++zeroBytesWriteCount == 3)
+        {
+          length = 0; // break do{}while
+          rv = NS_OK;
+          break;
+        }
+      } 
+      else
+      {
+        length = 0; // break do{}while
+        rv = NS_ERROR_FAILURE;
+        break;
+      }  
+    } // end of for(;;)
+  } while ((PRInt32)(length) > 0);
 
   return rv;
 }
 
-
-///////////////////////////////////////////////////////////////////////////////
-NS_IMETHODIMP
-ns4xPluginStreamListener::OnDataAvailable(nsIPluginStreamInfo* pluginInfo,
-                                          nsIInputStream* input,
-                                          PRUint32 sourceOffset, 
-                                          PRUint32 length)
-{
-  if (!mInst || !mInst->IsStarted())
-    return NS_ERROR_FAILURE;
-
-  const NPPluginFuncs *callbacks = nsnull;
-  NPP npp;
-
-  mInst->GetCallbacks(&callbacks);
-  mInst->GetNPP(&npp);
-
-  if(!callbacks)
-    return NS_ERROR_FAILURE;
-
-  PRInt32   numtowrite = 0;
-  PRInt32   amountRead = 0;
-  PRInt32   writeCount = 0;
-  PRUint32  leftToRead = 0;         // just in case OnDataaAvail tries to overflow our buffer
-  PRBool   createdHere = PR_FALSE;  // we did malloc in locally, so we must free locally
-
-  // if we are getting a range request, reset mPosition.  
-  if (sourceOffset != mCurrentStreamOffset ||
-      mCurrentStreamOffset == -1 ) 
-  {
-    mPosition = 0;
-    mCurrentStreamOffset = sourceOffset;
-  }
-
-  pluginInfo->GetURL(&mNPStream.url);
-  pluginInfo->GetLastModified((PRUint32*)&(mNPStream.lastmodified));
-
-  if (callbacks->write == nsnull || length == 0)
-    return NS_OK; // XXX ?
-
-  if (nsnull == mStreamBuffer)
-  {
-    // create the buffer here because we failed in OnStartBinding
-    // XXX why don't we always get an OnStartBinding? This will protect us.
-    mStreamBuffer = (char*) PR_Malloc(length);
-    if (!mStreamBuffer)
-    return NS_ERROR_OUT_OF_MEMORY;
-    createdHere = PR_TRUE;
-  }
-
-  if (length > MAX_PLUGIN_NECKO_BUFFER)  // what if Necko gives us a lot of data?
-  {
-    leftToRead = length - MAX_PLUGIN_NECKO_BUFFER; // break it up
-    length     = MAX_PLUGIN_NECKO_BUFFER;
-  }
-  nsresult rv = input->Read(mStreamBuffer, length, (PRUint32*)&amountRead);
-  if (NS_FAILED(rv))
-    goto error;
-
-  // amountRead tells us how many bytes were put in the buffer
-  // WriteReady returns to us how many bytes the plugin is 
-  // ready to handle  - we have to keep calling WriteReady and
-  // Write until the buffer is empty
-  while (amountRead > 0)
-  {
-    if (callbacks->writeready != NULL)
-    {
-      PRLibrary* lib = nsnull;
-      lib = mInst->fLibrary;
-
-      NS_TRY_SAFE_CALL_RETURN(numtowrite, CallNPP_WriteReadyProc(callbacks->writeready,
-                                                                   npp,
-                                                                   &mNPStream), lib);
-
-      NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
-      ("NPP WriteReady called: this=%p, npp=%p, return(towrite)=%d, url=%s\n",
-      this, npp, numtowrite, mNPStream.url));
-
-      // if WriteReady returned 0, the plugin is not ready to handle 
-      // the data, return FAILURE for now
-      if (numtowrite <= 0) {
-        rv = NS_ERROR_FAILURE;
-        goto error;
-      }
-
-      if (numtowrite > amountRead)
-        numtowrite = amountRead;
-    }
-    else 
-    {
-      // if WriteReady is not supported by the plugin, 
-      // just write the whole buffer
-      numtowrite = length;
-    }
-
-    if (numtowrite > 0)
-    {
-#if 0 // useful for debugging problems with byte range buffers
-      printf(">  %d - %d \n", sourceOffset + writeCount + mPosition, numtowrite);
-      nsCString x; 
-      x.Append("d:\\parts\\");
-      x.AppendInt(sourceOffset);
-
-      PRFileDesc* fd;
-      fd = PR_Open(x, PR_CREATE_FILE |PR_SYNC| PR_APPEND | PR_RDWR, 777);
-      PR_Write(fd, mStreamBuffer, numtowrite);
-      PR_Close(fd);
-#endif
-
-      NS_TRY_SAFE_CALL_RETURN(writeCount, CallNPP_WriteProc(callbacks->write,
-                                                            npp,
-                                                            &mNPStream, 
-                                                            sourceOffset + writeCount + mPosition,
-                                                            numtowrite,
-                                                            (void *)mStreamBuffer), mInst->fLibrary);
-
-     NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
-     ("NPP Write called: this=%p, npp=%p, pos=%d, len=%d, buf=%s, return(written)=%d, url=%s\n",
-     this, npp, mPosition, numtowrite, (char *)mStreamBuffer, writeCount, mNPStream.url));
-
-      if (writeCount < 0) {
-        rv = NS_ERROR_FAILURE;
-        goto error;
-      }
-
-      if (sourceOffset == 0)
-          mPosition += numtowrite;
-
-      amountRead -= numtowrite;
-    }
-  }
-
-  rv = NS_OK;
-
-error:
-  if (PR_TRUE == createdHere) // cleanup buffer if we made it locally
-  {
-    PR_Free(mStreamBuffer);
-    mStreamBuffer=nsnull;
-  }
-
-  if (leftToRead > 0)  // if we have more to read in this pass, do it recursivly
-  {
-    OnDataAvailable(pluginInfo, input, sourceOffset, leftToRead);
-  }
-
-  return rv;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 NS_IMETHODIMP
 ns4xPluginStreamListener::OnFileAvailable(nsIPluginStreamInfo* pluginInfo, 
@@ -579,7 +493,7 @@ ns4xPluginStreamListener::OnFileAvailable(nsIPluginStreamInfo* pluginInfo,
   NS_TRY_SAFE_CALL_VOID(CallNPP_StreamAsFileProc(callbacks->asfile,
                                                    npp,
                                                    &mNPStream,
-                                                   fileName), lib);
+                                                   fileName), lib, mInst);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPP StreamAsFile called: this=%p, npp=%p, url=%s, file=%s\n",
@@ -605,8 +519,14 @@ ns4xPluginStreamListener::OnStopBinding(nsIPluginStreamInfo* pluginInfo,
   // check if the stream is of seekable type and later its destruction
   // see bug 91140    
   nsresult rv = NS_OK;
-  if(mStreamType != nsPluginStreamType_Seek)
-    rv = CleanUpStream(NPRES_DONE);
+  if(mStreamType != nsPluginStreamType_Seek) {
+    NPReason reason = NPRES_DONE;
+
+    if (NS_FAILED(status))
+      reason = NPRES_NETWORK_ERR;   // since the stream failed, we need to tell the plugin that
+
+    rv = CleanUpStream(reason);
+  }
   
   if(rv != NPERR_NO_ERROR)
     return NS_ERROR_FAILURE;
@@ -659,6 +579,7 @@ ns4xPluginInstance :: ns4xPluginInstance(NPPluginFuncs* callbacks, PRLibrary* aL
   mTransparent = PR_FALSE;
   mStarted = PR_FALSE;
   mStreams = nsnull;
+  mCached = PR_FALSE;
 
   PLUGIN_LOG(PLUGIN_LOG_BASIC, ("ns4xPluginInstance ctor: this=%p\n",this));
 }
@@ -752,11 +673,15 @@ NS_IMETHODIMP ns4xPluginInstance::Stop(void)
   NPError error;
 
 #if defined(MOZ_WIDGET_GTK)
-  if (mXtBin)
+  if (mXtBin) {
     gtk_widget_destroy(mXtBin);
+    mXtBin = 0;
+  }
 #elif defined(MOZ_WIDGET_XLIB)
-  if (mXlibXtBin)
+  if (mXlibXtBin) {
     mXlibXtBin->xtbin_destroy();
+    mXlibXtBin = 0;
+  }
 #endif
 
   if(!mStarted)
@@ -765,20 +690,24 @@ NS_IMETHODIMP ns4xPluginInstance::Stop(void)
   if (fCallbacks->destroy == NULL)
     return NS_ERROR_FAILURE; // XXX right error?
 
-  NPSavedData *sdata;
+  NPSavedData *sdata = 0;
 
   // clean up open streams
   for(nsInstanceStream *is = mStreams; is != nsnull;) {
-    if(is->mPluginStreamListener)
-      is->mPluginStreamListener->CleanUpStream(NPRES_USER_BREAK);
+    ns4xPluginStreamListener * listener = is->mPluginStreamListener;
 
     nsInstanceStream *next = is->mNext;
     delete is;
     is = next;
     mStreams = is;
+
+    // Clean up our stream after removing it from the list because 
+    // it may be released and destroyed at this point.
+    if(listener)
+      listener->CleanUpStream(NPRES_USER_BREAK);
   }
 
-  NS_TRY_SAFE_CALL_RETURN(error, CallNPP_DestroyProc(fCallbacks->destroy, &fNPP, &sdata), fLibrary);
+  NS_TRY_SAFE_CALL_RETURN(error, CallNPP_DestroyProc(fCallbacks->destroy, &fNPP, &sdata), fLibrary, this);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPP Destroy called: this=%p, npp=%p, return=%d\n", this, &fNPP, error));
@@ -796,9 +725,8 @@ NS_IMETHODIMP ns4xPluginInstance::Stop(void)
 nsresult ns4xPluginInstance::InitializePlugin(nsIPluginInstancePeer* peer)
 {
   NS_ENSURE_ARG_POINTER(peer);
-  mPeer = peer;
  
-  nsCOMPtr<nsIPluginTagInfo2> taginfo = do_QueryInterface(mPeer);
+  nsCOMPtr<nsIPluginTagInfo2> taginfo = do_QueryInterface(peer);
   NS_ENSURE_TRUE(taginfo, NS_ERROR_NO_INTERFACE);
   
   PRUint16 count = 0;
@@ -837,8 +765,42 @@ nsresult ns4xPluginInstance::InitializePlugin(nsIPluginInstancePeer* peer)
   nsMIMEType    mimetype;
   NPError       error;
 
-  mPeer->GetMode(&mode);
-  mPeer->GetMIMEType(&mimetype);
+  peer->GetMode(&mode);
+  peer->GetMIMEType(&mimetype);
+
+#ifdef XP_UNIX
+  // hack swliveconnect argument for flash plugins on unix,
+  // because if it set flash corrupts the stack in NPP_NewProc call (bug 149336)
+  static const char flashMimeType[] = "application/x-shockwave-flash";
+  static const char blockedParam[] = "swliveconnect";
+  if (count && !PL_strcasecmp(mimetype, flashMimeType)) {
+    static int cachedDisableHack = 0;
+    if (!cachedDisableHack) {
+       if (PR_GetEnv("MOZILLA_PLUGIN_DISABLE_FLASH_SWLIVECONNECT_HACK"))
+         cachedDisableHack = -1;
+       else
+         cachedDisableHack = 1;
+    }
+    if (cachedDisableHack > 0) {
+      for (PRUint16 i=0; i<count; i++) {
+        if (!PL_strcasecmp(names[i], blockedParam)) {
+          // BIG FAT WARNIG:
+          // I'm ugly casting |const char*| to |char*| and altering it
+          // because I know we do malloc it values in
+          // http://bonsai.mozilla.org/cvsblame.cgi?file=mozilla/layout/html/base/src/nsObjectFrame.cpp&rev=1.349&root=/cvsroot#3020
+          // and free it at line #2096, so it couldn't be a const ptr to string literal
+          char *val = (char*) values[i];
+          if (val && *val) {
+            // we cannot just *val=0, it wont be free properly in such case
+            val[0] = '0';
+            val[1] = 0;
+          }
+          break;
+        }
+      }
+    }
+  }
+#endif
 
   NS_TRY_SAFE_CALL_RETURN(error, CallNPP_NewProc(fCallbacks->newp,
                                           (char *)mimetype,
@@ -847,18 +809,21 @@ nsresult ns4xPluginInstance::InitializePlugin(nsIPluginInstancePeer* peer)
                                           count,
                                           (char**)names,
                                           (char**)values,
-                                          NULL), fLibrary);
+                                          NULL), fLibrary,this);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPP New called: this=%p, npp=%p, mime=%s, mode=%d, argc=%d, return=%d\n",
   this, &fNPP, mimetype, mode, count, error));
 
   if(error != NPERR_NO_ERROR)
-    rv = NS_ERROR_FAILURE;
+    return NS_ERROR_FAILURE;
+  
+  // assign  mPeer only if there was no error
+  mPeer = peer;
   
   mStarted = PR_TRUE;
   
-  return rv;
+  return NS_OK;
 }
 
 
@@ -913,7 +878,7 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
              window->width, window->height, win);
 #endif
 
-#if 1
+#if 0
       // if we destroyed the plugin when we left the page, we could remove this
       // code (i believe) the problem here is that the window gets destroyed when
       // its parent, etc does by changing a page the plugin instance is being
@@ -953,9 +918,9 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
 
     // fill in window info structure 
     ws->type = 0; // OK, that was a guess!!
-    ws->depth = gdk_rgb_get_visual()->depth;
+    ws->depth = gdk_window_get_visual(win)->depth;
     ws->display = GTK_XTBIN(mXtBin)->xtdisplay;
-    ws->visual = GDK_VISUAL_XVISUAL(gdk_rgb_get_visual());
+    ws->visual = GDK_VISUAL_XVISUAL(gdk_window_get_visual(win));
     ws->colormap = GDK_COLORMAP_XCOLORMAP(gdk_window_get_colormap(win));
 
     XFlush(ws->display);
@@ -968,6 +933,7 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
   // to the actual X window
   window->window = (nsPluginPort *)GTK_XTBIN(mXtBin)->xtwindow;
   
+  gtk_xtbin_resize(mXtBin, window->width, window->height);
   
 #elif defined(MOZ_WIDGET_XLIB)
 
@@ -1042,11 +1008,12 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
 
     NS_TRY_SAFE_CALL_RETURN(error, CallNPP_SetWindowProc(fCallbacks->setwindow,
                                   &fNPP,
-                                  (NPWindow*) window), fLibrary);
+                                  (NPWindow*) window), fLibrary, this);
+
 
     NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
     ("NPP SetWindow called: this=%p, [x=%d,y=%d,w=%d,h=%d], clip[t=%d,b=%d,l=%d,r=%d], return=%d\n",
-    this, &fNPP, window->x, window->y, window->width, window->height,
+    this, window->x, window->y, window->width, window->height,
     window->clipRect.top, window->clipRect.bottom, window->clipRect.left, window->clipRect.right, error));
       
     // XXX In the old code, we'd just ignore any errors coming
@@ -1059,29 +1026,31 @@ NS_IMETHODIMP ns4xPluginInstance::SetWindow(nsPluginWindow* window)
 
 ////////////////////////////////////////////////////////////////////////
 /* NOTE: the caller must free the stream listener */
+// Create a normal stream, one without a urlnotify callback
 NS_IMETHODIMP ns4xPluginInstance::NewStream(nsIPluginStreamListener** listener)
 {
-  return NewNotifyStream(listener, nsnull);
+  return NewNotifyStream(listener, nsnull, PR_FALSE, nsnull);
 }
 
 
 ////////////////////////////////////////////////////////////////////////
-nsresult ns4xPluginInstance::NewNotifyStream(nsIPluginStreamListener** listener, void* notifyData)
+// Create a stream that will notify when complete
+nsresult ns4xPluginInstance::NewNotifyStream(nsIPluginStreamListener** listener, 
+                                             void* notifyData,
+                                             PRBool aCallNotify,
+                                             const char* aURL)
 {
-  ns4xPluginStreamListener* stream = new ns4xPluginStreamListener(this, notifyData);
-
-  if(stream == nsnull)
-    return NS_ERROR_OUT_OF_MEMORY;
+  ns4xPluginStreamListener* stream = new ns4xPluginStreamListener(this, notifyData, aURL);
+  NS_ENSURE_TRUE(stream, NS_ERROR_OUT_OF_MEMORY);
 
   // add it to the list
   nsInstanceStream * is = new nsInstanceStream();
-
-  if(!is)
-    return NS_ERROR_FAILURE;
+  NS_ENSURE_TRUE(is, NS_ERROR_OUT_OF_MEMORY);
 
   is->mNext = mStreams;
   is->mPluginStreamListener = stream;
   mStreams = is;
+  stream->SetCallNotify(aCallNotify);  // set flag in stream to call URLNotify
 
   NS_ADDREF(stream);  // Stabilize
     
@@ -1121,7 +1090,7 @@ NS_IMETHODIMP ns4xPluginInstance::Print(nsPluginPrint* platformPrint)
 
   NS_TRY_SAFE_CALL_VOID(CallNPP_PrintProc(fCallbacks->print,
                                           &fNPP,
-                                          thePrint), fLibrary);
+                                          thePrint), fLibrary, this);
 
   NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
   ("NPP PrintProc called: this=%p, pDC=%p, [x=%d,y=%d,w=%d,h=%d], clip[t=%d,b=%d,l=%d,r=%d]\n",
@@ -1165,7 +1134,7 @@ NS_IMETHODIMP ns4xPluginInstance::HandleEvent(nsPluginEvent* event, PRBool* hand
 
       NS_TRY_SAFE_CALL_RETURN(result, CallNPP_HandleEventProc(fCallbacks->event,
                                     &fNPP,
-                                    (void*)&npEvent), fLibrary);
+                                    (void*)&npEvent), fLibrary, this);
 #endif
 
       NPP_PLUGIN_LOG(PLUGIN_LOG_NOISY,
@@ -1183,13 +1152,9 @@ NS_IMETHODIMP ns4xPluginInstance::HandleEvent(nsPluginEvent* event, PRBool* hand
 NS_IMETHODIMP ns4xPluginInstance :: GetValue(nsPluginInstanceVariable variable,
                                              void *value)
 {
-  if(!mStarted)
-    return NS_OK;
-
   nsresult  res = NS_OK;
 
-  switch (variable)
-  {
+  switch (variable) {
     case nsPluginInstanceVariable_WindowlessBool:
       *(PRBool *)value = mWindowless;
       break;
@@ -1198,22 +1163,28 @@ NS_IMETHODIMP ns4xPluginInstance :: GetValue(nsPluginInstanceVariable variable,
       *(PRBool *)value = mTransparent;
       break;
 
+    case nsPluginInstanceVariable_DoCacheBool:
+      *(PRBool *)value = mCached;
+      break;
+
+    case nsPluginInstanceVariable_CallSetWindowAfterDestroyBool:
+      *(PRBool *)value = 0;  // not supported for 4.x plugins
+      break;
+
     default:
-      if(fCallbacks->getvalue)
-      {
+      if(fCallbacks->getvalue && mStarted) {
         NS_TRY_SAFE_CALL_RETURN(res, 
                                 CallNPP_GetValueProc(fCallbacks->getvalue, 
                                                      &fNPP, 
                                                      (NPPVariable)variable, 
                                                      value), 
-                                fLibrary);
+                                                     fLibrary, this);
+        NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
+        ("NPP GetValue called: this=%p, npp=%p, var=%d, value=%d, return=%d\n", 
+        this, &fNPP, variable, value, res));
       }
   }
 
-  NPP_PLUGIN_LOG(PLUGIN_LOG_NORMAL,
-  ("NPP GetValue called: this=%p, npp=%p, var=%d, value=%d, return=%d\n", 
-  this, &fNPP, variable, value, res));
-
   return res;
 }
 
