@@ -51,7 +51,7 @@
 #include "nsILinkHandler.h"
 #include "nsIWebShell.h"
 #include "nsIContentViewerEdit.h"
-#include "nsIContentViewerFile.h"
+#include "nsIWebBrowserPrint.h"
 #include "nsIWidget.h"
 #include "nsIContent.h"
 #include "nsIDocument.h"
@@ -64,12 +64,13 @@
 #include "nsPluginViewer.h"
 #include "nsGUIEvent.h"
 #include "nsIPluginViewer.h"
-
-
+#include "nsContentCID.h"
+#include "nsIScriptGlobalObject.h"
 #include "nsITimer.h"
 #include "nsITimerCallback.h"
 
 class nsIPrintSettings;
+class nsIDOMWindow;
 
 // Class IDs
 static NS_DEFINE_IID(kChildWindowCID, NS_CHILD_CID);
@@ -80,7 +81,7 @@ static NS_DEFINE_IID(kIContentViewerIID, NS_ICONTENTVIEWER_IID);
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 static NS_DEFINE_IID(kCPluginManagerCID, NS_PLUGINMANAGER_CID);
 static NS_DEFINE_IID(kIDocumentIID, NS_IDOCUMENT_IID);
-
+static NS_DEFINE_IID(kHTMLDocumentCID, NS_HTMLDOCUMENT_CID);
 
 class PluginViewerImpl;
 
@@ -154,9 +155,10 @@ public:
   NS_IMETHOD_(void) Notify(nsITimer *timer);
   void CancelTimer();
 
+  nsPluginPort* GetPluginPort();
+
 #ifdef XP_MAC
   void GUItoMacEvent(const nsGUIEvent& anEvent, EventRecord& aMacEvent);
-  nsPluginPort* GetPluginPort();
   void FixUpPluginWindow();
 #endif
                    
@@ -166,62 +168,15 @@ private:
   nsIWidget         *mWindow;       //we do not addref this...
   PluginViewerImpl  *mViewer;       //we do not addref this...
   nsCOMPtr<nsITimer> mPluginTimer;
+  PRPackedBool       mWidgetVisible;    // used on Mac to store our widget's visible state
 };
 
-  static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY,
-                                nsRect& aClipRect);
+#ifdef XP_MAC
+  static void GetWidgetPosClipAndVis(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY, 
+                                     nsRect& aClipRect, PRBool& aIsVisible);
+#endif
 
 
-class PluginViewerImpl : public nsIPluginViewer,
-                         public nsIContentViewer,
-                         public nsIContentViewerEdit,
-                         public nsIContentViewerFile
-{
-public:
-  PluginViewerImpl(const char* aCommand);
-  nsresult Init(nsIStreamListener** aDocListener);
-    
-  NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
-
-  // nsISupports
-  NS_DECL_ISUPPORTS
-
-  // nsIPluginViewer
-  NS_IMETHOD StartLoad(nsIRequest* request, nsIStreamListener*& aResult);
-
-  // nsIContentViewer
-  NS_DECL_NSICONTENTVIEWER
-
-  // nsIContentViewerEdit
-  NS_DECL_NSICONTENTVIEWEREDIT
-
-  // nsIContentViewerFile
-  NS_DECL_NSICONTENTVIEWERFILE
-
-  virtual ~PluginViewerImpl();
-
-  nsresult CreatePlugin(nsIRequest* request, nsIPluginHost* aHost, const nsRect& aBounds,
-                        nsIStreamListener*& aResult);
-
-  nsresult MakeWindow(nsNativeWidget aParent,
-                      nsIDeviceContext* aDeviceContext,
-                      const nsRect& aBounds);
-
-  void ForceRefresh(void);
-
-  nsresult GetURI(nsIURI* *aURI);
-
-  nsresult GetDocument(nsIDocument* *aDocument);
-
-  nsIWidget* mWindow;
-  nsIDocument* mDocument;
-  nsCOMPtr<nsISupports> mContainer;
-  nsIChannel* mChannel;
-  pluginInstanceOwner *mOwner;
-  PRBool mEnableRendering;
-
-};
-
 //----------------------------------------------------------------------
 
 nsresult
@@ -266,7 +221,7 @@ NS_IMPL_QUERY_INTERFACE4(PluginViewerImpl,
                          nsIPluginViewer,
                          nsIContentViewer,
                          nsIContentViewerEdit,
-                         nsIContentViewerFile)
+                         nsIWebBrowserPrint)
 
 
 PluginViewerImpl::~PluginViewerImpl()
@@ -320,6 +275,7 @@ PluginViewerImpl::Init(nsIWidget* aParentWidget,
                        nsIDeviceContext* aDeviceContext,
                        const nsRect& aBounds)
 {
+  NS_ENSURE_TRUE(aParentWidget, NS_ERROR_NULL_POINTER);
   nsresult rv = MakeWindow(aParentWidget->GetNativeData(NS_NATIVE_WIDGET),
    aDeviceContext, aBounds);
   if (NS_OK == rv) {
@@ -343,10 +299,9 @@ PluginViewerImpl::StartLoad(nsIRequest* request, nsIStreamListener*& aResult)
   NS_ADDREF(mChannel);
 
 #ifdef DEBUG
-  char* contentType;
-  channel->GetContentType(&contentType);
-  printf("PluginViewerImpl::StartLoad: content-type=%s\n", contentType);
-  nsCRT::free(contentType);
+  nsCAutoString contentType;
+  channel->GetContentType(contentType);
+  printf("PluginViewerImpl::StartLoad: content-type=%s\n", contentType.get());
 #endif
 
   aResult = nsnull;
@@ -358,9 +313,42 @@ PluginViewerImpl::StartLoad(nsIRequest* request, nsIStreamListener*& aResult)
   nsresult rv = NS_ERROR_FAILURE;
   if(host) 
   {
+    // create a document so that we can pass something back to plugin
+    // instance if it wants one
+    nsCOMPtr<nsIDocument> doc(do_CreateInstance(kHTMLDocumentCID));
+    if (doc) {    
+      mDocument = doc;
+      NS_ADDREF(mDocument);  // released in ~nsPluginViewer
+      
+      // set the document's URL 
+      // so it can be fetched later for resolving relative URLs
+      nsCOMPtr<nsIURI> uri;
+      GetURI(getter_AddRefs(uri));
+      if (uri)
+        mDocument->SetDocumentURL(uri);
+       
+      // we're going to need the docshell later to reload this full-page plugin in the event of a plugins refresh
+      // so stuff it in the document we created
+      nsCOMPtr<nsIScriptGlobalObject> global (do_GetInterface(mContainer));
+      if (global) {
+        mDocument->SetScriptGlobalObject(global);
+        nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(mDocument));
+        if (domdoc)
+          global->SetNewDocument(domdoc, PR_TRUE, PR_TRUE);
+      }
+
+    }
+
     nsRect r;
     mWindow->GetClientBounds(r);
     rv = CreatePlugin(request, host, nsRect(0, 0, r.width, r.height), aResult);
+
+#ifdef XP_MAC
+    // On Mac, we need to initiate the intial invalidate for full-page plugins to ensure
+    // the entire window gets cleared. Otherwise, Acrobat won't initially repaint on top 
+    // of our previous presentation and we may have garbage leftover
+    mWindow->Invalidate(PR_FALSE);
+#endif
   }
 
   return rv;
@@ -393,21 +381,18 @@ PluginViewerImpl::CreatePlugin(nsIRequest* request, nsIPluginHost* aHost, const
     rv = mChannel->GetURI(&uri);
     if (NS_FAILED(rv)) return rv;
 
-    char* spec;
-    rv = uri->GetSpec(&spec);
+    nsCAutoString spec;
+    rv = uri->GetSpec(spec);
     NS_RELEASE(uri);
     if (NS_FAILED(rv)) return rv;
-    nsAutoString str; str.AssignWithConversion(spec);
-    nsCRT::free(spec);
+    NS_ConvertUTF8toUCS2 str(spec);
 
-    char* ct;
+    nsCAutoString ct;
   
     nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
-    channel->GetContentType(&ct);
+    channel->GetContentType(ct);
     if (NS_FAILED(rv)) return rv;
-    rv = aHost->InstantiateFullPagePlugin(ct, str, aResult, mOwner);
-   
-    delete[] ct;
+    rv = aHost->InstantiateFullPagePlugin(ct.get(), str, aResult, mOwner);
   }
 
   return rv;
@@ -450,6 +435,12 @@ PluginViewerImpl::Unload(void)
 NS_IMETHODIMP
 PluginViewerImpl::Close(void)
 {
+  nsCOMPtr<nsIScriptGlobalObject> sgo(do_GetInterface(mContainer));
+
+  if (sgo) {
+    sgo->SetNewDocument(nsnull, PR_TRUE, PR_TRUE);
+  }
+
   return NS_OK;
 }
 
@@ -526,7 +517,7 @@ HandlePluginEvent(nsGUIEvent *aEvent)
     void *clientData;
     (nsIWidget*)(aEvent->widget)->GetClientData(clientData);
     PluginViewerImpl * pluginViewer = (PluginViewerImpl *)clientData;
-    if (pluginViewer != nsnull && pluginViewer->mOwner != nsnull)
+    if (pluginViewer && pluginViewer->mOwner && pluginViewer->mWindow == aEvent->widget)
       return pluginViewer->mOwner->ProcessEvent(*aEvent);
 #endif // XP_MAC
   }
@@ -567,6 +558,15 @@ PluginViewerImpl::GetBounds(nsRect& aResult)
   return NS_OK;
 }
 
+#ifdef XP_WIN
+NS_IMETHODIMP
+PluginViewerImpl::GetPluginPort(HWND *aPort)
+{
+  *aPort = (HWND) mOwner->GetPluginPort();
+  return NS_OK;
+}
+#endif
+
 NS_IMETHODIMP
 PluginViewerImpl::GetPreviousViewer(nsIContentViewer** aViewer)
 {
@@ -602,7 +602,10 @@ PluginViewerImpl::SetBounds(const nsRect& aBounds)
         win->clipRect.left = aBounds.x;
         win->clipRect.bottom = aBounds.YMost();
         win->clipRect.right = aBounds.XMost();
-
+        
+#ifdef XP_MAC   // On Mac we also need to add in the widget offset to the plugin window
+        mOwner->FixUpPluginWindow();
+#endif        
         inst->SetWindow(win);
       }
       NS_RELEASE(inst);
@@ -628,6 +631,9 @@ PluginViewerImpl::Move(PRInt32 aX, PRInt32 aY)
         win->clipRect.top = aY;
         win->clipRect.left = aX;
 
+#ifdef XP_MAC   // On Mac we also need to add in the widget offset to the plugin window
+        mOwner->FixUpPluginWindow();
+#endif
         inst->SetWindow(win);
       }
       NS_RELEASE(inst);
@@ -679,6 +685,20 @@ PluginViewerImpl::GetEnableRendering(PRBool* aResult)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+PluginViewerImpl::SetSticky(PRBool aSticky)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+PluginViewerImpl::GetSticky(PRBool *aSticky)
+{
+  *aSticky = PR_FALSE;
+
+  return NS_OK;
+}
+
 void
 PluginViewerImpl::ForceRefresh()
 {
@@ -717,7 +737,7 @@ NS_IMETHODIMP pluginInstanceOwner::GetValue(nsPluginInstancePeerVariable variabl
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 /* ========================================================================================
- * nsIContentViewerFile
+ * nsIContentViewerEdit
  * ======================================================================================== */
 
 NS_IMETHODIMP PluginViewerImpl::Search()
@@ -781,35 +801,10 @@ NS_IMETHODIMP PluginViewerImpl::GetPasteable(PRBool *aPasteable)
 }
 
 /* ========================================================================================
- * nsIContentViewerEdit
+ * nsIWebBrowserPrint
  * ======================================================================================== */
 NS_IMETHODIMP
-PluginViewerImpl::Save()
-{
-  NS_ASSERTION(0, "NOT IMPLEMENTED");
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-PluginViewerImpl::GetSaveable(PRBool *aSaveable)
-{
-  NS_ASSERTION(0, "NOT IMPLEMENTED");
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-#ifdef NS_DEBUG
-NS_IMETHODIMP
-PluginViewerImpl::Print(PRBool            aSilent,
-                        FILE *            aDebugFile, 
-                        nsIPrintSettings* aPrintSettings)
-{
-  return NS_ERROR_FAILURE;
-}
-#endif
-
-NS_IMETHODIMP
-PluginViewerImpl::Print(PRBool            aSilent,
-                        nsIPrintSettings* aPrintSettings,
+PluginViewerImpl::Print(nsIPrintSettings* aPrintSettings,
                         nsIWebProgressListener* aWebProgressListener)
 {
   nsPluginPrint npprint;
@@ -827,32 +822,111 @@ PluginViewerImpl::Print(PRBool            aSilent,
 
 }
 
+/* readonly attribute nsIPrintSettings globalPrintSettings; */
 NS_IMETHODIMP
-PluginViewerImpl::PrintPreview(nsIPrintSettings* aPrintSettings)
+PluginViewerImpl::GetGlobalPrintSettings(nsIPrintSettings * *aGlobalPrintSettings)
 {
-  return NS_OK;      // XXX: hey, plug in guys!  implement me!
+  return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP
-PluginViewerImpl::GetPrintable(PRBool *aPrintable) 
+/* void printPreview (in nsIPrintSettings aThePrintSettings); */
+NS_IMETHODIMP 
+PluginViewerImpl::PrintPreview(nsIPrintSettings *aThePrintSettings, 
+                               nsIDOMWindow *aChildDOMWin, 
+                               nsIWebProgressListener* aWebProgressListener)
 {
-  NS_ENSURE_ARG_POINTER(aPrintable);
-
-  *aPrintable = PR_FALSE;   // XXX: hey, plug in guys!  implement me!
-  return NS_OK;
+    return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-
-NS_IMETHODIMP
-PluginViewerImpl::PrintContent(nsIWebShell *      aParent,
-                               nsIDeviceContext * aDContext,
-                               nsIDOMWindow     * aDOMWin,
-                               PRBool             aIsSubDoc)
+/* void printPreviewNavigate (in short aNavType, in long aPageNum); */
+NS_IMETHODIMP 
+PluginViewerImpl::PrintPreviewNavigate(PRInt16 aNavType, PRInt32 aPageNum)
 {
-  NS_ENSURE_ARG_POINTER(aParent);
-  NS_ENSURE_ARG_POINTER(aDContext);
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
 
-  return NS_OK;
+/* readonly attribute boolean doingPrintPreview; */
+NS_IMETHODIMP
+PluginViewerImpl::GetDoingPrintPreview(PRBool *aDoingPrintPreview)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean doingPrint; */
+NS_IMETHODIMP
+PluginViewerImpl::GetDoingPrint(PRBool *aDoingPrint)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute nsIPrintSettings currentPrintSettings; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetCurrentPrintSettings(nsIPrintSettings * *aCurrentPrintSettings)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute nsIDOMWindow currentChildDOMWindow; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetCurrentChildDOMWindow(nsIDOMWindow * *aCurrentChildDOMWindow)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void cancel (); */
+NS_IMETHODIMP 
+PluginViewerImpl::Cancel()
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void enumerateDocumentNames (out PRUint32 aCount, [array, size_is (aCount), retval] out wstring aResult); */
+NS_IMETHODIMP 
+PluginViewerImpl::EnumerateDocumentNames(PRUint32 *aCount, PRUnichar ***aResult)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isFramesetDocument; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsFramesetDocument(PRBool *aIsFramesetDocument)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isIFrameSelected; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsIFrameSelected(PRBool *aIsIFrameSelected)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isRangeSelection; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsRangeSelection(PRBool *aIsRangeSelection)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute boolean isFramesetFrameSelected; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetIsFramesetFrameSelected(PRBool *aIsFramesetFrameSelected)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* readonly attribute long printPreviewNumPages; */
+NS_IMETHODIMP 
+PluginViewerImpl::GetPrintPreviewNumPages(PRInt32 *aPrintPreviewNumPages)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+/* void exitPrintPreview (); */
+NS_IMETHODIMP 
+PluginViewerImpl::ExitPrintPreview()
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
@@ -911,14 +985,7 @@ NS_IMETHODIMP
 PluginListener::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
   nsresult rv;
-  char* contentType = nsnull;
 
-  nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
-  rv = channel->GetContentType(&contentType);
-
-  if (NS_FAILED(rv)) {
-    return rv;
-  }
   rv = mViewer->StartLoad(request, mNextStream);
 
   if (NS_FAILED(rv)) {
@@ -959,6 +1026,7 @@ pluginInstanceOwner :: pluginInstanceOwner()
   mInstance = nsnull;
   mWindow = nsnull;
   mViewer = nsnull;
+  mWidgetVisible = PR_TRUE;
 }
 
 pluginInstanceOwner :: ~pluginInstanceOwner()
@@ -1019,7 +1087,7 @@ NS_IMETHODIMP pluginInstanceOwner :: CreateWidget(void)
           // start a periodic timer to provide null events to the plugin instance.
           mPluginTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
           if (rv == NS_OK)
-            rv = mPluginTimer->Init(this, 1020 / 60, NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_SLACK);
+            rv = mPluginTimer->Init(this, 1020 / 60, PR_TRUE, NS_TYPE_REPEATING_SLACK);
 #endif
 
 
@@ -1075,18 +1143,9 @@ NS_IMETHODIMP pluginInstanceOwner::GetURL(const char *aURL, const char *aTarget,
 
   // deal with post data, either in a file or raw data, and any headers
   if (aPostData) {
-    const char * dataToPost = (const char *)aPostData;  // default to raw data
-    nsXPIDLCString filename;
-
-    if (isFile) {
-      // convert file:///c:/ to c: if needed
-      nsCOMPtr<nsILocalFile> aFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);
-      if (NS_SUCCEEDED(NS_InitFileFromURLSpec(aFile,dataToPost)) &&
-          NS_SUCCEEDED(aFile->GetPath(getter_Copies(filename))))
-          dataToPost = filename;
-    }
-
-    rv = NS_NewPluginPostDataStream(getter_AddRefs(postDataStream), dataToPost, aPostDataLen, isFile);
+    rv = NS_NewPluginPostDataStream(getter_AddRefs(postDataStream), (const char *)aPostData, aPostDataLen, isFile);
+    NS_ASSERTION(NS_SUCCEEDED(rv),"failed in creating plugin post data stream");
+    if (NS_FAILED(rv)) return rv;
 
     if (aHeadersData) {
       rv = NS_NewPluginPostDataStream(getter_AddRefs(headersDataStream), 
@@ -1095,6 +1154,7 @@ NS_IMETHODIMP pluginInstanceOwner::GetURL(const char *aURL, const char *aTarget,
                                       PR_FALSE,
                                       PR_TRUE);  // last arg says we are headers
       NS_ASSERTION(NS_SUCCEEDED(rv),"failed in creating plugin header data stream");
+      if (NS_FAILED(rv)) return rv;
     }
   }
 
@@ -1170,92 +1230,80 @@ NS_IMETHODIMP pluginInstanceOwner :: Init(PluginViewerImpl *aViewer, nsIWidget *
 #ifdef XP_MAC
 
 #if TARGET_CARBON
-inline Boolean OSEventAvail(EventMask mask, EventRecord* event) { return EventAvail(mask, event); }
+static void InitializeEventRecord(EventRecord* event)
+{
+    memset(event, 0, sizeof(EventRecord));
+    GetGlobalMouse(&event->where);
+    event->when = TickCount();
+    event->modifiers = GetCurrentKeyModifiers();
+}
+#else
+inline void InitializeEventRecord(EventRecord* event) { ::OSEventAvail(0, event); }
 #endif
 
 void pluginInstanceOwner::GUItoMacEvent(const nsGUIEvent& anEvent, EventRecord& aMacEvent)
 {
-       ::OSEventAvail(0, &aMacEvent);
-       switch (anEvent.message) {
-       case NS_GOTFOCUS:
-       case NS_FOCUS_EVENT_START:
-               aMacEvent.what = nsPluginEventType_GetFocusEvent;
-               break;
-       case NS_LOSTFOCUS:
-       case NS_MOUSE_EXIT:
-               aMacEvent.what = nsPluginEventType_LoseFocusEvent;
-               break;
-       case NS_MOUSE_MOVE:
-       case NS_MOUSE_ENTER:
-         mWindow->SetFocus();
-               aMacEvent.what = nsPluginEventType_AdjustCursorEvent;
-               break;
-       case NS_PAINT:
-          aMacEvent.what = updateEvt;
-         break;
-       case NS_KEY_DOWN:
-       case NS_KEY_PRESS:
-         break;
-         
-       default:
-               aMacEvent.what = nullEvent;
-               break;
-       }
+    InitializeEventRecord(&aMacEvent);
+    switch (anEvent.message) {
+    case NS_GOTFOCUS:
+    case NS_FOCUS_EVENT_START:
+        aMacEvent.what = nsPluginEventType_GetFocusEvent;
+        break;
+    case NS_LOSTFOCUS:
+    case NS_MOUSE_EXIT:
+        aMacEvent.what = nsPluginEventType_LoseFocusEvent;
+        break;
+    case NS_MOUSE_MOVE:
+    case NS_MOUSE_ENTER:
+        mWindow->SetFocus();
+        aMacEvent.what = nsPluginEventType_AdjustCursorEvent;
+        break;
+    case NS_PAINT:
+        aMacEvent.what = updateEvt;
+        break;
+    case NS_KEY_DOWN:
+    case NS_KEY_PRESS:
+        break;
+
+    default:
+        aMacEvent.what = nullEvent;
+        break;
+    }
 }
 #endif
 
 nsEventStatus pluginInstanceOwner::ProcessEvent(const nsGUIEvent& anEvent)
 {
   nsEventStatus rv = nsEventStatus_eIgnore;
-  if (!mInstance || !mWindow)   // if the instance or the window is null, we shouldn't be here
+  if (!mInstance || !mWindow || anEvent.message == NS_MENU_SELECTED)
     return rv;
 
 #ifdef XP_MAC
     //if (mWidget != NULL) {  // check for null mWidget
         EventRecord* event = (EventRecord*)anEvent.nativeMsg;
         if (event == NULL || event->what == nullEvent ||
-            anEvent.message == NS_KEY_PRESS           ||
             anEvent.message == NS_CONTEXTMENU_MESSAGE_START) {
             EventRecord macEvent;
             GUItoMacEvent(anEvent, macEvent);
             event = &macEvent;
             if (event->what == updateEvt) {
-              nsPluginPort* pluginPort = GetPluginPort();
+                nsPluginPort* pluginPort = GetPluginPort();
                 // Add in child windows absolute position to get make the dirty rect
                 // relative to the top-level window.
-                nscoord absWidgetX = 0;
-                nscoord absWidgetY = 0;
-               nsRect widgetClip(0,0,0,0);
-               GetWidgetPosAndClip(mWindow,absWidgetX,absWidgetY,widgetClip);
-               //mViewer->GetBounds(widgetClip);
-               //absWidgetX = widgetClip.x;
-               //absWidgetY = widgetClip.y;
-               
-                // set the port
-    mPluginWindow.x = absWidgetX;
-    mPluginWindow.y = absWidgetY;
+                FixUpPluginWindow();
+                
+                EventRecord updateEvent;
+                InitializeEventRecord(&updateEvent);
+                updateEvent.what = updateEvt;
+                updateEvent.message = UInt32(pluginPort->port);
 
-
-    // fix up the clipping region
-    mPluginWindow.clipRect.top = widgetClip.y;
-    mPluginWindow.clipRect.left = widgetClip.x;
-    mPluginWindow.clipRect.bottom =  mPluginWindow.clipRect.top + widgetClip.height;
-    mPluginWindow.clipRect.right =  mPluginWindow.clipRect.left + widgetClip.width;  
-
-    EventRecord updateEvent;
-    ::OSEventAvail(0, &updateEvent);
-    updateEvent.what = updateEvt;
-    updateEvent.message = UInt32(pluginPort->port);
-
-    nsPluginEvent pluginEvent = { &updateEvent, nsPluginPlatformWindowRef(pluginPort->port) };
-    PRBool eventHandled = PR_FALSE;
-    mInstance->HandleEvent(&pluginEvent, &eventHandled);
-     }
-            
-            return  nsEventStatus_eConsumeNoDefault;
+                nsPluginEvent pluginEvent = { &updateEvent, nsPluginPlatformWindowRef(pluginPort->port) };
+                PRBool eventHandled = PR_FALSE;
+                mInstance->HandleEvent(&pluginEvent, &eventHandled);
+            }
+            return nsEventStatus_eConsumeNoDefault;
             
         }
-        //nsPluginPort* port = (nsPluginPort*)mWidget->GetNativeData(NS_NATIVE_PLUGIN_PORT);
         nsPluginPort* port = (nsPluginPort*)mWindow->GetNativeData(NS_NATIVE_PLUGIN_PORT);
         nsPluginEvent pluginEvent = { event, nsPluginPlatformWindowRef(port->port) };
         PRBool eventHandled = PR_FALSE;
@@ -1268,10 +1316,12 @@ nsEventStatus pluginInstanceOwner::ProcessEvent(const nsGUIEvent& anEvent)
 //~~~
 #ifdef XP_WIN
         nsPluginEvent * pPluginEvent = (nsPluginEvent *)anEvent.nativeMsg;
-        PRBool eventHandled = PR_FALSE;
-        mInstance->HandleEvent(pPluginEvent, &eventHandled);
-        if (eventHandled)
-            rv = nsEventStatus_eConsumeNoDefault;
+        if (pPluginEvent) {
+            PRBool eventHandled = PR_FALSE;
+            mInstance->HandleEvent(pPluginEvent, &eventHandled);
+            if (eventHandled)
+                rv = nsEventStatus_eConsumeNoDefault;
+        }
 #endif
 
   return rv;
@@ -1289,8 +1339,13 @@ NS_IMETHODIMP_(void) pluginInstanceOwner::Notify(nsITimer* /* timer */)
     FixUpPluginWindow();
     if (mInstance != NULL) {
         EventRecord idleEvent;
-        ::OSEventAvail(0, &idleEvent);
+        InitializeEventRecord(&idleEvent);
         idleEvent.what = nullEvent;
+
+        // give a bogus 'where' field of our null event when hidden, so Flash
+        // won't respond to mouse moves in other tabs, see bug 120875
+        if (!mWidgetVisible)
+          idleEvent.where.h = idleEvent.where.v = 20000;
         
         nsPluginPort* pluginPort = GetPluginPort();
         nsPluginEvent pluginEvent = { &idleEvent, nsPluginPlatformWindowRef(pluginPort->port) };
@@ -1319,22 +1374,29 @@ void pluginInstanceOwner::CancelTimer()
     }
 }
 
-
-#ifdef XP_MAC
 nsPluginPort* pluginInstanceOwner::GetPluginPort()
 {
+//!!! Port must be released for windowless plugins on Windows, because it is HDC !!!
 
   nsPluginPort* result = NULL;
-    if (mWindow != NULL)
-      result = (nsPluginPort*) mWindow->GetNativeData(NS_NATIVE_PLUGIN_PORT);
-    return result;
+  if (mWindow != NULL)
+  {
+    result = (nsPluginPort*) mWindow->GetNativeData(NS_NATIVE_PLUGIN_PORT);
+  }
+  return result;
 }
 
+#ifdef XP_MAC
+
   // calculate the absolute position and clip for a widget 
   // and use other windows in calculating the clip
-static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY,
-                                nsRect& aClipRect) 
+  // also find out if we are visible or not
+static void GetWidgetPosClipAndVis(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbsY,
+                                   nsRect& aClipRect, PRBool& aIsVisible) 
 {
+  if (aIsVisible)
+    aWidget->IsVisible(aIsVisible);
+
   aWidget->GetBounds(aClipRect); 
   aAbsX = aClipRect.x; 
   aAbsY = aClipRect.y; 
@@ -1344,10 +1406,12 @@ static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbs
   aClipRect.x = 0; 
   aClipRect.y = 0; 
 
-   // Gather up the absolute position of the widget 
-   // + clip window 
+  // Gather up the absolute position of the widget, clip window, and visibilty 
   nsCOMPtr<nsIWidget> widget = getter_AddRefs(aWidget->GetParent());
   while (widget != nsnull) { 
+    if (aIsVisible)
+      widget->IsVisible(aIsVisible);
+
     nsRect wrect; 
     widget->GetClientBounds(wrect); 
     nscoord wx, wy; 
@@ -1372,6 +1436,10 @@ static void GetWidgetPosAndClip(nsIWidget* aWidget,nscoord& aAbsX, nscoord& aAbs
   aClipRect.x += aAbsX; 
   aClipRect.y += aAbsY; 
 
+  // if we are not visible, clear out the plugin's clip so it won't paint
+  if (!aIsVisible)
+    aClipRect.Empty();
+
   //printf("--------------\n"); 
   //printf("Widget clip X %d Y %d rect %d %d %d %d\n", aAbsX, aAbsY,  aClipRect.x,  aClipRect.y, aClipRect.width,  aClipRect.height ); 
   //printf("--------------\n"); 
@@ -1384,7 +1452,11 @@ void pluginInstanceOwner::FixUpPluginWindow()
     nscoord absWidgetX = 0;
     nscoord absWidgetY = 0;
     nsRect widgetClip(0,0,0,0);
-    GetWidgetPosAndClip(mWindow,absWidgetX,absWidgetY,widgetClip);
+    PRBool isVisible = PR_TRUE;
+    GetWidgetPosClipAndVis(mWindow,absWidgetX,absWidgetY,widgetClip,isVisible);
+
+    if (mWidgetVisible != isVisible)
+      mWidgetVisible = isVisible;
 
     // set the port coordinates
     mPluginWindow.x = absWidgetX;
@@ -1394,7 +1466,7 @@ void pluginInstanceOwner::FixUpPluginWindow()
     mPluginWindow.clipRect.top = widgetClip.y;
     mPluginWindow.clipRect.left = widgetClip.x;
     mPluginWindow.clipRect.bottom =  mPluginWindow.clipRect.top + widgetClip.height;
-    mPluginWindow.clipRect.right =  mPluginWindow.clipRect.left + widgetClip.width; 
+    mPluginWindow.clipRect.right =  mPluginWindow.clipRect.left + widgetClip.width;  
   }
 }
 
