@@ -49,6 +49,7 @@
 #include "nsReadableUtils.h"
 #include "nsIProxyObjectManager.h"
 #include "nsNetUtil.h"
+#include "nsProxyRelease.h"
 
 static NS_DEFINE_CID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
 
@@ -101,14 +102,8 @@ public:
         mLastError = rv;
         return rv;
     }
-    NS_IMETHOD GetNonBlocking(PRBool *aValue) {
-        return mSource->GetNonBlocking(aValue);
-    }
-    NS_IMETHOD GetObserver(nsIInputStreamObserver **aObserver) {
-        return mSource->GetObserver(aObserver);
-    }
-    NS_IMETHOD SetObserver(nsIInputStreamObserver *aObserver) {
-        return mSource->SetObserver(aObserver);
+    NS_IMETHOD IsNonBlocking(PRBool *aValue) {
+        return mSource->IsNonBlocking(aValue);
     }
 
     //
@@ -182,17 +177,8 @@ public:
         mLastError = rv;
         return rv;
     }
-    NS_IMETHOD GetNonBlocking(PRBool *aValue) {
-        return mSink->GetNonBlocking(aValue);
-    }
-    NS_IMETHOD SetNonBlocking(PRBool aValue) {
-        return mSink->SetNonBlocking(aValue);
-    }
-    NS_IMETHOD GetObserver(nsIOutputStreamObserver **aObserver) {
-        return mSink->GetObserver(aObserver);
-    }
-    NS_IMETHOD SetObserver(nsIOutputStreamObserver *aObserver) {
-        return mSink->SetObserver(aObserver);
+    NS_IMETHOD IsNonBlocking(PRBool *aValue) {
+        return mSink->IsNonBlocking(aValue);
     }
 
     //
@@ -223,8 +209,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS1(nsFileTransportSinkWrapper, nsIOutputStream)
 //////////////////////////////////////////////////////////////////////////////////
 
 nsFileTransport::nsFileTransport()
-    : mContentType(nsnull),
-      mBufferSegmentSize(NS_FILE_TRANSPORT_DEFAULT_SEGMENT_SIZE),
+    : mBufferSegmentSize(NS_FILE_TRANSPORT_DEFAULT_SEGMENT_SIZE),
       mBufferMaxSize(NS_FILE_TRANSPORT_DEFAULT_BUFFER_SIZE),
       mXferState(CLOSED),
       mRunState(RUNNING),
@@ -250,31 +235,41 @@ nsFileTransport::nsFileTransport()
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, nsIFile* file, PRInt32 ioFlags, PRInt32 perm)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      nsIFile *file,
+                      PRInt32 ioFlags,
+                      PRInt32 perm,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv;
     nsCOMPtr<nsIFileIO> io;
     rv = NS_NewFileIO(getter_AddRefs(io), file, ioFlags, perm);
     if (NS_FAILED(rv)) return rv;
 
-    return Init(aService, io);
+    return Init(aService, io, closeStreamWhenDone);
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, const char* name, nsIInputStream* inStr,
-                      const char* contentType, PRInt32 contentLength, PRBool closeStreamWhenDone)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      const nsACString &name,
+                      nsIInputStream* inStr,
+                      const nsACString &contentType,
+                      const nsACString &contentCharset,
+                      PRInt32 contentLength,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv;
     nsCOMPtr<nsIInputStreamIO> io;
-    rv = NS_NewInputStreamIO(getter_AddRefs(io),
-                             name, inStr, contentType, contentLength);
+    rv = NS_NewInputStreamIO(getter_AddRefs(io), name, inStr,
+                             contentType, contentCharset, contentLength);
     if (NS_FAILED(rv)) return rv;
-    mCloseStreamWhenDone = closeStreamWhenDone;
-    return Init(aService, io);
+    return Init(aService, io, closeStreamWhenDone);
 }
 
 nsresult
-nsFileTransport::Init(nsFileTransportService *aService, nsIStreamIO* io)
+nsFileTransport::Init(nsFileTransportService *aService,
+                      nsIStreamIO* io,
+                      PRBool closeStreamWhenDone)
 {
     nsresult rv = NS_OK;
     if (mLock == nsnull) {
@@ -283,11 +278,11 @@ nsFileTransport::Init(nsFileTransportService *aService, nsIStreamIO* io)
             return NS_ERROR_OUT_OF_MEMORY;
     }
     mStreamIO = io;
-    nsXPIDLCString name;
-    rv = mStreamIO->GetName(getter_Copies(name));
-    mStreamName = NS_STATIC_CAST(const char*, name);
+    rv = mStreamIO->GetName(mStreamName);
     NS_ASSERTION(NS_SUCCEEDED(rv), "GetName failed");
 
+    mCloseStreamWhenDone = closeStreamWhenDone;
+
     NS_ADDREF(mService = aService);
     PR_AtomicIncrement(&mService->mTotalTransports);
 
@@ -307,11 +302,6 @@ nsFileTransport::~nsFileTransport()
         PR_DestroyLock(mLock);
         mLock = nsnull;
     }
-    if (mContentType) {
-        nsCRT::free(mContentType);
-        mContentType = nsnull;
-    }
-
     NS_IF_RELEASE(mService);
 }
 
@@ -338,10 +328,10 @@ nsFileTransport::Create(nsISupports* aOuter, const nsIID& aIID, void* *aResult)
 ////////////////////////////////////////////////////////////////////////////////
 
 NS_IMETHODIMP
-nsFileTransport::GetName(PRUnichar* *result)
+nsFileTransport::GetName(nsACString &result)
 {
-    *result = ToNewUnicode(mStreamName);
-    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    result = mStreamName;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -519,7 +509,8 @@ nsFileTransport::AsyncRead(nsIStreamListener *aListener,
     if (mXferState != CLOSED)
         return NS_ERROR_IN_PROGRESS;
 
-    NS_ASSERTION(aListener, "need to supply an nsIStreamListener");
+    NS_ENSURE_TRUE(aListener, NS_ERROR_INVALID_ARG);
+
     rv = NS_NewStreamListenerProxy(getter_AddRefs(mListener),
                                    aListener, nsnull,
                                    mBufferSegmentSize,
@@ -531,6 +522,10 @@ nsFileTransport::AsyncRead(nsIStreamListener *aListener,
     mTransferAmount = aTransferCount;
     mXferState = OPEN_FOR_READ;
 
+    nsIEventQueueService* eqService = mService->GetCachedEventQueueService(); 
+    eqService->GetSpecialEventQueue(nsIEventQueueService::CURRENT_THREAD_EVENT_QUEUE, getter_AddRefs(mEventQ));
+    NS_ASSERTION(mEventQ, "No Event Queue on calling thread");
+
     LOG(("nsFileTransport: AsyncRead [this=%x %s] mOffset=%d mTransferAmount=%d\n",
         this, mStreamName.get(), mOffset, mTransferAmount));
 
@@ -561,7 +556,8 @@ nsFileTransport::AsyncWrite(nsIStreamProvider *aProvider,
     if (mXferState != CLOSED)
         return NS_ERROR_IN_PROGRESS;
 
-    NS_ASSERTION(aProvider, "need to supply an nsIStreamProvider");
+    NS_ENSURE_TRUE(aProvider, NS_ERROR_INVALID_ARG);
+
     rv = NS_NewStreamProviderProxy(getter_AddRefs(mProvider),
                                    aProvider, nsnull,
                                    mBufferSegmentSize,
@@ -574,6 +570,10 @@ nsFileTransport::AsyncWrite(nsIStreamProvider *aProvider,
     mTransferAmount = aTransferCount;
     mXferState = OPEN_FOR_WRITE;
 
+    nsIEventQueueService* eqService = mService->GetCachedEventQueueService(); 
+    eqService->GetSpecialEventQueue(nsIEventQueueService::CURRENT_THREAD_EVENT_QUEUE, getter_AddRefs(mEventQ));
+    NS_ASSERTION(mEventQ, "No Event Queue on calling thread");
+
     LOG(("nsFileTransport: AsyncWrite [this=%x %s] mOffset=%d mTransferAmount=%d\n",
         this, mStreamName.get(), mOffset, mTransferAmount));
 
@@ -606,8 +606,10 @@ nsFileTransport::Run(void)
 #endif
 #endif
 
-    if (mRunState == SUSPENDED && NS_FAILED(mCancelStatus))
+    if (mRunState == SUSPENDED && NS_FAILED(mCancelStatus)) {
         mRunState = CANCELED;
+        mService->RemoveSuspendedTransport(this);
+    }
 
     while (mXferState != CLOSED && mRunState != SUSPENDED) {
         //
@@ -667,7 +669,9 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
 
     switch (mXferState) {
       case OPEN_FOR_READ: { 
-        mStatus = mStreamIO->Open(&mContentType, &mTotalAmount);
+        mStatus = mStreamIO->Open();
+        if (NS_SUCCEEDED(mStatus))
+            mStreamIO->GetContentLength(&mTotalAmount);
         LOG(("nsFileTransport: OPEN_FOR_READ [this=%x %s] status=%x\n", this, mStreamName.get(), mStatus));
         if (mListener) {
             nsresult rv = mListener->OnStartRequest(this, mContext);  // always send the start notification
@@ -746,10 +750,14 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
 
         // Give the listener a chance to read at most transferAmt bytes from
         // the source input stream.
-        nsresult status = mListener->OnDataAvailable(this, mContext,
-                                                     mSourceWrapper,
-                                                     mOffset, transferAmt);
-
+        nsresult status = NS_ERROR_NOT_INITIALIZED;
+        if (mListener)
+            status = mListener->OnDataAvailable(this, 
+                                                mContext,
+                                                mSourceWrapper,
+                                                mOffset, 
+                                                transferAmt);
+        
         if (NS_SUCCEEDED(status)) {
             // Find out what was read.
             total = mSourceWrapper->GetBytesRead() - offset;
@@ -832,32 +840,46 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
         // is reusing the stream.
         mXferState = CLOSING;
         DoClose();
-        nsCOMPtr <nsISupports> saveContext = mContext;
-        nsCOMPtr <nsIStreamListener> saveListener = mListener;
-        mListener = nsnull;
-        mContext = nsnull;
 
         // close the data source
         NS_IF_RELEASE(mSourceWrapper);
         mSourceWrapper = nsnull;
 
-        if (saveListener) {
-            saveListener->OnStopRequest(this, saveContext, mStatus);
-            saveListener = 0;
-        }
         if (progressSink) {
-            progressSink->OnStatus(this, saveContext, 
-                                    NS_NET_STATUS_READ_FROM, 
-                                    NS_ConvertASCIItoUCS2(mStreamName).get());
+            progressSink->OnStatus(this, 
+                                   mContext, 
+                                   NS_NET_STATUS_READ_FROM, 
+                                   NS_ConvertASCIItoUCS2(mStreamName).get());
         }
 
+        // see http://bugzilla.mozilla.org/show_bug.cgi?id=139556#c64
+        // for the reason behind this evil reference counting.
+        nsISupports* doomed = mContext.get();
+        NS_IF_ADDREF(doomed);
+        mContext = 0;
+        if (mListener) {
+            nsCOMPtr<nsIStreamListener> listener  = do_QueryInterface(mListener);
+            mListener = 0;
+            listener->OnStopRequest(this, doomed, mStatus);
+        }
+
+        // if we have a context, we have to ensure that it is released on the 
+        // proper thread.
+        if (doomed) {
+            if (mEventQ) 
+                NS_ProxyRelease(mEventQ, doomed);
+            else
+                NS_IF_RELEASE(doomed);
+        }
         break;
       }
 
       case OPEN_FOR_WRITE: {
         LOG(("nsFileTransport: OPEN_FOR_WRITE [this=%x %s]\n",
             this, mStreamName.get()));
-        mStatus = mStreamIO->Open(&mContentType, &mTotalAmount);
+        mStatus = mStreamIO->Open();
+        if (NS_SUCCEEDED(mStatus)) // XXX why do we care about this when writing?
+            mStatus = mStreamIO->GetContentLength(&mTotalAmount);
         
         if (mStatus == NS_ERROR_FILE_NOT_FOUND)
             mStatus = NS_OK;
@@ -929,12 +951,16 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
         if (mTransferAmount >= 0)
             transferAmt = PR_MIN(transferAmt, (PRUint32)mTransferAmount);
 
-        PRUint32 total, offset = mSinkWrapper->GetBytesWritten();
+        PRUint32 total = 0, offset = mSinkWrapper->GetBytesWritten();
 
         // Ask the provider for data
-        nsresult status = mProvider->OnDataWritable(this, mContext,
-                                                    mSinkWrapper,
-                                                    mOffset, transferAmt);
+        nsresult status = NS_ERROR_NOT_INITIALIZED;
+        if (mProvider)
+            status = mProvider->OnDataWritable(this, 
+                                               mContext,
+                                               mSinkWrapper,
+                                               mOffset, 
+                                               transferAmt);
 
         if (NS_SUCCEEDED(status)) {
             // Find out what was written.
@@ -1007,16 +1033,30 @@ nsFileTransport::Process(nsIProgressEventSink *progressSink)
         NS_IF_RELEASE(mSinkWrapper);
         mSinkWrapper = nsnull;
 
-        if (mProvider) {
-            mProvider->OnStopRequest(this, mContext, mStatus);
-            mProvider = 0;
-        }
         if (progressSink)
             progressSink->OnStatus(this, mContext,
                                    NS_NET_STATUS_WROTE_TO, 
                                    NS_ConvertASCIItoUCS2(mStreamName).get());
-        mContext = 0;
 
+        // see http://bugzilla.mozilla.org/show_bug.cgi?id=139556#c64
+        // for the reason behind this evil reference counting.
+        nsISupports* doomed = mContext.get();
+        NS_ADDREF(doomed);
+        mContext = 0;
+        if (mProvider) {
+            nsCOMPtr <nsIStreamProvider> provider = do_QueryInterface(mProvider);
+            mProvider = 0;
+            provider->OnStopRequest(this, doomed, mStatus);
+        }
+
+        // if we have a context, we have to ensure that it is released on the
+        // proper thread.
+        if (doomed) {
+            if (mEventQ)
+                NS_ProxyRelease(mEventQ, doomed);
+            else
+                NS_RELEASE(doomed);
+        }
         mXferState = CLOSING;
         break;
       }
