@@ -55,11 +55,17 @@
 #include "nsIProxyObjectManager.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
+#include "nsPrintfCString.h"
 #include "nsNetUtil.h"
 #include "nsISSLSocketControl.h"
 #include "nsITransportSecurityInfo.h"
 #include "nsMemory.h"
 #include "nsIProxyInfo.h"
+#include "nsProxyRelease.h"
+
+#if defined(XP_WIN)
+#include "nsNativeConnectionHelper.h"
+#endif
 
 #if defined(PR_LOGGING)
 static PRLogModuleInfo *gSocketTransportLog = nsnull;
@@ -70,6 +76,24 @@ static PRLogModuleInfo *gSocketTransportLog = nsnull;
 static NS_DEFINE_CID(kSocketProviderService, NS_SOCKETPROVIDERSERVICE_CID);
 static NS_DEFINE_CID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
 
+static nsresult
+ErrorAccordingToNSPR()
+{
+    nsresult rv = NS_ErrorAccordingToNSPR();
+    if (rv == NS_ERROR_FAILURE) {
+        LOG(("PR_GetError() returned %d\n", PR_GetError()));
+        // maybe this really maps to a more specific necko error
+        switch (PR_GetError()) {
+        case PR_CONNECT_ABORTED_ERROR:
+        case PR_CONNECT_RESET_ERROR:
+            LOG(("mapping to NS_ERROR_NET_RESET\n"));
+            rv = NS_ERROR_NET_RESET;
+            break;
+        }
+    }
+    return rv;
+}
+
 //
 //----------------------------------------------------------------------------
 // nsSocketTransport
@@ -143,6 +167,7 @@ nsSocketTransport::nsSocketTransport():
     mHostName(nsnull),
     mPort(-1),
     mMonitor(nsnull),
+    mNetAddress(nsnull),
     mOperation(eSocketOperation_None),
     mProxyPort(-1),
     mProxyHost(nsnull),
@@ -182,12 +207,6 @@ nsSocketTransport::nsSocketTransport():
     SetReadType (eSocketRead_None);
     SetWriteType(eSocketWrite_None);
     
-    //
-    // Set up Internet defaults...
-    //
-    memset(&mNetAddress, 0, sizeof(mNetAddress));
-    PR_SetNetAddr(PR_IpAddrAny, PR_AF_INET6, 0, &mNetAddress);
-    
     //
     // Initialize the global connect timeout value if necessary...
     //
@@ -218,7 +237,7 @@ nsSocketTransport::~nsSocketTransport()
     }
     
     if (mService) {
-        PR_AtomicDecrement(&mService->mTotalTransports);
+        mService->OnTransportDestroyed();
         NS_RELEASE(mService);
     }
     
@@ -355,7 +374,7 @@ nsresult nsSocketTransport::Init(nsSocketTransportService* aService,
     
     // Update the active time for timeout purposes...
     mLastActiveTime = PR_IntervalNow();
-    PR_AtomicIncrement(&mService->mTotalTransports);
+    mService->OnTransportCreated();
     
     LOG(("nsSocketTransport: Initializing [%s:%d %x].  rv = %x",
         mHostName, mPort, this, rv));
@@ -414,6 +433,14 @@ nsresult nsSocketTransport::Process(PRInt16 aSelectFlags)
         done = PR_TRUE; // nothing to process
 
     while (!done) {
+        //
+        // Check for failure on any socket requests
+        //
+        if (mReadRequest && mReadRequest->HasFailed())
+            mReadRequest->GetStatus(&mStatus);
+        else if (mWriteRequest && mWriteRequest->HasFailed())
+            mWriteRequest->GetStatus(&mStatus);
+
         //
         // If an error has occurred then move into the error state...
         //
@@ -437,8 +464,10 @@ nsresult nsSocketTransport::Process(PRInt16 aSelectFlags)
             //
             // A connection has been established with the server
             //
-            PR_AtomicIncrement(&mService->mConnectedTransports);
-            mWasConnected = PR_TRUE;
+            if (!mWasConnected) {
+                mService->OnTransportConnected(GetSocketHost(), mNetAddress);
+                mWasConnected = PR_TRUE;
+            }
 
             // Initialize select flags
             mSelectFlags = PR_POLL_EXCEPT;
@@ -495,17 +524,29 @@ nsresult nsSocketTransport::Process(PRInt16 aSelectFlags)
             // Send status message
             // only send a status if doResolveHost is going to do some
             // resolution
-            if (mStatus != NS_OK)
+            if (mStatus == NS_BASE_STREAM_WOULD_BLOCK)
                 OnStatus_Locked(NS_NET_STATUS_RESOLVING_HOST);
             break;
 
         case eSocketState_WaitConnect:
             LOG(("nsSocketTransport: Transport [host=%s:%d this=%x] is in WaitConnect state.\n",
                 mHostName, mPort, this));
-            mStatus = doConnection(aSelectFlags);
 
             // Send status message
             OnStatus_Locked(NS_NET_STATUS_CONNECTING_TO);
+
+            mStatus = doConnection(aSelectFlags);
+
+            // on connection failure, reuse next address if one exists
+            if (mStatus == NS_ERROR_CONNECTION_REFUSED) {
+                LOG(("connection failed [this=%x error=%x]\n", this, mStatus));
+
+                // Try again? 
+                if (OnConnectionFailed(PR_TRUE)) { 
+                    done = PR_TRUE;
+                    continue;
+                }
+            }
             break;
 
         case eSocketState_WaitReadWrite:
@@ -519,6 +560,25 @@ nsresult nsSocketTransport::Process(PRInt16 aSelectFlags)
             LOG(("nsSocketTransport: Transport [host=%s:%d this=%x] is in Timeout state.\n",
                 mHostName, mPort, this));
             mStatus = NS_ERROR_NET_TIMEOUT;
+
+            // on timeout, reuse next address if one exists ... do this only
+            // if we haven't already fired OnStartRequest.
+            if (mReadRequest || mWriteRequest) {
+                PRBool firedOnStart = PR_TRUE; // initial value doesn't matter
+                if (mReadRequest)
+                    firedOnStart = mReadRequest->IsInitialized();
+                if (!firedOnStart && mWriteRequest)
+                    firedOnStart = mWriteRequest->IsInitialized();
+
+                // Try again? 
+                if (!firedOnStart && OnConnectionFailed(PR_TRUE)) {
+                    // a little bit of hackery here so we'll end up in the
+                    // WaitConnect state...
+                    mCurrentState = eSocketState_WaitConnect;
+                    done = PR_TRUE;
+                    continue;
+                }
+            }
             break;
 
         default:
@@ -567,6 +627,68 @@ nsSocketTransport::Cancel(nsresult status)
     return NS_OK;
 }
 
+// Try next address if param says to. Otherwise, try to use autodial. If a 
+// connection is made, we try the network again. Otherwise, we don't.
+// Blocks until connection is made or fails.
+// Returns true to cause connection failures to try again.
+//
+PRBool
+nsSocketTransport::OnConnectionFailed(PRBool tryNextAddress)
+{
+    LOG(("nsSocketTransport Entering OnConnectionFailed()"));
+
+    PRBool tryAgain = PR_FALSE;
+
+    // If required, retry using next address.
+    if (tryNextAddress) {
+
+        PRNetAddr* nextNetAddress = mNetAddrList.GetNext(mNetAddress);
+    
+        if (nextNetAddress) {
+            mNetAddress = nextNetAddress;
+
+#if defined(PR_LOGGING)
+            char buf[64];
+            PR_NetAddrToString(mNetAddress, buf, sizeof(buf));
+            LOG(("  ...trying next address: %s\n", buf));
+#endif
+            tryAgain = PR_TRUE;
+        }
+    }
+
+#if defined(XP_WIN)
+
+    // If not trying next address, try to make a connection using dialup. 
+    // Retry if that connection is made.
+    if (!tryAgain && mService->mAutodialEnabled) {
+
+        tryAgain = nsNativeConnectionHelper::OnConnectionFailed(GetSocketHost());
+    }
+
+#endif
+
+    // Prepare to try again.
+    if (tryAgain)
+    {
+        if (mSocketFD)
+            PR_Close(mSocketFD);
+        mSocketFD = nsnull;
+
+        // mask error status so we'll return to this state
+        mStatus = NS_OK;
+
+        mService->AddToWorkQ(this);
+    }
+    else
+    {
+        if (tryNextAddress)
+            mNetAddress = nsnull;
+    }
+
+    LOG(("nsSocketTransport Leaving OnConnectionFailed() tryAgain = %d",tryAgain));
+    return tryAgain;
+}
+
 void
 nsSocketTransport::CompleteAsyncRead()
 {
@@ -633,64 +755,76 @@ nsresult nsSocketTransport::doResolveHost(void)
 
     NS_ASSERTION(eSocketState_WaitDNS == mCurrentState, "Wrong state.");
 
-    LOG(("nsSocketTransport: Entering doResolveHost() [host=%s:%d this=%x].\n", 
+    LOG(("nsSocketTransport: Entering doResolveHost() [host=%s:%d %x].\n", 
         mHostName, mPort, this));
 
+    // 
+    // The hostname is being resolved...
+    //
+    if (mDNSRequest)
+        return NS_BASE_STREAM_WOULD_BLOCK;
+
     //
     // The hostname has not been resolved yet...
     //
-    if (PR_IsNetAddrType(&mNetAddress, PR_IpAddrAny)) {
+    if (mNetAddress == nsnull) {
+        const char *host = GetSocketHost();
         //
-        // Initialize the port used for the connection...
+        // Check the socket transport service's hostname database first.
         //
-        // XXX: The list of ports must be restricted - see net_bad_ports_table[] in 
-        //      mozilla/network/main/mkconect.c
-        //
-        mNetAddress.ipv6.port = PR_htons(((mProxyPort != -1 && !mProxyTransparent) ? mProxyPort : mPort));
-
-        PR_ExitMonitor(mMonitor);
-
-        nsIDNSService* pDNSService = mService->GetCachedDNSService();
-        if (!pDNSService) {
-            return NS_ERROR_UNEXPECTED;
+        PRIPv6Addr addr;
+        if (mService->LookupHost(host, &addr)) {
+            // found address!
+            mNetAddrList.Init(1);
+            mNetAddress = mNetAddrList.GetNext(nsnull);
+            PR_SetNetAddr(PR_IpAddrAny, PR_AF_INET6, GetSocketPort(), mNetAddress);
+            memcpy(&mNetAddress->ipv6.ip, &addr, sizeof(addr));
+#ifdef PR_LOGGING
+            char buf[128];
+            PR_NetAddrToString(mNetAddress, buf, sizeof(buf));
+            LOG((" -> using cached ip address [%s]\n", buf));
+#endif
         }
+        else {
+            PR_ExitMonitor(mMonitor);
 
-        //
-        // Give up the SocketTransport lock.  This allows the DNS thread to call the
-        // nsIDNSListener notifications without blocking...
-        //
-        rv = pDNSService->Lookup((mProxyHost && !mProxyTransparent) ? mProxyHost : mHostName, 
-                                 this, 
-                                 nsnull, 
-                                 getter_AddRefs(mDNSRequest));
-        //
-        // Aquire the SocketTransport lock again...
-        //
-        PR_EnterMonitor(mMonitor);
+            nsIDNSService* pDNSService = mService->GetCachedDNSService();
+            if (!pDNSService) {
+                return NS_ERROR_UNEXPECTED;
+            }
 
-        if (NS_SUCCEEDED(rv)) {
             //
-            // The DNS lookup has finished...  It has either failed or succeeded.
+            // Give up the SocketTransport lock.  This allows the DNS thread to call the
+            // nsIDNSListener notifications without blocking...
             //
-            if (NS_FAILED(mStatus) || !PR_IsNetAddrType(&mNetAddress, PR_IpAddrAny)) {
-                mDNSRequest = 0;
-                rv = mStatus;
-            } 
+            rv = pDNSService->Lookup(host, this, nsnull, getter_AddRefs(mDNSRequest));
             //
-            // The DNS lookup is being processed...  Mark the transport as waiting
-            // until the result is available...
+            // Aquire the SocketTransport lock again...
             //
-            else {
-                SetFlag(eSocketDNS_Wait);
-                rv = NS_BASE_STREAM_WOULD_BLOCK;
+            PR_EnterMonitor(mMonitor);
+
+            if (NS_SUCCEEDED(rv)) {
+                //
+                // The DNS lookup has finished...  It has either failed or succeeded.
+                //
+                if (NS_FAILED(mStatus) || mNetAddress) {
+                    mDNSRequest = 0;
+                    rv = mStatus;
+                } 
+                //
+                // The DNS lookup is being processed...  Mark the transport as waiting
+                // until the result is available...
+                //
+                else {
+                    SetFlag(eSocketDNS_Wait);
+                    rv = NS_BASE_STREAM_WOULD_BLOCK;
+                }
             }
         }
     }
 
-    LOG(("nsSocketTransport: Leaving doResolveHost() [%s:%d %x].\t"
-        "rv = %x.\n\n",
+    LOG(("nsSocketTransport: Leaving doResolveHost() [%s:%d %x] rv = %x\n",
         mHostName, mPort, this, rv));
-
     return rv;
 }
 
@@ -781,14 +915,6 @@ nsresult nsSocketTransport::doConnection(PRInt16 aSelectFlags)
                     if (secCtrl)
                         secCtrl->SetNotificationCallbacks(mNotificationCallbacks);
                 }
-                else if (nsCRT::strcmp(mSocketTypes[type], "ssl-forcehandshake") == 0) {
-                    mSecurityInfo = socketInfo;
-                    nsCOMPtr<nsISSLSocketControl> secCtrl(do_QueryInterface(mSecurityInfo));
-                    if (secCtrl) {
-                        secCtrl->SetForceHandshake(PR_TRUE);
-                        secCtrl->SetNotificationCallbacks(mNotificationCallbacks);
-                    }
-                }
                 else if ((nsCRT::strcmp(mSocketTypes[type], "socks") == 0) 
                          || (nsCRT::strcmp(mSocketTypes[type], "socks4") == 0)) {
                     // since socks is transparent, any layers above
@@ -832,7 +958,7 @@ nsresult nsSocketTransport::doConnection(PRInt16 aSelectFlags)
         //    This is only done the first time doConnection(...) is called.
         //
         if (NS_SUCCEEDED(rv)) {
-            status = PR_Connect(mSocketFD, &mNetAddress, gConnectTimeout);
+            status = PR_Connect(mSocketFD, mNetAddress, gConnectTimeout);
             if (PR_SUCCESS != status) {
                 PRErrorCode code = PR_GetError();
                 //
@@ -932,14 +1058,17 @@ nsSocketTransport::doBlockingConnection()
     //
     // The hostname has not been resolved yet...
     //
-    if (PR_IsNetAddrType(&mNetAddress, PR_IpAddrAny)) {
+    if (mNetAddress == nsnull) {
         nsIDNSService* pDNSService = mService->GetCachedDNSService();
         if (!pDNSService) {
             return NS_ERROR_UNEXPECTED;
         }
 
         nsXPIDLCString result;
-        const char *host = (mProxyHost && !mProxyTransparent) ? mProxyHost : mHostName;
+        const char *host = GetSocketHost();
+
+        // XXX DNS service does not provide a synchronous resolver that returns
+        // the entire nsHostEnt struct.
         rv = pDNSService->Resolve(host, getter_Copies(result));
         if (NS_FAILED(rv)) return rv;
 
@@ -953,16 +1082,18 @@ nsSocketTransport::doBlockingConnection()
             return NS_ERROR_FAILURE;
         }
 
-        if (addr.raw.family == PR_AF_INET)
-            PR_ConvertIPv4AddrToIPv6(addr.inet.ip, &mNetAddress.ipv6.ip);
-        else
-            memcpy(&mNetAddress.ipv6.ip, &addr.ipv6.ip, sizeof(mNetAddress.ipv6.ip));
+        mNetAddrList.Init(1);
+        mNetAddress = mNetAddrList.GetNext(nsnull);
 
-        mNetAddress.ipv6.port
-            = PR_htons(((mProxyPort != -1 && !mProxyTransparent) ? mProxyPort : mPort));
+        PR_SetNetAddr(PR_IpAddrAny, PR_AF_INET6, GetSocketPort(), mNetAddress);
+
+        if (addr.raw.family == PR_AF_INET)
+            PR_ConvertIPv4AddrToIPv6(addr.inet.ip, &mNetAddress->ipv6.ip);
+        else
+            memcpy(&mNetAddress->ipv6.ip, &addr.ipv6.ip, sizeof(mNetAddress->ipv6.ip));
 
         LOG(("address { family=%hu, port=%hu }\n",
-            mNetAddress.ipv6.family, PR_ntohs(mNetAddress.ipv6.port)));
+            mNetAddress->ipv6.family, PR_ntohs(mNetAddress->ipv6.port)));
     }
 
     //
@@ -1036,7 +1167,6 @@ nsSocketTransport::doReadWrite(PRInt16 aSelectFlags)
     if (mReadRequest) {
         if (mReadRequest->IsCanceled() || (mBytesExpected == 0)) {
             LOG(("nsSocketTransport: [this=%x] completing read request due to cancelation\n", this));
-            mSelectFlags &= ~PR_POLL_READ;
             mReadRequest->GetStatus(&readStatus);
             CompleteAsyncRead();
             if (NS_FAILED(readStatus))
@@ -1078,7 +1208,6 @@ nsSocketTransport::doReadWrite(PRInt16 aSelectFlags)
     if (mWriteRequest) {
         if (mWriteRequest->IsCanceled()) {
             LOG(("nsSocketTransport: [this=%x] completing write request due to cancelation\n", this));
-            mSelectFlags &= ~PR_POLL_WRITE;
             mWriteRequest->GetStatus(&writeStatus);
             CompleteAsyncWrite();
             if (NS_FAILED(writeStatus))
@@ -1163,7 +1292,7 @@ nsresult nsSocketTransport::CloseConnection()
 
     if (mWasConnected) {
         if (mService)
-            PR_AtomicDecrement(&mService->mConnectedTransports);
+            mService->OnTransportClosed();
         mWasConnected = PR_FALSE;
     }
     
@@ -1362,14 +1491,12 @@ nsSocketTransport::SetNotificationCallbacks(nsIInterfaceRequestor *aCallbacks,
 //
 
 nsresult
-nsSocketTransport::GetName(PRUnichar **result)
+nsSocketTransport::GetName(nsACString &result)
 {
-    nsString name;
-    name.AppendWithConversion(mHostName);
-    name.Append(NS_LITERAL_STRING(":"));
-    name.AppendInt(mPort);
-    *result = ToNewUnicode(name);
-    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    result = nsDependentCString(mHostName)
+           + NS_LITERAL_CSTRING(":")
+           + nsPrintfCString("%d", mPort);
+    return NS_OK;
 }
 
 nsresult
@@ -1485,23 +1612,44 @@ nsSocketTransport::OnFound(nsISupports *aContext,
     nsAutoMonitor mon(mMonitor);
     nsresult rv = NS_OK;
 
-    if (aHostEnt->hostEnt.h_addr_list && aHostEnt->hostEnt.h_addr_list[0]) {
-        if (aHostEnt->hostEnt.h_addrtype == PR_AF_INET6)
-            memcpy(&mNetAddress.ipv6.ip, aHostEnt->hostEnt.h_addr_list[0], sizeof(mNetAddress.ipv6.ip));
-        else
-            PR_ConvertIPv4AddrToIPv6(*(PRUint32*)aHostEnt->hostEnt.h_addr_list[0], &mNetAddress.ipv6.ip);
+    char **addrList = aHostEnt->hostEnt.h_addr_list;
+    
+    if (addrList && addrList[0]) {
+        PRUint32 len = 0;
+
+        LOG(("nsSocketTransport::OnFound [%s:%d %x] lookup succeeded [FQDN=%s]\n",
+            mHostName, mPort, this, aHostEnt->hostEnt.h_name));
+
+        // determine the number of address in the list
+        for (; *addrList; ++addrList)
+            ++len;
+        addrList -= len;
+
+        // allocate space for the addresses
+        mNetAddrList.Init(len);
+
+        // populate the address list
+        PRNetAddr *addr = nsnull;
+        while ((addr = mNetAddrList.GetNext(addr)) != nsnull) {
+            PR_SetNetAddr(PR_IpAddrAny, PR_AF_INET6, GetSocketPort(), addr);
+            if (aHostEnt->hostEnt.h_addrtype == PR_AF_INET6)
+                memcpy(&addr->ipv6.ip, *addrList, sizeof(addr->ipv6.ip));
+            else
+                PR_ConvertIPv4AddrToIPv6(*(PRUint32 *)(*addrList), &addr->ipv6.ip);
+            ++addrList;
 #if defined(PR_LOGGING)
-        char addrbuf[50];
-        PR_NetAddrToString(&mNetAddress, addrbuf, sizeof(addrbuf));
-        LOG(("nsSocketTransport: OnFound(...) [%s:%d %x]."
-            "  DNS lookup succeeded => %s (%s)\n",
-            mHostName, mPort, this,
-            aHostEnt->hostEnt.h_name,
-            addrbuf));
+            char buf[50];
+            PR_NetAddrToString(addr, buf, sizeof(buf));
+            LOG(("  => %s\n", buf));
 #endif
-    } else {
-        // XXX: What should happen here?  The GetHostByName(...) succeeded but 
-        //      there are *no* A records...
+        }
+
+        // start with first address in list
+        mNetAddress = mNetAddrList.GetNext(nsnull);
+    }
+    else {
+        // XXX: What should happen here?  The DNS lookup succeeded, but there
+        //      are no address records...
         rv = NS_ERROR_FAILURE;
 
         LOG(("nsSocketTransport: OnFound(...) [%s:%d %x]."
@@ -1528,11 +1676,21 @@ nsSocketTransport::OnStopLookup(nsISupports *aContext,
     // Release our reference to the DNS Request...
     mDNSRequest = 0;
 
-    // If the lookup failed, set the status...
-    if (NS_FAILED(aStatus))
-        mStatus = aStatus;
-    else if (PR_IsNetAddrType(&mNetAddress, PR_IpAddrAny))
+    // If the lookup failed...
+    if (NS_FAILED(aStatus)) {
+        
+        // Retry? 
+        // Don't want to set the tryNextAddress param because DNS lookup has
+        // just failed so hostname has not been resolved yet.
+        if (aStatus != NS_BASE_STREAM_WOULD_BLOCK && OnConnectionFailed(PR_FALSE))
+            mStatus = NS_OK;
+        else
+            mStatus = aStatus;
+    }
+    else if (mNetAddress == nsnull) {
         mStatus = NS_ERROR_ABORT;
+    }
+
 
     // Start processing the transport again - if necessary...
     if (GetFlag(eSocketDNS_Wait)) {
@@ -1571,6 +1729,8 @@ nsSocketTransport::AsyncRead(nsIStreamListener* aListener,
     if (GetReadType() != eSocketRead_None)
         rv = NS_ERROR_IN_PROGRESS;
     
+    NS_ENSURE_TRUE(aListener, NS_ERROR_INVALID_ARG);
+
     nsCOMPtr<nsIStreamListener> listener;
     nsCOMPtr<nsIRequestObserver> observer;
 
@@ -1649,6 +1809,8 @@ nsSocketTransport::AsyncWrite(nsIStreamProvider* aProvider,
     if (GetWriteType() != eSocketWrite_None)
         rv = NS_ERROR_IN_PROGRESS;
 
+    NS_ENSURE_TRUE(aProvider, NS_ERROR_INVALID_ARG);
+
     nsCOMPtr<nsIStreamProvider> provider;
     nsCOMPtr<nsIRequestObserver> observer;
     
@@ -1852,7 +2014,7 @@ nsSocketTransport::GetIPStr(PRUint32 aLen, char **_retval)
     *_retval = (char*)nsMemory::Alloc(aLen);
     if (!*_retval) return NS_ERROR_FAILURE;
 
-    PRStatus status = PR_NetAddrToString(&mNetAddress, *_retval, aLen);
+    PRStatus status = PR_NetAddrToString(mNetAddress, *_retval, aLen);
 
     if (PR_FAILURE == status) {
         nsMemory::Free(*_retval);
@@ -1970,6 +2132,37 @@ nsSocketTransport::ClearSocketBS(nsSocketBS *blockingStream)
         CloseConnection();
 }
 
+//
+//----------------------------------------------------------------------------
+// nsSocketTransport::nsNetAddrList
+//----------------------------------------------------------------------------
+//
+
+nsresult nsSocketTransport::
+nsNetAddrList::Init(PRUint32 len)
+{
+    NS_ASSERTION(!mList, "already initialized");
+    mList = new PRNetAddr[len];
+    if (!mList)
+        return NS_ERROR_OUT_OF_MEMORY;
+    mLen = len;
+    return NS_OK;
+}
+
+PRNetAddr *nsSocketTransport::
+nsNetAddrList::GetNext(PRNetAddr *addr)
+{
+    if (!addr)
+        return mList;
+
+    PRUint32 offset = addr - mList;
+    NS_ASSERTION(offset < mLen, "invalid address");
+    if (offset + 1 < mLen)
+        return addr + 1;
+
+    return nsnull;
+}
+
 //
 //----------------------------------------------------------------------------
 // nsSocketBS
@@ -2143,7 +2336,7 @@ nsSocketBIS::Read(char *aBuf, PRUint32 aCount, PRUint32 *aBytesRead)
 tryRead:
     total = PR_Read(sock, aBuf, aCount);
     if (total < 0) {
-        rv = NS_ErrorAccordingToNSPR();
+        rv = ErrorAccordingToNSPR();
         if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
             //
             // Block until the socket is readable and then try again.
@@ -2156,7 +2349,7 @@ tryRead:
             LOG(("nsSocketBIS::Read [this=%x] Poll succeeded; looping back to PR_Read\n", this));
             goto tryRead;
         }
-        LOG(("nsSocketBIS::Read [this=%x] PR_Read failed  [error=%x]\n", this, PR_GetError()));
+        LOG(("nsSocketBIS::Read [this=%x] PR_Read failed [error=%d]\n", this, PR_GetError()));
         goto end;
     }
     *aBytesRead = (PRUint32) total;
@@ -2173,21 +2366,10 @@ nsSocketBIS::ReadSegments(nsWriteSegmentFun aWriter, void *aClosure,
 }
 
 NS_IMETHODIMP
-nsSocketBIS::GetNonBlocking(PRBool *aValue)
+nsSocketBIS::IsNonBlocking(PRBool *aValue)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketBIS::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketBIS::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
+    *aValue = PR_FALSE;
+    return NS_OK;
 }
 
 //
@@ -2238,7 +2420,7 @@ tryWrite:
     LOG(("nsSocketBOS PR_Write [this=%x] wrote %d of %d\n", this, total, aCount));
 
     if (written < 0) {
-        rv = NS_ErrorAccordingToNSPR();
+        rv = ErrorAccordingToNSPR();
         if (rv == NS_BASE_STREAM_WOULD_BLOCK) {
             //
             // Block until the socket is writable and then try again.
@@ -2251,7 +2433,7 @@ tryWrite:
             LOG(("nsSocketBOS::Write [this=%x] Poll succeeded; looping back to PR_Write\n", this));
             goto tryWrite;
         }
-        LOG(("nsSocketBOS::Write [this=%x] PR_Write failed [error=%x]\n", this, PR_GetError()));
+        LOG(("nsSocketBOS::Write [this=%x] PR_Write failed [error=%d]\n", this, PR_GetError()));
         goto end;
     }
     
@@ -2283,27 +2465,10 @@ nsSocketBOS::WriteSegments(nsReadSegmentFun aReader, void *aClosure,
 }
 
 NS_IMETHODIMP
-nsSocketBOS::GetNonBlocking(PRBool *aValue)
+nsSocketBOS::IsNonBlocking(PRBool *aValue)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketBOS::SetNonBlocking(PRBool aValue)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketBOS::GetObserver(nsIOutputStreamObserver **aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketBOS::SetObserver(nsIOutputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
+    *aValue = PR_FALSE;
+    return NS_OK;
 }
 
 //
@@ -2359,9 +2524,9 @@ nsSocketIS::Read(char *aBuf, PRUint32 aCount, PRUint32 *aBytesRead)
             rv = NS_BASE_STREAM_WOULD_BLOCK;
         }
         else {
-            LOG(("nsSocketIS: PR_Read() failed [error=%x, os_error=%x]\n",
+            LOG(("nsSocketIS: PR_Read() failed [error=%d, os_error=%d]\n",
                 mError, PR_GetOSError()));
-            rv = NS_ERROR_FAILURE;
+            rv = ErrorAccordingToNSPR();
         }
         *aBytesRead = 0;
     }
@@ -2381,27 +2546,13 @@ nsSocketIS::ReadSegments(nsWriteSegmentFun aWriter, void *aClosure,
 }
 
 NS_IMETHODIMP
-nsSocketIS::GetNonBlocking(PRBool *aValue)
+nsSocketIS::IsNonBlocking(PRBool *aValue)
 {
     NS_ENSURE_ARG_POINTER(aValue);
     *aValue = PR_TRUE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsSocketIS::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    NS_NOTREACHED("nsSocketIS::GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketIS::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    NS_NOTREACHED("nsSocketIS::SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 NS_IMETHODIMP
 nsSocketIS::Seek(PRInt32 whence, PRInt32 offset)
 {
@@ -2479,9 +2630,9 @@ nsSocketOS::Write(const char *aBuf, PRUint32 aCount, PRUint32 *aBytesWritten)
             LOG(("nsSocketTransport: PR_Write() failed with PR_WOULD_BLOCK_ERROR\n"));
             rv = NS_BASE_STREAM_WOULD_BLOCK;
         } else {
-            LOG(("nsSocketTransport: PR_Write() failed [error=%x, os_error=%x]\n",
+            LOG(("nsSocketTransport: PR_Write() failed [error=%d, os_error=%d]\n",
                 mError, PR_GetOSError()));
-            rv = NS_ERROR_FAILURE;
+            rv = ErrorAccordingToNSPR();
         }
         *aBytesWritten = 0;
     }
@@ -2507,34 +2658,13 @@ nsSocketOS::WriteSegments(nsReadSegmentFun aReader, void *aClosure,
 }
 
 NS_IMETHODIMP
-nsSocketOS::GetNonBlocking(PRBool *aValue)
+nsSocketOS::IsNonBlocking(PRBool *aValue)
 {
     NS_ENSURE_ARG_POINTER(aValue);
     *aValue = PR_TRUE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsSocketOS::SetNonBlocking(PRBool aValue)
-{
-    NS_NOTREACHED("nsSocketOS::SetNonBlocking");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketOS::GetObserver(nsIOutputStreamObserver **aObserver)
-{
-    NS_NOTREACHED("nsSocketOS::GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsSocketOS::SetObserver(nsIOutputStreamObserver *aObserver)
-{
-    NS_NOTREACHED("nsSocketOS::SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 NS_IMETHODIMP
 nsSocketOS::Seek(PRInt32 whence, PRInt32 offset)
 {
@@ -2589,6 +2719,12 @@ nsSocketRequest::SetTransport(nsSocketTransport *aTransport)
     //
     NS_IF_RELEASE(mTransport);
     NS_IF_ADDREF(mTransport = aTransport);
+
+    //
+    // Set the event queue
+    //
+    nsIEventQueueService* eqService = aTransport->mService->GetCachedEventQueueService(); 
+    eqService->GetSpecialEventQueue(nsIEventQueueService::CURRENT_THREAD_EVENT_QUEUE, getter_AddRefs(mEventQ));
 }
 
 nsresult
@@ -2609,15 +2745,28 @@ nsSocketRequest::OnStop()
             mObserver->OnStartRequest(this, mContext);
             mStartFired = PR_TRUE;
         }
+
         mObserver->OnStopRequest(this, mContext, mStatus);
         mObserver = 0;
+
+        if (mContext) {
+            if (mEventQ) {
+                nsISupports* doomed = mContext.get();
+                NS_ADDREF(doomed);
+                mContext = 0;
+                NS_ProxyRelease(mEventQ, doomed);
+            }
+            else {
+                mContext = 0;
+            }
+        }
         mStopFired = PR_TRUE;
     }
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsSocketRequest::GetName(PRUnichar **aResult)
+nsSocketRequest::GetName(nsACString &aResult)
 {
     return mTransport ?
         mTransport->GetName(aResult) : NS_ERROR_NOT_INITIALIZED;
