@@ -27,6 +27,7 @@
 #include "nsCRT.h"
 #include "prmem.h"
 #include "netCore.h"
+#include "nsAString.h"
 
 #define MAX_IO_CHUNK 8192 // maximum count reported per OnDataAvailable
 #define MAX_COUNT ((PRUint32) -1)
@@ -86,6 +87,7 @@ nsStorageTransport::nsStorageTransport()
     , mSegmentSize(DEFAULT_SEGMENT_SIZE)
     , mMaxSize(DEFAULT_BUFFER_SIZE)
     , mSegments(nsnull)
+    , mSegmentsLast(nsnull)
     , mWriteSegment(nsnull)
     , mWriteCursor(0)
 {
@@ -226,11 +228,9 @@ nsStorageTransport::AppendSegment(nsSegment *aSegment)
 {
     if (!mSegments)
         mSegments = aSegment;
-    else {
-        nsSegment *s = mSegments;
-        for (; s && s->next; s = s->next);
-        s->next = aSegment;
-    }
+    else
+        mSegmentsLast->next = aSegment;
+    mSegmentsLast = aSegment;
 }
 
 void
@@ -249,7 +249,7 @@ nsStorageTransport::TruncateTo(PRUint32 aOffset)
     if (aOffset < mWriteCursor) {
         if (aOffset == 0) {
             DeleteSegments(mSegments);
-            mSegments = nsnull;
+            mSegments = mSegmentsLast = nsnull;
             mWriteSegment = nsnull;
         }
         else {
@@ -265,6 +265,7 @@ nsStorageTransport::TruncateTo(PRUint32 aOffset)
                 DeleteSegments(s->next);
                 s->next = nsnull;
             }
+            mSegmentsLast = s; 
             mWriteSegment = s;
         }
     }
@@ -408,7 +409,6 @@ nsStorageTransport::nsReadRequest::nsReadRequest()
     , mTransferOffset(0)
     , mTransferCount(MAX_COUNT)
     , mStatus(NS_OK)
-    , mCanceled(PR_FALSE)
     , mOnStartFired(PR_FALSE)
     , mWaitingForWrite(PR_FALSE)
 {
@@ -459,14 +459,14 @@ nsStorageTransport::nsReadRequest::Process()
     mWaitingForWrite = PR_FALSE;
 
     if (!mOnStartFired) {
-        // no need to proxy this callback
-        (void) mListener->OnStartRequest(this, mListenerContext);
+        // must proxy since called from AsyncRead - bug 136216
+        (void) mListenerProxy->OnStartRequest(this, mListenerContext);
         mOnStartFired = PR_TRUE;
     }
 
     PRUint32 count = 0;
 
-    if (mCanceled) {
+    if (NS_FAILED(mStatus)) {
         // forcing the transfer count to zero indicates that we are done.
         mTransferCount = 0;
     }
@@ -497,12 +497,7 @@ nsStorageTransport::nsReadRequest::Process()
         mTransport->ReadRequestCompleted(this);
         
         // no need to proxy this callback
-        (void) mListener->OnStopRequest(this, mListenerContext, mStatus);
-
-        //OnStopRequest completed and listeners no longer needed. 
-        mListener=nsnull;
-        mListenerContext=nsnull;
-        mListenerProxy=nsnull;
+        (void) mListenerProxy->OnStopRequest(this, mListenerContext, mStatus);
     }
     else
         mWaitingForWrite = PR_TRUE;
@@ -526,10 +521,9 @@ nsStorageTransport::nsReadRequest::GetTransport(nsITransport **aTransport)
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetName(PRUnichar **aName)
+nsStorageTransport::nsReadRequest::GetName(nsACString &aName)
 {
-    NS_ENSURE_ARG_POINTER(aName);
-    *aName = nsnull;
+    aName.Truncate();
     return NS_OK;
 }
 
@@ -552,7 +546,6 @@ nsStorageTransport::nsReadRequest::GetStatus(nsresult *aStatus)
 NS_IMETHODIMP
 nsStorageTransport::nsReadRequest::Cancel(nsresult aStatus)
 {
-    mCanceled = PR_TRUE;
     mStatus = aStatus;
     return NS_OK;
 }
@@ -599,8 +592,15 @@ NS_IMETHODIMP
 nsStorageTransport::nsReadRequest::OnStartRequest(nsIRequest *aRequest,
                                                   nsISupports *aContext)
 {
-    NS_NOTREACHED("nsStorageTransport::nsReadRequest::OnStartRequest");
-    return NS_ERROR_FAILURE;
+    nsresult rv;
+
+    NS_ASSERTION(mListener, "no listener");
+    rv = mListener->OnStartRequest(aRequest, aContext);
+
+    if (NS_FAILED(rv)) {
+        Cancel(rv);
+    }
+    return rv;
 }
 
 NS_IMETHODIMP
@@ -608,8 +608,27 @@ nsStorageTransport::nsReadRequest::OnStopRequest(nsIRequest *aRequest,
                                                  nsISupports *aContext,
                                                  nsresult aStatus)
 {
-    NS_NOTREACHED("nsStorageTransport::nsReadRequest::OnStopRequest");
-    return NS_ERROR_FAILURE;
+    // If the request was canceled, it is possible that the listener is null
+    // because OnStopRequest has already been called (from OnDataAvailable)
+    //
+    // If this is the case, them mStatus should be a failure code... and all
+    // is well.  Otherwise, something bad is going on...
+    //
+    if (!mListener) {
+        NS_ASSERTION(NS_FAILED(mStatus), "Null listener!!!");
+        return NS_OK;
+    }
+
+    // Clear mListener before calling OnStopRequest to prevent multiple, 
+    // re-enterent calls...
+    nsCOMPtr<nsIStreamListener> listener(mListener);
+
+    mListener = 0;
+    (void) listener->OnStopRequest(aRequest, aContext, aStatus);
+    // OnStopRequest completed, so listeners are no longer needed. 
+    mListenerContext = 0;
+    mListenerProxy = 0;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -622,13 +641,26 @@ nsStorageTransport::nsReadRequest::OnDataAvailable(nsIRequest *aRequest,
     nsresult rv = NS_OK;
     PRUint32 priorOffset = mTransferOffset;
 
+    if (NS_FAILED(mStatus)) {
+        // When an error occurs, it is important that OnStopRequest(...) is
+        // called.  
+        //
+        // Once OnStopRequest() has been called, mListener should be nsnull.
+        if (mListener) {
+            OnStopRequest(aRequest, aContext, mStatus);
+            NS_ASSERTION(mListener == nsnull, "The Listener was not detached!");
+        }
+        return mStatus;
+    }
+
     rv = mListener->OnDataAvailable(aRequest, aContext, aInput, aOffset, aCount);
     NS_ASSERTION(rv != NS_BASE_STREAM_WOULD_BLOCK, "not implemented");
 
-    if (NS_FAILED(rv)) return rv;
-
+    if (NS_FAILED(rv)) {
+        Cancel(rv);
+    }
     // avoid an infinite loop
-    if (priorOffset == mTransferOffset) {
+    else if (priorOffset == mTransferOffset) {
         NS_WARNING("nsIStreamListener::OnDataAvailable implementation did not "
                    "consume any data!");
         // end the read
@@ -711,27 +743,13 @@ nsStorageTransport::nsReadRequest::ReadSegments(nsWriteSegmentFun aWriter,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsReadRequest::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_FALSE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsReadRequest::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //----------------------------------------------------------------------------
 // nsStorageTransport::nsBlockingStream
 //----------------------------------------------------------------------------
@@ -791,27 +809,13 @@ nsStorageTransport::nsInputStream::ReadSegments(nsWriteSegmentFun aWriter,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsInputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsInputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_TRUE;
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStorageTransport::nsInputStream::GetObserver(nsIInputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsInputStream::SetObserver(nsIInputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //----------------------------------------------------------------------------
 // nsStorageTransport::nsOutputStream
 //----------------------------------------------------------------------------
@@ -901,29 +905,9 @@ nsStorageTransport::nsOutputStream::WriteSegments(nsReadSegmentFun aReader,
 }
 
 NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::GetNonBlocking(PRBool *aNonBlocking)
+nsStorageTransport::nsOutputStream::IsNonBlocking(PRBool *aNonBlocking)
 {
     NS_ENSURE_ARG_POINTER(aNonBlocking);
     *aNonBlocking = PR_FALSE;
     return NS_OK;
 }
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::SetNonBlocking(PRBool aNonBlocking)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::GetObserver(nsIOutputStreamObserver **aObserver)
-{
-    NS_ENSURE_ARG_POINTER(aObserver);
-    *aObserver = nsnull;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsStorageTransport::nsOutputStream::SetObserver(nsIOutputStreamObserver *aObserver)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
