@@ -42,6 +42,7 @@
 #include "nsIURI.h"
 #include "prtypes.h"
 #include "nsString.h"
+#include "netCore.h"
 
 //----------------------------------------------------------------------------
 
@@ -143,7 +144,7 @@ nsBaseURLParser::ParseURL(const char *spec, PRInt32 specLen,
         // spec = <scheme>:<authority>
         // spec = <scheme>:<path-no-slashes>
         //
-        if (!IsValidScheme(spec, colon - spec)) {
+        if (!IsValidScheme(spec, colon - spec) || (*(colon+1) == ':')) {
             NS_WARNING("malformed uri");
             return NS_ERROR_MALFORMED_URI;
         }
@@ -391,18 +392,21 @@ nsNoAuthURLParser::ParseAfterScheme(const char *spec, PRInt32 specLen,
         break;
     case 2:
         {
-            // looks like there is an authority section
-            const char *p = (const char *) memchr(spec + 2, '/', specLen - 2);
+            const char *p = nsnull;
+            if (specLen > 2) {
+                // looks like there is an authority section
 #if defined(XP_PC)
-            // if the authority looks like a drive number then we
-            // really want to treat it as part of the path
-            if (((p && (p - (spec + 2) == 2)) || (specLen == 4)) &&
-                (spec[3] == ':' || spec[3] == '|') &&
-                (nsCRT::IsAsciiAlpha(spec[2]))) {
-                pos = 1;
-                break;
-            }
+                // if the authority looks like a drive number then we
+                // really want to treat it as part of the path
+                if ((specLen > 3) && (spec[3] == ':' || spec[3] == '|') &&
+                    nsCRT::IsAsciiAlpha(spec[2]) &&
+                    ((specLen == 4) || (spec[4] == '/') || (spec[4] == '\\'))) {
+                    pos = 1;
+                    break;  
+                } 
 #endif
+                p = (const char *) memchr(spec + 2, '/', specLen - 2);
+            }
             if (p) {
                 SET_RESULT(auth, 2, p - (spec + 2));
                 SET_RESULT(path, p - spec, specLen - (p - spec));
@@ -468,29 +472,45 @@ nsAuthURLParser::ParseAuthority(const char *auth, PRInt32 authLen,
                                 PRUint32 *hostnamePos, PRInt32 *hostnameLen,
                                 PRInt32 *port)
 {
+    nsresult rv;
+
     NS_PRECONDITION(auth, "null pointer");
 
     if (authLen < 0)
         authLen = strlen(auth);
 
-    const char *p = (const char *) memchr(auth, '@', authLen);
-    if (p) {
+    if (authLen == 0) {
+        SET_RESULT(username, 0, -1);
+        SET_RESULT(password, 0, -1);
+        SET_RESULT(hostname, 0, 0);
+        if (port)
+            *port = -1;
+        return NS_OK;
+    }
+
+    // search backwards for @
+    const char *p = auth + authLen - 1;
+    for (; (*p != '@') && (p > auth); --p);
+    if ( *p == '@' ) {
         // auth = <user-info@server-info>
-        ParseUserInfo(auth, p - auth,
-                      usernamePos, usernameLen,
-                      passwordPos, passwordLen);
-        ParseServerInfo(p + 1, authLen - (p - auth + 1),
-                        hostnamePos, hostnameLen,
-                        port);
+        rv = ParseUserInfo(auth, p - auth,
+                           usernamePos, usernameLen,
+                           passwordPos, passwordLen);
+        if (NS_FAILED(rv)) return rv;
+        rv = ParseServerInfo(p + 1, authLen - (p - auth + 1),
+                             hostnamePos, hostnameLen,
+                             port);
+        if (NS_FAILED(rv)) return rv;
         OFFSET_RESULT(hostname, p + 1 - auth);
     }
     else {
         // auth = <server-info>
         SET_RESULT(username, 0, -1);
         SET_RESULT(password, 0, -1);
-        ParseServerInfo(auth, authLen,
-                        hostnamePos, hostnameLen,
-                        port);
+        rv = ParseServerInfo(auth, authLen,
+                             hostnamePos, hostnameLen,
+                             port);
+        if (NS_FAILED(rv)) return rv;
     }
     return NS_OK;
 }
@@ -529,18 +549,39 @@ nsAuthURLParser::ParseServerInfo(const char *serverinfo, PRInt32 serverinfoLen,
     if (serverinfoLen < 0)
         serverinfoLen = strlen(serverinfo);
 
-    // search backwards for a ':' but stop on ']' (IPv6 address literal
-    // delimiter)
-    const char *p = serverinfo + serverinfoLen - 1;
-    for (; (*p != ']') && (*p != ':') && (p > serverinfo); --p);
+    if (serverinfoLen == 0) {
+        SET_RESULT(hostname, 0, 0);
+        if (port)
+            *port = -1;
+        return NS_OK;
+    }
 
-    //const char *p = (const char *) memchr(serverinfo, ':', serverinfoLen);
-    if (*p == ':') {
+    // search backwards for a ':' but stop on ']' (IPv6 address literal
+    // delimiter).  check for illegal characters in the hostname.
+    const char *p = serverinfo + serverinfoLen - 1;
+    const char *colon = nsnull, *bracket = nsnull;
+    for (; p > serverinfo; --p) {
+        switch (*p) {
+            case ']':
+                bracket = p;
+                break;
+            case ':':
+                if (bracket == nsnull)
+                    colon = p;
+                break;
+            case ' ':
+                // hostname must not contain a space
+                NS_WARNING("malformed hostname");
+                return NS_ERROR_MALFORMED_URI;
+        }
+    }
+
+    if (colon) {
         // serverinfo = <hostname:port>
-        SET_RESULT(hostname, 0, p - serverinfo);
+        SET_RESULT(hostname, 0, colon - serverinfo);
         if (port) {
             // XXX unfortunately ToInteger is not defined for substrings
-            nsCAutoString buf(p+1, serverinfoLen - (p + 1 - serverinfo));
+            nsCAutoString buf(colon+1, serverinfoLen - (colon + 1 - serverinfo));
             PRInt32 err;
            *port = buf.ToInteger(&err);
             if (NS_FAILED(err))
