@@ -28,9 +28,11 @@
 #include "nsCacheEntry.h"
 #include "nsReadableUtils.h"
 #include "nsIOutputStream.h"
+#include "nsCRT.h"
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsCacheEntryDescriptor, nsICacheEntryDescriptor)
-
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsCacheEntryDescriptor,
+                              nsICacheEntryDescriptor,
+                              nsICacheEntryInfo)
 
 nsCacheEntryDescriptor::nsCacheEntryDescriptor(nsCacheEntry * entry,
                                                nsCacheAccessMode accessGranted)
@@ -264,6 +266,12 @@ nsCacheEntryDescriptor::SetStoragePolicy(nsCacheStoragePolicy policy)
 {
     if (!mCacheEntry)  return NS_ERROR_NOT_AVAILABLE;
     // XXX validate policy against session?
+    
+    PRBool      storageEnabled = PR_FALSE;
+    nsresult    rv = nsCacheService::IsStorageEnabledForPolicy(policy, &storageEnabled);
+    if (NS_FAILED(rv))      return rv;
+    if (!storageEnabled)    return NS_ERROR_FAILURE;
+    
     mCacheEntry->SetStoragePolicy(policy);
     mCacheEntry->MarkEntryDirty();
     return NS_OK;
@@ -349,7 +357,7 @@ nsCacheEntryDescriptor::GetMetaDataElement(const char *key, char ** result)
     if (!mCacheEntry)  return NS_ERROR_NOT_AVAILABLE;
 
     if (!key | !result) return NS_ERROR_NULL_POINTER;
-    nsAReadableCString *value;
+    const nsACString *value;
     *result = nsnull;
 
     // XXX not thread safe    
