@@ -149,7 +149,7 @@ nsDiskCacheMap::Open(nsILocalFile *  cacheDirectory)
     rv = cacheDirectory->Clone(getter_AddRefs(file));
     nsCOMPtr<nsILocalFile> localFile(do_QueryInterface(file, &rv));
     if (NS_FAILED(rv))  return rv;
-    rv = localFile->Append("_CACHE_MAP_");
+    rv = localFile->AppendNative(NS_LITERAL_CSTRING("_CACHE_MAP_"));
     if (NS_FAILED(rv))  return rv;
     
     // open the file
@@ -173,8 +173,8 @@ nsDiskCacheMap::Open(nsILocalFile *  cacheDirectory)
             mHeader.mEvictionRank[i] = 0;
         }
 
-        nsCRT::zero(mHeader.reserved, nsDiskCacheHeader::kReservedBytes);
-        nsCRT::zero(mBuckets, sizeof(nsDiskCacheBucket) * kBucketsPerTable);
+        memset(mHeader.reserved, 0, nsDiskCacheHeader::kReservedBytes);
+        memset(mBuckets, 0, sizeof(nsDiskCacheBucket) * kBucketsPerTable);
         
     } else if (mapSize == kCacheMapSize) {
         // read it in
@@ -799,7 +799,7 @@ nsDiskCacheMap::GetFileForDiskCacheRecord(nsDiskCacheRecord * record,
     PRInt16 generation = record->Generation();
     char name[32];
     ::sprintf(name, "%08X%c%02X", record->HashNumber(),  (meta ? 'm' : 'd'), generation);
-    rv = file->Append(name);
+    rv = file->AppendNative(nsDependentCString(name));
     if (NS_FAILED(rv))  return rv;
     
     NS_IF_ADDREF(*result = file);
@@ -834,7 +834,7 @@ nsDiskCacheMap::GetBlockFileForIndex(PRUint32 index, nsILocalFile ** result)
     
     char name[32];
     ::sprintf(name, "_CACHE_%03d_", index + 1);
-    rv = file->Append(name);
+    rv = file->AppendNative(nsDependentCString(name));
     if (NS_FAILED(rv))  return rv;
     
     nsCOMPtr<nsILocalFile> localFile = do_QueryInterface(file, &rv);
