@@ -35,6 +35,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#ifdef HAVE_RES_NINIT
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>   
+#include <arpa/nameser.h>
+#include <resolv.h>
+#endif
+
 #include "nsDnsService.h"
 
 #include "nsIDNSListener.h"
@@ -47,6 +55,7 @@
 #include "nsIServiceManager.h"
 #include "nsXPIDLString.h"
 #include "nsTime.h"
+#include "nsCRT.h"
 
 #include "nsError.h"
 #include "prnetdb.h"
@@ -98,6 +107,7 @@
  */
 #if defined(XP_WIN)
 #define WM_DNS_SHUTDOWN         (WM_USER + 200)
+static char *windowClass = "Mozilla:DNSWindowClass";
 #endif /* XP_WIN */
 
 
@@ -268,8 +278,8 @@ private:
 
 
 #if defined(XP_UNIX) || defined(XP_BEOS) || defined(XP_OS2)
-
     void                DoSyncLookup();
+    PRStatus            DoSyncLookupInternal();
 #endif
 };
 
@@ -415,7 +425,7 @@ nsDNSRequest::FireStop(nsresult  status)
 
 
 NS_IMETHODIMP
-nsDNSRequest::GetName(PRUnichar **  result)
+nsDNSRequest::GetName(nsACString & result)
 {
     NS_NOTREACHED("nsDNSRequest::GetName");
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -440,19 +450,23 @@ NS_IMETHODIMP
 nsDNSRequest::Cancel(nsresult  status)
 {
     NS_ASSERTION(NS_FAILED(status), "shouldn't cancel with a success code");
-    nsresult  rv = NS_OK;
-    mStatus = status;
-    
-    NS_ASSERTION(!PR_CLIST_IS_EMPTY(this), "request is not queue on lookup");
-    if (!PR_CLIST_IS_EMPTY(this)) {
-        nsDNSService::Lock();
-        PR_REMOVE_AND_INIT_LINK(this);
-        nsDNSService::Unlock();
-    }
 
-    if (mUserListener)  rv = FireStop(status);
-    mLookup = nsnull;
-    return rv;
+    PRBool ignore = PR_FALSE;
+    nsDNSService::Lock();
+    if (PR_CLIST_IS_EMPTY(this)) {
+        // ignore this cancelation since we've already called OnStopLookup or
+        // are in the process of calling OnStopLookup (on a different thread).
+        ignore = PR_TRUE;
+    } else {
+        PR_REMOVE_AND_INIT_LINK(this);
+        ignore = (mUserListener == nsnull);
+    }
+    nsDNSService::Unlock();
+
+    if (ignore)
+        return NS_OK;
+
+    return FireStop(status);
 }
 
 NS_IMETHODIMP
@@ -585,7 +599,7 @@ nsDNSLookup::ConvertHostEntry()
     mStatus = NS_ERROR_OUT_OF_MEMORY;
     
     // copy name
-    len = nsCRT::strlen(mInetHostInfo.name);
+    len = strlen(mInetHostInfo.name);
     mHostEntry.hostEnt.h_name = BufAlloc(len + 1, &mHostEntry.bufPtr, &mHostEntry.bufLen, 0);
     NS_ASSERTION(nsnull != mHostEntry.hostEnt.h_name,"nsHostEnt buffer full.");
     if (mHostEntry.hostEnt.h_name == nsnull)  return;
@@ -650,7 +664,7 @@ nsDNSLookup::HostnameIsIPAddress()
     // slam the IP in and move on.
     mHostEntry.bufLen = PR_NETDB_BUF_SIZE;
 
-    PRUint32 hostNameLen = nsCRT::strlen(mHostName);
+    PRUint32 hostNameLen = strlen(mHostName);
     mHostEntry.hostEnt.h_name = (char*)BufAlloc(hostNameLen + 1,
                                                 (char**)&mHostEntry.bufPtr,
                                                 &mHostEntry.bufLen,
@@ -743,7 +757,8 @@ nsDNSLookup::MarkComplete(nsresult status)
 {
     mStatus = status;
     mState  = LOOKUP_COMPLETE;
-    if (NS_FAILED(status))  MarkNotCacheable();
+    if (NS_FAILED(status)) 
+        MarkNotCacheable();
 }
 
 
@@ -819,11 +834,17 @@ nsDNSLookup::IsExpired()
 
 #if defined(XP_UNIX) || defined(XP_BEOS) || defined(XP_OS2)
 
-void
-nsDNSLookup::DoSyncLookup()
+
+PRStatus
+nsDNSLookup::DoSyncLookupInternal()
 {
     PRStatus status;
-    nsresult rv = NS_OK;
+
+    // must not hold the service lock while resolving, otherwise we could
+    // end up locking up the DNS service and anyone who accesses it until
+    // this function returns, which could be a while if the DNS server is
+    // unreachable. XXX is this thread-safe?
+    nsDNSService::Unlock();
 
     status = PR_GetIPNodeByName(mHostName,
                                 PR_AF_INET6,
@@ -832,10 +853,20 @@ nsDNSLookup::DoSyncLookup()
                                 PR_NETDB_BUF_SIZE,
                                 &(mHostEntry.hostEnt));
 
-    if (PR_SUCCESS != status)
-        rv = NS_ERROR_UNKNOWN_HOST;
+    nsDNSService::Lock();
+    return status;
+}
 
-    MarkComplete(rv);
+void
+nsDNSLookup::DoSyncLookup()
+{
+    PRStatus status = DoSyncLookupInternal();
+    
+    if (PR_SUCCESS != status) {
+        if (nsDNSService::Reset());
+            status = DoSyncLookupInternal();
+    }
+    MarkComplete(PR_SUCCESS == status ? NS_OK : NS_ERROR_UNKNOWN_HOST);
 }
 
 #endif /* XP_UNIX || XP_BEOS || XP_OS2*/
@@ -866,7 +897,8 @@ PLDHashTableOps nsDNSService::gHashTableOps =
 
 
 nsDNSService::nsDNSService()
-    : mDNSServiceLock(nsnull)
+    : mPrefService(nsnull)
+    , mDNSServiceLock(nsnull)
     , mDNSCondVar(nsnull)
     , mEvictionQCount(0)
     , mMaxCachedLookups(32)
@@ -894,6 +926,10 @@ nsDNSService::nsDNSService()
     PR_INIT_CLIST(&mPendingQ);
     PR_INIT_CLIST(&mEvictionQ);
 
+    // Let us say that the most times a reset can happen is one time every second.
+    mResetMaxInterval = PR_SecondsToInterval(1);
+    mLastReset = PR_IntervalNow();
+
 #if defined(XP_MAC)
     gNeedLateInitialization = PR_TRUE;
     
@@ -933,9 +969,22 @@ nsDNSService::Init()
 	PRStatus status = PR_SUCCESS;
 #endif
 
+    if (mState == DNS_OFFLINE) {
+        mState = DNS_ONLINE;
+        return NS_OK;
+    }
+    
     NS_ASSERTION(mDNSServiceLock == nsnull, "nsDNSService not shut down");
     if (mDNSServiceLock)  return NS_ERROR_ALREADY_INITIALIZED;
 
+    // install xpcom shutdown observer
+    nsCOMPtr<nsIObserverService> observerService =
+        do_GetService("@mozilla.org/observer-service;1", &rv);
+    if (NS_FAILED(rv)) return rv;
+
+    rv = observerService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID, PR_FALSE);
+    if (NS_FAILED(rv)) return rv;
+
 #if defined(XP_MAC)
     OSStatus errOT = INIT_OPEN_TRANSPORT();
     NS_ASSERTION(errOT == kOTNoError, "InitOpenTransport failed.");
@@ -986,7 +1035,7 @@ nsDNSService::Init()
     rv = InstallPrefObserver();
     if (NS_FAILED(rv))  return rv;
 
-    mState = DNS_RUNNING;
+    mState = DNS_ONLINE;
     return NS_OK;
 
 #if defined(XP_WIN)
@@ -1047,7 +1096,7 @@ nsDNSService::LateInit()
 
 nsDNSService::~nsDNSService()
 {
-    nsresult rv = Shutdown();
+    nsresult rv = ShutdownInternal();
     NS_ASSERTION(NS_SUCCEEDED(rv), "DNS shutdown failed");
     NS_ASSERTION(mThread == nsnull, "DNS shutdown failed");
     NS_ASSERTION(mDNSServiceLock == nsnull, "DNS shutdown failed");
@@ -1073,6 +1122,10 @@ nsDNSService::~nsDNSService()
     }
 #endif
     CRTFREEIF(mMyIPAddress);
+
+#if defined(XP_WIN)
+    UnregisterClass(windowClass, (HINSTANCE) NULL);
+#endif
 }
 
 
@@ -1116,6 +1169,30 @@ nsDNSService::Unlock()
     }
 }
 
+PRBool
+nsDNSService::Reset()
+{
+#ifdef HAVE_RES_NINIT
+    // On platforms where res_ninit() is available and supported
+    // by Mozilla, we shall attempt to reinitialize the dns
+    // resolver.  Since this method may be called many times in
+    // a row, we shall restrict calling into res_init no more than
+    // once every mResetMaxInterval.
+    //
+    // For the linux multithreaded resolver discussion see:
+    // http://sources.redhat.com/ml/libc-hacker/2000-07/msg00487.html
+
+    if (gService &&
+        PR_IntervalNow() - gService->mLastReset >= gService->mResetMaxInterval) {
+
+        (void) res_ninit(&_res);
+
+        gService->mLastReset = PR_IntervalNow();
+        return PR_TRUE;
+    }
+#endif
+    return PR_FALSE;
+}
 
 PRInt32
 nsDNSService::ExpirationInterval()
@@ -1136,6 +1213,9 @@ nsDNSService::InstallPrefObserver()
     nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv))  return rv;
     
+    // save for later
+    mPrefService = getter_AddRefs( NS_GetWeakReference(prefs));
+    
     nsCOMPtr<nsIPrefBranchInternal> prefInternal = do_QueryInterface(prefs, &rv);
     if (NS_FAILED(rv))  return rv;
     
@@ -1175,8 +1255,8 @@ nsDNSService::RemovePrefObserver()
 {
     nsresult rv, rv2;
     
-    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
-    if (NS_FAILED(rv))  return rv;
+    nsCOMPtr<nsIPrefService> prefs = do_QueryReferent(mPrefService);
+    if (!prefs)  return NS_OK;  // no pref service, no need to unregister
     
     nsCOMPtr<nsIPrefBranchInternal> prefInternal = do_QueryInterface(prefs, &rv);
     if (NS_FAILED(rv))  return rv;
@@ -1198,6 +1278,13 @@ nsDNSService::Observe(nsISupports *      subject,
 {
     nsresult rv = NS_OK;
     
+    if (!nsCRT::strcmp(NS_XPCOM_SHUTDOWN_OBSERVER_ID, topic))
+    {
+        // we need to shutdown!
+        ShutdownInternal(); 
+        return NS_OK;
+    }
+
     if (nsCRT::strcmp(NS_PREFBRANCH_PREFCHANGE_TOPIC_ID, topic))  
         return NS_OK;
     
@@ -1235,7 +1322,6 @@ nsDNSService::Observe(nsISupports *      subject,
             mIDNConverter = nsnull;
         }
     }
-    
     return rv;
 }
 
@@ -1260,12 +1346,12 @@ nsDNSService::Run()
                 // convert InetHostInfo to nsHostEnt
                 lookup->ConvertHostEntry();
             }
-            lookup->ProcessRequests();
             if (lookup->IsNotCacheable()) {
                 EvictLookup(lookup);
             } else {
                 AddToEvictionQ(lookup);
             }
+            lookup->ProcessRequests();
             NS_RELEASE(lookup);
         }
         
@@ -1290,12 +1376,12 @@ nsDNSService::Run()
             // Got a request!!
             NS_ADDREF(lookup);      // keep the lookup while we process it
             lookup->DoSyncLookup();
-            lookup->ProcessRequests();
             if (lookup->IsNotCacheable()) {
                 EvictLookup(lookup);
             } else {
                 AddToEvictionQ(lookup);
             }
+            lookup->ProcessRequests();
             NS_RELEASE(lookup);
         } else {
             // Woken up without a request --> shutdown
@@ -1313,7 +1399,6 @@ NS_IMETHODIMP
 nsDNSService::Run()
 {
     WNDCLASS    wc;
-    char *      windowClass = "Mozilla:DNSWindowClass";
 
     // register window class for DNS event receiver window
     memset(&wc, 0, sizeof(wc));
@@ -1372,7 +1457,7 @@ nsDNSService::Lookup(const char*     hostName,
     nsresult rv;
     nsDNSRequest * request = nsnull;
     *result = nsnull;
-    if (!mDNSServiceLock || (mState != DNS_RUNNING))  return NS_ERROR_OFFLINE;
+    if (!mDNSServiceLock || (mState != DNS_ONLINE))  return NS_ERROR_OFFLINE;
     else {
         nsAutoLock  dnsLock(mDNSServiceLock);
         
@@ -1388,7 +1473,7 @@ nsDNSService::Lookup(const char*     hostName,
         // IDN handling
         if (mIDNConverter && !nsCRT::IsAscii(hostName)) {
             nsXPIDLCString hostNameACE;
-            rv = mIDNConverter->UTF8ToIDNHostName(hostName, getter_Copies(hostNameACE));
+            rv = mIDNConverter->ConvertUTF8toACE(hostName, getter_Copies(hostNameACE));
             if (!hostNameACE.get()) return NS_ERROR_OUT_OF_MEMORY;
             lookup = FindOrCreateLookup(hostNameACE);
         }
@@ -1410,7 +1495,6 @@ nsDNSService::Lookup(const char*     hostName,
         if (NS_FAILED(rv))  goto exit;
 
         if (lookup->IsComplete()) {
-            lookup->ProcessRequests();      // releases and re-acquires dns lock
             if (lookup->IsNotCacheable()) {
                 // non-cacheable lookups are released here.
                 EvictLookup(lookup);
@@ -1420,6 +1504,7 @@ nsDNSService::Lookup(const char*     hostName,
                     AddToEvictionQ(lookup);
 #endif                        
             }
+            lookup->ProcessRequests();      // releases and re-acquires dns lock
         }
 
 exit:
@@ -1554,8 +1639,10 @@ nsDNSService::AbortLookups()
     while(lookup != &mPendingQ) {
         PR_REMOVE_AND_INIT_LINK(lookup);
         lookup->MarkComplete(NS_BINDING_ABORTED);
-        lookup->ProcessRequests();
+        NS_ADDREF(lookup);
         EvictLookup(lookup);
+        lookup->ProcessRequests();
+        NS_RELEASE(lookup);
         lookup = (nsDNSLookup *)PR_LIST_HEAD(&mPendingQ);
     }
     
@@ -1574,33 +1661,50 @@ nsDNSService::AbortLookups()
 NS_IMETHODIMP
 nsDNSService::Resolve(const char *i_hostname, char **o_ip)
 {
-    // Note that this does not check for an IP address already in hostname
-    // So beware!
+    if (!mDNSServiceLock || (mState != DNS_ONLINE))  return NS_ERROR_OFFLINE;
+
+    // Beware! This does not check for an IP address already in hostname.
     NS_ENSURE_ARG_POINTER(o_ip);
     *o_ip = 0;
     NS_ENSURE_ARG_POINTER(i_hostname);
 
-    PRHostEnt he;
-    char netdbbuf[PR_NETDB_BUF_SIZE];
+    PRNetAddr   netAddr;
+    PRIntn      index = 0;
+    char        ipBuffer[64];
 
-    if (PR_SUCCESS == PR_GetHostByName(i_hostname, 
-                netdbbuf, 
-                sizeof(netdbbuf), 
-                &he))
-    {
-        struct in_addr in;
-        memcpy(&in.s_addr, he.h_addr, he.h_length);
-        char* ip = 0;
-
-        ip = inet_ntoa(in);
-        
-        if (ip)
-        {
-            return (*o_ip = nsCRT::strdup(ip)) ? 
-                NS_OK: NS_ERROR_OUT_OF_MEMORY;
+    // aquire lock to check hash table for existing lookup
+    // release lock before calling synchronous PR_GetHostByName()
+    {    
+        nsAutoLock    dnsLock(mDNSServiceLock);
+        PLDHashEntryHdr * hashEntry = PL_DHashTableOperate(&mHashTable, i_hostname, PL_DHASH_LOOKUP);
+        if (PL_DHASH_ENTRY_IS_BUSY(hashEntry)) {
+            nsDNSLookup * lookup = ((DNSHashTableEntry *)hashEntry)->mLookup;
+            if (lookup->IsComplete() && !lookup->IsExpired()) {
+                nsHostEnt * hep = lookup->HostEntry();
+                NS_ASSERTION(hep != nsnull, "null DNS Lookup HostEntry.\n");
+                if (hep) {
+                    index = PR_EnumerateHostEnt(0, &hep->hostEnt, 0, &netAddr);
+                }
+            }
         }
     }
-    return NS_ERROR_FAILURE;
+
+    if (index == 0) {
+        PRHostEnt   he;
+        char        netdbbuf[PR_NETDB_BUF_SIZE];
+
+        if (PR_SUCCESS == PR_GetHostByName(i_hostname, netdbbuf, sizeof(netdbbuf), &he)) {
+            index = PR_EnumerateHostEnt(0, &he, 0, &netAddr);
+        }
+    }
+            
+    if (index == 0)  return NS_ERROR_FAILURE;
+    else {
+        PRStatus  status = PR_NetAddrToString(&netAddr, ipBuffer, sizeof(ipBuffer));
+        if (status != PR_SUCCESS)   return NS_ERROR_FAILURE;
+    }
+    
+    return (*o_ip = nsCRT::strdup(ipBuffer)) ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 
@@ -1688,6 +1792,13 @@ nsDNSService::IsInNet(const char * ipaddr,
 
 NS_IMETHODIMP
 nsDNSService::Shutdown()
+{
+    mState = DNS_OFFLINE;
+    return NS_OK;
+}
+
+nsresult
+nsDNSService::ShutdownInternal()
 {
     nsresult rv = NS_OK;
 
@@ -1724,7 +1835,7 @@ nsDNSService::Shutdown()
 #elif defined(XP_WIN)
 
     SendMessage(mDNSWindow, WM_DNS_SHUTDOWN, 0, 0);
-
+    UnregisterClass(windowClass, NULL);
 #endif
 
     PR_Unlock(mDNSServiceLock);  // so dns thread can aquire it.
@@ -1738,6 +1849,15 @@ nsDNSService::Shutdown()
 
     (void) RemovePrefObserver();
 
+
+    // remove xpcom shutdown observer
+    nsCOMPtr<nsIObserverService> observerService =
+        do_GetService("@mozilla.org/observer-service;1", &rv);
+    if (NS_FAILED(rv)) return rv;
+
+    rv = observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
+    if (NS_FAILED(rv)) return rv;
+
     // reset hashtable
     // XXX assert hashtable is empty
     PL_DHashTableFinish(&mHashTable);
@@ -1856,12 +1976,12 @@ nsDNSService::ProcessLookup(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
     FreeMsgID(lookup->mMsgID);
 
     PR_REMOVE_AND_INIT_LINK(lookup);
-    lookup->ProcessRequests();
     if (lookup->IsNotCacheable()) {
         EvictLookup(lookup);
     } else {
         AddToEvictionQ(lookup);
     }
+    lookup->ProcessRequests();
     NS_RELEASE(lookup);
 
     return error ? -1 : 0;
