@@ -186,10 +186,10 @@ nsXMLMIMEDataSource::Add( nsIMIMEInfo* aMapper )
     if (NS_FAILED(rv)) return rv;
 
     // Finally add an extension mapping.
-    char** extensions;
-    PRUint32 count;
+    char** extensions = nsnull;
+    PRUint32 count = 0;
     rv = aMapper->GetFileExtensions(& count, &extensions );
-    if ( NS_FAILED ( rv ) )
+    if ( NS_FAILED ( rv ) || !extensions)
         return rv;
     for ( PRUint32 i = 0; i<count; i++ )
     {
@@ -308,13 +308,17 @@ nsXMLMIMEDataSource::Serialize() {
            do_GetService(kFileTransportServiceCID, &rv) ;
   if(NS_FAILED(rv)) return rv ;
  
-  rv = fts->CreateTransport(mFile, PR_WRONLY|PR_CREATE_FILE, PR_IRWXU, getter_AddRefs(transport)) ;
+  rv = fts->CreateTransport(mFile,
+                            PR_WRONLY|PR_CREATE_FILE,
+                            PR_IRWXU,
+                            PR_TRUE,
+                            getter_AddRefs(transport)) ;
   if(NS_FAILED(rv))
     return rv ;
   
   // we don't need to worry about notification callbacks
   nsCOMPtr<nsIOutputStream> stream;
-  rv = transport->OpenOutputStream(0, -1, 0, getter_AddRefs( stream ) ) ;
+  rv = transport->OpenOutputStream(0, PRUint32(-1), 0, getter_AddRefs( stream ) ) ;
 	if(NS_FAILED(rv))
     return rv ;
 	nsCOMPtr<nsISimpleEnumerator> enumerator;	
@@ -361,10 +365,10 @@ nsXMLMIMEDataSource::Serialize() {
 		buffer+=cdata;
 		buffer+="\" ";
 		
-		char** extensions;
-		PRUint32 count;
+		char** extensions = nsnull;
+		PRUint32 count = 0;
 		rv = info->GetFileExtensions(& count, &extensions );
-		if ( NS_FAILED ( rv ) )
+		if ( NS_FAILED ( rv ) || !extensions)
 			return rv;
 		buffer+=kExtensions;
 		buffer+="=\"";
@@ -507,6 +511,9 @@ nsXMLMIMEDataSource::InitFromHack() {
     rv = AddMapping(IMAGE_PNG, "png", "PNG Image", nsnull);
     if (NS_FAILED(rv)) return rv;
 
+    rv = AddMapping(IMAGE_XBM, "xbm", "XBM Image", nsnull);
+    if (NS_FAILED(rv)) return rv;
+
     rv = AddMapping(IMAGE_MNG, "mng", "MNG Image", nsnull);
     if (NS_FAILED(rv)) return rv;
 
@@ -726,13 +733,17 @@ nsXMLMIMEDataSource::InitFromFile( nsIFile*  aFile  )
           do_GetService(kFileTransportServiceCID, &rv) ;
  if(NS_FAILED(rv)) return rv ;
  // Made second parameter 0 since I really don't know what it is used for
- rv = fts->CreateTransport(aFile, PR_RDONLY, PR_IRWXU, getter_AddRefs(transport)) ;
+ rv = fts->CreateTransport(aFile,
+                           PR_RDONLY,
+                           PR_IRWXU,
+                           PR_TRUE,
+                           getter_AddRefs(transport)) ;
  if(NS_FAILED(rv))
    return rv ;
   
   // we don't need to worry about notification callbacks
  nsCOMPtr<nsIInputStream> stream;
- rv = transport->OpenInputStream(0, -1, 0, getter_AddRefs( stream ) ) ;
+ rv = transport->OpenInputStream(0, PRUint32(-1), 0, getter_AddRefs( stream ) ) ;
  if(NS_FAILED(rv))  return rv ;
     
  PRUint32 streamLength;
