@@ -59,10 +59,15 @@ nsHttpConnection::nsHttpConnection()
     , mKeepAliveMask(1)
     , mWriteDone(0)
     , mReadDone(0)
+    , mSupportsPipelining(PR_FALSE) // assume low-grade server
 {
     LOG(("Creating nsHttpConnection @%x\n", this));
 
     NS_INIT_ISUPPORTS();
+
+    // grab a reference to the handler to ensure that it doesn't go away.
+    nsHttpHandler *handler = nsHttpHandler::get();
+    NS_ADDREF(handler);
 }
 
 nsHttpConnection::~nsHttpConnection()
@@ -76,6 +81,10 @@ nsHttpConnection::~nsHttpConnection()
         PR_DestroyLock(mLock);
         mLock = nsnull;
     }
+
+    // release our reference to the handler
+    nsHttpHandler *handler = nsHttpHandler::get();
+    NS_RELEASE(handler);
 }
 
 nsresult
@@ -97,7 +106,7 @@ nsHttpConnection::Init(nsHttpConnectionInfo *info, PRUint16 maxHangTime)
     return NS_OK;
 }
 
-// called from any thread, with the connection lock held
+// called from any thread, without the connection lock held
 nsresult
 nsHttpConnection::SetTransaction(nsAHttpTransaction *transaction,
                                  PRUint8 caps)
@@ -156,6 +165,43 @@ nsHttpConnection::IsAlive()
     return isAlive;
 }
 
+PRBool
+nsHttpConnection::SupportsPipelining(nsHttpResponseHead *responseHead)
+{
+    // XXX there should be a strict mode available that disables this
+    // blacklisting.
+
+    // assuming connection is HTTP/1.1 with keep-alive enabled
+    if (mConnectionInfo->UsingHttpProxy() && !mConnectionInfo->UsingSSL()) {
+        // XXX check for bad proxy servers...
+        return PR_TRUE;
+    }
+
+    // XXX what about checking for a Via header? (transparent proxies)
+
+    // check for bad origin servers
+    const char *val = responseHead->PeekHeader(nsHttp::Server);
+    if (!val)
+        return PR_FALSE; // no header, no love
+
+    // the list of servers known to do bad things with pipelined requests
+    static const char *bad_servers[] = {
+        "Microsoft-IIS/4.",
+        "Netscape-Enterprise/3.",
+        nsnull
+    };
+
+    for (const char **server = bad_servers; *server; ++server) {
+        if (PL_strcasestr(val, *server) != nsnull) {
+            LOG(("looks like this server does not support pipelining"));
+            return PR_FALSE;
+        }
+    }
+
+    // ok, let's allow pipelining to this server
+    return PR_TRUE;
+}
+
 //----------------------------------------------------------------------------
 // nsHttpConnection::nsAHttpConnection
 //----------------------------------------------------------------------------
@@ -163,6 +209,7 @@ nsHttpConnection::IsAlive()
 // called from the socket thread
 nsresult
 nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
+                                     nsHttpRequestHead *requestHead,
                                      nsHttpResponseHead *responseHead,
                                      PRBool *reset)
 {
@@ -185,8 +232,11 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
     if (!val)
         val = responseHead->PeekHeader(nsHttp::Proxy_Connection);
 
+    // reset to default (the server may have changed since we last checked)
+    mSupportsPipelining = PR_FALSE;
+
     if ((responseHead->Version() < NS_HTTP_VERSION_1_1) ||
-        (nsHttpHandler::get()->DefaultVersion() < NS_HTTP_VERSION_1_1)) {
+        (requestHead->Version() < NS_HTTP_VERSION_1_1)) {
         // HTTP/1.0 connections are by default NOT persistent
         if (val && !PL_strcasecmp(val, "keep-alive"))
             mKeepAlive = PR_TRUE;
@@ -197,8 +247,10 @@ nsHttpConnection::OnHeadersAvailable(nsAHttpTransaction *trans,
         // HTTP/1.1 connections are by default persistent
         if (val && !PL_strcasecmp(val, "close")) 
             mKeepAlive = PR_FALSE;
-        else
+        else {
             mKeepAlive = PR_TRUE;
+            mSupportsPipelining = SupportsPipelining(responseHead);
+        }
     }
     mKeepAliveMask = mKeepAlive;
 
@@ -445,12 +497,8 @@ nsHttpConnection::CreateTransport()
     // configure the socket type based on the connection type requested.
     const char* type = nsnull;
 
-    if (mConnectionInfo->UsingSSL()) {
-        if (mConnectionInfo->UsingHttpProxy())
-            type = "tlsstepup";
-        else
-            type = "ssl";
-    }
+    if (mConnectionInfo->UsingSSL())
+        type = "ssl";
 
     nsCOMPtr<nsITransport> transport;
     rv = sts->CreateTransportOfType(type,
@@ -490,7 +538,7 @@ nsHttpConnection::SetupSSLProxyConnect()
     // CONNECT host:port HTTP/1.1
     nsHttpRequestHead request;
     request.SetMethod(nsHttp::Connect);
-    request.SetVersion(nsHttpHandler::get()->DefaultVersion());
+    request.SetVersion(nsHttpHandler::get()->HttpVersion());
     request.SetRequestURI(buf.get());
     request.SetHeader(nsHttp::User_Agent, nsHttpHandler::get()->UserAgent());
 
@@ -502,25 +550,22 @@ nsHttpConnection::SetupSSLProxyConnect()
     if (val) {
         // all HTTP/1.1 requests must include a Host header (even though it
         // may seem redundant in this case; see bug 82388).
-        request.SetHeader(nsHttp::Host, val);
+        request.SetHeader(nsHttp::Host, nsDependentCString(val));
     }
 
     val = trans->RequestHead()->PeekHeader(nsHttp::Proxy_Authorization);
     if (val) {
         // we don't know for sure if this authorization is intended for the
         // SSL proxy, so we add it just in case.
-        request.SetHeader(nsHttp::Proxy_Authorization, val);
+        request.SetHeader(nsHttp::Proxy_Authorization, nsDependentCString(val));
     }
 
     buf.Truncate(0);
-    request.Flatten(buf);
+    request.Flatten(buf, PR_FALSE);
     buf.Append("\r\n");
 
-    nsCOMPtr<nsISupports> sup;
-    rv = NS_NewCStringInputStream(getter_AddRefs(sup), buf);
-    if (NS_FAILED(rv)) return rv;
+    rv = NS_NewCStringInputStream(getter_AddRefs(mSSLProxyConnectStream), buf);
 
-    mSSLProxyConnectStream = do_QueryInterface(sup, &rv);
     return rv;
 }
 
