@@ -34,11 +34,14 @@
 #include "nsHttpResponseHead.h"
 #include "nsHttpTransaction.h"
 #include "nsHttpAuthCache.h"
+#include "nsHttpPipeline.h"
 #include "nsIHttpChannel.h"
 #include "nsIHttpNotify.h"
 #include "nsIURL.h"
+#include "nsIStandardURL.h"
 #include "nsICacheService.h"
 #include "nsICategoryManager.h"
+#include "nsCategoryManagerUtils.h"
 #include "nsIObserverService.h"
 #include "nsISupportsPrimitives.h"
 #include "nsINetModRegEntry.h"
@@ -76,7 +79,6 @@ static NS_DEFINE_CID(kNetModuleMgrCID, NS_NETMODULEMGR_CID);
 static NS_DEFINE_CID(kStreamConverterServiceCID, NS_STREAMCONVERTERSERVICE_CID);
 static NS_DEFINE_CID(kCacheServiceCID, NS_CACHESERVICE_CID);
 static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);
-static NS_DEFINE_CID(kMimeServiceCID, NS_MIMESERVICE_CID);
 
 #define UA_PREF_PREFIX          "general.useragent."
 #define UA_APPNAME              "Mozilla"
@@ -100,9 +102,10 @@ nsHttpHandler *nsHttpHandler::mGlobalInstance = 0;
 nsHttpHandler::nsHttpHandler()
     : mAuthCache(nsnull)
     , mHttpVersion(NS_HTTP_VERSION_1_1)
-    , mReferrerLevel(0xff) // by default we always send a referrer
+    , mProxyHttpVersion(NS_HTTP_VERSION_1_1)
     , mCapabilities(NS_HTTP_ALLOW_KEEPALIVE)
     , mProxyCapabilities(NS_HTTP_ALLOW_KEEPALIVE)
+    , mReferrerLevel(0xff) // by default we always send a referrer
     , mIdleTimeout(10)
     , mMaxRequestAttempts(10)
     , mMaxRequestDelay(10)
@@ -110,6 +113,7 @@ nsHttpHandler::nsHttpHandler()
     , mMaxConnectionsPerServer(8)
     , mMaxPersistentConnectionsPerServer(2)
     , mMaxPersistentConnectionsPerProxy(4)
+    , mMaxPipelinedRequests(2)
     , mRedirectionLimit(10)
     , mLastUniqueID(NowInSeconds())
     , mSessionStartTime(0)
@@ -119,6 +123,7 @@ nsHttpHandler::nsHttpHandler()
     , mConnectionLock(nsnull)
     , mUserAgentIsDirty(PR_TRUE)
     , mUseCache(PR_TRUE)
+    , mSendSecureXSiteReferrer(PR_FALSE)
 {
     NS_INIT_ISUPPORTS();
 
@@ -236,7 +241,7 @@ nsHttpHandler::Init()
     LOG(("> product = %s\n", mProduct.get()));
     LOG(("> product-sub = %s\n", mProductSub.get()));
     LOG(("> product-comment = %s\n", mProductComment.get()));
-    LOG(("> user-agent = %s\n", UserAgent()));
+    LOG(("> user-agent = %s\n", UserAgent().get()));
 #endif
 
     mSessionStartTime = NowInSeconds();
@@ -266,7 +271,7 @@ nsHttpHandler::Init()
     // may not be cleaned up as aggressively.
     if (mTimer)
         mTimer->Init(DeadConnectionCleanupCB, this, 15*1000, // 15 seconds
-                     NS_PRIORITY_NORMAL,
+                     PR_TRUE,
                      NS_TYPE_REPEATING_SLACK);
  
     return NS_OK;
@@ -287,22 +292,22 @@ nsHttpHandler::AddStandardRequestHeaders(nsHttpHeaderArray *request,
 
     // MIME based content negotiation lives!
     // Add the "Accept" header
-    rv = request->SetHeader(nsHttp::Accept, mAccept.get());
+    rv = request->SetHeader(nsHttp::Accept, mAccept);
     if (NS_FAILED(rv)) return rv;
 
     // Add the "Accept-Language" header
     if (!mAcceptLanguages.IsEmpty()) {
         // Add the "Accept-Language" header
-        rv = request->SetHeader(nsHttp::Accept_Language, mAcceptLanguages.get());
+        rv = request->SetHeader(nsHttp::Accept_Language, mAcceptLanguages);
         if (NS_FAILED(rv)) return rv;
     }
 
     // Add the "Accept-Encoding" header
-    rv = request->SetHeader(nsHttp::Accept_Encoding, mAcceptEncodings.get());
+    rv = request->SetHeader(nsHttp::Accept_Encoding, mAcceptEncodings);
     if (NS_FAILED(rv)) return rv;
 
     // Add the "Accept-Charset" header
-    rv = request->SetHeader(nsHttp::Accept_Charset, mAcceptCharsets.get());
+    rv = request->SetHeader(nsHttp::Accept_Charset, mAcceptCharsets);
     if (NS_FAILED(rv)) return rv;
 
     // RFC2616 section 19.6.2 states that the "Connection: keep-alive"
@@ -316,22 +321,17 @@ nsHttpHandler::AddStandardRequestHeaders(nsHttpHeaderArray *request,
     
     const char* connectionType = "close";
     if (caps & NS_HTTP_ALLOW_KEEPALIVE) {
-        char buf[32];
-        
-        PR_snprintf(buf, sizeof(buf), "%u", (PRUintn) mIdleTimeout);
-        
-        rv = request->SetHeader(nsHttp::Keep_Alive, buf);
+        rv = request->SetHeader(nsHttp::Keep_Alive, nsPrintfCString("%u", mIdleTimeout));
         if (NS_FAILED(rv)) return rv;
-        
         connectionType = "keep-alive";
     } else if (useProxy) {
         // Bug 92006
-        request->SetHeader(nsHttp::Connection, "close");
+        request->SetHeader(nsHttp::Connection, NS_LITERAL_CSTRING("close"));
     }
 
     const nsHttpAtom &header =
         useProxy ? nsHttp::Proxy_Connection : nsHttp::Connection;
-    return request->SetHeader(header, connectionType);
+    return request->SetHeader(header, nsDependentCString(connectionType));
 }
 
 PRBool
@@ -412,7 +412,7 @@ nsHttpHandler::InitiateTransaction(nsHttpTransaction *trans,
         PR_Unlock(mConnectionLock);
     }
     else {
-        rv = DispatchTransaction_Locked(trans, conn);
+        rv = DispatchTransaction_Locked(trans, trans->Capabilities(), conn);
         NS_RELEASE(conn);
     }
     return rv;
@@ -423,9 +423,7 @@ nsresult
 nsHttpHandler::ReclaimConnection(nsHttpConnection *conn)
 {
     NS_ENSURE_ARG_POINTER(conn);
-#ifdef DEBUG
-    NS_PRECONDITION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
-#endif
+    NS_ASSERTION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
 
     PRBool reusable = conn->CanReuse();
 
@@ -475,9 +473,7 @@ nsresult
 nsHttpHandler::ProcessTransactionQ()
 {
     LOG(("nsHttpHandler::ProcessTransactionQ\n"));
-#ifdef DEBUG
-    NS_PRECONDITION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
-#endif
+    NS_ASSERTION(PR_GetCurrentThread() == NS_SOCKET_THREAD, "wrong thread");
 
     PR_Lock(mConnectionLock);
 
@@ -565,7 +561,7 @@ nsHttpHandler::GetMimeService(nsIMIMEService **result)
 {
     if (!mMimeService) {
         nsresult rv;
-        mMimeService = do_GetService(kMimeServiceCID, &rv);
+        mMimeService = do_GetService("@mozilla.org/mime;1", &rv);
         if (NS_FAILED(rv)) return rv;
     }
     *result = mMimeService;
@@ -665,12 +661,12 @@ nsHttpHandler::OnExamineResponse(nsIHttpChannel *chan)
 // nsHttpHandler <private>
 //-----------------------------------------------------------------------------
 
-const char *
+const nsAFlatCString &
 nsHttpHandler::UserAgent()
 {
     if (mUserAgentOverride) {
         LOG(("using general.useragent.override : %s\n", mUserAgentOverride.get()));
-        return mUserAgentOverride.get();
+        return mUserAgentOverride;
     }
 
     if (mUserAgentIsDirty) {
@@ -678,7 +674,7 @@ nsHttpHandler::UserAgent()
         mUserAgentIsDirty = PR_FALSE;
     }
 
-    return mUserAgent.get();
+    return mUserAgent;
 }
 
 nsresult
@@ -737,6 +733,24 @@ nsHttpHandler::GetConnection_Locked(nsHttpConnectionInfo *ci,
             NS_RELEASE(conn);
             return rv;
         }
+        
+        // We created a new connection
+        // If idle + active connections > max connections, then purge the oldest idle one.
+        if (mIdleConnections.Count() + mActiveConnections.Count() > mMaxConnections) {
+            LOG(("Created a new connection and popping oldest one [idlecount=%d activecount=%d maxConn=%d]\n",
+                mIdleConnections.Count(), mActiveConnections.Count(), mMaxConnections));
+            NS_ASSERTION(mIdleConnections.Count() > 0, "idle connection list is empty");
+            if (mIdleConnections.Count() > 0) {
+                nsHttpConnection *conn = (nsHttpConnection *) mIdleConnections[0];
+                NS_ASSERTION(conn, "mIdleConnections[0] is null but count > 0");
+                if (conn) {
+                    LOG(("deleting connection [conn=%x host=%s:%d]\n",
+                    conn, conn->ConnectionInfo()->Host(), conn->ConnectionInfo()->Port()));
+                    mIdleConnections.RemoveElementAt(0);
+                    NS_RELEASE(conn);
+                }
+            }
+        }
     }
     else {
         // Update the connectionInfo (bug 94038)
@@ -765,7 +779,8 @@ nsHttpHandler::EnqueueTransaction_Locked(nsHttpTransaction *trans,
 
 // dispatch this transaction, return unlocked
 nsresult
-nsHttpHandler::DispatchTransaction_Locked(nsHttpTransaction *trans,
+nsHttpHandler::DispatchTransaction_Locked(nsAHttpTransaction *trans,
+                                          PRUint8 transCaps,
                                           nsHttpConnection *conn)
 {
     nsresult rv;
@@ -785,7 +800,7 @@ nsHttpHandler::DispatchTransaction_Locked(nsHttpTransaction *trans,
     // we must not hold the connection lock while making the call to
     // SetTransaction, as it could lead to deadlocks.
     PR_Unlock(mConnectionLock);
-    rv = conn->SetTransaction(trans, trans->Capabilities());
+    rv = conn->SetTransaction(trans, transCaps);
 
     if (NS_FAILED(rv)) {
         LOG(("nsHttpConnection::SetTransaction failed [rv=%x]\n", rv));
@@ -838,24 +853,53 @@ nsHttpHandler::ProcessTransactionQ_Locked()
     mTransactionQ.RemoveElementAt(i);
 
     // 
-    // step 3: dispatch this transaction
+    // step 3: determine if we can pipeline any other transactions along with
+    // this transaction.  if pipelining is disabled or there are no other
+    // transactions that can be sent with this transaction, then simply send
+    // this transaction.
     //
-    nsresult rv = DispatchTransaction_Locked(pt->Transaction(), conn);
+    nsAHttpTransaction *trans = pt->Transaction();
+    PRUint8 caps = pt->Transaction()->Capabilities();
+    nsPipelineEnqueueState pipelineState;
+    if (conn->SupportsPipelining() &&
+        caps & NS_HTTP_ALLOW_PIPELINING &&
+        BuildPipeline_Locked(pipelineState,
+                             pt->Transaction(),
+                             pt->ConnectionInfo())) {
+        // ok... let's rock!
+        trans = pipelineState.Transaction();
+        caps = pipelineState.TransactionCaps();
+        NS_ASSERTION(trans, "no transaction");
+    }
+#if defined(PR_LOGGING)
+    else
+        LOG(("no pipelining [because-of-server=%d because-of-caps=%d]\n",
+            conn->SupportsPipelining() == PR_FALSE, 
+            caps & NS_HTTP_ALLOW_PIPELINING == PR_FALSE));
+#endif
+
+    // 
+    // step 4: dispatch this transaction
+    //
+    nsresult rv = DispatchTransaction_Locked(trans, caps, conn);
 
     // we're no longer inside mConnectionLock
 
     // 
-    // step 4: handle errors / cleanup
+    // step 5: handle errors / cleanup
     //
     if (NS_FAILED(rv)) {
         LOG((">> DispatchTransaction_Locked failed [rv=%x]\n", rv));
         nsAutoLock lock(mConnectionLock);
         // there must have been something wrong with the connection,
         // requeue... we'll try again later.
+        if (caps & NS_HTTP_ALLOW_PIPELINING)
+            PipelineFailed_Locked(pipelineState);
         mTransactionQ.AppendElement(pt);
     }
     else
         delete pt;
+    pipelineState.Cleanup();
     NS_RELEASE(conn);
 }
 
@@ -943,18 +987,109 @@ nsHttpHandler::DropConnections(nsVoidArray &connections)
     connections.Clear();
 }
 
+PRBool
+nsHttpHandler::BuildPipeline_Locked(nsPipelineEnqueueState &state,
+                                    nsHttpTransaction *firstTrans,
+                                    nsHttpConnectionInfo *ci)
+{
+    if (mMaxPipelinedRequests < 2)
+        return PR_FALSE;
+
+    LOG(("BuildPipeline_Locked [trans=%x]\n", firstTrans));
+
+    //
+    // need to search the pending transaction list for other transactions
+    // that can be pipelined along with |firstTrans|.
+    //
+    nsresult rv = NS_ERROR_FAILURE; // by default, nothing to pipeline
+    PRUint8 numAppended = 0;
+    PRInt32 i = 0;
+    while (i < mTransactionQ.Count()) {
+        nsPendingTransaction *pt = (nsPendingTransaction *) mTransactionQ[i];
+        if (pt->Transaction()->Capabilities() & (NS_HTTP_ALLOW_KEEPALIVE |
+                                                 NS_HTTP_ALLOW_PIPELINING) &&
+            pt->ConnectionInfo()->Equals(ci)) {
+
+            //
+            // ok, we can add this transaction to our pipeline
+            //
+            if (numAppended == 0) {
+                rv = state.Init(firstTrans);
+                if (NS_FAILED(rv)) break;
+            }
+            rv = state.AppendTransaction(pt);
+            if (NS_FAILED(rv)) break;
+
+            //
+            // ok, remove the transaction from the pending queue; next time
+            // around the loop we'll still check the i-th element :-)
+            //
+            mTransactionQ.RemoveElementAt(i);
+
+            //
+            // we may have reached the pipelined requests limit...
+            //
+            if (++numAppended == (mMaxPipelinedRequests - 1))
+                break;
+        }
+        else
+            i++; // advance to next pending transaction
+    }
+    if (NS_FAILED(rv)) {
+        LOG(("  unable to pipeline any transactions with this one\n"));
+        state.Cleanup();
+        return PR_FALSE;
+    }
+    LOG(("  pipelined %u transactions\n", numAppended + 1));
+    return PR_TRUE;
+}
+
+void
+nsHttpHandler::PipelineFailed_Locked(nsPipelineEnqueueState &state)
+{
+    if ((mMaxPipelinedRequests < 2) || !state.HasPipeline())
+        return;
+
+    LOG(("PipelineFailed_Locked\n"));
+
+    // need to put any "appended" transactions back on the queue
+    PRInt32 i = 0;
+    while (i < state.NumAppendedTrans()) {
+        mTransactionQ.AppendElement(state.GetAppendedTrans(i));
+        i++;
+    }
+    state.DropAppendedTrans();
+}
+
 void
 nsHttpHandler::BuildUserAgent()
 {
     LOG(("nsHttpHandler::BuildUserAgent\n"));
 
-    NS_ASSERTION(mAppName &&
-                 mAppVersion &&
-                 mPlatform &&
-                 mSecurity &&
-                 mOscpu,
+    NS_ASSERTION(!mAppName.IsEmpty() &&
+                 !mAppVersion.IsEmpty() &&
+                 !mPlatform.IsEmpty() &&
+                 !mSecurity.IsEmpty() &&
+                 !mOscpu.IsEmpty(),
                  "HTTP cannot send practical requests without this much");
 
+    // preallocate to worst-case size, which should always be better
+    // than if we didn't preallocate at all.
+    mUserAgent.SetCapacity(mAppName.Length() + 
+                           mAppVersion.Length() + 
+                           mPlatform.Length() + 
+                           mSecurity.Length() +
+                           mOscpu.Length() +
+                           mLanguage.Length() +
+                           mMisc.Length() +
+                           mProduct.Length() +
+                           mProductSub.Length() +
+                           mProductComment.Length() +
+                           mVendor.Length() +
+                           mVendorSub.Length() +
+                           mVendorComment.Length() +
+                           22);
+
     // Application portion
     mUserAgent.Assign(mAppName);
     mUserAgent += '/';
@@ -968,25 +1103,25 @@ nsHttpHandler::BuildUserAgent()
     mUserAgent += mSecurity;
     mUserAgent += "; ";
     mUserAgent += mOscpu;
-    if (mLanguage) {
+    if (!mLanguage.IsEmpty()) {
         mUserAgent += "; ";
         mUserAgent += mLanguage;
     }
-    if (mMisc) {
+    if (!mMisc.IsEmpty()) {
         mUserAgent += "; ";
         mUserAgent += mMisc;
     }
     mUserAgent += ')';
 
     // Product portion
-    if (mProduct) {
+    if (!mProduct.IsEmpty()) {
         mUserAgent += ' ';
         mUserAgent += mProduct;
-        if (mProductSub) {
+        if (!mProductSub.IsEmpty()) {
             mUserAgent += '/';
             mUserAgent += mProductSub;
         }
-        if (mProductComment) {
+        if (!mProductComment.IsEmpty()) {
             mUserAgent += " (";
             mUserAgent += mProductComment;
             mUserAgent += ')';
@@ -994,14 +1129,14 @@ nsHttpHandler::BuildUserAgent()
     }
 
     // Vendor portion
-    if (mVendor) {
+    if (!mVendor.IsEmpty()) {
         mUserAgent += ' ';
         mUserAgent += mVendor;
-        if (mVendorSub) {
+        if (!mVendorSub.IsEmpty()) {
             mUserAgent += '/';
             mUserAgent += mVendorSub;
         }
-        if (mVendorComment) {
+        if (!mVendorComment.IsEmpty()) {
             mUserAgent += " (";
             mUserAgent += mVendorComment;
             mUserAgent += ')';
@@ -1085,6 +1220,14 @@ nsHttpHandler::InitUserAgentComponents()
             }
         }
     }
+#elif defined (XP_MACOSX)
+    mOscpu.Adopt(nsCRT::strdup("PPC Mac OS X"));
+#elif defined (XP_MAC)
+    long version;
+    if (::Gestalt(gestaltSystemVersion, &version) == noErr && version >= 0x00001000)
+        mOscpu.Adopt(nsCRT::strdup("PPC Mac OS X"));
+    else
+        mOscpu.Adopt(nsCRT::strdup("PPC"));
 #elif defined (XP_UNIX) || defined (XP_BEOS)
     struct utsname name;
     
@@ -1096,12 +1239,6 @@ nsHttpHandler::InitUserAgentComponents()
         buf += (char*)name.machine;
         mOscpu.Assign(buf);
     }
-#elif defined (XP_MAC)
-    long version;
-    if (::Gestalt(gestaltSystemVersion, &version) == noErr && version >= 0x00001000)
-        mOscpu.Adopt(nsCRT::strdup("PPC Mac OS X"));
-    else
-        mOscpu.Adopt(nsCRT::strdup("PPC"));
 #endif
 
     mUserAgentIsDirty = PR_TRUE;
@@ -1252,7 +1389,6 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
     if (PREF_CHANGED(HTTP_PREF("version"))) {
         nsXPIDLCString httpVersion;
         prefs->GetCharPref(HTTP_PREF("version"), getter_Copies(httpVersion));
-	
         if (httpVersion) {
             if (!PL_strcmp(httpVersion, "1.1"))
                 mHttpVersion = NS_HTTP_VERSION_1_1;
@@ -1261,14 +1397,17 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
             else
                 mHttpVersion = NS_HTTP_VERSION_1_0;
         }
+    }
 
-        if (mHttpVersion == NS_HTTP_VERSION_1_1) {
-            mCapabilities = NS_HTTP_ALLOW_KEEPALIVE;
-            mProxyCapabilities = NS_HTTP_ALLOW_KEEPALIVE;
-        }
-        else {
-            mCapabilities = 0;
-            mProxyCapabilities = 0;
+    if (PREF_CHANGED(HTTP_PREF("proxy.version"))) {
+        nsXPIDLCString httpVersion;
+        prefs->GetCharPref(HTTP_PREF("proxy.version"), getter_Copies(httpVersion));
+        if (httpVersion) {
+            if (!PL_strcmp(httpVersion, "1.1"))
+                mProxyHttpVersion = NS_HTTP_VERSION_1_1;
+            else
+                mProxyHttpVersion = NS_HTTP_VERSION_1_0;
+            // it does not make sense to issue a HTTP/0.9 request to a proxy server
         }
     }
 
@@ -1304,10 +1443,11 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         }
     }
 
-    /*
-    mPipelineMaxRequests  = DEFAULT_PIPELINE_MAX_REQUESTS;
-    rv = prefs->GetIntPref("network.http.pipelining.maxrequests", &mPipelineMaxRequests );
-    */
+    if (PREF_CHANGED(HTTP_PREF("pipelining.maxrequests"))) {
+        rv = prefs->GetIntPref(HTTP_PREF("pipelining.maxrequests"), &val);
+        if (NS_SUCCEEDED(rv))
+            mMaxPipelinedRequests = CLAMP(val, 1, NS_HTTP_MAX_PIPELINED_REQUESTS);
+    }
 
     if (PREF_CHANGED(HTTP_PREF("proxy.pipelining"))) {
         rv = prefs->GetBoolPref(HTTP_PREF("proxy.pipelining"), &cVar);
@@ -1319,6 +1459,12 @@ nsHttpHandler::PrefsChanged(nsIPrefBranch *prefs, const char *pref)
         }
     }
 
+    if (PREF_CHANGED(HTTP_PREF("sendSecureXSiteReferrer"))) {
+        rv = prefs->GetBoolPref(HTTP_PREF("sendSecureXSiteReferrer"), &cVar);
+        if (NS_SUCCEEDED(rv))
+            mSendSecureXSiteReferrer = cVar;
+    }
+
     /*
     if (bChangedAll || PL_strcmp(pref, "network.http.connect.timeout") == 0)
         prefs->GetIntPref("network.http.connect.timeout", &mConnectTimeout);
@@ -1495,7 +1641,7 @@ nsHttpHandler::SetAcceptLanguages(const char *aAcceptLanguages)
     nsCString buf;
     nsresult rv = PrepareAcceptLanguages(aAcceptLanguages, buf);
     if (NS_SUCCEEDED(rv))
-        mAcceptLanguages.Assign(buf.get());
+        mAcceptLanguages.Assign(buf);
     return rv;
 }
 
@@ -1623,7 +1769,7 @@ nsHttpHandler::SetAcceptCharsets(const char *aAcceptCharsets)
     nsCString buf;
     nsresult rv = PrepareAcceptCharsets(aAcceptCharsets, buf);
     if (NS_SUCCEEDED(rv))
-        mAcceptCharsets.Assign(buf.get());
+        mAcceptCharsets.Assign(buf);
     return rv;
 }
 
@@ -1657,7 +1803,7 @@ NS_IMPL_THREADSAFE_ISUPPORTS5(nsHttpHandler,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsHttpHandler::GetScheme(char **aScheme)
+nsHttpHandler::GetScheme(nsACString &aScheme)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -1676,7 +1822,10 @@ nsHttpHandler::GetProtocolFlags(PRUint32 *result)
 }
 
 NS_IMETHODIMP
-nsHttpHandler::NewURI(const char *aSpec, nsIURI *aBaseURI, nsIURI **aURI)
+nsHttpHandler::NewURI(const nsACString &aSpec,
+                      const char *aCharset,
+                      nsIURI *aBaseURI,
+                      nsIURI **aURI)
 {
     nsresult rv = NS_OK;
 
@@ -1686,7 +1835,7 @@ nsHttpHandler::NewURI(const char *aSpec, nsIURI *aBaseURI, nsIURI **aURI)
     if (NS_FAILED(rv)) return rv;
 
     // XXX need to choose the default port based on the scheme
-    rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY, 80, aSpec, aBaseURI);
+    rv = url->Init(nsIStandardURL::URLTYPE_AUTHORITY, 80, aSpec, aCharset, aBaseURI);
     if (NS_FAILED(rv)) return rv;
 
     return CallQueryInterface(url, aURI);
@@ -1714,10 +1863,7 @@ nsHttpHandler::NewChannel(nsIURI *uri, nsIChannel **result)
         }
     }
     
-    rv = NewProxiedChannel(uri,
-                           nsnull,
-                           result);
-    return rv;
+    return NewProxiedChannel(uri, nsnull, result);
 }
 
 NS_IMETHODIMP 
@@ -1747,16 +1893,28 @@ nsHttpHandler::NewProxiedChannel(nsIURI *uri,
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(httpChannel);
 
-    nsresult rv = httpChannel->Init(uri,
-                                    mCapabilities,
-                                    proxyInfo);
+    nsresult rv;
 
-    if (NS_SUCCEEDED(rv))
-        rv = httpChannel->
-                QueryInterface(NS_GET_IID(nsIChannel), (void **) result);
+    // select proxy caps if using a non-transparent proxy
+    PRInt8 caps = mCapabilities;
+    if (proxyInfo && !nsCRT::strcmp(proxyInfo->Type(), "http")) {
+        // SSL tunneling should not use proxy settings
+        PRBool isHTTPS;
+        rv = uri->SchemeIs("https", &isHTTPS);
+        if (NS_FAILED(rv)) return rv;
+        if (!isHTTPS)
+            caps = mProxyCapabilities;
+    }
 
-    NS_RELEASE(httpChannel);
-    return rv;
+    rv = httpChannel->Init(uri, caps, proxyInfo);
+
+    if (NS_FAILED(rv)) {
+        NS_RELEASE(httpChannel);
+        return rv;
+    }
+
+    *result = httpChannel;
+    return NS_OK;
 }
 
 //-----------------------------------------------------------------------------
@@ -1764,135 +1922,148 @@ nsHttpHandler::NewProxiedChannel(nsIURI *uri,
 //-----------------------------------------------------------------------------
 
 NS_IMETHODIMP
-nsHttpHandler::GetUserAgent(char **aUserAgent)
+nsHttpHandler::GetUserAgent(nsACString &value)
 {
-    return DupString(UserAgent(), aUserAgent);
+    value = UserAgent();
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetAppName(char **aAppName)
+nsHttpHandler::GetAppName(nsACString &value)
 {
-    return DupString(mAppName, aAppName);
+    value = mAppName;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetAppVersion(char **aAppVersion)
+nsHttpHandler::GetAppVersion(nsACString &value)
 {
-    return DupString(mAppVersion, aAppVersion);
+    value = mAppVersion;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendor(char **aVendor)
+nsHttpHandler::GetVendor(nsACString &value)
 {
-    return DupString(mVendor, aVendor);
+    value = mVendor;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendor(const char *aVendor)
+nsHttpHandler::SetVendor(const nsACString &value)
 {
-    mVendor.Adopt(aVendor ? nsCRT::strdup(aVendor) : 0);
+    mVendor = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendorSub(char **aVendorSub)
+nsHttpHandler::GetVendorSub(nsACString &value)
 {
-    return DupString(mVendorSub, aVendorSub);
+    value = mVendorSub;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendorSub(const char *aVendorSub)
+nsHttpHandler::SetVendorSub(const nsACString &value)
 {
-    mVendorSub.Adopt(aVendorSub ? nsCRT::strdup(aVendorSub) : 0);
+    mVendorSub = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetVendorComment(char **aVendorComment)
+nsHttpHandler::GetVendorComment(nsACString &value)
 {
-    return DupString(mVendorComment, aVendorComment);
+    value = mVendorComment;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetVendorComment(const char *aVendorComment)
+nsHttpHandler::SetVendorComment(const nsACString &value)
 {
-    mVendorComment.Adopt(aVendorComment ? nsCRT::strdup(aVendorComment) : 0);
+    mVendorComment = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProduct(char **aProduct)
+nsHttpHandler::GetProduct(nsACString &value)
 {
-    return DupString(mProduct, aProduct);
+    value = mProduct;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProduct(const char *aProduct)
+nsHttpHandler::SetProduct(const nsACString &value)
 {
-    mProduct.Adopt(aProduct ? nsCRT::strdup(aProduct) : 0);
+    mProduct = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProductSub(char **aProductSub)
+nsHttpHandler::GetProductSub(nsACString &value)
 {
-    return DupString(mProductSub, aProductSub);
+    value = mProductSub;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProductSub(const char *aProductSub)
+nsHttpHandler::SetProductSub(const nsACString &value)
 {
-    mProductSub.Adopt(aProductSub ? nsCRT::strdup(aProductSub) : 0);
+    mProductSub = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetProductComment(char **aProductComment)
+nsHttpHandler::GetProductComment(nsACString &value)
 {
-    return DupString(mProductComment, aProductComment);
+    value = mProductComment;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetProductComment(const char *aProductComment)
+nsHttpHandler::SetProductComment(const nsACString &value)
 {
-    mProductComment.Adopt(aProductComment ? nsCRT::strdup(aProductComment) : 0);
+    mProductComment = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetPlatform(char **aPlatform)
+nsHttpHandler::GetPlatform(nsACString &value)
 {
-    return DupString(mPlatform, aPlatform);
+    value = mPlatform;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetOscpu(char **aOscpu)
+nsHttpHandler::GetOscpu(nsACString &value)
 {
-    return DupString(mOscpu, aOscpu);
+    value = mOscpu;
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetLanguage(char **aLanguage)
+nsHttpHandler::GetLanguage(nsACString &value)
 {
-    return DupString(mLanguage, aLanguage);
+    value = mLanguage;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetLanguage(const char *aLanguage)
+nsHttpHandler::SetLanguage(const nsACString &value)
 {
-    mLanguage.Adopt(aLanguage ? nsCRT::strdup(aLanguage) : 0);
+    mLanguage = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsHttpHandler::GetMisc(char **aMisc)
+nsHttpHandler::GetMisc(nsACString &value)
 {
-    return DupString(mMisc, aMisc);
+    value = mMisc;
+    return NS_OK;
 }
 NS_IMETHODIMP
-nsHttpHandler::SetMisc(const char *aMisc)
+nsHttpHandler::SetMisc(const nsACString &value)
 {
-    mMisc.Adopt(aMisc ? nsCRT::strdup(aMisc) : 0);
+    mMisc = value;
     mUserAgentIsDirty = PR_TRUE;
     return NS_OK;
 }
@@ -1906,6 +2077,8 @@ nsHttpHandler::Observe(nsISupports *subject,
                        const char *topic,
                        const PRUnichar *data)
 {
+    LOG(("nsHttpHandler::Observe [topic=\"%s\")]\n", topic));
+
     if (!nsCRT::strcmp(topic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
         nsCOMPtr<nsIPrefBranch> prefBranch = do_QueryInterface(subject);
         if (prefBranch)
@@ -1917,6 +2090,13 @@ nsHttpHandler::Observe(nsISupports *subject,
         if (mAuthCache)
             mAuthCache->ClearAll();
 
+        // kill mIdleConnections - these sockets could be holding onto other resources
+        // that need to be freed.
+        {
+          nsAutoLock lock(mConnectionLock);
+          DropConnections(mIdleConnections);
+        }
+
         // need to reset the session start time since cache validation may
         // depend on this value.
         mSessionStartTime = NowInSeconds();
@@ -1959,3 +2139,42 @@ nsPendingTransaction::~nsPendingTransaction()
     NS_RELEASE(mTransaction);
     NS_RELEASE(mConnectionInfo);
 }
+
+//-----------------------------------------------------------------------------
+// nsHttpHandler::nsPipelineEnqueueState
+//-----------------------------------------------------------------------------
+
+nsresult nsHttpHandler::
+nsPipelineEnqueueState::Init(nsHttpTransaction *firstTrans)
+{
+    NS_ASSERTION(mPipeline == nsnull, "already initialized");
+
+    mPipeline = new nsHttpPipeline;
+    if (!mPipeline)
+        return NS_ERROR_OUT_OF_MEMORY;
+    NS_ADDREF(mPipeline);
+
+    return mPipeline->Init(firstTrans);
+}
+
+nsresult nsHttpHandler::
+nsPipelineEnqueueState::AppendTransaction(nsPendingTransaction *pt)
+{
+    nsresult rv = mPipeline->AppendTransaction(pt->Transaction());
+    if (NS_FAILED(rv)) return rv;
+
+    // remember this pending transaction object
+    mAppendedTrans.AppendElement(pt);
+    return NS_OK;
+}
+
+void nsHttpHandler::
+nsPipelineEnqueueState::Cleanup()
+{
+    NS_IF_RELEASE(mPipeline);
+
+    // free any appended pending transaction objects
+    for (PRInt32 i=0; i < mAppendedTrans.Count(); i++)
+        delete GetAppendedTrans(i);
+    mAppendedTrans.Clear();
+}
