@@ -24,6 +24,7 @@
 #include "nsCRT.h"
 #include "nsIFileTransportService.h"
 #include "nsIURI.h"
+#include "nsIFileURL.h"
 #include "nsCExternalHandlerService.h"
 #include "nsIMIMEService.h"
 #include "nsAutoLock.h"
@@ -63,9 +64,7 @@ PRLogModuleInfo* gJarProtocolLog = nsnull;
 
 nsJARChannel::nsJARChannel()
     : mLoadFlags(LOAD_NORMAL)
-    , mContentType(nsnull)
     , mContentLength(-1)
-    , mJAREntry(nsnull)
     , mStatus(NS_OK)
 #ifdef DEBUG
     , mInitiator(nsnull)
@@ -86,10 +85,6 @@ nsJARChannel::nsJARChannel()
 
 nsJARChannel::~nsJARChannel()
 {
-    if (mContentType)
-        nsCRT::free(mContentType);
-    if (mJAREntry)
-        nsCRT::free(mJAREntry);
     NS_IF_RELEASE(mJARProtocolHandler);
 }
 
@@ -135,16 +130,9 @@ nsJARChannel::Init(nsJARProtocolHandler* aHandler, nsIURI* uri)
 // nsIRequest methods
 
 NS_IMETHODIMP
-nsJARChannel::GetName(PRUnichar* *result)
+nsJARChannel::GetName(nsACString &result)
 {
-    nsresult rv;
-    nsXPIDLCString urlStr;
-    rv = mURI->GetSpec(getter_Copies(urlStr));
-    if (NS_FAILED(rv)) return rv;
-    nsString name;
-    name.AppendWithConversion(urlStr);
-    *result = ToNewUnicode(name);
-    return *result ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    return mURI->GetSpec(result);
 }
 
 NS_IMETHODIMP
@@ -244,7 +232,7 @@ nsJARChannel::OpenJARElement()
 {
     nsresult rv;
     nsAutoCMonitor mon(this);
-    rv = Open(nsnull, nsnull);
+    rv = Open();
     if (NS_SUCCEEDED(rv))
         rv = GetInputStream(getter_AddRefs(mSynchronousInputStream));
     mon.Notify();       // wake up nsIChannel::Open
@@ -301,17 +289,17 @@ nsJARChannel::EnsureJARFileAvailable()
 
 #ifdef PR_LOGGING
     if (PR_LOG_TEST(gJarProtocolLog, PR_LOG_DEBUG)) {
-        nsXPIDLCString jarURLStr;
-        mURI->GetSpec(getter_Copies(jarURLStr));
+        nsCAutoString jarURLStr;
+        mURI->GetSpec(jarURLStr);
         PR_LOG(gJarProtocolLog, PR_LOG_DEBUG,
-               ("nsJarProtocol: EnsureJARFileAvailable %s", (const char*)jarURLStr));
+               ("nsJarProtocol: EnsureJARFileAvailable %s", jarURLStr.get()));
     }
 #endif
 
     rv = mURI->GetJARFile(getter_AddRefs(mJARBaseURI));
     if (NS_FAILED(rv)) return rv;
 
-    rv = mURI->GetJAREntry(&mJAREntry);
+    rv = mURI->GetJAREntry(mJAREntry);
     if (NS_FAILED(rv)) return rv;
 
     // try to get a nsIFile directly from the url, which will often succeed.
@@ -351,7 +339,7 @@ nsJARChannel::AsyncReadJARElement()
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsITransport> jarTransport;
-    rv = fts->CreateTransportFromStreamIO(this, getter_AddRefs(jarTransport));
+    rv = fts->CreateTransportFromStreamIO(this, PR_TRUE, getter_AddRefs(jarTransport));
     if (NS_FAILED(rv)) return rv;
 
     if (mCallbacks) {
@@ -364,10 +352,10 @@ nsJARChannel::AsyncReadJARElement()
 
 #ifdef PR_LOGGING
     if (PR_LOG_TEST(gJarProtocolLog, PR_LOG_DEBUG)) {
-        nsXPIDLCString jarURLStr;
-        mURI->GetSpec(getter_Copies(jarURLStr));
+        nsCAutoString jarURLStr;
+        mURI->GetSpec(jarURLStr);
         PR_LOG(gJarProtocolLog, PR_LOG_DEBUG,
-               ("nsJarProtocol: AsyncRead jar entry %s", (const char*)jarURLStr));
+               ("nsJarProtocol: AsyncRead jar entry %s", jarURLStr.get()));
     }
 #endif
 
@@ -392,72 +380,79 @@ nsJARChannel::SetLoadFlags(PRUint32 aLoadFlags)
 }
 
 NS_IMETHODIMP
-nsJARChannel::GetContentType(char* *aContentType)
+nsJARChannel::GetContentType(nsACString &aContentType)
 {
     nsresult rv = NS_OK;
-    if (mContentType == nsnull) {
-        if (!mJAREntry)
-            return NS_ERROR_FAILURE;
-        char* fileName = nsCRT::strdup(mJAREntry);
-        if (fileName != nsnull) {
-            PRInt32 len = nsCRT::strlen(fileName);
-            const char* ext = nsnull;
-            for (PRInt32 i = len-1; i >= 0; i--) {
-                if (fileName[i] == '.') {
-                    ext = &fileName[i + 1];
-                    break;
-                }
+    if (mContentType.IsEmpty()) {
+        if (mJAREntry.IsEmpty())
+            return NS_ERROR_NOT_AVAILABLE;
+        const char *ext = nsnull, *fileName = mJAREntry.get();
+        PRInt32 len = mJAREntry.Length();
+        for (PRInt32 i = len-1; i >= 0; i--) {
+            if (fileName[i] == '.') {
+                ext = &fileName[i + 1];
+                break;
             }
-
-            if (ext) {
-                nsIMIMEService* mimeServ = mJARProtocolHandler->GetCachedMimeService();
-                if (mimeServ) {
-                    rv = mimeServ->GetTypeFromExtension(ext, &mContentType);
-                }
-            }
-            else 
-                rv = NS_ERROR_OUT_OF_MEMORY;
-            
-            nsCRT::free(fileName);
-        } 
-        else {
-            rv = NS_ERROR_OUT_OF_MEMORY;
         }
+        if (ext) {
+            nsIMIMEService* mimeServ = mJARProtocolHandler->GetCachedMimeService();
+            if (mimeServ) {
+                nsXPIDLCString mimeType;
+                rv = mimeServ->GetTypeFromExtension(ext, getter_Copies(mimeType));
+                if (NS_SUCCEEDED(rv))
+                    mContentType = mimeType;
+            }
+        }
+        else
+            rv = NS_ERROR_NOT_AVAILABLE;
 
         if (NS_FAILED(rv)) {
-            mContentType = nsCRT::strdup(UNKNOWN_CONTENT_TYPE);
-            if (mContentType == nsnull)
-                rv = NS_ERROR_OUT_OF_MEMORY;
-            else
-                rv = NS_OK;
+            mContentType = NS_LITERAL_CSTRING(UNKNOWN_CONTENT_TYPE);
+            rv = NS_OK;
         }
     }
-    if (NS_SUCCEEDED(rv)) {
-        *aContentType = nsCRT::strdup(mContentType);
-        if (*aContentType == nsnull)
-            rv = NS_ERROR_OUT_OF_MEMORY;
-    }
+    if (NS_SUCCEEDED(rv))
+        aContentType = mContentType;
+    else
+        aContentType.Truncate();
     return rv;
 }
 
 NS_IMETHODIMP
-nsJARChannel::SetContentType(const char *aContentType)
+nsJARChannel::SetContentType(const nsACString &aContentType)
 {
-    if (mContentType) {
-        nsCRT::free(mContentType);
-    }
+    mContentType = aContentType;
+    return NS_OK;
+}
 
-    mContentType = nsCRT::strdup(aContentType);
-    if (!mContentType) return NS_ERROR_OUT_OF_MEMORY;
+NS_IMETHODIMP
+nsJARChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset = mContentCharset;
+    return NS_OK;
+}
 
+NS_IMETHODIMP
+nsJARChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    mContentCharset = aContentCharset;
     return NS_OK;
 }
 
 NS_IMETHODIMP
 nsJARChannel::GetContentLength(PRInt32* aContentLength)
 {
-    if (mContentLength == -1)
-        return NS_ERROR_FAILURE;
+    NS_ENSURE_ARG_POINTER(aContentLength);
+    if (mContentLength == -1 && mJAR) {
+        // ask the zip entry for the content length
+        nsresult rv;
+        nsCOMPtr<nsIZipEntry> entry;
+        rv = mJAR->GetEntry(mJAREntry.get(), getter_AddRefs(entry));
+        if (NS_FAILED(rv)) return rv;
+
+        rv = entry->GetRealSize((PRUint32*)&mContentLength);
+        if (NS_FAILED(rv)) return rv;
+    }
     *aContentLength = mContentLength;
     return NS_OK;
 }
@@ -496,7 +491,7 @@ nsJARChannel::GetOwner(nsISupports* *aOwner)
         nsCOMPtr<nsIJAR> jar = do_QueryInterface(mJAR, &rv);
         NS_ASSERTION(NS_SUCCEEDED(rv), "Zip reader is not an nsIJAR");
         nsCOMPtr<nsIPrincipal> certificate;
-        rv = jar->GetCertificatePrincipal(mJAREntry, 
+        rv = jar->GetCertificatePrincipal(mJAREntry.get(), 
                                            getter_AddRefs(certificate));
         if (NS_FAILED(rv)) return rv;
         if (certificate)
@@ -598,12 +593,12 @@ nsJARChannel::OnStopRequest(nsIRequest* jarExtractionTransport, nsISupports* con
 #ifdef PR_LOGGING
     if (PR_LOG_TEST(gJarProtocolLog, PR_LOG_DEBUG)) {
         nsCOMPtr<nsIURI> jarURI;
-        nsXPIDLCString jarURLStr;
-        rv = mURI->GetSpec(getter_Copies(jarURLStr));
+        nsCAutoString jarURLStr;
+        rv = mURI->GetSpec(jarURLStr);
         if (NS_SUCCEEDED(rv)) {
             PR_LOG(gJarProtocolLog, PR_LOG_DEBUG,
                    ("nsJarProtocol: jar extraction complete %s status=%x",
-                    (const char*)jarURLStr, aStatus));
+                    jarURLStr.get(), aStatus));
         }
     }
 #endif
@@ -659,26 +654,9 @@ nsJARChannel::EnsureZipReader()
 }
 
 NS_IMETHODIMP
-nsJARChannel::Open(char* *contentType, PRInt32 *contentLength) 
+nsJARChannel::Open()
 {
-    nsresult rv;
-    rv = EnsureZipReader();
-    if (NS_FAILED(rv)) return rv;
-
-    nsCOMPtr<nsIZipEntry> entry;
-    rv = mJAR->GetEntry(mJAREntry, getter_AddRefs(entry));
-    if (NS_FAILED(rv)) return rv;
-
-    if (contentLength) {
-        rv = entry->GetRealSize((PRUint32*)contentLength);
-        if (NS_FAILED(rv)) return rv;
-    }
-
-    if (contentType) {
-        rv = GetContentType(contentType);
-        if (NS_FAILED(rv)) return rv;
-    }
-    return rv;
+    return EnsureZipReader();
 }
 
 NS_IMETHODIMP
@@ -692,14 +670,14 @@ nsJARChannel::GetInputStream(nsIInputStream* *aInputStream)
 {
 #ifdef PR_LOGGING
     if (PR_LOG_TEST(gJarProtocolLog, PR_LOG_DEBUG)) {
-        nsXPIDLCString jarURLStr;
-        mURI->GetSpec(getter_Copies(jarURLStr));
+        nsCAutoString jarURLStr;
+        mURI->GetSpec(jarURLStr);
         PR_LOG(gJarProtocolLog, PR_LOG_DEBUG,
-               ("nsJarProtocol: GetInputStream jar entry %s", (const char*)jarURLStr));
+               ("nsJarProtocol: GetInputStream jar entry %s", jarURLStr.get()));
     }
 #endif
     NS_ENSURE_TRUE(mJAR, NS_ERROR_NULL_POINTER);
-    nsresult rv = mJAR->GetInputStream(mJAREntry, aInputStream);
+    nsresult rv = mJAR->GetInputStream(mJAREntry.get(), aInputStream);
     if (NS_SUCCEEDED(rv))
         (*aInputStream)->Available((PRUint32 *) &mContentLength);
     return rv;
@@ -715,7 +693,11 @@ nsJARChannel::GetOutputStream(nsIOutputStream* *aOutputStream)
 NS_IMETHODIMP
 nsJARChannel::GetName(char* *aName) 
 {
-    return mURI->GetSpec(aName);
+    nsCAutoString spec;
+    nsresult rv = mURI->GetSpec(spec);
+    if (NS_FAILED(rv)) return rv;
+    *aName = ToNewCString(spec);
+    return *aName ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
