@@ -39,13 +39,13 @@
 #include "nsMemory.h"
 #include "plstr.h"
 #include "nsIHttpChannel.h"
-#include "nsIAtom.h"
 #include "nsIServiceManager.h"
 #include "nsNetUtil.h"
 #include "nsMimeTypes.h"
-#include "nsIByteArrayInputStream.h"
+#include "nsIStringStream.h"
 #include "nsReadableUtils.h"
 #include "nsIMultiPartChannel.h"
+#include "nsCRT.h"
 
 
 //
@@ -82,6 +82,8 @@ protected:
   nsCOMPtr<nsILoadGroup>  mLoadGroup;
 
   nsCString               mContentType;
+  nsCString               mContentCharset;
+  nsCString               mContentDisposition;
   PRInt32                 mContentLength;
 
   PRBool                  mIsByteRangeRequest;
@@ -139,7 +141,7 @@ NS_INTERFACE_MAP_END
 //
 
 NS_IMETHODIMP
-nsPartChannel::GetName(PRUnichar * *aResult)
+nsPartChannel::GetName(nsACString &aResult)
 {
     return mMultipartChannel->GetName(aResult);
 }
@@ -290,16 +292,30 @@ nsPartChannel::GetSecurityInfo(nsISupports * *aSecurityInfo)
 }
 
 NS_IMETHODIMP
-nsPartChannel::GetContentType(char * *aContentType)
+nsPartChannel::GetContentType(nsACString &aContentType)
 {
-    *aContentType = ToNewCString(mContentType);
+    aContentType = mContentType;
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsPartChannel::SetContentType(const char *aContentType)
+nsPartChannel::SetContentType(const nsACString &aContentType)
 {
-    mContentType.Assign(aContentType);
+    NS_ParseContentType(aContentType, mContentType, mContentCharset);
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPartChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset = mContentCharset;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPartChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    mContentCharset = aContentCharset;
     return NS_OK;
 }
 
@@ -317,6 +333,19 @@ nsPartChannel::SetContentLength(PRInt32 aContentLength)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsPartChannel::GetContentDisposition(nsACString &aContentDisposition)
+{
+    aContentDisposition = mContentDisposition;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsPartChannel::SetContentDisposition(const nsACString &aContentDisposition)
+{
+    mContentDisposition = aContentDisposition;
+    return NS_OK;
+}
 
 //
 // nsIByteRangeRequest implementation...
@@ -391,47 +420,45 @@ nsMultiMixedConv::AsyncConvertData(const PRUnichar *aFromType, const PRUnichar *
     return NS_OK;
 }
 
-#define ERR_OUT { nsMemory::Free(buffer); return rv; }
+#define ERR_OUT { free(buffer); return rv; }
 
 // nsIStreamListener implementation
 NS_IMETHODIMP
 nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
                                   nsIInputStream *inStr, PRUint32 sourceOffset, PRUint32 count) {
 
-    if (!mToken.get()) // no token, no love.
+    if (mToken.IsEmpty()) // no token, no love.
         return NS_ERROR_FAILURE;
 
     nsresult rv = NS_OK;
     char *buffer = nsnull;
-    PRUint32 bufLen = count, read;
+    PRUint32 bufLen = 0, read = 0;
 
     NS_ASSERTION(request, "multimixed converter needs a request");
 
     nsCOMPtr<nsIChannel> channel = do_QueryInterface(request, &rv);
     if (NS_FAILED(rv)) return rv;
-    
-    buffer = (char*)nsMemory::Alloc(bufLen);
-    if (!buffer) return NS_ERROR_OUT_OF_MEMORY;
 
-    rv = inStr->Read(buffer, bufLen, &read);
-    if (NS_FAILED(rv) || read == 0) return rv;
-    NS_ASSERTION(read == bufLen, "poor data size assumption");
-    
-    if (mBufLen) {
-		// incorporate any buffered data into the parsing
-        char *tmp = (char*)nsMemory::Alloc(mBufLen + bufLen);
-        if (!tmp) {
-            nsMemory::Free(buffer);
+    // fill buffer
+    {
+        bufLen = count + mBufLen;
+        buffer = (char *) malloc(bufLen);
+        if (!buffer)
             return NS_ERROR_OUT_OF_MEMORY;
+
+        if (mBufLen) {
+            // incorporate any buffered data into the parsing
+            memcpy(buffer, mBuffer, mBufLen);
+            free(mBuffer);
+            mBuffer = 0;
         }
-        memcpy(tmp, mBuffer, mBufLen);
-        nsMemory::Free(mBuffer);
-        mBuffer = nsnull;
-        memcpy(tmp+mBufLen, buffer, bufLen);
-        nsMemory::Free(buffer);
-        buffer = tmp;
-        bufLen += mBufLen;
+        
+        rv = inStr->Read(buffer + mBufLen, count, &read);
+
         mBufLen = 0;
+
+        if (NS_FAILED(rv) || read == 0) return rv;
+        NS_ASSERTION(read == count, "poor data size assumption");
     }
 
     char *cursor = buffer;
@@ -443,7 +470,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         // boundary token in for the server.
         mFirstOnData = PR_FALSE;
         NS_ASSERTION(!mBufLen, "this is our first time through, we can't have buffered data");
-        const char * token = mToken;
+        const char * token = mToken.get();
            
         PushOverLine(cursor, bufLen);
 
@@ -452,18 +479,18 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             // skip this check, and try again the next time OnData()
             // is called.
             mFirstOnData = PR_TRUE;
-        } else if (!PL_strnstr(cursor, token, mTokenLen+2)) {
-            bufLen = read + mTokenLen + 1;
-            char *tmp = (char*)nsMemory::Alloc(bufLen);
-            if (!tmp) {
-                nsMemory::Free(buffer);
+        }
+        else if (!PL_strnstr(cursor, token, mTokenLen+2)) {
+            buffer = (char *) realloc(buffer, bufLen + mTokenLen + 1);
+            if (!buffer)
                 return NS_ERROR_OUT_OF_MEMORY;
-            }
-            memcpy(tmp, token, mTokenLen);
-            memcpy(tmp+mTokenLen, "\n", 1);
-            memcpy(tmp+mTokenLen+1, cursor, read);
-            nsMemory::Free(buffer);
-            buffer = tmp;
+
+            memmove(buffer+mTokenLen+1, buffer, bufLen);
+            memcpy(buffer, token, mTokenLen);
+            buffer[mTokenLen] = '\n';
+
+            bufLen += (mTokenLen + 1);
+
             // need to reset cursor to the buffer again (bug 100595)
             cursor = buffer;
         }
@@ -491,12 +518,9 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
 
         if (*(token+mTokenLen+1) == '-') {
 			// This was the last delimiter so we can stop processing
-            bufLen = token - cursor;
-            rv = SendData(cursor, bufLen);
-            nsMemory::Free(buffer);
+            rv = SendData(cursor, token - cursor);
+            free(buffer);
             if (NS_FAILED(rv)) return rv;
-            buffer = nsnull;
-            bufLen = 0;
             return SendStop(NS_OK);
         }
 
@@ -507,6 +531,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             bufLen -= token - cursor;
             if (NS_FAILED(rv)) ERR_OUT
         }
+        // XXX else NS_ASSERTION(token == cursor, "?");
         token += mTokenLen;
         bufLen -= mTokenLen;
         tokenLinefeed = PushOverLine(token, bufLen);
@@ -521,19 +546,20 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
             if (done) {
                 rv = SendStart(channel);
                 if (NS_FAILED(rv)) ERR_OUT
-            } else {
+            }
+            else {
                 // we haven't finished processing header info.
                 // we'll break out and try to process later.
                 mProcessingHeaders = PR_TRUE;
                 break;
             }
-        } else {
+        }
+        else {
             mNewPart = PR_TRUE;
             rv = SendStop(NS_OK);
             if (NS_FAILED(rv)) ERR_OUT
-            // reset to the token to front.
-            // this allows us to treat the token
-            // as a starting token.
+            // reset the token to front. this allows us to treat
+            // the token as a starting token.
             token -= mTokenLen + tokenLinefeed;
             bufLen += mTokenLen + tokenLinefeed;
             cursor = token;
@@ -546,9 +572,9 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
 
     // carry over
     PRUint32 bufAmt = 0;
-    if (mProcessingHeaders) {
+    if (mProcessingHeaders)
         bufAmt = bufLen;
-    } else {
+    else if (bufLen) {
         // if the data ends in a linefeed, and we're in the middle
         // of a "part" (ie. mPartChannel exists) don't bother
         // buffering, go ahead and send the data we have. Otherwise
@@ -570,8 +596,7 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
         if (NS_FAILED(rv)) ERR_OUT
     }
 
-    nsMemory::Free(buffer);
-
+    free(buffer);
     return rv;
 }
 
@@ -580,9 +605,9 @@ nsMultiMixedConv::OnDataAvailable(nsIRequest *request, nsISupports *context,
 NS_IMETHODIMP
 nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
 	// we're assuming the content-type is available at this stage
-	NS_ASSERTION(!mToken, "a second on start???");
-    char *bndry = nsnull;
-    nsXPIDLCString delimiter;
+	NS_ASSERTION(mToken.IsEmpty(), "a second on start???");
+    const char *bndry = nsnull;
+    nsCAutoString delimiter;
     nsresult rv = NS_OK;
     mContext = ctxt;
 
@@ -595,24 +620,23 @@ nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
     // ask the HTTP channel for the content-type and extract the boundary from it.
     nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(channel, &rv);
     if (NS_SUCCEEDED(rv)) {
-        rv = httpChannel->GetResponseHeader("content-type", getter_Copies(delimiter));
+        rv = httpChannel->GetResponseHeader(NS_LITERAL_CSTRING("content-type"), delimiter);
         if (NS_FAILED(rv)) return rv;
     } else {
         // try asking the channel directly
-        rv = channel->GetContentType(getter_Copies(delimiter));
-        if (!delimiter || NS_FAILED(rv)) return NS_ERROR_FAILURE;
+        rv = channel->GetContentType(delimiter);
+        if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
     }
 
-    if (!delimiter) return NS_ERROR_FAILURE;
-    bndry = PL_strstr(delimiter, "boundary");
+    bndry = strstr(delimiter.get(), "boundary");
     if (!bndry) return NS_ERROR_FAILURE;
 
-    bndry = PL_strchr(bndry, '=');
+    bndry = strchr(bndry, '=');
     if (!bndry) return NS_ERROR_FAILURE;
 
     bndry++; // move past the equals sign
 
-    char *attrib = PL_strchr(bndry, ';');
+    char *attrib = (char *) strchr(bndry, ';');
     if (attrib) *attrib = '\0';
 
     nsCAutoString boundaryString(bndry);
@@ -620,10 +644,10 @@ nsMultiMixedConv::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
 
     boundaryString.Trim(" \"");
 
-    mToken.Adopt(nsCRT::strdup(boundaryString.get()));
+    mToken = boundaryString;
     mTokenLen = boundaryString.Length();
     
-    if (mTokenLen == 0 || !boundaryString.get())
+    if (mTokenLen == 0)
         return NS_ERROR_FAILURE;
 
     return NS_OK;
@@ -633,7 +657,7 @@ NS_IMETHODIMP
 nsMultiMixedConv::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
                                 nsresult aStatus) {
 
-    if (!mToken.get())  // no token, no love.
+    if (mToken.IsEmpty())  // no token, no love.
         return NS_ERROR_FAILURE;
 
     if (mPartChannel) {
@@ -643,7 +667,7 @@ nsMultiMixedConv::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
             (void) SendData(mBuffer, mBufLen);
             // don't bother checking the return value here, if the send failed
             // we're done anyway as we're in the OnStop() callback.
-            nsMemory::Free(mBuffer);
+            free(mBuffer);
             mBuffer = nsnull;
             mBufLen = 0;
         }
@@ -683,7 +707,7 @@ nsMultiMixedConv::nsMultiMixedConv() {
 nsMultiMixedConv::~nsMultiMixedConv() {
     NS_ASSERTION(!mBuffer, "all buffered data should be gone");
     if (mBuffer) {
-        nsMemory::Free(mBuffer);
+        free(mBuffer);
         mBuffer = nsnull;
     }
 }
@@ -692,7 +716,7 @@ nsresult
 nsMultiMixedConv::BufferData(char *aData, PRUint32 aLen) {
     NS_ASSERTION(!mBuffer, "trying to over-write buffer");
 
-    char *buffer = (char*)nsMemory::Alloc(aLen);
+    char *buffer = (char *) malloc(aLen);
     if (!buffer) return NS_ERROR_OUT_OF_MEMORY;
 
     memcpy(buffer, aData, aLen);
@@ -707,7 +731,7 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
     nsresult rv = NS_OK;
 
     if (mContentType.IsEmpty())
-        mContentType = UNKNOWN_CONTENT_TYPE;
+        mContentType = NS_LITERAL_CSTRING(UNKNOWN_CONTENT_TYPE);
 
     // if we already have an mPartChannel, that means we never sent a Stop()
     // before starting up another "part." that would be bad.
@@ -727,12 +751,18 @@ nsMultiMixedConv::SendStart(nsIChannel *aChannel) {
     // Set up the new part channel...
     mPartChannel = newChannel;
 
-    rv = mPartChannel->SetContentType(mContentType.get());
+    rv = mPartChannel->SetContentType(mContentType);
     if (NS_FAILED(rv)) return rv;
 
-    mPartChannel->SetContentLength(mContentLength);
+    rv = mPartChannel->SetContentLength(mContentLength);
     if (NS_FAILED(rv)) return rv;
 
+    nsCOMPtr<nsIMultiPartChannel> partChannel(do_QueryInterface(mPartChannel));
+    if (partChannel) {
+        rv = partChannel->SetContentDisposition(mContentDisposition);
+        if (NS_FAILED(rv)) return rv;
+    }
+
     nsLoadFlags loadFlags = 0;
     mPartChannel->GetLoadFlags(&loadFlags);
     loadFlags |= nsIChannel::LOAD_REPLACE;
@@ -780,51 +810,44 @@ nsMultiMixedConv::SendData(char *aBuffer, PRUint32 aLen) {
     
     if (!mPartChannel) return NS_ERROR_FAILURE; // something went wrong w/ processing
 
-
     if (mContentLength != -1) {
         // make sure that we don't send more than the mContentLength
-        if ((aLen + mTotalSent) > mContentLength)
+        // XXX why? perhaps the Content-Length header was actually wrong!!
+        if ((aLen + mTotalSent) > PRUint32(mContentLength))
             aLen = mContentLength - mTotalSent;
 
         if (aLen == 0)
             return NS_OK;
     }
 
+    PRUint32 offset = mTotalSent;
     mTotalSent += aLen;
 
-    char *tmp = (char*)nsMemory::Alloc(aLen); // byteArray stream owns this mem
-    if (!tmp) return NS_ERROR_OUT_OF_MEMORY;
-    
-    memcpy(tmp, aBuffer, aLen);
-    
-    nsCOMPtr<nsIByteArrayInputStream> byteArrayStream;
-    
-    rv = NS_NewByteArrayInputStream(getter_AddRefs(byteArrayStream), tmp, aLen);
-    if (NS_FAILED(rv)) {
-        nsMemory::Free(tmp);
+    nsCOMPtr<nsIStringInputStream> ss(
+            do_CreateInstance("@mozilla.org/io/string-input-stream;1", &rv));
+    if (NS_FAILED(rv))
         return rv;
-    }
 
-    nsCOMPtr<nsIInputStream> inStream = do_QueryInterface(byteArrayStream, &rv);
+    rv = ss->ShareData(aBuffer, aLen);
+    if (NS_FAILED(rv))
+        return rv;
+
+    nsCOMPtr<nsIInputStream> inStream(do_QueryInterface(ss, &rv));
     if (NS_FAILED(rv)) return rv;
 
-    PRUint32 len;
-    rv = inStream->Available(&len);
-    if (NS_FAILED(rv)) return rv;
-
-    return mFinalListener->OnDataAvailable(mPartChannel, mContext, inStream, 0, len);
+    return mFinalListener->OnDataAvailable(mPartChannel, mContext, inStream, offset, aLen);
 }
 
 PRInt32
 nsMultiMixedConv::PushOverLine(char *&aPtr, PRUint32 &aLen) {
     PRInt32 chars = 0;
-    if (*aPtr == nsCRT::CR || *aPtr == nsCRT::LF) {
-        if (aPtr[1] == nsCRT::LF)
+    if ((aLen > 0) && (*aPtr == nsCRT::CR || *aPtr == nsCRT::LF)) {
+        if ((aLen > 1) && (aPtr[1] == nsCRT::LF))
             chars++;
         chars++;
+        aPtr += chars;
+        aLen -= chars;
     }
-    aPtr += chars;
-    aLen -= chars;
     return chars;
 }
 
@@ -832,36 +855,37 @@ nsresult
 nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr, 
                                PRUint32 &aLen, PRBool *_retval) {
     // NOTE: this data must be ascii.
+    // NOTE: aPtr is NOT null terminated!
 	nsresult rv = NS_OK;
     char *cursor = aPtr, *newLine = nsnull;
     PRUint32 cursorLen = aLen;
     PRBool done = PR_FALSE;
     PRUint32 lineFeedIncrement = 1;
     
-    mContentLength = -1;
-    while ( (cursorLen > 0) 
-            && (newLine = PL_strchr(cursor, nsCRT::LF)) ) {
+    mContentLength = -1; // XXX what if we were already called?
+    while (cursorLen && (newLine = (char *) memchr(cursor, nsCRT::LF, cursorLen))) {
         // adjust for linefeeds
-        if ( (newLine > cursor) && (newLine[-1] == nsCRT::CR) ) { // CRLF
-             lineFeedIncrement = 2;
-             newLine--;
+        if ((newLine > cursor) && (newLine[-1] == nsCRT::CR) ) { // CRLF
+            lineFeedIncrement = 2;
+            newLine--;
         }
+        else
+            lineFeedIncrement = 1; // reset
 
         if (newLine == cursor) {
-            // move the newLine beyond the double linefeed marker
-            if ( (newLine - cursor) <= ((PRInt32)cursorLen - lineFeedIncrement) ) {
-                newLine += lineFeedIncrement;
-            }
-            cursorLen -= (newLine - cursor);
-            cursor = newLine;
+            // move the newLine beyond the linefeed marker
+            NS_ASSERTION(cursorLen >= lineFeedIncrement, "oops!");
+
+            cursor += lineFeedIncrement;
+            cursorLen -= lineFeedIncrement;
 
             done = PR_TRUE;
             break;
         }
 
         char tmpChar = *newLine;
-        *newLine = '\0';
-        char *colon = PL_strchr(cursor, ':');
+        *newLine = '\0'; // cursor is now null terminated
+        char *colon = (char *) strchr(cursor, ':');
         if (colon) {
             *colon = '\0';
             nsCAutoString headerStr(cursor);
@@ -873,18 +897,18 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
 
             // examine header
             if (headerStr.EqualsIgnoreCase("content-type")) {
-                char *tmpString = ToNewCString(headerVal);
-                ParseContentType(tmpString);
-                nsCRT::free(tmpString);
+                mContentType = headerVal;
             } else if (headerStr.EqualsIgnoreCase("content-length")) {
                 mContentLength = atoi(headerVal.get());
+            } else if (headerStr.EqualsIgnoreCase("content-disposition")) {
+                mContentDisposition = headerVal;
             } else if (headerStr.EqualsIgnoreCase("set-cookie")) {
                 // setting headers on the HTTP channel
                 // causes HTTP to notify, again if necessary,
                 // it's header observers.
                 nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
                 if (httpChannel) {
-                    rv = httpChannel->SetResponseHeader(headerStr.get(), headerVal.get());
+                    rv = httpChannel->SetResponseHeader(headerStr, headerVal);
                     if (NS_FAILED(rv)) return rv;
                 }
             } else if (headerStr.EqualsIgnoreCase("content-range") || 
@@ -892,12 +916,12 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
                 // something like: Content-range: bytes 7000-7999/8000
                 char* tmpPtr;
 
-                tmpPtr = PL_strchr(colon + 1, '/');
+                tmpPtr = (char *) strchr(colon + 1, '/');
                 if (tmpPtr) 
                     *tmpPtr = '\0';
 
                 // pass the bytes-unit and the SP
-                char *range = PL_strchr(colon + 2, ' ');
+                char *range = (char *) strchr(colon + 2, ' ');
 
                 if (!range)
                     return NS_ERROR_FAILURE;
@@ -906,7 +930,7 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
                     mByteRangeStart = mByteRangeEnd = 0;
                 }
                 else {
-                    tmpPtr = PL_strchr(range, '-');
+                    tmpPtr = (char *) strchr(range, '-');
                     if (!tmpPtr)
                         return NS_ERROR_FAILURE;
                     
@@ -926,7 +950,7 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
         newLine += lineFeedIncrement;
         cursorLen -= (newLine - cursor);
         cursor = newLine;
-    } // end while (newLine)
+    }
 
     aPtr = cursor;
     aLen = cursorLen;
@@ -935,86 +959,33 @@ nsMultiMixedConv::ParseHeaders(nsIChannel *aChannel, char *&aPtr,
     return rv;
 }
 
-// This code is duplicated in nsHttpResponseHead.cpp.  If you change it
-// here, change it there, too!
-
-nsresult
-nsMultiMixedConv::ParseContentType(char *type)
-{
-    // don't bother with an empty content-type header - bug 83465
-    if (!*type)
-        return NS_OK;
-
-    // we don't care about comments (although they are invalid here)
-    char *p = PL_strchr(type, '(');
-    if (p)
-        *p = 0;
-
-    // check if the content-type has additional fields...
-    if ((p = PL_strchr(type, ';')) != nsnull) {
-        char *p2, *p3;
-        // is there a charset field?
-        if ((p2 = PL_strcasestr(p, "charset=")) != nsnull) {
-            p2 += 8;
-
-            // check end of charset parameter
-            if ((p3 = PL_strchr(p2, ';')) == nsnull)
-                p3 = p2 + PL_strlen(p2);
-
-            // trim any trailing whitespace
-            do {
-                --p3;
-            } while ((*p3 == ' ') || (*p3 == '\t'));
-            *++p3 = 0; // overwrite first char after the charset field
-
-            mContentCharset = p2;
-        }
-    }
-    else
-        p = type + PL_strlen(type);
-
-    // trim any trailing whitespace
-    while (--p >= type && ((*p == ' ') || (*p == '\t')))
-        ;
-    *++p = 0; // overwrite first char after the media type
-
-    // force the content-type to lowercase
-    while (--p >= type)
-        *p = nsCRT::ToLower(*p);
-
-    mContentType = type;
-
-    return NS_OK;
-}
-
 char *
 nsMultiMixedConv::FindToken(char *aCursor, PRUint32 aLen) {
     // strnstr without looking for null termination
-    const char *token = mToken;
+    const char *token = mToken.get();
     char *cur = aCursor;
 
-    NS_ASSERTION(token && aCursor && *token, "bad data");
+    if (!(token && aCursor && *token)) {
+        NS_WARNING("bad data");
+        return nsnull;
+    }
 
-    if( mTokenLen > aLen ) return nsnull;
+    for (; aLen >= mTokenLen; aCursor++, aLen--) {
+        if (!memcmp(aCursor, token, mTokenLen) ) {
+            if ((aCursor - cur) >= 2) {
+                // back the cursor up over a double dash for backwards compat.
+                if ((*(aCursor-1) == '-') && (*(aCursor-2) == '-')) {
+                    aCursor -= 2;
+                    aLen += 2;
 
-    for( ; aLen >= mTokenLen; aCursor++, aLen-- )
-        if( *token == *aCursor)
-            if(!memcmp(aCursor, token, mTokenLen) ) {
-                if ( (aCursor - cur) >= 2) {
-                    // back the cursor up over a double dash for backwards compat.
-                    if ( (*(aCursor-1) == '-') && (*(aCursor-2) == '-') ) {
-                        aCursor -= 2;
-                        aLen += 2;
-
-                        // we're playing w/ double dash tokens, adjust.
-                        nsCString newToken("--");
-                        newToken.Append(mToken);
-                        mToken.Adopt(nsCRT::strdup(newToken.get()));
-                        mTokenLen += 2;
-                    }
+                    // we're playing w/ double dash tokens, adjust.
+                    mToken.Assign(aCursor, mTokenLen + 2);
+                    mTokenLen = mToken.Length();
                 }
-                return aCursor;
             }
+            return aCursor;
+        }
+    }
 
     return nsnull;
 }
