@@ -39,7 +39,7 @@
     The TestProtocols tests the basic protocols architecture and can 
     be used to test individual protocols as well. If this grows too
     big then we should split it to individual protocols. 
-    
+
     -Gagan Saksena 04/29/99
 */
 
@@ -58,6 +58,8 @@
 #include "nsIInputStream.h"
 #include "nsCRT.h"
 #include "nsIChannel.h"
+#include "nsIResumableChannel.h"
+#include "nsIResumableEntityID.h"
 #include "nsIURL.h"
 #include "nsIHttpChannel.h"
 #include "nsIHttpEventSink.h" 
@@ -77,6 +79,8 @@ static int gKeepRunning = 0;
 static PRBool gVerbose = PR_FALSE;
 static nsIEventQueue* gEventQ = nsnull;
 static PRBool gAskUserForInput = PR_FALSE;
+static PRBool gResume = PR_FALSE;
+static PRUint32 gStartAt = 0;
 
 //-----------------------------------------------------------------------------
 // HeaderVisitor
@@ -94,9 +98,11 @@ public:
 NS_IMPL_ISUPPORTS1(HeaderVisitor, nsIHttpHeaderVisitor)
 
 NS_IMETHODIMP
-HeaderVisitor::VisitHeader(const char *header, const char *value)
+HeaderVisitor::VisitHeader(const nsACString &header, const nsACString &value)
 {
-  printf("%s: %s\n", header, value);
+  printf("%s: %s\n",
+    PromiseFlatCString(header).get(),
+    PromiseFlatCString(value).get());
   return NS_OK;
 }
 
@@ -202,7 +208,7 @@ InputTestConsumer::~InputTestConsumer()
 {
 }
 
-NS_IMPL_ISUPPORTS1(InputTestConsumer, nsIStreamListener)
+NS_IMPL_ISUPPORTS2(InputTestConsumer, nsIStreamListener, nsIRequestObserver)
 
 NS_IMETHODIMP
 InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
@@ -214,7 +220,7 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
   if (gVerbose)
     printf("\nStarted loading: %s\n", info ? info->Name() : "UNKNOWN URL");
 
-  nsXPIDLCString value;
+  nsCAutoString value;
 
   nsCOMPtr<nsIChannel> channel = do_QueryInterface(request);
   if (channel) {
@@ -222,8 +228,12 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
     channel->GetStatus(&status);
     if (NS_SUCCEEDED(status)) {
       printf("Channel Info:\n");
-      channel->GetContentType(getter_Copies(value));
-      printf("\tContent-Type: %s\n", (const char*)value);
+
+      channel->GetContentType(value);
+      printf("\tContent-Type: %s\n", value.get());
+
+      channel->GetContentCharset(value);
+      printf("\tContent-Charset: %s\n", value.get());
 
       PRInt32 length = -1;
       if (NS_SUCCEEDED(channel->GetContentLength(&length)))
@@ -240,9 +250,6 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
       return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(visitor);
 
-    httpChannel->GetCharset(getter_Copies(value));
-    printf("\tCharset: %s\n", (const char*)value);
-
     printf("\nHTTP request headers:\n");
     httpChannel->VisitRequestHeaders(visitor);
 
@@ -251,6 +258,34 @@ InputTestConsumer::OnStartRequest(nsIRequest *request, nsISupports* context)
 
     NS_RELEASE(visitor);
   }
+
+  nsCOMPtr<nsIResumableChannel> resChannel = do_QueryInterface(request);
+  if (resChannel) {
+      printf("Resumable entity identification:\n");
+      nsCOMPtr<nsIResumableEntityID> entityID;
+      nsresult rv = resChannel->GetEntityID(getter_AddRefs(entityID));
+      if (NS_SUCCEEDED(rv) && entityID) {
+          PRUint32 size;
+          if (NS_SUCCEEDED(entityID->GetSize(&size)) &&
+              size != PRUint32(-1))
+              printf("\tSize: %d\n", size);
+          else
+              printf("\tSize: Unknown\n");
+          PRTime lastModified;
+          if (NS_SUCCEEDED(entityID->GetLastModified(&lastModified)) &&
+              lastModified != -1) {
+              PRExplodedTime exploded;
+              PR_ExplodeTime(lastModified, PR_LocalTimeParameters, &exploded);
+
+              char buf[100];
+              PR_FormatTime(buf, 100, "%c", &exploded);
+
+              printf("\tLast Modified: %s\n", buf);
+          } else
+              printf("\tLast Modified: Unknown\n");
+      }
+  }
+
   return NS_OK;
 }
 
@@ -401,7 +436,7 @@ nsresult StartLoadingURL(const char* aUrlString)
     if (pService) {
         nsCOMPtr<nsIURI> pURL;
 
-        rv = pService->NewURI(aUrlString, nsnull, getter_AddRefs(pURL));
+        rv = pService->NewURI(nsDependentCString(aUrlString), nsnull, nsnull, getter_AddRefs(pURL));
         if (NS_FAILED(rv)) {
             printf("ERROR: NewURI failed for %s\n", aUrlString);
             return rv;
@@ -416,9 +451,9 @@ nsresult StartLoadingURL(const char* aUrlString)
         NS_ADDREF(callbacks);
 
         // Async reading thru the calls of the event sink interface
-        rv = NS_OpenURI(getter_AddRefs(pChannel), pURL, pService,
-                        nsnull,     // loadGroup
-                        callbacks); // notificationCallbacks
+        rv = NS_NewChannel(getter_AddRefs(pChannel), pURL, pService,
+                           nsnull,     // loadGroup
+                           callbacks); // notificationCallbacks
         NS_RELEASE(callbacks);
         if (NS_FAILED(rv)) {
             printf("ERROR: NS_OpenURI failed for %s\n", aUrlString);
@@ -434,7 +469,8 @@ nsresult StartLoadingURL(const char* aUrlString)
 
         if (pHTTPCon) {
             // Setting a sample header.
-            rv = pHTTPCon->SetRequestHeader("sample-header", "Sample-Value");
+            rv = pHTTPCon->SetRequestHeader(NS_LITERAL_CSTRING("sample-header"),
+                                            NS_LITERAL_CSTRING("Sample-Value"));
             if (NS_FAILED(rv)) return rv;
         }            
         InputTestConsumer* listener;
@@ -453,9 +489,21 @@ nsresult StartLoadingURL(const char* aUrlString)
             NS_ERROR("Failed to create a load info!");
             return NS_ERROR_OUT_OF_MEMORY;
         }
-        
-        rv = pChannel->AsyncOpen(listener,  // IStreamListener consumer
-                                 info);
+
+        if (gResume) {
+            nsCOMPtr<nsIResumableChannel> res = do_QueryInterface(pChannel);
+            if (!res) {
+                NS_ERROR("Channel is not resumable!");
+                return NS_ERROR_UNEXPECTED;
+            }
+            rv = res->AsyncOpenAt(listener,
+                                  info,
+                                  gStartAt,
+                                  nsnull);
+        } else {            
+            rv = pChannel->AsyncOpen(listener,  // IStreamListener consumer
+                                     info);
+        }
 
         if (NS_SUCCEEDED(rv)) {
             gKeepRunning += 1;
@@ -507,7 +555,7 @@ nsresult LoadURLsFromFile(char *aFileName)
         printf("\t%s\n", fileBuffer.get());
         StartLoadingURL(fileBuffer.get());
     }
-    
+
     PR_Close(fd);
     return NS_OK;
 }
@@ -542,61 +590,66 @@ main(int argc, char* argv[])
     rv = NS_InitXPCOM2(nsnull, nsnull, nsnull);
     if (NS_FAILED(rv)) return rv;
 
-    
-    // Create the Event Queue for this thread...
-    nsCOMPtr<nsIEventQueueService> eventQService = 
-             do_GetService(kEventQueueServiceCID, &rv);
-    if (NS_FAILED(rv)) return rv;
+    {
+        // Create the Event Queue for this thread...
+        nsCOMPtr<nsIEventQueueService> eventQService =
+                 do_GetService(kEventQueueServiceCID, &rv);
+        if (NS_FAILED(rv)) return rv;
 
-    rv = eventQService->CreateThreadEventQueue();
-    if (NS_FAILED(rv)) return rv;
+        eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, &gEventQ);
 
-    eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, &gEventQ);
+        int i;
+        printf("\nTrying to load:\n");
+        for (i=1; i<argc; i++) {
+            // Turn on verbose printing...
+            if (PL_strcasecmp(argv[i], "-verbose") == 0) {
+                gVerbose = PR_TRUE;
+                continue;
+            }
 
-    int i;
-    printf("\nTrying to load:\n");
-    for (i=1; i<argc; i++) {
-        // Turn on verbose printing...
-        if (PL_strcasecmp(argv[i], "-verbose") == 0) {
-            gVerbose = PR_TRUE;
-            continue;
-        } 
+            // Turn on netlib tracing...
+            if (PL_strcasecmp(argv[i], "-file") == 0) {
+                LoadURLsFromFile(argv[++i]);
+                continue;
+            }
 
-        // Turn on netlib tracing...
-        if (PL_strcasecmp(argv[i], "-file") == 0) {
-            LoadURLsFromFile(argv[++i]);
-            continue;
-        } 
-        
-        if (PL_strcasecmp(argv[i], "-console") == 0) {
-            gAskUserForInput = PR_TRUE;
-            continue;
-        } 
-           
-        printf("\t%s\n", argv[i]);
-        rv = StartLoadingURL(argv[i]);
-    }
-  // Enter the message pump to allow the URL load to proceed.
-    while ( gKeepRunning ) {
+            if (PL_strcasecmp(argv[i], "-console") == 0) {
+                gAskUserForInput = PR_TRUE;
+                continue;
+            }
+
+            if (PL_strcasecmp(argv[i], "-resume") == 0) {
+                gResume = PR_TRUE;
+                gStartAt = atoi(argv[++i]);
+                continue;
+            }
+
+            printf("\t%s\n", argv[i]);
+            rv = StartLoadingURL(argv[i]);
+        }
+      // Enter the message pump to allow the URL load to proceed.
+        while ( gKeepRunning ) {
 #ifdef WIN32
-        MSG msg;
+            MSG msg;
 
-        if (GetMessage(&msg, NULL, 0, 0)) {
-            TranslateMessage(&msg);
-            DispatchMessage(&msg);
-        } else {
-            gKeepRunning = 0;
-    }
+            if (GetMessage(&msg, NULL, 0, 0)) {
+                TranslateMessage(&msg);
+                DispatchMessage(&msg);
+            } else {
+                gKeepRunning = 0;
+            }
 #else
 #ifdef XP_MAC
-    /* Mac stuff is missing here! */
+            /* Mac stuff is missing here! */
 #else
-    PLEvent *gEvent;
-    rv = gEventQ->WaitForEvent(&gEvent);
-    rv = gEventQ->HandleEvent(gEvent);
+            PLEvent *gEvent;
+            rv = gEventQ->WaitForEvent(&gEvent);
+            rv = gEventQ->HandleEvent(gEvent);
 #endif /* XP_UNIX */
 #endif /* !WIN32 */
-    }
+        }
+    } // this scopes the nsCOMPtrs
+    // no nsCOMPtrs are allowed to be alive when you call NS_ShutdownXPCOM
     NS_ShutdownXPCOM(nsnull);
     return rv;
 }
