@@ -42,15 +42,12 @@
 
 #include "LiveConnectNativeMethods.h"
 
-#include "nsIComponentManager.h"
+#include "nsIComponentManagerObsolete.h"
 #include "nsIServiceManager.h"
 #include "nsIPluginManager.h"
 #include "nsIJVMManager.h"
 #include "nsILiveconnect.h"
 #include "nsIPluginInstancePeer2.h"
-#include "nsIIOService.h"
-#include "nsIURI.h"
-#include "nsNetCID.h"
 
 #include "MRJPlugin.h"
 #include "MRJContext.h"
@@ -66,11 +63,10 @@
 #include "netscape_javascript_JSObject.h"   /* javah-generated headers */
 
 extern nsIPluginManager* thePluginManager;
-extern nsIServiceManager* theServiceManager;    // needs to be in badaptor.cpp.
 
 static MRJPlugin* theJVMPlugin = NULL;
 static nsILiveconnect* theLiveConnectManager = NULL;
-static nsIComponentManager* theComponentManager = NULL;
+static nsIComponentManagerObsolete* theComponentManager = NULL;
 
 static jclass netscape_javascript_JSObject = NULL;
 static jmethodID netscape_javascript_JSObject_JSObject;
@@ -85,10 +81,9 @@ static jmethodID netscape_oji_JNIUtils_GetObjectClassLoader = NULL;
 static NS_DEFINE_IID(kLiveConnectCID, NS_CLIVECONNECT_CID);
 static NS_DEFINE_IID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);
 
-static nsresult getGlobalComponentManager(nsIComponentManager* *result)
+static nsresult getGlobalComponentManager(nsIComponentManagerObsolete* *result)
 {
-    return theServiceManager->GetService(kComponentManagerCID, NS_GET_IID(nsIComponentManager),
-                                         (void**)result);
+    return MRJPlugin::GetService(kComponentManagerCID, NS_GET_IID(nsIComponentManagerObsolete), (void**)result);
 }
 
 nsresult InitLiveConnectSupport(MRJPlugin* jvmPlugin)
@@ -97,7 +92,7 @@ nsresult InitLiveConnectSupport(MRJPlugin* jvmPlugin)
 
     getGlobalComponentManager(&theComponentManager);
 
-    nsresult result = theServiceManager->GetService(kLiveConnectCID, NS_GET_IID(nsILiveconnect),
+    nsresult result = MRJPlugin::GetService(kLiveConnectCID, NS_GET_IID(nsILiveconnect),
                                                     (void**)&theLiveConnectManager);
     if (result != NS_OK)
         return result;
@@ -155,7 +150,6 @@ nsresult InitLiveConnectSupport(MRJPlugin* jvmPlugin)
 nsresult ShutdownLiveConnectSupport()
 {
     NS_IF_RELEASE(theLiveConnectManager);
-
     NS_IF_RELEASE(theComponentManager);
     
     if (theJVMPlugin != NULL) {
@@ -204,26 +198,22 @@ static jobject GetCurrentThread(JNIEnv* env)
  * Security Considerations.
  */
 
-static nsresult
-NS_NewURI(nsIURI* *result, 
-          const char* spec, 
-          nsIURI* baseURI = nsnull)     // pass in nsIIOService to optimize callers
-{
-    nsIIOService* ioService;
-    static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
-    nsresult rv = theServiceManager->GetService(kIOServiceCID, NS_GET_IID(nsIIOService), (void**)&ioService);
-    if (rv == NS_OK)
-        rv = ioService->NewURI(spec, baseURI, result);
-    NS_RELEASE(ioService);
-    return rv;
-}
-
 MRJSecurityContext::MRJSecurityContext(const char* location)
     :   mLocation(nsnull), mConnection(nsnull)
 {
     NS_INIT_REFCNT();
     
-    NS_NewURI(&mLocation, location, nsnull);
+    mLocation = ::strdup(location);
+    if (mLocation) {
+        // find the 3rd slash of the URL, hopefully it's in canonical form.
+        char* colon = ::strchr(mLocation, ':');
+        if (colon) {
+            if (colon[1] == '/' && colon[2] == '/') {
+                char* slash = ::strchr(colon + 3, '/');
+                if (slash) *slash = '\0';
+            }
+        }
+    }
 
     if (theComponentManager) {
         theComponentManager->CreateInstance(kLiveConnectCID, nsnull, NS_GET_IID(nsILiveconnect),
@@ -236,7 +226,7 @@ MRJSecurityContext::MRJSecurityContext(const char* location)
 
 MRJSecurityContext::~MRJSecurityContext()
 {
-    NS_IF_RELEASE(mLocation);
+    delete[] mLocation;
     NS_IF_RELEASE(mConnection);
 }
 
@@ -252,10 +242,8 @@ NS_METHOD MRJSecurityContext::Implies(const char* target, const char* action, PR
 NS_METHOD 
 MRJSecurityContext::GetOrigin(char* buf, int len)
 {
-    char* origin = nsnull;
-    if (mLocation && NS_SUCCEEDED(mLocation->GetPrePath(&origin))) {
-        ::strncpy(buf, origin, len);
-        delete[] origin;
+    if (mLocation) {
+        ::strncpy(buf, mLocation, len);
         return NS_OK;
     }
     return NS_ERROR_FAILURE;
@@ -265,7 +253,6 @@ NS_METHOD
 MRJSecurityContext::GetCertificateID(char* buf, int len)
 {
     // ACTION: Implement me.
-
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
@@ -354,7 +341,7 @@ void MessageRunnable::execute(JNIEnv* env)
     // because a spontaneous Java thread called us, we have to switch to the JavaScript thread
     // to handle this request.
     nsIThreadManager* threadManager = NULL;
-    if (theServiceManager->GetService(nsIJVMManager::GetCID(), NS_GET_IID(nsIThreadManager), (void **)&threadManager) == NS_OK) {
+    if (MRJPlugin::GetService(nsIJVMManager::GetCID(), NS_GET_IID(nsIThreadManager), (void **)&threadManager) == NS_OK) {
         threadManager->PostEvent(mThreadID, this, PR_FALSE);
         NS_RELEASE(threadManager);
     }
@@ -363,7 +350,7 @@ void MessageRunnable::execute(JNIEnv* env)
 NS_IMETHODIMP MessageRunnable::Run()
 {
     nsIJVMManager* javaManager = NULL;
-    if (theServiceManager->GetService(nsIJVMManager::GetCID(), NS_GET_IID(nsIJVMManager), (void **)&javaManager) == NS_OK) {
+    if (MRJPlugin::GetService(nsIJVMManager::GetCID(), NS_GET_IID(nsIJVMManager), (void **)&javaManager) == NS_OK) {
         JNIEnv* proxyEnv = NULL;
         if (javaManager->GetProxyJNI(&proxyEnv) == NS_OK && proxyEnv != NULL)
             mMessage->execute(proxyEnv);
@@ -877,7 +864,6 @@ Java_netscape_javascript_JSObject_eval(JNIEnv* env,
 #else
     /* determine the plugin instance so we can obtain its codebase. */
     // beard: should file a bug with Apple that JMJNIToAWTContext doesn't work.
-    // MRJPluginInstance* pluginInstance = theJVMPlugin->getPluginInstance(env);
     MRJPluginInstance* pluginInstance = GetCurrentInstance(env);
 #if 0
     if (pluginInstance == NULL) {
@@ -963,6 +949,11 @@ public:
     {
     }
     
+    ~GetWindowMessage()
+    {
+        NS_RELEASE(mPluginInstance);
+    }
+    
     virtual void execute(JNIEnv* env)
     {
         MRJSecurityContext* securityContext = getSecurityContext(mPluginInstance);
