@@ -40,15 +40,15 @@
 
 #include "pratom.h"
 #include "nsIComponentManager.h"
-#include "nsIAppShellService.h"
-#include "nsAppShellCIDs.h"
 #include "nsIComponentManager.h"
 #include "nsIDialogParamBlock.h"
 #include "nsIServiceManager.h"
-#include "nsIWindowMediator.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptContext.h"
-#include "nsSpecialSystemDirectory.h"
+#include "nsILocalFile.h"
+#include "nsDirectoryServiceUtils.h"
+#include "nsDirectoryServiceDefs.h"
+#include "nsDependentString.h"
 #include "nsFileStream.h"
 #include "nsIFileSpec.h"
 #include "nsCOMPtr.h"
@@ -85,10 +85,8 @@
 #define DEBUG_UTF8_CONVERSION 1
 #endif 
 
-#define NS_IMPL_IDS
 #include "nsICharsetConverterManager.h"
 #include "nsIPlatformCharset.h"
-#undef NS_IMPL_IDS
 
 #define CHROME_STYLE nsIWebBrowserChrome::CHROME_ALL | nsIWebBrowserChrome::CHROME_CENTER_SCREEN
 #define MIGRATION_PROPERTIES_URL "chrome://communicator/locale/profile/migration.properties"
@@ -109,10 +107,12 @@
 #define NEWS_SUMMARY_SUFFIX_IN_4x ".snm"
 #define COOKIES_FILE_NAME_IN_4x "cookies"
 #define BOOKMARKS_FILE_NAME_IN_4x "bookmarks.html"
-#define HISTORY_FILE_NAME_IN_4x "history.dat"
 #define NEWSRC_PREFIX_IN_4x ".newsrc-"
 #define SNEWSRC_PREFIX_IN_4x ".snewsrc-"
 #define POPSTATE_FILE_IN_4x "popstate"
+#define PSM_CERT7_DB "cert7.db"
+#define PSM_KEY3_DB "key3.db"
+#define PSM_SECMODULE_DB "secmodule.db"
 #elif defined(XP_MAC)
 #define IMAP_MAIL_FILTER_FILE_NAME_IN_4x "<hostname> Rules"
 #define POP_MAIL_FILTER_FILE_NAME_IN_4x "Filter Rules"
@@ -120,8 +120,11 @@
 #define NEWS_SUMMARY_SUFFIX_IN_4x ".snm"
 #define COOKIES_FILE_NAME_IN_4x "MagicCookie"
 #define BOOKMARKS_FILE_NAME_IN_4x "Bookmarks.html"
-#define HISTORY_FILE_NAME_IN_4x "Netscape History"
 #define POPSTATE_FILE_IN_4x "Pop State"
+#define SECURITY_PATH "Security"
+#define PSM_CERT7_DB "Certificates7"
+#define PSM_KEY3_DB "Key Database3"
+#define PSM_SECMODULE_DB "Security Modules"
 #else /* XP_PC */
 #define IMAP_MAIL_FILTER_FILE_NAME_IN_4x "rules.dat"
 #define POP_MAIL_FILTER_FILE_NAME_IN_4x "rules.dat"
@@ -129,7 +132,9 @@
 #define NEWS_SUMMARY_SUFFIX_IN_4x ".snm"
 #define COOKIES_FILE_NAME_IN_4x "cookies.txt"
 #define BOOKMARKS_FILE_NAME_IN_4x "bookmark.htm"
-#define HISTORY_FILE_NAME_IN_4x "history.dat"
+#define PSM_CERT7_DB "cert7.db"
+#define PSM_KEY3_DB "key3.db"
+#define PSM_SECMODULE_DB "secmod.db"
 #endif /* XP_UNIX */
 
 #define SUMMARY_SUFFIX_IN_5x ".msf"
@@ -139,7 +144,6 @@
 #define POPSTATE_FILE_IN_5x	"popstate.dat"
 #define BOOKMARKS_FILE_NAME_IN_5x "bookmarks.html"
 #define HISTORY_FILE_NAME_IN_5x "history.dat"
-#define RENAMED_OLD_HISTORY_FILE_NAME "old "HISTORY_FILE_NAME_IN_4x
 
 // only UNIX had movemail in 4.x
 #ifdef XP_UNIX
@@ -147,6 +151,27 @@
 #endif /* XP_UNIX */
 
 #define PREMIGRATION_PREFIX "premigration."
+
+// this is for the hidden preference setting in mozilla/modules/libpref/src/init/mailnews.js
+// pref("mail.migration.copyMailFiles", true);
+//
+// see bugzilla bug 80035 (http://bugzilla.mozilla.org/show_bug.cgi?id=80035)
+//
+// the default value for this setting is true which means when migrating from
+// Netscape 4.x, mozilla will copy all the contents of Local Folders and Imap
+// Folder to the newly created subfolders of migrated mozilla profile
+// when this value is set to false, mozilla will not copy these contents and
+// still share them with Netscape 4.x
+//
+// Advantages of forbidding copy operation:
+//     reduce the disk usage
+//     quick migration
+// Disadvantage of forbidding copy operation:
+//     without perfect lock mechamism, there is possibility of data corruption
+//     when Netscape 4.x and mozilla run at the same time and access the same
+//     mail file at the same time
+#define PREF_MIGRATION_MODE_FOR_MAIL "mail.migration.copyMailFiles"
+
 #define PREF_MAIL_DIRECTORY "mail.directory"
 #define PREF_NEWS_DIRECTORY "news.directory"
 #define PREF_MAIL_IMAP_ROOT_DIR "mail.imap.root_dir"
@@ -180,20 +205,6 @@
 
 #define NEW_DIR_SUFFIX "5"
 
-/* who's going to win the file name battle? */
-#if defined(XP_UNIX)
-#define PREF_FILE_NAME_IN_4x "preferences.js"
-#elif defined(XP_MAC)
-#define PREF_FILE_NAME_IN_4x "Netscape Preferences"
-#elif defined(XP_PC)
-#define PREF_FILE_NAME_IN_4x "prefs.js"
-#else
-/* this will cause a failure at run time, as it should, since we don't know
-   how to migrate platforms other than Mac, Windows and UNIX */
-#define PREF_FILE_NAME_IN_4x ""
-#endif /* XP_UNIX */
-
-/* and the winner is:  Windows */
 #define PREF_FILE_NAME_IN_5x "prefs.js"
 
 #define PREF_MIGRATION_PROGRESS_URL "chrome://communicator/content/profile/profileMigrationProgress.xul"
@@ -235,14 +246,11 @@ typedef struct
  *-----------------------------------------------------------------*/
 static NS_DEFINE_IID(kISupportsIID, NS_ISUPPORTS_IID);
 static NS_DEFINE_IID(kIFactoryIID, NS_IFACTORY_IID);
-static NS_DEFINE_IID(kAppShellServiceCID, NS_APPSHELL_SERVICE_CID );
 static NS_DEFINE_IID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
 static NS_DEFINE_IID(kIPrefMigrationIID, NS_IPREFMIGRATION_IID);
 static NS_DEFINE_IID(kPrefMigrationCID,  NS_PREFMIGRATION_CID);
 
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-static NS_DEFINE_CID(kComponentManagerCID, NS_COMPONENTMANAGER_CID);
-static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
 
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 static NS_DEFINE_CID(kStringBundleServiceCID, NS_STRINGBUNDLESERVICE_CID);
@@ -416,7 +424,7 @@ extern "C" void ProfileMigrationController(void *data)
 
     if (migrator->mErrorCode != 0)
     {
-      if (migrator->mErrorCode == RETRY)
+      if (migrator->mErrorCode == MIGRATION_RETRY)
       {
         rv = prefProxy->ShowSpaceDialog(&choice);
         if (NS_FAILED(rv))
@@ -428,7 +436,7 @@ extern "C" void ProfileMigrationController(void *data)
       }
     }
 
-  } while (choice == RETRY);
+  } while (choice == MIGRATION_RETRY);
 
   prefProxy->WindowCloseCallback();
   migrator->mErrorCode = choice;
@@ -554,7 +562,8 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
          enoughSpace             = PR_TRUE,
          localMailDriveDefault   = PR_FALSE,
          summaryMailDriveDefault = PR_FALSE,
-         newsDriveDefault        = PR_FALSE;
+         newsDriveDefault        = PR_FALSE,
+         copyMailFileInMigration = PR_TRUE;
 
   nsFileSpec localMailSpec,
              summaryMailSpec,
@@ -630,34 +639,28 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
                      getter_AddRefs(PrefsFile4xAsIFile));
   if (NS_FAILED(rv)) return rv;
 
-  nsSpecialSystemDirectory systemTempDir(nsSpecialSystemDirectory::OS_TemporaryDirectory);
-  nsFileSpec tempPrefsFile = systemTempDir;
-  
-  systemTempDir += "migrate"; 
-  
-  //Convert the systemTempDir to an nsILocalFile
-  nsCOMPtr<nsILocalFile> systemTempIFileDir;
-  rv = NS_NewLocalFile(systemTempDir, PR_TRUE, getter_AddRefs(systemTempIFileDir));
+  nsCOMPtr<nsIFile> systemTempDir;
+  rv = NS_GetSpecialDirectory(NS_OS_TEMP_DIR, getter_AddRefs(systemTempDir));
   if (NS_FAILED(rv)) return rv;
 
+  systemTempDir->AppendNative(NS_LITERAL_CSTRING("migrate"));
+  
   //Create a unique directory in the system temp dir based on the name of the 4.x prefs file
-  rv = systemTempIFileDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700); 
+  rv = systemTempDir->CreateUnique(nsIFile::DIRECTORY_TYPE, 0700); 
   if (NS_FAILED(rv)) return rv;
 
-  rv = PrefsFile4xAsIFile->CopyTo(systemTempIFileDir, PREF_FILE_NAME_IN_4x);
+  rv = PrefsFile4xAsIFile->CopyToNative(systemTempDir, NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_4x));
   if (NS_FAILED(rv)) return rv;
   
-  //Get the new unique directory name and append it to the system temp dir
-  nsXPIDLCString uniquePrefsDir;
-  systemTempIFileDir->GetLeafName(getter_Copies(uniquePrefsDir));
-  tempPrefsFile += uniquePrefsDir;
-  tempPrefsFile += PREF_FILE_NAME_IN_4x;
+  nsCOMPtr<nsIFile> cloneFile;
+  rv = systemTempDir->Clone(getter_AddRefs(cloneFile));
+  if (NS_FAILED(rv)) return rv;
 
-  //Create the m_prefsFile fileSpec for use in ReadUserPrefsFrom
-//  rv = NS_NewFileSpecWithSpec(tempPrefsFile, getter_AddRefs(m_prefsFile));
-//  if (NS_FAILED(rv)) return rv;
+  m_prefsFile = do_QueryInterface(cloneFile, &rv);
+  if (NS_FAILED(rv)) return rv;
 
-  NS_NewLocalFile((const char *)tempPrefsFile, PR_TRUE, getter_AddRefs(m_prefsFile));
+  rv = m_prefsFile->AppendNative(NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_4x));
+  if (NS_FAILED(rv)) return rv;
 
   //Clear the prefs in case a previous set was read in.
   m_prefs->ResetPrefs();
@@ -673,7 +676,12 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
 
   rv = m_prefs->GetIntPref(PREF_MAIL_SERVER_TYPE, &serverType);
   if (NS_FAILED(rv)) return rv;
-           
+
+  // get the migration mode for mail
+  rv = m_prefs->GetBoolPref(PREF_MIGRATION_MODE_FOR_MAIL, &copyMailFileInMigration);
+  if (NS_FAILED(rv))
+    return rv;
+
   if (serverType == POP_4X_MAIL_TYPE) {
     summaryMailDriveDefault = PR_TRUE; //summary files are only used in IMAP so just set it to true here.
     summaryMailDrive = profileDrive;   //just set the drive for summary files to be the same as the new profile
@@ -848,7 +856,7 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
 
     if (!enoughSpace)
     {
-      mErrorCode = RETRY; 
+      mErrorCode = MIGRATION_RETRY; 
       return NS_OK;
     }
 
@@ -917,8 +925,8 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
     }
   }
   else if (serverType == IMAP_4X_MAIL_TYPE) {
-      
-
+   if( copyMailFileInMigration )  // copy mail files in migration 
+   {
     rv = newIMAPLocalMailPath->Exists(&exists);
     if (NS_FAILED(rv)) return rv;
     if (!exists)  {
@@ -988,6 +996,34 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
       rv = m_prefs->SetFileXPref(PREF_MAIL_IMAP_ROOT_DIR, newIMAPMailPathFile);
       if (NS_FAILED(rv)) return rv;
     }
+   }
+   else
+   {
+    {
+      // temporarily go through nsFileSpec
+      nsFileSpec oldIMAPLocalMailPathSpec;
+      oldIMAPLocalMailPath->GetFileSpec(&oldIMAPLocalMailPathSpec);
+
+      nsCOMPtr<nsILocalFile> oldIMAPLocalMailPathFile;
+      NS_FileSpecToIFile(&oldIMAPLocalMailPathSpec,
+                         getter_AddRefs(oldIMAPLocalMailPathFile));
+
+      rv = m_prefs->SetFileXPref(PREF_MAIL_DIRECTORY, oldIMAPLocalMailPathFile);
+      if (NS_FAILED(rv)) return rv;
+    }
+    {
+      // temporarily go through nsFileSpec
+      nsFileSpec oldIMAPMailPathSpec;
+      oldIMAPMailPath->GetFileSpec(&oldIMAPMailPathSpec);
+
+      nsCOMPtr<nsILocalFile> oldIMAPMailPathFile;
+      NS_FileSpecToIFile(&oldIMAPMailPathSpec,
+                         getter_AddRefs(oldIMAPMailPathFile));
+
+      rv = m_prefs->SetFileXPref(PREF_MAIL_IMAP_ROOT_DIR, oldIMAPMailPathFile);
+      if (NS_FAILED(rv)) return rv;
+    }
+   }
   }
 
 #ifdef HAVE_MOVEMAIL
@@ -1089,8 +1125,23 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
     needToRenameFilterFiles = PR_FALSE;
   }
   
-  rv = DoTheCopy(oldProfilePath, newProfilePath, PR_FALSE);
+  // just copy what we need
+  rv = DoTheCopy(oldProfilePath, newProfilePath, COOKIES_FILE_NAME_IN_4x);
   if (NS_FAILED(rv)) return rv;
+  rv = DoTheCopy(oldProfilePath, newProfilePath, BOOKMARKS_FILE_NAME_IN_4x);
+  if (NS_FAILED(rv)) return rv;
+#if defined(XP_MAC)
+  rv = DoTheCopy(oldProfilePath, newProfilePath, SECURITY_PATH, PR_TRUE);
+  if (NS_FAILED(rv)) return rv;
+#else
+  rv = DoTheCopy(oldProfilePath, newProfilePath, PSM_CERT7_DB);
+  if (NS_FAILED(rv)) return rv;
+  rv = DoTheCopy(oldProfilePath, newProfilePath, PSM_KEY3_DB);
+  if (NS_FAILED(rv)) return rv;
+  rv = DoTheCopy(oldProfilePath, newProfilePath, PSM_SECMODULE_DB);
+  if (NS_FAILED(rv)) return rv;
+#endif /* XP_MAC */
+
   rv = DoTheCopy(oldNewsPath, newNewsPath, PR_TRUE);
   if (NS_FAILED(rv)) return rv;
 
@@ -1103,11 +1154,25 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
   rv = CopyAndRenameNewsrcFiles(newNewsPath);
   if (NS_FAILED(rv)) return rv;
 #endif /* NEED_TO_COPY_AND_RENAME_NEWSRC_FILES */
+
   if(serverType == IMAP_4X_MAIL_TYPE) {
+    if( copyMailFileInMigration )  // copy mail files in migration
+    {
     rv = DoTheCopyAndRename(oldIMAPMailPath, newIMAPMailPath, PR_TRUE, needToRenameFilterFiles, IMAP_MAIL_FILTER_FILE_NAME_IN_4x, IMAP_MAIL_FILTER_FILE_NAME_IN_5x);
     if (NS_FAILED(rv)) return rv;
     rv = DoTheCopyAndRename(oldIMAPLocalMailPath, newIMAPLocalMailPath, PR_TRUE, needToRenameFilterFiles,IMAP_MAIL_FILTER_FILE_NAME_IN_4x,IMAP_MAIL_FILTER_FILE_NAME_IN_5x);
     if (NS_FAILED(rv)) return rv;
+    }
+    else  // Copy & Rename filter files
+    {
+      // IMAP path
+      rv = DoTheCopyAndRename(oldIMAPMailPath, PR_TRUE, IMAP_MAIL_FILTER_FILE_NAME_IN_4x, IMAP_MAIL_FILTER_FILE_NAME_IN_5x);
+      // if (NS_FAILED(rv)) return rv;  // don't care it
+      
+      // Local Folders path
+      rv = DoTheCopyAndRename(oldIMAPLocalMailPath, PR_TRUE, IMAP_MAIL_FILTER_FILE_NAME_IN_4x, IMAP_MAIL_FILTER_FILE_NAME_IN_5x);
+      // if (NS_FAILED(rv)) return rv;  // don't care it
+    }
   }
   else if (serverType == POP_4X_MAIL_TYPE) {
     // we take care of the POP filter file later, in DoSpecialUpdates()
@@ -1136,9 +1201,9 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
   nsXPIDLCString path;
 
   newProfilePath->GetNativePath(getter_Copies(path));
-  NS_NewLocalFile(path, PR_TRUE, getter_AddRefs(newPrefsFile));
+  NS_NewNativeLocalFile(path, PR_TRUE, getter_AddRefs(newPrefsFile));
 
-  rv = newPrefsFile->Append(PREF_FILE_NAME_IN_5x);
+  rv = newPrefsFile->AppendNative(NS_LITERAL_CSTRING(PREF_FILE_NAME_IN_5x));
   if (NS_FAILED(rv)) return rv;
 
   rv=m_prefs->SavePrefFile(newPrefsFile);
@@ -1151,9 +1216,9 @@ nsPrefMigration::ProcessPrefsCallback(const char* oldProfilePathStr, const char
   if (flagExists)
     m_prefsFile->Remove(PR_FALSE);
   
-  systemTempIFileDir->Exists(&flagExists); //Delete the unique dir in the system temp dir.
+  systemTempDir->Exists(&flagExists); //Delete the unique dir in the system temp dir.
   if (flagExists)
-    systemTempIFileDir->Remove(PR_FALSE);
+    systemTempDir->Remove(PR_FALSE);
 
   return rv;
 }
@@ -1266,7 +1331,7 @@ nsPrefMigration::GetDirFromPref(nsIFileSpec * oldProfilePath, nsIFileSpec * newP
   if (NS_FAILED(rv)) return rv;
   
   // convert nsILocalFile to nsIFileSpec
-  rv = oldPrefPathFile->GetPath(getter_Copies(oldPrefPathStr));
+  rv = oldPrefPathFile->GetNativePath(oldPrefPathStr);
   if (NS_FAILED(rv)) return rv;
 
   rv = NS_NewFileSpec(getter_AddRefs(oldPrefPath));
@@ -1571,6 +1636,67 @@ nsPrefMigration::DoTheCopyAndRename(nsIFileSpec * oldPathSpec, nsIFileSpec *newP
   return NS_OK;
 }
 
+/*-------------------------------------------------------------------------
+ * DoTheCopyAndRename copies and renames files
+ *
+ * INPUT: aPath - the path
+ *
+ *        aReadSubdirs - if sub directories should be handled
+ *
+ *        aOldFile - old file name (used for renaming)
+ *
+ *        aNewFile - new file name (used for renaming)
+ *
+ * RETURNS: NS_OK if successful
+ *          NS_ERROR_FAILURE if failed
+ *
+ *--------------------------------------------------------------------------*/
+nsresult
+nsPrefMigration::DoTheCopyAndRename(nsIFileSpec * aPathSpec, PRBool aReadSubdirs, const char *aOldName, const char *aNewName)
+{
+  if( !aOldName || !aNewName || !strcmp(aOldName, aNewName) )
+    return NS_ERROR_FAILURE;
+
+  nsresult rv;
+  nsFileSpec path, file;
+  
+  rv = aPathSpec->GetFileSpec(&path);
+  if (NS_FAILED(rv))
+    return rv;
+  rv = aPathSpec->GetFileSpec(&file);
+  if (NS_FAILED(rv))
+    return rv;
+  file += aOldName;
+  
+  // Handle sub folders
+  for (nsDirectoryIterator dir(path, PR_FALSE); dir.Exists(); dir++)
+  {
+    nsFileSpec fileOrDirName = dir.Spec(); //set first file or dir to a nsFileSpec
+    if (fileOrDirName.IsDirectory())
+    {
+      if( aReadSubdirs )
+      {
+        nsCOMPtr<nsIFileSpec>fileOrDirNameSpec;
+        rv = NS_NewFileSpecWithSpec(fileOrDirName, getter_AddRefs(fileOrDirNameSpec));
+        DoTheCopyAndRename(fileOrDirNameSpec, aReadSubdirs, aOldName, aNewName); /* re-enter the DoTheCopyAndRename function */
+      }
+      else
+        continue;
+    }
+  }
+
+  nsCOMPtr<nsILocalFile> localFileOld, localFileDirectory;
+  rv = NS_FileSpecToIFile(&file, getter_AddRefs(localFileOld));
+  if (NS_FAILED(rv))
+    return rv;
+  rv = NS_FileSpecToIFile(&path, getter_AddRefs(localFileDirectory));
+  if (NS_FAILED(rv))
+    return rv;
+  nsAutoString newName = NS_ConvertUTF8toUCS2(aNewName);
+  localFileOld->CopyTo(localFileDirectory, newName);
+
+  return NS_OK;
+}
 
 nsresult
 nsPrefMigration::DoTheCopy(nsIFileSpec * oldPath, nsIFileSpec * newPath, PRBool readSubdirs)
@@ -1578,6 +1704,61 @@ nsPrefMigration::DoTheCopy(nsIFileSpec * oldPath, nsIFileSpec * newPath, PRBool
   return DoTheCopyAndRename(oldPath, newPath, readSubdirs, PR_FALSE, "", "");
 }
 
+nsresult
+nsPrefMigration::DoTheCopy(nsIFileSpec * oldPath, nsIFileSpec * newPath, const char *fileOrDirName, PRBool isDirectory)
+{
+  nsresult rv;
+
+  if (isDirectory)
+  {
+    nsCOMPtr<nsIFileSpec> oldSubPath;
+
+    NS_NewFileSpec(getter_AddRefs(oldSubPath));
+    oldSubPath->FromFileSpec(oldPath);
+    rv = oldSubPath->AppendRelativeUnixPath(fileOrDirName);
+    if (NS_FAILED(rv)) return rv;
+    PRBool exist;
+    rv = oldSubPath->Exists(&exist);
+    if (NS_FAILED(rv)) return rv;
+    if (!exist)
+    {
+      rv = oldSubPath->CreateDir();
+      if (NS_FAILED(rv)) return rv;
+    }
+
+    nsCOMPtr<nsIFileSpec> newSubPath;
+
+    NS_NewFileSpec(getter_AddRefs(newSubPath));
+    newSubPath->FromFileSpec(newPath);
+    rv = newSubPath->AppendRelativeUnixPath(fileOrDirName);
+    if (NS_FAILED(rv)) return rv;
+    rv = newSubPath->Exists(&exist);
+    if (NS_FAILED(rv)) return rv;
+    if (!exist)
+    {
+      rv = newSubPath->CreateDir();
+      if (NS_FAILED(rv)) return rv;
+    }
+
+    DoTheCopy(oldSubPath, newSubPath, PR_TRUE);
+  }
+  else
+  {
+    nsCOMPtr<nsIFileSpec> file;
+    NS_NewFileSpec(getter_AddRefs(file));
+    file->FromFileSpec(oldPath);
+    rv = file->AppendRelativeUnixPath(fileOrDirName);
+    if( NS_FAILED(rv) ) return rv;
+    PRBool exist;
+    rv = file->Exists(&exist);
+    if( NS_FAILED(rv) ) return rv;
+    if( exist) {
+      file->CopyToDir(newPath);
+    }
+  }
+
+  return rv;
+}
 
 #if defined(NEED_TO_FIX_4X_COOKIES)
 /* this code only works on the mac.  in 4.x, the line endings where '\r' on the mac.
@@ -1599,7 +1780,7 @@ GetCookieLine(nsInputFileStream &strm, nsAutoString& aLine)
 
     /* stop at the '\r' */
     if (c != '\r') {
-      aLine.AppendWithConversion(c);
+      aLine.Append(PRUnichar(c));
     }
     else {
       break;
@@ -1670,12 +1851,12 @@ Fix4xCookies(nsIFileSpec * profilePath) {
     /* locate expire field, skip line if it does not contain all its fields */
     int hostIndex, isDomainIndex, pathIndex, xxxIndex, expiresIndex, nameIndex, cookieIndex;
     hostIndex = 0;
-    if ((isDomainIndex=inBuffer.FindChar('\t', PR_FALSE,hostIndex)+1) == 0 ||
-        (pathIndex=inBuffer.FindChar('\t', PR_FALSE,isDomainIndex)+1) == 0 ||
-        (xxxIndex=inBuffer.FindChar('\t', PR_FALSE,pathIndex)+1) == 0 ||
-        (expiresIndex=inBuffer.FindChar('\t', PR_FALSE,xxxIndex)+1) == 0 ||
-        (nameIndex=inBuffer.FindChar('\t', PR_FALSE,expiresIndex)+1) == 0 ||
-        (cookieIndex=inBuffer.FindChar('\t', PR_FALSE,nameIndex)+1) == 0 ) {
+    if ((isDomainIndex=inBuffer.FindChar('\t', hostIndex)+1) == 0 ||
+        (pathIndex=inBuffer.FindChar('\t', isDomainIndex)+1) == 0 ||
+        (xxxIndex=inBuffer.FindChar('\t', pathIndex)+1) == 0 ||
+        (expiresIndex=inBuffer.FindChar('\t', xxxIndex)+1) == 0 ||
+        (nameIndex=inBuffer.FindChar('\t', expiresIndex)+1) == 0 ||
+        (cookieIndex=inBuffer.FindChar('\t', nameIndex)+1) == 0 ) {
       continue;
     }
 
@@ -1725,7 +1906,6 @@ nsPrefMigration::DoSpecialUpdates(nsIFileSpec  * profilePath)
   nsresult rv;
   PRInt32 serverType;
   nsFileSpec fs;
-  nsCOMPtr<nsIFileSpec> historyFile;
 
   rv = profilePath->GetFileSpec(&fs);
   if (NS_FAILED(rv)) return rv;
@@ -1778,20 +1958,6 @@ nsPrefMigration::DoSpecialUpdates(nsIFileSpec  * profilePath)
   }
 #endif /* IMAP_MAIL_FILTER_FILE_NAME_FORMAT_IN_4x */
 
-  // TODO remove any 4.x files that should not be left around
-  //
-  // examples: prefs, history
-  NS_NewFileSpec(getter_AddRefs(historyFile));
-  historyFile->FromFileSpec(profilePath);
-
-  rv = historyFile->AppendRelativeUnixPath(HISTORY_FILE_NAME_IN_5x);
-  PRBool fileExists;
-  rv = historyFile->Exists(&fileExists);
-  if (NS_FAILED(rv)) return rv;
-  if (fileExists) {
-	historyFile->Rename(RENAMED_OLD_HISTORY_FILE_NAME);
-  }
-
   return rv;
 }
 
@@ -2030,7 +2196,7 @@ nsPrefMigration::DetermineOldPath(nsIFileSpec *profilePath, const char *oldPathN
 	rv = bundle->GetStringFromName(entityName.get(), getter_Copies(localizedDirName));
 	if (NS_FAILED(rv)) return rv;
 
-	rv = oldLocalFile->AppendRelativeUnicodePath((const PRUnichar *)localizedDirName);
+	rv = oldLocalFile->AppendRelativePath(localizedDirName);
 	if (NS_FAILED(rv)) return rv;
 
 	PRBool exists = PR_FALSE;
@@ -2047,10 +2213,10 @@ nsPrefMigration::DetermineOldPath(nsIFileSpec *profilePath, const char *oldPathN
 	}
 
 	/* at this point, the folder with the localized name exists, so use it */
-	nsXPIDLCString persistentDescriptor;
-	rv = oldLocalFile->GetPersistentDescriptor(getter_Copies(persistentDescriptor));
+	nsCAutoString persistentDescriptor;
+	rv = oldLocalFile->GetPersistentDescriptor(persistentDescriptor);
 	if (NS_FAILED(rv)) return rv;
-	rv = oldPath->SetPersistentDescriptorString((const char *)persistentDescriptor);
+	rv = oldPath->SetPersistentDescriptorString(persistentDescriptor.get());
 	if (NS_FAILED(rv)) return rv;
 
 	return NS_OK;
@@ -2116,7 +2282,7 @@ ConvertStringToUTF8(nsAutoString& aCharset, const char* inString, char** outStri
     rv = ccm->GetUnicodeDecoder(&aCharset, getter_AddRefs(decoder));
     if(NS_SUCCEEDED(rv) && decoder) {
       PRInt32 uniLength = 0;
-      PRInt32 srcLength = nsCRT::strlen(inString);
+      PRInt32 srcLength = strlen(inString);
       rv = decoder->GetMaxLength(inString, srcLength, &uniLength);
       if (NS_SUCCEEDED(rv)) {
         PRUnichar *unichars = new PRUnichar [uniLength];
@@ -2281,7 +2447,7 @@ nsPrefConverter::GetPlatformCharset(nsAutoString& aCharset)
   // we may cache it since the platform charset will not change through application life
   nsCOMPtr <nsIPlatformCharset> platformCharset = do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv);
   if (NS_SUCCEEDED(rv) && platformCharset) {
-   rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, aCharset);
+   rv = platformCharset->GetCharset(kPlatformCharsetSel_4xPrefsJS, aCharset);
   }
   if (NS_FAILED(rv)) {
    aCharset.Assign(NS_LITERAL_STRING("ISO-8859-1"));  // use ISO-8859-1 in case of any error
