@@ -238,6 +238,8 @@ CompositeEnumeratorImpl::HasMoreElements(PRBool* aResult)
 
             rv = GetEnumerator(datasource, &mCurrent);
             if (NS_FAILED(rv)) return rv;
+            if (rv == NS_RDF_NO_VALUE)
+                continue;
 
             NS_ASSERTION(mCurrent != nsnull, "you're always supposed to return an enumerator from GetEnumerator, punk.");
             if (! mCurrent)
@@ -409,7 +411,7 @@ private:
 
     // Hide so that only Create() and Destroy() can be used to
     // allocate and deallocate from the heap
-    static void* operator new(size_t) { return 0; }
+    static void* operator new(size_t) CPP_THROW_NEW { return 0; }
     static void operator delete(void*, size_t) {}
 };
 
@@ -528,7 +530,7 @@ private:
 
     // Hide so that only Create() and Destroy() can be used to
     // allocate and deallocate from the heap
-    static void* operator new(size_t) { return 0; }
+    static void* operator new(size_t) CPP_THROW_NEW { return 0; }
     static void operator delete(void*, size_t) {}
 };
 
