@@ -92,11 +92,11 @@
 #include "rdfutil.h"
 #include "nsReadableUtils.h"
 #include "nsIExpatSink.h"
+#include "nsCRT.h"
 
 ////////////////////////////////////////////////////////////////////////
 
 static const char kNameSpaceSeparator = ':';
-static const char kNameSpaceDef[] = "xmlns";
 
 static const char kRDFNameSpaceURI[] = RDF_NAMESPACE_URI;
 static const char kNCNameSpaceURI[] = NC_NAMESPACE_URI;
@@ -161,7 +161,7 @@ public:
     NS_IMETHOD WillResume(void);
     NS_IMETHOD SetParser(nsIParser* aParser);  
     NS_IMETHOD FlushPendingNotifications() { return NS_OK; }
-    NS_IMETHOD SetDocumentCharset(nsAWritableString& aCharset) { return NS_OK; }
+    NS_IMETHOD SetDocumentCharset(nsAString& aCharset) { return NS_OK; }
 
     // nsIRDFContentSink
     NS_IMETHOD Init(nsIURI* aURL);
@@ -200,8 +200,6 @@ protected:
                        PRBool* aDidFlush=nsnull);
     nsresult AddText(const PRUnichar* aText, PRInt32 aLength);
 
-    nsresult ParseTagString(const PRUnichar* aTagName, const char** aNameSpaceURI, nsIAtom** aTagAtom);
-    
     // RDF-specific parsing
     nsresult OpenRDF(const PRUnichar* aName);
     nsresult OpenObject(const PRUnichar* aName ,const PRUnichar** aAttributes);
@@ -216,7 +214,7 @@ protected:
 
     // namespace management
     nsresult PushNameSpacesFrom(const PRUnichar** aAttributes);
-    PRBool   IsXMLNSDirective(const nsAReadableString& aAttributeKey, nsIAtom** aPrefix = nsnull);
+    PRBool   IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix = nsnull);
     nsresult PopNameSpaces();
 
     PRUnichar* mText;
@@ -246,18 +244,18 @@ protected:
     nsAutoVoidArray mNameSpaceScopes;
 
     nsIAtom*
-    CutNameSpacePrefix(nsString& aString);
+    CutNameSpacePrefix(const nsAString& aString, nsIAtom** aTagAtom);
 
     nsresult
     GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI);
 
     nsresult
-    ParseTagString(const nsAReadableString& aTagName,
+    ParseTagString(const PRUnichar* aTagName,
                    const char** aNameSpaceURI,
                    nsIAtom** aTag);
 
     nsresult
-    ParseAttributeString(const nsAReadableString& aAttributeName,
+    ParseAttributeString(const nsAString& aAttributeName,
                          const char** aNameSpaceURI,
                          nsIAtom** aAttribute);
 
@@ -599,11 +597,15 @@ NS_IMETHODIMP
 RDFContentSinkImpl::HandleCDataSection(const PRUnichar *aData, 
                                        PRUint32 aLength)
 {
-    return NS_OK;
+  return aData ?  AddText(aData, aLength) : NS_OK;
 }
 
 NS_IMETHODIMP 
-RDFContentSinkImpl::HandleDoctypeDecl(const PRUnichar *aDoctype)
+RDFContentSinkImpl::HandleDoctypeDecl(const nsAString & aSubset, 
+                                      const nsAString & aName, 
+                                      const nsAString & aSystemId, 
+                                      const nsAString & aPublicId,
+                                      nsISupports* aCatalogData)
 {
     return NS_OK;
 }
@@ -612,11 +614,7 @@ NS_IMETHODIMP
 RDFContentSinkImpl::HandleCharacterData(const PRUnichar *aData, 
                                         PRUint32 aLength)
 {
-  nsresult result = NS_OK;
-  if (aData) {
-    result = result = AddText(aData,aLength);
-  }
-  return result;
+  return aData ?  AddText(aData, aLength) : NS_OK;
 }
 
 NS_IMETHODIMP 
@@ -914,20 +912,6 @@ RDFContentSinkImpl::PushNameSpacesFrom(const PRUnichar** aAttributes)
   return NS_OK;
 }
 
-nsresult
-RDFContentSinkImpl::ParseTagString(const PRUnichar* aTagName,
-                                   const char** aNameSpaceURI,
-                                   nsIAtom** aTag)
-{
-    // Split the fully-qualified name into a prefix and a tag part.
-    nsAutoString tag(aTagName);
-    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(tag));
-
-    GetNameSpaceURI(prefix, aNameSpaceURI);
-    *aTag = NS_NewAtom(tag);
-    return NS_OK;
-}
-
 nsresult
 RDFContentSinkImpl::GetIdAboutAttribute(const PRUnichar** aAttributes,
                                         nsIRDFResource** aResource,
@@ -936,8 +920,8 @@ RDFContentSinkImpl::GetIdAboutAttribute(const PRUnichar** aAttributes,
     // This corresponds to the dirty work of production [6.5]
     nsresult rv;
 
-    nsXPIDLCString docURI;
-    rv = mDocumentURL->GetSpec(getter_Copies(docURI));
+    nsCAutoString docURI;
+    rv = mDocumentURL->GetSpec(docURI);
     if (NS_FAILED(rv)) return rv;
 
     for (; *aAttributes; aAttributes += 2) {
@@ -1039,10 +1023,9 @@ RDFContentSinkImpl::GetResourceAttribute(const PRUnichar** aAttributes,
           // XXX Take the URI and make it fully qualified by
           // sticking it into the document's URL. This may not be
           // appropriate...
-          char* documentURL;
-          mDocumentURL->GetSpec(&documentURL);
+          nsCAutoString documentURL;
+          mDocumentURL->GetSpec(documentURL);
           rdf_MakeAbsoluteURI(NS_ConvertUTF8toUCS2(documentURL), uri);
-          nsCRT::free(documentURL);
 
           return gRDFService->GetUnicodeResource(uri.get(), aResource);
       }
@@ -1412,19 +1395,25 @@ RDFContentSinkImpl::OpenValue(const PRUnichar* aName, const PRUnichar** aAttribu
 // Qualified name resolution
 
 nsIAtom*
-RDFContentSinkImpl::CutNameSpacePrefix(nsString& aString)
+RDFContentSinkImpl::CutNameSpacePrefix(const nsAString& aString,
+                                       nsIAtom** aTagAtom)
 {
-    PRInt32 nsoffset = aString.FindChar(kNameSpaceSeparator);
+    nsIAtom* prefixAtom = nsnull;
+    NS_PRECONDITION(aTagAtom != nsnull, "null ptr");
+    *aTagAtom = nsnull;
 
-    if (nsoffset >= 0) {
-        nsAutoString prefix;
-        aString.Left(prefix, nsoffset);
-        aString.Cut(0, nsoffset+1);
-        return NS_NewAtom(prefix);
-    }
-    else {
-        return nsnull;
+    nsAString::const_iterator start, end;
+    aString.BeginReading(start);
+    aString.EndReading(end);
+    nsAString::const_iterator colon(start);
+
+    if (FindCharInReadable(kNameSpaceSeparator, colon, end)) {
+        prefixAtom = NS_NewAtom(Substring(start, colon));
+        *aTagAtom = NS_NewAtom(Substring(++colon, end));
     }
+    else
+        *aTagAtom = NS_NewAtom(aString);
+    return prefixAtom;
 }
 
 
@@ -1466,29 +1455,27 @@ RDFContentSinkImpl::GetNameSpaceURI(nsIAtom* aPrefix, const char** aNameSpaceURI
 }
 
 nsresult
-RDFContentSinkImpl::ParseTagString(const nsAReadableString& aTagName,
+RDFContentSinkImpl::ParseTagString(const PRUnichar* aTagName,
                                    const char** aNameSpaceURI,
                                    nsIAtom** aTag)
 {
     // Split the fully-qualified name into a prefix and a tag part.
-    nsAutoString tag(aTagName);
-    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(tag));
+    nsDependentString tagStr(aTagName);
+    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(tagStr, aTag));
 
     GetNameSpaceURI(prefix, aNameSpaceURI);
-    *aTag = NS_NewAtom(tag);
     return NS_OK;
 }
 
 
 nsresult
-RDFContentSinkImpl::ParseAttributeString(const nsAReadableString& aAttributeName,
+RDFContentSinkImpl::ParseAttributeString(const nsAString& aAttributeName,
                                          const char** aNameSpaceURI,
                                          nsIAtom** aAttribute)
 {
     // Split the fully-qualified name into a prefix and a tag part.
-    nsAutoString attr(aAttributeName);
-    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(attr));
-
+    nsCOMPtr<nsIAtom> prefix = getter_AddRefs(CutNameSpacePrefix(aAttributeName,
+                                                                 aAttribute));
     if (prefix) {
         GetNameSpaceURI(prefix, aNameSpaceURI);
     }
@@ -1496,7 +1483,6 @@ RDFContentSinkImpl::ParseAttributeString(const nsAReadableString& aAttributeName
         *aNameSpaceURI = nsnull;
     }
 
-    *aAttribute = NS_NewAtom(attr);
     return NS_OK;
 }
 
@@ -1665,32 +1651,32 @@ RDFContentSinkImpl::PopContext(nsIRDFResource         *&aResource,
 // Namespace management
 
 PRBool
-RDFContentSinkImpl::IsXMLNSDirective(const nsAReadableString& aAttributeKey, nsIAtom** aPrefix)
+RDFContentSinkImpl::IsXMLNSDirective(const nsAString& aAttributeKey, nsIAtom** aPrefix)
 {
-    nsAutoString attr(aAttributeKey);
+    NS_NAMED_LITERAL_STRING(kNameSpaceDef, "xmlns");
+    const size_t kNameSpaceDef_Length = sizeof("xmlns") - 1;
 
     // Look for `xmlns' at the start of the attribute name
-    PRInt32 offset = attr.Find(kNameSpaceDef);
-    if (offset != 0)
+    if (!Substring(aAttributeKey, 0, kNameSpaceDef_Length).Equals(kNameSpaceDef))
         return PR_FALSE;
 
-    PRInt32 prefixLen = attr.Length() - sizeof(kNameSpaceDef);
+    PRInt32 prefixLen = aAttributeKey.Length() - kNameSpaceDef_Length - 1;
     if (prefixLen <= 0) {
         // they're setting the default namespace; leave `prefix'
         // as nsnull.
     }
     else {
         // make sure there's a `:' character
-        if (attr[sizeof(kNameSpaceDef) - 1] != kNameSpaceSeparator)
+        nsAString::const_iterator iter;
+        if (*aAttributeKey.BeginReading(iter).advance(kNameSpaceDef_Length)
+                != kNameSpaceSeparator)
             return PR_FALSE;
 
         // if the caller wants the prefix back, compute it for them.
-        if (aPrefix) {
-            nsAutoString prefixStr;
-            attr.Right(prefixStr, prefixLen);
-
-            *aPrefix = NS_NewAtom(prefixStr);
-        }
+        if (aPrefix)
+            *aPrefix = NS_NewAtom(Substring(aAttributeKey,
+                                            kNameSpaceDef_Length + 1,
+                                            prefixLen));
     }
 
     return PR_TRUE;
