@@ -52,6 +52,7 @@
 #include "nsIEventQueueService.h"
 #include "nsIFastLoadService.h"
 #include "nsIFile.h"
+#include "nsIFileURL.h"
 #include "nsIFileChannel.h"
 #include "nsIIOService.h"
 #include "nsIJARChannel.h"
@@ -60,11 +61,13 @@
 #include "nsIObjectOutputStream.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIServiceManager.h"
+#include "nsIStandardURL.h"
 #include "nsIStreamListener.h"
 #include "nsIXULPrototypeCache.h"
 #include "nsIXULPrototypeDocument.h"
 #include "nsNetCID.h"
 #include "nsXPIDLString.h"
+#include "nsString.h"
 #include "prlog.h"
 
 //----------------------------------------------------------------------
@@ -134,7 +137,7 @@ public:
     NS_DECL_ISUPPORTS
 
     // nsIRequest
-    NS_IMETHOD GetName(PRUnichar* *result) { return NS_ERROR_NOT_IMPLEMENTED; }
+    NS_IMETHOD GetName(nsACString &result) { return NS_ERROR_NOT_IMPLEMENTED; }
     NS_IMETHOD IsPending(PRBool *_retval) { *_retval = PR_TRUE; return NS_OK; }
     NS_IMETHOD GetStatus(nsresult *status) { *status = mStatus; return NS_OK; }
     NS_IMETHOD Cancel(nsresult status)  { mStatus = status; return NS_OK; }
@@ -339,20 +342,35 @@ nsCachedChromeChannel::SetNotificationCallbacks(nsIInterfaceRequestor * aNotific
 }
 
 NS_IMETHODIMP
-nsCachedChromeChannel::GetContentType(char * *aContentType)
+nsCachedChromeChannel::GetContentType(nsACString &aContentType)
 {
-    *aContentType = nsCRT::strdup("mozilla.application/cached-xul");
-    return *aContentType ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
+    aContentType = NS_LITERAL_CSTRING("mozilla.application/cached-xul");
+    return NS_OK;
 }
 
 NS_IMETHODIMP
-nsCachedChromeChannel::SetContentType(const char *aContentType)
+nsCachedChromeChannel::SetContentType(const nsACString &aContentType)
 {
     // Do not allow the content-type to be changed.
     NS_NOTREACHED("don't do that");
     return NS_ERROR_FAILURE;
 }
 
+NS_IMETHODIMP
+nsCachedChromeChannel::GetContentCharset(nsACString &aContentCharset)
+{
+    aContentCharset.Truncate();
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsCachedChromeChannel::SetContentCharset(const nsACString &aContentCharset)
+{
+    // Do not allow the content charset to be changed.
+    NS_NOTREACHED("don't do that");
+    return NS_ERROR_FAILURE;
+}
+
 NS_IMETHODIMP
 nsCachedChromeChannel::GetContentLength(PRInt32 *aContentLength)
 {
@@ -515,11 +533,9 @@ nsChromeProtocolHandler::Create(nsISupports *aOuter, REFNSIID aIID, void **aResu
 // nsIProtocolHandler methods:
 
 NS_IMETHODIMP
-nsChromeProtocolHandler::GetScheme(char* *result)
+nsChromeProtocolHandler::GetScheme(nsACString &result)
 {
-    *result = nsCRT::strdup("chrome");
-    if (*result == nsnull)
-        return NS_ERROR_OUT_OF_MEMORY;
+    result = "chrome";
     return NS_OK;
 }
 
@@ -546,7 +562,9 @@ nsChromeProtocolHandler::GetProtocolFlags(PRUint32 *result)
 }
 
 NS_IMETHODIMP
-nsChromeProtocolHandler::NewURI(const char *aSpec, nsIURI *aBaseURI,
+nsChromeProtocolHandler::NewURI(const nsACString &aSpec,
+                                const char *aCharset,
+                                nsIURI *aBaseURI,
                                 nsIURI **result)
 {
     nsresult rv;
@@ -554,26 +572,15 @@ nsChromeProtocolHandler::NewURI(const char *aSpec, nsIURI *aBaseURI,
     // Chrome: URLs (currently) have no additional structure beyond that provided
     // by standard URLs, so there is no "outer" given to CreateInstance
 
-    nsCOMPtr<nsIURI> url(do_CreateInstance(kStandardURLCID, &rv));
+    nsCOMPtr<nsIStandardURL> url(do_CreateInstance(kStandardURLCID, &rv));
     if (NS_FAILED(rv))
         return rv;
 
-    if (aBaseURI) {
-        nsXPIDLCString aResolvedURI;
-        rv = aBaseURI->Resolve(aSpec, getter_Copies(aResolvedURI));
-        if (NS_SUCCEEDED(rv))
-            rv = url->SetSpec(aResolvedURI);
-    }
-    else {
-        rv = url->SetSpec((char*)aSpec);
-    }
-
+    rv = url->Init(nsIStandardURL::URLTYPE_STANDARD, -1, aSpec, aCharset, aBaseURI);
     if (NS_FAILED(rv))
         return rv;
 
-    *result = url;
-    NS_ADDREF(*result);
-    return rv;
+    return CallQueryInterface(url, result);
 }
 
 NS_IMETHODIMP
@@ -599,8 +606,23 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
     if (NS_FAILED(rv)) return rv;
 
     nsCOMPtr<nsIXULPrototypeDocument> proto;
-    rv = cache->GetPrototype(aURI, getter_AddRefs(proto));
-    if (NS_FAILED(rv)) return rv;
+    cache->GetPrototype(aURI, getter_AddRefs(proto));
+
+    // Same comment as nsXULDocument::StartDocumentLoad and
+    // nsXULDocument::ResumeWalk
+    // - Ben Goodger
+    //
+    // We don't abort on failure here because there are too many valid
+    // cases that can return failure, and the null-ness of |proto| is enough
+    // to trigger the fail-safe parse-from-disk solution. Example failure cases
+    // (for reference) include:
+    //
+    // NS_ERROR_NOT_AVAILABLE: the URI cannot be found in the FastLoad cache, 
+    //                         parse from disk
+    // other: the FastLoad cache file, XUL.mfl, could not be found, probably
+    //        due to being accessed before a profile has been selected (e.g.
+    //        loading chrome for the profile manager itself). This must be 
+    //        parsed from disk. 
 
     if (proto) {
         // ...in which case, we'll create a dummy stream that'll just
@@ -623,7 +645,7 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         if (NS_FAILED(rv)) return rv;
 
         nsCOMPtr<nsIURI> chromeURI;
-        rv = ioServ->NewURI(spec, nsnull, getter_AddRefs(chromeURI));
+        rv = ioServ->NewURI(spec, nsnull, nsnull, getter_AddRefs(chromeURI));
         if (NS_FAILED(rv)) return rv;
 
         rv = ioServ->NewChannelFromURI(chromeURI, getter_AddRefs(result));
@@ -649,11 +671,11 @@ nsChromeProtocolHandler::NewChannel(nsIURI* aURI,
         // Get a system principal for xul files and set the owner
         // property of the result
         nsCOMPtr<nsIURL> url = do_QueryInterface(aURI);
-        nsXPIDLCString fileExtension;
-        rv = url->GetFileExtension(getter_Copies(fileExtension));
-        if (PL_strcasecmp(fileExtension, "xul") == 0 ||
-            PL_strcasecmp(fileExtension, "html") == 0 ||
-            PL_strcasecmp(fileExtension, "xml") == 0)
+        nsCAutoString fileExtension;
+        rv = url->GetFileExtension(fileExtension);
+        if (PL_strcasecmp(fileExtension.get(), "xul") == 0 ||
+            PL_strcasecmp(fileExtension.get(), "html") == 0 ||
+            PL_strcasecmp(fileExtension.get(), "xml") == 0)
         {
             nsCOMPtr<nsIScriptSecurityManager> securityManager =
                      do_GetService(NS_SCRIPTSECURITYMANAGER_CONTRACTID, &rv);
