@@ -412,7 +412,7 @@ ns_can_escrow(nsKeyGenType keyGenType)
 //Retrieve crypto.version so that callers know what
 //version of PSM this is.
 NS_IMETHODIMP
-nsCrypto::GetVersion(nsAWritableString& aVersion)
+nsCrypto::GetVersion(nsAString& aVersion)
 {
   aVersion.Assign(NS_LITERAL_STRING(PSM_VERSION_STRING).get());
   return NS_OK;
@@ -621,7 +621,7 @@ cryptojs_generateOneKeyPair(JSContext *cx, nsKeyPairInfo *keyPairInfo,
 
   PRUint32 mechanism = cryptojs_convert_to_mechanism(keyPairInfo->keyGenType);
   void *keyGenParams = nsConvertToActualKeyGenParams(mechanism, params, 
-                                                     (params) ? nsCRT::strlen(params):0, 
+                                                     (params) ? strlen(params):0, 
                                                      keySize);
 
   // Make sure the token has password already set on it before trying
@@ -939,7 +939,7 @@ nsSetRegToken(CRMFCertRequest *certReq, char *regToken)
   
     SECItem src;
     src.data = (unsigned char*)regToken;
-    src.len  = nsCRT::strlen(regToken);
+    src.len  = strlen(regToken);
     SECItem *derEncoded = SEC_ASN1EncodeItem(nsnull, nsnull, &src, 
                                         __WRAPPER_SEC_ASN1EncodeItem_Param4(SEC_UTF8StringTemplate));
 
@@ -967,7 +967,7 @@ nsSetAuthenticator(CRMFCertRequest *certReq, char *authenticator)
     
     SECItem src;
     src.data = (unsigned char*)authenticator;
-    src.len  = nsCRT::strlen(authenticator);
+    src.len  = strlen(authenticator);
     SECItem *derEncoded = SEC_ASN1EncodeItem(nsnull, nsnull, &src,
                                      __WRAPPER_SEC_ASN1EncodeItem_Param4(SEC_UTF8StringTemplate));
     if (!derEncoded)
@@ -1341,11 +1341,11 @@ nsCreateReqFromKeyPairs(nsKeyPairInfo *keyids, PRInt32 numRequests,
   CRMFCertRequest *certReq;
   if (!certReqMsgs)
     return nsnull;
-  nsCRT::memset(certReqMsgs, 0, sizeof(CRMFCertReqMsg*)*(1+numRequests));
+  memset(certReqMsgs, 0, sizeof(CRMFCertReqMsg*)*(1+numRequests));
   SECStatus srv;
   nsresult rv;
   CRMFCertReqMessages messages;
-  nsCRT::memset(&messages, 0, sizeof(messages));
+  memset(&messages, 0, sizeof(messages));
   messages.messages = certReqMsgs;
   SECItem *encodedReq;
   char *retString;
@@ -1516,7 +1516,7 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
     JS_ReportError(cx, "%s\n", JS_ERROR_INTERNAL);
     return NS_ERROR_OUT_OF_MEMORY;
   }
-  nsCRT::memset(keyids, 0, sizeof(nsKeyPairInfo)*numRequests);
+  memset(keyids, 0, sizeof(nsKeyPairInfo)*numRequests);
   int keyInfoIndex;
   PRUint32 i;
   PK11SlotInfo *slot = nsnull;
@@ -1549,8 +1549,6 @@ nsCrypto::GenerateCRMFRequest(nsIDOMCRMFObject** aReturn)
   nsCRMFObject *newObject = new nsCRMFObject();
   if (newObject == nsnull) {
     JS_ReportError(cx, "%s%s\n", JS_ERROR, "could not create crmf JS object");
-    if (slot)
-      PK11_FreeSlot(slot);
 
     nsFreeKeyPairInfo(keyids,numRequests);
     return NS_ERROR_OUT_OF_MEMORY;
@@ -1765,7 +1763,7 @@ nsCryptoRunnable::Run()
   jsval retval;
   if (JS_EvaluateScriptForPrincipals(m_args->m_cx, m_args->m_scope, principals,
                                      m_args->m_jsCallback, 
-                                     nsCRT::strlen(m_args->m_jsCallback),
+                                     strlen(m_args->m_jsCallback),
                                      nsnull, 0,
                                      &retval) != JS_TRUE) {
     return NS_ERROR_FAILURE;
@@ -1829,16 +1827,16 @@ nsCertListCount(CERTCertList *certList)
 //Import user certificates that arrive as a CMMF base64 encoded
 //string.
 NS_IMETHODIMP
-nsCrypto::ImportUserCertificates(const nsAReadableString& aNickname, 
-                                 const nsAReadableString& aCmmfResponse, 
+nsCrypto::ImportUserCertificates(const nsAString& aNickname, 
+                                 const nsAString& aCmmfResponse, 
                                  PRBool aDoForcedBackup, 
-                                 nsAWritableString& aReturn)
+                                 nsAString& aReturn)
 {
   char *nickname=nsnull, *cmmfResponse=nsnull;
   char *retString=nsnull;
   char *freeString=nsnull;
   CMMFCertRepContent *certRepContent = nsnull;
-  int numResponses;
+  int numResponses = 0;
   nsIX509Cert **certArr = nsnull;
   int i;
   CMMFCertResponse *currResponse;
@@ -1887,7 +1885,7 @@ nsCrypto::ImportUserCertificates(const nsAReadableString& aNickname,
     if (!certArr)
       aDoForcedBackup = PR_FALSE;
 
-    nsCRT::memset(certArr, 0, sizeof(nsIX509Cert*)*numResponses);
+    memset(certArr, 0, sizeof(nsIX509Cert*)*numResponses);
   }
   for (i=0; i<numResponses; i++) {
     currResponse = CMMF_CertRepContentGetResponseAtIndex(certRepContent,i);
@@ -2043,20 +2041,20 @@ nsCrypto::ImportUserCertificates(const nsAReadableString& aNickname,
 }
 
 NS_IMETHODIMP
-nsCrypto::PopChallengeResponse(const nsAReadableString& aChallenge, 
-                               nsAWritableString& aReturn)
+nsCrypto::PopChallengeResponse(const nsAString& aChallenge, 
+                               nsAString& aReturn)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsCrypto::Random(PRInt32 aNumBytes, nsAWritableString& aReturn)
+nsCrypto::Random(PRInt32 aNumBytes, nsAString& aReturn)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsCrypto::SignText(nsAWritableString& aReturn)
+nsCrypto::SignText(nsAString& aReturn)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -2064,7 +2062,7 @@ nsCrypto::SignText(nsAWritableString& aReturn)
 
 
 NS_IMETHODIMP
-nsCrypto::Alert(const nsAReadableString& aMessage)
+nsCrypto::Alert(const nsAString& aMessage)
 {
   PRUnichar *message = ToNewUnicode(aMessage);
   alertUser(message);
@@ -2104,7 +2102,7 @@ nsCRMFObject::init()
 }
 
 NS_IMETHODIMP
-nsCRMFObject::GetRequest(nsAWritableString& aRequest)
+nsCRMFObject::GetRequest(nsAString& aRequest)
 {
   aRequest.Assign(mBase64Request);
   return NS_OK;
@@ -2145,7 +2143,7 @@ confirm_user(const PRUnichar *message)
 
 //Delete a PKCS11 module from the user's profile.
 NS_IMETHODIMP
-nsPkcs11::Deletemodule(const nsAReadableString& aModuleName, PRInt32* aReturn)
+nsPkcs11::Deletemodule(const nsAString& aModuleName, PRInt32* aReturn)
 {
   nsresult rv;
   nsString errorMessage;
@@ -2200,8 +2198,8 @@ nsPkcs11::Deletemodule(const nsAReadableString& aModuleName, PRInt32* aReturn)
 
 //Add a new PKCS11 module to the user's profile.
 NS_IMETHODIMP
-nsPkcs11::Addmodule(const nsAReadableString& aModuleName, 
-                    const nsAReadableString& aLibraryFullPath, 
+nsPkcs11::Addmodule(const nsAString& aModuleName, 
+                    const nsAString& aLibraryFullPath, 
                     PRInt32 aCryptoMechanismFlags, 
                     PRInt32 aCipherFlags, PRInt32* aReturn)
 {
