@@ -53,6 +53,7 @@
 #include "nsIEventQueue.h"
 #include "nsIRunnable.h"
 #include "plevent.h"
+#include "nsCRT.h"
 
 #include "nsIWindowWatcher.h"
 #include "nsIPrompt.h"
@@ -66,6 +67,7 @@
 #include "nsIEntropyCollector.h"
 #include "nsIBufEntropyCollector.h"
 #include "nsIServiceManager.h"
+#include "nsILocalFile.h"
 
 #include "nss.h"
 #include "pk11func.h"
@@ -92,31 +94,13 @@ static NS_DEFINE_CID(kDateTimeFormatCID, NS_DATETIMEFORMAT_CID);
 static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
 int nsNSSComponent::mInstanceCount = 0;
 
-#ifdef XP_MAC
-
-OSErr ConvertMacPathToUnixPath(const char *macPath, char **unixPath)
-{
-  PRIntn len;
-  char *cursor;
-  
-  len = PL_strlen(macPath);
-  cursor = (char*)PR_Malloc(len+2);
-  if (!cursor)
-    return memFullErr;
-    
-  memcpy(cursor+1, macPath, len+1);
-  *unixPath = cursor;
-  *cursor = '/';
-  while ((cursor = PL_strchr(cursor, ':')) != NULL) {
-    *cursor = '/';
-    cursor++;
-  }
-  return noErr;
-}
-#endif
-
 // XXX tmp callback for slot password
-extern char * pk11PasswordPrompt(PK11SlotInfo *slot, PRBool retry, void *arg);
+#ifdef NSS_3_4
+extern char * PR_CALLBACK 
+#else
+extern char *
+#endif
+pk11PasswordPrompt(PK11SlotInfo *slot, PRBool retry, void *arg);
 
 #define PIPNSS_STRBUNDLE_URL "chrome://pipnss/locale/pipnss.properties"
 
@@ -285,9 +269,9 @@ nsNSSComponent::~nsNSSComponent()
 
 #ifdef XP_MAC
 #ifdef DEBUG
-#define LOADABLE_CERTS_MODULE "NSSckbiDebug.shlb"
+#define LOADABLE_CERTS_MODULE NS_LITERAL_CSTRING("NSSckbiDebug.shlb")
 #else
-#define LOADABLE_CERTS_MODULE "NSSckbi.shlb"
+#define LOADABLE_CERTS_MODULE NS_LITERAL_CSTRING("NSSckbi.shlb")
 #endif /*DEBUG*/ 
 #endif /*XP_MAC*/
 
@@ -374,7 +358,7 @@ nsNSSComponent::InstallLoadableRoots()
   }
   if (!hasRoot) {
     nsresult rv;
-    nsString modName;
+    nsAutoString modName;
     rv = GetPIPNSSBundleString(NS_LITERAL_STRING("RootCertModuleName").get(),
                                modName);
     if (NS_FAILED(rv)) return;
@@ -385,7 +369,7 @@ nsNSSComponent::InstallLoadableRoots()
       return;
     
     directoryService->Get( NS_XPCOM_CURRENT_PROCESS_DIR,
-                           NS_GET_IID(nsIFile), 
+                           NS_GET_IID(nsILocalFile), 
                            getter_AddRefs(mozFile));
     
     if (!mozFile) {
@@ -393,26 +377,24 @@ nsNSSComponent::InstallLoadableRoots()
     }
     char *fullModuleName = nsnull;
 #ifdef XP_MAC
-    char *unixModulePath=nsnull;
-    mozFile->Append("Essential Files");
-    mozFile->Append(LOADABLE_CERTS_MODULE);
-    mozFile->GetPath(&fullModuleName);
-    ConvertMacPathToUnixPath(fullModuleName, &unixModulePath);
-    nsMemory::Free(fullModuleName);
-    fullModuleName = unixModulePath;
+    nsCAutoString nativePath;
+    mozFile->AppendNative(NS_LITERAL_CSTRING("Essential Files"));
+    mozFile->AppendNative(LOADABLE_CERTS_MODULE);
+    mozFile->GetNativePath(nativePath);    
+    fullModuleName = (char *) nativePath.get();
 #else
-    char *processDir = nsnull;
-    mozFile->GetPath(&processDir);
-    fullModuleName = PR_GetLibraryName(processDir, "nssckbi");
-    nsMemory::Free(processDir);
+    nsCAutoString processDir;
+    mozFile->GetNativePath(processDir);
+    fullModuleName = PR_GetLibraryName(processDir.get(), "nssckbi");
 #endif
     /* If a module exists with the same name, delete it. */
-    char *modNameCString = ToNewCString(modName);
+    NS_ConvertUCS2toUTF8 modNameUTF8(modName);
     int modType;
-    SECMOD_DeleteModule(modNameCString, &modType);
-    SECMOD_AddNewModule(modNameCString, fullModuleName, 0, 0);
-    nsMemory::Free(fullModuleName);
-    nsMemory::Free(modNameCString);
+    SECMOD_DeleteModule(NS_CONST_CAST(char*, modNameUTF8.get()), &modType);
+    SECMOD_AddNewModule(NS_CONST_CAST(char*, modNameUTF8.get()), fullModuleName, 0, 0);
+#ifndef XP_MAC
+    PR_Free(fullModuleName); // allocated by NSPR
+#endif
   }
 }
 
@@ -461,14 +443,14 @@ nsNSSComponent::ConfigureInternalPKCS11Token()
                              getter_Copies(fipsPrivateSlotDescription));
   if (NS_FAILED(rv)) return rv;
 
-  PK11_ConfigurePKCS11(NS_LossyConvertUCS2toASCII(manufacturerID).get(),
-                       NS_LossyConvertUCS2toASCII(libraryDescription).get(),
-                       NS_LossyConvertUCS2toASCII(tokenDescription).get(),
-                       NS_LossyConvertUCS2toASCII(privateTokenDescription).get(),
-                       NS_LossyConvertUCS2toASCII(slotDescription).get(),
-                       NS_LossyConvertUCS2toASCII(privateSlotDescription).get(),
-                       NS_LossyConvertUCS2toASCII(fipsSlotDescription).get(),
-                       NS_LossyConvertUCS2toASCII(fipsPrivateSlotDescription).get(),
+  PK11_ConfigurePKCS11(NS_ConvertUCS2toUTF8(manufacturerID).get(),
+                       NS_ConvertUCS2toUTF8(libraryDescription).get(),
+                       NS_ConvertUCS2toUTF8(tokenDescription).get(),
+                       NS_ConvertUCS2toUTF8(privateTokenDescription).get(),
+                       NS_ConvertUCS2toUTF8(slotDescription).get(),
+                       NS_ConvertUCS2toUTF8(privateSlotDescription).get(),
+                       NS_ConvertUCS2toUTF8(fipsSlotDescription).get(),
+                       NS_ConvertUCS2toUTF8(fipsPrivateSlotDescription).get(),
                        0, 0);
   return NS_OK;
 }
@@ -882,7 +864,7 @@ nsNSSComponent::InitializeNSS()
       certHashtable_valueCompare, 0, 0 );
 
     nsresult rv;
-    nsXPIDLCString profileStr;
+    nsCAutoString profileStr;
     nsCOMPtr<nsIFile> profilePath;
 
     rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILE_50_DIR,
@@ -895,12 +877,12 @@ nsNSSComponent::InitializeNSS()
   #ifdef XP_MAC
     // On the Mac we place all NSS DBs in the Security
     // Folder in the profile directory.
-    profilePath->Append("Security");
+    profilePath->AppendNative(NS_LITERAL_CSTRING("Security"));
     profilePath->Create(nsIFile::DIRECTORY_TYPE, 0); //This is for Mac, don't worry about
                                                      //permissions.
   #endif 
 
-    rv = profilePath->GetPath(getter_Copies(profileStr));
+    rv = profilePath->GetNativePath(profileStr);
     if (NS_FAILED(rv)) 
       return rv;
 
@@ -923,8 +905,8 @@ nsNSSComponent::InitializeNSS()
 
     ConfigureInternalPKCS11Token();
 
-    if (::NSS_InitReadWrite(profileStr) != SECSuccess) {
-      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("can not init NSS r/w in %s\n", profileStr));
+    if (::NSS_InitReadWrite(profileStr.get()) != SECSuccess) {
+      PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("can not init NSS r/w in %s\n", profileStr.get()));
 
       if (supress_warning_preference) {
         which_nss_problem = problem_none;
@@ -934,11 +916,11 @@ nsNSSComponent::InitializeNSS()
       }
 
       // try to init r/o
-      if (NSS_Init(profileStr) != SECSuccess) {
+      if (NSS_Init(profileStr.get()) != SECSuccess) {
         PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("can not init in r/o either\n"));
         which_nss_problem = problem_no_security_at_all;
 
-        NSS_NoDB_Init(profileStr);
+        NSS_NoDB_Init(profileStr.get());
       }
     }
 
@@ -1059,6 +1041,7 @@ nsNSSComponent::ShutdownNSS()
                                 (void*) this);
     }
 
+    SSL_ClearSessionCache();
     ::NSS_Shutdown();
   }
 
@@ -1660,15 +1643,18 @@ PSMContentDownloader::OnStartRequest(nsIRequest* request, nsISupports* context)
   // Get the URI //
   channel->GetURI(getter_AddRefs(mURI));
 
-  rv = channel->GetContentLength(&mContentLength);
-  if (rv != NS_OK || mContentLength == -1)
-    mContentLength = kDefaultCertAllocLength;
+  PRInt32 contentLength;
+  rv = channel->GetContentLength(&contentLength);
+  if (NS_FAILED(rv) || contentLength <= 0)
+    contentLength = kDefaultCertAllocLength;
   
   mBufferOffset = 0;
-  mByteData = (char*) nsMemory::Alloc(mContentLength);
+  mBufferSize = 0;
+  mByteData = (char*) nsMemory::Alloc(contentLength);
   if (!mByteData)
     return NS_ERROR_OUT_OF_MEMORY;
   
+  mBufferSize = contentLength;
   return NS_OK;
 }
 
@@ -1685,21 +1671,21 @@ PSMContentDownloader::OnDataAvailable(nsIRequest* request,
   PRUint32 amt;
   nsresult err;
   //Do a check to see if we need to allocate more memory.
-  if ((mBufferOffset + (PRInt32)aLength) > mContentLength) {
-      size_t newSize = mContentLength + kDefaultCertAllocLength;
+  if ((mBufferOffset + (PRInt32)aLength) > mBufferSize) {
+      size_t newSize = (mBufferOffset + aLength) *2; // grow some more than needed
       char *newBuffer;
       newBuffer = (char*)nsMemory::Realloc(mByteData, newSize);
       if (newBuffer == nsnull) {
         return NS_ERROR_OUT_OF_MEMORY;
       }
       mByteData = newBuffer;
-      mContentLength = newSize;
+      mBufferSize = newSize;
   }
   
   PR_LOG(gPIPNSSLog, PR_LOG_DEBUG, ("CertDownloader::OnDataAvailable\n"));
   do {
     err = aIStream->Read(mByteData+mBufferOffset,
-                         mContentLength-mBufferOffset, &amt);
+                         aLength, &amt);
     if (amt == 0) break;
     if (NS_FAILED(err)) return err;
     
@@ -1969,3 +1955,4 @@ PSMContentListener::SetParentContentListener(nsIURIContentListener * aContentLis
   mParentContentListener = aContentListener;
   return NS_OK;
 }
+
