@@ -217,7 +217,7 @@ GetCertRequest(PRFileDesc *inFile, PRBool ascii)
     do {
 	arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
 	if (arena == NULL) {
-	    GEN_BREAK (SEC_ERROR_NO_MEMORY);
+	    GEN_BREAK (SECFailure);
 	}
 	
  	rv = SECU_ReadDERFromFile(&reqDER, inFile, ascii);
@@ -233,13 +233,13 @@ GetCertRequest(PRFileDesc *inFile, PRBool ascii)
 	   data
 	 */
 	PORT_Memset(&signedData, 0, sizeof(signedData));
-	rv = SEC_ASN1DecodeItem(arena, &signedData, CERT_SignedDataTemplate, 
-				&reqDER);
+	rv = SEC_ASN1DecodeItem(arena, &signedData, 
+		SEC_ASN1_GET(CERT_SignedDataTemplate), &reqDER);
 	if (rv)
 	    break;
 	
-        rv = SEC_ASN1DecodeItem(arena, certReq, CERT_CertificateRequestTemplate,
-				&signedData.data);
+        rv = SEC_ASN1DecodeItem(arena, certReq, 
+		SEC_ASN1_GET(CERT_CertificateRequestTemplate), &signedData.data);
    } while (0);
 
    if (rv) {
@@ -272,7 +272,7 @@ GetYesNo(char *prompt)
 
 static SECStatus
 AddCert(PK11SlotInfo *slot, CERTCertDBHandle *handle, char *name, char *trusts, 
-        PRFileDesc *inFile, PRBool ascii, PRBool emailcert)
+        PRFileDesc *inFile, PRBool ascii, PRBool emailcert, void *pwdata)
 {
     CERTCertTrust *trust = NULL;
     CERTCertificate *cert = NULL, *tempCert = NULL;
@@ -308,38 +308,32 @@ AddCert(PK11SlotInfo *slot, CERTCertDBHandle *handle, char *name, char *trusts,
 	    GEN_BREAK(SECFailure);
 	}
 
-	/* CERT_ImportCert only collects certificates and returns the
-	* first certficate.  It does not insert these certificates into
-	* the dbase.  For now, just call CERT_NewTempCertificate.
-	* This will result in decoding the der twice.  This have to
-	* be handle properly.
-	*/
-
-	tempCert = CERT_NewTempCertificate(handle, &cert->derCert, NULL,
-	                                   PR_FALSE, PR_TRUE);
-
-	if (!PK11_IsInternal(slot)) {
-	    tempCert->trust = trust;
-	    rv = PK11_ImportCertForKeyToSlot(slot, tempCert, name,
-	                                     PR_FALSE, NULL);
+	if (!PK11_IsFriendly(slot)) {
+	    rv = PK11_Authenticate(slot, PR_TRUE, pwdata);
+	    if (rv != SECSuccess) {
+		SECU_PrintError(progName, "could not authenticate to token or database");
+		GEN_BREAK(SECFailure);
+	    }
 	}
 
-	if (tempCert == NULL) {
-	    SECU_PrintError(progName,"unable to add cert to the temp database");
+	rv =  PK11_ImportCert(slot, cert, CK_INVALID_HANDLE, name, PR_FALSE);
+	if (rv != SECSuccess) {
+	    SECU_PrintError(progName, "could not add certificate to token or database");
 	    GEN_BREAK(SECFailure);
 	}
 
-	rv = CERT_AddTempCertToPerm(tempCert, name, trust);
-	if (rv) {
-	    SECU_PrintError(progName, "could not add certificate to database");
+	rv = CERT_ChangeCertTrust(handle, cert, trust);
+	if (rv != SECSuccess) {
+	    SECU_PrintError(progName, "could not change trust on certificate");
 	    GEN_BREAK(SECFailure);
 	}
 
-	if ( emailcert )
-	    CERT_SaveSMimeProfile(tempCert, NULL, NULL);
+	if ( emailcert ) {
+	    CERT_SaveSMimeProfile(cert, NULL, pwdata);
+	}
+
     } while (0);
 
-    CERT_DestroyCertificate (tempCert);
     CERT_DestroyCertificate (cert);
     PORT_Free(trust);
     PORT_Free(certDER.data);
@@ -381,7 +375,7 @@ CertReq(SECKEYPrivateKey *privk, SECKEYPublicKey *pubk, KeyType keyType,
     
     /* Der encode the request */
     encoding = SEC_ASN1EncodeItem(arena, NULL, cr,
-				  CERT_CertificateRequestTemplate);
+		  SEC_ASN1_GET(CERT_CertificateRequestTemplate));
     if (encoding == NULL) {
 	SECU_PrintError(progName, "der encoding of request failed");
 	return SECFailure;
@@ -502,6 +496,7 @@ ChangeTrustAttributes(CERTCertDBHandle *handle, char *name, char *trusts)
 	SECU_PrintError(progName, "unable to modify trust attributes");
 	return SECFailure;
     }
+    CERT_DestroyCertificate(cert);
 
     return SECSuccess;
 }
@@ -525,8 +520,8 @@ printCertCB(CERTCertificate *cert, void *arg)
     if (trust) {
 	SECU_PrintTrustFlags(stdout, trust,
 	                     "Certificate Trust Flags", 1);
-    } else {
-	SECU_PrintTrustFlags(stdout, &cert->dbEntry->trust,
+    } else if (cert->trust) {
+	SECU_PrintTrustFlags(stdout, cert->trust,
 	                     "Certificate Trust Flags", 1);
     }
 
@@ -535,6 +530,36 @@ printCertCB(CERTCertificate *cert, void *arg)
     return(SECSuccess);
 }
 
+static SECStatus
+dumpChain(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
+          PRBool raw, PRBool ascii, PRFileDesc *outfile, void *pwarg)
+{
+    SECStatus rv;
+    CERTCertificate *the_cert;
+    CERTCertificateList *chain;
+    int i, j;
+    the_cert = PK11_FindCertFromNickname(name, NULL);
+    if (!the_cert) {
+	SECU_PrintError(progName, "Could not find: %s\n", name);
+	return SECFailure;
+    }
+    chain = CERT_CertChainFromCert(the_cert, 0, PR_TRUE);
+    CERT_DestroyCertificate(the_cert);
+    if (!chain) {
+	SECU_PrintError(progName, "Could not obtain chain for: %s\n", name);
+	return SECFailure;
+    }
+    for (i=chain->len-1; i>=0; i--) {
+	CERTCertificate *c;
+	c = CERT_FindCertByDERCert(handle, &chain->certs[i]);
+	for (j=i; j<chain->len-1; j++) printf("  ");
+	printf("\"%s\" [%s]\n\n", c->nickname, c->subjectName);
+	CERT_DestroyCertificate(c);
+    }
+    CERT_DestroyCertificateList(chain);
+    return SECSuccess;
+}
+
 static SECStatus
 listCerts(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
           PRBool raw, PRBool ascii, PRFileDesc *outfile, void *pwarg)
@@ -542,66 +567,52 @@ listCerts(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
     CERTCertificate *cert;
     SECItem data;
     PRInt32 numBytes;
-    SECStatus rv;
+    SECStatus rv = SECFailure;
 
-    /* For now, split handling of slot to internal vs. other.  slot should
-     * probably be allowed to be NULL so that all slots can be listed.
-     * In that case, need to add a call to PK11_TraverseSlotCerts().
-     */
-    if (PK11_IsInternal(slot)) {
-	if (name == NULL) {
-	    /* Print all certs in internal slot db. */
-	    rv = SECU_PrintCertificateNames(handle, PR_STDOUT, 
-	                                    PR_FALSE, PR_TRUE);
-	    if (rv) {
-		SECU_PrintError(progName, 
-		                "problem printing certificate nicknames");
-		return SECFailure;
-	    }
-	} else if (raw || ascii) {
-	    /* Dump binary or ascii DER for the cert to stdout. */
-	    cert = CERT_FindCertByNicknameOrEmailAddr(handle, name);
-	    if (!cert) {
-		SECU_PrintError(progName,
-		               "could not find certificate named \"%s\"", name);
-		return SECFailure;
-	    }
-	    data.data = cert->derCert.data;
-	    data.len = cert->derCert.len;
-	    if (ascii) {
-		PR_fprintf(outfile, "%s\n%s\n%s\n", NS_CERT_HEADER, 
-		        BTOA_DataToAscii(data.data, data.len), NS_CERT_TRAILER);
-	    } else if (raw) {
-	        numBytes = PR_Write(outfile, data.data, data.len);
-	        if (numBytes != data.len) {
-		    SECU_PrintSystemError(progName, "error writing raw cert");
-		    return SECFailure;
-		}
-	    }
-	} else {
-	    /* Pretty-print cert. */
-	    rv = CERT_TraversePermCertsForNickname(handle, name, printCertCB,
-	                                           NULL);
-	}
-    } else {
-	/* List certs on a non-internal slot. */
-	if (PK11_NeedLogin(slot))
+    /* List certs on a non-internal slot. */
+    if (!PK11_IsFriendly(slot) && PK11_NeedLogin(slot))
 	    PK11_Authenticate(slot, PR_TRUE, pwarg);
-	if (name) {
-	    CERTCertificate *the_cert;
-	    the_cert = PK11_FindCertFromNickname(name, NULL);
-	    if (!the_cert) {
-		SECU_PrintError(progName, "Could not find: %s\n", name);
-		return SECFailure;
-	    }
-	    rv = printCertCB(the_cert, the_cert->trust);
-	} else {
-	    rv = PK11_TraverseCertsInSlot(slot, SECU_PrintCertNickname, stdout);
-	}
-	if (rv) {
-	    SECU_PrintError(progName, "problem printing certificate nicknames");
+    if (name) {
+	CERTCertificate *the_cert;
+	the_cert = PK11_FindCertFromNickname(name, NULL);
+	if (!the_cert) {
+	    SECU_PrintError(progName, "Could not find: %s\n", name);
 	    return SECFailure;
 	}
+	data.data = the_cert->derCert.data;
+	data.len = the_cert->derCert.len;
+	if (ascii) {
+	    PR_fprintf(outfile, "%s\n%s\n%s\n", NS_CERT_HEADER, 
+		        BTOA_DataToAscii(data.data, data.len), NS_CERT_TRAILER);
+	    rv = SECSuccess;
+	} else if (raw) {
+	    numBytes = PR_Write(outfile, data.data, data.len);
+	    if (numBytes != data.len) {
+		SECU_PrintSystemError(progName, "error writing raw cert");
+		rv = SECFailure;
+	    }
+	    rv = SECSuccess;
+	} else {
+	    rv = printCertCB(the_cert, the_cert->trust);
+	}
+	CERT_DestroyCertificate(the_cert);
+    } else {
+	CERTCertList *certs;
+	CERTCertListNode *node;
+
+	certs = PK11_ListCertsInSlot(slot);
+	if (certs) {
+	    for (node = CERT_LIST_HEAD(certs); !CERT_LIST_END(node,certs);
+						node = CERT_LIST_NEXT(node)) {
+		SECU_PrintCertNickname(node->cert,stdout);
+	    }
+	    CERT_DestroyCertList(certs);
+	    rv = SECSuccess;
+	}
+    }
+    if (rv) {
+	SECU_PrintError(progName, "problem printing certificate nicknames");
+	return SECFailure;
     }
 
     return SECSuccess;	/* not rv ?? */
@@ -619,8 +630,11 @@ ListCerts(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
 
 	list= PK11_GetAllTokens(CKM_INVALID_MECHANISM,
 						PR_FALSE,PR_FALSE,pwdata);
-	if (list) for (le = list->head; le; le = le->next) {
-	    rv = listCerts(handle,name,le->slot,raw,ascii,outfile,pwdata);
+	if (list) {
+	    for (le = list->head; le; le = le->next) {
+		rv = listCerts(handle,name,le->slot,raw,ascii,outfile,pwdata);
+	    }
+	    PK11_FreeSlotList(list);
 	}
     } else {
 	rv = listCerts(handle,name,slot,raw,ascii,outfile,pwdata);
@@ -628,6 +642,28 @@ ListCerts(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
     return rv;
 }
 
+static SECStatus
+DumpChain(CERTCertDBHandle *handle, char *name, PK11SlotInfo *slot,
+          PRBool raw, PRBool ascii, PRFileDesc *outfile, secuPWData *pwdata)
+{
+    SECStatus rv;
+
+    if (slot == NULL) {
+	PK11SlotList *list;
+	PK11SlotListElement *le;
+
+	list= PK11_GetAllTokens(CKM_INVALID_MECHANISM,
+						PR_FALSE,PR_FALSE,pwdata);
+	if (list) for (le = list->head; le; le = le->next) {
+	    printf("\n\nListing certificates for %s:\n\n", PK11_GetSlotName(le->slot));
+	    rv = dumpChain(handle,name,le->slot,raw,ascii,outfile,pwdata);
+	}
+    } else {
+	rv = dumpChain(handle,name,slot,raw,ascii,outfile,pwdata);
+    }
+    return rv;
+}
+
 
 static SECStatus 
 DeleteCert(CERTCertDBHandle *handle, char *name)
@@ -643,6 +679,7 @@ DeleteCert(CERTCertDBHandle *handle, char *name)
     }
 
     rv = SEC_DeletePermCertificate(cert);
+    CERT_DestroyCertificate(cert);
     if (rv) {
 	SECU_PrintError(progName, "unable to delete certificate");
 	return SECFailure;
@@ -656,7 +693,7 @@ ValidateCert(CERTCertDBHandle *handle, char *name, char *date,
 	     char *certUsage, PRBool checkSig, PRBool logit, secuPWData *pwdata)
 {
     SECStatus rv;
-    CERTCertificate *cert;
+    CERTCertificate *cert = NULL;
     int64 timeBoundary;
     SECCertUsage usage;
     CERTVerifyLog reallog;
@@ -745,84 +782,13 @@ ValidateCert(CERTCertDBHandle *handle, char *name, char *date,
 	}
     } while (0);
 
+    if (cert) {
+        CERT_DestroyCertificate(cert);
+    }
+
     return (rv);
 }
 
-#ifdef notdef
-static SECStatus
-DumpPublicKey(int dbindex, char *nickname, FILE *out)
-{
-    SECKEYLowPrivateKey *privKey;
-    SECKEYLowPublicKey *publicKey;
-
-    if (dbindex) {
-	/*privKey = secu_GetPrivKeyFromIndex(dbindex);*/
-    } else {
-	privKey = GetPrivKeyFromNickname(nickname);
-    }
-    publicKey = SECKEY_LowConvertToPublicKey(privKey);
-
-    /* Output public key (in the clear) */
-    switch(publicKey->keyType) {
-      case rsaKey:
-	fprintf(out, "RSA Public-Key:\n");
-	SECU_PrintInteger(out, &publicKey->u.rsa.modulus, "modulus", 1);
-	SECU_PrintInteger(out, &publicKey->u.rsa.publicExponent,
-			  "publicExponent", 1);
-	break;
-      case dsaKey:
-	fprintf(out, "DSA Public-Key:\n");
-	SECU_PrintInteger(out, &publicKey->u.dsa.params.prime, "prime", 1);
-	SECU_PrintInteger(out, &publicKey->u.dsa.params.subPrime,
-			  "subPrime", 1);
-	SECU_PrintInteger(out, &publicKey->u.dsa.params.base, "base", 1);
-	SECU_PrintInteger(out, &publicKey->u.dsa.publicValue, "publicValue", 1);
-	break;
-      default:
-	fprintf(out, "unknown key type\n");
-	break;
-    }
-    return SECSuccess;
-}
-
-static SECStatus
-DumpPrivateKey(int dbindex, char *nickname, FILE *out)
-{
-    SECKEYLowPrivateKey *key;
-    if (dbindex) {
-	/*key = secu_GetPrivKeyFromIndex(dbindex);*/
-    } else {
-	key = GetPrivKeyFromNickname(nickname);
-    }
-
-    switch(key->keyType) {
-      case rsaKey:
-	fprintf(out, "RSA Private-Key:\n");
-	SECU_PrintInteger(out, &key->u.rsa.modulus, "modulus", 1);
-	SECU_PrintInteger(out, &key->u.rsa.publicExponent, "publicExponent", 1);
-	SECU_PrintInteger(out, &key->u.rsa.privateExponent,
-			  "privateExponent", 1);
-	SECU_PrintInteger(out, &key->u.rsa.prime1, "prime1", 1);
-	SECU_PrintInteger(out, &key->u.rsa.prime2, "prime2", 1);
-	SECU_PrintInteger(out, &key->u.rsa.exponent1, "exponent2", 1);
-	SECU_PrintInteger(out, &key->u.rsa.exponent2, "exponent2", 1);
-	SECU_PrintInteger(out, &key->u.rsa.coefficient, "coefficient", 1);
-	break;
-      case dsaKey:
-	fprintf(out, "DSA Private-Key:\n");
-	SECU_PrintInteger(out, &key->u.dsa.params.prime, "prime", 1);
-	SECU_PrintInteger(out, &key->u.dsa.params.subPrime, "subPrime", 1);
-	SECU_PrintInteger(out, &key->u.dsa.params.base, "base", 1);
-	SECU_PrintInteger(out, &key->u.dsa.publicValue, "publicValue", 1);
-	SECU_PrintInteger(out, &key->u.dsa.privateValue, "privateValue", 1);
-	break;
-      default:
-	fprintf(out, "unknown key type\n");
-	break;
-    }
-    return SECSuccess;
-}
-#endif
 
 static SECStatus
 printKeyCB(SECKEYPublicKey *key, SECItem *data, void *arg)
@@ -838,20 +804,18 @@ printKeyCB(SECKEYPublicKey *key, SECItem *data, void *arg)
 }
 
 /* callback for listing certs through pkcs11 */
-SECStatus
-secu_PrintKeyFromCert(CERTCertificate *cert, void *data)
+static SECStatus
+secu_PrintKey(FILE *out, int count, SECKEYPrivateKey *key)
 {
-    FILE *out;
-    SECKEYPrivateKey *key;
+    char *name;
 
-    out = (FILE *)data;
-    key = PK11_FindPrivateKeyFromCert(PK11_GetInternalKeySlot(), cert, NULL);
-    if (!key) {
-	fprintf(out, "XXX could not extract key for %s.\n", cert->nickname);
-	return SECFailure;
+    name = PK11_GetPrivateKeyNickname(key);
+    if (name == NULL) {
+	/* should look up associated cert */
+	name = PORT_Strdup("< orphaned >");
     }
-    /* XXX should have a type field also */
-    fprintf(out, "<%d> %s\n", 0, cert->nickname);
+    fprintf(out, "<%d> %s\n", count, name);
+    PORT_Free(name);
 
     return SECSuccess;
 }
@@ -859,34 +823,29 @@ secu_PrintKeyFromCert(CERTCertificate *cert, void *data)
 static SECStatus
 listKeys(PK11SlotInfo *slot, KeyType keyType, void *pwarg)
 {
-    SECStatus rv = SECSuccess;
+    SECKEYPrivateKeyList *list;
+    SECKEYPrivateKeyListNode *node;
+    int count;
 
-#ifdef notdef
-    if (PK11_IsInternal(slot)) {
-	/* Print all certs in internal slot db. */
-	rv = SECU_PrintKeyNames(SECKEY_GetDefaultKeyDB(), stdout);
-	if (rv) {
-	    SECU_PrintError(progName, "problem listing keys");
-	    return SECFailure;
-	}
-    } else {
-#endif
-	/* XXX need a function as below */
-	/* could iterate over certs on slot and print keys */
-	/* this would miss stranded keys */
-    /*rv = PK11_TraverseSlotKeys(slotname, keyType, printKeyCB, NULL, NULL);*/
-	if (PK11_NeedLogin(slot))
+    if (PK11_NeedLogin(slot))
 	    PK11_Authenticate(slot, PR_TRUE, pwarg);
-	rv = PK11_TraverseCertsInSlot(slot, secu_PrintKeyFromCert, stdout);
-	if (rv) {
-	    SECU_PrintError(progName, "problem listing keys");
-	    return SECFailure;
-	}
-	return SECSuccess;
-#ifdef notdef
+
+    list = PK11_ListPrivateKeysInSlot(slot);
+    if (list == NULL) {
+	SECU_PrintError(progName, "problem listing keys");
+	return SECFailure;
     }
-    return rv;
-#endif
+    for (count=0, node=PRIVKEY_LIST_HEAD(list) ; !PRIVKEY_LIST_END(node,list);
+			  node= PRIVKEY_LIST_NEXT(node),count++) {
+	secu_PrintKey(stdout, count, node->key);
+    }
+    SECKEY_DestroyPrivateKeyList(list);
+
+    if (count == 0) {
+	SECU_PrintError(progName, "no keys found");
+	return SECFailure;
+    }
+    return SECSuccess;
 }
 
 static SECStatus
@@ -895,19 +854,6 @@ ListKeys(PK11SlotInfo *slot, char *keyname, int index,
 {
     SECStatus rv = SECSuccess;
 
-#ifdef notdef
-    if (keyname) {
-	if (dopriv) {
-	    return DumpPrivateKey(index, keyname, stdout);
-	} else {
-	    return DumpPublicKey(index, keyname, stdout);
-	}
-    }
-#endif
-    /* For now, split handling of slot to internal vs. other.  slot should
-     * probably be allowed to be NULL so that all slots can be listed.
-     * In that case, need to add a call to PK11_TraverseSlotCerts().
-     */
     if (slot == NULL) {
 	PK11SlotList *list;
 	PK11SlotListElement *le;
@@ -933,11 +879,16 @@ DeleteKey(char *nickname, secuPWData *pwdata)
     if (PK11_NeedLogin(slot))
 	PK11_Authenticate(slot, PR_TRUE, pwdata);
     cert = PK11_FindCertFromNickname(nickname, pwdata);
-    if (!cert) return SECFailure;
+    if (!cert) {
+	PK11_FreeSlot(slot);
+	return SECFailure;
+    }
     rv = PK11_DeleteTokenCertAndKey(cert, pwdata);
     if (rv != SECSuccess) {
 	SECU_PrintError("problem deleting private key \"%s\"\n", nickname);
     }
+    CERT_DestroyCertificate(cert);
+    PK11_FreeSlot(slot);
     return rv;
 }
 
@@ -993,14 +944,16 @@ Usage(char *progName)
 	"\t\t [-z noisefile] [-d certdir] [-P dbprefix]\n", progName);
     FPS "\t%s -K [-n key-name] [-h token-name] [-k dsa|rsa|all]\n", 
 	progName);
-    FPS "\t\t [-f pwfile] [-d certdir] [-P dbprefix]\n");
-    FPS "\t%s -L [-n cert-name] [-d certdir] [-P dbprefix] [-r] [-a]\n", progName);
+    FPS "\t\t [-f pwfile] [-X] [-d certdir] [-P dbprefix]\n");
+    FPS "\t%s -L [-n cert-name] [-X] [-d certdir] [-P dbprefix] [-r] [-a]\n", progName);
     FPS "\t%s -M -n cert-name -t trustargs [-d certdir] [-P dbprefix]\n",
 	progName);
+    FPS "\t%s -O -n cert-name [-X] [-d certdir] [-P dbprefix]\n", progName);
     FPS "\t%s -R -s subj -o cert-request-file [-d certdir] [-P dbprefix] [-p phone] [-a]\n"
 	"\t\t [-k key-type] [-h token-name] [-f pwfile] [-g key-size]\n",
 	progName);
-    FPS "\t%s -V -n cert-name -u usage [-b time] [-e] [-d certdir] [-P dbprefix]\n",
+    FPS "\t%s -V -n cert-name -u usage [-b time] [-e] \n"
+	"\t\t[-X] [-d certdir] [-P dbprefix]\n",
 	progName);
     FPS "\t%s -S -n cert-name -s subj [-c issuer-name | -x]  -t trustargs\n"
 	"\t\t [-k key-type] [-h token-name] [-g key-size]\n"
@@ -1008,7 +961,7 @@ Usage(char *progName)
 	"\t\t [-f pwfile] [-d certdir] [-P dbprefix]\n"
         "\t\t [-p phone] [-1] [-2] [-3] [-4] [-5] [-6]\n",
 	progName);
-    FPS "\t%s -U [-d certdir] [-P dbprefix]\n", progName);
+    FPS "\t%s -U [-X] [-d certdir] [-P dbprefix]\n", progName);
     exit(1);
 }
 
@@ -1119,6 +1072,9 @@ static void LongUsage(char *progName)
         "   -d moddir");
     FPS "%-20s Cert & Key database prefix\n",
 	"   -P dbprefix");
+    FPS "%-20s force the database to open R/W\n",
+	"   -X");
+    FPS "\n");
 
     FPS "%-15s List all keys\n", /*, or print out a single named key\n",*/
         "-K");
@@ -1133,6 +1089,8 @@ static void LongUsage(char *progName)
 	"   -d keydir");
     FPS "%-20s Cert & Key database prefix\n",
 	"   -P dbprefix");
+    FPS "%-20s force the database to open R/W\n",
+	"   -X");
     FPS "\n");
 
     FPS "%-15s List all certs, or print out a single named cert\n",
@@ -1143,6 +1101,8 @@ static void LongUsage(char *progName)
 	"   -d certdir");
     FPS "%-20s Cert & Key database prefix\n",
 	"   -P dbprefix");
+    FPS "%-20s force the database to open R/W\n",
+	"   -X");
     FPS "%-20s For single cert, print binary DER encoding\n",
 	"   -r");
     FPS "%-20s For single cert, print ASCII encoding (RFC1113)\n",
@@ -1174,10 +1134,23 @@ static void LongUsage(char *progName)
 	"   -d certdir");
     FPS "%-20s Cert & Key database prefix\n",
 	"   -P dbprefix");
-    FPS "%-20s Token to reset (default is internal)\n"
+    FPS "%-20s Token to reset (default is internal)\n",
 	"   -h token-name");
     FPS "\n");
 
+    FPS "\n");
+    FPS "%-15s Print the chain of a certificate\n",
+	"-O");
+    FPS "%-20s The nickname of the cert to modify\n",
+	"   -n cert-name");
+    FPS "%-20s Cert database directory (default is ~/.netscape)\n",
+	"   -d certdir");
+    FPS "%-20s Cert & Key database prefix\n",
+	"   -P dbprefix");
+    FPS "%-20s force the database to open R/W\n",
+	"   -X");
+    FPS "\n");
+
     FPS "%-15s Generate a certificate request (stdout)\n",
 	"-R");
     FPS "%-20s Specify the subject name (using RFC1485)\n",
@@ -1219,6 +1192,8 @@ static void LongUsage(char *progName)
 	"   -d certdir");
     FPS "%-20s Cert & Key database prefix\n",
 	"   -P dbprefix");
+    FPS "%-20s force the database to open R/W\n",
+	"   -X");
     FPS "\n");
 
     FPS "%-15s Make a certificate and add to database\n",
@@ -1364,8 +1339,8 @@ AddKeyUsage (void *extHandle)
 	fprintf(stdout, "%-25s 2 - Key encipherment\n", "");
 	fprintf(stdout, "%-25s 3 - Data encipherment\n", "");   
 	fprintf(stdout, "%-25s 4 - Key agreement\n", "");
-	fprintf(stdout, "%-25s 5 - Cert signning key\n", "");   
-	fprintf(stdout, "%-25s 6 - CRL signning key\n", "");
+	fprintf(stdout, "%-25s 5 - Cert signing key\n", "");   
+	fprintf(stdout, "%-25s 6 - CRL signing key\n", "");
 	fprintf(stdout, "%-25s Other to finish\n", "");
 	if (gets (buffer)) {
 	    value = atoi (buffer);
@@ -1423,6 +1398,14 @@ CreateOidSequence(void)
   return (CERTOidSequence *)NULL;
 }
 
+static void
+DestroyOidSequence(CERTOidSequence *os)
+{
+  if (os->arena) {
+    PORT_FreeArena(os->arena, PR_FALSE);
+  }
+}
+
 static SECStatus
 AddOidToSequence(CERTOidSequence *os, SECOidTag oidTag)
 {
@@ -1462,11 +1445,19 @@ AddOidToSequence(CERTOidSequence *os, SECOidTag oidTag)
   return SECSuccess;
 }
 
+SEC_ASN1_MKSUB(SEC_ObjectIDTemplate);
+
+const SEC_ASN1Template CERT_OidSeqTemplate[] = {
+    { SEC_ASN1_SEQUENCE_OF | SEC_ASN1_XTRN,
+	  offsetof(CERTOidSequence, oids),
+	  SEC_ASN1_SUB(SEC_ObjectIDTemplate) }
+};
+
+
 static SECItem *
 EncodeOidSequence(CERTOidSequence *os)
 {
   SECItem *rv;
-  extern const SEC_ASN1Template CERT_OidSeqTemplate[];
 
   rv = (SECItem *)PORT_ArenaZAlloc(os->arena, sizeof(SECItem));
   if( (SECItem *)NULL == rv ) {
@@ -1555,7 +1546,7 @@ AddExtKeyUsage (void *extHandle)
                           ? PR_TRUE : PR_FALSE), PR_TRUE);
   /*FALLTHROUGH*/
  loser:
-  CERT_DestroyOidSequence(os);
+  DestroyOidSequence(os);
   return rv;
 }
 
@@ -1667,7 +1658,7 @@ AddBasicConstraint(void *extHandle)
 static SECItem *
 SignCert(CERTCertDBHandle *handle, 
 CERTCertificate *cert, PRBool selfsign, 
-SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
+SECKEYPrivateKey *privKey, char *issuerNickName, void *pwarg)
 {
     SECItem der;
     SECItem *result = NULL;
@@ -1677,10 +1668,7 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
     SECOidTag algID;
     void *dummy;
 
-    if( selfsign ) {
-      caPrivateKey = selfsignprivkey;
-    } else {
-      /*CERTCertificate *issuer = CERT_FindCertByNickname(handle, issuerNickName);*/
+    if( !selfsign ) {
       CERTCertificate *issuer = PK11_FindCertFromNickname(issuerNickName, pwarg);
       if( (CERTCertificate *)NULL == issuer ) {
         SECU_PrintError(progName, "unable to find issuer with nickname %s", 
@@ -1688,7 +1676,8 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
         return (SECItem *)NULL;
       }
 
-      caPrivateKey = PK11_FindKeyByAnyCert(issuer, pwarg);
+      privKey = caPrivateKey = PK11_FindKeyByAnyCert(issuer, pwarg);
+      CERT_DestroyCertificate(issuer);
       if (caPrivateKey == NULL) {
 	SECU_PrintError(progName, "unable to retrieve key %s", issuerNickName);
 	return NULL;
@@ -1697,7 +1686,7 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
 	
     arena = cert->arena;
 
-    switch(caPrivateKey->keyType) {
+    switch(privKey->keyType) {
       case rsaKey:
 	algID = SEC_OID_PKCS1_MD5_WITH_RSA_ENCRYPTION;
 	break;
@@ -1722,7 +1711,8 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
 
     der.len = 0;
     der.data = NULL;
-    dummy = SEC_ASN1EncodeItem (arena, &der, cert, CERT_CertificateTemplate);
+    dummy = SEC_ASN1EncodeItem (arena, &der, cert,
+			 	SEC_ASN1_GET(CERT_CertificateTemplate));
     if (!dummy) {
 	fprintf (stderr, "Could not encode certificate.\n");
 	goto done;
@@ -1734,7 +1724,7 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
 	goto done;
     }
 
-    rv = SEC_DerSignData (arena, result, der.data, der.len, caPrivateKey,
+    rv = SEC_DerSignData (arena, result, der.data, der.len, privKey,
 			  algID);
     if (rv != SECSuccess) {
 	fprintf (stderr, "Could not sign encoded certificate data.\n");
@@ -1744,7 +1734,9 @@ SECKEYPrivateKey *selfsignprivkey, char *issuerNickName, void *pwarg)
     }
     cert->derCert = *result;
 done:
-    SECKEY_DestroyPrivateKey(caPrivateKey);
+    if (caPrivateKey) {
+	SECKEY_DestroyPrivateKey(caPrivateKey);
+    }
     return result;
 }
 
@@ -1953,7 +1945,7 @@ CreateCert(
     do {
 	arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
 	if (!arena) {
-	    GEN_BREAK (SEC_ERROR_NO_MEMORY);
+	    GEN_BREAK (SECFailure);
 	}
 	
 	/* Create a certrequest object from the input cert request der */
@@ -2052,6 +2044,7 @@ enum {
     cmd_ListCerts,
     cmd_ModifyCertTrust,
     cmd_NewDBs,
+    cmd_DumpChain,
     cmd_CertReq,
     cmd_CreateAndAddCert,
     cmd_TokenReset,
@@ -2095,6 +2088,7 @@ enum {
     opt_Validity,
     opt_OffsetMonths,
     opt_SelfSign,
+    opt_RW,
     opt_Exponent,
     opt_NoiseFile
 };
@@ -2112,6 +2106,7 @@ static secuCommandFlag certutil_commands[] =
 	{ /* cmd_ListCerts           */  'L', PR_FALSE, 0, PR_FALSE },
 	{ /* cmd_ModifyCertTrust     */  'M', PR_FALSE, 0, PR_FALSE },
 	{ /* cmd_NewDBs              */  'N', PR_FALSE, 0, PR_FALSE },
+	{ /* cmd_DumpChain           */  'O', PR_FALSE, 0, PR_FALSE },
 	{ /* cmd_CertReq             */  'R', PR_FALSE, 0, PR_FALSE },
 	{ /* cmd_CreateAndAddCert    */  'S', PR_FALSE, 0, PR_FALSE },
 	{ /* cmd_TokenReset          */  'T', PR_FALSE, 0, PR_FALSE },
@@ -2155,6 +2150,7 @@ static secuCommandFlag certutil_options[] =
 	{ /* opt_Validity            */  'v', PR_TRUE,  0, PR_FALSE },
 	{ /* opt_OffsetMonths        */  'w', PR_TRUE,  0, PR_FALSE },
 	{ /* opt_SelfSign            */  'x', PR_FALSE, 0, PR_FALSE },
+	{ /* opt_RW                  */  'X', PR_FALSE, 0, PR_FALSE },
 	{ /* opt_Exponent            */  'y', PR_TRUE,  0, PR_FALSE },
 	{ /* opt_NoiseFile           */  'z', PR_TRUE,  0, PR_FALSE }
 };
@@ -2183,8 +2179,9 @@ main(int argc, char **argv)
     int         commandsEntered = 0;
     char        commandToRun    = '\0';
     secuPWData  pwdata          = { PW_NONE, 0 };
+    PRBool 	readOnly	= PR_FALSE;
 
-    SECKEYPrivateKey *privkey;
+    SECKEYPrivateKey *privkey = NULL;
     SECKEYPublicKey *pubkey = NULL;
 
     int i;
@@ -2333,6 +2330,15 @@ main(int argc, char **argv)
 	Usage(progName);
     }
 
+    if (certutil.commands[cmd_ListCerts].activated ||
+         certutil.commands[cmd_PrintHelp].activated ||
+         certutil.commands[cmd_ListKeys].activated ||
+         certutil.commands[cmd_ListModules].activated ||
+         certutil.commands[cmd_CheckCertValidity].activated ||
+         certutil.commands[cmd_Version].activated ) {
+	readOnly = !certutil.options[opt_RW].activated;
+    }
+
     /*  -A, -D, -F, -M, -S, -V, and all require -n  */
     if ((certutil.commands[cmd_AddCert].activated ||
          certutil.commands[cmd_DeleteCert].activated ||
@@ -2405,7 +2411,8 @@ main(int argc, char **argv)
          !certutil.options[opt_SerialNumber].activated) {
 	/*  Make a default serial number from the current time.  */
 	PRTime now = PR_Now();
-	serialNumber = LL_L2UI(serialNumber, now);
+	serialNumber = LL_L2I(serialNumber, now);
+	if (serialNumber < 0) serialNumber *= -1;
     }
 
     /*  Validation needs the usage to validate for.  */
@@ -2430,6 +2437,7 @@ main(int argc, char **argv)
     /*  Using slotname == NULL for listing keys and certs on all slots, 
      *  but only that. */
     if (!(certutil.commands[cmd_ListKeys].activated ||
+	  certutil.commands[cmd_DumpChain].activated ||
     	  certutil.commands[cmd_ListCerts].activated) && slotname == NULL) {
 	PR_fprintf(PR_STDERR,
 	           "%s -%c: cannot use \"-h all\" for this command.\n",
@@ -2489,10 +2497,11 @@ main(int argc, char **argv)
     /*  Initialize NSPR and NSS.  */
     PR_Init(PR_SYSTEM_THREAD, PR_PRIORITY_NORMAL, 1);
     rv = NSS_Initialize(SECU_ConfigDirectory(NULL), certPrefix, certPrefix,
-                        "secmod.db", 0);
+                        "secmod.db", readOnly ? NSS_INIT_READONLY: 0);
     if (rv != SECSuccess) {
 	SECU_PrintPRandOSError(progName);
-	return 255;
+	rv = SECFailure;
+	goto shutdown;
     }
     certHandle = CERT_GetDefaultCertDB();
 
@@ -2519,40 +2528,47 @@ main(int argc, char **argv)
 	               certutil.options[opt_BinaryDER].activated,
 	               certutil.options[opt_ASCIIForIO].activated, 
                        (outFile) ? outFile : PR_STDOUT, &pwdata);
-	return rv ? 255 : 0;
+	goto shutdown;
+    }
+    if (certutil.commands[cmd_DumpChain].activated) {
+	rv = DumpChain(certHandle, name, slot,
+	               certutil.options[opt_BinaryDER].activated,
+	               certutil.options[opt_ASCIIForIO].activated, 
+                       (outFile) ? outFile : PR_STDOUT, &pwdata);
+	goto shutdown;
     }
     /*  XXX needs work  */
     /*  List keys (-K)  */
     if (certutil.commands[cmd_ListKeys].activated) {
 	rv = ListKeys(slot, name, 0 /*keyindex*/, keytype, PR_FALSE /*dopriv*/,
 	              &pwdata);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  List modules (-U)  */
     if (certutil.commands[cmd_ListModules].activated) {
 	rv = ListModules();
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  Delete cert (-D)  */
     if (certutil.commands[cmd_DeleteCert].activated) {
 	rv = DeleteCert(certHandle, name);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  Delete key (-F)  */
     if (certutil.commands[cmd_DeleteKey].activated) {
 	rv = DeleteKey(name, &pwdata);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  Modify trust attribute for cert (-M)  */
     if (certutil.commands[cmd_ModifyCertTrust].activated) {
 	rv = ChangeTrustAttributes(certHandle, name, 
 	                           certutil.options[opt_Trust].arg);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  Change key db password (-W) (future - change pw to slot?)  */
     if (certutil.commands[cmd_ChangePassword].activated) {
 	rv = SECU_ChangePW(slot, 0, certutil.options[opt_PasswordFile].arg);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
     /*  Reset the a token */
     if (certutil.commands[cmd_TokenReset].activated) {
@@ -2563,9 +2579,8 @@ main(int argc, char **argv)
  	}
 	rv = PK11_ResetToken(slot,sso_pass);
 
- 	return !rv - 1;
+	goto shutdown;
     }
-
     /*  Check cert validity against current time (-V)  */
     if (certutil.commands[cmd_CheckCertValidity].activated) {
 	/* XXX temporary hack for fips - must log in to get priv key */
@@ -2579,7 +2594,7 @@ main(int argc, char **argv)
 			  certutil.options[opt_VerifySig].activated,
 			  certutil.options[opt_DetailedInfo].activated,
 	                  &pwdata);
-	return rv ? 255 : 0;
+	goto shutdown;
     }
 
     /*
@@ -2600,14 +2615,16 @@ main(int argc, char **argv)
 	                                &pwdata);
 	if (privkey == NULL) {
 	    SECU_PrintError(progName, "unable to generate key(s)\n");
-	    return 255;
+	    rv = SECFailure;
+	    goto shutdown;
 	}
 	privkey->wincx = &pwdata;
 	PORT_Assert(pubkey != NULL);
 
 	/*  If all that was needed was keygen, exit.  */
 	if (certutil.commands[cmd_GenKeyPair].activated) {
-	    return SECSuccess;
+	    rv = SECSuccess;
+	    goto shutdown;
 	}
     }
 
@@ -2623,7 +2640,7 @@ main(int argc, char **argv)
 	             certutil.options[opt_ASCIIForIO].activated,
 		     outFile ? outFile : PR_STDOUT);
 	if (rv) 
-	    return 255;
+	    goto shutdown;
 	privkey->wincx = &pwdata;
     }
 
@@ -2640,13 +2657,15 @@ main(int argc, char **argv)
 	if (!inFile) {
 	    PR_fprintf(PR_STDERR, "Failed to open file \"%s\" (%ld, %ld).\n",
                        certreqfile, PR_GetError(), PR_GetOSError());
-	    return 255;
+	    rv = SECFailure;
+	    goto shutdown;
 	}
 	outFile = PR_Open(certfile, PR_RDWR | PR_CREATE_FILE, 00660);
 	if (!outFile) {
 	    PR_fprintf(PR_STDERR, "Failed to open file \"%s\" (%ld, %ld).\n",
                        certfile, PR_GetError(), PR_GetOSError());
-	    return 255;
+	    rv = SECFailure;
+	    goto shutdown;
 	}
     }
 
@@ -2666,7 +2685,7 @@ main(int argc, char **argv)
 	                certutil.options[opt_AddCRLDistPtsExt].activated,
 	                certutil.options[opt_AddNSCertTypeExt].activated);
 	if (rv) 
-	    return 255;
+	    goto shutdown;
     }
 
     /* 
@@ -2680,7 +2699,8 @@ main(int argc, char **argv)
 	if (!inFile) {
 	    PR_fprintf(PR_STDERR, "Failed to open file \"%s\" (%ld, %ld).\n",
                        certfile, PR_GetError(), PR_GetOSError());
-	    return 255;
+	    rv = SECFailure;
+	    goto shutdown;
 	}
     }
 
@@ -2691,9 +2711,9 @@ main(int argc, char **argv)
 	             certutil.options[opt_Trust].arg,
 	             inFile, 
 	             certutil.options[opt_ASCIIForIO].activated,
-	             certutil.commands[cmd_AddEmailCert].activated);
+	             certutil.commands[cmd_AddEmailCert].activated,&pwdata);
 	if (rv) 
-	    return 255;
+	    goto shutdown;
     }
 
     if (certutil.commands[cmd_CreateAndAddCert].activated) {
@@ -2702,9 +2722,21 @@ main(int argc, char **argv)
 	PR_Delete(certreqfile);
     }
 
-    if ( certHandle ) {
-	CERT_ClosePermCertDB(certHandle);
+shutdown:
+    if (slot) {
+	PK11_FreeSlot(slot);
     }
+    if (privkey) {
+	SECKEY_DestroyPrivateKey(privkey);
+    }
+    if (pubkey) {
+	SECKEY_DestroyPublicKey(pubkey);
+    }
+    NSS_Shutdown();
 
-    return rv;  
+    if (rv == SECSuccess) {
+	return 0;
+    } else {
+	return 255;
+    }
 }
