@@ -38,6 +38,7 @@
 #include "p12plcy.h"
 #include "pk12util.h"
 #include "nss.h"
+#include "secport.h"
 
 #define PKCS12_IN_BUFFER_SIZE	200
 
@@ -94,10 +95,6 @@ p12u_DestroyExportFileInfo(p12uContext **exp_ptr, PRBool removeFile)
 	PR_Close((*exp_ptr)->file);
     }
 
-    if((*exp_ptr)->data != NULL) {
-	SECITEM_ZfreeItem((*exp_ptr)->data, PR_FALSE);
-    }
-
     if((*exp_ptr)->filename != NULL) {
 	if(removeFile) {
 	    PR_Delete((*exp_ptr)->filename);
@@ -139,139 +136,6 @@ p12u_InitFile(PRBool fileImport, char *filename)
     return p12cxt;
 }
 
-#define TEMPFILE "Pk12uTemp"
-
-static p12uContext *
-p12u_CreateTemporaryDigestFile(void)
-{
-    p12uContext *p12cxt;
-#if defined(_WIN32) || defined(_WINDOWS) || defined(XP_OS2)
-    char *tmpdir,*filename,*last;
-    int len;
-#endif
-
-    p12cxt = (p12uContext *)PR_CALLOC(sizeof(p12uContext));
-    if(!p12cxt) {
-	PR_SetError(SEC_ERROR_NO_MEMORY, 0);
-	return NULL;
-    }
-#if defined(_WIN32) || defined(_WINDOWS) || defined(XP_OS2)
-    tmpdir = getenv("TMP");
-    if (!tmpdir)
-	tmpdir = getenv("TMPDIR");
-    if (!tmpdir)
-	tmpdir = getenv("TEMP");
-    if (!tmpdir)
-	tmpdir = ".";
-    len = strlen(tmpdir);
-    filename = PORT_Alloc(len+sizeof(TEMPFILE)+2);
-    if (filename == NULL) {
-	PR_SetError(SEC_ERROR_NO_MEMORY, 0);
-	p12u_DestroyExportFileInfo(&p12cxt, PR_FALSE);
-	return NULL;
-    }
-    strcpy(filename, tmpdir);
-    len = strlen(filename);
-    last = tmpdir[len - 1];
-    if ((last != '/') && (last != '\\')) {
-	strcat(filename,"\\");
-    }
-    strcat(filename,TEMPFILE);
-    p12cxt->filename = filename;
-#else
-    p12cxt->filename = strdup("/tmp/"TEMPFILE); 
-#endif
-
-    if (!p12cxt->filename) {
-	PR_SetError(SEC_ERROR_NO_MEMORY, 0);
-	p12u_DestroyExportFileInfo(&p12cxt, PR_FALSE);
-	return NULL;
-    }
-
-    return p12cxt;
-}
-
-static SECStatus
-p12u_DigestOpen(void *arg, PRBool readData)
-{
-    p12uContext *p12cxt = arg;
-
-
-    if(!p12cxt || p12cxt->error || !p12cxt->filename) {
-	return SECFailure;
-    }
-
-    if(readData) {
-	p12cxt->file = PR_Open(p12cxt->filename, PR_RDONLY, 0400);
-    } else {
-	p12cxt->file = PR_Open(p12cxt->filename,
-				   PR_CREATE_FILE | PR_RDWR | PR_TRUNCATE,
-				   0600);
-    }
-
-    if(p12cxt->file == NULL) {
-	p12cxt->error = PR_TRUE;
-	return SECFailure;
-    }
-
-    return SECSuccess;
-}
-
-static SECStatus
-p12u_DigestClose(void *arg, PRBool removeFile)
-{
-    p12uContext *p12cxt = arg;
-    if(!p12cxt || p12cxt->error ||
-       !p12cxt->filename || !p12cxt->file) {
-	return SECFailure;
-    }
-
-    PR_Close(p12cxt->file);
-    p12cxt->file = NULL;
-
-    if(removeFile) {
-	PR_Delete(p12cxt->filename);
-	PR_Free(p12cxt->filename);
-	p12cxt->filename = NULL;
-    }
-
-    return SECSuccess;
-}
-
-static int
-p12u_DigestRead(void *arg, unsigned char *buf, unsigned long len)
-{
-    p12uContext *p12cxt = arg;
-
-    if(!p12cxt || p12cxt->error ||
-       !p12cxt->filename || !p12cxt->file) {
-	return -1;
-    }
-
-    if(!buf || len == 0) {
-	return -1;
-    }
-
-    return PR_Read(p12cxt->file, buf, len);
-}
-
-static int
-p12u_DigestWrite(void *arg, unsigned char *buf, unsigned long len)
-{
-    p12uContext *p12cxt = arg;
-
-    if(!p12cxt || p12cxt->error ||
-       !p12cxt->filename || !p12cxt->file) {
-	return -1;
-    }
-
-    if(!buf || len == 0) {
-	return -1;
-    }
-
-    return PR_Write(p12cxt->file, buf, len);
-}
-
 SECItem *
 P12U_NicknameCollisionCallback(SECItem *old_nick, PRBool *cancel, void *wincx)
 {
@@ -496,7 +360,7 @@ PRIntn
 P12U_ImportPKCS12Object(char *in_file, PK11SlotInfo *slot,
 			secuPWData *slotPw, secuPWData *p12FilePw)
 {
-    p12uContext *p12cxt = NULL, *tmpcxt = NULL;
+    p12uContext *p12cxt = NULL;
     unsigned char inBuf[PKCS12_IN_BUFFER_SIZE];
     SEC_PKCS12DecoderContext *p12dcx = NULL;
     SECItem *pwitem = NULL, uniPwitem = { 0 };
@@ -538,18 +402,9 @@ P12U_ImportPKCS12Object(char *in_file, PK11SlotInfo *slot,
 	goto loser;
     }
 
-    tmpcxt = p12u_CreateTemporaryDigestFile();
-    if(!tmpcxt) {
-	SECU_PrintError(progName,"Create Temporary digest file failed");
-	pk12uErrno = PK12UERR_TMPDIGCREATE;
-	goto loser;
-    }
-
     /* init the decoder context */
     p12dcx = SEC_PKCS12DecoderStart(&uniPwitem, slot, slotPw,
-				    p12u_DigestOpen, p12u_DigestClose,
-				    p12u_DigestRead, p12u_DigestWrite,
-				    tmpcxt);
+				    NULL, NULL, NULL, NULL, NULL);
     if(!p12dcx) {
 	SECU_PrintError(progName,"PKCS12 decoder start failed");
 	pk12uErrno = PK12UERR_PK12DECODESTART;
@@ -621,7 +476,7 @@ P12U_ImportPKCS12Object(char *in_file, PK11SlotInfo *slot,
 
     PR_Close(p12cxt->file);
     p12cxt->file = NULL;
-    PK11_FreeSlot(slot);
+    /* PK11_FreeSlot(slot); */
 
     rv = SECSuccess;
 
@@ -977,6 +832,7 @@ main(int argc, char **argv)
     }
 
 done:
+    if (slot) PK11_FreeSlot(slot);
     NSS_Shutdown();
     exit(pk12uErrno);
 }
