@@ -285,6 +285,11 @@ VerifyCertDir(char *dir, char *keyName)
 {
   char fn [FNSIZE];
 
+  /* don't try verifying if we don't have a local directory */
+  if (strncmp(dir,"multiaccess:",sizeof("multiaccess:")-1) == 0) {
+     return;
+  }
+
   sprintf (fn, "%s/cert7.db", dir);
 
   if (PR_Access (fn, PR_ACCESS_EXISTS))
@@ -445,12 +450,12 @@ static int is_dir (char *filename)
  *
  */
 SECItem *
-password_hardcode(void *arg, SECKEYKeyDBHandle *handle)
+password_hardcode(void *arg, void *handle)
 {
   SECItem *pw = NULL;
   if (password) {
     pw = SECITEM_AllocItem(NULL, NULL, PL_strlen(password));
-    pw->data = PL_strdup(password);
+    pw->data = (unsigned char *)PL_strdup(password);
     password = NULL;
   }
   return pw;
@@ -460,8 +465,12 @@ char *
 pk11_password_hardcode(PK11SlotInfo *slot, PRBool retry, void *arg)
 {
   char *pw;
+  if (retry) {
+    return NULL; /* the password is incorrect, fail */
+  }
   pw = password ? PORT_Strdup (password) : NULL;
-  password = NULL;
+  /* XXX don't do this, or FIPS won't work */
+  /*password = NULL;*/
   return pw;
 }
 
@@ -800,8 +809,10 @@ InitCrypto(char *cert_dir, PRBool readOnly)
 			  NULL /*wincx*/) != SECSuccess) {
 				fprintf(stderr, "%s: Unable to authenticate to %s.\n",
 					PROGRAM_NAME, PK11_GetSlotName(slotinfo));
+				PK11_FreeSlot(slotinfo);
 				return -1;
 			}
+			PK11_FreeSlot(slotinfo);
 		}
 
 		/* Make sure there is a password set on the internal key slot */
@@ -824,9 +835,11 @@ InitCrypto(char *cert_dir, PRBool readOnly)
 			  NULL /*wincx*/) != SECSuccess) {
 				fprintf(stderr, "%s: Unable to authenticate to %s.\n",
 					PROGRAM_NAME, PK11_GetSlotName(slotinfo));
+				PK11_FreeSlot(slotinfo);
 				return -1;
 			}
 		}
+		PK11_FreeSlot(slotinfo);
 	}
 
 	return 0;
