@@ -34,7 +34,7 @@
 /*
  * cmsutil -- A command to work with CMS data
  *
- * $Id: cmsutil.c,v 1.25 2001/03/23 19:33:59 relyea%netscape.com Exp $
+ * $Id: cmsutil.c,v 1.30 2002/04/12 19:04:57 relyea%netscape.com Exp $
  */
 
 #include "nspr.h"
@@ -390,7 +390,7 @@ signed_data(struct signOptionsStr *signOptions)
     NSSCMSContentInfo *cinfo;
     NSSCMSSignedData *sigd;
     NSSCMSSignerInfo *signerinfo;
-    CERTCertificate *cert, *ekpcert;
+    CERTCertificate *cert= NULL, *ekpcert = NULL;
 
     if (cms_verbose) {
 	fprintf(stderr, "Input to signed_data:\n");
@@ -400,7 +400,7 @@ signed_data(struct signOptionsStr *signOptions)
 	    fprintf(stderr, "password [NULL]\n");
 	fprintf(stderr, "certUsage [%d]\n", signOptions->options->certUsage);
 	if (signOptions->options->certHandle)
-	    fprintf(stderr, "certdb [%x]\n", signOptions->options->certHandle);
+	    fprintf(stderr, "certdb [%p]\n", signOptions->options->certHandle);
 	else
 	    fprintf(stderr, "certdb [NULL]\n");
 	if (signOptions->nickname)
@@ -413,8 +413,11 @@ signed_data(struct signOptionsStr *signOptions)
         "ERROR: please indicate the nickname of a certificate to sign with.\n");
 	return NULL;
     }
-    if ((cert = CERT_FindCertByNickname(signOptions->options->certHandle, 
-                                        signOptions->nickname)) == NULL) {
+    if ((cert = CERT_FindUserCertByUsage(signOptions->options->certHandle, 
+                                         signOptions->nickname,
+                                         certUsageEmailSigner,
+                                         PR_FALSE,
+                                         NULL)) == NULL) {
 	SECU_PrintError(progName, 
 	                "the corresponding cert for key \"%s\" does not exist",
 	                signOptions->nickname);
@@ -487,18 +490,21 @@ signed_data(struct signOptionsStr *signOptions)
 	    goto loser;
 	}
     }
+
     if (signOptions->encryptionKeyPreferenceNick) {
 	/* get the cert, add it to the message */
-	if ((ekpcert = CERT_FindCertByNickname(signOptions->options->certHandle, 
-	                               signOptions->encryptionKeyPreferenceNick))
+	if ((ekpcert = CERT_FindUserCertByUsage(
+                                     signOptions->options->certHandle, 
+	                             signOptions->encryptionKeyPreferenceNick,
+                                     certUsageEmailRecipient, PR_FALSE, NULL))
 	      == NULL) {
 	    SECU_PrintError(progName, 
-	                 "the corresponding cert for key \"%s\" does not exist",
-	                    signOptions->encryptionKeyPreferenceNick);
+	               "the corresponding cert for key \"%s\" does not exist",
+	                signOptions->encryptionKeyPreferenceNick);
 	    goto loser;
 	}
 	if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, ekpcert, 
-	                                        signOptions->options->certHandle)
+	                                     signOptions->options->certHandle)
 	      != SECSuccess) {
 	    fprintf(stderr, "ERROR: cannot add SMIMEEncKeyPrefs attribute.\n");
 	    goto loser;
@@ -509,15 +515,47 @@ signed_data(struct signOptionsStr *signOptions)
 	}
     } else {
 	/* check signing cert for fitness as encryption cert */
-	/* if yes, add signing cert as EncryptionKeyPreference */
-	if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, cert, 
-	                                        signOptions->options->certHandle)
-	      != SECSuccess) {
-	    fprintf(stderr, 
-	            "ERROR: cannot add default SMIMEEncKeyPrefs attribute.\n");
-	    goto loser;
-	}
+        SECStatus FitForEncrypt = CERT_CheckCertUsage(cert,
+                                                      certUsageEmailRecipient);
+
+        if (SECSuccess == FitForEncrypt) {
+            /* if yes, add signing cert as EncryptionKeyPreference */
+            if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, cert, 
+                                              signOptions->options->certHandle)
+                  != SECSuccess) {
+                fprintf(stderr, 
+                    "ERROR: cannot add default SMIMEEncKeyPrefs attribute.\n");
+                goto loser;
+            }
+        } else {
+            /* this is a dual-key cert case, we need to look for the encryption
+               certificate under the same nickname as the signing cert */
+            /* get the cert, add it to the message */
+            if ((ekpcert = CERT_FindUserCertByUsage(
+                                              signOptions->options->certHandle,
+                                              signOptions->nickname,
+                                              certUsageEmailRecipient,
+                                              PR_FALSE,
+                                              NULL)) == NULL) {
+                SECU_PrintError(progName, 
+                         "the corresponding cert for key \"%s\" does not exist",
+                         signOptions->encryptionKeyPreferenceNick);
+                goto loser;
+            }
+            if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs(signerinfo, ekpcert, 
+                                              signOptions->options->certHandle)
+                  != SECSuccess) {
+                fprintf(stderr, 
+                        "ERROR: cannot add SMIMEEncKeyPrefs attribute.\n");
+                goto loser;
+            }
+            if (NSS_CMSSignedData_AddCertificate(sigd, ekpcert) != SECSuccess) {
+                fprintf(stderr, "ERROR: cannot add encryption certificate.\n");
+                goto loser;
+            }
+        }
     }
+
     if (NSS_CMSSignedData_AddSignerInfo(sigd, signerinfo) != SECSuccess) {
 	fprintf(stderr, "ERROR: cannot add CMS signerInfo object.\n");
 	goto loser;
@@ -525,8 +563,20 @@ signed_data(struct signOptionsStr *signOptions)
     if (cms_verbose) {
 	fprintf(stderr, "created signed-date message\n");
     }
+    if (ekpcert) {
+	CERT_DestroyCertificate(ekpcert);
+    }
+    if (cert) {
+	CERT_DestroyCertificate(cert);
+    }
     return cmsg;
 loser:
+    if (ekpcert) {
+	CERT_DestroyCertificate(ekpcert);
+    }
+    if (cert) {
+	CERT_DestroyCertificate(cert);
+    }
     NSS_CMSMessage_Destroy(cmsg);
     return NULL;
 }
@@ -538,11 +588,11 @@ enveloped_data(struct envelopeOptionsStr *envelopeOptions)
     NSSCMSContentInfo *cinfo;
     NSSCMSEnvelopedData *envd;
     NSSCMSRecipientInfo *recipientinfo;
-    CERTCertificate **recipientcerts;
+    CERTCertificate **recipientcerts = NULL;
     CERTCertDBHandle *dbhandle;
     PLArenaPool *tmppoolp = NULL;
     SECOidTag bulkalgtag;
-    int keysize, i;
+    int keysize, i = 0;
     int cnt;
     dbhandle = envelopeOptions->options->certHandle;
     /* count the recipients */
@@ -569,10 +619,12 @@ enveloped_data(struct envelopeOptionsStr *envelopeOptions)
 	        == NULL) {
 	    SECU_PrintError(progName, "cannot find certificate for \"%s\"", 
 	                    envelopeOptions->recipients[i]);
+	    i=0;
 	    goto loser;
 	}
     }
     recipientcerts[i] = NULL;
+    i=0;
     /* find a nice bulk algorithm */
     if (NSS_SMIMEUtil_FindBulkAlgForRecipients(recipientcerts, &bulkalgtag, 
                                                &keysize) != SECSuccess) {
@@ -623,11 +675,17 @@ enveloped_data(struct envelopeOptionsStr *envelopeOptions)
 	    fprintf(stderr, "ERROR: cannot add CMS recipientInfo object.\n");
 	    goto loser;
 	}
+	CERT_DestroyCertificate(recipientcerts[i]);
     }
     if (tmppoolp)
 	PORT_FreeArena(tmppoolp, PR_FALSE);
     return cmsg;
 loser:
+    if (recipientcerts) {
+	for (; recipientcerts[i] != NULL; i++) {
+	    CERT_DestroyCertificate(recipientcerts[i]);
+	}
+    }
     if (cmsg)
 	NSS_CMSMessage_Destroy(cmsg);
     if (tmppoolp)
@@ -791,10 +849,10 @@ signed_data_certsonly(struct certsonlyOptionsStr *certsonlyOptions)
     NSSCMSMessage *cmsg = NULL;
     NSSCMSContentInfo *cinfo;
     NSSCMSSignedData *sigd;
-    CERTCertificate **certs;
+    CERTCertificate **certs = NULL;
     CERTCertDBHandle *dbhandle;
     PLArenaPool *tmppoolp = NULL;
-    int i, cnt;
+    int i = 0, cnt;
     dbhandle = certsonlyOptions->options->certHandle;
     if ((cnt = nss_CMSArray_Count((void**)certsonlyOptions->recipients)) == 0) {
 	fprintf(stderr, 
@@ -819,10 +877,12 @@ signed_data_certsonly(struct certsonlyOptionsStr *certsonlyOptions)
 	        == NULL) {
 	    SECU_PrintError(progName, "cannot find certificate for \"%s\"", 
 	                    certsonlyOptions->recipients[i]);
+	    i=0;
 	    goto loser;
 	}
     }
     certs[i] = NULL;
+    i=0;
     /*
      * create the message object
      */
@@ -839,12 +899,14 @@ signed_data_certsonly(struct certsonlyOptionsStr *certsonlyOptions)
 	fprintf(stderr, "ERROR: cannot create CMS signedData object.\n");
 	goto loser;
     }
+    CERT_DestroyCertificate(certs[0]);
     for (i=1; i<cnt; i++) {
 	if (NSS_CMSSignedData_AddCertChain(sigd, certs[i])) {
 	    fprintf(stderr, "ERROR: cannot add cert chain for \"%s\".\n",
 	            certsonlyOptions->recipients[i]);
 	    goto loser;
 	}
+	CERT_DestroyCertificate(certs[i]);
     }
     cinfo = NSS_CMSMessage_GetContentInfo(cmsg);
     if (NSS_CMSContentInfo_SetContent_SignedData(cmsg, cinfo, sigd) 
@@ -862,6 +924,11 @@ signed_data_certsonly(struct certsonlyOptionsStr *certsonlyOptions)
 	PORT_FreeArena(tmppoolp, PR_FALSE);
     return cmsg;
 loser:
+    if (certs) {
+	for (; i<cnt; i++) {
+	    CERT_DestroyCertificate(certs[i]);
+	}
+    }
     if (cmsg)
 	NSS_CMSMessage_Destroy(cmsg);
     if (tmppoolp)
@@ -886,7 +953,7 @@ int
 main(int argc, char **argv)
 {
     FILE *outFile;
-    NSSCMSMessage *cmsg;
+    NSSCMSMessage *cmsg = NULL;
     PRFileDesc *inFile;
     PLOptState *optstate;
     PLOptStatus status;
@@ -1261,6 +1328,10 @@ main(int argc, char **argv)
 	    SECU_PrintError(progName, "problem encrypting");
 	    exitstatus = 1;
 	}
+	if (encryptOptions.bulkkey) {
+	    PK11_FreeSymKey(encryptOptions.bulkkey);
+	    encryptOptions.bulkkey = NULL;
+	}
 	break;
     case ENVELOPE:
 	envelopeOptions.options = &options;
@@ -1287,7 +1358,8 @@ main(int argc, char **argv)
 	Usage(progName);
 	exitstatus = 1;
     }
-    if (mode == SIGN || mode == ENVELOPE || mode == CERTSONLY) {
+    if ( (mode == SIGN || mode == ENVELOPE || mode == CERTSONLY)
+         && (!exitstatus) ) {
 	PLArenaPool *arena = PORT_NewArena(1024);
 	NSSCMSEncoderContext *ecx;
 	SECItem output = { 0, 0, 0 };
@@ -1298,8 +1370,8 @@ main(int argc, char **argv)
 	pwcb     = (options.password != NULL) ? ownpw                    : NULL;
 	pwcb_arg = (options.password != NULL) ? (void *)options.password : NULL;
 	if (cms_verbose) {
-	    fprintf(stderr, "cmsg [%x]\n", cmsg);
-	    fprintf(stderr, "arena [%x]\n", arena);
+	    fprintf(stderr, "cmsg [%p]\n", cmsg);
+	    fprintf(stderr, "arena [%p]\n", arena);
 	    if (pwcb_arg)
 		fprintf(stderr, "password [%s]\n", (char *)pwcb_arg);
 	    else
@@ -1332,7 +1404,7 @@ main(int argc, char **argv)
 	}
 	rv = NSS_CMSEncoder_Finish(ecx);
 	if (rv) {
-	    fprintf(stderr, "%s: failed to encode data.\n", progName);
+            SECU_PrintError(progName, "failed to encode data");
 	    exit(1);
 	}
 
@@ -1353,5 +1425,6 @@ main(int argc, char **argv)
 
     if (decodeOptions.contentFile)
 	PR_Close(decodeOptions.contentFile);
+    NSS_Shutdown();
     exit(exitstatus);
 }
