@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.2 $ $Date: 2000/04/19 21:23:13 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.5 $ $Date: 2002/04/04 16:14:26 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -52,6 +52,8 @@ static const char CVS_ID[] = "@(#) $RCSfile: arena.c,v $ $Revision: 1.2 $ $Date:
 #include "prlock.h"
 #include "plarena.h"
 
+#include <string.h>
+
 /*
  * NSSArena
  *
@@ -542,6 +544,8 @@ nssArena_Destroy
   return PR_SUCCESS;
 }
 
+static void *nss_zalloc_arena_locked(NSSArena *arena, PRUint32 size);
+
 /*
  * nssArena_Mark
  *
@@ -604,7 +608,7 @@ nssArena_Mark
   /* No error possible */
 
   /* Do this after the mark */
-  rv = nss_ZNEW(arena, nssArenaMark);
+  rv = (nssArenaMark *)nss_zalloc_arena_locked(arena, sizeof(nssArenaMark));
   if( (nssArenaMark *)NULL == rv ) {
     PR_Unlock(arena->lock);
     nss_SetError(NSS_ERROR_NO_MEMORY);
@@ -612,8 +616,13 @@ nssArena_Mark
   }
 
 #ifdef ARENA_THREADMARK
-  arena->last_mark->next = rv;
-  arena->last_mark = rv;
+  if ( (nssArenaMark *)NULL == arena->first_mark) {
+    arena->first_mark = rv;
+    arena->last_mark = rv;
+  } else {
+    arena->last_mark->next = rv;
+    arena->last_mark = rv;
+  }
 #endif /* ARENA_THREADMARK */
 
   rv->mark = p;
@@ -806,6 +815,35 @@ struct pointer_header {
   PRUint32 size;
 };
 
+static void *
+nss_zalloc_arena_locked
+(
+  NSSArena *arena,
+  PRUint32 size
+)
+{
+  void *p;
+  void *rv;
+  struct pointer_header *h;
+  PRUint32 my_size = size + sizeof(struct pointer_header);
+  PR_ARENA_ALLOCATE(p, &arena->pool, my_size);
+  if( (void *)NULL == p ) {
+    nss_SetError(NSS_ERROR_NO_MEMORY);
+    return (void *)NULL;
+  }
+  /* 
+   * Do this before we unlock.  This way if the user is using
+   * an arena in one thread while destroying it in another, he'll
+   * fault/FMR in his code, not ours.
+   */
+  h = (struct pointer_header *)p;
+  h->arena = arena;
+  h->size = size;
+  rv = (void *)((char *)h + sizeof(struct pointer_header));
+  (void)nsslibc_memset(rv, 0, size);
+  return rv;
+}
+
 /*
  * nss_ZAlloc
  *
@@ -859,7 +897,6 @@ nss_ZAlloc
 
     return (void *)((char *)h + sizeof(struct pointer_header));
   } else {
-    void *p;
     void *rv;
     /* Arena allocation */
 #ifdef NSSDEBUG
@@ -885,26 +922,9 @@ nss_ZAlloc
     }
 #endif /* ARENA_THREADMARK */
 
-    PR_ARENA_ALLOCATE(p, &arenaOpt->pool, my_size);
-    if( (void *)NULL == p ) {
-      PR_Unlock(arenaOpt->lock);
-      nss_SetError(NSS_ERROR_NO_MEMORY);
-      return (void *)NULL;
-    }
-
-    /* 
-     * Do this before we unlock.  This way if the user is using
-     * an arena in one thread while destroying it in another, he'll
-     * fault/FMR in his code, not ours.
-     */
-    h = (struct pointer_header *)p;
-    h->arena = arenaOpt;
-    h->size = size;
-    rv = (void *)((char *)h + sizeof(struct pointer_header));
-    (void)nsslibc_memset(rv, 0, size);
+    rv = nss_zalloc_arena_locked(arenaOpt, size);
 
     PR_Unlock(arenaOpt->lock);
-
     return rv;
   }
   /*NOTREACHED*/
@@ -1107,14 +1127,15 @@ nss_ZRealloc
     new_h = (struct pointer_header *)p;
     new_h->arena = h->arena;
     new_h->size = newSize;
-    rv = (void *)((char *)h + sizeof(struct pointer_header));
-    (void)nsslibc_memcpy(rv, pointer, h->size);
-    (void)nsslibc_memset(&((char *)rv)[ h->size ], 0, 
-                         (newSize - h->size));
-    (void)nsslibc_memset(pointer, 0, h->size);
+    rv = (void *)((char *)new_h + sizeof(struct pointer_header));
+    if (rv != pointer) {
+	(void)nsslibc_memcpy(rv, pointer, h->size);
+	(void)nsslibc_memset(pointer, 0, h->size);
+    }
+    (void)nsslibc_memset(&((char *)rv)[ h->size ], 0, (newSize - h->size));
     h->arena = (NSSArena *)NULL;
     h->size = 0;
-    PR_Unlock(h->arena->lock);
+    PR_Unlock(new_h->arena->lock);
     return rv;
   }
   /*NOTREACHED*/
