@@ -35,7 +35,7 @@
 #define BASET_H
 
 #ifdef DEBUG
-static const char BASET_CVS_ID[] = "@(#) $RCSfile: baset.h,v $ $Revision: 1.2 $ $Date: 2001/01/03 19:48:42 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char BASET_CVS_ID[] = "@(#) $RCSfile: baset.h,v $ $Revision: 1.6 $ $Date: 2001/10/15 16:50:01 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -99,6 +99,17 @@ typedef struct nssArenaMarkStr nssArenaMark;
 
 #endif /* DEBUG */
 
+typedef struct nssListStr nssList;
+typedef struct nssListIteratorStr nssListIterator;
+typedef PRBool (* nssListCompareFunc)(void *a, void *b);
+typedef PRIntn (* nssListSortFunc)(void *a, void *b);
+typedef void (* nssListElementDestructorFunc)(void *el);
+
+typedef struct nssHashStr nssHash;
+typedef void (PR_CALLBACK *nssHashIterator)(const void *key, 
+                                            void *value, 
+                                            void *arg);
+
 /*
  * nssPointerTracker
  *
