@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: hashops.c,v $ $Revision: 1.2 $ $Date: 2001/01/31 18:01:45 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: hashops.c,v $ $Revision: 1.4 $ $Date: 2002/02/13 02:26:43 $ $Name: MOZILLA_1_1b_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -52,7 +52,7 @@ nss_arena_hash_alloc_table
   PRSize size
 )
 {
-  NSSArena *arena = (NSSArena *)pool;
+  NSSArena *arena = (NSSArena *)NULL;
 
 #ifdef NSSDEBUG
   if( (void *)NULL != arena ) {
@@ -82,7 +82,7 @@ nss_arena_hash_alloc_entry
   const void *key
 )
 {
-  NSSArena *arena = (NSSArena *)pool;
+  NSSArena *arena = NULL;
 
 #ifdef NSSDEBUG
   if( (void *)NULL != arena ) {
