@@ -34,7 +34,7 @@
 /*
  * Certificate handling code
  *
- * $Id: certdb.c,v 1.13 2001/06/21 03:20:09 nelsonb%netscape.com Exp $
+ * $Id: certdb.c,v 1.32.2.1 2002/08/04 04:50:34 wtc%netscape.com Exp $
  */
 
 #include "nssilock.h"
@@ -58,7 +58,14 @@
 #include "secerr.h"
 #include "sslerr.h"
 #include "nsslocks.h"
-#include "cdbhdl.h"
+#include "pk11func.h"
+#include "xconst.h"   /* for  CERT_DecodeAltNameExtension */
+
+#ifndef NSS_3_4_CODE
+#define NSS_3_4_CODE
+#endif /* NSS_3_4_CODE */
+#include "pki.h"
+#include "pki3hack.h"
 
 /*
  * Certificate database handling code
@@ -208,6 +215,7 @@ const SEC_ASN1Template CERT_CertKeyTemplate[] = {
 };
 
 SEC_ASN1_CHOOSER_IMPLEMENT(CERT_CertificateTemplate)
+SEC_ASN1_CHOOSER_IMPLEMENT(SEC_SignedCertificateTemplate)
 
 SECStatus
 CERT_KeyFromIssuerAndSN(PRArenaPool *arena, SECItem *issuer, SECItem *sn,
@@ -457,7 +465,7 @@ fortezzaIsCA( CERTCertificate *cert) {
 	unsigned char *end;
 	int len;
 
-        rawkey = spki->subjectPublicKey;
+	rawkey = spki->subjectPublicKey;
 	DER_ConvertBitString(&rawkey);
 	rawptr = rawkey.data;
 	end = rawkey.data + rawkey.len;
@@ -484,7 +492,7 @@ fortezzaIsCA( CERTCertificate *cert) {
 
 	/* DSSPrivilege (the string up to the first byte with the hi-bit on */
 	if (*rawptr & 0x30) isCA = PR_TRUE;
-        
+	
    }
    return isCA;
 }
@@ -622,12 +630,12 @@ CERT_GetCertType(CERTCertificate *cert)
 	cert->nsCertType = NS_CERT_TYPE_SSL_CLIENT | NS_CERT_TYPE_SSL_SERVER |
 	    NS_CERT_TYPE_EMAIL;
 
-        /* if the basic constraint extension says the cert is a CA, then
+	/* if the basic constraint extension says the cert is a CA, then
 	   allow SSL CA and EMAIL CA and Status Responder */
 	if ((basicConstraintPresent == PR_TRUE)
 	    && (basicConstraint.isCA)) {
-	        cert->nsCertType |= NS_CERT_TYPE_SSL_CA;
-	        cert->nsCertType |= NS_CERT_TYPE_EMAIL_CA;
+		cert->nsCertType |= NS_CERT_TYPE_SSL_CA;
+		cert->nsCertType |= NS_CERT_TYPE_EMAIL_CA;
 		cert->nsCertType |= EXT_KEY_USAGE_STATUS_RESPONDER;
 	} else if (CERT_IsCACert(cert, NULL) == PR_TRUE) {
 		cert->nsCertType |= EXT_KEY_USAGE_STATUS_RESPONDER;
@@ -635,8 +643,8 @@ CERT_GetCertType(CERTCertificate *cert)
 
 	/* if the cert is a fortezza CA cert, then allow SSL CA and EMAIL CA */
 	if (fortezzaIsCA(cert)) {
-	        cert->nsCertType |= NS_CERT_TYPE_SSL_CA;
-	        cert->nsCertType |= NS_CERT_TYPE_EMAIL_CA;
+		cert->nsCertType |= NS_CERT_TYPE_SSL_CA;
+		cert->nsCertType |= NS_CERT_TYPE_EMAIL_CA;
 	}
     }
 
@@ -683,9 +691,9 @@ cert_GetKeyID(CERTCertificate *cert)
 
 	    cert->subjectKeyID.data = (unsigned char *)PORT_ArenaAlloc(cert->arena, 8);
 	    if ( cert->subjectKeyID.data != NULL ) {
-	        PORT_Memcpy(cert->subjectKeyID.data, key->u.fortezza.KMID, 8);
-	        cert->subjectKeyID.len = 8;
-	        cert->keyIDGenerated = PR_FALSE;
+		PORT_Memcpy(cert->subjectKeyID.data, key->u.fortezza.KMID, 8);
+		cert->subjectKeyID.len = 8;
+		cert->keyIDGenerated = PR_FALSE;
 	    }
 	}
 		
@@ -700,7 +708,7 @@ cert_GetKeyID(CERTCertificate *cert)
 	 */
 	cert->subjectKeyID.data = (unsigned char *)PORT_ArenaAlloc(cert->arena, SHA1_LENGTH);
 	if ( cert->subjectKeyID.data != NULL ) {
-	    rv = SHA1_HashBuf(cert->subjectKeyID.data,
+	    rv = PK11_HashBuf(SEC_OID_SHA1,cert->subjectKeyID.data,
 			      cert->derPublicKey.data,
 			      cert->derPublicKey.len);
 	    if ( rv == SECSuccess ) {
@@ -720,7 +728,7 @@ cert_GetKeyID(CERTCertificate *cert)
  * take a DER certificate and decode it into a certificate structure
  */
 CERTCertificate *
-__CERT_DecodeDERCertificate(SECItem *derSignedCert, PRBool copyDER,
+CERT_DecodeDERCertificate(SECItem *derSignedCert, PRBool copyDER,
 			 char *nickname)
 {
     CERTCertificate *cert;
@@ -828,7 +836,7 @@ __CERT_DecodeDERCertificate(SECItem *derSignedCert, PRBool copyDER,
     
     cert->referenceCount = 1;
     cert->slot = NULL;
-    cert->pkcs11ID = CK_INVALID_KEY;
+    cert->pkcs11ID = CK_INVALID_HANDLE;
     cert->dbnickname = NULL;
     
     return(cert);
@@ -843,12 +851,13 @@ loser:
 }
 
 CERTCertificate *
-CERT_DecodeDERCertificate(SECItem *derSignedCert, PRBool copyDER,
+__CERT_DecodeDERCertificate(SECItem *derSignedCert, PRBool copyDER,
 			 char *nickname)
 {
-    return(__CERT_DecodeDERCertificate(derSignedCert, copyDER, nickname));
+    return CERT_DecodeDERCertificate(derSignedCert, copyDER, nickname);
 }
 
+
 /*
 ** Amount of time that a certifiate is allowed good before it is actually
 ** good. This is used for pending certificates, ones that are about to be
@@ -904,7 +913,7 @@ CERT_CheckCertValidTimes(CERTCertificate *c, PRTime t, PRBool allowOverride)
 
     /* if cert is already marked OK, then don't bother to check */
     if ( allowOverride && c->timeOK ) {
-        return(secCertTimeValid);
+	return(secCertTimeValid);
     }
 
     rv = CERT_GetCertTimes(c, &notBefore, &notAfter);
@@ -1152,9 +1161,14 @@ CERTCertificate *
 CERT_DupCertificate(CERTCertificate *c)
 {
     if (c) {
+#ifdef NSS_CLASSIC
 	CERT_LockCertRefCount(c);
 	++c->referenceCount;
 	CERT_UnlockCertRefCount(c);
+#else
+	NSSCertificate *tmp = STAN_GetNSSCertificate(c);
+	nssCertificate_AddRef(tmp);
+#endif
     }
     return c;
 }
@@ -1179,68 +1193,6 @@ CERT_GetDefaultCertDB(void)
     return(default_cert_db_handle);
 }
 
-/*
- * Open volatile certificate database and index databases.  This is a
- * fallback if the real databases can't be opened or created.  It is only
- * resident in memory, so it will not be persistent.  We do this so that
- * we don't crash if the databases can't be created.
- */
-SECStatus
-CERT_OpenVolatileCertDB(CERTCertDBHandle *handle)
-{
-#define DBM_DEFAULT 0
-    static const HASHINFO hashInfo = {
-        DBM_DEFAULT,    /* bucket size */
-        DBM_DEFAULT,    /* fill factor */
-        DBM_DEFAULT,    /* number of elements */
-        256 * 1024, 	/* bytes to cache */
-        DBM_DEFAULT,    /* hash function */
-        DBM_DEFAULT     /* byte order */
-    };
-    /*
-     * Open the memory resident perm cert database.
-     */
-    handle->permCertDB = dbopen(0, O_RDWR | O_CREAT, 0600, DB_HASH, &hashInfo);
-    if ( !handle->permCertDB ) {
-	goto loser;
-    }
-
-    /*
-     * Open the memory resident decoded cert database.
-     */
-    handle->tempCertDB = dbopen(0, O_RDWR | O_CREAT, 0600, DB_HASH, &hashInfo);
-    if ( !handle->tempCertDB ) {
-	goto loser;
-    }
-
-    handle->dbMon = PZ_NewMonitor(nssILockCertDB);
-    PORT_Assert(handle->dbMon != NULL);
-
-    handle->spkDigestInfo = NULL;
-    handle->statusConfig = NULL;
-
-    /* initialize the cert database */
-    (void) CERT_InitCertDB(handle);
-
-    return (SECSuccess);
-    
-loser:
-
-    PORT_SetError(SEC_ERROR_BAD_DATABASE);
-
-    if ( handle->permCertDB ) {
-	(* handle->permCertDB->close)(handle->permCertDB);
-	handle->permCertDB = 0;
-    }
-
-    if ( handle->tempCertDB ) {
-	(* handle->tempCertDB->close)(handle->tempCertDB);
-	handle->tempCertDB = 0;
-    }
-
-    return(SECFailure);
-}
-
 /* XXX this would probably be okay/better as an xp routine? */
 static void
 sec_lower_string(char *s)
@@ -1265,14 +1217,14 @@ SECStatus
 CERT_AddOKDomainName(CERTCertificate *cert, const char *hn)
 {
     CERTOKDomainName *domainOK;
-    int               newNameLen;
+    int	       newNameLen;
 
     if (!hn || !(newNameLen = strlen(hn))) {
     	PORT_SetError(SEC_ERROR_INVALID_ARGS);
 	return SECFailure;
     }
     domainOK = (CERTOKDomainName *)PORT_ArenaZAlloc(cert->arena, 
-                                  (sizeof *domainOK) + newNameLen);
+				  (sizeof *domainOK) + newNameLen);
     if (!domainOK) 
     	return SECFailure;	/* error code is already set. */
 
@@ -1285,6 +1237,186 @@ CERT_AddOKDomainName(CERTCertificate *cert, const char *hn)
     return SECSuccess;
 }
 
+/* returns SECSuccess if hn matches pattern cn,
+** returns SECFailure with SSL_ERROR_BAD_CERT_DOMAIN if no match,
+** returns SECFailure with some other error code if another error occurs.
+**
+** may modify cn, so caller must pass a modifiable copy.
+*/
+static SECStatus
+cert_TestHostName(char * cn, const char * hn)
+{
+    char * hndomain;
+    int    regvalid;
+
+    if ((hndomain = PORT_Strchr(hn, '.')) == NULL) {
+	/* No domain in URI host name */
+	char * cndomain;
+	if ((cndomain = PORT_Strchr(cn, '.')) != NULL &&
+	    (cndomain - cn) > 0) {
+	    /* there is a domain in the cn string, so chop it off */
+	    *cndomain = '\0';
+	}
+    }
+
+    regvalid = PORT_RegExpValid(cn);
+    if (regvalid != NON_SXP) {
+	SECStatus rv;
+	/* cn is a regular expression, try to match the shexp */
+	int match = PORT_RegExpCaseSearch(hn, cn);
+
+	if ( match == 0 ) {
+	    rv = SECSuccess;
+	} else {
+	    PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
+	    rv = SECFailure;
+	}
+	return rv;
+    } 
+    /* cn is not a regular expression */
+
+    /* compare entire hn with cert name */
+    if (PORT_Strcasecmp(hn, cn) == 0) {
+	return SECSuccess;
+    }
+	    
+    if ( hndomain ) {
+	/* compare just domain name with cert name */
+	if ( PORT_Strcasecmp(hndomain+1, cn) == 0 ) {
+	    return SECSuccess;
+	}
+    }
+
+    PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
+    return SECFailure;
+}
+
+
+SECStatus
+cert_VerifySubjectAltName(CERTCertificate *cert, const char *hn)
+{
+    PRArenaPool *     arena          = NULL;
+    CERTGeneralName * nameList       = NULL;
+    CERTGeneralName * current;
+    char *            cn;
+    int               cnBufLen;
+    unsigned int      hnLen;
+    int               DNSextCount    = 0;
+    int               IPextCount     = 0;
+    PRBool            isIPaddr;
+    SECStatus         rv             = SECFailure;
+    SECItem           subAltName;
+    PRNetAddr         netAddr;
+    char              cnbuf[128];
+
+    subAltName.data = NULL;
+    hnLen    = strlen(hn);
+    cn       = cnbuf;
+    cnBufLen = sizeof cnbuf;
+
+    rv = CERT_FindCertExtension(cert, SEC_OID_X509_SUBJECT_ALT_NAME, 
+				&subAltName);
+    if (rv != SECSuccess) {
+	goto finish;
+    }
+    isIPaddr = (PR_SUCCESS == PR_StringToNetAddr(hn, &netAddr));
+    rv = SECFailure;
+    arena = PORT_NewArena(DER_DEFAULT_CHUNKSIZE);
+    if (!arena) 
+	goto finish;
+
+    nameList = current = CERT_DecodeAltNameExtension(arena, &subAltName);
+    if (!current)
+    	goto finish;
+
+    do {
+	switch (current->type) {
+	case certDNSName:
+	    if (!isIPaddr) {
+		/* DNS name current->name.other.data is not null terminated.
+		** so must copy it.  
+		*/
+		int cnLen = current->name.other.len;
+		if (cnLen + 1 > cnBufLen) {
+		    cnBufLen = cnLen + 1;
+		    cn = (char *)PORT_ArenaAlloc(arena, cnBufLen);
+		    if (!cn)
+			goto finish;
+		}
+		PORT_Memcpy(cn, current->name.other.data, 
+		                current->name.other.len);
+		cn[cnLen] = 0;
+		rv = cert_TestHostName(cn ,hn);
+		if (rv == SECSuccess)
+		    goto finish;
+	    }
+	    DNSextCount++;
+	    break;
+	case certIPAddress:
+	    if (isIPaddr) {
+		int match = 0;
+		PRIPv6Addr v6Addr;
+		if (current->name.other.len == 4 &&         /* IP v4 address */
+		    netAddr.inet.family == PR_AF_INET) {
+		    match = !memcmp(&netAddr.inet.ip, 
+		                    current->name.other.data, 4);
+		} else if (current->name.other.len == 16 && /* IP v6 address */
+		    netAddr.ipv6.family == PR_AF_INET6) {
+		    match = !memcmp(&netAddr.ipv6.ip,
+		                     current->name.other.data, 16);
+		} else if (current->name.other.len == 16 && /* IP v6 address */
+		    netAddr.inet.family == PR_AF_INET) {
+		    /* convert netAddr to ipv6, then compare. */
+		    /* ipv4 must be in Network Byte Order on input. */
+		    PR_ConvertIPv4AddrToIPv6(netAddr.inet.ip, &v6Addr);
+		    match = !memcmp(&v6Addr, current->name.other.data, 16);
+		} else if (current->name.other.len == 4 &&  /* IP v4 address */
+		    netAddr.inet.family == PR_AF_INET6) {
+		    /* convert netAddr to ipv6, then compare. */
+		    PRUint32 ipv4 = (current->name.other.data[0] << 24) |
+		                    (current->name.other.data[1] << 16) |
+				    (current->name.other.data[2] <<  8) |
+				     current->name.other.data[3];
+		    /* ipv4 must be in Network Byte Order on input. */
+		    PR_ConvertIPv4AddrToIPv6(PR_htonl(ipv4), &v6Addr);
+		    match = !memcmp(&netAddr.ipv6.ip, &v6Addr, 16);
+		} 
+		if (match) {
+		    rv = SECSuccess;
+		    goto finish;
+		}
+	    }
+	    IPextCount++;
+	    break;
+	default:
+	    break;
+	}
+	current = cert_get_next_general_name(current);
+    } while (current != nameList);
+
+    if ((!isIPaddr && !DNSextCount) || (isIPaddr && !IPextCount)) {
+	/* no relevant value in the extension was found. */
+	PORT_SetError(SEC_ERROR_EXTENSION_NOT_FOUND);
+    } else {
+	PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
+    }
+    rv = SECFailure;
+
+finish:
+
+    /* Don't free nameList, it's part of the arena. */
+    if (arena) {
+	PORT_FreeArena(arena, PR_FALSE);
+    }
+
+    if (subAltName.data) {
+	SECITEM_FreeItem(&subAltName, PR_FALSE);
+    }
+
+    return rv;
+}
+
+
 /* Make sure that the name of the host we are connecting to matches the
  * name that is incoded in the common-name component of the certificate
  * that they are using.
@@ -1293,11 +1425,6 @@ SECStatus
 CERT_VerifyCertName(CERTCertificate *cert, const char *hn)
 {
     char *    cn;
-    char *    domain;
-    char *    hndomain;
-    char *    hostname;
-    int       regvalid;
-    int       match;
     SECStatus rv;
     CERTOKDomainName *domainOK;
 
@@ -1306,86 +1433,31 @@ CERT_VerifyCertName(CERTCertificate *cert, const char *hn)
 	return SECFailure;
     }
 
-    hostname = PORT_Strdup(hn);
-    if ( hostname == NULL ) {
-	return(SECFailure);
-    }
-    sec_lower_string(hostname);
-
     /* if the name is one that the user has already approved, it's OK. */
     for (domainOK = cert->domainOK; domainOK; domainOK = domainOK->next) {
-	if (0 == PORT_Strcmp(hostname, domainOK->name)) {
-	    PORT_Free(hostname);
+	if (0 == PORT_Strcasecmp(hn, domainOK->name)) {
 	    return SECSuccess;
     	}
     }
 
+    /* Per RFC 2818, if the SubjectAltName extension is present, it must
+    ** be used as the cert's identity.
+    */
+    rv = cert_VerifySubjectAltName(cert, hn);
+    if (rv == SECSuccess || PORT_GetError() != SEC_ERROR_EXTENSION_NOT_FOUND)
+    	return rv;
+
     /* try the cert extension first, then the common name */
     cn = CERT_FindNSStringExtension(cert, SEC_OID_NS_CERT_EXT_SSL_SERVER_NAME);
-    if ( cn == NULL ) {
+    if ( !cn ) {
 	cn = CERT_GetCommonName(&cert->subject);
     }
-    
-    sec_lower_string(cn);
-
-    if ( cn ) {
-	if ( ( hndomain = PORT_Strchr(hostname, '.') ) == NULL ) {
-	    /* No domain in server name */
-	    if ( ( domain = PORT_Strchr(cn, '.') ) != NULL ) {
-		/* there is a domain in the cn string, so chop it off */
-		*domain = '\0';
-	    }
-	}
-
-	regvalid = PORT_RegExpValid(cn);
-	
-	if ( regvalid == NON_SXP ) {
-	    /* compare entire hostname with cert name */
-	    if ( PORT_Strcmp(hostname, cn) == 0 ) {
-		rv = SECSuccess;
-		goto done;
-	    }
-	    
-	    if ( hndomain ) {
-		/* compare just domain name with cert name */
-		if ( PORT_Strcmp(hndomain+1, cn) == 0 ) {
-		    rv = SECSuccess;
-		    goto done;
-		}
-	    }
-
-	    PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
-	    rv = SECFailure;
-	    goto done;
-	    
-	} else {
-	    /* try to match the shexp */
-	    match = PORT_RegExpCaseSearch(hostname, cn);
-
-	    if ( match == 0 ) {
-		rv = SECSuccess;
-	    } else {
-		PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
-		rv = SECFailure;
-	    }
-	    goto done;
-	}
-    }
-
-    PORT_SetError(SEC_ERROR_NO_MEMORY);
-    rv = SECFailure;
-
-done:
-    /* free the common name */
     if ( cn ) {
+	rv = cert_TestHostName(cn, hn);
 	PORT_Free(cn);
-    }
-    
-    if ( hostname ) {
-	PORT_Free(hostname);
-    }
-    
-    return(rv);
+    } else 
+	PORT_SetError(SSL_ERROR_BAD_CERT_DOMAIN);
+    return rv;
 }
 
 PRBool
@@ -1649,20 +1721,25 @@ CERT_IsCACert(CERTCertificate *cert, unsigned int *rettype)
     ret = PR_FALSE;
     type = 0;
     
-    if ( cert->isperm ) {
+    if ( cert->trust && (cert->trust->sslFlags|cert->trust->emailFlags|
+				cert->trust->objectSigningFlags)) {
 	trust = cert->trust;
-	if ( ( trust->sslFlags & CERTDB_VALID_CA ) == CERTDB_VALID_CA ) {
+	if ( ( ( trust->sslFlags & CERTDB_VALID_CA ) == CERTDB_VALID_CA ) ||
+	   ( ( trust->sslFlags & CERTDB_TRUSTED_CA ) == CERTDB_TRUSTED_CA ) ) {
 	    ret = PR_TRUE;
 	    type |= NS_CERT_TYPE_SSL_CA;
 	}
 	
-	if ( ( trust->emailFlags & CERTDB_VALID_CA ) == CERTDB_VALID_CA ) {
+	if ( ( ( trust->emailFlags & CERTDB_VALID_CA ) == CERTDB_VALID_CA ) ||
+	  ( ( trust->emailFlags & CERTDB_TRUSTED_CA ) == CERTDB_TRUSTED_CA ) ) {
 	    ret = PR_TRUE;
 	    type |= NS_CERT_TYPE_EMAIL_CA;
 	}
 	
-	if ( ( trust->objectSigningFlags & CERTDB_VALID_CA ) ==
-	    CERTDB_VALID_CA ) {
+	if ( ( ( trust->objectSigningFlags & CERTDB_VALID_CA ) 
+						== CERTDB_VALID_CA ) ||
+          ( ( trust->objectSigningFlags & CERTDB_TRUSTED_CA ) 
+						== CERTDB_TRUSTED_CA ) ) {
 	    ret = PR_TRUE;
 	    type |= NS_CERT_TYPE_OBJECT_SIGNING_CA;
 	}
@@ -1698,15 +1775,14 @@ CERT_IsCACert(CERTCertificate *cert, unsigned int *rettype)
     return(ret);
 }
 
-
 PRBool
 CERT_IsCADERCert(SECItem *derCert, unsigned int *type) {
     CERTCertificate *cert;
     PRBool isCA;
 
-    cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), derCert, NULL,
-	                                   PR_FALSE, PR_TRUE);
-    if (cert == NULL) return NULL;
+    /* This is okay -- only looks at extensions */
+    cert = CERT_DecodeDERCertificate(derCert, PR_FALSE, NULL);
+    if (cert == NULL) return PR_FALSE;
 
     isCA = CERT_IsCACert(cert,type);
     CERT_DestroyCertificate (cert);
@@ -1935,6 +2011,127 @@ CERT_EncodeTrustString(CERTCertTrust *trust)
     return(retstr);
 }
 
+/* in 3.4, this will only set trust */
+SECStatus
+CERT_SaveImportedCert(CERTCertificate *cert, SECCertUsage usage,
+		      PRBool caOnly, char *nickname)
+{
+    SECStatus rv;
+    PRBool saveit;
+    CERTCertTrust trust;
+    PRBool isCA;
+    unsigned int certtype;
+    
+    isCA = CERT_IsCACert(cert, NULL);
+    if ( caOnly && ( !isCA ) ) {
+	return(SECSuccess);
+    }
+    /* In NSS 3.4, certs are given zero trust upon import.  However, this
+    * function needs to set up default CA trust (CERTDB_VALID_CA), or
+    * PKCS#12 imported certs will not show up correctly.  In the case of a
+    * CA cert with zero trust, continue with this function.  But if the cert
+    * does already have some trust bits, exit and do not change them.
+    */
+    if (isCA && cert->trust && 
+        (cert->trust->sslFlags |
+         cert->trust->emailFlags |
+         cert->trust->objectSigningFlags)) {
+	return(SECSuccess);
+    }
+
+    saveit = PR_TRUE;
+    
+    PORT_Memset((void *)&trust, 0, sizeof(trust));
+
+    certtype = cert->nsCertType;
+
+    /* if no CA bits in cert type, then set all CA bits */
+    if ( isCA && ( ! ( certtype & NS_CERT_TYPE_CA ) ) ) {
+	certtype |= NS_CERT_TYPE_CA;
+    }
+
+    /* if no app bits in cert type, then set all app bits */
+    if ( ( !isCA ) && ( ! ( certtype & NS_CERT_TYPE_APP ) ) ) {
+	certtype |= NS_CERT_TYPE_APP;
+    }
+
+    switch ( usage ) {
+      case certUsageEmailSigner:
+      case certUsageEmailRecipient:
+	if ( isCA ) {
+	    if ( certtype & NS_CERT_TYPE_EMAIL_CA ) {
+		trust.emailFlags = CERTDB_VALID_CA;
+	    }
+	} else {
+	    if ( cert->emailAddr == NULL ) {
+		saveit = PR_FALSE;
+	    }
+	    
+	    if ( certtype & NS_CERT_TYPE_EMAIL ) {
+		trust.emailFlags = CERTDB_VALID_PEER;
+		if ( ! ( cert->rawKeyUsage & KU_KEY_ENCIPHERMENT ) ) {
+		    /* don't save it if KeyEncipherment is not allowed */
+		    saveit = PR_FALSE;
+		}
+	    }
+	}
+	break;
+      case certUsageUserCertImport:
+	if ( isCA ) {
+	    if ( certtype & NS_CERT_TYPE_SSL_CA ) {
+		trust.sslFlags = CERTDB_VALID_CA;
+	    }
+	    
+	    if ( certtype & NS_CERT_TYPE_EMAIL_CA ) {
+		trust.emailFlags = CERTDB_VALID_CA;
+	    }
+	    
+	    if ( certtype & NS_CERT_TYPE_OBJECT_SIGNING_CA ) {
+		trust.objectSigningFlags = CERTDB_VALID_CA;
+	    }
+	    
+	} else {
+	    if ( certtype & NS_CERT_TYPE_SSL_CLIENT ) {
+		trust.sslFlags = CERTDB_VALID_PEER;
+	    }
+	    
+	    if ( certtype & NS_CERT_TYPE_EMAIL ) {
+		trust.emailFlags = CERTDB_VALID_PEER;
+	    }
+	    
+	    if ( certtype & NS_CERT_TYPE_OBJECT_SIGNING ) {
+		trust.objectSigningFlags = CERTDB_VALID_PEER;
+	    }
+	}
+	break;
+      case certUsageAnyCA:
+	trust.sslFlags = CERTDB_VALID_CA;
+	break;
+      case certUsageSSLCA:
+	trust.sslFlags = CERTDB_VALID_CA | 
+			CERTDB_TRUSTED_CA | CERTDB_TRUSTED_CLIENT_CA;
+	break;
+      default:	/* XXX added to quiet warnings; no other cases needed? */
+	break;
+    }
+
+    if ( saveit ) {
+	rv = CERT_ChangeCertTrust(cert->dbhandle, cert, &trust);
+	if ( rv != SECSuccess ) {
+	    goto loser;
+	}
+    }
+
+    rv = SECSuccess;
+    goto done;
+
+loser:
+    rv = SECFailure;
+done:
+
+    return(rv);
+}
+
 SECStatus
 CERT_ImportCerts(CERTCertDBHandle *certdb, SECCertUsage usage,
 		 unsigned int ncerts, SECItem **derCerts,
@@ -1944,7 +2141,7 @@ CERT_ImportCerts(CERTCertDBHandle *certdb, SECCertUsage usage,
     int i;
     CERTCertificate **certs = NULL;
     SECStatus rv;
-    int fcerts;
+    int fcerts = 0;
 
     if ( ncerts ) {
 	certs = (CERTCertificate**)PORT_ZAlloc(sizeof(CERTCertificate *) * ncerts );
@@ -1954,25 +2151,47 @@ CERT_ImportCerts(CERTCertDBHandle *certdb, SECCertUsage usage,
     
 	/* decode all of the certs into the temporary DB */
 	for ( i = 0, fcerts= 0; i < ncerts; i++) {
-	    certs[fcerts] = CERT_NewTempCertificate(certdb, derCerts[i], NULL,
-					       PR_FALSE, PR_TRUE);
+	    certs[fcerts] = CERT_NewTempCertificate(certdb,
+	                                            derCerts[i],
+	                                            NULL,
+	                                            PR_FALSE,
+	                                            PR_TRUE);
 	    if (certs[fcerts]) fcerts++;
 	}
 
 	if ( keepCerts ) {
 	    for ( i = 0; i < fcerts; i++ ) {
+                char* canickname = NULL;
+                PRBool freeNickname = PR_FALSE;
+
 		SECKEY_UpdateCertPQG(certs[i]);
+                
+                if ( CERT_IsCACert(certs[i], NULL) ) {
+                    canickname = CERT_MakeCANickname(certs[i]);
+                    if ( canickname != NULL ) {
+                        freeNickname = PR_TRUE;
+                    }
+                }
+
 		if(CERT_IsCACert(certs[i], NULL) && (fcerts > 1)) {
 		    /* if we are importing only a single cert and specifying
 		     * a nickname, we want to use that nickname if it a CA,
 		     * otherwise if there are more than one cert, we don't
-		     * know which cert it belongs to.
+		     * know which cert it belongs to. But we still may try
+                     * the individual canickname from the cert itself.
 		     */
-		    rv = CERT_SaveImportedCert(certs[i], usage, caOnly, NULL);
+		    rv = CERT_AddTempCertToPerm(certs[i], canickname, NULL);
 		} else {
-		    rv = CERT_SaveImportedCert(certs[i], usage, caOnly, 
-                			       nickname);
+		    rv = CERT_AddTempCertToPerm(certs[i],
+                                                nickname?nickname:canickname, NULL);
 		}
+		if (rv == SECSuccess) {
+		    CERT_SaveImportedCert(certs[i], usage, caOnly, NULL);
+		}
+
+                if (PR_TRUE == freeNickname) {
+                    PORT_Free(canickname);
+                }
 		/* don't care if it fails - keep going */
 	    }
 	}
@@ -1981,7 +2200,9 @@ CERT_ImportCerts(CERTCertDBHandle *certdb, SECCertUsage usage,
     if ( retCerts ) {
 	*retCerts = certs;
     } else {
-	CERT_DestroyCertArray(certs, fcerts);
+	if (certs) {
+	    CERT_DestroyCertArray(certs, fcerts);
+	}
     }
 
     return(SECSuccess);
@@ -2082,7 +2303,7 @@ loser:
 SECStatus
 CERT_AddCertToListTail(CERTCertList *certs, CERTCertificate *cert)
 {
-    CERT_AddCertToListTailWithData(certs, cert, NULL);
+    return CERT_AddCertToListTailWithData(certs, cert, NULL);
 }
 
 SECStatus
@@ -2312,36 +2533,6 @@ loser:
     return(SECFailure);
 }
 
-/*
- * Acquire the global lock on the cert database.
- * This lock is currently used for the following operations:
- *	adding or deleting a cert to either the temp or perm databases
- *	converting a temp to perm or perm to temp
- *	changing(maybe just adding????) the trust of a cert
- *      chaning the DB status checking Configuration
- */
-void
-CERT_LockDB(CERTCertDBHandle *handle)
-{
-    PZ_EnterMonitor(handle->dbMon);
-    return;
-}
-
-/*
- * Free the global cert database lock.
- */
-void
-CERT_UnlockDB(CERTCertDBHandle *handle)
-{
-    PRStatus prstat;
-    
-    prstat = PZ_ExitMonitor(handle->dbMon);
-    
-    PORT_Assert(prstat == PR_SUCCESS);
-    
-    return;
-}
-
 static PZLock *certRefCountLock = NULL;
 
 /*
@@ -2434,6 +2625,5 @@ void
 CERT_SetStatusConfig(CERTCertDBHandle *handle, CERTStatusConfig *statusConfig)
 {
   PORT_Assert(handle->statusConfig == NULL);
-
   handle->statusConfig = statusConfig;
 }
