@@ -33,7 +33,7 @@
 /*
  * certt.h - public data structures for the certificate library
  *
- * $Id: certt.h,v 1.7 2001/05/02 21:34:09 wtc%netscape.com Exp $
+ * $Id: certt.h,v 1.17.6.2 2002/06/27 00:20:56 jpierre%netscape.com Exp $
  */
 #ifndef _CERTT_H_
 #define _CERTT_H_
@@ -49,13 +49,21 @@
 #include "prio.h"
 #include "prmon.h"
 
+/* Stan data types */
+struct NSSCertificateStr;
+struct NSSTrustDomainStr;
+
 /* Non-opaque objects */
 typedef struct CERTAVAStr                        CERTAVA;
 typedef struct CERTAttributeStr                  CERTAttribute;
 typedef struct CERTAuthInfoAccessStr             CERTAuthInfoAccess;
 typedef struct CERTAuthKeyIDStr                  CERTAuthKeyID;
 typedef struct CERTBasicConstraintsStr           CERTBasicConstraints;
+#ifdef NSS_CLASSIC
 typedef struct CERTCertDBHandleStr               CERTCertDBHandle;
+#else
+typedef struct NSSTrustDomainStr                 CERTCertDBHandle;
+#endif
 typedef struct CERTCertExtensionStr              CERTCertExtension;
 typedef struct CERTCertKeyStr                    CERTCertKey;
 typedef struct CERTCertListStr                   CERTCertList;
@@ -160,9 +168,6 @@ struct CERTPublicKeyAndChallengeStr {
     SECItem challenge;
 };
 
-typedef struct _certDBEntryCert certDBEntryCert;
-typedef struct _certDBEntryRevocation certDBEntryRevocation;
-
 struct CERTCertTrustStr {
     unsigned int sslFlags;
     unsigned int emailFlags;
@@ -172,7 +177,7 @@ struct CERTCertTrustStr {
 /*
  * defined the types of trust that exist
  */
-typedef enum {
+typedef enum SECTrustTypeEnum {
     trustSSL = 0,
     trustEmail = 1,
     trustObjectSigning = 2,
@@ -206,7 +211,7 @@ struct CERTSubjectListStr {
     char *emailAddr;
     CERTSubjectNode *head;
     CERTSubjectNode *tail; /* do we need tail? */
-    struct _certDBEntrySubject *entry;
+    void *entry;
 };
 
 /*
@@ -266,7 +271,7 @@ struct CERTCertificateStr {
     PRBool istemp;
     char *nickname;
     char *dbnickname;
-    certDBEntryCert *dbEntry;		/* database entry struct */
+    struct NSSCertificateStr *nssCertificate;	/* This is Stan stuff. */
     CERTCertTrust *trust;
 
     /* the reference count is modified whenever someone looks up, dups
@@ -392,14 +397,17 @@ struct CERTCrlKeyStr {
 struct CERTSignedCrlStr {
     PRArenaPool *arena;
     CERTCrl crl;
-    certDBEntryRevocation *dbEntry;	/* database entry struct */
-    PRBool keep;			/* keep this crl in the cache for the  session*/
+    void *reserved1;
+    PRBool reserved2;
     PRBool isperm;
     PRBool istemp;
     int referenceCount;
     CERTCertDBHandle *dbhandle;
     CERTSignedData signatureWrap;	/* XXX */
     char *url;
+    SECItem *derCrl;
+    PK11SlotInfo *slot;
+    CK_OBJECT_HANDLE pkcs11ID;
 };
 
 
@@ -450,7 +458,7 @@ struct CERTDistNamesStr {
 			 NS_CERT_TYPE_EMAIL_CA | \
 			 NS_CERT_TYPE_OBJECT_SIGNING_CA | \
 			 EXT_KEY_USAGE_STATUS_RESPONDER )
-typedef enum {
+typedef enum SECCertUsageEnum {
     certUsageSSLClient = 0,
     certUsageSSLServer = 1,
     certUsageSSLServerWithStepUp = 2,
@@ -468,7 +476,7 @@ typedef enum {
 /*
  * Does the cert belong to the user, a peer, or a CA.
  */
-typedef enum {
+typedef enum CERTCertOwnerEnum {
     certOwnerUser = 0,
     certOwnerPeer = 1,
     certOwnerCA = 2
@@ -477,7 +485,7 @@ typedef enum {
 /*
  * This enum represents the state of validity times of a certificate
  */
-typedef enum {
+typedef enum SECCertTimeValidityEnum {
     secCertTimeValid = 0,
     secCertTimeExpired = 1,
     secCertTimeNotValidYet = 2
@@ -568,7 +576,7 @@ struct CERTBasicConstraintsStr {
 
 /* If we needed to extract the general name field, use this */
 /* General Name types */
-typedef enum {
+typedef enum CERTGeneralNameTypeEnum {
     certOtherName = 1,
     certRFC822Name = 2,
     certDNSName = 3,
@@ -644,7 +652,7 @@ struct CERTAuthKeyIDStr {
 /*
  * defined the types of CRL Distribution points
  */
-typedef enum {
+typedef enum DistributionPointTypesEnum {
     generalName = 1,			/* only support this for now */
     relativeDistinguishedName = 2
 } DistributionPointTypes;
@@ -726,7 +734,7 @@ typedef char * (*CERTDBNameFunc)(void *arg, int dbVersion);
 /*
  * types of cert packages that we can decode
  */
-typedef enum {
+typedef enum CERTPackageTypeEnum {
     certPackageNone = 0,
     certPackageCert = 1,
     certPackagePKCS7 = 2,
@@ -810,8 +818,11 @@ SEC_ASN1_CHOOSER_DECLARE(CERT_CertificateRequestTemplate)
 SEC_ASN1_CHOOSER_DECLARE(CERT_CertificateTemplate)
 SEC_ASN1_CHOOSER_DECLARE(CERT_CrlTemplate)
 SEC_ASN1_CHOOSER_DECLARE(CERT_IssuerAndSNTemplate)
+SEC_ASN1_CHOOSER_DECLARE(CERT_NameTemplate)
 SEC_ASN1_CHOOSER_DECLARE(CERT_SetOfSignedCrlTemplate)
 SEC_ASN1_CHOOSER_DECLARE(CERT_SignedDataTemplate)
+SEC_ASN1_CHOOSER_DECLARE(CERT_SubjectPublicKeyInfoTemplate)
+SEC_ASN1_CHOOSER_DECLARE(SEC_SignedCertificateTemplate)
 
 SEC_END_PROTOS
 
