@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: find.c,v $ $Revision: 1.1 $ $Date: 2000/03/31 19:43:14 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: find.c,v $ $Revision: 1.3 $ $Date: 2002/02/08 00:10:00 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -134,7 +134,7 @@ nssCKFWFindObjects_Create
   CK_RV *pError
 )
 {
-  NSSCKFWFindObjects *fwFindObjects;
+  NSSCKFWFindObjects *fwFindObjects = NULL;
   NSSArena *arena;
   NSSCKMDSession *mdSession;
   NSSCKMDToken *mdToken;
@@ -144,12 +144,14 @@ nssCKFWFindObjects_Create
   mdToken = nssCKFWToken_GetMDToken(fwToken);
   mdInstance = nssCKFWInstance_GetMDInstance(fwInstance);
 
+#ifdef notdef
   arena = nssCKFWSession_GetArena(fwSession, pError);
   if( (NSSArena *)NULL == arena ) {
     goto loser;
   }
+#endif
 
-  fwFindObjects = nss_ZNEW(arena, NSSCKFWFindObjects);
+  fwFindObjects = nss_ZNEW(NULL, NSSCKFWFindObjects);
   if( (NSSCKFWFindObjects *)NULL == fwFindObjects ) {
     *pError = CKR_HOST_MEMORY;
     goto loser;
@@ -164,7 +166,7 @@ nssCKFWFindObjects_Create
   fwFindObjects->fwInstance = fwInstance;
   fwFindObjects->mdInstance = mdInstance;
 
-  fwFindObjects->mutex = nssCKFWInstance_CreateMutex(fwInstance, arena, pError);
+  fwFindObjects->mutex = nssCKFWInstance_CreateMutex(fwInstance, NULL, pError);
   if( (NSSCKFWMutex *)NULL == fwFindObjects->mutex ) {
     goto loser;
   }
