@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: object.c,v $ $Revision: 1.5 $ $Date: 2001/08/23 00:00:05 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: object.c,v $ $Revision: 1.9 $ $Date: 2002/03/29 18:53:15 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -420,10 +420,6 @@ nssCKFWObject_IsTokenObject
   }
 #endif /* NSSDEBUG */
 
-  if( CKR_OK != nssCKFWMutex_Lock(fwObject->mutex) ) {
-    return CK_FALSE;
-  }
-
   if( (void *)NULL == (void *)fwObject->mdObject->IsTokenObject ) {
     NSSItem item;
     NSSItem *pItem;
@@ -448,7 +444,6 @@ nssCKFWObject_IsTokenObject
     fwObject->fwToken, fwObject->mdInstance, fwObject->fwInstance);
 
  done:
-  (void)nssCKFWMutex_Unlock(fwObject->mutex);
   return b;
 }
 
@@ -603,7 +598,7 @@ nssCKFWObject_GetAttribute
 )
 {
   NSSItem *rv = (NSSItem *)NULL;
-  const NSSItem *mdItem;
+  NSSCKFWItem mdItem;
 
 #ifdef NSSDEBUG
   if( (CK_RV *)NULL == pError ) {
@@ -616,7 +611,7 @@ nssCKFWObject_GetAttribute
   }
 #endif /* NSSDEBUG */
 
-  if( (void *)NULL == (void *)fwObject->mdObject->GetAttributeSize ) {
+  if( (void *)NULL == (void *)fwObject->mdObject->GetAttribute ) {
     *pError = CKR_GENERAL_ERROR;
     return (NSSItem *)NULL;
   }
@@ -631,7 +626,7 @@ nssCKFWObject_GetAttribute
     fwObject->fwToken, fwObject->mdInstance, fwObject->fwInstance,
     attribute, pError);
 
-  if( (NSSItem *)NULL == mdItem ) {
+  if( (NSSItem *)NULL == mdItem.item ) {
     if( CKR_OK == *pError ) {
       *pError = CKR_GENERAL_ERROR;
     }
@@ -650,7 +645,7 @@ nssCKFWObject_GetAttribute
   }
 
   if( (void *)NULL == rv->data ) {
-    rv->size = mdItem->size;
+    rv->size = mdItem.item->size;
     rv->data = nss_ZAlloc(arenaOpt, rv->size);
     if( (void *)NULL == rv->data ) {
       *pError = CKR_HOST_MEMORY;
@@ -661,8 +656,8 @@ nssCKFWObject_GetAttribute
       goto done;
     }
   } else {
-    if( rv->size >= mdItem->size ) {
-      rv->size = mdItem->size;
+    if( rv->size >= mdItem.item->size ) {
+      rv->size = mdItem.item->size;
     } else {
       *pError = CKR_BUFFER_TOO_SMALL;
       /* Should we set rv->size to mdItem->size? */
@@ -672,7 +667,14 @@ nssCKFWObject_GetAttribute
     }
   }
 
-  (void)nsslibc_memcpy(rv->data, mdItem->data, rv->size);
+  (void)nsslibc_memcpy(rv->data, mdItem.item->data, rv->size);
+
+  if (PR_TRUE == mdItem.needsFreeing) {
+    PR_ASSERT(fwObject->mdObject->FreeAttribute);
+    if (fwObject->mdObject->FreeAttribute) {
+      *pError = fwObject->mdObject->FreeAttribute(&mdItem);
+    }
+  }
 
  done:
   (void)nssCKFWMutex_Unlock(fwObject->mutex);
@@ -948,9 +950,9 @@ NSSCKFWObject_GetAttributeTypes
   CK_ULONG ulCount
 )
 {
+#ifdef DEBUG
   CK_RV error = CKR_OK;
 
-#ifdef DEBUG
   error = nssCKFWObject_verifyPointer(fwObject);
   if( CKR_OK != error ) {
     return error;
