@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: sessobj.c,v $ $Revision: 1.5 $ $Date: 2000/09/06 22:23:56 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char CVS_ID[] = "@(#) $RCSfile: sessobj.c,v $ $Revision: 1.10 $ $Date: 2002/04/05 19:38:03 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -103,6 +103,7 @@ nss_ckmdSessionObject_remove_pointer
   return CKR_OK;
 }
 
+#ifdef NSS_DEBUG
 static CK_RV
 nss_ckmdSessionObject_verifyPointer
 (
@@ -111,6 +112,7 @@ nss_ckmdSessionObject_verifyPointer
 {
   return CKR_OK;
 }
+#endif
 
 #endif /* DEBUG */
 
@@ -200,7 +202,7 @@ nss_ckmdSessionObject_GetAttributeSize
   CK_RV *pError
 );
 
-static NSSItem *
+static NSSCKFWItem
 nss_ckmdSessionObject_GetAttribute
 (
   NSSCKMDObject *mdObject,
@@ -580,7 +582,7 @@ nss_ckmdSessionObject_GetAttributeSize
  * nss_ckmdSessionObject_GetAttribute
  *
  */
-static NSSItem *
+static NSSCKFWItem
 nss_ckmdSessionObject_GetAttribute
 (
   NSSCKMDObject *mdObject,
@@ -595,17 +597,20 @@ nss_ckmdSessionObject_GetAttribute
   CK_RV *pError
 )
 {
+  NSSCKFWItem item;
   nssCKMDSessionObject *obj;
   CK_ULONG i;
 
+  item.needsFreeing = PR_FALSE;
+  item.item = NULL;
 #ifdef NSSDEBUG
   if( (CK_RV *)NULL == pError ) {
-    return 0;
+    return item;
   }
 
   *pError = nss_ckmdSessionObject_verifyPointer(mdObject);
   if( CKR_OK != *pError ) {
-    return 0;
+    return item;
   }
 
   /* We could even check all the other arguments, for sanity. */
@@ -615,12 +620,13 @@ nss_ckmdSessionObject_GetAttribute
 
   for( i = 0; i < obj->n; i++ ) {
     if( attribute == obj->types[i] ) {
-      return &obj->attributes[i];
+      item.item = &obj->attributes[i];
+      return item;
     }
   }
 
   *pError = CKR_ATTRIBUTE_TYPE_INVALID;
-  return 0;
+  return item;
 }
 
 /*
@@ -819,6 +825,7 @@ nss_ckmdFindSessionObjects_remove_pointer
   return CKR_OK;
 }
 
+#ifdef NSS_DEBUG
 static CK_RV
 nss_ckmdFindSessionObjects_verifyPointer
 (
@@ -827,6 +834,7 @@ nss_ckmdFindSessionObjects_verifyPointer
 {
   return CKR_OK;
 }
+#endif
 
 #endif /* DEBUG */
 
@@ -1007,7 +1015,7 @@ nssCKMDFindSessionObjects_Create
   rv->Next = nss_ckmdFindSessionObjects_Next;
 
 #ifdef DEBUG
-  if( *pError != nss_ckmdFindSessionObjects_add_pointer(rv) ) {
+  if( (*pError = nss_ckmdFindSessionObjects_add_pointer(rv)) != CKR_OK ) {
     NSSArena_Destroy(arena);
     return (NSSCKMDFindObjects *)NULL;
   }
