@@ -135,6 +135,12 @@ struct SEC_PKCS12DecoderContextStr {
 
     /* import information */
     PRBool bagsVerified;
+
+    /* buffer management for the default callbacks implementation */
+    void        *buffer;      /* storage area */
+    PRInt32     filesize;     /* actual data size */
+    PRInt32     allocated;    /* total buffer size allocated */
+    PRInt32     currentpos;   /* position counter */
 };
 
 
@@ -1019,6 +1025,114 @@ loser:
     p12dcx->error = PR_TRUE;
 }
 
+/*  default implementations of the open/close/read/write functions for
+    SEC_PKCS12DecoderStart 
+*/
+
+#define DEFAULT_TEMP_SIZE 4096
+
+static SECStatus
+p12u_DigestOpen(void *arg, PRBool readData)
+{
+    SEC_PKCS12DecoderContext* p12cxt = arg;
+
+    p12cxt->currentpos = 0;
+
+    if (PR_FALSE == readData) {
+        /* allocate an initial buffer */
+        p12cxt->filesize = 0;
+        p12cxt->allocated = DEFAULT_TEMP_SIZE;
+        p12cxt->buffer = PORT_Alloc(DEFAULT_TEMP_SIZE);
+        PR_ASSERT(p12cxt->buffer);
+    }
+    else
+    {
+        PR_ASSERT(p12cxt->buffer);
+        if (!p12cxt->buffer) {
+            return SECFailure; /* no data to read */
+        }
+    }
+
+    return SECSuccess;
+}
+
+static SECStatus
+p12u_DigestClose(void *arg, PRBool removeFile)
+{
+    SEC_PKCS12DecoderContext* p12cxt = arg;
+
+    PR_ASSERT(p12cxt);
+    if (!p12cxt) {
+        return SECFailure;
+    }
+    p12cxt->currentpos = 0;
+
+    if (PR_TRUE == removeFile) {
+        PR_ASSERT(p12cxt->buffer);
+        if (!p12cxt->buffer) {
+            return SECFailure;
+        }
+        if (p12cxt->buffer) {
+            PORT_Free(p12cxt->buffer);
+            p12cxt->buffer = NULL;
+            p12cxt->allocated = 0;
+            p12cxt->filesize = 0;
+        }
+    }
+
+    return SECSuccess;
+}
+
+static int
+p12u_DigestRead(void *arg, unsigned char *buf, unsigned long len)
+{
+    int toread = len;
+    SEC_PKCS12DecoderContext* p12cxt = arg;
+
+    if(!buf || len == 0) {
+	return -1;
+    }
+
+    if (!p12cxt->buffer || ((p12cxt->filesize-p12cxt->currentpos)<(long)len) ) {
+        /* trying to read past the end of the buffer */
+        toread = p12cxt->filesize-p12cxt->currentpos;
+    }
+    memcpy(buf, (void*)((char*)p12cxt->buffer+p12cxt->currentpos), toread);
+    p12cxt->currentpos += toread;
+    return toread;
+}
+
+static int
+p12u_DigestWrite(void *arg, unsigned char *buf, unsigned long len)
+{
+    SEC_PKCS12DecoderContext* p12cxt = arg;
+
+    if(!buf || len == 0) {
+        return -1;
+    }
+
+    if (p12cxt->currentpos+(long)len > p12cxt->filesize) {
+        p12cxt->filesize = p12cxt->currentpos + len;
+    }
+    else {
+        p12cxt->filesize += len;
+    }
+    if (p12cxt->filesize > p12cxt->allocated) {
+        void* newbuffer;
+        size_t newsize = p12cxt->filesize + DEFAULT_TEMP_SIZE;
+        newbuffer  = PORT_Realloc(p12cxt->buffer, newsize);
+        if (NULL == newbuffer) {
+            return -1; /* can't extend the buffer */
+        }
+        p12cxt->buffer = newbuffer;
+        p12cxt->allocated = newsize;
+    }
+    PR_ASSERT(p12cxt->buffer);
+    memcpy((void*)((char*)p12cxt->buffer+p12cxt->currentpos), buf, len);
+    p12cxt->currentpos += len;
+    return len;
+}
+
 /* SEC_PKCS12DecoderStart
  *	Creates a decoder context for decoding a PKCS 12 PDU objct.
  *	This function sets up the initial decoding context for the
@@ -1039,6 +1153,9 @@ loser:
  *		 routines.
  *	dArg - the argument for dOpen, etc.
  *
+ *      if NULL == dOpen == dClose == dRead == dWrite == dArg, then default
+ *      implementations using a memory buffer are used
+ *
  *	This function returns the decoder context, if it was successful.
  *	Otherwise, null is returned.
  */
@@ -1062,9 +1179,20 @@ SEC_PKCS12DecoderStart(SECItem *pwitem, PK11SlotInfo *slot, void *wincx,
 	PORT_SetError(SEC_ERROR_NO_MEMORY);
 	goto loser;
     }
+
+    if (!dOpen && !dClose && !dRead && !dWrite && !dArg) {
+        /* use default implementations */
+        dOpen = p12u_DigestOpen;
+        dClose = p12u_DigestClose;
+        dRead = p12u_DigestRead;
+        dWrite = p12u_DigestWrite;
+        dArg = (void*)p12dcx;
+    }
+
     p12dcx->arena = arena;
     p12dcx->pwitem = pwitem;
-    p12dcx->slot = (slot ? slot : PK11_GetInternalKeySlot());
+    p12dcx->slot = (slot ? PK11_ReferenceSlot(slot) 
+						: PK11_GetInternalKeySlot());
     p12dcx->wincx = wincx;
 #ifdef IS_LITTLE_ENDIAN
     p12dcx->swapUnicodeBytes = PR_TRUE;
@@ -1074,12 +1202,6 @@ SEC_PKCS12DecoderStart(SECItem *pwitem, PK11SlotInfo *slot, void *wincx,
     p12dcx->errorValue = 0;
     p12dcx->error = PR_FALSE;
 
-    /* a slot is *required */
-    if(!slot) {
-	PORT_SetError(SEC_ERROR_NO_MEMORY);
-	goto loser;
-    }
-
     /* start the decoding of the PFX and set the notify proc
      * for the PFX item.
      */
@@ -1087,6 +1209,7 @@ SEC_PKCS12DecoderStart(SECItem *pwitem, PK11SlotInfo *slot, void *wincx,
     					  sec_PKCS12PFXItemTemplate);
     if(!p12dcx->pfxDcx) {
 	PORT_SetError(SEC_ERROR_NO_MEMORY); 
+	PK11_FreeSlot(p12dcx->slot);
 	goto loser;
     }
 
@@ -1152,14 +1275,15 @@ static SECStatus
 sec_pkcs12_decoder_verify_mac(SEC_PKCS12DecoderContext *p12dcx)
 {
     SECStatus rv = SECFailure;
+    SECStatus lrv;
     SECItem hmacRes;
     unsigned char buf[IN_BUF_LEN];
     unsigned int bufLen;
     int iteration;
-    PK11Context *pk11cx;
+    PK11Context *pk11cx = NULL;
+    PK11SymKey *symKey = NULL;
+    SECItem *params = NULL;
     SECItem ignore = {0};
-    PK11SymKey *symKey;
-    SECItem *params;
     SECOidTag algtag;
     CK_MECHANISM_TYPE integrityMech;
     
@@ -1191,15 +1315,18 @@ sec_pkcs12_decoder_verify_mac(SEC_PKCS12DecoderContext *p12dcx)
 
     symKey = PK11_KeyGen(NULL, integrityMech, params, 20, NULL);
     PK11_DestroyPBEParams(params);
+    params = NULL;
     if (!symKey) goto loser;
     /* init hmac */
     pk11cx = PK11_CreateContextBySymKey(sec_pkcs12_algtag_to_mech(algtag),
                                         CKA_SIGN, symKey, &ignore);
     if(!pk11cx) {
-	PORT_SetError(SEC_ERROR_NO_MEMORY);
-	return SECFailure;
+	goto loser;
+    }
+    lrv = PK11_DigestBegin(pk11cx);
+    if (lrv == SECFailure ) {
+	goto loser;
     }
-    rv = PK11_DigestBegin(pk11cx);
 
     /* try to open the data for readback */
     if(p12dcx->dOpen && ((*p12dcx->dOpen)(p12dcx->dArg, PR_TRUE) 
@@ -1219,14 +1346,20 @@ sec_pkcs12_decoder_verify_mac(SEC_PKCS12DecoderContext *p12dcx)
 	    goto loser;
 	}
 
-	rv = PK11_DigestOp(pk11cx, buf, bytesRead);
+	lrv = PK11_DigestOp(pk11cx, buf, bytesRead);
+	if (lrv == SECFailure) {
+	    goto loser;
+	}
 	if(bytesRead < IN_BUF_LEN) {
 	    break;
 	}
     }
 
     /* finish the hmac context */
-    rv = PK11_DigestFinal(pk11cx, buf, &bufLen, IN_BUF_LEN);
+    lrv = PK11_DigestFinal(pk11cx, buf, &bufLen, IN_BUF_LEN);
+    if (lrv == SECFailure ) {
+	goto loser;
+    }
 
     hmacRes.data = buf;
     hmacRes.len = bufLen;
@@ -1248,6 +1381,12 @@ loser:
     if(pk11cx) {
 	PK11_DestroyContext(pk11cx, PR_TRUE);
     }
+    if (params) {
+	PK11_DestroyPBEParams(params);
+    }
+    if (symKey) {
+	PK11_FreeSymKey(symKey);
+    }
 
     return rv;
 }
@@ -1269,6 +1408,12 @@ SEC_PKCS12DecoderVerify(SEC_PKCS12DecoderContext *p12dcx)
 	return SECFailure;
     }
 
+    rv = SEC_ASN1DecoderFinish(p12dcx->pfxDcx);
+    p12dcx->pfxDcx = NULL;
+    if(rv != SECSuccess) {
+	return rv;
+    }
+
     /* check the signature or the mac depending on the type of
      * integrity used.
      */
@@ -1305,8 +1450,6 @@ SEC_PKCS12DecoderVerify(SEC_PKCS12DecoderContext *p12dcx)
 void
 SEC_PKCS12DecoderFinish(SEC_PKCS12DecoderContext *p12dcx)
 {
-    void *freedCtxt = NULL;
-
     if(!p12dcx) {
 	return;
     }
@@ -1335,6 +1478,11 @@ SEC_PKCS12DecoderFinish(SEC_PKCS12DecoderContext *p12dcx)
 	p12dcx->hmacDcx = NULL;
     }
 
+    if(p12dcx->slot) {
+	PK11_FreeSlot(p12dcx->slot);
+	p12dcx->slot = NULL;
+    }
+
     if(p12dcx->arena) {
 	PORT_FreeArena(p12dcx->arena, PR_TRUE);
     }
@@ -1851,8 +1999,7 @@ sec_pkcs12_get_existing_nick_for_dn(sec_PKCS12SafeBag *cert, void *wincx)
 	return NULL;
     }
 
-    tempCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), derCert, NULL,
-				   PR_FALSE, PR_TRUE);
+    tempCert = CERT_DecodeDERCertificate(derCert, PR_FALSE, NULL);
     if(!tempCert) {
 	returnDn = NULL;
 	goto loser;
@@ -1877,15 +2024,6 @@ sec_pkcs12_get_existing_nick_for_dn(sec_PKCS12SafeBag *cert, void *wincx)
     /* if the token is local, first traverse the cert database 
      * then traverse the token.
      */
-    if(PK11_IsInternal(cert->slot)) {
-	if(CERT_TraversePermCertsForSubject(CERT_GetDefaultCertDB(), 
-					&tempCert->derSubject, gatherNicknames,
-					nickArg) != SECSuccess) {
-	    returnDn = NULL;
-	    goto loser;
-	}
-    }
-
     if(PK11_TraverseCertsForSubjectInSlot(tempCert, cert->slot, gatherNicknames,
 			(void *)nickArg) != SECSuccess) {
 	returnDn = NULL;
@@ -1946,12 +2084,6 @@ sec_pkcs12_certs_for_nickname_exist(SECItem *nickname, PK11SlotInfo *slot)
     }
 
     /* we want to check the local database first if we are importing to it */
-    if(PK11_IsInternal(slot)) {
-	CERT_TraversePermCertsForNickname(CERT_GetDefaultCertDB(), 
-				      (char *)nickname->data,
-				      countCertificate, (void *)&nCerts);
-    }
-
     PK11_TraverseCertsForNicknameInSlot(nickname, slot, countCertificate, 
 					(void *)&nCerts);
     if(nCerts) return PR_TRUE;
@@ -2114,13 +2246,12 @@ sec_pkcs12_validate_cert(sec_PKCS12SafeBag *cert,
     }
 
     cert->noInstall = PR_FALSE;
-    cert->removeExisting = PR_FALSE;
+    cert->unused = PR_FALSE;
     cert->problem = PR_FALSE;
     cert->error = 0;
 
-    leafCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-				&cert->safeBagContent.certBag->value.x509Cert,
-				NULL, PR_FALSE, PR_TRUE);
+    leafCert = CERT_DecodeDERCertificate(
+	 &cert->safeBagContent.certBag->value.x509Cert, PR_FALSE, NULL);
     if(!leafCert) {
 	cert->noInstall = PR_TRUE;
 	cert->problem = PR_TRUE;
@@ -2128,26 +2259,7 @@ sec_pkcs12_validate_cert(sec_PKCS12SafeBag *cert,
 	return;
     }
 
-    testCert = PK11_FindCertFromDERCert(cert->slot, leafCert, wincx);
     CERT_DestroyCertificate(leafCert);
-    /* if we can't find the certificate through the PKCS11 interface,
-     * we should check the cert database directly, if we are
-     * importing to an internal slot.
-     */
-    if(!testCert && PK11_IsInternal(cert->slot)) {
-	testCert = CERT_FindCertByDERCert(CERT_GetDefaultCertDB(),
-				 &cert->safeBagContent.certBag->value.x509Cert);
-    }
-
-    if(testCert) {
-	if(!testCert->nickname) {
-	    cert->removeExisting = PR_TRUE;
-	}
-	CERT_DestroyCertificate(testCert);
-	if(cert->noInstall && !cert->removeExisting) {
-	    return;
-	}
-    }
 
     sec_pkcs12_validate_cert_nickname(cert, key, nicknameCb, wincx);
 }
@@ -2172,9 +2284,8 @@ sec_pkcs12_validate_key_by_cert(sec_PKCS12SafeBag *cert, sec_PKCS12SafeBag *key,
 	return;
     }
 
-    leafCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-				&(cert->safeBagContent.certBag->value.x509Cert),
-				NULL, PR_FALSE, PR_TRUE);
+    leafCert = CERT_DecodeDERCertificate(
+	&(cert->safeBagContent.certBag->value.x509Cert), PR_FALSE, NULL);
     if(!leafCert) {
 	key->problem = PR_TRUE;
 	key->noInstall = PR_TRUE;
@@ -2195,61 +2306,6 @@ sec_pkcs12_validate_key_by_cert(sec_PKCS12SafeBag *cert, sec_PKCS12SafeBag *key,
     CERT_DestroyCertificate(leafCert);
 }
 
-static SECStatus
-sec_pkcs12_remove_existing_cert(sec_PKCS12SafeBag *cert, 
-				void *wincx)
-{
-    SECItem *derCert = NULL;
-    CERTCertificate *tempCert = NULL;
-    CK_OBJECT_HANDLE certObj;
-    PK11SlotInfo *slot = NULL;
-    PRBool removed = PR_FALSE;
-
-    if(!cert) {
-	return SECFailure;
-    }
-
-    PORT_Assert(cert->removeExisting);
-
-    cert->removeExisting = PR_FALSE;
-    derCert = &cert->safeBagContent.certBag->value.x509Cert;
-    tempCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), derCert,
-				       NULL, PR_FALSE, PR_TRUE);
-    if(!tempCert) {
-	return SECFailure;
-    }
-
-    certObj = PK11_FindCertInSlot(cert->slot, tempCert, wincx);
-    CERT_DestroyCertificate(tempCert);
-    tempCert = NULL;
-
-    if(certObj != CK_INVALID_KEY) {
-	PK11_DestroyObject(cert->slot, certObj);
-	removed = PR_TRUE;
-    } else if(PK11_IsInternal(cert->slot)) {
-	tempCert = CERT_FindCertByDERCert(CERT_GetDefaultCertDB(), derCert);
-	if(tempCert) {
-	    if(SEC_DeletePermCertificate(tempCert) == SECSuccess) {
-		removed = PR_TRUE;
-	    } 
-	    CERT_DestroyCertificate(tempCert);
-	    tempCert = NULL;
-	}
-    }
-
-    if(!removed) {
-	cert->problem = PR_TRUE;
-	cert->error = SEC_ERROR_NO_MEMORY;
-	cert->noInstall = PR_TRUE;
-    }
-	
-    if(tempCert) {
-	CERT_DestroyCertificate(tempCert);
-    }
-
-    return ((removed) ? SECSuccess : SECFailure);
-}
-
 static SECStatus
 sec_pkcs12_add_cert(sec_PKCS12SafeBag *cert, PRBool keyExists, void *wincx)
 {
@@ -2266,15 +2322,8 @@ sec_pkcs12_add_cert(sec_PKCS12SafeBag *cert, PRBool keyExists, void *wincx)
     }
 
     derCert = &cert->safeBagContent.certBag->value.x509Cert;
-    if(cert->removeExisting) {
-	if(sec_pkcs12_remove_existing_cert(cert, wincx) 
-			!= SECSuccess) {
-	    return SECFailure;
-	}
-	cert->removeExisting = PR_FALSE;
-    }
 
-    PORT_Assert(!cert->problem && !cert->removeExisting && !cert->noInstall);
+    PORT_Assert(!cert->problem && !cert->noInstall);
 
     nickName = sec_pkcs12_get_nickname(cert);
     if(nickName) {
@@ -2285,7 +2334,7 @@ sec_pkcs12_add_cert(sec_PKCS12SafeBag *cert, PRBool keyExists, void *wincx)
 	CERTCertificate *newCert;
 
 	newCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-					  derCert, NULL, PR_FALSE, PR_TRUE);
+	                                  derCert, NULL, PR_FALSE, PR_FALSE);
 	if(!newCert) {
 	     if(nickName) SECITEM_ZfreeItem(nickName, PR_TRUE);
 	     cert->error = SEC_ERROR_NO_MEMORY;
@@ -2320,12 +2369,6 @@ sec_pkcs12_add_key(sec_PKCS12SafeBag *key, SECItem *publicValue,
 	return SECFailure;
     }
 
-    if(key->removeExisting) {
-	key->problem = PR_TRUE;
-	key->error = SEC_ERROR_PKCS12_UNABLE_TO_IMPORT_KEY;
-	return SECFailure;
-    }
-
     if(key->problem || key->noInstall) {
 	return SECSuccess;
     }
@@ -2467,8 +2510,9 @@ SEC_PKCS12DecoderGetCerts(SEC_PKCS12DecoderContext *p12dcx)
 		CERTCertificate *tempCert = NULL;
 
 		if (derCert == NULL) continue;
-    		tempCert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), 
-			derCert, NULL, PR_FALSE, PR_TRUE);
+    		tempCert=CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
+		                                 derCert, NULL, 
+		                                 PR_FALSE, PR_TRUE);
 
 		if (tempCert) {
 		    CERT_AddCertToListTail(certList,tempCert);
@@ -2653,9 +2697,8 @@ sec_pkcs12_get_public_value_and_type(sec_PKCS12SafeBag *certBag,
 	return NULL;
     }
 
-    cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), 
-			&certBag->safeBagContent.certBag->value.x509Cert,
-			NULL, PR_FALSE, PR_FALSE);
+    cert = CERT_DecodeDERCertificate(
+	 &certBag->safeBagContent.certBag->value.x509Cert, PR_FALSE, NULL);
     if(!cert) {
 	return NULL;
     }
@@ -3210,7 +3253,7 @@ sec_PKCS12ConvertOldSafeToNew(PRArenaPool *arena, PK11SlotInfo *slot,
     }
 
     p12dcx->arena = arena;
-    p12dcx->slot = slot;
+    p12dcx->slot = PK11_ReferenceSlot(slot);
     p12dcx->wincx = wincx;
     p12dcx->error = PR_FALSE;
     p12dcx->swapUnicodeBytes = swapUnicode; 
