@@ -35,7 +35,7 @@
 #define NSSPKI_H
 
 #ifdef DEBUG
-static const char NSSPKI_CVS_ID[] = "@(#) $RCSfile: nsspki.h,v $ $Revision: 1.1 $ $Date: 2000/03/31 19:16:12 $ $Name: MOZILLA_0_9_8_RELEASE $";
+static const char NSSPKI_CVS_ID[] = "@(#) $RCSfile: nsspki.h,v $ $Revision: 1.7 $ $Date: 2001/12/14 17:32:19 $ $Name: MOZILLA_1_1_RELEASE $";
 #endif /* DEBUG */
 
 /*
@@ -44,10 +44,22 @@ static const char NSSPKI_CVS_ID[] = "@(#) $RCSfile: nsspki.h,v $ $Revision: 1.1
  * This file prototypes the methods of the top-level PKI objects.
  */
 
+#ifndef NSSDEVT_H
+#include "nssdevt.h"
+#endif /* NSSDEVT_H */
+
 #ifndef NSSPKIT_H
 #include "nsspkit.h"
 #endif /* NSSPKIT_H */
 
+#ifndef NSSPKI1_H
+#include "nsspki1.h"
+#endif /* NSSPKI1_H */
+
+#ifndef BASE_H
+#include "base.h"
+#endif /* BASE_H */
+
 PR_BEGIN_EXTERN_C
 
 /*
@@ -215,8 +227,16 @@ NSSCertificate_Encode
  * in the "chain" starting from the specified one up to and
  * including the root.  The zeroth element in the array is the
  * specified ("leaf") certificate.
+ *
+ * If statusOpt is supplied, and is returned as PR_FAILURE, possible
+ * error values are:
+ *
+ * NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND - the chain is incomplete
+ *
  */
 
+extern const NSSError NSS_ERROR_CERTIFICATE_ISSUER_NOT_FOUND;
+
 NSS_EXTERN NSSCertificate **
 NSSCertificate_BuildChain
 (
@@ -226,7 +246,8 @@ NSSCertificate_BuildChain
   NSSPolicies *policiesOpt,
   NSSCertificate **rvOpt,
   PRUint32 rvLimit, /* zero for no limit */
-  NSSArena *arenaOpt
+  NSSArena *arenaOpt,
+  PRStatus *statusOpt
 );
 
 /*
@@ -787,7 +808,7 @@ NSSPrivateKey_FindPublicKey
 NSS_EXTERN NSSCryptoContext *
 NSSPrivateKey_CreateCryptoContext
 (
-  NSSPrivateKey *vk
+  NSSPrivateKey *vk,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhh
 );
@@ -1002,7 +1023,7 @@ NSSPublicKey_WrapSymmetricKey
 NSS_EXTERN NSSCryptoContext *
 NSSPublicKey_CreateCryptoContext
 (
-  NSSPublicKey *bk
+  NSSPublicKey *bk,
   NSSAlgorithmAndParameters *apOpt,
   NSSCallback *uhh
 );
@@ -1447,6 +1468,7 @@ NSSTrustDomain_GetDefaultCallback
 NSS_EXTERN PRStatus
 NSSTrustDomain_LoadModule
 (
+  NSSTrustDomain *td,
   NSSUTF8 *moduleOpt,
   NSSUTF8 *uriOpt,
   NSSUTF8 *opaqueOpt,
@@ -1734,7 +1756,7 @@ NSS_EXTERN NSSCertificate *
 NSSTrustDomain_FindBestCertificateBySubject
 (
   NSSTrustDomain *td,
-  NSSUTF8 *subject,
+  NSSDER /*NSSUTF8*/ *subject,
   NSSTime *timeOpt,
   NSSUsage *usage,
   NSSPolicies *policiesOpt
@@ -1750,7 +1772,7 @@ NSS_EXTERN NSSCertificate **
 NSSTrustDomain_FindCertificatesBySubject
 (
   NSSTrustDomain *td,
-  NSSUTF8 *subject,
+  NSSDER /*NSSUTF8*/ *subject,
   NSSCertificate *rvOpt[],
   PRUint32 maximumOpt, /* 0 for no max */
   NSSArena *arenaOpt
@@ -1826,7 +1848,7 @@ NSSTrustDomain_FindCertificateByEmail
  */
 
 NSS_EXTERN NSSCertificate **
-NSSTrustDomain_FindCertificateByEmail
+NSSTrustDomain_FindCertificatesByEmail
 (
   NSSTrustDomain *td,
   NSSASCII7 *email,
@@ -1859,16 +1881,20 @@ NSSTrustDomain_FindCertificateByOCSPHash
  * discourage traversal.  Thus for now, this is commented out.
  * If it's needed, let's look at the situation more closely to
  * find out what the actual requirements are.
- *
- * 
- * NSS_EXTERN PRStatus *
- * NSSTrustDomain_TraverseCertificates
- * (
- *   NSSTrustDomain *td,
- *   PRStatus (*callback)(NSSCertificate *c, void *arg),
- *   void *arg
- * );
  */
+ 
+/* For now, adding this function.  This may only be for debugging
+ * purposes.
+ * Perhaps some equivalent function, on a specified token, will be
+ * needed in a "friend" header file?
+ */
+NSS_EXTERN PRStatus *
+NSSTrustDomain_TraverseCertificates
+(
+  NSSTrustDomain *td,
+  PRStatus (*callback)(NSSCertificate *c, void *arg),
+  void *arg
+);
 
 /*
  * NSSTrustDomain_FindBestUserCertificate
@@ -2154,7 +2180,7 @@ NSSTrustDomain_CreateCryptoContextForAlgorithmAndParameters
 NSS_EXTERN PRStatus
 NSSCryptoContext_Destroy
 (
-  NSSCryptoContext *td
+  NSSCryptoContext *cc
 );
 
 /* establishing a default callback */
@@ -2167,7 +2193,7 @@ NSSCryptoContext_Destroy
 NSS_EXTERN PRStatus
 NSSCryptoContext_SetDefaultCallback
 (
-  NSSCryptoContext *td,
+  NSSCryptoContext *cc,
   NSSCallback *newCallback,
   NSSCallback **oldCallbackOpt
 );
@@ -2180,7 +2206,7 @@ NSSCryptoContext_SetDefaultCallback
 NSS_EXTERN NSSCallback *
 NSSCryptoContext_GetDefaultCallback
 (
-  NSSCryptoContext *td,
+  NSSCryptoContext *cc,
   PRStatus *statusOpt
 );
 
@@ -2192,7 +2218,7 @@ NSSCryptoContext_GetDefaultCallback
 NSS_EXTERN NSSTrustDomain *
 NSSCryptoContext_GetTrustDomain
 (
-  NSSCryptoContext *td
+  NSSCryptoContext *cc
 );
 
 /* AddModule, etc: should we allow "temporary" changes here? */
@@ -2309,7 +2335,7 @@ NSS_EXTERN NSSCertificate *
 NSSCryptoContext_FindBestCertificateBySubject
 (
   NSSCryptoContext *cc,
-  NSSUTF8 *subject,
+  NSSDER /*NSSUTF8*/ *subject,
   NSSTime *timeOpt,
   NSSUsage *usage,
   NSSPolicies *policiesOpt
@@ -2325,7 +2351,7 @@ NSS_EXTERN NSSCertificate **
 NSSCryptoContext_FindCertificatesBySubject
 (
   NSSCryptoContext *cc,
-  NSSUTF8 *subject,
+  NSSDER /*NSSUTF8*/ *subject,
   NSSCertificate *rvOpt[],
   PRUint32 maximumOpt, /* 0 for no max */
   NSSArena *arenaOpt
@@ -2389,7 +2415,10 @@ NSS_EXTERN NSSCertificate *
 NSSCryptoContext_FindBestCertificateByEmail
 (
   NSSCryptoContext *cc,
-  NSSASCII7 *email
+  NSSASCII7 *email,
+  NSSTime *timeOpt,
+  NSSUsage *usage,
+  NSSPolicies *policiesOpt
 );
 
 /*
@@ -2397,7 +2426,7 @@ NSSCryptoContext_FindBestCertificateByEmail
  *
  */
 
-NSS_EXTERN NSSCertificate *
+NSS_EXTERN NSSCertificate **
 NSSCryptoContext_FindCertificatesByEmail
 (
   NSSCryptoContext *cc,
@@ -2416,7 +2445,7 @@ NSS_EXTERN NSSCertificate *
 NSSCryptoContext_FindCertificateByOCSPHash
 (
   NSSCryptoContext *cc,
-  NSSITem *hash
+  NSSItem *hash
 );
 
 /*
@@ -2756,7 +2785,7 @@ NSSCryptoContext_BeginSign
  */
 
 NSS_EXTERN PRStatus
-NSSCryptoContext_BeginSign
+NSSCryptoContext_ContinueSign
 (
   NSSCryptoContext *cc,
   NSSItem *data
