@@ -32,7 +32,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- * $Id: sslsecur.c,v 1.11.2.1 2001/07/03 18:11:17 wtc%netscape.com Exp $
+ * $Id: sslsecur.c,v 1.18 2002/05/01 00:21:50 wtc%netscape.com Exp $
  */
 #include "cert.h"
 #include "secitem.h"
@@ -43,14 +43,6 @@
 #include "secoid.h"	/* for SECOID_GetALgorithmTag */
 #include "pk11func.h"	/* for PK11_GenerateRandom */
 
-#if defined(_WINDOWS)
-#include "winsock.h"	/* for MSG_PEEK */
-#elif defined(XP_MAC)
-#include "macsocket.h"
-#else
-#include <sys/socket.h> /* for MSG_PEEK */
-#endif
-
 #define MAX_BLOCK_CYPHER_SIZE	32
 
 #define TEST_FOR_FAILURE	/* reminder */
@@ -115,8 +107,6 @@ ssl_Do1stHandshake(sslSocket *ss)
     int rv        = SECSuccess;
     int loopCount = 0;
 
-    PORT_Assert(ss->gather != 0);
-
     do {
 	PORT_Assert( ssl_Have1stHandshakeLock(ss) );
 	PORT_Assert( !ssl_HaveRecvBufLock(ss)   );
@@ -134,23 +124,22 @@ ssl_Do1stHandshake(sslSocket *ss)
 	}
 	if (ss->handshake == 0) {
 	    ssl_GetRecvBufLock(ss);
-	    ss->gather->recordLen = 0;
+	    ss->gs.recordLen = 0;
 	    ssl_ReleaseRecvBufLock(ss);
 
 	    SSL_TRC(3, ("%d: SSL[%d]: handshake is completed",
 			SSL_GETPID(), ss->fd));
             /* call handshake callback for ssl v2 */
 	    /* for v3 this is done in ssl3_HandleFinished() */
-	    if ((ss->sec != NULL) &&               /* used SSL */
-	        (ss->handshakeCallback != NULL) && /* has callback */
+	    if ((ss->handshakeCallback != NULL) && /* has callback */
 		(!ss->firstHsDone) &&              /* only first time */
 		(ss->version < SSL_LIBRARY_VERSION_3_0)) {  /* not ssl3 */
 		ss->firstHsDone     = PR_TRUE;
 		(ss->handshakeCallback)(ss->fd, ss->handshakeCallbackData);
 	    }
 	    ss->firstHsDone         = PR_TRUE;
-	    ss->gather->writeOffset = 0;
-	    ss->gather->readOffset  = 0;
+	    ss->gs.writeOffset = 0;
+	    ss->gs.readOffset  = 0;
 	    break;
 	}
 	rv = (*ss->handshake)(ss);
@@ -199,7 +188,7 @@ SECStatus
 SSL_ResetHandshake(PRFileDesc *s, PRBool asServer)
 {
     sslSocket *ss;
-    SECStatus rv;
+    SECStatus status;
     PRNetAddr addr;
 
     ss = ssl_FindSocket(s);
@@ -231,21 +220,16 @@ SSL_ResetHandshake(PRFileDesc *s, PRBool asServer)
     ss->securityHandshake   = 0;
 
     ssl_GetRecvBufLock(ss);
-    ss->gather->state       = GS_INIT;
-    ss->gather->writeOffset = 0;
-    ss->gather->readOffset  = 0;
+    status = ssl_InitGather(&ss->gs);
     ssl_ReleaseRecvBufLock(ss);
 
     /*
-    ** Blow away old security state and get a fresh setup. This way if
-    ** ssl was used to connect to the first point in communication, ssl
-    ** can be used for the next layer.
+    ** Blow away old security state and get a fresh setup.
     */
-    if (ss->sec) {
-	ssl_DestroySecurityInfo(ss->sec);
-	ss->sec = 0;
-    }
-    rv = ssl_CreateSecurityInfo(ss);
+    ssl_GetXmitBufLock(ss); 
+    ssl_ResetSecurityInfo(&ss->sec);
+    status = ssl_CreateSecurityInfo(ss);
+    ssl_ReleaseXmitBufLock(ss); 
 
     ssl_ReleaseSSL3HandshakeLock(ss);
     ssl_Release1stHandshakeLock(ss);
@@ -256,7 +240,7 @@ SSL_ResetHandshake(PRFileDesc *s, PRBool asServer)
     SSL_UNLOCK_WRITER(ss);
     SSL_UNLOCK_READER(ss);
 
-    return rv;
+    return status;
 }
 
 /* For SSLv2, does nothing but return an error.
@@ -326,7 +310,6 @@ SSL_HandshakeCallback(PRFileDesc *fd, SSLHandshakeCallback cb,
     ssl_Get1stHandshakeLock(ss);
     ssl_GetSSL3HandshakeLock(ss);
 
-    PORT_Assert(ss->sec);
     ss->handshakeCallback     = cb;
     ss->handshakeCallbackData = client_data;
 
@@ -498,16 +481,13 @@ ssl_SendSavedWriteData(sslSocket *ss, sslBuffer *buf, sslSendFunc send)
 static int 
 DoRecv(sslSocket *ss, unsigned char *out, int len, int flags)
 {
-    sslGather *      gs;
     int              rv;
     int              amount;
     int              available;
 
     ssl_GetRecvBufLock(ss);
-    PORT_Assert((ss->sec != 0) && (ss->gather != 0));
-    gs = ss->gather;
 
-    available = gs->writeOffset - gs->readOffset;
+    available = ss->gs.writeOffset - ss->gs.readOffset;
     if (available == 0) {
 	/* Get some more data */
 	if (ss->version >= SSL_LIBRARY_VERSION_3_0) {
@@ -539,7 +519,7 @@ DoRecv(sslSocket *ss, unsigned char *out, int len, int flags)
 	}
 
 	/* See if any clear data is now available */
-	available = gs->writeOffset - gs->readOffset;
+	available = ss->gs.writeOffset - ss->gs.readOffset;
 	if (available == 0) {
 	    /*
 	    ** No partial data is available. Force error code to
@@ -558,9 +538,9 @@ DoRecv(sslSocket *ss, unsigned char *out, int len, int flags)
 
     /* Dole out clear data to reader */
     amount = PR_MIN(len, available);
-    PORT_Memcpy(out, gs->buf.buf + gs->readOffset, amount);
-    if (!(flags & MSG_PEEK)) {
-	gs->readOffset += amount;
+    PORT_Memcpy(out, ss->gs.buf.buf + ss->gs.readOffset, amount);
+    if (!(flags & PR_MSG_PEEK)) {
+	ss->gs.readOffset += amount;
     }
     rv = amount;
 
@@ -618,23 +598,13 @@ SSL_ConfigSecureServer(PRFileDesc *fd, CERTCertificate *cert,
 {
     SECStatus rv;
     sslSocket *ss;
-    sslSecurityInfo *sec;
+    sslServerCerts  *sc;
 
     ss = ssl_FindSocket(fd);
     if (!ss) {
 	return SECFailure;
     }
 
-    if ((rv = ssl_CreateSecurityInfo(ss)) != SECSuccess) {
-	return rv;
-    }
-
-    sec = ss->sec;
-    if (sec == NULL) {
-	PORT_SetError(SEC_ERROR_INVALID_ARGS);
-	return SECFailure;
-    }
-
     /* Both key and cert must have a value or be NULL */
     /* Passing a value of NULL will turn off key exchange algorithms that were
      * previously turned on */
@@ -654,39 +624,49 @@ SSL_ConfigSecureServer(PRFileDesc *fd, CERTCertificate *cert,
 	return SECFailure;
     }
 
+    sc = ss->serverCerts + kea;
     /* load the server certificate */
-    if (ss->serverCert[kea] != NULL)
-	CERT_DestroyCertificate(ss->serverCert[kea]);
+    if (sc->serverCert != NULL) {
+	CERT_DestroyCertificate(sc->serverCert);
+    	sc->serverCert = NULL;
+    }
     if (cert) {
-	ss->serverCert[kea] = CERT_DupCertificate(cert);
-	if (ss->serverCert[kea] == NULL)
+	SECKEYPublicKey * pubKey;
+	sc->serverCert = CERT_DupCertificate(cert);
+	if (!sc->serverCert)
 	    goto loser;
-    } else ss->serverCert[kea] = NULL;
+    	/* get the size of the cert's public key, and remember it */
+	pubKey = CERT_ExtractPublicKey(cert);
+	if (!pubKey) 
+	    goto loser;
+	sc->serverKeyBits = SECKEY_PublicKeyStrength(pubKey) * BPB;
+	SECKEY_DestroyPublicKey(pubKey); 
+	pubKey = NULL;
+    }
 
 
     /* load the server cert chain */
-    if (ss->serverCertChain[kea] != NULL)
-	CERT_DestroyCertificateList(ss->serverCertChain[kea]);
+    if (sc->serverCertChain != NULL) {
+	CERT_DestroyCertificateList(sc->serverCertChain);
+    	sc->serverCertChain = NULL;
+    }
     if (cert) {
-	ss->serverCertChain[kea] = CERT_CertChainFromCert(
-			    ss->serverCert[kea], certUsageSSLServer, PR_TRUE);
-	if (ss->serverCertChain[kea] == NULL)
+	sc->serverCertChain = CERT_CertChainFromCert(
+			    sc->serverCert, certUsageSSLServer, PR_TRUE);
+	if (sc->serverCertChain == NULL)
 	     goto loser;
-    } else ss->serverCertChain[kea] = NULL;
-
-
-    /* Only do this once because it's global. */
-    if (ssl3_server_ca_list == NULL)
-	ssl3_server_ca_list = CERT_GetSSLCACerts(ss->dbHandle);
+    }
 
     /* load the private key */
-    if (ss->serverKey[kea] != NULL)
-	SECKEY_DestroyPrivateKey(ss->serverKey[kea]);
+    if (sc->serverKey != NULL) {
+	SECKEY_DestroyPrivateKey(sc->serverKey);
+    	sc->serverKey = NULL;
+    }
     if (key) {
-	ss->serverKey[kea] = SECKEY_CopyPrivateKey(key);
-	if (ss->serverKey[kea] == NULL)
+	sc->serverKey = SECKEY_CopyPrivateKey(key);
+	if (sc->serverKey == NULL)
 	    goto loser;
-    } else ss->serverKey[kea] = NULL;
+    }
 
     if (kea == kt_rsa) {
         rv = ssl3_CreateRSAStepDownKeys(ss);
@@ -695,20 +675,24 @@ SSL_ConfigSecureServer(PRFileDesc *fd, CERTCertificate *cert,
 	}
     }
 
+    /* Only do this once because it's global. */
+    if (ssl3_server_ca_list == NULL)
+	ssl3_server_ca_list = CERT_GetSSLCACerts(ss->dbHandle);
+
     return SECSuccess;
 
 loser:
-    if (ss->serverCert[kea] != NULL) {
-	CERT_DestroyCertificate(ss->serverCert[kea]);
-	ss->serverCert[kea] = NULL;
+    if (sc->serverCert != NULL) {
+	CERT_DestroyCertificate(sc->serverCert);
+	sc->serverCert = NULL;
     }
-    if (ss->serverCertChain != NULL) {
-	CERT_DestroyCertificateList(ss->serverCertChain[kea]);
-	ss->serverCertChain[kea] = NULL;
+    if (sc->serverCertChain != NULL) {
+	CERT_DestroyCertificateList(sc->serverCertChain);
+	sc->serverCertChain = NULL;
     }
-    if (ss->serverKey[kea] != NULL) {
-	SECKEY_DestroyPrivateKey(ss->serverKey[kea]);
-	ss->serverKey[kea] = NULL;
+    if (sc->serverKey != NULL) {
+	SECKEY_DestroyPrivateKey(sc->serverKey);
+	sc->serverKey = NULL;
     }
     return SECFailure;
 }
@@ -718,120 +702,72 @@ loser:
 SECStatus
 ssl_CreateSecurityInfo(sslSocket *ss)
 {
-    sslSecurityInfo * sec  = (sslSecurityInfo *)0;
-    sslGather *       gs   = (sslGather *      )0;
-    int               rv;
-
-    unsigned char     padbuf[MAX_BLOCK_CYPHER_SIZE];
-
-    if (ss->sec) {
-	return SECSuccess;
-    }
-
-    /* Force the global RNG to generate some random data that we never use */
-    PK11_GenerateRandom(padbuf, sizeof padbuf);
-
-    ss->sec = sec = (sslSecurityInfo*) PORT_ZAlloc(sizeof(sslSecurityInfo));
-    if (!sec) {
-	goto loser;
-    }
+    SECStatus status;
 
     /* initialize sslv2 socket to send data in the clear. */
     ssl2_UseClearSendFunc(ss);
 
-    sec->blockSize  = 1;
-    sec->blockShift = 0;
+    ss->sec.blockSize  = 1;
+    ss->sec.blockShift = 0;
 
-    ssl_GetRecvBufLock(ss);
-    if ((gs = ss->gather) == 0) {
-	ss->gather = gs = ssl_NewGather();
-	if (!gs) {
-	    goto loser;
-	}
-    }
+    ssl_GetXmitBufLock(ss); 
+    status = sslBuffer_Grow(&ss->sec.writeBuf, 4096);
+    ssl_ReleaseXmitBufLock(ss); 
 
-    rv = sslBuffer_Grow(&gs->buf, 4096);
-    if (rv) {
-	goto loser;
-    }
-    ssl_ReleaseRecvBufLock(ss);
-
-    ssl_GetXmitBufLock(ss);
-    rv = sslBuffer_Grow(&sec->writeBuf, 4096);
-    if (rv) {
-	goto loser;
-    }
-    ssl_ReleaseXmitBufLock(ss);
-
-    SSL_TRC(5, ("%d: SSL[%d]: security info created", SSL_GETPID(), ss->fd));
-    return SECSuccess;
-
-  loser:
-    if (sec) {
-	PORT_Free(sec);
-	ss->sec = sec = (sslSecurityInfo *)0;
-    }
-    if (gs) {
-    	ssl_DestroyGather(gs);
-	ss->gather = gs = (sslGather *)0;
-    }
-    return SECFailure;
+    return status;
 }
 
-/* XXX We should handle errors better in this function. */
-/* This function assumes that none of the pointers in ss need to be 
-** freed.
-*/
 SECStatus
 ssl_CopySecurityInfo(sslSocket *ss, sslSocket *os)
 {
-    sslSecurityInfo *sec, *osec;
     int rv;
 
-    rv = ssl_CreateSecurityInfo(ss);
-    if (rv < 0) {
-	goto loser;
-    }
-    sec = ss->sec;
-    osec = os->sec;
+    ss->sec.send 		= os->sec.send;
+    ss->sec.isServer 		= os->sec.isServer;
+    ss->sec.keyBits    		= os->sec.keyBits;
+    ss->sec.secretKeyBits 	= os->sec.secretKeyBits;
 
-    sec->send 		= osec->send;
-    sec->isServer 	= osec->isServer;
-    sec->keyBits    	= osec->keyBits;
-    sec->secretKeyBits 	= osec->secretKeyBits;
+    ss->sec.peerCert   		= CERT_DupCertificate(os->sec.peerCert);
+    if (os->sec.peerCert && !ss->sec.peerCert)
+    	goto loser;
 
-    sec->peerCert   	= CERT_DupCertificate(osec->peerCert);
-
-    sec->cache      	= osec->cache;
-    sec->uncache    	= osec->uncache;
+    ss->sec.cache      		= os->sec.cache;
+    ss->sec.uncache    		= os->sec.uncache;
 
     /* we don't dup the connection info. */
 
-    sec->sendSequence 	= osec->sendSequence;
-    sec->rcvSequence 	= osec->rcvSequence;
+    ss->sec.sendSequence 	= os->sec.sendSequence;
+    ss->sec.rcvSequence 	= os->sec.rcvSequence;
 
-    if (osec->hash && osec->hashcx) {
-	sec->hash 	= osec->hash;
-	sec->hashcx 	= osec->hash->clone(osec->hashcx);
+    if (os->sec.hash && os->sec.hashcx) {
+	ss->sec.hash 		= os->sec.hash;
+	ss->sec.hashcx 		= os->sec.hash->clone(os->sec.hashcx);
+	if (os->sec.hashcx && !ss->sec.hashcx)
+	    goto loser;
     } else {
-	sec->hash 	= NULL;
-	sec->hashcx 	= NULL;
+	ss->sec.hash 		= NULL;
+	ss->sec.hashcx 		= NULL;
     }
 
-    SECITEM_CopyItem(0, &sec->sendSecret, &osec->sendSecret);
-    SECITEM_CopyItem(0, &sec->rcvSecret,  &osec->rcvSecret);
+    SECITEM_CopyItem(0, &ss->sec.sendSecret, &os->sec.sendSecret);
+    if (os->sec.sendSecret.data && !ss->sec.sendSecret.data)
+    	goto loser;
+    SECITEM_CopyItem(0, &ss->sec.rcvSecret,  &os->sec.rcvSecret);
+    if (os->sec.rcvSecret.data && !ss->sec.rcvSecret.data)
+    	goto loser;
 
-    PORT_Assert(osec->readcx  == 0);
-    sec->readcx     	= osec->readcx;	/* XXX wrong if readcx  != 0 */
-    PORT_Assert(osec->writecx == 0);
-    sec->writecx    	= osec->writecx; /* XXX wrong if writecx != 0 */
-    sec->destroy    	= 0;		 /* XXX wrong if either cx != 0*/
+    /* XXX following code is wrong if either cx != 0 */
+    PORT_Assert(os->sec.readcx  == 0);
+    PORT_Assert(os->sec.writecx == 0);
+    ss->sec.readcx     		= os->sec.readcx;
+    ss->sec.writecx    		= os->sec.writecx;
+    ss->sec.destroy    		= 0;	
 
-    sec->enc        	= osec->enc;
-    sec->dec        	= osec->dec;
+    ss->sec.enc        		= os->sec.enc;
+    ss->sec.dec        		= os->sec.dec;
 
-    sec->blockShift 	= osec->blockShift;
-    sec->blockSize  	= osec->blockSize;
+    ss->sec.blockShift 		= os->sec.blockShift;
+    ss->sec.blockSize  		= os->sec.blockSize;
 
     return SECSuccess;
 
@@ -839,53 +775,69 @@ loser:
     return SECFailure;
 }
 
+/* Reset sec back to its initial state.
+** Caller holds any relevant locks.
+*/
+void 
+ssl_ResetSecurityInfo(sslSecurityInfo *sec)
+{
+    /* Destroy MAC */
+    if (sec->hash && sec->hashcx) {
+	(*sec->hash->destroy)(sec->hashcx, PR_TRUE);
+	sec->hashcx = NULL;
+	sec->hash = NULL;
+    }
+    SECITEM_ZfreeItem(&sec->sendSecret, PR_FALSE);
+    SECITEM_ZfreeItem(&sec->rcvSecret, PR_FALSE);
+
+    /* Destroy ciphers */
+    if (sec->destroy) {
+	(*sec->destroy)(sec->readcx, PR_TRUE);
+	(*sec->destroy)(sec->writecx, PR_TRUE);
+	sec->readcx = NULL;
+	sec->writecx = NULL;
+    } else {
+	PORT_Assert(sec->readcx == 0);
+	PORT_Assert(sec->writecx == 0);
+    }
+    sec->readcx = 0;
+    sec->writecx = 0;
+
+    if (sec->localCert) {
+	CERT_DestroyCertificate(sec->localCert);
+	sec->localCert = NULL;
+    }
+    if (sec->peerCert) {
+	CERT_DestroyCertificate(sec->peerCert);
+	sec->peerCert = NULL;
+    }
+    if (sec->peerKey) {
+	SECKEY_DestroyPublicKey(sec->peerKey);
+	sec->peerKey = NULL;
+    }
+
+    /* cleanup the ci */
+    if (sec->ci.sid != NULL) {
+	ssl_FreeSID(sec->ci.sid);
+    }
+    PORT_ZFree(sec->ci.sendBuf.buf, sec->ci.sendBuf.space);
+    memset(&sec->ci, 0, sizeof sec->ci);
+}
+
 /*
 ** Called from SSL_ResetHandshake (above), and 
 **        from ssl_FreeSocket     in sslsock.c
+** Caller should hold relevant locks (e.g. XmitBufLock)
 */
 void 
 ssl_DestroySecurityInfo(sslSecurityInfo *sec)
 {
-    if (sec != 0) {
-	/* Destroy MAC */
-	if (sec->hash && sec->hashcx) {
-	    (*sec->hash->destroy)(sec->hashcx, PR_TRUE);
-	    sec->hashcx = 0;
-	}
-	SECITEM_ZfreeItem(&sec->sendSecret, PR_FALSE);
-	SECITEM_ZfreeItem(&sec->rcvSecret, PR_FALSE);
+    ssl_ResetSecurityInfo(sec);
 
-	/* Destroy ciphers */
-	if (sec->destroy) {
-	    (*sec->destroy)(sec->readcx, PR_TRUE);
-	    (*sec->destroy)(sec->writecx, PR_TRUE);
-	} else {
-	    PORT_Assert(sec->readcx == 0);
-	    PORT_Assert(sec->writecx == 0);
-	}
-	sec->readcx = 0;
-	sec->writecx = 0;
+    PORT_ZFree(sec->writeBuf.buf, sec->writeBuf.space);
+    sec->writeBuf.buf = 0;
 
-	/* etc. */
-	PORT_ZFree(sec->writeBuf.buf, sec->writeBuf.space);
-	sec->writeBuf.buf = 0;
-
-	if (sec->peerCert) {
-	    CERT_DestroyCertificate(sec->peerCert);
-	    sec->peerCert = NULL;
-	}
-	if (sec->peerKey) {
-	    SECKEY_DestroyPublicKey(sec->peerKey);
-	    sec->peerKey = NULL;
-	}
-
-	PORT_ZFree(sec->ci.sendBuf.buf, sec->ci.sendBuf.space);
-	if (sec->ci.sid != NULL) {
-	    ssl_FreeSID(sec->ci.sid);
-	}
-
-	PORT_ZFree(sec, sizeof *sec);
-    }
+    memset(sec, 0, sizeof *sec);
 }
 
 /************************************************************************/
@@ -896,8 +848,6 @@ ssl_SecureConnect(sslSocket *ss, const PRNetAddr *sa)
     PRFileDesc *osfd = ss->fd->lower;
     int rv;
 
-    PORT_Assert(ss->sec != 0);
-
     if ( ss->handshakeAsServer ) {
 	ss->securityHandshake = ssl2_BeginServerHandshake;
 	ss->handshaking = sslHandshakingAsServer;
@@ -935,6 +885,12 @@ ssl_SecureClose(sslSocket *ss)
 	!ss->recvdCloseNotify                   &&
 	(ss->ssl3 != NULL)) {
 
+	/* We don't want the final alert to be Nagle delayed. */
+	if (!ss->delayDisabled) {
+	    ssl_EnableNagleDelay(ss, PR_FALSE);
+	    ss->delayDisabled = 1;
+	}
+
 	(void) SSL3_SendAlert(ss, alert_warning, close_notify);
     }
     rv = ssl_DefClose(ss);
@@ -980,14 +936,13 @@ ssl_SecureRecv(sslSocket *ss, unsigned char *buf, int len, int flags)
     sslSecurityInfo *sec;
     int              rv   = 0;
 
-    PORT_Assert(ss->sec != 0);
-    sec = ss->sec;
+    sec = &ss->sec;
 
     if (ss->shutdownHow & ssl_SHUTDOWN_RCV) {
 	PORT_SetError(PR_SOCKET_SHUTDOWN_ERROR);
     	return PR_FAILURE;
     }
-    if (flags & ~MSG_PEEK) {
+    if (flags & ~PR_MSG_PEEK) {
 	PORT_SetError(PR_INVALID_ARGUMENT_ERROR);
     	return PR_FAILURE;
     }
@@ -1036,12 +991,8 @@ ssl_SecureRead(sslSocket *ss, unsigned char *buf, int len)
 int
 ssl_SecureSend(sslSocket *ss, const unsigned char *buf, int len, int flags)
 {
-    sslSecurityInfo *sec;
     int              rv		= 0;
 
-    PORT_Assert(ss->sec != 0);
-    sec = ss->sec;
-
     if (ss->shutdownHow & ssl_SHUTDOWN_SEND) {
 	PORT_SetError(PR_SOCKET_SHUTDOWN_ERROR);
     	return PR_FAILURE;
@@ -1055,7 +1006,7 @@ ssl_SecureSend(sslSocket *ss, const unsigned char *buf, int len, int flags)
     if (ss->pendingBuf.len != 0) {
 	PORT_Assert(ss->pendingBuf.len > 0);
 	rv = ssl_SendSavedWriteData(ss, &ss->pendingBuf, ssl_DefSend);
-	if (ss->pendingBuf.len != 0) {
+	if (rv >= 0 && ss->pendingBuf.len != 0) {
 	    PORT_Assert(ss->pendingBuf.len > 0);
 	    PORT_SetError(PR_WOULD_BLOCK_ERROR);
 	    rv = SECFailure;
@@ -1097,7 +1048,7 @@ ssl_SecureSend(sslSocket *ss, const unsigned char *buf, int len, int flags)
      *  ssl3_SendApplicationData
      */
     ssl_GetXmitBufLock(ss);
-    rv = (*sec->send)(ss, buf, len, flags);
+    rv = (*ss->sec.send)(ss, buf, len, flags);
     ssl_ReleaseXmitBufLock(ss);
     ss->writerThread = NULL;
     return rv;
@@ -1122,9 +1073,6 @@ SSL_BadCertHook(PRFileDesc *fd, SSLBadCertHandler f, void *arg)
 	return SECFailure;
     }
 
-    if ((rv = ssl_CreateSecurityInfo(ss)) != 0) {
-	return rv;
-    }
     ss->handleBadCert = f;
     ss->badCertArg = arg;
 
@@ -1181,13 +1129,9 @@ SSL_DataPending(PRFileDesc *fd)
 	ssl_Get1stHandshakeLock(ss);
 	ssl_GetSSL3HandshakeLock(ss);
 
-	/* Create ss->sec if it doesn't already exist. */
-	rv = ssl_CreateSecurityInfo(ss);
-	if (rv == SECSuccess) {
-	    ssl_GetRecvBufLock(ss);
-	    rv = ss->gather->writeOffset - ss->gather->readOffset;
-	    ssl_ReleaseRecvBufLock(ss);
-	}
+	ssl_GetRecvBufLock(ss);
+	rv = ss->gs.writeOffset - ss->gs.readOffset;
+	ssl_ReleaseRecvBufLock(ss);
 
 	ssl_ReleaseSSL3HandshakeLock(ss);
 	ssl_Release1stHandshakeLock(ss);
@@ -1206,8 +1150,8 @@ SSL_InvalidateSession(PRFileDesc *fd)
 	ssl_Get1stHandshakeLock(ss);
 	ssl_GetSSL3HandshakeLock(ss);
 
-	if (ss->sec && ss->sec->ci.sid) {
-	    ss->sec->uncache(ss->sec->ci.sid);
+	if (ss->sec.ci.sid) {
+	    ss->sec.uncache(ss->sec.ci.sid);
 	    rv = SECSuccess;
 	}
 
@@ -1222,24 +1166,25 @@ SSL_GetSessionID(PRFileDesc *fd)
 {
     sslSocket *    ss;
     SECItem *      item = NULL;
-    sslSessionID * sid;
 
     ss = ssl_FindSocket(fd);
     if (ss) {
 	ssl_Get1stHandshakeLock(ss);
 	ssl_GetSSL3HandshakeLock(ss);
 
-	if (ss->useSecurity && ss->firstHsDone && ss->sec && ss->sec->ci.sid) {
-	    sid = ss->sec->ci.sid;
+	if (ss->useSecurity && ss->firstHsDone && ss->sec.ci.sid) {
 	    item = (SECItem *)PORT_Alloc(sizeof(SECItem));
-	    if (sid->version < SSL_LIBRARY_VERSION_3_0) {
-		item->len = SSL_SESSIONID_BYTES;
-		item->data = (unsigned char*)PORT_Alloc(item->len);
-		PORT_Memcpy(item->data, sid->u.ssl2.sessionID, item->len);
-	    } else {
-		item->len = sid->u.ssl3.sessionIDLength;
-		item->data = (unsigned char*)PORT_Alloc(item->len);
-		PORT_Memcpy(item->data, sid->u.ssl3.sessionID, item->len);
+	    if (item) {
+		sslSessionID * sid = ss->sec.ci.sid;
+		if (sid->version < SSL_LIBRARY_VERSION_3_0) {
+		    item->len = SSL2_SESSIONID_BYTES;
+		    item->data = (unsigned char*)PORT_Alloc(item->len);
+		    PORT_Memcpy(item->data, sid->u.ssl2.sessionID, item->len);
+		} else {
+		    item->len = sid->u.ssl3.sessionIDLength;
+		    item->data = (unsigned char*)PORT_Alloc(item->len);
+		    PORT_Memcpy(item->data, sid->u.ssl3.sessionID, item->len);
+		}
 	    }
 	}
 
