@@ -35,7 +35,7 @@
  * may use your version of this file under either the MPL or the
  * GPL.
  *
- * $Id: sslsock.c,v 1.19 2001/06/23 00:01:17 nelsonb%netscape.com Exp $
+ * $Id: sslsock.c,v 1.28 2002/04/04 00:14:11 nelsonb%netscape.com Exp $
  */
 #include "seccomon.h"
 #include "cert.h"
@@ -55,7 +55,11 @@ struct cipherPolicyStr {
 
 typedef struct cipherPolicyStr cipherPolicy;
 
-/* this table reflects Netscape's browser policies. */
+/* This table contains two preconfigured policies: Export and France.
+** It is used only by the functions SSL_SetDomesticPolicy, 
+** SSL_SetExportPolicy, and SSL_SetFrancyPolicy.
+** Order of entries is not important.
+*/
 static cipherPolicy ssl_ciphers[] = {	   /*   Export           France   */
  {  SSL_EN_RC4_128_WITH_MD5,		    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
  {  SSL_EN_RC4_128_EXPORT40_WITH_MD5,	    SSL_ALLOWED,     SSL_ALLOWED },
@@ -80,6 +84,12 @@ static cipherPolicy ssl_ciphers[] = {	   /*   Export           France   */
  {  SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA,      SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
  {  TLS_DHE_DSS_WITH_RC4_128_SHA,           SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
  {  SSL_RSA_WITH_NULL_MD5,		    SSL_ALLOWED,     SSL_ALLOWED },
+ {  TLS_DHE_DSS_WITH_AES_128_CBC_SHA, 	    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
+ {  TLS_DHE_RSA_WITH_AES_128_CBC_SHA,       SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
+ {  TLS_RSA_WITH_AES_128_CBC_SHA,     	    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
+ {  TLS_DHE_DSS_WITH_AES_256_CBC_SHA, 	    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
+ {  TLS_DHE_RSA_WITH_AES_256_CBC_SHA,       SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
+ {  TLS_RSA_WITH_AES_256_CBC_SHA,     	    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED },
  {  TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA,    SSL_ALLOWED,     SSL_NOT_ALLOWED },
  {  TLS_RSA_EXPORT1024_WITH_RC4_56_SHA,     SSL_ALLOWED,     SSL_NOT_ALLOWED },
  {  0,					    SSL_NOT_ALLOWED, SSL_NOT_ALLOWED }
@@ -246,18 +256,24 @@ ssl_DupSocket(sslSocket *os)
 	     * during the for loop.
 	     */
 	    int i;
+	    sslServerCerts * oc = os->serverCerts;
+	    sslServerCerts * sc = ss->serverCerts;
 
-	    for (i=kt_null; i < kt_kea_size; i++) {
-		if (os->serverCert[i] && os->serverCertChain[i]) {
-		    ss->serverCert[i] = CERT_DupCertificate(os->serverCert[i]);
-		    ss->serverCertChain[i] = CERT_DupCertList(
-		                                       os->serverCertChain[i]);
+	    for (i=kt_null; i < kt_kea_size; i++, oc++, sc++) {
+		if (oc->serverCert && oc->serverCertChain) {
+		    sc->serverCert      = CERT_DupCertificate(oc->serverCert);
+		    sc->serverCertChain = CERT_DupCertList(oc->serverCertChain);
+		    if (!sc->serverCertChain) 
+		    	goto loser;
 		} else {
-		    ss->serverCert[i]      = NULL;
-		    ss->serverCertChain[i] = NULL;
+		    sc->serverCert      = NULL;
+		    sc->serverCertChain = NULL;
 		}
-		ss->serverKey[i] = os->serverKey[i] ?
-				SECKEY_CopyPrivateKey(os->serverKey[i]) : NULL;
+		sc->serverKey = oc->serverKey ?
+				SECKEY_CopyPrivateKey(oc->serverKey) : NULL;
+		if (oc->serverKey && !sc->serverKey)
+		    goto loser;
+	        sc->serverKeyBits = oc->serverKeyBits;
 	    }
 	    ss->stepDownKeyPair = !os->stepDownKeyPair ? NULL :
 		                  ssl3_GetKeyPairRef(os->stepDownKeyPair);
@@ -278,29 +294,107 @@ ssl_DupSocket(sslSocket *os)
 	    /* Create security data */
 	    rv = ssl_CopySecurityInfo(ss, os);
 	    if (rv != SECSuccess) {
-		goto losage;
+		goto loser;
 	    }
 	}
     }
     return ss;
 
-  losage:
+loser:
+    ssl_FreeSocket(ss);
     return NULL;
 }
 
+static void
+ssl_DestroyLocks(sslSocket *ss)
+{
+
+    /* Destroy locks. */
+    if (ss->firstHandshakeLock) {
+    	PZ_DestroyMonitor(ss->firstHandshakeLock);
+	ss->firstHandshakeLock = NULL;
+    }
+    if (ss->ssl3HandshakeLock) {
+    	PZ_DestroyMonitor(ss->ssl3HandshakeLock);
+	ss->ssl3HandshakeLock = NULL;
+    }
+    if (ss->specLock) {
+    	NSSRWLock_Destroy(ss->specLock);
+	ss->specLock = NULL;
+    }
+
+    if (ss->recvLock) {
+    	PZ_DestroyLock(ss->recvLock);
+	ss->recvLock = NULL;
+    }
+    if (ss->sendLock) {
+    	PZ_DestroyLock(ss->sendLock);
+	ss->sendLock = NULL;
+    }
+    if (ss->xmitBufLock) {
+    	PZ_DestroyMonitor(ss->xmitBufLock);
+	ss->xmitBufLock = NULL;
+    }
+    if (ss->recvBufLock) {
+    	PZ_DestroyMonitor(ss->recvBufLock);
+	ss->recvBufLock = NULL;
+    }
+}
+
+/* Caller holds any relevant locks */
+static void
+ssl_DestroySocketContents(sslSocket *ss)
+{
+    /* "i" should be of type SSLKEAType, but CC on IRIX complains during
+     * the for loop.
+     */
+    int        i;
+
+    /* Free up socket */
+    ssl_DestroySecurityInfo(&ss->sec);
+
+    ssl3_DestroySSL3Info(ss->ssl3);
+
+    PORT_Free(ss->saveBuf.buf);
+    PORT_Free(ss->pendingBuf.buf);
+    ssl_DestroyGather(&ss->gs);
+
+    if (ss->peerID != NULL)
+	PORT_Free(ss->peerID);
+    if (ss->url != NULL)
+	PORT_Free((void *)ss->url);	/* CONST */
+    if (ss->cipherSpecs) {
+	PORT_Free(ss->cipherSpecs);
+	ss->cipherSpecs     = NULL;
+	ss->sizeCipherSpecs = 0;
+    }
+
+    /* Clean up server configuration */
+    for (i=kt_null; i < kt_kea_size; i++) {
+	sslServerCerts * sc = ss->serverCerts + i;
+	if (sc->serverCert != NULL)
+	    CERT_DestroyCertificate(sc->serverCert);
+	if (sc->serverCertChain != NULL)
+	    CERT_DestroyCertificateList(sc->serverCertChain);
+	if (sc->serverKey != NULL)
+	    SECKEY_DestroyPrivateKey(sc->serverKey);
+    }
+    if (ss->stepDownKeyPair) {
+	ssl3_FreeKeyPair(ss->stepDownKeyPair);
+	ss->stepDownKeyPair = NULL;
+    }
+}
+
 /*
  * free an sslSocket struct, and all the stuff that hangs off of it
  */
 void
 ssl_FreeSocket(sslSocket *ss)
 {
-    /* "i" should be of type SSLKEAType, but CC on IRIX complains during
-     * the for loop.
-     */
-    int        i;
-
+#ifdef DEBUG
     sslSocket *fs;
     sslSocket  lSock;
+#endif
 
 /* Get every lock you can imagine!
 ** Caller already holds these:
@@ -318,36 +412,10 @@ ssl_FreeSocket(sslSocket *ss)
     *fs = *ss;				/* Copy the old socket structure, */
     PORT_Memset(ss, 0x1f, sizeof *ss);  /* then blast the old struct ASAP. */
 #else
-    fs = ss;
+#define fs ss
 #endif
 
-    /* Free up socket */
-    ssl_DestroySecurityInfo(fs->sec);
-    ssl3_DestroySSL3Info(fs->ssl3);
-    PORT_Free(fs->saveBuf.buf);
-    PORT_Free(fs->pendingBuf.buf);
-    if (fs->gather) {
-	ssl_DestroyGather(fs->gather);
-    }
-    if (fs->peerID != NULL)
-	PORT_Free(fs->peerID);
-    if (fs->url != NULL)
-	PORT_Free((void *)fs->url);	/* CONST */
-
-    /* Clean up server configuration */
-    for (i=kt_null; i < kt_kea_size; i++) {
-	if (fs->serverCert[i] != NULL)
-	    CERT_DestroyCertificate(fs->serverCert[i]);
-	if (fs->serverCertChain[i] != NULL)
-	    CERT_DestroyCertificateList(fs->serverCertChain[i]);
-	if (fs->serverKey[i] != NULL)
-	    SECKEY_DestroyPrivateKey(fs->serverKey[i]);
-    }
-    if (fs->stepDownKeyPair) {
-	ssl3_FreeKeyPair(fs->stepDownKeyPair);
-	fs->stepDownKeyPair = NULL;
-    }
-
+    ssl_DestroySocketContents(fs);
 
     /* Release all the locks acquired above.  */
     SSL_UNLOCK_READER(fs);
@@ -358,45 +426,12 @@ ssl_FreeSocket(sslSocket *ss)
     ssl_ReleaseXmitBufLock(fs);
     ssl_ReleaseSpecWriteLock(fs);
 
-    /* Destroy locks. */
-    if (fs->firstHandshakeLock) {
-    	PZ_DestroyMonitor(fs->firstHandshakeLock);
-	fs->firstHandshakeLock = NULL;
-    }
-    if (fs->ssl3HandshakeLock) {
-    	PZ_DestroyMonitor(fs->ssl3HandshakeLock);
-	fs->ssl3HandshakeLock = NULL;
-    }
-    if (fs->specLock) {
-    	NSSRWLock_Destroy(fs->specLock);
-	fs->specLock = NULL;
-    }
-
-    if (fs->recvLock) {
-    	PZ_DestroyLock(fs->recvLock);
-	fs->recvLock = NULL;
-    }
-    if (fs->sendLock) {
-    	PZ_DestroyLock(fs->sendLock);
-	fs->sendLock = NULL;
-    }
-    if (fs->xmitBufLock) {
-    	PZ_DestroyMonitor(fs->xmitBufLock);
-	fs->xmitBufLock = NULL;
-    }
-    if (fs->recvBufLock) {
-    	PZ_DestroyMonitor(fs->recvBufLock);
-	fs->recvBufLock = NULL;
-    }
-    if (fs->cipherSpecs) {
-	PORT_Free(fs->cipherSpecs);
-	fs->cipherSpecs     = NULL;
-	fs->sizeCipherSpecs = 0;
-    }
+    ssl_DestroyLocks(fs);
 
     PORT_Free(ss);	/* free the caller's copy, not ours. */
     return;
 }
+#undef fs
 
 /************************************************************************/
 SECStatus 
@@ -426,13 +461,6 @@ PrepareSocket(sslSocket *ss)
 {
     SECStatus     rv = SECSuccess;
 
-    if (ss->useSecurity) {
-	rv = ssl_CreateSecurityInfo(ss);
-	if (rv != SECSuccess) {
-	    return rv;
-	}
-    }
-
     ssl_ChooseOps(ss);
     return rv;
 }
@@ -1252,15 +1280,11 @@ ssl_GetPeerName(PRFileDesc *fd, PRNetAddr *addr)
 SECStatus
 ssl_GetPeerInfo(sslSocket *ss)
 {
-    sslConnectInfo *  ci;
-    PRNetAddr         sin;
-    int               rv;
     PRFileDesc *      osfd;
-
-    PORT_Assert((ss->sec != 0));
+    int               rv;
+    PRNetAddr         sin;
 
     osfd = ss->fd->lower;
-    ci   = &ss->sec->ci;
 
     PORT_Memset(&sin, 0, sizeof(sin));
     rv = osfd->methods->getpeername(osfd, &sin);
@@ -1268,14 +1292,15 @@ ssl_GetPeerInfo(sslSocket *ss)
 	return SECFailure;
     }
     ss->TCPconnected = 1;
-    /* we have to mask off the high byte because AIX is lame */
-    if ((sin.inet.family & 0xff) == PR_AF_INET) {
-        PR_ConvertIPv4AddrToIPv6(sin.inet.ip, &ci->peer);
-	ci->port = sin.inet.port;
+    if (sin.inet.family == PR_AF_INET) {
+        PR_ConvertIPv4AddrToIPv6(sin.inet.ip, &ss->sec.ci.peer);
+	ss->sec.ci.port = sin.inet.port;
+    } else if (sin.ipv6.family == PR_AF_INET6) {
+	ss->sec.ci.peer = sin.ipv6.ip;
+	ss->sec.ci.port = sin.ipv6.port;
     } else {
-        PORT_Assert(sin.ipv6.family == PR_AF_INET6);
-	ci->peer = sin.ipv6.ip;
-	ci->port = sin.ipv6.port;
+	PORT_SetError(PR_ADDRESS_NOT_SUPPORTED_ERROR);
+    	return SECFailure;
     }
     return SECSuccess;
 }
@@ -1448,7 +1473,7 @@ ssl_WriteV(PRFileDesc *fd, const PRIOVec *iov, PRInt32 vectors,
 #define HANDLE_ERR(rv, len) \
     if (rv != len) { \
 	if (rv < 0) { \
-	    if (blocking \
+	    if (!blocking \
 		&& (PR_GetError() == PR_WOULD_BLOCK_ERROR) \
 		&& (sent > 0)) { \
 		return sent; \
@@ -1457,7 +1482,7 @@ ssl_WriteV(PRFileDesc *fd, const PRIOVec *iov, PRInt32 vectors,
 	    } \
 	} \
 	/* Only a nonblocking socket can have partial sends */ \
-	PR_ASSERT(blocking); \
+	PR_ASSERT(!blocking); \
 	return sent; \
     } 
 #define SEND(bfr, len) \
@@ -1776,14 +1801,11 @@ ssl_NewSocket(void)
 {
     sslSocket *ss;
 #ifdef DEBUG
+#if (defined(XP_UNIX) || defined(XP_WIN32) || defined(XP_BEOS)) && !defined(_WIN32_WCE)
     static int firsttime = 1;
-#endif
 
-#ifdef DEBUG
-#if defined(XP_UNIX) || defined(XP_WIN32)
     if (firsttime) {
 	firsttime = 0;
-
 	{
 	    char *ev = getenv("SSLDEBUG");
 	    if (ev && ev[0]) {
@@ -1811,6 +1833,7 @@ ssl_NewSocket(void)
 	 * complains during the for loop.
 	 */
 	int i;
+	SECStatus status;
  
 	ss->useSecurity        = ssl_defaults.useSecurity;
 	ss->useSocks           = PR_FALSE;
@@ -1835,9 +1858,11 @@ ssl_NewSocket(void)
         ss->url                = NULL;
 
 	for (i=kt_null; i < kt_kea_size; i++) {
-	    ss->serverCert[i]      = NULL;
-	    ss->serverCertChain[i] = NULL;
-	    ss->serverKey[i]       = NULL;
+	    sslServerCerts * sc = ss->serverCerts + i;
+	    sc->serverCert      = NULL;
+	    sc->serverCertChain = NULL;
+	    sc->serverKey       = NULL;
+	    sc->serverKeyBits   = 0;
 	}
 	ss->stepDownKeyPair    = NULL;
 	ss->dbHandle           = CERT_GetDefaultCertDB();
@@ -1855,14 +1880,31 @@ ssl_NewSocket(void)
 	ssl3_InitSocketPolicy(ss);
 
 	ss->firstHandshakeLock = PZ_NewMonitor(nssILockSSL);
+	if (!ss->firstHandshakeLock) goto loser;
 	ss->ssl3HandshakeLock  = PZ_NewMonitor(nssILockSSL);
+	if (!ss->ssl3HandshakeLock) goto loser;
 	ss->specLock           = NSSRWLock_New(SSL_LOCK_RANK_SPEC, NULL);
+	if (!ss->specLock) goto loser;
 	ss->recvBufLock        = PZ_NewMonitor(nssILockSSL);
+	if (!ss->recvBufLock) goto loser;
 	ss->xmitBufLock        = PZ_NewMonitor(nssILockSSL);
+	if (!ss->xmitBufLock) goto loser;
 	ss->writerThread       = NULL;
 	if (ssl_lock_readers) {
 	    ss->recvLock       = PZ_NewLock(nssILockSSL);
+	    if (!ss->recvLock) goto loser;
 	    ss->sendLock       = PZ_NewLock(nssILockSSL);
+	    if (!ss->sendLock) goto loser;
+	}
+	status = ssl_CreateSecurityInfo(ss);
+	if (status != SECSuccess) goto loser;
+	status = ssl_InitGather(&ss->gs);
+	if (status != SECSuccess) {
+loser:
+	    ssl_DestroySocketContents(ss);
+	    ssl_DestroyLocks(ss);
+	    PORT_Free(ss);
+	    ss = NULL;
 	}
     }
     return ss;
