@@ -41,11 +41,14 @@
 #include <string.h> 
 #include <stdio.h>
 #include <stdlib.h>
+#include "nsStrPrivate.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsDebug.h"
-#include "nsCRT.h"
-#include "nsDeque.h"
+
+#ifndef nsCharTraits_h___
+#include "nsCharTraits.h"
+#endif
 
 #ifndef RICKG_TESTBED
 #include "prdtoa.h"
@@ -66,11 +69,11 @@ nsCString::GetFlatBufferHandle() const
  * Default constructor. 
  */
 nsCString::nsCString()  {
-  Initialize(*this,eOneByte);
+  nsStrPrivate::Initialize(*this,eOneByte);
 }
 
 nsCString::nsCString(const char* aCString) {  
-  Initialize(*this,eOneByte);
+  nsStrPrivate::Initialize(*this,eOneByte);
   Assign(aCString);
 }
 
@@ -81,48 +84,25 @@ nsCString::nsCString(const char* aCString) {
  * @param   aLength tells us how many chars to copy from given CString
  */
 nsCString::nsCString(const char* aCString,PRInt32 aLength) {  
-  Initialize(*this,eOneByte);
+  nsStrPrivate::Initialize(*this,eOneByte);
   Assign(aCString,aLength);
 }
 
-#if 0
-/**
- * This constructor accepts a unicode string
- * @update  gess 1/4/99
- * @param   aString is a ptr to a unichar string
- * @param   aLength tells us how many chars to copy from given aString
- */
-nsCString::nsCString(const PRUnichar* aString,PRInt32 aLength)  {  
-  Initialize(*this,eOneByte);
-  AssignWithConversion(aString,aLength);
-}
-
-/**
- * This constructor works for all other nsSTr derivatives
- * @update  gess 1/4/99
- * @param   reference to another nsCString
- */
-nsCString::nsCString(const nsStr &aString)  {
-  Initialize(*this,eOneByte);
-  StrAssign(*this,aString,0,aString.mLength);
-}
-#endif
-
 /**
  * This is our copy constructor 
  * @update  gess 1/4/99
  * @param   reference to another nsCString
  */
 nsCString::nsCString(const nsCString& aString)  {
-  Initialize(*this,eCharSize(aString.mCharSize));
-  StrAssign(*this,aString,0,aString.mLength);
+  nsStrPrivate::Initialize(*this,aString.GetCharSize());
+  nsStrPrivate::StrAssign(*this,aString,0,aString.mLength);
 }
 
 /**
  * Destructor
  */
 nsCString::~nsCString() {
-  Destroy(*this);
+  nsStrPrivate::Destroy(*this);
 }
 
 const char* nsCString::GetReadableFragment( nsReadableFragment<char>& aFragment, nsFragmentRequest aRequest, PRUint32 aOffset ) const {
@@ -156,14 +136,14 @@ char* nsCString::GetWritableFragment( nsWritableFragment<char>& aFragment, nsFra
 }
 
 nsCString::nsCString( const nsACString& aReadable ) {
-  Initialize(*this,eOneByte);
+  nsStrPrivate::Initialize(*this,eOneByte);
   Assign(aReadable);
 }
 
 #ifdef DEBUG
 void nsCString::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const {
   if (aResult) {
-    *aResult = sizeof(*this) + mCapacity * mCharSize;
+    *aResult = sizeof(*this) + GetCapacity() * GetCharSize();
   }
 }
 #endif
@@ -176,12 +156,12 @@ void nsCString::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const {
  * @return  nada
  */
 void nsCString::SetLength(PRUint32 anIndex) {
-  if ( anIndex > mCapacity )
+  if ( anIndex > GetCapacity() )
     SetCapacity(anIndex);
-    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAWritableString.h),
+    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAString.h),
     //  we can only use it since our local implementation, |nsCString::SetCapacity|, is known to do what we want
 
-  nsStr::StrTruncate(*this,anIndex);
+  nsStrPrivate::StrTruncate(*this,anIndex);
 }
 
 
@@ -196,14 +176,14 @@ nsCString::SetCapacity( PRUint32 aNewCapacity )
   {
     if ( aNewCapacity )
       {
-        if( aNewCapacity > mCapacity )
-          GrowCapacity(*this,aNewCapacity);
+        if( aNewCapacity > GetCapacity() )
+          nsStrPrivate::GrowCapacity(*this,aNewCapacity);
         AddNullTerminator(*this);
       }
     else
       {
-        nsStr::Destroy(*this);
-        nsStr::Initialize(*this, eOneByte);
+        nsStrPrivate::Destroy(*this);
+        nsStrPrivate::Initialize(*this, eOneByte);
       }
   }
 
@@ -244,36 +224,20 @@ PRBool nsCString::SetCharAt(PRUnichar aChar,PRUint32 anIndex){
  *  @return *this 
  */
 void
-nsCString::StripChar(PRUnichar aChar,PRInt32 anOffset){
+nsCString::StripChar(char aChar,PRInt32 anOffset){
   if(mLength && (anOffset<PRInt32(mLength))) {
-    if(eOneByte==mCharSize) {
-      char*  to   = mStr + anOffset;
-      char*  from = mStr + anOffset;
-      char*  end  = mStr + mLength;
+    char*  to   = mStr + anOffset;
+    char*  from = mStr + anOffset;
+    char*  end  = mStr + mLength;
 
-      while (from < end) {
-        char theChar = *from++;
-        if(aChar!=theChar) {
-          *to++ = theChar;
-        }
+    while (from < end) {
+      char theChar = *from++;
+      if(aChar!=theChar) {
+        *to++ = theChar;
       }
-      *to = 0; //add the null
-      mLength=to - mStr;
-    }
-    else {
-      PRUnichar*  to   = mUStr + anOffset;
-      PRUnichar*  from = mUStr + anOffset;
-      PRUnichar*  end  = mUStr + mLength;
-
-      while (from < end) {
-        PRUnichar theChar = *from++;
-        if(aChar!=theChar) {
-          *to++ = theChar;
-        }
-      }
-      *to = 0; //add the null
-      mLength=to - mUStr;
     }
+    *to = 0; //add the null
+    mLength=to - mStr;
   }
 }
 
@@ -287,7 +251,7 @@ nsCString::StripChar(PRUnichar aChar,PRInt32 anOffset){
  */
 void
 nsCString::StripChars(const char* aSet){
-  nsStr::StripChars1(*this,aSet);
+  nsStrPrivate::StripChars1(*this,aSet);
 }
 
 
@@ -310,16 +274,16 @@ nsCString::StripWhitespace() {
  *  @return *this 
  */
 void
-nsCString::ReplaceChar(PRUnichar aOldChar, PRUnichar aNewChar) {
+nsCString::ReplaceChar(char aOldChar, char aNewChar) {
   PRUint32 theIndex=0;
   
-  if((aOldChar<256) && (aNewChar<256)){
-      //only execute this if oldchar and newchar are within legal ascii range
-    for(theIndex=0;theIndex<mLength;theIndex++){
-      if(mStr[theIndex]==(char)aOldChar) {
-        mStr[theIndex]=(char)aNewChar;
-      }//if
-    }
+  //only execute this if oldchar and newchar are within legal ascii range
+  for(theIndex=0; theIndex < mLength; theIndex++){
+    
+    if(mStr[theIndex] == aOldChar) {
+      mStr[theIndex] = aNewChar;
+    }//if
+    
   }
 }
 
@@ -331,14 +295,15 @@ nsCString::ReplaceChar(PRUnichar aOldChar, PRUnichar aNewChar) {
  *  @return *this 
  */
 void
-nsCString::ReplaceChar(const char* aSet, PRUnichar aNewChar){
-  if(aSet && (aNewChar<256)){
+nsCString::ReplaceChar(const char* aSet, char aNewChar){
+  if(aSet) {
       //only execute this if newchar is valid ascii, and aset isn't null.
 
     PRInt32 theIndex=FindCharInSet(aSet,0);
-    while(kNotFound<theIndex) {
-      mStr[theIndex]=(char)aNewChar;
-      theIndex=FindCharInSet(aSet,theIndex+1);
+    
+    while(kNotFound < theIndex) {
+      mStr[theIndex] = aNewChar;
+      theIndex = FindCharInSet(aSet,theIndex+1);
     }
   }
 }
@@ -354,12 +319,12 @@ void
 nsCString::ReplaceSubstring(const char* aTarget,const char* aNewValue){
   if(aTarget && aNewValue) {
 
-    PRInt32 len=strlen(aTarget);
+    PRInt32 len=nsCharTraits<char>::length(aTarget);
     if(0<len) {
       CBufDescriptor theDesc1(aTarget,PR_TRUE, len+1,len);
       nsCAutoString theTarget(theDesc1);
 
-      len=strlen(aNewValue);
+      len=nsCharTraits<char>::length(aNewValue);
       if(0<len) {
         CBufDescriptor theDesc2(aNewValue,PR_TRUE, len+1,len);
         nsCAutoString theNewValue(theDesc2);
@@ -392,19 +357,19 @@ nsCString::ReplaceSubstring(const nsCString& aTarget,const nsCString& aNewValue)
     }
     else {
       PRInt32 theIndex=0;
-      while(kNotFound!=(theIndex=nsStr::FindSubstr(*this,aTarget,PR_FALSE,theIndex,mLength))) {
+      while(kNotFound!=(theIndex=nsStrPrivate::FindSubstr1in1(*this,aTarget,PR_FALSE,theIndex,mLength))) {
         if(aNewValue.mLength<aTarget.mLength) {
           //Since target is longer than newValue, we should delete a few chars first, then overwrite.
           PRInt32 theDelLen=aTarget.mLength-aNewValue.mLength;
-          nsStr::Delete1(*this,theIndex,theDelLen);
-          nsStr::Overwrite(*this,aNewValue,theIndex);
+          nsStrPrivate::Delete1(*this,theIndex,theDelLen);
+          nsStrPrivate::Overwrite(*this,aNewValue,theIndex);
         }
         else {
           //this is the worst case: the newvalue is larger than the substr it's replacing
           //so we have to insert some characters...
           PRInt32 theInsLen=aNewValue.mLength-aTarget.mLength;
-          StrInsert1into1(*this,theIndex,aNewValue,0,theInsLen);
-          nsStr::Overwrite(*this,aNewValue,theIndex);
+          nsStrPrivate::StrInsert1into1(*this,theIndex,aNewValue,0,theInsLen);
+          nsStrPrivate::Overwrite(*this,aNewValue,theIndex);
           theIndex += aNewValue.mLength;
         }
       }
@@ -426,8 +391,8 @@ nsCString::Trim(const char* aTrimSet, PRBool aEliminateLeading,PRBool aEliminate
 
   if(aTrimSet){
     
-    PRUnichar theFirstChar=0;
-    PRUnichar theLastChar=0;
+    char theFirstChar = 0;
+    char theLastChar = 0;
     PRBool    theQuotesAreNeeded=PR_FALSE;
 
     if(aIgnoreQuotes && (mLength>2)) {
@@ -443,11 +408,11 @@ nsCString::Trim(const char* aTrimSet, PRBool aEliminateLeading,PRBool aEliminate
       }
     }
     
-    nsStr::Trim(*this,aTrimSet,aEliminateLeading,aEliminateTrailing);
+    nsStrPrivate::Trim(*this, aTrimSet, aEliminateLeading, aEliminateTrailing);
 
     if(aIgnoreQuotes && theQuotesAreNeeded) {
-      InsertWithConversion(theFirstChar,0);
-      AppendWithConversion(theLastChar);
+      Insert(theFirstChar, 0);
+      Append(theLastChar);
     }
 
   }
@@ -463,11 +428,13 @@ nsCString::Trim(const char* aTrimSet, PRBool aEliminateLeading,PRBool aEliminate
  *  @param   aEliminateTrailing controls stripping of trailing ws
  *  @return  this
  */
-void
-nsCString::CompressSet(const char* aSet, PRUnichar aChar,PRBool aEliminateLeading,PRBool aEliminateTrailing){
+static void
+CompressSet(nsCString& aStr, const char* aSet, char aChar,
+            PRBool aEliminateLeading, PRBool aEliminateTrailing){
   if(aSet){
-    ReplaceChar(aSet,aChar);
-    nsStr::CompressSet1(*this,aSet,aEliminateLeading,aEliminateTrailing);
+    aStr.ReplaceChar(aSet, aChar);
+    nsStrPrivate::CompressSet1(aStr, aSet,
+                               aEliminateLeading, aEliminateTrailing);
   }
 }
 
@@ -483,7 +450,7 @@ nsCString::CompressSet(const char* aSet, PRUnichar aChar,PRBool aEliminateLeadin
  */
 void
 nsCString::CompressWhitespace( PRBool aEliminateLeading,PRBool aEliminateTrailing){
-  CompressSet(kWhitespace,' ',aEliminateLeading,aEliminateTrailing);
+  CompressSet(*this, kWhitespace,' ',aEliminateLeading,aEliminateTrailing);
 }
 
 /**********************************************************************
@@ -568,11 +535,13 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
 
         //now iterate the numeric chars and build our result
       char* first=--cp;  //in case we have to back up.
+      PRBool haveValue = PR_FALSE;
 
       while(cp<endcp){
         theChar=*cp++;
         if(('0'<=theChar) && (theChar<='9')){
           result = (theRadix * result) + (theChar-'0');
+          haveValue = PR_TRUE;
         }
         else if((theChar>='A') && (theChar<='F')) {
           if(10==theRadix) {
@@ -580,6 +549,7 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
               theRadix=16;
               cp=first; //backup
               result=0;
+              haveValue = PR_FALSE;
             }
             else {
               *anErrorCode=NS_ERROR_ILLEGAL_VALUE;
@@ -589,6 +559,7 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
           }
           else {
             result = (theRadix * result) + ((theChar-'A')+10);
+            haveValue = PR_TRUE;
           }
         }
         else if((theChar>='a') && (theChar<='f')) {
@@ -597,6 +568,7 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
               theRadix=16;
               cp=first; //backup
               result=0;
+              haveValue = PR_FALSE;
             }
             else {
               *anErrorCode=NS_ERROR_ILLEGAL_VALUE;
@@ -606,9 +578,13 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
           }
           else {
             result = (theRadix * result) + ((theChar-'a')+10);
+            haveValue = PR_TRUE;
           }
         }
-        else if(('X'==theChar) || ('x'==theChar) || ('#'==theChar) || ('+'==theChar)) {
+        else if((('X'==theChar) || ('x'==theChar)) && (!haveValue || result == 0)) {
+          continue;
+        }
+        else if((('#'==theChar) || ('+'==theChar)) && !haveValue) {
           continue;
         }
         else {
@@ -638,11 +614,11 @@ PRInt32 nsCString::ToInteger(PRInt32* anErrorCode,PRUint32 aRadix) const {
  * @return  this
  */
 void nsCString::AssignWithConversion(const PRUnichar* aString,PRInt32 aCount) {
-  nsStr::StrTruncate(*this,0);
+  nsStrPrivate::StrTruncate(*this,0);
 
   if(aString && aCount){
     nsStr temp;
-    nsStr::Initialize(temp,eTwoByte);
+    nsStrPrivate::Initialize(temp,eTwoByte);
     temp.mUStr=(PRUnichar*)aString;
 
     if(0<aCount) {
@@ -651,17 +627,17 @@ void nsCString::AssignWithConversion(const PRUnichar* aString,PRInt32 aCount) {
       // If this assertion fires, the caller is probably lying about the length of
       //   the passed-in string.  File a bug on the caller.
 #ifdef NS_DEBUG
-      PRInt32 len=nsStr::FindChar(temp,0,PR_FALSE,0,temp.mLength);
+      PRInt32 len=nsStrPrivate::FindChar2(temp,0,PR_FALSE,temp.mLength);
       if(kNotFound<len) {
         NS_WARNING(kPossibleNull);
       }
 #endif
 
     }
-    else aCount=temp.mLength=nsCRT::strlen(aString);
+    else aCount=temp.mLength=nsCharTraits<PRUnichar>::length(aString);
 
     if(0<aCount)
-      StrAppend(*this,temp,0,aCount);
+      nsStrPrivate::StrAppend(*this,temp,0,aCount);
   }
 }
 
@@ -670,7 +646,7 @@ void nsCString::AssignWithConversion( const nsString& aString ) {
 }
 
 void nsCString::AssignWithConversion( const nsAString& aString ) {
-  nsStr::StrTruncate(*this,0);
+  nsStrPrivate::StrTruncate(*this,0);
   PRInt32 count = aString.Length();
 
   if(count){   
@@ -681,12 +657,12 @@ void nsCString::AssignWithConversion( const nsAString& aString ) {
       PRUint32 fraglen = start.size_forward();
 
       nsStr temp;
-      nsStr::Initialize(temp,eTwoByte);
+      nsStrPrivate::Initialize(temp,eTwoByte);
       temp.mUStr=(PRUnichar*)start.get();
 
       temp.mLength=fraglen;
 
-      StrAppend(*this,temp,0,fraglen);
+      nsStrPrivate::StrAppend(*this,temp,0,fraglen);
 
       start.advance(fraglen);
     }
@@ -704,51 +680,22 @@ void nsCString::AppendWithConversion( const nsAString& aString ) {
       PRUint32 fraglen = start.size_forward();
 
       nsStr temp;
-      nsStr::Initialize(temp,eTwoByte);
+      nsStrPrivate::Initialize(temp,eTwoByte);
       temp.mUStr=(PRUnichar*)start.get();
 
       temp.mLength=fraglen;
 
-      StrAppend(*this,temp,0,fraglen);
+      nsStrPrivate::StrAppend(*this,temp,0,fraglen);
       
       start.advance(fraglen);
     }
   }
 }
 
-/**
- * assign given unichar to this string
- * @update  gess 01/04/99
- * @param   aChar: char to be assignd to this
- * @return  this
- */
-void nsCString::AssignWithConversion(PRUnichar aChar) {
-  nsStr::StrTruncate(*this,0);
-  AppendWithConversion(aChar);
-}
-
-
-/**
- * append given char to this string
- * @update  gess 01/04/99
- * @param   aChar: char to be appended to this
- * @return  this
- */
-void nsCString::AppendWithConversion(PRUnichar aChar) {
-  PRUnichar buf[2]={0,0};
-  buf[0]=aChar;
-
-  nsStr temp;
-  nsStr::Initialize(temp,eTwoByte);
-  temp.mUStr=buf;
-  temp.mLength=1;
-  StrAppend(*this,temp,0,1);
-}
-
 void nsCString::AppendWithConversion( const PRUnichar* aBuffer, PRInt32 aLength )
   {
     nsStr temp;
-    nsStr::Initialize(temp, eTwoByte);
+    nsStrPrivate::Initialize(temp, eTwoByte);
     temp.mUStr = NS_CONST_CAST(PRUnichar*, aBuffer);
 
     if ( aLength < 0 )
@@ -757,7 +704,7 @@ void nsCString::AppendWithConversion( const PRUnichar* aBuffer, PRInt32 aLength
     if ( aLength > 0 )
       {
         temp.mLength = aLength;
-        StrAppend(*this, temp, 0, aLength);
+        nsStrPrivate::StrAppend(*this, temp, 0, aLength);
       }
   }
 
@@ -827,39 +774,32 @@ void nsCString::AppendWithConversion(const nsString& aString,PRInt32 aCount) {
   else aCount=MinInt(aCount,aString.mLength);
 
   if(0<aCount)
-    StrAppend(*this,aString,0,aCount);
+    nsStrPrivate::StrAppend(*this,aString,0,aCount);
 }
 
-
-/**
- * Insert a single unichar into this string at a specified offset.
- *
- * @update  gess4/22/98
- * @param   aChar unichar to be inserted into this string
- * @param   anOffset is insert pos in str 
- * @return  this
- */
-void nsCString::InsertWithConversion(PRUnichar aChar,PRUint32 anOffset){
-  PRUnichar theBuffer[2]={0,0};
-  theBuffer[0]=aChar;
-  nsStr temp;
-  nsStr::Initialize(temp,eTwoByte);
-  temp.mUStr=theBuffer;
-  temp.mLength=1;
-  StrInsert2into1(*this,anOffset,temp,0,1);
-}
-
-
 void nsCString::Adopt(char* aPtr, PRInt32 aLength) {
   NS_ASSERTION(aPtr, "Can't adopt |0|");
-  nsStr::Destroy(*this);
+  nsStrPrivate::Destroy(*this);
   if (aLength == -1)
     aLength = nsCharTraits<char>::length(aPtr);
   // We don't know the capacity, so we'll just have to assume
   // capacity = length.
-  nsStr::Initialize(*this, aPtr, aLength, aLength, eOneByte, PR_TRUE);
+  nsStrPrivate::Initialize(*this, aPtr, aLength, aLength, eOneByte, PR_TRUE);
 }
 
+nsCString::size_type
+nsCString::Mid( self_type& aResult, index_type aStartPos, size_type aLengthToCopy ) const
+  {
+      // If we're just assigning our entire self, give |aResult| the opportunity to share
+    if ( aStartPos == 0 && aLengthToCopy >= Length() )
+      aResult = *this;
+    else
+      aResult = Substring(*this, aStartPos, aLengthToCopy);
+
+    return aResult.Length();
+  }
+
+
 /**********************************************************************
   Searching methods...                
  *********************************************************************/
@@ -879,37 +819,14 @@ PRInt32 nsCString::Find(const char* aCString,PRBool aIgnoreCase,PRInt32 anOffset
   PRInt32 result=kNotFound;
   if(aCString) {
     nsStr temp;
-    nsStr::Initialize(temp,eOneByte);
-    temp.mLength = nsCRT::strlen(aCString);
+    nsStrPrivate::Initialize(temp,eOneByte);
+    temp.mLength = nsCharTraits<char>::length(aCString);
     temp.mStr=(char*)aCString;
-    result=nsStr::FindSubstr(*this,temp,aIgnoreCase,anOffset,aCount);
+    result=nsStrPrivate::FindSubstr1in1(*this,temp,aIgnoreCase,anOffset,aCount);
   }
   return result;
 }
 
-/** 
- *  Search for given buffer within this string
- *  
- *  @update  gess 3/25/98
- *  @param   aString - substr to be found
- *  @param   aIgnoreCase tells us whether or not to do caseless compare
- *  @param   anOffset tells us where in this string to start searching
- *  @param   aCount tells us how many iterations to make starting at the given offset
- *  @return  offset in string, or -1 (kNotFound)
- */
-PRInt32 nsCString::Find(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  NS_ASSERTION(0!=aString,kNullPointerError);
-
-  PRInt32 result=kNotFound;
-  if(aString) {
-    nsStr temp;
-    nsStr::Initialize(temp,eTwoByte);
-    temp.mLength = nsCRT::strlen(aString);
-    temp.mUStr=(PRUnichar*)aString;
-    result=nsStr::FindSubstr(*this,temp,aIgnoreCase,anOffset,aCount);
-  }
-  return result;
-}
 
 /**
  *  Search for given string within this string
@@ -921,12 +838,11 @@ PRInt32 nsCString::Find(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 anOf
  *  @param   aCount tells us how many iterations to make starting at the given offset
  *  @return  offset in string, or -1 (kNotFound)
  */
-PRInt32 nsCString::Find(const nsStr& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStr::FindSubstr(*this,aString,aIgnoreCase,anOffset,aCount);
+PRInt32 nsCString::Find(const nsCString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
+  PRInt32 result=nsStrPrivate::FindSubstr1in1(*this,aString,aIgnoreCase,anOffset,aCount);
   return result;
 }
 
-
 /**
  *  This searches this string for a given character 
  *  
@@ -937,8 +853,8 @@ PRInt32 nsCString::Find(const nsStr& aString,PRBool aIgnoreCase,PRInt32 anOffset
  *  @param   aCount tell us how many chars to search from offset; -1 means use full length.
  *  @return  index in aDest where member of aSet occurs, or -1 if not found
  */
-PRInt32 nsCString::FindChar(PRUnichar aChar,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStr::FindChar(*this,aChar,aIgnoreCase,anOffset,aCount);
+PRInt32 nsCString::FindChar(PRUnichar aChar,PRInt32 anOffset,PRInt32 aCount) const{
+  PRInt32 result=nsStrPrivate::FindChar1(*this,aChar,anOffset,aCount);
   return result;
 }
 
@@ -957,48 +873,16 @@ PRInt32 nsCString::FindCharInSet(const char* aCStringSet,PRInt32 anOffset) const
   PRInt32 result=kNotFound;
   if(aCStringSet) {
     nsStr temp;
-    nsStr::Initialize(temp,eOneByte);
-    temp.mLength=nsCRT::strlen(aCStringSet);
+    nsStrPrivate::Initialize(temp,eOneByte);
+    temp.mLength=nsCharTraits<char>::length(aCStringSet);
     temp.mStr=(char*)aCStringSet;
-    result=nsStr::FindCharInSet(*this,temp,PR_FALSE,anOffset);
+    result=nsStrPrivate::FindCharInSet1(*this,temp,PR_FALSE,anOffset);
   }
   return result;
 }
 
-/**
- *  This method finds the offset of the first char in this string that is
- *  a member of the given charset, starting the search at anOffset
- *  
- *  @update  gess 3/25/98
- *  @param   aStringSet
- *  @param   anOffset -- where in this string to start searching
- *  @return  
- */
-PRInt32 nsCString::FindCharInSet(const PRUnichar* aStringSet,PRInt32 anOffset) const{
-  NS_ASSERTION(0!=aStringSet,kNullPointerError);
-
-  PRInt32 result=kNotFound;
-  if(aStringSet) {
-    nsStr temp;
-    nsStr::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aStringSet);
-    temp.mStr=(char*)aStringSet;
-    result=nsStr::FindCharInSet(*this,temp,PR_FALSE,anOffset);
-  }
-  return result;
-}
-
-/**
- *  This method finds the offset of the first char in this string that is
- *  a member of the given charset, starting the search at anOffset
- *  
- *  @update  gess 3/25/98
- *  @param   aCStringSet
- *  @param   anOffset -- where in this string to start searching
- *  @return  
- */
-PRInt32 nsCString::FindCharInSet(const nsStr& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStr::FindCharInSet(*this,aSet,PR_FALSE,anOffset);
+PRInt32 nsCString::FindCharInSet(const nsCString& aSet,PRInt32 anOffset) const{
+  PRInt32 result=nsStrPrivate::FindCharInSet1(*this,aSet,PR_FALSE,anOffset);
   return result;
 }
 
@@ -1012,8 +896,8 @@ PRInt32 nsCString::FindCharInSet(const nsStr& aSet,PRInt32 anOffset) const{
  *  @param   aCount tells us how many iterations to make starting at the given offset
  *  @return  offset in string, or -1 (kNotFound)
  */
-PRInt32 nsCString::RFind(const nsStr& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStr::RFindSubstr(*this,aString,aIgnoreCase,anOffset,aCount);
+PRInt32 nsCString::RFind(const nsCString& aString,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
+  PRInt32 result=nsStrPrivate::RFindSubstr1in1(*this,aString,aIgnoreCase,anOffset,aCount);
   return result;
 }
 
@@ -1034,10 +918,10 @@ PRInt32 nsCString::RFind(const char* aString,PRBool aIgnoreCase,PRInt32 anOffset
   PRInt32 result=kNotFound;
   if(aString) {
     nsStr temp;
-    nsStr::Initialize(temp,eOneByte);
-    temp.mLength=nsCRT::strlen(aString);
+    nsStrPrivate::Initialize(temp,eOneByte);
+    temp.mLength=nsCharTraits<char>::length(aString);
     temp.mStr=(char*)aString;
-    result=nsStr::RFindSubstr(*this,temp,aIgnoreCase,anOffset,aCount);
+    result=nsStrPrivate::RFindSubstr1in1(*this,temp,aIgnoreCase,anOffset,aCount);
   }
   return result;
 }
@@ -1052,8 +936,8 @@ PRInt32 nsCString::RFind(const char* aString,PRBool aIgnoreCase,PRInt32 anOffset
  *  @param   aCount tell us how many chars to search from offset; -1 means use full length.
  *  @return  index in aDest where member of aSet occurs, or -1 if not found
  */
-PRInt32 nsCString::RFindChar(PRUnichar aChar,PRBool aIgnoreCase,PRInt32 anOffset,PRInt32 aCount) const{
-  PRInt32 result=nsStr::RFindChar1(*this,aChar,aIgnoreCase,anOffset,aCount);
+PRInt32 nsCString::RFindChar(PRUnichar aChar,PRInt32 anOffset,PRInt32 aCount) const{
+  PRInt32 result=nsStrPrivate::RFindChar1(*this,aChar,anOffset,aCount);
   return result;
 }
 
@@ -1071,94 +955,40 @@ PRInt32 nsCString::RFindCharInSet(const char* aCStringSet,PRInt32 anOffset) cons
   PRInt32 result=kNotFound;
   if(aCStringSet) {
     nsStr temp;
-    nsStr::Initialize(temp,eOneByte);
-    temp.mLength=nsCRT::strlen(aCStringSet);
+    nsStrPrivate::Initialize(temp,eOneByte);
+    temp.mLength=nsCharTraits<char>::length(aCStringSet);
     temp.mStr=(char*)aCStringSet;
-    result=nsStr::RFindCharInSet(*this,temp,PR_FALSE,anOffset);
+    result=nsStrPrivate::RFindCharInSet1(*this,temp,PR_FALSE,anOffset);
   }
   return result;
 }
 
-/**
- *  Reverse search for char in this string that is also a member of given charset
- *  
- *  @update  gess 3/25/98
- *  @param   aStringSet
- *  @param   anOffset
- *  @return  offset of found char, or -1 (kNotFound)
- */
-PRInt32 nsCString::RFindCharInSet(const PRUnichar* aStringSet,PRInt32 anOffset) const{
-  NS_ASSERTION(0!=aStringSet,kNullPointerError);
-
-  PRInt32 result=kNotFound;
-  if(aStringSet) {
-    nsStr temp;
-    nsStr::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aStringSet);
-    temp.mUStr=(PRUnichar*)aStringSet;
-    result=nsStr::RFindCharInSet(*this,temp,PR_FALSE,anOffset);
-  }
+PRInt32 nsCString::RFindCharInSet(const nsCString& aSet,PRInt32 anOffset) const{
+  PRInt32 result=nsStrPrivate::RFindCharInSet1(*this,aSet,PR_FALSE,anOffset);
   return result;
 }
 
-
-/**
- *  Reverse search for char in this string that is also a member of given charset
- *  
- *  @update  gess 3/25/98
- *  @param   aCStringSet
- *  @param   anOffset
- *  @return  offset of found char, or -1 (kNotFound)
- */
-PRInt32 nsCString::RFindCharInSet(const nsStr& aSet,PRInt32 anOffset) const{
-  PRInt32 result=nsStr::RFindCharInSet(*this,aSet,PR_FALSE,anOffset);
-  return result;
-}
-
-
 /**************************************************************
   COMPARISON METHODS...
  **************************************************************/
 
-
 /**
- * Compares given unistring to this string. 
- * @update  gess 01/04/99
- * @param   aString pts to a uni-string
- * @param   aIgnoreCase tells us how to treat case
- * @param   aCount tells us how many chars to test; -1 implies full length
- * @return  -1,0,1
- */
-PRInt32 nsCString::CompareWithConversion(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  NS_ASSERTION(0!=aString,kNullPointerError);
-
-  if(aString) {
-    nsStr temp;
-    nsStr::Initialize(temp,eTwoByte);
-    temp.mLength=nsCRT::strlen(aString);
-    temp.mUStr=(PRUnichar*)aString;
-    return nsStr::StrCompare1To2(*this,temp,aCount,aIgnoreCase);
-  }
-  return 0;
-}
-
-/**
- * Compares given cstring to this string. 
+ * Compares given cstring to this string.
  * @update  gess 01/04/99
  * @param   aCString points to a cstring
  * @param   aIgnoreCase tells us how to treat case
  * @param   aCount tells us how many chars to test; -1 implies full length
  * @return  -1,0,1
  */
-PRInt32 nsCString::CompareWithConversion(const char *aCString,PRBool aIgnoreCase,PRInt32 aCount) const {
+PRInt32 nsCString::Compare(const char *aCString,PRBool aIgnoreCase,PRInt32 aCount) const {
   NS_ASSERTION(0!=aCString,kNullPointerError);
 
   if(aCString) {
     nsStr temp;
-    nsStr::Initialize(temp,eOneByte);
-    temp.mLength=nsCRT::strlen(aCString);
+    nsStrPrivate::Initialize(temp,eOneByte);
+    temp.mLength=nsCharTraits<char>::length(aCString);
     temp.mStr=(char*)aCString;
-    return nsStr::StrCompare1To1(*this,temp,aCount,aIgnoreCase);
+    return nsStrPrivate::StrCompare1To1(*this,temp,aCount,aIgnoreCase);
   }
   return 0;
 }
@@ -1167,12 +997,6 @@ PRBool nsCString::EqualsIgnoreCase(const char* aString,PRInt32 aLength) const {
   return EqualsWithConversion(aString,PR_TRUE,aLength);
 }
 
-PRBool nsCString::EqualsIgnoreCase(const PRUnichar* aString,PRInt32 aLength) const {
-  return EqualsWithConversion(aString,PR_TRUE,aLength);
-}
-
-
-
 /**
  * Compare this to given string; note that we compare full strings here.
  *
@@ -1183,19 +1007,7 @@ PRBool nsCString::EqualsIgnoreCase(const PRUnichar* aString,PRInt32 aLength) con
  * @return TRUE if equal
  */
 PRBool nsCString::EqualsWithConversion(const char* aCString,PRBool aIgnoreCase,PRInt32 aCount) const{
-  PRInt32 theAnswer=CompareWithConversion(aCString,aIgnoreCase,aCount);
-  PRBool  result=PRBool(0==theAnswer);  
-  return result;
-}
-
-PRBool nsCString::EqualsWithConversion(const PRUnichar* aString,PRBool aIgnoreCase,PRInt32 aCount) const {
-  PRInt32 theAnswer=CompareWithConversion(aString,aIgnoreCase,aCount);
-  PRBool  result=PRBool(0==theAnswer);  
-  return result;
-}
-
- PRBool nsCString::EqualsWithConversion(const nsString& aString, PRBool aIgnoreCase,PRInt32 aCount) const {
-  PRInt32 theAnswer=CompareWithConversion(aString.get(),aIgnoreCase,aCount);
+  PRInt32 theAnswer=Compare(aCString,aIgnoreCase,aCount);
   PRBool  result=PRBool(0==theAnswer);  
   return result;
 }
@@ -1241,7 +1053,7 @@ NS_ConvertUCS2toUTF8::Append( const PRUnichar* aString, PRUint32 aLength )
     // multiple allocations.
     if(mLength+PRUint32(utf8len+1) > sizeof(mBuffer))
       SetCapacity(mLength+utf8len+1);
-    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAWritableString.h),
+    // |SetCapacity| normally doesn't guarantee the use we are putting it to here (see its interface comment in nsAString.h),
     //  we can only use it since our local implementation, |nsCString::SetCapacity|, is known to do what we want
 
     char* out = mStr+mLength;
@@ -1334,25 +1146,25 @@ NS_LossyConvertUCS2toASCII::NS_LossyConvertUCS2toASCII( const nsAString& aString
  *
  */
 nsCAutoString::nsCAutoString() : nsCString(){
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+  nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   AddNullTerminator(*this);
 
 }
 
 nsCAutoString::nsCAutoString( const nsCString& aString ) : nsCString(){
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+  nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   AddNullTerminator(*this);
   Append(aString);
 }
 
 nsCAutoString::nsCAutoString( const nsACString& aString ) : nsCString(){
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+  nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   AddNullTerminator(*this);
   Append(aString);
 }
 
 nsCAutoString::nsCAutoString(const char* aCString) : nsCString() {
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+  nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   AddNullTerminator(*this);
   Append(aCString);
 }
@@ -1362,7 +1174,7 @@ nsCAutoString::nsCAutoString(const char* aCString) : nsCString() {
  * @param   aCString is a ptr to a 1-byte cstr
  */
 nsCAutoString::nsCAutoString(const char* aCString,PRInt32 aLength) : nsCString() {
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+  nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   AddNullTerminator(*this);
   Append(aCString,aLength);
 }
@@ -1373,49 +1185,15 @@ nsCAutoString::nsCAutoString(const char* aCString,PRInt32 aLength) : nsCString()
  */
 nsCAutoString::nsCAutoString(const CBufDescriptor& aBuffer) : nsCString() {
   if(!aBuffer.mBuffer) {
-    Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
+    nsStrPrivate::Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
   }
   else {
-    Initialize(*this,aBuffer.mBuffer,aBuffer.mCapacity,aBuffer.mLength,aBuffer.mCharSize,!aBuffer.mStackBased);
+    nsStrPrivate::Initialize(*this,aBuffer.mBuffer,aBuffer.mCapacity,aBuffer.mLength,aBuffer.mCharSize,!aBuffer.mStackBased);
   }
   if(!aBuffer.mIsConst)
     AddNullTerminator(*this); //this isn't really needed, but it guarantees that folks don't pass string constants.
 }
 
-#if 0
-/**
- * Copy construct from uni-string
- * @param   aString is a ptr to a unistr
- */
-nsCAutoString::nsCAutoString(const PRUnichar* aString,PRInt32 aLength) : nsCString() {
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
-  AddNullTerminator(*this);
-  Append(aString,aLength);
-}
-
-/**
- * construct from an nsStr
- * @param   
- */
-nsCAutoString::nsCAutoString(const nsStr& aString) : nsCString() {
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
-  AddNullTerminator(*this);
-  Append(aString);
-}
-
-
-
-/**
- * construct from a char
- * @param   
- */
-nsCAutoString::nsCAutoString(PRUnichar aChar) : nsCString(){
-  Initialize(*this,mBuffer,sizeof(mBuffer)-1,0,eOneByte,PR_FALSE);
-  AddNullTerminator(*this);
-  Append(aChar);
-}
-#endif
-
 
 /**
  * deconstructor
@@ -1427,7 +1205,7 @@ nsCAutoString::~nsCAutoString(){
 #ifdef DEBUG
 void nsCAutoString::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const {
   if (aResult) {
-    *aResult = sizeof(*this) + mCapacity * mCharSize;
+    *aResult = sizeof(*this) + GetCapacity() * GetCharSize();
   }
 }
 #endif
