@@ -37,40 +37,44 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-/* nsString2.h --- rickg's original strings of 2-byte chars, |nsString| and |nsAutoString|;
-    these classes will be replaced by the new shared-buffer string (see bug #53065)
+/*
+ * nsString2.h --- rickg's original strings of 2-byte chars, |nsString|
+ * and |nsAutoString|; these classes will be replaced by the new
+ * shared-buffer string (see bug #53065)
  */
 
-
-
-#ifndef _nsString_
-#define _nsString_
-
-/***********************************************************************
-  MODULE NOTES:
-
-  See nsStr.h for a more general description of string classes.
-
-  This version of the nsString class offers many improvements over the
-  original version:
-    1. Wide and narrow chars
-    2. Allocators
-    3. Much smarter autostrings
-    4. Subsumable strings
- ***********************************************************************/
+#ifndef nsString2_h__
+#define nsString2_h__
 
 #include "prtypes.h"
 #include "nscore.h"
 #include <stdio.h>
-#include "nsString.h"
-#include "nsIAtom.h"
-#include "nsStr.h"
-#include "nsCRT.h"
+
+#ifndef nsAString_h__
+#include "nsAString.h"
+#endif
 
 #ifndef nsAFlatString_h___
 #include "nsAFlatString.h"
 #endif
+
+#ifndef nsLiteralString_h__
+#include "nsLiteralString.h"
+#endif
+
+#ifndef nsDependentSubstring_h__
+#include "nsDependentSubstring.h"
+#endif
+
+#ifndef nsPromiseFlatString_h__
+#include "nsPromiseFlatString.h"
+#endif
+
+#ifndef nsXPIDLString_h__
 #include "nsXPIDLString.h"
+#endif
+
+#include "nsStr.h"
 
 class UTF8traits
   {
@@ -89,6 +93,7 @@ class UTF8traits
 #endif
 
 class nsISizeOfHandler;
+class nsCString;
 
 
 class NS_COM nsString :
@@ -180,9 +185,6 @@ public:
    */
   void StripChars( const char* aSet );
   void StripChar( PRUnichar aChar, PRInt32 anOffset=0 );
-  void StripChar( char aChar, PRInt32 anOffset=0 )       { StripChar((PRUnichar) (unsigned char)aChar,anOffset); }
-  void StripChar( PRInt32 anInt, PRInt32 anOffset=0 )    { StripChar((PRUnichar)anInt,anOffset); }
-
   /**
    *  This method strips whitespace throughout the string
    *  
@@ -214,17 +216,6 @@ public:
    */
   void Trim(const char* aSet,PRBool aEliminateLeading=PR_TRUE,PRBool aEliminateTrailing=PR_TRUE,PRBool aIgnoreQuotes=PR_FALSE);
 
-  /**
-   *  This method strips whitespace from string.
-   *  You can control whether whitespace is yanked from
-   *  start and end of string as well.
-   *  
-   *  @param   aEliminateLeading controls stripping of leading ws
-   *  @param   aEliminateTrailing controls stripping of trailing ws
-   *  @return  this
-   */
-  void CompressSet(const char* aSet, PRUnichar aChar,PRBool aEliminateLeading=PR_TRUE,PRBool aEliminateTrailing=PR_TRUE);
-
   /**
    *  This method strips whitespace from string.
    *  You can control whether whitespace is yanked from
@@ -284,7 +275,6 @@ public:
   nsString& operator=( const PRUnichar* aPtr )                                { Assign(aPtr); return *this; }
   nsString& operator=( PRUnichar aChar )                                      { Assign(aChar); return *this; }
 
-  void AssignWithConversion(char);
   void AssignWithConversion(const char*);
   void AssignWithConversion(const char*, PRInt32);
 
@@ -300,7 +290,6 @@ public:
   void AppendInt(PRInt32, PRInt32=10); //radix=8,10 or 16
   void AppendFloat(double);
   void AppendWithConversion(const char*, PRInt32=-1);
-  void AppendWithConversion(char);
 
   virtual void do_AppendFromElement( PRUnichar );
 
@@ -311,6 +300,27 @@ public:
   // Takes ownership of aPtr, sets the current length to aLength if specified.
   void Adopt( PRUnichar* aPtr, PRInt32 aLength = -1 );
 
+  /*
+    |Left|, |Mid|, and |Right| are annoying signatures that seem better almost
+    any _other_ way than they are now.  Consider these alternatives
+    
+    aWritable = aReadable.Left(17);   // ...a member function that returns a |Substring|
+    aWritable = Left(aReadable, 17);  // ...a global function that returns a |Substring|
+    Left(aReadable, 17, aWritable);   // ...a global function that does the assignment
+    
+    as opposed to the current signature
+
+    aReadable.Left(aWritable, 17);    // ...a member function that does the assignment
+
+    or maybe just stamping them out in favor of |Substring|, they are just duplicate functionality
+
+    aWritable = Substring(aReadable, 0, 17);
+  */
+            
+  size_type  Left( self_type&, size_type ) const;
+  size_type  Mid( self_type&, PRUint32, PRUint32 ) const;
+  size_type  Right( self_type&, size_type ) const;
+
   /**********************************************************************
     Searching methods...                
    *********************************************************************/
@@ -324,23 +334,12 @@ public:
    *  @param   aCount tells us how many iterations to make starting at the given offset
    *  @return  offset in string, or -1 (kNotFound)
    */
-  PRInt32 Find(const nsString& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
-  PRInt32 Find(const nsStr& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
+  PRInt32 Find(const nsCString& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
   PRInt32 Find(const char* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
-  PRInt32 Find(const PRUnichar* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
 
-
-  /**
-   *  Search for given char within this string
-   *  
-   *  @param   aString is substring to be sought in this
-   *  @param   anOffset tells us where in this strig to start searching
-   *  @param   aIgnoreCase selects case sensitivity
-   *  @param   aCount tells us how many iterations to make starting at the given offset
-   *  @return  find pos in string, or -1 (kNotFound)
-   */
-  //PRInt32 Find(PRUnichar aChar,PRInt32 offset=-1,PRBool aIgnoreCase=PR_FALSE) const;
-  PRInt32 FindChar(PRUnichar aChar,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=0,PRInt32 aCount=-1) const;
+  PRInt32 Find(const nsAFlatString& aString, PRInt32 anOffset=0, PRInt32 aCount=-1) const;
+  
+  PRInt32 Find(const PRUnichar* aString, PRInt32 anOffset=0, PRInt32 aCount=-1) const;
 
   /**
    * This method searches this string for the first character
@@ -351,7 +350,8 @@ public:
    */
   PRInt32 FindCharInSet(const char* aString,PRInt32 anOffset=0) const;
   PRInt32 FindCharInSet(const PRUnichar* aString,PRInt32 anOffset=0) const;
-  PRInt32 FindCharInSet(const nsStr& aString,PRInt32 anOffset=0) const;
+  PRInt32 FindCharInSet(const nsString& aString,PRInt32 anOffset=0) const;
+  PRInt32 FindCharInSet(const nsCString& aString,PRInt32 anOffset=0) const;
 
 
   /**
@@ -363,9 +363,9 @@ public:
    * @return  offset in string, or -1 (kNotFound)
    */
   PRInt32 RFind(const char* aCString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
-  PRInt32 RFind(const nsString& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
-  PRInt32 RFind(const nsStr& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
-  PRInt32 RFind(const PRUnichar* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
+
+  PRInt32 RFind(const nsAFlatString& aString, PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
+  PRInt32 RFind(const PRUnichar* aString,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
 
 
   /**
@@ -378,7 +378,7 @@ public:
    *  @return  find pos in string, or -1 (kNotFound)
    */
   //PRInt32 RFind(PRUnichar aChar,PRInt32 offset=-1,PRBool aIgnoreCase=PR_FALSE) const;
-  PRInt32 RFindChar(PRUnichar aChar,PRBool aIgnoreCase=PR_FALSE,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
+  PRInt32 RFindChar(PRUnichar aChar,PRInt32 anOffset=-1,PRInt32 aCount=-1) const;
 
   /**
    * This method searches this string for the last character
@@ -387,9 +387,7 @@ public:
    *  @param   anOffset tells us where in this strig to start searching (counting from left)
    * @return -1 if not found, else the offset in this
    */
-  PRInt32 RFindCharInSet(const char* aString,PRInt32 anOffset=-1) const;
   PRInt32 RFindCharInSet(const PRUnichar* aString,PRInt32 anOffset=-1) const;
-  PRInt32 RFindCharInSet(const nsStr& aString,PRInt32 anOffset=-1) const;
 
 
   /**********************************************************************
@@ -405,35 +403,11 @@ public:
    * @return  -1,0,1
    */
 
-  PRInt32 CompareWithConversion(const char* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
-  PRInt32 CompareWithConversion(const nsString& aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
-  PRInt32 CompareWithConversion(const PRUnichar* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
-  /* a hack to make sure things that used to compile continue to compile
-     even on compilers that don't have proper |explicit| support */
-  inline PRInt32
-  CompareWithConversion(const nsXPIDLString& aString, PRBool aIgnoreCase=PR_FALSE, PRInt32 aCount=-1) const
-    {
-      return CompareWithConversion(aString.get(), aIgnoreCase, aCount);
-    }
-
-  PRBool  EqualsWithConversion(const nsString &aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
+  PRInt32 CompareWithConversion(const char* aString, PRBool aIgnoreCase=PR_FALSE, PRInt32 aCount=-1) const;
+  
   PRBool  EqualsWithConversion(const char* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
-  PRBool  EqualsWithConversion(const PRUnichar* aString,PRBool aIgnoreCase=PR_FALSE,PRInt32 aCount=-1) const;
-  /* a hack to make sure things that used to compile continue to compile
-     even on compilers that don't have proper |explicit| support */
-  inline PRBool
-  EqualsWithConversion(const nsXPIDLString &aString, PRBool aIgnoreCase=PR_FALSE, PRInt32 aCount=-1) const
-    {
-      return EqualsWithConversion(aString.get(), aIgnoreCase, aCount);
-    }
 
-
-  PRBool  EqualsAtom(/*FIX: const */nsIAtom* anAtom,PRBool aIgnoreCase) const;   
-
-  PRBool  EqualsIgnoreCase(const nsString& aString) const;
   PRBool  EqualsIgnoreCase(const char* aString,PRInt32 aCount=-1) const;
-  PRBool  EqualsIgnoreCase(/*FIX: const */nsIAtom *aAtom) const;
-
 
   /**
    *  Determine if given buffer is plain ascii
@@ -451,22 +425,6 @@ public:
    */
   static  PRBool IsSpace(PRUnichar ch);
 
-  /**
-   *  Determine if given char in valid alpha range
-   *  
-   *  @param   aChar is character to be tested
-   *  @return  TRUE if in alpha range
-   */
-  static  PRBool IsAlpha(PRUnichar ch);
-
-  /**
-   *  Determine if given char is valid digit
-   *  
-   *  @param   aChar is character to be tested
-   *  @return  TRUE if char is a valid digit
-   */
-  static  PRBool IsDigit(PRUnichar ch);
-
 #ifdef DEBUG
   /**
    * Retrieve the size of this string
@@ -479,13 +437,27 @@ private:
     // NOT TO BE IMPLEMENTED
     //  these signatures help clients not accidentally call the wrong thing helped by C++ automatic integral promotion
   void operator=( char );
-  void AssignWithConversion( PRUnichar );
   void AssignWithConversion( const PRUnichar*, PRInt32=-1 );
-  void AppendWithConversion( PRUnichar );
   void AppendWithConversion( const PRUnichar*, PRInt32=-1 );
   void InsertWithConversion( const PRUnichar*, PRUint32, PRInt32=-1 );
 };
 
+inline
+nsString::size_type
+nsString::Left( nsAString& aResult, size_type aLengthToCopy ) const
+  {
+    return Mid(aResult, 0, aLengthToCopy);
+  }
+
+inline
+nsString::size_type
+nsString::Right( self_type& aResult, size_type aLengthToCopy ) const
+  {
+    size_type myLength = Length();
+    aLengthToCopy = NS_MIN(myLength, aLengthToCopy);
+    return Mid(aResult, myLength-aLengthToCopy, aLengthToCopy);
+  }
+
 // NS_DEF_STRING_COMPARISON_OPERATORS(nsString, PRUnichar)
 // NS_DEF_DERIVED_STRING_OPERATOR_PLUS(nsString, PRUnichar)
 
@@ -590,7 +562,7 @@ class NS_COM NS_ConvertUTF8toUCS2
 
       NS_ConvertUTF8toUCS2( const char* aCString, PRUint32 aLength )
         {
-          Init( nsDependentCString( aCString, aLength ) );
+          Init( Substring( aCString, aCString + aLength ) );
         }
 
     protected:
@@ -600,6 +572,130 @@ class NS_COM NS_ConvertUTF8toUCS2
       NS_ConvertUTF8toUCS2( PRUnichar );
   };
 
-#endif
+class ConvertUTF8toUCS2
+  {
+    public:
+      typedef nsACString::char_type value_type;
+      typedef nsAString::char_type  buffer_type;
 
+    ConvertUTF8toUCS2( buffer_type* aBuffer )
+        : mStart(aBuffer), mBuffer(aBuffer), mErrorEncountered(PR_FALSE) {}
 
+    size_t Length() const { return mBuffer - mStart; }
+
+    PRUint32 write( const value_type* start, PRUint32 N )
+      {
+        if ( mErrorEncountered )
+          return N;
+
+        // algorithm assumes utf8 units won't
+        // be spread across fragments
+        const value_type* p = start;
+        const value_type* end = start + N;
+        for ( ; p != end /* && *p */; )
+          {
+            char c = *p++;
+
+            if ( UTF8traits::isASCII(c) )
+              {
+                *mBuffer++ = buffer_type(c);
+                continue;
+              }
+
+            PRUint32 ucs4;
+            PRUint32 minUcs4;
+            PRInt32 state = 0;
+
+            if ( UTF8traits::is2byte(c) )
+              {
+                ucs4 = (PRUint32(c) << 6) & 0x000007C0L;
+                state = 1;
+                minUcs4 = 0x00000080;
+              }
+            else if ( UTF8traits::is3byte(c) )
+              {
+                ucs4 = (PRUint32(c) << 12) & 0x0000F000L;
+                state = 2;
+                minUcs4 = 0x00000800;
+              }
+            else if ( UTF8traits::is4byte(c) )
+              {
+                ucs4 = (PRUint32(c) << 18) & 0x001F0000L;
+                state = 3;
+                minUcs4 = 0x00010000;
+              }
+            else if ( UTF8traits::is5byte(c) )
+              {
+                ucs4 = (PRUint32(c) << 24) & 0x03000000L;
+                state = 4;
+                minUcs4 = 0x00200000;
+              }
+            else if ( UTF8traits::is6byte(c) )
+              {
+                ucs4 = (PRUint32(c) << 30) & 0x40000000L;
+                state = 5;
+                minUcs4 = 0x04000000;
+              }
+            else
+              {
+                NS_ERROR("Not a UTF-8 string. This code should only be used for converting from known UTF-8 strings.");
+                mErrorEncountered = PR_TRUE;
+                return N;
+              }
+
+            while ( state-- )
+              {
+                c = *p++;
+
+                if ( UTF8traits::isInSeq(c) )
+                  {
+                    PRInt32 shift = state * 6;
+                    ucs4 |= (PRUint32(c) & 0x3F) << shift;
+                  }
+                else
+                  {
+                    NS_ERROR("not a UTF8 string");
+                    mErrorEncountered = PR_TRUE;
+                    return N;
+                  }
+              }
+
+            if ( ucs4 < minUcs4 )
+              {
+                // Overlong sequence
+                *mBuffer++ = 0xFFFD;
+              }
+            else if ( ucs4 <= 0xD7FF )
+              {
+                *mBuffer++ = ucs4;
+              }
+            else if ( /* ucs4 >= 0xD800 && */ ucs4 <= 0xDFFF )
+              {
+                // Surrogates
+                *mBuffer++ = 0xFFFD;
+              }
+            else if ( ucs4 == 0xFFFE || ucs4 == 0xFFFF )
+              {
+                // Prohibited characters
+                *mBuffer++ = 0xFFFD;
+              }
+            else if ( ucs4 >= 0x00010000 )
+              {
+                *mBuffer++ = 0xFFFD;
+              }
+            else
+              {
+                if ( ucs4 != 0xFEFF ) // ignore BOM
+                    *mBuffer++ = ucs4;
+              }
+          }
+        return p - start;
+      }
+
+    private:
+      buffer_type* mStart;
+      buffer_type* mBuffer;
+      PRBool mErrorEncountered;
+  };
+
+#endif /* !defined(nsString2_h__) */
