@@ -103,7 +103,13 @@ nsView::~nsView()
 
   while (GetFirstChild() != nsnull)
   {
-    GetFirstChild()->Destroy();
+    nsView* child = GetFirstChild();
+    if (child->GetViewManager() == mViewManager) {
+      child->Destroy();
+    } else {
+      // just unhook it. Someone else will want to destroy this.
+      RemoveChild(child);
+    }
   }
 
   if (nsnull != mViewManager)
@@ -112,24 +118,23 @@ nsView::~nsView()
     
     if (nsnull != rootView)
     {
+      // Root views can have parents!
+      if (nsnull != mParent)
+      {
+        mViewManager->RemoveChild(this);
+      }
+
       if (rootView == this)
       {
         // Inform the view manager that the root view has gone away...
         mViewManager->SetRootView(nsnull);
       }
-      else
-      {
-        if (nsnull != mParent)
-        {
-          mViewManager->RemoveChild(this);
-        }
-      }
-    } 
+    }
     else if (nsnull != mParent)
     {
       mParent->RemoveChild(this);
     }
-
+    
     nsView* grabbingView = mViewManager->GetMouseEventGrabber(); //check to see if we are capturing!!!
     if (grabbingView == this)
     {
@@ -229,7 +234,10 @@ NS_IMETHODIMP nsView::Init(nsIViewManager* aManager,
   mChildClip.mTop = 0;
   mChildClip.mBottom = 0;
 
-  SetBounds(aBounds);
+  SetPosition(aBounds.x, aBounds.y);
+  nsRect dim(0, 0, aBounds.width, aBounds.height);
+
+  SetDimensions(dim, PR_FALSE);
 
   //temporarily set it...
   SetParent(NS_CONST_CAST(nsView*, NS_STATIC_CAST(const nsView*, aParent)));
@@ -292,99 +300,9 @@ NS_IMETHODIMP nsView::Paint(nsIRenderingContext& rc, const nsIRegion& region,
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_IMETHODIMP nsView::HandleEvent(nsGUIEvent *event, PRUint32 aEventFlags,
-                                    nsEventStatus* aStatus, PRBool aForceHandle, PRBool& aHandled)
+nsEventStatus nsView::HandleEvent(nsViewManager* aVM, nsGUIEvent *aEvent, PRBool aCaptured)
 {
-  NS_ENSURE_ARG_POINTER(aStatus);
-
-//printf(" %d %d %d %d (%d,%d) \n", this, event->widget, event->widgetSupports, 
-//       event->message, event->point.x, event->point.y);
-
-  // Hold a refcount to the observer. The continued existence of the observer will
-  // delay deletion of this view hierarchy should the event want to cause its
-  // destruction in, say, some JavaScript event handler.
-  nsCOMPtr<nsIViewObserver> obs;
-  mViewManager->GetViewObserver(*getter_AddRefs(obs));
-
-  // if accessible event pass directly to the view observer
-  if (event->eventStructType == NS_ACCESSIBLE_EVENT || event->message == NS_CONTEXTMENU_KEY) {
-    if (obs)
-       obs->HandleEvent((nsIView *)this, event, aStatus, aForceHandle, aHandled);
-    return NS_OK;
-  }
-
-  *aStatus = nsEventStatus_eIgnore;
-
-  //see if any of this view's children can process the event
-  if ( !(mVFlags & NS_VIEW_FLAG_DONT_CHECK_CHILDREN) ) {
-    nsRect  trect;
-    nscoord x, y;
-
-    x = event->point.x;
-    y = event->point.y;
-
-    nsView* pKid = GetFirstChild();
-    PRInt32 cnt = 0; 
-    while (pKid != nsnull && (!aHandled)) 
-    {
-      pKid->GetBounds(trect);
-
-      mChildRemoved = PR_FALSE;
-      if (PointIsInside(*pKid, x, y))
-      {
-        //the x, y position of the event in question
-        //is inside this child view, so give it the
-        //opportunity to handle the event
-
-        event->point.x -= trect.x;
-        event->point.y -= trect.y;
-
-        pKid->HandleEvent(event, 0, aStatus, PR_FALSE, aHandled);
-
-        event->point.x += trect.x;
-        event->point.y += trect.y;
-      }
- 
-      if (!aHandled) {
-        if (mChildRemoved) {
-          // a child has been removed as the result of the HandleEvent
-          // so pick up at the same relative position in the child list.
-          // We may end up skipping over a child depending on which child
-          // or the number of children removed from the list, but this should
-          // not be an issue since it is unusual to have the views destroyed
-          // while traversing this list and the only negative effect will be
-          // that the event is dispatched to a view underneath the intended view.
-          pKid = GetChild(cnt);
-        } else {
-          pKid = pKid->GetNextSibling();
-        } 
-      }
-      cnt++;
-    }
-  }
-
-  // if the child handled the event(Ignore) or it handled the event but still wants
-  // default behavor(ConsumeDoDefault) and we are visible then pass the event down the view's
-  // frame hierarchy. -EDV
-  if (!aHandled && mVis == nsViewVisibility_kShow)
-  {
-    //if no child's bounds matched the event or we consumed but still want
-    //default behavior check the view itself. -EDV
-    if (nsnull != mClientData && nsnull != obs) {
-      obs->HandleEvent((nsIView *)this, event, aStatus, aForceHandle, aHandled);
-    }
-  } 
-  /* XXX Just some debug code to see what event are being thrown away because
-     we are not visible. -EDV
-  else if (mVis == nsViewVisibility_kHide) {
-      nsIFrame* frame = (nsIFrame*)mClientData;
-      printf("Throwing away=%d %d %d (%d,%d) \n", this, event->widget, 
-              event->message, event->point.x, event->point.y);
-
-  }
-  */
-
-  return NS_OK;
+  return aVM->HandleEvent(this, aEvent, aCaptured);
 }
 
 // XXX Start Temporary fix for Bug #19416
@@ -393,13 +311,13 @@ NS_IMETHODIMP nsView::IgnoreSetPosition(PRBool aShouldIgnore)
   mShouldIgnoreSetPosition = aShouldIgnore;
   // resync here
   if (!mShouldIgnoreSetPosition) {
-    SetPosition(mBounds.x, mBounds.y);
+    SetPosition(mPosX, mPosY);
   }
   return NS_OK;
 }
 // XXX End Temporary fix for Bug #19416
 
-NS_IMETHODIMP nsView::SetPosition(nscoord aX, nscoord aY)
+void nsView::SetPosition(nscoord aX, nscoord aY)
 {
   nscoord x = aX;
   nscoord y = aY;
@@ -413,11 +331,14 @@ NS_IMETHODIMP nsView::SetPosition(nscoord aX, nscoord aY)
     x += offsetX;
     y += offsetY;
   }
-  mBounds.MoveTo(x, y);
+  mDimBounds.x += aX - mPosX;
+  mDimBounds.y += aY - mPosY;
+  mPosX = aX;
+  mPosY = aY;
 
   // XXX Start Temporary fix for Bug #19416
   if (mShouldIgnoreSetPosition) {
-    return NS_OK;
+    return;
   }
   // XXX End Temporary fix for Bug #19416
 
@@ -430,7 +351,7 @@ NS_IMETHODIMP nsView::SetPosition(nscoord aX, nscoord aY)
     mViewManager->IsCachingWidgetChanges(&caching);
     if (caching) {
       mVFlags |= NS_VIEW_FLAG_WIDGET_MOVED;
-      return NS_OK;
+      return;
     }
 
     nsIDeviceContext  *dx;
@@ -444,12 +365,10 @@ NS_IMETHODIMP nsView::SetPosition(nscoord aX, nscoord aY)
 
     GetOffsetFromWidget(&parx, &pary, pwidget);
     NS_IF_RELEASE(pwidget);
-    
-    mWindow->Move(NSTwipsToIntPixels((x + parx), scale),
-                  NSTwipsToIntPixels((y + pary), scale));
-  }
 
-  return NS_OK;
+    mWindow->Move(NSTwipsToIntPixels((mDimBounds.x + parx), scale),
+                  NSTwipsToIntPixels((mDimBounds.y + pary), scale));
+  }
 }
 
 NS_IMETHODIMP nsView::SynchWidgetSizePosition()
@@ -478,10 +397,10 @@ NS_IMETHODIMP nsView::SynchWidgetSizePosition()
       GetOffsetFromWidget(&parx, &pary, pwidget);
       NS_IF_RELEASE(pwidget);
 
-      PRInt32 x = NSTwipsToIntPixels(mBounds.x + parx, t2p);
-      PRInt32 y = NSTwipsToIntPixels(mBounds.y + pary, t2p);
-      PRInt32 width = NSTwipsToIntPixels(mBounds.width, t2p);
-      PRInt32 height = NSTwipsToIntPixels(mBounds.height, t2p);
+      PRInt32 x = NSTwipsToIntPixels(mDimBounds.x + parx, t2p);
+      PRInt32 y = NSTwipsToIntPixels(mDimBounds.y + pary, t2p);
+      PRInt32 width = NSTwipsToIntPixels(mDimBounds.width, t2p);
+      PRInt32 height = NSTwipsToIntPixels(mDimBounds.height, t2p);
 
       nsRect bounds;
       mWindow->GetBounds(bounds);
@@ -490,7 +409,6 @@ NS_IMETHODIMP nsView::SynchWidgetSizePosition()
       else if (bounds.width == width && bounds.height == bounds.height)
          mVFlags &= ~NS_VIEW_FLAG_WIDGET_RESIZED;
       else {
-         printf("%d) SetBounds(%d,%d,%d,%d)\n", this, x, y, width, height);
          mWindow->Resize(x,y,width,height, PR_TRUE);
          mVFlags &= ~NS_VIEW_FLAG_WIDGET_RESIZED;
          mVFlags &= ~NS_VIEW_FLAG_WIDGET_MOVED;
@@ -502,8 +420,8 @@ NS_IMETHODIMP nsView::SynchWidgetSizePosition()
     if (mVFlags & NS_VIEW_FLAG_WIDGET_RESIZED) 
     {
 
-      PRInt32 width = NSTwipsToIntPixels(mBounds.width, t2p);
-      PRInt32 height = NSTwipsToIntPixels(mBounds.height, t2p);
+      PRInt32 width = NSTwipsToIntPixels(mDimBounds.width, t2p);
+      PRInt32 height = NSTwipsToIntPixels(mDimBounds.height, t2p);
 
       nsRect bounds;
       mWindow->GetBounds(bounds);
@@ -526,8 +444,8 @@ NS_IMETHODIMP nsView::SynchWidgetSizePosition()
       GetOffsetFromWidget(&parx, &pary, pwidget);
       NS_IF_RELEASE(pwidget);
 
-      PRInt32 x = NSTwipsToIntPixels(mBounds.x + parx, t2p);
-      PRInt32 y = NSTwipsToIntPixels(mBounds.y + pary, t2p);
+      PRInt32 x = NSTwipsToIntPixels(mDimBounds.x + parx, t2p);
+      PRInt32 y = NSTwipsToIntPixels(mDimBounds.y + pary, t2p);
 
       nsRect bounds;
       mWindow->GetBounds(bounds);
@@ -557,8 +475,8 @@ NS_IMETHODIMP nsView::GetPosition(nscoord *x, nscoord *y) const
   else
   {
 
-    *x = mBounds.x;
-    *y = mBounds.y;
+    *x = mPosX;
+    *y = mPosY;
 
   }
 
@@ -566,77 +484,53 @@ NS_IMETHODIMP nsView::GetPosition(nscoord *x, nscoord *y) const
   return NS_OK;
 }
 
-NS_IMETHODIMP nsView::SetDimensions(nscoord width, nscoord height, PRBool aPaint)
+void nsView::SetDimensions(const nsRect& aRect, PRBool aPaint)
 {
-  if ((mBounds.width == width) &&
-      (mBounds.height == height))
-    return NS_OK;
-  
-  mBounds.SizeTo(width, height);
+  nsRect dims = aRect;
+  dims.MoveBy(mPosX, mPosY);
 
-#if 0
-  if (nsnull != mParent)
-  {
-    nsIScrollableView *scroller;
-
-    static NS_DEFINE_IID(kscroller, NS_ISCROLLABLEVIEW_IID);
-
-    // XXX The scrolled view is a child of the clip view which is a child of
-    // the scrolling view. It's kind of yucky the way this works. A parent
-    // notification that the child's size changed would be cleaner.
-    nsIView *grandParent;
-    mParent->GetParent(grandParent);
-    if ((nsnull != grandParent) &&
-        (NS_OK == grandParent->QueryInterface(kscroller, (void **)&scroller)))
-    {
-      scroller->ComputeContainerSize();
-    }
+  if (mDimBounds.x == dims.x && mDimBounds.y == dims.y && mDimBounds.width == dims.width
+      && mDimBounds.height == dims.height) {
+    return;
   }
-#endif
-
-  if (nsnull != mWindow)
+  
+  if (nsnull == mWindow)
   {
+    mDimBounds = dims;
+  }
+  else
+  {
+    PRBool needToMoveWidget = mDimBounds.x != dims.x || mDimBounds.y != dims.y;
+
+    mDimBounds = dims;
+
     PRBool caching = PR_FALSE;
     mViewManager->IsCachingWidgetChanges(&caching);
     if (caching) {
-      mVFlags |= NS_VIEW_FLAG_WIDGET_RESIZED;
-      return NS_OK;
+      mVFlags |= NS_VIEW_FLAG_WIDGET_RESIZED | (needToMoveWidget ? NS_VIEW_FLAG_WIDGET_MOVED : 0);
+      return;
     }
 
     nsIDeviceContext  *dx;
     float             t2p;
+    nsIWidget         *pwidget = nsnull;
+    nscoord           parx = 0, pary = 0;
   
     mViewManager->GetDeviceContext(dx);
     dx->GetAppUnitsToDevUnits(t2p);
 
-    mWindow->Resize(NSTwipsToIntPixels(width, t2p), NSTwipsToIntPixels(height, t2p),
+    GetOffsetFromWidget(&parx, &pary, pwidget);
+    NS_IF_RELEASE(pwidget);
+    
+    if (needToMoveWidget) {
+      mWindow->Move(NSTwipsToIntPixels((mDimBounds.x + parx), t2p),
+                    NSTwipsToIntPixels((mDimBounds.y + pary), t2p));
+    }
+    mWindow->Resize(NSTwipsToIntPixels(mDimBounds.width, t2p), NSTwipsToIntPixels(mDimBounds.height, t2p),
                     aPaint);
 
     NS_RELEASE(dx);
   }
-
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::GetDimensions(nscoord *width, nscoord *height) const
-{
-  *width = mBounds.width;
-  *height = mBounds.height;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::SetBounds(const nsRect &aBounds, PRBool aPaint)
-{
-  SetPosition(aBounds.x, aBounds.y);
-  SetDimensions(aBounds.width, aBounds.height, aPaint);
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::SetBounds(nscoord aX, nscoord aY, nscoord aWidth, nscoord aHeight, PRBool aPaint)
-{
-  SetPosition(aX, aY);
-  SetDimensions(aWidth, aHeight, aPaint);
-  return NS_OK;
 }
 
 NS_IMETHODIMP nsView::GetBounds(nsRect &aBounds) const
@@ -648,7 +542,7 @@ NS_IMETHODIMP nsView::GetBounds(nsRect &aBounds) const
   }
 
   nsView *rootView = mViewManager->GetRootView();
-  aBounds = mBounds;
+  aBounds = mDimBounds;
 
   if (this == rootView)
     aBounds.x = aBounds.y = 0;
@@ -713,10 +607,11 @@ NS_IMETHODIMP nsView::GetVisibility(nsViewVisibility &aVisibility) const
   return NS_OK;
 }
 
-NS_IMETHODIMP nsView::GetZIndex(PRBool &aAuto, PRInt32 &aZIndex) const
+NS_IMETHODIMP nsView::GetZIndex(PRBool &aAuto, PRInt32 &aZIndex, PRBool &aTopMost) const
 {
-	aAuto = (mVFlags & NS_VIEW_FLAG_AUTO_ZINDEX) != 0;
+  aAuto = (mVFlags & NS_VIEW_FLAG_AUTO_ZINDEX) != 0;
   aZIndex = mZIndex;
+  aTopMost = (mVFlags & NS_VIEW_FLAG_TOPMOST) != 0;
   return NS_OK;
 }
 
@@ -872,7 +767,7 @@ NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
                                      PRBool aResetVisibility)
 {
   nsIDeviceContext  *dx;
-  nsRect            trect = mBounds;
+  nsRect            trect = mDimBounds;
   float             scale;
 
   NS_IF_RELEASE(mWindow);
@@ -894,8 +789,17 @@ NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
         mWindow->Create(aNative, trect, ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
       else
       {
+        nsWidgetInitData initData;
+        if (nsnull == aWidgetInitData && nsnull != GetParent()) {
+          if (GetParent()->GetViewManager() != mViewManager) {
+            initData.mListenForResizes = PR_TRUE;
+            aWidgetInitData = &initData;
+          }
+        }
+
         nsIWidget *parent;
         GetOffsetFromWidget(nsnull, nsnull, parent);
+
         mWindow->Create(parent, trect, ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
         NS_IF_RELEASE(parent);
       }
@@ -909,24 +813,25 @@ NS_IMETHODIMP nsView::CreateWidget(const nsIID &aWindowIID,
   }
 
   //make sure visibility state is accurate
-
+  
   if (aResetVisibility) {
     nsViewVisibility vis;
     
     GetVisibility(vis);
     SetVisibility(vis);
-  }
+    }
 
   NS_RELEASE(dx);
 
   return NS_OK;
 }
 
-void nsView::SetZIndex(PRBool aAuto, PRInt32 aZIndex)
+void nsView::SetZIndex(PRBool aAuto, PRInt32 aZIndex, PRBool aTopMost)
 {
   mVFlags = (mVFlags & ~NS_VIEW_FLAG_AUTO_ZINDEX) | (aAuto ? NS_VIEW_FLAG_AUTO_ZINDEX : 0);
   mZIndex = aZIndex;
-
+  SetTopMost(aTopMost);
+  
   if (nsnull != mWindow) {
     mWindow->SetZIndex(aZIndex);
   }
@@ -1016,24 +921,6 @@ NS_IMETHODIMP nsView::List(FILE* out, PRInt32 aIndent) const
   return NS_OK;
 }
 
-NS_IMETHODIMP nsView::SetViewFlags(PRUint32 aFlags)
-{
-  mVFlags |= aFlags;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::ClearViewFlags(PRUint32 aFlags)
-{
-  mVFlags &= ~aFlags;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::GetViewFlags(PRUint32 *aFlags) const
-{
-  *aFlags = mVFlags;
-  return NS_OK;
-}
-
 NS_IMETHODIMP nsView::GetOffsetFromWidget(nscoord *aDx, nscoord *aDy, nsIWidget *&aWidget)
 {
   nsView   *ancestor = GetParent();
@@ -1044,17 +931,18 @@ NS_IMETHODIMP nsView::GetOffsetFromWidget(nscoord *aDx, nscoord *aDy, nsIWidget
   while (nsnull != ancestor)
   {
     ancestor->GetWidget(aWidget);
-	  if (nsnull != aWidget)
+	  if (nsnull != aWidget) {
+      // the widget's (0,0) is at the top left of the view's bounds, NOT its position
+      nsRect r;
+      ancestor->GetDimensions(r);
+      aDx -= r.x;
+      aDy -= r.y;
 	    return NS_OK;
+    }
 
     if ((nsnull != aDx) && (nsnull != aDy))
     {
-      nscoord offx, offy;
-
-      ancestor->GetPosition(&offx, &offy);
-
-      *aDx += offx;
-      *aDy += offy;
+      ancestor->ConvertToParentCoords(aDx, aDy);
     }
 
 	  ancestor = ancestor->GetParent();
@@ -1133,7 +1021,6 @@ PRBool nsView::PointIsInside(nsView& aView, nscoord x, nscoord y) const
   } 
 }
 
-
 NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, PRBool& aEmpty) const
 {
   // Keep track of the view's offset
@@ -1145,14 +1032,19 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
   aIsClipped = PR_FALSE;
   
   GetBounds(aClippedRect);
-  nsView* parentView = GetParent();
+  const nsView* parentView = GetParent();
+  PRBool lastViewIsFloating = GetFloating();
 
   // Walk all of the way up the views to see if any
-  // ancestor sets the NS_VIEW_PUBLIC_FLAG_CLIPCHILDREN
-  while (parentView) {  
-     PRUint32 flags;
-     parentView->GetViewFlags(&flags);
-     if (flags & NS_VIEW_FLAG_CLIPCHILDREN) {
+  // ancestor sets the NS_VIEW_PUBLIC_FLAG_CLIPCHILDREN.
+  // don't consider non-floating ancestors of a floating view.
+  while (parentView) {
+    PRBool parentIsFloating = parentView->GetFloating();
+    if (lastViewIsFloating && !parentIsFloating) {
+      break;
+    }
+
+    if (parentView->GetClipChildren()) {
       aIsClipped = PR_TRUE;
       // Adjust for clip specified by ancestor
       nscoord clipLeft;
@@ -1173,11 +1065,9 @@ NS_IMETHODIMP nsView::GetClippedRect(nsRect& aClippedRect, PRBool& aIsClipped, P
       }
     }
 
-    nsRect bounds;
-    parentView->GetBounds(bounds);
-    ancestorX -= bounds.x;
-    ancestorY -= bounds.y;
+    parentView->ConvertFromParentCoords(&ancestorX, &ancestorY);
 
+    lastViewIsFloating = parentIsFloating;
     parentView = parentView->GetParent();
   }
  
