@@ -31,13 +31,7 @@
 #include "nsIPref.h" 
 #include "prmem.h"
 
-#ifdef XP_MAC
 #include "nsBrowserWindow.h"
-#define NS_IMPL_IDS
-#else
-#define NS_IMPL_IDS
-#include "nsBrowserWindow.h"
-#endif
 #include "nsIAppShell.h"
 #include "nsIWidget.h"
 #include "nsITextWidget.h"
@@ -48,7 +42,7 @@
 #include "nsIChannel.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIFilePicker.h"
-#include "nsIFileChannel.h"
+#include "nsIFileURL.h"
 #include "nsILookAndFeel.h"
 #include "nsIComponentManager.h"
 #include "nsIFactory.h"
@@ -82,8 +76,10 @@
 #include "nsGUIEvent.h"
 #include "nsIWebProgress.h"
 #include "nsIWebBrowserSetup.h"
+#include "nsIWebBrowserPrint.h"
 
 #include "nsCWebBrowser.h"
+#include "nsUnicharUtils.h"
 
 #include "nsIParser.h"
 #include "nsEditorMode.h"
@@ -385,6 +381,22 @@ NS_IMETHODIMP nsBrowserWindow::SetVisibility(PRBool aVisibility)
    return NS_OK;
 }
 
+NS_IMETHODIMP nsBrowserWindow::GetEnabled(PRBool *aEnabled)
+{
+  NS_ENSURE_ARG_POINTER(aEnabled);
+  *aEnabled = PR_TRUE;
+  if (mWindow)
+    mWindow->IsEnabled(aEnabled);
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsBrowserWindow::SetEnabled(PRBool aEnabled)
+{
+  if (mWindow)
+    mWindow->Enable(aEnabled);
+  return NS_OK;
+}
+
 NS_IMETHODIMP nsBrowserWindow::GetMainWidget(nsIWidget** aMainWidget)
 {
    NS_ENSURE_ARG_POINTER(aMainWidget);
@@ -397,7 +409,7 @@ NS_IMETHODIMP nsBrowserWindow::GetMainWidget(nsIWidget** aMainWidget)
 NS_IMETHODIMP nsBrowserWindow::SetFocus()
 {
    //XXX First Check In
-   NS_ASSERTION(PR_FALSE, "Not Yet Implemented");
+   NS_WARNING("Not Yet Implemented");
    return NS_OK;
 }
 
@@ -1011,7 +1023,7 @@ nsBrowserWindow::DispatchMenuItem(PRInt32 aID)
     if (NS_FAILED(res)) return nsEventStatus_eIgnore;
 
     nsIURI *uri = nsnull;
-    res = service->NewURI(WALLET_EDITOR_URL, nsnull, &uri);
+    res = service->NewURI(NS_LITERAL_CSTRING(WALLET_EDITOR_URL), nsnull, nsnull, &uri);
     if (NS_FAILED(res)) return nsEventStatus_eIgnore;
 
     res = uri->QueryInterface(NS_GET_IID(nsIURI), (void**)&url);
@@ -1130,12 +1142,12 @@ nsBrowserWindow::DoFileOpen()
     if (fileURL) {
       fileURL->SetFile(file);
 
-      nsXPIDLCString url;
-      fileURL->GetSpec(getter_Copies(url));
+      nsCAutoString url;
+      fileURL->GetSpec(url);
       
       // Ask the Web widget to load the file URL
       nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mWebBrowser));
-      webNav->LoadURI(NS_ConvertASCIItoUCS2(url.get()).get(),
+      webNav->LoadURI(NS_ConvertUTF8toUCS2(url).get(),
                       nsIWebNavigation::LOAD_FLAGS_NONE,
                       nsnull,
                       nsnull,
@@ -1263,7 +1275,7 @@ nsBrowserWindow::DoFind()
 
 //----------------------------------------------------------------------
 
-#define VIEWER_BUNDLE_URL "resource:/res/viewer.properties"
+#define VIEWER_BUNDLE_URL NS_LITERAL_CSTRING("resource:/res/viewer.properties")
 
 static nsString* gTitleSuffix = nsnull;
 
@@ -1283,7 +1295,7 @@ GetTitleSuffix(void)
     if (NS_FAILED(ret)) return ret;
 
     nsIURI *uri = nsnull;
-    ret = service->NewURI(VIEWER_BUNDLE_URL, nsnull, &uri);
+    ret = service->NewURI(VIEWER_BUNDLE_URL, nsnull, nsnull, &uri);
     if (NS_FAILED(ret)) return ret;
 
     ret = uri->QueryInterface(NS_GET_IID(nsIURI), (void**)&url);
@@ -1891,13 +1903,6 @@ nsBrowserWindow::GetWindowBounds(nsRect& aBounds)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsBrowserWindow::ShowModally(PRBool aPrepare)
-{
-  // unsupported by viewer
-  return NS_ERROR_FAILURE;
-}
-
 NS_IMETHODIMP
 nsBrowserWindow::SetChrome(PRUint32 aChromeMask)
 {
@@ -2090,10 +2095,9 @@ nsBrowserWindow::OnProgress(nsIRequest* request, nsISupports *ctxt,
   if (mStatus) {
     nsAutoString url;
     if (nsnull != aURL) {
-      char* str;
-      aURL->GetSpec(&str);
-      url.AssignWithConversion(str);
-      nsCRT::free(str);
+      nsCAutoString str;
+      aURL->GetSpec(str);
+      url = NS_ConvertUTF8toUCS2(str);
     }
     url.Append(NS_LITERAL_STRING(": progress "));
     url.AppendInt(aProgress, 10);
@@ -2192,7 +2196,7 @@ nsBrowserWindow::ShowPrintPreview(PRInt32 aID)
   {
     nsCOMPtr<nsIContentViewerFile> viewerFile = do_QueryInterface(viewer);
     if (viewerFile) {
-      viewerFile->PrintPreview(nsnull);
+      //viewerFile->PrintPreview(nsnull);
     }
   }
 }
@@ -2203,11 +2207,13 @@ void nsBrowserWindow::DoPrint(void)
 
   mDocShell->GetContentViewer(getter_AddRefs(viewer));
 
-  if (viewer)
-  {
-    nsCOMPtr<nsIContentViewerFile> viewerFile = do_QueryInterface(viewer);
-    if (viewerFile) {
-      viewerFile->Print(PR_FALSE, nsnull, (nsIWebProgressListener*)nsnull);
+  if (viewer) {
+    nsCOMPtr<nsIWebBrowserPrint> webBrowserPrint = do_QueryInterface(viewer);
+    if (webBrowserPrint) {
+      if (!mPrintSettings) {
+        webBrowserPrint->GetGlobalPrintSettings(getter_AddRefs(mPrintSettings));
+      }
+      webBrowserPrint->Print(mPrintSettings, (nsIWebProgressListener*)nsnull);
     }
   }
 }
@@ -3323,8 +3329,8 @@ nsBrowserWindow::DispatchStyleMenu(PRInt32 aID)
           titles.StringAt(index, title);
           fputs(NS_LossyConvertUCS2toASCII(title).get(), stdout);
           fprintf(stdout, "\" %s%s\n", 
-                  ((title.EqualsIgnoreCase(current)) ? "<< current " : ""), 
-                  ((title.EqualsIgnoreCase(defaultStyle)) ? "** default" : ""));
+                  ((title.Equals(current, nsCaseInsensitiveStringComparator())) ? "<< current " : ""), 
+                  ((title.Equals(defaultStyle, nsCaseInsensitiveStringComparator())) ? "** default" : ""));
         }
         NS_RELEASE(shell);
       }
@@ -3399,4 +3405,13 @@ NS_IMETHODIMP nsBrowserWindow::EnsureWebBrowserChrome()
    return NS_OK;
 }
 
+nsresult nsBrowserWindow::GetWindow(nsIWidget** aWindow)
+{
+  if (aWindow && mWindow) {
+    *aWindow = mWindow.get();
+    NS_IF_ADDREF(*aWindow);
+    return NS_OK;
+  }
+  return NS_ERROR_FAILURE;
+}
 
