@@ -35,7 +35,6 @@
 // Interfaces needed to be included
 #include "nsIDocShellTreeItem.h"
 #include "nsIWebProgress.h"
-#include "nsIChannel.h"
 #include "nsIURI.h"
 #include "nsIDOMWindow.h"
 
@@ -136,9 +135,9 @@ NS_IMETHODIMP nsWebBrowserChrome::SetWebBrowser(nsIWebBrowser* aWebBrowser)
 NS_IMETHODIMP nsWebBrowserChrome::GetWebBrowser(nsIWebBrowser** aWebBrowser)
 {
   // Unimplemented, and probably will remain so; xpfe windows have docshells,
-  // not webbrowsers. Note that we're relying on this to differentiate
-  // embedded instances from Mozilla in nsWindowWatcher::InitializeDocshell.
-  // (Don't let me stop you, but keep that in mind.)
+  // not webbrowsers.
+  NS_ENSURE_ARG_POINTER(aWebBrowser);
+  *aWebBrowser = 0;
   return NS_ERROR_FAILURE;
 }
 
@@ -304,13 +303,17 @@ NS_IMETHODIMP nsWebBrowserChrome::GetDimensions(PRUint32 aFlags, PRInt32 *aX, PR
   mBrowserWindow->GetPositionAndSize(&x, &y, &cx, &cy);
   if (aFlags & DIM_FLAGS_POSITION)
   {
-    *aX = x;
-    *aY = y;
+    if(aX)
+      *aX = x;
+    if(aY)
+      *aY = y;
   }
   if (aFlags & DIM_FLAGS_SIZE_INNER || aFlags & DIM_FLAGS_SIZE_OUTER)
   {
-    *aCX = cx;
-    *aCY = cy;
+    if(aCX)
+      *aCX = cx;
+    if(aCY)
+      *aCY = cy;
   }
   return NS_OK;
 }
@@ -319,8 +322,13 @@ NS_IMETHODIMP nsWebBrowserChrome::GetSiteWindow(void ** aParentNativeWindow)
 {
    NS_ENSURE_ARG_POINTER(aParentNativeWindow);
 
-   //XXX First Check In
-   NS_ASSERTION(PR_FALSE, "Not Yet Implemented");
+   if (mBrowserWindow) {
+     nsCOMPtr<nsIWidget> window;
+     mBrowserWindow->GetWindow(getter_AddRefs(window));
+     if (window) {
+       *aParentNativeWindow = window->GetNativeData(NS_NATIVE_WINDOW);
+     }
+   }
    return NS_OK;
 }
 
@@ -396,7 +404,7 @@ nsWebBrowserChrome::OnProgressChange(nsIWebProgress* aProgress,
 NS_IMETHODIMP
 nsWebBrowserChrome::OnStateChange(nsIWebProgress* aProgress,
                                   nsIRequest* aRequest,
-                                  PRInt32 aProgressStateFlags,
+                                  PRUint32 aProgressStateFlags,
                                   nsresult aStatus)
 {
   if (aProgressStateFlags & STATE_START) {
@@ -448,10 +456,10 @@ NS_IMETHODIMP nsWebBrowserChrome::OnLocationChange(nsIWebProgress* aWebProgress,
                                                    nsIRequest* aRequest,
                                                    nsIURI* aURI)
 {
-   nsXPIDLCString spec;
+   nsCAutoString spec;
 
    if(aURI)
-      aURI->GetSpec(getter_Copies(spec));
+      aURI->GetSpec(spec);
 
    PRBool isSubFrameLoad = PR_FALSE; // Is this a subframe load
    if (aWebProgress) {
@@ -468,8 +476,7 @@ NS_IMETHODIMP nsWebBrowserChrome::OnLocationChange(nsIWebProgress* aWebProgress,
    if(mBrowserWindow->mLocation && !isSubFrameLoad)
       {
       PRUint32 size;
-      nsAutoString tmp; tmp.AssignWithConversion(spec);
-
+      NS_ConvertUTF8toUCS2 tmp(spec);
       mBrowserWindow->mLocation->SetText(tmp,size);
       }
 
@@ -492,9 +499,9 @@ nsWebBrowserChrome::OnStatusChange(nsIWebProgress* aWebProgress,
 NS_IMETHODIMP 
 nsWebBrowserChrome::OnSecurityChange(nsIWebProgress *aWebProgress, 
                                      nsIRequest *aRequest, 
-                                     PRInt32 state)
+                                     PRUint32 state)
 {
-    return NS_ERROR_NOT_IMPLEMENTED;
+    return NS_OK;
 }
 
 //*****************************************************************************
@@ -536,13 +543,11 @@ mCurrent=mTotal=mProgress=mMaxProgress=0;
      
 #ifdef MOZ_PERF_METRICS
       if (PR_FALSE == mTimerSet) {
-        char* url;
+        nsCAutoString url;
         nsresult rv = NS_OK;
-        rv = uri->GetSpec(&url);
-        if (NS_SUCCEEDED(rv)) {
-          MOZ_TIMER_LOG(("*** Timing layout processes on url: '%s', WebBrowserChrome: %p\n", url, this));
-          delete [] url;
-        }
+        rv = uri->GetSpec(url);
+        if (NS_SUCCEEDED(rv))
+          MOZ_TIMER_LOG(("*** Timing layout processes on url: '%s', WebBrowserChrome: %p\n", url.get(), this));
         MOZ_TIMER_DEBUGLOG(("Reset and start: nsWebBrowserChrome::OnLoadStart(), this=%p\n", this));
         MOZ_TIMER_RESET(mTotalTime);
         MOZ_TIMER_START(mTotalTime);
@@ -551,11 +556,11 @@ mCurrent=mTotal=mProgress=mMaxProgress=0;
 #endif
 
       if(mBrowserWindow->mStatus) {
-        nsXPIDLCString uriString;
+        nsCAutoString uriString;
 
-        uri->GetSpec(getter_Copies(uriString));
+        uri->GetSpec(uriString);
 
-        nsAutoString url; url.AssignWithConversion(uriString);
+        NS_ConvertUTF8toUCS2 url(uriString);
         url.Append(NS_LITERAL_STRING(": start"));
         PRUint32 size;
         mBrowserWindow->mStatus->SetText(url,size);
@@ -577,7 +582,7 @@ void nsWebBrowserChrome::OnLoadFinished(nsIRequest* aRequest,
   }
 #endif
 
-  nsXPIDLCString uriString;
+  nsCAutoString uriString;
   if(aRequest) {
     nsresult rv;
     nsCOMPtr<nsIChannel> channel;
@@ -587,10 +592,10 @@ void nsWebBrowserChrome::OnLoadFinished(nsIRequest* aRequest,
     if (NS_SUCCEEDED(rv)) {
       channel->GetURI(getter_AddRefs(uri));
 
-      uri->GetSpec(getter_Copies(uriString));
+      uri->GetSpec(uriString);
     }
   }  
-  nsAutoString msg; msg.AssignWithConversion(uriString);
+  NS_ConvertUTF8toUCS2 msg(uriString);
 
   PRTime endLoadTime = PR_Now();
   if(mBrowserWindow->mShowLoadTimes)
@@ -620,17 +625,16 @@ void nsWebBrowserChrome::OnStatusDNS(nsIChannel* aChannel)
 
 void nsWebBrowserChrome::OnStatusConnecting(nsIChannel* aChannel)
 {
-   nsXPIDLCString uriString;
+   nsCAutoString uriString;
    if(aChannel)
       {
       nsCOMPtr<nsIURI> uri;
       aChannel->GetURI(getter_AddRefs(uri));
 
-      uri->GetSpec(getter_Copies(uriString));
+      uri->GetSpec(uriString);
       }
    
-   nsAutoString msg(NS_LITERAL_STRING("Connecting to "));
-   msg.AppendWithConversion(uriString);
+   NS_ConvertUTF8toUCS2 msg(NS_LITERAL_CSTRING("Connecting to ") + uriString);
       
    PRUint32 size;
    mBrowserWindow->mStatus->SetText(msg,size);
