@@ -105,7 +105,7 @@ typedef void* nsNativeWidget;
  * Border styles
  */
 
-enum nsWindowType {
+enum nsWindowType {     // Don't alter previously encoded enum values - 3rd party apps may look at these
   // default top level window
   eWindowType_toplevel,
   // top level window but usually handled differently by the OS
@@ -115,10 +115,13 @@ enum nsWindowType {
   // child windows (contained inside a window on the desktop (has no border))
   eWindowType_child,
   // windows that are invisible or offscreen
-  eWindowType_invisible
+  eWindowType_invisible,
+  // plugin window
+  eWindowType_plugin,
+  // java plugin window
+  eWindowType_java
 };
 
-
 enum nsBorderStyle
 {
   // no border, titlebar, etc.. opposite of all
@@ -149,6 +152,9 @@ enum nsBorderStyle
   // show the close button
   eBorderStyle_close    = 1 << 7,
 
+  // sheet hint (Mac OS X)
+  eBorderStyle_sheet    = 1 << 8,
+
   // whatever the OS wants... i.e. don't do anything
   eBorderStyle_default  = -1
 };
@@ -209,15 +215,19 @@ struct nsWidgetInitData {
     : clipChildren(PR_FALSE), 
       clipSiblings(PR_FALSE), 
       mDropShadow(PR_FALSE),
+      mListenForResizes(PR_FALSE),
       mWindowType(eWindowType_child),
-      mBorderStyle(eBorderStyle_default)
+      mBorderStyle(eBorderStyle_default),
+      mContentType(1)  // nsIDocShellTreeItem::typeContent
   {
   }
 
   // when painting exclude area occupied by child windows and sibling windows
   PRPackedBool  clipChildren, clipSiblings, mDropShadow;
+  PRPackedBool  mListenForResizes;
   nsWindowType mWindowType;
   nsBorderStyle mBorderStyle;
+  PRUint32 mContentType; // from nsIDocShellTreeItem.idl content types - typeChrome, typeContent, etc.
 };
 
 /**
@@ -442,6 +452,12 @@ class nsIWidget : public nsISupports {
      */
     NS_IMETHOD Enable(PRBool aState) = 0;
 
+    /*
+     * Ask whether the widget is enabled
+     * @param aState returns PR_TRUE if the widget is enabled
+     */
+    NS_IMETHOD IsEnabled(PRBool *aState) = 0;
+
     /**
      * Give focus to this widget.
      */
@@ -559,6 +575,12 @@ class nsIWidget : public nsISupports {
      */
     NS_IMETHOD GetWindowType(nsWindowType& aWindowType) = 0;
 
+    /** 
+     * Hide window chrome (borders, buttons) for this widget.
+     *
+     */
+    NS_IMETHOD HideWindowChrome(PRBool aShouldHide) = 0;
+
     /**
      * Validate the widget.
      *
@@ -720,7 +742,7 @@ class nsIWidget : public nsISupports {
      *                   file name will be constructed
      */
 
-    NS_IMETHOD SetIcon(const nsAReadableString& anIconSpec) = 0;
+    NS_IMETHOD SetIcon(const nsAString& anIconSpec) = 0;
 
     /**
      * Set the widget's MenuBar.
