@@ -84,7 +84,7 @@ static NS_DEFINE_CID(kCDragServiceCID,  NS_DRAGSERVICE_CID);
 extern nsIRollupListener * gRollupListener;
 extern nsIWidget         * gRollupWidget;
 
-PRBool OnMacOSX();
+static PRBool OnMacOSX();
 
 #define kWindowPositionSlop 20
 
@@ -324,8 +324,8 @@ nsCocoaWindow::nsCocoaWindow()
 nsCocoaWindow::~nsCocoaWindow()
 {
   if ( mWindow && mWindowMadeHere ) {
-    [mWindow release];
-    [mDelegate release];
+    [mWindow autorelease];
+    [mDelegate autorelease];
   }
   
 #if 0
@@ -393,9 +393,20 @@ nsresult nsCocoaWindow::StandardCreate(nsIWidget *aParent,
                               | NSResizableWindowMask;
     if ( mWindowType == eWindowType_popup || mWindowType == eWindowType_invisible )
       features = 0;
+
+    // XXXdwh Just don't make popup windows yet.  They mess up the world.
+    if (mWindowType == eWindowType_popup)
+      return NS_OK;
+
     mWindow = [[NSWindow alloc] initWithContentRect:rect styleMask:features 
                         backing:NSBackingStoreBuffered defer:NO];
     
+    // Popups will receive a "close" message when an app terminates
+    // that causes an extra release to occur.  Make sure popups
+    // are set not to release when closed.
+    if (features == 0)
+      [mWindow setReleasedWhenClosed: NO];
+
     // create a quickdraw view as the toplevel content view of the window
     NSQuickDrawView* content = [[[NSQuickDrawView alloc] init] autorelease];
     [content setFrame:[[mWindow contentView] frame]];
@@ -853,7 +864,13 @@ nsCocoaWindow::GetNativeData(PRUint32 aDataType)
   return retVal;
 }
 
-
+NS_IMETHODIMP
+nsCocoaWindow::IsVisible(PRBool & aState)
+{
+  aState = mVisible;
+  return NS_OK;
+}
+   
 //-------------------------------------------------------------------------
 //
 // Hide or show this window
@@ -865,7 +882,9 @@ NS_IMETHODIMP nsCocoaWindow::Show(PRBool bState)
     [mWindow orderFront:NULL];
   else
     [mWindow orderOut:NULL];
-    
+
+  mVisible = bState;
+
 #if 0
   // we need to make sure we call ::Show/HideWindow() to generate the 
   // necessary activate/deactivate events. Calling ::ShowHide() is
@@ -903,6 +922,20 @@ NS_IMETHODIMP nsCocoaWindow::Show(PRBool bState)
 }
 
 
+NS_IMETHODIMP nsCocoaWindow::Enable(PRBool aState)
+{
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP nsCocoaWindow::IsEnabled(PRBool *aState)
+{
+  if (aState)
+    *aState = PR_TRUE;
+  return NS_OK;
+}
+
+
 /*
 NS_METHOD nsWindow::Minimize(void)
 {
@@ -1009,7 +1042,7 @@ NS_IMETHODIMP nsCocoaWindow::Move(PRInt32 aX, PRInt32 aY)
     
     NSPoint coord = {aX, aY};
     //coord = [mWindow convertBaseToScreen:coord];
-printf("moving to %d %d. screen coords %f %f\n", aX, aY, coord.x, coord.y);
+//printf("moving to %d %d. screen coords %f %f\n", aX, aY, coord.x, coord.y);
 
  //FIXME -- ensure it's on the screen. Cocoa automatically corrects for windows
  //   with title bars, but for other windows, we have to do this...
@@ -1019,10 +1052,10 @@ printf("moving to %d %d. screen coords %f %f\n", aX, aY, coord.x, coord.y);
     //FIXME -- doesn't work on monitors other than primary
     NSRect screenRect = [[NSScreen mainScreen] frame];
     coord.y = (screenRect.origin.y + screenRect.size.height) - coord.y;
-printf("final coords %f %f\n", coord.x, coord.y);
-printf("- window coords before %f %f\n", [mWindow frame].origin.x, [mWindow frame].origin.y);
+//printf("final coords %f %f\n", coord.x, coord.y);
+//printf("- window coords before %f %f\n", [mWindow frame].origin.x, [mWindow frame].origin.y);
     [mWindow setFrameTopLeftPoint:coord];
-printf("- window coords after %f %f\n", [mWindow frame].origin.x, [mWindow frame].origin.y);
+//printf("- window coords after %f %f\n", [mWindow frame].origin.x, [mWindow frame].origin.y);
   }
   
 #if 0
@@ -1506,7 +1539,7 @@ void nsCocoaWindow::IsActive(PRBool* aActive)
 //
 // Return true if we are on Mac OS X, caching the result after the first call
 //
-PRBool
+static PRBool
 OnMacOSX()
 {
 
@@ -1695,7 +1728,6 @@ void StopResizing ( )
   return self;
 }
 
-
 - (void)windowDidResize:(NSNotification *)aNotification
 {
   if ( !mGeckoWindow->IsResizing() ) {
@@ -1735,7 +1767,6 @@ void StopResizing ( )
 
 - (void)windowDidMove:(NSNotification *)aNotification
 {
-  printf("did move\n");
 }
 
 @end
