@@ -38,11 +38,12 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nscore.h"
+#include "nsCRT.h"
 #include "nsIAllocator.h"
 #include "plstr.h"
 #include "nsVoidArray.h"
 
-#include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsNetUtil.h"
 #include "prmem.h"
 #include "nsGfxUtils.h"
@@ -265,7 +266,7 @@ nsSound::Play(nsIURL *aURL)
 
   // try to get from cache
   nsCOMPtr<nsISupports>   requestSupports;
-  (void)GetSoundFromCache(aURL, getter_AddRefs(requestSupports));
+  (void)GetSoundFromCache(NS_STATIC_CAST(nsIURI*,aURL), getter_AddRefs(requestSupports));
   if (requestSupports)
   {
     nsSoundRequest* cachedRequest = nsSoundRequest::GetFromISupports(requestSupports);
@@ -340,15 +341,15 @@ nsSound::GetSoundFromCache(nsIURI* inURI, nsISupports** outSound)
 {
   nsresult rv;
   
-  nsXPIDLCString uriSpec;
-  inURI->GetSpec(getter_Copies(uriSpec));
+  nsCAutoString uriSpec;
+  inURI->GetAsciiSpec(uriSpec);
 
   nsCOMPtr<nsICacheSession> cacheSession;
   rv = GetCacheSession(getter_AddRefs(cacheSession));
   if (NS_FAILED(rv)) return rv;
   
   nsCOMPtr<nsICacheEntryDescriptor> entry;
-  rv = cacheSession->OpenCacheEntry(uriSpec, nsICache::ACCESS_READ, nsICache::BLOCKING, getter_AddRefs(entry));
+  rv = cacheSession->OpenCacheEntry(uriSpec.get(), nsICache::ACCESS_READ, nsICache::BLOCKING, getter_AddRefs(entry));
 
 #ifdef SOUND_DEBUG
   printf("Got sound from cache with rv %ld\n", rv);
@@ -371,8 +372,8 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
   inChannel->GetOriginalURI(getter_AddRefs(uri));
   if (!uri) return NS_ERROR_FAILURE;
   
-  nsXPIDLCString uriSpec;
-  rv = uri->GetSpec(getter_Copies(uriSpec));
+  nsCAutoString uriSpec;
+  rv = uri->GetAsciiSpec(uriSpec);
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsICacheSession> cacheSession;
@@ -380,7 +381,7 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsICacheEntryDescriptor> entry;
-  rv = cacheSession->OpenCacheEntry(uriSpec, nsICache::ACCESS_WRITE, nsICache::BLOCKING, getter_AddRefs(entry));
+  rv = cacheSession->OpenCacheEntry(uriSpec.get(), nsICache::ACCESS_WRITE, nsICache::BLOCKING, getter_AddRefs(entry));
 #ifdef SOUND_DEBUG
   printf("Put sound in cache with rv %ld\n", rv);
 #endif
@@ -408,7 +409,7 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
     }
   }
   
-  if (expirationTime == 0)
+  if (expirationTime == PRUint32(-1))   // no expiration time (never expires)
   {
     // set it to some reasonable default, like now + 24 hours
     expirationTime = SecondsFromPRTime(PR_Now()) + 60 * 60 * 24;
@@ -600,7 +601,7 @@ nsSystemSoundRequest::PlaySound()
   
   const PRInt32   kSoundTimerInterval = 250;      // 250 milliseconds
   rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kSoundTimerInterval,
-          NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_PRECISE);
+          PR_TRUE, NS_TYPE_REPEATING_PRECISE);
   if (NS_FAILED(rv)) {
     Cleanup();
     return rv;
@@ -691,13 +692,13 @@ nsMovieSoundRequest::OnStreamComplete(nsIStreamLoader *aLoader,
   if (NS_FAILED(aStatus))
     return NS_ERROR_FAILURE;
 
-  nsXPIDLCString  contentType;
+  nsCAutoString contentType;
 
   nsCOMPtr<nsIRequest>    request;
   aLoader->GetRequest(getter_AddRefs(request));
   nsCOMPtr<nsIChannel>  channel = do_QueryInterface(request);
   if (channel)
-    channel->GetContentType(getter_Copies(contentType));
+    channel->GetContentType(contentType);
   
   // we could use a Pointer data handler type, and avoid this
   // allocation/copy, in QuickTime 5 and above.
@@ -804,7 +805,7 @@ nsMovieSoundRequest::PlaySound()
     
     const PRInt32   kMovieTimerInterval = 250;      // 250 milliseconds
     rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kMovieTimerInterval,
-            NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_PRECISE);
+            PR_TRUE, NS_TYPE_REPEATING_PRECISE);
     if (NS_FAILED(rv)) {
       Cleanup();
       return rv;
