@@ -58,31 +58,17 @@
 
 #include "nsIPref.h"
 
+static void
+ConvertKeyEventToContextMenuEvent(const nsKeyEvent* inKeyEvent,
+                                  nsMouseEvent* outCMEvent);
+
+static inline PRBool
+IsContextMenuKey(const nsKeyEvent& inKeyEvent);
+
 static NS_DEFINE_CID(kRegionCID, NS_REGION_CID);
 static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
 static NS_DEFINE_IID(kCDragServiceCID,  NS_DRAGSERVICE_CID);
 
-// keeping track of a list of simultaneously modal widgets
-class ModalWidgetList {
-public:
-  ModalWidgetList(nsWidget *aWidget);
-  ~ModalWidgetList();
-
-  static PRBool Find(nsWidget *aWidget);
-  static void   Append(nsWidget *aWidget);
-  static void   Remove(nsWidget *aWidget);
-  static void   RemoveLast(void);
-  static void   Suppress(PRBool aSuppress);
-
-private:
-  nsWidget        *mWidget;
-  ModalWidgetList *mNext,
-                  *mPrev,
-                  *mLast; // valid only for head of list
-};
-
-PRUint32 nsWidget::sWidgetCount = 0;
-
 // this is the nsWindow with the focus
 nsWidget *nsWidget::sFocusWindow = 0;
 
@@ -90,6 +76,76 @@ nsWidget *nsWidget::sFocusWindow = 0;
 // around so that we can synth drag events properly
 guint32 nsWidget::sLastEventTime = 0;
 
+// we should convert the context key to context menu event in the OnKey member
+// function, and dispatch the NS_CONTEXTMENU_KEY instead of a normal key event.
+PRBool nsWidget::OnKey(nsKeyEvent &aEvent)
+{
+
+  PRBool    ret = PR_FALSE;
+  PRBool    releaseWidget = PR_FALSE;
+  nsWidget *widget = nsnull;
+
+  // rewrite the key event to the window with 'de focus
+  if (sFocusWindow) {
+    widget = sFocusWindow;
+    NS_ADDREF(widget);
+    aEvent.widget = sFocusWindow;
+    releaseWidget = PR_TRUE;
+  }
+  if (mEventCallback) {
+    // before we dispatch a key, check if it's the context menu key.
+    // If so, send a context menu key event instead.
+    if (IsContextMenuKey(aEvent)) {
+      nsMouseEvent contextMenuEvent;
+      ConvertKeyEventToContextMenuEvent(&aEvent, &contextMenuEvent);
+      ret = DispatchWindowEvent(&contextMenuEvent);
+    }
+    else
+      ret = DispatchWindowEvent(&aEvent);
+  }
+
+  if (releaseWidget)
+    NS_RELEASE(widget);
+
+  return ret;
+}
+
+//
+// ConvertKeyEventToContextMenuEvent
+//
+// Take a key event and all of its attributes at convert it into
+// a context menu event. We want just about  everything (focused
+// widget, etc) but a few fields should be tweaked. See also the
+// implemention for the Mac
+
+static void
+ConvertKeyEventToContextMenuEvent(const nsKeyEvent* inKeyEvent,
+                                  nsMouseEvent* outCMEvent)
+{
+  *(nsInputEvent *)outCMEvent = *(nsInputEvent *)inKeyEvent;
+  outCMEvent->eventStructType = NS_MOUSE_EVENT;
+  outCMEvent->message = NS_CONTEXTMENU_KEY;
+  outCMEvent->isShift = outCMEvent->isControl = PR_FALSE;
+  outCMEvent->isAlt = outCMEvent->isMeta = PR_FALSE;
+  outCMEvent->clickCount = 0;
+  outCMEvent->acceptActivation = PR_FALSE;
+}
+ 
+
+// IsContextMenuKey
+//
+// Check if the event should be a context menu event instead. Currently,
+// that is a Shift-F10 in linux.
+//
+static inline PRBool
+IsContextMenuKey(const nsKeyEvent& inKeyEvent)
+{
+   enum { kContextMenuKey = NS_VK_F10 };
+
+   return (inKeyEvent.keyCode == kContextMenuKey && inKeyEvent.isShift &&
+            !inKeyEvent.isControl && !inKeyEvent.isMeta && !inKeyEvent.isAlt);
+}
+
 PRBool nsWidget::OnInput(nsInputEvent &aEvent)
 {
 
@@ -114,6 +170,13 @@ PRBool nsWidget::OnInput(nsInputEvent &aEvent)
   return ret;
 }
 
+/* virtual - really to be implemented by nsWindow */
+GtkWidget *nsWidget::GetOwningWidget()
+{
+  NS_WARNING("nsWidget::GetOwningWidget called!\n");
+  return nsnull;
+}
+
 void nsWidget::SetLastEventTime(guint32 aTime)
 {
   sLastEventTime = aTime;
@@ -125,39 +188,23 @@ void nsWidget::GetLastEventTime(guint32 *aTime)
     *aTime = sLastEventTime;
 }
 
+void nsWidget::DropMotionTarget(void)
+{
+  if (sButtonMotionTarget) {
+    GtkWidget *owningWidget = sButtonMotionTarget->GetOwningWidget();
+    if (owningWidget)
+      gtk_grab_remove(owningWidget);
+
+    sButtonMotionTarget = nsnull;
+  }
+}
+
 nsCOMPtr<nsIRollupListener> nsWidget::gRollupListener;
 nsWeakPtr          nsWidget::gRollupWidget;
-PRBool             nsWidget::gRollupConsumeRollupEvent = PR_FALSE;
 
 PRBool             nsWidget::mGDKHandlerInstalled = PR_FALSE;
 PRBool             nsWidget::sTimeCBSet = PR_FALSE;
 
-#ifdef NS_DEBUG
-// debugging window
-static GtkWidget *debugTopLevel = NULL;
-static GtkWidget *debugBox = NULL;
-static GtkWidget *debugEntryBox = NULL;
-static GtkWidget *debugButton = NULL;
-static GtkWidget *debugCheckBox = NULL;
-nsWidget  *nsWidget::debugWidget = NULL;
-PRBool     nsWidget::sDebugFeedback = PR_FALSE;
-static PRBool     debugCheckedDebugWindow = PR_FALSE;
-static PRBool     debugCallbackRegistered = PR_FALSE;
-
-static void      debugHandleActivate(GtkEditable *editable,
-                                     gpointer user_data);
-static void      debugHandleClicked (GtkButton   *button,
-                                     gpointer user_data);
-static void      debugHandleToggle  (GtkToggleButton *button,
-                                     gpointer user_data);
-static void      debugSetupWindow   (void);
-static void      debugDestroyWindow (void);
-static int       debugWindowPrefChanged (const char *newpref, void *data);
-static void      debugRegisterCallback  (void);
-static gint      debugHandleWindowClose(GtkWidget *window, void *data);
-const char      *debugPrefName = "nglayout.widget.debugWindow";
-#endif /* NS_DEBUG */
-
 //
 // Keep track of the last widget being "dragged"
 //
@@ -171,7 +218,6 @@ gint nsWidget::sButtonMotionWidgetY = -1;
 
 nsWidget::nsWidget()
 {
-  mGrabTime = 0;
   mWidget = nsnull;
   mMozBox = 0;
   mParent = nsnull;
@@ -190,8 +236,6 @@ nsWidget::nsWidget()
     mUpdateArea->SetTo(0, 0, 0, 0);
   }
   
-  sWidgetCount++;
-
   mListenForResizes = PR_FALSE;
   mHasFocus = PR_FALSE;
   if (mGDKHandlerInstalled == PR_FALSE) {
@@ -217,17 +261,6 @@ nsWidget::nsWidget()
     }
     dragServiceGTK->TargetSetTimeCallback(nsWidget::GetLastEventTime);
   }
-#ifdef NS_DEBUG
-  // see if we need to set up the debugging window
-  if (!debugCheckedDebugWindow) {
-    debugSetupWindow();
-  }
-  // this will set up the callback for when the debug
-  // pref changes
-  if (!debugCallbackRegistered) {
-    debugRegisterCallback();
-  }
-#endif /* NS_DEBUG */
 }
 
 nsWidget::~nsWidget()
@@ -237,18 +270,10 @@ nsWidget::~nsWidget()
   printf("nsWidget::~nsWidget:%p\n", this);
 #endif
 
-  sWidgetCount--;
-
   // it's safe to always call Destroy() because it will only allow itself
   // to be called once
   Destroy();
 
-  NS_ASSERTION(!ModalWidgetList::Find(this), "destroying widget without first clearing modality.");
-#ifdef NS_DEBUG
-  if (mIsToplevel) {
-    g_print("nsWidget::~nsWidget() of toplevel: %d widgets still exist.\n", sWidgetCount);
-  }
-#endif
 }
 
 
@@ -382,13 +407,6 @@ nsWidget::DestroySignal(GtkWidget* aGtkWidget, nsWidget* aWidget)
   return PR_TRUE;
 }
 
-void
-nsWidget::SuppressModality(PRBool aSuppress)
-{
-  ModalWidgetList::Suppress(aSuppress); 
-}
-
-
 void
 nsWidget::OnDestroySignal(GtkWidget* aGtkWidget)
 {
@@ -440,30 +458,6 @@ NS_IMETHODIMP nsWidget::CaptureRollupEvents(nsIRollupListener * aListener, PRBoo
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWidget::SetModal(PRBool aModal)
-{
-  GtkWindow *topWindow;
-
-  topWindow = GetTopLevelWindow();
-
-  if (!topWindow) {
-    if (!aModal) {
-      ModalWidgetList::RemoveLast();
-    }
-    return NS_ERROR_FAILURE;
-  }
-
-  if (aModal) {
-    ModalWidgetList::Append(this);
-    gtk_window_set_modal(topWindow, TRUE);
-  } else {
-    ModalWidgetList::Remove(this);
-    gtk_window_set_modal(topWindow, FALSE);
-  }
-
-	return NS_OK;
-}
-
 NS_IMETHODIMP nsWidget::IsVisible(PRBool &aState)
 {
   if (mWidget)
@@ -604,18 +598,25 @@ PRBool nsWidget::OnMove(PRInt32 aX, PRInt32 aY)
 // Enable/disable this component
 //
 //-------------------------------------------------------------------------
-NS_IMETHODIMP nsWidget::Enable(PRBool bState)
+NS_IMETHODIMP nsWidget::Enable(PRBool aState)
 {
   if (mWidget)
   {
-    if (GTK_WIDGET_SENSITIVE(mWidget) == bState)
+    if (GTK_WIDGET_SENSITIVE(mWidget) == aState)
       return NS_OK;
-    gtk_widget_set_sensitive(mWidget, bState);
+    gtk_widget_set_sensitive(mWidget, aState);
   }
 
   return NS_OK;
 }
 
+NS_IMETHODIMP nsWidget::IsEnabled(PRBool *aState)
+{
+  NS_ENSURE_ARG_POINTER(aState);
+  *aState = !mWidget || GTK_WIDGET_SENSITIVE(mWidget);
+  return NS_OK;
+}
+
 //-------------------------------------------------------------------------
 //
 // Give the focus to this component
@@ -869,7 +870,7 @@ NS_IMETHODIMP nsWidget::SetCursor(nsCursor aCursor)
 }
 
 #define CAPS_LOCK_IS_ON \
-(nsWidget::sDebugFeedback && (nsGtkUtils::gdk_keyboard_get_modifiers() & GDK_LOCK_MASK))
+(nsGtkUtils::gdk_keyboard_get_modifiers() & GDK_LOCK_MASK)
 
 NS_IMETHODIMP nsWidget::Validate()
 {
@@ -888,8 +889,10 @@ NS_IMETHODIMP nsWidget::Invalidate(PRBool aIsSynchronous)
   if (!GTK_WIDGET_REALIZED(mWidget) || !GTK_WIDGET_VISIBLE(mWidget))
     return NS_ERROR_FAILURE;
 
-#ifdef NS_DEBUG
-  if (CAPS_LOCK_IS_ON)
+#ifdef DEBUG
+  // Check the pref _before_ checking caps lock, because checking
+  // caps lock requires a server round-trip.
+  if (debug_GetCachedBoolPref("nglayout.debug.invalidate_dumping") && CAPS_LOCK_IS_ON)
   {
     debug_DumpInvalidate(stdout,
                          this,
@@ -898,7 +901,7 @@ NS_IMETHODIMP nsWidget::Invalidate(PRBool aIsSynchronous)
                          debug_GetName(mWidget),
                          debug_GetRenderXID(mWidget));
   }
-#endif // NS_DEBUG
+#endif // DEBUG
 
   mUpdateArea->SetTo(0, 0, mBounds.width, mBounds.height);
 
@@ -924,8 +927,10 @@ NS_IMETHODIMP nsWidget::Invalidate(const nsRect & aRect, PRBool aIsSynchronous)
 
   mUpdateArea->Union(aRect.x, aRect.y, aRect.width, aRect.height);
 
-#ifdef NS_DEBUG
-  if (CAPS_LOCK_IS_ON)
+#ifdef DEBUG
+  // Check the pref _before_ checking caps lock, because checking
+  // caps lock requires a server round-trip.
+  if (debug_GetCachedBoolPref("nglayout.debug.invalidate_dumping") && CAPS_LOCK_IS_ON)
   {
     debug_DumpInvalidate(stdout,
                          this,
@@ -934,7 +939,7 @@ NS_IMETHODIMP nsWidget::Invalidate(const nsRect & aRect, PRBool aIsSynchronous)
                          debug_GetName(mWidget),
                          debug_GetRenderXID(mWidget));
   }
-#endif // NS_DEBUG
+#endif // DEBUG
 
   if (aIsSynchronous)
   {
@@ -981,8 +986,10 @@ NS_IMETHODIMP nsWidget::InvalidateRegion(const nsIRegion *aRegion, PRBool aIsSyn
     nsRegionRect *r = &(regionRectSet->mRects[i]);
 
 
-#ifdef NS_DEBUG
-    if (CAPS_LOCK_IS_ON)
+#ifdef DEBUG
+    // Check the pref _before_ checking caps lock, because checking
+    // caps lock requires a server round-trip.
+    if (debug_GetCachedBoolPref("nglayout.debug.invalidate_dumping") && CAPS_LOCK_IS_ON)
     {
       nsRect rect(r->x, r->y, r->width, r->height);
       debug_DumpInvalidate(stdout,
@@ -992,7 +999,7 @@ NS_IMETHODIMP nsWidget::InvalidateRegion(const nsIRegion *aRegion, PRBool aIsSyn
                            debug_GetName(mWidget),
                            debug_GetRenderXID(mWidget));
     }
-#endif // NS_DEBUG
+#endif // DEBUG
 
 
     if (aIsSynchronous)
@@ -1151,11 +1158,11 @@ nsresult nsWidget::CreateWidget(nsIWidget *aParent,
 
   if (aNativeParent) {
     parentWidget = GTK_OBJECT(aNativeParent);
-    // we've got a native parent so listen for resizes
     mListenForResizes = PR_TRUE;
   } else if (aParent) {
     // this ups the refcount of the gtk widget, we must unref later.
     parentWidget = GTK_OBJECT(aParent->GetNativeData(NS_NATIVE_WIDGET));
+    mListenForResizes = aInitData ? aInitData->mListenForResizes : PR_FALSE;
   }
 
   mBounds = aRect;
@@ -1167,9 +1174,14 @@ nsresult nsWidget::CreateWidget(nsIWidget *aParent,
   gtk_widget_pop_visual();
 
   if (mWidget) {
-
-    InstallButtonPressSignal(mWidget);
-    InstallButtonReleaseSignal(mWidget);
+    /* we used to listen to motion notify signals, button press
+       signals and button release signals here but since nsWindow
+       became its own managed class we don't need to do that by
+       default anymore.  Any subclasses that need to listen to those
+       events should do so on their own. */
+    
+    // InstallButtonPressSignal(mWidget);
+    // InstallButtonReleaseSignal(mWidget);
     
     // InstallMotionNotifySignal(mWidget);
     
@@ -1330,7 +1342,7 @@ PRBool nsWidget::DispatchFocus(nsGUIEvent &aEvent)
 //
 //////////////////////////////////////////////////////////////////
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 PRInt32
 nsWidget::debug_GetRenderXID(GtkObject * aGtkWidget)
 {
@@ -1366,7 +1378,7 @@ nsWidget::debug_GetName(GtkWidget * aGtkWidget)
   return nsCAutoString("null");
 }
 
-#endif // NS_DEBUG
+#endif // DEBUG
 
 
 //////////////////////////////////////////////////////////////////
@@ -1382,13 +1394,16 @@ NS_IMETHODIMP nsWidget::DispatchEvent(nsGUIEvent *aEvent,
 {
   NS_ADDREF(aEvent->widget);
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
   GtkObject *gw;
   void *nativeWidget = aEvent->widget->GetNativeData(NS_NATIVE_WIDGET);
   if (nativeWidget) {
     gw = GTK_OBJECT(nativeWidget);
     
-    if (CAPS_LOCK_IS_ON)
+    // Check the pref _before_ checking caps lock, because checking
+    // caps lock requires a server round-trip.
+
+    if (debug_GetCachedBoolPref("nglayout.debug.event_dumping") && CAPS_LOCK_IS_ON)
       {
         debug_DumpEvent(stdout,
                         aEvent->widget,
@@ -1397,7 +1412,7 @@ NS_IMETHODIMP nsWidget::DispatchEvent(nsGUIEvent *aEvent,
                         (PRInt32) debug_GetRenderXID(gw));
       }
   }
-#endif // NS_DEBUG
+#endif // DEBUG
 
   if (nsnull != mMenuListener) {
     if (NS_MENU_EVENT == aEvent->eventStructType)
@@ -1498,17 +1513,6 @@ nsWidget::AddToEventMask(GtkWidget * aWidget,
 
   gtk_widget_add_events(aWidget,aEventMask);
 }
-//////////////////////////////////////////////////////////////////
-void 
-nsWidget::InstallMotionNotifySignal(GtkWidget * aWidget)
-{
-  NS_ASSERTION( nsnull != aWidget, "widget is null");
-
-  InstallSignal((GtkWidget *)aWidget,
-				(gchar *)"motion_notify_event",
-				GTK_SIGNAL_FUNC(nsWidget::MotionNotifySignal));
-}
-
 //////////////////////////////////////////////////////////////////
 void 
 nsWidget::InstallEnterNotifySignal(GtkWidget * aWidget)
@@ -1746,90 +1750,6 @@ nsWidget::OnLeaveNotifySignal(GdkEventCrossing * aGdkCrossingEvent)
 }
 
 
-//
-// IsMouseInWindow
-//
-// Check if the mouse x/y is within the given window
-//
-PRBool
-nsWidget :: IsMouseInWindow ( GdkWindow* inWindow, PRInt32 inMouseX, PRInt32 inMouseY )
-{
-  gint x, y;
-  gint w, h;
-
-  // XXX this causes a round trip to the ever lovely X server.  fix me.
-  gdk_window_get_origin(inWindow, &x, &y);
-
-  // this doesn't... it just pokes GdkWindowPrivate :-)
-  gdk_window_get_size(inWindow, &w, &h);
-
-  if ( inMouseX > x && inMouseX < x + w &&
-       inMouseY > y && inMouseY < y + h )
-    return PR_TRUE;
-
-  return PR_FALSE;
-
-} // IsMouseInWindow
-
-
-//
-// HandlePopup
-//
-// Deal with rollup of popups (xpmenus, etc)
-// 
-PRBool
-nsWidget::HandlePopup(PRInt32 inMouseX, PRInt32 inMouseY, PRBool isWheel)
-{
-  PRBool retVal = PR_FALSE;
-  nsCOMPtr<nsIWidget> rollupWidget = do_QueryReferent(gRollupWidget);
-  if (rollupWidget && gRollupListener)
-  {
-    GdkWindow *currentPopup = (GdkWindow *)rollupWidget->GetNativeData(NS_NATIVE_WINDOW);
-    if ( !IsMouseInWindow(currentPopup, inMouseX, inMouseY) ) {
-      PRBool rollup = PR_TRUE;
-      if (isWheel) {
-        gRollupListener->ShouldRollupOnMouseWheelEvent(&rollup);
-        retVal = PR_TRUE;
-      }
-      // if we're dealing with menus, we probably have submenus and we don't
-      // want to rollup if the clickis in a parent menu of the current submenu
-      nsCOMPtr<nsIMenuRollup> menuRollup ( do_QueryInterface(gRollupListener) );
-      if ( menuRollup ) {
-        nsCOMPtr<nsISupportsArray> widgetChain;
-        menuRollup->GetSubmenuWidgetChain ( getter_AddRefs(widgetChain) );
-        if ( widgetChain ) {
-          PRUint32 count = 0;
-          widgetChain->Count ( &count );
-          for ( PRUint32 i = 0; i < count; ++i ) {
-            nsCOMPtr<nsISupports> genericWidget;
-            widgetChain->GetElementAt ( i, getter_AddRefs(genericWidget) );
-            nsCOMPtr<nsIWidget> widget ( do_QueryInterface(genericWidget) );
-            if ( widget ) {
-              GdkWindow* currWindow = (GdkWindow*) widget->GetNativeData(NS_NATIVE_WINDOW);
-              if ( IsMouseInWindow(currWindow, inMouseX, inMouseY) ) {
-                rollup = PR_FALSE;
-                break;
-              }
-            }
-          } // foreach parent menu widget
-        }
-      } // if rollup listener knows about menus
-
-      // if we've determined that we should still rollup, do it.
-      if ( rollup ) {
-        gRollupListener->Rollup();
-        retVal = PR_TRUE;
-      }
-    }
-  } else {
-    gRollupWidget = nsnull;
-    gRollupListener = nsnull;
-  }
-
-  return retVal;
-} // HandlePopup
-
-
 //////////////////////////////////////////////////////////////////////
 /* virtual */ void
 nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
@@ -1838,11 +1758,6 @@ nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
   nsMouseScrollEvent scrollEvent;
   PRUint32 eventType = 0;
 
-  PRBool isWheel = (aGdkButtonEvent->button == 4 ||
-                    aGdkButtonEvent->button == 5);
-  if (HandlePopup(aGdkButtonEvent->x_root, aGdkButtonEvent->y_root, isWheel))
-    return;
-   
   // Switch on single, double, triple click.
   switch (aGdkButtonEvent->type) 
   {
@@ -1909,6 +1824,13 @@ nsWidget::OnButtonPressSignal(GdkEventButton * aGdkButtonEvent)
   // Set the button motion target and remeber the widget and root coords
   sButtonMotionTarget = this;
 
+  // Make sure to add this widget to the gtk grab list so that events
+  // are rewritten to this window.
+  GtkWidget *owningWidget;
+  owningWidget = GetOwningWidget();
+  if (owningWidget)
+    gtk_grab_add(owningWidget);
+
   sButtonMotionRootX = (gint) aGdkButtonEvent->x_root;
   sButtonMotionRootY = (gint) aGdkButtonEvent->y_root;
 
@@ -1979,6 +1901,10 @@ nsWidget::OnButtonReleaseSignal(GdkEventButton * aGdkButtonEvent)
     event.point.y = nscoord(sButtonMotionWidgetY + diffY);
   }
 
+  // Drop the motion target before dispatching the event so that we
+  // don't get events that we shouldn't.
+  DropMotionTarget();
+
   // event.widget can get set to null when calling DispatchMouseEvent,
   // so to release it we must make a copy
   nsWidget* theWidget = NS_STATIC_CAST(nsWidget*,event.widget);
@@ -1987,7 +1913,6 @@ nsWidget::OnButtonReleaseSignal(GdkEventButton * aGdkButtonEvent)
   theWidget->DispatchMouseEvent(event);
   NS_IF_RELEASE(theWidget);
 
-  DropMotionTarget();
 }
 //////////////////////////////////////////////////////////////////////
 /* virtual */ void
@@ -2113,89 +2038,11 @@ nsWidget::InitMouseEvent(GdkEventButton * aGdkButtonEvent,
 
   }
 }
-//////////////////////////////////////////////////////////////////
-PRBool
-nsWidget::DropEvent(GtkWidget * aWidget, 
-                    GdkWindow * aEventWindow) 
-{
-  NS_ASSERTION( nsnull != aWidget, "widget is null");
-  NS_ASSERTION( nsnull != aEventWindow, "event window is null");
-
-#if 0
-  static int count = 0;
-
-  if (GTK_IS_LAYOUT(aWidget))
-  {
-    GtkLayout * layout = GTK_LAYOUT(aWidget);
-
-    printf("%4d DropEvent(this=%p,widget=%p,event_win=%p,wid_win=%p,bin_win=%p)\n",
-           count++,
-           this,
-           aWidget,
-           aEventWindow,
-           aWidget->window,
-           layout->bin_window);
-  }
-  else
-  {
-    printf("%4d DropEvent(this=%p,widget=%p,event_win=%p,wid_win=%p)\n",
-           count++,
-           this,
-           aWidget,
-           aEventWindow,
-           aWidget->window);
-  }
-#endif
-
-
-  // For gtklayout widgets, we dont want to handle events
-  // that occur in the sub windows.  Check the window member
-  // of the GdkEvent, if it is not the gtklayout's bin_window,
-  // drop the event.
-  if (GTK_IS_LAYOUT(aWidget))
-  {
-    GtkLayout * layout = GTK_LAYOUT(aWidget);
-
-    if (aEventWindow != layout->bin_window)
-    {
-#ifdef DEBUG_pavlov
-      printf("dropping event!!!!!!!\n");
-#endif
-      return PR_TRUE;
-    }
-  }
-  return PR_FALSE;
-}
-//////////////////////////////////////////////////////////////////////
-
 
 //////////////////////////////////////////////////////////////////
 //
 // GTK widget signals
 //
-//////////////////////////////////////////////////////////////////
-/* static */ gint
-nsWidget::MotionNotifySignal(GtkWidget *      aWidget,
-							 GdkEventMotion * aGdkMotionEvent,
-							 gpointer         aData)
-{
-  NS_ASSERTION( nsnull != aWidget, "widget is null");
-  NS_ASSERTION( nsnull != aGdkMotionEvent, "event is null");
-
-  nsWidget * widget = (nsWidget *) aData;
-
-  NS_ASSERTION( nsnull != widget, "instance pointer is null");
-
-  if (widget->DropEvent(aWidget, aGdkMotionEvent->window))
-  {
-    return PR_TRUE;
-  }
-
-  widget->OnMotionNotifySignal(aGdkMotionEvent);
-
-  return PR_TRUE;
-}
-
 //////////////////////////////////////////////////////////////////
 /* static */ gint 
 nsWidget::EnterNotifySignal(GtkWidget *        aWidget, 
@@ -2209,11 +2056,6 @@ nsWidget::EnterNotifySignal(GtkWidget *        aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-  if (widget->DropEvent(aWidget, aGdkCrossingEvent->window))
-  {
-	return PR_TRUE;
-  }
-
   widget->OnEnterNotifySignal(aGdkCrossingEvent);
 
   return PR_TRUE;
@@ -2231,11 +2073,6 @@ nsWidget::LeaveNotifySignal(GtkWidget *        aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-  if (widget->DropEvent(aWidget, aGdkCrossingEvent->window))
-  {
-    return PR_TRUE;
-  }
-
   widget->OnLeaveNotifySignal(aGdkCrossingEvent);
 
   return PR_TRUE;
@@ -2255,11 +2092,6 @@ nsWidget::ButtonPressSignal(GtkWidget *      aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-  if (widget->DropEvent(aWidget, aGdkButtonEvent->window))
-  {
-    return PR_TRUE;
-  }
-
   widget->OnButtonPressSignal(aGdkButtonEvent);
 
   return PR_TRUE;
@@ -2279,11 +2111,6 @@ nsWidget::ButtonReleaseSignal(GtkWidget *      aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-  if (widget->DropEvent(aWidget, aGdkButtonEvent->window))
-  {
-    return PR_TRUE;
-  }
-
   widget->OnButtonReleaseSignal(aGdkButtonEvent);
 
   return PR_TRUE;
@@ -2318,11 +2145,6 @@ nsWidget::FocusInSignal(GtkWidget *      aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-//   if (widget->DropEvent(aWidget, aGdkFocusEvent->window))
-//   {
-// 	return PR_TRUE;
-//   }
-
   widget->OnFocusInSignal(aGdkFocusEvent);
 
   if (GTK_IS_WINDOW(aWidget))
@@ -2345,11 +2167,6 @@ nsWidget::FocusOutSignal(GtkWidget *      aWidget,
 
   NS_ASSERTION( nsnull != widget, "instance pointer is null");
 
-//   if (widget->DropEvent(aWidget, aGdkFocusEvent->window))
-//   {
-// 	return PR_TRUE;
-//   }
-
   widget->OnFocusOutSignal(aGdkFocusEvent);
 
   if (GTK_IS_WINDOW(aWidget))
@@ -2359,19 +2176,6 @@ nsWidget::FocusOutSignal(GtkWidget *      aWidget,
 }
 //////////////////////////////////////////////////////////////////////
 
-/* virtual */ GdkWindow *
-nsWidget::GetWindowForSetBackground()
-{
-  GdkWindow * gdk_window = nsnull;
-
-  if (mWidget)
-  {
-    gdk_window = mWidget->window;
-  }
-
-  return gdk_window;
-}
-
 /* virtual */ GdkWindow *
 nsWidget::GetRenderWindow(GtkObject * aGtkWidget)
 {
@@ -2384,6 +2188,28 @@ nsWidget::GetRenderWindow(GtkObject * aGtkWidget)
   return renderWindow;
 }
 
+void
+nsWidget::ThemeChanged()
+{
+  // Dispatch a NS_THEMECHANGED event for each of our children, recursively
+  nsCOMPtr<nsIEnumerator> children = dont_AddRef(GetChildren());
+  if (children) {
+    nsCOMPtr<nsISupports> isupp;
+
+    while (NS_SUCCEEDED(children->CurrentItem(getter_AddRefs(isupp))) && isupp) {
+
+      nsWidget* child = NS_REINTERPRET_CAST(nsWidget*, isupp.get());
+      child->ThemeChanged();
+
+      if (NS_FAILED(children->Next())) {
+        break;
+      }
+    }
+  }
+
+  DispatchStandardEvent(NS_THEMECHANGED);
+  Invalidate(PR_FALSE);
+}
 
 //////////////////////////////////////////////////////////////////////
 // default setfont for most widgets
@@ -2451,95 +2277,6 @@ NS_IMETHODIMP nsWidget::ResetInputState()
   return NS_OK;
 }
 
-/********************** class ModalWidgetList ***********************/
-/* This silly little thing is a linked list of widgets that have been
-   declared modal, in the order of their declaration. We do this only
-   so clients can stack modal dialogs on top of each other, as they
-   are wont to do. Yes, glib keeps its own list, but we need our own copy
-   so we can temporarily disable them all when a popup control window
-   makes an appearance within a modal dialog.
-*/
-static ModalWidgetList *gModalWidgets = nsnull;
-
-ModalWidgetList::ModalWidgetList(nsWidget *aWidget) {
-  mWidget = aWidget;
-  mNext = 0;
-  mPrev = 0;
-  mLast = 0;
-}
-
-ModalWidgetList::~ModalWidgetList() {
-}
-
-PRBool ModalWidgetList::Find(nsWidget *aWidget) {
-  ModalWidgetList *next;
-
-  for (next = gModalWidgets; next && next->mWidget != aWidget; next = next->mNext)
-    ;
-  return next ? PR_TRUE : PR_FALSE;
-}
-
-void ModalWidgetList::Append(nsWidget *aWidget) {
-
-  ModalWidgetList *newElement = new ModalWidgetList(aWidget);
-
-  NS_ASSERTION(newElement, "out of memory in modal widget list creation");
-  if (!newElement)
-    return;
-
-  if (gModalWidgets) {
-    newElement->mPrev = gModalWidgets->mLast;
-    gModalWidgets->mLast->mNext = newElement;
-    gModalWidgets->mLast = newElement;
-  } else {
-    newElement->mLast = newElement;
-    gModalWidgets = newElement;
-  }
-}
-
-void ModalWidgetList::Remove(nsWidget *aWidget) {
-  NS_ASSERTION(gModalWidgets && gModalWidgets->mLast->mWidget == aWidget,
-    "removing modal widgets out of order");
-  if (gModalWidgets && gModalWidgets->mLast->mWidget == aWidget)
-    ModalWidgetList::RemoveLast();
-}
-
-void ModalWidgetList::RemoveLast() {
-  NS_ASSERTION(gModalWidgets, "removing modal widgets from empty list");
-  if (!gModalWidgets)
-    return;
-
-  ModalWidgetList *deadElement = gModalWidgets->mLast;
-  if (deadElement->mPrev) {
-    deadElement->mPrev->mNext = 0;
-    gModalWidgets->mLast = deadElement->mPrev;
-  } else
-    gModalWidgets = 0;
-  delete deadElement;
-}
-
-void ModalWidgetList::Suppress(PRBool aSuppress) {
-
-  if (!gModalWidgets)
-    return;
-
-  GtkWindow       *window;
-  ModalWidgetList *widget;
-
-  if (aSuppress)
-    for (widget = gModalWidgets->mLast; widget; widget = widget->mPrev) {
-      window = widget->mWidget->GetTopLevelWindow();
-       NS_ASSERTION(window, "non-window in modality suppression list");
-      gtk_window_set_modal(window, FALSE);
-    }
-  else
-    for (widget = gModalWidgets; widget; widget = widget->mNext) {
-      window = widget->mWidget->GetTopLevelWindow();
-      NS_ASSERTION(window, "non-window in modality suppression list");
-      gtk_window_set_modal(window, TRUE);
-    }
-}
-
 /* virtual */
 GtkWindow *nsWidget::GetTopLevelWindow(void)
 {
@@ -2549,162 +2286,6 @@ GtkWindow *nsWidget::GetTopLevelWindow(void)
     return NULL;
 }
 
-#ifdef NS_DEBUG
-
-static void setDebugWindow(void)
-{
-  gchar *text = NULL;
-  int val = 0;
-
-  text = gtk_editable_get_chars(GTK_EDITABLE(debugEntryBox), 0, -1);
-  if (!text) {
-    return;
-  }
-
-  if (strlen(text) == 0) {
-    g_print("setting value to null\n");
-    nsWidget::debugWidget = NULL;
-    g_free(text);
-    return;
-  }
-  
-  if (strlen(text) < 3) {
-    g_print("string not long enough\n");
-    return;
-  }
-
-  if (memcmp(text, "0x", 2) != 0) {
-    g_print("string must begin in 0x\n");
-    return;
-  }
-
-  sscanf(&text[2], "%x", &val);
-
-#ifdef DEBUG
-  printf("setting value to 0x%x\n", val);
-#endif
-  nsWidget::debugWidget = (nsWidget *)val;
-
-  g_free(text);
-}
-
-static void debugHandleActivate(GtkEditable *editable,
-                                gpointer user_data)
-{
-  setDebugWindow();
-}
-
-static void      debugHandleClicked (GtkButton   *button,
-                                     gpointer user_data)
-{
-  setDebugWindow();
-}
-
-static void      debugHandleToggle  (GtkToggleButton *button,
-                                     gpointer user_data)
-{
-  if (gtk_toggle_button_get_active(button) == TRUE) {
-    nsWidget::sDebugFeedback = PR_TRUE;
-  }
-  else {
-    nsWidget::sDebugFeedback = PR_FALSE;
-  }
-}
-
-static void      debugDestroyWindow (void)
-{
-  // this will destroy all of the widgets inside the window, too.
-  gtk_widget_destroy(debugTopLevel);
-  debugTopLevel = NULL;
-  debugBox = NULL;
-  debugEntryBox = NULL;
-  debugButton = NULL;
-  debugCheckBox = NULL;
-  nsWidget::debugWidget = NULL;  
-}
-
-static void      debugSetupWindow   (void)
-{
-  PRBool   enable_window = PR_FALSE;
-  nsresult rv;
-  
-  debugCheckedDebugWindow = PR_TRUE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
-  if (NS_SUCCEEDED(rv) && (prefs)) {
-    rv = prefs->GetBoolPref(debugPrefName, &enable_window);
-    if (NS_SUCCEEDED(rv) && enable_window) {
-      debugTopLevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_signal_connect(GTK_OBJECT(debugTopLevel),
-                         "delete_event",
-                         GTK_SIGNAL_FUNC(debugHandleWindowClose),
-                         NULL);
-      
-      debugBox = gtk_hbox_new(PR_FALSE, 0);
-      gtk_container_add(GTK_CONTAINER(debugTopLevel), debugBox);
-      
-      debugEntryBox = gtk_entry_new();
-      gtk_box_pack_start_defaults(GTK_BOX(debugBox), debugEntryBox);
-      gtk_signal_connect(GTK_OBJECT(debugEntryBox), "activate",
-                         GTK_SIGNAL_FUNC(debugHandleActivate), NULL);
-      
-      debugButton = gtk_button_new_with_label("Set Window");
-      gtk_box_pack_start_defaults(GTK_BOX(debugBox), debugButton);
-      gtk_signal_connect(GTK_OBJECT(debugButton), "clicked",
-                         GTK_SIGNAL_FUNC(debugHandleClicked), NULL);
-
-      debugCheckBox = gtk_check_button_new_with_label("Debug Feedback");
-      gtk_box_pack_start_defaults(GTK_BOX(debugBox), debugCheckBox);
-      gtk_signal_connect(GTK_OBJECT(debugCheckBox), "toggled",
-                         GTK_SIGNAL_FUNC(debugHandleToggle), NULL);
-      
-      gtk_widget_show_all(debugTopLevel);
-    }
-  }
-}
-
-static int debugWindowPrefChanged (const char *newpref, void *data)
-{
-  PRBool enable_window;
-  nsresult rv;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
-  if (NS_SUCCEEDED(rv) && (prefs)) {
-    rv = prefs->GetBoolPref(debugPrefName, &enable_window);
-    if (NS_SUCCEEDED(rv) && enable_window) {
-      if (!debugTopLevel) {
-        // this will trigger the creation of the window
-        debugCheckedDebugWindow = PR_FALSE;
-        debugSetupWindow();
-      }
-    }
-    else if (NS_SUCCEEDED(rv) && (!enable_window)) {
-      if (debugTopLevel) {
-        debugDestroyWindow();
-      }
-    }
-  }
-  return 0;
-}
-
-static void      debugRegisterCallback  (void)
-{
-  nsresult rv;
-  
-  // make sure we don't call in here again
-  debugCallbackRegistered = PR_TRUE;
-  nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID, &rv));
-  if (NS_SUCCEEDED(rv)) {
-    rv = prefs->RegisterCallback(debugPrefName, debugWindowPrefChanged, NULL);
-  }
-}
-
-static gint debugHandleWindowClose(GtkWidget *window, void *data)
-{
-  debugDestroyWindow();
-  return TRUE;
-}
-
-#endif /* NS_DEBUG */
-
 void nsWidget::IMECommitEvent(GdkEventKey *aEvent) {
   NS_ASSERTION(0, "nsWidget::IMECommitEvent() shouldn't be called!\n");
 }
