@@ -39,14 +39,12 @@
 
 #include "nsWindow.h"
 #include "nsMacWindow.h"
+#include "nsCRT.h"
 #include "prinrval.h"
 
 #include <ToolUtils.h>
-#include <DiskInit.h>
-#include <LowMem.h>
 #include <TextServices.h>
 #include <UnicodeConverter.h>
-#include <Script.h>
 
 #include "nsCarbonHelpers.h"
 #include "nsIRollupListener.h"
@@ -91,9 +89,11 @@ PRBool	nsMacEventHandler::sMouseInWidgetHit = PR_FALSE;
 
 nsMacEventDispatchHandler	gEventDispatchHandler;
 
-
 static nsEventStatus HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta,
                                           Point inMouseLoc, nsIWidget* inWidget ) ;
+static void ConvertKeyEventToContextMenuEvent(const nsKeyEvent* inKeyEvent, nsMouseEvent* outCMEvent);
+static inline PRBool IsContextMenuKey(const nsKeyEvent& inKeyEvent);
+
 
 #if !TARGET_CARBON
 //
@@ -101,7 +101,10 @@ static nsEventStatus HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inBy
 //
 // Called from ::TrackControl(), this senses which part of the phantom
 // scrollbar the click from the wheelMouse driver was in and sends
-// the correct NS_MOUSE_SCROLL event into Gecko
+// the correct NS_MOUSE_SCROLL event into Gecko. We have to retrieve the
+// mouse location from the event dispatcher because it will
+// just be the location of the phantom scrollbar, not actually the real
+// mouse position.
 //
 static pascal void ScrollActionProc(ControlHandle ctrl, ControlPartCode partCode)
 {
@@ -113,15 +116,18 @@ static pascal void ScrollActionProc(ControlHandle ctrl, ControlPartCode partCode
 		case kControlPageDownPart:
 		  PhantomScrollbarData* data = NS_REINTERPRET_CAST(PhantomScrollbarData*, ::GetControlReference(ctrl));
 		  if ( data && (data->mWidgetToGetEvent || gEventDispatchHandler.GetActive()) ) {
-        Point mouseLoc;
-        ::GetMouse ( &mouseLoc );
+		    WindowRef window = (**ctrl).contrlOwner;
+        StPortSetter portSetter(window);
+        StOriginSetter originSetter(window);
         PRBool scrollByLine = !(partCode == kControlPageUpPart || partCode == kControlPageDownPart);
         PRInt32 delta = 
           (partCode == kControlUpButtonPart || partCode == kControlPageUpPart) ? -1 : 1;
       	nsIWidget* widget = data->mWidgetToGetEvent ? 
                               data->mWidgetToGetEvent : gEventDispatchHandler.GetActive();
         
-        HandleScrollEvent ( kEventMouseWheelAxisY, scrollByLine, delta, mouseLoc, widget );
+        Point thePoint = gEventDispatchHandler.GetGlobalPoint();
+        ::GlobalToLocal(&thePoint);
+        HandleScrollEvent ( kEventMouseWheelAxisY, scrollByLine, delta, thePoint, widget );
       }
       break;
   }
@@ -133,12 +139,17 @@ static pascal void ScrollActionProc(ControlHandle ctrl, ControlPartCode partCode
 // HandleScrollEvent
 //
 // Actually dispatch the mouseWheel scroll event to the appropriate widget. If |inByLine| is false,
-// then scroll by a full page.
+// then scroll by a full page. |inMouseLoc| is in OS local coordinates. We convert it to widget-relative
+// coordinates before sending it into Gecko.
 //
 static nsEventStatus
 HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta,
                      Point inMouseLoc, nsIWidget* inWidget )
 {
+  NS_ASSERTION(inWidget, "HandleScrollEvent doesn't work with a null widget");
+  if (!inWidget)
+    return nsEventStatus_eIgnore;
+
   nsMouseScrollEvent scrollEvent;
   
   scrollEvent.scrollFlags = 
@@ -146,6 +157,15 @@ HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta
   if ( !inByLine )
     scrollEvent.scrollFlags |= nsMouseScrollEvent::kIsFullPage;
   
+  // convert window-relative (local) mouse coordinates to widget-relative
+  // coords for Gecko.
+  nsPoint mouseLocRelativeToWidget(inMouseLoc.h, inMouseLoc.v);
+  nsRect bounds;
+  inWidget->GetBounds(bounds);
+  nsPoint widgetOrigin(bounds.x, bounds.y);
+  inWidget->ConvertToDeviceCoordinates(widgetOrigin.x, widgetOrigin.y);
+  mouseLocRelativeToWidget.MoveBy(-widgetOrigin.x, -widgetOrigin.y);
+		
   scrollEvent.eventStructType = NS_MOUSE_SCROLL_EVENT;
   scrollEvent.isShift = PR_FALSE;
   scrollEvent.isControl = PR_FALSE;
@@ -153,8 +173,8 @@ HandleScrollEvent ( EventMouseWheelAxis inAxis, PRBool inByLine, PRInt32 inDelta
   scrollEvent.isAlt = PR_FALSE;
 	scrollEvent.message = NS_MOUSE_SCROLL;
   scrollEvent.delta = inDelta;
-	scrollEvent.point.x = inMouseLoc.h;
-	scrollEvent.point.y = inMouseLoc.v;
+	scrollEvent.point.x = mouseLocRelativeToWidget.x;
+	scrollEvent.point.y = mouseLocRelativeToWidget.y;
 	scrollEvent.time = PR_IntervalNow();
 	scrollEvent.widget = inWidget;
 	scrollEvent.nativeMsg = nsnull;
@@ -175,6 +195,9 @@ nsMacEventDispatchHandler::nsMacEventDispatchHandler()
 	mActiveWidget	= nsnull;
 	mWidgetHit		= nsnull;
 	mWidgetPointed	= nsnull;
+#if TRACK_MOUSE_LOC
+  mLastGlobalMouseLoc.h = mLastGlobalMouseLoc.v = 0;
+#endif
 }
 
 
@@ -400,6 +423,16 @@ void nsMacEventDispatchHandler::NotifyDelete(void* aDeletedObject)
 }
 
 
+#if TRACK_MOUSE_LOC
+
+void nsMacEventDispatchHandler::SetGlobalPoint(Point inPoint)
+{
+  mLastGlobalMouseLoc = inPoint;
+}
+
+#endif
+
+
 #pragma mark -
 
 static PRBool gUseUnicodeAPI = PR_FALSE;
@@ -620,11 +653,10 @@ PRBool nsMacEventHandler::DragEvent ( unsigned int aMessage, Point aMouseGlobal,
 	Point hitPointLocal = aMouseGlobal;
 	WindowRef wind = reinterpret_cast<WindowRef>(mTopLevelWidget->GetNativeData(NS_NATIVE_DISPLAY));
 	nsGraphicsUtils::SafeSetPortWindowPort(wind);
-	Rect savePortRect;
-	::GetWindowPortBounds(wind, &savePortRect);
-	::SetOrigin(0, 0);
-	::GlobalToLocal(&hitPointLocal);
-	::SetOrigin(savePortRect.left, savePortRect.top);
+	{
+	    StOriginSetter  originSetter(wind);
+    	::GlobalToLocal(&hitPointLocal);
+	}
 	nsPoint widgetHitPoint(hitPointLocal.h, hitPointLocal.v);
 
 	nsWindow* widgetHit = mTopLevelWidget->FindWidgetHit(hitPointLocal);
@@ -740,7 +772,7 @@ enum
 	kReturnKeyCode          = 0x24,
 	kPowerbookEnterKeyCode  = 0x34,     // Enter on Powerbook's keyboard is different
 	
-	kInsertKeyCode					= 0x72,				// also help key
+	kHelpKeyCode						= 0x72,				// also insert key
 	kDeleteKeyCode					= 0x75,				// also forward delete key
 	kTabKeyCode							= 0x30,
 	kBackspaceKeyCode       = 0x33,
@@ -763,71 +795,70 @@ static PRUint32 ConvertMacToRaptorKeyCode(UInt32 eventMessage, UInt32 eventModif
 	
 	switch (keyCode)
 	{
-//	case ??							:				raptorKeyCode = NS_VK_CANCEL;		break;			// don't know what this key means. Nor does joki
+//  case ??            :       raptorKeyCode = nsIDOMKeyEvent::DOM_VK_CANCEL;    break;     // don't know what this key means. Nor does joki
 
 // modifiers. We don't get separate events for these
-		case kEscapeKeyCode:				raptorKeyCode = NS_VK_ESCAPE;					break;
-		case kShiftKeyCode:					raptorKeyCode = NS_VK_SHIFT;					break;
-//		case kCommandKeyCode:       raptorKeyCode = NS_VK_META;           break;
-		case kCapsLockKeyCode:			raptorKeyCode = NS_VK_CAPS_LOCK;			break;
-		case kControlKeyCode:				raptorKeyCode = NS_VK_CONTROL;				break;
-		case kOptionkeyCode:				raptorKeyCode = NS_VK_ALT;						break;
-		case kClearKeyCode:					raptorKeyCode = NS_VK_CLEAR;					break;
+    case kShiftKeyCode:         raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SHIFT;         break;
+    case kCommandKeyCode:       raptorKeyCode = nsIDOMKeyEvent::DOM_VK_META;          break;
+    case kCapsLockKeyCode:      raptorKeyCode = nsIDOMKeyEvent::DOM_VK_CAPS_LOCK;     break;
+    case kControlKeyCode:       raptorKeyCode = nsIDOMKeyEvent::DOM_VK_CONTROL;       break;
+    case kOptionkeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_ALT;           break;
 
 // function keys
-		case kF1KeyCode:						raptorKeyCode = NS_VK_F1;							break;
-		case kF2KeyCode:						raptorKeyCode = NS_VK_F2;							break;
-		case kF3KeyCode:						raptorKeyCode = NS_VK_F3;							break;
-		case kF4KeyCode:						raptorKeyCode = NS_VK_F4;							break;
-		case kF5KeyCode:						raptorKeyCode = NS_VK_F5;							break;
-		case kF6KeyCode:						raptorKeyCode = NS_VK_F6;							break;
-		case kF7KeyCode:						raptorKeyCode = NS_VK_F7;							break;
-		case kF8KeyCode:						raptorKeyCode = NS_VK_F8;							break;
-		case kF9KeyCode:						raptorKeyCode = NS_VK_F9;							break;
-		case kF10KeyCode:						raptorKeyCode = NS_VK_F10;						break;
-		case kF11KeyCode:						raptorKeyCode = NS_VK_F11;						break;
-		case kF12KeyCode:						raptorKeyCode = NS_VK_F12;						break;
-//	case kF13KeyCode:						raptorKeyCode = NS_VK_F13;						break;		// clash with the 3 below
-//	case kF14KeyCode:						raptorKeyCode = NS_VK_F14;						break;
-//	case kF15KeyCode:						raptorKeyCode = NS_VK_F15;						break;
-		case kPauseKeyCode:					raptorKeyCode = NS_VK_PAUSE;					break;
-		case kScrollLockKeyCode:		raptorKeyCode = NS_VK_SCROLL_LOCK;		break;
-		case kPrintScreenKeyCode:		raptorKeyCode = NS_VK_PRINTSCREEN;		break;
-	
+    case kF1KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F1;            break;
+    case kF2KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F2;            break;
+    case kF3KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F3;            break;
+    case kF4KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F4;            break;
+    case kF5KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F5;            break;
+    case kF6KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F6;            break;
+    case kF7KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F7;            break;
+    case kF8KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F8;            break;
+    case kF9KeyCode:            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F9;            break;
+    case kF10KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F10;           break;
+    case kF11KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F11;           break;
+    case kF12KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F12;           break;
+//  case kF13KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F13;           break;    // clash with the 3 below
+//  case kF14KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F14;           break;
+//  case kF15KeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_F15;           break;
+    case kPauseKeyCode:         raptorKeyCode = nsIDOMKeyEvent::DOM_VK_PAUSE;         break;
+    case kScrollLockKeyCode:    raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SCROLL_LOCK;   break;
+    case kPrintScreenKeyCode:   raptorKeyCode = nsIDOMKeyEvent::DOM_VK_PRINTSCREEN;   break;
+
 // keypad
-		case kKeypad0KeyCode:				raptorKeyCode = NS_VK_NUMPAD0;				break;
-		case kKeypad1KeyCode:				raptorKeyCode = NS_VK_NUMPAD1;				break;
-		case kKeypad2KeyCode:				raptorKeyCode = NS_VK_NUMPAD2;				break;
-		case kKeypad3KeyCode:				raptorKeyCode = NS_VK_NUMPAD3;				break;
-		case kKeypad4KeyCode:				raptorKeyCode = NS_VK_NUMPAD4;				break;
-		case kKeypad5KeyCode:				raptorKeyCode = NS_VK_NUMPAD5;				break;
-		case kKeypad6KeyCode:				raptorKeyCode = NS_VK_NUMPAD6;				break;
-		case kKeypad7KeyCode:				raptorKeyCode = NS_VK_NUMPAD7;				break;
-		case kKeypad8KeyCode:				raptorKeyCode = NS_VK_NUMPAD8;				break;
-		case kKeypad9KeyCode:				raptorKeyCode = NS_VK_NUMPAD9;				break;
+    case kKeypad0KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD0;      break;
+    case kKeypad1KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD1;      break;
+    case kKeypad2KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD2;      break;
+    case kKeypad3KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD3;      break;
+    case kKeypad4KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD4;      break;
+    case kKeypad5KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD5;      break;
+    case kKeypad6KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD6;      break;
+    case kKeypad7KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD7;      break;
+    case kKeypad8KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD8;      break;
+    case kKeypad9KeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_NUMPAD9;      break;
 
-		case kKeypadMultiplyKeyCode:	raptorKeyCode = NS_VK_MULTIPLY;			break;
-		case kKeypadAddKeyCode:				raptorKeyCode = NS_VK_ADD;					break;
-		case kKeypadSubtractKeyCode:	raptorKeyCode = NS_VK_SUBTRACT;			break;
-		case kKeypadDecimalKeyCode:		raptorKeyCode = NS_VK_DECIMAL;			break;
-		case kKeypadDivideKeyCode:		raptorKeyCode = NS_VK_DIVIDE;				break;
-//	case ??								:				raptorKeyCode = NS_VK_SEPARATOR;		break;
+    case kKeypadMultiplyKeyCode: raptorKeyCode = nsIDOMKeyEvent::DOM_VK_MULTIPLY;     break;
+    case kKeypadAddKeyCode:      raptorKeyCode = nsIDOMKeyEvent::DOM_VK_ADD;          break;
+    case kKeypadSubtractKeyCode: raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SUBTRACT;     break;
+    case kKeypadDecimalKeyCode:  raptorKeyCode = nsIDOMKeyEvent::DOM_VK_DECIMAL;      break;
+    case kKeypadDivideKeyCode:   raptorKeyCode = nsIDOMKeyEvent::DOM_VK_DIVIDE;       break;
+//  case ??             :        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SEPARATOR;    break;
 
+// this may clash with vk_insert, but help key is more useful in mozilla
+    case kHelpKeyCode:          raptorKeyCode = nsIDOMKeyEvent::DOM_VK_HELP;          break;
+    case kDeleteKeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_DELETE;        break;
+    case kEscapeKeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_ESCAPE;        break;
+    case kClearKeyCode:         raptorKeyCode = nsIDOMKeyEvent::DOM_VK_CLEAR;         break;
 
-// these may clash with forward delete and help
-    case kInsertKeyCode:				raptorKeyCode = NS_VK_INSERT;					break;
-    case kDeleteKeyCode:				raptorKeyCode = NS_VK_DELETE;					break;
-
-    case kBackspaceKeyCode:     raptorKeyCode = NS_VK_BACK;           break;
-    case kTabKeyCode:           raptorKeyCode = NS_VK_TAB;            break;
-    case kHomeKeyCode:          raptorKeyCode = NS_VK_HOME;           break;
-    case kEndKeyCode:           raptorKeyCode = NS_VK_END;            break;
-		case kPageUpKeyCode:				raptorKeyCode = NS_VK_PAGE_UP;        break;
-		case kPageDownKeyCode:			raptorKeyCode = NS_VK_PAGE_DOWN;      break;
-		case kLeftArrowKeyCode:			raptorKeyCode = NS_VK_LEFT;           break;
-		case kRightArrowKeyCode:		raptorKeyCode = NS_VK_RIGHT;          break;
-		case kUpArrowKeyCode:				raptorKeyCode = NS_VK_UP;             break;
-		case kDownArrowKeyCode:			raptorKeyCode = NS_VK_DOWN;           break;
+    case kBackspaceKeyCode:     raptorKeyCode = nsIDOMKeyEvent::DOM_VK_BACK_SPACE;    break;
+    case kTabKeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_TAB;           break;
+    case kHomeKeyCode:          raptorKeyCode = nsIDOMKeyEvent::DOM_VK_HOME;          break;
+    case kEndKeyCode:           raptorKeyCode = nsIDOMKeyEvent::DOM_VK_END;           break;
+    case kPageUpKeyCode:        raptorKeyCode = nsIDOMKeyEvent::DOM_VK_PAGE_UP;       break;
+    case kPageDownKeyCode:      raptorKeyCode = nsIDOMKeyEvent::DOM_VK_PAGE_DOWN;     break;
+    case kLeftArrowKeyCode:     raptorKeyCode = nsIDOMKeyEvent::DOM_VK_LEFT;          break;
+    case kRightArrowKeyCode:    raptorKeyCode = nsIDOMKeyEvent::DOM_VK_RIGHT;         break;
+    case kUpArrowKeyCode:       raptorKeyCode = nsIDOMKeyEvent::DOM_VK_UP;            break;
+    case kDownArrowKeyCode:     raptorKeyCode = nsIDOMKeyEvent::DOM_VK_DOWN;          break;
 
 		default:
 				if ((eventModifiers & controlKey) != 0)
@@ -836,23 +867,23 @@ static PRUint32 ConvertMacToRaptorKeyCode(UInt32 eventMessage, UInt32 eventModif
 				// if we haven't gotten the key code already, look at the char code
 				switch (charCode)
 				{
-					case kReturnCharCode:				raptorKeyCode = NS_VK_RETURN;				break;
-					case kEnterCharCode:				raptorKeyCode = NS_VK_RETURN;				break;			// fix me!
-					case ' ':										raptorKeyCode = NS_VK_SPACE;				break;
-					case ';':										raptorKeyCode = NS_VK_SEMICOLON;		break;
-					case '=':										raptorKeyCode = NS_VK_EQUALS;				break;
-					case ',':										raptorKeyCode = NS_VK_COMMA;				break;
-					case '.':										raptorKeyCode = NS_VK_PERIOD;				break;
-					case '/':										raptorKeyCode = NS_VK_SLASH;				break;
-					case '`':										raptorKeyCode = NS_VK_BACK_QUOTE;		break;
-					case '{':
-					case '[':										raptorKeyCode = NS_VK_OPEN_BRACKET;	break;
-					case '\\':									raptorKeyCode = NS_VK_BACK_SLASH;		break;
-					case '}':
-					case ']':										raptorKeyCode = NS_VK_CLOSE_BRACKET;	break;
-					case '\'':
-					case '"':										raptorKeyCode = NS_VK_QUOTE;				break;
-					
+          case kReturnCharCode: raptorKeyCode = nsIDOMKeyEvent::DOM_VK_RETURN;       break;
+          case kEnterCharCode:  raptorKeyCode = nsIDOMKeyEvent::DOM_VK_RETURN;       break;    // fix me!
+          case ' ':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SPACE;        break;
+          case ';':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SEMICOLON;    break;
+          case '=':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_EQUALS;       break;
+          case ',':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_COMMA;        break;
+          case '.':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_PERIOD;       break;
+          case '/':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_SLASH;        break;
+          case '`':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_BACK_QUOTE;   break;
+          case '{':
+          case '[':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_OPEN_BRACKET; break;
+          case '\\':            raptorKeyCode = nsIDOMKeyEvent::DOM_VK_BACK_SLASH;   break;
+          case '}':
+          case ']':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_CLOSE_BRACKET; break;
+          case '\'':
+          case '"':             raptorKeyCode = nsIDOMKeyEvent::DOM_VK_QUOTE;        break;
+
 					default:
 						
 						if (charCode >= '0' && charCode <= '9')		// numerals
@@ -966,12 +997,13 @@ void nsMacEventHandler::InitializeKeyEvent(nsKeyEvent& aKeyEvent,
   //
   if (aMessage == NS_KEY_PRESS 
 		&& !aKeyEvent.isMeta 
-		&& aKeyEvent.keyCode != NS_VK_PAGE_UP && aKeyEvent.keyCode != NS_VK_PAGE_DOWN
+		&& aKeyEvent.keyCode != nsIDOMKeyEvent::DOM_VK_PAGE_UP 
+		&& aKeyEvent.keyCode != nsIDOMKeyEvent::DOM_VK_PAGE_DOWN
 		// also consider:  function keys and sole modifier keys
 	) 
 	{
     ::ObscureCursor();
-  } // if (message == NS_KEY_PRESS && !aKeyEvent.isMeta && aKeyEvent.keyCode != NS_VK_PAGE_UP && aKeyEvent.keyCode != NS_VK_PAGE_DOWN
+  }
 }
 
 
@@ -1018,7 +1050,7 @@ PRBool nsMacEventHandler::IsSpecialRaptorKey(UInt32 macKeyCode)
 		case kScrollLockKeyCode:	isSpecial = PR_TRUE; break;
 		case kPrintScreenKeyCode:	isSpecial = PR_TRUE; break;
 
-		case kInsertKeyCode:        isSpecial = PR_TRUE; break;
+		case kHelpKeyCode:					isSpecial = PR_TRUE; break;
 		case kDeleteKeyCode:				isSpecial = PR_TRUE; break;
 		case kTabKeyCode:						isSpecial = PR_TRUE; break;
 		case kBackspaceKeyCode:     isSpecial = PR_TRUE; break;
@@ -1150,11 +1182,20 @@ PRBool nsMacEventHandler::HandleKeyEvent(EventRecord& aOSEvent)
 			if (checkFocusedWidget != focusedWidget)
 				return result;
 
-			//if (result == PR_FALSE) // continue processing???  talk to Tague about this (key event spec)
-			{
-				InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_PRESS);
-				result = focusedWidget->DispatchWindowEvent(keyEvent);
-			}
+      InitializeKeyEvent(keyEvent,aOSEvent,focusedWidget,NS_KEY_PRESS);
+
+      // before we dispatch this key, check if it's the contextmenu key.
+      // If so, send a context menu event instead.
+      if ( IsContextMenuKey(keyEvent) ) {
+        nsMouseEvent contextMenuEvent;
+        ConvertKeyEventToContextMenuEvent(&keyEvent, &contextMenuEvent);
+        result = focusedWidget->DispatchWindowEvent(contextMenuEvent);
+        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      }
+      else {
+        result = focusedWidget->DispatchWindowEvent(keyEvent);
+        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      }
 			break;
 		
 		case autoKey:
@@ -1166,6 +1207,45 @@ PRBool nsMacEventHandler::HandleKeyEvent(EventRecord& aOSEvent)
 	return result;
 }
 
+
+
+//
+// ConvertKeyEventToContextMenuEvent
+//
+// Take a key event and all of its attributes at convert it into
+// a context menu event. We want just about everything (focused
+// widget, etc) but a few things need to bt tweaked.
+//
+static void
+ConvertKeyEventToContextMenuEvent(const nsKeyEvent* inKeyEvent, nsMouseEvent* outCMEvent)
+{
+  *(nsInputEvent*)outCMEvent = *(nsInputEvent*)inKeyEvent;
+  
+  outCMEvent->eventStructType = NS_MOUSE_EVENT;
+  outCMEvent->message = NS_CONTEXTMENU_KEY;
+  outCMEvent->isShift = outCMEvent->isControl = outCMEvent->isAlt = outCMEvent->isMeta = PR_FALSE;
+  
+  outCMEvent->clickCount = 0;
+  outCMEvent->acceptActivation = PR_FALSE;
+}
+
+
+//
+// IsContextMenuKey
+//
+// Check if the event should be a context menu event instead. Currently,
+// that is a control-space.
+//
+static inline PRBool
+IsContextMenuKey(const nsKeyEvent& inKeyEvent)
+{
+  enum { kContextMenuKey = ' ' } ;
+
+  return ( inKeyEvent.charCode == kContextMenuKey && inKeyEvent.isControl &&
+            !inKeyEvent.isShift && !inKeyEvent.isMeta && !inKeyEvent.isAlt );
+}
+
+
 //-------------------------------------------------------------------------
 //
 // HandleUKeyEvent
@@ -1193,10 +1273,22 @@ PRBool nsMacEventHandler::HandleUKeyEvent(PRUnichar* text, long charCount, Event
       if (keyEvent.isShift && keyEvent.charCode <= 'z' && keyEvent.charCode >= 'a') 
         keyEvent.charCode -= 32;
 
-      result = focusedWidget->DispatchWindowEvent(keyEvent);
-      NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      // before we dispatch a key, check if it's the context menu key.
+      // If so, send a context menu event instead.
+      if ( IsContextMenuKey(keyEvent) ) {
+        nsMouseEvent contextMenuEvent;
+        ConvertKeyEventToContextMenuEvent(&keyEvent, &contextMenuEvent);
+        result = focusedWidget->DispatchWindowEvent(contextMenuEvent);
+        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      }
+      else {
+        // command / shift keys, etc. only send once
+        result = focusedWidget->DispatchWindowEvent(keyEvent);
+        NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
+      }
     }
-  } else {
+  }
+  else {
     // command / shift keys, etc. only send once
     result = focusedWidget->DispatchWindowEvent(keyEvent);
     NS_ASSERTION(NS_SUCCEEDED(result), "cannot DispatchWindowEvent");
@@ -1324,9 +1416,10 @@ PRBool nsMacEventHandler::ResizeEvent ( WindowRef inWindow )
 	::GetWindowPortBounds ( inWindow, &macRect );
 	::LocalToGlobal(&topLeft(macRect));
 	::LocalToGlobal(&botRight(macRect));
-	mTopLevelWidget->Resize(macRect.right - macRect.left + 1, macRect.bottom - macRect.top + 1, PR_FALSE);
+	mTopLevelWidget->Resize(macRect.right - macRect.left, macRect.bottom - macRect.top, PR_FALSE);
 	if (nsnull != gRollupListener && (nsnull != gRollupWidget) )
 		gRollupListener->Rollup();
+	mTopLevelWidget->UserStateForResize(); // size a zoomed window and it's no longer zoomed
 
 	return PR_TRUE;
 }
@@ -1421,9 +1514,6 @@ PRBool nsMacEventHandler::HandleMouseDownEvent(EventRecord&	aOSEvent)
 			macPoint = topLeft(portRect);
 			::LocalToGlobal(&macPoint);
 			mTopLevelWidget->MoveToGlobalPoint(macPoint.h, macPoint.v);
-			if (nsnull != gRollupListener && (nsnull != gRollupWidget) ) {
-				gRollupListener->Rollup();
-			}
 			break;
 		}
 
@@ -1518,15 +1608,6 @@ PRBool nsMacEventHandler::HandleMouseDownEvent(EventRecord&	aOSEvent)
 		{
 			gEventDispatchHandler.DispatchSizeModeEvent(mTopLevelWidget,
 				partCode == inZoomIn ? nsSizeMode_Normal : nsSizeMode_Maximized);
-
-			// Now that we have found the partcode it is ok to actually zoom the window
-			ZoomWindow(whichWindow, partCode, (whichWindow == FrontWindow()));
-			
-			Rect macRect;
-			::GetWindowPortBounds(whichWindow, &macRect);
-			::LocalToGlobal(&topLeft(macRect));
-			::LocalToGlobal(&botRight(macRect));
-			mTopLevelWidget->Resize(macRect.right - macRect.left, macRect.bottom - macRect.top, PR_FALSE);
 			break;
 		}
 
@@ -1612,7 +1693,10 @@ PRBool nsMacEventHandler::HandleMouseMoveEvent( EventRecord& aOSEvent )
 				retVal |= lastWidgetPointed->DispatchMouseEvent(mouseEvent);
 			}
 
-			gEventDispatchHandler.SetWidgetPointed(widgetPointed);
+      gEventDispatchHandler.SetWidgetPointed(widgetPointed);
+#if TRACK_MOUSE_LOC
+      gEventDispatchHandler.SetGlobalPoint(aOSEvent.where);
+#endif
 
 			if (widgetPointed)
 			{
@@ -1676,21 +1760,13 @@ void nsMacEventHandler::ConvertOSEventToMouseEvent(
 	}
 
 	// get the widget hit and the hit point inside that widget
-	Point hitPoint = aOSEvent.where;
-  nsWindowType wtype;
-  mTopLevelWidget->GetWindowType(wtype);
-	PRBool topLevelIsAPopup = (wtype == eWindowType_popup);
-	WindowRef wind = reinterpret_cast<WindowRef>(mTopLevelWidget->GetNativeData(NS_NATIVE_DISPLAY));
-	nsGraphicsUtils::SafeSetPortWindowPort(wind);
-	Rect savePortRect;
-	::GetWindowPortBounds(wind, &savePortRect);
-	::SetOrigin(0, 0);
-	::GlobalToLocal(&hitPoint);
-	::SetOrigin(savePortRect.left, savePortRect.top);
-	nsPoint widgetHitPoint(hitPoint.h, hitPoint.v);
+	nsWindowType wtype;
+	mTopLevelWidget->GetWindowType(wtype);
+	PRBool          topLevelIsAPopup = (wtype == eWindowType_popup);
+	WindowRef       eventTargetWindow = reinterpret_cast<WindowRef>(mTopLevelWidget->GetNativeData(NS_NATIVE_DISPLAY));
 
-	WindowPtr windowThatHasEvent = nsnull;
-	short partCode = ::FindWindow ( aOSEvent.where, &windowThatHasEvent );
+	WindowPtr       windowThatHasEvent = nsnull;
+	ControlPartCode partCode = ::FindWindow(aOSEvent.where, &windowThatHasEvent);
 
 	// if the mouse button is still down, send events to the last widget hit unless the
 	// new event is in a popup window.
@@ -1698,23 +1774,36 @@ void nsMacEventHandler::ConvertOSEventToMouseEvent(
 	nsWindow* widgetHit = nsnull;
 	if (lastWidgetHit)
 	{
-	  // make sure we in the same window as where we started before we go assuming
-	  // that we know where the event will go.
-	  WindowRef lastWind = reinterpret_cast<WindowRef>(lastWidgetHit->GetNativeData(NS_NATIVE_DISPLAY));
-	  PRBool eventInSameWindowAsLastEvent = (windowThatHasEvent == lastWind);
-	  if ( eventInSameWindowAsLastEvent || !topLevelIsAPopup ) {	  
- 		  if (::StillDown() || aMessage == NS_MOUSE_LEFT_BUTTON_UP)
-	  		widgetHit = lastWidgetHit;
-	  	else
-	  	{
-	 	  	// Some widgets can eat mouseUp events (text widgets in TEClick, sbars in TrackControl).
-	 	  	// In that case, stop considering this widget as being still hit.
-	 	  	gEventDispatchHandler.SetWidgetHit(nsnull);
-	  	}
-	  }
+        // make sure we in the same window as where we started before we go assuming
+        // that we know where the event will go.
+        WindowRef   lastWind = reinterpret_cast<WindowRef>(lastWidgetHit->GetNativeData(NS_NATIVE_DISPLAY));
+        PRBool      eventInSameWindowAsLastEvent = (windowThatHasEvent == lastWind);
+        if ( eventInSameWindowAsLastEvent || !topLevelIsAPopup ) {	  
+            if (::StillDown() || aMessage == NS_MOUSE_LEFT_BUTTON_UP)
+            {
+                widgetHit = lastWidgetHit;
+                eventTargetWindow = lastWind;   // make sure we use the correct window to fix the coords
+            }
+            else
+            {
+                // Some widgets can eat mouseUp events (text widgets in TEClick, sbars in TrackControl).
+                // In that case, stop considering this widget as being still hit.
+                gEventDispatchHandler.SetWidgetHit(nsnull);
+            }
+        }
 	}
 
-	// if the mouse is in the grow box, pretend like it has left the window
+	Point hitPoint = aOSEvent.where;
+	nsGraphicsUtils::SafeSetPortWindowPort(eventTargetWindow);
+
+	{
+	    StOriginSetter  originSetter(eventTargetWindow);
+    	::GlobalToLocal(&hitPoint);     // gives the mouse loc in local coords of the hit window
+	}
+	
+	nsPoint widgetHitPoint(hitPoint.h, hitPoint.v);
+
+	// if the mouse is in the grow box, pretend that it has left the window
 	if ( partCode != inGrow ) {
 		if (! widgetHit)
 			widgetHit = mTopLevelWidget->FindWidgetHit(hitPoint);
@@ -1729,33 +1818,33 @@ void nsMacEventHandler::ConvertOSEventToMouseEvent(
 		}
 	}
 
-  // if we haven't found anything and we're tracking the mouse for a popup, then
-  // it's probable that the coordinates are just negative and we were dispatched
-  // here just cuz we're the top window in the app right now. In that case, just
-  // set the widget hit to this one. It's harmless (I hope), and it avoids asserts
-  // in the view code about null widgets.
-  if ( !widgetHit && topLevelIsAPopup && (hitPoint.h < 0 || hitPoint.v < 0) )
-    widgetHit = mTopLevelWidget;
-	
-	// nsEvent
-	aMouseEvent.eventStructType = NS_MOUSE_EVENT;
-	aMouseEvent.message		= aMessage;
-	aMouseEvent.point			= widgetHitPoint;
-	aMouseEvent.time			= PR_IntervalNow();
+    // if we haven't found anything and we're tracking the mouse for a popup, then
+    // it's probable that the coordinates are just negative and we were dispatched
+    // here just cuz we're the top window in the app right now. In that case, just
+    // set the widget hit to this one. It's harmless (I hope), and it avoids asserts
+    // in the view code about null widgets.
+    if ( !widgetHit && topLevelIsAPopup && (hitPoint.h < 0 || hitPoint.v < 0) )
+        widgetHit = mTopLevelWidget;
+		
+    // nsEvent
+    aMouseEvent.eventStructType = NS_MOUSE_EVENT;
+    aMouseEvent.message     = aMessage;
+    aMouseEvent.point       = widgetHitPoint;
+    aMouseEvent.time        = PR_IntervalNow();
 
-	// nsGUIEvent
-	aMouseEvent.widget		= widgetHit;
-	aMouseEvent.nativeMsg	= (void*)&aOSEvent;
+    // nsGUIEvent
+    aMouseEvent.widget      = widgetHit;
+    aMouseEvent.nativeMsg   = (void*)&aOSEvent;
 
-	// nsInputEvent
-	aMouseEvent.isShift		= ((aOSEvent.modifiers & shiftKey) != 0);
-	aMouseEvent.isControl	= ((aOSEvent.modifiers & controlKey) != 0);
-	aMouseEvent.isAlt			= ((aOSEvent.modifiers & optionKey) != 0);
-	aMouseEvent.isMeta	= ((aOSEvent.modifiers & cmdKey) != 0);
+    // nsInputEvent
+    aMouseEvent.isShift     = ((aOSEvent.modifiers & shiftKey) != 0);
+    aMouseEvent.isControl   = ((aOSEvent.modifiers & controlKey) != 0);
+    aMouseEvent.isAlt       = ((aOSEvent.modifiers & optionKey) != 0);
+    aMouseEvent.isMeta      = ((aOSEvent.modifiers & cmdKey) != 0);
 
-	// nsMouseEvent
-	aMouseEvent.clickCount = sLastClickCount;
-	aMouseEvent.acceptActivation = PR_TRUE;
+    // nsMouseEvent
+    aMouseEvent.clickCount  = sLastClickCount;
+    aMouseEvent.acceptActivation = PR_TRUE;
 }
 
 //-------------------------------------------------------------------------
@@ -1778,13 +1867,13 @@ nsresult nsMacEventHandler::HandleOffsetToPosition(long offset,Point* thePoint)
 {
 	thePoint->v = mIMEPos.y;
 	thePoint->h = mIMEPos.x;
-	printf("local (x,y) = (%d, %d)\n", thePoint->h, thePoint->v);
+	//printf("local (x,y) = (%d, %d)\n", thePoint->h, thePoint->v);
 	WindowRef wind = reinterpret_cast<WindowRef>(mTopLevelWidget->GetNativeData(NS_NATIVE_DISPLAY));
 	nsGraphicsUtils::SafeSetPortWindowPort(wind);
 	Rect savePortRect;
 	::GetWindowPortBounds(wind, &savePortRect);
 	::LocalToGlobal(thePoint);
-	printf("global (x,y) = (%d, %d)\n", thePoint->h, thePoint->v);
+	//printf("global (x,y) = (%d, %d)\n", thePoint->h, thePoint->v);
 
 	return PR_TRUE;
 }
