@@ -52,6 +52,7 @@
 #include "nsMacResources.h"
 #include "nsRegionMac.h"
 #include "nsIRollupListener.h"
+#include "nsCRT.h"
 
 #if TARGET_CARBON
 #include <CFString.h>
@@ -59,6 +60,7 @@
 
 #include <Gestalt.h>
 #include <Quickdraw.h>
+#include <MacWindows.h>
 
 #if UNIVERSAL_INTERFACES_VERSION < 0x0340
 enum {
@@ -67,7 +69,7 @@ enum {
 const UInt32 kWindowLiveResizeAttribute = (1L << 28);
 #endif
 
-static const char *sScreenManagerContractID = "@mozilla.org/gfx/screenmanager;1";
+static const char sScreenManagerContractID[] = "@mozilla.org/gfx/screenmanager;1";
 
 // from MacHeaders.c
 #ifndef topLeft
@@ -85,7 +87,6 @@ extern nsIWidget         * gRollupWidget;
 pascal long BorderlessWDEF ( short inCode, WindowPtr inWindow, short inMessage, long inParam ) ;
 long CallSystemWDEF ( short inCode, WindowPtr inWindow, short inMessage, long inParam ) ;
 #endif
-PRBool OnMacOSX();
 
 #define kWindowPositionSlop 20
 
@@ -176,7 +177,8 @@ nsMacWindow :: DragReceiveHandler (WindowPtr theWindow, void *handlerRefCon,
 } // DragReceiveHandler
 
 
-NS_IMPL_ISUPPORTS_INHERITED3(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac, nsPIEventSinkStandalone);
+NS_IMPL_ISUPPORTS_INHERITED4(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac, nsPIEventSinkStandalone, 
+                                          nsIMacTextInputEventSink);
 
 
 //-------------------------------------------------------------------------
@@ -187,15 +189,18 @@ NS_IMPL_ISUPPORTS_INHERITED3(nsMacWindow, Inherited, nsIEventSink, nsPIWidgetMac
 nsMacWindow::nsMacWindow() : Inherited()
   , mWindowMadeHere(PR_FALSE)
   , mIsDialog(PR_FALSE)
+  , mIsSheet(PR_FALSE)
+  , mIgnoreDeactivate(PR_FALSE)
   , mMacEventHandler(nsnull)
   , mAcceptsActivation(PR_TRUE)
   , mIsActive(PR_FALSE)
   , mZoomOnShow(PR_FALSE)
+  , mZooming(PR_FALSE)
+  , mResizeIsFromUs(PR_FALSE)
 #if !TARGET_CARBON
   , mPhantomScrollbar(nsnull)
   , mPhantomScrollbarData(nsnull)
 #endif
-  , mResizeIsFromUs(PR_FALSE)
 {
   mMacEventHandler.reset(new nsMacEventHandler(this));
   WIDGET_SET_CLASSNAME("nsMacWindow");  
@@ -315,6 +320,7 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
         break;
 
       case eWindowType_dialog:
+        mIsTopWidgetWindow = PR_TRUE;
         if (aInitData)
         {
           // Prior to Carbon, defProcs were solely about appearance. If told to create a dialog,
@@ -348,6 +354,24 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
               break;
             
             default:
+#if TARGET_CARBON
+              if (nsToolkit::OnMacOSX() && aParent && (aInitData->mBorderStyle & eBorderStyle_sheet))
+              {
+                nsWindowType parentType;
+                aParent->GetWindowType(parentType);
+                if (parentType != eWindowType_invisible)
+                {
+                  // Mac OS X sheet support
+                  mIsSheet = PR_TRUE;
+                  wDefProcID = kWindowSheetProc;
+                  if (aInitData->mBorderStyle & eBorderStyle_resizeh)
+                  {
+                    resizable = true;
+                  }
+                }
+              }
+              else
+#endif
               // we ignore the close flag here, since mac dialogs should never have a close box.
               switch(aInitData->mBorderStyle & (eBorderStyle_resizeh | eBorderStyle_title))
               {
@@ -387,6 +411,7 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
         break;
 
       case eWindowType_toplevel:
+        mIsTopWidgetWindow = PR_TRUE;
         if (aInitData &&
           aInitData->mBorderStyle != eBorderStyle_all &&
           aInitData->mBorderStyle != eBorderStyle_default &&
@@ -467,9 +492,18 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
   // and adjust for any adjustment required to requested window bottom
   nsRect bounds(0, 0, aRect.width, aRect.height - bottomPinDelta);
 
+  // this code used to pass in null for aParent when init'ing the base class
+  // (Mac OS 9.x and earlier has no use for aParent regarding window modality)
+  // but for Mac OS X, we need a valid aParent IFF we have a sheet
+#if TARGET_CARBON
+    if ((!aInitData) || (aInitData->mBorderStyle == eBorderStyle_default) ||
+      !(aInitData->mBorderStyle & eBorderStyle_sheet))
+#endif
+      aParent = nil;
+
   // init base class
   // (note: aParent is ignored. Mac (real) windows don't want parents)
-  Inherited::StandardCreate(nil, bounds, aHandleEventFunction, aContext, aAppShell, theToolkit, aInitData);
+  Inherited::StandardCreate(aParent, bounds, aHandleEventFunction, aContext, aAppShell, theToolkit, aInitData);
 
   // there is a lot of work that we only want to do if we actually created 
   // the window. Embedding apps would be really torqed off if we mucked around
@@ -496,7 +530,6 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
     }
     else if ( mWindowType == eWindowType_toplevel ) {
       // enable toolbar collapse/expand box 
-      WindowAttributes removeAttributes = kWindowNoAttributes;
       ::ChangeWindowAttributes(mWindowPtr, kWindowToolbarButtonAttribute, kWindowNoAttributes );
       
       EventTypeSpec scrollEventList[] = { {kEventClassMouse, kEventMouseWheelMoved} };
@@ -504,28 +537,45 @@ nsresult nsMacWindow::StandardCreate(nsIWidget *aParent,
         // note, passing NULL as the final param to IWEH() causes the UPP to be disposed automatically
         // when the event target (the window) goes away. See CarbonEvents.h for info.
       
-      NS_ASSERTION(err == noErr, "Couldn't install Carbon Event handlers");
+      NS_ASSERTION(err == noErr, "Couldn't install Carbon Scroll Event handlers");
     }
     else if ( mWindowType == eWindowType_invisible ) {
       // for an invisible window (like the hidden window), remove it from the
       // window menu and make sure the position-constrain handler is in place so
       // the window server doesn't try to move it back on screen.
-      ::ChangeWindowAttributes(mWindowPtr, kWindowNoAttributes, kWindowInWindowMenuAttribute );
-      
-      EventTypeSpec windEventList[] = { {kEventClassWindow, kEventWindowConstrain} };
-      OSStatus err = ::InstallWindowEventHandler ( mWindowPtr, NewEventHandlerUPP(WindowEventHandler), 1, windEventList, this, NULL );
-    
-      NS_ASSERTION(err == noErr, "Couldn't install Carbon constrain event handler");    
+      ::ChangeWindowAttributes(mWindowPtr, kWindowNoAttributes, kWindowInWindowMenuAttribute );      
+    }
+
+    if (mIsSheet)
+    {
+      // Mac OS X sheet support
+      if ( resizable )
+      {
+        ::ChangeWindowAttributes ( mWindowPtr, kWindowResizableAttribute, kWindowNoAttributes );
+      }
+      ::SetWindowClass(mWindowPtr, kSheetWindowClass);
+      EventTypeSpec windEventList[] = { {kEventClassWindow, kEventWindowUpdate},
+                                        {kEventClassWindow, kEventWindowDrawContent} };
+      OSStatus err1 = ::InstallWindowEventHandler ( mWindowPtr,
+          NewEventHandlerUPP(WindowEventHandler), 2, windEventList, this, NULL );
     }
     
     // Setup the live window resizing if appropriate
-    if ( resizable ) {
+    if ( resizable ) 
       ::ChangeWindowAttributes ( mWindowPtr, kWindowLiveResizeAttribute, kWindowNoAttributes );
+      
+    // Since we can only call IWEH() once for each event class such as kEventClassWindow, we register all the event types that
+    // we are going to handle here
+    const EventTypeSpec windEventList[] = {
+                                            {kEventClassWindow, kEventWindowBoundsChanged}, // to enable live resizing
+                                            {kEventClassWindow, kEventWindowCollapse},      // to roll up popups when we're minimized
+                                            {kEventClassWindow, kEventWindowConstrain}      // to keep invisible windows off the screen
+                                          };
+    OSStatus err = ::InstallWindowEventHandler ( mWindowPtr, NewEventHandlerUPP(WindowEventHandler),
+                                                 GetEventTypeCount(windEventList), windEventList, this, NULL );
+    NS_ASSERTION(err == noErr, "Couldn't install Carbon window event handler");
     
-      EventTypeSpec windEventList[] = { {kEventClassWindow, kEventWindowBoundsChanged} };
-      OSStatus err = ::InstallWindowEventHandler ( mWindowPtr, NewEventHandlerUPP(WindowEventHandler), 1, windEventList, this, NULL );    
-      NS_ASSERTION(err == noErr, "Couldn't install Carbon resize event handler");
-    }  
+
 #endif
   
 #if !TARGET_CARBON
@@ -606,8 +656,9 @@ nsMacWindow :: ScrollEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
   if ( err1 == noErr && err2 == noErr && err3 == noErr ) {
     nsMacWindow* self = NS_REINTERPRET_CAST(nsMacWindow*, userData);
     if ( self ) {
-      // convert mouse to local coordinates since that's how gecko wants them
+      // convert mouse to local coordinates since that's how the event handler wants them
       StPortSetter portSetter(self->mWindowPtr);
+      StOriginSetter originSetter(self->mWindowPtr);
       ::GlobalToLocal(&mouseLoc);
       self->mMacEventHandler->Scroll ( axis, delta, mouseLoc );
     }
@@ -655,12 +706,29 @@ nsMacWindow :: WindowEventHandler ( EventHandlerCallRef inHandlerChain, EventRef
         nsMacWindow* self = NS_REINTERPRET_CAST(nsMacWindow*, userData);
         if ( self ) {
           if ( self->mWindowType != eWindowType_invisible )
-            retVal = ::CallNextEventHandler( inHandlerChain, inEvent );
+            retVal = ::CallNextEventHandler(inHandlerChain, inEvent);
           else
             retVal = noErr;  // consume the event for the hidden window
         }
         break;
       }      
+
+      case kEventWindowUpdate:
+      case kEventWindowDrawContent:
+      {
+        nsMacWindow *self = NS_REINTERPRET_CAST(nsMacWindow *, userData);
+        if (self) self->mMacEventHandler->UpdateEvent();
+      }
+      break;
+
+      // roll up popups when we're minimized
+      case kEventWindowCollapse:
+      {
+        if ( gRollupListener && gRollupWidget )
+          gRollupListener->Rollup();        
+        retVal = ::CallNextEventHandler(inHandlerChain, inEvent);
+      }
+      break;
         
       default:
         // do nothing...
@@ -744,12 +812,41 @@ nsMacWindow :: RemoveBorderlessDefProc ( WindowPtr inWindow )
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsMacWindow::Show(PRBool bState)
 {
+#if TARGET_CARBON
+  // Mac OS X sheet support
+  nsIWidget *parentWidget = mParent;
+  nsCOMPtr<nsPIWidgetMac> piParentWidget ( do_QueryInterface(parentWidget) );
+  WindowRef parentWindowRef = (parentWidget) ?
+    reinterpret_cast<WindowRef>(parentWidget->GetNativeData(NS_NATIVE_DISPLAY)) : nsnull;
+#endif
+
   Inherited::Show(bState);
   
   // we need to make sure we call ::Show/HideWindow() to generate the 
   // necessary activate/deactivate events. Calling ::ShowHide() is
   // not adequate, unless we don't want activation (popups). (pinkerton).
   if ( bState && !mBounds.IsEmpty() ) {
+#if TARGET_CARBON
+    if ( mIsSheet && parentWindowRef ) {
+        WindowPtr top = GetWindowTop(parentWindowRef);
+        if (piParentWidget)
+        {
+          piParentWidget->SetIgnoreDeactivate(PR_TRUE);
+          PRBool sheetFlag = PR_FALSE;
+          if (parentWindowRef &&
+            NS_SUCCEEDED(piParentWidget->GetIsSheet(&sheetFlag)) && (sheetFlag))
+          {
+            ::GetSheetWindowParent(parentWindowRef, &top);
+            ::HideSheetWindow(parentWindowRef);
+          }
+        }
+        ::ShowSheetWindow(mWindowPtr, top);
+        UpdateWindowMenubar(parentWindowRef, PR_FALSE);
+        gEventDispatchHandler.DispatchGuiEvent(this, NS_GOTFOCUS);
+        gEventDispatchHandler.DispatchGuiEvent(this, NS_ACTIVATE);
+    }
+    else
+#endif
     if ( mAcceptsActivation )
       ::ShowWindow(mWindowPtr);
     else {
@@ -773,10 +870,137 @@ NS_IMETHODIMP nsMacWindow::Show(PRBool bState)
       NS_IF_RELEASE(gRollupListener);
       NS_IF_RELEASE(gRollupWidget);
     }
-    ::HideWindow(mWindowPtr);
-  }
-  
-  return NS_OK;
+#if TARGET_CARBON
+    // Mac OS X sheet support
+    if (mIsSheet) {
+      if (piParentWidget)
+        piParentWidget->SetIgnoreDeactivate(PR_FALSE);
+
+      // get sheet's parent *before* hiding the sheet (which breaks the linkage)
+      WindowPtr sheetParent = nsnull;
+      ::GetSheetWindowParent(mWindowPtr, &sheetParent);
+
+      ::HideSheetWindow(mWindowPtr);
+
+      gEventDispatchHandler.DispatchGuiEvent(this, NS_DEACTIVATE);
+
+      // if we had several sheets open, when the last one goes away
+      // we need to ensure that the top app window is active
+      WindowPtr top = GetWindowTop(parentWindowRef);
+      piParentWidget = do_QueryInterface(parentWidget);
+
+      PRBool sheetFlag = PR_FALSE;
+      if (parentWindowRef && piParentWidget &&
+        NS_SUCCEEDED(piParentWidget->GetIsSheet(&sheetFlag)) && (sheetFlag))
+      {
+        nsCOMPtr<nsIWidget> parentWidget;
+        nsToolkit::GetTopWidget(sheetParent, getter_AddRefs(parentWidget));
+        piParentWidget = do_QueryInterface(parentWidget);
+        if (piParentWidget)
+          piParentWidget->SetIgnoreDeactivate(PR_TRUE);
+        ::ShowSheetWindow(parentWindowRef, sheetParent);
+      }
+      else if ( mAcceptsActivation ) {
+        ::ShowWindow(top);
+      }
+      else {
+        ::ShowHide(top, true);
+        ::BringToFront(top); // competes with ComeToFront, but makes popups work
+      }
+      ComeToFront();
+
+      if (top == parentWindowRef) {
+        UpdateWindowMenubar(parentWindowRef, PR_TRUE);
+      }
+    }
+    else
+#endif
+      if ( mWindowPtr ) {
+        ::HideWindow(mWindowPtr);
+      }
+    }
+    return NS_OK;
+}
+
+
+//-------------------------------------------------------------------------
+//
+// GetWindowTop
+//
+// find the topmost sheet currently assigned to a baseWindowRef or,
+// if no sheet is attached to it, just return the baseWindowRef itself
+//
+//-------------------------------------------------------------------------
+WindowPtr
+nsMacWindow::GetWindowTop(WindowPtr baseWindowRef)
+{
+    if (!baseWindowRef) return(nsnull);
+
+/*
+    Note: it would be nice to use window groups... something like the
+          following;  however, ::CountWindowGroupContents() and
+          ::GetIndexedWindow() are available on Mac OS X but NOT in
+          CarbonLib on pre-Mac OS X systems which we need to support
+
+    WindowGroupRef group = ::WindowGroupRef(baseWindowRef);
+    if (group)
+    {
+        WindowGroupContentOptions options =
+            kWindowGroupContentsReturnWindows | kWindowGroupContentsVisible;
+
+        ItemCount numWins = ::CountWindowGroupContents(group, options);
+        if (numWins >= 1)
+        {
+            ::GetIndexedWindow(group, 1, options, &baseWindowRef);
+        }
+    }
+*/
+
+    WindowPtr aSheet = ::GetFrontWindowOfClass(kSheetWindowClass, true);
+    while(aSheet)
+    {
+        WindowRef   sheetParent;
+        GetSheetWindowParent(aSheet, &sheetParent);
+        if (sheetParent == baseWindowRef)
+        {
+            return(aSheet);
+        }
+        aSheet = GetNextWindowOfClass(aSheet, kSheetWindowClass, true);
+    }
+    return(baseWindowRef);
+}
+
+
+void
+nsMacWindow::UpdateWindowMenubar(WindowPtr nativeWindowPtr, PRBool enableFlag)
+{
+    // Mac OS X sheet support
+    // when a sheet is up, disable parent window's menus
+    // (at least, until there is a better architecture for menus)
+
+    if (!nativeWindowPtr) return;
+
+    nsCOMPtr<nsIWidget> windowWidget;
+    nsToolkit::GetTopWidget ( nativeWindowPtr, getter_AddRefs(windowWidget));
+    if (!windowWidget) return;
+
+    nsCOMPtr<nsPIWidgetMac> parentWindow ( do_QueryInterface(windowWidget) );
+    if (!parentWindow)  return;
+    nsCOMPtr<nsIMenuBar> menubar;
+    parentWindow->GetMenuBar(getter_AddRefs(menubar));
+    if (!menubar) return;
+
+    PRUint32    numMenus=0;
+    menubar->GetMenuCount(numMenus);
+    for (PRInt32 i = numMenus-1; i >= 0; i--)
+    {
+        nsCOMPtr<nsIMenu> menu;
+        menubar->GetMenuAt(i, *getter_AddRefs(menu));
+        if (menu)
+        {
+            menu->SetEnabled(enableFlag);
+        }
+    }
 }
 
 
@@ -995,8 +1219,14 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
   if (aMode == nsSizeMode_Minimized) // unlikely on the Mac
     return NS_ERROR_UNEXPECTED;
 
+  // resize during zoom may attempt to unzoom us. here's where we put a stop to that.
+  if (mZooming)
+    return NS_OK;
+
   // already done? it's bad to rezoom a window, so do nothing
   rv = nsBaseWidget::GetSizeMode(&currentMode);
+  if (currentMode == nsSizeMode_Normal && !mVisible && mZoomOnShow)
+    currentMode = nsSizeMode_Maximized;
   if (NS_SUCCEEDED(rv) && currentMode == aMode)
     return NS_OK;
 
@@ -1004,10 +1234,10 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
     /* zooming on the Mac doesn't seem to work until the window is visible.
        the rest of the app is structured to zoom before the window is visible
        to avoid flashing. here's where we defeat that. */
-    if (aMode == nsSizeMode_Maximized)
-      mZoomOnShow = PR_TRUE;
+    mZoomOnShow = aMode == nsSizeMode_Maximized;
   } else {
     Rect macRect;
+    mZooming = PR_TRUE;
     rv = nsBaseWidget::SetSizeMode(aMode);
     if (NS_SUCCEEDED(rv)) {
       if (aMode == nsSizeMode_Maximized) {
@@ -1018,12 +1248,47 @@ NS_METHOD nsMacWindow::SetSizeMode(PRInt32 aMode)
       ::GetWindowPortBounds(mWindowPtr, &macRect);
       Resize(macRect.right - macRect.left, macRect.bottom - macRect.top, PR_FALSE);
     }
+    mZooming = PR_FALSE;
   }
 
   return rv;
 }
 
 
+// we're resizing. if we happen to be zoomed ("standard" state),
+// switch to the user state without changing the window size.
+void
+nsMacWindow::UserStateForResize()
+{
+  nsresult rv;
+  PRInt32  currentMode;
+
+  // but not if we're currently zooming
+  if (mZooming)
+    return;
+
+  // nothing to do if we're already in the user state
+  rv = nsBaseWidget::GetSizeMode(&currentMode);
+  if (NS_SUCCEEDED(rv) && currentMode == nsSizeMode_Normal)
+    return;
+
+  // we've just finished sizing, so make our current size our user state,
+  // and switch to the user state
+  StPortSetter portState(mWindowPtr);
+  StOriginSetter originState(mWindowPtr);
+  Rect bounds;
+
+  ::GetWindowPortBounds(mWindowPtr, &bounds);
+  ::LocalToGlobal((Point *)&bounds.top);
+  ::LocalToGlobal((Point *)&bounds.bottom);
+  ::SetWindowUserState(mWindowPtr, &bounds);
+  ::ZoomWindow(mWindowPtr, inZoomIn, false);
+
+  // finally, reset our internal state to match
+  nsBaseWidget::SetSizeMode(nsSizeMode_Normal);
+}
+
+
 //
 // CalculateAndSetZoomedSize
 //
@@ -1081,7 +1346,7 @@ nsMacWindow::CalculateAndSetZoomedSize()
       if (screen == primaryScreen) {
         int iconSpace = 96;
 #if TARGET_CARBON
-        if(::OnMacOSX()) {
+        if(nsToolkit::OnMacOSX()) {
           iconSpace = 128;  //icons/grid is wider on Mac OS X
         }
 #endif
@@ -1103,6 +1368,47 @@ nsMacWindow::CalculateAndSetZoomedSize()
 } // CalculateAndSetZoomedSize
 
 
+//-------------------------------------------------------------------------
+//
+// Obtain the menubar for a window
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP
+nsMacWindow::GetMenuBar(nsIMenuBar **_retval)
+{
+  *_retval = mMenuBar;
+  NS_IF_ADDREF(*_retval);
+  return(NS_OK);
+}
+
+//-------------------------------------------------------------------------
+//
+// getter/setter for window to ignore the next deactivate event received
+// if a Mac OS X sheet is being opened
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP
+nsMacWindow::GetIgnoreDeactivate(PRBool *_retval)
+{
+  *_retval = mIgnoreDeactivate;
+  return(NS_OK);
+}
+
+NS_IMETHODIMP
+nsMacWindow::SetIgnoreDeactivate(PRBool ignoreDeactivate)
+{
+  mIgnoreDeactivate = ignoreDeactivate;
+  return(NS_OK);
+}
+
+NS_IMETHODIMP
+nsMacWindow::GetIsSheet(PRBool *_retval)
+{
+  *_retval = mIsSheet;
+  return(NS_OK);
+}
+
+
 //-------------------------------------------------------------------------
 //
 // Resize this window to a point given in global (screen) coordinates. This
@@ -1254,7 +1560,7 @@ PRBool nsMacWindow::OnPaint(nsPaintEvent &event)
 NS_IMETHODIMP nsMacWindow::SetTitle(const nsString& aTitle)
 {
 #if TARGET_CARBON
-  if(::OnMacOSX()) {
+  if(nsToolkit::OnMacOSX()) {
     // On MacOS X try to use the unicode friendly CFString version first
     CFStringRef labelRef = ::CFStringCreateWithCharacters(kCFAllocatorDefault, (UniChar*)aTitle.get(), aTitle.Length());
     if(labelRef) {
@@ -1272,6 +1578,99 @@ NS_IMETHODIMP nsMacWindow::SetTitle(const nsString& aTitle)
 }
 
 
+#pragma mark -
+
+//
+// impleemnt nsIMacTextInputEventSink: forward to the right method in nsMacEventHandler 
+//
+
+/* OSStatus HandleUpdateActiveInputArea (in wstring text, in long textLength, in short script, in short language, in long fixLen, in voidPtr hiliteRng); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUpdateActiveInputArea(const nsAString & text, 
+                                         PRInt16 script, PRInt16 language, PRInt32 fixLen, void * hiliteRng, 
+                                         OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  const nsPromiseFlatString& buffer = PromiseFlatString(text);
+  // ignore script and langauge information for now. 
+  nsresult res = mMacEventHandler->UnicodeHandleUpdateInputArea((PRUnichar*)buffer.get(), buffer.Length(), fixLen, (TextRangeArray*) hiliteRng);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if (NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleUpdateActiveInputAreaForNonUnicode (in string text, in long textLength, in short script, in short language, in long fixLen, in voidPtr hiliteRng); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUpdateActiveInputAreaForNonUnicode(const nsACString & text, 
+                                                      PRInt16 script, PRInt16 language, 
+                                                      PRInt32 fixLen, void * hiliteRng, 
+                                                      OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  const nsPromiseFlatCString& buffer = PromiseFlatCString(text);
+  // ignore langauge information for now. 
+  nsresult res = mMacEventHandler->HandleUpdateInputArea((char*)buffer.get(), buffer.Length(), script, fixLen, (TextRangeArray*) hiliteRng);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if (NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleUnicodeForKeyEvent (in wstring text, in long textLength, in short script, in short language, in voidPtr keyboardEvent); */
+NS_IMETHODIMP 
+nsMacWindow::HandleUnicodeForKeyEvent(const nsAString & text, 
+                                      PRInt16 script, PRInt16 language, void * keyboardEvent, 
+                                      OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  // ignore langauge information for now. 
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  EventRecord* eventPtr = (EventRecord*)keyboardEvent;
+  const nsPromiseFlatString& buffer = PromiseFlatString(text);
+  nsresult res = mMacEventHandler->HandleUKeyEvent((PRUnichar*)buffer.get(), buffer.Length(), *eventPtr);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if(NS_SUCCEEDED(res))
+    *_retval = noErr;
+  return res;
+}
+
+/* OSStatus HandleOffsetToPos (in long offset, out short pointX, out short pointY); */
+NS_IMETHODIMP 
+nsMacWindow::HandleOffsetToPos(PRInt32 offset, PRInt16 *pointX, PRInt16 *pointY, OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  *pointX = *pointY = 0;
+  Point thePoint = {0,0};
+  nsresult res = mMacEventHandler->HandleOffsetToPosition(offset, &thePoint);
+  // we will lost the real OSStatus for now untill we change the nsMacEventHandler
+  if(NS_SUCCEEDED(res))
+    *_retval = noErr;
+  *pointX = thePoint.h;
+  *pointY = thePoint.v;  
+  return res;
+}
+
+/* OSStatus 
+HandlePosToOffset (in short currentPointX, in short currentPointY, out long offset, out short regionClass); */
+NS_IMETHODIMP 
+nsMacWindow::HandlePosToOffset(PRInt16 currentPointX, PRInt16 currentPointY, 
+                               PRInt32 *offset, PRInt16 *regionClass, OSStatus *_retval)
+{
+  *_retval = eventNotHandledErr;
+  NS_ENSURE_TRUE(mMacEventHandler.get(), NS_ERROR_FAILURE);
+  *_retval = noErr;
+  Point thePoint;
+  thePoint.h = currentPointX;
+  thePoint.v = currentPointY;
+  *offset = mMacEventHandler->HandlePositionToOffset(thePoint, regionClass);
+  return NS_OK;
+}
+
 #pragma mark -
 
 
@@ -1466,21 +1865,3 @@ CallSystemWDEF ( short inCode, WindowPtr inWindow, short inMessage, long inParam
 #pragma options align=reset
 
 #endif
-
-//
-// Return true if we are on Mac OS X, caching the result after the first call
-//
-PRBool
-OnMacOSX()
-{
-
-  static PRBool gInitVer = PR_FALSE;
-  static PRBool gOnMacOSX = PR_FALSE;
-  if(! gInitVer) {
-    long version;
-    OSErr err = ::Gestalt(gestaltSystemVersion, &version);
-    gOnMacOSX = (err == noErr && version >= 0x00001000);
-    gInitVer = PR_TRUE;
-  }
-  return gOnMacOSX;
-}
