@@ -42,7 +42,7 @@
 #include "plstr.h"
 #include "nsVoidArray.h"
 
-#include "nsIURI.h"
+#include "nsIURL.h"
 #include "nsNetUtil.h"
 #include "prmem.h"
 #include "nsGfxUtils.h"
@@ -57,6 +57,7 @@
 
 #include "nsITimer.h"
 #include "nsITimerCallback.h"
+#include "nsCRT.h"
 
 #include <Gestalt.h>
 #include <Sound.h>
@@ -161,17 +162,15 @@ protected:
   OSErr             TaskActiveMovies(PRBool *outAllMoviesDone);
 
   static PRBool     TaskOneMovie(Movie inMovie);    // return true if done
-
+  
 protected:
-
-  nsCOMPtr<nsISound>        mSound;       // back ptr, owned and released when play done  
-  nsCOMPtr<nsITimer>        mTimer;
   
   Movie                     mMovie;       // the original movie, kept around as long as this request is cached
   Handle                    mDataHandle;  // data handle, has to persist for the lifetime of any movies
                                           // depending on it
   
   nsVoidArray               mMovies;      // list of playing movie clones, which are transient.
+
 };
 
 #pragma mark -
@@ -265,7 +264,7 @@ nsSound::Play(nsIURL *aURL)
 
   // try to get from cache
   nsCOMPtr<nsISupports>   requestSupports;
-  (void)GetSoundFromCache(aURL, getter_AddRefs(requestSupports));
+  (void)GetSoundFromCache(NS_STATIC_CAST(nsIURI*,aURL), getter_AddRefs(requestSupports));
   if (requestSupports)
   {
     nsSoundRequest* cachedRequest = nsSoundRequest::GetFromISupports(requestSupports);
@@ -340,15 +339,15 @@ nsSound::GetSoundFromCache(nsIURI* inURI, nsISupports** outSound)
 {
   nsresult rv;
   
-  nsXPIDLCString uriSpec;
-  inURI->GetSpec(getter_Copies(uriSpec));
+  nsCAutoString uriSpec;
+  inURI->GetAsciiSpec(uriSpec);
 
   nsCOMPtr<nsICacheSession> cacheSession;
   rv = GetCacheSession(getter_AddRefs(cacheSession));
   if (NS_FAILED(rv)) return rv;
   
   nsCOMPtr<nsICacheEntryDescriptor> entry;
-  rv = cacheSession->OpenCacheEntry(uriSpec, nsICache::ACCESS_READ, nsICache::BLOCKING, getter_AddRefs(entry));
+  rv = cacheSession->OpenCacheEntry(uriSpec.get(), nsICache::ACCESS_READ, nsICache::BLOCKING, getter_AddRefs(entry));
 
 #ifdef SOUND_DEBUG
   printf("Got sound from cache with rv %ld\n", rv);
@@ -371,8 +370,8 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
   inChannel->GetOriginalURI(getter_AddRefs(uri));
   if (!uri) return NS_ERROR_FAILURE;
   
-  nsXPIDLCString uriSpec;
-  rv = uri->GetSpec(getter_Copies(uriSpec));
+  nsCAutoString uriSpec;
+  rv = uri->GetAsciiSpec(uriSpec);
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsICacheSession> cacheSession;
@@ -380,7 +379,7 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsICacheEntryDescriptor> entry;
-  rv = cacheSession->OpenCacheEntry(uriSpec, nsICache::ACCESS_WRITE, nsICache::BLOCKING, getter_AddRefs(entry));
+  rv = cacheSession->OpenCacheEntry(uriSpec.get(), nsICache::ACCESS_WRITE, nsICache::BLOCKING, getter_AddRefs(entry));
 #ifdef SOUND_DEBUG
   printf("Put sound in cache with rv %ld\n", rv);
 #endif
@@ -408,7 +407,7 @@ nsSound::PutSoundInCache(nsIChannel* inChannel, PRUint32 inDataSize, nsISupports
     }
   }
   
-  if (expirationTime == 0)
+  if (expirationTime == PRUint32(-1))   // no expiration time (never expires)
   {
     // set it to some reasonable default, like now + 24 hours
     expirationTime = SecondsFromPRTime(PR_Now()) + 60 * 60 * 24;
@@ -600,7 +599,7 @@ nsSystemSoundRequest::PlaySound()
   
   const PRInt32   kSoundTimerInterval = 250;      // 250 milliseconds
   rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kSoundTimerInterval,
-          NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_PRECISE);
+          PR_TRUE, NS_TYPE_REPEATING_PRECISE);
   if (NS_FAILED(rv)) {
     Cleanup();
     return rv;
@@ -638,6 +637,58 @@ nsSystemSoundRequest::SoundCallback(SndChannelPtr chan, SndCommand *theCmd)
 
 #pragma mark -
 
+// This class should only ever be instantiated once, statically.
+// Its job is to create, own and destroy the singleton GWorld.
+class nsMoviePortOwner
+{
+public:
+
+  static GWorldPtr	GetSingletonMoviePort();
+  
+private:
+
+              // private ctor and dtor. Only GetSingletonMoviePort() can instantiate us.
+              nsMoviePortOwner() {}
+              
+              ~nsMoviePortOwner()
+              {
+                if (sMoviePort)
+                {
+                  ::DisposeGWorld(sMoviePort);
+                  sMoviePort = nsnull;
+                }
+              }
+
+  void        EnsureMoviePort()
+              {
+                if (!sMoviePort)
+                {
+                  Rect        gWorldBounds = {0, 0, 12, 12};
+                  (void)::NewGWorld(&sMoviePort, 8, &gWorldBounds, nil, nil, 0);
+                }
+              }
+
+  GWorldPtr   GetMoviePort()
+              {
+                EnsureMoviePort();
+                return sMoviePort;
+              }
+
+private:
+
+  static GWorldPtr    sMoviePort;
+
+};
+
+GWorldPtr nsMoviePortOwner::sMoviePort = nsnull;
+
+GWorldPtr nsMoviePortOwner::GetSingletonMoviePort()
+{
+  static nsMoviePortOwner   sMoviePortOwner;
+  return sMoviePortOwner.GetMoviePort();
+}
+
+
 NS_IMPL_ISUPPORTS_INHERITED1(nsMovieSoundRequest, nsSoundRequest, nsIStreamLoaderObserver);
 
 ////////////////////////////////////////////////////////////////////////
@@ -691,13 +742,13 @@ nsMovieSoundRequest::OnStreamComplete(nsIStreamLoader *aLoader,
   if (NS_FAILED(aStatus))
     return NS_ERROR_FAILURE;
 
-  nsXPIDLCString  contentType;
+  nsCAutoString contentType;
 
   nsCOMPtr<nsIRequest>    request;
   aLoader->GetRequest(getter_AddRefs(request));
   nsCOMPtr<nsIChannel>  channel = do_QueryInterface(request);
   if (channel)
-    channel->GetContentType(getter_Copies(contentType));
+    channel->GetContentType(contentType);
   
   // we could use a Pointer data handler type, and avoid this
   // allocation/copy, in QuickTime 5 and above.
@@ -792,7 +843,7 @@ nsMovieSoundRequest::PlaySound()
   nsresult rv;
 
   // we'll have a timer already if the sound is still playing from a previous
-  // request. In that case, we cloen the movie into a new one, so we can play it
+  // request. In that case, we clone the movie into a new one, so we can play it
   // again from the start.
   if (!mTimer)
   {  
@@ -804,7 +855,7 @@ nsMovieSoundRequest::PlaySound()
     
     const PRInt32   kMovieTimerInterval = 250;      // 250 milliseconds
     rv = mTimer->Init(NS_STATIC_CAST(nsITimerCallback*, this), kMovieTimerInterval,
-            NS_PRIORITY_NORMAL, NS_TYPE_REPEATING_PRECISE);
+            PR_TRUE, NS_TYPE_REPEATING_PRECISE);
     if (NS_FAILED(rv)) {
       Cleanup();
       return rv;
@@ -874,10 +925,14 @@ nsMovieSoundRequest::Notify(nsITimer *timer)
 OSErr
 nsMovieSoundRequest::ImportMovie(Handle inDataHandle, long inDataSize, const nsACString& contentType)
 {
+  GWorldPtr               moviePort = nsMoviePortOwner::GetSingletonMoviePort();
   Handle                  dataRef = nil;
   OSErr                   err = noErr;
   OSType                  fileFormat;
   
+  if (!moviePort)
+    return memFullErr;
+  
   {
     StHandleLocker  locker(inDataHandle);
     fileFormat = GetFileFormat(*inDataHandle, inDataSize, contentType);
@@ -894,6 +949,11 @@ nsMovieSoundRequest::ImportMovie(Handle inDataHandle, long inDataSize, const nsA
     long                  outFlags = 0;
     ComponentResult       compErr = noErr;
 
+    // set the port to our singleton GWorld before creating
+    // the movie. This will ensure that the movie uses this port, and
+    // not one of our (transient) windows.
+    StGWorldPortSetter    gWorldSetter(moviePort);
+
     if (!miComponent) {
       err = paramErr;
       goto bail;
@@ -932,7 +992,7 @@ nsMovieSoundRequest::ImportMovie(Handle inDataHandle, long inDataSize, const nsA
     }
     
     ::GoToEndOfMovie(mMovie);   // simplifies the logic in PlaySound()
-    
+        
   bail:
     if (miComponent)
       ::CloseComponent(miComponent);
@@ -950,7 +1010,7 @@ nsMovieSoundRequest::DisposeMovieData()
   for (PRInt32 i = 0; i < mMovies.Count(); i ++)
   {
     Movie   thisMovie = (Movie)mMovies.ElementAt(i);
-    ::DisposeMovie(mMovie);
+    ::DisposeMovie(thisMovie);
   }
   
   mMovies.Clear();
@@ -993,11 +1053,11 @@ nsMovieSoundRequest::TaskActiveMovies(PRBool *outAllMoviesDone)
   PRBool    allMoviesDone = PR_FALSE;
 
   allMoviesDone = TaskOneMovie(mMovie);
-
+  
   PRInt32 initMovieCount = mMovies.Count();
   PRInt32 curIndex = 0;
-  
-  for (PRInt32 i = 0; i < initMovieCount; i ++)
+
+  while (curIndex < mMovies.Count())
   {
     Movie   thisMovie     = (Movie)mMovies.ElementAt(curIndex);
     PRBool  thisMovieDone = TaskOneMovie(thisMovie);
@@ -1044,3 +1104,4 @@ nsMovieSoundRequest::HaveQuickTime()
   return (err == noErr) && ((long)EnterMovies != kUnresolvedCFragSymbolAddress);
 }
 
+
