@@ -68,6 +68,8 @@
 #include "nsGfxUtils.h"
 #include "nsRegionPool.h"
 
+#include <Gestalt.h>
+
 #if PINK_PROFILING
 #include "profilerutils.h"
 #endif
@@ -94,15 +96,148 @@ void EachRegionRect (RgnHandle r, void (* proc)(Rect *, void *), void* data) ;
 
 #pragma mark -
 
+
 //#define PAINT_DEBUGGING         // flash areas as they are painted
 //#define INVALIDATE_DEBUGGING    // flash areas as they are invalidated
 
 #if defined(INVALIDATE_DEBUGGING) || defined(PAINT_DEBUGGING)
-static void blinkRect(Rect* r);
-static void blinkRgn(RgnHandle rgn);
+static void blinkRect(const Rect* r, PRBool isPaint);
+static void blinkRgn(RgnHandle rgn, PRBool isPaint);
 #endif
 
 
+static Boolean KeyDown(const UInt8 theKey)
+{
+	KeyMap map;
+	GetKeys(map);
+	return ((*((UInt8 *)map + (theKey >> 3)) >> (theKey & 7)) & 1) != 0;
+}
+
+#if defined(INVALIDATE_DEBUGGING) || defined(PAINT_DEBUGGING)
+
+static Boolean caps_lock()
+{
+  return KeyDown(0x39);
+}
+
+
+static void FlushCurPortBuffer()
+{
+#if TARGET_CARBON
+    CGrafPtr    curPort;
+    ::GetPort((GrafPtr*)&curPort);
+    ::QDFlushPortBuffer(curPort, nil);      // OK to call on 9/carbon (does nothing)
+#endif
+}
+
+static void blinkRect(const Rect* r, PRBool isPaint)
+{
+	StRegionFromPool oldClip;
+	UInt32 endTicks;
+
+    if (oldClip != NULL)
+    ::GetClip(oldClip);
+
+    ::ClipRect(r);
+
+    if (isPaint)
+    {
+        Pattern grayPattern;
+#if TARGET_CARBON
+        ::GetQDGlobalsGray(&grayPattern);
+#else
+        grayPattern = qd.gray;
+#endif
+
+        ::ForeColor(blackColor);
+        ::BackColor(whiteColor);
+
+        ::PenMode(patXor);
+        ::PenPat(&grayPattern);
+        ::PaintRect(r);
+        FlushCurPortBuffer();
+        ::Delay(5, &endTicks);
+        ::PaintRect(r);
+        FlushCurPortBuffer();
+        ::PenNormal();
+    }
+    else
+    {
+        ::InvertRect(r);
+        FlushCurPortBuffer();
+        ::Delay(5, &endTicks);
+        ::InvertRect(r);
+        FlushCurPortBuffer();
+    }
+
+	if (oldClip != NULL)
+		::SetClip(oldClip);
+}
+
+static void blinkRgn(RgnHandle rgn, PRBool isPaint)
+{
+    StRegionFromPool oldClip;
+    UInt32 endTicks;
+
+    if (oldClip != NULL)
+        ::GetClip(oldClip);
+
+    ::SetClip(rgn);
+
+    if (isPaint)
+    {
+        Pattern grayPattern;
+#if TARGET_CARBON
+        ::GetQDGlobalsGray(&grayPattern);
+#else
+        grayPattern = qd.gray;
+#endif
+
+        ::ForeColor(blackColor);
+        ::BackColor(whiteColor);
+
+        ::PenMode(patXor);
+        ::PenPat(&grayPattern);
+        ::PaintRgn(rgn);
+        FlushCurPortBuffer();
+
+        ::Delay(5, &endTicks);
+        ::PaintRgn(rgn);
+        FlushCurPortBuffer();
+        ::PenNormal();
+    }
+    else
+    {
+        ::InvertRgn(rgn);
+        FlushCurPortBuffer();
+        ::Delay(5, &endTicks);
+        ::InvertRgn(rgn);
+        FlushCurPortBuffer();
+    }
+
+    if (oldClip != NULL)
+        ::SetClip(oldClip);
+}
+
+#endif
+
+
+static Boolean control_key_down()
+{
+	EventRecord event;
+	::EventAvail(0, &event);
+	return (event.modifiers & controlKey) != 0;
+}
+
+static long long microseconds()
+{
+	unsigned long long micros;
+	Microseconds((UnsignedWide*)&micros);
+	return micros;
+}
+
+#pragma mark -
+
 //-------------------------------------------------------------------------
 //
 // nsWindow constructor
@@ -113,6 +248,7 @@ nsWindow::nsWindow() : nsBaseWidget() , nsDeleteObserved(this), nsIKBStateContro
 	WIDGET_SET_CLASSNAME("nsWindow");
 
   mParent = nsnull;
+  mIsTopWidgetWindow = PR_FALSE;
   mBounds.SetRect(0,0,0,0);
 
   mResizingChildren = PR_FALSE;
@@ -320,6 +456,7 @@ NS_IMETHODIMP nsWindow::Destroy()
 //-------------------------------------------------------------------------
 nsIWidget* nsWindow::GetParent(void)
 {
+  if (mIsTopWidgetWindow) return nsnull;
   NS_IF_ADDREF(mParent);
   return  mParent;
 }
@@ -420,95 +557,8 @@ NS_IMETHODIMP nsWindow::Show(PRBool bState)
 NS_IMETHODIMP nsWindow::ModalEventFilter(PRBool aRealEvent, void *aEvent,
                                          PRBool *aForWindow)
 {
-	*aForWindow = PR_FALSE;
-	EventRecord *theEvent = (EventRecord *) aEvent;
-
-	if (aRealEvent && theEvent->what != nullEvent ) {
-
-		WindowPtr window = (WindowPtr) GetNativeData(NS_NATIVE_DISPLAY);
-		WindowPtr rollupWindow = gRollupWidget ? (WindowPtr) gRollupWidget->GetNativeData(NS_NATIVE_DISPLAY) : nsnull;
-		WindowPtr eventWindow = nsnull;
-		
-		PRInt16 where = ::FindWindow ( theEvent->where, &eventWindow );
-		PRBool inWindow = eventWindow && (eventWindow == window || eventWindow == rollupWindow);
-
-		switch ( theEvent->what ) {
-			// is it a mouse event?
-			case mouseUp:
-					*aForWindow = PR_TRUE;
-				break;
-			case mouseDown:
-				// is it in the given window?
-				// (note we also let some events questionable for modal dialogs pass through.
-				// but it makes sense that the draggability et.al. of a modal window should
-				// be controlled by whether the window has a drag bar).
-				if (inWindow) {
-				     if ( where == inContent || where == inDrag   || where == inGrow ||
-				          where == inGoAway  || where == inZoomIn || where == inZoomOut )
-					*aForWindow = PR_TRUE;
-				}
-				else      // we're in another window.
-				{
-				  // let's handle dragging of background windows here
-				  if (eventWindow && (where == inDrag) && (theEvent->modifiers & cmdKey))
-				  {
-    				Rect screenRect;
-    				::GetRegionBounds(::GetGrayRgn(), &screenRect);
-				    ::DragWindow(eventWindow, theEvent->where, &screenRect);
-				  }
-				  
-  				*aForWindow = PR_FALSE;
-				}
-				break;
-			case keyDown:
-			case keyUp:
-			case autoKey:
-				*aForWindow = PR_TRUE;
-				break;
-
-			case diskEvt:
-			    // I think dialogs might want to support floppy insertion, and it doesn't
-			    // interfere with modality...
-			case updateEvt:
-				// always let update events through, because if we don't handle them, we're
-				// doomed!
-			case activateEvt:
-				// activate events aren't so much a request as simply informative. might
-				// as well acknowledge them.
-				*aForWindow = PR_TRUE;
-				break;
-
-			case osEvt:
-				// check for mouseMoved or suspend/resume events. We especially need to
-				// let suspend/resume events through in order to make sure the clipboard is
-				// converted correctly.
-				unsigned char eventType = (theEvent->message >> 24) & 0x00ff;
-				if (eventType == mouseMovedMessage) {
-					// I'm guessing we don't want to let these through unless the mouse is
-					// in the modal dialog so we don't see rollover feedback in windows behind
-					// the dialog.
-					if ( where == inContent && inWindow )
-						*aForWindow = PR_TRUE;
-				}
-				if ( eventType == suspendResumeMessage ) {
-					*aForWindow = PR_TRUE;
-					if (theEvent->message & resumeFlag) {
-						// divert it to our window if it isn't naturally
-						if (!inWindow) {
-							StPortSetter portSetter(window);
-							theEvent->where.v = 0;
-							theEvent->where.h = 0;
-							::LocalToGlobal(&theEvent->where);
-						}
-				  	}
-				}
-
-				break;
-		} // case of which event type
-	} else
-		*aForWindow = PR_TRUE;
-
-	return NS_OK;
+  *aForWindow = PR_TRUE;
+  return NS_OK;
 }
 
 //-------------------------------------------------------------------------
@@ -516,13 +566,20 @@ NS_IMETHODIMP nsWindow::ModalEventFilter(PRBool aRealEvent, void *aEvent,
 // Enable/disable this component
 //
 //-------------------------------------------------------------------------
-NS_IMETHODIMP nsWindow::Enable(PRBool bState)
+NS_IMETHODIMP nsWindow::Enable(PRBool aState)
 {
-	mEnabled = bState;
+	mEnabled = aState;
 	return NS_OK;
 }
 
     
+NS_IMETHODIMP nsWindow::IsEnabled(PRBool *aState)
+{
+	NS_ENSURE_ARG_POINTER(aState);
+	*aState = mEnabled;
+	return NS_OK;
+}
+
 static Boolean we_are_front_process()
 {
 	ProcessSerialNumber	thisPSN;
@@ -798,7 +855,7 @@ NS_IMETHODIMP nsWindow::Move(PRInt32 aX, PRInt32 aY)
 	if ((mBounds.x != aX) || (mBounds.y != aY))
 	{
 		// Invalidate the current location (unless it's the top-level window)
-		if (mParent != nsnull)
+		if ((mParent != nsnull) && (!mIsTopWidgetWindow))
 			Invalidate(PR_FALSE);
 	  	
 		// Set the bounds
@@ -902,55 +959,6 @@ NS_IMETHODIMP nsWindow::EndResizingChildren(void)
 	return NS_OK;
 }
 
-#pragma mark -
-
-static Boolean KeyDown(const UInt8 theKey)
-{
-	KeyMap map;
-	GetKeys(map);
-	return ((*((UInt8 *)map + (theKey >> 3)) >> (theKey & 7)) & 1) != 0;
-}
-
-#if defined(INVALIDATE_DEBUGGING) || defined(PAINT_DEBUGGING)
-
-static Boolean caps_lock()
-{
-  return KeyDown(0x39);
-}
-
-static void blinkRect(Rect* r)
-{
-	StRegionFromPool oldClip;
-	if (oldClip != NULL)
-		::GetClip(oldClip);
-
-	::ClipRect(r);
-	::InvertRect(r);
-	UInt32 end = ::TickCount() + 5;
-	while (::TickCount() < end) ;
-	::InvertRect(r);
-
-	if (oldClip != NULL)
-		::SetClip(oldClip);
-}
-
-static void blinkRgn(RgnHandle rgn)
-{
-	StRegionFromPool oldClip;
-	if (oldClip != NULL)
-		::GetClip(oldClip);
-
-	::SetClip(rgn);
-	::InvertRgn(rgn);
-	UInt32 end = ::TickCount() + 5;
-	while (::TickCount() < end) ;
-	::InvertRgn(rgn);
-
-	if (oldClip != NULL)
-		::SetClip(oldClip);
-}
-
-#endif
 
 //-------------------------------------------------------------------------
 //
@@ -959,7 +967,7 @@ static void blinkRgn(RgnHandle rgn)
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsWindow::Validate()
 {
-	if (!mWindowPtr || !mVisible)
+	if (!mWindowPtr || !mVisible || !ContainerHierarchyIsVisible())
 		return NS_OK;
 
 	nsRect wRect = mBounds;
@@ -967,13 +975,10 @@ NS_IMETHODIMP nsWindow::Validate()
 	Rect macRect;
 	nsRectToMacRect(wRect, macRect);
 
-	StPortSetter portSetter(mWindowPtr);
-	Rect savePortRect;
-	::GetWindowPortBounds(mWindowPtr, &savePortRect);
-	::SetOrigin(0, 0);
+	StPortSetter    portSetter(mWindowPtr);
+	StOriginSetter  originSetter(mWindowPtr);
 
 	::ValidWindowRect(mWindowPtr, &macRect);
-	::SetOrigin(savePortRect.left, savePortRect.top);
 
 	return NS_OK;
 }
@@ -1011,20 +1016,16 @@ NS_IMETHODIMP nsWindow::Invalidate(const nsRect &aRect, PRBool aIsSynchronous)
 	nsRectToMacRect(wRect, macRect);
 
 	StPortSetter portSetter(mWindowPtr);
-
-	Rect savePortRect;
-	::GetWindowPortBounds(mWindowPtr, &savePortRect);
-	::SetOrigin(0, 0);
+    StOriginSetter  originSetter(mWindowPtr);
 
 #ifdef INVALIDATE_DEBUGGING
 	if (caps_lock())
-		::blinkRect(&macRect);
+		::blinkRect(&macRect, PR_FALSE);
 #endif
 
 	::InvalWindowRect(mWindowPtr, &macRect);
 	if ( aIsSynchronous )
 	  Update();
-	::SetOrigin(savePortRect.left, savePortRect.top);
 
 	return NS_OK;
 }
@@ -1051,24 +1052,20 @@ NS_IMETHODIMP nsWindow::InvalidateRegion(const nsIRegion *aRegion, PRBool aIsSyn
 
 	// translate this region into window coordinates.
 	PRInt32	offX, offY;
-	this->CalcOffset(offX, offY);
+	CalcOffset(offX, offY);
 	::OffsetRgn(windowRgn, mBounds.x + offX, mBounds.y + offY);
 	
-	StPortSetter portSetter(mWindowPtr);
-
-	Rect savePortRect;
-	::GetWindowPortBounds(mWindowPtr, &savePortRect);
-	::SetOrigin(0, 0);
+	StPortSetter    portSetter(mWindowPtr);
+    StOriginSetter  originSetter(mWindowPtr);
 
 #ifdef INVALIDATE_DEBUGGING
 	if (caps_lock())
-		::blinkRgn(windowRgn);
+		::blinkRgn(windowRgn, PR_FALSE);
 #endif
 
 	::InvalWindowRgn(mWindowPtr, windowRgn);
 	if ( aIsSynchronous )
 	  Update();
-	::SetOrigin(savePortRect.left, savePortRect.top);
 
 	return NS_OK;
 }
@@ -1144,10 +1141,12 @@ void nsWindow::EndDraw()
 		return;
 	mDrawing = PR_FALSE;
 
-		PRBool clipEmpty;
-		mTempRenderingContext->PopState(clipEmpty);
+	PRBool clipEmpty;
+	mTempRenderingContext->PopState(clipEmpty);
 
 	NS_RELEASE(mTempRenderingContext);
+	
+	// note that we may leave the window origin in an unspecified state here
 }
 
 
@@ -1197,73 +1196,61 @@ nsWindow::OnPaint(nsPaintEvent &event)
 //-------------------------------------------------------------------------
 NS_IMETHODIMP	nsWindow::Update()
 {
-	if (! mVisible || !mWindowPtr)
-		return NS_OK;
+  if (! mVisible || !mWindowPtr || !ContainerHierarchyIsVisible())
+    return NS_OK;
 
-	static PRBool  reentrant = PR_FALSE;
+  static PRBool  reentrant = PR_FALSE;
 
-	if (reentrant)
-		HandleUpdateEvent(nil);
-	else
-	{
-		reentrant = PR_TRUE;
+  if (reentrant)
+    HandleUpdateEvent(nil);
+  else
+  {
+    reentrant = PR_TRUE;
+    
+    StRegionFromPool regionToValidate;
+    if (!regionToValidate)
+      return NS_ERROR_OUT_OF_MEMORY;
 
-		StRegionFromPool regionToValidate;
-		if (!regionToValidate)
-			return NS_ERROR_OUT_OF_MEMORY;
+    // make a copy of the window update rgn (which is in global coords)
+    StRegionFromPool saveUpdateRgn;
+    if (!saveUpdateRgn)
+      return NS_ERROR_OUT_OF_MEMORY;
 
-		// make a copy of the window update rgn
-		StRegionFromPool saveUpdateRgn;
-		if (!saveUpdateRgn)
-			return NS_ERROR_OUT_OF_MEMORY;
-		if(mWindowPtr)
-			::GetWindowUpdateRegion ( mWindowPtr, saveUpdateRgn );
+    if (mWindowPtr)
+      ::GetWindowUpdateRegion(mWindowPtr, saveUpdateRgn);
 
-		// draw the widget
-		StPortSetter portSetter(mWindowPtr);
+    // draw the widget
+    StPortSetter portSetter(mWindowPtr);
+    ::SetOrigin(0, 0);
 
-		::BeginUpdate(mWindowPtr);
-		HandleUpdateEvent(regionToValidate);
-		::EndUpdate(mWindowPtr);
+    // BeginUpate replaces the visRgn with the intersection of the
+    // visRgn and the updateRgn.
+    ::BeginUpdate(mWindowPtr);
 
-		// restore the window update rgn
-#if TARGET_CARBON
-		// saveUpdateRgn is in global coords, so we need to shift it to local coords
-		Point origin = {0, 0};
-		::GlobalToLocal(&origin);
-		::OffsetRgn(saveUpdateRgn, origin.h, origin.v);
-		::InvalWindowRgn(mWindowPtr, saveUpdateRgn);
-#else
-	  ::CopyRgn(saveUpdateRgn, ((WindowRecord*)mWindowPtr)->updateRgn);
-#endif
+    HandleUpdateEvent(regionToValidate);
+
+    // EndUpdate replaces the normal visRgn
+    ::EndUpdate(mWindowPtr);
+
+    // restore the window update rgn
+    // saveUpdateRgn is in global coords, so we need to shift it to local coords
+    Point origin = {0, 0};
+    ::GlobalToLocal(&origin);
+    ::OffsetRgn(saveUpdateRgn, origin.h, origin.v);
+    ::InvalWindowRgn(mWindowPtr, saveUpdateRgn);
 
     ::ValidWindowRgn(mWindowPtr, regionToValidate);
     
-		reentrant = PR_FALSE;
-	}
+    reentrant = PR_FALSE;
+  }
 
-	return NS_OK;
+  return NS_OK;
 }
 
 
 #pragma mark -
 
 
-static Boolean control_key_down()
-{
-	EventRecord event;
-	::EventAvail(0, &event);
-	return (event.modifiers & controlKey) != 0;
-}
-
-static long long microseconds()
-{
-	unsigned long long micros;
-	Microseconds((UnsignedWide*)&micros);
-	return micros;
-}
-
-
 #if TARGET_CARBON
 
 
@@ -1349,7 +1336,7 @@ nsWindow::CountRectProc (UInt16 message, RgnHandle rgn, const Rect *inDirtyRect,
 // this. |inData| contains the |nsWindow| being updated. 
 //
 void 
-nsWindow::PaintUpdateRect (Rect *inDirtyRect, void* inData)
+nsWindow::PaintUpdateRect(Rect *inDirtyRect, void* inData)
 {
   nsWindow* self = NS_REINTERPRET_CAST(nsWindow*, inData);
   Rect dirtyRect = *inDirtyRect;
@@ -1357,16 +1344,16 @@ nsWindow::PaintUpdateRect (Rect *inDirtyRect, void* inData)
 	nsCOMPtr<nsIRenderingContext> renderingContext ( dont_AddRef(self->GetRenderingContext()) );
 	if (renderingContext)
 	{
-  	nsRect bounds = self->mBounds;
-  	self->LocalToWindowCoordinate(bounds);
-  	
-		// determine the rect to draw
-		::OffsetRect(&dirtyRect, -bounds.x, -bounds.y);
-		nsRect rect ( dirtyRect.left, dirtyRect.top, dirtyRect.right - dirtyRect.left,
-		                dirtyRect.bottom - dirtyRect.top );
+        nsRect bounds = self->mBounds;
+        self->LocalToWindowCoordinate(bounds);
 
-		// update the widget
-		self->UpdateWidget(rect, renderingContext);
+        // determine the rect to draw
+        ::OffsetRect(&dirtyRect, -bounds.x, -bounds.y);
+        nsRect rect ( dirtyRect.left, dirtyRect.top, dirtyRect.right - dirtyRect.left,
+                        dirtyRect.bottom - dirtyRect.top );
+
+        // update the widget
+        self->UpdateWidget(rect, renderingContext);
   }
 
 } // PaintUpdateRect
@@ -1421,23 +1408,26 @@ else
 	ProfileStop(); 
 #endif
 
-	if (! mVisible)
+	if (! mVisible || !ContainerHierarchyIsVisible())
 		return NS_OK;
 
 	// make sure the port is set and origin is (0, 0).
-	StPortSetter portSetter(mWindowPtr);
-	::SetOrigin(0, 0);
+	StPortSetter    portSetter(mWindowPtr);
+	// zero the origin, and set it back after drawing widgets
+	StOriginSetter  originSetter(mWindowPtr);
 	
 	// get the damaged region from the OS
 	StRegionFromPool damagedRgn;
 	if (!damagedRgn)
 		return NS_ERROR_OUT_OF_MEMORY;
-	::GetPortVisibleRegion(GetWindowPort(mWindowPtr), damagedRgn);
+	::GetPortVisibleRegion(::GetWindowPort(mWindowPtr), damagedRgn);
 
+/*
 #ifdef PAINT_DEBUGGING	
-	blinkRgn(damagedRgn);
+	if (caps_lock())
+  	blinkRgn(damagedRgn, PR_TRUE);
 #endif
-
+*/
 	// calculate the update region relatively to the window port rect
 	// (at this point, the grafPort origin should always be 0,0
 	// so mWindowRegion has to be converted to window coordinates)
@@ -1450,14 +1440,24 @@ else
 	LocalToWindowCoordinate(bounds);
 	::OffsetRgn(updateRgn, bounds.x, bounds.y);
 
-#ifdef PAINT_DEBUGGING
-	blinkRgn(updateRgn);
-#endif
-	
 	// check if the update region is visible
 	::SectRgn(damagedRgn, updateRgn, updateRgn);
+
+#ifdef PAINT_DEBUGGING
+	if (caps_lock())
+  	blinkRgn(updateRgn, PR_TRUE);
+#endif
+
 	if (!::EmptyRgn(updateRgn))
 	{
+
+#if TARGET_CARBON
+    // Ref: http://developer.apple.com/technotes/tn/tn2051.html
+    //      short-circuit QD's implicit LockPortBits()
+    //      Note: MUST unlock before exiting this scope (see below)
+    ::LockPortBits(::GetWindowPort(mWindowPtr));
+#endif
+
 #if DEBUG
 		// measure the time it takes to refresh the window, if the control key is down.
     unsigned long long start, finish;
@@ -1550,6 +1550,14 @@ else
     // Copy updateRgn to regionToValidate
     if (regionToValidate)
       ::CopyRgn(updateRgn, regionToValidate);
+
+#if TARGET_CARBON
+    // Ref: http://developer.apple.com/technotes/tn/tn2051.html
+    //      short-circuit QD's implicit LockPortBits()
+    //      Note: unlock before exiting (locked above)
+    ::UnlockPortBits(::GetWindowPort(mWindowPtr));
+#endif
+
 	}
 
 #if PINK_PROFILING
@@ -1663,7 +1671,7 @@ nsWindow::CombineRects ( TRectArray & rectArray )
 //-------------------------------------------------------------------------
 void nsWindow::UpdateWidget(nsRect& aRect, nsIRenderingContext* aContext)
 {
-	if (! mVisible)
+	if (! mVisible || !ContainerHierarchyIsVisible())
 		return;
 
 	// initialize the paint event
@@ -1753,94 +1761,95 @@ nsWindow::ScrollBits ( Rect & inRectToScroll, PRInt32 inLeftDelta, PRInt32 inTop
   ::ScrollWindowRect ( mWindowPtr, &inRectToScroll, inLeftDelta, inTopDelta, 
                         kScrollWindowInvalidate, NULL );
 #else
-	// Get Frame in local coords from clip rect (there might be a border around view)
-	StRegionFromPool clipRgn;
-	if ( !clipRgn ) return;
-	::GetClip(clipRgn);
-	::SectRgn(clipRgn, mVisRegion, clipRgn);
+  // Get Frame in local coords from clip rect (there might be a border around view)
+  StRegionFromPool clipRgn;
+  if ( !clipRgn ) return;
+  ::GetClip(clipRgn);
+  ::SectRgn(clipRgn, mVisRegion, clipRgn);
 
-	Rect frame;
-	::GetRegionBounds(clipRgn, &frame);
+  StRegionFromPool localVisRgn;
+  if ( !localVisRgn ) return;
 
-	StRegionFromPool totalVisRgn;
-	if ( !totalVisRgn ) return;
-	::RectRgn(totalVisRgn, &frame);
-	
-		// compute the source and destination of copybits
-	Rect source = inRectToScroll;
-	SectRect(&source, &frame, &source);
+  Rect frame;
+  ::GetRegionBounds(clipRgn, &frame);
 
-	Rect dest = source;
-	::OffsetRect(&dest, inLeftDelta, inTopDelta);
+  StRegionFromPool totalVisRgn;
+  if ( !totalVisRgn ) return;
+  ::RectRgn(totalVisRgn, &frame);
+  
+    // compute the source and destination of copybits
+  Rect source = inRectToScroll;
+  SectRect(&source, &frame, &source);
 
-	// compute the area that is to be updated by subtracting the dest from the visible area
-	StRegionFromPool destRgn;
-	if ( !destRgn ) return;
-	::RectRgn(destRgn, &dest);		
+  Rect dest = source;
+  ::OffsetRect(&dest, inLeftDelta, inTopDelta);
 
-	StRegionFromPool updateRgn;
-	if ( !updateRgn ) return;
-	::RectRgn(updateRgn, &frame);
-	::DiffRgn (updateRgn, destRgn, updateRgn);
+  // compute the area that is to be updated by subtracting the dest from the visible area
+  StRegionFromPool destRgn;
+  if ( !destRgn ) return;
+  ::RectRgn(destRgn, &dest);    
 
-	if(::EmptyRgn(mWindowPtr->visRgn))		
-	{
-		::CopyBits ( 
-			&mWindowPtr->portBits, 
-			&mWindowPtr->portBits, 
-			&source, 
-			&dest, 
-			srcCopy, 
-			nil);
-	}
-	else
-	{
-		// compute the non-visible region
-		StRegionFromPool nonVisibleRgn;
-		if ( !nonVisibleRgn ) return;
+  StRegionFromPool updateRgn;
+  if ( !updateRgn ) return;
+  ::RectRgn(updateRgn, &frame);
+  ::DiffRgn (updateRgn, destRgn, updateRgn);
 
+  if(::EmptyRgn(mWindowPtr->visRgn))    
+  {
+    ::CopyBits ( 
+      &mWindowPtr->portBits, 
+      &mWindowPtr->portBits, 
+      &source, 
+      &dest, 
+      srcCopy, 
+      nil);
+  }
+  else
+  {
+    // compute the non-visible region by subtracting what's currently
+    // visible (the window's visRgn) from the whole area we're updating
+    StRegionFromPool nonVisibleRgn;
+    if ( !nonVisibleRgn ) return;
     ::DiffRgn ( totalVisRgn, mWindowPtr->visRgn, nonVisibleRgn );
-		
-		// compute the extra area that may need to be updated
-		// scoll the non-visible region to determine what needs updating
-		::OffsetRgn ( nonVisibleRgn, inLeftDelta, inTopDelta );
-		
-		// calculate a mask region to not copy the non-visble portions of the window from the port
-		StRegionFromPool copyMaskRgn;
-		if ( !copyMaskRgn ) return;
-		::DiffRgn(totalVisRgn, nonVisibleRgn, copyMaskRgn);
-		
-		// use copybits to simulate a ScrollRect()
-		RGBColor black = { 0, 0, 0 };
-		RGBColor white = { 0xFFFF, 0xFFFF, 0xFFFF } ;
-		::RGBForeColor(&black);
-		::RGBBackColor(&white);
-		::PenNormal();	
-		::CopyBits ( 
-			&mWindowPtr->portBits, 
-			&mWindowPtr->portBits, 
-			&source, 
-			&dest, 
-			srcCopy, 
-			copyMaskRgn);
+    
+    // compute the extra area that may need to be updated
+    // scoll the non-visible region to determine what needs updating
+    ::OffsetRgn ( nonVisibleRgn, inLeftDelta, inTopDelta );
+    
+    // calculate a mask region to not copy the non-visble portions of the window from the port
+    StRegionFromPool copyMaskRgn;
+    if ( !copyMaskRgn ) return;
+    ::DiffRgn(totalVisRgn, nonVisibleRgn, copyMaskRgn);
+    
+    // use copybits to simulate a ScrollRect()
+    RGBColor black = { 0, 0, 0 };
+    RGBColor white = { 0xFFFF, 0xFFFF, 0xFFFF } ;
+    ::RGBForeColor(&black);
+    ::RGBBackColor(&white);
+    ::PenNormal();  
+    ::CopyBits ( 
+      &mWindowPtr->portBits, 
+      &mWindowPtr->portBits, 
+      &source, 
+      &dest, 
+      srcCopy, 
+      copyMaskRgn);
 
-		// union the update regions together and invalidate them
-		::UnionRgn(nonVisibleRgn, updateRgn, updateRgn);
-	}
-	
+    // union the update regions together and invalidate them
+    ::UnionRgn(nonVisibleRgn, updateRgn, updateRgn);
+  }
+  
   // If the region to be scrolled contains regions which are currently dirty,
   // we must scroll those too, and union them with the updateRgn.
-  // get a copy of the dirty region
-  StRegionFromPool  winUpdateRgn;
-  if (!winUpdateRgn) return;
+  // get a copy of the dirty region.
   ::BeginUpdate(mWindowPtr);
-  ::CopyRgn(mWindowPtr->visRgn, winUpdateRgn);
+  ::CopyRgn(mWindowPtr->visRgn, localVisRgn);   // re-use localVisRgn
   ::EndUpdate(mWindowPtr);
-  
+
   StRegionFromPool  dirtyRgn;
   if (!dirtyRgn) return;
   // get only the part of the dirtyRgn that intersects the frame
-  ::SectRgn(winUpdateRgn, totalVisRgn, dirtyRgn);
+  ::SectRgn(localVisRgn, totalVisRgn, dirtyRgn);
   // offset by the amount scrolled
   ::OffsetRgn(dirtyRgn, inLeftDelta, inTopDelta);
   // now intersect with the frame again
@@ -1850,12 +1859,12 @@ nsWindow::ScrollBits ( Rect & inRectToScroll, PRInt32 inLeftDelta, PRInt32 inTop
   
   // we also need to re-dirty the dirty rgn outside out frame,
   // since BeginUpdate/EndUpdate cleared it.
-  ::DiffRgn(winUpdateRgn, totalVisRgn, winUpdateRgn);
+  ::DiffRgn(localVisRgn, totalVisRgn, localVisRgn);
   // and add it to the dirty region
-  ::UnionRgn(updateRgn, winUpdateRgn, updateRgn);
+  ::UnionRgn(updateRgn, localVisRgn, updateRgn);
+  
+  ::InvalWindowRgn(mWindowPtr, updateRgn);
   
-	::InvalWindowRgn(mWindowPtr, updateRgn);
-	
 #endif // !TARGET_CARBON
 }
 
@@ -1867,7 +1876,7 @@ nsWindow::ScrollBits ( Rect & inRectToScroll, PRInt32 inLeftDelta, PRInt32 inTop
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsWindow::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
 {
-	if (! mVisible)
+	if (! mVisible || !ContainerHierarchyIsVisible())
 		return NS_OK;
 
   nsRect scrollRect;	
@@ -1962,7 +1971,7 @@ PRBool nsWindow::ConvertStatus(nsEventStatus aStatus)
 NS_IMETHODIMP nsWindow::DispatchEvent(nsGUIEvent* event, nsEventStatus& aStatus)
 {
   aStatus = nsEventStatus_eIgnore;
-	if (! mDestructorCalled)
+	if (mEnabled && !mDestructorCalled)
 	{
 		nsIWidget* aWidget = event->widget;
 		NS_IF_ADDREF(aWidget);
@@ -2012,11 +2021,10 @@ PRBool nsWindow::DispatchMouseEvent(nsMouseEvent &aEvent)
   }
 
   // call the event callback 
-  if (nsnull != mEventCallback) 
-  	{
+  if (mEventCallback && (mEnabled || aEvent.message == NS_MOUSE_EXIT)) {
     result = (DispatchWindowEvent(aEvent));
     return result;
-  	}
+  }
 
   if (nsnull != mMouseListener) {
     switch (aEvent.message) {
@@ -2152,7 +2160,11 @@ void nsWindow::CalcWindowRegions()
 	// intersect with all the parents
 	nsWindow* parent = (nsWindow*)mParent;
 	nsPoint origin(-mBounds.x, -mBounds.y);
-	while (parent)
+
+	// must stop enumerating when hitting a native window boundary
+	if (!mIsTopWidgetWindow)
+	{
+		while (parent && (!parent->mIsTopWidgetWindow))
 	{
 		if (parent->mWindowRegion)
 		{
@@ -2163,6 +2175,7 @@ void nsWindow::CalcWindowRegions()
 		origin.x -= parent->mBounds.x;
 		origin.y -= parent->mBounds.y;
 		parent = (nsWindow*)parent->mParent;
+		}
 	}
 
 	//------
@@ -2299,7 +2312,7 @@ PRBool nsWindow::PointInWidget(Point aThePoint)
 //-------------------------------------------------------------------------
 nsWindow*  nsWindow::FindWidgetHit(Point aThePoint)
 {
-	if (!mVisible || !PointInWidget(aThePoint))
+	if (!mVisible || !ContainerHierarchyIsVisible() || !PointInWidget(aThePoint))
 		return nsnull;
 
 	nsWindow* widgetHit = this;
@@ -2338,7 +2351,7 @@ nsWindow*  nsWindow::FindWidgetHit(Point aThePoint)
 //-------------------------------------------------------------------------
 // WidgetToScreen
 //		Walk up the parent tree, converting the given rect to global coordinates.
-//      This is similiar to CalcOffset() but we can't use GetBounds() because it
+//      This is similar to CalcOffset() but we can't use GetBounds() because it
 //      only knows how to give us local coordinates.
 //		@param aLocalRect  -- rect in local coordinates of this widget
 //		@param aGlobalRect -- |aLocalRect| in global coordinates
