@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Sean Echevarria <Sean@Beatnik.com>
+ *   Blake Ross <blaker@netscape.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or 
@@ -37,6 +38,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include "nsDataObj.h"
+#include "nsClipboard.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsVoidArray.h"
@@ -50,6 +52,7 @@
 #include "nsIImage.h"
 #include "nsImageClipboard.h"
 #include "prprf.h"
+#include "nsCRT.h"
 
 #include "OLE2.h"
 #include "URLMON.h"
@@ -362,7 +365,7 @@ HRESULT nsDataObj::AddGetFormat(FORMATETC& aFE)
 
 //-----------------------------------------------------
 HRESULT 
-nsDataObj::GetBitmap ( nsAReadableCString& , FORMATETC&, STGMEDIUM& )
+nsDataObj::GetBitmap ( const nsACString& , FORMATETC&, STGMEDIUM& )
 {
   PRNTDEBUG("nsDataObj::GetBitmap\n");
 	return ResultFromScode(E_NOTIMPL);
@@ -376,15 +379,26 @@ nsDataObj::GetBitmap ( nsAReadableCString& , FORMATETC&, STGMEDIUM& )
 // nsIImage, so just QI it.
 //
 HRESULT 
-nsDataObj :: GetDib ( nsAReadableCString& inFlavor, FORMATETC &, STGMEDIUM & aSTG )
+nsDataObj :: GetDib ( const nsACString& inFlavor, FORMATETC &, STGMEDIUM & aSTG )
 {
   PRNTDEBUG("nsDataObj::GetDib\n");
   ULONG result = E_FAIL;
   
+  
   PRUint32 len = 0;
   nsCOMPtr<nsISupports> genericDataWrapper;
   mTransferable->GetTransferData(PromiseFlatCString(inFlavor).get(), getter_AddRefs(genericDataWrapper), &len);
   nsCOMPtr<nsIImage> image ( do_QueryInterface(genericDataWrapper) );
+  if ( !image ) {
+    // In the 0.9.4 timeframe, I had some embedding clients put the nsIImage directly into the
+    // transferable. Newer code, however, wraps the nsIImage in a nsISupportsInterfacePointer.
+    // We should be backwards compatibile with code already out in the field. If we can't find
+    // the image directly out of the transferable,  unwrap the image from its wrapper.
+    nsCOMPtr<nsISupportsInterfacePointer> ptr(do_QueryInterface(genericDataWrapper));
+    if ( ptr )
+      ptr->GetData(getter_AddRefs(image));
+  }
+  
   if ( image ) {
     // use a the helper class to build up a bitmap. We now own the bits,
     // and pass them back to the OS in |aSTG|.
@@ -477,8 +491,18 @@ nsDataObj :: GetFileDescriptorInternetShortcut ( FORMATETC& aFE, STGMEDIUM& aSTG
       } 
     }
     titleStr[lenTitleStr] = '\0';
+ 
+    // replace any forbidden characters with
+    // dash, a la IE
+    char* pos;
+    const char* forbiddenChars = "\\:/";
+    for (int i = 0; i < PL_strlen(forbiddenChars); ++i) {
+      while (pos = PL_strchr((const char*)titleStr, forbiddenChars[i])) {
+        *pos = '-';
+      }
+    }
 
-    // one file in the file descriptor block
+    // one file in the file file:///c:/testcases/hellodescriptor block
     fileGroupDesc->cItems = 1;
     fileGroupDesc->fgd[0].dwFlags = FD_LINKUI;
     
@@ -593,7 +617,7 @@ nsDataObj :: IsInternetShortcut ( )
 
 
 //-----------------------------------------------------
-HRESULT nsDataObj::GetText(nsAReadableCString & aDataFlavor, FORMATETC& aFE, STGMEDIUM& aSTG)
+HRESULT nsDataObj::GetText(const nsACString & aDataFlavor, FORMATETC& aFE, STGMEDIUM& aSTG)
 {
   void* data;
   PRUint32   len;
@@ -618,9 +642,6 @@ HRESULT nsDataObj::GetText(nsAReadableCString & aDataFlavor, FORMATETC& aFE, STG
   aSTG.tymed          = TYMED_HGLOBAL;
   aSTG.pUnkForRelease = NULL;
 
-  // oddly, this isn't in the MSVC headers anywhere.
-  static int CF_HTML = ::RegisterClipboardFormat("HTML Format");
-
   // We play games under the hood and advertise flavors that we know we
   // can support, only they require a bit of conversion or munging of the data.
   // Do that here.
@@ -650,7 +671,7 @@ HRESULT nsDataObj::GetText(nsAReadableCString & aDataFlavor, FORMATETC& aFE, STG
       return ResultFromScode(S_OK);
     }
   }
-  else if ( aFE.cfFormat == CF_HTML ) {
+  else if ( aFE.cfFormat == nsClipboard::CF_HTML ) {
     // Someone is asking for win32's HTML flavor. Convert our html fragment
     // from unicode to UTF-8 then put it into a format specified by msft.
     NS_ConvertUCS2toUTF8 converter ( NS_REINTERPRET_CAST(PRUnichar*, data) );
@@ -880,9 +901,6 @@ nsDataObj :: BuildPlatformHTML ( const char* inOurHTML, char** outPlatformHTML )
   if( !buf )
     return NS_ERROR_FAILURE;
 
-  // Get clipboard id for HTML format...
-  static int cfid = RegisterClipboardFormat("HTML Format");;
-
   // Create a template string for the HTML header...
   strcpy(buf,
       "Version:0.9\r\n"
