@@ -30,21 +30,32 @@
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
 #include "nsIServiceManager.h"
-#define NS_IMPL_IDS
 #include "nsIPlatformCharset.h"
-#undef NS_IMPL_IDS
 #include "nsFilePicker.h"
 #include "nsILocalFile.h"
-#include "nsIFileChannel.h"
 #include "nsIURL.h"
+#include "nsIFileURL.h"
 #include "nsIStringBundle.h"
+#include "nsCRT.h"
 #include <windows.h>
 #include <SHLOBJ.H>
 
+#ifdef MOZ_UNICODE
+#include "nsString.h"
+#include "nsToolkit.h"
+#endif // MOZ_UNICODE
+
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
 
 NS_IMPL_ISUPPORTS1(nsFilePicker, nsIFilePicker)
 
+#ifdef MOZ_UNICODE
+nsString nsFilePicker::mLastUsedUnicodeDirectory;
+#endif
+char nsFilePicker::mLastUsedDirectory[MAX_PATH+1] = { 0 };
+
+#define MAX_EXTENSION_LENGTH 10
+
 //-------------------------------------------------------------------------
 //
 // nsFilePicker constructor
@@ -56,6 +67,7 @@ nsFilePicker::nsFilePicker()
   mWnd = NULL;
   mUnicodeEncoder = nsnull;
   mUnicodeDecoder = nsnull;
+  mSelectedType   = 1;
   mDisplayDirectory = do_CreateInstance("@mozilla.org/file/local;1");
 }
 
@@ -75,9 +87,193 @@ nsFilePicker::~nsFilePicker()
 // Show - Display the file dialog
 //
 //-------------------------------------------------------------------------
-NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
+#ifdef MOZ_UNICODE
+NS_IMETHODIMP nsFilePicker::ShowW(PRInt16 *aReturnVal)
 {
-  NS_ENSURE_ARG_POINTER(retval);
+  NS_ENSURE_ARG_POINTER(aReturnVal);
+
+  PRBool result = PR_FALSE;
+  PRUnichar fileBuffer[MAX_PATH+1];
+  wcsncpy(fileBuffer,  mDefault.get(), MAX_PATH);
+
+  nsAutoString htmExt(NS_LITERAL_STRING("html"));
+  PRUnichar *title = ToNewUnicode(mTitle);
+  nsAutoString initialDir;
+  mDisplayDirectory->GetPath(initialDir);
+
+  // If no display directory, re-use the last one.
+  if(initialDir.IsEmpty()) {
+    // Allocate copy of last used dir.
+    initialDir = mLastUsedUnicodeDirectory;
+  }
+
+  mUnicodeFile.SetLength(0);
+
+  if (mMode == modeGetFolder) {
+    PRUnichar dirBuffer[MAX_PATH+1];
+    wcsncpy(dirBuffer, initialDir.get(), MAX_PATH);
+
+    BROWSEINFOW browserInfo;
+    browserInfo.hwndOwner      = mWnd;
+    browserInfo.pidlRoot       = nsnull;
+    browserInfo.pszDisplayName = (LPWSTR)dirBuffer;
+    browserInfo.lpszTitle      = title;
+    browserInfo.ulFlags        = BIF_RETURNONLYFSDIRS;//BIF_STATUSTEXT | BIF_RETURNONLYFSDIRS;
+    browserInfo.lpfn           = nsnull;
+    browserInfo.lParam         = nsnull;
+    browserInfo.iImage         = nsnull;
+
+    // XXX UNICODE support is needed here --> DONE
+    LPITEMIDLIST list = ::SHBrowseForFolderW(&browserInfo);
+    if (list != NULL) {
+      result = ::SHGetPathFromIDListW(list, (LPWSTR)fileBuffer);
+      if (result) {
+          mUnicodeFile.Append(fileBuffer);
+      }
+  
+      // free PIDL
+      LPMALLOC pMalloc = NULL;
+      ::SHGetMalloc(&pMalloc);
+      if(pMalloc) {
+         pMalloc->Free(list);
+         pMalloc->Release();
+      }
+    }
+  }
+  else {
+
+    OPENFILENAMEW ofn;
+    memset(&ofn, 0, sizeof(ofn));
+
+    ofn.lStructSize = sizeof(ofn);
+
+    nsString filterBuffer = mFilterList;
+                                  
+    if (!initialDir.IsEmpty()) {
+      ofn.lpstrInitialDir = initialDir.get();
+    }
+    
+    ofn.lpstrTitle   = (LPCWSTR)title;
+    ofn.lpstrFilter  = (LPCWSTR)filterBuffer.get();
+    ofn.nFilterIndex = mSelectedType;
+    ofn.hwndOwner    = mWnd;
+    ofn.lpstrFile    = fileBuffer;
+    ofn.nMaxFile     = MAX_PATH;
+
+    ofn.Flags = OFN_NOCHANGEDIR | OFN_SHAREAWARE | OFN_LONGNAMES | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
+
+    if (!mDefaultExtension.IsEmpty()) {
+      ofn.lpstrDefExt = mDefaultExtension.get();
+    }
+    else {
+      // Get file extension from suggested filename
+      //  to detect if we are saving an html file
+      //XXX: nsIFile SHOULD HAVE A GetExtension() METHOD!
+      PRInt32 extIndex = mDefault.RFind(".");
+      if ( extIndex >= 0) {
+        nsAutoString ext;
+        mDefault.Right(ext, mDefault.Length() - extIndex);
+        // Should we test for ".cgi", ".asp", ".jsp" and other
+        // "generated" html pages?
+
+        if ( ext.EqualsIgnoreCase(".htm")  ||
+             ext.EqualsIgnoreCase(".html") ||
+             ext.EqualsIgnoreCase(".shtml") ) {
+          // This is supposed to append ".htm" if user doesn't supply an extension
+          //XXX Actually, behavior is sort of weird:
+          //    often appends ".html" even if you have an extension
+          //    It obeys your extension if you put quotes around name
+          ofn.lpstrDefExt = htmExt.get();
+        }
+      }
+    }
+
+    if (mMode == modeOpen) {
+      // FILE MUST EXIST!
+      ofn.Flags |= OFN_FILEMUSTEXIST;
+      result = ::GetOpenFileNameW(&ofn);
+    }
+    else if (mMode == modeSave) {
+      ofn.Flags |= OFN_NOREADONLYRETURN;
+      result = ::GetSaveFileNameW(&ofn);
+      if (!result) {
+        // Error, find out what kind.
+        if (::GetLastError() == ERROR_INVALID_PARAMETER ||
+            ::CommDlgExtendedError() == FNERR_INVALIDFILENAME) {
+          // probably the default file name is too long or contains illegal characters!
+          // Try again, without a starting file name.
+          ofn.lpstrFile[0] = 0;
+          result = ::GetSaveFileNameW(&ofn);
+        }
+      }
+    }
+    else {
+      NS_ASSERTION(0, "Only load, save and getFolder are supported modes"); 
+    }
+  
+    // Remember what filter type the user selected
+    mSelectedType = (PRInt16)ofn.nFilterIndex;
+
+    // Set user-selected location of file or directory
+    if (result == PR_TRUE) {
+      // I think it also needs a conversion here (to unicode since appending to nsString) 
+      // but doing that generates garbage file name, weird.
+      mUnicodeFile.Append(fileBuffer);
+    }
+
+  }
+
+  if (title)
+    nsMemory::Free( title );
+
+  if (result) {
+    PRInt16 returnOKorReplace = returnOK;
+
+    // Remember last used directory.
+    nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
+    NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
+
+    // work around.  InitWithPath() will convert UCS2 to FS path !!!  corrupts unicode 
+    file->InitWithPath(mUnicodeFile);
+    nsCOMPtr<nsIFile> dir;
+    if (NS_SUCCEEDED(file->GetParent(getter_AddRefs(dir)))) {
+      nsCOMPtr<nsILocalFile> localDir(do_QueryInterface(dir));
+      if (localDir) {
+        nsAutoString newDir;
+        localDir->GetPath(newDir);
+        if(!newDir.IsEmpty())
+          mLastUsedUnicodeDirectory.Assign(newDir);
+        // Update mDisplayDirectory with this directory, also.
+        // Some callers rely on this.
+        mDisplayDirectory->InitWithPath(mLastUsedUnicodeDirectory);
+      }
+    }
+
+    if (mMode == modeSave) {
+      // Windows does not return resultReplace,
+      //   we must check if file already exists
+      PRBool exists = PR_FALSE;
+      file->Exists(&exists);
+      if (exists)
+        returnOKorReplace = returnReplace;
+    }
+    *aReturnVal = returnOKorReplace;
+  }
+  else {
+    *aReturnVal = returnCancel;
+  }
+  return NS_OK;
+}
+#endif
+
+NS_IMETHODIMP nsFilePicker::Show(PRInt16 *aReturnVal)
+#ifdef MOZ_UNICODE
+{
+  return ShowW(aReturnVal);
+}
+#else
+{
+  NS_ENSURE_ARG_POINTER(aReturnVal);
 
   PRBool result = PR_FALSE;
   char fileBuffer[MAX_PATH+1] = "";
@@ -95,15 +291,20 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
   char *title = ConvertToFileSystemCharset(mTitle.get());
   if (nsnull == title)
     title = ToNewCString(mTitle);
-  char *initialDir;
-  mDisplayDirectory->GetPath(&initialDir);
+  nsCAutoString initialDir;
+  mDisplayDirectory->GetNativePath(initialDir);
+  // If no display directory, re-use the last one.
+  if(initialDir.IsEmpty()) {
+    // Allocate copy of last used dir.
+    initialDir = mLastUsedDirectory;
+  }
 
   mFile.SetLength(0);
 
   if (mMode == modeGetFolder) {
 
     char dirBuffer[MAX_PATH+1];
-    PL_strncpy(dirBuffer, initialDir, MAX_PATH);
+    PL_strncpy(dirBuffer, initialDir.get(), MAX_PATH);
     BROWSEINFO browserInfo;
     browserInfo.hwndOwner      = mWnd;
     browserInfo.pidlRoot       = nsnull;
@@ -138,6 +339,8 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
 
     ofn.lStructSize = sizeof(ofn);
 
+    char extensionBuffer[MAX_EXTENSION_LENGTH+1] = "";
+    
     PRInt32 l = (mFilterList.Length()+2)*2;
     char *filterBuffer = (char*) nsMemory::Alloc(l);
     int len = WideCharToMultiByte(CP_ACP, 0,
@@ -148,10 +351,10 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     filterBuffer[len] = NULL;
     filterBuffer[len+1] = NULL;
                                   
-    if (initialDir && *initialDir) {
-      ofn.lpstrInitialDir = initialDir;
+    if (!initialDir.IsEmpty()) {
+      ofn.lpstrInitialDir = initialDir.get();
     }
-
+    
     ofn.lpstrTitle   = title;
     ofn.lpstrFilter  = filterBuffer;
     ofn.nFilterIndex = mSelectedType;
@@ -161,22 +364,38 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
 
     ofn.Flags = OFN_NOCHANGEDIR | OFN_SHAREAWARE | OFN_LONGNAMES | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
 
-    // Get file extension from suggested filename
-    //  to detect if we are saving an html file
-    //XXX: nsIFile SHOULD HAVE A GetExtension() METHOD!
-    PRInt32 extIndex = mDefault.RFind(".");
-    if ( extIndex >= 0) {
-      nsAutoString ext;
-      mDefault.Right(ext, mDefault.Length() - extIndex);
-      // Should we test for ".cgi", ".asp", ".jsp" and other "generated" html pages?
-      if ( ext.EqualsIgnoreCase(".htm")  || 
-           ext.EqualsIgnoreCase(".html") || 
-           ext.EqualsIgnoreCase(".shtml") ) {
-        // This is supposed to append ".htm" if user doesn't supply an extension
-        //XXX Actually, behavior is sort of weird:
-        //    often appends ".html" even if you have an extension
-        //    It obeys your extension if you put quotes around name
-        ofn.lpstrDefExt = htmExt;
+    if (!mDefaultExtension.IsEmpty()) {
+      // Someone was cool and told us what to do
+      char *convertedExt = ConvertToFileSystemCharset(mDefaultExtension.get());
+      if (!convertedExt) {
+        mDefaultExtension.ToCString(extensionBuffer, MAX_EXTENSION_LENGTH);
+      }
+      else {
+        PL_strncpyz(extensionBuffer, convertedExt, MAX_EXTENSION_LENGTH+1);
+        nsMemory::Free( convertedExt );
+      }
+      ofn.lpstrDefExt = extensionBuffer;
+    }
+    else {
+      // Get file extension from suggested filename
+      //  to detect if we are saving an html file
+      //XXX: nsIFile SHOULD HAVE A GetExtension() METHOD!
+      PRInt32 extIndex = mDefault.RFind(".");
+      if ( extIndex >= 0) {
+        nsAutoString ext;
+        mDefault.Right(ext, mDefault.Length() - extIndex);
+        // Should we test for ".cgi", ".asp", ".jsp" and other
+        // "generated" html pages?
+
+        if ( ext.EqualsIgnoreCase(".htm")  ||
+             ext.EqualsIgnoreCase(".html") ||
+             ext.EqualsIgnoreCase(".shtml") ) {
+          // This is supposed to append ".htm" if user doesn't supply an extension
+          //XXX Actually, behavior is sort of weird:
+          //    often appends ".html" even if you have an extension
+          //    It obeys your extension if you put quotes around name
+          ofn.lpstrDefExt = htmExt;
+        }
       }
     }
 
@@ -186,12 +405,13 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
       result = ::GetOpenFileName(&ofn);
     }
     else if (mMode == modeSave) {
+      ofn.Flags |= OFN_NOREADONLYRETURN;
       result = ::GetSaveFileName(&ofn);
       if (!result) {
         // Error, find out what kind.
-        DWORD error = ::GetLastError();
-        if (error == ERROR_INVALID_PARAMETER) {
-          // Parameter error; probably the default file name is too long.
+        if (::GetLastError() == ERROR_INVALID_PARAMETER ||
+            ::CommDlgExtendedError() == FNERR_INVALIDFILENAME) {
+          // probably the default file name is too long or contains illegal characters!
           // Try again, without a starting file name.
           ofn.lpstrFile[0] = 0;
           result = ::GetSaveFileName(&ofn);
@@ -205,12 +425,6 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
     // Remember what filter type the user selected
     mSelectedType = (PRInt16)ofn.nFilterIndex;
 
-    // Store the current directory in mDisplayDirectory
-    char* newCurrentDirectory = NS_STATIC_CAST( char*, nsMemory::Alloc( MAX_PATH+1 ) );
-    VERIFY(::GetCurrentDirectory(MAX_PATH, newCurrentDirectory) > 0);
-    mDisplayDirectory->InitWithPath(newCurrentDirectory);
-    nsMemory::Free( newCurrentDirectory );
-
     // Clean up filter buffers
     if (filterBuffer)
       nsMemory::Free( filterBuffer );
@@ -224,51 +438,69 @@ NS_IMETHODIMP nsFilePicker::Show(PRInt16 *retval)
 
   }
 
-  if (initialDir)
-    nsMemory::Free(initialDir);
-
   if (title)
     nsMemory::Free( title );
 
   if (result) {
     PRInt16 returnOKorReplace = returnOK;
 
+    // Remember last used directory.
+    nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
+    NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
+
+    file->InitWithNativePath(mFile);
+    nsCOMPtr<nsIFile> dir;
+    if (NS_SUCCEEDED(file->GetParent(getter_AddRefs(dir)))) {
+      nsCOMPtr<nsILocalFile> localDir(do_QueryInterface(dir));
+      if (localDir) {
+        nsCAutoString newDir;
+        localDir->GetNativePath(newDir);
+        if(!newDir.IsEmpty())
+          PL_strncpyz(mLastUsedDirectory, newDir.get(), MAX_PATH+1);
+        // Update mDisplayDirectory with this directory, also.
+        // Some callers rely on this.
+        mDisplayDirectory->InitWithNativePath( nsDependentCString(mLastUsedDirectory) );
+      }
+    }
+
     if (mMode == modeSave) {
       // Windows does not return resultReplace,
       //   we must check if file already exists
-      nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
-
-      NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
-
-      file->InitWithPath(mFile.get());
-
       PRBool exists = PR_FALSE;
       file->Exists(&exists);
       if (exists)
         returnOKorReplace = returnReplace;
     }
-    *retval = returnOKorReplace;
+    *aReturnVal = returnOKorReplace;
   }
   else {
-    *retval = returnCancel;
+    *aReturnVal = returnCancel;
   }
   return NS_OK;
 }
-
-
+#endif
 
 NS_IMETHODIMP nsFilePicker::GetFile(nsILocalFile **aFile)
 {
   NS_ENSURE_ARG_POINTER(aFile);
 
+#ifdef MOZ_UNICODE
+  if (mUnicodeFile.IsEmpty())
+      return NS_OK;
+#else
   if (mFile.IsEmpty())
       return NS_OK;
+#endif
 
   nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
     
   NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
 
-  file->InitWithPath(mFile.get());
+#ifdef MOZ_UNICODE
+  file->InitWithPath(mUnicodeFile);
+#else
+  file->InitWithNativePath(mFile);
+#endif
 
   NS_ADDREF(*aFile = file);
 
@@ -280,7 +512,11 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
 {
   nsCOMPtr<nsILocalFile> file(do_CreateInstance("@mozilla.org/file/local;1"));
   NS_ENSURE_TRUE(file, NS_ERROR_FAILURE);
-  file->InitWithPath(mFile.get());
+#ifdef MOZ_UNICODE
+  file->InitWithPath(mUnicodeFile);
+#else
+  file->InitWithNativePath(mFile);
+#endif
 
   nsCOMPtr<nsIURI> uri;
   NS_NewFileURI(getter_AddRefs(uri), file);
@@ -300,6 +536,32 @@ NS_IMETHODIMP nsFilePicker::GetFileURL(nsIFileURL **aFileURL)
 NS_IMETHODIMP nsFilePicker::SetDefaultString(const PRUnichar *aString)
 {
   mDefault = aString;
+
+  //First, make sure the file name is not too long!
+  PRInt32 nameLength;
+  PRInt32 nameIndex = mDefault.RFind("\\");
+  if (nameIndex == kNotFound)
+    nameIndex = 0;
+  else
+    nameIndex ++;
+  nameLength = mDefault.Length() - nameIndex;
+  
+  if (nameLength > _MAX_FNAME) {
+    PRInt32 extIndex = mDefault.RFind(".");
+    if (extIndex == kNotFound)
+      extIndex = mDefault.Length();
+
+    //Let's try to shave the needed characters from the name part
+    PRInt32 charsToRemove = nameLength - _MAX_FNAME;
+    if (extIndex - nameIndex >= charsToRemove) {
+      mDefault.Cut(extIndex - charsToRemove, charsToRemove);
+    }
+  }
+
+  //Then, we need to replace illegal characters.
+  //At this stage, we cannot replace the backslash as the string might represent a file path.
+  mDefault.ReplaceChar(FILE_ILLEGAL_CHARACTERS, '-');
+
   return NS_OK;
 }
 
@@ -308,6 +570,25 @@ NS_IMETHODIMP nsFilePicker::GetDefaultString(PRUnichar **aString)
   return NS_ERROR_FAILURE;
 }
 
+//-------------------------------------------------------------------------
+//
+// The default extension to use for files
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP nsFilePicker::GetDefaultExtension(PRUnichar **aExtension)
+{
+  *aExtension = ToNewUnicode(mDefaultExtension);
+  if (!*aExtension)
+    return NS_ERROR_OUT_OF_MEMORY;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsFilePicker::SetDefaultExtension(const PRUnichar *aExtension)
+{
+  mDefaultExtension = aExtension;
+  return NS_OK;
+}
+
 //-------------------------------------------------------------------------
 //
 // Set the filter index
@@ -441,7 +722,7 @@ PRUnichar * nsFilePicker::ConvertFromFileSystemCharset(const char *inString)
 
   // converts from the file system charset to unicode
   if (NS_SUCCEEDED(rv)) {
-    PRInt32 inLength = nsCRT::strlen(inString);
+    PRInt32 inLength = strlen(inString);
     PRInt32 outLength;
     rv = mUnicodeDecoder->GetMaxLength(inString, inLength, &outLength);
     if (NS_SUCCEEDED(rv)) {
@@ -466,7 +747,12 @@ nsFilePicker::AppendFilter(const PRUnichar *aTitle, const PRUnichar *aFilter)
 {
   mFilterList.Append(aTitle);
   mFilterList.Append(PRUnichar('\0'));
-  mFilterList.Append(aFilter);
+
+  if (!nsCRT::strcmp(aFilter, NS_LITERAL_STRING("..apps").get()))
+    mFilterList.Append(NS_LITERAL_STRING("*.exe; *.com"));
+  else
+    mFilterList.Append(aFilter);
+
   mFilterList.Append(PRUnichar('\0'));
 
   return NS_OK;
