@@ -83,7 +83,11 @@ class nsIMM
     typedef LONG (CALLBACK *SetCStatusPtr)  (HIMC,DWORD,DWORD);
     typedef LONG (CALLBACK *NotifyIMEPtr)   (HIMC,DWORD,DWORD,DWORD);
     typedef LONG (CALLBACK *SetCandWindowPtr)  (HIMC,LPCANDIDATEFORM);
+    typedef LONG (CALLBACK *SetCompWindowPtr)  (HIMC,LPCOMPOSITIONFORM);
+    typedef LONG (CALLBACK *GetCompFontPtr)  (HIMC, PLOGFONT);
+    typedef LONG (CALLBACK *SetCompFontPtr)  (HIMC, PLOGFONT);
     typedef LONG (CALLBACK *GetCompWindowPtr)  (HIMC,LPCOMPOSITIONFORM);
+    typedef LONG (CALLBACK *GetPropertyPtr)  (HKL, DWORD);
 
 public:
 
@@ -122,6 +126,24 @@ public:
       
 	    mGetCompositionWindow=(mInstance) ? (GetCompWindowPtr)GetProcAddress(mInstance,"ImmGetCompositionWindow") : 0;
 	    NS_ASSERTION(mGetCompositionWindow!=NULL,"nsIMM.ImmGetCompositionWindow failed.");
+      
+	    mGetCompositionFontA=(mInstance) ? (GetCompFontPtr)GetProcAddress(mInstance,"ImmGetCompositionFontA") : 0;
+	    NS_ASSERTION(mGetCompositionFontA!=NULL,"nsIMM.ImmGetCompositionFontA failed.");
+
+	    mGetCompositionFontW=(mInstance) ? (GetCompFontPtr)GetProcAddress(mInstance,"ImmGetCompositionFontW") : 0;
+	    NS_ASSERTION(mGetCompositionFontW!=NULL,"nsIMM.ImmGetCompositionFontW failed.");
+
+	    mSetCompositionFontA=(mInstance) ? (SetCompFontPtr)GetProcAddress(mInstance,"ImmSetCompositionFontA") : 0;
+	    NS_ASSERTION(mSetCompositionFontA!=NULL,"nsIMM.ImmSetCompositionFontA failed.");
+
+	    mSetCompositionFontW=(mInstance) ? (SetCompFontPtr)GetProcAddress(mInstance,"ImmSetCompositionFontW") : 0;
+	    NS_ASSERTION(mSetCompositionFontW!=NULL,"nsIMM.ImmSetCompositionFontW failed.");
+
+	    mSetCompositionWindow=(mInstance) ? (SetCompWindowPtr)GetProcAddress(mInstance,"ImmSetCompositionWindow") : 0;
+	    NS_ASSERTION(mSetCompositionWindow!=NULL,"nsIMM.ImmSetCompositionWindow failed.");
+
+	    mGetProperty=(mInstance) ? (GetPropertyPtr)GetProcAddress(mInstance,"ImmGetProperty") : 0;
+	    NS_ASSERTION(mGetProperty!=NULL,"nsIMM.ImmGetProperty failed.");
     }
 
     ~nsIMM() {
@@ -137,6 +159,12 @@ public:
       mNotifyIME=0;
       mSetCandiateWindow=0;
       mGetCompositionWindow=0;
+      mSetCompositionWindow=0;
+      mGetCompositionFontA=0;
+      mGetCompositionFontA=0;
+      mSetCompositionFontW=0;
+      mSetCompositionFontW=0;
+      mGetProperty=0;
     }
 
     LONG GetCompositionStringA(HIMC h,DWORD d1,LPVOID v,DWORD d2) {
@@ -171,10 +199,34 @@ public:
       return (mSetCandiateWindow) ? mSetCandiateWindow(h,l) : 0L;
     }
 
+    LONG SetCompositionWindow(HIMC h,LPCOMPOSITIONFORM l) {
+      return (mSetCompositionWindow) ? mSetCompositionWindow(h,l) : 0L;
+    }
+
+    LONG GetCompositionFontA(HIMC h, PLOGFONT l) {
+      return (mGetCompositionFontA) ? mGetCompositionFontA(h,l) : 0L;
+    }
+
+    LONG GetCompositionFontW(HIMC h, PLOGFONT l) {
+      return (mGetCompositionFontW) ? mGetCompositionFontW(h,l) : 0L;
+    }
+
+    LONG SetCompositionFontA(HIMC h, PLOGFONT l) {
+      return (mSetCompositionFontA) ? mSetCompositionFontA(h,l) : 0L;
+    }
+
+    LONG SetCompositionFontW(HIMC h, PLOGFONT l) {
+      return (mSetCompositionFontW) ? mSetCompositionFontW(h,l) : 0L;
+    }
+
     LONG GetCompositionWindow(HIMC h,LPCOMPOSITIONFORM l) {
       return (mGetCompositionWindow) ? mGetCompositionWindow(h,l) : 0L;
     }
 
+    LONG GetProperty(HKL hKL, DWORD dwIndex) {
+      return (mGetProperty) ? mGetProperty(hKL, dwIndex) : 0L;
+    }
+
 private:
 
     HINSTANCE mInstance;
@@ -186,7 +238,13 @@ private:
     SetCStatusPtr  mSetConversionStatus;
     NotifyIMEPtr   mNotifyIME;  
     SetCandWindowPtr  mSetCandiateWindow;   
+    SetCompWindowPtr  mSetCompositionWindow;   
+    GetCompFontPtr  mGetCompositionFontA;   
+    GetCompFontPtr  mGetCompositionFontW;   
+    SetCompFontPtr  mSetCompositionFontA;   
+    SetCompFontPtr  mSetCompositionFontW;   
     GetCompWindowPtr  mGetCompositionWindow;   
+    GetPropertyPtr  mGetProperty;
 };
 
 /**
@@ -258,7 +316,8 @@ public:
                                    PRInt32 aWidth,
                                    PRInt32 aHeight,
                                    PRBool   aRepaint);
-    NS_IMETHOD              Enable(PRBool bState);
+    NS_IMETHOD              Enable(PRBool aState);
+    NS_IMETHOD              IsEnabled(PRBool *aState);
     NS_IMETHOD              SetFocus(PRBool aRaise);
     NS_IMETHOD              GetBounds(nsRect &aRect);
     NS_IMETHOD              GetClientBounds(nsRect &aRect);
@@ -266,6 +325,7 @@ public:
     virtual nsIFontMetrics* GetFont(void);
     NS_IMETHOD              SetFont(const nsFont &aFont);
     NS_IMETHOD              SetCursor(nsCursor aCursor);
+    NS_IMETHOD              HideWindowChrome(PRBool aShouldHide);
     NS_IMETHOD              Validate();
     NS_IMETHOD              Invalidate(PRBool aIsSynchronous);
     NS_IMETHOD              Invalidate(const nsRect & aRect, PRBool aIsSynchronous);
@@ -279,7 +339,7 @@ public:
     NS_IMETHOD              ScrollWidgets(PRInt32 aDx, PRInt32 aDy);
     NS_IMETHOD              ScrollRect(nsRect &aRect, PRInt32 aDx, PRInt32 aDy);
     NS_IMETHOD              SetTitle(const nsString& aTitle); 
-    NS_IMETHOD              SetIcon(const nsAReadableString& aIconSpec); 
+    NS_IMETHOD              SetIcon(const nsAString& aIconSpec); 
     NS_IMETHOD              SetMenuBar(nsIMenuBar * aMenuBar) { return NS_ERROR_FAILURE; } 
     NS_IMETHOD              ShowMenuBar(PRBool aShow)         { return NS_ERROR_FAILURE; } 
     NS_IMETHOD              WidgetToScreen(const nsRect& aOldRect, nsRect& aNewRect);
@@ -310,7 +370,7 @@ public:
             HWND            GetWindowHandle() { return mWnd; }
             WNDPROC         GetPrevWindowProc() { return mPrevWndProc; }
 
-    virtual PRBool          DispatchMouseEvent(PRUint32 aEventType, nsPoint* aPoint = nsnull);
+    virtual PRBool          DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam = NULL, nsPoint* aPoint = nsnull);
 #ifdef ACCESSIBILITY
     virtual PRBool          DispatchAccessibleEvent(PRUint32 aEventType, nsIAccessible** aAccessible, nsPoint* aPoint = nsnull);
     void                    CreateRootAccessible();
@@ -347,6 +407,10 @@ protected:
      // Allow Derived classes to modify the height that is passed
      // when the window is created or resized.
     virtual PRInt32         GetHeight(PRInt32 aProposedHeight);
+#ifdef MOZ_UNICODE
+    virtual LPCWSTR         WindowClassW();
+    virtual LPCWSTR         WindowPopupClassW();
+#endif
     virtual LPCTSTR         WindowClass();
     virtual LPCTSTR         WindowPopupClass();
     virtual DWORD           WindowStyle();
@@ -361,8 +425,8 @@ protected:
 
     BOOL                    OnChar(UINT mbcsCharCode, UINT virtualKeyCode, bool isMultibyte);
 
-    BOOL                    OnKeyDown( UINT aVirtualKeyCode, UINT aScanCode);
-    BOOL                    OnKeyUp( UINT aVirtualKeyCode, UINT aScanCode);
+    BOOL                    OnKeyDown( UINT aVirtualKeyCode, UINT aScanCode, LPARAM aKeyCode);
+    BOOL                    OnKeyUp( UINT aVirtualKeyCode, UINT aScanCode, LPARAM aKeyCode);
     UINT                    MapFromNativeToDOM(UINT aNativeKeyCode);
 
 
@@ -379,7 +443,7 @@ protected:
     BOOL                    OnIMEReconvert(LPARAM aData, LRESULT *oResult, PRBool aUseUnicode);
 
     ULONG                   IsSpecialChar(UINT aVirtualKeyCode, WORD *aAsciiKey);
-    virtual PRBool          DispatchKeyEvent(PRUint32 aEventType, WORD aCharCode, UINT aVirtualCharCode);
+    virtual PRBool          DispatchKeyEvent(PRUint32 aEventType, WORD aCharCode, UINT aVirtualCharCode, LPARAM aKeyCode);
 
     virtual PRBool          DispatchFocus(PRUint32 aEventType, PRBool isMozWindowTakingFocus);
     virtual PRBool          OnScroll(UINT scrollCode, int cPos);
@@ -400,7 +464,7 @@ protected:
 
     void GetNonClientBounds(nsRect &aRect);
     void HandleTextEvent(HIMC hIMEContext, PRBool aCheckAttr=PR_TRUE);
-    void HandleStartComposition(HIMC hIMEContext);
+    BOOL HandleStartComposition(HIMC hIMEContext);
     void HandleEndComposition(void);
     void MapDBCSAtrributeArrayToUnicodeOffsets(PRUint32* textRangeListLengthResult, nsTextRangeArray* textRangeListResult);
 
@@ -417,22 +481,29 @@ private:
 protected:
     nsSize      mLastSize;
     static      nsWindow* gCurrentWindow;
-    PRBool      mIsTopWidgetWindow;
     nsPoint     mLastPoint;
     HWND        mWnd;
 #if 0
     HPALETTE    mPalette;
 #endif
     WNDPROC     mPrevWndProc;
-
-    PRBool      mHas3DBorder;
     HBRUSH      mBrush;
-    PRBool      mIsShiftDown;
-    PRBool      mIsControlDown;
-    PRBool      mIsAltDown;
-    PRBool      mIsDestroying;
-    PRBool      mOnDestroyCalled;
-    PRBool      mIsVisible;
+
+    PRPackedBool  mIsTopWidgetWindow;
+    PRPackedBool  mHas3DBorder;
+    PRPackedBool  mIsShiftDown;
+    PRPackedBool  mIsControlDown;
+    PRPackedBool  mIsAltDown;
+    PRPackedBool  mIsDestroying;
+    PRPackedBool  mOnDestroyCalled;
+    PRPackedBool  mIsVisible;
+    PRPackedBool  mIMEIsComposing;
+    PRPackedBool  mIMEIsStatusChanged;
+    PRPackedBool  mIsInMouseCapture;
+    PRPackedBool  mIsInMouseWheelProcessing;
+
+    char        mLeadByte;
+
     // XXX Temporary, should not be caching the font
     nsFont *    mFont;
 
@@ -441,10 +512,12 @@ protected:
 
     PRInt32       mMenuCmdId;
 
+    // Window styles used by this window before chrome was hidden
+    DWORD       mOldStyle;
+    DWORD       mOldExStyle;
+
 	// For Input Method Support
 	DWORD		mIMEProperty;
-	PRBool		mIMEIsComposing;
-  PRBool    mIMEIsStatusChanged;
 	nsCString*	mIMECompString;
 	nsString*	mIMECompUnicode;
 	PRUint8*	mIMEAttributeString;
@@ -460,9 +533,6 @@ protected:
 	static UINT		gCurrentKeyboardCP;
 	static HKL		gKeyboardLayout;
 
-    PRBool  mIsInMouseCapture;
-    PRBool  mIsInMouseWheelProcessing;
-
     // Drag & Drop
     nsNativeDragTarget * mNativeDragTarget;
 
@@ -489,10 +559,18 @@ protected:
     static UINT uWM_MSIME_MOUSE;     // mouse messge for MSIME
     static UINT uWM_ATOK_RECONVERT;  // reconvert messge for ATOK
 
+    // Heap dump
+    static UINT uWM_HEAP_DUMP;       // Dump heap to a file
+
 #ifdef ACCESSIBILITY
     IAccessible* mRootAccessible;
     static BOOL gIsAccessibilityOn;
 #endif
+
+   static BOOL CALLBACK BroadcastMsgToChildren(HWND aWnd, LPARAM aMsg);
+   static BOOL CALLBACK BroadcastMsg(HWND aTopWindow, LPARAM aMsg);
+public:
+   static void GlobalMsgWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
 };
 
 //
@@ -502,7 +580,7 @@ class ChildWindow : public nsWindow {
 
 public:
                             ChildWindow(){}
-    PRBool          DispatchMouseEvent(PRUint32 aEventType, nsPoint* aPoint = nsnull);
+    PRBool          DispatchMouseEvent(PRUint32 aEventType, WPARAM wParam = NULL, nsPoint* aPoint = nsnull);
 
 protected:
     virtual DWORD           WindowStyle();
