@@ -43,7 +43,7 @@
 #include "nsGfxCIID.h"
 #include "nsWidgetsCID.h"
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 #include "nsIServiceManager.h"
 #include "nsIPref.h"
 
@@ -92,7 +92,7 @@ nsBaseWidget::nsBaseWidget()
   printf("WIDGETS+ = %d\n", gNumWidgets);
 #endif
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
     debug_RegisterPrefCallbacks();
 #endif
 
@@ -461,6 +461,21 @@ NS_IMETHODIMP nsBaseWidget::GetWindowType(nsWindowType& aWindowType)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsBaseWidget::SetWindowType(nsWindowType aWindowType) 
+{
+  mWindowType = aWindowType;
+  return NS_OK;
+}
+
+//-------------------------------------------------------------------------
+//
+// Hide window borders/decorations for this widget
+//
+//-------------------------------------------------------------------------
+NS_IMETHODIMP nsBaseWidget::HideWindowChrome(PRBool aShouldHide)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 //-------------------------------------------------------------------------
 //
@@ -469,10 +484,10 @@ NS_IMETHODIMP nsBaseWidget::GetWindowType(nsWindowType& aWindowType)
 //-------------------------------------------------------------------------
 nsIRenderingContext* nsBaseWidget::GetRenderingContext()
 {
-  static NS_DEFINE_CID(kRenderingContextCID, NS_RENDERING_CONTEXT_CID);
-  nsresult rv;
+  nsresult                      rv;
+  nsCOMPtr<nsIRenderingContext> renderingCtx;
 
-  nsCOMPtr<nsIRenderingContext> renderingCtx = do_CreateInstance(kRenderingContextCID, &rv);
+  rv = mContext->CreateRenderingContextInstance(*getter_AddRefs(renderingCtx));
   if (NS_SUCCEEDED(rv)) {
     rv = renderingCtx->Init(mContext, this);
     if (NS_SUCCEEDED(rv)) {
@@ -490,7 +505,7 @@ nsIRenderingContext* nsBaseWidget::GetRenderingContext()
     NS_WARNING("GetRenderingContext: Cannot create RenderingContext.");
   }  
   
-  return nsnull;  
+  return nsnull;
 }
 
 //-------------------------------------------------------------------------
@@ -544,13 +559,6 @@ void nsBaseWidget::OnDestroy()
 }
 
 
-NS_METHOD nsBaseWidget::SetWindowType(nsWindowType aWindowType) 
-{
-  mWindowType = aWindowType;
-  return NS_OK;
-}
-
-
 NS_METHOD nsBaseWidget::SetBorderStyle(nsBorderStyle aBorderStyle)
 {
   mBorderStyle = aBorderStyle;
@@ -796,12 +804,12 @@ nsBaseWidget::GetAttention() {
 }
 
 NS_IMETHODIMP
-nsBaseWidget::SetIcon(const nsAReadableString&)
+nsBaseWidget::SetIcon(const nsAString&)
 {
   return NS_OK;
 }
 
-#ifdef NS_DEBUG
+#ifdef DEBUG
 //////////////////////////////////////////////////////////////
 //
 // Convert a GUI event message code to a string.
@@ -932,7 +940,8 @@ static PRBool debug_GetBoolPref(nsIPref * aPrefs,const char * aPrefName)
   return value;
 }
 //////////////////////////////////////////////////////////////
-static PRBool debug_GetCachedBoolPref(const char * aPrefName)
+PRBool
+nsBaseWidget::debug_GetCachedBoolPref(const char * aPrefName)
 {
   NS_ASSERTION(nsnull != aPrefName,"cmon, pref name is null.");
 
@@ -1168,7 +1177,7 @@ nsBaseWidget::debug_DumpInvalidate(FILE *                aFileOut,
 }
 //////////////////////////////////////////////////////////////
 
-#endif // NS_DEBUG
+#endif // DEBUG
 
 
 
