@@ -41,23 +41,14 @@
 #include "nsHashtableEnumerator.h"
 #include "nsXPIDLString.h"
 #include "nsCRT.h"
-
 #include "nsIObserverService.h"
 
-#ifndef OBSOLETE_MODULE_LOADING
-#include "nsObsoleteModuleLoading.h"
-#endif
-
 #if defined(XP_MAC)  // sdagley dougt fix
 #include <Files.h>
 #include <Errors.h>
 #include "nsILocalFileMac.h"
 #endif
 
-#define PRINT_CRITICAL_ERROR_TO_SCREEN 1
-#define USE_REGISTRY 1
-#define XPCOM_USE_NSGETFACTORY 1
-
 // Logging of debug output
 #ifdef MOZ_LOGGING
 #define FORCE_PR_LOG /* Allow logging in the release build */
@@ -66,7 +57,7 @@
 extern PRLogModuleInfo *nsComponentManagerLog;
 
 nsNativeComponentLoader::nsNativeComponentLoader() :
-    mRegistry(nsnull), mCompMgr(nsnull), mDllStore(nsnull)
+    mCompMgr(nsnull), mDllStore(nsnull)
 {
     NS_INIT_REFCNT();
 }
@@ -81,13 +72,12 @@ nsDll_Destroy(nsHashKey *aKey, void *aData, void* closure)
 
 nsNativeComponentLoader::~nsNativeComponentLoader()
 {
-    mRegistry = nsnull;
     mCompMgr = nsnull;
 
     delete mDllStore;
 }
     
-NS_IMPL_ISUPPORTS1(nsNativeComponentLoader, nsIComponentLoader);
+NS_IMPL_THREADSAFE_ISUPPORTS1(nsNativeComponentLoader, nsIComponentLoader);
 
 NS_IMETHODIMP
 nsNativeComponentLoader::GetFactory(const nsIID & aCID,
@@ -104,8 +94,7 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
 
     /* Should this all live in xcDll? */
     nsDll *dll;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(nsnull, aLocation, &mod, &size, &dll);
+    rv = CreateDll(nsnull, aLocation, &dll);
     if (NS_FAILED(rv))
         return rv;
 
@@ -140,7 +129,7 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
         return rv;      // XXX translate error code?
     
     rv = GetFactoryFromModule(dll, aCID, _retval);
-#ifdef XPCOM_USE_NSGETFACTORY
+#ifdef OBSOLETE_MODULE_LOADING
     if (NS_FAILED(rv)) {
         if (rv == NS_ERROR_FACTORY_NOT_LOADED) {
             rv = GetFactoryFromNSGetFactory(dll, aCID, serviceMgr, _retval);
@@ -168,79 +157,15 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
 NS_IMETHODIMP
 nsNativeComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
 {
-    nsresult rv;
-
     mCompMgr = aCompMgr;
-    mRegistry = do_QueryInterface(aReg);
-    if (!mCompMgr || !mRegistry)
+    if (!mCompMgr)
         return NS_ERROR_INVALID_ARG;
 
-    rv = mRegistry->GetSubtree(nsIRegistry::Common, xpcomComponentsKeyName,
-                               &mXPCOMKey);
-    if (NS_FAILED(rv))
-        return rv;
-
-    if (!mDllStore) {
-        mDllStore = new nsObjectHashtable(nsnull, nsnull, // never copy
-                                          nsDll_Destroy, nsnull,
-                                          256, /* Thead Safe */ PR_TRUE);
-        if (!mDllStore)
-            return NS_ERROR_OUT_OF_MEMORY;
-    }
-
-
-    // Read in all dll entries and populate the mDllStore
-    nsCOMPtr<nsIEnumerator> dllEnum;
-    rv = mRegistry->EnumerateSubtrees( mXPCOMKey, getter_AddRefs(dllEnum));
-    if (NS_FAILED(rv)) return rv;
-
-    rv = dllEnum->First();
-    for (; NS_SUCCEEDED(rv) && (dllEnum->IsDone() != NS_OK); (rv = dllEnum->Next()))
-    {
-        nsCOMPtr<nsISupports> base;
-        rv = dllEnum->CurrentItem(getter_AddRefs(base));
-        if (NS_FAILED(rv))  continue;
-
-        // Get specific interface.
-        nsIID nodeIID = NS_IREGISTRYNODE_IID;
-        nsCOMPtr<nsIRegistryNode> node;
-        rv = base->QueryInterface( nodeIID, getter_AddRefs(node) );
-        if (NS_FAILED(rv)) continue;
-
-        // Get library name
-        nsXPIDLCString library;
-        rv = node->GetNameUTF8(getter_Copies(library));
-
-        if (NS_FAILED(rv)) continue;
-
-        char* uLibrary;
-        char* eLibrary = (char*)library.operator const char*();
-        PRUint32 length = strlen(eLibrary);
-        rv = mRegistry->UnescapeKey((PRUint8*)eLibrary, 1, &length, (PRUint8**)&uLibrary);
-        
-        if (NS_FAILED(rv)) continue;
-
-        if (uLibrary == nsnull)
-            uLibrary = eLibrary;
-
-        // Get key associated with library
-        nsRegistryKey libKey;
-        rv = node->GetKey(&libKey);
-        if (NS_SUCCEEDED(rv)) //  Cannot continue here, because we have to free unescape
-        {
-            // Create nsDll with this name
-            nsDll *dll = NULL;
-            PRInt64 lastModTime;
-            PRInt64 fileSize;
-            GetRegistryDllInfo(libKey, &lastModTime, &fileSize);
-            rv = CreateDll(NULL, uLibrary, &lastModTime, &fileSize, &dll);
-        }
-        if (uLibrary && (uLibrary != eLibrary))
-            nsMemory::Free(uLibrary);
-
-        if (NS_FAILED(rv)) continue;
-    }
-
+    mDllStore = new nsObjectHashtable(nsnull, nsnull, // never copy
+                                      nsDll_Destroy, nsnull,
+                                      256, /* Thead Safe */ PR_TRUE);
+    if (!mDllStore)
+        return NS_ERROR_OUT_OF_MEMORY;
     return NS_OK;
 }
 
@@ -250,13 +175,13 @@ nsNativeComponentLoader::AutoRegisterComponents(PRInt32 aWhen,
 {
 #ifdef DEBUG
     /* do we _really_ want to print this every time? */
-    printf("nsNativeComponentLoader: autoregistering begins.\n");
+    fprintf(stderr, "nsNativeComponentLoader: autoregistering begins.\n");
 #endif
 
     nsresult rv = RegisterComponentsInDir(aWhen, aDirectory);
 
 #ifdef DEBUG
-    printf("nsNativeComponentLoader: autoregistering %s\n",
+    fprintf(stderr, "nsNativeComponentLoader: autoregistering %s\n",
            NS_FAILED(rv) ? "FAILED" : "succeeded");
 #endif
 
@@ -352,7 +277,7 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     {
         rv = mobj->CanUnload(nsComponentManagerImpl::gComponentManager, &canUnload);
     }
-#ifndef OBSOLETE_MODULE_LOADING
+#ifdef OBSOLETE_MODULE_LOADING
     else
     {
         // Try the old method of module unloading
@@ -373,7 +298,6 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
 #endif /* OBSOLETE_MODULE_LOADING */
 
     mobj = nsnull; // Release our reference to the module object
-
     // When shutting down, whether we can unload the dll or not,
     // we will shutdown the dll to release any memory it has got
     if (when == nsIComponentManagerObsolete::NS_Shutdown)
@@ -470,11 +394,11 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
            ("nsNativeComponentLoader: Loaded \"%s\".", dll->GetDisplayPath()));
 
     // Tell the module to self register
+    nsCOMPtr<nsIFile> fs;
     nsCOMPtr<nsIModule> mobj;
     res = dll->GetModule(mCompMgr, getter_AddRefs(mobj));
     if (NS_SUCCEEDED(res))
     {
-        nsCOMPtr<nsIFile> fs;
         /*************************************************************
          * WARNING: Why are use introducing 'res2' here and then     *
          * later assigning it to 'res' rather than just using 'res'? *
@@ -497,7 +421,7 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
         }
         mobj = NULL;    // Force a release of the Module object before unload()
     }
-#ifndef OBSOLETE_MODULE_LOADING
+#ifdef OBSOLETE_MODULE_LOADING
     else
     {
         res = NS_ERROR_NO_INTERFACE;
@@ -518,13 +442,20 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
     // able to register on some later autoreg, after another component has been
     // installed.
     if (res != NS_ERROR_FACTORY_REGISTER_AGAIN) {
-        dll->Sync();
+        PRInt64 modTime;
+        if (!fs)
+            return res;
         
-        PRInt64 modTime, size;
-        dll->GetLastModifiedTime(&modTime);
-        dll->GetSize(&size);
-
-        SetRegistryDllInfo(registryLocation, &modTime, &size);
+        fs->GetLastModifiedTime(&modTime);
+        nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr);
+        if (!manager)
+            return NS_ERROR_FAILURE;
+        
+        nsCOMPtr<nsIFile> fs;
+        res = dll->GetDllSpec(getter_AddRefs(fs));
+        if (NS_FAILED(res)) return res;
+        
+        manager->SaveFileInfo(fs, registryLocation, modTime);
     }
 
     return res;
@@ -601,7 +532,8 @@ nsNativeComponentLoader::DumpLoadError(nsDll *dll,
 
     // Dump to screen if needed
 #ifdef PRINT_CRITICAL_ERROR_TO_SCREEN
-    printf("**************************************************\n"
+    fprintf(stderr, 
+           "**************************************************\n"
            "nsNativeComponentLoader: %s(%s) Load FAILED with error: %s\n"
            "**************************************************\n",
            aCallerName,
@@ -648,7 +580,7 @@ nsNativeComponentLoader::SelfUnregisterDll(nsDll *dll)
         if (NS_FAILED(res)) return res;
         mobj->UnregisterSelf(mCompMgr, fs, registryName);
     }
-#ifndef OBSOLETE_MODULE_LOADING
+#ifdef OBSOLETE_MODULE_LOADING
     else
     {
         res = NS_ERROR_NO_INTERFACE;
@@ -696,15 +628,19 @@ nsNativeComponentLoader::AutoUnregisterComponent(PRInt32 when,
     }
 
     nsDll *dll = NULL;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(component, persistentDescriptor, &mod, &size, &dll);
+    rv = CreateDll(component, persistentDescriptor, &dll);
     if (NS_FAILED(rv) || dll == NULL) return rv;
 
     rv = SelfUnregisterDll(dll);
 
     // Remove any autoreg info about this dll
-    if (NS_SUCCEEDED(rv))
-        RemoveRegistryDllInfo(persistentDescriptor);
+    nsCStringKey key(persistentDescriptor);
+    mDllStore->RemoveAndDelete(&key);
+    
+    nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr, &rv);
+    if (NS_FAILED(rv))
+        return rv;
+    manager->RemoveFileInfo(component, nsnull);
 
     PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
            ("nsNativeComponentLoader: AutoUnregistration for %s %s.",
@@ -773,24 +709,23 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
     }
 
 #else
-    char *leafName = NULL;
-    rv = component->GetLeafName(&leafName);
+    nsCAutoString leafName;
+    rv = component->GetNativeLeafName(leafName);
     if (NS_FAILED(rv)) return rv;
-    int flen = PL_strlen(leafName);
+    int flen = leafName.Length();
     for (int i=0; ValidDllExtensions[i] != NULL; i++)
     {
         int extlen = PL_strlen(ValidDllExtensions[i]);
             
         // Does fullname end with this extension
         if (flen >= extlen &&
-            !PL_strcasecmp(&(leafName[flen - extlen]), ValidDllExtensions[i])
+            !PL_strcasecmp(leafName.get() + (flen - extlen), ValidDllExtensions[i])
             )
         {
             validExtension = PR_TRUE;
             break;
         }
     }
-    if (leafName) nsMemory::Free(leafName);
 #endif
         
     if (validExtension == PR_FALSE)
@@ -811,8 +746,7 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
 
     // Get the registry representation of the dll, if any
     nsDll *dll;
-    PRInt64 mod = LL_Zero(), size = LL_Zero();
-    rv = CreateDll(component, persistentDescriptor, &mod, &size, &dll);
+    rv = CreateDll(component, persistentDescriptor, &dll);
     if (NS_FAILED(rv))
         return rv;
 
@@ -858,9 +792,7 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
             nsCOMPtr<nsIFile> dllSpec;
             if (NS_SUCCEEDED(dll->GetDllSpec(getter_AddRefs(dllSpec))) && dllSpec)
             {
-              nsXPIDLCString dllLeafName;
-              dllSpec->GetLeafName(getter_Copies(dllLeafName));
-              fileName.AssignWithConversion(dllLeafName);
+              dllSpec->GetLeafName(fileName);
             }
             
             // this string can't come from a string bundle, because we
@@ -980,13 +912,14 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
             mDeferredComponents.RemoveElementAt(i);
         }
     }
-
+#ifdef DEBUG
     if (*aRegistered)
         fprintf(stderr, "nNCL: registered deferred, %d left\n",
                 mDeferredComponents.Count());
     else
         fprintf(stderr, "nNCL: didn't register any components, %d left\n",
                 mDeferredComponents.Count());
+#endif
     /* are there any fatal errors? */
     return NS_OK;
 }
@@ -1016,98 +949,6 @@ nsNativeComponentLoader::UnloadAll(PRInt32 aWhen)
     return NS_OK;
 }
 
-nsresult
-nsNativeComponentLoader::GetRegistryDllInfo(const char *aLocation,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-    return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-
-    nsRegistryKey key;
-    rv = mRegistry->GetSubtreeRaw(mXPCOMKey, eLocation, &key);
-    if (NS_FAILED(rv)) return rv;
-
-    rv = GetRegistryDllInfo(key, lastModifiedTime, fileSize);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
-nsresult
-nsNativeComponentLoader::GetRegistryDllInfo(nsRegistryKey key,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    PRInt64 lastMod;
-    nsresult rv = mRegistry->GetLongLong(key, lastModValueName, &lastMod);
-    if (NS_FAILED(rv)) return rv;
-    *lastModifiedTime = lastMod;
-        
-    PRInt64 fsize;
-    rv = mRegistry->GetLongLong(key, fileSizeValueName, &fsize);
-    if (NS_FAILED(rv)) return rv;
-        *fileSize = fsize;
-    return NS_OK;
-}
-
-nsresult
-nsNativeComponentLoader::SetRegistryDllInfo(const char *aLocation,
-                                            PRInt64 *lastModifiedTime,
-                                            PRInt64 *fileSize)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-    nsRegistryKey key;
-    rv = mRegistry->GetSubtreeRaw(mXPCOMKey, eLocation, &key);
-    if (NS_FAILED(rv)) return rv;
-
-    rv = mRegistry->SetLongLong(key, lastModValueName, lastModifiedTime);
-    if (NS_FAILED(rv)) return rv;
-    rv = mRegistry->SetLongLong(key, fileSizeValueName, fileSize);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
-nsresult
-nsNativeComponentLoader::RemoveRegistryDllInfo(const char *aLocation)
-{
-    nsresult rv;
-    PRUint32 length = strlen(aLocation);
-    char* eLocation;
-    rv = mRegistry->EscapeKey((PRUint8*)aLocation, 1, &length, (PRUint8**)&eLocation);
-    if (rv != NS_OK)
-    {
-        return rv;
-    }
-    if (eLocation == nsnull)    //  No escaping required
-    eLocation = (char*)aLocation;
-
-    rv = mRegistry->RemoveSubtree(mXPCOMKey, eLocation);
-    if (aLocation != eLocation)
-        nsMemory::Free(eLocation);
-    return rv;
-}
-
 //
 // CreateDll
 // The only way to create a dll or get it from the dll cache. This will
@@ -1127,8 +968,8 @@ nsNativeComponentLoader::RemoveRegistryDllInfo(const char *aLocation)
 //    registry along with its    lastModTime and fileSize.
 //    {NULL, rel:libpref.so, 8985659, 20987}
 nsresult
-nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
-                                   PRInt64 *modificationTime, PRInt64 *fileSize,
+nsNativeComponentLoader::CreateDll(nsIFile *aSpec, 
+                                   const char *aLocation,
                                    nsDll **aDll)
 {
     nsDll *dll;
@@ -1167,22 +1008,11 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec, const char *aLocation,
 
     if (!dll)
     {
-        PR_ASSERT(modificationTime != NULL);
-        PR_ASSERT(fileSize != NULL);
-
-        PRInt64 zit = LL_Zero();
-        
-        if (LL_EQ(*modificationTime,zit) && LL_EQ(*fileSize,zit))
-        {
-            // Get the modtime and filesize from the registry
-            rv = GetRegistryDllInfo(aLocation, modificationTime, fileSize);
-        }
-        dll = new nsDll(spec, aLocation, modificationTime, fileSize);
+        dll = new nsDll(spec, aLocation);
+        if (!dll)
+            return NS_ERROR_OUT_OF_MEMORY;
     }
 
-    if (!dll)
-        return NS_ERROR_OUT_OF_MEMORY;
-
     *aDll = dll;
     mDllStore->Put(&key, dll);
     return NS_OK;
@@ -1210,27 +1040,5 @@ nsNativeComponentLoader::GetFactoryFromNSGetFactory(nsDll *aDll,
                                                     nsIServiceManager *aServMgr,
                                                     nsIFactory **aFactory)
 {
-#ifdef XPCOM_USE_NSGETFACTORY
-    nsFactoryProc getFactory =
-        (nsFactoryProc) aDll->FindSymbol("NSGetFactory");
-
-    if (!getFactory)
-        return NS_ERROR_FACTORY_NOT_LOADED;
-    
-    PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-           ("nsNativeComponentLoader: %s using OBSOLETE NSGetFactory()\n",
-            aDll->GetDisplayPath()));
-
-    /*
-     * There was a time when CLSIDToContractID was used to get className
-     * and contractID, but that day is long past.  This code is not long
-     * for this earth, so we just pass nsnull.
-     */
-
-    return getFactory(aServMgr, aCID, nsnull /*className */,
-                      nsnull /* contractID */, aFactory);
-
-#else /* !XPCOM_USE_NSGETFACTORY */
     return NS_ERROR_FACTORY_NOT_LOADED;
-#endif /* XPCOM_USE_NSGETFACTORY */
 }
