@@ -29,6 +29,8 @@
 
 #include "nsLocalFileOS2.h"
 
+#include <uconv.h>
+
 #include "nsISimpleEnumerator.h"
 #include "nsIComponentManager.h"
 #include "prtypes.h"
@@ -40,6 +42,7 @@
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
 #include "prproces.h"
+#include "prthread.h"
 
 
 static unsigned char* PR_CALLBACK
@@ -154,24 +157,22 @@ class nsDirEnumerator : public nsISimpleEnumerator
 
         nsresult Init(nsILocalFile* parent) 
         {
-            char* filepath;
-            parent->GetTarget(&filepath);
+            nsCAutoString filepath;
+            parent->GetNativeTarget(filepath);
         
-            if (filepath == nsnull)
+            if (filepath.IsEmpty())
             {
-                parent->GetPath(&filepath);
+                parent->GetNativePath(filepath);
             }
             
-            if (filepath == nsnull)
+            if (filepath.IsEmpty())
             {
-                return NS_ERROR_OUT_OF_MEMORY;
+                return NS_ERROR_UNEXPECTED;
             }
 
-            mDir = PR_OpenDir(filepath);
+            mDir = PR_OpenDir(filepath.get());
             if (mDir == nsnull)    // not a directory?
                 return NS_ERROR_FAILURE;
-			
-			nsMemory::Free(filepath);
 
             mParent          = parent;    
             return NS_OK;
@@ -201,13 +202,13 @@ class nsDirEnumerator : public nsISimpleEnumerator
                 if (NS_FAILED(rv)) 
                     return rv;
                 
-                rv = file->Append(entry->name);
+                rv = file->AppendNative(nsDependentCString(entry->name));
                 if (NS_FAILED(rv)) 
                     return rv;
                 
                 // make sure the thing exists.  If it does, try the next one.
                 PRBool exists;
-                    rv = file->Exists(&exists);
+                rv = file->Exists(&exists);
                 if (NS_FAILED(rv) || !exists) 
                 {
                     return HasMoreElements(result); 
@@ -622,13 +623,10 @@ NS_IMETHODIMP
 nsLocalFile::Clone(nsIFile **file)
 {
     nsresult rv;
-    char * aFilePath;
-    GetPath(&aFilePath);
 
     nsCOMPtr<nsILocalFile> localFile;
 
-    rv = NS_NewLocalFile(aFilePath, mFollowSymlinks, getter_AddRefs(localFile));
-    nsMemory::Free(aFilePath);
+    rv = NS_NewNativeLocalFile(mWorkingPath, mFollowSymlinks, getter_AddRefs(localFile));
     
     if (NS_SUCCEEDED(rv) && localFile)
     {
@@ -639,55 +637,47 @@ nsLocalFile::Clone(nsIFile **file)
 }
 
 NS_IMETHODIMP  
-nsLocalFile::InitWithPath(const char *filePath)
+nsLocalFile::InitWithNativePath(const nsACString &filePath)
 {
     MakeDirty();
-    NS_ENSURE_ARG(filePath);
-    
-    char* nativeFilePath = nsnull;
+
+    nsACString::const_iterator begin, end;
+    filePath.BeginReading(begin);
+    filePath.EndReading(end);
+
+    // input string must not be empty
+    if (begin == end)
+        return NS_ERROR_FAILURE;
+
+    char firstChar = *begin;
+    char secondChar = *(++begin);
     
     // just do a sanity check.  if it has any forward slashes, it is not a Native path
     // on windows.  Also, it must have a colon at after the first char.
-    if (filePath[0] == 0)
-        return NS_ERROR_FAILURE;
     
-    if ( (filePath[2] == 0) && (filePath[1] == ':') )
-    {
-        // C : //
-        nativeFilePath = (char*) nsMemory::Alloc( 4 );
-        if (!nativeFilePath)
-            return NS_ERROR_OUT_OF_MEMORY;
-        nativeFilePath[0] = filePath[0];
-        nativeFilePath[1] = ':';
-        nativeFilePath[2] = '\\';
-        nativeFilePath[3] = 0;
-    }
-    // XXX is this an 'else'? Otherwise 'nativeFilePath' could leak.
-    if ( ( (filePath[1] == ':')  && (strchr(filePath, '/') == 0) ) ||  // normal windows path
-         ( (filePath[0] == '\\') && (filePath[1] == '\\') ) )  // netwerk path
+    char *path = nsnull;
+    PRInt32 pathLen = 0;
+
+    if ( ( (secondChar == ':') && !FindCharInReadable('/', begin, end) ) ||  // normal path
+         ( (firstChar == '\\') && (secondChar == '\\') ) )  // network path
     {
         // This is a native path
-        if (nativeFilePath) nsCRT::free(nativeFilePath);
-        nativeFilePath = (char*) nsMemory::Clone( filePath, strlen(filePath)+1 );
+        path = ToNewCString(filePath);
+        pathLen = filePath.Length();
     }
-    
-    if (nativeFilePath == nsnull)
+
+    if (path == nsnull)
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
 
-    // kill any trailing separator
-    char* temp = nativeFilePath;
-    int len = strlen(temp) - 1;
-    // Is '\' second charactor of DBCS?
-#ifdef XP_OS2
-    if(temp[len] == '\\' && !::isleadbyte(temp[len-1]))
-#else
-    if(temp[len] == '\\' && !::IsDBCSLeadByte(temp[len-1]))
-#endif
-        temp[len] = '\0';
+    // kill any trailing '\' provided it isn't the second char of DBCS
+    PRInt32 len = pathLen - 1;
+    if (path[len] == '\\' && !::isleadbyte(path[len-1]))
+    {
+        path[len] = '\0';
+        pathLen = len;
+    }
     
-    mWorkingPath.Assign(nativeFilePath);
-    nsMemory::Free( nativeFilePath );
-
+    mWorkingPath.Adopt(path, pathLen);
     return NS_OK;
 }
 
@@ -736,30 +726,34 @@ nsLocalFile::Create(PRUint32 type, PRUint32 attributes)
     
    // create nested directories to target
     unsigned char* slash = _mbschr((const unsigned char*) mResolvedPath.get(), '\\');
-    // skip the first '\\'
-    ++slash;
-    slash = _mbschr(slash, '\\');
 
-    while (slash)
+
+    if (slash)
     {
-        *slash = '\0';
-
-#ifdef XP_OS2
-        rv = CreateDirectoryA(NS_CONST_CAST(char*, mResolvedPath.get()), NULL);
-        if (rv) {
-            rv = ConvertOS2Error(rv);
-#else
-        if (!CreateDirectoryA(NS_CONST_CAST(char*, mResolvedPath.get()), NULL)) {
-            rv = ConvertWinError(GetLastError());
-#endif
-            if (rv != NS_ERROR_FILE_ALREADY_EXISTS) return rv;
-        }
-        *slash = '\\';
+        // skip the first '\\'
         ++slash;
         slash = _mbschr(slash, '\\');
+    
+        while (slash)
+        {
+            *slash = '\0';
+            
+#ifdef XP_OS2
+                rv = CreateDirectoryA(NS_CONST_CAST(char*, mResolvedPath.get()), NULL);
+                if (rv) {
+                    rv = ConvertOS2Error(rv);
+#else
+                if (!CreateDirectoryA(NS_CONST_CAST(char*, mResolvedPath.get()), NULL)) {
+                    rv = ConvertWinError(GetLastError());
+#endif
+                    if (rv != NS_ERROR_FILE_ALREADY_EXISTS) return rv;
+                }
+                *slash = '\\';
+                ++slash;
+                slash = _mbschr(slash, '\\');
+        }
     }
 
-
     if (type == NORMAL_FILE_TYPE)
     {
         PRFileDesc* file = PR_Open(mResolvedPath.get(), PR_RDONLY | PR_CREATE_FILE | PR_APPEND | PR_EXCL, attributes);
@@ -787,27 +781,30 @@ nsLocalFile::Create(PRUint32 type, PRUint32 attributes)
 }
     
 NS_IMETHODIMP  
-nsLocalFile::Append(const char *node)
+nsLocalFile::AppendNative(const nsACString &node)
 {
     // Append only one component. Check for subdirs.
-    if (!node || (_mbschr((const unsigned char*) node, '\\') != nsnull))
+    // XXX can we avoid the PromiseFlatCString call?
+    if (node.IsEmpty() || (_mbschr((const unsigned char*) PromiseFlatCString(node).get(), '\\') != nsnull))
     {
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
     }
-    return AppendRelativePath(node);
+    return AppendRelativeNativePath(node);
 }
 
 NS_IMETHODIMP  
-nsLocalFile::AppendRelativePath(const char *node)
+nsLocalFile::AppendRelativeNativePath(const nsACString &node)
 {
     // Cannot start with a / or have .. or have / anywhere
-    if (!node || strchr(node, '/'))
+    nsACString::const_iterator begin, end;
+    node.BeginReading(begin);
+    node.EndReading(end);
+    if (node.IsEmpty() || FindCharInReadable('/', begin, end))
     {
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
     }
     MakeDirty();
-    mWorkingPath.Append("\\");
-    mWorkingPath.Append(node);
+    mWorkingPath.Append(NS_LITERAL_CSTRING("\\") + node);
     return NS_OK;
 }
 
@@ -818,9 +815,9 @@ nsLocalFile::Normalize()
 }
 
 NS_IMETHODIMP  
-nsLocalFile::GetLeafName(char * *aLeafName)
+nsLocalFile::GetNativeLeafName(nsACString &aLeafName)
 {
-    NS_ENSURE_ARG_POINTER(aLeafName);
+    aLeafName.Truncate();
 
     const char* temp = mWorkingPath.get();
     if(temp == nsnull)
@@ -834,12 +831,12 @@ nsLocalFile::GetLeafName(char * *aLeafName)
     else
         leaf++;
 
-    *aLeafName = (char*) nsMemory::Clone(leaf, strlen(leaf)+1);
+    aLeafName.Assign(leaf);
     return NS_OK;
 }
 
 NS_IMETHODIMP  
-nsLocalFile::SetLeafName(const char * aLeafName)
+nsLocalFile::SetNativeLeafName(const nsACString &aLeafName)
 {
     MakeDirty();
     
@@ -860,36 +857,32 @@ nsLocalFile::SetLeafName(const char * aLeafName)
 
 
 NS_IMETHODIMP  
-nsLocalFile::GetPath(char **_retval)
+nsLocalFile::GetNativePath(nsACString &_retval)
 {
-    NS_ENSURE_ARG_POINTER(_retval);
-    *_retval = (char*) nsMemory::Clone(mWorkingPath.get(), mWorkingPath.Length()+1);
+    _retval = mWorkingPath;
     return NS_OK;
 }
 
 nsresult
-nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char * newName, PRBool followSymlinks, PRBool move)
+nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const nsACString &newName, PRBool followSymlinks, PRBool move)
 {
     nsresult rv;
-    char* filePath;
+    nsCAutoString filePath;
 
     // get the path that we are going to copy to.
     // Since windows does not know how to auto
     // resolve shortcust, we must work with the
     // target.
-    char* inFilePath;
-    destParent->GetTarget(&inFilePath);  
-    nsCString destPath(inFilePath);
-    nsMemory::Free(inFilePath);
+    nsCAutoString destPath;
+    destParent->GetNativeTarget(destPath);  
 
     destPath.Append("\\");
 
-    if (newName == nsnull)
+    if (newName.IsEmpty())
     {
-        char *aFileName;
-        sourceFile->GetLeafName(&aFileName);
+        nsCAutoString aFileName;
+        sourceFile->GetNativeLeafName(aFileName);
         destPath.Append(aFileName);
-        nsMemory::Free(aFileName);
     }
     else
     {
@@ -899,13 +892,13 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char
            
     if (followSymlinks)
     {
-        rv = sourceFile->GetTarget(&filePath);
-        if (!filePath)
-            rv = sourceFile->GetPath(&filePath);
+        rv = sourceFile->GetNativeTarget(filePath);
+        if (filePath.IsEmpty())
+            rv = sourceFile->GetNativePath(filePath);
     }
     else
     {
-        rv = sourceFile->GetPath(&filePath);
+        rv = sourceFile->GetNativePath(filePath);
     }
 
     if (NS_FAILED(rv))
@@ -915,7 +908,7 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char
 
     if( move )
     {
-        rc = DosMove(filePath, (PSZ)NS_CONST_CAST(char*, destPath.get()));
+        rc = DosMove(filePath.get(), (PSZ)NS_CONST_CAST(char*, destPath.get()));
     }
 
     if (!move || rc == ERROR_NOT_SAME_DEVICE) {
@@ -923,7 +916,19 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char
          * same drive.  "MoveFile()" on Windows will go ahead and move the
          * file without error, so we need to do the same   IBM-AKR
          */
-        rc = DosCopy(filePath, (PSZ)NS_CONST_CAST(char*, destPath.get()), DCPY_EXISTING);
+        do {
+          rc = DosCopy(filePath.get(), (PSZ)NS_CONST_CAST(char*, destPath.get()), DCPY_EXISTING);
+          if (rc == ERROR_TOO_MANY_OPEN_FILES) {
+              ULONG CurMaxFH = 0;
+              LONG ReqCount = 20;
+              APIRET rc2;
+              rc2 = DosSetRelMaxFH(&ReqCount, &CurMaxFH);
+              if (rc2 != NO_ERROR) {
+                break;
+              }
+          }
+        } while (rc == ERROR_TOO_MANY_OPEN_FILES);
+
         /* WSOD2 HACK */
         if (rc == 65) { // NETWORK_ACCESS_DENIED
           CHAR         achProgram[CCHMAXPATH];  // buffer for program name, parameters
@@ -931,7 +936,7 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char
 
           strcpy(achProgram, "CMD.EXE  /C ");
           strcat(achProgram, """COPY ");
-          strcat(achProgram, filePath);
+          strcat(achProgram, filePath.get());
           strcat(achProgram, " ");
           strcat(achProgram, (PSZ)NS_CONST_CAST(char*, destPath.get()));
           strcat(achProgram, """");
@@ -948,21 +953,19 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent, const char
          */
         if( move && (rc == NO_ERROR) )
         {
-          DosDelete( filePath );
+          DosDelete( filePath.get() );
         }
     } /* !move or ERROR */
     
     if (rc)
         rv = ConvertOS2Error(rc);
-    
-    nsMemory::Free(filePath);
 
     return rv;
 }
 
 
 nsresult
-nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSymlinks, PRBool move)
+nsLocalFile::CopyMove(nsIFile *aParentDir, const nsACString &newName, PRBool followSymlinks, PRBool move)
 {
     nsCOMPtr<nsIFile> newParentDir = aParentDir;
     // check to see if this exists, otherwise return an error.
@@ -977,7 +980,7 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSym
     {
         // no parent was specified.  We must rename.
         
-        if (!newName)
+        if (newName.IsEmpty())
             return NS_ERROR_INVALID_ARG;
 
         move = PR_TRUE;
@@ -1011,16 +1014,14 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSym
                 newParentDir->IsSymlink(&isLink);
                 if (isLink)
                 {
-                    char* target;
-                    newParentDir->GetTarget(&target);
+                    nsCAutoString target;
+                    newParentDir->GetNativeTarget(target);
 
                     nsCOMPtr<nsILocalFile> realDest = new nsLocalFile();
                     if (realDest == nsnull)
                         return NS_ERROR_OUT_OF_MEMORY;
 
-                    rv = realDest->InitWithPath(target);
-                    
-                    nsMemory::Free(target);
+                    rv = realDest->InitWithNativePath(target);
 
                     if (NS_FAILED(rv)) 
                         return rv;
@@ -1057,35 +1058,35 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSym
         if (NS_FAILED(rv)) 
             return rv;
         
-        char *allocatedNewName;
-        if (!newName)
+        nsCAutoString allocatedNewName;
+        if (newName.IsEmpty())
         {
             PRBool isLink;
             IsSymlink(&isLink);
             if (isLink)
             {
-                char* temp;
-                GetTarget(&temp);
-                const char* leaf = (const char*) _mbsrchr((const unsigned char*) temp, '\\');
+                nsCAutoString temp;
+                GetNativeTarget(temp);
+                const char* leaf = (const char*) _mbsrchr((const unsigned char*) temp.get(), '\\');
                 if (leaf[0] == '\\')
                     leaf++;
-                allocatedNewName = (char*) nsMemory::Clone( leaf, strlen(leaf)+1 );
+                allocatedNewName = leaf;
             }
             else
             {
-                GetLeafName(&allocatedNewName);// this should be the leaf name of the 
+                GetNativeLeafName(allocatedNewName);// this should be the leaf name of the 
             }
         }
         else
         {
-            allocatedNewName = (char*) nsMemory::Clone( newName, strlen(newName)+1 );
+            allocatedNewName = newName;
         }
         
-        rv = target->Append(allocatedNewName);
+        rv = target->AppendNative(allocatedNewName);
         if (NS_FAILED(rv)) 
             return rv;
 
-        nsMemory::Free(allocatedNewName);
+        allocatedNewName.Truncate();
 
         target->Create(DIRECTORY_TYPE, 0644);  // TODO, what permissions should we use
         if (NS_FAILED(rv))
@@ -1117,14 +1118,14 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSym
                 if (followSymlinks)
                     rv = NS_ERROR_FAILURE;
                 else
-                    rv = file->MoveTo(target, nsnull);
+                    rv = file->MoveToNative(target, nsCString());
             }
             else
             {   
                 if (followSymlinks)
-                    rv = file->CopyToFollowingLinks(target, nsnull);
+                    rv = file->CopyToFollowingLinksNative(target, nsCString());
                 else
-                    rv = file->CopyTo(target, nsnull);
+                    rv = file->CopyToNative(target, nsCString());
             }
                     
             iterator->HasMoreElements(&more);
@@ -1137,48 +1138,44 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const char *newName, PRBool followSym
     {
         MakeDirty();
         
-        char* newParentPath;
-        newParentDir->GetPath(&newParentPath);
+        nsCAutoString newParentPath;
+        newParentDir->GetNativePath(newParentPath);
         
-        if (newParentPath == nsnull)
+        if (newParentPath.IsEmpty())
             return NS_ERROR_FAILURE;
 
-        if (newName == nsnull)
+        if (newName.IsEmpty())
         {
-            char *aFileName;
-            GetLeafName(&aFileName);
+            nsCAutoString aFileName;
+            GetNativeLeafName(aFileName);
             
-            InitWithPath(newParentPath);
-            Append(aFileName); 
-
-            nsMemory::Free(aFileName);
+            InitWithNativePath(newParentPath);
+            AppendNative(aFileName); 
         }
         else
         {
-            InitWithPath(newParentPath);
-            Append(newName);
+            InitWithNativePath(newParentPath);
+            AppendNative(newName);
         }
-        
-        nsMemory::Free(newParentPath);
     }
         
     return NS_OK;
 }
 
 NS_IMETHODIMP  
-nsLocalFile::CopyTo(nsIFile *newParentDir, const char *newName)
+nsLocalFile::CopyToNative(nsIFile *newParentDir, const nsACString &newName)
 {
     return CopyMove(newParentDir, newName, PR_FALSE, PR_FALSE);
 }
 
 NS_IMETHODIMP  
-nsLocalFile::CopyToFollowingLinks(nsIFile *newParentDir, const char *newName)
+nsLocalFile::CopyToFollowingLinksNative(nsIFile *newParentDir, const nsACString &newName)
 {
     return CopyMove(newParentDir, newName, PR_TRUE, PR_FALSE);
 }
 
 NS_IMETHODIMP  
-nsLocalFile::MoveTo(nsIFile *newParentDir, const char *newName)
+nsLocalFile::MoveToNative(nsIFile *newParentDir, const nsACString &newName)
 {
     return CopyMove(newParentDir, newName, PR_FALSE, PR_TRUE);
 }
@@ -1241,18 +1238,18 @@ nsLocalFile::Remove(PRBool recursive)
             }
         }
 #ifdef XP_OS2_VACPP
-        rmdir((char *) filePath);  // todo: save return value?
+        rv = rmdir((char *) filePath);  // todo: save return value?
 #else
-        rmdir(filePath);  // todo: save return value?
+        rv = rmdir(filePath);  // todo: save return value?
 #endif
     }
     else
     {
-        remove(filePath); // todo: save return value?
+        rv = remove(filePath); // todo: save return value?
     }
     
     MakeDirty();
-    return NS_OK;
+    return rv;
 }
 
 NS_IMETHODIMP  
@@ -1504,7 +1501,7 @@ nsLocalFile::SetFileSize(PRInt64 aFileSize)
                        OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS,
                        OPEN_SHARE_DENYREADWRITE | OPEN_ACCESS_READWRITE,
                        NULL);
-                                 
+
     if (rc != NO_ERROR)
     {
         MakeDirty();
@@ -1587,70 +1584,21 @@ nsLocalFile::GetDiskSpaceAvailable(PRInt64 *aDiskSpaceAvailable)
     
     ResolveAndStat(PR_FALSE);
     
-#ifdef XP_OS2
     ULONG ulDriveNo = toupper(mResolvedPath.CharAt(0)) + 1 - 'A';
     FSALLOCATE fsAllocate;
-    APIRET rv = DosQueryFSInfo(ulDriveNo,
-                                            FSIL_ALLOC,
-                                            &fsAllocate,
-                                            sizeof(fsAllocate));
+    APIRET rc = DosQueryFSInfo(ulDriveNo,
+                               FSIL_ALLOC,
+                               &fsAllocate,
+                               sizeof(fsAllocate));
 
-    if (NS_FAILED(rv)) 
-        return rv;
+    if (rc != NO_ERROR)
+       return NS_ERROR_FAILURE;
 
-    *aDiskSpaceAvailable = (PRInt64)(fsAllocate.cUnitAvail  *
-                                                    fsAllocate.cSectorUnit *
-                                                    (ULONG)fsAllocate.cbSector);
-#else   
-    const char *filePath = mResolvedPath.get();
-
-    PRInt64 int64;
-    
-    LL_I2L(int64 , LONG_MAX);
-
-    char aDrive[_MAX_DRIVE + 2];
-	_splitpath( (const char*)filePath, aDrive, NULL, NULL, NULL);
-	strcat(aDrive, "\\");
-
-    // Check disk space
-    DWORD dwSecPerClus, dwBytesPerSec, dwFreeClus, dwTotalClus;
-    ULARGE_INTEGER liFreeBytesAvailableToCaller, liTotalNumberOfBytes, liTotalNumberOfFreeBytes;
-    double nBytes = 0;
-
-    BOOL (WINAPI* getDiskFreeSpaceExA)(LPCTSTR lpDirectoryName, 
-                                       PULARGE_INTEGER lpFreeBytesAvailableToCaller,
-                                       PULARGE_INTEGER lpTotalNumberOfBytes,    
-                                       PULARGE_INTEGER lpTotalNumberOfFreeBytes) = NULL;
-
-    HINSTANCE hInst = LoadLibrary("KERNEL32.DLL");
-    NS_ASSERTION(hInst != NULL, "COULD NOT LOAD KERNEL32.DLL");
-    if (hInst != NULL)
-    {
-        getDiskFreeSpaceExA =  (BOOL (WINAPI*)(LPCTSTR lpDirectoryName, 
-                                               PULARGE_INTEGER lpFreeBytesAvailableToCaller,
-                                               PULARGE_INTEGER lpTotalNumberOfBytes,    
-                                               PULARGE_INTEGER lpTotalNumberOfFreeBytes)) 
-        GetProcAddress(hInst, "GetDiskFreeSpaceExA");
-        FreeLibrary(hInst);
-    }
-
-    if (getDiskFreeSpaceExA && (*getDiskFreeSpaceExA)(aDrive,
-                                                      &liFreeBytesAvailableToCaller, 
-                                                      &liTotalNumberOfBytes,  
-                                                      &liTotalNumberOfFreeBytes))
-    {
-        nBytes = (double)(signed __int64)liFreeBytesAvailableToCaller.QuadPart;
-    }
-    else if ( GetDiskFreeSpace(aDrive, &dwSecPerClus, &dwBytesPerSec, &dwFreeClus, &dwTotalClus))
-    {
-        nBytes = (double)dwFreeClus*(double)dwSecPerClus*(double) dwBytesPerSec;
-    }
-
-    LL_D2L(*aDiskSpaceAvailable, nBytes);
-#endif
+    *aDiskSpaceAvailable = fsAllocate.cUnitAvail;
+    *aDiskSpaceAvailable *= fsAllocate.cSectorUnit;
+    *aDiskSpaceAvailable *= fsAllocate.cbSector;
 
     return NS_OK;
-
 }
 
 NS_IMETHODIMP  
@@ -1658,7 +1606,7 @@ nsLocalFile::GetParent(nsIFile * *aParent)
 {
     NS_ENSURE_ARG_POINTER(aParent);
 
-    nsCString parentPath = mWorkingPath;
+    nsCAutoString parentPath(mWorkingPath);
 
     // cannot use nsCString::RFindChar() due to 0x5c problem
     PRInt32 offset = (PRInt32) (_mbsrchr((const unsigned char *) parentPath.get(), '\\')
@@ -1669,11 +1617,11 @@ nsLocalFile::GetParent(nsIFile * *aParent)
     parentPath.Truncate(offset);
 
     nsCOMPtr<nsILocalFile> localFile;
-    nsresult rv =  NS_NewLocalFile(parentPath.get(), mFollowSymlinks, getter_AddRefs(localFile));
+    nsresult rv =  NS_NewNativeLocalFile(parentPath, mFollowSymlinks, getter_AddRefs(localFile));
     
     if(NS_SUCCEEDED(rv) && localFile)
     {
-        return localFile->QueryInterface(NS_GET_IID(nsIFile), (void**)aParent);
+        return CallQueryInterface(localFile, aParent);
     }
     return rv;
 }
@@ -1758,7 +1706,7 @@ nsLocalFile::IsExecutable(PRBool *_retval)
     if (NS_FAILED(rv))
         return rv;
 
-    char* path = nsnull;
+    nsCAutoString path;
     PRBool symLink;
     
     rv = IsSymlink(&symLink);
@@ -1766,11 +1714,11 @@ nsLocalFile::IsExecutable(PRBool *_retval)
         return rv;
     
     if (symLink)
-        GetTarget(&path);
+        GetNativeTarget(path);
     else
-        GetPath(&path);
+        GetNativePath(path);
 
-    const char* leaf = (const char*) _mbsrchr((const unsigned char*) path, '\\');
+    const char* leaf = (const char*) _mbsrchr((const unsigned char*) path.get(), '\\');
 
     // XXX On Windows NT / 2000, it should use "PATHEXT" environment value
 #ifdef XP_OS2
@@ -1787,8 +1735,6 @@ nsLocalFile::IsExecutable(PRBool *_retval)
         *_retval = PR_FALSE;
     }
     
-    nsMemory::Free(path);
-    
     return NS_OK;
 }
 
@@ -1871,20 +1817,19 @@ nsLocalFile::IsSymlink(PRBool *_retval)
     *_retval = PR_FALSE;
 
 #ifndef XP_OS2   // No Symlinks on OS/2
-    char* path;
+    nsCAutoString path;
     int   pathLen;
     
-    GetPath(&path);
-    pathLen = strlen(path);
+    GetNativePath(path);
+    pathLen = path.Length();
     
-    const char* leaf = path + pathLen - 4;
+    const char* leaf = path.get() + pathLen - 4;
     
     if ( (strcmp(leaf, ".lnk") == 0)) 
     {
         *_retval = PR_TRUE;
     }
     
-    nsMemory::Free(path);
 #endif
     
     return NS_OK;
@@ -1935,18 +1880,10 @@ nsLocalFile::Equals(nsIFile *inFile, PRBool *_retval)
     NS_ENSURE_ARG(_retval);
     *_retval = PR_FALSE;
 
-    char* inFilePath;
-    inFile->GetPath(&inFilePath);
+    nsCAutoString inFilePath;
+    inFile->GetNativePath(inFilePath);
     
-    char* filePath;
-    GetPath(&filePath);
-
-    if (strcmp(inFilePath, filePath) == 0)
-        *_retval = PR_TRUE;
-    
-    nsMemory::Free(inFilePath);
-    nsMemory::Free(filePath);
-
+    *_retval = inFilePath.Equals(mWorkingPath);
     return NS_OK;
 }
 
@@ -1955,17 +1892,17 @@ nsLocalFile::Contains(nsIFile *inFile, PRBool recur, PRBool *_retval)
 {
     *_retval = PR_FALSE;
        
-    char* myFilePath;
-    if ( NS_FAILED(GetTarget(&myFilePath)))
-        GetPath(&myFilePath);
+    nsCAutoString myFilePath;
+    if ( NS_FAILED(GetNativeTarget(myFilePath)))
+        GetNativePath(myFilePath);
     
-    PRInt32 myFilePathLen = strlen(myFilePath);
+    PRInt32 myFilePathLen = myFilePath.Length();
     
-    char* inFilePath;
-    if ( NS_FAILED(inFile->GetTarget(&inFilePath)))
-        inFile->GetPath(&inFilePath);
+    nsCAutoString inFilePath;
+    if ( NS_FAILED(inFile->GetNativeTarget(inFilePath)))
+        inFile->GetNativePath(inFilePath);
 
-    if ( strncmp( myFilePath, inFilePath, myFilePathLen) == 0)
+    if ( strncmp( myFilePath.get(), inFilePath.get(), myFilePathLen) == 0)
     {
         // now make sure that the |inFile|'s path has a trailing
         // separator.
@@ -1977,19 +1914,15 @@ nsLocalFile::Contains(nsIFile *inFile, PRBool recur, PRBool *_retval)
 
     }
         
-    nsMemory::Free(inFilePath);
-    nsMemory::Free(myFilePath);
-
     return NS_OK;
 }
 
 
 
 NS_IMETHODIMP
-nsLocalFile::GetTarget(char **_retval)
+nsLocalFile::GetNativeTarget(nsACString &_retval)
 {   
-    NS_ENSURE_ARG(_retval);
-    *_retval = nsnull;
+    _retval.Truncate();
 #if STRICT_FAKE_SYMLINKS    
     PRBool symLink;
     
@@ -2004,7 +1937,7 @@ nsLocalFile::GetTarget(char **_retval)
 #endif
     ResolveAndStat(PR_TRUE);
  
-    *_retval = (char*) nsMemory::Clone( mResolvedPath.get(), mResolvedPath.Length()+1 );
+    _retval = mResolvedPath;
     return NS_OK;
 }
 
@@ -2055,17 +1988,15 @@ nsLocalFile::GetDirectoryEntries(nsISimpleEnumerator * *entries)
 }
 
 NS_IMETHODIMP
-nsLocalFile::GetPersistentDescriptor(char * *aPersistentDescriptor)
+nsLocalFile::GetPersistentDescriptor(nsACString &aPersistentDescriptor)
 {
-    NS_ENSURE_ARG_POINTER(aPersistentDescriptor);
-    return GetPath(aPersistentDescriptor);
+    return GetNativePath(aPersistentDescriptor);
 }
 
 NS_IMETHODIMP
-nsLocalFile::SetPersistentDescriptor(const char * aPersistentDescriptor)
+nsLocalFile::SetPersistentDescriptor(const nsACString &aPersistentDescriptor)
 {
-   NS_ENSURE_ARG(aPersistentDescriptor); 
-   return InitWithPath(aPersistentDescriptor);   
+   return InitWithNativePath(aPersistentDescriptor);   
 }
 
 #ifndef OPEN_DEFAULT
@@ -2077,22 +2008,22 @@ NS_IMETHODIMP
 nsLocalFile::Reveal()
 {
   PRBool isDirectory = PR_FALSE;
-  nsXPIDLCString path;
+  nsCAutoString path;
 
   IsDirectory(&isDirectory);
   if (isDirectory)
   {
-    GetPath(getter_Copies(path));  
+    GetNativePath(path);  
   }
   else
   {
     nsCOMPtr<nsIFile> parent;
     GetParent(getter_AddRefs(parent));
     if (parent)
-      parent->GetPath(getter_Copies(path));  
+      parent->GetNativePath(path);  
   }
 
-  HOBJECT hobject = WinQueryObject(path);
+  HOBJECT hobject = WinQueryObject(path.get());
   WinOpenObject( hobject, OPEN_DEFAULT, TRUE);
 
   // we don't care if it succeeded or failed.
@@ -2103,11 +2034,7 @@ nsLocalFile::Reveal()
 NS_IMETHODIMP
 nsLocalFile::Launch()
 {
-  nsXPIDLCString platformPath;
-
-  GetPath(getter_Copies(platformPath));  
-
-  HOBJECT hobject = WinQueryObject(platformPath);
+  HOBJECT hobject = WinQueryObject(mWorkingPath.get());
   WinOpenObject( hobject, OPEN_DEFAULT, TRUE);
 
   // we don't care if it succeeded or failed.
@@ -2115,7 +2042,7 @@ nsLocalFile::Launch()
 }
 
 nsresult 
-NS_NewLocalFile(const char* path, PRBool followLinks, nsILocalFile* *result)
+NS_NewNativeLocalFile(const nsACString &path, PRBool followLinks, nsILocalFile* *result)
 {
     nsLocalFile* file = new nsLocalFile();
     if (file == nsnull)
@@ -2124,8 +2051,8 @@ NS_NewLocalFile(const char* path, PRBool followLinks, nsILocalFile* *result)
 
     file->SetFollowLinks(followLinks);
 
-    if (path) {
-        nsresult rv = file->InitWithPath(path);
+    if (!path.IsEmpty()) {
+        nsresult rv = file->InitWithNativePath(path);
         if (NS_FAILED(rv)) {
             NS_RELEASE(file);
             return rv;
@@ -2227,3 +2154,259 @@ static int isleadbyte(int c)
 
   return retval;
 }
+
+static UconvObject  UnicodeConverter = NULL;
+
+static nsresult NS_CreateUnicodeConverters()
+{
+    ULONG ulLength;
+    ULONG ulCodePage;
+    DosQueryCp(sizeof(ULONG), &ulCodePage, &ulLength);
+
+    UniChar codepage[20];
+    int unirc = ::UniMapCpToUcsCp(ulCodePage, codepage, 20);
+    if (unirc == ULS_SUCCESS) {
+        unirc = ::UniCreateUconvObject(codepage, &UnicodeConverter);
+        if (unirc == ULS_SUCCESS) {
+            uconv_attribute_t attr;
+            ::UniQueryUconvObject(UnicodeConverter, &attr, sizeof(uconv_attribute_t), 
+                                  NULL, NULL, NULL);
+            attr.options = UCONV_OPTION_SUBSTITUTE_BOTH;
+            attr.subchar_len=1;
+            attr.subchar[0]='_';
+            ::UniSetUconvObject(UnicodeConverter, &attr);
+        }
+    }
+}
+
+static void NS_DestroyUnicodeConverters()
+{
+    ::UniFreeUconvObject(UnicodeConverter);
+}
+
+static nsresult
+UCS2toFS(const PRUnichar *aBuffer, char **aResult)
+{
+    size_t   inLength   = UniStrlen(aBuffer)+1;
+    size_t   outLength  = CCHMAXPATH;
+    UniChar *inString   = (UniChar*)aBuffer;
+    char    *outString  = (char*)nsMemory::Alloc(outLength);
+    size_t   cSubs      = 0;
+
+    char    *tempString = outString;
+  
+    int unirc = ::UniUconvFromUcs(UnicodeConverter, &inString, &inLength,
+                                  (void**)&tempString, &outLength, &cSubs);
+
+    NS_ASSERTION(unirc != UCONV_E2BIG, "Path too big");
+  
+    if( unirc != ULS_SUCCESS )
+      return NS_ERROR_FAILURE;
+
+    *aResult = outString;
+  
+    return NS_OK;
+}
+
+static nsresult
+FStoUCS2(const char* aBuffer, PRUnichar **aResult)
+{
+    size_t   inLength   = strlen(aBuffer)+1;
+    size_t   outLength  = CCHMAXPATH;
+    char    *inString   = (char*)aBuffer;
+    UniChar *outString  = (UniChar*)nsMemory::Alloc(outLength);
+                       
+    size_t   cSubs      = 0;
+
+    UniChar *tempString = outString;
+  
+    int unirc = ::UniUconvToUcs(UnicodeConverter, (void**)&inString, &inLength,
+                                &tempString, &outLength, &cSubs);
+                                 
+    NS_ASSERTION(unirc != UCONV_E2BIG, "Path too big");
+
+    if( unirc != ULS_SUCCESS )
+      return NS_ERROR_FAILURE;
+  
+    *aResult = (PRUnichar*)outString;
+  
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsLocalFile::InitWithPath(const nsAString &filePath)
+{
+    if (filePath.IsEmpty())
+        return InitWithNativePath(nsCString());
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(filePath).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return InitWithNativePath(tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::Append(const nsAString &node)
+{
+    if (node.IsEmpty())
+        return NS_OK;
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(node).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return AppendNative(tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::AppendRelativePath(const nsAString &node)
+{
+    if (node.IsEmpty())
+        return NS_OK;
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(node).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return AppendRelativeNativePath(tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::GetLeafName(nsAString &aLeafName)
+{
+    nsCAutoString tmp;
+    nsresult rv = GetNativeLeafName(tmp);
+
+    if (NS_SUCCEEDED(rv)) {
+        nsXPIDLString ucsBuf;
+        rv = FStoUCS2(tmp.get(), getter_Copies(ucsBuf));
+        if (NS_SUCCEEDED(rv))
+            aLeafName = ucsBuf;
+    }
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::SetLeafName(const nsAString &aLeafName)
+{
+    if (aLeafName.IsEmpty())
+        return SetNativeLeafName(nsCString());
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(aLeafName).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return SetNativeLeafName(tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::GetPath(nsAString &_retval)
+{
+    nsXPIDLString ucsBuf;
+    nsresult rv = FStoUCS2(mWorkingPath.get(), getter_Copies(ucsBuf));
+    if (NS_SUCCEEDED(rv))
+        _retval = ucsBuf;
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::CopyTo(nsIFile *newParentDir, const nsAString &newName)
+{
+    if (newName.IsEmpty())
+        return CopyToNative(newParentDir, nsCString());
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return CopyToNative(newParentDir, tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::CopyToFollowingLinks(nsIFile *newParentDir, const nsAString &newName)
+{
+    if (newName.IsEmpty())
+        return CopyToFollowingLinksNative(newParentDir, nsCString());
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return CopyToFollowingLinksNative(newParentDir, tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::MoveTo(nsIFile *newParentDir, const nsAString &newName)
+{
+    if (newName.IsEmpty())
+        return MoveToNative(newParentDir, nsCString());
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(newName).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return MoveToNative(newParentDir, tmp);
+
+    return rv;
+}
+
+NS_IMETHODIMP
+nsLocalFile::GetTarget(nsAString &_retval)
+{
+    nsCAutoString tmp;
+    nsresult rv = GetNativeTarget(tmp);
+
+    if (NS_SUCCEEDED(rv)) {
+        nsXPIDLString ucsBuf;
+        rv = FStoUCS2(tmp.get(), getter_Copies(ucsBuf));
+        if (NS_SUCCEEDED(rv))
+            _retval = ucsBuf;
+    }
+
+    return rv;
+}
+
+nsresult
+NS_NewLocalFile(const nsAString &path, PRBool followLinks, nsILocalFile* *result)
+{
+    if (path.IsEmpty())
+        return NS_NewNativeLocalFile(nsCString(), followLinks, result);
+
+    nsXPIDLCString tmp;
+    nsresult rv = UCS2toFS(PromiseFlatString(path).get(), getter_Copies(tmp));
+
+    if (NS_SUCCEEDED(rv))
+        return NS_NewNativeLocalFile(tmp, followLinks, result);
+
+    return NS_OK;
+}
+
+//----------------------------------------------------------------------------
+// global init/shutdown
+//----------------------------------------------------------------------------
+
+void
+nsLocalFile::GlobalInit()
+{
+    NS_CreateUnicodeConverters();
+}
+
+void
+nsLocalFile::GlobalShutdown()
+{
+    NS_DestroyUnicodeConverters();
+}
