@@ -77,8 +77,6 @@ class BasicStringImpl
                                             PRUint32 aCount,
                                             PRUint32 *aReadCount);
         NS_IMETHOD                      ReadSegments(nsWriteSegmentFun writer, void * closure, PRUint32 count, PRUint32 *_retval) = 0;
-        NS_IMETHOD                      GetObserver(nsIInputStreamObserver * *aObserver);
-        NS_IMETHOD                      SetObserver(nsIInputStreamObserver * aObserver); 
 
         // nsIOutputStream interface
         NS_IMETHOD                      Write(const char* aBuf,
@@ -86,10 +84,7 @@ class BasicStringImpl
                                             PRUint32 *aWriteCount);
         NS_IMETHOD                      WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_retval);
         NS_IMETHOD                      WriteSegments(nsReadSegmentFun reader, void * closure, PRUint32 count, PRUint32 *_retval);
-        NS_IMETHOD                      GetNonBlocking(PRBool *aNonBlocking);
-        NS_IMETHOD                      SetNonBlocking(PRBool aNonBlocking);
-        NS_IMETHOD                      GetObserver(nsIOutputStreamObserver * *aObserver);
-        NS_IMETHOD                      SetObserver(nsIOutputStreamObserver * aObserver);
+        NS_IMETHOD                      IsNonBlocking(PRBool *aNonBlocking);
 
     public:
         
@@ -162,7 +157,7 @@ NS_IMETHODIMP BasicStringImpl::Seek(PRInt32 whence, PRInt32 offset)
     }
     mOffset = newPosition;
     return NS_OK;
-} // StringImpl::Seek
+} // BasicStringImpl::Seek
 
 //----------------------------------------------------------------------------------------
 NS_IMETHODIMP BasicStringImpl::Tell(PRUint32* outWhere)
@@ -231,20 +226,6 @@ NS_IMETHODIMP BasicStringImpl::Read(char* aBuf, PRUint32 aCount, PRUint32 *aRead
   return NS_OK;
 }
 
-NS_IMETHODIMP
-BasicStringImpl::GetObserver(nsIInputStreamObserver * *aObserver)
-{
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-BasicStringImpl::SetObserver(nsIInputStreamObserver * aObserver)
-{
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //----------------------------------------------------------------------------------------
 NS_IMETHODIMP BasicStringImpl::Write(const char* aBuf, PRUint32 aCount, PRUint32 *aWriteCount)
 //----------------------------------------------------------------------------------------
@@ -280,31 +261,10 @@ BasicStringImpl::WriteSegments(nsReadSegmentFun reader, void * closure,
 }
 
 NS_IMETHODIMP
-BasicStringImpl::GetNonBlocking(PRBool *aNonBlocking)
+BasicStringImpl::IsNonBlocking(PRBool *aNonBlocking)
 {
-    NS_NOTREACHED("GetNonBlocking");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-BasicStringImpl::SetNonBlocking(PRBool aNonBlocking)
-{
-    NS_NOTREACHED("SetNonBlocking");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-BasicStringImpl::GetObserver(nsIOutputStreamObserver * *aObserver)
-{
-    NS_NOTREACHED("GetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP
-BasicStringImpl::SetObserver(nsIOutputStreamObserver * aObserver)
-{
-    NS_NOTREACHED("SetObserver");
-    return NS_ERROR_NOT_IMPLEMENTED;
+    *aNonBlocking = PR_TRUE;
+    return NS_OK;
 }
 
 //----------------------------------------------------------------------------------------
@@ -369,90 +329,19 @@ class ConstCharImpl
         
 }; // class ConstCharImpl
 
-//========================================================================================
-class CharImpl
-    : public ConstCharImpl
-//========================================================================================
-{
-    enum { kAllocQuantum = 256 };
-    
-    public:
-                                        CharImpl(char** inString, PRInt32 inLength = -1)
-                                            : ConstCharImpl(*inString, inLength)
-                                            , mString(*inString)
-                                            , mAllocLength(mLength + 1)
-                                            , mOriginalLength(mLength)
-                                        {
-                                            if (!mString)
-                                            {
-                                                mAllocLength = kAllocQuantum;
-                                                mString = new char[mAllocLength];
-                                                if (!mString)
-                                                {
-                                                    mLastResult = NS_ERROR_OUT_OF_MEMORY;
-                                                    return;
-                                                }
-                                                mConstString = mString;
-                                                *mString = '\0';
-                                                
-                                            }
-                                        }
-                                        
-                                        ~CharImpl()
-                                        {
-                                            if (mString) 
-                                            {
-                                                delete [] mString;
-                                            }
-                                        }
-
-        virtual PRInt32                 write(const char* buf, PRUint32 aCount)
-                                        {
-                                            if (!buf)
-                                                return 0;
-                                            PRInt32 maxCount = mAllocLength - 1 - mOffset;
-                                            if ((PRInt32)aCount > maxCount)
-                                            {
-                                                mAllocLength = aCount + 1 + mOffset + kAllocQuantum;
-                                                char* newString = new char[mAllocLength];
-                                                if (!newString)
-                                                {
-                                                    mLastResult = NS_ERROR_OUT_OF_MEMORY;
-                                                    return 0;
-                                                }
-                                                memcpy(newString, mString, mLength);
-                                                delete [] mString;
-                                                mString = newString;
-                                                mConstString = newString;
-                                            }
-                                            memcpy(mString + mOffset, buf, aCount);
-                                            mOffset += aCount;
-                                            mLength += aCount;
-                                            if (mOffset > mOriginalLength)
-                                                mString[mOffset] = 0;
-                                            return aCount;
-                                        }
-    protected:
-    
-        char*&                          mString;
-        size_t                          mAllocLength;
-        size_t                          mOriginalLength;
-        
-}; // class CharImpl
-                                       
 //========================================================================================
 class ConstStringImpl
     : public ConstCharImpl
 //========================================================================================
 {
     public:
-                                        ConstStringImpl(const nsString& inString)
+                                        ConstStringImpl(const nsAString& inString)
                                             : ConstCharImpl(ToNewCString(inString),
                                                             inString.Length())
                                         {
                                         }
 
-                                        ConstStringImpl(const nsCString& inString)
+                                        ConstStringImpl(const nsACString& inString)
                                             : ConstCharImpl(ToNewCString(inString),
                                                             inString.Length())
                                         {
@@ -470,49 +359,6 @@ class ConstStringImpl
 }; // class ConstStringImpl
 
 
-//========================================================================================
-class StringImpl
-    : public ConstStringImpl
-// This is wrong, since it really converts to 1-char strings.
-//========================================================================================
-{
-    public:
-                                        StringImpl(nsString& inString)
-                                            : ConstStringImpl(inString)
-                                            , mString(inString)
-                                        {
-                                        }
-
-    protected:
-    
-        virtual PRInt32                 write(const char* buf, PRUint32 count)
-                                        {
-                                            if (!buf)
-                                                return 0;
-                                            // Clone our string as chars
-                                            char* cstring = ToNewCString(mString);
-                                            // Make a CharImpl and do the write
-                                            CharImpl chars(&cstring, mString.Length());
-                                            chars.Seek(PR_SEEK_SET, mOffset);
-                                            // Get the bytecount and result from the CharImpl
-                                            PRInt32 result = chars.write(buf,count);
-                                            mLastResult = chars.get_result();
-                                            // Set our string to match the new chars
-                                            chars.Seek(PR_SEEK_SET, 0);
-                                            PRUint32 newLength;
-                                            chars.Available(&newLength);
-                                            mString.AssignWithConversion(cstring, newLength);
-                                            // Set our const string also...
-                                            delete [] (char*)mConstString;
-                                            mConstString = cstring;
-                                            return result;
-                                        }
-    protected:
-    
-        nsString&                       mString;
-        
-}; // class StringImpl
-
 NS_IMPL_THREADSAFE_ADDREF(BasicStringImpl)
 NS_IMPL_THREADSAFE_RELEASE(BasicStringImpl)
 
@@ -525,8 +371,8 @@ NS_IMPL_QUERY_TAIL(nsIOutputStream)
 
 //----------------------------------------------------------------------------------------
 extern "C" NS_COM nsresult NS_NewStringInputStream(
-    nsISupports** aStreamResult,
-    const nsString& aStringToRead)
+    nsIInputStream** aStreamResult,
+    const nsAString& aStringToRead)
     // Factory method to get an nsInputStream from a string.  Result will implement all the
     // file stream interfaces in nsIFileStream.h
 //----------------------------------------------------------------------------------------
@@ -540,14 +386,14 @@ extern "C" NS_COM nsresult NS_NewStringInputStream(
         return NS_ERROR_OUT_OF_MEMORY;
 
     NS_ADDREF(stream);
-    *aStreamResult = (nsISupports*)(void*)stream;
+    *aStreamResult = NS_STATIC_CAST(nsIInputStream*,stream);
     return NS_OK;
 }
 
 //----------------------------------------------------------------------------------------
 extern "C" NS_COM nsresult NS_NewCStringInputStream(
-    nsISupports** aStreamResult,
-    const nsCString& aStringToRead)
+    nsIInputStream** aStreamResult,
+    const nsACString& aStringToRead)
     // Factory method to get an nsInputStream from a cstring.  Result will implement all the
     // file stream interfaces in nsIFileStream.h
 //----------------------------------------------------------------------------------------
@@ -561,28 +407,7 @@ extern "C" NS_COM nsresult NS_NewCStringInputStream(
         return NS_ERROR_OUT_OF_MEMORY;
 
     NS_ADDREF(stream);
-    *aStreamResult = (nsISupports*)(void*)stream;
-    return NS_OK;
-}
-
-//----------------------------------------------------------------------------------------
-extern "C" NS_COM nsresult NS_NewStringOutputStream(
-    nsISupports** aStreamResult,
-    nsString& aStringToChange)
-    // Factory method to get an nsOutputStream from a string.  Result will implement all the
-    // file stream interfaces in nsIFileStream.h
-//----------------------------------------------------------------------------------------
-{
-    NS_PRECONDITION(aStreamResult != nsnull, "null ptr");
-    if (! aStreamResult)
-        return NS_ERROR_NULL_POINTER;
-
-    StringImpl* stream = new StringImpl(aStringToChange);
-    if (! stream)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    NS_ADDREF(stream);
-    *aStreamResult = (nsISupports*)(void*)stream;
+    *aStreamResult = NS_STATIC_CAST(nsIInputStream*,stream);
     return NS_OK;
 }
 
@@ -607,50 +432,6 @@ extern "C" NS_COM nsresult NS_NewCharInputStream(
     return NS_OK;
 }
 
-//----------------------------------------------------------------------------------------
-extern "C" NS_COM nsresult NS_NewCharOutputStream(
-    nsISupports** aStreamResult,
-    char** aStringToChange)
-    // Factory method to get an nsOutputStream to a string.  Result will implement all the
-    // file stream interfaces in nsIFileStream.h
-//----------------------------------------------------------------------------------------
-{
-    NS_PRECONDITION(aStreamResult != nsnull, "null ptr");
-    NS_PRECONDITION(aStringToChange != nsnull, "null ptr");
-    if (!aStreamResult || !aStringToChange)
-        return NS_ERROR_NULL_POINTER;
-
-    CharImpl* stream = new CharImpl(aStringToChange);
-    if (! stream)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    NS_ADDREF(stream);
-    *aStreamResult = (nsISupports*)(void*)stream;
-    return NS_OK;
-}
-
-//----------------------------------------------------------------------------------------
-extern "C" NS_COM nsresult NS_NewStringIOStream(
-    nsISupports** aStreamResult,
-    nsString& aStringToChange)
-    // Factory method to get an nsOutputStream to a string.  Result will implement all the
-// file stream interfaces in nsIFileStream.h
-//----------------------------------------------------------------------------------------
-{
-    return NS_NewStringOutputStream(aStreamResult, aStringToChange);
-}
-
-//----------------------------------------------------------------------------------------
-extern "C" NS_COM nsresult NS_NewCharIOStream(
-    nsISupports** aStreamResult,
-    char** aStringToChange)
-    // Factory method to get an nsOutputStream to a string.  Result will implement all the
-    // file stream interfaces in nsIFileStream.h
-//----------------------------------------------------------------------------------------
-{
-    return NS_NewCharOutputStream(aStreamResult, aStringToChange);
-}
-
 //----------------------------------------------------------------------------------------
 extern "C" NS_COM nsresult NS_NewByteInputStream(
     nsISupports** aStreamResult,
