@@ -40,10 +40,19 @@
 /* Implementation of xptiInterfaceInfoManager. */
 
 #include "xptiprivate.h"
+#include "nsDependentString.h"
+#include "nsString.h"
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(xptiInterfaceInfoManager, nsIInterfaceInfoManager)
+#define NS_ZIPLOADER_CONTRACTID NS_XPTLOADER_CONTRACTID_PREFIX "zip"
+
+NS_IMPL_THREADSAFE_ISUPPORTS2(xptiInterfaceInfoManager, 
+                              nsIInterfaceInfoManager,
+                              nsIInterfaceInfoSuperManager)
 
 static xptiInterfaceInfoManager* gInterfaceInfoManager = nsnull;
+#ifdef DEBUG
+static int gCallCount = 0;
+#endif
 
 // static
 xptiInterfaceInfoManager*
@@ -103,7 +112,8 @@ xptiInterfaceInfoManager::IsValid()
     return mWorkingSet.IsValid() &&
            mResolveLock &&
            mAutoRegLock &&
-           mInfoMonitor;
+           mInfoMonitor &&
+           mAdditionalManagersLock;
 }        
 
 xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath)
@@ -112,6 +122,7 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
         mResolveLock(PR_NewLock()),
         mAutoRegLock(PR_NewLock()),
         mInfoMonitor(nsAutoMonitor::NewMonitor("xptiInfoMonitor")),
+        mAdditionalManagersLock(PR_NewLock()),
         mSearchPath(aSearchPath)
 {
     NS_INIT_ISUPPORTS();
@@ -121,7 +132,7 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
     {
         mStatsLogFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);         
         if(mStatsLogFile && 
-           NS_SUCCEEDED(mStatsLogFile->InitWithPath(statsFilename)))
+           NS_SUCCEEDED(mStatsLogFile->InitWithNativePath(nsDependentCString(statsFilename))))
         {
             printf("* Logging xptinfo stats to: %s\n", statsFilename);
         }
@@ -137,7 +148,7 @@ xptiInterfaceInfoManager::xptiInterfaceInfoManager(nsISupportsArray* aSearchPath
     {
         mAutoRegLogFile = do_CreateInstance(NS_LOCAL_FILE_CONTRACTID);         
         if(mAutoRegLogFile && 
-           NS_SUCCEEDED(mAutoRegLogFile->InitWithPath(autoRegFilename)))
+           NS_SUCCEEDED(mAutoRegLogFile->InitWithNativePath(nsDependentCString(autoRegFilename))))
         {
             printf("* Logging xptinfo autoreg to: %s\n", autoRegFilename);
         }
@@ -154,19 +165,19 @@ xptiInterfaceInfoManager::~xptiInterfaceInfoManager()
     // We only do this on shutdown of the service.
     mWorkingSet.InvalidateInterfaceInfos();
 
-#ifdef XPTI_HAS_ZIP_SUPPORT
-    xptiZipLoader::Shutdown();
-#endif /* XPTI_HAS_ZIP_SUPPORT */
-
     if(mResolveLock)
         PR_DestroyLock(mResolveLock);
     if(mAutoRegLock)
         PR_DestroyLock(mAutoRegLock);
     if(mInfoMonitor)
         nsAutoMonitor::DestroyMonitor(mInfoMonitor);
+    if(mAdditionalManagersLock)
+        PR_DestroyLock(mAdditionalManagersLock);
 
+    gInterfaceInfoManager = nsnull;
 #ifdef DEBUG
     xptiInterfaceInfo::DEBUG_ShutdownNotification();
+    gCallCount = 0;
 #endif
 }
 
@@ -210,6 +221,7 @@ AppendFromDirServiceList(const char* codename, nsISupportsArray* aPath)
         if(!dir || !aPath->AppendElement(dir))
             return PR_FALSE;
     }
+
     return PR_TRUE;
 }        
 
@@ -217,8 +229,7 @@ AppendFromDirServiceList(const char* codename, nsISupportsArray* aPath)
 PRBool xptiInterfaceInfoManager::BuildFileSearchPath(nsISupportsArray** aPath)
 {
 #ifdef DEBUG
-    static int callCount = 0;
-    NS_ASSERTION(!callCount++, "Expected only one call!");
+    NS_ASSERTION(!gCallCount++, "Expected only one call!");
 #endif
 
     nsCOMPtr<nsISupportsArray> searchPath;
@@ -239,6 +250,22 @@ PRBool xptiInterfaceInfoManager::BuildFileSearchPath(nsISupportsArray** aPath)
 
     // Add additional plugins dirs
     // No error checking here since this is optional in some embeddings
+    
+    // Add the MRE's component directory to searchPath if the 
+    // application is using an MRE.
+    // An application indicates that it's using an MRE by returning
+    // a valid nsIFile via it's directory service provider interface.
+    //
+    // Please see http://www.mozilla.org/projects/embedding/MRE.html
+    // for more info. on MREs
+    //
+    nsCOMPtr<nsILocalFile> mreComponentDirectory;
+    nsresult rv = GetDirectoryFromDirService(NS_MRE_COMPONENT_DIR, 
+                                    getter_AddRefs(mreComponentDirectory));
+    if (NS_SUCCEEDED(rv))
+    {
+        searchPath->AppendElement(mreComponentDirectory);
+    }
 
     (void) AppendFromDirServiceList(NS_APP_PLUGINS_DIR_LIST, searchPath);
 
@@ -324,11 +351,11 @@ xptiInterfaceInfoManager::BuildFileList(nsISupportsArray* aSearchPath,
                 continue;
             }
      
-            nsXPIDLCString name;
-            if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+            nsCAutoString name;
+            if(NS_FAILED(file->GetNativeLeafName(name)))
                 return PR_FALSE;
 
-            if(xptiFileType::IsUnknown(name))
+            if(xptiFileType::IsUnknown(name.get()))
                 continue;
 
             LOG_AUTOREG(("found file: %s\n", name.get()));
@@ -424,14 +451,13 @@ xptiInterfaceInfoManager::LoadFile(const xptiTypelib& aTypelibRecord,
                                     getter_AddRefs(file))) || !file)
         return PR_FALSE;
 
-    if(NS_FAILED(file->Append(fileRecord->GetName())))
+    if(NS_FAILED(file->AppendNative(nsDependentCString(fileRecord->GetName()))))
         return PR_FALSE;
 
     XPTHeader* header;
 
     if(aTypelibRecord.IsZip())
     {
-#ifdef XPTI_HAS_ZIP_SUPPORT
         zipItem = &aWorkingSet->GetZipItemAt(aTypelibRecord.GetZipItemIndex());
         
         // See the big comment below in the 'non-zip' case...
@@ -446,10 +472,26 @@ xptiInterfaceInfoManager::LoadFile(const xptiTypelib& aTypelibRecord,
         }
         
         LOG_LOAD(("# loading zip item %s::%s\n", fileRecord->GetName(), zipItem->GetName()));
-        header = xptiZipLoader::ReadXPTFileFromZip(file, zipItem->GetName(), aWorkingSet);
-#else
-        header = nsnull;
-#endif /* XPTI_HAS_ZIP_SUPPORT */
+        
+        nsCOMPtr<nsIXPTLoader> loader =
+            do_GetService(NS_ZIPLOADER_CONTRACTID);
+
+        if (loader) {
+            nsresult rv;
+            
+            nsCOMPtr<nsIInputStream> stream;
+            rv = loader->LoadEntry(file, zipItem->GetName(),
+                                   getter_AddRefs(stream));
+
+            if (NS_FAILED(rv))
+                return PR_FALSE;
+            
+            header =
+                xptiZipLoader::ReadXPTFileFromInputStream(stream, aWorkingSet);
+        } else {
+            header = nsnull;
+            NS_WARNING("Could not load XPT Zip loader");
+        }
     } 
     else
     {
@@ -603,27 +645,27 @@ xptiSortFileList(const void * p1, const void *p2, void * closure)
     nsILocalFile* pFile2 = *((nsILocalFile**) p2);
     SortData* data = (SortData*) closure;
     
-    nsXPIDLCString name1;
-    nsXPIDLCString name2;
+    nsCAutoString name1;
+    nsCAutoString name2;
         
-    if(NS_FAILED(pFile1->GetLeafName(getter_Copies(name1))))
+    if(NS_FAILED(pFile1->GetNativeLeafName(name1)))
     {
         NS_ERROR("way bad, with no happy out!");
         return 0;    
     }    
-    if(NS_FAILED(pFile2->GetLeafName(getter_Copies(name2))))
+    if(NS_FAILED(pFile2->GetNativeLeafName(name2)))
     {
         NS_ERROR("way bad, with no happy out!");
         return 0;    
     }    
 
-    int index1 = IndexOfFileWithName(name1, data->mWorkingSet); 
-    int index2 = IndexOfFileWithName(name2, data->mWorkingSet); 
+    int index1 = IndexOfFileWithName(name1.get(), data->mWorkingSet); 
+    int index2 = IndexOfFileWithName(name2.get(), data->mWorkingSet); 
    
     // Get these now in case we need them later.
-    PRBool isXPT1 = xptiFileType::IsXPT(name1);
-    PRBool isXPT2 = xptiFileType::IsXPT(name2);
-    int nameOrder = PL_strcmp(name1, name2);
+    PRBool isXPT1 = xptiFileType::IsXPT(name1.get());
+    PRBool isXPT2 = xptiFileType::IsXPT(name2.get());
+    int nameOrder = Compare(name1, name2);
     
     // both in workingSet, preserve old order
     if(index1 != -1 && index2 != -1)
@@ -765,12 +807,12 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
 
             PRInt64 size;
             PRInt64 date;
-            nsXPIDLCString name;
+            nsCAutoString name;
             PRUint32 directory;
 
             if(NS_FAILED(file->GetFileSize(&size)) ||
                NS_FAILED(file->GetLastModifiedTime(&date)) ||
-               NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+               NS_FAILED(file->GetNativeLeafName(name)) ||
                !aWorkingSet->FindDirectoryOfFile(file, &directory))
             {
                 NS_ERROR("unexpected!");
@@ -782,7 +824,7 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
                 xptiFile& target = aWorkingSet->GetFileAt(k);
                 
                 if(directory == target.GetDirectory() &&
-                   0 == PL_strcmp(name, target.GetName()))
+                   name.Equals(target.GetName()))
                 {
                     if(nsInt64(size) != target.GetSize() ||
                        nsInt64(date) != target.GetDate())
@@ -814,18 +856,18 @@ xptiInterfaceInfoManager::DetermineAutoRegStrategy(nsISupportsArray* aSearchPath
                 aFileList->QueryElementAt(k, NS_GET_IID(nsILocalFile), getter_AddRefs(file));
                 NS_ASSERTION(file, "loser!");
                 
-                nsXPIDLCString name;
+                nsCAutoString name;
                 PRInt64 size;
                 PRInt64 date;
                 if(NS_FAILED(file->GetFileSize(&size)) ||
                    NS_FAILED(file->GetLastModifiedTime(&date)) ||
-                   NS_FAILED(file->GetLeafName(getter_Copies(name))))
+                   NS_FAILED(file->GetNativeLeafName(name)))
                 {
                     NS_ERROR("unexpected!");
                     return FULL_VALIDATION_REQUIRED;
                 }    
             
-                PRBool sameName = (0 == PL_strcmp(name, target.GetName()));
+                PRBool sameName = name.Equals(target.GetName());
                 if(sameName)
                 {
                     if(nsInt64(size) != target.GetSize() ||
@@ -881,20 +923,20 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
     {
         nsILocalFile* file = orderedFileArray[i];
 
-        nsXPIDLCString name;
+        nsCAutoString name;
         PRInt64 size;
         PRInt64 date;
         PRUint32 dir;
         if(NS_FAILED(file->GetFileSize(&size)) ||
            NS_FAILED(file->GetLastModifiedTime(&date)) ||
-           NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+           NS_FAILED(file->GetNativeLeafName(name)) ||
            !aWorkingSet->FindDirectoryOfFile(file, &dir))
         {
             return PR_FALSE;
         }    
     
 
-        if(xptiWorkingSet::NOT_FOUND != aWorkingSet->FindFile(dir, name))
+        if(xptiWorkingSet::NOT_FOUND != aWorkingSet->FindFile(dir, name.get()))
         {
             // This file was found in the working set, so skip it.       
             continue;
@@ -904,7 +946,7 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
 
         xptiFile fileRecord;
         fileRecord = xptiFile(nsInt64(size), nsInt64(date), dir,
-                              name, aWorkingSet);
+                              name.get(), aWorkingSet);
 
         if(xptiFileType::IsXPT(fileRecord.GetName()))
         {
@@ -958,20 +1000,29 @@ xptiInterfaceInfoManager::AddOnlyNewFilesFromFileList(nsISupportsArray* aSearchP
             // This will correspond to typelibRecord above.
             aWorkingSet->AppendFile(fileRecord);
         }
-#ifdef XPTI_HAS_ZIP_SUPPORT
-        else // It is a zip file, Oh boy!
+        else // its another kind of archive
         {
-            if(!xptiZipLoader::EnumerateZipEntries(file, 
-                          NS_STATIC_CAST(xptiEntrySink*, this), 
-                          aWorkingSet))
-            {
-                return PR_FALSE;    
+            nsCOMPtr<nsIXPTLoader> loader =
+                do_GetService(NS_ZIPLOADER_CONTRACTID);
+            
+            if (loader) {
+                nsresult rv;
+                
+                nsCOMPtr<nsIXPTLoaderSink> sink =
+                    new xptiZipLoaderSink(this, aWorkingSet);
+                if (!sink)
+                    return PR_FALSE;
+                
+                rv = loader->EnumerateEntries(file, sink);
+                if (NS_FAILED(rv))
+                    return PR_FALSE;
+                // This will correspond to typelibRecord used in
+                // xptiInterfaceInfoManager::FoundEntry.
+                aWorkingSet->AppendFile(fileRecord);
+            } else {
+                NS_WARNING("Could not load XPT Zip loader");
             }
-            // This will correspond to typelibRecord used in
-            // xptiInterfaceInfoManager::FoundEntry.
-            aWorkingSet->AppendFile(fileRecord);
         }
-#endif /* XPTI_HAS_ZIP_SUPPORT */
     }
 
     return PR_TRUE;
@@ -1022,13 +1073,13 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
     {
         nsILocalFile* file = orderedFileArray[i];
 
-        nsXPIDLCString name;
+        nsCAutoString name;
         PRInt64 size;
         PRInt64 date;
         PRUint32 dir;
         if(NS_FAILED(file->GetFileSize(&size)) ||
            NS_FAILED(file->GetLastModifiedTime(&date)) ||
-           NS_FAILED(file->GetLeafName(getter_Copies(name))) ||
+           NS_FAILED(file->GetNativeLeafName(name)) ||
            !aWorkingSet->FindDirectoryOfFile(file, &dir))
         {
             return PR_FALSE;
@@ -1038,7 +1089,7 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
     
         xptiFile fileRecord;
         fileRecord = xptiFile(nsInt64(size), nsInt64(date), dir,
-                              name, aWorkingSet);
+                              name.get(), aWorkingSet);
 
 //        printf("* found %s\n", fileRecord.GetName());
 
@@ -1095,30 +1146,59 @@ xptiInterfaceInfoManager::DoFullValidationMergeFromFileList(nsISupportsArray* aS
             // This will correspond to typelibRecord above.
             aWorkingSet->AppendFile(fileRecord);
         }
-#ifdef XPTI_HAS_ZIP_SUPPORT
-        else // It is a zip file, Oh boy!
+
+        else
         {
-            if(!xptiZipLoader::EnumerateZipEntries(file, 
-                          NS_STATIC_CAST(xptiEntrySink*, this), 
-                          aWorkingSet))
-            {
-                return PR_FALSE;    
+            nsCOMPtr<nsIXPTLoader> loader =
+                do_GetService(NS_ZIPLOADER_CONTRACTID);
+            
+            if (loader) {
+                nsresult rv;
+                
+                nsCOMPtr<nsIXPTLoaderSink> sink =
+                    new xptiZipLoaderSink(this, aWorkingSet);
+                if (!sink)
+                    return PR_FALSE;
+                
+                rv = loader->EnumerateEntries(file, sink);
+                if (NS_FAILED(rv))
+                    return PR_FALSE;
+                // This will correspond to typelibRecord used in
+                // xptiInterfaceInfoManager::FoundEntry.
+                aWorkingSet->AppendFile(fileRecord);
+            } else {
+                NS_WARNING("Could not load XPT Zip loader");
             }
-            // This will correspond to typelibRecord used in
-            // xptiInterfaceInfoManager::FoundEntry.
-            aWorkingSet->AppendFile(fileRecord);
         }
-#endif /* XPTI_HAS_ZIP_SUPPORT */
     }
     return PR_TRUE;
 }        
 
+NS_IMPL_ISUPPORTS1(xptiZipLoaderSink, nsIXPTLoaderSink)
+
+// implement nsIXPTLoader
+NS_IMETHODIMP
+xptiZipLoaderSink::FoundEntry(const char* entryName,
+                              PRInt32 index,
+                              nsIInputStream *aStream)
+{
+    XPTHeader *header =
+        xptiZipLoader::ReadXPTFileFromInputStream(aStream, mWorkingSet);
+    if (!header)
+        return NS_ERROR_OUT_OF_MEMORY;
+    
+    if (!mManager->FoundZipEntry(entryName, index, header, mWorkingSet))
+        return NS_ERROR_FAILURE;
+    
+    return NS_OK;
+}
+
 // implement xptiEntrySink
 PRBool 
-xptiInterfaceInfoManager::FoundEntry(const char* entryName,
-                                     int index,
-                                     XPTHeader* header,
-                                     xptiWorkingSet* aWorkingSet)
+xptiInterfaceInfoManager::FoundZipEntry(const char* entryName,
+                                        int index,
+                                        XPTHeader* header,
+                                        xptiWorkingSet* aWorkingSet)
 {
 
     NS_ASSERTION(entryName, "loser!");
@@ -1484,8 +1564,8 @@ xptiInterfaceInfoManager::DEBUG_DumpFileList(nsISupportsArray* aFileList)
         if(!file)
             return PR_FALSE;
 
-        nsXPIDLCString name;
-        if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+        nsCAutoString name;
+        if(NS_FAILED(file->GetNativeLeafName(name)))
             return PR_FALSE;
 
         printf("* found %s\n", name.get());
@@ -1514,8 +1594,8 @@ xptiInterfaceInfoManager::DEBUG_DumpFileArray(nsILocalFile** aFileArray,
     {
         nsILocalFile* file = aFileArray[i];
     
-        nsXPIDLCString name;
-        if(NS_FAILED(file->GetLeafName(getter_Copies(name))))
+        nsCAutoString name;
+        if(NS_FAILED(file->GetNativeLeafName(name)))
             return PR_FALSE;
 
         printf("found file: %s\n", name.get());
@@ -1792,7 +1872,7 @@ NS_IMETHODIMP xptiInterfaceInfoManager::AutoRegisterInterfaces()
     AutoRegMode mode;
     PRBool ok;
 
-    nsAutoLock lock(xptiInterfaceInfoManager::GetAutoRegLock());
+    nsAutoLock lock(xptiInterfaceInfoManager::GetAutoRegLock(this));
 
     xptiWorkingSet workingSet(mSearchPath);
     if(!workingSet.IsValid())
@@ -1872,6 +1952,163 @@ NS_IMETHODIMP xptiInterfaceInfoManager::AutoRegisterInterfaces()
 
 /***************************************************************************/
 
+class xptiAdditionalManagersEnumerator : public nsISimpleEnumerator 
+{
+public:
+    NS_DECL_ISUPPORTS
+    NS_DECL_NSISIMPLEENUMERATOR
+
+    xptiAdditionalManagersEnumerator();
+    virtual ~xptiAdditionalManagersEnumerator() {}
+
+    PRBool SizeTo(PRUint32 likelyCount) {return mArray.SizeTo(likelyCount);}
+    PRBool AppendElement(nsIInterfaceInfoManager* element);
+
+private:
+    nsSupportsArray mArray;
+    PRUint32        mIndex;
+    PRUint32        mCount;
+};
+
+NS_IMPL_ISUPPORTS1(xptiAdditionalManagersEnumerator, nsISimpleEnumerator)
+
+xptiAdditionalManagersEnumerator::xptiAdditionalManagersEnumerator()
+    : mIndex(0), mCount(0)
+{
+    NS_INIT_ISUPPORTS();
+}
+
+PRBool xptiAdditionalManagersEnumerator::AppendElement(nsIInterfaceInfoManager* element)
+{
+    if(!mArray.AppendElement(NS_STATIC_CAST(nsISupports*, element)))
+        return PR_FALSE;
+    mCount++;
+    return PR_TRUE;
+}
+
+/* boolean hasMoreElements (); */
+NS_IMETHODIMP xptiAdditionalManagersEnumerator::HasMoreElements(PRBool *_retval)
+{
+    *_retval = mIndex < mCount;
+    return NS_OK;
+}
+
+/* nsISupports getNext (); */
+NS_IMETHODIMP xptiAdditionalManagersEnumerator::GetNext(nsISupports **_retval)
+{
+    if(!(mIndex < mCount))
+    {
+        NS_ERROR("Bad nsISimpleEnumerator caller!");
+        return NS_ERROR_FAILURE;    
+    }
+
+    *_retval = mArray.ElementAt(mIndex++);
+    return *_retval ? NS_OK : NS_ERROR_FAILURE;
+}
+
+/***************************************************************************/
+
+/* void addAdditionalManager (in nsIInterfaceInfoManager manager); */
+NS_IMETHODIMP xptiInterfaceInfoManager::AddAdditionalManager(nsIInterfaceInfoManager *manager)
+{
+    nsCOMPtr<nsIWeakReference> weakRef = do_GetWeakReference(manager);
+    nsISupports* ptrToAdd = weakRef ? 
+                    NS_STATIC_CAST(nsISupports*, weakRef) :
+                    NS_STATIC_CAST(nsISupports*, manager);
+    { // scoped lock...
+        nsAutoLock lock(mAdditionalManagersLock);
+        PRInt32 index;
+        nsresult rv = mAdditionalManagers.GetIndexOf(ptrToAdd, &index);
+        if(NS_FAILED(rv) || -1 != index)
+            return NS_ERROR_FAILURE;
+        if(!mAdditionalManagers.AppendElement(ptrToAdd))
+            return NS_ERROR_OUT_OF_MEMORY;
+    }
+    return NS_OK;
+}
+
+/* void removeAdditionalManager (in nsIInterfaceInfoManager manager); */
+NS_IMETHODIMP xptiInterfaceInfoManager::RemoveAdditionalManager(nsIInterfaceInfoManager *manager)
+{
+    nsCOMPtr<nsIWeakReference> weakRef = do_GetWeakReference(manager);
+    nsISupports* ptrToRemove = weakRef ? 
+                    NS_STATIC_CAST(nsISupports*, weakRef) :
+                    NS_STATIC_CAST(nsISupports*, manager);
+    { // scoped lock...
+        nsAutoLock lock(mAdditionalManagersLock);
+        if(!mAdditionalManagers.RemoveElement(ptrToRemove))
+            return NS_ERROR_FAILURE;
+    }
+    return NS_OK;
+}
+
+/* PRBool hasAdditionalManagers (); */
+NS_IMETHODIMP xptiInterfaceInfoManager::HasAdditionalManagers(PRBool *_retval)
+{
+    PRUint32 count;
+    nsresult rv = mAdditionalManagers.Count(&count);
+    *_retval = count != 0;
+    return rv;
+}
+
+/* nsISimpleEnumerator enumerateAdditionalManagers (); */
+NS_IMETHODIMP xptiInterfaceInfoManager::EnumerateAdditionalManagers(nsISimpleEnumerator **_retval)
+{
+    nsAutoLock lock(mAdditionalManagersLock);
+
+    PRUint32 count;
+    nsresult rv = mAdditionalManagers.Count(&count);
+    if(NS_FAILED(rv))
+        return rv;
+
+    nsCOMPtr<xptiAdditionalManagersEnumerator> enumerator = 
+        new xptiAdditionalManagersEnumerator();
+    if(!enumerator)
+        return NS_ERROR_OUT_OF_MEMORY;
+
+    enumerator->SizeTo(count);
+
+    for(PRUint32 i = 0; i < count; /* i incremented in the loop body */)
+    {
+        nsCOMPtr<nsISupports> raw = 
+            dont_AddRef(mAdditionalManagers.ElementAt(i++));
+        if(!raw)
+            return NS_ERROR_FAILURE;
+        nsCOMPtr<nsIWeakReference> weakRef = do_QueryInterface(raw);
+        if(weakRef)
+        {
+            nsCOMPtr<nsIInterfaceInfoManager> manager = 
+                do_QueryReferent(weakRef);
+            if(manager)
+            {
+                if(!enumerator->AppendElement(manager))
+                    return NS_ERROR_FAILURE;
+            }
+            else
+            {
+                // The manager is no more. Remove the element.
+                if(!mAdditionalManagers.RemoveElementAt(--i))
+                    return NS_ERROR_FAILURE;
+                count--;
+            }
+        }
+        else
+        {
+            // We *know* we put a pointer to either a nsIWeakReference or
+            // an nsIInterfaceInfoManager into the array, so we can avoid an
+            // extra QI here and just do a cast.
+            if(!enumerator->AppendElement(
+                    NS_REINTERPRET_CAST(nsIInterfaceInfoManager*, raw.get())))
+                return NS_ERROR_FAILURE;
+        }
+    }
+    
+    NS_ADDREF(*_retval = enumerator);
+    return NS_OK;
+}
+
+/***************************************************************************/
+
 XPTI_PUBLIC_API(nsIInterfaceInfoManager*)
 XPTI_GetInterfaceInfoManager()
 {
