@@ -40,6 +40,8 @@
 #include "nsIRunnable.h"
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
+#include "nsIObservableInputStream.h"
+#include "nsIObservableOutputStream.h"
 #include "nsIServiceManager.h"
 #include "prprf.h"
 #include "prinrval.h"
@@ -126,7 +128,7 @@ TestPipe(nsIInputStream* in, nsIOutputStream* out)
     for (PRUint32 i = 0; i < ITERATIONS; i++) {
         PRUint32 writeCount;
         char* buf = PR_smprintf("%d %s", i, kTestPattern);
-        rv = out->Write(buf, nsCRT::strlen(buf), &writeCount);
+        rv = out->Write(buf, strlen(buf), &writeCount);
         if (gTrace) {
             printf("wrote: ");
             for (PRUint32 j = 0; j < writeCount; j++) {
@@ -237,7 +239,7 @@ TestShortWrites(nsIInputStream* in, nsIOutputStream* out)
     for (PRUint32 i = 0; i < ITERATIONS; i++) {
         PRUint32 writeCount;
         char* buf = PR_smprintf("%d %s", i, kTestPattern);
-        PRUint32 len = nsCRT::strlen(buf);
+        PRUint32 len = strlen(buf);
         len = len * rand() / RAND_MAX;
         len = PR_MAX(1, len);
         rv = out->Write(buf, len, &writeCount);
@@ -314,9 +316,14 @@ TestPipeObserver()
     rv = NS_NewPipe(&in, &out, 18, 36, PR_TRUE, PR_TRUE);
     if (NS_FAILED(rv)) return rv;
 
-    rv = in->SetObserver(obs);
+    nsCOMPtr<nsIObservableInputStream> observableIn(do_QueryInterface(in, &rv));
     if (NS_FAILED(rv)) return rv;
-    rv = out->SetObserver(obs);
+    nsCOMPtr<nsIObservableInputStream> observableOut(do_QueryInterface(out, &rv));
+    if (NS_FAILED(rv)) return rv;
+
+    rv = observableIn->SetObserver(obs);
+    if (NS_FAILED(rv)) return rv;
+    rv = observableOut->SetObserver(obs);
     if (NS_FAILED(rv)) return rv;
 
     char buf[] = "puirt a beul: a style of Gaelic vocal music intended for dancing.";
@@ -489,7 +496,7 @@ TestChainedPipes()
     for (PRUint32 i = 0; i < ITERATIONS; i++) {
         PRUint32 writeCount;
         char* buf = PR_smprintf("%d %s", i, kTestPattern);
-        PRUint32 len = nsCRT::strlen(buf);
+        PRUint32 len = strlen(buf);
         len = len * rand() / RAND_MAX;
         len = PR_MAX(1, len);
         rv = out1->Write(buf, len, &writeCount);
@@ -647,6 +654,9 @@ main(int argc, char* argv[])
     NS_ASSERTION(NS_SUCCEEDED(rv), "TestChainedPipes failed");
     RunTests(16, 1);
     RunTests(4096, 16);
+    NS_RELEASE(servMgr);
+    rv = NS_ShutdownXPCOM( NULL );
+    NS_ASSERTION(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM failed");
     return 0;
 }
 
