@@ -67,6 +67,11 @@
 #include "prprf.h"    
 #endif
 
+#if defined(XP_MAC) || defined(XP_MACOSX)
+#include <Gestalt.h>
+static PRBool OnMacOSX();
+#endif
+
 #include "nsWidgetsCID.h"
 #include "nsIRequestObserver.h"
 
@@ -75,6 +80,12 @@
 #include "nsIScriptContext.h"
 #include "jsapi.h"
 
+/* for the "remigration" stuff */
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
+#include "nsIPromptService.h"
+#include "nsIStringBundle.h"
+
 #include "nsAppShellService.h"
 #include "nsIProfileInternal.h"
 
@@ -92,13 +103,9 @@ static NS_DEFINE_CID(kXPConnectCID, NS_XPCONNECT_CID);
 #define gInstallRestartTopic     "xpinstall-restart"
 
 nsAppShellService::nsAppShellService() : 
-  mAppShell( nsnull ),
-  mCmdLineService( nsnull ),
-  mWindowMediator( nsnull ), 
-  mHiddenWindow( nsnull ),
   mDeleteCalled( PR_FALSE ),
-  mSplashScreen( nsnull ),
-  mNativeAppSupport( nsnull ),
+  mInProfileStartup( PR_FALSE ),
+  mModalWindowCount( 0 ),
   mShuttingDown( PR_FALSE ),
   mQuitOnLastWindowClosing( PR_TRUE )
 {
@@ -228,9 +235,9 @@ nsAppShellService::DoProfileStartup(nsICmdLineService *aCmdLineService, PRBool c
     nsresult rv;
 
     nsCOMPtr<nsIProfileInternal> profileMgr(do_GetService(NS_PROFILE_CONTRACTID, &rv));
-    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get profile manager");
-    if (NS_FAILED(rv)) return rv;
+    NS_ENSURE_SUCCESS(rv,rv);
 
+    mInProfileStartup = PR_TRUE;
     PRBool saveQuitOnLastWindowClosing = mQuitOnLastWindowClosing;
     mQuitOnLastWindowClosing = PR_FALSE;
         
@@ -241,11 +248,153 @@ nsAppShellService::DoProfileStartup(nsICmdLineService *aCmdLineService, PRBool c
         rv = NS_OK;
     }
     
-    mQuitOnLastWindowClosing = saveQuitOnLastWindowClosing;
+    if (NS_SUCCEEDED(rv)) {
+        rv = CheckAndRemigrateDefunctProfile();
+        NS_ASSERTION(NS_SUCCEEDED(rv), "failed to check and remigrate profile");
+        rv = NS_OK;
+    }
 
+    mQuitOnLastWindowClosing = saveQuitOnLastWindowClosing;
+    mInProfileStartup = PR_FALSE;
+
+    // if Quit() was called while we were starting up we have a failure situation...
+    if (mShuttingDown)
+      return NS_ERROR_FAILURE;
     return rv;
 }
 
+nsresult
+nsAppShellService::CheckAndRemigrateDefunctProfile()
+{
+  nsresult rv;
+
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv,rv);
+  rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  PRInt32 secondsBeforeDefunct;
+  rv = prefBranch->GetIntPref("profile.seconds_until_defunct", &secondsBeforeDefunct);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  // -1 is the value for "never go defunct"
+  // if the pref is set to -1, we'll never prompt the user to remigrate
+  // see all.js (and all-ns.js)
+  if (secondsBeforeDefunct == -1)
+    return NS_OK;
+
+  // used for converting
+  // seconds -> millisecs
+  // and microsecs -> millisecs
+  PRInt64 oneThousand = LL_INIT(0, 1000);
+  
+  PRInt64 defunctInterval;
+  // Init as seconds
+  LL_I2L(defunctInterval, secondsBeforeDefunct);
+  // Convert secs to millisecs
+  LL_MUL(defunctInterval, defunctInterval, oneThousand);
+        
+  nsCOMPtr<nsIProfileInternal> profileMgr(do_GetService(NS_PROFILE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsXPIDLString profileName;
+  PRInt64 lastModTime;
+  profileMgr->GetCurrentProfile(getter_Copies(profileName));
+  rv = profileMgr->GetProfileLastModTime(profileName.get(), &lastModTime);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  // convert "now" from microsecs to millisecs
+  PRInt64 nowInMilliSecs = PR_Now(); 
+  LL_DIV(nowInMilliSecs, nowInMilliSecs, oneThousand);
+  
+  // determine (using the pref value) when the profile would be considered defunct
+  PRInt64 defunctIntervalAgo;
+  LL_SUB(defunctIntervalAgo, nowInMilliSecs, defunctInterval);
+
+  // if we've used our current 6.x / mozilla profile more recently than
+  // when we'd consider it defunct, don't remigrate
+  if (LL_CMP(lastModTime, >, defunctIntervalAgo))
+    return NS_OK;
+  
+  nsCOMPtr<nsILocalFile> origProfileDir;
+  rv = profileMgr->GetOriginalProfileDir(profileName, getter_AddRefs(origProfileDir));
+  // if this fails
+  // then the current profile is a new one (not from 4.x) 
+  // so we are done.
+  if (NS_FAILED(rv))
+    return NS_OK;
+  
+  // Now, we know that a matching 4.x profile exists
+  // See if it has any newer files in it than our defunct profile.
+  nsCOMPtr<nsISimpleEnumerator> dirEnum;
+  rv = origProfileDir->GetDirectoryEntries(getter_AddRefs(dirEnum));
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  PRBool promptForRemigration = PR_FALSE;
+  PRBool hasMore;
+  while (NS_SUCCEEDED(dirEnum->HasMoreElements(&hasMore)) && hasMore) {
+    nsCOMPtr<nsILocalFile> currElem;
+    rv = dirEnum->GetNext(getter_AddRefs(currElem));
+    NS_ENSURE_SUCCESS(rv,rv);
+    
+    PRInt64 currElemModTime;
+    rv = currElem->GetLastModifiedTime(&currElemModTime);
+    NS_ENSURE_SUCCESS(rv,rv);
+    // if this file in our 4.x profile is more recent than when we last used our mozilla / 6.x profile
+    // we should prompt for re-migration
+    if (LL_CMP(currElemModTime, >, lastModTime)) {
+      promptForRemigration = PR_TRUE;
+      break;
+    }
+  }
+  
+  // If nothing in the 4.x dir is newer than our defunct profile, return.
+  if (!promptForRemigration)
+    return NS_OK;
+ 
+  nsCOMPtr<nsIStringBundleService> stringBundleService(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIStringBundle> migrationBundle, brandBundle;
+  rv = stringBundleService->CreateBundle("chrome://communicator/locale/profile/migration.properties", getter_AddRefs(migrationBundle));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  rv = stringBundleService->CreateBundle("chrome://global/locale/brand.properties", getter_AddRefs(brandBundle));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsXPIDLString brandName;
+  rv = brandBundle->GetStringFromName(NS_LITERAL_STRING("brandShortName").get(), getter_Copies(brandName));
+  NS_ENSURE_SUCCESS(rv,rv);
+ 
+  nsXPIDLString dialogText;
+  rv = migrationBundle->GetStringFromName(NS_LITERAL_STRING("confirmRemigration").get(), getter_Copies(dialogText));
+  NS_ENSURE_SUCCESS(rv,rv);
+
+  nsCOMPtr<nsIPromptService> promptService(do_GetService("@mozilla.org/embedcomp/prompt-service;1", &rv));
+  NS_ENSURE_SUCCESS(rv,rv);
+  PRInt32 buttonPressed;
+  rv = promptService->ConfirmEx(nsnull, brandName.get(),
+    dialogText.get(),
+    (nsIPromptService::BUTTON_POS_0 * 
+    nsIPromptService::BUTTON_TITLE_YES) + 
+    (nsIPromptService::BUTTON_POS_1 * 
+    nsIPromptService::BUTTON_TITLE_NO),
+    nsnull, nsnull, nsnull, nsnull, nsnull, &buttonPressed);
+  NS_ENSURE_SUCCESS(rv,rv);
+  
+  if (buttonPressed == 0) {
+    // Need to shut down the current profile before remigrating it
+    profileMgr->ShutDownCurrentProfile(nsIProfile::SHUTDOWN_PERSIST);
+    // If this fails, it will restore what was there.
+    rv = profileMgr->RemigrateProfile(profileName.get());
+    NS_ASSERTION(NS_SUCCEEDED(rv), "Remigration of profile failed.");
+    // Whether or not we succeeded or failed, need to reset this.
+    profileMgr->SetCurrentProfile(profileName.get());
+  }
+  return NS_OK;
+}   
+
 NS_IMETHODIMP
 nsAppShellService::CreateHiddenWindow()
 {
@@ -405,7 +554,9 @@ nsAppShellService::HandleExitEvent(PLEvent* aEvent)
   event->mService->mAppShell->Exit();
 
   // We're done "shutting down".
-  event->mService->mShuttingDown = PR_FALSE;
+  // unless we are currently "starting up" in which case this event wasn't properly handled.
+  if (!event->mService->mInProfileStartup)
+    event->mService->mShuttingDown = PR_FALSE;
 
   return nsnull;
 }
@@ -478,6 +629,11 @@ nsAppShellService::JustCreateTopWindow(nsIXULWindow *aParent,
       widgetInitData.mWindowType = aChromeMask & nsIWebBrowserChrome::CHROME_OPENAS_DIALOG ?
                                     eWindowType_dialog : eWindowType_toplevel;
 
+    if (aChromeMask & nsIWebBrowserChrome::CHROME_WINDOW_POPUP)
+      widgetInitData.mWindowType = eWindowType_popup;
+
+    widgetInitData.mContentType = nsIDocShellTreeItem::typeChrome;
+    
     // note default chrome overrides other OS chrome settings, but
     // not internal chrome
     if (aChromeMask & nsIWebBrowserChrome::CHROME_DEFAULT)
@@ -494,35 +650,52 @@ nsAppShellService::JustCreateTopWindow(nsIXULWindow *aParent,
         widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_close);
       if (aChromeMask & nsIWebBrowserChrome::CHROME_WINDOW_RESIZE) {
         widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_resizeh);
-        /* Associate the resize flag with min/max buttons and system menu.
-           but not for dialogs. This is logic better associated with the
-           platform implementation, and partially covered by the
-           eBorderStyle_default style. But since I know of no platform
-           that wants min/max buttons on dialogs, it works here, too.
-           If you have such a platform, this is where the fun starts: */
+        // only resizable windows get the maximize button (but not dialogs)
         if (!(aChromeMask & nsIWebBrowserChrome::CHROME_OPENAS_DIALOG))
-          widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_minimize | eBorderStyle_maximize | eBorderStyle_menu);
+          widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_maximize);
       }
-      // Min button only?
+      // all windows (except dialogs) get minimize buttons and the system menu
+      if (!(aChromeMask & nsIWebBrowserChrome::CHROME_OPENAS_DIALOG))
+        widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_minimize | eBorderStyle_menu);
+      // but anyone can explicitly ask for a minimize button
       if (aChromeMask & nsIWebBrowserChrome::CHROME_WINDOW_MIN) {
         widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_minimize );
       }  
     }
 
+#if TARGET_CARBON
+    // Mac OS X sheet support
+    PRUint32 sheetMask = nsIWebBrowserChrome::CHROME_OPENAS_DIALOG |
+                         nsIWebBrowserChrome::CHROME_MODAL;
+    if (aParent && ((aChromeMask & sheetMask) == sheetMask))
+    {
+        widgetInitData.mBorderStyle = NS_STATIC_CAST(enum nsBorderStyle, widgetInitData.mBorderStyle | eBorderStyle_sheet );
+    }
+#endif
+
     zlevel = nsIXULWindow::normalZ;
     if (aChromeMask & nsIWebBrowserChrome::CHROME_WINDOW_RAISED)
       zlevel = nsIXULWindow::raisedZ;
     else if (aChromeMask & nsIWebBrowserChrome::CHROME_WINDOW_LOWERED)
       zlevel = nsIXULWindow::loweredZ;
+
 #if defined(XP_MAC) || defined(XP_MACOSX)
     /* Platforms on which modal windows are always application-modal, not
        window-modal (that's just the Mac, right?) want modal windows to
-       be stacked on top of everyone else. */
-    if ((aChromeMask & nsIWebBrowserChrome::CHROME_MODAL) && aParent)
-      zlevel = nsIXULWindow::highestZ;
+       be stacked on top of everyone else.
+
+       On Mac OS X, bind modality to parent window instead of app (ala Mac OS 9)
+    */
+    PRUint32 modalDepMask = nsIWebBrowserChrome::CHROME_MODAL |
+                            nsIWebBrowserChrome::CHROME_DEPENDENT;
+    if (aParent && (aChromeMask & modalDepMask))
+    {
+      if (::OnMacOSX()) aParent->GetZlevel(&zlevel);
+      else  zlevel = nsIXULWindow::highestZ;
+    }
 #else
     /* Platforms with native support for dependent windows (that's everyone
-       but the Mac, right?) know how to stack dependent windows. On these
+       but pre-Mac OS X, right?) know how to stack dependent windows. On these
        platforms, give the dependent window the same level as its parent,
        so we won't try to override the normal platform behaviour. */
     if ((aChromeMask & nsIWebBrowserChrome::CHROME_DEPENDENT) && aParent)
@@ -580,6 +753,8 @@ nsAppShellService::GetHiddenDOMWindow(nsIDOMWindowInternal **aWindow)
 {
   nsresult rv;
   nsCOMPtr<nsIDocShell> docShell;
+  NS_ENSURE_TRUE(mHiddenWindow, NS_ERROR_FAILURE);
+
   rv = mHiddenWindow->GetDocShell(getter_AddRefs(docShell));
   if (NS_FAILED(rv)) return rv;
   
@@ -737,6 +912,63 @@ nsAppShellService::UnregisterTopLevelWindow(nsIXULWindow* aWindow)
   return NS_OK;
 }
 
+/* The only thing we do with this knowledge is deactivate all other windows
+   on Mac OS 9. */
+NS_IMETHODIMP
+nsAppShellService::TopLevelWindowIsModal(nsIXULWindow *aWindow, PRBool aModal)
+{
+#if defined(XP_MAC) || defined(XP_MACOSX)
+
+  PRBool enable,
+         takeAction = PR_FALSE;
+
+  // adjust our counter and prepare to take action if we just opened
+  // our first modal window or closed our last.
+  NS_ASSERTION(aModal || !aModal && mModalWindowCount > 0, "modal window count error");
+  if (aModal && ++mModalWindowCount == 1) {
+    enable = PR_FALSE;
+    takeAction = PR_TRUE;
+  } else if (!aModal && --mModalWindowCount == 0) {
+    enable = PR_TRUE;
+    takeAction = PR_TRUE;
+  }
+  if (!takeAction)
+    return NS_OK;
+
+  if (::OnMacOSX()) // application modal windows only happen on the classic Mac
+    return NS_OK;
+
+  // prepare to visit each open window
+  if (!mWindowMediator)
+    return NS_ERROR_FAILURE;
+
+  nsCOMPtr<nsISimpleEnumerator> windowList;
+  mWindowMediator->GetXULWindowEnumerator(0, getter_AddRefs(windowList));
+  if (!windowList)
+    return NS_ERROR_FAILURE;
+
+  // if we just closed our last modal window, enable every window.
+  // if we just opened our first, disable every window but aWindow
+  do {
+    PRBool more;
+    windowList->HasMoreElements(&more);
+    if (!more)
+      break;
+
+    nsCOMPtr<nsISupports> supportsWindow;
+    windowList->GetNext(getter_AddRefs(supportsWindow));
+
+    nsCOMPtr<nsIXULWindow> listXULWindow(do_QueryInterface(supportsWindow));
+    if (enable || listXULWindow != aWindow) {
+      nsCOMPtr<nsIBaseWindow> listBaseWindow(do_QueryInterface(supportsWindow));
+      if (listBaseWindow)
+        listBaseWindow->SetEnabled(enable);
+    }
+  } while(1);
+#endif
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsAppShellService::GetQuitOnLastWindowClosing(PRBool *aQuitOnLastWindowClosing)
 {
@@ -839,3 +1071,24 @@ nsAppShellService::GetNativeAppSupport( nsINativeAppSupport **aResult ) {
     NS_IF_ADDREF( *aResult );
     return *aResult ? NS_OK : NS_ERROR_NULL_POINTER;
 }
+
+
+#if defined(XP_MAC) || defined(XP_MACOSX)
+//
+// Return true if we are on Mac OS X, caching the result after the first call
+//
+static PRBool
+OnMacOSX()
+{
+
+  static PRBool gInitVer = PR_FALSE;
+  static PRBool gOnMacOSX = PR_FALSE;
+  if(! gInitVer) {
+    long version;
+    OSErr err = ::Gestalt(gestaltSystemVersion, &version);
+    gOnMacOSX = (err == noErr && version >= 0x00001000);
+    gInitVer = PR_TRUE;
+  }
+  return gOnMacOSX;
+}
+#endif
