@@ -394,9 +394,9 @@ NS_IMETHODIMP nsContentTreeOwner::SetWebBrowser(nsIWebBrowser* aWebBrowser)
 NS_IMETHODIMP nsContentTreeOwner::GetWebBrowser(nsIWebBrowser** aWebBrowser)
 {
   // Unimplemented, and probably will remain so; xpfe windows have docshells,
-  // not webbrowsers. Note that we're relying on this to differentiate
-  // embedded instances from Mozilla in nsWindowWatcher::InitializeDocshell.
-  // (Don't let me stop you, but keep that in mind.)
+  // not webbrowsers.
+  NS_ENSURE_ARG_POINTER(aWebBrowser);
+  *aWebBrowser = 0;
   return NS_ERROR_FAILURE;
 }
 
@@ -410,10 +410,26 @@ NS_IMETHODIMP nsContentTreeOwner::SetChromeFlags(PRUint32 aChromeFlags)
 
 NS_IMETHODIMP nsContentTreeOwner::GetChromeFlags(PRUint32* aChromeFlags)
 {
-   NS_ENSURE_ARG_POINTER(aChromeFlags);
+  NS_ENSURE_ARG_POINTER(aChromeFlags);
 
-   *aChromeFlags = mChromeFlags;
-   return NS_OK;
+  *aChromeFlags = mChromeFlags;
+
+  /* mChromeFlags is kept up to date, except for scrollbar visibility.
+     That can be changed directly by the content DOM window, which
+     doesn't know to update the chrome window. So that we must check
+     separately. */
+
+  // however, it's pointless to ask if the window isn't set up yet
+  if (!mXULWindow->mChromeLoaded)
+    return NS_OK;
+
+  PRBool scrollbarVisibility = mXULWindow->GetContentScrollbarVisibility();
+  if (scrollbarVisibility)
+    *aChromeFlags |= nsIWebBrowserChrome::CHROME_SCROLLBARS;
+  else
+    *aChromeFlags &= ~nsIWebBrowserChrome::CHROME_SCROLLBARS;
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsContentTreeOwner::DestroyBrowserWindow()
@@ -430,7 +446,7 @@ NS_IMETHODIMP nsContentTreeOwner::SizeBrowserTo(PRInt32 aCX, PRInt32 aCY)
 
 NS_IMETHODIMP nsContentTreeOwner::ShowAsModal()
 {
-   return mXULWindow->ShowModal();   
+   return mXULWindow->ShowModal();
 }
 
 NS_IMETHODIMP nsContentTreeOwner::IsWindowModal(PRBool *_retval)
@@ -537,6 +553,16 @@ NS_IMETHODIMP nsContentTreeOwner::SetVisibility(PRBool aVisibility)
    return mXULWindow->SetVisibility(aVisibility);
 }
 
+NS_IMETHODIMP nsContentTreeOwner::GetEnabled(PRBool *aEnabled)
+{
+   return mXULWindow->GetEnabled(aEnabled);
+}
+
+NS_IMETHODIMP nsContentTreeOwner::SetEnabled(PRBool aEnable)
+{
+   return mXULWindow->SetEnabled(aEnable);
+}
+
 NS_IMETHODIMP nsContentTreeOwner::GetMainWidget(nsIWidget** aMainWidget)
 {
    NS_ENSURE_ARG_POINTER(aMainWidget);
@@ -614,7 +640,11 @@ NS_IMETHODIMP nsContentTreeOwner::ApplyChromeFlags()
                                      nsIWebBrowserChrome::CHROME_MENUBAR ? 
                                    PR_TRUE : PR_FALSE);
 
-  // scrollbars have their own special treatment
+  /* Scrollbars have their own special treatment. (note here we *do* use
+     mChromeFlags directly, without going through the accessor. This method
+     is intended to be used right after setting mChromeFlags, so this is
+     where the content window's scrollbar state is set to match mChromeFlags
+     in the first place. */
   mXULWindow->SetContentScrollbarVisibility(mChromeFlags &
                                               nsIWebBrowserChrome::CHROME_SCROLLBARS ?
                                             PR_TRUE : PR_FALSE);
