@@ -67,7 +67,6 @@
 
 #include "nsIXULPopupListener.h"
 #include "nsIDOMXULElement.h"
-#include "nsRDFCID.h"
 
 #include "nsGUIEvent.h"
 #include "nsWidgetsCID.h"
@@ -303,9 +302,9 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
 
   // Create web shell
   mDocShell = do_CreateInstance(kWebShellCID);
-  CallQueryInterface(mDocShell, &mWebShell);
-
   NS_ENSURE_TRUE(mDocShell, NS_ERROR_FAILURE);
+  CallQueryInterface(mDocShell, &mWebShell);
+  NS_ENSURE_TRUE(mWebShell, NS_ERROR_FAILURE);
 
   r.x = r.y = 0;
   nsCOMPtr<nsIBaseWindow> docShellAsWin(do_QueryInterface(mDocShell));
@@ -317,7 +316,7 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   // Attach a WebProgress listener.during initialization...
   nsCOMPtr<nsIWebProgress> webProgress(do_GetInterface(mDocShell, &rv));
   if (webProgress) {
-    webProgress->AddProgressListener(this);
+    webProgress->AddProgressListener(this, nsIWebProgress::NOTIFY_STATE_NETWORK);
   }
 
   nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(mDocShell));
@@ -328,13 +327,12 @@ nsresult nsWebShellWindow::Initialize(nsIXULWindow* aParent,
   docShellAsItem->SetItemType(nsIDocShellTreeItem::typeChrome);
 
   if (nsnull != aUrl)  {
-    char *tmpStr = NULL;
-    nsAutoString urlString;
+    nsCAutoString tmpStr;
 
-    rv = aUrl->GetSpec(&tmpStr);
+    rv = aUrl->GetSpec(tmpStr);
     if (NS_FAILED(rv)) return rv;
-    urlString.AssignWithConversion(tmpStr);
-    nsCRT::free(tmpStr);
+
+    NS_ConvertUTF8toUCS2 urlString(tmpStr);
     nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(mDocShell));
     NS_ENSURE_TRUE(webNav, NS_ERROR_FAILURE);
     rv = webNav->LoadURI(urlString.get(),
@@ -422,30 +420,54 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
       case NS_MOVE: {
         // persist position, but not immediately, in case this OS is firing
         // repeated move events as the user drags the window
-        eventWindow->SetPersistenceTimer(PR_FALSE, PR_TRUE);
+        eventWindow->SetPersistenceTimer(PR_FALSE, PR_TRUE, PR_FALSE);
         break;
       }
       case NS_SIZE: {
+        PRBool chromeLock = PR_FALSE;
         nsSizeEvent* sizeEvent = (nsSizeEvent*)aEvent;
         nsCOMPtr<nsIBaseWindow> shellAsWin(do_QueryInterface(webShell));
         shellAsWin->SetPositionAndSize(0, 0, sizeEvent->windowSize->width, 
           sizeEvent->windowSize->height, PR_FALSE);  
         // persist size, but not immediately, in case this OS is firing
         // repeated size events as the user drags the sizing handle
-        eventWindow->SetPersistenceTimer(PR_TRUE, PR_FALSE);
+        if (NS_FAILED(eventWindow->GetLockedState(chromeLock)) || !chromeLock)
+          eventWindow->SetPersistenceTimer(PR_TRUE, PR_FALSE, PR_TRUE);
         result = nsEventStatus_eConsumeNoDefault;
         break;
       }
       case NS_SIZEMODE: {
         nsSizeModeEvent* modeEvent = (nsSizeModeEvent*)aEvent;
         aEvent->widget->SetSizeMode(modeEvent->mSizeMode);
-        eventWindow->StoreBoundsToXUL(PR_FALSE, PR_FALSE, PR_TRUE);
+        // persist mode, but not immediately, because in many (all?)
+        // cases this will merge with the similar call in NS_SIZE and
+        // write the attribute values only once.
+        eventWindow->SetPersistenceTimer(PR_FALSE, PR_FALSE, PR_TRUE);
         result = nsEventStatus_eConsumeDoDefault;
         // Note the current implementation of SetSizeMode just stores
         // the new state; it doesn't actually resize. So here we store
         // the state and pass the event on to the OS. The day is coming
         // when we'll handle the event here, and the return result will
         // then need to be different.
+#ifdef XP_WIN
+        // This is a nasty hack to get around the fact that win32 sends the kill focus
+        // event in a different sequence than the deactivate depending on if you're
+        // minimizing the window vs. just clicking in a different window to cause
+        // the deactivation. Bug #82534
+        if(modeEvent->mSizeMode == nsSizeMode_Minimized) {
+          nsCOMPtr<nsIDOMWindowInternal> domWindow;
+          eventWindow->ConvertWebShellToDOMWindow(webShell, getter_AddRefs(domWindow));
+          if (domWindow) {
+            nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(domWindow);
+            if(privateDOMWindow) {
+              nsCOMPtr<nsIFocusController> focusController;
+              privateDOMWindow->GetRootFocusController(getter_AddRefs(focusController));
+              if (focusController)
+                focusController->RewindFocusState();
+            }
+          }
+        }
+#endif
         break;
       }
       case NS_OS_TOOLBAR: {
@@ -488,9 +510,6 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
 #ifdef DEBUG_saari
         printf("nsWebShellWindow::NS_ACTIVATE\n");
 #endif
-// Sucky platform specific code to get around event dispatch ordering
-#if defined(WIN32) || defined(XP_OS2) || defined (MOZ_WIDGET_GTK) || defined (MOZ_WIDGET_XLIB)
-
         nsCOMPtr<nsIDOMWindowInternal> domWindow;
         eventWindow->ConvertWebShellToDOMWindow(webShell, getter_AddRefs(domWindow));
         /*
@@ -513,7 +532,6 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
           if(privateDOMWindow)
             privateDOMWindow->Activate();
         }
-#endif /* defined(WIN32) || defined(XP_OS2) || defined (MOZ_WIDGET_GTK) || defined (MOZ_WIDGET_XLIB) */
         break;
 
       }
@@ -542,8 +560,13 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
         else */
         if (domWindow) {
           nsCOMPtr<nsPIDOMWindow> privateDOMWindow = do_QueryInterface(domWindow);
-          if(privateDOMWindow)
+          if(privateDOMWindow) {
+            nsCOMPtr<nsIFocusController> focusController;
+            privateDOMWindow->GetRootFocusController(getter_AddRefs(focusController));
+            if (focusController)
+              focusController->SetActive(PR_FALSE);
             privateDOMWindow->Deactivate();
+          }
         }
         break;
       }
@@ -587,7 +610,7 @@ nsWebShellWindow::HandleEvent(nsGUIEvent *aEvent)
             // since the window has been activated, replace persistent size data
             // with the newly activated window's
             if (eventWindow->mChromeLoaded)
-              eventWindow->StoreBoundsToXUL(PR_TRUE, PR_TRUE, PR_TRUE);
+              eventWindow->PersistPositionAndSize(PR_TRUE, PR_TRUE, PR_TRUE);
 
             break;
           }
@@ -1050,22 +1073,6 @@ void nsWebShellWindow::LoadMenus(nsIDOMDocument * aDOMDoc, nsIWidget * aParentWi
 } // nsWebShellWindow::LoadMenus
 #endif
 
-//------------------------------------------------------------------------------
-NS_IMETHODIMP
-nsWebShellWindow::GetContentShellById(const nsString& aID, nsIWebShell** aChildShell)
-{
-	// Set to null just to be certain
-   *aChildShell = nsnull;
-
-   nsCOMPtr<nsIDocShellTreeItem> content;
-
-   nsXULWindow::GetContentShellById(aID.get(), getter_AddRefs(content));
-   if(!content)
-      return NS_ERROR_FAILURE;
-   CallQueryInterface(content, aChildShell);
-
-   return NS_OK;
-}
 
 //------------------------------------------------------------------------------
 NS_IMETHODIMP
@@ -1101,17 +1108,6 @@ nsWebShellWindow::ShowModal()
   return nsXULWindow::ShowModal();
 }
 
-
-// yes, this one's name and ShowModal are a confusing pair. plan is to merge
-// the two someday.
-NS_IMETHODIMP
-nsWebShellWindow::ShowModally(PRBool aPrepare)
-{ 
-   NS_ERROR("Can't use this anymore");
-   return NS_ERROR_FAILURE;
-}
-
-
 /* return the main, outermost webshell in this window */
 NS_IMETHODIMP 
 nsWebShellWindow::GetWebShell(nsIWebShell *& aWebShell)
@@ -1175,13 +1171,14 @@ nsWebShellWindow::DestroyModalDialogEvent(PLEvent *aEvent)
 }
 
 void
-nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition)
+nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition, PRBool aMode)
 {
   PR_Lock(mSPTimerLock);
   if (mSPTimer) {
     mSPTimer->SetDelay(SIZE_PERSISTENCE_TIMEOUT);
     mSPTimerSize |= aSize;
     mSPTimerPosition |= aPosition;
+    mSPTimerMode |= aMode;
   } else {
     nsresult rv;
     mSPTimer = do_CreateInstance("@mozilla.org/timer;1", &rv);
@@ -1190,6 +1187,7 @@ nsWebShellWindow::SetPersistenceTimer(PRBool aSize, PRBool aPosition)
                      SIZE_PERSISTENCE_TIMEOUT, NS_TYPE_ONE_SHOT);
       mSPTimerSize = aSize;
       mSPTimerPosition = aPosition;
+      mSPTimerMode = aMode;
     }
   }
   PR_Unlock(mSPTimerLock);
@@ -1200,9 +1198,12 @@ nsWebShellWindow::FirePersistenceTimer(nsITimer *aTimer, void *aClosure)
 {
   nsWebShellWindow *win = NS_STATIC_CAST(nsWebShellWindow *, aClosure);
   PR_Lock(win->mSPTimerLock);
-  win->mSPTimer = nsnull;
+  win->PersistPositionAndSize(win->mSPTimerPosition, win->mSPTimerSize,
+                              win->mSPTimerMode);
+  win->mSPTimerSize = PR_FALSE;
+  win->mSPTimerPosition = PR_FALSE;
+  win->mSPTimerMode = PR_FALSE;
   PR_Unlock(win->mSPTimerLock);
-  win->StoreBoundsToXUL(win->mSPTimerPosition, win->mSPTimerSize, PR_FALSE);
 }
 
 
@@ -1217,13 +1218,14 @@ nsWebShellWindow::OnProgressChange(nsIWebProgress *aProgress,
                                    PRInt32 aCurTotalProgress,
                                    PRInt32 aMaxTotalProgress)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsWebShellWindow::OnStateChange(nsIWebProgress *aProgress,
                                 nsIRequest *aRequest,
-                                PRInt32 aStateFlags,
+                                PRUint32 aStateFlags,
                                 nsresult aStatus)
 {
   // If the notification is not about a document finishing, then just
@@ -1310,6 +1312,7 @@ nsWebShellWindow::OnLocationChange(nsIWebProgress *aProgress,
                                    nsIRequest *aRequest,
                                    nsIURI *aURI)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -1319,14 +1322,16 @@ nsWebShellWindow::OnStatusChange(nsIWebProgress* aWebProgress,
                                  nsresult aStatus,
                                  const PRUnichar* aMessage)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsWebShellWindow::OnSecurityChange(nsIWebProgress *aWebProgress,
                                    nsIRequest *aRequest,
-                                   PRInt32 state)
+                                   PRUint32 state)
 {
+  NS_NOTREACHED("notification excluded in AddProgressListener(...)");
   return NS_OK;
 }
 
@@ -1401,12 +1406,6 @@ nsCOMPtr<nsIDOMDocument> nsWebShellWindow::GetNamedDOMDoc(const nsAString & aWeb
 
 //----------------------------------------
 
-/* copy the window's size and position to the window tag */
-void nsWebShellWindow::StoreBoundsToXUL(PRBool aPosition, PRBool aSize, PRBool aSizeMode)
-{
-   PersistPositionAndSize(aPosition, aSize, aSizeMode);
-} // StoreBoundsToXUL
-
 // if the main document URL specified URLs for any content areas, start them loading
 void nsWebShellWindow::LoadContentAreas() {
 
@@ -1426,12 +1425,11 @@ void nsWebShellWindow::LoadContentAreas() {
       nsCOMPtr<nsIURI> mainURL;
       doc->GetDocumentURL(getter_AddRefs(mainURL));
       if (mainURL) {
-        char *search = nsnull;
+        nsCAutoString search;
         nsCOMPtr<nsIURL> url = do_QueryInterface(mainURL);
         if (url)
-          url->GetQuery(&search);
-        searchSpec.AssignWithConversion(search);
-        nsCRT::free(search);
+          url->GetQuery(search);
+        searchSpec = NS_ConvertUTF8toUCS2(search);
       }
     }
   }
@@ -1445,16 +1443,15 @@ void nsWebShellWindow::LoadContentAreas() {
     nsString    contentAreaID,
                 contentURL;
     char        *urlChar;
-    nsIWebShell *contentShell;
     nsresult rv;
     for (endPos = 0; endPos < (PRInt32)searchSpec.Length(); ) {
       // extract contentAreaID and URL substrings
       begPos = endPos;
-      eqPos = searchSpec.FindChar('=', PR_FALSE,begPos);
+      eqPos = searchSpec.FindChar('=', begPos);
       if (eqPos < 0)
         break;
 
-      endPos = searchSpec.FindChar(';', PR_FALSE,eqPos);
+      endPos = searchSpec.FindChar(';', eqPos);
       if (endPos < 0)
         endPos = searchSpec.Length();
       searchSpec.Mid(contentAreaID, begPos, eqPos-begPos);
@@ -1462,21 +1459,23 @@ void nsWebShellWindow::LoadContentAreas() {
       endPos++;
 
       // see if we have a webshell with a matching contentAreaID
-      rv = GetContentShellById(contentAreaID, &contentShell);
-      if (NS_SUCCEEDED(rv)) {
-        urlChar = ToNewCString(contentURL);
-        if (urlChar) {
-          nsUnescape(urlChar);
-          contentURL.AssignWithConversion(urlChar);
-          nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(contentShell));
-          webNav->LoadURI(contentURL.get(),
+      nsCOMPtr<nsIDocShellTreeItem> content;
+      rv = GetContentShellById(contentAreaID.get(), getter_AddRefs(content));
+      if (NS_SUCCEEDED(rv) && content) {
+        nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(content));
+        if (webNav) {
+          urlChar = ToNewCString(contentURL);
+          if (urlChar) {
+            nsUnescape(urlChar);
+            contentURL.AssignWithConversion(urlChar);
+            webNav->LoadURI(contentURL.get(),
                           nsIWebNavigation::LOAD_FLAGS_NONE,
                           nsnull,
                           nsnull,
                           nsnull);
-          delete [] urlChar;
+            nsMemory::Free(urlChar);
+          }
         }
-        NS_RELEASE(contentShell);
       }
     }
   }
@@ -1577,7 +1576,8 @@ nsWebShellWindow::ContentChanged(nsIDocument *aDocument,
 NS_IMETHODIMP
 nsWebShellWindow::ContentStatesChanged(nsIDocument *aDocument,
                                        nsIContent* aContent1,
-                                       nsIContent* aContent2)
+                                       nsIContent* aContent2,
+                                       PRInt32 aStateMask)
 {
   return NS_OK;
 }
@@ -1758,7 +1758,7 @@ NS_IMETHODIMP nsWebShellWindow::Destroy()
   if (mSPTimer) {
     mSPTimer->Cancel();
     mSPTimer = nsnull;
-    StoreBoundsToXUL(mSPTimerPosition, mSPTimerSize, PR_FALSE);
+    PersistPositionAndSize(mSPTimerPosition, mSPTimerSize, mSPTimerMode);
   }
   PR_Unlock(mSPTimerLock);
 
