@@ -42,7 +42,6 @@
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "nsVoidArray.h"
-#include "rdf.h"
 #include "nsIBaseWindow.h"
 #include "nsIWidget.h"
 #include "nsIDOMWindow.h"
@@ -51,18 +50,12 @@
 #include "nsIDocumentViewer.h"
 #include "nsIDocument.h"
 #include "nsIDOMDocument.h"
-#include "nsIRDFContainer.h"
-#include "nsIRDFContainerUtils.h"
-#include "nsIRDFDataSource.h"
-#include "nsIRDFNode.h"
-#include "nsIRDFObserver.h"
-#include "nsIRDFService.h"
 #include "nsIServiceManager.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIWebShell.h"
 #include "nsAppShellWindowEnumerator.h"
-#include "nsRDFCID.h"
 #include "nsWindowMediator.h"
+#include "nsIWindowMediatorListener.h"
 #include "nsXPIDLString.h"
 
 // Interfaces Needed
@@ -71,128 +64,50 @@
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIXULWindow.h"
 
-static NS_DEFINE_CID(kRDFInMemoryDataSourceCID, NS_RDFINMEMORYDATASOURCE_CID);
-static NS_DEFINE_CID(kRDFServiceCID,            NS_RDFSERVICE_CID);
-static NS_DEFINE_CID(kRDFContainerUtilsCID,     NS_RDFCONTAINERUTILS_CID);
-
-static NS_DEFINE_CID(kRDFContainerCID,                    NS_RDFCONTAINER_CID);
-static const char kURINC_WindowMediatorRoot[] = "NC:WindowMediatorRoot";
-
-
-DEFINE_RDF_VOCAB(NC_NAMESPACE_URI, NC, Name);
-DEFINE_RDF_VOCAB(NC_NAMESPACE_URI, NC, URL);
-DEFINE_RDF_VOCAB(NC_NAMESPACE_URI, NC, KeyIndex);
-static	nsIRDFService* gRDFService = nsnull;
 
 static nsresult GetDOMWindow( nsIXULWindow* inWindow,
                   nsCOMPtr< nsIDOMWindowInternal>& outDOMWindow);
-static nsCOMPtr<nsIDOMNode> GetDOMNodeFromDocShell(nsIDocShell *aShell);
-static void GetAttribute( nsIXULWindow* inWindow,
-                          const nsAString& inAttribute,
-                          nsAString& outValue);
-static void GetWindowType( nsIXULWindow* inWindow, nsAutoString& outType);
 static inline PRUint32 GetWindowZ( nsIXULWindow *inWindow);
 
-static nsresult NS_NewMediatorListRDFContainer(nsIRDFDataSource* aDataSource,
-                   nsIRDFResource* aResource,
-                   nsIRDFContainer** aResult)
+static PRBool notifyOpenWindow(nsISupports *aElement, void* aData);
+static PRBool notifyCloseWindow(nsISupports *aElement, void* aData);
+static PRBool notifyWindowTitleChange(nsISupports *aElement, void* aData);
+
+// for notifyWindowTitleChange
+struct windowData {
+  nsIXULWindow* mWindow;
+  const PRUnichar *mTitle;
+};
+
+
+nsresult
+GetDOMWindow( nsIXULWindow* inWindow, nsCOMPtr< nsIDOMWindowInternal>& outDOMWindow)
 {
-    nsresult rv;
-  
-    rv = nsComponentManager::CreateInstance( 
-                                        kRDFContainerCID, NULL, NS_GET_IID(nsIRDFContainer),  (void**)aResult);
-    if (NS_FAILED(rv))
-    	return rv;
+  nsCOMPtr<nsIDocShell> docShell;
 
-    rv = (*aResult)->Init(aDataSource, aResource);
-    if (NS_FAILED(rv))
-    {
-    	NS_RELEASE(*aResult);
-    }
-    return rv;
-}
-
-nsresult GetDOMWindow( nsIXULWindow* inWindow, nsCOMPtr< nsIDOMWindowInternal>& outDOMWindow)
-{
-	nsCOMPtr<nsIDocShell> docShell;
-
-	inWindow->GetDocShell(getter_AddRefs(docShell));
+  inWindow->GetDocShell(getter_AddRefs(docShell));
    outDOMWindow = do_GetInterface(docShell);
    return outDOMWindow ? NS_OK : NS_ERROR_FAILURE;
 }
 
 
-nsCOMPtr<nsIDOMNode> GetDOMNodeFromDocShell(nsIDocShell *aShell)
-{
-  nsCOMPtr<nsIDOMNode> node;
-
-  nsCOMPtr<nsIContentViewer> cv;
-  aShell->GetContentViewer(getter_AddRefs(cv));
-  if (cv) {
-    nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
-    if (docv) {
-      nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
-      if (doc) {
-        nsCOMPtr<nsIDOMDocument> domdoc(do_QueryInterface(doc));
-        if (domdoc) {
-          nsCOMPtr<nsIDOMElement> element;
-          domdoc->GetDocumentElement(getter_AddRefs(element));
-          if (element)
-            node = do_QueryInterface(element);
-        }
-      }
-    }
-  }
-
-  return node;
-}
-
-
-void GetAttribute( nsIXULWindow* inWindow,
-                   const nsAString& inAttribute, nsAString& outValue)
-{
-  nsCOMPtr<nsIDocShell> shell;
-  if ( inWindow &&
-  		NS_SUCCEEDED(inWindow->GetDocShell(getter_AddRefs(shell))))
-  {
-  		nsCOMPtr<nsIDOMNode> node( GetDOMNodeFromDocShell(shell));
- 		
-	  if (node)
-	  {
-	  	nsCOMPtr<nsIDOMElement> webshellElement( do_QueryInterface(node));
-  		if ( webshellElement.get())
-  			webshellElement->GetAttribute(nsAutoString(inAttribute), outValue);
-  	  }
-  }
-}
-
-void GetWindowType(nsIXULWindow* inWindow, nsAString& outType)
-{
- 	GetAttribute(inWindow, NS_LITERAL_STRING("windowtype"), outType);
-}
-
 /* return an integer corresponding to the relative z order of the window.
    should probably be an explicit read-only method on nsIXULWindow */
-PRUint32 GetWindowZ(nsIXULWindow *inWindow)
+PRUint32
+GetWindowZ(nsIXULWindow *inWindow)
 {
-	PRUint32 order;
-	inWindow->GetZlevel(&order);
-	return order;
+  PRUint32 order;
+  inWindow->GetZlevel(&order);
+  return order;
 }
 
-nsIRDFResource	*nsWindowMediator::kNC_WindowMediatorRoot = NULL;
-nsIRDFResource	*nsWindowMediator::kNC_Name = NULL;
-nsIRDFResource	*nsWindowMediator::kNC_URL = NULL;
-nsIRDFResource	*nsWindowMediator::kNC_KeyIndex = NULL;
 
-PRInt32		nsWindowMediator::gRefCnt;
-nsIRDFDataSource *nsWindowMediator::mInner = NULL;
 
+PRInt32   nsWindowMediator::gRefCnt = 0;
 
 nsWindowMediator::nsWindowMediator() :
-	mEnumeratorList(), mOldestWindow(0), mTopmostWindow(0),
-	mTimeStamp(0), mListLock(0)
+  mEnumeratorList(), mOldestWindow(0), mTopmostWindow(0),
+  mTimeStamp(0), mUpdateBatchNest(0), mListLock(0)
 {
    NS_INIT_REFCNT();
 
@@ -210,17 +125,6 @@ nsWindowMediator::~nsWindowMediator()
     while (mOldestWindow)
       UnregisterWindow(mOldestWindow);
 
-    // unregister this from the RDF service
-    if (gRDFService) {
-      gRDFService->UnregisterDataSource(this);
-      nsServiceManager::ReleaseService(kRDFServiceCID, gRDFService);
-      gRDFService = nsnull;
-    }
-    NS_IF_RELEASE(kNC_WindowMediatorRoot);
-    NS_IF_RELEASE(kNC_Name);
-    NS_IF_RELEASE(kNC_URL);
-    NS_IF_RELEASE(kNC_KeyIndex);
-    NS_IF_RELEASE(mInner);
     if (mListLock)
       PR_DestroyLock(mListLock);
   }
@@ -230,32 +134,31 @@ nsWindowMediator::~nsWindowMediator()
 
 NS_IMETHODIMP nsWindowMediator::RegisterWindow(nsIXULWindow* inWindow)
 {
-	if (inWindow == NULL)
-		return NS_ERROR_INVALID_ARG;
-	
-	mTimeStamp++;
+  mTimeStamp++;
 
-	// Create window info struct and add to list of windows
-	nsWindowInfo* windowInfo = new nsWindowInfo (inWindow, mTimeStamp);
-	if (windowInfo == NULL)
-		return NS_ERROR_OUT_OF_MEMORY;
+  // Create window info struct and add to list of windows
+  nsWindowInfo* windowInfo = new nsWindowInfo (inWindow, mTimeStamp);
+  if (windowInfo == NULL)
+    return NS_ERROR_OUT_OF_MEMORY;
 
-	nsresult returnValue = AddWindowToRDF(windowInfo);
-	if (NS_FAILED(returnValue)) {
- 		delete windowInfo;
-		return returnValue;
-	}
+  if (mListeners) {
+    windowData winData = { inWindow, nsnull };
+    mListeners->EnumerateForwards(notifyOpenWindow, (void*)&winData);
+  }
+  
+  nsAutoLock lock(mListLock);
+  if (mOldestWindow)
+    windowInfo->InsertAfter(mOldestWindow->mOlder, 0);
+  else
+    mOldestWindow = windowInfo;
 
-	nsAutoLock lock(mListLock);
-	if (mOldestWindow)
-		windowInfo->InsertAfter(mOldestWindow->mOlder, 0);
-	else
-		mOldestWindow = windowInfo;
-
-	return NS_OK;
+  return NS_OK;
 }
 
-NS_IMETHODIMP nsWindowMediator::UnregisterWindow(nsIXULWindow* inWindow)
+
+
+NS_IMETHODIMP
+nsWindowMediator::UnregisterWindow(nsIXULWindow* inWindow)
 {
   // Find Window info
   nsWindowInfo *info,
@@ -275,19 +178,20 @@ NS_IMETHODIMP nsWindowMediator::UnregisterWindow(nsIXULWindow* inWindow)
 }
 
 
-NS_IMETHODIMP nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
+
+NS_IMETHODIMP
+nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
 {
   // Inform the iterators
   PRInt32 index = -1;
   while (++index < mEnumeratorList.Count()) 
     ((nsAppShellWindowEnumerator*)mEnumeratorList[index])->WindowRemoved(inInfo);
+  
+  if (mListeners) {
+    windowData winData = {inInfo->mWindow.get(), nsnull };
+    mListeners->EnumerateForwards(notifyCloseWindow, (void*)&winData);
+  }
 
-  // Remove From RDF
-  nsCOMPtr<nsIRDFContainer> container;
-  nsresult rv = NS_NewMediatorListRDFContainer(mInner, kNC_WindowMediatorRoot, getter_AddRefs(container));
-  if (NS_SUCCEEDED(rv))
-    container->RemoveElement(inInfo->mRDFID, PR_TRUE);
- 	
   // Remove from the lists and free up 
   if (inInfo == mOldestWindow)
     mOldestWindow = inInfo->mYounger;
@@ -298,7 +202,7 @@ NS_IMETHODIMP nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
     mOldestWindow = 0;
   if (inInfo == mTopmostWindow)
     mTopmostWindow = 0;
-  delete inInfo;	
+  delete inInfo;  
 
   // inform WindowWatcher
   nsWindowInfo *info = MostRecentWindowInfo(0);
@@ -316,7 +220,9 @@ NS_IMETHODIMP nsWindowMediator::UnregisterWindow(nsWindowInfo *inInfo)
 }
 
 
-NS_METHOD nsWindowMediator::GetEnumerator( const PRUnichar* inType, nsISimpleEnumerator** outEnumerator)
+
+NS_METHOD
+nsWindowMediator::GetEnumerator( const PRUnichar* inType, nsISimpleEnumerator** outEnumerator)
 {
   if (outEnumerator == NULL)
     return NS_ERROR_INVALID_ARG;
@@ -328,8 +234,11 @@ NS_METHOD nsWindowMediator::GetEnumerator( const PRUnichar* inType, nsISimpleEnu
 
   return NS_ERROR_OUT_OF_MEMORY;
 }
-	
-NS_METHOD nsWindowMediator::GetXULWindowEnumerator(const PRUnichar* inType, nsISimpleEnumerator** outEnumerator)
+
+
+
+NS_METHOD
+nsWindowMediator::GetXULWindowEnumerator(const PRUnichar* inType, nsISimpleEnumerator** outEnumerator)
 {
   if (outEnumerator == NULL)
     return NS_ERROR_INVALID_ARG;
@@ -340,9 +249,12 @@ NS_METHOD nsWindowMediator::GetXULWindowEnumerator(const PRUnichar* inType, nsIS
     return enumerator->QueryInterface( NS_GET_IID(nsISimpleEnumerator) , (void**)outEnumerator);
 
   return NS_ERROR_OUT_OF_MEMORY;
-}	
+} 
 
-NS_METHOD nsWindowMediator::GetZOrderDOMWindowEnumerator(
+
+
+NS_METHOD
+nsWindowMediator::GetZOrderDOMWindowEnumerator(
             const PRUnichar *aWindowType, PRBool aFrontToBack,
             nsISimpleEnumerator **_retval)
 {
@@ -361,7 +273,10 @@ NS_METHOD nsWindowMediator::GetZOrderDOMWindowEnumerator(
   return NS_ERROR_OUT_OF_MEMORY;
 }
 
-NS_METHOD nsWindowMediator::GetZOrderXULWindowEnumerator(
+
+
+NS_METHOD
+nsWindowMediator::GetZOrderXULWindowEnumerator(
             const PRUnichar *aWindowType, PRBool aFrontToBack,
             nsISimpleEnumerator **_retval)
 {
@@ -380,22 +295,31 @@ NS_METHOD nsWindowMediator::GetZOrderXULWindowEnumerator(
   return NS_ERROR_OUT_OF_MEMORY;
 }
 
-PRInt32 nsWindowMediator::AddEnumerator(nsAppShellWindowEnumerator * inEnumerator)
+
+
+PRInt32
+nsWindowMediator::AddEnumerator(nsAppShellWindowEnumerator * inEnumerator)
 {
   return mEnumeratorList.AppendElement(inEnumerator);
 }
 
-PRInt32 nsWindowMediator::RemoveEnumerator( nsAppShellWindowEnumerator * inEnumerator)
+
+
+PRInt32
+nsWindowMediator::RemoveEnumerator( nsAppShellWindowEnumerator * inEnumerator)
 {
   return mEnumeratorList.RemoveElement(inEnumerator);
 }
 
-/*
-	Returns the window of type inType ( if null return any window type ) which has the most recent
-	time stamp
-*/
-NS_IMETHODIMP nsWindowMediator::GetMostRecentWindow( const PRUnichar* inType, nsIDOMWindowInternal** outWindow) {
 
+
+/*
+  Returns the window of type inType ( if null return any window type ) which has the most recent
+  time stamp
+*/
+NS_IMETHODIMP
+nsWindowMediator::GetMostRecentWindow( const PRUnichar* inType, nsIDOMWindowInternal** outWindow)
+{
   NS_ENSURE_ARG_POINTER(outWindow);
   *outWindow = NULL;
 
@@ -407,7 +331,7 @@ NS_IMETHODIMP nsWindowMediator::GetMostRecentWindow( const PRUnichar* inType, ns
 
   if (info && info->mWindow) {
     nsCOMPtr <nsIDOMWindowInternal> DOMWindow;
-    if(NS_SUCCEEDED(GetDOMWindow(info->mWindow, DOMWindow))) {	
+    if(NS_SUCCEEDED(GetDOMWindow(info->mWindow, DOMWindow))) {  
       *outWindow = DOMWindow;
       NS_ADDREF(*outWindow);
       return NS_OK;
@@ -419,6 +343,7 @@ NS_IMETHODIMP nsWindowMediator::GetMostRecentWindow( const PRUnichar* inType, ns
 }
 
 
+
 nsWindowInfo *
 nsWindowMediator::MostRecentWindowInfo(const PRUnichar* inType)
 {
@@ -448,7 +373,9 @@ nsWindowMediator::MostRecentWindowInfo(const PRUnichar* inType)
 }
 
 
-NS_IMETHODIMP nsWindowMediator::UpdateWindowTimeStamp( nsIXULWindow* inWindow)
+
+NS_IMETHODIMP
+nsWindowMediator::UpdateWindowTimeStamp( nsIXULWindow* inWindow)
 {
   nsWindowInfo *info,
                *listEnd;
@@ -478,11 +405,13 @@ NS_IMETHODIMP nsWindowMediator::UpdateWindowTimeStamp( nsIXULWindow* inWindow)
 }
 
 
-NS_IMETHODIMP  nsWindowMediator::UpdateWindowTitle( nsIXULWindow* inWindow, const PRUnichar* inTitle )
+
+NS_IMETHODIMP
+nsWindowMediator::UpdateWindowTitle( nsIXULWindow* inWindow,
+                                     const PRUnichar* inTitle )
 {
   nsWindowInfo *info,
                *listEnd;
-  nsresult     rv;
 
   nsAutoLock lock(mListLock);
   for (info = mOldestWindow, listEnd = 0;
@@ -492,49 +421,25 @@ NS_IMETHODIMP  nsWindowMediator::UpdateWindowTitle( nsIXULWindow* inWindow, cons
     if (info->mWindow.get() != inWindow)
       continue;
 
-    nsIRDFResource* window = info->mRDFID;
-    // Get rid of the old value
-    nsIRDFNode* target = NULL;
-    if ( NS_SUCCEEDED(mInner->GetTarget( window, kNC_Name, PR_TRUE, &target) ) && (target != nsnull) )
-    {
-      mInner->Unassert( window, kNC_Name, target );
-      NS_IF_RELEASE( target );
-    }
+	if (mListeners) {
+      windowData winData = { inWindow, inTitle };
+	  mListeners->EnumerateForwards(notifyWindowTitleChange, (void *)&winData);
+	}
 
-    // Add new title             
-    nsCOMPtr<nsIRDFLiteral> newTitle;
-    if ( gRDFService && NS_FAILED(rv = gRDFService->GetLiteral( inTitle, getter_AddRefs(newTitle) ) ) )
-    {
-      NS_ERROR("unable to create literal for window name");
-      return rv;
-    }
-
-    // Should this title be displayed
-    PRBool display = PR_TRUE;
-    nsAutoString displayString;
-    GetAttribute( inWindow, NS_LITERAL_STRING("intaskslist"), displayString );
-    ToLowerCase(displayString);
-
-    if ( displayString.Equals(NS_LITERAL_STRING("false")) )
-      display=PR_FALSE;
-
-    rv = Assert( window , kNC_Name, newTitle, display );
-    if (rv != NS_RDF_ASSERTION_ACCEPTED)
-    {
-      NS_ERROR("unable to set window name");
-    }
-    return NS_OK;
   }
   return NS_ERROR_FAILURE;
 }
 
+
+
 /* This method's plan is to intervene only when absolutely necessary.
    We will get requests to place our windows behind unknown windows.
    For the most part, we need to leave those alone (turning them into
    explicit requests to be on top breaks Windows.) So generally we
    calculate a change as seldom as possible.
 */
-NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
+NS_IMETHODIMP
+nsWindowMediator::CalculateZPosition(
                 nsIXULWindow   *inWindow,
                 PRUint32        inPosition,
                 nsIWidget      *inBelow,
@@ -557,9 +462,9 @@ NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
 
     return NS_ERROR_INVALID_ARG;
 
-  nsWindowInfo *info;
+  nsWindowInfo *info = mTopmostWindow;
   nsIXULWindow *belowWindow = 0;
-  PRBool        found;
+  PRBool        found = PR_FALSE;
   nsresult      result = NS_OK;
   
   PRUint32      inZ;
@@ -573,8 +478,6 @@ NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
   if (inPosition == nsIWindowMediator::zLevelBelow) {
 
     // locate inBelow. it had better be in the z-list if it's a valid window.
-    info = mTopmostWindow;
-    found = PR_FALSE;
     if (inBelow && info)
       do {
         nsCOMPtr<nsIWidget> scanWidget;
@@ -603,7 +506,6 @@ NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
     if (mTopmostWindow && GetWindowZ(mTopmostWindow->mWindow) > inZ) {
 
       // asked for topmost, can't have it. locate highest allowed position.
-      info = mTopmostWindow;
       do {
         if (GetWindowZ(info->mWindow) <= inZ)
           break;
@@ -619,7 +521,6 @@ NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
     if (mTopmostWindow && GetWindowZ(mTopmostWindow->mHigher->mWindow) < inZ) {
 
       // asked for bottommost, can't have it. locate lowest allowed position.
-      info = mTopmostWindow;
       do {
         info = info->mHigher;
         if (GetWindowZ(info->mWindow) >= inZ)
@@ -682,7 +583,10 @@ NS_IMETHODIMP nsWindowMediator::CalculateZPosition(
   return result;
 }
 
-NS_IMETHODIMP nsWindowMediator::SetZPosition(
+
+
+NS_IMETHODIMP
+nsWindowMediator::SetZPosition(
                 nsIXULWindow *inWindow,
                 PRUint32      inPosition,
                 nsIXULWindow *inBelow) {
@@ -749,255 +653,88 @@ NS_IMETHODIMP nsWindowMediator::SetZPosition(
   return NS_OK;
 }
 
-NS_IMETHODIMP  nsWindowMediator::GetWindowForResource( const PRUnichar* inResource, nsIDOMWindowInternal** outWindow )
-{
-	if ( outWindow == NULL )
-		return NS_ERROR_INVALID_ARG;
-	*outWindow = NULL;
-
-	// Find the window
-	//nsresult result = NS_ERROR_FAILURE;
-	nsAutoString temp( inResource );
-	char* resourceString = ToNewCString(temp);
-	nsWindowInfo *info,
-	             *listEnd;
-
-	nsAutoLock lock(mListLock);
-	info = mOldestWindow;
-	listEnd = 0;
-	while (info != listEnd)
-	{
-		PRBool isMatch = PR_FALSE;
-
-		if ( NS_SUCCEEDED( info->mRDFID->EqualsString( resourceString, &isMatch) ) && isMatch ) 
-		{
-			nsCOMPtr <nsIDOMWindowInternal> DOMWindow;
-			if( NS_SUCCEEDED ( GetDOMWindow( info->mWindow, DOMWindow  ) ) )
-			{	
-        *outWindow = DOMWindow;
-        NS_ADDREF(*outWindow);
-			}
-			break;
-		}
-
-		info = info->mYounger;
-		listEnd = mOldestWindow;
-	}
-
-	delete resourceString;
-	// Rather than returning an error when you pass in crap, return NS_OK
-	// Doing this to avoid throwing a JS exception becasue event bubbling is not working as of 7/26/99
-	//return result;
-	return NS_OK;
-}
-
-NS_IMETHODIMP nsWindowMediator::ConvertISupportsToDOMWindow( nsISupports* inInterface, nsIDOMWindowInternal** outWindow )
-{
-   return inInterface->QueryInterface(NS_GET_IID(nsIDOMWindowInternal)  , (void**) outWindow );
-} 
-
-
 // COM
 NS_IMPL_ADDREF( nsWindowMediator );
-NS_IMPL_RELEASE( nsWindowMediator );
-NS_IMPL_QUERY_INTERFACE2(nsWindowMediator, nsIWindowMediator, nsIRDFDataSource)
+NS_IMPL_QUERY_INTERFACE1(nsWindowMediator, nsIWindowMediator);
+NS_IMPL_RELEASE(nsWindowMediator);
 
-// RDF
 nsresult
 nsWindowMediator::Init()
 {
   nsresult rv;
-
-  if (gRefCnt++ == 0) {
-
+  
+  if (gRefCnt++ == 0)
+  {
     mListLock = PR_NewLock();
     if (!mListLock)
-      return NS_ERROR_OUT_OF_MEMORY;
+        return NS_ERROR_OUT_OF_MEMORY;
+  }
 
-    rv = nsServiceManager::GetService( kRDFServiceCID, NS_GET_IID(nsIRDFService), (nsISupports**) &gRDFService );
-    if (NS_FAILED(rv)) return rv;
-
-      gRDFService->GetResource( kURINC_WindowMediatorRoot,   &kNC_WindowMediatorRoot );
-      gRDFService->GetResource( kURINC_Name, &kNC_Name );
-      gRDFService->GetResource( kURINC_URL, &kNC_URL );
-      gRDFService->GetResource( kURINC_KeyIndex, &kNC_KeyIndex );
-    }
-
-    if (NS_FAILED(rv = nsComponentManager::CreateInstance(kRDFInMemoryDataSourceCID,
-                                                        nsnull,
-                                                        NS_GET_IID(nsIRDFDataSource),
-                                                        (void**) &mInner)))
-      return rv;
-
-    nsCOMPtr<nsIRDFContainerUtils> rdfc = 
-             do_GetService(kRDFContainerUtilsCID, &rv);
-    if (NS_FAILED(rv))
-      return rv;
-
-    rv = rdfc->MakeSeq(mInner, kNC_WindowMediatorRoot, NULL );
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Unable to make NC:WindowMediatorRoot a sequence");
-    if (NS_FAILED(rv))
-      return rv;
-    if ( gRDFService == NULL )
-      return NS_ERROR_NULL_POINTER;
-
-    mWatcher = do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv);
-    if (NS_FAILED(rv))
-      return rv;
-
-    // register this as a named data source with the RDF service
-    return gRDFService->RegisterDataSource(this, PR_FALSE);
-}
-
-NS_IMETHODIMP nsWindowMediator::GetTarget(nsIRDFResource* source,
-                                          nsIRDFResource* property,
-                                          PRBool tv,
-                                          nsIRDFNode** target)
-{
-    if (property == kNC_KeyIndex)
-    {
-        nsCOMPtr<nsIRDFContainer> container;
-        nsresult rv = NS_NewMediatorListRDFContainer(mInner,
-            kNC_WindowMediatorRoot, getter_AddRefs(container));
-        if (NS_FAILED(rv)) return(rv);
-        if (!container) return(NS_ERROR_FAILURE);
-
-        PRInt32   theIndex = 0;
-        rv = container->IndexOf(source, &theIndex);
-        if (NS_FAILED(rv)) return(rv);
-
-        // only allow the range of 1 to 9 for single key access        
-        if (theIndex < 1 || theIndex > 9) return(NS_RDF_NO_VALUE);
-
-        nsCOMPtr<nsIRDFInt> indexInt;
-        rv = gRDFService->GetIntLiteral(theIndex, getter_AddRefs(indexInt));
-        if (NS_FAILED(rv)) return(rv);
-        if (!indexInt) return(NS_ERROR_FAILURE);
-
-        return(CallQueryInterface(indexInt, target));
-    }
-    return mInner->GetTarget(source, property, tv, target);
-}
-
-NS_IMETHODIMP nsWindowMediator::Assert(nsIRDFResource* aSource,
-                               nsIRDFResource* aProperty,
-                               nsIRDFNode* aTarget,
-                               PRBool aTruthValue)
-{
-    // XXX TODO: filter out asserts we don't care about
-    return mInner->Assert(aSource, aProperty, aTarget, aTruthValue);
-}
-
-NS_IMETHODIMP nsWindowMediator::Unassert(nsIRDFResource* aSource,
-                                 nsIRDFResource* aProperty,
-                                 nsIRDFNode* aTarget)
-{
-    // XXX TODO: filter out unasserts we don't care about
-    return mInner->Unassert(aSource, aProperty, aTarget);
-}
-
-
-NS_IMETHODIMP nsWindowMediator::Change(nsIRDFResource* aSource,
-                                       nsIRDFResource* aProperty,
-                                       nsIRDFNode* aOldTarget,
-                                       nsIRDFNode* aNewTarget)
-{
-  // XXX TODO: filter out changes we don't care about
-  return mInner->Change(aSource, aProperty, aOldTarget, aNewTarget);
-}
-
-NS_IMETHODIMP nsWindowMediator::Move(nsIRDFResource* aOldSource,
-                                     nsIRDFResource* aNewSource,
-                                     nsIRDFResource* aProperty,
-                                     nsIRDFNode* aTarget)
-{
-  // XXX TODO: filter out changes we don't care about
-  return mInner->Move(aOldSource, aNewSource, aProperty, aTarget);
-}
-
-
-NS_IMETHODIMP nsWindowMediator::GetAllCommands(nsIRDFResource* source,
-                                       nsIEnumerator/*<nsIRDFResource>*/** commands)
-{
-    NS_NOTYETIMPLEMENTED("write me!");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsWindowMediator::GetAllCmds(nsIRDFResource* source,
-                                       nsISimpleEnumerator/*<nsIRDFResource>*/** commands)
-{
-    NS_NOTYETIMPLEMENTED("write me!");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsWindowMediator::IsCommandEnabled(nsISupportsArray/*<nsIRDFResource>*/* aSources,
-                                         nsIRDFResource*   aCommand,
-                                         nsISupportsArray/*<nsIRDFResource>*/* aArguments,
-                                         PRBool* aResult)
-{
-    NS_NOTYETIMPLEMENTED("write me!");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-NS_IMETHODIMP nsWindowMediator::DoCommand(nsISupportsArray* aSources,
-                                  nsIRDFResource*   aCommand,
-                                  nsISupportsArray* aArguments)
-{
-    NS_NOTYETIMPLEMENTED("write me!");
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
-
-// Private helpers
-nsresult nsWindowMediator::AddWindowToRDF( nsWindowInfo* ioWindowInfo )
-{
-	if ( !ioWindowInfo || !(ioWindowInfo->mWindow) || !gRDFService )
-		return NS_ERROR_NULL_POINTER;
-		
-	nsCOMPtr<nsIRDFResource> window;
-	nsresult rv;	
-	
-	nsAutoString	windowTitle;
-	
-	// Make up a unique ID and create the RDF NODE
-	nsAutoString uniqueID(NS_LITERAL_STRING("window-"));
-	uniqueID.Append(windowTitle );
-	uniqueID.AppendInt( mTimeStamp, 10 );
-	char cID[ 256];
-	uniqueID.ToCString( cID, 256);
-	if (NS_FAILED(rv = gRDFService->GetResource(cID, getter_AddRefs(window) )))
-	{
-		NS_ERROR("unable to get window resource");
-		return rv;
-	}
-	ioWindowInfo->mRDFID = window ;
-	#if 0	
-	// Get the RDF literal and add it to our node 
-	nsCOMPtr<nsIRDFLiteral> windowTitleLiteral;
-	if (NS_FAILED(rv = gRDFService->GetLiteral( windowTitle.get(), getter_AddRefs(windowTitleLiteral))))
-	{
-		NS_ERROR("unable to create literal for window name");
-		return rv;
-	}
-
-	rv = Assert(window, kNC_Name, windowTitleLiteral, PR_TRUE);
-    if (rv != NS_RDF_ASSERTION_ACCEPTED)
-	{
-		NS_ERROR("unable to set window name");
-		return rv;
-	}
-	#endif
-	// Add the element to the container
-  nsCOMPtr<nsIRDFContainer> container;
-  rv = NS_NewMediatorListRDFContainer(mInner, kNC_WindowMediatorRoot, getter_AddRefs(container));
+  mWatcher = do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv);
   if (NS_FAILED(rv))
-  	return rv;
-
-	if (NS_FAILED(rv = container->AppendElement( window ) ) /* rdf_ContainerAppendElement(mInner, kNC_WindowMediatorRoot, window) */ )
-	{
-		NS_ERROR("unable to add window to container");
-		return rv;
-	}
-	return NS_OK;
+    return rv;
+  
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowMediator::AddListener(nsIWindowMediatorListener* aListener)
+{
+  NS_ENSURE_ARG_POINTER(aListener);
+  
+  nsresult rv;
+  if (!mListeners) {
+    rv = NS_NewISupportsArray(getter_AddRefs(mListeners));
+    if (NS_FAILED(rv)) return rv;
+  }
+
+  mListeners->AppendElement(aListener);
+  
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWindowMediator::RemoveListener(nsIWindowMediatorListener* aListener)
+{
+  NS_ENSURE_ARG_POINTER(aListener);
+
+  if (!mListeners) return NS_OK;
+
+  mListeners->RemoveElement(aListener);
+  
+  return NS_OK;
+}
+
+PRBool notifyOpenWindow(nsISupports *aElement, void* aData)
+{
+  nsIWindowMediatorListener* listener =
+    NS_REINTERPRET_CAST(nsIWindowMediatorListener*, aElement);
+  windowData* winData = (windowData*) aData;
+
+  listener->OnOpenWindow(winData->mWindow);
+  return PR_TRUE;
+}
+
+PRBool notifyCloseWindow(nsISupports *aElement, void* aData)
+{
+  nsIWindowMediatorListener* listener =
+    NS_REINTERPRET_CAST(nsIWindowMediatorListener*, aElement);
+  windowData* winData = (windowData*) aData;
+  
+  listener->OnCloseWindow(winData->mWindow);
+  return PR_TRUE;
+}
+
+PRBool notifyWindowTitleChange(nsISupports *aElement, void* aData)
+{
+  nsIWindowMediatorListener* listener =
+    NS_REINTERPRET_CAST(nsIWindowMediatorListener*, aElement);
+
+  windowData* titleData =
+    NS_REINTERPRET_CAST(windowData*, aData);
+  listener->OnWindowTitleChange(titleData->mWindow,
+                                titleData->mTitle);
+
+  return PR_TRUE;
 }
 
