@@ -30,6 +30,7 @@
 #include "nsString.h"
 #include "nsWidgetsCID.h"
 #include "prprf.h"
+#include "nsCRT.h"
 
 //Interfaces needed to be included
 #include "nsIAppShell.h"
@@ -37,6 +38,7 @@
 #include "nsIServiceManager.h"
 #include "nsIDocumentViewer.h"
 #include "nsIDocument.h"
+#include "nsIDOMBarProp.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMWindowInternal.h"
@@ -66,8 +68,6 @@
 
 #include "nsStyleConsts.h"
 
-#define ABS(x) ((x)<0?-(x):x)
-
 // XXX Get rid of this
 #pragma message("WARNING: XXX bad include, remove it.")
 #include "nsIWebShellWindow.h"
@@ -118,14 +118,15 @@ nsXULWindow::~nsXULWindow()
 // nsXULWindow::nsISupports
 //*****************************************************************************   
 
-NS_IMPL_ADDREF(nsXULWindow)
-NS_IMPL_RELEASE(nsXULWindow)
+NS_IMPL_THREADSAFE_ADDREF(nsXULWindow)
+NS_IMPL_THREADSAFE_RELEASE(nsXULWindow)
 
 NS_INTERFACE_MAP_BEGIN(nsXULWindow)
    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIXULWindow)
    NS_INTERFACE_MAP_ENTRY(nsIXULWindow)
    NS_INTERFACE_MAP_ENTRY(nsIBaseWindow)
    NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
+   NS_INTERFACE_MAP_ENTRY(nsISupportsWeakReference)
 NS_INTERFACE_MAP_END
 
 //*****************************************************************************
@@ -231,7 +232,7 @@ NS_IMETHODIMP nsXULWindow::GetContentShellById(const PRUnichar* aID,
       return NS_OK;
     }
   }
-  return NS_OK;
+  return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP nsXULWindow::AddChildWindow(nsIXULWindow *aChild)
@@ -249,51 +250,64 @@ NS_IMETHODIMP nsXULWindow::RemoveChildWindow(nsIXULWindow *aChild)
 
 NS_IMETHODIMP nsXULWindow::ShowModal()
 {
-   nsCOMPtr<nsIAppShell> appShell(do_CreateInstance(kAppShellCID));
-   NS_ENSURE_TRUE(appShell, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIAppShell> appShell(do_CreateInstance(kAppShellCID));
+  NS_ENSURE_TRUE(appShell, NS_ERROR_FAILURE);
 
-   appShell->Create(0, nsnull);
-   appShell->Spinup();
-   // Store locally so it doesn't die on us
-   nsCOMPtr<nsIWidget> window = mWindow;
-   nsCOMPtr<nsIXULWindow> tempRef = this;  
-                                          
+  appShell->Create(0, nsnull);
+  appShell->Spinup();
+  // Store locally so it doesn't die on us
+  nsCOMPtr<nsIWidget> window = mWindow;
+  nsCOMPtr<nsIXULWindow> tempRef = this;  
 
-   window->SetModal(PR_TRUE);
-   mContinueModalLoop = PR_TRUE;
-   EnableParent(PR_FALSE);
+  window->SetModal(PR_TRUE);
+  mContinueModalLoop = PR_TRUE;
+  EnableParent(PR_FALSE);
 
-   nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
-   nsresult rv = NS_OK;
-   if(stack  && NS_SUCCEEDED(stack->Push(nsnull)))
-      {
-      while(NS_SUCCEEDED(rv) && mContinueModalLoop)
-         {
-         void* data;
-         PRBool isRealEvent;
-         PRBool processEvent;
+  nsCOMPtr<nsIAppShellService> appShellService(do_GetService(kAppShellServiceCID));
+  if (appShellService)
+      appShellService->TopLevelWindowIsModal(
+                         NS_STATIC_CAST(nsIXULWindow*, this), PR_TRUE);
 
-         rv = appShell->GetNativeEvent(isRealEvent, data);
-         if(NS_SUCCEEDED(rv))
-            {
-            window->ModalEventFilter(isRealEvent, data, &processEvent);
-            if(processEvent)
-               appShell->DispatchNativeEvent(isRealEvent, data);
-            }
-         }
-      JSContext* cx;
-      stack->Pop(&cx);
-      NS_ASSERTION(cx == nsnull, "JSContextStack mismatch");
+  nsCOMPtr<nsIJSContextStack> stack(do_GetService("@mozilla.org/js/xpc/ContextStack;1"));
+  nsresult rv = NS_OK;
+  if (stack && NS_SUCCEEDED(stack->Push(nsnull))) {
+    while(NS_SUCCEEDED(rv) && mContinueModalLoop) {
+      void* data;
+      PRBool isRealEvent;
+      PRBool processEvent;
+
+      rv = appShell->GetNativeEvent(isRealEvent, data);
+      if(NS_SUCCEEDED(rv)) {
+        window->ModalEventFilter(isRealEvent, data, &processEvent);
+        if(processEvent)
+          appShell->DispatchNativeEvent(isRealEvent, data);
       }
-   else
-      rv = NS_ERROR_FAILURE;
+    }
+    JSContext* cx;
+    stack->Pop(&cx);
+    NS_ASSERTION(cx == nsnull, "JSContextStack mismatch");
+  } else
+    rv = NS_ERROR_FAILURE;
 
-   mContinueModalLoop = PR_FALSE;
+  mContinueModalLoop = PR_FALSE;
+  window->SetModal(PR_FALSE);
+  if (appShellService)
+      appShellService->TopLevelWindowIsModal(
+                         NS_STATIC_CAST(nsIXULWindow*, this), PR_FALSE);
+  /*   Note there's no EnableParent(PR_TRUE) here to match the PR_FALSE one
+     above. That's done in ExitModalLoop. It's important that the parent
+     be re-enabled before this window is made invisible; to do otherwise
+     causes bizarre z-ordering problems. At this point, the window is
+     already invisible.
+       No known current implementation of Enable would have a problem with
+     re-enabling the parent twice, so we could do it again here without
+     breaking any current implementation. But that's unnecessary if the
+     modal loop is always exited using ExitModalLoop (the other way would be
+     to change the protected member variable directly.)
+  */
+  appShell->Spindown();
 
-   window->SetModal(PR_FALSE);
-   appShell->Spindown();
-
-   return mModalStatus;
+  return mModalStatus;
 }
 
 //*****************************************************************************
@@ -317,18 +331,29 @@ NS_IMETHODIMP nsXULWindow::Create()
 
 NS_IMETHODIMP nsXULWindow::Destroy()
 {
-   mBeingDestroyed = PR_TRUE;
    if(!mWindow)
       return NS_OK;
 
+#ifdef XP_PC
+   /* must activate parent before unregistering, since unregistering
+      affects the topmost window status, which activateparent uses. */
+   ActivateParent();
+#endif
+
+   {
+    /* unregister before setting mBeingDestroyed because -turbo code
+        wants to be able to pose a dialog. */
+    nsCOMPtr<nsIAppShellService> appShell(do_GetService(kAppShellServiceCID));
+    if(appShell)
+        appShell->UnregisterTopLevelWindow(NS_STATIC_CAST(nsIXULWindow*, this));
+   }
+   
+   mBeingDestroyed = PR_TRUE;
+
    nsCOMPtr<nsIXULWindow> parentWindow(do_QueryReferent(mParentWindow));
    if (parentWindow)
      parentWindow->RemoveChildWindow(this);
 
-#ifdef XP_PC
-   ActivateParent();
-#endif
-
 // Anyone still using native menus should add themselves here.
 #if defined(XP_MAC) || defined(XP_MACOSX)
   {
@@ -348,10 +373,6 @@ NS_IMETHODIMP nsXULWindow::Destroy()
    }
 #endif
 
-   nsCOMPtr<nsIAppShellService> appShell(do_GetService(kAppShellServiceCID));
-   if(appShell)
-      appShell->UnregisterTopLevelWindow(NS_STATIC_CAST(nsIXULWindow*, this));
-   
    // let's make sure the window doesn't get deleted out from under us
    // while we are trying to close....this can happen if the docshell
    // we close ends up being the last owning reference to this xulwindow
@@ -460,20 +481,23 @@ NS_IMETHODIMP nsXULWindow::SetPositionAndSize(PRInt32 aX, PRInt32 aY,
 NS_IMETHODIMP nsXULWindow::GetPositionAndSize(PRInt32* x, PRInt32* y, PRInt32* cx,
    PRInt32* cy)
 {
-   nsRect      rect;
+  nsRect rect;
 
-   mWindow->GetScreenBounds(rect);
+  if (!mWindow)
+    return NS_ERROR_FAILURE;
 
-   if(x)
-      *x = rect.x;
-   if(y)
-      *y = rect.y;
-   if(cx)
-      *cx = rect.width;
-   if(cy)
-      *cy = rect.height;
+  mWindow->GetScreenBounds(rect);
 
-   return NS_OK;
+  if(x)
+    *x = rect.x;
+  if(y)
+    *y = rect.y;
+  if(cx)
+    *cx = rect.width;
+  if(cy)
+    *cy = rect.height;
+
+  return NS_OK;
 }
 
 NS_IMETHODIMP nsXULWindow::Center(nsIXULWindow *aRelative, PRBool aScreen, PRBool aAlert) {
@@ -502,13 +526,23 @@ NS_IMETHODIMP nsXULWindow::Center(nsIXULWindow *aRelative, PRBool aScreen, PRBoo
   if (aRelative) {
     nsCOMPtr<nsIBaseWindow> base(do_QueryInterface(aRelative, &result));
     if (base) {
-      base->GetPositionAndSize(&left, &top, &width, &height);
-      if (aScreen)
-        screenmgr->ScreenForRect(left, top, width, height, getter_AddRefs(screen));
-      else
-        windowCoordinates = PR_TRUE;
+      // get window rect
+      result = base->GetPositionAndSize(&left, &top, &width, &height);
+      if (NS_SUCCEEDED(result)) {
+        // if centering on screen, convert that to the corresponding screen
+        if (aScreen)
+          screenmgr->ScreenForRect(left, top, width, height, getter_AddRefs(screen));
+        else
+          windowCoordinates = PR_TRUE;
+      } else {
+        // something's wrong with the reference window.
+        // fall back to the primary screen
+        aRelative = 0;
+        aScreen = PR_TRUE;
+      }
     }
-  } else
+  }
+  if (!aRelative)
     screenmgr->GetPrimaryScreen(getter_AddRefs(screen));
 
   if (aScreen && screen) {
@@ -619,6 +653,25 @@ NS_IMETHODIMP nsXULWindow::SetVisibility(PRBool aVisibility)
    return NS_OK;
 }
 
+NS_IMETHODIMP nsXULWindow::GetEnabled(PRBool *aEnabled)
+{
+  NS_ENSURE_ARG_POINTER(aEnabled);
+  if (mWindow)
+    return mWindow->IsEnabled(aEnabled);
+
+  *aEnabled = PR_TRUE; // better guess than most
+  return NS_ERROR_FAILURE;
+}
+
+NS_IMETHODIMP nsXULWindow::SetEnabled(PRBool aEnable)
+{
+  if (mWindow) {
+    mWindow->Enable(aEnable);
+    return NS_OK;
+  }
+  return NS_ERROR_FAILURE;
+}
+
 NS_IMETHODIMP nsXULWindow::GetMainWidget(nsIWidget** aMainWidget)
 {
    NS_ENSURE_ARG_POINTER(aMainWidget);
@@ -952,7 +1005,7 @@ void nsXULWindow::StaggerPosition(PRInt32 &aRequestedX, PRInt32 &aRequestedY,
   PRBool   keepTrying;
 
   // look for any other windows of this type
-  nsCOMPtr<nsIWindowMediator> wm(do_GetService("@mozilla.org/rdf/datasource;1?name=window-mediator"));
+  nsCOMPtr<nsIWindowMediator> wm(do_GetService(NS_WINDOWMEDIATOR_CONTRACTID));
   if (!wm)
     return;
 
@@ -995,7 +1048,7 @@ void nsXULWindow::StaggerPosition(PRInt32 &aRequestedX, PRInt32 &aRequestedY,
       if (listXULWindow != ourXULWindow) {
         listBaseWindow->GetPosition(&listX, &listY);
 
-        if (ABS(listX-aRequestedX) <= kSlop && ABS(listY-aRequestedY) <= kSlop) {
+        if (PR_ABS(listX-aRequestedX) <= kSlop && PR_ABS(listY-aRequestedY) <= kSlop) {
           // collision! offset and stop. save the DOMWindow corresponding
           // to the colliding window for later. (we'll need its nsIDOMScreen
           nsCOMPtr<nsIDocShell> listDocShell;
@@ -1067,6 +1120,15 @@ NS_IMETHODIMP nsXULWindow::LoadIconFromXUL()
     GetWindowDOMElement(getter_AddRefs(windowElement));
     NS_ENSURE_TRUE(windowElement, NS_ERROR_FAILURE);
 
+    // XXX The following code is being #if 0'd out since it
+    // basically does nothing until bug 70974 is fixed.
+    // After bug 70974 is fixed, we will also need to implement
+    // computed style for that property before this will
+    // be of any use.  And even then, it will *still* 
+    // do nothing on platforms which don't implement 
+    // nsWindow::SetIcon(). See bug 76211 for that.
+    // Also see bug 57576 and its dependency tree.
+#if 0
     // Get document in which this <window> is contained.
     nsCOMPtr<nsIDOMDocument> document;
     windowElement->GetOwnerDocument(getter_AddRefs(document));
@@ -1094,29 +1156,23 @@ NS_IMETHODIMP nsXULWindow::LoadIconFromXUL()
     // Whew.  Now get "list-style-image" property value.
     nsAutoString windowIcon;
     windowIcon.Assign(NS_LITERAL_STRING("-moz-window-icon"));
-    nsAutoString value;
-    cssDecl->GetPropertyValue(windowIcon, value);
+    nsAutoString icon;
+    cssDecl->GetPropertyValue(windowIcon, icon);
+#endif
 
-    // If no icon specified via -moz-window-icon, then use id= attr.
-    if ( value.IsEmpty() )
-    {
-        value.Assign(NS_LITERAL_STRING("resource:///chrome/icons/default/"));
-        nsAutoString attr;
-        attr.Assign(NS_LITERAL_STRING("id"));
-        nsAutoString id;
-        windowElement->GetAttribute(attr,id);
-        if(!id.IsEmpty())
-        {
-            value.Append(id);
-        }
-        else
-        {
-            value.Append(NS_LITERAL_STRING("default"));
-        }
+    nsAutoString icon;
+    icon.Assign(NS_LITERAL_STRING("resource:///chrome/icons/default/"));
+
+    nsAutoString id;
+    windowElement->GetAttribute(NS_LITERAL_STRING("id"), id);
+
+    if (id.IsEmpty()) {
+        id.Assign(NS_LITERAL_STRING("default"));
     }
-    
-    // Finally, set the icon using that attribute value.
-    mWindow->SetIcon(value);
+
+    icon.Append(id);
+
+    mWindow->SetIcon(icon);
     return NS_OK;
 }
 
@@ -1184,10 +1240,9 @@ NS_IMETHODIMP nsXULWindow::PersistPositionAndSize(PRBool aPosition, PRBool aSize
     }
   }
 
-  if (aSizeMode && persistString.Find("sizemode") >= 0) {
-    if (sizeMode == nsSizeMode_Minimized)
-      sizeString.Assign(SIZEMODE_MINIMIZED);
-    else if (sizeMode == nsSizeMode_Maximized)
+  if (aSizeMode && sizeMode != nsSizeMode_Minimized &&
+      persistString.Find("sizemode") >= 0) {
+    if (sizeMode == nsSizeMode_Maximized)
       sizeString.Assign(SIZEMODE_MAXIMIZED);
     else
       sizeString.Assign(SIZEMODE_NORMAL);
@@ -1355,11 +1410,11 @@ NS_IMETHODIMP nsXULWindow::SizeShellTo(nsIDocShellTreeItem* aShellItem,
 
 NS_IMETHODIMP nsXULWindow::ExitModalLoop(nsresult aStatus)
 {
-   if (mContinueModalLoop) // was a modal window
-     EnableParent(PR_TRUE);
-   mContinueModalLoop = PR_FALSE;
-   mModalStatus = aStatus;
-   return NS_OK;
+  if (mContinueModalLoop)
+    EnableParent(PR_TRUE);
+  mContinueModalLoop = PR_FALSE;
+  mModalStatus = aStatus;
+  return NS_OK;
 }
 
 // top-level function to create a new window
@@ -1452,7 +1507,7 @@ NS_IMETHODIMP nsXULWindow::CreateNewContentWindow(PRInt32 aChromeFlags,
 
      nsCOMPtr<nsIIOService> service(do_GetService(kIOServiceCID));
      if (service)
-       service->NewURI(urlStr, nsnull, getter_AddRefs(uri));
+       service->NewURI(nsDependentCString(urlStr), nsnull, nsnull, getter_AddRefs(uri));
      if (strAllocated)
        PL_strfree(urlStr);
    }
@@ -1682,16 +1737,32 @@ PRBool nsXULWindow::ConstrainToZLevel(
 void nsXULWindow::SetContentScrollbarVisibility(PRBool aVisible) {
 
   nsCOMPtr<nsIDocShellTreeItem> content;
-  if (NS_SUCCEEDED(GetPrimaryContentShell(getter_AddRefs(content))) && content) {
-    nsCOMPtr<nsIScrollable> shell(do_QueryInterface(content));
-    if (shell) {
-      long prefValue = aVisible ? NS_STYLE_OVERFLOW_AUTO : NS_STYLE_OVERFLOW_HIDDEN;
-      shell->SetDefaultScrollbarPreferences(nsIScrollable::ScrollOrientation_Y, prefValue);
-      shell->SetDefaultScrollbarPreferences(nsIScrollable::ScrollOrientation_X, prefValue);
-    }
+  GetPrimaryContentShell(getter_AddRefs(content));
+  nsCOMPtr<nsIDOMWindow> contentWin(do_GetInterface(content));
+  if (contentWin) {
+    nsCOMPtr<nsIDOMBarProp> scrollbars;
+    contentWin->GetScrollbars(getter_AddRefs(scrollbars));
+    if (scrollbars)
+      scrollbars->SetVisible(aVisible);
   }
 }
 
+PRBool nsXULWindow::GetContentScrollbarVisibility() {
+
+  PRBool visible = PR_TRUE;
+
+  nsCOMPtr<nsIDocShellTreeItem> content;
+  GetPrimaryContentShell(getter_AddRefs(content));
+  nsCOMPtr<nsIDOMWindow> contentWin(do_GetInterface(content));
+  if (contentWin) {
+    nsCOMPtr<nsIDOMBarProp> scrollbars;
+    contentWin->GetScrollbars(getter_AddRefs(scrollbars));
+    if (scrollbars)
+      scrollbars->GetVisible(&visible);
+  }
+  return visible;
+}
+
 //*****************************************************************************
 // nsXULWindow: Accessors
 //*****************************************************************************
