@@ -43,6 +43,7 @@
 #include "nsIURI.h"
 #include "nsNetUtil.h"
 #include "nsIPref.h"
+#include "nsILocaleService.h"
 #include "plevent.h"
 #include "prmem.h"
 #include "prnetdb.h"
@@ -53,6 +54,8 @@
 #include "nsIAppShellService.h"
 #include "nsIAppStartupNotifier.h"
 #include "nsIObserverService.h"
+#include "nsIPlatformCharset.h"
+#include "nsICharsetConverterManager.h"
 #include "nsAppShellCIDs.h"
 #include "prprf.h"
 #include "nsCRT.h"
@@ -71,7 +74,6 @@
 #include "nsIContentHandler.h"
 #include "nsIBrowserInstance.h"
 #include "nsIEventQueueService.h"
-#include "nsMPFileLocProvider.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsIHttpProtocolHandler.h"
 #include "nsBuildID.h"
@@ -94,13 +96,20 @@
 #include "nsIXRemoteService.h"
 #endif
 
+// see DoOnShutdown()
+#include "nsIProfile.h"
+
 #ifdef NS_TRACE_MALLOC
 #include "nsTraceMalloc.h"
 #endif
 
+#if defined(DEBUG) && defined(XP_WIN32)
+#include <malloc.h>
+#endif
+
 #include "nsITimelineService.h"
 
-#if defined(DEBUG_sspitzer) || defined(DEBUG_seth) || defined(DEBUG_pra)
+#if defined(DEBUG_pra)
 #define DEBUG_CMD_LINE
 #endif
 
@@ -129,7 +138,7 @@ extern "C" void ShowOSAlert(const char* aMessage);
 #ifdef _BUILD_STATIC_BIN
 #include "nsStaticComponent.h"
 nsresult PR_CALLBACK
-apprunner_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
+app_getModuleInfo(nsStaticModuleInfo **info, PRUint32 *count);
 #endif
 
 #if defined(XP_UNIX) || defined(XP_BEOS)
@@ -184,7 +193,7 @@ private:
         appFileInfo.SetTo(&file) != B_OK ||
         appFileInfo.GetSignature(sig) != B_OK)
     {
-      return "application/x-vnd.mozilla.apprunner";
+      return "application/x-vnd.Mozilla";
     }
     return sig;
   }
@@ -228,6 +237,7 @@ static nsresult InitializeProfileService(nsICmdLineService *cmdLineArgs);
 
 // Install global locale if possible
 static nsresult InstallGlobalLocale(nsICmdLineService *cmdLineArgs);
+static nsresult getUILangCountry(nsAString& aUILang, nsAString& aCountry);
 
 class stTSMCloser
 {
@@ -279,7 +289,7 @@ static const char sWatcherServiceContractID[] = "@mozilla.org/embedcomp/window-w
 /*********************************************/
 // Default implemenations for nativeAppSupport
 // If your platform implements these functions if def out this code.
-#if !defined (XP_MAC ) && !defined(MOZ_WIDGET_PHOTON) && !defined( XP_PC ) && !defined( XP_BEOS ) && !defined(MOZ_WIDGET_GTK)
+#if !defined (XP_MAC) && !defined(MOZ_WIDGET_COCOA) && !defined(MOZ_WIDGET_PHOTON) && !defined( XP_PC ) && !defined( XP_BEOS ) && !defined(MOZ_WIDGET_GTK)
 
 nsresult NS_CreateSplashScreen( nsISplashScreen **aResult )
 {
@@ -313,7 +323,7 @@ PRBool NS_CanRun()
 //       then rely on nsINativeAppSupport and its use of
 //       nsISplashScreen will be removed.
 //
-#if !defined( XP_PC ) && !defined( XP_BEOS ) && !defined(MOZ_WIDGET_GTK)
+#if !defined( XP_PC ) && !defined(MOZ_WIDGET_GTK)
 
 nsresult NS_CreateNativeAppSupport( nsINativeAppSupport **aResult )
 {
@@ -398,6 +408,31 @@ static nsresult OpenWindow(const nsAFlatCString& aChromeURL,
   if (!wwatch || !sarg)
     return NS_ERROR_FAILURE;
 
+  // Make sure a profile is selected.
+
+  // We need the native app support object, which we get from
+  // the app shell service.  If this fails, we still proceed.
+  // That's because some platforms don't have a native app
+  // support implementation.  On those platforms, "ensuring a
+  // profile" is moot (because they don't support "-turbo",
+  // basically).  Specifically, because they don't do turbo, they will
+  // *always* have a profile selected.
+  nsCOMPtr<nsIAppShellService> appShell(do_GetService("@mozilla.org/appshell/appShellService;1"));
+  nsCOMPtr <nsICmdLineService> cmdLine(do_GetService("@mozilla.org/appshell/commandLineService;1"));
+  if (appShell && cmdLine)
+  {
+    nsCOMPtr<nsINativeAppSupport> nativeApp;
+    if (NS_SUCCEEDED(appShell->GetNativeAppSupport(getter_AddRefs(nativeApp))))
+    {
+      // Make sure profile has been selected.
+      // At this point, we have to look for failure.  That
+      // handles the case where the user chooses "Exit" on
+      // the profile manager window.
+      if (NS_FAILED(nativeApp->EnsureProfile(cmdLine)))
+        return NS_ERROR_NOT_INITIALIZED;
+    }
+  }
+
   sarg->SetData(aAppArgs.get());
 
   nsCAutoString features("chrome,dialog=no,all");
@@ -737,6 +772,9 @@ static nsresult HandleArbitraryStartup( nsICmdLineService* cmdLineArgs, nsIPref
         rv = LaunchApplicationWithArgs((const char *)(argv[i]),
                                        cmdLineArgs, command,
                                        height, width, windowOpened);
+        if (rv == NS_ERROR_NOT_AVAILABLE || rv == NS_ERROR_ABORT) {
+          return rv;
+        }
       }
     }
   }
@@ -763,6 +801,17 @@ static nsresult DoOnShutdown()
 {
   nsresult rv;
 
+  // call ShutDownCurrentProfile() so we update the last modified time of the profile
+  {
+    // scoping this in a block to force release
+    nsCOMPtr<nsIProfile> profileMgr(do_GetService(NS_PROFILE_CONTRACTID, &rv));
+    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get profile manager, so unable to update last modified time");
+    if (NS_SUCCEEDED(rv)) {
+      // 0 is undefined, we use this secret value so that we don't notify
+      profileMgr->ShutDownCurrentProfile(0);
+    }
+  }
+
   // save the prefs, in case they weren't saved
   {
     // scoping this in a block to force release
@@ -787,7 +836,43 @@ static nsresult DoOnShutdown()
   return rv;
 }
 
+static nsresult ConvertToUnicode(nsString& aCharset, const char* inString, nsAString& outString)
+{
+  nsresult rv;
 
+  // convert result to unicode
+  nsCOMPtr<nsICharsetConverterManager> ccm(do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID , &rv));
+  if (NS_FAILED(rv))
+    return rv;
+
+  nsCOMPtr <nsIUnicodeDecoder> decoder; 
+  rv = ccm->GetUnicodeDecoder(&aCharset, getter_AddRefs(decoder));
+  if (NS_FAILED(rv))
+    return rv;
+
+  PRInt32 uniLength = 0;
+  PRInt32 srcLength = strlen(inString);
+  rv = decoder->GetMaxLength(inString, srcLength, &uniLength);
+  if (NS_FAILED(rv))
+    return rv;
+
+  PRUnichar *unichars = new PRUnichar [uniLength];
+  if (nsnull != unichars) {
+    // convert to unicode
+    rv = decoder->Convert(inString, &srcLength, unichars, &uniLength);
+    if (NS_SUCCEEDED(rv)) {
+      // Pass back the unicode string
+      outString.Assign(unichars, uniLength);
+    }
+    delete [] unichars;
+  }
+  else {
+    rv = NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  return rv;
+}
+ 
 static nsresult OpenBrowserWindow(PRInt32 height, PRInt32 width)
 {
     nsresult rv;
@@ -811,8 +896,32 @@ static nsresult OpenBrowserWindow(PRInt32 height, PRInt32 width)
       printf("url to load: %s\n", urlToLoad.get());
 #endif /* DEBUG_CMD_LINE */
 
-      NS_ConvertUTF8toUCS2 url(urlToLoad);
+      nsAutoString url; 
+      if (nsCRT::IsAscii(urlToLoad))  {
+        url.AssignWithConversion(urlToLoad);
+      }
+      else {
+        // get a platform charset
+        nsAutoString charSet;
+        nsCOMPtr <nsIPlatformCharset> platformCharset(do_GetService(NS_PLATFORMCHARSET_CONTRACTID, &rv));
+        if (NS_FAILED(rv)) {
+          NS_ASSERTION(0, "Failed to get a platform charset");
+          return rv;
+        }
 
+        rv = platformCharset->GetCharset(kPlatformCharsetSel_FileName, charSet);
+        if (NS_FAILED(rv)) {
+          NS_ASSERTION(0, "Failed to get a charset");
+          return rv;
+        }
+
+        // convert the cmdLine URL to Unicode
+        rv = ConvertToUnicode(charSet, urlToLoad, url);
+        if (NS_FAILED(rv)) {
+          NS_ASSERTION(0, "Failed to convert commandline url to unicode");
+          return rv;
+        }
+      }
       rv = OpenWindow(chromeUrlForTask, url, width, height);
 
     } else {
@@ -893,33 +1002,99 @@ static nsresult Ensure1Window( nsICmdLineService* cmdLineArgs)
   return rv;
 }
 
+// match OS locale
+static char kMatchOSLocalePref[] = "intl.locale.matchOS";
+
+nsresult
+getCountry(const nsAString& lc_name, nsAString& aCountry)
+{
+
+  nsresult        result = NS_OK;
+
+  PRInt32 dash = lc_name.FindChar('-');
+  if (dash > 0)
+    aCountry = Substring(lc_name, dash+1, lc_name.Length()-dash);
+  else
+    result = NS_ERROR_FAILURE;
+
+  return result;
+}
+
+static nsresult
+getUILangCountry(nsAString& aUILang, nsAString& aCountry)
+{
+  nsresult	 result;
+  // get a locale service 
+  nsCOMPtr<nsILocaleService> localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID, &result);
+  NS_ASSERTION(NS_SUCCEEDED(result),"getUILangCountry: get locale service failed");
+
+  nsXPIDLString uiLang;
+  result = localeService->GetLocaleComponentForUserAgent(getter_Copies(uiLang));
+  aUILang = uiLang;
+  result = getCountry(aUILang, aCountry);
+  return result;
+}
+
 // update global locale if possible (in case when user-*.rdf can be updated)
 // so that any apps after this can be invoked in the UILocale and contentLocale
 static nsresult InstallGlobalLocale(nsICmdLineService *cmdLineArgs)
 {
     nsresult rv = NS_OK;
+
+    // check the pref first
+    nsCOMPtr<nsIPref> prefService(do_GetService("@mozilla.org/preferences;1"));
+    PRBool matchOS = PR_FALSE;
+    if (prefService)
+      prefService->GetBoolPref(kMatchOSLocalePref, &matchOS);
+
+    // match os locale
+    nsAutoString uiLang;
+    nsAutoString country;
+    if (matchOS) {
+      // compute lang and region code only when needed!
+      rv = getUILangCountry(uiLang, country);
+    }
+
     nsXPIDLCString cmdUI;
     rv = cmdLineArgs->GetCmdLineValue(UILOCALE_CMD_LINE_ARG, getter_Copies(cmdUI));
     if (NS_SUCCEEDED(rv)){
         if (cmdUI) {
             nsAutoString UILocaleName;
             UILocaleName.AssignWithConversion(cmdUI);
-            nsCOMPtr<nsIChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
+            nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
             if (chromeRegistry)
                 rv = chromeRegistry->SelectLocale(UILocaleName.get(), PR_FALSE);
         }
     }
+    // match OS when no cmdline override
+    if (!cmdUI && matchOS) {
+      nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
+      if (chromeRegistry) {
+        chromeRegistry->SetRuntimeProvider(PR_TRUE);
+        rv = chromeRegistry->SelectLocale(uiLang.get(), PR_FALSE);
+      }
+    }
+
     nsXPIDLCString cmdContent;
     rv = cmdLineArgs->GetCmdLineValue(CONTENTLOCALE_CMD_LINE_ARG, getter_Copies(cmdContent));
     if (NS_SUCCEEDED(rv)){
         if (cmdContent) {
             nsAutoString ContentLocaleName;
             ContentLocaleName.AssignWithConversion(cmdContent);
-            nsCOMPtr<nsIChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
+            nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
             if(chromeRegistry)
                 rv = chromeRegistry->SelectLocale(ContentLocaleName.get(), PR_FALSE);
         }
     }
+    // match OS when no cmdline override
+    if (!cmdContent && matchOS) {
+      nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(kChromeRegistryCID, &rv);
+      if (chromeRegistry) {
+        chromeRegistry->SetRuntimeProvider(PR_TRUE);        
+        rv = chromeRegistry->SelectLocale(country.get(), PR_FALSE);
+      }
+    }
+
     return NS_OK;
 }
 
@@ -974,7 +1149,7 @@ static void ShowOSAlertFromFile(int argc, char **argv, const char *alert_filenam
                                NS_GET_IID(nsIFile),
                                getter_AddRefs(fileName));
     if (NS_SUCCEEDED(rv) && fileName) {
-      fileName->Append(alert_filename);
+      fileName->AppendNative(nsDependentCString(alert_filename));
       PRFileDesc* fd = 0;
       fileName->OpenNSPRFileDesc(PR_RDONLY, 0664, &fd);
       if (fd) {
@@ -1003,16 +1178,13 @@ static nsresult VerifyInstallation(int argc, char **argv)
            do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv))
     return NS_OK;
-  rv = directoryService->Get(NS_OS_CURRENT_PROCESS_DIR,
+  rv = directoryService->Get(NS_APP_INSTALL_CLEANUP_DIR,
                              NS_GET_IID(nsIFile),
                              getter_AddRefs(registryFile));
   if (NS_FAILED(rv) || !registryFile)
-    return NS_OK;
+    return NS_ERROR_FAILURE;
 
-#ifdef XP_MAC
-  registryFile->Append(ESSENTIAL_FILES);
-#endif
-  registryFile->Append(CLEANUP_REGISTRY);
+  registryFile->AppendNative(CLEANUP_REGISTRY);
 
   PRBool exists;
   registryFile->Exists(&exists);
@@ -1021,11 +1193,11 @@ static nsresult VerifyInstallation(int argc, char **argv)
     nsCOMPtr<nsIFile> binPath;
     const char lastResortMessage[] = "A previous install did not complete correctly.  Finishing install.";
 
-    ShowOSAlertFromFile(argc, argv, CLEANUP_MESSAGE_FILENAME, lastResortMessage);
+    ShowOSAlertFromFile(argc, argv, CLEANUP_MESSAGE_FILENAME.get(), lastResortMessage);
 
     nsCOMPtr<nsIFile> cleanupUtility;
     registryFile->Clone(getter_AddRefs(cleanupUtility));
-    cleanupUtility->SetLeafName(CLEANUP_UTIL);
+    cleanupUtility->SetNativeLeafName(CLEANUP_UTIL);
 
     //Create the process framework to run the cleanup utility
     nsCOMPtr<nsIProcess> cleanupProcess = do_CreateInstance(kIProcessCID);
@@ -1048,7 +1220,7 @@ static nsresult VerifyInstallation(int argc, char **argv)
 
 #if defined(FREEBSD)
 // pick up fpsetmask prototype.
-#include <floatingpoint.h>
+#include <ieeefp.h>
 #endif
 
 // Note: nativeApp is an owning reference that this function has responsibility
@@ -1204,7 +1376,6 @@ static nsresult main1(int argc, char* argv[], nsISupports *nativeApp )
         splashScreen->Hide();
       }
     }
-
     // Release argument object.
     NS_IF_RELEASE(nativeApp);
     return rv;
@@ -1324,6 +1495,7 @@ static void DumpHelp(char *appname)
   printf("%s--gtk-debug=FLAGS%sGtk+ debugging flags to set\n", HELP_SPACER_1, HELP_SPACER_2);
   printf("%s--gtk-no-debug=FLAGS%sGtk+ debugging flags to unset\n", HELP_SPACER_1, HELP_SPACER_2);
   printf("%s--gtk-module=MODULE%sLoad an additional Gtk module\n", HELP_SPACER_1, HELP_SPACER_2);
+  printf("%s-install%sInstall a private colormap\n", HELP_SPACER_1, HELP_SPACER_2);
 
   /* end gtk toolkit options */
 #endif /* MOZ_WIDGET_GTK */
@@ -1397,8 +1569,8 @@ static nsresult DumpVersion(char *appname)
   nsCOMPtr <nsIHttpProtocolHandler> httpHandler(do_GetService("@mozilla.org/network/protocol;1?name=http", &rv));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  nsXPIDLCString agent;
-  httpHandler->GetUserAgent(getter_Copies(agent));
+  nsCAutoString agent;
+  httpHandler->GetUserAgent(agent);
 
   printf("%s", agent.get());
 
@@ -1527,6 +1699,14 @@ int main(int argc, char* argv[])
 {
   NS_TIMELINE_MARK("enter main");
 
+#if defined(DEBUG) && defined(XP_WIN32)
+  // Disable small heap allocator to get heapwalk() giving us
+  // accurate heap numbers. Win2k non-debug does not use small heap allocator.
+  // Win2k debug seems to be still using it.
+  // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vclib/html/_crt__set_sbh_threshold.asp
+  _set_sbh_threshold(0);
+#endif
+
 #if defined(XP_UNIX) || defined(XP_BEOS)
   InstallUnixSignalHandlers(argv[0]);
 #endif
@@ -1547,7 +1727,7 @@ int main(int argc, char* argv[])
 
 #ifdef _BUILD_STATIC_BIN
   // Initialize XPCOM's module info table
-  NSGetStaticModuleInfo = apprunner_getModuleInfo;
+  NSGetStaticModuleInfo = app_getModuleInfo;
 #endif
 
   // Handle -help and -version command line arguments.
