@@ -206,6 +206,10 @@ public:
     HWND mDlg;
     HBITMAP mBitmap;
     nsrefcnt mRefCnt;
+    HDC hdcMemory;
+    HPS hpsMemory;
+    LONG mBitmapCX;
+    LONG mBitmapCY;
 }; // class nsSplashScreenOS2
 
 MRESULT EXPENTRY DialogProc( HWND dlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
@@ -410,11 +414,13 @@ private:
     static DWORD mInstance;
     static char *mAppName;
     static nsIDOMWindow *mInitialWindow;
+    static PRBool mUseDDE;
     friend struct MessageWindow;
 }; // nsNativeAppSupportOS2
 
 nsSplashScreenOS2::nsSplashScreenOS2()
-    : mDlg( 0 ), mBitmap( 0 ), mRefCnt( 0 ) {
+    : mDlg( 0 ), mBitmap( 0 ), mRefCnt( 0 ),
+      hdcMemory( 0 ), hpsMemory( 0 ), mBitmapCX(0), mBitmapCY(0) {
 }
 
 nsSplashScreenOS2::~nsSplashScreenOS2() {
@@ -455,56 +461,131 @@ nsSplashScreenOS2::Hide() {
 
         // Dismiss the dialog.
         WinPostMsg(mDlg, WM_CLOSE, 0, 0);
-        // Release custom bitmap (if there is one).
-        if ( mBitmap ) {
-            BOOL ok = GpiDeleteBitmap( mBitmap );
-        }
-        mBitmap = 0;
         mDlg = 0;
+        GpiSetBitmap(hpsMemory, NULLHANDLE);
+        if (mBitmap) {
+            GpiDeleteBitmap(mBitmap);
+            mBitmap = 0;
+        }
+        if (hdcMemory) {
+            DevCloseDC(hdcMemory);
+            hdcMemory = 0;
+        }
+        if (hpsMemory) {
+           GpiDestroyPS(hpsMemory);
+           hpsMemory = 0;
+        }
     }
     return NS_OK;
 }
 
+HBITMAP LoadAndSetBitmapFromFile(HPS hps, PSZ pszFilename)
+{
+   FILE *fp = fopen(pszFilename, "rb");
+   if (fp == NULL) {
+      return NULLHANDLE;
+   }
+   fseek(fp, 0, SEEK_END );
+   ULONG cbFile = ftell(fp);
+   if (cbFile ==0) {
+      fclose(fp);
+      return NULLHANDLE;
+   }
+   fseek(fp, 0, SEEK_SET );
+   PBYTE pBitmapData = (PBYTE)malloc(cbFile);
+   fread((PVOID)pBitmapData, cbFile, 1, fp);
+   fclose(fp);
+
+   PBITMAPFILEHEADER2 pbfh2 = (PBITMAPFILEHEADER2)pBitmapData;
+   PBITMAPINFOHEADER2 pbmp2 = NULL;
+
+   switch (pbfh2->usType)
+   {
+      case BFT_BITMAPARRAY:
+         /*
+          *   If this is a Bitmap-Array, adjust pointer to the normal
+          *   file header.  We'll just use the first bitmap in the
+          *   array and ignore other device forms.
+          */
+         pbfh2 = &(((PBITMAPARRAYFILEHEADER2) pBitmapData)->bfh2);
+         pbmp2 = &pbfh2->bmp2;
+         break;
+      case BFT_BMAP:
+         pbmp2 = &pbfh2->bmp2;
+         break;
+      case BFT_ICON:
+      case BFT_POINTER:
+      case BFT_COLORICON:
+      case BFT_COLORPOINTER:
+      default:
+         break;
+   }
+    
+   if (pbmp2 == NULL) {
+      free(pBitmapData);
+      return NULLHANDLE;
+   }
+
+   LONG lScans;
+   if (pbmp2->cbFix == sizeof(BITMAPINFOHEADER))
+      lScans = (LONG) ((PBITMAPINFOHEADER)pbmp2)->cy;
+   else
+      lScans = pbmp2->cy;
+
+   HBITMAP hbmp = GpiCreateBitmap(hps, pbmp2, 0L, NULL, NULL);
+   if (!hbmp) {
+      free(pBitmapData);
+      return NULLHANDLE;
+   }
+
+   if (GpiSetBitmap(hps, hbmp) == HBM_ERROR) {
+      GpiDeleteBitmap(hbmp);
+      free(pBitmapData);
+      return NULLHANDLE;
+   }
+
+   LONG lScansSet = GpiSetBitmapBits(hps, 0L, lScans, pBitmapData + pbfh2->offBits,
+                                     (PBITMAPINFO2) pbmp2);
+   free(pBitmapData);
+
+   if (lScansSet != lScans) {
+      GpiSetBitmap(hps, NULLHANDLE);
+      GpiDeleteBitmap(hbmp);
+      return NULLHANDLE;
+   }
+
+   return hbmp;
+}
+
 void
 nsSplashScreenOS2::LoadBitmap() {
-#ifdef XP_OS2
-     HPS hps = WinGetPS( mDlg );
-     mBitmap = GpiLoadBitmap (hps, NULL, IDB_SPLASH, 0L, 0L);
-     WinReleasePS( hps );
-#else
+    hdcMemory = DevOpenDC((HAB)0, OD_MEMORY, "*", 0L, NULL, 0L);
+    SIZEL sizel = {0, 0};
+    hpsMemory = GpiCreatePS((HAB)0, hdcMemory, &sizel,
+                            PU_PELS | GPIF_DEFAULT | GPIT_MICRO | GPIA_ASSOC );
+
     // Check for '<program-name>.bmp" in same directory as executable.
-    char fileName[ _MAX_PATH ];
-    int fileNameLen = ::GetModuleFileName( NULL, fileName, sizeof fileName );
-    if ( fileNameLen >= 3 ) {
+    PPIB ppib;
+    PTIB ptib;
+    char fileName[CCHMAXPATH];
+    DosGetInfoBlocks( &ptib, &ppib);
+    DosQueryModuleName( ppib->pib_hmte, CCHMAXPATH, fileName);
+    int fileNameLen = strlen(fileName);
+    if (fileNameLen >=3) {
         fileName[ fileNameLen - 3 ] = 0;
         strcat( fileName, "bmp" );
         // Try to load bitmap from that file.
-        HBITMAP bitmap = (HBITMAP)::LoadImage( NULL,
-                                               fileName,
-                                               IMAGE_BITMAP,
-                                               0,
-                                               0,
-                                               LR_LOADFROMFILE );
-        if ( bitmap ) {
-            HWND bitmapControl = GetDlgItem( mDlg, IDB_SPLASH );
-            if ( bitmapControl ) {
-                HBITMAP old = (HBITMAP)SendMessage( bitmapControl,
-                                                    STM_SETIMAGE,
-                                                    IMAGE_BITMAP,
-                                                    (LPARAM)bitmap );
-                // Remember bitmap so we can delete it later.
-                mBitmap = bitmap;
-                // Delete old bitmap.
-                if ( old ) {
-                    BOOL ok = DeleteObject( old );
-                }
-            } else {
-                // Delete bitmap since it isn't going to be used.
-                DeleteObject( bitmap );
-            }
-        }
+        mBitmap = LoadAndSetBitmapFromFile(hpsMemory, fileName);
     }
-#endif
+    if (!mBitmap) {
+        mBitmap = GpiLoadBitmap(hpsMemory, NULL, IDB_SPLASH, 0L, 0L);
+        GpiSetBitmap(hpsMemory, mBitmap);
+    }
+    BITMAPINFOHEADER bitmap;
+    bitmap.cbFix = sizeof(BITMAPINFOHEADER);
+    GpiQueryBitmapParameters(mBitmap, &bitmap);
+    mBitmapCX = bitmap.cx;
+    mBitmapCY = bitmap.cy;
 }
 
 MRESULT EXPENTRY DialogProc( HWND dlg, ULONG msg, MPARAM mp1, MPARAM mp2 ) {
@@ -524,15 +605,12 @@ MRESULT EXPENTRY DialogProc( HWND dlg, ULONG msg, MPARAM mp1, MPARAM mp2 ) {
          */ 
         HBITMAP hbitmap = splashScreen->mBitmap;
         if ( hbitmap ) {
-            BITMAPINFOHEADER bitmap;
-            bitmap.cbFix = sizeof (BITMAPINFOHEADER);
-            GpiQueryBitmapParameters (splashScreen->mBitmap, &bitmap);
             WinSetWindowPos( dlg,
                              HWND_TOP,
-                             WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN )/2 - bitmap.cx/2,
-                             WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN )/2 - bitmap.cy/2,
-                             bitmap.cx,
-                             bitmap.cy,
+                             WinQuerySysValue( HWND_DESKTOP, SV_CXSCREEN )/2 - splashScreen->mBitmapCX/2,
+                             WinQuerySysValue( HWND_DESKTOP, SV_CYSCREEN )/2 - splashScreen->mBitmapCY/2,
+                             splashScreen->mBitmapCX,
+                             splashScreen->mBitmapCY,
                              SWP_ACTIVATE | SWP_MOVE | SWP_SIZE );
             WinShowWindow( dlg, TRUE );
         }
@@ -542,12 +620,12 @@ MRESULT EXPENTRY DialogProc( HWND dlg, ULONG msg, MPARAM mp1, MPARAM mp2 ) {
         nsSplashScreenOS2 *splashScreen = (nsSplashScreenOS2*)WinQueryWindowPtr( dlg, QWL_USER );
         HPS hps = WinBeginPaint (dlg, NULLHANDLE, NULL);
         GpiErase (hps);
-        if (splashScreen->mBitmap) {
-            POINTL ptl;
-            ptl.x = 0;
-            ptl.y = 0;
-            WinDrawBitmap( hps, splashScreen->mBitmap, NULL, &ptl, CLR_NEUTRAL, CLR_BACKGROUND, DBM_NORMAL);
-        }
+        POINTL aptl[8] = {0, 0, splashScreen->mBitmapCX, splashScreen->mBitmapCY,
+                          0, 0, 0, 0,
+                          0, 0, 0, 0,
+                          0, 0, 0, 0};
+
+        GpiBitBlt( hps, splashScreen->hpsMemory, 3L, aptl, ROP_SRCCOPY, 0L );
         WinEndPaint( hps );
         return (MRESULT)TRUE;
     }
@@ -1076,6 +1154,7 @@ private:
 
 static char nameBuffer[128] = { 0 };
 char *nsNativeAppSupportOS2::mAppName = nameBuffer;
+PRBool nsNativeAppSupportOS2::mUseDDE = PR_FALSE;
 
 /* Start: Tries to find the "message window" to determine if it
  *        exists.  If so, then Mozilla is already running.  In that
@@ -1094,15 +1173,13 @@ nsNativeAppSupportOS2::Start( PRBool *aResult ) {
     NS_ENSURE_ARG( aResult );
     NS_ENSURE_TRUE( mInstance == 0, NS_ERROR_NOT_INITIALIZED );
 
-    PRBool useDDE = PR_TRUE;
-
     nsresult rv = NS_ERROR_FAILURE;
     *aResult = PR_FALSE;
 
     for ( int i = 1; i < __argc; i++ ) {
-        if ( strcmp( "-nodde", __argv[i] ) == 0 ||
-             strcmp( "/nodde", __argv[i] ) == 0 ) {
-            useDDE = PR_FALSE;
+        if ( strcmp( "-dde", __argv[i] ) == 0 ||
+             strcmp( "/dde", __argv[i] ) == 0 ) {
+            mUseDDE = PR_TRUE;
         }
     }
 
@@ -1152,7 +1229,7 @@ nsNativeAppSupportOS2::Start( PRBool *aResult ) {
         // We will be server.
         rv = msgWindow.Create();
         if ( NS_SUCCEEDED( rv ) ) {
-            if (useDDE) {
+            if (mUseDDE) {
                 // Start up DDE server.
                 this->StartDDE();
             }
@@ -1255,6 +1332,10 @@ nsNativeAppSupportOS2::Stop( PRBool *aResult ) {
     nsresult rv = NS_OK;
     *aResult = PR_TRUE;
 
+    if (!mUseDDE) {
+       return rv;
+    }
+
     Mutex ddeLock( MOZ_STARTUP_MUTEX_NAME );
 
     if ( ddeLock.Lock( MOZ_DDE_STOP_TIMEOUT ) ) {
@@ -1491,10 +1572,10 @@ nsCString nsNativeAppSupportOS2::ParseDDEArg( HSZ args, int index ) {
         // (which may not be accurate, but makes life so much easier).
         const char *p = temp.get();
         // Skip index commas.
-        PRInt32 offset = index ? temp.FindChar( ',', PR_FALSE, 0, index ) : 0;
+        PRInt32 offset = index ? temp.FindChar( ',', 0, index ) : 0;
         if ( offset != -1 ) {
             // Desired argument starts there and ends at next comma.
-            PRInt32 end = temp.FindChar( ',', PR_FALSE, offset+1 );
+            PRInt32 end = temp.FindChar( ',', offset+1 );
             if ( end == -1 ) {
                 // Rest of string.
                 end = temp.Length();
