@@ -113,7 +113,7 @@ function UpdateBookmarksLastVisitedDate(event)
     var bmks = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
                          .getService(Components.interfaces.nsIBookmarksService);
 
-    bmks.UpdateBookmarkLastVisitedDate(url, _content.document.characterSet);
+    bmks.updateLastVisitedDate(url, _content.document.characterSet);
   }
 }
 
@@ -318,7 +318,7 @@ function Startup()
   // hook up UI through progress listener
   var interfaceRequestor = getBrowser().docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
   var webProgress = interfaceRequestor.getInterface(Components.interfaces.nsIWebProgress);
-  webProgress.addProgressListener(window.XULBrowserWindow);
+  webProgress.addProgressListener(window.XULBrowserWindow, Components.interfaces.nsIWebProgress.NOTIFY_ALL);
 
   // XXXjag see bug 68662 (needed to hook up web progress listener)
   getBrowser().boxObject.setPropertyAsSupports("listenerkungfu", window.XULBrowserWindow);
@@ -366,10 +366,6 @@ function Startup()
     // Perform default browser checking.
     checkForDefaultBrowser();
   }
-
-  if (document.getElementById("main-window").getAttribute("fullScreen") == "true") {
-    BrowserFullScreenEnter();
-  }
 }
 
 function Shutdown()
@@ -651,7 +647,7 @@ function OpenSearch(tabName, forceDialogFlag, searchStr)
     }
     if (forceDialogFlag || searchMode == 1) {
       // Use a single search dialog
-      var windowManager = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"]
+      var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                     .getService(Components.interfaces.nsIWindowMediator);
 
       var searchWindow = windowManager.getMostRecentWindow("search:window");
@@ -676,7 +672,7 @@ function OpenSearch(tabName, forceDialogFlag, searchStr)
         try {
           var searchEngineURI = pref.getCharPref("browser.search.defaultengine");
           if (searchEngineURI) {
-            var searchURL = searchDS.GetInternetSearchURL(searchEngineURI, escapedSearchStr);
+            var searchURL = searchDS.GetInternetSearchURL(searchEngineURI, escapedSearchStr, 0, 0, {value:0});
           if (searchURL)
               defaultSearchURL = searchURL;
           }
@@ -744,7 +740,7 @@ function enableBookmarks()
 function BrowserEditBookmarks()
 {
   // Use a single sidebar bookmarks dialog
-  var windowManager = Components.classes["@mozilla.org/rdf/datasource;1?name=window-mediator"]
+  var windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                 .getService(Components.interfaces.nsIWindowMediator);
 
   var bookmarksWindow = windowManager.getMostRecentWindow("bookmarks:manager");
@@ -872,7 +868,7 @@ function getShortcutOrURI(url)
     var bmks = Components.classes["@mozilla.org/browser/bookmarks-service;1"]
                          .getService(Components.interfaces.nsIBookmarksService);
 
-    var shortcutURL = bmks.FindShortcut(url);
+    var shortcutURL = bmks.resolveKeyword(url);
     if (!shortcutURL) {
       // rjc: add support for string substitution with shortcuts (4/4/2000)
       //      (see bug # 29871 for details)
@@ -880,7 +876,7 @@ function getShortcutOrURI(url)
       if (aOffset > 0) {
         var cmd = url.substr(0, aOffset);
         var text = url.substr(aOffset+1);
-        shortcutURL = bmks.FindShortcut(cmd);
+        shortcutURL = bmks.resolveKeyword(cmd);
         if (shortcutURL && text) {
           aOffset = shortcutURL.indexOf("%s");
           if (aOffset >= 0)
@@ -939,52 +935,6 @@ function OpenAddressbook()
   open("chrome://messenger/content/addressbook/addressbook.xul", "_blank", "chrome,menubar,toolbar,resizable");
 }
 
-var gFullScreen = false;
-
-function BrowserFullScreenToggle()
-{
-  // toggle the attributes as indicated in the fullScreenElementChanges table
-  var idx, attribute, value;
-  var element, currentValue
-  var fullScreenElementChanges = document.getElementById("fullScreenElementChanges");
-  var next = fullScreenElementChanges.firstChild;
-  while (next) {
-    idx = next.getAttribute("idx");
-    attribute = next.getAttribute("attribute");
-    value = next.getAttribute("value");
-
-    element = document.getElementById(idx);
-    currentValue =  element.getAttribute(attribute);
-
-    if (value) {
-      element.setAttribute(attribute, value);
-    } else {
-      element.removeAttribute(attribute);
-    }
-    next.setAttribute("value", currentValue);
-
-    next = next.nextSibling;
-  }
-
-  // toggle and save the fullScreen indicator
-  gFullScreen = !gFullScreen;
-  document.getElementById("main-window").setAttribute("fullScreen", gFullScreen?"true":"false");
-}
-
-function BrowserFullScreenEnter()
-{
-  if (!gFullScreen) {
-    BrowserFullScreenToggle();
-  }
-}
-
-function BrowserFullScreenExit()
-{
-  if (gFullScreen) {
-    BrowserFullScreenToggle();
-  }
-}
-
 function BrowserViewSource()
 {
   var focusedWindow = document.commandDispatcher.focusedWindow;
@@ -1304,7 +1254,7 @@ function stylesheetSwitchAll(frameset, title) {
 function applyTheme(themeName)
 {
   var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"]
-                                 .getService(Components.interfaces.nsIChromeRegistry);
+                                 .getService(Components.interfaces.nsIXULChromeRegistry);
 
   chromeRegistry.selectSkin(themeName.getAttribute("name"), true);
   chromeRegistry.refreshSkins();
