@@ -44,7 +44,6 @@
 #include "nsIGenericFactory.h"
 
 // Interfaces Needed
-#include "nsIXULWindow.h"
 #include "nsIBaseWindow.h"
 #include "nsIDocShell.h"
 #include "nsIDocShellTreeItem.h"
@@ -55,7 +54,7 @@
 
 ///  Unsorted Includes
 
-#include "nsIMarkupDocumentViewer.h"
+#include "nsIObserver.h"
 #include "pratom.h"
 #include "prprf.h"
 #include "nsIComponentManager.h"
@@ -66,11 +65,9 @@
 #include "nsIDocument.h"
 #include "nsIDOMWindowInternal.h"
 
-#include "nsIScriptGlobalObject.h"
 #include "nsIContentViewer.h"
 #include "nsIContentViewerEdit.h"
 #include "nsIWebShell.h"
-#include "nsIDocShell.h"
 #include "nsIWebShellWindow.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsIWindowWatcher.h"
@@ -82,7 +79,6 @@
 #include "nsIServiceManager.h"
 #include "nsIURL.h"
 #include "nsIIOService.h"
-#include "nsIURL.h"
 #include "nsIWidget.h"
 #include "plevent.h"
 
@@ -91,60 +87,24 @@
 #include "nsAppShellCIDs.h"
 
 #include "nsIDocumentViewer.h"
-#include "nsIDOMHTMLImageElement.h"
 #include "nsICmdLineService.h"
 #include "nsIGlobalHistory.h"
 #include "nsIBrowserHistory.h"
-#include "nsIUrlbarHistory.h"
 
-#include "nsIDOMXULDocument.h"
-#include "nsIDOMNodeList.h"
-#include "nsIDOMNode.h"
-#include "nsIDOMElement.h"
-
-
-#include "nsIPresContext.h"
-#include "nsIPresShell.h"
-
-#include "nsIDocumentLoader.h"
 #include "nsIObserverService.h"
 
-#include "nsIFileSpec.h"
+#include "nsILocalFile.h"
+#include "nsIFileStreams.h"
 
 #include "nsCURILoader.h"
 #include "nsIContentHandler.h"
 #include "nsNetUtil.h"
 #include "nsICmdLineHandler.h"
 
-#include "nsIWindowMediator.h"
-
-#include "nsDocumentCharsetInfoCID.h"
-#include "nsIDocumentCharsetInfo.h"
-#include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
-
-#include "nsIDocShellHistory.h"
-
 // Stuff to implement file download dialog.
-#include "nsIDocumentObserver.h"
-#include "nsIContent.h"
-#include "nsINameSpaceManager.h"
 #include "nsFileStream.h"
 #include "nsIProxyObjectManager.h" 
 
-static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
-static NS_DEFINE_CID(kPrefServiceCID, NS_PREF_CID);
-static NS_DEFINE_CID(kDocumentCharsetInfoCID, NS_DOCUMENTCHARSETINFO_CID);
-static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
-static NS_DEFINE_IID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
-
-// Stuff to implement find/findnext
-#include "nsIFindComponent.h"
-
-// if DEBUG or MOZ_PERF_METRICS are defined, enable the PageCycler
-#ifdef DEBUG
-#define ENABLE_PAGE_CYCLER
-#endif
 
 // if DEBUG, NS_BUILD_REFCNT_LOGGING, or MOZ_PERF_METRICS is defined,
 // enable the PageCycler
@@ -152,13 +112,14 @@ static NS_DEFINE_IID(kProxyObjectManagerCID, NS_PROXYEVENT_MANAGER_CID);
 #define ENABLE_PAGE_CYCLER
 #endif
 
-/* Define Class IDs */
-static NS_DEFINE_IID(kAppShellServiceCID,       NS_APPSHELL_SERVICE_CID);
-static NS_DEFINE_IID(kCmdLineServiceCID,        NS_COMMANDLINE_SERVICE_CID);
-static NS_DEFINE_IID(kCGlobalHistoryCID,        NS_GLOBALHISTORY_CID);
-static NS_DEFINE_CID(kCPrefServiceCID,          NS_PREF_CID);
-static NS_DEFINE_CID(kWindowMediatorCID, NS_WINDOWMEDIATOR_CID);
+#include "nsITimeBomb.h"
 
+/* Define Class IDs */
+static NS_DEFINE_CID(kPrefServiceCID,           NS_PREF_CID);
+static NS_DEFINE_CID(kProxyObjectManagerCID,    NS_PROXYEVENT_MANAGER_CID);
+static NS_DEFINE_CID(kAppShellServiceCID,       NS_APPSHELL_SERVICE_CID);
+static NS_DEFINE_CID(kCmdLineServiceCID,        NS_COMMANDLINE_SERVICE_CID);
+static NS_DEFINE_CID(kCGlobalHistoryCID,        NS_GLOBALHISTORY_CID);
 
 #ifdef DEBUG                                                           
 static int APP_DEBUG = 0; // Set to 1 in debugger to turn on debugging.
@@ -168,7 +129,6 @@ static int APP_DEBUG = 0; // Set to 1 in debugger to turn on debugging.
 
 
 #define PREF_HOMEPAGE_OVERRIDE_URL "startup.homepage_override_url"
-#define PREF_HOMEPAGE_OVERRIDE "browser.startup.homepage_override.1"
 #define PREF_HOMEPAGE_OVERRIDE_MSTONE "browser.startup.homepage_override.mstone"
 #define PREF_BROWSER_STARTUP_PAGE "browser.startup.page"
 #define PREF_BROWSER_STARTUP_HOMEPAGE "browser.startup.homepage"
@@ -209,15 +169,16 @@ public:
 
   nsresult Init(const char* nativePath) {
     nsresult rv;
-    mFile = nativePath;
-    if (!mFile.IsFile())
-      return mFile.Error();
+    if (!mFile) {
+      rv = NS_NewNativeLocalFile(nsDependentCString(nativePath),
+                                 PR_TRUE, getter_AddRefs(mFile));
+      if (NS_FAILED(rv)) return rv;
+    }
 
-    nsCOMPtr<nsISupports> in;
-    rv = NS_NewTypicalInputFileStream(getter_AddRefs(in), mFile);
-    if (NS_FAILED(rv)) return rv;
-    nsCOMPtr<nsIInputStream> inStr = do_QueryInterface(in, &rv);
+    nsCOMPtr<nsIInputStream> inStr;
+    rv = NS_NewLocalFileInputStream(getter_AddRefs(inStr), mFile);
     if (NS_FAILED(rv)) return rv;
+
     PRUint32 avail;
     rv = inStr->Available(&avail);
     if (NS_FAILED(rv)) return rv;
@@ -297,7 +258,7 @@ public:
           // if the aSubject arg is 'this' then we were called by the Timer
           // Stop the current load and move on to the next
           nsCOMPtr<nsIDocShell> docShell;
-          mAppCore->GetContentDocShell(getter_AddRefs(docShell));
+          mAppCore->GetContentAreaDocShell(getter_AddRefs(docShell));
 
           nsCOMPtr<nsIWebNavigation> webNav(do_QueryInterface(docShell));
           if(webNav)
@@ -392,7 +353,7 @@ public:
 
 protected:
   nsBrowserInstance*    mAppCore;
-  nsFileSpec            mFile;
+  nsCOMPtr<nsILocalFile> mFile;
   char*                 mBuffer;
   char*                 mCursor;
   nsAutoString          mLastRequest;
@@ -440,8 +401,6 @@ PRBool nsBrowserInstance::sCmdLineURLUsed = PR_FALSE;
 
 nsBrowserInstance::nsBrowserInstance() : mIsClosed(PR_FALSE)
 {
-  mWebShellWin          = nsnull;
-  mDocShell             = nsnull;
   mDOMWindow            = nsnull;
   mContentAreaDocShellWeak  = nsnull;
   NS_INIT_REFCNT();
@@ -452,40 +411,12 @@ nsBrowserInstance::~nsBrowserInstance()
   Close();
 }
 
-NS_IMETHODIMP
-nsBrowserInstance::SetDefaultCharacterSet(const PRUnichar *aCharset)
-{
-  nsCOMPtr<nsIDOMWindowInternal> contentWindow;
-  GetContentWindow(getter_AddRefs(contentWindow));
-
-  nsCOMPtr<nsIScriptGlobalObject> globalObj(do_QueryInterface(contentWindow));
-
-  if (!globalObj)
-   return NS_ERROR_FAILURE;
-
-  nsCOMPtr<nsIDocShell> docShell;
-  globalObj->GetDocShell(getter_AddRefs(docShell));
-
-  if (docShell) {
-    nsCOMPtr<nsIContentViewer> childCV;
-    NS_ENSURE_SUCCESS(docShell->GetContentViewer(getter_AddRefs(childCV)), NS_ERROR_FAILURE);
-
-    nsCOMPtr<nsIMarkupDocumentViewer> markupCV(do_QueryInterface(childCV));
-
-    if (markupCV) {
-      NS_ENSURE_SUCCESS(markupCV->SetDefaultCharacterSet(aCharset), NS_ERROR_FAILURE);
-    }
-  }
-  return NS_OK;
-}
-
 void
 nsBrowserInstance::ReinitializeContentVariables()
 {
   NS_ASSERTION(mDOMWindow,"Reinitializing Content Variables without a window will cause a crash. see Bugzilla Bug 46454");
   if (!mDOMWindow)
     return;
-  nsresult rv;
 
   nsCOMPtr<nsIDOMWindow> contentWindow;
   mDOMWindow->GetContent(getter_AddRefs(contentWindow));
@@ -495,24 +426,16 @@ nsBrowserInstance::ReinitializeContentVariables()
   if (globalObj) {
     nsCOMPtr<nsIDocShell> docShell;
     globalObj->GetDocShell(getter_AddRefs(docShell));
-    nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(docShell));
 
-    if (webShell) {
-      mContentAreaDocShellWeak = getter_AddRefs(NS_GetWeakReference(docShell)); // Weak reference
+    mContentAreaDocShellWeak = dont_AddRef(NS_GetWeakReference(docShell)); // Weak reference
 
-      if (APP_DEBUG) {
-        nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
+    if (APP_DEBUG) {
+      nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
+      if (docShellAsItem) {
         nsXPIDLString name;
         docShellAsItem->GetName(getter_Copies(name));
-        nsCAutoString str;
-        str.AssignWithConversion(name);
-        printf("Attaching to Content WebShell [%s]\n", str.get());
+        printf("Attaching to Content WebShell [%s]\n", NS_LossyConvertUCS2toASCII(name).get());
       }
-
-      nsCOMPtr<nsIUrlbarHistory> ubHistory = do_GetService(NS_URLBARHISTORY_CONTRACTID, &rv);
-
-      if (ubHistory)
-        mUrlbarHistory = ubHistory;
     }
   }
 }
@@ -540,53 +463,6 @@ nsresult nsBrowserInstance::GetContentAreaDocShell(nsIDocShell** outDocShell)
   return NS_OK;
 }
     
-
-nsresult nsBrowserInstance::GetContentWindow(nsIDOMWindowInternal** outDOMWindow)
-{
-  nsCOMPtr<nsIDOMWindow> contentWindow;
-  mDOMWindow->GetContent(getter_AddRefs(contentWindow));
-
-  if (contentWindow) {
-    return CallQueryInterface(contentWindow, outDOMWindow);
-  }
-
-  return NS_OK;
-}
-
-nsresult nsBrowserInstance::GetFocussedContentWindow(nsIDOMWindowInternal** outFocussedWindow)
-{
-  nsCOMPtr<nsIDOMWindow> focussedWindow;
-  
-  // get the window that has focus (e.g. framesets)
-  if (mDocShell)
-  {    
-    nsCOMPtr<nsIContentViewer> cv;
-    mDocShell->GetContentViewer(getter_AddRefs(cv));    
-    nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
-    if (docv)
-    {
-      // Get the document from the doc viewer.
-      nsCOMPtr<nsIDocument> doc;
-      docv->GetDocument(*getter_AddRefs(doc));
-      nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(doc));
-      if (xulDoc)
-      {
-        nsCOMPtr<nsIDOMXULCommandDispatcher>  commandDispatcher;
-        xulDoc->GetCommandDispatcher(getter_AddRefs(commandDispatcher));
-        
-        if (commandDispatcher)
-          commandDispatcher->GetFocusedWindow(getter_AddRefs(focussedWindow));
-      }
-    }
-  }
-  
-  if (!focussedWindow)
-    return GetContentWindow(outFocussedWindow);   // default to content window
-
-  return CallQueryInterface(focussedWindow, outFocussedWindow);
-}
-
-
 //*****************************************************************************
 //    nsBrowserInstance: nsISupports
 //*****************************************************************************
@@ -604,7 +480,7 @@ NS_INTERFACE_MAP_END
 //    nsBrowserInstance: nsIBrowserInstance
 //*****************************************************************************
 
-NS_IMETHODIMP    
+nsresult
 nsBrowserInstance::LoadUrl(const PRUnichar * urlToLoad)
 {
   nsresult rv = NS_OK;
@@ -698,7 +574,7 @@ nsBrowserInstance::StartPageCycler(PRBool* aIsPageCycling)
   return NS_OK;
 }
 
-NS_IMETHODIMP    
+NS_IMETHODIMP
 nsBrowserInstance::Init()
 {
   nsresult rv = NS_OK;
@@ -706,33 +582,6 @@ nsBrowserInstance::Init()
   return rv;
 }
 
-NS_IMETHODIMP
-nsBrowserInstance::GetContentDocShell(nsIDocShell** aDocShell)
-{
-  NS_ENSURE_ARG_POINTER(aDocShell);
-
-  return GetContentAreaDocShell(aDocShell);
-}
-
-
-NS_IMETHODIMP
-nsBrowserInstance::SetUrlbarHistory(nsIUrlbarHistory* aUBHistory)
-{
-   mUrlbarHistory = aUBHistory;
-   return NS_OK;
-}
-	
-
-NS_IMETHODIMP
-nsBrowserInstance::GetUrlbarHistory(nsIUrlbarHistory** aUrlbarHistory)
-{
-   NS_ENSURE_ARG_POINTER(aUrlbarHistory);
-
-   *aUrlbarHistory = mUrlbarHistory;
-   NS_IF_ADDREF(*aUrlbarHistory);
-   return NS_OK;
-}
-
 NS_IMETHODIMP    
 nsBrowserInstance::SetWebShellWindow(nsIDOMWindowInternal* aWin)
 {
@@ -744,28 +593,15 @@ nsBrowserInstance::SetWebShellWindow(nsIDOMWindowInternal* aWin)
     return NS_ERROR_FAILURE;
   }
 
-  nsCOMPtr<nsIDocShell> docShell;
-  globalObj->GetDocShell(getter_AddRefs(docShell));
-  if (docShell) {
-    mDocShell = docShell.get();
-    //NS_ADDREF(mDocShell); WE DO NOT OWN THIS
-    // inform our top level webshell that we are its parent URI content listener...
-    if (APP_DEBUG) {
-      nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
+  if (APP_DEBUG) {
+    nsCOMPtr<nsIDocShell> docShell;
+    globalObj->GetDocShell(getter_AddRefs(docShell));
+    nsCOMPtr<nsIDocShellTreeItem> docShellAsItem(do_QueryInterface(docShell));
+    if (docShellAsItem) {
+      // inform our top level webshell that we are its parent URI content listener...
       nsXPIDLString name;
       docShellAsItem->GetName(getter_Copies(name));
-      nsCAutoString str;
-      str.AssignWithConversion(name);
-      printf("Attaching to WebShellWindow[%s]\n", str.get());
-    }
-
-    nsCOMPtr<nsIWebShell> webShell(do_QueryInterface(docShell));
-    nsCOMPtr<nsIWebShellContainer> webShellContainer;
-    webShell->GetContainer(*getter_AddRefs(webShellContainer));
-    if (webShellContainer) {
-      nsCOMPtr<nsIWebShellWindow> webShellWin;
-      if (NS_OK == webShellContainer->QueryInterface(NS_GET_IID(nsIWebShellWindow), getter_AddRefs(webShellWin)))
-        mWebShellWin = webShellWin;   // WE DO NOT OWN THIS
+      printf("Attaching to WebShellWindow[%s]\n", NS_LossyConvertUCS2toASCII(name).get());
     }
   }
 
@@ -783,32 +619,9 @@ nsBrowserInstance::Close()
   else
     mIsClosed = PR_TRUE;
 
-  // Release search context.
-  mSearchContext = nsnull;;
-
-  //Release Urlbar History
-  mUrlbarHistory = nsnull;
-
   return NS_OK;
 }
 
-NS_IMETHODIMP    
-nsBrowserInstance::Copy()
-{
-  nsCOMPtr<nsIDocShell> docShell;
-  GetContentAreaDocShell(getter_AddRefs(docShell));
-  NS_ENSURE_TRUE(docShell, NS_ERROR_FAILURE);
-  
-  nsCOMPtr<nsIContentViewer> viewer;
-  docShell->GetContentViewer(getter_AddRefs(viewer));
-  nsCOMPtr<nsIContentViewerEdit> edit(do_QueryInterface(viewer));
-  if (edit) {
-      edit->CopySelection();
-  }
-  return NS_OK;
-}
-
-
 //*****************************************************************************
 // nsBrowserInstance: Helpers
 //*****************************************************************************
@@ -861,8 +674,8 @@ nsBrowserContentHandler::~nsBrowserContentHandler()
 {
 }
 
-CMDLINEHANDLER_OTHERS_IMPL(nsBrowserContentHandler,"-chrome","general.startup.browser","Load the specified chrome.",NS_BROWSERSTARTUPHANDLER_CONTRACTID,"Browser Startup Handler", PR_TRUE, PR_FALSE)
-
+CMDLINEHANDLER_OTHERS_IMPL(nsBrowserContentHandler,"-chrome","general.startup.browser","Load the specified chrome.", PR_TRUE, PR_FALSE)
+CMDLINEHANDLER_REGISTERPROC_IMPL(nsBrowserContentHandler, "Browser Startup Handler", NS_BROWSERSTARTUPHANDLER_CONTRACTID);
 NS_IMETHODIMP nsBrowserContentHandler::GetChromeUrlForTask(char **aChromeUrlForTask) {
 
   if (!aChromeUrlForTask)
@@ -893,8 +706,8 @@ PRBool nsBrowserContentHandler::NeedHomepageOverride(nsIPref *aPrefService)
       do_GetService("@mozilla.org/network/protocol;1?name=http", &rv));
   if (NS_FAILED(rv))
     return PR_TRUE;
-  nsXPIDLCString currMilestone;
-  httpHandler->GetMisc(getter_Copies(currMilestone));
+  nsCAutoString currMilestone;
+  httpHandler->GetMisc(currMilestone);
 
   // get saved milestone from user's prefs
   nsXPIDLCString savedMilestone;
@@ -922,10 +735,34 @@ NS_IMETHODIMP nsBrowserContentHandler::GetDefaultArgs(PRUnichar **aDefaultArgs)
     return NS_ERROR_NULL_POINTER;
 
   nsresult rv;
+  static PRBool timebombChecked = PR_FALSE;
   nsAutoString args;
 
+  if (!timebombChecked) {
+    // timebomb check
+    timebombChecked = PR_TRUE;
+
+    PRBool expired;
+    nsCOMPtr<nsITimeBomb> timeBomb(do_GetService(NS_TIMEBOMB_CONTRACTID, &rv));
+    if (NS_FAILED(rv)) return rv;
+
+    rv = timeBomb->Init();
+    if (NS_FAILED(rv)) return rv;
+
+    rv = timeBomb->CheckWithUI(&expired);
+    if (NS_FAILED(rv)) return rv;
+
+    if (expired) {
+      nsXPIDLCString urlString;
+      rv = timeBomb->GetTimebombURL(getter_Copies(urlString));
+      if (NS_FAILED(rv)) return rv;
+
+      args.AssignWithConversion(urlString);
+    }
+  }
+
   if (args.IsEmpty()) {
-    nsCOMPtr<nsIPref> prefs(do_GetService(kCPrefServiceCID));
+    nsCOMPtr<nsIPref> prefs(do_GetService(kPrefServiceCID));
     if (!prefs) return NS_ERROR_FAILURE;
 
     if (NeedHomepageOverride(prefs)) {
@@ -991,13 +828,13 @@ NS_IMETHODIMP nsBrowserContentHandler::HandleContent(const char * aContentType,
   nsCOMPtr<nsIURI> uri;
   aChannel->GetURI(getter_AddRefs(uri));
   NS_ENSURE_TRUE(uri, NS_ERROR_FAILURE);
-  nsXPIDLCString spec;
-  uri->GetSpec(getter_Copies(spec));
+  nsCAutoString spec;
+  uri->GetSpec(spec);
 
   nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService("@mozilla.org/embedcomp/window-watcher;1"));
   if (wwatch) {
     nsCOMPtr<nsIDOMWindow> newWindow;
-    wwatch->OpenWindow(parentWindow, spec, "", 0, 0,
+    wwatch->OpenWindow(parentWindow, spec.get(), "", 0, 0,
               getter_AddRefs(newWindow));
   }
 
@@ -1010,7 +847,7 @@ NS_IMETHODIMP nsBrowserContentHandler::HandleContent(const char * aContentType,
 NS_GENERIC_FACTORY_CONSTRUCTOR_INIT(nsBrowserInstance, Init)
 NS_GENERIC_FACTORY_CONSTRUCTOR(nsBrowserContentHandler)
 
-static nsModuleComponentInfo components[] = {
+static const nsModuleComponentInfo components[] = {
   { "nsBrowserInstance",
     NS_BROWSERINSTANCE_CID,
     NS_BROWSERINSTANCE_CONTRACTID, 
@@ -1073,6 +910,11 @@ static nsModuleComponentInfo components[] = {
     NS_CONTENT_HANDLER_CONTRACTID_PREFIX"image/jpeg", 
     nsBrowserContentHandlerConstructor 
   },
+  { "Browser Content Handler",
+    NS_BROWSERCONTENTHANDLER_CID,
+    NS_CONTENT_HANDLER_CONTRACTID_PREFIX"image/jpg", 
+    nsBrowserContentHandlerConstructor 
+  },
   { "Browser Content Handler",
     NS_BROWSERCONTENTHANDLER_CID,
     NS_CONTENT_HANDLER_CONTRACTID_PREFIX"image/png", 
@@ -1098,6 +940,11 @@ static nsModuleComponentInfo components[] = {
     NS_CONTENT_HANDLER_CONTRACTID_PREFIX"image/x-jng", 
     nsBrowserContentHandlerConstructor 
   },
+  { "Browser Content Handler",
+    NS_BROWSERCONTENTHANDLER_CID,
+    NS_CONTENT_HANDLER_CONTRACTID_PREFIX"image/x-xbitmap", 
+    nsBrowserContentHandlerConstructor 
+  },
   { "Browser Content Handler",
     NS_BROWSERCONTENTHANDLER_CID,
     NS_CONTENT_HANDLER_CONTRACTID_PREFIX"application/http-index-format", 
