@@ -41,13 +41,25 @@
 #include "nsReadableUtils.h"
 #include "nspr.h"
 #include "nsLDAP.h"
+#include "nsIStringBundle.h"
+#include "nsCRT.h"
+#include "nsIObserverService.h"
+#include "nsNetUtil.h"
+#include "nsICategoryManager.h"
+#include "nsCategoryManagerUtils.h"
 
 #ifdef PR_LOGGING
 static PRLogModuleInfo *sLDAPAutoCompleteLogModule = 0;
 #endif
 
-NS_IMPL_ISUPPORTS3(nsLDAPAutoCompleteSession, nsIAutoCompleteSession, 
-                   nsILDAPMessageListener, nsILDAPAutoCompleteSession)
+
+// Because this object gets called via proxies, we need to use a THREADSAFE
+// ISUPPORTS; if bug 101252 gets fixed, we can go back to the non-threadsafe
+// version.
+//
+NS_IMPL_THREADSAFE_ISUPPORTS3(nsLDAPAutoCompleteSession, 
+                              nsIAutoCompleteSession, nsILDAPMessageListener,
+                              nsILDAPAutoCompleteSession)
 
 nsLDAPAutoCompleteSession::nsLDAPAutoCompleteSession() :
     mState(UNBOUND), 
@@ -427,6 +439,7 @@ nsLDAPAutoCompleteSession::OnLDAPInit(nsresult aStatus)
 {
     nsresult rv;        // temp for xpcom return values
     nsCOMPtr<nsILDAPMessageListener> selfProxy;
+    nsXPIDLString passwd;   // passwd to use to connect to server
 
     // Check the status from the initialization of the LDAP connection
     //
@@ -436,6 +449,116 @@ nsLDAPAutoCompleteSession::OnLDAPInit(nsresult aStatus)
         return NS_ERROR_FAILURE;
     }
 
+    // If mAuthPrompter is set, we're expected to use it to get a password.
+    //
+    if (mAuthPrompter) {
+        nsUTF8String spec;
+        PRBool status;
+
+        // we're going to use the URL spec of the server as the "realm" for 
+        // wallet to remember the password by / for.
+        //
+        rv = mServerURL->GetSpec(spec);
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPInit(): GetSpec"
+                     " failed\n");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     rv, UNBOUND);
+            return NS_ERROR_FAILURE;
+        }
+
+        // get the string bundle service
+        //
+        nsCOMPtr<nsIStringBundleService> 
+            stringBundleSvc(do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv)); 
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPInit():"
+                     " error getting string bundle service");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     rv, UNBOUND);
+            return rv;
+        }
+
+        // get the LDAP string bundle
+        //
+        nsCOMPtr<nsIStringBundle> ldapBundle;
+        rv = stringBundleSvc->CreateBundle(
+            "chrome://mozldap/locale/ldap.properties",
+            getter_AddRefs(ldapBundle));
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPInit():"
+                     " error creating string bundle"
+                     " chrome://mozldap/locale/ldap.properties");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     rv, UNBOUND);
+            return rv;
+        } 
+
+        // get the title for the authentication prompt
+        //
+        nsXPIDLString authPromptTitle;
+        rv = ldapBundle->GetStringFromName(
+            NS_LITERAL_STRING("authPromptTitle").get(), 
+            getter_Copies(authPromptTitle));
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPInit():"
+                     "error getting 'authPromptTitle' string from bundle "
+                     "chrome://mozldap/locale/ldap.properties");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     rv, UNBOUND);
+            return rv;
+        }
+
+        // get the host name for the auth prompt
+        //
+        nsCAutoString host;
+        rv = mServerURL->GetAsciiHost(host);
+        if (NS_FAILED(rv)) {
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv, 
+                                     UNBOUND);
+            return rv;
+        }
+
+        // hostTemp is only necessary to work around a code-generation 
+        // bug in egcs 1.1.2 (the version of gcc that comes with Red Hat 6.2),
+        // which is the default compiler for Mozilla on linux at the moment.
+        //
+        NS_ConvertASCIItoUCS2 hostTemp(host);
+        const PRUnichar *hostArray[1] = { hostTemp.get() };
+
+        // format the hostname into the authprompt text string
+        //
+        nsXPIDLString authPromptText;
+        rv = ldapBundle->FormatStringFromName(
+            NS_LITERAL_STRING("authPromptText").get(),
+            hostArray, sizeof(hostArray) / sizeof(const PRUnichar *),
+            getter_Copies(authPromptText));
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPInit():"
+                     "error getting 'authPromptText' string from bundle "
+                     "chrome://mozldap/locale/ldap.properties");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     rv, UNBOUND);
+            return rv;
+        }
+
+        // get authentication password, prompting the user if necessary
+        //
+        rv = mAuthPrompter->PromptPassword(
+            authPromptTitle.get(), authPromptText.get(),
+            NS_ConvertUTF8toUCS2(spec).get(),
+            nsIAuthPrompt::SAVE_PASSWORD_PERMANENTLY, getter_Copies(passwd),
+            &status);
+        if (NS_FAILED(rv) || status == PR_FALSE) {
+            PR_LOG(sLDAPAutoCompleteLogModule, PR_LOG_DEBUG, 
+                   ("nsLDAPAutoCompleteSession::OnLDAPInit(): PromptPassword"
+                    " encountered an error or was cancelled by the user"));
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
+                                     NS_ERROR_FAILURE, UNBOUND);
+            return NS_ERROR_FAILURE;
+        }
+    }
+
     // create and initialize an LDAP operation (to be used for the bind)
     //  
     mOperation = do_CreateInstance("@mozilla.org/network/ldap-operation;1", 
@@ -475,7 +598,7 @@ nsLDAPAutoCompleteSession::OnLDAPInit(nsresult aStatus)
     PR_LOG(sLDAPAutoCompleteLogModule, PR_LOG_DEBUG, 
            ("nsLDAPAutoCompleteSession:OnLDAPInit(): initiating "
             "SimpleBind\n"));
-    rv = mOperation->SimpleBind(NULL); 
+    rv = mOperation->SimpleBind(passwd); 
     if (NS_FAILED(rv)) {
 
         switch (rv) {
@@ -512,7 +635,6 @@ nsLDAPAutoCompleteSession::OnLDAPInit(nsresult aStatus)
 nsresult
 nsLDAPAutoCompleteSession::OnLDAPBind(nsILDAPMessage *aMessage)
 {
-
     PRInt32 errCode;
 
     mOperation = 0;  // done with bind op; make nsCOMPtr release it
@@ -525,21 +647,57 @@ nsLDAPAutoCompleteSession::OnLDAPBind(nsILDAPMessage *aMessage)
         NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPBind(): couldn't get "
                  "error code from aMessage");
 
-        // reset to the default state
+        // reset to the default state, and pass along the LDAP error code
+        // to the formatter
         //
-        FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems,
-                                 NS_ERROR_FAILURE, UNBOUND);
+        FinishAutoCompleteLookup(
+            nsIAutoCompleteStatus::failureItems,
+            NS_ERROR_GENERATE_FAILURE(NS_ERROR_MODULE_LDAP, errCode), 
+            UNBOUND);
         return NS_ERROR_FAILURE;
     }
 
-
     // check to be sure the bind succeeded
     //
-    if ( errCode != nsILDAPErrors::SUCCESS) {
+    if (errCode != nsILDAPErrors::SUCCESS) {
 
         PR_LOG(sLDAPAutoCompleteLogModule, PR_LOG_WARNING, 
-               ("nsLDAPAutoCompleteSession::OnLDAPBind(): error binding to "
-                "LDAP server, errCode = %d", errCode));
+                ("nsLDAPAutoCompleteSession::OnLDAPBind(): error binding to "
+                "LDAP server, errCode = 0x%x", errCode));
+
+        // if the login failed, tell the wallet to forget this password
+        //
+        if ( errCode == nsILDAPErrors::INAPPROPRIATE_AUTH ||
+             errCode == nsILDAPErrors::INVALID_CREDENTIALS ) {
+
+            // make sure the wallet service has been created, and in doing so,
+            // pass in a login-failed message to tell it to forget this passwd.
+            //
+            // apparently getting passwords stored in the wallet
+            // doesn't require the service to be running, which is why
+            // this might not exist yet.
+            //
+            rv = NS_CreateServicesFromCategory("passwordmanager", mServerURL,
+                                               "login-failed");
+            if (NS_FAILED(rv)) {
+                NS_ERROR("nsLDAPAutoCompleteSession::ForgetPassword(): error"
+                         " creating password manager service");
+                // not much to do at this point, though conceivably we could 
+                // pop up a dialog telling the user to go manually delete
+                // this password in the password manager.
+            }
+
+            // XXXdmose We should probably pop up an error dialog telling
+            // the user that the login failed here, rather than just bringing 
+            // up the password dialog again, which is what calling OnLDAPInit()
+            // does.  See bug 152997.
+
+            PR_LOG(sLDAPAutoCompleteLogModule, PR_LOG_DEBUG, 
+                   ("nsLDAPAutoCompleteSession::OnLDAPBind(): auth error;"
+                    " calling OnLDAPInit() again"));
+        
+            return OnLDAPInit(NS_OK);
+        }
 
         // reset to the default state
         //
@@ -621,9 +779,23 @@ nsLDAPAutoCompleteSession::OnLDAPSearchResult(nsILDAPMessage *aMessage)
     AutoCompleteStatus status;
     PRInt32 lderrno;
 
-    switch (mEntriesReturned) {
+    if (mEntriesReturned) {
 
-    case 0:
+        status = nsIAutoCompleteStatus::matchFound;
+
+        // there's at least one match, so the default index should
+        // point to the first thing here.  This ensures that if the local
+        // addressbook autocomplete session only found foo@local.domain, 
+        // this will be given preference
+        //
+        rv = mResults->SetDefaultItemIndex(0);
+        if (NS_FAILED(rv)) {
+            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPSearchResult(): "
+                     "mResults->SetDefaultItemIndex(0) failed");
+            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv, 
+                                     BOUND);
+        }
+    } else {
         // note that we only look at the error code if there are no results for
         // this session; if we got results and then an error happened, this
         // is ignored, in part because it seems likely to be confusing to the
@@ -649,38 +821,6 @@ nsLDAPAutoCompleteSession::OnLDAPSearchResult(nsILDAPMessage *aMessage)
         // that this actually buys us anything though.
         //
         status = nsIAutoCompleteStatus::noMatch;
-        break;
-
-    case 1:
-        status = nsIAutoCompleteStatus::matchFound;
-
-        // there's only one match, so the default index should point to it
-        //
-        rv = mResults->SetDefaultItemIndex(0);
-        if (NS_FAILED(rv)) {
-            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPSearchResult(): "
-                     "mResults->SetDefaultItemIndex(0) failed");
-            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv, 
-                                     BOUND);
-        }
-
-        break;
-
-    default:
-        status = nsIAutoCompleteStatus::matchFound;
-
-        // we must have more than one match, so the default index should be
-        // unset (-1)
-        //
-        rv = mResults->SetDefaultItemIndex(-1);
-        if (NS_FAILED(rv)) {
-            NS_ERROR("nsLDAPAutoCompleteSession::OnLDAPSearchResult(): "
-                     "mResults->SetDefaultItemIndex(-1) failed");
-            FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv,
-                                     BOUND);
-        }
-
-        break;
     }
 
     // call the mListener's OnAutoComplete and clean up
@@ -951,8 +1091,8 @@ nsLDAPAutoCompleteSession::InitConnection()
 
     // host to connect to
     //
-    nsXPIDLCString host;
-    rv = mServerURL->GetHost(getter_Copies(host));
+    nsCAutoString host;
+    rv = mServerURL->GetAsciiHost(host);
     if (NS_FAILED(rv)) {
         FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv, 
                                  UNBOUND);
@@ -969,6 +1109,16 @@ nsLDAPAutoCompleteSession::InitConnection()
         return NS_ERROR_FAILURE;
     }
         
+    // which options
+    //
+    PRUint32 options;
+    rv = mServerURL->GetOptions(&options);
+    if (NS_FAILED(rv)) {
+        FinishAutoCompleteLookup(nsIAutoCompleteStatus::failureItems, rv,
+                                 UNBOUND);
+        return NS_ERROR_FAILURE;
+    }
+        
     // get a proxy object so the callback happens on the main thread
     //
     rv = NS_GetProxyForObject(NS_UI_THREAD_EVENTQ,
@@ -988,7 +1138,10 @@ nsLDAPAutoCompleteSession::InitConnection()
     // lookup to occur, and we'll finish the binding of the connection
     // in the OnLDAPInit() listener function.
     //
-    rv = mConnection->Init(host, port, 0, selfProxy);
+    rv = mConnection->Init(host.get(), port,
+                           (options & nsILDAPURL::OPT_SECURE) ? PR_TRUE 
+                           : PR_FALSE, NS_ConvertUTF8toUCS2(mLogin).get(), 
+                           selfProxy);
     if NS_FAILED(rv) {
         switch (rv) {
 
@@ -1149,7 +1302,7 @@ nsLDAPAutoCompleteSession::FinishAutoCompleteLookup(
                    "error calling mListener->OnAutoComplete()");
     }
 
-    // set the state approrpiately
+    // set the state appropriately
     //
     mState = aEndState;
 
@@ -1339,7 +1492,7 @@ nsLDAPAutoCompleteSession::IsMessageCurrent(nsILDAPMessage *aMessage,
 }
 
 // attribute nsILDAPAutoCompFormatter formatter
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsLDAPAutoCompleteSession::GetFormatter(nsILDAPAutoCompFormatter* *aFormatter)
 {
     if (! aFormatter ) {
@@ -1375,4 +1528,29 @@ nsLDAPAutoCompleteSession::SetFormatter(nsILDAPAutoCompFormatter* aFormatter)
     return NS_OK;
 }
 
+NS_IMETHODIMP
+nsLDAPAutoCompleteSession::SetLogin(const nsACString & aLogin)
+{
+    mLogin = aLogin;
+    return NS_OK;
+}
+NS_IMETHODIMP
+nsLDAPAutoCompleteSession::GetLogin(nsACString & aLogin) 
+{
+    aLogin = mLogin;
+    return NS_OK;
+}
+
+NS_IMETHODIMP
+nsLDAPAutoCompleteSession::GetAuthPrompter(nsIAuthPrompt **aAuthPrompter)
+{
+    NS_IF_ADDREF(*aAuthPrompter = mAuthPrompter);
+    return NS_OK;
+}
+NS_IMETHODIMP
+nsLDAPAutoCompleteSession::SetAuthPrompter(nsIAuthPrompt *aAuthPrompter)
+{
+    mAuthPrompter = aAuthPrompter;
+    return NS_OK;
+}
 #endif
