@@ -30,8 +30,10 @@
 #include <nsWidgetsCID.h>
 #include <nsIXRemoteWidgetHelper.h>
 #include <nsIServiceManager.h>
+#include <nsIObserverService.h>
 #include <nsRect.h>
 #include <nsString.h>
+#include <nsCRT.h>
 #include <nsIPref.h>
 #include <nsIWindowWatcher.h>
 #include <nsISupportsPrimitives.h>
@@ -44,7 +46,6 @@
 #include <nsIURI.h>
 #include <nsNetUtil.h>
 #include <nsIWindowMediator.h>
-#include <rdf.h>
 
 NS_DEFINE_CID(kWindowCID, NS_WINDOW_CID);
 
@@ -595,15 +596,18 @@ XRemoteService::OpenURL(nsCString &aArgument,
   // the eventual toplevel target of the load
   nsCOMPtr<nsIDOMWindowInternal> finalWindow = aParent;
 
-  // see if there's a new window argument on the end
+  // see if there's a new-window or new-tab argument on the end
   nsCString lastArgument;
-  PRBool    newWindow = PR_FALSE;
+  PRBool    newWindow = PR_FALSE, newTab = PR_FALSE;
   PRUint32  index = 0;
   FindLastInList(aArgument, lastArgument, &index);
-  if (lastArgument.EqualsIgnoreCase("new-window")) {
+
+  newTab = lastArgument.EqualsIgnoreCase("new-tab");
+
+  if (newTab || lastArgument.EqualsIgnoreCase("new-window")) {
     aArgument.Truncate(index);
     // only open new windows if it's OK to do so
-    if (aOpenBrowser)
+    if (!newTab && aOpenBrowser)
       newWindow = PR_TRUE;
     // recheck for a possible noraise argument since it might have
     // been before the new-window argument
@@ -614,8 +618,11 @@ XRemoteService::OpenURL(nsCString &aArgument,
 
   // If it's OK to open a new browser window and a new window flag
   // wasn't passed in then try to find a current window.  If that's
-  // not found then go ahead and open a new window/
-  if (aOpenBrowser && !newWindow) {
+  // not found then go ahead and open a new window.
+  // If we're trying to open a new tab, we'll fall back to opening
+  // a new window if there's no browser window open, so look for it
+  // here.
+  if (aOpenBrowser && (!newWindow || newTab)) {
     nsCOMPtr<nsIDOMWindowInternal> lastUsedWindow;
     FindWindow(NS_LITERAL_STRING("navigator:browser").get(),
 	       getter_AddRefs(lastUsedWindow));
@@ -627,7 +634,6 @@ XRemoteService::OpenURL(nsCString &aArgument,
   }
 
   // try to fixup the argument passed in
-
   nsString url;
   url.AssignWithConversion(aArgument.get());
 
@@ -681,7 +687,7 @@ XRemoteService::OpenURL(nsCString &aArgument,
 
     // open a channel
     nsCOMPtr<nsIChannel> channel;
-    rv = NS_OpenURI(getter_AddRefs(channel), uri);
+    rv = NS_NewChannel(getter_AddRefs(channel), uri);
     if (NS_FAILED(rv))
       return NS_ERROR_FAILURE;
 
@@ -689,6 +695,22 @@ XRemoteService::OpenURL(nsCString &aArgument,
     rv = loader->OpenURI(channel, PR_TRUE, listenerRef);
   }
 
+  else if (newTab && aOpenBrowser) {
+    // We open new tabs simply by broadcasting a request to all interested
+    // observers, which will typically be all open windows.  The observers
+    // can check the notification subject against themselves (in the case of
+    // a browser window observer, anyway) to determine if they should be
+    // opening the tab.
+    nsCOMPtr<nsIObserverService>
+      obsServ(do_GetService("@mozilla.org/observer-service;1"));
+
+    if (!obsServ)
+      return NS_ERROR_FAILURE;
+
+    return obsServ->NotifyObservers(finalWindow, "open-new-tab-request",
+                                    url.get());
+  }
+
   else {
     // find the primary content shell for the window that we've been
     // asked to load into.
@@ -845,8 +867,7 @@ XRemoteService::FindWindow(const PRUnichar *aType,
 			   nsIDOMWindowInternal **_retval)
 {
   nsCOMPtr<nsIWindowMediator> mediator;
-  mediator = do_GetService(NS_RDF_DATASOURCE_CONTRACTID_PREFIX
-			   "window-mediator");
+  mediator = do_GetService(NS_WINDOWMEDIATOR_CONTRACTID);
 
   if (!mediator)
     return NS_ERROR_FAILURE;
@@ -856,7 +877,7 @@ XRemoteService::FindWindow(const PRUnichar *aType,
 
 NS_GENERIC_FACTORY_CONSTRUCTOR(XRemoteService)
 
-static nsModuleComponentInfo components[] = {
+static const nsModuleComponentInfo components[] = {
   { NS_IXREMOTESERVICE_CLASSNAME,
     NS_XREMOTESERVICE_CID,
     NS_IXREMOTESERVICE_CONTRACTID,
