@@ -139,14 +139,14 @@ void
 nsInstallFolder::GetDirectoryPath(nsCString& aDirectoryPath)
 {
   PRBool flagIsDir;
-  nsXPIDLCString  thePath;
+  nsCAutoString thePath;
   
   aDirectoryPath.SetLength(0);
 
     if (mFileSpec != nsnull)
     {
       // We want the NATIVE path.
-      mFileSpec->GetPath(getter_Copies(thePath));
+      mFileSpec->GetNativePath(thePath);
       aDirectoryPath.Assign(thePath);
 
       mFileSpec->IsDirectory(&flagIsDir);
@@ -175,12 +175,7 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
                          do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
                 if (!directoryService) return;
 
-                directoryService->Get(NS_OS_CURRENT_PROCESS_DIR, NS_GET_IID(nsIFile), getter_AddRefs(mFileSpec));
-#ifdef XP_MAC
-                mFileSpec->Append("Plug-ins");
-#else
-                mFileSpec->Append("plugins");
-#endif
+                directoryService->Get(NS_APP_PLUGINS_DIR, NS_GET_IID(nsIFile), getter_AddRefs(mFileSpec));
             }
             else
             {
@@ -188,11 +183,7 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
 
                 if (NS_SUCCEEDED(rv))
                 {
-#ifdef XP_MAC
-                    mFileSpec->Append("Plug-ins");
-#else
-                    mFileSpec->Append("plugins");
-#endif
+                    mFileSpec->AppendNative(INSTALL_PLUGINS_DIR);
                 }
                 else
                     mFileSpec = nsnull;
@@ -285,18 +276,9 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
                 nsCOMPtr<nsIProperties> directoryService = 
                          do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
                 if (!directoryService) return;
-                directoryService->Get(NS_OS_CURRENT_PROCESS_DIR, 
+                directoryService->Get(NS_XPCOM_COMPONENT_DIR, 
                                        NS_GET_IID(nsIFile), 
                                        getter_AddRefs(mFileSpec));
-
-                if (NS_SUCCEEDED(rv))
-                {
-#ifdef XP_MAC
-                    mFileSpec->Append("Components");
-#else
-                    mFileSpec->Append("components");
-#endif
-                }
             }
             else
             {
@@ -304,11 +286,7 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
 
                 if (NS_SUCCEEDED(rv))
                 {
-#ifdef XP_MAC
-                    mFileSpec->Append("Components");
-#else
-                    mFileSpec->Append("components");
-#endif
+                    mFileSpec->AppendNative(INSTALL_COMPONENTS_DIR);
                 }
                 else
                   mFileSpec = nsnull;
@@ -321,19 +299,9 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
                 nsCOMPtr<nsIProperties> directoryService = 
                          do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
                 if (!directoryService) return;
-                directoryService->Get(NS_OS_CURRENT_PROCESS_DIR, 
+                directoryService->Get(NS_APP_CHROME_DIR, 
                                        NS_GET_IID(nsIFile), 
                                        getter_AddRefs(mFileSpec));
-
-
-                if (NS_SUCCEEDED(rv))
-                {
-#ifdef XP_MAC
-                    mFileSpec->Append("Chrome");
-#else
-                    mFileSpec->Append("chrome");
-#endif
-                }
             }
             else
             {
@@ -341,11 +309,7 @@ nsInstallFolder::SetDirectoryPath(const nsString& aFolderID, const nsString& aRe
 
                 if (NS_SUCCEEDED(rv))
                 {
-#ifdef XP_MAC
-                    mFileSpec->Append("Chrome");
-#else
-                    mFileSpec->Append("chrome");
-#endif
+                    mFileSpec->AppendNative(INSTALL_CHROME_DIR);
                 }
             }
             break;
@@ -544,7 +508,7 @@ nsInstallFolder::AppendXPPath(const nsString& aRelativePath)
 
     NS_ASSERTION(!aRelativePath.IsEmpty(),"InstallFolder appending null path");
     do {
-        curr = aRelativePath.FindChar('/',PR_FALSE,start);
+        curr = aRelativePath.FindChar('/',start);
         if ( curr == start )
         {
             // illegal, two slashes in a row (or not a relative path)
@@ -564,14 +528,12 @@ nsInstallFolder::AppendXPPath(const nsString& aRelativePath)
             start = curr+1;
         }
             
-        nsresult rv = mFileSpec->AppendUnicode(segment.get());
+        nsresult rv = mFileSpec->Append(segment);
         if (NS_FAILED(rv))
         {
             // Unicode converters not present (likely wizard case)
             // so do our best with the vanilla conversion.
-            nsCAutoString tmp;
-            tmp.AssignWithConversion(segment);
-            mFileSpec->Append(tmp.get());
+            mFileSpec->AppendNative(NS_LossyConvertUCS2toASCII(segment));
         }
     } while ( start < aRelativePath.Length() );
 }
@@ -587,9 +549,11 @@ nsInstallFolder::MapNameToEnum(const nsString& name)
 	if ( name.IsEmpty())
         return -1;
 
+	// safe because all strings in DirectoryTable are ASCII
+	NS_LossyConvertUCS2toASCII asciiName(name);
 	while ( DirectoryTable[i].directoryName[0] != 0 )
 	{
-		if ( name.EqualsIgnoreCase(DirectoryTable[i].directoryName) )
+		if ( asciiName.EqualsIgnoreCase(DirectoryTable[i].directoryName) )
 			return DirectoryTable[i].folderEnum;
 		i++;
 	}
@@ -611,35 +575,25 @@ nsInstallFolder::ToString(nsAutoString* outString)
   //     This will appear to work on Latin-1 charsets but won't work on Mac or other charsets.
   //     On the other hand doing it right requires intl charset converters
   //     which we don't yet have in the initial install case.
+  //
+  //     However, nsLocalFile now uses the native charset conversion implementation, so this
+  //     code should always work.
 
   if (!mFileSpec || !outString)
       return NS_ERROR_NULL_POINTER;
 
-  nsXPIDLString  tempUC;
-  nsresult rv = mFileSpec->GetUnicodePath(getter_Copies(tempUC));
-  if (NS_SUCCEEDED(rv))
-  {
-      outString->Assign(tempUC);
-  }
-  else
-  {
-      // converters not present, most likely in wizard case;
-      // do best we can with stock ASCII conversion
-
-      // XXX NOTE we can make sure our filenames are ASCII, but we have no
-      // control over the directory name which might be localized!!!
-      nsXPIDLCString temp;
-      rv = mFileSpec->GetPath(getter_Copies(temp));
-      outString->AssignWithConversion(temp);
-  }
+  nsresult rv = mFileSpec->GetPath(*outString);
+  if (NS_FAILED(rv))
+      return rv;
 
   PRBool flagIsFile;
   mFileSpec->IsFile(&flagIsFile);
   if (!flagIsFile)
   {
       // assume directory, thus end with slash.
-      outString->AppendWithConversion(FILESEP);
+      outString->Append(PRUnichar(FILESEP));
   }
 
   return rv;
 }
+
