@@ -17,7 +17,7 @@
  * Copyright (C) 1998 Netscape Communications Corporation. All
  * Rights Reserved.
  *
- * Contributor(s): 
+ * Contributor(s):
  *   Pierre Phaneuf <pp@ludusdesign.com>
  */
 
@@ -27,6 +27,7 @@
 #include "nsIFactory.h"
 #include "nsISupports.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentRegistrar.h"
 #include "nsIServiceManager.h"
 #include "nsICategoryManager.h"
 #include "nsCOMPtr.h"
@@ -40,6 +41,7 @@
 #include "VerReg.h"
 #include "nsIDirectoryService.h"
 #include "nsDirectoryServiceDefs.h"
+#include "nsAppDirectoryServiceDefs.h"
 
 #include "nsInstall.h"
 #include "nsSoftwareUpdateIIDs.h"
@@ -84,10 +86,9 @@ static NS_DEFINE_CID(kInstallTrigger_CID, NS_SoftwareUpdateInstallTrigger_CID);
 
 static NS_DEFINE_CID(kInstallVersion_CID, NS_SoftwareUpdateInstallVersion_CID);
 
-static NS_DEFINE_CID(kChromeRegistryCID, NS_CHROMEREGISTRY_CID);
 static NS_DEFINE_CID(knsRegistryCID, NS_REGISTRY_CID);
 
-static NS_DEFINE_CID(kIProcessCID, NS_PROCESS_CID); 
+static NS_DEFINE_CID(kIProcessCID, NS_PROCESS_CID);
 
 nsSoftwareUpdate* nsSoftwareUpdate::mInstance = nsnull;
 nsCOMPtr<nsIFile> nsSoftwareUpdate::mProgramDir = nsnull;
@@ -98,7 +99,7 @@ PRBool            nsSoftwareUpdate::mNeedCleanup = PR_FALSE;
 nsSoftwareUpdate *
 nsSoftwareUpdate::GetInstance()
 {
-    if (mInstance == nsnull) 
+    if (mInstance == nsnull)
         mInstance = new nsSoftwareUpdate();
 
     NS_IF_ADDREF(mInstance);
@@ -113,7 +114,7 @@ nsSoftwareUpdate::nsSoftwareUpdate()
   mReg(0)
 {
     NS_INIT_ISUPPORTS();
-    
+
     mLock = PR_NewLock();
 
     /***************************************/
@@ -121,30 +122,28 @@ nsSoftwareUpdate::nsSoftwareUpdate()
     /***************************************/
 
     NR_StartupRegistry();   /* startup the registry; if already started, this will essentially be a noop */
-    
+
 
     nsresult rv;
-    nsCOMPtr<nsIProperties> directoryService = 
+    nsCOMPtr<nsIProperties> directoryService =
              do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
-    
+
     if(!directoryService) return;
-    
+
     nsCOMPtr<nsILocalFile> dir;
-    directoryService->Get(NS_XPCOM_CURRENT_PROCESS_DIR, NS_GET_IID(nsIFile), getter_AddRefs(dir));
+    directoryService->Get(NS_XPCOM_CURRENT_PROCESS_DIR, NS_GET_IID(nsILocalFile), getter_AddRefs(dir));
     if (dir)
     {
-        char* nativePath;
-        dir->GetPath(&nativePath);
+        nsCAutoString nativePath;
+        dir->GetNativePath(nativePath);
         // EVIL version registry does not take a nsIFile.;
-        VR_SetRegDirectory( nativePath );
-        if (nativePath)
-            nsMemory::Free(nativePath);
-            
+        VR_SetRegDirectory( nativePath.get() );
+
     }
     /***************************************/
     /* Add this as a shutdown observer     */
     /***************************************/
-    nsCOMPtr<nsIObserverService> observerService = 
+    nsCOMPtr<nsIObserverService> observerService =
              do_GetService("@mozilla.org/observer-service;1", &rv);
 
     if (NS_SUCCEEDED(rv))
@@ -196,16 +195,30 @@ nsSoftwareUpdate::Shutdown()
         nsresult rv;
         nsCOMPtr<nsILocalFile> pathToCleanupUtility;
         //Get the program directory
-        nsCOMPtr<nsIProperties> directoryService = 
+        nsCOMPtr<nsIProperties> directoryService =
                  do_GetService(NS_DIRECTORY_SERVICE_CONTRACTID, &rv);
-        directoryService->Get(NS_OS_CURRENT_PROCESS_DIR, 
-                              NS_GET_IID(nsIFile), 
-                              getter_AddRefs(pathToCleanupUtility));
+
+        if (nsSoftwareUpdate::GetProgramDirectory()) // In the stub installer
+        {
+            nsCOMPtr<nsIFile> tmp;
+            rv = nsSoftwareUpdate::GetProgramDirectory()->Clone(getter_AddRefs(tmp));
 #if defined (XP_MAC)
-        pathToCleanupUtility->Append(ESSENTIAL_FILES);
+            tmp->AppendNative(ESSENTIAL_FILES);
 #endif
+            pathToCleanupUtility = do_QueryInterface(tmp);
+
+        }
+        else
+        {
+            rv = directoryService->Get(NS_APP_INSTALL_CLEANUP_DIR,
+                                      NS_GET_IID(nsIFile),
+                                      getter_AddRefs(pathToCleanupUtility));
+        }
+
+        NS_ASSERTION(pathToCleanupUtility,"No path to cleanup utility in nsSoftwareUpdate::Shutdown()");
+
         //Create the Process framework
-        pathToCleanupUtility->Append(CLEANUP_UTIL);
+        pathToCleanupUtility->AppendNative(CLEANUP_UTIL);
         nsCOMPtr<nsIProcess> cleanupProcess = do_CreateInstance(kIProcessCID);
         rv = cleanupProcess->Init(pathToCleanupUtility);
         if (NS_SUCCEEDED(rv))
@@ -216,17 +229,17 @@ nsSoftwareUpdate::Shutdown()
     }
 }
 
-NS_IMETHODIMP nsSoftwareUpdate::Observe(nsISupports *aSubject, 
-                                        const char *aTopic, 
+NS_IMETHODIMP nsSoftwareUpdate::Observe(nsISupports *aSubject,
+                                        const char *aTopic,
                                         const PRUnichar *aData)
 {
     if (!nsCRT::strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID))
       Shutdown();
-     
+
     return NS_OK;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 nsSoftwareUpdate::RegisterListener(nsIXPIListener *aListener)
 {
     // once you register a Listener, you can not remove it.
@@ -237,7 +250,7 @@ nsSoftwareUpdate::RegisterListener(nsIXPIListener *aListener)
 
     if (!mMasterListener)
         return NS_ERROR_FAILURE;
-    
+
     mMasterListener->RegisterListener(aListener);
     return NS_OK;
 }
@@ -290,7 +303,6 @@ NS_IMETHODIMP
 nsSoftwareUpdate::InstallJar(  nsIFile* aLocalFile,
                                const PRUnichar* aURL,
                                const PRUnichar* aArguments,
-                               nsIDOMWindowInternal* aParentWindow,
                                PRUint32 flags,
                                nsIXPIListener* aListener)
 {
@@ -299,18 +311,18 @@ nsSoftwareUpdate::InstallJar(  nsIFile* aLocalFile,
 
     // -- grab a proxied Chrome Registry now while we can
     nsresult rv;
-    nsIChromeRegistry* chromeReg = nsnull;
-    NS_WITH_ALWAYS_PROXIED_SERVICE( nsIChromeRegistry, 
+    nsIXULChromeRegistry* chromeRegistry = nsnull;
+    NS_WITH_ALWAYS_PROXIED_SERVICE( nsIXULChromeRegistry,
                                     tmpReg,
-                                    kChromeRegistryCID, 
+                                    NS_CHROMEREGISTRY_CONTRACTID,
                                     NS_UI_THREAD_EVENTQ, &rv);
     if (NS_SUCCEEDED(rv))
-        chromeReg = tmpReg;
+        chromeRegistry = tmpReg;
 
     // we want to call this with or without a chrome registry
     nsInstallInfo *info = new nsInstallInfo( 0, aLocalFile, aURL, aArguments,
-                                             flags, aListener, aParentWindow, chromeReg );
-    
+                                             flags, aListener, chromeRegistry );
+
     if (!info)
         return NS_ERROR_OUT_OF_MEMORY;
 
@@ -332,9 +344,9 @@ nsSoftwareUpdate::InstallChrome( PRUint32 aType,
                                  nsIXPIListener* aListener)
 {
     nsresult rv;
-    NS_WITH_ALWAYS_PROXIED_SERVICE( nsIChromeRegistry,
-                                    chromeReg,
-                                    kChromeRegistryCID, 
+    NS_WITH_ALWAYS_PROXIED_SERVICE( nsIXULChromeRegistry,
+                                    chromeRegistry,
+                                    NS_CHROMEREGISTRY_CONTRACTID,
                                     NS_UI_THREAD_EVENTQ, &rv);
     if (NS_FAILED(rv))
         return rv;
@@ -345,8 +357,7 @@ nsSoftwareUpdate::InstallChrome( PRUint32 aType,
                                              aName,
                                              (PRUint32)aSelect,
                                              aListener,
-                                             nsnull, 
-                                             chromeReg);
+                                             chromeRegistry);
     if (!info)
         return NS_ERROR_OUT_OF_MEMORY;
 
@@ -437,7 +448,11 @@ nsSoftwareUpdate::StartupTasks( PRBool *needAutoreg )
     // is not the actual BuildID, or if we couldn't get the BuildID
     if ( autoReg || NS_FAILED(rv) || buildID != NS_BUILD_ID )
     {
-        rv = nsComponentManager::AutoRegister(nsIComponentManagerObsolete::NS_Startup,0);
+        nsCOMPtr<nsIServiceManager> servManager;
+        NS_GetServiceManager(getter_AddRefs(servManager));
+        nsCOMPtr<nsIComponentRegistrar> registrar = do_QueryInterface(servManager);
+        NS_ASSERTION(registrar, "No nsIComponentRegistrar from get service. see dougt");
+        rv = registrar->AutoRegister(nsnull);
 
         if (NS_SUCCEEDED(rv))
         {
@@ -477,7 +492,7 @@ nsSoftwareUpdate::RunNextInstall()
     if (!mMasterListener)
         CreateMasterListener();
 
-    if (!mInstalling) 
+    if (!mInstalling)
     {
         if ( mJarInstallQueue.Count() > 0 )
         {
@@ -485,7 +500,7 @@ nsSoftwareUpdate::RunNextInstall()
 
             if ( info )
                 mInstalling = PR_TRUE;
-            else 
+            else
             {
                 // bogus elements got into the queue
                 NS_ERROR("leaks remaining nsInstallInfos, please file bug!");
@@ -519,10 +534,10 @@ nsSoftwareUpdate::StubInitialize(nsIFile *aDir, const char* logName)
     nsresult rv = aDir->Clone(getter_AddRefs(mProgramDir));
 
     // make sure registry updates go to the right place
-    nsXPIDLCString tempPath;
-    rv = aDir->GetPath(getter_Copies(tempPath));
+    nsCAutoString tempPath;
+    rv = aDir->GetNativePath(tempPath);
     if (NS_SUCCEEDED(rv))
-        VR_SetRegDirectory( tempPath );
+        VR_SetRegDirectory( tempPath.get() );
 
     // Optionally set logfile leafname
     if (logName)
@@ -586,7 +601,7 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(nsSoftwareUpdateNameSet);
 #define NS_SOFTWAREUPDATENAMESET_CONTRACTID \
   "@mozilla.org/xpinstall/softwareupdatenameset;1"
 
-static NS_METHOD 
+static NS_METHOD
 RegisterSoftwareUpdate( nsIComponentManager *aCompMgr,
                         nsIFile *aPath,
                         const char *registryLocation,
@@ -617,37 +632,37 @@ RegisterSoftwareUpdate( nsIComponentManager *aCompMgr,
 
 
 // The list of components we register
-static nsModuleComponentInfo components[] = 
+static const nsModuleComponentInfo components[] =
 {
-    { "SoftwareUpdate Component", 
+    { "SoftwareUpdate Component",
        NS_SoftwareUpdate_CID,
        NS_IXPINSTALLCOMPONENT_CONTRACTID,
        nsSoftwareUpdateConstructor,
        RegisterSoftwareUpdate
     },
-   
-    { "InstallTrigger Component", 
+
+    { "InstallTrigger Component",
        NS_SoftwareUpdateInstallTrigger_CID,
-       NS_INSTALLTRIGGERCOMPONENT_CONTRACTID, 
+       NS_INSTALLTRIGGERCOMPONENT_CONTRACTID,
        nsInstallTriggerConstructor
     },
-    
-    { "InstallVersion Component", 
+
+    { "InstallVersion Component",
        NS_SoftwareUpdateInstallVersion_CID,
        NS_INSTALLVERSIONCOMPONENT_CONTRACTID,
-       nsInstallVersionConstructor 
+       nsInstallVersionConstructor
     },
 
     { "XPInstall Content Handler",
       NS_SoftwareUpdateInstallTrigger_CID,
       NS_CONTENT_HANDLER_CONTRACTID_PREFIX"application/x-xpinstall",
-      nsInstallTriggerConstructor 
+      nsInstallTriggerConstructor
     },
 
     { "Software update nameset",
       NS_SOFTWAREUPDATENAMESET_CID,
       NS_SOFTWAREUPDATENAMESET_CONTRACTID,
-      nsSoftwareUpdateNameSetConstructor 
+      nsSoftwareUpdateNameSetConstructor
     }
 };
 
