@@ -35,8 +35,8 @@ const char kFTPProto[8] = "ftp://";
 const int kHTTPPort = 80;
 const int kFTPPort = 21;
 const int kRespBufSize = 1024;
-const int kReqBufSize = 1024;
-const int kHdrBufSize = 256;
+const int kReqBufSize = 4096;
+const int kHdrBufSize = 4096;
 const char kCRLF[3] = "\r\n";
 const char kHdrBodyDelim[5] = "\r\n\r\n";
 const char kDefaultDestFile[11] = "index.html";
@@ -250,7 +250,7 @@ nsHTTPConn::Request(int aResumePos)
     char req[kReqBufSize];
     char hdr[kHdrBufSize];
     int rv;
-    
+
     memset(req, 0, kReqBufSize);
 
     // format header buf:
@@ -259,32 +259,39 @@ nsHTTPConn::Request(int aResumePos)
     memset(hdr, 0, kHdrBufSize);
     if (mProxiedURL)
     {
-        sprintf(hdr, "GET %s HTTP/1.1%s", mProxiedURL, kCRLF);
+        char *host = NULL, *path = NULL;
+        char proto[8];
+        int port;
+#ifdef DEBUG
+        assert(sizeof hdr > (strlen(mProxiedURL) + 15 ));
+#endif
+        sprintf(hdr, "GET %s HTTP/1.0%s", mProxiedURL, kCRLF);
         strcpy(req, hdr);
-
-        memset(hdr, 0, kHdrBufSize);
-        if (strncmp(mProxiedURL, kFTPProto, strlen(kFTPProto)) == 0)
+        if (strncmp(mProxiedURL, kFTPProto, strlen(kFTPProto)) == 0) 
         {
-            char *ftpHost, *ftpPath;
-            int ftpPort = kFTPPort;
-            rv = ParseURL(kFTPProto, mProxiedURL, 
-                          &ftpHost, &ftpPort, &ftpPath);
-
-            if (rv == OK)
-                sprintf(hdr, "Host: %s:%d%s", ftpHost, ftpPort, kCRLF);
-
-            if (ftpHost)
-                free(ftpHost);
-            if (ftpPath)
-                free(ftpPath);
+            strcpy(proto,kFTPProto);
+            port = kFTPPort;
+        } 
+        else 
+        {
+            strcpy(proto,kHTTPProto);
+            port = kHTTPPort;
         }
-        else
-            sprintf(hdr, "Host: %s%s", mHost, kCRLF);
-        strcat(req, hdr);
+        rv = ParseURL(proto, mProxiedURL,
+                      &host, &port, &path);
+        if (rv == OK) {
+            memset(hdr, 0, kHdrBufSize);
+            sprintf(hdr, "Host: %s:%d%s", host, port, kCRLF);
+            strcat(req, hdr);
+        }
+        if (host)
+            free(host);
+        if (path)
+            free(path);
     }
     else
     {
-        sprintf(hdr, "GET %s HTTP/1.1%s", mPath, kCRLF);
+        sprintf(hdr, "GET %s HTTP/1.0%s", mPath, kCRLF);
         strcpy(req, hdr);
 
         memset(hdr, 0, kHdrBufSize);
@@ -434,8 +441,8 @@ nsHTTPConn::Response(HTTPGetCB aCallback, char *aDestFile, int aResumePos)
         if ( mEventPumpCB )
             mEventPumpCB();
 
-    } while (bytesWritten < expectedSize && (rv == nsSocket::E_READ_MORE || rv == nsSocket::OK));
-
+    } while ( rv == nsSocket::E_READ_MORE || rv == nsSocket::OK);
+    
     if ( bytesWritten == expectedSize )
         rv = nsSocket::E_EOF_FOUND;
         
