@@ -29,6 +29,7 @@
 #include "nsXIEngine.h"
 #include <signal.h>
 #include <gtk/gtk.h>
+#include <errno.h>
 
 #define NUM_PS_ENTRIES 4
 
@@ -308,6 +309,10 @@ nsInstallDlg::RunApps()
     extern char **environ; /* globally available to all processes */
     int pid;
 
+    dest = gCtx->opt->mDestination;
+    if (chdir(dest) < 0) 
+        fprintf(stderr,"chdir(%s): %s\n",dest,strerror(errno));
+
     while (currRunApp)
     {
         /* run application with supplied args */
@@ -315,7 +320,6 @@ nsInstallDlg::RunApps()
         {
             /* child */
 
-            dest = gCtx->opt->mDestination;
             if (*(dest + strlen(dest)) == '/') /* trailing slash */
                 sprintf(apppath, "%s%s", dest, currRunApp->GetApp());
             else                               /* no trailing slash */
@@ -880,11 +884,7 @@ nsInstallDlg::ShowCompleteDlg()
     gtk_widget_grab_default(okButton);
     gtk_widget_show_all(completeDlg);
 
-    while (!bComplete)
-    {
-        XI_GTK_UPDATE_UI();
-    }
-
+    gtk_main();
     gtk_main_quit();
 }
 
@@ -896,6 +896,7 @@ nsInstallDlg::CompleteOK(GtkWidget *aWidget, gpointer aData)
     if (dlg)
         gtk_widget_destroy(dlg);
 
+    gtk_main_quit();
     bComplete = TRUE;
 }
 
@@ -1392,7 +1393,8 @@ nsInstallDlg::InitDLProgress( int isFirst )
 void
 nsInstallDlg::InitInstallProgress()
 {
-    gtk_widget_hide(sDLProgress.vbox);
+    if ( sDLProgress.vbox )
+            gtk_widget_hide(sDLProgress.vbox);
     gCtx->idlg->ShowTable();
 }
 
